<!DOCTYPE html>
<html>
    <head>
        <title>Khoj Feedback Form</title>
    </head>
<body>
<body style="font-family: 'Verdana', sans-serif; font-weight: 400; font-style: normal; padding: 0; text-align: left; width: 600px; margin: 20px auto;">
<meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no">
<a class="logo" href="https://khoj.dev" target="_blank" style="text-decoration: none; text-decoration: underline dotted;">
    <img src="https://khoj.dev/khoj-logo-sideways-500.png" alt="Khoj Logo" style="width: 100px;">
</a>
<div class="calls-to-action" style="margin-top: 20px;">
    <div>
        <h1 style="color: #333; font-size: large; font-weight: bold; margin: 10px; line-height: 1.5; background-color: #fee285; padding: 8px; box-shadow: 6px 6px rgba(0, 0, 0, 1.5);">User Feedback:</h1>
        <div>
        <h3>User Query</h3>
        <p>{{uquery}}</p>
        </div>
        <div>
        <h3>Khoj's Response</h3>
        {{kquery}}
        </div>
        <div>
        <h3>Sentiment</h3>
        <p>{{sentiment}}</p>
        </div>
        <div>
        <h3>User Email</h3>
        <p>{{user_email}}</p>
        </div>
    </div>
</div>
</body>
</html>
