*,:after,:before{--tw-border-spacing-x:0;--tw-border-spacing-y:0;--tw-translate-x:0;--tw-translate-y:0;--tw-rotate:0;--tw-skew-x:0;--tw-skew-y:0;--tw-scale-x:1;--tw-scale-y:1;--tw-pan-x: ;--tw-pan-y: ;--tw-pinch-zoom: ;--tw-scroll-snap-strictness:proximity;--tw-gradient-from-position: ;--tw-gradient-via-position: ;--tw-gradient-to-position: ;--tw-ordinal: ;--tw-slashed-zero: ;--tw-numeric-figure: ;--tw-numeric-spacing: ;--tw-numeric-fraction: ;--tw-ring-inset: ;--tw-ring-offset-width:0px;--tw-ring-offset-color:#fff;--tw-ring-color:rgba(59,130,246,.5);--tw-ring-offset-shadow:0 0 #0000;--tw-ring-shadow:0 0 #0000;--tw-shadow:0 0 #0000;--tw-shadow-colored:0 0 #0000;--tw-blur: ;--tw-brightness: ;--tw-contrast: ;--tw-grayscale: ;--tw-hue-rotate: ;--tw-invert: ;--tw-saturate: ;--tw-sepia: ;--tw-drop-shadow: ;--tw-backdrop-blur: ;--tw-backdrop-brightness: ;--tw-backdrop-contrast: ;--tw-backdrop-grayscale: ;--tw-backdrop-hue-rotate: ;--tw-backdrop-invert: ;--tw-backdrop-opacity: ;--tw-backdrop-saturate: ;--tw-backdrop-sepia: ;--tw-contain-size: ;--tw-contain-layout: ;--tw-contain-paint: ;--tw-contain-style: }::backdrop{--tw-border-spacing-x:0;--tw-border-spacing-y:0;--tw-translate-x:0;--tw-translate-y:0;--tw-rotate:0;--tw-skew-x:0;--tw-skew-y:0;--tw-scale-x:1;--tw-scale-y:1;--tw-pan-x: ;--tw-pan-y: ;--tw-pinch-zoom: ;--tw-scroll-snap-strictness:proximity;--tw-gradient-from-position: ;--tw-gradient-via-position: ;--tw-gradient-to-position: ;--tw-ordinal: ;--tw-slashed-zero: ;--tw-numeric-figure: ;--tw-numeric-spacing: ;--tw-numeric-fraction: ;--tw-ring-inset: ;--tw-ring-offset-width:0px;--tw-ring-offset-color:#fff;--tw-ring-color:rgba(59,130,246,.5);--tw-ring-offset-shadow:0 0 #0000;--tw-ring-shadow:0 0 #0000;--tw-shadow:0 0 #0000;--tw-shadow-colored:0 0 #0000;--tw-blur: ;--tw-brightness: ;--tw-contrast: ;--tw-grayscale: ;--tw-hue-rotate: ;--tw-invert: ;--tw-saturate: ;--tw-sepia: ;--tw-drop-shadow: ;--tw-backdrop-blur: ;--tw-backdrop-brightness: ;--tw-backdrop-contrast: ;--tw-backdrop-grayscale: ;--tw-backdrop-hue-rotate: ;--tw-backdrop-invert: ;--tw-backdrop-opacity: ;--tw-backdrop-saturate: ;--tw-backdrop-sepia: ;--tw-contain-size: ;--tw-contain-layout: ;--tw-contain-paint: ;--tw-contain-style: }/*
! tailwindcss v3.4.15 | MIT License | https://tailwindcss.com
*/*,:after,:before{box-sizing:border-box;border:0 solid #e5e7eb}:after,:before{--tw-content:""}:host,html{line-height:1.5;-webkit-text-size-adjust:100%;-moz-tab-size:4;-o-tab-size:4;tab-size:4;font-family:var(--font-noto-sans),var(--font-noto-sans-arabic);font-feature-settings:normal;font-variation-settings:normal;-webkit-tap-highlight-color:transparent}body{margin:0;line-height:inherit}hr{height:0;color:inherit;border-top-width:1px}abbr:where([title]){-webkit-text-decoration:underline dotted;text-decoration:underline dotted}h1,h2,h3,h4,h5,h6{font-size:inherit;font-weight:inherit}a{color:inherit;text-decoration:inherit}b,strong{font-weight:bolder}code,kbd,pre,samp{font-family:ui-monospace,SFMono-Regular,Menlo,Monaco,Consolas,Liberation Mono,Courier New,monospace;font-feature-settings:normal;font-variation-settings:normal;font-size:1em}small{font-size:80%}sub,sup{font-size:75%;line-height:0;position:relative;vertical-align:baseline}sub{bottom:-.25em}sup{top:-.5em}table{text-indent:0;border-color:inherit;border-collapse:collapse}button,input,optgroup,select,textarea{font-family:inherit;font-feature-settings:inherit;font-variation-settings:inherit;font-size:100%;font-weight:inherit;line-height:inherit;letter-spacing:inherit;color:inherit;margin:0;padding:0}button,select{text-transform:none}button,input:where([type=button]),input:where([type=reset]),input:where([type=submit]){-webkit-appearance:button;background-color:transparent;background-image:none}:-moz-focusring{outline:auto}:-moz-ui-invalid{box-shadow:none}progress{vertical-align:baseline}::-webkit-inner-spin-button,::-webkit-outer-spin-button{height:auto}[type=search]{-webkit-appearance:textfield;outline-offset:-2px}::-webkit-search-decoration{-webkit-appearance:none}::-webkit-file-upload-button{-webkit-appearance:button;font:inherit}summary{display:list-item}blockquote,dd,dl,figure,h1,h2,h3,h4,h5,h6,hr,p,pre{margin:0}fieldset{margin:0}fieldset,legend{padding:0}menu,ol,ul{list-style:none;margin:0;padding:0}dialog{padding:0}textarea{resize:vertical}input::-moz-placeholder,textarea::-moz-placeholder{opacity:1;color:#9ca3af}input::placeholder,textarea::placeholder{opacity:1;color:#9ca3af}[role=button],button{cursor:pointer}:disabled{cursor:default}audio,canvas,embed,iframe,img,object,svg,video{display:block;vertical-align:middle}img,video{max-width:100%;height:auto}[hidden]:where(:not([hidden=until-found])){display:none}:root{--background:0 0% 100%;--foreground:224 71.4% 4.1%;--card:0 0% 100%;--card-foreground:224 71.4% 4.1%;--popover:0 0% 100%;--popover-foreground:224 71.4% 4.1%;--primary:24.6 95% 53.1%;--primary-foreground:60 9.1% 97.8%;--secondary:220 14.3% 95.9%;--secondary-foreground:220.9 39.3% 11%;--muted:220 14.3% 95.9%;--muted-foreground:220 8.9% 46.1%;--accent:220 14.3% 95.9%;--accent-foreground:220.9 39.3% 11%;--destructive:0 84.2% 60.2%;--destructive-foreground:210 20% 98%;--border:220 13% 91%;--input:220 13% 91%;--ring:24.6 95% 53.1%;--radius:0.5rem;--font-family:var(--font-noto-sans),var(--font-noto-sans-arabic),sans-serif!important;--frosted-background-color:20 13% 95%;--border-color:#e2e2e2;pre code.hljs{display:block;overflow-x:auto;padding:1em}code.hljs{padding:3px 5px}

        /*!
            Theme: GitHub
            Description: Light theme as seen on github.com
            Author: github.com
            Maintainer: @Hirse
            Updated: 2021-05-15

            Outdated base version: https://github.com/primer/github-syntax-light
            Current colors taken from GitHub's CSS
          */.hljs{color:#24292e;background:#fff}.hljs-doctag,.hljs-keyword,.hljs-meta .hljs-keyword,.hljs-template-tag,.hljs-template-variable,.hljs-type,.hljs-variable.language_{color:#d73a49}.hljs-title,.hljs-title.class_,.hljs-title.class_.inherited__,.hljs-title.function_{color:#6f42c1}.hljs-attr,.hljs-attribute,.hljs-literal,.hljs-meta,.hljs-number,.hljs-operator,.hljs-selector-attr,.hljs-selector-class,.hljs-selector-id,.hljs-variable{color:#005cc5}.hljs-meta .hljs-string,.hljs-regexp,.hljs-string{color:#032f62}.hljs-built_in,.hljs-symbol{color:#e36209}.hljs-code,.hljs-comment,.hljs-formula{color:#6a737d}.hljs-name,.hljs-quote,.hljs-selector-pseudo,.hljs-selector-tag{color:#22863a}.hljs-subst{color:#24292e}.hljs-section{color:#005cc5;font-weight:700}.hljs-bullet{color:#735c0f}.hljs-emphasis{color:#24292e;font-style:italic}.hljs-strong{color:#24292e;font-weight:700}.hljs-addition{color:#22863a;background-color:#f0fff4}.hljs-deletion{color:#b31d28;background-color:#ffeef0}}.dark{--background:0 0% 14%;--foreground:210 20% 98%;--card:0 0% 14%;--card-foreground:210 20% 98%;--popover:0 0% 14%;--popover-foreground:210 20% 98%;--popover:20 14.3% 4.1%;--popover-foreground:60 9.1% 97.8%;--secondary:0 0% 9%;--secondary-foreground:210 20% 98%;--muted:0 0% 9%;--muted-foreground:217.9 10.6% 64.9%;--accent:0 0% 9%;--accent-foreground:210 20% 98%;--destructive:0 62.8% 30.6%;--destructive-foreground:210 20% 98%;--border:0 0% 9%;--input:0 0% 9%;--ring:20.5 90.2% 48.2%;--font-family:var(--font-noto-sans),var(--font-noto-sans-arabic),sans-serif!important;pre code.hljs{display:block;overflow-x:auto;padding:1em}code.hljs{padding:3px 5px}

        /*!
            Theme: GitHub Dark
            Description: Dark theme as seen on github.com
            Author: github.com
            Maintainer: @Hirse
            Updated: 2021-05-15

            Outdated base version: https://github.com/primer/github-syntax-dark
            Current colors taken from GitHub's CSS
          */.hljs{color:#c9d1d9;background:#0d1117}.hljs-doctag,.hljs-keyword,.hljs-meta .hljs-keyword,.hljs-template-tag,.hljs-template-variable,.hljs-type,.hljs-variable.language_{color:#ff7b72}.hljs-title,.hljs-title.class_,.hljs-title.class_.inherited__,.hljs-title.function_{color:#d2a8ff}.hljs-attr,.hljs-attribute,.hljs-literal,.hljs-meta,.hljs-number,.hljs-operator,.hljs-selector-attr,.hljs-selector-class,.hljs-selector-id,.hljs-variable{color:#79c0ff}.hljs-meta .hljs-string,.hljs-regexp,.hljs-string{color:#a5d6ff}.hljs-built_in,.hljs-symbol{color:#ffa657}.hljs-code,.hljs-comment,.hljs-formula{color:#8b949e}.hljs-name,.hljs-quote,.hljs-selector-pseudo,.hljs-selector-tag{color:#7ee787}.hljs-subst{color:#c9d1d9}.hljs-section{color:#1f6feb;font-weight:700}.hljs-bullet{color:#f2cc60}.hljs-emphasis{color:#c9d1d9;font-style:italic}.hljs-strong{color:#c9d1d9;font-weight:700}.hljs-addition{color:#aff5b4;background-color:#033a16}.hljs-deletion{color:#ffdcd7;background-color:#67060c}}*{border-color:hsl(var(--border))}menu,ol,ul{list-style:revert;padding:revert;margin:revert}body{background-color:hsl(var(--background));color:hsl(var(--foreground))}h1{font-size:2.25rem;line-height:2.5rem;font-weight:700}body{font-family:var(--font-family)}pre code.hljs{white-space:preserve}a{text-decoration:underline}input{background-color:hsla(var(--background)/.1)}.sr-only{position:absolute;width:1px;height:1px;padding:0;margin:-1px;overflow:hidden;clip:rect(0,0,0,0);white-space:nowrap;border-width:0}.pointer-events-none{pointer-events:none}.pointer-events-auto{pointer-events:auto}.visible{visibility:visible}.fixed{position:fixed}.absolute{position:absolute}.relative{position:relative}.inset-0{inset:0}.inset-x-0{left:0;right:0}.inset-y-0{top:0;bottom:0}.-right-2{right:-.5rem}.-top-0{top:0}.bottom-0{bottom:0}.bottom-\[20\%\]{bottom:20%}.left-0{left:0}.left-2{left:.5rem}.left-4{left:1rem}.left-\[50\%\]{left:50%}.right-0{right:0}.right-2{right:.5rem}.right-4{right:1rem}.top-0{top:0}.top-2{top:.5rem}.top-4{top:1rem}.top-\[0\.9rem\]{top:.9rem}.top-\[1px\]{top:1px}.top-\[50\%\]{top:50%}.top-\[60\%\]{top:60%}.top-full{top:100%}.z-10{z-index:10}.z-50{z-index:50}.z-\[100\]{z-index:100}.z-\[1\]{z-index:1}.z-\[60\]{z-index:60}.col-auto{grid-column:auto}.float-right{float:right}.\!m-2{margin:.5rem!important}.m-0{margin:0}.m-1{margin:.25rem}.m-2{margin:.5rem}.m-8{margin:2rem}.m-auto{margin:auto}.-mx-1{margin-left:-.25rem;margin-right:-.25rem}.mx-1{margin-left:.25rem;margin-right:.25rem}.mx-10{margin-left:2.5rem;margin-right:2.5rem}.mx-2{margin-left:.5rem;margin-right:.5rem}.mx-3{margin-left:.75rem;margin-right:.75rem}.mx-4{margin-left:1rem;margin-right:1rem}.mx-auto{margin-left:auto;margin-right:auto}.my-1{margin-top:.25rem;margin-bottom:.25rem}.my-2{margin-top:.5rem;margin-bottom:.5rem}.my-4{margin-top:1rem;margin-bottom:1rem}.my-8{margin-top:2rem;margin-bottom:2rem}.\!mt-0{margin-top:0!important}.mb-1{margin-bottom:.25rem}.mb-2{margin-bottom:.5rem}.mb-4{margin-bottom:1rem}.ml-1{margin-left:.25rem}.ml-2{margin-left:.5rem}.ml-3{margin-left:.75rem}.ml-6{margin-left:1.5rem}.ml-auto{margin-left:auto}.mr-1{margin-right:.25rem}.mr-2{margin-right:.5rem}.mr-8{margin-right:2rem}.mr-auto{margin-right:auto}.mt-0{margin-top:0}.mt-1{margin-top:.25rem}.mt-1\.5{margin-top:.375rem}.mt-2{margin-top:.5rem}.mt-24{margin-top:6rem}.mt-4{margin-top:1rem}.mt-auto{margin-top:auto}.line-clamp-1{-webkit-line-clamp:1}.line-clamp-1,.line-clamp-2{overflow:hidden;display:-webkit-box;-webkit-box-orient:vertical}.line-clamp-2{-webkit-line-clamp:2}.line-clamp-3{-webkit-line-clamp:3}.line-clamp-3,.line-clamp-4{overflow:hidden;display:-webkit-box;-webkit-box-orient:vertical}.line-clamp-4{-webkit-line-clamp:4}.line-clamp-5{overflow:hidden;display:-webkit-box;-webkit-box-orient:vertical;-webkit-line-clamp:5}.block{display:block}.inline-block{display:inline-block}.inline{display:inline}.flex{display:flex}.inline-flex{display:inline-flex}.table{display:table}.grid{display:grid}.hidden{display:none}.aspect-square{aspect-ratio:1/1}.h-1\.5{height:.375rem}.h-10{height:2.5rem}.h-11{height:2.75rem}.h-12{height:3rem}.h-14{height:3.5rem}.h-16{height:4rem}.h-2{height:.5rem}.h-2\.5{height:.625rem}.h-3{height:.75rem}.h-3\.5{height:.875rem}.h-32{height:8rem}.h-4{height:1rem}.h-5{height:1.25rem}.h-6{height:1.5rem}.h-7{height:1.75rem}.h-72{height:18rem}.h-8{height:2rem}.h-9{height:2.25rem}.h-\[200px\]{height:200px}.h-\[500px\]{height:500px}.h-\[73vh\]{height:73vh}.h-\[80vh\]{height:80vh}.h-\[var\(--radix-navigation-menu-viewport-height\)\]{height:var(--radix-navigation-menu-viewport-height)}.h-\[var\(--radix-select-trigger-height\)\]{height:var(--radix-select-trigger-height)}.h-auto{height:auto}.h-fit{height:-moz-fit-content;height:fit-content}.h-full{height:100%}.h-px{height:1px}.h-screen{height:100vh}.max-h-32{max-height:8rem}.max-h-96{max-height:24rem}.max-h-\[128px\]{max-height:128px}.max-h-\[300px\]{max-height:300px}.max-h-\[60vh\]{max-height:60vh}.max-h-\[80vh\]{max-height:80vh}.max-h-\[98vh\]{max-height:98vh}.max-h-screen{max-height:100vh}.min-h-16{min-height:4rem}.min-h-\[80px\]{min-height:80px}.\!w-4{width:1rem!important}.w-10{width:2.5rem}.w-14{width:3.5rem}.w-2{width:.5rem}.w-2\.5{width:.625rem}.w-3{width:.75rem}.w-3\.5{width:.875rem}.w-3\/4{width:75%}.w-4{width:1rem}.w-4\/6{width:66.666667%}.w-5{width:1.25rem}.w-6{width:1.5rem}.w-7{width:1.75rem}.w-72{width:18rem}.w-8{width:2rem}.w-80{width:20rem}.w-9{width:2.25rem}.w-\[100px\]{width:100px}.w-\[100vw\]{width:100vw}.w-\[200px\]{width:200px}.w-\[400px\]{width:400px}.w-\[80vw\]{width:80vw}.w-auto{width:auto}.w-fit{width:-moz-fit-content;width:fit-content}.w-full{width:100%}.w-max{width:-moz-max-content;width:max-content}.w-px{width:1px}.min-w-0{min-width:0}.min-w-\[12rem\]{min-width:12rem}.min-w-\[8rem\]{min-width:8rem}.min-w-\[var\(--radix-select-trigger-width\)\]{min-width:var(--radix-select-trigger-width)}.\!max-w-full{max-width:100%!important}.max-w-\[200px\]{max-width:200px}.max-w-\[600px\]{max-width:600px}.max-w-\[85vw\]{max-width:85vw}.max-w-\[90vw\]{max-width:90vw}.max-w-\[92vw\]{max-width:92vw}.max-w-full{max-width:100%}.max-w-lg{max-width:32rem}.max-w-max{max-width:-moz-max-content;max-width:max-content}.max-w-screen-md{max-width:768px}.flex-1{flex:1 1 0%}.flex-none{flex:none}.flex-shrink-0,.shrink-0{flex-shrink:0}.flex-grow,.grow{flex-grow:1}.caption-bottom{caption-side:bottom}.translate-x-\[-50\%\]{--tw-translate-x:-50%}.translate-x-\[-50\%\],.translate-y-\[-50\%\]{transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}.translate-y-\[-50\%\]{--tw-translate-y:-50%}.rotate-45{--tw-rotate:45deg}.rotate-45,.transform{transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}@keyframes caret-blink{0%,70%,to{opacity:1}20%,50%{opacity:0}}.animate-caret-blink{animation:caret-blink 1.25s ease-out infinite}@keyframes pulse{50%{opacity:.5}}.animate-pulse{animation:pulse 2s cubic-bezier(.4,0,.6,1) infinite}@keyframes spin{to{transform:rotate(1turn)}}.animate-spin{animation:spin 1s linear infinite}.cursor-default{cursor:default}.cursor-pointer{cursor:pointer}.cursor-text{cursor:text}.touch-none{touch-action:none}.select-none{-webkit-user-select:none;-moz-user-select:none;user-select:none}.resize-none{resize:none}.resize{resize:both}.list-none{list-style-type:none}.grid-flow-col{grid-auto-flow:column}.grid-cols-1{grid-template-columns:repeat(1,minmax(0,1fr))}.grid-cols-\[1fr_auto\]{grid-template-columns:1fr auto}.flex-row{flex-direction:row}.flex-col{flex-direction:column}.flex-col-reverse{flex-direction:column-reverse}.flex-wrap{flex-wrap:wrap}.content-center{align-content:center}.content-start{align-content:flex-start}.content-end{align-content:flex-end}.items-start{align-items:flex-start}.items-end{align-items:flex-end}.items-center{align-items:center}.justify-start{justify-content:flex-start}.justify-end{justify-content:flex-end}.justify-center{justify-content:center}.justify-between{justify-content:space-between}.gap-1{gap:.25rem}.gap-1\.5{gap:.375rem}.gap-16{gap:4rem}.gap-2{gap:.5rem}.gap-4{gap:1rem}.gap-6{gap:1.5rem}.gap-8{gap:2rem}.space-x-1>:not([hidden])~:not([hidden]){--tw-space-x-reverse:0;margin-right:calc(.25rem * var(--tw-space-x-reverse));margin-left:calc(.25rem * calc(1 - var(--tw-space-x-reverse)))}.space-x-2>:not([hidden])~:not([hidden]){--tw-space-x-reverse:0;margin-right:calc(.5rem * var(--tw-space-x-reverse));margin-left:calc(.5rem * calc(1 - var(--tw-space-x-reverse)))}.space-x-4>:not([hidden])~:not([hidden]){--tw-space-x-reverse:0;margin-right:calc(1rem * var(--tw-space-x-reverse));margin-left:calc(1rem * calc(1 - var(--tw-space-x-reverse)))}.space-y-0>:not([hidden])~:not([hidden]){--tw-space-y-reverse:0;margin-top:calc(0px * calc(1 - var(--tw-space-y-reverse)));margin-bottom:calc(0px * var(--tw-space-y-reverse))}.space-y-1>:not([hidden])~:not([hidden]){--tw-space-y-reverse:0;margin-top:calc(.25rem * calc(1 - var(--tw-space-y-reverse)));margin-bottom:calc(.25rem * var(--tw-space-y-reverse))}.space-y-1\.5>:not([hidden])~:not([hidden]){--tw-space-y-reverse:0;margin-top:calc(.375rem * calc(1 - var(--tw-space-y-reverse)));margin-bottom:calc(.375rem * var(--tw-space-y-reverse))}.space-y-2>:not([hidden])~:not([hidden]){--tw-space-y-reverse:0;margin-top:calc(.5rem * calc(1 - var(--tw-space-y-reverse)));margin-bottom:calc(.5rem * var(--tw-space-y-reverse))}.space-y-3>:not([hidden])~:not([hidden]){--tw-space-y-reverse:0;margin-top:calc(.75rem * calc(1 - var(--tw-space-y-reverse)));margin-bottom:calc(.75rem * var(--tw-space-y-reverse))}.space-y-6>:not([hidden])~:not([hidden]){--tw-space-y-reverse:0;margin-top:calc(1.5rem * calc(1 - var(--tw-space-y-reverse)));margin-bottom:calc(1.5rem * var(--tw-space-y-reverse))}.overflow-auto{overflow:auto}.overflow-hidden{overflow:hidden}.overflow-x-auto{overflow-x:auto}.overflow-y-auto{overflow-y:auto}.overflow-x-hidden{overflow-x:hidden}.overflow-y-scroll{overflow-y:scroll}.truncate{overflow:hidden;white-space:nowrap}.text-ellipsis,.truncate{text-overflow:ellipsis}.whitespace-normal{white-space:normal}.whitespace-nowrap{white-space:nowrap}.whitespace-pre-line{white-space:pre-line}.whitespace-pre-wrap{white-space:pre-wrap}.text-wrap{text-wrap:wrap}.text-nowrap{text-wrap:nowrap}.text-balance{text-wrap:balance}.break-words{overflow-wrap:break-word}.break-all{word-break:break-all}.rounded{border-radius:.25rem}.rounded-2xl{border-radius:1rem}.rounded-\[inherit\]{border-radius:inherit}.rounded-full{border-radius:9999px}.rounded-lg{border-radius:var(--radius)}.rounded-md{border-radius:calc(var(--radius) - 2px)}.rounded-sm{border-radius:calc(var(--radius) - 4px)}.rounded-xl{border-radius:.75rem}.rounded-b-none{border-bottom-right-radius:0;border-bottom-left-radius:0}.rounded-t-2xl{border-top-left-radius:1rem;border-top-right-radius:1rem}.rounded-t-\[10px\]{border-top-left-radius:10px;border-top-right-radius:10px}.rounded-tl-sm{border-top-left-radius:calc(var(--radius) - 4px)}.border{border-width:1px}.border-2{border-width:2px}.border-y{border-top-width:1px}.border-b,.border-y{border-bottom-width:1px}.border-l{border-left-width:1px}.border-l-4{border-left-width:4px}.border-r{border-right-width:1px}.border-t{border-top-width:1px}.border-dashed{border-style:dashed}.border-none{border-style:none}.border-amber-100{--tw-border-opacity:1;border-color:rgb(254 243 199/var(--tw-border-opacity,1))}.border-amber-100\/0{border-color:hsla(48,96%,89%,0)}.border-amber-100\/10{border-color:hsla(48,96%,89%,.1)}.border-amber-100\/100{border-color:#fef3c7}.border-amber-100\/15{border-color:hsla(48,96%,89%,.15)}.border-amber-100\/20{border-color:hsla(48,96%,89%,.2)}.border-amber-100\/25{border-color:hsla(48,96%,89%,.25)}.border-amber-100\/30{border-color:hsla(48,96%,89%,.3)}.border-amber-100\/35{border-color:hsla(48,96%,89%,.35)}.border-amber-100\/40{border-color:hsla(48,96%,89%,.4)}.border-amber-100\/45{border-color:hsla(48,96%,89%,.45)}.border-amber-100\/5{border-color:hsla(48,96%,89%,.05)}.border-amber-100\/50{border-color:hsla(48,96%,89%,.5)}.border-amber-100\/55{border-color:hsla(48,96%,89%,.55)}.border-amber-100\/60{border-color:hsla(48,96%,89%,.6)}.border-amber-100\/65{border-color:hsla(48,96%,89%,.65)}.border-amber-100\/70{border-color:hsla(48,96%,89%,.7)}.border-amber-100\/75{border-color:hsla(48,96%,89%,.75)}.border-amber-100\/80{border-color:hsla(48,96%,89%,.8)}.border-amber-100\/85{border-color:hsla(48,96%,89%,.85)}.border-amber-100\/90{border-color:hsla(48,96%,89%,.9)}.border-amber-100\/95{border-color:hsla(48,96%,89%,.95)}.border-amber-200{--tw-border-opacity:1;border-color:rgb(253 230 138/var(--tw-border-opacity,1))}.border-amber-200\/0{border-color:hsla(48,97%,77%,0)}.border-amber-200\/10{border-color:hsla(48,97%,77%,.1)}.border-amber-200\/100{border-color:#fde68a}.border-amber-200\/15{border-color:hsla(48,97%,77%,.15)}.border-amber-200\/20{border-color:hsla(48,97%,77%,.2)}.border-amber-200\/25{border-color:hsla(48,97%,77%,.25)}.border-amber-200\/30{border-color:hsla(48,97%,77%,.3)}.border-amber-200\/35{border-color:hsla(48,97%,77%,.35)}.border-amber-200\/40{border-color:hsla(48,97%,77%,.4)}.border-amber-200\/45{border-color:hsla(48,97%,77%,.45)}.border-amber-200\/5{border-color:hsla(48,97%,77%,.05)}.border-amber-200\/50{border-color:hsla(48,97%,77%,.5)}.border-amber-200\/55{border-color:hsla(48,97%,77%,.55)}.border-amber-200\/60{border-color:hsla(48,97%,77%,.6)}.border-amber-200\/65{border-color:hsla(48,97%,77%,.65)}.border-amber-200\/70{border-color:hsla(48,97%,77%,.7)}.border-amber-200\/75{border-color:hsla(48,97%,77%,.75)}.border-amber-200\/80{border-color:hsla(48,97%,77%,.8)}.border-amber-200\/85{border-color:hsla(48,97%,77%,.85)}.border-amber-200\/90{border-color:hsla(48,97%,77%,.9)}.border-amber-200\/95{border-color:hsla(48,97%,77%,.95)}.border-amber-50{--tw-border-opacity:1;border-color:rgb(255 251 235/var(--tw-border-opacity,1))}.border-amber-50\/0{border-color:rgba(255,251,235,0)}.border-amber-50\/10{border-color:rgba(255,251,235,.1)}.border-amber-50\/100{border-color:#fffbeb}.border-amber-50\/15{border-color:rgba(255,251,235,.15)}.border-amber-50\/20{border-color:rgba(255,251,235,.2)}.border-amber-50\/25{border-color:rgba(255,251,235,.25)}.border-amber-50\/30{border-color:rgba(255,251,235,.3)}.border-amber-50\/35{border-color:rgba(255,251,235,.35)}.border-amber-50\/40{border-color:rgba(255,251,235,.4)}.border-amber-50\/45{border-color:rgba(255,251,235,.45)}.border-amber-50\/5{border-color:rgba(255,251,235,.05)}.border-amber-50\/50{border-color:rgba(255,251,235,.5)}.border-amber-50\/55{border-color:rgba(255,251,235,.55)}.border-amber-50\/60{border-color:rgba(255,251,235,.6)}.border-amber-50\/65{border-color:rgba(255,251,235,.65)}.border-amber-50\/70{border-color:rgba(255,251,235,.7)}.border-amber-50\/75{border-color:rgba(255,251,235,.75)}.border-amber-50\/80{border-color:rgba(255,251,235,.8)}.border-amber-50\/85{border-color:rgba(255,251,235,.85)}.border-amber-50\/90{border-color:rgba(255,251,235,.9)}.border-amber-50\/95{border-color:rgba(255,251,235,.95)}.border-amber-500{--tw-border-opacity:1;border-color:rgb(245 158 11/var(--tw-border-opacity,1))}.border-amber-500\/0{border-color:rgba(245,158,11,0)}.border-amber-500\/10{border-color:rgba(245,158,11,.1)}.border-amber-500\/100{border-color:#f59e0b}.border-amber-500\/15{border-color:rgba(245,158,11,.15)}.border-amber-500\/20{border-color:rgba(245,158,11,.2)}.border-amber-500\/25{border-color:rgba(245,158,11,.25)}.border-amber-500\/30{border-color:rgba(245,158,11,.3)}.border-amber-500\/35{border-color:rgba(245,158,11,.35)}.border-amber-500\/40{border-color:rgba(245,158,11,.4)}.border-amber-500\/45{border-color:rgba(245,158,11,.45)}.border-amber-500\/5{border-color:rgba(245,158,11,.05)}.border-amber-500\/50{border-color:rgba(245,158,11,.5)}.border-amber-500\/55{border-color:rgba(245,158,11,.55)}.border-amber-500\/60{border-color:rgba(245,158,11,.6)}.border-amber-500\/65{border-color:rgba(245,158,11,.65)}.border-amber-500\/70{border-color:rgba(245,158,11,.7)}.border-amber-500\/75{border-color:rgba(245,158,11,.75)}.border-amber-500\/80{border-color:rgba(245,158,11,.8)}.border-amber-500\/85{border-color:rgba(245,158,11,.85)}.border-amber-500\/90{border-color:rgba(245,158,11,.9)}.border-amber-500\/95{border-color:rgba(245,158,11,.95)}.border-amber-950{--tw-border-opacity:1;border-color:rgb(69 26 3/var(--tw-border-opacity,1))}.border-amber-950\/0{border-color:rgba(69,26,3,0)}.border-amber-950\/10{border-color:rgba(69,26,3,.1)}.border-amber-950\/100{border-color:#451a03}.border-amber-950\/15{border-color:rgba(69,26,3,.15)}.border-amber-950\/20{border-color:rgba(69,26,3,.2)}.border-amber-950\/25{border-color:rgba(69,26,3,.25)}.border-amber-950\/30{border-color:rgba(69,26,3,.3)}.border-amber-950\/35{border-color:rgba(69,26,3,.35)}.border-amber-950\/40{border-color:rgba(69,26,3,.4)}.border-amber-950\/45{border-color:rgba(69,26,3,.45)}.border-amber-950\/5{border-color:rgba(69,26,3,.05)}.border-amber-950\/50{border-color:rgba(69,26,3,.5)}.border-amber-950\/55{border-color:rgba(69,26,3,.55)}.border-amber-950\/60{border-color:rgba(69,26,3,.6)}.border-amber-950\/65{border-color:rgba(69,26,3,.65)}.border-amber-950\/70{border-color:rgba(69,26,3,.7)}.border-amber-950\/75{border-color:rgba(69,26,3,.75)}.border-amber-950\/80{border-color:rgba(69,26,3,.8)}.border-amber-950\/85{border-color:rgba(69,26,3,.85)}.border-amber-950\/90{border-color:rgba(69,26,3,.9)}.border-amber-950\/95{border-color:rgba(69,26,3,.95)}.border-blue-100{--tw-border-opacity:1;border-color:rgb(219 234 254/var(--tw-border-opacity,1))}.border-blue-100\/0{border-color:rgba(219,234,254,0)}.border-blue-100\/10{border-color:rgba(219,234,254,.1)}.border-blue-100\/100{border-color:#dbeafe}.border-blue-100\/15{border-color:rgba(219,234,254,.15)}.border-blue-100\/20{border-color:rgba(219,234,254,.2)}.border-blue-100\/25{border-color:rgba(219,234,254,.25)}.border-blue-100\/30{border-color:rgba(219,234,254,.3)}.border-blue-100\/35{border-color:rgba(219,234,254,.35)}.border-blue-100\/40{border-color:rgba(219,234,254,.4)}.border-blue-100\/45{border-color:rgba(219,234,254,.45)}.border-blue-100\/5{border-color:rgba(219,234,254,.05)}.border-blue-100\/50{border-color:rgba(219,234,254,.5)}.border-blue-100\/55{border-color:rgba(219,234,254,.55)}.border-blue-100\/60{border-color:rgba(219,234,254,.6)}.border-blue-100\/65{border-color:rgba(219,234,254,.65)}.border-blue-100\/70{border-color:rgba(219,234,254,.7)}.border-blue-100\/75{border-color:rgba(219,234,254,.75)}.border-blue-100\/80{border-color:rgba(219,234,254,.8)}.border-blue-100\/85{border-color:rgba(219,234,254,.85)}.border-blue-100\/90{border-color:rgba(219,234,254,.9)}.border-blue-100\/95{border-color:rgba(219,234,254,.95)}.border-blue-200{--tw-border-opacity:1;border-color:rgb(191 219 254/var(--tw-border-opacity,1))}.border-blue-200\/0{border-color:rgba(191,219,254,0)}.border-blue-200\/10{border-color:rgba(191,219,254,.1)}.border-blue-200\/100{border-color:#bfdbfe}.border-blue-200\/15{border-color:rgba(191,219,254,.15)}.border-blue-200\/20{border-color:rgba(191,219,254,.2)}.border-blue-200\/25{border-color:rgba(191,219,254,.25)}.border-blue-200\/30{border-color:rgba(191,219,254,.3)}.border-blue-200\/35{border-color:rgba(191,219,254,.35)}.border-blue-200\/40{border-color:rgba(191,219,254,.4)}.border-blue-200\/45{border-color:rgba(191,219,254,.45)}.border-blue-200\/5{border-color:rgba(191,219,254,.05)}.border-blue-200\/50{border-color:rgba(191,219,254,.5)}.border-blue-200\/55{border-color:rgba(191,219,254,.55)}.border-blue-200\/60{border-color:rgba(191,219,254,.6)}.border-blue-200\/65{border-color:rgba(191,219,254,.65)}.border-blue-200\/70{border-color:rgba(191,219,254,.7)}.border-blue-200\/75{border-color:rgba(191,219,254,.75)}.border-blue-200\/80{border-color:rgba(191,219,254,.8)}.border-blue-200\/85{border-color:rgba(191,219,254,.85)}.border-blue-200\/90{border-color:rgba(191,219,254,.9)}.border-blue-200\/95{border-color:rgba(191,219,254,.95)}.border-blue-50{--tw-border-opacity:1;border-color:rgb(239 246 255/var(--tw-border-opacity,1))}.border-blue-50\/0{border-color:rgba(239,246,255,0)}.border-blue-50\/10{border-color:rgba(239,246,255,.1)}.border-blue-50\/100{border-color:#eff6ff}.border-blue-50\/15{border-color:rgba(239,246,255,.15)}.border-blue-50\/20{border-color:rgba(239,246,255,.2)}.border-blue-50\/25{border-color:rgba(239,246,255,.25)}.border-blue-50\/30{border-color:rgba(239,246,255,.3)}.border-blue-50\/35{border-color:rgba(239,246,255,.35)}.border-blue-50\/40{border-color:rgba(239,246,255,.4)}.border-blue-50\/45{border-color:rgba(239,246,255,.45)}.border-blue-50\/5{border-color:rgba(239,246,255,.05)}.border-blue-50\/50{border-color:rgba(239,246,255,.5)}.border-blue-50\/55{border-color:rgba(239,246,255,.55)}.border-blue-50\/60{border-color:rgba(239,246,255,.6)}.border-blue-50\/65{border-color:rgba(239,246,255,.65)}.border-blue-50\/70{border-color:rgba(239,246,255,.7)}.border-blue-50\/75{border-color:rgba(239,246,255,.75)}.border-blue-50\/80{border-color:rgba(239,246,255,.8)}.border-blue-50\/85{border-color:rgba(239,246,255,.85)}.border-blue-50\/90{border-color:rgba(239,246,255,.9)}.border-blue-50\/95{border-color:rgba(239,246,255,.95)}.border-blue-500{--tw-border-opacity:1;border-color:rgb(59 130 246/var(--tw-border-opacity,1))}.border-blue-500\/0{border-color:rgba(59,130,246,0)}.border-blue-500\/10{border-color:rgba(59,130,246,.1)}.border-blue-500\/100{border-color:#3b82f6}.border-blue-500\/15{border-color:rgba(59,130,246,.15)}.border-blue-500\/20{border-color:rgba(59,130,246,.2)}.border-blue-500\/25{border-color:rgba(59,130,246,.25)}.border-blue-500\/30{border-color:rgba(59,130,246,.3)}.border-blue-500\/35{border-color:rgba(59,130,246,.35)}.border-blue-500\/40{border-color:rgba(59,130,246,.4)}.border-blue-500\/45{border-color:rgba(59,130,246,.45)}.border-blue-500\/5{border-color:rgba(59,130,246,.05)}.border-blue-500\/50{border-color:rgba(59,130,246,.5)}.border-blue-500\/55{border-color:rgba(59,130,246,.55)}.border-blue-500\/60{border-color:rgba(59,130,246,.6)}.border-blue-500\/65{border-color:rgba(59,130,246,.65)}.border-blue-500\/70{border-color:rgba(59,130,246,.7)}.border-blue-500\/75{border-color:rgba(59,130,246,.75)}.border-blue-500\/80{border-color:rgba(59,130,246,.8)}.border-blue-500\/85{border-color:rgba(59,130,246,.85)}.border-blue-500\/90{border-color:rgba(59,130,246,.9)}.border-blue-500\/95{border-color:rgba(59,130,246,.95)}.border-blue-950{--tw-border-opacity:1;border-color:rgb(23 37 84/var(--tw-border-opacity,1))}.border-blue-950\/0{border-color:rgba(23,37,84,0)}.border-blue-950\/10{border-color:rgba(23,37,84,.1)}.border-blue-950\/100{border-color:#172554}.border-blue-950\/15{border-color:rgba(23,37,84,.15)}.border-blue-950\/20{border-color:rgba(23,37,84,.2)}.border-blue-950\/25{border-color:rgba(23,37,84,.25)}.border-blue-950\/30{border-color:rgba(23,37,84,.3)}.border-blue-950\/35{border-color:rgba(23,37,84,.35)}.border-blue-950\/40{border-color:rgba(23,37,84,.4)}.border-blue-950\/45{border-color:rgba(23,37,84,.45)}.border-blue-950\/5{border-color:rgba(23,37,84,.05)}.border-blue-950\/50{border-color:rgba(23,37,84,.5)}.border-blue-950\/55{border-color:rgba(23,37,84,.55)}.border-blue-950\/60{border-color:rgba(23,37,84,.6)}.border-blue-950\/65{border-color:rgba(23,37,84,.65)}.border-blue-950\/70{border-color:rgba(23,37,84,.7)}.border-blue-950\/75{border-color:rgba(23,37,84,.75)}.border-blue-950\/80{border-color:rgba(23,37,84,.8)}.border-blue-950\/85{border-color:rgba(23,37,84,.85)}.border-blue-950\/90{border-color:rgba(23,37,84,.9)}.border-blue-950\/95{border-color:rgba(23,37,84,.95)}.border-cyan-100{--tw-border-opacity:1;border-color:rgb(207 250 254/var(--tw-border-opacity,1))}.border-cyan-100\/0{border-color:rgba(207,250,254,0)}.border-cyan-100\/10{border-color:rgba(207,250,254,.1)}.border-cyan-100\/100{border-color:#cffafe}.border-cyan-100\/15{border-color:rgba(207,250,254,.15)}.border-cyan-100\/20{border-color:rgba(207,250,254,.2)}.border-cyan-100\/25{border-color:rgba(207,250,254,.25)}.border-cyan-100\/30{border-color:rgba(207,250,254,.3)}.border-cyan-100\/35{border-color:rgba(207,250,254,.35)}.border-cyan-100\/40{border-color:rgba(207,250,254,.4)}.border-cyan-100\/45{border-color:rgba(207,250,254,.45)}.border-cyan-100\/5{border-color:rgba(207,250,254,.05)}.border-cyan-100\/50{border-color:rgba(207,250,254,.5)}.border-cyan-100\/55{border-color:rgba(207,250,254,.55)}.border-cyan-100\/60{border-color:rgba(207,250,254,.6)}.border-cyan-100\/65{border-color:rgba(207,250,254,.65)}.border-cyan-100\/70{border-color:rgba(207,250,254,.7)}.border-cyan-100\/75{border-color:rgba(207,250,254,.75)}.border-cyan-100\/80{border-color:rgba(207,250,254,.8)}.border-cyan-100\/85{border-color:rgba(207,250,254,.85)}.border-cyan-100\/90{border-color:rgba(207,250,254,.9)}.border-cyan-100\/95{border-color:rgba(207,250,254,.95)}.border-cyan-200{--tw-border-opacity:1;border-color:rgb(165 243 252/var(--tw-border-opacity,1))}.border-cyan-200\/0{border-color:rgba(165,243,252,0)}.border-cyan-200\/10{border-color:rgba(165,243,252,.1)}.border-cyan-200\/100{border-color:#a5f3fc}.border-cyan-200\/15{border-color:rgba(165,243,252,.15)}.border-cyan-200\/20{border-color:rgba(165,243,252,.2)}.border-cyan-200\/25{border-color:rgba(165,243,252,.25)}.border-cyan-200\/30{border-color:rgba(165,243,252,.3)}.border-cyan-200\/35{border-color:rgba(165,243,252,.35)}.border-cyan-200\/40{border-color:rgba(165,243,252,.4)}.border-cyan-200\/45{border-color:rgba(165,243,252,.45)}.border-cyan-200\/5{border-color:rgba(165,243,252,.05)}.border-cyan-200\/50{border-color:rgba(165,243,252,.5)}.border-cyan-200\/55{border-color:rgba(165,243,252,.55)}.border-cyan-200\/60{border-color:rgba(165,243,252,.6)}.border-cyan-200\/65{border-color:rgba(165,243,252,.65)}.border-cyan-200\/70{border-color:rgba(165,243,252,.7)}.border-cyan-200\/75{border-color:rgba(165,243,252,.75)}.border-cyan-200\/80{border-color:rgba(165,243,252,.8)}.border-cyan-200\/85{border-color:rgba(165,243,252,.85)}.border-cyan-200\/90{border-color:rgba(165,243,252,.9)}.border-cyan-200\/95{border-color:rgba(165,243,252,.95)}.border-cyan-50{--tw-border-opacity:1;border-color:rgb(236 254 255/var(--tw-border-opacity,1))}.border-cyan-50\/0{border-color:rgba(236,254,255,0)}.border-cyan-50\/10{border-color:rgba(236,254,255,.1)}.border-cyan-50\/100{border-color:#ecfeff}.border-cyan-50\/15{border-color:rgba(236,254,255,.15)}.border-cyan-50\/20{border-color:rgba(236,254,255,.2)}.border-cyan-50\/25{border-color:rgba(236,254,255,.25)}.border-cyan-50\/30{border-color:rgba(236,254,255,.3)}.border-cyan-50\/35{border-color:rgba(236,254,255,.35)}.border-cyan-50\/40{border-color:rgba(236,254,255,.4)}.border-cyan-50\/45{border-color:rgba(236,254,255,.45)}.border-cyan-50\/5{border-color:rgba(236,254,255,.05)}.border-cyan-50\/50{border-color:rgba(236,254,255,.5)}.border-cyan-50\/55{border-color:rgba(236,254,255,.55)}.border-cyan-50\/60{border-color:rgba(236,254,255,.6)}.border-cyan-50\/65{border-color:rgba(236,254,255,.65)}.border-cyan-50\/70{border-color:rgba(236,254,255,.7)}.border-cyan-50\/75{border-color:rgba(236,254,255,.75)}.border-cyan-50\/80{border-color:rgba(236,254,255,.8)}.border-cyan-50\/85{border-color:rgba(236,254,255,.85)}.border-cyan-50\/90{border-color:rgba(236,254,255,.9)}.border-cyan-50\/95{border-color:rgba(236,254,255,.95)}.border-cyan-500{--tw-border-opacity:1;border-color:rgb(6 182 212/var(--tw-border-opacity,1))}.border-cyan-500\/0{border-color:rgba(6,182,212,0)}.border-cyan-500\/10{border-color:rgba(6,182,212,.1)}.border-cyan-500\/100{border-color:#06b6d4}.border-cyan-500\/15{border-color:rgba(6,182,212,.15)}.border-cyan-500\/20{border-color:rgba(6,182,212,.2)}.border-cyan-500\/25{border-color:rgba(6,182,212,.25)}.border-cyan-500\/30{border-color:rgba(6,182,212,.3)}.border-cyan-500\/35{border-color:rgba(6,182,212,.35)}.border-cyan-500\/40{border-color:rgba(6,182,212,.4)}.border-cyan-500\/45{border-color:rgba(6,182,212,.45)}.border-cyan-500\/5{border-color:rgba(6,182,212,.05)}.border-cyan-500\/50{border-color:rgba(6,182,212,.5)}.border-cyan-500\/55{border-color:rgba(6,182,212,.55)}.border-cyan-500\/60{border-color:rgba(6,182,212,.6)}.border-cyan-500\/65{border-color:rgba(6,182,212,.65)}.border-cyan-500\/70{border-color:rgba(6,182,212,.7)}.border-cyan-500\/75{border-color:rgba(6,182,212,.75)}.border-cyan-500\/80{border-color:rgba(6,182,212,.8)}.border-cyan-500\/85{border-color:rgba(6,182,212,.85)}.border-cyan-500\/90{border-color:rgba(6,182,212,.9)}.border-cyan-500\/95{border-color:rgba(6,182,212,.95)}.border-cyan-950{--tw-border-opacity:1;border-color:rgb(8 51 68/var(--tw-border-opacity,1))}.border-cyan-950\/0{border-color:rgba(8,51,68,0)}.border-cyan-950\/10{border-color:rgba(8,51,68,.1)}.border-cyan-950\/100{border-color:#083344}.border-cyan-950\/15{border-color:rgba(8,51,68,.15)}.border-cyan-950\/20{border-color:rgba(8,51,68,.2)}.border-cyan-950\/25{border-color:rgba(8,51,68,.25)}.border-cyan-950\/30{border-color:rgba(8,51,68,.3)}.border-cyan-950\/35{border-color:rgba(8,51,68,.35)}.border-cyan-950\/40{border-color:rgba(8,51,68,.4)}.border-cyan-950\/45{border-color:rgba(8,51,68,.45)}.border-cyan-950\/5{border-color:rgba(8,51,68,.05)}.border-cyan-950\/50{border-color:rgba(8,51,68,.5)}.border-cyan-950\/55{border-color:rgba(8,51,68,.55)}.border-cyan-950\/60{border-color:rgba(8,51,68,.6)}.border-cyan-950\/65{border-color:rgba(8,51,68,.65)}.border-cyan-950\/70{border-color:rgba(8,51,68,.7)}.border-cyan-950\/75{border-color:rgba(8,51,68,.75)}.border-cyan-950\/80{border-color:rgba(8,51,68,.8)}.border-cyan-950\/85{border-color:rgba(8,51,68,.85)}.border-cyan-950\/90{border-color:rgba(8,51,68,.9)}.border-cyan-950\/95{border-color:rgba(8,51,68,.95)}.border-destructive{border-color:hsl(var(--destructive))}.border-destructive\/50{border-color:hsl(var(--destructive)/.5)}.border-emerald-100{--tw-border-opacity:1;border-color:rgb(209 250 229/var(--tw-border-opacity,1))}.border-emerald-100\/0{border-color:rgba(209,250,229,0)}.border-emerald-100\/10{border-color:rgba(209,250,229,.1)}.border-emerald-100\/100{border-color:#d1fae5}.border-emerald-100\/15{border-color:rgba(209,250,229,.15)}.border-emerald-100\/20{border-color:rgba(209,250,229,.2)}.border-emerald-100\/25{border-color:rgba(209,250,229,.25)}.border-emerald-100\/30{border-color:rgba(209,250,229,.3)}.border-emerald-100\/35{border-color:rgba(209,250,229,.35)}.border-emerald-100\/40{border-color:rgba(209,250,229,.4)}.border-emerald-100\/45{border-color:rgba(209,250,229,.45)}.border-emerald-100\/5{border-color:rgba(209,250,229,.05)}.border-emerald-100\/50{border-color:rgba(209,250,229,.5)}.border-emerald-100\/55{border-color:rgba(209,250,229,.55)}.border-emerald-100\/60{border-color:rgba(209,250,229,.6)}.border-emerald-100\/65{border-color:rgba(209,250,229,.65)}.border-emerald-100\/70{border-color:rgba(209,250,229,.7)}.border-emerald-100\/75{border-color:rgba(209,250,229,.75)}.border-emerald-100\/80{border-color:rgba(209,250,229,.8)}.border-emerald-100\/85{border-color:rgba(209,250,229,.85)}.border-emerald-100\/90{border-color:rgba(209,250,229,.9)}.border-emerald-100\/95{border-color:rgba(209,250,229,.95)}.border-emerald-200{--tw-border-opacity:1;border-color:rgb(167 243 208/var(--tw-border-opacity,1))}.border-emerald-200\/0{border-color:rgba(167,243,208,0)}.border-emerald-200\/10{border-color:rgba(167,243,208,.1)}.border-emerald-200\/100{border-color:#a7f3d0}.border-emerald-200\/15{border-color:rgba(167,243,208,.15)}.border-emerald-200\/20{border-color:rgba(167,243,208,.2)}.border-emerald-200\/25{border-color:rgba(167,243,208,.25)}.border-emerald-200\/30{border-color:rgba(167,243,208,.3)}.border-emerald-200\/35{border-color:rgba(167,243,208,.35)}.border-emerald-200\/40{border-color:rgba(167,243,208,.4)}.border-emerald-200\/45{border-color:rgba(167,243,208,.45)}.border-emerald-200\/5{border-color:rgba(167,243,208,.05)}.border-emerald-200\/50{border-color:rgba(167,243,208,.5)}.border-emerald-200\/55{border-color:rgba(167,243,208,.55)}.border-emerald-200\/60{border-color:rgba(167,243,208,.6)}.border-emerald-200\/65{border-color:rgba(167,243,208,.65)}.border-emerald-200\/70{border-color:rgba(167,243,208,.7)}.border-emerald-200\/75{border-color:rgba(167,243,208,.75)}.border-emerald-200\/80{border-color:rgba(167,243,208,.8)}.border-emerald-200\/85{border-color:rgba(167,243,208,.85)}.border-emerald-200\/90{border-color:rgba(167,243,208,.9)}.border-emerald-200\/95{border-color:rgba(167,243,208,.95)}.border-emerald-50{--tw-border-opacity:1;border-color:rgb(236 253 245/var(--tw-border-opacity,1))}.border-emerald-50\/0{border-color:rgba(236,253,245,0)}.border-emerald-50\/10{border-color:rgba(236,253,245,.1)}.border-emerald-50\/100{border-color:#ecfdf5}.border-emerald-50\/15{border-color:rgba(236,253,245,.15)}.border-emerald-50\/20{border-color:rgba(236,253,245,.2)}.border-emerald-50\/25{border-color:rgba(236,253,245,.25)}.border-emerald-50\/30{border-color:rgba(236,253,245,.3)}.border-emerald-50\/35{border-color:rgba(236,253,245,.35)}.border-emerald-50\/40{border-color:rgba(236,253,245,.4)}.border-emerald-50\/45{border-color:rgba(236,253,245,.45)}.border-emerald-50\/5{border-color:rgba(236,253,245,.05)}.border-emerald-50\/50{border-color:rgba(236,253,245,.5)}.border-emerald-50\/55{border-color:rgba(236,253,245,.55)}.border-emerald-50\/60{border-color:rgba(236,253,245,.6)}.border-emerald-50\/65{border-color:rgba(236,253,245,.65)}.border-emerald-50\/70{border-color:rgba(236,253,245,.7)}.border-emerald-50\/75{border-color:rgba(236,253,245,.75)}.border-emerald-50\/80{border-color:rgba(236,253,245,.8)}.border-emerald-50\/85{border-color:rgba(236,253,245,.85)}.border-emerald-50\/90{border-color:rgba(236,253,245,.9)}.border-emerald-50\/95{border-color:rgba(236,253,245,.95)}.border-emerald-500{--tw-border-opacity:1;border-color:rgb(16 185 129/var(--tw-border-opacity,1))}.border-emerald-500\/0{border-color:rgba(16,185,129,0)}.border-emerald-500\/10{border-color:rgba(16,185,129,.1)}.border-emerald-500\/100{border-color:#10b981}.border-emerald-500\/15{border-color:rgba(16,185,129,.15)}.border-emerald-500\/20{border-color:rgba(16,185,129,.2)}.border-emerald-500\/25{border-color:rgba(16,185,129,.25)}.border-emerald-500\/30{border-color:rgba(16,185,129,.3)}.border-emerald-500\/35{border-color:rgba(16,185,129,.35)}.border-emerald-500\/40{border-color:rgba(16,185,129,.4)}.border-emerald-500\/45{border-color:rgba(16,185,129,.45)}.border-emerald-500\/5{border-color:rgba(16,185,129,.05)}.border-emerald-500\/50{border-color:rgba(16,185,129,.5)}.border-emerald-500\/55{border-color:rgba(16,185,129,.55)}.border-emerald-500\/60{border-color:rgba(16,185,129,.6)}.border-emerald-500\/65{border-color:rgba(16,185,129,.65)}.border-emerald-500\/70{border-color:rgba(16,185,129,.7)}.border-emerald-500\/75{border-color:rgba(16,185,129,.75)}.border-emerald-500\/80{border-color:rgba(16,185,129,.8)}.border-emerald-500\/85{border-color:rgba(16,185,129,.85)}.border-emerald-500\/90{border-color:rgba(16,185,129,.9)}.border-emerald-500\/95{border-color:rgba(16,185,129,.95)}.border-emerald-950{--tw-border-opacity:1;border-color:rgb(2 44 34/var(--tw-border-opacity,1))}.border-emerald-950\/0{border-color:rgba(2,44,34,0)}.border-emerald-950\/10{border-color:rgba(2,44,34,.1)}.border-emerald-950\/100{border-color:#022c22}.border-emerald-950\/15{border-color:rgba(2,44,34,.15)}.border-emerald-950\/20{border-color:rgba(2,44,34,.2)}.border-emerald-950\/25{border-color:rgba(2,44,34,.25)}.border-emerald-950\/30{border-color:rgba(2,44,34,.3)}.border-emerald-950\/35{border-color:rgba(2,44,34,.35)}.border-emerald-950\/40{border-color:rgba(2,44,34,.4)}.border-emerald-950\/45{border-color:rgba(2,44,34,.45)}.border-emerald-950\/5{border-color:rgba(2,44,34,.05)}.border-emerald-950\/50{border-color:rgba(2,44,34,.5)}.border-emerald-950\/55{border-color:rgba(2,44,34,.55)}.border-emerald-950\/60{border-color:rgba(2,44,34,.6)}.border-emerald-950\/65{border-color:rgba(2,44,34,.65)}.border-emerald-950\/70{border-color:rgba(2,44,34,.7)}.border-emerald-950\/75{border-color:rgba(2,44,34,.75)}.border-emerald-950\/80{border-color:rgba(2,44,34,.8)}.border-emerald-950\/85{border-color:rgba(2,44,34,.85)}.border-emerald-950\/90{border-color:rgba(2,44,34,.9)}.border-emerald-950\/95{border-color:rgba(2,44,34,.95)}.border-fuchsia-100{--tw-border-opacity:1;border-color:rgb(250 232 255/var(--tw-border-opacity,1))}.border-fuchsia-100\/0{border-color:rgba(250,232,255,0)}.border-fuchsia-100\/10{border-color:rgba(250,232,255,.1)}.border-fuchsia-100\/100{border-color:#fae8ff}.border-fuchsia-100\/15{border-color:rgba(250,232,255,.15)}.border-fuchsia-100\/20{border-color:rgba(250,232,255,.2)}.border-fuchsia-100\/25{border-color:rgba(250,232,255,.25)}.border-fuchsia-100\/30{border-color:rgba(250,232,255,.3)}.border-fuchsia-100\/35{border-color:rgba(250,232,255,.35)}.border-fuchsia-100\/40{border-color:rgba(250,232,255,.4)}.border-fuchsia-100\/45{border-color:rgba(250,232,255,.45)}.border-fuchsia-100\/5{border-color:rgba(250,232,255,.05)}.border-fuchsia-100\/50{border-color:rgba(250,232,255,.5)}.border-fuchsia-100\/55{border-color:rgba(250,232,255,.55)}.border-fuchsia-100\/60{border-color:rgba(250,232,255,.6)}.border-fuchsia-100\/65{border-color:rgba(250,232,255,.65)}.border-fuchsia-100\/70{border-color:rgba(250,232,255,.7)}.border-fuchsia-100\/75{border-color:rgba(250,232,255,.75)}.border-fuchsia-100\/80{border-color:rgba(250,232,255,.8)}.border-fuchsia-100\/85{border-color:rgba(250,232,255,.85)}.border-fuchsia-100\/90{border-color:rgba(250,232,255,.9)}.border-fuchsia-100\/95{border-color:rgba(250,232,255,.95)}.border-fuchsia-200{--tw-border-opacity:1;border-color:rgb(245 208 254/var(--tw-border-opacity,1))}.border-fuchsia-200\/0{border-color:rgba(245,208,254,0)}.border-fuchsia-200\/10{border-color:rgba(245,208,254,.1)}.border-fuchsia-200\/100{border-color:#f5d0fe}.border-fuchsia-200\/15{border-color:rgba(245,208,254,.15)}.border-fuchsia-200\/20{border-color:rgba(245,208,254,.2)}.border-fuchsia-200\/25{border-color:rgba(245,208,254,.25)}.border-fuchsia-200\/30{border-color:rgba(245,208,254,.3)}.border-fuchsia-200\/35{border-color:rgba(245,208,254,.35)}.border-fuchsia-200\/40{border-color:rgba(245,208,254,.4)}.border-fuchsia-200\/45{border-color:rgba(245,208,254,.45)}.border-fuchsia-200\/5{border-color:rgba(245,208,254,.05)}.border-fuchsia-200\/50{border-color:rgba(245,208,254,.5)}.border-fuchsia-200\/55{border-color:rgba(245,208,254,.55)}.border-fuchsia-200\/60{border-color:rgba(245,208,254,.6)}.border-fuchsia-200\/65{border-color:rgba(245,208,254,.65)}.border-fuchsia-200\/70{border-color:rgba(245,208,254,.7)}.border-fuchsia-200\/75{border-color:rgba(245,208,254,.75)}.border-fuchsia-200\/80{border-color:rgba(245,208,254,.8)}.border-fuchsia-200\/85{border-color:rgba(245,208,254,.85)}.border-fuchsia-200\/90{border-color:rgba(245,208,254,.9)}.border-fuchsia-200\/95{border-color:rgba(245,208,254,.95)}.border-fuchsia-50{--tw-border-opacity:1;border-color:rgb(253 244 255/var(--tw-border-opacity,1))}.border-fuchsia-50\/0{border-color:rgba(253,244,255,0)}.border-fuchsia-50\/10{border-color:rgba(253,244,255,.1)}.border-fuchsia-50\/100{border-color:#fdf4ff}.border-fuchsia-50\/15{border-color:rgba(253,244,255,.15)}.border-fuchsia-50\/20{border-color:rgba(253,244,255,.2)}.border-fuchsia-50\/25{border-color:rgba(253,244,255,.25)}.border-fuchsia-50\/30{border-color:rgba(253,244,255,.3)}.border-fuchsia-50\/35{border-color:rgba(253,244,255,.35)}.border-fuchsia-50\/40{border-color:rgba(253,244,255,.4)}.border-fuchsia-50\/45{border-color:rgba(253,244,255,.45)}.border-fuchsia-50\/5{border-color:rgba(253,244,255,.05)}.border-fuchsia-50\/50{border-color:rgba(253,244,255,.5)}.border-fuchsia-50\/55{border-color:rgba(253,244,255,.55)}.border-fuchsia-50\/60{border-color:rgba(253,244,255,.6)}.border-fuchsia-50\/65{border-color:rgba(253,244,255,.65)}.border-fuchsia-50\/70{border-color:rgba(253,244,255,.7)}.border-fuchsia-50\/75{border-color:rgba(253,244,255,.75)}.border-fuchsia-50\/80{border-color:rgba(253,244,255,.8)}.border-fuchsia-50\/85{border-color:rgba(253,244,255,.85)}.border-fuchsia-50\/90{border-color:rgba(253,244,255,.9)}.border-fuchsia-50\/95{border-color:rgba(253,244,255,.95)}.border-fuchsia-500{--tw-border-opacity:1;border-color:rgb(217 70 239/var(--tw-border-opacity,1))}.border-fuchsia-500\/0{border-color:rgba(217,70,239,0)}.border-fuchsia-500\/10{border-color:rgba(217,70,239,.1)}.border-fuchsia-500\/100{border-color:#d946ef}.border-fuchsia-500\/15{border-color:rgba(217,70,239,.15)}.border-fuchsia-500\/20{border-color:rgba(217,70,239,.2)}.border-fuchsia-500\/25{border-color:rgba(217,70,239,.25)}.border-fuchsia-500\/30{border-color:rgba(217,70,239,.3)}.border-fuchsia-500\/35{border-color:rgba(217,70,239,.35)}.border-fuchsia-500\/40{border-color:rgba(217,70,239,.4)}.border-fuchsia-500\/45{border-color:rgba(217,70,239,.45)}.border-fuchsia-500\/5{border-color:rgba(217,70,239,.05)}.border-fuchsia-500\/50{border-color:rgba(217,70,239,.5)}.border-fuchsia-500\/55{border-color:rgba(217,70,239,.55)}.border-fuchsia-500\/60{border-color:rgba(217,70,239,.6)}.border-fuchsia-500\/65{border-color:rgba(217,70,239,.65)}.border-fuchsia-500\/70{border-color:rgba(217,70,239,.7)}.border-fuchsia-500\/75{border-color:rgba(217,70,239,.75)}.border-fuchsia-500\/80{border-color:rgba(217,70,239,.8)}.border-fuchsia-500\/85{border-color:rgba(217,70,239,.85)}.border-fuchsia-500\/90{border-color:rgba(217,70,239,.9)}.border-fuchsia-500\/95{border-color:rgba(217,70,239,.95)}.border-fuchsia-950{--tw-border-opacity:1;border-color:rgb(74 4 78/var(--tw-border-opacity,1))}.border-fuchsia-950\/0{border-color:rgba(74,4,78,0)}.border-fuchsia-950\/10{border-color:rgba(74,4,78,.1)}.border-fuchsia-950\/100{border-color:#4a044e}.border-fuchsia-950\/15{border-color:rgba(74,4,78,.15)}.border-fuchsia-950\/20{border-color:rgba(74,4,78,.2)}.border-fuchsia-950\/25{border-color:rgba(74,4,78,.25)}.border-fuchsia-950\/30{border-color:rgba(74,4,78,.3)}.border-fuchsia-950\/35{border-color:rgba(74,4,78,.35)}.border-fuchsia-950\/40{border-color:rgba(74,4,78,.4)}.border-fuchsia-950\/45{border-color:rgba(74,4,78,.45)}.border-fuchsia-950\/5{border-color:rgba(74,4,78,.05)}.border-fuchsia-950\/50{border-color:rgba(74,4,78,.5)}.border-fuchsia-950\/55{border-color:rgba(74,4,78,.55)}.border-fuchsia-950\/60{border-color:rgba(74,4,78,.6)}.border-fuchsia-950\/65{border-color:rgba(74,4,78,.65)}.border-fuchsia-950\/70{border-color:rgba(74,4,78,.7)}.border-fuchsia-950\/75{border-color:rgba(74,4,78,.75)}.border-fuchsia-950\/80{border-color:rgba(74,4,78,.8)}.border-fuchsia-950\/85{border-color:rgba(74,4,78,.85)}.border-fuchsia-950\/90{border-color:rgba(74,4,78,.9)}.border-fuchsia-950\/95{border-color:rgba(74,4,78,.95)}.border-gray-100{--tw-border-opacity:1;border-color:rgb(243 244 246/var(--tw-border-opacity,1))}.border-gray-100\/0{border-color:rgba(243,244,246,0)}.border-gray-100\/10{border-color:rgba(243,244,246,.1)}.border-gray-100\/100{border-color:#f3f4f6}.border-gray-100\/15{border-color:rgba(243,244,246,.15)}.border-gray-100\/20{border-color:rgba(243,244,246,.2)}.border-gray-100\/25{border-color:rgba(243,244,246,.25)}.border-gray-100\/30{border-color:rgba(243,244,246,.3)}.border-gray-100\/35{border-color:rgba(243,244,246,.35)}.border-gray-100\/40{border-color:rgba(243,244,246,.4)}.border-gray-100\/45{border-color:rgba(243,244,246,.45)}.border-gray-100\/5{border-color:rgba(243,244,246,.05)}.border-gray-100\/50{border-color:rgba(243,244,246,.5)}.border-gray-100\/55{border-color:rgba(243,244,246,.55)}.border-gray-100\/60{border-color:rgba(243,244,246,.6)}.border-gray-100\/65{border-color:rgba(243,244,246,.65)}.border-gray-100\/70{border-color:rgba(243,244,246,.7)}.border-gray-100\/75{border-color:rgba(243,244,246,.75)}.border-gray-100\/80{border-color:rgba(243,244,246,.8)}.border-gray-100\/85{border-color:rgba(243,244,246,.85)}.border-gray-100\/90{border-color:rgba(243,244,246,.9)}.border-gray-100\/95{border-color:rgba(243,244,246,.95)}.border-gray-200{--tw-border-opacity:1;border-color:rgb(229 231 235/var(--tw-border-opacity,1))}.border-gray-200\/0{border-color:rgba(229,231,235,0)}.border-gray-200\/10{border-color:rgba(229,231,235,.1)}.border-gray-200\/100{border-color:#e5e7eb}.border-gray-200\/15{border-color:rgba(229,231,235,.15)}.border-gray-200\/20{border-color:rgba(229,231,235,.2)}.border-gray-200\/25{border-color:rgba(229,231,235,.25)}.border-gray-200\/30{border-color:rgba(229,231,235,.3)}.border-gray-200\/35{border-color:rgba(229,231,235,.35)}.border-gray-200\/40{border-color:rgba(229,231,235,.4)}.border-gray-200\/45{border-color:rgba(229,231,235,.45)}.border-gray-200\/5{border-color:rgba(229,231,235,.05)}.border-gray-200\/50{border-color:rgba(229,231,235,.5)}.border-gray-200\/55{border-color:rgba(229,231,235,.55)}.border-gray-200\/60{border-color:rgba(229,231,235,.6)}.border-gray-200\/65{border-color:rgba(229,231,235,.65)}.border-gray-200\/70{border-color:rgba(229,231,235,.7)}.border-gray-200\/75{border-color:rgba(229,231,235,.75)}.border-gray-200\/80{border-color:rgba(229,231,235,.8)}.border-gray-200\/85{border-color:rgba(229,231,235,.85)}.border-gray-200\/90{border-color:rgba(229,231,235,.9)}.border-gray-200\/95{border-color:rgba(229,231,235,.95)}.border-gray-300{--tw-border-opacity:1;border-color:rgb(209 213 219/var(--tw-border-opacity,1))}.border-gray-50{--tw-border-opacity:1;border-color:rgb(249 250 251/var(--tw-border-opacity,1))}.border-gray-50\/0{border-color:rgba(249,250,251,0)}.border-gray-50\/10{border-color:rgba(249,250,251,.1)}.border-gray-50\/100{border-color:#f9fafb}.border-gray-50\/15{border-color:rgba(249,250,251,.15)}.border-gray-50\/20{border-color:rgba(249,250,251,.2)}.border-gray-50\/25{border-color:rgba(249,250,251,.25)}.border-gray-50\/30{border-color:rgba(249,250,251,.3)}.border-gray-50\/35{border-color:rgba(249,250,251,.35)}.border-gray-50\/40{border-color:rgba(249,250,251,.4)}.border-gray-50\/45{border-color:rgba(249,250,251,.45)}.border-gray-50\/5{border-color:rgba(249,250,251,.05)}.border-gray-50\/50{border-color:rgba(249,250,251,.5)}.border-gray-50\/55{border-color:rgba(249,250,251,.55)}.border-gray-50\/60{border-color:rgba(249,250,251,.6)}.border-gray-50\/65{border-color:rgba(249,250,251,.65)}.border-gray-50\/70{border-color:rgba(249,250,251,.7)}.border-gray-50\/75{border-color:rgba(249,250,251,.75)}.border-gray-50\/80{border-color:rgba(249,250,251,.8)}.border-gray-50\/85{border-color:rgba(249,250,251,.85)}.border-gray-50\/90{border-color:rgba(249,250,251,.9)}.border-gray-50\/95{border-color:rgba(249,250,251,.95)}.border-gray-500{--tw-border-opacity:1;border-color:rgb(107 114 128/var(--tw-border-opacity,1))}.border-gray-500\/0{border-color:hsla(220,9%,46%,0)}.border-gray-500\/10{border-color:hsla(220,9%,46%,.1)}.border-gray-500\/100{border-color:#6b7280}.border-gray-500\/15{border-color:hsla(220,9%,46%,.15)}.border-gray-500\/20{border-color:hsla(220,9%,46%,.2)}.border-gray-500\/25{border-color:hsla(220,9%,46%,.25)}.border-gray-500\/30{border-color:hsla(220,9%,46%,.3)}.border-gray-500\/35{border-color:hsla(220,9%,46%,.35)}.border-gray-500\/40{border-color:hsla(220,9%,46%,.4)}.border-gray-500\/45{border-color:hsla(220,9%,46%,.45)}.border-gray-500\/5{border-color:hsla(220,9%,46%,.05)}.border-gray-500\/50{border-color:hsla(220,9%,46%,.5)}.border-gray-500\/55{border-color:hsla(220,9%,46%,.55)}.border-gray-500\/60{border-color:hsla(220,9%,46%,.6)}.border-gray-500\/65{border-color:hsla(220,9%,46%,.65)}.border-gray-500\/70{border-color:hsla(220,9%,46%,.7)}.border-gray-500\/75{border-color:hsla(220,9%,46%,.75)}.border-gray-500\/80{border-color:hsla(220,9%,46%,.8)}.border-gray-500\/85{border-color:hsla(220,9%,46%,.85)}.border-gray-500\/90{border-color:hsla(220,9%,46%,.9)}.border-gray-500\/95{border-color:hsla(220,9%,46%,.95)}.border-gray-950{--tw-border-opacity:1;border-color:rgb(3 7 18/var(--tw-border-opacity,1))}.border-gray-950\/0{border-color:rgba(3,7,18,0)}.border-gray-950\/10{border-color:rgba(3,7,18,.1)}.border-gray-950\/100{border-color:#030712}.border-gray-950\/15{border-color:rgba(3,7,18,.15)}.border-gray-950\/20{border-color:rgba(3,7,18,.2)}.border-gray-950\/25{border-color:rgba(3,7,18,.25)}.border-gray-950\/30{border-color:rgba(3,7,18,.3)}.border-gray-950\/35{border-color:rgba(3,7,18,.35)}.border-gray-950\/40{border-color:rgba(3,7,18,.4)}.border-gray-950\/45{border-color:rgba(3,7,18,.45)}.border-gray-950\/5{border-color:rgba(3,7,18,.05)}.border-gray-950\/50{border-color:rgba(3,7,18,.5)}.border-gray-950\/55{border-color:rgba(3,7,18,.55)}.border-gray-950\/60{border-color:rgba(3,7,18,.6)}.border-gray-950\/65{border-color:rgba(3,7,18,.65)}.border-gray-950\/70{border-color:rgba(3,7,18,.7)}.border-gray-950\/75{border-color:rgba(3,7,18,.75)}.border-gray-950\/80{border-color:rgba(3,7,18,.8)}.border-gray-950\/85{border-color:rgba(3,7,18,.85)}.border-gray-950\/90{border-color:rgba(3,7,18,.9)}.border-gray-950\/95{border-color:rgba(3,7,18,.95)}.border-green-100{--tw-border-opacity:1;border-color:rgb(220 252 231/var(--tw-border-opacity,1))}.border-green-100\/0{border-color:rgba(220,252,231,0)}.border-green-100\/10{border-color:rgba(220,252,231,.1)}.border-green-100\/100{border-color:#dcfce7}.border-green-100\/15{border-color:rgba(220,252,231,.15)}.border-green-100\/20{border-color:rgba(220,252,231,.2)}.border-green-100\/25{border-color:rgba(220,252,231,.25)}.border-green-100\/30{border-color:rgba(220,252,231,.3)}.border-green-100\/35{border-color:rgba(220,252,231,.35)}.border-green-100\/40{border-color:rgba(220,252,231,.4)}.border-green-100\/45{border-color:rgba(220,252,231,.45)}.border-green-100\/5{border-color:rgba(220,252,231,.05)}.border-green-100\/50{border-color:rgba(220,252,231,.5)}.border-green-100\/55{border-color:rgba(220,252,231,.55)}.border-green-100\/60{border-color:rgba(220,252,231,.6)}.border-green-100\/65{border-color:rgba(220,252,231,.65)}.border-green-100\/70{border-color:rgba(220,252,231,.7)}.border-green-100\/75{border-color:rgba(220,252,231,.75)}.border-green-100\/80{border-color:rgba(220,252,231,.8)}.border-green-100\/85{border-color:rgba(220,252,231,.85)}.border-green-100\/90{border-color:rgba(220,252,231,.9)}.border-green-100\/95{border-color:rgba(220,252,231,.95)}.border-green-200{--tw-border-opacity:1;border-color:rgb(187 247 208/var(--tw-border-opacity,1))}.border-green-200\/0{border-color:rgba(187,247,208,0)}.border-green-200\/10{border-color:rgba(187,247,208,.1)}.border-green-200\/100{border-color:#bbf7d0}.border-green-200\/15{border-color:rgba(187,247,208,.15)}.border-green-200\/20{border-color:rgba(187,247,208,.2)}.border-green-200\/25{border-color:rgba(187,247,208,.25)}.border-green-200\/30{border-color:rgba(187,247,208,.3)}.border-green-200\/35{border-color:rgba(187,247,208,.35)}.border-green-200\/40{border-color:rgba(187,247,208,.4)}.border-green-200\/45{border-color:rgba(187,247,208,.45)}.border-green-200\/5{border-color:rgba(187,247,208,.05)}.border-green-200\/50{border-color:rgba(187,247,208,.5)}.border-green-200\/55{border-color:rgba(187,247,208,.55)}.border-green-200\/60{border-color:rgba(187,247,208,.6)}.border-green-200\/65{border-color:rgba(187,247,208,.65)}.border-green-200\/70{border-color:rgba(187,247,208,.7)}.border-green-200\/75{border-color:rgba(187,247,208,.75)}.border-green-200\/80{border-color:rgba(187,247,208,.8)}.border-green-200\/85{border-color:rgba(187,247,208,.85)}.border-green-200\/90{border-color:rgba(187,247,208,.9)}.border-green-200\/95{border-color:rgba(187,247,208,.95)}.border-green-50{--tw-border-opacity:1;border-color:rgb(240 253 244/var(--tw-border-opacity,1))}.border-green-50\/0{border-color:rgba(240,253,244,0)}.border-green-50\/10{border-color:rgba(240,253,244,.1)}.border-green-50\/100{border-color:#f0fdf4}.border-green-50\/15{border-color:rgba(240,253,244,.15)}.border-green-50\/20{border-color:rgba(240,253,244,.2)}.border-green-50\/25{border-color:rgba(240,253,244,.25)}.border-green-50\/30{border-color:rgba(240,253,244,.3)}.border-green-50\/35{border-color:rgba(240,253,244,.35)}.border-green-50\/40{border-color:rgba(240,253,244,.4)}.border-green-50\/45{border-color:rgba(240,253,244,.45)}.border-green-50\/5{border-color:rgba(240,253,244,.05)}.border-green-50\/50{border-color:rgba(240,253,244,.5)}.border-green-50\/55{border-color:rgba(240,253,244,.55)}.border-green-50\/60{border-color:rgba(240,253,244,.6)}.border-green-50\/65{border-color:rgba(240,253,244,.65)}.border-green-50\/70{border-color:rgba(240,253,244,.7)}.border-green-50\/75{border-color:rgba(240,253,244,.75)}.border-green-50\/80{border-color:rgba(240,253,244,.8)}.border-green-50\/85{border-color:rgba(240,253,244,.85)}.border-green-50\/90{border-color:rgba(240,253,244,.9)}.border-green-50\/95{border-color:rgba(240,253,244,.95)}.border-green-500{--tw-border-opacity:1;border-color:rgb(34 197 94/var(--tw-border-opacity,1))}.border-green-500\/0{border-color:rgba(34,197,94,0)}.border-green-500\/10{border-color:rgba(34,197,94,.1)}.border-green-500\/100{border-color:#22c55e}.border-green-500\/15{border-color:rgba(34,197,94,.15)}.border-green-500\/20{border-color:rgba(34,197,94,.2)}.border-green-500\/25{border-color:rgba(34,197,94,.25)}.border-green-500\/30{border-color:rgba(34,197,94,.3)}.border-green-500\/35{border-color:rgba(34,197,94,.35)}.border-green-500\/40{border-color:rgba(34,197,94,.4)}.border-green-500\/45{border-color:rgba(34,197,94,.45)}.border-green-500\/5{border-color:rgba(34,197,94,.05)}.border-green-500\/50{border-color:rgba(34,197,94,.5)}.border-green-500\/55{border-color:rgba(34,197,94,.55)}.border-green-500\/60{border-color:rgba(34,197,94,.6)}.border-green-500\/65{border-color:rgba(34,197,94,.65)}.border-green-500\/70{border-color:rgba(34,197,94,.7)}.border-green-500\/75{border-color:rgba(34,197,94,.75)}.border-green-500\/80{border-color:rgba(34,197,94,.8)}.border-green-500\/85{border-color:rgba(34,197,94,.85)}.border-green-500\/90{border-color:rgba(34,197,94,.9)}.border-green-500\/95{border-color:rgba(34,197,94,.95)}.border-green-950{--tw-border-opacity:1;border-color:rgb(5 46 22/var(--tw-border-opacity,1))}.border-green-950\/0{border-color:rgba(5,46,22,0)}.border-green-950\/10{border-color:rgba(5,46,22,.1)}.border-green-950\/100{border-color:#052e16}.border-green-950\/15{border-color:rgba(5,46,22,.15)}.border-green-950\/20{border-color:rgba(5,46,22,.2)}.border-green-950\/25{border-color:rgba(5,46,22,.25)}.border-green-950\/30{border-color:rgba(5,46,22,.3)}.border-green-950\/35{border-color:rgba(5,46,22,.35)}.border-green-950\/40{border-color:rgba(5,46,22,.4)}.border-green-950\/45{border-color:rgba(5,46,22,.45)}.border-green-950\/5{border-color:rgba(5,46,22,.05)}.border-green-950\/50{border-color:rgba(5,46,22,.5)}.border-green-950\/55{border-color:rgba(5,46,22,.55)}.border-green-950\/60{border-color:rgba(5,46,22,.6)}.border-green-950\/65{border-color:rgba(5,46,22,.65)}.border-green-950\/70{border-color:rgba(5,46,22,.7)}.border-green-950\/75{border-color:rgba(5,46,22,.75)}.border-green-950\/80{border-color:rgba(5,46,22,.8)}.border-green-950\/85{border-color:rgba(5,46,22,.85)}.border-green-950\/90{border-color:rgba(5,46,22,.9)}.border-green-950\/95{border-color:rgba(5,46,22,.95)}.border-indigo-100{--tw-border-opacity:1;border-color:rgb(224 231 255/var(--tw-border-opacity,1))}.border-indigo-100\/0{border-color:rgba(224,231,255,0)}.border-indigo-100\/10{border-color:rgba(224,231,255,.1)}.border-indigo-100\/100{border-color:#e0e7ff}.border-indigo-100\/15{border-color:rgba(224,231,255,.15)}.border-indigo-100\/20{border-color:rgba(224,231,255,.2)}.border-indigo-100\/25{border-color:rgba(224,231,255,.25)}.border-indigo-100\/30{border-color:rgba(224,231,255,.3)}.border-indigo-100\/35{border-color:rgba(224,231,255,.35)}.border-indigo-100\/40{border-color:rgba(224,231,255,.4)}.border-indigo-100\/45{border-color:rgba(224,231,255,.45)}.border-indigo-100\/5{border-color:rgba(224,231,255,.05)}.border-indigo-100\/50{border-color:rgba(224,231,255,.5)}.border-indigo-100\/55{border-color:rgba(224,231,255,.55)}.border-indigo-100\/60{border-color:rgba(224,231,255,.6)}.border-indigo-100\/65{border-color:rgba(224,231,255,.65)}.border-indigo-100\/70{border-color:rgba(224,231,255,.7)}.border-indigo-100\/75{border-color:rgba(224,231,255,.75)}.border-indigo-100\/80{border-color:rgba(224,231,255,.8)}.border-indigo-100\/85{border-color:rgba(224,231,255,.85)}.border-indigo-100\/90{border-color:rgba(224,231,255,.9)}.border-indigo-100\/95{border-color:rgba(224,231,255,.95)}.border-indigo-200{--tw-border-opacity:1;border-color:rgb(199 210 254/var(--tw-border-opacity,1))}.border-indigo-200\/0{border-color:rgba(199,210,254,0)}.border-indigo-200\/10{border-color:rgba(199,210,254,.1)}.border-indigo-200\/100{border-color:#c7d2fe}.border-indigo-200\/15{border-color:rgba(199,210,254,.15)}.border-indigo-200\/20{border-color:rgba(199,210,254,.2)}.border-indigo-200\/25{border-color:rgba(199,210,254,.25)}.border-indigo-200\/30{border-color:rgba(199,210,254,.3)}.border-indigo-200\/35{border-color:rgba(199,210,254,.35)}.border-indigo-200\/40{border-color:rgba(199,210,254,.4)}.border-indigo-200\/45{border-color:rgba(199,210,254,.45)}.border-indigo-200\/5{border-color:rgba(199,210,254,.05)}.border-indigo-200\/50{border-color:rgba(199,210,254,.5)}.border-indigo-200\/55{border-color:rgba(199,210,254,.55)}.border-indigo-200\/60{border-color:rgba(199,210,254,.6)}.border-indigo-200\/65{border-color:rgba(199,210,254,.65)}.border-indigo-200\/70{border-color:rgba(199,210,254,.7)}.border-indigo-200\/75{border-color:rgba(199,210,254,.75)}.border-indigo-200\/80{border-color:rgba(199,210,254,.8)}.border-indigo-200\/85{border-color:rgba(199,210,254,.85)}.border-indigo-200\/90{border-color:rgba(199,210,254,.9)}.border-indigo-200\/95{border-color:rgba(199,210,254,.95)}.border-indigo-50{--tw-border-opacity:1;border-color:rgb(238 242 255/var(--tw-border-opacity,1))}.border-indigo-50\/0{border-color:rgba(238,242,255,0)}.border-indigo-50\/10{border-color:rgba(238,242,255,.1)}.border-indigo-50\/100{border-color:#eef2ff}.border-indigo-50\/15{border-color:rgba(238,242,255,.15)}.border-indigo-50\/20{border-color:rgba(238,242,255,.2)}.border-indigo-50\/25{border-color:rgba(238,242,255,.25)}.border-indigo-50\/30{border-color:rgba(238,242,255,.3)}.border-indigo-50\/35{border-color:rgba(238,242,255,.35)}.border-indigo-50\/40{border-color:rgba(238,242,255,.4)}.border-indigo-50\/45{border-color:rgba(238,242,255,.45)}.border-indigo-50\/5{border-color:rgba(238,242,255,.05)}.border-indigo-50\/50{border-color:rgba(238,242,255,.5)}.border-indigo-50\/55{border-color:rgba(238,242,255,.55)}.border-indigo-50\/60{border-color:rgba(238,242,255,.6)}.border-indigo-50\/65{border-color:rgba(238,242,255,.65)}.border-indigo-50\/70{border-color:rgba(238,242,255,.7)}.border-indigo-50\/75{border-color:rgba(238,242,255,.75)}.border-indigo-50\/80{border-color:rgba(238,242,255,.8)}.border-indigo-50\/85{border-color:rgba(238,242,255,.85)}.border-indigo-50\/90{border-color:rgba(238,242,255,.9)}.border-indigo-50\/95{border-color:rgba(238,242,255,.95)}.border-indigo-500{--tw-border-opacity:1;border-color:rgb(99 102 241/var(--tw-border-opacity,1))}.border-indigo-500\/0{border-color:rgba(99,102,241,0)}.border-indigo-500\/10{border-color:rgba(99,102,241,.1)}.border-indigo-500\/100{border-color:#6366f1}.border-indigo-500\/15{border-color:rgba(99,102,241,.15)}.border-indigo-500\/20{border-color:rgba(99,102,241,.2)}.border-indigo-500\/25{border-color:rgba(99,102,241,.25)}.border-indigo-500\/30{border-color:rgba(99,102,241,.3)}.border-indigo-500\/35{border-color:rgba(99,102,241,.35)}.border-indigo-500\/40{border-color:rgba(99,102,241,.4)}.border-indigo-500\/45{border-color:rgba(99,102,241,.45)}.border-indigo-500\/5{border-color:rgba(99,102,241,.05)}.border-indigo-500\/50{border-color:rgba(99,102,241,.5)}.border-indigo-500\/55{border-color:rgba(99,102,241,.55)}.border-indigo-500\/60{border-color:rgba(99,102,241,.6)}.border-indigo-500\/65{border-color:rgba(99,102,241,.65)}.border-indigo-500\/70{border-color:rgba(99,102,241,.7)}.border-indigo-500\/75{border-color:rgba(99,102,241,.75)}.border-indigo-500\/80{border-color:rgba(99,102,241,.8)}.border-indigo-500\/85{border-color:rgba(99,102,241,.85)}.border-indigo-500\/90{border-color:rgba(99,102,241,.9)}.border-indigo-500\/95{border-color:rgba(99,102,241,.95)}.border-indigo-950{--tw-border-opacity:1;border-color:rgb(30 27 75/var(--tw-border-opacity,1))}.border-indigo-950\/0{border-color:rgba(30,27,75,0)}.border-indigo-950\/10{border-color:rgba(30,27,75,.1)}.border-indigo-950\/100{border-color:#1e1b4b}.border-indigo-950\/15{border-color:rgba(30,27,75,.15)}.border-indigo-950\/20{border-color:rgba(30,27,75,.2)}.border-indigo-950\/25{border-color:rgba(30,27,75,.25)}.border-indigo-950\/30{border-color:rgba(30,27,75,.3)}.border-indigo-950\/35{border-color:rgba(30,27,75,.35)}.border-indigo-950\/40{border-color:rgba(30,27,75,.4)}.border-indigo-950\/45{border-color:rgba(30,27,75,.45)}.border-indigo-950\/5{border-color:rgba(30,27,75,.05)}.border-indigo-950\/50{border-color:rgba(30,27,75,.5)}.border-indigo-950\/55{border-color:rgba(30,27,75,.55)}.border-indigo-950\/60{border-color:rgba(30,27,75,.6)}.border-indigo-950\/65{border-color:rgba(30,27,75,.65)}.border-indigo-950\/70{border-color:rgba(30,27,75,.7)}.border-indigo-950\/75{border-color:rgba(30,27,75,.75)}.border-indigo-950\/80{border-color:rgba(30,27,75,.8)}.border-indigo-950\/85{border-color:rgba(30,27,75,.85)}.border-indigo-950\/90{border-color:rgba(30,27,75,.9)}.border-indigo-950\/95{border-color:rgba(30,27,75,.95)}.border-input{border-color:hsl(var(--input))}.border-lime-100{--tw-border-opacity:1;border-color:rgb(236 252 203/var(--tw-border-opacity,1))}.border-lime-100\/0{border-color:hsla(80,89%,89%,0)}.border-lime-100\/10{border-color:hsla(80,89%,89%,.1)}.border-lime-100\/100{border-color:#ecfccb}.border-lime-100\/15{border-color:hsla(80,89%,89%,.15)}.border-lime-100\/20{border-color:hsla(80,89%,89%,.2)}.border-lime-100\/25{border-color:hsla(80,89%,89%,.25)}.border-lime-100\/30{border-color:hsla(80,89%,89%,.3)}.border-lime-100\/35{border-color:hsla(80,89%,89%,.35)}.border-lime-100\/40{border-color:hsla(80,89%,89%,.4)}.border-lime-100\/45{border-color:hsla(80,89%,89%,.45)}.border-lime-100\/5{border-color:hsla(80,89%,89%,.05)}.border-lime-100\/50{border-color:hsla(80,89%,89%,.5)}.border-lime-100\/55{border-color:hsla(80,89%,89%,.55)}.border-lime-100\/60{border-color:hsla(80,89%,89%,.6)}.border-lime-100\/65{border-color:hsla(80,89%,89%,.65)}.border-lime-100\/70{border-color:hsla(80,89%,89%,.7)}.border-lime-100\/75{border-color:hsla(80,89%,89%,.75)}.border-lime-100\/80{border-color:hsla(80,89%,89%,.8)}.border-lime-100\/85{border-color:hsla(80,89%,89%,.85)}.border-lime-100\/90{border-color:hsla(80,89%,89%,.9)}.border-lime-100\/95{border-color:hsla(80,89%,89%,.95)}.border-lime-200{--tw-border-opacity:1;border-color:rgb(217 249 157/var(--tw-border-opacity,1))}.border-lime-200\/0{border-color:hsla(81,88%,80%,0)}.border-lime-200\/10{border-color:hsla(81,88%,80%,.1)}.border-lime-200\/100{border-color:#d9f99d}.border-lime-200\/15{border-color:hsla(81,88%,80%,.15)}.border-lime-200\/20{border-color:hsla(81,88%,80%,.2)}.border-lime-200\/25{border-color:hsla(81,88%,80%,.25)}.border-lime-200\/30{border-color:hsla(81,88%,80%,.3)}.border-lime-200\/35{border-color:hsla(81,88%,80%,.35)}.border-lime-200\/40{border-color:hsla(81,88%,80%,.4)}.border-lime-200\/45{border-color:hsla(81,88%,80%,.45)}.border-lime-200\/5{border-color:hsla(81,88%,80%,.05)}.border-lime-200\/50{border-color:hsla(81,88%,80%,.5)}.border-lime-200\/55{border-color:hsla(81,88%,80%,.55)}.border-lime-200\/60{border-color:hsla(81,88%,80%,.6)}.border-lime-200\/65{border-color:hsla(81,88%,80%,.65)}.border-lime-200\/70{border-color:hsla(81,88%,80%,.7)}.border-lime-200\/75{border-color:hsla(81,88%,80%,.75)}.border-lime-200\/80{border-color:hsla(81,88%,80%,.8)}.border-lime-200\/85{border-color:hsla(81,88%,80%,.85)}.border-lime-200\/90{border-color:hsla(81,88%,80%,.9)}.border-lime-200\/95{border-color:hsla(81,88%,80%,.95)}.border-lime-50{--tw-border-opacity:1;border-color:rgb(247 254 231/var(--tw-border-opacity,1))}.border-lime-50\/0{border-color:hsla(78,92%,95%,0)}.border-lime-50\/10{border-color:hsla(78,92%,95%,.1)}.border-lime-50\/100{border-color:#f7fee7}.border-lime-50\/15{border-color:hsla(78,92%,95%,.15)}.border-lime-50\/20{border-color:hsla(78,92%,95%,.2)}.border-lime-50\/25{border-color:hsla(78,92%,95%,.25)}.border-lime-50\/30{border-color:hsla(78,92%,95%,.3)}.border-lime-50\/35{border-color:hsla(78,92%,95%,.35)}.border-lime-50\/40{border-color:hsla(78,92%,95%,.4)}.border-lime-50\/45{border-color:hsla(78,92%,95%,.45)}.border-lime-50\/5{border-color:hsla(78,92%,95%,.05)}.border-lime-50\/50{border-color:hsla(78,92%,95%,.5)}.border-lime-50\/55{border-color:hsla(78,92%,95%,.55)}.border-lime-50\/60{border-color:hsla(78,92%,95%,.6)}.border-lime-50\/65{border-color:hsla(78,92%,95%,.65)}.border-lime-50\/70{border-color:hsla(78,92%,95%,.7)}.border-lime-50\/75{border-color:hsla(78,92%,95%,.75)}.border-lime-50\/80{border-color:hsla(78,92%,95%,.8)}.border-lime-50\/85{border-color:hsla(78,92%,95%,.85)}.border-lime-50\/90{border-color:hsla(78,92%,95%,.9)}.border-lime-50\/95{border-color:hsla(78,92%,95%,.95)}.border-lime-500{--tw-border-opacity:1;border-color:rgb(132 204 22/var(--tw-border-opacity,1))}.border-lime-500\/0{border-color:rgba(132,204,22,0)}.border-lime-500\/10{border-color:rgba(132,204,22,.1)}.border-lime-500\/100{border-color:#84cc16}.border-lime-500\/15{border-color:rgba(132,204,22,.15)}.border-lime-500\/20{border-color:rgba(132,204,22,.2)}.border-lime-500\/25{border-color:rgba(132,204,22,.25)}.border-lime-500\/30{border-color:rgba(132,204,22,.3)}.border-lime-500\/35{border-color:rgba(132,204,22,.35)}.border-lime-500\/40{border-color:rgba(132,204,22,.4)}.border-lime-500\/45{border-color:rgba(132,204,22,.45)}.border-lime-500\/5{border-color:rgba(132,204,22,.05)}.border-lime-500\/50{border-color:rgba(132,204,22,.5)}.border-lime-500\/55{border-color:rgba(132,204,22,.55)}.border-lime-500\/60{border-color:rgba(132,204,22,.6)}.border-lime-500\/65{border-color:rgba(132,204,22,.65)}.border-lime-500\/70{border-color:rgba(132,204,22,.7)}.border-lime-500\/75{border-color:rgba(132,204,22,.75)}.border-lime-500\/80{border-color:rgba(132,204,22,.8)}.border-lime-500\/85{border-color:rgba(132,204,22,.85)}.border-lime-500\/90{border-color:rgba(132,204,22,.9)}.border-lime-500\/95{border-color:rgba(132,204,22,.95)}.border-lime-950{--tw-border-opacity:1;border-color:rgb(26 46 5/var(--tw-border-opacity,1))}.border-lime-950\/0{border-color:rgba(26,46,5,0)}.border-lime-950\/10{border-color:rgba(26,46,5,.1)}.border-lime-950\/100{border-color:#1a2e05}.border-lime-950\/15{border-color:rgba(26,46,5,.15)}.border-lime-950\/20{border-color:rgba(26,46,5,.2)}.border-lime-950\/25{border-color:rgba(26,46,5,.25)}.border-lime-950\/30{border-color:rgba(26,46,5,.3)}.border-lime-950\/35{border-color:rgba(26,46,5,.35)}.border-lime-950\/40{border-color:rgba(26,46,5,.4)}.border-lime-950\/45{border-color:rgba(26,46,5,.45)}.border-lime-950\/5{border-color:rgba(26,46,5,.05)}.border-lime-950\/50{border-color:rgba(26,46,5,.5)}.border-lime-950\/55{border-color:rgba(26,46,5,.55)}.border-lime-950\/60{border-color:rgba(26,46,5,.6)}.border-lime-950\/65{border-color:rgba(26,46,5,.65)}.border-lime-950\/70{border-color:rgba(26,46,5,.7)}.border-lime-950\/75{border-color:rgba(26,46,5,.75)}.border-lime-950\/80{border-color:rgba(26,46,5,.8)}.border-lime-950\/85{border-color:rgba(26,46,5,.85)}.border-lime-950\/90{border-color:rgba(26,46,5,.9)}.border-lime-950\/95{border-color:rgba(26,46,5,.95)}.border-muted{border-color:hsl(var(--muted))}.border-muted-foreground{border-color:hsl(var(--muted-foreground))}.border-neutral-100{--tw-border-opacity:1;border-color:rgb(245 245 245/var(--tw-border-opacity,1))}.border-neutral-100\/0{border-color:hsla(0,0%,96%,0)}.border-neutral-100\/10{border-color:hsla(0,0%,96%,.1)}.border-neutral-100\/100{border-color:#f5f5f5}.border-neutral-100\/15{border-color:hsla(0,0%,96%,.15)}.border-neutral-100\/20{border-color:hsla(0,0%,96%,.2)}.border-neutral-100\/25{border-color:hsla(0,0%,96%,.25)}.border-neutral-100\/30{border-color:hsla(0,0%,96%,.3)}.border-neutral-100\/35{border-color:hsla(0,0%,96%,.35)}.border-neutral-100\/40{border-color:hsla(0,0%,96%,.4)}.border-neutral-100\/45{border-color:hsla(0,0%,96%,.45)}.border-neutral-100\/5{border-color:hsla(0,0%,96%,.05)}.border-neutral-100\/50{border-color:hsla(0,0%,96%,.5)}.border-neutral-100\/55{border-color:hsla(0,0%,96%,.55)}.border-neutral-100\/60{border-color:hsla(0,0%,96%,.6)}.border-neutral-100\/65{border-color:hsla(0,0%,96%,.65)}.border-neutral-100\/70{border-color:hsla(0,0%,96%,.7)}.border-neutral-100\/75{border-color:hsla(0,0%,96%,.75)}.border-neutral-100\/80{border-color:hsla(0,0%,96%,.8)}.border-neutral-100\/85{border-color:hsla(0,0%,96%,.85)}.border-neutral-100\/90{border-color:hsla(0,0%,96%,.9)}.border-neutral-100\/95{border-color:hsla(0,0%,96%,.95)}.border-neutral-200{--tw-border-opacity:1;border-color:rgb(229 229 229/var(--tw-border-opacity,1))}.border-neutral-200\/0{border-color:hsla(0,0%,90%,0)}.border-neutral-200\/10{border-color:hsla(0,0%,90%,.1)}.border-neutral-200\/100{border-color:#e5e5e5}.border-neutral-200\/15{border-color:hsla(0,0%,90%,.15)}.border-neutral-200\/20{border-color:hsla(0,0%,90%,.2)}.border-neutral-200\/25{border-color:hsla(0,0%,90%,.25)}.border-neutral-200\/30{border-color:hsla(0,0%,90%,.3)}.border-neutral-200\/35{border-color:hsla(0,0%,90%,.35)}.border-neutral-200\/40{border-color:hsla(0,0%,90%,.4)}.border-neutral-200\/45{border-color:hsla(0,0%,90%,.45)}.border-neutral-200\/5{border-color:hsla(0,0%,90%,.05)}.border-neutral-200\/50{border-color:hsla(0,0%,90%,.5)}.border-neutral-200\/55{border-color:hsla(0,0%,90%,.55)}.border-neutral-200\/60{border-color:hsla(0,0%,90%,.6)}.border-neutral-200\/65{border-color:hsla(0,0%,90%,.65)}.border-neutral-200\/70{border-color:hsla(0,0%,90%,.7)}.border-neutral-200\/75{border-color:hsla(0,0%,90%,.75)}.border-neutral-200\/80{border-color:hsla(0,0%,90%,.8)}.border-neutral-200\/85{border-color:hsla(0,0%,90%,.85)}.border-neutral-200\/90{border-color:hsla(0,0%,90%,.9)}.border-neutral-200\/95{border-color:hsla(0,0%,90%,.95)}.border-neutral-50{--tw-border-opacity:1;border-color:rgb(250 250 250/var(--tw-border-opacity,1))}.border-neutral-50\/0{border-color:hsla(0,0%,98%,0)}.border-neutral-50\/10{border-color:hsla(0,0%,98%,.1)}.border-neutral-50\/100{border-color:#fafafa}.border-neutral-50\/15{border-color:hsla(0,0%,98%,.15)}.border-neutral-50\/20{border-color:hsla(0,0%,98%,.2)}.border-neutral-50\/25{border-color:hsla(0,0%,98%,.25)}.border-neutral-50\/30{border-color:hsla(0,0%,98%,.3)}.border-neutral-50\/35{border-color:hsla(0,0%,98%,.35)}.border-neutral-50\/40{border-color:hsla(0,0%,98%,.4)}.border-neutral-50\/45{border-color:hsla(0,0%,98%,.45)}.border-neutral-50\/5{border-color:hsla(0,0%,98%,.05)}.border-neutral-50\/50{border-color:hsla(0,0%,98%,.5)}.border-neutral-50\/55{border-color:hsla(0,0%,98%,.55)}.border-neutral-50\/60{border-color:hsla(0,0%,98%,.6)}.border-neutral-50\/65{border-color:hsla(0,0%,98%,.65)}.border-neutral-50\/70{border-color:hsla(0,0%,98%,.7)}.border-neutral-50\/75{border-color:hsla(0,0%,98%,.75)}.border-neutral-50\/80{border-color:hsla(0,0%,98%,.8)}.border-neutral-50\/85{border-color:hsla(0,0%,98%,.85)}.border-neutral-50\/90{border-color:hsla(0,0%,98%,.9)}.border-neutral-50\/95{border-color:hsla(0,0%,98%,.95)}.border-neutral-500{--tw-border-opacity:1;border-color:rgb(115 115 115/var(--tw-border-opacity,1))}.border-neutral-500\/0{border-color:hsla(0,0%,45%,0)}.border-neutral-500\/10{border-color:hsla(0,0%,45%,.1)}.border-neutral-500\/100{border-color:#737373}.border-neutral-500\/15{border-color:hsla(0,0%,45%,.15)}.border-neutral-500\/20{border-color:hsla(0,0%,45%,.2)}.border-neutral-500\/25{border-color:hsla(0,0%,45%,.25)}.border-neutral-500\/30{border-color:hsla(0,0%,45%,.3)}.border-neutral-500\/35{border-color:hsla(0,0%,45%,.35)}.border-neutral-500\/40{border-color:hsla(0,0%,45%,.4)}.border-neutral-500\/45{border-color:hsla(0,0%,45%,.45)}.border-neutral-500\/5{border-color:hsla(0,0%,45%,.05)}.border-neutral-500\/50{border-color:hsla(0,0%,45%,.5)}.border-neutral-500\/55{border-color:hsla(0,0%,45%,.55)}.border-neutral-500\/60{border-color:hsla(0,0%,45%,.6)}.border-neutral-500\/65{border-color:hsla(0,0%,45%,.65)}.border-neutral-500\/70{border-color:hsla(0,0%,45%,.7)}.border-neutral-500\/75{border-color:hsla(0,0%,45%,.75)}.border-neutral-500\/80{border-color:hsla(0,0%,45%,.8)}.border-neutral-500\/85{border-color:hsla(0,0%,45%,.85)}.border-neutral-500\/90{border-color:hsla(0,0%,45%,.9)}.border-neutral-500\/95{border-color:hsla(0,0%,45%,.95)}.border-neutral-950{--tw-border-opacity:1;border-color:rgb(10 10 10/var(--tw-border-opacity,1))}.border-neutral-950\/0{border-color:hsla(0,0%,4%,0)}.border-neutral-950\/10{border-color:hsla(0,0%,4%,.1)}.border-neutral-950\/100{border-color:#0a0a0a}.border-neutral-950\/15{border-color:hsla(0,0%,4%,.15)}.border-neutral-950\/20{border-color:hsla(0,0%,4%,.2)}.border-neutral-950\/25{border-color:hsla(0,0%,4%,.25)}.border-neutral-950\/30{border-color:hsla(0,0%,4%,.3)}.border-neutral-950\/35{border-color:hsla(0,0%,4%,.35)}.border-neutral-950\/40{border-color:hsla(0,0%,4%,.4)}.border-neutral-950\/45{border-color:hsla(0,0%,4%,.45)}.border-neutral-950\/5{border-color:hsla(0,0%,4%,.05)}.border-neutral-950\/50{border-color:hsla(0,0%,4%,.5)}.border-neutral-950\/55{border-color:hsla(0,0%,4%,.55)}.border-neutral-950\/60{border-color:hsla(0,0%,4%,.6)}.border-neutral-950\/65{border-color:hsla(0,0%,4%,.65)}.border-neutral-950\/70{border-color:hsla(0,0%,4%,.7)}.border-neutral-950\/75{border-color:hsla(0,0%,4%,.75)}.border-neutral-950\/80{border-color:hsla(0,0%,4%,.8)}.border-neutral-950\/85{border-color:hsla(0,0%,4%,.85)}.border-neutral-950\/90{border-color:hsla(0,0%,4%,.9)}.border-neutral-950\/95{border-color:hsla(0,0%,4%,.95)}.border-orange-100{--tw-border-opacity:1;border-color:rgb(255 237 213/var(--tw-border-opacity,1))}.border-orange-100\/0{border-color:rgba(255,237,213,0)}.border-orange-100\/10{border-color:rgba(255,237,213,.1)}.border-orange-100\/100{border-color:#ffedd5}.border-orange-100\/15{border-color:rgba(255,237,213,.15)}.border-orange-100\/20{border-color:rgba(255,237,213,.2)}.border-orange-100\/25{border-color:rgba(255,237,213,.25)}.border-orange-100\/30{border-color:rgba(255,237,213,.3)}.border-orange-100\/35{border-color:rgba(255,237,213,.35)}.border-orange-100\/40{border-color:rgba(255,237,213,.4)}.border-orange-100\/45{border-color:rgba(255,237,213,.45)}.border-orange-100\/5{border-color:rgba(255,237,213,.05)}.border-orange-100\/50{border-color:rgba(255,237,213,.5)}.border-orange-100\/55{border-color:rgba(255,237,213,.55)}.border-orange-100\/60{border-color:rgba(255,237,213,.6)}.border-orange-100\/65{border-color:rgba(255,237,213,.65)}.border-orange-100\/70{border-color:rgba(255,237,213,.7)}.border-orange-100\/75{border-color:rgba(255,237,213,.75)}.border-orange-100\/80{border-color:rgba(255,237,213,.8)}.border-orange-100\/85{border-color:rgba(255,237,213,.85)}.border-orange-100\/90{border-color:rgba(255,237,213,.9)}.border-orange-100\/95{border-color:rgba(255,237,213,.95)}.border-orange-200{--tw-border-opacity:1;border-color:rgb(254 215 170/var(--tw-border-opacity,1))}.border-orange-200\/0{border-color:hsla(32,98%,83%,0)}.border-orange-200\/10{border-color:hsla(32,98%,83%,.1)}.border-orange-200\/100{border-color:#fed7aa}.border-orange-200\/15{border-color:hsla(32,98%,83%,.15)}.border-orange-200\/20{border-color:hsla(32,98%,83%,.2)}.border-orange-200\/25{border-color:hsla(32,98%,83%,.25)}.border-orange-200\/30{border-color:hsla(32,98%,83%,.3)}.border-orange-200\/35{border-color:hsla(32,98%,83%,.35)}.border-orange-200\/40{border-color:hsla(32,98%,83%,.4)}.border-orange-200\/45{border-color:hsla(32,98%,83%,.45)}.border-orange-200\/5{border-color:hsla(32,98%,83%,.05)}.border-orange-200\/50{border-color:hsla(32,98%,83%,.5)}.border-orange-200\/55{border-color:hsla(32,98%,83%,.55)}.border-orange-200\/60{border-color:hsla(32,98%,83%,.6)}.border-orange-200\/65{border-color:hsla(32,98%,83%,.65)}.border-orange-200\/70{border-color:hsla(32,98%,83%,.7)}.border-orange-200\/75{border-color:hsla(32,98%,83%,.75)}.border-orange-200\/80{border-color:hsla(32,98%,83%,.8)}.border-orange-200\/85{border-color:hsla(32,98%,83%,.85)}.border-orange-200\/90{border-color:hsla(32,98%,83%,.9)}.border-orange-200\/95{border-color:hsla(32,98%,83%,.95)}.border-orange-50{--tw-border-opacity:1;border-color:rgb(255 247 237/var(--tw-border-opacity,1))}.border-orange-50\/0{border-color:rgba(255,247,237,0)}.border-orange-50\/10{border-color:rgba(255,247,237,.1)}.border-orange-50\/100{border-color:#fff7ed}.border-orange-50\/15{border-color:rgba(255,247,237,.15)}.border-orange-50\/20{border-color:rgba(255,247,237,.2)}.border-orange-50\/25{border-color:rgba(255,247,237,.25)}.border-orange-50\/30{border-color:rgba(255,247,237,.3)}.border-orange-50\/35{border-color:rgba(255,247,237,.35)}.border-orange-50\/40{border-color:rgba(255,247,237,.4)}.border-orange-50\/45{border-color:rgba(255,247,237,.45)}.border-orange-50\/5{border-color:rgba(255,247,237,.05)}.border-orange-50\/50{border-color:rgba(255,247,237,.5)}.border-orange-50\/55{border-color:rgba(255,247,237,.55)}.border-orange-50\/60{border-color:rgba(255,247,237,.6)}.border-orange-50\/65{border-color:rgba(255,247,237,.65)}.border-orange-50\/70{border-color:rgba(255,247,237,.7)}.border-orange-50\/75{border-color:rgba(255,247,237,.75)}.border-orange-50\/80{border-color:rgba(255,247,237,.8)}.border-orange-50\/85{border-color:rgba(255,247,237,.85)}.border-orange-50\/90{border-color:rgba(255,247,237,.9)}.border-orange-50\/95{border-color:rgba(255,247,237,.95)}.border-orange-500{--tw-border-opacity:1;border-color:rgb(249 115 22/var(--tw-border-opacity,1))}.border-orange-500\/0{border-color:rgba(249,115,22,0)}.border-orange-500\/10{border-color:rgba(249,115,22,.1)}.border-orange-500\/100{border-color:#f97316}.border-orange-500\/15{border-color:rgba(249,115,22,.15)}.border-orange-500\/20{border-color:rgba(249,115,22,.2)}.border-orange-500\/25{border-color:rgba(249,115,22,.25)}.border-orange-500\/30{border-color:rgba(249,115,22,.3)}.border-orange-500\/35{border-color:rgba(249,115,22,.35)}.border-orange-500\/40{border-color:rgba(249,115,22,.4)}.border-orange-500\/45{border-color:rgba(249,115,22,.45)}.border-orange-500\/5{border-color:rgba(249,115,22,.05)}.border-orange-500\/50{border-color:rgba(249,115,22,.5)}.border-orange-500\/55{border-color:rgba(249,115,22,.55)}.border-orange-500\/60{border-color:rgba(249,115,22,.6)}.border-orange-500\/65{border-color:rgba(249,115,22,.65)}.border-orange-500\/70{border-color:rgba(249,115,22,.7)}.border-orange-500\/75{border-color:rgba(249,115,22,.75)}.border-orange-500\/80{border-color:rgba(249,115,22,.8)}.border-orange-500\/85{border-color:rgba(249,115,22,.85)}.border-orange-500\/90{border-color:rgba(249,115,22,.9)}.border-orange-500\/95{border-color:rgba(249,115,22,.95)}.border-orange-950{--tw-border-opacity:1;border-color:rgb(67 20 7/var(--tw-border-opacity,1))}.border-orange-950\/0{border-color:rgba(67,20,7,0)}.border-orange-950\/10{border-color:rgba(67,20,7,.1)}.border-orange-950\/100{border-color:#431407}.border-orange-950\/15{border-color:rgba(67,20,7,.15)}.border-orange-950\/20{border-color:rgba(67,20,7,.2)}.border-orange-950\/25{border-color:rgba(67,20,7,.25)}.border-orange-950\/30{border-color:rgba(67,20,7,.3)}.border-orange-950\/35{border-color:rgba(67,20,7,.35)}.border-orange-950\/40{border-color:rgba(67,20,7,.4)}.border-orange-950\/45{border-color:rgba(67,20,7,.45)}.border-orange-950\/5{border-color:rgba(67,20,7,.05)}.border-orange-950\/50{border-color:rgba(67,20,7,.5)}.border-orange-950\/55{border-color:rgba(67,20,7,.55)}.border-orange-950\/60{border-color:rgba(67,20,7,.6)}.border-orange-950\/65{border-color:rgba(67,20,7,.65)}.border-orange-950\/70{border-color:rgba(67,20,7,.7)}.border-orange-950\/75{border-color:rgba(67,20,7,.75)}.border-orange-950\/80{border-color:rgba(67,20,7,.8)}.border-orange-950\/85{border-color:rgba(67,20,7,.85)}.border-orange-950\/90{border-color:rgba(67,20,7,.9)}.border-orange-950\/95{border-color:rgba(67,20,7,.95)}.border-pink-100{--tw-border-opacity:1;border-color:rgb(252 231 243/var(--tw-border-opacity,1))}.border-pink-100\/0{border-color:rgba(252,231,243,0)}.border-pink-100\/10{border-color:rgba(252,231,243,.1)}.border-pink-100\/100{border-color:#fce7f3}.border-pink-100\/15{border-color:rgba(252,231,243,.15)}.border-pink-100\/20{border-color:rgba(252,231,243,.2)}.border-pink-100\/25{border-color:rgba(252,231,243,.25)}.border-pink-100\/30{border-color:rgba(252,231,243,.3)}.border-pink-100\/35{border-color:rgba(252,231,243,.35)}.border-pink-100\/40{border-color:rgba(252,231,243,.4)}.border-pink-100\/45{border-color:rgba(252,231,243,.45)}.border-pink-100\/5{border-color:rgba(252,231,243,.05)}.border-pink-100\/50{border-color:rgba(252,231,243,.5)}.border-pink-100\/55{border-color:rgba(252,231,243,.55)}.border-pink-100\/60{border-color:rgba(252,231,243,.6)}.border-pink-100\/65{border-color:rgba(252,231,243,.65)}.border-pink-100\/70{border-color:rgba(252,231,243,.7)}.border-pink-100\/75{border-color:rgba(252,231,243,.75)}.border-pink-100\/80{border-color:rgba(252,231,243,.8)}.border-pink-100\/85{border-color:rgba(252,231,243,.85)}.border-pink-100\/90{border-color:rgba(252,231,243,.9)}.border-pink-100\/95{border-color:rgba(252,231,243,.95)}.border-pink-200{--tw-border-opacity:1;border-color:rgb(251 207 232/var(--tw-border-opacity,1))}.border-pink-200\/0{border-color:rgba(251,207,232,0)}.border-pink-200\/10{border-color:rgba(251,207,232,.1)}.border-pink-200\/100{border-color:#fbcfe8}.border-pink-200\/15{border-color:rgba(251,207,232,.15)}.border-pink-200\/20{border-color:rgba(251,207,232,.2)}.border-pink-200\/25{border-color:rgba(251,207,232,.25)}.border-pink-200\/30{border-color:rgba(251,207,232,.3)}.border-pink-200\/35{border-color:rgba(251,207,232,.35)}.border-pink-200\/40{border-color:rgba(251,207,232,.4)}.border-pink-200\/45{border-color:rgba(251,207,232,.45)}.border-pink-200\/5{border-color:rgba(251,207,232,.05)}.border-pink-200\/50{border-color:rgba(251,207,232,.5)}.border-pink-200\/55{border-color:rgba(251,207,232,.55)}.border-pink-200\/60{border-color:rgba(251,207,232,.6)}.border-pink-200\/65{border-color:rgba(251,207,232,.65)}.border-pink-200\/70{border-color:rgba(251,207,232,.7)}.border-pink-200\/75{border-color:rgba(251,207,232,.75)}.border-pink-200\/80{border-color:rgba(251,207,232,.8)}.border-pink-200\/85{border-color:rgba(251,207,232,.85)}.border-pink-200\/90{border-color:rgba(251,207,232,.9)}.border-pink-200\/95{border-color:rgba(251,207,232,.95)}.border-pink-50{--tw-border-opacity:1;border-color:rgb(253 242 248/var(--tw-border-opacity,1))}.border-pink-50\/0{border-color:rgba(253,242,248,0)}.border-pink-50\/10{border-color:rgba(253,242,248,.1)}.border-pink-50\/100{border-color:#fdf2f8}.border-pink-50\/15{border-color:rgba(253,242,248,.15)}.border-pink-50\/20{border-color:rgba(253,242,248,.2)}.border-pink-50\/25{border-color:rgba(253,242,248,.25)}.border-pink-50\/30{border-color:rgba(253,242,248,.3)}.border-pink-50\/35{border-color:rgba(253,242,248,.35)}.border-pink-50\/40{border-color:rgba(253,242,248,.4)}.border-pink-50\/45{border-color:rgba(253,242,248,.45)}.border-pink-50\/5{border-color:rgba(253,242,248,.05)}.border-pink-50\/50{border-color:rgba(253,242,248,.5)}.border-pink-50\/55{border-color:rgba(253,242,248,.55)}.border-pink-50\/60{border-color:rgba(253,242,248,.6)}.border-pink-50\/65{border-color:rgba(253,242,248,.65)}.border-pink-50\/70{border-color:rgba(253,242,248,.7)}.border-pink-50\/75{border-color:rgba(253,242,248,.75)}.border-pink-50\/80{border-color:rgba(253,242,248,.8)}.border-pink-50\/85{border-color:rgba(253,242,248,.85)}.border-pink-50\/90{border-color:rgba(253,242,248,.9)}.border-pink-50\/95{border-color:rgba(253,242,248,.95)}.border-pink-500{--tw-border-opacity:1;border-color:rgb(236 72 153/var(--tw-border-opacity,1))}.border-pink-500\/0{border-color:rgba(236,72,153,0)}.border-pink-500\/10{border-color:rgba(236,72,153,.1)}.border-pink-500\/100{border-color:#ec4899}.border-pink-500\/15{border-color:rgba(236,72,153,.15)}.border-pink-500\/20{border-color:rgba(236,72,153,.2)}.border-pink-500\/25{border-color:rgba(236,72,153,.25)}.border-pink-500\/30{border-color:rgba(236,72,153,.3)}.border-pink-500\/35{border-color:rgba(236,72,153,.35)}.border-pink-500\/40{border-color:rgba(236,72,153,.4)}.border-pink-500\/45{border-color:rgba(236,72,153,.45)}.border-pink-500\/5{border-color:rgba(236,72,153,.05)}.border-pink-500\/50{border-color:rgba(236,72,153,.5)}.border-pink-500\/55{border-color:rgba(236,72,153,.55)}.border-pink-500\/60{border-color:rgba(236,72,153,.6)}.border-pink-500\/65{border-color:rgba(236,72,153,.65)}.border-pink-500\/70{border-color:rgba(236,72,153,.7)}.border-pink-500\/75{border-color:rgba(236,72,153,.75)}.border-pink-500\/80{border-color:rgba(236,72,153,.8)}.border-pink-500\/85{border-color:rgba(236,72,153,.85)}.border-pink-500\/90{border-color:rgba(236,72,153,.9)}.border-pink-500\/95{border-color:rgba(236,72,153,.95)}.border-pink-950{--tw-border-opacity:1;border-color:rgb(80 7 36/var(--tw-border-opacity,1))}.border-pink-950\/0{border-color:rgba(80,7,36,0)}.border-pink-950\/10{border-color:rgba(80,7,36,.1)}.border-pink-950\/100{border-color:#500724}.border-pink-950\/15{border-color:rgba(80,7,36,.15)}.border-pink-950\/20{border-color:rgba(80,7,36,.2)}.border-pink-950\/25{border-color:rgba(80,7,36,.25)}.border-pink-950\/30{border-color:rgba(80,7,36,.3)}.border-pink-950\/35{border-color:rgba(80,7,36,.35)}.border-pink-950\/40{border-color:rgba(80,7,36,.4)}.border-pink-950\/45{border-color:rgba(80,7,36,.45)}.border-pink-950\/5{border-color:rgba(80,7,36,.05)}.border-pink-950\/50{border-color:rgba(80,7,36,.5)}.border-pink-950\/55{border-color:rgba(80,7,36,.55)}.border-pink-950\/60{border-color:rgba(80,7,36,.6)}.border-pink-950\/65{border-color:rgba(80,7,36,.65)}.border-pink-950\/70{border-color:rgba(80,7,36,.7)}.border-pink-950\/75{border-color:rgba(80,7,36,.75)}.border-pink-950\/80{border-color:rgba(80,7,36,.8)}.border-pink-950\/85{border-color:rgba(80,7,36,.85)}.border-pink-950\/90{border-color:rgba(80,7,36,.9)}.border-pink-950\/95{border-color:rgba(80,7,36,.95)}.border-purple-100{--tw-border-opacity:1;border-color:rgb(243 232 255/var(--tw-border-opacity,1))}.border-purple-100\/0{border-color:rgba(243,232,255,0)}.border-purple-100\/10{border-color:rgba(243,232,255,.1)}.border-purple-100\/100{border-color:#f3e8ff}.border-purple-100\/15{border-color:rgba(243,232,255,.15)}.border-purple-100\/20{border-color:rgba(243,232,255,.2)}.border-purple-100\/25{border-color:rgba(243,232,255,.25)}.border-purple-100\/30{border-color:rgba(243,232,255,.3)}.border-purple-100\/35{border-color:rgba(243,232,255,.35)}.border-purple-100\/40{border-color:rgba(243,232,255,.4)}.border-purple-100\/45{border-color:rgba(243,232,255,.45)}.border-purple-100\/5{border-color:rgba(243,232,255,.05)}.border-purple-100\/50{border-color:rgba(243,232,255,.5)}.border-purple-100\/55{border-color:rgba(243,232,255,.55)}.border-purple-100\/60{border-color:rgba(243,232,255,.6)}.border-purple-100\/65{border-color:rgba(243,232,255,.65)}.border-purple-100\/70{border-color:rgba(243,232,255,.7)}.border-purple-100\/75{border-color:rgba(243,232,255,.75)}.border-purple-100\/80{border-color:rgba(243,232,255,.8)}.border-purple-100\/85{border-color:rgba(243,232,255,.85)}.border-purple-100\/90{border-color:rgba(243,232,255,.9)}.border-purple-100\/95{border-color:rgba(243,232,255,.95)}.border-purple-200{--tw-border-opacity:1;border-color:rgb(233 213 255/var(--tw-border-opacity,1))}.border-purple-200\/0{border-color:rgba(233,213,255,0)}.border-purple-200\/10{border-color:rgba(233,213,255,.1)}.border-purple-200\/100{border-color:#e9d5ff}.border-purple-200\/15{border-color:rgba(233,213,255,.15)}.border-purple-200\/20{border-color:rgba(233,213,255,.2)}.border-purple-200\/25{border-color:rgba(233,213,255,.25)}.border-purple-200\/30{border-color:rgba(233,213,255,.3)}.border-purple-200\/35{border-color:rgba(233,213,255,.35)}.border-purple-200\/40{border-color:rgba(233,213,255,.4)}.border-purple-200\/45{border-color:rgba(233,213,255,.45)}.border-purple-200\/5{border-color:rgba(233,213,255,.05)}.border-purple-200\/50{border-color:rgba(233,213,255,.5)}.border-purple-200\/55{border-color:rgba(233,213,255,.55)}.border-purple-200\/60{border-color:rgba(233,213,255,.6)}.border-purple-200\/65{border-color:rgba(233,213,255,.65)}.border-purple-200\/70{border-color:rgba(233,213,255,.7)}.border-purple-200\/75{border-color:rgba(233,213,255,.75)}.border-purple-200\/80{border-color:rgba(233,213,255,.8)}.border-purple-200\/85{border-color:rgba(233,213,255,.85)}.border-purple-200\/90{border-color:rgba(233,213,255,.9)}.border-purple-200\/95{border-color:rgba(233,213,255,.95)}.border-purple-50{--tw-border-opacity:1;border-color:rgb(250 245 255/var(--tw-border-opacity,1))}.border-purple-50\/0{border-color:rgba(250,245,255,0)}.border-purple-50\/10{border-color:rgba(250,245,255,.1)}.border-purple-50\/100{border-color:#faf5ff}.border-purple-50\/15{border-color:rgba(250,245,255,.15)}.border-purple-50\/20{border-color:rgba(250,245,255,.2)}.border-purple-50\/25{border-color:rgba(250,245,255,.25)}.border-purple-50\/30{border-color:rgba(250,245,255,.3)}.border-purple-50\/35{border-color:rgba(250,245,255,.35)}.border-purple-50\/40{border-color:rgba(250,245,255,.4)}.border-purple-50\/45{border-color:rgba(250,245,255,.45)}.border-purple-50\/5{border-color:rgba(250,245,255,.05)}.border-purple-50\/50{border-color:rgba(250,245,255,.5)}.border-purple-50\/55{border-color:rgba(250,245,255,.55)}.border-purple-50\/60{border-color:rgba(250,245,255,.6)}.border-purple-50\/65{border-color:rgba(250,245,255,.65)}.border-purple-50\/70{border-color:rgba(250,245,255,.7)}.border-purple-50\/75{border-color:rgba(250,245,255,.75)}.border-purple-50\/80{border-color:rgba(250,245,255,.8)}.border-purple-50\/85{border-color:rgba(250,245,255,.85)}.border-purple-50\/90{border-color:rgba(250,245,255,.9)}.border-purple-50\/95{border-color:rgba(250,245,255,.95)}.border-purple-500{--tw-border-opacity:1;border-color:rgb(168 85 247/var(--tw-border-opacity,1))}.border-purple-500\/0{border-color:rgba(168,85,247,0)}.border-purple-500\/10{border-color:rgba(168,85,247,.1)}.border-purple-500\/100{border-color:#a855f7}.border-purple-500\/15{border-color:rgba(168,85,247,.15)}.border-purple-500\/20{border-color:rgba(168,85,247,.2)}.border-purple-500\/25{border-color:rgba(168,85,247,.25)}.border-purple-500\/30{border-color:rgba(168,85,247,.3)}.border-purple-500\/35{border-color:rgba(168,85,247,.35)}.border-purple-500\/40{border-color:rgba(168,85,247,.4)}.border-purple-500\/45{border-color:rgba(168,85,247,.45)}.border-purple-500\/5{border-color:rgba(168,85,247,.05)}.border-purple-500\/50{border-color:rgba(168,85,247,.5)}.border-purple-500\/55{border-color:rgba(168,85,247,.55)}.border-purple-500\/60{border-color:rgba(168,85,247,.6)}.border-purple-500\/65{border-color:rgba(168,85,247,.65)}.border-purple-500\/70{border-color:rgba(168,85,247,.7)}.border-purple-500\/75{border-color:rgba(168,85,247,.75)}.border-purple-500\/80{border-color:rgba(168,85,247,.8)}.border-purple-500\/85{border-color:rgba(168,85,247,.85)}.border-purple-500\/90{border-color:rgba(168,85,247,.9)}.border-purple-500\/95{border-color:rgba(168,85,247,.95)}.border-purple-950{--tw-border-opacity:1;border-color:rgb(59 7 100/var(--tw-border-opacity,1))}.border-purple-950\/0{border-color:rgba(59,7,100,0)}.border-purple-950\/10{border-color:rgba(59,7,100,.1)}.border-purple-950\/100{border-color:#3b0764}.border-purple-950\/15{border-color:rgba(59,7,100,.15)}.border-purple-950\/20{border-color:rgba(59,7,100,.2)}.border-purple-950\/25{border-color:rgba(59,7,100,.25)}.border-purple-950\/30{border-color:rgba(59,7,100,.3)}.border-purple-950\/35{border-color:rgba(59,7,100,.35)}.border-purple-950\/40{border-color:rgba(59,7,100,.4)}.border-purple-950\/45{border-color:rgba(59,7,100,.45)}.border-purple-950\/5{border-color:rgba(59,7,100,.05)}.border-purple-950\/50{border-color:rgba(59,7,100,.5)}.border-purple-950\/55{border-color:rgba(59,7,100,.55)}.border-purple-950\/60{border-color:rgba(59,7,100,.6)}.border-purple-950\/65{border-color:rgba(59,7,100,.65)}.border-purple-950\/70{border-color:rgba(59,7,100,.7)}.border-purple-950\/75{border-color:rgba(59,7,100,.75)}.border-purple-950\/80{border-color:rgba(59,7,100,.8)}.border-purple-950\/85{border-color:rgba(59,7,100,.85)}.border-purple-950\/90{border-color:rgba(59,7,100,.9)}.border-purple-950\/95{border-color:rgba(59,7,100,.95)}.border-red-100{--tw-border-opacity:1;border-color:rgb(254 226 226/var(--tw-border-opacity,1))}.border-red-100\/0{border-color:hsla(0,93%,94%,0)}.border-red-100\/10{border-color:hsla(0,93%,94%,.1)}.border-red-100\/100{border-color:#fee2e2}.border-red-100\/15{border-color:hsla(0,93%,94%,.15)}.border-red-100\/20{border-color:hsla(0,93%,94%,.2)}.border-red-100\/25{border-color:hsla(0,93%,94%,.25)}.border-red-100\/30{border-color:hsla(0,93%,94%,.3)}.border-red-100\/35{border-color:hsla(0,93%,94%,.35)}.border-red-100\/40{border-color:hsla(0,93%,94%,.4)}.border-red-100\/45{border-color:hsla(0,93%,94%,.45)}.border-red-100\/5{border-color:hsla(0,93%,94%,.05)}.border-red-100\/50{border-color:hsla(0,93%,94%,.5)}.border-red-100\/55{border-color:hsla(0,93%,94%,.55)}.border-red-100\/60{border-color:hsla(0,93%,94%,.6)}.border-red-100\/65{border-color:hsla(0,93%,94%,.65)}.border-red-100\/70{border-color:hsla(0,93%,94%,.7)}.border-red-100\/75{border-color:hsla(0,93%,94%,.75)}.border-red-100\/80{border-color:hsla(0,93%,94%,.8)}.border-red-100\/85{border-color:hsla(0,93%,94%,.85)}.border-red-100\/90{border-color:hsla(0,93%,94%,.9)}.border-red-100\/95{border-color:hsla(0,93%,94%,.95)}.border-red-200{--tw-border-opacity:1;border-color:rgb(254 202 202/var(--tw-border-opacity,1))}.border-red-200\/0{border-color:hsla(0,96%,89%,0)}.border-red-200\/10{border-color:hsla(0,96%,89%,.1)}.border-red-200\/100{border-color:#fecaca}.border-red-200\/15{border-color:hsla(0,96%,89%,.15)}.border-red-200\/20{border-color:hsla(0,96%,89%,.2)}.border-red-200\/25{border-color:hsla(0,96%,89%,.25)}.border-red-200\/30{border-color:hsla(0,96%,89%,.3)}.border-red-200\/35{border-color:hsla(0,96%,89%,.35)}.border-red-200\/40{border-color:hsla(0,96%,89%,.4)}.border-red-200\/45{border-color:hsla(0,96%,89%,.45)}.border-red-200\/5{border-color:hsla(0,96%,89%,.05)}.border-red-200\/50{border-color:hsla(0,96%,89%,.5)}.border-red-200\/55{border-color:hsla(0,96%,89%,.55)}.border-red-200\/60{border-color:hsla(0,96%,89%,.6)}.border-red-200\/65{border-color:hsla(0,96%,89%,.65)}.border-red-200\/70{border-color:hsla(0,96%,89%,.7)}.border-red-200\/75{border-color:hsla(0,96%,89%,.75)}.border-red-200\/80{border-color:hsla(0,96%,89%,.8)}.border-red-200\/85{border-color:hsla(0,96%,89%,.85)}.border-red-200\/90{border-color:hsla(0,96%,89%,.9)}.border-red-200\/95{border-color:hsla(0,96%,89%,.95)}.border-red-50{--tw-border-opacity:1;border-color:rgb(254 242 242/var(--tw-border-opacity,1))}.border-red-50\/0{border-color:hsla(0,86%,97%,0)}.border-red-50\/10{border-color:hsla(0,86%,97%,.1)}.border-red-50\/100{border-color:#fef2f2}.border-red-50\/15{border-color:hsla(0,86%,97%,.15)}.border-red-50\/20{border-color:hsla(0,86%,97%,.2)}.border-red-50\/25{border-color:hsla(0,86%,97%,.25)}.border-red-50\/30{border-color:hsla(0,86%,97%,.3)}.border-red-50\/35{border-color:hsla(0,86%,97%,.35)}.border-red-50\/40{border-color:hsla(0,86%,97%,.4)}.border-red-50\/45{border-color:hsla(0,86%,97%,.45)}.border-red-50\/5{border-color:hsla(0,86%,97%,.05)}.border-red-50\/50{border-color:hsla(0,86%,97%,.5)}.border-red-50\/55{border-color:hsla(0,86%,97%,.55)}.border-red-50\/60{border-color:hsla(0,86%,97%,.6)}.border-red-50\/65{border-color:hsla(0,86%,97%,.65)}.border-red-50\/70{border-color:hsla(0,86%,97%,.7)}.border-red-50\/75{border-color:hsla(0,86%,97%,.75)}.border-red-50\/80{border-color:hsla(0,86%,97%,.8)}.border-red-50\/85{border-color:hsla(0,86%,97%,.85)}.border-red-50\/90{border-color:hsla(0,86%,97%,.9)}.border-red-50\/95{border-color:hsla(0,86%,97%,.95)}.border-red-500{--tw-border-opacity:1;border-color:rgb(239 68 68/var(--tw-border-opacity,1))}.border-red-500\/0{border-color:rgba(239,68,68,0)}.border-red-500\/10{border-color:rgba(239,68,68,.1)}.border-red-500\/100{border-color:#ef4444}.border-red-500\/15{border-color:rgba(239,68,68,.15)}.border-red-500\/20{border-color:rgba(239,68,68,.2)}.border-red-500\/25{border-color:rgba(239,68,68,.25)}.border-red-500\/30{border-color:rgba(239,68,68,.3)}.border-red-500\/35{border-color:rgba(239,68,68,.35)}.border-red-500\/40{border-color:rgba(239,68,68,.4)}.border-red-500\/45{border-color:rgba(239,68,68,.45)}.border-red-500\/5{border-color:rgba(239,68,68,.05)}.border-red-500\/50{border-color:rgba(239,68,68,.5)}.border-red-500\/55{border-color:rgba(239,68,68,.55)}.border-red-500\/60{border-color:rgba(239,68,68,.6)}.border-red-500\/65{border-color:rgba(239,68,68,.65)}.border-red-500\/70{border-color:rgba(239,68,68,.7)}.border-red-500\/75{border-color:rgba(239,68,68,.75)}.border-red-500\/80{border-color:rgba(239,68,68,.8)}.border-red-500\/85{border-color:rgba(239,68,68,.85)}.border-red-500\/90{border-color:rgba(239,68,68,.9)}.border-red-500\/95{border-color:rgba(239,68,68,.95)}.border-red-950{--tw-border-opacity:1;border-color:rgb(69 10 10/var(--tw-border-opacity,1))}.border-red-950\/0{border-color:rgba(69,10,10,0)}.border-red-950\/10{border-color:rgba(69,10,10,.1)}.border-red-950\/100{border-color:#450a0a}.border-red-950\/15{border-color:rgba(69,10,10,.15)}.border-red-950\/20{border-color:rgba(69,10,10,.2)}.border-red-950\/25{border-color:rgba(69,10,10,.25)}.border-red-950\/30{border-color:rgba(69,10,10,.3)}.border-red-950\/35{border-color:rgba(69,10,10,.35)}.border-red-950\/40{border-color:rgba(69,10,10,.4)}.border-red-950\/45{border-color:rgba(69,10,10,.45)}.border-red-950\/5{border-color:rgba(69,10,10,.05)}.border-red-950\/50{border-color:rgba(69,10,10,.5)}.border-red-950\/55{border-color:rgba(69,10,10,.55)}.border-red-950\/60{border-color:rgba(69,10,10,.6)}.border-red-950\/65{border-color:rgba(69,10,10,.65)}.border-red-950\/70{border-color:rgba(69,10,10,.7)}.border-red-950\/75{border-color:rgba(69,10,10,.75)}.border-red-950\/80{border-color:rgba(69,10,10,.8)}.border-red-950\/85{border-color:rgba(69,10,10,.85)}.border-red-950\/90{border-color:rgba(69,10,10,.9)}.border-red-950\/95{border-color:rgba(69,10,10,.95)}.border-rose-100{--tw-border-opacity:1;border-color:rgb(255 228 230/var(--tw-border-opacity,1))}.border-rose-100\/0{border-color:rgba(255,228,230,0)}.border-rose-100\/10{border-color:rgba(255,228,230,.1)}.border-rose-100\/100{border-color:#ffe4e6}.border-rose-100\/15{border-color:rgba(255,228,230,.15)}.border-rose-100\/20{border-color:rgba(255,228,230,.2)}.border-rose-100\/25{border-color:rgba(255,228,230,.25)}.border-rose-100\/30{border-color:rgba(255,228,230,.3)}.border-rose-100\/35{border-color:rgba(255,228,230,.35)}.border-rose-100\/40{border-color:rgba(255,228,230,.4)}.border-rose-100\/45{border-color:rgba(255,228,230,.45)}.border-rose-100\/5{border-color:rgba(255,228,230,.05)}.border-rose-100\/50{border-color:rgba(255,228,230,.5)}.border-rose-100\/55{border-color:rgba(255,228,230,.55)}.border-rose-100\/60{border-color:rgba(255,228,230,.6)}.border-rose-100\/65{border-color:rgba(255,228,230,.65)}.border-rose-100\/70{border-color:rgba(255,228,230,.7)}.border-rose-100\/75{border-color:rgba(255,228,230,.75)}.border-rose-100\/80{border-color:rgba(255,228,230,.8)}.border-rose-100\/85{border-color:rgba(255,228,230,.85)}.border-rose-100\/90{border-color:rgba(255,228,230,.9)}.border-rose-100\/95{border-color:rgba(255,228,230,.95)}.border-rose-200{--tw-border-opacity:1;border-color:rgb(254 205 211/var(--tw-border-opacity,1))}.border-rose-200\/0{border-color:rgba(254,205,211,0)}.border-rose-200\/10{border-color:rgba(254,205,211,.1)}.border-rose-200\/100{border-color:#fecdd3}.border-rose-200\/15{border-color:rgba(254,205,211,.15)}.border-rose-200\/20{border-color:rgba(254,205,211,.2)}.border-rose-200\/25{border-color:rgba(254,205,211,.25)}.border-rose-200\/30{border-color:rgba(254,205,211,.3)}.border-rose-200\/35{border-color:rgba(254,205,211,.35)}.border-rose-200\/40{border-color:rgba(254,205,211,.4)}.border-rose-200\/45{border-color:rgba(254,205,211,.45)}.border-rose-200\/5{border-color:rgba(254,205,211,.05)}.border-rose-200\/50{border-color:rgba(254,205,211,.5)}.border-rose-200\/55{border-color:rgba(254,205,211,.55)}.border-rose-200\/60{border-color:rgba(254,205,211,.6)}.border-rose-200\/65{border-color:rgba(254,205,211,.65)}.border-rose-200\/70{border-color:rgba(254,205,211,.7)}.border-rose-200\/75{border-color:rgba(254,205,211,.75)}.border-rose-200\/80{border-color:rgba(254,205,211,.8)}.border-rose-200\/85{border-color:rgba(254,205,211,.85)}.border-rose-200\/90{border-color:rgba(254,205,211,.9)}.border-rose-200\/95{border-color:rgba(254,205,211,.95)}.border-rose-50{--tw-border-opacity:1;border-color:rgb(255 241 242/var(--tw-border-opacity,1))}.border-rose-50\/0{border-color:rgba(255,241,242,0)}.border-rose-50\/10{border-color:rgba(255,241,242,.1)}.border-rose-50\/100{border-color:#fff1f2}.border-rose-50\/15{border-color:rgba(255,241,242,.15)}.border-rose-50\/20{border-color:rgba(255,241,242,.2)}.border-rose-50\/25{border-color:rgba(255,241,242,.25)}.border-rose-50\/30{border-color:rgba(255,241,242,.3)}.border-rose-50\/35{border-color:rgba(255,241,242,.35)}.border-rose-50\/40{border-color:rgba(255,241,242,.4)}.border-rose-50\/45{border-color:rgba(255,241,242,.45)}.border-rose-50\/5{border-color:rgba(255,241,242,.05)}.border-rose-50\/50{border-color:rgba(255,241,242,.5)}.border-rose-50\/55{border-color:rgba(255,241,242,.55)}.border-rose-50\/60{border-color:rgba(255,241,242,.6)}.border-rose-50\/65{border-color:rgba(255,241,242,.65)}.border-rose-50\/70{border-color:rgba(255,241,242,.7)}.border-rose-50\/75{border-color:rgba(255,241,242,.75)}.border-rose-50\/80{border-color:rgba(255,241,242,.8)}.border-rose-50\/85{border-color:rgba(255,241,242,.85)}.border-rose-50\/90{border-color:rgba(255,241,242,.9)}.border-rose-50\/95{border-color:rgba(255,241,242,.95)}.border-rose-500{--tw-border-opacity:1;border-color:rgb(244 63 94/var(--tw-border-opacity,1))}.border-rose-500\/0{border-color:rgba(244,63,94,0)}.border-rose-500\/10{border-color:rgba(244,63,94,.1)}.border-rose-500\/100{border-color:#f43f5e}.border-rose-500\/15{border-color:rgba(244,63,94,.15)}.border-rose-500\/20{border-color:rgba(244,63,94,.2)}.border-rose-500\/25{border-color:rgba(244,63,94,.25)}.border-rose-500\/30{border-color:rgba(244,63,94,.3)}.border-rose-500\/35{border-color:rgba(244,63,94,.35)}.border-rose-500\/40{border-color:rgba(244,63,94,.4)}.border-rose-500\/45{border-color:rgba(244,63,94,.45)}.border-rose-500\/5{border-color:rgba(244,63,94,.05)}.border-rose-500\/50{border-color:rgba(244,63,94,.5)}.border-rose-500\/55{border-color:rgba(244,63,94,.55)}.border-rose-500\/60{border-color:rgba(244,63,94,.6)}.border-rose-500\/65{border-color:rgba(244,63,94,.65)}.border-rose-500\/70{border-color:rgba(244,63,94,.7)}.border-rose-500\/75{border-color:rgba(244,63,94,.75)}.border-rose-500\/80{border-color:rgba(244,63,94,.8)}.border-rose-500\/85{border-color:rgba(244,63,94,.85)}.border-rose-500\/90{border-color:rgba(244,63,94,.9)}.border-rose-500\/95{border-color:rgba(244,63,94,.95)}.border-rose-950{--tw-border-opacity:1;border-color:rgb(76 5 25/var(--tw-border-opacity,1))}.border-rose-950\/0{border-color:rgba(76,5,25,0)}.border-rose-950\/10{border-color:rgba(76,5,25,.1)}.border-rose-950\/100{border-color:#4c0519}.border-rose-950\/15{border-color:rgba(76,5,25,.15)}.border-rose-950\/20{border-color:rgba(76,5,25,.2)}.border-rose-950\/25{border-color:rgba(76,5,25,.25)}.border-rose-950\/30{border-color:rgba(76,5,25,.3)}.border-rose-950\/35{border-color:rgba(76,5,25,.35)}.border-rose-950\/40{border-color:rgba(76,5,25,.4)}.border-rose-950\/45{border-color:rgba(76,5,25,.45)}.border-rose-950\/5{border-color:rgba(76,5,25,.05)}.border-rose-950\/50{border-color:rgba(76,5,25,.5)}.border-rose-950\/55{border-color:rgba(76,5,25,.55)}.border-rose-950\/60{border-color:rgba(76,5,25,.6)}.border-rose-950\/65{border-color:rgba(76,5,25,.65)}.border-rose-950\/70{border-color:rgba(76,5,25,.7)}.border-rose-950\/75{border-color:rgba(76,5,25,.75)}.border-rose-950\/80{border-color:rgba(76,5,25,.8)}.border-rose-950\/85{border-color:rgba(76,5,25,.85)}.border-rose-950\/90{border-color:rgba(76,5,25,.9)}.border-rose-950\/95{border-color:rgba(76,5,25,.95)}.border-secondary{border-color:hsl(var(--secondary))}.border-sky-100{--tw-border-opacity:1;border-color:rgb(224 242 254/var(--tw-border-opacity,1))}.border-sky-100\/0{border-color:rgba(224,242,254,0)}.border-sky-100\/10{border-color:rgba(224,242,254,.1)}.border-sky-100\/100{border-color:#e0f2fe}.border-sky-100\/15{border-color:rgba(224,242,254,.15)}.border-sky-100\/20{border-color:rgba(224,242,254,.2)}.border-sky-100\/25{border-color:rgba(224,242,254,.25)}.border-sky-100\/30{border-color:rgba(224,242,254,.3)}.border-sky-100\/35{border-color:rgba(224,242,254,.35)}.border-sky-100\/40{border-color:rgba(224,242,254,.4)}.border-sky-100\/45{border-color:rgba(224,242,254,.45)}.border-sky-100\/5{border-color:rgba(224,242,254,.05)}.border-sky-100\/50{border-color:rgba(224,242,254,.5)}.border-sky-100\/55{border-color:rgba(224,242,254,.55)}.border-sky-100\/60{border-color:rgba(224,242,254,.6)}.border-sky-100\/65{border-color:rgba(224,242,254,.65)}.border-sky-100\/70{border-color:rgba(224,242,254,.7)}.border-sky-100\/75{border-color:rgba(224,242,254,.75)}.border-sky-100\/80{border-color:rgba(224,242,254,.8)}.border-sky-100\/85{border-color:rgba(224,242,254,.85)}.border-sky-100\/90{border-color:rgba(224,242,254,.9)}.border-sky-100\/95{border-color:rgba(224,242,254,.95)}.border-sky-200{--tw-border-opacity:1;border-color:rgb(186 230 253/var(--tw-border-opacity,1))}.border-sky-200\/0{border-color:rgba(186,230,253,0)}.border-sky-200\/10{border-color:rgba(186,230,253,.1)}.border-sky-200\/100{border-color:#bae6fd}.border-sky-200\/15{border-color:rgba(186,230,253,.15)}.border-sky-200\/20{border-color:rgba(186,230,253,.2)}.border-sky-200\/25{border-color:rgba(186,230,253,.25)}.border-sky-200\/30{border-color:rgba(186,230,253,.3)}.border-sky-200\/35{border-color:rgba(186,230,253,.35)}.border-sky-200\/40{border-color:rgba(186,230,253,.4)}.border-sky-200\/45{border-color:rgba(186,230,253,.45)}.border-sky-200\/5{border-color:rgba(186,230,253,.05)}.border-sky-200\/50{border-color:rgba(186,230,253,.5)}.border-sky-200\/55{border-color:rgba(186,230,253,.55)}.border-sky-200\/60{border-color:rgba(186,230,253,.6)}.border-sky-200\/65{border-color:rgba(186,230,253,.65)}.border-sky-200\/70{border-color:rgba(186,230,253,.7)}.border-sky-200\/75{border-color:rgba(186,230,253,.75)}.border-sky-200\/80{border-color:rgba(186,230,253,.8)}.border-sky-200\/85{border-color:rgba(186,230,253,.85)}.border-sky-200\/90{border-color:rgba(186,230,253,.9)}.border-sky-200\/95{border-color:rgba(186,230,253,.95)}.border-sky-50{--tw-border-opacity:1;border-color:rgb(240 249 255/var(--tw-border-opacity,1))}.border-sky-50\/0{border-color:rgba(240,249,255,0)}.border-sky-50\/10{border-color:rgba(240,249,255,.1)}.border-sky-50\/100{border-color:#f0f9ff}.border-sky-50\/15{border-color:rgba(240,249,255,.15)}.border-sky-50\/20{border-color:rgba(240,249,255,.2)}.border-sky-50\/25{border-color:rgba(240,249,255,.25)}.border-sky-50\/30{border-color:rgba(240,249,255,.3)}.border-sky-50\/35{border-color:rgba(240,249,255,.35)}.border-sky-50\/40{border-color:rgba(240,249,255,.4)}.border-sky-50\/45{border-color:rgba(240,249,255,.45)}.border-sky-50\/5{border-color:rgba(240,249,255,.05)}.border-sky-50\/50{border-color:rgba(240,249,255,.5)}.border-sky-50\/55{border-color:rgba(240,249,255,.55)}.border-sky-50\/60{border-color:rgba(240,249,255,.6)}.border-sky-50\/65{border-color:rgba(240,249,255,.65)}.border-sky-50\/70{border-color:rgba(240,249,255,.7)}.border-sky-50\/75{border-color:rgba(240,249,255,.75)}.border-sky-50\/80{border-color:rgba(240,249,255,.8)}.border-sky-50\/85{border-color:rgba(240,249,255,.85)}.border-sky-50\/90{border-color:rgba(240,249,255,.9)}.border-sky-50\/95{border-color:rgba(240,249,255,.95)}.border-sky-500{--tw-border-opacity:1;border-color:rgb(14 165 233/var(--tw-border-opacity,1))}.border-sky-500\/0{border-color:rgba(14,165,233,0)}.border-sky-500\/10{border-color:rgba(14,165,233,.1)}.border-sky-500\/100{border-color:#0ea5e9}.border-sky-500\/15{border-color:rgba(14,165,233,.15)}.border-sky-500\/20{border-color:rgba(14,165,233,.2)}.border-sky-500\/25{border-color:rgba(14,165,233,.25)}.border-sky-500\/30{border-color:rgba(14,165,233,.3)}.border-sky-500\/35{border-color:rgba(14,165,233,.35)}.border-sky-500\/40{border-color:rgba(14,165,233,.4)}.border-sky-500\/45{border-color:rgba(14,165,233,.45)}.border-sky-500\/5{border-color:rgba(14,165,233,.05)}.border-sky-500\/50{border-color:rgba(14,165,233,.5)}.border-sky-500\/55{border-color:rgba(14,165,233,.55)}.border-sky-500\/60{border-color:rgba(14,165,233,.6)}.border-sky-500\/65{border-color:rgba(14,165,233,.65)}.border-sky-500\/70{border-color:rgba(14,165,233,.7)}.border-sky-500\/75{border-color:rgba(14,165,233,.75)}.border-sky-500\/80{border-color:rgba(14,165,233,.8)}.border-sky-500\/85{border-color:rgba(14,165,233,.85)}.border-sky-500\/90{border-color:rgba(14,165,233,.9)}.border-sky-500\/95{border-color:rgba(14,165,233,.95)}.border-sky-950{--tw-border-opacity:1;border-color:rgb(8 47 73/var(--tw-border-opacity,1))}.border-sky-950\/0{border-color:rgba(8,47,73,0)}.border-sky-950\/10{border-color:rgba(8,47,73,.1)}.border-sky-950\/100{border-color:#082f49}.border-sky-950\/15{border-color:rgba(8,47,73,.15)}.border-sky-950\/20{border-color:rgba(8,47,73,.2)}.border-sky-950\/25{border-color:rgba(8,47,73,.25)}.border-sky-950\/30{border-color:rgba(8,47,73,.3)}.border-sky-950\/35{border-color:rgba(8,47,73,.35)}.border-sky-950\/40{border-color:rgba(8,47,73,.4)}.border-sky-950\/45{border-color:rgba(8,47,73,.45)}.border-sky-950\/5{border-color:rgba(8,47,73,.05)}.border-sky-950\/50{border-color:rgba(8,47,73,.5)}.border-sky-950\/55{border-color:rgba(8,47,73,.55)}.border-sky-950\/60{border-color:rgba(8,47,73,.6)}.border-sky-950\/65{border-color:rgba(8,47,73,.65)}.border-sky-950\/70{border-color:rgba(8,47,73,.7)}.border-sky-950\/75{border-color:rgba(8,47,73,.75)}.border-sky-950\/80{border-color:rgba(8,47,73,.8)}.border-sky-950\/85{border-color:rgba(8,47,73,.85)}.border-sky-950\/90{border-color:rgba(8,47,73,.9)}.border-sky-950\/95{border-color:rgba(8,47,73,.95)}.border-slate-100{--tw-border-opacity:1;border-color:rgb(241 245 249/var(--tw-border-opacity,1))}.border-slate-100\/0{border-color:rgba(241,245,249,0)}.border-slate-100\/10{border-color:rgba(241,245,249,.1)}.border-slate-100\/100{border-color:#f1f5f9}.border-slate-100\/15{border-color:rgba(241,245,249,.15)}.border-slate-100\/20{border-color:rgba(241,245,249,.2)}.border-slate-100\/25{border-color:rgba(241,245,249,.25)}.border-slate-100\/30{border-color:rgba(241,245,249,.3)}.border-slate-100\/35{border-color:rgba(241,245,249,.35)}.border-slate-100\/40{border-color:rgba(241,245,249,.4)}.border-slate-100\/45{border-color:rgba(241,245,249,.45)}.border-slate-100\/5{border-color:rgba(241,245,249,.05)}.border-slate-100\/50{border-color:rgba(241,245,249,.5)}.border-slate-100\/55{border-color:rgba(241,245,249,.55)}.border-slate-100\/60{border-color:rgba(241,245,249,.6)}.border-slate-100\/65{border-color:rgba(241,245,249,.65)}.border-slate-100\/70{border-color:rgba(241,245,249,.7)}.border-slate-100\/75{border-color:rgba(241,245,249,.75)}.border-slate-100\/80{border-color:rgba(241,245,249,.8)}.border-slate-100\/85{border-color:rgba(241,245,249,.85)}.border-slate-100\/90{border-color:rgba(241,245,249,.9)}.border-slate-100\/95{border-color:rgba(241,245,249,.95)}.border-slate-200{--tw-border-opacity:1;border-color:rgb(226 232 240/var(--tw-border-opacity,1))}.border-slate-200\/0{border-color:rgba(226,232,240,0)}.border-slate-200\/10{border-color:rgba(226,232,240,.1)}.border-slate-200\/100{border-color:#e2e8f0}.border-slate-200\/15{border-color:rgba(226,232,240,.15)}.border-slate-200\/20{border-color:rgba(226,232,240,.2)}.border-slate-200\/25{border-color:rgba(226,232,240,.25)}.border-slate-200\/30{border-color:rgba(226,232,240,.3)}.border-slate-200\/35{border-color:rgba(226,232,240,.35)}.border-slate-200\/40{border-color:rgba(226,232,240,.4)}.border-slate-200\/45{border-color:rgba(226,232,240,.45)}.border-slate-200\/5{border-color:rgba(226,232,240,.05)}.border-slate-200\/50{border-color:rgba(226,232,240,.5)}.border-slate-200\/55{border-color:rgba(226,232,240,.55)}.border-slate-200\/60{border-color:rgba(226,232,240,.6)}.border-slate-200\/65{border-color:rgba(226,232,240,.65)}.border-slate-200\/70{border-color:rgba(226,232,240,.7)}.border-slate-200\/75{border-color:rgba(226,232,240,.75)}.border-slate-200\/80{border-color:rgba(226,232,240,.8)}.border-slate-200\/85{border-color:rgba(226,232,240,.85)}.border-slate-200\/90{border-color:rgba(226,232,240,.9)}.border-slate-200\/95{border-color:rgba(226,232,240,.95)}.border-slate-50{--tw-border-opacity:1;border-color:rgb(248 250 252/var(--tw-border-opacity,1))}.border-slate-50\/0{border-color:rgba(248,250,252,0)}.border-slate-50\/10{border-color:rgba(248,250,252,.1)}.border-slate-50\/100{border-color:#f8fafc}.border-slate-50\/15{border-color:rgba(248,250,252,.15)}.border-slate-50\/20{border-color:rgba(248,250,252,.2)}.border-slate-50\/25{border-color:rgba(248,250,252,.25)}.border-slate-50\/30{border-color:rgba(248,250,252,.3)}.border-slate-50\/35{border-color:rgba(248,250,252,.35)}.border-slate-50\/40{border-color:rgba(248,250,252,.4)}.border-slate-50\/45{border-color:rgba(248,250,252,.45)}.border-slate-50\/5{border-color:rgba(248,250,252,.05)}.border-slate-50\/50{border-color:rgba(248,250,252,.5)}.border-slate-50\/55{border-color:rgba(248,250,252,.55)}.border-slate-50\/60{border-color:rgba(248,250,252,.6)}.border-slate-50\/65{border-color:rgba(248,250,252,.65)}.border-slate-50\/70{border-color:rgba(248,250,252,.7)}.border-slate-50\/75{border-color:rgba(248,250,252,.75)}.border-slate-50\/80{border-color:rgba(248,250,252,.8)}.border-slate-50\/85{border-color:rgba(248,250,252,.85)}.border-slate-50\/90{border-color:rgba(248,250,252,.9)}.border-slate-50\/95{border-color:rgba(248,250,252,.95)}.border-slate-500{--tw-border-opacity:1;border-color:rgb(100 116 139/var(--tw-border-opacity,1))}.border-slate-500\/0{border-color:rgba(100,116,139,0)}.border-slate-500\/10{border-color:rgba(100,116,139,.1)}.border-slate-500\/100{border-color:#64748b}.border-slate-500\/15{border-color:rgba(100,116,139,.15)}.border-slate-500\/20{border-color:rgba(100,116,139,.2)}.border-slate-500\/25{border-color:rgba(100,116,139,.25)}.border-slate-500\/30{border-color:rgba(100,116,139,.3)}.border-slate-500\/35{border-color:rgba(100,116,139,.35)}.border-slate-500\/40{border-color:rgba(100,116,139,.4)}.border-slate-500\/45{border-color:rgba(100,116,139,.45)}.border-slate-500\/5{border-color:rgba(100,116,139,.05)}.border-slate-500\/50{border-color:rgba(100,116,139,.5)}.border-slate-500\/55{border-color:rgba(100,116,139,.55)}.border-slate-500\/60{border-color:rgba(100,116,139,.6)}.border-slate-500\/65{border-color:rgba(100,116,139,.65)}.border-slate-500\/70{border-color:rgba(100,116,139,.7)}.border-slate-500\/75{border-color:rgba(100,116,139,.75)}.border-slate-500\/80{border-color:rgba(100,116,139,.8)}.border-slate-500\/85{border-color:rgba(100,116,139,.85)}.border-slate-500\/90{border-color:rgba(100,116,139,.9)}.border-slate-500\/95{border-color:rgba(100,116,139,.95)}.border-slate-950{--tw-border-opacity:1;border-color:rgb(2 6 23/var(--tw-border-opacity,1))}.border-slate-950\/0{border-color:rgba(2,6,23,0)}.border-slate-950\/10{border-color:rgba(2,6,23,.1)}.border-slate-950\/100{border-color:#020617}.border-slate-950\/15{border-color:rgba(2,6,23,.15)}.border-slate-950\/20{border-color:rgba(2,6,23,.2)}.border-slate-950\/25{border-color:rgba(2,6,23,.25)}.border-slate-950\/30{border-color:rgba(2,6,23,.3)}.border-slate-950\/35{border-color:rgba(2,6,23,.35)}.border-slate-950\/40{border-color:rgba(2,6,23,.4)}.border-slate-950\/45{border-color:rgba(2,6,23,.45)}.border-slate-950\/5{border-color:rgba(2,6,23,.05)}.border-slate-950\/50{border-color:rgba(2,6,23,.5)}.border-slate-950\/55{border-color:rgba(2,6,23,.55)}.border-slate-950\/60{border-color:rgba(2,6,23,.6)}.border-slate-950\/65{border-color:rgba(2,6,23,.65)}.border-slate-950\/70{border-color:rgba(2,6,23,.7)}.border-slate-950\/75{border-color:rgba(2,6,23,.75)}.border-slate-950\/80{border-color:rgba(2,6,23,.8)}.border-slate-950\/85{border-color:rgba(2,6,23,.85)}.border-slate-950\/90{border-color:rgba(2,6,23,.9)}.border-slate-950\/95{border-color:rgba(2,6,23,.95)}.border-stone-100{--tw-border-opacity:1;border-color:rgb(245 245 244/var(--tw-border-opacity,1))}.border-stone-100\/0{border-color:hsla(60,5%,96%,0)}.border-stone-100\/10{border-color:hsla(60,5%,96%,.1)}.border-stone-100\/100{border-color:#f5f5f4}.border-stone-100\/15{border-color:hsla(60,5%,96%,.15)}.border-stone-100\/20{border-color:hsla(60,5%,96%,.2)}.border-stone-100\/25{border-color:hsla(60,5%,96%,.25)}.border-stone-100\/30{border-color:hsla(60,5%,96%,.3)}.border-stone-100\/35{border-color:hsla(60,5%,96%,.35)}.border-stone-100\/40{border-color:hsla(60,5%,96%,.4)}.border-stone-100\/45{border-color:hsla(60,5%,96%,.45)}.border-stone-100\/5{border-color:hsla(60,5%,96%,.05)}.border-stone-100\/50{border-color:hsla(60,5%,96%,.5)}.border-stone-100\/55{border-color:hsla(60,5%,96%,.55)}.border-stone-100\/60{border-color:hsla(60,5%,96%,.6)}.border-stone-100\/65{border-color:hsla(60,5%,96%,.65)}.border-stone-100\/70{border-color:hsla(60,5%,96%,.7)}.border-stone-100\/75{border-color:hsla(60,5%,96%,.75)}.border-stone-100\/80{border-color:hsla(60,5%,96%,.8)}.border-stone-100\/85{border-color:hsla(60,5%,96%,.85)}.border-stone-100\/90{border-color:hsla(60,5%,96%,.9)}.border-stone-100\/95{border-color:hsla(60,5%,96%,.95)}.border-stone-200{--tw-border-opacity:1;border-color:rgb(231 229 228/var(--tw-border-opacity,1))}.border-stone-200\/0{border-color:hsla(20,6%,90%,0)}.border-stone-200\/10{border-color:hsla(20,6%,90%,.1)}.border-stone-200\/100{border-color:#e7e5e4}.border-stone-200\/15{border-color:hsla(20,6%,90%,.15)}.border-stone-200\/20{border-color:hsla(20,6%,90%,.2)}.border-stone-200\/25{border-color:hsla(20,6%,90%,.25)}.border-stone-200\/30{border-color:hsla(20,6%,90%,.3)}.border-stone-200\/35{border-color:hsla(20,6%,90%,.35)}.border-stone-200\/40{border-color:hsla(20,6%,90%,.4)}.border-stone-200\/45{border-color:hsla(20,6%,90%,.45)}.border-stone-200\/5{border-color:hsla(20,6%,90%,.05)}.border-stone-200\/50{border-color:hsla(20,6%,90%,.5)}.border-stone-200\/55{border-color:hsla(20,6%,90%,.55)}.border-stone-200\/60{border-color:hsla(20,6%,90%,.6)}.border-stone-200\/65{border-color:hsla(20,6%,90%,.65)}.border-stone-200\/70{border-color:hsla(20,6%,90%,.7)}.border-stone-200\/75{border-color:hsla(20,6%,90%,.75)}.border-stone-200\/80{border-color:hsla(20,6%,90%,.8)}.border-stone-200\/85{border-color:hsla(20,6%,90%,.85)}.border-stone-200\/90{border-color:hsla(20,6%,90%,.9)}.border-stone-200\/95{border-color:hsla(20,6%,90%,.95)}.border-stone-50{--tw-border-opacity:1;border-color:rgb(250 250 249/var(--tw-border-opacity,1))}.border-stone-50\/0{border-color:hsla(60,9%,98%,0)}.border-stone-50\/10{border-color:hsla(60,9%,98%,.1)}.border-stone-50\/100{border-color:#fafaf9}.border-stone-50\/15{border-color:hsla(60,9%,98%,.15)}.border-stone-50\/20{border-color:hsla(60,9%,98%,.2)}.border-stone-50\/25{border-color:hsla(60,9%,98%,.25)}.border-stone-50\/30{border-color:hsla(60,9%,98%,.3)}.border-stone-50\/35{border-color:hsla(60,9%,98%,.35)}.border-stone-50\/40{border-color:hsla(60,9%,98%,.4)}.border-stone-50\/45{border-color:hsla(60,9%,98%,.45)}.border-stone-50\/5{border-color:hsla(60,9%,98%,.05)}.border-stone-50\/50{border-color:hsla(60,9%,98%,.5)}.border-stone-50\/55{border-color:hsla(60,9%,98%,.55)}.border-stone-50\/60{border-color:hsla(60,9%,98%,.6)}.border-stone-50\/65{border-color:hsla(60,9%,98%,.65)}.border-stone-50\/70{border-color:hsla(60,9%,98%,.7)}.border-stone-50\/75{border-color:hsla(60,9%,98%,.75)}.border-stone-50\/80{border-color:hsla(60,9%,98%,.8)}.border-stone-50\/85{border-color:hsla(60,9%,98%,.85)}.border-stone-50\/90{border-color:hsla(60,9%,98%,.9)}.border-stone-50\/95{border-color:hsla(60,9%,98%,.95)}.border-stone-500{--tw-border-opacity:1;border-color:rgb(120 113 108/var(--tw-border-opacity,1))}.border-stone-500\/0{border-color:hsla(25,5%,45%,0)}.border-stone-500\/10{border-color:hsla(25,5%,45%,.1)}.border-stone-500\/100{border-color:#78716c}.border-stone-500\/15{border-color:hsla(25,5%,45%,.15)}.border-stone-500\/20{border-color:hsla(25,5%,45%,.2)}.border-stone-500\/25{border-color:hsla(25,5%,45%,.25)}.border-stone-500\/30{border-color:hsla(25,5%,45%,.3)}.border-stone-500\/35{border-color:hsla(25,5%,45%,.35)}.border-stone-500\/40{border-color:hsla(25,5%,45%,.4)}.border-stone-500\/45{border-color:hsla(25,5%,45%,.45)}.border-stone-500\/5{border-color:hsla(25,5%,45%,.05)}.border-stone-500\/50{border-color:hsla(25,5%,45%,.5)}.border-stone-500\/55{border-color:hsla(25,5%,45%,.55)}.border-stone-500\/60{border-color:hsla(25,5%,45%,.6)}.border-stone-500\/65{border-color:hsla(25,5%,45%,.65)}.border-stone-500\/70{border-color:hsla(25,5%,45%,.7)}.border-stone-500\/75{border-color:hsla(25,5%,45%,.75)}.border-stone-500\/80{border-color:hsla(25,5%,45%,.8)}.border-stone-500\/85{border-color:hsla(25,5%,45%,.85)}.border-stone-500\/90{border-color:hsla(25,5%,45%,.9)}.border-stone-500\/95{border-color:hsla(25,5%,45%,.95)}.border-stone-700{--tw-border-opacity:1;border-color:rgb(68 64 60/var(--tw-border-opacity,1))}.border-stone-950{--tw-border-opacity:1;border-color:rgb(12 10 9/var(--tw-border-opacity,1))}.border-stone-950\/0{border-color:rgba(12,10,9,0)}.border-stone-950\/10{border-color:rgba(12,10,9,.1)}.border-stone-950\/100{border-color:#0c0a09}.border-stone-950\/15{border-color:rgba(12,10,9,.15)}.border-stone-950\/20{border-color:rgba(12,10,9,.2)}.border-stone-950\/25{border-color:rgba(12,10,9,.25)}.border-stone-950\/30{border-color:rgba(12,10,9,.3)}.border-stone-950\/35{border-color:rgba(12,10,9,.35)}.border-stone-950\/40{border-color:rgba(12,10,9,.4)}.border-stone-950\/45{border-color:rgba(12,10,9,.45)}.border-stone-950\/5{border-color:rgba(12,10,9,.05)}.border-stone-950\/50{border-color:rgba(12,10,9,.5)}.border-stone-950\/55{border-color:rgba(12,10,9,.55)}.border-stone-950\/60{border-color:rgba(12,10,9,.6)}.border-stone-950\/65{border-color:rgba(12,10,9,.65)}.border-stone-950\/70{border-color:rgba(12,10,9,.7)}.border-stone-950\/75{border-color:rgba(12,10,9,.75)}.border-stone-950\/80{border-color:rgba(12,10,9,.8)}.border-stone-950\/85{border-color:rgba(12,10,9,.85)}.border-stone-950\/90{border-color:rgba(12,10,9,.9)}.border-stone-950\/95{border-color:rgba(12,10,9,.95)}.border-teal-100{--tw-border-opacity:1;border-color:rgb(204 251 241/var(--tw-border-opacity,1))}.border-teal-100\/0{border-color:rgba(204,251,241,0)}.border-teal-100\/10{border-color:rgba(204,251,241,.1)}.border-teal-100\/100{border-color:#ccfbf1}.border-teal-100\/15{border-color:rgba(204,251,241,.15)}.border-teal-100\/20{border-color:rgba(204,251,241,.2)}.border-teal-100\/25{border-color:rgba(204,251,241,.25)}.border-teal-100\/30{border-color:rgba(204,251,241,.3)}.border-teal-100\/35{border-color:rgba(204,251,241,.35)}.border-teal-100\/40{border-color:rgba(204,251,241,.4)}.border-teal-100\/45{border-color:rgba(204,251,241,.45)}.border-teal-100\/5{border-color:rgba(204,251,241,.05)}.border-teal-100\/50{border-color:rgba(204,251,241,.5)}.border-teal-100\/55{border-color:rgba(204,251,241,.55)}.border-teal-100\/60{border-color:rgba(204,251,241,.6)}.border-teal-100\/65{border-color:rgba(204,251,241,.65)}.border-teal-100\/70{border-color:rgba(204,251,241,.7)}.border-teal-100\/75{border-color:rgba(204,251,241,.75)}.border-teal-100\/80{border-color:rgba(204,251,241,.8)}.border-teal-100\/85{border-color:rgba(204,251,241,.85)}.border-teal-100\/90{border-color:rgba(204,251,241,.9)}.border-teal-100\/95{border-color:rgba(204,251,241,.95)}.border-teal-200{--tw-border-opacity:1;border-color:rgb(153 246 228/var(--tw-border-opacity,1))}.border-teal-200\/0{border-color:rgba(153,246,228,0)}.border-teal-200\/10{border-color:rgba(153,246,228,.1)}.border-teal-200\/100{border-color:#99f6e4}.border-teal-200\/15{border-color:rgba(153,246,228,.15)}.border-teal-200\/20{border-color:rgba(153,246,228,.2)}.border-teal-200\/25{border-color:rgba(153,246,228,.25)}.border-teal-200\/30{border-color:rgba(153,246,228,.3)}.border-teal-200\/35{border-color:rgba(153,246,228,.35)}.border-teal-200\/40{border-color:rgba(153,246,228,.4)}.border-teal-200\/45{border-color:rgba(153,246,228,.45)}.border-teal-200\/5{border-color:rgba(153,246,228,.05)}.border-teal-200\/50{border-color:rgba(153,246,228,.5)}.border-teal-200\/55{border-color:rgba(153,246,228,.55)}.border-teal-200\/60{border-color:rgba(153,246,228,.6)}.border-teal-200\/65{border-color:rgba(153,246,228,.65)}.border-teal-200\/70{border-color:rgba(153,246,228,.7)}.border-teal-200\/75{border-color:rgba(153,246,228,.75)}.border-teal-200\/80{border-color:rgba(153,246,228,.8)}.border-teal-200\/85{border-color:rgba(153,246,228,.85)}.border-teal-200\/90{border-color:rgba(153,246,228,.9)}.border-teal-200\/95{border-color:rgba(153,246,228,.95)}.border-teal-50{--tw-border-opacity:1;border-color:rgb(240 253 250/var(--tw-border-opacity,1))}.border-teal-50\/0{border-color:rgba(240,253,250,0)}.border-teal-50\/10{border-color:rgba(240,253,250,.1)}.border-teal-50\/100{border-color:#f0fdfa}.border-teal-50\/15{border-color:rgba(240,253,250,.15)}.border-teal-50\/20{border-color:rgba(240,253,250,.2)}.border-teal-50\/25{border-color:rgba(240,253,250,.25)}.border-teal-50\/30{border-color:rgba(240,253,250,.3)}.border-teal-50\/35{border-color:rgba(240,253,250,.35)}.border-teal-50\/40{border-color:rgba(240,253,250,.4)}.border-teal-50\/45{border-color:rgba(240,253,250,.45)}.border-teal-50\/5{border-color:rgba(240,253,250,.05)}.border-teal-50\/50{border-color:rgba(240,253,250,.5)}.border-teal-50\/55{border-color:rgba(240,253,250,.55)}.border-teal-50\/60{border-color:rgba(240,253,250,.6)}.border-teal-50\/65{border-color:rgba(240,253,250,.65)}.border-teal-50\/70{border-color:rgba(240,253,250,.7)}.border-teal-50\/75{border-color:rgba(240,253,250,.75)}.border-teal-50\/80{border-color:rgba(240,253,250,.8)}.border-teal-50\/85{border-color:rgba(240,253,250,.85)}.border-teal-50\/90{border-color:rgba(240,253,250,.9)}.border-teal-50\/95{border-color:rgba(240,253,250,.95)}.border-teal-500{--tw-border-opacity:1;border-color:rgb(20 184 166/var(--tw-border-opacity,1))}.border-teal-500\/0{border-color:rgba(20,184,166,0)}.border-teal-500\/10{border-color:rgba(20,184,166,.1)}.border-teal-500\/100{border-color:#14b8a6}.border-teal-500\/15{border-color:rgba(20,184,166,.15)}.border-teal-500\/20{border-color:rgba(20,184,166,.2)}.border-teal-500\/25{border-color:rgba(20,184,166,.25)}.border-teal-500\/30{border-color:rgba(20,184,166,.3)}.border-teal-500\/35{border-color:rgba(20,184,166,.35)}.border-teal-500\/40{border-color:rgba(20,184,166,.4)}.border-teal-500\/45{border-color:rgba(20,184,166,.45)}.border-teal-500\/5{border-color:rgba(20,184,166,.05)}.border-teal-500\/50{border-color:rgba(20,184,166,.5)}.border-teal-500\/55{border-color:rgba(20,184,166,.55)}.border-teal-500\/60{border-color:rgba(20,184,166,.6)}.border-teal-500\/65{border-color:rgba(20,184,166,.65)}.border-teal-500\/70{border-color:rgba(20,184,166,.7)}.border-teal-500\/75{border-color:rgba(20,184,166,.75)}.border-teal-500\/80{border-color:rgba(20,184,166,.8)}.border-teal-500\/85{border-color:rgba(20,184,166,.85)}.border-teal-500\/90{border-color:rgba(20,184,166,.9)}.border-teal-500\/95{border-color:rgba(20,184,166,.95)}.border-teal-950{--tw-border-opacity:1;border-color:rgb(4 47 46/var(--tw-border-opacity,1))}.border-teal-950\/0{border-color:rgba(4,47,46,0)}.border-teal-950\/10{border-color:rgba(4,47,46,.1)}.border-teal-950\/100{border-color:#042f2e}.border-teal-950\/15{border-color:rgba(4,47,46,.15)}.border-teal-950\/20{border-color:rgba(4,47,46,.2)}.border-teal-950\/25{border-color:rgba(4,47,46,.25)}.border-teal-950\/30{border-color:rgba(4,47,46,.3)}.border-teal-950\/35{border-color:rgba(4,47,46,.35)}.border-teal-950\/40{border-color:rgba(4,47,46,.4)}.border-teal-950\/45{border-color:rgba(4,47,46,.45)}.border-teal-950\/5{border-color:rgba(4,47,46,.05)}.border-teal-950\/50{border-color:rgba(4,47,46,.5)}.border-teal-950\/55{border-color:rgba(4,47,46,.55)}.border-teal-950\/60{border-color:rgba(4,47,46,.6)}.border-teal-950\/65{border-color:rgba(4,47,46,.65)}.border-teal-950\/70{border-color:rgba(4,47,46,.7)}.border-teal-950\/75{border-color:rgba(4,47,46,.75)}.border-teal-950\/80{border-color:rgba(4,47,46,.8)}.border-teal-950\/85{border-color:rgba(4,47,46,.85)}.border-teal-950\/90{border-color:rgba(4,47,46,.9)}.border-teal-950\/95{border-color:rgba(4,47,46,.95)}.border-violet-100{--tw-border-opacity:1;border-color:rgb(237 233 254/var(--tw-border-opacity,1))}.border-violet-100\/0{border-color:rgba(237,233,254,0)}.border-violet-100\/10{border-color:rgba(237,233,254,.1)}.border-violet-100\/100{border-color:#ede9fe}.border-violet-100\/15{border-color:rgba(237,233,254,.15)}.border-violet-100\/20{border-color:rgba(237,233,254,.2)}.border-violet-100\/25{border-color:rgba(237,233,254,.25)}.border-violet-100\/30{border-color:rgba(237,233,254,.3)}.border-violet-100\/35{border-color:rgba(237,233,254,.35)}.border-violet-100\/40{border-color:rgba(237,233,254,.4)}.border-violet-100\/45{border-color:rgba(237,233,254,.45)}.border-violet-100\/5{border-color:rgba(237,233,254,.05)}.border-violet-100\/50{border-color:rgba(237,233,254,.5)}.border-violet-100\/55{border-color:rgba(237,233,254,.55)}.border-violet-100\/60{border-color:rgba(237,233,254,.6)}.border-violet-100\/65{border-color:rgba(237,233,254,.65)}.border-violet-100\/70{border-color:rgba(237,233,254,.7)}.border-violet-100\/75{border-color:rgba(237,233,254,.75)}.border-violet-100\/80{border-color:rgba(237,233,254,.8)}.border-violet-100\/85{border-color:rgba(237,233,254,.85)}.border-violet-100\/90{border-color:rgba(237,233,254,.9)}.border-violet-100\/95{border-color:rgba(237,233,254,.95)}.border-violet-200{--tw-border-opacity:1;border-color:rgb(221 214 254/var(--tw-border-opacity,1))}.border-violet-200\/0{border-color:rgba(221,214,254,0)}.border-violet-200\/10{border-color:rgba(221,214,254,.1)}.border-violet-200\/100{border-color:#ddd6fe}.border-violet-200\/15{border-color:rgba(221,214,254,.15)}.border-violet-200\/20{border-color:rgba(221,214,254,.2)}.border-violet-200\/25{border-color:rgba(221,214,254,.25)}.border-violet-200\/30{border-color:rgba(221,214,254,.3)}.border-violet-200\/35{border-color:rgba(221,214,254,.35)}.border-violet-200\/40{border-color:rgba(221,214,254,.4)}.border-violet-200\/45{border-color:rgba(221,214,254,.45)}.border-violet-200\/5{border-color:rgba(221,214,254,.05)}.border-violet-200\/50{border-color:rgba(221,214,254,.5)}.border-violet-200\/55{border-color:rgba(221,214,254,.55)}.border-violet-200\/60{border-color:rgba(221,214,254,.6)}.border-violet-200\/65{border-color:rgba(221,214,254,.65)}.border-violet-200\/70{border-color:rgba(221,214,254,.7)}.border-violet-200\/75{border-color:rgba(221,214,254,.75)}.border-violet-200\/80{border-color:rgba(221,214,254,.8)}.border-violet-200\/85{border-color:rgba(221,214,254,.85)}.border-violet-200\/90{border-color:rgba(221,214,254,.9)}.border-violet-200\/95{border-color:rgba(221,214,254,.95)}.border-violet-50{--tw-border-opacity:1;border-color:rgb(245 243 255/var(--tw-border-opacity,1))}.border-violet-50\/0{border-color:rgba(245,243,255,0)}.border-violet-50\/10{border-color:rgba(245,243,255,.1)}.border-violet-50\/100{border-color:#f5f3ff}.border-violet-50\/15{border-color:rgba(245,243,255,.15)}.border-violet-50\/20{border-color:rgba(245,243,255,.2)}.border-violet-50\/25{border-color:rgba(245,243,255,.25)}.border-violet-50\/30{border-color:rgba(245,243,255,.3)}.border-violet-50\/35{border-color:rgba(245,243,255,.35)}.border-violet-50\/40{border-color:rgba(245,243,255,.4)}.border-violet-50\/45{border-color:rgba(245,243,255,.45)}.border-violet-50\/5{border-color:rgba(245,243,255,.05)}.border-violet-50\/50{border-color:rgba(245,243,255,.5)}.border-violet-50\/55{border-color:rgba(245,243,255,.55)}.border-violet-50\/60{border-color:rgba(245,243,255,.6)}.border-violet-50\/65{border-color:rgba(245,243,255,.65)}.border-violet-50\/70{border-color:rgba(245,243,255,.7)}.border-violet-50\/75{border-color:rgba(245,243,255,.75)}.border-violet-50\/80{border-color:rgba(245,243,255,.8)}.border-violet-50\/85{border-color:rgba(245,243,255,.85)}.border-violet-50\/90{border-color:rgba(245,243,255,.9)}.border-violet-50\/95{border-color:rgba(245,243,255,.95)}.border-violet-500{--tw-border-opacity:1;border-color:rgb(139 92 246/var(--tw-border-opacity,1))}.border-violet-500\/0{border-color:rgba(139,92,246,0)}.border-violet-500\/10{border-color:rgba(139,92,246,.1)}.border-violet-500\/100{border-color:#8b5cf6}.border-violet-500\/15{border-color:rgba(139,92,246,.15)}.border-violet-500\/20{border-color:rgba(139,92,246,.2)}.border-violet-500\/25{border-color:rgba(139,92,246,.25)}.border-violet-500\/30{border-color:rgba(139,92,246,.3)}.border-violet-500\/35{border-color:rgba(139,92,246,.35)}.border-violet-500\/40{border-color:rgba(139,92,246,.4)}.border-violet-500\/45{border-color:rgba(139,92,246,.45)}.border-violet-500\/5{border-color:rgba(139,92,246,.05)}.border-violet-500\/50{border-color:rgba(139,92,246,.5)}.border-violet-500\/55{border-color:rgba(139,92,246,.55)}.border-violet-500\/60{border-color:rgba(139,92,246,.6)}.border-violet-500\/65{border-color:rgba(139,92,246,.65)}.border-violet-500\/70{border-color:rgba(139,92,246,.7)}.border-violet-500\/75{border-color:rgba(139,92,246,.75)}.border-violet-500\/80{border-color:rgba(139,92,246,.8)}.border-violet-500\/85{border-color:rgba(139,92,246,.85)}.border-violet-500\/90{border-color:rgba(139,92,246,.9)}.border-violet-500\/95{border-color:rgba(139,92,246,.95)}.border-violet-950{--tw-border-opacity:1;border-color:rgb(46 16 101/var(--tw-border-opacity,1))}.border-violet-950\/0{border-color:rgba(46,16,101,0)}.border-violet-950\/10{border-color:rgba(46,16,101,.1)}.border-violet-950\/100{border-color:#2e1065}.border-violet-950\/15{border-color:rgba(46,16,101,.15)}.border-violet-950\/20{border-color:rgba(46,16,101,.2)}.border-violet-950\/25{border-color:rgba(46,16,101,.25)}.border-violet-950\/30{border-color:rgba(46,16,101,.3)}.border-violet-950\/35{border-color:rgba(46,16,101,.35)}.border-violet-950\/40{border-color:rgba(46,16,101,.4)}.border-violet-950\/45{border-color:rgba(46,16,101,.45)}.border-violet-950\/5{border-color:rgba(46,16,101,.05)}.border-violet-950\/50{border-color:rgba(46,16,101,.5)}.border-violet-950\/55{border-color:rgba(46,16,101,.55)}.border-violet-950\/60{border-color:rgba(46,16,101,.6)}.border-violet-950\/65{border-color:rgba(46,16,101,.65)}.border-violet-950\/70{border-color:rgba(46,16,101,.7)}.border-violet-950\/75{border-color:rgba(46,16,101,.75)}.border-violet-950\/80{border-color:rgba(46,16,101,.8)}.border-violet-950\/85{border-color:rgba(46,16,101,.85)}.border-violet-950\/90{border-color:rgba(46,16,101,.9)}.border-violet-950\/95{border-color:rgba(46,16,101,.95)}.border-yellow-100{--tw-border-opacity:1;border-color:rgb(254 249 195/var(--tw-border-opacity,1))}.border-yellow-100\/0{border-color:hsla(55,97%,88%,0)}.border-yellow-100\/10{border-color:hsla(55,97%,88%,.1)}.border-yellow-100\/100{border-color:#fef9c3}.border-yellow-100\/15{border-color:hsla(55,97%,88%,.15)}.border-yellow-100\/20{border-color:hsla(55,97%,88%,.2)}.border-yellow-100\/25{border-color:hsla(55,97%,88%,.25)}.border-yellow-100\/30{border-color:hsla(55,97%,88%,.3)}.border-yellow-100\/35{border-color:hsla(55,97%,88%,.35)}.border-yellow-100\/40{border-color:hsla(55,97%,88%,.4)}.border-yellow-100\/45{border-color:hsla(55,97%,88%,.45)}.border-yellow-100\/5{border-color:hsla(55,97%,88%,.05)}.border-yellow-100\/50{border-color:hsla(55,97%,88%,.5)}.border-yellow-100\/55{border-color:hsla(55,97%,88%,.55)}.border-yellow-100\/60{border-color:hsla(55,97%,88%,.6)}.border-yellow-100\/65{border-color:hsla(55,97%,88%,.65)}.border-yellow-100\/70{border-color:hsla(55,97%,88%,.7)}.border-yellow-100\/75{border-color:hsla(55,97%,88%,.75)}.border-yellow-100\/80{border-color:hsla(55,97%,88%,.8)}.border-yellow-100\/85{border-color:hsla(55,97%,88%,.85)}.border-yellow-100\/90{border-color:hsla(55,97%,88%,.9)}.border-yellow-100\/95{border-color:hsla(55,97%,88%,.95)}.border-yellow-200{--tw-border-opacity:1;border-color:rgb(254 240 138/var(--tw-border-opacity,1))}.border-yellow-200\/0{border-color:hsla(53,98%,77%,0)}.border-yellow-200\/10{border-color:hsla(53,98%,77%,.1)}.border-yellow-200\/100{border-color:#fef08a}.border-yellow-200\/15{border-color:hsla(53,98%,77%,.15)}.border-yellow-200\/20{border-color:hsla(53,98%,77%,.2)}.border-yellow-200\/25{border-color:hsla(53,98%,77%,.25)}.border-yellow-200\/30{border-color:hsla(53,98%,77%,.3)}.border-yellow-200\/35{border-color:hsla(53,98%,77%,.35)}.border-yellow-200\/40{border-color:hsla(53,98%,77%,.4)}.border-yellow-200\/45{border-color:hsla(53,98%,77%,.45)}.border-yellow-200\/5{border-color:hsla(53,98%,77%,.05)}.border-yellow-200\/50{border-color:hsla(53,98%,77%,.5)}.border-yellow-200\/55{border-color:hsla(53,98%,77%,.55)}.border-yellow-200\/60{border-color:hsla(53,98%,77%,.6)}.border-yellow-200\/65{border-color:hsla(53,98%,77%,.65)}.border-yellow-200\/70{border-color:hsla(53,98%,77%,.7)}.border-yellow-200\/75{border-color:hsla(53,98%,77%,.75)}.border-yellow-200\/80{border-color:hsla(53,98%,77%,.8)}.border-yellow-200\/85{border-color:hsla(53,98%,77%,.85)}.border-yellow-200\/90{border-color:hsla(53,98%,77%,.9)}.border-yellow-200\/95{border-color:hsla(53,98%,77%,.95)}.border-yellow-50{--tw-border-opacity:1;border-color:rgb(254 252 232/var(--tw-border-opacity,1))}.border-yellow-50\/0{border-color:hsla(55,92%,95%,0)}.border-yellow-50\/10{border-color:hsla(55,92%,95%,.1)}.border-yellow-50\/100{border-color:#fefce8}.border-yellow-50\/15{border-color:hsla(55,92%,95%,.15)}.border-yellow-50\/20{border-color:hsla(55,92%,95%,.2)}.border-yellow-50\/25{border-color:hsla(55,92%,95%,.25)}.border-yellow-50\/30{border-color:hsla(55,92%,95%,.3)}.border-yellow-50\/35{border-color:hsla(55,92%,95%,.35)}.border-yellow-50\/40{border-color:hsla(55,92%,95%,.4)}.border-yellow-50\/45{border-color:hsla(55,92%,95%,.45)}.border-yellow-50\/5{border-color:hsla(55,92%,95%,.05)}.border-yellow-50\/50{border-color:hsla(55,92%,95%,.5)}.border-yellow-50\/55{border-color:hsla(55,92%,95%,.55)}.border-yellow-50\/60{border-color:hsla(55,92%,95%,.6)}.border-yellow-50\/65{border-color:hsla(55,92%,95%,.65)}.border-yellow-50\/70{border-color:hsla(55,92%,95%,.7)}.border-yellow-50\/75{border-color:hsla(55,92%,95%,.75)}.border-yellow-50\/80{border-color:hsla(55,92%,95%,.8)}.border-yellow-50\/85{border-color:hsla(55,92%,95%,.85)}.border-yellow-50\/90{border-color:hsla(55,92%,95%,.9)}.border-yellow-50\/95{border-color:hsla(55,92%,95%,.95)}.border-yellow-500{--tw-border-opacity:1;border-color:rgb(234 179 8/var(--tw-border-opacity,1))}.border-yellow-500\/0{border-color:rgba(234,179,8,0)}.border-yellow-500\/10{border-color:rgba(234,179,8,.1)}.border-yellow-500\/100{border-color:#eab308}.border-yellow-500\/15{border-color:rgba(234,179,8,.15)}.border-yellow-500\/20{border-color:rgba(234,179,8,.2)}.border-yellow-500\/25{border-color:rgba(234,179,8,.25)}.border-yellow-500\/30{border-color:rgba(234,179,8,.3)}.border-yellow-500\/35{border-color:rgba(234,179,8,.35)}.border-yellow-500\/40{border-color:rgba(234,179,8,.4)}.border-yellow-500\/45{border-color:rgba(234,179,8,.45)}.border-yellow-500\/5{border-color:rgba(234,179,8,.05)}.border-yellow-500\/50{border-color:rgba(234,179,8,.5)}.border-yellow-500\/55{border-color:rgba(234,179,8,.55)}.border-yellow-500\/60{border-color:rgba(234,179,8,.6)}.border-yellow-500\/65{border-color:rgba(234,179,8,.65)}.border-yellow-500\/70{border-color:rgba(234,179,8,.7)}.border-yellow-500\/75{border-color:rgba(234,179,8,.75)}.border-yellow-500\/80{border-color:rgba(234,179,8,.8)}.border-yellow-500\/85{border-color:rgba(234,179,8,.85)}.border-yellow-500\/90{border-color:rgba(234,179,8,.9)}.border-yellow-500\/95{border-color:rgba(234,179,8,.95)}.border-yellow-950{--tw-border-opacity:1;border-color:rgb(66 32 6/var(--tw-border-opacity,1))}.border-yellow-950\/0{border-color:rgba(66,32,6,0)}.border-yellow-950\/10{border-color:rgba(66,32,6,.1)}.border-yellow-950\/100{border-color:#422006}.border-yellow-950\/15{border-color:rgba(66,32,6,.15)}.border-yellow-950\/20{border-color:rgba(66,32,6,.2)}.border-yellow-950\/25{border-color:rgba(66,32,6,.25)}.border-yellow-950\/30{border-color:rgba(66,32,6,.3)}.border-yellow-950\/35{border-color:rgba(66,32,6,.35)}.border-yellow-950\/40{border-color:rgba(66,32,6,.4)}.border-yellow-950\/45{border-color:rgba(66,32,6,.45)}.border-yellow-950\/5{border-color:rgba(66,32,6,.05)}.border-yellow-950\/50{border-color:rgba(66,32,6,.5)}.border-yellow-950\/55{border-color:rgba(66,32,6,.55)}.border-yellow-950\/60{border-color:rgba(66,32,6,.6)}.border-yellow-950\/65{border-color:rgba(66,32,6,.65)}.border-yellow-950\/70{border-color:rgba(66,32,6,.7)}.border-yellow-950\/75{border-color:rgba(66,32,6,.75)}.border-yellow-950\/80{border-color:rgba(66,32,6,.8)}.border-yellow-950\/85{border-color:rgba(66,32,6,.85)}.border-yellow-950\/90{border-color:rgba(66,32,6,.9)}.border-yellow-950\/95{border-color:rgba(66,32,6,.95)}.border-zinc-100{--tw-border-opacity:1;border-color:rgb(244 244 245/var(--tw-border-opacity,1))}.border-zinc-100\/0{border-color:hsla(240,5%,96%,0)}.border-zinc-100\/10{border-color:hsla(240,5%,96%,.1)}.border-zinc-100\/100{border-color:#f4f4f5}.border-zinc-100\/15{border-color:hsla(240,5%,96%,.15)}.border-zinc-100\/20{border-color:hsla(240,5%,96%,.2)}.border-zinc-100\/25{border-color:hsla(240,5%,96%,.25)}.border-zinc-100\/30{border-color:hsla(240,5%,96%,.3)}.border-zinc-100\/35{border-color:hsla(240,5%,96%,.35)}.border-zinc-100\/40{border-color:hsla(240,5%,96%,.4)}.border-zinc-100\/45{border-color:hsla(240,5%,96%,.45)}.border-zinc-100\/5{border-color:hsla(240,5%,96%,.05)}.border-zinc-100\/50{border-color:hsla(240,5%,96%,.5)}.border-zinc-100\/55{border-color:hsla(240,5%,96%,.55)}.border-zinc-100\/60{border-color:hsla(240,5%,96%,.6)}.border-zinc-100\/65{border-color:hsla(240,5%,96%,.65)}.border-zinc-100\/70{border-color:hsla(240,5%,96%,.7)}.border-zinc-100\/75{border-color:hsla(240,5%,96%,.75)}.border-zinc-100\/80{border-color:hsla(240,5%,96%,.8)}.border-zinc-100\/85{border-color:hsla(240,5%,96%,.85)}.border-zinc-100\/90{border-color:hsla(240,5%,96%,.9)}.border-zinc-100\/95{border-color:hsla(240,5%,96%,.95)}.border-zinc-200{--tw-border-opacity:1;border-color:rgb(228 228 231/var(--tw-border-opacity,1))}.border-zinc-200\/0{border-color:hsla(240,6%,90%,0)}.border-zinc-200\/10{border-color:hsla(240,6%,90%,.1)}.border-zinc-200\/100{border-color:#e4e4e7}.border-zinc-200\/15{border-color:hsla(240,6%,90%,.15)}.border-zinc-200\/20{border-color:hsla(240,6%,90%,.2)}.border-zinc-200\/25{border-color:hsla(240,6%,90%,.25)}.border-zinc-200\/30{border-color:hsla(240,6%,90%,.3)}.border-zinc-200\/35{border-color:hsla(240,6%,90%,.35)}.border-zinc-200\/40{border-color:hsla(240,6%,90%,.4)}.border-zinc-200\/45{border-color:hsla(240,6%,90%,.45)}.border-zinc-200\/5{border-color:hsla(240,6%,90%,.05)}.border-zinc-200\/50{border-color:hsla(240,6%,90%,.5)}.border-zinc-200\/55{border-color:hsla(240,6%,90%,.55)}.border-zinc-200\/60{border-color:hsla(240,6%,90%,.6)}.border-zinc-200\/65{border-color:hsla(240,6%,90%,.65)}.border-zinc-200\/70{border-color:hsla(240,6%,90%,.7)}.border-zinc-200\/75{border-color:hsla(240,6%,90%,.75)}.border-zinc-200\/80{border-color:hsla(240,6%,90%,.8)}.border-zinc-200\/85{border-color:hsla(240,6%,90%,.85)}.border-zinc-200\/90{border-color:hsla(240,6%,90%,.9)}.border-zinc-200\/95{border-color:hsla(240,6%,90%,.95)}.border-zinc-50{--tw-border-opacity:1;border-color:rgb(250 250 250/var(--tw-border-opacity,1))}.border-zinc-50\/0{border-color:hsla(0,0%,98%,0)}.border-zinc-50\/10{border-color:hsla(0,0%,98%,.1)}.border-zinc-50\/100{border-color:#fafafa}.border-zinc-50\/15{border-color:hsla(0,0%,98%,.15)}.border-zinc-50\/20{border-color:hsla(0,0%,98%,.2)}.border-zinc-50\/25{border-color:hsla(0,0%,98%,.25)}.border-zinc-50\/30{border-color:hsla(0,0%,98%,.3)}.border-zinc-50\/35{border-color:hsla(0,0%,98%,.35)}.border-zinc-50\/40{border-color:hsla(0,0%,98%,.4)}.border-zinc-50\/45{border-color:hsla(0,0%,98%,.45)}.border-zinc-50\/5{border-color:hsla(0,0%,98%,.05)}.border-zinc-50\/50{border-color:hsla(0,0%,98%,.5)}.border-zinc-50\/55{border-color:hsla(0,0%,98%,.55)}.border-zinc-50\/60{border-color:hsla(0,0%,98%,.6)}.border-zinc-50\/65{border-color:hsla(0,0%,98%,.65)}.border-zinc-50\/70{border-color:hsla(0,0%,98%,.7)}.border-zinc-50\/75{border-color:hsla(0,0%,98%,.75)}.border-zinc-50\/80{border-color:hsla(0,0%,98%,.8)}.border-zinc-50\/85{border-color:hsla(0,0%,98%,.85)}.border-zinc-50\/90{border-color:hsla(0,0%,98%,.9)}.border-zinc-50\/95{border-color:hsla(0,0%,98%,.95)}.border-zinc-500{--tw-border-opacity:1;border-color:rgb(113 113 122/var(--tw-border-opacity,1))}.border-zinc-500\/0{border-color:hsla(240,4%,46%,0)}.border-zinc-500\/10{border-color:hsla(240,4%,46%,.1)}.border-zinc-500\/100{border-color:#71717a}.border-zinc-500\/15{border-color:hsla(240,4%,46%,.15)}.border-zinc-500\/20{border-color:hsla(240,4%,46%,.2)}.border-zinc-500\/25{border-color:hsla(240,4%,46%,.25)}.border-zinc-500\/30{border-color:hsla(240,4%,46%,.3)}.border-zinc-500\/35{border-color:hsla(240,4%,46%,.35)}.border-zinc-500\/40{border-color:hsla(240,4%,46%,.4)}.border-zinc-500\/45{border-color:hsla(240,4%,46%,.45)}.border-zinc-500\/5{border-color:hsla(240,4%,46%,.05)}.border-zinc-500\/50{border-color:hsla(240,4%,46%,.5)}.border-zinc-500\/55{border-color:hsla(240,4%,46%,.55)}.border-zinc-500\/60{border-color:hsla(240,4%,46%,.6)}.border-zinc-500\/65{border-color:hsla(240,4%,46%,.65)}.border-zinc-500\/70{border-color:hsla(240,4%,46%,.7)}.border-zinc-500\/75{border-color:hsla(240,4%,46%,.75)}.border-zinc-500\/80{border-color:hsla(240,4%,46%,.8)}.border-zinc-500\/85{border-color:hsla(240,4%,46%,.85)}.border-zinc-500\/90{border-color:hsla(240,4%,46%,.9)}.border-zinc-500\/95{border-color:hsla(240,4%,46%,.95)}.border-zinc-950{--tw-border-opacity:1;border-color:rgb(9 9 11/var(--tw-border-opacity,1))}.border-zinc-950\/0{border-color:rgba(9,9,11,0)}.border-zinc-950\/10{border-color:rgba(9,9,11,.1)}.border-zinc-950\/100{border-color:#09090b}.border-zinc-950\/15{border-color:rgba(9,9,11,.15)}.border-zinc-950\/20{border-color:rgba(9,9,11,.2)}.border-zinc-950\/25{border-color:rgba(9,9,11,.25)}.border-zinc-950\/30{border-color:rgba(9,9,11,.3)}.border-zinc-950\/35{border-color:rgba(9,9,11,.35)}.border-zinc-950\/40{border-color:rgba(9,9,11,.4)}.border-zinc-950\/45{border-color:rgba(9,9,11,.45)}.border-zinc-950\/5{border-color:rgba(9,9,11,.05)}.border-zinc-950\/50{border-color:rgba(9,9,11,.5)}.border-zinc-950\/55{border-color:rgba(9,9,11,.55)}.border-zinc-950\/60{border-color:rgba(9,9,11,.6)}.border-zinc-950\/65{border-color:rgba(9,9,11,.65)}.border-zinc-950\/70{border-color:rgba(9,9,11,.7)}.border-zinc-950\/75{border-color:rgba(9,9,11,.75)}.border-zinc-950\/80{border-color:rgba(9,9,11,.8)}.border-zinc-950\/85{border-color:rgba(9,9,11,.85)}.border-zinc-950\/90{border-color:rgba(9,9,11,.9)}.border-zinc-950\/95{border-color:rgba(9,9,11,.95)}.border-l-amber-100{--tw-border-opacity:1;border-left-color:rgb(254 243 199/var(--tw-border-opacity,1))}.border-l-amber-100\/0{border-left-color:hsla(48,96%,89%,0)}.border-l-amber-100\/10{border-left-color:hsla(48,96%,89%,.1)}.border-l-amber-100\/100{border-left-color:#fef3c7}.border-l-amber-100\/15{border-left-color:hsla(48,96%,89%,.15)}.border-l-amber-100\/20{border-left-color:hsla(48,96%,89%,.2)}.border-l-amber-100\/25{border-left-color:hsla(48,96%,89%,.25)}.border-l-amber-100\/30{border-left-color:hsla(48,96%,89%,.3)}.border-l-amber-100\/35{border-left-color:hsla(48,96%,89%,.35)}.border-l-amber-100\/40{border-left-color:hsla(48,96%,89%,.4)}.border-l-amber-100\/45{border-left-color:hsla(48,96%,89%,.45)}.border-l-amber-100\/5{border-left-color:hsla(48,96%,89%,.05)}.border-l-amber-100\/50{border-left-color:hsla(48,96%,89%,.5)}.border-l-amber-100\/55{border-left-color:hsla(48,96%,89%,.55)}.border-l-amber-100\/60{border-left-color:hsla(48,96%,89%,.6)}.border-l-amber-100\/65{border-left-color:hsla(48,96%,89%,.65)}.border-l-amber-100\/70{border-left-color:hsla(48,96%,89%,.7)}.border-l-amber-100\/75{border-left-color:hsla(48,96%,89%,.75)}.border-l-amber-100\/80{border-left-color:hsla(48,96%,89%,.8)}.border-l-amber-100\/85{border-left-color:hsla(48,96%,89%,.85)}.border-l-amber-100\/90{border-left-color:hsla(48,96%,89%,.9)}.border-l-amber-100\/95{border-left-color:hsla(48,96%,89%,.95)}.border-l-amber-200{--tw-border-opacity:1;border-left-color:rgb(253 230 138/var(--tw-border-opacity,1))}.border-l-amber-200\/0{border-left-color:hsla(48,97%,77%,0)}.border-l-amber-200\/10{border-left-color:hsla(48,97%,77%,.1)}.border-l-amber-200\/100{border-left-color:#fde68a}.border-l-amber-200\/15{border-left-color:hsla(48,97%,77%,.15)}.border-l-amber-200\/20{border-left-color:hsla(48,97%,77%,.2)}.border-l-amber-200\/25{border-left-color:hsla(48,97%,77%,.25)}.border-l-amber-200\/30{border-left-color:hsla(48,97%,77%,.3)}.border-l-amber-200\/35{border-left-color:hsla(48,97%,77%,.35)}.border-l-amber-200\/40{border-left-color:hsla(48,97%,77%,.4)}.border-l-amber-200\/45{border-left-color:hsla(48,97%,77%,.45)}.border-l-amber-200\/5{border-left-color:hsla(48,97%,77%,.05)}.border-l-amber-200\/50{border-left-color:hsla(48,97%,77%,.5)}.border-l-amber-200\/55{border-left-color:hsla(48,97%,77%,.55)}.border-l-amber-200\/60{border-left-color:hsla(48,97%,77%,.6)}.border-l-amber-200\/65{border-left-color:hsla(48,97%,77%,.65)}.border-l-amber-200\/70{border-left-color:hsla(48,97%,77%,.7)}.border-l-amber-200\/75{border-left-color:hsla(48,97%,77%,.75)}.border-l-amber-200\/80{border-left-color:hsla(48,97%,77%,.8)}.border-l-amber-200\/85{border-left-color:hsla(48,97%,77%,.85)}.border-l-amber-200\/90{border-left-color:hsla(48,97%,77%,.9)}.border-l-amber-200\/95{border-left-color:hsla(48,97%,77%,.95)}.border-l-amber-400{--tw-border-opacity:1;border-left-color:rgb(251 191 36/var(--tw-border-opacity,1))}.border-l-amber-400\/0{border-left-color:rgba(251,191,36,0)}.border-l-amber-400\/10{border-left-color:rgba(251,191,36,.1)}.border-l-amber-400\/100{border-left-color:#fbbf24}.border-l-amber-400\/15{border-left-color:rgba(251,191,36,.15)}.border-l-amber-400\/20{border-left-color:rgba(251,191,36,.2)}.border-l-amber-400\/25{border-left-color:rgba(251,191,36,.25)}.border-l-amber-400\/30{border-left-color:rgba(251,191,36,.3)}.border-l-amber-400\/35{border-left-color:rgba(251,191,36,.35)}.border-l-amber-400\/40{border-left-color:rgba(251,191,36,.4)}.border-l-amber-400\/45{border-left-color:rgba(251,191,36,.45)}.border-l-amber-400\/5{border-left-color:rgba(251,191,36,.05)}.border-l-amber-400\/50{border-left-color:rgba(251,191,36,.5)}.border-l-amber-400\/55{border-left-color:rgba(251,191,36,.55)}.border-l-amber-400\/60{border-left-color:rgba(251,191,36,.6)}.border-l-amber-400\/65{border-left-color:rgba(251,191,36,.65)}.border-l-amber-400\/70{border-left-color:rgba(251,191,36,.7)}.border-l-amber-400\/75{border-left-color:rgba(251,191,36,.75)}.border-l-amber-400\/80{border-left-color:rgba(251,191,36,.8)}.border-l-amber-400\/85{border-left-color:rgba(251,191,36,.85)}.border-l-amber-400\/90{border-left-color:rgba(251,191,36,.9)}.border-l-amber-400\/95{border-left-color:rgba(251,191,36,.95)}.border-l-amber-50{--tw-border-opacity:1;border-left-color:rgb(255 251 235/var(--tw-border-opacity,1))}.border-l-amber-50\/0{border-left-color:rgba(255,251,235,0)}.border-l-amber-50\/10{border-left-color:rgba(255,251,235,.1)}.border-l-amber-50\/100{border-left-color:#fffbeb}.border-l-amber-50\/15{border-left-color:rgba(255,251,235,.15)}.border-l-amber-50\/20{border-left-color:rgba(255,251,235,.2)}.border-l-amber-50\/25{border-left-color:rgba(255,251,235,.25)}.border-l-amber-50\/30{border-left-color:rgba(255,251,235,.3)}.border-l-amber-50\/35{border-left-color:rgba(255,251,235,.35)}.border-l-amber-50\/40{border-left-color:rgba(255,251,235,.4)}.border-l-amber-50\/45{border-left-color:rgba(255,251,235,.45)}.border-l-amber-50\/5{border-left-color:rgba(255,251,235,.05)}.border-l-amber-50\/50{border-left-color:rgba(255,251,235,.5)}.border-l-amber-50\/55{border-left-color:rgba(255,251,235,.55)}.border-l-amber-50\/60{border-left-color:rgba(255,251,235,.6)}.border-l-amber-50\/65{border-left-color:rgba(255,251,235,.65)}.border-l-amber-50\/70{border-left-color:rgba(255,251,235,.7)}.border-l-amber-50\/75{border-left-color:rgba(255,251,235,.75)}.border-l-amber-50\/80{border-left-color:rgba(255,251,235,.8)}.border-l-amber-50\/85{border-left-color:rgba(255,251,235,.85)}.border-l-amber-50\/90{border-left-color:rgba(255,251,235,.9)}.border-l-amber-50\/95{border-left-color:rgba(255,251,235,.95)}.border-l-amber-500{--tw-border-opacity:1;border-left-color:rgb(245 158 11/var(--tw-border-opacity,1))}.border-l-amber-500\/0{border-left-color:rgba(245,158,11,0)}.border-l-amber-500\/10{border-left-color:rgba(245,158,11,.1)}.border-l-amber-500\/100{border-left-color:#f59e0b}.border-l-amber-500\/15{border-left-color:rgba(245,158,11,.15)}.border-l-amber-500\/20{border-left-color:rgba(245,158,11,.2)}.border-l-amber-500\/25{border-left-color:rgba(245,158,11,.25)}.border-l-amber-500\/30{border-left-color:rgba(245,158,11,.3)}.border-l-amber-500\/35{border-left-color:rgba(245,158,11,.35)}.border-l-amber-500\/40{border-left-color:rgba(245,158,11,.4)}.border-l-amber-500\/45{border-left-color:rgba(245,158,11,.45)}.border-l-amber-500\/5{border-left-color:rgba(245,158,11,.05)}.border-l-amber-500\/50{border-left-color:rgba(245,158,11,.5)}.border-l-amber-500\/55{border-left-color:rgba(245,158,11,.55)}.border-l-amber-500\/60{border-left-color:rgba(245,158,11,.6)}.border-l-amber-500\/65{border-left-color:rgba(245,158,11,.65)}.border-l-amber-500\/70{border-left-color:rgba(245,158,11,.7)}.border-l-amber-500\/75{border-left-color:rgba(245,158,11,.75)}.border-l-amber-500\/80{border-left-color:rgba(245,158,11,.8)}.border-l-amber-500\/85{border-left-color:rgba(245,158,11,.85)}.border-l-amber-500\/90{border-left-color:rgba(245,158,11,.9)}.border-l-amber-500\/95{border-left-color:rgba(245,158,11,.95)}.border-l-amber-950{--tw-border-opacity:1;border-left-color:rgb(69 26 3/var(--tw-border-opacity,1))}.border-l-amber-950\/0{border-left-color:rgba(69,26,3,0)}.border-l-amber-950\/10{border-left-color:rgba(69,26,3,.1)}.border-l-amber-950\/100{border-left-color:#451a03}.border-l-amber-950\/15{border-left-color:rgba(69,26,3,.15)}.border-l-amber-950\/20{border-left-color:rgba(69,26,3,.2)}.border-l-amber-950\/25{border-left-color:rgba(69,26,3,.25)}.border-l-amber-950\/30{border-left-color:rgba(69,26,3,.3)}.border-l-amber-950\/35{border-left-color:rgba(69,26,3,.35)}.border-l-amber-950\/40{border-left-color:rgba(69,26,3,.4)}.border-l-amber-950\/45{border-left-color:rgba(69,26,3,.45)}.border-l-amber-950\/5{border-left-color:rgba(69,26,3,.05)}.border-l-amber-950\/50{border-left-color:rgba(69,26,3,.5)}.border-l-amber-950\/55{border-left-color:rgba(69,26,3,.55)}.border-l-amber-950\/60{border-left-color:rgba(69,26,3,.6)}.border-l-amber-950\/65{border-left-color:rgba(69,26,3,.65)}.border-l-amber-950\/70{border-left-color:rgba(69,26,3,.7)}.border-l-amber-950\/75{border-left-color:rgba(69,26,3,.75)}.border-l-amber-950\/80{border-left-color:rgba(69,26,3,.8)}.border-l-amber-950\/85{border-left-color:rgba(69,26,3,.85)}.border-l-amber-950\/90{border-left-color:rgba(69,26,3,.9)}.border-l-amber-950\/95{border-left-color:rgba(69,26,3,.95)}.border-l-blue-100{--tw-border-opacity:1;border-left-color:rgb(219 234 254/var(--tw-border-opacity,1))}.border-l-blue-100\/0{border-left-color:rgba(219,234,254,0)}.border-l-blue-100\/10{border-left-color:rgba(219,234,254,.1)}.border-l-blue-100\/100{border-left-color:#dbeafe}.border-l-blue-100\/15{border-left-color:rgba(219,234,254,.15)}.border-l-blue-100\/20{border-left-color:rgba(219,234,254,.2)}.border-l-blue-100\/25{border-left-color:rgba(219,234,254,.25)}.border-l-blue-100\/30{border-left-color:rgba(219,234,254,.3)}.border-l-blue-100\/35{border-left-color:rgba(219,234,254,.35)}.border-l-blue-100\/40{border-left-color:rgba(219,234,254,.4)}.border-l-blue-100\/45{border-left-color:rgba(219,234,254,.45)}.border-l-blue-100\/5{border-left-color:rgba(219,234,254,.05)}.border-l-blue-100\/50{border-left-color:rgba(219,234,254,.5)}.border-l-blue-100\/55{border-left-color:rgba(219,234,254,.55)}.border-l-blue-100\/60{border-left-color:rgba(219,234,254,.6)}.border-l-blue-100\/65{border-left-color:rgba(219,234,254,.65)}.border-l-blue-100\/70{border-left-color:rgba(219,234,254,.7)}.border-l-blue-100\/75{border-left-color:rgba(219,234,254,.75)}.border-l-blue-100\/80{border-left-color:rgba(219,234,254,.8)}.border-l-blue-100\/85{border-left-color:rgba(219,234,254,.85)}.border-l-blue-100\/90{border-left-color:rgba(219,234,254,.9)}.border-l-blue-100\/95{border-left-color:rgba(219,234,254,.95)}.border-l-blue-200{--tw-border-opacity:1;border-left-color:rgb(191 219 254/var(--tw-border-opacity,1))}.border-l-blue-200\/0{border-left-color:rgba(191,219,254,0)}.border-l-blue-200\/10{border-left-color:rgba(191,219,254,.1)}.border-l-blue-200\/100{border-left-color:#bfdbfe}.border-l-blue-200\/15{border-left-color:rgba(191,219,254,.15)}.border-l-blue-200\/20{border-left-color:rgba(191,219,254,.2)}.border-l-blue-200\/25{border-left-color:rgba(191,219,254,.25)}.border-l-blue-200\/30{border-left-color:rgba(191,219,254,.3)}.border-l-blue-200\/35{border-left-color:rgba(191,219,254,.35)}.border-l-blue-200\/40{border-left-color:rgba(191,219,254,.4)}.border-l-blue-200\/45{border-left-color:rgba(191,219,254,.45)}.border-l-blue-200\/5{border-left-color:rgba(191,219,254,.05)}.border-l-blue-200\/50{border-left-color:rgba(191,219,254,.5)}.border-l-blue-200\/55{border-left-color:rgba(191,219,254,.55)}.border-l-blue-200\/60{border-left-color:rgba(191,219,254,.6)}.border-l-blue-200\/65{border-left-color:rgba(191,219,254,.65)}.border-l-blue-200\/70{border-left-color:rgba(191,219,254,.7)}.border-l-blue-200\/75{border-left-color:rgba(191,219,254,.75)}.border-l-blue-200\/80{border-left-color:rgba(191,219,254,.8)}.border-l-blue-200\/85{border-left-color:rgba(191,219,254,.85)}.border-l-blue-200\/90{border-left-color:rgba(191,219,254,.9)}.border-l-blue-200\/95{border-left-color:rgba(191,219,254,.95)}.border-l-blue-400{--tw-border-opacity:1;border-left-color:rgb(96 165 250/var(--tw-border-opacity,1))}.border-l-blue-400\/0{border-left-color:rgba(96,165,250,0)}.border-l-blue-400\/10{border-left-color:rgba(96,165,250,.1)}.border-l-blue-400\/100{border-left-color:#60a5fa}.border-l-blue-400\/15{border-left-color:rgba(96,165,250,.15)}.border-l-blue-400\/20{border-left-color:rgba(96,165,250,.2)}.border-l-blue-400\/25{border-left-color:rgba(96,165,250,.25)}.border-l-blue-400\/30{border-left-color:rgba(96,165,250,.3)}.border-l-blue-400\/35{border-left-color:rgba(96,165,250,.35)}.border-l-blue-400\/40{border-left-color:rgba(96,165,250,.4)}.border-l-blue-400\/45{border-left-color:rgba(96,165,250,.45)}.border-l-blue-400\/5{border-left-color:rgba(96,165,250,.05)}.border-l-blue-400\/50{border-left-color:rgba(96,165,250,.5)}.border-l-blue-400\/55{border-left-color:rgba(96,165,250,.55)}.border-l-blue-400\/60{border-left-color:rgba(96,165,250,.6)}.border-l-blue-400\/65{border-left-color:rgba(96,165,250,.65)}.border-l-blue-400\/70{border-left-color:rgba(96,165,250,.7)}.border-l-blue-400\/75{border-left-color:rgba(96,165,250,.75)}.border-l-blue-400\/80{border-left-color:rgba(96,165,250,.8)}.border-l-blue-400\/85{border-left-color:rgba(96,165,250,.85)}.border-l-blue-400\/90{border-left-color:rgba(96,165,250,.9)}.border-l-blue-400\/95{border-left-color:rgba(96,165,250,.95)}.border-l-blue-50{--tw-border-opacity:1;border-left-color:rgb(239 246 255/var(--tw-border-opacity,1))}.border-l-blue-50\/0{border-left-color:rgba(239,246,255,0)}.border-l-blue-50\/10{border-left-color:rgba(239,246,255,.1)}.border-l-blue-50\/100{border-left-color:#eff6ff}.border-l-blue-50\/15{border-left-color:rgba(239,246,255,.15)}.border-l-blue-50\/20{border-left-color:rgba(239,246,255,.2)}.border-l-blue-50\/25{border-left-color:rgba(239,246,255,.25)}.border-l-blue-50\/30{border-left-color:rgba(239,246,255,.3)}.border-l-blue-50\/35{border-left-color:rgba(239,246,255,.35)}.border-l-blue-50\/40{border-left-color:rgba(239,246,255,.4)}.border-l-blue-50\/45{border-left-color:rgba(239,246,255,.45)}.border-l-blue-50\/5{border-left-color:rgba(239,246,255,.05)}.border-l-blue-50\/50{border-left-color:rgba(239,246,255,.5)}.border-l-blue-50\/55{border-left-color:rgba(239,246,255,.55)}.border-l-blue-50\/60{border-left-color:rgba(239,246,255,.6)}.border-l-blue-50\/65{border-left-color:rgba(239,246,255,.65)}.border-l-blue-50\/70{border-left-color:rgba(239,246,255,.7)}.border-l-blue-50\/75{border-left-color:rgba(239,246,255,.75)}.border-l-blue-50\/80{border-left-color:rgba(239,246,255,.8)}.border-l-blue-50\/85{border-left-color:rgba(239,246,255,.85)}.border-l-blue-50\/90{border-left-color:rgba(239,246,255,.9)}.border-l-blue-50\/95{border-left-color:rgba(239,246,255,.95)}.border-l-blue-500{--tw-border-opacity:1;border-left-color:rgb(59 130 246/var(--tw-border-opacity,1))}.border-l-blue-500\/0{border-left-color:rgba(59,130,246,0)}.border-l-blue-500\/10{border-left-color:rgba(59,130,246,.1)}.border-l-blue-500\/100{border-left-color:#3b82f6}.border-l-blue-500\/15{border-left-color:rgba(59,130,246,.15)}.border-l-blue-500\/20{border-left-color:rgba(59,130,246,.2)}.border-l-blue-500\/25{border-left-color:rgba(59,130,246,.25)}.border-l-blue-500\/30{border-left-color:rgba(59,130,246,.3)}.border-l-blue-500\/35{border-left-color:rgba(59,130,246,.35)}.border-l-blue-500\/40{border-left-color:rgba(59,130,246,.4)}.border-l-blue-500\/45{border-left-color:rgba(59,130,246,.45)}.border-l-blue-500\/5{border-left-color:rgba(59,130,246,.05)}.border-l-blue-500\/50{border-left-color:rgba(59,130,246,.5)}.border-l-blue-500\/55{border-left-color:rgba(59,130,246,.55)}.border-l-blue-500\/60{border-left-color:rgba(59,130,246,.6)}.border-l-blue-500\/65{border-left-color:rgba(59,130,246,.65)}.border-l-blue-500\/70{border-left-color:rgba(59,130,246,.7)}.border-l-blue-500\/75{border-left-color:rgba(59,130,246,.75)}.border-l-blue-500\/80{border-left-color:rgba(59,130,246,.8)}.border-l-blue-500\/85{border-left-color:rgba(59,130,246,.85)}.border-l-blue-500\/90{border-left-color:rgba(59,130,246,.9)}.border-l-blue-500\/95{border-left-color:rgba(59,130,246,.95)}.border-l-blue-950{--tw-border-opacity:1;border-left-color:rgb(23 37 84/var(--tw-border-opacity,1))}.border-l-blue-950\/0{border-left-color:rgba(23,37,84,0)}.border-l-blue-950\/10{border-left-color:rgba(23,37,84,.1)}.border-l-blue-950\/100{border-left-color:#172554}.border-l-blue-950\/15{border-left-color:rgba(23,37,84,.15)}.border-l-blue-950\/20{border-left-color:rgba(23,37,84,.2)}.border-l-blue-950\/25{border-left-color:rgba(23,37,84,.25)}.border-l-blue-950\/30{border-left-color:rgba(23,37,84,.3)}.border-l-blue-950\/35{border-left-color:rgba(23,37,84,.35)}.border-l-blue-950\/40{border-left-color:rgba(23,37,84,.4)}.border-l-blue-950\/45{border-left-color:rgba(23,37,84,.45)}.border-l-blue-950\/5{border-left-color:rgba(23,37,84,.05)}.border-l-blue-950\/50{border-left-color:rgba(23,37,84,.5)}.border-l-blue-950\/55{border-left-color:rgba(23,37,84,.55)}.border-l-blue-950\/60{border-left-color:rgba(23,37,84,.6)}.border-l-blue-950\/65{border-left-color:rgba(23,37,84,.65)}.border-l-blue-950\/70{border-left-color:rgba(23,37,84,.7)}.border-l-blue-950\/75{border-left-color:rgba(23,37,84,.75)}.border-l-blue-950\/80{border-left-color:rgba(23,37,84,.8)}.border-l-blue-950\/85{border-left-color:rgba(23,37,84,.85)}.border-l-blue-950\/90{border-left-color:rgba(23,37,84,.9)}.border-l-blue-950\/95{border-left-color:rgba(23,37,84,.95)}.border-l-cyan-100{--tw-border-opacity:1;border-left-color:rgb(207 250 254/var(--tw-border-opacity,1))}.border-l-cyan-100\/0{border-left-color:rgba(207,250,254,0)}.border-l-cyan-100\/10{border-left-color:rgba(207,250,254,.1)}.border-l-cyan-100\/100{border-left-color:#cffafe}.border-l-cyan-100\/15{border-left-color:rgba(207,250,254,.15)}.border-l-cyan-100\/20{border-left-color:rgba(207,250,254,.2)}.border-l-cyan-100\/25{border-left-color:rgba(207,250,254,.25)}.border-l-cyan-100\/30{border-left-color:rgba(207,250,254,.3)}.border-l-cyan-100\/35{border-left-color:rgba(207,250,254,.35)}.border-l-cyan-100\/40{border-left-color:rgba(207,250,254,.4)}.border-l-cyan-100\/45{border-left-color:rgba(207,250,254,.45)}.border-l-cyan-100\/5{border-left-color:rgba(207,250,254,.05)}.border-l-cyan-100\/50{border-left-color:rgba(207,250,254,.5)}.border-l-cyan-100\/55{border-left-color:rgba(207,250,254,.55)}.border-l-cyan-100\/60{border-left-color:rgba(207,250,254,.6)}.border-l-cyan-100\/65{border-left-color:rgba(207,250,254,.65)}.border-l-cyan-100\/70{border-left-color:rgba(207,250,254,.7)}.border-l-cyan-100\/75{border-left-color:rgba(207,250,254,.75)}.border-l-cyan-100\/80{border-left-color:rgba(207,250,254,.8)}.border-l-cyan-100\/85{border-left-color:rgba(207,250,254,.85)}.border-l-cyan-100\/90{border-left-color:rgba(207,250,254,.9)}.border-l-cyan-100\/95{border-left-color:rgba(207,250,254,.95)}.border-l-cyan-200{--tw-border-opacity:1;border-left-color:rgb(165 243 252/var(--tw-border-opacity,1))}.border-l-cyan-200\/0{border-left-color:rgba(165,243,252,0)}.border-l-cyan-200\/10{border-left-color:rgba(165,243,252,.1)}.border-l-cyan-200\/100{border-left-color:#a5f3fc}.border-l-cyan-200\/15{border-left-color:rgba(165,243,252,.15)}.border-l-cyan-200\/20{border-left-color:rgba(165,243,252,.2)}.border-l-cyan-200\/25{border-left-color:rgba(165,243,252,.25)}.border-l-cyan-200\/30{border-left-color:rgba(165,243,252,.3)}.border-l-cyan-200\/35{border-left-color:rgba(165,243,252,.35)}.border-l-cyan-200\/40{border-left-color:rgba(165,243,252,.4)}.border-l-cyan-200\/45{border-left-color:rgba(165,243,252,.45)}.border-l-cyan-200\/5{border-left-color:rgba(165,243,252,.05)}.border-l-cyan-200\/50{border-left-color:rgba(165,243,252,.5)}.border-l-cyan-200\/55{border-left-color:rgba(165,243,252,.55)}.border-l-cyan-200\/60{border-left-color:rgba(165,243,252,.6)}.border-l-cyan-200\/65{border-left-color:rgba(165,243,252,.65)}.border-l-cyan-200\/70{border-left-color:rgba(165,243,252,.7)}.border-l-cyan-200\/75{border-left-color:rgba(165,243,252,.75)}.border-l-cyan-200\/80{border-left-color:rgba(165,243,252,.8)}.border-l-cyan-200\/85{border-left-color:rgba(165,243,252,.85)}.border-l-cyan-200\/90{border-left-color:rgba(165,243,252,.9)}.border-l-cyan-200\/95{border-left-color:rgba(165,243,252,.95)}.border-l-cyan-400{--tw-border-opacity:1;border-left-color:rgb(34 211 238/var(--tw-border-opacity,1))}.border-l-cyan-400\/0{border-left-color:rgba(34,211,238,0)}.border-l-cyan-400\/10{border-left-color:rgba(34,211,238,.1)}.border-l-cyan-400\/100{border-left-color:#22d3ee}.border-l-cyan-400\/15{border-left-color:rgba(34,211,238,.15)}.border-l-cyan-400\/20{border-left-color:rgba(34,211,238,.2)}.border-l-cyan-400\/25{border-left-color:rgba(34,211,238,.25)}.border-l-cyan-400\/30{border-left-color:rgba(34,211,238,.3)}.border-l-cyan-400\/35{border-left-color:rgba(34,211,238,.35)}.border-l-cyan-400\/40{border-left-color:rgba(34,211,238,.4)}.border-l-cyan-400\/45{border-left-color:rgba(34,211,238,.45)}.border-l-cyan-400\/5{border-left-color:rgba(34,211,238,.05)}.border-l-cyan-400\/50{border-left-color:rgba(34,211,238,.5)}.border-l-cyan-400\/55{border-left-color:rgba(34,211,238,.55)}.border-l-cyan-400\/60{border-left-color:rgba(34,211,238,.6)}.border-l-cyan-400\/65{border-left-color:rgba(34,211,238,.65)}.border-l-cyan-400\/70{border-left-color:rgba(34,211,238,.7)}.border-l-cyan-400\/75{border-left-color:rgba(34,211,238,.75)}.border-l-cyan-400\/80{border-left-color:rgba(34,211,238,.8)}.border-l-cyan-400\/85{border-left-color:rgba(34,211,238,.85)}.border-l-cyan-400\/90{border-left-color:rgba(34,211,238,.9)}.border-l-cyan-400\/95{border-left-color:rgba(34,211,238,.95)}.border-l-cyan-50{--tw-border-opacity:1;border-left-color:rgb(236 254 255/var(--tw-border-opacity,1))}.border-l-cyan-50\/0{border-left-color:rgba(236,254,255,0)}.border-l-cyan-50\/10{border-left-color:rgba(236,254,255,.1)}.border-l-cyan-50\/100{border-left-color:#ecfeff}.border-l-cyan-50\/15{border-left-color:rgba(236,254,255,.15)}.border-l-cyan-50\/20{border-left-color:rgba(236,254,255,.2)}.border-l-cyan-50\/25{border-left-color:rgba(236,254,255,.25)}.border-l-cyan-50\/30{border-left-color:rgba(236,254,255,.3)}.border-l-cyan-50\/35{border-left-color:rgba(236,254,255,.35)}.border-l-cyan-50\/40{border-left-color:rgba(236,254,255,.4)}.border-l-cyan-50\/45{border-left-color:rgba(236,254,255,.45)}.border-l-cyan-50\/5{border-left-color:rgba(236,254,255,.05)}.border-l-cyan-50\/50{border-left-color:rgba(236,254,255,.5)}.border-l-cyan-50\/55{border-left-color:rgba(236,254,255,.55)}.border-l-cyan-50\/60{border-left-color:rgba(236,254,255,.6)}.border-l-cyan-50\/65{border-left-color:rgba(236,254,255,.65)}.border-l-cyan-50\/70{border-left-color:rgba(236,254,255,.7)}.border-l-cyan-50\/75{border-left-color:rgba(236,254,255,.75)}.border-l-cyan-50\/80{border-left-color:rgba(236,254,255,.8)}.border-l-cyan-50\/85{border-left-color:rgba(236,254,255,.85)}.border-l-cyan-50\/90{border-left-color:rgba(236,254,255,.9)}.border-l-cyan-50\/95{border-left-color:rgba(236,254,255,.95)}.border-l-cyan-500{--tw-border-opacity:1;border-left-color:rgb(6 182 212/var(--tw-border-opacity,1))}.border-l-cyan-500\/0{border-left-color:rgba(6,182,212,0)}.border-l-cyan-500\/10{border-left-color:rgba(6,182,212,.1)}.border-l-cyan-500\/100{border-left-color:#06b6d4}.border-l-cyan-500\/15{border-left-color:rgba(6,182,212,.15)}.border-l-cyan-500\/20{border-left-color:rgba(6,182,212,.2)}.border-l-cyan-500\/25{border-left-color:rgba(6,182,212,.25)}.border-l-cyan-500\/30{border-left-color:rgba(6,182,212,.3)}.border-l-cyan-500\/35{border-left-color:rgba(6,182,212,.35)}.border-l-cyan-500\/40{border-left-color:rgba(6,182,212,.4)}.border-l-cyan-500\/45{border-left-color:rgba(6,182,212,.45)}.border-l-cyan-500\/5{border-left-color:rgba(6,182,212,.05)}.border-l-cyan-500\/50{border-left-color:rgba(6,182,212,.5)}.border-l-cyan-500\/55{border-left-color:rgba(6,182,212,.55)}.border-l-cyan-500\/60{border-left-color:rgba(6,182,212,.6)}.border-l-cyan-500\/65{border-left-color:rgba(6,182,212,.65)}.border-l-cyan-500\/70{border-left-color:rgba(6,182,212,.7)}.border-l-cyan-500\/75{border-left-color:rgba(6,182,212,.75)}.border-l-cyan-500\/80{border-left-color:rgba(6,182,212,.8)}.border-l-cyan-500\/85{border-left-color:rgba(6,182,212,.85)}.border-l-cyan-500\/90{border-left-color:rgba(6,182,212,.9)}.border-l-cyan-500\/95{border-left-color:rgba(6,182,212,.95)}.border-l-cyan-950{--tw-border-opacity:1;border-left-color:rgb(8 51 68/var(--tw-border-opacity,1))}.border-l-cyan-950\/0{border-left-color:rgba(8,51,68,0)}.border-l-cyan-950\/10{border-left-color:rgba(8,51,68,.1)}.border-l-cyan-950\/100{border-left-color:#083344}.border-l-cyan-950\/15{border-left-color:rgba(8,51,68,.15)}.border-l-cyan-950\/20{border-left-color:rgba(8,51,68,.2)}.border-l-cyan-950\/25{border-left-color:rgba(8,51,68,.25)}.border-l-cyan-950\/30{border-left-color:rgba(8,51,68,.3)}.border-l-cyan-950\/35{border-left-color:rgba(8,51,68,.35)}.border-l-cyan-950\/40{border-left-color:rgba(8,51,68,.4)}.border-l-cyan-950\/45{border-left-color:rgba(8,51,68,.45)}.border-l-cyan-950\/5{border-left-color:rgba(8,51,68,.05)}.border-l-cyan-950\/50{border-left-color:rgba(8,51,68,.5)}.border-l-cyan-950\/55{border-left-color:rgba(8,51,68,.55)}.border-l-cyan-950\/60{border-left-color:rgba(8,51,68,.6)}.border-l-cyan-950\/65{border-left-color:rgba(8,51,68,.65)}.border-l-cyan-950\/70{border-left-color:rgba(8,51,68,.7)}.border-l-cyan-950\/75{border-left-color:rgba(8,51,68,.75)}.border-l-cyan-950\/80{border-left-color:rgba(8,51,68,.8)}.border-l-cyan-950\/85{border-left-color:rgba(8,51,68,.85)}.border-l-cyan-950\/90{border-left-color:rgba(8,51,68,.9)}.border-l-cyan-950\/95{border-left-color:rgba(8,51,68,.95)}.border-l-emerald-100{--tw-border-opacity:1;border-left-color:rgb(209 250 229/var(--tw-border-opacity,1))}.border-l-emerald-100\/0{border-left-color:rgba(209,250,229,0)}.border-l-emerald-100\/10{border-left-color:rgba(209,250,229,.1)}.border-l-emerald-100\/100{border-left-color:#d1fae5}.border-l-emerald-100\/15{border-left-color:rgba(209,250,229,.15)}.border-l-emerald-100\/20{border-left-color:rgba(209,250,229,.2)}.border-l-emerald-100\/25{border-left-color:rgba(209,250,229,.25)}.border-l-emerald-100\/30{border-left-color:rgba(209,250,229,.3)}.border-l-emerald-100\/35{border-left-color:rgba(209,250,229,.35)}.border-l-emerald-100\/40{border-left-color:rgba(209,250,229,.4)}.border-l-emerald-100\/45{border-left-color:rgba(209,250,229,.45)}.border-l-emerald-100\/5{border-left-color:rgba(209,250,229,.05)}.border-l-emerald-100\/50{border-left-color:rgba(209,250,229,.5)}.border-l-emerald-100\/55{border-left-color:rgba(209,250,229,.55)}.border-l-emerald-100\/60{border-left-color:rgba(209,250,229,.6)}.border-l-emerald-100\/65{border-left-color:rgba(209,250,229,.65)}.border-l-emerald-100\/70{border-left-color:rgba(209,250,229,.7)}.border-l-emerald-100\/75{border-left-color:rgba(209,250,229,.75)}.border-l-emerald-100\/80{border-left-color:rgba(209,250,229,.8)}.border-l-emerald-100\/85{border-left-color:rgba(209,250,229,.85)}.border-l-emerald-100\/90{border-left-color:rgba(209,250,229,.9)}.border-l-emerald-100\/95{border-left-color:rgba(209,250,229,.95)}.border-l-emerald-200{--tw-border-opacity:1;border-left-color:rgb(167 243 208/var(--tw-border-opacity,1))}.border-l-emerald-200\/0{border-left-color:rgba(167,243,208,0)}.border-l-emerald-200\/10{border-left-color:rgba(167,243,208,.1)}.border-l-emerald-200\/100{border-left-color:#a7f3d0}.border-l-emerald-200\/15{border-left-color:rgba(167,243,208,.15)}.border-l-emerald-200\/20{border-left-color:rgba(167,243,208,.2)}.border-l-emerald-200\/25{border-left-color:rgba(167,243,208,.25)}.border-l-emerald-200\/30{border-left-color:rgba(167,243,208,.3)}.border-l-emerald-200\/35{border-left-color:rgba(167,243,208,.35)}.border-l-emerald-200\/40{border-left-color:rgba(167,243,208,.4)}.border-l-emerald-200\/45{border-left-color:rgba(167,243,208,.45)}.border-l-emerald-200\/5{border-left-color:rgba(167,243,208,.05)}.border-l-emerald-200\/50{border-left-color:rgba(167,243,208,.5)}.border-l-emerald-200\/55{border-left-color:rgba(167,243,208,.55)}.border-l-emerald-200\/60{border-left-color:rgba(167,243,208,.6)}.border-l-emerald-200\/65{border-left-color:rgba(167,243,208,.65)}.border-l-emerald-200\/70{border-left-color:rgba(167,243,208,.7)}.border-l-emerald-200\/75{border-left-color:rgba(167,243,208,.75)}.border-l-emerald-200\/80{border-left-color:rgba(167,243,208,.8)}.border-l-emerald-200\/85{border-left-color:rgba(167,243,208,.85)}.border-l-emerald-200\/90{border-left-color:rgba(167,243,208,.9)}.border-l-emerald-200\/95{border-left-color:rgba(167,243,208,.95)}.border-l-emerald-400{--tw-border-opacity:1;border-left-color:rgb(52 211 153/var(--tw-border-opacity,1))}.border-l-emerald-400\/0{border-left-color:rgba(52,211,153,0)}.border-l-emerald-400\/10{border-left-color:rgba(52,211,153,.1)}.border-l-emerald-400\/100{border-left-color:#34d399}.border-l-emerald-400\/15{border-left-color:rgba(52,211,153,.15)}.border-l-emerald-400\/20{border-left-color:rgba(52,211,153,.2)}.border-l-emerald-400\/25{border-left-color:rgba(52,211,153,.25)}.border-l-emerald-400\/30{border-left-color:rgba(52,211,153,.3)}.border-l-emerald-400\/35{border-left-color:rgba(52,211,153,.35)}.border-l-emerald-400\/40{border-left-color:rgba(52,211,153,.4)}.border-l-emerald-400\/45{border-left-color:rgba(52,211,153,.45)}.border-l-emerald-400\/5{border-left-color:rgba(52,211,153,.05)}.border-l-emerald-400\/50{border-left-color:rgba(52,211,153,.5)}.border-l-emerald-400\/55{border-left-color:rgba(52,211,153,.55)}.border-l-emerald-400\/60{border-left-color:rgba(52,211,153,.6)}.border-l-emerald-400\/65{border-left-color:rgba(52,211,153,.65)}.border-l-emerald-400\/70{border-left-color:rgba(52,211,153,.7)}.border-l-emerald-400\/75{border-left-color:rgba(52,211,153,.75)}.border-l-emerald-400\/80{border-left-color:rgba(52,211,153,.8)}.border-l-emerald-400\/85{border-left-color:rgba(52,211,153,.85)}.border-l-emerald-400\/90{border-left-color:rgba(52,211,153,.9)}.border-l-emerald-400\/95{border-left-color:rgba(52,211,153,.95)}.border-l-emerald-50{--tw-border-opacity:1;border-left-color:rgb(236 253 245/var(--tw-border-opacity,1))}.border-l-emerald-50\/0{border-left-color:rgba(236,253,245,0)}.border-l-emerald-50\/10{border-left-color:rgba(236,253,245,.1)}.border-l-emerald-50\/100{border-left-color:#ecfdf5}.border-l-emerald-50\/15{border-left-color:rgba(236,253,245,.15)}.border-l-emerald-50\/20{border-left-color:rgba(236,253,245,.2)}.border-l-emerald-50\/25{border-left-color:rgba(236,253,245,.25)}.border-l-emerald-50\/30{border-left-color:rgba(236,253,245,.3)}.border-l-emerald-50\/35{border-left-color:rgba(236,253,245,.35)}.border-l-emerald-50\/40{border-left-color:rgba(236,253,245,.4)}.border-l-emerald-50\/45{border-left-color:rgba(236,253,245,.45)}.border-l-emerald-50\/5{border-left-color:rgba(236,253,245,.05)}.border-l-emerald-50\/50{border-left-color:rgba(236,253,245,.5)}.border-l-emerald-50\/55{border-left-color:rgba(236,253,245,.55)}.border-l-emerald-50\/60{border-left-color:rgba(236,253,245,.6)}.border-l-emerald-50\/65{border-left-color:rgba(236,253,245,.65)}.border-l-emerald-50\/70{border-left-color:rgba(236,253,245,.7)}.border-l-emerald-50\/75{border-left-color:rgba(236,253,245,.75)}.border-l-emerald-50\/80{border-left-color:rgba(236,253,245,.8)}.border-l-emerald-50\/85{border-left-color:rgba(236,253,245,.85)}.border-l-emerald-50\/90{border-left-color:rgba(236,253,245,.9)}.border-l-emerald-50\/95{border-left-color:rgba(236,253,245,.95)}.border-l-emerald-500{--tw-border-opacity:1;border-left-color:rgb(16 185 129/var(--tw-border-opacity,1))}.border-l-emerald-500\/0{border-left-color:rgba(16,185,129,0)}.border-l-emerald-500\/10{border-left-color:rgba(16,185,129,.1)}.border-l-emerald-500\/100{border-left-color:#10b981}.border-l-emerald-500\/15{border-left-color:rgba(16,185,129,.15)}.border-l-emerald-500\/20{border-left-color:rgba(16,185,129,.2)}.border-l-emerald-500\/25{border-left-color:rgba(16,185,129,.25)}.border-l-emerald-500\/30{border-left-color:rgba(16,185,129,.3)}.border-l-emerald-500\/35{border-left-color:rgba(16,185,129,.35)}.border-l-emerald-500\/40{border-left-color:rgba(16,185,129,.4)}.border-l-emerald-500\/45{border-left-color:rgba(16,185,129,.45)}.border-l-emerald-500\/5{border-left-color:rgba(16,185,129,.05)}.border-l-emerald-500\/50{border-left-color:rgba(16,185,129,.5)}.border-l-emerald-500\/55{border-left-color:rgba(16,185,129,.55)}.border-l-emerald-500\/60{border-left-color:rgba(16,185,129,.6)}.border-l-emerald-500\/65{border-left-color:rgba(16,185,129,.65)}.border-l-emerald-500\/70{border-left-color:rgba(16,185,129,.7)}.border-l-emerald-500\/75{border-left-color:rgba(16,185,129,.75)}.border-l-emerald-500\/80{border-left-color:rgba(16,185,129,.8)}.border-l-emerald-500\/85{border-left-color:rgba(16,185,129,.85)}.border-l-emerald-500\/90{border-left-color:rgba(16,185,129,.9)}.border-l-emerald-500\/95{border-left-color:rgba(16,185,129,.95)}.border-l-emerald-950{--tw-border-opacity:1;border-left-color:rgb(2 44 34/var(--tw-border-opacity,1))}.border-l-emerald-950\/0{border-left-color:rgba(2,44,34,0)}.border-l-emerald-950\/10{border-left-color:rgba(2,44,34,.1)}.border-l-emerald-950\/100{border-left-color:#022c22}.border-l-emerald-950\/15{border-left-color:rgba(2,44,34,.15)}.border-l-emerald-950\/20{border-left-color:rgba(2,44,34,.2)}.border-l-emerald-950\/25{border-left-color:rgba(2,44,34,.25)}.border-l-emerald-950\/30{border-left-color:rgba(2,44,34,.3)}.border-l-emerald-950\/35{border-left-color:rgba(2,44,34,.35)}.border-l-emerald-950\/40{border-left-color:rgba(2,44,34,.4)}.border-l-emerald-950\/45{border-left-color:rgba(2,44,34,.45)}.border-l-emerald-950\/5{border-left-color:rgba(2,44,34,.05)}.border-l-emerald-950\/50{border-left-color:rgba(2,44,34,.5)}.border-l-emerald-950\/55{border-left-color:rgba(2,44,34,.55)}.border-l-emerald-950\/60{border-left-color:rgba(2,44,34,.6)}.border-l-emerald-950\/65{border-left-color:rgba(2,44,34,.65)}.border-l-emerald-950\/70{border-left-color:rgba(2,44,34,.7)}.border-l-emerald-950\/75{border-left-color:rgba(2,44,34,.75)}.border-l-emerald-950\/80{border-left-color:rgba(2,44,34,.8)}.border-l-emerald-950\/85{border-left-color:rgba(2,44,34,.85)}.border-l-emerald-950\/90{border-left-color:rgba(2,44,34,.9)}.border-l-emerald-950\/95{border-left-color:rgba(2,44,34,.95)}.border-l-fuchsia-100{--tw-border-opacity:1;border-left-color:rgb(250 232 255/var(--tw-border-opacity,1))}.border-l-fuchsia-100\/0{border-left-color:rgba(250,232,255,0)}.border-l-fuchsia-100\/10{border-left-color:rgba(250,232,255,.1)}.border-l-fuchsia-100\/100{border-left-color:#fae8ff}.border-l-fuchsia-100\/15{border-left-color:rgba(250,232,255,.15)}.border-l-fuchsia-100\/20{border-left-color:rgba(250,232,255,.2)}.border-l-fuchsia-100\/25{border-left-color:rgba(250,232,255,.25)}.border-l-fuchsia-100\/30{border-left-color:rgba(250,232,255,.3)}.border-l-fuchsia-100\/35{border-left-color:rgba(250,232,255,.35)}.border-l-fuchsia-100\/40{border-left-color:rgba(250,232,255,.4)}.border-l-fuchsia-100\/45{border-left-color:rgba(250,232,255,.45)}.border-l-fuchsia-100\/5{border-left-color:rgba(250,232,255,.05)}.border-l-fuchsia-100\/50{border-left-color:rgba(250,232,255,.5)}.border-l-fuchsia-100\/55{border-left-color:rgba(250,232,255,.55)}.border-l-fuchsia-100\/60{border-left-color:rgba(250,232,255,.6)}.border-l-fuchsia-100\/65{border-left-color:rgba(250,232,255,.65)}.border-l-fuchsia-100\/70{border-left-color:rgba(250,232,255,.7)}.border-l-fuchsia-100\/75{border-left-color:rgba(250,232,255,.75)}.border-l-fuchsia-100\/80{border-left-color:rgba(250,232,255,.8)}.border-l-fuchsia-100\/85{border-left-color:rgba(250,232,255,.85)}.border-l-fuchsia-100\/90{border-left-color:rgba(250,232,255,.9)}.border-l-fuchsia-100\/95{border-left-color:rgba(250,232,255,.95)}.border-l-fuchsia-200{--tw-border-opacity:1;border-left-color:rgb(245 208 254/var(--tw-border-opacity,1))}.border-l-fuchsia-200\/0{border-left-color:rgba(245,208,254,0)}.border-l-fuchsia-200\/10{border-left-color:rgba(245,208,254,.1)}.border-l-fuchsia-200\/100{border-left-color:#f5d0fe}.border-l-fuchsia-200\/15{border-left-color:rgba(245,208,254,.15)}.border-l-fuchsia-200\/20{border-left-color:rgba(245,208,254,.2)}.border-l-fuchsia-200\/25{border-left-color:rgba(245,208,254,.25)}.border-l-fuchsia-200\/30{border-left-color:rgba(245,208,254,.3)}.border-l-fuchsia-200\/35{border-left-color:rgba(245,208,254,.35)}.border-l-fuchsia-200\/40{border-left-color:rgba(245,208,254,.4)}.border-l-fuchsia-200\/45{border-left-color:rgba(245,208,254,.45)}.border-l-fuchsia-200\/5{border-left-color:rgba(245,208,254,.05)}.border-l-fuchsia-200\/50{border-left-color:rgba(245,208,254,.5)}.border-l-fuchsia-200\/55{border-left-color:rgba(245,208,254,.55)}.border-l-fuchsia-200\/60{border-left-color:rgba(245,208,254,.6)}.border-l-fuchsia-200\/65{border-left-color:rgba(245,208,254,.65)}.border-l-fuchsia-200\/70{border-left-color:rgba(245,208,254,.7)}.border-l-fuchsia-200\/75{border-left-color:rgba(245,208,254,.75)}.border-l-fuchsia-200\/80{border-left-color:rgba(245,208,254,.8)}.border-l-fuchsia-200\/85{border-left-color:rgba(245,208,254,.85)}.border-l-fuchsia-200\/90{border-left-color:rgba(245,208,254,.9)}.border-l-fuchsia-200\/95{border-left-color:rgba(245,208,254,.95)}.border-l-fuchsia-400{--tw-border-opacity:1;border-left-color:rgb(232 121 249/var(--tw-border-opacity,1))}.border-l-fuchsia-400\/0{border-left-color:rgba(232,121,249,0)}.border-l-fuchsia-400\/10{border-left-color:rgba(232,121,249,.1)}.border-l-fuchsia-400\/100{border-left-color:#e879f9}.border-l-fuchsia-400\/15{border-left-color:rgba(232,121,249,.15)}.border-l-fuchsia-400\/20{border-left-color:rgba(232,121,249,.2)}.border-l-fuchsia-400\/25{border-left-color:rgba(232,121,249,.25)}.border-l-fuchsia-400\/30{border-left-color:rgba(232,121,249,.3)}.border-l-fuchsia-400\/35{border-left-color:rgba(232,121,249,.35)}.border-l-fuchsia-400\/40{border-left-color:rgba(232,121,249,.4)}.border-l-fuchsia-400\/45{border-left-color:rgba(232,121,249,.45)}.border-l-fuchsia-400\/5{border-left-color:rgba(232,121,249,.05)}.border-l-fuchsia-400\/50{border-left-color:rgba(232,121,249,.5)}.border-l-fuchsia-400\/55{border-left-color:rgba(232,121,249,.55)}.border-l-fuchsia-400\/60{border-left-color:rgba(232,121,249,.6)}.border-l-fuchsia-400\/65{border-left-color:rgba(232,121,249,.65)}.border-l-fuchsia-400\/70{border-left-color:rgba(232,121,249,.7)}.border-l-fuchsia-400\/75{border-left-color:rgba(232,121,249,.75)}.border-l-fuchsia-400\/80{border-left-color:rgba(232,121,249,.8)}.border-l-fuchsia-400\/85{border-left-color:rgba(232,121,249,.85)}.border-l-fuchsia-400\/90{border-left-color:rgba(232,121,249,.9)}.border-l-fuchsia-400\/95{border-left-color:rgba(232,121,249,.95)}.border-l-fuchsia-50{--tw-border-opacity:1;border-left-color:rgb(253 244 255/var(--tw-border-opacity,1))}.border-l-fuchsia-50\/0{border-left-color:rgba(253,244,255,0)}.border-l-fuchsia-50\/10{border-left-color:rgba(253,244,255,.1)}.border-l-fuchsia-50\/100{border-left-color:#fdf4ff}.border-l-fuchsia-50\/15{border-left-color:rgba(253,244,255,.15)}.border-l-fuchsia-50\/20{border-left-color:rgba(253,244,255,.2)}.border-l-fuchsia-50\/25{border-left-color:rgba(253,244,255,.25)}.border-l-fuchsia-50\/30{border-left-color:rgba(253,244,255,.3)}.border-l-fuchsia-50\/35{border-left-color:rgba(253,244,255,.35)}.border-l-fuchsia-50\/40{border-left-color:rgba(253,244,255,.4)}.border-l-fuchsia-50\/45{border-left-color:rgba(253,244,255,.45)}.border-l-fuchsia-50\/5{border-left-color:rgba(253,244,255,.05)}.border-l-fuchsia-50\/50{border-left-color:rgba(253,244,255,.5)}.border-l-fuchsia-50\/55{border-left-color:rgba(253,244,255,.55)}.border-l-fuchsia-50\/60{border-left-color:rgba(253,244,255,.6)}.border-l-fuchsia-50\/65{border-left-color:rgba(253,244,255,.65)}.border-l-fuchsia-50\/70{border-left-color:rgba(253,244,255,.7)}.border-l-fuchsia-50\/75{border-left-color:rgba(253,244,255,.75)}.border-l-fuchsia-50\/80{border-left-color:rgba(253,244,255,.8)}.border-l-fuchsia-50\/85{border-left-color:rgba(253,244,255,.85)}.border-l-fuchsia-50\/90{border-left-color:rgba(253,244,255,.9)}.border-l-fuchsia-50\/95{border-left-color:rgba(253,244,255,.95)}.border-l-fuchsia-500{--tw-border-opacity:1;border-left-color:rgb(217 70 239/var(--tw-border-opacity,1))}.border-l-fuchsia-500\/0{border-left-color:rgba(217,70,239,0)}.border-l-fuchsia-500\/10{border-left-color:rgba(217,70,239,.1)}.border-l-fuchsia-500\/100{border-left-color:#d946ef}.border-l-fuchsia-500\/15{border-left-color:rgba(217,70,239,.15)}.border-l-fuchsia-500\/20{border-left-color:rgba(217,70,239,.2)}.border-l-fuchsia-500\/25{border-left-color:rgba(217,70,239,.25)}.border-l-fuchsia-500\/30{border-left-color:rgba(217,70,239,.3)}.border-l-fuchsia-500\/35{border-left-color:rgba(217,70,239,.35)}.border-l-fuchsia-500\/40{border-left-color:rgba(217,70,239,.4)}.border-l-fuchsia-500\/45{border-left-color:rgba(217,70,239,.45)}.border-l-fuchsia-500\/5{border-left-color:rgba(217,70,239,.05)}.border-l-fuchsia-500\/50{border-left-color:rgba(217,70,239,.5)}.border-l-fuchsia-500\/55{border-left-color:rgba(217,70,239,.55)}.border-l-fuchsia-500\/60{border-left-color:rgba(217,70,239,.6)}.border-l-fuchsia-500\/65{border-left-color:rgba(217,70,239,.65)}.border-l-fuchsia-500\/70{border-left-color:rgba(217,70,239,.7)}.border-l-fuchsia-500\/75{border-left-color:rgba(217,70,239,.75)}.border-l-fuchsia-500\/80{border-left-color:rgba(217,70,239,.8)}.border-l-fuchsia-500\/85{border-left-color:rgba(217,70,239,.85)}.border-l-fuchsia-500\/90{border-left-color:rgba(217,70,239,.9)}.border-l-fuchsia-500\/95{border-left-color:rgba(217,70,239,.95)}.border-l-fuchsia-950{--tw-border-opacity:1;border-left-color:rgb(74 4 78/var(--tw-border-opacity,1))}.border-l-fuchsia-950\/0{border-left-color:rgba(74,4,78,0)}.border-l-fuchsia-950\/10{border-left-color:rgba(74,4,78,.1)}.border-l-fuchsia-950\/100{border-left-color:#4a044e}.border-l-fuchsia-950\/15{border-left-color:rgba(74,4,78,.15)}.border-l-fuchsia-950\/20{border-left-color:rgba(74,4,78,.2)}.border-l-fuchsia-950\/25{border-left-color:rgba(74,4,78,.25)}.border-l-fuchsia-950\/30{border-left-color:rgba(74,4,78,.3)}.border-l-fuchsia-950\/35{border-left-color:rgba(74,4,78,.35)}.border-l-fuchsia-950\/40{border-left-color:rgba(74,4,78,.4)}.border-l-fuchsia-950\/45{border-left-color:rgba(74,4,78,.45)}.border-l-fuchsia-950\/5{border-left-color:rgba(74,4,78,.05)}.border-l-fuchsia-950\/50{border-left-color:rgba(74,4,78,.5)}.border-l-fuchsia-950\/55{border-left-color:rgba(74,4,78,.55)}.border-l-fuchsia-950\/60{border-left-color:rgba(74,4,78,.6)}.border-l-fuchsia-950\/65{border-left-color:rgba(74,4,78,.65)}.border-l-fuchsia-950\/70{border-left-color:rgba(74,4,78,.7)}.border-l-fuchsia-950\/75{border-left-color:rgba(74,4,78,.75)}.border-l-fuchsia-950\/80{border-left-color:rgba(74,4,78,.8)}.border-l-fuchsia-950\/85{border-left-color:rgba(74,4,78,.85)}.border-l-fuchsia-950\/90{border-left-color:rgba(74,4,78,.9)}.border-l-fuchsia-950\/95{border-left-color:rgba(74,4,78,.95)}.border-l-gray-100{--tw-border-opacity:1;border-left-color:rgb(243 244 246/var(--tw-border-opacity,1))}.border-l-gray-100\/0{border-left-color:rgba(243,244,246,0)}.border-l-gray-100\/10{border-left-color:rgba(243,244,246,.1)}.border-l-gray-100\/100{border-left-color:#f3f4f6}.border-l-gray-100\/15{border-left-color:rgba(243,244,246,.15)}.border-l-gray-100\/20{border-left-color:rgba(243,244,246,.2)}.border-l-gray-100\/25{border-left-color:rgba(243,244,246,.25)}.border-l-gray-100\/30{border-left-color:rgba(243,244,246,.3)}.border-l-gray-100\/35{border-left-color:rgba(243,244,246,.35)}.border-l-gray-100\/40{border-left-color:rgba(243,244,246,.4)}.border-l-gray-100\/45{border-left-color:rgba(243,244,246,.45)}.border-l-gray-100\/5{border-left-color:rgba(243,244,246,.05)}.border-l-gray-100\/50{border-left-color:rgba(243,244,246,.5)}.border-l-gray-100\/55{border-left-color:rgba(243,244,246,.55)}.border-l-gray-100\/60{border-left-color:rgba(243,244,246,.6)}.border-l-gray-100\/65{border-left-color:rgba(243,244,246,.65)}.border-l-gray-100\/70{border-left-color:rgba(243,244,246,.7)}.border-l-gray-100\/75{border-left-color:rgba(243,244,246,.75)}.border-l-gray-100\/80{border-left-color:rgba(243,244,246,.8)}.border-l-gray-100\/85{border-left-color:rgba(243,244,246,.85)}.border-l-gray-100\/90{border-left-color:rgba(243,244,246,.9)}.border-l-gray-100\/95{border-left-color:rgba(243,244,246,.95)}.border-l-gray-200{--tw-border-opacity:1;border-left-color:rgb(229 231 235/var(--tw-border-opacity,1))}.border-l-gray-200\/0{border-left-color:rgba(229,231,235,0)}.border-l-gray-200\/10{border-left-color:rgba(229,231,235,.1)}.border-l-gray-200\/100{border-left-color:#e5e7eb}.border-l-gray-200\/15{border-left-color:rgba(229,231,235,.15)}.border-l-gray-200\/20{border-left-color:rgba(229,231,235,.2)}.border-l-gray-200\/25{border-left-color:rgba(229,231,235,.25)}.border-l-gray-200\/30{border-left-color:rgba(229,231,235,.3)}.border-l-gray-200\/35{border-left-color:rgba(229,231,235,.35)}.border-l-gray-200\/40{border-left-color:rgba(229,231,235,.4)}.border-l-gray-200\/45{border-left-color:rgba(229,231,235,.45)}.border-l-gray-200\/5{border-left-color:rgba(229,231,235,.05)}.border-l-gray-200\/50{border-left-color:rgba(229,231,235,.5)}.border-l-gray-200\/55{border-left-color:rgba(229,231,235,.55)}.border-l-gray-200\/60{border-left-color:rgba(229,231,235,.6)}.border-l-gray-200\/65{border-left-color:rgba(229,231,235,.65)}.border-l-gray-200\/70{border-left-color:rgba(229,231,235,.7)}.border-l-gray-200\/75{border-left-color:rgba(229,231,235,.75)}.border-l-gray-200\/80{border-left-color:rgba(229,231,235,.8)}.border-l-gray-200\/85{border-left-color:rgba(229,231,235,.85)}.border-l-gray-200\/90{border-left-color:rgba(229,231,235,.9)}.border-l-gray-200\/95{border-left-color:rgba(229,231,235,.95)}.border-l-gray-400{--tw-border-opacity:1;border-left-color:rgb(156 163 175/var(--tw-border-opacity,1))}.border-l-gray-400\/0{border-left-color:rgba(156,163,175,0)}.border-l-gray-400\/10{border-left-color:rgba(156,163,175,.1)}.border-l-gray-400\/100{border-left-color:#9ca3af}.border-l-gray-400\/15{border-left-color:rgba(156,163,175,.15)}.border-l-gray-400\/20{border-left-color:rgba(156,163,175,.2)}.border-l-gray-400\/25{border-left-color:rgba(156,163,175,.25)}.border-l-gray-400\/30{border-left-color:rgba(156,163,175,.3)}.border-l-gray-400\/35{border-left-color:rgba(156,163,175,.35)}.border-l-gray-400\/40{border-left-color:rgba(156,163,175,.4)}.border-l-gray-400\/45{border-left-color:rgba(156,163,175,.45)}.border-l-gray-400\/5{border-left-color:rgba(156,163,175,.05)}.border-l-gray-400\/50{border-left-color:rgba(156,163,175,.5)}.border-l-gray-400\/55{border-left-color:rgba(156,163,175,.55)}.border-l-gray-400\/60{border-left-color:rgba(156,163,175,.6)}.border-l-gray-400\/65{border-left-color:rgba(156,163,175,.65)}.border-l-gray-400\/70{border-left-color:rgba(156,163,175,.7)}.border-l-gray-400\/75{border-left-color:rgba(156,163,175,.75)}.border-l-gray-400\/80{border-left-color:rgba(156,163,175,.8)}.border-l-gray-400\/85{border-left-color:rgba(156,163,175,.85)}.border-l-gray-400\/90{border-left-color:rgba(156,163,175,.9)}.border-l-gray-400\/95{border-left-color:rgba(156,163,175,.95)}.border-l-gray-50{--tw-border-opacity:1;border-left-color:rgb(249 250 251/var(--tw-border-opacity,1))}.border-l-gray-50\/0{border-left-color:rgba(249,250,251,0)}.border-l-gray-50\/10{border-left-color:rgba(249,250,251,.1)}.border-l-gray-50\/100{border-left-color:#f9fafb}.border-l-gray-50\/15{border-left-color:rgba(249,250,251,.15)}.border-l-gray-50\/20{border-left-color:rgba(249,250,251,.2)}.border-l-gray-50\/25{border-left-color:rgba(249,250,251,.25)}.border-l-gray-50\/30{border-left-color:rgba(249,250,251,.3)}.border-l-gray-50\/35{border-left-color:rgba(249,250,251,.35)}.border-l-gray-50\/40{border-left-color:rgba(249,250,251,.4)}.border-l-gray-50\/45{border-left-color:rgba(249,250,251,.45)}.border-l-gray-50\/5{border-left-color:rgba(249,250,251,.05)}.border-l-gray-50\/50{border-left-color:rgba(249,250,251,.5)}.border-l-gray-50\/55{border-left-color:rgba(249,250,251,.55)}.border-l-gray-50\/60{border-left-color:rgba(249,250,251,.6)}.border-l-gray-50\/65{border-left-color:rgba(249,250,251,.65)}.border-l-gray-50\/70{border-left-color:rgba(249,250,251,.7)}.border-l-gray-50\/75{border-left-color:rgba(249,250,251,.75)}.border-l-gray-50\/80{border-left-color:rgba(249,250,251,.8)}.border-l-gray-50\/85{border-left-color:rgba(249,250,251,.85)}.border-l-gray-50\/90{border-left-color:rgba(249,250,251,.9)}.border-l-gray-50\/95{border-left-color:rgba(249,250,251,.95)}.border-l-gray-500{--tw-border-opacity:1;border-left-color:rgb(107 114 128/var(--tw-border-opacity,1))}.border-l-gray-500\/0{border-left-color:hsla(220,9%,46%,0)}.border-l-gray-500\/10{border-left-color:hsla(220,9%,46%,.1)}.border-l-gray-500\/100{border-left-color:#6b7280}.border-l-gray-500\/15{border-left-color:hsla(220,9%,46%,.15)}.border-l-gray-500\/20{border-left-color:hsla(220,9%,46%,.2)}.border-l-gray-500\/25{border-left-color:hsla(220,9%,46%,.25)}.border-l-gray-500\/30{border-left-color:hsla(220,9%,46%,.3)}.border-l-gray-500\/35{border-left-color:hsla(220,9%,46%,.35)}.border-l-gray-500\/40{border-left-color:hsla(220,9%,46%,.4)}.border-l-gray-500\/45{border-left-color:hsla(220,9%,46%,.45)}.border-l-gray-500\/5{border-left-color:hsla(220,9%,46%,.05)}.border-l-gray-500\/50{border-left-color:hsla(220,9%,46%,.5)}.border-l-gray-500\/55{border-left-color:hsla(220,9%,46%,.55)}.border-l-gray-500\/60{border-left-color:hsla(220,9%,46%,.6)}.border-l-gray-500\/65{border-left-color:hsla(220,9%,46%,.65)}.border-l-gray-500\/70{border-left-color:hsla(220,9%,46%,.7)}.border-l-gray-500\/75{border-left-color:hsla(220,9%,46%,.75)}.border-l-gray-500\/80{border-left-color:hsla(220,9%,46%,.8)}.border-l-gray-500\/85{border-left-color:hsla(220,9%,46%,.85)}.border-l-gray-500\/90{border-left-color:hsla(220,9%,46%,.9)}.border-l-gray-500\/95{border-left-color:hsla(220,9%,46%,.95)}.border-l-gray-950{--tw-border-opacity:1;border-left-color:rgb(3 7 18/var(--tw-border-opacity,1))}.border-l-gray-950\/0{border-left-color:rgba(3,7,18,0)}.border-l-gray-950\/10{border-left-color:rgba(3,7,18,.1)}.border-l-gray-950\/100{border-left-color:#030712}.border-l-gray-950\/15{border-left-color:rgba(3,7,18,.15)}.border-l-gray-950\/20{border-left-color:rgba(3,7,18,.2)}.border-l-gray-950\/25{border-left-color:rgba(3,7,18,.25)}.border-l-gray-950\/30{border-left-color:rgba(3,7,18,.3)}.border-l-gray-950\/35{border-left-color:rgba(3,7,18,.35)}.border-l-gray-950\/40{border-left-color:rgba(3,7,18,.4)}.border-l-gray-950\/45{border-left-color:rgba(3,7,18,.45)}.border-l-gray-950\/5{border-left-color:rgba(3,7,18,.05)}.border-l-gray-950\/50{border-left-color:rgba(3,7,18,.5)}.border-l-gray-950\/55{border-left-color:rgba(3,7,18,.55)}.border-l-gray-950\/60{border-left-color:rgba(3,7,18,.6)}.border-l-gray-950\/65{border-left-color:rgba(3,7,18,.65)}.border-l-gray-950\/70{border-left-color:rgba(3,7,18,.7)}.border-l-gray-950\/75{border-left-color:rgba(3,7,18,.75)}.border-l-gray-950\/80{border-left-color:rgba(3,7,18,.8)}.border-l-gray-950\/85{border-left-color:rgba(3,7,18,.85)}.border-l-gray-950\/90{border-left-color:rgba(3,7,18,.9)}.border-l-gray-950\/95{border-left-color:rgba(3,7,18,.95)}.border-l-green-100{--tw-border-opacity:1;border-left-color:rgb(220 252 231/var(--tw-border-opacity,1))}.border-l-green-100\/0{border-left-color:rgba(220,252,231,0)}.border-l-green-100\/10{border-left-color:rgba(220,252,231,.1)}.border-l-green-100\/100{border-left-color:#dcfce7}.border-l-green-100\/15{border-left-color:rgba(220,252,231,.15)}.border-l-green-100\/20{border-left-color:rgba(220,252,231,.2)}.border-l-green-100\/25{border-left-color:rgba(220,252,231,.25)}.border-l-green-100\/30{border-left-color:rgba(220,252,231,.3)}.border-l-green-100\/35{border-left-color:rgba(220,252,231,.35)}.border-l-green-100\/40{border-left-color:rgba(220,252,231,.4)}.border-l-green-100\/45{border-left-color:rgba(220,252,231,.45)}.border-l-green-100\/5{border-left-color:rgba(220,252,231,.05)}.border-l-green-100\/50{border-left-color:rgba(220,252,231,.5)}.border-l-green-100\/55{border-left-color:rgba(220,252,231,.55)}.border-l-green-100\/60{border-left-color:rgba(220,252,231,.6)}.border-l-green-100\/65{border-left-color:rgba(220,252,231,.65)}.border-l-green-100\/70{border-left-color:rgba(220,252,231,.7)}.border-l-green-100\/75{border-left-color:rgba(220,252,231,.75)}.border-l-green-100\/80{border-left-color:rgba(220,252,231,.8)}.border-l-green-100\/85{border-left-color:rgba(220,252,231,.85)}.border-l-green-100\/90{border-left-color:rgba(220,252,231,.9)}.border-l-green-100\/95{border-left-color:rgba(220,252,231,.95)}.border-l-green-200{--tw-border-opacity:1;border-left-color:rgb(187 247 208/var(--tw-border-opacity,1))}.border-l-green-200\/0{border-left-color:rgba(187,247,208,0)}.border-l-green-200\/10{border-left-color:rgba(187,247,208,.1)}.border-l-green-200\/100{border-left-color:#bbf7d0}.border-l-green-200\/15{border-left-color:rgba(187,247,208,.15)}.border-l-green-200\/20{border-left-color:rgba(187,247,208,.2)}.border-l-green-200\/25{border-left-color:rgba(187,247,208,.25)}.border-l-green-200\/30{border-left-color:rgba(187,247,208,.3)}.border-l-green-200\/35{border-left-color:rgba(187,247,208,.35)}.border-l-green-200\/40{border-left-color:rgba(187,247,208,.4)}.border-l-green-200\/45{border-left-color:rgba(187,247,208,.45)}.border-l-green-200\/5{border-left-color:rgba(187,247,208,.05)}.border-l-green-200\/50{border-left-color:rgba(187,247,208,.5)}.border-l-green-200\/55{border-left-color:rgba(187,247,208,.55)}.border-l-green-200\/60{border-left-color:rgba(187,247,208,.6)}.border-l-green-200\/65{border-left-color:rgba(187,247,208,.65)}.border-l-green-200\/70{border-left-color:rgba(187,247,208,.7)}.border-l-green-200\/75{border-left-color:rgba(187,247,208,.75)}.border-l-green-200\/80{border-left-color:rgba(187,247,208,.8)}.border-l-green-200\/85{border-left-color:rgba(187,247,208,.85)}.border-l-green-200\/90{border-left-color:rgba(187,247,208,.9)}.border-l-green-200\/95{border-left-color:rgba(187,247,208,.95)}.border-l-green-400{--tw-border-opacity:1;border-left-color:rgb(74 222 128/var(--tw-border-opacity,1))}.border-l-green-400\/0{border-left-color:rgba(74,222,128,0)}.border-l-green-400\/10{border-left-color:rgba(74,222,128,.1)}.border-l-green-400\/100{border-left-color:#4ade80}.border-l-green-400\/15{border-left-color:rgba(74,222,128,.15)}.border-l-green-400\/20{border-left-color:rgba(74,222,128,.2)}.border-l-green-400\/25{border-left-color:rgba(74,222,128,.25)}.border-l-green-400\/30{border-left-color:rgba(74,222,128,.3)}.border-l-green-400\/35{border-left-color:rgba(74,222,128,.35)}.border-l-green-400\/40{border-left-color:rgba(74,222,128,.4)}.border-l-green-400\/45{border-left-color:rgba(74,222,128,.45)}.border-l-green-400\/5{border-left-color:rgba(74,222,128,.05)}.border-l-green-400\/50{border-left-color:rgba(74,222,128,.5)}.border-l-green-400\/55{border-left-color:rgba(74,222,128,.55)}.border-l-green-400\/60{border-left-color:rgba(74,222,128,.6)}.border-l-green-400\/65{border-left-color:rgba(74,222,128,.65)}.border-l-green-400\/70{border-left-color:rgba(74,222,128,.7)}.border-l-green-400\/75{border-left-color:rgba(74,222,128,.75)}.border-l-green-400\/80{border-left-color:rgba(74,222,128,.8)}.border-l-green-400\/85{border-left-color:rgba(74,222,128,.85)}.border-l-green-400\/90{border-left-color:rgba(74,222,128,.9)}.border-l-green-400\/95{border-left-color:rgba(74,222,128,.95)}.border-l-green-50{--tw-border-opacity:1;border-left-color:rgb(240 253 244/var(--tw-border-opacity,1))}.border-l-green-50\/0{border-left-color:rgba(240,253,244,0)}.border-l-green-50\/10{border-left-color:rgba(240,253,244,.1)}.border-l-green-50\/100{border-left-color:#f0fdf4}.border-l-green-50\/15{border-left-color:rgba(240,253,244,.15)}.border-l-green-50\/20{border-left-color:rgba(240,253,244,.2)}.border-l-green-50\/25{border-left-color:rgba(240,253,244,.25)}.border-l-green-50\/30{border-left-color:rgba(240,253,244,.3)}.border-l-green-50\/35{border-left-color:rgba(240,253,244,.35)}.border-l-green-50\/40{border-left-color:rgba(240,253,244,.4)}.border-l-green-50\/45{border-left-color:rgba(240,253,244,.45)}.border-l-green-50\/5{border-left-color:rgba(240,253,244,.05)}.border-l-green-50\/50{border-left-color:rgba(240,253,244,.5)}.border-l-green-50\/55{border-left-color:rgba(240,253,244,.55)}.border-l-green-50\/60{border-left-color:rgba(240,253,244,.6)}.border-l-green-50\/65{border-left-color:rgba(240,253,244,.65)}.border-l-green-50\/70{border-left-color:rgba(240,253,244,.7)}.border-l-green-50\/75{border-left-color:rgba(240,253,244,.75)}.border-l-green-50\/80{border-left-color:rgba(240,253,244,.8)}.border-l-green-50\/85{border-left-color:rgba(240,253,244,.85)}.border-l-green-50\/90{border-left-color:rgba(240,253,244,.9)}.border-l-green-50\/95{border-left-color:rgba(240,253,244,.95)}.border-l-green-500{--tw-border-opacity:1;border-left-color:rgb(34 197 94/var(--tw-border-opacity,1))}.border-l-green-500\/0{border-left-color:rgba(34,197,94,0)}.border-l-green-500\/10{border-left-color:rgba(34,197,94,.1)}.border-l-green-500\/100{border-left-color:#22c55e}.border-l-green-500\/15{border-left-color:rgba(34,197,94,.15)}.border-l-green-500\/20{border-left-color:rgba(34,197,94,.2)}.border-l-green-500\/25{border-left-color:rgba(34,197,94,.25)}.border-l-green-500\/30{border-left-color:rgba(34,197,94,.3)}.border-l-green-500\/35{border-left-color:rgba(34,197,94,.35)}.border-l-green-500\/40{border-left-color:rgba(34,197,94,.4)}.border-l-green-500\/45{border-left-color:rgba(34,197,94,.45)}.border-l-green-500\/5{border-left-color:rgba(34,197,94,.05)}.border-l-green-500\/50{border-left-color:rgba(34,197,94,.5)}.border-l-green-500\/55{border-left-color:rgba(34,197,94,.55)}.border-l-green-500\/60{border-left-color:rgba(34,197,94,.6)}.border-l-green-500\/65{border-left-color:rgba(34,197,94,.65)}.border-l-green-500\/70{border-left-color:rgba(34,197,94,.7)}.border-l-green-500\/75{border-left-color:rgba(34,197,94,.75)}.border-l-green-500\/80{border-left-color:rgba(34,197,94,.8)}.border-l-green-500\/85{border-left-color:rgba(34,197,94,.85)}.border-l-green-500\/90{border-left-color:rgba(34,197,94,.9)}.border-l-green-500\/95{border-left-color:rgba(34,197,94,.95)}.border-l-green-950{--tw-border-opacity:1;border-left-color:rgb(5 46 22/var(--tw-border-opacity,1))}.border-l-green-950\/0{border-left-color:rgba(5,46,22,0)}.border-l-green-950\/10{border-left-color:rgba(5,46,22,.1)}.border-l-green-950\/100{border-left-color:#052e16}.border-l-green-950\/15{border-left-color:rgba(5,46,22,.15)}.border-l-green-950\/20{border-left-color:rgba(5,46,22,.2)}.border-l-green-950\/25{border-left-color:rgba(5,46,22,.25)}.border-l-green-950\/30{border-left-color:rgba(5,46,22,.3)}.border-l-green-950\/35{border-left-color:rgba(5,46,22,.35)}.border-l-green-950\/40{border-left-color:rgba(5,46,22,.4)}.border-l-green-950\/45{border-left-color:rgba(5,46,22,.45)}.border-l-green-950\/5{border-left-color:rgba(5,46,22,.05)}.border-l-green-950\/50{border-left-color:rgba(5,46,22,.5)}.border-l-green-950\/55{border-left-color:rgba(5,46,22,.55)}.border-l-green-950\/60{border-left-color:rgba(5,46,22,.6)}.border-l-green-950\/65{border-left-color:rgba(5,46,22,.65)}.border-l-green-950\/70{border-left-color:rgba(5,46,22,.7)}.border-l-green-950\/75{border-left-color:rgba(5,46,22,.75)}.border-l-green-950\/80{border-left-color:rgba(5,46,22,.8)}.border-l-green-950\/85{border-left-color:rgba(5,46,22,.85)}.border-l-green-950\/90{border-left-color:rgba(5,46,22,.9)}.border-l-green-950\/95{border-left-color:rgba(5,46,22,.95)}.border-l-indigo-100{--tw-border-opacity:1;border-left-color:rgb(224 231 255/var(--tw-border-opacity,1))}.border-l-indigo-100\/0{border-left-color:rgba(224,231,255,0)}.border-l-indigo-100\/10{border-left-color:rgba(224,231,255,.1)}.border-l-indigo-100\/100{border-left-color:#e0e7ff}.border-l-indigo-100\/15{border-left-color:rgba(224,231,255,.15)}.border-l-indigo-100\/20{border-left-color:rgba(224,231,255,.2)}.border-l-indigo-100\/25{border-left-color:rgba(224,231,255,.25)}.border-l-indigo-100\/30{border-left-color:rgba(224,231,255,.3)}.border-l-indigo-100\/35{border-left-color:rgba(224,231,255,.35)}.border-l-indigo-100\/40{border-left-color:rgba(224,231,255,.4)}.border-l-indigo-100\/45{border-left-color:rgba(224,231,255,.45)}.border-l-indigo-100\/5{border-left-color:rgba(224,231,255,.05)}.border-l-indigo-100\/50{border-left-color:rgba(224,231,255,.5)}.border-l-indigo-100\/55{border-left-color:rgba(224,231,255,.55)}.border-l-indigo-100\/60{border-left-color:rgba(224,231,255,.6)}.border-l-indigo-100\/65{border-left-color:rgba(224,231,255,.65)}.border-l-indigo-100\/70{border-left-color:rgba(224,231,255,.7)}.border-l-indigo-100\/75{border-left-color:rgba(224,231,255,.75)}.border-l-indigo-100\/80{border-left-color:rgba(224,231,255,.8)}.border-l-indigo-100\/85{border-left-color:rgba(224,231,255,.85)}.border-l-indigo-100\/90{border-left-color:rgba(224,231,255,.9)}.border-l-indigo-100\/95{border-left-color:rgba(224,231,255,.95)}.border-l-indigo-200{--tw-border-opacity:1;border-left-color:rgb(199 210 254/var(--tw-border-opacity,1))}.border-l-indigo-200\/0{border-left-color:rgba(199,210,254,0)}.border-l-indigo-200\/10{border-left-color:rgba(199,210,254,.1)}.border-l-indigo-200\/100{border-left-color:#c7d2fe}.border-l-indigo-200\/15{border-left-color:rgba(199,210,254,.15)}.border-l-indigo-200\/20{border-left-color:rgba(199,210,254,.2)}.border-l-indigo-200\/25{border-left-color:rgba(199,210,254,.25)}.border-l-indigo-200\/30{border-left-color:rgba(199,210,254,.3)}.border-l-indigo-200\/35{border-left-color:rgba(199,210,254,.35)}.border-l-indigo-200\/40{border-left-color:rgba(199,210,254,.4)}.border-l-indigo-200\/45{border-left-color:rgba(199,210,254,.45)}.border-l-indigo-200\/5{border-left-color:rgba(199,210,254,.05)}.border-l-indigo-200\/50{border-left-color:rgba(199,210,254,.5)}.border-l-indigo-200\/55{border-left-color:rgba(199,210,254,.55)}.border-l-indigo-200\/60{border-left-color:rgba(199,210,254,.6)}.border-l-indigo-200\/65{border-left-color:rgba(199,210,254,.65)}.border-l-indigo-200\/70{border-left-color:rgba(199,210,254,.7)}.border-l-indigo-200\/75{border-left-color:rgba(199,210,254,.75)}.border-l-indigo-200\/80{border-left-color:rgba(199,210,254,.8)}.border-l-indigo-200\/85{border-left-color:rgba(199,210,254,.85)}.border-l-indigo-200\/90{border-left-color:rgba(199,210,254,.9)}.border-l-indigo-200\/95{border-left-color:rgba(199,210,254,.95)}.border-l-indigo-400{--tw-border-opacity:1;border-left-color:rgb(129 140 248/var(--tw-border-opacity,1))}.border-l-indigo-400\/0{border-left-color:rgba(129,140,248,0)}.border-l-indigo-400\/10{border-left-color:rgba(129,140,248,.1)}.border-l-indigo-400\/100{border-left-color:#818cf8}.border-l-indigo-400\/15{border-left-color:rgba(129,140,248,.15)}.border-l-indigo-400\/20{border-left-color:rgba(129,140,248,.2)}.border-l-indigo-400\/25{border-left-color:rgba(129,140,248,.25)}.border-l-indigo-400\/30{border-left-color:rgba(129,140,248,.3)}.border-l-indigo-400\/35{border-left-color:rgba(129,140,248,.35)}.border-l-indigo-400\/40{border-left-color:rgba(129,140,248,.4)}.border-l-indigo-400\/45{border-left-color:rgba(129,140,248,.45)}.border-l-indigo-400\/5{border-left-color:rgba(129,140,248,.05)}.border-l-indigo-400\/50{border-left-color:rgba(129,140,248,.5)}.border-l-indigo-400\/55{border-left-color:rgba(129,140,248,.55)}.border-l-indigo-400\/60{border-left-color:rgba(129,140,248,.6)}.border-l-indigo-400\/65{border-left-color:rgba(129,140,248,.65)}.border-l-indigo-400\/70{border-left-color:rgba(129,140,248,.7)}.border-l-indigo-400\/75{border-left-color:rgba(129,140,248,.75)}.border-l-indigo-400\/80{border-left-color:rgba(129,140,248,.8)}.border-l-indigo-400\/85{border-left-color:rgba(129,140,248,.85)}.border-l-indigo-400\/90{border-left-color:rgba(129,140,248,.9)}.border-l-indigo-400\/95{border-left-color:rgba(129,140,248,.95)}.border-l-indigo-50{--tw-border-opacity:1;border-left-color:rgb(238 242 255/var(--tw-border-opacity,1))}.border-l-indigo-50\/0{border-left-color:rgba(238,242,255,0)}.border-l-indigo-50\/10{border-left-color:rgba(238,242,255,.1)}.border-l-indigo-50\/100{border-left-color:#eef2ff}.border-l-indigo-50\/15{border-left-color:rgba(238,242,255,.15)}.border-l-indigo-50\/20{border-left-color:rgba(238,242,255,.2)}.border-l-indigo-50\/25{border-left-color:rgba(238,242,255,.25)}.border-l-indigo-50\/30{border-left-color:rgba(238,242,255,.3)}.border-l-indigo-50\/35{border-left-color:rgba(238,242,255,.35)}.border-l-indigo-50\/40{border-left-color:rgba(238,242,255,.4)}.border-l-indigo-50\/45{border-left-color:rgba(238,242,255,.45)}.border-l-indigo-50\/5{border-left-color:rgba(238,242,255,.05)}.border-l-indigo-50\/50{border-left-color:rgba(238,242,255,.5)}.border-l-indigo-50\/55{border-left-color:rgba(238,242,255,.55)}.border-l-indigo-50\/60{border-left-color:rgba(238,242,255,.6)}.border-l-indigo-50\/65{border-left-color:rgba(238,242,255,.65)}.border-l-indigo-50\/70{border-left-color:rgba(238,242,255,.7)}.border-l-indigo-50\/75{border-left-color:rgba(238,242,255,.75)}.border-l-indigo-50\/80{border-left-color:rgba(238,242,255,.8)}.border-l-indigo-50\/85{border-left-color:rgba(238,242,255,.85)}.border-l-indigo-50\/90{border-left-color:rgba(238,242,255,.9)}.border-l-indigo-50\/95{border-left-color:rgba(238,242,255,.95)}.border-l-indigo-500{--tw-border-opacity:1;border-left-color:rgb(99 102 241/var(--tw-border-opacity,1))}.border-l-indigo-500\/0{border-left-color:rgba(99,102,241,0)}.border-l-indigo-500\/10{border-left-color:rgba(99,102,241,.1)}.border-l-indigo-500\/100{border-left-color:#6366f1}.border-l-indigo-500\/15{border-left-color:rgba(99,102,241,.15)}.border-l-indigo-500\/20{border-left-color:rgba(99,102,241,.2)}.border-l-indigo-500\/25{border-left-color:rgba(99,102,241,.25)}.border-l-indigo-500\/30{border-left-color:rgba(99,102,241,.3)}.border-l-indigo-500\/35{border-left-color:rgba(99,102,241,.35)}.border-l-indigo-500\/40{border-left-color:rgba(99,102,241,.4)}.border-l-indigo-500\/45{border-left-color:rgba(99,102,241,.45)}.border-l-indigo-500\/5{border-left-color:rgba(99,102,241,.05)}.border-l-indigo-500\/50{border-left-color:rgba(99,102,241,.5)}.border-l-indigo-500\/55{border-left-color:rgba(99,102,241,.55)}.border-l-indigo-500\/60{border-left-color:rgba(99,102,241,.6)}.border-l-indigo-500\/65{border-left-color:rgba(99,102,241,.65)}.border-l-indigo-500\/70{border-left-color:rgba(99,102,241,.7)}.border-l-indigo-500\/75{border-left-color:rgba(99,102,241,.75)}.border-l-indigo-500\/80{border-left-color:rgba(99,102,241,.8)}.border-l-indigo-500\/85{border-left-color:rgba(99,102,241,.85)}.border-l-indigo-500\/90{border-left-color:rgba(99,102,241,.9)}.border-l-indigo-500\/95{border-left-color:rgba(99,102,241,.95)}.border-l-indigo-950{--tw-border-opacity:1;border-left-color:rgb(30 27 75/var(--tw-border-opacity,1))}.border-l-indigo-950\/0{border-left-color:rgba(30,27,75,0)}.border-l-indigo-950\/10{border-left-color:rgba(30,27,75,.1)}.border-l-indigo-950\/100{border-left-color:#1e1b4b}.border-l-indigo-950\/15{border-left-color:rgba(30,27,75,.15)}.border-l-indigo-950\/20{border-left-color:rgba(30,27,75,.2)}.border-l-indigo-950\/25{border-left-color:rgba(30,27,75,.25)}.border-l-indigo-950\/30{border-left-color:rgba(30,27,75,.3)}.border-l-indigo-950\/35{border-left-color:rgba(30,27,75,.35)}.border-l-indigo-950\/40{border-left-color:rgba(30,27,75,.4)}.border-l-indigo-950\/45{border-left-color:rgba(30,27,75,.45)}.border-l-indigo-950\/5{border-left-color:rgba(30,27,75,.05)}.border-l-indigo-950\/50{border-left-color:rgba(30,27,75,.5)}.border-l-indigo-950\/55{border-left-color:rgba(30,27,75,.55)}.border-l-indigo-950\/60{border-left-color:rgba(30,27,75,.6)}.border-l-indigo-950\/65{border-left-color:rgba(30,27,75,.65)}.border-l-indigo-950\/70{border-left-color:rgba(30,27,75,.7)}.border-l-indigo-950\/75{border-left-color:rgba(30,27,75,.75)}.border-l-indigo-950\/80{border-left-color:rgba(30,27,75,.8)}.border-l-indigo-950\/85{border-left-color:rgba(30,27,75,.85)}.border-l-indigo-950\/90{border-left-color:rgba(30,27,75,.9)}.border-l-indigo-950\/95{border-left-color:rgba(30,27,75,.95)}.border-l-lime-100{--tw-border-opacity:1;border-left-color:rgb(236 252 203/var(--tw-border-opacity,1))}.border-l-lime-100\/0{border-left-color:hsla(80,89%,89%,0)}.border-l-lime-100\/10{border-left-color:hsla(80,89%,89%,.1)}.border-l-lime-100\/100{border-left-color:#ecfccb}.border-l-lime-100\/15{border-left-color:hsla(80,89%,89%,.15)}.border-l-lime-100\/20{border-left-color:hsla(80,89%,89%,.2)}.border-l-lime-100\/25{border-left-color:hsla(80,89%,89%,.25)}.border-l-lime-100\/30{border-left-color:hsla(80,89%,89%,.3)}.border-l-lime-100\/35{border-left-color:hsla(80,89%,89%,.35)}.border-l-lime-100\/40{border-left-color:hsla(80,89%,89%,.4)}.border-l-lime-100\/45{border-left-color:hsla(80,89%,89%,.45)}.border-l-lime-100\/5{border-left-color:hsla(80,89%,89%,.05)}.border-l-lime-100\/50{border-left-color:hsla(80,89%,89%,.5)}.border-l-lime-100\/55{border-left-color:hsla(80,89%,89%,.55)}.border-l-lime-100\/60{border-left-color:hsla(80,89%,89%,.6)}.border-l-lime-100\/65{border-left-color:hsla(80,89%,89%,.65)}.border-l-lime-100\/70{border-left-color:hsla(80,89%,89%,.7)}.border-l-lime-100\/75{border-left-color:hsla(80,89%,89%,.75)}.border-l-lime-100\/80{border-left-color:hsla(80,89%,89%,.8)}.border-l-lime-100\/85{border-left-color:hsla(80,89%,89%,.85)}.border-l-lime-100\/90{border-left-color:hsla(80,89%,89%,.9)}.border-l-lime-100\/95{border-left-color:hsla(80,89%,89%,.95)}.border-l-lime-200{--tw-border-opacity:1;border-left-color:rgb(217 249 157/var(--tw-border-opacity,1))}.border-l-lime-200\/0{border-left-color:hsla(81,88%,80%,0)}.border-l-lime-200\/10{border-left-color:hsla(81,88%,80%,.1)}.border-l-lime-200\/100{border-left-color:#d9f99d}.border-l-lime-200\/15{border-left-color:hsla(81,88%,80%,.15)}.border-l-lime-200\/20{border-left-color:hsla(81,88%,80%,.2)}.border-l-lime-200\/25{border-left-color:hsla(81,88%,80%,.25)}.border-l-lime-200\/30{border-left-color:hsla(81,88%,80%,.3)}.border-l-lime-200\/35{border-left-color:hsla(81,88%,80%,.35)}.border-l-lime-200\/40{border-left-color:hsla(81,88%,80%,.4)}.border-l-lime-200\/45{border-left-color:hsla(81,88%,80%,.45)}.border-l-lime-200\/5{border-left-color:hsla(81,88%,80%,.05)}.border-l-lime-200\/50{border-left-color:hsla(81,88%,80%,.5)}.border-l-lime-200\/55{border-left-color:hsla(81,88%,80%,.55)}.border-l-lime-200\/60{border-left-color:hsla(81,88%,80%,.6)}.border-l-lime-200\/65{border-left-color:hsla(81,88%,80%,.65)}.border-l-lime-200\/70{border-left-color:hsla(81,88%,80%,.7)}.border-l-lime-200\/75{border-left-color:hsla(81,88%,80%,.75)}.border-l-lime-200\/80{border-left-color:hsla(81,88%,80%,.8)}.border-l-lime-200\/85{border-left-color:hsla(81,88%,80%,.85)}.border-l-lime-200\/90{border-left-color:hsla(81,88%,80%,.9)}.border-l-lime-200\/95{border-left-color:hsla(81,88%,80%,.95)}.border-l-lime-400{--tw-border-opacity:1;border-left-color:rgb(163 230 53/var(--tw-border-opacity,1))}.border-l-lime-400\/0{border-left-color:rgba(163,230,53,0)}.border-l-lime-400\/10{border-left-color:rgba(163,230,53,.1)}.border-l-lime-400\/100{border-left-color:#a3e635}.border-l-lime-400\/15{border-left-color:rgba(163,230,53,.15)}.border-l-lime-400\/20{border-left-color:rgba(163,230,53,.2)}.border-l-lime-400\/25{border-left-color:rgba(163,230,53,.25)}.border-l-lime-400\/30{border-left-color:rgba(163,230,53,.3)}.border-l-lime-400\/35{border-left-color:rgba(163,230,53,.35)}.border-l-lime-400\/40{border-left-color:rgba(163,230,53,.4)}.border-l-lime-400\/45{border-left-color:rgba(163,230,53,.45)}.border-l-lime-400\/5{border-left-color:rgba(163,230,53,.05)}.border-l-lime-400\/50{border-left-color:rgba(163,230,53,.5)}.border-l-lime-400\/55{border-left-color:rgba(163,230,53,.55)}.border-l-lime-400\/60{border-left-color:rgba(163,230,53,.6)}.border-l-lime-400\/65{border-left-color:rgba(163,230,53,.65)}.border-l-lime-400\/70{border-left-color:rgba(163,230,53,.7)}.border-l-lime-400\/75{border-left-color:rgba(163,230,53,.75)}.border-l-lime-400\/80{border-left-color:rgba(163,230,53,.8)}.border-l-lime-400\/85{border-left-color:rgba(163,230,53,.85)}.border-l-lime-400\/90{border-left-color:rgba(163,230,53,.9)}.border-l-lime-400\/95{border-left-color:rgba(163,230,53,.95)}.border-l-lime-50{--tw-border-opacity:1;border-left-color:rgb(247 254 231/var(--tw-border-opacity,1))}.border-l-lime-50\/0{border-left-color:hsla(78,92%,95%,0)}.border-l-lime-50\/10{border-left-color:hsla(78,92%,95%,.1)}.border-l-lime-50\/100{border-left-color:#f7fee7}.border-l-lime-50\/15{border-left-color:hsla(78,92%,95%,.15)}.border-l-lime-50\/20{border-left-color:hsla(78,92%,95%,.2)}.border-l-lime-50\/25{border-left-color:hsla(78,92%,95%,.25)}.border-l-lime-50\/30{border-left-color:hsla(78,92%,95%,.3)}.border-l-lime-50\/35{border-left-color:hsla(78,92%,95%,.35)}.border-l-lime-50\/40{border-left-color:hsla(78,92%,95%,.4)}.border-l-lime-50\/45{border-left-color:hsla(78,92%,95%,.45)}.border-l-lime-50\/5{border-left-color:hsla(78,92%,95%,.05)}.border-l-lime-50\/50{border-left-color:hsla(78,92%,95%,.5)}.border-l-lime-50\/55{border-left-color:hsla(78,92%,95%,.55)}.border-l-lime-50\/60{border-left-color:hsla(78,92%,95%,.6)}.border-l-lime-50\/65{border-left-color:hsla(78,92%,95%,.65)}.border-l-lime-50\/70{border-left-color:hsla(78,92%,95%,.7)}.border-l-lime-50\/75{border-left-color:hsla(78,92%,95%,.75)}.border-l-lime-50\/80{border-left-color:hsla(78,92%,95%,.8)}.border-l-lime-50\/85{border-left-color:hsla(78,92%,95%,.85)}.border-l-lime-50\/90{border-left-color:hsla(78,92%,95%,.9)}.border-l-lime-50\/95{border-left-color:hsla(78,92%,95%,.95)}.border-l-lime-500{--tw-border-opacity:1;border-left-color:rgb(132 204 22/var(--tw-border-opacity,1))}.border-l-lime-500\/0{border-left-color:rgba(132,204,22,0)}.border-l-lime-500\/10{border-left-color:rgba(132,204,22,.1)}.border-l-lime-500\/100{border-left-color:#84cc16}.border-l-lime-500\/15{border-left-color:rgba(132,204,22,.15)}.border-l-lime-500\/20{border-left-color:rgba(132,204,22,.2)}.border-l-lime-500\/25{border-left-color:rgba(132,204,22,.25)}.border-l-lime-500\/30{border-left-color:rgba(132,204,22,.3)}.border-l-lime-500\/35{border-left-color:rgba(132,204,22,.35)}.border-l-lime-500\/40{border-left-color:rgba(132,204,22,.4)}.border-l-lime-500\/45{border-left-color:rgba(132,204,22,.45)}.border-l-lime-500\/5{border-left-color:rgba(132,204,22,.05)}.border-l-lime-500\/50{border-left-color:rgba(132,204,22,.5)}.border-l-lime-500\/55{border-left-color:rgba(132,204,22,.55)}.border-l-lime-500\/60{border-left-color:rgba(132,204,22,.6)}.border-l-lime-500\/65{border-left-color:rgba(132,204,22,.65)}.border-l-lime-500\/70{border-left-color:rgba(132,204,22,.7)}.border-l-lime-500\/75{border-left-color:rgba(132,204,22,.75)}.border-l-lime-500\/80{border-left-color:rgba(132,204,22,.8)}.border-l-lime-500\/85{border-left-color:rgba(132,204,22,.85)}.border-l-lime-500\/90{border-left-color:rgba(132,204,22,.9)}.border-l-lime-500\/95{border-left-color:rgba(132,204,22,.95)}.border-l-lime-950{--tw-border-opacity:1;border-left-color:rgb(26 46 5/var(--tw-border-opacity,1))}.border-l-lime-950\/0{border-left-color:rgba(26,46,5,0)}.border-l-lime-950\/10{border-left-color:rgba(26,46,5,.1)}.border-l-lime-950\/100{border-left-color:#1a2e05}.border-l-lime-950\/15{border-left-color:rgba(26,46,5,.15)}.border-l-lime-950\/20{border-left-color:rgba(26,46,5,.2)}.border-l-lime-950\/25{border-left-color:rgba(26,46,5,.25)}.border-l-lime-950\/30{border-left-color:rgba(26,46,5,.3)}.border-l-lime-950\/35{border-left-color:rgba(26,46,5,.35)}.border-l-lime-950\/40{border-left-color:rgba(26,46,5,.4)}.border-l-lime-950\/45{border-left-color:rgba(26,46,5,.45)}.border-l-lime-950\/5{border-left-color:rgba(26,46,5,.05)}.border-l-lime-950\/50{border-left-color:rgba(26,46,5,.5)}.border-l-lime-950\/55{border-left-color:rgba(26,46,5,.55)}.border-l-lime-950\/60{border-left-color:rgba(26,46,5,.6)}.border-l-lime-950\/65{border-left-color:rgba(26,46,5,.65)}.border-l-lime-950\/70{border-left-color:rgba(26,46,5,.7)}.border-l-lime-950\/75{border-left-color:rgba(26,46,5,.75)}.border-l-lime-950\/80{border-left-color:rgba(26,46,5,.8)}.border-l-lime-950\/85{border-left-color:rgba(26,46,5,.85)}.border-l-lime-950\/90{border-left-color:rgba(26,46,5,.9)}.border-l-lime-950\/95{border-left-color:rgba(26,46,5,.95)}.border-l-neutral-100{--tw-border-opacity:1;border-left-color:rgb(245 245 245/var(--tw-border-opacity,1))}.border-l-neutral-100\/0{border-left-color:hsla(0,0%,96%,0)}.border-l-neutral-100\/10{border-left-color:hsla(0,0%,96%,.1)}.border-l-neutral-100\/100{border-left-color:#f5f5f5}.border-l-neutral-100\/15{border-left-color:hsla(0,0%,96%,.15)}.border-l-neutral-100\/20{border-left-color:hsla(0,0%,96%,.2)}.border-l-neutral-100\/25{border-left-color:hsla(0,0%,96%,.25)}.border-l-neutral-100\/30{border-left-color:hsla(0,0%,96%,.3)}.border-l-neutral-100\/35{border-left-color:hsla(0,0%,96%,.35)}.border-l-neutral-100\/40{border-left-color:hsla(0,0%,96%,.4)}.border-l-neutral-100\/45{border-left-color:hsla(0,0%,96%,.45)}.border-l-neutral-100\/5{border-left-color:hsla(0,0%,96%,.05)}.border-l-neutral-100\/50{border-left-color:hsla(0,0%,96%,.5)}.border-l-neutral-100\/55{border-left-color:hsla(0,0%,96%,.55)}.border-l-neutral-100\/60{border-left-color:hsla(0,0%,96%,.6)}.border-l-neutral-100\/65{border-left-color:hsla(0,0%,96%,.65)}.border-l-neutral-100\/70{border-left-color:hsla(0,0%,96%,.7)}.border-l-neutral-100\/75{border-left-color:hsla(0,0%,96%,.75)}.border-l-neutral-100\/80{border-left-color:hsla(0,0%,96%,.8)}.border-l-neutral-100\/85{border-left-color:hsla(0,0%,96%,.85)}.border-l-neutral-100\/90{border-left-color:hsla(0,0%,96%,.9)}.border-l-neutral-100\/95{border-left-color:hsla(0,0%,96%,.95)}.border-l-neutral-200{--tw-border-opacity:1;border-left-color:rgb(229 229 229/var(--tw-border-opacity,1))}.border-l-neutral-200\/0{border-left-color:hsla(0,0%,90%,0)}.border-l-neutral-200\/10{border-left-color:hsla(0,0%,90%,.1)}.border-l-neutral-200\/100{border-left-color:#e5e5e5}.border-l-neutral-200\/15{border-left-color:hsla(0,0%,90%,.15)}.border-l-neutral-200\/20{border-left-color:hsla(0,0%,90%,.2)}.border-l-neutral-200\/25{border-left-color:hsla(0,0%,90%,.25)}.border-l-neutral-200\/30{border-left-color:hsla(0,0%,90%,.3)}.border-l-neutral-200\/35{border-left-color:hsla(0,0%,90%,.35)}.border-l-neutral-200\/40{border-left-color:hsla(0,0%,90%,.4)}.border-l-neutral-200\/45{border-left-color:hsla(0,0%,90%,.45)}.border-l-neutral-200\/5{border-left-color:hsla(0,0%,90%,.05)}.border-l-neutral-200\/50{border-left-color:hsla(0,0%,90%,.5)}.border-l-neutral-200\/55{border-left-color:hsla(0,0%,90%,.55)}.border-l-neutral-200\/60{border-left-color:hsla(0,0%,90%,.6)}.border-l-neutral-200\/65{border-left-color:hsla(0,0%,90%,.65)}.border-l-neutral-200\/70{border-left-color:hsla(0,0%,90%,.7)}.border-l-neutral-200\/75{border-left-color:hsla(0,0%,90%,.75)}.border-l-neutral-200\/80{border-left-color:hsla(0,0%,90%,.8)}.border-l-neutral-200\/85{border-left-color:hsla(0,0%,90%,.85)}.border-l-neutral-200\/90{border-left-color:hsla(0,0%,90%,.9)}.border-l-neutral-200\/95{border-left-color:hsla(0,0%,90%,.95)}.border-l-neutral-400{--tw-border-opacity:1;border-left-color:rgb(163 163 163/var(--tw-border-opacity,1))}.border-l-neutral-400\/0{border-left-color:hsla(0,0%,64%,0)}.border-l-neutral-400\/10{border-left-color:hsla(0,0%,64%,.1)}.border-l-neutral-400\/100{border-left-color:#a3a3a3}.border-l-neutral-400\/15{border-left-color:hsla(0,0%,64%,.15)}.border-l-neutral-400\/20{border-left-color:hsla(0,0%,64%,.2)}.border-l-neutral-400\/25{border-left-color:hsla(0,0%,64%,.25)}.border-l-neutral-400\/30{border-left-color:hsla(0,0%,64%,.3)}.border-l-neutral-400\/35{border-left-color:hsla(0,0%,64%,.35)}.border-l-neutral-400\/40{border-left-color:hsla(0,0%,64%,.4)}.border-l-neutral-400\/45{border-left-color:hsla(0,0%,64%,.45)}.border-l-neutral-400\/5{border-left-color:hsla(0,0%,64%,.05)}.border-l-neutral-400\/50{border-left-color:hsla(0,0%,64%,.5)}.border-l-neutral-400\/55{border-left-color:hsla(0,0%,64%,.55)}.border-l-neutral-400\/60{border-left-color:hsla(0,0%,64%,.6)}.border-l-neutral-400\/65{border-left-color:hsla(0,0%,64%,.65)}.border-l-neutral-400\/70{border-left-color:hsla(0,0%,64%,.7)}.border-l-neutral-400\/75{border-left-color:hsla(0,0%,64%,.75)}.border-l-neutral-400\/80{border-left-color:hsla(0,0%,64%,.8)}.border-l-neutral-400\/85{border-left-color:hsla(0,0%,64%,.85)}.border-l-neutral-400\/90{border-left-color:hsla(0,0%,64%,.9)}.border-l-neutral-400\/95{border-left-color:hsla(0,0%,64%,.95)}.border-l-neutral-50{--tw-border-opacity:1;border-left-color:rgb(250 250 250/var(--tw-border-opacity,1))}.border-l-neutral-50\/0{border-left-color:hsla(0,0%,98%,0)}.border-l-neutral-50\/10{border-left-color:hsla(0,0%,98%,.1)}.border-l-neutral-50\/100{border-left-color:#fafafa}.border-l-neutral-50\/15{border-left-color:hsla(0,0%,98%,.15)}.border-l-neutral-50\/20{border-left-color:hsla(0,0%,98%,.2)}.border-l-neutral-50\/25{border-left-color:hsla(0,0%,98%,.25)}.border-l-neutral-50\/30{border-left-color:hsla(0,0%,98%,.3)}.border-l-neutral-50\/35{border-left-color:hsla(0,0%,98%,.35)}.border-l-neutral-50\/40{border-left-color:hsla(0,0%,98%,.4)}.border-l-neutral-50\/45{border-left-color:hsla(0,0%,98%,.45)}.border-l-neutral-50\/5{border-left-color:hsla(0,0%,98%,.05)}.border-l-neutral-50\/50{border-left-color:hsla(0,0%,98%,.5)}.border-l-neutral-50\/55{border-left-color:hsla(0,0%,98%,.55)}.border-l-neutral-50\/60{border-left-color:hsla(0,0%,98%,.6)}.border-l-neutral-50\/65{border-left-color:hsla(0,0%,98%,.65)}.border-l-neutral-50\/70{border-left-color:hsla(0,0%,98%,.7)}.border-l-neutral-50\/75{border-left-color:hsla(0,0%,98%,.75)}.border-l-neutral-50\/80{border-left-color:hsla(0,0%,98%,.8)}.border-l-neutral-50\/85{border-left-color:hsla(0,0%,98%,.85)}.border-l-neutral-50\/90{border-left-color:hsla(0,0%,98%,.9)}.border-l-neutral-50\/95{border-left-color:hsla(0,0%,98%,.95)}.border-l-neutral-500{--tw-border-opacity:1;border-left-color:rgb(115 115 115/var(--tw-border-opacity,1))}.border-l-neutral-500\/0{border-left-color:hsla(0,0%,45%,0)}.border-l-neutral-500\/10{border-left-color:hsla(0,0%,45%,.1)}.border-l-neutral-500\/100{border-left-color:#737373}.border-l-neutral-500\/15{border-left-color:hsla(0,0%,45%,.15)}.border-l-neutral-500\/20{border-left-color:hsla(0,0%,45%,.2)}.border-l-neutral-500\/25{border-left-color:hsla(0,0%,45%,.25)}.border-l-neutral-500\/30{border-left-color:hsla(0,0%,45%,.3)}.border-l-neutral-500\/35{border-left-color:hsla(0,0%,45%,.35)}.border-l-neutral-500\/40{border-left-color:hsla(0,0%,45%,.4)}.border-l-neutral-500\/45{border-left-color:hsla(0,0%,45%,.45)}.border-l-neutral-500\/5{border-left-color:hsla(0,0%,45%,.05)}.border-l-neutral-500\/50{border-left-color:hsla(0,0%,45%,.5)}.border-l-neutral-500\/55{border-left-color:hsla(0,0%,45%,.55)}.border-l-neutral-500\/60{border-left-color:hsla(0,0%,45%,.6)}.border-l-neutral-500\/65{border-left-color:hsla(0,0%,45%,.65)}.border-l-neutral-500\/70{border-left-color:hsla(0,0%,45%,.7)}.border-l-neutral-500\/75{border-left-color:hsla(0,0%,45%,.75)}.border-l-neutral-500\/80{border-left-color:hsla(0,0%,45%,.8)}.border-l-neutral-500\/85{border-left-color:hsla(0,0%,45%,.85)}.border-l-neutral-500\/90{border-left-color:hsla(0,0%,45%,.9)}.border-l-neutral-500\/95{border-left-color:hsla(0,0%,45%,.95)}.border-l-neutral-950{--tw-border-opacity:1;border-left-color:rgb(10 10 10/var(--tw-border-opacity,1))}.border-l-neutral-950\/0{border-left-color:hsla(0,0%,4%,0)}.border-l-neutral-950\/10{border-left-color:hsla(0,0%,4%,.1)}.border-l-neutral-950\/100{border-left-color:#0a0a0a}.border-l-neutral-950\/15{border-left-color:hsla(0,0%,4%,.15)}.border-l-neutral-950\/20{border-left-color:hsla(0,0%,4%,.2)}.border-l-neutral-950\/25{border-left-color:hsla(0,0%,4%,.25)}.border-l-neutral-950\/30{border-left-color:hsla(0,0%,4%,.3)}.border-l-neutral-950\/35{border-left-color:hsla(0,0%,4%,.35)}.border-l-neutral-950\/40{border-left-color:hsla(0,0%,4%,.4)}.border-l-neutral-950\/45{border-left-color:hsla(0,0%,4%,.45)}.border-l-neutral-950\/5{border-left-color:hsla(0,0%,4%,.05)}.border-l-neutral-950\/50{border-left-color:hsla(0,0%,4%,.5)}.border-l-neutral-950\/55{border-left-color:hsla(0,0%,4%,.55)}.border-l-neutral-950\/60{border-left-color:hsla(0,0%,4%,.6)}.border-l-neutral-950\/65{border-left-color:hsla(0,0%,4%,.65)}.border-l-neutral-950\/70{border-left-color:hsla(0,0%,4%,.7)}.border-l-neutral-950\/75{border-left-color:hsla(0,0%,4%,.75)}.border-l-neutral-950\/80{border-left-color:hsla(0,0%,4%,.8)}.border-l-neutral-950\/85{border-left-color:hsla(0,0%,4%,.85)}.border-l-neutral-950\/90{border-left-color:hsla(0,0%,4%,.9)}.border-l-neutral-950\/95{border-left-color:hsla(0,0%,4%,.95)}.border-l-orange-100{--tw-border-opacity:1;border-left-color:rgb(255 237 213/var(--tw-border-opacity,1))}.border-l-orange-100\/0{border-left-color:rgba(255,237,213,0)}.border-l-orange-100\/10{border-left-color:rgba(255,237,213,.1)}.border-l-orange-100\/100{border-left-color:#ffedd5}.border-l-orange-100\/15{border-left-color:rgba(255,237,213,.15)}.border-l-orange-100\/20{border-left-color:rgba(255,237,213,.2)}.border-l-orange-100\/25{border-left-color:rgba(255,237,213,.25)}.border-l-orange-100\/30{border-left-color:rgba(255,237,213,.3)}.border-l-orange-100\/35{border-left-color:rgba(255,237,213,.35)}.border-l-orange-100\/40{border-left-color:rgba(255,237,213,.4)}.border-l-orange-100\/45{border-left-color:rgba(255,237,213,.45)}.border-l-orange-100\/5{border-left-color:rgba(255,237,213,.05)}.border-l-orange-100\/50{border-left-color:rgba(255,237,213,.5)}.border-l-orange-100\/55{border-left-color:rgba(255,237,213,.55)}.border-l-orange-100\/60{border-left-color:rgba(255,237,213,.6)}.border-l-orange-100\/65{border-left-color:rgba(255,237,213,.65)}.border-l-orange-100\/70{border-left-color:rgba(255,237,213,.7)}.border-l-orange-100\/75{border-left-color:rgba(255,237,213,.75)}.border-l-orange-100\/80{border-left-color:rgba(255,237,213,.8)}.border-l-orange-100\/85{border-left-color:rgba(255,237,213,.85)}.border-l-orange-100\/90{border-left-color:rgba(255,237,213,.9)}.border-l-orange-100\/95{border-left-color:rgba(255,237,213,.95)}.border-l-orange-200{--tw-border-opacity:1;border-left-color:rgb(254 215 170/var(--tw-border-opacity,1))}.border-l-orange-200\/0{border-left-color:hsla(32,98%,83%,0)}.border-l-orange-200\/10{border-left-color:hsla(32,98%,83%,.1)}.border-l-orange-200\/100{border-left-color:#fed7aa}.border-l-orange-200\/15{border-left-color:hsla(32,98%,83%,.15)}.border-l-orange-200\/20{border-left-color:hsla(32,98%,83%,.2)}.border-l-orange-200\/25{border-left-color:hsla(32,98%,83%,.25)}.border-l-orange-200\/30{border-left-color:hsla(32,98%,83%,.3)}.border-l-orange-200\/35{border-left-color:hsla(32,98%,83%,.35)}.border-l-orange-200\/40{border-left-color:hsla(32,98%,83%,.4)}.border-l-orange-200\/45{border-left-color:hsla(32,98%,83%,.45)}.border-l-orange-200\/5{border-left-color:hsla(32,98%,83%,.05)}.border-l-orange-200\/50{border-left-color:hsla(32,98%,83%,.5)}.border-l-orange-200\/55{border-left-color:hsla(32,98%,83%,.55)}.border-l-orange-200\/60{border-left-color:hsla(32,98%,83%,.6)}.border-l-orange-200\/65{border-left-color:hsla(32,98%,83%,.65)}.border-l-orange-200\/70{border-left-color:hsla(32,98%,83%,.7)}.border-l-orange-200\/75{border-left-color:hsla(32,98%,83%,.75)}.border-l-orange-200\/80{border-left-color:hsla(32,98%,83%,.8)}.border-l-orange-200\/85{border-left-color:hsla(32,98%,83%,.85)}.border-l-orange-200\/90{border-left-color:hsla(32,98%,83%,.9)}.border-l-orange-200\/95{border-left-color:hsla(32,98%,83%,.95)}.border-l-orange-400{--tw-border-opacity:1;border-left-color:rgb(251 146 60/var(--tw-border-opacity,1))}.border-l-orange-400\/0{border-left-color:rgba(251,146,60,0)}.border-l-orange-400\/10{border-left-color:rgba(251,146,60,.1)}.border-l-orange-400\/100{border-left-color:#fb923c}.border-l-orange-400\/15{border-left-color:rgba(251,146,60,.15)}.border-l-orange-400\/20{border-left-color:rgba(251,146,60,.2)}.border-l-orange-400\/25{border-left-color:rgba(251,146,60,.25)}.border-l-orange-400\/30{border-left-color:rgba(251,146,60,.3)}.border-l-orange-400\/35{border-left-color:rgba(251,146,60,.35)}.border-l-orange-400\/40{border-left-color:rgba(251,146,60,.4)}.border-l-orange-400\/45{border-left-color:rgba(251,146,60,.45)}.border-l-orange-400\/5{border-left-color:rgba(251,146,60,.05)}.border-l-orange-400\/50{border-left-color:rgba(251,146,60,.5)}.border-l-orange-400\/55{border-left-color:rgba(251,146,60,.55)}.border-l-orange-400\/60{border-left-color:rgba(251,146,60,.6)}.border-l-orange-400\/65{border-left-color:rgba(251,146,60,.65)}.border-l-orange-400\/70{border-left-color:rgba(251,146,60,.7)}.border-l-orange-400\/75{border-left-color:rgba(251,146,60,.75)}.border-l-orange-400\/80{border-left-color:rgba(251,146,60,.8)}.border-l-orange-400\/85{border-left-color:rgba(251,146,60,.85)}.border-l-orange-400\/90{border-left-color:rgba(251,146,60,.9)}.border-l-orange-400\/95{border-left-color:rgba(251,146,60,.95)}.border-l-orange-50{--tw-border-opacity:1;border-left-color:rgb(255 247 237/var(--tw-border-opacity,1))}.border-l-orange-50\/0{border-left-color:rgba(255,247,237,0)}.border-l-orange-50\/10{border-left-color:rgba(255,247,237,.1)}.border-l-orange-50\/100{border-left-color:#fff7ed}.border-l-orange-50\/15{border-left-color:rgba(255,247,237,.15)}.border-l-orange-50\/20{border-left-color:rgba(255,247,237,.2)}.border-l-orange-50\/25{border-left-color:rgba(255,247,237,.25)}.border-l-orange-50\/30{border-left-color:rgba(255,247,237,.3)}.border-l-orange-50\/35{border-left-color:rgba(255,247,237,.35)}.border-l-orange-50\/40{border-left-color:rgba(255,247,237,.4)}.border-l-orange-50\/45{border-left-color:rgba(255,247,237,.45)}.border-l-orange-50\/5{border-left-color:rgba(255,247,237,.05)}.border-l-orange-50\/50{border-left-color:rgba(255,247,237,.5)}.border-l-orange-50\/55{border-left-color:rgba(255,247,237,.55)}.border-l-orange-50\/60{border-left-color:rgba(255,247,237,.6)}.border-l-orange-50\/65{border-left-color:rgba(255,247,237,.65)}.border-l-orange-50\/70{border-left-color:rgba(255,247,237,.7)}.border-l-orange-50\/75{border-left-color:rgba(255,247,237,.75)}.border-l-orange-50\/80{border-left-color:rgba(255,247,237,.8)}.border-l-orange-50\/85{border-left-color:rgba(255,247,237,.85)}.border-l-orange-50\/90{border-left-color:rgba(255,247,237,.9)}.border-l-orange-50\/95{border-left-color:rgba(255,247,237,.95)}.border-l-orange-500{--tw-border-opacity:1;border-left-color:rgb(249 115 22/var(--tw-border-opacity,1))}.border-l-orange-500\/0{border-left-color:rgba(249,115,22,0)}.border-l-orange-500\/10{border-left-color:rgba(249,115,22,.1)}.border-l-orange-500\/100{border-left-color:#f97316}.border-l-orange-500\/15{border-left-color:rgba(249,115,22,.15)}.border-l-orange-500\/20{border-left-color:rgba(249,115,22,.2)}.border-l-orange-500\/25{border-left-color:rgba(249,115,22,.25)}.border-l-orange-500\/30{border-left-color:rgba(249,115,22,.3)}.border-l-orange-500\/35{border-left-color:rgba(249,115,22,.35)}.border-l-orange-500\/40{border-left-color:rgba(249,115,22,.4)}.border-l-orange-500\/45{border-left-color:rgba(249,115,22,.45)}.border-l-orange-500\/5{border-left-color:rgba(249,115,22,.05)}.border-l-orange-500\/50{border-left-color:rgba(249,115,22,.5)}.border-l-orange-500\/55{border-left-color:rgba(249,115,22,.55)}.border-l-orange-500\/60{border-left-color:rgba(249,115,22,.6)}.border-l-orange-500\/65{border-left-color:rgba(249,115,22,.65)}.border-l-orange-500\/70{border-left-color:rgba(249,115,22,.7)}.border-l-orange-500\/75{border-left-color:rgba(249,115,22,.75)}.border-l-orange-500\/80{border-left-color:rgba(249,115,22,.8)}.border-l-orange-500\/85{border-left-color:rgba(249,115,22,.85)}.border-l-orange-500\/90{border-left-color:rgba(249,115,22,.9)}.border-l-orange-500\/95{border-left-color:rgba(249,115,22,.95)}.border-l-orange-950{--tw-border-opacity:1;border-left-color:rgb(67 20 7/var(--tw-border-opacity,1))}.border-l-orange-950\/0{border-left-color:rgba(67,20,7,0)}.border-l-orange-950\/10{border-left-color:rgba(67,20,7,.1)}.border-l-orange-950\/100{border-left-color:#431407}.border-l-orange-950\/15{border-left-color:rgba(67,20,7,.15)}.border-l-orange-950\/20{border-left-color:rgba(67,20,7,.2)}.border-l-orange-950\/25{border-left-color:rgba(67,20,7,.25)}.border-l-orange-950\/30{border-left-color:rgba(67,20,7,.3)}.border-l-orange-950\/35{border-left-color:rgba(67,20,7,.35)}.border-l-orange-950\/40{border-left-color:rgba(67,20,7,.4)}.border-l-orange-950\/45{border-left-color:rgba(67,20,7,.45)}.border-l-orange-950\/5{border-left-color:rgba(67,20,7,.05)}.border-l-orange-950\/50{border-left-color:rgba(67,20,7,.5)}.border-l-orange-950\/55{border-left-color:rgba(67,20,7,.55)}.border-l-orange-950\/60{border-left-color:rgba(67,20,7,.6)}.border-l-orange-950\/65{border-left-color:rgba(67,20,7,.65)}.border-l-orange-950\/70{border-left-color:rgba(67,20,7,.7)}.border-l-orange-950\/75{border-left-color:rgba(67,20,7,.75)}.border-l-orange-950\/80{border-left-color:rgba(67,20,7,.8)}.border-l-orange-950\/85{border-left-color:rgba(67,20,7,.85)}.border-l-orange-950\/90{border-left-color:rgba(67,20,7,.9)}.border-l-orange-950\/95{border-left-color:rgba(67,20,7,.95)}.border-l-pink-100{--tw-border-opacity:1;border-left-color:rgb(252 231 243/var(--tw-border-opacity,1))}.border-l-pink-100\/0{border-left-color:rgba(252,231,243,0)}.border-l-pink-100\/10{border-left-color:rgba(252,231,243,.1)}.border-l-pink-100\/100{border-left-color:#fce7f3}.border-l-pink-100\/15{border-left-color:rgba(252,231,243,.15)}.border-l-pink-100\/20{border-left-color:rgba(252,231,243,.2)}.border-l-pink-100\/25{border-left-color:rgba(252,231,243,.25)}.border-l-pink-100\/30{border-left-color:rgba(252,231,243,.3)}.border-l-pink-100\/35{border-left-color:rgba(252,231,243,.35)}.border-l-pink-100\/40{border-left-color:rgba(252,231,243,.4)}.border-l-pink-100\/45{border-left-color:rgba(252,231,243,.45)}.border-l-pink-100\/5{border-left-color:rgba(252,231,243,.05)}.border-l-pink-100\/50{border-left-color:rgba(252,231,243,.5)}.border-l-pink-100\/55{border-left-color:rgba(252,231,243,.55)}.border-l-pink-100\/60{border-left-color:rgba(252,231,243,.6)}.border-l-pink-100\/65{border-left-color:rgba(252,231,243,.65)}.border-l-pink-100\/70{border-left-color:rgba(252,231,243,.7)}.border-l-pink-100\/75{border-left-color:rgba(252,231,243,.75)}.border-l-pink-100\/80{border-left-color:rgba(252,231,243,.8)}.border-l-pink-100\/85{border-left-color:rgba(252,231,243,.85)}.border-l-pink-100\/90{border-left-color:rgba(252,231,243,.9)}.border-l-pink-100\/95{border-left-color:rgba(252,231,243,.95)}.border-l-pink-200{--tw-border-opacity:1;border-left-color:rgb(251 207 232/var(--tw-border-opacity,1))}.border-l-pink-200\/0{border-left-color:rgba(251,207,232,0)}.border-l-pink-200\/10{border-left-color:rgba(251,207,232,.1)}.border-l-pink-200\/100{border-left-color:#fbcfe8}.border-l-pink-200\/15{border-left-color:rgba(251,207,232,.15)}.border-l-pink-200\/20{border-left-color:rgba(251,207,232,.2)}.border-l-pink-200\/25{border-left-color:rgba(251,207,232,.25)}.border-l-pink-200\/30{border-left-color:rgba(251,207,232,.3)}.border-l-pink-200\/35{border-left-color:rgba(251,207,232,.35)}.border-l-pink-200\/40{border-left-color:rgba(251,207,232,.4)}.border-l-pink-200\/45{border-left-color:rgba(251,207,232,.45)}.border-l-pink-200\/5{border-left-color:rgba(251,207,232,.05)}.border-l-pink-200\/50{border-left-color:rgba(251,207,232,.5)}.border-l-pink-200\/55{border-left-color:rgba(251,207,232,.55)}.border-l-pink-200\/60{border-left-color:rgba(251,207,232,.6)}.border-l-pink-200\/65{border-left-color:rgba(251,207,232,.65)}.border-l-pink-200\/70{border-left-color:rgba(251,207,232,.7)}.border-l-pink-200\/75{border-left-color:rgba(251,207,232,.75)}.border-l-pink-200\/80{border-left-color:rgba(251,207,232,.8)}.border-l-pink-200\/85{border-left-color:rgba(251,207,232,.85)}.border-l-pink-200\/90{border-left-color:rgba(251,207,232,.9)}.border-l-pink-200\/95{border-left-color:rgba(251,207,232,.95)}.border-l-pink-400{--tw-border-opacity:1;border-left-color:rgb(244 114 182/var(--tw-border-opacity,1))}.border-l-pink-400\/0{border-left-color:rgba(244,114,182,0)}.border-l-pink-400\/10{border-left-color:rgba(244,114,182,.1)}.border-l-pink-400\/100{border-left-color:#f472b6}.border-l-pink-400\/15{border-left-color:rgba(244,114,182,.15)}.border-l-pink-400\/20{border-left-color:rgba(244,114,182,.2)}.border-l-pink-400\/25{border-left-color:rgba(244,114,182,.25)}.border-l-pink-400\/30{border-left-color:rgba(244,114,182,.3)}.border-l-pink-400\/35{border-left-color:rgba(244,114,182,.35)}.border-l-pink-400\/40{border-left-color:rgba(244,114,182,.4)}.border-l-pink-400\/45{border-left-color:rgba(244,114,182,.45)}.border-l-pink-400\/5{border-left-color:rgba(244,114,182,.05)}.border-l-pink-400\/50{border-left-color:rgba(244,114,182,.5)}.border-l-pink-400\/55{border-left-color:rgba(244,114,182,.55)}.border-l-pink-400\/60{border-left-color:rgba(244,114,182,.6)}.border-l-pink-400\/65{border-left-color:rgba(244,114,182,.65)}.border-l-pink-400\/70{border-left-color:rgba(244,114,182,.7)}.border-l-pink-400\/75{border-left-color:rgba(244,114,182,.75)}.border-l-pink-400\/80{border-left-color:rgba(244,114,182,.8)}.border-l-pink-400\/85{border-left-color:rgba(244,114,182,.85)}.border-l-pink-400\/90{border-left-color:rgba(244,114,182,.9)}.border-l-pink-400\/95{border-left-color:rgba(244,114,182,.95)}.border-l-pink-50{--tw-border-opacity:1;border-left-color:rgb(253 242 248/var(--tw-border-opacity,1))}.border-l-pink-50\/0{border-left-color:rgba(253,242,248,0)}.border-l-pink-50\/10{border-left-color:rgba(253,242,248,.1)}.border-l-pink-50\/100{border-left-color:#fdf2f8}.border-l-pink-50\/15{border-left-color:rgba(253,242,248,.15)}.border-l-pink-50\/20{border-left-color:rgba(253,242,248,.2)}.border-l-pink-50\/25{border-left-color:rgba(253,242,248,.25)}.border-l-pink-50\/30{border-left-color:rgba(253,242,248,.3)}.border-l-pink-50\/35{border-left-color:rgba(253,242,248,.35)}.border-l-pink-50\/40{border-left-color:rgba(253,242,248,.4)}.border-l-pink-50\/45{border-left-color:rgba(253,242,248,.45)}.border-l-pink-50\/5{border-left-color:rgba(253,242,248,.05)}.border-l-pink-50\/50{border-left-color:rgba(253,242,248,.5)}.border-l-pink-50\/55{border-left-color:rgba(253,242,248,.55)}.border-l-pink-50\/60{border-left-color:rgba(253,242,248,.6)}.border-l-pink-50\/65{border-left-color:rgba(253,242,248,.65)}.border-l-pink-50\/70{border-left-color:rgba(253,242,248,.7)}.border-l-pink-50\/75{border-left-color:rgba(253,242,248,.75)}.border-l-pink-50\/80{border-left-color:rgba(253,242,248,.8)}.border-l-pink-50\/85{border-left-color:rgba(253,242,248,.85)}.border-l-pink-50\/90{border-left-color:rgba(253,242,248,.9)}.border-l-pink-50\/95{border-left-color:rgba(253,242,248,.95)}.border-l-pink-500{--tw-border-opacity:1;border-left-color:rgb(236 72 153/var(--tw-border-opacity,1))}.border-l-pink-500\/0{border-left-color:rgba(236,72,153,0)}.border-l-pink-500\/10{border-left-color:rgba(236,72,153,.1)}.border-l-pink-500\/100{border-left-color:#ec4899}.border-l-pink-500\/15{border-left-color:rgba(236,72,153,.15)}.border-l-pink-500\/20{border-left-color:rgba(236,72,153,.2)}.border-l-pink-500\/25{border-left-color:rgba(236,72,153,.25)}.border-l-pink-500\/30{border-left-color:rgba(236,72,153,.3)}.border-l-pink-500\/35{border-left-color:rgba(236,72,153,.35)}.border-l-pink-500\/40{border-left-color:rgba(236,72,153,.4)}.border-l-pink-500\/45{border-left-color:rgba(236,72,153,.45)}.border-l-pink-500\/5{border-left-color:rgba(236,72,153,.05)}.border-l-pink-500\/50{border-left-color:rgba(236,72,153,.5)}.border-l-pink-500\/55{border-left-color:rgba(236,72,153,.55)}.border-l-pink-500\/60{border-left-color:rgba(236,72,153,.6)}.border-l-pink-500\/65{border-left-color:rgba(236,72,153,.65)}.border-l-pink-500\/70{border-left-color:rgba(236,72,153,.7)}.border-l-pink-500\/75{border-left-color:rgba(236,72,153,.75)}.border-l-pink-500\/80{border-left-color:rgba(236,72,153,.8)}.border-l-pink-500\/85{border-left-color:rgba(236,72,153,.85)}.border-l-pink-500\/90{border-left-color:rgba(236,72,153,.9)}.border-l-pink-500\/95{border-left-color:rgba(236,72,153,.95)}.border-l-pink-950{--tw-border-opacity:1;border-left-color:rgb(80 7 36/var(--tw-border-opacity,1))}.border-l-pink-950\/0{border-left-color:rgba(80,7,36,0)}.border-l-pink-950\/10{border-left-color:rgba(80,7,36,.1)}.border-l-pink-950\/100{border-left-color:#500724}.border-l-pink-950\/15{border-left-color:rgba(80,7,36,.15)}.border-l-pink-950\/20{border-left-color:rgba(80,7,36,.2)}.border-l-pink-950\/25{border-left-color:rgba(80,7,36,.25)}.border-l-pink-950\/30{border-left-color:rgba(80,7,36,.3)}.border-l-pink-950\/35{border-left-color:rgba(80,7,36,.35)}.border-l-pink-950\/40{border-left-color:rgba(80,7,36,.4)}.border-l-pink-950\/45{border-left-color:rgba(80,7,36,.45)}.border-l-pink-950\/5{border-left-color:rgba(80,7,36,.05)}.border-l-pink-950\/50{border-left-color:rgba(80,7,36,.5)}.border-l-pink-950\/55{border-left-color:rgba(80,7,36,.55)}.border-l-pink-950\/60{border-left-color:rgba(80,7,36,.6)}.border-l-pink-950\/65{border-left-color:rgba(80,7,36,.65)}.border-l-pink-950\/70{border-left-color:rgba(80,7,36,.7)}.border-l-pink-950\/75{border-left-color:rgba(80,7,36,.75)}.border-l-pink-950\/80{border-left-color:rgba(80,7,36,.8)}.border-l-pink-950\/85{border-left-color:rgba(80,7,36,.85)}.border-l-pink-950\/90{border-left-color:rgba(80,7,36,.9)}.border-l-pink-950\/95{border-left-color:rgba(80,7,36,.95)}.border-l-purple-100{--tw-border-opacity:1;border-left-color:rgb(243 232 255/var(--tw-border-opacity,1))}.border-l-purple-100\/0{border-left-color:rgba(243,232,255,0)}.border-l-purple-100\/10{border-left-color:rgba(243,232,255,.1)}.border-l-purple-100\/100{border-left-color:#f3e8ff}.border-l-purple-100\/15{border-left-color:rgba(243,232,255,.15)}.border-l-purple-100\/20{border-left-color:rgba(243,232,255,.2)}.border-l-purple-100\/25{border-left-color:rgba(243,232,255,.25)}.border-l-purple-100\/30{border-left-color:rgba(243,232,255,.3)}.border-l-purple-100\/35{border-left-color:rgba(243,232,255,.35)}.border-l-purple-100\/40{border-left-color:rgba(243,232,255,.4)}.border-l-purple-100\/45{border-left-color:rgba(243,232,255,.45)}.border-l-purple-100\/5{border-left-color:rgba(243,232,255,.05)}.border-l-purple-100\/50{border-left-color:rgba(243,232,255,.5)}.border-l-purple-100\/55{border-left-color:rgba(243,232,255,.55)}.border-l-purple-100\/60{border-left-color:rgba(243,232,255,.6)}.border-l-purple-100\/65{border-left-color:rgba(243,232,255,.65)}.border-l-purple-100\/70{border-left-color:rgba(243,232,255,.7)}.border-l-purple-100\/75{border-left-color:rgba(243,232,255,.75)}.border-l-purple-100\/80{border-left-color:rgba(243,232,255,.8)}.border-l-purple-100\/85{border-left-color:rgba(243,232,255,.85)}.border-l-purple-100\/90{border-left-color:rgba(243,232,255,.9)}.border-l-purple-100\/95{border-left-color:rgba(243,232,255,.95)}.border-l-purple-200{--tw-border-opacity:1;border-left-color:rgb(233 213 255/var(--tw-border-opacity,1))}.border-l-purple-200\/0{border-left-color:rgba(233,213,255,0)}.border-l-purple-200\/10{border-left-color:rgba(233,213,255,.1)}.border-l-purple-200\/100{border-left-color:#e9d5ff}.border-l-purple-200\/15{border-left-color:rgba(233,213,255,.15)}.border-l-purple-200\/20{border-left-color:rgba(233,213,255,.2)}.border-l-purple-200\/25{border-left-color:rgba(233,213,255,.25)}.border-l-purple-200\/30{border-left-color:rgba(233,213,255,.3)}.border-l-purple-200\/35{border-left-color:rgba(233,213,255,.35)}.border-l-purple-200\/40{border-left-color:rgba(233,213,255,.4)}.border-l-purple-200\/45{border-left-color:rgba(233,213,255,.45)}.border-l-purple-200\/5{border-left-color:rgba(233,213,255,.05)}.border-l-purple-200\/50{border-left-color:rgba(233,213,255,.5)}.border-l-purple-200\/55{border-left-color:rgba(233,213,255,.55)}.border-l-purple-200\/60{border-left-color:rgba(233,213,255,.6)}.border-l-purple-200\/65{border-left-color:rgba(233,213,255,.65)}.border-l-purple-200\/70{border-left-color:rgba(233,213,255,.7)}.border-l-purple-200\/75{border-left-color:rgba(233,213,255,.75)}.border-l-purple-200\/80{border-left-color:rgba(233,213,255,.8)}.border-l-purple-200\/85{border-left-color:rgba(233,213,255,.85)}.border-l-purple-200\/90{border-left-color:rgba(233,213,255,.9)}.border-l-purple-200\/95{border-left-color:rgba(233,213,255,.95)}.border-l-purple-400{--tw-border-opacity:1;border-left-color:rgb(192 132 252/var(--tw-border-opacity,1))}.border-l-purple-400\/0{border-left-color:rgba(192,132,252,0)}.border-l-purple-400\/10{border-left-color:rgba(192,132,252,.1)}.border-l-purple-400\/100{border-left-color:#c084fc}.border-l-purple-400\/15{border-left-color:rgba(192,132,252,.15)}.border-l-purple-400\/20{border-left-color:rgba(192,132,252,.2)}.border-l-purple-400\/25{border-left-color:rgba(192,132,252,.25)}.border-l-purple-400\/30{border-left-color:rgba(192,132,252,.3)}.border-l-purple-400\/35{border-left-color:rgba(192,132,252,.35)}.border-l-purple-400\/40{border-left-color:rgba(192,132,252,.4)}.border-l-purple-400\/45{border-left-color:rgba(192,132,252,.45)}.border-l-purple-400\/5{border-left-color:rgba(192,132,252,.05)}.border-l-purple-400\/50{border-left-color:rgba(192,132,252,.5)}.border-l-purple-400\/55{border-left-color:rgba(192,132,252,.55)}.border-l-purple-400\/60{border-left-color:rgba(192,132,252,.6)}.border-l-purple-400\/65{border-left-color:rgba(192,132,252,.65)}.border-l-purple-400\/70{border-left-color:rgba(192,132,252,.7)}.border-l-purple-400\/75{border-left-color:rgba(192,132,252,.75)}.border-l-purple-400\/80{border-left-color:rgba(192,132,252,.8)}.border-l-purple-400\/85{border-left-color:rgba(192,132,252,.85)}.border-l-purple-400\/90{border-left-color:rgba(192,132,252,.9)}.border-l-purple-400\/95{border-left-color:rgba(192,132,252,.95)}.border-l-purple-50{--tw-border-opacity:1;border-left-color:rgb(250 245 255/var(--tw-border-opacity,1))}.border-l-purple-50\/0{border-left-color:rgba(250,245,255,0)}.border-l-purple-50\/10{border-left-color:rgba(250,245,255,.1)}.border-l-purple-50\/100{border-left-color:#faf5ff}.border-l-purple-50\/15{border-left-color:rgba(250,245,255,.15)}.border-l-purple-50\/20{border-left-color:rgba(250,245,255,.2)}.border-l-purple-50\/25{border-left-color:rgba(250,245,255,.25)}.border-l-purple-50\/30{border-left-color:rgba(250,245,255,.3)}.border-l-purple-50\/35{border-left-color:rgba(250,245,255,.35)}.border-l-purple-50\/40{border-left-color:rgba(250,245,255,.4)}.border-l-purple-50\/45{border-left-color:rgba(250,245,255,.45)}.border-l-purple-50\/5{border-left-color:rgba(250,245,255,.05)}.border-l-purple-50\/50{border-left-color:rgba(250,245,255,.5)}.border-l-purple-50\/55{border-left-color:rgba(250,245,255,.55)}.border-l-purple-50\/60{border-left-color:rgba(250,245,255,.6)}.border-l-purple-50\/65{border-left-color:rgba(250,245,255,.65)}.border-l-purple-50\/70{border-left-color:rgba(250,245,255,.7)}.border-l-purple-50\/75{border-left-color:rgba(250,245,255,.75)}.border-l-purple-50\/80{border-left-color:rgba(250,245,255,.8)}.border-l-purple-50\/85{border-left-color:rgba(250,245,255,.85)}.border-l-purple-50\/90{border-left-color:rgba(250,245,255,.9)}.border-l-purple-50\/95{border-left-color:rgba(250,245,255,.95)}.border-l-purple-500{--tw-border-opacity:1;border-left-color:rgb(168 85 247/var(--tw-border-opacity,1))}.border-l-purple-500\/0{border-left-color:rgba(168,85,247,0)}.border-l-purple-500\/10{border-left-color:rgba(168,85,247,.1)}.border-l-purple-500\/100{border-left-color:#a855f7}.border-l-purple-500\/15{border-left-color:rgba(168,85,247,.15)}.border-l-purple-500\/20{border-left-color:rgba(168,85,247,.2)}.border-l-purple-500\/25{border-left-color:rgba(168,85,247,.25)}.border-l-purple-500\/30{border-left-color:rgba(168,85,247,.3)}.border-l-purple-500\/35{border-left-color:rgba(168,85,247,.35)}.border-l-purple-500\/40{border-left-color:rgba(168,85,247,.4)}.border-l-purple-500\/45{border-left-color:rgba(168,85,247,.45)}.border-l-purple-500\/5{border-left-color:rgba(168,85,247,.05)}.border-l-purple-500\/50{border-left-color:rgba(168,85,247,.5)}.border-l-purple-500\/55{border-left-color:rgba(168,85,247,.55)}.border-l-purple-500\/60{border-left-color:rgba(168,85,247,.6)}.border-l-purple-500\/65{border-left-color:rgba(168,85,247,.65)}.border-l-purple-500\/70{border-left-color:rgba(168,85,247,.7)}.border-l-purple-500\/75{border-left-color:rgba(168,85,247,.75)}.border-l-purple-500\/80{border-left-color:rgba(168,85,247,.8)}.border-l-purple-500\/85{border-left-color:rgba(168,85,247,.85)}.border-l-purple-500\/90{border-left-color:rgba(168,85,247,.9)}.border-l-purple-500\/95{border-left-color:rgba(168,85,247,.95)}.border-l-purple-950{--tw-border-opacity:1;border-left-color:rgb(59 7 100/var(--tw-border-opacity,1))}.border-l-purple-950\/0{border-left-color:rgba(59,7,100,0)}.border-l-purple-950\/10{border-left-color:rgba(59,7,100,.1)}.border-l-purple-950\/100{border-left-color:#3b0764}.border-l-purple-950\/15{border-left-color:rgba(59,7,100,.15)}.border-l-purple-950\/20{border-left-color:rgba(59,7,100,.2)}.border-l-purple-950\/25{border-left-color:rgba(59,7,100,.25)}.border-l-purple-950\/30{border-left-color:rgba(59,7,100,.3)}.border-l-purple-950\/35{border-left-color:rgba(59,7,100,.35)}.border-l-purple-950\/40{border-left-color:rgba(59,7,100,.4)}.border-l-purple-950\/45{border-left-color:rgba(59,7,100,.45)}.border-l-purple-950\/5{border-left-color:rgba(59,7,100,.05)}.border-l-purple-950\/50{border-left-color:rgba(59,7,100,.5)}.border-l-purple-950\/55{border-left-color:rgba(59,7,100,.55)}.border-l-purple-950\/60{border-left-color:rgba(59,7,100,.6)}.border-l-purple-950\/65{border-left-color:rgba(59,7,100,.65)}.border-l-purple-950\/70{border-left-color:rgba(59,7,100,.7)}.border-l-purple-950\/75{border-left-color:rgba(59,7,100,.75)}.border-l-purple-950\/80{border-left-color:rgba(59,7,100,.8)}.border-l-purple-950\/85{border-left-color:rgba(59,7,100,.85)}.border-l-purple-950\/90{border-left-color:rgba(59,7,100,.9)}.border-l-purple-950\/95{border-left-color:rgba(59,7,100,.95)}.border-l-red-100{--tw-border-opacity:1;border-left-color:rgb(254 226 226/var(--tw-border-opacity,1))}.border-l-red-100\/0{border-left-color:hsla(0,93%,94%,0)}.border-l-red-100\/10{border-left-color:hsla(0,93%,94%,.1)}.border-l-red-100\/100{border-left-color:#fee2e2}.border-l-red-100\/15{border-left-color:hsla(0,93%,94%,.15)}.border-l-red-100\/20{border-left-color:hsla(0,93%,94%,.2)}.border-l-red-100\/25{border-left-color:hsla(0,93%,94%,.25)}.border-l-red-100\/30{border-left-color:hsla(0,93%,94%,.3)}.border-l-red-100\/35{border-left-color:hsla(0,93%,94%,.35)}.border-l-red-100\/40{border-left-color:hsla(0,93%,94%,.4)}.border-l-red-100\/45{border-left-color:hsla(0,93%,94%,.45)}.border-l-red-100\/5{border-left-color:hsla(0,93%,94%,.05)}.border-l-red-100\/50{border-left-color:hsla(0,93%,94%,.5)}.border-l-red-100\/55{border-left-color:hsla(0,93%,94%,.55)}.border-l-red-100\/60{border-left-color:hsla(0,93%,94%,.6)}.border-l-red-100\/65{border-left-color:hsla(0,93%,94%,.65)}.border-l-red-100\/70{border-left-color:hsla(0,93%,94%,.7)}.border-l-red-100\/75{border-left-color:hsla(0,93%,94%,.75)}.border-l-red-100\/80{border-left-color:hsla(0,93%,94%,.8)}.border-l-red-100\/85{border-left-color:hsla(0,93%,94%,.85)}.border-l-red-100\/90{border-left-color:hsla(0,93%,94%,.9)}.border-l-red-100\/95{border-left-color:hsla(0,93%,94%,.95)}.border-l-red-200{--tw-border-opacity:1;border-left-color:rgb(254 202 202/var(--tw-border-opacity,1))}.border-l-red-200\/0{border-left-color:hsla(0,96%,89%,0)}.border-l-red-200\/10{border-left-color:hsla(0,96%,89%,.1)}.border-l-red-200\/100{border-left-color:#fecaca}.border-l-red-200\/15{border-left-color:hsla(0,96%,89%,.15)}.border-l-red-200\/20{border-left-color:hsla(0,96%,89%,.2)}.border-l-red-200\/25{border-left-color:hsla(0,96%,89%,.25)}.border-l-red-200\/30{border-left-color:hsla(0,96%,89%,.3)}.border-l-red-200\/35{border-left-color:hsla(0,96%,89%,.35)}.border-l-red-200\/40{border-left-color:hsla(0,96%,89%,.4)}.border-l-red-200\/45{border-left-color:hsla(0,96%,89%,.45)}.border-l-red-200\/5{border-left-color:hsla(0,96%,89%,.05)}.border-l-red-200\/50{border-left-color:hsla(0,96%,89%,.5)}.border-l-red-200\/55{border-left-color:hsla(0,96%,89%,.55)}.border-l-red-200\/60{border-left-color:hsla(0,96%,89%,.6)}.border-l-red-200\/65{border-left-color:hsla(0,96%,89%,.65)}.border-l-red-200\/70{border-left-color:hsla(0,96%,89%,.7)}.border-l-red-200\/75{border-left-color:hsla(0,96%,89%,.75)}.border-l-red-200\/80{border-left-color:hsla(0,96%,89%,.8)}.border-l-red-200\/85{border-left-color:hsla(0,96%,89%,.85)}.border-l-red-200\/90{border-left-color:hsla(0,96%,89%,.9)}.border-l-red-200\/95{border-left-color:hsla(0,96%,89%,.95)}.border-l-red-400{--tw-border-opacity:1;border-left-color:rgb(248 113 113/var(--tw-border-opacity,1))}.border-l-red-400\/0{border-left-color:hsla(0,91%,71%,0)}.border-l-red-400\/10{border-left-color:hsla(0,91%,71%,.1)}.border-l-red-400\/100{border-left-color:#f87171}.border-l-red-400\/15{border-left-color:hsla(0,91%,71%,.15)}.border-l-red-400\/20{border-left-color:hsla(0,91%,71%,.2)}.border-l-red-400\/25{border-left-color:hsla(0,91%,71%,.25)}.border-l-red-400\/30{border-left-color:hsla(0,91%,71%,.3)}.border-l-red-400\/35{border-left-color:hsla(0,91%,71%,.35)}.border-l-red-400\/40{border-left-color:hsla(0,91%,71%,.4)}.border-l-red-400\/45{border-left-color:hsla(0,91%,71%,.45)}.border-l-red-400\/5{border-left-color:hsla(0,91%,71%,.05)}.border-l-red-400\/50{border-left-color:hsla(0,91%,71%,.5)}.border-l-red-400\/55{border-left-color:hsla(0,91%,71%,.55)}.border-l-red-400\/60{border-left-color:hsla(0,91%,71%,.6)}.border-l-red-400\/65{border-left-color:hsla(0,91%,71%,.65)}.border-l-red-400\/70{border-left-color:hsla(0,91%,71%,.7)}.border-l-red-400\/75{border-left-color:hsla(0,91%,71%,.75)}.border-l-red-400\/80{border-left-color:hsla(0,91%,71%,.8)}.border-l-red-400\/85{border-left-color:hsla(0,91%,71%,.85)}.border-l-red-400\/90{border-left-color:hsla(0,91%,71%,.9)}.border-l-red-400\/95{border-left-color:hsla(0,91%,71%,.95)}.border-l-red-50{--tw-border-opacity:1;border-left-color:rgb(254 242 242/var(--tw-border-opacity,1))}.border-l-red-50\/0{border-left-color:hsla(0,86%,97%,0)}.border-l-red-50\/10{border-left-color:hsla(0,86%,97%,.1)}.border-l-red-50\/100{border-left-color:#fef2f2}.border-l-red-50\/15{border-left-color:hsla(0,86%,97%,.15)}.border-l-red-50\/20{border-left-color:hsla(0,86%,97%,.2)}.border-l-red-50\/25{border-left-color:hsla(0,86%,97%,.25)}.border-l-red-50\/30{border-left-color:hsla(0,86%,97%,.3)}.border-l-red-50\/35{border-left-color:hsla(0,86%,97%,.35)}.border-l-red-50\/40{border-left-color:hsla(0,86%,97%,.4)}.border-l-red-50\/45{border-left-color:hsla(0,86%,97%,.45)}.border-l-red-50\/5{border-left-color:hsla(0,86%,97%,.05)}.border-l-red-50\/50{border-left-color:hsla(0,86%,97%,.5)}.border-l-red-50\/55{border-left-color:hsla(0,86%,97%,.55)}.border-l-red-50\/60{border-left-color:hsla(0,86%,97%,.6)}.border-l-red-50\/65{border-left-color:hsla(0,86%,97%,.65)}.border-l-red-50\/70{border-left-color:hsla(0,86%,97%,.7)}.border-l-red-50\/75{border-left-color:hsla(0,86%,97%,.75)}.border-l-red-50\/80{border-left-color:hsla(0,86%,97%,.8)}.border-l-red-50\/85{border-left-color:hsla(0,86%,97%,.85)}.border-l-red-50\/90{border-left-color:hsla(0,86%,97%,.9)}.border-l-red-50\/95{border-left-color:hsla(0,86%,97%,.95)}.border-l-red-500{--tw-border-opacity:1;border-left-color:rgb(239 68 68/var(--tw-border-opacity,1))}.border-l-red-500\/0{border-left-color:rgba(239,68,68,0)}.border-l-red-500\/10{border-left-color:rgba(239,68,68,.1)}.border-l-red-500\/100{border-left-color:#ef4444}.border-l-red-500\/15{border-left-color:rgba(239,68,68,.15)}.border-l-red-500\/20{border-left-color:rgba(239,68,68,.2)}.border-l-red-500\/25{border-left-color:rgba(239,68,68,.25)}.border-l-red-500\/30{border-left-color:rgba(239,68,68,.3)}.border-l-red-500\/35{border-left-color:rgba(239,68,68,.35)}.border-l-red-500\/40{border-left-color:rgba(239,68,68,.4)}.border-l-red-500\/45{border-left-color:rgba(239,68,68,.45)}.border-l-red-500\/5{border-left-color:rgba(239,68,68,.05)}.border-l-red-500\/50{border-left-color:rgba(239,68,68,.5)}.border-l-red-500\/55{border-left-color:rgba(239,68,68,.55)}.border-l-red-500\/60{border-left-color:rgba(239,68,68,.6)}.border-l-red-500\/65{border-left-color:rgba(239,68,68,.65)}.border-l-red-500\/70{border-left-color:rgba(239,68,68,.7)}.border-l-red-500\/75{border-left-color:rgba(239,68,68,.75)}.border-l-red-500\/80{border-left-color:rgba(239,68,68,.8)}.border-l-red-500\/85{border-left-color:rgba(239,68,68,.85)}.border-l-red-500\/90{border-left-color:rgba(239,68,68,.9)}.border-l-red-500\/95{border-left-color:rgba(239,68,68,.95)}.border-l-red-950{--tw-border-opacity:1;border-left-color:rgb(69 10 10/var(--tw-border-opacity,1))}.border-l-red-950\/0{border-left-color:rgba(69,10,10,0)}.border-l-red-950\/10{border-left-color:rgba(69,10,10,.1)}.border-l-red-950\/100{border-left-color:#450a0a}.border-l-red-950\/15{border-left-color:rgba(69,10,10,.15)}.border-l-red-950\/20{border-left-color:rgba(69,10,10,.2)}.border-l-red-950\/25{border-left-color:rgba(69,10,10,.25)}.border-l-red-950\/30{border-left-color:rgba(69,10,10,.3)}.border-l-red-950\/35{border-left-color:rgba(69,10,10,.35)}.border-l-red-950\/40{border-left-color:rgba(69,10,10,.4)}.border-l-red-950\/45{border-left-color:rgba(69,10,10,.45)}.border-l-red-950\/5{border-left-color:rgba(69,10,10,.05)}.border-l-red-950\/50{border-left-color:rgba(69,10,10,.5)}.border-l-red-950\/55{border-left-color:rgba(69,10,10,.55)}.border-l-red-950\/60{border-left-color:rgba(69,10,10,.6)}.border-l-red-950\/65{border-left-color:rgba(69,10,10,.65)}.border-l-red-950\/70{border-left-color:rgba(69,10,10,.7)}.border-l-red-950\/75{border-left-color:rgba(69,10,10,.75)}.border-l-red-950\/80{border-left-color:rgba(69,10,10,.8)}.border-l-red-950\/85{border-left-color:rgba(69,10,10,.85)}.border-l-red-950\/90{border-left-color:rgba(69,10,10,.9)}.border-l-red-950\/95{border-left-color:rgba(69,10,10,.95)}.border-l-rose-100{--tw-border-opacity:1;border-left-color:rgb(255 228 230/var(--tw-border-opacity,1))}.border-l-rose-100\/0{border-left-color:rgba(255,228,230,0)}.border-l-rose-100\/10{border-left-color:rgba(255,228,230,.1)}.border-l-rose-100\/100{border-left-color:#ffe4e6}.border-l-rose-100\/15{border-left-color:rgba(255,228,230,.15)}.border-l-rose-100\/20{border-left-color:rgba(255,228,230,.2)}.border-l-rose-100\/25{border-left-color:rgba(255,228,230,.25)}.border-l-rose-100\/30{border-left-color:rgba(255,228,230,.3)}.border-l-rose-100\/35{border-left-color:rgba(255,228,230,.35)}.border-l-rose-100\/40{border-left-color:rgba(255,228,230,.4)}.border-l-rose-100\/45{border-left-color:rgba(255,228,230,.45)}.border-l-rose-100\/5{border-left-color:rgba(255,228,230,.05)}.border-l-rose-100\/50{border-left-color:rgba(255,228,230,.5)}.border-l-rose-100\/55{border-left-color:rgba(255,228,230,.55)}.border-l-rose-100\/60{border-left-color:rgba(255,228,230,.6)}.border-l-rose-100\/65{border-left-color:rgba(255,228,230,.65)}.border-l-rose-100\/70{border-left-color:rgba(255,228,230,.7)}.border-l-rose-100\/75{border-left-color:rgba(255,228,230,.75)}.border-l-rose-100\/80{border-left-color:rgba(255,228,230,.8)}.border-l-rose-100\/85{border-left-color:rgba(255,228,230,.85)}.border-l-rose-100\/90{border-left-color:rgba(255,228,230,.9)}.border-l-rose-100\/95{border-left-color:rgba(255,228,230,.95)}.border-l-rose-200{--tw-border-opacity:1;border-left-color:rgb(254 205 211/var(--tw-border-opacity,1))}.border-l-rose-200\/0{border-left-color:rgba(254,205,211,0)}.border-l-rose-200\/10{border-left-color:rgba(254,205,211,.1)}.border-l-rose-200\/100{border-left-color:#fecdd3}.border-l-rose-200\/15{border-left-color:rgba(254,205,211,.15)}.border-l-rose-200\/20{border-left-color:rgba(254,205,211,.2)}.border-l-rose-200\/25{border-left-color:rgba(254,205,211,.25)}.border-l-rose-200\/30{border-left-color:rgba(254,205,211,.3)}.border-l-rose-200\/35{border-left-color:rgba(254,205,211,.35)}.border-l-rose-200\/40{border-left-color:rgba(254,205,211,.4)}.border-l-rose-200\/45{border-left-color:rgba(254,205,211,.45)}.border-l-rose-200\/5{border-left-color:rgba(254,205,211,.05)}.border-l-rose-200\/50{border-left-color:rgba(254,205,211,.5)}.border-l-rose-200\/55{border-left-color:rgba(254,205,211,.55)}.border-l-rose-200\/60{border-left-color:rgba(254,205,211,.6)}.border-l-rose-200\/65{border-left-color:rgba(254,205,211,.65)}.border-l-rose-200\/70{border-left-color:rgba(254,205,211,.7)}.border-l-rose-200\/75{border-left-color:rgba(254,205,211,.75)}.border-l-rose-200\/80{border-left-color:rgba(254,205,211,.8)}.border-l-rose-200\/85{border-left-color:rgba(254,205,211,.85)}.border-l-rose-200\/90{border-left-color:rgba(254,205,211,.9)}.border-l-rose-200\/95{border-left-color:rgba(254,205,211,.95)}.border-l-rose-400{--tw-border-opacity:1;border-left-color:rgb(251 113 133/var(--tw-border-opacity,1))}.border-l-rose-400\/0{border-left-color:rgba(251,113,133,0)}.border-l-rose-400\/10{border-left-color:rgba(251,113,133,.1)}.border-l-rose-400\/100{border-left-color:#fb7185}.border-l-rose-400\/15{border-left-color:rgba(251,113,133,.15)}.border-l-rose-400\/20{border-left-color:rgba(251,113,133,.2)}.border-l-rose-400\/25{border-left-color:rgba(251,113,133,.25)}.border-l-rose-400\/30{border-left-color:rgba(251,113,133,.3)}.border-l-rose-400\/35{border-left-color:rgba(251,113,133,.35)}.border-l-rose-400\/40{border-left-color:rgba(251,113,133,.4)}.border-l-rose-400\/45{border-left-color:rgba(251,113,133,.45)}.border-l-rose-400\/5{border-left-color:rgba(251,113,133,.05)}.border-l-rose-400\/50{border-left-color:rgba(251,113,133,.5)}.border-l-rose-400\/55{border-left-color:rgba(251,113,133,.55)}.border-l-rose-400\/60{border-left-color:rgba(251,113,133,.6)}.border-l-rose-400\/65{border-left-color:rgba(251,113,133,.65)}.border-l-rose-400\/70{border-left-color:rgba(251,113,133,.7)}.border-l-rose-400\/75{border-left-color:rgba(251,113,133,.75)}.border-l-rose-400\/80{border-left-color:rgba(251,113,133,.8)}.border-l-rose-400\/85{border-left-color:rgba(251,113,133,.85)}.border-l-rose-400\/90{border-left-color:rgba(251,113,133,.9)}.border-l-rose-400\/95{border-left-color:rgba(251,113,133,.95)}.border-l-rose-50{--tw-border-opacity:1;border-left-color:rgb(255 241 242/var(--tw-border-opacity,1))}.border-l-rose-50\/0{border-left-color:rgba(255,241,242,0)}.border-l-rose-50\/10{border-left-color:rgba(255,241,242,.1)}.border-l-rose-50\/100{border-left-color:#fff1f2}.border-l-rose-50\/15{border-left-color:rgba(255,241,242,.15)}.border-l-rose-50\/20{border-left-color:rgba(255,241,242,.2)}.border-l-rose-50\/25{border-left-color:rgba(255,241,242,.25)}.border-l-rose-50\/30{border-left-color:rgba(255,241,242,.3)}.border-l-rose-50\/35{border-left-color:rgba(255,241,242,.35)}.border-l-rose-50\/40{border-left-color:rgba(255,241,242,.4)}.border-l-rose-50\/45{border-left-color:rgba(255,241,242,.45)}.border-l-rose-50\/5{border-left-color:rgba(255,241,242,.05)}.border-l-rose-50\/50{border-left-color:rgba(255,241,242,.5)}.border-l-rose-50\/55{border-left-color:rgba(255,241,242,.55)}.border-l-rose-50\/60{border-left-color:rgba(255,241,242,.6)}.border-l-rose-50\/65{border-left-color:rgba(255,241,242,.65)}.border-l-rose-50\/70{border-left-color:rgba(255,241,242,.7)}.border-l-rose-50\/75{border-left-color:rgba(255,241,242,.75)}.border-l-rose-50\/80{border-left-color:rgba(255,241,242,.8)}.border-l-rose-50\/85{border-left-color:rgba(255,241,242,.85)}.border-l-rose-50\/90{border-left-color:rgba(255,241,242,.9)}.border-l-rose-50\/95{border-left-color:rgba(255,241,242,.95)}.border-l-rose-500{--tw-border-opacity:1;border-left-color:rgb(244 63 94/var(--tw-border-opacity,1))}.border-l-rose-500\/0{border-left-color:rgba(244,63,94,0)}.border-l-rose-500\/10{border-left-color:rgba(244,63,94,.1)}.border-l-rose-500\/100{border-left-color:#f43f5e}.border-l-rose-500\/15{border-left-color:rgba(244,63,94,.15)}.border-l-rose-500\/20{border-left-color:rgba(244,63,94,.2)}.border-l-rose-500\/25{border-left-color:rgba(244,63,94,.25)}.border-l-rose-500\/30{border-left-color:rgba(244,63,94,.3)}.border-l-rose-500\/35{border-left-color:rgba(244,63,94,.35)}.border-l-rose-500\/40{border-left-color:rgba(244,63,94,.4)}.border-l-rose-500\/45{border-left-color:rgba(244,63,94,.45)}.border-l-rose-500\/5{border-left-color:rgba(244,63,94,.05)}.border-l-rose-500\/50{border-left-color:rgba(244,63,94,.5)}.border-l-rose-500\/55{border-left-color:rgba(244,63,94,.55)}.border-l-rose-500\/60{border-left-color:rgba(244,63,94,.6)}.border-l-rose-500\/65{border-left-color:rgba(244,63,94,.65)}.border-l-rose-500\/70{border-left-color:rgba(244,63,94,.7)}.border-l-rose-500\/75{border-left-color:rgba(244,63,94,.75)}.border-l-rose-500\/80{border-left-color:rgba(244,63,94,.8)}.border-l-rose-500\/85{border-left-color:rgba(244,63,94,.85)}.border-l-rose-500\/90{border-left-color:rgba(244,63,94,.9)}.border-l-rose-500\/95{border-left-color:rgba(244,63,94,.95)}.border-l-rose-950{--tw-border-opacity:1;border-left-color:rgb(76 5 25/var(--tw-border-opacity,1))}.border-l-rose-950\/0{border-left-color:rgba(76,5,25,0)}.border-l-rose-950\/10{border-left-color:rgba(76,5,25,.1)}.border-l-rose-950\/100{border-left-color:#4c0519}.border-l-rose-950\/15{border-left-color:rgba(76,5,25,.15)}.border-l-rose-950\/20{border-left-color:rgba(76,5,25,.2)}.border-l-rose-950\/25{border-left-color:rgba(76,5,25,.25)}.border-l-rose-950\/30{border-left-color:rgba(76,5,25,.3)}.border-l-rose-950\/35{border-left-color:rgba(76,5,25,.35)}.border-l-rose-950\/40{border-left-color:rgba(76,5,25,.4)}.border-l-rose-950\/45{border-left-color:rgba(76,5,25,.45)}.border-l-rose-950\/5{border-left-color:rgba(76,5,25,.05)}.border-l-rose-950\/50{border-left-color:rgba(76,5,25,.5)}.border-l-rose-950\/55{border-left-color:rgba(76,5,25,.55)}.border-l-rose-950\/60{border-left-color:rgba(76,5,25,.6)}.border-l-rose-950\/65{border-left-color:rgba(76,5,25,.65)}.border-l-rose-950\/70{border-left-color:rgba(76,5,25,.7)}.border-l-rose-950\/75{border-left-color:rgba(76,5,25,.75)}.border-l-rose-950\/80{border-left-color:rgba(76,5,25,.8)}.border-l-rose-950\/85{border-left-color:rgba(76,5,25,.85)}.border-l-rose-950\/90{border-left-color:rgba(76,5,25,.9)}.border-l-rose-950\/95{border-left-color:rgba(76,5,25,.95)}.border-l-sky-100{--tw-border-opacity:1;border-left-color:rgb(224 242 254/var(--tw-border-opacity,1))}.border-l-sky-100\/0{border-left-color:rgba(224,242,254,0)}.border-l-sky-100\/10{border-left-color:rgba(224,242,254,.1)}.border-l-sky-100\/100{border-left-color:#e0f2fe}.border-l-sky-100\/15{border-left-color:rgba(224,242,254,.15)}.border-l-sky-100\/20{border-left-color:rgba(224,242,254,.2)}.border-l-sky-100\/25{border-left-color:rgba(224,242,254,.25)}.border-l-sky-100\/30{border-left-color:rgba(224,242,254,.3)}.border-l-sky-100\/35{border-left-color:rgba(224,242,254,.35)}.border-l-sky-100\/40{border-left-color:rgba(224,242,254,.4)}.border-l-sky-100\/45{border-left-color:rgba(224,242,254,.45)}.border-l-sky-100\/5{border-left-color:rgba(224,242,254,.05)}.border-l-sky-100\/50{border-left-color:rgba(224,242,254,.5)}.border-l-sky-100\/55{border-left-color:rgba(224,242,254,.55)}.border-l-sky-100\/60{border-left-color:rgba(224,242,254,.6)}.border-l-sky-100\/65{border-left-color:rgba(224,242,254,.65)}.border-l-sky-100\/70{border-left-color:rgba(224,242,254,.7)}.border-l-sky-100\/75{border-left-color:rgba(224,242,254,.75)}.border-l-sky-100\/80{border-left-color:rgba(224,242,254,.8)}.border-l-sky-100\/85{border-left-color:rgba(224,242,254,.85)}.border-l-sky-100\/90{border-left-color:rgba(224,242,254,.9)}.border-l-sky-100\/95{border-left-color:rgba(224,242,254,.95)}.border-l-sky-200{--tw-border-opacity:1;border-left-color:rgb(186 230 253/var(--tw-border-opacity,1))}.border-l-sky-200\/0{border-left-color:rgba(186,230,253,0)}.border-l-sky-200\/10{border-left-color:rgba(186,230,253,.1)}.border-l-sky-200\/100{border-left-color:#bae6fd}.border-l-sky-200\/15{border-left-color:rgba(186,230,253,.15)}.border-l-sky-200\/20{border-left-color:rgba(186,230,253,.2)}.border-l-sky-200\/25{border-left-color:rgba(186,230,253,.25)}.border-l-sky-200\/30{border-left-color:rgba(186,230,253,.3)}.border-l-sky-200\/35{border-left-color:rgba(186,230,253,.35)}.border-l-sky-200\/40{border-left-color:rgba(186,230,253,.4)}.border-l-sky-200\/45{border-left-color:rgba(186,230,253,.45)}.border-l-sky-200\/5{border-left-color:rgba(186,230,253,.05)}.border-l-sky-200\/50{border-left-color:rgba(186,230,253,.5)}.border-l-sky-200\/55{border-left-color:rgba(186,230,253,.55)}.border-l-sky-200\/60{border-left-color:rgba(186,230,253,.6)}.border-l-sky-200\/65{border-left-color:rgba(186,230,253,.65)}.border-l-sky-200\/70{border-left-color:rgba(186,230,253,.7)}.border-l-sky-200\/75{border-left-color:rgba(186,230,253,.75)}.border-l-sky-200\/80{border-left-color:rgba(186,230,253,.8)}.border-l-sky-200\/85{border-left-color:rgba(186,230,253,.85)}.border-l-sky-200\/90{border-left-color:rgba(186,230,253,.9)}.border-l-sky-200\/95{border-left-color:rgba(186,230,253,.95)}.border-l-sky-400{--tw-border-opacity:1;border-left-color:rgb(56 189 248/var(--tw-border-opacity,1))}.border-l-sky-400\/0{border-left-color:rgba(56,189,248,0)}.border-l-sky-400\/10{border-left-color:rgba(56,189,248,.1)}.border-l-sky-400\/100{border-left-color:#38bdf8}.border-l-sky-400\/15{border-left-color:rgba(56,189,248,.15)}.border-l-sky-400\/20{border-left-color:rgba(56,189,248,.2)}.border-l-sky-400\/25{border-left-color:rgba(56,189,248,.25)}.border-l-sky-400\/30{border-left-color:rgba(56,189,248,.3)}.border-l-sky-400\/35{border-left-color:rgba(56,189,248,.35)}.border-l-sky-400\/40{border-left-color:rgba(56,189,248,.4)}.border-l-sky-400\/45{border-left-color:rgba(56,189,248,.45)}.border-l-sky-400\/5{border-left-color:rgba(56,189,248,.05)}.border-l-sky-400\/50{border-left-color:rgba(56,189,248,.5)}.border-l-sky-400\/55{border-left-color:rgba(56,189,248,.55)}.border-l-sky-400\/60{border-left-color:rgba(56,189,248,.6)}.border-l-sky-400\/65{border-left-color:rgba(56,189,248,.65)}.border-l-sky-400\/70{border-left-color:rgba(56,189,248,.7)}.border-l-sky-400\/75{border-left-color:rgba(56,189,248,.75)}.border-l-sky-400\/80{border-left-color:rgba(56,189,248,.8)}.border-l-sky-400\/85{border-left-color:rgba(56,189,248,.85)}.border-l-sky-400\/90{border-left-color:rgba(56,189,248,.9)}.border-l-sky-400\/95{border-left-color:rgba(56,189,248,.95)}.border-l-sky-50{--tw-border-opacity:1;border-left-color:rgb(240 249 255/var(--tw-border-opacity,1))}.border-l-sky-50\/0{border-left-color:rgba(240,249,255,0)}.border-l-sky-50\/10{border-left-color:rgba(240,249,255,.1)}.border-l-sky-50\/100{border-left-color:#f0f9ff}.border-l-sky-50\/15{border-left-color:rgba(240,249,255,.15)}.border-l-sky-50\/20{border-left-color:rgba(240,249,255,.2)}.border-l-sky-50\/25{border-left-color:rgba(240,249,255,.25)}.border-l-sky-50\/30{border-left-color:rgba(240,249,255,.3)}.border-l-sky-50\/35{border-left-color:rgba(240,249,255,.35)}.border-l-sky-50\/40{border-left-color:rgba(240,249,255,.4)}.border-l-sky-50\/45{border-left-color:rgba(240,249,255,.45)}.border-l-sky-50\/5{border-left-color:rgba(240,249,255,.05)}.border-l-sky-50\/50{border-left-color:rgba(240,249,255,.5)}.border-l-sky-50\/55{border-left-color:rgba(240,249,255,.55)}.border-l-sky-50\/60{border-left-color:rgba(240,249,255,.6)}.border-l-sky-50\/65{border-left-color:rgba(240,249,255,.65)}.border-l-sky-50\/70{border-left-color:rgba(240,249,255,.7)}.border-l-sky-50\/75{border-left-color:rgba(240,249,255,.75)}.border-l-sky-50\/80{border-left-color:rgba(240,249,255,.8)}.border-l-sky-50\/85{border-left-color:rgba(240,249,255,.85)}.border-l-sky-50\/90{border-left-color:rgba(240,249,255,.9)}.border-l-sky-50\/95{border-left-color:rgba(240,249,255,.95)}.border-l-sky-500{--tw-border-opacity:1;border-left-color:rgb(14 165 233/var(--tw-border-opacity,1))}.border-l-sky-500\/0{border-left-color:rgba(14,165,233,0)}.border-l-sky-500\/10{border-left-color:rgba(14,165,233,.1)}.border-l-sky-500\/100{border-left-color:#0ea5e9}.border-l-sky-500\/15{border-left-color:rgba(14,165,233,.15)}.border-l-sky-500\/20{border-left-color:rgba(14,165,233,.2)}.border-l-sky-500\/25{border-left-color:rgba(14,165,233,.25)}.border-l-sky-500\/30{border-left-color:rgba(14,165,233,.3)}.border-l-sky-500\/35{border-left-color:rgba(14,165,233,.35)}.border-l-sky-500\/40{border-left-color:rgba(14,165,233,.4)}.border-l-sky-500\/45{border-left-color:rgba(14,165,233,.45)}.border-l-sky-500\/5{border-left-color:rgba(14,165,233,.05)}.border-l-sky-500\/50{border-left-color:rgba(14,165,233,.5)}.border-l-sky-500\/55{border-left-color:rgba(14,165,233,.55)}.border-l-sky-500\/60{border-left-color:rgba(14,165,233,.6)}.border-l-sky-500\/65{border-left-color:rgba(14,165,233,.65)}.border-l-sky-500\/70{border-left-color:rgba(14,165,233,.7)}.border-l-sky-500\/75{border-left-color:rgba(14,165,233,.75)}.border-l-sky-500\/80{border-left-color:rgba(14,165,233,.8)}.border-l-sky-500\/85{border-left-color:rgba(14,165,233,.85)}.border-l-sky-500\/90{border-left-color:rgba(14,165,233,.9)}.border-l-sky-500\/95{border-left-color:rgba(14,165,233,.95)}.border-l-sky-950{--tw-border-opacity:1;border-left-color:rgb(8 47 73/var(--tw-border-opacity,1))}.border-l-sky-950\/0{border-left-color:rgba(8,47,73,0)}.border-l-sky-950\/10{border-left-color:rgba(8,47,73,.1)}.border-l-sky-950\/100{border-left-color:#082f49}.border-l-sky-950\/15{border-left-color:rgba(8,47,73,.15)}.border-l-sky-950\/20{border-left-color:rgba(8,47,73,.2)}.border-l-sky-950\/25{border-left-color:rgba(8,47,73,.25)}.border-l-sky-950\/30{border-left-color:rgba(8,47,73,.3)}.border-l-sky-950\/35{border-left-color:rgba(8,47,73,.35)}.border-l-sky-950\/40{border-left-color:rgba(8,47,73,.4)}.border-l-sky-950\/45{border-left-color:rgba(8,47,73,.45)}.border-l-sky-950\/5{border-left-color:rgba(8,47,73,.05)}.border-l-sky-950\/50{border-left-color:rgba(8,47,73,.5)}.border-l-sky-950\/55{border-left-color:rgba(8,47,73,.55)}.border-l-sky-950\/60{border-left-color:rgba(8,47,73,.6)}.border-l-sky-950\/65{border-left-color:rgba(8,47,73,.65)}.border-l-sky-950\/70{border-left-color:rgba(8,47,73,.7)}.border-l-sky-950\/75{border-left-color:rgba(8,47,73,.75)}.border-l-sky-950\/80{border-left-color:rgba(8,47,73,.8)}.border-l-sky-950\/85{border-left-color:rgba(8,47,73,.85)}.border-l-sky-950\/90{border-left-color:rgba(8,47,73,.9)}.border-l-sky-950\/95{border-left-color:rgba(8,47,73,.95)}.border-l-slate-100{--tw-border-opacity:1;border-left-color:rgb(241 245 249/var(--tw-border-opacity,1))}.border-l-slate-100\/0{border-left-color:rgba(241,245,249,0)}.border-l-slate-100\/10{border-left-color:rgba(241,245,249,.1)}.border-l-slate-100\/100{border-left-color:#f1f5f9}.border-l-slate-100\/15{border-left-color:rgba(241,245,249,.15)}.border-l-slate-100\/20{border-left-color:rgba(241,245,249,.2)}.border-l-slate-100\/25{border-left-color:rgba(241,245,249,.25)}.border-l-slate-100\/30{border-left-color:rgba(241,245,249,.3)}.border-l-slate-100\/35{border-left-color:rgba(241,245,249,.35)}.border-l-slate-100\/40{border-left-color:rgba(241,245,249,.4)}.border-l-slate-100\/45{border-left-color:rgba(241,245,249,.45)}.border-l-slate-100\/5{border-left-color:rgba(241,245,249,.05)}.border-l-slate-100\/50{border-left-color:rgba(241,245,249,.5)}.border-l-slate-100\/55{border-left-color:rgba(241,245,249,.55)}.border-l-slate-100\/60{border-left-color:rgba(241,245,249,.6)}.border-l-slate-100\/65{border-left-color:rgba(241,245,249,.65)}.border-l-slate-100\/70{border-left-color:rgba(241,245,249,.7)}.border-l-slate-100\/75{border-left-color:rgba(241,245,249,.75)}.border-l-slate-100\/80{border-left-color:rgba(241,245,249,.8)}.border-l-slate-100\/85{border-left-color:rgba(241,245,249,.85)}.border-l-slate-100\/90{border-left-color:rgba(241,245,249,.9)}.border-l-slate-100\/95{border-left-color:rgba(241,245,249,.95)}.border-l-slate-200{--tw-border-opacity:1;border-left-color:rgb(226 232 240/var(--tw-border-opacity,1))}.border-l-slate-200\/0{border-left-color:rgba(226,232,240,0)}.border-l-slate-200\/10{border-left-color:rgba(226,232,240,.1)}.border-l-slate-200\/100{border-left-color:#e2e8f0}.border-l-slate-200\/15{border-left-color:rgba(226,232,240,.15)}.border-l-slate-200\/20{border-left-color:rgba(226,232,240,.2)}.border-l-slate-200\/25{border-left-color:rgba(226,232,240,.25)}.border-l-slate-200\/30{border-left-color:rgba(226,232,240,.3)}.border-l-slate-200\/35{border-left-color:rgba(226,232,240,.35)}.border-l-slate-200\/40{border-left-color:rgba(226,232,240,.4)}.border-l-slate-200\/45{border-left-color:rgba(226,232,240,.45)}.border-l-slate-200\/5{border-left-color:rgba(226,232,240,.05)}.border-l-slate-200\/50{border-left-color:rgba(226,232,240,.5)}.border-l-slate-200\/55{border-left-color:rgba(226,232,240,.55)}.border-l-slate-200\/60{border-left-color:rgba(226,232,240,.6)}.border-l-slate-200\/65{border-left-color:rgba(226,232,240,.65)}.border-l-slate-200\/70{border-left-color:rgba(226,232,240,.7)}.border-l-slate-200\/75{border-left-color:rgba(226,232,240,.75)}.border-l-slate-200\/80{border-left-color:rgba(226,232,240,.8)}.border-l-slate-200\/85{border-left-color:rgba(226,232,240,.85)}.border-l-slate-200\/90{border-left-color:rgba(226,232,240,.9)}.border-l-slate-200\/95{border-left-color:rgba(226,232,240,.95)}.border-l-slate-400{--tw-border-opacity:1;border-left-color:rgb(148 163 184/var(--tw-border-opacity,1))}.border-l-slate-400\/0{border-left-color:rgba(148,163,184,0)}.border-l-slate-400\/10{border-left-color:rgba(148,163,184,.1)}.border-l-slate-400\/100{border-left-color:#94a3b8}.border-l-slate-400\/15{border-left-color:rgba(148,163,184,.15)}.border-l-slate-400\/20{border-left-color:rgba(148,163,184,.2)}.border-l-slate-400\/25{border-left-color:rgba(148,163,184,.25)}.border-l-slate-400\/30{border-left-color:rgba(148,163,184,.3)}.border-l-slate-400\/35{border-left-color:rgba(148,163,184,.35)}.border-l-slate-400\/40{border-left-color:rgba(148,163,184,.4)}.border-l-slate-400\/45{border-left-color:rgba(148,163,184,.45)}.border-l-slate-400\/5{border-left-color:rgba(148,163,184,.05)}.border-l-slate-400\/50{border-left-color:rgba(148,163,184,.5)}.border-l-slate-400\/55{border-left-color:rgba(148,163,184,.55)}.border-l-slate-400\/60{border-left-color:rgba(148,163,184,.6)}.border-l-slate-400\/65{border-left-color:rgba(148,163,184,.65)}.border-l-slate-400\/70{border-left-color:rgba(148,163,184,.7)}.border-l-slate-400\/75{border-left-color:rgba(148,163,184,.75)}.border-l-slate-400\/80{border-left-color:rgba(148,163,184,.8)}.border-l-slate-400\/85{border-left-color:rgba(148,163,184,.85)}.border-l-slate-400\/90{border-left-color:rgba(148,163,184,.9)}.border-l-slate-400\/95{border-left-color:rgba(148,163,184,.95)}.border-l-slate-50{--tw-border-opacity:1;border-left-color:rgb(248 250 252/var(--tw-border-opacity,1))}.border-l-slate-50\/0{border-left-color:rgba(248,250,252,0)}.border-l-slate-50\/10{border-left-color:rgba(248,250,252,.1)}.border-l-slate-50\/100{border-left-color:#f8fafc}.border-l-slate-50\/15{border-left-color:rgba(248,250,252,.15)}.border-l-slate-50\/20{border-left-color:rgba(248,250,252,.2)}.border-l-slate-50\/25{border-left-color:rgba(248,250,252,.25)}.border-l-slate-50\/30{border-left-color:rgba(248,250,252,.3)}.border-l-slate-50\/35{border-left-color:rgba(248,250,252,.35)}.border-l-slate-50\/40{border-left-color:rgba(248,250,252,.4)}.border-l-slate-50\/45{border-left-color:rgba(248,250,252,.45)}.border-l-slate-50\/5{border-left-color:rgba(248,250,252,.05)}.border-l-slate-50\/50{border-left-color:rgba(248,250,252,.5)}.border-l-slate-50\/55{border-left-color:rgba(248,250,252,.55)}.border-l-slate-50\/60{border-left-color:rgba(248,250,252,.6)}.border-l-slate-50\/65{border-left-color:rgba(248,250,252,.65)}.border-l-slate-50\/70{border-left-color:rgba(248,250,252,.7)}.border-l-slate-50\/75{border-left-color:rgba(248,250,252,.75)}.border-l-slate-50\/80{border-left-color:rgba(248,250,252,.8)}.border-l-slate-50\/85{border-left-color:rgba(248,250,252,.85)}.border-l-slate-50\/90{border-left-color:rgba(248,250,252,.9)}.border-l-slate-50\/95{border-left-color:rgba(248,250,252,.95)}.border-l-slate-500{--tw-border-opacity:1;border-left-color:rgb(100 116 139/var(--tw-border-opacity,1))}.border-l-slate-500\/0{border-left-color:rgba(100,116,139,0)}.border-l-slate-500\/10{border-left-color:rgba(100,116,139,.1)}.border-l-slate-500\/100{border-left-color:#64748b}.border-l-slate-500\/15{border-left-color:rgba(100,116,139,.15)}.border-l-slate-500\/20{border-left-color:rgba(100,116,139,.2)}.border-l-slate-500\/25{border-left-color:rgba(100,116,139,.25)}.border-l-slate-500\/30{border-left-color:rgba(100,116,139,.3)}.border-l-slate-500\/35{border-left-color:rgba(100,116,139,.35)}.border-l-slate-500\/40{border-left-color:rgba(100,116,139,.4)}.border-l-slate-500\/45{border-left-color:rgba(100,116,139,.45)}.border-l-slate-500\/5{border-left-color:rgba(100,116,139,.05)}.border-l-slate-500\/50{border-left-color:rgba(100,116,139,.5)}.border-l-slate-500\/55{border-left-color:rgba(100,116,139,.55)}.border-l-slate-500\/60{border-left-color:rgba(100,116,139,.6)}.border-l-slate-500\/65{border-left-color:rgba(100,116,139,.65)}.border-l-slate-500\/70{border-left-color:rgba(100,116,139,.7)}.border-l-slate-500\/75{border-left-color:rgba(100,116,139,.75)}.border-l-slate-500\/80{border-left-color:rgba(100,116,139,.8)}.border-l-slate-500\/85{border-left-color:rgba(100,116,139,.85)}.border-l-slate-500\/90{border-left-color:rgba(100,116,139,.9)}.border-l-slate-500\/95{border-left-color:rgba(100,116,139,.95)}.border-l-slate-950{--tw-border-opacity:1;border-left-color:rgb(2 6 23/var(--tw-border-opacity,1))}.border-l-slate-950\/0{border-left-color:rgba(2,6,23,0)}.border-l-slate-950\/10{border-left-color:rgba(2,6,23,.1)}.border-l-slate-950\/100{border-left-color:#020617}.border-l-slate-950\/15{border-left-color:rgba(2,6,23,.15)}.border-l-slate-950\/20{border-left-color:rgba(2,6,23,.2)}.border-l-slate-950\/25{border-left-color:rgba(2,6,23,.25)}.border-l-slate-950\/30{border-left-color:rgba(2,6,23,.3)}.border-l-slate-950\/35{border-left-color:rgba(2,6,23,.35)}.border-l-slate-950\/40{border-left-color:rgba(2,6,23,.4)}.border-l-slate-950\/45{border-left-color:rgba(2,6,23,.45)}.border-l-slate-950\/5{border-left-color:rgba(2,6,23,.05)}.border-l-slate-950\/50{border-left-color:rgba(2,6,23,.5)}.border-l-slate-950\/55{border-left-color:rgba(2,6,23,.55)}.border-l-slate-950\/60{border-left-color:rgba(2,6,23,.6)}.border-l-slate-950\/65{border-left-color:rgba(2,6,23,.65)}.border-l-slate-950\/70{border-left-color:rgba(2,6,23,.7)}.border-l-slate-950\/75{border-left-color:rgba(2,6,23,.75)}.border-l-slate-950\/80{border-left-color:rgba(2,6,23,.8)}.border-l-slate-950\/85{border-left-color:rgba(2,6,23,.85)}.border-l-slate-950\/90{border-left-color:rgba(2,6,23,.9)}.border-l-slate-950\/95{border-left-color:rgba(2,6,23,.95)}.border-l-stone-100{--tw-border-opacity:1;border-left-color:rgb(245 245 244/var(--tw-border-opacity,1))}.border-l-stone-100\/0{border-left-color:hsla(60,5%,96%,0)}.border-l-stone-100\/10{border-left-color:hsla(60,5%,96%,.1)}.border-l-stone-100\/100{border-left-color:#f5f5f4}.border-l-stone-100\/15{border-left-color:hsla(60,5%,96%,.15)}.border-l-stone-100\/20{border-left-color:hsla(60,5%,96%,.2)}.border-l-stone-100\/25{border-left-color:hsla(60,5%,96%,.25)}.border-l-stone-100\/30{border-left-color:hsla(60,5%,96%,.3)}.border-l-stone-100\/35{border-left-color:hsla(60,5%,96%,.35)}.border-l-stone-100\/40{border-left-color:hsla(60,5%,96%,.4)}.border-l-stone-100\/45{border-left-color:hsla(60,5%,96%,.45)}.border-l-stone-100\/5{border-left-color:hsla(60,5%,96%,.05)}.border-l-stone-100\/50{border-left-color:hsla(60,5%,96%,.5)}.border-l-stone-100\/55{border-left-color:hsla(60,5%,96%,.55)}.border-l-stone-100\/60{border-left-color:hsla(60,5%,96%,.6)}.border-l-stone-100\/65{border-left-color:hsla(60,5%,96%,.65)}.border-l-stone-100\/70{border-left-color:hsla(60,5%,96%,.7)}.border-l-stone-100\/75{border-left-color:hsla(60,5%,96%,.75)}.border-l-stone-100\/80{border-left-color:hsla(60,5%,96%,.8)}.border-l-stone-100\/85{border-left-color:hsla(60,5%,96%,.85)}.border-l-stone-100\/90{border-left-color:hsla(60,5%,96%,.9)}.border-l-stone-100\/95{border-left-color:hsla(60,5%,96%,.95)}.border-l-stone-200{--tw-border-opacity:1;border-left-color:rgb(231 229 228/var(--tw-border-opacity,1))}.border-l-stone-200\/0{border-left-color:hsla(20,6%,90%,0)}.border-l-stone-200\/10{border-left-color:hsla(20,6%,90%,.1)}.border-l-stone-200\/100{border-left-color:#e7e5e4}.border-l-stone-200\/15{border-left-color:hsla(20,6%,90%,.15)}.border-l-stone-200\/20{border-left-color:hsla(20,6%,90%,.2)}.border-l-stone-200\/25{border-left-color:hsla(20,6%,90%,.25)}.border-l-stone-200\/30{border-left-color:hsla(20,6%,90%,.3)}.border-l-stone-200\/35{border-left-color:hsla(20,6%,90%,.35)}.border-l-stone-200\/40{border-left-color:hsla(20,6%,90%,.4)}.border-l-stone-200\/45{border-left-color:hsla(20,6%,90%,.45)}.border-l-stone-200\/5{border-left-color:hsla(20,6%,90%,.05)}.border-l-stone-200\/50{border-left-color:hsla(20,6%,90%,.5)}.border-l-stone-200\/55{border-left-color:hsla(20,6%,90%,.55)}.border-l-stone-200\/60{border-left-color:hsla(20,6%,90%,.6)}.border-l-stone-200\/65{border-left-color:hsla(20,6%,90%,.65)}.border-l-stone-200\/70{border-left-color:hsla(20,6%,90%,.7)}.border-l-stone-200\/75{border-left-color:hsla(20,6%,90%,.75)}.border-l-stone-200\/80{border-left-color:hsla(20,6%,90%,.8)}.border-l-stone-200\/85{border-left-color:hsla(20,6%,90%,.85)}.border-l-stone-200\/90{border-left-color:hsla(20,6%,90%,.9)}.border-l-stone-200\/95{border-left-color:hsla(20,6%,90%,.95)}.border-l-stone-400{--tw-border-opacity:1;border-left-color:rgb(168 162 158/var(--tw-border-opacity,1))}.border-l-stone-400\/0{border-left-color:hsla(24,5%,64%,0)}.border-l-stone-400\/10{border-left-color:hsla(24,5%,64%,.1)}.border-l-stone-400\/100{border-left-color:#a8a29e}.border-l-stone-400\/15{border-left-color:hsla(24,5%,64%,.15)}.border-l-stone-400\/20{border-left-color:hsla(24,5%,64%,.2)}.border-l-stone-400\/25{border-left-color:hsla(24,5%,64%,.25)}.border-l-stone-400\/30{border-left-color:hsla(24,5%,64%,.3)}.border-l-stone-400\/35{border-left-color:hsla(24,5%,64%,.35)}.border-l-stone-400\/40{border-left-color:hsla(24,5%,64%,.4)}.border-l-stone-400\/45{border-left-color:hsla(24,5%,64%,.45)}.border-l-stone-400\/5{border-left-color:hsla(24,5%,64%,.05)}.border-l-stone-400\/50{border-left-color:hsla(24,5%,64%,.5)}.border-l-stone-400\/55{border-left-color:hsla(24,5%,64%,.55)}.border-l-stone-400\/60{border-left-color:hsla(24,5%,64%,.6)}.border-l-stone-400\/65{border-left-color:hsla(24,5%,64%,.65)}.border-l-stone-400\/70{border-left-color:hsla(24,5%,64%,.7)}.border-l-stone-400\/75{border-left-color:hsla(24,5%,64%,.75)}.border-l-stone-400\/80{border-left-color:hsla(24,5%,64%,.8)}.border-l-stone-400\/85{border-left-color:hsla(24,5%,64%,.85)}.border-l-stone-400\/90{border-left-color:hsla(24,5%,64%,.9)}.border-l-stone-400\/95{border-left-color:hsla(24,5%,64%,.95)}.border-l-stone-50{--tw-border-opacity:1;border-left-color:rgb(250 250 249/var(--tw-border-opacity,1))}.border-l-stone-50\/0{border-left-color:hsla(60,9%,98%,0)}.border-l-stone-50\/10{border-left-color:hsla(60,9%,98%,.1)}.border-l-stone-50\/100{border-left-color:#fafaf9}.border-l-stone-50\/15{border-left-color:hsla(60,9%,98%,.15)}.border-l-stone-50\/20{border-left-color:hsla(60,9%,98%,.2)}.border-l-stone-50\/25{border-left-color:hsla(60,9%,98%,.25)}.border-l-stone-50\/30{border-left-color:hsla(60,9%,98%,.3)}.border-l-stone-50\/35{border-left-color:hsla(60,9%,98%,.35)}.border-l-stone-50\/40{border-left-color:hsla(60,9%,98%,.4)}.border-l-stone-50\/45{border-left-color:hsla(60,9%,98%,.45)}.border-l-stone-50\/5{border-left-color:hsla(60,9%,98%,.05)}.border-l-stone-50\/50{border-left-color:hsla(60,9%,98%,.5)}.border-l-stone-50\/55{border-left-color:hsla(60,9%,98%,.55)}.border-l-stone-50\/60{border-left-color:hsla(60,9%,98%,.6)}.border-l-stone-50\/65{border-left-color:hsla(60,9%,98%,.65)}.border-l-stone-50\/70{border-left-color:hsla(60,9%,98%,.7)}.border-l-stone-50\/75{border-left-color:hsla(60,9%,98%,.75)}.border-l-stone-50\/80{border-left-color:hsla(60,9%,98%,.8)}.border-l-stone-50\/85{border-left-color:hsla(60,9%,98%,.85)}.border-l-stone-50\/90{border-left-color:hsla(60,9%,98%,.9)}.border-l-stone-50\/95{border-left-color:hsla(60,9%,98%,.95)}.border-l-stone-500{--tw-border-opacity:1;border-left-color:rgb(120 113 108/var(--tw-border-opacity,1))}.border-l-stone-500\/0{border-left-color:hsla(25,5%,45%,0)}.border-l-stone-500\/10{border-left-color:hsla(25,5%,45%,.1)}.border-l-stone-500\/100{border-left-color:#78716c}.border-l-stone-500\/15{border-left-color:hsla(25,5%,45%,.15)}.border-l-stone-500\/20{border-left-color:hsla(25,5%,45%,.2)}.border-l-stone-500\/25{border-left-color:hsla(25,5%,45%,.25)}.border-l-stone-500\/30{border-left-color:hsla(25,5%,45%,.3)}.border-l-stone-500\/35{border-left-color:hsla(25,5%,45%,.35)}.border-l-stone-500\/40{border-left-color:hsla(25,5%,45%,.4)}.border-l-stone-500\/45{border-left-color:hsla(25,5%,45%,.45)}.border-l-stone-500\/5{border-left-color:hsla(25,5%,45%,.05)}.border-l-stone-500\/50{border-left-color:hsla(25,5%,45%,.5)}.border-l-stone-500\/55{border-left-color:hsla(25,5%,45%,.55)}.border-l-stone-500\/60{border-left-color:hsla(25,5%,45%,.6)}.border-l-stone-500\/65{border-left-color:hsla(25,5%,45%,.65)}.border-l-stone-500\/70{border-left-color:hsla(25,5%,45%,.7)}.border-l-stone-500\/75{border-left-color:hsla(25,5%,45%,.75)}.border-l-stone-500\/80{border-left-color:hsla(25,5%,45%,.8)}.border-l-stone-500\/85{border-left-color:hsla(25,5%,45%,.85)}.border-l-stone-500\/90{border-left-color:hsla(25,5%,45%,.9)}.border-l-stone-500\/95{border-left-color:hsla(25,5%,45%,.95)}.border-l-stone-950{--tw-border-opacity:1;border-left-color:rgb(12 10 9/var(--tw-border-opacity,1))}.border-l-stone-950\/0{border-left-color:rgba(12,10,9,0)}.border-l-stone-950\/10{border-left-color:rgba(12,10,9,.1)}.border-l-stone-950\/100{border-left-color:#0c0a09}.border-l-stone-950\/15{border-left-color:rgba(12,10,9,.15)}.border-l-stone-950\/20{border-left-color:rgba(12,10,9,.2)}.border-l-stone-950\/25{border-left-color:rgba(12,10,9,.25)}.border-l-stone-950\/30{border-left-color:rgba(12,10,9,.3)}.border-l-stone-950\/35{border-left-color:rgba(12,10,9,.35)}.border-l-stone-950\/40{border-left-color:rgba(12,10,9,.4)}.border-l-stone-950\/45{border-left-color:rgba(12,10,9,.45)}.border-l-stone-950\/5{border-left-color:rgba(12,10,9,.05)}.border-l-stone-950\/50{border-left-color:rgba(12,10,9,.5)}.border-l-stone-950\/55{border-left-color:rgba(12,10,9,.55)}.border-l-stone-950\/60{border-left-color:rgba(12,10,9,.6)}.border-l-stone-950\/65{border-left-color:rgba(12,10,9,.65)}.border-l-stone-950\/70{border-left-color:rgba(12,10,9,.7)}.border-l-stone-950\/75{border-left-color:rgba(12,10,9,.75)}.border-l-stone-950\/80{border-left-color:rgba(12,10,9,.8)}.border-l-stone-950\/85{border-left-color:rgba(12,10,9,.85)}.border-l-stone-950\/90{border-left-color:rgba(12,10,9,.9)}.border-l-stone-950\/95{border-left-color:rgba(12,10,9,.95)}.border-l-teal-100{--tw-border-opacity:1;border-left-color:rgb(204 251 241/var(--tw-border-opacity,1))}.border-l-teal-100\/0{border-left-color:rgba(204,251,241,0)}.border-l-teal-100\/10{border-left-color:rgba(204,251,241,.1)}.border-l-teal-100\/100{border-left-color:#ccfbf1}.border-l-teal-100\/15{border-left-color:rgba(204,251,241,.15)}.border-l-teal-100\/20{border-left-color:rgba(204,251,241,.2)}.border-l-teal-100\/25{border-left-color:rgba(204,251,241,.25)}.border-l-teal-100\/30{border-left-color:rgba(204,251,241,.3)}.border-l-teal-100\/35{border-left-color:rgba(204,251,241,.35)}.border-l-teal-100\/40{border-left-color:rgba(204,251,241,.4)}.border-l-teal-100\/45{border-left-color:rgba(204,251,241,.45)}.border-l-teal-100\/5{border-left-color:rgba(204,251,241,.05)}.border-l-teal-100\/50{border-left-color:rgba(204,251,241,.5)}.border-l-teal-100\/55{border-left-color:rgba(204,251,241,.55)}.border-l-teal-100\/60{border-left-color:rgba(204,251,241,.6)}.border-l-teal-100\/65{border-left-color:rgba(204,251,241,.65)}.border-l-teal-100\/70{border-left-color:rgba(204,251,241,.7)}.border-l-teal-100\/75{border-left-color:rgba(204,251,241,.75)}.border-l-teal-100\/80{border-left-color:rgba(204,251,241,.8)}.border-l-teal-100\/85{border-left-color:rgba(204,251,241,.85)}.border-l-teal-100\/90{border-left-color:rgba(204,251,241,.9)}.border-l-teal-100\/95{border-left-color:rgba(204,251,241,.95)}.border-l-teal-200{--tw-border-opacity:1;border-left-color:rgb(153 246 228/var(--tw-border-opacity,1))}.border-l-teal-200\/0{border-left-color:rgba(153,246,228,0)}.border-l-teal-200\/10{border-left-color:rgba(153,246,228,.1)}.border-l-teal-200\/100{border-left-color:#99f6e4}.border-l-teal-200\/15{border-left-color:rgba(153,246,228,.15)}.border-l-teal-200\/20{border-left-color:rgba(153,246,228,.2)}.border-l-teal-200\/25{border-left-color:rgba(153,246,228,.25)}.border-l-teal-200\/30{border-left-color:rgba(153,246,228,.3)}.border-l-teal-200\/35{border-left-color:rgba(153,246,228,.35)}.border-l-teal-200\/40{border-left-color:rgba(153,246,228,.4)}.border-l-teal-200\/45{border-left-color:rgba(153,246,228,.45)}.border-l-teal-200\/5{border-left-color:rgba(153,246,228,.05)}.border-l-teal-200\/50{border-left-color:rgba(153,246,228,.5)}.border-l-teal-200\/55{border-left-color:rgba(153,246,228,.55)}.border-l-teal-200\/60{border-left-color:rgba(153,246,228,.6)}.border-l-teal-200\/65{border-left-color:rgba(153,246,228,.65)}.border-l-teal-200\/70{border-left-color:rgba(153,246,228,.7)}.border-l-teal-200\/75{border-left-color:rgba(153,246,228,.75)}.border-l-teal-200\/80{border-left-color:rgba(153,246,228,.8)}.border-l-teal-200\/85{border-left-color:rgba(153,246,228,.85)}.border-l-teal-200\/90{border-left-color:rgba(153,246,228,.9)}.border-l-teal-200\/95{border-left-color:rgba(153,246,228,.95)}.border-l-teal-400{--tw-border-opacity:1;border-left-color:rgb(45 212 191/var(--tw-border-opacity,1))}.border-l-teal-400\/0{border-left-color:rgba(45,212,191,0)}.border-l-teal-400\/10{border-left-color:rgba(45,212,191,.1)}.border-l-teal-400\/100{border-left-color:#2dd4bf}.border-l-teal-400\/15{border-left-color:rgba(45,212,191,.15)}.border-l-teal-400\/20{border-left-color:rgba(45,212,191,.2)}.border-l-teal-400\/25{border-left-color:rgba(45,212,191,.25)}.border-l-teal-400\/30{border-left-color:rgba(45,212,191,.3)}.border-l-teal-400\/35{border-left-color:rgba(45,212,191,.35)}.border-l-teal-400\/40{border-left-color:rgba(45,212,191,.4)}.border-l-teal-400\/45{border-left-color:rgba(45,212,191,.45)}.border-l-teal-400\/5{border-left-color:rgba(45,212,191,.05)}.border-l-teal-400\/50{border-left-color:rgba(45,212,191,.5)}.border-l-teal-400\/55{border-left-color:rgba(45,212,191,.55)}.border-l-teal-400\/60{border-left-color:rgba(45,212,191,.6)}.border-l-teal-400\/65{border-left-color:rgba(45,212,191,.65)}.border-l-teal-400\/70{border-left-color:rgba(45,212,191,.7)}.border-l-teal-400\/75{border-left-color:rgba(45,212,191,.75)}.border-l-teal-400\/80{border-left-color:rgba(45,212,191,.8)}.border-l-teal-400\/85{border-left-color:rgba(45,212,191,.85)}.border-l-teal-400\/90{border-left-color:rgba(45,212,191,.9)}.border-l-teal-400\/95{border-left-color:rgba(45,212,191,.95)}.border-l-teal-50{--tw-border-opacity:1;border-left-color:rgb(240 253 250/var(--tw-border-opacity,1))}.border-l-teal-50\/0{border-left-color:rgba(240,253,250,0)}.border-l-teal-50\/10{border-left-color:rgba(240,253,250,.1)}.border-l-teal-50\/100{border-left-color:#f0fdfa}.border-l-teal-50\/15{border-left-color:rgba(240,253,250,.15)}.border-l-teal-50\/20{border-left-color:rgba(240,253,250,.2)}.border-l-teal-50\/25{border-left-color:rgba(240,253,250,.25)}.border-l-teal-50\/30{border-left-color:rgba(240,253,250,.3)}.border-l-teal-50\/35{border-left-color:rgba(240,253,250,.35)}.border-l-teal-50\/40{border-left-color:rgba(240,253,250,.4)}.border-l-teal-50\/45{border-left-color:rgba(240,253,250,.45)}.border-l-teal-50\/5{border-left-color:rgba(240,253,250,.05)}.border-l-teal-50\/50{border-left-color:rgba(240,253,250,.5)}.border-l-teal-50\/55{border-left-color:rgba(240,253,250,.55)}.border-l-teal-50\/60{border-left-color:rgba(240,253,250,.6)}.border-l-teal-50\/65{border-left-color:rgba(240,253,250,.65)}.border-l-teal-50\/70{border-left-color:rgba(240,253,250,.7)}.border-l-teal-50\/75{border-left-color:rgba(240,253,250,.75)}.border-l-teal-50\/80{border-left-color:rgba(240,253,250,.8)}.border-l-teal-50\/85{border-left-color:rgba(240,253,250,.85)}.border-l-teal-50\/90{border-left-color:rgba(240,253,250,.9)}.border-l-teal-50\/95{border-left-color:rgba(240,253,250,.95)}.border-l-teal-500{--tw-border-opacity:1;border-left-color:rgb(20 184 166/var(--tw-border-opacity,1))}.border-l-teal-500\/0{border-left-color:rgba(20,184,166,0)}.border-l-teal-500\/10{border-left-color:rgba(20,184,166,.1)}.border-l-teal-500\/100{border-left-color:#14b8a6}.border-l-teal-500\/15{border-left-color:rgba(20,184,166,.15)}.border-l-teal-500\/20{border-left-color:rgba(20,184,166,.2)}.border-l-teal-500\/25{border-left-color:rgba(20,184,166,.25)}.border-l-teal-500\/30{border-left-color:rgba(20,184,166,.3)}.border-l-teal-500\/35{border-left-color:rgba(20,184,166,.35)}.border-l-teal-500\/40{border-left-color:rgba(20,184,166,.4)}.border-l-teal-500\/45{border-left-color:rgba(20,184,166,.45)}.border-l-teal-500\/5{border-left-color:rgba(20,184,166,.05)}.border-l-teal-500\/50{border-left-color:rgba(20,184,166,.5)}.border-l-teal-500\/55{border-left-color:rgba(20,184,166,.55)}.border-l-teal-500\/60{border-left-color:rgba(20,184,166,.6)}.border-l-teal-500\/65{border-left-color:rgba(20,184,166,.65)}.border-l-teal-500\/70{border-left-color:rgba(20,184,166,.7)}.border-l-teal-500\/75{border-left-color:rgba(20,184,166,.75)}.border-l-teal-500\/80{border-left-color:rgba(20,184,166,.8)}.border-l-teal-500\/85{border-left-color:rgba(20,184,166,.85)}.border-l-teal-500\/90{border-left-color:rgba(20,184,166,.9)}.border-l-teal-500\/95{border-left-color:rgba(20,184,166,.95)}.border-l-teal-950{--tw-border-opacity:1;border-left-color:rgb(4 47 46/var(--tw-border-opacity,1))}.border-l-teal-950\/0{border-left-color:rgba(4,47,46,0)}.border-l-teal-950\/10{border-left-color:rgba(4,47,46,.1)}.border-l-teal-950\/100{border-left-color:#042f2e}.border-l-teal-950\/15{border-left-color:rgba(4,47,46,.15)}.border-l-teal-950\/20{border-left-color:rgba(4,47,46,.2)}.border-l-teal-950\/25{border-left-color:rgba(4,47,46,.25)}.border-l-teal-950\/30{border-left-color:rgba(4,47,46,.3)}.border-l-teal-950\/35{border-left-color:rgba(4,47,46,.35)}.border-l-teal-950\/40{border-left-color:rgba(4,47,46,.4)}.border-l-teal-950\/45{border-left-color:rgba(4,47,46,.45)}.border-l-teal-950\/5{border-left-color:rgba(4,47,46,.05)}.border-l-teal-950\/50{border-left-color:rgba(4,47,46,.5)}.border-l-teal-950\/55{border-left-color:rgba(4,47,46,.55)}.border-l-teal-950\/60{border-left-color:rgba(4,47,46,.6)}.border-l-teal-950\/65{border-left-color:rgba(4,47,46,.65)}.border-l-teal-950\/70{border-left-color:rgba(4,47,46,.7)}.border-l-teal-950\/75{border-left-color:rgba(4,47,46,.75)}.border-l-teal-950\/80{border-left-color:rgba(4,47,46,.8)}.border-l-teal-950\/85{border-left-color:rgba(4,47,46,.85)}.border-l-teal-950\/90{border-left-color:rgba(4,47,46,.9)}.border-l-teal-950\/95{border-left-color:rgba(4,47,46,.95)}.border-l-transparent{border-left-color:transparent}.border-l-violet-100{--tw-border-opacity:1;border-left-color:rgb(237 233 254/var(--tw-border-opacity,1))}.border-l-violet-100\/0{border-left-color:rgba(237,233,254,0)}.border-l-violet-100\/10{border-left-color:rgba(237,233,254,.1)}.border-l-violet-100\/100{border-left-color:#ede9fe}.border-l-violet-100\/15{border-left-color:rgba(237,233,254,.15)}.border-l-violet-100\/20{border-left-color:rgba(237,233,254,.2)}.border-l-violet-100\/25{border-left-color:rgba(237,233,254,.25)}.border-l-violet-100\/30{border-left-color:rgba(237,233,254,.3)}.border-l-violet-100\/35{border-left-color:rgba(237,233,254,.35)}.border-l-violet-100\/40{border-left-color:rgba(237,233,254,.4)}.border-l-violet-100\/45{border-left-color:rgba(237,233,254,.45)}.border-l-violet-100\/5{border-left-color:rgba(237,233,254,.05)}.border-l-violet-100\/50{border-left-color:rgba(237,233,254,.5)}.border-l-violet-100\/55{border-left-color:rgba(237,233,254,.55)}.border-l-violet-100\/60{border-left-color:rgba(237,233,254,.6)}.border-l-violet-100\/65{border-left-color:rgba(237,233,254,.65)}.border-l-violet-100\/70{border-left-color:rgba(237,233,254,.7)}.border-l-violet-100\/75{border-left-color:rgba(237,233,254,.75)}.border-l-violet-100\/80{border-left-color:rgba(237,233,254,.8)}.border-l-violet-100\/85{border-left-color:rgba(237,233,254,.85)}.border-l-violet-100\/90{border-left-color:rgba(237,233,254,.9)}.border-l-violet-100\/95{border-left-color:rgba(237,233,254,.95)}.border-l-violet-200{--tw-border-opacity:1;border-left-color:rgb(221 214 254/var(--tw-border-opacity,1))}.border-l-violet-200\/0{border-left-color:rgba(221,214,254,0)}.border-l-violet-200\/10{border-left-color:rgba(221,214,254,.1)}.border-l-violet-200\/100{border-left-color:#ddd6fe}.border-l-violet-200\/15{border-left-color:rgba(221,214,254,.15)}.border-l-violet-200\/20{border-left-color:rgba(221,214,254,.2)}.border-l-violet-200\/25{border-left-color:rgba(221,214,254,.25)}.border-l-violet-200\/30{border-left-color:rgba(221,214,254,.3)}.border-l-violet-200\/35{border-left-color:rgba(221,214,254,.35)}.border-l-violet-200\/40{border-left-color:rgba(221,214,254,.4)}.border-l-violet-200\/45{border-left-color:rgba(221,214,254,.45)}.border-l-violet-200\/5{border-left-color:rgba(221,214,254,.05)}.border-l-violet-200\/50{border-left-color:rgba(221,214,254,.5)}.border-l-violet-200\/55{border-left-color:rgba(221,214,254,.55)}.border-l-violet-200\/60{border-left-color:rgba(221,214,254,.6)}.border-l-violet-200\/65{border-left-color:rgba(221,214,254,.65)}.border-l-violet-200\/70{border-left-color:rgba(221,214,254,.7)}.border-l-violet-200\/75{border-left-color:rgba(221,214,254,.75)}.border-l-violet-200\/80{border-left-color:rgba(221,214,254,.8)}.border-l-violet-200\/85{border-left-color:rgba(221,214,254,.85)}.border-l-violet-200\/90{border-left-color:rgba(221,214,254,.9)}.border-l-violet-200\/95{border-left-color:rgba(221,214,254,.95)}.border-l-violet-400{--tw-border-opacity:1;border-left-color:rgb(167 139 250/var(--tw-border-opacity,1))}.border-l-violet-400\/0{border-left-color:rgba(167,139,250,0)}.border-l-violet-400\/10{border-left-color:rgba(167,139,250,.1)}.border-l-violet-400\/100{border-left-color:#a78bfa}.border-l-violet-400\/15{border-left-color:rgba(167,139,250,.15)}.border-l-violet-400\/20{border-left-color:rgba(167,139,250,.2)}.border-l-violet-400\/25{border-left-color:rgba(167,139,250,.25)}.border-l-violet-400\/30{border-left-color:rgba(167,139,250,.3)}.border-l-violet-400\/35{border-left-color:rgba(167,139,250,.35)}.border-l-violet-400\/40{border-left-color:rgba(167,139,250,.4)}.border-l-violet-400\/45{border-left-color:rgba(167,139,250,.45)}.border-l-violet-400\/5{border-left-color:rgba(167,139,250,.05)}.border-l-violet-400\/50{border-left-color:rgba(167,139,250,.5)}.border-l-violet-400\/55{border-left-color:rgba(167,139,250,.55)}.border-l-violet-400\/60{border-left-color:rgba(167,139,250,.6)}.border-l-violet-400\/65{border-left-color:rgba(167,139,250,.65)}.border-l-violet-400\/70{border-left-color:rgba(167,139,250,.7)}.border-l-violet-400\/75{border-left-color:rgba(167,139,250,.75)}.border-l-violet-400\/80{border-left-color:rgba(167,139,250,.8)}.border-l-violet-400\/85{border-left-color:rgba(167,139,250,.85)}.border-l-violet-400\/90{border-left-color:rgba(167,139,250,.9)}.border-l-violet-400\/95{border-left-color:rgba(167,139,250,.95)}.border-l-violet-50{--tw-border-opacity:1;border-left-color:rgb(245 243 255/var(--tw-border-opacity,1))}.border-l-violet-50\/0{border-left-color:rgba(245,243,255,0)}.border-l-violet-50\/10{border-left-color:rgba(245,243,255,.1)}.border-l-violet-50\/100{border-left-color:#f5f3ff}.border-l-violet-50\/15{border-left-color:rgba(245,243,255,.15)}.border-l-violet-50\/20{border-left-color:rgba(245,243,255,.2)}.border-l-violet-50\/25{border-left-color:rgba(245,243,255,.25)}.border-l-violet-50\/30{border-left-color:rgba(245,243,255,.3)}.border-l-violet-50\/35{border-left-color:rgba(245,243,255,.35)}.border-l-violet-50\/40{border-left-color:rgba(245,243,255,.4)}.border-l-violet-50\/45{border-left-color:rgba(245,243,255,.45)}.border-l-violet-50\/5{border-left-color:rgba(245,243,255,.05)}.border-l-violet-50\/50{border-left-color:rgba(245,243,255,.5)}.border-l-violet-50\/55{border-left-color:rgba(245,243,255,.55)}.border-l-violet-50\/60{border-left-color:rgba(245,243,255,.6)}.border-l-violet-50\/65{border-left-color:rgba(245,243,255,.65)}.border-l-violet-50\/70{border-left-color:rgba(245,243,255,.7)}.border-l-violet-50\/75{border-left-color:rgba(245,243,255,.75)}.border-l-violet-50\/80{border-left-color:rgba(245,243,255,.8)}.border-l-violet-50\/85{border-left-color:rgba(245,243,255,.85)}.border-l-violet-50\/90{border-left-color:rgba(245,243,255,.9)}.border-l-violet-50\/95{border-left-color:rgba(245,243,255,.95)}.border-l-violet-500{--tw-border-opacity:1;border-left-color:rgb(139 92 246/var(--tw-border-opacity,1))}.border-l-violet-500\/0{border-left-color:rgba(139,92,246,0)}.border-l-violet-500\/10{border-left-color:rgba(139,92,246,.1)}.border-l-violet-500\/100{border-left-color:#8b5cf6}.border-l-violet-500\/15{border-left-color:rgba(139,92,246,.15)}.border-l-violet-500\/20{border-left-color:rgba(139,92,246,.2)}.border-l-violet-500\/25{border-left-color:rgba(139,92,246,.25)}.border-l-violet-500\/30{border-left-color:rgba(139,92,246,.3)}.border-l-violet-500\/35{border-left-color:rgba(139,92,246,.35)}.border-l-violet-500\/40{border-left-color:rgba(139,92,246,.4)}.border-l-violet-500\/45{border-left-color:rgba(139,92,246,.45)}.border-l-violet-500\/5{border-left-color:rgba(139,92,246,.05)}.border-l-violet-500\/50{border-left-color:rgba(139,92,246,.5)}.border-l-violet-500\/55{border-left-color:rgba(139,92,246,.55)}.border-l-violet-500\/60{border-left-color:rgba(139,92,246,.6)}.border-l-violet-500\/65{border-left-color:rgba(139,92,246,.65)}.border-l-violet-500\/70{border-left-color:rgba(139,92,246,.7)}.border-l-violet-500\/75{border-left-color:rgba(139,92,246,.75)}.border-l-violet-500\/80{border-left-color:rgba(139,92,246,.8)}.border-l-violet-500\/85{border-left-color:rgba(139,92,246,.85)}.border-l-violet-500\/90{border-left-color:rgba(139,92,246,.9)}.border-l-violet-500\/95{border-left-color:rgba(139,92,246,.95)}.border-l-violet-950{--tw-border-opacity:1;border-left-color:rgb(46 16 101/var(--tw-border-opacity,1))}.border-l-violet-950\/0{border-left-color:rgba(46,16,101,0)}.border-l-violet-950\/10{border-left-color:rgba(46,16,101,.1)}.border-l-violet-950\/100{border-left-color:#2e1065}.border-l-violet-950\/15{border-left-color:rgba(46,16,101,.15)}.border-l-violet-950\/20{border-left-color:rgba(46,16,101,.2)}.border-l-violet-950\/25{border-left-color:rgba(46,16,101,.25)}.border-l-violet-950\/30{border-left-color:rgba(46,16,101,.3)}.border-l-violet-950\/35{border-left-color:rgba(46,16,101,.35)}.border-l-violet-950\/40{border-left-color:rgba(46,16,101,.4)}.border-l-violet-950\/45{border-left-color:rgba(46,16,101,.45)}.border-l-violet-950\/5{border-left-color:rgba(46,16,101,.05)}.border-l-violet-950\/50{border-left-color:rgba(46,16,101,.5)}.border-l-violet-950\/55{border-left-color:rgba(46,16,101,.55)}.border-l-violet-950\/60{border-left-color:rgba(46,16,101,.6)}.border-l-violet-950\/65{border-left-color:rgba(46,16,101,.65)}.border-l-violet-950\/70{border-left-color:rgba(46,16,101,.7)}.border-l-violet-950\/75{border-left-color:rgba(46,16,101,.75)}.border-l-violet-950\/80{border-left-color:rgba(46,16,101,.8)}.border-l-violet-950\/85{border-left-color:rgba(46,16,101,.85)}.border-l-violet-950\/90{border-left-color:rgba(46,16,101,.9)}.border-l-violet-950\/95{border-left-color:rgba(46,16,101,.95)}.border-l-yellow-100{--tw-border-opacity:1;border-left-color:rgb(254 249 195/var(--tw-border-opacity,1))}.border-l-yellow-100\/0{border-left-color:hsla(55,97%,88%,0)}.border-l-yellow-100\/10{border-left-color:hsla(55,97%,88%,.1)}.border-l-yellow-100\/100{border-left-color:#fef9c3}.border-l-yellow-100\/15{border-left-color:hsla(55,97%,88%,.15)}.border-l-yellow-100\/20{border-left-color:hsla(55,97%,88%,.2)}.border-l-yellow-100\/25{border-left-color:hsla(55,97%,88%,.25)}.border-l-yellow-100\/30{border-left-color:hsla(55,97%,88%,.3)}.border-l-yellow-100\/35{border-left-color:hsla(55,97%,88%,.35)}.border-l-yellow-100\/40{border-left-color:hsla(55,97%,88%,.4)}.border-l-yellow-100\/45{border-left-color:hsla(55,97%,88%,.45)}.border-l-yellow-100\/5{border-left-color:hsla(55,97%,88%,.05)}.border-l-yellow-100\/50{border-left-color:hsla(55,97%,88%,.5)}.border-l-yellow-100\/55{border-left-color:hsla(55,97%,88%,.55)}.border-l-yellow-100\/60{border-left-color:hsla(55,97%,88%,.6)}.border-l-yellow-100\/65{border-left-color:hsla(55,97%,88%,.65)}.border-l-yellow-100\/70{border-left-color:hsla(55,97%,88%,.7)}.border-l-yellow-100\/75{border-left-color:hsla(55,97%,88%,.75)}.border-l-yellow-100\/80{border-left-color:hsla(55,97%,88%,.8)}.border-l-yellow-100\/85{border-left-color:hsla(55,97%,88%,.85)}.border-l-yellow-100\/90{border-left-color:hsla(55,97%,88%,.9)}.border-l-yellow-100\/95{border-left-color:hsla(55,97%,88%,.95)}.border-l-yellow-200{--tw-border-opacity:1;border-left-color:rgb(254 240 138/var(--tw-border-opacity,1))}.border-l-yellow-200\/0{border-left-color:hsla(53,98%,77%,0)}.border-l-yellow-200\/10{border-left-color:hsla(53,98%,77%,.1)}.border-l-yellow-200\/100{border-left-color:#fef08a}.border-l-yellow-200\/15{border-left-color:hsla(53,98%,77%,.15)}.border-l-yellow-200\/20{border-left-color:hsla(53,98%,77%,.2)}.border-l-yellow-200\/25{border-left-color:hsla(53,98%,77%,.25)}.border-l-yellow-200\/30{border-left-color:hsla(53,98%,77%,.3)}.border-l-yellow-200\/35{border-left-color:hsla(53,98%,77%,.35)}.border-l-yellow-200\/40{border-left-color:hsla(53,98%,77%,.4)}.border-l-yellow-200\/45{border-left-color:hsla(53,98%,77%,.45)}.border-l-yellow-200\/5{border-left-color:hsla(53,98%,77%,.05)}.border-l-yellow-200\/50{border-left-color:hsla(53,98%,77%,.5)}.border-l-yellow-200\/55{border-left-color:hsla(53,98%,77%,.55)}.border-l-yellow-200\/60{border-left-color:hsla(53,98%,77%,.6)}.border-l-yellow-200\/65{border-left-color:hsla(53,98%,77%,.65)}.border-l-yellow-200\/70{border-left-color:hsla(53,98%,77%,.7)}.border-l-yellow-200\/75{border-left-color:hsla(53,98%,77%,.75)}.border-l-yellow-200\/80{border-left-color:hsla(53,98%,77%,.8)}.border-l-yellow-200\/85{border-left-color:hsla(53,98%,77%,.85)}.border-l-yellow-200\/90{border-left-color:hsla(53,98%,77%,.9)}.border-l-yellow-200\/95{border-left-color:hsla(53,98%,77%,.95)}.border-l-yellow-400{--tw-border-opacity:1;border-left-color:rgb(250 204 21/var(--tw-border-opacity,1))}.border-l-yellow-400\/0{border-left-color:rgba(250,204,21,0)}.border-l-yellow-400\/10{border-left-color:rgba(250,204,21,.1)}.border-l-yellow-400\/100{border-left-color:#facc15}.border-l-yellow-400\/15{border-left-color:rgba(250,204,21,.15)}.border-l-yellow-400\/20{border-left-color:rgba(250,204,21,.2)}.border-l-yellow-400\/25{border-left-color:rgba(250,204,21,.25)}.border-l-yellow-400\/30{border-left-color:rgba(250,204,21,.3)}.border-l-yellow-400\/35{border-left-color:rgba(250,204,21,.35)}.border-l-yellow-400\/40{border-left-color:rgba(250,204,21,.4)}.border-l-yellow-400\/45{border-left-color:rgba(250,204,21,.45)}.border-l-yellow-400\/5{border-left-color:rgba(250,204,21,.05)}.border-l-yellow-400\/50{border-left-color:rgba(250,204,21,.5)}.border-l-yellow-400\/55{border-left-color:rgba(250,204,21,.55)}.border-l-yellow-400\/60{border-left-color:rgba(250,204,21,.6)}.border-l-yellow-400\/65{border-left-color:rgba(250,204,21,.65)}.border-l-yellow-400\/70{border-left-color:rgba(250,204,21,.7)}.border-l-yellow-400\/75{border-left-color:rgba(250,204,21,.75)}.border-l-yellow-400\/80{border-left-color:rgba(250,204,21,.8)}.border-l-yellow-400\/85{border-left-color:rgba(250,204,21,.85)}.border-l-yellow-400\/90{border-left-color:rgba(250,204,21,.9)}.border-l-yellow-400\/95{border-left-color:rgba(250,204,21,.95)}.border-l-yellow-50{--tw-border-opacity:1;border-left-color:rgb(254 252 232/var(--tw-border-opacity,1))}.border-l-yellow-50\/0{border-left-color:hsla(55,92%,95%,0)}.border-l-yellow-50\/10{border-left-color:hsla(55,92%,95%,.1)}.border-l-yellow-50\/100{border-left-color:#fefce8}.border-l-yellow-50\/15{border-left-color:hsla(55,92%,95%,.15)}.border-l-yellow-50\/20{border-left-color:hsla(55,92%,95%,.2)}.border-l-yellow-50\/25{border-left-color:hsla(55,92%,95%,.25)}.border-l-yellow-50\/30{border-left-color:hsla(55,92%,95%,.3)}.border-l-yellow-50\/35{border-left-color:hsla(55,92%,95%,.35)}.border-l-yellow-50\/40{border-left-color:hsla(55,92%,95%,.4)}.border-l-yellow-50\/45{border-left-color:hsla(55,92%,95%,.45)}.border-l-yellow-50\/5{border-left-color:hsla(55,92%,95%,.05)}.border-l-yellow-50\/50{border-left-color:hsla(55,92%,95%,.5)}.border-l-yellow-50\/55{border-left-color:hsla(55,92%,95%,.55)}.border-l-yellow-50\/60{border-left-color:hsla(55,92%,95%,.6)}.border-l-yellow-50\/65{border-left-color:hsla(55,92%,95%,.65)}.border-l-yellow-50\/70{border-left-color:hsla(55,92%,95%,.7)}.border-l-yellow-50\/75{border-left-color:hsla(55,92%,95%,.75)}.border-l-yellow-50\/80{border-left-color:hsla(55,92%,95%,.8)}.border-l-yellow-50\/85{border-left-color:hsla(55,92%,95%,.85)}.border-l-yellow-50\/90{border-left-color:hsla(55,92%,95%,.9)}.border-l-yellow-50\/95{border-left-color:hsla(55,92%,95%,.95)}.border-l-yellow-500{--tw-border-opacity:1;border-left-color:rgb(234 179 8/var(--tw-border-opacity,1))}.border-l-yellow-500\/0{border-left-color:rgba(234,179,8,0)}.border-l-yellow-500\/10{border-left-color:rgba(234,179,8,.1)}.border-l-yellow-500\/100{border-left-color:#eab308}.border-l-yellow-500\/15{border-left-color:rgba(234,179,8,.15)}.border-l-yellow-500\/20{border-left-color:rgba(234,179,8,.2)}.border-l-yellow-500\/25{border-left-color:rgba(234,179,8,.25)}.border-l-yellow-500\/30{border-left-color:rgba(234,179,8,.3)}.border-l-yellow-500\/35{border-left-color:rgba(234,179,8,.35)}.border-l-yellow-500\/40{border-left-color:rgba(234,179,8,.4)}.border-l-yellow-500\/45{border-left-color:rgba(234,179,8,.45)}.border-l-yellow-500\/5{border-left-color:rgba(234,179,8,.05)}.border-l-yellow-500\/50{border-left-color:rgba(234,179,8,.5)}.border-l-yellow-500\/55{border-left-color:rgba(234,179,8,.55)}.border-l-yellow-500\/60{border-left-color:rgba(234,179,8,.6)}.border-l-yellow-500\/65{border-left-color:rgba(234,179,8,.65)}.border-l-yellow-500\/70{border-left-color:rgba(234,179,8,.7)}.border-l-yellow-500\/75{border-left-color:rgba(234,179,8,.75)}.border-l-yellow-500\/80{border-left-color:rgba(234,179,8,.8)}.border-l-yellow-500\/85{border-left-color:rgba(234,179,8,.85)}.border-l-yellow-500\/90{border-left-color:rgba(234,179,8,.9)}.border-l-yellow-500\/95{border-left-color:rgba(234,179,8,.95)}.border-l-yellow-950{--tw-border-opacity:1;border-left-color:rgb(66 32 6/var(--tw-border-opacity,1))}.border-l-yellow-950\/0{border-left-color:rgba(66,32,6,0)}.border-l-yellow-950\/10{border-left-color:rgba(66,32,6,.1)}.border-l-yellow-950\/100{border-left-color:#422006}.border-l-yellow-950\/15{border-left-color:rgba(66,32,6,.15)}.border-l-yellow-950\/20{border-left-color:rgba(66,32,6,.2)}.border-l-yellow-950\/25{border-left-color:rgba(66,32,6,.25)}.border-l-yellow-950\/30{border-left-color:rgba(66,32,6,.3)}.border-l-yellow-950\/35{border-left-color:rgba(66,32,6,.35)}.border-l-yellow-950\/40{border-left-color:rgba(66,32,6,.4)}.border-l-yellow-950\/45{border-left-color:rgba(66,32,6,.45)}.border-l-yellow-950\/5{border-left-color:rgba(66,32,6,.05)}.border-l-yellow-950\/50{border-left-color:rgba(66,32,6,.5)}.border-l-yellow-950\/55{border-left-color:rgba(66,32,6,.55)}.border-l-yellow-950\/60{border-left-color:rgba(66,32,6,.6)}.border-l-yellow-950\/65{border-left-color:rgba(66,32,6,.65)}.border-l-yellow-950\/70{border-left-color:rgba(66,32,6,.7)}.border-l-yellow-950\/75{border-left-color:rgba(66,32,6,.75)}.border-l-yellow-950\/80{border-left-color:rgba(66,32,6,.8)}.border-l-yellow-950\/85{border-left-color:rgba(66,32,6,.85)}.border-l-yellow-950\/90{border-left-color:rgba(66,32,6,.9)}.border-l-yellow-950\/95{border-left-color:rgba(66,32,6,.95)}.border-l-zinc-100{--tw-border-opacity:1;border-left-color:rgb(244 244 245/var(--tw-border-opacity,1))}.border-l-zinc-100\/0{border-left-color:hsla(240,5%,96%,0)}.border-l-zinc-100\/10{border-left-color:hsla(240,5%,96%,.1)}.border-l-zinc-100\/100{border-left-color:#f4f4f5}.border-l-zinc-100\/15{border-left-color:hsla(240,5%,96%,.15)}.border-l-zinc-100\/20{border-left-color:hsla(240,5%,96%,.2)}.border-l-zinc-100\/25{border-left-color:hsla(240,5%,96%,.25)}.border-l-zinc-100\/30{border-left-color:hsla(240,5%,96%,.3)}.border-l-zinc-100\/35{border-left-color:hsla(240,5%,96%,.35)}.border-l-zinc-100\/40{border-left-color:hsla(240,5%,96%,.4)}.border-l-zinc-100\/45{border-left-color:hsla(240,5%,96%,.45)}.border-l-zinc-100\/5{border-left-color:hsla(240,5%,96%,.05)}.border-l-zinc-100\/50{border-left-color:hsla(240,5%,96%,.5)}.border-l-zinc-100\/55{border-left-color:hsla(240,5%,96%,.55)}.border-l-zinc-100\/60{border-left-color:hsla(240,5%,96%,.6)}.border-l-zinc-100\/65{border-left-color:hsla(240,5%,96%,.65)}.border-l-zinc-100\/70{border-left-color:hsla(240,5%,96%,.7)}.border-l-zinc-100\/75{border-left-color:hsla(240,5%,96%,.75)}.border-l-zinc-100\/80{border-left-color:hsla(240,5%,96%,.8)}.border-l-zinc-100\/85{border-left-color:hsla(240,5%,96%,.85)}.border-l-zinc-100\/90{border-left-color:hsla(240,5%,96%,.9)}.border-l-zinc-100\/95{border-left-color:hsla(240,5%,96%,.95)}.border-l-zinc-200{--tw-border-opacity:1;border-left-color:rgb(228 228 231/var(--tw-border-opacity,1))}.border-l-zinc-200\/0{border-left-color:hsla(240,6%,90%,0)}.border-l-zinc-200\/10{border-left-color:hsla(240,6%,90%,.1)}.border-l-zinc-200\/100{border-left-color:#e4e4e7}.border-l-zinc-200\/15{border-left-color:hsla(240,6%,90%,.15)}.border-l-zinc-200\/20{border-left-color:hsla(240,6%,90%,.2)}.border-l-zinc-200\/25{border-left-color:hsla(240,6%,90%,.25)}.border-l-zinc-200\/30{border-left-color:hsla(240,6%,90%,.3)}.border-l-zinc-200\/35{border-left-color:hsla(240,6%,90%,.35)}.border-l-zinc-200\/40{border-left-color:hsla(240,6%,90%,.4)}.border-l-zinc-200\/45{border-left-color:hsla(240,6%,90%,.45)}.border-l-zinc-200\/5{border-left-color:hsla(240,6%,90%,.05)}.border-l-zinc-200\/50{border-left-color:hsla(240,6%,90%,.5)}.border-l-zinc-200\/55{border-left-color:hsla(240,6%,90%,.55)}.border-l-zinc-200\/60{border-left-color:hsla(240,6%,90%,.6)}.border-l-zinc-200\/65{border-left-color:hsla(240,6%,90%,.65)}.border-l-zinc-200\/70{border-left-color:hsla(240,6%,90%,.7)}.border-l-zinc-200\/75{border-left-color:hsla(240,6%,90%,.75)}.border-l-zinc-200\/80{border-left-color:hsla(240,6%,90%,.8)}.border-l-zinc-200\/85{border-left-color:hsla(240,6%,90%,.85)}.border-l-zinc-200\/90{border-left-color:hsla(240,6%,90%,.9)}.border-l-zinc-200\/95{border-left-color:hsla(240,6%,90%,.95)}.border-l-zinc-400{--tw-border-opacity:1;border-left-color:rgb(161 161 170/var(--tw-border-opacity,1))}.border-l-zinc-400\/0{border-left-color:hsla(240,5%,65%,0)}.border-l-zinc-400\/10{border-left-color:hsla(240,5%,65%,.1)}.border-l-zinc-400\/100{border-left-color:#a1a1aa}.border-l-zinc-400\/15{border-left-color:hsla(240,5%,65%,.15)}.border-l-zinc-400\/20{border-left-color:hsla(240,5%,65%,.2)}.border-l-zinc-400\/25{border-left-color:hsla(240,5%,65%,.25)}.border-l-zinc-400\/30{border-left-color:hsla(240,5%,65%,.3)}.border-l-zinc-400\/35{border-left-color:hsla(240,5%,65%,.35)}.border-l-zinc-400\/40{border-left-color:hsla(240,5%,65%,.4)}.border-l-zinc-400\/45{border-left-color:hsla(240,5%,65%,.45)}.border-l-zinc-400\/5{border-left-color:hsla(240,5%,65%,.05)}.border-l-zinc-400\/50{border-left-color:hsla(240,5%,65%,.5)}.border-l-zinc-400\/55{border-left-color:hsla(240,5%,65%,.55)}.border-l-zinc-400\/60{border-left-color:hsla(240,5%,65%,.6)}.border-l-zinc-400\/65{border-left-color:hsla(240,5%,65%,.65)}.border-l-zinc-400\/70{border-left-color:hsla(240,5%,65%,.7)}.border-l-zinc-400\/75{border-left-color:hsla(240,5%,65%,.75)}.border-l-zinc-400\/80{border-left-color:hsla(240,5%,65%,.8)}.border-l-zinc-400\/85{border-left-color:hsla(240,5%,65%,.85)}.border-l-zinc-400\/90{border-left-color:hsla(240,5%,65%,.9)}.border-l-zinc-400\/95{border-left-color:hsla(240,5%,65%,.95)}.border-l-zinc-50{--tw-border-opacity:1;border-left-color:rgb(250 250 250/var(--tw-border-opacity,1))}.border-l-zinc-50\/0{border-left-color:hsla(0,0%,98%,0)}.border-l-zinc-50\/10{border-left-color:hsla(0,0%,98%,.1)}.border-l-zinc-50\/100{border-left-color:#fafafa}.border-l-zinc-50\/15{border-left-color:hsla(0,0%,98%,.15)}.border-l-zinc-50\/20{border-left-color:hsla(0,0%,98%,.2)}.border-l-zinc-50\/25{border-left-color:hsla(0,0%,98%,.25)}.border-l-zinc-50\/30{border-left-color:hsla(0,0%,98%,.3)}.border-l-zinc-50\/35{border-left-color:hsla(0,0%,98%,.35)}.border-l-zinc-50\/40{border-left-color:hsla(0,0%,98%,.4)}.border-l-zinc-50\/45{border-left-color:hsla(0,0%,98%,.45)}.border-l-zinc-50\/5{border-left-color:hsla(0,0%,98%,.05)}.border-l-zinc-50\/50{border-left-color:hsla(0,0%,98%,.5)}.border-l-zinc-50\/55{border-left-color:hsla(0,0%,98%,.55)}.border-l-zinc-50\/60{border-left-color:hsla(0,0%,98%,.6)}.border-l-zinc-50\/65{border-left-color:hsla(0,0%,98%,.65)}.border-l-zinc-50\/70{border-left-color:hsla(0,0%,98%,.7)}.border-l-zinc-50\/75{border-left-color:hsla(0,0%,98%,.75)}.border-l-zinc-50\/80{border-left-color:hsla(0,0%,98%,.8)}.border-l-zinc-50\/85{border-left-color:hsla(0,0%,98%,.85)}.border-l-zinc-50\/90{border-left-color:hsla(0,0%,98%,.9)}.border-l-zinc-50\/95{border-left-color:hsla(0,0%,98%,.95)}.border-l-zinc-500{--tw-border-opacity:1;border-left-color:rgb(113 113 122/var(--tw-border-opacity,1))}.border-l-zinc-500\/0{border-left-color:hsla(240,4%,46%,0)}.border-l-zinc-500\/10{border-left-color:hsla(240,4%,46%,.1)}.border-l-zinc-500\/100{border-left-color:#71717a}.border-l-zinc-500\/15{border-left-color:hsla(240,4%,46%,.15)}.border-l-zinc-500\/20{border-left-color:hsla(240,4%,46%,.2)}.border-l-zinc-500\/25{border-left-color:hsla(240,4%,46%,.25)}.border-l-zinc-500\/30{border-left-color:hsla(240,4%,46%,.3)}.border-l-zinc-500\/35{border-left-color:hsla(240,4%,46%,.35)}.border-l-zinc-500\/40{border-left-color:hsla(240,4%,46%,.4)}.border-l-zinc-500\/45{border-left-color:hsla(240,4%,46%,.45)}.border-l-zinc-500\/5{border-left-color:hsla(240,4%,46%,.05)}.border-l-zinc-500\/50{border-left-color:hsla(240,4%,46%,.5)}.border-l-zinc-500\/55{border-left-color:hsla(240,4%,46%,.55)}.border-l-zinc-500\/60{border-left-color:hsla(240,4%,46%,.6)}.border-l-zinc-500\/65{border-left-color:hsla(240,4%,46%,.65)}.border-l-zinc-500\/70{border-left-color:hsla(240,4%,46%,.7)}.border-l-zinc-500\/75{border-left-color:hsla(240,4%,46%,.75)}.border-l-zinc-500\/80{border-left-color:hsla(240,4%,46%,.8)}.border-l-zinc-500\/85{border-left-color:hsla(240,4%,46%,.85)}.border-l-zinc-500\/90{border-left-color:hsla(240,4%,46%,.9)}.border-l-zinc-500\/95{border-left-color:hsla(240,4%,46%,.95)}.border-l-zinc-950{--tw-border-opacity:1;border-left-color:rgb(9 9 11/var(--tw-border-opacity,1))}.border-l-zinc-950\/0{border-left-color:rgba(9,9,11,0)}.border-l-zinc-950\/10{border-left-color:rgba(9,9,11,.1)}.border-l-zinc-950\/100{border-left-color:#09090b}.border-l-zinc-950\/15{border-left-color:rgba(9,9,11,.15)}.border-l-zinc-950\/20{border-left-color:rgba(9,9,11,.2)}.border-l-zinc-950\/25{border-left-color:rgba(9,9,11,.25)}.border-l-zinc-950\/30{border-left-color:rgba(9,9,11,.3)}.border-l-zinc-950\/35{border-left-color:rgba(9,9,11,.35)}.border-l-zinc-950\/40{border-left-color:rgba(9,9,11,.4)}.border-l-zinc-950\/45{border-left-color:rgba(9,9,11,.45)}.border-l-zinc-950\/5{border-left-color:rgba(9,9,11,.05)}.border-l-zinc-950\/50{border-left-color:rgba(9,9,11,.5)}.border-l-zinc-950\/55{border-left-color:rgba(9,9,11,.55)}.border-l-zinc-950\/60{border-left-color:rgba(9,9,11,.6)}.border-l-zinc-950\/65{border-left-color:rgba(9,9,11,.65)}.border-l-zinc-950\/70{border-left-color:rgba(9,9,11,.7)}.border-l-zinc-950\/75{border-left-color:rgba(9,9,11,.75)}.border-l-zinc-950\/80{border-left-color:rgba(9,9,11,.8)}.border-l-zinc-950\/85{border-left-color:rgba(9,9,11,.85)}.border-l-zinc-950\/90{border-left-color:rgba(9,9,11,.9)}.border-l-zinc-950\/95{border-left-color:rgba(9,9,11,.95)}.border-t-transparent{border-top-color:transparent}.border-opacity-50{--tw-border-opacity:0.5}.bg-\[hsl\(var\(--background\)\)\]{background-color:hsl(var(--background))}.bg-accent{background-color:hsl(var(--accent))}.bg-amber-100{--tw-bg-opacity:1;background-color:rgb(254 243 199/var(--tw-bg-opacity,1))}.bg-amber-100\/0{background-color:hsla(48,96%,89%,0)}.bg-amber-100\/10{background-color:hsla(48,96%,89%,.1)}.bg-amber-100\/100{background-color:#fef3c7}.bg-amber-100\/15{background-color:hsla(48,96%,89%,.15)}.bg-amber-100\/20{background-color:hsla(48,96%,89%,.2)}.bg-amber-100\/25{background-color:hsla(48,96%,89%,.25)}.bg-amber-100\/30{background-color:hsla(48,96%,89%,.3)}.bg-amber-100\/35{background-color:hsla(48,96%,89%,.35)}.bg-amber-100\/40{background-color:hsla(48,96%,89%,.4)}.bg-amber-100\/45{background-color:hsla(48,96%,89%,.45)}.bg-amber-100\/5{background-color:hsla(48,96%,89%,.05)}.bg-amber-100\/50{background-color:hsla(48,96%,89%,.5)}.bg-amber-100\/55{background-color:hsla(48,96%,89%,.55)}.bg-amber-100\/60{background-color:hsla(48,96%,89%,.6)}.bg-amber-100\/65{background-color:hsla(48,96%,89%,.65)}.bg-amber-100\/70{background-color:hsla(48,96%,89%,.7)}.bg-amber-100\/75{background-color:hsla(48,96%,89%,.75)}.bg-amber-100\/80{background-color:hsla(48,96%,89%,.8)}.bg-amber-100\/85{background-color:hsla(48,96%,89%,.85)}.bg-amber-100\/90{background-color:hsla(48,96%,89%,.9)}.bg-amber-100\/95{background-color:hsla(48,96%,89%,.95)}.bg-amber-200{--tw-bg-opacity:1;background-color:rgb(253 230 138/var(--tw-bg-opacity,1))}.bg-amber-200\/0{background-color:hsla(48,97%,77%,0)}.bg-amber-200\/10{background-color:hsla(48,97%,77%,.1)}.bg-amber-200\/100{background-color:#fde68a}.bg-amber-200\/15{background-color:hsla(48,97%,77%,.15)}.bg-amber-200\/20{background-color:hsla(48,97%,77%,.2)}.bg-amber-200\/25{background-color:hsla(48,97%,77%,.25)}.bg-amber-200\/30{background-color:hsla(48,97%,77%,.3)}.bg-amber-200\/35{background-color:hsla(48,97%,77%,.35)}.bg-amber-200\/40{background-color:hsla(48,97%,77%,.4)}.bg-amber-200\/45{background-color:hsla(48,97%,77%,.45)}.bg-amber-200\/5{background-color:hsla(48,97%,77%,.05)}.bg-amber-200\/50{background-color:hsla(48,97%,77%,.5)}.bg-amber-200\/55{background-color:hsla(48,97%,77%,.55)}.bg-amber-200\/60{background-color:hsla(48,97%,77%,.6)}.bg-amber-200\/65{background-color:hsla(48,97%,77%,.65)}.bg-amber-200\/70{background-color:hsla(48,97%,77%,.7)}.bg-amber-200\/75{background-color:hsla(48,97%,77%,.75)}.bg-amber-200\/80{background-color:hsla(48,97%,77%,.8)}.bg-amber-200\/85{background-color:hsla(48,97%,77%,.85)}.bg-amber-200\/90{background-color:hsla(48,97%,77%,.9)}.bg-amber-200\/95{background-color:hsla(48,97%,77%,.95)}.bg-amber-400{--tw-bg-opacity:1;background-color:rgb(251 191 36/var(--tw-bg-opacity,1))}.bg-amber-400\/0{background-color:rgba(251,191,36,0)}.bg-amber-400\/10{background-color:rgba(251,191,36,.1)}.bg-amber-400\/100{background-color:#fbbf24}.bg-amber-400\/15{background-color:rgba(251,191,36,.15)}.bg-amber-400\/20{background-color:rgba(251,191,36,.2)}.bg-amber-400\/25{background-color:rgba(251,191,36,.25)}.bg-amber-400\/30{background-color:rgba(251,191,36,.3)}.bg-amber-400\/35{background-color:rgba(251,191,36,.35)}.bg-amber-400\/40{background-color:rgba(251,191,36,.4)}.bg-amber-400\/45{background-color:rgba(251,191,36,.45)}.bg-amber-400\/5{background-color:rgba(251,191,36,.05)}.bg-amber-400\/50{background-color:rgba(251,191,36,.5)}.bg-amber-400\/55{background-color:rgba(251,191,36,.55)}.bg-amber-400\/60{background-color:rgba(251,191,36,.6)}.bg-amber-400\/65{background-color:rgba(251,191,36,.65)}.bg-amber-400\/70{background-color:rgba(251,191,36,.7)}.bg-amber-400\/75{background-color:rgba(251,191,36,.75)}.bg-amber-400\/80{background-color:rgba(251,191,36,.8)}.bg-amber-400\/85{background-color:rgba(251,191,36,.85)}.bg-amber-400\/90{background-color:rgba(251,191,36,.9)}.bg-amber-400\/95{background-color:rgba(251,191,36,.95)}.bg-amber-50{--tw-bg-opacity:1;background-color:rgb(255 251 235/var(--tw-bg-opacity,1))}.bg-amber-50\/0{background-color:rgba(255,251,235,0)}.bg-amber-50\/10{background-color:rgba(255,251,235,.1)}.bg-amber-50\/100{background-color:#fffbeb}.bg-amber-50\/15{background-color:rgba(255,251,235,.15)}.bg-amber-50\/20{background-color:rgba(255,251,235,.2)}.bg-amber-50\/25{background-color:rgba(255,251,235,.25)}.bg-amber-50\/30{background-color:rgba(255,251,235,.3)}.bg-amber-50\/35{background-color:rgba(255,251,235,.35)}.bg-amber-50\/40{background-color:rgba(255,251,235,.4)}.bg-amber-50\/45{background-color:rgba(255,251,235,.45)}.bg-amber-50\/5{background-color:rgba(255,251,235,.05)}.bg-amber-50\/50{background-color:rgba(255,251,235,.5)}.bg-amber-50\/55{background-color:rgba(255,251,235,.55)}.bg-amber-50\/60{background-color:rgba(255,251,235,.6)}.bg-amber-50\/65{background-color:rgba(255,251,235,.65)}.bg-amber-50\/70{background-color:rgba(255,251,235,.7)}.bg-amber-50\/75{background-color:rgba(255,251,235,.75)}.bg-amber-50\/80{background-color:rgba(255,251,235,.8)}.bg-amber-50\/85{background-color:rgba(255,251,235,.85)}.bg-amber-50\/90{background-color:rgba(255,251,235,.9)}.bg-amber-50\/95{background-color:rgba(255,251,235,.95)}.bg-amber-500{--tw-bg-opacity:1;background-color:rgb(245 158 11/var(--tw-bg-opacity,1))}.bg-amber-500\/0{background-color:rgba(245,158,11,0)}.bg-amber-500\/10{background-color:rgba(245,158,11,.1)}.bg-amber-500\/100{background-color:#f59e0b}.bg-amber-500\/15{background-color:rgba(245,158,11,.15)}.bg-amber-500\/20{background-color:rgba(245,158,11,.2)}.bg-amber-500\/25{background-color:rgba(245,158,11,.25)}.bg-amber-500\/30{background-color:rgba(245,158,11,.3)}.bg-amber-500\/35{background-color:rgba(245,158,11,.35)}.bg-amber-500\/40{background-color:rgba(245,158,11,.4)}.bg-amber-500\/45{background-color:rgba(245,158,11,.45)}.bg-amber-500\/5{background-color:rgba(245,158,11,.05)}.bg-amber-500\/50{background-color:rgba(245,158,11,.5)}.bg-amber-500\/55{background-color:rgba(245,158,11,.55)}.bg-amber-500\/60{background-color:rgba(245,158,11,.6)}.bg-amber-500\/65{background-color:rgba(245,158,11,.65)}.bg-amber-500\/70{background-color:rgba(245,158,11,.7)}.bg-amber-500\/75{background-color:rgba(245,158,11,.75)}.bg-amber-500\/80{background-color:rgba(245,158,11,.8)}.bg-amber-500\/85{background-color:rgba(245,158,11,.85)}.bg-amber-500\/90{background-color:rgba(245,158,11,.9)}.bg-amber-500\/95{background-color:rgba(245,158,11,.95)}.bg-amber-950{--tw-bg-opacity:1;background-color:rgb(69 26 3/var(--tw-bg-opacity,1))}.bg-amber-950\/0{background-color:rgba(69,26,3,0)}.bg-amber-950\/10{background-color:rgba(69,26,3,.1)}.bg-amber-950\/100{background-color:#451a03}.bg-amber-950\/15{background-color:rgba(69,26,3,.15)}.bg-amber-950\/20{background-color:rgba(69,26,3,.2)}.bg-amber-950\/25{background-color:rgba(69,26,3,.25)}.bg-amber-950\/30{background-color:rgba(69,26,3,.3)}.bg-amber-950\/35{background-color:rgba(69,26,3,.35)}.bg-amber-950\/40{background-color:rgba(69,26,3,.4)}.bg-amber-950\/45{background-color:rgba(69,26,3,.45)}.bg-amber-950\/5{background-color:rgba(69,26,3,.05)}.bg-amber-950\/50{background-color:rgba(69,26,3,.5)}.bg-amber-950\/55{background-color:rgba(69,26,3,.55)}.bg-amber-950\/60{background-color:rgba(69,26,3,.6)}.bg-amber-950\/65{background-color:rgba(69,26,3,.65)}.bg-amber-950\/70{background-color:rgba(69,26,3,.7)}.bg-amber-950\/75{background-color:rgba(69,26,3,.75)}.bg-amber-950\/80{background-color:rgba(69,26,3,.8)}.bg-amber-950\/85{background-color:rgba(69,26,3,.85)}.bg-amber-950\/90{background-color:rgba(69,26,3,.9)}.bg-amber-950\/95{background-color:rgba(69,26,3,.95)}.bg-background{background-color:hsl(var(--background))}.bg-black{--tw-bg-opacity:1;background-color:rgb(0 0 0/var(--tw-bg-opacity,1))}.bg-black\/80{background-color:rgba(0,0,0,.8)}.bg-blue-100{--tw-bg-opacity:1;background-color:rgb(219 234 254/var(--tw-bg-opacity,1))}.bg-blue-100\/0{background-color:rgba(219,234,254,0)}.bg-blue-100\/10{background-color:rgba(219,234,254,.1)}.bg-blue-100\/100{background-color:#dbeafe}.bg-blue-100\/15{background-color:rgba(219,234,254,.15)}.bg-blue-100\/20{background-color:rgba(219,234,254,.2)}.bg-blue-100\/25{background-color:rgba(219,234,254,.25)}.bg-blue-100\/30{background-color:rgba(219,234,254,.3)}.bg-blue-100\/35{background-color:rgba(219,234,254,.35)}.bg-blue-100\/40{background-color:rgba(219,234,254,.4)}.bg-blue-100\/45{background-color:rgba(219,234,254,.45)}.bg-blue-100\/5{background-color:rgba(219,234,254,.05)}.bg-blue-100\/50{background-color:rgba(219,234,254,.5)}.bg-blue-100\/55{background-color:rgba(219,234,254,.55)}.bg-blue-100\/60{background-color:rgba(219,234,254,.6)}.bg-blue-100\/65{background-color:rgba(219,234,254,.65)}.bg-blue-100\/70{background-color:rgba(219,234,254,.7)}.bg-blue-100\/75{background-color:rgba(219,234,254,.75)}.bg-blue-100\/80{background-color:rgba(219,234,254,.8)}.bg-blue-100\/85{background-color:rgba(219,234,254,.85)}.bg-blue-100\/90{background-color:rgba(219,234,254,.9)}.bg-blue-100\/95{background-color:rgba(219,234,254,.95)}.bg-blue-200{--tw-bg-opacity:1;background-color:rgb(191 219 254/var(--tw-bg-opacity,1))}.bg-blue-200\/0{background-color:rgba(191,219,254,0)}.bg-blue-200\/10{background-color:rgba(191,219,254,.1)}.bg-blue-200\/100{background-color:#bfdbfe}.bg-blue-200\/15{background-color:rgba(191,219,254,.15)}.bg-blue-200\/20{background-color:rgba(191,219,254,.2)}.bg-blue-200\/25{background-color:rgba(191,219,254,.25)}.bg-blue-200\/30{background-color:rgba(191,219,254,.3)}.bg-blue-200\/35{background-color:rgba(191,219,254,.35)}.bg-blue-200\/40{background-color:rgba(191,219,254,.4)}.bg-blue-200\/45{background-color:rgba(191,219,254,.45)}.bg-blue-200\/5{background-color:rgba(191,219,254,.05)}.bg-blue-200\/50{background-color:rgba(191,219,254,.5)}.bg-blue-200\/55{background-color:rgba(191,219,254,.55)}.bg-blue-200\/60{background-color:rgba(191,219,254,.6)}.bg-blue-200\/65{background-color:rgba(191,219,254,.65)}.bg-blue-200\/70{background-color:rgba(191,219,254,.7)}.bg-blue-200\/75{background-color:rgba(191,219,254,.75)}.bg-blue-200\/80{background-color:rgba(191,219,254,.8)}.bg-blue-200\/85{background-color:rgba(191,219,254,.85)}.bg-blue-200\/90{background-color:rgba(191,219,254,.9)}.bg-blue-200\/95{background-color:rgba(191,219,254,.95)}.bg-blue-400{--tw-bg-opacity:1;background-color:rgb(96 165 250/var(--tw-bg-opacity,1))}.bg-blue-400\/0{background-color:rgba(96,165,250,0)}.bg-blue-400\/10{background-color:rgba(96,165,250,.1)}.bg-blue-400\/100{background-color:#60a5fa}.bg-blue-400\/15{background-color:rgba(96,165,250,.15)}.bg-blue-400\/20{background-color:rgba(96,165,250,.2)}.bg-blue-400\/25{background-color:rgba(96,165,250,.25)}.bg-blue-400\/30{background-color:rgba(96,165,250,.3)}.bg-blue-400\/35{background-color:rgba(96,165,250,.35)}.bg-blue-400\/40{background-color:rgba(96,165,250,.4)}.bg-blue-400\/45{background-color:rgba(96,165,250,.45)}.bg-blue-400\/5{background-color:rgba(96,165,250,.05)}.bg-blue-400\/50{background-color:rgba(96,165,250,.5)}.bg-blue-400\/55{background-color:rgba(96,165,250,.55)}.bg-blue-400\/60{background-color:rgba(96,165,250,.6)}.bg-blue-400\/65{background-color:rgba(96,165,250,.65)}.bg-blue-400\/70{background-color:rgba(96,165,250,.7)}.bg-blue-400\/75{background-color:rgba(96,165,250,.75)}.bg-blue-400\/80{background-color:rgba(96,165,250,.8)}.bg-blue-400\/85{background-color:rgba(96,165,250,.85)}.bg-blue-400\/90{background-color:rgba(96,165,250,.9)}.bg-blue-400\/95{background-color:rgba(96,165,250,.95)}.bg-blue-50{--tw-bg-opacity:1;background-color:rgb(239 246 255/var(--tw-bg-opacity,1))}.bg-blue-50\/0{background-color:rgba(239,246,255,0)}.bg-blue-50\/10{background-color:rgba(239,246,255,.1)}.bg-blue-50\/100{background-color:#eff6ff}.bg-blue-50\/15{background-color:rgba(239,246,255,.15)}.bg-blue-50\/20{background-color:rgba(239,246,255,.2)}.bg-blue-50\/25{background-color:rgba(239,246,255,.25)}.bg-blue-50\/30{background-color:rgba(239,246,255,.3)}.bg-blue-50\/35{background-color:rgba(239,246,255,.35)}.bg-blue-50\/40{background-color:rgba(239,246,255,.4)}.bg-blue-50\/45{background-color:rgba(239,246,255,.45)}.bg-blue-50\/5{background-color:rgba(239,246,255,.05)}.bg-blue-50\/50{background-color:rgba(239,246,255,.5)}.bg-blue-50\/55{background-color:rgba(239,246,255,.55)}.bg-blue-50\/60{background-color:rgba(239,246,255,.6)}.bg-blue-50\/65{background-color:rgba(239,246,255,.65)}.bg-blue-50\/70{background-color:rgba(239,246,255,.7)}.bg-blue-50\/75{background-color:rgba(239,246,255,.75)}.bg-blue-50\/80{background-color:rgba(239,246,255,.8)}.bg-blue-50\/85{background-color:rgba(239,246,255,.85)}.bg-blue-50\/90{background-color:rgba(239,246,255,.9)}.bg-blue-50\/95{background-color:rgba(239,246,255,.95)}.bg-blue-500{--tw-bg-opacity:1;background-color:rgb(59 130 246/var(--tw-bg-opacity,1))}.bg-blue-500\/0{background-color:rgba(59,130,246,0)}.bg-blue-500\/10{background-color:rgba(59,130,246,.1)}.bg-blue-500\/100{background-color:#3b82f6}.bg-blue-500\/15{background-color:rgba(59,130,246,.15)}.bg-blue-500\/20{background-color:rgba(59,130,246,.2)}.bg-blue-500\/25{background-color:rgba(59,130,246,.25)}.bg-blue-500\/30{background-color:rgba(59,130,246,.3)}.bg-blue-500\/35{background-color:rgba(59,130,246,.35)}.bg-blue-500\/40{background-color:rgba(59,130,246,.4)}.bg-blue-500\/45{background-color:rgba(59,130,246,.45)}.bg-blue-500\/5{background-color:rgba(59,130,246,.05)}.bg-blue-500\/50{background-color:rgba(59,130,246,.5)}.bg-blue-500\/55{background-color:rgba(59,130,246,.55)}.bg-blue-500\/60{background-color:rgba(59,130,246,.6)}.bg-blue-500\/65{background-color:rgba(59,130,246,.65)}.bg-blue-500\/70{background-color:rgba(59,130,246,.7)}.bg-blue-500\/75{background-color:rgba(59,130,246,.75)}.bg-blue-500\/80{background-color:rgba(59,130,246,.8)}.bg-blue-500\/85{background-color:rgba(59,130,246,.85)}.bg-blue-500\/90{background-color:rgba(59,130,246,.9)}.bg-blue-500\/95{background-color:rgba(59,130,246,.95)}.bg-blue-950{--tw-bg-opacity:1;background-color:rgb(23 37 84/var(--tw-bg-opacity,1))}.bg-blue-950\/0{background-color:rgba(23,37,84,0)}.bg-blue-950\/10{background-color:rgba(23,37,84,.1)}.bg-blue-950\/100{background-color:#172554}.bg-blue-950\/15{background-color:rgba(23,37,84,.15)}.bg-blue-950\/20{background-color:rgba(23,37,84,.2)}.bg-blue-950\/25{background-color:rgba(23,37,84,.25)}.bg-blue-950\/30{background-color:rgba(23,37,84,.3)}.bg-blue-950\/35{background-color:rgba(23,37,84,.35)}.bg-blue-950\/40{background-color:rgba(23,37,84,.4)}.bg-blue-950\/45{background-color:rgba(23,37,84,.45)}.bg-blue-950\/5{background-color:rgba(23,37,84,.05)}.bg-blue-950\/50{background-color:rgba(23,37,84,.5)}.bg-blue-950\/55{background-color:rgba(23,37,84,.55)}.bg-blue-950\/60{background-color:rgba(23,37,84,.6)}.bg-blue-950\/65{background-color:rgba(23,37,84,.65)}.bg-blue-950\/70{background-color:rgba(23,37,84,.7)}.bg-blue-950\/75{background-color:rgba(23,37,84,.75)}.bg-blue-950\/80{background-color:rgba(23,37,84,.8)}.bg-blue-950\/85{background-color:rgba(23,37,84,.85)}.bg-blue-950\/90{background-color:rgba(23,37,84,.9)}.bg-blue-950\/95{background-color:rgba(23,37,84,.95)}.bg-border{background-color:hsl(var(--border))}.bg-card{background-color:hsl(var(--card))}.bg-cyan-100{--tw-bg-opacity:1;background-color:rgb(207 250 254/var(--tw-bg-opacity,1))}.bg-cyan-100\/0{background-color:rgba(207,250,254,0)}.bg-cyan-100\/10{background-color:rgba(207,250,254,.1)}.bg-cyan-100\/100{background-color:#cffafe}.bg-cyan-100\/15{background-color:rgba(207,250,254,.15)}.bg-cyan-100\/20{background-color:rgba(207,250,254,.2)}.bg-cyan-100\/25{background-color:rgba(207,250,254,.25)}.bg-cyan-100\/30{background-color:rgba(207,250,254,.3)}.bg-cyan-100\/35{background-color:rgba(207,250,254,.35)}.bg-cyan-100\/40{background-color:rgba(207,250,254,.4)}.bg-cyan-100\/45{background-color:rgba(207,250,254,.45)}.bg-cyan-100\/5{background-color:rgba(207,250,254,.05)}.bg-cyan-100\/50{background-color:rgba(207,250,254,.5)}.bg-cyan-100\/55{background-color:rgba(207,250,254,.55)}.bg-cyan-100\/60{background-color:rgba(207,250,254,.6)}.bg-cyan-100\/65{background-color:rgba(207,250,254,.65)}.bg-cyan-100\/70{background-color:rgba(207,250,254,.7)}.bg-cyan-100\/75{background-color:rgba(207,250,254,.75)}.bg-cyan-100\/80{background-color:rgba(207,250,254,.8)}.bg-cyan-100\/85{background-color:rgba(207,250,254,.85)}.bg-cyan-100\/90{background-color:rgba(207,250,254,.9)}.bg-cyan-100\/95{background-color:rgba(207,250,254,.95)}.bg-cyan-200{--tw-bg-opacity:1;background-color:rgb(165 243 252/var(--tw-bg-opacity,1))}.bg-cyan-200\/0{background-color:rgba(165,243,252,0)}.bg-cyan-200\/10{background-color:rgba(165,243,252,.1)}.bg-cyan-200\/100{background-color:#a5f3fc}.bg-cyan-200\/15{background-color:rgba(165,243,252,.15)}.bg-cyan-200\/20{background-color:rgba(165,243,252,.2)}.bg-cyan-200\/25{background-color:rgba(165,243,252,.25)}.bg-cyan-200\/30{background-color:rgba(165,243,252,.3)}.bg-cyan-200\/35{background-color:rgba(165,243,252,.35)}.bg-cyan-200\/40{background-color:rgba(165,243,252,.4)}.bg-cyan-200\/45{background-color:rgba(165,243,252,.45)}.bg-cyan-200\/5{background-color:rgba(165,243,252,.05)}.bg-cyan-200\/50{background-color:rgba(165,243,252,.5)}.bg-cyan-200\/55{background-color:rgba(165,243,252,.55)}.bg-cyan-200\/60{background-color:rgba(165,243,252,.6)}.bg-cyan-200\/65{background-color:rgba(165,243,252,.65)}.bg-cyan-200\/70{background-color:rgba(165,243,252,.7)}.bg-cyan-200\/75{background-color:rgba(165,243,252,.75)}.bg-cyan-200\/80{background-color:rgba(165,243,252,.8)}.bg-cyan-200\/85{background-color:rgba(165,243,252,.85)}.bg-cyan-200\/90{background-color:rgba(165,243,252,.9)}.bg-cyan-200\/95{background-color:rgba(165,243,252,.95)}.bg-cyan-400{--tw-bg-opacity:1;background-color:rgb(34 211 238/var(--tw-bg-opacity,1))}.bg-cyan-400\/0{background-color:rgba(34,211,238,0)}.bg-cyan-400\/10{background-color:rgba(34,211,238,.1)}.bg-cyan-400\/100{background-color:#22d3ee}.bg-cyan-400\/15{background-color:rgba(34,211,238,.15)}.bg-cyan-400\/20{background-color:rgba(34,211,238,.2)}.bg-cyan-400\/25{background-color:rgba(34,211,238,.25)}.bg-cyan-400\/30{background-color:rgba(34,211,238,.3)}.bg-cyan-400\/35{background-color:rgba(34,211,238,.35)}.bg-cyan-400\/40{background-color:rgba(34,211,238,.4)}.bg-cyan-400\/45{background-color:rgba(34,211,238,.45)}.bg-cyan-400\/5{background-color:rgba(34,211,238,.05)}.bg-cyan-400\/50{background-color:rgba(34,211,238,.5)}.bg-cyan-400\/55{background-color:rgba(34,211,238,.55)}.bg-cyan-400\/60{background-color:rgba(34,211,238,.6)}.bg-cyan-400\/65{background-color:rgba(34,211,238,.65)}.bg-cyan-400\/70{background-color:rgba(34,211,238,.7)}.bg-cyan-400\/75{background-color:rgba(34,211,238,.75)}.bg-cyan-400\/80{background-color:rgba(34,211,238,.8)}.bg-cyan-400\/85{background-color:rgba(34,211,238,.85)}.bg-cyan-400\/90{background-color:rgba(34,211,238,.9)}.bg-cyan-400\/95{background-color:rgba(34,211,238,.95)}.bg-cyan-50{--tw-bg-opacity:1;background-color:rgb(236 254 255/var(--tw-bg-opacity,1))}.bg-cyan-50\/0{background-color:rgba(236,254,255,0)}.bg-cyan-50\/10{background-color:rgba(236,254,255,.1)}.bg-cyan-50\/100{background-color:#ecfeff}.bg-cyan-50\/15{background-color:rgba(236,254,255,.15)}.bg-cyan-50\/20{background-color:rgba(236,254,255,.2)}.bg-cyan-50\/25{background-color:rgba(236,254,255,.25)}.bg-cyan-50\/30{background-color:rgba(236,254,255,.3)}.bg-cyan-50\/35{background-color:rgba(236,254,255,.35)}.bg-cyan-50\/40{background-color:rgba(236,254,255,.4)}.bg-cyan-50\/45{background-color:rgba(236,254,255,.45)}.bg-cyan-50\/5{background-color:rgba(236,254,255,.05)}.bg-cyan-50\/50{background-color:rgba(236,254,255,.5)}.bg-cyan-50\/55{background-color:rgba(236,254,255,.55)}.bg-cyan-50\/60{background-color:rgba(236,254,255,.6)}.bg-cyan-50\/65{background-color:rgba(236,254,255,.65)}.bg-cyan-50\/70{background-color:rgba(236,254,255,.7)}.bg-cyan-50\/75{background-color:rgba(236,254,255,.75)}.bg-cyan-50\/80{background-color:rgba(236,254,255,.8)}.bg-cyan-50\/85{background-color:rgba(236,254,255,.85)}.bg-cyan-50\/90{background-color:rgba(236,254,255,.9)}.bg-cyan-50\/95{background-color:rgba(236,254,255,.95)}.bg-cyan-500{--tw-bg-opacity:1;background-color:rgb(6 182 212/var(--tw-bg-opacity,1))}.bg-cyan-500\/0{background-color:rgba(6,182,212,0)}.bg-cyan-500\/10{background-color:rgba(6,182,212,.1)}.bg-cyan-500\/100{background-color:#06b6d4}.bg-cyan-500\/15{background-color:rgba(6,182,212,.15)}.bg-cyan-500\/20{background-color:rgba(6,182,212,.2)}.bg-cyan-500\/25{background-color:rgba(6,182,212,.25)}.bg-cyan-500\/30{background-color:rgba(6,182,212,.3)}.bg-cyan-500\/35{background-color:rgba(6,182,212,.35)}.bg-cyan-500\/40{background-color:rgba(6,182,212,.4)}.bg-cyan-500\/45{background-color:rgba(6,182,212,.45)}.bg-cyan-500\/5{background-color:rgba(6,182,212,.05)}.bg-cyan-500\/50{background-color:rgba(6,182,212,.5)}.bg-cyan-500\/55{background-color:rgba(6,182,212,.55)}.bg-cyan-500\/60{background-color:rgba(6,182,212,.6)}.bg-cyan-500\/65{background-color:rgba(6,182,212,.65)}.bg-cyan-500\/70{background-color:rgba(6,182,212,.7)}.bg-cyan-500\/75{background-color:rgba(6,182,212,.75)}.bg-cyan-500\/80{background-color:rgba(6,182,212,.8)}.bg-cyan-500\/85{background-color:rgba(6,182,212,.85)}.bg-cyan-500\/90{background-color:rgba(6,182,212,.9)}.bg-cyan-500\/95{background-color:rgba(6,182,212,.95)}.bg-cyan-950{--tw-bg-opacity:1;background-color:rgb(8 51 68/var(--tw-bg-opacity,1))}.bg-cyan-950\/0{background-color:rgba(8,51,68,0)}.bg-cyan-950\/10{background-color:rgba(8,51,68,.1)}.bg-cyan-950\/100{background-color:#083344}.bg-cyan-950\/15{background-color:rgba(8,51,68,.15)}.bg-cyan-950\/20{background-color:rgba(8,51,68,.2)}.bg-cyan-950\/25{background-color:rgba(8,51,68,.25)}.bg-cyan-950\/30{background-color:rgba(8,51,68,.3)}.bg-cyan-950\/35{background-color:rgba(8,51,68,.35)}.bg-cyan-950\/40{background-color:rgba(8,51,68,.4)}.bg-cyan-950\/45{background-color:rgba(8,51,68,.45)}.bg-cyan-950\/5{background-color:rgba(8,51,68,.05)}.bg-cyan-950\/50{background-color:rgba(8,51,68,.5)}.bg-cyan-950\/55{background-color:rgba(8,51,68,.55)}.bg-cyan-950\/60{background-color:rgba(8,51,68,.6)}.bg-cyan-950\/65{background-color:rgba(8,51,68,.65)}.bg-cyan-950\/70{background-color:rgba(8,51,68,.7)}.bg-cyan-950\/75{background-color:rgba(8,51,68,.75)}.bg-cyan-950\/80{background-color:rgba(8,51,68,.8)}.bg-cyan-950\/85{background-color:rgba(8,51,68,.85)}.bg-cyan-950\/90{background-color:rgba(8,51,68,.9)}.bg-cyan-950\/95{background-color:rgba(8,51,68,.95)}.bg-destructive{background-color:hsl(var(--destructive))}.bg-emerald-100{--tw-bg-opacity:1;background-color:rgb(209 250 229/var(--tw-bg-opacity,1))}.bg-emerald-100\/0{background-color:rgba(209,250,229,0)}.bg-emerald-100\/10{background-color:rgba(209,250,229,.1)}.bg-emerald-100\/100{background-color:#d1fae5}.bg-emerald-100\/15{background-color:rgba(209,250,229,.15)}.bg-emerald-100\/20{background-color:rgba(209,250,229,.2)}.bg-emerald-100\/25{background-color:rgba(209,250,229,.25)}.bg-emerald-100\/30{background-color:rgba(209,250,229,.3)}.bg-emerald-100\/35{background-color:rgba(209,250,229,.35)}.bg-emerald-100\/40{background-color:rgba(209,250,229,.4)}.bg-emerald-100\/45{background-color:rgba(209,250,229,.45)}.bg-emerald-100\/5{background-color:rgba(209,250,229,.05)}.bg-emerald-100\/50{background-color:rgba(209,250,229,.5)}.bg-emerald-100\/55{background-color:rgba(209,250,229,.55)}.bg-emerald-100\/60{background-color:rgba(209,250,229,.6)}.bg-emerald-100\/65{background-color:rgba(209,250,229,.65)}.bg-emerald-100\/70{background-color:rgba(209,250,229,.7)}.bg-emerald-100\/75{background-color:rgba(209,250,229,.75)}.bg-emerald-100\/80{background-color:rgba(209,250,229,.8)}.bg-emerald-100\/85{background-color:rgba(209,250,229,.85)}.bg-emerald-100\/90{background-color:rgba(209,250,229,.9)}.bg-emerald-100\/95{background-color:rgba(209,250,229,.95)}.bg-emerald-200{--tw-bg-opacity:1;background-color:rgb(167 243 208/var(--tw-bg-opacity,1))}.bg-emerald-200\/0{background-color:rgba(167,243,208,0)}.bg-emerald-200\/10{background-color:rgba(167,243,208,.1)}.bg-emerald-200\/100{background-color:#a7f3d0}.bg-emerald-200\/15{background-color:rgba(167,243,208,.15)}.bg-emerald-200\/20{background-color:rgba(167,243,208,.2)}.bg-emerald-200\/25{background-color:rgba(167,243,208,.25)}.bg-emerald-200\/30{background-color:rgba(167,243,208,.3)}.bg-emerald-200\/35{background-color:rgba(167,243,208,.35)}.bg-emerald-200\/40{background-color:rgba(167,243,208,.4)}.bg-emerald-200\/45{background-color:rgba(167,243,208,.45)}.bg-emerald-200\/5{background-color:rgba(167,243,208,.05)}.bg-emerald-200\/50{background-color:rgba(167,243,208,.5)}.bg-emerald-200\/55{background-color:rgba(167,243,208,.55)}.bg-emerald-200\/60{background-color:rgba(167,243,208,.6)}.bg-emerald-200\/65{background-color:rgba(167,243,208,.65)}.bg-emerald-200\/70{background-color:rgba(167,243,208,.7)}.bg-emerald-200\/75{background-color:rgba(167,243,208,.75)}.bg-emerald-200\/80{background-color:rgba(167,243,208,.8)}.bg-emerald-200\/85{background-color:rgba(167,243,208,.85)}.bg-emerald-200\/90{background-color:rgba(167,243,208,.9)}.bg-emerald-200\/95{background-color:rgba(167,243,208,.95)}.bg-emerald-400{--tw-bg-opacity:1;background-color:rgb(52 211 153/var(--tw-bg-opacity,1))}.bg-emerald-400\/0{background-color:rgba(52,211,153,0)}.bg-emerald-400\/10{background-color:rgba(52,211,153,.1)}.bg-emerald-400\/100{background-color:#34d399}.bg-emerald-400\/15{background-color:rgba(52,211,153,.15)}.bg-emerald-400\/20{background-color:rgba(52,211,153,.2)}.bg-emerald-400\/25{background-color:rgba(52,211,153,.25)}.bg-emerald-400\/30{background-color:rgba(52,211,153,.3)}.bg-emerald-400\/35{background-color:rgba(52,211,153,.35)}.bg-emerald-400\/40{background-color:rgba(52,211,153,.4)}.bg-emerald-400\/45{background-color:rgba(52,211,153,.45)}.bg-emerald-400\/5{background-color:rgba(52,211,153,.05)}.bg-emerald-400\/50{background-color:rgba(52,211,153,.5)}.bg-emerald-400\/55{background-color:rgba(52,211,153,.55)}.bg-emerald-400\/60{background-color:rgba(52,211,153,.6)}.bg-emerald-400\/65{background-color:rgba(52,211,153,.65)}.bg-emerald-400\/70{background-color:rgba(52,211,153,.7)}.bg-emerald-400\/75{background-color:rgba(52,211,153,.75)}.bg-emerald-400\/80{background-color:rgba(52,211,153,.8)}.bg-emerald-400\/85{background-color:rgba(52,211,153,.85)}.bg-emerald-400\/90{background-color:rgba(52,211,153,.9)}.bg-emerald-400\/95{background-color:rgba(52,211,153,.95)}.bg-emerald-50{--tw-bg-opacity:1;background-color:rgb(236 253 245/var(--tw-bg-opacity,1))}.bg-emerald-50\/0{background-color:rgba(236,253,245,0)}.bg-emerald-50\/10{background-color:rgba(236,253,245,.1)}.bg-emerald-50\/100{background-color:#ecfdf5}.bg-emerald-50\/15{background-color:rgba(236,253,245,.15)}.bg-emerald-50\/20{background-color:rgba(236,253,245,.2)}.bg-emerald-50\/25{background-color:rgba(236,253,245,.25)}.bg-emerald-50\/30{background-color:rgba(236,253,245,.3)}.bg-emerald-50\/35{background-color:rgba(236,253,245,.35)}.bg-emerald-50\/40{background-color:rgba(236,253,245,.4)}.bg-emerald-50\/45{background-color:rgba(236,253,245,.45)}.bg-emerald-50\/5{background-color:rgba(236,253,245,.05)}.bg-emerald-50\/50{background-color:rgba(236,253,245,.5)}.bg-emerald-50\/55{background-color:rgba(236,253,245,.55)}.bg-emerald-50\/60{background-color:rgba(236,253,245,.6)}.bg-emerald-50\/65{background-color:rgba(236,253,245,.65)}.bg-emerald-50\/70{background-color:rgba(236,253,245,.7)}.bg-emerald-50\/75{background-color:rgba(236,253,245,.75)}.bg-emerald-50\/80{background-color:rgba(236,253,245,.8)}.bg-emerald-50\/85{background-color:rgba(236,253,245,.85)}.bg-emerald-50\/90{background-color:rgba(236,253,245,.9)}.bg-emerald-50\/95{background-color:rgba(236,253,245,.95)}.bg-emerald-500{--tw-bg-opacity:1;background-color:rgb(16 185 129/var(--tw-bg-opacity,1))}.bg-emerald-500\/0{background-color:rgba(16,185,129,0)}.bg-emerald-500\/10{background-color:rgba(16,185,129,.1)}.bg-emerald-500\/100{background-color:#10b981}.bg-emerald-500\/15{background-color:rgba(16,185,129,.15)}.bg-emerald-500\/20{background-color:rgba(16,185,129,.2)}.bg-emerald-500\/25{background-color:rgba(16,185,129,.25)}.bg-emerald-500\/30{background-color:rgba(16,185,129,.3)}.bg-emerald-500\/35{background-color:rgba(16,185,129,.35)}.bg-emerald-500\/40{background-color:rgba(16,185,129,.4)}.bg-emerald-500\/45{background-color:rgba(16,185,129,.45)}.bg-emerald-500\/5{background-color:rgba(16,185,129,.05)}.bg-emerald-500\/50{background-color:rgba(16,185,129,.5)}.bg-emerald-500\/55{background-color:rgba(16,185,129,.55)}.bg-emerald-500\/60{background-color:rgba(16,185,129,.6)}.bg-emerald-500\/65{background-color:rgba(16,185,129,.65)}.bg-emerald-500\/70{background-color:rgba(16,185,129,.7)}.bg-emerald-500\/75{background-color:rgba(16,185,129,.75)}.bg-emerald-500\/80{background-color:rgba(16,185,129,.8)}.bg-emerald-500\/85{background-color:rgba(16,185,129,.85)}.bg-emerald-500\/90{background-color:rgba(16,185,129,.9)}.bg-emerald-500\/95{background-color:rgba(16,185,129,.95)}.bg-emerald-950{--tw-bg-opacity:1;background-color:rgb(2 44 34/var(--tw-bg-opacity,1))}.bg-emerald-950\/0{background-color:rgba(2,44,34,0)}.bg-emerald-950\/10{background-color:rgba(2,44,34,.1)}.bg-emerald-950\/100{background-color:#022c22}.bg-emerald-950\/15{background-color:rgba(2,44,34,.15)}.bg-emerald-950\/20{background-color:rgba(2,44,34,.2)}.bg-emerald-950\/25{background-color:rgba(2,44,34,.25)}.bg-emerald-950\/30{background-color:rgba(2,44,34,.3)}.bg-emerald-950\/35{background-color:rgba(2,44,34,.35)}.bg-emerald-950\/40{background-color:rgba(2,44,34,.4)}.bg-emerald-950\/45{background-color:rgba(2,44,34,.45)}.bg-emerald-950\/5{background-color:rgba(2,44,34,.05)}.bg-emerald-950\/50{background-color:rgba(2,44,34,.5)}.bg-emerald-950\/55{background-color:rgba(2,44,34,.55)}.bg-emerald-950\/60{background-color:rgba(2,44,34,.6)}.bg-emerald-950\/65{background-color:rgba(2,44,34,.65)}.bg-emerald-950\/70{background-color:rgba(2,44,34,.7)}.bg-emerald-950\/75{background-color:rgba(2,44,34,.75)}.bg-emerald-950\/80{background-color:rgba(2,44,34,.8)}.bg-emerald-950\/85{background-color:rgba(2,44,34,.85)}.bg-emerald-950\/90{background-color:rgba(2,44,34,.9)}.bg-emerald-950\/95{background-color:rgba(2,44,34,.95)}.bg-foreground{background-color:hsl(var(--foreground))}.bg-fuchsia-100{--tw-bg-opacity:1;background-color:rgb(250 232 255/var(--tw-bg-opacity,1))}.bg-fuchsia-100\/0{background-color:rgba(250,232,255,0)}.bg-fuchsia-100\/10{background-color:rgba(250,232,255,.1)}.bg-fuchsia-100\/100{background-color:#fae8ff}.bg-fuchsia-100\/15{background-color:rgba(250,232,255,.15)}.bg-fuchsia-100\/20{background-color:rgba(250,232,255,.2)}.bg-fuchsia-100\/25{background-color:rgba(250,232,255,.25)}.bg-fuchsia-100\/30{background-color:rgba(250,232,255,.3)}.bg-fuchsia-100\/35{background-color:rgba(250,232,255,.35)}.bg-fuchsia-100\/40{background-color:rgba(250,232,255,.4)}.bg-fuchsia-100\/45{background-color:rgba(250,232,255,.45)}.bg-fuchsia-100\/5{background-color:rgba(250,232,255,.05)}.bg-fuchsia-100\/50{background-color:rgba(250,232,255,.5)}.bg-fuchsia-100\/55{background-color:rgba(250,232,255,.55)}.bg-fuchsia-100\/60{background-color:rgba(250,232,255,.6)}.bg-fuchsia-100\/65{background-color:rgba(250,232,255,.65)}.bg-fuchsia-100\/70{background-color:rgba(250,232,255,.7)}.bg-fuchsia-100\/75{background-color:rgba(250,232,255,.75)}.bg-fuchsia-100\/80{background-color:rgba(250,232,255,.8)}.bg-fuchsia-100\/85{background-color:rgba(250,232,255,.85)}.bg-fuchsia-100\/90{background-color:rgba(250,232,255,.9)}.bg-fuchsia-100\/95{background-color:rgba(250,232,255,.95)}.bg-fuchsia-200{--tw-bg-opacity:1;background-color:rgb(245 208 254/var(--tw-bg-opacity,1))}.bg-fuchsia-200\/0{background-color:rgba(245,208,254,0)}.bg-fuchsia-200\/10{background-color:rgba(245,208,254,.1)}.bg-fuchsia-200\/100{background-color:#f5d0fe}.bg-fuchsia-200\/15{background-color:rgba(245,208,254,.15)}.bg-fuchsia-200\/20{background-color:rgba(245,208,254,.2)}.bg-fuchsia-200\/25{background-color:rgba(245,208,254,.25)}.bg-fuchsia-200\/30{background-color:rgba(245,208,254,.3)}.bg-fuchsia-200\/35{background-color:rgba(245,208,254,.35)}.bg-fuchsia-200\/40{background-color:rgba(245,208,254,.4)}.bg-fuchsia-200\/45{background-color:rgba(245,208,254,.45)}.bg-fuchsia-200\/5{background-color:rgba(245,208,254,.05)}.bg-fuchsia-200\/50{background-color:rgba(245,208,254,.5)}.bg-fuchsia-200\/55{background-color:rgba(245,208,254,.55)}.bg-fuchsia-200\/60{background-color:rgba(245,208,254,.6)}.bg-fuchsia-200\/65{background-color:rgba(245,208,254,.65)}.bg-fuchsia-200\/70{background-color:rgba(245,208,254,.7)}.bg-fuchsia-200\/75{background-color:rgba(245,208,254,.75)}.bg-fuchsia-200\/80{background-color:rgba(245,208,254,.8)}.bg-fuchsia-200\/85{background-color:rgba(245,208,254,.85)}.bg-fuchsia-200\/90{background-color:rgba(245,208,254,.9)}.bg-fuchsia-200\/95{background-color:rgba(245,208,254,.95)}.bg-fuchsia-400{--tw-bg-opacity:1;background-color:rgb(232 121 249/var(--tw-bg-opacity,1))}.bg-fuchsia-400\/0{background-color:rgba(232,121,249,0)}.bg-fuchsia-400\/10{background-color:rgba(232,121,249,.1)}.bg-fuchsia-400\/100{background-color:#e879f9}.bg-fuchsia-400\/15{background-color:rgba(232,121,249,.15)}.bg-fuchsia-400\/20{background-color:rgba(232,121,249,.2)}.bg-fuchsia-400\/25{background-color:rgba(232,121,249,.25)}.bg-fuchsia-400\/30{background-color:rgba(232,121,249,.3)}.bg-fuchsia-400\/35{background-color:rgba(232,121,249,.35)}.bg-fuchsia-400\/40{background-color:rgba(232,121,249,.4)}.bg-fuchsia-400\/45{background-color:rgba(232,121,249,.45)}.bg-fuchsia-400\/5{background-color:rgba(232,121,249,.05)}.bg-fuchsia-400\/50{background-color:rgba(232,121,249,.5)}.bg-fuchsia-400\/55{background-color:rgba(232,121,249,.55)}.bg-fuchsia-400\/60{background-color:rgba(232,121,249,.6)}.bg-fuchsia-400\/65{background-color:rgba(232,121,249,.65)}.bg-fuchsia-400\/70{background-color:rgba(232,121,249,.7)}.bg-fuchsia-400\/75{background-color:rgba(232,121,249,.75)}.bg-fuchsia-400\/80{background-color:rgba(232,121,249,.8)}.bg-fuchsia-400\/85{background-color:rgba(232,121,249,.85)}.bg-fuchsia-400\/90{background-color:rgba(232,121,249,.9)}.bg-fuchsia-400\/95{background-color:rgba(232,121,249,.95)}.bg-fuchsia-50{--tw-bg-opacity:1;background-color:rgb(253 244 255/var(--tw-bg-opacity,1))}.bg-fuchsia-50\/0{background-color:rgba(253,244,255,0)}.bg-fuchsia-50\/10{background-color:rgba(253,244,255,.1)}.bg-fuchsia-50\/100{background-color:#fdf4ff}.bg-fuchsia-50\/15{background-color:rgba(253,244,255,.15)}.bg-fuchsia-50\/20{background-color:rgba(253,244,255,.2)}.bg-fuchsia-50\/25{background-color:rgba(253,244,255,.25)}.bg-fuchsia-50\/30{background-color:rgba(253,244,255,.3)}.bg-fuchsia-50\/35{background-color:rgba(253,244,255,.35)}.bg-fuchsia-50\/40{background-color:rgba(253,244,255,.4)}.bg-fuchsia-50\/45{background-color:rgba(253,244,255,.45)}.bg-fuchsia-50\/5{background-color:rgba(253,244,255,.05)}.bg-fuchsia-50\/50{background-color:rgba(253,244,255,.5)}.bg-fuchsia-50\/55{background-color:rgba(253,244,255,.55)}.bg-fuchsia-50\/60{background-color:rgba(253,244,255,.6)}.bg-fuchsia-50\/65{background-color:rgba(253,244,255,.65)}.bg-fuchsia-50\/70{background-color:rgba(253,244,255,.7)}.bg-fuchsia-50\/75{background-color:rgba(253,244,255,.75)}.bg-fuchsia-50\/80{background-color:rgba(253,244,255,.8)}.bg-fuchsia-50\/85{background-color:rgba(253,244,255,.85)}.bg-fuchsia-50\/90{background-color:rgba(253,244,255,.9)}.bg-fuchsia-50\/95{background-color:rgba(253,244,255,.95)}.bg-fuchsia-500{--tw-bg-opacity:1;background-color:rgb(217 70 239/var(--tw-bg-opacity,1))}.bg-fuchsia-500\/0{background-color:rgba(217,70,239,0)}.bg-fuchsia-500\/10{background-color:rgba(217,70,239,.1)}.bg-fuchsia-500\/100{background-color:#d946ef}.bg-fuchsia-500\/15{background-color:rgba(217,70,239,.15)}.bg-fuchsia-500\/20{background-color:rgba(217,70,239,.2)}.bg-fuchsia-500\/25{background-color:rgba(217,70,239,.25)}.bg-fuchsia-500\/30{background-color:rgba(217,70,239,.3)}.bg-fuchsia-500\/35{background-color:rgba(217,70,239,.35)}.bg-fuchsia-500\/40{background-color:rgba(217,70,239,.4)}.bg-fuchsia-500\/45{background-color:rgba(217,70,239,.45)}.bg-fuchsia-500\/5{background-color:rgba(217,70,239,.05)}.bg-fuchsia-500\/50{background-color:rgba(217,70,239,.5)}.bg-fuchsia-500\/55{background-color:rgba(217,70,239,.55)}.bg-fuchsia-500\/60{background-color:rgba(217,70,239,.6)}.bg-fuchsia-500\/65{background-color:rgba(217,70,239,.65)}.bg-fuchsia-500\/70{background-color:rgba(217,70,239,.7)}.bg-fuchsia-500\/75{background-color:rgba(217,70,239,.75)}.bg-fuchsia-500\/80{background-color:rgba(217,70,239,.8)}.bg-fuchsia-500\/85{background-color:rgba(217,70,239,.85)}.bg-fuchsia-500\/90{background-color:rgba(217,70,239,.9)}.bg-fuchsia-500\/95{background-color:rgba(217,70,239,.95)}.bg-fuchsia-950{--tw-bg-opacity:1;background-color:rgb(74 4 78/var(--tw-bg-opacity,1))}.bg-fuchsia-950\/0{background-color:rgba(74,4,78,0)}.bg-fuchsia-950\/10{background-color:rgba(74,4,78,.1)}.bg-fuchsia-950\/100{background-color:#4a044e}.bg-fuchsia-950\/15{background-color:rgba(74,4,78,.15)}.bg-fuchsia-950\/20{background-color:rgba(74,4,78,.2)}.bg-fuchsia-950\/25{background-color:rgba(74,4,78,.25)}.bg-fuchsia-950\/30{background-color:rgba(74,4,78,.3)}.bg-fuchsia-950\/35{background-color:rgba(74,4,78,.35)}.bg-fuchsia-950\/40{background-color:rgba(74,4,78,.4)}.bg-fuchsia-950\/45{background-color:rgba(74,4,78,.45)}.bg-fuchsia-950\/5{background-color:rgba(74,4,78,.05)}.bg-fuchsia-950\/50{background-color:rgba(74,4,78,.5)}.bg-fuchsia-950\/55{background-color:rgba(74,4,78,.55)}.bg-fuchsia-950\/60{background-color:rgba(74,4,78,.6)}.bg-fuchsia-950\/65{background-color:rgba(74,4,78,.65)}.bg-fuchsia-950\/70{background-color:rgba(74,4,78,.7)}.bg-fuchsia-950\/75{background-color:rgba(74,4,78,.75)}.bg-fuchsia-950\/80{background-color:rgba(74,4,78,.8)}.bg-fuchsia-950\/85{background-color:rgba(74,4,78,.85)}.bg-fuchsia-950\/90{background-color:rgba(74,4,78,.9)}.bg-fuchsia-950\/95{background-color:rgba(74,4,78,.95)}.bg-gray-100{--tw-bg-opacity:1;background-color:rgb(243 244 246/var(--tw-bg-opacity,1))}.bg-gray-100\/0{background-color:rgba(243,244,246,0)}.bg-gray-100\/10{background-color:rgba(243,244,246,.1)}.bg-gray-100\/100{background-color:#f3f4f6}.bg-gray-100\/15{background-color:rgba(243,244,246,.15)}.bg-gray-100\/20{background-color:rgba(243,244,246,.2)}.bg-gray-100\/25{background-color:rgba(243,244,246,.25)}.bg-gray-100\/30{background-color:rgba(243,244,246,.3)}.bg-gray-100\/35{background-color:rgba(243,244,246,.35)}.bg-gray-100\/40{background-color:rgba(243,244,246,.4)}.bg-gray-100\/45{background-color:rgba(243,244,246,.45)}.bg-gray-100\/5{background-color:rgba(243,244,246,.05)}.bg-gray-100\/50{background-color:rgba(243,244,246,.5)}.bg-gray-100\/55{background-color:rgba(243,244,246,.55)}.bg-gray-100\/60{background-color:rgba(243,244,246,.6)}.bg-gray-100\/65{background-color:rgba(243,244,246,.65)}.bg-gray-100\/70{background-color:rgba(243,244,246,.7)}.bg-gray-100\/75{background-color:rgba(243,244,246,.75)}.bg-gray-100\/80{background-color:rgba(243,244,246,.8)}.bg-gray-100\/85{background-color:rgba(243,244,246,.85)}.bg-gray-100\/90{background-color:rgba(243,244,246,.9)}.bg-gray-100\/95{background-color:rgba(243,244,246,.95)}.bg-gray-200{--tw-bg-opacity:1;background-color:rgb(229 231 235/var(--tw-bg-opacity,1))}.bg-gray-200\/0{background-color:rgba(229,231,235,0)}.bg-gray-200\/10{background-color:rgba(229,231,235,.1)}.bg-gray-200\/100{background-color:#e5e7eb}.bg-gray-200\/15{background-color:rgba(229,231,235,.15)}.bg-gray-200\/20{background-color:rgba(229,231,235,.2)}.bg-gray-200\/25{background-color:rgba(229,231,235,.25)}.bg-gray-200\/30{background-color:rgba(229,231,235,.3)}.bg-gray-200\/35{background-color:rgba(229,231,235,.35)}.bg-gray-200\/40{background-color:rgba(229,231,235,.4)}.bg-gray-200\/45{background-color:rgba(229,231,235,.45)}.bg-gray-200\/5{background-color:rgba(229,231,235,.05)}.bg-gray-200\/50{background-color:rgba(229,231,235,.5)}.bg-gray-200\/55{background-color:rgba(229,231,235,.55)}.bg-gray-200\/60{background-color:rgba(229,231,235,.6)}.bg-gray-200\/65{background-color:rgba(229,231,235,.65)}.bg-gray-200\/70{background-color:rgba(229,231,235,.7)}.bg-gray-200\/75{background-color:rgba(229,231,235,.75)}.bg-gray-200\/80{background-color:rgba(229,231,235,.8)}.bg-gray-200\/85{background-color:rgba(229,231,235,.85)}.bg-gray-200\/90{background-color:rgba(229,231,235,.9)}.bg-gray-200\/95{background-color:rgba(229,231,235,.95)}.bg-gray-400{--tw-bg-opacity:1;background-color:rgb(156 163 175/var(--tw-bg-opacity,1))}.bg-gray-400\/0{background-color:rgba(156,163,175,0)}.bg-gray-400\/10{background-color:rgba(156,163,175,.1)}.bg-gray-400\/100{background-color:#9ca3af}.bg-gray-400\/15{background-color:rgba(156,163,175,.15)}.bg-gray-400\/20{background-color:rgba(156,163,175,.2)}.bg-gray-400\/25{background-color:rgba(156,163,175,.25)}.bg-gray-400\/30{background-color:rgba(156,163,175,.3)}.bg-gray-400\/35{background-color:rgba(156,163,175,.35)}.bg-gray-400\/40{background-color:rgba(156,163,175,.4)}.bg-gray-400\/45{background-color:rgba(156,163,175,.45)}.bg-gray-400\/5{background-color:rgba(156,163,175,.05)}.bg-gray-400\/50{background-color:rgba(156,163,175,.5)}.bg-gray-400\/55{background-color:rgba(156,163,175,.55)}.bg-gray-400\/60{background-color:rgba(156,163,175,.6)}.bg-gray-400\/65{background-color:rgba(156,163,175,.65)}.bg-gray-400\/70{background-color:rgba(156,163,175,.7)}.bg-gray-400\/75{background-color:rgba(156,163,175,.75)}.bg-gray-400\/80{background-color:rgba(156,163,175,.8)}.bg-gray-400\/85{background-color:rgba(156,163,175,.85)}.bg-gray-400\/90{background-color:rgba(156,163,175,.9)}.bg-gray-400\/95{background-color:rgba(156,163,175,.95)}.bg-gray-50{--tw-bg-opacity:1;background-color:rgb(249 250 251/var(--tw-bg-opacity,1))}.bg-gray-50\/0{background-color:rgba(249,250,251,0)}.bg-gray-50\/10{background-color:rgba(249,250,251,.1)}.bg-gray-50\/100{background-color:#f9fafb}.bg-gray-50\/15{background-color:rgba(249,250,251,.15)}.bg-gray-50\/20{background-color:rgba(249,250,251,.2)}.bg-gray-50\/25{background-color:rgba(249,250,251,.25)}.bg-gray-50\/30{background-color:rgba(249,250,251,.3)}.bg-gray-50\/35{background-color:rgba(249,250,251,.35)}.bg-gray-50\/40{background-color:rgba(249,250,251,.4)}.bg-gray-50\/45{background-color:rgba(249,250,251,.45)}.bg-gray-50\/5{background-color:rgba(249,250,251,.05)}.bg-gray-50\/50{background-color:rgba(249,250,251,.5)}.bg-gray-50\/55{background-color:rgba(249,250,251,.55)}.bg-gray-50\/60{background-color:rgba(249,250,251,.6)}.bg-gray-50\/65{background-color:rgba(249,250,251,.65)}.bg-gray-50\/70{background-color:rgba(249,250,251,.7)}.bg-gray-50\/75{background-color:rgba(249,250,251,.75)}.bg-gray-50\/80{background-color:rgba(249,250,251,.8)}.bg-gray-50\/85{background-color:rgba(249,250,251,.85)}.bg-gray-50\/90{background-color:rgba(249,250,251,.9)}.bg-gray-50\/95{background-color:rgba(249,250,251,.95)}.bg-gray-500{--tw-bg-opacity:1;background-color:rgb(107 114 128/var(--tw-bg-opacity,1))}.bg-gray-500\/0{background-color:hsla(220,9%,46%,0)}.bg-gray-500\/10{background-color:hsla(220,9%,46%,.1)}.bg-gray-500\/100{background-color:#6b7280}.bg-gray-500\/15{background-color:hsla(220,9%,46%,.15)}.bg-gray-500\/20{background-color:hsla(220,9%,46%,.2)}.bg-gray-500\/25{background-color:hsla(220,9%,46%,.25)}.bg-gray-500\/30{background-color:hsla(220,9%,46%,.3)}.bg-gray-500\/35{background-color:hsla(220,9%,46%,.35)}.bg-gray-500\/40{background-color:hsla(220,9%,46%,.4)}.bg-gray-500\/45{background-color:hsla(220,9%,46%,.45)}.bg-gray-500\/5{background-color:hsla(220,9%,46%,.05)}.bg-gray-500\/50{background-color:hsla(220,9%,46%,.5)}.bg-gray-500\/55{background-color:hsla(220,9%,46%,.55)}.bg-gray-500\/60{background-color:hsla(220,9%,46%,.6)}.bg-gray-500\/65{background-color:hsla(220,9%,46%,.65)}.bg-gray-500\/70{background-color:hsla(220,9%,46%,.7)}.bg-gray-500\/75{background-color:hsla(220,9%,46%,.75)}.bg-gray-500\/80{background-color:hsla(220,9%,46%,.8)}.bg-gray-500\/85{background-color:hsla(220,9%,46%,.85)}.bg-gray-500\/90{background-color:hsla(220,9%,46%,.9)}.bg-gray-500\/95{background-color:hsla(220,9%,46%,.95)}.bg-gray-950{--tw-bg-opacity:1;background-color:rgb(3 7 18/var(--tw-bg-opacity,1))}.bg-gray-950\/0{background-color:rgba(3,7,18,0)}.bg-gray-950\/10{background-color:rgba(3,7,18,.1)}.bg-gray-950\/100{background-color:#030712}.bg-gray-950\/15{background-color:rgba(3,7,18,.15)}.bg-gray-950\/20{background-color:rgba(3,7,18,.2)}.bg-gray-950\/25{background-color:rgba(3,7,18,.25)}.bg-gray-950\/30{background-color:rgba(3,7,18,.3)}.bg-gray-950\/35{background-color:rgba(3,7,18,.35)}.bg-gray-950\/40{background-color:rgba(3,7,18,.4)}.bg-gray-950\/45{background-color:rgba(3,7,18,.45)}.bg-gray-950\/5{background-color:rgba(3,7,18,.05)}.bg-gray-950\/50{background-color:rgba(3,7,18,.5)}.bg-gray-950\/55{background-color:rgba(3,7,18,.55)}.bg-gray-950\/60{background-color:rgba(3,7,18,.6)}.bg-gray-950\/65{background-color:rgba(3,7,18,.65)}.bg-gray-950\/70{background-color:rgba(3,7,18,.7)}.bg-gray-950\/75{background-color:rgba(3,7,18,.75)}.bg-gray-950\/80{background-color:rgba(3,7,18,.8)}.bg-gray-950\/85{background-color:rgba(3,7,18,.85)}.bg-gray-950\/90{background-color:rgba(3,7,18,.9)}.bg-gray-950\/95{background-color:rgba(3,7,18,.95)}.bg-green-100{--tw-bg-opacity:1;background-color:rgb(220 252 231/var(--tw-bg-opacity,1))}.bg-green-100\/0{background-color:rgba(220,252,231,0)}.bg-green-100\/10{background-color:rgba(220,252,231,.1)}.bg-green-100\/100{background-color:#dcfce7}.bg-green-100\/15{background-color:rgba(220,252,231,.15)}.bg-green-100\/20{background-color:rgba(220,252,231,.2)}.bg-green-100\/25{background-color:rgba(220,252,231,.25)}.bg-green-100\/30{background-color:rgba(220,252,231,.3)}.bg-green-100\/35{background-color:rgba(220,252,231,.35)}.bg-green-100\/40{background-color:rgba(220,252,231,.4)}.bg-green-100\/45{background-color:rgba(220,252,231,.45)}.bg-green-100\/5{background-color:rgba(220,252,231,.05)}.bg-green-100\/50{background-color:rgba(220,252,231,.5)}.bg-green-100\/55{background-color:rgba(220,252,231,.55)}.bg-green-100\/60{background-color:rgba(220,252,231,.6)}.bg-green-100\/65{background-color:rgba(220,252,231,.65)}.bg-green-100\/70{background-color:rgba(220,252,231,.7)}.bg-green-100\/75{background-color:rgba(220,252,231,.75)}.bg-green-100\/80{background-color:rgba(220,252,231,.8)}.bg-green-100\/85{background-color:rgba(220,252,231,.85)}.bg-green-100\/90{background-color:rgba(220,252,231,.9)}.bg-green-100\/95{background-color:rgba(220,252,231,.95)}.bg-green-200{--tw-bg-opacity:1;background-color:rgb(187 247 208/var(--tw-bg-opacity,1))}.bg-green-200\/0{background-color:rgba(187,247,208,0)}.bg-green-200\/10{background-color:rgba(187,247,208,.1)}.bg-green-200\/100{background-color:#bbf7d0}.bg-green-200\/15{background-color:rgba(187,247,208,.15)}.bg-green-200\/20{background-color:rgba(187,247,208,.2)}.bg-green-200\/25{background-color:rgba(187,247,208,.25)}.bg-green-200\/30{background-color:rgba(187,247,208,.3)}.bg-green-200\/35{background-color:rgba(187,247,208,.35)}.bg-green-200\/40{background-color:rgba(187,247,208,.4)}.bg-green-200\/45{background-color:rgba(187,247,208,.45)}.bg-green-200\/5{background-color:rgba(187,247,208,.05)}.bg-green-200\/50{background-color:rgba(187,247,208,.5)}.bg-green-200\/55{background-color:rgba(187,247,208,.55)}.bg-green-200\/60{background-color:rgba(187,247,208,.6)}.bg-green-200\/65{background-color:rgba(187,247,208,.65)}.bg-green-200\/70{background-color:rgba(187,247,208,.7)}.bg-green-200\/75{background-color:rgba(187,247,208,.75)}.bg-green-200\/80{background-color:rgba(187,247,208,.8)}.bg-green-200\/85{background-color:rgba(187,247,208,.85)}.bg-green-200\/90{background-color:rgba(187,247,208,.9)}.bg-green-200\/95{background-color:rgba(187,247,208,.95)}.bg-green-400{--tw-bg-opacity:1;background-color:rgb(74 222 128/var(--tw-bg-opacity,1))}.bg-green-400\/0{background-color:rgba(74,222,128,0)}.bg-green-400\/10{background-color:rgba(74,222,128,.1)}.bg-green-400\/100{background-color:#4ade80}.bg-green-400\/15{background-color:rgba(74,222,128,.15)}.bg-green-400\/20{background-color:rgba(74,222,128,.2)}.bg-green-400\/25{background-color:rgba(74,222,128,.25)}.bg-green-400\/30{background-color:rgba(74,222,128,.3)}.bg-green-400\/35{background-color:rgba(74,222,128,.35)}.bg-green-400\/40{background-color:rgba(74,222,128,.4)}.bg-green-400\/45{background-color:rgba(74,222,128,.45)}.bg-green-400\/5{background-color:rgba(74,222,128,.05)}.bg-green-400\/50{background-color:rgba(74,222,128,.5)}.bg-green-400\/55{background-color:rgba(74,222,128,.55)}.bg-green-400\/60{background-color:rgba(74,222,128,.6)}.bg-green-400\/65{background-color:rgba(74,222,128,.65)}.bg-green-400\/70{background-color:rgba(74,222,128,.7)}.bg-green-400\/75{background-color:rgba(74,222,128,.75)}.bg-green-400\/80{background-color:rgba(74,222,128,.8)}.bg-green-400\/85{background-color:rgba(74,222,128,.85)}.bg-green-400\/90{background-color:rgba(74,222,128,.9)}.bg-green-400\/95{background-color:rgba(74,222,128,.95)}.bg-green-50{--tw-bg-opacity:1;background-color:rgb(240 253 244/var(--tw-bg-opacity,1))}.bg-green-50\/0{background-color:rgba(240,253,244,0)}.bg-green-50\/10{background-color:rgba(240,253,244,.1)}.bg-green-50\/100{background-color:#f0fdf4}.bg-green-50\/15{background-color:rgba(240,253,244,.15)}.bg-green-50\/20{background-color:rgba(240,253,244,.2)}.bg-green-50\/25{background-color:rgba(240,253,244,.25)}.bg-green-50\/30{background-color:rgba(240,253,244,.3)}.bg-green-50\/35{background-color:rgba(240,253,244,.35)}.bg-green-50\/40{background-color:rgba(240,253,244,.4)}.bg-green-50\/45{background-color:rgba(240,253,244,.45)}.bg-green-50\/5{background-color:rgba(240,253,244,.05)}.bg-green-50\/50{background-color:rgba(240,253,244,.5)}.bg-green-50\/55{background-color:rgba(240,253,244,.55)}.bg-green-50\/60{background-color:rgba(240,253,244,.6)}.bg-green-50\/65{background-color:rgba(240,253,244,.65)}.bg-green-50\/70{background-color:rgba(240,253,244,.7)}.bg-green-50\/75{background-color:rgba(240,253,244,.75)}.bg-green-50\/80{background-color:rgba(240,253,244,.8)}.bg-green-50\/85{background-color:rgba(240,253,244,.85)}.bg-green-50\/90{background-color:rgba(240,253,244,.9)}.bg-green-50\/95{background-color:rgba(240,253,244,.95)}.bg-green-500{--tw-bg-opacity:1;background-color:rgb(34 197 94/var(--tw-bg-opacity,1))}.bg-green-500\/0{background-color:rgba(34,197,94,0)}.bg-green-500\/10{background-color:rgba(34,197,94,.1)}.bg-green-500\/100{background-color:#22c55e}.bg-green-500\/15{background-color:rgba(34,197,94,.15)}.bg-green-500\/20{background-color:rgba(34,197,94,.2)}.bg-green-500\/25{background-color:rgba(34,197,94,.25)}.bg-green-500\/30{background-color:rgba(34,197,94,.3)}.bg-green-500\/35{background-color:rgba(34,197,94,.35)}.bg-green-500\/40{background-color:rgba(34,197,94,.4)}.bg-green-500\/45{background-color:rgba(34,197,94,.45)}.bg-green-500\/5{background-color:rgba(34,197,94,.05)}.bg-green-500\/50{background-color:rgba(34,197,94,.5)}.bg-green-500\/55{background-color:rgba(34,197,94,.55)}.bg-green-500\/60{background-color:rgba(34,197,94,.6)}.bg-green-500\/65{background-color:rgba(34,197,94,.65)}.bg-green-500\/70{background-color:rgba(34,197,94,.7)}.bg-green-500\/75{background-color:rgba(34,197,94,.75)}.bg-green-500\/80{background-color:rgba(34,197,94,.8)}.bg-green-500\/85{background-color:rgba(34,197,94,.85)}.bg-green-500\/90{background-color:rgba(34,197,94,.9)}.bg-green-500\/95{background-color:rgba(34,197,94,.95)}.bg-green-950{--tw-bg-opacity:1;background-color:rgb(5 46 22/var(--tw-bg-opacity,1))}.bg-green-950\/0{background-color:rgba(5,46,22,0)}.bg-green-950\/10{background-color:rgba(5,46,22,.1)}.bg-green-950\/100{background-color:#052e16}.bg-green-950\/15{background-color:rgba(5,46,22,.15)}.bg-green-950\/20{background-color:rgba(5,46,22,.2)}.bg-green-950\/25{background-color:rgba(5,46,22,.25)}.bg-green-950\/30{background-color:rgba(5,46,22,.3)}.bg-green-950\/35{background-color:rgba(5,46,22,.35)}.bg-green-950\/40{background-color:rgba(5,46,22,.4)}.bg-green-950\/45{background-color:rgba(5,46,22,.45)}.bg-green-950\/5{background-color:rgba(5,46,22,.05)}.bg-green-950\/50{background-color:rgba(5,46,22,.5)}.bg-green-950\/55{background-color:rgba(5,46,22,.55)}.bg-green-950\/60{background-color:rgba(5,46,22,.6)}.bg-green-950\/65{background-color:rgba(5,46,22,.65)}.bg-green-950\/70{background-color:rgba(5,46,22,.7)}.bg-green-950\/75{background-color:rgba(5,46,22,.75)}.bg-green-950\/80{background-color:rgba(5,46,22,.8)}.bg-green-950\/85{background-color:rgba(5,46,22,.85)}.bg-green-950\/90{background-color:rgba(5,46,22,.9)}.bg-green-950\/95{background-color:rgba(5,46,22,.95)}.bg-indigo-100{--tw-bg-opacity:1;background-color:rgb(224 231 255/var(--tw-bg-opacity,1))}.bg-indigo-100\/0{background-color:rgba(224,231,255,0)}.bg-indigo-100\/10{background-color:rgba(224,231,255,.1)}.bg-indigo-100\/100{background-color:#e0e7ff}.bg-indigo-100\/15{background-color:rgba(224,231,255,.15)}.bg-indigo-100\/20{background-color:rgba(224,231,255,.2)}.bg-indigo-100\/25{background-color:rgba(224,231,255,.25)}.bg-indigo-100\/30{background-color:rgba(224,231,255,.3)}.bg-indigo-100\/35{background-color:rgba(224,231,255,.35)}.bg-indigo-100\/40{background-color:rgba(224,231,255,.4)}.bg-indigo-100\/45{background-color:rgba(224,231,255,.45)}.bg-indigo-100\/5{background-color:rgba(224,231,255,.05)}.bg-indigo-100\/50{background-color:rgba(224,231,255,.5)}.bg-indigo-100\/55{background-color:rgba(224,231,255,.55)}.bg-indigo-100\/60{background-color:rgba(224,231,255,.6)}.bg-indigo-100\/65{background-color:rgba(224,231,255,.65)}.bg-indigo-100\/70{background-color:rgba(224,231,255,.7)}.bg-indigo-100\/75{background-color:rgba(224,231,255,.75)}.bg-indigo-100\/80{background-color:rgba(224,231,255,.8)}.bg-indigo-100\/85{background-color:rgba(224,231,255,.85)}.bg-indigo-100\/90{background-color:rgba(224,231,255,.9)}.bg-indigo-100\/95{background-color:rgba(224,231,255,.95)}.bg-indigo-200{--tw-bg-opacity:1;background-color:rgb(199 210 254/var(--tw-bg-opacity,1))}.bg-indigo-200\/0{background-color:rgba(199,210,254,0)}.bg-indigo-200\/10{background-color:rgba(199,210,254,.1)}.bg-indigo-200\/100{background-color:#c7d2fe}.bg-indigo-200\/15{background-color:rgba(199,210,254,.15)}.bg-indigo-200\/20{background-color:rgba(199,210,254,.2)}.bg-indigo-200\/25{background-color:rgba(199,210,254,.25)}.bg-indigo-200\/30{background-color:rgba(199,210,254,.3)}.bg-indigo-200\/35{background-color:rgba(199,210,254,.35)}.bg-indigo-200\/40{background-color:rgba(199,210,254,.4)}.bg-indigo-200\/45{background-color:rgba(199,210,254,.45)}.bg-indigo-200\/5{background-color:rgba(199,210,254,.05)}.bg-indigo-200\/50{background-color:rgba(199,210,254,.5)}.bg-indigo-200\/55{background-color:rgba(199,210,254,.55)}.bg-indigo-200\/60{background-color:rgba(199,210,254,.6)}.bg-indigo-200\/65{background-color:rgba(199,210,254,.65)}.bg-indigo-200\/70{background-color:rgba(199,210,254,.7)}.bg-indigo-200\/75{background-color:rgba(199,210,254,.75)}.bg-indigo-200\/80{background-color:rgba(199,210,254,.8)}.bg-indigo-200\/85{background-color:rgba(199,210,254,.85)}.bg-indigo-200\/90{background-color:rgba(199,210,254,.9)}.bg-indigo-200\/95{background-color:rgba(199,210,254,.95)}.bg-indigo-400{--tw-bg-opacity:1;background-color:rgb(129 140 248/var(--tw-bg-opacity,1))}.bg-indigo-400\/0{background-color:rgba(129,140,248,0)}.bg-indigo-400\/10{background-color:rgba(129,140,248,.1)}.bg-indigo-400\/100{background-color:#818cf8}.bg-indigo-400\/15{background-color:rgba(129,140,248,.15)}.bg-indigo-400\/20{background-color:rgba(129,140,248,.2)}.bg-indigo-400\/25{background-color:rgba(129,140,248,.25)}.bg-indigo-400\/30{background-color:rgba(129,140,248,.3)}.bg-indigo-400\/35{background-color:rgba(129,140,248,.35)}.bg-indigo-400\/40{background-color:rgba(129,140,248,.4)}.bg-indigo-400\/45{background-color:rgba(129,140,248,.45)}.bg-indigo-400\/5{background-color:rgba(129,140,248,.05)}.bg-indigo-400\/50{background-color:rgba(129,140,248,.5)}.bg-indigo-400\/55{background-color:rgba(129,140,248,.55)}.bg-indigo-400\/60{background-color:rgba(129,140,248,.6)}.bg-indigo-400\/65{background-color:rgba(129,140,248,.65)}.bg-indigo-400\/70{background-color:rgba(129,140,248,.7)}.bg-indigo-400\/75{background-color:rgba(129,140,248,.75)}.bg-indigo-400\/80{background-color:rgba(129,140,248,.8)}.bg-indigo-400\/85{background-color:rgba(129,140,248,.85)}.bg-indigo-400\/90{background-color:rgba(129,140,248,.9)}.bg-indigo-400\/95{background-color:rgba(129,140,248,.95)}.bg-indigo-50{--tw-bg-opacity:1;background-color:rgb(238 242 255/var(--tw-bg-opacity,1))}.bg-indigo-50\/0{background-color:rgba(238,242,255,0)}.bg-indigo-50\/10{background-color:rgba(238,242,255,.1)}.bg-indigo-50\/100{background-color:#eef2ff}.bg-indigo-50\/15{background-color:rgba(238,242,255,.15)}.bg-indigo-50\/20{background-color:rgba(238,242,255,.2)}.bg-indigo-50\/25{background-color:rgba(238,242,255,.25)}.bg-indigo-50\/30{background-color:rgba(238,242,255,.3)}.bg-indigo-50\/35{background-color:rgba(238,242,255,.35)}.bg-indigo-50\/40{background-color:rgba(238,242,255,.4)}.bg-indigo-50\/45{background-color:rgba(238,242,255,.45)}.bg-indigo-50\/5{background-color:rgba(238,242,255,.05)}.bg-indigo-50\/50{background-color:rgba(238,242,255,.5)}.bg-indigo-50\/55{background-color:rgba(238,242,255,.55)}.bg-indigo-50\/60{background-color:rgba(238,242,255,.6)}.bg-indigo-50\/65{background-color:rgba(238,242,255,.65)}.bg-indigo-50\/70{background-color:rgba(238,242,255,.7)}.bg-indigo-50\/75{background-color:rgba(238,242,255,.75)}.bg-indigo-50\/80{background-color:rgba(238,242,255,.8)}.bg-indigo-50\/85{background-color:rgba(238,242,255,.85)}.bg-indigo-50\/90{background-color:rgba(238,242,255,.9)}.bg-indigo-50\/95{background-color:rgba(238,242,255,.95)}.bg-indigo-500{--tw-bg-opacity:1;background-color:rgb(99 102 241/var(--tw-bg-opacity,1))}.bg-indigo-500\/0{background-color:rgba(99,102,241,0)}.bg-indigo-500\/10{background-color:rgba(99,102,241,.1)}.bg-indigo-500\/100{background-color:#6366f1}.bg-indigo-500\/15{background-color:rgba(99,102,241,.15)}.bg-indigo-500\/20{background-color:rgba(99,102,241,.2)}.bg-indigo-500\/25{background-color:rgba(99,102,241,.25)}.bg-indigo-500\/30{background-color:rgba(99,102,241,.3)}.bg-indigo-500\/35{background-color:rgba(99,102,241,.35)}.bg-indigo-500\/40{background-color:rgba(99,102,241,.4)}.bg-indigo-500\/45{background-color:rgba(99,102,241,.45)}.bg-indigo-500\/5{background-color:rgba(99,102,241,.05)}.bg-indigo-500\/50{background-color:rgba(99,102,241,.5)}.bg-indigo-500\/55{background-color:rgba(99,102,241,.55)}.bg-indigo-500\/60{background-color:rgba(99,102,241,.6)}.bg-indigo-500\/65{background-color:rgba(99,102,241,.65)}.bg-indigo-500\/70{background-color:rgba(99,102,241,.7)}.bg-indigo-500\/75{background-color:rgba(99,102,241,.75)}.bg-indigo-500\/80{background-color:rgba(99,102,241,.8)}.bg-indigo-500\/85{background-color:rgba(99,102,241,.85)}.bg-indigo-500\/90{background-color:rgba(99,102,241,.9)}.bg-indigo-500\/95{background-color:rgba(99,102,241,.95)}.bg-indigo-950{--tw-bg-opacity:1;background-color:rgb(30 27 75/var(--tw-bg-opacity,1))}.bg-indigo-950\/0{background-color:rgba(30,27,75,0)}.bg-indigo-950\/10{background-color:rgba(30,27,75,.1)}.bg-indigo-950\/100{background-color:#1e1b4b}.bg-indigo-950\/15{background-color:rgba(30,27,75,.15)}.bg-indigo-950\/20{background-color:rgba(30,27,75,.2)}.bg-indigo-950\/25{background-color:rgba(30,27,75,.25)}.bg-indigo-950\/30{background-color:rgba(30,27,75,.3)}.bg-indigo-950\/35{background-color:rgba(30,27,75,.35)}.bg-indigo-950\/40{background-color:rgba(30,27,75,.4)}.bg-indigo-950\/45{background-color:rgba(30,27,75,.45)}.bg-indigo-950\/5{background-color:rgba(30,27,75,.05)}.bg-indigo-950\/50{background-color:rgba(30,27,75,.5)}.bg-indigo-950\/55{background-color:rgba(30,27,75,.55)}.bg-indigo-950\/60{background-color:rgba(30,27,75,.6)}.bg-indigo-950\/65{background-color:rgba(30,27,75,.65)}.bg-indigo-950\/70{background-color:rgba(30,27,75,.7)}.bg-indigo-950\/75{background-color:rgba(30,27,75,.75)}.bg-indigo-950\/80{background-color:rgba(30,27,75,.8)}.bg-indigo-950\/85{background-color:rgba(30,27,75,.85)}.bg-indigo-950\/90{background-color:rgba(30,27,75,.9)}.bg-indigo-950\/95{background-color:rgba(30,27,75,.95)}.bg-inherit{background-color:inherit}.bg-lime-100{--tw-bg-opacity:1;background-color:rgb(236 252 203/var(--tw-bg-opacity,1))}.bg-lime-100\/0{background-color:hsla(80,89%,89%,0)}.bg-lime-100\/10{background-color:hsla(80,89%,89%,.1)}.bg-lime-100\/100{background-color:#ecfccb}.bg-lime-100\/15{background-color:hsla(80,89%,89%,.15)}.bg-lime-100\/20{background-color:hsla(80,89%,89%,.2)}.bg-lime-100\/25{background-color:hsla(80,89%,89%,.25)}.bg-lime-100\/30{background-color:hsla(80,89%,89%,.3)}.bg-lime-100\/35{background-color:hsla(80,89%,89%,.35)}.bg-lime-100\/40{background-color:hsla(80,89%,89%,.4)}.bg-lime-100\/45{background-color:hsla(80,89%,89%,.45)}.bg-lime-100\/5{background-color:hsla(80,89%,89%,.05)}.bg-lime-100\/50{background-color:hsla(80,89%,89%,.5)}.bg-lime-100\/55{background-color:hsla(80,89%,89%,.55)}.bg-lime-100\/60{background-color:hsla(80,89%,89%,.6)}.bg-lime-100\/65{background-color:hsla(80,89%,89%,.65)}.bg-lime-100\/70{background-color:hsla(80,89%,89%,.7)}.bg-lime-100\/75{background-color:hsla(80,89%,89%,.75)}.bg-lime-100\/80{background-color:hsla(80,89%,89%,.8)}.bg-lime-100\/85{background-color:hsla(80,89%,89%,.85)}.bg-lime-100\/90{background-color:hsla(80,89%,89%,.9)}.bg-lime-100\/95{background-color:hsla(80,89%,89%,.95)}.bg-lime-200{--tw-bg-opacity:1;background-color:rgb(217 249 157/var(--tw-bg-opacity,1))}.bg-lime-200\/0{background-color:hsla(81,88%,80%,0)}.bg-lime-200\/10{background-color:hsla(81,88%,80%,.1)}.bg-lime-200\/100{background-color:#d9f99d}.bg-lime-200\/15{background-color:hsla(81,88%,80%,.15)}.bg-lime-200\/20{background-color:hsla(81,88%,80%,.2)}.bg-lime-200\/25{background-color:hsla(81,88%,80%,.25)}.bg-lime-200\/30{background-color:hsla(81,88%,80%,.3)}.bg-lime-200\/35{background-color:hsla(81,88%,80%,.35)}.bg-lime-200\/40{background-color:hsla(81,88%,80%,.4)}.bg-lime-200\/45{background-color:hsla(81,88%,80%,.45)}.bg-lime-200\/5{background-color:hsla(81,88%,80%,.05)}.bg-lime-200\/50{background-color:hsla(81,88%,80%,.5)}.bg-lime-200\/55{background-color:hsla(81,88%,80%,.55)}.bg-lime-200\/60{background-color:hsla(81,88%,80%,.6)}.bg-lime-200\/65{background-color:hsla(81,88%,80%,.65)}.bg-lime-200\/70{background-color:hsla(81,88%,80%,.7)}.bg-lime-200\/75{background-color:hsla(81,88%,80%,.75)}.bg-lime-200\/80{background-color:hsla(81,88%,80%,.8)}.bg-lime-200\/85{background-color:hsla(81,88%,80%,.85)}.bg-lime-200\/90{background-color:hsla(81,88%,80%,.9)}.bg-lime-200\/95{background-color:hsla(81,88%,80%,.95)}.bg-lime-400{--tw-bg-opacity:1;background-color:rgb(163 230 53/var(--tw-bg-opacity,1))}.bg-lime-400\/0{background-color:rgba(163,230,53,0)}.bg-lime-400\/10{background-color:rgba(163,230,53,.1)}.bg-lime-400\/100{background-color:#a3e635}.bg-lime-400\/15{background-color:rgba(163,230,53,.15)}.bg-lime-400\/20{background-color:rgba(163,230,53,.2)}.bg-lime-400\/25{background-color:rgba(163,230,53,.25)}.bg-lime-400\/30{background-color:rgba(163,230,53,.3)}.bg-lime-400\/35{background-color:rgba(163,230,53,.35)}.bg-lime-400\/40{background-color:rgba(163,230,53,.4)}.bg-lime-400\/45{background-color:rgba(163,230,53,.45)}.bg-lime-400\/5{background-color:rgba(163,230,53,.05)}.bg-lime-400\/50{background-color:rgba(163,230,53,.5)}.bg-lime-400\/55{background-color:rgba(163,230,53,.55)}.bg-lime-400\/60{background-color:rgba(163,230,53,.6)}.bg-lime-400\/65{background-color:rgba(163,230,53,.65)}.bg-lime-400\/70{background-color:rgba(163,230,53,.7)}.bg-lime-400\/75{background-color:rgba(163,230,53,.75)}.bg-lime-400\/80{background-color:rgba(163,230,53,.8)}.bg-lime-400\/85{background-color:rgba(163,230,53,.85)}.bg-lime-400\/90{background-color:rgba(163,230,53,.9)}.bg-lime-400\/95{background-color:rgba(163,230,53,.95)}.bg-lime-50{--tw-bg-opacity:1;background-color:rgb(247 254 231/var(--tw-bg-opacity,1))}.bg-lime-50\/0{background-color:hsla(78,92%,95%,0)}.bg-lime-50\/10{background-color:hsla(78,92%,95%,.1)}.bg-lime-50\/100{background-color:#f7fee7}.bg-lime-50\/15{background-color:hsla(78,92%,95%,.15)}.bg-lime-50\/20{background-color:hsla(78,92%,95%,.2)}.bg-lime-50\/25{background-color:hsla(78,92%,95%,.25)}.bg-lime-50\/30{background-color:hsla(78,92%,95%,.3)}.bg-lime-50\/35{background-color:hsla(78,92%,95%,.35)}.bg-lime-50\/40{background-color:hsla(78,92%,95%,.4)}.bg-lime-50\/45{background-color:hsla(78,92%,95%,.45)}.bg-lime-50\/5{background-color:hsla(78,92%,95%,.05)}.bg-lime-50\/50{background-color:hsla(78,92%,95%,.5)}.bg-lime-50\/55{background-color:hsla(78,92%,95%,.55)}.bg-lime-50\/60{background-color:hsla(78,92%,95%,.6)}.bg-lime-50\/65{background-color:hsla(78,92%,95%,.65)}.bg-lime-50\/70{background-color:hsla(78,92%,95%,.7)}.bg-lime-50\/75{background-color:hsla(78,92%,95%,.75)}.bg-lime-50\/80{background-color:hsla(78,92%,95%,.8)}.bg-lime-50\/85{background-color:hsla(78,92%,95%,.85)}.bg-lime-50\/90{background-color:hsla(78,92%,95%,.9)}.bg-lime-50\/95{background-color:hsla(78,92%,95%,.95)}.bg-lime-500{--tw-bg-opacity:1;background-color:rgb(132 204 22/var(--tw-bg-opacity,1))}.bg-lime-500\/0{background-color:rgba(132,204,22,0)}.bg-lime-500\/10{background-color:rgba(132,204,22,.1)}.bg-lime-500\/100{background-color:#84cc16}.bg-lime-500\/15{background-color:rgba(132,204,22,.15)}.bg-lime-500\/20{background-color:rgba(132,204,22,.2)}.bg-lime-500\/25{background-color:rgba(132,204,22,.25)}.bg-lime-500\/30{background-color:rgba(132,204,22,.3)}.bg-lime-500\/35{background-color:rgba(132,204,22,.35)}.bg-lime-500\/40{background-color:rgba(132,204,22,.4)}.bg-lime-500\/45{background-color:rgba(132,204,22,.45)}.bg-lime-500\/5{background-color:rgba(132,204,22,.05)}.bg-lime-500\/50{background-color:rgba(132,204,22,.5)}.bg-lime-500\/55{background-color:rgba(132,204,22,.55)}.bg-lime-500\/60{background-color:rgba(132,204,22,.6)}.bg-lime-500\/65{background-color:rgba(132,204,22,.65)}.bg-lime-500\/70{background-color:rgba(132,204,22,.7)}.bg-lime-500\/75{background-color:rgba(132,204,22,.75)}.bg-lime-500\/80{background-color:rgba(132,204,22,.8)}.bg-lime-500\/85{background-color:rgba(132,204,22,.85)}.bg-lime-500\/90{background-color:rgba(132,204,22,.9)}.bg-lime-500\/95{background-color:rgba(132,204,22,.95)}.bg-lime-950{--tw-bg-opacity:1;background-color:rgb(26 46 5/var(--tw-bg-opacity,1))}.bg-lime-950\/0{background-color:rgba(26,46,5,0)}.bg-lime-950\/10{background-color:rgba(26,46,5,.1)}.bg-lime-950\/100{background-color:#1a2e05}.bg-lime-950\/15{background-color:rgba(26,46,5,.15)}.bg-lime-950\/20{background-color:rgba(26,46,5,.2)}.bg-lime-950\/25{background-color:rgba(26,46,5,.25)}.bg-lime-950\/30{background-color:rgba(26,46,5,.3)}.bg-lime-950\/35{background-color:rgba(26,46,5,.35)}.bg-lime-950\/40{background-color:rgba(26,46,5,.4)}.bg-lime-950\/45{background-color:rgba(26,46,5,.45)}.bg-lime-950\/5{background-color:rgba(26,46,5,.05)}.bg-lime-950\/50{background-color:rgba(26,46,5,.5)}.bg-lime-950\/55{background-color:rgba(26,46,5,.55)}.bg-lime-950\/60{background-color:rgba(26,46,5,.6)}.bg-lime-950\/65{background-color:rgba(26,46,5,.65)}.bg-lime-950\/70{background-color:rgba(26,46,5,.7)}.bg-lime-950\/75{background-color:rgba(26,46,5,.75)}.bg-lime-950\/80{background-color:rgba(26,46,5,.8)}.bg-lime-950\/85{background-color:rgba(26,46,5,.85)}.bg-lime-950\/90{background-color:rgba(26,46,5,.9)}.bg-lime-950\/95{background-color:rgba(26,46,5,.95)}.bg-muted{background-color:hsl(var(--muted))}.bg-muted\/50{background-color:hsl(var(--muted)/.5)}.bg-neutral-100{--tw-bg-opacity:1;background-color:rgb(245 245 245/var(--tw-bg-opacity,1))}.bg-neutral-100\/0{background-color:hsla(0,0%,96%,0)}.bg-neutral-100\/10{background-color:hsla(0,0%,96%,.1)}.bg-neutral-100\/100{background-color:#f5f5f5}.bg-neutral-100\/15{background-color:hsla(0,0%,96%,.15)}.bg-neutral-100\/20{background-color:hsla(0,0%,96%,.2)}.bg-neutral-100\/25{background-color:hsla(0,0%,96%,.25)}.bg-neutral-100\/30{background-color:hsla(0,0%,96%,.3)}.bg-neutral-100\/35{background-color:hsla(0,0%,96%,.35)}.bg-neutral-100\/40{background-color:hsla(0,0%,96%,.4)}.bg-neutral-100\/45{background-color:hsla(0,0%,96%,.45)}.bg-neutral-100\/5{background-color:hsla(0,0%,96%,.05)}.bg-neutral-100\/50{background-color:hsla(0,0%,96%,.5)}.bg-neutral-100\/55{background-color:hsla(0,0%,96%,.55)}.bg-neutral-100\/60{background-color:hsla(0,0%,96%,.6)}.bg-neutral-100\/65{background-color:hsla(0,0%,96%,.65)}.bg-neutral-100\/70{background-color:hsla(0,0%,96%,.7)}.bg-neutral-100\/75{background-color:hsla(0,0%,96%,.75)}.bg-neutral-100\/80{background-color:hsla(0,0%,96%,.8)}.bg-neutral-100\/85{background-color:hsla(0,0%,96%,.85)}.bg-neutral-100\/90{background-color:hsla(0,0%,96%,.9)}.bg-neutral-100\/95{background-color:hsla(0,0%,96%,.95)}.bg-neutral-200{--tw-bg-opacity:1;background-color:rgb(229 229 229/var(--tw-bg-opacity,1))}.bg-neutral-200\/0{background-color:hsla(0,0%,90%,0)}.bg-neutral-200\/10{background-color:hsla(0,0%,90%,.1)}.bg-neutral-200\/100{background-color:#e5e5e5}.bg-neutral-200\/15{background-color:hsla(0,0%,90%,.15)}.bg-neutral-200\/20{background-color:hsla(0,0%,90%,.2)}.bg-neutral-200\/25{background-color:hsla(0,0%,90%,.25)}.bg-neutral-200\/30{background-color:hsla(0,0%,90%,.3)}.bg-neutral-200\/35{background-color:hsla(0,0%,90%,.35)}.bg-neutral-200\/40{background-color:hsla(0,0%,90%,.4)}.bg-neutral-200\/45{background-color:hsla(0,0%,90%,.45)}.bg-neutral-200\/5{background-color:hsla(0,0%,90%,.05)}.bg-neutral-200\/50{background-color:hsla(0,0%,90%,.5)}.bg-neutral-200\/55{background-color:hsla(0,0%,90%,.55)}.bg-neutral-200\/60{background-color:hsla(0,0%,90%,.6)}.bg-neutral-200\/65{background-color:hsla(0,0%,90%,.65)}.bg-neutral-200\/70{background-color:hsla(0,0%,90%,.7)}.bg-neutral-200\/75{background-color:hsla(0,0%,90%,.75)}.bg-neutral-200\/80{background-color:hsla(0,0%,90%,.8)}.bg-neutral-200\/85{background-color:hsla(0,0%,90%,.85)}.bg-neutral-200\/90{background-color:hsla(0,0%,90%,.9)}.bg-neutral-200\/95{background-color:hsla(0,0%,90%,.95)}.bg-neutral-400{--tw-bg-opacity:1;background-color:rgb(163 163 163/var(--tw-bg-opacity,1))}.bg-neutral-400\/0{background-color:hsla(0,0%,64%,0)}.bg-neutral-400\/10{background-color:hsla(0,0%,64%,.1)}.bg-neutral-400\/100{background-color:#a3a3a3}.bg-neutral-400\/15{background-color:hsla(0,0%,64%,.15)}.bg-neutral-400\/20{background-color:hsla(0,0%,64%,.2)}.bg-neutral-400\/25{background-color:hsla(0,0%,64%,.25)}.bg-neutral-400\/30{background-color:hsla(0,0%,64%,.3)}.bg-neutral-400\/35{background-color:hsla(0,0%,64%,.35)}.bg-neutral-400\/40{background-color:hsla(0,0%,64%,.4)}.bg-neutral-400\/45{background-color:hsla(0,0%,64%,.45)}.bg-neutral-400\/5{background-color:hsla(0,0%,64%,.05)}.bg-neutral-400\/50{background-color:hsla(0,0%,64%,.5)}.bg-neutral-400\/55{background-color:hsla(0,0%,64%,.55)}.bg-neutral-400\/60{background-color:hsla(0,0%,64%,.6)}.bg-neutral-400\/65{background-color:hsla(0,0%,64%,.65)}.bg-neutral-400\/70{background-color:hsla(0,0%,64%,.7)}.bg-neutral-400\/75{background-color:hsla(0,0%,64%,.75)}.bg-neutral-400\/80{background-color:hsla(0,0%,64%,.8)}.bg-neutral-400\/85{background-color:hsla(0,0%,64%,.85)}.bg-neutral-400\/90{background-color:hsla(0,0%,64%,.9)}.bg-neutral-400\/95{background-color:hsla(0,0%,64%,.95)}.bg-neutral-50{--tw-bg-opacity:1;background-color:rgb(250 250 250/var(--tw-bg-opacity,1))}.bg-neutral-50\/0{background-color:hsla(0,0%,98%,0)}.bg-neutral-50\/10{background-color:hsla(0,0%,98%,.1)}.bg-neutral-50\/100{background-color:#fafafa}.bg-neutral-50\/15{background-color:hsla(0,0%,98%,.15)}.bg-neutral-50\/20{background-color:hsla(0,0%,98%,.2)}.bg-neutral-50\/25{background-color:hsla(0,0%,98%,.25)}.bg-neutral-50\/30{background-color:hsla(0,0%,98%,.3)}.bg-neutral-50\/35{background-color:hsla(0,0%,98%,.35)}.bg-neutral-50\/40{background-color:hsla(0,0%,98%,.4)}.bg-neutral-50\/45{background-color:hsla(0,0%,98%,.45)}.bg-neutral-50\/5{background-color:hsla(0,0%,98%,.05)}.bg-neutral-50\/50{background-color:hsla(0,0%,98%,.5)}.bg-neutral-50\/55{background-color:hsla(0,0%,98%,.55)}.bg-neutral-50\/60{background-color:hsla(0,0%,98%,.6)}.bg-neutral-50\/65{background-color:hsla(0,0%,98%,.65)}.bg-neutral-50\/70{background-color:hsla(0,0%,98%,.7)}.bg-neutral-50\/75{background-color:hsla(0,0%,98%,.75)}.bg-neutral-50\/80{background-color:hsla(0,0%,98%,.8)}.bg-neutral-50\/85{background-color:hsla(0,0%,98%,.85)}.bg-neutral-50\/90{background-color:hsla(0,0%,98%,.9)}.bg-neutral-50\/95{background-color:hsla(0,0%,98%,.95)}.bg-neutral-500{--tw-bg-opacity:1;background-color:rgb(115 115 115/var(--tw-bg-opacity,1))}.bg-neutral-500\/0{background-color:hsla(0,0%,45%,0)}.bg-neutral-500\/10{background-color:hsla(0,0%,45%,.1)}.bg-neutral-500\/100{background-color:#737373}.bg-neutral-500\/15{background-color:hsla(0,0%,45%,.15)}.bg-neutral-500\/20{background-color:hsla(0,0%,45%,.2)}.bg-neutral-500\/25{background-color:hsla(0,0%,45%,.25)}.bg-neutral-500\/30{background-color:hsla(0,0%,45%,.3)}.bg-neutral-500\/35{background-color:hsla(0,0%,45%,.35)}.bg-neutral-500\/40{background-color:hsla(0,0%,45%,.4)}.bg-neutral-500\/45{background-color:hsla(0,0%,45%,.45)}.bg-neutral-500\/5{background-color:hsla(0,0%,45%,.05)}.bg-neutral-500\/50{background-color:hsla(0,0%,45%,.5)}.bg-neutral-500\/55{background-color:hsla(0,0%,45%,.55)}.bg-neutral-500\/60{background-color:hsla(0,0%,45%,.6)}.bg-neutral-500\/65{background-color:hsla(0,0%,45%,.65)}.bg-neutral-500\/70{background-color:hsla(0,0%,45%,.7)}.bg-neutral-500\/75{background-color:hsla(0,0%,45%,.75)}.bg-neutral-500\/80{background-color:hsla(0,0%,45%,.8)}.bg-neutral-500\/85{background-color:hsla(0,0%,45%,.85)}.bg-neutral-500\/90{background-color:hsla(0,0%,45%,.9)}.bg-neutral-500\/95{background-color:hsla(0,0%,45%,.95)}.bg-neutral-950{--tw-bg-opacity:1;background-color:rgb(10 10 10/var(--tw-bg-opacity,1))}.bg-neutral-950\/0{background-color:hsla(0,0%,4%,0)}.bg-neutral-950\/10{background-color:hsla(0,0%,4%,.1)}.bg-neutral-950\/100{background-color:#0a0a0a}.bg-neutral-950\/15{background-color:hsla(0,0%,4%,.15)}.bg-neutral-950\/20{background-color:hsla(0,0%,4%,.2)}.bg-neutral-950\/25{background-color:hsla(0,0%,4%,.25)}.bg-neutral-950\/30{background-color:hsla(0,0%,4%,.3)}.bg-neutral-950\/35{background-color:hsla(0,0%,4%,.35)}.bg-neutral-950\/40{background-color:hsla(0,0%,4%,.4)}.bg-neutral-950\/45{background-color:hsla(0,0%,4%,.45)}.bg-neutral-950\/5{background-color:hsla(0,0%,4%,.05)}.bg-neutral-950\/50{background-color:hsla(0,0%,4%,.5)}.bg-neutral-950\/55{background-color:hsla(0,0%,4%,.55)}.bg-neutral-950\/60{background-color:hsla(0,0%,4%,.6)}.bg-neutral-950\/65{background-color:hsla(0,0%,4%,.65)}.bg-neutral-950\/70{background-color:hsla(0,0%,4%,.7)}.bg-neutral-950\/75{background-color:hsla(0,0%,4%,.75)}.bg-neutral-950\/80{background-color:hsla(0,0%,4%,.8)}.bg-neutral-950\/85{background-color:hsla(0,0%,4%,.85)}.bg-neutral-950\/90{background-color:hsla(0,0%,4%,.9)}.bg-neutral-950\/95{background-color:hsla(0,0%,4%,.95)}.bg-orange-100{--tw-bg-opacity:1;background-color:rgb(255 237 213/var(--tw-bg-opacity,1))}.bg-orange-100\/0{background-color:rgba(255,237,213,0)}.bg-orange-100\/10{background-color:rgba(255,237,213,.1)}.bg-orange-100\/100{background-color:#ffedd5}.bg-orange-100\/15{background-color:rgba(255,237,213,.15)}.bg-orange-100\/20{background-color:rgba(255,237,213,.2)}.bg-orange-100\/25{background-color:rgba(255,237,213,.25)}.bg-orange-100\/30{background-color:rgba(255,237,213,.3)}.bg-orange-100\/35{background-color:rgba(255,237,213,.35)}.bg-orange-100\/40{background-color:rgba(255,237,213,.4)}.bg-orange-100\/45{background-color:rgba(255,237,213,.45)}.bg-orange-100\/5{background-color:rgba(255,237,213,.05)}.bg-orange-100\/50{background-color:rgba(255,237,213,.5)}.bg-orange-100\/55{background-color:rgba(255,237,213,.55)}.bg-orange-100\/60{background-color:rgba(255,237,213,.6)}.bg-orange-100\/65{background-color:rgba(255,237,213,.65)}.bg-orange-100\/70{background-color:rgba(255,237,213,.7)}.bg-orange-100\/75{background-color:rgba(255,237,213,.75)}.bg-orange-100\/80{background-color:rgba(255,237,213,.8)}.bg-orange-100\/85{background-color:rgba(255,237,213,.85)}.bg-orange-100\/90{background-color:rgba(255,237,213,.9)}.bg-orange-100\/95{background-color:rgba(255,237,213,.95)}.bg-orange-200{--tw-bg-opacity:1;background-color:rgb(254 215 170/var(--tw-bg-opacity,1))}.bg-orange-200\/0{background-color:hsla(32,98%,83%,0)}.bg-orange-200\/10{background-color:hsla(32,98%,83%,.1)}.bg-orange-200\/100{background-color:#fed7aa}.bg-orange-200\/15{background-color:hsla(32,98%,83%,.15)}.bg-orange-200\/20{background-color:hsla(32,98%,83%,.2)}.bg-orange-200\/25{background-color:hsla(32,98%,83%,.25)}.bg-orange-200\/30{background-color:hsla(32,98%,83%,.3)}.bg-orange-200\/35{background-color:hsla(32,98%,83%,.35)}.bg-orange-200\/40{background-color:hsla(32,98%,83%,.4)}.bg-orange-200\/45{background-color:hsla(32,98%,83%,.45)}.bg-orange-200\/5{background-color:hsla(32,98%,83%,.05)}.bg-orange-200\/50{background-color:hsla(32,98%,83%,.5)}.bg-orange-200\/55{background-color:hsla(32,98%,83%,.55)}.bg-orange-200\/60{background-color:hsla(32,98%,83%,.6)}.bg-orange-200\/65{background-color:hsla(32,98%,83%,.65)}.bg-orange-200\/70{background-color:hsla(32,98%,83%,.7)}.bg-orange-200\/75{background-color:hsla(32,98%,83%,.75)}.bg-orange-200\/80{background-color:hsla(32,98%,83%,.8)}.bg-orange-200\/85{background-color:hsla(32,98%,83%,.85)}.bg-orange-200\/90{background-color:hsla(32,98%,83%,.9)}.bg-orange-200\/95{background-color:hsla(32,98%,83%,.95)}.bg-orange-300{--tw-bg-opacity:1;background-color:rgb(253 186 116/var(--tw-bg-opacity,1))}.bg-orange-400{--tw-bg-opacity:1;background-color:rgb(251 146 60/var(--tw-bg-opacity,1))}.bg-orange-400\/0{background-color:rgba(251,146,60,0)}.bg-orange-400\/10{background-color:rgba(251,146,60,.1)}.bg-orange-400\/100{background-color:#fb923c}.bg-orange-400\/15{background-color:rgba(251,146,60,.15)}.bg-orange-400\/20{background-color:rgba(251,146,60,.2)}.bg-orange-400\/25{background-color:rgba(251,146,60,.25)}.bg-orange-400\/30{background-color:rgba(251,146,60,.3)}.bg-orange-400\/35{background-color:rgba(251,146,60,.35)}.bg-orange-400\/40{background-color:rgba(251,146,60,.4)}.bg-orange-400\/45{background-color:rgba(251,146,60,.45)}.bg-orange-400\/5{background-color:rgba(251,146,60,.05)}.bg-orange-400\/50{background-color:rgba(251,146,60,.5)}.bg-orange-400\/55{background-color:rgba(251,146,60,.55)}.bg-orange-400\/60{background-color:rgba(251,146,60,.6)}.bg-orange-400\/65{background-color:rgba(251,146,60,.65)}.bg-orange-400\/70{background-color:rgba(251,146,60,.7)}.bg-orange-400\/75{background-color:rgba(251,146,60,.75)}.bg-orange-400\/80{background-color:rgba(251,146,60,.8)}.bg-orange-400\/85{background-color:rgba(251,146,60,.85)}.bg-orange-400\/90{background-color:rgba(251,146,60,.9)}.bg-orange-400\/95{background-color:rgba(251,146,60,.95)}.bg-orange-50{--tw-bg-opacity:1;background-color:rgb(255 247 237/var(--tw-bg-opacity,1))}.bg-orange-50\/0{background-color:rgba(255,247,237,0)}.bg-orange-50\/10{background-color:rgba(255,247,237,.1)}.bg-orange-50\/100{background-color:#fff7ed}.bg-orange-50\/15{background-color:rgba(255,247,237,.15)}.bg-orange-50\/20{background-color:rgba(255,247,237,.2)}.bg-orange-50\/25{background-color:rgba(255,247,237,.25)}.bg-orange-50\/30{background-color:rgba(255,247,237,.3)}.bg-orange-50\/35{background-color:rgba(255,247,237,.35)}.bg-orange-50\/40{background-color:rgba(255,247,237,.4)}.bg-orange-50\/45{background-color:rgba(255,247,237,.45)}.bg-orange-50\/5{background-color:rgba(255,247,237,.05)}.bg-orange-50\/50{background-color:rgba(255,247,237,.5)}.bg-orange-50\/55{background-color:rgba(255,247,237,.55)}.bg-orange-50\/60{background-color:rgba(255,247,237,.6)}.bg-orange-50\/65{background-color:rgba(255,247,237,.65)}.bg-orange-50\/70{background-color:rgba(255,247,237,.7)}.bg-orange-50\/75{background-color:rgba(255,247,237,.75)}.bg-orange-50\/80{background-color:rgba(255,247,237,.8)}.bg-orange-50\/85{background-color:rgba(255,247,237,.85)}.bg-orange-50\/90{background-color:rgba(255,247,237,.9)}.bg-orange-50\/95{background-color:rgba(255,247,237,.95)}.bg-orange-500{--tw-bg-opacity:1;background-color:rgb(249 115 22/var(--tw-bg-opacity,1))}.bg-orange-500\/0{background-color:rgba(249,115,22,0)}.bg-orange-500\/10{background-color:rgba(249,115,22,.1)}.bg-orange-500\/100{background-color:#f97316}.bg-orange-500\/15{background-color:rgba(249,115,22,.15)}.bg-orange-500\/20{background-color:rgba(249,115,22,.2)}.bg-orange-500\/25{background-color:rgba(249,115,22,.25)}.bg-orange-500\/30{background-color:rgba(249,115,22,.3)}.bg-orange-500\/35{background-color:rgba(249,115,22,.35)}.bg-orange-500\/40{background-color:rgba(249,115,22,.4)}.bg-orange-500\/45{background-color:rgba(249,115,22,.45)}.bg-orange-500\/5{background-color:rgba(249,115,22,.05)}.bg-orange-500\/50{background-color:rgba(249,115,22,.5)}.bg-orange-500\/55{background-color:rgba(249,115,22,.55)}.bg-orange-500\/60{background-color:rgba(249,115,22,.6)}.bg-orange-500\/65{background-color:rgba(249,115,22,.65)}.bg-orange-500\/70{background-color:rgba(249,115,22,.7)}.bg-orange-500\/75{background-color:rgba(249,115,22,.75)}.bg-orange-500\/80{background-color:rgba(249,115,22,.8)}.bg-orange-500\/85{background-color:rgba(249,115,22,.85)}.bg-orange-500\/90{background-color:rgba(249,115,22,.9)}.bg-orange-500\/95{background-color:rgba(249,115,22,.95)}.bg-orange-950{--tw-bg-opacity:1;background-color:rgb(67 20 7/var(--tw-bg-opacity,1))}.bg-orange-950\/0{background-color:rgba(67,20,7,0)}.bg-orange-950\/10{background-color:rgba(67,20,7,.1)}.bg-orange-950\/100{background-color:#431407}.bg-orange-950\/15{background-color:rgba(67,20,7,.15)}.bg-orange-950\/20{background-color:rgba(67,20,7,.2)}.bg-orange-950\/25{background-color:rgba(67,20,7,.25)}.bg-orange-950\/30{background-color:rgba(67,20,7,.3)}.bg-orange-950\/35{background-color:rgba(67,20,7,.35)}.bg-orange-950\/40{background-color:rgba(67,20,7,.4)}.bg-orange-950\/45{background-color:rgba(67,20,7,.45)}.bg-orange-950\/5{background-color:rgba(67,20,7,.05)}.bg-orange-950\/50{background-color:rgba(67,20,7,.5)}.bg-orange-950\/55{background-color:rgba(67,20,7,.55)}.bg-orange-950\/60{background-color:rgba(67,20,7,.6)}.bg-orange-950\/65{background-color:rgba(67,20,7,.65)}.bg-orange-950\/70{background-color:rgba(67,20,7,.7)}.bg-orange-950\/75{background-color:rgba(67,20,7,.75)}.bg-orange-950\/80{background-color:rgba(67,20,7,.8)}.bg-orange-950\/85{background-color:rgba(67,20,7,.85)}.bg-orange-950\/90{background-color:rgba(67,20,7,.9)}.bg-orange-950\/95{background-color:rgba(67,20,7,.95)}.bg-pink-100{--tw-bg-opacity:1;background-color:rgb(252 231 243/var(--tw-bg-opacity,1))}.bg-pink-100\/0{background-color:rgba(252,231,243,0)}.bg-pink-100\/10{background-color:rgba(252,231,243,.1)}.bg-pink-100\/100{background-color:#fce7f3}.bg-pink-100\/15{background-color:rgba(252,231,243,.15)}.bg-pink-100\/20{background-color:rgba(252,231,243,.2)}.bg-pink-100\/25{background-color:rgba(252,231,243,.25)}.bg-pink-100\/30{background-color:rgba(252,231,243,.3)}.bg-pink-100\/35{background-color:rgba(252,231,243,.35)}.bg-pink-100\/40{background-color:rgba(252,231,243,.4)}.bg-pink-100\/45{background-color:rgba(252,231,243,.45)}.bg-pink-100\/5{background-color:rgba(252,231,243,.05)}.bg-pink-100\/50{background-color:rgba(252,231,243,.5)}.bg-pink-100\/55{background-color:rgba(252,231,243,.55)}.bg-pink-100\/60{background-color:rgba(252,231,243,.6)}.bg-pink-100\/65{background-color:rgba(252,231,243,.65)}.bg-pink-100\/70{background-color:rgba(252,231,243,.7)}.bg-pink-100\/75{background-color:rgba(252,231,243,.75)}.bg-pink-100\/80{background-color:rgba(252,231,243,.8)}.bg-pink-100\/85{background-color:rgba(252,231,243,.85)}.bg-pink-100\/90{background-color:rgba(252,231,243,.9)}.bg-pink-100\/95{background-color:rgba(252,231,243,.95)}.bg-pink-200{--tw-bg-opacity:1;background-color:rgb(251 207 232/var(--tw-bg-opacity,1))}.bg-pink-200\/0{background-color:rgba(251,207,232,0)}.bg-pink-200\/10{background-color:rgba(251,207,232,.1)}.bg-pink-200\/100{background-color:#fbcfe8}.bg-pink-200\/15{background-color:rgba(251,207,232,.15)}.bg-pink-200\/20{background-color:rgba(251,207,232,.2)}.bg-pink-200\/25{background-color:rgba(251,207,232,.25)}.bg-pink-200\/30{background-color:rgba(251,207,232,.3)}.bg-pink-200\/35{background-color:rgba(251,207,232,.35)}.bg-pink-200\/40{background-color:rgba(251,207,232,.4)}.bg-pink-200\/45{background-color:rgba(251,207,232,.45)}.bg-pink-200\/5{background-color:rgba(251,207,232,.05)}.bg-pink-200\/50{background-color:rgba(251,207,232,.5)}.bg-pink-200\/55{background-color:rgba(251,207,232,.55)}.bg-pink-200\/60{background-color:rgba(251,207,232,.6)}.bg-pink-200\/65{background-color:rgba(251,207,232,.65)}.bg-pink-200\/70{background-color:rgba(251,207,232,.7)}.bg-pink-200\/75{background-color:rgba(251,207,232,.75)}.bg-pink-200\/80{background-color:rgba(251,207,232,.8)}.bg-pink-200\/85{background-color:rgba(251,207,232,.85)}.bg-pink-200\/90{background-color:rgba(251,207,232,.9)}.bg-pink-200\/95{background-color:rgba(251,207,232,.95)}.bg-pink-400{--tw-bg-opacity:1;background-color:rgb(244 114 182/var(--tw-bg-opacity,1))}.bg-pink-400\/0{background-color:rgba(244,114,182,0)}.bg-pink-400\/10{background-color:rgba(244,114,182,.1)}.bg-pink-400\/100{background-color:#f472b6}.bg-pink-400\/15{background-color:rgba(244,114,182,.15)}.bg-pink-400\/20{background-color:rgba(244,114,182,.2)}.bg-pink-400\/25{background-color:rgba(244,114,182,.25)}.bg-pink-400\/30{background-color:rgba(244,114,182,.3)}.bg-pink-400\/35{background-color:rgba(244,114,182,.35)}.bg-pink-400\/40{background-color:rgba(244,114,182,.4)}.bg-pink-400\/45{background-color:rgba(244,114,182,.45)}.bg-pink-400\/5{background-color:rgba(244,114,182,.05)}.bg-pink-400\/50{background-color:rgba(244,114,182,.5)}.bg-pink-400\/55{background-color:rgba(244,114,182,.55)}.bg-pink-400\/60{background-color:rgba(244,114,182,.6)}.bg-pink-400\/65{background-color:rgba(244,114,182,.65)}.bg-pink-400\/70{background-color:rgba(244,114,182,.7)}.bg-pink-400\/75{background-color:rgba(244,114,182,.75)}.bg-pink-400\/80{background-color:rgba(244,114,182,.8)}.bg-pink-400\/85{background-color:rgba(244,114,182,.85)}.bg-pink-400\/90{background-color:rgba(244,114,182,.9)}.bg-pink-400\/95{background-color:rgba(244,114,182,.95)}.bg-pink-50{--tw-bg-opacity:1;background-color:rgb(253 242 248/var(--tw-bg-opacity,1))}.bg-pink-50\/0{background-color:rgba(253,242,248,0)}.bg-pink-50\/10{background-color:rgba(253,242,248,.1)}.bg-pink-50\/100{background-color:#fdf2f8}.bg-pink-50\/15{background-color:rgba(253,242,248,.15)}.bg-pink-50\/20{background-color:rgba(253,242,248,.2)}.bg-pink-50\/25{background-color:rgba(253,242,248,.25)}.bg-pink-50\/30{background-color:rgba(253,242,248,.3)}.bg-pink-50\/35{background-color:rgba(253,242,248,.35)}.bg-pink-50\/40{background-color:rgba(253,242,248,.4)}.bg-pink-50\/45{background-color:rgba(253,242,248,.45)}.bg-pink-50\/5{background-color:rgba(253,242,248,.05)}.bg-pink-50\/50{background-color:rgba(253,242,248,.5)}.bg-pink-50\/55{background-color:rgba(253,242,248,.55)}.bg-pink-50\/60{background-color:rgba(253,242,248,.6)}.bg-pink-50\/65{background-color:rgba(253,242,248,.65)}.bg-pink-50\/70{background-color:rgba(253,242,248,.7)}.bg-pink-50\/75{background-color:rgba(253,242,248,.75)}.bg-pink-50\/80{background-color:rgba(253,242,248,.8)}.bg-pink-50\/85{background-color:rgba(253,242,248,.85)}.bg-pink-50\/90{background-color:rgba(253,242,248,.9)}.bg-pink-50\/95{background-color:rgba(253,242,248,.95)}.bg-pink-500{--tw-bg-opacity:1;background-color:rgb(236 72 153/var(--tw-bg-opacity,1))}.bg-pink-500\/0{background-color:rgba(236,72,153,0)}.bg-pink-500\/10{background-color:rgba(236,72,153,.1)}.bg-pink-500\/100{background-color:#ec4899}.bg-pink-500\/15{background-color:rgba(236,72,153,.15)}.bg-pink-500\/20{background-color:rgba(236,72,153,.2)}.bg-pink-500\/25{background-color:rgba(236,72,153,.25)}.bg-pink-500\/30{background-color:rgba(236,72,153,.3)}.bg-pink-500\/35{background-color:rgba(236,72,153,.35)}.bg-pink-500\/40{background-color:rgba(236,72,153,.4)}.bg-pink-500\/45{background-color:rgba(236,72,153,.45)}.bg-pink-500\/5{background-color:rgba(236,72,153,.05)}.bg-pink-500\/50{background-color:rgba(236,72,153,.5)}.bg-pink-500\/55{background-color:rgba(236,72,153,.55)}.bg-pink-500\/60{background-color:rgba(236,72,153,.6)}.bg-pink-500\/65{background-color:rgba(236,72,153,.65)}.bg-pink-500\/70{background-color:rgba(236,72,153,.7)}.bg-pink-500\/75{background-color:rgba(236,72,153,.75)}.bg-pink-500\/80{background-color:rgba(236,72,153,.8)}.bg-pink-500\/85{background-color:rgba(236,72,153,.85)}.bg-pink-500\/90{background-color:rgba(236,72,153,.9)}.bg-pink-500\/95{background-color:rgba(236,72,153,.95)}.bg-pink-950{--tw-bg-opacity:1;background-color:rgb(80 7 36/var(--tw-bg-opacity,1))}.bg-pink-950\/0{background-color:rgba(80,7,36,0)}.bg-pink-950\/10{background-color:rgba(80,7,36,.1)}.bg-pink-950\/100{background-color:#500724}.bg-pink-950\/15{background-color:rgba(80,7,36,.15)}.bg-pink-950\/20{background-color:rgba(80,7,36,.2)}.bg-pink-950\/25{background-color:rgba(80,7,36,.25)}.bg-pink-950\/30{background-color:rgba(80,7,36,.3)}.bg-pink-950\/35{background-color:rgba(80,7,36,.35)}.bg-pink-950\/40{background-color:rgba(80,7,36,.4)}.bg-pink-950\/45{background-color:rgba(80,7,36,.45)}.bg-pink-950\/5{background-color:rgba(80,7,36,.05)}.bg-pink-950\/50{background-color:rgba(80,7,36,.5)}.bg-pink-950\/55{background-color:rgba(80,7,36,.55)}.bg-pink-950\/60{background-color:rgba(80,7,36,.6)}.bg-pink-950\/65{background-color:rgba(80,7,36,.65)}.bg-pink-950\/70{background-color:rgba(80,7,36,.7)}.bg-pink-950\/75{background-color:rgba(80,7,36,.75)}.bg-pink-950\/80{background-color:rgba(80,7,36,.8)}.bg-pink-950\/85{background-color:rgba(80,7,36,.85)}.bg-pink-950\/90{background-color:rgba(80,7,36,.9)}.bg-pink-950\/95{background-color:rgba(80,7,36,.95)}.bg-popover{background-color:hsl(var(--popover))}.bg-primary{background-color:hsl(var(--primary))}.bg-purple-100{--tw-bg-opacity:1;background-color:rgb(243 232 255/var(--tw-bg-opacity,1))}.bg-purple-100\/0{background-color:rgba(243,232,255,0)}.bg-purple-100\/10{background-color:rgba(243,232,255,.1)}.bg-purple-100\/100{background-color:#f3e8ff}.bg-purple-100\/15{background-color:rgba(243,232,255,.15)}.bg-purple-100\/20{background-color:rgba(243,232,255,.2)}.bg-purple-100\/25{background-color:rgba(243,232,255,.25)}.bg-purple-100\/30{background-color:rgba(243,232,255,.3)}.bg-purple-100\/35{background-color:rgba(243,232,255,.35)}.bg-purple-100\/40{background-color:rgba(243,232,255,.4)}.bg-purple-100\/45{background-color:rgba(243,232,255,.45)}.bg-purple-100\/5{background-color:rgba(243,232,255,.05)}.bg-purple-100\/50{background-color:rgba(243,232,255,.5)}.bg-purple-100\/55{background-color:rgba(243,232,255,.55)}.bg-purple-100\/60{background-color:rgba(243,232,255,.6)}.bg-purple-100\/65{background-color:rgba(243,232,255,.65)}.bg-purple-100\/70{background-color:rgba(243,232,255,.7)}.bg-purple-100\/75{background-color:rgba(243,232,255,.75)}.bg-purple-100\/80{background-color:rgba(243,232,255,.8)}.bg-purple-100\/85{background-color:rgba(243,232,255,.85)}.bg-purple-100\/90{background-color:rgba(243,232,255,.9)}.bg-purple-100\/95{background-color:rgba(243,232,255,.95)}.bg-purple-200{--tw-bg-opacity:1;background-color:rgb(233 213 255/var(--tw-bg-opacity,1))}.bg-purple-200\/0{background-color:rgba(233,213,255,0)}.bg-purple-200\/10{background-color:rgba(233,213,255,.1)}.bg-purple-200\/100{background-color:#e9d5ff}.bg-purple-200\/15{background-color:rgba(233,213,255,.15)}.bg-purple-200\/20{background-color:rgba(233,213,255,.2)}.bg-purple-200\/25{background-color:rgba(233,213,255,.25)}.bg-purple-200\/30{background-color:rgba(233,213,255,.3)}.bg-purple-200\/35{background-color:rgba(233,213,255,.35)}.bg-purple-200\/40{background-color:rgba(233,213,255,.4)}.bg-purple-200\/45{background-color:rgba(233,213,255,.45)}.bg-purple-200\/5{background-color:rgba(233,213,255,.05)}.bg-purple-200\/50{background-color:rgba(233,213,255,.5)}.bg-purple-200\/55{background-color:rgba(233,213,255,.55)}.bg-purple-200\/60{background-color:rgba(233,213,255,.6)}.bg-purple-200\/65{background-color:rgba(233,213,255,.65)}.bg-purple-200\/70{background-color:rgba(233,213,255,.7)}.bg-purple-200\/75{background-color:rgba(233,213,255,.75)}.bg-purple-200\/80{background-color:rgba(233,213,255,.8)}.bg-purple-200\/85{background-color:rgba(233,213,255,.85)}.bg-purple-200\/90{background-color:rgba(233,213,255,.9)}.bg-purple-200\/95{background-color:rgba(233,213,255,.95)}.bg-purple-400{--tw-bg-opacity:1;background-color:rgb(192 132 252/var(--tw-bg-opacity,1))}.bg-purple-400\/0{background-color:rgba(192,132,252,0)}.bg-purple-400\/10{background-color:rgba(192,132,252,.1)}.bg-purple-400\/100{background-color:#c084fc}.bg-purple-400\/15{background-color:rgba(192,132,252,.15)}.bg-purple-400\/20{background-color:rgba(192,132,252,.2)}.bg-purple-400\/25{background-color:rgba(192,132,252,.25)}.bg-purple-400\/30{background-color:rgba(192,132,252,.3)}.bg-purple-400\/35{background-color:rgba(192,132,252,.35)}.bg-purple-400\/40{background-color:rgba(192,132,252,.4)}.bg-purple-400\/45{background-color:rgba(192,132,252,.45)}.bg-purple-400\/5{background-color:rgba(192,132,252,.05)}.bg-purple-400\/50{background-color:rgba(192,132,252,.5)}.bg-purple-400\/55{background-color:rgba(192,132,252,.55)}.bg-purple-400\/60{background-color:rgba(192,132,252,.6)}.bg-purple-400\/65{background-color:rgba(192,132,252,.65)}.bg-purple-400\/70{background-color:rgba(192,132,252,.7)}.bg-purple-400\/75{background-color:rgba(192,132,252,.75)}.bg-purple-400\/80{background-color:rgba(192,132,252,.8)}.bg-purple-400\/85{background-color:rgba(192,132,252,.85)}.bg-purple-400\/90{background-color:rgba(192,132,252,.9)}.bg-purple-400\/95{background-color:rgba(192,132,252,.95)}.bg-purple-50{--tw-bg-opacity:1;background-color:rgb(250 245 255/var(--tw-bg-opacity,1))}.bg-purple-50\/0{background-color:rgba(250,245,255,0)}.bg-purple-50\/10{background-color:rgba(250,245,255,.1)}.bg-purple-50\/100{background-color:#faf5ff}.bg-purple-50\/15{background-color:rgba(250,245,255,.15)}.bg-purple-50\/20{background-color:rgba(250,245,255,.2)}.bg-purple-50\/25{background-color:rgba(250,245,255,.25)}.bg-purple-50\/30{background-color:rgba(250,245,255,.3)}.bg-purple-50\/35{background-color:rgba(250,245,255,.35)}.bg-purple-50\/40{background-color:rgba(250,245,255,.4)}.bg-purple-50\/45{background-color:rgba(250,245,255,.45)}.bg-purple-50\/5{background-color:rgba(250,245,255,.05)}.bg-purple-50\/50{background-color:rgba(250,245,255,.5)}.bg-purple-50\/55{background-color:rgba(250,245,255,.55)}.bg-purple-50\/60{background-color:rgba(250,245,255,.6)}.bg-purple-50\/65{background-color:rgba(250,245,255,.65)}.bg-purple-50\/70{background-color:rgba(250,245,255,.7)}.bg-purple-50\/75{background-color:rgba(250,245,255,.75)}.bg-purple-50\/80{background-color:rgba(250,245,255,.8)}.bg-purple-50\/85{background-color:rgba(250,245,255,.85)}.bg-purple-50\/90{background-color:rgba(250,245,255,.9)}.bg-purple-50\/95{background-color:rgba(250,245,255,.95)}.bg-purple-500{--tw-bg-opacity:1;background-color:rgb(168 85 247/var(--tw-bg-opacity,1))}.bg-purple-500\/0{background-color:rgba(168,85,247,0)}.bg-purple-500\/10{background-color:rgba(168,85,247,.1)}.bg-purple-500\/100{background-color:#a855f7}.bg-purple-500\/15{background-color:rgba(168,85,247,.15)}.bg-purple-500\/20{background-color:rgba(168,85,247,.2)}.bg-purple-500\/25{background-color:rgba(168,85,247,.25)}.bg-purple-500\/30{background-color:rgba(168,85,247,.3)}.bg-purple-500\/35{background-color:rgba(168,85,247,.35)}.bg-purple-500\/40{background-color:rgba(168,85,247,.4)}.bg-purple-500\/45{background-color:rgba(168,85,247,.45)}.bg-purple-500\/5{background-color:rgba(168,85,247,.05)}.bg-purple-500\/50{background-color:rgba(168,85,247,.5)}.bg-purple-500\/55{background-color:rgba(168,85,247,.55)}.bg-purple-500\/60{background-color:rgba(168,85,247,.6)}.bg-purple-500\/65{background-color:rgba(168,85,247,.65)}.bg-purple-500\/70{background-color:rgba(168,85,247,.7)}.bg-purple-500\/75{background-color:rgba(168,85,247,.75)}.bg-purple-500\/80{background-color:rgba(168,85,247,.8)}.bg-purple-500\/85{background-color:rgba(168,85,247,.85)}.bg-purple-500\/90{background-color:rgba(168,85,247,.9)}.bg-purple-500\/95{background-color:rgba(168,85,247,.95)}.bg-purple-950{--tw-bg-opacity:1;background-color:rgb(59 7 100/var(--tw-bg-opacity,1))}.bg-purple-950\/0{background-color:rgba(59,7,100,0)}.bg-purple-950\/10{background-color:rgba(59,7,100,.1)}.bg-purple-950\/100{background-color:#3b0764}.bg-purple-950\/15{background-color:rgba(59,7,100,.15)}.bg-purple-950\/20{background-color:rgba(59,7,100,.2)}.bg-purple-950\/25{background-color:rgba(59,7,100,.25)}.bg-purple-950\/30{background-color:rgba(59,7,100,.3)}.bg-purple-950\/35{background-color:rgba(59,7,100,.35)}.bg-purple-950\/40{background-color:rgba(59,7,100,.4)}.bg-purple-950\/45{background-color:rgba(59,7,100,.45)}.bg-purple-950\/5{background-color:rgba(59,7,100,.05)}.bg-purple-950\/50{background-color:rgba(59,7,100,.5)}.bg-purple-950\/55{background-color:rgba(59,7,100,.55)}.bg-purple-950\/60{background-color:rgba(59,7,100,.6)}.bg-purple-950\/65{background-color:rgba(59,7,100,.65)}.bg-purple-950\/70{background-color:rgba(59,7,100,.7)}.bg-purple-950\/75{background-color:rgba(59,7,100,.75)}.bg-purple-950\/80{background-color:rgba(59,7,100,.8)}.bg-purple-950\/85{background-color:rgba(59,7,100,.85)}.bg-purple-950\/90{background-color:rgba(59,7,100,.9)}.bg-purple-950\/95{background-color:rgba(59,7,100,.95)}.bg-red-100{--tw-bg-opacity:1;background-color:rgb(254 226 226/var(--tw-bg-opacity,1))}.bg-red-100\/0{background-color:hsla(0,93%,94%,0)}.bg-red-100\/10{background-color:hsla(0,93%,94%,.1)}.bg-red-100\/100{background-color:#fee2e2}.bg-red-100\/15{background-color:hsla(0,93%,94%,.15)}.bg-red-100\/20{background-color:hsla(0,93%,94%,.2)}.bg-red-100\/25{background-color:hsla(0,93%,94%,.25)}.bg-red-100\/30{background-color:hsla(0,93%,94%,.3)}.bg-red-100\/35{background-color:hsla(0,93%,94%,.35)}.bg-red-100\/40{background-color:hsla(0,93%,94%,.4)}.bg-red-100\/45{background-color:hsla(0,93%,94%,.45)}.bg-red-100\/5{background-color:hsla(0,93%,94%,.05)}.bg-red-100\/50{background-color:hsla(0,93%,94%,.5)}.bg-red-100\/55{background-color:hsla(0,93%,94%,.55)}.bg-red-100\/60{background-color:hsla(0,93%,94%,.6)}.bg-red-100\/65{background-color:hsla(0,93%,94%,.65)}.bg-red-100\/70{background-color:hsla(0,93%,94%,.7)}.bg-red-100\/75{background-color:hsla(0,93%,94%,.75)}.bg-red-100\/80{background-color:hsla(0,93%,94%,.8)}.bg-red-100\/85{background-color:hsla(0,93%,94%,.85)}.bg-red-100\/90{background-color:hsla(0,93%,94%,.9)}.bg-red-100\/95{background-color:hsla(0,93%,94%,.95)}.bg-red-200{--tw-bg-opacity:1;background-color:rgb(254 202 202/var(--tw-bg-opacity,1))}.bg-red-200\/0{background-color:hsla(0,96%,89%,0)}.bg-red-200\/10{background-color:hsla(0,96%,89%,.1)}.bg-red-200\/100{background-color:#fecaca}.bg-red-200\/15{background-color:hsla(0,96%,89%,.15)}.bg-red-200\/20{background-color:hsla(0,96%,89%,.2)}.bg-red-200\/25{background-color:hsla(0,96%,89%,.25)}.bg-red-200\/30{background-color:hsla(0,96%,89%,.3)}.bg-red-200\/35{background-color:hsla(0,96%,89%,.35)}.bg-red-200\/40{background-color:hsla(0,96%,89%,.4)}.bg-red-200\/45{background-color:hsla(0,96%,89%,.45)}.bg-red-200\/5{background-color:hsla(0,96%,89%,.05)}.bg-red-200\/50{background-color:hsla(0,96%,89%,.5)}.bg-red-200\/55{background-color:hsla(0,96%,89%,.55)}.bg-red-200\/60{background-color:hsla(0,96%,89%,.6)}.bg-red-200\/65{background-color:hsla(0,96%,89%,.65)}.bg-red-200\/70{background-color:hsla(0,96%,89%,.7)}.bg-red-200\/75{background-color:hsla(0,96%,89%,.75)}.bg-red-200\/80{background-color:hsla(0,96%,89%,.8)}.bg-red-200\/85{background-color:hsla(0,96%,89%,.85)}.bg-red-200\/90{background-color:hsla(0,96%,89%,.9)}.bg-red-200\/95{background-color:hsla(0,96%,89%,.95)}.bg-red-400{--tw-bg-opacity:1;background-color:rgb(248 113 113/var(--tw-bg-opacity,1))}.bg-red-400\/0{background-color:hsla(0,91%,71%,0)}.bg-red-400\/10{background-color:hsla(0,91%,71%,.1)}.bg-red-400\/100{background-color:#f87171}.bg-red-400\/15{background-color:hsla(0,91%,71%,.15)}.bg-red-400\/20{background-color:hsla(0,91%,71%,.2)}.bg-red-400\/25{background-color:hsla(0,91%,71%,.25)}.bg-red-400\/30{background-color:hsla(0,91%,71%,.3)}.bg-red-400\/35{background-color:hsla(0,91%,71%,.35)}.bg-red-400\/40{background-color:hsla(0,91%,71%,.4)}.bg-red-400\/45{background-color:hsla(0,91%,71%,.45)}.bg-red-400\/5{background-color:hsla(0,91%,71%,.05)}.bg-red-400\/50{background-color:hsla(0,91%,71%,.5)}.bg-red-400\/55{background-color:hsla(0,91%,71%,.55)}.bg-red-400\/60{background-color:hsla(0,91%,71%,.6)}.bg-red-400\/65{background-color:hsla(0,91%,71%,.65)}.bg-red-400\/70{background-color:hsla(0,91%,71%,.7)}.bg-red-400\/75{background-color:hsla(0,91%,71%,.75)}.bg-red-400\/80{background-color:hsla(0,91%,71%,.8)}.bg-red-400\/85{background-color:hsla(0,91%,71%,.85)}.bg-red-400\/90{background-color:hsla(0,91%,71%,.9)}.bg-red-400\/95{background-color:hsla(0,91%,71%,.95)}.bg-red-50{--tw-bg-opacity:1;background-color:rgb(254 242 242/var(--tw-bg-opacity,1))}.bg-red-50\/0{background-color:hsla(0,86%,97%,0)}.bg-red-50\/10{background-color:hsla(0,86%,97%,.1)}.bg-red-50\/100{background-color:#fef2f2}.bg-red-50\/15{background-color:hsla(0,86%,97%,.15)}.bg-red-50\/20{background-color:hsla(0,86%,97%,.2)}.bg-red-50\/25{background-color:hsla(0,86%,97%,.25)}.bg-red-50\/30{background-color:hsla(0,86%,97%,.3)}.bg-red-50\/35{background-color:hsla(0,86%,97%,.35)}.bg-red-50\/40{background-color:hsla(0,86%,97%,.4)}.bg-red-50\/45{background-color:hsla(0,86%,97%,.45)}.bg-red-50\/5{background-color:hsla(0,86%,97%,.05)}.bg-red-50\/50{background-color:hsla(0,86%,97%,.5)}.bg-red-50\/55{background-color:hsla(0,86%,97%,.55)}.bg-red-50\/60{background-color:hsla(0,86%,97%,.6)}.bg-red-50\/65{background-color:hsla(0,86%,97%,.65)}.bg-red-50\/70{background-color:hsla(0,86%,97%,.7)}.bg-red-50\/75{background-color:hsla(0,86%,97%,.75)}.bg-red-50\/80{background-color:hsla(0,86%,97%,.8)}.bg-red-50\/85{background-color:hsla(0,86%,97%,.85)}.bg-red-50\/90{background-color:hsla(0,86%,97%,.9)}.bg-red-50\/95{background-color:hsla(0,86%,97%,.95)}.bg-red-500{--tw-bg-opacity:1;background-color:rgb(239 68 68/var(--tw-bg-opacity,1))}.bg-red-500\/0{background-color:rgba(239,68,68,0)}.bg-red-500\/10{background-color:rgba(239,68,68,.1)}.bg-red-500\/100{background-color:#ef4444}.bg-red-500\/15{background-color:rgba(239,68,68,.15)}.bg-red-500\/20{background-color:rgba(239,68,68,.2)}.bg-red-500\/25{background-color:rgba(239,68,68,.25)}.bg-red-500\/30{background-color:rgba(239,68,68,.3)}.bg-red-500\/35{background-color:rgba(239,68,68,.35)}.bg-red-500\/40{background-color:rgba(239,68,68,.4)}.bg-red-500\/45{background-color:rgba(239,68,68,.45)}.bg-red-500\/5{background-color:rgba(239,68,68,.05)}.bg-red-500\/50{background-color:rgba(239,68,68,.5)}.bg-red-500\/55{background-color:rgba(239,68,68,.55)}.bg-red-500\/60{background-color:rgba(239,68,68,.6)}.bg-red-500\/65{background-color:rgba(239,68,68,.65)}.bg-red-500\/70{background-color:rgba(239,68,68,.7)}.bg-red-500\/75{background-color:rgba(239,68,68,.75)}.bg-red-500\/80{background-color:rgba(239,68,68,.8)}.bg-red-500\/85{background-color:rgba(239,68,68,.85)}.bg-red-500\/90{background-color:rgba(239,68,68,.9)}.bg-red-500\/95{background-color:rgba(239,68,68,.95)}.bg-red-950{--tw-bg-opacity:1;background-color:rgb(69 10 10/var(--tw-bg-opacity,1))}.bg-red-950\/0{background-color:rgba(69,10,10,0)}.bg-red-950\/10{background-color:rgba(69,10,10,.1)}.bg-red-950\/100{background-color:#450a0a}.bg-red-950\/15{background-color:rgba(69,10,10,.15)}.bg-red-950\/20{background-color:rgba(69,10,10,.2)}.bg-red-950\/25{background-color:rgba(69,10,10,.25)}.bg-red-950\/30{background-color:rgba(69,10,10,.3)}.bg-red-950\/35{background-color:rgba(69,10,10,.35)}.bg-red-950\/40{background-color:rgba(69,10,10,.4)}.bg-red-950\/45{background-color:rgba(69,10,10,.45)}.bg-red-950\/5{background-color:rgba(69,10,10,.05)}.bg-red-950\/50{background-color:rgba(69,10,10,.5)}.bg-red-950\/55{background-color:rgba(69,10,10,.55)}.bg-red-950\/60{background-color:rgba(69,10,10,.6)}.bg-red-950\/65{background-color:rgba(69,10,10,.65)}.bg-red-950\/70{background-color:rgba(69,10,10,.7)}.bg-red-950\/75{background-color:rgba(69,10,10,.75)}.bg-red-950\/80{background-color:rgba(69,10,10,.8)}.bg-red-950\/85{background-color:rgba(69,10,10,.85)}.bg-red-950\/90{background-color:rgba(69,10,10,.9)}.bg-red-950\/95{background-color:rgba(69,10,10,.95)}.bg-rose-100{--tw-bg-opacity:1;background-color:rgb(255 228 230/var(--tw-bg-opacity,1))}.bg-rose-100\/0{background-color:rgba(255,228,230,0)}.bg-rose-100\/10{background-color:rgba(255,228,230,.1)}.bg-rose-100\/100{background-color:#ffe4e6}.bg-rose-100\/15{background-color:rgba(255,228,230,.15)}.bg-rose-100\/20{background-color:rgba(255,228,230,.2)}.bg-rose-100\/25{background-color:rgba(255,228,230,.25)}.bg-rose-100\/30{background-color:rgba(255,228,230,.3)}.bg-rose-100\/35{background-color:rgba(255,228,230,.35)}.bg-rose-100\/40{background-color:rgba(255,228,230,.4)}.bg-rose-100\/45{background-color:rgba(255,228,230,.45)}.bg-rose-100\/5{background-color:rgba(255,228,230,.05)}.bg-rose-100\/50{background-color:rgba(255,228,230,.5)}.bg-rose-100\/55{background-color:rgba(255,228,230,.55)}.bg-rose-100\/60{background-color:rgba(255,228,230,.6)}.bg-rose-100\/65{background-color:rgba(255,228,230,.65)}.bg-rose-100\/70{background-color:rgba(255,228,230,.7)}.bg-rose-100\/75{background-color:rgba(255,228,230,.75)}.bg-rose-100\/80{background-color:rgba(255,228,230,.8)}.bg-rose-100\/85{background-color:rgba(255,228,230,.85)}.bg-rose-100\/90{background-color:rgba(255,228,230,.9)}.bg-rose-100\/95{background-color:rgba(255,228,230,.95)}.bg-rose-200{--tw-bg-opacity:1;background-color:rgb(254 205 211/var(--tw-bg-opacity,1))}.bg-rose-200\/0{background-color:rgba(254,205,211,0)}.bg-rose-200\/10{background-color:rgba(254,205,211,.1)}.bg-rose-200\/100{background-color:#fecdd3}.bg-rose-200\/15{background-color:rgba(254,205,211,.15)}.bg-rose-200\/20{background-color:rgba(254,205,211,.2)}.bg-rose-200\/25{background-color:rgba(254,205,211,.25)}.bg-rose-200\/30{background-color:rgba(254,205,211,.3)}.bg-rose-200\/35{background-color:rgba(254,205,211,.35)}.bg-rose-200\/40{background-color:rgba(254,205,211,.4)}.bg-rose-200\/45{background-color:rgba(254,205,211,.45)}.bg-rose-200\/5{background-color:rgba(254,205,211,.05)}.bg-rose-200\/50{background-color:rgba(254,205,211,.5)}.bg-rose-200\/55{background-color:rgba(254,205,211,.55)}.bg-rose-200\/60{background-color:rgba(254,205,211,.6)}.bg-rose-200\/65{background-color:rgba(254,205,211,.65)}.bg-rose-200\/70{background-color:rgba(254,205,211,.7)}.bg-rose-200\/75{background-color:rgba(254,205,211,.75)}.bg-rose-200\/80{background-color:rgba(254,205,211,.8)}.bg-rose-200\/85{background-color:rgba(254,205,211,.85)}.bg-rose-200\/90{background-color:rgba(254,205,211,.9)}.bg-rose-200\/95{background-color:rgba(254,205,211,.95)}.bg-rose-400{--tw-bg-opacity:1;background-color:rgb(251 113 133/var(--tw-bg-opacity,1))}.bg-rose-400\/0{background-color:rgba(251,113,133,0)}.bg-rose-400\/10{background-color:rgba(251,113,133,.1)}.bg-rose-400\/100{background-color:#fb7185}.bg-rose-400\/15{background-color:rgba(251,113,133,.15)}.bg-rose-400\/20{background-color:rgba(251,113,133,.2)}.bg-rose-400\/25{background-color:rgba(251,113,133,.25)}.bg-rose-400\/30{background-color:rgba(251,113,133,.3)}.bg-rose-400\/35{background-color:rgba(251,113,133,.35)}.bg-rose-400\/40{background-color:rgba(251,113,133,.4)}.bg-rose-400\/45{background-color:rgba(251,113,133,.45)}.bg-rose-400\/5{background-color:rgba(251,113,133,.05)}.bg-rose-400\/50{background-color:rgba(251,113,133,.5)}.bg-rose-400\/55{background-color:rgba(251,113,133,.55)}.bg-rose-400\/60{background-color:rgba(251,113,133,.6)}.bg-rose-400\/65{background-color:rgba(251,113,133,.65)}.bg-rose-400\/70{background-color:rgba(251,113,133,.7)}.bg-rose-400\/75{background-color:rgba(251,113,133,.75)}.bg-rose-400\/80{background-color:rgba(251,113,133,.8)}.bg-rose-400\/85{background-color:rgba(251,113,133,.85)}.bg-rose-400\/90{background-color:rgba(251,113,133,.9)}.bg-rose-400\/95{background-color:rgba(251,113,133,.95)}.bg-rose-50{--tw-bg-opacity:1;background-color:rgb(255 241 242/var(--tw-bg-opacity,1))}.bg-rose-50\/0{background-color:rgba(255,241,242,0)}.bg-rose-50\/10{background-color:rgba(255,241,242,.1)}.bg-rose-50\/100{background-color:#fff1f2}.bg-rose-50\/15{background-color:rgba(255,241,242,.15)}.bg-rose-50\/20{background-color:rgba(255,241,242,.2)}.bg-rose-50\/25{background-color:rgba(255,241,242,.25)}.bg-rose-50\/30{background-color:rgba(255,241,242,.3)}.bg-rose-50\/35{background-color:rgba(255,241,242,.35)}.bg-rose-50\/40{background-color:rgba(255,241,242,.4)}.bg-rose-50\/45{background-color:rgba(255,241,242,.45)}.bg-rose-50\/5{background-color:rgba(255,241,242,.05)}.bg-rose-50\/50{background-color:rgba(255,241,242,.5)}.bg-rose-50\/55{background-color:rgba(255,241,242,.55)}.bg-rose-50\/60{background-color:rgba(255,241,242,.6)}.bg-rose-50\/65{background-color:rgba(255,241,242,.65)}.bg-rose-50\/70{background-color:rgba(255,241,242,.7)}.bg-rose-50\/75{background-color:rgba(255,241,242,.75)}.bg-rose-50\/80{background-color:rgba(255,241,242,.8)}.bg-rose-50\/85{background-color:rgba(255,241,242,.85)}.bg-rose-50\/90{background-color:rgba(255,241,242,.9)}.bg-rose-50\/95{background-color:rgba(255,241,242,.95)}.bg-rose-500{--tw-bg-opacity:1;background-color:rgb(244 63 94/var(--tw-bg-opacity,1))}.bg-rose-500\/0{background-color:rgba(244,63,94,0)}.bg-rose-500\/10{background-color:rgba(244,63,94,.1)}.bg-rose-500\/100{background-color:#f43f5e}.bg-rose-500\/15{background-color:rgba(244,63,94,.15)}.bg-rose-500\/20{background-color:rgba(244,63,94,.2)}.bg-rose-500\/25{background-color:rgba(244,63,94,.25)}.bg-rose-500\/30{background-color:rgba(244,63,94,.3)}.bg-rose-500\/35{background-color:rgba(244,63,94,.35)}.bg-rose-500\/40{background-color:rgba(244,63,94,.4)}.bg-rose-500\/45{background-color:rgba(244,63,94,.45)}.bg-rose-500\/5{background-color:rgba(244,63,94,.05)}.bg-rose-500\/50{background-color:rgba(244,63,94,.5)}.bg-rose-500\/55{background-color:rgba(244,63,94,.55)}.bg-rose-500\/60{background-color:rgba(244,63,94,.6)}.bg-rose-500\/65{background-color:rgba(244,63,94,.65)}.bg-rose-500\/70{background-color:rgba(244,63,94,.7)}.bg-rose-500\/75{background-color:rgba(244,63,94,.75)}.bg-rose-500\/80{background-color:rgba(244,63,94,.8)}.bg-rose-500\/85{background-color:rgba(244,63,94,.85)}.bg-rose-500\/90{background-color:rgba(244,63,94,.9)}.bg-rose-500\/95{background-color:rgba(244,63,94,.95)}.bg-rose-950{--tw-bg-opacity:1;background-color:rgb(76 5 25/var(--tw-bg-opacity,1))}.bg-rose-950\/0{background-color:rgba(76,5,25,0)}.bg-rose-950\/10{background-color:rgba(76,5,25,.1)}.bg-rose-950\/100{background-color:#4c0519}.bg-rose-950\/15{background-color:rgba(76,5,25,.15)}.bg-rose-950\/20{background-color:rgba(76,5,25,.2)}.bg-rose-950\/25{background-color:rgba(76,5,25,.25)}.bg-rose-950\/30{background-color:rgba(76,5,25,.3)}.bg-rose-950\/35{background-color:rgba(76,5,25,.35)}.bg-rose-950\/40{background-color:rgba(76,5,25,.4)}.bg-rose-950\/45{background-color:rgba(76,5,25,.45)}.bg-rose-950\/5{background-color:rgba(76,5,25,.05)}.bg-rose-950\/50{background-color:rgba(76,5,25,.5)}.bg-rose-950\/55{background-color:rgba(76,5,25,.55)}.bg-rose-950\/60{background-color:rgba(76,5,25,.6)}.bg-rose-950\/65{background-color:rgba(76,5,25,.65)}.bg-rose-950\/70{background-color:rgba(76,5,25,.7)}.bg-rose-950\/75{background-color:rgba(76,5,25,.75)}.bg-rose-950\/80{background-color:rgba(76,5,25,.8)}.bg-rose-950\/85{background-color:rgba(76,5,25,.85)}.bg-rose-950\/90{background-color:rgba(76,5,25,.9)}.bg-rose-950\/95{background-color:rgba(76,5,25,.95)}.bg-secondary{background-color:hsl(var(--secondary))}.bg-sky-100{--tw-bg-opacity:1;background-color:rgb(224 242 254/var(--tw-bg-opacity,1))}.bg-sky-100\/0{background-color:rgba(224,242,254,0)}.bg-sky-100\/10{background-color:rgba(224,242,254,.1)}.bg-sky-100\/100{background-color:#e0f2fe}.bg-sky-100\/15{background-color:rgba(224,242,254,.15)}.bg-sky-100\/20{background-color:rgba(224,242,254,.2)}.bg-sky-100\/25{background-color:rgba(224,242,254,.25)}.bg-sky-100\/30{background-color:rgba(224,242,254,.3)}.bg-sky-100\/35{background-color:rgba(224,242,254,.35)}.bg-sky-100\/40{background-color:rgba(224,242,254,.4)}.bg-sky-100\/45{background-color:rgba(224,242,254,.45)}.bg-sky-100\/5{background-color:rgba(224,242,254,.05)}.bg-sky-100\/50{background-color:rgba(224,242,254,.5)}.bg-sky-100\/55{background-color:rgba(224,242,254,.55)}.bg-sky-100\/60{background-color:rgba(224,242,254,.6)}.bg-sky-100\/65{background-color:rgba(224,242,254,.65)}.bg-sky-100\/70{background-color:rgba(224,242,254,.7)}.bg-sky-100\/75{background-color:rgba(224,242,254,.75)}.bg-sky-100\/80{background-color:rgba(224,242,254,.8)}.bg-sky-100\/85{background-color:rgba(224,242,254,.85)}.bg-sky-100\/90{background-color:rgba(224,242,254,.9)}.bg-sky-100\/95{background-color:rgba(224,242,254,.95)}.bg-sky-200{--tw-bg-opacity:1;background-color:rgb(186 230 253/var(--tw-bg-opacity,1))}.bg-sky-200\/0{background-color:rgba(186,230,253,0)}.bg-sky-200\/10{background-color:rgba(186,230,253,.1)}.bg-sky-200\/100{background-color:#bae6fd}.bg-sky-200\/15{background-color:rgba(186,230,253,.15)}.bg-sky-200\/20{background-color:rgba(186,230,253,.2)}.bg-sky-200\/25{background-color:rgba(186,230,253,.25)}.bg-sky-200\/30{background-color:rgba(186,230,253,.3)}.bg-sky-200\/35{background-color:rgba(186,230,253,.35)}.bg-sky-200\/40{background-color:rgba(186,230,253,.4)}.bg-sky-200\/45{background-color:rgba(186,230,253,.45)}.bg-sky-200\/5{background-color:rgba(186,230,253,.05)}.bg-sky-200\/50{background-color:rgba(186,230,253,.5)}.bg-sky-200\/55{background-color:rgba(186,230,253,.55)}.bg-sky-200\/60{background-color:rgba(186,230,253,.6)}.bg-sky-200\/65{background-color:rgba(186,230,253,.65)}.bg-sky-200\/70{background-color:rgba(186,230,253,.7)}.bg-sky-200\/75{background-color:rgba(186,230,253,.75)}.bg-sky-200\/80{background-color:rgba(186,230,253,.8)}.bg-sky-200\/85{background-color:rgba(186,230,253,.85)}.bg-sky-200\/90{background-color:rgba(186,230,253,.9)}.bg-sky-200\/95{background-color:rgba(186,230,253,.95)}.bg-sky-400{--tw-bg-opacity:1;background-color:rgb(56 189 248/var(--tw-bg-opacity,1))}.bg-sky-400\/0{background-color:rgba(56,189,248,0)}.bg-sky-400\/10{background-color:rgba(56,189,248,.1)}.bg-sky-400\/100{background-color:#38bdf8}.bg-sky-400\/15{background-color:rgba(56,189,248,.15)}.bg-sky-400\/20{background-color:rgba(56,189,248,.2)}.bg-sky-400\/25{background-color:rgba(56,189,248,.25)}.bg-sky-400\/30{background-color:rgba(56,189,248,.3)}.bg-sky-400\/35{background-color:rgba(56,189,248,.35)}.bg-sky-400\/40{background-color:rgba(56,189,248,.4)}.bg-sky-400\/45{background-color:rgba(56,189,248,.45)}.bg-sky-400\/5{background-color:rgba(56,189,248,.05)}.bg-sky-400\/50{background-color:rgba(56,189,248,.5)}.bg-sky-400\/55{background-color:rgba(56,189,248,.55)}.bg-sky-400\/60{background-color:rgba(56,189,248,.6)}.bg-sky-400\/65{background-color:rgba(56,189,248,.65)}.bg-sky-400\/70{background-color:rgba(56,189,248,.7)}.bg-sky-400\/75{background-color:rgba(56,189,248,.75)}.bg-sky-400\/80{background-color:rgba(56,189,248,.8)}.bg-sky-400\/85{background-color:rgba(56,189,248,.85)}.bg-sky-400\/90{background-color:rgba(56,189,248,.9)}.bg-sky-400\/95{background-color:rgba(56,189,248,.95)}.bg-sky-50{--tw-bg-opacity:1;background-color:rgb(240 249 255/var(--tw-bg-opacity,1))}.bg-sky-50\/0{background-color:rgba(240,249,255,0)}.bg-sky-50\/10{background-color:rgba(240,249,255,.1)}.bg-sky-50\/100{background-color:#f0f9ff}.bg-sky-50\/15{background-color:rgba(240,249,255,.15)}.bg-sky-50\/20{background-color:rgba(240,249,255,.2)}.bg-sky-50\/25{background-color:rgba(240,249,255,.25)}.bg-sky-50\/30{background-color:rgba(240,249,255,.3)}.bg-sky-50\/35{background-color:rgba(240,249,255,.35)}.bg-sky-50\/40{background-color:rgba(240,249,255,.4)}.bg-sky-50\/45{background-color:rgba(240,249,255,.45)}.bg-sky-50\/5{background-color:rgba(240,249,255,.05)}.bg-sky-50\/50{background-color:rgba(240,249,255,.5)}.bg-sky-50\/55{background-color:rgba(240,249,255,.55)}.bg-sky-50\/60{background-color:rgba(240,249,255,.6)}.bg-sky-50\/65{background-color:rgba(240,249,255,.65)}.bg-sky-50\/70{background-color:rgba(240,249,255,.7)}.bg-sky-50\/75{background-color:rgba(240,249,255,.75)}.bg-sky-50\/80{background-color:rgba(240,249,255,.8)}.bg-sky-50\/85{background-color:rgba(240,249,255,.85)}.bg-sky-50\/90{background-color:rgba(240,249,255,.9)}.bg-sky-50\/95{background-color:rgba(240,249,255,.95)}.bg-sky-500{--tw-bg-opacity:1;background-color:rgb(14 165 233/var(--tw-bg-opacity,1))}.bg-sky-500\/0{background-color:rgba(14,165,233,0)}.bg-sky-500\/10{background-color:rgba(14,165,233,.1)}.bg-sky-500\/100{background-color:#0ea5e9}.bg-sky-500\/15{background-color:rgba(14,165,233,.15)}.bg-sky-500\/20{background-color:rgba(14,165,233,.2)}.bg-sky-500\/25{background-color:rgba(14,165,233,.25)}.bg-sky-500\/30{background-color:rgba(14,165,233,.3)}.bg-sky-500\/35{background-color:rgba(14,165,233,.35)}.bg-sky-500\/40{background-color:rgba(14,165,233,.4)}.bg-sky-500\/45{background-color:rgba(14,165,233,.45)}.bg-sky-500\/5{background-color:rgba(14,165,233,.05)}.bg-sky-500\/50{background-color:rgba(14,165,233,.5)}.bg-sky-500\/55{background-color:rgba(14,165,233,.55)}.bg-sky-500\/60{background-color:rgba(14,165,233,.6)}.bg-sky-500\/65{background-color:rgba(14,165,233,.65)}.bg-sky-500\/70{background-color:rgba(14,165,233,.7)}.bg-sky-500\/75{background-color:rgba(14,165,233,.75)}.bg-sky-500\/80{background-color:rgba(14,165,233,.8)}.bg-sky-500\/85{background-color:rgba(14,165,233,.85)}.bg-sky-500\/90{background-color:rgba(14,165,233,.9)}.bg-sky-500\/95{background-color:rgba(14,165,233,.95)}.bg-sky-950{--tw-bg-opacity:1;background-color:rgb(8 47 73/var(--tw-bg-opacity,1))}.bg-sky-950\/0{background-color:rgba(8,47,73,0)}.bg-sky-950\/10{background-color:rgba(8,47,73,.1)}.bg-sky-950\/100{background-color:#082f49}.bg-sky-950\/15{background-color:rgba(8,47,73,.15)}.bg-sky-950\/20{background-color:rgba(8,47,73,.2)}.bg-sky-950\/25{background-color:rgba(8,47,73,.25)}.bg-sky-950\/30{background-color:rgba(8,47,73,.3)}.bg-sky-950\/35{background-color:rgba(8,47,73,.35)}.bg-sky-950\/40{background-color:rgba(8,47,73,.4)}.bg-sky-950\/45{background-color:rgba(8,47,73,.45)}.bg-sky-950\/5{background-color:rgba(8,47,73,.05)}.bg-sky-950\/50{background-color:rgba(8,47,73,.5)}.bg-sky-950\/55{background-color:rgba(8,47,73,.55)}.bg-sky-950\/60{background-color:rgba(8,47,73,.6)}.bg-sky-950\/65{background-color:rgba(8,47,73,.65)}.bg-sky-950\/70{background-color:rgba(8,47,73,.7)}.bg-sky-950\/75{background-color:rgba(8,47,73,.75)}.bg-sky-950\/80{background-color:rgba(8,47,73,.8)}.bg-sky-950\/85{background-color:rgba(8,47,73,.85)}.bg-sky-950\/90{background-color:rgba(8,47,73,.9)}.bg-sky-950\/95{background-color:rgba(8,47,73,.95)}.bg-slate-100{--tw-bg-opacity:1;background-color:rgb(241 245 249/var(--tw-bg-opacity,1))}.bg-slate-100\/0{background-color:rgba(241,245,249,0)}.bg-slate-100\/10{background-color:rgba(241,245,249,.1)}.bg-slate-100\/100{background-color:#f1f5f9}.bg-slate-100\/15{background-color:rgba(241,245,249,.15)}.bg-slate-100\/20{background-color:rgba(241,245,249,.2)}.bg-slate-100\/25{background-color:rgba(241,245,249,.25)}.bg-slate-100\/30{background-color:rgba(241,245,249,.3)}.bg-slate-100\/35{background-color:rgba(241,245,249,.35)}.bg-slate-100\/40{background-color:rgba(241,245,249,.4)}.bg-slate-100\/45{background-color:rgba(241,245,249,.45)}.bg-slate-100\/5{background-color:rgba(241,245,249,.05)}.bg-slate-100\/50{background-color:rgba(241,245,249,.5)}.bg-slate-100\/55{background-color:rgba(241,245,249,.55)}.bg-slate-100\/60{background-color:rgba(241,245,249,.6)}.bg-slate-100\/65{background-color:rgba(241,245,249,.65)}.bg-slate-100\/70{background-color:rgba(241,245,249,.7)}.bg-slate-100\/75{background-color:rgba(241,245,249,.75)}.bg-slate-100\/80{background-color:rgba(241,245,249,.8)}.bg-slate-100\/85{background-color:rgba(241,245,249,.85)}.bg-slate-100\/90{background-color:rgba(241,245,249,.9)}.bg-slate-100\/95{background-color:rgba(241,245,249,.95)}.bg-slate-200{--tw-bg-opacity:1;background-color:rgb(226 232 240/var(--tw-bg-opacity,1))}.bg-slate-200\/0{background-color:rgba(226,232,240,0)}.bg-slate-200\/10{background-color:rgba(226,232,240,.1)}.bg-slate-200\/100{background-color:#e2e8f0}.bg-slate-200\/15{background-color:rgba(226,232,240,.15)}.bg-slate-200\/20{background-color:rgba(226,232,240,.2)}.bg-slate-200\/25{background-color:rgba(226,232,240,.25)}.bg-slate-200\/30{background-color:rgba(226,232,240,.3)}.bg-slate-200\/35{background-color:rgba(226,232,240,.35)}.bg-slate-200\/40{background-color:rgba(226,232,240,.4)}.bg-slate-200\/45{background-color:rgba(226,232,240,.45)}.bg-slate-200\/5{background-color:rgba(226,232,240,.05)}.bg-slate-200\/50{background-color:rgba(226,232,240,.5)}.bg-slate-200\/55{background-color:rgba(226,232,240,.55)}.bg-slate-200\/60{background-color:rgba(226,232,240,.6)}.bg-slate-200\/65{background-color:rgba(226,232,240,.65)}.bg-slate-200\/70{background-color:rgba(226,232,240,.7)}.bg-slate-200\/75{background-color:rgba(226,232,240,.75)}.bg-slate-200\/80{background-color:rgba(226,232,240,.8)}.bg-slate-200\/85{background-color:rgba(226,232,240,.85)}.bg-slate-200\/90{background-color:rgba(226,232,240,.9)}.bg-slate-200\/95{background-color:rgba(226,232,240,.95)}.bg-slate-400{--tw-bg-opacity:1;background-color:rgb(148 163 184/var(--tw-bg-opacity,1))}.bg-slate-400\/0{background-color:rgba(148,163,184,0)}.bg-slate-400\/10{background-color:rgba(148,163,184,.1)}.bg-slate-400\/100{background-color:#94a3b8}.bg-slate-400\/15{background-color:rgba(148,163,184,.15)}.bg-slate-400\/20{background-color:rgba(148,163,184,.2)}.bg-slate-400\/25{background-color:rgba(148,163,184,.25)}.bg-slate-400\/30{background-color:rgba(148,163,184,.3)}.bg-slate-400\/35{background-color:rgba(148,163,184,.35)}.bg-slate-400\/40{background-color:rgba(148,163,184,.4)}.bg-slate-400\/45{background-color:rgba(148,163,184,.45)}.bg-slate-400\/5{background-color:rgba(148,163,184,.05)}.bg-slate-400\/50{background-color:rgba(148,163,184,.5)}.bg-slate-400\/55{background-color:rgba(148,163,184,.55)}.bg-slate-400\/60{background-color:rgba(148,163,184,.6)}.bg-slate-400\/65{background-color:rgba(148,163,184,.65)}.bg-slate-400\/70{background-color:rgba(148,163,184,.7)}.bg-slate-400\/75{background-color:rgba(148,163,184,.75)}.bg-slate-400\/80{background-color:rgba(148,163,184,.8)}.bg-slate-400\/85{background-color:rgba(148,163,184,.85)}.bg-slate-400\/90{background-color:rgba(148,163,184,.9)}.bg-slate-400\/95{background-color:rgba(148,163,184,.95)}.bg-slate-50{--tw-bg-opacity:1;background-color:rgb(248 250 252/var(--tw-bg-opacity,1))}.bg-slate-50\/0{background-color:rgba(248,250,252,0)}.bg-slate-50\/10{background-color:rgba(248,250,252,.1)}.bg-slate-50\/100{background-color:#f8fafc}.bg-slate-50\/15{background-color:rgba(248,250,252,.15)}.bg-slate-50\/20{background-color:rgba(248,250,252,.2)}.bg-slate-50\/25{background-color:rgba(248,250,252,.25)}.bg-slate-50\/30{background-color:rgba(248,250,252,.3)}.bg-slate-50\/35{background-color:rgba(248,250,252,.35)}.bg-slate-50\/40{background-color:rgba(248,250,252,.4)}.bg-slate-50\/45{background-color:rgba(248,250,252,.45)}.bg-slate-50\/5{background-color:rgba(248,250,252,.05)}.bg-slate-50\/50{background-color:rgba(248,250,252,.5)}.bg-slate-50\/55{background-color:rgba(248,250,252,.55)}.bg-slate-50\/60{background-color:rgba(248,250,252,.6)}.bg-slate-50\/65{background-color:rgba(248,250,252,.65)}.bg-slate-50\/70{background-color:rgba(248,250,252,.7)}.bg-slate-50\/75{background-color:rgba(248,250,252,.75)}.bg-slate-50\/80{background-color:rgba(248,250,252,.8)}.bg-slate-50\/85{background-color:rgba(248,250,252,.85)}.bg-slate-50\/90{background-color:rgba(248,250,252,.9)}.bg-slate-50\/95{background-color:rgba(248,250,252,.95)}.bg-slate-500{--tw-bg-opacity:1;background-color:rgb(100 116 139/var(--tw-bg-opacity,1))}.bg-slate-500\/0{background-color:rgba(100,116,139,0)}.bg-slate-500\/10{background-color:rgba(100,116,139,.1)}.bg-slate-500\/100{background-color:#64748b}.bg-slate-500\/15{background-color:rgba(100,116,139,.15)}.bg-slate-500\/20{background-color:rgba(100,116,139,.2)}.bg-slate-500\/25{background-color:rgba(100,116,139,.25)}.bg-slate-500\/30{background-color:rgba(100,116,139,.3)}.bg-slate-500\/35{background-color:rgba(100,116,139,.35)}.bg-slate-500\/40{background-color:rgba(100,116,139,.4)}.bg-slate-500\/45{background-color:rgba(100,116,139,.45)}.bg-slate-500\/5{background-color:rgba(100,116,139,.05)}.bg-slate-500\/50{background-color:rgba(100,116,139,.5)}.bg-slate-500\/55{background-color:rgba(100,116,139,.55)}.bg-slate-500\/60{background-color:rgba(100,116,139,.6)}.bg-slate-500\/65{background-color:rgba(100,116,139,.65)}.bg-slate-500\/70{background-color:rgba(100,116,139,.7)}.bg-slate-500\/75{background-color:rgba(100,116,139,.75)}.bg-slate-500\/80{background-color:rgba(100,116,139,.8)}.bg-slate-500\/85{background-color:rgba(100,116,139,.85)}.bg-slate-500\/90{background-color:rgba(100,116,139,.9)}.bg-slate-500\/95{background-color:rgba(100,116,139,.95)}.bg-slate-950{--tw-bg-opacity:1;background-color:rgb(2 6 23/var(--tw-bg-opacity,1))}.bg-slate-950\/0{background-color:rgba(2,6,23,0)}.bg-slate-950\/10{background-color:rgba(2,6,23,.1)}.bg-slate-950\/100{background-color:#020617}.bg-slate-950\/15{background-color:rgba(2,6,23,.15)}.bg-slate-950\/20{background-color:rgba(2,6,23,.2)}.bg-slate-950\/25{background-color:rgba(2,6,23,.25)}.bg-slate-950\/30{background-color:rgba(2,6,23,.3)}.bg-slate-950\/35{background-color:rgba(2,6,23,.35)}.bg-slate-950\/40{background-color:rgba(2,6,23,.4)}.bg-slate-950\/45{background-color:rgba(2,6,23,.45)}.bg-slate-950\/5{background-color:rgba(2,6,23,.05)}.bg-slate-950\/50{background-color:rgba(2,6,23,.5)}.bg-slate-950\/55{background-color:rgba(2,6,23,.55)}.bg-slate-950\/60{background-color:rgba(2,6,23,.6)}.bg-slate-950\/65{background-color:rgba(2,6,23,.65)}.bg-slate-950\/70{background-color:rgba(2,6,23,.7)}.bg-slate-950\/75{background-color:rgba(2,6,23,.75)}.bg-slate-950\/80{background-color:rgba(2,6,23,.8)}.bg-slate-950\/85{background-color:rgba(2,6,23,.85)}.bg-slate-950\/90{background-color:rgba(2,6,23,.9)}.bg-slate-950\/95{background-color:rgba(2,6,23,.95)}.bg-stone-100{--tw-bg-opacity:1;background-color:rgb(245 245 244/var(--tw-bg-opacity,1))}.bg-stone-100\/0{background-color:hsla(60,5%,96%,0)}.bg-stone-100\/10{background-color:hsla(60,5%,96%,.1)}.bg-stone-100\/100{background-color:#f5f5f4}.bg-stone-100\/15{background-color:hsla(60,5%,96%,.15)}.bg-stone-100\/20{background-color:hsla(60,5%,96%,.2)}.bg-stone-100\/25{background-color:hsla(60,5%,96%,.25)}.bg-stone-100\/30{background-color:hsla(60,5%,96%,.3)}.bg-stone-100\/35{background-color:hsla(60,5%,96%,.35)}.bg-stone-100\/40{background-color:hsla(60,5%,96%,.4)}.bg-stone-100\/45{background-color:hsla(60,5%,96%,.45)}.bg-stone-100\/5{background-color:hsla(60,5%,96%,.05)}.bg-stone-100\/50{background-color:hsla(60,5%,96%,.5)}.bg-stone-100\/55{background-color:hsla(60,5%,96%,.55)}.bg-stone-100\/60{background-color:hsla(60,5%,96%,.6)}.bg-stone-100\/65{background-color:hsla(60,5%,96%,.65)}.bg-stone-100\/70{background-color:hsla(60,5%,96%,.7)}.bg-stone-100\/75{background-color:hsla(60,5%,96%,.75)}.bg-stone-100\/80{background-color:hsla(60,5%,96%,.8)}.bg-stone-100\/85{background-color:hsla(60,5%,96%,.85)}.bg-stone-100\/90{background-color:hsla(60,5%,96%,.9)}.bg-stone-100\/95{background-color:hsla(60,5%,96%,.95)}.bg-stone-200{--tw-bg-opacity:1;background-color:rgb(231 229 228/var(--tw-bg-opacity,1))}.bg-stone-200\/0{background-color:hsla(20,6%,90%,0)}.bg-stone-200\/10{background-color:hsla(20,6%,90%,.1)}.bg-stone-200\/100{background-color:#e7e5e4}.bg-stone-200\/15{background-color:hsla(20,6%,90%,.15)}.bg-stone-200\/20{background-color:hsla(20,6%,90%,.2)}.bg-stone-200\/25{background-color:hsla(20,6%,90%,.25)}.bg-stone-200\/30{background-color:hsla(20,6%,90%,.3)}.bg-stone-200\/35{background-color:hsla(20,6%,90%,.35)}.bg-stone-200\/40{background-color:hsla(20,6%,90%,.4)}.bg-stone-200\/45{background-color:hsla(20,6%,90%,.45)}.bg-stone-200\/5{background-color:hsla(20,6%,90%,.05)}.bg-stone-200\/50{background-color:hsla(20,6%,90%,.5)}.bg-stone-200\/55{background-color:hsla(20,6%,90%,.55)}.bg-stone-200\/60{background-color:hsla(20,6%,90%,.6)}.bg-stone-200\/65{background-color:hsla(20,6%,90%,.65)}.bg-stone-200\/70{background-color:hsla(20,6%,90%,.7)}.bg-stone-200\/75{background-color:hsla(20,6%,90%,.75)}.bg-stone-200\/80{background-color:hsla(20,6%,90%,.8)}.bg-stone-200\/85{background-color:hsla(20,6%,90%,.85)}.bg-stone-200\/90{background-color:hsla(20,6%,90%,.9)}.bg-stone-200\/95{background-color:hsla(20,6%,90%,.95)}.bg-stone-400{--tw-bg-opacity:1;background-color:rgb(168 162 158/var(--tw-bg-opacity,1))}.bg-stone-400\/0{background-color:hsla(24,5%,64%,0)}.bg-stone-400\/10{background-color:hsla(24,5%,64%,.1)}.bg-stone-400\/100{background-color:#a8a29e}.bg-stone-400\/15{background-color:hsla(24,5%,64%,.15)}.bg-stone-400\/20{background-color:hsla(24,5%,64%,.2)}.bg-stone-400\/25{background-color:hsla(24,5%,64%,.25)}.bg-stone-400\/30{background-color:hsla(24,5%,64%,.3)}.bg-stone-400\/35{background-color:hsla(24,5%,64%,.35)}.bg-stone-400\/40{background-color:hsla(24,5%,64%,.4)}.bg-stone-400\/45{background-color:hsla(24,5%,64%,.45)}.bg-stone-400\/5{background-color:hsla(24,5%,64%,.05)}.bg-stone-400\/50{background-color:hsla(24,5%,64%,.5)}.bg-stone-400\/55{background-color:hsla(24,5%,64%,.55)}.bg-stone-400\/60{background-color:hsla(24,5%,64%,.6)}.bg-stone-400\/65{background-color:hsla(24,5%,64%,.65)}.bg-stone-400\/70{background-color:hsla(24,5%,64%,.7)}.bg-stone-400\/75{background-color:hsla(24,5%,64%,.75)}.bg-stone-400\/80{background-color:hsla(24,5%,64%,.8)}.bg-stone-400\/85{background-color:hsla(24,5%,64%,.85)}.bg-stone-400\/90{background-color:hsla(24,5%,64%,.9)}.bg-stone-400\/95{background-color:hsla(24,5%,64%,.95)}.bg-stone-50{--tw-bg-opacity:1;background-color:rgb(250 250 249/var(--tw-bg-opacity,1))}.bg-stone-50\/0{background-color:hsla(60,9%,98%,0)}.bg-stone-50\/10{background-color:hsla(60,9%,98%,.1)}.bg-stone-50\/100{background-color:#fafaf9}.bg-stone-50\/15{background-color:hsla(60,9%,98%,.15)}.bg-stone-50\/20{background-color:hsla(60,9%,98%,.2)}.bg-stone-50\/25{background-color:hsla(60,9%,98%,.25)}.bg-stone-50\/30{background-color:hsla(60,9%,98%,.3)}.bg-stone-50\/35{background-color:hsla(60,9%,98%,.35)}.bg-stone-50\/40{background-color:hsla(60,9%,98%,.4)}.bg-stone-50\/45{background-color:hsla(60,9%,98%,.45)}.bg-stone-50\/5{background-color:hsla(60,9%,98%,.05)}.bg-stone-50\/50{background-color:hsla(60,9%,98%,.5)}.bg-stone-50\/55{background-color:hsla(60,9%,98%,.55)}.bg-stone-50\/60{background-color:hsla(60,9%,98%,.6)}.bg-stone-50\/65{background-color:hsla(60,9%,98%,.65)}.bg-stone-50\/70{background-color:hsla(60,9%,98%,.7)}.bg-stone-50\/75{background-color:hsla(60,9%,98%,.75)}.bg-stone-50\/80{background-color:hsla(60,9%,98%,.8)}.bg-stone-50\/85{background-color:hsla(60,9%,98%,.85)}.bg-stone-50\/90{background-color:hsla(60,9%,98%,.9)}.bg-stone-50\/95{background-color:hsla(60,9%,98%,.95)}.bg-stone-500{--tw-bg-opacity:1;background-color:rgb(120 113 108/var(--tw-bg-opacity,1))}.bg-stone-500\/0{background-color:hsla(25,5%,45%,0)}.bg-stone-500\/10{background-color:hsla(25,5%,45%,.1)}.bg-stone-500\/100{background-color:#78716c}.bg-stone-500\/15{background-color:hsla(25,5%,45%,.15)}.bg-stone-500\/20{background-color:hsla(25,5%,45%,.2)}.bg-stone-500\/25{background-color:hsla(25,5%,45%,.25)}.bg-stone-500\/30{background-color:hsla(25,5%,45%,.3)}.bg-stone-500\/35{background-color:hsla(25,5%,45%,.35)}.bg-stone-500\/40{background-color:hsla(25,5%,45%,.4)}.bg-stone-500\/45{background-color:hsla(25,5%,45%,.45)}.bg-stone-500\/5{background-color:hsla(25,5%,45%,.05)}.bg-stone-500\/50{background-color:hsla(25,5%,45%,.5)}.bg-stone-500\/55{background-color:hsla(25,5%,45%,.55)}.bg-stone-500\/60{background-color:hsla(25,5%,45%,.6)}.bg-stone-500\/65{background-color:hsla(25,5%,45%,.65)}.bg-stone-500\/70{background-color:hsla(25,5%,45%,.7)}.bg-stone-500\/75{background-color:hsla(25,5%,45%,.75)}.bg-stone-500\/80{background-color:hsla(25,5%,45%,.8)}.bg-stone-500\/85{background-color:hsla(25,5%,45%,.85)}.bg-stone-500\/90{background-color:hsla(25,5%,45%,.9)}.bg-stone-500\/95{background-color:hsla(25,5%,45%,.95)}.bg-stone-950{--tw-bg-opacity:1;background-color:rgb(12 10 9/var(--tw-bg-opacity,1))}.bg-stone-950\/0{background-color:rgba(12,10,9,0)}.bg-stone-950\/10{background-color:rgba(12,10,9,.1)}.bg-stone-950\/100{background-color:#0c0a09}.bg-stone-950\/15{background-color:rgba(12,10,9,.15)}.bg-stone-950\/20{background-color:rgba(12,10,9,.2)}.bg-stone-950\/25{background-color:rgba(12,10,9,.25)}.bg-stone-950\/30{background-color:rgba(12,10,9,.3)}.bg-stone-950\/35{background-color:rgba(12,10,9,.35)}.bg-stone-950\/40{background-color:rgba(12,10,9,.4)}.bg-stone-950\/45{background-color:rgba(12,10,9,.45)}.bg-stone-950\/5{background-color:rgba(12,10,9,.05)}.bg-stone-950\/50{background-color:rgba(12,10,9,.5)}.bg-stone-950\/55{background-color:rgba(12,10,9,.55)}.bg-stone-950\/60{background-color:rgba(12,10,9,.6)}.bg-stone-950\/65{background-color:rgba(12,10,9,.65)}.bg-stone-950\/70{background-color:rgba(12,10,9,.7)}.bg-stone-950\/75{background-color:rgba(12,10,9,.75)}.bg-stone-950\/80{background-color:rgba(12,10,9,.8)}.bg-stone-950\/85{background-color:rgba(12,10,9,.85)}.bg-stone-950\/90{background-color:rgba(12,10,9,.9)}.bg-stone-950\/95{background-color:rgba(12,10,9,.95)}.bg-teal-100{--tw-bg-opacity:1;background-color:rgb(204 251 241/var(--tw-bg-opacity,1))}.bg-teal-100\/0{background-color:rgba(204,251,241,0)}.bg-teal-100\/10{background-color:rgba(204,251,241,.1)}.bg-teal-100\/100{background-color:#ccfbf1}.bg-teal-100\/15{background-color:rgba(204,251,241,.15)}.bg-teal-100\/20{background-color:rgba(204,251,241,.2)}.bg-teal-100\/25{background-color:rgba(204,251,241,.25)}.bg-teal-100\/30{background-color:rgba(204,251,241,.3)}.bg-teal-100\/35{background-color:rgba(204,251,241,.35)}.bg-teal-100\/40{background-color:rgba(204,251,241,.4)}.bg-teal-100\/45{background-color:rgba(204,251,241,.45)}.bg-teal-100\/5{background-color:rgba(204,251,241,.05)}.bg-teal-100\/50{background-color:rgba(204,251,241,.5)}.bg-teal-100\/55{background-color:rgba(204,251,241,.55)}.bg-teal-100\/60{background-color:rgba(204,251,241,.6)}.bg-teal-100\/65{background-color:rgba(204,251,241,.65)}.bg-teal-100\/70{background-color:rgba(204,251,241,.7)}.bg-teal-100\/75{background-color:rgba(204,251,241,.75)}.bg-teal-100\/80{background-color:rgba(204,251,241,.8)}.bg-teal-100\/85{background-color:rgba(204,251,241,.85)}.bg-teal-100\/90{background-color:rgba(204,251,241,.9)}.bg-teal-100\/95{background-color:rgba(204,251,241,.95)}.bg-teal-200{--tw-bg-opacity:1;background-color:rgb(153 246 228/var(--tw-bg-opacity,1))}.bg-teal-200\/0{background-color:rgba(153,246,228,0)}.bg-teal-200\/10{background-color:rgba(153,246,228,.1)}.bg-teal-200\/100{background-color:#99f6e4}.bg-teal-200\/15{background-color:rgba(153,246,228,.15)}.bg-teal-200\/20{background-color:rgba(153,246,228,.2)}.bg-teal-200\/25{background-color:rgba(153,246,228,.25)}.bg-teal-200\/30{background-color:rgba(153,246,228,.3)}.bg-teal-200\/35{background-color:rgba(153,246,228,.35)}.bg-teal-200\/40{background-color:rgba(153,246,228,.4)}.bg-teal-200\/45{background-color:rgba(153,246,228,.45)}.bg-teal-200\/5{background-color:rgba(153,246,228,.05)}.bg-teal-200\/50{background-color:rgba(153,246,228,.5)}.bg-teal-200\/55{background-color:rgba(153,246,228,.55)}.bg-teal-200\/60{background-color:rgba(153,246,228,.6)}.bg-teal-200\/65{background-color:rgba(153,246,228,.65)}.bg-teal-200\/70{background-color:rgba(153,246,228,.7)}.bg-teal-200\/75{background-color:rgba(153,246,228,.75)}.bg-teal-200\/80{background-color:rgba(153,246,228,.8)}.bg-teal-200\/85{background-color:rgba(153,246,228,.85)}.bg-teal-200\/90{background-color:rgba(153,246,228,.9)}.bg-teal-200\/95{background-color:rgba(153,246,228,.95)}.bg-teal-400{--tw-bg-opacity:1;background-color:rgb(45 212 191/var(--tw-bg-opacity,1))}.bg-teal-400\/0{background-color:rgba(45,212,191,0)}.bg-teal-400\/10{background-color:rgba(45,212,191,.1)}.bg-teal-400\/100{background-color:#2dd4bf}.bg-teal-400\/15{background-color:rgba(45,212,191,.15)}.bg-teal-400\/20{background-color:rgba(45,212,191,.2)}.bg-teal-400\/25{background-color:rgba(45,212,191,.25)}.bg-teal-400\/30{background-color:rgba(45,212,191,.3)}.bg-teal-400\/35{background-color:rgba(45,212,191,.35)}.bg-teal-400\/40{background-color:rgba(45,212,191,.4)}.bg-teal-400\/45{background-color:rgba(45,212,191,.45)}.bg-teal-400\/5{background-color:rgba(45,212,191,.05)}.bg-teal-400\/50{background-color:rgba(45,212,191,.5)}.bg-teal-400\/55{background-color:rgba(45,212,191,.55)}.bg-teal-400\/60{background-color:rgba(45,212,191,.6)}.bg-teal-400\/65{background-color:rgba(45,212,191,.65)}.bg-teal-400\/70{background-color:rgba(45,212,191,.7)}.bg-teal-400\/75{background-color:rgba(45,212,191,.75)}.bg-teal-400\/80{background-color:rgba(45,212,191,.8)}.bg-teal-400\/85{background-color:rgba(45,212,191,.85)}.bg-teal-400\/90{background-color:rgba(45,212,191,.9)}.bg-teal-400\/95{background-color:rgba(45,212,191,.95)}.bg-teal-50{--tw-bg-opacity:1;background-color:rgb(240 253 250/var(--tw-bg-opacity,1))}.bg-teal-50\/0{background-color:rgba(240,253,250,0)}.bg-teal-50\/10{background-color:rgba(240,253,250,.1)}.bg-teal-50\/100{background-color:#f0fdfa}.bg-teal-50\/15{background-color:rgba(240,253,250,.15)}.bg-teal-50\/20{background-color:rgba(240,253,250,.2)}.bg-teal-50\/25{background-color:rgba(240,253,250,.25)}.bg-teal-50\/30{background-color:rgba(240,253,250,.3)}.bg-teal-50\/35{background-color:rgba(240,253,250,.35)}.bg-teal-50\/40{background-color:rgba(240,253,250,.4)}.bg-teal-50\/45{background-color:rgba(240,253,250,.45)}.bg-teal-50\/5{background-color:rgba(240,253,250,.05)}.bg-teal-50\/50{background-color:rgba(240,253,250,.5)}.bg-teal-50\/55{background-color:rgba(240,253,250,.55)}.bg-teal-50\/60{background-color:rgba(240,253,250,.6)}.bg-teal-50\/65{background-color:rgba(240,253,250,.65)}.bg-teal-50\/70{background-color:rgba(240,253,250,.7)}.bg-teal-50\/75{background-color:rgba(240,253,250,.75)}.bg-teal-50\/80{background-color:rgba(240,253,250,.8)}.bg-teal-50\/85{background-color:rgba(240,253,250,.85)}.bg-teal-50\/90{background-color:rgba(240,253,250,.9)}.bg-teal-50\/95{background-color:rgba(240,253,250,.95)}.bg-teal-500{--tw-bg-opacity:1;background-color:rgb(20 184 166/var(--tw-bg-opacity,1))}.bg-teal-500\/0{background-color:rgba(20,184,166,0)}.bg-teal-500\/10{background-color:rgba(20,184,166,.1)}.bg-teal-500\/100{background-color:#14b8a6}.bg-teal-500\/15{background-color:rgba(20,184,166,.15)}.bg-teal-500\/20{background-color:rgba(20,184,166,.2)}.bg-teal-500\/25{background-color:rgba(20,184,166,.25)}.bg-teal-500\/30{background-color:rgba(20,184,166,.3)}.bg-teal-500\/35{background-color:rgba(20,184,166,.35)}.bg-teal-500\/40{background-color:rgba(20,184,166,.4)}.bg-teal-500\/45{background-color:rgba(20,184,166,.45)}.bg-teal-500\/5{background-color:rgba(20,184,166,.05)}.bg-teal-500\/50{background-color:rgba(20,184,166,.5)}.bg-teal-500\/55{background-color:rgba(20,184,166,.55)}.bg-teal-500\/60{background-color:rgba(20,184,166,.6)}.bg-teal-500\/65{background-color:rgba(20,184,166,.65)}.bg-teal-500\/70{background-color:rgba(20,184,166,.7)}.bg-teal-500\/75{background-color:rgba(20,184,166,.75)}.bg-teal-500\/80{background-color:rgba(20,184,166,.8)}.bg-teal-500\/85{background-color:rgba(20,184,166,.85)}.bg-teal-500\/90{background-color:rgba(20,184,166,.9)}.bg-teal-500\/95{background-color:rgba(20,184,166,.95)}.bg-teal-950{--tw-bg-opacity:1;background-color:rgb(4 47 46/var(--tw-bg-opacity,1))}.bg-teal-950\/0{background-color:rgba(4,47,46,0)}.bg-teal-950\/10{background-color:rgba(4,47,46,.1)}.bg-teal-950\/100{background-color:#042f2e}.bg-teal-950\/15{background-color:rgba(4,47,46,.15)}.bg-teal-950\/20{background-color:rgba(4,47,46,.2)}.bg-teal-950\/25{background-color:rgba(4,47,46,.25)}.bg-teal-950\/30{background-color:rgba(4,47,46,.3)}.bg-teal-950\/35{background-color:rgba(4,47,46,.35)}.bg-teal-950\/40{background-color:rgba(4,47,46,.4)}.bg-teal-950\/45{background-color:rgba(4,47,46,.45)}.bg-teal-950\/5{background-color:rgba(4,47,46,.05)}.bg-teal-950\/50{background-color:rgba(4,47,46,.5)}.bg-teal-950\/55{background-color:rgba(4,47,46,.55)}.bg-teal-950\/60{background-color:rgba(4,47,46,.6)}.bg-teal-950\/65{background-color:rgba(4,47,46,.65)}.bg-teal-950\/70{background-color:rgba(4,47,46,.7)}.bg-teal-950\/75{background-color:rgba(4,47,46,.75)}.bg-teal-950\/80{background-color:rgba(4,47,46,.8)}.bg-teal-950\/85{background-color:rgba(4,47,46,.85)}.bg-teal-950\/90{background-color:rgba(4,47,46,.9)}.bg-teal-950\/95{background-color:rgba(4,47,46,.95)}.bg-transparent{background-color:transparent}.bg-violet-100{--tw-bg-opacity:1;background-color:rgb(237 233 254/var(--tw-bg-opacity,1))}.bg-violet-100\/0{background-color:rgba(237,233,254,0)}.bg-violet-100\/10{background-color:rgba(237,233,254,.1)}.bg-violet-100\/100{background-color:#ede9fe}.bg-violet-100\/15{background-color:rgba(237,233,254,.15)}.bg-violet-100\/20{background-color:rgba(237,233,254,.2)}.bg-violet-100\/25{background-color:rgba(237,233,254,.25)}.bg-violet-100\/30{background-color:rgba(237,233,254,.3)}.bg-violet-100\/35{background-color:rgba(237,233,254,.35)}.bg-violet-100\/40{background-color:rgba(237,233,254,.4)}.bg-violet-100\/45{background-color:rgba(237,233,254,.45)}.bg-violet-100\/5{background-color:rgba(237,233,254,.05)}.bg-violet-100\/50{background-color:rgba(237,233,254,.5)}.bg-violet-100\/55{background-color:rgba(237,233,254,.55)}.bg-violet-100\/60{background-color:rgba(237,233,254,.6)}.bg-violet-100\/65{background-color:rgba(237,233,254,.65)}.bg-violet-100\/70{background-color:rgba(237,233,254,.7)}.bg-violet-100\/75{background-color:rgba(237,233,254,.75)}.bg-violet-100\/80{background-color:rgba(237,233,254,.8)}.bg-violet-100\/85{background-color:rgba(237,233,254,.85)}.bg-violet-100\/90{background-color:rgba(237,233,254,.9)}.bg-violet-100\/95{background-color:rgba(237,233,254,.95)}.bg-violet-200{--tw-bg-opacity:1;background-color:rgb(221 214 254/var(--tw-bg-opacity,1))}.bg-violet-200\/0{background-color:rgba(221,214,254,0)}.bg-violet-200\/10{background-color:rgba(221,214,254,.1)}.bg-violet-200\/100{background-color:#ddd6fe}.bg-violet-200\/15{background-color:rgba(221,214,254,.15)}.bg-violet-200\/20{background-color:rgba(221,214,254,.2)}.bg-violet-200\/25{background-color:rgba(221,214,254,.25)}.bg-violet-200\/30{background-color:rgba(221,214,254,.3)}.bg-violet-200\/35{background-color:rgba(221,214,254,.35)}.bg-violet-200\/40{background-color:rgba(221,214,254,.4)}.bg-violet-200\/45{background-color:rgba(221,214,254,.45)}.bg-violet-200\/5{background-color:rgba(221,214,254,.05)}.bg-violet-200\/50{background-color:rgba(221,214,254,.5)}.bg-violet-200\/55{background-color:rgba(221,214,254,.55)}.bg-violet-200\/60{background-color:rgba(221,214,254,.6)}.bg-violet-200\/65{background-color:rgba(221,214,254,.65)}.bg-violet-200\/70{background-color:rgba(221,214,254,.7)}.bg-violet-200\/75{background-color:rgba(221,214,254,.75)}.bg-violet-200\/80{background-color:rgba(221,214,254,.8)}.bg-violet-200\/85{background-color:rgba(221,214,254,.85)}.bg-violet-200\/90{background-color:rgba(221,214,254,.9)}.bg-violet-200\/95{background-color:rgba(221,214,254,.95)}.bg-violet-400{--tw-bg-opacity:1;background-color:rgb(167 139 250/var(--tw-bg-opacity,1))}.bg-violet-400\/0{background-color:rgba(167,139,250,0)}.bg-violet-400\/10{background-color:rgba(167,139,250,.1)}.bg-violet-400\/100{background-color:#a78bfa}.bg-violet-400\/15{background-color:rgba(167,139,250,.15)}.bg-violet-400\/20{background-color:rgba(167,139,250,.2)}.bg-violet-400\/25{background-color:rgba(167,139,250,.25)}.bg-violet-400\/30{background-color:rgba(167,139,250,.3)}.bg-violet-400\/35{background-color:rgba(167,139,250,.35)}.bg-violet-400\/40{background-color:rgba(167,139,250,.4)}.bg-violet-400\/45{background-color:rgba(167,139,250,.45)}.bg-violet-400\/5{background-color:rgba(167,139,250,.05)}.bg-violet-400\/50{background-color:rgba(167,139,250,.5)}.bg-violet-400\/55{background-color:rgba(167,139,250,.55)}.bg-violet-400\/60{background-color:rgba(167,139,250,.6)}.bg-violet-400\/65{background-color:rgba(167,139,250,.65)}.bg-violet-400\/70{background-color:rgba(167,139,250,.7)}.bg-violet-400\/75{background-color:rgba(167,139,250,.75)}.bg-violet-400\/80{background-color:rgba(167,139,250,.8)}.bg-violet-400\/85{background-color:rgba(167,139,250,.85)}.bg-violet-400\/90{background-color:rgba(167,139,250,.9)}.bg-violet-400\/95{background-color:rgba(167,139,250,.95)}.bg-violet-50{--tw-bg-opacity:1;background-color:rgb(245 243 255/var(--tw-bg-opacity,1))}.bg-violet-50\/0{background-color:rgba(245,243,255,0)}.bg-violet-50\/10{background-color:rgba(245,243,255,.1)}.bg-violet-50\/100{background-color:#f5f3ff}.bg-violet-50\/15{background-color:rgba(245,243,255,.15)}.bg-violet-50\/20{background-color:rgba(245,243,255,.2)}.bg-violet-50\/25{background-color:rgba(245,243,255,.25)}.bg-violet-50\/30{background-color:rgba(245,243,255,.3)}.bg-violet-50\/35{background-color:rgba(245,243,255,.35)}.bg-violet-50\/40{background-color:rgba(245,243,255,.4)}.bg-violet-50\/45{background-color:rgba(245,243,255,.45)}.bg-violet-50\/5{background-color:rgba(245,243,255,.05)}.bg-violet-50\/50{background-color:rgba(245,243,255,.5)}.bg-violet-50\/55{background-color:rgba(245,243,255,.55)}.bg-violet-50\/60{background-color:rgba(245,243,255,.6)}.bg-violet-50\/65{background-color:rgba(245,243,255,.65)}.bg-violet-50\/70{background-color:rgba(245,243,255,.7)}.bg-violet-50\/75{background-color:rgba(245,243,255,.75)}.bg-violet-50\/80{background-color:rgba(245,243,255,.8)}.bg-violet-50\/85{background-color:rgba(245,243,255,.85)}.bg-violet-50\/90{background-color:rgba(245,243,255,.9)}.bg-violet-50\/95{background-color:rgba(245,243,255,.95)}.bg-violet-500{--tw-bg-opacity:1;background-color:rgb(139 92 246/var(--tw-bg-opacity,1))}.bg-violet-500\/0{background-color:rgba(139,92,246,0)}.bg-violet-500\/10{background-color:rgba(139,92,246,.1)}.bg-violet-500\/100{background-color:#8b5cf6}.bg-violet-500\/15{background-color:rgba(139,92,246,.15)}.bg-violet-500\/20{background-color:rgba(139,92,246,.2)}.bg-violet-500\/25{background-color:rgba(139,92,246,.25)}.bg-violet-500\/30{background-color:rgba(139,92,246,.3)}.bg-violet-500\/35{background-color:rgba(139,92,246,.35)}.bg-violet-500\/40{background-color:rgba(139,92,246,.4)}.bg-violet-500\/45{background-color:rgba(139,92,246,.45)}.bg-violet-500\/5{background-color:rgba(139,92,246,.05)}.bg-violet-500\/50{background-color:rgba(139,92,246,.5)}.bg-violet-500\/55{background-color:rgba(139,92,246,.55)}.bg-violet-500\/60{background-color:rgba(139,92,246,.6)}.bg-violet-500\/65{background-color:rgba(139,92,246,.65)}.bg-violet-500\/70{background-color:rgba(139,92,246,.7)}.bg-violet-500\/75{background-color:rgba(139,92,246,.75)}.bg-violet-500\/80{background-color:rgba(139,92,246,.8)}.bg-violet-500\/85{background-color:rgba(139,92,246,.85)}.bg-violet-500\/90{background-color:rgba(139,92,246,.9)}.bg-violet-500\/95{background-color:rgba(139,92,246,.95)}.bg-violet-950{--tw-bg-opacity:1;background-color:rgb(46 16 101/var(--tw-bg-opacity,1))}.bg-violet-950\/0{background-color:rgba(46,16,101,0)}.bg-violet-950\/10{background-color:rgba(46,16,101,.1)}.bg-violet-950\/100{background-color:#2e1065}.bg-violet-950\/15{background-color:rgba(46,16,101,.15)}.bg-violet-950\/20{background-color:rgba(46,16,101,.2)}.bg-violet-950\/25{background-color:rgba(46,16,101,.25)}.bg-violet-950\/30{background-color:rgba(46,16,101,.3)}.bg-violet-950\/35{background-color:rgba(46,16,101,.35)}.bg-violet-950\/40{background-color:rgba(46,16,101,.4)}.bg-violet-950\/45{background-color:rgba(46,16,101,.45)}.bg-violet-950\/5{background-color:rgba(46,16,101,.05)}.bg-violet-950\/50{background-color:rgba(46,16,101,.5)}.bg-violet-950\/55{background-color:rgba(46,16,101,.55)}.bg-violet-950\/60{background-color:rgba(46,16,101,.6)}.bg-violet-950\/65{background-color:rgba(46,16,101,.65)}.bg-violet-950\/70{background-color:rgba(46,16,101,.7)}.bg-violet-950\/75{background-color:rgba(46,16,101,.75)}.bg-violet-950\/80{background-color:rgba(46,16,101,.8)}.bg-violet-950\/85{background-color:rgba(46,16,101,.85)}.bg-violet-950\/90{background-color:rgba(46,16,101,.9)}.bg-violet-950\/95{background-color:rgba(46,16,101,.95)}.bg-white{--tw-bg-opacity:1;background-color:rgb(255 255 255/var(--tw-bg-opacity,1))}.bg-yellow-100{--tw-bg-opacity:1;background-color:rgb(254 249 195/var(--tw-bg-opacity,1))}.bg-yellow-100\/0{background-color:hsla(55,97%,88%,0)}.bg-yellow-100\/10{background-color:hsla(55,97%,88%,.1)}.bg-yellow-100\/100{background-color:#fef9c3}.bg-yellow-100\/15{background-color:hsla(55,97%,88%,.15)}.bg-yellow-100\/20{background-color:hsla(55,97%,88%,.2)}.bg-yellow-100\/25{background-color:hsla(55,97%,88%,.25)}.bg-yellow-100\/30{background-color:hsla(55,97%,88%,.3)}.bg-yellow-100\/35{background-color:hsla(55,97%,88%,.35)}.bg-yellow-100\/40{background-color:hsla(55,97%,88%,.4)}.bg-yellow-100\/45{background-color:hsla(55,97%,88%,.45)}.bg-yellow-100\/5{background-color:hsla(55,97%,88%,.05)}.bg-yellow-100\/50{background-color:hsla(55,97%,88%,.5)}.bg-yellow-100\/55{background-color:hsla(55,97%,88%,.55)}.bg-yellow-100\/60{background-color:hsla(55,97%,88%,.6)}.bg-yellow-100\/65{background-color:hsla(55,97%,88%,.65)}.bg-yellow-100\/70{background-color:hsla(55,97%,88%,.7)}.bg-yellow-100\/75{background-color:hsla(55,97%,88%,.75)}.bg-yellow-100\/80{background-color:hsla(55,97%,88%,.8)}.bg-yellow-100\/85{background-color:hsla(55,97%,88%,.85)}.bg-yellow-100\/90{background-color:hsla(55,97%,88%,.9)}.bg-yellow-100\/95{background-color:hsla(55,97%,88%,.95)}.bg-yellow-200{--tw-bg-opacity:1;background-color:rgb(254 240 138/var(--tw-bg-opacity,1))}.bg-yellow-200\/0{background-color:hsla(53,98%,77%,0)}.bg-yellow-200\/10{background-color:hsla(53,98%,77%,.1)}.bg-yellow-200\/100{background-color:#fef08a}.bg-yellow-200\/15{background-color:hsla(53,98%,77%,.15)}.bg-yellow-200\/20{background-color:hsla(53,98%,77%,.2)}.bg-yellow-200\/25{background-color:hsla(53,98%,77%,.25)}.bg-yellow-200\/30{background-color:hsla(53,98%,77%,.3)}.bg-yellow-200\/35{background-color:hsla(53,98%,77%,.35)}.bg-yellow-200\/40{background-color:hsla(53,98%,77%,.4)}.bg-yellow-200\/45{background-color:hsla(53,98%,77%,.45)}.bg-yellow-200\/5{background-color:hsla(53,98%,77%,.05)}.bg-yellow-200\/50{background-color:hsla(53,98%,77%,.5)}.bg-yellow-200\/55{background-color:hsla(53,98%,77%,.55)}.bg-yellow-200\/60{background-color:hsla(53,98%,77%,.6)}.bg-yellow-200\/65{background-color:hsla(53,98%,77%,.65)}.bg-yellow-200\/70{background-color:hsla(53,98%,77%,.7)}.bg-yellow-200\/75{background-color:hsla(53,98%,77%,.75)}.bg-yellow-200\/80{background-color:hsla(53,98%,77%,.8)}.bg-yellow-200\/85{background-color:hsla(53,98%,77%,.85)}.bg-yellow-200\/90{background-color:hsla(53,98%,77%,.9)}.bg-yellow-200\/95{background-color:hsla(53,98%,77%,.95)}.bg-yellow-400{--tw-bg-opacity:1;background-color:rgb(250 204 21/var(--tw-bg-opacity,1))}.bg-yellow-400\/0{background-color:rgba(250,204,21,0)}.bg-yellow-400\/10{background-color:rgba(250,204,21,.1)}.bg-yellow-400\/100{background-color:#facc15}.bg-yellow-400\/15{background-color:rgba(250,204,21,.15)}.bg-yellow-400\/20{background-color:rgba(250,204,21,.2)}.bg-yellow-400\/25{background-color:rgba(250,204,21,.25)}.bg-yellow-400\/30{background-color:rgba(250,204,21,.3)}.bg-yellow-400\/35{background-color:rgba(250,204,21,.35)}.bg-yellow-400\/40{background-color:rgba(250,204,21,.4)}.bg-yellow-400\/45{background-color:rgba(250,204,21,.45)}.bg-yellow-400\/5{background-color:rgba(250,204,21,.05)}.bg-yellow-400\/50{background-color:rgba(250,204,21,.5)}.bg-yellow-400\/55{background-color:rgba(250,204,21,.55)}.bg-yellow-400\/60{background-color:rgba(250,204,21,.6)}.bg-yellow-400\/65{background-color:rgba(250,204,21,.65)}.bg-yellow-400\/70{background-color:rgba(250,204,21,.7)}.bg-yellow-400\/75{background-color:rgba(250,204,21,.75)}.bg-yellow-400\/80{background-color:rgba(250,204,21,.8)}.bg-yellow-400\/85{background-color:rgba(250,204,21,.85)}.bg-yellow-400\/90{background-color:rgba(250,204,21,.9)}.bg-yellow-400\/95{background-color:rgba(250,204,21,.95)}.bg-yellow-50{--tw-bg-opacity:1;background-color:rgb(254 252 232/var(--tw-bg-opacity,1))}.bg-yellow-50\/0{background-color:hsla(55,92%,95%,0)}.bg-yellow-50\/10{background-color:hsla(55,92%,95%,.1)}.bg-yellow-50\/100{background-color:#fefce8}.bg-yellow-50\/15{background-color:hsla(55,92%,95%,.15)}.bg-yellow-50\/20{background-color:hsla(55,92%,95%,.2)}.bg-yellow-50\/25{background-color:hsla(55,92%,95%,.25)}.bg-yellow-50\/30{background-color:hsla(55,92%,95%,.3)}.bg-yellow-50\/35{background-color:hsla(55,92%,95%,.35)}.bg-yellow-50\/40{background-color:hsla(55,92%,95%,.4)}.bg-yellow-50\/45{background-color:hsla(55,92%,95%,.45)}.bg-yellow-50\/5{background-color:hsla(55,92%,95%,.05)}.bg-yellow-50\/50{background-color:hsla(55,92%,95%,.5)}.bg-yellow-50\/55{background-color:hsla(55,92%,95%,.55)}.bg-yellow-50\/60{background-color:hsla(55,92%,95%,.6)}.bg-yellow-50\/65{background-color:hsla(55,92%,95%,.65)}.bg-yellow-50\/70{background-color:hsla(55,92%,95%,.7)}.bg-yellow-50\/75{background-color:hsla(55,92%,95%,.75)}.bg-yellow-50\/80{background-color:hsla(55,92%,95%,.8)}.bg-yellow-50\/85{background-color:hsla(55,92%,95%,.85)}.bg-yellow-50\/90{background-color:hsla(55,92%,95%,.9)}.bg-yellow-50\/95{background-color:hsla(55,92%,95%,.95)}.bg-yellow-500{--tw-bg-opacity:1;background-color:rgb(234 179 8/var(--tw-bg-opacity,1))}.bg-yellow-500\/0{background-color:rgba(234,179,8,0)}.bg-yellow-500\/10{background-color:rgba(234,179,8,.1)}.bg-yellow-500\/100{background-color:#eab308}.bg-yellow-500\/15{background-color:rgba(234,179,8,.15)}.bg-yellow-500\/20{background-color:rgba(234,179,8,.2)}.bg-yellow-500\/25{background-color:rgba(234,179,8,.25)}.bg-yellow-500\/30{background-color:rgba(234,179,8,.3)}.bg-yellow-500\/35{background-color:rgba(234,179,8,.35)}.bg-yellow-500\/40{background-color:rgba(234,179,8,.4)}.bg-yellow-500\/45{background-color:rgba(234,179,8,.45)}.bg-yellow-500\/5{background-color:rgba(234,179,8,.05)}.bg-yellow-500\/50{background-color:rgba(234,179,8,.5)}.bg-yellow-500\/55{background-color:rgba(234,179,8,.55)}.bg-yellow-500\/60{background-color:rgba(234,179,8,.6)}.bg-yellow-500\/65{background-color:rgba(234,179,8,.65)}.bg-yellow-500\/70{background-color:rgba(234,179,8,.7)}.bg-yellow-500\/75{background-color:rgba(234,179,8,.75)}.bg-yellow-500\/80{background-color:rgba(234,179,8,.8)}.bg-yellow-500\/85{background-color:rgba(234,179,8,.85)}.bg-yellow-500\/90{background-color:rgba(234,179,8,.9)}.bg-yellow-500\/95{background-color:rgba(234,179,8,.95)}.bg-yellow-950{--tw-bg-opacity:1;background-color:rgb(66 32 6/var(--tw-bg-opacity,1))}.bg-yellow-950\/0{background-color:rgba(66,32,6,0)}.bg-yellow-950\/10{background-color:rgba(66,32,6,.1)}.bg-yellow-950\/100{background-color:#422006}.bg-yellow-950\/15{background-color:rgba(66,32,6,.15)}.bg-yellow-950\/20{background-color:rgba(66,32,6,.2)}.bg-yellow-950\/25{background-color:rgba(66,32,6,.25)}.bg-yellow-950\/30{background-color:rgba(66,32,6,.3)}.bg-yellow-950\/35{background-color:rgba(66,32,6,.35)}.bg-yellow-950\/40{background-color:rgba(66,32,6,.4)}.bg-yellow-950\/45{background-color:rgba(66,32,6,.45)}.bg-yellow-950\/5{background-color:rgba(66,32,6,.05)}.bg-yellow-950\/50{background-color:rgba(66,32,6,.5)}.bg-yellow-950\/55{background-color:rgba(66,32,6,.55)}.bg-yellow-950\/60{background-color:rgba(66,32,6,.6)}.bg-yellow-950\/65{background-color:rgba(66,32,6,.65)}.bg-yellow-950\/70{background-color:rgba(66,32,6,.7)}.bg-yellow-950\/75{background-color:rgba(66,32,6,.75)}.bg-yellow-950\/80{background-color:rgba(66,32,6,.8)}.bg-yellow-950\/85{background-color:rgba(66,32,6,.85)}.bg-yellow-950\/90{background-color:rgba(66,32,6,.9)}.bg-yellow-950\/95{background-color:rgba(66,32,6,.95)}.bg-zinc-100{--tw-bg-opacity:1;background-color:rgb(244 244 245/var(--tw-bg-opacity,1))}.bg-zinc-100\/0{background-color:hsla(240,5%,96%,0)}.bg-zinc-100\/10{background-color:hsla(240,5%,96%,.1)}.bg-zinc-100\/100{background-color:#f4f4f5}.bg-zinc-100\/15{background-color:hsla(240,5%,96%,.15)}.bg-zinc-100\/20{background-color:hsla(240,5%,96%,.2)}.bg-zinc-100\/25{background-color:hsla(240,5%,96%,.25)}.bg-zinc-100\/30{background-color:hsla(240,5%,96%,.3)}.bg-zinc-100\/35{background-color:hsla(240,5%,96%,.35)}.bg-zinc-100\/40{background-color:hsla(240,5%,96%,.4)}.bg-zinc-100\/45{background-color:hsla(240,5%,96%,.45)}.bg-zinc-100\/5{background-color:hsla(240,5%,96%,.05)}.bg-zinc-100\/50{background-color:hsla(240,5%,96%,.5)}.bg-zinc-100\/55{background-color:hsla(240,5%,96%,.55)}.bg-zinc-100\/60{background-color:hsla(240,5%,96%,.6)}.bg-zinc-100\/65{background-color:hsla(240,5%,96%,.65)}.bg-zinc-100\/70{background-color:hsla(240,5%,96%,.7)}.bg-zinc-100\/75{background-color:hsla(240,5%,96%,.75)}.bg-zinc-100\/80{background-color:hsla(240,5%,96%,.8)}.bg-zinc-100\/85{background-color:hsla(240,5%,96%,.85)}.bg-zinc-100\/90{background-color:hsla(240,5%,96%,.9)}.bg-zinc-100\/95{background-color:hsla(240,5%,96%,.95)}.bg-zinc-200{--tw-bg-opacity:1;background-color:rgb(228 228 231/var(--tw-bg-opacity,1))}.bg-zinc-200\/0{background-color:hsla(240,6%,90%,0)}.bg-zinc-200\/10{background-color:hsla(240,6%,90%,.1)}.bg-zinc-200\/100{background-color:#e4e4e7}.bg-zinc-200\/15{background-color:hsla(240,6%,90%,.15)}.bg-zinc-200\/20{background-color:hsla(240,6%,90%,.2)}.bg-zinc-200\/25{background-color:hsla(240,6%,90%,.25)}.bg-zinc-200\/30{background-color:hsla(240,6%,90%,.3)}.bg-zinc-200\/35{background-color:hsla(240,6%,90%,.35)}.bg-zinc-200\/40{background-color:hsla(240,6%,90%,.4)}.bg-zinc-200\/45{background-color:hsla(240,6%,90%,.45)}.bg-zinc-200\/5{background-color:hsla(240,6%,90%,.05)}.bg-zinc-200\/50{background-color:hsla(240,6%,90%,.5)}.bg-zinc-200\/55{background-color:hsla(240,6%,90%,.55)}.bg-zinc-200\/60{background-color:hsla(240,6%,90%,.6)}.bg-zinc-200\/65{background-color:hsla(240,6%,90%,.65)}.bg-zinc-200\/70{background-color:hsla(240,6%,90%,.7)}.bg-zinc-200\/75{background-color:hsla(240,6%,90%,.75)}.bg-zinc-200\/80{background-color:hsla(240,6%,90%,.8)}.bg-zinc-200\/85{background-color:hsla(240,6%,90%,.85)}.bg-zinc-200\/90{background-color:hsla(240,6%,90%,.9)}.bg-zinc-200\/95{background-color:hsla(240,6%,90%,.95)}.bg-zinc-400{--tw-bg-opacity:1;background-color:rgb(161 161 170/var(--tw-bg-opacity,1))}.bg-zinc-400\/0{background-color:hsla(240,5%,65%,0)}.bg-zinc-400\/10{background-color:hsla(240,5%,65%,.1)}.bg-zinc-400\/100{background-color:#a1a1aa}.bg-zinc-400\/15{background-color:hsla(240,5%,65%,.15)}.bg-zinc-400\/20{background-color:hsla(240,5%,65%,.2)}.bg-zinc-400\/25{background-color:hsla(240,5%,65%,.25)}.bg-zinc-400\/30{background-color:hsla(240,5%,65%,.3)}.bg-zinc-400\/35{background-color:hsla(240,5%,65%,.35)}.bg-zinc-400\/40{background-color:hsla(240,5%,65%,.4)}.bg-zinc-400\/45{background-color:hsla(240,5%,65%,.45)}.bg-zinc-400\/5{background-color:hsla(240,5%,65%,.05)}.bg-zinc-400\/50{background-color:hsla(240,5%,65%,.5)}.bg-zinc-400\/55{background-color:hsla(240,5%,65%,.55)}.bg-zinc-400\/60{background-color:hsla(240,5%,65%,.6)}.bg-zinc-400\/65{background-color:hsla(240,5%,65%,.65)}.bg-zinc-400\/70{background-color:hsla(240,5%,65%,.7)}.bg-zinc-400\/75{background-color:hsla(240,5%,65%,.75)}.bg-zinc-400\/80{background-color:hsla(240,5%,65%,.8)}.bg-zinc-400\/85{background-color:hsla(240,5%,65%,.85)}.bg-zinc-400\/90{background-color:hsla(240,5%,65%,.9)}.bg-zinc-400\/95{background-color:hsla(240,5%,65%,.95)}.bg-zinc-50{--tw-bg-opacity:1;background-color:rgb(250 250 250/var(--tw-bg-opacity,1))}.bg-zinc-50\/0{background-color:hsla(0,0%,98%,0)}.bg-zinc-50\/10{background-color:hsla(0,0%,98%,.1)}.bg-zinc-50\/100{background-color:#fafafa}.bg-zinc-50\/15{background-color:hsla(0,0%,98%,.15)}.bg-zinc-50\/20{background-color:hsla(0,0%,98%,.2)}.bg-zinc-50\/25{background-color:hsla(0,0%,98%,.25)}.bg-zinc-50\/30{background-color:hsla(0,0%,98%,.3)}.bg-zinc-50\/35{background-color:hsla(0,0%,98%,.35)}.bg-zinc-50\/40{background-color:hsla(0,0%,98%,.4)}.bg-zinc-50\/45{background-color:hsla(0,0%,98%,.45)}.bg-zinc-50\/5{background-color:hsla(0,0%,98%,.05)}.bg-zinc-50\/50{background-color:hsla(0,0%,98%,.5)}.bg-zinc-50\/55{background-color:hsla(0,0%,98%,.55)}.bg-zinc-50\/60{background-color:hsla(0,0%,98%,.6)}.bg-zinc-50\/65{background-color:hsla(0,0%,98%,.65)}.bg-zinc-50\/70{background-color:hsla(0,0%,98%,.7)}.bg-zinc-50\/75{background-color:hsla(0,0%,98%,.75)}.bg-zinc-50\/80{background-color:hsla(0,0%,98%,.8)}.bg-zinc-50\/85{background-color:hsla(0,0%,98%,.85)}.bg-zinc-50\/90{background-color:hsla(0,0%,98%,.9)}.bg-zinc-50\/95{background-color:hsla(0,0%,98%,.95)}.bg-zinc-500{--tw-bg-opacity:1;background-color:rgb(113 113 122/var(--tw-bg-opacity,1))}.bg-zinc-500\/0{background-color:hsla(240,4%,46%,0)}.bg-zinc-500\/10{background-color:hsla(240,4%,46%,.1)}.bg-zinc-500\/100{background-color:#71717a}.bg-zinc-500\/15{background-color:hsla(240,4%,46%,.15)}.bg-zinc-500\/20{background-color:hsla(240,4%,46%,.2)}.bg-zinc-500\/25{background-color:hsla(240,4%,46%,.25)}.bg-zinc-500\/30{background-color:hsla(240,4%,46%,.3)}.bg-zinc-500\/35{background-color:hsla(240,4%,46%,.35)}.bg-zinc-500\/40{background-color:hsla(240,4%,46%,.4)}.bg-zinc-500\/45{background-color:hsla(240,4%,46%,.45)}.bg-zinc-500\/5{background-color:hsla(240,4%,46%,.05)}.bg-zinc-500\/50{background-color:hsla(240,4%,46%,.5)}.bg-zinc-500\/55{background-color:hsla(240,4%,46%,.55)}.bg-zinc-500\/60{background-color:hsla(240,4%,46%,.6)}.bg-zinc-500\/65{background-color:hsla(240,4%,46%,.65)}.bg-zinc-500\/70{background-color:hsla(240,4%,46%,.7)}.bg-zinc-500\/75{background-color:hsla(240,4%,46%,.75)}.bg-zinc-500\/80{background-color:hsla(240,4%,46%,.8)}.bg-zinc-500\/85{background-color:hsla(240,4%,46%,.85)}.bg-zinc-500\/90{background-color:hsla(240,4%,46%,.9)}.bg-zinc-500\/95{background-color:hsla(240,4%,46%,.95)}.bg-zinc-950{--tw-bg-opacity:1;background-color:rgb(9 9 11/var(--tw-bg-opacity,1))}.bg-zinc-950\/0{background-color:rgba(9,9,11,0)}.bg-zinc-950\/10{background-color:rgba(9,9,11,.1)}.bg-zinc-950\/100{background-color:#09090b}.bg-zinc-950\/15{background-color:rgba(9,9,11,.15)}.bg-zinc-950\/20{background-color:rgba(9,9,11,.2)}.bg-zinc-950\/25{background-color:rgba(9,9,11,.25)}.bg-zinc-950\/30{background-color:rgba(9,9,11,.3)}.bg-zinc-950\/35{background-color:rgba(9,9,11,.35)}.bg-zinc-950\/40{background-color:rgba(9,9,11,.4)}.bg-zinc-950\/45{background-color:rgba(9,9,11,.45)}.bg-zinc-950\/5{background-color:rgba(9,9,11,.05)}.bg-zinc-950\/50{background-color:rgba(9,9,11,.5)}.bg-zinc-950\/55{background-color:rgba(9,9,11,.55)}.bg-zinc-950\/60{background-color:rgba(9,9,11,.6)}.bg-zinc-950\/65{background-color:rgba(9,9,11,.65)}.bg-zinc-950\/70{background-color:rgba(9,9,11,.7)}.bg-zinc-950\/75{background-color:rgba(9,9,11,.75)}.bg-zinc-950\/80{background-color:rgba(9,9,11,.8)}.bg-zinc-950\/85{background-color:rgba(9,9,11,.85)}.bg-zinc-950\/90{background-color:rgba(9,9,11,.9)}.bg-zinc-950\/95{background-color:rgba(9,9,11,.95)}.bg-opacity-15{--tw-bg-opacity:0.15}.bg-opacity-25{--tw-bg-opacity:0.25}.bg-opacity-50{--tw-bg-opacity:0.5}.\!bg-none{background-image:none!important}.bg-gradient-to-b{background-image:linear-gradient(to bottom,var(--tw-gradient-stops))}.from-\[hsl\(var\(--background\)\)\],.from-background{--tw-gradient-from:hsl(var(--background)) var(--tw-gradient-from-position);--tw-gradient-to:hsl(var(--background)/0) var(--tw-gradient-to-position);--tw-gradient-stops:var(--tw-gradient-from),var(--tw-gradient-to)}.from-white{--tw-gradient-from:#fff var(--tw-gradient-from-position);--tw-gradient-to:hsla(0,0%,100%,0) var(--tw-gradient-to-position);--tw-gradient-stops:var(--tw-gradient-from),var(--tw-gradient-to)}.to-amber-100{--tw-gradient-to:#fef3c7 var(--tw-gradient-to-position)}.to-amber-100\/0{--tw-gradient-to:hsla(48,96%,89%,0) var(--tw-gradient-to-position)}.to-amber-100\/10{--tw-gradient-to:hsla(48,96%,89%,.1) var(--tw-gradient-to-position)}.to-amber-100\/100{--tw-gradient-to:#fef3c7 var(--tw-gradient-to-position)}.to-amber-100\/15{--tw-gradient-to:hsla(48,96%,89%,.15) var(--tw-gradient-to-position)}.to-amber-100\/20{--tw-gradient-to:hsla(48,96%,89%,.2) var(--tw-gradient-to-position)}.to-amber-100\/25{--tw-gradient-to:hsla(48,96%,89%,.25) var(--tw-gradient-to-position)}.to-amber-100\/30{--tw-gradient-to:hsla(48,96%,89%,.3) var(--tw-gradient-to-position)}.to-amber-100\/35{--tw-gradient-to:hsla(48,96%,89%,.35) var(--tw-gradient-to-position)}.to-amber-100\/40{--tw-gradient-to:hsla(48,96%,89%,.4) var(--tw-gradient-to-position)}.to-amber-100\/45{--tw-gradient-to:hsla(48,96%,89%,.45) var(--tw-gradient-to-position)}.to-amber-100\/5{--tw-gradient-to:hsla(48,96%,89%,.05) var(--tw-gradient-to-position)}.to-amber-100\/50{--tw-gradient-to:hsla(48,96%,89%,.5) var(--tw-gradient-to-position)}.to-amber-100\/55{--tw-gradient-to:hsla(48,96%,89%,.55) var(--tw-gradient-to-position)}.to-amber-100\/60{--tw-gradient-to:hsla(48,96%,89%,.6) var(--tw-gradient-to-position)}.to-amber-100\/65{--tw-gradient-to:hsla(48,96%,89%,.65) var(--tw-gradient-to-position)}.to-amber-100\/70{--tw-gradient-to:hsla(48,96%,89%,.7) var(--tw-gradient-to-position)}.to-amber-100\/75{--tw-gradient-to:hsla(48,96%,89%,.75) var(--tw-gradient-to-position)}.to-amber-100\/80{--tw-gradient-to:hsla(48,96%,89%,.8) var(--tw-gradient-to-position)}.to-amber-100\/85{--tw-gradient-to:hsla(48,96%,89%,.85) var(--tw-gradient-to-position)}.to-amber-100\/90{--tw-gradient-to:hsla(48,96%,89%,.9) var(--tw-gradient-to-position)}.to-amber-100\/95{--tw-gradient-to:hsla(48,96%,89%,.95) var(--tw-gradient-to-position)}.to-amber-200{--tw-gradient-to:#fde68a var(--tw-gradient-to-position)}.to-amber-200\/0{--tw-gradient-to:hsla(48,97%,77%,0) var(--tw-gradient-to-position)}.to-amber-200\/10{--tw-gradient-to:hsla(48,97%,77%,.1) var(--tw-gradient-to-position)}.to-amber-200\/100{--tw-gradient-to:#fde68a var(--tw-gradient-to-position)}.to-amber-200\/15{--tw-gradient-to:hsla(48,97%,77%,.15) var(--tw-gradient-to-position)}.to-amber-200\/20{--tw-gradient-to:hsla(48,97%,77%,.2) var(--tw-gradient-to-position)}.to-amber-200\/25{--tw-gradient-to:hsla(48,97%,77%,.25) var(--tw-gradient-to-position)}.to-amber-200\/30{--tw-gradient-to:hsla(48,97%,77%,.3) var(--tw-gradient-to-position)}.to-amber-200\/35{--tw-gradient-to:hsla(48,97%,77%,.35) var(--tw-gradient-to-position)}.to-amber-200\/40{--tw-gradient-to:hsla(48,97%,77%,.4) var(--tw-gradient-to-position)}.to-amber-200\/45{--tw-gradient-to:hsla(48,97%,77%,.45) var(--tw-gradient-to-position)}.to-amber-200\/5{--tw-gradient-to:hsla(48,97%,77%,.05) var(--tw-gradient-to-position)}.to-amber-200\/50{--tw-gradient-to:hsla(48,97%,77%,.5) var(--tw-gradient-to-position)}.to-amber-200\/55{--tw-gradient-to:hsla(48,97%,77%,.55) var(--tw-gradient-to-position)}.to-amber-200\/60{--tw-gradient-to:hsla(48,97%,77%,.6) var(--tw-gradient-to-position)}.to-amber-200\/65{--tw-gradient-to:hsla(48,97%,77%,.65) var(--tw-gradient-to-position)}.to-amber-200\/70{--tw-gradient-to:hsla(48,97%,77%,.7) var(--tw-gradient-to-position)}.to-amber-200\/75{--tw-gradient-to:hsla(48,97%,77%,.75) var(--tw-gradient-to-position)}.to-amber-200\/80{--tw-gradient-to:hsla(48,97%,77%,.8) var(--tw-gradient-to-position)}.to-amber-200\/85{--tw-gradient-to:hsla(48,97%,77%,.85) var(--tw-gradient-to-position)}.to-amber-200\/90{--tw-gradient-to:hsla(48,97%,77%,.9) var(--tw-gradient-to-position)}.to-amber-200\/95{--tw-gradient-to:hsla(48,97%,77%,.95) var(--tw-gradient-to-position)}.to-amber-50{--tw-gradient-to:#fffbeb var(--tw-gradient-to-position)}.to-amber-50\/0{--tw-gradient-to:rgba(255,251,235,0) var(--tw-gradient-to-position)}.to-amber-50\/10{--tw-gradient-to:rgba(255,251,235,.1) var(--tw-gradient-to-position)}.to-amber-50\/100{--tw-gradient-to:#fffbeb var(--tw-gradient-to-position)}.to-amber-50\/15{--tw-gradient-to:rgba(255,251,235,.15) var(--tw-gradient-to-position)}.to-amber-50\/20{--tw-gradient-to:rgba(255,251,235,.2) var(--tw-gradient-to-position)}.to-amber-50\/25{--tw-gradient-to:rgba(255,251,235,.25) var(--tw-gradient-to-position)}.to-amber-50\/30{--tw-gradient-to:rgba(255,251,235,.3) var(--tw-gradient-to-position)}.to-amber-50\/35{--tw-gradient-to:rgba(255,251,235,.35) var(--tw-gradient-to-position)}.to-amber-50\/40{--tw-gradient-to:rgba(255,251,235,.4) var(--tw-gradient-to-position)}.to-amber-50\/45{--tw-gradient-to:rgba(255,251,235,.45) var(--tw-gradient-to-position)}.to-amber-50\/5{--tw-gradient-to:rgba(255,251,235,.05) var(--tw-gradient-to-position)}.to-amber-50\/50{--tw-gradient-to:rgba(255,251,235,.5) var(--tw-gradient-to-position)}.to-amber-50\/55{--tw-gradient-to:rgba(255,251,235,.55) var(--tw-gradient-to-position)}.to-amber-50\/60{--tw-gradient-to:rgba(255,251,235,.6) var(--tw-gradient-to-position)}.to-amber-50\/65{--tw-gradient-to:rgba(255,251,235,.65) var(--tw-gradient-to-position)}.to-amber-50\/70{--tw-gradient-to:rgba(255,251,235,.7) var(--tw-gradient-to-position)}.to-amber-50\/75{--tw-gradient-to:rgba(255,251,235,.75) var(--tw-gradient-to-position)}.to-amber-50\/80{--tw-gradient-to:rgba(255,251,235,.8) var(--tw-gradient-to-position)}.to-amber-50\/85{--tw-gradient-to:rgba(255,251,235,.85) var(--tw-gradient-to-position)}.to-amber-50\/90{--tw-gradient-to:rgba(255,251,235,.9) var(--tw-gradient-to-position)}.to-amber-50\/95{--tw-gradient-to:rgba(255,251,235,.95) var(--tw-gradient-to-position)}.to-amber-500{--tw-gradient-to:#f59e0b var(--tw-gradient-to-position)}.to-amber-500\/0{--tw-gradient-to:rgba(245,158,11,0) var(--tw-gradient-to-position)}.to-amber-500\/10{--tw-gradient-to:rgba(245,158,11,.1) var(--tw-gradient-to-position)}.to-amber-500\/100{--tw-gradient-to:#f59e0b var(--tw-gradient-to-position)}.to-amber-500\/15{--tw-gradient-to:rgba(245,158,11,.15) var(--tw-gradient-to-position)}.to-amber-500\/20{--tw-gradient-to:rgba(245,158,11,.2) var(--tw-gradient-to-position)}.to-amber-500\/25{--tw-gradient-to:rgba(245,158,11,.25) var(--tw-gradient-to-position)}.to-amber-500\/30{--tw-gradient-to:rgba(245,158,11,.3) var(--tw-gradient-to-position)}.to-amber-500\/35{--tw-gradient-to:rgba(245,158,11,.35) var(--tw-gradient-to-position)}.to-amber-500\/40{--tw-gradient-to:rgba(245,158,11,.4) var(--tw-gradient-to-position)}.to-amber-500\/45{--tw-gradient-to:rgba(245,158,11,.45) var(--tw-gradient-to-position)}.to-amber-500\/5{--tw-gradient-to:rgba(245,158,11,.05) var(--tw-gradient-to-position)}.to-amber-500\/50{--tw-gradient-to:rgba(245,158,11,.5) var(--tw-gradient-to-position)}.to-amber-500\/55{--tw-gradient-to:rgba(245,158,11,.55) var(--tw-gradient-to-position)}.to-amber-500\/60{--tw-gradient-to:rgba(245,158,11,.6) var(--tw-gradient-to-position)}.to-amber-500\/65{--tw-gradient-to:rgba(245,158,11,.65) var(--tw-gradient-to-position)}.to-amber-500\/70{--tw-gradient-to:rgba(245,158,11,.7) var(--tw-gradient-to-position)}.to-amber-500\/75{--tw-gradient-to:rgba(245,158,11,.75) var(--tw-gradient-to-position)}.to-amber-500\/80{--tw-gradient-to:rgba(245,158,11,.8) var(--tw-gradient-to-position)}.to-amber-500\/85{--tw-gradient-to:rgba(245,158,11,.85) var(--tw-gradient-to-position)}.to-amber-500\/90{--tw-gradient-to:rgba(245,158,11,.9) var(--tw-gradient-to-position)}.to-amber-500\/95{--tw-gradient-to:rgba(245,158,11,.95) var(--tw-gradient-to-position)}.to-amber-950{--tw-gradient-to:#451a03 var(--tw-gradient-to-position)}.to-amber-950\/0{--tw-gradient-to:rgba(69,26,3,0) var(--tw-gradient-to-position)}.to-amber-950\/10{--tw-gradient-to:rgba(69,26,3,.1) var(--tw-gradient-to-position)}.to-amber-950\/100{--tw-gradient-to:#451a03 var(--tw-gradient-to-position)}.to-amber-950\/15{--tw-gradient-to:rgba(69,26,3,.15) var(--tw-gradient-to-position)}.to-amber-950\/20{--tw-gradient-to:rgba(69,26,3,.2) var(--tw-gradient-to-position)}.to-amber-950\/25{--tw-gradient-to:rgba(69,26,3,.25) var(--tw-gradient-to-position)}.to-amber-950\/30{--tw-gradient-to:rgba(69,26,3,.3) var(--tw-gradient-to-position)}.to-amber-950\/35{--tw-gradient-to:rgba(69,26,3,.35) var(--tw-gradient-to-position)}.to-amber-950\/40{--tw-gradient-to:rgba(69,26,3,.4) var(--tw-gradient-to-position)}.to-amber-950\/45{--tw-gradient-to:rgba(69,26,3,.45) var(--tw-gradient-to-position)}.to-amber-950\/5{--tw-gradient-to:rgba(69,26,3,.05) var(--tw-gradient-to-position)}.to-amber-950\/50{--tw-gradient-to:rgba(69,26,3,.5) var(--tw-gradient-to-position)}.to-amber-950\/55{--tw-gradient-to:rgba(69,26,3,.55) var(--tw-gradient-to-position)}.to-amber-950\/60{--tw-gradient-to:rgba(69,26,3,.6) var(--tw-gradient-to-position)}.to-amber-950\/65{--tw-gradient-to:rgba(69,26,3,.65) var(--tw-gradient-to-position)}.to-amber-950\/70{--tw-gradient-to:rgba(69,26,3,.7) var(--tw-gradient-to-position)}.to-amber-950\/75{--tw-gradient-to:rgba(69,26,3,.75) var(--tw-gradient-to-position)}.to-amber-950\/80{--tw-gradient-to:rgba(69,26,3,.8) var(--tw-gradient-to-position)}.to-amber-950\/85{--tw-gradient-to:rgba(69,26,3,.85) var(--tw-gradient-to-position)}.to-amber-950\/90{--tw-gradient-to:rgba(69,26,3,.9) var(--tw-gradient-to-position)}.to-amber-950\/95{--tw-gradient-to:rgba(69,26,3,.95) var(--tw-gradient-to-position)}.to-blue-100{--tw-gradient-to:#dbeafe var(--tw-gradient-to-position)}.to-blue-100\/0{--tw-gradient-to:rgba(219,234,254,0) var(--tw-gradient-to-position)}.to-blue-100\/10{--tw-gradient-to:rgba(219,234,254,.1) var(--tw-gradient-to-position)}.to-blue-100\/100{--tw-gradient-to:#dbeafe var(--tw-gradient-to-position)}.to-blue-100\/15{--tw-gradient-to:rgba(219,234,254,.15) var(--tw-gradient-to-position)}.to-blue-100\/20{--tw-gradient-to:rgba(219,234,254,.2) var(--tw-gradient-to-position)}.to-blue-100\/25{--tw-gradient-to:rgba(219,234,254,.25) var(--tw-gradient-to-position)}.to-blue-100\/30{--tw-gradient-to:rgba(219,234,254,.3) var(--tw-gradient-to-position)}.to-blue-100\/35{--tw-gradient-to:rgba(219,234,254,.35) var(--tw-gradient-to-position)}.to-blue-100\/40{--tw-gradient-to:rgba(219,234,254,.4) var(--tw-gradient-to-position)}.to-blue-100\/45{--tw-gradient-to:rgba(219,234,254,.45) var(--tw-gradient-to-position)}.to-blue-100\/5{--tw-gradient-to:rgba(219,234,254,.05) var(--tw-gradient-to-position)}.to-blue-100\/50{--tw-gradient-to:rgba(219,234,254,.5) var(--tw-gradient-to-position)}.to-blue-100\/55{--tw-gradient-to:rgba(219,234,254,.55) var(--tw-gradient-to-position)}.to-blue-100\/60{--tw-gradient-to:rgba(219,234,254,.6) var(--tw-gradient-to-position)}.to-blue-100\/65{--tw-gradient-to:rgba(219,234,254,.65) var(--tw-gradient-to-position)}.to-blue-100\/70{--tw-gradient-to:rgba(219,234,254,.7) var(--tw-gradient-to-position)}.to-blue-100\/75{--tw-gradient-to:rgba(219,234,254,.75) var(--tw-gradient-to-position)}.to-blue-100\/80{--tw-gradient-to:rgba(219,234,254,.8) var(--tw-gradient-to-position)}.to-blue-100\/85{--tw-gradient-to:rgba(219,234,254,.85) var(--tw-gradient-to-position)}.to-blue-100\/90{--tw-gradient-to:rgba(219,234,254,.9) var(--tw-gradient-to-position)}.to-blue-100\/95{--tw-gradient-to:rgba(219,234,254,.95) var(--tw-gradient-to-position)}.to-blue-200{--tw-gradient-to:#bfdbfe var(--tw-gradient-to-position)}.to-blue-200\/0{--tw-gradient-to:rgba(191,219,254,0) var(--tw-gradient-to-position)}.to-blue-200\/10{--tw-gradient-to:rgba(191,219,254,.1) var(--tw-gradient-to-position)}.to-blue-200\/100{--tw-gradient-to:#bfdbfe var(--tw-gradient-to-position)}.to-blue-200\/15{--tw-gradient-to:rgba(191,219,254,.15) var(--tw-gradient-to-position)}.to-blue-200\/20{--tw-gradient-to:rgba(191,219,254,.2) var(--tw-gradient-to-position)}.to-blue-200\/25{--tw-gradient-to:rgba(191,219,254,.25) var(--tw-gradient-to-position)}.to-blue-200\/30{--tw-gradient-to:rgba(191,219,254,.3) var(--tw-gradient-to-position)}.to-blue-200\/35{--tw-gradient-to:rgba(191,219,254,.35) var(--tw-gradient-to-position)}.to-blue-200\/40{--tw-gradient-to:rgba(191,219,254,.4) var(--tw-gradient-to-position)}.to-blue-200\/45{--tw-gradient-to:rgba(191,219,254,.45) var(--tw-gradient-to-position)}.to-blue-200\/5{--tw-gradient-to:rgba(191,219,254,.05) var(--tw-gradient-to-position)}.to-blue-200\/50{--tw-gradient-to:rgba(191,219,254,.5) var(--tw-gradient-to-position)}.to-blue-200\/55{--tw-gradient-to:rgba(191,219,254,.55) var(--tw-gradient-to-position)}.to-blue-200\/60{--tw-gradient-to:rgba(191,219,254,.6) var(--tw-gradient-to-position)}.to-blue-200\/65{--tw-gradient-to:rgba(191,219,254,.65) var(--tw-gradient-to-position)}.to-blue-200\/70{--tw-gradient-to:rgba(191,219,254,.7) var(--tw-gradient-to-position)}.to-blue-200\/75{--tw-gradient-to:rgba(191,219,254,.75) var(--tw-gradient-to-position)}.to-blue-200\/80{--tw-gradient-to:rgba(191,219,254,.8) var(--tw-gradient-to-position)}.to-blue-200\/85{--tw-gradient-to:rgba(191,219,254,.85) var(--tw-gradient-to-position)}.to-blue-200\/90{--tw-gradient-to:rgba(191,219,254,.9) var(--tw-gradient-to-position)}.to-blue-200\/95{--tw-gradient-to:rgba(191,219,254,.95) var(--tw-gradient-to-position)}.to-blue-50{--tw-gradient-to:#eff6ff var(--tw-gradient-to-position)}.to-blue-50\/0{--tw-gradient-to:rgba(239,246,255,0) var(--tw-gradient-to-position)}.to-blue-50\/10{--tw-gradient-to:rgba(239,246,255,.1) var(--tw-gradient-to-position)}.to-blue-50\/100{--tw-gradient-to:#eff6ff var(--tw-gradient-to-position)}.to-blue-50\/15{--tw-gradient-to:rgba(239,246,255,.15) var(--tw-gradient-to-position)}.to-blue-50\/20{--tw-gradient-to:rgba(239,246,255,.2) var(--tw-gradient-to-position)}.to-blue-50\/25{--tw-gradient-to:rgba(239,246,255,.25) var(--tw-gradient-to-position)}.to-blue-50\/30{--tw-gradient-to:rgba(239,246,255,.3) var(--tw-gradient-to-position)}.to-blue-50\/35{--tw-gradient-to:rgba(239,246,255,.35) var(--tw-gradient-to-position)}.to-blue-50\/40{--tw-gradient-to:rgba(239,246,255,.4) var(--tw-gradient-to-position)}.to-blue-50\/45{--tw-gradient-to:rgba(239,246,255,.45) var(--tw-gradient-to-position)}.to-blue-50\/5{--tw-gradient-to:rgba(239,246,255,.05) var(--tw-gradient-to-position)}.to-blue-50\/50{--tw-gradient-to:rgba(239,246,255,.5) var(--tw-gradient-to-position)}.to-blue-50\/55{--tw-gradient-to:rgba(239,246,255,.55) var(--tw-gradient-to-position)}.to-blue-50\/60{--tw-gradient-to:rgba(239,246,255,.6) var(--tw-gradient-to-position)}.to-blue-50\/65{--tw-gradient-to:rgba(239,246,255,.65) var(--tw-gradient-to-position)}.to-blue-50\/70{--tw-gradient-to:rgba(239,246,255,.7) var(--tw-gradient-to-position)}.to-blue-50\/75{--tw-gradient-to:rgba(239,246,255,.75) var(--tw-gradient-to-position)}.to-blue-50\/80{--tw-gradient-to:rgba(239,246,255,.8) var(--tw-gradient-to-position)}.to-blue-50\/85{--tw-gradient-to:rgba(239,246,255,.85) var(--tw-gradient-to-position)}.to-blue-50\/90{--tw-gradient-to:rgba(239,246,255,.9) var(--tw-gradient-to-position)}.to-blue-50\/95{--tw-gradient-to:rgba(239,246,255,.95) var(--tw-gradient-to-position)}.to-blue-500{--tw-gradient-to:#3b82f6 var(--tw-gradient-to-position)}.to-blue-500\/0{--tw-gradient-to:rgba(59,130,246,0) var(--tw-gradient-to-position)}.to-blue-500\/10{--tw-gradient-to:rgba(59,130,246,.1) var(--tw-gradient-to-position)}.to-blue-500\/100{--tw-gradient-to:#3b82f6 var(--tw-gradient-to-position)}.to-blue-500\/15{--tw-gradient-to:rgba(59,130,246,.15) var(--tw-gradient-to-position)}.to-blue-500\/20{--tw-gradient-to:rgba(59,130,246,.2) var(--tw-gradient-to-position)}.to-blue-500\/25{--tw-gradient-to:rgba(59,130,246,.25) var(--tw-gradient-to-position)}.to-blue-500\/30{--tw-gradient-to:rgba(59,130,246,.3) var(--tw-gradient-to-position)}.to-blue-500\/35{--tw-gradient-to:rgba(59,130,246,.35) var(--tw-gradient-to-position)}.to-blue-500\/40{--tw-gradient-to:rgba(59,130,246,.4) var(--tw-gradient-to-position)}.to-blue-500\/45{--tw-gradient-to:rgba(59,130,246,.45) var(--tw-gradient-to-position)}.to-blue-500\/5{--tw-gradient-to:rgba(59,130,246,.05) var(--tw-gradient-to-position)}.to-blue-500\/50{--tw-gradient-to:rgba(59,130,246,.5) var(--tw-gradient-to-position)}.to-blue-500\/55{--tw-gradient-to:rgba(59,130,246,.55) var(--tw-gradient-to-position)}.to-blue-500\/60{--tw-gradient-to:rgba(59,130,246,.6) var(--tw-gradient-to-position)}.to-blue-500\/65{--tw-gradient-to:rgba(59,130,246,.65) var(--tw-gradient-to-position)}.to-blue-500\/70{--tw-gradient-to:rgba(59,130,246,.7) var(--tw-gradient-to-position)}.to-blue-500\/75{--tw-gradient-to:rgba(59,130,246,.75) var(--tw-gradient-to-position)}.to-blue-500\/80{--tw-gradient-to:rgba(59,130,246,.8) var(--tw-gradient-to-position)}.to-blue-500\/85{--tw-gradient-to:rgba(59,130,246,.85) var(--tw-gradient-to-position)}.to-blue-500\/90{--tw-gradient-to:rgba(59,130,246,.9) var(--tw-gradient-to-position)}.to-blue-500\/95{--tw-gradient-to:rgba(59,130,246,.95) var(--tw-gradient-to-position)}.to-blue-950{--tw-gradient-to:#172554 var(--tw-gradient-to-position)}.to-blue-950\/0{--tw-gradient-to:rgba(23,37,84,0) var(--tw-gradient-to-position)}.to-blue-950\/10{--tw-gradient-to:rgba(23,37,84,.1) var(--tw-gradient-to-position)}.to-blue-950\/100{--tw-gradient-to:#172554 var(--tw-gradient-to-position)}.to-blue-950\/15{--tw-gradient-to:rgba(23,37,84,.15) var(--tw-gradient-to-position)}.to-blue-950\/20{--tw-gradient-to:rgba(23,37,84,.2) var(--tw-gradient-to-position)}.to-blue-950\/25{--tw-gradient-to:rgba(23,37,84,.25) var(--tw-gradient-to-position)}.to-blue-950\/30{--tw-gradient-to:rgba(23,37,84,.3) var(--tw-gradient-to-position)}.to-blue-950\/35{--tw-gradient-to:rgba(23,37,84,.35) var(--tw-gradient-to-position)}.to-blue-950\/40{--tw-gradient-to:rgba(23,37,84,.4) var(--tw-gradient-to-position)}.to-blue-950\/45{--tw-gradient-to:rgba(23,37,84,.45) var(--tw-gradient-to-position)}.to-blue-950\/5{--tw-gradient-to:rgba(23,37,84,.05) var(--tw-gradient-to-position)}.to-blue-950\/50{--tw-gradient-to:rgba(23,37,84,.5) var(--tw-gradient-to-position)}.to-blue-950\/55{--tw-gradient-to:rgba(23,37,84,.55) var(--tw-gradient-to-position)}.to-blue-950\/60{--tw-gradient-to:rgba(23,37,84,.6) var(--tw-gradient-to-position)}.to-blue-950\/65{--tw-gradient-to:rgba(23,37,84,.65) var(--tw-gradient-to-position)}.to-blue-950\/70{--tw-gradient-to:rgba(23,37,84,.7) var(--tw-gradient-to-position)}.to-blue-950\/75{--tw-gradient-to:rgba(23,37,84,.75) var(--tw-gradient-to-position)}.to-blue-950\/80{--tw-gradient-to:rgba(23,37,84,.8) var(--tw-gradient-to-position)}.to-blue-950\/85{--tw-gradient-to:rgba(23,37,84,.85) var(--tw-gradient-to-position)}.to-blue-950\/90{--tw-gradient-to:rgba(23,37,84,.9) var(--tw-gradient-to-position)}.to-blue-950\/95{--tw-gradient-to:rgba(23,37,84,.95) var(--tw-gradient-to-position)}.to-cyan-100{--tw-gradient-to:#cffafe var(--tw-gradient-to-position)}.to-cyan-100\/0{--tw-gradient-to:rgba(207,250,254,0) var(--tw-gradient-to-position)}.to-cyan-100\/10{--tw-gradient-to:rgba(207,250,254,.1) var(--tw-gradient-to-position)}.to-cyan-100\/100{--tw-gradient-to:#cffafe var(--tw-gradient-to-position)}.to-cyan-100\/15{--tw-gradient-to:rgba(207,250,254,.15) var(--tw-gradient-to-position)}.to-cyan-100\/20{--tw-gradient-to:rgba(207,250,254,.2) var(--tw-gradient-to-position)}.to-cyan-100\/25{--tw-gradient-to:rgba(207,250,254,.25) var(--tw-gradient-to-position)}.to-cyan-100\/30{--tw-gradient-to:rgba(207,250,254,.3) var(--tw-gradient-to-position)}.to-cyan-100\/35{--tw-gradient-to:rgba(207,250,254,.35) var(--tw-gradient-to-position)}.to-cyan-100\/40{--tw-gradient-to:rgba(207,250,254,.4) var(--tw-gradient-to-position)}.to-cyan-100\/45{--tw-gradient-to:rgba(207,250,254,.45) var(--tw-gradient-to-position)}.to-cyan-100\/5{--tw-gradient-to:rgba(207,250,254,.05) var(--tw-gradient-to-position)}.to-cyan-100\/50{--tw-gradient-to:rgba(207,250,254,.5) var(--tw-gradient-to-position)}.to-cyan-100\/55{--tw-gradient-to:rgba(207,250,254,.55) var(--tw-gradient-to-position)}.to-cyan-100\/60{--tw-gradient-to:rgba(207,250,254,.6) var(--tw-gradient-to-position)}.to-cyan-100\/65{--tw-gradient-to:rgba(207,250,254,.65) var(--tw-gradient-to-position)}.to-cyan-100\/70{--tw-gradient-to:rgba(207,250,254,.7) var(--tw-gradient-to-position)}.to-cyan-100\/75{--tw-gradient-to:rgba(207,250,254,.75) var(--tw-gradient-to-position)}.to-cyan-100\/80{--tw-gradient-to:rgba(207,250,254,.8) var(--tw-gradient-to-position)}.to-cyan-100\/85{--tw-gradient-to:rgba(207,250,254,.85) var(--tw-gradient-to-position)}.to-cyan-100\/90{--tw-gradient-to:rgba(207,250,254,.9) var(--tw-gradient-to-position)}.to-cyan-100\/95{--tw-gradient-to:rgba(207,250,254,.95) var(--tw-gradient-to-position)}.to-cyan-200{--tw-gradient-to:#a5f3fc var(--tw-gradient-to-position)}.to-cyan-200\/0{--tw-gradient-to:rgba(165,243,252,0) var(--tw-gradient-to-position)}.to-cyan-200\/10{--tw-gradient-to:rgba(165,243,252,.1) var(--tw-gradient-to-position)}.to-cyan-200\/100{--tw-gradient-to:#a5f3fc var(--tw-gradient-to-position)}.to-cyan-200\/15{--tw-gradient-to:rgba(165,243,252,.15) var(--tw-gradient-to-position)}.to-cyan-200\/20{--tw-gradient-to:rgba(165,243,252,.2) var(--tw-gradient-to-position)}.to-cyan-200\/25{--tw-gradient-to:rgba(165,243,252,.25) var(--tw-gradient-to-position)}.to-cyan-200\/30{--tw-gradient-to:rgba(165,243,252,.3) var(--tw-gradient-to-position)}.to-cyan-200\/35{--tw-gradient-to:rgba(165,243,252,.35) var(--tw-gradient-to-position)}.to-cyan-200\/40{--tw-gradient-to:rgba(165,243,252,.4) var(--tw-gradient-to-position)}.to-cyan-200\/45{--tw-gradient-to:rgba(165,243,252,.45) var(--tw-gradient-to-position)}.to-cyan-200\/5{--tw-gradient-to:rgba(165,243,252,.05) var(--tw-gradient-to-position)}.to-cyan-200\/50{--tw-gradient-to:rgba(165,243,252,.5) var(--tw-gradient-to-position)}.to-cyan-200\/55{--tw-gradient-to:rgba(165,243,252,.55) var(--tw-gradient-to-position)}.to-cyan-200\/60{--tw-gradient-to:rgba(165,243,252,.6) var(--tw-gradient-to-position)}.to-cyan-200\/65{--tw-gradient-to:rgba(165,243,252,.65) var(--tw-gradient-to-position)}.to-cyan-200\/70{--tw-gradient-to:rgba(165,243,252,.7) var(--tw-gradient-to-position)}.to-cyan-200\/75{--tw-gradient-to:rgba(165,243,252,.75) var(--tw-gradient-to-position)}.to-cyan-200\/80{--tw-gradient-to:rgba(165,243,252,.8) var(--tw-gradient-to-position)}.to-cyan-200\/85{--tw-gradient-to:rgba(165,243,252,.85) var(--tw-gradient-to-position)}.to-cyan-200\/90{--tw-gradient-to:rgba(165,243,252,.9) var(--tw-gradient-to-position)}.to-cyan-200\/95{--tw-gradient-to:rgba(165,243,252,.95) var(--tw-gradient-to-position)}.to-cyan-50{--tw-gradient-to:#ecfeff var(--tw-gradient-to-position)}.to-cyan-50\/0{--tw-gradient-to:rgba(236,254,255,0) var(--tw-gradient-to-position)}.to-cyan-50\/10{--tw-gradient-to:rgba(236,254,255,.1) var(--tw-gradient-to-position)}.to-cyan-50\/100{--tw-gradient-to:#ecfeff var(--tw-gradient-to-position)}.to-cyan-50\/15{--tw-gradient-to:rgba(236,254,255,.15) var(--tw-gradient-to-position)}.to-cyan-50\/20{--tw-gradient-to:rgba(236,254,255,.2) var(--tw-gradient-to-position)}.to-cyan-50\/25{--tw-gradient-to:rgba(236,254,255,.25) var(--tw-gradient-to-position)}.to-cyan-50\/30{--tw-gradient-to:rgba(236,254,255,.3) var(--tw-gradient-to-position)}.to-cyan-50\/35{--tw-gradient-to:rgba(236,254,255,.35) var(--tw-gradient-to-position)}.to-cyan-50\/40{--tw-gradient-to:rgba(236,254,255,.4) var(--tw-gradient-to-position)}.to-cyan-50\/45{--tw-gradient-to:rgba(236,254,255,.45) var(--tw-gradient-to-position)}.to-cyan-50\/5{--tw-gradient-to:rgba(236,254,255,.05) var(--tw-gradient-to-position)}.to-cyan-50\/50{--tw-gradient-to:rgba(236,254,255,.5) var(--tw-gradient-to-position)}.to-cyan-50\/55{--tw-gradient-to:rgba(236,254,255,.55) var(--tw-gradient-to-position)}.to-cyan-50\/60{--tw-gradient-to:rgba(236,254,255,.6) var(--tw-gradient-to-position)}.to-cyan-50\/65{--tw-gradient-to:rgba(236,254,255,.65) var(--tw-gradient-to-position)}.to-cyan-50\/70{--tw-gradient-to:rgba(236,254,255,.7) var(--tw-gradient-to-position)}.to-cyan-50\/75{--tw-gradient-to:rgba(236,254,255,.75) var(--tw-gradient-to-position)}.to-cyan-50\/80{--tw-gradient-to:rgba(236,254,255,.8) var(--tw-gradient-to-position)}.to-cyan-50\/85{--tw-gradient-to:rgba(236,254,255,.85) var(--tw-gradient-to-position)}.to-cyan-50\/90{--tw-gradient-to:rgba(236,254,255,.9) var(--tw-gradient-to-position)}.to-cyan-50\/95{--tw-gradient-to:rgba(236,254,255,.95) var(--tw-gradient-to-position)}.to-cyan-500{--tw-gradient-to:#06b6d4 var(--tw-gradient-to-position)}.to-cyan-500\/0{--tw-gradient-to:rgba(6,182,212,0) var(--tw-gradient-to-position)}.to-cyan-500\/10{--tw-gradient-to:rgba(6,182,212,.1) var(--tw-gradient-to-position)}.to-cyan-500\/100{--tw-gradient-to:#06b6d4 var(--tw-gradient-to-position)}.to-cyan-500\/15{--tw-gradient-to:rgba(6,182,212,.15) var(--tw-gradient-to-position)}.to-cyan-500\/20{--tw-gradient-to:rgba(6,182,212,.2) var(--tw-gradient-to-position)}.to-cyan-500\/25{--tw-gradient-to:rgba(6,182,212,.25) var(--tw-gradient-to-position)}.to-cyan-500\/30{--tw-gradient-to:rgba(6,182,212,.3) var(--tw-gradient-to-position)}.to-cyan-500\/35{--tw-gradient-to:rgba(6,182,212,.35) var(--tw-gradient-to-position)}.to-cyan-500\/40{--tw-gradient-to:rgba(6,182,212,.4) var(--tw-gradient-to-position)}.to-cyan-500\/45{--tw-gradient-to:rgba(6,182,212,.45) var(--tw-gradient-to-position)}.to-cyan-500\/5{--tw-gradient-to:rgba(6,182,212,.05) var(--tw-gradient-to-position)}.to-cyan-500\/50{--tw-gradient-to:rgba(6,182,212,.5) var(--tw-gradient-to-position)}.to-cyan-500\/55{--tw-gradient-to:rgba(6,182,212,.55) var(--tw-gradient-to-position)}.to-cyan-500\/60{--tw-gradient-to:rgba(6,182,212,.6) var(--tw-gradient-to-position)}.to-cyan-500\/65{--tw-gradient-to:rgba(6,182,212,.65) var(--tw-gradient-to-position)}.to-cyan-500\/70{--tw-gradient-to:rgba(6,182,212,.7) var(--tw-gradient-to-position)}.to-cyan-500\/75{--tw-gradient-to:rgba(6,182,212,.75) var(--tw-gradient-to-position)}.to-cyan-500\/80{--tw-gradient-to:rgba(6,182,212,.8) var(--tw-gradient-to-position)}.to-cyan-500\/85{--tw-gradient-to:rgba(6,182,212,.85) var(--tw-gradient-to-position)}.to-cyan-500\/90{--tw-gradient-to:rgba(6,182,212,.9) var(--tw-gradient-to-position)}.to-cyan-500\/95{--tw-gradient-to:rgba(6,182,212,.95) var(--tw-gradient-to-position)}.to-cyan-950{--tw-gradient-to:#083344 var(--tw-gradient-to-position)}.to-cyan-950\/0{--tw-gradient-to:rgba(8,51,68,0) var(--tw-gradient-to-position)}.to-cyan-950\/10{--tw-gradient-to:rgba(8,51,68,.1) var(--tw-gradient-to-position)}.to-cyan-950\/100{--tw-gradient-to:#083344 var(--tw-gradient-to-position)}.to-cyan-950\/15{--tw-gradient-to:rgba(8,51,68,.15) var(--tw-gradient-to-position)}.to-cyan-950\/20{--tw-gradient-to:rgba(8,51,68,.2) var(--tw-gradient-to-position)}.to-cyan-950\/25{--tw-gradient-to:rgba(8,51,68,.25) var(--tw-gradient-to-position)}.to-cyan-950\/30{--tw-gradient-to:rgba(8,51,68,.3) var(--tw-gradient-to-position)}.to-cyan-950\/35{--tw-gradient-to:rgba(8,51,68,.35) var(--tw-gradient-to-position)}.to-cyan-950\/40{--tw-gradient-to:rgba(8,51,68,.4) var(--tw-gradient-to-position)}.to-cyan-950\/45{--tw-gradient-to:rgba(8,51,68,.45) var(--tw-gradient-to-position)}.to-cyan-950\/5{--tw-gradient-to:rgba(8,51,68,.05) var(--tw-gradient-to-position)}.to-cyan-950\/50{--tw-gradient-to:rgba(8,51,68,.5) var(--tw-gradient-to-position)}.to-cyan-950\/55{--tw-gradient-to:rgba(8,51,68,.55) var(--tw-gradient-to-position)}.to-cyan-950\/60{--tw-gradient-to:rgba(8,51,68,.6) var(--tw-gradient-to-position)}.to-cyan-950\/65{--tw-gradient-to:rgba(8,51,68,.65) var(--tw-gradient-to-position)}.to-cyan-950\/70{--tw-gradient-to:rgba(8,51,68,.7) var(--tw-gradient-to-position)}.to-cyan-950\/75{--tw-gradient-to:rgba(8,51,68,.75) var(--tw-gradient-to-position)}.to-cyan-950\/80{--tw-gradient-to:rgba(8,51,68,.8) var(--tw-gradient-to-position)}.to-cyan-950\/85{--tw-gradient-to:rgba(8,51,68,.85) var(--tw-gradient-to-position)}.to-cyan-950\/90{--tw-gradient-to:rgba(8,51,68,.9) var(--tw-gradient-to-position)}.to-cyan-950\/95{--tw-gradient-to:rgba(8,51,68,.95) var(--tw-gradient-to-position)}.to-emerald-100{--tw-gradient-to:#d1fae5 var(--tw-gradient-to-position)}.to-emerald-100\/0{--tw-gradient-to:rgba(209,250,229,0) var(--tw-gradient-to-position)}.to-emerald-100\/10{--tw-gradient-to:rgba(209,250,229,.1) var(--tw-gradient-to-position)}.to-emerald-100\/100{--tw-gradient-to:#d1fae5 var(--tw-gradient-to-position)}.to-emerald-100\/15{--tw-gradient-to:rgba(209,250,229,.15) var(--tw-gradient-to-position)}.to-emerald-100\/20{--tw-gradient-to:rgba(209,250,229,.2) var(--tw-gradient-to-position)}.to-emerald-100\/25{--tw-gradient-to:rgba(209,250,229,.25) var(--tw-gradient-to-position)}.to-emerald-100\/30{--tw-gradient-to:rgba(209,250,229,.3) var(--tw-gradient-to-position)}.to-emerald-100\/35{--tw-gradient-to:rgba(209,250,229,.35) var(--tw-gradient-to-position)}.to-emerald-100\/40{--tw-gradient-to:rgba(209,250,229,.4) var(--tw-gradient-to-position)}.to-emerald-100\/45{--tw-gradient-to:rgba(209,250,229,.45) var(--tw-gradient-to-position)}.to-emerald-100\/5{--tw-gradient-to:rgba(209,250,229,.05) var(--tw-gradient-to-position)}.to-emerald-100\/50{--tw-gradient-to:rgba(209,250,229,.5) var(--tw-gradient-to-position)}.to-emerald-100\/55{--tw-gradient-to:rgba(209,250,229,.55) var(--tw-gradient-to-position)}.to-emerald-100\/60{--tw-gradient-to:rgba(209,250,229,.6) var(--tw-gradient-to-position)}.to-emerald-100\/65{--tw-gradient-to:rgba(209,250,229,.65) var(--tw-gradient-to-position)}.to-emerald-100\/70{--tw-gradient-to:rgba(209,250,229,.7) var(--tw-gradient-to-position)}.to-emerald-100\/75{--tw-gradient-to:rgba(209,250,229,.75) var(--tw-gradient-to-position)}.to-emerald-100\/80{--tw-gradient-to:rgba(209,250,229,.8) var(--tw-gradient-to-position)}.to-emerald-100\/85{--tw-gradient-to:rgba(209,250,229,.85) var(--tw-gradient-to-position)}.to-emerald-100\/90{--tw-gradient-to:rgba(209,250,229,.9) var(--tw-gradient-to-position)}.to-emerald-100\/95{--tw-gradient-to:rgba(209,250,229,.95) var(--tw-gradient-to-position)}.to-emerald-200{--tw-gradient-to:#a7f3d0 var(--tw-gradient-to-position)}.to-emerald-200\/0{--tw-gradient-to:rgba(167,243,208,0) var(--tw-gradient-to-position)}.to-emerald-200\/10{--tw-gradient-to:rgba(167,243,208,.1) var(--tw-gradient-to-position)}.to-emerald-200\/100{--tw-gradient-to:#a7f3d0 var(--tw-gradient-to-position)}.to-emerald-200\/15{--tw-gradient-to:rgba(167,243,208,.15) var(--tw-gradient-to-position)}.to-emerald-200\/20{--tw-gradient-to:rgba(167,243,208,.2) var(--tw-gradient-to-position)}.to-emerald-200\/25{--tw-gradient-to:rgba(167,243,208,.25) var(--tw-gradient-to-position)}.to-emerald-200\/30{--tw-gradient-to:rgba(167,243,208,.3) var(--tw-gradient-to-position)}.to-emerald-200\/35{--tw-gradient-to:rgba(167,243,208,.35) var(--tw-gradient-to-position)}.to-emerald-200\/40{--tw-gradient-to:rgba(167,243,208,.4) var(--tw-gradient-to-position)}.to-emerald-200\/45{--tw-gradient-to:rgba(167,243,208,.45) var(--tw-gradient-to-position)}.to-emerald-200\/5{--tw-gradient-to:rgba(167,243,208,.05) var(--tw-gradient-to-position)}.to-emerald-200\/50{--tw-gradient-to:rgba(167,243,208,.5) var(--tw-gradient-to-position)}.to-emerald-200\/55{--tw-gradient-to:rgba(167,243,208,.55) var(--tw-gradient-to-position)}.to-emerald-200\/60{--tw-gradient-to:rgba(167,243,208,.6) var(--tw-gradient-to-position)}.to-emerald-200\/65{--tw-gradient-to:rgba(167,243,208,.65) var(--tw-gradient-to-position)}.to-emerald-200\/70{--tw-gradient-to:rgba(167,243,208,.7) var(--tw-gradient-to-position)}.to-emerald-200\/75{--tw-gradient-to:rgba(167,243,208,.75) var(--tw-gradient-to-position)}.to-emerald-200\/80{--tw-gradient-to:rgba(167,243,208,.8) var(--tw-gradient-to-position)}.to-emerald-200\/85{--tw-gradient-to:rgba(167,243,208,.85) var(--tw-gradient-to-position)}.to-emerald-200\/90{--tw-gradient-to:rgba(167,243,208,.9) var(--tw-gradient-to-position)}.to-emerald-200\/95{--tw-gradient-to:rgba(167,243,208,.95) var(--tw-gradient-to-position)}.to-emerald-50{--tw-gradient-to:#ecfdf5 var(--tw-gradient-to-position)}.to-emerald-50\/0{--tw-gradient-to:rgba(236,253,245,0) var(--tw-gradient-to-position)}.to-emerald-50\/10{--tw-gradient-to:rgba(236,253,245,.1) var(--tw-gradient-to-position)}.to-emerald-50\/100{--tw-gradient-to:#ecfdf5 var(--tw-gradient-to-position)}.to-emerald-50\/15{--tw-gradient-to:rgba(236,253,245,.15) var(--tw-gradient-to-position)}.to-emerald-50\/20{--tw-gradient-to:rgba(236,253,245,.2) var(--tw-gradient-to-position)}.to-emerald-50\/25{--tw-gradient-to:rgba(236,253,245,.25) var(--tw-gradient-to-position)}.to-emerald-50\/30{--tw-gradient-to:rgba(236,253,245,.3) var(--tw-gradient-to-position)}.to-emerald-50\/35{--tw-gradient-to:rgba(236,253,245,.35) var(--tw-gradient-to-position)}.to-emerald-50\/40{--tw-gradient-to:rgba(236,253,245,.4) var(--tw-gradient-to-position)}.to-emerald-50\/45{--tw-gradient-to:rgba(236,253,245,.45) var(--tw-gradient-to-position)}.to-emerald-50\/5{--tw-gradient-to:rgba(236,253,245,.05) var(--tw-gradient-to-position)}.to-emerald-50\/50{--tw-gradient-to:rgba(236,253,245,.5) var(--tw-gradient-to-position)}.to-emerald-50\/55{--tw-gradient-to:rgba(236,253,245,.55) var(--tw-gradient-to-position)}.to-emerald-50\/60{--tw-gradient-to:rgba(236,253,245,.6) var(--tw-gradient-to-position)}.to-emerald-50\/65{--tw-gradient-to:rgba(236,253,245,.65) var(--tw-gradient-to-position)}.to-emerald-50\/70{--tw-gradient-to:rgba(236,253,245,.7) var(--tw-gradient-to-position)}.to-emerald-50\/75{--tw-gradient-to:rgba(236,253,245,.75) var(--tw-gradient-to-position)}.to-emerald-50\/80{--tw-gradient-to:rgba(236,253,245,.8) var(--tw-gradient-to-position)}.to-emerald-50\/85{--tw-gradient-to:rgba(236,253,245,.85) var(--tw-gradient-to-position)}.to-emerald-50\/90{--tw-gradient-to:rgba(236,253,245,.9) var(--tw-gradient-to-position)}.to-emerald-50\/95{--tw-gradient-to:rgba(236,253,245,.95) var(--tw-gradient-to-position)}.to-emerald-500{--tw-gradient-to:#10b981 var(--tw-gradient-to-position)}.to-emerald-500\/0{--tw-gradient-to:rgba(16,185,129,0) var(--tw-gradient-to-position)}.to-emerald-500\/10{--tw-gradient-to:rgba(16,185,129,.1) var(--tw-gradient-to-position)}.to-emerald-500\/100{--tw-gradient-to:#10b981 var(--tw-gradient-to-position)}.to-emerald-500\/15{--tw-gradient-to:rgba(16,185,129,.15) var(--tw-gradient-to-position)}.to-emerald-500\/20{--tw-gradient-to:rgba(16,185,129,.2) var(--tw-gradient-to-position)}.to-emerald-500\/25{--tw-gradient-to:rgba(16,185,129,.25) var(--tw-gradient-to-position)}.to-emerald-500\/30{--tw-gradient-to:rgba(16,185,129,.3) var(--tw-gradient-to-position)}.to-emerald-500\/35{--tw-gradient-to:rgba(16,185,129,.35) var(--tw-gradient-to-position)}.to-emerald-500\/40{--tw-gradient-to:rgba(16,185,129,.4) var(--tw-gradient-to-position)}.to-emerald-500\/45{--tw-gradient-to:rgba(16,185,129,.45) var(--tw-gradient-to-position)}.to-emerald-500\/5{--tw-gradient-to:rgba(16,185,129,.05) var(--tw-gradient-to-position)}.to-emerald-500\/50{--tw-gradient-to:rgba(16,185,129,.5) var(--tw-gradient-to-position)}.to-emerald-500\/55{--tw-gradient-to:rgba(16,185,129,.55) var(--tw-gradient-to-position)}.to-emerald-500\/60{--tw-gradient-to:rgba(16,185,129,.6) var(--tw-gradient-to-position)}.to-emerald-500\/65{--tw-gradient-to:rgba(16,185,129,.65) var(--tw-gradient-to-position)}.to-emerald-500\/70{--tw-gradient-to:rgba(16,185,129,.7) var(--tw-gradient-to-position)}.to-emerald-500\/75{--tw-gradient-to:rgba(16,185,129,.75) var(--tw-gradient-to-position)}.to-emerald-500\/80{--tw-gradient-to:rgba(16,185,129,.8) var(--tw-gradient-to-position)}.to-emerald-500\/85{--tw-gradient-to:rgba(16,185,129,.85) var(--tw-gradient-to-position)}.to-emerald-500\/90{--tw-gradient-to:rgba(16,185,129,.9) var(--tw-gradient-to-position)}.to-emerald-500\/95{--tw-gradient-to:rgba(16,185,129,.95) var(--tw-gradient-to-position)}.to-emerald-950{--tw-gradient-to:#022c22 var(--tw-gradient-to-position)}.to-emerald-950\/0{--tw-gradient-to:rgba(2,44,34,0) var(--tw-gradient-to-position)}.to-emerald-950\/10{--tw-gradient-to:rgba(2,44,34,.1) var(--tw-gradient-to-position)}.to-emerald-950\/100{--tw-gradient-to:#022c22 var(--tw-gradient-to-position)}.to-emerald-950\/15{--tw-gradient-to:rgba(2,44,34,.15) var(--tw-gradient-to-position)}.to-emerald-950\/20{--tw-gradient-to:rgba(2,44,34,.2) var(--tw-gradient-to-position)}.to-emerald-950\/25{--tw-gradient-to:rgba(2,44,34,.25) var(--tw-gradient-to-position)}.to-emerald-950\/30{--tw-gradient-to:rgba(2,44,34,.3) var(--tw-gradient-to-position)}.to-emerald-950\/35{--tw-gradient-to:rgba(2,44,34,.35) var(--tw-gradient-to-position)}.to-emerald-950\/40{--tw-gradient-to:rgba(2,44,34,.4) var(--tw-gradient-to-position)}.to-emerald-950\/45{--tw-gradient-to:rgba(2,44,34,.45) var(--tw-gradient-to-position)}.to-emerald-950\/5{--tw-gradient-to:rgba(2,44,34,.05) var(--tw-gradient-to-position)}.to-emerald-950\/50{--tw-gradient-to:rgba(2,44,34,.5) var(--tw-gradient-to-position)}.to-emerald-950\/55{--tw-gradient-to:rgba(2,44,34,.55) var(--tw-gradient-to-position)}.to-emerald-950\/60{--tw-gradient-to:rgba(2,44,34,.6) var(--tw-gradient-to-position)}.to-emerald-950\/65{--tw-gradient-to:rgba(2,44,34,.65) var(--tw-gradient-to-position)}.to-emerald-950\/70{--tw-gradient-to:rgba(2,44,34,.7) var(--tw-gradient-to-position)}.to-emerald-950\/75{--tw-gradient-to:rgba(2,44,34,.75) var(--tw-gradient-to-position)}.to-emerald-950\/80{--tw-gradient-to:rgba(2,44,34,.8) var(--tw-gradient-to-position)}.to-emerald-950\/85{--tw-gradient-to:rgba(2,44,34,.85) var(--tw-gradient-to-position)}.to-emerald-950\/90{--tw-gradient-to:rgba(2,44,34,.9) var(--tw-gradient-to-position)}.to-emerald-950\/95{--tw-gradient-to:rgba(2,44,34,.95) var(--tw-gradient-to-position)}.to-fuchsia-100{--tw-gradient-to:#fae8ff var(--tw-gradient-to-position)}.to-fuchsia-100\/0{--tw-gradient-to:rgba(250,232,255,0) var(--tw-gradient-to-position)}.to-fuchsia-100\/10{--tw-gradient-to:rgba(250,232,255,.1) var(--tw-gradient-to-position)}.to-fuchsia-100\/100{--tw-gradient-to:#fae8ff var(--tw-gradient-to-position)}.to-fuchsia-100\/15{--tw-gradient-to:rgba(250,232,255,.15) var(--tw-gradient-to-position)}.to-fuchsia-100\/20{--tw-gradient-to:rgba(250,232,255,.2) var(--tw-gradient-to-position)}.to-fuchsia-100\/25{--tw-gradient-to:rgba(250,232,255,.25) var(--tw-gradient-to-position)}.to-fuchsia-100\/30{--tw-gradient-to:rgba(250,232,255,.3) var(--tw-gradient-to-position)}.to-fuchsia-100\/35{--tw-gradient-to:rgba(250,232,255,.35) var(--tw-gradient-to-position)}.to-fuchsia-100\/40{--tw-gradient-to:rgba(250,232,255,.4) var(--tw-gradient-to-position)}.to-fuchsia-100\/45{--tw-gradient-to:rgba(250,232,255,.45) var(--tw-gradient-to-position)}.to-fuchsia-100\/5{--tw-gradient-to:rgba(250,232,255,.05) var(--tw-gradient-to-position)}.to-fuchsia-100\/50{--tw-gradient-to:rgba(250,232,255,.5) var(--tw-gradient-to-position)}.to-fuchsia-100\/55{--tw-gradient-to:rgba(250,232,255,.55) var(--tw-gradient-to-position)}.to-fuchsia-100\/60{--tw-gradient-to:rgba(250,232,255,.6) var(--tw-gradient-to-position)}.to-fuchsia-100\/65{--tw-gradient-to:rgba(250,232,255,.65) var(--tw-gradient-to-position)}.to-fuchsia-100\/70{--tw-gradient-to:rgba(250,232,255,.7) var(--tw-gradient-to-position)}.to-fuchsia-100\/75{--tw-gradient-to:rgba(250,232,255,.75) var(--tw-gradient-to-position)}.to-fuchsia-100\/80{--tw-gradient-to:rgba(250,232,255,.8) var(--tw-gradient-to-position)}.to-fuchsia-100\/85{--tw-gradient-to:rgba(250,232,255,.85) var(--tw-gradient-to-position)}.to-fuchsia-100\/90{--tw-gradient-to:rgba(250,232,255,.9) var(--tw-gradient-to-position)}.to-fuchsia-100\/95{--tw-gradient-to:rgba(250,232,255,.95) var(--tw-gradient-to-position)}.to-fuchsia-200{--tw-gradient-to:#f5d0fe var(--tw-gradient-to-position)}.to-fuchsia-200\/0{--tw-gradient-to:rgba(245,208,254,0) var(--tw-gradient-to-position)}.to-fuchsia-200\/10{--tw-gradient-to:rgba(245,208,254,.1) var(--tw-gradient-to-position)}.to-fuchsia-200\/100{--tw-gradient-to:#f5d0fe var(--tw-gradient-to-position)}.to-fuchsia-200\/15{--tw-gradient-to:rgba(245,208,254,.15) var(--tw-gradient-to-position)}.to-fuchsia-200\/20{--tw-gradient-to:rgba(245,208,254,.2) var(--tw-gradient-to-position)}.to-fuchsia-200\/25{--tw-gradient-to:rgba(245,208,254,.25) var(--tw-gradient-to-position)}.to-fuchsia-200\/30{--tw-gradient-to:rgba(245,208,254,.3) var(--tw-gradient-to-position)}.to-fuchsia-200\/35{--tw-gradient-to:rgba(245,208,254,.35) var(--tw-gradient-to-position)}.to-fuchsia-200\/40{--tw-gradient-to:rgba(245,208,254,.4) var(--tw-gradient-to-position)}.to-fuchsia-200\/45{--tw-gradient-to:rgba(245,208,254,.45) var(--tw-gradient-to-position)}.to-fuchsia-200\/5{--tw-gradient-to:rgba(245,208,254,.05) var(--tw-gradient-to-position)}.to-fuchsia-200\/50{--tw-gradient-to:rgba(245,208,254,.5) var(--tw-gradient-to-position)}.to-fuchsia-200\/55{--tw-gradient-to:rgba(245,208,254,.55) var(--tw-gradient-to-position)}.to-fuchsia-200\/60{--tw-gradient-to:rgba(245,208,254,.6) var(--tw-gradient-to-position)}.to-fuchsia-200\/65{--tw-gradient-to:rgba(245,208,254,.65) var(--tw-gradient-to-position)}.to-fuchsia-200\/70{--tw-gradient-to:rgba(245,208,254,.7) var(--tw-gradient-to-position)}.to-fuchsia-200\/75{--tw-gradient-to:rgba(245,208,254,.75) var(--tw-gradient-to-position)}.to-fuchsia-200\/80{--tw-gradient-to:rgba(245,208,254,.8) var(--tw-gradient-to-position)}.to-fuchsia-200\/85{--tw-gradient-to:rgba(245,208,254,.85) var(--tw-gradient-to-position)}.to-fuchsia-200\/90{--tw-gradient-to:rgba(245,208,254,.9) var(--tw-gradient-to-position)}.to-fuchsia-200\/95{--tw-gradient-to:rgba(245,208,254,.95) var(--tw-gradient-to-position)}.to-fuchsia-50{--tw-gradient-to:#fdf4ff var(--tw-gradient-to-position)}.to-fuchsia-50\/0{--tw-gradient-to:rgba(253,244,255,0) var(--tw-gradient-to-position)}.to-fuchsia-50\/10{--tw-gradient-to:rgba(253,244,255,.1) var(--tw-gradient-to-position)}.to-fuchsia-50\/100{--tw-gradient-to:#fdf4ff var(--tw-gradient-to-position)}.to-fuchsia-50\/15{--tw-gradient-to:rgba(253,244,255,.15) var(--tw-gradient-to-position)}.to-fuchsia-50\/20{--tw-gradient-to:rgba(253,244,255,.2) var(--tw-gradient-to-position)}.to-fuchsia-50\/25{--tw-gradient-to:rgba(253,244,255,.25) var(--tw-gradient-to-position)}.to-fuchsia-50\/30{--tw-gradient-to:rgba(253,244,255,.3) var(--tw-gradient-to-position)}.to-fuchsia-50\/35{--tw-gradient-to:rgba(253,244,255,.35) var(--tw-gradient-to-position)}.to-fuchsia-50\/40{--tw-gradient-to:rgba(253,244,255,.4) var(--tw-gradient-to-position)}.to-fuchsia-50\/45{--tw-gradient-to:rgba(253,244,255,.45) var(--tw-gradient-to-position)}.to-fuchsia-50\/5{--tw-gradient-to:rgba(253,244,255,.05) var(--tw-gradient-to-position)}.to-fuchsia-50\/50{--tw-gradient-to:rgba(253,244,255,.5) var(--tw-gradient-to-position)}.to-fuchsia-50\/55{--tw-gradient-to:rgba(253,244,255,.55) var(--tw-gradient-to-position)}.to-fuchsia-50\/60{--tw-gradient-to:rgba(253,244,255,.6) var(--tw-gradient-to-position)}.to-fuchsia-50\/65{--tw-gradient-to:rgba(253,244,255,.65) var(--tw-gradient-to-position)}.to-fuchsia-50\/70{--tw-gradient-to:rgba(253,244,255,.7) var(--tw-gradient-to-position)}.to-fuchsia-50\/75{--tw-gradient-to:rgba(253,244,255,.75) var(--tw-gradient-to-position)}.to-fuchsia-50\/80{--tw-gradient-to:rgba(253,244,255,.8) var(--tw-gradient-to-position)}.to-fuchsia-50\/85{--tw-gradient-to:rgba(253,244,255,.85) var(--tw-gradient-to-position)}.to-fuchsia-50\/90{--tw-gradient-to:rgba(253,244,255,.9) var(--tw-gradient-to-position)}.to-fuchsia-50\/95{--tw-gradient-to:rgba(253,244,255,.95) var(--tw-gradient-to-position)}.to-fuchsia-500{--tw-gradient-to:#d946ef var(--tw-gradient-to-position)}.to-fuchsia-500\/0{--tw-gradient-to:rgba(217,70,239,0) var(--tw-gradient-to-position)}.to-fuchsia-500\/10{--tw-gradient-to:rgba(217,70,239,.1) var(--tw-gradient-to-position)}.to-fuchsia-500\/100{--tw-gradient-to:#d946ef var(--tw-gradient-to-position)}.to-fuchsia-500\/15{--tw-gradient-to:rgba(217,70,239,.15) var(--tw-gradient-to-position)}.to-fuchsia-500\/20{--tw-gradient-to:rgba(217,70,239,.2) var(--tw-gradient-to-position)}.to-fuchsia-500\/25{--tw-gradient-to:rgba(217,70,239,.25) var(--tw-gradient-to-position)}.to-fuchsia-500\/30{--tw-gradient-to:rgba(217,70,239,.3) var(--tw-gradient-to-position)}.to-fuchsia-500\/35{--tw-gradient-to:rgba(217,70,239,.35) var(--tw-gradient-to-position)}.to-fuchsia-500\/40{--tw-gradient-to:rgba(217,70,239,.4) var(--tw-gradient-to-position)}.to-fuchsia-500\/45{--tw-gradient-to:rgba(217,70,239,.45) var(--tw-gradient-to-position)}.to-fuchsia-500\/5{--tw-gradient-to:rgba(217,70,239,.05) var(--tw-gradient-to-position)}.to-fuchsia-500\/50{--tw-gradient-to:rgba(217,70,239,.5) var(--tw-gradient-to-position)}.to-fuchsia-500\/55{--tw-gradient-to:rgba(217,70,239,.55) var(--tw-gradient-to-position)}.to-fuchsia-500\/60{--tw-gradient-to:rgba(217,70,239,.6) var(--tw-gradient-to-position)}.to-fuchsia-500\/65{--tw-gradient-to:rgba(217,70,239,.65) var(--tw-gradient-to-position)}.to-fuchsia-500\/70{--tw-gradient-to:rgba(217,70,239,.7) var(--tw-gradient-to-position)}.to-fuchsia-500\/75{--tw-gradient-to:rgba(217,70,239,.75) var(--tw-gradient-to-position)}.to-fuchsia-500\/80{--tw-gradient-to:rgba(217,70,239,.8) var(--tw-gradient-to-position)}.to-fuchsia-500\/85{--tw-gradient-to:rgba(217,70,239,.85) var(--tw-gradient-to-position)}.to-fuchsia-500\/90{--tw-gradient-to:rgba(217,70,239,.9) var(--tw-gradient-to-position)}.to-fuchsia-500\/95{--tw-gradient-to:rgba(217,70,239,.95) var(--tw-gradient-to-position)}.to-fuchsia-950{--tw-gradient-to:#4a044e var(--tw-gradient-to-position)}.to-fuchsia-950\/0{--tw-gradient-to:rgba(74,4,78,0) var(--tw-gradient-to-position)}.to-fuchsia-950\/10{--tw-gradient-to:rgba(74,4,78,.1) var(--tw-gradient-to-position)}.to-fuchsia-950\/100{--tw-gradient-to:#4a044e var(--tw-gradient-to-position)}.to-fuchsia-950\/15{--tw-gradient-to:rgba(74,4,78,.15) var(--tw-gradient-to-position)}.to-fuchsia-950\/20{--tw-gradient-to:rgba(74,4,78,.2) var(--tw-gradient-to-position)}.to-fuchsia-950\/25{--tw-gradient-to:rgba(74,4,78,.25) var(--tw-gradient-to-position)}.to-fuchsia-950\/30{--tw-gradient-to:rgba(74,4,78,.3) var(--tw-gradient-to-position)}.to-fuchsia-950\/35{--tw-gradient-to:rgba(74,4,78,.35) var(--tw-gradient-to-position)}.to-fuchsia-950\/40{--tw-gradient-to:rgba(74,4,78,.4) var(--tw-gradient-to-position)}.to-fuchsia-950\/45{--tw-gradient-to:rgba(74,4,78,.45) var(--tw-gradient-to-position)}.to-fuchsia-950\/5{--tw-gradient-to:rgba(74,4,78,.05) var(--tw-gradient-to-position)}.to-fuchsia-950\/50{--tw-gradient-to:rgba(74,4,78,.5) var(--tw-gradient-to-position)}.to-fuchsia-950\/55{--tw-gradient-to:rgba(74,4,78,.55) var(--tw-gradient-to-position)}.to-fuchsia-950\/60{--tw-gradient-to:rgba(74,4,78,.6) var(--tw-gradient-to-position)}.to-fuchsia-950\/65{--tw-gradient-to:rgba(74,4,78,.65) var(--tw-gradient-to-position)}.to-fuchsia-950\/70{--tw-gradient-to:rgba(74,4,78,.7) var(--tw-gradient-to-position)}.to-fuchsia-950\/75{--tw-gradient-to:rgba(74,4,78,.75) var(--tw-gradient-to-position)}.to-fuchsia-950\/80{--tw-gradient-to:rgba(74,4,78,.8) var(--tw-gradient-to-position)}.to-fuchsia-950\/85{--tw-gradient-to:rgba(74,4,78,.85) var(--tw-gradient-to-position)}.to-fuchsia-950\/90{--tw-gradient-to:rgba(74,4,78,.9) var(--tw-gradient-to-position)}.to-fuchsia-950\/95{--tw-gradient-to:rgba(74,4,78,.95) var(--tw-gradient-to-position)}.to-gray-100{--tw-gradient-to:#f3f4f6 var(--tw-gradient-to-position)}.to-gray-100\/0{--tw-gradient-to:rgba(243,244,246,0) var(--tw-gradient-to-position)}.to-gray-100\/10{--tw-gradient-to:rgba(243,244,246,.1) var(--tw-gradient-to-position)}.to-gray-100\/100{--tw-gradient-to:#f3f4f6 var(--tw-gradient-to-position)}.to-gray-100\/15{--tw-gradient-to:rgba(243,244,246,.15) var(--tw-gradient-to-position)}.to-gray-100\/20{--tw-gradient-to:rgba(243,244,246,.2) var(--tw-gradient-to-position)}.to-gray-100\/25{--tw-gradient-to:rgba(243,244,246,.25) var(--tw-gradient-to-position)}.to-gray-100\/30{--tw-gradient-to:rgba(243,244,246,.3) var(--tw-gradient-to-position)}.to-gray-100\/35{--tw-gradient-to:rgba(243,244,246,.35) var(--tw-gradient-to-position)}.to-gray-100\/40{--tw-gradient-to:rgba(243,244,246,.4) var(--tw-gradient-to-position)}.to-gray-100\/45{--tw-gradient-to:rgba(243,244,246,.45) var(--tw-gradient-to-position)}.to-gray-100\/5{--tw-gradient-to:rgba(243,244,246,.05) var(--tw-gradient-to-position)}.to-gray-100\/50{--tw-gradient-to:rgba(243,244,246,.5) var(--tw-gradient-to-position)}.to-gray-100\/55{--tw-gradient-to:rgba(243,244,246,.55) var(--tw-gradient-to-position)}.to-gray-100\/60{--tw-gradient-to:rgba(243,244,246,.6) var(--tw-gradient-to-position)}.to-gray-100\/65{--tw-gradient-to:rgba(243,244,246,.65) var(--tw-gradient-to-position)}.to-gray-100\/70{--tw-gradient-to:rgba(243,244,246,.7) var(--tw-gradient-to-position)}.to-gray-100\/75{--tw-gradient-to:rgba(243,244,246,.75) var(--tw-gradient-to-position)}.to-gray-100\/80{--tw-gradient-to:rgba(243,244,246,.8) var(--tw-gradient-to-position)}.to-gray-100\/85{--tw-gradient-to:rgba(243,244,246,.85) var(--tw-gradient-to-position)}.to-gray-100\/90{--tw-gradient-to:rgba(243,244,246,.9) var(--tw-gradient-to-position)}.to-gray-100\/95{--tw-gradient-to:rgba(243,244,246,.95) var(--tw-gradient-to-position)}.to-gray-200{--tw-gradient-to:#e5e7eb var(--tw-gradient-to-position)}.to-gray-200\/0{--tw-gradient-to:rgba(229,231,235,0) var(--tw-gradient-to-position)}.to-gray-200\/10{--tw-gradient-to:rgba(229,231,235,.1) var(--tw-gradient-to-position)}.to-gray-200\/100{--tw-gradient-to:#e5e7eb var(--tw-gradient-to-position)}.to-gray-200\/15{--tw-gradient-to:rgba(229,231,235,.15) var(--tw-gradient-to-position)}.to-gray-200\/20{--tw-gradient-to:rgba(229,231,235,.2) var(--tw-gradient-to-position)}.to-gray-200\/25{--tw-gradient-to:rgba(229,231,235,.25) var(--tw-gradient-to-position)}.to-gray-200\/30{--tw-gradient-to:rgba(229,231,235,.3) var(--tw-gradient-to-position)}.to-gray-200\/35{--tw-gradient-to:rgba(229,231,235,.35) var(--tw-gradient-to-position)}.to-gray-200\/40{--tw-gradient-to:rgba(229,231,235,.4) var(--tw-gradient-to-position)}.to-gray-200\/45{--tw-gradient-to:rgba(229,231,235,.45) var(--tw-gradient-to-position)}.to-gray-200\/5{--tw-gradient-to:rgba(229,231,235,.05) var(--tw-gradient-to-position)}.to-gray-200\/50{--tw-gradient-to:rgba(229,231,235,.5) var(--tw-gradient-to-position)}.to-gray-200\/55{--tw-gradient-to:rgba(229,231,235,.55) var(--tw-gradient-to-position)}.to-gray-200\/60{--tw-gradient-to:rgba(229,231,235,.6) var(--tw-gradient-to-position)}.to-gray-200\/65{--tw-gradient-to:rgba(229,231,235,.65) var(--tw-gradient-to-position)}.to-gray-200\/70{--tw-gradient-to:rgba(229,231,235,.7) var(--tw-gradient-to-position)}.to-gray-200\/75{--tw-gradient-to:rgba(229,231,235,.75) var(--tw-gradient-to-position)}.to-gray-200\/80{--tw-gradient-to:rgba(229,231,235,.8) var(--tw-gradient-to-position)}.to-gray-200\/85{--tw-gradient-to:rgba(229,231,235,.85) var(--tw-gradient-to-position)}.to-gray-200\/90{--tw-gradient-to:rgba(229,231,235,.9) var(--tw-gradient-to-position)}.to-gray-200\/95{--tw-gradient-to:rgba(229,231,235,.95) var(--tw-gradient-to-position)}.to-gray-50{--tw-gradient-to:#f9fafb var(--tw-gradient-to-position)}.to-gray-50\/0{--tw-gradient-to:rgba(249,250,251,0) var(--tw-gradient-to-position)}.to-gray-50\/10{--tw-gradient-to:rgba(249,250,251,.1) var(--tw-gradient-to-position)}.to-gray-50\/100{--tw-gradient-to:#f9fafb var(--tw-gradient-to-position)}.to-gray-50\/15{--tw-gradient-to:rgba(249,250,251,.15) var(--tw-gradient-to-position)}.to-gray-50\/20{--tw-gradient-to:rgba(249,250,251,.2) var(--tw-gradient-to-position)}.to-gray-50\/25{--tw-gradient-to:rgba(249,250,251,.25) var(--tw-gradient-to-position)}.to-gray-50\/30{--tw-gradient-to:rgba(249,250,251,.3) var(--tw-gradient-to-position)}.to-gray-50\/35{--tw-gradient-to:rgba(249,250,251,.35) var(--tw-gradient-to-position)}.to-gray-50\/40{--tw-gradient-to:rgba(249,250,251,.4) var(--tw-gradient-to-position)}.to-gray-50\/45{--tw-gradient-to:rgba(249,250,251,.45) var(--tw-gradient-to-position)}.to-gray-50\/5{--tw-gradient-to:rgba(249,250,251,.05) var(--tw-gradient-to-position)}.to-gray-50\/50{--tw-gradient-to:rgba(249,250,251,.5) var(--tw-gradient-to-position)}.to-gray-50\/55{--tw-gradient-to:rgba(249,250,251,.55) var(--tw-gradient-to-position)}.to-gray-50\/60{--tw-gradient-to:rgba(249,250,251,.6) var(--tw-gradient-to-position)}.to-gray-50\/65{--tw-gradient-to:rgba(249,250,251,.65) var(--tw-gradient-to-position)}.to-gray-50\/70{--tw-gradient-to:rgba(249,250,251,.7) var(--tw-gradient-to-position)}.to-gray-50\/75{--tw-gradient-to:rgba(249,250,251,.75) var(--tw-gradient-to-position)}.to-gray-50\/80{--tw-gradient-to:rgba(249,250,251,.8) var(--tw-gradient-to-position)}.to-gray-50\/85{--tw-gradient-to:rgba(249,250,251,.85) var(--tw-gradient-to-position)}.to-gray-50\/90{--tw-gradient-to:rgba(249,250,251,.9) var(--tw-gradient-to-position)}.to-gray-50\/95{--tw-gradient-to:rgba(249,250,251,.95) var(--tw-gradient-to-position)}.to-gray-500{--tw-gradient-to:#6b7280 var(--tw-gradient-to-position)}.to-gray-500\/0{--tw-gradient-to:hsla(220,9%,46%,0) var(--tw-gradient-to-position)}.to-gray-500\/10{--tw-gradient-to:hsla(220,9%,46%,.1) var(--tw-gradient-to-position)}.to-gray-500\/100{--tw-gradient-to:#6b7280 var(--tw-gradient-to-position)}.to-gray-500\/15{--tw-gradient-to:hsla(220,9%,46%,.15) var(--tw-gradient-to-position)}.to-gray-500\/20{--tw-gradient-to:hsla(220,9%,46%,.2) var(--tw-gradient-to-position)}.to-gray-500\/25{--tw-gradient-to:hsla(220,9%,46%,.25) var(--tw-gradient-to-position)}.to-gray-500\/30{--tw-gradient-to:hsla(220,9%,46%,.3) var(--tw-gradient-to-position)}.to-gray-500\/35{--tw-gradient-to:hsla(220,9%,46%,.35) var(--tw-gradient-to-position)}.to-gray-500\/40{--tw-gradient-to:hsla(220,9%,46%,.4) var(--tw-gradient-to-position)}.to-gray-500\/45{--tw-gradient-to:hsla(220,9%,46%,.45) var(--tw-gradient-to-position)}.to-gray-500\/5{--tw-gradient-to:hsla(220,9%,46%,.05) var(--tw-gradient-to-position)}.to-gray-500\/50{--tw-gradient-to:hsla(220,9%,46%,.5) var(--tw-gradient-to-position)}.to-gray-500\/55{--tw-gradient-to:hsla(220,9%,46%,.55) var(--tw-gradient-to-position)}.to-gray-500\/60{--tw-gradient-to:hsla(220,9%,46%,.6) var(--tw-gradient-to-position)}.to-gray-500\/65{--tw-gradient-to:hsla(220,9%,46%,.65) var(--tw-gradient-to-position)}.to-gray-500\/70{--tw-gradient-to:hsla(220,9%,46%,.7) var(--tw-gradient-to-position)}.to-gray-500\/75{--tw-gradient-to:hsla(220,9%,46%,.75) var(--tw-gradient-to-position)}.to-gray-500\/80{--tw-gradient-to:hsla(220,9%,46%,.8) var(--tw-gradient-to-position)}.to-gray-500\/85{--tw-gradient-to:hsla(220,9%,46%,.85) var(--tw-gradient-to-position)}.to-gray-500\/90{--tw-gradient-to:hsla(220,9%,46%,.9) var(--tw-gradient-to-position)}.to-gray-500\/95{--tw-gradient-to:hsla(220,9%,46%,.95) var(--tw-gradient-to-position)}.to-gray-950{--tw-gradient-to:#030712 var(--tw-gradient-to-position)}.to-gray-950\/0{--tw-gradient-to:rgba(3,7,18,0) var(--tw-gradient-to-position)}.to-gray-950\/10{--tw-gradient-to:rgba(3,7,18,.1) var(--tw-gradient-to-position)}.to-gray-950\/100{--tw-gradient-to:#030712 var(--tw-gradient-to-position)}.to-gray-950\/15{--tw-gradient-to:rgba(3,7,18,.15) var(--tw-gradient-to-position)}.to-gray-950\/20{--tw-gradient-to:rgba(3,7,18,.2) var(--tw-gradient-to-position)}.to-gray-950\/25{--tw-gradient-to:rgba(3,7,18,.25) var(--tw-gradient-to-position)}.to-gray-950\/30{--tw-gradient-to:rgba(3,7,18,.3) var(--tw-gradient-to-position)}.to-gray-950\/35{--tw-gradient-to:rgba(3,7,18,.35) var(--tw-gradient-to-position)}.to-gray-950\/40{--tw-gradient-to:rgba(3,7,18,.4) var(--tw-gradient-to-position)}.to-gray-950\/45{--tw-gradient-to:rgba(3,7,18,.45) var(--tw-gradient-to-position)}.to-gray-950\/5{--tw-gradient-to:rgba(3,7,18,.05) var(--tw-gradient-to-position)}.to-gray-950\/50{--tw-gradient-to:rgba(3,7,18,.5) var(--tw-gradient-to-position)}.to-gray-950\/55{--tw-gradient-to:rgba(3,7,18,.55) var(--tw-gradient-to-position)}.to-gray-950\/60{--tw-gradient-to:rgba(3,7,18,.6) var(--tw-gradient-to-position)}.to-gray-950\/65{--tw-gradient-to:rgba(3,7,18,.65) var(--tw-gradient-to-position)}.to-gray-950\/70{--tw-gradient-to:rgba(3,7,18,.7) var(--tw-gradient-to-position)}.to-gray-950\/75{--tw-gradient-to:rgba(3,7,18,.75) var(--tw-gradient-to-position)}.to-gray-950\/80{--tw-gradient-to:rgba(3,7,18,.8) var(--tw-gradient-to-position)}.to-gray-950\/85{--tw-gradient-to:rgba(3,7,18,.85) var(--tw-gradient-to-position)}.to-gray-950\/90{--tw-gradient-to:rgba(3,7,18,.9) var(--tw-gradient-to-position)}.to-gray-950\/95{--tw-gradient-to:rgba(3,7,18,.95) var(--tw-gradient-to-position)}.to-green-100{--tw-gradient-to:#dcfce7 var(--tw-gradient-to-position)}.to-green-100\/0{--tw-gradient-to:rgba(220,252,231,0) var(--tw-gradient-to-position)}.to-green-100\/10{--tw-gradient-to:rgba(220,252,231,.1) var(--tw-gradient-to-position)}.to-green-100\/100{--tw-gradient-to:#dcfce7 var(--tw-gradient-to-position)}.to-green-100\/15{--tw-gradient-to:rgba(220,252,231,.15) var(--tw-gradient-to-position)}.to-green-100\/20{--tw-gradient-to:rgba(220,252,231,.2) var(--tw-gradient-to-position)}.to-green-100\/25{--tw-gradient-to:rgba(220,252,231,.25) var(--tw-gradient-to-position)}.to-green-100\/30{--tw-gradient-to:rgba(220,252,231,.3) var(--tw-gradient-to-position)}.to-green-100\/35{--tw-gradient-to:rgba(220,252,231,.35) var(--tw-gradient-to-position)}.to-green-100\/40{--tw-gradient-to:rgba(220,252,231,.4) var(--tw-gradient-to-position)}.to-green-100\/45{--tw-gradient-to:rgba(220,252,231,.45) var(--tw-gradient-to-position)}.to-green-100\/5{--tw-gradient-to:rgba(220,252,231,.05) var(--tw-gradient-to-position)}.to-green-100\/50{--tw-gradient-to:rgba(220,252,231,.5) var(--tw-gradient-to-position)}.to-green-100\/55{--tw-gradient-to:rgba(220,252,231,.55) var(--tw-gradient-to-position)}.to-green-100\/60{--tw-gradient-to:rgba(220,252,231,.6) var(--tw-gradient-to-position)}.to-green-100\/65{--tw-gradient-to:rgba(220,252,231,.65) var(--tw-gradient-to-position)}.to-green-100\/70{--tw-gradient-to:rgba(220,252,231,.7) var(--tw-gradient-to-position)}.to-green-100\/75{--tw-gradient-to:rgba(220,252,231,.75) var(--tw-gradient-to-position)}.to-green-100\/80{--tw-gradient-to:rgba(220,252,231,.8) var(--tw-gradient-to-position)}.to-green-100\/85{--tw-gradient-to:rgba(220,252,231,.85) var(--tw-gradient-to-position)}.to-green-100\/90{--tw-gradient-to:rgba(220,252,231,.9) var(--tw-gradient-to-position)}.to-green-100\/95{--tw-gradient-to:rgba(220,252,231,.95) var(--tw-gradient-to-position)}.to-green-200{--tw-gradient-to:#bbf7d0 var(--tw-gradient-to-position)}.to-green-200\/0{--tw-gradient-to:rgba(187,247,208,0) var(--tw-gradient-to-position)}.to-green-200\/10{--tw-gradient-to:rgba(187,247,208,.1) var(--tw-gradient-to-position)}.to-green-200\/100{--tw-gradient-to:#bbf7d0 var(--tw-gradient-to-position)}.to-green-200\/15{--tw-gradient-to:rgba(187,247,208,.15) var(--tw-gradient-to-position)}.to-green-200\/20{--tw-gradient-to:rgba(187,247,208,.2) var(--tw-gradient-to-position)}.to-green-200\/25{--tw-gradient-to:rgba(187,247,208,.25) var(--tw-gradient-to-position)}.to-green-200\/30{--tw-gradient-to:rgba(187,247,208,.3) var(--tw-gradient-to-position)}.to-green-200\/35{--tw-gradient-to:rgba(187,247,208,.35) var(--tw-gradient-to-position)}.to-green-200\/40{--tw-gradient-to:rgba(187,247,208,.4) var(--tw-gradient-to-position)}.to-green-200\/45{--tw-gradient-to:rgba(187,247,208,.45) var(--tw-gradient-to-position)}.to-green-200\/5{--tw-gradient-to:rgba(187,247,208,.05) var(--tw-gradient-to-position)}.to-green-200\/50{--tw-gradient-to:rgba(187,247,208,.5) var(--tw-gradient-to-position)}.to-green-200\/55{--tw-gradient-to:rgba(187,247,208,.55) var(--tw-gradient-to-position)}.to-green-200\/60{--tw-gradient-to:rgba(187,247,208,.6) var(--tw-gradient-to-position)}.to-green-200\/65{--tw-gradient-to:rgba(187,247,208,.65) var(--tw-gradient-to-position)}.to-green-200\/70{--tw-gradient-to:rgba(187,247,208,.7) var(--tw-gradient-to-position)}.to-green-200\/75{--tw-gradient-to:rgba(187,247,208,.75) var(--tw-gradient-to-position)}.to-green-200\/80{--tw-gradient-to:rgba(187,247,208,.8) var(--tw-gradient-to-position)}.to-green-200\/85{--tw-gradient-to:rgba(187,247,208,.85) var(--tw-gradient-to-position)}.to-green-200\/90{--tw-gradient-to:rgba(187,247,208,.9) var(--tw-gradient-to-position)}.to-green-200\/95{--tw-gradient-to:rgba(187,247,208,.95) var(--tw-gradient-to-position)}.to-green-50{--tw-gradient-to:#f0fdf4 var(--tw-gradient-to-position)}.to-green-50\/0{--tw-gradient-to:rgba(240,253,244,0) var(--tw-gradient-to-position)}.to-green-50\/10{--tw-gradient-to:rgba(240,253,244,.1) var(--tw-gradient-to-position)}.to-green-50\/100{--tw-gradient-to:#f0fdf4 var(--tw-gradient-to-position)}.to-green-50\/15{--tw-gradient-to:rgba(240,253,244,.15) var(--tw-gradient-to-position)}.to-green-50\/20{--tw-gradient-to:rgba(240,253,244,.2) var(--tw-gradient-to-position)}.to-green-50\/25{--tw-gradient-to:rgba(240,253,244,.25) var(--tw-gradient-to-position)}.to-green-50\/30{--tw-gradient-to:rgba(240,253,244,.3) var(--tw-gradient-to-position)}.to-green-50\/35{--tw-gradient-to:rgba(240,253,244,.35) var(--tw-gradient-to-position)}.to-green-50\/40{--tw-gradient-to:rgba(240,253,244,.4) var(--tw-gradient-to-position)}.to-green-50\/45{--tw-gradient-to:rgba(240,253,244,.45) var(--tw-gradient-to-position)}.to-green-50\/5{--tw-gradient-to:rgba(240,253,244,.05) var(--tw-gradient-to-position)}.to-green-50\/50{--tw-gradient-to:rgba(240,253,244,.5) var(--tw-gradient-to-position)}.to-green-50\/55{--tw-gradient-to:rgba(240,253,244,.55) var(--tw-gradient-to-position)}.to-green-50\/60{--tw-gradient-to:rgba(240,253,244,.6) var(--tw-gradient-to-position)}.to-green-50\/65{--tw-gradient-to:rgba(240,253,244,.65) var(--tw-gradient-to-position)}.to-green-50\/70{--tw-gradient-to:rgba(240,253,244,.7) var(--tw-gradient-to-position)}.to-green-50\/75{--tw-gradient-to:rgba(240,253,244,.75) var(--tw-gradient-to-position)}.to-green-50\/80{--tw-gradient-to:rgba(240,253,244,.8) var(--tw-gradient-to-position)}.to-green-50\/85{--tw-gradient-to:rgba(240,253,244,.85) var(--tw-gradient-to-position)}.to-green-50\/90{--tw-gradient-to:rgba(240,253,244,.9) var(--tw-gradient-to-position)}.to-green-50\/95{--tw-gradient-to:rgba(240,253,244,.95) var(--tw-gradient-to-position)}.to-green-500{--tw-gradient-to:#22c55e var(--tw-gradient-to-position)}.to-green-500\/0{--tw-gradient-to:rgba(34,197,94,0) var(--tw-gradient-to-position)}.to-green-500\/10{--tw-gradient-to:rgba(34,197,94,.1) var(--tw-gradient-to-position)}.to-green-500\/100{--tw-gradient-to:#22c55e var(--tw-gradient-to-position)}.to-green-500\/15{--tw-gradient-to:rgba(34,197,94,.15) var(--tw-gradient-to-position)}.to-green-500\/20{--tw-gradient-to:rgba(34,197,94,.2) var(--tw-gradient-to-position)}.to-green-500\/25{--tw-gradient-to:rgba(34,197,94,.25) var(--tw-gradient-to-position)}.to-green-500\/30{--tw-gradient-to:rgba(34,197,94,.3) var(--tw-gradient-to-position)}.to-green-500\/35{--tw-gradient-to:rgba(34,197,94,.35) var(--tw-gradient-to-position)}.to-green-500\/40{--tw-gradient-to:rgba(34,197,94,.4) var(--tw-gradient-to-position)}.to-green-500\/45{--tw-gradient-to:rgba(34,197,94,.45) var(--tw-gradient-to-position)}.to-green-500\/5{--tw-gradient-to:rgba(34,197,94,.05) var(--tw-gradient-to-position)}.to-green-500\/50{--tw-gradient-to:rgba(34,197,94,.5) var(--tw-gradient-to-position)}.to-green-500\/55{--tw-gradient-to:rgba(34,197,94,.55) var(--tw-gradient-to-position)}.to-green-500\/60{--tw-gradient-to:rgba(34,197,94,.6) var(--tw-gradient-to-position)}.to-green-500\/65{--tw-gradient-to:rgba(34,197,94,.65) var(--tw-gradient-to-position)}.to-green-500\/70{--tw-gradient-to:rgba(34,197,94,.7) var(--tw-gradient-to-position)}.to-green-500\/75{--tw-gradient-to:rgba(34,197,94,.75) var(--tw-gradient-to-position)}.to-green-500\/80{--tw-gradient-to:rgba(34,197,94,.8) var(--tw-gradient-to-position)}.to-green-500\/85{--tw-gradient-to:rgba(34,197,94,.85) var(--tw-gradient-to-position)}.to-green-500\/90{--tw-gradient-to:rgba(34,197,94,.9) var(--tw-gradient-to-position)}.to-green-500\/95{--tw-gradient-to:rgba(34,197,94,.95) var(--tw-gradient-to-position)}.to-green-950{--tw-gradient-to:#052e16 var(--tw-gradient-to-position)}.to-green-950\/0{--tw-gradient-to:rgba(5,46,22,0) var(--tw-gradient-to-position)}.to-green-950\/10{--tw-gradient-to:rgba(5,46,22,.1) var(--tw-gradient-to-position)}.to-green-950\/100{--tw-gradient-to:#052e16 var(--tw-gradient-to-position)}.to-green-950\/15{--tw-gradient-to:rgba(5,46,22,.15) var(--tw-gradient-to-position)}.to-green-950\/20{--tw-gradient-to:rgba(5,46,22,.2) var(--tw-gradient-to-position)}.to-green-950\/25{--tw-gradient-to:rgba(5,46,22,.25) var(--tw-gradient-to-position)}.to-green-950\/30{--tw-gradient-to:rgba(5,46,22,.3) var(--tw-gradient-to-position)}.to-green-950\/35{--tw-gradient-to:rgba(5,46,22,.35) var(--tw-gradient-to-position)}.to-green-950\/40{--tw-gradient-to:rgba(5,46,22,.4) var(--tw-gradient-to-position)}.to-green-950\/45{--tw-gradient-to:rgba(5,46,22,.45) var(--tw-gradient-to-position)}.to-green-950\/5{--tw-gradient-to:rgba(5,46,22,.05) var(--tw-gradient-to-position)}.to-green-950\/50{--tw-gradient-to:rgba(5,46,22,.5) var(--tw-gradient-to-position)}.to-green-950\/55{--tw-gradient-to:rgba(5,46,22,.55) var(--tw-gradient-to-position)}.to-green-950\/60{--tw-gradient-to:rgba(5,46,22,.6) var(--tw-gradient-to-position)}.to-green-950\/65{--tw-gradient-to:rgba(5,46,22,.65) var(--tw-gradient-to-position)}.to-green-950\/70{--tw-gradient-to:rgba(5,46,22,.7) var(--tw-gradient-to-position)}.to-green-950\/75{--tw-gradient-to:rgba(5,46,22,.75) var(--tw-gradient-to-position)}.to-green-950\/80{--tw-gradient-to:rgba(5,46,22,.8) var(--tw-gradient-to-position)}.to-green-950\/85{--tw-gradient-to:rgba(5,46,22,.85) var(--tw-gradient-to-position)}.to-green-950\/90{--tw-gradient-to:rgba(5,46,22,.9) var(--tw-gradient-to-position)}.to-green-950\/95{--tw-gradient-to:rgba(5,46,22,.95) var(--tw-gradient-to-position)}.to-indigo-100{--tw-gradient-to:#e0e7ff var(--tw-gradient-to-position)}.to-indigo-100\/0{--tw-gradient-to:rgba(224,231,255,0) var(--tw-gradient-to-position)}.to-indigo-100\/10{--tw-gradient-to:rgba(224,231,255,.1) var(--tw-gradient-to-position)}.to-indigo-100\/100{--tw-gradient-to:#e0e7ff var(--tw-gradient-to-position)}.to-indigo-100\/15{--tw-gradient-to:rgba(224,231,255,.15) var(--tw-gradient-to-position)}.to-indigo-100\/20{--tw-gradient-to:rgba(224,231,255,.2) var(--tw-gradient-to-position)}.to-indigo-100\/25{--tw-gradient-to:rgba(224,231,255,.25) var(--tw-gradient-to-position)}.to-indigo-100\/30{--tw-gradient-to:rgba(224,231,255,.3) var(--tw-gradient-to-position)}.to-indigo-100\/35{--tw-gradient-to:rgba(224,231,255,.35) var(--tw-gradient-to-position)}.to-indigo-100\/40{--tw-gradient-to:rgba(224,231,255,.4) var(--tw-gradient-to-position)}.to-indigo-100\/45{--tw-gradient-to:rgba(224,231,255,.45) var(--tw-gradient-to-position)}.to-indigo-100\/5{--tw-gradient-to:rgba(224,231,255,.05) var(--tw-gradient-to-position)}.to-indigo-100\/50{--tw-gradient-to:rgba(224,231,255,.5) var(--tw-gradient-to-position)}.to-indigo-100\/55{--tw-gradient-to:rgba(224,231,255,.55) var(--tw-gradient-to-position)}.to-indigo-100\/60{--tw-gradient-to:rgba(224,231,255,.6) var(--tw-gradient-to-position)}.to-indigo-100\/65{--tw-gradient-to:rgba(224,231,255,.65) var(--tw-gradient-to-position)}.to-indigo-100\/70{--tw-gradient-to:rgba(224,231,255,.7) var(--tw-gradient-to-position)}.to-indigo-100\/75{--tw-gradient-to:rgba(224,231,255,.75) var(--tw-gradient-to-position)}.to-indigo-100\/80{--tw-gradient-to:rgba(224,231,255,.8) var(--tw-gradient-to-position)}.to-indigo-100\/85{--tw-gradient-to:rgba(224,231,255,.85) var(--tw-gradient-to-position)}.to-indigo-100\/90{--tw-gradient-to:rgba(224,231,255,.9) var(--tw-gradient-to-position)}.to-indigo-100\/95{--tw-gradient-to:rgba(224,231,255,.95) var(--tw-gradient-to-position)}.to-indigo-200{--tw-gradient-to:#c7d2fe var(--tw-gradient-to-position)}.to-indigo-200\/0{--tw-gradient-to:rgba(199,210,254,0) var(--tw-gradient-to-position)}.to-indigo-200\/10{--tw-gradient-to:rgba(199,210,254,.1) var(--tw-gradient-to-position)}.to-indigo-200\/100{--tw-gradient-to:#c7d2fe var(--tw-gradient-to-position)}.to-indigo-200\/15{--tw-gradient-to:rgba(199,210,254,.15) var(--tw-gradient-to-position)}.to-indigo-200\/20{--tw-gradient-to:rgba(199,210,254,.2) var(--tw-gradient-to-position)}.to-indigo-200\/25{--tw-gradient-to:rgba(199,210,254,.25) var(--tw-gradient-to-position)}.to-indigo-200\/30{--tw-gradient-to:rgba(199,210,254,.3) var(--tw-gradient-to-position)}.to-indigo-200\/35{--tw-gradient-to:rgba(199,210,254,.35) var(--tw-gradient-to-position)}.to-indigo-200\/40{--tw-gradient-to:rgba(199,210,254,.4) var(--tw-gradient-to-position)}.to-indigo-200\/45{--tw-gradient-to:rgba(199,210,254,.45) var(--tw-gradient-to-position)}.to-indigo-200\/5{--tw-gradient-to:rgba(199,210,254,.05) var(--tw-gradient-to-position)}.to-indigo-200\/50{--tw-gradient-to:rgba(199,210,254,.5) var(--tw-gradient-to-position)}.to-indigo-200\/55{--tw-gradient-to:rgba(199,210,254,.55) var(--tw-gradient-to-position)}.to-indigo-200\/60{--tw-gradient-to:rgba(199,210,254,.6) var(--tw-gradient-to-position)}.to-indigo-200\/65{--tw-gradient-to:rgba(199,210,254,.65) var(--tw-gradient-to-position)}.to-indigo-200\/70{--tw-gradient-to:rgba(199,210,254,.7) var(--tw-gradient-to-position)}.to-indigo-200\/75{--tw-gradient-to:rgba(199,210,254,.75) var(--tw-gradient-to-position)}.to-indigo-200\/80{--tw-gradient-to:rgba(199,210,254,.8) var(--tw-gradient-to-position)}.to-indigo-200\/85{--tw-gradient-to:rgba(199,210,254,.85) var(--tw-gradient-to-position)}.to-indigo-200\/90{--tw-gradient-to:rgba(199,210,254,.9) var(--tw-gradient-to-position)}.to-indigo-200\/95{--tw-gradient-to:rgba(199,210,254,.95) var(--tw-gradient-to-position)}.to-indigo-50{--tw-gradient-to:#eef2ff var(--tw-gradient-to-position)}.to-indigo-50\/0{--tw-gradient-to:rgba(238,242,255,0) var(--tw-gradient-to-position)}.to-indigo-50\/10{--tw-gradient-to:rgba(238,242,255,.1) var(--tw-gradient-to-position)}.to-indigo-50\/100{--tw-gradient-to:#eef2ff var(--tw-gradient-to-position)}.to-indigo-50\/15{--tw-gradient-to:rgba(238,242,255,.15) var(--tw-gradient-to-position)}.to-indigo-50\/20{--tw-gradient-to:rgba(238,242,255,.2) var(--tw-gradient-to-position)}.to-indigo-50\/25{--tw-gradient-to:rgba(238,242,255,.25) var(--tw-gradient-to-position)}.to-indigo-50\/30{--tw-gradient-to:rgba(238,242,255,.3) var(--tw-gradient-to-position)}.to-indigo-50\/35{--tw-gradient-to:rgba(238,242,255,.35) var(--tw-gradient-to-position)}.to-indigo-50\/40{--tw-gradient-to:rgba(238,242,255,.4) var(--tw-gradient-to-position)}.to-indigo-50\/45{--tw-gradient-to:rgba(238,242,255,.45) var(--tw-gradient-to-position)}.to-indigo-50\/5{--tw-gradient-to:rgba(238,242,255,.05) var(--tw-gradient-to-position)}.to-indigo-50\/50{--tw-gradient-to:rgba(238,242,255,.5) var(--tw-gradient-to-position)}.to-indigo-50\/55{--tw-gradient-to:rgba(238,242,255,.55) var(--tw-gradient-to-position)}.to-indigo-50\/60{--tw-gradient-to:rgba(238,242,255,.6) var(--tw-gradient-to-position)}.to-indigo-50\/65{--tw-gradient-to:rgba(238,242,255,.65) var(--tw-gradient-to-position)}.to-indigo-50\/70{--tw-gradient-to:rgba(238,242,255,.7) var(--tw-gradient-to-position)}.to-indigo-50\/75{--tw-gradient-to:rgba(238,242,255,.75) var(--tw-gradient-to-position)}.to-indigo-50\/80{--tw-gradient-to:rgba(238,242,255,.8) var(--tw-gradient-to-position)}.to-indigo-50\/85{--tw-gradient-to:rgba(238,242,255,.85) var(--tw-gradient-to-position)}.to-indigo-50\/90{--tw-gradient-to:rgba(238,242,255,.9) var(--tw-gradient-to-position)}.to-indigo-50\/95{--tw-gradient-to:rgba(238,242,255,.95) var(--tw-gradient-to-position)}.to-indigo-500{--tw-gradient-to:#6366f1 var(--tw-gradient-to-position)}.to-indigo-500\/0{--tw-gradient-to:rgba(99,102,241,0) var(--tw-gradient-to-position)}.to-indigo-500\/10{--tw-gradient-to:rgba(99,102,241,.1) var(--tw-gradient-to-position)}.to-indigo-500\/100{--tw-gradient-to:#6366f1 var(--tw-gradient-to-position)}.to-indigo-500\/15{--tw-gradient-to:rgba(99,102,241,.15) var(--tw-gradient-to-position)}.to-indigo-500\/20{--tw-gradient-to:rgba(99,102,241,.2) var(--tw-gradient-to-position)}.to-indigo-500\/25{--tw-gradient-to:rgba(99,102,241,.25) var(--tw-gradient-to-position)}.to-indigo-500\/30{--tw-gradient-to:rgba(99,102,241,.3) var(--tw-gradient-to-position)}.to-indigo-500\/35{--tw-gradient-to:rgba(99,102,241,.35) var(--tw-gradient-to-position)}.to-indigo-500\/40{--tw-gradient-to:rgba(99,102,241,.4) var(--tw-gradient-to-position)}.to-indigo-500\/45{--tw-gradient-to:rgba(99,102,241,.45) var(--tw-gradient-to-position)}.to-indigo-500\/5{--tw-gradient-to:rgba(99,102,241,.05) var(--tw-gradient-to-position)}.to-indigo-500\/50{--tw-gradient-to:rgba(99,102,241,.5) var(--tw-gradient-to-position)}.to-indigo-500\/55{--tw-gradient-to:rgba(99,102,241,.55) var(--tw-gradient-to-position)}.to-indigo-500\/60{--tw-gradient-to:rgba(99,102,241,.6) var(--tw-gradient-to-position)}.to-indigo-500\/65{--tw-gradient-to:rgba(99,102,241,.65) var(--tw-gradient-to-position)}.to-indigo-500\/70{--tw-gradient-to:rgba(99,102,241,.7) var(--tw-gradient-to-position)}.to-indigo-500\/75{--tw-gradient-to:rgba(99,102,241,.75) var(--tw-gradient-to-position)}.to-indigo-500\/80{--tw-gradient-to:rgba(99,102,241,.8) var(--tw-gradient-to-position)}.to-indigo-500\/85{--tw-gradient-to:rgba(99,102,241,.85) var(--tw-gradient-to-position)}.to-indigo-500\/90{--tw-gradient-to:rgba(99,102,241,.9) var(--tw-gradient-to-position)}.to-indigo-500\/95{--tw-gradient-to:rgba(99,102,241,.95) var(--tw-gradient-to-position)}.to-indigo-950{--tw-gradient-to:#1e1b4b var(--tw-gradient-to-position)}.to-indigo-950\/0{--tw-gradient-to:rgba(30,27,75,0) var(--tw-gradient-to-position)}.to-indigo-950\/10{--tw-gradient-to:rgba(30,27,75,.1) var(--tw-gradient-to-position)}.to-indigo-950\/100{--tw-gradient-to:#1e1b4b var(--tw-gradient-to-position)}.to-indigo-950\/15{--tw-gradient-to:rgba(30,27,75,.15) var(--tw-gradient-to-position)}.to-indigo-950\/20{--tw-gradient-to:rgba(30,27,75,.2) var(--tw-gradient-to-position)}.to-indigo-950\/25{--tw-gradient-to:rgba(30,27,75,.25) var(--tw-gradient-to-position)}.to-indigo-950\/30{--tw-gradient-to:rgba(30,27,75,.3) var(--tw-gradient-to-position)}.to-indigo-950\/35{--tw-gradient-to:rgba(30,27,75,.35) var(--tw-gradient-to-position)}.to-indigo-950\/40{--tw-gradient-to:rgba(30,27,75,.4) var(--tw-gradient-to-position)}.to-indigo-950\/45{--tw-gradient-to:rgba(30,27,75,.45) var(--tw-gradient-to-position)}.to-indigo-950\/5{--tw-gradient-to:rgba(30,27,75,.05) var(--tw-gradient-to-position)}.to-indigo-950\/50{--tw-gradient-to:rgba(30,27,75,.5) var(--tw-gradient-to-position)}.to-indigo-950\/55{--tw-gradient-to:rgba(30,27,75,.55) var(--tw-gradient-to-position)}.to-indigo-950\/60{--tw-gradient-to:rgba(30,27,75,.6) var(--tw-gradient-to-position)}.to-indigo-950\/65{--tw-gradient-to:rgba(30,27,75,.65) var(--tw-gradient-to-position)}.to-indigo-950\/70{--tw-gradient-to:rgba(30,27,75,.7) var(--tw-gradient-to-position)}.to-indigo-950\/75{--tw-gradient-to:rgba(30,27,75,.75) var(--tw-gradient-to-position)}.to-indigo-950\/80{--tw-gradient-to:rgba(30,27,75,.8) var(--tw-gradient-to-position)}.to-indigo-950\/85{--tw-gradient-to:rgba(30,27,75,.85) var(--tw-gradient-to-position)}.to-indigo-950\/90{--tw-gradient-to:rgba(30,27,75,.9) var(--tw-gradient-to-position)}.to-indigo-950\/95{--tw-gradient-to:rgba(30,27,75,.95) var(--tw-gradient-to-position)}.to-lime-100{--tw-gradient-to:#ecfccb var(--tw-gradient-to-position)}.to-lime-100\/0{--tw-gradient-to:hsla(80,89%,89%,0) var(--tw-gradient-to-position)}.to-lime-100\/10{--tw-gradient-to:hsla(80,89%,89%,.1) var(--tw-gradient-to-position)}.to-lime-100\/100{--tw-gradient-to:#ecfccb var(--tw-gradient-to-position)}.to-lime-100\/15{--tw-gradient-to:hsla(80,89%,89%,.15) var(--tw-gradient-to-position)}.to-lime-100\/20{--tw-gradient-to:hsla(80,89%,89%,.2) var(--tw-gradient-to-position)}.to-lime-100\/25{--tw-gradient-to:hsla(80,89%,89%,.25) var(--tw-gradient-to-position)}.to-lime-100\/30{--tw-gradient-to:hsla(80,89%,89%,.3) var(--tw-gradient-to-position)}.to-lime-100\/35{--tw-gradient-to:hsla(80,89%,89%,.35) var(--tw-gradient-to-position)}.to-lime-100\/40{--tw-gradient-to:hsla(80,89%,89%,.4) var(--tw-gradient-to-position)}.to-lime-100\/45{--tw-gradient-to:hsla(80,89%,89%,.45) var(--tw-gradient-to-position)}.to-lime-100\/5{--tw-gradient-to:hsla(80,89%,89%,.05) var(--tw-gradient-to-position)}.to-lime-100\/50{--tw-gradient-to:hsla(80,89%,89%,.5) var(--tw-gradient-to-position)}.to-lime-100\/55{--tw-gradient-to:hsla(80,89%,89%,.55) var(--tw-gradient-to-position)}.to-lime-100\/60{--tw-gradient-to:hsla(80,89%,89%,.6) var(--tw-gradient-to-position)}.to-lime-100\/65{--tw-gradient-to:hsla(80,89%,89%,.65) var(--tw-gradient-to-position)}.to-lime-100\/70{--tw-gradient-to:hsla(80,89%,89%,.7) var(--tw-gradient-to-position)}.to-lime-100\/75{--tw-gradient-to:hsla(80,89%,89%,.75) var(--tw-gradient-to-position)}.to-lime-100\/80{--tw-gradient-to:hsla(80,89%,89%,.8) var(--tw-gradient-to-position)}.to-lime-100\/85{--tw-gradient-to:hsla(80,89%,89%,.85) var(--tw-gradient-to-position)}.to-lime-100\/90{--tw-gradient-to:hsla(80,89%,89%,.9) var(--tw-gradient-to-position)}.to-lime-100\/95{--tw-gradient-to:hsla(80,89%,89%,.95) var(--tw-gradient-to-position)}.to-lime-200{--tw-gradient-to:#d9f99d var(--tw-gradient-to-position)}.to-lime-200\/0{--tw-gradient-to:hsla(81,88%,80%,0) var(--tw-gradient-to-position)}.to-lime-200\/10{--tw-gradient-to:hsla(81,88%,80%,.1) var(--tw-gradient-to-position)}.to-lime-200\/100{--tw-gradient-to:#d9f99d var(--tw-gradient-to-position)}.to-lime-200\/15{--tw-gradient-to:hsla(81,88%,80%,.15) var(--tw-gradient-to-position)}.to-lime-200\/20{--tw-gradient-to:hsla(81,88%,80%,.2) var(--tw-gradient-to-position)}.to-lime-200\/25{--tw-gradient-to:hsla(81,88%,80%,.25) var(--tw-gradient-to-position)}.to-lime-200\/30{--tw-gradient-to:hsla(81,88%,80%,.3) var(--tw-gradient-to-position)}.to-lime-200\/35{--tw-gradient-to:hsla(81,88%,80%,.35) var(--tw-gradient-to-position)}.to-lime-200\/40{--tw-gradient-to:hsla(81,88%,80%,.4) var(--tw-gradient-to-position)}.to-lime-200\/45{--tw-gradient-to:hsla(81,88%,80%,.45) var(--tw-gradient-to-position)}.to-lime-200\/5{--tw-gradient-to:hsla(81,88%,80%,.05) var(--tw-gradient-to-position)}.to-lime-200\/50{--tw-gradient-to:hsla(81,88%,80%,.5) var(--tw-gradient-to-position)}.to-lime-200\/55{--tw-gradient-to:hsla(81,88%,80%,.55) var(--tw-gradient-to-position)}.to-lime-200\/60{--tw-gradient-to:hsla(81,88%,80%,.6) var(--tw-gradient-to-position)}.to-lime-200\/65{--tw-gradient-to:hsla(81,88%,80%,.65) var(--tw-gradient-to-position)}.to-lime-200\/70{--tw-gradient-to:hsla(81,88%,80%,.7) var(--tw-gradient-to-position)}.to-lime-200\/75{--tw-gradient-to:hsla(81,88%,80%,.75) var(--tw-gradient-to-position)}.to-lime-200\/80{--tw-gradient-to:hsla(81,88%,80%,.8) var(--tw-gradient-to-position)}.to-lime-200\/85{--tw-gradient-to:hsla(81,88%,80%,.85) var(--tw-gradient-to-position)}.to-lime-200\/90{--tw-gradient-to:hsla(81,88%,80%,.9) var(--tw-gradient-to-position)}.to-lime-200\/95{--tw-gradient-to:hsla(81,88%,80%,.95) var(--tw-gradient-to-position)}.to-lime-50{--tw-gradient-to:#f7fee7 var(--tw-gradient-to-position)}.to-lime-50\/0{--tw-gradient-to:hsla(78,92%,95%,0) var(--tw-gradient-to-position)}.to-lime-50\/10{--tw-gradient-to:hsla(78,92%,95%,.1) var(--tw-gradient-to-position)}.to-lime-50\/100{--tw-gradient-to:#f7fee7 var(--tw-gradient-to-position)}.to-lime-50\/15{--tw-gradient-to:hsla(78,92%,95%,.15) var(--tw-gradient-to-position)}.to-lime-50\/20{--tw-gradient-to:hsla(78,92%,95%,.2) var(--tw-gradient-to-position)}.to-lime-50\/25{--tw-gradient-to:hsla(78,92%,95%,.25) var(--tw-gradient-to-position)}.to-lime-50\/30{--tw-gradient-to:hsla(78,92%,95%,.3) var(--tw-gradient-to-position)}.to-lime-50\/35{--tw-gradient-to:hsla(78,92%,95%,.35) var(--tw-gradient-to-position)}.to-lime-50\/40{--tw-gradient-to:hsla(78,92%,95%,.4) var(--tw-gradient-to-position)}.to-lime-50\/45{--tw-gradient-to:hsla(78,92%,95%,.45) var(--tw-gradient-to-position)}.to-lime-50\/5{--tw-gradient-to:hsla(78,92%,95%,.05) var(--tw-gradient-to-position)}.to-lime-50\/50{--tw-gradient-to:hsla(78,92%,95%,.5) var(--tw-gradient-to-position)}.to-lime-50\/55{--tw-gradient-to:hsla(78,92%,95%,.55) var(--tw-gradient-to-position)}.to-lime-50\/60{--tw-gradient-to:hsla(78,92%,95%,.6) var(--tw-gradient-to-position)}.to-lime-50\/65{--tw-gradient-to:hsla(78,92%,95%,.65) var(--tw-gradient-to-position)}.to-lime-50\/70{--tw-gradient-to:hsla(78,92%,95%,.7) var(--tw-gradient-to-position)}.to-lime-50\/75{--tw-gradient-to:hsla(78,92%,95%,.75) var(--tw-gradient-to-position)}.to-lime-50\/80{--tw-gradient-to:hsla(78,92%,95%,.8) var(--tw-gradient-to-position)}.to-lime-50\/85{--tw-gradient-to:hsla(78,92%,95%,.85) var(--tw-gradient-to-position)}.to-lime-50\/90{--tw-gradient-to:hsla(78,92%,95%,.9) var(--tw-gradient-to-position)}.to-lime-50\/95{--tw-gradient-to:hsla(78,92%,95%,.95) var(--tw-gradient-to-position)}.to-lime-500{--tw-gradient-to:#84cc16 var(--tw-gradient-to-position)}.to-lime-500\/0{--tw-gradient-to:rgba(132,204,22,0) var(--tw-gradient-to-position)}.to-lime-500\/10{--tw-gradient-to:rgba(132,204,22,.1) var(--tw-gradient-to-position)}.to-lime-500\/100{--tw-gradient-to:#84cc16 var(--tw-gradient-to-position)}.to-lime-500\/15{--tw-gradient-to:rgba(132,204,22,.15) var(--tw-gradient-to-position)}.to-lime-500\/20{--tw-gradient-to:rgba(132,204,22,.2) var(--tw-gradient-to-position)}.to-lime-500\/25{--tw-gradient-to:rgba(132,204,22,.25) var(--tw-gradient-to-position)}.to-lime-500\/30{--tw-gradient-to:rgba(132,204,22,.3) var(--tw-gradient-to-position)}.to-lime-500\/35{--tw-gradient-to:rgba(132,204,22,.35) var(--tw-gradient-to-position)}.to-lime-500\/40{--tw-gradient-to:rgba(132,204,22,.4) var(--tw-gradient-to-position)}.to-lime-500\/45{--tw-gradient-to:rgba(132,204,22,.45) var(--tw-gradient-to-position)}.to-lime-500\/5{--tw-gradient-to:rgba(132,204,22,.05) var(--tw-gradient-to-position)}.to-lime-500\/50{--tw-gradient-to:rgba(132,204,22,.5) var(--tw-gradient-to-position)}.to-lime-500\/55{--tw-gradient-to:rgba(132,204,22,.55) var(--tw-gradient-to-position)}.to-lime-500\/60{--tw-gradient-to:rgba(132,204,22,.6) var(--tw-gradient-to-position)}.to-lime-500\/65{--tw-gradient-to:rgba(132,204,22,.65) var(--tw-gradient-to-position)}.to-lime-500\/70{--tw-gradient-to:rgba(132,204,22,.7) var(--tw-gradient-to-position)}.to-lime-500\/75{--tw-gradient-to:rgba(132,204,22,.75) var(--tw-gradient-to-position)}.to-lime-500\/80{--tw-gradient-to:rgba(132,204,22,.8) var(--tw-gradient-to-position)}.to-lime-500\/85{--tw-gradient-to:rgba(132,204,22,.85) var(--tw-gradient-to-position)}.to-lime-500\/90{--tw-gradient-to:rgba(132,204,22,.9) var(--tw-gradient-to-position)}.to-lime-500\/95{--tw-gradient-to:rgba(132,204,22,.95) var(--tw-gradient-to-position)}.to-lime-950{--tw-gradient-to:#1a2e05 var(--tw-gradient-to-position)}.to-lime-950\/0{--tw-gradient-to:rgba(26,46,5,0) var(--tw-gradient-to-position)}.to-lime-950\/10{--tw-gradient-to:rgba(26,46,5,.1) var(--tw-gradient-to-position)}.to-lime-950\/100{--tw-gradient-to:#1a2e05 var(--tw-gradient-to-position)}.to-lime-950\/15{--tw-gradient-to:rgba(26,46,5,.15) var(--tw-gradient-to-position)}.to-lime-950\/20{--tw-gradient-to:rgba(26,46,5,.2) var(--tw-gradient-to-position)}.to-lime-950\/25{--tw-gradient-to:rgba(26,46,5,.25) var(--tw-gradient-to-position)}.to-lime-950\/30{--tw-gradient-to:rgba(26,46,5,.3) var(--tw-gradient-to-position)}.to-lime-950\/35{--tw-gradient-to:rgba(26,46,5,.35) var(--tw-gradient-to-position)}.to-lime-950\/40{--tw-gradient-to:rgba(26,46,5,.4) var(--tw-gradient-to-position)}.to-lime-950\/45{--tw-gradient-to:rgba(26,46,5,.45) var(--tw-gradient-to-position)}.to-lime-950\/5{--tw-gradient-to:rgba(26,46,5,.05) var(--tw-gradient-to-position)}.to-lime-950\/50{--tw-gradient-to:rgba(26,46,5,.5) var(--tw-gradient-to-position)}.to-lime-950\/55{--tw-gradient-to:rgba(26,46,5,.55) var(--tw-gradient-to-position)}.to-lime-950\/60{--tw-gradient-to:rgba(26,46,5,.6) var(--tw-gradient-to-position)}.to-lime-950\/65{--tw-gradient-to:rgba(26,46,5,.65) var(--tw-gradient-to-position)}.to-lime-950\/70{--tw-gradient-to:rgba(26,46,5,.7) var(--tw-gradient-to-position)}.to-lime-950\/75{--tw-gradient-to:rgba(26,46,5,.75) var(--tw-gradient-to-position)}.to-lime-950\/80{--tw-gradient-to:rgba(26,46,5,.8) var(--tw-gradient-to-position)}.to-lime-950\/85{--tw-gradient-to:rgba(26,46,5,.85) var(--tw-gradient-to-position)}.to-lime-950\/90{--tw-gradient-to:rgba(26,46,5,.9) var(--tw-gradient-to-position)}.to-lime-950\/95{--tw-gradient-to:rgba(26,46,5,.95) var(--tw-gradient-to-position)}.to-neutral-100{--tw-gradient-to:#f5f5f5 var(--tw-gradient-to-position)}.to-neutral-100\/0{--tw-gradient-to:hsla(0,0%,96%,0) var(--tw-gradient-to-position)}.to-neutral-100\/10{--tw-gradient-to:hsla(0,0%,96%,.1) var(--tw-gradient-to-position)}.to-neutral-100\/100{--tw-gradient-to:#f5f5f5 var(--tw-gradient-to-position)}.to-neutral-100\/15{--tw-gradient-to:hsla(0,0%,96%,.15) var(--tw-gradient-to-position)}.to-neutral-100\/20{--tw-gradient-to:hsla(0,0%,96%,.2) var(--tw-gradient-to-position)}.to-neutral-100\/25{--tw-gradient-to:hsla(0,0%,96%,.25) var(--tw-gradient-to-position)}.to-neutral-100\/30{--tw-gradient-to:hsla(0,0%,96%,.3) var(--tw-gradient-to-position)}.to-neutral-100\/35{--tw-gradient-to:hsla(0,0%,96%,.35) var(--tw-gradient-to-position)}.to-neutral-100\/40{--tw-gradient-to:hsla(0,0%,96%,.4) var(--tw-gradient-to-position)}.to-neutral-100\/45{--tw-gradient-to:hsla(0,0%,96%,.45) var(--tw-gradient-to-position)}.to-neutral-100\/5{--tw-gradient-to:hsla(0,0%,96%,.05) var(--tw-gradient-to-position)}.to-neutral-100\/50{--tw-gradient-to:hsla(0,0%,96%,.5) var(--tw-gradient-to-position)}.to-neutral-100\/55{--tw-gradient-to:hsla(0,0%,96%,.55) var(--tw-gradient-to-position)}.to-neutral-100\/60{--tw-gradient-to:hsla(0,0%,96%,.6) var(--tw-gradient-to-position)}.to-neutral-100\/65{--tw-gradient-to:hsla(0,0%,96%,.65) var(--tw-gradient-to-position)}.to-neutral-100\/70{--tw-gradient-to:hsla(0,0%,96%,.7) var(--tw-gradient-to-position)}.to-neutral-100\/75{--tw-gradient-to:hsla(0,0%,96%,.75) var(--tw-gradient-to-position)}.to-neutral-100\/80{--tw-gradient-to:hsla(0,0%,96%,.8) var(--tw-gradient-to-position)}.to-neutral-100\/85{--tw-gradient-to:hsla(0,0%,96%,.85) var(--tw-gradient-to-position)}.to-neutral-100\/90{--tw-gradient-to:hsla(0,0%,96%,.9) var(--tw-gradient-to-position)}.to-neutral-100\/95{--tw-gradient-to:hsla(0,0%,96%,.95) var(--tw-gradient-to-position)}.to-neutral-200{--tw-gradient-to:#e5e5e5 var(--tw-gradient-to-position)}.to-neutral-200\/0{--tw-gradient-to:hsla(0,0%,90%,0) var(--tw-gradient-to-position)}.to-neutral-200\/10{--tw-gradient-to:hsla(0,0%,90%,.1) var(--tw-gradient-to-position)}.to-neutral-200\/100{--tw-gradient-to:#e5e5e5 var(--tw-gradient-to-position)}.to-neutral-200\/15{--tw-gradient-to:hsla(0,0%,90%,.15) var(--tw-gradient-to-position)}.to-neutral-200\/20{--tw-gradient-to:hsla(0,0%,90%,.2) var(--tw-gradient-to-position)}.to-neutral-200\/25{--tw-gradient-to:hsla(0,0%,90%,.25) var(--tw-gradient-to-position)}.to-neutral-200\/30{--tw-gradient-to:hsla(0,0%,90%,.3) var(--tw-gradient-to-position)}.to-neutral-200\/35{--tw-gradient-to:hsla(0,0%,90%,.35) var(--tw-gradient-to-position)}.to-neutral-200\/40{--tw-gradient-to:hsla(0,0%,90%,.4) var(--tw-gradient-to-position)}.to-neutral-200\/45{--tw-gradient-to:hsla(0,0%,90%,.45) var(--tw-gradient-to-position)}.to-neutral-200\/5{--tw-gradient-to:hsla(0,0%,90%,.05) var(--tw-gradient-to-position)}.to-neutral-200\/50{--tw-gradient-to:hsla(0,0%,90%,.5) var(--tw-gradient-to-position)}.to-neutral-200\/55{--tw-gradient-to:hsla(0,0%,90%,.55) var(--tw-gradient-to-position)}.to-neutral-200\/60{--tw-gradient-to:hsla(0,0%,90%,.6) var(--tw-gradient-to-position)}.to-neutral-200\/65{--tw-gradient-to:hsla(0,0%,90%,.65) var(--tw-gradient-to-position)}.to-neutral-200\/70{--tw-gradient-to:hsla(0,0%,90%,.7) var(--tw-gradient-to-position)}.to-neutral-200\/75{--tw-gradient-to:hsla(0,0%,90%,.75) var(--tw-gradient-to-position)}.to-neutral-200\/80{--tw-gradient-to:hsla(0,0%,90%,.8) var(--tw-gradient-to-position)}.to-neutral-200\/85{--tw-gradient-to:hsla(0,0%,90%,.85) var(--tw-gradient-to-position)}.to-neutral-200\/90{--tw-gradient-to:hsla(0,0%,90%,.9) var(--tw-gradient-to-position)}.to-neutral-200\/95{--tw-gradient-to:hsla(0,0%,90%,.95) var(--tw-gradient-to-position)}.to-neutral-50{--tw-gradient-to:#fafafa var(--tw-gradient-to-position)}.to-neutral-50\/0{--tw-gradient-to:hsla(0,0%,98%,0) var(--tw-gradient-to-position)}.to-neutral-50\/10{--tw-gradient-to:hsla(0,0%,98%,.1) var(--tw-gradient-to-position)}.to-neutral-50\/100{--tw-gradient-to:#fafafa var(--tw-gradient-to-position)}.to-neutral-50\/15{--tw-gradient-to:hsla(0,0%,98%,.15) var(--tw-gradient-to-position)}.to-neutral-50\/20{--tw-gradient-to:hsla(0,0%,98%,.2) var(--tw-gradient-to-position)}.to-neutral-50\/25{--tw-gradient-to:hsla(0,0%,98%,.25) var(--tw-gradient-to-position)}.to-neutral-50\/30{--tw-gradient-to:hsla(0,0%,98%,.3) var(--tw-gradient-to-position)}.to-neutral-50\/35{--tw-gradient-to:hsla(0,0%,98%,.35) var(--tw-gradient-to-position)}.to-neutral-50\/40{--tw-gradient-to:hsla(0,0%,98%,.4) var(--tw-gradient-to-position)}.to-neutral-50\/45{--tw-gradient-to:hsla(0,0%,98%,.45) var(--tw-gradient-to-position)}.to-neutral-50\/5{--tw-gradient-to:hsla(0,0%,98%,.05) var(--tw-gradient-to-position)}.to-neutral-50\/50{--tw-gradient-to:hsla(0,0%,98%,.5) var(--tw-gradient-to-position)}.to-neutral-50\/55{--tw-gradient-to:hsla(0,0%,98%,.55) var(--tw-gradient-to-position)}.to-neutral-50\/60{--tw-gradient-to:hsla(0,0%,98%,.6) var(--tw-gradient-to-position)}.to-neutral-50\/65{--tw-gradient-to:hsla(0,0%,98%,.65) var(--tw-gradient-to-position)}.to-neutral-50\/70{--tw-gradient-to:hsla(0,0%,98%,.7) var(--tw-gradient-to-position)}.to-neutral-50\/75{--tw-gradient-to:hsla(0,0%,98%,.75) var(--tw-gradient-to-position)}.to-neutral-50\/80{--tw-gradient-to:hsla(0,0%,98%,.8) var(--tw-gradient-to-position)}.to-neutral-50\/85{--tw-gradient-to:hsla(0,0%,98%,.85) var(--tw-gradient-to-position)}.to-neutral-50\/90{--tw-gradient-to:hsla(0,0%,98%,.9) var(--tw-gradient-to-position)}.to-neutral-50\/95{--tw-gradient-to:hsla(0,0%,98%,.95) var(--tw-gradient-to-position)}.to-neutral-500{--tw-gradient-to:#737373 var(--tw-gradient-to-position)}.to-neutral-500\/0{--tw-gradient-to:hsla(0,0%,45%,0) var(--tw-gradient-to-position)}.to-neutral-500\/10{--tw-gradient-to:hsla(0,0%,45%,.1) var(--tw-gradient-to-position)}.to-neutral-500\/100{--tw-gradient-to:#737373 var(--tw-gradient-to-position)}.to-neutral-500\/15{--tw-gradient-to:hsla(0,0%,45%,.15) var(--tw-gradient-to-position)}.to-neutral-500\/20{--tw-gradient-to:hsla(0,0%,45%,.2) var(--tw-gradient-to-position)}.to-neutral-500\/25{--tw-gradient-to:hsla(0,0%,45%,.25) var(--tw-gradient-to-position)}.to-neutral-500\/30{--tw-gradient-to:hsla(0,0%,45%,.3) var(--tw-gradient-to-position)}.to-neutral-500\/35{--tw-gradient-to:hsla(0,0%,45%,.35) var(--tw-gradient-to-position)}.to-neutral-500\/40{--tw-gradient-to:hsla(0,0%,45%,.4) var(--tw-gradient-to-position)}.to-neutral-500\/45{--tw-gradient-to:hsla(0,0%,45%,.45) var(--tw-gradient-to-position)}.to-neutral-500\/5{--tw-gradient-to:hsla(0,0%,45%,.05) var(--tw-gradient-to-position)}.to-neutral-500\/50{--tw-gradient-to:hsla(0,0%,45%,.5) var(--tw-gradient-to-position)}.to-neutral-500\/55{--tw-gradient-to:hsla(0,0%,45%,.55) var(--tw-gradient-to-position)}.to-neutral-500\/60{--tw-gradient-to:hsla(0,0%,45%,.6) var(--tw-gradient-to-position)}.to-neutral-500\/65{--tw-gradient-to:hsla(0,0%,45%,.65) var(--tw-gradient-to-position)}.to-neutral-500\/70{--tw-gradient-to:hsla(0,0%,45%,.7) var(--tw-gradient-to-position)}.to-neutral-500\/75{--tw-gradient-to:hsla(0,0%,45%,.75) var(--tw-gradient-to-position)}.to-neutral-500\/80{--tw-gradient-to:hsla(0,0%,45%,.8) var(--tw-gradient-to-position)}.to-neutral-500\/85{--tw-gradient-to:hsla(0,0%,45%,.85) var(--tw-gradient-to-position)}.to-neutral-500\/90{--tw-gradient-to:hsla(0,0%,45%,.9) var(--tw-gradient-to-position)}.to-neutral-500\/95{--tw-gradient-to:hsla(0,0%,45%,.95) var(--tw-gradient-to-position)}.to-neutral-950{--tw-gradient-to:#0a0a0a var(--tw-gradient-to-position)}.to-neutral-950\/0{--tw-gradient-to:hsla(0,0%,4%,0) var(--tw-gradient-to-position)}.to-neutral-950\/10{--tw-gradient-to:hsla(0,0%,4%,.1) var(--tw-gradient-to-position)}.to-neutral-950\/100{--tw-gradient-to:#0a0a0a var(--tw-gradient-to-position)}.to-neutral-950\/15{--tw-gradient-to:hsla(0,0%,4%,.15) var(--tw-gradient-to-position)}.to-neutral-950\/20{--tw-gradient-to:hsla(0,0%,4%,.2) var(--tw-gradient-to-position)}.to-neutral-950\/25{--tw-gradient-to:hsla(0,0%,4%,.25) var(--tw-gradient-to-position)}.to-neutral-950\/30{--tw-gradient-to:hsla(0,0%,4%,.3) var(--tw-gradient-to-position)}.to-neutral-950\/35{--tw-gradient-to:hsla(0,0%,4%,.35) var(--tw-gradient-to-position)}.to-neutral-950\/40{--tw-gradient-to:hsla(0,0%,4%,.4) var(--tw-gradient-to-position)}.to-neutral-950\/45{--tw-gradient-to:hsla(0,0%,4%,.45) var(--tw-gradient-to-position)}.to-neutral-950\/5{--tw-gradient-to:hsla(0,0%,4%,.05) var(--tw-gradient-to-position)}.to-neutral-950\/50{--tw-gradient-to:hsla(0,0%,4%,.5) var(--tw-gradient-to-position)}.to-neutral-950\/55{--tw-gradient-to:hsla(0,0%,4%,.55) var(--tw-gradient-to-position)}.to-neutral-950\/60{--tw-gradient-to:hsla(0,0%,4%,.6) var(--tw-gradient-to-position)}.to-neutral-950\/65{--tw-gradient-to:hsla(0,0%,4%,.65) var(--tw-gradient-to-position)}.to-neutral-950\/70{--tw-gradient-to:hsla(0,0%,4%,.7) var(--tw-gradient-to-position)}.to-neutral-950\/75{--tw-gradient-to:hsla(0,0%,4%,.75) var(--tw-gradient-to-position)}.to-neutral-950\/80{--tw-gradient-to:hsla(0,0%,4%,.8) var(--tw-gradient-to-position)}.to-neutral-950\/85{--tw-gradient-to:hsla(0,0%,4%,.85) var(--tw-gradient-to-position)}.to-neutral-950\/90{--tw-gradient-to:hsla(0,0%,4%,.9) var(--tw-gradient-to-position)}.to-neutral-950\/95{--tw-gradient-to:hsla(0,0%,4%,.95) var(--tw-gradient-to-position)}.to-orange-100{--tw-gradient-to:#ffedd5 var(--tw-gradient-to-position)}.to-orange-100\/0{--tw-gradient-to:rgba(255,237,213,0) var(--tw-gradient-to-position)}.to-orange-100\/10{--tw-gradient-to:rgba(255,237,213,.1) var(--tw-gradient-to-position)}.to-orange-100\/100{--tw-gradient-to:#ffedd5 var(--tw-gradient-to-position)}.to-orange-100\/15{--tw-gradient-to:rgba(255,237,213,.15) var(--tw-gradient-to-position)}.to-orange-100\/20{--tw-gradient-to:rgba(255,237,213,.2) var(--tw-gradient-to-position)}.to-orange-100\/25{--tw-gradient-to:rgba(255,237,213,.25) var(--tw-gradient-to-position)}.to-orange-100\/30{--tw-gradient-to:rgba(255,237,213,.3) var(--tw-gradient-to-position)}.to-orange-100\/35{--tw-gradient-to:rgba(255,237,213,.35) var(--tw-gradient-to-position)}.to-orange-100\/40{--tw-gradient-to:rgba(255,237,213,.4) var(--tw-gradient-to-position)}.to-orange-100\/45{--tw-gradient-to:rgba(255,237,213,.45) var(--tw-gradient-to-position)}.to-orange-100\/5{--tw-gradient-to:rgba(255,237,213,.05) var(--tw-gradient-to-position)}.to-orange-100\/50{--tw-gradient-to:rgba(255,237,213,.5) var(--tw-gradient-to-position)}.to-orange-100\/55{--tw-gradient-to:rgba(255,237,213,.55) var(--tw-gradient-to-position)}.to-orange-100\/60{--tw-gradient-to:rgba(255,237,213,.6) var(--tw-gradient-to-position)}.to-orange-100\/65{--tw-gradient-to:rgba(255,237,213,.65) var(--tw-gradient-to-position)}.to-orange-100\/70{--tw-gradient-to:rgba(255,237,213,.7) var(--tw-gradient-to-position)}.to-orange-100\/75{--tw-gradient-to:rgba(255,237,213,.75) var(--tw-gradient-to-position)}.to-orange-100\/80{--tw-gradient-to:rgba(255,237,213,.8) var(--tw-gradient-to-position)}.to-orange-100\/85{--tw-gradient-to:rgba(255,237,213,.85) var(--tw-gradient-to-position)}.to-orange-100\/90{--tw-gradient-to:rgba(255,237,213,.9) var(--tw-gradient-to-position)}.to-orange-100\/95{--tw-gradient-to:rgba(255,237,213,.95) var(--tw-gradient-to-position)}.to-orange-200{--tw-gradient-to:#fed7aa var(--tw-gradient-to-position)}.to-orange-200\/0{--tw-gradient-to:hsla(32,98%,83%,0) var(--tw-gradient-to-position)}.to-orange-200\/10{--tw-gradient-to:hsla(32,98%,83%,.1) var(--tw-gradient-to-position)}.to-orange-200\/100{--tw-gradient-to:#fed7aa var(--tw-gradient-to-position)}.to-orange-200\/15{--tw-gradient-to:hsla(32,98%,83%,.15) var(--tw-gradient-to-position)}.to-orange-200\/20{--tw-gradient-to:hsla(32,98%,83%,.2) var(--tw-gradient-to-position)}.to-orange-200\/25{--tw-gradient-to:hsla(32,98%,83%,.25) var(--tw-gradient-to-position)}.to-orange-200\/30{--tw-gradient-to:hsla(32,98%,83%,.3) var(--tw-gradient-to-position)}.to-orange-200\/35{--tw-gradient-to:hsla(32,98%,83%,.35) var(--tw-gradient-to-position)}.to-orange-200\/40{--tw-gradient-to:hsla(32,98%,83%,.4) var(--tw-gradient-to-position)}.to-orange-200\/45{--tw-gradient-to:hsla(32,98%,83%,.45) var(--tw-gradient-to-position)}.to-orange-200\/5{--tw-gradient-to:hsla(32,98%,83%,.05) var(--tw-gradient-to-position)}.to-orange-200\/50{--tw-gradient-to:hsla(32,98%,83%,.5) var(--tw-gradient-to-position)}.to-orange-200\/55{--tw-gradient-to:hsla(32,98%,83%,.55) var(--tw-gradient-to-position)}.to-orange-200\/60{--tw-gradient-to:hsla(32,98%,83%,.6) var(--tw-gradient-to-position)}.to-orange-200\/65{--tw-gradient-to:hsla(32,98%,83%,.65) var(--tw-gradient-to-position)}.to-orange-200\/70{--tw-gradient-to:hsla(32,98%,83%,.7) var(--tw-gradient-to-position)}.to-orange-200\/75{--tw-gradient-to:hsla(32,98%,83%,.75) var(--tw-gradient-to-position)}.to-orange-200\/80{--tw-gradient-to:hsla(32,98%,83%,.8) var(--tw-gradient-to-position)}.to-orange-200\/85{--tw-gradient-to:hsla(32,98%,83%,.85) var(--tw-gradient-to-position)}.to-orange-200\/90{--tw-gradient-to:hsla(32,98%,83%,.9) var(--tw-gradient-to-position)}.to-orange-200\/95{--tw-gradient-to:hsla(32,98%,83%,.95) var(--tw-gradient-to-position)}.to-orange-50{--tw-gradient-to:#fff7ed var(--tw-gradient-to-position)}.to-orange-50\/0{--tw-gradient-to:rgba(255,247,237,0) var(--tw-gradient-to-position)}.to-orange-50\/10{--tw-gradient-to:rgba(255,247,237,.1) var(--tw-gradient-to-position)}.to-orange-50\/100{--tw-gradient-to:#fff7ed var(--tw-gradient-to-position)}.to-orange-50\/15{--tw-gradient-to:rgba(255,247,237,.15) var(--tw-gradient-to-position)}.to-orange-50\/20{--tw-gradient-to:rgba(255,247,237,.2) var(--tw-gradient-to-position)}.to-orange-50\/25{--tw-gradient-to:rgba(255,247,237,.25) var(--tw-gradient-to-position)}.to-orange-50\/30{--tw-gradient-to:rgba(255,247,237,.3) var(--tw-gradient-to-position)}.to-orange-50\/35{--tw-gradient-to:rgba(255,247,237,.35) var(--tw-gradient-to-position)}.to-orange-50\/40{--tw-gradient-to:rgba(255,247,237,.4) var(--tw-gradient-to-position)}.to-orange-50\/45{--tw-gradient-to:rgba(255,247,237,.45) var(--tw-gradient-to-position)}.to-orange-50\/5{--tw-gradient-to:rgba(255,247,237,.05) var(--tw-gradient-to-position)}.to-orange-50\/50{--tw-gradient-to:rgba(255,247,237,.5) var(--tw-gradient-to-position)}.to-orange-50\/55{--tw-gradient-to:rgba(255,247,237,.55) var(--tw-gradient-to-position)}.to-orange-50\/60{--tw-gradient-to:rgba(255,247,237,.6) var(--tw-gradient-to-position)}.to-orange-50\/65{--tw-gradient-to:rgba(255,247,237,.65) var(--tw-gradient-to-position)}.to-orange-50\/70{--tw-gradient-to:rgba(255,247,237,.7) var(--tw-gradient-to-position)}.to-orange-50\/75{--tw-gradient-to:rgba(255,247,237,.75) var(--tw-gradient-to-position)}.to-orange-50\/80{--tw-gradient-to:rgba(255,247,237,.8) var(--tw-gradient-to-position)}.to-orange-50\/85{--tw-gradient-to:rgba(255,247,237,.85) var(--tw-gradient-to-position)}.to-orange-50\/90{--tw-gradient-to:rgba(255,247,237,.9) var(--tw-gradient-to-position)}.to-orange-50\/95{--tw-gradient-to:rgba(255,247,237,.95) var(--tw-gradient-to-position)}.to-orange-500{--tw-gradient-to:#f97316 var(--tw-gradient-to-position)}.to-orange-500\/0{--tw-gradient-to:rgba(249,115,22,0) var(--tw-gradient-to-position)}.to-orange-500\/10{--tw-gradient-to:rgba(249,115,22,.1) var(--tw-gradient-to-position)}.to-orange-500\/100{--tw-gradient-to:#f97316 var(--tw-gradient-to-position)}.to-orange-500\/15{--tw-gradient-to:rgba(249,115,22,.15) var(--tw-gradient-to-position)}.to-orange-500\/20{--tw-gradient-to:rgba(249,115,22,.2) var(--tw-gradient-to-position)}.to-orange-500\/25{--tw-gradient-to:rgba(249,115,22,.25) var(--tw-gradient-to-position)}.to-orange-500\/30{--tw-gradient-to:rgba(249,115,22,.3) var(--tw-gradient-to-position)}.to-orange-500\/35{--tw-gradient-to:rgba(249,115,22,.35) var(--tw-gradient-to-position)}.to-orange-500\/40{--tw-gradient-to:rgba(249,115,22,.4) var(--tw-gradient-to-position)}.to-orange-500\/45{--tw-gradient-to:rgba(249,115,22,.45) var(--tw-gradient-to-position)}.to-orange-500\/5{--tw-gradient-to:rgba(249,115,22,.05) var(--tw-gradient-to-position)}.to-orange-500\/50{--tw-gradient-to:rgba(249,115,22,.5) var(--tw-gradient-to-position)}.to-orange-500\/55{--tw-gradient-to:rgba(249,115,22,.55) var(--tw-gradient-to-position)}.to-orange-500\/60{--tw-gradient-to:rgba(249,115,22,.6) var(--tw-gradient-to-position)}.to-orange-500\/65{--tw-gradient-to:rgba(249,115,22,.65) var(--tw-gradient-to-position)}.to-orange-500\/70{--tw-gradient-to:rgba(249,115,22,.7) var(--tw-gradient-to-position)}.to-orange-500\/75{--tw-gradient-to:rgba(249,115,22,.75) var(--tw-gradient-to-position)}.to-orange-500\/80{--tw-gradient-to:rgba(249,115,22,.8) var(--tw-gradient-to-position)}.to-orange-500\/85{--tw-gradient-to:rgba(249,115,22,.85) var(--tw-gradient-to-position)}.to-orange-500\/90{--tw-gradient-to:rgba(249,115,22,.9) var(--tw-gradient-to-position)}.to-orange-500\/95{--tw-gradient-to:rgba(249,115,22,.95) var(--tw-gradient-to-position)}.to-orange-950{--tw-gradient-to:#431407 var(--tw-gradient-to-position)}.to-orange-950\/0{--tw-gradient-to:rgba(67,20,7,0) var(--tw-gradient-to-position)}.to-orange-950\/10{--tw-gradient-to:rgba(67,20,7,.1) var(--tw-gradient-to-position)}.to-orange-950\/100{--tw-gradient-to:#431407 var(--tw-gradient-to-position)}.to-orange-950\/15{--tw-gradient-to:rgba(67,20,7,.15) var(--tw-gradient-to-position)}.to-orange-950\/20{--tw-gradient-to:rgba(67,20,7,.2) var(--tw-gradient-to-position)}.to-orange-950\/25{--tw-gradient-to:rgba(67,20,7,.25) var(--tw-gradient-to-position)}.to-orange-950\/30{--tw-gradient-to:rgba(67,20,7,.3) var(--tw-gradient-to-position)}.to-orange-950\/35{--tw-gradient-to:rgba(67,20,7,.35) var(--tw-gradient-to-position)}.to-orange-950\/40{--tw-gradient-to:rgba(67,20,7,.4) var(--tw-gradient-to-position)}.to-orange-950\/45{--tw-gradient-to:rgba(67,20,7,.45) var(--tw-gradient-to-position)}.to-orange-950\/5{--tw-gradient-to:rgba(67,20,7,.05) var(--tw-gradient-to-position)}.to-orange-950\/50{--tw-gradient-to:rgba(67,20,7,.5) var(--tw-gradient-to-position)}.to-orange-950\/55{--tw-gradient-to:rgba(67,20,7,.55) var(--tw-gradient-to-position)}.to-orange-950\/60{--tw-gradient-to:rgba(67,20,7,.6) var(--tw-gradient-to-position)}.to-orange-950\/65{--tw-gradient-to:rgba(67,20,7,.65) var(--tw-gradient-to-position)}.to-orange-950\/70{--tw-gradient-to:rgba(67,20,7,.7) var(--tw-gradient-to-position)}.to-orange-950\/75{--tw-gradient-to:rgba(67,20,7,.75) var(--tw-gradient-to-position)}.to-orange-950\/80{--tw-gradient-to:rgba(67,20,7,.8) var(--tw-gradient-to-position)}.to-orange-950\/85{--tw-gradient-to:rgba(67,20,7,.85) var(--tw-gradient-to-position)}.to-orange-950\/90{--tw-gradient-to:rgba(67,20,7,.9) var(--tw-gradient-to-position)}.to-orange-950\/95{--tw-gradient-to:rgba(67,20,7,.95) var(--tw-gradient-to-position)}.to-pink-100{--tw-gradient-to:#fce7f3 var(--tw-gradient-to-position)}.to-pink-100\/0{--tw-gradient-to:rgba(252,231,243,0) var(--tw-gradient-to-position)}.to-pink-100\/10{--tw-gradient-to:rgba(252,231,243,.1) var(--tw-gradient-to-position)}.to-pink-100\/100{--tw-gradient-to:#fce7f3 var(--tw-gradient-to-position)}.to-pink-100\/15{--tw-gradient-to:rgba(252,231,243,.15) var(--tw-gradient-to-position)}.to-pink-100\/20{--tw-gradient-to:rgba(252,231,243,.2) var(--tw-gradient-to-position)}.to-pink-100\/25{--tw-gradient-to:rgba(252,231,243,.25) var(--tw-gradient-to-position)}.to-pink-100\/30{--tw-gradient-to:rgba(252,231,243,.3) var(--tw-gradient-to-position)}.to-pink-100\/35{--tw-gradient-to:rgba(252,231,243,.35) var(--tw-gradient-to-position)}.to-pink-100\/40{--tw-gradient-to:rgba(252,231,243,.4) var(--tw-gradient-to-position)}.to-pink-100\/45{--tw-gradient-to:rgba(252,231,243,.45) var(--tw-gradient-to-position)}.to-pink-100\/5{--tw-gradient-to:rgba(252,231,243,.05) var(--tw-gradient-to-position)}.to-pink-100\/50{--tw-gradient-to:rgba(252,231,243,.5) var(--tw-gradient-to-position)}.to-pink-100\/55{--tw-gradient-to:rgba(252,231,243,.55) var(--tw-gradient-to-position)}.to-pink-100\/60{--tw-gradient-to:rgba(252,231,243,.6) var(--tw-gradient-to-position)}.to-pink-100\/65{--tw-gradient-to:rgba(252,231,243,.65) var(--tw-gradient-to-position)}.to-pink-100\/70{--tw-gradient-to:rgba(252,231,243,.7) var(--tw-gradient-to-position)}.to-pink-100\/75{--tw-gradient-to:rgba(252,231,243,.75) var(--tw-gradient-to-position)}.to-pink-100\/80{--tw-gradient-to:rgba(252,231,243,.8) var(--tw-gradient-to-position)}.to-pink-100\/85{--tw-gradient-to:rgba(252,231,243,.85) var(--tw-gradient-to-position)}.to-pink-100\/90{--tw-gradient-to:rgba(252,231,243,.9) var(--tw-gradient-to-position)}.to-pink-100\/95{--tw-gradient-to:rgba(252,231,243,.95) var(--tw-gradient-to-position)}.to-pink-200{--tw-gradient-to:#fbcfe8 var(--tw-gradient-to-position)}.to-pink-200\/0{--tw-gradient-to:rgba(251,207,232,0) var(--tw-gradient-to-position)}.to-pink-200\/10{--tw-gradient-to:rgba(251,207,232,.1) var(--tw-gradient-to-position)}.to-pink-200\/100{--tw-gradient-to:#fbcfe8 var(--tw-gradient-to-position)}.to-pink-200\/15{--tw-gradient-to:rgba(251,207,232,.15) var(--tw-gradient-to-position)}.to-pink-200\/20{--tw-gradient-to:rgba(251,207,232,.2) var(--tw-gradient-to-position)}.to-pink-200\/25{--tw-gradient-to:rgba(251,207,232,.25) var(--tw-gradient-to-position)}.to-pink-200\/30{--tw-gradient-to:rgba(251,207,232,.3) var(--tw-gradient-to-position)}.to-pink-200\/35{--tw-gradient-to:rgba(251,207,232,.35) var(--tw-gradient-to-position)}.to-pink-200\/40{--tw-gradient-to:rgba(251,207,232,.4) var(--tw-gradient-to-position)}.to-pink-200\/45{--tw-gradient-to:rgba(251,207,232,.45) var(--tw-gradient-to-position)}.to-pink-200\/5{--tw-gradient-to:rgba(251,207,232,.05) var(--tw-gradient-to-position)}.to-pink-200\/50{--tw-gradient-to:rgba(251,207,232,.5) var(--tw-gradient-to-position)}.to-pink-200\/55{--tw-gradient-to:rgba(251,207,232,.55) var(--tw-gradient-to-position)}.to-pink-200\/60{--tw-gradient-to:rgba(251,207,232,.6) var(--tw-gradient-to-position)}.to-pink-200\/65{--tw-gradient-to:rgba(251,207,232,.65) var(--tw-gradient-to-position)}.to-pink-200\/70{--tw-gradient-to:rgba(251,207,232,.7) var(--tw-gradient-to-position)}.to-pink-200\/75{--tw-gradient-to:rgba(251,207,232,.75) var(--tw-gradient-to-position)}.to-pink-200\/80{--tw-gradient-to:rgba(251,207,232,.8) var(--tw-gradient-to-position)}.to-pink-200\/85{--tw-gradient-to:rgba(251,207,232,.85) var(--tw-gradient-to-position)}.to-pink-200\/90{--tw-gradient-to:rgba(251,207,232,.9) var(--tw-gradient-to-position)}.to-pink-200\/95{--tw-gradient-to:rgba(251,207,232,.95) var(--tw-gradient-to-position)}.to-pink-50{--tw-gradient-to:#fdf2f8 var(--tw-gradient-to-position)}.to-pink-50\/0{--tw-gradient-to:rgba(253,242,248,0) var(--tw-gradient-to-position)}.to-pink-50\/10{--tw-gradient-to:rgba(253,242,248,.1) var(--tw-gradient-to-position)}.to-pink-50\/100{--tw-gradient-to:#fdf2f8 var(--tw-gradient-to-position)}.to-pink-50\/15{--tw-gradient-to:rgba(253,242,248,.15) var(--tw-gradient-to-position)}.to-pink-50\/20{--tw-gradient-to:rgba(253,242,248,.2) var(--tw-gradient-to-position)}.to-pink-50\/25{--tw-gradient-to:rgba(253,242,248,.25) var(--tw-gradient-to-position)}.to-pink-50\/30{--tw-gradient-to:rgba(253,242,248,.3) var(--tw-gradient-to-position)}.to-pink-50\/35{--tw-gradient-to:rgba(253,242,248,.35) var(--tw-gradient-to-position)}.to-pink-50\/40{--tw-gradient-to:rgba(253,242,248,.4) var(--tw-gradient-to-position)}.to-pink-50\/45{--tw-gradient-to:rgba(253,242,248,.45) var(--tw-gradient-to-position)}.to-pink-50\/5{--tw-gradient-to:rgba(253,242,248,.05) var(--tw-gradient-to-position)}.to-pink-50\/50{--tw-gradient-to:rgba(253,242,248,.5) var(--tw-gradient-to-position)}.to-pink-50\/55{--tw-gradient-to:rgba(253,242,248,.55) var(--tw-gradient-to-position)}.to-pink-50\/60{--tw-gradient-to:rgba(253,242,248,.6) var(--tw-gradient-to-position)}.to-pink-50\/65{--tw-gradient-to:rgba(253,242,248,.65) var(--tw-gradient-to-position)}.to-pink-50\/70{--tw-gradient-to:rgba(253,242,248,.7) var(--tw-gradient-to-position)}.to-pink-50\/75{--tw-gradient-to:rgba(253,242,248,.75) var(--tw-gradient-to-position)}.to-pink-50\/80{--tw-gradient-to:rgba(253,242,248,.8) var(--tw-gradient-to-position)}.to-pink-50\/85{--tw-gradient-to:rgba(253,242,248,.85) var(--tw-gradient-to-position)}.to-pink-50\/90{--tw-gradient-to:rgba(253,242,248,.9) var(--tw-gradient-to-position)}.to-pink-50\/95{--tw-gradient-to:rgba(253,242,248,.95) var(--tw-gradient-to-position)}.to-pink-500{--tw-gradient-to:#ec4899 var(--tw-gradient-to-position)}.to-pink-500\/0{--tw-gradient-to:rgba(236,72,153,0) var(--tw-gradient-to-position)}.to-pink-500\/10{--tw-gradient-to:rgba(236,72,153,.1) var(--tw-gradient-to-position)}.to-pink-500\/100{--tw-gradient-to:#ec4899 var(--tw-gradient-to-position)}.to-pink-500\/15{--tw-gradient-to:rgba(236,72,153,.15) var(--tw-gradient-to-position)}.to-pink-500\/20{--tw-gradient-to:rgba(236,72,153,.2) var(--tw-gradient-to-position)}.to-pink-500\/25{--tw-gradient-to:rgba(236,72,153,.25) var(--tw-gradient-to-position)}.to-pink-500\/30{--tw-gradient-to:rgba(236,72,153,.3) var(--tw-gradient-to-position)}.to-pink-500\/35{--tw-gradient-to:rgba(236,72,153,.35) var(--tw-gradient-to-position)}.to-pink-500\/40{--tw-gradient-to:rgba(236,72,153,.4) var(--tw-gradient-to-position)}.to-pink-500\/45{--tw-gradient-to:rgba(236,72,153,.45) var(--tw-gradient-to-position)}.to-pink-500\/5{--tw-gradient-to:rgba(236,72,153,.05) var(--tw-gradient-to-position)}.to-pink-500\/50{--tw-gradient-to:rgba(236,72,153,.5) var(--tw-gradient-to-position)}.to-pink-500\/55{--tw-gradient-to:rgba(236,72,153,.55) var(--tw-gradient-to-position)}.to-pink-500\/60{--tw-gradient-to:rgba(236,72,153,.6) var(--tw-gradient-to-position)}.to-pink-500\/65{--tw-gradient-to:rgba(236,72,153,.65) var(--tw-gradient-to-position)}.to-pink-500\/70{--tw-gradient-to:rgba(236,72,153,.7) var(--tw-gradient-to-position)}.to-pink-500\/75{--tw-gradient-to:rgba(236,72,153,.75) var(--tw-gradient-to-position)}.to-pink-500\/80{--tw-gradient-to:rgba(236,72,153,.8) var(--tw-gradient-to-position)}.to-pink-500\/85{--tw-gradient-to:rgba(236,72,153,.85) var(--tw-gradient-to-position)}.to-pink-500\/90{--tw-gradient-to:rgba(236,72,153,.9) var(--tw-gradient-to-position)}.to-pink-500\/95{--tw-gradient-to:rgba(236,72,153,.95) var(--tw-gradient-to-position)}.to-pink-950{--tw-gradient-to:#500724 var(--tw-gradient-to-position)}.to-pink-950\/0{--tw-gradient-to:rgba(80,7,36,0) var(--tw-gradient-to-position)}.to-pink-950\/10{--tw-gradient-to:rgba(80,7,36,.1) var(--tw-gradient-to-position)}.to-pink-950\/100{--tw-gradient-to:#500724 var(--tw-gradient-to-position)}.to-pink-950\/15{--tw-gradient-to:rgba(80,7,36,.15) var(--tw-gradient-to-position)}.to-pink-950\/20{--tw-gradient-to:rgba(80,7,36,.2) var(--tw-gradient-to-position)}.to-pink-950\/25{--tw-gradient-to:rgba(80,7,36,.25) var(--tw-gradient-to-position)}.to-pink-950\/30{--tw-gradient-to:rgba(80,7,36,.3) var(--tw-gradient-to-position)}.to-pink-950\/35{--tw-gradient-to:rgba(80,7,36,.35) var(--tw-gradient-to-position)}.to-pink-950\/40{--tw-gradient-to:rgba(80,7,36,.4) var(--tw-gradient-to-position)}.to-pink-950\/45{--tw-gradient-to:rgba(80,7,36,.45) var(--tw-gradient-to-position)}.to-pink-950\/5{--tw-gradient-to:rgba(80,7,36,.05) var(--tw-gradient-to-position)}.to-pink-950\/50{--tw-gradient-to:rgba(80,7,36,.5) var(--tw-gradient-to-position)}.to-pink-950\/55{--tw-gradient-to:rgba(80,7,36,.55) var(--tw-gradient-to-position)}.to-pink-950\/60{--tw-gradient-to:rgba(80,7,36,.6) var(--tw-gradient-to-position)}.to-pink-950\/65{--tw-gradient-to:rgba(80,7,36,.65) var(--tw-gradient-to-position)}.to-pink-950\/70{--tw-gradient-to:rgba(80,7,36,.7) var(--tw-gradient-to-position)}.to-pink-950\/75{--tw-gradient-to:rgba(80,7,36,.75) var(--tw-gradient-to-position)}.to-pink-950\/80{--tw-gradient-to:rgba(80,7,36,.8) var(--tw-gradient-to-position)}.to-pink-950\/85{--tw-gradient-to:rgba(80,7,36,.85) var(--tw-gradient-to-position)}.to-pink-950\/90{--tw-gradient-to:rgba(80,7,36,.9) var(--tw-gradient-to-position)}.to-pink-950\/95{--tw-gradient-to:rgba(80,7,36,.95) var(--tw-gradient-to-position)}.to-purple-100{--tw-gradient-to:#f3e8ff var(--tw-gradient-to-position)}.to-purple-100\/0{--tw-gradient-to:rgba(243,232,255,0) var(--tw-gradient-to-position)}.to-purple-100\/10{--tw-gradient-to:rgba(243,232,255,.1) var(--tw-gradient-to-position)}.to-purple-100\/100{--tw-gradient-to:#f3e8ff var(--tw-gradient-to-position)}.to-purple-100\/15{--tw-gradient-to:rgba(243,232,255,.15) var(--tw-gradient-to-position)}.to-purple-100\/20{--tw-gradient-to:rgba(243,232,255,.2) var(--tw-gradient-to-position)}.to-purple-100\/25{--tw-gradient-to:rgba(243,232,255,.25) var(--tw-gradient-to-position)}.to-purple-100\/30{--tw-gradient-to:rgba(243,232,255,.3) var(--tw-gradient-to-position)}.to-purple-100\/35{--tw-gradient-to:rgba(243,232,255,.35) var(--tw-gradient-to-position)}.to-purple-100\/40{--tw-gradient-to:rgba(243,232,255,.4) var(--tw-gradient-to-position)}.to-purple-100\/45{--tw-gradient-to:rgba(243,232,255,.45) var(--tw-gradient-to-position)}.to-purple-100\/5{--tw-gradient-to:rgba(243,232,255,.05) var(--tw-gradient-to-position)}.to-purple-100\/50{--tw-gradient-to:rgba(243,232,255,.5) var(--tw-gradient-to-position)}.to-purple-100\/55{--tw-gradient-to:rgba(243,232,255,.55) var(--tw-gradient-to-position)}.to-purple-100\/60{--tw-gradient-to:rgba(243,232,255,.6) var(--tw-gradient-to-position)}.to-purple-100\/65{--tw-gradient-to:rgba(243,232,255,.65) var(--tw-gradient-to-position)}.to-purple-100\/70{--tw-gradient-to:rgba(243,232,255,.7) var(--tw-gradient-to-position)}.to-purple-100\/75{--tw-gradient-to:rgba(243,232,255,.75) var(--tw-gradient-to-position)}.to-purple-100\/80{--tw-gradient-to:rgba(243,232,255,.8) var(--tw-gradient-to-position)}.to-purple-100\/85{--tw-gradient-to:rgba(243,232,255,.85) var(--tw-gradient-to-position)}.to-purple-100\/90{--tw-gradient-to:rgba(243,232,255,.9) var(--tw-gradient-to-position)}.to-purple-100\/95{--tw-gradient-to:rgba(243,232,255,.95) var(--tw-gradient-to-position)}.to-purple-200{--tw-gradient-to:#e9d5ff var(--tw-gradient-to-position)}.to-purple-200\/0{--tw-gradient-to:rgba(233,213,255,0) var(--tw-gradient-to-position)}.to-purple-200\/10{--tw-gradient-to:rgba(233,213,255,.1) var(--tw-gradient-to-position)}.to-purple-200\/100{--tw-gradient-to:#e9d5ff var(--tw-gradient-to-position)}.to-purple-200\/15{--tw-gradient-to:rgba(233,213,255,.15) var(--tw-gradient-to-position)}.to-purple-200\/20{--tw-gradient-to:rgba(233,213,255,.2) var(--tw-gradient-to-position)}.to-purple-200\/25{--tw-gradient-to:rgba(233,213,255,.25) var(--tw-gradient-to-position)}.to-purple-200\/30{--tw-gradient-to:rgba(233,213,255,.3) var(--tw-gradient-to-position)}.to-purple-200\/35{--tw-gradient-to:rgba(233,213,255,.35) var(--tw-gradient-to-position)}.to-purple-200\/40{--tw-gradient-to:rgba(233,213,255,.4) var(--tw-gradient-to-position)}.to-purple-200\/45{--tw-gradient-to:rgba(233,213,255,.45) var(--tw-gradient-to-position)}.to-purple-200\/5{--tw-gradient-to:rgba(233,213,255,.05) var(--tw-gradient-to-position)}.to-purple-200\/50{--tw-gradient-to:rgba(233,213,255,.5) var(--tw-gradient-to-position)}.to-purple-200\/55{--tw-gradient-to:rgba(233,213,255,.55) var(--tw-gradient-to-position)}.to-purple-200\/60{--tw-gradient-to:rgba(233,213,255,.6) var(--tw-gradient-to-position)}.to-purple-200\/65{--tw-gradient-to:rgba(233,213,255,.65) var(--tw-gradient-to-position)}.to-purple-200\/70{--tw-gradient-to:rgba(233,213,255,.7) var(--tw-gradient-to-position)}.to-purple-200\/75{--tw-gradient-to:rgba(233,213,255,.75) var(--tw-gradient-to-position)}.to-purple-200\/80{--tw-gradient-to:rgba(233,213,255,.8) var(--tw-gradient-to-position)}.to-purple-200\/85{--tw-gradient-to:rgba(233,213,255,.85) var(--tw-gradient-to-position)}.to-purple-200\/90{--tw-gradient-to:rgba(233,213,255,.9) var(--tw-gradient-to-position)}.to-purple-200\/95{--tw-gradient-to:rgba(233,213,255,.95) var(--tw-gradient-to-position)}.to-purple-50{--tw-gradient-to:#faf5ff var(--tw-gradient-to-position)}.to-purple-50\/0{--tw-gradient-to:rgba(250,245,255,0) var(--tw-gradient-to-position)}.to-purple-50\/10{--tw-gradient-to:rgba(250,245,255,.1) var(--tw-gradient-to-position)}.to-purple-50\/100{--tw-gradient-to:#faf5ff var(--tw-gradient-to-position)}.to-purple-50\/15{--tw-gradient-to:rgba(250,245,255,.15) var(--tw-gradient-to-position)}.to-purple-50\/20{--tw-gradient-to:rgba(250,245,255,.2) var(--tw-gradient-to-position)}.to-purple-50\/25{--tw-gradient-to:rgba(250,245,255,.25) var(--tw-gradient-to-position)}.to-purple-50\/30{--tw-gradient-to:rgba(250,245,255,.3) var(--tw-gradient-to-position)}.to-purple-50\/35{--tw-gradient-to:rgba(250,245,255,.35) var(--tw-gradient-to-position)}.to-purple-50\/40{--tw-gradient-to:rgba(250,245,255,.4) var(--tw-gradient-to-position)}.to-purple-50\/45{--tw-gradient-to:rgba(250,245,255,.45) var(--tw-gradient-to-position)}.to-purple-50\/5{--tw-gradient-to:rgba(250,245,255,.05) var(--tw-gradient-to-position)}.to-purple-50\/50{--tw-gradient-to:rgba(250,245,255,.5) var(--tw-gradient-to-position)}.to-purple-50\/55{--tw-gradient-to:rgba(250,245,255,.55) var(--tw-gradient-to-position)}.to-purple-50\/60{--tw-gradient-to:rgba(250,245,255,.6) var(--tw-gradient-to-position)}.to-purple-50\/65{--tw-gradient-to:rgba(250,245,255,.65) var(--tw-gradient-to-position)}.to-purple-50\/70{--tw-gradient-to:rgba(250,245,255,.7) var(--tw-gradient-to-position)}.to-purple-50\/75{--tw-gradient-to:rgba(250,245,255,.75) var(--tw-gradient-to-position)}.to-purple-50\/80{--tw-gradient-to:rgba(250,245,255,.8) var(--tw-gradient-to-position)}.to-purple-50\/85{--tw-gradient-to:rgba(250,245,255,.85) var(--tw-gradient-to-position)}.to-purple-50\/90{--tw-gradient-to:rgba(250,245,255,.9) var(--tw-gradient-to-position)}.to-purple-50\/95{--tw-gradient-to:rgba(250,245,255,.95) var(--tw-gradient-to-position)}.to-purple-500{--tw-gradient-to:#a855f7 var(--tw-gradient-to-position)}.to-purple-500\/0{--tw-gradient-to:rgba(168,85,247,0) var(--tw-gradient-to-position)}.to-purple-500\/10{--tw-gradient-to:rgba(168,85,247,.1) var(--tw-gradient-to-position)}.to-purple-500\/100{--tw-gradient-to:#a855f7 var(--tw-gradient-to-position)}.to-purple-500\/15{--tw-gradient-to:rgba(168,85,247,.15) var(--tw-gradient-to-position)}.to-purple-500\/20{--tw-gradient-to:rgba(168,85,247,.2) var(--tw-gradient-to-position)}.to-purple-500\/25{--tw-gradient-to:rgba(168,85,247,.25) var(--tw-gradient-to-position)}.to-purple-500\/30{--tw-gradient-to:rgba(168,85,247,.3) var(--tw-gradient-to-position)}.to-purple-500\/35{--tw-gradient-to:rgba(168,85,247,.35) var(--tw-gradient-to-position)}.to-purple-500\/40{--tw-gradient-to:rgba(168,85,247,.4) var(--tw-gradient-to-position)}.to-purple-500\/45{--tw-gradient-to:rgba(168,85,247,.45) var(--tw-gradient-to-position)}.to-purple-500\/5{--tw-gradient-to:rgba(168,85,247,.05) var(--tw-gradient-to-position)}.to-purple-500\/50{--tw-gradient-to:rgba(168,85,247,.5) var(--tw-gradient-to-position)}.to-purple-500\/55{--tw-gradient-to:rgba(168,85,247,.55) var(--tw-gradient-to-position)}.to-purple-500\/60{--tw-gradient-to:rgba(168,85,247,.6) var(--tw-gradient-to-position)}.to-purple-500\/65{--tw-gradient-to:rgba(168,85,247,.65) var(--tw-gradient-to-position)}.to-purple-500\/70{--tw-gradient-to:rgba(168,85,247,.7) var(--tw-gradient-to-position)}.to-purple-500\/75{--tw-gradient-to:rgba(168,85,247,.75) var(--tw-gradient-to-position)}.to-purple-500\/80{--tw-gradient-to:rgba(168,85,247,.8) var(--tw-gradient-to-position)}.to-purple-500\/85{--tw-gradient-to:rgba(168,85,247,.85) var(--tw-gradient-to-position)}.to-purple-500\/90{--tw-gradient-to:rgba(168,85,247,.9) var(--tw-gradient-to-position)}.to-purple-500\/95{--tw-gradient-to:rgba(168,85,247,.95) var(--tw-gradient-to-position)}.to-purple-950{--tw-gradient-to:#3b0764 var(--tw-gradient-to-position)}.to-purple-950\/0{--tw-gradient-to:rgba(59,7,100,0) var(--tw-gradient-to-position)}.to-purple-950\/10{--tw-gradient-to:rgba(59,7,100,.1) var(--tw-gradient-to-position)}.to-purple-950\/100{--tw-gradient-to:#3b0764 var(--tw-gradient-to-position)}.to-purple-950\/15{--tw-gradient-to:rgba(59,7,100,.15) var(--tw-gradient-to-position)}.to-purple-950\/20{--tw-gradient-to:rgba(59,7,100,.2) var(--tw-gradient-to-position)}.to-purple-950\/25{--tw-gradient-to:rgba(59,7,100,.25) var(--tw-gradient-to-position)}.to-purple-950\/30{--tw-gradient-to:rgba(59,7,100,.3) var(--tw-gradient-to-position)}.to-purple-950\/35{--tw-gradient-to:rgba(59,7,100,.35) var(--tw-gradient-to-position)}.to-purple-950\/40{--tw-gradient-to:rgba(59,7,100,.4) var(--tw-gradient-to-position)}.to-purple-950\/45{--tw-gradient-to:rgba(59,7,100,.45) var(--tw-gradient-to-position)}.to-purple-950\/5{--tw-gradient-to:rgba(59,7,100,.05) var(--tw-gradient-to-position)}.to-purple-950\/50{--tw-gradient-to:rgba(59,7,100,.5) var(--tw-gradient-to-position)}.to-purple-950\/55{--tw-gradient-to:rgba(59,7,100,.55) var(--tw-gradient-to-position)}.to-purple-950\/60{--tw-gradient-to:rgba(59,7,100,.6) var(--tw-gradient-to-position)}.to-purple-950\/65{--tw-gradient-to:rgba(59,7,100,.65) var(--tw-gradient-to-position)}.to-purple-950\/70{--tw-gradient-to:rgba(59,7,100,.7) var(--tw-gradient-to-position)}.to-purple-950\/75{--tw-gradient-to:rgba(59,7,100,.75) var(--tw-gradient-to-position)}.to-purple-950\/80{--tw-gradient-to:rgba(59,7,100,.8) var(--tw-gradient-to-position)}.to-purple-950\/85{--tw-gradient-to:rgba(59,7,100,.85) var(--tw-gradient-to-position)}.to-purple-950\/90{--tw-gradient-to:rgba(59,7,100,.9) var(--tw-gradient-to-position)}.to-purple-950\/95{--tw-gradient-to:rgba(59,7,100,.95) var(--tw-gradient-to-position)}.to-red-100{--tw-gradient-to:#fee2e2 var(--tw-gradient-to-position)}.to-red-100\/0{--tw-gradient-to:hsla(0,93%,94%,0) var(--tw-gradient-to-position)}.to-red-100\/10{--tw-gradient-to:hsla(0,93%,94%,.1) var(--tw-gradient-to-position)}.to-red-100\/100{--tw-gradient-to:#fee2e2 var(--tw-gradient-to-position)}.to-red-100\/15{--tw-gradient-to:hsla(0,93%,94%,.15) var(--tw-gradient-to-position)}.to-red-100\/20{--tw-gradient-to:hsla(0,93%,94%,.2) var(--tw-gradient-to-position)}.to-red-100\/25{--tw-gradient-to:hsla(0,93%,94%,.25) var(--tw-gradient-to-position)}.to-red-100\/30{--tw-gradient-to:hsla(0,93%,94%,.3) var(--tw-gradient-to-position)}.to-red-100\/35{--tw-gradient-to:hsla(0,93%,94%,.35) var(--tw-gradient-to-position)}.to-red-100\/40{--tw-gradient-to:hsla(0,93%,94%,.4) var(--tw-gradient-to-position)}.to-red-100\/45{--tw-gradient-to:hsla(0,93%,94%,.45) var(--tw-gradient-to-position)}.to-red-100\/5{--tw-gradient-to:hsla(0,93%,94%,.05) var(--tw-gradient-to-position)}.to-red-100\/50{--tw-gradient-to:hsla(0,93%,94%,.5) var(--tw-gradient-to-position)}.to-red-100\/55{--tw-gradient-to:hsla(0,93%,94%,.55) var(--tw-gradient-to-position)}.to-red-100\/60{--tw-gradient-to:hsla(0,93%,94%,.6) var(--tw-gradient-to-position)}.to-red-100\/65{--tw-gradient-to:hsla(0,93%,94%,.65) var(--tw-gradient-to-position)}.to-red-100\/70{--tw-gradient-to:hsla(0,93%,94%,.7) var(--tw-gradient-to-position)}.to-red-100\/75{--tw-gradient-to:hsla(0,93%,94%,.75) var(--tw-gradient-to-position)}.to-red-100\/80{--tw-gradient-to:hsla(0,93%,94%,.8) var(--tw-gradient-to-position)}.to-red-100\/85{--tw-gradient-to:hsla(0,93%,94%,.85) var(--tw-gradient-to-position)}.to-red-100\/90{--tw-gradient-to:hsla(0,93%,94%,.9) var(--tw-gradient-to-position)}.to-red-100\/95{--tw-gradient-to:hsla(0,93%,94%,.95) var(--tw-gradient-to-position)}.to-red-200{--tw-gradient-to:#fecaca var(--tw-gradient-to-position)}.to-red-200\/0{--tw-gradient-to:hsla(0,96%,89%,0) var(--tw-gradient-to-position)}.to-red-200\/10{--tw-gradient-to:hsla(0,96%,89%,.1) var(--tw-gradient-to-position)}.to-red-200\/100{--tw-gradient-to:#fecaca var(--tw-gradient-to-position)}.to-red-200\/15{--tw-gradient-to:hsla(0,96%,89%,.15) var(--tw-gradient-to-position)}.to-red-200\/20{--tw-gradient-to:hsla(0,96%,89%,.2) var(--tw-gradient-to-position)}.to-red-200\/25{--tw-gradient-to:hsla(0,96%,89%,.25) var(--tw-gradient-to-position)}.to-red-200\/30{--tw-gradient-to:hsla(0,96%,89%,.3) var(--tw-gradient-to-position)}.to-red-200\/35{--tw-gradient-to:hsla(0,96%,89%,.35) var(--tw-gradient-to-position)}.to-red-200\/40{--tw-gradient-to:hsla(0,96%,89%,.4) var(--tw-gradient-to-position)}.to-red-200\/45{--tw-gradient-to:hsla(0,96%,89%,.45) var(--tw-gradient-to-position)}.to-red-200\/5{--tw-gradient-to:hsla(0,96%,89%,.05) var(--tw-gradient-to-position)}.to-red-200\/50{--tw-gradient-to:hsla(0,96%,89%,.5) var(--tw-gradient-to-position)}.to-red-200\/55{--tw-gradient-to:hsla(0,96%,89%,.55) var(--tw-gradient-to-position)}.to-red-200\/60{--tw-gradient-to:hsla(0,96%,89%,.6) var(--tw-gradient-to-position)}.to-red-200\/65{--tw-gradient-to:hsla(0,96%,89%,.65) var(--tw-gradient-to-position)}.to-red-200\/70{--tw-gradient-to:hsla(0,96%,89%,.7) var(--tw-gradient-to-position)}.to-red-200\/75{--tw-gradient-to:hsla(0,96%,89%,.75) var(--tw-gradient-to-position)}.to-red-200\/80{--tw-gradient-to:hsla(0,96%,89%,.8) var(--tw-gradient-to-position)}.to-red-200\/85{--tw-gradient-to:hsla(0,96%,89%,.85) var(--tw-gradient-to-position)}.to-red-200\/90{--tw-gradient-to:hsla(0,96%,89%,.9) var(--tw-gradient-to-position)}.to-red-200\/95{--tw-gradient-to:hsla(0,96%,89%,.95) var(--tw-gradient-to-position)}.to-red-50{--tw-gradient-to:#fef2f2 var(--tw-gradient-to-position)}.to-red-50\/0{--tw-gradient-to:hsla(0,86%,97%,0) var(--tw-gradient-to-position)}.to-red-50\/10{--tw-gradient-to:hsla(0,86%,97%,.1) var(--tw-gradient-to-position)}.to-red-50\/100{--tw-gradient-to:#fef2f2 var(--tw-gradient-to-position)}.to-red-50\/15{--tw-gradient-to:hsla(0,86%,97%,.15) var(--tw-gradient-to-position)}.to-red-50\/20{--tw-gradient-to:hsla(0,86%,97%,.2) var(--tw-gradient-to-position)}.to-red-50\/25{--tw-gradient-to:hsla(0,86%,97%,.25) var(--tw-gradient-to-position)}.to-red-50\/30{--tw-gradient-to:hsla(0,86%,97%,.3) var(--tw-gradient-to-position)}.to-red-50\/35{--tw-gradient-to:hsla(0,86%,97%,.35) var(--tw-gradient-to-position)}.to-red-50\/40{--tw-gradient-to:hsla(0,86%,97%,.4) var(--tw-gradient-to-position)}.to-red-50\/45{--tw-gradient-to:hsla(0,86%,97%,.45) var(--tw-gradient-to-position)}.to-red-50\/5{--tw-gradient-to:hsla(0,86%,97%,.05) var(--tw-gradient-to-position)}.to-red-50\/50{--tw-gradient-to:hsla(0,86%,97%,.5) var(--tw-gradient-to-position)}.to-red-50\/55{--tw-gradient-to:hsla(0,86%,97%,.55) var(--tw-gradient-to-position)}.to-red-50\/60{--tw-gradient-to:hsla(0,86%,97%,.6) var(--tw-gradient-to-position)}.to-red-50\/65{--tw-gradient-to:hsla(0,86%,97%,.65) var(--tw-gradient-to-position)}.to-red-50\/70{--tw-gradient-to:hsla(0,86%,97%,.7) var(--tw-gradient-to-position)}.to-red-50\/75{--tw-gradient-to:hsla(0,86%,97%,.75) var(--tw-gradient-to-position)}.to-red-50\/80{--tw-gradient-to:hsla(0,86%,97%,.8) var(--tw-gradient-to-position)}.to-red-50\/85{--tw-gradient-to:hsla(0,86%,97%,.85) var(--tw-gradient-to-position)}.to-red-50\/90{--tw-gradient-to:hsla(0,86%,97%,.9) var(--tw-gradient-to-position)}.to-red-50\/95{--tw-gradient-to:hsla(0,86%,97%,.95) var(--tw-gradient-to-position)}.to-red-500{--tw-gradient-to:#ef4444 var(--tw-gradient-to-position)}.to-red-500\/0{--tw-gradient-to:rgba(239,68,68,0) var(--tw-gradient-to-position)}.to-red-500\/10{--tw-gradient-to:rgba(239,68,68,.1) var(--tw-gradient-to-position)}.to-red-500\/100{--tw-gradient-to:#ef4444 var(--tw-gradient-to-position)}.to-red-500\/15{--tw-gradient-to:rgba(239,68,68,.15) var(--tw-gradient-to-position)}.to-red-500\/20{--tw-gradient-to:rgba(239,68,68,.2) var(--tw-gradient-to-position)}.to-red-500\/25{--tw-gradient-to:rgba(239,68,68,.25) var(--tw-gradient-to-position)}.to-red-500\/30{--tw-gradient-to:rgba(239,68,68,.3) var(--tw-gradient-to-position)}.to-red-500\/35{--tw-gradient-to:rgba(239,68,68,.35) var(--tw-gradient-to-position)}.to-red-500\/40{--tw-gradient-to:rgba(239,68,68,.4) var(--tw-gradient-to-position)}.to-red-500\/45{--tw-gradient-to:rgba(239,68,68,.45) var(--tw-gradient-to-position)}.to-red-500\/5{--tw-gradient-to:rgba(239,68,68,.05) var(--tw-gradient-to-position)}.to-red-500\/50{--tw-gradient-to:rgba(239,68,68,.5) var(--tw-gradient-to-position)}.to-red-500\/55{--tw-gradient-to:rgba(239,68,68,.55) var(--tw-gradient-to-position)}.to-red-500\/60{--tw-gradient-to:rgba(239,68,68,.6) var(--tw-gradient-to-position)}.to-red-500\/65{--tw-gradient-to:rgba(239,68,68,.65) var(--tw-gradient-to-position)}.to-red-500\/70{--tw-gradient-to:rgba(239,68,68,.7) var(--tw-gradient-to-position)}.to-red-500\/75{--tw-gradient-to:rgba(239,68,68,.75) var(--tw-gradient-to-position)}.to-red-500\/80{--tw-gradient-to:rgba(239,68,68,.8) var(--tw-gradient-to-position)}.to-red-500\/85{--tw-gradient-to:rgba(239,68,68,.85) var(--tw-gradient-to-position)}.to-red-500\/90{--tw-gradient-to:rgba(239,68,68,.9) var(--tw-gradient-to-position)}.to-red-500\/95{--tw-gradient-to:rgba(239,68,68,.95) var(--tw-gradient-to-position)}.to-red-950{--tw-gradient-to:#450a0a var(--tw-gradient-to-position)}.to-red-950\/0{--tw-gradient-to:rgba(69,10,10,0) var(--tw-gradient-to-position)}.to-red-950\/10{--tw-gradient-to:rgba(69,10,10,.1) var(--tw-gradient-to-position)}.to-red-950\/100{--tw-gradient-to:#450a0a var(--tw-gradient-to-position)}.to-red-950\/15{--tw-gradient-to:rgba(69,10,10,.15) var(--tw-gradient-to-position)}.to-red-950\/20{--tw-gradient-to:rgba(69,10,10,.2) var(--tw-gradient-to-position)}.to-red-950\/25{--tw-gradient-to:rgba(69,10,10,.25) var(--tw-gradient-to-position)}.to-red-950\/30{--tw-gradient-to:rgba(69,10,10,.3) var(--tw-gradient-to-position)}.to-red-950\/35{--tw-gradient-to:rgba(69,10,10,.35) var(--tw-gradient-to-position)}.to-red-950\/40{--tw-gradient-to:rgba(69,10,10,.4) var(--tw-gradient-to-position)}.to-red-950\/45{--tw-gradient-to:rgba(69,10,10,.45) var(--tw-gradient-to-position)}.to-red-950\/5{--tw-gradient-to:rgba(69,10,10,.05) var(--tw-gradient-to-position)}.to-red-950\/50{--tw-gradient-to:rgba(69,10,10,.5) var(--tw-gradient-to-position)}.to-red-950\/55{--tw-gradient-to:rgba(69,10,10,.55) var(--tw-gradient-to-position)}.to-red-950\/60{--tw-gradient-to:rgba(69,10,10,.6) var(--tw-gradient-to-position)}.to-red-950\/65{--tw-gradient-to:rgba(69,10,10,.65) var(--tw-gradient-to-position)}.to-red-950\/70{--tw-gradient-to:rgba(69,10,10,.7) var(--tw-gradient-to-position)}.to-red-950\/75{--tw-gradient-to:rgba(69,10,10,.75) var(--tw-gradient-to-position)}.to-red-950\/80{--tw-gradient-to:rgba(69,10,10,.8) var(--tw-gradient-to-position)}.to-red-950\/85{--tw-gradient-to:rgba(69,10,10,.85) var(--tw-gradient-to-position)}.to-red-950\/90{--tw-gradient-to:rgba(69,10,10,.9) var(--tw-gradient-to-position)}.to-red-950\/95{--tw-gradient-to:rgba(69,10,10,.95) var(--tw-gradient-to-position)}.to-rose-100{--tw-gradient-to:#ffe4e6 var(--tw-gradient-to-position)}.to-rose-100\/0{--tw-gradient-to:rgba(255,228,230,0) var(--tw-gradient-to-position)}.to-rose-100\/10{--tw-gradient-to:rgba(255,228,230,.1) var(--tw-gradient-to-position)}.to-rose-100\/100{--tw-gradient-to:#ffe4e6 var(--tw-gradient-to-position)}.to-rose-100\/15{--tw-gradient-to:rgba(255,228,230,.15) var(--tw-gradient-to-position)}.to-rose-100\/20{--tw-gradient-to:rgba(255,228,230,.2) var(--tw-gradient-to-position)}.to-rose-100\/25{--tw-gradient-to:rgba(255,228,230,.25) var(--tw-gradient-to-position)}.to-rose-100\/30{--tw-gradient-to:rgba(255,228,230,.3) var(--tw-gradient-to-position)}.to-rose-100\/35{--tw-gradient-to:rgba(255,228,230,.35) var(--tw-gradient-to-position)}.to-rose-100\/40{--tw-gradient-to:rgba(255,228,230,.4) var(--tw-gradient-to-position)}.to-rose-100\/45{--tw-gradient-to:rgba(255,228,230,.45) var(--tw-gradient-to-position)}.to-rose-100\/5{--tw-gradient-to:rgba(255,228,230,.05) var(--tw-gradient-to-position)}.to-rose-100\/50{--tw-gradient-to:rgba(255,228,230,.5) var(--tw-gradient-to-position)}.to-rose-100\/55{--tw-gradient-to:rgba(255,228,230,.55) var(--tw-gradient-to-position)}.to-rose-100\/60{--tw-gradient-to:rgba(255,228,230,.6) var(--tw-gradient-to-position)}.to-rose-100\/65{--tw-gradient-to:rgba(255,228,230,.65) var(--tw-gradient-to-position)}.to-rose-100\/70{--tw-gradient-to:rgba(255,228,230,.7) var(--tw-gradient-to-position)}.to-rose-100\/75{--tw-gradient-to:rgba(255,228,230,.75) var(--tw-gradient-to-position)}.to-rose-100\/80{--tw-gradient-to:rgba(255,228,230,.8) var(--tw-gradient-to-position)}.to-rose-100\/85{--tw-gradient-to:rgba(255,228,230,.85) var(--tw-gradient-to-position)}.to-rose-100\/90{--tw-gradient-to:rgba(255,228,230,.9) var(--tw-gradient-to-position)}.to-rose-100\/95{--tw-gradient-to:rgba(255,228,230,.95) var(--tw-gradient-to-position)}.to-rose-200{--tw-gradient-to:#fecdd3 var(--tw-gradient-to-position)}.to-rose-200\/0{--tw-gradient-to:rgba(254,205,211,0) var(--tw-gradient-to-position)}.to-rose-200\/10{--tw-gradient-to:rgba(254,205,211,.1) var(--tw-gradient-to-position)}.to-rose-200\/100{--tw-gradient-to:#fecdd3 var(--tw-gradient-to-position)}.to-rose-200\/15{--tw-gradient-to:rgba(254,205,211,.15) var(--tw-gradient-to-position)}.to-rose-200\/20{--tw-gradient-to:rgba(254,205,211,.2) var(--tw-gradient-to-position)}.to-rose-200\/25{--tw-gradient-to:rgba(254,205,211,.25) var(--tw-gradient-to-position)}.to-rose-200\/30{--tw-gradient-to:rgba(254,205,211,.3) var(--tw-gradient-to-position)}.to-rose-200\/35{--tw-gradient-to:rgba(254,205,211,.35) var(--tw-gradient-to-position)}.to-rose-200\/40{--tw-gradient-to:rgba(254,205,211,.4) var(--tw-gradient-to-position)}.to-rose-200\/45{--tw-gradient-to:rgba(254,205,211,.45) var(--tw-gradient-to-position)}.to-rose-200\/5{--tw-gradient-to:rgba(254,205,211,.05) var(--tw-gradient-to-position)}.to-rose-200\/50{--tw-gradient-to:rgba(254,205,211,.5) var(--tw-gradient-to-position)}.to-rose-200\/55{--tw-gradient-to:rgba(254,205,211,.55) var(--tw-gradient-to-position)}.to-rose-200\/60{--tw-gradient-to:rgba(254,205,211,.6) var(--tw-gradient-to-position)}.to-rose-200\/65{--tw-gradient-to:rgba(254,205,211,.65) var(--tw-gradient-to-position)}.to-rose-200\/70{--tw-gradient-to:rgba(254,205,211,.7) var(--tw-gradient-to-position)}.to-rose-200\/75{--tw-gradient-to:rgba(254,205,211,.75) var(--tw-gradient-to-position)}.to-rose-200\/80{--tw-gradient-to:rgba(254,205,211,.8) var(--tw-gradient-to-position)}.to-rose-200\/85{--tw-gradient-to:rgba(254,205,211,.85) var(--tw-gradient-to-position)}.to-rose-200\/90{--tw-gradient-to:rgba(254,205,211,.9) var(--tw-gradient-to-position)}.to-rose-200\/95{--tw-gradient-to:rgba(254,205,211,.95) var(--tw-gradient-to-position)}.to-rose-50{--tw-gradient-to:#fff1f2 var(--tw-gradient-to-position)}.to-rose-50\/0{--tw-gradient-to:rgba(255,241,242,0) var(--tw-gradient-to-position)}.to-rose-50\/10{--tw-gradient-to:rgba(255,241,242,.1) var(--tw-gradient-to-position)}.to-rose-50\/100{--tw-gradient-to:#fff1f2 var(--tw-gradient-to-position)}.to-rose-50\/15{--tw-gradient-to:rgba(255,241,242,.15) var(--tw-gradient-to-position)}.to-rose-50\/20{--tw-gradient-to:rgba(255,241,242,.2) var(--tw-gradient-to-position)}.to-rose-50\/25{--tw-gradient-to:rgba(255,241,242,.25) var(--tw-gradient-to-position)}.to-rose-50\/30{--tw-gradient-to:rgba(255,241,242,.3) var(--tw-gradient-to-position)}.to-rose-50\/35{--tw-gradient-to:rgba(255,241,242,.35) var(--tw-gradient-to-position)}.to-rose-50\/40{--tw-gradient-to:rgba(255,241,242,.4) var(--tw-gradient-to-position)}.to-rose-50\/45{--tw-gradient-to:rgba(255,241,242,.45) var(--tw-gradient-to-position)}.to-rose-50\/5{--tw-gradient-to:rgba(255,241,242,.05) var(--tw-gradient-to-position)}.to-rose-50\/50{--tw-gradient-to:rgba(255,241,242,.5) var(--tw-gradient-to-position)}.to-rose-50\/55{--tw-gradient-to:rgba(255,241,242,.55) var(--tw-gradient-to-position)}.to-rose-50\/60{--tw-gradient-to:rgba(255,241,242,.6) var(--tw-gradient-to-position)}.to-rose-50\/65{--tw-gradient-to:rgba(255,241,242,.65) var(--tw-gradient-to-position)}.to-rose-50\/70{--tw-gradient-to:rgba(255,241,242,.7) var(--tw-gradient-to-position)}.to-rose-50\/75{--tw-gradient-to:rgba(255,241,242,.75) var(--tw-gradient-to-position)}.to-rose-50\/80{--tw-gradient-to:rgba(255,241,242,.8) var(--tw-gradient-to-position)}.to-rose-50\/85{--tw-gradient-to:rgba(255,241,242,.85) var(--tw-gradient-to-position)}.to-rose-50\/90{--tw-gradient-to:rgba(255,241,242,.9) var(--tw-gradient-to-position)}.to-rose-50\/95{--tw-gradient-to:rgba(255,241,242,.95) var(--tw-gradient-to-position)}.to-rose-500{--tw-gradient-to:#f43f5e var(--tw-gradient-to-position)}.to-rose-500\/0{--tw-gradient-to:rgba(244,63,94,0) var(--tw-gradient-to-position)}.to-rose-500\/10{--tw-gradient-to:rgba(244,63,94,.1) var(--tw-gradient-to-position)}.to-rose-500\/100{--tw-gradient-to:#f43f5e var(--tw-gradient-to-position)}.to-rose-500\/15{--tw-gradient-to:rgba(244,63,94,.15) var(--tw-gradient-to-position)}.to-rose-500\/20{--tw-gradient-to:rgba(244,63,94,.2) var(--tw-gradient-to-position)}.to-rose-500\/25{--tw-gradient-to:rgba(244,63,94,.25) var(--tw-gradient-to-position)}.to-rose-500\/30{--tw-gradient-to:rgba(244,63,94,.3) var(--tw-gradient-to-position)}.to-rose-500\/35{--tw-gradient-to:rgba(244,63,94,.35) var(--tw-gradient-to-position)}.to-rose-500\/40{--tw-gradient-to:rgba(244,63,94,.4) var(--tw-gradient-to-position)}.to-rose-500\/45{--tw-gradient-to:rgba(244,63,94,.45) var(--tw-gradient-to-position)}.to-rose-500\/5{--tw-gradient-to:rgba(244,63,94,.05) var(--tw-gradient-to-position)}.to-rose-500\/50{--tw-gradient-to:rgba(244,63,94,.5) var(--tw-gradient-to-position)}.to-rose-500\/55{--tw-gradient-to:rgba(244,63,94,.55) var(--tw-gradient-to-position)}.to-rose-500\/60{--tw-gradient-to:rgba(244,63,94,.6) var(--tw-gradient-to-position)}.to-rose-500\/65{--tw-gradient-to:rgba(244,63,94,.65) var(--tw-gradient-to-position)}.to-rose-500\/70{--tw-gradient-to:rgba(244,63,94,.7) var(--tw-gradient-to-position)}.to-rose-500\/75{--tw-gradient-to:rgba(244,63,94,.75) var(--tw-gradient-to-position)}.to-rose-500\/80{--tw-gradient-to:rgba(244,63,94,.8) var(--tw-gradient-to-position)}.to-rose-500\/85{--tw-gradient-to:rgba(244,63,94,.85) var(--tw-gradient-to-position)}.to-rose-500\/90{--tw-gradient-to:rgba(244,63,94,.9) var(--tw-gradient-to-position)}.to-rose-500\/95{--tw-gradient-to:rgba(244,63,94,.95) var(--tw-gradient-to-position)}.to-rose-950{--tw-gradient-to:#4c0519 var(--tw-gradient-to-position)}.to-rose-950\/0{--tw-gradient-to:rgba(76,5,25,0) var(--tw-gradient-to-position)}.to-rose-950\/10{--tw-gradient-to:rgba(76,5,25,.1) var(--tw-gradient-to-position)}.to-rose-950\/100{--tw-gradient-to:#4c0519 var(--tw-gradient-to-position)}.to-rose-950\/15{--tw-gradient-to:rgba(76,5,25,.15) var(--tw-gradient-to-position)}.to-rose-950\/20{--tw-gradient-to:rgba(76,5,25,.2) var(--tw-gradient-to-position)}.to-rose-950\/25{--tw-gradient-to:rgba(76,5,25,.25) var(--tw-gradient-to-position)}.to-rose-950\/30{--tw-gradient-to:rgba(76,5,25,.3) var(--tw-gradient-to-position)}.to-rose-950\/35{--tw-gradient-to:rgba(76,5,25,.35) var(--tw-gradient-to-position)}.to-rose-950\/40{--tw-gradient-to:rgba(76,5,25,.4) var(--tw-gradient-to-position)}.to-rose-950\/45{--tw-gradient-to:rgba(76,5,25,.45) var(--tw-gradient-to-position)}.to-rose-950\/5{--tw-gradient-to:rgba(76,5,25,.05) var(--tw-gradient-to-position)}.to-rose-950\/50{--tw-gradient-to:rgba(76,5,25,.5) var(--tw-gradient-to-position)}.to-rose-950\/55{--tw-gradient-to:rgba(76,5,25,.55) var(--tw-gradient-to-position)}.to-rose-950\/60{--tw-gradient-to:rgba(76,5,25,.6) var(--tw-gradient-to-position)}.to-rose-950\/65{--tw-gradient-to:rgba(76,5,25,.65) var(--tw-gradient-to-position)}.to-rose-950\/70{--tw-gradient-to:rgba(76,5,25,.7) var(--tw-gradient-to-position)}.to-rose-950\/75{--tw-gradient-to:rgba(76,5,25,.75) var(--tw-gradient-to-position)}.to-rose-950\/80{--tw-gradient-to:rgba(76,5,25,.8) var(--tw-gradient-to-position)}.to-rose-950\/85{--tw-gradient-to:rgba(76,5,25,.85) var(--tw-gradient-to-position)}.to-rose-950\/90{--tw-gradient-to:rgba(76,5,25,.9) var(--tw-gradient-to-position)}.to-rose-950\/95{--tw-gradient-to:rgba(76,5,25,.95) var(--tw-gradient-to-position)}.to-sky-100{--tw-gradient-to:#e0f2fe var(--tw-gradient-to-position)}.to-sky-100\/0{--tw-gradient-to:rgba(224,242,254,0) var(--tw-gradient-to-position)}.to-sky-100\/10{--tw-gradient-to:rgba(224,242,254,.1) var(--tw-gradient-to-position)}.to-sky-100\/100{--tw-gradient-to:#e0f2fe var(--tw-gradient-to-position)}.to-sky-100\/15{--tw-gradient-to:rgba(224,242,254,.15) var(--tw-gradient-to-position)}.to-sky-100\/20{--tw-gradient-to:rgba(224,242,254,.2) var(--tw-gradient-to-position)}.to-sky-100\/25{--tw-gradient-to:rgba(224,242,254,.25) var(--tw-gradient-to-position)}.to-sky-100\/30{--tw-gradient-to:rgba(224,242,254,.3) var(--tw-gradient-to-position)}.to-sky-100\/35{--tw-gradient-to:rgba(224,242,254,.35) var(--tw-gradient-to-position)}.to-sky-100\/40{--tw-gradient-to:rgba(224,242,254,.4) var(--tw-gradient-to-position)}.to-sky-100\/45{--tw-gradient-to:rgba(224,242,254,.45) var(--tw-gradient-to-position)}.to-sky-100\/5{--tw-gradient-to:rgba(224,242,254,.05) var(--tw-gradient-to-position)}.to-sky-100\/50{--tw-gradient-to:rgba(224,242,254,.5) var(--tw-gradient-to-position)}.to-sky-100\/55{--tw-gradient-to:rgba(224,242,254,.55) var(--tw-gradient-to-position)}.to-sky-100\/60{--tw-gradient-to:rgba(224,242,254,.6) var(--tw-gradient-to-position)}.to-sky-100\/65{--tw-gradient-to:rgba(224,242,254,.65) var(--tw-gradient-to-position)}.to-sky-100\/70{--tw-gradient-to:rgba(224,242,254,.7) var(--tw-gradient-to-position)}.to-sky-100\/75{--tw-gradient-to:rgba(224,242,254,.75) var(--tw-gradient-to-position)}.to-sky-100\/80{--tw-gradient-to:rgba(224,242,254,.8) var(--tw-gradient-to-position)}.to-sky-100\/85{--tw-gradient-to:rgba(224,242,254,.85) var(--tw-gradient-to-position)}.to-sky-100\/90{--tw-gradient-to:rgba(224,242,254,.9) var(--tw-gradient-to-position)}.to-sky-100\/95{--tw-gradient-to:rgba(224,242,254,.95) var(--tw-gradient-to-position)}.to-sky-200{--tw-gradient-to:#bae6fd var(--tw-gradient-to-position)}.to-sky-200\/0{--tw-gradient-to:rgba(186,230,253,0) var(--tw-gradient-to-position)}.to-sky-200\/10{--tw-gradient-to:rgba(186,230,253,.1) var(--tw-gradient-to-position)}.to-sky-200\/100{--tw-gradient-to:#bae6fd var(--tw-gradient-to-position)}.to-sky-200\/15{--tw-gradient-to:rgba(186,230,253,.15) var(--tw-gradient-to-position)}.to-sky-200\/20{--tw-gradient-to:rgba(186,230,253,.2) var(--tw-gradient-to-position)}.to-sky-200\/25{--tw-gradient-to:rgba(186,230,253,.25) var(--tw-gradient-to-position)}.to-sky-200\/30{--tw-gradient-to:rgba(186,230,253,.3) var(--tw-gradient-to-position)}.to-sky-200\/35{--tw-gradient-to:rgba(186,230,253,.35) var(--tw-gradient-to-position)}.to-sky-200\/40{--tw-gradient-to:rgba(186,230,253,.4) var(--tw-gradient-to-position)}.to-sky-200\/45{--tw-gradient-to:rgba(186,230,253,.45) var(--tw-gradient-to-position)}.to-sky-200\/5{--tw-gradient-to:rgba(186,230,253,.05) var(--tw-gradient-to-position)}.to-sky-200\/50{--tw-gradient-to:rgba(186,230,253,.5) var(--tw-gradient-to-position)}.to-sky-200\/55{--tw-gradient-to:rgba(186,230,253,.55) var(--tw-gradient-to-position)}.to-sky-200\/60{--tw-gradient-to:rgba(186,230,253,.6) var(--tw-gradient-to-position)}.to-sky-200\/65{--tw-gradient-to:rgba(186,230,253,.65) var(--tw-gradient-to-position)}.to-sky-200\/70{--tw-gradient-to:rgba(186,230,253,.7) var(--tw-gradient-to-position)}.to-sky-200\/75{--tw-gradient-to:rgba(186,230,253,.75) var(--tw-gradient-to-position)}.to-sky-200\/80{--tw-gradient-to:rgba(186,230,253,.8) var(--tw-gradient-to-position)}.to-sky-200\/85{--tw-gradient-to:rgba(186,230,253,.85) var(--tw-gradient-to-position)}.to-sky-200\/90{--tw-gradient-to:rgba(186,230,253,.9) var(--tw-gradient-to-position)}.to-sky-200\/95{--tw-gradient-to:rgba(186,230,253,.95) var(--tw-gradient-to-position)}.to-sky-50{--tw-gradient-to:#f0f9ff var(--tw-gradient-to-position)}.to-sky-50\/0{--tw-gradient-to:rgba(240,249,255,0) var(--tw-gradient-to-position)}.to-sky-50\/10{--tw-gradient-to:rgba(240,249,255,.1) var(--tw-gradient-to-position)}.to-sky-50\/100{--tw-gradient-to:#f0f9ff var(--tw-gradient-to-position)}.to-sky-50\/15{--tw-gradient-to:rgba(240,249,255,.15) var(--tw-gradient-to-position)}.to-sky-50\/20{--tw-gradient-to:rgba(240,249,255,.2) var(--tw-gradient-to-position)}.to-sky-50\/25{--tw-gradient-to:rgba(240,249,255,.25) var(--tw-gradient-to-position)}.to-sky-50\/30{--tw-gradient-to:rgba(240,249,255,.3) var(--tw-gradient-to-position)}.to-sky-50\/35{--tw-gradient-to:rgba(240,249,255,.35) var(--tw-gradient-to-position)}.to-sky-50\/40{--tw-gradient-to:rgba(240,249,255,.4) var(--tw-gradient-to-position)}.to-sky-50\/45{--tw-gradient-to:rgba(240,249,255,.45) var(--tw-gradient-to-position)}.to-sky-50\/5{--tw-gradient-to:rgba(240,249,255,.05) var(--tw-gradient-to-position)}.to-sky-50\/50{--tw-gradient-to:rgba(240,249,255,.5) var(--tw-gradient-to-position)}.to-sky-50\/55{--tw-gradient-to:rgba(240,249,255,.55) var(--tw-gradient-to-position)}.to-sky-50\/60{--tw-gradient-to:rgba(240,249,255,.6) var(--tw-gradient-to-position)}.to-sky-50\/65{--tw-gradient-to:rgba(240,249,255,.65) var(--tw-gradient-to-position)}.to-sky-50\/70{--tw-gradient-to:rgba(240,249,255,.7) var(--tw-gradient-to-position)}.to-sky-50\/75{--tw-gradient-to:rgba(240,249,255,.75) var(--tw-gradient-to-position)}.to-sky-50\/80{--tw-gradient-to:rgba(240,249,255,.8) var(--tw-gradient-to-position)}.to-sky-50\/85{--tw-gradient-to:rgba(240,249,255,.85) var(--tw-gradient-to-position)}.to-sky-50\/90{--tw-gradient-to:rgba(240,249,255,.9) var(--tw-gradient-to-position)}.to-sky-50\/95{--tw-gradient-to:rgba(240,249,255,.95) var(--tw-gradient-to-position)}.to-sky-500{--tw-gradient-to:#0ea5e9 var(--tw-gradient-to-position)}.to-sky-500\/0{--tw-gradient-to:rgba(14,165,233,0) var(--tw-gradient-to-position)}.to-sky-500\/10{--tw-gradient-to:rgba(14,165,233,.1) var(--tw-gradient-to-position)}.to-sky-500\/100{--tw-gradient-to:#0ea5e9 var(--tw-gradient-to-position)}.to-sky-500\/15{--tw-gradient-to:rgba(14,165,233,.15) var(--tw-gradient-to-position)}.to-sky-500\/20{--tw-gradient-to:rgba(14,165,233,.2) var(--tw-gradient-to-position)}.to-sky-500\/25{--tw-gradient-to:rgba(14,165,233,.25) var(--tw-gradient-to-position)}.to-sky-500\/30{--tw-gradient-to:rgba(14,165,233,.3) var(--tw-gradient-to-position)}.to-sky-500\/35{--tw-gradient-to:rgba(14,165,233,.35) var(--tw-gradient-to-position)}.to-sky-500\/40{--tw-gradient-to:rgba(14,165,233,.4) var(--tw-gradient-to-position)}.to-sky-500\/45{--tw-gradient-to:rgba(14,165,233,.45) var(--tw-gradient-to-position)}.to-sky-500\/5{--tw-gradient-to:rgba(14,165,233,.05) var(--tw-gradient-to-position)}.to-sky-500\/50{--tw-gradient-to:rgba(14,165,233,.5) var(--tw-gradient-to-position)}.to-sky-500\/55{--tw-gradient-to:rgba(14,165,233,.55) var(--tw-gradient-to-position)}.to-sky-500\/60{--tw-gradient-to:rgba(14,165,233,.6) var(--tw-gradient-to-position)}.to-sky-500\/65{--tw-gradient-to:rgba(14,165,233,.65) var(--tw-gradient-to-position)}.to-sky-500\/70{--tw-gradient-to:rgba(14,165,233,.7) var(--tw-gradient-to-position)}.to-sky-500\/75{--tw-gradient-to:rgba(14,165,233,.75) var(--tw-gradient-to-position)}.to-sky-500\/80{--tw-gradient-to:rgba(14,165,233,.8) var(--tw-gradient-to-position)}.to-sky-500\/85{--tw-gradient-to:rgba(14,165,233,.85) var(--tw-gradient-to-position)}.to-sky-500\/90{--tw-gradient-to:rgba(14,165,233,.9) var(--tw-gradient-to-position)}.to-sky-500\/95{--tw-gradient-to:rgba(14,165,233,.95) var(--tw-gradient-to-position)}.to-sky-950{--tw-gradient-to:#082f49 var(--tw-gradient-to-position)}.to-sky-950\/0{--tw-gradient-to:rgba(8,47,73,0) var(--tw-gradient-to-position)}.to-sky-950\/10{--tw-gradient-to:rgba(8,47,73,.1) var(--tw-gradient-to-position)}.to-sky-950\/100{--tw-gradient-to:#082f49 var(--tw-gradient-to-position)}.to-sky-950\/15{--tw-gradient-to:rgba(8,47,73,.15) var(--tw-gradient-to-position)}.to-sky-950\/20{--tw-gradient-to:rgba(8,47,73,.2) var(--tw-gradient-to-position)}.to-sky-950\/25{--tw-gradient-to:rgba(8,47,73,.25) var(--tw-gradient-to-position)}.to-sky-950\/30{--tw-gradient-to:rgba(8,47,73,.3) var(--tw-gradient-to-position)}.to-sky-950\/35{--tw-gradient-to:rgba(8,47,73,.35) var(--tw-gradient-to-position)}.to-sky-950\/40{--tw-gradient-to:rgba(8,47,73,.4) var(--tw-gradient-to-position)}.to-sky-950\/45{--tw-gradient-to:rgba(8,47,73,.45) var(--tw-gradient-to-position)}.to-sky-950\/5{--tw-gradient-to:rgba(8,47,73,.05) var(--tw-gradient-to-position)}.to-sky-950\/50{--tw-gradient-to:rgba(8,47,73,.5) var(--tw-gradient-to-position)}.to-sky-950\/55{--tw-gradient-to:rgba(8,47,73,.55) var(--tw-gradient-to-position)}.to-sky-950\/60{--tw-gradient-to:rgba(8,47,73,.6) var(--tw-gradient-to-position)}.to-sky-950\/65{--tw-gradient-to:rgba(8,47,73,.65) var(--tw-gradient-to-position)}.to-sky-950\/70{--tw-gradient-to:rgba(8,47,73,.7) var(--tw-gradient-to-position)}.to-sky-950\/75{--tw-gradient-to:rgba(8,47,73,.75) var(--tw-gradient-to-position)}.to-sky-950\/80{--tw-gradient-to:rgba(8,47,73,.8) var(--tw-gradient-to-position)}.to-sky-950\/85{--tw-gradient-to:rgba(8,47,73,.85) var(--tw-gradient-to-position)}.to-sky-950\/90{--tw-gradient-to:rgba(8,47,73,.9) var(--tw-gradient-to-position)}.to-sky-950\/95{--tw-gradient-to:rgba(8,47,73,.95) var(--tw-gradient-to-position)}.to-slate-100{--tw-gradient-to:#f1f5f9 var(--tw-gradient-to-position)}.to-slate-100\/0{--tw-gradient-to:rgba(241,245,249,0) var(--tw-gradient-to-position)}.to-slate-100\/10{--tw-gradient-to:rgba(241,245,249,.1) var(--tw-gradient-to-position)}.to-slate-100\/100{--tw-gradient-to:#f1f5f9 var(--tw-gradient-to-position)}.to-slate-100\/15{--tw-gradient-to:rgba(241,245,249,.15) var(--tw-gradient-to-position)}.to-slate-100\/20{--tw-gradient-to:rgba(241,245,249,.2) var(--tw-gradient-to-position)}.to-slate-100\/25{--tw-gradient-to:rgba(241,245,249,.25) var(--tw-gradient-to-position)}.to-slate-100\/30{--tw-gradient-to:rgba(241,245,249,.3) var(--tw-gradient-to-position)}.to-slate-100\/35{--tw-gradient-to:rgba(241,245,249,.35) var(--tw-gradient-to-position)}.to-slate-100\/40{--tw-gradient-to:rgba(241,245,249,.4) var(--tw-gradient-to-position)}.to-slate-100\/45{--tw-gradient-to:rgba(241,245,249,.45) var(--tw-gradient-to-position)}.to-slate-100\/5{--tw-gradient-to:rgba(241,245,249,.05) var(--tw-gradient-to-position)}.to-slate-100\/50{--tw-gradient-to:rgba(241,245,249,.5) var(--tw-gradient-to-position)}.to-slate-100\/55{--tw-gradient-to:rgba(241,245,249,.55) var(--tw-gradient-to-position)}.to-slate-100\/60{--tw-gradient-to:rgba(241,245,249,.6) var(--tw-gradient-to-position)}.to-slate-100\/65{--tw-gradient-to:rgba(241,245,249,.65) var(--tw-gradient-to-position)}.to-slate-100\/70{--tw-gradient-to:rgba(241,245,249,.7) var(--tw-gradient-to-position)}.to-slate-100\/75{--tw-gradient-to:rgba(241,245,249,.75) var(--tw-gradient-to-position)}.to-slate-100\/80{--tw-gradient-to:rgba(241,245,249,.8) var(--tw-gradient-to-position)}.to-slate-100\/85{--tw-gradient-to:rgba(241,245,249,.85) var(--tw-gradient-to-position)}.to-slate-100\/90{--tw-gradient-to:rgba(241,245,249,.9) var(--tw-gradient-to-position)}.to-slate-100\/95{--tw-gradient-to:rgba(241,245,249,.95) var(--tw-gradient-to-position)}.to-slate-200{--tw-gradient-to:#e2e8f0 var(--tw-gradient-to-position)}.to-slate-200\/0{--tw-gradient-to:rgba(226,232,240,0) var(--tw-gradient-to-position)}.to-slate-200\/10{--tw-gradient-to:rgba(226,232,240,.1) var(--tw-gradient-to-position)}.to-slate-200\/100{--tw-gradient-to:#e2e8f0 var(--tw-gradient-to-position)}.to-slate-200\/15{--tw-gradient-to:rgba(226,232,240,.15) var(--tw-gradient-to-position)}.to-slate-200\/20{--tw-gradient-to:rgba(226,232,240,.2) var(--tw-gradient-to-position)}.to-slate-200\/25{--tw-gradient-to:rgba(226,232,240,.25) var(--tw-gradient-to-position)}.to-slate-200\/30{--tw-gradient-to:rgba(226,232,240,.3) var(--tw-gradient-to-position)}.to-slate-200\/35{--tw-gradient-to:rgba(226,232,240,.35) var(--tw-gradient-to-position)}.to-slate-200\/40{--tw-gradient-to:rgba(226,232,240,.4) var(--tw-gradient-to-position)}.to-slate-200\/45{--tw-gradient-to:rgba(226,232,240,.45) var(--tw-gradient-to-position)}.to-slate-200\/5{--tw-gradient-to:rgba(226,232,240,.05) var(--tw-gradient-to-position)}.to-slate-200\/50{--tw-gradient-to:rgba(226,232,240,.5) var(--tw-gradient-to-position)}.to-slate-200\/55{--tw-gradient-to:rgba(226,232,240,.55) var(--tw-gradient-to-position)}.to-slate-200\/60{--tw-gradient-to:rgba(226,232,240,.6) var(--tw-gradient-to-position)}.to-slate-200\/65{--tw-gradient-to:rgba(226,232,240,.65) var(--tw-gradient-to-position)}.to-slate-200\/70{--tw-gradient-to:rgba(226,232,240,.7) var(--tw-gradient-to-position)}.to-slate-200\/75{--tw-gradient-to:rgba(226,232,240,.75) var(--tw-gradient-to-position)}.to-slate-200\/80{--tw-gradient-to:rgba(226,232,240,.8) var(--tw-gradient-to-position)}.to-slate-200\/85{--tw-gradient-to:rgba(226,232,240,.85) var(--tw-gradient-to-position)}.to-slate-200\/90{--tw-gradient-to:rgba(226,232,240,.9) var(--tw-gradient-to-position)}.to-slate-200\/95{--tw-gradient-to:rgba(226,232,240,.95) var(--tw-gradient-to-position)}.to-slate-50{--tw-gradient-to:#f8fafc var(--tw-gradient-to-position)}.to-slate-50\/0{--tw-gradient-to:rgba(248,250,252,0) var(--tw-gradient-to-position)}.to-slate-50\/10{--tw-gradient-to:rgba(248,250,252,.1) var(--tw-gradient-to-position)}.to-slate-50\/100{--tw-gradient-to:#f8fafc var(--tw-gradient-to-position)}.to-slate-50\/15{--tw-gradient-to:rgba(248,250,252,.15) var(--tw-gradient-to-position)}.to-slate-50\/20{--tw-gradient-to:rgba(248,250,252,.2) var(--tw-gradient-to-position)}.to-slate-50\/25{--tw-gradient-to:rgba(248,250,252,.25) var(--tw-gradient-to-position)}.to-slate-50\/30{--tw-gradient-to:rgba(248,250,252,.3) var(--tw-gradient-to-position)}.to-slate-50\/35{--tw-gradient-to:rgba(248,250,252,.35) var(--tw-gradient-to-position)}.to-slate-50\/40{--tw-gradient-to:rgba(248,250,252,.4) var(--tw-gradient-to-position)}.to-slate-50\/45{--tw-gradient-to:rgba(248,250,252,.45) var(--tw-gradient-to-position)}.to-slate-50\/5{--tw-gradient-to:rgba(248,250,252,.05) var(--tw-gradient-to-position)}.to-slate-50\/50{--tw-gradient-to:rgba(248,250,252,.5) var(--tw-gradient-to-position)}.to-slate-50\/55{--tw-gradient-to:rgba(248,250,252,.55) var(--tw-gradient-to-position)}.to-slate-50\/60{--tw-gradient-to:rgba(248,250,252,.6) var(--tw-gradient-to-position)}.to-slate-50\/65{--tw-gradient-to:rgba(248,250,252,.65) var(--tw-gradient-to-position)}.to-slate-50\/70{--tw-gradient-to:rgba(248,250,252,.7) var(--tw-gradient-to-position)}.to-slate-50\/75{--tw-gradient-to:rgba(248,250,252,.75) var(--tw-gradient-to-position)}.to-slate-50\/80{--tw-gradient-to:rgba(248,250,252,.8) var(--tw-gradient-to-position)}.to-slate-50\/85{--tw-gradient-to:rgba(248,250,252,.85) var(--tw-gradient-to-position)}.to-slate-50\/90{--tw-gradient-to:rgba(248,250,252,.9) var(--tw-gradient-to-position)}.to-slate-50\/95{--tw-gradient-to:rgba(248,250,252,.95) var(--tw-gradient-to-position)}.to-slate-500{--tw-gradient-to:#64748b var(--tw-gradient-to-position)}.to-slate-500\/0{--tw-gradient-to:rgba(100,116,139,0) var(--tw-gradient-to-position)}.to-slate-500\/10{--tw-gradient-to:rgba(100,116,139,.1) var(--tw-gradient-to-position)}.to-slate-500\/100{--tw-gradient-to:#64748b var(--tw-gradient-to-position)}.to-slate-500\/15{--tw-gradient-to:rgba(100,116,139,.15) var(--tw-gradient-to-position)}.to-slate-500\/20{--tw-gradient-to:rgba(100,116,139,.2) var(--tw-gradient-to-position)}.to-slate-500\/25{--tw-gradient-to:rgba(100,116,139,.25) var(--tw-gradient-to-position)}.to-slate-500\/30{--tw-gradient-to:rgba(100,116,139,.3) var(--tw-gradient-to-position)}.to-slate-500\/35{--tw-gradient-to:rgba(100,116,139,.35) var(--tw-gradient-to-position)}.to-slate-500\/40{--tw-gradient-to:rgba(100,116,139,.4) var(--tw-gradient-to-position)}.to-slate-500\/45{--tw-gradient-to:rgba(100,116,139,.45) var(--tw-gradient-to-position)}.to-slate-500\/5{--tw-gradient-to:rgba(100,116,139,.05) var(--tw-gradient-to-position)}.to-slate-500\/50{--tw-gradient-to:rgba(100,116,139,.5) var(--tw-gradient-to-position)}.to-slate-500\/55{--tw-gradient-to:rgba(100,116,139,.55) var(--tw-gradient-to-position)}.to-slate-500\/60{--tw-gradient-to:rgba(100,116,139,.6) var(--tw-gradient-to-position)}.to-slate-500\/65{--tw-gradient-to:rgba(100,116,139,.65) var(--tw-gradient-to-position)}.to-slate-500\/70{--tw-gradient-to:rgba(100,116,139,.7) var(--tw-gradient-to-position)}.to-slate-500\/75{--tw-gradient-to:rgba(100,116,139,.75) var(--tw-gradient-to-position)}.to-slate-500\/80{--tw-gradient-to:rgba(100,116,139,.8) var(--tw-gradient-to-position)}.to-slate-500\/85{--tw-gradient-to:rgba(100,116,139,.85) var(--tw-gradient-to-position)}.to-slate-500\/90{--tw-gradient-to:rgba(100,116,139,.9) var(--tw-gradient-to-position)}.to-slate-500\/95{--tw-gradient-to:rgba(100,116,139,.95) var(--tw-gradient-to-position)}.to-slate-950{--tw-gradient-to:#020617 var(--tw-gradient-to-position)}.to-slate-950\/0{--tw-gradient-to:rgba(2,6,23,0) var(--tw-gradient-to-position)}.to-slate-950\/10{--tw-gradient-to:rgba(2,6,23,.1) var(--tw-gradient-to-position)}.to-slate-950\/100{--tw-gradient-to:#020617 var(--tw-gradient-to-position)}.to-slate-950\/15{--tw-gradient-to:rgba(2,6,23,.15) var(--tw-gradient-to-position)}.to-slate-950\/20{--tw-gradient-to:rgba(2,6,23,.2) var(--tw-gradient-to-position)}.to-slate-950\/25{--tw-gradient-to:rgba(2,6,23,.25) var(--tw-gradient-to-position)}.to-slate-950\/30{--tw-gradient-to:rgba(2,6,23,.3) var(--tw-gradient-to-position)}.to-slate-950\/35{--tw-gradient-to:rgba(2,6,23,.35) var(--tw-gradient-to-position)}.to-slate-950\/40{--tw-gradient-to:rgba(2,6,23,.4) var(--tw-gradient-to-position)}.to-slate-950\/45{--tw-gradient-to:rgba(2,6,23,.45) var(--tw-gradient-to-position)}.to-slate-950\/5{--tw-gradient-to:rgba(2,6,23,.05) var(--tw-gradient-to-position)}.to-slate-950\/50{--tw-gradient-to:rgba(2,6,23,.5) var(--tw-gradient-to-position)}.to-slate-950\/55{--tw-gradient-to:rgba(2,6,23,.55) var(--tw-gradient-to-position)}.to-slate-950\/60{--tw-gradient-to:rgba(2,6,23,.6) var(--tw-gradient-to-position)}.to-slate-950\/65{--tw-gradient-to:rgba(2,6,23,.65) var(--tw-gradient-to-position)}.to-slate-950\/70{--tw-gradient-to:rgba(2,6,23,.7) var(--tw-gradient-to-position)}.to-slate-950\/75{--tw-gradient-to:rgba(2,6,23,.75) var(--tw-gradient-to-position)}.to-slate-950\/80{--tw-gradient-to:rgba(2,6,23,.8) var(--tw-gradient-to-position)}.to-slate-950\/85{--tw-gradient-to:rgba(2,6,23,.85) var(--tw-gradient-to-position)}.to-slate-950\/90{--tw-gradient-to:rgba(2,6,23,.9) var(--tw-gradient-to-position)}.to-slate-950\/95{--tw-gradient-to:rgba(2,6,23,.95) var(--tw-gradient-to-position)}.to-stone-100{--tw-gradient-to:#f5f5f4 var(--tw-gradient-to-position)}.to-stone-100\/0{--tw-gradient-to:hsla(60,5%,96%,0) var(--tw-gradient-to-position)}.to-stone-100\/10{--tw-gradient-to:hsla(60,5%,96%,.1) var(--tw-gradient-to-position)}.to-stone-100\/100{--tw-gradient-to:#f5f5f4 var(--tw-gradient-to-position)}.to-stone-100\/15{--tw-gradient-to:hsla(60,5%,96%,.15) var(--tw-gradient-to-position)}.to-stone-100\/20{--tw-gradient-to:hsla(60,5%,96%,.2) var(--tw-gradient-to-position)}.to-stone-100\/25{--tw-gradient-to:hsla(60,5%,96%,.25) var(--tw-gradient-to-position)}.to-stone-100\/30{--tw-gradient-to:hsla(60,5%,96%,.3) var(--tw-gradient-to-position)}.to-stone-100\/35{--tw-gradient-to:hsla(60,5%,96%,.35) var(--tw-gradient-to-position)}.to-stone-100\/40{--tw-gradient-to:hsla(60,5%,96%,.4) var(--tw-gradient-to-position)}.to-stone-100\/45{--tw-gradient-to:hsla(60,5%,96%,.45) var(--tw-gradient-to-position)}.to-stone-100\/5{--tw-gradient-to:hsla(60,5%,96%,.05) var(--tw-gradient-to-position)}.to-stone-100\/50{--tw-gradient-to:hsla(60,5%,96%,.5) var(--tw-gradient-to-position)}.to-stone-100\/55{--tw-gradient-to:hsla(60,5%,96%,.55) var(--tw-gradient-to-position)}.to-stone-100\/60{--tw-gradient-to:hsla(60,5%,96%,.6) var(--tw-gradient-to-position)}.to-stone-100\/65{--tw-gradient-to:hsla(60,5%,96%,.65) var(--tw-gradient-to-position)}.to-stone-100\/70{--tw-gradient-to:hsla(60,5%,96%,.7) var(--tw-gradient-to-position)}.to-stone-100\/75{--tw-gradient-to:hsla(60,5%,96%,.75) var(--tw-gradient-to-position)}.to-stone-100\/80{--tw-gradient-to:hsla(60,5%,96%,.8) var(--tw-gradient-to-position)}.to-stone-100\/85{--tw-gradient-to:hsla(60,5%,96%,.85) var(--tw-gradient-to-position)}.to-stone-100\/90{--tw-gradient-to:hsla(60,5%,96%,.9) var(--tw-gradient-to-position)}.to-stone-100\/95{--tw-gradient-to:hsla(60,5%,96%,.95) var(--tw-gradient-to-position)}.to-stone-200{--tw-gradient-to:#e7e5e4 var(--tw-gradient-to-position)}.to-stone-200\/0{--tw-gradient-to:hsla(20,6%,90%,0) var(--tw-gradient-to-position)}.to-stone-200\/10{--tw-gradient-to:hsla(20,6%,90%,.1) var(--tw-gradient-to-position)}.to-stone-200\/100{--tw-gradient-to:#e7e5e4 var(--tw-gradient-to-position)}.to-stone-200\/15{--tw-gradient-to:hsla(20,6%,90%,.15) var(--tw-gradient-to-position)}.to-stone-200\/20{--tw-gradient-to:hsla(20,6%,90%,.2) var(--tw-gradient-to-position)}.to-stone-200\/25{--tw-gradient-to:hsla(20,6%,90%,.25) var(--tw-gradient-to-position)}.to-stone-200\/30{--tw-gradient-to:hsla(20,6%,90%,.3) var(--tw-gradient-to-position)}.to-stone-200\/35{--tw-gradient-to:hsla(20,6%,90%,.35) var(--tw-gradient-to-position)}.to-stone-200\/40{--tw-gradient-to:hsla(20,6%,90%,.4) var(--tw-gradient-to-position)}.to-stone-200\/45{--tw-gradient-to:hsla(20,6%,90%,.45) var(--tw-gradient-to-position)}.to-stone-200\/5{--tw-gradient-to:hsla(20,6%,90%,.05) var(--tw-gradient-to-position)}.to-stone-200\/50{--tw-gradient-to:hsla(20,6%,90%,.5) var(--tw-gradient-to-position)}.to-stone-200\/55{--tw-gradient-to:hsla(20,6%,90%,.55) var(--tw-gradient-to-position)}.to-stone-200\/60{--tw-gradient-to:hsla(20,6%,90%,.6) var(--tw-gradient-to-position)}.to-stone-200\/65{--tw-gradient-to:hsla(20,6%,90%,.65) var(--tw-gradient-to-position)}.to-stone-200\/70{--tw-gradient-to:hsla(20,6%,90%,.7) var(--tw-gradient-to-position)}.to-stone-200\/75{--tw-gradient-to:hsla(20,6%,90%,.75) var(--tw-gradient-to-position)}.to-stone-200\/80{--tw-gradient-to:hsla(20,6%,90%,.8) var(--tw-gradient-to-position)}.to-stone-200\/85{--tw-gradient-to:hsla(20,6%,90%,.85) var(--tw-gradient-to-position)}.to-stone-200\/90{--tw-gradient-to:hsla(20,6%,90%,.9) var(--tw-gradient-to-position)}.to-stone-200\/95{--tw-gradient-to:hsla(20,6%,90%,.95) var(--tw-gradient-to-position)}.to-stone-50{--tw-gradient-to:#fafaf9 var(--tw-gradient-to-position)}.to-stone-50\/0{--tw-gradient-to:hsla(60,9%,98%,0) var(--tw-gradient-to-position)}.to-stone-50\/10{--tw-gradient-to:hsla(60,9%,98%,.1) var(--tw-gradient-to-position)}.to-stone-50\/100{--tw-gradient-to:#fafaf9 var(--tw-gradient-to-position)}.to-stone-50\/15{--tw-gradient-to:hsla(60,9%,98%,.15) var(--tw-gradient-to-position)}.to-stone-50\/20{--tw-gradient-to:hsla(60,9%,98%,.2) var(--tw-gradient-to-position)}.to-stone-50\/25{--tw-gradient-to:hsla(60,9%,98%,.25) var(--tw-gradient-to-position)}.to-stone-50\/30{--tw-gradient-to:hsla(60,9%,98%,.3) var(--tw-gradient-to-position)}.to-stone-50\/35{--tw-gradient-to:hsla(60,9%,98%,.35) var(--tw-gradient-to-position)}.to-stone-50\/40{--tw-gradient-to:hsla(60,9%,98%,.4) var(--tw-gradient-to-position)}.to-stone-50\/45{--tw-gradient-to:hsla(60,9%,98%,.45) var(--tw-gradient-to-position)}.to-stone-50\/5{--tw-gradient-to:hsla(60,9%,98%,.05) var(--tw-gradient-to-position)}.to-stone-50\/50{--tw-gradient-to:hsla(60,9%,98%,.5) var(--tw-gradient-to-position)}.to-stone-50\/55{--tw-gradient-to:hsla(60,9%,98%,.55) var(--tw-gradient-to-position)}.to-stone-50\/60{--tw-gradient-to:hsla(60,9%,98%,.6) var(--tw-gradient-to-position)}.to-stone-50\/65{--tw-gradient-to:hsla(60,9%,98%,.65) var(--tw-gradient-to-position)}.to-stone-50\/70{--tw-gradient-to:hsla(60,9%,98%,.7) var(--tw-gradient-to-position)}.to-stone-50\/75{--tw-gradient-to:hsla(60,9%,98%,.75) var(--tw-gradient-to-position)}.to-stone-50\/80{--tw-gradient-to:hsla(60,9%,98%,.8) var(--tw-gradient-to-position)}.to-stone-50\/85{--tw-gradient-to:hsla(60,9%,98%,.85) var(--tw-gradient-to-position)}.to-stone-50\/90{--tw-gradient-to:hsla(60,9%,98%,.9) var(--tw-gradient-to-position)}.to-stone-50\/95{--tw-gradient-to:hsla(60,9%,98%,.95) var(--tw-gradient-to-position)}.to-stone-500{--tw-gradient-to:#78716c var(--tw-gradient-to-position)}.to-stone-500\/0{--tw-gradient-to:hsla(25,5%,45%,0) var(--tw-gradient-to-position)}.to-stone-500\/10{--tw-gradient-to:hsla(25,5%,45%,.1) var(--tw-gradient-to-position)}.to-stone-500\/100{--tw-gradient-to:#78716c var(--tw-gradient-to-position)}.to-stone-500\/15{--tw-gradient-to:hsla(25,5%,45%,.15) var(--tw-gradient-to-position)}.to-stone-500\/20{--tw-gradient-to:hsla(25,5%,45%,.2) var(--tw-gradient-to-position)}.to-stone-500\/25{--tw-gradient-to:hsla(25,5%,45%,.25) var(--tw-gradient-to-position)}.to-stone-500\/30{--tw-gradient-to:hsla(25,5%,45%,.3) var(--tw-gradient-to-position)}.to-stone-500\/35{--tw-gradient-to:hsla(25,5%,45%,.35) var(--tw-gradient-to-position)}.to-stone-500\/40{--tw-gradient-to:hsla(25,5%,45%,.4) var(--tw-gradient-to-position)}.to-stone-500\/45{--tw-gradient-to:hsla(25,5%,45%,.45) var(--tw-gradient-to-position)}.to-stone-500\/5{--tw-gradient-to:hsla(25,5%,45%,.05) var(--tw-gradient-to-position)}.to-stone-500\/50{--tw-gradient-to:hsla(25,5%,45%,.5) var(--tw-gradient-to-position)}.to-stone-500\/55{--tw-gradient-to:hsla(25,5%,45%,.55) var(--tw-gradient-to-position)}.to-stone-500\/60{--tw-gradient-to:hsla(25,5%,45%,.6) var(--tw-gradient-to-position)}.to-stone-500\/65{--tw-gradient-to:hsla(25,5%,45%,.65) var(--tw-gradient-to-position)}.to-stone-500\/70{--tw-gradient-to:hsla(25,5%,45%,.7) var(--tw-gradient-to-position)}.to-stone-500\/75{--tw-gradient-to:hsla(25,5%,45%,.75) var(--tw-gradient-to-position)}.to-stone-500\/80{--tw-gradient-to:hsla(25,5%,45%,.8) var(--tw-gradient-to-position)}.to-stone-500\/85{--tw-gradient-to:hsla(25,5%,45%,.85) var(--tw-gradient-to-position)}.to-stone-500\/90{--tw-gradient-to:hsla(25,5%,45%,.9) var(--tw-gradient-to-position)}.to-stone-500\/95{--tw-gradient-to:hsla(25,5%,45%,.95) var(--tw-gradient-to-position)}.to-stone-950{--tw-gradient-to:#0c0a09 var(--tw-gradient-to-position)}.to-stone-950\/0{--tw-gradient-to:rgba(12,10,9,0) var(--tw-gradient-to-position)}.to-stone-950\/10{--tw-gradient-to:rgba(12,10,9,.1) var(--tw-gradient-to-position)}.to-stone-950\/100{--tw-gradient-to:#0c0a09 var(--tw-gradient-to-position)}.to-stone-950\/15{--tw-gradient-to:rgba(12,10,9,.15) var(--tw-gradient-to-position)}.to-stone-950\/20{--tw-gradient-to:rgba(12,10,9,.2) var(--tw-gradient-to-position)}.to-stone-950\/25{--tw-gradient-to:rgba(12,10,9,.25) var(--tw-gradient-to-position)}.to-stone-950\/30{--tw-gradient-to:rgba(12,10,9,.3) var(--tw-gradient-to-position)}.to-stone-950\/35{--tw-gradient-to:rgba(12,10,9,.35) var(--tw-gradient-to-position)}.to-stone-950\/40{--tw-gradient-to:rgba(12,10,9,.4) var(--tw-gradient-to-position)}.to-stone-950\/45{--tw-gradient-to:rgba(12,10,9,.45) var(--tw-gradient-to-position)}.to-stone-950\/5{--tw-gradient-to:rgba(12,10,9,.05) var(--tw-gradient-to-position)}.to-stone-950\/50{--tw-gradient-to:rgba(12,10,9,.5) var(--tw-gradient-to-position)}.to-stone-950\/55{--tw-gradient-to:rgba(12,10,9,.55) var(--tw-gradient-to-position)}.to-stone-950\/60{--tw-gradient-to:rgba(12,10,9,.6) var(--tw-gradient-to-position)}.to-stone-950\/65{--tw-gradient-to:rgba(12,10,9,.65) var(--tw-gradient-to-position)}.to-stone-950\/70{--tw-gradient-to:rgba(12,10,9,.7) var(--tw-gradient-to-position)}.to-stone-950\/75{--tw-gradient-to:rgba(12,10,9,.75) var(--tw-gradient-to-position)}.to-stone-950\/80{--tw-gradient-to:rgba(12,10,9,.8) var(--tw-gradient-to-position)}.to-stone-950\/85{--tw-gradient-to:rgba(12,10,9,.85) var(--tw-gradient-to-position)}.to-stone-950\/90{--tw-gradient-to:rgba(12,10,9,.9) var(--tw-gradient-to-position)}.to-stone-950\/95{--tw-gradient-to:rgba(12,10,9,.95) var(--tw-gradient-to-position)}.to-teal-100{--tw-gradient-to:#ccfbf1 var(--tw-gradient-to-position)}.to-teal-100\/0{--tw-gradient-to:rgba(204,251,241,0) var(--tw-gradient-to-position)}.to-teal-100\/10{--tw-gradient-to:rgba(204,251,241,.1) var(--tw-gradient-to-position)}.to-teal-100\/100{--tw-gradient-to:#ccfbf1 var(--tw-gradient-to-position)}.to-teal-100\/15{--tw-gradient-to:rgba(204,251,241,.15) var(--tw-gradient-to-position)}.to-teal-100\/20{--tw-gradient-to:rgba(204,251,241,.2) var(--tw-gradient-to-position)}.to-teal-100\/25{--tw-gradient-to:rgba(204,251,241,.25) var(--tw-gradient-to-position)}.to-teal-100\/30{--tw-gradient-to:rgba(204,251,241,.3) var(--tw-gradient-to-position)}.to-teal-100\/35{--tw-gradient-to:rgba(204,251,241,.35) var(--tw-gradient-to-position)}.to-teal-100\/40{--tw-gradient-to:rgba(204,251,241,.4) var(--tw-gradient-to-position)}.to-teal-100\/45{--tw-gradient-to:rgba(204,251,241,.45) var(--tw-gradient-to-position)}.to-teal-100\/5{--tw-gradient-to:rgba(204,251,241,.05) var(--tw-gradient-to-position)}.to-teal-100\/50{--tw-gradient-to:rgba(204,251,241,.5) var(--tw-gradient-to-position)}.to-teal-100\/55{--tw-gradient-to:rgba(204,251,241,.55) var(--tw-gradient-to-position)}.to-teal-100\/60{--tw-gradient-to:rgba(204,251,241,.6) var(--tw-gradient-to-position)}.to-teal-100\/65{--tw-gradient-to:rgba(204,251,241,.65) var(--tw-gradient-to-position)}.to-teal-100\/70{--tw-gradient-to:rgba(204,251,241,.7) var(--tw-gradient-to-position)}.to-teal-100\/75{--tw-gradient-to:rgba(204,251,241,.75) var(--tw-gradient-to-position)}.to-teal-100\/80{--tw-gradient-to:rgba(204,251,241,.8) var(--tw-gradient-to-position)}.to-teal-100\/85{--tw-gradient-to:rgba(204,251,241,.85) var(--tw-gradient-to-position)}.to-teal-100\/90{--tw-gradient-to:rgba(204,251,241,.9) var(--tw-gradient-to-position)}.to-teal-100\/95{--tw-gradient-to:rgba(204,251,241,.95) var(--tw-gradient-to-position)}.to-teal-200{--tw-gradient-to:#99f6e4 var(--tw-gradient-to-position)}.to-teal-200\/0{--tw-gradient-to:rgba(153,246,228,0) var(--tw-gradient-to-position)}.to-teal-200\/10{--tw-gradient-to:rgba(153,246,228,.1) var(--tw-gradient-to-position)}.to-teal-200\/100{--tw-gradient-to:#99f6e4 var(--tw-gradient-to-position)}.to-teal-200\/15{--tw-gradient-to:rgba(153,246,228,.15) var(--tw-gradient-to-position)}.to-teal-200\/20{--tw-gradient-to:rgba(153,246,228,.2) var(--tw-gradient-to-position)}.to-teal-200\/25{--tw-gradient-to:rgba(153,246,228,.25) var(--tw-gradient-to-position)}.to-teal-200\/30{--tw-gradient-to:rgba(153,246,228,.3) var(--tw-gradient-to-position)}.to-teal-200\/35{--tw-gradient-to:rgba(153,246,228,.35) var(--tw-gradient-to-position)}.to-teal-200\/40{--tw-gradient-to:rgba(153,246,228,.4) var(--tw-gradient-to-position)}.to-teal-200\/45{--tw-gradient-to:rgba(153,246,228,.45) var(--tw-gradient-to-position)}.to-teal-200\/5{--tw-gradient-to:rgba(153,246,228,.05) var(--tw-gradient-to-position)}.to-teal-200\/50{--tw-gradient-to:rgba(153,246,228,.5) var(--tw-gradient-to-position)}.to-teal-200\/55{--tw-gradient-to:rgba(153,246,228,.55) var(--tw-gradient-to-position)}.to-teal-200\/60{--tw-gradient-to:rgba(153,246,228,.6) var(--tw-gradient-to-position)}.to-teal-200\/65{--tw-gradient-to:rgba(153,246,228,.65) var(--tw-gradient-to-position)}.to-teal-200\/70{--tw-gradient-to:rgba(153,246,228,.7) var(--tw-gradient-to-position)}.to-teal-200\/75{--tw-gradient-to:rgba(153,246,228,.75) var(--tw-gradient-to-position)}.to-teal-200\/80{--tw-gradient-to:rgba(153,246,228,.8) var(--tw-gradient-to-position)}.to-teal-200\/85{--tw-gradient-to:rgba(153,246,228,.85) var(--tw-gradient-to-position)}.to-teal-200\/90{--tw-gradient-to:rgba(153,246,228,.9) var(--tw-gradient-to-position)}.to-teal-200\/95{--tw-gradient-to:rgba(153,246,228,.95) var(--tw-gradient-to-position)}.to-teal-50{--tw-gradient-to:#f0fdfa var(--tw-gradient-to-position)}.to-teal-50\/0{--tw-gradient-to:rgba(240,253,250,0) var(--tw-gradient-to-position)}.to-teal-50\/10{--tw-gradient-to:rgba(240,253,250,.1) var(--tw-gradient-to-position)}.to-teal-50\/100{--tw-gradient-to:#f0fdfa var(--tw-gradient-to-position)}.to-teal-50\/15{--tw-gradient-to:rgba(240,253,250,.15) var(--tw-gradient-to-position)}.to-teal-50\/20{--tw-gradient-to:rgba(240,253,250,.2) var(--tw-gradient-to-position)}.to-teal-50\/25{--tw-gradient-to:rgba(240,253,250,.25) var(--tw-gradient-to-position)}.to-teal-50\/30{--tw-gradient-to:rgba(240,253,250,.3) var(--tw-gradient-to-position)}.to-teal-50\/35{--tw-gradient-to:rgba(240,253,250,.35) var(--tw-gradient-to-position)}.to-teal-50\/40{--tw-gradient-to:rgba(240,253,250,.4) var(--tw-gradient-to-position)}.to-teal-50\/45{--tw-gradient-to:rgba(240,253,250,.45) var(--tw-gradient-to-position)}.to-teal-50\/5{--tw-gradient-to:rgba(240,253,250,.05) var(--tw-gradient-to-position)}.to-teal-50\/50{--tw-gradient-to:rgba(240,253,250,.5) var(--tw-gradient-to-position)}.to-teal-50\/55{--tw-gradient-to:rgba(240,253,250,.55) var(--tw-gradient-to-position)}.to-teal-50\/60{--tw-gradient-to:rgba(240,253,250,.6) var(--tw-gradient-to-position)}.to-teal-50\/65{--tw-gradient-to:rgba(240,253,250,.65) var(--tw-gradient-to-position)}.to-teal-50\/70{--tw-gradient-to:rgba(240,253,250,.7) var(--tw-gradient-to-position)}.to-teal-50\/75{--tw-gradient-to:rgba(240,253,250,.75) var(--tw-gradient-to-position)}.to-teal-50\/80{--tw-gradient-to:rgba(240,253,250,.8) var(--tw-gradient-to-position)}.to-teal-50\/85{--tw-gradient-to:rgba(240,253,250,.85) var(--tw-gradient-to-position)}.to-teal-50\/90{--tw-gradient-to:rgba(240,253,250,.9) var(--tw-gradient-to-position)}.to-teal-50\/95{--tw-gradient-to:rgba(240,253,250,.95) var(--tw-gradient-to-position)}.to-teal-500{--tw-gradient-to:#14b8a6 var(--tw-gradient-to-position)}.to-teal-500\/0{--tw-gradient-to:rgba(20,184,166,0) var(--tw-gradient-to-position)}.to-teal-500\/10{--tw-gradient-to:rgba(20,184,166,.1) var(--tw-gradient-to-position)}.to-teal-500\/100{--tw-gradient-to:#14b8a6 var(--tw-gradient-to-position)}.to-teal-500\/15{--tw-gradient-to:rgba(20,184,166,.15) var(--tw-gradient-to-position)}.to-teal-500\/20{--tw-gradient-to:rgba(20,184,166,.2) var(--tw-gradient-to-position)}.to-teal-500\/25{--tw-gradient-to:rgba(20,184,166,.25) var(--tw-gradient-to-position)}.to-teal-500\/30{--tw-gradient-to:rgba(20,184,166,.3) var(--tw-gradient-to-position)}.to-teal-500\/35{--tw-gradient-to:rgba(20,184,166,.35) var(--tw-gradient-to-position)}.to-teal-500\/40{--tw-gradient-to:rgba(20,184,166,.4) var(--tw-gradient-to-position)}.to-teal-500\/45{--tw-gradient-to:rgba(20,184,166,.45) var(--tw-gradient-to-position)}.to-teal-500\/5{--tw-gradient-to:rgba(20,184,166,.05) var(--tw-gradient-to-position)}.to-teal-500\/50{--tw-gradient-to:rgba(20,184,166,.5) var(--tw-gradient-to-position)}.to-teal-500\/55{--tw-gradient-to:rgba(20,184,166,.55) var(--tw-gradient-to-position)}.to-teal-500\/60{--tw-gradient-to:rgba(20,184,166,.6) var(--tw-gradient-to-position)}.to-teal-500\/65{--tw-gradient-to:rgba(20,184,166,.65) var(--tw-gradient-to-position)}.to-teal-500\/70{--tw-gradient-to:rgba(20,184,166,.7) var(--tw-gradient-to-position)}.to-teal-500\/75{--tw-gradient-to:rgba(20,184,166,.75) var(--tw-gradient-to-position)}.to-teal-500\/80{--tw-gradient-to:rgba(20,184,166,.8) var(--tw-gradient-to-position)}.to-teal-500\/85{--tw-gradient-to:rgba(20,184,166,.85) var(--tw-gradient-to-position)}.to-teal-500\/90{--tw-gradient-to:rgba(20,184,166,.9) var(--tw-gradient-to-position)}.to-teal-500\/95{--tw-gradient-to:rgba(20,184,166,.95) var(--tw-gradient-to-position)}.to-teal-950{--tw-gradient-to:#042f2e var(--tw-gradient-to-position)}.to-teal-950\/0{--tw-gradient-to:rgba(4,47,46,0) var(--tw-gradient-to-position)}.to-teal-950\/10{--tw-gradient-to:rgba(4,47,46,.1) var(--tw-gradient-to-position)}.to-teal-950\/100{--tw-gradient-to:#042f2e var(--tw-gradient-to-position)}.to-teal-950\/15{--tw-gradient-to:rgba(4,47,46,.15) var(--tw-gradient-to-position)}.to-teal-950\/20{--tw-gradient-to:rgba(4,47,46,.2) var(--tw-gradient-to-position)}.to-teal-950\/25{--tw-gradient-to:rgba(4,47,46,.25) var(--tw-gradient-to-position)}.to-teal-950\/30{--tw-gradient-to:rgba(4,47,46,.3) var(--tw-gradient-to-position)}.to-teal-950\/35{--tw-gradient-to:rgba(4,47,46,.35) var(--tw-gradient-to-position)}.to-teal-950\/40{--tw-gradient-to:rgba(4,47,46,.4) var(--tw-gradient-to-position)}.to-teal-950\/45{--tw-gradient-to:rgba(4,47,46,.45) var(--tw-gradient-to-position)}.to-teal-950\/5{--tw-gradient-to:rgba(4,47,46,.05) var(--tw-gradient-to-position)}.to-teal-950\/50{--tw-gradient-to:rgba(4,47,46,.5) var(--tw-gradient-to-position)}.to-teal-950\/55{--tw-gradient-to:rgba(4,47,46,.55) var(--tw-gradient-to-position)}.to-teal-950\/60{--tw-gradient-to:rgba(4,47,46,.6) var(--tw-gradient-to-position)}.to-teal-950\/65{--tw-gradient-to:rgba(4,47,46,.65) var(--tw-gradient-to-position)}.to-teal-950\/70{--tw-gradient-to:rgba(4,47,46,.7) var(--tw-gradient-to-position)}.to-teal-950\/75{--tw-gradient-to:rgba(4,47,46,.75) var(--tw-gradient-to-position)}.to-teal-950\/80{--tw-gradient-to:rgba(4,47,46,.8) var(--tw-gradient-to-position)}.to-teal-950\/85{--tw-gradient-to:rgba(4,47,46,.85) var(--tw-gradient-to-position)}.to-teal-950\/90{--tw-gradient-to:rgba(4,47,46,.9) var(--tw-gradient-to-position)}.to-teal-950\/95{--tw-gradient-to:rgba(4,47,46,.95) var(--tw-gradient-to-position)}.to-violet-100{--tw-gradient-to:#ede9fe var(--tw-gradient-to-position)}.to-violet-100\/0{--tw-gradient-to:rgba(237,233,254,0) var(--tw-gradient-to-position)}.to-violet-100\/10{--tw-gradient-to:rgba(237,233,254,.1) var(--tw-gradient-to-position)}.to-violet-100\/100{--tw-gradient-to:#ede9fe var(--tw-gradient-to-position)}.to-violet-100\/15{--tw-gradient-to:rgba(237,233,254,.15) var(--tw-gradient-to-position)}.to-violet-100\/20{--tw-gradient-to:rgba(237,233,254,.2) var(--tw-gradient-to-position)}.to-violet-100\/25{--tw-gradient-to:rgba(237,233,254,.25) var(--tw-gradient-to-position)}.to-violet-100\/30{--tw-gradient-to:rgba(237,233,254,.3) var(--tw-gradient-to-position)}.to-violet-100\/35{--tw-gradient-to:rgba(237,233,254,.35) var(--tw-gradient-to-position)}.to-violet-100\/40{--tw-gradient-to:rgba(237,233,254,.4) var(--tw-gradient-to-position)}.to-violet-100\/45{--tw-gradient-to:rgba(237,233,254,.45) var(--tw-gradient-to-position)}.to-violet-100\/5{--tw-gradient-to:rgba(237,233,254,.05) var(--tw-gradient-to-position)}.to-violet-100\/50{--tw-gradient-to:rgba(237,233,254,.5) var(--tw-gradient-to-position)}.to-violet-100\/55{--tw-gradient-to:rgba(237,233,254,.55) var(--tw-gradient-to-position)}.to-violet-100\/60{--tw-gradient-to:rgba(237,233,254,.6) var(--tw-gradient-to-position)}.to-violet-100\/65{--tw-gradient-to:rgba(237,233,254,.65) var(--tw-gradient-to-position)}.to-violet-100\/70{--tw-gradient-to:rgba(237,233,254,.7) var(--tw-gradient-to-position)}.to-violet-100\/75{--tw-gradient-to:rgba(237,233,254,.75) var(--tw-gradient-to-position)}.to-violet-100\/80{--tw-gradient-to:rgba(237,233,254,.8) var(--tw-gradient-to-position)}.to-violet-100\/85{--tw-gradient-to:rgba(237,233,254,.85) var(--tw-gradient-to-position)}.to-violet-100\/90{--tw-gradient-to:rgba(237,233,254,.9) var(--tw-gradient-to-position)}.to-violet-100\/95{--tw-gradient-to:rgba(237,233,254,.95) var(--tw-gradient-to-position)}.to-violet-200{--tw-gradient-to:#ddd6fe var(--tw-gradient-to-position)}.to-violet-200\/0{--tw-gradient-to:rgba(221,214,254,0) var(--tw-gradient-to-position)}.to-violet-200\/10{--tw-gradient-to:rgba(221,214,254,.1) var(--tw-gradient-to-position)}.to-violet-200\/100{--tw-gradient-to:#ddd6fe var(--tw-gradient-to-position)}.to-violet-200\/15{--tw-gradient-to:rgba(221,214,254,.15) var(--tw-gradient-to-position)}.to-violet-200\/20{--tw-gradient-to:rgba(221,214,254,.2) var(--tw-gradient-to-position)}.to-violet-200\/25{--tw-gradient-to:rgba(221,214,254,.25) var(--tw-gradient-to-position)}.to-violet-200\/30{--tw-gradient-to:rgba(221,214,254,.3) var(--tw-gradient-to-position)}.to-violet-200\/35{--tw-gradient-to:rgba(221,214,254,.35) var(--tw-gradient-to-position)}.to-violet-200\/40{--tw-gradient-to:rgba(221,214,254,.4) var(--tw-gradient-to-position)}.to-violet-200\/45{--tw-gradient-to:rgba(221,214,254,.45) var(--tw-gradient-to-position)}.to-violet-200\/5{--tw-gradient-to:rgba(221,214,254,.05) var(--tw-gradient-to-position)}.to-violet-200\/50{--tw-gradient-to:rgba(221,214,254,.5) var(--tw-gradient-to-position)}.to-violet-200\/55{--tw-gradient-to:rgba(221,214,254,.55) var(--tw-gradient-to-position)}.to-violet-200\/60{--tw-gradient-to:rgba(221,214,254,.6) var(--tw-gradient-to-position)}.to-violet-200\/65{--tw-gradient-to:rgba(221,214,254,.65) var(--tw-gradient-to-position)}.to-violet-200\/70{--tw-gradient-to:rgba(221,214,254,.7) var(--tw-gradient-to-position)}.to-violet-200\/75{--tw-gradient-to:rgba(221,214,254,.75) var(--tw-gradient-to-position)}.to-violet-200\/80{--tw-gradient-to:rgba(221,214,254,.8) var(--tw-gradient-to-position)}.to-violet-200\/85{--tw-gradient-to:rgba(221,214,254,.85) var(--tw-gradient-to-position)}.to-violet-200\/90{--tw-gradient-to:rgba(221,214,254,.9) var(--tw-gradient-to-position)}.to-violet-200\/95{--tw-gradient-to:rgba(221,214,254,.95) var(--tw-gradient-to-position)}.to-violet-50{--tw-gradient-to:#f5f3ff var(--tw-gradient-to-position)}.to-violet-50\/0{--tw-gradient-to:rgba(245,243,255,0) var(--tw-gradient-to-position)}.to-violet-50\/10{--tw-gradient-to:rgba(245,243,255,.1) var(--tw-gradient-to-position)}.to-violet-50\/100{--tw-gradient-to:#f5f3ff var(--tw-gradient-to-position)}.to-violet-50\/15{--tw-gradient-to:rgba(245,243,255,.15) var(--tw-gradient-to-position)}.to-violet-50\/20{--tw-gradient-to:rgba(245,243,255,.2) var(--tw-gradient-to-position)}.to-violet-50\/25{--tw-gradient-to:rgba(245,243,255,.25) var(--tw-gradient-to-position)}.to-violet-50\/30{--tw-gradient-to:rgba(245,243,255,.3) var(--tw-gradient-to-position)}.to-violet-50\/35{--tw-gradient-to:rgba(245,243,255,.35) var(--tw-gradient-to-position)}.to-violet-50\/40{--tw-gradient-to:rgba(245,243,255,.4) var(--tw-gradient-to-position)}.to-violet-50\/45{--tw-gradient-to:rgba(245,243,255,.45) var(--tw-gradient-to-position)}.to-violet-50\/5{--tw-gradient-to:rgba(245,243,255,.05) var(--tw-gradient-to-position)}.to-violet-50\/50{--tw-gradient-to:rgba(245,243,255,.5) var(--tw-gradient-to-position)}.to-violet-50\/55{--tw-gradient-to:rgba(245,243,255,.55) var(--tw-gradient-to-position)}.to-violet-50\/60{--tw-gradient-to:rgba(245,243,255,.6) var(--tw-gradient-to-position)}.to-violet-50\/65{--tw-gradient-to:rgba(245,243,255,.65) var(--tw-gradient-to-position)}.to-violet-50\/70{--tw-gradient-to:rgba(245,243,255,.7) var(--tw-gradient-to-position)}.to-violet-50\/75{--tw-gradient-to:rgba(245,243,255,.75) var(--tw-gradient-to-position)}.to-violet-50\/80{--tw-gradient-to:rgba(245,243,255,.8) var(--tw-gradient-to-position)}.to-violet-50\/85{--tw-gradient-to:rgba(245,243,255,.85) var(--tw-gradient-to-position)}.to-violet-50\/90{--tw-gradient-to:rgba(245,243,255,.9) var(--tw-gradient-to-position)}.to-violet-50\/95{--tw-gradient-to:rgba(245,243,255,.95) var(--tw-gradient-to-position)}.to-violet-500{--tw-gradient-to:#8b5cf6 var(--tw-gradient-to-position)}.to-violet-500\/0{--tw-gradient-to:rgba(139,92,246,0) var(--tw-gradient-to-position)}.to-violet-500\/10{--tw-gradient-to:rgba(139,92,246,.1) var(--tw-gradient-to-position)}.to-violet-500\/100{--tw-gradient-to:#8b5cf6 var(--tw-gradient-to-position)}.to-violet-500\/15{--tw-gradient-to:rgba(139,92,246,.15) var(--tw-gradient-to-position)}.to-violet-500\/20{--tw-gradient-to:rgba(139,92,246,.2) var(--tw-gradient-to-position)}.to-violet-500\/25{--tw-gradient-to:rgba(139,92,246,.25) var(--tw-gradient-to-position)}.to-violet-500\/30{--tw-gradient-to:rgba(139,92,246,.3) var(--tw-gradient-to-position)}.to-violet-500\/35{--tw-gradient-to:rgba(139,92,246,.35) var(--tw-gradient-to-position)}.to-violet-500\/40{--tw-gradient-to:rgba(139,92,246,.4) var(--tw-gradient-to-position)}.to-violet-500\/45{--tw-gradient-to:rgba(139,92,246,.45) var(--tw-gradient-to-position)}.to-violet-500\/5{--tw-gradient-to:rgba(139,92,246,.05) var(--tw-gradient-to-position)}.to-violet-500\/50{--tw-gradient-to:rgba(139,92,246,.5) var(--tw-gradient-to-position)}.to-violet-500\/55{--tw-gradient-to:rgba(139,92,246,.55) var(--tw-gradient-to-position)}.to-violet-500\/60{--tw-gradient-to:rgba(139,92,246,.6) var(--tw-gradient-to-position)}.to-violet-500\/65{--tw-gradient-to:rgba(139,92,246,.65) var(--tw-gradient-to-position)}.to-violet-500\/70{--tw-gradient-to:rgba(139,92,246,.7) var(--tw-gradient-to-position)}.to-violet-500\/75{--tw-gradient-to:rgba(139,92,246,.75) var(--tw-gradient-to-position)}.to-violet-500\/80{--tw-gradient-to:rgba(139,92,246,.8) var(--tw-gradient-to-position)}.to-violet-500\/85{--tw-gradient-to:rgba(139,92,246,.85) var(--tw-gradient-to-position)}.to-violet-500\/90{--tw-gradient-to:rgba(139,92,246,.9) var(--tw-gradient-to-position)}.to-violet-500\/95{--tw-gradient-to:rgba(139,92,246,.95) var(--tw-gradient-to-position)}.to-violet-950{--tw-gradient-to:#2e1065 var(--tw-gradient-to-position)}.to-violet-950\/0{--tw-gradient-to:rgba(46,16,101,0) var(--tw-gradient-to-position)}.to-violet-950\/10{--tw-gradient-to:rgba(46,16,101,.1) var(--tw-gradient-to-position)}.to-violet-950\/100{--tw-gradient-to:#2e1065 var(--tw-gradient-to-position)}.to-violet-950\/15{--tw-gradient-to:rgba(46,16,101,.15) var(--tw-gradient-to-position)}.to-violet-950\/20{--tw-gradient-to:rgba(46,16,101,.2) var(--tw-gradient-to-position)}.to-violet-950\/25{--tw-gradient-to:rgba(46,16,101,.25) var(--tw-gradient-to-position)}.to-violet-950\/30{--tw-gradient-to:rgba(46,16,101,.3) var(--tw-gradient-to-position)}.to-violet-950\/35{--tw-gradient-to:rgba(46,16,101,.35) var(--tw-gradient-to-position)}.to-violet-950\/40{--tw-gradient-to:rgba(46,16,101,.4) var(--tw-gradient-to-position)}.to-violet-950\/45{--tw-gradient-to:rgba(46,16,101,.45) var(--tw-gradient-to-position)}.to-violet-950\/5{--tw-gradient-to:rgba(46,16,101,.05) var(--tw-gradient-to-position)}.to-violet-950\/50{--tw-gradient-to:rgba(46,16,101,.5) var(--tw-gradient-to-position)}.to-violet-950\/55{--tw-gradient-to:rgba(46,16,101,.55) var(--tw-gradient-to-position)}.to-violet-950\/60{--tw-gradient-to:rgba(46,16,101,.6) var(--tw-gradient-to-position)}.to-violet-950\/65{--tw-gradient-to:rgba(46,16,101,.65) var(--tw-gradient-to-position)}.to-violet-950\/70{--tw-gradient-to:rgba(46,16,101,.7) var(--tw-gradient-to-position)}.to-violet-950\/75{--tw-gradient-to:rgba(46,16,101,.75) var(--tw-gradient-to-position)}.to-violet-950\/80{--tw-gradient-to:rgba(46,16,101,.8) var(--tw-gradient-to-position)}.to-violet-950\/85{--tw-gradient-to:rgba(46,16,101,.85) var(--tw-gradient-to-position)}.to-violet-950\/90{--tw-gradient-to:rgba(46,16,101,.9) var(--tw-gradient-to-position)}.to-violet-950\/95{--tw-gradient-to:rgba(46,16,101,.95) var(--tw-gradient-to-position)}.to-yellow-100{--tw-gradient-to:#fef9c3 var(--tw-gradient-to-position)}.to-yellow-100\/0{--tw-gradient-to:hsla(55,97%,88%,0) var(--tw-gradient-to-position)}.to-yellow-100\/10{--tw-gradient-to:hsla(55,97%,88%,.1) var(--tw-gradient-to-position)}.to-yellow-100\/100{--tw-gradient-to:#fef9c3 var(--tw-gradient-to-position)}.to-yellow-100\/15{--tw-gradient-to:hsla(55,97%,88%,.15) var(--tw-gradient-to-position)}.to-yellow-100\/20{--tw-gradient-to:hsla(55,97%,88%,.2) var(--tw-gradient-to-position)}.to-yellow-100\/25{--tw-gradient-to:hsla(55,97%,88%,.25) var(--tw-gradient-to-position)}.to-yellow-100\/30{--tw-gradient-to:hsla(55,97%,88%,.3) var(--tw-gradient-to-position)}.to-yellow-100\/35{--tw-gradient-to:hsla(55,97%,88%,.35) var(--tw-gradient-to-position)}.to-yellow-100\/40{--tw-gradient-to:hsla(55,97%,88%,.4) var(--tw-gradient-to-position)}.to-yellow-100\/45{--tw-gradient-to:hsla(55,97%,88%,.45) var(--tw-gradient-to-position)}.to-yellow-100\/5{--tw-gradient-to:hsla(55,97%,88%,.05) var(--tw-gradient-to-position)}.to-yellow-100\/50{--tw-gradient-to:hsla(55,97%,88%,.5) var(--tw-gradient-to-position)}.to-yellow-100\/55{--tw-gradient-to:hsla(55,97%,88%,.55) var(--tw-gradient-to-position)}.to-yellow-100\/60{--tw-gradient-to:hsla(55,97%,88%,.6) var(--tw-gradient-to-position)}.to-yellow-100\/65{--tw-gradient-to:hsla(55,97%,88%,.65) var(--tw-gradient-to-position)}.to-yellow-100\/70{--tw-gradient-to:hsla(55,97%,88%,.7) var(--tw-gradient-to-position)}.to-yellow-100\/75{--tw-gradient-to:hsla(55,97%,88%,.75) var(--tw-gradient-to-position)}.to-yellow-100\/80{--tw-gradient-to:hsla(55,97%,88%,.8) var(--tw-gradient-to-position)}.to-yellow-100\/85{--tw-gradient-to:hsla(55,97%,88%,.85) var(--tw-gradient-to-position)}.to-yellow-100\/90{--tw-gradient-to:hsla(55,97%,88%,.9) var(--tw-gradient-to-position)}.to-yellow-100\/95{--tw-gradient-to:hsla(55,97%,88%,.95) var(--tw-gradient-to-position)}.to-yellow-200{--tw-gradient-to:#fef08a var(--tw-gradient-to-position)}.to-yellow-200\/0{--tw-gradient-to:hsla(53,98%,77%,0) var(--tw-gradient-to-position)}.to-yellow-200\/10{--tw-gradient-to:hsla(53,98%,77%,.1) var(--tw-gradient-to-position)}.to-yellow-200\/100{--tw-gradient-to:#fef08a var(--tw-gradient-to-position)}.to-yellow-200\/15{--tw-gradient-to:hsla(53,98%,77%,.15) var(--tw-gradient-to-position)}.to-yellow-200\/20{--tw-gradient-to:hsla(53,98%,77%,.2) var(--tw-gradient-to-position)}.to-yellow-200\/25{--tw-gradient-to:hsla(53,98%,77%,.25) var(--tw-gradient-to-position)}.to-yellow-200\/30{--tw-gradient-to:hsla(53,98%,77%,.3) var(--tw-gradient-to-position)}.to-yellow-200\/35{--tw-gradient-to:hsla(53,98%,77%,.35) var(--tw-gradient-to-position)}.to-yellow-200\/40{--tw-gradient-to:hsla(53,98%,77%,.4) var(--tw-gradient-to-position)}.to-yellow-200\/45{--tw-gradient-to:hsla(53,98%,77%,.45) var(--tw-gradient-to-position)}.to-yellow-200\/5{--tw-gradient-to:hsla(53,98%,77%,.05) var(--tw-gradient-to-position)}.to-yellow-200\/50{--tw-gradient-to:hsla(53,98%,77%,.5) var(--tw-gradient-to-position)}.to-yellow-200\/55{--tw-gradient-to:hsla(53,98%,77%,.55) var(--tw-gradient-to-position)}.to-yellow-200\/60{--tw-gradient-to:hsla(53,98%,77%,.6) var(--tw-gradient-to-position)}.to-yellow-200\/65{--tw-gradient-to:hsla(53,98%,77%,.65) var(--tw-gradient-to-position)}.to-yellow-200\/70{--tw-gradient-to:hsla(53,98%,77%,.7) var(--tw-gradient-to-position)}.to-yellow-200\/75{--tw-gradient-to:hsla(53,98%,77%,.75) var(--tw-gradient-to-position)}.to-yellow-200\/80{--tw-gradient-to:hsla(53,98%,77%,.8) var(--tw-gradient-to-position)}.to-yellow-200\/85{--tw-gradient-to:hsla(53,98%,77%,.85) var(--tw-gradient-to-position)}.to-yellow-200\/90{--tw-gradient-to:hsla(53,98%,77%,.9) var(--tw-gradient-to-position)}.to-yellow-200\/95{--tw-gradient-to:hsla(53,98%,77%,.95) var(--tw-gradient-to-position)}.to-yellow-50{--tw-gradient-to:#fefce8 var(--tw-gradient-to-position)}.to-yellow-50\/0{--tw-gradient-to:hsla(55,92%,95%,0) var(--tw-gradient-to-position)}.to-yellow-50\/10{--tw-gradient-to:hsla(55,92%,95%,.1) var(--tw-gradient-to-position)}.to-yellow-50\/100{--tw-gradient-to:#fefce8 var(--tw-gradient-to-position)}.to-yellow-50\/15{--tw-gradient-to:hsla(55,92%,95%,.15) var(--tw-gradient-to-position)}.to-yellow-50\/20{--tw-gradient-to:hsla(55,92%,95%,.2) var(--tw-gradient-to-position)}.to-yellow-50\/25{--tw-gradient-to:hsla(55,92%,95%,.25) var(--tw-gradient-to-position)}.to-yellow-50\/30{--tw-gradient-to:hsla(55,92%,95%,.3) var(--tw-gradient-to-position)}.to-yellow-50\/35{--tw-gradient-to:hsla(55,92%,95%,.35) var(--tw-gradient-to-position)}.to-yellow-50\/40{--tw-gradient-to:hsla(55,92%,95%,.4) var(--tw-gradient-to-position)}.to-yellow-50\/45{--tw-gradient-to:hsla(55,92%,95%,.45) var(--tw-gradient-to-position)}.to-yellow-50\/5{--tw-gradient-to:hsla(55,92%,95%,.05) var(--tw-gradient-to-position)}.to-yellow-50\/50{--tw-gradient-to:hsla(55,92%,95%,.5) var(--tw-gradient-to-position)}.to-yellow-50\/55{--tw-gradient-to:hsla(55,92%,95%,.55) var(--tw-gradient-to-position)}.to-yellow-50\/60{--tw-gradient-to:hsla(55,92%,95%,.6) var(--tw-gradient-to-position)}.to-yellow-50\/65{--tw-gradient-to:hsla(55,92%,95%,.65) var(--tw-gradient-to-position)}.to-yellow-50\/70{--tw-gradient-to:hsla(55,92%,95%,.7) var(--tw-gradient-to-position)}.to-yellow-50\/75{--tw-gradient-to:hsla(55,92%,95%,.75) var(--tw-gradient-to-position)}.to-yellow-50\/80{--tw-gradient-to:hsla(55,92%,95%,.8) var(--tw-gradient-to-position)}.to-yellow-50\/85{--tw-gradient-to:hsla(55,92%,95%,.85) var(--tw-gradient-to-position)}.to-yellow-50\/90{--tw-gradient-to:hsla(55,92%,95%,.9) var(--tw-gradient-to-position)}.to-yellow-50\/95{--tw-gradient-to:hsla(55,92%,95%,.95) var(--tw-gradient-to-position)}.to-yellow-500{--tw-gradient-to:#eab308 var(--tw-gradient-to-position)}.to-yellow-500\/0{--tw-gradient-to:rgba(234,179,8,0) var(--tw-gradient-to-position)}.to-yellow-500\/10{--tw-gradient-to:rgba(234,179,8,.1) var(--tw-gradient-to-position)}.to-yellow-500\/100{--tw-gradient-to:#eab308 var(--tw-gradient-to-position)}.to-yellow-500\/15{--tw-gradient-to:rgba(234,179,8,.15) var(--tw-gradient-to-position)}.to-yellow-500\/20{--tw-gradient-to:rgba(234,179,8,.2) var(--tw-gradient-to-position)}.to-yellow-500\/25{--tw-gradient-to:rgba(234,179,8,.25) var(--tw-gradient-to-position)}.to-yellow-500\/30{--tw-gradient-to:rgba(234,179,8,.3) var(--tw-gradient-to-position)}.to-yellow-500\/35{--tw-gradient-to:rgba(234,179,8,.35) var(--tw-gradient-to-position)}.to-yellow-500\/40{--tw-gradient-to:rgba(234,179,8,.4) var(--tw-gradient-to-position)}.to-yellow-500\/45{--tw-gradient-to:rgba(234,179,8,.45) var(--tw-gradient-to-position)}.to-yellow-500\/5{--tw-gradient-to:rgba(234,179,8,.05) var(--tw-gradient-to-position)}.to-yellow-500\/50{--tw-gradient-to:rgba(234,179,8,.5) var(--tw-gradient-to-position)}.to-yellow-500\/55{--tw-gradient-to:rgba(234,179,8,.55) var(--tw-gradient-to-position)}.to-yellow-500\/60{--tw-gradient-to:rgba(234,179,8,.6) var(--tw-gradient-to-position)}.to-yellow-500\/65{--tw-gradient-to:rgba(234,179,8,.65) var(--tw-gradient-to-position)}.to-yellow-500\/70{--tw-gradient-to:rgba(234,179,8,.7) var(--tw-gradient-to-position)}.to-yellow-500\/75{--tw-gradient-to:rgba(234,179,8,.75) var(--tw-gradient-to-position)}.to-yellow-500\/80{--tw-gradient-to:rgba(234,179,8,.8) var(--tw-gradient-to-position)}.to-yellow-500\/85{--tw-gradient-to:rgba(234,179,8,.85) var(--tw-gradient-to-position)}.to-yellow-500\/90{--tw-gradient-to:rgba(234,179,8,.9) var(--tw-gradient-to-position)}.to-yellow-500\/95{--tw-gradient-to:rgba(234,179,8,.95) var(--tw-gradient-to-position)}.to-yellow-950{--tw-gradient-to:#422006 var(--tw-gradient-to-position)}.to-yellow-950\/0{--tw-gradient-to:rgba(66,32,6,0) var(--tw-gradient-to-position)}.to-yellow-950\/10{--tw-gradient-to:rgba(66,32,6,.1) var(--tw-gradient-to-position)}.to-yellow-950\/100{--tw-gradient-to:#422006 var(--tw-gradient-to-position)}.to-yellow-950\/15{--tw-gradient-to:rgba(66,32,6,.15) var(--tw-gradient-to-position)}.to-yellow-950\/20{--tw-gradient-to:rgba(66,32,6,.2) var(--tw-gradient-to-position)}.to-yellow-950\/25{--tw-gradient-to:rgba(66,32,6,.25) var(--tw-gradient-to-position)}.to-yellow-950\/30{--tw-gradient-to:rgba(66,32,6,.3) var(--tw-gradient-to-position)}.to-yellow-950\/35{--tw-gradient-to:rgba(66,32,6,.35) var(--tw-gradient-to-position)}.to-yellow-950\/40{--tw-gradient-to:rgba(66,32,6,.4) var(--tw-gradient-to-position)}.to-yellow-950\/45{--tw-gradient-to:rgba(66,32,6,.45) var(--tw-gradient-to-position)}.to-yellow-950\/5{--tw-gradient-to:rgba(66,32,6,.05) var(--tw-gradient-to-position)}.to-yellow-950\/50{--tw-gradient-to:rgba(66,32,6,.5) var(--tw-gradient-to-position)}.to-yellow-950\/55{--tw-gradient-to:rgba(66,32,6,.55) var(--tw-gradient-to-position)}.to-yellow-950\/60{--tw-gradient-to:rgba(66,32,6,.6) var(--tw-gradient-to-position)}.to-yellow-950\/65{--tw-gradient-to:rgba(66,32,6,.65) var(--tw-gradient-to-position)}.to-yellow-950\/70{--tw-gradient-to:rgba(66,32,6,.7) var(--tw-gradient-to-position)}.to-yellow-950\/75{--tw-gradient-to:rgba(66,32,6,.75) var(--tw-gradient-to-position)}.to-yellow-950\/80{--tw-gradient-to:rgba(66,32,6,.8) var(--tw-gradient-to-position)}.to-yellow-950\/85{--tw-gradient-to:rgba(66,32,6,.85) var(--tw-gradient-to-position)}.to-yellow-950\/90{--tw-gradient-to:rgba(66,32,6,.9) var(--tw-gradient-to-position)}.to-yellow-950\/95{--tw-gradient-to:rgba(66,32,6,.95) var(--tw-gradient-to-position)}.to-zinc-100{--tw-gradient-to:#f4f4f5 var(--tw-gradient-to-position)}.to-zinc-100\/0{--tw-gradient-to:hsla(240,5%,96%,0) var(--tw-gradient-to-position)}.to-zinc-100\/10{--tw-gradient-to:hsla(240,5%,96%,.1) var(--tw-gradient-to-position)}.to-zinc-100\/100{--tw-gradient-to:#f4f4f5 var(--tw-gradient-to-position)}.to-zinc-100\/15{--tw-gradient-to:hsla(240,5%,96%,.15) var(--tw-gradient-to-position)}.to-zinc-100\/20{--tw-gradient-to:hsla(240,5%,96%,.2) var(--tw-gradient-to-position)}.to-zinc-100\/25{--tw-gradient-to:hsla(240,5%,96%,.25) var(--tw-gradient-to-position)}.to-zinc-100\/30{--tw-gradient-to:hsla(240,5%,96%,.3) var(--tw-gradient-to-position)}.to-zinc-100\/35{--tw-gradient-to:hsla(240,5%,96%,.35) var(--tw-gradient-to-position)}.to-zinc-100\/40{--tw-gradient-to:hsla(240,5%,96%,.4) var(--tw-gradient-to-position)}.to-zinc-100\/45{--tw-gradient-to:hsla(240,5%,96%,.45) var(--tw-gradient-to-position)}.to-zinc-100\/5{--tw-gradient-to:hsla(240,5%,96%,.05) var(--tw-gradient-to-position)}.to-zinc-100\/50{--tw-gradient-to:hsla(240,5%,96%,.5) var(--tw-gradient-to-position)}.to-zinc-100\/55{--tw-gradient-to:hsla(240,5%,96%,.55) var(--tw-gradient-to-position)}.to-zinc-100\/60{--tw-gradient-to:hsla(240,5%,96%,.6) var(--tw-gradient-to-position)}.to-zinc-100\/65{--tw-gradient-to:hsla(240,5%,96%,.65) var(--tw-gradient-to-position)}.to-zinc-100\/70{--tw-gradient-to:hsla(240,5%,96%,.7) var(--tw-gradient-to-position)}.to-zinc-100\/75{--tw-gradient-to:hsla(240,5%,96%,.75) var(--tw-gradient-to-position)}.to-zinc-100\/80{--tw-gradient-to:hsla(240,5%,96%,.8) var(--tw-gradient-to-position)}.to-zinc-100\/85{--tw-gradient-to:hsla(240,5%,96%,.85) var(--tw-gradient-to-position)}.to-zinc-100\/90{--tw-gradient-to:hsla(240,5%,96%,.9) var(--tw-gradient-to-position)}.to-zinc-100\/95{--tw-gradient-to:hsla(240,5%,96%,.95) var(--tw-gradient-to-position)}.to-zinc-200{--tw-gradient-to:#e4e4e7 var(--tw-gradient-to-position)}.to-zinc-200\/0{--tw-gradient-to:hsla(240,6%,90%,0) var(--tw-gradient-to-position)}.to-zinc-200\/10{--tw-gradient-to:hsla(240,6%,90%,.1) var(--tw-gradient-to-position)}.to-zinc-200\/100{--tw-gradient-to:#e4e4e7 var(--tw-gradient-to-position)}.to-zinc-200\/15{--tw-gradient-to:hsla(240,6%,90%,.15) var(--tw-gradient-to-position)}.to-zinc-200\/20{--tw-gradient-to:hsla(240,6%,90%,.2) var(--tw-gradient-to-position)}.to-zinc-200\/25{--tw-gradient-to:hsla(240,6%,90%,.25) var(--tw-gradient-to-position)}.to-zinc-200\/30{--tw-gradient-to:hsla(240,6%,90%,.3) var(--tw-gradient-to-position)}.to-zinc-200\/35{--tw-gradient-to:hsla(240,6%,90%,.35) var(--tw-gradient-to-position)}.to-zinc-200\/40{--tw-gradient-to:hsla(240,6%,90%,.4) var(--tw-gradient-to-position)}.to-zinc-200\/45{--tw-gradient-to:hsla(240,6%,90%,.45) var(--tw-gradient-to-position)}.to-zinc-200\/5{--tw-gradient-to:hsla(240,6%,90%,.05) var(--tw-gradient-to-position)}.to-zinc-200\/50{--tw-gradient-to:hsla(240,6%,90%,.5) var(--tw-gradient-to-position)}.to-zinc-200\/55{--tw-gradient-to:hsla(240,6%,90%,.55) var(--tw-gradient-to-position)}.to-zinc-200\/60{--tw-gradient-to:hsla(240,6%,90%,.6) var(--tw-gradient-to-position)}.to-zinc-200\/65{--tw-gradient-to:hsla(240,6%,90%,.65) var(--tw-gradient-to-position)}.to-zinc-200\/70{--tw-gradient-to:hsla(240,6%,90%,.7) var(--tw-gradient-to-position)}.to-zinc-200\/75{--tw-gradient-to:hsla(240,6%,90%,.75) var(--tw-gradient-to-position)}.to-zinc-200\/80{--tw-gradient-to:hsla(240,6%,90%,.8) var(--tw-gradient-to-position)}.to-zinc-200\/85{--tw-gradient-to:hsla(240,6%,90%,.85) var(--tw-gradient-to-position)}.to-zinc-200\/90{--tw-gradient-to:hsla(240,6%,90%,.9) var(--tw-gradient-to-position)}.to-zinc-200\/95{--tw-gradient-to:hsla(240,6%,90%,.95) var(--tw-gradient-to-position)}.to-zinc-50{--tw-gradient-to:#fafafa var(--tw-gradient-to-position)}.to-zinc-50\/0{--tw-gradient-to:hsla(0,0%,98%,0) var(--tw-gradient-to-position)}.to-zinc-50\/10{--tw-gradient-to:hsla(0,0%,98%,.1) var(--tw-gradient-to-position)}.to-zinc-50\/100{--tw-gradient-to:#fafafa var(--tw-gradient-to-position)}.to-zinc-50\/15{--tw-gradient-to:hsla(0,0%,98%,.15) var(--tw-gradient-to-position)}.to-zinc-50\/20{--tw-gradient-to:hsla(0,0%,98%,.2) var(--tw-gradient-to-position)}.to-zinc-50\/25{--tw-gradient-to:hsla(0,0%,98%,.25) var(--tw-gradient-to-position)}.to-zinc-50\/30{--tw-gradient-to:hsla(0,0%,98%,.3) var(--tw-gradient-to-position)}.to-zinc-50\/35{--tw-gradient-to:hsla(0,0%,98%,.35) var(--tw-gradient-to-position)}.to-zinc-50\/40{--tw-gradient-to:hsla(0,0%,98%,.4) var(--tw-gradient-to-position)}.to-zinc-50\/45{--tw-gradient-to:hsla(0,0%,98%,.45) var(--tw-gradient-to-position)}.to-zinc-50\/5{--tw-gradient-to:hsla(0,0%,98%,.05) var(--tw-gradient-to-position)}.to-zinc-50\/50{--tw-gradient-to:hsla(0,0%,98%,.5) var(--tw-gradient-to-position)}.to-zinc-50\/55{--tw-gradient-to:hsla(0,0%,98%,.55) var(--tw-gradient-to-position)}.to-zinc-50\/60{--tw-gradient-to:hsla(0,0%,98%,.6) var(--tw-gradient-to-position)}.to-zinc-50\/65{--tw-gradient-to:hsla(0,0%,98%,.65) var(--tw-gradient-to-position)}.to-zinc-50\/70{--tw-gradient-to:hsla(0,0%,98%,.7) var(--tw-gradient-to-position)}.to-zinc-50\/75{--tw-gradient-to:hsla(0,0%,98%,.75) var(--tw-gradient-to-position)}.to-zinc-50\/80{--tw-gradient-to:hsla(0,0%,98%,.8) var(--tw-gradient-to-position)}.to-zinc-50\/85{--tw-gradient-to:hsla(0,0%,98%,.85) var(--tw-gradient-to-position)}.to-zinc-50\/90{--tw-gradient-to:hsla(0,0%,98%,.9) var(--tw-gradient-to-position)}.to-zinc-50\/95{--tw-gradient-to:hsla(0,0%,98%,.95) var(--tw-gradient-to-position)}.to-zinc-500{--tw-gradient-to:#71717a var(--tw-gradient-to-position)}.to-zinc-500\/0{--tw-gradient-to:hsla(240,4%,46%,0) var(--tw-gradient-to-position)}.to-zinc-500\/10{--tw-gradient-to:hsla(240,4%,46%,.1) var(--tw-gradient-to-position)}.to-zinc-500\/100{--tw-gradient-to:#71717a var(--tw-gradient-to-position)}.to-zinc-500\/15{--tw-gradient-to:hsla(240,4%,46%,.15) var(--tw-gradient-to-position)}.to-zinc-500\/20{--tw-gradient-to:hsla(240,4%,46%,.2) var(--tw-gradient-to-position)}.to-zinc-500\/25{--tw-gradient-to:hsla(240,4%,46%,.25) var(--tw-gradient-to-position)}.to-zinc-500\/30{--tw-gradient-to:hsla(240,4%,46%,.3) var(--tw-gradient-to-position)}.to-zinc-500\/35{--tw-gradient-to:hsla(240,4%,46%,.35) var(--tw-gradient-to-position)}.to-zinc-500\/40{--tw-gradient-to:hsla(240,4%,46%,.4) var(--tw-gradient-to-position)}.to-zinc-500\/45{--tw-gradient-to:hsla(240,4%,46%,.45) var(--tw-gradient-to-position)}.to-zinc-500\/5{--tw-gradient-to:hsla(240,4%,46%,.05) var(--tw-gradient-to-position)}.to-zinc-500\/50{--tw-gradient-to:hsla(240,4%,46%,.5) var(--tw-gradient-to-position)}.to-zinc-500\/55{--tw-gradient-to:hsla(240,4%,46%,.55) var(--tw-gradient-to-position)}.to-zinc-500\/60{--tw-gradient-to:hsla(240,4%,46%,.6) var(--tw-gradient-to-position)}.to-zinc-500\/65{--tw-gradient-to:hsla(240,4%,46%,.65) var(--tw-gradient-to-position)}.to-zinc-500\/70{--tw-gradient-to:hsla(240,4%,46%,.7) var(--tw-gradient-to-position)}.to-zinc-500\/75{--tw-gradient-to:hsla(240,4%,46%,.75) var(--tw-gradient-to-position)}.to-zinc-500\/80{--tw-gradient-to:hsla(240,4%,46%,.8) var(--tw-gradient-to-position)}.to-zinc-500\/85{--tw-gradient-to:hsla(240,4%,46%,.85) var(--tw-gradient-to-position)}.to-zinc-500\/90{--tw-gradient-to:hsla(240,4%,46%,.9) var(--tw-gradient-to-position)}.to-zinc-500\/95{--tw-gradient-to:hsla(240,4%,46%,.95) var(--tw-gradient-to-position)}.to-zinc-950{--tw-gradient-to:#09090b var(--tw-gradient-to-position)}.to-zinc-950\/0{--tw-gradient-to:rgba(9,9,11,0) var(--tw-gradient-to-position)}.to-zinc-950\/10{--tw-gradient-to:rgba(9,9,11,.1) var(--tw-gradient-to-position)}.to-zinc-950\/100{--tw-gradient-to:#09090b var(--tw-gradient-to-position)}.to-zinc-950\/15{--tw-gradient-to:rgba(9,9,11,.15) var(--tw-gradient-to-position)}.to-zinc-950\/20{--tw-gradient-to:rgba(9,9,11,.2) var(--tw-gradient-to-position)}.to-zinc-950\/25{--tw-gradient-to:rgba(9,9,11,.25) var(--tw-gradient-to-position)}.to-zinc-950\/30{--tw-gradient-to:rgba(9,9,11,.3) var(--tw-gradient-to-position)}.to-zinc-950\/35{--tw-gradient-to:rgba(9,9,11,.35) var(--tw-gradient-to-position)}.to-zinc-950\/40{--tw-gradient-to:rgba(9,9,11,.4) var(--tw-gradient-to-position)}.to-zinc-950\/45{--tw-gradient-to:rgba(9,9,11,.45) var(--tw-gradient-to-position)}.to-zinc-950\/5{--tw-gradient-to:rgba(9,9,11,.05) var(--tw-gradient-to-position)}.to-zinc-950\/50{--tw-gradient-to:rgba(9,9,11,.5) var(--tw-gradient-to-position)}.to-zinc-950\/55{--tw-gradient-to:rgba(9,9,11,.55) var(--tw-gradient-to-position)}.to-zinc-950\/60{--tw-gradient-to:rgba(9,9,11,.6) var(--tw-gradient-to-position)}.to-zinc-950\/65{--tw-gradient-to:rgba(9,9,11,.65) var(--tw-gradient-to-position)}.to-zinc-950\/70{--tw-gradient-to:rgba(9,9,11,.7) var(--tw-gradient-to-position)}.to-zinc-950\/75{--tw-gradient-to:rgba(9,9,11,.75) var(--tw-gradient-to-position)}.to-zinc-950\/80{--tw-gradient-to:rgba(9,9,11,.8) var(--tw-gradient-to-position)}.to-zinc-950\/85{--tw-gradient-to:rgba(9,9,11,.85) var(--tw-gradient-to-position)}.to-zinc-950\/90{--tw-gradient-to:rgba(9,9,11,.9) var(--tw-gradient-to-position)}.to-zinc-950\/95{--tw-gradient-to:rgba(9,9,11,.95) var(--tw-gradient-to-position)}.fill-current{fill:currentColor}.fill-zinc-950{fill:#09090b}.object-cover{-o-object-fit:cover;object-fit:cover}.p-0{padding:0}.p-1{padding:.25rem}.p-1\.5{padding:.375rem}.p-2{padding:.5rem}.p-3{padding:.75rem}.p-4{padding:1rem}.p-6{padding:1.5rem}.p-\[0\.1rem\]{padding:.1rem}.p-\[0\.5rem\]{padding:.5rem}.p-\[1px\]{padding:1px}.px-1{padding-left:.25rem;padding-right:.25rem}.px-1\.5{padding-left:.375rem;padding-right:.375rem}.px-12{padding-left:3rem;padding-right:3rem}.px-2{padding-left:.5rem;padding-right:.5rem}.px-2\.5{padding-left:.625rem;padding-right:.625rem}.px-3{padding-left:.75rem;padding-right:.75rem}.px-4{padding-left:1rem;padding-right:1rem}.px-5{padding-left:1.25rem;padding-right:1.25rem}.px-8{padding-left:2rem;padding-right:2rem}.py-1{padding-top:.25rem;padding-bottom:.25rem}.py-1\.5{padding-top:.375rem;padding-bottom:.375rem}.py-2{padding-top:.5rem;padding-bottom:.5rem}.py-3{padding-top:.75rem;padding-bottom:.75rem}.py-4{padding-top:1rem;padding-bottom:1rem}.py-6{padding-top:1.5rem;padding-bottom:1.5rem}.pb-0{padding-bottom:0}.pb-12{padding-bottom:3rem}.pb-2{padding-bottom:.5rem}.pb-3{padding-bottom:.75rem}.pb-4{padding-bottom:1rem}.pb-6{padding-bottom:1.5rem}.pl-0{padding-left:0}.pl-4{padding-left:1rem}.pl-8{padding-left:2rem}.pr-2{padding-right:.5rem}.pr-8{padding-right:2rem}.pt-0{padding-top:0}.pt-1{padding-top:.25rem}.pt-2{padding-top:.5rem}.pt-3{padding-top:.75rem}.pt-6{padding-top:1.5rem}.text-left{text-align:left}.text-center{text-align:center}.align-middle{vertical-align:middle}.text-2xl{font-size:1.5rem;line-height:2rem}.text-3xl{font-size:1.875rem;line-height:2.25rem}.text-5xl{font-size:3rem;line-height:1}.text-lg{font-size:1.125rem;line-height:1.75rem}.text-sm{font-size:.875rem;line-height:1.25rem}.text-xl{font-size:1.25rem;line-height:1.75rem}.text-xs{font-size:.75rem;line-height:1rem}.font-bold{font-weight:700}.font-medium{font-weight:500}.font-semibold{font-weight:600}.leading-none{line-height:1}.leading-normal{line-height:1.5}.tracking-tight{letter-spacing:-.025em}.tracking-widest{letter-spacing:.1em}.text-accent-foreground{color:hsl(var(--accent-foreground))}.text-amber-100{--tw-text-opacity:1;color:rgb(254 243 199/var(--tw-text-opacity,1))}.text-amber-100\/0{color:hsla(48,96%,89%,0)}.text-amber-100\/10{color:hsla(48,96%,89%,.1)}.text-amber-100\/100{color:#fef3c7}.text-amber-100\/15{color:hsla(48,96%,89%,.15)}.text-amber-100\/20{color:hsla(48,96%,89%,.2)}.text-amber-100\/25{color:hsla(48,96%,89%,.25)}.text-amber-100\/30{color:hsla(48,96%,89%,.3)}.text-amber-100\/35{color:hsla(48,96%,89%,.35)}.text-amber-100\/40{color:hsla(48,96%,89%,.4)}.text-amber-100\/45{color:hsla(48,96%,89%,.45)}.text-amber-100\/5{color:hsla(48,96%,89%,.05)}.text-amber-100\/50{color:hsla(48,96%,89%,.5)}.text-amber-100\/55{color:hsla(48,96%,89%,.55)}.text-amber-100\/60{color:hsla(48,96%,89%,.6)}.text-amber-100\/65{color:hsla(48,96%,89%,.65)}.text-amber-100\/70{color:hsla(48,96%,89%,.7)}.text-amber-100\/75{color:hsla(48,96%,89%,.75)}.text-amber-100\/80{color:hsla(48,96%,89%,.8)}.text-amber-100\/85{color:hsla(48,96%,89%,.85)}.text-amber-100\/90{color:hsla(48,96%,89%,.9)}.text-amber-100\/95{color:hsla(48,96%,89%,.95)}.text-amber-200{--tw-text-opacity:1;color:rgb(253 230 138/var(--tw-text-opacity,1))}.text-amber-200\/0{color:hsla(48,97%,77%,0)}.text-amber-200\/10{color:hsla(48,97%,77%,.1)}.text-amber-200\/100{color:#fde68a}.text-amber-200\/15{color:hsla(48,97%,77%,.15)}.text-amber-200\/20{color:hsla(48,97%,77%,.2)}.text-amber-200\/25{color:hsla(48,97%,77%,.25)}.text-amber-200\/30{color:hsla(48,97%,77%,.3)}.text-amber-200\/35{color:hsla(48,97%,77%,.35)}.text-amber-200\/40{color:hsla(48,97%,77%,.4)}.text-amber-200\/45{color:hsla(48,97%,77%,.45)}.text-amber-200\/5{color:hsla(48,97%,77%,.05)}.text-amber-200\/50{color:hsla(48,97%,77%,.5)}.text-amber-200\/55{color:hsla(48,97%,77%,.55)}.text-amber-200\/60{color:hsla(48,97%,77%,.6)}.text-amber-200\/65{color:hsla(48,97%,77%,.65)}.text-amber-200\/70{color:hsla(48,97%,77%,.7)}.text-amber-200\/75{color:hsla(48,97%,77%,.75)}.text-amber-200\/80{color:hsla(48,97%,77%,.8)}.text-amber-200\/85{color:hsla(48,97%,77%,.85)}.text-amber-200\/90{color:hsla(48,97%,77%,.9)}.text-amber-200\/95{color:hsla(48,97%,77%,.95)}.text-amber-50{--tw-text-opacity:1;color:rgb(255 251 235/var(--tw-text-opacity,1))}.text-amber-50\/0{color:rgba(255,251,235,0)}.text-amber-50\/10{color:rgba(255,251,235,.1)}.text-amber-50\/100{color:#fffbeb}.text-amber-50\/15{color:rgba(255,251,235,.15)}.text-amber-50\/20{color:rgba(255,251,235,.2)}.text-amber-50\/25{color:rgba(255,251,235,.25)}.text-amber-50\/30{color:rgba(255,251,235,.3)}.text-amber-50\/35{color:rgba(255,251,235,.35)}.text-amber-50\/40{color:rgba(255,251,235,.4)}.text-amber-50\/45{color:rgba(255,251,235,.45)}.text-amber-50\/5{color:rgba(255,251,235,.05)}.text-amber-50\/50{color:rgba(255,251,235,.5)}.text-amber-50\/55{color:rgba(255,251,235,.55)}.text-amber-50\/60{color:rgba(255,251,235,.6)}.text-amber-50\/65{color:rgba(255,251,235,.65)}.text-amber-50\/70{color:rgba(255,251,235,.7)}.text-amber-50\/75{color:rgba(255,251,235,.75)}.text-amber-50\/80{color:rgba(255,251,235,.8)}.text-amber-50\/85{color:rgba(255,251,235,.85)}.text-amber-50\/90{color:rgba(255,251,235,.9)}.text-amber-50\/95{color:rgba(255,251,235,.95)}.text-amber-500{--tw-text-opacity:1;color:rgb(245 158 11/var(--tw-text-opacity,1))}.text-amber-500\/0{color:rgba(245,158,11,0)}.text-amber-500\/10{color:rgba(245,158,11,.1)}.text-amber-500\/100{color:#f59e0b}.text-amber-500\/15{color:rgba(245,158,11,.15)}.text-amber-500\/20{color:rgba(245,158,11,.2)}.text-amber-500\/25{color:rgba(245,158,11,.25)}.text-amber-500\/30{color:rgba(245,158,11,.3)}.text-amber-500\/35{color:rgba(245,158,11,.35)}.text-amber-500\/40{color:rgba(245,158,11,.4)}.text-amber-500\/45{color:rgba(245,158,11,.45)}.text-amber-500\/5{color:rgba(245,158,11,.05)}.text-amber-500\/50{color:rgba(245,158,11,.5)}.text-amber-500\/55{color:rgba(245,158,11,.55)}.text-amber-500\/60{color:rgba(245,158,11,.6)}.text-amber-500\/65{color:rgba(245,158,11,.65)}.text-amber-500\/70{color:rgba(245,158,11,.7)}.text-amber-500\/75{color:rgba(245,158,11,.75)}.text-amber-500\/80{color:rgba(245,158,11,.8)}.text-amber-500\/85{color:rgba(245,158,11,.85)}.text-amber-500\/90{color:rgba(245,158,11,.9)}.text-amber-500\/95{color:rgba(245,158,11,.95)}.text-amber-950{--tw-text-opacity:1;color:rgb(69 26 3/var(--tw-text-opacity,1))}.text-amber-950\/0{color:rgba(69,26,3,0)}.text-amber-950\/10{color:rgba(69,26,3,.1)}.text-amber-950\/100{color:#451a03}.text-amber-950\/15{color:rgba(69,26,3,.15)}.text-amber-950\/20{color:rgba(69,26,3,.2)}.text-amber-950\/25{color:rgba(69,26,3,.25)}.text-amber-950\/30{color:rgba(69,26,3,.3)}.text-amber-950\/35{color:rgba(69,26,3,.35)}.text-amber-950\/40{color:rgba(69,26,3,.4)}.text-amber-950\/45{color:rgba(69,26,3,.45)}.text-amber-950\/5{color:rgba(69,26,3,.05)}.text-amber-950\/50{color:rgba(69,26,3,.5)}.text-amber-950\/55{color:rgba(69,26,3,.55)}.text-amber-950\/60{color:rgba(69,26,3,.6)}.text-amber-950\/65{color:rgba(69,26,3,.65)}.text-amber-950\/70{color:rgba(69,26,3,.7)}.text-amber-950\/75{color:rgba(69,26,3,.75)}.text-amber-950\/80{color:rgba(69,26,3,.8)}.text-amber-950\/85{color:rgba(69,26,3,.85)}.text-amber-950\/90{color:rgba(69,26,3,.9)}.text-amber-950\/95{color:rgba(69,26,3,.95)}.text-blue-100{--tw-text-opacity:1;color:rgb(219 234 254/var(--tw-text-opacity,1))}.text-blue-100\/0{color:rgba(219,234,254,0)}.text-blue-100\/10{color:rgba(219,234,254,.1)}.text-blue-100\/100{color:#dbeafe}.text-blue-100\/15{color:rgba(219,234,254,.15)}.text-blue-100\/20{color:rgba(219,234,254,.2)}.text-blue-100\/25{color:rgba(219,234,254,.25)}.text-blue-100\/30{color:rgba(219,234,254,.3)}.text-blue-100\/35{color:rgba(219,234,254,.35)}.text-blue-100\/40{color:rgba(219,234,254,.4)}.text-blue-100\/45{color:rgba(219,234,254,.45)}.text-blue-100\/5{color:rgba(219,234,254,.05)}.text-blue-100\/50{color:rgba(219,234,254,.5)}.text-blue-100\/55{color:rgba(219,234,254,.55)}.text-blue-100\/60{color:rgba(219,234,254,.6)}.text-blue-100\/65{color:rgba(219,234,254,.65)}.text-blue-100\/70{color:rgba(219,234,254,.7)}.text-blue-100\/75{color:rgba(219,234,254,.75)}.text-blue-100\/80{color:rgba(219,234,254,.8)}.text-blue-100\/85{color:rgba(219,234,254,.85)}.text-blue-100\/90{color:rgba(219,234,254,.9)}.text-blue-100\/95{color:rgba(219,234,254,.95)}.text-blue-200{--tw-text-opacity:1;color:rgb(191 219 254/var(--tw-text-opacity,1))}.text-blue-200\/0{color:rgba(191,219,254,0)}.text-blue-200\/10{color:rgba(191,219,254,.1)}.text-blue-200\/100{color:#bfdbfe}.text-blue-200\/15{color:rgba(191,219,254,.15)}.text-blue-200\/20{color:rgba(191,219,254,.2)}.text-blue-200\/25{color:rgba(191,219,254,.25)}.text-blue-200\/30{color:rgba(191,219,254,.3)}.text-blue-200\/35{color:rgba(191,219,254,.35)}.text-blue-200\/40{color:rgba(191,219,254,.4)}.text-blue-200\/45{color:rgba(191,219,254,.45)}.text-blue-200\/5{color:rgba(191,219,254,.05)}.text-blue-200\/50{color:rgba(191,219,254,.5)}.text-blue-200\/55{color:rgba(191,219,254,.55)}.text-blue-200\/60{color:rgba(191,219,254,.6)}.text-blue-200\/65{color:rgba(191,219,254,.65)}.text-blue-200\/70{color:rgba(191,219,254,.7)}.text-blue-200\/75{color:rgba(191,219,254,.75)}.text-blue-200\/80{color:rgba(191,219,254,.8)}.text-blue-200\/85{color:rgba(191,219,254,.85)}.text-blue-200\/90{color:rgba(191,219,254,.9)}.text-blue-200\/95{color:rgba(191,219,254,.95)}.text-blue-50{--tw-text-opacity:1;color:rgb(239 246 255/var(--tw-text-opacity,1))}.text-blue-50\/0{color:rgba(239,246,255,0)}.text-blue-50\/10{color:rgba(239,246,255,.1)}.text-blue-50\/100{color:#eff6ff}.text-blue-50\/15{color:rgba(239,246,255,.15)}.text-blue-50\/20{color:rgba(239,246,255,.2)}.text-blue-50\/25{color:rgba(239,246,255,.25)}.text-blue-50\/30{color:rgba(239,246,255,.3)}.text-blue-50\/35{color:rgba(239,246,255,.35)}.text-blue-50\/40{color:rgba(239,246,255,.4)}.text-blue-50\/45{color:rgba(239,246,255,.45)}.text-blue-50\/5{color:rgba(239,246,255,.05)}.text-blue-50\/50{color:rgba(239,246,255,.5)}.text-blue-50\/55{color:rgba(239,246,255,.55)}.text-blue-50\/60{color:rgba(239,246,255,.6)}.text-blue-50\/65{color:rgba(239,246,255,.65)}.text-blue-50\/70{color:rgba(239,246,255,.7)}.text-blue-50\/75{color:rgba(239,246,255,.75)}.text-blue-50\/80{color:rgba(239,246,255,.8)}.text-blue-50\/85{color:rgba(239,246,255,.85)}.text-blue-50\/90{color:rgba(239,246,255,.9)}.text-blue-50\/95{color:rgba(239,246,255,.95)}.text-blue-500{--tw-text-opacity:1;color:rgb(59 130 246/var(--tw-text-opacity,1))}.text-blue-500\/0{color:rgba(59,130,246,0)}.text-blue-500\/10{color:rgba(59,130,246,.1)}.text-blue-500\/100{color:#3b82f6}.text-blue-500\/15{color:rgba(59,130,246,.15)}.text-blue-500\/20{color:rgba(59,130,246,.2)}.text-blue-500\/25{color:rgba(59,130,246,.25)}.text-blue-500\/30{color:rgba(59,130,246,.3)}.text-blue-500\/35{color:rgba(59,130,246,.35)}.text-blue-500\/40{color:rgba(59,130,246,.4)}.text-blue-500\/45{color:rgba(59,130,246,.45)}.text-blue-500\/5{color:rgba(59,130,246,.05)}.text-blue-500\/50{color:rgba(59,130,246,.5)}.text-blue-500\/55{color:rgba(59,130,246,.55)}.text-blue-500\/60{color:rgba(59,130,246,.6)}.text-blue-500\/65{color:rgba(59,130,246,.65)}.text-blue-500\/70{color:rgba(59,130,246,.7)}.text-blue-500\/75{color:rgba(59,130,246,.75)}.text-blue-500\/80{color:rgba(59,130,246,.8)}.text-blue-500\/85{color:rgba(59,130,246,.85)}.text-blue-500\/90{color:rgba(59,130,246,.9)}.text-blue-500\/95{color:rgba(59,130,246,.95)}.text-blue-700{--tw-text-opacity:1;color:rgb(29 78 216/var(--tw-text-opacity,1))}.text-blue-950{--tw-text-opacity:1;color:rgb(23 37 84/var(--tw-text-opacity,1))}.text-blue-950\/0{color:rgba(23,37,84,0)}.text-blue-950\/10{color:rgba(23,37,84,.1)}.text-blue-950\/100{color:#172554}.text-blue-950\/15{color:rgba(23,37,84,.15)}.text-blue-950\/20{color:rgba(23,37,84,.2)}.text-blue-950\/25{color:rgba(23,37,84,.25)}.text-blue-950\/30{color:rgba(23,37,84,.3)}.text-blue-950\/35{color:rgba(23,37,84,.35)}.text-blue-950\/40{color:rgba(23,37,84,.4)}.text-blue-950\/45{color:rgba(23,37,84,.45)}.text-blue-950\/5{color:rgba(23,37,84,.05)}.text-blue-950\/50{color:rgba(23,37,84,.5)}.text-blue-950\/55{color:rgba(23,37,84,.55)}.text-blue-950\/60{color:rgba(23,37,84,.6)}.text-blue-950\/65{color:rgba(23,37,84,.65)}.text-blue-950\/70{color:rgba(23,37,84,.7)}.text-blue-950\/75{color:rgba(23,37,84,.75)}.text-blue-950\/80{color:rgba(23,37,84,.8)}.text-blue-950\/85{color:rgba(23,37,84,.85)}.text-blue-950\/90{color:rgba(23,37,84,.9)}.text-blue-950\/95{color:rgba(23,37,84,.95)}.text-card-foreground{color:hsl(var(--card-foreground))}.text-cyan-100{--tw-text-opacity:1;color:rgb(207 250 254/var(--tw-text-opacity,1))}.text-cyan-100\/0{color:rgba(207,250,254,0)}.text-cyan-100\/10{color:rgba(207,250,254,.1)}.text-cyan-100\/100{color:#cffafe}.text-cyan-100\/15{color:rgba(207,250,254,.15)}.text-cyan-100\/20{color:rgba(207,250,254,.2)}.text-cyan-100\/25{color:rgba(207,250,254,.25)}.text-cyan-100\/30{color:rgba(207,250,254,.3)}.text-cyan-100\/35{color:rgba(207,250,254,.35)}.text-cyan-100\/40{color:rgba(207,250,254,.4)}.text-cyan-100\/45{color:rgba(207,250,254,.45)}.text-cyan-100\/5{color:rgba(207,250,254,.05)}.text-cyan-100\/50{color:rgba(207,250,254,.5)}.text-cyan-100\/55{color:rgba(207,250,254,.55)}.text-cyan-100\/60{color:rgba(207,250,254,.6)}.text-cyan-100\/65{color:rgba(207,250,254,.65)}.text-cyan-100\/70{color:rgba(207,250,254,.7)}.text-cyan-100\/75{color:rgba(207,250,254,.75)}.text-cyan-100\/80{color:rgba(207,250,254,.8)}.text-cyan-100\/85{color:rgba(207,250,254,.85)}.text-cyan-100\/90{color:rgba(207,250,254,.9)}.text-cyan-100\/95{color:rgba(207,250,254,.95)}.text-cyan-200{--tw-text-opacity:1;color:rgb(165 243 252/var(--tw-text-opacity,1))}.text-cyan-200\/0{color:rgba(165,243,252,0)}.text-cyan-200\/10{color:rgba(165,243,252,.1)}.text-cyan-200\/100{color:#a5f3fc}.text-cyan-200\/15{color:rgba(165,243,252,.15)}.text-cyan-200\/20{color:rgba(165,243,252,.2)}.text-cyan-200\/25{color:rgba(165,243,252,.25)}.text-cyan-200\/30{color:rgba(165,243,252,.3)}.text-cyan-200\/35{color:rgba(165,243,252,.35)}.text-cyan-200\/40{color:rgba(165,243,252,.4)}.text-cyan-200\/45{color:rgba(165,243,252,.45)}.text-cyan-200\/5{color:rgba(165,243,252,.05)}.text-cyan-200\/50{color:rgba(165,243,252,.5)}.text-cyan-200\/55{color:rgba(165,243,252,.55)}.text-cyan-200\/60{color:rgba(165,243,252,.6)}.text-cyan-200\/65{color:rgba(165,243,252,.65)}.text-cyan-200\/70{color:rgba(165,243,252,.7)}.text-cyan-200\/75{color:rgba(165,243,252,.75)}.text-cyan-200\/80{color:rgba(165,243,252,.8)}.text-cyan-200\/85{color:rgba(165,243,252,.85)}.text-cyan-200\/90{color:rgba(165,243,252,.9)}.text-cyan-200\/95{color:rgba(165,243,252,.95)}.text-cyan-50{--tw-text-opacity:1;color:rgb(236 254 255/var(--tw-text-opacity,1))}.text-cyan-50\/0{color:rgba(236,254,255,0)}.text-cyan-50\/10{color:rgba(236,254,255,.1)}.text-cyan-50\/100{color:#ecfeff}.text-cyan-50\/15{color:rgba(236,254,255,.15)}.text-cyan-50\/20{color:rgba(236,254,255,.2)}.text-cyan-50\/25{color:rgba(236,254,255,.25)}.text-cyan-50\/30{color:rgba(236,254,255,.3)}.text-cyan-50\/35{color:rgba(236,254,255,.35)}.text-cyan-50\/40{color:rgba(236,254,255,.4)}.text-cyan-50\/45{color:rgba(236,254,255,.45)}.text-cyan-50\/5{color:rgba(236,254,255,.05)}.text-cyan-50\/50{color:rgba(236,254,255,.5)}.text-cyan-50\/55{color:rgba(236,254,255,.55)}.text-cyan-50\/60{color:rgba(236,254,255,.6)}.text-cyan-50\/65{color:rgba(236,254,255,.65)}.text-cyan-50\/70{color:rgba(236,254,255,.7)}.text-cyan-50\/75{color:rgba(236,254,255,.75)}.text-cyan-50\/80{color:rgba(236,254,255,.8)}.text-cyan-50\/85{color:rgba(236,254,255,.85)}.text-cyan-50\/90{color:rgba(236,254,255,.9)}.text-cyan-50\/95{color:rgba(236,254,255,.95)}.text-cyan-500{--tw-text-opacity:1;color:rgb(6 182 212/var(--tw-text-opacity,1))}.text-cyan-500\/0{color:rgba(6,182,212,0)}.text-cyan-500\/10{color:rgba(6,182,212,.1)}.text-cyan-500\/100{color:#06b6d4}.text-cyan-500\/15{color:rgba(6,182,212,.15)}.text-cyan-500\/20{color:rgba(6,182,212,.2)}.text-cyan-500\/25{color:rgba(6,182,212,.25)}.text-cyan-500\/30{color:rgba(6,182,212,.3)}.text-cyan-500\/35{color:rgba(6,182,212,.35)}.text-cyan-500\/40{color:rgba(6,182,212,.4)}.text-cyan-500\/45{color:rgba(6,182,212,.45)}.text-cyan-500\/5{color:rgba(6,182,212,.05)}.text-cyan-500\/50{color:rgba(6,182,212,.5)}.text-cyan-500\/55{color:rgba(6,182,212,.55)}.text-cyan-500\/60{color:rgba(6,182,212,.6)}.text-cyan-500\/65{color:rgba(6,182,212,.65)}.text-cyan-500\/70{color:rgba(6,182,212,.7)}.text-cyan-500\/75{color:rgba(6,182,212,.75)}.text-cyan-500\/80{color:rgba(6,182,212,.8)}.text-cyan-500\/85{color:rgba(6,182,212,.85)}.text-cyan-500\/90{color:rgba(6,182,212,.9)}.text-cyan-500\/95{color:rgba(6,182,212,.95)}.text-cyan-950{--tw-text-opacity:1;color:rgb(8 51 68/var(--tw-text-opacity,1))}.text-cyan-950\/0{color:rgba(8,51,68,0)}.text-cyan-950\/10{color:rgba(8,51,68,.1)}.text-cyan-950\/100{color:#083344}.text-cyan-950\/15{color:rgba(8,51,68,.15)}.text-cyan-950\/20{color:rgba(8,51,68,.2)}.text-cyan-950\/25{color:rgba(8,51,68,.25)}.text-cyan-950\/30{color:rgba(8,51,68,.3)}.text-cyan-950\/35{color:rgba(8,51,68,.35)}.text-cyan-950\/40{color:rgba(8,51,68,.4)}.text-cyan-950\/45{color:rgba(8,51,68,.45)}.text-cyan-950\/5{color:rgba(8,51,68,.05)}.text-cyan-950\/50{color:rgba(8,51,68,.5)}.text-cyan-950\/55{color:rgba(8,51,68,.55)}.text-cyan-950\/60{color:rgba(8,51,68,.6)}.text-cyan-950\/65{color:rgba(8,51,68,.65)}.text-cyan-950\/70{color:rgba(8,51,68,.7)}.text-cyan-950\/75{color:rgba(8,51,68,.75)}.text-cyan-950\/80{color:rgba(8,51,68,.8)}.text-cyan-950\/85{color:rgba(8,51,68,.85)}.text-cyan-950\/90{color:rgba(8,51,68,.9)}.text-cyan-950\/95{color:rgba(8,51,68,.95)}.text-destructive{color:hsl(var(--destructive))}.text-destructive-foreground{color:hsl(var(--destructive-foreground))}.text-emerald-100{--tw-text-opacity:1;color:rgb(209 250 229/var(--tw-text-opacity,1))}.text-emerald-100\/0{color:rgba(209,250,229,0)}.text-emerald-100\/10{color:rgba(209,250,229,.1)}.text-emerald-100\/100{color:#d1fae5}.text-emerald-100\/15{color:rgba(209,250,229,.15)}.text-emerald-100\/20{color:rgba(209,250,229,.2)}.text-emerald-100\/25{color:rgba(209,250,229,.25)}.text-emerald-100\/30{color:rgba(209,250,229,.3)}.text-emerald-100\/35{color:rgba(209,250,229,.35)}.text-emerald-100\/40{color:rgba(209,250,229,.4)}.text-emerald-100\/45{color:rgba(209,250,229,.45)}.text-emerald-100\/5{color:rgba(209,250,229,.05)}.text-emerald-100\/50{color:rgba(209,250,229,.5)}.text-emerald-100\/55{color:rgba(209,250,229,.55)}.text-emerald-100\/60{color:rgba(209,250,229,.6)}.text-emerald-100\/65{color:rgba(209,250,229,.65)}.text-emerald-100\/70{color:rgba(209,250,229,.7)}.text-emerald-100\/75{color:rgba(209,250,229,.75)}.text-emerald-100\/80{color:rgba(209,250,229,.8)}.text-emerald-100\/85{color:rgba(209,250,229,.85)}.text-emerald-100\/90{color:rgba(209,250,229,.9)}.text-emerald-100\/95{color:rgba(209,250,229,.95)}.text-emerald-200{--tw-text-opacity:1;color:rgb(167 243 208/var(--tw-text-opacity,1))}.text-emerald-200\/0{color:rgba(167,243,208,0)}.text-emerald-200\/10{color:rgba(167,243,208,.1)}.text-emerald-200\/100{color:#a7f3d0}.text-emerald-200\/15{color:rgba(167,243,208,.15)}.text-emerald-200\/20{color:rgba(167,243,208,.2)}.text-emerald-200\/25{color:rgba(167,243,208,.25)}.text-emerald-200\/30{color:rgba(167,243,208,.3)}.text-emerald-200\/35{color:rgba(167,243,208,.35)}.text-emerald-200\/40{color:rgba(167,243,208,.4)}.text-emerald-200\/45{color:rgba(167,243,208,.45)}.text-emerald-200\/5{color:rgba(167,243,208,.05)}.text-emerald-200\/50{color:rgba(167,243,208,.5)}.text-emerald-200\/55{color:rgba(167,243,208,.55)}.text-emerald-200\/60{color:rgba(167,243,208,.6)}.text-emerald-200\/65{color:rgba(167,243,208,.65)}.text-emerald-200\/70{color:rgba(167,243,208,.7)}.text-emerald-200\/75{color:rgba(167,243,208,.75)}.text-emerald-200\/80{color:rgba(167,243,208,.8)}.text-emerald-200\/85{color:rgba(167,243,208,.85)}.text-emerald-200\/90{color:rgba(167,243,208,.9)}.text-emerald-200\/95{color:rgba(167,243,208,.95)}.text-emerald-50{--tw-text-opacity:1;color:rgb(236 253 245/var(--tw-text-opacity,1))}.text-emerald-50\/0{color:rgba(236,253,245,0)}.text-emerald-50\/10{color:rgba(236,253,245,.1)}.text-emerald-50\/100{color:#ecfdf5}.text-emerald-50\/15{color:rgba(236,253,245,.15)}.text-emerald-50\/20{color:rgba(236,253,245,.2)}.text-emerald-50\/25{color:rgba(236,253,245,.25)}.text-emerald-50\/30{color:rgba(236,253,245,.3)}.text-emerald-50\/35{color:rgba(236,253,245,.35)}.text-emerald-50\/40{color:rgba(236,253,245,.4)}.text-emerald-50\/45{color:rgba(236,253,245,.45)}.text-emerald-50\/5{color:rgba(236,253,245,.05)}.text-emerald-50\/50{color:rgba(236,253,245,.5)}.text-emerald-50\/55{color:rgba(236,253,245,.55)}.text-emerald-50\/60{color:rgba(236,253,245,.6)}.text-emerald-50\/65{color:rgba(236,253,245,.65)}.text-emerald-50\/70{color:rgba(236,253,245,.7)}.text-emerald-50\/75{color:rgba(236,253,245,.75)}.text-emerald-50\/80{color:rgba(236,253,245,.8)}.text-emerald-50\/85{color:rgba(236,253,245,.85)}.text-emerald-50\/90{color:rgba(236,253,245,.9)}.text-emerald-50\/95{color:rgba(236,253,245,.95)}.text-emerald-500{--tw-text-opacity:1;color:rgb(16 185 129/var(--tw-text-opacity,1))}.text-emerald-500\/0{color:rgba(16,185,129,0)}.text-emerald-500\/10{color:rgba(16,185,129,.1)}.text-emerald-500\/100{color:#10b981}.text-emerald-500\/15{color:rgba(16,185,129,.15)}.text-emerald-500\/20{color:rgba(16,185,129,.2)}.text-emerald-500\/25{color:rgba(16,185,129,.25)}.text-emerald-500\/30{color:rgba(16,185,129,.3)}.text-emerald-500\/35{color:rgba(16,185,129,.35)}.text-emerald-500\/40{color:rgba(16,185,129,.4)}.text-emerald-500\/45{color:rgba(16,185,129,.45)}.text-emerald-500\/5{color:rgba(16,185,129,.05)}.text-emerald-500\/50{color:rgba(16,185,129,.5)}.text-emerald-500\/55{color:rgba(16,185,129,.55)}.text-emerald-500\/60{color:rgba(16,185,129,.6)}.text-emerald-500\/65{color:rgba(16,185,129,.65)}.text-emerald-500\/70{color:rgba(16,185,129,.7)}.text-emerald-500\/75{color:rgba(16,185,129,.75)}.text-emerald-500\/80{color:rgba(16,185,129,.8)}.text-emerald-500\/85{color:rgba(16,185,129,.85)}.text-emerald-500\/90{color:rgba(16,185,129,.9)}.text-emerald-500\/95{color:rgba(16,185,129,.95)}.text-emerald-950{--tw-text-opacity:1;color:rgb(2 44 34/var(--tw-text-opacity,1))}.text-emerald-950\/0{color:rgba(2,44,34,0)}.text-emerald-950\/10{color:rgba(2,44,34,.1)}.text-emerald-950\/100{color:#022c22}.text-emerald-950\/15{color:rgba(2,44,34,.15)}.text-emerald-950\/20{color:rgba(2,44,34,.2)}.text-emerald-950\/25{color:rgba(2,44,34,.25)}.text-emerald-950\/30{color:rgba(2,44,34,.3)}.text-emerald-950\/35{color:rgba(2,44,34,.35)}.text-emerald-950\/40{color:rgba(2,44,34,.4)}.text-emerald-950\/45{color:rgba(2,44,34,.45)}.text-emerald-950\/5{color:rgba(2,44,34,.05)}.text-emerald-950\/50{color:rgba(2,44,34,.5)}.text-emerald-950\/55{color:rgba(2,44,34,.55)}.text-emerald-950\/60{color:rgba(2,44,34,.6)}.text-emerald-950\/65{color:rgba(2,44,34,.65)}.text-emerald-950\/70{color:rgba(2,44,34,.7)}.text-emerald-950\/75{color:rgba(2,44,34,.75)}.text-emerald-950\/80{color:rgba(2,44,34,.8)}.text-emerald-950\/85{color:rgba(2,44,34,.85)}.text-emerald-950\/90{color:rgba(2,44,34,.9)}.text-emerald-950\/95{color:rgba(2,44,34,.95)}.text-foreground{color:hsl(var(--foreground))}.text-foreground\/50{color:hsl(var(--foreground)/.5)}.text-fuchsia-100{--tw-text-opacity:1;color:rgb(250 232 255/var(--tw-text-opacity,1))}.text-fuchsia-100\/0{color:rgba(250,232,255,0)}.text-fuchsia-100\/10{color:rgba(250,232,255,.1)}.text-fuchsia-100\/100{color:#fae8ff}.text-fuchsia-100\/15{color:rgba(250,232,255,.15)}.text-fuchsia-100\/20{color:rgba(250,232,255,.2)}.text-fuchsia-100\/25{color:rgba(250,232,255,.25)}.text-fuchsia-100\/30{color:rgba(250,232,255,.3)}.text-fuchsia-100\/35{color:rgba(250,232,255,.35)}.text-fuchsia-100\/40{color:rgba(250,232,255,.4)}.text-fuchsia-100\/45{color:rgba(250,232,255,.45)}.text-fuchsia-100\/5{color:rgba(250,232,255,.05)}.text-fuchsia-100\/50{color:rgba(250,232,255,.5)}.text-fuchsia-100\/55{color:rgba(250,232,255,.55)}.text-fuchsia-100\/60{color:rgba(250,232,255,.6)}.text-fuchsia-100\/65{color:rgba(250,232,255,.65)}.text-fuchsia-100\/70{color:rgba(250,232,255,.7)}.text-fuchsia-100\/75{color:rgba(250,232,255,.75)}.text-fuchsia-100\/80{color:rgba(250,232,255,.8)}.text-fuchsia-100\/85{color:rgba(250,232,255,.85)}.text-fuchsia-100\/90{color:rgba(250,232,255,.9)}.text-fuchsia-100\/95{color:rgba(250,232,255,.95)}.text-fuchsia-200{--tw-text-opacity:1;color:rgb(245 208 254/var(--tw-text-opacity,1))}.text-fuchsia-200\/0{color:rgba(245,208,254,0)}.text-fuchsia-200\/10{color:rgba(245,208,254,.1)}.text-fuchsia-200\/100{color:#f5d0fe}.text-fuchsia-200\/15{color:rgba(245,208,254,.15)}.text-fuchsia-200\/20{color:rgba(245,208,254,.2)}.text-fuchsia-200\/25{color:rgba(245,208,254,.25)}.text-fuchsia-200\/30{color:rgba(245,208,254,.3)}.text-fuchsia-200\/35{color:rgba(245,208,254,.35)}.text-fuchsia-200\/40{color:rgba(245,208,254,.4)}.text-fuchsia-200\/45{color:rgba(245,208,254,.45)}.text-fuchsia-200\/5{color:rgba(245,208,254,.05)}.text-fuchsia-200\/50{color:rgba(245,208,254,.5)}.text-fuchsia-200\/55{color:rgba(245,208,254,.55)}.text-fuchsia-200\/60{color:rgba(245,208,254,.6)}.text-fuchsia-200\/65{color:rgba(245,208,254,.65)}.text-fuchsia-200\/70{color:rgba(245,208,254,.7)}.text-fuchsia-200\/75{color:rgba(245,208,254,.75)}.text-fuchsia-200\/80{color:rgba(245,208,254,.8)}.text-fuchsia-200\/85{color:rgba(245,208,254,.85)}.text-fuchsia-200\/90{color:rgba(245,208,254,.9)}.text-fuchsia-200\/95{color:rgba(245,208,254,.95)}.text-fuchsia-50{--tw-text-opacity:1;color:rgb(253 244 255/var(--tw-text-opacity,1))}.text-fuchsia-50\/0{color:rgba(253,244,255,0)}.text-fuchsia-50\/10{color:rgba(253,244,255,.1)}.text-fuchsia-50\/100{color:#fdf4ff}.text-fuchsia-50\/15{color:rgba(253,244,255,.15)}.text-fuchsia-50\/20{color:rgba(253,244,255,.2)}.text-fuchsia-50\/25{color:rgba(253,244,255,.25)}.text-fuchsia-50\/30{color:rgba(253,244,255,.3)}.text-fuchsia-50\/35{color:rgba(253,244,255,.35)}.text-fuchsia-50\/40{color:rgba(253,244,255,.4)}.text-fuchsia-50\/45{color:rgba(253,244,255,.45)}.text-fuchsia-50\/5{color:rgba(253,244,255,.05)}.text-fuchsia-50\/50{color:rgba(253,244,255,.5)}.text-fuchsia-50\/55{color:rgba(253,244,255,.55)}.text-fuchsia-50\/60{color:rgba(253,244,255,.6)}.text-fuchsia-50\/65{color:rgba(253,244,255,.65)}.text-fuchsia-50\/70{color:rgba(253,244,255,.7)}.text-fuchsia-50\/75{color:rgba(253,244,255,.75)}.text-fuchsia-50\/80{color:rgba(253,244,255,.8)}.text-fuchsia-50\/85{color:rgba(253,244,255,.85)}.text-fuchsia-50\/90{color:rgba(253,244,255,.9)}.text-fuchsia-50\/95{color:rgba(253,244,255,.95)}.text-fuchsia-500{--tw-text-opacity:1;color:rgb(217 70 239/var(--tw-text-opacity,1))}.text-fuchsia-500\/0{color:rgba(217,70,239,0)}.text-fuchsia-500\/10{color:rgba(217,70,239,.1)}.text-fuchsia-500\/100{color:#d946ef}.text-fuchsia-500\/15{color:rgba(217,70,239,.15)}.text-fuchsia-500\/20{color:rgba(217,70,239,.2)}.text-fuchsia-500\/25{color:rgba(217,70,239,.25)}.text-fuchsia-500\/30{color:rgba(217,70,239,.3)}.text-fuchsia-500\/35{color:rgba(217,70,239,.35)}.text-fuchsia-500\/40{color:rgba(217,70,239,.4)}.text-fuchsia-500\/45{color:rgba(217,70,239,.45)}.text-fuchsia-500\/5{color:rgba(217,70,239,.05)}.text-fuchsia-500\/50{color:rgba(217,70,239,.5)}.text-fuchsia-500\/55{color:rgba(217,70,239,.55)}.text-fuchsia-500\/60{color:rgba(217,70,239,.6)}.text-fuchsia-500\/65{color:rgba(217,70,239,.65)}.text-fuchsia-500\/70{color:rgba(217,70,239,.7)}.text-fuchsia-500\/75{color:rgba(217,70,239,.75)}.text-fuchsia-500\/80{color:rgba(217,70,239,.8)}.text-fuchsia-500\/85{color:rgba(217,70,239,.85)}.text-fuchsia-500\/90{color:rgba(217,70,239,.9)}.text-fuchsia-500\/95{color:rgba(217,70,239,.95)}.text-fuchsia-950{--tw-text-opacity:1;color:rgb(74 4 78/var(--tw-text-opacity,1))}.text-fuchsia-950\/0{color:rgba(74,4,78,0)}.text-fuchsia-950\/10{color:rgba(74,4,78,.1)}.text-fuchsia-950\/100{color:#4a044e}.text-fuchsia-950\/15{color:rgba(74,4,78,.15)}.text-fuchsia-950\/20{color:rgba(74,4,78,.2)}.text-fuchsia-950\/25{color:rgba(74,4,78,.25)}.text-fuchsia-950\/30{color:rgba(74,4,78,.3)}.text-fuchsia-950\/35{color:rgba(74,4,78,.35)}.text-fuchsia-950\/40{color:rgba(74,4,78,.4)}.text-fuchsia-950\/45{color:rgba(74,4,78,.45)}.text-fuchsia-950\/5{color:rgba(74,4,78,.05)}.text-fuchsia-950\/50{color:rgba(74,4,78,.5)}.text-fuchsia-950\/55{color:rgba(74,4,78,.55)}.text-fuchsia-950\/60{color:rgba(74,4,78,.6)}.text-fuchsia-950\/65{color:rgba(74,4,78,.65)}.text-fuchsia-950\/70{color:rgba(74,4,78,.7)}.text-fuchsia-950\/75{color:rgba(74,4,78,.75)}.text-fuchsia-950\/80{color:rgba(74,4,78,.8)}.text-fuchsia-950\/85{color:rgba(74,4,78,.85)}.text-fuchsia-950\/90{color:rgba(74,4,78,.9)}.text-fuchsia-950\/95{color:rgba(74,4,78,.95)}.text-gray-100{--tw-text-opacity:1;color:rgb(243 244 246/var(--tw-text-opacity,1))}.text-gray-100\/0{color:rgba(243,244,246,0)}.text-gray-100\/10{color:rgba(243,244,246,.1)}.text-gray-100\/100{color:#f3f4f6}.text-gray-100\/15{color:rgba(243,244,246,.15)}.text-gray-100\/20{color:rgba(243,244,246,.2)}.text-gray-100\/25{color:rgba(243,244,246,.25)}.text-gray-100\/30{color:rgba(243,244,246,.3)}.text-gray-100\/35{color:rgba(243,244,246,.35)}.text-gray-100\/40{color:rgba(243,244,246,.4)}.text-gray-100\/45{color:rgba(243,244,246,.45)}.text-gray-100\/5{color:rgba(243,244,246,.05)}.text-gray-100\/50{color:rgba(243,244,246,.5)}.text-gray-100\/55{color:rgba(243,244,246,.55)}.text-gray-100\/60{color:rgba(243,244,246,.6)}.text-gray-100\/65{color:rgba(243,244,246,.65)}.text-gray-100\/70{color:rgba(243,244,246,.7)}.text-gray-100\/75{color:rgba(243,244,246,.75)}.text-gray-100\/80{color:rgba(243,244,246,.8)}.text-gray-100\/85{color:rgba(243,244,246,.85)}.text-gray-100\/90{color:rgba(243,244,246,.9)}.text-gray-100\/95{color:rgba(243,244,246,.95)}.text-gray-200{--tw-text-opacity:1;color:rgb(229 231 235/var(--tw-text-opacity,1))}.text-gray-200\/0{color:rgba(229,231,235,0)}.text-gray-200\/10{color:rgba(229,231,235,.1)}.text-gray-200\/100{color:#e5e7eb}.text-gray-200\/15{color:rgba(229,231,235,.15)}.text-gray-200\/20{color:rgba(229,231,235,.2)}.text-gray-200\/25{color:rgba(229,231,235,.25)}.text-gray-200\/30{color:rgba(229,231,235,.3)}.text-gray-200\/35{color:rgba(229,231,235,.35)}.text-gray-200\/40{color:rgba(229,231,235,.4)}.text-gray-200\/45{color:rgba(229,231,235,.45)}.text-gray-200\/5{color:rgba(229,231,235,.05)}.text-gray-200\/50{color:rgba(229,231,235,.5)}.text-gray-200\/55{color:rgba(229,231,235,.55)}.text-gray-200\/60{color:rgba(229,231,235,.6)}.text-gray-200\/65{color:rgba(229,231,235,.65)}.text-gray-200\/70{color:rgba(229,231,235,.7)}.text-gray-200\/75{color:rgba(229,231,235,.75)}.text-gray-200\/80{color:rgba(229,231,235,.8)}.text-gray-200\/85{color:rgba(229,231,235,.85)}.text-gray-200\/90{color:rgba(229,231,235,.9)}.text-gray-200\/95{color:rgba(229,231,235,.95)}.text-gray-300{--tw-text-opacity:1;color:rgb(209 213 219/var(--tw-text-opacity,1))}.text-gray-400{--tw-text-opacity:1;color:rgb(156 163 175/var(--tw-text-opacity,1))}.text-gray-50{--tw-text-opacity:1;color:rgb(249 250 251/var(--tw-text-opacity,1))}.text-gray-50\/0{color:rgba(249,250,251,0)}.text-gray-50\/10{color:rgba(249,250,251,.1)}.text-gray-50\/100{color:#f9fafb}.text-gray-50\/15{color:rgba(249,250,251,.15)}.text-gray-50\/20{color:rgba(249,250,251,.2)}.text-gray-50\/25{color:rgba(249,250,251,.25)}.text-gray-50\/30{color:rgba(249,250,251,.3)}.text-gray-50\/35{color:rgba(249,250,251,.35)}.text-gray-50\/40{color:rgba(249,250,251,.4)}.text-gray-50\/45{color:rgba(249,250,251,.45)}.text-gray-50\/5{color:rgba(249,250,251,.05)}.text-gray-50\/50{color:rgba(249,250,251,.5)}.text-gray-50\/55{color:rgba(249,250,251,.55)}.text-gray-50\/60{color:rgba(249,250,251,.6)}.text-gray-50\/65{color:rgba(249,250,251,.65)}.text-gray-50\/70{color:rgba(249,250,251,.7)}.text-gray-50\/75{color:rgba(249,250,251,.75)}.text-gray-50\/80{color:rgba(249,250,251,.8)}.text-gray-50\/85{color:rgba(249,250,251,.85)}.text-gray-50\/90{color:rgba(249,250,251,.9)}.text-gray-50\/95{color:rgba(249,250,251,.95)}.text-gray-500{--tw-text-opacity:1;color:rgb(107 114 128/var(--tw-text-opacity,1))}.text-gray-500\/0{color:hsla(220,9%,46%,0)}.text-gray-500\/10{color:hsla(220,9%,46%,.1)}.text-gray-500\/100{color:#6b7280}.text-gray-500\/15{color:hsla(220,9%,46%,.15)}.text-gray-500\/20{color:hsla(220,9%,46%,.2)}.text-gray-500\/25{color:hsla(220,9%,46%,.25)}.text-gray-500\/30{color:hsla(220,9%,46%,.3)}.text-gray-500\/35{color:hsla(220,9%,46%,.35)}.text-gray-500\/40{color:hsla(220,9%,46%,.4)}.text-gray-500\/45{color:hsla(220,9%,46%,.45)}.text-gray-500\/5{color:hsla(220,9%,46%,.05)}.text-gray-500\/50{color:hsla(220,9%,46%,.5)}.text-gray-500\/55{color:hsla(220,9%,46%,.55)}.text-gray-500\/60{color:hsla(220,9%,46%,.6)}.text-gray-500\/65{color:hsla(220,9%,46%,.65)}.text-gray-500\/70{color:hsla(220,9%,46%,.7)}.text-gray-500\/75{color:hsla(220,9%,46%,.75)}.text-gray-500\/80{color:hsla(220,9%,46%,.8)}.text-gray-500\/85{color:hsla(220,9%,46%,.85)}.text-gray-500\/90{color:hsla(220,9%,46%,.9)}.text-gray-500\/95{color:hsla(220,9%,46%,.95)}.text-gray-600{--tw-text-opacity:1;color:rgb(75 85 99/var(--tw-text-opacity,1))}.text-gray-800{--tw-text-opacity:1;color:rgb(31 41 55/var(--tw-text-opacity,1))}.text-gray-950{--tw-text-opacity:1;color:rgb(3 7 18/var(--tw-text-opacity,1))}.text-gray-950\/0{color:rgba(3,7,18,0)}.text-gray-950\/10{color:rgba(3,7,18,.1)}.text-gray-950\/100{color:#030712}.text-gray-950\/15{color:rgba(3,7,18,.15)}.text-gray-950\/20{color:rgba(3,7,18,.2)}.text-gray-950\/25{color:rgba(3,7,18,.25)}.text-gray-950\/30{color:rgba(3,7,18,.3)}.text-gray-950\/35{color:rgba(3,7,18,.35)}.text-gray-950\/40{color:rgba(3,7,18,.4)}.text-gray-950\/45{color:rgba(3,7,18,.45)}.text-gray-950\/5{color:rgba(3,7,18,.05)}.text-gray-950\/50{color:rgba(3,7,18,.5)}.text-gray-950\/55{color:rgba(3,7,18,.55)}.text-gray-950\/60{color:rgba(3,7,18,.6)}.text-gray-950\/65{color:rgba(3,7,18,.65)}.text-gray-950\/70{color:rgba(3,7,18,.7)}.text-gray-950\/75{color:rgba(3,7,18,.75)}.text-gray-950\/80{color:rgba(3,7,18,.8)}.text-gray-950\/85{color:rgba(3,7,18,.85)}.text-gray-950\/90{color:rgba(3,7,18,.9)}.text-gray-950\/95{color:rgba(3,7,18,.95)}.text-green-100{--tw-text-opacity:1;color:rgb(220 252 231/var(--tw-text-opacity,1))}.text-green-100\/0{color:rgba(220,252,231,0)}.text-green-100\/10{color:rgba(220,252,231,.1)}.text-green-100\/100{color:#dcfce7}.text-green-100\/15{color:rgba(220,252,231,.15)}.text-green-100\/20{color:rgba(220,252,231,.2)}.text-green-100\/25{color:rgba(220,252,231,.25)}.text-green-100\/30{color:rgba(220,252,231,.3)}.text-green-100\/35{color:rgba(220,252,231,.35)}.text-green-100\/40{color:rgba(220,252,231,.4)}.text-green-100\/45{color:rgba(220,252,231,.45)}.text-green-100\/5{color:rgba(220,252,231,.05)}.text-green-100\/50{color:rgba(220,252,231,.5)}.text-green-100\/55{color:rgba(220,252,231,.55)}.text-green-100\/60{color:rgba(220,252,231,.6)}.text-green-100\/65{color:rgba(220,252,231,.65)}.text-green-100\/70{color:rgba(220,252,231,.7)}.text-green-100\/75{color:rgba(220,252,231,.75)}.text-green-100\/80{color:rgba(220,252,231,.8)}.text-green-100\/85{color:rgba(220,252,231,.85)}.text-green-100\/90{color:rgba(220,252,231,.9)}.text-green-100\/95{color:rgba(220,252,231,.95)}.text-green-200{--tw-text-opacity:1;color:rgb(187 247 208/var(--tw-text-opacity,1))}.text-green-200\/0{color:rgba(187,247,208,0)}.text-green-200\/10{color:rgba(187,247,208,.1)}.text-green-200\/100{color:#bbf7d0}.text-green-200\/15{color:rgba(187,247,208,.15)}.text-green-200\/20{color:rgba(187,247,208,.2)}.text-green-200\/25{color:rgba(187,247,208,.25)}.text-green-200\/30{color:rgba(187,247,208,.3)}.text-green-200\/35{color:rgba(187,247,208,.35)}.text-green-200\/40{color:rgba(187,247,208,.4)}.text-green-200\/45{color:rgba(187,247,208,.45)}.text-green-200\/5{color:rgba(187,247,208,.05)}.text-green-200\/50{color:rgba(187,247,208,.5)}.text-green-200\/55{color:rgba(187,247,208,.55)}.text-green-200\/60{color:rgba(187,247,208,.6)}.text-green-200\/65{color:rgba(187,247,208,.65)}.text-green-200\/70{color:rgba(187,247,208,.7)}.text-green-200\/75{color:rgba(187,247,208,.75)}.text-green-200\/80{color:rgba(187,247,208,.8)}.text-green-200\/85{color:rgba(187,247,208,.85)}.text-green-200\/90{color:rgba(187,247,208,.9)}.text-green-200\/95{color:rgba(187,247,208,.95)}.text-green-400{--tw-text-opacity:1;color:rgb(74 222 128/var(--tw-text-opacity,1))}.text-green-50{--tw-text-opacity:1;color:rgb(240 253 244/var(--tw-text-opacity,1))}.text-green-50\/0{color:rgba(240,253,244,0)}.text-green-50\/10{color:rgba(240,253,244,.1)}.text-green-50\/100{color:#f0fdf4}.text-green-50\/15{color:rgba(240,253,244,.15)}.text-green-50\/20{color:rgba(240,253,244,.2)}.text-green-50\/25{color:rgba(240,253,244,.25)}.text-green-50\/30{color:rgba(240,253,244,.3)}.text-green-50\/35{color:rgba(240,253,244,.35)}.text-green-50\/40{color:rgba(240,253,244,.4)}.text-green-50\/45{color:rgba(240,253,244,.45)}.text-green-50\/5{color:rgba(240,253,244,.05)}.text-green-50\/50{color:rgba(240,253,244,.5)}.text-green-50\/55{color:rgba(240,253,244,.55)}.text-green-50\/60{color:rgba(240,253,244,.6)}.text-green-50\/65{color:rgba(240,253,244,.65)}.text-green-50\/70{color:rgba(240,253,244,.7)}.text-green-50\/75{color:rgba(240,253,244,.75)}.text-green-50\/80{color:rgba(240,253,244,.8)}.text-green-50\/85{color:rgba(240,253,244,.85)}.text-green-50\/90{color:rgba(240,253,244,.9)}.text-green-50\/95{color:rgba(240,253,244,.95)}.text-green-500{--tw-text-opacity:1;color:rgb(34 197 94/var(--tw-text-opacity,1))}.text-green-500\/0{color:rgba(34,197,94,0)}.text-green-500\/10{color:rgba(34,197,94,.1)}.text-green-500\/100{color:#22c55e}.text-green-500\/15{color:rgba(34,197,94,.15)}.text-green-500\/20{color:rgba(34,197,94,.2)}.text-green-500\/25{color:rgba(34,197,94,.25)}.text-green-500\/30{color:rgba(34,197,94,.3)}.text-green-500\/35{color:rgba(34,197,94,.35)}.text-green-500\/40{color:rgba(34,197,94,.4)}.text-green-500\/45{color:rgba(34,197,94,.45)}.text-green-500\/5{color:rgba(34,197,94,.05)}.text-green-500\/50{color:rgba(34,197,94,.5)}.text-green-500\/55{color:rgba(34,197,94,.55)}.text-green-500\/60{color:rgba(34,197,94,.6)}.text-green-500\/65{color:rgba(34,197,94,.65)}.text-green-500\/70{color:rgba(34,197,94,.7)}.text-green-500\/75{color:rgba(34,197,94,.75)}.text-green-500\/80{color:rgba(34,197,94,.8)}.text-green-500\/85{color:rgba(34,197,94,.85)}.text-green-500\/90{color:rgba(34,197,94,.9)}.text-green-500\/95{color:rgba(34,197,94,.95)}.text-green-950{--tw-text-opacity:1;color:rgb(5 46 22/var(--tw-text-opacity,1))}.text-green-950\/0{color:rgba(5,46,22,0)}.text-green-950\/10{color:rgba(5,46,22,.1)}.text-green-950\/100{color:#052e16}.text-green-950\/15{color:rgba(5,46,22,.15)}.text-green-950\/20{color:rgba(5,46,22,.2)}.text-green-950\/25{color:rgba(5,46,22,.25)}.text-green-950\/30{color:rgba(5,46,22,.3)}.text-green-950\/35{color:rgba(5,46,22,.35)}.text-green-950\/40{color:rgba(5,46,22,.4)}.text-green-950\/45{color:rgba(5,46,22,.45)}.text-green-950\/5{color:rgba(5,46,22,.05)}.text-green-950\/50{color:rgba(5,46,22,.5)}.text-green-950\/55{color:rgba(5,46,22,.55)}.text-green-950\/60{color:rgba(5,46,22,.6)}.text-green-950\/65{color:rgba(5,46,22,.65)}.text-green-950\/70{color:rgba(5,46,22,.7)}.text-green-950\/75{color:rgba(5,46,22,.75)}.text-green-950\/80{color:rgba(5,46,22,.8)}.text-green-950\/85{color:rgba(5,46,22,.85)}.text-green-950\/90{color:rgba(5,46,22,.9)}.text-green-950\/95{color:rgba(5,46,22,.95)}.text-indigo-100{--tw-text-opacity:1;color:rgb(224 231 255/var(--tw-text-opacity,1))}.text-indigo-100\/0{color:rgba(224,231,255,0)}.text-indigo-100\/10{color:rgba(224,231,255,.1)}.text-indigo-100\/100{color:#e0e7ff}.text-indigo-100\/15{color:rgba(224,231,255,.15)}.text-indigo-100\/20{color:rgba(224,231,255,.2)}.text-indigo-100\/25{color:rgba(224,231,255,.25)}.text-indigo-100\/30{color:rgba(224,231,255,.3)}.text-indigo-100\/35{color:rgba(224,231,255,.35)}.text-indigo-100\/40{color:rgba(224,231,255,.4)}.text-indigo-100\/45{color:rgba(224,231,255,.45)}.text-indigo-100\/5{color:rgba(224,231,255,.05)}.text-indigo-100\/50{color:rgba(224,231,255,.5)}.text-indigo-100\/55{color:rgba(224,231,255,.55)}.text-indigo-100\/60{color:rgba(224,231,255,.6)}.text-indigo-100\/65{color:rgba(224,231,255,.65)}.text-indigo-100\/70{color:rgba(224,231,255,.7)}.text-indigo-100\/75{color:rgba(224,231,255,.75)}.text-indigo-100\/80{color:rgba(224,231,255,.8)}.text-indigo-100\/85{color:rgba(224,231,255,.85)}.text-indigo-100\/90{color:rgba(224,231,255,.9)}.text-indigo-100\/95{color:rgba(224,231,255,.95)}.text-indigo-200{--tw-text-opacity:1;color:rgb(199 210 254/var(--tw-text-opacity,1))}.text-indigo-200\/0{color:rgba(199,210,254,0)}.text-indigo-200\/10{color:rgba(199,210,254,.1)}.text-indigo-200\/100{color:#c7d2fe}.text-indigo-200\/15{color:rgba(199,210,254,.15)}.text-indigo-200\/20{color:rgba(199,210,254,.2)}.text-indigo-200\/25{color:rgba(199,210,254,.25)}.text-indigo-200\/30{color:rgba(199,210,254,.3)}.text-indigo-200\/35{color:rgba(199,210,254,.35)}.text-indigo-200\/40{color:rgba(199,210,254,.4)}.text-indigo-200\/45{color:rgba(199,210,254,.45)}.text-indigo-200\/5{color:rgba(199,210,254,.05)}.text-indigo-200\/50{color:rgba(199,210,254,.5)}.text-indigo-200\/55{color:rgba(199,210,254,.55)}.text-indigo-200\/60{color:rgba(199,210,254,.6)}.text-indigo-200\/65{color:rgba(199,210,254,.65)}.text-indigo-200\/70{color:rgba(199,210,254,.7)}.text-indigo-200\/75{color:rgba(199,210,254,.75)}.text-indigo-200\/80{color:rgba(199,210,254,.8)}.text-indigo-200\/85{color:rgba(199,210,254,.85)}.text-indigo-200\/90{color:rgba(199,210,254,.9)}.text-indigo-200\/95{color:rgba(199,210,254,.95)}.text-indigo-50{--tw-text-opacity:1;color:rgb(238 242 255/var(--tw-text-opacity,1))}.text-indigo-50\/0{color:rgba(238,242,255,0)}.text-indigo-50\/10{color:rgba(238,242,255,.1)}.text-indigo-50\/100{color:#eef2ff}.text-indigo-50\/15{color:rgba(238,242,255,.15)}.text-indigo-50\/20{color:rgba(238,242,255,.2)}.text-indigo-50\/25{color:rgba(238,242,255,.25)}.text-indigo-50\/30{color:rgba(238,242,255,.3)}.text-indigo-50\/35{color:rgba(238,242,255,.35)}.text-indigo-50\/40{color:rgba(238,242,255,.4)}.text-indigo-50\/45{color:rgba(238,242,255,.45)}.text-indigo-50\/5{color:rgba(238,242,255,.05)}.text-indigo-50\/50{color:rgba(238,242,255,.5)}.text-indigo-50\/55{color:rgba(238,242,255,.55)}.text-indigo-50\/60{color:rgba(238,242,255,.6)}.text-indigo-50\/65{color:rgba(238,242,255,.65)}.text-indigo-50\/70{color:rgba(238,242,255,.7)}.text-indigo-50\/75{color:rgba(238,242,255,.75)}.text-indigo-50\/80{color:rgba(238,242,255,.8)}.text-indigo-50\/85{color:rgba(238,242,255,.85)}.text-indigo-50\/90{color:rgba(238,242,255,.9)}.text-indigo-50\/95{color:rgba(238,242,255,.95)}.text-indigo-500{--tw-text-opacity:1;color:rgb(99 102 241/var(--tw-text-opacity,1))}.text-indigo-500\/0{color:rgba(99,102,241,0)}.text-indigo-500\/10{color:rgba(99,102,241,.1)}.text-indigo-500\/100{color:#6366f1}.text-indigo-500\/15{color:rgba(99,102,241,.15)}.text-indigo-500\/20{color:rgba(99,102,241,.2)}.text-indigo-500\/25{color:rgba(99,102,241,.25)}.text-indigo-500\/30{color:rgba(99,102,241,.3)}.text-indigo-500\/35{color:rgba(99,102,241,.35)}.text-indigo-500\/40{color:rgba(99,102,241,.4)}.text-indigo-500\/45{color:rgba(99,102,241,.45)}.text-indigo-500\/5{color:rgba(99,102,241,.05)}.text-indigo-500\/50{color:rgba(99,102,241,.5)}.text-indigo-500\/55{color:rgba(99,102,241,.55)}.text-indigo-500\/60{color:rgba(99,102,241,.6)}.text-indigo-500\/65{color:rgba(99,102,241,.65)}.text-indigo-500\/70{color:rgba(99,102,241,.7)}.text-indigo-500\/75{color:rgba(99,102,241,.75)}.text-indigo-500\/80{color:rgba(99,102,241,.8)}.text-indigo-500\/85{color:rgba(99,102,241,.85)}.text-indigo-500\/90{color:rgba(99,102,241,.9)}.text-indigo-500\/95{color:rgba(99,102,241,.95)}.text-indigo-950{--tw-text-opacity:1;color:rgb(30 27 75/var(--tw-text-opacity,1))}.text-indigo-950\/0{color:rgba(30,27,75,0)}.text-indigo-950\/10{color:rgba(30,27,75,.1)}.text-indigo-950\/100{color:#1e1b4b}.text-indigo-950\/15{color:rgba(30,27,75,.15)}.text-indigo-950\/20{color:rgba(30,27,75,.2)}.text-indigo-950\/25{color:rgba(30,27,75,.25)}.text-indigo-950\/30{color:rgba(30,27,75,.3)}.text-indigo-950\/35{color:rgba(30,27,75,.35)}.text-indigo-950\/40{color:rgba(30,27,75,.4)}.text-indigo-950\/45{color:rgba(30,27,75,.45)}.text-indigo-950\/5{color:rgba(30,27,75,.05)}.text-indigo-950\/50{color:rgba(30,27,75,.5)}.text-indigo-950\/55{color:rgba(30,27,75,.55)}.text-indigo-950\/60{color:rgba(30,27,75,.6)}.text-indigo-950\/65{color:rgba(30,27,75,.65)}.text-indigo-950\/70{color:rgba(30,27,75,.7)}.text-indigo-950\/75{color:rgba(30,27,75,.75)}.text-indigo-950\/80{color:rgba(30,27,75,.8)}.text-indigo-950\/85{color:rgba(30,27,75,.85)}.text-indigo-950\/90{color:rgba(30,27,75,.9)}.text-indigo-950\/95{color:rgba(30,27,75,.95)}.text-lime-100{--tw-text-opacity:1;color:rgb(236 252 203/var(--tw-text-opacity,1))}.text-lime-100\/0{color:hsla(80,89%,89%,0)}.text-lime-100\/10{color:hsla(80,89%,89%,.1)}.text-lime-100\/100{color:#ecfccb}.text-lime-100\/15{color:hsla(80,89%,89%,.15)}.text-lime-100\/20{color:hsla(80,89%,89%,.2)}.text-lime-100\/25{color:hsla(80,89%,89%,.25)}.text-lime-100\/30{color:hsla(80,89%,89%,.3)}.text-lime-100\/35{color:hsla(80,89%,89%,.35)}.text-lime-100\/40{color:hsla(80,89%,89%,.4)}.text-lime-100\/45{color:hsla(80,89%,89%,.45)}.text-lime-100\/5{color:hsla(80,89%,89%,.05)}.text-lime-100\/50{color:hsla(80,89%,89%,.5)}.text-lime-100\/55{color:hsla(80,89%,89%,.55)}.text-lime-100\/60{color:hsla(80,89%,89%,.6)}.text-lime-100\/65{color:hsla(80,89%,89%,.65)}.text-lime-100\/70{color:hsla(80,89%,89%,.7)}.text-lime-100\/75{color:hsla(80,89%,89%,.75)}.text-lime-100\/80{color:hsla(80,89%,89%,.8)}.text-lime-100\/85{color:hsla(80,89%,89%,.85)}.text-lime-100\/90{color:hsla(80,89%,89%,.9)}.text-lime-100\/95{color:hsla(80,89%,89%,.95)}.text-lime-200{--tw-text-opacity:1;color:rgb(217 249 157/var(--tw-text-opacity,1))}.text-lime-200\/0{color:hsla(81,88%,80%,0)}.text-lime-200\/10{color:hsla(81,88%,80%,.1)}.text-lime-200\/100{color:#d9f99d}.text-lime-200\/15{color:hsla(81,88%,80%,.15)}.text-lime-200\/20{color:hsla(81,88%,80%,.2)}.text-lime-200\/25{color:hsla(81,88%,80%,.25)}.text-lime-200\/30{color:hsla(81,88%,80%,.3)}.text-lime-200\/35{color:hsla(81,88%,80%,.35)}.text-lime-200\/40{color:hsla(81,88%,80%,.4)}.text-lime-200\/45{color:hsla(81,88%,80%,.45)}.text-lime-200\/5{color:hsla(81,88%,80%,.05)}.text-lime-200\/50{color:hsla(81,88%,80%,.5)}.text-lime-200\/55{color:hsla(81,88%,80%,.55)}.text-lime-200\/60{color:hsla(81,88%,80%,.6)}.text-lime-200\/65{color:hsla(81,88%,80%,.65)}.text-lime-200\/70{color:hsla(81,88%,80%,.7)}.text-lime-200\/75{color:hsla(81,88%,80%,.75)}.text-lime-200\/80{color:hsla(81,88%,80%,.8)}.text-lime-200\/85{color:hsla(81,88%,80%,.85)}.text-lime-200\/90{color:hsla(81,88%,80%,.9)}.text-lime-200\/95{color:hsla(81,88%,80%,.95)}.text-lime-50{--tw-text-opacity:1;color:rgb(247 254 231/var(--tw-text-opacity,1))}.text-lime-50\/0{color:hsla(78,92%,95%,0)}.text-lime-50\/10{color:hsla(78,92%,95%,.1)}.text-lime-50\/100{color:#f7fee7}.text-lime-50\/15{color:hsla(78,92%,95%,.15)}.text-lime-50\/20{color:hsla(78,92%,95%,.2)}.text-lime-50\/25{color:hsla(78,92%,95%,.25)}.text-lime-50\/30{color:hsla(78,92%,95%,.3)}.text-lime-50\/35{color:hsla(78,92%,95%,.35)}.text-lime-50\/40{color:hsla(78,92%,95%,.4)}.text-lime-50\/45{color:hsla(78,92%,95%,.45)}.text-lime-50\/5{color:hsla(78,92%,95%,.05)}.text-lime-50\/50{color:hsla(78,92%,95%,.5)}.text-lime-50\/55{color:hsla(78,92%,95%,.55)}.text-lime-50\/60{color:hsla(78,92%,95%,.6)}.text-lime-50\/65{color:hsla(78,92%,95%,.65)}.text-lime-50\/70{color:hsla(78,92%,95%,.7)}.text-lime-50\/75{color:hsla(78,92%,95%,.75)}.text-lime-50\/80{color:hsla(78,92%,95%,.8)}.text-lime-50\/85{color:hsla(78,92%,95%,.85)}.text-lime-50\/90{color:hsla(78,92%,95%,.9)}.text-lime-50\/95{color:hsla(78,92%,95%,.95)}.text-lime-500{--tw-text-opacity:1;color:rgb(132 204 22/var(--tw-text-opacity,1))}.text-lime-500\/0{color:rgba(132,204,22,0)}.text-lime-500\/10{color:rgba(132,204,22,.1)}.text-lime-500\/100{color:#84cc16}.text-lime-500\/15{color:rgba(132,204,22,.15)}.text-lime-500\/20{color:rgba(132,204,22,.2)}.text-lime-500\/25{color:rgba(132,204,22,.25)}.text-lime-500\/30{color:rgba(132,204,22,.3)}.text-lime-500\/35{color:rgba(132,204,22,.35)}.text-lime-500\/40{color:rgba(132,204,22,.4)}.text-lime-500\/45{color:rgba(132,204,22,.45)}.text-lime-500\/5{color:rgba(132,204,22,.05)}.text-lime-500\/50{color:rgba(132,204,22,.5)}.text-lime-500\/55{color:rgba(132,204,22,.55)}.text-lime-500\/60{color:rgba(132,204,22,.6)}.text-lime-500\/65{color:rgba(132,204,22,.65)}.text-lime-500\/70{color:rgba(132,204,22,.7)}.text-lime-500\/75{color:rgba(132,204,22,.75)}.text-lime-500\/80{color:rgba(132,204,22,.8)}.text-lime-500\/85{color:rgba(132,204,22,.85)}.text-lime-500\/90{color:rgba(132,204,22,.9)}.text-lime-500\/95{color:rgba(132,204,22,.95)}.text-lime-950{--tw-text-opacity:1;color:rgb(26 46 5/var(--tw-text-opacity,1))}.text-lime-950\/0{color:rgba(26,46,5,0)}.text-lime-950\/10{color:rgba(26,46,5,.1)}.text-lime-950\/100{color:#1a2e05}.text-lime-950\/15{color:rgba(26,46,5,.15)}.text-lime-950\/20{color:rgba(26,46,5,.2)}.text-lime-950\/25{color:rgba(26,46,5,.25)}.text-lime-950\/30{color:rgba(26,46,5,.3)}.text-lime-950\/35{color:rgba(26,46,5,.35)}.text-lime-950\/40{color:rgba(26,46,5,.4)}.text-lime-950\/45{color:rgba(26,46,5,.45)}.text-lime-950\/5{color:rgba(26,46,5,.05)}.text-lime-950\/50{color:rgba(26,46,5,.5)}.text-lime-950\/55{color:rgba(26,46,5,.55)}.text-lime-950\/60{color:rgba(26,46,5,.6)}.text-lime-950\/65{color:rgba(26,46,5,.65)}.text-lime-950\/70{color:rgba(26,46,5,.7)}.text-lime-950\/75{color:rgba(26,46,5,.75)}.text-lime-950\/80{color:rgba(26,46,5,.8)}.text-lime-950\/85{color:rgba(26,46,5,.85)}.text-lime-950\/90{color:rgba(26,46,5,.9)}.text-lime-950\/95{color:rgba(26,46,5,.95)}.text-muted-foreground{color:hsl(var(--muted-foreground))}.text-neutral-100{--tw-text-opacity:1;color:rgb(245 245 245/var(--tw-text-opacity,1))}.text-neutral-100\/0{color:hsla(0,0%,96%,0)}.text-neutral-100\/10{color:hsla(0,0%,96%,.1)}.text-neutral-100\/100{color:#f5f5f5}.text-neutral-100\/15{color:hsla(0,0%,96%,.15)}.text-neutral-100\/20{color:hsla(0,0%,96%,.2)}.text-neutral-100\/25{color:hsla(0,0%,96%,.25)}.text-neutral-100\/30{color:hsla(0,0%,96%,.3)}.text-neutral-100\/35{color:hsla(0,0%,96%,.35)}.text-neutral-100\/40{color:hsla(0,0%,96%,.4)}.text-neutral-100\/45{color:hsla(0,0%,96%,.45)}.text-neutral-100\/5{color:hsla(0,0%,96%,.05)}.text-neutral-100\/50{color:hsla(0,0%,96%,.5)}.text-neutral-100\/55{color:hsla(0,0%,96%,.55)}.text-neutral-100\/60{color:hsla(0,0%,96%,.6)}.text-neutral-100\/65{color:hsla(0,0%,96%,.65)}.text-neutral-100\/70{color:hsla(0,0%,96%,.7)}.text-neutral-100\/75{color:hsla(0,0%,96%,.75)}.text-neutral-100\/80{color:hsla(0,0%,96%,.8)}.text-neutral-100\/85{color:hsla(0,0%,96%,.85)}.text-neutral-100\/90{color:hsla(0,0%,96%,.9)}.text-neutral-100\/95{color:hsla(0,0%,96%,.95)}.text-neutral-200{--tw-text-opacity:1;color:rgb(229 229 229/var(--tw-text-opacity,1))}.text-neutral-200\/0{color:hsla(0,0%,90%,0)}.text-neutral-200\/10{color:hsla(0,0%,90%,.1)}.text-neutral-200\/100{color:#e5e5e5}.text-neutral-200\/15{color:hsla(0,0%,90%,.15)}.text-neutral-200\/20{color:hsla(0,0%,90%,.2)}.text-neutral-200\/25{color:hsla(0,0%,90%,.25)}.text-neutral-200\/30{color:hsla(0,0%,90%,.3)}.text-neutral-200\/35{color:hsla(0,0%,90%,.35)}.text-neutral-200\/40{color:hsla(0,0%,90%,.4)}.text-neutral-200\/45{color:hsla(0,0%,90%,.45)}.text-neutral-200\/5{color:hsla(0,0%,90%,.05)}.text-neutral-200\/50{color:hsla(0,0%,90%,.5)}.text-neutral-200\/55{color:hsla(0,0%,90%,.55)}.text-neutral-200\/60{color:hsla(0,0%,90%,.6)}.text-neutral-200\/65{color:hsla(0,0%,90%,.65)}.text-neutral-200\/70{color:hsla(0,0%,90%,.7)}.text-neutral-200\/75{color:hsla(0,0%,90%,.75)}.text-neutral-200\/80{color:hsla(0,0%,90%,.8)}.text-neutral-200\/85{color:hsla(0,0%,90%,.85)}.text-neutral-200\/90{color:hsla(0,0%,90%,.9)}.text-neutral-200\/95{color:hsla(0,0%,90%,.95)}.text-neutral-50{--tw-text-opacity:1;color:rgb(250 250 250/var(--tw-text-opacity,1))}.text-neutral-50\/0{color:hsla(0,0%,98%,0)}.text-neutral-50\/10{color:hsla(0,0%,98%,.1)}.text-neutral-50\/100{color:#fafafa}.text-neutral-50\/15{color:hsla(0,0%,98%,.15)}.text-neutral-50\/20{color:hsla(0,0%,98%,.2)}.text-neutral-50\/25{color:hsla(0,0%,98%,.25)}.text-neutral-50\/30{color:hsla(0,0%,98%,.3)}.text-neutral-50\/35{color:hsla(0,0%,98%,.35)}.text-neutral-50\/40{color:hsla(0,0%,98%,.4)}.text-neutral-50\/45{color:hsla(0,0%,98%,.45)}.text-neutral-50\/5{color:hsla(0,0%,98%,.05)}.text-neutral-50\/50{color:hsla(0,0%,98%,.5)}.text-neutral-50\/55{color:hsla(0,0%,98%,.55)}.text-neutral-50\/60{color:hsla(0,0%,98%,.6)}.text-neutral-50\/65{color:hsla(0,0%,98%,.65)}.text-neutral-50\/70{color:hsla(0,0%,98%,.7)}.text-neutral-50\/75{color:hsla(0,0%,98%,.75)}.text-neutral-50\/80{color:hsla(0,0%,98%,.8)}.text-neutral-50\/85{color:hsla(0,0%,98%,.85)}.text-neutral-50\/90{color:hsla(0,0%,98%,.9)}.text-neutral-50\/95{color:hsla(0,0%,98%,.95)}.text-neutral-500{--tw-text-opacity:1;color:rgb(115 115 115/var(--tw-text-opacity,1))}.text-neutral-500\/0{color:hsla(0,0%,45%,0)}.text-neutral-500\/10{color:hsla(0,0%,45%,.1)}.text-neutral-500\/100{color:#737373}.text-neutral-500\/15{color:hsla(0,0%,45%,.15)}.text-neutral-500\/20{color:hsla(0,0%,45%,.2)}.text-neutral-500\/25{color:hsla(0,0%,45%,.25)}.text-neutral-500\/30{color:hsla(0,0%,45%,.3)}.text-neutral-500\/35{color:hsla(0,0%,45%,.35)}.text-neutral-500\/40{color:hsla(0,0%,45%,.4)}.text-neutral-500\/45{color:hsla(0,0%,45%,.45)}.text-neutral-500\/5{color:hsla(0,0%,45%,.05)}.text-neutral-500\/50{color:hsla(0,0%,45%,.5)}.text-neutral-500\/55{color:hsla(0,0%,45%,.55)}.text-neutral-500\/60{color:hsla(0,0%,45%,.6)}.text-neutral-500\/65{color:hsla(0,0%,45%,.65)}.text-neutral-500\/70{color:hsla(0,0%,45%,.7)}.text-neutral-500\/75{color:hsla(0,0%,45%,.75)}.text-neutral-500\/80{color:hsla(0,0%,45%,.8)}.text-neutral-500\/85{color:hsla(0,0%,45%,.85)}.text-neutral-500\/90{color:hsla(0,0%,45%,.9)}.text-neutral-500\/95{color:hsla(0,0%,45%,.95)}.text-neutral-950{--tw-text-opacity:1;color:rgb(10 10 10/var(--tw-text-opacity,1))}.text-neutral-950\/0{color:hsla(0,0%,4%,0)}.text-neutral-950\/10{color:hsla(0,0%,4%,.1)}.text-neutral-950\/100{color:#0a0a0a}.text-neutral-950\/15{color:hsla(0,0%,4%,.15)}.text-neutral-950\/20{color:hsla(0,0%,4%,.2)}.text-neutral-950\/25{color:hsla(0,0%,4%,.25)}.text-neutral-950\/30{color:hsla(0,0%,4%,.3)}.text-neutral-950\/35{color:hsla(0,0%,4%,.35)}.text-neutral-950\/40{color:hsla(0,0%,4%,.4)}.text-neutral-950\/45{color:hsla(0,0%,4%,.45)}.text-neutral-950\/5{color:hsla(0,0%,4%,.05)}.text-neutral-950\/50{color:hsla(0,0%,4%,.5)}.text-neutral-950\/55{color:hsla(0,0%,4%,.55)}.text-neutral-950\/60{color:hsla(0,0%,4%,.6)}.text-neutral-950\/65{color:hsla(0,0%,4%,.65)}.text-neutral-950\/70{color:hsla(0,0%,4%,.7)}.text-neutral-950\/75{color:hsla(0,0%,4%,.75)}.text-neutral-950\/80{color:hsla(0,0%,4%,.8)}.text-neutral-950\/85{color:hsla(0,0%,4%,.85)}.text-neutral-950\/90{color:hsla(0,0%,4%,.9)}.text-neutral-950\/95{color:hsla(0,0%,4%,.95)}.text-orange-100{--tw-text-opacity:1;color:rgb(255 237 213/var(--tw-text-opacity,1))}.text-orange-100\/0{color:rgba(255,237,213,0)}.text-orange-100\/10{color:rgba(255,237,213,.1)}.text-orange-100\/100{color:#ffedd5}.text-orange-100\/15{color:rgba(255,237,213,.15)}.text-orange-100\/20{color:rgba(255,237,213,.2)}.text-orange-100\/25{color:rgba(255,237,213,.25)}.text-orange-100\/30{color:rgba(255,237,213,.3)}.text-orange-100\/35{color:rgba(255,237,213,.35)}.text-orange-100\/40{color:rgba(255,237,213,.4)}.text-orange-100\/45{color:rgba(255,237,213,.45)}.text-orange-100\/5{color:rgba(255,237,213,.05)}.text-orange-100\/50{color:rgba(255,237,213,.5)}.text-orange-100\/55{color:rgba(255,237,213,.55)}.text-orange-100\/60{color:rgba(255,237,213,.6)}.text-orange-100\/65{color:rgba(255,237,213,.65)}.text-orange-100\/70{color:rgba(255,237,213,.7)}.text-orange-100\/75{color:rgba(255,237,213,.75)}.text-orange-100\/80{color:rgba(255,237,213,.8)}.text-orange-100\/85{color:rgba(255,237,213,.85)}.text-orange-100\/90{color:rgba(255,237,213,.9)}.text-orange-100\/95{color:rgba(255,237,213,.95)}.text-orange-200{--tw-text-opacity:1;color:rgb(254 215 170/var(--tw-text-opacity,1))}.text-orange-200\/0{color:hsla(32,98%,83%,0)}.text-orange-200\/10{color:hsla(32,98%,83%,.1)}.text-orange-200\/100{color:#fed7aa}.text-orange-200\/15{color:hsla(32,98%,83%,.15)}.text-orange-200\/20{color:hsla(32,98%,83%,.2)}.text-orange-200\/25{color:hsla(32,98%,83%,.25)}.text-orange-200\/30{color:hsla(32,98%,83%,.3)}.text-orange-200\/35{color:hsla(32,98%,83%,.35)}.text-orange-200\/40{color:hsla(32,98%,83%,.4)}.text-orange-200\/45{color:hsla(32,98%,83%,.45)}.text-orange-200\/5{color:hsla(32,98%,83%,.05)}.text-orange-200\/50{color:hsla(32,98%,83%,.5)}.text-orange-200\/55{color:hsla(32,98%,83%,.55)}.text-orange-200\/60{color:hsla(32,98%,83%,.6)}.text-orange-200\/65{color:hsla(32,98%,83%,.65)}.text-orange-200\/70{color:hsla(32,98%,83%,.7)}.text-orange-200\/75{color:hsla(32,98%,83%,.75)}.text-orange-200\/80{color:hsla(32,98%,83%,.8)}.text-orange-200\/85{color:hsla(32,98%,83%,.85)}.text-orange-200\/90{color:hsla(32,98%,83%,.9)}.text-orange-200\/95{color:hsla(32,98%,83%,.95)}.text-orange-400{--tw-text-opacity:1;color:rgb(251 146 60/var(--tw-text-opacity,1))}.text-orange-50{--tw-text-opacity:1;color:rgb(255 247 237/var(--tw-text-opacity,1))}.text-orange-50\/0{color:rgba(255,247,237,0)}.text-orange-50\/10{color:rgba(255,247,237,.1)}.text-orange-50\/100{color:#fff7ed}.text-orange-50\/15{color:rgba(255,247,237,.15)}.text-orange-50\/20{color:rgba(255,247,237,.2)}.text-orange-50\/25{color:rgba(255,247,237,.25)}.text-orange-50\/30{color:rgba(255,247,237,.3)}.text-orange-50\/35{color:rgba(255,247,237,.35)}.text-orange-50\/40{color:rgba(255,247,237,.4)}.text-orange-50\/45{color:rgba(255,247,237,.45)}.text-orange-50\/5{color:rgba(255,247,237,.05)}.text-orange-50\/50{color:rgba(255,247,237,.5)}.text-orange-50\/55{color:rgba(255,247,237,.55)}.text-orange-50\/60{color:rgba(255,247,237,.6)}.text-orange-50\/65{color:rgba(255,247,237,.65)}.text-orange-50\/70{color:rgba(255,247,237,.7)}.text-orange-50\/75{color:rgba(255,247,237,.75)}.text-orange-50\/80{color:rgba(255,247,237,.8)}.text-orange-50\/85{color:rgba(255,247,237,.85)}.text-orange-50\/90{color:rgba(255,247,237,.9)}.text-orange-50\/95{color:rgba(255,247,237,.95)}.text-orange-500{--tw-text-opacity:1;color:rgb(249 115 22/var(--tw-text-opacity,1))}.text-orange-500\/0{color:rgba(249,115,22,0)}.text-orange-500\/10{color:rgba(249,115,22,.1)}.text-orange-500\/100{color:#f97316}.text-orange-500\/15{color:rgba(249,115,22,.15)}.text-orange-500\/20{color:rgba(249,115,22,.2)}.text-orange-500\/25{color:rgba(249,115,22,.25)}.text-orange-500\/30{color:rgba(249,115,22,.3)}.text-orange-500\/35{color:rgba(249,115,22,.35)}.text-orange-500\/40{color:rgba(249,115,22,.4)}.text-orange-500\/45{color:rgba(249,115,22,.45)}.text-orange-500\/5{color:rgba(249,115,22,.05)}.text-orange-500\/50{color:rgba(249,115,22,.5)}.text-orange-500\/55{color:rgba(249,115,22,.55)}.text-orange-500\/60{color:rgba(249,115,22,.6)}.text-orange-500\/65{color:rgba(249,115,22,.65)}.text-orange-500\/70{color:rgba(249,115,22,.7)}.text-orange-500\/75{color:rgba(249,115,22,.75)}.text-orange-500\/80{color:rgba(249,115,22,.8)}.text-orange-500\/85{color:rgba(249,115,22,.85)}.text-orange-500\/90{color:rgba(249,115,22,.9)}.text-orange-500\/95{color:rgba(249,115,22,.95)}.text-orange-950{--tw-text-opacity:1;color:rgb(67 20 7/var(--tw-text-opacity,1))}.text-orange-950\/0{color:rgba(67,20,7,0)}.text-orange-950\/10{color:rgba(67,20,7,.1)}.text-orange-950\/100{color:#431407}.text-orange-950\/15{color:rgba(67,20,7,.15)}.text-orange-950\/20{color:rgba(67,20,7,.2)}.text-orange-950\/25{color:rgba(67,20,7,.25)}.text-orange-950\/30{color:rgba(67,20,7,.3)}.text-orange-950\/35{color:rgba(67,20,7,.35)}.text-orange-950\/40{color:rgba(67,20,7,.4)}.text-orange-950\/45{color:rgba(67,20,7,.45)}.text-orange-950\/5{color:rgba(67,20,7,.05)}.text-orange-950\/50{color:rgba(67,20,7,.5)}.text-orange-950\/55{color:rgba(67,20,7,.55)}.text-orange-950\/60{color:rgba(67,20,7,.6)}.text-orange-950\/65{color:rgba(67,20,7,.65)}.text-orange-950\/70{color:rgba(67,20,7,.7)}.text-orange-950\/75{color:rgba(67,20,7,.75)}.text-orange-950\/80{color:rgba(67,20,7,.8)}.text-orange-950\/85{color:rgba(67,20,7,.85)}.text-orange-950\/90{color:rgba(67,20,7,.9)}.text-orange-950\/95{color:rgba(67,20,7,.95)}.text-pink-100{--tw-text-opacity:1;color:rgb(252 231 243/var(--tw-text-opacity,1))}.text-pink-100\/0{color:rgba(252,231,243,0)}.text-pink-100\/10{color:rgba(252,231,243,.1)}.text-pink-100\/100{color:#fce7f3}.text-pink-100\/15{color:rgba(252,231,243,.15)}.text-pink-100\/20{color:rgba(252,231,243,.2)}.text-pink-100\/25{color:rgba(252,231,243,.25)}.text-pink-100\/30{color:rgba(252,231,243,.3)}.text-pink-100\/35{color:rgba(252,231,243,.35)}.text-pink-100\/40{color:rgba(252,231,243,.4)}.text-pink-100\/45{color:rgba(252,231,243,.45)}.text-pink-100\/5{color:rgba(252,231,243,.05)}.text-pink-100\/50{color:rgba(252,231,243,.5)}.text-pink-100\/55{color:rgba(252,231,243,.55)}.text-pink-100\/60{color:rgba(252,231,243,.6)}.text-pink-100\/65{color:rgba(252,231,243,.65)}.text-pink-100\/70{color:rgba(252,231,243,.7)}.text-pink-100\/75{color:rgba(252,231,243,.75)}.text-pink-100\/80{color:rgba(252,231,243,.8)}.text-pink-100\/85{color:rgba(252,231,243,.85)}.text-pink-100\/90{color:rgba(252,231,243,.9)}.text-pink-100\/95{color:rgba(252,231,243,.95)}.text-pink-200{--tw-text-opacity:1;color:rgb(251 207 232/var(--tw-text-opacity,1))}.text-pink-200\/0{color:rgba(251,207,232,0)}.text-pink-200\/10{color:rgba(251,207,232,.1)}.text-pink-200\/100{color:#fbcfe8}.text-pink-200\/15{color:rgba(251,207,232,.15)}.text-pink-200\/20{color:rgba(251,207,232,.2)}.text-pink-200\/25{color:rgba(251,207,232,.25)}.text-pink-200\/30{color:rgba(251,207,232,.3)}.text-pink-200\/35{color:rgba(251,207,232,.35)}.text-pink-200\/40{color:rgba(251,207,232,.4)}.text-pink-200\/45{color:rgba(251,207,232,.45)}.text-pink-200\/5{color:rgba(251,207,232,.05)}.text-pink-200\/50{color:rgba(251,207,232,.5)}.text-pink-200\/55{color:rgba(251,207,232,.55)}.text-pink-200\/60{color:rgba(251,207,232,.6)}.text-pink-200\/65{color:rgba(251,207,232,.65)}.text-pink-200\/70{color:rgba(251,207,232,.7)}.text-pink-200\/75{color:rgba(251,207,232,.75)}.text-pink-200\/80{color:rgba(251,207,232,.8)}.text-pink-200\/85{color:rgba(251,207,232,.85)}.text-pink-200\/90{color:rgba(251,207,232,.9)}.text-pink-200\/95{color:rgba(251,207,232,.95)}.text-pink-50{--tw-text-opacity:1;color:rgb(253 242 248/var(--tw-text-opacity,1))}.text-pink-50\/0{color:rgba(253,242,248,0)}.text-pink-50\/10{color:rgba(253,242,248,.1)}.text-pink-50\/100{color:#fdf2f8}.text-pink-50\/15{color:rgba(253,242,248,.15)}.text-pink-50\/20{color:rgba(253,242,248,.2)}.text-pink-50\/25{color:rgba(253,242,248,.25)}.text-pink-50\/30{color:rgba(253,242,248,.3)}.text-pink-50\/35{color:rgba(253,242,248,.35)}.text-pink-50\/40{color:rgba(253,242,248,.4)}.text-pink-50\/45{color:rgba(253,242,248,.45)}.text-pink-50\/5{color:rgba(253,242,248,.05)}.text-pink-50\/50{color:rgba(253,242,248,.5)}.text-pink-50\/55{color:rgba(253,242,248,.55)}.text-pink-50\/60{color:rgba(253,242,248,.6)}.text-pink-50\/65{color:rgba(253,242,248,.65)}.text-pink-50\/70{color:rgba(253,242,248,.7)}.text-pink-50\/75{color:rgba(253,242,248,.75)}.text-pink-50\/80{color:rgba(253,242,248,.8)}.text-pink-50\/85{color:rgba(253,242,248,.85)}.text-pink-50\/90{color:rgba(253,242,248,.9)}.text-pink-50\/95{color:rgba(253,242,248,.95)}.text-pink-500{--tw-text-opacity:1;color:rgb(236 72 153/var(--tw-text-opacity,1))}.text-pink-500\/0{color:rgba(236,72,153,0)}.text-pink-500\/10{color:rgba(236,72,153,.1)}.text-pink-500\/100{color:#ec4899}.text-pink-500\/15{color:rgba(236,72,153,.15)}.text-pink-500\/20{color:rgba(236,72,153,.2)}.text-pink-500\/25{color:rgba(236,72,153,.25)}.text-pink-500\/30{color:rgba(236,72,153,.3)}.text-pink-500\/35{color:rgba(236,72,153,.35)}.text-pink-500\/40{color:rgba(236,72,153,.4)}.text-pink-500\/45{color:rgba(236,72,153,.45)}.text-pink-500\/5{color:rgba(236,72,153,.05)}.text-pink-500\/50{color:rgba(236,72,153,.5)}.text-pink-500\/55{color:rgba(236,72,153,.55)}.text-pink-500\/60{color:rgba(236,72,153,.6)}.text-pink-500\/65{color:rgba(236,72,153,.65)}.text-pink-500\/70{color:rgba(236,72,153,.7)}.text-pink-500\/75{color:rgba(236,72,153,.75)}.text-pink-500\/80{color:rgba(236,72,153,.8)}.text-pink-500\/85{color:rgba(236,72,153,.85)}.text-pink-500\/90{color:rgba(236,72,153,.9)}.text-pink-500\/95{color:rgba(236,72,153,.95)}.text-pink-950{--tw-text-opacity:1;color:rgb(80 7 36/var(--tw-text-opacity,1))}.text-pink-950\/0{color:rgba(80,7,36,0)}.text-pink-950\/10{color:rgba(80,7,36,.1)}.text-pink-950\/100{color:#500724}.text-pink-950\/15{color:rgba(80,7,36,.15)}.text-pink-950\/20{color:rgba(80,7,36,.2)}.text-pink-950\/25{color:rgba(80,7,36,.25)}.text-pink-950\/30{color:rgba(80,7,36,.3)}.text-pink-950\/35{color:rgba(80,7,36,.35)}.text-pink-950\/40{color:rgba(80,7,36,.4)}.text-pink-950\/45{color:rgba(80,7,36,.45)}.text-pink-950\/5{color:rgba(80,7,36,.05)}.text-pink-950\/50{color:rgba(80,7,36,.5)}.text-pink-950\/55{color:rgba(80,7,36,.55)}.text-pink-950\/60{color:rgba(80,7,36,.6)}.text-pink-950\/65{color:rgba(80,7,36,.65)}.text-pink-950\/70{color:rgba(80,7,36,.7)}.text-pink-950\/75{color:rgba(80,7,36,.75)}.text-pink-950\/80{color:rgba(80,7,36,.8)}.text-pink-950\/85{color:rgba(80,7,36,.85)}.text-pink-950\/90{color:rgba(80,7,36,.9)}.text-pink-950\/95{color:rgba(80,7,36,.95)}.text-popover-foreground{color:hsl(var(--popover-foreground))}.text-primary{color:hsl(var(--primary))}.text-primary-foreground{color:hsl(var(--primary-foreground))}.text-primary\/80{color:hsl(var(--primary)/.8)}.text-purple-100{--tw-text-opacity:1;color:rgb(243 232 255/var(--tw-text-opacity,1))}.text-purple-100\/0{color:rgba(243,232,255,0)}.text-purple-100\/10{color:rgba(243,232,255,.1)}.text-purple-100\/100{color:#f3e8ff}.text-purple-100\/15{color:rgba(243,232,255,.15)}.text-purple-100\/20{color:rgba(243,232,255,.2)}.text-purple-100\/25{color:rgba(243,232,255,.25)}.text-purple-100\/30{color:rgba(243,232,255,.3)}.text-purple-100\/35{color:rgba(243,232,255,.35)}.text-purple-100\/40{color:rgba(243,232,255,.4)}.text-purple-100\/45{color:rgba(243,232,255,.45)}.text-purple-100\/5{color:rgba(243,232,255,.05)}.text-purple-100\/50{color:rgba(243,232,255,.5)}.text-purple-100\/55{color:rgba(243,232,255,.55)}.text-purple-100\/60{color:rgba(243,232,255,.6)}.text-purple-100\/65{color:rgba(243,232,255,.65)}.text-purple-100\/70{color:rgba(243,232,255,.7)}.text-purple-100\/75{color:rgba(243,232,255,.75)}.text-purple-100\/80{color:rgba(243,232,255,.8)}.text-purple-100\/85{color:rgba(243,232,255,.85)}.text-purple-100\/90{color:rgba(243,232,255,.9)}.text-purple-100\/95{color:rgba(243,232,255,.95)}.text-purple-200{--tw-text-opacity:1;color:rgb(233 213 255/var(--tw-text-opacity,1))}.text-purple-200\/0{color:rgba(233,213,255,0)}.text-purple-200\/10{color:rgba(233,213,255,.1)}.text-purple-200\/100{color:#e9d5ff}.text-purple-200\/15{color:rgba(233,213,255,.15)}.text-purple-200\/20{color:rgba(233,213,255,.2)}.text-purple-200\/25{color:rgba(233,213,255,.25)}.text-purple-200\/30{color:rgba(233,213,255,.3)}.text-purple-200\/35{color:rgba(233,213,255,.35)}.text-purple-200\/40{color:rgba(233,213,255,.4)}.text-purple-200\/45{color:rgba(233,213,255,.45)}.text-purple-200\/5{color:rgba(233,213,255,.05)}.text-purple-200\/50{color:rgba(233,213,255,.5)}.text-purple-200\/55{color:rgba(233,213,255,.55)}.text-purple-200\/60{color:rgba(233,213,255,.6)}.text-purple-200\/65{color:rgba(233,213,255,.65)}.text-purple-200\/70{color:rgba(233,213,255,.7)}.text-purple-200\/75{color:rgba(233,213,255,.75)}.text-purple-200\/80{color:rgba(233,213,255,.8)}.text-purple-200\/85{color:rgba(233,213,255,.85)}.text-purple-200\/90{color:rgba(233,213,255,.9)}.text-purple-200\/95{color:rgba(233,213,255,.95)}.text-purple-400{--tw-text-opacity:1;color:rgb(192 132 252/var(--tw-text-opacity,1))}.text-purple-50{--tw-text-opacity:1;color:rgb(250 245 255/var(--tw-text-opacity,1))}.text-purple-50\/0{color:rgba(250,245,255,0)}.text-purple-50\/10{color:rgba(250,245,255,.1)}.text-purple-50\/100{color:#faf5ff}.text-purple-50\/15{color:rgba(250,245,255,.15)}.text-purple-50\/20{color:rgba(250,245,255,.2)}.text-purple-50\/25{color:rgba(250,245,255,.25)}.text-purple-50\/30{color:rgba(250,245,255,.3)}.text-purple-50\/35{color:rgba(250,245,255,.35)}.text-purple-50\/40{color:rgba(250,245,255,.4)}.text-purple-50\/45{color:rgba(250,245,255,.45)}.text-purple-50\/5{color:rgba(250,245,255,.05)}.text-purple-50\/50{color:rgba(250,245,255,.5)}.text-purple-50\/55{color:rgba(250,245,255,.55)}.text-purple-50\/60{color:rgba(250,245,255,.6)}.text-purple-50\/65{color:rgba(250,245,255,.65)}.text-purple-50\/70{color:rgba(250,245,255,.7)}.text-purple-50\/75{color:rgba(250,245,255,.75)}.text-purple-50\/80{color:rgba(250,245,255,.8)}.text-purple-50\/85{color:rgba(250,245,255,.85)}.text-purple-50\/90{color:rgba(250,245,255,.9)}.text-purple-50\/95{color:rgba(250,245,255,.95)}.text-purple-500{--tw-text-opacity:1;color:rgb(168 85 247/var(--tw-text-opacity,1))}.text-purple-500\/0{color:rgba(168,85,247,0)}.text-purple-500\/10{color:rgba(168,85,247,.1)}.text-purple-500\/100{color:#a855f7}.text-purple-500\/15{color:rgba(168,85,247,.15)}.text-purple-500\/20{color:rgba(168,85,247,.2)}.text-purple-500\/25{color:rgba(168,85,247,.25)}.text-purple-500\/30{color:rgba(168,85,247,.3)}.text-purple-500\/35{color:rgba(168,85,247,.35)}.text-purple-500\/40{color:rgba(168,85,247,.4)}.text-purple-500\/45{color:rgba(168,85,247,.45)}.text-purple-500\/5{color:rgba(168,85,247,.05)}.text-purple-500\/50{color:rgba(168,85,247,.5)}.text-purple-500\/55{color:rgba(168,85,247,.55)}.text-purple-500\/60{color:rgba(168,85,247,.6)}.text-purple-500\/65{color:rgba(168,85,247,.65)}.text-purple-500\/70{color:rgba(168,85,247,.7)}.text-purple-500\/75{color:rgba(168,85,247,.75)}.text-purple-500\/80{color:rgba(168,85,247,.8)}.text-purple-500\/85{color:rgba(168,85,247,.85)}.text-purple-500\/90{color:rgba(168,85,247,.9)}.text-purple-500\/95{color:rgba(168,85,247,.95)}.text-purple-700{--tw-text-opacity:1;color:rgb(126 34 206/var(--tw-text-opacity,1))}.text-purple-950{--tw-text-opacity:1;color:rgb(59 7 100/var(--tw-text-opacity,1))}.text-purple-950\/0{color:rgba(59,7,100,0)}.text-purple-950\/10{color:rgba(59,7,100,.1)}.text-purple-950\/100{color:#3b0764}.text-purple-950\/15{color:rgba(59,7,100,.15)}.text-purple-950\/20{color:rgba(59,7,100,.2)}.text-purple-950\/25{color:rgba(59,7,100,.25)}.text-purple-950\/30{color:rgba(59,7,100,.3)}.text-purple-950\/35{color:rgba(59,7,100,.35)}.text-purple-950\/40{color:rgba(59,7,100,.4)}.text-purple-950\/45{color:rgba(59,7,100,.45)}.text-purple-950\/5{color:rgba(59,7,100,.05)}.text-purple-950\/50{color:rgba(59,7,100,.5)}.text-purple-950\/55{color:rgba(59,7,100,.55)}.text-purple-950\/60{color:rgba(59,7,100,.6)}.text-purple-950\/65{color:rgba(59,7,100,.65)}.text-purple-950\/70{color:rgba(59,7,100,.7)}.text-purple-950\/75{color:rgba(59,7,100,.75)}.text-purple-950\/80{color:rgba(59,7,100,.8)}.text-purple-950\/85{color:rgba(59,7,100,.85)}.text-purple-950\/90{color:rgba(59,7,100,.9)}.text-purple-950\/95{color:rgba(59,7,100,.95)}.text-red-100{--tw-text-opacity:1;color:rgb(254 226 226/var(--tw-text-opacity,1))}.text-red-100\/0{color:hsla(0,93%,94%,0)}.text-red-100\/10{color:hsla(0,93%,94%,.1)}.text-red-100\/100{color:#fee2e2}.text-red-100\/15{color:hsla(0,93%,94%,.15)}.text-red-100\/20{color:hsla(0,93%,94%,.2)}.text-red-100\/25{color:hsla(0,93%,94%,.25)}.text-red-100\/30{color:hsla(0,93%,94%,.3)}.text-red-100\/35{color:hsla(0,93%,94%,.35)}.text-red-100\/40{color:hsla(0,93%,94%,.4)}.text-red-100\/45{color:hsla(0,93%,94%,.45)}.text-red-100\/5{color:hsla(0,93%,94%,.05)}.text-red-100\/50{color:hsla(0,93%,94%,.5)}.text-red-100\/55{color:hsla(0,93%,94%,.55)}.text-red-100\/60{color:hsla(0,93%,94%,.6)}.text-red-100\/65{color:hsla(0,93%,94%,.65)}.text-red-100\/70{color:hsla(0,93%,94%,.7)}.text-red-100\/75{color:hsla(0,93%,94%,.75)}.text-red-100\/80{color:hsla(0,93%,94%,.8)}.text-red-100\/85{color:hsla(0,93%,94%,.85)}.text-red-100\/90{color:hsla(0,93%,94%,.9)}.text-red-100\/95{color:hsla(0,93%,94%,.95)}.text-red-200{--tw-text-opacity:1;color:rgb(254 202 202/var(--tw-text-opacity,1))}.text-red-200\/0{color:hsla(0,96%,89%,0)}.text-red-200\/10{color:hsla(0,96%,89%,.1)}.text-red-200\/100{color:#fecaca}.text-red-200\/15{color:hsla(0,96%,89%,.15)}.text-red-200\/20{color:hsla(0,96%,89%,.2)}.text-red-200\/25{color:hsla(0,96%,89%,.25)}.text-red-200\/30{color:hsla(0,96%,89%,.3)}.text-red-200\/35{color:hsla(0,96%,89%,.35)}.text-red-200\/40{color:hsla(0,96%,89%,.4)}.text-red-200\/45{color:hsla(0,96%,89%,.45)}.text-red-200\/5{color:hsla(0,96%,89%,.05)}.text-red-200\/50{color:hsla(0,96%,89%,.5)}.text-red-200\/55{color:hsla(0,96%,89%,.55)}.text-red-200\/60{color:hsla(0,96%,89%,.6)}.text-red-200\/65{color:hsla(0,96%,89%,.65)}.text-red-200\/70{color:hsla(0,96%,89%,.7)}.text-red-200\/75{color:hsla(0,96%,89%,.75)}.text-red-200\/80{color:hsla(0,96%,89%,.8)}.text-red-200\/85{color:hsla(0,96%,89%,.85)}.text-red-200\/90{color:hsla(0,96%,89%,.9)}.text-red-200\/95{color:hsla(0,96%,89%,.95)}.text-red-400{--tw-text-opacity:1;color:rgb(248 113 113/var(--tw-text-opacity,1))}.text-red-50{--tw-text-opacity:1;color:rgb(254 242 242/var(--tw-text-opacity,1))}.text-red-50\/0{color:hsla(0,86%,97%,0)}.text-red-50\/10{color:hsla(0,86%,97%,.1)}.text-red-50\/100{color:#fef2f2}.text-red-50\/15{color:hsla(0,86%,97%,.15)}.text-red-50\/20{color:hsla(0,86%,97%,.2)}.text-red-50\/25{color:hsla(0,86%,97%,.25)}.text-red-50\/30{color:hsla(0,86%,97%,.3)}.text-red-50\/35{color:hsla(0,86%,97%,.35)}.text-red-50\/40{color:hsla(0,86%,97%,.4)}.text-red-50\/45{color:hsla(0,86%,97%,.45)}.text-red-50\/5{color:hsla(0,86%,97%,.05)}.text-red-50\/50{color:hsla(0,86%,97%,.5)}.text-red-50\/55{color:hsla(0,86%,97%,.55)}.text-red-50\/60{color:hsla(0,86%,97%,.6)}.text-red-50\/65{color:hsla(0,86%,97%,.65)}.text-red-50\/70{color:hsla(0,86%,97%,.7)}.text-red-50\/75{color:hsla(0,86%,97%,.75)}.text-red-50\/80{color:hsla(0,86%,97%,.8)}.text-red-50\/85{color:hsla(0,86%,97%,.85)}.text-red-50\/90{color:hsla(0,86%,97%,.9)}.text-red-50\/95{color:hsla(0,86%,97%,.95)}.text-red-500{--tw-text-opacity:1;color:rgb(239 68 68/var(--tw-text-opacity,1))}.text-red-500\/0{color:rgba(239,68,68,0)}.text-red-500\/10{color:rgba(239,68,68,.1)}.text-red-500\/100{color:#ef4444}.text-red-500\/15{color:rgba(239,68,68,.15)}.text-red-500\/20{color:rgba(239,68,68,.2)}.text-red-500\/25{color:rgba(239,68,68,.25)}.text-red-500\/30{color:rgba(239,68,68,.3)}.text-red-500\/35{color:rgba(239,68,68,.35)}.text-red-500\/40{color:rgba(239,68,68,.4)}.text-red-500\/45{color:rgba(239,68,68,.45)}.text-red-500\/5{color:rgba(239,68,68,.05)}.text-red-500\/50{color:rgba(239,68,68,.5)}.text-red-500\/55{color:rgba(239,68,68,.55)}.text-red-500\/60{color:rgba(239,68,68,.6)}.text-red-500\/65{color:rgba(239,68,68,.65)}.text-red-500\/70{color:rgba(239,68,68,.7)}.text-red-500\/75{color:rgba(239,68,68,.75)}.text-red-500\/80{color:rgba(239,68,68,.8)}.text-red-500\/85{color:rgba(239,68,68,.85)}.text-red-500\/90{color:rgba(239,68,68,.9)}.text-red-500\/95{color:rgba(239,68,68,.95)}.text-red-950{--tw-text-opacity:1;color:rgb(69 10 10/var(--tw-text-opacity,1))}.text-red-950\/0{color:rgba(69,10,10,0)}.text-red-950\/10{color:rgba(69,10,10,.1)}.text-red-950\/100{color:#450a0a}.text-red-950\/15{color:rgba(69,10,10,.15)}.text-red-950\/20{color:rgba(69,10,10,.2)}.text-red-950\/25{color:rgba(69,10,10,.25)}.text-red-950\/30{color:rgba(69,10,10,.3)}.text-red-950\/35{color:rgba(69,10,10,.35)}.text-red-950\/40{color:rgba(69,10,10,.4)}.text-red-950\/45{color:rgba(69,10,10,.45)}.text-red-950\/5{color:rgba(69,10,10,.05)}.text-red-950\/50{color:rgba(69,10,10,.5)}.text-red-950\/55{color:rgba(69,10,10,.55)}.text-red-950\/60{color:rgba(69,10,10,.6)}.text-red-950\/65{color:rgba(69,10,10,.65)}.text-red-950\/70{color:rgba(69,10,10,.7)}.text-red-950\/75{color:rgba(69,10,10,.75)}.text-red-950\/80{color:rgba(69,10,10,.8)}.text-red-950\/85{color:rgba(69,10,10,.85)}.text-red-950\/90{color:rgba(69,10,10,.9)}.text-red-950\/95{color:rgba(69,10,10,.95)}.text-rose-100{--tw-text-opacity:1;color:rgb(255 228 230/var(--tw-text-opacity,1))}.text-rose-100\/0{color:rgba(255,228,230,0)}.text-rose-100\/10{color:rgba(255,228,230,.1)}.text-rose-100\/100{color:#ffe4e6}.text-rose-100\/15{color:rgba(255,228,230,.15)}.text-rose-100\/20{color:rgba(255,228,230,.2)}.text-rose-100\/25{color:rgba(255,228,230,.25)}.text-rose-100\/30{color:rgba(255,228,230,.3)}.text-rose-100\/35{color:rgba(255,228,230,.35)}.text-rose-100\/40{color:rgba(255,228,230,.4)}.text-rose-100\/45{color:rgba(255,228,230,.45)}.text-rose-100\/5{color:rgba(255,228,230,.05)}.text-rose-100\/50{color:rgba(255,228,230,.5)}.text-rose-100\/55{color:rgba(255,228,230,.55)}.text-rose-100\/60{color:rgba(255,228,230,.6)}.text-rose-100\/65{color:rgba(255,228,230,.65)}.text-rose-100\/70{color:rgba(255,228,230,.7)}.text-rose-100\/75{color:rgba(255,228,230,.75)}.text-rose-100\/80{color:rgba(255,228,230,.8)}.text-rose-100\/85{color:rgba(255,228,230,.85)}.text-rose-100\/90{color:rgba(255,228,230,.9)}.text-rose-100\/95{color:rgba(255,228,230,.95)}.text-rose-200{--tw-text-opacity:1;color:rgb(254 205 211/var(--tw-text-opacity,1))}.text-rose-200\/0{color:rgba(254,205,211,0)}.text-rose-200\/10{color:rgba(254,205,211,.1)}.text-rose-200\/100{color:#fecdd3}.text-rose-200\/15{color:rgba(254,205,211,.15)}.text-rose-200\/20{color:rgba(254,205,211,.2)}.text-rose-200\/25{color:rgba(254,205,211,.25)}.text-rose-200\/30{color:rgba(254,205,211,.3)}.text-rose-200\/35{color:rgba(254,205,211,.35)}.text-rose-200\/40{color:rgba(254,205,211,.4)}.text-rose-200\/45{color:rgba(254,205,211,.45)}.text-rose-200\/5{color:rgba(254,205,211,.05)}.text-rose-200\/50{color:rgba(254,205,211,.5)}.text-rose-200\/55{color:rgba(254,205,211,.55)}.text-rose-200\/60{color:rgba(254,205,211,.6)}.text-rose-200\/65{color:rgba(254,205,211,.65)}.text-rose-200\/70{color:rgba(254,205,211,.7)}.text-rose-200\/75{color:rgba(254,205,211,.75)}.text-rose-200\/80{color:rgba(254,205,211,.8)}.text-rose-200\/85{color:rgba(254,205,211,.85)}.text-rose-200\/90{color:rgba(254,205,211,.9)}.text-rose-200\/95{color:rgba(254,205,211,.95)}.text-rose-300{--tw-text-opacity:1;color:rgb(253 164 175/var(--tw-text-opacity,1))}.text-rose-50{--tw-text-opacity:1;color:rgb(255 241 242/var(--tw-text-opacity,1))}.text-rose-50\/0{color:rgba(255,241,242,0)}.text-rose-50\/10{color:rgba(255,241,242,.1)}.text-rose-50\/100{color:#fff1f2}.text-rose-50\/15{color:rgba(255,241,242,.15)}.text-rose-50\/20{color:rgba(255,241,242,.2)}.text-rose-50\/25{color:rgba(255,241,242,.25)}.text-rose-50\/30{color:rgba(255,241,242,.3)}.text-rose-50\/35{color:rgba(255,241,242,.35)}.text-rose-50\/40{color:rgba(255,241,242,.4)}.text-rose-50\/45{color:rgba(255,241,242,.45)}.text-rose-50\/5{color:rgba(255,241,242,.05)}.text-rose-50\/50{color:rgba(255,241,242,.5)}.text-rose-50\/55{color:rgba(255,241,242,.55)}.text-rose-50\/60{color:rgba(255,241,242,.6)}.text-rose-50\/65{color:rgba(255,241,242,.65)}.text-rose-50\/70{color:rgba(255,241,242,.7)}.text-rose-50\/75{color:rgba(255,241,242,.75)}.text-rose-50\/80{color:rgba(255,241,242,.8)}.text-rose-50\/85{color:rgba(255,241,242,.85)}.text-rose-50\/90{color:rgba(255,241,242,.9)}.text-rose-50\/95{color:rgba(255,241,242,.95)}.text-rose-500{--tw-text-opacity:1;color:rgb(244 63 94/var(--tw-text-opacity,1))}.text-rose-500\/0{color:rgba(244,63,94,0)}.text-rose-500\/10{color:rgba(244,63,94,.1)}.text-rose-500\/100{color:#f43f5e}.text-rose-500\/15{color:rgba(244,63,94,.15)}.text-rose-500\/20{color:rgba(244,63,94,.2)}.text-rose-500\/25{color:rgba(244,63,94,.25)}.text-rose-500\/30{color:rgba(244,63,94,.3)}.text-rose-500\/35{color:rgba(244,63,94,.35)}.text-rose-500\/40{color:rgba(244,63,94,.4)}.text-rose-500\/45{color:rgba(244,63,94,.45)}.text-rose-500\/5{color:rgba(244,63,94,.05)}.text-rose-500\/50{color:rgba(244,63,94,.5)}.text-rose-500\/55{color:rgba(244,63,94,.55)}.text-rose-500\/60{color:rgba(244,63,94,.6)}.text-rose-500\/65{color:rgba(244,63,94,.65)}.text-rose-500\/70{color:rgba(244,63,94,.7)}.text-rose-500\/75{color:rgba(244,63,94,.75)}.text-rose-500\/80{color:rgba(244,63,94,.8)}.text-rose-500\/85{color:rgba(244,63,94,.85)}.text-rose-500\/90{color:rgba(244,63,94,.9)}.text-rose-500\/95{color:rgba(244,63,94,.95)}.text-rose-950{--tw-text-opacity:1;color:rgb(76 5 25/var(--tw-text-opacity,1))}.text-rose-950\/0{color:rgba(76,5,25,0)}.text-rose-950\/10{color:rgba(76,5,25,.1)}.text-rose-950\/100{color:#4c0519}.text-rose-950\/15{color:rgba(76,5,25,.15)}.text-rose-950\/20{color:rgba(76,5,25,.2)}.text-rose-950\/25{color:rgba(76,5,25,.25)}.text-rose-950\/30{color:rgba(76,5,25,.3)}.text-rose-950\/35{color:rgba(76,5,25,.35)}.text-rose-950\/40{color:rgba(76,5,25,.4)}.text-rose-950\/45{color:rgba(76,5,25,.45)}.text-rose-950\/5{color:rgba(76,5,25,.05)}.text-rose-950\/50{color:rgba(76,5,25,.5)}.text-rose-950\/55{color:rgba(76,5,25,.55)}.text-rose-950\/60{color:rgba(76,5,25,.6)}.text-rose-950\/65{color:rgba(76,5,25,.65)}.text-rose-950\/70{color:rgba(76,5,25,.7)}.text-rose-950\/75{color:rgba(76,5,25,.75)}.text-rose-950\/80{color:rgba(76,5,25,.8)}.text-rose-950\/85{color:rgba(76,5,25,.85)}.text-rose-950\/90{color:rgba(76,5,25,.9)}.text-rose-950\/95{color:rgba(76,5,25,.95)}.text-secondary-foreground{color:hsl(var(--secondary-foreground))}.text-sky-100{--tw-text-opacity:1;color:rgb(224 242 254/var(--tw-text-opacity,1))}.text-sky-100\/0{color:rgba(224,242,254,0)}.text-sky-100\/10{color:rgba(224,242,254,.1)}.text-sky-100\/100{color:#e0f2fe}.text-sky-100\/15{color:rgba(224,242,254,.15)}.text-sky-100\/20{color:rgba(224,242,254,.2)}.text-sky-100\/25{color:rgba(224,242,254,.25)}.text-sky-100\/30{color:rgba(224,242,254,.3)}.text-sky-100\/35{color:rgba(224,242,254,.35)}.text-sky-100\/40{color:rgba(224,242,254,.4)}.text-sky-100\/45{color:rgba(224,242,254,.45)}.text-sky-100\/5{color:rgba(224,242,254,.05)}.text-sky-100\/50{color:rgba(224,242,254,.5)}.text-sky-100\/55{color:rgba(224,242,254,.55)}.text-sky-100\/60{color:rgba(224,242,254,.6)}.text-sky-100\/65{color:rgba(224,242,254,.65)}.text-sky-100\/70{color:rgba(224,242,254,.7)}.text-sky-100\/75{color:rgba(224,242,254,.75)}.text-sky-100\/80{color:rgba(224,242,254,.8)}.text-sky-100\/85{color:rgba(224,242,254,.85)}.text-sky-100\/90{color:rgba(224,242,254,.9)}.text-sky-100\/95{color:rgba(224,242,254,.95)}.text-sky-200{--tw-text-opacity:1;color:rgb(186 230 253/var(--tw-text-opacity,1))}.text-sky-200\/0{color:rgba(186,230,253,0)}.text-sky-200\/10{color:rgba(186,230,253,.1)}.text-sky-200\/100{color:#bae6fd}.text-sky-200\/15{color:rgba(186,230,253,.15)}.text-sky-200\/20{color:rgba(186,230,253,.2)}.text-sky-200\/25{color:rgba(186,230,253,.25)}.text-sky-200\/30{color:rgba(186,230,253,.3)}.text-sky-200\/35{color:rgba(186,230,253,.35)}.text-sky-200\/40{color:rgba(186,230,253,.4)}.text-sky-200\/45{color:rgba(186,230,253,.45)}.text-sky-200\/5{color:rgba(186,230,253,.05)}.text-sky-200\/50{color:rgba(186,230,253,.5)}.text-sky-200\/55{color:rgba(186,230,253,.55)}.text-sky-200\/60{color:rgba(186,230,253,.6)}.text-sky-200\/65{color:rgba(186,230,253,.65)}.text-sky-200\/70{color:rgba(186,230,253,.7)}.text-sky-200\/75{color:rgba(186,230,253,.75)}.text-sky-200\/80{color:rgba(186,230,253,.8)}.text-sky-200\/85{color:rgba(186,230,253,.85)}.text-sky-200\/90{color:rgba(186,230,253,.9)}.text-sky-200\/95{color:rgba(186,230,253,.95)}.text-sky-50{--tw-text-opacity:1;color:rgb(240 249 255/var(--tw-text-opacity,1))}.text-sky-50\/0{color:rgba(240,249,255,0)}.text-sky-50\/10{color:rgba(240,249,255,.1)}.text-sky-50\/100{color:#f0f9ff}.text-sky-50\/15{color:rgba(240,249,255,.15)}.text-sky-50\/20{color:rgba(240,249,255,.2)}.text-sky-50\/25{color:rgba(240,249,255,.25)}.text-sky-50\/30{color:rgba(240,249,255,.3)}.text-sky-50\/35{color:rgba(240,249,255,.35)}.text-sky-50\/40{color:rgba(240,249,255,.4)}.text-sky-50\/45{color:rgba(240,249,255,.45)}.text-sky-50\/5{color:rgba(240,249,255,.05)}.text-sky-50\/50{color:rgba(240,249,255,.5)}.text-sky-50\/55{color:rgba(240,249,255,.55)}.text-sky-50\/60{color:rgba(240,249,255,.6)}.text-sky-50\/65{color:rgba(240,249,255,.65)}.text-sky-50\/70{color:rgba(240,249,255,.7)}.text-sky-50\/75{color:rgba(240,249,255,.75)}.text-sky-50\/80{color:rgba(240,249,255,.8)}.text-sky-50\/85{color:rgba(240,249,255,.85)}.text-sky-50\/90{color:rgba(240,249,255,.9)}.text-sky-50\/95{color:rgba(240,249,255,.95)}.text-sky-500{--tw-text-opacity:1;color:rgb(14 165 233/var(--tw-text-opacity,1))}.text-sky-500\/0{color:rgba(14,165,233,0)}.text-sky-500\/10{color:rgba(14,165,233,.1)}.text-sky-500\/100{color:#0ea5e9}.text-sky-500\/15{color:rgba(14,165,233,.15)}.text-sky-500\/20{color:rgba(14,165,233,.2)}.text-sky-500\/25{color:rgba(14,165,233,.25)}.text-sky-500\/30{color:rgba(14,165,233,.3)}.text-sky-500\/35{color:rgba(14,165,233,.35)}.text-sky-500\/40{color:rgba(14,165,233,.4)}.text-sky-500\/45{color:rgba(14,165,233,.45)}.text-sky-500\/5{color:rgba(14,165,233,.05)}.text-sky-500\/50{color:rgba(14,165,233,.5)}.text-sky-500\/55{color:rgba(14,165,233,.55)}.text-sky-500\/60{color:rgba(14,165,233,.6)}.text-sky-500\/65{color:rgba(14,165,233,.65)}.text-sky-500\/70{color:rgba(14,165,233,.7)}.text-sky-500\/75{color:rgba(14,165,233,.75)}.text-sky-500\/80{color:rgba(14,165,233,.8)}.text-sky-500\/85{color:rgba(14,165,233,.85)}.text-sky-500\/90{color:rgba(14,165,233,.9)}.text-sky-500\/95{color:rgba(14,165,233,.95)}.text-sky-950{--tw-text-opacity:1;color:rgb(8 47 73/var(--tw-text-opacity,1))}.text-sky-950\/0{color:rgba(8,47,73,0)}.text-sky-950\/10{color:rgba(8,47,73,.1)}.text-sky-950\/100{color:#082f49}.text-sky-950\/15{color:rgba(8,47,73,.15)}.text-sky-950\/20{color:rgba(8,47,73,.2)}.text-sky-950\/25{color:rgba(8,47,73,.25)}.text-sky-950\/30{color:rgba(8,47,73,.3)}.text-sky-950\/35{color:rgba(8,47,73,.35)}.text-sky-950\/40{color:rgba(8,47,73,.4)}.text-sky-950\/45{color:rgba(8,47,73,.45)}.text-sky-950\/5{color:rgba(8,47,73,.05)}.text-sky-950\/50{color:rgba(8,47,73,.5)}.text-sky-950\/55{color:rgba(8,47,73,.55)}.text-sky-950\/60{color:rgba(8,47,73,.6)}.text-sky-950\/65{color:rgba(8,47,73,.65)}.text-sky-950\/70{color:rgba(8,47,73,.7)}.text-sky-950\/75{color:rgba(8,47,73,.75)}.text-sky-950\/80{color:rgba(8,47,73,.8)}.text-sky-950\/85{color:rgba(8,47,73,.85)}.text-sky-950\/90{color:rgba(8,47,73,.9)}.text-sky-950\/95{color:rgba(8,47,73,.95)}.text-slate-100{--tw-text-opacity:1;color:rgb(241 245 249/var(--tw-text-opacity,1))}.text-slate-100\/0{color:rgba(241,245,249,0)}.text-slate-100\/10{color:rgba(241,245,249,.1)}.text-slate-100\/100{color:#f1f5f9}.text-slate-100\/15{color:rgba(241,245,249,.15)}.text-slate-100\/20{color:rgba(241,245,249,.2)}.text-slate-100\/25{color:rgba(241,245,249,.25)}.text-slate-100\/30{color:rgba(241,245,249,.3)}.text-slate-100\/35{color:rgba(241,245,249,.35)}.text-slate-100\/40{color:rgba(241,245,249,.4)}.text-slate-100\/45{color:rgba(241,245,249,.45)}.text-slate-100\/5{color:rgba(241,245,249,.05)}.text-slate-100\/50{color:rgba(241,245,249,.5)}.text-slate-100\/55{color:rgba(241,245,249,.55)}.text-slate-100\/60{color:rgba(241,245,249,.6)}.text-slate-100\/65{color:rgba(241,245,249,.65)}.text-slate-100\/70{color:rgba(241,245,249,.7)}.text-slate-100\/75{color:rgba(241,245,249,.75)}.text-slate-100\/80{color:rgba(241,245,249,.8)}.text-slate-100\/85{color:rgba(241,245,249,.85)}.text-slate-100\/90{color:rgba(241,245,249,.9)}.text-slate-100\/95{color:rgba(241,245,249,.95)}.text-slate-200{--tw-text-opacity:1;color:rgb(226 232 240/var(--tw-text-opacity,1))}.text-slate-200\/0{color:rgba(226,232,240,0)}.text-slate-200\/10{color:rgba(226,232,240,.1)}.text-slate-200\/100{color:#e2e8f0}.text-slate-200\/15{color:rgba(226,232,240,.15)}.text-slate-200\/20{color:rgba(226,232,240,.2)}.text-slate-200\/25{color:rgba(226,232,240,.25)}.text-slate-200\/30{color:rgba(226,232,240,.3)}.text-slate-200\/35{color:rgba(226,232,240,.35)}.text-slate-200\/40{color:rgba(226,232,240,.4)}.text-slate-200\/45{color:rgba(226,232,240,.45)}.text-slate-200\/5{color:rgba(226,232,240,.05)}.text-slate-200\/50{color:rgba(226,232,240,.5)}.text-slate-200\/55{color:rgba(226,232,240,.55)}.text-slate-200\/60{color:rgba(226,232,240,.6)}.text-slate-200\/65{color:rgba(226,232,240,.65)}.text-slate-200\/70{color:rgba(226,232,240,.7)}.text-slate-200\/75{color:rgba(226,232,240,.75)}.text-slate-200\/80{color:rgba(226,232,240,.8)}.text-slate-200\/85{color:rgba(226,232,240,.85)}.text-slate-200\/90{color:rgba(226,232,240,.9)}.text-slate-200\/95{color:rgba(226,232,240,.95)}.text-slate-50{--tw-text-opacity:1;color:rgb(248 250 252/var(--tw-text-opacity,1))}.text-slate-50\/0{color:rgba(248,250,252,0)}.text-slate-50\/10{color:rgba(248,250,252,.1)}.text-slate-50\/100{color:#f8fafc}.text-slate-50\/15{color:rgba(248,250,252,.15)}.text-slate-50\/20{color:rgba(248,250,252,.2)}.text-slate-50\/25{color:rgba(248,250,252,.25)}.text-slate-50\/30{color:rgba(248,250,252,.3)}.text-slate-50\/35{color:rgba(248,250,252,.35)}.text-slate-50\/40{color:rgba(248,250,252,.4)}.text-slate-50\/45{color:rgba(248,250,252,.45)}.text-slate-50\/5{color:rgba(248,250,252,.05)}.text-slate-50\/50{color:rgba(248,250,252,.5)}.text-slate-50\/55{color:rgba(248,250,252,.55)}.text-slate-50\/60{color:rgba(248,250,252,.6)}.text-slate-50\/65{color:rgba(248,250,252,.65)}.text-slate-50\/70{color:rgba(248,250,252,.7)}.text-slate-50\/75{color:rgba(248,250,252,.75)}.text-slate-50\/80{color:rgba(248,250,252,.8)}.text-slate-50\/85{color:rgba(248,250,252,.85)}.text-slate-50\/90{color:rgba(248,250,252,.9)}.text-slate-50\/95{color:rgba(248,250,252,.95)}.text-slate-500{--tw-text-opacity:1;color:rgb(100 116 139/var(--tw-text-opacity,1))}.text-slate-500\/0{color:rgba(100,116,139,0)}.text-slate-500\/10{color:rgba(100,116,139,.1)}.text-slate-500\/100{color:#64748b}.text-slate-500\/15{color:rgba(100,116,139,.15)}.text-slate-500\/20{color:rgba(100,116,139,.2)}.text-slate-500\/25{color:rgba(100,116,139,.25)}.text-slate-500\/30{color:rgba(100,116,139,.3)}.text-slate-500\/35{color:rgba(100,116,139,.35)}.text-slate-500\/40{color:rgba(100,116,139,.4)}.text-slate-500\/45{color:rgba(100,116,139,.45)}.text-slate-500\/5{color:rgba(100,116,139,.05)}.text-slate-500\/50{color:rgba(100,116,139,.5)}.text-slate-500\/55{color:rgba(100,116,139,.55)}.text-slate-500\/60{color:rgba(100,116,139,.6)}.text-slate-500\/65{color:rgba(100,116,139,.65)}.text-slate-500\/70{color:rgba(100,116,139,.7)}.text-slate-500\/75{color:rgba(100,116,139,.75)}.text-slate-500\/80{color:rgba(100,116,139,.8)}.text-slate-500\/85{color:rgba(100,116,139,.85)}.text-slate-500\/90{color:rgba(100,116,139,.9)}.text-slate-500\/95{color:rgba(100,116,139,.95)}.text-slate-950{--tw-text-opacity:1;color:rgb(2 6 23/var(--tw-text-opacity,1))}.text-slate-950\/0{color:rgba(2,6,23,0)}.text-slate-950\/10{color:rgba(2,6,23,.1)}.text-slate-950\/100{color:#020617}.text-slate-950\/15{color:rgba(2,6,23,.15)}.text-slate-950\/20{color:rgba(2,6,23,.2)}.text-slate-950\/25{color:rgba(2,6,23,.25)}.text-slate-950\/30{color:rgba(2,6,23,.3)}.text-slate-950\/35{color:rgba(2,6,23,.35)}.text-slate-950\/40{color:rgba(2,6,23,.4)}.text-slate-950\/45{color:rgba(2,6,23,.45)}.text-slate-950\/5{color:rgba(2,6,23,.05)}.text-slate-950\/50{color:rgba(2,6,23,.5)}.text-slate-950\/55{color:rgba(2,6,23,.55)}.text-slate-950\/60{color:rgba(2,6,23,.6)}.text-slate-950\/65{color:rgba(2,6,23,.65)}.text-slate-950\/70{color:rgba(2,6,23,.7)}.text-slate-950\/75{color:rgba(2,6,23,.75)}.text-slate-950\/80{color:rgba(2,6,23,.8)}.text-slate-950\/85{color:rgba(2,6,23,.85)}.text-slate-950\/90{color:rgba(2,6,23,.9)}.text-slate-950\/95{color:rgba(2,6,23,.95)}.text-stone-100{--tw-text-opacity:1;color:rgb(245 245 244/var(--tw-text-opacity,1))}.text-stone-100\/0{color:hsla(60,5%,96%,0)}.text-stone-100\/10{color:hsla(60,5%,96%,.1)}.text-stone-100\/100{color:#f5f5f4}.text-stone-100\/15{color:hsla(60,5%,96%,.15)}.text-stone-100\/20{color:hsla(60,5%,96%,.2)}.text-stone-100\/25{color:hsla(60,5%,96%,.25)}.text-stone-100\/30{color:hsla(60,5%,96%,.3)}.text-stone-100\/35{color:hsla(60,5%,96%,.35)}.text-stone-100\/40{color:hsla(60,5%,96%,.4)}.text-stone-100\/45{color:hsla(60,5%,96%,.45)}.text-stone-100\/5{color:hsla(60,5%,96%,.05)}.text-stone-100\/50{color:hsla(60,5%,96%,.5)}.text-stone-100\/55{color:hsla(60,5%,96%,.55)}.text-stone-100\/60{color:hsla(60,5%,96%,.6)}.text-stone-100\/65{color:hsla(60,5%,96%,.65)}.text-stone-100\/70{color:hsla(60,5%,96%,.7)}.text-stone-100\/75{color:hsla(60,5%,96%,.75)}.text-stone-100\/80{color:hsla(60,5%,96%,.8)}.text-stone-100\/85{color:hsla(60,5%,96%,.85)}.text-stone-100\/90{color:hsla(60,5%,96%,.9)}.text-stone-100\/95{color:hsla(60,5%,96%,.95)}.text-stone-200{--tw-text-opacity:1;color:rgb(231 229 228/var(--tw-text-opacity,1))}.text-stone-200\/0{color:hsla(20,6%,90%,0)}.text-stone-200\/10{color:hsla(20,6%,90%,.1)}.text-stone-200\/100{color:#e7e5e4}.text-stone-200\/15{color:hsla(20,6%,90%,.15)}.text-stone-200\/20{color:hsla(20,6%,90%,.2)}.text-stone-200\/25{color:hsla(20,6%,90%,.25)}.text-stone-200\/30{color:hsla(20,6%,90%,.3)}.text-stone-200\/35{color:hsla(20,6%,90%,.35)}.text-stone-200\/40{color:hsla(20,6%,90%,.4)}.text-stone-200\/45{color:hsla(20,6%,90%,.45)}.text-stone-200\/5{color:hsla(20,6%,90%,.05)}.text-stone-200\/50{color:hsla(20,6%,90%,.5)}.text-stone-200\/55{color:hsla(20,6%,90%,.55)}.text-stone-200\/60{color:hsla(20,6%,90%,.6)}.text-stone-200\/65{color:hsla(20,6%,90%,.65)}.text-stone-200\/70{color:hsla(20,6%,90%,.7)}.text-stone-200\/75{color:hsla(20,6%,90%,.75)}.text-stone-200\/80{color:hsla(20,6%,90%,.8)}.text-stone-200\/85{color:hsla(20,6%,90%,.85)}.text-stone-200\/90{color:hsla(20,6%,90%,.9)}.text-stone-200\/95{color:hsla(20,6%,90%,.95)}.text-stone-50{--tw-text-opacity:1;color:rgb(250 250 249/var(--tw-text-opacity,1))}.text-stone-50\/0{color:hsla(60,9%,98%,0)}.text-stone-50\/10{color:hsla(60,9%,98%,.1)}.text-stone-50\/100{color:#fafaf9}.text-stone-50\/15{color:hsla(60,9%,98%,.15)}.text-stone-50\/20{color:hsla(60,9%,98%,.2)}.text-stone-50\/25{color:hsla(60,9%,98%,.25)}.text-stone-50\/30{color:hsla(60,9%,98%,.3)}.text-stone-50\/35{color:hsla(60,9%,98%,.35)}.text-stone-50\/40{color:hsla(60,9%,98%,.4)}.text-stone-50\/45{color:hsla(60,9%,98%,.45)}.text-stone-50\/5{color:hsla(60,9%,98%,.05)}.text-stone-50\/50{color:hsla(60,9%,98%,.5)}.text-stone-50\/55{color:hsla(60,9%,98%,.55)}.text-stone-50\/60{color:hsla(60,9%,98%,.6)}.text-stone-50\/65{color:hsla(60,9%,98%,.65)}.text-stone-50\/70{color:hsla(60,9%,98%,.7)}.text-stone-50\/75{color:hsla(60,9%,98%,.75)}.text-stone-50\/80{color:hsla(60,9%,98%,.8)}.text-stone-50\/85{color:hsla(60,9%,98%,.85)}.text-stone-50\/90{color:hsla(60,9%,98%,.9)}.text-stone-50\/95{color:hsla(60,9%,98%,.95)}.text-stone-500{--tw-text-opacity:1;color:rgb(120 113 108/var(--tw-text-opacity,1))}.text-stone-500\/0{color:hsla(25,5%,45%,0)}.text-stone-500\/10{color:hsla(25,5%,45%,.1)}.text-stone-500\/100{color:#78716c}.text-stone-500\/15{color:hsla(25,5%,45%,.15)}.text-stone-500\/20{color:hsla(25,5%,45%,.2)}.text-stone-500\/25{color:hsla(25,5%,45%,.25)}.text-stone-500\/30{color:hsla(25,5%,45%,.3)}.text-stone-500\/35{color:hsla(25,5%,45%,.35)}.text-stone-500\/40{color:hsla(25,5%,45%,.4)}.text-stone-500\/45{color:hsla(25,5%,45%,.45)}.text-stone-500\/5{color:hsla(25,5%,45%,.05)}.text-stone-500\/50{color:hsla(25,5%,45%,.5)}.text-stone-500\/55{color:hsla(25,5%,45%,.55)}.text-stone-500\/60{color:hsla(25,5%,45%,.6)}.text-stone-500\/65{color:hsla(25,5%,45%,.65)}.text-stone-500\/70{color:hsla(25,5%,45%,.7)}.text-stone-500\/75{color:hsla(25,5%,45%,.75)}.text-stone-500\/80{color:hsla(25,5%,45%,.8)}.text-stone-500\/85{color:hsla(25,5%,45%,.85)}.text-stone-500\/90{color:hsla(25,5%,45%,.9)}.text-stone-500\/95{color:hsla(25,5%,45%,.95)}.text-stone-950{--tw-text-opacity:1;color:rgb(12 10 9/var(--tw-text-opacity,1))}.text-stone-950\/0{color:rgba(12,10,9,0)}.text-stone-950\/10{color:rgba(12,10,9,.1)}.text-stone-950\/100{color:#0c0a09}.text-stone-950\/15{color:rgba(12,10,9,.15)}.text-stone-950\/20{color:rgba(12,10,9,.2)}.text-stone-950\/25{color:rgba(12,10,9,.25)}.text-stone-950\/30{color:rgba(12,10,9,.3)}.text-stone-950\/35{color:rgba(12,10,9,.35)}.text-stone-950\/40{color:rgba(12,10,9,.4)}.text-stone-950\/45{color:rgba(12,10,9,.45)}.text-stone-950\/5{color:rgba(12,10,9,.05)}.text-stone-950\/50{color:rgba(12,10,9,.5)}.text-stone-950\/55{color:rgba(12,10,9,.55)}.text-stone-950\/60{color:rgba(12,10,9,.6)}.text-stone-950\/65{color:rgba(12,10,9,.65)}.text-stone-950\/70{color:rgba(12,10,9,.7)}.text-stone-950\/75{color:rgba(12,10,9,.75)}.text-stone-950\/80{color:rgba(12,10,9,.8)}.text-stone-950\/85{color:rgba(12,10,9,.85)}.text-stone-950\/90{color:rgba(12,10,9,.9)}.text-stone-950\/95{color:rgba(12,10,9,.95)}.text-teal-100{--tw-text-opacity:1;color:rgb(204 251 241/var(--tw-text-opacity,1))}.text-teal-100\/0{color:rgba(204,251,241,0)}.text-teal-100\/10{color:rgba(204,251,241,.1)}.text-teal-100\/100{color:#ccfbf1}.text-teal-100\/15{color:rgba(204,251,241,.15)}.text-teal-100\/20{color:rgba(204,251,241,.2)}.text-teal-100\/25{color:rgba(204,251,241,.25)}.text-teal-100\/30{color:rgba(204,251,241,.3)}.text-teal-100\/35{color:rgba(204,251,241,.35)}.text-teal-100\/40{color:rgba(204,251,241,.4)}.text-teal-100\/45{color:rgba(204,251,241,.45)}.text-teal-100\/5{color:rgba(204,251,241,.05)}.text-teal-100\/50{color:rgba(204,251,241,.5)}.text-teal-100\/55{color:rgba(204,251,241,.55)}.text-teal-100\/60{color:rgba(204,251,241,.6)}.text-teal-100\/65{color:rgba(204,251,241,.65)}.text-teal-100\/70{color:rgba(204,251,241,.7)}.text-teal-100\/75{color:rgba(204,251,241,.75)}.text-teal-100\/80{color:rgba(204,251,241,.8)}.text-teal-100\/85{color:rgba(204,251,241,.85)}.text-teal-100\/90{color:rgba(204,251,241,.9)}.text-teal-100\/95{color:rgba(204,251,241,.95)}.text-teal-200{--tw-text-opacity:1;color:rgb(153 246 228/var(--tw-text-opacity,1))}.text-teal-200\/0{color:rgba(153,246,228,0)}.text-teal-200\/10{color:rgba(153,246,228,.1)}.text-teal-200\/100{color:#99f6e4}.text-teal-200\/15{color:rgba(153,246,228,.15)}.text-teal-200\/20{color:rgba(153,246,228,.2)}.text-teal-200\/25{color:rgba(153,246,228,.25)}.text-teal-200\/30{color:rgba(153,246,228,.3)}.text-teal-200\/35{color:rgba(153,246,228,.35)}.text-teal-200\/40{color:rgba(153,246,228,.4)}.text-teal-200\/45{color:rgba(153,246,228,.45)}.text-teal-200\/5{color:rgba(153,246,228,.05)}.text-teal-200\/50{color:rgba(153,246,228,.5)}.text-teal-200\/55{color:rgba(153,246,228,.55)}.text-teal-200\/60{color:rgba(153,246,228,.6)}.text-teal-200\/65{color:rgba(153,246,228,.65)}.text-teal-200\/70{color:rgba(153,246,228,.7)}.text-teal-200\/75{color:rgba(153,246,228,.75)}.text-teal-200\/80{color:rgba(153,246,228,.8)}.text-teal-200\/85{color:rgba(153,246,228,.85)}.text-teal-200\/90{color:rgba(153,246,228,.9)}.text-teal-200\/95{color:rgba(153,246,228,.95)}.text-teal-50{--tw-text-opacity:1;color:rgb(240 253 250/var(--tw-text-opacity,1))}.text-teal-50\/0{color:rgba(240,253,250,0)}.text-teal-50\/10{color:rgba(240,253,250,.1)}.text-teal-50\/100{color:#f0fdfa}.text-teal-50\/15{color:rgba(240,253,250,.15)}.text-teal-50\/20{color:rgba(240,253,250,.2)}.text-teal-50\/25{color:rgba(240,253,250,.25)}.text-teal-50\/30{color:rgba(240,253,250,.3)}.text-teal-50\/35{color:rgba(240,253,250,.35)}.text-teal-50\/40{color:rgba(240,253,250,.4)}.text-teal-50\/45{color:rgba(240,253,250,.45)}.text-teal-50\/5{color:rgba(240,253,250,.05)}.text-teal-50\/50{color:rgba(240,253,250,.5)}.text-teal-50\/55{color:rgba(240,253,250,.55)}.text-teal-50\/60{color:rgba(240,253,250,.6)}.text-teal-50\/65{color:rgba(240,253,250,.65)}.text-teal-50\/70{color:rgba(240,253,250,.7)}.text-teal-50\/75{color:rgba(240,253,250,.75)}.text-teal-50\/80{color:rgba(240,253,250,.8)}.text-teal-50\/85{color:rgba(240,253,250,.85)}.text-teal-50\/90{color:rgba(240,253,250,.9)}.text-teal-50\/95{color:rgba(240,253,250,.95)}.text-teal-500{--tw-text-opacity:1;color:rgb(20 184 166/var(--tw-text-opacity,1))}.text-teal-500\/0{color:rgba(20,184,166,0)}.text-teal-500\/10{color:rgba(20,184,166,.1)}.text-teal-500\/100{color:#14b8a6}.text-teal-500\/15{color:rgba(20,184,166,.15)}.text-teal-500\/20{color:rgba(20,184,166,.2)}.text-teal-500\/25{color:rgba(20,184,166,.25)}.text-teal-500\/30{color:rgba(20,184,166,.3)}.text-teal-500\/35{color:rgba(20,184,166,.35)}.text-teal-500\/40{color:rgba(20,184,166,.4)}.text-teal-500\/45{color:rgba(20,184,166,.45)}.text-teal-500\/5{color:rgba(20,184,166,.05)}.text-teal-500\/50{color:rgba(20,184,166,.5)}.text-teal-500\/55{color:rgba(20,184,166,.55)}.text-teal-500\/60{color:rgba(20,184,166,.6)}.text-teal-500\/65{color:rgba(20,184,166,.65)}.text-teal-500\/70{color:rgba(20,184,166,.7)}.text-teal-500\/75{color:rgba(20,184,166,.75)}.text-teal-500\/80{color:rgba(20,184,166,.8)}.text-teal-500\/85{color:rgba(20,184,166,.85)}.text-teal-500\/90{color:rgba(20,184,166,.9)}.text-teal-500\/95{color:rgba(20,184,166,.95)}.text-teal-950{--tw-text-opacity:1;color:rgb(4 47 46/var(--tw-text-opacity,1))}.text-teal-950\/0{color:rgba(4,47,46,0)}.text-teal-950\/10{color:rgba(4,47,46,.1)}.text-teal-950\/100{color:#042f2e}.text-teal-950\/15{color:rgba(4,47,46,.15)}.text-teal-950\/20{color:rgba(4,47,46,.2)}.text-teal-950\/25{color:rgba(4,47,46,.25)}.text-teal-950\/30{color:rgba(4,47,46,.3)}.text-teal-950\/35{color:rgba(4,47,46,.35)}.text-teal-950\/40{color:rgba(4,47,46,.4)}.text-teal-950\/45{color:rgba(4,47,46,.45)}.text-teal-950\/5{color:rgba(4,47,46,.05)}.text-teal-950\/50{color:rgba(4,47,46,.5)}.text-teal-950\/55{color:rgba(4,47,46,.55)}.text-teal-950\/60{color:rgba(4,47,46,.6)}.text-teal-950\/65{color:rgba(4,47,46,.65)}.text-teal-950\/70{color:rgba(4,47,46,.7)}.text-teal-950\/75{color:rgba(4,47,46,.75)}.text-teal-950\/80{color:rgba(4,47,46,.8)}.text-teal-950\/85{color:rgba(4,47,46,.85)}.text-teal-950\/90{color:rgba(4,47,46,.9)}.text-teal-950\/95{color:rgba(4,47,46,.95)}.text-violet-100{--tw-text-opacity:1;color:rgb(237 233 254/var(--tw-text-opacity,1))}.text-violet-100\/0{color:rgba(237,233,254,0)}.text-violet-100\/10{color:rgba(237,233,254,.1)}.text-violet-100\/100{color:#ede9fe}.text-violet-100\/15{color:rgba(237,233,254,.15)}.text-violet-100\/20{color:rgba(237,233,254,.2)}.text-violet-100\/25{color:rgba(237,233,254,.25)}.text-violet-100\/30{color:rgba(237,233,254,.3)}.text-violet-100\/35{color:rgba(237,233,254,.35)}.text-violet-100\/40{color:rgba(237,233,254,.4)}.text-violet-100\/45{color:rgba(237,233,254,.45)}.text-violet-100\/5{color:rgba(237,233,254,.05)}.text-violet-100\/50{color:rgba(237,233,254,.5)}.text-violet-100\/55{color:rgba(237,233,254,.55)}.text-violet-100\/60{color:rgba(237,233,254,.6)}.text-violet-100\/65{color:rgba(237,233,254,.65)}.text-violet-100\/70{color:rgba(237,233,254,.7)}.text-violet-100\/75{color:rgba(237,233,254,.75)}.text-violet-100\/80{color:rgba(237,233,254,.8)}.text-violet-100\/85{color:rgba(237,233,254,.85)}.text-violet-100\/90{color:rgba(237,233,254,.9)}.text-violet-100\/95{color:rgba(237,233,254,.95)}.text-violet-200{--tw-text-opacity:1;color:rgb(221 214 254/var(--tw-text-opacity,1))}.text-violet-200\/0{color:rgba(221,214,254,0)}.text-violet-200\/10{color:rgba(221,214,254,.1)}.text-violet-200\/100{color:#ddd6fe}.text-violet-200\/15{color:rgba(221,214,254,.15)}.text-violet-200\/20{color:rgba(221,214,254,.2)}.text-violet-200\/25{color:rgba(221,214,254,.25)}.text-violet-200\/30{color:rgba(221,214,254,.3)}.text-violet-200\/35{color:rgba(221,214,254,.35)}.text-violet-200\/40{color:rgba(221,214,254,.4)}.text-violet-200\/45{color:rgba(221,214,254,.45)}.text-violet-200\/5{color:rgba(221,214,254,.05)}.text-violet-200\/50{color:rgba(221,214,254,.5)}.text-violet-200\/55{color:rgba(221,214,254,.55)}.text-violet-200\/60{color:rgba(221,214,254,.6)}.text-violet-200\/65{color:rgba(221,214,254,.65)}.text-violet-200\/70{color:rgba(221,214,254,.7)}.text-violet-200\/75{color:rgba(221,214,254,.75)}.text-violet-200\/80{color:rgba(221,214,254,.8)}.text-violet-200\/85{color:rgba(221,214,254,.85)}.text-violet-200\/90{color:rgba(221,214,254,.9)}.text-violet-200\/95{color:rgba(221,214,254,.95)}.text-violet-50{--tw-text-opacity:1;color:rgb(245 243 255/var(--tw-text-opacity,1))}.text-violet-50\/0{color:rgba(245,243,255,0)}.text-violet-50\/10{color:rgba(245,243,255,.1)}.text-violet-50\/100{color:#f5f3ff}.text-violet-50\/15{color:rgba(245,243,255,.15)}.text-violet-50\/20{color:rgba(245,243,255,.2)}.text-violet-50\/25{color:rgba(245,243,255,.25)}.text-violet-50\/30{color:rgba(245,243,255,.3)}.text-violet-50\/35{color:rgba(245,243,255,.35)}.text-violet-50\/40{color:rgba(245,243,255,.4)}.text-violet-50\/45{color:rgba(245,243,255,.45)}.text-violet-50\/5{color:rgba(245,243,255,.05)}.text-violet-50\/50{color:rgba(245,243,255,.5)}.text-violet-50\/55{color:rgba(245,243,255,.55)}.text-violet-50\/60{color:rgba(245,243,255,.6)}.text-violet-50\/65{color:rgba(245,243,255,.65)}.text-violet-50\/70{color:rgba(245,243,255,.7)}.text-violet-50\/75{color:rgba(245,243,255,.75)}.text-violet-50\/80{color:rgba(245,243,255,.8)}.text-violet-50\/85{color:rgba(245,243,255,.85)}.text-violet-50\/90{color:rgba(245,243,255,.9)}.text-violet-50\/95{color:rgba(245,243,255,.95)}.text-violet-500{--tw-text-opacity:1;color:rgb(139 92 246/var(--tw-text-opacity,1))}.text-violet-500\/0{color:rgba(139,92,246,0)}.text-violet-500\/10{color:rgba(139,92,246,.1)}.text-violet-500\/100{color:#8b5cf6}.text-violet-500\/15{color:rgba(139,92,246,.15)}.text-violet-500\/20{color:rgba(139,92,246,.2)}.text-violet-500\/25{color:rgba(139,92,246,.25)}.text-violet-500\/30{color:rgba(139,92,246,.3)}.text-violet-500\/35{color:rgba(139,92,246,.35)}.text-violet-500\/40{color:rgba(139,92,246,.4)}.text-violet-500\/45{color:rgba(139,92,246,.45)}.text-violet-500\/5{color:rgba(139,92,246,.05)}.text-violet-500\/50{color:rgba(139,92,246,.5)}.text-violet-500\/55{color:rgba(139,92,246,.55)}.text-violet-500\/60{color:rgba(139,92,246,.6)}.text-violet-500\/65{color:rgba(139,92,246,.65)}.text-violet-500\/70{color:rgba(139,92,246,.7)}.text-violet-500\/75{color:rgba(139,92,246,.75)}.text-violet-500\/80{color:rgba(139,92,246,.8)}.text-violet-500\/85{color:rgba(139,92,246,.85)}.text-violet-500\/90{color:rgba(139,92,246,.9)}.text-violet-500\/95{color:rgba(139,92,246,.95)}.text-violet-950{--tw-text-opacity:1;color:rgb(46 16 101/var(--tw-text-opacity,1))}.text-violet-950\/0{color:rgba(46,16,101,0)}.text-violet-950\/10{color:rgba(46,16,101,.1)}.text-violet-950\/100{color:#2e1065}.text-violet-950\/15{color:rgba(46,16,101,.15)}.text-violet-950\/20{color:rgba(46,16,101,.2)}.text-violet-950\/25{color:rgba(46,16,101,.25)}.text-violet-950\/30{color:rgba(46,16,101,.3)}.text-violet-950\/35{color:rgba(46,16,101,.35)}.text-violet-950\/40{color:rgba(46,16,101,.4)}.text-violet-950\/45{color:rgba(46,16,101,.45)}.text-violet-950\/5{color:rgba(46,16,101,.05)}.text-violet-950\/50{color:rgba(46,16,101,.5)}.text-violet-950\/55{color:rgba(46,16,101,.55)}.text-violet-950\/60{color:rgba(46,16,101,.6)}.text-violet-950\/65{color:rgba(46,16,101,.65)}.text-violet-950\/70{color:rgba(46,16,101,.7)}.text-violet-950\/75{color:rgba(46,16,101,.75)}.text-violet-950\/80{color:rgba(46,16,101,.8)}.text-violet-950\/85{color:rgba(46,16,101,.85)}.text-violet-950\/90{color:rgba(46,16,101,.9)}.text-violet-950\/95{color:rgba(46,16,101,.95)}.text-white{--tw-text-opacity:1;color:rgb(255 255 255/var(--tw-text-opacity,1))}.text-yellow-100{--tw-text-opacity:1;color:rgb(254 249 195/var(--tw-text-opacity,1))}.text-yellow-100\/0{color:hsla(55,97%,88%,0)}.text-yellow-100\/10{color:hsla(55,97%,88%,.1)}.text-yellow-100\/100{color:#fef9c3}.text-yellow-100\/15{color:hsla(55,97%,88%,.15)}.text-yellow-100\/20{color:hsla(55,97%,88%,.2)}.text-yellow-100\/25{color:hsla(55,97%,88%,.25)}.text-yellow-100\/30{color:hsla(55,97%,88%,.3)}.text-yellow-100\/35{color:hsla(55,97%,88%,.35)}.text-yellow-100\/40{color:hsla(55,97%,88%,.4)}.text-yellow-100\/45{color:hsla(55,97%,88%,.45)}.text-yellow-100\/5{color:hsla(55,97%,88%,.05)}.text-yellow-100\/50{color:hsla(55,97%,88%,.5)}.text-yellow-100\/55{color:hsla(55,97%,88%,.55)}.text-yellow-100\/60{color:hsla(55,97%,88%,.6)}.text-yellow-100\/65{color:hsla(55,97%,88%,.65)}.text-yellow-100\/70{color:hsla(55,97%,88%,.7)}.text-yellow-100\/75{color:hsla(55,97%,88%,.75)}.text-yellow-100\/80{color:hsla(55,97%,88%,.8)}.text-yellow-100\/85{color:hsla(55,97%,88%,.85)}.text-yellow-100\/90{color:hsla(55,97%,88%,.9)}.text-yellow-100\/95{color:hsla(55,97%,88%,.95)}.text-yellow-200{--tw-text-opacity:1;color:rgb(254 240 138/var(--tw-text-opacity,1))}.text-yellow-200\/0{color:hsla(53,98%,77%,0)}.text-yellow-200\/10{color:hsla(53,98%,77%,.1)}.text-yellow-200\/100{color:#fef08a}.text-yellow-200\/15{color:hsla(53,98%,77%,.15)}.text-yellow-200\/20{color:hsla(53,98%,77%,.2)}.text-yellow-200\/25{color:hsla(53,98%,77%,.25)}.text-yellow-200\/30{color:hsla(53,98%,77%,.3)}.text-yellow-200\/35{color:hsla(53,98%,77%,.35)}.text-yellow-200\/40{color:hsla(53,98%,77%,.4)}.text-yellow-200\/45{color:hsla(53,98%,77%,.45)}.text-yellow-200\/5{color:hsla(53,98%,77%,.05)}.text-yellow-200\/50{color:hsla(53,98%,77%,.5)}.text-yellow-200\/55{color:hsla(53,98%,77%,.55)}.text-yellow-200\/60{color:hsla(53,98%,77%,.6)}.text-yellow-200\/65{color:hsla(53,98%,77%,.65)}.text-yellow-200\/70{color:hsla(53,98%,77%,.7)}.text-yellow-200\/75{color:hsla(53,98%,77%,.75)}.text-yellow-200\/80{color:hsla(53,98%,77%,.8)}.text-yellow-200\/85{color:hsla(53,98%,77%,.85)}.text-yellow-200\/90{color:hsla(53,98%,77%,.9)}.text-yellow-200\/95{color:hsla(53,98%,77%,.95)}.text-yellow-400{--tw-text-opacity:1;color:rgb(250 204 21/var(--tw-text-opacity,1))}.text-yellow-50{--tw-text-opacity:1;color:rgb(254 252 232/var(--tw-text-opacity,1))}.text-yellow-50\/0{color:hsla(55,92%,95%,0)}.text-yellow-50\/10{color:hsla(55,92%,95%,.1)}.text-yellow-50\/100{color:#fefce8}.text-yellow-50\/15{color:hsla(55,92%,95%,.15)}.text-yellow-50\/20{color:hsla(55,92%,95%,.2)}.text-yellow-50\/25{color:hsla(55,92%,95%,.25)}.text-yellow-50\/30{color:hsla(55,92%,95%,.3)}.text-yellow-50\/35{color:hsla(55,92%,95%,.35)}.text-yellow-50\/40{color:hsla(55,92%,95%,.4)}.text-yellow-50\/45{color:hsla(55,92%,95%,.45)}.text-yellow-50\/5{color:hsla(55,92%,95%,.05)}.text-yellow-50\/50{color:hsla(55,92%,95%,.5)}.text-yellow-50\/55{color:hsla(55,92%,95%,.55)}.text-yellow-50\/60{color:hsla(55,92%,95%,.6)}.text-yellow-50\/65{color:hsla(55,92%,95%,.65)}.text-yellow-50\/70{color:hsla(55,92%,95%,.7)}.text-yellow-50\/75{color:hsla(55,92%,95%,.75)}.text-yellow-50\/80{color:hsla(55,92%,95%,.8)}.text-yellow-50\/85{color:hsla(55,92%,95%,.85)}.text-yellow-50\/90{color:hsla(55,92%,95%,.9)}.text-yellow-50\/95{color:hsla(55,92%,95%,.95)}.text-yellow-500{--tw-text-opacity:1;color:rgb(234 179 8/var(--tw-text-opacity,1))}.text-yellow-500\/0{color:rgba(234,179,8,0)}.text-yellow-500\/10{color:rgba(234,179,8,.1)}.text-yellow-500\/100{color:#eab308}.text-yellow-500\/15{color:rgba(234,179,8,.15)}.text-yellow-500\/20{color:rgba(234,179,8,.2)}.text-yellow-500\/25{color:rgba(234,179,8,.25)}.text-yellow-500\/30{color:rgba(234,179,8,.3)}.text-yellow-500\/35{color:rgba(234,179,8,.35)}.text-yellow-500\/40{color:rgba(234,179,8,.4)}.text-yellow-500\/45{color:rgba(234,179,8,.45)}.text-yellow-500\/5{color:rgba(234,179,8,.05)}.text-yellow-500\/50{color:rgba(234,179,8,.5)}.text-yellow-500\/55{color:rgba(234,179,8,.55)}.text-yellow-500\/60{color:rgba(234,179,8,.6)}.text-yellow-500\/65{color:rgba(234,179,8,.65)}.text-yellow-500\/70{color:rgba(234,179,8,.7)}.text-yellow-500\/75{color:rgba(234,179,8,.75)}.text-yellow-500\/80{color:rgba(234,179,8,.8)}.text-yellow-500\/85{color:rgba(234,179,8,.85)}.text-yellow-500\/90{color:rgba(234,179,8,.9)}.text-yellow-500\/95{color:rgba(234,179,8,.95)}.text-yellow-950{--tw-text-opacity:1;color:rgb(66 32 6/var(--tw-text-opacity,1))}.text-yellow-950\/0{color:rgba(66,32,6,0)}.text-yellow-950\/10{color:rgba(66,32,6,.1)}.text-yellow-950\/100{color:#422006}.text-yellow-950\/15{color:rgba(66,32,6,.15)}.text-yellow-950\/20{color:rgba(66,32,6,.2)}.text-yellow-950\/25{color:rgba(66,32,6,.25)}.text-yellow-950\/30{color:rgba(66,32,6,.3)}.text-yellow-950\/35{color:rgba(66,32,6,.35)}.text-yellow-950\/40{color:rgba(66,32,6,.4)}.text-yellow-950\/45{color:rgba(66,32,6,.45)}.text-yellow-950\/5{color:rgba(66,32,6,.05)}.text-yellow-950\/50{color:rgba(66,32,6,.5)}.text-yellow-950\/55{color:rgba(66,32,6,.55)}.text-yellow-950\/60{color:rgba(66,32,6,.6)}.text-yellow-950\/65{color:rgba(66,32,6,.65)}.text-yellow-950\/70{color:rgba(66,32,6,.7)}.text-yellow-950\/75{color:rgba(66,32,6,.75)}.text-yellow-950\/80{color:rgba(66,32,6,.8)}.text-yellow-950\/85{color:rgba(66,32,6,.85)}.text-yellow-950\/90{color:rgba(66,32,6,.9)}.text-yellow-950\/95{color:rgba(66,32,6,.95)}.text-zinc-100{--tw-text-opacity:1;color:rgb(244 244 245/var(--tw-text-opacity,1))}.text-zinc-100\/0{color:hsla(240,5%,96%,0)}.text-zinc-100\/10{color:hsla(240,5%,96%,.1)}.text-zinc-100\/100{color:#f4f4f5}.text-zinc-100\/15{color:hsla(240,5%,96%,.15)}.text-zinc-100\/20{color:hsla(240,5%,96%,.2)}.text-zinc-100\/25{color:hsla(240,5%,96%,.25)}.text-zinc-100\/30{color:hsla(240,5%,96%,.3)}.text-zinc-100\/35{color:hsla(240,5%,96%,.35)}.text-zinc-100\/40{color:hsla(240,5%,96%,.4)}.text-zinc-100\/45{color:hsla(240,5%,96%,.45)}.text-zinc-100\/5{color:hsla(240,5%,96%,.05)}.text-zinc-100\/50{color:hsla(240,5%,96%,.5)}.text-zinc-100\/55{color:hsla(240,5%,96%,.55)}.text-zinc-100\/60{color:hsla(240,5%,96%,.6)}.text-zinc-100\/65{color:hsla(240,5%,96%,.65)}.text-zinc-100\/70{color:hsla(240,5%,96%,.7)}.text-zinc-100\/75{color:hsla(240,5%,96%,.75)}.text-zinc-100\/80{color:hsla(240,5%,96%,.8)}.text-zinc-100\/85{color:hsla(240,5%,96%,.85)}.text-zinc-100\/90{color:hsla(240,5%,96%,.9)}.text-zinc-100\/95{color:hsla(240,5%,96%,.95)}.text-zinc-200{--tw-text-opacity:1;color:rgb(228 228 231/var(--tw-text-opacity,1))}.text-zinc-200\/0{color:hsla(240,6%,90%,0)}.text-zinc-200\/10{color:hsla(240,6%,90%,.1)}.text-zinc-200\/100{color:#e4e4e7}.text-zinc-200\/15{color:hsla(240,6%,90%,.15)}.text-zinc-200\/20{color:hsla(240,6%,90%,.2)}.text-zinc-200\/25{color:hsla(240,6%,90%,.25)}.text-zinc-200\/30{color:hsla(240,6%,90%,.3)}.text-zinc-200\/35{color:hsla(240,6%,90%,.35)}.text-zinc-200\/40{color:hsla(240,6%,90%,.4)}.text-zinc-200\/45{color:hsla(240,6%,90%,.45)}.text-zinc-200\/5{color:hsla(240,6%,90%,.05)}.text-zinc-200\/50{color:hsla(240,6%,90%,.5)}.text-zinc-200\/55{color:hsla(240,6%,90%,.55)}.text-zinc-200\/60{color:hsla(240,6%,90%,.6)}.text-zinc-200\/65{color:hsla(240,6%,90%,.65)}.text-zinc-200\/70{color:hsla(240,6%,90%,.7)}.text-zinc-200\/75{color:hsla(240,6%,90%,.75)}.text-zinc-200\/80{color:hsla(240,6%,90%,.8)}.text-zinc-200\/85{color:hsla(240,6%,90%,.85)}.text-zinc-200\/90{color:hsla(240,6%,90%,.9)}.text-zinc-200\/95{color:hsla(240,6%,90%,.95)}.text-zinc-50{--tw-text-opacity:1;color:rgb(250 250 250/var(--tw-text-opacity,1))}.text-zinc-50\/0{color:hsla(0,0%,98%,0)}.text-zinc-50\/10{color:hsla(0,0%,98%,.1)}.text-zinc-50\/100{color:#fafafa}.text-zinc-50\/15{color:hsla(0,0%,98%,.15)}.text-zinc-50\/20{color:hsla(0,0%,98%,.2)}.text-zinc-50\/25{color:hsla(0,0%,98%,.25)}.text-zinc-50\/30{color:hsla(0,0%,98%,.3)}.text-zinc-50\/35{color:hsla(0,0%,98%,.35)}.text-zinc-50\/40{color:hsla(0,0%,98%,.4)}.text-zinc-50\/45{color:hsla(0,0%,98%,.45)}.text-zinc-50\/5{color:hsla(0,0%,98%,.05)}.text-zinc-50\/50{color:hsla(0,0%,98%,.5)}.text-zinc-50\/55{color:hsla(0,0%,98%,.55)}.text-zinc-50\/60{color:hsla(0,0%,98%,.6)}.text-zinc-50\/65{color:hsla(0,0%,98%,.65)}.text-zinc-50\/70{color:hsla(0,0%,98%,.7)}.text-zinc-50\/75{color:hsla(0,0%,98%,.75)}.text-zinc-50\/80{color:hsla(0,0%,98%,.8)}.text-zinc-50\/85{color:hsla(0,0%,98%,.85)}.text-zinc-50\/90{color:hsla(0,0%,98%,.9)}.text-zinc-50\/95{color:hsla(0,0%,98%,.95)}.text-zinc-500{--tw-text-opacity:1;color:rgb(113 113 122/var(--tw-text-opacity,1))}.text-zinc-500\/0{color:hsla(240,4%,46%,0)}.text-zinc-500\/10{color:hsla(240,4%,46%,.1)}.text-zinc-500\/100{color:#71717a}.text-zinc-500\/15{color:hsla(240,4%,46%,.15)}.text-zinc-500\/20{color:hsla(240,4%,46%,.2)}.text-zinc-500\/25{color:hsla(240,4%,46%,.25)}.text-zinc-500\/30{color:hsla(240,4%,46%,.3)}.text-zinc-500\/35{color:hsla(240,4%,46%,.35)}.text-zinc-500\/40{color:hsla(240,4%,46%,.4)}.text-zinc-500\/45{color:hsla(240,4%,46%,.45)}.text-zinc-500\/5{color:hsla(240,4%,46%,.05)}.text-zinc-500\/50{color:hsla(240,4%,46%,.5)}.text-zinc-500\/55{color:hsla(240,4%,46%,.55)}.text-zinc-500\/60{color:hsla(240,4%,46%,.6)}.text-zinc-500\/65{color:hsla(240,4%,46%,.65)}.text-zinc-500\/70{color:hsla(240,4%,46%,.7)}.text-zinc-500\/75{color:hsla(240,4%,46%,.75)}.text-zinc-500\/80{color:hsla(240,4%,46%,.8)}.text-zinc-500\/85{color:hsla(240,4%,46%,.85)}.text-zinc-500\/90{color:hsla(240,4%,46%,.9)}.text-zinc-500\/95{color:hsla(240,4%,46%,.95)}.text-zinc-950{--tw-text-opacity:1;color:rgb(9 9 11/var(--tw-text-opacity,1))}.text-zinc-950\/0{color:rgba(9,9,11,0)}.text-zinc-950\/10{color:rgba(9,9,11,.1)}.text-zinc-950\/100{color:#09090b}.text-zinc-950\/15{color:rgba(9,9,11,.15)}.text-zinc-950\/20{color:rgba(9,9,11,.2)}.text-zinc-950\/25{color:rgba(9,9,11,.25)}.text-zinc-950\/30{color:rgba(9,9,11,.3)}.text-zinc-950\/35{color:rgba(9,9,11,.35)}.text-zinc-950\/40{color:rgba(9,9,11,.4)}.text-zinc-950\/45{color:rgba(9,9,11,.45)}.text-zinc-950\/5{color:rgba(9,9,11,.05)}.text-zinc-950\/50{color:rgba(9,9,11,.5)}.text-zinc-950\/55{color:rgba(9,9,11,.55)}.text-zinc-950\/60{color:rgba(9,9,11,.6)}.text-zinc-950\/65{color:rgba(9,9,11,.65)}.text-zinc-950\/70{color:rgba(9,9,11,.7)}.text-zinc-950\/75{color:rgba(9,9,11,.75)}.text-zinc-950\/80{color:rgba(9,9,11,.8)}.text-zinc-950\/85{color:rgba(9,9,11,.85)}.text-zinc-950\/90{color:rgba(9,9,11,.9)}.text-zinc-950\/95{color:rgba(9,9,11,.95)}.underline{text-decoration-line:underline}.\!no-underline{text-decoration-line:none!important}.no-underline{text-decoration-line:none}.underline-offset-4{text-underline-offset:4px}.caret-amber-100{caret-color:#fef3c7}.caret-amber-100\/0{caret-color:hsla(48,96%,89%,0)}.caret-amber-100\/10{caret-color:hsla(48,96%,89%,.1)}.caret-amber-100\/100{caret-color:#fef3c7}.caret-amber-100\/15{caret-color:hsla(48,96%,89%,.15)}.caret-amber-100\/20{caret-color:hsla(48,96%,89%,.2)}.caret-amber-100\/25{caret-color:hsla(48,96%,89%,.25)}.caret-amber-100\/30{caret-color:hsla(48,96%,89%,.3)}.caret-amber-100\/35{caret-color:hsla(48,96%,89%,.35)}.caret-amber-100\/40{caret-color:hsla(48,96%,89%,.4)}.caret-amber-100\/45{caret-color:hsla(48,96%,89%,.45)}.caret-amber-100\/5{caret-color:hsla(48,96%,89%,.05)}.caret-amber-100\/50{caret-color:hsla(48,96%,89%,.5)}.caret-amber-100\/55{caret-color:hsla(48,96%,89%,.55)}.caret-amber-100\/60{caret-color:hsla(48,96%,89%,.6)}.caret-amber-100\/65{caret-color:hsla(48,96%,89%,.65)}.caret-amber-100\/70{caret-color:hsla(48,96%,89%,.7)}.caret-amber-100\/75{caret-color:hsla(48,96%,89%,.75)}.caret-amber-100\/80{caret-color:hsla(48,96%,89%,.8)}.caret-amber-100\/85{caret-color:hsla(48,96%,89%,.85)}.caret-amber-100\/90{caret-color:hsla(48,96%,89%,.9)}.caret-amber-100\/95{caret-color:hsla(48,96%,89%,.95)}.caret-amber-200{caret-color:#fde68a}.caret-amber-200\/0{caret-color:hsla(48,97%,77%,0)}.caret-amber-200\/10{caret-color:hsla(48,97%,77%,.1)}.caret-amber-200\/100{caret-color:#fde68a}.caret-amber-200\/15{caret-color:hsla(48,97%,77%,.15)}.caret-amber-200\/20{caret-color:hsla(48,97%,77%,.2)}.caret-amber-200\/25{caret-color:hsla(48,97%,77%,.25)}.caret-amber-200\/30{caret-color:hsla(48,97%,77%,.3)}.caret-amber-200\/35{caret-color:hsla(48,97%,77%,.35)}.caret-amber-200\/40{caret-color:hsla(48,97%,77%,.4)}.caret-amber-200\/45{caret-color:hsla(48,97%,77%,.45)}.caret-amber-200\/5{caret-color:hsla(48,97%,77%,.05)}.caret-amber-200\/50{caret-color:hsla(48,97%,77%,.5)}.caret-amber-200\/55{caret-color:hsla(48,97%,77%,.55)}.caret-amber-200\/60{caret-color:hsla(48,97%,77%,.6)}.caret-amber-200\/65{caret-color:hsla(48,97%,77%,.65)}.caret-amber-200\/70{caret-color:hsla(48,97%,77%,.7)}.caret-amber-200\/75{caret-color:hsla(48,97%,77%,.75)}.caret-amber-200\/80{caret-color:hsla(48,97%,77%,.8)}.caret-amber-200\/85{caret-color:hsla(48,97%,77%,.85)}.caret-amber-200\/90{caret-color:hsla(48,97%,77%,.9)}.caret-amber-200\/95{caret-color:hsla(48,97%,77%,.95)}.caret-amber-400{caret-color:#fbbf24}.caret-amber-400\/0{caret-color:rgba(251,191,36,0)}.caret-amber-400\/10{caret-color:rgba(251,191,36,.1)}.caret-amber-400\/100{caret-color:#fbbf24}.caret-amber-400\/15{caret-color:rgba(251,191,36,.15)}.caret-amber-400\/20{caret-color:rgba(251,191,36,.2)}.caret-amber-400\/25{caret-color:rgba(251,191,36,.25)}.caret-amber-400\/30{caret-color:rgba(251,191,36,.3)}.caret-amber-400\/35{caret-color:rgba(251,191,36,.35)}.caret-amber-400\/40{caret-color:rgba(251,191,36,.4)}.caret-amber-400\/45{caret-color:rgba(251,191,36,.45)}.caret-amber-400\/5{caret-color:rgba(251,191,36,.05)}.caret-amber-400\/50{caret-color:rgba(251,191,36,.5)}.caret-amber-400\/55{caret-color:rgba(251,191,36,.55)}.caret-amber-400\/60{caret-color:rgba(251,191,36,.6)}.caret-amber-400\/65{caret-color:rgba(251,191,36,.65)}.caret-amber-400\/70{caret-color:rgba(251,191,36,.7)}.caret-amber-400\/75{caret-color:rgba(251,191,36,.75)}.caret-amber-400\/80{caret-color:rgba(251,191,36,.8)}.caret-amber-400\/85{caret-color:rgba(251,191,36,.85)}.caret-amber-400\/90{caret-color:rgba(251,191,36,.9)}.caret-amber-400\/95{caret-color:rgba(251,191,36,.95)}.caret-amber-50{caret-color:#fffbeb}.caret-amber-50\/0{caret-color:rgba(255,251,235,0)}.caret-amber-50\/10{caret-color:rgba(255,251,235,.1)}.caret-amber-50\/100{caret-color:#fffbeb}.caret-amber-50\/15{caret-color:rgba(255,251,235,.15)}.caret-amber-50\/20{caret-color:rgba(255,251,235,.2)}.caret-amber-50\/25{caret-color:rgba(255,251,235,.25)}.caret-amber-50\/30{caret-color:rgba(255,251,235,.3)}.caret-amber-50\/35{caret-color:rgba(255,251,235,.35)}.caret-amber-50\/40{caret-color:rgba(255,251,235,.4)}.caret-amber-50\/45{caret-color:rgba(255,251,235,.45)}.caret-amber-50\/5{caret-color:rgba(255,251,235,.05)}.caret-amber-50\/50{caret-color:rgba(255,251,235,.5)}.caret-amber-50\/55{caret-color:rgba(255,251,235,.55)}.caret-amber-50\/60{caret-color:rgba(255,251,235,.6)}.caret-amber-50\/65{caret-color:rgba(255,251,235,.65)}.caret-amber-50\/70{caret-color:rgba(255,251,235,.7)}.caret-amber-50\/75{caret-color:rgba(255,251,235,.75)}.caret-amber-50\/80{caret-color:rgba(255,251,235,.8)}.caret-amber-50\/85{caret-color:rgba(255,251,235,.85)}.caret-amber-50\/90{caret-color:rgba(255,251,235,.9)}.caret-amber-50\/95{caret-color:rgba(255,251,235,.95)}.caret-amber-500{caret-color:#f59e0b}.caret-amber-500\/0{caret-color:rgba(245,158,11,0)}.caret-amber-500\/10{caret-color:rgba(245,158,11,.1)}.caret-amber-500\/100{caret-color:#f59e0b}.caret-amber-500\/15{caret-color:rgba(245,158,11,.15)}.caret-amber-500\/20{caret-color:rgba(245,158,11,.2)}.caret-amber-500\/25{caret-color:rgba(245,158,11,.25)}.caret-amber-500\/30{caret-color:rgba(245,158,11,.3)}.caret-amber-500\/35{caret-color:rgba(245,158,11,.35)}.caret-amber-500\/40{caret-color:rgba(245,158,11,.4)}.caret-amber-500\/45{caret-color:rgba(245,158,11,.45)}.caret-amber-500\/5{caret-color:rgba(245,158,11,.05)}.caret-amber-500\/50{caret-color:rgba(245,158,11,.5)}.caret-amber-500\/55{caret-color:rgba(245,158,11,.55)}.caret-amber-500\/60{caret-color:rgba(245,158,11,.6)}.caret-amber-500\/65{caret-color:rgba(245,158,11,.65)}.caret-amber-500\/70{caret-color:rgba(245,158,11,.7)}.caret-amber-500\/75{caret-color:rgba(245,158,11,.75)}.caret-amber-500\/80{caret-color:rgba(245,158,11,.8)}.caret-amber-500\/85{caret-color:rgba(245,158,11,.85)}.caret-amber-500\/90{caret-color:rgba(245,158,11,.9)}.caret-amber-500\/95{caret-color:rgba(245,158,11,.95)}.caret-amber-950{caret-color:#451a03}.caret-amber-950\/0{caret-color:rgba(69,26,3,0)}.caret-amber-950\/10{caret-color:rgba(69,26,3,.1)}.caret-amber-950\/100{caret-color:#451a03}.caret-amber-950\/15{caret-color:rgba(69,26,3,.15)}.caret-amber-950\/20{caret-color:rgba(69,26,3,.2)}.caret-amber-950\/25{caret-color:rgba(69,26,3,.25)}.caret-amber-950\/30{caret-color:rgba(69,26,3,.3)}.caret-amber-950\/35{caret-color:rgba(69,26,3,.35)}.caret-amber-950\/40{caret-color:rgba(69,26,3,.4)}.caret-amber-950\/45{caret-color:rgba(69,26,3,.45)}.caret-amber-950\/5{caret-color:rgba(69,26,3,.05)}.caret-amber-950\/50{caret-color:rgba(69,26,3,.5)}.caret-amber-950\/55{caret-color:rgba(69,26,3,.55)}.caret-amber-950\/60{caret-color:rgba(69,26,3,.6)}.caret-amber-950\/65{caret-color:rgba(69,26,3,.65)}.caret-amber-950\/70{caret-color:rgba(69,26,3,.7)}.caret-amber-950\/75{caret-color:rgba(69,26,3,.75)}.caret-amber-950\/80{caret-color:rgba(69,26,3,.8)}.caret-amber-950\/85{caret-color:rgba(69,26,3,.85)}.caret-amber-950\/90{caret-color:rgba(69,26,3,.9)}.caret-amber-950\/95{caret-color:rgba(69,26,3,.95)}.caret-blue-100{caret-color:#dbeafe}.caret-blue-100\/0{caret-color:rgba(219,234,254,0)}.caret-blue-100\/10{caret-color:rgba(219,234,254,.1)}.caret-blue-100\/100{caret-color:#dbeafe}.caret-blue-100\/15{caret-color:rgba(219,234,254,.15)}.caret-blue-100\/20{caret-color:rgba(219,234,254,.2)}.caret-blue-100\/25{caret-color:rgba(219,234,254,.25)}.caret-blue-100\/30{caret-color:rgba(219,234,254,.3)}.caret-blue-100\/35{caret-color:rgba(219,234,254,.35)}.caret-blue-100\/40{caret-color:rgba(219,234,254,.4)}.caret-blue-100\/45{caret-color:rgba(219,234,254,.45)}.caret-blue-100\/5{caret-color:rgba(219,234,254,.05)}.caret-blue-100\/50{caret-color:rgba(219,234,254,.5)}.caret-blue-100\/55{caret-color:rgba(219,234,254,.55)}.caret-blue-100\/60{caret-color:rgba(219,234,254,.6)}.caret-blue-100\/65{caret-color:rgba(219,234,254,.65)}.caret-blue-100\/70{caret-color:rgba(219,234,254,.7)}.caret-blue-100\/75{caret-color:rgba(219,234,254,.75)}.caret-blue-100\/80{caret-color:rgba(219,234,254,.8)}.caret-blue-100\/85{caret-color:rgba(219,234,254,.85)}.caret-blue-100\/90{caret-color:rgba(219,234,254,.9)}.caret-blue-100\/95{caret-color:rgba(219,234,254,.95)}.caret-blue-200{caret-color:#bfdbfe}.caret-blue-200\/0{caret-color:rgba(191,219,254,0)}.caret-blue-200\/10{caret-color:rgba(191,219,254,.1)}.caret-blue-200\/100{caret-color:#bfdbfe}.caret-blue-200\/15{caret-color:rgba(191,219,254,.15)}.caret-blue-200\/20{caret-color:rgba(191,219,254,.2)}.caret-blue-200\/25{caret-color:rgba(191,219,254,.25)}.caret-blue-200\/30{caret-color:rgba(191,219,254,.3)}.caret-blue-200\/35{caret-color:rgba(191,219,254,.35)}.caret-blue-200\/40{caret-color:rgba(191,219,254,.4)}.caret-blue-200\/45{caret-color:rgba(191,219,254,.45)}.caret-blue-200\/5{caret-color:rgba(191,219,254,.05)}.caret-blue-200\/50{caret-color:rgba(191,219,254,.5)}.caret-blue-200\/55{caret-color:rgba(191,219,254,.55)}.caret-blue-200\/60{caret-color:rgba(191,219,254,.6)}.caret-blue-200\/65{caret-color:rgba(191,219,254,.65)}.caret-blue-200\/70{caret-color:rgba(191,219,254,.7)}.caret-blue-200\/75{caret-color:rgba(191,219,254,.75)}.caret-blue-200\/80{caret-color:rgba(191,219,254,.8)}.caret-blue-200\/85{caret-color:rgba(191,219,254,.85)}.caret-blue-200\/90{caret-color:rgba(191,219,254,.9)}.caret-blue-200\/95{caret-color:rgba(191,219,254,.95)}.caret-blue-400{caret-color:#60a5fa}.caret-blue-400\/0{caret-color:rgba(96,165,250,0)}.caret-blue-400\/10{caret-color:rgba(96,165,250,.1)}.caret-blue-400\/100{caret-color:#60a5fa}.caret-blue-400\/15{caret-color:rgba(96,165,250,.15)}.caret-blue-400\/20{caret-color:rgba(96,165,250,.2)}.caret-blue-400\/25{caret-color:rgba(96,165,250,.25)}.caret-blue-400\/30{caret-color:rgba(96,165,250,.3)}.caret-blue-400\/35{caret-color:rgba(96,165,250,.35)}.caret-blue-400\/40{caret-color:rgba(96,165,250,.4)}.caret-blue-400\/45{caret-color:rgba(96,165,250,.45)}.caret-blue-400\/5{caret-color:rgba(96,165,250,.05)}.caret-blue-400\/50{caret-color:rgba(96,165,250,.5)}.caret-blue-400\/55{caret-color:rgba(96,165,250,.55)}.caret-blue-400\/60{caret-color:rgba(96,165,250,.6)}.caret-blue-400\/65{caret-color:rgba(96,165,250,.65)}.caret-blue-400\/70{caret-color:rgba(96,165,250,.7)}.caret-blue-400\/75{caret-color:rgba(96,165,250,.75)}.caret-blue-400\/80{caret-color:rgba(96,165,250,.8)}.caret-blue-400\/85{caret-color:rgba(96,165,250,.85)}.caret-blue-400\/90{caret-color:rgba(96,165,250,.9)}.caret-blue-400\/95{caret-color:rgba(96,165,250,.95)}.caret-blue-50{caret-color:#eff6ff}.caret-blue-50\/0{caret-color:rgba(239,246,255,0)}.caret-blue-50\/10{caret-color:rgba(239,246,255,.1)}.caret-blue-50\/100{caret-color:#eff6ff}.caret-blue-50\/15{caret-color:rgba(239,246,255,.15)}.caret-blue-50\/20{caret-color:rgba(239,246,255,.2)}.caret-blue-50\/25{caret-color:rgba(239,246,255,.25)}.caret-blue-50\/30{caret-color:rgba(239,246,255,.3)}.caret-blue-50\/35{caret-color:rgba(239,246,255,.35)}.caret-blue-50\/40{caret-color:rgba(239,246,255,.4)}.caret-blue-50\/45{caret-color:rgba(239,246,255,.45)}.caret-blue-50\/5{caret-color:rgba(239,246,255,.05)}.caret-blue-50\/50{caret-color:rgba(239,246,255,.5)}.caret-blue-50\/55{caret-color:rgba(239,246,255,.55)}.caret-blue-50\/60{caret-color:rgba(239,246,255,.6)}.caret-blue-50\/65{caret-color:rgba(239,246,255,.65)}.caret-blue-50\/70{caret-color:rgba(239,246,255,.7)}.caret-blue-50\/75{caret-color:rgba(239,246,255,.75)}.caret-blue-50\/80{caret-color:rgba(239,246,255,.8)}.caret-blue-50\/85{caret-color:rgba(239,246,255,.85)}.caret-blue-50\/90{caret-color:rgba(239,246,255,.9)}.caret-blue-50\/95{caret-color:rgba(239,246,255,.95)}.caret-blue-500{caret-color:#3b82f6}.caret-blue-500\/0{caret-color:rgba(59,130,246,0)}.caret-blue-500\/10{caret-color:rgba(59,130,246,.1)}.caret-blue-500\/100{caret-color:#3b82f6}.caret-blue-500\/15{caret-color:rgba(59,130,246,.15)}.caret-blue-500\/20{caret-color:rgba(59,130,246,.2)}.caret-blue-500\/25{caret-color:rgba(59,130,246,.25)}.caret-blue-500\/30{caret-color:rgba(59,130,246,.3)}.caret-blue-500\/35{caret-color:rgba(59,130,246,.35)}.caret-blue-500\/40{caret-color:rgba(59,130,246,.4)}.caret-blue-500\/45{caret-color:rgba(59,130,246,.45)}.caret-blue-500\/5{caret-color:rgba(59,130,246,.05)}.caret-blue-500\/50{caret-color:rgba(59,130,246,.5)}.caret-blue-500\/55{caret-color:rgba(59,130,246,.55)}.caret-blue-500\/60{caret-color:rgba(59,130,246,.6)}.caret-blue-500\/65{caret-color:rgba(59,130,246,.65)}.caret-blue-500\/70{caret-color:rgba(59,130,246,.7)}.caret-blue-500\/75{caret-color:rgba(59,130,246,.75)}.caret-blue-500\/80{caret-color:rgba(59,130,246,.8)}.caret-blue-500\/85{caret-color:rgba(59,130,246,.85)}.caret-blue-500\/90{caret-color:rgba(59,130,246,.9)}.caret-blue-500\/95{caret-color:rgba(59,130,246,.95)}.caret-blue-950{caret-color:#172554}.caret-blue-950\/0{caret-color:rgba(23,37,84,0)}.caret-blue-950\/10{caret-color:rgba(23,37,84,.1)}.caret-blue-950\/100{caret-color:#172554}.caret-blue-950\/15{caret-color:rgba(23,37,84,.15)}.caret-blue-950\/20{caret-color:rgba(23,37,84,.2)}.caret-blue-950\/25{caret-color:rgba(23,37,84,.25)}.caret-blue-950\/30{caret-color:rgba(23,37,84,.3)}.caret-blue-950\/35{caret-color:rgba(23,37,84,.35)}.caret-blue-950\/40{caret-color:rgba(23,37,84,.4)}.caret-blue-950\/45{caret-color:rgba(23,37,84,.45)}.caret-blue-950\/5{caret-color:rgba(23,37,84,.05)}.caret-blue-950\/50{caret-color:rgba(23,37,84,.5)}.caret-blue-950\/55{caret-color:rgba(23,37,84,.55)}.caret-blue-950\/60{caret-color:rgba(23,37,84,.6)}.caret-blue-950\/65{caret-color:rgba(23,37,84,.65)}.caret-blue-950\/70{caret-color:rgba(23,37,84,.7)}.caret-blue-950\/75{caret-color:rgba(23,37,84,.75)}.caret-blue-950\/80{caret-color:rgba(23,37,84,.8)}.caret-blue-950\/85{caret-color:rgba(23,37,84,.85)}.caret-blue-950\/90{caret-color:rgba(23,37,84,.9)}.caret-blue-950\/95{caret-color:rgba(23,37,84,.95)}.caret-cyan-100{caret-color:#cffafe}.caret-cyan-100\/0{caret-color:rgba(207,250,254,0)}.caret-cyan-100\/10{caret-color:rgba(207,250,254,.1)}.caret-cyan-100\/100{caret-color:#cffafe}.caret-cyan-100\/15{caret-color:rgba(207,250,254,.15)}.caret-cyan-100\/20{caret-color:rgba(207,250,254,.2)}.caret-cyan-100\/25{caret-color:rgba(207,250,254,.25)}.caret-cyan-100\/30{caret-color:rgba(207,250,254,.3)}.caret-cyan-100\/35{caret-color:rgba(207,250,254,.35)}.caret-cyan-100\/40{caret-color:rgba(207,250,254,.4)}.caret-cyan-100\/45{caret-color:rgba(207,250,254,.45)}.caret-cyan-100\/5{caret-color:rgba(207,250,254,.05)}.caret-cyan-100\/50{caret-color:rgba(207,250,254,.5)}.caret-cyan-100\/55{caret-color:rgba(207,250,254,.55)}.caret-cyan-100\/60{caret-color:rgba(207,250,254,.6)}.caret-cyan-100\/65{caret-color:rgba(207,250,254,.65)}.caret-cyan-100\/70{caret-color:rgba(207,250,254,.7)}.caret-cyan-100\/75{caret-color:rgba(207,250,254,.75)}.caret-cyan-100\/80{caret-color:rgba(207,250,254,.8)}.caret-cyan-100\/85{caret-color:rgba(207,250,254,.85)}.caret-cyan-100\/90{caret-color:rgba(207,250,254,.9)}.caret-cyan-100\/95{caret-color:rgba(207,250,254,.95)}.caret-cyan-200{caret-color:#a5f3fc}.caret-cyan-200\/0{caret-color:rgba(165,243,252,0)}.caret-cyan-200\/10{caret-color:rgba(165,243,252,.1)}.caret-cyan-200\/100{caret-color:#a5f3fc}.caret-cyan-200\/15{caret-color:rgba(165,243,252,.15)}.caret-cyan-200\/20{caret-color:rgba(165,243,252,.2)}.caret-cyan-200\/25{caret-color:rgba(165,243,252,.25)}.caret-cyan-200\/30{caret-color:rgba(165,243,252,.3)}.caret-cyan-200\/35{caret-color:rgba(165,243,252,.35)}.caret-cyan-200\/40{caret-color:rgba(165,243,252,.4)}.caret-cyan-200\/45{caret-color:rgba(165,243,252,.45)}.caret-cyan-200\/5{caret-color:rgba(165,243,252,.05)}.caret-cyan-200\/50{caret-color:rgba(165,243,252,.5)}.caret-cyan-200\/55{caret-color:rgba(165,243,252,.55)}.caret-cyan-200\/60{caret-color:rgba(165,243,252,.6)}.caret-cyan-200\/65{caret-color:rgba(165,243,252,.65)}.caret-cyan-200\/70{caret-color:rgba(165,243,252,.7)}.caret-cyan-200\/75{caret-color:rgba(165,243,252,.75)}.caret-cyan-200\/80{caret-color:rgba(165,243,252,.8)}.caret-cyan-200\/85{caret-color:rgba(165,243,252,.85)}.caret-cyan-200\/90{caret-color:rgba(165,243,252,.9)}.caret-cyan-200\/95{caret-color:rgba(165,243,252,.95)}.caret-cyan-400{caret-color:#22d3ee}.caret-cyan-400\/0{caret-color:rgba(34,211,238,0)}.caret-cyan-400\/10{caret-color:rgba(34,211,238,.1)}.caret-cyan-400\/100{caret-color:#22d3ee}.caret-cyan-400\/15{caret-color:rgba(34,211,238,.15)}.caret-cyan-400\/20{caret-color:rgba(34,211,238,.2)}.caret-cyan-400\/25{caret-color:rgba(34,211,238,.25)}.caret-cyan-400\/30{caret-color:rgba(34,211,238,.3)}.caret-cyan-400\/35{caret-color:rgba(34,211,238,.35)}.caret-cyan-400\/40{caret-color:rgba(34,211,238,.4)}.caret-cyan-400\/45{caret-color:rgba(34,211,238,.45)}.caret-cyan-400\/5{caret-color:rgba(34,211,238,.05)}.caret-cyan-400\/50{caret-color:rgba(34,211,238,.5)}.caret-cyan-400\/55{caret-color:rgba(34,211,238,.55)}.caret-cyan-400\/60{caret-color:rgba(34,211,238,.6)}.caret-cyan-400\/65{caret-color:rgba(34,211,238,.65)}.caret-cyan-400\/70{caret-color:rgba(34,211,238,.7)}.caret-cyan-400\/75{caret-color:rgba(34,211,238,.75)}.caret-cyan-400\/80{caret-color:rgba(34,211,238,.8)}.caret-cyan-400\/85{caret-color:rgba(34,211,238,.85)}.caret-cyan-400\/90{caret-color:rgba(34,211,238,.9)}.caret-cyan-400\/95{caret-color:rgba(34,211,238,.95)}.caret-cyan-50{caret-color:#ecfeff}.caret-cyan-50\/0{caret-color:rgba(236,254,255,0)}.caret-cyan-50\/10{caret-color:rgba(236,254,255,.1)}.caret-cyan-50\/100{caret-color:#ecfeff}.caret-cyan-50\/15{caret-color:rgba(236,254,255,.15)}.caret-cyan-50\/20{caret-color:rgba(236,254,255,.2)}.caret-cyan-50\/25{caret-color:rgba(236,254,255,.25)}.caret-cyan-50\/30{caret-color:rgba(236,254,255,.3)}.caret-cyan-50\/35{caret-color:rgba(236,254,255,.35)}.caret-cyan-50\/40{caret-color:rgba(236,254,255,.4)}.caret-cyan-50\/45{caret-color:rgba(236,254,255,.45)}.caret-cyan-50\/5{caret-color:rgba(236,254,255,.05)}.caret-cyan-50\/50{caret-color:rgba(236,254,255,.5)}.caret-cyan-50\/55{caret-color:rgba(236,254,255,.55)}.caret-cyan-50\/60{caret-color:rgba(236,254,255,.6)}.caret-cyan-50\/65{caret-color:rgba(236,254,255,.65)}.caret-cyan-50\/70{caret-color:rgba(236,254,255,.7)}.caret-cyan-50\/75{caret-color:rgba(236,254,255,.75)}.caret-cyan-50\/80{caret-color:rgba(236,254,255,.8)}.caret-cyan-50\/85{caret-color:rgba(236,254,255,.85)}.caret-cyan-50\/90{caret-color:rgba(236,254,255,.9)}.caret-cyan-50\/95{caret-color:rgba(236,254,255,.95)}.caret-cyan-500{caret-color:#06b6d4}.caret-cyan-500\/0{caret-color:rgba(6,182,212,0)}.caret-cyan-500\/10{caret-color:rgba(6,182,212,.1)}.caret-cyan-500\/100{caret-color:#06b6d4}.caret-cyan-500\/15{caret-color:rgba(6,182,212,.15)}.caret-cyan-500\/20{caret-color:rgba(6,182,212,.2)}.caret-cyan-500\/25{caret-color:rgba(6,182,212,.25)}.caret-cyan-500\/30{caret-color:rgba(6,182,212,.3)}.caret-cyan-500\/35{caret-color:rgba(6,182,212,.35)}.caret-cyan-500\/40{caret-color:rgba(6,182,212,.4)}.caret-cyan-500\/45{caret-color:rgba(6,182,212,.45)}.caret-cyan-500\/5{caret-color:rgba(6,182,212,.05)}.caret-cyan-500\/50{caret-color:rgba(6,182,212,.5)}.caret-cyan-500\/55{caret-color:rgba(6,182,212,.55)}.caret-cyan-500\/60{caret-color:rgba(6,182,212,.6)}.caret-cyan-500\/65{caret-color:rgba(6,182,212,.65)}.caret-cyan-500\/70{caret-color:rgba(6,182,212,.7)}.caret-cyan-500\/75{caret-color:rgba(6,182,212,.75)}.caret-cyan-500\/80{caret-color:rgba(6,182,212,.8)}.caret-cyan-500\/85{caret-color:rgba(6,182,212,.85)}.caret-cyan-500\/90{caret-color:rgba(6,182,212,.9)}.caret-cyan-500\/95{caret-color:rgba(6,182,212,.95)}.caret-cyan-950{caret-color:#083344}.caret-cyan-950\/0{caret-color:rgba(8,51,68,0)}.caret-cyan-950\/10{caret-color:rgba(8,51,68,.1)}.caret-cyan-950\/100{caret-color:#083344}.caret-cyan-950\/15{caret-color:rgba(8,51,68,.15)}.caret-cyan-950\/20{caret-color:rgba(8,51,68,.2)}.caret-cyan-950\/25{caret-color:rgba(8,51,68,.25)}.caret-cyan-950\/30{caret-color:rgba(8,51,68,.3)}.caret-cyan-950\/35{caret-color:rgba(8,51,68,.35)}.caret-cyan-950\/40{caret-color:rgba(8,51,68,.4)}.caret-cyan-950\/45{caret-color:rgba(8,51,68,.45)}.caret-cyan-950\/5{caret-color:rgba(8,51,68,.05)}.caret-cyan-950\/50{caret-color:rgba(8,51,68,.5)}.caret-cyan-950\/55{caret-color:rgba(8,51,68,.55)}.caret-cyan-950\/60{caret-color:rgba(8,51,68,.6)}.caret-cyan-950\/65{caret-color:rgba(8,51,68,.65)}.caret-cyan-950\/70{caret-color:rgba(8,51,68,.7)}.caret-cyan-950\/75{caret-color:rgba(8,51,68,.75)}.caret-cyan-950\/80{caret-color:rgba(8,51,68,.8)}.caret-cyan-950\/85{caret-color:rgba(8,51,68,.85)}.caret-cyan-950\/90{caret-color:rgba(8,51,68,.9)}.caret-cyan-950\/95{caret-color:rgba(8,51,68,.95)}.caret-emerald-100{caret-color:#d1fae5}.caret-emerald-100\/0{caret-color:rgba(209,250,229,0)}.caret-emerald-100\/10{caret-color:rgba(209,250,229,.1)}.caret-emerald-100\/100{caret-color:#d1fae5}.caret-emerald-100\/15{caret-color:rgba(209,250,229,.15)}.caret-emerald-100\/20{caret-color:rgba(209,250,229,.2)}.caret-emerald-100\/25{caret-color:rgba(209,250,229,.25)}.caret-emerald-100\/30{caret-color:rgba(209,250,229,.3)}.caret-emerald-100\/35{caret-color:rgba(209,250,229,.35)}.caret-emerald-100\/40{caret-color:rgba(209,250,229,.4)}.caret-emerald-100\/45{caret-color:rgba(209,250,229,.45)}.caret-emerald-100\/5{caret-color:rgba(209,250,229,.05)}.caret-emerald-100\/50{caret-color:rgba(209,250,229,.5)}.caret-emerald-100\/55{caret-color:rgba(209,250,229,.55)}.caret-emerald-100\/60{caret-color:rgba(209,250,229,.6)}.caret-emerald-100\/65{caret-color:rgba(209,250,229,.65)}.caret-emerald-100\/70{caret-color:rgba(209,250,229,.7)}.caret-emerald-100\/75{caret-color:rgba(209,250,229,.75)}.caret-emerald-100\/80{caret-color:rgba(209,250,229,.8)}.caret-emerald-100\/85{caret-color:rgba(209,250,229,.85)}.caret-emerald-100\/90{caret-color:rgba(209,250,229,.9)}.caret-emerald-100\/95{caret-color:rgba(209,250,229,.95)}.caret-emerald-200{caret-color:#a7f3d0}.caret-emerald-200\/0{caret-color:rgba(167,243,208,0)}.caret-emerald-200\/10{caret-color:rgba(167,243,208,.1)}.caret-emerald-200\/100{caret-color:#a7f3d0}.caret-emerald-200\/15{caret-color:rgba(167,243,208,.15)}.caret-emerald-200\/20{caret-color:rgba(167,243,208,.2)}.caret-emerald-200\/25{caret-color:rgba(167,243,208,.25)}.caret-emerald-200\/30{caret-color:rgba(167,243,208,.3)}.caret-emerald-200\/35{caret-color:rgba(167,243,208,.35)}.caret-emerald-200\/40{caret-color:rgba(167,243,208,.4)}.caret-emerald-200\/45{caret-color:rgba(167,243,208,.45)}.caret-emerald-200\/5{caret-color:rgba(167,243,208,.05)}.caret-emerald-200\/50{caret-color:rgba(167,243,208,.5)}.caret-emerald-200\/55{caret-color:rgba(167,243,208,.55)}.caret-emerald-200\/60{caret-color:rgba(167,243,208,.6)}.caret-emerald-200\/65{caret-color:rgba(167,243,208,.65)}.caret-emerald-200\/70{caret-color:rgba(167,243,208,.7)}.caret-emerald-200\/75{caret-color:rgba(167,243,208,.75)}.caret-emerald-200\/80{caret-color:rgba(167,243,208,.8)}.caret-emerald-200\/85{caret-color:rgba(167,243,208,.85)}.caret-emerald-200\/90{caret-color:rgba(167,243,208,.9)}.caret-emerald-200\/95{caret-color:rgba(167,243,208,.95)}.caret-emerald-400{caret-color:#34d399}.caret-emerald-400\/0{caret-color:rgba(52,211,153,0)}.caret-emerald-400\/10{caret-color:rgba(52,211,153,.1)}.caret-emerald-400\/100{caret-color:#34d399}.caret-emerald-400\/15{caret-color:rgba(52,211,153,.15)}.caret-emerald-400\/20{caret-color:rgba(52,211,153,.2)}.caret-emerald-400\/25{caret-color:rgba(52,211,153,.25)}.caret-emerald-400\/30{caret-color:rgba(52,211,153,.3)}.caret-emerald-400\/35{caret-color:rgba(52,211,153,.35)}.caret-emerald-400\/40{caret-color:rgba(52,211,153,.4)}.caret-emerald-400\/45{caret-color:rgba(52,211,153,.45)}.caret-emerald-400\/5{caret-color:rgba(52,211,153,.05)}.caret-emerald-400\/50{caret-color:rgba(52,211,153,.5)}.caret-emerald-400\/55{caret-color:rgba(52,211,153,.55)}.caret-emerald-400\/60{caret-color:rgba(52,211,153,.6)}.caret-emerald-400\/65{caret-color:rgba(52,211,153,.65)}.caret-emerald-400\/70{caret-color:rgba(52,211,153,.7)}.caret-emerald-400\/75{caret-color:rgba(52,211,153,.75)}.caret-emerald-400\/80{caret-color:rgba(52,211,153,.8)}.caret-emerald-400\/85{caret-color:rgba(52,211,153,.85)}.caret-emerald-400\/90{caret-color:rgba(52,211,153,.9)}.caret-emerald-400\/95{caret-color:rgba(52,211,153,.95)}.caret-emerald-50{caret-color:#ecfdf5}.caret-emerald-50\/0{caret-color:rgba(236,253,245,0)}.caret-emerald-50\/10{caret-color:rgba(236,253,245,.1)}.caret-emerald-50\/100{caret-color:#ecfdf5}.caret-emerald-50\/15{caret-color:rgba(236,253,245,.15)}.caret-emerald-50\/20{caret-color:rgba(236,253,245,.2)}.caret-emerald-50\/25{caret-color:rgba(236,253,245,.25)}.caret-emerald-50\/30{caret-color:rgba(236,253,245,.3)}.caret-emerald-50\/35{caret-color:rgba(236,253,245,.35)}.caret-emerald-50\/40{caret-color:rgba(236,253,245,.4)}.caret-emerald-50\/45{caret-color:rgba(236,253,245,.45)}.caret-emerald-50\/5{caret-color:rgba(236,253,245,.05)}.caret-emerald-50\/50{caret-color:rgba(236,253,245,.5)}.caret-emerald-50\/55{caret-color:rgba(236,253,245,.55)}.caret-emerald-50\/60{caret-color:rgba(236,253,245,.6)}.caret-emerald-50\/65{caret-color:rgba(236,253,245,.65)}.caret-emerald-50\/70{caret-color:rgba(236,253,245,.7)}.caret-emerald-50\/75{caret-color:rgba(236,253,245,.75)}.caret-emerald-50\/80{caret-color:rgba(236,253,245,.8)}.caret-emerald-50\/85{caret-color:rgba(236,253,245,.85)}.caret-emerald-50\/90{caret-color:rgba(236,253,245,.9)}.caret-emerald-50\/95{caret-color:rgba(236,253,245,.95)}.caret-emerald-500{caret-color:#10b981}.caret-emerald-500\/0{caret-color:rgba(16,185,129,0)}.caret-emerald-500\/10{caret-color:rgba(16,185,129,.1)}.caret-emerald-500\/100{caret-color:#10b981}.caret-emerald-500\/15{caret-color:rgba(16,185,129,.15)}.caret-emerald-500\/20{caret-color:rgba(16,185,129,.2)}.caret-emerald-500\/25{caret-color:rgba(16,185,129,.25)}.caret-emerald-500\/30{caret-color:rgba(16,185,129,.3)}.caret-emerald-500\/35{caret-color:rgba(16,185,129,.35)}.caret-emerald-500\/40{caret-color:rgba(16,185,129,.4)}.caret-emerald-500\/45{caret-color:rgba(16,185,129,.45)}.caret-emerald-500\/5{caret-color:rgba(16,185,129,.05)}.caret-emerald-500\/50{caret-color:rgba(16,185,129,.5)}.caret-emerald-500\/55{caret-color:rgba(16,185,129,.55)}.caret-emerald-500\/60{caret-color:rgba(16,185,129,.6)}.caret-emerald-500\/65{caret-color:rgba(16,185,129,.65)}.caret-emerald-500\/70{caret-color:rgba(16,185,129,.7)}.caret-emerald-500\/75{caret-color:rgba(16,185,129,.75)}.caret-emerald-500\/80{caret-color:rgba(16,185,129,.8)}.caret-emerald-500\/85{caret-color:rgba(16,185,129,.85)}.caret-emerald-500\/90{caret-color:rgba(16,185,129,.9)}.caret-emerald-500\/95{caret-color:rgba(16,185,129,.95)}.caret-emerald-950{caret-color:#022c22}.caret-emerald-950\/0{caret-color:rgba(2,44,34,0)}.caret-emerald-950\/10{caret-color:rgba(2,44,34,.1)}.caret-emerald-950\/100{caret-color:#022c22}.caret-emerald-950\/15{caret-color:rgba(2,44,34,.15)}.caret-emerald-950\/20{caret-color:rgba(2,44,34,.2)}.caret-emerald-950\/25{caret-color:rgba(2,44,34,.25)}.caret-emerald-950\/30{caret-color:rgba(2,44,34,.3)}.caret-emerald-950\/35{caret-color:rgba(2,44,34,.35)}.caret-emerald-950\/40{caret-color:rgba(2,44,34,.4)}.caret-emerald-950\/45{caret-color:rgba(2,44,34,.45)}.caret-emerald-950\/5{caret-color:rgba(2,44,34,.05)}.caret-emerald-950\/50{caret-color:rgba(2,44,34,.5)}.caret-emerald-950\/55{caret-color:rgba(2,44,34,.55)}.caret-emerald-950\/60{caret-color:rgba(2,44,34,.6)}.caret-emerald-950\/65{caret-color:rgba(2,44,34,.65)}.caret-emerald-950\/70{caret-color:rgba(2,44,34,.7)}.caret-emerald-950\/75{caret-color:rgba(2,44,34,.75)}.caret-emerald-950\/80{caret-color:rgba(2,44,34,.8)}.caret-emerald-950\/85{caret-color:rgba(2,44,34,.85)}.caret-emerald-950\/90{caret-color:rgba(2,44,34,.9)}.caret-emerald-950\/95{caret-color:rgba(2,44,34,.95)}.caret-fuchsia-100{caret-color:#fae8ff}.caret-fuchsia-100\/0{caret-color:rgba(250,232,255,0)}.caret-fuchsia-100\/10{caret-color:rgba(250,232,255,.1)}.caret-fuchsia-100\/100{caret-color:#fae8ff}.caret-fuchsia-100\/15{caret-color:rgba(250,232,255,.15)}.caret-fuchsia-100\/20{caret-color:rgba(250,232,255,.2)}.caret-fuchsia-100\/25{caret-color:rgba(250,232,255,.25)}.caret-fuchsia-100\/30{caret-color:rgba(250,232,255,.3)}.caret-fuchsia-100\/35{caret-color:rgba(250,232,255,.35)}.caret-fuchsia-100\/40{caret-color:rgba(250,232,255,.4)}.caret-fuchsia-100\/45{caret-color:rgba(250,232,255,.45)}.caret-fuchsia-100\/5{caret-color:rgba(250,232,255,.05)}.caret-fuchsia-100\/50{caret-color:rgba(250,232,255,.5)}.caret-fuchsia-100\/55{caret-color:rgba(250,232,255,.55)}.caret-fuchsia-100\/60{caret-color:rgba(250,232,255,.6)}.caret-fuchsia-100\/65{caret-color:rgba(250,232,255,.65)}.caret-fuchsia-100\/70{caret-color:rgba(250,232,255,.7)}.caret-fuchsia-100\/75{caret-color:rgba(250,232,255,.75)}.caret-fuchsia-100\/80{caret-color:rgba(250,232,255,.8)}.caret-fuchsia-100\/85{caret-color:rgba(250,232,255,.85)}.caret-fuchsia-100\/90{caret-color:rgba(250,232,255,.9)}.caret-fuchsia-100\/95{caret-color:rgba(250,232,255,.95)}.caret-fuchsia-200{caret-color:#f5d0fe}.caret-fuchsia-200\/0{caret-color:rgba(245,208,254,0)}.caret-fuchsia-200\/10{caret-color:rgba(245,208,254,.1)}.caret-fuchsia-200\/100{caret-color:#f5d0fe}.caret-fuchsia-200\/15{caret-color:rgba(245,208,254,.15)}.caret-fuchsia-200\/20{caret-color:rgba(245,208,254,.2)}.caret-fuchsia-200\/25{caret-color:rgba(245,208,254,.25)}.caret-fuchsia-200\/30{caret-color:rgba(245,208,254,.3)}.caret-fuchsia-200\/35{caret-color:rgba(245,208,254,.35)}.caret-fuchsia-200\/40{caret-color:rgba(245,208,254,.4)}.caret-fuchsia-200\/45{caret-color:rgba(245,208,254,.45)}.caret-fuchsia-200\/5{caret-color:rgba(245,208,254,.05)}.caret-fuchsia-200\/50{caret-color:rgba(245,208,254,.5)}.caret-fuchsia-200\/55{caret-color:rgba(245,208,254,.55)}.caret-fuchsia-200\/60{caret-color:rgba(245,208,254,.6)}.caret-fuchsia-200\/65{caret-color:rgba(245,208,254,.65)}.caret-fuchsia-200\/70{caret-color:rgba(245,208,254,.7)}.caret-fuchsia-200\/75{caret-color:rgba(245,208,254,.75)}.caret-fuchsia-200\/80{caret-color:rgba(245,208,254,.8)}.caret-fuchsia-200\/85{caret-color:rgba(245,208,254,.85)}.caret-fuchsia-200\/90{caret-color:rgba(245,208,254,.9)}.caret-fuchsia-200\/95{caret-color:rgba(245,208,254,.95)}.caret-fuchsia-400{caret-color:#e879f9}.caret-fuchsia-400\/0{caret-color:rgba(232,121,249,0)}.caret-fuchsia-400\/10{caret-color:rgba(232,121,249,.1)}.caret-fuchsia-400\/100{caret-color:#e879f9}.caret-fuchsia-400\/15{caret-color:rgba(232,121,249,.15)}.caret-fuchsia-400\/20{caret-color:rgba(232,121,249,.2)}.caret-fuchsia-400\/25{caret-color:rgba(232,121,249,.25)}.caret-fuchsia-400\/30{caret-color:rgba(232,121,249,.3)}.caret-fuchsia-400\/35{caret-color:rgba(232,121,249,.35)}.caret-fuchsia-400\/40{caret-color:rgba(232,121,249,.4)}.caret-fuchsia-400\/45{caret-color:rgba(232,121,249,.45)}.caret-fuchsia-400\/5{caret-color:rgba(232,121,249,.05)}.caret-fuchsia-400\/50{caret-color:rgba(232,121,249,.5)}.caret-fuchsia-400\/55{caret-color:rgba(232,121,249,.55)}.caret-fuchsia-400\/60{caret-color:rgba(232,121,249,.6)}.caret-fuchsia-400\/65{caret-color:rgba(232,121,249,.65)}.caret-fuchsia-400\/70{caret-color:rgba(232,121,249,.7)}.caret-fuchsia-400\/75{caret-color:rgba(232,121,249,.75)}.caret-fuchsia-400\/80{caret-color:rgba(232,121,249,.8)}.caret-fuchsia-400\/85{caret-color:rgba(232,121,249,.85)}.caret-fuchsia-400\/90{caret-color:rgba(232,121,249,.9)}.caret-fuchsia-400\/95{caret-color:rgba(232,121,249,.95)}.caret-fuchsia-50{caret-color:#fdf4ff}.caret-fuchsia-50\/0{caret-color:rgba(253,244,255,0)}.caret-fuchsia-50\/10{caret-color:rgba(253,244,255,.1)}.caret-fuchsia-50\/100{caret-color:#fdf4ff}.caret-fuchsia-50\/15{caret-color:rgba(253,244,255,.15)}.caret-fuchsia-50\/20{caret-color:rgba(253,244,255,.2)}.caret-fuchsia-50\/25{caret-color:rgba(253,244,255,.25)}.caret-fuchsia-50\/30{caret-color:rgba(253,244,255,.3)}.caret-fuchsia-50\/35{caret-color:rgba(253,244,255,.35)}.caret-fuchsia-50\/40{caret-color:rgba(253,244,255,.4)}.caret-fuchsia-50\/45{caret-color:rgba(253,244,255,.45)}.caret-fuchsia-50\/5{caret-color:rgba(253,244,255,.05)}.caret-fuchsia-50\/50{caret-color:rgba(253,244,255,.5)}.caret-fuchsia-50\/55{caret-color:rgba(253,244,255,.55)}.caret-fuchsia-50\/60{caret-color:rgba(253,244,255,.6)}.caret-fuchsia-50\/65{caret-color:rgba(253,244,255,.65)}.caret-fuchsia-50\/70{caret-color:rgba(253,244,255,.7)}.caret-fuchsia-50\/75{caret-color:rgba(253,244,255,.75)}.caret-fuchsia-50\/80{caret-color:rgba(253,244,255,.8)}.caret-fuchsia-50\/85{caret-color:rgba(253,244,255,.85)}.caret-fuchsia-50\/90{caret-color:rgba(253,244,255,.9)}.caret-fuchsia-50\/95{caret-color:rgba(253,244,255,.95)}.caret-fuchsia-500{caret-color:#d946ef}.caret-fuchsia-500\/0{caret-color:rgba(217,70,239,0)}.caret-fuchsia-500\/10{caret-color:rgba(217,70,239,.1)}.caret-fuchsia-500\/100{caret-color:#d946ef}.caret-fuchsia-500\/15{caret-color:rgba(217,70,239,.15)}.caret-fuchsia-500\/20{caret-color:rgba(217,70,239,.2)}.caret-fuchsia-500\/25{caret-color:rgba(217,70,239,.25)}.caret-fuchsia-500\/30{caret-color:rgba(217,70,239,.3)}.caret-fuchsia-500\/35{caret-color:rgba(217,70,239,.35)}.caret-fuchsia-500\/40{caret-color:rgba(217,70,239,.4)}.caret-fuchsia-500\/45{caret-color:rgba(217,70,239,.45)}.caret-fuchsia-500\/5{caret-color:rgba(217,70,239,.05)}.caret-fuchsia-500\/50{caret-color:rgba(217,70,239,.5)}.caret-fuchsia-500\/55{caret-color:rgba(217,70,239,.55)}.caret-fuchsia-500\/60{caret-color:rgba(217,70,239,.6)}.caret-fuchsia-500\/65{caret-color:rgba(217,70,239,.65)}.caret-fuchsia-500\/70{caret-color:rgba(217,70,239,.7)}.caret-fuchsia-500\/75{caret-color:rgba(217,70,239,.75)}.caret-fuchsia-500\/80{caret-color:rgba(217,70,239,.8)}.caret-fuchsia-500\/85{caret-color:rgba(217,70,239,.85)}.caret-fuchsia-500\/90{caret-color:rgba(217,70,239,.9)}.caret-fuchsia-500\/95{caret-color:rgba(217,70,239,.95)}.caret-fuchsia-950{caret-color:#4a044e}.caret-fuchsia-950\/0{caret-color:rgba(74,4,78,0)}.caret-fuchsia-950\/10{caret-color:rgba(74,4,78,.1)}.caret-fuchsia-950\/100{caret-color:#4a044e}.caret-fuchsia-950\/15{caret-color:rgba(74,4,78,.15)}.caret-fuchsia-950\/20{caret-color:rgba(74,4,78,.2)}.caret-fuchsia-950\/25{caret-color:rgba(74,4,78,.25)}.caret-fuchsia-950\/30{caret-color:rgba(74,4,78,.3)}.caret-fuchsia-950\/35{caret-color:rgba(74,4,78,.35)}.caret-fuchsia-950\/40{caret-color:rgba(74,4,78,.4)}.caret-fuchsia-950\/45{caret-color:rgba(74,4,78,.45)}.caret-fuchsia-950\/5{caret-color:rgba(74,4,78,.05)}.caret-fuchsia-950\/50{caret-color:rgba(74,4,78,.5)}.caret-fuchsia-950\/55{caret-color:rgba(74,4,78,.55)}.caret-fuchsia-950\/60{caret-color:rgba(74,4,78,.6)}.caret-fuchsia-950\/65{caret-color:rgba(74,4,78,.65)}.caret-fuchsia-950\/70{caret-color:rgba(74,4,78,.7)}.caret-fuchsia-950\/75{caret-color:rgba(74,4,78,.75)}.caret-fuchsia-950\/80{caret-color:rgba(74,4,78,.8)}.caret-fuchsia-950\/85{caret-color:rgba(74,4,78,.85)}.caret-fuchsia-950\/90{caret-color:rgba(74,4,78,.9)}.caret-fuchsia-950\/95{caret-color:rgba(74,4,78,.95)}.caret-gray-100{caret-color:#f3f4f6}.caret-gray-100\/0{caret-color:rgba(243,244,246,0)}.caret-gray-100\/10{caret-color:rgba(243,244,246,.1)}.caret-gray-100\/100{caret-color:#f3f4f6}.caret-gray-100\/15{caret-color:rgba(243,244,246,.15)}.caret-gray-100\/20{caret-color:rgba(243,244,246,.2)}.caret-gray-100\/25{caret-color:rgba(243,244,246,.25)}.caret-gray-100\/30{caret-color:rgba(243,244,246,.3)}.caret-gray-100\/35{caret-color:rgba(243,244,246,.35)}.caret-gray-100\/40{caret-color:rgba(243,244,246,.4)}.caret-gray-100\/45{caret-color:rgba(243,244,246,.45)}.caret-gray-100\/5{caret-color:rgba(243,244,246,.05)}.caret-gray-100\/50{caret-color:rgba(243,244,246,.5)}.caret-gray-100\/55{caret-color:rgba(243,244,246,.55)}.caret-gray-100\/60{caret-color:rgba(243,244,246,.6)}.caret-gray-100\/65{caret-color:rgba(243,244,246,.65)}.caret-gray-100\/70{caret-color:rgba(243,244,246,.7)}.caret-gray-100\/75{caret-color:rgba(243,244,246,.75)}.caret-gray-100\/80{caret-color:rgba(243,244,246,.8)}.caret-gray-100\/85{caret-color:rgba(243,244,246,.85)}.caret-gray-100\/90{caret-color:rgba(243,244,246,.9)}.caret-gray-100\/95{caret-color:rgba(243,244,246,.95)}.caret-gray-200{caret-color:#e5e7eb}.caret-gray-200\/0{caret-color:rgba(229,231,235,0)}.caret-gray-200\/10{caret-color:rgba(229,231,235,.1)}.caret-gray-200\/100{caret-color:#e5e7eb}.caret-gray-200\/15{caret-color:rgba(229,231,235,.15)}.caret-gray-200\/20{caret-color:rgba(229,231,235,.2)}.caret-gray-200\/25{caret-color:rgba(229,231,235,.25)}.caret-gray-200\/30{caret-color:rgba(229,231,235,.3)}.caret-gray-200\/35{caret-color:rgba(229,231,235,.35)}.caret-gray-200\/40{caret-color:rgba(229,231,235,.4)}.caret-gray-200\/45{caret-color:rgba(229,231,235,.45)}.caret-gray-200\/5{caret-color:rgba(229,231,235,.05)}.caret-gray-200\/50{caret-color:rgba(229,231,235,.5)}.caret-gray-200\/55{caret-color:rgba(229,231,235,.55)}.caret-gray-200\/60{caret-color:rgba(229,231,235,.6)}.caret-gray-200\/65{caret-color:rgba(229,231,235,.65)}.caret-gray-200\/70{caret-color:rgba(229,231,235,.7)}.caret-gray-200\/75{caret-color:rgba(229,231,235,.75)}.caret-gray-200\/80{caret-color:rgba(229,231,235,.8)}.caret-gray-200\/85{caret-color:rgba(229,231,235,.85)}.caret-gray-200\/90{caret-color:rgba(229,231,235,.9)}.caret-gray-200\/95{caret-color:rgba(229,231,235,.95)}.caret-gray-400{caret-color:#9ca3af}.caret-gray-400\/0{caret-color:rgba(156,163,175,0)}.caret-gray-400\/10{caret-color:rgba(156,163,175,.1)}.caret-gray-400\/100{caret-color:#9ca3af}.caret-gray-400\/15{caret-color:rgba(156,163,175,.15)}.caret-gray-400\/20{caret-color:rgba(156,163,175,.2)}.caret-gray-400\/25{caret-color:rgba(156,163,175,.25)}.caret-gray-400\/30{caret-color:rgba(156,163,175,.3)}.caret-gray-400\/35{caret-color:rgba(156,163,175,.35)}.caret-gray-400\/40{caret-color:rgba(156,163,175,.4)}.caret-gray-400\/45{caret-color:rgba(156,163,175,.45)}.caret-gray-400\/5{caret-color:rgba(156,163,175,.05)}.caret-gray-400\/50{caret-color:rgba(156,163,175,.5)}.caret-gray-400\/55{caret-color:rgba(156,163,175,.55)}.caret-gray-400\/60{caret-color:rgba(156,163,175,.6)}.caret-gray-400\/65{caret-color:rgba(156,163,175,.65)}.caret-gray-400\/70{caret-color:rgba(156,163,175,.7)}.caret-gray-400\/75{caret-color:rgba(156,163,175,.75)}.caret-gray-400\/80{caret-color:rgba(156,163,175,.8)}.caret-gray-400\/85{caret-color:rgba(156,163,175,.85)}.caret-gray-400\/90{caret-color:rgba(156,163,175,.9)}.caret-gray-400\/95{caret-color:rgba(156,163,175,.95)}.caret-gray-50{caret-color:#f9fafb}.caret-gray-50\/0{caret-color:rgba(249,250,251,0)}.caret-gray-50\/10{caret-color:rgba(249,250,251,.1)}.caret-gray-50\/100{caret-color:#f9fafb}.caret-gray-50\/15{caret-color:rgba(249,250,251,.15)}.caret-gray-50\/20{caret-color:rgba(249,250,251,.2)}.caret-gray-50\/25{caret-color:rgba(249,250,251,.25)}.caret-gray-50\/30{caret-color:rgba(249,250,251,.3)}.caret-gray-50\/35{caret-color:rgba(249,250,251,.35)}.caret-gray-50\/40{caret-color:rgba(249,250,251,.4)}.caret-gray-50\/45{caret-color:rgba(249,250,251,.45)}.caret-gray-50\/5{caret-color:rgba(249,250,251,.05)}.caret-gray-50\/50{caret-color:rgba(249,250,251,.5)}.caret-gray-50\/55{caret-color:rgba(249,250,251,.55)}.caret-gray-50\/60{caret-color:rgba(249,250,251,.6)}.caret-gray-50\/65{caret-color:rgba(249,250,251,.65)}.caret-gray-50\/70{caret-color:rgba(249,250,251,.7)}.caret-gray-50\/75{caret-color:rgba(249,250,251,.75)}.caret-gray-50\/80{caret-color:rgba(249,250,251,.8)}.caret-gray-50\/85{caret-color:rgba(249,250,251,.85)}.caret-gray-50\/90{caret-color:rgba(249,250,251,.9)}.caret-gray-50\/95{caret-color:rgba(249,250,251,.95)}.caret-gray-500{caret-color:#6b7280}.caret-gray-500\/0{caret-color:hsla(220,9%,46%,0)}.caret-gray-500\/10{caret-color:hsla(220,9%,46%,.1)}.caret-gray-500\/100{caret-color:#6b7280}.caret-gray-500\/15{caret-color:hsla(220,9%,46%,.15)}.caret-gray-500\/20{caret-color:hsla(220,9%,46%,.2)}.caret-gray-500\/25{caret-color:hsla(220,9%,46%,.25)}.caret-gray-500\/30{caret-color:hsla(220,9%,46%,.3)}.caret-gray-500\/35{caret-color:hsla(220,9%,46%,.35)}.caret-gray-500\/40{caret-color:hsla(220,9%,46%,.4)}.caret-gray-500\/45{caret-color:hsla(220,9%,46%,.45)}.caret-gray-500\/5{caret-color:hsla(220,9%,46%,.05)}.caret-gray-500\/50{caret-color:hsla(220,9%,46%,.5)}.caret-gray-500\/55{caret-color:hsla(220,9%,46%,.55)}.caret-gray-500\/60{caret-color:hsla(220,9%,46%,.6)}.caret-gray-500\/65{caret-color:hsla(220,9%,46%,.65)}.caret-gray-500\/70{caret-color:hsla(220,9%,46%,.7)}.caret-gray-500\/75{caret-color:hsla(220,9%,46%,.75)}.caret-gray-500\/80{caret-color:hsla(220,9%,46%,.8)}.caret-gray-500\/85{caret-color:hsla(220,9%,46%,.85)}.caret-gray-500\/90{caret-color:hsla(220,9%,46%,.9)}.caret-gray-500\/95{caret-color:hsla(220,9%,46%,.95)}.caret-gray-950{caret-color:#030712}.caret-gray-950\/0{caret-color:rgba(3,7,18,0)}.caret-gray-950\/10{caret-color:rgba(3,7,18,.1)}.caret-gray-950\/100{caret-color:#030712}.caret-gray-950\/15{caret-color:rgba(3,7,18,.15)}.caret-gray-950\/20{caret-color:rgba(3,7,18,.2)}.caret-gray-950\/25{caret-color:rgba(3,7,18,.25)}.caret-gray-950\/30{caret-color:rgba(3,7,18,.3)}.caret-gray-950\/35{caret-color:rgba(3,7,18,.35)}.caret-gray-950\/40{caret-color:rgba(3,7,18,.4)}.caret-gray-950\/45{caret-color:rgba(3,7,18,.45)}.caret-gray-950\/5{caret-color:rgba(3,7,18,.05)}.caret-gray-950\/50{caret-color:rgba(3,7,18,.5)}.caret-gray-950\/55{caret-color:rgba(3,7,18,.55)}.caret-gray-950\/60{caret-color:rgba(3,7,18,.6)}.caret-gray-950\/65{caret-color:rgba(3,7,18,.65)}.caret-gray-950\/70{caret-color:rgba(3,7,18,.7)}.caret-gray-950\/75{caret-color:rgba(3,7,18,.75)}.caret-gray-950\/80{caret-color:rgba(3,7,18,.8)}.caret-gray-950\/85{caret-color:rgba(3,7,18,.85)}.caret-gray-950\/90{caret-color:rgba(3,7,18,.9)}.caret-gray-950\/95{caret-color:rgba(3,7,18,.95)}.caret-green-100{caret-color:#dcfce7}.caret-green-100\/0{caret-color:rgba(220,252,231,0)}.caret-green-100\/10{caret-color:rgba(220,252,231,.1)}.caret-green-100\/100{caret-color:#dcfce7}.caret-green-100\/15{caret-color:rgba(220,252,231,.15)}.caret-green-100\/20{caret-color:rgba(220,252,231,.2)}.caret-green-100\/25{caret-color:rgba(220,252,231,.25)}.caret-green-100\/30{caret-color:rgba(220,252,231,.3)}.caret-green-100\/35{caret-color:rgba(220,252,231,.35)}.caret-green-100\/40{caret-color:rgba(220,252,231,.4)}.caret-green-100\/45{caret-color:rgba(220,252,231,.45)}.caret-green-100\/5{caret-color:rgba(220,252,231,.05)}.caret-green-100\/50{caret-color:rgba(220,252,231,.5)}.caret-green-100\/55{caret-color:rgba(220,252,231,.55)}.caret-green-100\/60{caret-color:rgba(220,252,231,.6)}.caret-green-100\/65{caret-color:rgba(220,252,231,.65)}.caret-green-100\/70{caret-color:rgba(220,252,231,.7)}.caret-green-100\/75{caret-color:rgba(220,252,231,.75)}.caret-green-100\/80{caret-color:rgba(220,252,231,.8)}.caret-green-100\/85{caret-color:rgba(220,252,231,.85)}.caret-green-100\/90{caret-color:rgba(220,252,231,.9)}.caret-green-100\/95{caret-color:rgba(220,252,231,.95)}.caret-green-200{caret-color:#bbf7d0}.caret-green-200\/0{caret-color:rgba(187,247,208,0)}.caret-green-200\/10{caret-color:rgba(187,247,208,.1)}.caret-green-200\/100{caret-color:#bbf7d0}.caret-green-200\/15{caret-color:rgba(187,247,208,.15)}.caret-green-200\/20{caret-color:rgba(187,247,208,.2)}.caret-green-200\/25{caret-color:rgba(187,247,208,.25)}.caret-green-200\/30{caret-color:rgba(187,247,208,.3)}.caret-green-200\/35{caret-color:rgba(187,247,208,.35)}.caret-green-200\/40{caret-color:rgba(187,247,208,.4)}.caret-green-200\/45{caret-color:rgba(187,247,208,.45)}.caret-green-200\/5{caret-color:rgba(187,247,208,.05)}.caret-green-200\/50{caret-color:rgba(187,247,208,.5)}.caret-green-200\/55{caret-color:rgba(187,247,208,.55)}.caret-green-200\/60{caret-color:rgba(187,247,208,.6)}.caret-green-200\/65{caret-color:rgba(187,247,208,.65)}.caret-green-200\/70{caret-color:rgba(187,247,208,.7)}.caret-green-200\/75{caret-color:rgba(187,247,208,.75)}.caret-green-200\/80{caret-color:rgba(187,247,208,.8)}.caret-green-200\/85{caret-color:rgba(187,247,208,.85)}.caret-green-200\/90{caret-color:rgba(187,247,208,.9)}.caret-green-200\/95{caret-color:rgba(187,247,208,.95)}.caret-green-400{caret-color:#4ade80}.caret-green-400\/0{caret-color:rgba(74,222,128,0)}.caret-green-400\/10{caret-color:rgba(74,222,128,.1)}.caret-green-400\/100{caret-color:#4ade80}.caret-green-400\/15{caret-color:rgba(74,222,128,.15)}.caret-green-400\/20{caret-color:rgba(74,222,128,.2)}.caret-green-400\/25{caret-color:rgba(74,222,128,.25)}.caret-green-400\/30{caret-color:rgba(74,222,128,.3)}.caret-green-400\/35{caret-color:rgba(74,222,128,.35)}.caret-green-400\/40{caret-color:rgba(74,222,128,.4)}.caret-green-400\/45{caret-color:rgba(74,222,128,.45)}.caret-green-400\/5{caret-color:rgba(74,222,128,.05)}.caret-green-400\/50{caret-color:rgba(74,222,128,.5)}.caret-green-400\/55{caret-color:rgba(74,222,128,.55)}.caret-green-400\/60{caret-color:rgba(74,222,128,.6)}.caret-green-400\/65{caret-color:rgba(74,222,128,.65)}.caret-green-400\/70{caret-color:rgba(74,222,128,.7)}.caret-green-400\/75{caret-color:rgba(74,222,128,.75)}.caret-green-400\/80{caret-color:rgba(74,222,128,.8)}.caret-green-400\/85{caret-color:rgba(74,222,128,.85)}.caret-green-400\/90{caret-color:rgba(74,222,128,.9)}.caret-green-400\/95{caret-color:rgba(74,222,128,.95)}.caret-green-50{caret-color:#f0fdf4}.caret-green-50\/0{caret-color:rgba(240,253,244,0)}.caret-green-50\/10{caret-color:rgba(240,253,244,.1)}.caret-green-50\/100{caret-color:#f0fdf4}.caret-green-50\/15{caret-color:rgba(240,253,244,.15)}.caret-green-50\/20{caret-color:rgba(240,253,244,.2)}.caret-green-50\/25{caret-color:rgba(240,253,244,.25)}.caret-green-50\/30{caret-color:rgba(240,253,244,.3)}.caret-green-50\/35{caret-color:rgba(240,253,244,.35)}.caret-green-50\/40{caret-color:rgba(240,253,244,.4)}.caret-green-50\/45{caret-color:rgba(240,253,244,.45)}.caret-green-50\/5{caret-color:rgba(240,253,244,.05)}.caret-green-50\/50{caret-color:rgba(240,253,244,.5)}.caret-green-50\/55{caret-color:rgba(240,253,244,.55)}.caret-green-50\/60{caret-color:rgba(240,253,244,.6)}.caret-green-50\/65{caret-color:rgba(240,253,244,.65)}.caret-green-50\/70{caret-color:rgba(240,253,244,.7)}.caret-green-50\/75{caret-color:rgba(240,253,244,.75)}.caret-green-50\/80{caret-color:rgba(240,253,244,.8)}.caret-green-50\/85{caret-color:rgba(240,253,244,.85)}.caret-green-50\/90{caret-color:rgba(240,253,244,.9)}.caret-green-50\/95{caret-color:rgba(240,253,244,.95)}.caret-green-500{caret-color:#22c55e}.caret-green-500\/0{caret-color:rgba(34,197,94,0)}.caret-green-500\/10{caret-color:rgba(34,197,94,.1)}.caret-green-500\/100{caret-color:#22c55e}.caret-green-500\/15{caret-color:rgba(34,197,94,.15)}.caret-green-500\/20{caret-color:rgba(34,197,94,.2)}.caret-green-500\/25{caret-color:rgba(34,197,94,.25)}.caret-green-500\/30{caret-color:rgba(34,197,94,.3)}.caret-green-500\/35{caret-color:rgba(34,197,94,.35)}.caret-green-500\/40{caret-color:rgba(34,197,94,.4)}.caret-green-500\/45{caret-color:rgba(34,197,94,.45)}.caret-green-500\/5{caret-color:rgba(34,197,94,.05)}.caret-green-500\/50{caret-color:rgba(34,197,94,.5)}.caret-green-500\/55{caret-color:rgba(34,197,94,.55)}.caret-green-500\/60{caret-color:rgba(34,197,94,.6)}.caret-green-500\/65{caret-color:rgba(34,197,94,.65)}.caret-green-500\/70{caret-color:rgba(34,197,94,.7)}.caret-green-500\/75{caret-color:rgba(34,197,94,.75)}.caret-green-500\/80{caret-color:rgba(34,197,94,.8)}.caret-green-500\/85{caret-color:rgba(34,197,94,.85)}.caret-green-500\/90{caret-color:rgba(34,197,94,.9)}.caret-green-500\/95{caret-color:rgba(34,197,94,.95)}.caret-green-950{caret-color:#052e16}.caret-green-950\/0{caret-color:rgba(5,46,22,0)}.caret-green-950\/10{caret-color:rgba(5,46,22,.1)}.caret-green-950\/100{caret-color:#052e16}.caret-green-950\/15{caret-color:rgba(5,46,22,.15)}.caret-green-950\/20{caret-color:rgba(5,46,22,.2)}.caret-green-950\/25{caret-color:rgba(5,46,22,.25)}.caret-green-950\/30{caret-color:rgba(5,46,22,.3)}.caret-green-950\/35{caret-color:rgba(5,46,22,.35)}.caret-green-950\/40{caret-color:rgba(5,46,22,.4)}.caret-green-950\/45{caret-color:rgba(5,46,22,.45)}.caret-green-950\/5{caret-color:rgba(5,46,22,.05)}.caret-green-950\/50{caret-color:rgba(5,46,22,.5)}.caret-green-950\/55{caret-color:rgba(5,46,22,.55)}.caret-green-950\/60{caret-color:rgba(5,46,22,.6)}.caret-green-950\/65{caret-color:rgba(5,46,22,.65)}.caret-green-950\/70{caret-color:rgba(5,46,22,.7)}.caret-green-950\/75{caret-color:rgba(5,46,22,.75)}.caret-green-950\/80{caret-color:rgba(5,46,22,.8)}.caret-green-950\/85{caret-color:rgba(5,46,22,.85)}.caret-green-950\/90{caret-color:rgba(5,46,22,.9)}.caret-green-950\/95{caret-color:rgba(5,46,22,.95)}.caret-indigo-100{caret-color:#e0e7ff}.caret-indigo-100\/0{caret-color:rgba(224,231,255,0)}.caret-indigo-100\/10{caret-color:rgba(224,231,255,.1)}.caret-indigo-100\/100{caret-color:#e0e7ff}.caret-indigo-100\/15{caret-color:rgba(224,231,255,.15)}.caret-indigo-100\/20{caret-color:rgba(224,231,255,.2)}.caret-indigo-100\/25{caret-color:rgba(224,231,255,.25)}.caret-indigo-100\/30{caret-color:rgba(224,231,255,.3)}.caret-indigo-100\/35{caret-color:rgba(224,231,255,.35)}.caret-indigo-100\/40{caret-color:rgba(224,231,255,.4)}.caret-indigo-100\/45{caret-color:rgba(224,231,255,.45)}.caret-indigo-100\/5{caret-color:rgba(224,231,255,.05)}.caret-indigo-100\/50{caret-color:rgba(224,231,255,.5)}.caret-indigo-100\/55{caret-color:rgba(224,231,255,.55)}.caret-indigo-100\/60{caret-color:rgba(224,231,255,.6)}.caret-indigo-100\/65{caret-color:rgba(224,231,255,.65)}.caret-indigo-100\/70{caret-color:rgba(224,231,255,.7)}.caret-indigo-100\/75{caret-color:rgba(224,231,255,.75)}.caret-indigo-100\/80{caret-color:rgba(224,231,255,.8)}.caret-indigo-100\/85{caret-color:rgba(224,231,255,.85)}.caret-indigo-100\/90{caret-color:rgba(224,231,255,.9)}.caret-indigo-100\/95{caret-color:rgba(224,231,255,.95)}.caret-indigo-200{caret-color:#c7d2fe}.caret-indigo-200\/0{caret-color:rgba(199,210,254,0)}.caret-indigo-200\/10{caret-color:rgba(199,210,254,.1)}.caret-indigo-200\/100{caret-color:#c7d2fe}.caret-indigo-200\/15{caret-color:rgba(199,210,254,.15)}.caret-indigo-200\/20{caret-color:rgba(199,210,254,.2)}.caret-indigo-200\/25{caret-color:rgba(199,210,254,.25)}.caret-indigo-200\/30{caret-color:rgba(199,210,254,.3)}.caret-indigo-200\/35{caret-color:rgba(199,210,254,.35)}.caret-indigo-200\/40{caret-color:rgba(199,210,254,.4)}.caret-indigo-200\/45{caret-color:rgba(199,210,254,.45)}.caret-indigo-200\/5{caret-color:rgba(199,210,254,.05)}.caret-indigo-200\/50{caret-color:rgba(199,210,254,.5)}.caret-indigo-200\/55{caret-color:rgba(199,210,254,.55)}.caret-indigo-200\/60{caret-color:rgba(199,210,254,.6)}.caret-indigo-200\/65{caret-color:rgba(199,210,254,.65)}.caret-indigo-200\/70{caret-color:rgba(199,210,254,.7)}.caret-indigo-200\/75{caret-color:rgba(199,210,254,.75)}.caret-indigo-200\/80{caret-color:rgba(199,210,254,.8)}.caret-indigo-200\/85{caret-color:rgba(199,210,254,.85)}.caret-indigo-200\/90{caret-color:rgba(199,210,254,.9)}.caret-indigo-200\/95{caret-color:rgba(199,210,254,.95)}.caret-indigo-400{caret-color:#818cf8}.caret-indigo-400\/0{caret-color:rgba(129,140,248,0)}.caret-indigo-400\/10{caret-color:rgba(129,140,248,.1)}.caret-indigo-400\/100{caret-color:#818cf8}.caret-indigo-400\/15{caret-color:rgba(129,140,248,.15)}.caret-indigo-400\/20{caret-color:rgba(129,140,248,.2)}.caret-indigo-400\/25{caret-color:rgba(129,140,248,.25)}.caret-indigo-400\/30{caret-color:rgba(129,140,248,.3)}.caret-indigo-400\/35{caret-color:rgba(129,140,248,.35)}.caret-indigo-400\/40{caret-color:rgba(129,140,248,.4)}.caret-indigo-400\/45{caret-color:rgba(129,140,248,.45)}.caret-indigo-400\/5{caret-color:rgba(129,140,248,.05)}.caret-indigo-400\/50{caret-color:rgba(129,140,248,.5)}.caret-indigo-400\/55{caret-color:rgba(129,140,248,.55)}.caret-indigo-400\/60{caret-color:rgba(129,140,248,.6)}.caret-indigo-400\/65{caret-color:rgba(129,140,248,.65)}.caret-indigo-400\/70{caret-color:rgba(129,140,248,.7)}.caret-indigo-400\/75{caret-color:rgba(129,140,248,.75)}.caret-indigo-400\/80{caret-color:rgba(129,140,248,.8)}.caret-indigo-400\/85{caret-color:rgba(129,140,248,.85)}.caret-indigo-400\/90{caret-color:rgba(129,140,248,.9)}.caret-indigo-400\/95{caret-color:rgba(129,140,248,.95)}.caret-indigo-50{caret-color:#eef2ff}.caret-indigo-50\/0{caret-color:rgba(238,242,255,0)}.caret-indigo-50\/10{caret-color:rgba(238,242,255,.1)}.caret-indigo-50\/100{caret-color:#eef2ff}.caret-indigo-50\/15{caret-color:rgba(238,242,255,.15)}.caret-indigo-50\/20{caret-color:rgba(238,242,255,.2)}.caret-indigo-50\/25{caret-color:rgba(238,242,255,.25)}.caret-indigo-50\/30{caret-color:rgba(238,242,255,.3)}.caret-indigo-50\/35{caret-color:rgba(238,242,255,.35)}.caret-indigo-50\/40{caret-color:rgba(238,242,255,.4)}.caret-indigo-50\/45{caret-color:rgba(238,242,255,.45)}.caret-indigo-50\/5{caret-color:rgba(238,242,255,.05)}.caret-indigo-50\/50{caret-color:rgba(238,242,255,.5)}.caret-indigo-50\/55{caret-color:rgba(238,242,255,.55)}.caret-indigo-50\/60{caret-color:rgba(238,242,255,.6)}.caret-indigo-50\/65{caret-color:rgba(238,242,255,.65)}.caret-indigo-50\/70{caret-color:rgba(238,242,255,.7)}.caret-indigo-50\/75{caret-color:rgba(238,242,255,.75)}.caret-indigo-50\/80{caret-color:rgba(238,242,255,.8)}.caret-indigo-50\/85{caret-color:rgba(238,242,255,.85)}.caret-indigo-50\/90{caret-color:rgba(238,242,255,.9)}.caret-indigo-50\/95{caret-color:rgba(238,242,255,.95)}.caret-indigo-500{caret-color:#6366f1}.caret-indigo-500\/0{caret-color:rgba(99,102,241,0)}.caret-indigo-500\/10{caret-color:rgba(99,102,241,.1)}.caret-indigo-500\/100{caret-color:#6366f1}.caret-indigo-500\/15{caret-color:rgba(99,102,241,.15)}.caret-indigo-500\/20{caret-color:rgba(99,102,241,.2)}.caret-indigo-500\/25{caret-color:rgba(99,102,241,.25)}.caret-indigo-500\/30{caret-color:rgba(99,102,241,.3)}.caret-indigo-500\/35{caret-color:rgba(99,102,241,.35)}.caret-indigo-500\/40{caret-color:rgba(99,102,241,.4)}.caret-indigo-500\/45{caret-color:rgba(99,102,241,.45)}.caret-indigo-500\/5{caret-color:rgba(99,102,241,.05)}.caret-indigo-500\/50{caret-color:rgba(99,102,241,.5)}.caret-indigo-500\/55{caret-color:rgba(99,102,241,.55)}.caret-indigo-500\/60{caret-color:rgba(99,102,241,.6)}.caret-indigo-500\/65{caret-color:rgba(99,102,241,.65)}.caret-indigo-500\/70{caret-color:rgba(99,102,241,.7)}.caret-indigo-500\/75{caret-color:rgba(99,102,241,.75)}.caret-indigo-500\/80{caret-color:rgba(99,102,241,.8)}.caret-indigo-500\/85{caret-color:rgba(99,102,241,.85)}.caret-indigo-500\/90{caret-color:rgba(99,102,241,.9)}.caret-indigo-500\/95{caret-color:rgba(99,102,241,.95)}.caret-indigo-950{caret-color:#1e1b4b}.caret-indigo-950\/0{caret-color:rgba(30,27,75,0)}.caret-indigo-950\/10{caret-color:rgba(30,27,75,.1)}.caret-indigo-950\/100{caret-color:#1e1b4b}.caret-indigo-950\/15{caret-color:rgba(30,27,75,.15)}.caret-indigo-950\/20{caret-color:rgba(30,27,75,.2)}.caret-indigo-950\/25{caret-color:rgba(30,27,75,.25)}.caret-indigo-950\/30{caret-color:rgba(30,27,75,.3)}.caret-indigo-950\/35{caret-color:rgba(30,27,75,.35)}.caret-indigo-950\/40{caret-color:rgba(30,27,75,.4)}.caret-indigo-950\/45{caret-color:rgba(30,27,75,.45)}.caret-indigo-950\/5{caret-color:rgba(30,27,75,.05)}.caret-indigo-950\/50{caret-color:rgba(30,27,75,.5)}.caret-indigo-950\/55{caret-color:rgba(30,27,75,.55)}.caret-indigo-950\/60{caret-color:rgba(30,27,75,.6)}.caret-indigo-950\/65{caret-color:rgba(30,27,75,.65)}.caret-indigo-950\/70{caret-color:rgba(30,27,75,.7)}.caret-indigo-950\/75{caret-color:rgba(30,27,75,.75)}.caret-indigo-950\/80{caret-color:rgba(30,27,75,.8)}.caret-indigo-950\/85{caret-color:rgba(30,27,75,.85)}.caret-indigo-950\/90{caret-color:rgba(30,27,75,.9)}.caret-indigo-950\/95{caret-color:rgba(30,27,75,.95)}.caret-lime-100{caret-color:#ecfccb}.caret-lime-100\/0{caret-color:hsla(80,89%,89%,0)}.caret-lime-100\/10{caret-color:hsla(80,89%,89%,.1)}.caret-lime-100\/100{caret-color:#ecfccb}.caret-lime-100\/15{caret-color:hsla(80,89%,89%,.15)}.caret-lime-100\/20{caret-color:hsla(80,89%,89%,.2)}.caret-lime-100\/25{caret-color:hsla(80,89%,89%,.25)}.caret-lime-100\/30{caret-color:hsla(80,89%,89%,.3)}.caret-lime-100\/35{caret-color:hsla(80,89%,89%,.35)}.caret-lime-100\/40{caret-color:hsla(80,89%,89%,.4)}.caret-lime-100\/45{caret-color:hsla(80,89%,89%,.45)}.caret-lime-100\/5{caret-color:hsla(80,89%,89%,.05)}.caret-lime-100\/50{caret-color:hsla(80,89%,89%,.5)}.caret-lime-100\/55{caret-color:hsla(80,89%,89%,.55)}.caret-lime-100\/60{caret-color:hsla(80,89%,89%,.6)}.caret-lime-100\/65{caret-color:hsla(80,89%,89%,.65)}.caret-lime-100\/70{caret-color:hsla(80,89%,89%,.7)}.caret-lime-100\/75{caret-color:hsla(80,89%,89%,.75)}.caret-lime-100\/80{caret-color:hsla(80,89%,89%,.8)}.caret-lime-100\/85{caret-color:hsla(80,89%,89%,.85)}.caret-lime-100\/90{caret-color:hsla(80,89%,89%,.9)}.caret-lime-100\/95{caret-color:hsla(80,89%,89%,.95)}.caret-lime-200{caret-color:#d9f99d}.caret-lime-200\/0{caret-color:hsla(81,88%,80%,0)}.caret-lime-200\/10{caret-color:hsla(81,88%,80%,.1)}.caret-lime-200\/100{caret-color:#d9f99d}.caret-lime-200\/15{caret-color:hsla(81,88%,80%,.15)}.caret-lime-200\/20{caret-color:hsla(81,88%,80%,.2)}.caret-lime-200\/25{caret-color:hsla(81,88%,80%,.25)}.caret-lime-200\/30{caret-color:hsla(81,88%,80%,.3)}.caret-lime-200\/35{caret-color:hsla(81,88%,80%,.35)}.caret-lime-200\/40{caret-color:hsla(81,88%,80%,.4)}.caret-lime-200\/45{caret-color:hsla(81,88%,80%,.45)}.caret-lime-200\/5{caret-color:hsla(81,88%,80%,.05)}.caret-lime-200\/50{caret-color:hsla(81,88%,80%,.5)}.caret-lime-200\/55{caret-color:hsla(81,88%,80%,.55)}.caret-lime-200\/60{caret-color:hsla(81,88%,80%,.6)}.caret-lime-200\/65{caret-color:hsla(81,88%,80%,.65)}.caret-lime-200\/70{caret-color:hsla(81,88%,80%,.7)}.caret-lime-200\/75{caret-color:hsla(81,88%,80%,.75)}.caret-lime-200\/80{caret-color:hsla(81,88%,80%,.8)}.caret-lime-200\/85{caret-color:hsla(81,88%,80%,.85)}.caret-lime-200\/90{caret-color:hsla(81,88%,80%,.9)}.caret-lime-200\/95{caret-color:hsla(81,88%,80%,.95)}.caret-lime-400{caret-color:#a3e635}.caret-lime-400\/0{caret-color:rgba(163,230,53,0)}.caret-lime-400\/10{caret-color:rgba(163,230,53,.1)}.caret-lime-400\/100{caret-color:#a3e635}.caret-lime-400\/15{caret-color:rgba(163,230,53,.15)}.caret-lime-400\/20{caret-color:rgba(163,230,53,.2)}.caret-lime-400\/25{caret-color:rgba(163,230,53,.25)}.caret-lime-400\/30{caret-color:rgba(163,230,53,.3)}.caret-lime-400\/35{caret-color:rgba(163,230,53,.35)}.caret-lime-400\/40{caret-color:rgba(163,230,53,.4)}.caret-lime-400\/45{caret-color:rgba(163,230,53,.45)}.caret-lime-400\/5{caret-color:rgba(163,230,53,.05)}.caret-lime-400\/50{caret-color:rgba(163,230,53,.5)}.caret-lime-400\/55{caret-color:rgba(163,230,53,.55)}.caret-lime-400\/60{caret-color:rgba(163,230,53,.6)}.caret-lime-400\/65{caret-color:rgba(163,230,53,.65)}.caret-lime-400\/70{caret-color:rgba(163,230,53,.7)}.caret-lime-400\/75{caret-color:rgba(163,230,53,.75)}.caret-lime-400\/80{caret-color:rgba(163,230,53,.8)}.caret-lime-400\/85{caret-color:rgba(163,230,53,.85)}.caret-lime-400\/90{caret-color:rgba(163,230,53,.9)}.caret-lime-400\/95{caret-color:rgba(163,230,53,.95)}.caret-lime-50{caret-color:#f7fee7}.caret-lime-50\/0{caret-color:hsla(78,92%,95%,0)}.caret-lime-50\/10{caret-color:hsla(78,92%,95%,.1)}.caret-lime-50\/100{caret-color:#f7fee7}.caret-lime-50\/15{caret-color:hsla(78,92%,95%,.15)}.caret-lime-50\/20{caret-color:hsla(78,92%,95%,.2)}.caret-lime-50\/25{caret-color:hsla(78,92%,95%,.25)}.caret-lime-50\/30{caret-color:hsla(78,92%,95%,.3)}.caret-lime-50\/35{caret-color:hsla(78,92%,95%,.35)}.caret-lime-50\/40{caret-color:hsla(78,92%,95%,.4)}.caret-lime-50\/45{caret-color:hsla(78,92%,95%,.45)}.caret-lime-50\/5{caret-color:hsla(78,92%,95%,.05)}.caret-lime-50\/50{caret-color:hsla(78,92%,95%,.5)}.caret-lime-50\/55{caret-color:hsla(78,92%,95%,.55)}.caret-lime-50\/60{caret-color:hsla(78,92%,95%,.6)}.caret-lime-50\/65{caret-color:hsla(78,92%,95%,.65)}.caret-lime-50\/70{caret-color:hsla(78,92%,95%,.7)}.caret-lime-50\/75{caret-color:hsla(78,92%,95%,.75)}.caret-lime-50\/80{caret-color:hsla(78,92%,95%,.8)}.caret-lime-50\/85{caret-color:hsla(78,92%,95%,.85)}.caret-lime-50\/90{caret-color:hsla(78,92%,95%,.9)}.caret-lime-50\/95{caret-color:hsla(78,92%,95%,.95)}.caret-lime-500{caret-color:#84cc16}.caret-lime-500\/0{caret-color:rgba(132,204,22,0)}.caret-lime-500\/10{caret-color:rgba(132,204,22,.1)}.caret-lime-500\/100{caret-color:#84cc16}.caret-lime-500\/15{caret-color:rgba(132,204,22,.15)}.caret-lime-500\/20{caret-color:rgba(132,204,22,.2)}.caret-lime-500\/25{caret-color:rgba(132,204,22,.25)}.caret-lime-500\/30{caret-color:rgba(132,204,22,.3)}.caret-lime-500\/35{caret-color:rgba(132,204,22,.35)}.caret-lime-500\/40{caret-color:rgba(132,204,22,.4)}.caret-lime-500\/45{caret-color:rgba(132,204,22,.45)}.caret-lime-500\/5{caret-color:rgba(132,204,22,.05)}.caret-lime-500\/50{caret-color:rgba(132,204,22,.5)}.caret-lime-500\/55{caret-color:rgba(132,204,22,.55)}.caret-lime-500\/60{caret-color:rgba(132,204,22,.6)}.caret-lime-500\/65{caret-color:rgba(132,204,22,.65)}.caret-lime-500\/70{caret-color:rgba(132,204,22,.7)}.caret-lime-500\/75{caret-color:rgba(132,204,22,.75)}.caret-lime-500\/80{caret-color:rgba(132,204,22,.8)}.caret-lime-500\/85{caret-color:rgba(132,204,22,.85)}.caret-lime-500\/90{caret-color:rgba(132,204,22,.9)}.caret-lime-500\/95{caret-color:rgba(132,204,22,.95)}.caret-lime-950{caret-color:#1a2e05}.caret-lime-950\/0{caret-color:rgba(26,46,5,0)}.caret-lime-950\/10{caret-color:rgba(26,46,5,.1)}.caret-lime-950\/100{caret-color:#1a2e05}.caret-lime-950\/15{caret-color:rgba(26,46,5,.15)}.caret-lime-950\/20{caret-color:rgba(26,46,5,.2)}.caret-lime-950\/25{caret-color:rgba(26,46,5,.25)}.caret-lime-950\/30{caret-color:rgba(26,46,5,.3)}.caret-lime-950\/35{caret-color:rgba(26,46,5,.35)}.caret-lime-950\/40{caret-color:rgba(26,46,5,.4)}.caret-lime-950\/45{caret-color:rgba(26,46,5,.45)}.caret-lime-950\/5{caret-color:rgba(26,46,5,.05)}.caret-lime-950\/50{caret-color:rgba(26,46,5,.5)}.caret-lime-950\/55{caret-color:rgba(26,46,5,.55)}.caret-lime-950\/60{caret-color:rgba(26,46,5,.6)}.caret-lime-950\/65{caret-color:rgba(26,46,5,.65)}.caret-lime-950\/70{caret-color:rgba(26,46,5,.7)}.caret-lime-950\/75{caret-color:rgba(26,46,5,.75)}.caret-lime-950\/80{caret-color:rgba(26,46,5,.8)}.caret-lime-950\/85{caret-color:rgba(26,46,5,.85)}.caret-lime-950\/90{caret-color:rgba(26,46,5,.9)}.caret-lime-950\/95{caret-color:rgba(26,46,5,.95)}.caret-neutral-100{caret-color:#f5f5f5}.caret-neutral-100\/0{caret-color:hsla(0,0%,96%,0)}.caret-neutral-100\/10{caret-color:hsla(0,0%,96%,.1)}.caret-neutral-100\/100{caret-color:#f5f5f5}.caret-neutral-100\/15{caret-color:hsla(0,0%,96%,.15)}.caret-neutral-100\/20{caret-color:hsla(0,0%,96%,.2)}.caret-neutral-100\/25{caret-color:hsla(0,0%,96%,.25)}.caret-neutral-100\/30{caret-color:hsla(0,0%,96%,.3)}.caret-neutral-100\/35{caret-color:hsla(0,0%,96%,.35)}.caret-neutral-100\/40{caret-color:hsla(0,0%,96%,.4)}.caret-neutral-100\/45{caret-color:hsla(0,0%,96%,.45)}.caret-neutral-100\/5{caret-color:hsla(0,0%,96%,.05)}.caret-neutral-100\/50{caret-color:hsla(0,0%,96%,.5)}.caret-neutral-100\/55{caret-color:hsla(0,0%,96%,.55)}.caret-neutral-100\/60{caret-color:hsla(0,0%,96%,.6)}.caret-neutral-100\/65{caret-color:hsla(0,0%,96%,.65)}.caret-neutral-100\/70{caret-color:hsla(0,0%,96%,.7)}.caret-neutral-100\/75{caret-color:hsla(0,0%,96%,.75)}.caret-neutral-100\/80{caret-color:hsla(0,0%,96%,.8)}.caret-neutral-100\/85{caret-color:hsla(0,0%,96%,.85)}.caret-neutral-100\/90{caret-color:hsla(0,0%,96%,.9)}.caret-neutral-100\/95{caret-color:hsla(0,0%,96%,.95)}.caret-neutral-200{caret-color:#e5e5e5}.caret-neutral-200\/0{caret-color:hsla(0,0%,90%,0)}.caret-neutral-200\/10{caret-color:hsla(0,0%,90%,.1)}.caret-neutral-200\/100{caret-color:#e5e5e5}.caret-neutral-200\/15{caret-color:hsla(0,0%,90%,.15)}.caret-neutral-200\/20{caret-color:hsla(0,0%,90%,.2)}.caret-neutral-200\/25{caret-color:hsla(0,0%,90%,.25)}.caret-neutral-200\/30{caret-color:hsla(0,0%,90%,.3)}.caret-neutral-200\/35{caret-color:hsla(0,0%,90%,.35)}.caret-neutral-200\/40{caret-color:hsla(0,0%,90%,.4)}.caret-neutral-200\/45{caret-color:hsla(0,0%,90%,.45)}.caret-neutral-200\/5{caret-color:hsla(0,0%,90%,.05)}.caret-neutral-200\/50{caret-color:hsla(0,0%,90%,.5)}.caret-neutral-200\/55{caret-color:hsla(0,0%,90%,.55)}.caret-neutral-200\/60{caret-color:hsla(0,0%,90%,.6)}.caret-neutral-200\/65{caret-color:hsla(0,0%,90%,.65)}.caret-neutral-200\/70{caret-color:hsla(0,0%,90%,.7)}.caret-neutral-200\/75{caret-color:hsla(0,0%,90%,.75)}.caret-neutral-200\/80{caret-color:hsla(0,0%,90%,.8)}.caret-neutral-200\/85{caret-color:hsla(0,0%,90%,.85)}.caret-neutral-200\/90{caret-color:hsla(0,0%,90%,.9)}.caret-neutral-200\/95{caret-color:hsla(0,0%,90%,.95)}.caret-neutral-400{caret-color:#a3a3a3}.caret-neutral-400\/0{caret-color:hsla(0,0%,64%,0)}.caret-neutral-400\/10{caret-color:hsla(0,0%,64%,.1)}.caret-neutral-400\/100{caret-color:#a3a3a3}.caret-neutral-400\/15{caret-color:hsla(0,0%,64%,.15)}.caret-neutral-400\/20{caret-color:hsla(0,0%,64%,.2)}.caret-neutral-400\/25{caret-color:hsla(0,0%,64%,.25)}.caret-neutral-400\/30{caret-color:hsla(0,0%,64%,.3)}.caret-neutral-400\/35{caret-color:hsla(0,0%,64%,.35)}.caret-neutral-400\/40{caret-color:hsla(0,0%,64%,.4)}.caret-neutral-400\/45{caret-color:hsla(0,0%,64%,.45)}.caret-neutral-400\/5{caret-color:hsla(0,0%,64%,.05)}.caret-neutral-400\/50{caret-color:hsla(0,0%,64%,.5)}.caret-neutral-400\/55{caret-color:hsla(0,0%,64%,.55)}.caret-neutral-400\/60{caret-color:hsla(0,0%,64%,.6)}.caret-neutral-400\/65{caret-color:hsla(0,0%,64%,.65)}.caret-neutral-400\/70{caret-color:hsla(0,0%,64%,.7)}.caret-neutral-400\/75{caret-color:hsla(0,0%,64%,.75)}.caret-neutral-400\/80{caret-color:hsla(0,0%,64%,.8)}.caret-neutral-400\/85{caret-color:hsla(0,0%,64%,.85)}.caret-neutral-400\/90{caret-color:hsla(0,0%,64%,.9)}.caret-neutral-400\/95{caret-color:hsla(0,0%,64%,.95)}.caret-neutral-50{caret-color:#fafafa}.caret-neutral-50\/0{caret-color:hsla(0,0%,98%,0)}.caret-neutral-50\/10{caret-color:hsla(0,0%,98%,.1)}.caret-neutral-50\/100{caret-color:#fafafa}.caret-neutral-50\/15{caret-color:hsla(0,0%,98%,.15)}.caret-neutral-50\/20{caret-color:hsla(0,0%,98%,.2)}.caret-neutral-50\/25{caret-color:hsla(0,0%,98%,.25)}.caret-neutral-50\/30{caret-color:hsla(0,0%,98%,.3)}.caret-neutral-50\/35{caret-color:hsla(0,0%,98%,.35)}.caret-neutral-50\/40{caret-color:hsla(0,0%,98%,.4)}.caret-neutral-50\/45{caret-color:hsla(0,0%,98%,.45)}.caret-neutral-50\/5{caret-color:hsla(0,0%,98%,.05)}.caret-neutral-50\/50{caret-color:hsla(0,0%,98%,.5)}.caret-neutral-50\/55{caret-color:hsla(0,0%,98%,.55)}.caret-neutral-50\/60{caret-color:hsla(0,0%,98%,.6)}.caret-neutral-50\/65{caret-color:hsla(0,0%,98%,.65)}.caret-neutral-50\/70{caret-color:hsla(0,0%,98%,.7)}.caret-neutral-50\/75{caret-color:hsla(0,0%,98%,.75)}.caret-neutral-50\/80{caret-color:hsla(0,0%,98%,.8)}.caret-neutral-50\/85{caret-color:hsla(0,0%,98%,.85)}.caret-neutral-50\/90{caret-color:hsla(0,0%,98%,.9)}.caret-neutral-50\/95{caret-color:hsla(0,0%,98%,.95)}.caret-neutral-500{caret-color:#737373}.caret-neutral-500\/0{caret-color:hsla(0,0%,45%,0)}.caret-neutral-500\/10{caret-color:hsla(0,0%,45%,.1)}.caret-neutral-500\/100{caret-color:#737373}.caret-neutral-500\/15{caret-color:hsla(0,0%,45%,.15)}.caret-neutral-500\/20{caret-color:hsla(0,0%,45%,.2)}.caret-neutral-500\/25{caret-color:hsla(0,0%,45%,.25)}.caret-neutral-500\/30{caret-color:hsla(0,0%,45%,.3)}.caret-neutral-500\/35{caret-color:hsla(0,0%,45%,.35)}.caret-neutral-500\/40{caret-color:hsla(0,0%,45%,.4)}.caret-neutral-500\/45{caret-color:hsla(0,0%,45%,.45)}.caret-neutral-500\/5{caret-color:hsla(0,0%,45%,.05)}.caret-neutral-500\/50{caret-color:hsla(0,0%,45%,.5)}.caret-neutral-500\/55{caret-color:hsla(0,0%,45%,.55)}.caret-neutral-500\/60{caret-color:hsla(0,0%,45%,.6)}.caret-neutral-500\/65{caret-color:hsla(0,0%,45%,.65)}.caret-neutral-500\/70{caret-color:hsla(0,0%,45%,.7)}.caret-neutral-500\/75{caret-color:hsla(0,0%,45%,.75)}.caret-neutral-500\/80{caret-color:hsla(0,0%,45%,.8)}.caret-neutral-500\/85{caret-color:hsla(0,0%,45%,.85)}.caret-neutral-500\/90{caret-color:hsla(0,0%,45%,.9)}.caret-neutral-500\/95{caret-color:hsla(0,0%,45%,.95)}.caret-neutral-950{caret-color:#0a0a0a}.caret-neutral-950\/0{caret-color:hsla(0,0%,4%,0)}.caret-neutral-950\/10{caret-color:hsla(0,0%,4%,.1)}.caret-neutral-950\/100{caret-color:#0a0a0a}.caret-neutral-950\/15{caret-color:hsla(0,0%,4%,.15)}.caret-neutral-950\/20{caret-color:hsla(0,0%,4%,.2)}.caret-neutral-950\/25{caret-color:hsla(0,0%,4%,.25)}.caret-neutral-950\/30{caret-color:hsla(0,0%,4%,.3)}.caret-neutral-950\/35{caret-color:hsla(0,0%,4%,.35)}.caret-neutral-950\/40{caret-color:hsla(0,0%,4%,.4)}.caret-neutral-950\/45{caret-color:hsla(0,0%,4%,.45)}.caret-neutral-950\/5{caret-color:hsla(0,0%,4%,.05)}.caret-neutral-950\/50{caret-color:hsla(0,0%,4%,.5)}.caret-neutral-950\/55{caret-color:hsla(0,0%,4%,.55)}.caret-neutral-950\/60{caret-color:hsla(0,0%,4%,.6)}.caret-neutral-950\/65{caret-color:hsla(0,0%,4%,.65)}.caret-neutral-950\/70{caret-color:hsla(0,0%,4%,.7)}.caret-neutral-950\/75{caret-color:hsla(0,0%,4%,.75)}.caret-neutral-950\/80{caret-color:hsla(0,0%,4%,.8)}.caret-neutral-950\/85{caret-color:hsla(0,0%,4%,.85)}.caret-neutral-950\/90{caret-color:hsla(0,0%,4%,.9)}.caret-neutral-950\/95{caret-color:hsla(0,0%,4%,.95)}.caret-orange-100{caret-color:#ffedd5}.caret-orange-100\/0{caret-color:rgba(255,237,213,0)}.caret-orange-100\/10{caret-color:rgba(255,237,213,.1)}.caret-orange-100\/100{caret-color:#ffedd5}.caret-orange-100\/15{caret-color:rgba(255,237,213,.15)}.caret-orange-100\/20{caret-color:rgba(255,237,213,.2)}.caret-orange-100\/25{caret-color:rgba(255,237,213,.25)}.caret-orange-100\/30{caret-color:rgba(255,237,213,.3)}.caret-orange-100\/35{caret-color:rgba(255,237,213,.35)}.caret-orange-100\/40{caret-color:rgba(255,237,213,.4)}.caret-orange-100\/45{caret-color:rgba(255,237,213,.45)}.caret-orange-100\/5{caret-color:rgba(255,237,213,.05)}.caret-orange-100\/50{caret-color:rgba(255,237,213,.5)}.caret-orange-100\/55{caret-color:rgba(255,237,213,.55)}.caret-orange-100\/60{caret-color:rgba(255,237,213,.6)}.caret-orange-100\/65{caret-color:rgba(255,237,213,.65)}.caret-orange-100\/70{caret-color:rgba(255,237,213,.7)}.caret-orange-100\/75{caret-color:rgba(255,237,213,.75)}.caret-orange-100\/80{caret-color:rgba(255,237,213,.8)}.caret-orange-100\/85{caret-color:rgba(255,237,213,.85)}.caret-orange-100\/90{caret-color:rgba(255,237,213,.9)}.caret-orange-100\/95{caret-color:rgba(255,237,213,.95)}.caret-orange-200{caret-color:#fed7aa}.caret-orange-200\/0{caret-color:hsla(32,98%,83%,0)}.caret-orange-200\/10{caret-color:hsla(32,98%,83%,.1)}.caret-orange-200\/100{caret-color:#fed7aa}.caret-orange-200\/15{caret-color:hsla(32,98%,83%,.15)}.caret-orange-200\/20{caret-color:hsla(32,98%,83%,.2)}.caret-orange-200\/25{caret-color:hsla(32,98%,83%,.25)}.caret-orange-200\/30{caret-color:hsla(32,98%,83%,.3)}.caret-orange-200\/35{caret-color:hsla(32,98%,83%,.35)}.caret-orange-200\/40{caret-color:hsla(32,98%,83%,.4)}.caret-orange-200\/45{caret-color:hsla(32,98%,83%,.45)}.caret-orange-200\/5{caret-color:hsla(32,98%,83%,.05)}.caret-orange-200\/50{caret-color:hsla(32,98%,83%,.5)}.caret-orange-200\/55{caret-color:hsla(32,98%,83%,.55)}.caret-orange-200\/60{caret-color:hsla(32,98%,83%,.6)}.caret-orange-200\/65{caret-color:hsla(32,98%,83%,.65)}.caret-orange-200\/70{caret-color:hsla(32,98%,83%,.7)}.caret-orange-200\/75{caret-color:hsla(32,98%,83%,.75)}.caret-orange-200\/80{caret-color:hsla(32,98%,83%,.8)}.caret-orange-200\/85{caret-color:hsla(32,98%,83%,.85)}.caret-orange-200\/90{caret-color:hsla(32,98%,83%,.9)}.caret-orange-200\/95{caret-color:hsla(32,98%,83%,.95)}.caret-orange-400{caret-color:#fb923c}.caret-orange-400\/0{caret-color:rgba(251,146,60,0)}.caret-orange-400\/10{caret-color:rgba(251,146,60,.1)}.caret-orange-400\/100{caret-color:#fb923c}.caret-orange-400\/15{caret-color:rgba(251,146,60,.15)}.caret-orange-400\/20{caret-color:rgba(251,146,60,.2)}.caret-orange-400\/25{caret-color:rgba(251,146,60,.25)}.caret-orange-400\/30{caret-color:rgba(251,146,60,.3)}.caret-orange-400\/35{caret-color:rgba(251,146,60,.35)}.caret-orange-400\/40{caret-color:rgba(251,146,60,.4)}.caret-orange-400\/45{caret-color:rgba(251,146,60,.45)}.caret-orange-400\/5{caret-color:rgba(251,146,60,.05)}.caret-orange-400\/50{caret-color:rgba(251,146,60,.5)}.caret-orange-400\/55{caret-color:rgba(251,146,60,.55)}.caret-orange-400\/60{caret-color:rgba(251,146,60,.6)}.caret-orange-400\/65{caret-color:rgba(251,146,60,.65)}.caret-orange-400\/70{caret-color:rgba(251,146,60,.7)}.caret-orange-400\/75{caret-color:rgba(251,146,60,.75)}.caret-orange-400\/80{caret-color:rgba(251,146,60,.8)}.caret-orange-400\/85{caret-color:rgba(251,146,60,.85)}.caret-orange-400\/90{caret-color:rgba(251,146,60,.9)}.caret-orange-400\/95{caret-color:rgba(251,146,60,.95)}.caret-orange-50{caret-color:#fff7ed}.caret-orange-50\/0{caret-color:rgba(255,247,237,0)}.caret-orange-50\/10{caret-color:rgba(255,247,237,.1)}.caret-orange-50\/100{caret-color:#fff7ed}.caret-orange-50\/15{caret-color:rgba(255,247,237,.15)}.caret-orange-50\/20{caret-color:rgba(255,247,237,.2)}.caret-orange-50\/25{caret-color:rgba(255,247,237,.25)}.caret-orange-50\/30{caret-color:rgba(255,247,237,.3)}.caret-orange-50\/35{caret-color:rgba(255,247,237,.35)}.caret-orange-50\/40{caret-color:rgba(255,247,237,.4)}.caret-orange-50\/45{caret-color:rgba(255,247,237,.45)}.caret-orange-50\/5{caret-color:rgba(255,247,237,.05)}.caret-orange-50\/50{caret-color:rgba(255,247,237,.5)}.caret-orange-50\/55{caret-color:rgba(255,247,237,.55)}.caret-orange-50\/60{caret-color:rgba(255,247,237,.6)}.caret-orange-50\/65{caret-color:rgba(255,247,237,.65)}.caret-orange-50\/70{caret-color:rgba(255,247,237,.7)}.caret-orange-50\/75{caret-color:rgba(255,247,237,.75)}.caret-orange-50\/80{caret-color:rgba(255,247,237,.8)}.caret-orange-50\/85{caret-color:rgba(255,247,237,.85)}.caret-orange-50\/90{caret-color:rgba(255,247,237,.9)}.caret-orange-50\/95{caret-color:rgba(255,247,237,.95)}.caret-orange-500{caret-color:#f97316}.caret-orange-500\/0{caret-color:rgba(249,115,22,0)}.caret-orange-500\/10{caret-color:rgba(249,115,22,.1)}.caret-orange-500\/100{caret-color:#f97316}.caret-orange-500\/15{caret-color:rgba(249,115,22,.15)}.caret-orange-500\/20{caret-color:rgba(249,115,22,.2)}.caret-orange-500\/25{caret-color:rgba(249,115,22,.25)}.caret-orange-500\/30{caret-color:rgba(249,115,22,.3)}.caret-orange-500\/35{caret-color:rgba(249,115,22,.35)}.caret-orange-500\/40{caret-color:rgba(249,115,22,.4)}.caret-orange-500\/45{caret-color:rgba(249,115,22,.45)}.caret-orange-500\/5{caret-color:rgba(249,115,22,.05)}.caret-orange-500\/50{caret-color:rgba(249,115,22,.5)}.caret-orange-500\/55{caret-color:rgba(249,115,22,.55)}.caret-orange-500\/60{caret-color:rgba(249,115,22,.6)}.caret-orange-500\/65{caret-color:rgba(249,115,22,.65)}.caret-orange-500\/70{caret-color:rgba(249,115,22,.7)}.caret-orange-500\/75{caret-color:rgba(249,115,22,.75)}.caret-orange-500\/80{caret-color:rgba(249,115,22,.8)}.caret-orange-500\/85{caret-color:rgba(249,115,22,.85)}.caret-orange-500\/90{caret-color:rgba(249,115,22,.9)}.caret-orange-500\/95{caret-color:rgba(249,115,22,.95)}.caret-orange-950{caret-color:#431407}.caret-orange-950\/0{caret-color:rgba(67,20,7,0)}.caret-orange-950\/10{caret-color:rgba(67,20,7,.1)}.caret-orange-950\/100{caret-color:#431407}.caret-orange-950\/15{caret-color:rgba(67,20,7,.15)}.caret-orange-950\/20{caret-color:rgba(67,20,7,.2)}.caret-orange-950\/25{caret-color:rgba(67,20,7,.25)}.caret-orange-950\/30{caret-color:rgba(67,20,7,.3)}.caret-orange-950\/35{caret-color:rgba(67,20,7,.35)}.caret-orange-950\/40{caret-color:rgba(67,20,7,.4)}.caret-orange-950\/45{caret-color:rgba(67,20,7,.45)}.caret-orange-950\/5{caret-color:rgba(67,20,7,.05)}.caret-orange-950\/50{caret-color:rgba(67,20,7,.5)}.caret-orange-950\/55{caret-color:rgba(67,20,7,.55)}.caret-orange-950\/60{caret-color:rgba(67,20,7,.6)}.caret-orange-950\/65{caret-color:rgba(67,20,7,.65)}.caret-orange-950\/70{caret-color:rgba(67,20,7,.7)}.caret-orange-950\/75{caret-color:rgba(67,20,7,.75)}.caret-orange-950\/80{caret-color:rgba(67,20,7,.8)}.caret-orange-950\/85{caret-color:rgba(67,20,7,.85)}.caret-orange-950\/90{caret-color:rgba(67,20,7,.9)}.caret-orange-950\/95{caret-color:rgba(67,20,7,.95)}.caret-pink-100{caret-color:#fce7f3}.caret-pink-100\/0{caret-color:rgba(252,231,243,0)}.caret-pink-100\/10{caret-color:rgba(252,231,243,.1)}.caret-pink-100\/100{caret-color:#fce7f3}.caret-pink-100\/15{caret-color:rgba(252,231,243,.15)}.caret-pink-100\/20{caret-color:rgba(252,231,243,.2)}.caret-pink-100\/25{caret-color:rgba(252,231,243,.25)}.caret-pink-100\/30{caret-color:rgba(252,231,243,.3)}.caret-pink-100\/35{caret-color:rgba(252,231,243,.35)}.caret-pink-100\/40{caret-color:rgba(252,231,243,.4)}.caret-pink-100\/45{caret-color:rgba(252,231,243,.45)}.caret-pink-100\/5{caret-color:rgba(252,231,243,.05)}.caret-pink-100\/50{caret-color:rgba(252,231,243,.5)}.caret-pink-100\/55{caret-color:rgba(252,231,243,.55)}.caret-pink-100\/60{caret-color:rgba(252,231,243,.6)}.caret-pink-100\/65{caret-color:rgba(252,231,243,.65)}.caret-pink-100\/70{caret-color:rgba(252,231,243,.7)}.caret-pink-100\/75{caret-color:rgba(252,231,243,.75)}.caret-pink-100\/80{caret-color:rgba(252,231,243,.8)}.caret-pink-100\/85{caret-color:rgba(252,231,243,.85)}.caret-pink-100\/90{caret-color:rgba(252,231,243,.9)}.caret-pink-100\/95{caret-color:rgba(252,231,243,.95)}.caret-pink-200{caret-color:#fbcfe8}.caret-pink-200\/0{caret-color:rgba(251,207,232,0)}.caret-pink-200\/10{caret-color:rgba(251,207,232,.1)}.caret-pink-200\/100{caret-color:#fbcfe8}.caret-pink-200\/15{caret-color:rgba(251,207,232,.15)}.caret-pink-200\/20{caret-color:rgba(251,207,232,.2)}.caret-pink-200\/25{caret-color:rgba(251,207,232,.25)}.caret-pink-200\/30{caret-color:rgba(251,207,232,.3)}.caret-pink-200\/35{caret-color:rgba(251,207,232,.35)}.caret-pink-200\/40{caret-color:rgba(251,207,232,.4)}.caret-pink-200\/45{caret-color:rgba(251,207,232,.45)}.caret-pink-200\/5{caret-color:rgba(251,207,232,.05)}.caret-pink-200\/50{caret-color:rgba(251,207,232,.5)}.caret-pink-200\/55{caret-color:rgba(251,207,232,.55)}.caret-pink-200\/60{caret-color:rgba(251,207,232,.6)}.caret-pink-200\/65{caret-color:rgba(251,207,232,.65)}.caret-pink-200\/70{caret-color:rgba(251,207,232,.7)}.caret-pink-200\/75{caret-color:rgba(251,207,232,.75)}.caret-pink-200\/80{caret-color:rgba(251,207,232,.8)}.caret-pink-200\/85{caret-color:rgba(251,207,232,.85)}.caret-pink-200\/90{caret-color:rgba(251,207,232,.9)}.caret-pink-200\/95{caret-color:rgba(251,207,232,.95)}.caret-pink-400{caret-color:#f472b6}.caret-pink-400\/0{caret-color:rgba(244,114,182,0)}.caret-pink-400\/10{caret-color:rgba(244,114,182,.1)}.caret-pink-400\/100{caret-color:#f472b6}.caret-pink-400\/15{caret-color:rgba(244,114,182,.15)}.caret-pink-400\/20{caret-color:rgba(244,114,182,.2)}.caret-pink-400\/25{caret-color:rgba(244,114,182,.25)}.caret-pink-400\/30{caret-color:rgba(244,114,182,.3)}.caret-pink-400\/35{caret-color:rgba(244,114,182,.35)}.caret-pink-400\/40{caret-color:rgba(244,114,182,.4)}.caret-pink-400\/45{caret-color:rgba(244,114,182,.45)}.caret-pink-400\/5{caret-color:rgba(244,114,182,.05)}.caret-pink-400\/50{caret-color:rgba(244,114,182,.5)}.caret-pink-400\/55{caret-color:rgba(244,114,182,.55)}.caret-pink-400\/60{caret-color:rgba(244,114,182,.6)}.caret-pink-400\/65{caret-color:rgba(244,114,182,.65)}.caret-pink-400\/70{caret-color:rgba(244,114,182,.7)}.caret-pink-400\/75{caret-color:rgba(244,114,182,.75)}.caret-pink-400\/80{caret-color:rgba(244,114,182,.8)}.caret-pink-400\/85{caret-color:rgba(244,114,182,.85)}.caret-pink-400\/90{caret-color:rgba(244,114,182,.9)}.caret-pink-400\/95{caret-color:rgba(244,114,182,.95)}.caret-pink-50{caret-color:#fdf2f8}.caret-pink-50\/0{caret-color:rgba(253,242,248,0)}.caret-pink-50\/10{caret-color:rgba(253,242,248,.1)}.caret-pink-50\/100{caret-color:#fdf2f8}.caret-pink-50\/15{caret-color:rgba(253,242,248,.15)}.caret-pink-50\/20{caret-color:rgba(253,242,248,.2)}.caret-pink-50\/25{caret-color:rgba(253,242,248,.25)}.caret-pink-50\/30{caret-color:rgba(253,242,248,.3)}.caret-pink-50\/35{caret-color:rgba(253,242,248,.35)}.caret-pink-50\/40{caret-color:rgba(253,242,248,.4)}.caret-pink-50\/45{caret-color:rgba(253,242,248,.45)}.caret-pink-50\/5{caret-color:rgba(253,242,248,.05)}.caret-pink-50\/50{caret-color:rgba(253,242,248,.5)}.caret-pink-50\/55{caret-color:rgba(253,242,248,.55)}.caret-pink-50\/60{caret-color:rgba(253,242,248,.6)}.caret-pink-50\/65{caret-color:rgba(253,242,248,.65)}.caret-pink-50\/70{caret-color:rgba(253,242,248,.7)}.caret-pink-50\/75{caret-color:rgba(253,242,248,.75)}.caret-pink-50\/80{caret-color:rgba(253,242,248,.8)}.caret-pink-50\/85{caret-color:rgba(253,242,248,.85)}.caret-pink-50\/90{caret-color:rgba(253,242,248,.9)}.caret-pink-50\/95{caret-color:rgba(253,242,248,.95)}.caret-pink-500{caret-color:#ec4899}.caret-pink-500\/0{caret-color:rgba(236,72,153,0)}.caret-pink-500\/10{caret-color:rgba(236,72,153,.1)}.caret-pink-500\/100{caret-color:#ec4899}.caret-pink-500\/15{caret-color:rgba(236,72,153,.15)}.caret-pink-500\/20{caret-color:rgba(236,72,153,.2)}.caret-pink-500\/25{caret-color:rgba(236,72,153,.25)}.caret-pink-500\/30{caret-color:rgba(236,72,153,.3)}.caret-pink-500\/35{caret-color:rgba(236,72,153,.35)}.caret-pink-500\/40{caret-color:rgba(236,72,153,.4)}.caret-pink-500\/45{caret-color:rgba(236,72,153,.45)}.caret-pink-500\/5{caret-color:rgba(236,72,153,.05)}.caret-pink-500\/50{caret-color:rgba(236,72,153,.5)}.caret-pink-500\/55{caret-color:rgba(236,72,153,.55)}.caret-pink-500\/60{caret-color:rgba(236,72,153,.6)}.caret-pink-500\/65{caret-color:rgba(236,72,153,.65)}.caret-pink-500\/70{caret-color:rgba(236,72,153,.7)}.caret-pink-500\/75{caret-color:rgba(236,72,153,.75)}.caret-pink-500\/80{caret-color:rgba(236,72,153,.8)}.caret-pink-500\/85{caret-color:rgba(236,72,153,.85)}.caret-pink-500\/90{caret-color:rgba(236,72,153,.9)}.caret-pink-500\/95{caret-color:rgba(236,72,153,.95)}.caret-pink-950{caret-color:#500724}.caret-pink-950\/0{caret-color:rgba(80,7,36,0)}.caret-pink-950\/10{caret-color:rgba(80,7,36,.1)}.caret-pink-950\/100{caret-color:#500724}.caret-pink-950\/15{caret-color:rgba(80,7,36,.15)}.caret-pink-950\/20{caret-color:rgba(80,7,36,.2)}.caret-pink-950\/25{caret-color:rgba(80,7,36,.25)}.caret-pink-950\/30{caret-color:rgba(80,7,36,.3)}.caret-pink-950\/35{caret-color:rgba(80,7,36,.35)}.caret-pink-950\/40{caret-color:rgba(80,7,36,.4)}.caret-pink-950\/45{caret-color:rgba(80,7,36,.45)}.caret-pink-950\/5{caret-color:rgba(80,7,36,.05)}.caret-pink-950\/50{caret-color:rgba(80,7,36,.5)}.caret-pink-950\/55{caret-color:rgba(80,7,36,.55)}.caret-pink-950\/60{caret-color:rgba(80,7,36,.6)}.caret-pink-950\/65{caret-color:rgba(80,7,36,.65)}.caret-pink-950\/70{caret-color:rgba(80,7,36,.7)}.caret-pink-950\/75{caret-color:rgba(80,7,36,.75)}.caret-pink-950\/80{caret-color:rgba(80,7,36,.8)}.caret-pink-950\/85{caret-color:rgba(80,7,36,.85)}.caret-pink-950\/90{caret-color:rgba(80,7,36,.9)}.caret-pink-950\/95{caret-color:rgba(80,7,36,.95)}.caret-purple-100{caret-color:#f3e8ff}.caret-purple-100\/0{caret-color:rgba(243,232,255,0)}.caret-purple-100\/10{caret-color:rgba(243,232,255,.1)}.caret-purple-100\/100{caret-color:#f3e8ff}.caret-purple-100\/15{caret-color:rgba(243,232,255,.15)}.caret-purple-100\/20{caret-color:rgba(243,232,255,.2)}.caret-purple-100\/25{caret-color:rgba(243,232,255,.25)}.caret-purple-100\/30{caret-color:rgba(243,232,255,.3)}.caret-purple-100\/35{caret-color:rgba(243,232,255,.35)}.caret-purple-100\/40{caret-color:rgba(243,232,255,.4)}.caret-purple-100\/45{caret-color:rgba(243,232,255,.45)}.caret-purple-100\/5{caret-color:rgba(243,232,255,.05)}.caret-purple-100\/50{caret-color:rgba(243,232,255,.5)}.caret-purple-100\/55{caret-color:rgba(243,232,255,.55)}.caret-purple-100\/60{caret-color:rgba(243,232,255,.6)}.caret-purple-100\/65{caret-color:rgba(243,232,255,.65)}.caret-purple-100\/70{caret-color:rgba(243,232,255,.7)}.caret-purple-100\/75{caret-color:rgba(243,232,255,.75)}.caret-purple-100\/80{caret-color:rgba(243,232,255,.8)}.caret-purple-100\/85{caret-color:rgba(243,232,255,.85)}.caret-purple-100\/90{caret-color:rgba(243,232,255,.9)}.caret-purple-100\/95{caret-color:rgba(243,232,255,.95)}.caret-purple-200{caret-color:#e9d5ff}.caret-purple-200\/0{caret-color:rgba(233,213,255,0)}.caret-purple-200\/10{caret-color:rgba(233,213,255,.1)}.caret-purple-200\/100{caret-color:#e9d5ff}.caret-purple-200\/15{caret-color:rgba(233,213,255,.15)}.caret-purple-200\/20{caret-color:rgba(233,213,255,.2)}.caret-purple-200\/25{caret-color:rgba(233,213,255,.25)}.caret-purple-200\/30{caret-color:rgba(233,213,255,.3)}.caret-purple-200\/35{caret-color:rgba(233,213,255,.35)}.caret-purple-200\/40{caret-color:rgba(233,213,255,.4)}.caret-purple-200\/45{caret-color:rgba(233,213,255,.45)}.caret-purple-200\/5{caret-color:rgba(233,213,255,.05)}.caret-purple-200\/50{caret-color:rgba(233,213,255,.5)}.caret-purple-200\/55{caret-color:rgba(233,213,255,.55)}.caret-purple-200\/60{caret-color:rgba(233,213,255,.6)}.caret-purple-200\/65{caret-color:rgba(233,213,255,.65)}.caret-purple-200\/70{caret-color:rgba(233,213,255,.7)}.caret-purple-200\/75{caret-color:rgba(233,213,255,.75)}.caret-purple-200\/80{caret-color:rgba(233,213,255,.8)}.caret-purple-200\/85{caret-color:rgba(233,213,255,.85)}.caret-purple-200\/90{caret-color:rgba(233,213,255,.9)}.caret-purple-200\/95{caret-color:rgba(233,213,255,.95)}.caret-purple-400{caret-color:#c084fc}.caret-purple-400\/0{caret-color:rgba(192,132,252,0)}.caret-purple-400\/10{caret-color:rgba(192,132,252,.1)}.caret-purple-400\/100{caret-color:#c084fc}.caret-purple-400\/15{caret-color:rgba(192,132,252,.15)}.caret-purple-400\/20{caret-color:rgba(192,132,252,.2)}.caret-purple-400\/25{caret-color:rgba(192,132,252,.25)}.caret-purple-400\/30{caret-color:rgba(192,132,252,.3)}.caret-purple-400\/35{caret-color:rgba(192,132,252,.35)}.caret-purple-400\/40{caret-color:rgba(192,132,252,.4)}.caret-purple-400\/45{caret-color:rgba(192,132,252,.45)}.caret-purple-400\/5{caret-color:rgba(192,132,252,.05)}.caret-purple-400\/50{caret-color:rgba(192,132,252,.5)}.caret-purple-400\/55{caret-color:rgba(192,132,252,.55)}.caret-purple-400\/60{caret-color:rgba(192,132,252,.6)}.caret-purple-400\/65{caret-color:rgba(192,132,252,.65)}.caret-purple-400\/70{caret-color:rgba(192,132,252,.7)}.caret-purple-400\/75{caret-color:rgba(192,132,252,.75)}.caret-purple-400\/80{caret-color:rgba(192,132,252,.8)}.caret-purple-400\/85{caret-color:rgba(192,132,252,.85)}.caret-purple-400\/90{caret-color:rgba(192,132,252,.9)}.caret-purple-400\/95{caret-color:rgba(192,132,252,.95)}.caret-purple-50{caret-color:#faf5ff}.caret-purple-50\/0{caret-color:rgba(250,245,255,0)}.caret-purple-50\/10{caret-color:rgba(250,245,255,.1)}.caret-purple-50\/100{caret-color:#faf5ff}.caret-purple-50\/15{caret-color:rgba(250,245,255,.15)}.caret-purple-50\/20{caret-color:rgba(250,245,255,.2)}.caret-purple-50\/25{caret-color:rgba(250,245,255,.25)}.caret-purple-50\/30{caret-color:rgba(250,245,255,.3)}.caret-purple-50\/35{caret-color:rgba(250,245,255,.35)}.caret-purple-50\/40{caret-color:rgba(250,245,255,.4)}.caret-purple-50\/45{caret-color:rgba(250,245,255,.45)}.caret-purple-50\/5{caret-color:rgba(250,245,255,.05)}.caret-purple-50\/50{caret-color:rgba(250,245,255,.5)}.caret-purple-50\/55{caret-color:rgba(250,245,255,.55)}.caret-purple-50\/60{caret-color:rgba(250,245,255,.6)}.caret-purple-50\/65{caret-color:rgba(250,245,255,.65)}.caret-purple-50\/70{caret-color:rgba(250,245,255,.7)}.caret-purple-50\/75{caret-color:rgba(250,245,255,.75)}.caret-purple-50\/80{caret-color:rgba(250,245,255,.8)}.caret-purple-50\/85{caret-color:rgba(250,245,255,.85)}.caret-purple-50\/90{caret-color:rgba(250,245,255,.9)}.caret-purple-50\/95{caret-color:rgba(250,245,255,.95)}.caret-purple-500{caret-color:#a855f7}.caret-purple-500\/0{caret-color:rgba(168,85,247,0)}.caret-purple-500\/10{caret-color:rgba(168,85,247,.1)}.caret-purple-500\/100{caret-color:#a855f7}.caret-purple-500\/15{caret-color:rgba(168,85,247,.15)}.caret-purple-500\/20{caret-color:rgba(168,85,247,.2)}.caret-purple-500\/25{caret-color:rgba(168,85,247,.25)}.caret-purple-500\/30{caret-color:rgba(168,85,247,.3)}.caret-purple-500\/35{caret-color:rgba(168,85,247,.35)}.caret-purple-500\/40{caret-color:rgba(168,85,247,.4)}.caret-purple-500\/45{caret-color:rgba(168,85,247,.45)}.caret-purple-500\/5{caret-color:rgba(168,85,247,.05)}.caret-purple-500\/50{caret-color:rgba(168,85,247,.5)}.caret-purple-500\/55{caret-color:rgba(168,85,247,.55)}.caret-purple-500\/60{caret-color:rgba(168,85,247,.6)}.caret-purple-500\/65{caret-color:rgba(168,85,247,.65)}.caret-purple-500\/70{caret-color:rgba(168,85,247,.7)}.caret-purple-500\/75{caret-color:rgba(168,85,247,.75)}.caret-purple-500\/80{caret-color:rgba(168,85,247,.8)}.caret-purple-500\/85{caret-color:rgba(168,85,247,.85)}.caret-purple-500\/90{caret-color:rgba(168,85,247,.9)}.caret-purple-500\/95{caret-color:rgba(168,85,247,.95)}.caret-purple-950{caret-color:#3b0764}.caret-purple-950\/0{caret-color:rgba(59,7,100,0)}.caret-purple-950\/10{caret-color:rgba(59,7,100,.1)}.caret-purple-950\/100{caret-color:#3b0764}.caret-purple-950\/15{caret-color:rgba(59,7,100,.15)}.caret-purple-950\/20{caret-color:rgba(59,7,100,.2)}.caret-purple-950\/25{caret-color:rgba(59,7,100,.25)}.caret-purple-950\/30{caret-color:rgba(59,7,100,.3)}.caret-purple-950\/35{caret-color:rgba(59,7,100,.35)}.caret-purple-950\/40{caret-color:rgba(59,7,100,.4)}.caret-purple-950\/45{caret-color:rgba(59,7,100,.45)}.caret-purple-950\/5{caret-color:rgba(59,7,100,.05)}.caret-purple-950\/50{caret-color:rgba(59,7,100,.5)}.caret-purple-950\/55{caret-color:rgba(59,7,100,.55)}.caret-purple-950\/60{caret-color:rgba(59,7,100,.6)}.caret-purple-950\/65{caret-color:rgba(59,7,100,.65)}.caret-purple-950\/70{caret-color:rgba(59,7,100,.7)}.caret-purple-950\/75{caret-color:rgba(59,7,100,.75)}.caret-purple-950\/80{caret-color:rgba(59,7,100,.8)}.caret-purple-950\/85{caret-color:rgba(59,7,100,.85)}.caret-purple-950\/90{caret-color:rgba(59,7,100,.9)}.caret-purple-950\/95{caret-color:rgba(59,7,100,.95)}.caret-red-100{caret-color:#fee2e2}.caret-red-100\/0{caret-color:hsla(0,93%,94%,0)}.caret-red-100\/10{caret-color:hsla(0,93%,94%,.1)}.caret-red-100\/100{caret-color:#fee2e2}.caret-red-100\/15{caret-color:hsla(0,93%,94%,.15)}.caret-red-100\/20{caret-color:hsla(0,93%,94%,.2)}.caret-red-100\/25{caret-color:hsla(0,93%,94%,.25)}.caret-red-100\/30{caret-color:hsla(0,93%,94%,.3)}.caret-red-100\/35{caret-color:hsla(0,93%,94%,.35)}.caret-red-100\/40{caret-color:hsla(0,93%,94%,.4)}.caret-red-100\/45{caret-color:hsla(0,93%,94%,.45)}.caret-red-100\/5{caret-color:hsla(0,93%,94%,.05)}.caret-red-100\/50{caret-color:hsla(0,93%,94%,.5)}.caret-red-100\/55{caret-color:hsla(0,93%,94%,.55)}.caret-red-100\/60{caret-color:hsla(0,93%,94%,.6)}.caret-red-100\/65{caret-color:hsla(0,93%,94%,.65)}.caret-red-100\/70{caret-color:hsla(0,93%,94%,.7)}.caret-red-100\/75{caret-color:hsla(0,93%,94%,.75)}.caret-red-100\/80{caret-color:hsla(0,93%,94%,.8)}.caret-red-100\/85{caret-color:hsla(0,93%,94%,.85)}.caret-red-100\/90{caret-color:hsla(0,93%,94%,.9)}.caret-red-100\/95{caret-color:hsla(0,93%,94%,.95)}.caret-red-200{caret-color:#fecaca}.caret-red-200\/0{caret-color:hsla(0,96%,89%,0)}.caret-red-200\/10{caret-color:hsla(0,96%,89%,.1)}.caret-red-200\/100{caret-color:#fecaca}.caret-red-200\/15{caret-color:hsla(0,96%,89%,.15)}.caret-red-200\/20{caret-color:hsla(0,96%,89%,.2)}.caret-red-200\/25{caret-color:hsla(0,96%,89%,.25)}.caret-red-200\/30{caret-color:hsla(0,96%,89%,.3)}.caret-red-200\/35{caret-color:hsla(0,96%,89%,.35)}.caret-red-200\/40{caret-color:hsla(0,96%,89%,.4)}.caret-red-200\/45{caret-color:hsla(0,96%,89%,.45)}.caret-red-200\/5{caret-color:hsla(0,96%,89%,.05)}.caret-red-200\/50{caret-color:hsla(0,96%,89%,.5)}.caret-red-200\/55{caret-color:hsla(0,96%,89%,.55)}.caret-red-200\/60{caret-color:hsla(0,96%,89%,.6)}.caret-red-200\/65{caret-color:hsla(0,96%,89%,.65)}.caret-red-200\/70{caret-color:hsla(0,96%,89%,.7)}.caret-red-200\/75{caret-color:hsla(0,96%,89%,.75)}.caret-red-200\/80{caret-color:hsla(0,96%,89%,.8)}.caret-red-200\/85{caret-color:hsla(0,96%,89%,.85)}.caret-red-200\/90{caret-color:hsla(0,96%,89%,.9)}.caret-red-200\/95{caret-color:hsla(0,96%,89%,.95)}.caret-red-400{caret-color:#f87171}.caret-red-400\/0{caret-color:hsla(0,91%,71%,0)}.caret-red-400\/10{caret-color:hsla(0,91%,71%,.1)}.caret-red-400\/100{caret-color:#f87171}.caret-red-400\/15{caret-color:hsla(0,91%,71%,.15)}.caret-red-400\/20{caret-color:hsla(0,91%,71%,.2)}.caret-red-400\/25{caret-color:hsla(0,91%,71%,.25)}.caret-red-400\/30{caret-color:hsla(0,91%,71%,.3)}.caret-red-400\/35{caret-color:hsla(0,91%,71%,.35)}.caret-red-400\/40{caret-color:hsla(0,91%,71%,.4)}.caret-red-400\/45{caret-color:hsla(0,91%,71%,.45)}.caret-red-400\/5{caret-color:hsla(0,91%,71%,.05)}.caret-red-400\/50{caret-color:hsla(0,91%,71%,.5)}.caret-red-400\/55{caret-color:hsla(0,91%,71%,.55)}.caret-red-400\/60{caret-color:hsla(0,91%,71%,.6)}.caret-red-400\/65{caret-color:hsla(0,91%,71%,.65)}.caret-red-400\/70{caret-color:hsla(0,91%,71%,.7)}.caret-red-400\/75{caret-color:hsla(0,91%,71%,.75)}.caret-red-400\/80{caret-color:hsla(0,91%,71%,.8)}.caret-red-400\/85{caret-color:hsla(0,91%,71%,.85)}.caret-red-400\/90{caret-color:hsla(0,91%,71%,.9)}.caret-red-400\/95{caret-color:hsla(0,91%,71%,.95)}.caret-red-50{caret-color:#fef2f2}.caret-red-50\/0{caret-color:hsla(0,86%,97%,0)}.caret-red-50\/10{caret-color:hsla(0,86%,97%,.1)}.caret-red-50\/100{caret-color:#fef2f2}.caret-red-50\/15{caret-color:hsla(0,86%,97%,.15)}.caret-red-50\/20{caret-color:hsla(0,86%,97%,.2)}.caret-red-50\/25{caret-color:hsla(0,86%,97%,.25)}.caret-red-50\/30{caret-color:hsla(0,86%,97%,.3)}.caret-red-50\/35{caret-color:hsla(0,86%,97%,.35)}.caret-red-50\/40{caret-color:hsla(0,86%,97%,.4)}.caret-red-50\/45{caret-color:hsla(0,86%,97%,.45)}.caret-red-50\/5{caret-color:hsla(0,86%,97%,.05)}.caret-red-50\/50{caret-color:hsla(0,86%,97%,.5)}.caret-red-50\/55{caret-color:hsla(0,86%,97%,.55)}.caret-red-50\/60{caret-color:hsla(0,86%,97%,.6)}.caret-red-50\/65{caret-color:hsla(0,86%,97%,.65)}.caret-red-50\/70{caret-color:hsla(0,86%,97%,.7)}.caret-red-50\/75{caret-color:hsla(0,86%,97%,.75)}.caret-red-50\/80{caret-color:hsla(0,86%,97%,.8)}.caret-red-50\/85{caret-color:hsla(0,86%,97%,.85)}.caret-red-50\/90{caret-color:hsla(0,86%,97%,.9)}.caret-red-50\/95{caret-color:hsla(0,86%,97%,.95)}.caret-red-500{caret-color:#ef4444}.caret-red-500\/0{caret-color:rgba(239,68,68,0)}.caret-red-500\/10{caret-color:rgba(239,68,68,.1)}.caret-red-500\/100{caret-color:#ef4444}.caret-red-500\/15{caret-color:rgba(239,68,68,.15)}.caret-red-500\/20{caret-color:rgba(239,68,68,.2)}.caret-red-500\/25{caret-color:rgba(239,68,68,.25)}.caret-red-500\/30{caret-color:rgba(239,68,68,.3)}.caret-red-500\/35{caret-color:rgba(239,68,68,.35)}.caret-red-500\/40{caret-color:rgba(239,68,68,.4)}.caret-red-500\/45{caret-color:rgba(239,68,68,.45)}.caret-red-500\/5{caret-color:rgba(239,68,68,.05)}.caret-red-500\/50{caret-color:rgba(239,68,68,.5)}.caret-red-500\/55{caret-color:rgba(239,68,68,.55)}.caret-red-500\/60{caret-color:rgba(239,68,68,.6)}.caret-red-500\/65{caret-color:rgba(239,68,68,.65)}.caret-red-500\/70{caret-color:rgba(239,68,68,.7)}.caret-red-500\/75{caret-color:rgba(239,68,68,.75)}.caret-red-500\/80{caret-color:rgba(239,68,68,.8)}.caret-red-500\/85{caret-color:rgba(239,68,68,.85)}.caret-red-500\/90{caret-color:rgba(239,68,68,.9)}.caret-red-500\/95{caret-color:rgba(239,68,68,.95)}.caret-red-950{caret-color:#450a0a}.caret-red-950\/0{caret-color:rgba(69,10,10,0)}.caret-red-950\/10{caret-color:rgba(69,10,10,.1)}.caret-red-950\/100{caret-color:#450a0a}.caret-red-950\/15{caret-color:rgba(69,10,10,.15)}.caret-red-950\/20{caret-color:rgba(69,10,10,.2)}.caret-red-950\/25{caret-color:rgba(69,10,10,.25)}.caret-red-950\/30{caret-color:rgba(69,10,10,.3)}.caret-red-950\/35{caret-color:rgba(69,10,10,.35)}.caret-red-950\/40{caret-color:rgba(69,10,10,.4)}.caret-red-950\/45{caret-color:rgba(69,10,10,.45)}.caret-red-950\/5{caret-color:rgba(69,10,10,.05)}.caret-red-950\/50{caret-color:rgba(69,10,10,.5)}.caret-red-950\/55{caret-color:rgba(69,10,10,.55)}.caret-red-950\/60{caret-color:rgba(69,10,10,.6)}.caret-red-950\/65{caret-color:rgba(69,10,10,.65)}.caret-red-950\/70{caret-color:rgba(69,10,10,.7)}.caret-red-950\/75{caret-color:rgba(69,10,10,.75)}.caret-red-950\/80{caret-color:rgba(69,10,10,.8)}.caret-red-950\/85{caret-color:rgba(69,10,10,.85)}.caret-red-950\/90{caret-color:rgba(69,10,10,.9)}.caret-red-950\/95{caret-color:rgba(69,10,10,.95)}.caret-rose-100{caret-color:#ffe4e6}.caret-rose-100\/0{caret-color:rgba(255,228,230,0)}.caret-rose-100\/10{caret-color:rgba(255,228,230,.1)}.caret-rose-100\/100{caret-color:#ffe4e6}.caret-rose-100\/15{caret-color:rgba(255,228,230,.15)}.caret-rose-100\/20{caret-color:rgba(255,228,230,.2)}.caret-rose-100\/25{caret-color:rgba(255,228,230,.25)}.caret-rose-100\/30{caret-color:rgba(255,228,230,.3)}.caret-rose-100\/35{caret-color:rgba(255,228,230,.35)}.caret-rose-100\/40{caret-color:rgba(255,228,230,.4)}.caret-rose-100\/45{caret-color:rgba(255,228,230,.45)}.caret-rose-100\/5{caret-color:rgba(255,228,230,.05)}.caret-rose-100\/50{caret-color:rgba(255,228,230,.5)}.caret-rose-100\/55{caret-color:rgba(255,228,230,.55)}.caret-rose-100\/60{caret-color:rgba(255,228,230,.6)}.caret-rose-100\/65{caret-color:rgba(255,228,230,.65)}.caret-rose-100\/70{caret-color:rgba(255,228,230,.7)}.caret-rose-100\/75{caret-color:rgba(255,228,230,.75)}.caret-rose-100\/80{caret-color:rgba(255,228,230,.8)}.caret-rose-100\/85{caret-color:rgba(255,228,230,.85)}.caret-rose-100\/90{caret-color:rgba(255,228,230,.9)}.caret-rose-100\/95{caret-color:rgba(255,228,230,.95)}.caret-rose-200{caret-color:#fecdd3}.caret-rose-200\/0{caret-color:rgba(254,205,211,0)}.caret-rose-200\/10{caret-color:rgba(254,205,211,.1)}.caret-rose-200\/100{caret-color:#fecdd3}.caret-rose-200\/15{caret-color:rgba(254,205,211,.15)}.caret-rose-200\/20{caret-color:rgba(254,205,211,.2)}.caret-rose-200\/25{caret-color:rgba(254,205,211,.25)}.caret-rose-200\/30{caret-color:rgba(254,205,211,.3)}.caret-rose-200\/35{caret-color:rgba(254,205,211,.35)}.caret-rose-200\/40{caret-color:rgba(254,205,211,.4)}.caret-rose-200\/45{caret-color:rgba(254,205,211,.45)}.caret-rose-200\/5{caret-color:rgba(254,205,211,.05)}.caret-rose-200\/50{caret-color:rgba(254,205,211,.5)}.caret-rose-200\/55{caret-color:rgba(254,205,211,.55)}.caret-rose-200\/60{caret-color:rgba(254,205,211,.6)}.caret-rose-200\/65{caret-color:rgba(254,205,211,.65)}.caret-rose-200\/70{caret-color:rgba(254,205,211,.7)}.caret-rose-200\/75{caret-color:rgba(254,205,211,.75)}.caret-rose-200\/80{caret-color:rgba(254,205,211,.8)}.caret-rose-200\/85{caret-color:rgba(254,205,211,.85)}.caret-rose-200\/90{caret-color:rgba(254,205,211,.9)}.caret-rose-200\/95{caret-color:rgba(254,205,211,.95)}.caret-rose-400{caret-color:#fb7185}.caret-rose-400\/0{caret-color:rgba(251,113,133,0)}.caret-rose-400\/10{caret-color:rgba(251,113,133,.1)}.caret-rose-400\/100{caret-color:#fb7185}.caret-rose-400\/15{caret-color:rgba(251,113,133,.15)}.caret-rose-400\/20{caret-color:rgba(251,113,133,.2)}.caret-rose-400\/25{caret-color:rgba(251,113,133,.25)}.caret-rose-400\/30{caret-color:rgba(251,113,133,.3)}.caret-rose-400\/35{caret-color:rgba(251,113,133,.35)}.caret-rose-400\/40{caret-color:rgba(251,113,133,.4)}.caret-rose-400\/45{caret-color:rgba(251,113,133,.45)}.caret-rose-400\/5{caret-color:rgba(251,113,133,.05)}.caret-rose-400\/50{caret-color:rgba(251,113,133,.5)}.caret-rose-400\/55{caret-color:rgba(251,113,133,.55)}.caret-rose-400\/60{caret-color:rgba(251,113,133,.6)}.caret-rose-400\/65{caret-color:rgba(251,113,133,.65)}.caret-rose-400\/70{caret-color:rgba(251,113,133,.7)}.caret-rose-400\/75{caret-color:rgba(251,113,133,.75)}.caret-rose-400\/80{caret-color:rgba(251,113,133,.8)}.caret-rose-400\/85{caret-color:rgba(251,113,133,.85)}.caret-rose-400\/90{caret-color:rgba(251,113,133,.9)}.caret-rose-400\/95{caret-color:rgba(251,113,133,.95)}.caret-rose-50{caret-color:#fff1f2}.caret-rose-50\/0{caret-color:rgba(255,241,242,0)}.caret-rose-50\/10{caret-color:rgba(255,241,242,.1)}.caret-rose-50\/100{caret-color:#fff1f2}.caret-rose-50\/15{caret-color:rgba(255,241,242,.15)}.caret-rose-50\/20{caret-color:rgba(255,241,242,.2)}.caret-rose-50\/25{caret-color:rgba(255,241,242,.25)}.caret-rose-50\/30{caret-color:rgba(255,241,242,.3)}.caret-rose-50\/35{caret-color:rgba(255,241,242,.35)}.caret-rose-50\/40{caret-color:rgba(255,241,242,.4)}.caret-rose-50\/45{caret-color:rgba(255,241,242,.45)}.caret-rose-50\/5{caret-color:rgba(255,241,242,.05)}.caret-rose-50\/50{caret-color:rgba(255,241,242,.5)}.caret-rose-50\/55{caret-color:rgba(255,241,242,.55)}.caret-rose-50\/60{caret-color:rgba(255,241,242,.6)}.caret-rose-50\/65{caret-color:rgba(255,241,242,.65)}.caret-rose-50\/70{caret-color:rgba(255,241,242,.7)}.caret-rose-50\/75{caret-color:rgba(255,241,242,.75)}.caret-rose-50\/80{caret-color:rgba(255,241,242,.8)}.caret-rose-50\/85{caret-color:rgba(255,241,242,.85)}.caret-rose-50\/90{caret-color:rgba(255,241,242,.9)}.caret-rose-50\/95{caret-color:rgba(255,241,242,.95)}.caret-rose-500{caret-color:#f43f5e}.caret-rose-500\/0{caret-color:rgba(244,63,94,0)}.caret-rose-500\/10{caret-color:rgba(244,63,94,.1)}.caret-rose-500\/100{caret-color:#f43f5e}.caret-rose-500\/15{caret-color:rgba(244,63,94,.15)}.caret-rose-500\/20{caret-color:rgba(244,63,94,.2)}.caret-rose-500\/25{caret-color:rgba(244,63,94,.25)}.caret-rose-500\/30{caret-color:rgba(244,63,94,.3)}.caret-rose-500\/35{caret-color:rgba(244,63,94,.35)}.caret-rose-500\/40{caret-color:rgba(244,63,94,.4)}.caret-rose-500\/45{caret-color:rgba(244,63,94,.45)}.caret-rose-500\/5{caret-color:rgba(244,63,94,.05)}.caret-rose-500\/50{caret-color:rgba(244,63,94,.5)}.caret-rose-500\/55{caret-color:rgba(244,63,94,.55)}.caret-rose-500\/60{caret-color:rgba(244,63,94,.6)}.caret-rose-500\/65{caret-color:rgba(244,63,94,.65)}.caret-rose-500\/70{caret-color:rgba(244,63,94,.7)}.caret-rose-500\/75{caret-color:rgba(244,63,94,.75)}.caret-rose-500\/80{caret-color:rgba(244,63,94,.8)}.caret-rose-500\/85{caret-color:rgba(244,63,94,.85)}.caret-rose-500\/90{caret-color:rgba(244,63,94,.9)}.caret-rose-500\/95{caret-color:rgba(244,63,94,.95)}.caret-rose-950{caret-color:#4c0519}.caret-rose-950\/0{caret-color:rgba(76,5,25,0)}.caret-rose-950\/10{caret-color:rgba(76,5,25,.1)}.caret-rose-950\/100{caret-color:#4c0519}.caret-rose-950\/15{caret-color:rgba(76,5,25,.15)}.caret-rose-950\/20{caret-color:rgba(76,5,25,.2)}.caret-rose-950\/25{caret-color:rgba(76,5,25,.25)}.caret-rose-950\/30{caret-color:rgba(76,5,25,.3)}.caret-rose-950\/35{caret-color:rgba(76,5,25,.35)}.caret-rose-950\/40{caret-color:rgba(76,5,25,.4)}.caret-rose-950\/45{caret-color:rgba(76,5,25,.45)}.caret-rose-950\/5{caret-color:rgba(76,5,25,.05)}.caret-rose-950\/50{caret-color:rgba(76,5,25,.5)}.caret-rose-950\/55{caret-color:rgba(76,5,25,.55)}.caret-rose-950\/60{caret-color:rgba(76,5,25,.6)}.caret-rose-950\/65{caret-color:rgba(76,5,25,.65)}.caret-rose-950\/70{caret-color:rgba(76,5,25,.7)}.caret-rose-950\/75{caret-color:rgba(76,5,25,.75)}.caret-rose-950\/80{caret-color:rgba(76,5,25,.8)}.caret-rose-950\/85{caret-color:rgba(76,5,25,.85)}.caret-rose-950\/90{caret-color:rgba(76,5,25,.9)}.caret-rose-950\/95{caret-color:rgba(76,5,25,.95)}.caret-sky-100{caret-color:#e0f2fe}.caret-sky-100\/0{caret-color:rgba(224,242,254,0)}.caret-sky-100\/10{caret-color:rgba(224,242,254,.1)}.caret-sky-100\/100{caret-color:#e0f2fe}.caret-sky-100\/15{caret-color:rgba(224,242,254,.15)}.caret-sky-100\/20{caret-color:rgba(224,242,254,.2)}.caret-sky-100\/25{caret-color:rgba(224,242,254,.25)}.caret-sky-100\/30{caret-color:rgba(224,242,254,.3)}.caret-sky-100\/35{caret-color:rgba(224,242,254,.35)}.caret-sky-100\/40{caret-color:rgba(224,242,254,.4)}.caret-sky-100\/45{caret-color:rgba(224,242,254,.45)}.caret-sky-100\/5{caret-color:rgba(224,242,254,.05)}.caret-sky-100\/50{caret-color:rgba(224,242,254,.5)}.caret-sky-100\/55{caret-color:rgba(224,242,254,.55)}.caret-sky-100\/60{caret-color:rgba(224,242,254,.6)}.caret-sky-100\/65{caret-color:rgba(224,242,254,.65)}.caret-sky-100\/70{caret-color:rgba(224,242,254,.7)}.caret-sky-100\/75{caret-color:rgba(224,242,254,.75)}.caret-sky-100\/80{caret-color:rgba(224,242,254,.8)}.caret-sky-100\/85{caret-color:rgba(224,242,254,.85)}.caret-sky-100\/90{caret-color:rgba(224,242,254,.9)}.caret-sky-100\/95{caret-color:rgba(224,242,254,.95)}.caret-sky-200{caret-color:#bae6fd}.caret-sky-200\/0{caret-color:rgba(186,230,253,0)}.caret-sky-200\/10{caret-color:rgba(186,230,253,.1)}.caret-sky-200\/100{caret-color:#bae6fd}.caret-sky-200\/15{caret-color:rgba(186,230,253,.15)}.caret-sky-200\/20{caret-color:rgba(186,230,253,.2)}.caret-sky-200\/25{caret-color:rgba(186,230,253,.25)}.caret-sky-200\/30{caret-color:rgba(186,230,253,.3)}.caret-sky-200\/35{caret-color:rgba(186,230,253,.35)}.caret-sky-200\/40{caret-color:rgba(186,230,253,.4)}.caret-sky-200\/45{caret-color:rgba(186,230,253,.45)}.caret-sky-200\/5{caret-color:rgba(186,230,253,.05)}.caret-sky-200\/50{caret-color:rgba(186,230,253,.5)}.caret-sky-200\/55{caret-color:rgba(186,230,253,.55)}.caret-sky-200\/60{caret-color:rgba(186,230,253,.6)}.caret-sky-200\/65{caret-color:rgba(186,230,253,.65)}.caret-sky-200\/70{caret-color:rgba(186,230,253,.7)}.caret-sky-200\/75{caret-color:rgba(186,230,253,.75)}.caret-sky-200\/80{caret-color:rgba(186,230,253,.8)}.caret-sky-200\/85{caret-color:rgba(186,230,253,.85)}.caret-sky-200\/90{caret-color:rgba(186,230,253,.9)}.caret-sky-200\/95{caret-color:rgba(186,230,253,.95)}.caret-sky-400{caret-color:#38bdf8}.caret-sky-400\/0{caret-color:rgba(56,189,248,0)}.caret-sky-400\/10{caret-color:rgba(56,189,248,.1)}.caret-sky-400\/100{caret-color:#38bdf8}.caret-sky-400\/15{caret-color:rgba(56,189,248,.15)}.caret-sky-400\/20{caret-color:rgba(56,189,248,.2)}.caret-sky-400\/25{caret-color:rgba(56,189,248,.25)}.caret-sky-400\/30{caret-color:rgba(56,189,248,.3)}.caret-sky-400\/35{caret-color:rgba(56,189,248,.35)}.caret-sky-400\/40{caret-color:rgba(56,189,248,.4)}.caret-sky-400\/45{caret-color:rgba(56,189,248,.45)}.caret-sky-400\/5{caret-color:rgba(56,189,248,.05)}.caret-sky-400\/50{caret-color:rgba(56,189,248,.5)}.caret-sky-400\/55{caret-color:rgba(56,189,248,.55)}.caret-sky-400\/60{caret-color:rgba(56,189,248,.6)}.caret-sky-400\/65{caret-color:rgba(56,189,248,.65)}.caret-sky-400\/70{caret-color:rgba(56,189,248,.7)}.caret-sky-400\/75{caret-color:rgba(56,189,248,.75)}.caret-sky-400\/80{caret-color:rgba(56,189,248,.8)}.caret-sky-400\/85{caret-color:rgba(56,189,248,.85)}.caret-sky-400\/90{caret-color:rgba(56,189,248,.9)}.caret-sky-400\/95{caret-color:rgba(56,189,248,.95)}.caret-sky-50{caret-color:#f0f9ff}.caret-sky-50\/0{caret-color:rgba(240,249,255,0)}.caret-sky-50\/10{caret-color:rgba(240,249,255,.1)}.caret-sky-50\/100{caret-color:#f0f9ff}.caret-sky-50\/15{caret-color:rgba(240,249,255,.15)}.caret-sky-50\/20{caret-color:rgba(240,249,255,.2)}.caret-sky-50\/25{caret-color:rgba(240,249,255,.25)}.caret-sky-50\/30{caret-color:rgba(240,249,255,.3)}.caret-sky-50\/35{caret-color:rgba(240,249,255,.35)}.caret-sky-50\/40{caret-color:rgba(240,249,255,.4)}.caret-sky-50\/45{caret-color:rgba(240,249,255,.45)}.caret-sky-50\/5{caret-color:rgba(240,249,255,.05)}.caret-sky-50\/50{caret-color:rgba(240,249,255,.5)}.caret-sky-50\/55{caret-color:rgba(240,249,255,.55)}.caret-sky-50\/60{caret-color:rgba(240,249,255,.6)}.caret-sky-50\/65{caret-color:rgba(240,249,255,.65)}.caret-sky-50\/70{caret-color:rgba(240,249,255,.7)}.caret-sky-50\/75{caret-color:rgba(240,249,255,.75)}.caret-sky-50\/80{caret-color:rgba(240,249,255,.8)}.caret-sky-50\/85{caret-color:rgba(240,249,255,.85)}.caret-sky-50\/90{caret-color:rgba(240,249,255,.9)}.caret-sky-50\/95{caret-color:rgba(240,249,255,.95)}.caret-sky-500{caret-color:#0ea5e9}.caret-sky-500\/0{caret-color:rgba(14,165,233,0)}.caret-sky-500\/10{caret-color:rgba(14,165,233,.1)}.caret-sky-500\/100{caret-color:#0ea5e9}.caret-sky-500\/15{caret-color:rgba(14,165,233,.15)}.caret-sky-500\/20{caret-color:rgba(14,165,233,.2)}.caret-sky-500\/25{caret-color:rgba(14,165,233,.25)}.caret-sky-500\/30{caret-color:rgba(14,165,233,.3)}.caret-sky-500\/35{caret-color:rgba(14,165,233,.35)}.caret-sky-500\/40{caret-color:rgba(14,165,233,.4)}.caret-sky-500\/45{caret-color:rgba(14,165,233,.45)}.caret-sky-500\/5{caret-color:rgba(14,165,233,.05)}.caret-sky-500\/50{caret-color:rgba(14,165,233,.5)}.caret-sky-500\/55{caret-color:rgba(14,165,233,.55)}.caret-sky-500\/60{caret-color:rgba(14,165,233,.6)}.caret-sky-500\/65{caret-color:rgba(14,165,233,.65)}.caret-sky-500\/70{caret-color:rgba(14,165,233,.7)}.caret-sky-500\/75{caret-color:rgba(14,165,233,.75)}.caret-sky-500\/80{caret-color:rgba(14,165,233,.8)}.caret-sky-500\/85{caret-color:rgba(14,165,233,.85)}.caret-sky-500\/90{caret-color:rgba(14,165,233,.9)}.caret-sky-500\/95{caret-color:rgba(14,165,233,.95)}.caret-sky-950{caret-color:#082f49}.caret-sky-950\/0{caret-color:rgba(8,47,73,0)}.caret-sky-950\/10{caret-color:rgba(8,47,73,.1)}.caret-sky-950\/100{caret-color:#082f49}.caret-sky-950\/15{caret-color:rgba(8,47,73,.15)}.caret-sky-950\/20{caret-color:rgba(8,47,73,.2)}.caret-sky-950\/25{caret-color:rgba(8,47,73,.25)}.caret-sky-950\/30{caret-color:rgba(8,47,73,.3)}.caret-sky-950\/35{caret-color:rgba(8,47,73,.35)}.caret-sky-950\/40{caret-color:rgba(8,47,73,.4)}.caret-sky-950\/45{caret-color:rgba(8,47,73,.45)}.caret-sky-950\/5{caret-color:rgba(8,47,73,.05)}.caret-sky-950\/50{caret-color:rgba(8,47,73,.5)}.caret-sky-950\/55{caret-color:rgba(8,47,73,.55)}.caret-sky-950\/60{caret-color:rgba(8,47,73,.6)}.caret-sky-950\/65{caret-color:rgba(8,47,73,.65)}.caret-sky-950\/70{caret-color:rgba(8,47,73,.7)}.caret-sky-950\/75{caret-color:rgba(8,47,73,.75)}.caret-sky-950\/80{caret-color:rgba(8,47,73,.8)}.caret-sky-950\/85{caret-color:rgba(8,47,73,.85)}.caret-sky-950\/90{caret-color:rgba(8,47,73,.9)}.caret-sky-950\/95{caret-color:rgba(8,47,73,.95)}.caret-slate-100{caret-color:#f1f5f9}.caret-slate-100\/0{caret-color:rgba(241,245,249,0)}.caret-slate-100\/10{caret-color:rgba(241,245,249,.1)}.caret-slate-100\/100{caret-color:#f1f5f9}.caret-slate-100\/15{caret-color:rgba(241,245,249,.15)}.caret-slate-100\/20{caret-color:rgba(241,245,249,.2)}.caret-slate-100\/25{caret-color:rgba(241,245,249,.25)}.caret-slate-100\/30{caret-color:rgba(241,245,249,.3)}.caret-slate-100\/35{caret-color:rgba(241,245,249,.35)}.caret-slate-100\/40{caret-color:rgba(241,245,249,.4)}.caret-slate-100\/45{caret-color:rgba(241,245,249,.45)}.caret-slate-100\/5{caret-color:rgba(241,245,249,.05)}.caret-slate-100\/50{caret-color:rgba(241,245,249,.5)}.caret-slate-100\/55{caret-color:rgba(241,245,249,.55)}.caret-slate-100\/60{caret-color:rgba(241,245,249,.6)}.caret-slate-100\/65{caret-color:rgba(241,245,249,.65)}.caret-slate-100\/70{caret-color:rgba(241,245,249,.7)}.caret-slate-100\/75{caret-color:rgba(241,245,249,.75)}.caret-slate-100\/80{caret-color:rgba(241,245,249,.8)}.caret-slate-100\/85{caret-color:rgba(241,245,249,.85)}.caret-slate-100\/90{caret-color:rgba(241,245,249,.9)}.caret-slate-100\/95{caret-color:rgba(241,245,249,.95)}.caret-slate-200{caret-color:#e2e8f0}.caret-slate-200\/0{caret-color:rgba(226,232,240,0)}.caret-slate-200\/10{caret-color:rgba(226,232,240,.1)}.caret-slate-200\/100{caret-color:#e2e8f0}.caret-slate-200\/15{caret-color:rgba(226,232,240,.15)}.caret-slate-200\/20{caret-color:rgba(226,232,240,.2)}.caret-slate-200\/25{caret-color:rgba(226,232,240,.25)}.caret-slate-200\/30{caret-color:rgba(226,232,240,.3)}.caret-slate-200\/35{caret-color:rgba(226,232,240,.35)}.caret-slate-200\/40{caret-color:rgba(226,232,240,.4)}.caret-slate-200\/45{caret-color:rgba(226,232,240,.45)}.caret-slate-200\/5{caret-color:rgba(226,232,240,.05)}.caret-slate-200\/50{caret-color:rgba(226,232,240,.5)}.caret-slate-200\/55{caret-color:rgba(226,232,240,.55)}.caret-slate-200\/60{caret-color:rgba(226,232,240,.6)}.caret-slate-200\/65{caret-color:rgba(226,232,240,.65)}.caret-slate-200\/70{caret-color:rgba(226,232,240,.7)}.caret-slate-200\/75{caret-color:rgba(226,232,240,.75)}.caret-slate-200\/80{caret-color:rgba(226,232,240,.8)}.caret-slate-200\/85{caret-color:rgba(226,232,240,.85)}.caret-slate-200\/90{caret-color:rgba(226,232,240,.9)}.caret-slate-200\/95{caret-color:rgba(226,232,240,.95)}.caret-slate-400{caret-color:#94a3b8}.caret-slate-400\/0{caret-color:rgba(148,163,184,0)}.caret-slate-400\/10{caret-color:rgba(148,163,184,.1)}.caret-slate-400\/100{caret-color:#94a3b8}.caret-slate-400\/15{caret-color:rgba(148,163,184,.15)}.caret-slate-400\/20{caret-color:rgba(148,163,184,.2)}.caret-slate-400\/25{caret-color:rgba(148,163,184,.25)}.caret-slate-400\/30{caret-color:rgba(148,163,184,.3)}.caret-slate-400\/35{caret-color:rgba(148,163,184,.35)}.caret-slate-400\/40{caret-color:rgba(148,163,184,.4)}.caret-slate-400\/45{caret-color:rgba(148,163,184,.45)}.caret-slate-400\/5{caret-color:rgba(148,163,184,.05)}.caret-slate-400\/50{caret-color:rgba(148,163,184,.5)}.caret-slate-400\/55{caret-color:rgba(148,163,184,.55)}.caret-slate-400\/60{caret-color:rgba(148,163,184,.6)}.caret-slate-400\/65{caret-color:rgba(148,163,184,.65)}.caret-slate-400\/70{caret-color:rgba(148,163,184,.7)}.caret-slate-400\/75{caret-color:rgba(148,163,184,.75)}.caret-slate-400\/80{caret-color:rgba(148,163,184,.8)}.caret-slate-400\/85{caret-color:rgba(148,163,184,.85)}.caret-slate-400\/90{caret-color:rgba(148,163,184,.9)}.caret-slate-400\/95{caret-color:rgba(148,163,184,.95)}.caret-slate-50{caret-color:#f8fafc}.caret-slate-50\/0{caret-color:rgba(248,250,252,0)}.caret-slate-50\/10{caret-color:rgba(248,250,252,.1)}.caret-slate-50\/100{caret-color:#f8fafc}.caret-slate-50\/15{caret-color:rgba(248,250,252,.15)}.caret-slate-50\/20{caret-color:rgba(248,250,252,.2)}.caret-slate-50\/25{caret-color:rgba(248,250,252,.25)}.caret-slate-50\/30{caret-color:rgba(248,250,252,.3)}.caret-slate-50\/35{caret-color:rgba(248,250,252,.35)}.caret-slate-50\/40{caret-color:rgba(248,250,252,.4)}.caret-slate-50\/45{caret-color:rgba(248,250,252,.45)}.caret-slate-50\/5{caret-color:rgba(248,250,252,.05)}.caret-slate-50\/50{caret-color:rgba(248,250,252,.5)}.caret-slate-50\/55{caret-color:rgba(248,250,252,.55)}.caret-slate-50\/60{caret-color:rgba(248,250,252,.6)}.caret-slate-50\/65{caret-color:rgba(248,250,252,.65)}.caret-slate-50\/70{caret-color:rgba(248,250,252,.7)}.caret-slate-50\/75{caret-color:rgba(248,250,252,.75)}.caret-slate-50\/80{caret-color:rgba(248,250,252,.8)}.caret-slate-50\/85{caret-color:rgba(248,250,252,.85)}.caret-slate-50\/90{caret-color:rgba(248,250,252,.9)}.caret-slate-50\/95{caret-color:rgba(248,250,252,.95)}.caret-slate-500{caret-color:#64748b}.caret-slate-500\/0{caret-color:rgba(100,116,139,0)}.caret-slate-500\/10{caret-color:rgba(100,116,139,.1)}.caret-slate-500\/100{caret-color:#64748b}.caret-slate-500\/15{caret-color:rgba(100,116,139,.15)}.caret-slate-500\/20{caret-color:rgba(100,116,139,.2)}.caret-slate-500\/25{caret-color:rgba(100,116,139,.25)}.caret-slate-500\/30{caret-color:rgba(100,116,139,.3)}.caret-slate-500\/35{caret-color:rgba(100,116,139,.35)}.caret-slate-500\/40{caret-color:rgba(100,116,139,.4)}.caret-slate-500\/45{caret-color:rgba(100,116,139,.45)}.caret-slate-500\/5{caret-color:rgba(100,116,139,.05)}.caret-slate-500\/50{caret-color:rgba(100,116,139,.5)}.caret-slate-500\/55{caret-color:rgba(100,116,139,.55)}.caret-slate-500\/60{caret-color:rgba(100,116,139,.6)}.caret-slate-500\/65{caret-color:rgba(100,116,139,.65)}.caret-slate-500\/70{caret-color:rgba(100,116,139,.7)}.caret-slate-500\/75{caret-color:rgba(100,116,139,.75)}.caret-slate-500\/80{caret-color:rgba(100,116,139,.8)}.caret-slate-500\/85{caret-color:rgba(100,116,139,.85)}.caret-slate-500\/90{caret-color:rgba(100,116,139,.9)}.caret-slate-500\/95{caret-color:rgba(100,116,139,.95)}.caret-slate-950{caret-color:#020617}.caret-slate-950\/0{caret-color:rgba(2,6,23,0)}.caret-slate-950\/10{caret-color:rgba(2,6,23,.1)}.caret-slate-950\/100{caret-color:#020617}.caret-slate-950\/15{caret-color:rgba(2,6,23,.15)}.caret-slate-950\/20{caret-color:rgba(2,6,23,.2)}.caret-slate-950\/25{caret-color:rgba(2,6,23,.25)}.caret-slate-950\/30{caret-color:rgba(2,6,23,.3)}.caret-slate-950\/35{caret-color:rgba(2,6,23,.35)}.caret-slate-950\/40{caret-color:rgba(2,6,23,.4)}.caret-slate-950\/45{caret-color:rgba(2,6,23,.45)}.caret-slate-950\/5{caret-color:rgba(2,6,23,.05)}.caret-slate-950\/50{caret-color:rgba(2,6,23,.5)}.caret-slate-950\/55{caret-color:rgba(2,6,23,.55)}.caret-slate-950\/60{caret-color:rgba(2,6,23,.6)}.caret-slate-950\/65{caret-color:rgba(2,6,23,.65)}.caret-slate-950\/70{caret-color:rgba(2,6,23,.7)}.caret-slate-950\/75{caret-color:rgba(2,6,23,.75)}.caret-slate-950\/80{caret-color:rgba(2,6,23,.8)}.caret-slate-950\/85{caret-color:rgba(2,6,23,.85)}.caret-slate-950\/90{caret-color:rgba(2,6,23,.9)}.caret-slate-950\/95{caret-color:rgba(2,6,23,.95)}.caret-stone-100{caret-color:#f5f5f4}.caret-stone-100\/0{caret-color:hsla(60,5%,96%,0)}.caret-stone-100\/10{caret-color:hsla(60,5%,96%,.1)}.caret-stone-100\/100{caret-color:#f5f5f4}.caret-stone-100\/15{caret-color:hsla(60,5%,96%,.15)}.caret-stone-100\/20{caret-color:hsla(60,5%,96%,.2)}.caret-stone-100\/25{caret-color:hsla(60,5%,96%,.25)}.caret-stone-100\/30{caret-color:hsla(60,5%,96%,.3)}.caret-stone-100\/35{caret-color:hsla(60,5%,96%,.35)}.caret-stone-100\/40{caret-color:hsla(60,5%,96%,.4)}.caret-stone-100\/45{caret-color:hsla(60,5%,96%,.45)}.caret-stone-100\/5{caret-color:hsla(60,5%,96%,.05)}.caret-stone-100\/50{caret-color:hsla(60,5%,96%,.5)}.caret-stone-100\/55{caret-color:hsla(60,5%,96%,.55)}.caret-stone-100\/60{caret-color:hsla(60,5%,96%,.6)}.caret-stone-100\/65{caret-color:hsla(60,5%,96%,.65)}.caret-stone-100\/70{caret-color:hsla(60,5%,96%,.7)}.caret-stone-100\/75{caret-color:hsla(60,5%,96%,.75)}.caret-stone-100\/80{caret-color:hsla(60,5%,96%,.8)}.caret-stone-100\/85{caret-color:hsla(60,5%,96%,.85)}.caret-stone-100\/90{caret-color:hsla(60,5%,96%,.9)}.caret-stone-100\/95{caret-color:hsla(60,5%,96%,.95)}.caret-stone-200{caret-color:#e7e5e4}.caret-stone-200\/0{caret-color:hsla(20,6%,90%,0)}.caret-stone-200\/10{caret-color:hsla(20,6%,90%,.1)}.caret-stone-200\/100{caret-color:#e7e5e4}.caret-stone-200\/15{caret-color:hsla(20,6%,90%,.15)}.caret-stone-200\/20{caret-color:hsla(20,6%,90%,.2)}.caret-stone-200\/25{caret-color:hsla(20,6%,90%,.25)}.caret-stone-200\/30{caret-color:hsla(20,6%,90%,.3)}.caret-stone-200\/35{caret-color:hsla(20,6%,90%,.35)}.caret-stone-200\/40{caret-color:hsla(20,6%,90%,.4)}.caret-stone-200\/45{caret-color:hsla(20,6%,90%,.45)}.caret-stone-200\/5{caret-color:hsla(20,6%,90%,.05)}.caret-stone-200\/50{caret-color:hsla(20,6%,90%,.5)}.caret-stone-200\/55{caret-color:hsla(20,6%,90%,.55)}.caret-stone-200\/60{caret-color:hsla(20,6%,90%,.6)}.caret-stone-200\/65{caret-color:hsla(20,6%,90%,.65)}.caret-stone-200\/70{caret-color:hsla(20,6%,90%,.7)}.caret-stone-200\/75{caret-color:hsla(20,6%,90%,.75)}.caret-stone-200\/80{caret-color:hsla(20,6%,90%,.8)}.caret-stone-200\/85{caret-color:hsla(20,6%,90%,.85)}.caret-stone-200\/90{caret-color:hsla(20,6%,90%,.9)}.caret-stone-200\/95{caret-color:hsla(20,6%,90%,.95)}.caret-stone-400{caret-color:#a8a29e}.caret-stone-400\/0{caret-color:hsla(24,5%,64%,0)}.caret-stone-400\/10{caret-color:hsla(24,5%,64%,.1)}.caret-stone-400\/100{caret-color:#a8a29e}.caret-stone-400\/15{caret-color:hsla(24,5%,64%,.15)}.caret-stone-400\/20{caret-color:hsla(24,5%,64%,.2)}.caret-stone-400\/25{caret-color:hsla(24,5%,64%,.25)}.caret-stone-400\/30{caret-color:hsla(24,5%,64%,.3)}.caret-stone-400\/35{caret-color:hsla(24,5%,64%,.35)}.caret-stone-400\/40{caret-color:hsla(24,5%,64%,.4)}.caret-stone-400\/45{caret-color:hsla(24,5%,64%,.45)}.caret-stone-400\/5{caret-color:hsla(24,5%,64%,.05)}.caret-stone-400\/50{caret-color:hsla(24,5%,64%,.5)}.caret-stone-400\/55{caret-color:hsla(24,5%,64%,.55)}.caret-stone-400\/60{caret-color:hsla(24,5%,64%,.6)}.caret-stone-400\/65{caret-color:hsla(24,5%,64%,.65)}.caret-stone-400\/70{caret-color:hsla(24,5%,64%,.7)}.caret-stone-400\/75{caret-color:hsla(24,5%,64%,.75)}.caret-stone-400\/80{caret-color:hsla(24,5%,64%,.8)}.caret-stone-400\/85{caret-color:hsla(24,5%,64%,.85)}.caret-stone-400\/90{caret-color:hsla(24,5%,64%,.9)}.caret-stone-400\/95{caret-color:hsla(24,5%,64%,.95)}.caret-stone-50{caret-color:#fafaf9}.caret-stone-50\/0{caret-color:hsla(60,9%,98%,0)}.caret-stone-50\/10{caret-color:hsla(60,9%,98%,.1)}.caret-stone-50\/100{caret-color:#fafaf9}.caret-stone-50\/15{caret-color:hsla(60,9%,98%,.15)}.caret-stone-50\/20{caret-color:hsla(60,9%,98%,.2)}.caret-stone-50\/25{caret-color:hsla(60,9%,98%,.25)}.caret-stone-50\/30{caret-color:hsla(60,9%,98%,.3)}.caret-stone-50\/35{caret-color:hsla(60,9%,98%,.35)}.caret-stone-50\/40{caret-color:hsla(60,9%,98%,.4)}.caret-stone-50\/45{caret-color:hsla(60,9%,98%,.45)}.caret-stone-50\/5{caret-color:hsla(60,9%,98%,.05)}.caret-stone-50\/50{caret-color:hsla(60,9%,98%,.5)}.caret-stone-50\/55{caret-color:hsla(60,9%,98%,.55)}.caret-stone-50\/60{caret-color:hsla(60,9%,98%,.6)}.caret-stone-50\/65{caret-color:hsla(60,9%,98%,.65)}.caret-stone-50\/70{caret-color:hsla(60,9%,98%,.7)}.caret-stone-50\/75{caret-color:hsla(60,9%,98%,.75)}.caret-stone-50\/80{caret-color:hsla(60,9%,98%,.8)}.caret-stone-50\/85{caret-color:hsla(60,9%,98%,.85)}.caret-stone-50\/90{caret-color:hsla(60,9%,98%,.9)}.caret-stone-50\/95{caret-color:hsla(60,9%,98%,.95)}.caret-stone-500{caret-color:#78716c}.caret-stone-500\/0{caret-color:hsla(25,5%,45%,0)}.caret-stone-500\/10{caret-color:hsla(25,5%,45%,.1)}.caret-stone-500\/100{caret-color:#78716c}.caret-stone-500\/15{caret-color:hsla(25,5%,45%,.15)}.caret-stone-500\/20{caret-color:hsla(25,5%,45%,.2)}.caret-stone-500\/25{caret-color:hsla(25,5%,45%,.25)}.caret-stone-500\/30{caret-color:hsla(25,5%,45%,.3)}.caret-stone-500\/35{caret-color:hsla(25,5%,45%,.35)}.caret-stone-500\/40{caret-color:hsla(25,5%,45%,.4)}.caret-stone-500\/45{caret-color:hsla(25,5%,45%,.45)}.caret-stone-500\/5{caret-color:hsla(25,5%,45%,.05)}.caret-stone-500\/50{caret-color:hsla(25,5%,45%,.5)}.caret-stone-500\/55{caret-color:hsla(25,5%,45%,.55)}.caret-stone-500\/60{caret-color:hsla(25,5%,45%,.6)}.caret-stone-500\/65{caret-color:hsla(25,5%,45%,.65)}.caret-stone-500\/70{caret-color:hsla(25,5%,45%,.7)}.caret-stone-500\/75{caret-color:hsla(25,5%,45%,.75)}.caret-stone-500\/80{caret-color:hsla(25,5%,45%,.8)}.caret-stone-500\/85{caret-color:hsla(25,5%,45%,.85)}.caret-stone-500\/90{caret-color:hsla(25,5%,45%,.9)}.caret-stone-500\/95{caret-color:hsla(25,5%,45%,.95)}.caret-stone-950{caret-color:#0c0a09}.caret-stone-950\/0{caret-color:rgba(12,10,9,0)}.caret-stone-950\/10{caret-color:rgba(12,10,9,.1)}.caret-stone-950\/100{caret-color:#0c0a09}.caret-stone-950\/15{caret-color:rgba(12,10,9,.15)}.caret-stone-950\/20{caret-color:rgba(12,10,9,.2)}.caret-stone-950\/25{caret-color:rgba(12,10,9,.25)}.caret-stone-950\/30{caret-color:rgba(12,10,9,.3)}.caret-stone-950\/35{caret-color:rgba(12,10,9,.35)}.caret-stone-950\/40{caret-color:rgba(12,10,9,.4)}.caret-stone-950\/45{caret-color:rgba(12,10,9,.45)}.caret-stone-950\/5{caret-color:rgba(12,10,9,.05)}.caret-stone-950\/50{caret-color:rgba(12,10,9,.5)}.caret-stone-950\/55{caret-color:rgba(12,10,9,.55)}.caret-stone-950\/60{caret-color:rgba(12,10,9,.6)}.caret-stone-950\/65{caret-color:rgba(12,10,9,.65)}.caret-stone-950\/70{caret-color:rgba(12,10,9,.7)}.caret-stone-950\/75{caret-color:rgba(12,10,9,.75)}.caret-stone-950\/80{caret-color:rgba(12,10,9,.8)}.caret-stone-950\/85{caret-color:rgba(12,10,9,.85)}.caret-stone-950\/90{caret-color:rgba(12,10,9,.9)}.caret-stone-950\/95{caret-color:rgba(12,10,9,.95)}.caret-teal-100{caret-color:#ccfbf1}.caret-teal-100\/0{caret-color:rgba(204,251,241,0)}.caret-teal-100\/10{caret-color:rgba(204,251,241,.1)}.caret-teal-100\/100{caret-color:#ccfbf1}.caret-teal-100\/15{caret-color:rgba(204,251,241,.15)}.caret-teal-100\/20{caret-color:rgba(204,251,241,.2)}.caret-teal-100\/25{caret-color:rgba(204,251,241,.25)}.caret-teal-100\/30{caret-color:rgba(204,251,241,.3)}.caret-teal-100\/35{caret-color:rgba(204,251,241,.35)}.caret-teal-100\/40{caret-color:rgba(204,251,241,.4)}.caret-teal-100\/45{caret-color:rgba(204,251,241,.45)}.caret-teal-100\/5{caret-color:rgba(204,251,241,.05)}.caret-teal-100\/50{caret-color:rgba(204,251,241,.5)}.caret-teal-100\/55{caret-color:rgba(204,251,241,.55)}.caret-teal-100\/60{caret-color:rgba(204,251,241,.6)}.caret-teal-100\/65{caret-color:rgba(204,251,241,.65)}.caret-teal-100\/70{caret-color:rgba(204,251,241,.7)}.caret-teal-100\/75{caret-color:rgba(204,251,241,.75)}.caret-teal-100\/80{caret-color:rgba(204,251,241,.8)}.caret-teal-100\/85{caret-color:rgba(204,251,241,.85)}.caret-teal-100\/90{caret-color:rgba(204,251,241,.9)}.caret-teal-100\/95{caret-color:rgba(204,251,241,.95)}.caret-teal-200{caret-color:#99f6e4}.caret-teal-200\/0{caret-color:rgba(153,246,228,0)}.caret-teal-200\/10{caret-color:rgba(153,246,228,.1)}.caret-teal-200\/100{caret-color:#99f6e4}.caret-teal-200\/15{caret-color:rgba(153,246,228,.15)}.caret-teal-200\/20{caret-color:rgba(153,246,228,.2)}.caret-teal-200\/25{caret-color:rgba(153,246,228,.25)}.caret-teal-200\/30{caret-color:rgba(153,246,228,.3)}.caret-teal-200\/35{caret-color:rgba(153,246,228,.35)}.caret-teal-200\/40{caret-color:rgba(153,246,228,.4)}.caret-teal-200\/45{caret-color:rgba(153,246,228,.45)}.caret-teal-200\/5{caret-color:rgba(153,246,228,.05)}.caret-teal-200\/50{caret-color:rgba(153,246,228,.5)}.caret-teal-200\/55{caret-color:rgba(153,246,228,.55)}.caret-teal-200\/60{caret-color:rgba(153,246,228,.6)}.caret-teal-200\/65{caret-color:rgba(153,246,228,.65)}.caret-teal-200\/70{caret-color:rgba(153,246,228,.7)}.caret-teal-200\/75{caret-color:rgba(153,246,228,.75)}.caret-teal-200\/80{caret-color:rgba(153,246,228,.8)}.caret-teal-200\/85{caret-color:rgba(153,246,228,.85)}.caret-teal-200\/90{caret-color:rgba(153,246,228,.9)}.caret-teal-200\/95{caret-color:rgba(153,246,228,.95)}.caret-teal-400{caret-color:#2dd4bf}.caret-teal-400\/0{caret-color:rgba(45,212,191,0)}.caret-teal-400\/10{caret-color:rgba(45,212,191,.1)}.caret-teal-400\/100{caret-color:#2dd4bf}.caret-teal-400\/15{caret-color:rgba(45,212,191,.15)}.caret-teal-400\/20{caret-color:rgba(45,212,191,.2)}.caret-teal-400\/25{caret-color:rgba(45,212,191,.25)}.caret-teal-400\/30{caret-color:rgba(45,212,191,.3)}.caret-teal-400\/35{caret-color:rgba(45,212,191,.35)}.caret-teal-400\/40{caret-color:rgba(45,212,191,.4)}.caret-teal-400\/45{caret-color:rgba(45,212,191,.45)}.caret-teal-400\/5{caret-color:rgba(45,212,191,.05)}.caret-teal-400\/50{caret-color:rgba(45,212,191,.5)}.caret-teal-400\/55{caret-color:rgba(45,212,191,.55)}.caret-teal-400\/60{caret-color:rgba(45,212,191,.6)}.caret-teal-400\/65{caret-color:rgba(45,212,191,.65)}.caret-teal-400\/70{caret-color:rgba(45,212,191,.7)}.caret-teal-400\/75{caret-color:rgba(45,212,191,.75)}.caret-teal-400\/80{caret-color:rgba(45,212,191,.8)}.caret-teal-400\/85{caret-color:rgba(45,212,191,.85)}.caret-teal-400\/90{caret-color:rgba(45,212,191,.9)}.caret-teal-400\/95{caret-color:rgba(45,212,191,.95)}.caret-teal-50{caret-color:#f0fdfa}.caret-teal-50\/0{caret-color:rgba(240,253,250,0)}.caret-teal-50\/10{caret-color:rgba(240,253,250,.1)}.caret-teal-50\/100{caret-color:#f0fdfa}.caret-teal-50\/15{caret-color:rgba(240,253,250,.15)}.caret-teal-50\/20{caret-color:rgba(240,253,250,.2)}.caret-teal-50\/25{caret-color:rgba(240,253,250,.25)}.caret-teal-50\/30{caret-color:rgba(240,253,250,.3)}.caret-teal-50\/35{caret-color:rgba(240,253,250,.35)}.caret-teal-50\/40{caret-color:rgba(240,253,250,.4)}.caret-teal-50\/45{caret-color:rgba(240,253,250,.45)}.caret-teal-50\/5{caret-color:rgba(240,253,250,.05)}.caret-teal-50\/50{caret-color:rgba(240,253,250,.5)}.caret-teal-50\/55{caret-color:rgba(240,253,250,.55)}.caret-teal-50\/60{caret-color:rgba(240,253,250,.6)}.caret-teal-50\/65{caret-color:rgba(240,253,250,.65)}.caret-teal-50\/70{caret-color:rgba(240,253,250,.7)}.caret-teal-50\/75{caret-color:rgba(240,253,250,.75)}.caret-teal-50\/80{caret-color:rgba(240,253,250,.8)}.caret-teal-50\/85{caret-color:rgba(240,253,250,.85)}.caret-teal-50\/90{caret-color:rgba(240,253,250,.9)}.caret-teal-50\/95{caret-color:rgba(240,253,250,.95)}.caret-teal-500{caret-color:#14b8a6}.caret-teal-500\/0{caret-color:rgba(20,184,166,0)}.caret-teal-500\/10{caret-color:rgba(20,184,166,.1)}.caret-teal-500\/100{caret-color:#14b8a6}.caret-teal-500\/15{caret-color:rgba(20,184,166,.15)}.caret-teal-500\/20{caret-color:rgba(20,184,166,.2)}.caret-teal-500\/25{caret-color:rgba(20,184,166,.25)}.caret-teal-500\/30{caret-color:rgba(20,184,166,.3)}.caret-teal-500\/35{caret-color:rgba(20,184,166,.35)}.caret-teal-500\/40{caret-color:rgba(20,184,166,.4)}.caret-teal-500\/45{caret-color:rgba(20,184,166,.45)}.caret-teal-500\/5{caret-color:rgba(20,184,166,.05)}.caret-teal-500\/50{caret-color:rgba(20,184,166,.5)}.caret-teal-500\/55{caret-color:rgba(20,184,166,.55)}.caret-teal-500\/60{caret-color:rgba(20,184,166,.6)}.caret-teal-500\/65{caret-color:rgba(20,184,166,.65)}.caret-teal-500\/70{caret-color:rgba(20,184,166,.7)}.caret-teal-500\/75{caret-color:rgba(20,184,166,.75)}.caret-teal-500\/80{caret-color:rgba(20,184,166,.8)}.caret-teal-500\/85{caret-color:rgba(20,184,166,.85)}.caret-teal-500\/90{caret-color:rgba(20,184,166,.9)}.caret-teal-500\/95{caret-color:rgba(20,184,166,.95)}.caret-teal-950{caret-color:#042f2e}.caret-teal-950\/0{caret-color:rgba(4,47,46,0)}.caret-teal-950\/10{caret-color:rgba(4,47,46,.1)}.caret-teal-950\/100{caret-color:#042f2e}.caret-teal-950\/15{caret-color:rgba(4,47,46,.15)}.caret-teal-950\/20{caret-color:rgba(4,47,46,.2)}.caret-teal-950\/25{caret-color:rgba(4,47,46,.25)}.caret-teal-950\/30{caret-color:rgba(4,47,46,.3)}.caret-teal-950\/35{caret-color:rgba(4,47,46,.35)}.caret-teal-950\/40{caret-color:rgba(4,47,46,.4)}.caret-teal-950\/45{caret-color:rgba(4,47,46,.45)}.caret-teal-950\/5{caret-color:rgba(4,47,46,.05)}.caret-teal-950\/50{caret-color:rgba(4,47,46,.5)}.caret-teal-950\/55{caret-color:rgba(4,47,46,.55)}.caret-teal-950\/60{caret-color:rgba(4,47,46,.6)}.caret-teal-950\/65{caret-color:rgba(4,47,46,.65)}.caret-teal-950\/70{caret-color:rgba(4,47,46,.7)}.caret-teal-950\/75{caret-color:rgba(4,47,46,.75)}.caret-teal-950\/80{caret-color:rgba(4,47,46,.8)}.caret-teal-950\/85{caret-color:rgba(4,47,46,.85)}.caret-teal-950\/90{caret-color:rgba(4,47,46,.9)}.caret-teal-950\/95{caret-color:rgba(4,47,46,.95)}.caret-violet-100{caret-color:#ede9fe}.caret-violet-100\/0{caret-color:rgba(237,233,254,0)}.caret-violet-100\/10{caret-color:rgba(237,233,254,.1)}.caret-violet-100\/100{caret-color:#ede9fe}.caret-violet-100\/15{caret-color:rgba(237,233,254,.15)}.caret-violet-100\/20{caret-color:rgba(237,233,254,.2)}.caret-violet-100\/25{caret-color:rgba(237,233,254,.25)}.caret-violet-100\/30{caret-color:rgba(237,233,254,.3)}.caret-violet-100\/35{caret-color:rgba(237,233,254,.35)}.caret-violet-100\/40{caret-color:rgba(237,233,254,.4)}.caret-violet-100\/45{caret-color:rgba(237,233,254,.45)}.caret-violet-100\/5{caret-color:rgba(237,233,254,.05)}.caret-violet-100\/50{caret-color:rgba(237,233,254,.5)}.caret-violet-100\/55{caret-color:rgba(237,233,254,.55)}.caret-violet-100\/60{caret-color:rgba(237,233,254,.6)}.caret-violet-100\/65{caret-color:rgba(237,233,254,.65)}.caret-violet-100\/70{caret-color:rgba(237,233,254,.7)}.caret-violet-100\/75{caret-color:rgba(237,233,254,.75)}.caret-violet-100\/80{caret-color:rgba(237,233,254,.8)}.caret-violet-100\/85{caret-color:rgba(237,233,254,.85)}.caret-violet-100\/90{caret-color:rgba(237,233,254,.9)}.caret-violet-100\/95{caret-color:rgba(237,233,254,.95)}.caret-violet-200{caret-color:#ddd6fe}.caret-violet-200\/0{caret-color:rgba(221,214,254,0)}.caret-violet-200\/10{caret-color:rgba(221,214,254,.1)}.caret-violet-200\/100{caret-color:#ddd6fe}.caret-violet-200\/15{caret-color:rgba(221,214,254,.15)}.caret-violet-200\/20{caret-color:rgba(221,214,254,.2)}.caret-violet-200\/25{caret-color:rgba(221,214,254,.25)}.caret-violet-200\/30{caret-color:rgba(221,214,254,.3)}.caret-violet-200\/35{caret-color:rgba(221,214,254,.35)}.caret-violet-200\/40{caret-color:rgba(221,214,254,.4)}.caret-violet-200\/45{caret-color:rgba(221,214,254,.45)}.caret-violet-200\/5{caret-color:rgba(221,214,254,.05)}.caret-violet-200\/50{caret-color:rgba(221,214,254,.5)}.caret-violet-200\/55{caret-color:rgba(221,214,254,.55)}.caret-violet-200\/60{caret-color:rgba(221,214,254,.6)}.caret-violet-200\/65{caret-color:rgba(221,214,254,.65)}.caret-violet-200\/70{caret-color:rgba(221,214,254,.7)}.caret-violet-200\/75{caret-color:rgba(221,214,254,.75)}.caret-violet-200\/80{caret-color:rgba(221,214,254,.8)}.caret-violet-200\/85{caret-color:rgba(221,214,254,.85)}.caret-violet-200\/90{caret-color:rgba(221,214,254,.9)}.caret-violet-200\/95{caret-color:rgba(221,214,254,.95)}.caret-violet-400{caret-color:#a78bfa}.caret-violet-400\/0{caret-color:rgba(167,139,250,0)}.caret-violet-400\/10{caret-color:rgba(167,139,250,.1)}.caret-violet-400\/100{caret-color:#a78bfa}.caret-violet-400\/15{caret-color:rgba(167,139,250,.15)}.caret-violet-400\/20{caret-color:rgba(167,139,250,.2)}.caret-violet-400\/25{caret-color:rgba(167,139,250,.25)}.caret-violet-400\/30{caret-color:rgba(167,139,250,.3)}.caret-violet-400\/35{caret-color:rgba(167,139,250,.35)}.caret-violet-400\/40{caret-color:rgba(167,139,250,.4)}.caret-violet-400\/45{caret-color:rgba(167,139,250,.45)}.caret-violet-400\/5{caret-color:rgba(167,139,250,.05)}.caret-violet-400\/50{caret-color:rgba(167,139,250,.5)}.caret-violet-400\/55{caret-color:rgba(167,139,250,.55)}.caret-violet-400\/60{caret-color:rgba(167,139,250,.6)}.caret-violet-400\/65{caret-color:rgba(167,139,250,.65)}.caret-violet-400\/70{caret-color:rgba(167,139,250,.7)}.caret-violet-400\/75{caret-color:rgba(167,139,250,.75)}.caret-violet-400\/80{caret-color:rgba(167,139,250,.8)}.caret-violet-400\/85{caret-color:rgba(167,139,250,.85)}.caret-violet-400\/90{caret-color:rgba(167,139,250,.9)}.caret-violet-400\/95{caret-color:rgba(167,139,250,.95)}.caret-violet-50{caret-color:#f5f3ff}.caret-violet-50\/0{caret-color:rgba(245,243,255,0)}.caret-violet-50\/10{caret-color:rgba(245,243,255,.1)}.caret-violet-50\/100{caret-color:#f5f3ff}.caret-violet-50\/15{caret-color:rgba(245,243,255,.15)}.caret-violet-50\/20{caret-color:rgba(245,243,255,.2)}.caret-violet-50\/25{caret-color:rgba(245,243,255,.25)}.caret-violet-50\/30{caret-color:rgba(245,243,255,.3)}.caret-violet-50\/35{caret-color:rgba(245,243,255,.35)}.caret-violet-50\/40{caret-color:rgba(245,243,255,.4)}.caret-violet-50\/45{caret-color:rgba(245,243,255,.45)}.caret-violet-50\/5{caret-color:rgba(245,243,255,.05)}.caret-violet-50\/50{caret-color:rgba(245,243,255,.5)}.caret-violet-50\/55{caret-color:rgba(245,243,255,.55)}.caret-violet-50\/60{caret-color:rgba(245,243,255,.6)}.caret-violet-50\/65{caret-color:rgba(245,243,255,.65)}.caret-violet-50\/70{caret-color:rgba(245,243,255,.7)}.caret-violet-50\/75{caret-color:rgba(245,243,255,.75)}.caret-violet-50\/80{caret-color:rgba(245,243,255,.8)}.caret-violet-50\/85{caret-color:rgba(245,243,255,.85)}.caret-violet-50\/90{caret-color:rgba(245,243,255,.9)}.caret-violet-50\/95{caret-color:rgba(245,243,255,.95)}.caret-violet-500{caret-color:#8b5cf6}.caret-violet-500\/0{caret-color:rgba(139,92,246,0)}.caret-violet-500\/10{caret-color:rgba(139,92,246,.1)}.caret-violet-500\/100{caret-color:#8b5cf6}.caret-violet-500\/15{caret-color:rgba(139,92,246,.15)}.caret-violet-500\/20{caret-color:rgba(139,92,246,.2)}.caret-violet-500\/25{caret-color:rgba(139,92,246,.25)}.caret-violet-500\/30{caret-color:rgba(139,92,246,.3)}.caret-violet-500\/35{caret-color:rgba(139,92,246,.35)}.caret-violet-500\/40{caret-color:rgba(139,92,246,.4)}.caret-violet-500\/45{caret-color:rgba(139,92,246,.45)}.caret-violet-500\/5{caret-color:rgba(139,92,246,.05)}.caret-violet-500\/50{caret-color:rgba(139,92,246,.5)}.caret-violet-500\/55{caret-color:rgba(139,92,246,.55)}.caret-violet-500\/60{caret-color:rgba(139,92,246,.6)}.caret-violet-500\/65{caret-color:rgba(139,92,246,.65)}.caret-violet-500\/70{caret-color:rgba(139,92,246,.7)}.caret-violet-500\/75{caret-color:rgba(139,92,246,.75)}.caret-violet-500\/80{caret-color:rgba(139,92,246,.8)}.caret-violet-500\/85{caret-color:rgba(139,92,246,.85)}.caret-violet-500\/90{caret-color:rgba(139,92,246,.9)}.caret-violet-500\/95{caret-color:rgba(139,92,246,.95)}.caret-violet-950{caret-color:#2e1065}.caret-violet-950\/0{caret-color:rgba(46,16,101,0)}.caret-violet-950\/10{caret-color:rgba(46,16,101,.1)}.caret-violet-950\/100{caret-color:#2e1065}.caret-violet-950\/15{caret-color:rgba(46,16,101,.15)}.caret-violet-950\/20{caret-color:rgba(46,16,101,.2)}.caret-violet-950\/25{caret-color:rgba(46,16,101,.25)}.caret-violet-950\/30{caret-color:rgba(46,16,101,.3)}.caret-violet-950\/35{caret-color:rgba(46,16,101,.35)}.caret-violet-950\/40{caret-color:rgba(46,16,101,.4)}.caret-violet-950\/45{caret-color:rgba(46,16,101,.45)}.caret-violet-950\/5{caret-color:rgba(46,16,101,.05)}.caret-violet-950\/50{caret-color:rgba(46,16,101,.5)}.caret-violet-950\/55{caret-color:rgba(46,16,101,.55)}.caret-violet-950\/60{caret-color:rgba(46,16,101,.6)}.caret-violet-950\/65{caret-color:rgba(46,16,101,.65)}.caret-violet-950\/70{caret-color:rgba(46,16,101,.7)}.caret-violet-950\/75{caret-color:rgba(46,16,101,.75)}.caret-violet-950\/80{caret-color:rgba(46,16,101,.8)}.caret-violet-950\/85{caret-color:rgba(46,16,101,.85)}.caret-violet-950\/90{caret-color:rgba(46,16,101,.9)}.caret-violet-950\/95{caret-color:rgba(46,16,101,.95)}.caret-yellow-100{caret-color:#fef9c3}.caret-yellow-100\/0{caret-color:hsla(55,97%,88%,0)}.caret-yellow-100\/10{caret-color:hsla(55,97%,88%,.1)}.caret-yellow-100\/100{caret-color:#fef9c3}.caret-yellow-100\/15{caret-color:hsla(55,97%,88%,.15)}.caret-yellow-100\/20{caret-color:hsla(55,97%,88%,.2)}.caret-yellow-100\/25{caret-color:hsla(55,97%,88%,.25)}.caret-yellow-100\/30{caret-color:hsla(55,97%,88%,.3)}.caret-yellow-100\/35{caret-color:hsla(55,97%,88%,.35)}.caret-yellow-100\/40{caret-color:hsla(55,97%,88%,.4)}.caret-yellow-100\/45{caret-color:hsla(55,97%,88%,.45)}.caret-yellow-100\/5{caret-color:hsla(55,97%,88%,.05)}.caret-yellow-100\/50{caret-color:hsla(55,97%,88%,.5)}.caret-yellow-100\/55{caret-color:hsla(55,97%,88%,.55)}.caret-yellow-100\/60{caret-color:hsla(55,97%,88%,.6)}.caret-yellow-100\/65{caret-color:hsla(55,97%,88%,.65)}.caret-yellow-100\/70{caret-color:hsla(55,97%,88%,.7)}.caret-yellow-100\/75{caret-color:hsla(55,97%,88%,.75)}.caret-yellow-100\/80{caret-color:hsla(55,97%,88%,.8)}.caret-yellow-100\/85{caret-color:hsla(55,97%,88%,.85)}.caret-yellow-100\/90{caret-color:hsla(55,97%,88%,.9)}.caret-yellow-100\/95{caret-color:hsla(55,97%,88%,.95)}.caret-yellow-200{caret-color:#fef08a}.caret-yellow-200\/0{caret-color:hsla(53,98%,77%,0)}.caret-yellow-200\/10{caret-color:hsla(53,98%,77%,.1)}.caret-yellow-200\/100{caret-color:#fef08a}.caret-yellow-200\/15{caret-color:hsla(53,98%,77%,.15)}.caret-yellow-200\/20{caret-color:hsla(53,98%,77%,.2)}.caret-yellow-200\/25{caret-color:hsla(53,98%,77%,.25)}.caret-yellow-200\/30{caret-color:hsla(53,98%,77%,.3)}.caret-yellow-200\/35{caret-color:hsla(53,98%,77%,.35)}.caret-yellow-200\/40{caret-color:hsla(53,98%,77%,.4)}.caret-yellow-200\/45{caret-color:hsla(53,98%,77%,.45)}.caret-yellow-200\/5{caret-color:hsla(53,98%,77%,.05)}.caret-yellow-200\/50{caret-color:hsla(53,98%,77%,.5)}.caret-yellow-200\/55{caret-color:hsla(53,98%,77%,.55)}.caret-yellow-200\/60{caret-color:hsla(53,98%,77%,.6)}.caret-yellow-200\/65{caret-color:hsla(53,98%,77%,.65)}.caret-yellow-200\/70{caret-color:hsla(53,98%,77%,.7)}.caret-yellow-200\/75{caret-color:hsla(53,98%,77%,.75)}.caret-yellow-200\/80{caret-color:hsla(53,98%,77%,.8)}.caret-yellow-200\/85{caret-color:hsla(53,98%,77%,.85)}.caret-yellow-200\/90{caret-color:hsla(53,98%,77%,.9)}.caret-yellow-200\/95{caret-color:hsla(53,98%,77%,.95)}.caret-yellow-400{caret-color:#facc15}.caret-yellow-400\/0{caret-color:rgba(250,204,21,0)}.caret-yellow-400\/10{caret-color:rgba(250,204,21,.1)}.caret-yellow-400\/100{caret-color:#facc15}.caret-yellow-400\/15{caret-color:rgba(250,204,21,.15)}.caret-yellow-400\/20{caret-color:rgba(250,204,21,.2)}.caret-yellow-400\/25{caret-color:rgba(250,204,21,.25)}.caret-yellow-400\/30{caret-color:rgba(250,204,21,.3)}.caret-yellow-400\/35{caret-color:rgba(250,204,21,.35)}.caret-yellow-400\/40{caret-color:rgba(250,204,21,.4)}.caret-yellow-400\/45{caret-color:rgba(250,204,21,.45)}.caret-yellow-400\/5{caret-color:rgba(250,204,21,.05)}.caret-yellow-400\/50{caret-color:rgba(250,204,21,.5)}.caret-yellow-400\/55{caret-color:rgba(250,204,21,.55)}.caret-yellow-400\/60{caret-color:rgba(250,204,21,.6)}.caret-yellow-400\/65{caret-color:rgba(250,204,21,.65)}.caret-yellow-400\/70{caret-color:rgba(250,204,21,.7)}.caret-yellow-400\/75{caret-color:rgba(250,204,21,.75)}.caret-yellow-400\/80{caret-color:rgba(250,204,21,.8)}.caret-yellow-400\/85{caret-color:rgba(250,204,21,.85)}.caret-yellow-400\/90{caret-color:rgba(250,204,21,.9)}.caret-yellow-400\/95{caret-color:rgba(250,204,21,.95)}.caret-yellow-50{caret-color:#fefce8}.caret-yellow-50\/0{caret-color:hsla(55,92%,95%,0)}.caret-yellow-50\/10{caret-color:hsla(55,92%,95%,.1)}.caret-yellow-50\/100{caret-color:#fefce8}.caret-yellow-50\/15{caret-color:hsla(55,92%,95%,.15)}.caret-yellow-50\/20{caret-color:hsla(55,92%,95%,.2)}.caret-yellow-50\/25{caret-color:hsla(55,92%,95%,.25)}.caret-yellow-50\/30{caret-color:hsla(55,92%,95%,.3)}.caret-yellow-50\/35{caret-color:hsla(55,92%,95%,.35)}.caret-yellow-50\/40{caret-color:hsla(55,92%,95%,.4)}.caret-yellow-50\/45{caret-color:hsla(55,92%,95%,.45)}.caret-yellow-50\/5{caret-color:hsla(55,92%,95%,.05)}.caret-yellow-50\/50{caret-color:hsla(55,92%,95%,.5)}.caret-yellow-50\/55{caret-color:hsla(55,92%,95%,.55)}.caret-yellow-50\/60{caret-color:hsla(55,92%,95%,.6)}.caret-yellow-50\/65{caret-color:hsla(55,92%,95%,.65)}.caret-yellow-50\/70{caret-color:hsla(55,92%,95%,.7)}.caret-yellow-50\/75{caret-color:hsla(55,92%,95%,.75)}.caret-yellow-50\/80{caret-color:hsla(55,92%,95%,.8)}.caret-yellow-50\/85{caret-color:hsla(55,92%,95%,.85)}.caret-yellow-50\/90{caret-color:hsla(55,92%,95%,.9)}.caret-yellow-50\/95{caret-color:hsla(55,92%,95%,.95)}.caret-yellow-500{caret-color:#eab308}.caret-yellow-500\/0{caret-color:rgba(234,179,8,0)}.caret-yellow-500\/10{caret-color:rgba(234,179,8,.1)}.caret-yellow-500\/100{caret-color:#eab308}.caret-yellow-500\/15{caret-color:rgba(234,179,8,.15)}.caret-yellow-500\/20{caret-color:rgba(234,179,8,.2)}.caret-yellow-500\/25{caret-color:rgba(234,179,8,.25)}.caret-yellow-500\/30{caret-color:rgba(234,179,8,.3)}.caret-yellow-500\/35{caret-color:rgba(234,179,8,.35)}.caret-yellow-500\/40{caret-color:rgba(234,179,8,.4)}.caret-yellow-500\/45{caret-color:rgba(234,179,8,.45)}.caret-yellow-500\/5{caret-color:rgba(234,179,8,.05)}.caret-yellow-500\/50{caret-color:rgba(234,179,8,.5)}.caret-yellow-500\/55{caret-color:rgba(234,179,8,.55)}.caret-yellow-500\/60{caret-color:rgba(234,179,8,.6)}.caret-yellow-500\/65{caret-color:rgba(234,179,8,.65)}.caret-yellow-500\/70{caret-color:rgba(234,179,8,.7)}.caret-yellow-500\/75{caret-color:rgba(234,179,8,.75)}.caret-yellow-500\/80{caret-color:rgba(234,179,8,.8)}.caret-yellow-500\/85{caret-color:rgba(234,179,8,.85)}.caret-yellow-500\/90{caret-color:rgba(234,179,8,.9)}.caret-yellow-500\/95{caret-color:rgba(234,179,8,.95)}.caret-yellow-950{caret-color:#422006}.caret-yellow-950\/0{caret-color:rgba(66,32,6,0)}.caret-yellow-950\/10{caret-color:rgba(66,32,6,.1)}.caret-yellow-950\/100{caret-color:#422006}.caret-yellow-950\/15{caret-color:rgba(66,32,6,.15)}.caret-yellow-950\/20{caret-color:rgba(66,32,6,.2)}.caret-yellow-950\/25{caret-color:rgba(66,32,6,.25)}.caret-yellow-950\/30{caret-color:rgba(66,32,6,.3)}.caret-yellow-950\/35{caret-color:rgba(66,32,6,.35)}.caret-yellow-950\/40{caret-color:rgba(66,32,6,.4)}.caret-yellow-950\/45{caret-color:rgba(66,32,6,.45)}.caret-yellow-950\/5{caret-color:rgba(66,32,6,.05)}.caret-yellow-950\/50{caret-color:rgba(66,32,6,.5)}.caret-yellow-950\/55{caret-color:rgba(66,32,6,.55)}.caret-yellow-950\/60{caret-color:rgba(66,32,6,.6)}.caret-yellow-950\/65{caret-color:rgba(66,32,6,.65)}.caret-yellow-950\/70{caret-color:rgba(66,32,6,.7)}.caret-yellow-950\/75{caret-color:rgba(66,32,6,.75)}.caret-yellow-950\/80{caret-color:rgba(66,32,6,.8)}.caret-yellow-950\/85{caret-color:rgba(66,32,6,.85)}.caret-yellow-950\/90{caret-color:rgba(66,32,6,.9)}.caret-yellow-950\/95{caret-color:rgba(66,32,6,.95)}.caret-zinc-100{caret-color:#f4f4f5}.caret-zinc-100\/0{caret-color:hsla(240,5%,96%,0)}.caret-zinc-100\/10{caret-color:hsla(240,5%,96%,.1)}.caret-zinc-100\/100{caret-color:#f4f4f5}.caret-zinc-100\/15{caret-color:hsla(240,5%,96%,.15)}.caret-zinc-100\/20{caret-color:hsla(240,5%,96%,.2)}.caret-zinc-100\/25{caret-color:hsla(240,5%,96%,.25)}.caret-zinc-100\/30{caret-color:hsla(240,5%,96%,.3)}.caret-zinc-100\/35{caret-color:hsla(240,5%,96%,.35)}.caret-zinc-100\/40{caret-color:hsla(240,5%,96%,.4)}.caret-zinc-100\/45{caret-color:hsla(240,5%,96%,.45)}.caret-zinc-100\/5{caret-color:hsla(240,5%,96%,.05)}.caret-zinc-100\/50{caret-color:hsla(240,5%,96%,.5)}.caret-zinc-100\/55{caret-color:hsla(240,5%,96%,.55)}.caret-zinc-100\/60{caret-color:hsla(240,5%,96%,.6)}.caret-zinc-100\/65{caret-color:hsla(240,5%,96%,.65)}.caret-zinc-100\/70{caret-color:hsla(240,5%,96%,.7)}.caret-zinc-100\/75{caret-color:hsla(240,5%,96%,.75)}.caret-zinc-100\/80{caret-color:hsla(240,5%,96%,.8)}.caret-zinc-100\/85{caret-color:hsla(240,5%,96%,.85)}.caret-zinc-100\/90{caret-color:hsla(240,5%,96%,.9)}.caret-zinc-100\/95{caret-color:hsla(240,5%,96%,.95)}.caret-zinc-200{caret-color:#e4e4e7}.caret-zinc-200\/0{caret-color:hsla(240,6%,90%,0)}.caret-zinc-200\/10{caret-color:hsla(240,6%,90%,.1)}.caret-zinc-200\/100{caret-color:#e4e4e7}.caret-zinc-200\/15{caret-color:hsla(240,6%,90%,.15)}.caret-zinc-200\/20{caret-color:hsla(240,6%,90%,.2)}.caret-zinc-200\/25{caret-color:hsla(240,6%,90%,.25)}.caret-zinc-200\/30{caret-color:hsla(240,6%,90%,.3)}.caret-zinc-200\/35{caret-color:hsla(240,6%,90%,.35)}.caret-zinc-200\/40{caret-color:hsla(240,6%,90%,.4)}.caret-zinc-200\/45{caret-color:hsla(240,6%,90%,.45)}.caret-zinc-200\/5{caret-color:hsla(240,6%,90%,.05)}.caret-zinc-200\/50{caret-color:hsla(240,6%,90%,.5)}.caret-zinc-200\/55{caret-color:hsla(240,6%,90%,.55)}.caret-zinc-200\/60{caret-color:hsla(240,6%,90%,.6)}.caret-zinc-200\/65{caret-color:hsla(240,6%,90%,.65)}.caret-zinc-200\/70{caret-color:hsla(240,6%,90%,.7)}.caret-zinc-200\/75{caret-color:hsla(240,6%,90%,.75)}.caret-zinc-200\/80{caret-color:hsla(240,6%,90%,.8)}.caret-zinc-200\/85{caret-color:hsla(240,6%,90%,.85)}.caret-zinc-200\/90{caret-color:hsla(240,6%,90%,.9)}.caret-zinc-200\/95{caret-color:hsla(240,6%,90%,.95)}.caret-zinc-400{caret-color:#a1a1aa}.caret-zinc-400\/0{caret-color:hsla(240,5%,65%,0)}.caret-zinc-400\/10{caret-color:hsla(240,5%,65%,.1)}.caret-zinc-400\/100{caret-color:#a1a1aa}.caret-zinc-400\/15{caret-color:hsla(240,5%,65%,.15)}.caret-zinc-400\/20{caret-color:hsla(240,5%,65%,.2)}.caret-zinc-400\/25{caret-color:hsla(240,5%,65%,.25)}.caret-zinc-400\/30{caret-color:hsla(240,5%,65%,.3)}.caret-zinc-400\/35{caret-color:hsla(240,5%,65%,.35)}.caret-zinc-400\/40{caret-color:hsla(240,5%,65%,.4)}.caret-zinc-400\/45{caret-color:hsla(240,5%,65%,.45)}.caret-zinc-400\/5{caret-color:hsla(240,5%,65%,.05)}.caret-zinc-400\/50{caret-color:hsla(240,5%,65%,.5)}.caret-zinc-400\/55{caret-color:hsla(240,5%,65%,.55)}.caret-zinc-400\/60{caret-color:hsla(240,5%,65%,.6)}.caret-zinc-400\/65{caret-color:hsla(240,5%,65%,.65)}.caret-zinc-400\/70{caret-color:hsla(240,5%,65%,.7)}.caret-zinc-400\/75{caret-color:hsla(240,5%,65%,.75)}.caret-zinc-400\/80{caret-color:hsla(240,5%,65%,.8)}.caret-zinc-400\/85{caret-color:hsla(240,5%,65%,.85)}.caret-zinc-400\/90{caret-color:hsla(240,5%,65%,.9)}.caret-zinc-400\/95{caret-color:hsla(240,5%,65%,.95)}.caret-zinc-50{caret-color:#fafafa}.caret-zinc-50\/0{caret-color:hsla(0,0%,98%,0)}.caret-zinc-50\/10{caret-color:hsla(0,0%,98%,.1)}.caret-zinc-50\/100{caret-color:#fafafa}.caret-zinc-50\/15{caret-color:hsla(0,0%,98%,.15)}.caret-zinc-50\/20{caret-color:hsla(0,0%,98%,.2)}.caret-zinc-50\/25{caret-color:hsla(0,0%,98%,.25)}.caret-zinc-50\/30{caret-color:hsla(0,0%,98%,.3)}.caret-zinc-50\/35{caret-color:hsla(0,0%,98%,.35)}.caret-zinc-50\/40{caret-color:hsla(0,0%,98%,.4)}.caret-zinc-50\/45{caret-color:hsla(0,0%,98%,.45)}.caret-zinc-50\/5{caret-color:hsla(0,0%,98%,.05)}.caret-zinc-50\/50{caret-color:hsla(0,0%,98%,.5)}.caret-zinc-50\/55{caret-color:hsla(0,0%,98%,.55)}.caret-zinc-50\/60{caret-color:hsla(0,0%,98%,.6)}.caret-zinc-50\/65{caret-color:hsla(0,0%,98%,.65)}.caret-zinc-50\/70{caret-color:hsla(0,0%,98%,.7)}.caret-zinc-50\/75{caret-color:hsla(0,0%,98%,.75)}.caret-zinc-50\/80{caret-color:hsla(0,0%,98%,.8)}.caret-zinc-50\/85{caret-color:hsla(0,0%,98%,.85)}.caret-zinc-50\/90{caret-color:hsla(0,0%,98%,.9)}.caret-zinc-50\/95{caret-color:hsla(0,0%,98%,.95)}.caret-zinc-500{caret-color:#71717a}.caret-zinc-500\/0{caret-color:hsla(240,4%,46%,0)}.caret-zinc-500\/10{caret-color:hsla(240,4%,46%,.1)}.caret-zinc-500\/100{caret-color:#71717a}.caret-zinc-500\/15{caret-color:hsla(240,4%,46%,.15)}.caret-zinc-500\/20{caret-color:hsla(240,4%,46%,.2)}.caret-zinc-500\/25{caret-color:hsla(240,4%,46%,.25)}.caret-zinc-500\/30{caret-color:hsla(240,4%,46%,.3)}.caret-zinc-500\/35{caret-color:hsla(240,4%,46%,.35)}.caret-zinc-500\/40{caret-color:hsla(240,4%,46%,.4)}.caret-zinc-500\/45{caret-color:hsla(240,4%,46%,.45)}.caret-zinc-500\/5{caret-color:hsla(240,4%,46%,.05)}.caret-zinc-500\/50{caret-color:hsla(240,4%,46%,.5)}.caret-zinc-500\/55{caret-color:hsla(240,4%,46%,.55)}.caret-zinc-500\/60{caret-color:hsla(240,4%,46%,.6)}.caret-zinc-500\/65{caret-color:hsla(240,4%,46%,.65)}.caret-zinc-500\/70{caret-color:hsla(240,4%,46%,.7)}.caret-zinc-500\/75{caret-color:hsla(240,4%,46%,.75)}.caret-zinc-500\/80{caret-color:hsla(240,4%,46%,.8)}.caret-zinc-500\/85{caret-color:hsla(240,4%,46%,.85)}.caret-zinc-500\/90{caret-color:hsla(240,4%,46%,.9)}.caret-zinc-500\/95{caret-color:hsla(240,4%,46%,.95)}.caret-zinc-950{caret-color:#09090b}.caret-zinc-950\/0{caret-color:rgba(9,9,11,0)}.caret-zinc-950\/10{caret-color:rgba(9,9,11,.1)}.caret-zinc-950\/100{caret-color:#09090b}.caret-zinc-950\/15{caret-color:rgba(9,9,11,.15)}.caret-zinc-950\/20{caret-color:rgba(9,9,11,.2)}.caret-zinc-950\/25{caret-color:rgba(9,9,11,.25)}.caret-zinc-950\/30{caret-color:rgba(9,9,11,.3)}.caret-zinc-950\/35{caret-color:rgba(9,9,11,.35)}.caret-zinc-950\/40{caret-color:rgba(9,9,11,.4)}.caret-zinc-950\/45{caret-color:rgba(9,9,11,.45)}.caret-zinc-950\/5{caret-color:rgba(9,9,11,.05)}.caret-zinc-950\/50{caret-color:rgba(9,9,11,.5)}.caret-zinc-950\/55{caret-color:rgba(9,9,11,.55)}.caret-zinc-950\/60{caret-color:rgba(9,9,11,.6)}.caret-zinc-950\/65{caret-color:rgba(9,9,11,.65)}.caret-zinc-950\/70{caret-color:rgba(9,9,11,.7)}.caret-zinc-950\/75{caret-color:rgba(9,9,11,.75)}.caret-zinc-950\/80{caret-color:rgba(9,9,11,.8)}.caret-zinc-950\/85{caret-color:rgba(9,9,11,.85)}.caret-zinc-950\/90{caret-color:rgba(9,9,11,.9)}.caret-zinc-950\/95{caret-color:rgba(9,9,11,.95)}.opacity-0{opacity:0}.opacity-100{opacity:1}.opacity-50{opacity:.5}.opacity-60{opacity:.6}.opacity-70{opacity:.7}.opacity-90{opacity:.9}.shadow-\[0_-20px_25px_-5px_rgba\(0\2c 0\2c 0\2c 0\.1\)\]{--tw-shadow:0 -20px 25px -5px rgba(0,0,0,.1);--tw-shadow-colored:0 -20px 25px -5px var(--tw-shadow-color)}.shadow-\[0_-20px_25px_-5px_rgba\(0\2c 0\2c 0\2c 0\.1\)\],.shadow-lg{box-shadow:var(--tw-ring-offset-shadow,0 0 #0000),var(--tw-ring-shadow,0 0 #0000),var(--tw-shadow)}.shadow-lg{--tw-shadow:0 10px 15px -3px rgba(0,0,0,.1),0 4px 6px -4px rgba(0,0,0,.1);--tw-shadow-colored:0 10px 15px -3px var(--tw-shadow-color),0 4px 6px -4px var(--tw-shadow-color)}.shadow-md{--tw-shadow:0 4px 6px -1px rgba(0,0,0,.1),0 2px 4px -2px rgba(0,0,0,.1);--tw-shadow-colored:0 4px 6px -1px var(--tw-shadow-color),0 2px 4px -2px var(--tw-shadow-color)}.shadow-md,.shadow-none{box-shadow:var(--tw-ring-offset-shadow,0 0 #0000),var(--tw-ring-shadow,0 0 #0000),var(--tw-shadow)}.shadow-none{--tw-shadow:0 0 #0000;--tw-shadow-colored:0 0 #0000}.shadow-sm{--tw-shadow:0 1px 2px 0 rgba(0,0,0,.05);--tw-shadow-colored:0 1px 2px 0 var(--tw-shadow-color)}.shadow-sm,.shadow-xl{box-shadow:var(--tw-ring-offset-shadow,0 0 #0000),var(--tw-ring-shadow,0 0 #0000),var(--tw-shadow)}.shadow-xl{--tw-shadow:0 20px 25px -5px rgba(0,0,0,.1),0 8px 10px -6px rgba(0,0,0,.1);--tw-shadow-colored:0 20px 25px -5px var(--tw-shadow-color),0 8px 10px -6px var(--tw-shadow-color)}.outline-none{outline:2px solid transparent;outline-offset:2px}.outline{outline-style:solid}.ring-2{--tw-ring-offset-shadow:var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);--tw-ring-shadow:var(--tw-ring-inset) 0 0 0 calc(2px + var(--tw-ring-offset-width)) var(--tw-ring-color);box-shadow:var(--tw-ring-offset-shadow),var(--tw-ring-shadow),var(--tw-shadow,0 0 #0000)}.ring-amber-100{--tw-ring-opacity:1;--tw-ring-color:rgb(254 243 199/var(--tw-ring-opacity,1))}.ring-amber-100\/0{--tw-ring-color:hsla(48,96%,89%,0)}.ring-amber-100\/10{--tw-ring-color:hsla(48,96%,89%,.1)}.ring-amber-100\/100{--tw-ring-color:#fef3c7}.ring-amber-100\/15{--tw-ring-color:hsla(48,96%,89%,.15)}.ring-amber-100\/20{--tw-ring-color:hsla(48,96%,89%,.2)}.ring-amber-100\/25{--tw-ring-color:hsla(48,96%,89%,.25)}.ring-amber-100\/30{--tw-ring-color:hsla(48,96%,89%,.3)}.ring-amber-100\/35{--tw-ring-color:hsla(48,96%,89%,.35)}.ring-amber-100\/40{--tw-ring-color:hsla(48,96%,89%,.4)}.ring-amber-100\/45{--tw-ring-color:hsla(48,96%,89%,.45)}.ring-amber-100\/5{--tw-ring-color:hsla(48,96%,89%,.05)}.ring-amber-100\/50{--tw-ring-color:hsla(48,96%,89%,.5)}.ring-amber-100\/55{--tw-ring-color:hsla(48,96%,89%,.55)}.ring-amber-100\/60{--tw-ring-color:hsla(48,96%,89%,.6)}.ring-amber-100\/65{--tw-ring-color:hsla(48,96%,89%,.65)}.ring-amber-100\/70{--tw-ring-color:hsla(48,96%,89%,.7)}.ring-amber-100\/75{--tw-ring-color:hsla(48,96%,89%,.75)}.ring-amber-100\/80{--tw-ring-color:hsla(48,96%,89%,.8)}.ring-amber-100\/85{--tw-ring-color:hsla(48,96%,89%,.85)}.ring-amber-100\/90{--tw-ring-color:hsla(48,96%,89%,.9)}.ring-amber-100\/95{--tw-ring-color:hsla(48,96%,89%,.95)}.ring-amber-200{--tw-ring-opacity:1;--tw-ring-color:rgb(253 230 138/var(--tw-ring-opacity,1))}.ring-amber-200\/0{--tw-ring-color:hsla(48,97%,77%,0)}.ring-amber-200\/10{--tw-ring-color:hsla(48,97%,77%,.1)}.ring-amber-200\/100{--tw-ring-color:#fde68a}.ring-amber-200\/15{--tw-ring-color:hsla(48,97%,77%,.15)}.ring-amber-200\/20{--tw-ring-color:hsla(48,97%,77%,.2)}.ring-amber-200\/25{--tw-ring-color:hsla(48,97%,77%,.25)}.ring-amber-200\/30{--tw-ring-color:hsla(48,97%,77%,.3)}.ring-amber-200\/35{--tw-ring-color:hsla(48,97%,77%,.35)}.ring-amber-200\/40{--tw-ring-color:hsla(48,97%,77%,.4)}.ring-amber-200\/45{--tw-ring-color:hsla(48,97%,77%,.45)}.ring-amber-200\/5{--tw-ring-color:hsla(48,97%,77%,.05)}.ring-amber-200\/50{--tw-ring-color:hsla(48,97%,77%,.5)}.ring-amber-200\/55{--tw-ring-color:hsla(48,97%,77%,.55)}.ring-amber-200\/60{--tw-ring-color:hsla(48,97%,77%,.6)}.ring-amber-200\/65{--tw-ring-color:hsla(48,97%,77%,.65)}.ring-amber-200\/70{--tw-ring-color:hsla(48,97%,77%,.7)}.ring-amber-200\/75{--tw-ring-color:hsla(48,97%,77%,.75)}.ring-amber-200\/80{--tw-ring-color:hsla(48,97%,77%,.8)}.ring-amber-200\/85{--tw-ring-color:hsla(48,97%,77%,.85)}.ring-amber-200\/90{--tw-ring-color:hsla(48,97%,77%,.9)}.ring-amber-200\/95{--tw-ring-color:hsla(48,97%,77%,.95)}.ring-amber-400{--tw-ring-opacity:1;--tw-ring-color:rgb(251 191 36/var(--tw-ring-opacity,1))}.ring-amber-400\/0{--tw-ring-color:rgba(251,191,36,0)}.ring-amber-400\/10{--tw-ring-color:rgba(251,191,36,.1)}.ring-amber-400\/100{--tw-ring-color:#fbbf24}.ring-amber-400\/15{--tw-ring-color:rgba(251,191,36,.15)}.ring-amber-400\/20{--tw-ring-color:rgba(251,191,36,.2)}.ring-amber-400\/25{--tw-ring-color:rgba(251,191,36,.25)}.ring-amber-400\/30{--tw-ring-color:rgba(251,191,36,.3)}.ring-amber-400\/35{--tw-ring-color:rgba(251,191,36,.35)}.ring-amber-400\/40{--tw-ring-color:rgba(251,191,36,.4)}.ring-amber-400\/45{--tw-ring-color:rgba(251,191,36,.45)}.ring-amber-400\/5{--tw-ring-color:rgba(251,191,36,.05)}.ring-amber-400\/50{--tw-ring-color:rgba(251,191,36,.5)}.ring-amber-400\/55{--tw-ring-color:rgba(251,191,36,.55)}.ring-amber-400\/60{--tw-ring-color:rgba(251,191,36,.6)}.ring-amber-400\/65{--tw-ring-color:rgba(251,191,36,.65)}.ring-amber-400\/70{--tw-ring-color:rgba(251,191,36,.7)}.ring-amber-400\/75{--tw-ring-color:rgba(251,191,36,.75)}.ring-amber-400\/80{--tw-ring-color:rgba(251,191,36,.8)}.ring-amber-400\/85{--tw-ring-color:rgba(251,191,36,.85)}.ring-amber-400\/90{--tw-ring-color:rgba(251,191,36,.9)}.ring-amber-400\/95{--tw-ring-color:rgba(251,191,36,.95)}.ring-amber-50{--tw-ring-opacity:1;--tw-ring-color:rgb(255 251 235/var(--tw-ring-opacity,1))}.ring-amber-50\/0{--tw-ring-color:rgba(255,251,235,0)}.ring-amber-50\/10{--tw-ring-color:rgba(255,251,235,.1)}.ring-amber-50\/100{--tw-ring-color:#fffbeb}.ring-amber-50\/15{--tw-ring-color:rgba(255,251,235,.15)}.ring-amber-50\/20{--tw-ring-color:rgba(255,251,235,.2)}.ring-amber-50\/25{--tw-ring-color:rgba(255,251,235,.25)}.ring-amber-50\/30{--tw-ring-color:rgba(255,251,235,.3)}.ring-amber-50\/35{--tw-ring-color:rgba(255,251,235,.35)}.ring-amber-50\/40{--tw-ring-color:rgba(255,251,235,.4)}.ring-amber-50\/45{--tw-ring-color:rgba(255,251,235,.45)}.ring-amber-50\/5{--tw-ring-color:rgba(255,251,235,.05)}.ring-amber-50\/50{--tw-ring-color:rgba(255,251,235,.5)}.ring-amber-50\/55{--tw-ring-color:rgba(255,251,235,.55)}.ring-amber-50\/60{--tw-ring-color:rgba(255,251,235,.6)}.ring-amber-50\/65{--tw-ring-color:rgba(255,251,235,.65)}.ring-amber-50\/70{--tw-ring-color:rgba(255,251,235,.7)}.ring-amber-50\/75{--tw-ring-color:rgba(255,251,235,.75)}.ring-amber-50\/80{--tw-ring-color:rgba(255,251,235,.8)}.ring-amber-50\/85{--tw-ring-color:rgba(255,251,235,.85)}.ring-amber-50\/90{--tw-ring-color:rgba(255,251,235,.9)}.ring-amber-50\/95{--tw-ring-color:rgba(255,251,235,.95)}.ring-amber-500{--tw-ring-opacity:1;--tw-ring-color:rgb(245 158 11/var(--tw-ring-opacity,1))}.ring-amber-500\/0{--tw-ring-color:rgba(245,158,11,0)}.ring-amber-500\/10{--tw-ring-color:rgba(245,158,11,.1)}.ring-amber-500\/100{--tw-ring-color:#f59e0b}.ring-amber-500\/15{--tw-ring-color:rgba(245,158,11,.15)}.ring-amber-500\/20{--tw-ring-color:rgba(245,158,11,.2)}.ring-amber-500\/25{--tw-ring-color:rgba(245,158,11,.25)}.ring-amber-500\/30{--tw-ring-color:rgba(245,158,11,.3)}.ring-amber-500\/35{--tw-ring-color:rgba(245,158,11,.35)}.ring-amber-500\/40{--tw-ring-color:rgba(245,158,11,.4)}.ring-amber-500\/45{--tw-ring-color:rgba(245,158,11,.45)}.ring-amber-500\/5{--tw-ring-color:rgba(245,158,11,.05)}.ring-amber-500\/50{--tw-ring-color:rgba(245,158,11,.5)}.ring-amber-500\/55{--tw-ring-color:rgba(245,158,11,.55)}.ring-amber-500\/60{--tw-ring-color:rgba(245,158,11,.6)}.ring-amber-500\/65{--tw-ring-color:rgba(245,158,11,.65)}.ring-amber-500\/70{--tw-ring-color:rgba(245,158,11,.7)}.ring-amber-500\/75{--tw-ring-color:rgba(245,158,11,.75)}.ring-amber-500\/80{--tw-ring-color:rgba(245,158,11,.8)}.ring-amber-500\/85{--tw-ring-color:rgba(245,158,11,.85)}.ring-amber-500\/90{--tw-ring-color:rgba(245,158,11,.9)}.ring-amber-500\/95{--tw-ring-color:rgba(245,158,11,.95)}.ring-amber-950{--tw-ring-opacity:1;--tw-ring-color:rgb(69 26 3/var(--tw-ring-opacity,1))}.ring-amber-950\/0{--tw-ring-color:rgba(69,26,3,0)}.ring-amber-950\/10{--tw-ring-color:rgba(69,26,3,.1)}.ring-amber-950\/100{--tw-ring-color:#451a03}.ring-amber-950\/15{--tw-ring-color:rgba(69,26,3,.15)}.ring-amber-950\/20{--tw-ring-color:rgba(69,26,3,.2)}.ring-amber-950\/25{--tw-ring-color:rgba(69,26,3,.25)}.ring-amber-950\/30{--tw-ring-color:rgba(69,26,3,.3)}.ring-amber-950\/35{--tw-ring-color:rgba(69,26,3,.35)}.ring-amber-950\/40{--tw-ring-color:rgba(69,26,3,.4)}.ring-amber-950\/45{--tw-ring-color:rgba(69,26,3,.45)}.ring-amber-950\/5{--tw-ring-color:rgba(69,26,3,.05)}.ring-amber-950\/50{--tw-ring-color:rgba(69,26,3,.5)}.ring-amber-950\/55{--tw-ring-color:rgba(69,26,3,.55)}.ring-amber-950\/60{--tw-ring-color:rgba(69,26,3,.6)}.ring-amber-950\/65{--tw-ring-color:rgba(69,26,3,.65)}.ring-amber-950\/70{--tw-ring-color:rgba(69,26,3,.7)}.ring-amber-950\/75{--tw-ring-color:rgba(69,26,3,.75)}.ring-amber-950\/80{--tw-ring-color:rgba(69,26,3,.8)}.ring-amber-950\/85{--tw-ring-color:rgba(69,26,3,.85)}.ring-amber-950\/90{--tw-ring-color:rgba(69,26,3,.9)}.ring-amber-950\/95{--tw-ring-color:rgba(69,26,3,.95)}.ring-blue-100{--tw-ring-opacity:1;--tw-ring-color:rgb(219 234 254/var(--tw-ring-opacity,1))}.ring-blue-100\/0{--tw-ring-color:rgba(219,234,254,0)}.ring-blue-100\/10{--tw-ring-color:rgba(219,234,254,.1)}.ring-blue-100\/100{--tw-ring-color:#dbeafe}.ring-blue-100\/15{--tw-ring-color:rgba(219,234,254,.15)}.ring-blue-100\/20{--tw-ring-color:rgba(219,234,254,.2)}.ring-blue-100\/25{--tw-ring-color:rgba(219,234,254,.25)}.ring-blue-100\/30{--tw-ring-color:rgba(219,234,254,.3)}.ring-blue-100\/35{--tw-ring-color:rgba(219,234,254,.35)}.ring-blue-100\/40{--tw-ring-color:rgba(219,234,254,.4)}.ring-blue-100\/45{--tw-ring-color:rgba(219,234,254,.45)}.ring-blue-100\/5{--tw-ring-color:rgba(219,234,254,.05)}.ring-blue-100\/50{--tw-ring-color:rgba(219,234,254,.5)}.ring-blue-100\/55{--tw-ring-color:rgba(219,234,254,.55)}.ring-blue-100\/60{--tw-ring-color:rgba(219,234,254,.6)}.ring-blue-100\/65{--tw-ring-color:rgba(219,234,254,.65)}.ring-blue-100\/70{--tw-ring-color:rgba(219,234,254,.7)}.ring-blue-100\/75{--tw-ring-color:rgba(219,234,254,.75)}.ring-blue-100\/80{--tw-ring-color:rgba(219,234,254,.8)}.ring-blue-100\/85{--tw-ring-color:rgba(219,234,254,.85)}.ring-blue-100\/90{--tw-ring-color:rgba(219,234,254,.9)}.ring-blue-100\/95{--tw-ring-color:rgba(219,234,254,.95)}.ring-blue-200{--tw-ring-opacity:1;--tw-ring-color:rgb(191 219 254/var(--tw-ring-opacity,1))}.ring-blue-200\/0{--tw-ring-color:rgba(191,219,254,0)}.ring-blue-200\/10{--tw-ring-color:rgba(191,219,254,.1)}.ring-blue-200\/100{--tw-ring-color:#bfdbfe}.ring-blue-200\/15{--tw-ring-color:rgba(191,219,254,.15)}.ring-blue-200\/20{--tw-ring-color:rgba(191,219,254,.2)}.ring-blue-200\/25{--tw-ring-color:rgba(191,219,254,.25)}.ring-blue-200\/30{--tw-ring-color:rgba(191,219,254,.3)}.ring-blue-200\/35{--tw-ring-color:rgba(191,219,254,.35)}.ring-blue-200\/40{--tw-ring-color:rgba(191,219,254,.4)}.ring-blue-200\/45{--tw-ring-color:rgba(191,219,254,.45)}.ring-blue-200\/5{--tw-ring-color:rgba(191,219,254,.05)}.ring-blue-200\/50{--tw-ring-color:rgba(191,219,254,.5)}.ring-blue-200\/55{--tw-ring-color:rgba(191,219,254,.55)}.ring-blue-200\/60{--tw-ring-color:rgba(191,219,254,.6)}.ring-blue-200\/65{--tw-ring-color:rgba(191,219,254,.65)}.ring-blue-200\/70{--tw-ring-color:rgba(191,219,254,.7)}.ring-blue-200\/75{--tw-ring-color:rgba(191,219,254,.75)}.ring-blue-200\/80{--tw-ring-color:rgba(191,219,254,.8)}.ring-blue-200\/85{--tw-ring-color:rgba(191,219,254,.85)}.ring-blue-200\/90{--tw-ring-color:rgba(191,219,254,.9)}.ring-blue-200\/95{--tw-ring-color:rgba(191,219,254,.95)}.ring-blue-400{--tw-ring-opacity:1;--tw-ring-color:rgb(96 165 250/var(--tw-ring-opacity,1))}.ring-blue-400\/0{--tw-ring-color:rgba(96,165,250,0)}.ring-blue-400\/10{--tw-ring-color:rgba(96,165,250,.1)}.ring-blue-400\/100{--tw-ring-color:#60a5fa}.ring-blue-400\/15{--tw-ring-color:rgba(96,165,250,.15)}.ring-blue-400\/20{--tw-ring-color:rgba(96,165,250,.2)}.ring-blue-400\/25{--tw-ring-color:rgba(96,165,250,.25)}.ring-blue-400\/30{--tw-ring-color:rgba(96,165,250,.3)}.ring-blue-400\/35{--tw-ring-color:rgba(96,165,250,.35)}.ring-blue-400\/40{--tw-ring-color:rgba(96,165,250,.4)}.ring-blue-400\/45{--tw-ring-color:rgba(96,165,250,.45)}.ring-blue-400\/5{--tw-ring-color:rgba(96,165,250,.05)}.ring-blue-400\/50{--tw-ring-color:rgba(96,165,250,.5)}.ring-blue-400\/55{--tw-ring-color:rgba(96,165,250,.55)}.ring-blue-400\/60{--tw-ring-color:rgba(96,165,250,.6)}.ring-blue-400\/65{--tw-ring-color:rgba(96,165,250,.65)}.ring-blue-400\/70{--tw-ring-color:rgba(96,165,250,.7)}.ring-blue-400\/75{--tw-ring-color:rgba(96,165,250,.75)}.ring-blue-400\/80{--tw-ring-color:rgba(96,165,250,.8)}.ring-blue-400\/85{--tw-ring-color:rgba(96,165,250,.85)}.ring-blue-400\/90{--tw-ring-color:rgba(96,165,250,.9)}.ring-blue-400\/95{--tw-ring-color:rgba(96,165,250,.95)}.ring-blue-50{--tw-ring-opacity:1;--tw-ring-color:rgb(239 246 255/var(--tw-ring-opacity,1))}.ring-blue-50\/0{--tw-ring-color:rgba(239,246,255,0)}.ring-blue-50\/10{--tw-ring-color:rgba(239,246,255,.1)}.ring-blue-50\/100{--tw-ring-color:#eff6ff}.ring-blue-50\/15{--tw-ring-color:rgba(239,246,255,.15)}.ring-blue-50\/20{--tw-ring-color:rgba(239,246,255,.2)}.ring-blue-50\/25{--tw-ring-color:rgba(239,246,255,.25)}.ring-blue-50\/30{--tw-ring-color:rgba(239,246,255,.3)}.ring-blue-50\/35{--tw-ring-color:rgba(239,246,255,.35)}.ring-blue-50\/40{--tw-ring-color:rgba(239,246,255,.4)}.ring-blue-50\/45{--tw-ring-color:rgba(239,246,255,.45)}.ring-blue-50\/5{--tw-ring-color:rgba(239,246,255,.05)}.ring-blue-50\/50{--tw-ring-color:rgba(239,246,255,.5)}.ring-blue-50\/55{--tw-ring-color:rgba(239,246,255,.55)}.ring-blue-50\/60{--tw-ring-color:rgba(239,246,255,.6)}.ring-blue-50\/65{--tw-ring-color:rgba(239,246,255,.65)}.ring-blue-50\/70{--tw-ring-color:rgba(239,246,255,.7)}.ring-blue-50\/75{--tw-ring-color:rgba(239,246,255,.75)}.ring-blue-50\/80{--tw-ring-color:rgba(239,246,255,.8)}.ring-blue-50\/85{--tw-ring-color:rgba(239,246,255,.85)}.ring-blue-50\/90{--tw-ring-color:rgba(239,246,255,.9)}.ring-blue-50\/95{--tw-ring-color:rgba(239,246,255,.95)}.ring-blue-500{--tw-ring-opacity:1;--tw-ring-color:rgb(59 130 246/var(--tw-ring-opacity,1))}.ring-blue-500\/0{--tw-ring-color:rgba(59,130,246,0)}.ring-blue-500\/10{--tw-ring-color:rgba(59,130,246,.1)}.ring-blue-500\/100{--tw-ring-color:#3b82f6}.ring-blue-500\/15{--tw-ring-color:rgba(59,130,246,.15)}.ring-blue-500\/20{--tw-ring-color:rgba(59,130,246,.2)}.ring-blue-500\/25{--tw-ring-color:rgba(59,130,246,.25)}.ring-blue-500\/30{--tw-ring-color:rgba(59,130,246,.3)}.ring-blue-500\/35{--tw-ring-color:rgba(59,130,246,.35)}.ring-blue-500\/40{--tw-ring-color:rgba(59,130,246,.4)}.ring-blue-500\/45{--tw-ring-color:rgba(59,130,246,.45)}.ring-blue-500\/5{--tw-ring-color:rgba(59,130,246,.05)}.ring-blue-500\/50{--tw-ring-color:rgba(59,130,246,.5)}.ring-blue-500\/55{--tw-ring-color:rgba(59,130,246,.55)}.ring-blue-500\/60{--tw-ring-color:rgba(59,130,246,.6)}.ring-blue-500\/65{--tw-ring-color:rgba(59,130,246,.65)}.ring-blue-500\/70{--tw-ring-color:rgba(59,130,246,.7)}.ring-blue-500\/75{--tw-ring-color:rgba(59,130,246,.75)}.ring-blue-500\/80{--tw-ring-color:rgba(59,130,246,.8)}.ring-blue-500\/85{--tw-ring-color:rgba(59,130,246,.85)}.ring-blue-500\/90{--tw-ring-color:rgba(59,130,246,.9)}.ring-blue-500\/95{--tw-ring-color:rgba(59,130,246,.95)}.ring-blue-950{--tw-ring-opacity:1;--tw-ring-color:rgb(23 37 84/var(--tw-ring-opacity,1))}.ring-blue-950\/0{--tw-ring-color:rgba(23,37,84,0)}.ring-blue-950\/10{--tw-ring-color:rgba(23,37,84,.1)}.ring-blue-950\/100{--tw-ring-color:#172554}.ring-blue-950\/15{--tw-ring-color:rgba(23,37,84,.15)}.ring-blue-950\/20{--tw-ring-color:rgba(23,37,84,.2)}.ring-blue-950\/25{--tw-ring-color:rgba(23,37,84,.25)}.ring-blue-950\/30{--tw-ring-color:rgba(23,37,84,.3)}.ring-blue-950\/35{--tw-ring-color:rgba(23,37,84,.35)}.ring-blue-950\/40{--tw-ring-color:rgba(23,37,84,.4)}.ring-blue-950\/45{--tw-ring-color:rgba(23,37,84,.45)}.ring-blue-950\/5{--tw-ring-color:rgba(23,37,84,.05)}.ring-blue-950\/50{--tw-ring-color:rgba(23,37,84,.5)}.ring-blue-950\/55{--tw-ring-color:rgba(23,37,84,.55)}.ring-blue-950\/60{--tw-ring-color:rgba(23,37,84,.6)}.ring-blue-950\/65{--tw-ring-color:rgba(23,37,84,.65)}.ring-blue-950\/70{--tw-ring-color:rgba(23,37,84,.7)}.ring-blue-950\/75{--tw-ring-color:rgba(23,37,84,.75)}.ring-blue-950\/80{--tw-ring-color:rgba(23,37,84,.8)}.ring-blue-950\/85{--tw-ring-color:rgba(23,37,84,.85)}.ring-blue-950\/90{--tw-ring-color:rgba(23,37,84,.9)}.ring-blue-950\/95{--tw-ring-color:rgba(23,37,84,.95)}.ring-cyan-100{--tw-ring-opacity:1;--tw-ring-color:rgb(207 250 254/var(--tw-ring-opacity,1))}.ring-cyan-100\/0{--tw-ring-color:rgba(207,250,254,0)}.ring-cyan-100\/10{--tw-ring-color:rgba(207,250,254,.1)}.ring-cyan-100\/100{--tw-ring-color:#cffafe}.ring-cyan-100\/15{--tw-ring-color:rgba(207,250,254,.15)}.ring-cyan-100\/20{--tw-ring-color:rgba(207,250,254,.2)}.ring-cyan-100\/25{--tw-ring-color:rgba(207,250,254,.25)}.ring-cyan-100\/30{--tw-ring-color:rgba(207,250,254,.3)}.ring-cyan-100\/35{--tw-ring-color:rgba(207,250,254,.35)}.ring-cyan-100\/40{--tw-ring-color:rgba(207,250,254,.4)}.ring-cyan-100\/45{--tw-ring-color:rgba(207,250,254,.45)}.ring-cyan-100\/5{--tw-ring-color:rgba(207,250,254,.05)}.ring-cyan-100\/50{--tw-ring-color:rgba(207,250,254,.5)}.ring-cyan-100\/55{--tw-ring-color:rgba(207,250,254,.55)}.ring-cyan-100\/60{--tw-ring-color:rgba(207,250,254,.6)}.ring-cyan-100\/65{--tw-ring-color:rgba(207,250,254,.65)}.ring-cyan-100\/70{--tw-ring-color:rgba(207,250,254,.7)}.ring-cyan-100\/75{--tw-ring-color:rgba(207,250,254,.75)}.ring-cyan-100\/80{--tw-ring-color:rgba(207,250,254,.8)}.ring-cyan-100\/85{--tw-ring-color:rgba(207,250,254,.85)}.ring-cyan-100\/90{--tw-ring-color:rgba(207,250,254,.9)}.ring-cyan-100\/95{--tw-ring-color:rgba(207,250,254,.95)}.ring-cyan-200{--tw-ring-opacity:1;--tw-ring-color:rgb(165 243 252/var(--tw-ring-opacity,1))}.ring-cyan-200\/0{--tw-ring-color:rgba(165,243,252,0)}.ring-cyan-200\/10{--tw-ring-color:rgba(165,243,252,.1)}.ring-cyan-200\/100{--tw-ring-color:#a5f3fc}.ring-cyan-200\/15{--tw-ring-color:rgba(165,243,252,.15)}.ring-cyan-200\/20{--tw-ring-color:rgba(165,243,252,.2)}.ring-cyan-200\/25{--tw-ring-color:rgba(165,243,252,.25)}.ring-cyan-200\/30{--tw-ring-color:rgba(165,243,252,.3)}.ring-cyan-200\/35{--tw-ring-color:rgba(165,243,252,.35)}.ring-cyan-200\/40{--tw-ring-color:rgba(165,243,252,.4)}.ring-cyan-200\/45{--tw-ring-color:rgba(165,243,252,.45)}.ring-cyan-200\/5{--tw-ring-color:rgba(165,243,252,.05)}.ring-cyan-200\/50{--tw-ring-color:rgba(165,243,252,.5)}.ring-cyan-200\/55{--tw-ring-color:rgba(165,243,252,.55)}.ring-cyan-200\/60{--tw-ring-color:rgba(165,243,252,.6)}.ring-cyan-200\/65{--tw-ring-color:rgba(165,243,252,.65)}.ring-cyan-200\/70{--tw-ring-color:rgba(165,243,252,.7)}.ring-cyan-200\/75{--tw-ring-color:rgba(165,243,252,.75)}.ring-cyan-200\/80{--tw-ring-color:rgba(165,243,252,.8)}.ring-cyan-200\/85{--tw-ring-color:rgba(165,243,252,.85)}.ring-cyan-200\/90{--tw-ring-color:rgba(165,243,252,.9)}.ring-cyan-200\/95{--tw-ring-color:rgba(165,243,252,.95)}.ring-cyan-400{--tw-ring-opacity:1;--tw-ring-color:rgb(34 211 238/var(--tw-ring-opacity,1))}.ring-cyan-400\/0{--tw-ring-color:rgba(34,211,238,0)}.ring-cyan-400\/10{--tw-ring-color:rgba(34,211,238,.1)}.ring-cyan-400\/100{--tw-ring-color:#22d3ee}.ring-cyan-400\/15{--tw-ring-color:rgba(34,211,238,.15)}.ring-cyan-400\/20{--tw-ring-color:rgba(34,211,238,.2)}.ring-cyan-400\/25{--tw-ring-color:rgba(34,211,238,.25)}.ring-cyan-400\/30{--tw-ring-color:rgba(34,211,238,.3)}.ring-cyan-400\/35{--tw-ring-color:rgba(34,211,238,.35)}.ring-cyan-400\/40{--tw-ring-color:rgba(34,211,238,.4)}.ring-cyan-400\/45{--tw-ring-color:rgba(34,211,238,.45)}.ring-cyan-400\/5{--tw-ring-color:rgba(34,211,238,.05)}.ring-cyan-400\/50{--tw-ring-color:rgba(34,211,238,.5)}.ring-cyan-400\/55{--tw-ring-color:rgba(34,211,238,.55)}.ring-cyan-400\/60{--tw-ring-color:rgba(34,211,238,.6)}.ring-cyan-400\/65{--tw-ring-color:rgba(34,211,238,.65)}.ring-cyan-400\/70{--tw-ring-color:rgba(34,211,238,.7)}.ring-cyan-400\/75{--tw-ring-color:rgba(34,211,238,.75)}.ring-cyan-400\/80{--tw-ring-color:rgba(34,211,238,.8)}.ring-cyan-400\/85{--tw-ring-color:rgba(34,211,238,.85)}.ring-cyan-400\/90{--tw-ring-color:rgba(34,211,238,.9)}.ring-cyan-400\/95{--tw-ring-color:rgba(34,211,238,.95)}.ring-cyan-50{--tw-ring-opacity:1;--tw-ring-color:rgb(236 254 255/var(--tw-ring-opacity,1))}.ring-cyan-50\/0{--tw-ring-color:rgba(236,254,255,0)}.ring-cyan-50\/10{--tw-ring-color:rgba(236,254,255,.1)}.ring-cyan-50\/100{--tw-ring-color:#ecfeff}.ring-cyan-50\/15{--tw-ring-color:rgba(236,254,255,.15)}.ring-cyan-50\/20{--tw-ring-color:rgba(236,254,255,.2)}.ring-cyan-50\/25{--tw-ring-color:rgba(236,254,255,.25)}.ring-cyan-50\/30{--tw-ring-color:rgba(236,254,255,.3)}.ring-cyan-50\/35{--tw-ring-color:rgba(236,254,255,.35)}.ring-cyan-50\/40{--tw-ring-color:rgba(236,254,255,.4)}.ring-cyan-50\/45{--tw-ring-color:rgba(236,254,255,.45)}.ring-cyan-50\/5{--tw-ring-color:rgba(236,254,255,.05)}.ring-cyan-50\/50{--tw-ring-color:rgba(236,254,255,.5)}.ring-cyan-50\/55{--tw-ring-color:rgba(236,254,255,.55)}.ring-cyan-50\/60{--tw-ring-color:rgba(236,254,255,.6)}.ring-cyan-50\/65{--tw-ring-color:rgba(236,254,255,.65)}.ring-cyan-50\/70{--tw-ring-color:rgba(236,254,255,.7)}.ring-cyan-50\/75{--tw-ring-color:rgba(236,254,255,.75)}.ring-cyan-50\/80{--tw-ring-color:rgba(236,254,255,.8)}.ring-cyan-50\/85{--tw-ring-color:rgba(236,254,255,.85)}.ring-cyan-50\/90{--tw-ring-color:rgba(236,254,255,.9)}.ring-cyan-50\/95{--tw-ring-color:rgba(236,254,255,.95)}.ring-cyan-500{--tw-ring-opacity:1;--tw-ring-color:rgb(6 182 212/var(--tw-ring-opacity,1))}.ring-cyan-500\/0{--tw-ring-color:rgba(6,182,212,0)}.ring-cyan-500\/10{--tw-ring-color:rgba(6,182,212,.1)}.ring-cyan-500\/100{--tw-ring-color:#06b6d4}.ring-cyan-500\/15{--tw-ring-color:rgba(6,182,212,.15)}.ring-cyan-500\/20{--tw-ring-color:rgba(6,182,212,.2)}.ring-cyan-500\/25{--tw-ring-color:rgba(6,182,212,.25)}.ring-cyan-500\/30{--tw-ring-color:rgba(6,182,212,.3)}.ring-cyan-500\/35{--tw-ring-color:rgba(6,182,212,.35)}.ring-cyan-500\/40{--tw-ring-color:rgba(6,182,212,.4)}.ring-cyan-500\/45{--tw-ring-color:rgba(6,182,212,.45)}.ring-cyan-500\/5{--tw-ring-color:rgba(6,182,212,.05)}.ring-cyan-500\/50{--tw-ring-color:rgba(6,182,212,.5)}.ring-cyan-500\/55{--tw-ring-color:rgba(6,182,212,.55)}.ring-cyan-500\/60{--tw-ring-color:rgba(6,182,212,.6)}.ring-cyan-500\/65{--tw-ring-color:rgba(6,182,212,.65)}.ring-cyan-500\/70{--tw-ring-color:rgba(6,182,212,.7)}.ring-cyan-500\/75{--tw-ring-color:rgba(6,182,212,.75)}.ring-cyan-500\/80{--tw-ring-color:rgba(6,182,212,.8)}.ring-cyan-500\/85{--tw-ring-color:rgba(6,182,212,.85)}.ring-cyan-500\/90{--tw-ring-color:rgba(6,182,212,.9)}.ring-cyan-500\/95{--tw-ring-color:rgba(6,182,212,.95)}.ring-cyan-950{--tw-ring-opacity:1;--tw-ring-color:rgb(8 51 68/var(--tw-ring-opacity,1))}.ring-cyan-950\/0{--tw-ring-color:rgba(8,51,68,0)}.ring-cyan-950\/10{--tw-ring-color:rgba(8,51,68,.1)}.ring-cyan-950\/100{--tw-ring-color:#083344}.ring-cyan-950\/15{--tw-ring-color:rgba(8,51,68,.15)}.ring-cyan-950\/20{--tw-ring-color:rgba(8,51,68,.2)}.ring-cyan-950\/25{--tw-ring-color:rgba(8,51,68,.25)}.ring-cyan-950\/30{--tw-ring-color:rgba(8,51,68,.3)}.ring-cyan-950\/35{--tw-ring-color:rgba(8,51,68,.35)}.ring-cyan-950\/40{--tw-ring-color:rgba(8,51,68,.4)}.ring-cyan-950\/45{--tw-ring-color:rgba(8,51,68,.45)}.ring-cyan-950\/5{--tw-ring-color:rgba(8,51,68,.05)}.ring-cyan-950\/50{--tw-ring-color:rgba(8,51,68,.5)}.ring-cyan-950\/55{--tw-ring-color:rgba(8,51,68,.55)}.ring-cyan-950\/60{--tw-ring-color:rgba(8,51,68,.6)}.ring-cyan-950\/65{--tw-ring-color:rgba(8,51,68,.65)}.ring-cyan-950\/70{--tw-ring-color:rgba(8,51,68,.7)}.ring-cyan-950\/75{--tw-ring-color:rgba(8,51,68,.75)}.ring-cyan-950\/80{--tw-ring-color:rgba(8,51,68,.8)}.ring-cyan-950\/85{--tw-ring-color:rgba(8,51,68,.85)}.ring-cyan-950\/90{--tw-ring-color:rgba(8,51,68,.9)}.ring-cyan-950\/95{--tw-ring-color:rgba(8,51,68,.95)}.ring-emerald-100{--tw-ring-opacity:1;--tw-ring-color:rgb(209 250 229/var(--tw-ring-opacity,1))}.ring-emerald-100\/0{--tw-ring-color:rgba(209,250,229,0)}.ring-emerald-100\/10{--tw-ring-color:rgba(209,250,229,.1)}.ring-emerald-100\/100{--tw-ring-color:#d1fae5}.ring-emerald-100\/15{--tw-ring-color:rgba(209,250,229,.15)}.ring-emerald-100\/20{--tw-ring-color:rgba(209,250,229,.2)}.ring-emerald-100\/25{--tw-ring-color:rgba(209,250,229,.25)}.ring-emerald-100\/30{--tw-ring-color:rgba(209,250,229,.3)}.ring-emerald-100\/35{--tw-ring-color:rgba(209,250,229,.35)}.ring-emerald-100\/40{--tw-ring-color:rgba(209,250,229,.4)}.ring-emerald-100\/45{--tw-ring-color:rgba(209,250,229,.45)}.ring-emerald-100\/5{--tw-ring-color:rgba(209,250,229,.05)}.ring-emerald-100\/50{--tw-ring-color:rgba(209,250,229,.5)}.ring-emerald-100\/55{--tw-ring-color:rgba(209,250,229,.55)}.ring-emerald-100\/60{--tw-ring-color:rgba(209,250,229,.6)}.ring-emerald-100\/65{--tw-ring-color:rgba(209,250,229,.65)}.ring-emerald-100\/70{--tw-ring-color:rgba(209,250,229,.7)}.ring-emerald-100\/75{--tw-ring-color:rgba(209,250,229,.75)}.ring-emerald-100\/80{--tw-ring-color:rgba(209,250,229,.8)}.ring-emerald-100\/85{--tw-ring-color:rgba(209,250,229,.85)}.ring-emerald-100\/90{--tw-ring-color:rgba(209,250,229,.9)}.ring-emerald-100\/95{--tw-ring-color:rgba(209,250,229,.95)}.ring-emerald-200{--tw-ring-opacity:1;--tw-ring-color:rgb(167 243 208/var(--tw-ring-opacity,1))}.ring-emerald-200\/0{--tw-ring-color:rgba(167,243,208,0)}.ring-emerald-200\/10{--tw-ring-color:rgba(167,243,208,.1)}.ring-emerald-200\/100{--tw-ring-color:#a7f3d0}.ring-emerald-200\/15{--tw-ring-color:rgba(167,243,208,.15)}.ring-emerald-200\/20{--tw-ring-color:rgba(167,243,208,.2)}.ring-emerald-200\/25{--tw-ring-color:rgba(167,243,208,.25)}.ring-emerald-200\/30{--tw-ring-color:rgba(167,243,208,.3)}.ring-emerald-200\/35{--tw-ring-color:rgba(167,243,208,.35)}.ring-emerald-200\/40{--tw-ring-color:rgba(167,243,208,.4)}.ring-emerald-200\/45{--tw-ring-color:rgba(167,243,208,.45)}.ring-emerald-200\/5{--tw-ring-color:rgba(167,243,208,.05)}.ring-emerald-200\/50{--tw-ring-color:rgba(167,243,208,.5)}.ring-emerald-200\/55{--tw-ring-color:rgba(167,243,208,.55)}.ring-emerald-200\/60{--tw-ring-color:rgba(167,243,208,.6)}.ring-emerald-200\/65{--tw-ring-color:rgba(167,243,208,.65)}.ring-emerald-200\/70{--tw-ring-color:rgba(167,243,208,.7)}.ring-emerald-200\/75{--tw-ring-color:rgba(167,243,208,.75)}.ring-emerald-200\/80{--tw-ring-color:rgba(167,243,208,.8)}.ring-emerald-200\/85{--tw-ring-color:rgba(167,243,208,.85)}.ring-emerald-200\/90{--tw-ring-color:rgba(167,243,208,.9)}.ring-emerald-200\/95{--tw-ring-color:rgba(167,243,208,.95)}.ring-emerald-400{--tw-ring-opacity:1;--tw-ring-color:rgb(52 211 153/var(--tw-ring-opacity,1))}.ring-emerald-400\/0{--tw-ring-color:rgba(52,211,153,0)}.ring-emerald-400\/10{--tw-ring-color:rgba(52,211,153,.1)}.ring-emerald-400\/100{--tw-ring-color:#34d399}.ring-emerald-400\/15{--tw-ring-color:rgba(52,211,153,.15)}.ring-emerald-400\/20{--tw-ring-color:rgba(52,211,153,.2)}.ring-emerald-400\/25{--tw-ring-color:rgba(52,211,153,.25)}.ring-emerald-400\/30{--tw-ring-color:rgba(52,211,153,.3)}.ring-emerald-400\/35{--tw-ring-color:rgba(52,211,153,.35)}.ring-emerald-400\/40{--tw-ring-color:rgba(52,211,153,.4)}.ring-emerald-400\/45{--tw-ring-color:rgba(52,211,153,.45)}.ring-emerald-400\/5{--tw-ring-color:rgba(52,211,153,.05)}.ring-emerald-400\/50{--tw-ring-color:rgba(52,211,153,.5)}.ring-emerald-400\/55{--tw-ring-color:rgba(52,211,153,.55)}.ring-emerald-400\/60{--tw-ring-color:rgba(52,211,153,.6)}.ring-emerald-400\/65{--tw-ring-color:rgba(52,211,153,.65)}.ring-emerald-400\/70{--tw-ring-color:rgba(52,211,153,.7)}.ring-emerald-400\/75{--tw-ring-color:rgba(52,211,153,.75)}.ring-emerald-400\/80{--tw-ring-color:rgba(52,211,153,.8)}.ring-emerald-400\/85{--tw-ring-color:rgba(52,211,153,.85)}.ring-emerald-400\/90{--tw-ring-color:rgba(52,211,153,.9)}.ring-emerald-400\/95{--tw-ring-color:rgba(52,211,153,.95)}.ring-emerald-50{--tw-ring-opacity:1;--tw-ring-color:rgb(236 253 245/var(--tw-ring-opacity,1))}.ring-emerald-50\/0{--tw-ring-color:rgba(236,253,245,0)}.ring-emerald-50\/10{--tw-ring-color:rgba(236,253,245,.1)}.ring-emerald-50\/100{--tw-ring-color:#ecfdf5}.ring-emerald-50\/15{--tw-ring-color:rgba(236,253,245,.15)}.ring-emerald-50\/20{--tw-ring-color:rgba(236,253,245,.2)}.ring-emerald-50\/25{--tw-ring-color:rgba(236,253,245,.25)}.ring-emerald-50\/30{--tw-ring-color:rgba(236,253,245,.3)}.ring-emerald-50\/35{--tw-ring-color:rgba(236,253,245,.35)}.ring-emerald-50\/40{--tw-ring-color:rgba(236,253,245,.4)}.ring-emerald-50\/45{--tw-ring-color:rgba(236,253,245,.45)}.ring-emerald-50\/5{--tw-ring-color:rgba(236,253,245,.05)}.ring-emerald-50\/50{--tw-ring-color:rgba(236,253,245,.5)}.ring-emerald-50\/55{--tw-ring-color:rgba(236,253,245,.55)}.ring-emerald-50\/60{--tw-ring-color:rgba(236,253,245,.6)}.ring-emerald-50\/65{--tw-ring-color:rgba(236,253,245,.65)}.ring-emerald-50\/70{--tw-ring-color:rgba(236,253,245,.7)}.ring-emerald-50\/75{--tw-ring-color:rgba(236,253,245,.75)}.ring-emerald-50\/80{--tw-ring-color:rgba(236,253,245,.8)}.ring-emerald-50\/85{--tw-ring-color:rgba(236,253,245,.85)}.ring-emerald-50\/90{--tw-ring-color:rgba(236,253,245,.9)}.ring-emerald-50\/95{--tw-ring-color:rgba(236,253,245,.95)}.ring-emerald-500{--tw-ring-opacity:1;--tw-ring-color:rgb(16 185 129/var(--tw-ring-opacity,1))}.ring-emerald-500\/0{--tw-ring-color:rgba(16,185,129,0)}.ring-emerald-500\/10{--tw-ring-color:rgba(16,185,129,.1)}.ring-emerald-500\/100{--tw-ring-color:#10b981}.ring-emerald-500\/15{--tw-ring-color:rgba(16,185,129,.15)}.ring-emerald-500\/20{--tw-ring-color:rgba(16,185,129,.2)}.ring-emerald-500\/25{--tw-ring-color:rgba(16,185,129,.25)}.ring-emerald-500\/30{--tw-ring-color:rgba(16,185,129,.3)}.ring-emerald-500\/35{--tw-ring-color:rgba(16,185,129,.35)}.ring-emerald-500\/40{--tw-ring-color:rgba(16,185,129,.4)}.ring-emerald-500\/45{--tw-ring-color:rgba(16,185,129,.45)}.ring-emerald-500\/5{--tw-ring-color:rgba(16,185,129,.05)}.ring-emerald-500\/50{--tw-ring-color:rgba(16,185,129,.5)}.ring-emerald-500\/55{--tw-ring-color:rgba(16,185,129,.55)}.ring-emerald-500\/60{--tw-ring-color:rgba(16,185,129,.6)}.ring-emerald-500\/65{--tw-ring-color:rgba(16,185,129,.65)}.ring-emerald-500\/70{--tw-ring-color:rgba(16,185,129,.7)}.ring-emerald-500\/75{--tw-ring-color:rgba(16,185,129,.75)}.ring-emerald-500\/80{--tw-ring-color:rgba(16,185,129,.8)}.ring-emerald-500\/85{--tw-ring-color:rgba(16,185,129,.85)}.ring-emerald-500\/90{--tw-ring-color:rgba(16,185,129,.9)}.ring-emerald-500\/95{--tw-ring-color:rgba(16,185,129,.95)}.ring-emerald-950{--tw-ring-opacity:1;--tw-ring-color:rgb(2 44 34/var(--tw-ring-opacity,1))}.ring-emerald-950\/0{--tw-ring-color:rgba(2,44,34,0)}.ring-emerald-950\/10{--tw-ring-color:rgba(2,44,34,.1)}.ring-emerald-950\/100{--tw-ring-color:#022c22}.ring-emerald-950\/15{--tw-ring-color:rgba(2,44,34,.15)}.ring-emerald-950\/20{--tw-ring-color:rgba(2,44,34,.2)}.ring-emerald-950\/25{--tw-ring-color:rgba(2,44,34,.25)}.ring-emerald-950\/30{--tw-ring-color:rgba(2,44,34,.3)}.ring-emerald-950\/35{--tw-ring-color:rgba(2,44,34,.35)}.ring-emerald-950\/40{--tw-ring-color:rgba(2,44,34,.4)}.ring-emerald-950\/45{--tw-ring-color:rgba(2,44,34,.45)}.ring-emerald-950\/5{--tw-ring-color:rgba(2,44,34,.05)}.ring-emerald-950\/50{--tw-ring-color:rgba(2,44,34,.5)}.ring-emerald-950\/55{--tw-ring-color:rgba(2,44,34,.55)}.ring-emerald-950\/60{--tw-ring-color:rgba(2,44,34,.6)}.ring-emerald-950\/65{--tw-ring-color:rgba(2,44,34,.65)}.ring-emerald-950\/70{--tw-ring-color:rgba(2,44,34,.7)}.ring-emerald-950\/75{--tw-ring-color:rgba(2,44,34,.75)}.ring-emerald-950\/80{--tw-ring-color:rgba(2,44,34,.8)}.ring-emerald-950\/85{--tw-ring-color:rgba(2,44,34,.85)}.ring-emerald-950\/90{--tw-ring-color:rgba(2,44,34,.9)}.ring-emerald-950\/95{--tw-ring-color:rgba(2,44,34,.95)}.ring-fuchsia-100{--tw-ring-opacity:1;--tw-ring-color:rgb(250 232 255/var(--tw-ring-opacity,1))}.ring-fuchsia-100\/0{--tw-ring-color:rgba(250,232,255,0)}.ring-fuchsia-100\/10{--tw-ring-color:rgba(250,232,255,.1)}.ring-fuchsia-100\/100{--tw-ring-color:#fae8ff}.ring-fuchsia-100\/15{--tw-ring-color:rgba(250,232,255,.15)}.ring-fuchsia-100\/20{--tw-ring-color:rgba(250,232,255,.2)}.ring-fuchsia-100\/25{--tw-ring-color:rgba(250,232,255,.25)}.ring-fuchsia-100\/30{--tw-ring-color:rgba(250,232,255,.3)}.ring-fuchsia-100\/35{--tw-ring-color:rgba(250,232,255,.35)}.ring-fuchsia-100\/40{--tw-ring-color:rgba(250,232,255,.4)}.ring-fuchsia-100\/45{--tw-ring-color:rgba(250,232,255,.45)}.ring-fuchsia-100\/5{--tw-ring-color:rgba(250,232,255,.05)}.ring-fuchsia-100\/50{--tw-ring-color:rgba(250,232,255,.5)}.ring-fuchsia-100\/55{--tw-ring-color:rgba(250,232,255,.55)}.ring-fuchsia-100\/60{--tw-ring-color:rgba(250,232,255,.6)}.ring-fuchsia-100\/65{--tw-ring-color:rgba(250,232,255,.65)}.ring-fuchsia-100\/70{--tw-ring-color:rgba(250,232,255,.7)}.ring-fuchsia-100\/75{--tw-ring-color:rgba(250,232,255,.75)}.ring-fuchsia-100\/80{--tw-ring-color:rgba(250,232,255,.8)}.ring-fuchsia-100\/85{--tw-ring-color:rgba(250,232,255,.85)}.ring-fuchsia-100\/90{--tw-ring-color:rgba(250,232,255,.9)}.ring-fuchsia-100\/95{--tw-ring-color:rgba(250,232,255,.95)}.ring-fuchsia-200{--tw-ring-opacity:1;--tw-ring-color:rgb(245 208 254/var(--tw-ring-opacity,1))}.ring-fuchsia-200\/0{--tw-ring-color:rgba(245,208,254,0)}.ring-fuchsia-200\/10{--tw-ring-color:rgba(245,208,254,.1)}.ring-fuchsia-200\/100{--tw-ring-color:#f5d0fe}.ring-fuchsia-200\/15{--tw-ring-color:rgba(245,208,254,.15)}.ring-fuchsia-200\/20{--tw-ring-color:rgba(245,208,254,.2)}.ring-fuchsia-200\/25{--tw-ring-color:rgba(245,208,254,.25)}.ring-fuchsia-200\/30{--tw-ring-color:rgba(245,208,254,.3)}.ring-fuchsia-200\/35{--tw-ring-color:rgba(245,208,254,.35)}.ring-fuchsia-200\/40{--tw-ring-color:rgba(245,208,254,.4)}.ring-fuchsia-200\/45{--tw-ring-color:rgba(245,208,254,.45)}.ring-fuchsia-200\/5{--tw-ring-color:rgba(245,208,254,.05)}.ring-fuchsia-200\/50{--tw-ring-color:rgba(245,208,254,.5)}.ring-fuchsia-200\/55{--tw-ring-color:rgba(245,208,254,.55)}.ring-fuchsia-200\/60{--tw-ring-color:rgba(245,208,254,.6)}.ring-fuchsia-200\/65{--tw-ring-color:rgba(245,208,254,.65)}.ring-fuchsia-200\/70{--tw-ring-color:rgba(245,208,254,.7)}.ring-fuchsia-200\/75{--tw-ring-color:rgba(245,208,254,.75)}.ring-fuchsia-200\/80{--tw-ring-color:rgba(245,208,254,.8)}.ring-fuchsia-200\/85{--tw-ring-color:rgba(245,208,254,.85)}.ring-fuchsia-200\/90{--tw-ring-color:rgba(245,208,254,.9)}.ring-fuchsia-200\/95{--tw-ring-color:rgba(245,208,254,.95)}.ring-fuchsia-400{--tw-ring-opacity:1;--tw-ring-color:rgb(232 121 249/var(--tw-ring-opacity,1))}.ring-fuchsia-400\/0{--tw-ring-color:rgba(232,121,249,0)}.ring-fuchsia-400\/10{--tw-ring-color:rgba(232,121,249,.1)}.ring-fuchsia-400\/100{--tw-ring-color:#e879f9}.ring-fuchsia-400\/15{--tw-ring-color:rgba(232,121,249,.15)}.ring-fuchsia-400\/20{--tw-ring-color:rgba(232,121,249,.2)}.ring-fuchsia-400\/25{--tw-ring-color:rgba(232,121,249,.25)}.ring-fuchsia-400\/30{--tw-ring-color:rgba(232,121,249,.3)}.ring-fuchsia-400\/35{--tw-ring-color:rgba(232,121,249,.35)}.ring-fuchsia-400\/40{--tw-ring-color:rgba(232,121,249,.4)}.ring-fuchsia-400\/45{--tw-ring-color:rgba(232,121,249,.45)}.ring-fuchsia-400\/5{--tw-ring-color:rgba(232,121,249,.05)}.ring-fuchsia-400\/50{--tw-ring-color:rgba(232,121,249,.5)}.ring-fuchsia-400\/55{--tw-ring-color:rgba(232,121,249,.55)}.ring-fuchsia-400\/60{--tw-ring-color:rgba(232,121,249,.6)}.ring-fuchsia-400\/65{--tw-ring-color:rgba(232,121,249,.65)}.ring-fuchsia-400\/70{--tw-ring-color:rgba(232,121,249,.7)}.ring-fuchsia-400\/75{--tw-ring-color:rgba(232,121,249,.75)}.ring-fuchsia-400\/80{--tw-ring-color:rgba(232,121,249,.8)}.ring-fuchsia-400\/85{--tw-ring-color:rgba(232,121,249,.85)}.ring-fuchsia-400\/90{--tw-ring-color:rgba(232,121,249,.9)}.ring-fuchsia-400\/95{--tw-ring-color:rgba(232,121,249,.95)}.ring-fuchsia-50{--tw-ring-opacity:1;--tw-ring-color:rgb(253 244 255/var(--tw-ring-opacity,1))}.ring-fuchsia-50\/0{--tw-ring-color:rgba(253,244,255,0)}.ring-fuchsia-50\/10{--tw-ring-color:rgba(253,244,255,.1)}.ring-fuchsia-50\/100{--tw-ring-color:#fdf4ff}.ring-fuchsia-50\/15{--tw-ring-color:rgba(253,244,255,.15)}.ring-fuchsia-50\/20{--tw-ring-color:rgba(253,244,255,.2)}.ring-fuchsia-50\/25{--tw-ring-color:rgba(253,244,255,.25)}.ring-fuchsia-50\/30{--tw-ring-color:rgba(253,244,255,.3)}.ring-fuchsia-50\/35{--tw-ring-color:rgba(253,244,255,.35)}.ring-fuchsia-50\/40{--tw-ring-color:rgba(253,244,255,.4)}.ring-fuchsia-50\/45{--tw-ring-color:rgba(253,244,255,.45)}.ring-fuchsia-50\/5{--tw-ring-color:rgba(253,244,255,.05)}.ring-fuchsia-50\/50{--tw-ring-color:rgba(253,244,255,.5)}.ring-fuchsia-50\/55{--tw-ring-color:rgba(253,244,255,.55)}.ring-fuchsia-50\/60{--tw-ring-color:rgba(253,244,255,.6)}.ring-fuchsia-50\/65{--tw-ring-color:rgba(253,244,255,.65)}.ring-fuchsia-50\/70{--tw-ring-color:rgba(253,244,255,.7)}.ring-fuchsia-50\/75{--tw-ring-color:rgba(253,244,255,.75)}.ring-fuchsia-50\/80{--tw-ring-color:rgba(253,244,255,.8)}.ring-fuchsia-50\/85{--tw-ring-color:rgba(253,244,255,.85)}.ring-fuchsia-50\/90{--tw-ring-color:rgba(253,244,255,.9)}.ring-fuchsia-50\/95{--tw-ring-color:rgba(253,244,255,.95)}.ring-fuchsia-500{--tw-ring-opacity:1;--tw-ring-color:rgb(217 70 239/var(--tw-ring-opacity,1))}.ring-fuchsia-500\/0{--tw-ring-color:rgba(217,70,239,0)}.ring-fuchsia-500\/10{--tw-ring-color:rgba(217,70,239,.1)}.ring-fuchsia-500\/100{--tw-ring-color:#d946ef}.ring-fuchsia-500\/15{--tw-ring-color:rgba(217,70,239,.15)}.ring-fuchsia-500\/20{--tw-ring-color:rgba(217,70,239,.2)}.ring-fuchsia-500\/25{--tw-ring-color:rgba(217,70,239,.25)}.ring-fuchsia-500\/30{--tw-ring-color:rgba(217,70,239,.3)}.ring-fuchsia-500\/35{--tw-ring-color:rgba(217,70,239,.35)}.ring-fuchsia-500\/40{--tw-ring-color:rgba(217,70,239,.4)}.ring-fuchsia-500\/45{--tw-ring-color:rgba(217,70,239,.45)}.ring-fuchsia-500\/5{--tw-ring-color:rgba(217,70,239,.05)}.ring-fuchsia-500\/50{--tw-ring-color:rgba(217,70,239,.5)}.ring-fuchsia-500\/55{--tw-ring-color:rgba(217,70,239,.55)}.ring-fuchsia-500\/60{--tw-ring-color:rgba(217,70,239,.6)}.ring-fuchsia-500\/65{--tw-ring-color:rgba(217,70,239,.65)}.ring-fuchsia-500\/70{--tw-ring-color:rgba(217,70,239,.7)}.ring-fuchsia-500\/75{--tw-ring-color:rgba(217,70,239,.75)}.ring-fuchsia-500\/80{--tw-ring-color:rgba(217,70,239,.8)}.ring-fuchsia-500\/85{--tw-ring-color:rgba(217,70,239,.85)}.ring-fuchsia-500\/90{--tw-ring-color:rgba(217,70,239,.9)}.ring-fuchsia-500\/95{--tw-ring-color:rgba(217,70,239,.95)}.ring-fuchsia-950{--tw-ring-opacity:1;--tw-ring-color:rgb(74 4 78/var(--tw-ring-opacity,1))}.ring-fuchsia-950\/0{--tw-ring-color:rgba(74,4,78,0)}.ring-fuchsia-950\/10{--tw-ring-color:rgba(74,4,78,.1)}.ring-fuchsia-950\/100{--tw-ring-color:#4a044e}.ring-fuchsia-950\/15{--tw-ring-color:rgba(74,4,78,.15)}.ring-fuchsia-950\/20{--tw-ring-color:rgba(74,4,78,.2)}.ring-fuchsia-950\/25{--tw-ring-color:rgba(74,4,78,.25)}.ring-fuchsia-950\/30{--tw-ring-color:rgba(74,4,78,.3)}.ring-fuchsia-950\/35{--tw-ring-color:rgba(74,4,78,.35)}.ring-fuchsia-950\/40{--tw-ring-color:rgba(74,4,78,.4)}.ring-fuchsia-950\/45{--tw-ring-color:rgba(74,4,78,.45)}.ring-fuchsia-950\/5{--tw-ring-color:rgba(74,4,78,.05)}.ring-fuchsia-950\/50{--tw-ring-color:rgba(74,4,78,.5)}.ring-fuchsia-950\/55{--tw-ring-color:rgba(74,4,78,.55)}.ring-fuchsia-950\/60{--tw-ring-color:rgba(74,4,78,.6)}.ring-fuchsia-950\/65{--tw-ring-color:rgba(74,4,78,.65)}.ring-fuchsia-950\/70{--tw-ring-color:rgba(74,4,78,.7)}.ring-fuchsia-950\/75{--tw-ring-color:rgba(74,4,78,.75)}.ring-fuchsia-950\/80{--tw-ring-color:rgba(74,4,78,.8)}.ring-fuchsia-950\/85{--tw-ring-color:rgba(74,4,78,.85)}.ring-fuchsia-950\/90{--tw-ring-color:rgba(74,4,78,.9)}.ring-fuchsia-950\/95{--tw-ring-color:rgba(74,4,78,.95)}.ring-gray-100{--tw-ring-opacity:1;--tw-ring-color:rgb(243 244 246/var(--tw-ring-opacity,1))}.ring-gray-100\/0{--tw-ring-color:rgba(243,244,246,0)}.ring-gray-100\/10{--tw-ring-color:rgba(243,244,246,.1)}.ring-gray-100\/100{--tw-ring-color:#f3f4f6}.ring-gray-100\/15{--tw-ring-color:rgba(243,244,246,.15)}.ring-gray-100\/20{--tw-ring-color:rgba(243,244,246,.2)}.ring-gray-100\/25{--tw-ring-color:rgba(243,244,246,.25)}.ring-gray-100\/30{--tw-ring-color:rgba(243,244,246,.3)}.ring-gray-100\/35{--tw-ring-color:rgba(243,244,246,.35)}.ring-gray-100\/40{--tw-ring-color:rgba(243,244,246,.4)}.ring-gray-100\/45{--tw-ring-color:rgba(243,244,246,.45)}.ring-gray-100\/5{--tw-ring-color:rgba(243,244,246,.05)}.ring-gray-100\/50{--tw-ring-color:rgba(243,244,246,.5)}.ring-gray-100\/55{--tw-ring-color:rgba(243,244,246,.55)}.ring-gray-100\/60{--tw-ring-color:rgba(243,244,246,.6)}.ring-gray-100\/65{--tw-ring-color:rgba(243,244,246,.65)}.ring-gray-100\/70{--tw-ring-color:rgba(243,244,246,.7)}.ring-gray-100\/75{--tw-ring-color:rgba(243,244,246,.75)}.ring-gray-100\/80{--tw-ring-color:rgba(243,244,246,.8)}.ring-gray-100\/85{--tw-ring-color:rgba(243,244,246,.85)}.ring-gray-100\/90{--tw-ring-color:rgba(243,244,246,.9)}.ring-gray-100\/95{--tw-ring-color:rgba(243,244,246,.95)}.ring-gray-200{--tw-ring-opacity:1;--tw-ring-color:rgb(229 231 235/var(--tw-ring-opacity,1))}.ring-gray-200\/0{--tw-ring-color:rgba(229,231,235,0)}.ring-gray-200\/10{--tw-ring-color:rgba(229,231,235,.1)}.ring-gray-200\/100{--tw-ring-color:#e5e7eb}.ring-gray-200\/15{--tw-ring-color:rgba(229,231,235,.15)}.ring-gray-200\/20{--tw-ring-color:rgba(229,231,235,.2)}.ring-gray-200\/25{--tw-ring-color:rgba(229,231,235,.25)}.ring-gray-200\/30{--tw-ring-color:rgba(229,231,235,.3)}.ring-gray-200\/35{--tw-ring-color:rgba(229,231,235,.35)}.ring-gray-200\/40{--tw-ring-color:rgba(229,231,235,.4)}.ring-gray-200\/45{--tw-ring-color:rgba(229,231,235,.45)}.ring-gray-200\/5{--tw-ring-color:rgba(229,231,235,.05)}.ring-gray-200\/50{--tw-ring-color:rgba(229,231,235,.5)}.ring-gray-200\/55{--tw-ring-color:rgba(229,231,235,.55)}.ring-gray-200\/60{--tw-ring-color:rgba(229,231,235,.6)}.ring-gray-200\/65{--tw-ring-color:rgba(229,231,235,.65)}.ring-gray-200\/70{--tw-ring-color:rgba(229,231,235,.7)}.ring-gray-200\/75{--tw-ring-color:rgba(229,231,235,.75)}.ring-gray-200\/80{--tw-ring-color:rgba(229,231,235,.8)}.ring-gray-200\/85{--tw-ring-color:rgba(229,231,235,.85)}.ring-gray-200\/90{--tw-ring-color:rgba(229,231,235,.9)}.ring-gray-200\/95{--tw-ring-color:rgba(229,231,235,.95)}.ring-gray-400{--tw-ring-opacity:1;--tw-ring-color:rgb(156 163 175/var(--tw-ring-opacity,1))}.ring-gray-400\/0{--tw-ring-color:rgba(156,163,175,0)}.ring-gray-400\/10{--tw-ring-color:rgba(156,163,175,.1)}.ring-gray-400\/100{--tw-ring-color:#9ca3af}.ring-gray-400\/15{--tw-ring-color:rgba(156,163,175,.15)}.ring-gray-400\/20{--tw-ring-color:rgba(156,163,175,.2)}.ring-gray-400\/25{--tw-ring-color:rgba(156,163,175,.25)}.ring-gray-400\/30{--tw-ring-color:rgba(156,163,175,.3)}.ring-gray-400\/35{--tw-ring-color:rgba(156,163,175,.35)}.ring-gray-400\/40{--tw-ring-color:rgba(156,163,175,.4)}.ring-gray-400\/45{--tw-ring-color:rgba(156,163,175,.45)}.ring-gray-400\/5{--tw-ring-color:rgba(156,163,175,.05)}.ring-gray-400\/50{--tw-ring-color:rgba(156,163,175,.5)}.ring-gray-400\/55{--tw-ring-color:rgba(156,163,175,.55)}.ring-gray-400\/60{--tw-ring-color:rgba(156,163,175,.6)}.ring-gray-400\/65{--tw-ring-color:rgba(156,163,175,.65)}.ring-gray-400\/70{--tw-ring-color:rgba(156,163,175,.7)}.ring-gray-400\/75{--tw-ring-color:rgba(156,163,175,.75)}.ring-gray-400\/80{--tw-ring-color:rgba(156,163,175,.8)}.ring-gray-400\/85{--tw-ring-color:rgba(156,163,175,.85)}.ring-gray-400\/90{--tw-ring-color:rgba(156,163,175,.9)}.ring-gray-400\/95{--tw-ring-color:rgba(156,163,175,.95)}.ring-gray-50{--tw-ring-opacity:1;--tw-ring-color:rgb(249 250 251/var(--tw-ring-opacity,1))}.ring-gray-50\/0{--tw-ring-color:rgba(249,250,251,0)}.ring-gray-50\/10{--tw-ring-color:rgba(249,250,251,.1)}.ring-gray-50\/100{--tw-ring-color:#f9fafb}.ring-gray-50\/15{--tw-ring-color:rgba(249,250,251,.15)}.ring-gray-50\/20{--tw-ring-color:rgba(249,250,251,.2)}.ring-gray-50\/25{--tw-ring-color:rgba(249,250,251,.25)}.ring-gray-50\/30{--tw-ring-color:rgba(249,250,251,.3)}.ring-gray-50\/35{--tw-ring-color:rgba(249,250,251,.35)}.ring-gray-50\/40{--tw-ring-color:rgba(249,250,251,.4)}.ring-gray-50\/45{--tw-ring-color:rgba(249,250,251,.45)}.ring-gray-50\/5{--tw-ring-color:rgba(249,250,251,.05)}.ring-gray-50\/50{--tw-ring-color:rgba(249,250,251,.5)}.ring-gray-50\/55{--tw-ring-color:rgba(249,250,251,.55)}.ring-gray-50\/60{--tw-ring-color:rgba(249,250,251,.6)}.ring-gray-50\/65{--tw-ring-color:rgba(249,250,251,.65)}.ring-gray-50\/70{--tw-ring-color:rgba(249,250,251,.7)}.ring-gray-50\/75{--tw-ring-color:rgba(249,250,251,.75)}.ring-gray-50\/80{--tw-ring-color:rgba(249,250,251,.8)}.ring-gray-50\/85{--tw-ring-color:rgba(249,250,251,.85)}.ring-gray-50\/90{--tw-ring-color:rgba(249,250,251,.9)}.ring-gray-50\/95{--tw-ring-color:rgba(249,250,251,.95)}.ring-gray-500{--tw-ring-opacity:1;--tw-ring-color:rgb(107 114 128/var(--tw-ring-opacity,1))}.ring-gray-500\/0{--tw-ring-color:hsla(220,9%,46%,0)}.ring-gray-500\/10{--tw-ring-color:hsla(220,9%,46%,.1)}.ring-gray-500\/100{--tw-ring-color:#6b7280}.ring-gray-500\/15{--tw-ring-color:hsla(220,9%,46%,.15)}.ring-gray-500\/20{--tw-ring-color:hsla(220,9%,46%,.2)}.ring-gray-500\/25{--tw-ring-color:hsla(220,9%,46%,.25)}.ring-gray-500\/30{--tw-ring-color:hsla(220,9%,46%,.3)}.ring-gray-500\/35{--tw-ring-color:hsla(220,9%,46%,.35)}.ring-gray-500\/40{--tw-ring-color:hsla(220,9%,46%,.4)}.ring-gray-500\/45{--tw-ring-color:hsla(220,9%,46%,.45)}.ring-gray-500\/5{--tw-ring-color:hsla(220,9%,46%,.05)}.ring-gray-500\/50{--tw-ring-color:hsla(220,9%,46%,.5)}.ring-gray-500\/55{--tw-ring-color:hsla(220,9%,46%,.55)}.ring-gray-500\/60{--tw-ring-color:hsla(220,9%,46%,.6)}.ring-gray-500\/65{--tw-ring-color:hsla(220,9%,46%,.65)}.ring-gray-500\/70{--tw-ring-color:hsla(220,9%,46%,.7)}.ring-gray-500\/75{--tw-ring-color:hsla(220,9%,46%,.75)}.ring-gray-500\/80{--tw-ring-color:hsla(220,9%,46%,.8)}.ring-gray-500\/85{--tw-ring-color:hsla(220,9%,46%,.85)}.ring-gray-500\/90{--tw-ring-color:hsla(220,9%,46%,.9)}.ring-gray-500\/95{--tw-ring-color:hsla(220,9%,46%,.95)}.ring-gray-950{--tw-ring-opacity:1;--tw-ring-color:rgb(3 7 18/var(--tw-ring-opacity,1))}.ring-gray-950\/0{--tw-ring-color:rgba(3,7,18,0)}.ring-gray-950\/10{--tw-ring-color:rgba(3,7,18,.1)}.ring-gray-950\/100{--tw-ring-color:#030712}.ring-gray-950\/15{--tw-ring-color:rgba(3,7,18,.15)}.ring-gray-950\/20{--tw-ring-color:rgba(3,7,18,.2)}.ring-gray-950\/25{--tw-ring-color:rgba(3,7,18,.25)}.ring-gray-950\/30{--tw-ring-color:rgba(3,7,18,.3)}.ring-gray-950\/35{--tw-ring-color:rgba(3,7,18,.35)}.ring-gray-950\/40{--tw-ring-color:rgba(3,7,18,.4)}.ring-gray-950\/45{--tw-ring-color:rgba(3,7,18,.45)}.ring-gray-950\/5{--tw-ring-color:rgba(3,7,18,.05)}.ring-gray-950\/50{--tw-ring-color:rgba(3,7,18,.5)}.ring-gray-950\/55{--tw-ring-color:rgba(3,7,18,.55)}.ring-gray-950\/60{--tw-ring-color:rgba(3,7,18,.6)}.ring-gray-950\/65{--tw-ring-color:rgba(3,7,18,.65)}.ring-gray-950\/70{--tw-ring-color:rgba(3,7,18,.7)}.ring-gray-950\/75{--tw-ring-color:rgba(3,7,18,.75)}.ring-gray-950\/80{--tw-ring-color:rgba(3,7,18,.8)}.ring-gray-950\/85{--tw-ring-color:rgba(3,7,18,.85)}.ring-gray-950\/90{--tw-ring-color:rgba(3,7,18,.9)}.ring-gray-950\/95{--tw-ring-color:rgba(3,7,18,.95)}.ring-green-100{--tw-ring-opacity:1;--tw-ring-color:rgb(220 252 231/var(--tw-ring-opacity,1))}.ring-green-100\/0{--tw-ring-color:rgba(220,252,231,0)}.ring-green-100\/10{--tw-ring-color:rgba(220,252,231,.1)}.ring-green-100\/100{--tw-ring-color:#dcfce7}.ring-green-100\/15{--tw-ring-color:rgba(220,252,231,.15)}.ring-green-100\/20{--tw-ring-color:rgba(220,252,231,.2)}.ring-green-100\/25{--tw-ring-color:rgba(220,252,231,.25)}.ring-green-100\/30{--tw-ring-color:rgba(220,252,231,.3)}.ring-green-100\/35{--tw-ring-color:rgba(220,252,231,.35)}.ring-green-100\/40{--tw-ring-color:rgba(220,252,231,.4)}.ring-green-100\/45{--tw-ring-color:rgba(220,252,231,.45)}.ring-green-100\/5{--tw-ring-color:rgba(220,252,231,.05)}.ring-green-100\/50{--tw-ring-color:rgba(220,252,231,.5)}.ring-green-100\/55{--tw-ring-color:rgba(220,252,231,.55)}.ring-green-100\/60{--tw-ring-color:rgba(220,252,231,.6)}.ring-green-100\/65{--tw-ring-color:rgba(220,252,231,.65)}.ring-green-100\/70{--tw-ring-color:rgba(220,252,231,.7)}.ring-green-100\/75{--tw-ring-color:rgba(220,252,231,.75)}.ring-green-100\/80{--tw-ring-color:rgba(220,252,231,.8)}.ring-green-100\/85{--tw-ring-color:rgba(220,252,231,.85)}.ring-green-100\/90{--tw-ring-color:rgba(220,252,231,.9)}.ring-green-100\/95{--tw-ring-color:rgba(220,252,231,.95)}.ring-green-200{--tw-ring-opacity:1;--tw-ring-color:rgb(187 247 208/var(--tw-ring-opacity,1))}.ring-green-200\/0{--tw-ring-color:rgba(187,247,208,0)}.ring-green-200\/10{--tw-ring-color:rgba(187,247,208,.1)}.ring-green-200\/100{--tw-ring-color:#bbf7d0}.ring-green-200\/15{--tw-ring-color:rgba(187,247,208,.15)}.ring-green-200\/20{--tw-ring-color:rgba(187,247,208,.2)}.ring-green-200\/25{--tw-ring-color:rgba(187,247,208,.25)}.ring-green-200\/30{--tw-ring-color:rgba(187,247,208,.3)}.ring-green-200\/35{--tw-ring-color:rgba(187,247,208,.35)}.ring-green-200\/40{--tw-ring-color:rgba(187,247,208,.4)}.ring-green-200\/45{--tw-ring-color:rgba(187,247,208,.45)}.ring-green-200\/5{--tw-ring-color:rgba(187,247,208,.05)}.ring-green-200\/50{--tw-ring-color:rgba(187,247,208,.5)}.ring-green-200\/55{--tw-ring-color:rgba(187,247,208,.55)}.ring-green-200\/60{--tw-ring-color:rgba(187,247,208,.6)}.ring-green-200\/65{--tw-ring-color:rgba(187,247,208,.65)}.ring-green-200\/70{--tw-ring-color:rgba(187,247,208,.7)}.ring-green-200\/75{--tw-ring-color:rgba(187,247,208,.75)}.ring-green-200\/80{--tw-ring-color:rgba(187,247,208,.8)}.ring-green-200\/85{--tw-ring-color:rgba(187,247,208,.85)}.ring-green-200\/90{--tw-ring-color:rgba(187,247,208,.9)}.ring-green-200\/95{--tw-ring-color:rgba(187,247,208,.95)}.ring-green-400{--tw-ring-opacity:1;--tw-ring-color:rgb(74 222 128/var(--tw-ring-opacity,1))}.ring-green-400\/0{--tw-ring-color:rgba(74,222,128,0)}.ring-green-400\/10{--tw-ring-color:rgba(74,222,128,.1)}.ring-green-400\/100{--tw-ring-color:#4ade80}.ring-green-400\/15{--tw-ring-color:rgba(74,222,128,.15)}.ring-green-400\/20{--tw-ring-color:rgba(74,222,128,.2)}.ring-green-400\/25{--tw-ring-color:rgba(74,222,128,.25)}.ring-green-400\/30{--tw-ring-color:rgba(74,222,128,.3)}.ring-green-400\/35{--tw-ring-color:rgba(74,222,128,.35)}.ring-green-400\/40{--tw-ring-color:rgba(74,222,128,.4)}.ring-green-400\/45{--tw-ring-color:rgba(74,222,128,.45)}.ring-green-400\/5{--tw-ring-color:rgba(74,222,128,.05)}.ring-green-400\/50{--tw-ring-color:rgba(74,222,128,.5)}.ring-green-400\/55{--tw-ring-color:rgba(74,222,128,.55)}.ring-green-400\/60{--tw-ring-color:rgba(74,222,128,.6)}.ring-green-400\/65{--tw-ring-color:rgba(74,222,128,.65)}.ring-green-400\/70{--tw-ring-color:rgba(74,222,128,.7)}.ring-green-400\/75{--tw-ring-color:rgba(74,222,128,.75)}.ring-green-400\/80{--tw-ring-color:rgba(74,222,128,.8)}.ring-green-400\/85{--tw-ring-color:rgba(74,222,128,.85)}.ring-green-400\/90{--tw-ring-color:rgba(74,222,128,.9)}.ring-green-400\/95{--tw-ring-color:rgba(74,222,128,.95)}.ring-green-50{--tw-ring-opacity:1;--tw-ring-color:rgb(240 253 244/var(--tw-ring-opacity,1))}.ring-green-50\/0{--tw-ring-color:rgba(240,253,244,0)}.ring-green-50\/10{--tw-ring-color:rgba(240,253,244,.1)}.ring-green-50\/100{--tw-ring-color:#f0fdf4}.ring-green-50\/15{--tw-ring-color:rgba(240,253,244,.15)}.ring-green-50\/20{--tw-ring-color:rgba(240,253,244,.2)}.ring-green-50\/25{--tw-ring-color:rgba(240,253,244,.25)}.ring-green-50\/30{--tw-ring-color:rgba(240,253,244,.3)}.ring-green-50\/35{--tw-ring-color:rgba(240,253,244,.35)}.ring-green-50\/40{--tw-ring-color:rgba(240,253,244,.4)}.ring-green-50\/45{--tw-ring-color:rgba(240,253,244,.45)}.ring-green-50\/5{--tw-ring-color:rgba(240,253,244,.05)}.ring-green-50\/50{--tw-ring-color:rgba(240,253,244,.5)}.ring-green-50\/55{--tw-ring-color:rgba(240,253,244,.55)}.ring-green-50\/60{--tw-ring-color:rgba(240,253,244,.6)}.ring-green-50\/65{--tw-ring-color:rgba(240,253,244,.65)}.ring-green-50\/70{--tw-ring-color:rgba(240,253,244,.7)}.ring-green-50\/75{--tw-ring-color:rgba(240,253,244,.75)}.ring-green-50\/80{--tw-ring-color:rgba(240,253,244,.8)}.ring-green-50\/85{--tw-ring-color:rgba(240,253,244,.85)}.ring-green-50\/90{--tw-ring-color:rgba(240,253,244,.9)}.ring-green-50\/95{--tw-ring-color:rgba(240,253,244,.95)}.ring-green-500{--tw-ring-opacity:1;--tw-ring-color:rgb(34 197 94/var(--tw-ring-opacity,1))}.ring-green-500\/0{--tw-ring-color:rgba(34,197,94,0)}.ring-green-500\/10{--tw-ring-color:rgba(34,197,94,.1)}.ring-green-500\/100{--tw-ring-color:#22c55e}.ring-green-500\/15{--tw-ring-color:rgba(34,197,94,.15)}.ring-green-500\/20{--tw-ring-color:rgba(34,197,94,.2)}.ring-green-500\/25{--tw-ring-color:rgba(34,197,94,.25)}.ring-green-500\/30{--tw-ring-color:rgba(34,197,94,.3)}.ring-green-500\/35{--tw-ring-color:rgba(34,197,94,.35)}.ring-green-500\/40{--tw-ring-color:rgba(34,197,94,.4)}.ring-green-500\/45{--tw-ring-color:rgba(34,197,94,.45)}.ring-green-500\/5{--tw-ring-color:rgba(34,197,94,.05)}.ring-green-500\/50{--tw-ring-color:rgba(34,197,94,.5)}.ring-green-500\/55{--tw-ring-color:rgba(34,197,94,.55)}.ring-green-500\/60{--tw-ring-color:rgba(34,197,94,.6)}.ring-green-500\/65{--tw-ring-color:rgba(34,197,94,.65)}.ring-green-500\/70{--tw-ring-color:rgba(34,197,94,.7)}.ring-green-500\/75{--tw-ring-color:rgba(34,197,94,.75)}.ring-green-500\/80{--tw-ring-color:rgba(34,197,94,.8)}.ring-green-500\/85{--tw-ring-color:rgba(34,197,94,.85)}.ring-green-500\/90{--tw-ring-color:rgba(34,197,94,.9)}.ring-green-500\/95{--tw-ring-color:rgba(34,197,94,.95)}.ring-green-950{--tw-ring-opacity:1;--tw-ring-color:rgb(5 46 22/var(--tw-ring-opacity,1))}.ring-green-950\/0{--tw-ring-color:rgba(5,46,22,0)}.ring-green-950\/10{--tw-ring-color:rgba(5,46,22,.1)}.ring-green-950\/100{--tw-ring-color:#052e16}.ring-green-950\/15{--tw-ring-color:rgba(5,46,22,.15)}.ring-green-950\/20{--tw-ring-color:rgba(5,46,22,.2)}.ring-green-950\/25{--tw-ring-color:rgba(5,46,22,.25)}.ring-green-950\/30{--tw-ring-color:rgba(5,46,22,.3)}.ring-green-950\/35{--tw-ring-color:rgba(5,46,22,.35)}.ring-green-950\/40{--tw-ring-color:rgba(5,46,22,.4)}.ring-green-950\/45{--tw-ring-color:rgba(5,46,22,.45)}.ring-green-950\/5{--tw-ring-color:rgba(5,46,22,.05)}.ring-green-950\/50{--tw-ring-color:rgba(5,46,22,.5)}.ring-green-950\/55{--tw-ring-color:rgba(5,46,22,.55)}.ring-green-950\/60{--tw-ring-color:rgba(5,46,22,.6)}.ring-green-950\/65{--tw-ring-color:rgba(5,46,22,.65)}.ring-green-950\/70{--tw-ring-color:rgba(5,46,22,.7)}.ring-green-950\/75{--tw-ring-color:rgba(5,46,22,.75)}.ring-green-950\/80{--tw-ring-color:rgba(5,46,22,.8)}.ring-green-950\/85{--tw-ring-color:rgba(5,46,22,.85)}.ring-green-950\/90{--tw-ring-color:rgba(5,46,22,.9)}.ring-green-950\/95{--tw-ring-color:rgba(5,46,22,.95)}.ring-indigo-100{--tw-ring-opacity:1;--tw-ring-color:rgb(224 231 255/var(--tw-ring-opacity,1))}.ring-indigo-100\/0{--tw-ring-color:rgba(224,231,255,0)}.ring-indigo-100\/10{--tw-ring-color:rgba(224,231,255,.1)}.ring-indigo-100\/100{--tw-ring-color:#e0e7ff}.ring-indigo-100\/15{--tw-ring-color:rgba(224,231,255,.15)}.ring-indigo-100\/20{--tw-ring-color:rgba(224,231,255,.2)}.ring-indigo-100\/25{--tw-ring-color:rgba(224,231,255,.25)}.ring-indigo-100\/30{--tw-ring-color:rgba(224,231,255,.3)}.ring-indigo-100\/35{--tw-ring-color:rgba(224,231,255,.35)}.ring-indigo-100\/40{--tw-ring-color:rgba(224,231,255,.4)}.ring-indigo-100\/45{--tw-ring-color:rgba(224,231,255,.45)}.ring-indigo-100\/5{--tw-ring-color:rgba(224,231,255,.05)}.ring-indigo-100\/50{--tw-ring-color:rgba(224,231,255,.5)}.ring-indigo-100\/55{--tw-ring-color:rgba(224,231,255,.55)}.ring-indigo-100\/60{--tw-ring-color:rgba(224,231,255,.6)}.ring-indigo-100\/65{--tw-ring-color:rgba(224,231,255,.65)}.ring-indigo-100\/70{--tw-ring-color:rgba(224,231,255,.7)}.ring-indigo-100\/75{--tw-ring-color:rgba(224,231,255,.75)}.ring-indigo-100\/80{--tw-ring-color:rgba(224,231,255,.8)}.ring-indigo-100\/85{--tw-ring-color:rgba(224,231,255,.85)}.ring-indigo-100\/90{--tw-ring-color:rgba(224,231,255,.9)}.ring-indigo-100\/95{--tw-ring-color:rgba(224,231,255,.95)}.ring-indigo-200{--tw-ring-opacity:1;--tw-ring-color:rgb(199 210 254/var(--tw-ring-opacity,1))}.ring-indigo-200\/0{--tw-ring-color:rgba(199,210,254,0)}.ring-indigo-200\/10{--tw-ring-color:rgba(199,210,254,.1)}.ring-indigo-200\/100{--tw-ring-color:#c7d2fe}.ring-indigo-200\/15{--tw-ring-color:rgba(199,210,254,.15)}.ring-indigo-200\/20{--tw-ring-color:rgba(199,210,254,.2)}.ring-indigo-200\/25{--tw-ring-color:rgba(199,210,254,.25)}.ring-indigo-200\/30{--tw-ring-color:rgba(199,210,254,.3)}.ring-indigo-200\/35{--tw-ring-color:rgba(199,210,254,.35)}.ring-indigo-200\/40{--tw-ring-color:rgba(199,210,254,.4)}.ring-indigo-200\/45{--tw-ring-color:rgba(199,210,254,.45)}.ring-indigo-200\/5{--tw-ring-color:rgba(199,210,254,.05)}.ring-indigo-200\/50{--tw-ring-color:rgba(199,210,254,.5)}.ring-indigo-200\/55{--tw-ring-color:rgba(199,210,254,.55)}.ring-indigo-200\/60{--tw-ring-color:rgba(199,210,254,.6)}.ring-indigo-200\/65{--tw-ring-color:rgba(199,210,254,.65)}.ring-indigo-200\/70{--tw-ring-color:rgba(199,210,254,.7)}.ring-indigo-200\/75{--tw-ring-color:rgba(199,210,254,.75)}.ring-indigo-200\/80{--tw-ring-color:rgba(199,210,254,.8)}.ring-indigo-200\/85{--tw-ring-color:rgba(199,210,254,.85)}.ring-indigo-200\/90{--tw-ring-color:rgba(199,210,254,.9)}.ring-indigo-200\/95{--tw-ring-color:rgba(199,210,254,.95)}.ring-indigo-400{--tw-ring-opacity:1;--tw-ring-color:rgb(129 140 248/var(--tw-ring-opacity,1))}.ring-indigo-400\/0{--tw-ring-color:rgba(129,140,248,0)}.ring-indigo-400\/10{--tw-ring-color:rgba(129,140,248,.1)}.ring-indigo-400\/100{--tw-ring-color:#818cf8}.ring-indigo-400\/15{--tw-ring-color:rgba(129,140,248,.15)}.ring-indigo-400\/20{--tw-ring-color:rgba(129,140,248,.2)}.ring-indigo-400\/25{--tw-ring-color:rgba(129,140,248,.25)}.ring-indigo-400\/30{--tw-ring-color:rgba(129,140,248,.3)}.ring-indigo-400\/35{--tw-ring-color:rgba(129,140,248,.35)}.ring-indigo-400\/40{--tw-ring-color:rgba(129,140,248,.4)}.ring-indigo-400\/45{--tw-ring-color:rgba(129,140,248,.45)}.ring-indigo-400\/5{--tw-ring-color:rgba(129,140,248,.05)}.ring-indigo-400\/50{--tw-ring-color:rgba(129,140,248,.5)}.ring-indigo-400\/55{--tw-ring-color:rgba(129,140,248,.55)}.ring-indigo-400\/60{--tw-ring-color:rgba(129,140,248,.6)}.ring-indigo-400\/65{--tw-ring-color:rgba(129,140,248,.65)}.ring-indigo-400\/70{--tw-ring-color:rgba(129,140,248,.7)}.ring-indigo-400\/75{--tw-ring-color:rgba(129,140,248,.75)}.ring-indigo-400\/80{--tw-ring-color:rgba(129,140,248,.8)}.ring-indigo-400\/85{--tw-ring-color:rgba(129,140,248,.85)}.ring-indigo-400\/90{--tw-ring-color:rgba(129,140,248,.9)}.ring-indigo-400\/95{--tw-ring-color:rgba(129,140,248,.95)}.ring-indigo-50{--tw-ring-opacity:1;--tw-ring-color:rgb(238 242 255/var(--tw-ring-opacity,1))}.ring-indigo-50\/0{--tw-ring-color:rgba(238,242,255,0)}.ring-indigo-50\/10{--tw-ring-color:rgba(238,242,255,.1)}.ring-indigo-50\/100{--tw-ring-color:#eef2ff}.ring-indigo-50\/15{--tw-ring-color:rgba(238,242,255,.15)}.ring-indigo-50\/20{--tw-ring-color:rgba(238,242,255,.2)}.ring-indigo-50\/25{--tw-ring-color:rgba(238,242,255,.25)}.ring-indigo-50\/30{--tw-ring-color:rgba(238,242,255,.3)}.ring-indigo-50\/35{--tw-ring-color:rgba(238,242,255,.35)}.ring-indigo-50\/40{--tw-ring-color:rgba(238,242,255,.4)}.ring-indigo-50\/45{--tw-ring-color:rgba(238,242,255,.45)}.ring-indigo-50\/5{--tw-ring-color:rgba(238,242,255,.05)}.ring-indigo-50\/50{--tw-ring-color:rgba(238,242,255,.5)}.ring-indigo-50\/55{--tw-ring-color:rgba(238,242,255,.55)}.ring-indigo-50\/60{--tw-ring-color:rgba(238,242,255,.6)}.ring-indigo-50\/65{--tw-ring-color:rgba(238,242,255,.65)}.ring-indigo-50\/70{--tw-ring-color:rgba(238,242,255,.7)}.ring-indigo-50\/75{--tw-ring-color:rgba(238,242,255,.75)}.ring-indigo-50\/80{--tw-ring-color:rgba(238,242,255,.8)}.ring-indigo-50\/85{--tw-ring-color:rgba(238,242,255,.85)}.ring-indigo-50\/90{--tw-ring-color:rgba(238,242,255,.9)}.ring-indigo-50\/95{--tw-ring-color:rgba(238,242,255,.95)}.ring-indigo-500{--tw-ring-opacity:1;--tw-ring-color:rgb(99 102 241/var(--tw-ring-opacity,1))}.ring-indigo-500\/0{--tw-ring-color:rgba(99,102,241,0)}.ring-indigo-500\/10{--tw-ring-color:rgba(99,102,241,.1)}.ring-indigo-500\/100{--tw-ring-color:#6366f1}.ring-indigo-500\/15{--tw-ring-color:rgba(99,102,241,.15)}.ring-indigo-500\/20{--tw-ring-color:rgba(99,102,241,.2)}.ring-indigo-500\/25{--tw-ring-color:rgba(99,102,241,.25)}.ring-indigo-500\/30{--tw-ring-color:rgba(99,102,241,.3)}.ring-indigo-500\/35{--tw-ring-color:rgba(99,102,241,.35)}.ring-indigo-500\/40{--tw-ring-color:rgba(99,102,241,.4)}.ring-indigo-500\/45{--tw-ring-color:rgba(99,102,241,.45)}.ring-indigo-500\/5{--tw-ring-color:rgba(99,102,241,.05)}.ring-indigo-500\/50{--tw-ring-color:rgba(99,102,241,.5)}.ring-indigo-500\/55{--tw-ring-color:rgba(99,102,241,.55)}.ring-indigo-500\/60{--tw-ring-color:rgba(99,102,241,.6)}.ring-indigo-500\/65{--tw-ring-color:rgba(99,102,241,.65)}.ring-indigo-500\/70{--tw-ring-color:rgba(99,102,241,.7)}.ring-indigo-500\/75{--tw-ring-color:rgba(99,102,241,.75)}.ring-indigo-500\/80{--tw-ring-color:rgba(99,102,241,.8)}.ring-indigo-500\/85{--tw-ring-color:rgba(99,102,241,.85)}.ring-indigo-500\/90{--tw-ring-color:rgba(99,102,241,.9)}.ring-indigo-500\/95{--tw-ring-color:rgba(99,102,241,.95)}.ring-indigo-950{--tw-ring-opacity:1;--tw-ring-color:rgb(30 27 75/var(--tw-ring-opacity,1))}.ring-indigo-950\/0{--tw-ring-color:rgba(30,27,75,0)}.ring-indigo-950\/10{--tw-ring-color:rgba(30,27,75,.1)}.ring-indigo-950\/100{--tw-ring-color:#1e1b4b}.ring-indigo-950\/15{--tw-ring-color:rgba(30,27,75,.15)}.ring-indigo-950\/20{--tw-ring-color:rgba(30,27,75,.2)}.ring-indigo-950\/25{--tw-ring-color:rgba(30,27,75,.25)}.ring-indigo-950\/30{--tw-ring-color:rgba(30,27,75,.3)}.ring-indigo-950\/35{--tw-ring-color:rgba(30,27,75,.35)}.ring-indigo-950\/40{--tw-ring-color:rgba(30,27,75,.4)}.ring-indigo-950\/45{--tw-ring-color:rgba(30,27,75,.45)}.ring-indigo-950\/5{--tw-ring-color:rgba(30,27,75,.05)}.ring-indigo-950\/50{--tw-ring-color:rgba(30,27,75,.5)}.ring-indigo-950\/55{--tw-ring-color:rgba(30,27,75,.55)}.ring-indigo-950\/60{--tw-ring-color:rgba(30,27,75,.6)}.ring-indigo-950\/65{--tw-ring-color:rgba(30,27,75,.65)}.ring-indigo-950\/70{--tw-ring-color:rgba(30,27,75,.7)}.ring-indigo-950\/75{--tw-ring-color:rgba(30,27,75,.75)}.ring-indigo-950\/80{--tw-ring-color:rgba(30,27,75,.8)}.ring-indigo-950\/85{--tw-ring-color:rgba(30,27,75,.85)}.ring-indigo-950\/90{--tw-ring-color:rgba(30,27,75,.9)}.ring-indigo-950\/95{--tw-ring-color:rgba(30,27,75,.95)}.ring-lime-100{--tw-ring-opacity:1;--tw-ring-color:rgb(236 252 203/var(--tw-ring-opacity,1))}.ring-lime-100\/0{--tw-ring-color:hsla(80,89%,89%,0)}.ring-lime-100\/10{--tw-ring-color:hsla(80,89%,89%,.1)}.ring-lime-100\/100{--tw-ring-color:#ecfccb}.ring-lime-100\/15{--tw-ring-color:hsla(80,89%,89%,.15)}.ring-lime-100\/20{--tw-ring-color:hsla(80,89%,89%,.2)}.ring-lime-100\/25{--tw-ring-color:hsla(80,89%,89%,.25)}.ring-lime-100\/30{--tw-ring-color:hsla(80,89%,89%,.3)}.ring-lime-100\/35{--tw-ring-color:hsla(80,89%,89%,.35)}.ring-lime-100\/40{--tw-ring-color:hsla(80,89%,89%,.4)}.ring-lime-100\/45{--tw-ring-color:hsla(80,89%,89%,.45)}.ring-lime-100\/5{--tw-ring-color:hsla(80,89%,89%,.05)}.ring-lime-100\/50{--tw-ring-color:hsla(80,89%,89%,.5)}.ring-lime-100\/55{--tw-ring-color:hsla(80,89%,89%,.55)}.ring-lime-100\/60{--tw-ring-color:hsla(80,89%,89%,.6)}.ring-lime-100\/65{--tw-ring-color:hsla(80,89%,89%,.65)}.ring-lime-100\/70{--tw-ring-color:hsla(80,89%,89%,.7)}.ring-lime-100\/75{--tw-ring-color:hsla(80,89%,89%,.75)}.ring-lime-100\/80{--tw-ring-color:hsla(80,89%,89%,.8)}.ring-lime-100\/85{--tw-ring-color:hsla(80,89%,89%,.85)}.ring-lime-100\/90{--tw-ring-color:hsla(80,89%,89%,.9)}.ring-lime-100\/95{--tw-ring-color:hsla(80,89%,89%,.95)}.ring-lime-200{--tw-ring-opacity:1;--tw-ring-color:rgb(217 249 157/var(--tw-ring-opacity,1))}.ring-lime-200\/0{--tw-ring-color:hsla(81,88%,80%,0)}.ring-lime-200\/10{--tw-ring-color:hsla(81,88%,80%,.1)}.ring-lime-200\/100{--tw-ring-color:#d9f99d}.ring-lime-200\/15{--tw-ring-color:hsla(81,88%,80%,.15)}.ring-lime-200\/20{--tw-ring-color:hsla(81,88%,80%,.2)}.ring-lime-200\/25{--tw-ring-color:hsla(81,88%,80%,.25)}.ring-lime-200\/30{--tw-ring-color:hsla(81,88%,80%,.3)}.ring-lime-200\/35{--tw-ring-color:hsla(81,88%,80%,.35)}.ring-lime-200\/40{--tw-ring-color:hsla(81,88%,80%,.4)}.ring-lime-200\/45{--tw-ring-color:hsla(81,88%,80%,.45)}.ring-lime-200\/5{--tw-ring-color:hsla(81,88%,80%,.05)}.ring-lime-200\/50{--tw-ring-color:hsla(81,88%,80%,.5)}.ring-lime-200\/55{--tw-ring-color:hsla(81,88%,80%,.55)}.ring-lime-200\/60{--tw-ring-color:hsla(81,88%,80%,.6)}.ring-lime-200\/65{--tw-ring-color:hsla(81,88%,80%,.65)}.ring-lime-200\/70{--tw-ring-color:hsla(81,88%,80%,.7)}.ring-lime-200\/75{--tw-ring-color:hsla(81,88%,80%,.75)}.ring-lime-200\/80{--tw-ring-color:hsla(81,88%,80%,.8)}.ring-lime-200\/85{--tw-ring-color:hsla(81,88%,80%,.85)}.ring-lime-200\/90{--tw-ring-color:hsla(81,88%,80%,.9)}.ring-lime-200\/95{--tw-ring-color:hsla(81,88%,80%,.95)}.ring-lime-400{--tw-ring-opacity:1;--tw-ring-color:rgb(163 230 53/var(--tw-ring-opacity,1))}.ring-lime-400\/0{--tw-ring-color:rgba(163,230,53,0)}.ring-lime-400\/10{--tw-ring-color:rgba(163,230,53,.1)}.ring-lime-400\/100{--tw-ring-color:#a3e635}.ring-lime-400\/15{--tw-ring-color:rgba(163,230,53,.15)}.ring-lime-400\/20{--tw-ring-color:rgba(163,230,53,.2)}.ring-lime-400\/25{--tw-ring-color:rgba(163,230,53,.25)}.ring-lime-400\/30{--tw-ring-color:rgba(163,230,53,.3)}.ring-lime-400\/35{--tw-ring-color:rgba(163,230,53,.35)}.ring-lime-400\/40{--tw-ring-color:rgba(163,230,53,.4)}.ring-lime-400\/45{--tw-ring-color:rgba(163,230,53,.45)}.ring-lime-400\/5{--tw-ring-color:rgba(163,230,53,.05)}.ring-lime-400\/50{--tw-ring-color:rgba(163,230,53,.5)}.ring-lime-400\/55{--tw-ring-color:rgba(163,230,53,.55)}.ring-lime-400\/60{--tw-ring-color:rgba(163,230,53,.6)}.ring-lime-400\/65{--tw-ring-color:rgba(163,230,53,.65)}.ring-lime-400\/70{--tw-ring-color:rgba(163,230,53,.7)}.ring-lime-400\/75{--tw-ring-color:rgba(163,230,53,.75)}.ring-lime-400\/80{--tw-ring-color:rgba(163,230,53,.8)}.ring-lime-400\/85{--tw-ring-color:rgba(163,230,53,.85)}.ring-lime-400\/90{--tw-ring-color:rgba(163,230,53,.9)}.ring-lime-400\/95{--tw-ring-color:rgba(163,230,53,.95)}.ring-lime-50{--tw-ring-opacity:1;--tw-ring-color:rgb(247 254 231/var(--tw-ring-opacity,1))}.ring-lime-50\/0{--tw-ring-color:hsla(78,92%,95%,0)}.ring-lime-50\/10{--tw-ring-color:hsla(78,92%,95%,.1)}.ring-lime-50\/100{--tw-ring-color:#f7fee7}.ring-lime-50\/15{--tw-ring-color:hsla(78,92%,95%,.15)}.ring-lime-50\/20{--tw-ring-color:hsla(78,92%,95%,.2)}.ring-lime-50\/25{--tw-ring-color:hsla(78,92%,95%,.25)}.ring-lime-50\/30{--tw-ring-color:hsla(78,92%,95%,.3)}.ring-lime-50\/35{--tw-ring-color:hsla(78,92%,95%,.35)}.ring-lime-50\/40{--tw-ring-color:hsla(78,92%,95%,.4)}.ring-lime-50\/45{--tw-ring-color:hsla(78,92%,95%,.45)}.ring-lime-50\/5{--tw-ring-color:hsla(78,92%,95%,.05)}.ring-lime-50\/50{--tw-ring-color:hsla(78,92%,95%,.5)}.ring-lime-50\/55{--tw-ring-color:hsla(78,92%,95%,.55)}.ring-lime-50\/60{--tw-ring-color:hsla(78,92%,95%,.6)}.ring-lime-50\/65{--tw-ring-color:hsla(78,92%,95%,.65)}.ring-lime-50\/70{--tw-ring-color:hsla(78,92%,95%,.7)}.ring-lime-50\/75{--tw-ring-color:hsla(78,92%,95%,.75)}.ring-lime-50\/80{--tw-ring-color:hsla(78,92%,95%,.8)}.ring-lime-50\/85{--tw-ring-color:hsla(78,92%,95%,.85)}.ring-lime-50\/90{--tw-ring-color:hsla(78,92%,95%,.9)}.ring-lime-50\/95{--tw-ring-color:hsla(78,92%,95%,.95)}.ring-lime-500{--tw-ring-opacity:1;--tw-ring-color:rgb(132 204 22/var(--tw-ring-opacity,1))}.ring-lime-500\/0{--tw-ring-color:rgba(132,204,22,0)}.ring-lime-500\/10{--tw-ring-color:rgba(132,204,22,.1)}.ring-lime-500\/100{--tw-ring-color:#84cc16}.ring-lime-500\/15{--tw-ring-color:rgba(132,204,22,.15)}.ring-lime-500\/20{--tw-ring-color:rgba(132,204,22,.2)}.ring-lime-500\/25{--tw-ring-color:rgba(132,204,22,.25)}.ring-lime-500\/30{--tw-ring-color:rgba(132,204,22,.3)}.ring-lime-500\/35{--tw-ring-color:rgba(132,204,22,.35)}.ring-lime-500\/40{--tw-ring-color:rgba(132,204,22,.4)}.ring-lime-500\/45{--tw-ring-color:rgba(132,204,22,.45)}.ring-lime-500\/5{--tw-ring-color:rgba(132,204,22,.05)}.ring-lime-500\/50{--tw-ring-color:rgba(132,204,22,.5)}.ring-lime-500\/55{--tw-ring-color:rgba(132,204,22,.55)}.ring-lime-500\/60{--tw-ring-color:rgba(132,204,22,.6)}.ring-lime-500\/65{--tw-ring-color:rgba(132,204,22,.65)}.ring-lime-500\/70{--tw-ring-color:rgba(132,204,22,.7)}.ring-lime-500\/75{--tw-ring-color:rgba(132,204,22,.75)}.ring-lime-500\/80{--tw-ring-color:rgba(132,204,22,.8)}.ring-lime-500\/85{--tw-ring-color:rgba(132,204,22,.85)}.ring-lime-500\/90{--tw-ring-color:rgba(132,204,22,.9)}.ring-lime-500\/95{--tw-ring-color:rgba(132,204,22,.95)}.ring-lime-950{--tw-ring-opacity:1;--tw-ring-color:rgb(26 46 5/var(--tw-ring-opacity,1))}.ring-lime-950\/0{--tw-ring-color:rgba(26,46,5,0)}.ring-lime-950\/10{--tw-ring-color:rgba(26,46,5,.1)}.ring-lime-950\/100{--tw-ring-color:#1a2e05}.ring-lime-950\/15{--tw-ring-color:rgba(26,46,5,.15)}.ring-lime-950\/20{--tw-ring-color:rgba(26,46,5,.2)}.ring-lime-950\/25{--tw-ring-color:rgba(26,46,5,.25)}.ring-lime-950\/30{--tw-ring-color:rgba(26,46,5,.3)}.ring-lime-950\/35{--tw-ring-color:rgba(26,46,5,.35)}.ring-lime-950\/40{--tw-ring-color:rgba(26,46,5,.4)}.ring-lime-950\/45{--tw-ring-color:rgba(26,46,5,.45)}.ring-lime-950\/5{--tw-ring-color:rgba(26,46,5,.05)}.ring-lime-950\/50{--tw-ring-color:rgba(26,46,5,.5)}.ring-lime-950\/55{--tw-ring-color:rgba(26,46,5,.55)}.ring-lime-950\/60{--tw-ring-color:rgba(26,46,5,.6)}.ring-lime-950\/65{--tw-ring-color:rgba(26,46,5,.65)}.ring-lime-950\/70{--tw-ring-color:rgba(26,46,5,.7)}.ring-lime-950\/75{--tw-ring-color:rgba(26,46,5,.75)}.ring-lime-950\/80{--tw-ring-color:rgba(26,46,5,.8)}.ring-lime-950\/85{--tw-ring-color:rgba(26,46,5,.85)}.ring-lime-950\/90{--tw-ring-color:rgba(26,46,5,.9)}.ring-lime-950\/95{--tw-ring-color:rgba(26,46,5,.95)}.ring-neutral-100{--tw-ring-opacity:1;--tw-ring-color:rgb(245 245 245/var(--tw-ring-opacity,1))}.ring-neutral-100\/0{--tw-ring-color:hsla(0,0%,96%,0)}.ring-neutral-100\/10{--tw-ring-color:hsla(0,0%,96%,.1)}.ring-neutral-100\/100{--tw-ring-color:#f5f5f5}.ring-neutral-100\/15{--tw-ring-color:hsla(0,0%,96%,.15)}.ring-neutral-100\/20{--tw-ring-color:hsla(0,0%,96%,.2)}.ring-neutral-100\/25{--tw-ring-color:hsla(0,0%,96%,.25)}.ring-neutral-100\/30{--tw-ring-color:hsla(0,0%,96%,.3)}.ring-neutral-100\/35{--tw-ring-color:hsla(0,0%,96%,.35)}.ring-neutral-100\/40{--tw-ring-color:hsla(0,0%,96%,.4)}.ring-neutral-100\/45{--tw-ring-color:hsla(0,0%,96%,.45)}.ring-neutral-100\/5{--tw-ring-color:hsla(0,0%,96%,.05)}.ring-neutral-100\/50{--tw-ring-color:hsla(0,0%,96%,.5)}.ring-neutral-100\/55{--tw-ring-color:hsla(0,0%,96%,.55)}.ring-neutral-100\/60{--tw-ring-color:hsla(0,0%,96%,.6)}.ring-neutral-100\/65{--tw-ring-color:hsla(0,0%,96%,.65)}.ring-neutral-100\/70{--tw-ring-color:hsla(0,0%,96%,.7)}.ring-neutral-100\/75{--tw-ring-color:hsla(0,0%,96%,.75)}.ring-neutral-100\/80{--tw-ring-color:hsla(0,0%,96%,.8)}.ring-neutral-100\/85{--tw-ring-color:hsla(0,0%,96%,.85)}.ring-neutral-100\/90{--tw-ring-color:hsla(0,0%,96%,.9)}.ring-neutral-100\/95{--tw-ring-color:hsla(0,0%,96%,.95)}.ring-neutral-200{--tw-ring-opacity:1;--tw-ring-color:rgb(229 229 229/var(--tw-ring-opacity,1))}.ring-neutral-200\/0{--tw-ring-color:hsla(0,0%,90%,0)}.ring-neutral-200\/10{--tw-ring-color:hsla(0,0%,90%,.1)}.ring-neutral-200\/100{--tw-ring-color:#e5e5e5}.ring-neutral-200\/15{--tw-ring-color:hsla(0,0%,90%,.15)}.ring-neutral-200\/20{--tw-ring-color:hsla(0,0%,90%,.2)}.ring-neutral-200\/25{--tw-ring-color:hsla(0,0%,90%,.25)}.ring-neutral-200\/30{--tw-ring-color:hsla(0,0%,90%,.3)}.ring-neutral-200\/35{--tw-ring-color:hsla(0,0%,90%,.35)}.ring-neutral-200\/40{--tw-ring-color:hsla(0,0%,90%,.4)}.ring-neutral-200\/45{--tw-ring-color:hsla(0,0%,90%,.45)}.ring-neutral-200\/5{--tw-ring-color:hsla(0,0%,90%,.05)}.ring-neutral-200\/50{--tw-ring-color:hsla(0,0%,90%,.5)}.ring-neutral-200\/55{--tw-ring-color:hsla(0,0%,90%,.55)}.ring-neutral-200\/60{--tw-ring-color:hsla(0,0%,90%,.6)}.ring-neutral-200\/65{--tw-ring-color:hsla(0,0%,90%,.65)}.ring-neutral-200\/70{--tw-ring-color:hsla(0,0%,90%,.7)}.ring-neutral-200\/75{--tw-ring-color:hsla(0,0%,90%,.75)}.ring-neutral-200\/80{--tw-ring-color:hsla(0,0%,90%,.8)}.ring-neutral-200\/85{--tw-ring-color:hsla(0,0%,90%,.85)}.ring-neutral-200\/90{--tw-ring-color:hsla(0,0%,90%,.9)}.ring-neutral-200\/95{--tw-ring-color:hsla(0,0%,90%,.95)}.ring-neutral-400{--tw-ring-opacity:1;--tw-ring-color:rgb(163 163 163/var(--tw-ring-opacity,1))}.ring-neutral-400\/0{--tw-ring-color:hsla(0,0%,64%,0)}.ring-neutral-400\/10{--tw-ring-color:hsla(0,0%,64%,.1)}.ring-neutral-400\/100{--tw-ring-color:#a3a3a3}.ring-neutral-400\/15{--tw-ring-color:hsla(0,0%,64%,.15)}.ring-neutral-400\/20{--tw-ring-color:hsla(0,0%,64%,.2)}.ring-neutral-400\/25{--tw-ring-color:hsla(0,0%,64%,.25)}.ring-neutral-400\/30{--tw-ring-color:hsla(0,0%,64%,.3)}.ring-neutral-400\/35{--tw-ring-color:hsla(0,0%,64%,.35)}.ring-neutral-400\/40{--tw-ring-color:hsla(0,0%,64%,.4)}.ring-neutral-400\/45{--tw-ring-color:hsla(0,0%,64%,.45)}.ring-neutral-400\/5{--tw-ring-color:hsla(0,0%,64%,.05)}.ring-neutral-400\/50{--tw-ring-color:hsla(0,0%,64%,.5)}.ring-neutral-400\/55{--tw-ring-color:hsla(0,0%,64%,.55)}.ring-neutral-400\/60{--tw-ring-color:hsla(0,0%,64%,.6)}.ring-neutral-400\/65{--tw-ring-color:hsla(0,0%,64%,.65)}.ring-neutral-400\/70{--tw-ring-color:hsla(0,0%,64%,.7)}.ring-neutral-400\/75{--tw-ring-color:hsla(0,0%,64%,.75)}.ring-neutral-400\/80{--tw-ring-color:hsla(0,0%,64%,.8)}.ring-neutral-400\/85{--tw-ring-color:hsla(0,0%,64%,.85)}.ring-neutral-400\/90{--tw-ring-color:hsla(0,0%,64%,.9)}.ring-neutral-400\/95{--tw-ring-color:hsla(0,0%,64%,.95)}.ring-neutral-50{--tw-ring-opacity:1;--tw-ring-color:rgb(250 250 250/var(--tw-ring-opacity,1))}.ring-neutral-50\/0{--tw-ring-color:hsla(0,0%,98%,0)}.ring-neutral-50\/10{--tw-ring-color:hsla(0,0%,98%,.1)}.ring-neutral-50\/100{--tw-ring-color:#fafafa}.ring-neutral-50\/15{--tw-ring-color:hsla(0,0%,98%,.15)}.ring-neutral-50\/20{--tw-ring-color:hsla(0,0%,98%,.2)}.ring-neutral-50\/25{--tw-ring-color:hsla(0,0%,98%,.25)}.ring-neutral-50\/30{--tw-ring-color:hsla(0,0%,98%,.3)}.ring-neutral-50\/35{--tw-ring-color:hsla(0,0%,98%,.35)}.ring-neutral-50\/40{--tw-ring-color:hsla(0,0%,98%,.4)}.ring-neutral-50\/45{--tw-ring-color:hsla(0,0%,98%,.45)}.ring-neutral-50\/5{--tw-ring-color:hsla(0,0%,98%,.05)}.ring-neutral-50\/50{--tw-ring-color:hsla(0,0%,98%,.5)}.ring-neutral-50\/55{--tw-ring-color:hsla(0,0%,98%,.55)}.ring-neutral-50\/60{--tw-ring-color:hsla(0,0%,98%,.6)}.ring-neutral-50\/65{--tw-ring-color:hsla(0,0%,98%,.65)}.ring-neutral-50\/70{--tw-ring-color:hsla(0,0%,98%,.7)}.ring-neutral-50\/75{--tw-ring-color:hsla(0,0%,98%,.75)}.ring-neutral-50\/80{--tw-ring-color:hsla(0,0%,98%,.8)}.ring-neutral-50\/85{--tw-ring-color:hsla(0,0%,98%,.85)}.ring-neutral-50\/90{--tw-ring-color:hsla(0,0%,98%,.9)}.ring-neutral-50\/95{--tw-ring-color:hsla(0,0%,98%,.95)}.ring-neutral-500{--tw-ring-opacity:1;--tw-ring-color:rgb(115 115 115/var(--tw-ring-opacity,1))}.ring-neutral-500\/0{--tw-ring-color:hsla(0,0%,45%,0)}.ring-neutral-500\/10{--tw-ring-color:hsla(0,0%,45%,.1)}.ring-neutral-500\/100{--tw-ring-color:#737373}.ring-neutral-500\/15{--tw-ring-color:hsla(0,0%,45%,.15)}.ring-neutral-500\/20{--tw-ring-color:hsla(0,0%,45%,.2)}.ring-neutral-500\/25{--tw-ring-color:hsla(0,0%,45%,.25)}.ring-neutral-500\/30{--tw-ring-color:hsla(0,0%,45%,.3)}.ring-neutral-500\/35{--tw-ring-color:hsla(0,0%,45%,.35)}.ring-neutral-500\/40{--tw-ring-color:hsla(0,0%,45%,.4)}.ring-neutral-500\/45{--tw-ring-color:hsla(0,0%,45%,.45)}.ring-neutral-500\/5{--tw-ring-color:hsla(0,0%,45%,.05)}.ring-neutral-500\/50{--tw-ring-color:hsla(0,0%,45%,.5)}.ring-neutral-500\/55{--tw-ring-color:hsla(0,0%,45%,.55)}.ring-neutral-500\/60{--tw-ring-color:hsla(0,0%,45%,.6)}.ring-neutral-500\/65{--tw-ring-color:hsla(0,0%,45%,.65)}.ring-neutral-500\/70{--tw-ring-color:hsla(0,0%,45%,.7)}.ring-neutral-500\/75{--tw-ring-color:hsla(0,0%,45%,.75)}.ring-neutral-500\/80{--tw-ring-color:hsla(0,0%,45%,.8)}.ring-neutral-500\/85{--tw-ring-color:hsla(0,0%,45%,.85)}.ring-neutral-500\/90{--tw-ring-color:hsla(0,0%,45%,.9)}.ring-neutral-500\/95{--tw-ring-color:hsla(0,0%,45%,.95)}.ring-neutral-950{--tw-ring-opacity:1;--tw-ring-color:rgb(10 10 10/var(--tw-ring-opacity,1))}.ring-neutral-950\/0{--tw-ring-color:hsla(0,0%,4%,0)}.ring-neutral-950\/10{--tw-ring-color:hsla(0,0%,4%,.1)}.ring-neutral-950\/100{--tw-ring-color:#0a0a0a}.ring-neutral-950\/15{--tw-ring-color:hsla(0,0%,4%,.15)}.ring-neutral-950\/20{--tw-ring-color:hsla(0,0%,4%,.2)}.ring-neutral-950\/25{--tw-ring-color:hsla(0,0%,4%,.25)}.ring-neutral-950\/30{--tw-ring-color:hsla(0,0%,4%,.3)}.ring-neutral-950\/35{--tw-ring-color:hsla(0,0%,4%,.35)}.ring-neutral-950\/40{--tw-ring-color:hsla(0,0%,4%,.4)}.ring-neutral-950\/45{--tw-ring-color:hsla(0,0%,4%,.45)}.ring-neutral-950\/5{--tw-ring-color:hsla(0,0%,4%,.05)}.ring-neutral-950\/50{--tw-ring-color:hsla(0,0%,4%,.5)}.ring-neutral-950\/55{--tw-ring-color:hsla(0,0%,4%,.55)}.ring-neutral-950\/60{--tw-ring-color:hsla(0,0%,4%,.6)}.ring-neutral-950\/65{--tw-ring-color:hsla(0,0%,4%,.65)}.ring-neutral-950\/70{--tw-ring-color:hsla(0,0%,4%,.7)}.ring-neutral-950\/75{--tw-ring-color:hsla(0,0%,4%,.75)}.ring-neutral-950\/80{--tw-ring-color:hsla(0,0%,4%,.8)}.ring-neutral-950\/85{--tw-ring-color:hsla(0,0%,4%,.85)}.ring-neutral-950\/90{--tw-ring-color:hsla(0,0%,4%,.9)}.ring-neutral-950\/95{--tw-ring-color:hsla(0,0%,4%,.95)}.ring-orange-100{--tw-ring-opacity:1;--tw-ring-color:rgb(255 237 213/var(--tw-ring-opacity,1))}.ring-orange-100\/0{--tw-ring-color:rgba(255,237,213,0)}.ring-orange-100\/10{--tw-ring-color:rgba(255,237,213,.1)}.ring-orange-100\/100{--tw-ring-color:#ffedd5}.ring-orange-100\/15{--tw-ring-color:rgba(255,237,213,.15)}.ring-orange-100\/20{--tw-ring-color:rgba(255,237,213,.2)}.ring-orange-100\/25{--tw-ring-color:rgba(255,237,213,.25)}.ring-orange-100\/30{--tw-ring-color:rgba(255,237,213,.3)}.ring-orange-100\/35{--tw-ring-color:rgba(255,237,213,.35)}.ring-orange-100\/40{--tw-ring-color:rgba(255,237,213,.4)}.ring-orange-100\/45{--tw-ring-color:rgba(255,237,213,.45)}.ring-orange-100\/5{--tw-ring-color:rgba(255,237,213,.05)}.ring-orange-100\/50{--tw-ring-color:rgba(255,237,213,.5)}.ring-orange-100\/55{--tw-ring-color:rgba(255,237,213,.55)}.ring-orange-100\/60{--tw-ring-color:rgba(255,237,213,.6)}.ring-orange-100\/65{--tw-ring-color:rgba(255,237,213,.65)}.ring-orange-100\/70{--tw-ring-color:rgba(255,237,213,.7)}.ring-orange-100\/75{--tw-ring-color:rgba(255,237,213,.75)}.ring-orange-100\/80{--tw-ring-color:rgba(255,237,213,.8)}.ring-orange-100\/85{--tw-ring-color:rgba(255,237,213,.85)}.ring-orange-100\/90{--tw-ring-color:rgba(255,237,213,.9)}.ring-orange-100\/95{--tw-ring-color:rgba(255,237,213,.95)}.ring-orange-200{--tw-ring-opacity:1;--tw-ring-color:rgb(254 215 170/var(--tw-ring-opacity,1))}.ring-orange-200\/0{--tw-ring-color:hsla(32,98%,83%,0)}.ring-orange-200\/10{--tw-ring-color:hsla(32,98%,83%,.1)}.ring-orange-200\/100{--tw-ring-color:#fed7aa}.ring-orange-200\/15{--tw-ring-color:hsla(32,98%,83%,.15)}.ring-orange-200\/20{--tw-ring-color:hsla(32,98%,83%,.2)}.ring-orange-200\/25{--tw-ring-color:hsla(32,98%,83%,.25)}.ring-orange-200\/30{--tw-ring-color:hsla(32,98%,83%,.3)}.ring-orange-200\/35{--tw-ring-color:hsla(32,98%,83%,.35)}.ring-orange-200\/40{--tw-ring-color:hsla(32,98%,83%,.4)}.ring-orange-200\/45{--tw-ring-color:hsla(32,98%,83%,.45)}.ring-orange-200\/5{--tw-ring-color:hsla(32,98%,83%,.05)}.ring-orange-200\/50{--tw-ring-color:hsla(32,98%,83%,.5)}.ring-orange-200\/55{--tw-ring-color:hsla(32,98%,83%,.55)}.ring-orange-200\/60{--tw-ring-color:hsla(32,98%,83%,.6)}.ring-orange-200\/65{--tw-ring-color:hsla(32,98%,83%,.65)}.ring-orange-200\/70{--tw-ring-color:hsla(32,98%,83%,.7)}.ring-orange-200\/75{--tw-ring-color:hsla(32,98%,83%,.75)}.ring-orange-200\/80{--tw-ring-color:hsla(32,98%,83%,.8)}.ring-orange-200\/85{--tw-ring-color:hsla(32,98%,83%,.85)}.ring-orange-200\/90{--tw-ring-color:hsla(32,98%,83%,.9)}.ring-orange-200\/95{--tw-ring-color:hsla(32,98%,83%,.95)}.ring-orange-400{--tw-ring-opacity:1;--tw-ring-color:rgb(251 146 60/var(--tw-ring-opacity,1))}.ring-orange-400\/0{--tw-ring-color:rgba(251,146,60,0)}.ring-orange-400\/10{--tw-ring-color:rgba(251,146,60,.1)}.ring-orange-400\/100{--tw-ring-color:#fb923c}.ring-orange-400\/15{--tw-ring-color:rgba(251,146,60,.15)}.ring-orange-400\/20{--tw-ring-color:rgba(251,146,60,.2)}.ring-orange-400\/25{--tw-ring-color:rgba(251,146,60,.25)}.ring-orange-400\/30{--tw-ring-color:rgba(251,146,60,.3)}.ring-orange-400\/35{--tw-ring-color:rgba(251,146,60,.35)}.ring-orange-400\/40{--tw-ring-color:rgba(251,146,60,.4)}.ring-orange-400\/45{--tw-ring-color:rgba(251,146,60,.45)}.ring-orange-400\/5{--tw-ring-color:rgba(251,146,60,.05)}.ring-orange-400\/50{--tw-ring-color:rgba(251,146,60,.5)}.ring-orange-400\/55{--tw-ring-color:rgba(251,146,60,.55)}.ring-orange-400\/60{--tw-ring-color:rgba(251,146,60,.6)}.ring-orange-400\/65{--tw-ring-color:rgba(251,146,60,.65)}.ring-orange-400\/70{--tw-ring-color:rgba(251,146,60,.7)}.ring-orange-400\/75{--tw-ring-color:rgba(251,146,60,.75)}.ring-orange-400\/80{--tw-ring-color:rgba(251,146,60,.8)}.ring-orange-400\/85{--tw-ring-color:rgba(251,146,60,.85)}.ring-orange-400\/90{--tw-ring-color:rgba(251,146,60,.9)}.ring-orange-400\/95{--tw-ring-color:rgba(251,146,60,.95)}.ring-orange-50{--tw-ring-opacity:1;--tw-ring-color:rgb(255 247 237/var(--tw-ring-opacity,1))}.ring-orange-50\/0{--tw-ring-color:rgba(255,247,237,0)}.ring-orange-50\/10{--tw-ring-color:rgba(255,247,237,.1)}.ring-orange-50\/100{--tw-ring-color:#fff7ed}.ring-orange-50\/15{--tw-ring-color:rgba(255,247,237,.15)}.ring-orange-50\/20{--tw-ring-color:rgba(255,247,237,.2)}.ring-orange-50\/25{--tw-ring-color:rgba(255,247,237,.25)}.ring-orange-50\/30{--tw-ring-color:rgba(255,247,237,.3)}.ring-orange-50\/35{--tw-ring-color:rgba(255,247,237,.35)}.ring-orange-50\/40{--tw-ring-color:rgba(255,247,237,.4)}.ring-orange-50\/45{--tw-ring-color:rgba(255,247,237,.45)}.ring-orange-50\/5{--tw-ring-color:rgba(255,247,237,.05)}.ring-orange-50\/50{--tw-ring-color:rgba(255,247,237,.5)}.ring-orange-50\/55{--tw-ring-color:rgba(255,247,237,.55)}.ring-orange-50\/60{--tw-ring-color:rgba(255,247,237,.6)}.ring-orange-50\/65{--tw-ring-color:rgba(255,247,237,.65)}.ring-orange-50\/70{--tw-ring-color:rgba(255,247,237,.7)}.ring-orange-50\/75{--tw-ring-color:rgba(255,247,237,.75)}.ring-orange-50\/80{--tw-ring-color:rgba(255,247,237,.8)}.ring-orange-50\/85{--tw-ring-color:rgba(255,247,237,.85)}.ring-orange-50\/90{--tw-ring-color:rgba(255,247,237,.9)}.ring-orange-50\/95{--tw-ring-color:rgba(255,247,237,.95)}.ring-orange-500{--tw-ring-opacity:1;--tw-ring-color:rgb(249 115 22/var(--tw-ring-opacity,1))}.ring-orange-500\/0{--tw-ring-color:rgba(249,115,22,0)}.ring-orange-500\/10{--tw-ring-color:rgba(249,115,22,.1)}.ring-orange-500\/100{--tw-ring-color:#f97316}.ring-orange-500\/15{--tw-ring-color:rgba(249,115,22,.15)}.ring-orange-500\/20{--tw-ring-color:rgba(249,115,22,.2)}.ring-orange-500\/25{--tw-ring-color:rgba(249,115,22,.25)}.ring-orange-500\/30{--tw-ring-color:rgba(249,115,22,.3)}.ring-orange-500\/35{--tw-ring-color:rgba(249,115,22,.35)}.ring-orange-500\/40{--tw-ring-color:rgba(249,115,22,.4)}.ring-orange-500\/45{--tw-ring-color:rgba(249,115,22,.45)}.ring-orange-500\/5{--tw-ring-color:rgba(249,115,22,.05)}.ring-orange-500\/50{--tw-ring-color:rgba(249,115,22,.5)}.ring-orange-500\/55{--tw-ring-color:rgba(249,115,22,.55)}.ring-orange-500\/60{--tw-ring-color:rgba(249,115,22,.6)}.ring-orange-500\/65{--tw-ring-color:rgba(249,115,22,.65)}.ring-orange-500\/70{--tw-ring-color:rgba(249,115,22,.7)}.ring-orange-500\/75{--tw-ring-color:rgba(249,115,22,.75)}.ring-orange-500\/80{--tw-ring-color:rgba(249,115,22,.8)}.ring-orange-500\/85{--tw-ring-color:rgba(249,115,22,.85)}.ring-orange-500\/90{--tw-ring-color:rgba(249,115,22,.9)}.ring-orange-500\/95{--tw-ring-color:rgba(249,115,22,.95)}.ring-orange-950{--tw-ring-opacity:1;--tw-ring-color:rgb(67 20 7/var(--tw-ring-opacity,1))}.ring-orange-950\/0{--tw-ring-color:rgba(67,20,7,0)}.ring-orange-950\/10{--tw-ring-color:rgba(67,20,7,.1)}.ring-orange-950\/100{--tw-ring-color:#431407}.ring-orange-950\/15{--tw-ring-color:rgba(67,20,7,.15)}.ring-orange-950\/20{--tw-ring-color:rgba(67,20,7,.2)}.ring-orange-950\/25{--tw-ring-color:rgba(67,20,7,.25)}.ring-orange-950\/30{--tw-ring-color:rgba(67,20,7,.3)}.ring-orange-950\/35{--tw-ring-color:rgba(67,20,7,.35)}.ring-orange-950\/40{--tw-ring-color:rgba(67,20,7,.4)}.ring-orange-950\/45{--tw-ring-color:rgba(67,20,7,.45)}.ring-orange-950\/5{--tw-ring-color:rgba(67,20,7,.05)}.ring-orange-950\/50{--tw-ring-color:rgba(67,20,7,.5)}.ring-orange-950\/55{--tw-ring-color:rgba(67,20,7,.55)}.ring-orange-950\/60{--tw-ring-color:rgba(67,20,7,.6)}.ring-orange-950\/65{--tw-ring-color:rgba(67,20,7,.65)}.ring-orange-950\/70{--tw-ring-color:rgba(67,20,7,.7)}.ring-orange-950\/75{--tw-ring-color:rgba(67,20,7,.75)}.ring-orange-950\/80{--tw-ring-color:rgba(67,20,7,.8)}.ring-orange-950\/85{--tw-ring-color:rgba(67,20,7,.85)}.ring-orange-950\/90{--tw-ring-color:rgba(67,20,7,.9)}.ring-orange-950\/95{--tw-ring-color:rgba(67,20,7,.95)}.ring-pink-100{--tw-ring-opacity:1;--tw-ring-color:rgb(252 231 243/var(--tw-ring-opacity,1))}.ring-pink-100\/0{--tw-ring-color:rgba(252,231,243,0)}.ring-pink-100\/10{--tw-ring-color:rgba(252,231,243,.1)}.ring-pink-100\/100{--tw-ring-color:#fce7f3}.ring-pink-100\/15{--tw-ring-color:rgba(252,231,243,.15)}.ring-pink-100\/20{--tw-ring-color:rgba(252,231,243,.2)}.ring-pink-100\/25{--tw-ring-color:rgba(252,231,243,.25)}.ring-pink-100\/30{--tw-ring-color:rgba(252,231,243,.3)}.ring-pink-100\/35{--tw-ring-color:rgba(252,231,243,.35)}.ring-pink-100\/40{--tw-ring-color:rgba(252,231,243,.4)}.ring-pink-100\/45{--tw-ring-color:rgba(252,231,243,.45)}.ring-pink-100\/5{--tw-ring-color:rgba(252,231,243,.05)}.ring-pink-100\/50{--tw-ring-color:rgba(252,231,243,.5)}.ring-pink-100\/55{--tw-ring-color:rgba(252,231,243,.55)}.ring-pink-100\/60{--tw-ring-color:rgba(252,231,243,.6)}.ring-pink-100\/65{--tw-ring-color:rgba(252,231,243,.65)}.ring-pink-100\/70{--tw-ring-color:rgba(252,231,243,.7)}.ring-pink-100\/75{--tw-ring-color:rgba(252,231,243,.75)}.ring-pink-100\/80{--tw-ring-color:rgba(252,231,243,.8)}.ring-pink-100\/85{--tw-ring-color:rgba(252,231,243,.85)}.ring-pink-100\/90{--tw-ring-color:rgba(252,231,243,.9)}.ring-pink-100\/95{--tw-ring-color:rgba(252,231,243,.95)}.ring-pink-200{--tw-ring-opacity:1;--tw-ring-color:rgb(251 207 232/var(--tw-ring-opacity,1))}.ring-pink-200\/0{--tw-ring-color:rgba(251,207,232,0)}.ring-pink-200\/10{--tw-ring-color:rgba(251,207,232,.1)}.ring-pink-200\/100{--tw-ring-color:#fbcfe8}.ring-pink-200\/15{--tw-ring-color:rgba(251,207,232,.15)}.ring-pink-200\/20{--tw-ring-color:rgba(251,207,232,.2)}.ring-pink-200\/25{--tw-ring-color:rgba(251,207,232,.25)}.ring-pink-200\/30{--tw-ring-color:rgba(251,207,232,.3)}.ring-pink-200\/35{--tw-ring-color:rgba(251,207,232,.35)}.ring-pink-200\/40{--tw-ring-color:rgba(251,207,232,.4)}.ring-pink-200\/45{--tw-ring-color:rgba(251,207,232,.45)}.ring-pink-200\/5{--tw-ring-color:rgba(251,207,232,.05)}.ring-pink-200\/50{--tw-ring-color:rgba(251,207,232,.5)}.ring-pink-200\/55{--tw-ring-color:rgba(251,207,232,.55)}.ring-pink-200\/60{--tw-ring-color:rgba(251,207,232,.6)}.ring-pink-200\/65{--tw-ring-color:rgba(251,207,232,.65)}.ring-pink-200\/70{--tw-ring-color:rgba(251,207,232,.7)}.ring-pink-200\/75{--tw-ring-color:rgba(251,207,232,.75)}.ring-pink-200\/80{--tw-ring-color:rgba(251,207,232,.8)}.ring-pink-200\/85{--tw-ring-color:rgba(251,207,232,.85)}.ring-pink-200\/90{--tw-ring-color:rgba(251,207,232,.9)}.ring-pink-200\/95{--tw-ring-color:rgba(251,207,232,.95)}.ring-pink-400{--tw-ring-opacity:1;--tw-ring-color:rgb(244 114 182/var(--tw-ring-opacity,1))}.ring-pink-400\/0{--tw-ring-color:rgba(244,114,182,0)}.ring-pink-400\/10{--tw-ring-color:rgba(244,114,182,.1)}.ring-pink-400\/100{--tw-ring-color:#f472b6}.ring-pink-400\/15{--tw-ring-color:rgba(244,114,182,.15)}.ring-pink-400\/20{--tw-ring-color:rgba(244,114,182,.2)}.ring-pink-400\/25{--tw-ring-color:rgba(244,114,182,.25)}.ring-pink-400\/30{--tw-ring-color:rgba(244,114,182,.3)}.ring-pink-400\/35{--tw-ring-color:rgba(244,114,182,.35)}.ring-pink-400\/40{--tw-ring-color:rgba(244,114,182,.4)}.ring-pink-400\/45{--tw-ring-color:rgba(244,114,182,.45)}.ring-pink-400\/5{--tw-ring-color:rgba(244,114,182,.05)}.ring-pink-400\/50{--tw-ring-color:rgba(244,114,182,.5)}.ring-pink-400\/55{--tw-ring-color:rgba(244,114,182,.55)}.ring-pink-400\/60{--tw-ring-color:rgba(244,114,182,.6)}.ring-pink-400\/65{--tw-ring-color:rgba(244,114,182,.65)}.ring-pink-400\/70{--tw-ring-color:rgba(244,114,182,.7)}.ring-pink-400\/75{--tw-ring-color:rgba(244,114,182,.75)}.ring-pink-400\/80{--tw-ring-color:rgba(244,114,182,.8)}.ring-pink-400\/85{--tw-ring-color:rgba(244,114,182,.85)}.ring-pink-400\/90{--tw-ring-color:rgba(244,114,182,.9)}.ring-pink-400\/95{--tw-ring-color:rgba(244,114,182,.95)}.ring-pink-50{--tw-ring-opacity:1;--tw-ring-color:rgb(253 242 248/var(--tw-ring-opacity,1))}.ring-pink-50\/0{--tw-ring-color:rgba(253,242,248,0)}.ring-pink-50\/10{--tw-ring-color:rgba(253,242,248,.1)}.ring-pink-50\/100{--tw-ring-color:#fdf2f8}.ring-pink-50\/15{--tw-ring-color:rgba(253,242,248,.15)}.ring-pink-50\/20{--tw-ring-color:rgba(253,242,248,.2)}.ring-pink-50\/25{--tw-ring-color:rgba(253,242,248,.25)}.ring-pink-50\/30{--tw-ring-color:rgba(253,242,248,.3)}.ring-pink-50\/35{--tw-ring-color:rgba(253,242,248,.35)}.ring-pink-50\/40{--tw-ring-color:rgba(253,242,248,.4)}.ring-pink-50\/45{--tw-ring-color:rgba(253,242,248,.45)}.ring-pink-50\/5{--tw-ring-color:rgba(253,242,248,.05)}.ring-pink-50\/50{--tw-ring-color:rgba(253,242,248,.5)}.ring-pink-50\/55{--tw-ring-color:rgba(253,242,248,.55)}.ring-pink-50\/60{--tw-ring-color:rgba(253,242,248,.6)}.ring-pink-50\/65{--tw-ring-color:rgba(253,242,248,.65)}.ring-pink-50\/70{--tw-ring-color:rgba(253,242,248,.7)}.ring-pink-50\/75{--tw-ring-color:rgba(253,242,248,.75)}.ring-pink-50\/80{--tw-ring-color:rgba(253,242,248,.8)}.ring-pink-50\/85{--tw-ring-color:rgba(253,242,248,.85)}.ring-pink-50\/90{--tw-ring-color:rgba(253,242,248,.9)}.ring-pink-50\/95{--tw-ring-color:rgba(253,242,248,.95)}.ring-pink-500{--tw-ring-opacity:1;--tw-ring-color:rgb(236 72 153/var(--tw-ring-opacity,1))}.ring-pink-500\/0{--tw-ring-color:rgba(236,72,153,0)}.ring-pink-500\/10{--tw-ring-color:rgba(236,72,153,.1)}.ring-pink-500\/100{--tw-ring-color:#ec4899}.ring-pink-500\/15{--tw-ring-color:rgba(236,72,153,.15)}.ring-pink-500\/20{--tw-ring-color:rgba(236,72,153,.2)}.ring-pink-500\/25{--tw-ring-color:rgba(236,72,153,.25)}.ring-pink-500\/30{--tw-ring-color:rgba(236,72,153,.3)}.ring-pink-500\/35{--tw-ring-color:rgba(236,72,153,.35)}.ring-pink-500\/40{--tw-ring-color:rgba(236,72,153,.4)}.ring-pink-500\/45{--tw-ring-color:rgba(236,72,153,.45)}.ring-pink-500\/5{--tw-ring-color:rgba(236,72,153,.05)}.ring-pink-500\/50{--tw-ring-color:rgba(236,72,153,.5)}.ring-pink-500\/55{--tw-ring-color:rgba(236,72,153,.55)}.ring-pink-500\/60{--tw-ring-color:rgba(236,72,153,.6)}.ring-pink-500\/65{--tw-ring-color:rgba(236,72,153,.65)}.ring-pink-500\/70{--tw-ring-color:rgba(236,72,153,.7)}.ring-pink-500\/75{--tw-ring-color:rgba(236,72,153,.75)}.ring-pink-500\/80{--tw-ring-color:rgba(236,72,153,.8)}.ring-pink-500\/85{--tw-ring-color:rgba(236,72,153,.85)}.ring-pink-500\/90{--tw-ring-color:rgba(236,72,153,.9)}.ring-pink-500\/95{--tw-ring-color:rgba(236,72,153,.95)}.ring-pink-950{--tw-ring-opacity:1;--tw-ring-color:rgb(80 7 36/var(--tw-ring-opacity,1))}.ring-pink-950\/0{--tw-ring-color:rgba(80,7,36,0)}.ring-pink-950\/10{--tw-ring-color:rgba(80,7,36,.1)}.ring-pink-950\/100{--tw-ring-color:#500724}.ring-pink-950\/15{--tw-ring-color:rgba(80,7,36,.15)}.ring-pink-950\/20{--tw-ring-color:rgba(80,7,36,.2)}.ring-pink-950\/25{--tw-ring-color:rgba(80,7,36,.25)}.ring-pink-950\/30{--tw-ring-color:rgba(80,7,36,.3)}.ring-pink-950\/35{--tw-ring-color:rgba(80,7,36,.35)}.ring-pink-950\/40{--tw-ring-color:rgba(80,7,36,.4)}.ring-pink-950\/45{--tw-ring-color:rgba(80,7,36,.45)}.ring-pink-950\/5{--tw-ring-color:rgba(80,7,36,.05)}.ring-pink-950\/50{--tw-ring-color:rgba(80,7,36,.5)}.ring-pink-950\/55{--tw-ring-color:rgba(80,7,36,.55)}.ring-pink-950\/60{--tw-ring-color:rgba(80,7,36,.6)}.ring-pink-950\/65{--tw-ring-color:rgba(80,7,36,.65)}.ring-pink-950\/70{--tw-ring-color:rgba(80,7,36,.7)}.ring-pink-950\/75{--tw-ring-color:rgba(80,7,36,.75)}.ring-pink-950\/80{--tw-ring-color:rgba(80,7,36,.8)}.ring-pink-950\/85{--tw-ring-color:rgba(80,7,36,.85)}.ring-pink-950\/90{--tw-ring-color:rgba(80,7,36,.9)}.ring-pink-950\/95{--tw-ring-color:rgba(80,7,36,.95)}.ring-purple-100{--tw-ring-opacity:1;--tw-ring-color:rgb(243 232 255/var(--tw-ring-opacity,1))}.ring-purple-100\/0{--tw-ring-color:rgba(243,232,255,0)}.ring-purple-100\/10{--tw-ring-color:rgba(243,232,255,.1)}.ring-purple-100\/100{--tw-ring-color:#f3e8ff}.ring-purple-100\/15{--tw-ring-color:rgba(243,232,255,.15)}.ring-purple-100\/20{--tw-ring-color:rgba(243,232,255,.2)}.ring-purple-100\/25{--tw-ring-color:rgba(243,232,255,.25)}.ring-purple-100\/30{--tw-ring-color:rgba(243,232,255,.3)}.ring-purple-100\/35{--tw-ring-color:rgba(243,232,255,.35)}.ring-purple-100\/40{--tw-ring-color:rgba(243,232,255,.4)}.ring-purple-100\/45{--tw-ring-color:rgba(243,232,255,.45)}.ring-purple-100\/5{--tw-ring-color:rgba(243,232,255,.05)}.ring-purple-100\/50{--tw-ring-color:rgba(243,232,255,.5)}.ring-purple-100\/55{--tw-ring-color:rgba(243,232,255,.55)}.ring-purple-100\/60{--tw-ring-color:rgba(243,232,255,.6)}.ring-purple-100\/65{--tw-ring-color:rgba(243,232,255,.65)}.ring-purple-100\/70{--tw-ring-color:rgba(243,232,255,.7)}.ring-purple-100\/75{--tw-ring-color:rgba(243,232,255,.75)}.ring-purple-100\/80{--tw-ring-color:rgba(243,232,255,.8)}.ring-purple-100\/85{--tw-ring-color:rgba(243,232,255,.85)}.ring-purple-100\/90{--tw-ring-color:rgba(243,232,255,.9)}.ring-purple-100\/95{--tw-ring-color:rgba(243,232,255,.95)}.ring-purple-200{--tw-ring-opacity:1;--tw-ring-color:rgb(233 213 255/var(--tw-ring-opacity,1))}.ring-purple-200\/0{--tw-ring-color:rgba(233,213,255,0)}.ring-purple-200\/10{--tw-ring-color:rgba(233,213,255,.1)}.ring-purple-200\/100{--tw-ring-color:#e9d5ff}.ring-purple-200\/15{--tw-ring-color:rgba(233,213,255,.15)}.ring-purple-200\/20{--tw-ring-color:rgba(233,213,255,.2)}.ring-purple-200\/25{--tw-ring-color:rgba(233,213,255,.25)}.ring-purple-200\/30{--tw-ring-color:rgba(233,213,255,.3)}.ring-purple-200\/35{--tw-ring-color:rgba(233,213,255,.35)}.ring-purple-200\/40{--tw-ring-color:rgba(233,213,255,.4)}.ring-purple-200\/45{--tw-ring-color:rgba(233,213,255,.45)}.ring-purple-200\/5{--tw-ring-color:rgba(233,213,255,.05)}.ring-purple-200\/50{--tw-ring-color:rgba(233,213,255,.5)}.ring-purple-200\/55{--tw-ring-color:rgba(233,213,255,.55)}.ring-purple-200\/60{--tw-ring-color:rgba(233,213,255,.6)}.ring-purple-200\/65{--tw-ring-color:rgba(233,213,255,.65)}.ring-purple-200\/70{--tw-ring-color:rgba(233,213,255,.7)}.ring-purple-200\/75{--tw-ring-color:rgba(233,213,255,.75)}.ring-purple-200\/80{--tw-ring-color:rgba(233,213,255,.8)}.ring-purple-200\/85{--tw-ring-color:rgba(233,213,255,.85)}.ring-purple-200\/90{--tw-ring-color:rgba(233,213,255,.9)}.ring-purple-200\/95{--tw-ring-color:rgba(233,213,255,.95)}.ring-purple-400{--tw-ring-opacity:1;--tw-ring-color:rgb(192 132 252/var(--tw-ring-opacity,1))}.ring-purple-400\/0{--tw-ring-color:rgba(192,132,252,0)}.ring-purple-400\/10{--tw-ring-color:rgba(192,132,252,.1)}.ring-purple-400\/100{--tw-ring-color:#c084fc}.ring-purple-400\/15{--tw-ring-color:rgba(192,132,252,.15)}.ring-purple-400\/20{--tw-ring-color:rgba(192,132,252,.2)}.ring-purple-400\/25{--tw-ring-color:rgba(192,132,252,.25)}.ring-purple-400\/30{--tw-ring-color:rgba(192,132,252,.3)}.ring-purple-400\/35{--tw-ring-color:rgba(192,132,252,.35)}.ring-purple-400\/40{--tw-ring-color:rgba(192,132,252,.4)}.ring-purple-400\/45{--tw-ring-color:rgba(192,132,252,.45)}.ring-purple-400\/5{--tw-ring-color:rgba(192,132,252,.05)}.ring-purple-400\/50{--tw-ring-color:rgba(192,132,252,.5)}.ring-purple-400\/55{--tw-ring-color:rgba(192,132,252,.55)}.ring-purple-400\/60{--tw-ring-color:rgba(192,132,252,.6)}.ring-purple-400\/65{--tw-ring-color:rgba(192,132,252,.65)}.ring-purple-400\/70{--tw-ring-color:rgba(192,132,252,.7)}.ring-purple-400\/75{--tw-ring-color:rgba(192,132,252,.75)}.ring-purple-400\/80{--tw-ring-color:rgba(192,132,252,.8)}.ring-purple-400\/85{--tw-ring-color:rgba(192,132,252,.85)}.ring-purple-400\/90{--tw-ring-color:rgba(192,132,252,.9)}.ring-purple-400\/95{--tw-ring-color:rgba(192,132,252,.95)}.ring-purple-50{--tw-ring-opacity:1;--tw-ring-color:rgb(250 245 255/var(--tw-ring-opacity,1))}.ring-purple-50\/0{--tw-ring-color:rgba(250,245,255,0)}.ring-purple-50\/10{--tw-ring-color:rgba(250,245,255,.1)}.ring-purple-50\/100{--tw-ring-color:#faf5ff}.ring-purple-50\/15{--tw-ring-color:rgba(250,245,255,.15)}.ring-purple-50\/20{--tw-ring-color:rgba(250,245,255,.2)}.ring-purple-50\/25{--tw-ring-color:rgba(250,245,255,.25)}.ring-purple-50\/30{--tw-ring-color:rgba(250,245,255,.3)}.ring-purple-50\/35{--tw-ring-color:rgba(250,245,255,.35)}.ring-purple-50\/40{--tw-ring-color:rgba(250,245,255,.4)}.ring-purple-50\/45{--tw-ring-color:rgba(250,245,255,.45)}.ring-purple-50\/5{--tw-ring-color:rgba(250,245,255,.05)}.ring-purple-50\/50{--tw-ring-color:rgba(250,245,255,.5)}.ring-purple-50\/55{--tw-ring-color:rgba(250,245,255,.55)}.ring-purple-50\/60{--tw-ring-color:rgba(250,245,255,.6)}.ring-purple-50\/65{--tw-ring-color:rgba(250,245,255,.65)}.ring-purple-50\/70{--tw-ring-color:rgba(250,245,255,.7)}.ring-purple-50\/75{--tw-ring-color:rgba(250,245,255,.75)}.ring-purple-50\/80{--tw-ring-color:rgba(250,245,255,.8)}.ring-purple-50\/85{--tw-ring-color:rgba(250,245,255,.85)}.ring-purple-50\/90{--tw-ring-color:rgba(250,245,255,.9)}.ring-purple-50\/95{--tw-ring-color:rgba(250,245,255,.95)}.ring-purple-500{--tw-ring-opacity:1;--tw-ring-color:rgb(168 85 247/var(--tw-ring-opacity,1))}.ring-purple-500\/0{--tw-ring-color:rgba(168,85,247,0)}.ring-purple-500\/10{--tw-ring-color:rgba(168,85,247,.1)}.ring-purple-500\/100{--tw-ring-color:#a855f7}.ring-purple-500\/15{--tw-ring-color:rgba(168,85,247,.15)}.ring-purple-500\/20{--tw-ring-color:rgba(168,85,247,.2)}.ring-purple-500\/25{--tw-ring-color:rgba(168,85,247,.25)}.ring-purple-500\/30{--tw-ring-color:rgba(168,85,247,.3)}.ring-purple-500\/35{--tw-ring-color:rgba(168,85,247,.35)}.ring-purple-500\/40{--tw-ring-color:rgba(168,85,247,.4)}.ring-purple-500\/45{--tw-ring-color:rgba(168,85,247,.45)}.ring-purple-500\/5{--tw-ring-color:rgba(168,85,247,.05)}.ring-purple-500\/50{--tw-ring-color:rgba(168,85,247,.5)}.ring-purple-500\/55{--tw-ring-color:rgba(168,85,247,.55)}.ring-purple-500\/60{--tw-ring-color:rgba(168,85,247,.6)}.ring-purple-500\/65{--tw-ring-color:rgba(168,85,247,.65)}.ring-purple-500\/70{--tw-ring-color:rgba(168,85,247,.7)}.ring-purple-500\/75{--tw-ring-color:rgba(168,85,247,.75)}.ring-purple-500\/80{--tw-ring-color:rgba(168,85,247,.8)}.ring-purple-500\/85{--tw-ring-color:rgba(168,85,247,.85)}.ring-purple-500\/90{--tw-ring-color:rgba(168,85,247,.9)}.ring-purple-500\/95{--tw-ring-color:rgba(168,85,247,.95)}.ring-purple-950{--tw-ring-opacity:1;--tw-ring-color:rgb(59 7 100/var(--tw-ring-opacity,1))}.ring-purple-950\/0{--tw-ring-color:rgba(59,7,100,0)}.ring-purple-950\/10{--tw-ring-color:rgba(59,7,100,.1)}.ring-purple-950\/100{--tw-ring-color:#3b0764}.ring-purple-950\/15{--tw-ring-color:rgba(59,7,100,.15)}.ring-purple-950\/20{--tw-ring-color:rgba(59,7,100,.2)}.ring-purple-950\/25{--tw-ring-color:rgba(59,7,100,.25)}.ring-purple-950\/30{--tw-ring-color:rgba(59,7,100,.3)}.ring-purple-950\/35{--tw-ring-color:rgba(59,7,100,.35)}.ring-purple-950\/40{--tw-ring-color:rgba(59,7,100,.4)}.ring-purple-950\/45{--tw-ring-color:rgba(59,7,100,.45)}.ring-purple-950\/5{--tw-ring-color:rgba(59,7,100,.05)}.ring-purple-950\/50{--tw-ring-color:rgba(59,7,100,.5)}.ring-purple-950\/55{--tw-ring-color:rgba(59,7,100,.55)}.ring-purple-950\/60{--tw-ring-color:rgba(59,7,100,.6)}.ring-purple-950\/65{--tw-ring-color:rgba(59,7,100,.65)}.ring-purple-950\/70{--tw-ring-color:rgba(59,7,100,.7)}.ring-purple-950\/75{--tw-ring-color:rgba(59,7,100,.75)}.ring-purple-950\/80{--tw-ring-color:rgba(59,7,100,.8)}.ring-purple-950\/85{--tw-ring-color:rgba(59,7,100,.85)}.ring-purple-950\/90{--tw-ring-color:rgba(59,7,100,.9)}.ring-purple-950\/95{--tw-ring-color:rgba(59,7,100,.95)}.ring-red-100{--tw-ring-opacity:1;--tw-ring-color:rgb(254 226 226/var(--tw-ring-opacity,1))}.ring-red-100\/0{--tw-ring-color:hsla(0,93%,94%,0)}.ring-red-100\/10{--tw-ring-color:hsla(0,93%,94%,.1)}.ring-red-100\/100{--tw-ring-color:#fee2e2}.ring-red-100\/15{--tw-ring-color:hsla(0,93%,94%,.15)}.ring-red-100\/20{--tw-ring-color:hsla(0,93%,94%,.2)}.ring-red-100\/25{--tw-ring-color:hsla(0,93%,94%,.25)}.ring-red-100\/30{--tw-ring-color:hsla(0,93%,94%,.3)}.ring-red-100\/35{--tw-ring-color:hsla(0,93%,94%,.35)}.ring-red-100\/40{--tw-ring-color:hsla(0,93%,94%,.4)}.ring-red-100\/45{--tw-ring-color:hsla(0,93%,94%,.45)}.ring-red-100\/5{--tw-ring-color:hsla(0,93%,94%,.05)}.ring-red-100\/50{--tw-ring-color:hsla(0,93%,94%,.5)}.ring-red-100\/55{--tw-ring-color:hsla(0,93%,94%,.55)}.ring-red-100\/60{--tw-ring-color:hsla(0,93%,94%,.6)}.ring-red-100\/65{--tw-ring-color:hsla(0,93%,94%,.65)}.ring-red-100\/70{--tw-ring-color:hsla(0,93%,94%,.7)}.ring-red-100\/75{--tw-ring-color:hsla(0,93%,94%,.75)}.ring-red-100\/80{--tw-ring-color:hsla(0,93%,94%,.8)}.ring-red-100\/85{--tw-ring-color:hsla(0,93%,94%,.85)}.ring-red-100\/90{--tw-ring-color:hsla(0,93%,94%,.9)}.ring-red-100\/95{--tw-ring-color:hsla(0,93%,94%,.95)}.ring-red-200{--tw-ring-opacity:1;--tw-ring-color:rgb(254 202 202/var(--tw-ring-opacity,1))}.ring-red-200\/0{--tw-ring-color:hsla(0,96%,89%,0)}.ring-red-200\/10{--tw-ring-color:hsla(0,96%,89%,.1)}.ring-red-200\/100{--tw-ring-color:#fecaca}.ring-red-200\/15{--tw-ring-color:hsla(0,96%,89%,.15)}.ring-red-200\/20{--tw-ring-color:hsla(0,96%,89%,.2)}.ring-red-200\/25{--tw-ring-color:hsla(0,96%,89%,.25)}.ring-red-200\/30{--tw-ring-color:hsla(0,96%,89%,.3)}.ring-red-200\/35{--tw-ring-color:hsla(0,96%,89%,.35)}.ring-red-200\/40{--tw-ring-color:hsla(0,96%,89%,.4)}.ring-red-200\/45{--tw-ring-color:hsla(0,96%,89%,.45)}.ring-red-200\/5{--tw-ring-color:hsla(0,96%,89%,.05)}.ring-red-200\/50{--tw-ring-color:hsla(0,96%,89%,.5)}.ring-red-200\/55{--tw-ring-color:hsla(0,96%,89%,.55)}.ring-red-200\/60{--tw-ring-color:hsla(0,96%,89%,.6)}.ring-red-200\/65{--tw-ring-color:hsla(0,96%,89%,.65)}.ring-red-200\/70{--tw-ring-color:hsla(0,96%,89%,.7)}.ring-red-200\/75{--tw-ring-color:hsla(0,96%,89%,.75)}.ring-red-200\/80{--tw-ring-color:hsla(0,96%,89%,.8)}.ring-red-200\/85{--tw-ring-color:hsla(0,96%,89%,.85)}.ring-red-200\/90{--tw-ring-color:hsla(0,96%,89%,.9)}.ring-red-200\/95{--tw-ring-color:hsla(0,96%,89%,.95)}.ring-red-400{--tw-ring-opacity:1;--tw-ring-color:rgb(248 113 113/var(--tw-ring-opacity,1))}.ring-red-400\/0{--tw-ring-color:hsla(0,91%,71%,0)}.ring-red-400\/10{--tw-ring-color:hsla(0,91%,71%,.1)}.ring-red-400\/100{--tw-ring-color:#f87171}.ring-red-400\/15{--tw-ring-color:hsla(0,91%,71%,.15)}.ring-red-400\/20{--tw-ring-color:hsla(0,91%,71%,.2)}.ring-red-400\/25{--tw-ring-color:hsla(0,91%,71%,.25)}.ring-red-400\/30{--tw-ring-color:hsla(0,91%,71%,.3)}.ring-red-400\/35{--tw-ring-color:hsla(0,91%,71%,.35)}.ring-red-400\/40{--tw-ring-color:hsla(0,91%,71%,.4)}.ring-red-400\/45{--tw-ring-color:hsla(0,91%,71%,.45)}.ring-red-400\/5{--tw-ring-color:hsla(0,91%,71%,.05)}.ring-red-400\/50{--tw-ring-color:hsla(0,91%,71%,.5)}.ring-red-400\/55{--tw-ring-color:hsla(0,91%,71%,.55)}.ring-red-400\/60{--tw-ring-color:hsla(0,91%,71%,.6)}.ring-red-400\/65{--tw-ring-color:hsla(0,91%,71%,.65)}.ring-red-400\/70{--tw-ring-color:hsla(0,91%,71%,.7)}.ring-red-400\/75{--tw-ring-color:hsla(0,91%,71%,.75)}.ring-red-400\/80{--tw-ring-color:hsla(0,91%,71%,.8)}.ring-red-400\/85{--tw-ring-color:hsla(0,91%,71%,.85)}.ring-red-400\/90{--tw-ring-color:hsla(0,91%,71%,.9)}.ring-red-400\/95{--tw-ring-color:hsla(0,91%,71%,.95)}.ring-red-50{--tw-ring-opacity:1;--tw-ring-color:rgb(254 242 242/var(--tw-ring-opacity,1))}.ring-red-50\/0{--tw-ring-color:hsla(0,86%,97%,0)}.ring-red-50\/10{--tw-ring-color:hsla(0,86%,97%,.1)}.ring-red-50\/100{--tw-ring-color:#fef2f2}.ring-red-50\/15{--tw-ring-color:hsla(0,86%,97%,.15)}.ring-red-50\/20{--tw-ring-color:hsla(0,86%,97%,.2)}.ring-red-50\/25{--tw-ring-color:hsla(0,86%,97%,.25)}.ring-red-50\/30{--tw-ring-color:hsla(0,86%,97%,.3)}.ring-red-50\/35{--tw-ring-color:hsla(0,86%,97%,.35)}.ring-red-50\/40{--tw-ring-color:hsla(0,86%,97%,.4)}.ring-red-50\/45{--tw-ring-color:hsla(0,86%,97%,.45)}.ring-red-50\/5{--tw-ring-color:hsla(0,86%,97%,.05)}.ring-red-50\/50{--tw-ring-color:hsla(0,86%,97%,.5)}.ring-red-50\/55{--tw-ring-color:hsla(0,86%,97%,.55)}.ring-red-50\/60{--tw-ring-color:hsla(0,86%,97%,.6)}.ring-red-50\/65{--tw-ring-color:hsla(0,86%,97%,.65)}.ring-red-50\/70{--tw-ring-color:hsla(0,86%,97%,.7)}.ring-red-50\/75{--tw-ring-color:hsla(0,86%,97%,.75)}.ring-red-50\/80{--tw-ring-color:hsla(0,86%,97%,.8)}.ring-red-50\/85{--tw-ring-color:hsla(0,86%,97%,.85)}.ring-red-50\/90{--tw-ring-color:hsla(0,86%,97%,.9)}.ring-red-50\/95{--tw-ring-color:hsla(0,86%,97%,.95)}.ring-red-500{--tw-ring-opacity:1;--tw-ring-color:rgb(239 68 68/var(--tw-ring-opacity,1))}.ring-red-500\/0{--tw-ring-color:rgba(239,68,68,0)}.ring-red-500\/10{--tw-ring-color:rgba(239,68,68,.1)}.ring-red-500\/100{--tw-ring-color:#ef4444}.ring-red-500\/15{--tw-ring-color:rgba(239,68,68,.15)}.ring-red-500\/20{--tw-ring-color:rgba(239,68,68,.2)}.ring-red-500\/25{--tw-ring-color:rgba(239,68,68,.25)}.ring-red-500\/30{--tw-ring-color:rgba(239,68,68,.3)}.ring-red-500\/35{--tw-ring-color:rgba(239,68,68,.35)}.ring-red-500\/40{--tw-ring-color:rgba(239,68,68,.4)}.ring-red-500\/45{--tw-ring-color:rgba(239,68,68,.45)}.ring-red-500\/5{--tw-ring-color:rgba(239,68,68,.05)}.ring-red-500\/50{--tw-ring-color:rgba(239,68,68,.5)}.ring-red-500\/55{--tw-ring-color:rgba(239,68,68,.55)}.ring-red-500\/60{--tw-ring-color:rgba(239,68,68,.6)}.ring-red-500\/65{--tw-ring-color:rgba(239,68,68,.65)}.ring-red-500\/70{--tw-ring-color:rgba(239,68,68,.7)}.ring-red-500\/75{--tw-ring-color:rgba(239,68,68,.75)}.ring-red-500\/80{--tw-ring-color:rgba(239,68,68,.8)}.ring-red-500\/85{--tw-ring-color:rgba(239,68,68,.85)}.ring-red-500\/90{--tw-ring-color:rgba(239,68,68,.9)}.ring-red-500\/95{--tw-ring-color:rgba(239,68,68,.95)}.ring-red-950{--tw-ring-opacity:1;--tw-ring-color:rgb(69 10 10/var(--tw-ring-opacity,1))}.ring-red-950\/0{--tw-ring-color:rgba(69,10,10,0)}.ring-red-950\/10{--tw-ring-color:rgba(69,10,10,.1)}.ring-red-950\/100{--tw-ring-color:#450a0a}.ring-red-950\/15{--tw-ring-color:rgba(69,10,10,.15)}.ring-red-950\/20{--tw-ring-color:rgba(69,10,10,.2)}.ring-red-950\/25{--tw-ring-color:rgba(69,10,10,.25)}.ring-red-950\/30{--tw-ring-color:rgba(69,10,10,.3)}.ring-red-950\/35{--tw-ring-color:rgba(69,10,10,.35)}.ring-red-950\/40{--tw-ring-color:rgba(69,10,10,.4)}.ring-red-950\/45{--tw-ring-color:rgba(69,10,10,.45)}.ring-red-950\/5{--tw-ring-color:rgba(69,10,10,.05)}.ring-red-950\/50{--tw-ring-color:rgba(69,10,10,.5)}.ring-red-950\/55{--tw-ring-color:rgba(69,10,10,.55)}.ring-red-950\/60{--tw-ring-color:rgba(69,10,10,.6)}.ring-red-950\/65{--tw-ring-color:rgba(69,10,10,.65)}.ring-red-950\/70{--tw-ring-color:rgba(69,10,10,.7)}.ring-red-950\/75{--tw-ring-color:rgba(69,10,10,.75)}.ring-red-950\/80{--tw-ring-color:rgba(69,10,10,.8)}.ring-red-950\/85{--tw-ring-color:rgba(69,10,10,.85)}.ring-red-950\/90{--tw-ring-color:rgba(69,10,10,.9)}.ring-red-950\/95{--tw-ring-color:rgba(69,10,10,.95)}.ring-ring{--tw-ring-color:hsl(var(--ring))}.ring-rose-100{--tw-ring-opacity:1;--tw-ring-color:rgb(255 228 230/var(--tw-ring-opacity,1))}.ring-rose-100\/0{--tw-ring-color:rgba(255,228,230,0)}.ring-rose-100\/10{--tw-ring-color:rgba(255,228,230,.1)}.ring-rose-100\/100{--tw-ring-color:#ffe4e6}.ring-rose-100\/15{--tw-ring-color:rgba(255,228,230,.15)}.ring-rose-100\/20{--tw-ring-color:rgba(255,228,230,.2)}.ring-rose-100\/25{--tw-ring-color:rgba(255,228,230,.25)}.ring-rose-100\/30{--tw-ring-color:rgba(255,228,230,.3)}.ring-rose-100\/35{--tw-ring-color:rgba(255,228,230,.35)}.ring-rose-100\/40{--tw-ring-color:rgba(255,228,230,.4)}.ring-rose-100\/45{--tw-ring-color:rgba(255,228,230,.45)}.ring-rose-100\/5{--tw-ring-color:rgba(255,228,230,.05)}.ring-rose-100\/50{--tw-ring-color:rgba(255,228,230,.5)}.ring-rose-100\/55{--tw-ring-color:rgba(255,228,230,.55)}.ring-rose-100\/60{--tw-ring-color:rgba(255,228,230,.6)}.ring-rose-100\/65{--tw-ring-color:rgba(255,228,230,.65)}.ring-rose-100\/70{--tw-ring-color:rgba(255,228,230,.7)}.ring-rose-100\/75{--tw-ring-color:rgba(255,228,230,.75)}.ring-rose-100\/80{--tw-ring-color:rgba(255,228,230,.8)}.ring-rose-100\/85{--tw-ring-color:rgba(255,228,230,.85)}.ring-rose-100\/90{--tw-ring-color:rgba(255,228,230,.9)}.ring-rose-100\/95{--tw-ring-color:rgba(255,228,230,.95)}.ring-rose-200{--tw-ring-opacity:1;--tw-ring-color:rgb(254 205 211/var(--tw-ring-opacity,1))}.ring-rose-200\/0{--tw-ring-color:rgba(254,205,211,0)}.ring-rose-200\/10{--tw-ring-color:rgba(254,205,211,.1)}.ring-rose-200\/100{--tw-ring-color:#fecdd3}.ring-rose-200\/15{--tw-ring-color:rgba(254,205,211,.15)}.ring-rose-200\/20{--tw-ring-color:rgba(254,205,211,.2)}.ring-rose-200\/25{--tw-ring-color:rgba(254,205,211,.25)}.ring-rose-200\/30{--tw-ring-color:rgba(254,205,211,.3)}.ring-rose-200\/35{--tw-ring-color:rgba(254,205,211,.35)}.ring-rose-200\/40{--tw-ring-color:rgba(254,205,211,.4)}.ring-rose-200\/45{--tw-ring-color:rgba(254,205,211,.45)}.ring-rose-200\/5{--tw-ring-color:rgba(254,205,211,.05)}.ring-rose-200\/50{--tw-ring-color:rgba(254,205,211,.5)}.ring-rose-200\/55{--tw-ring-color:rgba(254,205,211,.55)}.ring-rose-200\/60{--tw-ring-color:rgba(254,205,211,.6)}.ring-rose-200\/65{--tw-ring-color:rgba(254,205,211,.65)}.ring-rose-200\/70{--tw-ring-color:rgba(254,205,211,.7)}.ring-rose-200\/75{--tw-ring-color:rgba(254,205,211,.75)}.ring-rose-200\/80{--tw-ring-color:rgba(254,205,211,.8)}.ring-rose-200\/85{--tw-ring-color:rgba(254,205,211,.85)}.ring-rose-200\/90{--tw-ring-color:rgba(254,205,211,.9)}.ring-rose-200\/95{--tw-ring-color:rgba(254,205,211,.95)}.ring-rose-400{--tw-ring-opacity:1;--tw-ring-color:rgb(251 113 133/var(--tw-ring-opacity,1))}.ring-rose-400\/0{--tw-ring-color:rgba(251,113,133,0)}.ring-rose-400\/10{--tw-ring-color:rgba(251,113,133,.1)}.ring-rose-400\/100{--tw-ring-color:#fb7185}.ring-rose-400\/15{--tw-ring-color:rgba(251,113,133,.15)}.ring-rose-400\/20{--tw-ring-color:rgba(251,113,133,.2)}.ring-rose-400\/25{--tw-ring-color:rgba(251,113,133,.25)}.ring-rose-400\/30{--tw-ring-color:rgba(251,113,133,.3)}.ring-rose-400\/35{--tw-ring-color:rgba(251,113,133,.35)}.ring-rose-400\/40{--tw-ring-color:rgba(251,113,133,.4)}.ring-rose-400\/45{--tw-ring-color:rgba(251,113,133,.45)}.ring-rose-400\/5{--tw-ring-color:rgba(251,113,133,.05)}.ring-rose-400\/50{--tw-ring-color:rgba(251,113,133,.5)}.ring-rose-400\/55{--tw-ring-color:rgba(251,113,133,.55)}.ring-rose-400\/60{--tw-ring-color:rgba(251,113,133,.6)}.ring-rose-400\/65{--tw-ring-color:rgba(251,113,133,.65)}.ring-rose-400\/70{--tw-ring-color:rgba(251,113,133,.7)}.ring-rose-400\/75{--tw-ring-color:rgba(251,113,133,.75)}.ring-rose-400\/80{--tw-ring-color:rgba(251,113,133,.8)}.ring-rose-400\/85{--tw-ring-color:rgba(251,113,133,.85)}.ring-rose-400\/90{--tw-ring-color:rgba(251,113,133,.9)}.ring-rose-400\/95{--tw-ring-color:rgba(251,113,133,.95)}.ring-rose-50{--tw-ring-opacity:1;--tw-ring-color:rgb(255 241 242/var(--tw-ring-opacity,1))}.ring-rose-50\/0{--tw-ring-color:rgba(255,241,242,0)}.ring-rose-50\/10{--tw-ring-color:rgba(255,241,242,.1)}.ring-rose-50\/100{--tw-ring-color:#fff1f2}.ring-rose-50\/15{--tw-ring-color:rgba(255,241,242,.15)}.ring-rose-50\/20{--tw-ring-color:rgba(255,241,242,.2)}.ring-rose-50\/25{--tw-ring-color:rgba(255,241,242,.25)}.ring-rose-50\/30{--tw-ring-color:rgba(255,241,242,.3)}.ring-rose-50\/35{--tw-ring-color:rgba(255,241,242,.35)}.ring-rose-50\/40{--tw-ring-color:rgba(255,241,242,.4)}.ring-rose-50\/45{--tw-ring-color:rgba(255,241,242,.45)}.ring-rose-50\/5{--tw-ring-color:rgba(255,241,242,.05)}.ring-rose-50\/50{--tw-ring-color:rgba(255,241,242,.5)}.ring-rose-50\/55{--tw-ring-color:rgba(255,241,242,.55)}.ring-rose-50\/60{--tw-ring-color:rgba(255,241,242,.6)}.ring-rose-50\/65{--tw-ring-color:rgba(255,241,242,.65)}.ring-rose-50\/70{--tw-ring-color:rgba(255,241,242,.7)}.ring-rose-50\/75{--tw-ring-color:rgba(255,241,242,.75)}.ring-rose-50\/80{--tw-ring-color:rgba(255,241,242,.8)}.ring-rose-50\/85{--tw-ring-color:rgba(255,241,242,.85)}.ring-rose-50\/90{--tw-ring-color:rgba(255,241,242,.9)}.ring-rose-50\/95{--tw-ring-color:rgba(255,241,242,.95)}.ring-rose-500{--tw-ring-opacity:1;--tw-ring-color:rgb(244 63 94/var(--tw-ring-opacity,1))}.ring-rose-500\/0{--tw-ring-color:rgba(244,63,94,0)}.ring-rose-500\/10{--tw-ring-color:rgba(244,63,94,.1)}.ring-rose-500\/100{--tw-ring-color:#f43f5e}.ring-rose-500\/15{--tw-ring-color:rgba(244,63,94,.15)}.ring-rose-500\/20{--tw-ring-color:rgba(244,63,94,.2)}.ring-rose-500\/25{--tw-ring-color:rgba(244,63,94,.25)}.ring-rose-500\/30{--tw-ring-color:rgba(244,63,94,.3)}.ring-rose-500\/35{--tw-ring-color:rgba(244,63,94,.35)}.ring-rose-500\/40{--tw-ring-color:rgba(244,63,94,.4)}.ring-rose-500\/45{--tw-ring-color:rgba(244,63,94,.45)}.ring-rose-500\/5{--tw-ring-color:rgba(244,63,94,.05)}.ring-rose-500\/50{--tw-ring-color:rgba(244,63,94,.5)}.ring-rose-500\/55{--tw-ring-color:rgba(244,63,94,.55)}.ring-rose-500\/60{--tw-ring-color:rgba(244,63,94,.6)}.ring-rose-500\/65{--tw-ring-color:rgba(244,63,94,.65)}.ring-rose-500\/70{--tw-ring-color:rgba(244,63,94,.7)}.ring-rose-500\/75{--tw-ring-color:rgba(244,63,94,.75)}.ring-rose-500\/80{--tw-ring-color:rgba(244,63,94,.8)}.ring-rose-500\/85{--tw-ring-color:rgba(244,63,94,.85)}.ring-rose-500\/90{--tw-ring-color:rgba(244,63,94,.9)}.ring-rose-500\/95{--tw-ring-color:rgba(244,63,94,.95)}.ring-rose-950{--tw-ring-opacity:1;--tw-ring-color:rgb(76 5 25/var(--tw-ring-opacity,1))}.ring-rose-950\/0{--tw-ring-color:rgba(76,5,25,0)}.ring-rose-950\/10{--tw-ring-color:rgba(76,5,25,.1)}.ring-rose-950\/100{--tw-ring-color:#4c0519}.ring-rose-950\/15{--tw-ring-color:rgba(76,5,25,.15)}.ring-rose-950\/20{--tw-ring-color:rgba(76,5,25,.2)}.ring-rose-950\/25{--tw-ring-color:rgba(76,5,25,.25)}.ring-rose-950\/30{--tw-ring-color:rgba(76,5,25,.3)}.ring-rose-950\/35{--tw-ring-color:rgba(76,5,25,.35)}.ring-rose-950\/40{--tw-ring-color:rgba(76,5,25,.4)}.ring-rose-950\/45{--tw-ring-color:rgba(76,5,25,.45)}.ring-rose-950\/5{--tw-ring-color:rgba(76,5,25,.05)}.ring-rose-950\/50{--tw-ring-color:rgba(76,5,25,.5)}.ring-rose-950\/55{--tw-ring-color:rgba(76,5,25,.55)}.ring-rose-950\/60{--tw-ring-color:rgba(76,5,25,.6)}.ring-rose-950\/65{--tw-ring-color:rgba(76,5,25,.65)}.ring-rose-950\/70{--tw-ring-color:rgba(76,5,25,.7)}.ring-rose-950\/75{--tw-ring-color:rgba(76,5,25,.75)}.ring-rose-950\/80{--tw-ring-color:rgba(76,5,25,.8)}.ring-rose-950\/85{--tw-ring-color:rgba(76,5,25,.85)}.ring-rose-950\/90{--tw-ring-color:rgba(76,5,25,.9)}.ring-rose-950\/95{--tw-ring-color:rgba(76,5,25,.95)}.ring-sky-100{--tw-ring-opacity:1;--tw-ring-color:rgb(224 242 254/var(--tw-ring-opacity,1))}.ring-sky-100\/0{--tw-ring-color:rgba(224,242,254,0)}.ring-sky-100\/10{--tw-ring-color:rgba(224,242,254,.1)}.ring-sky-100\/100{--tw-ring-color:#e0f2fe}.ring-sky-100\/15{--tw-ring-color:rgba(224,242,254,.15)}.ring-sky-100\/20{--tw-ring-color:rgba(224,242,254,.2)}.ring-sky-100\/25{--tw-ring-color:rgba(224,242,254,.25)}.ring-sky-100\/30{--tw-ring-color:rgba(224,242,254,.3)}.ring-sky-100\/35{--tw-ring-color:rgba(224,242,254,.35)}.ring-sky-100\/40{--tw-ring-color:rgba(224,242,254,.4)}.ring-sky-100\/45{--tw-ring-color:rgba(224,242,254,.45)}.ring-sky-100\/5{--tw-ring-color:rgba(224,242,254,.05)}.ring-sky-100\/50{--tw-ring-color:rgba(224,242,254,.5)}.ring-sky-100\/55{--tw-ring-color:rgba(224,242,254,.55)}.ring-sky-100\/60{--tw-ring-color:rgba(224,242,254,.6)}.ring-sky-100\/65{--tw-ring-color:rgba(224,242,254,.65)}.ring-sky-100\/70{--tw-ring-color:rgba(224,242,254,.7)}.ring-sky-100\/75{--tw-ring-color:rgba(224,242,254,.75)}.ring-sky-100\/80{--tw-ring-color:rgba(224,242,254,.8)}.ring-sky-100\/85{--tw-ring-color:rgba(224,242,254,.85)}.ring-sky-100\/90{--tw-ring-color:rgba(224,242,254,.9)}.ring-sky-100\/95{--tw-ring-color:rgba(224,242,254,.95)}.ring-sky-200{--tw-ring-opacity:1;--tw-ring-color:rgb(186 230 253/var(--tw-ring-opacity,1))}.ring-sky-200\/0{--tw-ring-color:rgba(186,230,253,0)}.ring-sky-200\/10{--tw-ring-color:rgba(186,230,253,.1)}.ring-sky-200\/100{--tw-ring-color:#bae6fd}.ring-sky-200\/15{--tw-ring-color:rgba(186,230,253,.15)}.ring-sky-200\/20{--tw-ring-color:rgba(186,230,253,.2)}.ring-sky-200\/25{--tw-ring-color:rgba(186,230,253,.25)}.ring-sky-200\/30{--tw-ring-color:rgba(186,230,253,.3)}.ring-sky-200\/35{--tw-ring-color:rgba(186,230,253,.35)}.ring-sky-200\/40{--tw-ring-color:rgba(186,230,253,.4)}.ring-sky-200\/45{--tw-ring-color:rgba(186,230,253,.45)}.ring-sky-200\/5{--tw-ring-color:rgba(186,230,253,.05)}.ring-sky-200\/50{--tw-ring-color:rgba(186,230,253,.5)}.ring-sky-200\/55{--tw-ring-color:rgba(186,230,253,.55)}.ring-sky-200\/60{--tw-ring-color:rgba(186,230,253,.6)}.ring-sky-200\/65{--tw-ring-color:rgba(186,230,253,.65)}.ring-sky-200\/70{--tw-ring-color:rgba(186,230,253,.7)}.ring-sky-200\/75{--tw-ring-color:rgba(186,230,253,.75)}.ring-sky-200\/80{--tw-ring-color:rgba(186,230,253,.8)}.ring-sky-200\/85{--tw-ring-color:rgba(186,230,253,.85)}.ring-sky-200\/90{--tw-ring-color:rgba(186,230,253,.9)}.ring-sky-200\/95{--tw-ring-color:rgba(186,230,253,.95)}.ring-sky-400{--tw-ring-opacity:1;--tw-ring-color:rgb(56 189 248/var(--tw-ring-opacity,1))}.ring-sky-400\/0{--tw-ring-color:rgba(56,189,248,0)}.ring-sky-400\/10{--tw-ring-color:rgba(56,189,248,.1)}.ring-sky-400\/100{--tw-ring-color:#38bdf8}.ring-sky-400\/15{--tw-ring-color:rgba(56,189,248,.15)}.ring-sky-400\/20{--tw-ring-color:rgba(56,189,248,.2)}.ring-sky-400\/25{--tw-ring-color:rgba(56,189,248,.25)}.ring-sky-400\/30{--tw-ring-color:rgba(56,189,248,.3)}.ring-sky-400\/35{--tw-ring-color:rgba(56,189,248,.35)}.ring-sky-400\/40{--tw-ring-color:rgba(56,189,248,.4)}.ring-sky-400\/45{--tw-ring-color:rgba(56,189,248,.45)}.ring-sky-400\/5{--tw-ring-color:rgba(56,189,248,.05)}.ring-sky-400\/50{--tw-ring-color:rgba(56,189,248,.5)}.ring-sky-400\/55{--tw-ring-color:rgba(56,189,248,.55)}.ring-sky-400\/60{--tw-ring-color:rgba(56,189,248,.6)}.ring-sky-400\/65{--tw-ring-color:rgba(56,189,248,.65)}.ring-sky-400\/70{--tw-ring-color:rgba(56,189,248,.7)}.ring-sky-400\/75{--tw-ring-color:rgba(56,189,248,.75)}.ring-sky-400\/80{--tw-ring-color:rgba(56,189,248,.8)}.ring-sky-400\/85{--tw-ring-color:rgba(56,189,248,.85)}.ring-sky-400\/90{--tw-ring-color:rgba(56,189,248,.9)}.ring-sky-400\/95{--tw-ring-color:rgba(56,189,248,.95)}.ring-sky-50{--tw-ring-opacity:1;--tw-ring-color:rgb(240 249 255/var(--tw-ring-opacity,1))}.ring-sky-50\/0{--tw-ring-color:rgba(240,249,255,0)}.ring-sky-50\/10{--tw-ring-color:rgba(240,249,255,.1)}.ring-sky-50\/100{--tw-ring-color:#f0f9ff}.ring-sky-50\/15{--tw-ring-color:rgba(240,249,255,.15)}.ring-sky-50\/20{--tw-ring-color:rgba(240,249,255,.2)}.ring-sky-50\/25{--tw-ring-color:rgba(240,249,255,.25)}.ring-sky-50\/30{--tw-ring-color:rgba(240,249,255,.3)}.ring-sky-50\/35{--tw-ring-color:rgba(240,249,255,.35)}.ring-sky-50\/40{--tw-ring-color:rgba(240,249,255,.4)}.ring-sky-50\/45{--tw-ring-color:rgba(240,249,255,.45)}.ring-sky-50\/5{--tw-ring-color:rgba(240,249,255,.05)}.ring-sky-50\/50{--tw-ring-color:rgba(240,249,255,.5)}.ring-sky-50\/55{--tw-ring-color:rgba(240,249,255,.55)}.ring-sky-50\/60{--tw-ring-color:rgba(240,249,255,.6)}.ring-sky-50\/65{--tw-ring-color:rgba(240,249,255,.65)}.ring-sky-50\/70{--tw-ring-color:rgba(240,249,255,.7)}.ring-sky-50\/75{--tw-ring-color:rgba(240,249,255,.75)}.ring-sky-50\/80{--tw-ring-color:rgba(240,249,255,.8)}.ring-sky-50\/85{--tw-ring-color:rgba(240,249,255,.85)}.ring-sky-50\/90{--tw-ring-color:rgba(240,249,255,.9)}.ring-sky-50\/95{--tw-ring-color:rgba(240,249,255,.95)}.ring-sky-500{--tw-ring-opacity:1;--tw-ring-color:rgb(14 165 233/var(--tw-ring-opacity,1))}.ring-sky-500\/0{--tw-ring-color:rgba(14,165,233,0)}.ring-sky-500\/10{--tw-ring-color:rgba(14,165,233,.1)}.ring-sky-500\/100{--tw-ring-color:#0ea5e9}.ring-sky-500\/15{--tw-ring-color:rgba(14,165,233,.15)}.ring-sky-500\/20{--tw-ring-color:rgba(14,165,233,.2)}.ring-sky-500\/25{--tw-ring-color:rgba(14,165,233,.25)}.ring-sky-500\/30{--tw-ring-color:rgba(14,165,233,.3)}.ring-sky-500\/35{--tw-ring-color:rgba(14,165,233,.35)}.ring-sky-500\/40{--tw-ring-color:rgba(14,165,233,.4)}.ring-sky-500\/45{--tw-ring-color:rgba(14,165,233,.45)}.ring-sky-500\/5{--tw-ring-color:rgba(14,165,233,.05)}.ring-sky-500\/50{--tw-ring-color:rgba(14,165,233,.5)}.ring-sky-500\/55{--tw-ring-color:rgba(14,165,233,.55)}.ring-sky-500\/60{--tw-ring-color:rgba(14,165,233,.6)}.ring-sky-500\/65{--tw-ring-color:rgba(14,165,233,.65)}.ring-sky-500\/70{--tw-ring-color:rgba(14,165,233,.7)}.ring-sky-500\/75{--tw-ring-color:rgba(14,165,233,.75)}.ring-sky-500\/80{--tw-ring-color:rgba(14,165,233,.8)}.ring-sky-500\/85{--tw-ring-color:rgba(14,165,233,.85)}.ring-sky-500\/90{--tw-ring-color:rgba(14,165,233,.9)}.ring-sky-500\/95{--tw-ring-color:rgba(14,165,233,.95)}.ring-sky-950{--tw-ring-opacity:1;--tw-ring-color:rgb(8 47 73/var(--tw-ring-opacity,1))}.ring-sky-950\/0{--tw-ring-color:rgba(8,47,73,0)}.ring-sky-950\/10{--tw-ring-color:rgba(8,47,73,.1)}.ring-sky-950\/100{--tw-ring-color:#082f49}.ring-sky-950\/15{--tw-ring-color:rgba(8,47,73,.15)}.ring-sky-950\/20{--tw-ring-color:rgba(8,47,73,.2)}.ring-sky-950\/25{--tw-ring-color:rgba(8,47,73,.25)}.ring-sky-950\/30{--tw-ring-color:rgba(8,47,73,.3)}.ring-sky-950\/35{--tw-ring-color:rgba(8,47,73,.35)}.ring-sky-950\/40{--tw-ring-color:rgba(8,47,73,.4)}.ring-sky-950\/45{--tw-ring-color:rgba(8,47,73,.45)}.ring-sky-950\/5{--tw-ring-color:rgba(8,47,73,.05)}.ring-sky-950\/50{--tw-ring-color:rgba(8,47,73,.5)}.ring-sky-950\/55{--tw-ring-color:rgba(8,47,73,.55)}.ring-sky-950\/60{--tw-ring-color:rgba(8,47,73,.6)}.ring-sky-950\/65{--tw-ring-color:rgba(8,47,73,.65)}.ring-sky-950\/70{--tw-ring-color:rgba(8,47,73,.7)}.ring-sky-950\/75{--tw-ring-color:rgba(8,47,73,.75)}.ring-sky-950\/80{--tw-ring-color:rgba(8,47,73,.8)}.ring-sky-950\/85{--tw-ring-color:rgba(8,47,73,.85)}.ring-sky-950\/90{--tw-ring-color:rgba(8,47,73,.9)}.ring-sky-950\/95{--tw-ring-color:rgba(8,47,73,.95)}.ring-slate-100{--tw-ring-opacity:1;--tw-ring-color:rgb(241 245 249/var(--tw-ring-opacity,1))}.ring-slate-100\/0{--tw-ring-color:rgba(241,245,249,0)}.ring-slate-100\/10{--tw-ring-color:rgba(241,245,249,.1)}.ring-slate-100\/100{--tw-ring-color:#f1f5f9}.ring-slate-100\/15{--tw-ring-color:rgba(241,245,249,.15)}.ring-slate-100\/20{--tw-ring-color:rgba(241,245,249,.2)}.ring-slate-100\/25{--tw-ring-color:rgba(241,245,249,.25)}.ring-slate-100\/30{--tw-ring-color:rgba(241,245,249,.3)}.ring-slate-100\/35{--tw-ring-color:rgba(241,245,249,.35)}.ring-slate-100\/40{--tw-ring-color:rgba(241,245,249,.4)}.ring-slate-100\/45{--tw-ring-color:rgba(241,245,249,.45)}.ring-slate-100\/5{--tw-ring-color:rgba(241,245,249,.05)}.ring-slate-100\/50{--tw-ring-color:rgba(241,245,249,.5)}.ring-slate-100\/55{--tw-ring-color:rgba(241,245,249,.55)}.ring-slate-100\/60{--tw-ring-color:rgba(241,245,249,.6)}.ring-slate-100\/65{--tw-ring-color:rgba(241,245,249,.65)}.ring-slate-100\/70{--tw-ring-color:rgba(241,245,249,.7)}.ring-slate-100\/75{--tw-ring-color:rgba(241,245,249,.75)}.ring-slate-100\/80{--tw-ring-color:rgba(241,245,249,.8)}.ring-slate-100\/85{--tw-ring-color:rgba(241,245,249,.85)}.ring-slate-100\/90{--tw-ring-color:rgba(241,245,249,.9)}.ring-slate-100\/95{--tw-ring-color:rgba(241,245,249,.95)}.ring-slate-200{--tw-ring-opacity:1;--tw-ring-color:rgb(226 232 240/var(--tw-ring-opacity,1))}.ring-slate-200\/0{--tw-ring-color:rgba(226,232,240,0)}.ring-slate-200\/10{--tw-ring-color:rgba(226,232,240,.1)}.ring-slate-200\/100{--tw-ring-color:#e2e8f0}.ring-slate-200\/15{--tw-ring-color:rgba(226,232,240,.15)}.ring-slate-200\/20{--tw-ring-color:rgba(226,232,240,.2)}.ring-slate-200\/25{--tw-ring-color:rgba(226,232,240,.25)}.ring-slate-200\/30{--tw-ring-color:rgba(226,232,240,.3)}.ring-slate-200\/35{--tw-ring-color:rgba(226,232,240,.35)}.ring-slate-200\/40{--tw-ring-color:rgba(226,232,240,.4)}.ring-slate-200\/45{--tw-ring-color:rgba(226,232,240,.45)}.ring-slate-200\/5{--tw-ring-color:rgba(226,232,240,.05)}.ring-slate-200\/50{--tw-ring-color:rgba(226,232,240,.5)}.ring-slate-200\/55{--tw-ring-color:rgba(226,232,240,.55)}.ring-slate-200\/60{--tw-ring-color:rgba(226,232,240,.6)}.ring-slate-200\/65{--tw-ring-color:rgba(226,232,240,.65)}.ring-slate-200\/70{--tw-ring-color:rgba(226,232,240,.7)}.ring-slate-200\/75{--tw-ring-color:rgba(226,232,240,.75)}.ring-slate-200\/80{--tw-ring-color:rgba(226,232,240,.8)}.ring-slate-200\/85{--tw-ring-color:rgba(226,232,240,.85)}.ring-slate-200\/90{--tw-ring-color:rgba(226,232,240,.9)}.ring-slate-200\/95{--tw-ring-color:rgba(226,232,240,.95)}.ring-slate-400{--tw-ring-opacity:1;--tw-ring-color:rgb(148 163 184/var(--tw-ring-opacity,1))}.ring-slate-400\/0{--tw-ring-color:rgba(148,163,184,0)}.ring-slate-400\/10{--tw-ring-color:rgba(148,163,184,.1)}.ring-slate-400\/100{--tw-ring-color:#94a3b8}.ring-slate-400\/15{--tw-ring-color:rgba(148,163,184,.15)}.ring-slate-400\/20{--tw-ring-color:rgba(148,163,184,.2)}.ring-slate-400\/25{--tw-ring-color:rgba(148,163,184,.25)}.ring-slate-400\/30{--tw-ring-color:rgba(148,163,184,.3)}.ring-slate-400\/35{--tw-ring-color:rgba(148,163,184,.35)}.ring-slate-400\/40{--tw-ring-color:rgba(148,163,184,.4)}.ring-slate-400\/45{--tw-ring-color:rgba(148,163,184,.45)}.ring-slate-400\/5{--tw-ring-color:rgba(148,163,184,.05)}.ring-slate-400\/50{--tw-ring-color:rgba(148,163,184,.5)}.ring-slate-400\/55{--tw-ring-color:rgba(148,163,184,.55)}.ring-slate-400\/60{--tw-ring-color:rgba(148,163,184,.6)}.ring-slate-400\/65{--tw-ring-color:rgba(148,163,184,.65)}.ring-slate-400\/70{--tw-ring-color:rgba(148,163,184,.7)}.ring-slate-400\/75{--tw-ring-color:rgba(148,163,184,.75)}.ring-slate-400\/80{--tw-ring-color:rgba(148,163,184,.8)}.ring-slate-400\/85{--tw-ring-color:rgba(148,163,184,.85)}.ring-slate-400\/90{--tw-ring-color:rgba(148,163,184,.9)}.ring-slate-400\/95{--tw-ring-color:rgba(148,163,184,.95)}.ring-slate-50{--tw-ring-opacity:1;--tw-ring-color:rgb(248 250 252/var(--tw-ring-opacity,1))}.ring-slate-50\/0{--tw-ring-color:rgba(248,250,252,0)}.ring-slate-50\/10{--tw-ring-color:rgba(248,250,252,.1)}.ring-slate-50\/100{--tw-ring-color:#f8fafc}.ring-slate-50\/15{--tw-ring-color:rgba(248,250,252,.15)}.ring-slate-50\/20{--tw-ring-color:rgba(248,250,252,.2)}.ring-slate-50\/25{--tw-ring-color:rgba(248,250,252,.25)}.ring-slate-50\/30{--tw-ring-color:rgba(248,250,252,.3)}.ring-slate-50\/35{--tw-ring-color:rgba(248,250,252,.35)}.ring-slate-50\/40{--tw-ring-color:rgba(248,250,252,.4)}.ring-slate-50\/45{--tw-ring-color:rgba(248,250,252,.45)}.ring-slate-50\/5{--tw-ring-color:rgba(248,250,252,.05)}.ring-slate-50\/50{--tw-ring-color:rgba(248,250,252,.5)}.ring-slate-50\/55{--tw-ring-color:rgba(248,250,252,.55)}.ring-slate-50\/60{--tw-ring-color:rgba(248,250,252,.6)}.ring-slate-50\/65{--tw-ring-color:rgba(248,250,252,.65)}.ring-slate-50\/70{--tw-ring-color:rgba(248,250,252,.7)}.ring-slate-50\/75{--tw-ring-color:rgba(248,250,252,.75)}.ring-slate-50\/80{--tw-ring-color:rgba(248,250,252,.8)}.ring-slate-50\/85{--tw-ring-color:rgba(248,250,252,.85)}.ring-slate-50\/90{--tw-ring-color:rgba(248,250,252,.9)}.ring-slate-50\/95{--tw-ring-color:rgba(248,250,252,.95)}.ring-slate-500{--tw-ring-opacity:1;--tw-ring-color:rgb(100 116 139/var(--tw-ring-opacity,1))}.ring-slate-500\/0{--tw-ring-color:rgba(100,116,139,0)}.ring-slate-500\/10{--tw-ring-color:rgba(100,116,139,.1)}.ring-slate-500\/100{--tw-ring-color:#64748b}.ring-slate-500\/15{--tw-ring-color:rgba(100,116,139,.15)}.ring-slate-500\/20{--tw-ring-color:rgba(100,116,139,.2)}.ring-slate-500\/25{--tw-ring-color:rgba(100,116,139,.25)}.ring-slate-500\/30{--tw-ring-color:rgba(100,116,139,.3)}.ring-slate-500\/35{--tw-ring-color:rgba(100,116,139,.35)}.ring-slate-500\/40{--tw-ring-color:rgba(100,116,139,.4)}.ring-slate-500\/45{--tw-ring-color:rgba(100,116,139,.45)}.ring-slate-500\/5{--tw-ring-color:rgba(100,116,139,.05)}.ring-slate-500\/50{--tw-ring-color:rgba(100,116,139,.5)}.ring-slate-500\/55{--tw-ring-color:rgba(100,116,139,.55)}.ring-slate-500\/60{--tw-ring-color:rgba(100,116,139,.6)}.ring-slate-500\/65{--tw-ring-color:rgba(100,116,139,.65)}.ring-slate-500\/70{--tw-ring-color:rgba(100,116,139,.7)}.ring-slate-500\/75{--tw-ring-color:rgba(100,116,139,.75)}.ring-slate-500\/80{--tw-ring-color:rgba(100,116,139,.8)}.ring-slate-500\/85{--tw-ring-color:rgba(100,116,139,.85)}.ring-slate-500\/90{--tw-ring-color:rgba(100,116,139,.9)}.ring-slate-500\/95{--tw-ring-color:rgba(100,116,139,.95)}.ring-slate-950{--tw-ring-opacity:1;--tw-ring-color:rgb(2 6 23/var(--tw-ring-opacity,1))}.ring-slate-950\/0{--tw-ring-color:rgba(2,6,23,0)}.ring-slate-950\/10{--tw-ring-color:rgba(2,6,23,.1)}.ring-slate-950\/100{--tw-ring-color:#020617}.ring-slate-950\/15{--tw-ring-color:rgba(2,6,23,.15)}.ring-slate-950\/20{--tw-ring-color:rgba(2,6,23,.2)}.ring-slate-950\/25{--tw-ring-color:rgba(2,6,23,.25)}.ring-slate-950\/30{--tw-ring-color:rgba(2,6,23,.3)}.ring-slate-950\/35{--tw-ring-color:rgba(2,6,23,.35)}.ring-slate-950\/40{--tw-ring-color:rgba(2,6,23,.4)}.ring-slate-950\/45{--tw-ring-color:rgba(2,6,23,.45)}.ring-slate-950\/5{--tw-ring-color:rgba(2,6,23,.05)}.ring-slate-950\/50{--tw-ring-color:rgba(2,6,23,.5)}.ring-slate-950\/55{--tw-ring-color:rgba(2,6,23,.55)}.ring-slate-950\/60{--tw-ring-color:rgba(2,6,23,.6)}.ring-slate-950\/65{--tw-ring-color:rgba(2,6,23,.65)}.ring-slate-950\/70{--tw-ring-color:rgba(2,6,23,.7)}.ring-slate-950\/75{--tw-ring-color:rgba(2,6,23,.75)}.ring-slate-950\/80{--tw-ring-color:rgba(2,6,23,.8)}.ring-slate-950\/85{--tw-ring-color:rgba(2,6,23,.85)}.ring-slate-950\/90{--tw-ring-color:rgba(2,6,23,.9)}.ring-slate-950\/95{--tw-ring-color:rgba(2,6,23,.95)}.ring-stone-100{--tw-ring-opacity:1;--tw-ring-color:rgb(245 245 244/var(--tw-ring-opacity,1))}.ring-stone-100\/0{--tw-ring-color:hsla(60,5%,96%,0)}.ring-stone-100\/10{--tw-ring-color:hsla(60,5%,96%,.1)}.ring-stone-100\/100{--tw-ring-color:#f5f5f4}.ring-stone-100\/15{--tw-ring-color:hsla(60,5%,96%,.15)}.ring-stone-100\/20{--tw-ring-color:hsla(60,5%,96%,.2)}.ring-stone-100\/25{--tw-ring-color:hsla(60,5%,96%,.25)}.ring-stone-100\/30{--tw-ring-color:hsla(60,5%,96%,.3)}.ring-stone-100\/35{--tw-ring-color:hsla(60,5%,96%,.35)}.ring-stone-100\/40{--tw-ring-color:hsla(60,5%,96%,.4)}.ring-stone-100\/45{--tw-ring-color:hsla(60,5%,96%,.45)}.ring-stone-100\/5{--tw-ring-color:hsla(60,5%,96%,.05)}.ring-stone-100\/50{--tw-ring-color:hsla(60,5%,96%,.5)}.ring-stone-100\/55{--tw-ring-color:hsla(60,5%,96%,.55)}.ring-stone-100\/60{--tw-ring-color:hsla(60,5%,96%,.6)}.ring-stone-100\/65{--tw-ring-color:hsla(60,5%,96%,.65)}.ring-stone-100\/70{--tw-ring-color:hsla(60,5%,96%,.7)}.ring-stone-100\/75{--tw-ring-color:hsla(60,5%,96%,.75)}.ring-stone-100\/80{--tw-ring-color:hsla(60,5%,96%,.8)}.ring-stone-100\/85{--tw-ring-color:hsla(60,5%,96%,.85)}.ring-stone-100\/90{--tw-ring-color:hsla(60,5%,96%,.9)}.ring-stone-100\/95{--tw-ring-color:hsla(60,5%,96%,.95)}.ring-stone-200{--tw-ring-opacity:1;--tw-ring-color:rgb(231 229 228/var(--tw-ring-opacity,1))}.ring-stone-200\/0{--tw-ring-color:hsla(20,6%,90%,0)}.ring-stone-200\/10{--tw-ring-color:hsla(20,6%,90%,.1)}.ring-stone-200\/100{--tw-ring-color:#e7e5e4}.ring-stone-200\/15{--tw-ring-color:hsla(20,6%,90%,.15)}.ring-stone-200\/20{--tw-ring-color:hsla(20,6%,90%,.2)}.ring-stone-200\/25{--tw-ring-color:hsla(20,6%,90%,.25)}.ring-stone-200\/30{--tw-ring-color:hsla(20,6%,90%,.3)}.ring-stone-200\/35{--tw-ring-color:hsla(20,6%,90%,.35)}.ring-stone-200\/40{--tw-ring-color:hsla(20,6%,90%,.4)}.ring-stone-200\/45{--tw-ring-color:hsla(20,6%,90%,.45)}.ring-stone-200\/5{--tw-ring-color:hsla(20,6%,90%,.05)}.ring-stone-200\/50{--tw-ring-color:hsla(20,6%,90%,.5)}.ring-stone-200\/55{--tw-ring-color:hsla(20,6%,90%,.55)}.ring-stone-200\/60{--tw-ring-color:hsla(20,6%,90%,.6)}.ring-stone-200\/65{--tw-ring-color:hsla(20,6%,90%,.65)}.ring-stone-200\/70{--tw-ring-color:hsla(20,6%,90%,.7)}.ring-stone-200\/75{--tw-ring-color:hsla(20,6%,90%,.75)}.ring-stone-200\/80{--tw-ring-color:hsla(20,6%,90%,.8)}.ring-stone-200\/85{--tw-ring-color:hsla(20,6%,90%,.85)}.ring-stone-200\/90{--tw-ring-color:hsla(20,6%,90%,.9)}.ring-stone-200\/95{--tw-ring-color:hsla(20,6%,90%,.95)}.ring-stone-400{--tw-ring-opacity:1;--tw-ring-color:rgb(168 162 158/var(--tw-ring-opacity,1))}.ring-stone-400\/0{--tw-ring-color:hsla(24,5%,64%,0)}.ring-stone-400\/10{--tw-ring-color:hsla(24,5%,64%,.1)}.ring-stone-400\/100{--tw-ring-color:#a8a29e}.ring-stone-400\/15{--tw-ring-color:hsla(24,5%,64%,.15)}.ring-stone-400\/20{--tw-ring-color:hsla(24,5%,64%,.2)}.ring-stone-400\/25{--tw-ring-color:hsla(24,5%,64%,.25)}.ring-stone-400\/30{--tw-ring-color:hsla(24,5%,64%,.3)}.ring-stone-400\/35{--tw-ring-color:hsla(24,5%,64%,.35)}.ring-stone-400\/40{--tw-ring-color:hsla(24,5%,64%,.4)}.ring-stone-400\/45{--tw-ring-color:hsla(24,5%,64%,.45)}.ring-stone-400\/5{--tw-ring-color:hsla(24,5%,64%,.05)}.ring-stone-400\/50{--tw-ring-color:hsla(24,5%,64%,.5)}.ring-stone-400\/55{--tw-ring-color:hsla(24,5%,64%,.55)}.ring-stone-400\/60{--tw-ring-color:hsla(24,5%,64%,.6)}.ring-stone-400\/65{--tw-ring-color:hsla(24,5%,64%,.65)}.ring-stone-400\/70{--tw-ring-color:hsla(24,5%,64%,.7)}.ring-stone-400\/75{--tw-ring-color:hsla(24,5%,64%,.75)}.ring-stone-400\/80{--tw-ring-color:hsla(24,5%,64%,.8)}.ring-stone-400\/85{--tw-ring-color:hsla(24,5%,64%,.85)}.ring-stone-400\/90{--tw-ring-color:hsla(24,5%,64%,.9)}.ring-stone-400\/95{--tw-ring-color:hsla(24,5%,64%,.95)}.ring-stone-50{--tw-ring-opacity:1;--tw-ring-color:rgb(250 250 249/var(--tw-ring-opacity,1))}.ring-stone-50\/0{--tw-ring-color:hsla(60,9%,98%,0)}.ring-stone-50\/10{--tw-ring-color:hsla(60,9%,98%,.1)}.ring-stone-50\/100{--tw-ring-color:#fafaf9}.ring-stone-50\/15{--tw-ring-color:hsla(60,9%,98%,.15)}.ring-stone-50\/20{--tw-ring-color:hsla(60,9%,98%,.2)}.ring-stone-50\/25{--tw-ring-color:hsla(60,9%,98%,.25)}.ring-stone-50\/30{--tw-ring-color:hsla(60,9%,98%,.3)}.ring-stone-50\/35{--tw-ring-color:hsla(60,9%,98%,.35)}.ring-stone-50\/40{--tw-ring-color:hsla(60,9%,98%,.4)}.ring-stone-50\/45{--tw-ring-color:hsla(60,9%,98%,.45)}.ring-stone-50\/5{--tw-ring-color:hsla(60,9%,98%,.05)}.ring-stone-50\/50{--tw-ring-color:hsla(60,9%,98%,.5)}.ring-stone-50\/55{--tw-ring-color:hsla(60,9%,98%,.55)}.ring-stone-50\/60{--tw-ring-color:hsla(60,9%,98%,.6)}.ring-stone-50\/65{--tw-ring-color:hsla(60,9%,98%,.65)}.ring-stone-50\/70{--tw-ring-color:hsla(60,9%,98%,.7)}.ring-stone-50\/75{--tw-ring-color:hsla(60,9%,98%,.75)}.ring-stone-50\/80{--tw-ring-color:hsla(60,9%,98%,.8)}.ring-stone-50\/85{--tw-ring-color:hsla(60,9%,98%,.85)}.ring-stone-50\/90{--tw-ring-color:hsla(60,9%,98%,.9)}.ring-stone-50\/95{--tw-ring-color:hsla(60,9%,98%,.95)}.ring-stone-500{--tw-ring-opacity:1;--tw-ring-color:rgb(120 113 108/var(--tw-ring-opacity,1))}.ring-stone-500\/0{--tw-ring-color:hsla(25,5%,45%,0)}.ring-stone-500\/10{--tw-ring-color:hsla(25,5%,45%,.1)}.ring-stone-500\/100{--tw-ring-color:#78716c}.ring-stone-500\/15{--tw-ring-color:hsla(25,5%,45%,.15)}.ring-stone-500\/20{--tw-ring-color:hsla(25,5%,45%,.2)}.ring-stone-500\/25{--tw-ring-color:hsla(25,5%,45%,.25)}.ring-stone-500\/30{--tw-ring-color:hsla(25,5%,45%,.3)}.ring-stone-500\/35{--tw-ring-color:hsla(25,5%,45%,.35)}.ring-stone-500\/40{--tw-ring-color:hsla(25,5%,45%,.4)}.ring-stone-500\/45{--tw-ring-color:hsla(25,5%,45%,.45)}.ring-stone-500\/5{--tw-ring-color:hsla(25,5%,45%,.05)}.ring-stone-500\/50{--tw-ring-color:hsla(25,5%,45%,.5)}.ring-stone-500\/55{--tw-ring-color:hsla(25,5%,45%,.55)}.ring-stone-500\/60{--tw-ring-color:hsla(25,5%,45%,.6)}.ring-stone-500\/65{--tw-ring-color:hsla(25,5%,45%,.65)}.ring-stone-500\/70{--tw-ring-color:hsla(25,5%,45%,.7)}.ring-stone-500\/75{--tw-ring-color:hsla(25,5%,45%,.75)}.ring-stone-500\/80{--tw-ring-color:hsla(25,5%,45%,.8)}.ring-stone-500\/85{--tw-ring-color:hsla(25,5%,45%,.85)}.ring-stone-500\/90{--tw-ring-color:hsla(25,5%,45%,.9)}.ring-stone-500\/95{--tw-ring-color:hsla(25,5%,45%,.95)}.ring-stone-950{--tw-ring-opacity:1;--tw-ring-color:rgb(12 10 9/var(--tw-ring-opacity,1))}.ring-stone-950\/0{--tw-ring-color:rgba(12,10,9,0)}.ring-stone-950\/10{--tw-ring-color:rgba(12,10,9,.1)}.ring-stone-950\/100{--tw-ring-color:#0c0a09}.ring-stone-950\/15{--tw-ring-color:rgba(12,10,9,.15)}.ring-stone-950\/20{--tw-ring-color:rgba(12,10,9,.2)}.ring-stone-950\/25{--tw-ring-color:rgba(12,10,9,.25)}.ring-stone-950\/30{--tw-ring-color:rgba(12,10,9,.3)}.ring-stone-950\/35{--tw-ring-color:rgba(12,10,9,.35)}.ring-stone-950\/40{--tw-ring-color:rgba(12,10,9,.4)}.ring-stone-950\/45{--tw-ring-color:rgba(12,10,9,.45)}.ring-stone-950\/5{--tw-ring-color:rgba(12,10,9,.05)}.ring-stone-950\/50{--tw-ring-color:rgba(12,10,9,.5)}.ring-stone-950\/55{--tw-ring-color:rgba(12,10,9,.55)}.ring-stone-950\/60{--tw-ring-color:rgba(12,10,9,.6)}.ring-stone-950\/65{--tw-ring-color:rgba(12,10,9,.65)}.ring-stone-950\/70{--tw-ring-color:rgba(12,10,9,.7)}.ring-stone-950\/75{--tw-ring-color:rgba(12,10,9,.75)}.ring-stone-950\/80{--tw-ring-color:rgba(12,10,9,.8)}.ring-stone-950\/85{--tw-ring-color:rgba(12,10,9,.85)}.ring-stone-950\/90{--tw-ring-color:rgba(12,10,9,.9)}.ring-stone-950\/95{--tw-ring-color:rgba(12,10,9,.95)}.ring-teal-100{--tw-ring-opacity:1;--tw-ring-color:rgb(204 251 241/var(--tw-ring-opacity,1))}.ring-teal-100\/0{--tw-ring-color:rgba(204,251,241,0)}.ring-teal-100\/10{--tw-ring-color:rgba(204,251,241,.1)}.ring-teal-100\/100{--tw-ring-color:#ccfbf1}.ring-teal-100\/15{--tw-ring-color:rgba(204,251,241,.15)}.ring-teal-100\/20{--tw-ring-color:rgba(204,251,241,.2)}.ring-teal-100\/25{--tw-ring-color:rgba(204,251,241,.25)}.ring-teal-100\/30{--tw-ring-color:rgba(204,251,241,.3)}.ring-teal-100\/35{--tw-ring-color:rgba(204,251,241,.35)}.ring-teal-100\/40{--tw-ring-color:rgba(204,251,241,.4)}.ring-teal-100\/45{--tw-ring-color:rgba(204,251,241,.45)}.ring-teal-100\/5{--tw-ring-color:rgba(204,251,241,.05)}.ring-teal-100\/50{--tw-ring-color:rgba(204,251,241,.5)}.ring-teal-100\/55{--tw-ring-color:rgba(204,251,241,.55)}.ring-teal-100\/60{--tw-ring-color:rgba(204,251,241,.6)}.ring-teal-100\/65{--tw-ring-color:rgba(204,251,241,.65)}.ring-teal-100\/70{--tw-ring-color:rgba(204,251,241,.7)}.ring-teal-100\/75{--tw-ring-color:rgba(204,251,241,.75)}.ring-teal-100\/80{--tw-ring-color:rgba(204,251,241,.8)}.ring-teal-100\/85{--tw-ring-color:rgba(204,251,241,.85)}.ring-teal-100\/90{--tw-ring-color:rgba(204,251,241,.9)}.ring-teal-100\/95{--tw-ring-color:rgba(204,251,241,.95)}.ring-teal-200{--tw-ring-opacity:1;--tw-ring-color:rgb(153 246 228/var(--tw-ring-opacity,1))}.ring-teal-200\/0{--tw-ring-color:rgba(153,246,228,0)}.ring-teal-200\/10{--tw-ring-color:rgba(153,246,228,.1)}.ring-teal-200\/100{--tw-ring-color:#99f6e4}.ring-teal-200\/15{--tw-ring-color:rgba(153,246,228,.15)}.ring-teal-200\/20{--tw-ring-color:rgba(153,246,228,.2)}.ring-teal-200\/25{--tw-ring-color:rgba(153,246,228,.25)}.ring-teal-200\/30{--tw-ring-color:rgba(153,246,228,.3)}.ring-teal-200\/35{--tw-ring-color:rgba(153,246,228,.35)}.ring-teal-200\/40{--tw-ring-color:rgba(153,246,228,.4)}.ring-teal-200\/45{--tw-ring-color:rgba(153,246,228,.45)}.ring-teal-200\/5{--tw-ring-color:rgba(153,246,228,.05)}.ring-teal-200\/50{--tw-ring-color:rgba(153,246,228,.5)}.ring-teal-200\/55{--tw-ring-color:rgba(153,246,228,.55)}.ring-teal-200\/60{--tw-ring-color:rgba(153,246,228,.6)}.ring-teal-200\/65{--tw-ring-color:rgba(153,246,228,.65)}.ring-teal-200\/70{--tw-ring-color:rgba(153,246,228,.7)}.ring-teal-200\/75{--tw-ring-color:rgba(153,246,228,.75)}.ring-teal-200\/80{--tw-ring-color:rgba(153,246,228,.8)}.ring-teal-200\/85{--tw-ring-color:rgba(153,246,228,.85)}.ring-teal-200\/90{--tw-ring-color:rgba(153,246,228,.9)}.ring-teal-200\/95{--tw-ring-color:rgba(153,246,228,.95)}.ring-teal-400{--tw-ring-opacity:1;--tw-ring-color:rgb(45 212 191/var(--tw-ring-opacity,1))}.ring-teal-400\/0{--tw-ring-color:rgba(45,212,191,0)}.ring-teal-400\/10{--tw-ring-color:rgba(45,212,191,.1)}.ring-teal-400\/100{--tw-ring-color:#2dd4bf}.ring-teal-400\/15{--tw-ring-color:rgba(45,212,191,.15)}.ring-teal-400\/20{--tw-ring-color:rgba(45,212,191,.2)}.ring-teal-400\/25{--tw-ring-color:rgba(45,212,191,.25)}.ring-teal-400\/30{--tw-ring-color:rgba(45,212,191,.3)}.ring-teal-400\/35{--tw-ring-color:rgba(45,212,191,.35)}.ring-teal-400\/40{--tw-ring-color:rgba(45,212,191,.4)}.ring-teal-400\/45{--tw-ring-color:rgba(45,212,191,.45)}.ring-teal-400\/5{--tw-ring-color:rgba(45,212,191,.05)}.ring-teal-400\/50{--tw-ring-color:rgba(45,212,191,.5)}.ring-teal-400\/55{--tw-ring-color:rgba(45,212,191,.55)}.ring-teal-400\/60{--tw-ring-color:rgba(45,212,191,.6)}.ring-teal-400\/65{--tw-ring-color:rgba(45,212,191,.65)}.ring-teal-400\/70{--tw-ring-color:rgba(45,212,191,.7)}.ring-teal-400\/75{--tw-ring-color:rgba(45,212,191,.75)}.ring-teal-400\/80{--tw-ring-color:rgba(45,212,191,.8)}.ring-teal-400\/85{--tw-ring-color:rgba(45,212,191,.85)}.ring-teal-400\/90{--tw-ring-color:rgba(45,212,191,.9)}.ring-teal-400\/95{--tw-ring-color:rgba(45,212,191,.95)}.ring-teal-50{--tw-ring-opacity:1;--tw-ring-color:rgb(240 253 250/var(--tw-ring-opacity,1))}.ring-teal-50\/0{--tw-ring-color:rgba(240,253,250,0)}.ring-teal-50\/10{--tw-ring-color:rgba(240,253,250,.1)}.ring-teal-50\/100{--tw-ring-color:#f0fdfa}.ring-teal-50\/15{--tw-ring-color:rgba(240,253,250,.15)}.ring-teal-50\/20{--tw-ring-color:rgba(240,253,250,.2)}.ring-teal-50\/25{--tw-ring-color:rgba(240,253,250,.25)}.ring-teal-50\/30{--tw-ring-color:rgba(240,253,250,.3)}.ring-teal-50\/35{--tw-ring-color:rgba(240,253,250,.35)}.ring-teal-50\/40{--tw-ring-color:rgba(240,253,250,.4)}.ring-teal-50\/45{--tw-ring-color:rgba(240,253,250,.45)}.ring-teal-50\/5{--tw-ring-color:rgba(240,253,250,.05)}.ring-teal-50\/50{--tw-ring-color:rgba(240,253,250,.5)}.ring-teal-50\/55{--tw-ring-color:rgba(240,253,250,.55)}.ring-teal-50\/60{--tw-ring-color:rgba(240,253,250,.6)}.ring-teal-50\/65{--tw-ring-color:rgba(240,253,250,.65)}.ring-teal-50\/70{--tw-ring-color:rgba(240,253,250,.7)}.ring-teal-50\/75{--tw-ring-color:rgba(240,253,250,.75)}.ring-teal-50\/80{--tw-ring-color:rgba(240,253,250,.8)}.ring-teal-50\/85{--tw-ring-color:rgba(240,253,250,.85)}.ring-teal-50\/90{--tw-ring-color:rgba(240,253,250,.9)}.ring-teal-50\/95{--tw-ring-color:rgba(240,253,250,.95)}.ring-teal-500{--tw-ring-opacity:1;--tw-ring-color:rgb(20 184 166/var(--tw-ring-opacity,1))}.ring-teal-500\/0{--tw-ring-color:rgba(20,184,166,0)}.ring-teal-500\/10{--tw-ring-color:rgba(20,184,166,.1)}.ring-teal-500\/100{--tw-ring-color:#14b8a6}.ring-teal-500\/15{--tw-ring-color:rgba(20,184,166,.15)}.ring-teal-500\/20{--tw-ring-color:rgba(20,184,166,.2)}.ring-teal-500\/25{--tw-ring-color:rgba(20,184,166,.25)}.ring-teal-500\/30{--tw-ring-color:rgba(20,184,166,.3)}.ring-teal-500\/35{--tw-ring-color:rgba(20,184,166,.35)}.ring-teal-500\/40{--tw-ring-color:rgba(20,184,166,.4)}.ring-teal-500\/45{--tw-ring-color:rgba(20,184,166,.45)}.ring-teal-500\/5{--tw-ring-color:rgba(20,184,166,.05)}.ring-teal-500\/50{--tw-ring-color:rgba(20,184,166,.5)}.ring-teal-500\/55{--tw-ring-color:rgba(20,184,166,.55)}.ring-teal-500\/60{--tw-ring-color:rgba(20,184,166,.6)}.ring-teal-500\/65{--tw-ring-color:rgba(20,184,166,.65)}.ring-teal-500\/70{--tw-ring-color:rgba(20,184,166,.7)}.ring-teal-500\/75{--tw-ring-color:rgba(20,184,166,.75)}.ring-teal-500\/80{--tw-ring-color:rgba(20,184,166,.8)}.ring-teal-500\/85{--tw-ring-color:rgba(20,184,166,.85)}.ring-teal-500\/90{--tw-ring-color:rgba(20,184,166,.9)}.ring-teal-500\/95{--tw-ring-color:rgba(20,184,166,.95)}.ring-teal-950{--tw-ring-opacity:1;--tw-ring-color:rgb(4 47 46/var(--tw-ring-opacity,1))}.ring-teal-950\/0{--tw-ring-color:rgba(4,47,46,0)}.ring-teal-950\/10{--tw-ring-color:rgba(4,47,46,.1)}.ring-teal-950\/100{--tw-ring-color:#042f2e}.ring-teal-950\/15{--tw-ring-color:rgba(4,47,46,.15)}.ring-teal-950\/20{--tw-ring-color:rgba(4,47,46,.2)}.ring-teal-950\/25{--tw-ring-color:rgba(4,47,46,.25)}.ring-teal-950\/30{--tw-ring-color:rgba(4,47,46,.3)}.ring-teal-950\/35{--tw-ring-color:rgba(4,47,46,.35)}.ring-teal-950\/40{--tw-ring-color:rgba(4,47,46,.4)}.ring-teal-950\/45{--tw-ring-color:rgba(4,47,46,.45)}.ring-teal-950\/5{--tw-ring-color:rgba(4,47,46,.05)}.ring-teal-950\/50{--tw-ring-color:rgba(4,47,46,.5)}.ring-teal-950\/55{--tw-ring-color:rgba(4,47,46,.55)}.ring-teal-950\/60{--tw-ring-color:rgba(4,47,46,.6)}.ring-teal-950\/65{--tw-ring-color:rgba(4,47,46,.65)}.ring-teal-950\/70{--tw-ring-color:rgba(4,47,46,.7)}.ring-teal-950\/75{--tw-ring-color:rgba(4,47,46,.75)}.ring-teal-950\/80{--tw-ring-color:rgba(4,47,46,.8)}.ring-teal-950\/85{--tw-ring-color:rgba(4,47,46,.85)}.ring-teal-950\/90{--tw-ring-color:rgba(4,47,46,.9)}.ring-teal-950\/95{--tw-ring-color:rgba(4,47,46,.95)}.ring-violet-100{--tw-ring-opacity:1;--tw-ring-color:rgb(237 233 254/var(--tw-ring-opacity,1))}.ring-violet-100\/0{--tw-ring-color:rgba(237,233,254,0)}.ring-violet-100\/10{--tw-ring-color:rgba(237,233,254,.1)}.ring-violet-100\/100{--tw-ring-color:#ede9fe}.ring-violet-100\/15{--tw-ring-color:rgba(237,233,254,.15)}.ring-violet-100\/20{--tw-ring-color:rgba(237,233,254,.2)}.ring-violet-100\/25{--tw-ring-color:rgba(237,233,254,.25)}.ring-violet-100\/30{--tw-ring-color:rgba(237,233,254,.3)}.ring-violet-100\/35{--tw-ring-color:rgba(237,233,254,.35)}.ring-violet-100\/40{--tw-ring-color:rgba(237,233,254,.4)}.ring-violet-100\/45{--tw-ring-color:rgba(237,233,254,.45)}.ring-violet-100\/5{--tw-ring-color:rgba(237,233,254,.05)}.ring-violet-100\/50{--tw-ring-color:rgba(237,233,254,.5)}.ring-violet-100\/55{--tw-ring-color:rgba(237,233,254,.55)}.ring-violet-100\/60{--tw-ring-color:rgba(237,233,254,.6)}.ring-violet-100\/65{--tw-ring-color:rgba(237,233,254,.65)}.ring-violet-100\/70{--tw-ring-color:rgba(237,233,254,.7)}.ring-violet-100\/75{--tw-ring-color:rgba(237,233,254,.75)}.ring-violet-100\/80{--tw-ring-color:rgba(237,233,254,.8)}.ring-violet-100\/85{--tw-ring-color:rgba(237,233,254,.85)}.ring-violet-100\/90{--tw-ring-color:rgba(237,233,254,.9)}.ring-violet-100\/95{--tw-ring-color:rgba(237,233,254,.95)}.ring-violet-200{--tw-ring-opacity:1;--tw-ring-color:rgb(221 214 254/var(--tw-ring-opacity,1))}.ring-violet-200\/0{--tw-ring-color:rgba(221,214,254,0)}.ring-violet-200\/10{--tw-ring-color:rgba(221,214,254,.1)}.ring-violet-200\/100{--tw-ring-color:#ddd6fe}.ring-violet-200\/15{--tw-ring-color:rgba(221,214,254,.15)}.ring-violet-200\/20{--tw-ring-color:rgba(221,214,254,.2)}.ring-violet-200\/25{--tw-ring-color:rgba(221,214,254,.25)}.ring-violet-200\/30{--tw-ring-color:rgba(221,214,254,.3)}.ring-violet-200\/35{--tw-ring-color:rgba(221,214,254,.35)}.ring-violet-200\/40{--tw-ring-color:rgba(221,214,254,.4)}.ring-violet-200\/45{--tw-ring-color:rgba(221,214,254,.45)}.ring-violet-200\/5{--tw-ring-color:rgba(221,214,254,.05)}.ring-violet-200\/50{--tw-ring-color:rgba(221,214,254,.5)}.ring-violet-200\/55{--tw-ring-color:rgba(221,214,254,.55)}.ring-violet-200\/60{--tw-ring-color:rgba(221,214,254,.6)}.ring-violet-200\/65{--tw-ring-color:rgba(221,214,254,.65)}.ring-violet-200\/70{--tw-ring-color:rgba(221,214,254,.7)}.ring-violet-200\/75{--tw-ring-color:rgba(221,214,254,.75)}.ring-violet-200\/80{--tw-ring-color:rgba(221,214,254,.8)}.ring-violet-200\/85{--tw-ring-color:rgba(221,214,254,.85)}.ring-violet-200\/90{--tw-ring-color:rgba(221,214,254,.9)}.ring-violet-200\/95{--tw-ring-color:rgba(221,214,254,.95)}.ring-violet-400{--tw-ring-opacity:1;--tw-ring-color:rgb(167 139 250/var(--tw-ring-opacity,1))}.ring-violet-400\/0{--tw-ring-color:rgba(167,139,250,0)}.ring-violet-400\/10{--tw-ring-color:rgba(167,139,250,.1)}.ring-violet-400\/100{--tw-ring-color:#a78bfa}.ring-violet-400\/15{--tw-ring-color:rgba(167,139,250,.15)}.ring-violet-400\/20{--tw-ring-color:rgba(167,139,250,.2)}.ring-violet-400\/25{--tw-ring-color:rgba(167,139,250,.25)}.ring-violet-400\/30{--tw-ring-color:rgba(167,139,250,.3)}.ring-violet-400\/35{--tw-ring-color:rgba(167,139,250,.35)}.ring-violet-400\/40{--tw-ring-color:rgba(167,139,250,.4)}.ring-violet-400\/45{--tw-ring-color:rgba(167,139,250,.45)}.ring-violet-400\/5{--tw-ring-color:rgba(167,139,250,.05)}.ring-violet-400\/50{--tw-ring-color:rgba(167,139,250,.5)}.ring-violet-400\/55{--tw-ring-color:rgba(167,139,250,.55)}.ring-violet-400\/60{--tw-ring-color:rgba(167,139,250,.6)}.ring-violet-400\/65{--tw-ring-color:rgba(167,139,250,.65)}.ring-violet-400\/70{--tw-ring-color:rgba(167,139,250,.7)}.ring-violet-400\/75{--tw-ring-color:rgba(167,139,250,.75)}.ring-violet-400\/80{--tw-ring-color:rgba(167,139,250,.8)}.ring-violet-400\/85{--tw-ring-color:rgba(167,139,250,.85)}.ring-violet-400\/90{--tw-ring-color:rgba(167,139,250,.9)}.ring-violet-400\/95{--tw-ring-color:rgba(167,139,250,.95)}.ring-violet-50{--tw-ring-opacity:1;--tw-ring-color:rgb(245 243 255/var(--tw-ring-opacity,1))}.ring-violet-50\/0{--tw-ring-color:rgba(245,243,255,0)}.ring-violet-50\/10{--tw-ring-color:rgba(245,243,255,.1)}.ring-violet-50\/100{--tw-ring-color:#f5f3ff}.ring-violet-50\/15{--tw-ring-color:rgba(245,243,255,.15)}.ring-violet-50\/20{--tw-ring-color:rgba(245,243,255,.2)}.ring-violet-50\/25{--tw-ring-color:rgba(245,243,255,.25)}.ring-violet-50\/30{--tw-ring-color:rgba(245,243,255,.3)}.ring-violet-50\/35{--tw-ring-color:rgba(245,243,255,.35)}.ring-violet-50\/40{--tw-ring-color:rgba(245,243,255,.4)}.ring-violet-50\/45{--tw-ring-color:rgba(245,243,255,.45)}.ring-violet-50\/5{--tw-ring-color:rgba(245,243,255,.05)}.ring-violet-50\/50{--tw-ring-color:rgba(245,243,255,.5)}.ring-violet-50\/55{--tw-ring-color:rgba(245,243,255,.55)}.ring-violet-50\/60{--tw-ring-color:rgba(245,243,255,.6)}.ring-violet-50\/65{--tw-ring-color:rgba(245,243,255,.65)}.ring-violet-50\/70{--tw-ring-color:rgba(245,243,255,.7)}.ring-violet-50\/75{--tw-ring-color:rgba(245,243,255,.75)}.ring-violet-50\/80{--tw-ring-color:rgba(245,243,255,.8)}.ring-violet-50\/85{--tw-ring-color:rgba(245,243,255,.85)}.ring-violet-50\/90{--tw-ring-color:rgba(245,243,255,.9)}.ring-violet-50\/95{--tw-ring-color:rgba(245,243,255,.95)}.ring-violet-500{--tw-ring-opacity:1;--tw-ring-color:rgb(139 92 246/var(--tw-ring-opacity,1))}.ring-violet-500\/0{--tw-ring-color:rgba(139,92,246,0)}.ring-violet-500\/10{--tw-ring-color:rgba(139,92,246,.1)}.ring-violet-500\/100{--tw-ring-color:#8b5cf6}.ring-violet-500\/15{--tw-ring-color:rgba(139,92,246,.15)}.ring-violet-500\/20{--tw-ring-color:rgba(139,92,246,.2)}.ring-violet-500\/25{--tw-ring-color:rgba(139,92,246,.25)}.ring-violet-500\/30{--tw-ring-color:rgba(139,92,246,.3)}.ring-violet-500\/35{--tw-ring-color:rgba(139,92,246,.35)}.ring-violet-500\/40{--tw-ring-color:rgba(139,92,246,.4)}.ring-violet-500\/45{--tw-ring-color:rgba(139,92,246,.45)}.ring-violet-500\/5{--tw-ring-color:rgba(139,92,246,.05)}.ring-violet-500\/50{--tw-ring-color:rgba(139,92,246,.5)}.ring-violet-500\/55{--tw-ring-color:rgba(139,92,246,.55)}.ring-violet-500\/60{--tw-ring-color:rgba(139,92,246,.6)}.ring-violet-500\/65{--tw-ring-color:rgba(139,92,246,.65)}.ring-violet-500\/70{--tw-ring-color:rgba(139,92,246,.7)}.ring-violet-500\/75{--tw-ring-color:rgba(139,92,246,.75)}.ring-violet-500\/80{--tw-ring-color:rgba(139,92,246,.8)}.ring-violet-500\/85{--tw-ring-color:rgba(139,92,246,.85)}.ring-violet-500\/90{--tw-ring-color:rgba(139,92,246,.9)}.ring-violet-500\/95{--tw-ring-color:rgba(139,92,246,.95)}.ring-violet-950{--tw-ring-opacity:1;--tw-ring-color:rgb(46 16 101/var(--tw-ring-opacity,1))}.ring-violet-950\/0{--tw-ring-color:rgba(46,16,101,0)}.ring-violet-950\/10{--tw-ring-color:rgba(46,16,101,.1)}.ring-violet-950\/100{--tw-ring-color:#2e1065}.ring-violet-950\/15{--tw-ring-color:rgba(46,16,101,.15)}.ring-violet-950\/20{--tw-ring-color:rgba(46,16,101,.2)}.ring-violet-950\/25{--tw-ring-color:rgba(46,16,101,.25)}.ring-violet-950\/30{--tw-ring-color:rgba(46,16,101,.3)}.ring-violet-950\/35{--tw-ring-color:rgba(46,16,101,.35)}.ring-violet-950\/40{--tw-ring-color:rgba(46,16,101,.4)}.ring-violet-950\/45{--tw-ring-color:rgba(46,16,101,.45)}.ring-violet-950\/5{--tw-ring-color:rgba(46,16,101,.05)}.ring-violet-950\/50{--tw-ring-color:rgba(46,16,101,.5)}.ring-violet-950\/55{--tw-ring-color:rgba(46,16,101,.55)}.ring-violet-950\/60{--tw-ring-color:rgba(46,16,101,.6)}.ring-violet-950\/65{--tw-ring-color:rgba(46,16,101,.65)}.ring-violet-950\/70{--tw-ring-color:rgba(46,16,101,.7)}.ring-violet-950\/75{--tw-ring-color:rgba(46,16,101,.75)}.ring-violet-950\/80{--tw-ring-color:rgba(46,16,101,.8)}.ring-violet-950\/85{--tw-ring-color:rgba(46,16,101,.85)}.ring-violet-950\/90{--tw-ring-color:rgba(46,16,101,.9)}.ring-violet-950\/95{--tw-ring-color:rgba(46,16,101,.95)}.ring-yellow-100{--tw-ring-opacity:1;--tw-ring-color:rgb(254 249 195/var(--tw-ring-opacity,1))}.ring-yellow-100\/0{--tw-ring-color:hsla(55,97%,88%,0)}.ring-yellow-100\/10{--tw-ring-color:hsla(55,97%,88%,.1)}.ring-yellow-100\/100{--tw-ring-color:#fef9c3}.ring-yellow-100\/15{--tw-ring-color:hsla(55,97%,88%,.15)}.ring-yellow-100\/20{--tw-ring-color:hsla(55,97%,88%,.2)}.ring-yellow-100\/25{--tw-ring-color:hsla(55,97%,88%,.25)}.ring-yellow-100\/30{--tw-ring-color:hsla(55,97%,88%,.3)}.ring-yellow-100\/35{--tw-ring-color:hsla(55,97%,88%,.35)}.ring-yellow-100\/40{--tw-ring-color:hsla(55,97%,88%,.4)}.ring-yellow-100\/45{--tw-ring-color:hsla(55,97%,88%,.45)}.ring-yellow-100\/5{--tw-ring-color:hsla(55,97%,88%,.05)}.ring-yellow-100\/50{--tw-ring-color:hsla(55,97%,88%,.5)}.ring-yellow-100\/55{--tw-ring-color:hsla(55,97%,88%,.55)}.ring-yellow-100\/60{--tw-ring-color:hsla(55,97%,88%,.6)}.ring-yellow-100\/65{--tw-ring-color:hsla(55,97%,88%,.65)}.ring-yellow-100\/70{--tw-ring-color:hsla(55,97%,88%,.7)}.ring-yellow-100\/75{--tw-ring-color:hsla(55,97%,88%,.75)}.ring-yellow-100\/80{--tw-ring-color:hsla(55,97%,88%,.8)}.ring-yellow-100\/85{--tw-ring-color:hsla(55,97%,88%,.85)}.ring-yellow-100\/90{--tw-ring-color:hsla(55,97%,88%,.9)}.ring-yellow-100\/95{--tw-ring-color:hsla(55,97%,88%,.95)}.ring-yellow-200{--tw-ring-opacity:1;--tw-ring-color:rgb(254 240 138/var(--tw-ring-opacity,1))}.ring-yellow-200\/0{--tw-ring-color:hsla(53,98%,77%,0)}.ring-yellow-200\/10{--tw-ring-color:hsla(53,98%,77%,.1)}.ring-yellow-200\/100{--tw-ring-color:#fef08a}.ring-yellow-200\/15{--tw-ring-color:hsla(53,98%,77%,.15)}.ring-yellow-200\/20{--tw-ring-color:hsla(53,98%,77%,.2)}.ring-yellow-200\/25{--tw-ring-color:hsla(53,98%,77%,.25)}.ring-yellow-200\/30{--tw-ring-color:hsla(53,98%,77%,.3)}.ring-yellow-200\/35{--tw-ring-color:hsla(53,98%,77%,.35)}.ring-yellow-200\/40{--tw-ring-color:hsla(53,98%,77%,.4)}.ring-yellow-200\/45{--tw-ring-color:hsla(53,98%,77%,.45)}.ring-yellow-200\/5{--tw-ring-color:hsla(53,98%,77%,.05)}.ring-yellow-200\/50{--tw-ring-color:hsla(53,98%,77%,.5)}.ring-yellow-200\/55{--tw-ring-color:hsla(53,98%,77%,.55)}.ring-yellow-200\/60{--tw-ring-color:hsla(53,98%,77%,.6)}.ring-yellow-200\/65{--tw-ring-color:hsla(53,98%,77%,.65)}.ring-yellow-200\/70{--tw-ring-color:hsla(53,98%,77%,.7)}.ring-yellow-200\/75{--tw-ring-color:hsla(53,98%,77%,.75)}.ring-yellow-200\/80{--tw-ring-color:hsla(53,98%,77%,.8)}.ring-yellow-200\/85{--tw-ring-color:hsla(53,98%,77%,.85)}.ring-yellow-200\/90{--tw-ring-color:hsla(53,98%,77%,.9)}.ring-yellow-200\/95{--tw-ring-color:hsla(53,98%,77%,.95)}.ring-yellow-400{--tw-ring-opacity:1;--tw-ring-color:rgb(250 204 21/var(--tw-ring-opacity,1))}.ring-yellow-400\/0{--tw-ring-color:rgba(250,204,21,0)}.ring-yellow-400\/10{--tw-ring-color:rgba(250,204,21,.1)}.ring-yellow-400\/100{--tw-ring-color:#facc15}.ring-yellow-400\/15{--tw-ring-color:rgba(250,204,21,.15)}.ring-yellow-400\/20{--tw-ring-color:rgba(250,204,21,.2)}.ring-yellow-400\/25{--tw-ring-color:rgba(250,204,21,.25)}.ring-yellow-400\/30{--tw-ring-color:rgba(250,204,21,.3)}.ring-yellow-400\/35{--tw-ring-color:rgba(250,204,21,.35)}.ring-yellow-400\/40{--tw-ring-color:rgba(250,204,21,.4)}.ring-yellow-400\/45{--tw-ring-color:rgba(250,204,21,.45)}.ring-yellow-400\/5{--tw-ring-color:rgba(250,204,21,.05)}.ring-yellow-400\/50{--tw-ring-color:rgba(250,204,21,.5)}.ring-yellow-400\/55{--tw-ring-color:rgba(250,204,21,.55)}.ring-yellow-400\/60{--tw-ring-color:rgba(250,204,21,.6)}.ring-yellow-400\/65{--tw-ring-color:rgba(250,204,21,.65)}.ring-yellow-400\/70{--tw-ring-color:rgba(250,204,21,.7)}.ring-yellow-400\/75{--tw-ring-color:rgba(250,204,21,.75)}.ring-yellow-400\/80{--tw-ring-color:rgba(250,204,21,.8)}.ring-yellow-400\/85{--tw-ring-color:rgba(250,204,21,.85)}.ring-yellow-400\/90{--tw-ring-color:rgba(250,204,21,.9)}.ring-yellow-400\/95{--tw-ring-color:rgba(250,204,21,.95)}.ring-yellow-50{--tw-ring-opacity:1;--tw-ring-color:rgb(254 252 232/var(--tw-ring-opacity,1))}.ring-yellow-50\/0{--tw-ring-color:hsla(55,92%,95%,0)}.ring-yellow-50\/10{--tw-ring-color:hsla(55,92%,95%,.1)}.ring-yellow-50\/100{--tw-ring-color:#fefce8}.ring-yellow-50\/15{--tw-ring-color:hsla(55,92%,95%,.15)}.ring-yellow-50\/20{--tw-ring-color:hsla(55,92%,95%,.2)}.ring-yellow-50\/25{--tw-ring-color:hsla(55,92%,95%,.25)}.ring-yellow-50\/30{--tw-ring-color:hsla(55,92%,95%,.3)}.ring-yellow-50\/35{--tw-ring-color:hsla(55,92%,95%,.35)}.ring-yellow-50\/40{--tw-ring-color:hsla(55,92%,95%,.4)}.ring-yellow-50\/45{--tw-ring-color:hsla(55,92%,95%,.45)}.ring-yellow-50\/5{--tw-ring-color:hsla(55,92%,95%,.05)}.ring-yellow-50\/50{--tw-ring-color:hsla(55,92%,95%,.5)}.ring-yellow-50\/55{--tw-ring-color:hsla(55,92%,95%,.55)}.ring-yellow-50\/60{--tw-ring-color:hsla(55,92%,95%,.6)}.ring-yellow-50\/65{--tw-ring-color:hsla(55,92%,95%,.65)}.ring-yellow-50\/70{--tw-ring-color:hsla(55,92%,95%,.7)}.ring-yellow-50\/75{--tw-ring-color:hsla(55,92%,95%,.75)}.ring-yellow-50\/80{--tw-ring-color:hsla(55,92%,95%,.8)}.ring-yellow-50\/85{--tw-ring-color:hsla(55,92%,95%,.85)}.ring-yellow-50\/90{--tw-ring-color:hsla(55,92%,95%,.9)}.ring-yellow-50\/95{--tw-ring-color:hsla(55,92%,95%,.95)}.ring-yellow-500{--tw-ring-opacity:1;--tw-ring-color:rgb(234 179 8/var(--tw-ring-opacity,1))}.ring-yellow-500\/0{--tw-ring-color:rgba(234,179,8,0)}.ring-yellow-500\/10{--tw-ring-color:rgba(234,179,8,.1)}.ring-yellow-500\/100{--tw-ring-color:#eab308}.ring-yellow-500\/15{--tw-ring-color:rgba(234,179,8,.15)}.ring-yellow-500\/20{--tw-ring-color:rgba(234,179,8,.2)}.ring-yellow-500\/25{--tw-ring-color:rgba(234,179,8,.25)}.ring-yellow-500\/30{--tw-ring-color:rgba(234,179,8,.3)}.ring-yellow-500\/35{--tw-ring-color:rgba(234,179,8,.35)}.ring-yellow-500\/40{--tw-ring-color:rgba(234,179,8,.4)}.ring-yellow-500\/45{--tw-ring-color:rgba(234,179,8,.45)}.ring-yellow-500\/5{--tw-ring-color:rgba(234,179,8,.05)}.ring-yellow-500\/50{--tw-ring-color:rgba(234,179,8,.5)}.ring-yellow-500\/55{--tw-ring-color:rgba(234,179,8,.55)}.ring-yellow-500\/60{--tw-ring-color:rgba(234,179,8,.6)}.ring-yellow-500\/65{--tw-ring-color:rgba(234,179,8,.65)}.ring-yellow-500\/70{--tw-ring-color:rgba(234,179,8,.7)}.ring-yellow-500\/75{--tw-ring-color:rgba(234,179,8,.75)}.ring-yellow-500\/80{--tw-ring-color:rgba(234,179,8,.8)}.ring-yellow-500\/85{--tw-ring-color:rgba(234,179,8,.85)}.ring-yellow-500\/90{--tw-ring-color:rgba(234,179,8,.9)}.ring-yellow-500\/95{--tw-ring-color:rgba(234,179,8,.95)}.ring-yellow-950{--tw-ring-opacity:1;--tw-ring-color:rgb(66 32 6/var(--tw-ring-opacity,1))}.ring-yellow-950\/0{--tw-ring-color:rgba(66,32,6,0)}.ring-yellow-950\/10{--tw-ring-color:rgba(66,32,6,.1)}.ring-yellow-950\/100{--tw-ring-color:#422006}.ring-yellow-950\/15{--tw-ring-color:rgba(66,32,6,.15)}.ring-yellow-950\/20{--tw-ring-color:rgba(66,32,6,.2)}.ring-yellow-950\/25{--tw-ring-color:rgba(66,32,6,.25)}.ring-yellow-950\/30{--tw-ring-color:rgba(66,32,6,.3)}.ring-yellow-950\/35{--tw-ring-color:rgba(66,32,6,.35)}.ring-yellow-950\/40{--tw-ring-color:rgba(66,32,6,.4)}.ring-yellow-950\/45{--tw-ring-color:rgba(66,32,6,.45)}.ring-yellow-950\/5{--tw-ring-color:rgba(66,32,6,.05)}.ring-yellow-950\/50{--tw-ring-color:rgba(66,32,6,.5)}.ring-yellow-950\/55{--tw-ring-color:rgba(66,32,6,.55)}.ring-yellow-950\/60{--tw-ring-color:rgba(66,32,6,.6)}.ring-yellow-950\/65{--tw-ring-color:rgba(66,32,6,.65)}.ring-yellow-950\/70{--tw-ring-color:rgba(66,32,6,.7)}.ring-yellow-950\/75{--tw-ring-color:rgba(66,32,6,.75)}.ring-yellow-950\/80{--tw-ring-color:rgba(66,32,6,.8)}.ring-yellow-950\/85{--tw-ring-color:rgba(66,32,6,.85)}.ring-yellow-950\/90{--tw-ring-color:rgba(66,32,6,.9)}.ring-yellow-950\/95{--tw-ring-color:rgba(66,32,6,.95)}.ring-zinc-100{--tw-ring-opacity:1;--tw-ring-color:rgb(244 244 245/var(--tw-ring-opacity,1))}.ring-zinc-100\/0{--tw-ring-color:hsla(240,5%,96%,0)}.ring-zinc-100\/10{--tw-ring-color:hsla(240,5%,96%,.1)}.ring-zinc-100\/100{--tw-ring-color:#f4f4f5}.ring-zinc-100\/15{--tw-ring-color:hsla(240,5%,96%,.15)}.ring-zinc-100\/20{--tw-ring-color:hsla(240,5%,96%,.2)}.ring-zinc-100\/25{--tw-ring-color:hsla(240,5%,96%,.25)}.ring-zinc-100\/30{--tw-ring-color:hsla(240,5%,96%,.3)}.ring-zinc-100\/35{--tw-ring-color:hsla(240,5%,96%,.35)}.ring-zinc-100\/40{--tw-ring-color:hsla(240,5%,96%,.4)}.ring-zinc-100\/45{--tw-ring-color:hsla(240,5%,96%,.45)}.ring-zinc-100\/5{--tw-ring-color:hsla(240,5%,96%,.05)}.ring-zinc-100\/50{--tw-ring-color:hsla(240,5%,96%,.5)}.ring-zinc-100\/55{--tw-ring-color:hsla(240,5%,96%,.55)}.ring-zinc-100\/60{--tw-ring-color:hsla(240,5%,96%,.6)}.ring-zinc-100\/65{--tw-ring-color:hsla(240,5%,96%,.65)}.ring-zinc-100\/70{--tw-ring-color:hsla(240,5%,96%,.7)}.ring-zinc-100\/75{--tw-ring-color:hsla(240,5%,96%,.75)}.ring-zinc-100\/80{--tw-ring-color:hsla(240,5%,96%,.8)}.ring-zinc-100\/85{--tw-ring-color:hsla(240,5%,96%,.85)}.ring-zinc-100\/90{--tw-ring-color:hsla(240,5%,96%,.9)}.ring-zinc-100\/95{--tw-ring-color:hsla(240,5%,96%,.95)}.ring-zinc-200{--tw-ring-opacity:1;--tw-ring-color:rgb(228 228 231/var(--tw-ring-opacity,1))}.ring-zinc-200\/0{--tw-ring-color:hsla(240,6%,90%,0)}.ring-zinc-200\/10{--tw-ring-color:hsla(240,6%,90%,.1)}.ring-zinc-200\/100{--tw-ring-color:#e4e4e7}.ring-zinc-200\/15{--tw-ring-color:hsla(240,6%,90%,.15)}.ring-zinc-200\/20{--tw-ring-color:hsla(240,6%,90%,.2)}.ring-zinc-200\/25{--tw-ring-color:hsla(240,6%,90%,.25)}.ring-zinc-200\/30{--tw-ring-color:hsla(240,6%,90%,.3)}.ring-zinc-200\/35{--tw-ring-color:hsla(240,6%,90%,.35)}.ring-zinc-200\/40{--tw-ring-color:hsla(240,6%,90%,.4)}.ring-zinc-200\/45{--tw-ring-color:hsla(240,6%,90%,.45)}.ring-zinc-200\/5{--tw-ring-color:hsla(240,6%,90%,.05)}.ring-zinc-200\/50{--tw-ring-color:hsla(240,6%,90%,.5)}.ring-zinc-200\/55{--tw-ring-color:hsla(240,6%,90%,.55)}.ring-zinc-200\/60{--tw-ring-color:hsla(240,6%,90%,.6)}.ring-zinc-200\/65{--tw-ring-color:hsla(240,6%,90%,.65)}.ring-zinc-200\/70{--tw-ring-color:hsla(240,6%,90%,.7)}.ring-zinc-200\/75{--tw-ring-color:hsla(240,6%,90%,.75)}.ring-zinc-200\/80{--tw-ring-color:hsla(240,6%,90%,.8)}.ring-zinc-200\/85{--tw-ring-color:hsla(240,6%,90%,.85)}.ring-zinc-200\/90{--tw-ring-color:hsla(240,6%,90%,.9)}.ring-zinc-200\/95{--tw-ring-color:hsla(240,6%,90%,.95)}.ring-zinc-400{--tw-ring-opacity:1;--tw-ring-color:rgb(161 161 170/var(--tw-ring-opacity,1))}.ring-zinc-400\/0{--tw-ring-color:hsla(240,5%,65%,0)}.ring-zinc-400\/10{--tw-ring-color:hsla(240,5%,65%,.1)}.ring-zinc-400\/100{--tw-ring-color:#a1a1aa}.ring-zinc-400\/15{--tw-ring-color:hsla(240,5%,65%,.15)}.ring-zinc-400\/20{--tw-ring-color:hsla(240,5%,65%,.2)}.ring-zinc-400\/25{--tw-ring-color:hsla(240,5%,65%,.25)}.ring-zinc-400\/30{--tw-ring-color:hsla(240,5%,65%,.3)}.ring-zinc-400\/35{--tw-ring-color:hsla(240,5%,65%,.35)}.ring-zinc-400\/40{--tw-ring-color:hsla(240,5%,65%,.4)}.ring-zinc-400\/45{--tw-ring-color:hsla(240,5%,65%,.45)}.ring-zinc-400\/5{--tw-ring-color:hsla(240,5%,65%,.05)}.ring-zinc-400\/50{--tw-ring-color:hsla(240,5%,65%,.5)}.ring-zinc-400\/55{--tw-ring-color:hsla(240,5%,65%,.55)}.ring-zinc-400\/60{--tw-ring-color:hsla(240,5%,65%,.6)}.ring-zinc-400\/65{--tw-ring-color:hsla(240,5%,65%,.65)}.ring-zinc-400\/70{--tw-ring-color:hsla(240,5%,65%,.7)}.ring-zinc-400\/75{--tw-ring-color:hsla(240,5%,65%,.75)}.ring-zinc-400\/80{--tw-ring-color:hsla(240,5%,65%,.8)}.ring-zinc-400\/85{--tw-ring-color:hsla(240,5%,65%,.85)}.ring-zinc-400\/90{--tw-ring-color:hsla(240,5%,65%,.9)}.ring-zinc-400\/95{--tw-ring-color:hsla(240,5%,65%,.95)}.ring-zinc-50{--tw-ring-opacity:1;--tw-ring-color:rgb(250 250 250/var(--tw-ring-opacity,1))}.ring-zinc-50\/0{--tw-ring-color:hsla(0,0%,98%,0)}.ring-zinc-50\/10{--tw-ring-color:hsla(0,0%,98%,.1)}.ring-zinc-50\/100{--tw-ring-color:#fafafa}.ring-zinc-50\/15{--tw-ring-color:hsla(0,0%,98%,.15)}.ring-zinc-50\/20{--tw-ring-color:hsla(0,0%,98%,.2)}.ring-zinc-50\/25{--tw-ring-color:hsla(0,0%,98%,.25)}.ring-zinc-50\/30{--tw-ring-color:hsla(0,0%,98%,.3)}.ring-zinc-50\/35{--tw-ring-color:hsla(0,0%,98%,.35)}.ring-zinc-50\/40{--tw-ring-color:hsla(0,0%,98%,.4)}.ring-zinc-50\/45{--tw-ring-color:hsla(0,0%,98%,.45)}.ring-zinc-50\/5{--tw-ring-color:hsla(0,0%,98%,.05)}.ring-zinc-50\/50{--tw-ring-color:hsla(0,0%,98%,.5)}.ring-zinc-50\/55{--tw-ring-color:hsla(0,0%,98%,.55)}.ring-zinc-50\/60{--tw-ring-color:hsla(0,0%,98%,.6)}.ring-zinc-50\/65{--tw-ring-color:hsla(0,0%,98%,.65)}.ring-zinc-50\/70{--tw-ring-color:hsla(0,0%,98%,.7)}.ring-zinc-50\/75{--tw-ring-color:hsla(0,0%,98%,.75)}.ring-zinc-50\/80{--tw-ring-color:hsla(0,0%,98%,.8)}.ring-zinc-50\/85{--tw-ring-color:hsla(0,0%,98%,.85)}.ring-zinc-50\/90{--tw-ring-color:hsla(0,0%,98%,.9)}.ring-zinc-50\/95{--tw-ring-color:hsla(0,0%,98%,.95)}.ring-zinc-500{--tw-ring-opacity:1;--tw-ring-color:rgb(113 113 122/var(--tw-ring-opacity,1))}.ring-zinc-500\/0{--tw-ring-color:hsla(240,4%,46%,0)}.ring-zinc-500\/10{--tw-ring-color:hsla(240,4%,46%,.1)}.ring-zinc-500\/100{--tw-ring-color:#71717a}.ring-zinc-500\/15{--tw-ring-color:hsla(240,4%,46%,.15)}.ring-zinc-500\/20{--tw-ring-color:hsla(240,4%,46%,.2)}.ring-zinc-500\/25{--tw-ring-color:hsla(240,4%,46%,.25)}.ring-zinc-500\/30{--tw-ring-color:hsla(240,4%,46%,.3)}.ring-zinc-500\/35{--tw-ring-color:hsla(240,4%,46%,.35)}.ring-zinc-500\/40{--tw-ring-color:hsla(240,4%,46%,.4)}.ring-zinc-500\/45{--tw-ring-color:hsla(240,4%,46%,.45)}.ring-zinc-500\/5{--tw-ring-color:hsla(240,4%,46%,.05)}.ring-zinc-500\/50{--tw-ring-color:hsla(240,4%,46%,.5)}.ring-zinc-500\/55{--tw-ring-color:hsla(240,4%,46%,.55)}.ring-zinc-500\/60{--tw-ring-color:hsla(240,4%,46%,.6)}.ring-zinc-500\/65{--tw-ring-color:hsla(240,4%,46%,.65)}.ring-zinc-500\/70{--tw-ring-color:hsla(240,4%,46%,.7)}.ring-zinc-500\/75{--tw-ring-color:hsla(240,4%,46%,.75)}.ring-zinc-500\/80{--tw-ring-color:hsla(240,4%,46%,.8)}.ring-zinc-500\/85{--tw-ring-color:hsla(240,4%,46%,.85)}.ring-zinc-500\/90{--tw-ring-color:hsla(240,4%,46%,.9)}.ring-zinc-500\/95{--tw-ring-color:hsla(240,4%,46%,.95)}.ring-zinc-950{--tw-ring-opacity:1;--tw-ring-color:rgb(9 9 11/var(--tw-ring-opacity,1))}.ring-zinc-950\/0{--tw-ring-color:rgba(9,9,11,0)}.ring-zinc-950\/10{--tw-ring-color:rgba(9,9,11,.1)}.ring-zinc-950\/100{--tw-ring-color:#09090b}.ring-zinc-950\/15{--tw-ring-color:rgba(9,9,11,.15)}.ring-zinc-950\/20{--tw-ring-color:rgba(9,9,11,.2)}.ring-zinc-950\/25{--tw-ring-color:rgba(9,9,11,.25)}.ring-zinc-950\/30{--tw-ring-color:rgba(9,9,11,.3)}.ring-zinc-950\/35{--tw-ring-color:rgba(9,9,11,.35)}.ring-zinc-950\/40{--tw-ring-color:rgba(9,9,11,.4)}.ring-zinc-950\/45{--tw-ring-color:rgba(9,9,11,.45)}.ring-zinc-950\/5{--tw-ring-color:rgba(9,9,11,.05)}.ring-zinc-950\/50{--tw-ring-color:rgba(9,9,11,.5)}.ring-zinc-950\/55{--tw-ring-color:rgba(9,9,11,.55)}.ring-zinc-950\/60{--tw-ring-color:rgba(9,9,11,.6)}.ring-zinc-950\/65{--tw-ring-color:rgba(9,9,11,.65)}.ring-zinc-950\/70{--tw-ring-color:rgba(9,9,11,.7)}.ring-zinc-950\/75{--tw-ring-color:rgba(9,9,11,.75)}.ring-zinc-950\/80{--tw-ring-color:rgba(9,9,11,.8)}.ring-zinc-950\/85{--tw-ring-color:rgba(9,9,11,.85)}.ring-zinc-950\/90{--tw-ring-color:rgba(9,9,11,.9)}.ring-zinc-950\/95{--tw-ring-color:rgba(9,9,11,.95)}.ring-offset-background{--tw-ring-offset-color:hsl(var(--background))}.drop-shadow-sm{--tw-drop-shadow:drop-shadow(0 1px 1px rgba(0,0,0,.05))}.drop-shadow-sm,.filter{filter:var(--tw-blur) var(--tw-brightness) var(--tw-contrast) var(--tw-grayscale) var(--tw-hue-rotate) var(--tw-invert) var(--tw-saturate) var(--tw-sepia) var(--tw-drop-shadow)}.backdrop-blur-sm{--tw-backdrop-blur:blur(4px);-webkit-backdrop-filter:var(--tw-backdrop-blur) var(--tw-backdrop-brightness) var(--tw-backdrop-contrast) var(--tw-backdrop-grayscale) var(--tw-backdrop-hue-rotate) var(--tw-backdrop-invert) var(--tw-backdrop-opacity) var(--tw-backdrop-saturate) var(--tw-backdrop-sepia);backdrop-filter:var(--tw-backdrop-blur) var(--tw-backdrop-brightness) var(--tw-backdrop-contrast) var(--tw-backdrop-grayscale) var(--tw-backdrop-hue-rotate) var(--tw-backdrop-invert) var(--tw-backdrop-opacity) var(--tw-backdrop-saturate) var(--tw-backdrop-sepia)}.transition{transition-property:color,background-color,border-color,text-decoration-color,fill,stroke,opacity,box-shadow,transform,filter,-webkit-backdrop-filter;transition-property:color,background-color,border-color,text-decoration-color,fill,stroke,opacity,box-shadow,transform,filter,backdrop-filter;transition-property:color,background-color,border-color,text-decoration-color,fill,stroke,opacity,box-shadow,transform,filter,backdrop-filter,-webkit-backdrop-filter;transition-timing-function:cubic-bezier(.4,0,.2,1);transition-duration:.15s}.transition-all{transition-property:all;transition-timing-function:cubic-bezier(.4,0,.2,1);transition-duration:.15s}.transition-colors{transition-property:color,background-color,border-color,text-decoration-color,fill,stroke;transition-timing-function:cubic-bezier(.4,0,.2,1);transition-duration:.15s}.transition-opacity{transition-property:opacity;transition-timing-function:cubic-bezier(.4,0,.2,1);transition-duration:.15s}.duration-1000{transition-duration:1s}.duration-200{transition-duration:.2s}.ease-in-out{transition-timing-function:cubic-bezier(.4,0,.2,1)}@keyframes enter{0%{opacity:var(--tw-enter-opacity,1);transform:translate3d(var(--tw-enter-translate-x,0),var(--tw-enter-translate-y,0),0) scale3d(var(--tw-enter-scale,1),var(--tw-enter-scale,1),var(--tw-enter-scale,1)) rotate(var(--tw-enter-rotate,0))}}@keyframes exit{to{opacity:var(--tw-exit-opacity,1);transform:translate3d(var(--tw-exit-translate-x,0),var(--tw-exit-translate-y,0),0) scale3d(var(--tw-exit-scale,1),var(--tw-exit-scale,1),var(--tw-exit-scale,1)) rotate(var(--tw-exit-rotate,0))}}.animate-in{animation-name:enter;animation-duration:.15s;--tw-enter-opacity:initial;--tw-enter-scale:initial;--tw-enter-rotate:initial;--tw-enter-translate-x:initial;--tw-enter-translate-y:initial}.fade-in-0{--tw-enter-opacity:0}.zoom-in-95{--tw-enter-scale:.95}.duration-1000{animation-duration:1s}.duration-200{animation-duration:.2s}.ease-in-out{animation-timing-function:cubic-bezier(.4,0,.2,1)}.running{animation-play-state:running}.file\:border-0::file-selector-button{border-width:0}.file\:bg-transparent::file-selector-button{background-color:transparent}.file\:text-sm::file-selector-button{font-size:.875rem;line-height:1.25rem}.file\:font-medium::file-selector-button{font-weight:500}.placeholder\:text-muted-foreground::-moz-placeholder{color:hsl(var(--muted-foreground))}.placeholder\:text-muted-foreground::placeholder{color:hsl(var(--muted-foreground))}.first\:rounded-l-md:first-child{border-top-left-radius:calc(var(--radius) - 2px);border-bottom-left-radius:calc(var(--radius) - 2px)}.first\:border-l:first-child{border-left-width:1px}.last\:rounded-r-md:last-child{border-top-right-radius:calc(var(--radius) - 2px);border-bottom-right-radius:calc(var(--radius) - 2px)}.hover\:cursor-pointer:hover{cursor:pointer}.hover\:bg-accent:hover{background-color:hsl(var(--accent))}.hover\:bg-destructive\/90:hover{background-color:hsl(var(--destructive)/.9)}.hover\:bg-inherit:hover{background-color:inherit}.hover\:bg-muted:hover{background-color:hsl(var(--muted))}.hover\:bg-muted\/50:hover{background-color:hsl(var(--muted)/.5)}.hover\:bg-neutral-300:hover{--tw-bg-opacity:1;background-color:rgb(212 212 212/var(--tw-bg-opacity,1))}.hover\:bg-orange-500:hover{--tw-bg-opacity:1;background-color:rgb(249 115 22/var(--tw-bg-opacity,1))}.hover\:bg-primary\/40:hover{background-color:hsl(var(--primary)/.4)}.hover\:bg-primary\/90:hover{background-color:hsl(var(--primary)/.9)}.hover\:bg-rose-600:hover{--tw-bg-opacity:1;background-color:rgb(225 29 72/var(--tw-bg-opacity,1))}.hover\:bg-secondary:hover{background-color:hsl(var(--secondary))}.hover\:bg-secondary\/80:hover{background-color:hsl(var(--secondary)/.8)}.hover\:bg-slate-500:hover{--tw-bg-opacity:1;background-color:rgb(100 116 139/var(--tw-bg-opacity,1))}.hover\:bg-stone-100:hover{--tw-bg-opacity:1;background-color:rgb(245 245 244/var(--tw-bg-opacity,1))}.hover\:text-accent-foreground:hover{color:hsl(var(--accent-foreground))}.hover\:text-foreground:hover{color:hsl(var(--foreground))}.hover\:text-gray-300:hover{--tw-text-opacity:1;color:rgb(209 213 219/var(--tw-text-opacity,1))}.hover\:text-gray-500:hover{--tw-text-opacity:1;color:rgb(107 114 128/var(--tw-text-opacity,1))}.hover\:text-green-500:hover{--tw-text-opacity:1;color:rgb(34 197 94/var(--tw-text-opacity,1))}.hover\:text-muted-foreground:hover{color:hsl(var(--muted-foreground))}.hover\:text-primary:hover{color:hsl(var(--primary))}.hover\:text-red-400:hover{--tw-text-opacity:1;color:rgb(248 113 113/var(--tw-text-opacity,1))}.hover\:text-red-500:hover{--tw-text-opacity:1;color:rgb(239 68 68/var(--tw-text-opacity,1))}.hover\:text-rose-400:hover{--tw-text-opacity:1;color:rgb(251 113 133/var(--tw-text-opacity,1))}.hover\:underline:hover{text-decoration-line:underline}.hover\:opacity-100:hover{opacity:1}.hover\:shadow-md:hover{--tw-shadow:0 4px 6px -1px rgba(0,0,0,.1),0 2px 4px -2px rgba(0,0,0,.1);--tw-shadow-colored:0 4px 6px -1px var(--tw-shadow-color),0 2px 4px -2px var(--tw-shadow-color);box-shadow:var(--tw-ring-offset-shadow,0 0 #0000),var(--tw-ring-shadow,0 0 #0000),var(--tw-shadow)}.focus\:border-none:focus{border-style:none}.focus\:bg-accent:focus{background-color:hsl(var(--accent))}.focus\:text-accent-foreground:focus{color:hsl(var(--accent-foreground))}.focus\:caret-amber-100:focus{caret-color:#fef3c7}.focus\:caret-amber-100\/0:focus{caret-color:hsla(48,96%,89%,0)}.focus\:caret-amber-100\/10:focus{caret-color:hsla(48,96%,89%,.1)}.focus\:caret-amber-100\/100:focus{caret-color:#fef3c7}.focus\:caret-amber-100\/15:focus{caret-color:hsla(48,96%,89%,.15)}.focus\:caret-amber-100\/20:focus{caret-color:hsla(48,96%,89%,.2)}.focus\:caret-amber-100\/25:focus{caret-color:hsla(48,96%,89%,.25)}.focus\:caret-amber-100\/30:focus{caret-color:hsla(48,96%,89%,.3)}.focus\:caret-amber-100\/35:focus{caret-color:hsla(48,96%,89%,.35)}.focus\:caret-amber-100\/40:focus{caret-color:hsla(48,96%,89%,.4)}.focus\:caret-amber-100\/45:focus{caret-color:hsla(48,96%,89%,.45)}.focus\:caret-amber-100\/5:focus{caret-color:hsla(48,96%,89%,.05)}.focus\:caret-amber-100\/50:focus{caret-color:hsla(48,96%,89%,.5)}.focus\:caret-amber-100\/55:focus{caret-color:hsla(48,96%,89%,.55)}.focus\:caret-amber-100\/60:focus{caret-color:hsla(48,96%,89%,.6)}.focus\:caret-amber-100\/65:focus{caret-color:hsla(48,96%,89%,.65)}.focus\:caret-amber-100\/70:focus{caret-color:hsla(48,96%,89%,.7)}.focus\:caret-amber-100\/75:focus{caret-color:hsla(48,96%,89%,.75)}.focus\:caret-amber-100\/80:focus{caret-color:hsla(48,96%,89%,.8)}.focus\:caret-amber-100\/85:focus{caret-color:hsla(48,96%,89%,.85)}.focus\:caret-amber-100\/90:focus{caret-color:hsla(48,96%,89%,.9)}.focus\:caret-amber-100\/95:focus{caret-color:hsla(48,96%,89%,.95)}.focus\:caret-amber-200:focus{caret-color:#fde68a}.focus\:caret-amber-200\/0:focus{caret-color:hsla(48,97%,77%,0)}.focus\:caret-amber-200\/10:focus{caret-color:hsla(48,97%,77%,.1)}.focus\:caret-amber-200\/100:focus{caret-color:#fde68a}.focus\:caret-amber-200\/15:focus{caret-color:hsla(48,97%,77%,.15)}.focus\:caret-amber-200\/20:focus{caret-color:hsla(48,97%,77%,.2)}.focus\:caret-amber-200\/25:focus{caret-color:hsla(48,97%,77%,.25)}.focus\:caret-amber-200\/30:focus{caret-color:hsla(48,97%,77%,.3)}.focus\:caret-amber-200\/35:focus{caret-color:hsla(48,97%,77%,.35)}.focus\:caret-amber-200\/40:focus{caret-color:hsla(48,97%,77%,.4)}.focus\:caret-amber-200\/45:focus{caret-color:hsla(48,97%,77%,.45)}.focus\:caret-amber-200\/5:focus{caret-color:hsla(48,97%,77%,.05)}.focus\:caret-amber-200\/50:focus{caret-color:hsla(48,97%,77%,.5)}.focus\:caret-amber-200\/55:focus{caret-color:hsla(48,97%,77%,.55)}.focus\:caret-amber-200\/60:focus{caret-color:hsla(48,97%,77%,.6)}.focus\:caret-amber-200\/65:focus{caret-color:hsla(48,97%,77%,.65)}.focus\:caret-amber-200\/70:focus{caret-color:hsla(48,97%,77%,.7)}.focus\:caret-amber-200\/75:focus{caret-color:hsla(48,97%,77%,.75)}.focus\:caret-amber-200\/80:focus{caret-color:hsla(48,97%,77%,.8)}.focus\:caret-amber-200\/85:focus{caret-color:hsla(48,97%,77%,.85)}.focus\:caret-amber-200\/90:focus{caret-color:hsla(48,97%,77%,.9)}.focus\:caret-amber-200\/95:focus{caret-color:hsla(48,97%,77%,.95)}.focus\:caret-amber-400:focus{caret-color:#fbbf24}.focus\:caret-amber-400\/0:focus{caret-color:rgba(251,191,36,0)}.focus\:caret-amber-400\/10:focus{caret-color:rgba(251,191,36,.1)}.focus\:caret-amber-400\/100:focus{caret-color:#fbbf24}.focus\:caret-amber-400\/15:focus{caret-color:rgba(251,191,36,.15)}.focus\:caret-amber-400\/20:focus{caret-color:rgba(251,191,36,.2)}.focus\:caret-amber-400\/25:focus{caret-color:rgba(251,191,36,.25)}.focus\:caret-amber-400\/30:focus{caret-color:rgba(251,191,36,.3)}.focus\:caret-amber-400\/35:focus{caret-color:rgba(251,191,36,.35)}.focus\:caret-amber-400\/40:focus{caret-color:rgba(251,191,36,.4)}.focus\:caret-amber-400\/45:focus{caret-color:rgba(251,191,36,.45)}.focus\:caret-amber-400\/5:focus{caret-color:rgba(251,191,36,.05)}.focus\:caret-amber-400\/50:focus{caret-color:rgba(251,191,36,.5)}.focus\:caret-amber-400\/55:focus{caret-color:rgba(251,191,36,.55)}.focus\:caret-amber-400\/60:focus{caret-color:rgba(251,191,36,.6)}.focus\:caret-amber-400\/65:focus{caret-color:rgba(251,191,36,.65)}.focus\:caret-amber-400\/70:focus{caret-color:rgba(251,191,36,.7)}.focus\:caret-amber-400\/75:focus{caret-color:rgba(251,191,36,.75)}.focus\:caret-amber-400\/80:focus{caret-color:rgba(251,191,36,.8)}.focus\:caret-amber-400\/85:focus{caret-color:rgba(251,191,36,.85)}.focus\:caret-amber-400\/90:focus{caret-color:rgba(251,191,36,.9)}.focus\:caret-amber-400\/95:focus{caret-color:rgba(251,191,36,.95)}.focus\:caret-amber-50:focus{caret-color:#fffbeb}.focus\:caret-amber-50\/0:focus{caret-color:rgba(255,251,235,0)}.focus\:caret-amber-50\/10:focus{caret-color:rgba(255,251,235,.1)}.focus\:caret-amber-50\/100:focus{caret-color:#fffbeb}.focus\:caret-amber-50\/15:focus{caret-color:rgba(255,251,235,.15)}.focus\:caret-amber-50\/20:focus{caret-color:rgba(255,251,235,.2)}.focus\:caret-amber-50\/25:focus{caret-color:rgba(255,251,235,.25)}.focus\:caret-amber-50\/30:focus{caret-color:rgba(255,251,235,.3)}.focus\:caret-amber-50\/35:focus{caret-color:rgba(255,251,235,.35)}.focus\:caret-amber-50\/40:focus{caret-color:rgba(255,251,235,.4)}.focus\:caret-amber-50\/45:focus{caret-color:rgba(255,251,235,.45)}.focus\:caret-amber-50\/5:focus{caret-color:rgba(255,251,235,.05)}.focus\:caret-amber-50\/50:focus{caret-color:rgba(255,251,235,.5)}.focus\:caret-amber-50\/55:focus{caret-color:rgba(255,251,235,.55)}.focus\:caret-amber-50\/60:focus{caret-color:rgba(255,251,235,.6)}.focus\:caret-amber-50\/65:focus{caret-color:rgba(255,251,235,.65)}.focus\:caret-amber-50\/70:focus{caret-color:rgba(255,251,235,.7)}.focus\:caret-amber-50\/75:focus{caret-color:rgba(255,251,235,.75)}.focus\:caret-amber-50\/80:focus{caret-color:rgba(255,251,235,.8)}.focus\:caret-amber-50\/85:focus{caret-color:rgba(255,251,235,.85)}.focus\:caret-amber-50\/90:focus{caret-color:rgba(255,251,235,.9)}.focus\:caret-amber-50\/95:focus{caret-color:rgba(255,251,235,.95)}.focus\:caret-amber-500:focus{caret-color:#f59e0b}.focus\:caret-amber-500\/0:focus{caret-color:rgba(245,158,11,0)}.focus\:caret-amber-500\/10:focus{caret-color:rgba(245,158,11,.1)}.focus\:caret-amber-500\/100:focus{caret-color:#f59e0b}.focus\:caret-amber-500\/15:focus{caret-color:rgba(245,158,11,.15)}.focus\:caret-amber-500\/20:focus{caret-color:rgba(245,158,11,.2)}.focus\:caret-amber-500\/25:focus{caret-color:rgba(245,158,11,.25)}.focus\:caret-amber-500\/30:focus{caret-color:rgba(245,158,11,.3)}.focus\:caret-amber-500\/35:focus{caret-color:rgba(245,158,11,.35)}.focus\:caret-amber-500\/40:focus{caret-color:rgba(245,158,11,.4)}.focus\:caret-amber-500\/45:focus{caret-color:rgba(245,158,11,.45)}.focus\:caret-amber-500\/5:focus{caret-color:rgba(245,158,11,.05)}.focus\:caret-amber-500\/50:focus{caret-color:rgba(245,158,11,.5)}.focus\:caret-amber-500\/55:focus{caret-color:rgba(245,158,11,.55)}.focus\:caret-amber-500\/60:focus{caret-color:rgba(245,158,11,.6)}.focus\:caret-amber-500\/65:focus{caret-color:rgba(245,158,11,.65)}.focus\:caret-amber-500\/70:focus{caret-color:rgba(245,158,11,.7)}.focus\:caret-amber-500\/75:focus{caret-color:rgba(245,158,11,.75)}.focus\:caret-amber-500\/80:focus{caret-color:rgba(245,158,11,.8)}.focus\:caret-amber-500\/85:focus{caret-color:rgba(245,158,11,.85)}.focus\:caret-amber-500\/90:focus{caret-color:rgba(245,158,11,.9)}.focus\:caret-amber-500\/95:focus{caret-color:rgba(245,158,11,.95)}.focus\:caret-amber-950:focus{caret-color:#451a03}.focus\:caret-amber-950\/0:focus{caret-color:rgba(69,26,3,0)}.focus\:caret-amber-950\/10:focus{caret-color:rgba(69,26,3,.1)}.focus\:caret-amber-950\/100:focus{caret-color:#451a03}.focus\:caret-amber-950\/15:focus{caret-color:rgba(69,26,3,.15)}.focus\:caret-amber-950\/20:focus{caret-color:rgba(69,26,3,.2)}.focus\:caret-amber-950\/25:focus{caret-color:rgba(69,26,3,.25)}.focus\:caret-amber-950\/30:focus{caret-color:rgba(69,26,3,.3)}.focus\:caret-amber-950\/35:focus{caret-color:rgba(69,26,3,.35)}.focus\:caret-amber-950\/40:focus{caret-color:rgba(69,26,3,.4)}.focus\:caret-amber-950\/45:focus{caret-color:rgba(69,26,3,.45)}.focus\:caret-amber-950\/5:focus{caret-color:rgba(69,26,3,.05)}.focus\:caret-amber-950\/50:focus{caret-color:rgba(69,26,3,.5)}.focus\:caret-amber-950\/55:focus{caret-color:rgba(69,26,3,.55)}.focus\:caret-amber-950\/60:focus{caret-color:rgba(69,26,3,.6)}.focus\:caret-amber-950\/65:focus{caret-color:rgba(69,26,3,.65)}.focus\:caret-amber-950\/70:focus{caret-color:rgba(69,26,3,.7)}.focus\:caret-amber-950\/75:focus{caret-color:rgba(69,26,3,.75)}.focus\:caret-amber-950\/80:focus{caret-color:rgba(69,26,3,.8)}.focus\:caret-amber-950\/85:focus{caret-color:rgba(69,26,3,.85)}.focus\:caret-amber-950\/90:focus{caret-color:rgba(69,26,3,.9)}.focus\:caret-amber-950\/95:focus{caret-color:rgba(69,26,3,.95)}.focus\:caret-blue-100:focus{caret-color:#dbeafe}.focus\:caret-blue-100\/0:focus{caret-color:rgba(219,234,254,0)}.focus\:caret-blue-100\/10:focus{caret-color:rgba(219,234,254,.1)}.focus\:caret-blue-100\/100:focus{caret-color:#dbeafe}.focus\:caret-blue-100\/15:focus{caret-color:rgba(219,234,254,.15)}.focus\:caret-blue-100\/20:focus{caret-color:rgba(219,234,254,.2)}.focus\:caret-blue-100\/25:focus{caret-color:rgba(219,234,254,.25)}.focus\:caret-blue-100\/30:focus{caret-color:rgba(219,234,254,.3)}.focus\:caret-blue-100\/35:focus{caret-color:rgba(219,234,254,.35)}.focus\:caret-blue-100\/40:focus{caret-color:rgba(219,234,254,.4)}.focus\:caret-blue-100\/45:focus{caret-color:rgba(219,234,254,.45)}.focus\:caret-blue-100\/5:focus{caret-color:rgba(219,234,254,.05)}.focus\:caret-blue-100\/50:focus{caret-color:rgba(219,234,254,.5)}.focus\:caret-blue-100\/55:focus{caret-color:rgba(219,234,254,.55)}.focus\:caret-blue-100\/60:focus{caret-color:rgba(219,234,254,.6)}.focus\:caret-blue-100\/65:focus{caret-color:rgba(219,234,254,.65)}.focus\:caret-blue-100\/70:focus{caret-color:rgba(219,234,254,.7)}.focus\:caret-blue-100\/75:focus{caret-color:rgba(219,234,254,.75)}.focus\:caret-blue-100\/80:focus{caret-color:rgba(219,234,254,.8)}.focus\:caret-blue-100\/85:focus{caret-color:rgba(219,234,254,.85)}.focus\:caret-blue-100\/90:focus{caret-color:rgba(219,234,254,.9)}.focus\:caret-blue-100\/95:focus{caret-color:rgba(219,234,254,.95)}.focus\:caret-blue-200:focus{caret-color:#bfdbfe}.focus\:caret-blue-200\/0:focus{caret-color:rgba(191,219,254,0)}.focus\:caret-blue-200\/10:focus{caret-color:rgba(191,219,254,.1)}.focus\:caret-blue-200\/100:focus{caret-color:#bfdbfe}.focus\:caret-blue-200\/15:focus{caret-color:rgba(191,219,254,.15)}.focus\:caret-blue-200\/20:focus{caret-color:rgba(191,219,254,.2)}.focus\:caret-blue-200\/25:focus{caret-color:rgba(191,219,254,.25)}.focus\:caret-blue-200\/30:focus{caret-color:rgba(191,219,254,.3)}.focus\:caret-blue-200\/35:focus{caret-color:rgba(191,219,254,.35)}.focus\:caret-blue-200\/40:focus{caret-color:rgba(191,219,254,.4)}.focus\:caret-blue-200\/45:focus{caret-color:rgba(191,219,254,.45)}.focus\:caret-blue-200\/5:focus{caret-color:rgba(191,219,254,.05)}.focus\:caret-blue-200\/50:focus{caret-color:rgba(191,219,254,.5)}.focus\:caret-blue-200\/55:focus{caret-color:rgba(191,219,254,.55)}.focus\:caret-blue-200\/60:focus{caret-color:rgba(191,219,254,.6)}.focus\:caret-blue-200\/65:focus{caret-color:rgba(191,219,254,.65)}.focus\:caret-blue-200\/70:focus{caret-color:rgba(191,219,254,.7)}.focus\:caret-blue-200\/75:focus{caret-color:rgba(191,219,254,.75)}.focus\:caret-blue-200\/80:focus{caret-color:rgba(191,219,254,.8)}.focus\:caret-blue-200\/85:focus{caret-color:rgba(191,219,254,.85)}.focus\:caret-blue-200\/90:focus{caret-color:rgba(191,219,254,.9)}.focus\:caret-blue-200\/95:focus{caret-color:rgba(191,219,254,.95)}.focus\:caret-blue-400:focus{caret-color:#60a5fa}.focus\:caret-blue-400\/0:focus{caret-color:rgba(96,165,250,0)}.focus\:caret-blue-400\/10:focus{caret-color:rgba(96,165,250,.1)}.focus\:caret-blue-400\/100:focus{caret-color:#60a5fa}.focus\:caret-blue-400\/15:focus{caret-color:rgba(96,165,250,.15)}.focus\:caret-blue-400\/20:focus{caret-color:rgba(96,165,250,.2)}.focus\:caret-blue-400\/25:focus{caret-color:rgba(96,165,250,.25)}.focus\:caret-blue-400\/30:focus{caret-color:rgba(96,165,250,.3)}.focus\:caret-blue-400\/35:focus{caret-color:rgba(96,165,250,.35)}.focus\:caret-blue-400\/40:focus{caret-color:rgba(96,165,250,.4)}.focus\:caret-blue-400\/45:focus{caret-color:rgba(96,165,250,.45)}.focus\:caret-blue-400\/5:focus{caret-color:rgba(96,165,250,.05)}.focus\:caret-blue-400\/50:focus{caret-color:rgba(96,165,250,.5)}.focus\:caret-blue-400\/55:focus{caret-color:rgba(96,165,250,.55)}.focus\:caret-blue-400\/60:focus{caret-color:rgba(96,165,250,.6)}.focus\:caret-blue-400\/65:focus{caret-color:rgba(96,165,250,.65)}.focus\:caret-blue-400\/70:focus{caret-color:rgba(96,165,250,.7)}.focus\:caret-blue-400\/75:focus{caret-color:rgba(96,165,250,.75)}.focus\:caret-blue-400\/80:focus{caret-color:rgba(96,165,250,.8)}.focus\:caret-blue-400\/85:focus{caret-color:rgba(96,165,250,.85)}.focus\:caret-blue-400\/90:focus{caret-color:rgba(96,165,250,.9)}.focus\:caret-blue-400\/95:focus{caret-color:rgba(96,165,250,.95)}.focus\:caret-blue-50:focus{caret-color:#eff6ff}.focus\:caret-blue-50\/0:focus{caret-color:rgba(239,246,255,0)}.focus\:caret-blue-50\/10:focus{caret-color:rgba(239,246,255,.1)}.focus\:caret-blue-50\/100:focus{caret-color:#eff6ff}.focus\:caret-blue-50\/15:focus{caret-color:rgba(239,246,255,.15)}.focus\:caret-blue-50\/20:focus{caret-color:rgba(239,246,255,.2)}.focus\:caret-blue-50\/25:focus{caret-color:rgba(239,246,255,.25)}.focus\:caret-blue-50\/30:focus{caret-color:rgba(239,246,255,.3)}.focus\:caret-blue-50\/35:focus{caret-color:rgba(239,246,255,.35)}.focus\:caret-blue-50\/40:focus{caret-color:rgba(239,246,255,.4)}.focus\:caret-blue-50\/45:focus{caret-color:rgba(239,246,255,.45)}.focus\:caret-blue-50\/5:focus{caret-color:rgba(239,246,255,.05)}.focus\:caret-blue-50\/50:focus{caret-color:rgba(239,246,255,.5)}.focus\:caret-blue-50\/55:focus{caret-color:rgba(239,246,255,.55)}.focus\:caret-blue-50\/60:focus{caret-color:rgba(239,246,255,.6)}.focus\:caret-blue-50\/65:focus{caret-color:rgba(239,246,255,.65)}.focus\:caret-blue-50\/70:focus{caret-color:rgba(239,246,255,.7)}.focus\:caret-blue-50\/75:focus{caret-color:rgba(239,246,255,.75)}.focus\:caret-blue-50\/80:focus{caret-color:rgba(239,246,255,.8)}.focus\:caret-blue-50\/85:focus{caret-color:rgba(239,246,255,.85)}.focus\:caret-blue-50\/90:focus{caret-color:rgba(239,246,255,.9)}.focus\:caret-blue-50\/95:focus{caret-color:rgba(239,246,255,.95)}.focus\:caret-blue-500:focus{caret-color:#3b82f6}.focus\:caret-blue-500\/0:focus{caret-color:rgba(59,130,246,0)}.focus\:caret-blue-500\/10:focus{caret-color:rgba(59,130,246,.1)}.focus\:caret-blue-500\/100:focus{caret-color:#3b82f6}.focus\:caret-blue-500\/15:focus{caret-color:rgba(59,130,246,.15)}.focus\:caret-blue-500\/20:focus{caret-color:rgba(59,130,246,.2)}.focus\:caret-blue-500\/25:focus{caret-color:rgba(59,130,246,.25)}.focus\:caret-blue-500\/30:focus{caret-color:rgba(59,130,246,.3)}.focus\:caret-blue-500\/35:focus{caret-color:rgba(59,130,246,.35)}.focus\:caret-blue-500\/40:focus{caret-color:rgba(59,130,246,.4)}.focus\:caret-blue-500\/45:focus{caret-color:rgba(59,130,246,.45)}.focus\:caret-blue-500\/5:focus{caret-color:rgba(59,130,246,.05)}.focus\:caret-blue-500\/50:focus{caret-color:rgba(59,130,246,.5)}.focus\:caret-blue-500\/55:focus{caret-color:rgba(59,130,246,.55)}.focus\:caret-blue-500\/60:focus{caret-color:rgba(59,130,246,.6)}.focus\:caret-blue-500\/65:focus{caret-color:rgba(59,130,246,.65)}.focus\:caret-blue-500\/70:focus{caret-color:rgba(59,130,246,.7)}.focus\:caret-blue-500\/75:focus{caret-color:rgba(59,130,246,.75)}.focus\:caret-blue-500\/80:focus{caret-color:rgba(59,130,246,.8)}.focus\:caret-blue-500\/85:focus{caret-color:rgba(59,130,246,.85)}.focus\:caret-blue-500\/90:focus{caret-color:rgba(59,130,246,.9)}.focus\:caret-blue-500\/95:focus{caret-color:rgba(59,130,246,.95)}.focus\:caret-blue-950:focus{caret-color:#172554}.focus\:caret-blue-950\/0:focus{caret-color:rgba(23,37,84,0)}.focus\:caret-blue-950\/10:focus{caret-color:rgba(23,37,84,.1)}.focus\:caret-blue-950\/100:focus{caret-color:#172554}.focus\:caret-blue-950\/15:focus{caret-color:rgba(23,37,84,.15)}.focus\:caret-blue-950\/20:focus{caret-color:rgba(23,37,84,.2)}.focus\:caret-blue-950\/25:focus{caret-color:rgba(23,37,84,.25)}.focus\:caret-blue-950\/30:focus{caret-color:rgba(23,37,84,.3)}.focus\:caret-blue-950\/35:focus{caret-color:rgba(23,37,84,.35)}.focus\:caret-blue-950\/40:focus{caret-color:rgba(23,37,84,.4)}.focus\:caret-blue-950\/45:focus{caret-color:rgba(23,37,84,.45)}.focus\:caret-blue-950\/5:focus{caret-color:rgba(23,37,84,.05)}.focus\:caret-blue-950\/50:focus{caret-color:rgba(23,37,84,.5)}.focus\:caret-blue-950\/55:focus{caret-color:rgba(23,37,84,.55)}.focus\:caret-blue-950\/60:focus{caret-color:rgba(23,37,84,.6)}.focus\:caret-blue-950\/65:focus{caret-color:rgba(23,37,84,.65)}.focus\:caret-blue-950\/70:focus{caret-color:rgba(23,37,84,.7)}.focus\:caret-blue-950\/75:focus{caret-color:rgba(23,37,84,.75)}.focus\:caret-blue-950\/80:focus{caret-color:rgba(23,37,84,.8)}.focus\:caret-blue-950\/85:focus{caret-color:rgba(23,37,84,.85)}.focus\:caret-blue-950\/90:focus{caret-color:rgba(23,37,84,.9)}.focus\:caret-blue-950\/95:focus{caret-color:rgba(23,37,84,.95)}.focus\:caret-cyan-100:focus{caret-color:#cffafe}.focus\:caret-cyan-100\/0:focus{caret-color:rgba(207,250,254,0)}.focus\:caret-cyan-100\/10:focus{caret-color:rgba(207,250,254,.1)}.focus\:caret-cyan-100\/100:focus{caret-color:#cffafe}.focus\:caret-cyan-100\/15:focus{caret-color:rgba(207,250,254,.15)}.focus\:caret-cyan-100\/20:focus{caret-color:rgba(207,250,254,.2)}.focus\:caret-cyan-100\/25:focus{caret-color:rgba(207,250,254,.25)}.focus\:caret-cyan-100\/30:focus{caret-color:rgba(207,250,254,.3)}.focus\:caret-cyan-100\/35:focus{caret-color:rgba(207,250,254,.35)}.focus\:caret-cyan-100\/40:focus{caret-color:rgba(207,250,254,.4)}.focus\:caret-cyan-100\/45:focus{caret-color:rgba(207,250,254,.45)}.focus\:caret-cyan-100\/5:focus{caret-color:rgba(207,250,254,.05)}.focus\:caret-cyan-100\/50:focus{caret-color:rgba(207,250,254,.5)}.focus\:caret-cyan-100\/55:focus{caret-color:rgba(207,250,254,.55)}.focus\:caret-cyan-100\/60:focus{caret-color:rgba(207,250,254,.6)}.focus\:caret-cyan-100\/65:focus{caret-color:rgba(207,250,254,.65)}.focus\:caret-cyan-100\/70:focus{caret-color:rgba(207,250,254,.7)}.focus\:caret-cyan-100\/75:focus{caret-color:rgba(207,250,254,.75)}.focus\:caret-cyan-100\/80:focus{caret-color:rgba(207,250,254,.8)}.focus\:caret-cyan-100\/85:focus{caret-color:rgba(207,250,254,.85)}.focus\:caret-cyan-100\/90:focus{caret-color:rgba(207,250,254,.9)}.focus\:caret-cyan-100\/95:focus{caret-color:rgba(207,250,254,.95)}.focus\:caret-cyan-200:focus{caret-color:#a5f3fc}.focus\:caret-cyan-200\/0:focus{caret-color:rgba(165,243,252,0)}.focus\:caret-cyan-200\/10:focus{caret-color:rgba(165,243,252,.1)}.focus\:caret-cyan-200\/100:focus{caret-color:#a5f3fc}.focus\:caret-cyan-200\/15:focus{caret-color:rgba(165,243,252,.15)}.focus\:caret-cyan-200\/20:focus{caret-color:rgba(165,243,252,.2)}.focus\:caret-cyan-200\/25:focus{caret-color:rgba(165,243,252,.25)}.focus\:caret-cyan-200\/30:focus{caret-color:rgba(165,243,252,.3)}.focus\:caret-cyan-200\/35:focus{caret-color:rgba(165,243,252,.35)}.focus\:caret-cyan-200\/40:focus{caret-color:rgba(165,243,252,.4)}.focus\:caret-cyan-200\/45:focus{caret-color:rgba(165,243,252,.45)}.focus\:caret-cyan-200\/5:focus{caret-color:rgba(165,243,252,.05)}.focus\:caret-cyan-200\/50:focus{caret-color:rgba(165,243,252,.5)}.focus\:caret-cyan-200\/55:focus{caret-color:rgba(165,243,252,.55)}.focus\:caret-cyan-200\/60:focus{caret-color:rgba(165,243,252,.6)}.focus\:caret-cyan-200\/65:focus{caret-color:rgba(165,243,252,.65)}.focus\:caret-cyan-200\/70:focus{caret-color:rgba(165,243,252,.7)}.focus\:caret-cyan-200\/75:focus{caret-color:rgba(165,243,252,.75)}.focus\:caret-cyan-200\/80:focus{caret-color:rgba(165,243,252,.8)}.focus\:caret-cyan-200\/85:focus{caret-color:rgba(165,243,252,.85)}.focus\:caret-cyan-200\/90:focus{caret-color:rgba(165,243,252,.9)}.focus\:caret-cyan-200\/95:focus{caret-color:rgba(165,243,252,.95)}.focus\:caret-cyan-400:focus{caret-color:#22d3ee}.focus\:caret-cyan-400\/0:focus{caret-color:rgba(34,211,238,0)}.focus\:caret-cyan-400\/10:focus{caret-color:rgba(34,211,238,.1)}.focus\:caret-cyan-400\/100:focus{caret-color:#22d3ee}.focus\:caret-cyan-400\/15:focus{caret-color:rgba(34,211,238,.15)}.focus\:caret-cyan-400\/20:focus{caret-color:rgba(34,211,238,.2)}.focus\:caret-cyan-400\/25:focus{caret-color:rgba(34,211,238,.25)}.focus\:caret-cyan-400\/30:focus{caret-color:rgba(34,211,238,.3)}.focus\:caret-cyan-400\/35:focus{caret-color:rgba(34,211,238,.35)}.focus\:caret-cyan-400\/40:focus{caret-color:rgba(34,211,238,.4)}.focus\:caret-cyan-400\/45:focus{caret-color:rgba(34,211,238,.45)}.focus\:caret-cyan-400\/5:focus{caret-color:rgba(34,211,238,.05)}.focus\:caret-cyan-400\/50:focus{caret-color:rgba(34,211,238,.5)}.focus\:caret-cyan-400\/55:focus{caret-color:rgba(34,211,238,.55)}.focus\:caret-cyan-400\/60:focus{caret-color:rgba(34,211,238,.6)}.focus\:caret-cyan-400\/65:focus{caret-color:rgba(34,211,238,.65)}.focus\:caret-cyan-400\/70:focus{caret-color:rgba(34,211,238,.7)}.focus\:caret-cyan-400\/75:focus{caret-color:rgba(34,211,238,.75)}.focus\:caret-cyan-400\/80:focus{caret-color:rgba(34,211,238,.8)}.focus\:caret-cyan-400\/85:focus{caret-color:rgba(34,211,238,.85)}.focus\:caret-cyan-400\/90:focus{caret-color:rgba(34,211,238,.9)}.focus\:caret-cyan-400\/95:focus{caret-color:rgba(34,211,238,.95)}.focus\:caret-cyan-50:focus{caret-color:#ecfeff}.focus\:caret-cyan-50\/0:focus{caret-color:rgba(236,254,255,0)}.focus\:caret-cyan-50\/10:focus{caret-color:rgba(236,254,255,.1)}.focus\:caret-cyan-50\/100:focus{caret-color:#ecfeff}.focus\:caret-cyan-50\/15:focus{caret-color:rgba(236,254,255,.15)}.focus\:caret-cyan-50\/20:focus{caret-color:rgba(236,254,255,.2)}.focus\:caret-cyan-50\/25:focus{caret-color:rgba(236,254,255,.25)}.focus\:caret-cyan-50\/30:focus{caret-color:rgba(236,254,255,.3)}.focus\:caret-cyan-50\/35:focus{caret-color:rgba(236,254,255,.35)}.focus\:caret-cyan-50\/40:focus{caret-color:rgba(236,254,255,.4)}.focus\:caret-cyan-50\/45:focus{caret-color:rgba(236,254,255,.45)}.focus\:caret-cyan-50\/5:focus{caret-color:rgba(236,254,255,.05)}.focus\:caret-cyan-50\/50:focus{caret-color:rgba(236,254,255,.5)}.focus\:caret-cyan-50\/55:focus{caret-color:rgba(236,254,255,.55)}.focus\:caret-cyan-50\/60:focus{caret-color:rgba(236,254,255,.6)}.focus\:caret-cyan-50\/65:focus{caret-color:rgba(236,254,255,.65)}.focus\:caret-cyan-50\/70:focus{caret-color:rgba(236,254,255,.7)}.focus\:caret-cyan-50\/75:focus{caret-color:rgba(236,254,255,.75)}.focus\:caret-cyan-50\/80:focus{caret-color:rgba(236,254,255,.8)}.focus\:caret-cyan-50\/85:focus{caret-color:rgba(236,254,255,.85)}.focus\:caret-cyan-50\/90:focus{caret-color:rgba(236,254,255,.9)}.focus\:caret-cyan-50\/95:focus{caret-color:rgba(236,254,255,.95)}.focus\:caret-cyan-500:focus{caret-color:#06b6d4}.focus\:caret-cyan-500\/0:focus{caret-color:rgba(6,182,212,0)}.focus\:caret-cyan-500\/10:focus{caret-color:rgba(6,182,212,.1)}.focus\:caret-cyan-500\/100:focus{caret-color:#06b6d4}.focus\:caret-cyan-500\/15:focus{caret-color:rgba(6,182,212,.15)}.focus\:caret-cyan-500\/20:focus{caret-color:rgba(6,182,212,.2)}.focus\:caret-cyan-500\/25:focus{caret-color:rgba(6,182,212,.25)}.focus\:caret-cyan-500\/30:focus{caret-color:rgba(6,182,212,.3)}.focus\:caret-cyan-500\/35:focus{caret-color:rgba(6,182,212,.35)}.focus\:caret-cyan-500\/40:focus{caret-color:rgba(6,182,212,.4)}.focus\:caret-cyan-500\/45:focus{caret-color:rgba(6,182,212,.45)}.focus\:caret-cyan-500\/5:focus{caret-color:rgba(6,182,212,.05)}.focus\:caret-cyan-500\/50:focus{caret-color:rgba(6,182,212,.5)}.focus\:caret-cyan-500\/55:focus{caret-color:rgba(6,182,212,.55)}.focus\:caret-cyan-500\/60:focus{caret-color:rgba(6,182,212,.6)}.focus\:caret-cyan-500\/65:focus{caret-color:rgba(6,182,212,.65)}.focus\:caret-cyan-500\/70:focus{caret-color:rgba(6,182,212,.7)}.focus\:caret-cyan-500\/75:focus{caret-color:rgba(6,182,212,.75)}.focus\:caret-cyan-500\/80:focus{caret-color:rgba(6,182,212,.8)}.focus\:caret-cyan-500\/85:focus{caret-color:rgba(6,182,212,.85)}.focus\:caret-cyan-500\/90:focus{caret-color:rgba(6,182,212,.9)}.focus\:caret-cyan-500\/95:focus{caret-color:rgba(6,182,212,.95)}.focus\:caret-cyan-950:focus{caret-color:#083344}.focus\:caret-cyan-950\/0:focus{caret-color:rgba(8,51,68,0)}.focus\:caret-cyan-950\/10:focus{caret-color:rgba(8,51,68,.1)}.focus\:caret-cyan-950\/100:focus{caret-color:#083344}.focus\:caret-cyan-950\/15:focus{caret-color:rgba(8,51,68,.15)}.focus\:caret-cyan-950\/20:focus{caret-color:rgba(8,51,68,.2)}.focus\:caret-cyan-950\/25:focus{caret-color:rgba(8,51,68,.25)}.focus\:caret-cyan-950\/30:focus{caret-color:rgba(8,51,68,.3)}.focus\:caret-cyan-950\/35:focus{caret-color:rgba(8,51,68,.35)}.focus\:caret-cyan-950\/40:focus{caret-color:rgba(8,51,68,.4)}.focus\:caret-cyan-950\/45:focus{caret-color:rgba(8,51,68,.45)}.focus\:caret-cyan-950\/5:focus{caret-color:rgba(8,51,68,.05)}.focus\:caret-cyan-950\/50:focus{caret-color:rgba(8,51,68,.5)}.focus\:caret-cyan-950\/55:focus{caret-color:rgba(8,51,68,.55)}.focus\:caret-cyan-950\/60:focus{caret-color:rgba(8,51,68,.6)}.focus\:caret-cyan-950\/65:focus{caret-color:rgba(8,51,68,.65)}.focus\:caret-cyan-950\/70:focus{caret-color:rgba(8,51,68,.7)}.focus\:caret-cyan-950\/75:focus{caret-color:rgba(8,51,68,.75)}.focus\:caret-cyan-950\/80:focus{caret-color:rgba(8,51,68,.8)}.focus\:caret-cyan-950\/85:focus{caret-color:rgba(8,51,68,.85)}.focus\:caret-cyan-950\/90:focus{caret-color:rgba(8,51,68,.9)}.focus\:caret-cyan-950\/95:focus{caret-color:rgba(8,51,68,.95)}.focus\:caret-emerald-100:focus{caret-color:#d1fae5}.focus\:caret-emerald-100\/0:focus{caret-color:rgba(209,250,229,0)}.focus\:caret-emerald-100\/10:focus{caret-color:rgba(209,250,229,.1)}.focus\:caret-emerald-100\/100:focus{caret-color:#d1fae5}.focus\:caret-emerald-100\/15:focus{caret-color:rgba(209,250,229,.15)}.focus\:caret-emerald-100\/20:focus{caret-color:rgba(209,250,229,.2)}.focus\:caret-emerald-100\/25:focus{caret-color:rgba(209,250,229,.25)}.focus\:caret-emerald-100\/30:focus{caret-color:rgba(209,250,229,.3)}.focus\:caret-emerald-100\/35:focus{caret-color:rgba(209,250,229,.35)}.focus\:caret-emerald-100\/40:focus{caret-color:rgba(209,250,229,.4)}.focus\:caret-emerald-100\/45:focus{caret-color:rgba(209,250,229,.45)}.focus\:caret-emerald-100\/5:focus{caret-color:rgba(209,250,229,.05)}.focus\:caret-emerald-100\/50:focus{caret-color:rgba(209,250,229,.5)}.focus\:caret-emerald-100\/55:focus{caret-color:rgba(209,250,229,.55)}.focus\:caret-emerald-100\/60:focus{caret-color:rgba(209,250,229,.6)}.focus\:caret-emerald-100\/65:focus{caret-color:rgba(209,250,229,.65)}.focus\:caret-emerald-100\/70:focus{caret-color:rgba(209,250,229,.7)}.focus\:caret-emerald-100\/75:focus{caret-color:rgba(209,250,229,.75)}.focus\:caret-emerald-100\/80:focus{caret-color:rgba(209,250,229,.8)}.focus\:caret-emerald-100\/85:focus{caret-color:rgba(209,250,229,.85)}.focus\:caret-emerald-100\/90:focus{caret-color:rgba(209,250,229,.9)}.focus\:caret-emerald-100\/95:focus{caret-color:rgba(209,250,229,.95)}.focus\:caret-emerald-200:focus{caret-color:#a7f3d0}.focus\:caret-emerald-200\/0:focus{caret-color:rgba(167,243,208,0)}.focus\:caret-emerald-200\/10:focus{caret-color:rgba(167,243,208,.1)}.focus\:caret-emerald-200\/100:focus{caret-color:#a7f3d0}.focus\:caret-emerald-200\/15:focus{caret-color:rgba(167,243,208,.15)}.focus\:caret-emerald-200\/20:focus{caret-color:rgba(167,243,208,.2)}.focus\:caret-emerald-200\/25:focus{caret-color:rgba(167,243,208,.25)}.focus\:caret-emerald-200\/30:focus{caret-color:rgba(167,243,208,.3)}.focus\:caret-emerald-200\/35:focus{caret-color:rgba(167,243,208,.35)}.focus\:caret-emerald-200\/40:focus{caret-color:rgba(167,243,208,.4)}.focus\:caret-emerald-200\/45:focus{caret-color:rgba(167,243,208,.45)}.focus\:caret-emerald-200\/5:focus{caret-color:rgba(167,243,208,.05)}.focus\:caret-emerald-200\/50:focus{caret-color:rgba(167,243,208,.5)}.focus\:caret-emerald-200\/55:focus{caret-color:rgba(167,243,208,.55)}.focus\:caret-emerald-200\/60:focus{caret-color:rgba(167,243,208,.6)}.focus\:caret-emerald-200\/65:focus{caret-color:rgba(167,243,208,.65)}.focus\:caret-emerald-200\/70:focus{caret-color:rgba(167,243,208,.7)}.focus\:caret-emerald-200\/75:focus{caret-color:rgba(167,243,208,.75)}.focus\:caret-emerald-200\/80:focus{caret-color:rgba(167,243,208,.8)}.focus\:caret-emerald-200\/85:focus{caret-color:rgba(167,243,208,.85)}.focus\:caret-emerald-200\/90:focus{caret-color:rgba(167,243,208,.9)}.focus\:caret-emerald-200\/95:focus{caret-color:rgba(167,243,208,.95)}.focus\:caret-emerald-400:focus{caret-color:#34d399}.focus\:caret-emerald-400\/0:focus{caret-color:rgba(52,211,153,0)}.focus\:caret-emerald-400\/10:focus{caret-color:rgba(52,211,153,.1)}.focus\:caret-emerald-400\/100:focus{caret-color:#34d399}.focus\:caret-emerald-400\/15:focus{caret-color:rgba(52,211,153,.15)}.focus\:caret-emerald-400\/20:focus{caret-color:rgba(52,211,153,.2)}.focus\:caret-emerald-400\/25:focus{caret-color:rgba(52,211,153,.25)}.focus\:caret-emerald-400\/30:focus{caret-color:rgba(52,211,153,.3)}.focus\:caret-emerald-400\/35:focus{caret-color:rgba(52,211,153,.35)}.focus\:caret-emerald-400\/40:focus{caret-color:rgba(52,211,153,.4)}.focus\:caret-emerald-400\/45:focus{caret-color:rgba(52,211,153,.45)}.focus\:caret-emerald-400\/5:focus{caret-color:rgba(52,211,153,.05)}.focus\:caret-emerald-400\/50:focus{caret-color:rgba(52,211,153,.5)}.focus\:caret-emerald-400\/55:focus{caret-color:rgba(52,211,153,.55)}.focus\:caret-emerald-400\/60:focus{caret-color:rgba(52,211,153,.6)}.focus\:caret-emerald-400\/65:focus{caret-color:rgba(52,211,153,.65)}.focus\:caret-emerald-400\/70:focus{caret-color:rgba(52,211,153,.7)}.focus\:caret-emerald-400\/75:focus{caret-color:rgba(52,211,153,.75)}.focus\:caret-emerald-400\/80:focus{caret-color:rgba(52,211,153,.8)}.focus\:caret-emerald-400\/85:focus{caret-color:rgba(52,211,153,.85)}.focus\:caret-emerald-400\/90:focus{caret-color:rgba(52,211,153,.9)}.focus\:caret-emerald-400\/95:focus{caret-color:rgba(52,211,153,.95)}.focus\:caret-emerald-50:focus{caret-color:#ecfdf5}.focus\:caret-emerald-50\/0:focus{caret-color:rgba(236,253,245,0)}.focus\:caret-emerald-50\/10:focus{caret-color:rgba(236,253,245,.1)}.focus\:caret-emerald-50\/100:focus{caret-color:#ecfdf5}.focus\:caret-emerald-50\/15:focus{caret-color:rgba(236,253,245,.15)}.focus\:caret-emerald-50\/20:focus{caret-color:rgba(236,253,245,.2)}.focus\:caret-emerald-50\/25:focus{caret-color:rgba(236,253,245,.25)}.focus\:caret-emerald-50\/30:focus{caret-color:rgba(236,253,245,.3)}.focus\:caret-emerald-50\/35:focus{caret-color:rgba(236,253,245,.35)}.focus\:caret-emerald-50\/40:focus{caret-color:rgba(236,253,245,.4)}.focus\:caret-emerald-50\/45:focus{caret-color:rgba(236,253,245,.45)}.focus\:caret-emerald-50\/5:focus{caret-color:rgba(236,253,245,.05)}.focus\:caret-emerald-50\/50:focus{caret-color:rgba(236,253,245,.5)}.focus\:caret-emerald-50\/55:focus{caret-color:rgba(236,253,245,.55)}.focus\:caret-emerald-50\/60:focus{caret-color:rgba(236,253,245,.6)}.focus\:caret-emerald-50\/65:focus{caret-color:rgba(236,253,245,.65)}.focus\:caret-emerald-50\/70:focus{caret-color:rgba(236,253,245,.7)}.focus\:caret-emerald-50\/75:focus{caret-color:rgba(236,253,245,.75)}.focus\:caret-emerald-50\/80:focus{caret-color:rgba(236,253,245,.8)}.focus\:caret-emerald-50\/85:focus{caret-color:rgba(236,253,245,.85)}.focus\:caret-emerald-50\/90:focus{caret-color:rgba(236,253,245,.9)}.focus\:caret-emerald-50\/95:focus{caret-color:rgba(236,253,245,.95)}.focus\:caret-emerald-500:focus{caret-color:#10b981}.focus\:caret-emerald-500\/0:focus{caret-color:rgba(16,185,129,0)}.focus\:caret-emerald-500\/10:focus{caret-color:rgba(16,185,129,.1)}.focus\:caret-emerald-500\/100:focus{caret-color:#10b981}.focus\:caret-emerald-500\/15:focus{caret-color:rgba(16,185,129,.15)}.focus\:caret-emerald-500\/20:focus{caret-color:rgba(16,185,129,.2)}.focus\:caret-emerald-500\/25:focus{caret-color:rgba(16,185,129,.25)}.focus\:caret-emerald-500\/30:focus{caret-color:rgba(16,185,129,.3)}.focus\:caret-emerald-500\/35:focus{caret-color:rgba(16,185,129,.35)}.focus\:caret-emerald-500\/40:focus{caret-color:rgba(16,185,129,.4)}.focus\:caret-emerald-500\/45:focus{caret-color:rgba(16,185,129,.45)}.focus\:caret-emerald-500\/5:focus{caret-color:rgba(16,185,129,.05)}.focus\:caret-emerald-500\/50:focus{caret-color:rgba(16,185,129,.5)}.focus\:caret-emerald-500\/55:focus{caret-color:rgba(16,185,129,.55)}.focus\:caret-emerald-500\/60:focus{caret-color:rgba(16,185,129,.6)}.focus\:caret-emerald-500\/65:focus{caret-color:rgba(16,185,129,.65)}.focus\:caret-emerald-500\/70:focus{caret-color:rgba(16,185,129,.7)}.focus\:caret-emerald-500\/75:focus{caret-color:rgba(16,185,129,.75)}.focus\:caret-emerald-500\/80:focus{caret-color:rgba(16,185,129,.8)}.focus\:caret-emerald-500\/85:focus{caret-color:rgba(16,185,129,.85)}.focus\:caret-emerald-500\/90:focus{caret-color:rgba(16,185,129,.9)}.focus\:caret-emerald-500\/95:focus{caret-color:rgba(16,185,129,.95)}.focus\:caret-emerald-950:focus{caret-color:#022c22}.focus\:caret-emerald-950\/0:focus{caret-color:rgba(2,44,34,0)}.focus\:caret-emerald-950\/10:focus{caret-color:rgba(2,44,34,.1)}.focus\:caret-emerald-950\/100:focus{caret-color:#022c22}.focus\:caret-emerald-950\/15:focus{caret-color:rgba(2,44,34,.15)}.focus\:caret-emerald-950\/20:focus{caret-color:rgba(2,44,34,.2)}.focus\:caret-emerald-950\/25:focus{caret-color:rgba(2,44,34,.25)}.focus\:caret-emerald-950\/30:focus{caret-color:rgba(2,44,34,.3)}.focus\:caret-emerald-950\/35:focus{caret-color:rgba(2,44,34,.35)}.focus\:caret-emerald-950\/40:focus{caret-color:rgba(2,44,34,.4)}.focus\:caret-emerald-950\/45:focus{caret-color:rgba(2,44,34,.45)}.focus\:caret-emerald-950\/5:focus{caret-color:rgba(2,44,34,.05)}.focus\:caret-emerald-950\/50:focus{caret-color:rgba(2,44,34,.5)}.focus\:caret-emerald-950\/55:focus{caret-color:rgba(2,44,34,.55)}.focus\:caret-emerald-950\/60:focus{caret-color:rgba(2,44,34,.6)}.focus\:caret-emerald-950\/65:focus{caret-color:rgba(2,44,34,.65)}.focus\:caret-emerald-950\/70:focus{caret-color:rgba(2,44,34,.7)}.focus\:caret-emerald-950\/75:focus{caret-color:rgba(2,44,34,.75)}.focus\:caret-emerald-950\/80:focus{caret-color:rgba(2,44,34,.8)}.focus\:caret-emerald-950\/85:focus{caret-color:rgba(2,44,34,.85)}.focus\:caret-emerald-950\/90:focus{caret-color:rgba(2,44,34,.9)}.focus\:caret-emerald-950\/95:focus{caret-color:rgba(2,44,34,.95)}.focus\:caret-fuchsia-100:focus{caret-color:#fae8ff}.focus\:caret-fuchsia-100\/0:focus{caret-color:rgba(250,232,255,0)}.focus\:caret-fuchsia-100\/10:focus{caret-color:rgba(250,232,255,.1)}.focus\:caret-fuchsia-100\/100:focus{caret-color:#fae8ff}.focus\:caret-fuchsia-100\/15:focus{caret-color:rgba(250,232,255,.15)}.focus\:caret-fuchsia-100\/20:focus{caret-color:rgba(250,232,255,.2)}.focus\:caret-fuchsia-100\/25:focus{caret-color:rgba(250,232,255,.25)}.focus\:caret-fuchsia-100\/30:focus{caret-color:rgba(250,232,255,.3)}.focus\:caret-fuchsia-100\/35:focus{caret-color:rgba(250,232,255,.35)}.focus\:caret-fuchsia-100\/40:focus{caret-color:rgba(250,232,255,.4)}.focus\:caret-fuchsia-100\/45:focus{caret-color:rgba(250,232,255,.45)}.focus\:caret-fuchsia-100\/5:focus{caret-color:rgba(250,232,255,.05)}.focus\:caret-fuchsia-100\/50:focus{caret-color:rgba(250,232,255,.5)}.focus\:caret-fuchsia-100\/55:focus{caret-color:rgba(250,232,255,.55)}.focus\:caret-fuchsia-100\/60:focus{caret-color:rgba(250,232,255,.6)}.focus\:caret-fuchsia-100\/65:focus{caret-color:rgba(250,232,255,.65)}.focus\:caret-fuchsia-100\/70:focus{caret-color:rgba(250,232,255,.7)}.focus\:caret-fuchsia-100\/75:focus{caret-color:rgba(250,232,255,.75)}.focus\:caret-fuchsia-100\/80:focus{caret-color:rgba(250,232,255,.8)}.focus\:caret-fuchsia-100\/85:focus{caret-color:rgba(250,232,255,.85)}.focus\:caret-fuchsia-100\/90:focus{caret-color:rgba(250,232,255,.9)}.focus\:caret-fuchsia-100\/95:focus{caret-color:rgba(250,232,255,.95)}.focus\:caret-fuchsia-200:focus{caret-color:#f5d0fe}.focus\:caret-fuchsia-200\/0:focus{caret-color:rgba(245,208,254,0)}.focus\:caret-fuchsia-200\/10:focus{caret-color:rgba(245,208,254,.1)}.focus\:caret-fuchsia-200\/100:focus{caret-color:#f5d0fe}.focus\:caret-fuchsia-200\/15:focus{caret-color:rgba(245,208,254,.15)}.focus\:caret-fuchsia-200\/20:focus{caret-color:rgba(245,208,254,.2)}.focus\:caret-fuchsia-200\/25:focus{caret-color:rgba(245,208,254,.25)}.focus\:caret-fuchsia-200\/30:focus{caret-color:rgba(245,208,254,.3)}.focus\:caret-fuchsia-200\/35:focus{caret-color:rgba(245,208,254,.35)}.focus\:caret-fuchsia-200\/40:focus{caret-color:rgba(245,208,254,.4)}.focus\:caret-fuchsia-200\/45:focus{caret-color:rgba(245,208,254,.45)}.focus\:caret-fuchsia-200\/5:focus{caret-color:rgba(245,208,254,.05)}.focus\:caret-fuchsia-200\/50:focus{caret-color:rgba(245,208,254,.5)}.focus\:caret-fuchsia-200\/55:focus{caret-color:rgba(245,208,254,.55)}.focus\:caret-fuchsia-200\/60:focus{caret-color:rgba(245,208,254,.6)}.focus\:caret-fuchsia-200\/65:focus{caret-color:rgba(245,208,254,.65)}.focus\:caret-fuchsia-200\/70:focus{caret-color:rgba(245,208,254,.7)}.focus\:caret-fuchsia-200\/75:focus{caret-color:rgba(245,208,254,.75)}.focus\:caret-fuchsia-200\/80:focus{caret-color:rgba(245,208,254,.8)}.focus\:caret-fuchsia-200\/85:focus{caret-color:rgba(245,208,254,.85)}.focus\:caret-fuchsia-200\/90:focus{caret-color:rgba(245,208,254,.9)}.focus\:caret-fuchsia-200\/95:focus{caret-color:rgba(245,208,254,.95)}.focus\:caret-fuchsia-400:focus{caret-color:#e879f9}.focus\:caret-fuchsia-400\/0:focus{caret-color:rgba(232,121,249,0)}.focus\:caret-fuchsia-400\/10:focus{caret-color:rgba(232,121,249,.1)}.focus\:caret-fuchsia-400\/100:focus{caret-color:#e879f9}.focus\:caret-fuchsia-400\/15:focus{caret-color:rgba(232,121,249,.15)}.focus\:caret-fuchsia-400\/20:focus{caret-color:rgba(232,121,249,.2)}.focus\:caret-fuchsia-400\/25:focus{caret-color:rgba(232,121,249,.25)}.focus\:caret-fuchsia-400\/30:focus{caret-color:rgba(232,121,249,.3)}.focus\:caret-fuchsia-400\/35:focus{caret-color:rgba(232,121,249,.35)}.focus\:caret-fuchsia-400\/40:focus{caret-color:rgba(232,121,249,.4)}.focus\:caret-fuchsia-400\/45:focus{caret-color:rgba(232,121,249,.45)}.focus\:caret-fuchsia-400\/5:focus{caret-color:rgba(232,121,249,.05)}.focus\:caret-fuchsia-400\/50:focus{caret-color:rgba(232,121,249,.5)}.focus\:caret-fuchsia-400\/55:focus{caret-color:rgba(232,121,249,.55)}.focus\:caret-fuchsia-400\/60:focus{caret-color:rgba(232,121,249,.6)}.focus\:caret-fuchsia-400\/65:focus{caret-color:rgba(232,121,249,.65)}.focus\:caret-fuchsia-400\/70:focus{caret-color:rgba(232,121,249,.7)}.focus\:caret-fuchsia-400\/75:focus{caret-color:rgba(232,121,249,.75)}.focus\:caret-fuchsia-400\/80:focus{caret-color:rgba(232,121,249,.8)}.focus\:caret-fuchsia-400\/85:focus{caret-color:rgba(232,121,249,.85)}.focus\:caret-fuchsia-400\/90:focus{caret-color:rgba(232,121,249,.9)}.focus\:caret-fuchsia-400\/95:focus{caret-color:rgba(232,121,249,.95)}.focus\:caret-fuchsia-50:focus{caret-color:#fdf4ff}.focus\:caret-fuchsia-50\/0:focus{caret-color:rgba(253,244,255,0)}.focus\:caret-fuchsia-50\/10:focus{caret-color:rgba(253,244,255,.1)}.focus\:caret-fuchsia-50\/100:focus{caret-color:#fdf4ff}.focus\:caret-fuchsia-50\/15:focus{caret-color:rgba(253,244,255,.15)}.focus\:caret-fuchsia-50\/20:focus{caret-color:rgba(253,244,255,.2)}.focus\:caret-fuchsia-50\/25:focus{caret-color:rgba(253,244,255,.25)}.focus\:caret-fuchsia-50\/30:focus{caret-color:rgba(253,244,255,.3)}.focus\:caret-fuchsia-50\/35:focus{caret-color:rgba(253,244,255,.35)}.focus\:caret-fuchsia-50\/40:focus{caret-color:rgba(253,244,255,.4)}.focus\:caret-fuchsia-50\/45:focus{caret-color:rgba(253,244,255,.45)}.focus\:caret-fuchsia-50\/5:focus{caret-color:rgba(253,244,255,.05)}.focus\:caret-fuchsia-50\/50:focus{caret-color:rgba(253,244,255,.5)}.focus\:caret-fuchsia-50\/55:focus{caret-color:rgba(253,244,255,.55)}.focus\:caret-fuchsia-50\/60:focus{caret-color:rgba(253,244,255,.6)}.focus\:caret-fuchsia-50\/65:focus{caret-color:rgba(253,244,255,.65)}.focus\:caret-fuchsia-50\/70:focus{caret-color:rgba(253,244,255,.7)}.focus\:caret-fuchsia-50\/75:focus{caret-color:rgba(253,244,255,.75)}.focus\:caret-fuchsia-50\/80:focus{caret-color:rgba(253,244,255,.8)}.focus\:caret-fuchsia-50\/85:focus{caret-color:rgba(253,244,255,.85)}.focus\:caret-fuchsia-50\/90:focus{caret-color:rgba(253,244,255,.9)}.focus\:caret-fuchsia-50\/95:focus{caret-color:rgba(253,244,255,.95)}.focus\:caret-fuchsia-500:focus{caret-color:#d946ef}.focus\:caret-fuchsia-500\/0:focus{caret-color:rgba(217,70,239,0)}.focus\:caret-fuchsia-500\/10:focus{caret-color:rgba(217,70,239,.1)}.focus\:caret-fuchsia-500\/100:focus{caret-color:#d946ef}.focus\:caret-fuchsia-500\/15:focus{caret-color:rgba(217,70,239,.15)}.focus\:caret-fuchsia-500\/20:focus{caret-color:rgba(217,70,239,.2)}.focus\:caret-fuchsia-500\/25:focus{caret-color:rgba(217,70,239,.25)}.focus\:caret-fuchsia-500\/30:focus{caret-color:rgba(217,70,239,.3)}.focus\:caret-fuchsia-500\/35:focus{caret-color:rgba(217,70,239,.35)}.focus\:caret-fuchsia-500\/40:focus{caret-color:rgba(217,70,239,.4)}.focus\:caret-fuchsia-500\/45:focus{caret-color:rgba(217,70,239,.45)}.focus\:caret-fuchsia-500\/5:focus{caret-color:rgba(217,70,239,.05)}.focus\:caret-fuchsia-500\/50:focus{caret-color:rgba(217,70,239,.5)}.focus\:caret-fuchsia-500\/55:focus{caret-color:rgba(217,70,239,.55)}.focus\:caret-fuchsia-500\/60:focus{caret-color:rgba(217,70,239,.6)}.focus\:caret-fuchsia-500\/65:focus{caret-color:rgba(217,70,239,.65)}.focus\:caret-fuchsia-500\/70:focus{caret-color:rgba(217,70,239,.7)}.focus\:caret-fuchsia-500\/75:focus{caret-color:rgba(217,70,239,.75)}.focus\:caret-fuchsia-500\/80:focus{caret-color:rgba(217,70,239,.8)}.focus\:caret-fuchsia-500\/85:focus{caret-color:rgba(217,70,239,.85)}.focus\:caret-fuchsia-500\/90:focus{caret-color:rgba(217,70,239,.9)}.focus\:caret-fuchsia-500\/95:focus{caret-color:rgba(217,70,239,.95)}.focus\:caret-fuchsia-950:focus{caret-color:#4a044e}.focus\:caret-fuchsia-950\/0:focus{caret-color:rgba(74,4,78,0)}.focus\:caret-fuchsia-950\/10:focus{caret-color:rgba(74,4,78,.1)}.focus\:caret-fuchsia-950\/100:focus{caret-color:#4a044e}.focus\:caret-fuchsia-950\/15:focus{caret-color:rgba(74,4,78,.15)}.focus\:caret-fuchsia-950\/20:focus{caret-color:rgba(74,4,78,.2)}.focus\:caret-fuchsia-950\/25:focus{caret-color:rgba(74,4,78,.25)}.focus\:caret-fuchsia-950\/30:focus{caret-color:rgba(74,4,78,.3)}.focus\:caret-fuchsia-950\/35:focus{caret-color:rgba(74,4,78,.35)}.focus\:caret-fuchsia-950\/40:focus{caret-color:rgba(74,4,78,.4)}.focus\:caret-fuchsia-950\/45:focus{caret-color:rgba(74,4,78,.45)}.focus\:caret-fuchsia-950\/5:focus{caret-color:rgba(74,4,78,.05)}.focus\:caret-fuchsia-950\/50:focus{caret-color:rgba(74,4,78,.5)}.focus\:caret-fuchsia-950\/55:focus{caret-color:rgba(74,4,78,.55)}.focus\:caret-fuchsia-950\/60:focus{caret-color:rgba(74,4,78,.6)}.focus\:caret-fuchsia-950\/65:focus{caret-color:rgba(74,4,78,.65)}.focus\:caret-fuchsia-950\/70:focus{caret-color:rgba(74,4,78,.7)}.focus\:caret-fuchsia-950\/75:focus{caret-color:rgba(74,4,78,.75)}.focus\:caret-fuchsia-950\/80:focus{caret-color:rgba(74,4,78,.8)}.focus\:caret-fuchsia-950\/85:focus{caret-color:rgba(74,4,78,.85)}.focus\:caret-fuchsia-950\/90:focus{caret-color:rgba(74,4,78,.9)}.focus\:caret-fuchsia-950\/95:focus{caret-color:rgba(74,4,78,.95)}.focus\:caret-gray-100:focus{caret-color:#f3f4f6}.focus\:caret-gray-100\/0:focus{caret-color:rgba(243,244,246,0)}.focus\:caret-gray-100\/10:focus{caret-color:rgba(243,244,246,.1)}.focus\:caret-gray-100\/100:focus{caret-color:#f3f4f6}.focus\:caret-gray-100\/15:focus{caret-color:rgba(243,244,246,.15)}.focus\:caret-gray-100\/20:focus{caret-color:rgba(243,244,246,.2)}.focus\:caret-gray-100\/25:focus{caret-color:rgba(243,244,246,.25)}.focus\:caret-gray-100\/30:focus{caret-color:rgba(243,244,246,.3)}.focus\:caret-gray-100\/35:focus{caret-color:rgba(243,244,246,.35)}.focus\:caret-gray-100\/40:focus{caret-color:rgba(243,244,246,.4)}.focus\:caret-gray-100\/45:focus{caret-color:rgba(243,244,246,.45)}.focus\:caret-gray-100\/5:focus{caret-color:rgba(243,244,246,.05)}.focus\:caret-gray-100\/50:focus{caret-color:rgba(243,244,246,.5)}.focus\:caret-gray-100\/55:focus{caret-color:rgba(243,244,246,.55)}.focus\:caret-gray-100\/60:focus{caret-color:rgba(243,244,246,.6)}.focus\:caret-gray-100\/65:focus{caret-color:rgba(243,244,246,.65)}.focus\:caret-gray-100\/70:focus{caret-color:rgba(243,244,246,.7)}.focus\:caret-gray-100\/75:focus{caret-color:rgba(243,244,246,.75)}.focus\:caret-gray-100\/80:focus{caret-color:rgba(243,244,246,.8)}.focus\:caret-gray-100\/85:focus{caret-color:rgba(243,244,246,.85)}.focus\:caret-gray-100\/90:focus{caret-color:rgba(243,244,246,.9)}.focus\:caret-gray-100\/95:focus{caret-color:rgba(243,244,246,.95)}.focus\:caret-gray-200:focus{caret-color:#e5e7eb}.focus\:caret-gray-200\/0:focus{caret-color:rgba(229,231,235,0)}.focus\:caret-gray-200\/10:focus{caret-color:rgba(229,231,235,.1)}.focus\:caret-gray-200\/100:focus{caret-color:#e5e7eb}.focus\:caret-gray-200\/15:focus{caret-color:rgba(229,231,235,.15)}.focus\:caret-gray-200\/20:focus{caret-color:rgba(229,231,235,.2)}.focus\:caret-gray-200\/25:focus{caret-color:rgba(229,231,235,.25)}.focus\:caret-gray-200\/30:focus{caret-color:rgba(229,231,235,.3)}.focus\:caret-gray-200\/35:focus{caret-color:rgba(229,231,235,.35)}.focus\:caret-gray-200\/40:focus{caret-color:rgba(229,231,235,.4)}.focus\:caret-gray-200\/45:focus{caret-color:rgba(229,231,235,.45)}.focus\:caret-gray-200\/5:focus{caret-color:rgba(229,231,235,.05)}.focus\:caret-gray-200\/50:focus{caret-color:rgba(229,231,235,.5)}.focus\:caret-gray-200\/55:focus{caret-color:rgba(229,231,235,.55)}.focus\:caret-gray-200\/60:focus{caret-color:rgba(229,231,235,.6)}.focus\:caret-gray-200\/65:focus{caret-color:rgba(229,231,235,.65)}.focus\:caret-gray-200\/70:focus{caret-color:rgba(229,231,235,.7)}.focus\:caret-gray-200\/75:focus{caret-color:rgba(229,231,235,.75)}.focus\:caret-gray-200\/80:focus{caret-color:rgba(229,231,235,.8)}.focus\:caret-gray-200\/85:focus{caret-color:rgba(229,231,235,.85)}.focus\:caret-gray-200\/90:focus{caret-color:rgba(229,231,235,.9)}.focus\:caret-gray-200\/95:focus{caret-color:rgba(229,231,235,.95)}.focus\:caret-gray-400:focus{caret-color:#9ca3af}.focus\:caret-gray-400\/0:focus{caret-color:rgba(156,163,175,0)}.focus\:caret-gray-400\/10:focus{caret-color:rgba(156,163,175,.1)}.focus\:caret-gray-400\/100:focus{caret-color:#9ca3af}.focus\:caret-gray-400\/15:focus{caret-color:rgba(156,163,175,.15)}.focus\:caret-gray-400\/20:focus{caret-color:rgba(156,163,175,.2)}.focus\:caret-gray-400\/25:focus{caret-color:rgba(156,163,175,.25)}.focus\:caret-gray-400\/30:focus{caret-color:rgba(156,163,175,.3)}.focus\:caret-gray-400\/35:focus{caret-color:rgba(156,163,175,.35)}.focus\:caret-gray-400\/40:focus{caret-color:rgba(156,163,175,.4)}.focus\:caret-gray-400\/45:focus{caret-color:rgba(156,163,175,.45)}.focus\:caret-gray-400\/5:focus{caret-color:rgba(156,163,175,.05)}.focus\:caret-gray-400\/50:focus{caret-color:rgba(156,163,175,.5)}.focus\:caret-gray-400\/55:focus{caret-color:rgba(156,163,175,.55)}.focus\:caret-gray-400\/60:focus{caret-color:rgba(156,163,175,.6)}.focus\:caret-gray-400\/65:focus{caret-color:rgba(156,163,175,.65)}.focus\:caret-gray-400\/70:focus{caret-color:rgba(156,163,175,.7)}.focus\:caret-gray-400\/75:focus{caret-color:rgba(156,163,175,.75)}.focus\:caret-gray-400\/80:focus{caret-color:rgba(156,163,175,.8)}.focus\:caret-gray-400\/85:focus{caret-color:rgba(156,163,175,.85)}.focus\:caret-gray-400\/90:focus{caret-color:rgba(156,163,175,.9)}.focus\:caret-gray-400\/95:focus{caret-color:rgba(156,163,175,.95)}.focus\:caret-gray-50:focus{caret-color:#f9fafb}.focus\:caret-gray-50\/0:focus{caret-color:rgba(249,250,251,0)}.focus\:caret-gray-50\/10:focus{caret-color:rgba(249,250,251,.1)}.focus\:caret-gray-50\/100:focus{caret-color:#f9fafb}.focus\:caret-gray-50\/15:focus{caret-color:rgba(249,250,251,.15)}.focus\:caret-gray-50\/20:focus{caret-color:rgba(249,250,251,.2)}.focus\:caret-gray-50\/25:focus{caret-color:rgba(249,250,251,.25)}.focus\:caret-gray-50\/30:focus{caret-color:rgba(249,250,251,.3)}.focus\:caret-gray-50\/35:focus{caret-color:rgba(249,250,251,.35)}.focus\:caret-gray-50\/40:focus{caret-color:rgba(249,250,251,.4)}.focus\:caret-gray-50\/45:focus{caret-color:rgba(249,250,251,.45)}.focus\:caret-gray-50\/5:focus{caret-color:rgba(249,250,251,.05)}.focus\:caret-gray-50\/50:focus{caret-color:rgba(249,250,251,.5)}.focus\:caret-gray-50\/55:focus{caret-color:rgba(249,250,251,.55)}.focus\:caret-gray-50\/60:focus{caret-color:rgba(249,250,251,.6)}.focus\:caret-gray-50\/65:focus{caret-color:rgba(249,250,251,.65)}.focus\:caret-gray-50\/70:focus{caret-color:rgba(249,250,251,.7)}.focus\:caret-gray-50\/75:focus{caret-color:rgba(249,250,251,.75)}.focus\:caret-gray-50\/80:focus{caret-color:rgba(249,250,251,.8)}.focus\:caret-gray-50\/85:focus{caret-color:rgba(249,250,251,.85)}.focus\:caret-gray-50\/90:focus{caret-color:rgba(249,250,251,.9)}.focus\:caret-gray-50\/95:focus{caret-color:rgba(249,250,251,.95)}.focus\:caret-gray-500:focus{caret-color:#6b7280}.focus\:caret-gray-500\/0:focus{caret-color:hsla(220,9%,46%,0)}.focus\:caret-gray-500\/10:focus{caret-color:hsla(220,9%,46%,.1)}.focus\:caret-gray-500\/100:focus{caret-color:#6b7280}.focus\:caret-gray-500\/15:focus{caret-color:hsla(220,9%,46%,.15)}.focus\:caret-gray-500\/20:focus{caret-color:hsla(220,9%,46%,.2)}.focus\:caret-gray-500\/25:focus{caret-color:hsla(220,9%,46%,.25)}.focus\:caret-gray-500\/30:focus{caret-color:hsla(220,9%,46%,.3)}.focus\:caret-gray-500\/35:focus{caret-color:hsla(220,9%,46%,.35)}.focus\:caret-gray-500\/40:focus{caret-color:hsla(220,9%,46%,.4)}.focus\:caret-gray-500\/45:focus{caret-color:hsla(220,9%,46%,.45)}.focus\:caret-gray-500\/5:focus{caret-color:hsla(220,9%,46%,.05)}.focus\:caret-gray-500\/50:focus{caret-color:hsla(220,9%,46%,.5)}.focus\:caret-gray-500\/55:focus{caret-color:hsla(220,9%,46%,.55)}.focus\:caret-gray-500\/60:focus{caret-color:hsla(220,9%,46%,.6)}.focus\:caret-gray-500\/65:focus{caret-color:hsla(220,9%,46%,.65)}.focus\:caret-gray-500\/70:focus{caret-color:hsla(220,9%,46%,.7)}.focus\:caret-gray-500\/75:focus{caret-color:hsla(220,9%,46%,.75)}.focus\:caret-gray-500\/80:focus{caret-color:hsla(220,9%,46%,.8)}.focus\:caret-gray-500\/85:focus{caret-color:hsla(220,9%,46%,.85)}.focus\:caret-gray-500\/90:focus{caret-color:hsla(220,9%,46%,.9)}.focus\:caret-gray-500\/95:focus{caret-color:hsla(220,9%,46%,.95)}.focus\:caret-gray-950:focus{caret-color:#030712}.focus\:caret-gray-950\/0:focus{caret-color:rgba(3,7,18,0)}.focus\:caret-gray-950\/10:focus{caret-color:rgba(3,7,18,.1)}.focus\:caret-gray-950\/100:focus{caret-color:#030712}.focus\:caret-gray-950\/15:focus{caret-color:rgba(3,7,18,.15)}.focus\:caret-gray-950\/20:focus{caret-color:rgba(3,7,18,.2)}.focus\:caret-gray-950\/25:focus{caret-color:rgba(3,7,18,.25)}.focus\:caret-gray-950\/30:focus{caret-color:rgba(3,7,18,.3)}.focus\:caret-gray-950\/35:focus{caret-color:rgba(3,7,18,.35)}.focus\:caret-gray-950\/40:focus{caret-color:rgba(3,7,18,.4)}.focus\:caret-gray-950\/45:focus{caret-color:rgba(3,7,18,.45)}.focus\:caret-gray-950\/5:focus{caret-color:rgba(3,7,18,.05)}.focus\:caret-gray-950\/50:focus{caret-color:rgba(3,7,18,.5)}.focus\:caret-gray-950\/55:focus{caret-color:rgba(3,7,18,.55)}.focus\:caret-gray-950\/60:focus{caret-color:rgba(3,7,18,.6)}.focus\:caret-gray-950\/65:focus{caret-color:rgba(3,7,18,.65)}.focus\:caret-gray-950\/70:focus{caret-color:rgba(3,7,18,.7)}.focus\:caret-gray-950\/75:focus{caret-color:rgba(3,7,18,.75)}.focus\:caret-gray-950\/80:focus{caret-color:rgba(3,7,18,.8)}.focus\:caret-gray-950\/85:focus{caret-color:rgba(3,7,18,.85)}.focus\:caret-gray-950\/90:focus{caret-color:rgba(3,7,18,.9)}.focus\:caret-gray-950\/95:focus{caret-color:rgba(3,7,18,.95)}.focus\:caret-green-100:focus{caret-color:#dcfce7}.focus\:caret-green-100\/0:focus{caret-color:rgba(220,252,231,0)}.focus\:caret-green-100\/10:focus{caret-color:rgba(220,252,231,.1)}.focus\:caret-green-100\/100:focus{caret-color:#dcfce7}.focus\:caret-green-100\/15:focus{caret-color:rgba(220,252,231,.15)}.focus\:caret-green-100\/20:focus{caret-color:rgba(220,252,231,.2)}.focus\:caret-green-100\/25:focus{caret-color:rgba(220,252,231,.25)}.focus\:caret-green-100\/30:focus{caret-color:rgba(220,252,231,.3)}.focus\:caret-green-100\/35:focus{caret-color:rgba(220,252,231,.35)}.focus\:caret-green-100\/40:focus{caret-color:rgba(220,252,231,.4)}.focus\:caret-green-100\/45:focus{caret-color:rgba(220,252,231,.45)}.focus\:caret-green-100\/5:focus{caret-color:rgba(220,252,231,.05)}.focus\:caret-green-100\/50:focus{caret-color:rgba(220,252,231,.5)}.focus\:caret-green-100\/55:focus{caret-color:rgba(220,252,231,.55)}.focus\:caret-green-100\/60:focus{caret-color:rgba(220,252,231,.6)}.focus\:caret-green-100\/65:focus{caret-color:rgba(220,252,231,.65)}.focus\:caret-green-100\/70:focus{caret-color:rgba(220,252,231,.7)}.focus\:caret-green-100\/75:focus{caret-color:rgba(220,252,231,.75)}.focus\:caret-green-100\/80:focus{caret-color:rgba(220,252,231,.8)}.focus\:caret-green-100\/85:focus{caret-color:rgba(220,252,231,.85)}.focus\:caret-green-100\/90:focus{caret-color:rgba(220,252,231,.9)}.focus\:caret-green-100\/95:focus{caret-color:rgba(220,252,231,.95)}.focus\:caret-green-200:focus{caret-color:#bbf7d0}.focus\:caret-green-200\/0:focus{caret-color:rgba(187,247,208,0)}.focus\:caret-green-200\/10:focus{caret-color:rgba(187,247,208,.1)}.focus\:caret-green-200\/100:focus{caret-color:#bbf7d0}.focus\:caret-green-200\/15:focus{caret-color:rgba(187,247,208,.15)}.focus\:caret-green-200\/20:focus{caret-color:rgba(187,247,208,.2)}.focus\:caret-green-200\/25:focus{caret-color:rgba(187,247,208,.25)}.focus\:caret-green-200\/30:focus{caret-color:rgba(187,247,208,.3)}.focus\:caret-green-200\/35:focus{caret-color:rgba(187,247,208,.35)}.focus\:caret-green-200\/40:focus{caret-color:rgba(187,247,208,.4)}.focus\:caret-green-200\/45:focus{caret-color:rgba(187,247,208,.45)}.focus\:caret-green-200\/5:focus{caret-color:rgba(187,247,208,.05)}.focus\:caret-green-200\/50:focus{caret-color:rgba(187,247,208,.5)}.focus\:caret-green-200\/55:focus{caret-color:rgba(187,247,208,.55)}.focus\:caret-green-200\/60:focus{caret-color:rgba(187,247,208,.6)}.focus\:caret-green-200\/65:focus{caret-color:rgba(187,247,208,.65)}.focus\:caret-green-200\/70:focus{caret-color:rgba(187,247,208,.7)}.focus\:caret-green-200\/75:focus{caret-color:rgba(187,247,208,.75)}.focus\:caret-green-200\/80:focus{caret-color:rgba(187,247,208,.8)}.focus\:caret-green-200\/85:focus{caret-color:rgba(187,247,208,.85)}.focus\:caret-green-200\/90:focus{caret-color:rgba(187,247,208,.9)}.focus\:caret-green-200\/95:focus{caret-color:rgba(187,247,208,.95)}.focus\:caret-green-400:focus{caret-color:#4ade80}.focus\:caret-green-400\/0:focus{caret-color:rgba(74,222,128,0)}.focus\:caret-green-400\/10:focus{caret-color:rgba(74,222,128,.1)}.focus\:caret-green-400\/100:focus{caret-color:#4ade80}.focus\:caret-green-400\/15:focus{caret-color:rgba(74,222,128,.15)}.focus\:caret-green-400\/20:focus{caret-color:rgba(74,222,128,.2)}.focus\:caret-green-400\/25:focus{caret-color:rgba(74,222,128,.25)}.focus\:caret-green-400\/30:focus{caret-color:rgba(74,222,128,.3)}.focus\:caret-green-400\/35:focus{caret-color:rgba(74,222,128,.35)}.focus\:caret-green-400\/40:focus{caret-color:rgba(74,222,128,.4)}.focus\:caret-green-400\/45:focus{caret-color:rgba(74,222,128,.45)}.focus\:caret-green-400\/5:focus{caret-color:rgba(74,222,128,.05)}.focus\:caret-green-400\/50:focus{caret-color:rgba(74,222,128,.5)}.focus\:caret-green-400\/55:focus{caret-color:rgba(74,222,128,.55)}.focus\:caret-green-400\/60:focus{caret-color:rgba(74,222,128,.6)}.focus\:caret-green-400\/65:focus{caret-color:rgba(74,222,128,.65)}.focus\:caret-green-400\/70:focus{caret-color:rgba(74,222,128,.7)}.focus\:caret-green-400\/75:focus{caret-color:rgba(74,222,128,.75)}.focus\:caret-green-400\/80:focus{caret-color:rgba(74,222,128,.8)}.focus\:caret-green-400\/85:focus{caret-color:rgba(74,222,128,.85)}.focus\:caret-green-400\/90:focus{caret-color:rgba(74,222,128,.9)}.focus\:caret-green-400\/95:focus{caret-color:rgba(74,222,128,.95)}.focus\:caret-green-50:focus{caret-color:#f0fdf4}.focus\:caret-green-50\/0:focus{caret-color:rgba(240,253,244,0)}.focus\:caret-green-50\/10:focus{caret-color:rgba(240,253,244,.1)}.focus\:caret-green-50\/100:focus{caret-color:#f0fdf4}.focus\:caret-green-50\/15:focus{caret-color:rgba(240,253,244,.15)}.focus\:caret-green-50\/20:focus{caret-color:rgba(240,253,244,.2)}.focus\:caret-green-50\/25:focus{caret-color:rgba(240,253,244,.25)}.focus\:caret-green-50\/30:focus{caret-color:rgba(240,253,244,.3)}.focus\:caret-green-50\/35:focus{caret-color:rgba(240,253,244,.35)}.focus\:caret-green-50\/40:focus{caret-color:rgba(240,253,244,.4)}.focus\:caret-green-50\/45:focus{caret-color:rgba(240,253,244,.45)}.focus\:caret-green-50\/5:focus{caret-color:rgba(240,253,244,.05)}.focus\:caret-green-50\/50:focus{caret-color:rgba(240,253,244,.5)}.focus\:caret-green-50\/55:focus{caret-color:rgba(240,253,244,.55)}.focus\:caret-green-50\/60:focus{caret-color:rgba(240,253,244,.6)}.focus\:caret-green-50\/65:focus{caret-color:rgba(240,253,244,.65)}.focus\:caret-green-50\/70:focus{caret-color:rgba(240,253,244,.7)}.focus\:caret-green-50\/75:focus{caret-color:rgba(240,253,244,.75)}.focus\:caret-green-50\/80:focus{caret-color:rgba(240,253,244,.8)}.focus\:caret-green-50\/85:focus{caret-color:rgba(240,253,244,.85)}.focus\:caret-green-50\/90:focus{caret-color:rgba(240,253,244,.9)}.focus\:caret-green-50\/95:focus{caret-color:rgba(240,253,244,.95)}.focus\:caret-green-500:focus{caret-color:#22c55e}.focus\:caret-green-500\/0:focus{caret-color:rgba(34,197,94,0)}.focus\:caret-green-500\/10:focus{caret-color:rgba(34,197,94,.1)}.focus\:caret-green-500\/100:focus{caret-color:#22c55e}.focus\:caret-green-500\/15:focus{caret-color:rgba(34,197,94,.15)}.focus\:caret-green-500\/20:focus{caret-color:rgba(34,197,94,.2)}.focus\:caret-green-500\/25:focus{caret-color:rgba(34,197,94,.25)}.focus\:caret-green-500\/30:focus{caret-color:rgba(34,197,94,.3)}.focus\:caret-green-500\/35:focus{caret-color:rgba(34,197,94,.35)}.focus\:caret-green-500\/40:focus{caret-color:rgba(34,197,94,.4)}.focus\:caret-green-500\/45:focus{caret-color:rgba(34,197,94,.45)}.focus\:caret-green-500\/5:focus{caret-color:rgba(34,197,94,.05)}.focus\:caret-green-500\/50:focus{caret-color:rgba(34,197,94,.5)}.focus\:caret-green-500\/55:focus{caret-color:rgba(34,197,94,.55)}.focus\:caret-green-500\/60:focus{caret-color:rgba(34,197,94,.6)}.focus\:caret-green-500\/65:focus{caret-color:rgba(34,197,94,.65)}.focus\:caret-green-500\/70:focus{caret-color:rgba(34,197,94,.7)}.focus\:caret-green-500\/75:focus{caret-color:rgba(34,197,94,.75)}.focus\:caret-green-500\/80:focus{caret-color:rgba(34,197,94,.8)}.focus\:caret-green-500\/85:focus{caret-color:rgba(34,197,94,.85)}.focus\:caret-green-500\/90:focus{caret-color:rgba(34,197,94,.9)}.focus\:caret-green-500\/95:focus{caret-color:rgba(34,197,94,.95)}.focus\:caret-green-950:focus{caret-color:#052e16}.focus\:caret-green-950\/0:focus{caret-color:rgba(5,46,22,0)}.focus\:caret-green-950\/10:focus{caret-color:rgba(5,46,22,.1)}.focus\:caret-green-950\/100:focus{caret-color:#052e16}.focus\:caret-green-950\/15:focus{caret-color:rgba(5,46,22,.15)}.focus\:caret-green-950\/20:focus{caret-color:rgba(5,46,22,.2)}.focus\:caret-green-950\/25:focus{caret-color:rgba(5,46,22,.25)}.focus\:caret-green-950\/30:focus{caret-color:rgba(5,46,22,.3)}.focus\:caret-green-950\/35:focus{caret-color:rgba(5,46,22,.35)}.focus\:caret-green-950\/40:focus{caret-color:rgba(5,46,22,.4)}.focus\:caret-green-950\/45:focus{caret-color:rgba(5,46,22,.45)}.focus\:caret-green-950\/5:focus{caret-color:rgba(5,46,22,.05)}.focus\:caret-green-950\/50:focus{caret-color:rgba(5,46,22,.5)}.focus\:caret-green-950\/55:focus{caret-color:rgba(5,46,22,.55)}.focus\:caret-green-950\/60:focus{caret-color:rgba(5,46,22,.6)}.focus\:caret-green-950\/65:focus{caret-color:rgba(5,46,22,.65)}.focus\:caret-green-950\/70:focus{caret-color:rgba(5,46,22,.7)}.focus\:caret-green-950\/75:focus{caret-color:rgba(5,46,22,.75)}.focus\:caret-green-950\/80:focus{caret-color:rgba(5,46,22,.8)}.focus\:caret-green-950\/85:focus{caret-color:rgba(5,46,22,.85)}.focus\:caret-green-950\/90:focus{caret-color:rgba(5,46,22,.9)}.focus\:caret-green-950\/95:focus{caret-color:rgba(5,46,22,.95)}.focus\:caret-indigo-100:focus{caret-color:#e0e7ff}.focus\:caret-indigo-100\/0:focus{caret-color:rgba(224,231,255,0)}.focus\:caret-indigo-100\/10:focus{caret-color:rgba(224,231,255,.1)}.focus\:caret-indigo-100\/100:focus{caret-color:#e0e7ff}.focus\:caret-indigo-100\/15:focus{caret-color:rgba(224,231,255,.15)}.focus\:caret-indigo-100\/20:focus{caret-color:rgba(224,231,255,.2)}.focus\:caret-indigo-100\/25:focus{caret-color:rgba(224,231,255,.25)}.focus\:caret-indigo-100\/30:focus{caret-color:rgba(224,231,255,.3)}.focus\:caret-indigo-100\/35:focus{caret-color:rgba(224,231,255,.35)}.focus\:caret-indigo-100\/40:focus{caret-color:rgba(224,231,255,.4)}.focus\:caret-indigo-100\/45:focus{caret-color:rgba(224,231,255,.45)}.focus\:caret-indigo-100\/5:focus{caret-color:rgba(224,231,255,.05)}.focus\:caret-indigo-100\/50:focus{caret-color:rgba(224,231,255,.5)}.focus\:caret-indigo-100\/55:focus{caret-color:rgba(224,231,255,.55)}.focus\:caret-indigo-100\/60:focus{caret-color:rgba(224,231,255,.6)}.focus\:caret-indigo-100\/65:focus{caret-color:rgba(224,231,255,.65)}.focus\:caret-indigo-100\/70:focus{caret-color:rgba(224,231,255,.7)}.focus\:caret-indigo-100\/75:focus{caret-color:rgba(224,231,255,.75)}.focus\:caret-indigo-100\/80:focus{caret-color:rgba(224,231,255,.8)}.focus\:caret-indigo-100\/85:focus{caret-color:rgba(224,231,255,.85)}.focus\:caret-indigo-100\/90:focus{caret-color:rgba(224,231,255,.9)}.focus\:caret-indigo-100\/95:focus{caret-color:rgba(224,231,255,.95)}.focus\:caret-indigo-200:focus{caret-color:#c7d2fe}.focus\:caret-indigo-200\/0:focus{caret-color:rgba(199,210,254,0)}.focus\:caret-indigo-200\/10:focus{caret-color:rgba(199,210,254,.1)}.focus\:caret-indigo-200\/100:focus{caret-color:#c7d2fe}.focus\:caret-indigo-200\/15:focus{caret-color:rgba(199,210,254,.15)}.focus\:caret-indigo-200\/20:focus{caret-color:rgba(199,210,254,.2)}.focus\:caret-indigo-200\/25:focus{caret-color:rgba(199,210,254,.25)}.focus\:caret-indigo-200\/30:focus{caret-color:rgba(199,210,254,.3)}.focus\:caret-indigo-200\/35:focus{caret-color:rgba(199,210,254,.35)}.focus\:caret-indigo-200\/40:focus{caret-color:rgba(199,210,254,.4)}.focus\:caret-indigo-200\/45:focus{caret-color:rgba(199,210,254,.45)}.focus\:caret-indigo-200\/5:focus{caret-color:rgba(199,210,254,.05)}.focus\:caret-indigo-200\/50:focus{caret-color:rgba(199,210,254,.5)}.focus\:caret-indigo-200\/55:focus{caret-color:rgba(199,210,254,.55)}.focus\:caret-indigo-200\/60:focus{caret-color:rgba(199,210,254,.6)}.focus\:caret-indigo-200\/65:focus{caret-color:rgba(199,210,254,.65)}.focus\:caret-indigo-200\/70:focus{caret-color:rgba(199,210,254,.7)}.focus\:caret-indigo-200\/75:focus{caret-color:rgba(199,210,254,.75)}.focus\:caret-indigo-200\/80:focus{caret-color:rgba(199,210,254,.8)}.focus\:caret-indigo-200\/85:focus{caret-color:rgba(199,210,254,.85)}.focus\:caret-indigo-200\/90:focus{caret-color:rgba(199,210,254,.9)}.focus\:caret-indigo-200\/95:focus{caret-color:rgba(199,210,254,.95)}.focus\:caret-indigo-400:focus{caret-color:#818cf8}.focus\:caret-indigo-400\/0:focus{caret-color:rgba(129,140,248,0)}.focus\:caret-indigo-400\/10:focus{caret-color:rgba(129,140,248,.1)}.focus\:caret-indigo-400\/100:focus{caret-color:#818cf8}.focus\:caret-indigo-400\/15:focus{caret-color:rgba(129,140,248,.15)}.focus\:caret-indigo-400\/20:focus{caret-color:rgba(129,140,248,.2)}.focus\:caret-indigo-400\/25:focus{caret-color:rgba(129,140,248,.25)}.focus\:caret-indigo-400\/30:focus{caret-color:rgba(129,140,248,.3)}.focus\:caret-indigo-400\/35:focus{caret-color:rgba(129,140,248,.35)}.focus\:caret-indigo-400\/40:focus{caret-color:rgba(129,140,248,.4)}.focus\:caret-indigo-400\/45:focus{caret-color:rgba(129,140,248,.45)}.focus\:caret-indigo-400\/5:focus{caret-color:rgba(129,140,248,.05)}.focus\:caret-indigo-400\/50:focus{caret-color:rgba(129,140,248,.5)}.focus\:caret-indigo-400\/55:focus{caret-color:rgba(129,140,248,.55)}.focus\:caret-indigo-400\/60:focus{caret-color:rgba(129,140,248,.6)}.focus\:caret-indigo-400\/65:focus{caret-color:rgba(129,140,248,.65)}.focus\:caret-indigo-400\/70:focus{caret-color:rgba(129,140,248,.7)}.focus\:caret-indigo-400\/75:focus{caret-color:rgba(129,140,248,.75)}.focus\:caret-indigo-400\/80:focus{caret-color:rgba(129,140,248,.8)}.focus\:caret-indigo-400\/85:focus{caret-color:rgba(129,140,248,.85)}.focus\:caret-indigo-400\/90:focus{caret-color:rgba(129,140,248,.9)}.focus\:caret-indigo-400\/95:focus{caret-color:rgba(129,140,248,.95)}.focus\:caret-indigo-50:focus{caret-color:#eef2ff}.focus\:caret-indigo-50\/0:focus{caret-color:rgba(238,242,255,0)}.focus\:caret-indigo-50\/10:focus{caret-color:rgba(238,242,255,.1)}.focus\:caret-indigo-50\/100:focus{caret-color:#eef2ff}.focus\:caret-indigo-50\/15:focus{caret-color:rgba(238,242,255,.15)}.focus\:caret-indigo-50\/20:focus{caret-color:rgba(238,242,255,.2)}.focus\:caret-indigo-50\/25:focus{caret-color:rgba(238,242,255,.25)}.focus\:caret-indigo-50\/30:focus{caret-color:rgba(238,242,255,.3)}.focus\:caret-indigo-50\/35:focus{caret-color:rgba(238,242,255,.35)}.focus\:caret-indigo-50\/40:focus{caret-color:rgba(238,242,255,.4)}.focus\:caret-indigo-50\/45:focus{caret-color:rgba(238,242,255,.45)}.focus\:caret-indigo-50\/5:focus{caret-color:rgba(238,242,255,.05)}.focus\:caret-indigo-50\/50:focus{caret-color:rgba(238,242,255,.5)}.focus\:caret-indigo-50\/55:focus{caret-color:rgba(238,242,255,.55)}.focus\:caret-indigo-50\/60:focus{caret-color:rgba(238,242,255,.6)}.focus\:caret-indigo-50\/65:focus{caret-color:rgba(238,242,255,.65)}.focus\:caret-indigo-50\/70:focus{caret-color:rgba(238,242,255,.7)}.focus\:caret-indigo-50\/75:focus{caret-color:rgba(238,242,255,.75)}.focus\:caret-indigo-50\/80:focus{caret-color:rgba(238,242,255,.8)}.focus\:caret-indigo-50\/85:focus{caret-color:rgba(238,242,255,.85)}.focus\:caret-indigo-50\/90:focus{caret-color:rgba(238,242,255,.9)}.focus\:caret-indigo-50\/95:focus{caret-color:rgba(238,242,255,.95)}.focus\:caret-indigo-500:focus{caret-color:#6366f1}.focus\:caret-indigo-500\/0:focus{caret-color:rgba(99,102,241,0)}.focus\:caret-indigo-500\/10:focus{caret-color:rgba(99,102,241,.1)}.focus\:caret-indigo-500\/100:focus{caret-color:#6366f1}.focus\:caret-indigo-500\/15:focus{caret-color:rgba(99,102,241,.15)}.focus\:caret-indigo-500\/20:focus{caret-color:rgba(99,102,241,.2)}.focus\:caret-indigo-500\/25:focus{caret-color:rgba(99,102,241,.25)}.focus\:caret-indigo-500\/30:focus{caret-color:rgba(99,102,241,.3)}.focus\:caret-indigo-500\/35:focus{caret-color:rgba(99,102,241,.35)}.focus\:caret-indigo-500\/40:focus{caret-color:rgba(99,102,241,.4)}.focus\:caret-indigo-500\/45:focus{caret-color:rgba(99,102,241,.45)}.focus\:caret-indigo-500\/5:focus{caret-color:rgba(99,102,241,.05)}.focus\:caret-indigo-500\/50:focus{caret-color:rgba(99,102,241,.5)}.focus\:caret-indigo-500\/55:focus{caret-color:rgba(99,102,241,.55)}.focus\:caret-indigo-500\/60:focus{caret-color:rgba(99,102,241,.6)}.focus\:caret-indigo-500\/65:focus{caret-color:rgba(99,102,241,.65)}.focus\:caret-indigo-500\/70:focus{caret-color:rgba(99,102,241,.7)}.focus\:caret-indigo-500\/75:focus{caret-color:rgba(99,102,241,.75)}.focus\:caret-indigo-500\/80:focus{caret-color:rgba(99,102,241,.8)}.focus\:caret-indigo-500\/85:focus{caret-color:rgba(99,102,241,.85)}.focus\:caret-indigo-500\/90:focus{caret-color:rgba(99,102,241,.9)}.focus\:caret-indigo-500\/95:focus{caret-color:rgba(99,102,241,.95)}.focus\:caret-indigo-950:focus{caret-color:#1e1b4b}.focus\:caret-indigo-950\/0:focus{caret-color:rgba(30,27,75,0)}.focus\:caret-indigo-950\/10:focus{caret-color:rgba(30,27,75,.1)}.focus\:caret-indigo-950\/100:focus{caret-color:#1e1b4b}.focus\:caret-indigo-950\/15:focus{caret-color:rgba(30,27,75,.15)}.focus\:caret-indigo-950\/20:focus{caret-color:rgba(30,27,75,.2)}.focus\:caret-indigo-950\/25:focus{caret-color:rgba(30,27,75,.25)}.focus\:caret-indigo-950\/30:focus{caret-color:rgba(30,27,75,.3)}.focus\:caret-indigo-950\/35:focus{caret-color:rgba(30,27,75,.35)}.focus\:caret-indigo-950\/40:focus{caret-color:rgba(30,27,75,.4)}.focus\:caret-indigo-950\/45:focus{caret-color:rgba(30,27,75,.45)}.focus\:caret-indigo-950\/5:focus{caret-color:rgba(30,27,75,.05)}.focus\:caret-indigo-950\/50:focus{caret-color:rgba(30,27,75,.5)}.focus\:caret-indigo-950\/55:focus{caret-color:rgba(30,27,75,.55)}.focus\:caret-indigo-950\/60:focus{caret-color:rgba(30,27,75,.6)}.focus\:caret-indigo-950\/65:focus{caret-color:rgba(30,27,75,.65)}.focus\:caret-indigo-950\/70:focus{caret-color:rgba(30,27,75,.7)}.focus\:caret-indigo-950\/75:focus{caret-color:rgba(30,27,75,.75)}.focus\:caret-indigo-950\/80:focus{caret-color:rgba(30,27,75,.8)}.focus\:caret-indigo-950\/85:focus{caret-color:rgba(30,27,75,.85)}.focus\:caret-indigo-950\/90:focus{caret-color:rgba(30,27,75,.9)}.focus\:caret-indigo-950\/95:focus{caret-color:rgba(30,27,75,.95)}.focus\:caret-lime-100:focus{caret-color:#ecfccb}.focus\:caret-lime-100\/0:focus{caret-color:hsla(80,89%,89%,0)}.focus\:caret-lime-100\/10:focus{caret-color:hsla(80,89%,89%,.1)}.focus\:caret-lime-100\/100:focus{caret-color:#ecfccb}.focus\:caret-lime-100\/15:focus{caret-color:hsla(80,89%,89%,.15)}.focus\:caret-lime-100\/20:focus{caret-color:hsla(80,89%,89%,.2)}.focus\:caret-lime-100\/25:focus{caret-color:hsla(80,89%,89%,.25)}.focus\:caret-lime-100\/30:focus{caret-color:hsla(80,89%,89%,.3)}.focus\:caret-lime-100\/35:focus{caret-color:hsla(80,89%,89%,.35)}.focus\:caret-lime-100\/40:focus{caret-color:hsla(80,89%,89%,.4)}.focus\:caret-lime-100\/45:focus{caret-color:hsla(80,89%,89%,.45)}.focus\:caret-lime-100\/5:focus{caret-color:hsla(80,89%,89%,.05)}.focus\:caret-lime-100\/50:focus{caret-color:hsla(80,89%,89%,.5)}.focus\:caret-lime-100\/55:focus{caret-color:hsla(80,89%,89%,.55)}.focus\:caret-lime-100\/60:focus{caret-color:hsla(80,89%,89%,.6)}.focus\:caret-lime-100\/65:focus{caret-color:hsla(80,89%,89%,.65)}.focus\:caret-lime-100\/70:focus{caret-color:hsla(80,89%,89%,.7)}.focus\:caret-lime-100\/75:focus{caret-color:hsla(80,89%,89%,.75)}.focus\:caret-lime-100\/80:focus{caret-color:hsla(80,89%,89%,.8)}.focus\:caret-lime-100\/85:focus{caret-color:hsla(80,89%,89%,.85)}.focus\:caret-lime-100\/90:focus{caret-color:hsla(80,89%,89%,.9)}.focus\:caret-lime-100\/95:focus{caret-color:hsla(80,89%,89%,.95)}.focus\:caret-lime-200:focus{caret-color:#d9f99d}.focus\:caret-lime-200\/0:focus{caret-color:hsla(81,88%,80%,0)}.focus\:caret-lime-200\/10:focus{caret-color:hsla(81,88%,80%,.1)}.focus\:caret-lime-200\/100:focus{caret-color:#d9f99d}.focus\:caret-lime-200\/15:focus{caret-color:hsla(81,88%,80%,.15)}.focus\:caret-lime-200\/20:focus{caret-color:hsla(81,88%,80%,.2)}.focus\:caret-lime-200\/25:focus{caret-color:hsla(81,88%,80%,.25)}.focus\:caret-lime-200\/30:focus{caret-color:hsla(81,88%,80%,.3)}.focus\:caret-lime-200\/35:focus{caret-color:hsla(81,88%,80%,.35)}.focus\:caret-lime-200\/40:focus{caret-color:hsla(81,88%,80%,.4)}.focus\:caret-lime-200\/45:focus{caret-color:hsla(81,88%,80%,.45)}.focus\:caret-lime-200\/5:focus{caret-color:hsla(81,88%,80%,.05)}.focus\:caret-lime-200\/50:focus{caret-color:hsla(81,88%,80%,.5)}.focus\:caret-lime-200\/55:focus{caret-color:hsla(81,88%,80%,.55)}.focus\:caret-lime-200\/60:focus{caret-color:hsla(81,88%,80%,.6)}.focus\:caret-lime-200\/65:focus{caret-color:hsla(81,88%,80%,.65)}.focus\:caret-lime-200\/70:focus{caret-color:hsla(81,88%,80%,.7)}.focus\:caret-lime-200\/75:focus{caret-color:hsla(81,88%,80%,.75)}.focus\:caret-lime-200\/80:focus{caret-color:hsla(81,88%,80%,.8)}.focus\:caret-lime-200\/85:focus{caret-color:hsla(81,88%,80%,.85)}.focus\:caret-lime-200\/90:focus{caret-color:hsla(81,88%,80%,.9)}.focus\:caret-lime-200\/95:focus{caret-color:hsla(81,88%,80%,.95)}.focus\:caret-lime-400:focus{caret-color:#a3e635}.focus\:caret-lime-400\/0:focus{caret-color:rgba(163,230,53,0)}.focus\:caret-lime-400\/10:focus{caret-color:rgba(163,230,53,.1)}.focus\:caret-lime-400\/100:focus{caret-color:#a3e635}.focus\:caret-lime-400\/15:focus{caret-color:rgba(163,230,53,.15)}.focus\:caret-lime-400\/20:focus{caret-color:rgba(163,230,53,.2)}.focus\:caret-lime-400\/25:focus{caret-color:rgba(163,230,53,.25)}.focus\:caret-lime-400\/30:focus{caret-color:rgba(163,230,53,.3)}.focus\:caret-lime-400\/35:focus{caret-color:rgba(163,230,53,.35)}.focus\:caret-lime-400\/40:focus{caret-color:rgba(163,230,53,.4)}.focus\:caret-lime-400\/45:focus{caret-color:rgba(163,230,53,.45)}.focus\:caret-lime-400\/5:focus{caret-color:rgba(163,230,53,.05)}.focus\:caret-lime-400\/50:focus{caret-color:rgba(163,230,53,.5)}.focus\:caret-lime-400\/55:focus{caret-color:rgba(163,230,53,.55)}.focus\:caret-lime-400\/60:focus{caret-color:rgba(163,230,53,.6)}.focus\:caret-lime-400\/65:focus{caret-color:rgba(163,230,53,.65)}.focus\:caret-lime-400\/70:focus{caret-color:rgba(163,230,53,.7)}.focus\:caret-lime-400\/75:focus{caret-color:rgba(163,230,53,.75)}.focus\:caret-lime-400\/80:focus{caret-color:rgba(163,230,53,.8)}.focus\:caret-lime-400\/85:focus{caret-color:rgba(163,230,53,.85)}.focus\:caret-lime-400\/90:focus{caret-color:rgba(163,230,53,.9)}.focus\:caret-lime-400\/95:focus{caret-color:rgba(163,230,53,.95)}.focus\:caret-lime-50:focus{caret-color:#f7fee7}.focus\:caret-lime-50\/0:focus{caret-color:hsla(78,92%,95%,0)}.focus\:caret-lime-50\/10:focus{caret-color:hsla(78,92%,95%,.1)}.focus\:caret-lime-50\/100:focus{caret-color:#f7fee7}.focus\:caret-lime-50\/15:focus{caret-color:hsla(78,92%,95%,.15)}.focus\:caret-lime-50\/20:focus{caret-color:hsla(78,92%,95%,.2)}.focus\:caret-lime-50\/25:focus{caret-color:hsla(78,92%,95%,.25)}.focus\:caret-lime-50\/30:focus{caret-color:hsla(78,92%,95%,.3)}.focus\:caret-lime-50\/35:focus{caret-color:hsla(78,92%,95%,.35)}.focus\:caret-lime-50\/40:focus{caret-color:hsla(78,92%,95%,.4)}.focus\:caret-lime-50\/45:focus{caret-color:hsla(78,92%,95%,.45)}.focus\:caret-lime-50\/5:focus{caret-color:hsla(78,92%,95%,.05)}.focus\:caret-lime-50\/50:focus{caret-color:hsla(78,92%,95%,.5)}.focus\:caret-lime-50\/55:focus{caret-color:hsla(78,92%,95%,.55)}.focus\:caret-lime-50\/60:focus{caret-color:hsla(78,92%,95%,.6)}.focus\:caret-lime-50\/65:focus{caret-color:hsla(78,92%,95%,.65)}.focus\:caret-lime-50\/70:focus{caret-color:hsla(78,92%,95%,.7)}.focus\:caret-lime-50\/75:focus{caret-color:hsla(78,92%,95%,.75)}.focus\:caret-lime-50\/80:focus{caret-color:hsla(78,92%,95%,.8)}.focus\:caret-lime-50\/85:focus{caret-color:hsla(78,92%,95%,.85)}.focus\:caret-lime-50\/90:focus{caret-color:hsla(78,92%,95%,.9)}.focus\:caret-lime-50\/95:focus{caret-color:hsla(78,92%,95%,.95)}.focus\:caret-lime-500:focus{caret-color:#84cc16}.focus\:caret-lime-500\/0:focus{caret-color:rgba(132,204,22,0)}.focus\:caret-lime-500\/10:focus{caret-color:rgba(132,204,22,.1)}.focus\:caret-lime-500\/100:focus{caret-color:#84cc16}.focus\:caret-lime-500\/15:focus{caret-color:rgba(132,204,22,.15)}.focus\:caret-lime-500\/20:focus{caret-color:rgba(132,204,22,.2)}.focus\:caret-lime-500\/25:focus{caret-color:rgba(132,204,22,.25)}.focus\:caret-lime-500\/30:focus{caret-color:rgba(132,204,22,.3)}.focus\:caret-lime-500\/35:focus{caret-color:rgba(132,204,22,.35)}.focus\:caret-lime-500\/40:focus{caret-color:rgba(132,204,22,.4)}.focus\:caret-lime-500\/45:focus{caret-color:rgba(132,204,22,.45)}.focus\:caret-lime-500\/5:focus{caret-color:rgba(132,204,22,.05)}.focus\:caret-lime-500\/50:focus{caret-color:rgba(132,204,22,.5)}.focus\:caret-lime-500\/55:focus{caret-color:rgba(132,204,22,.55)}.focus\:caret-lime-500\/60:focus{caret-color:rgba(132,204,22,.6)}.focus\:caret-lime-500\/65:focus{caret-color:rgba(132,204,22,.65)}.focus\:caret-lime-500\/70:focus{caret-color:rgba(132,204,22,.7)}.focus\:caret-lime-500\/75:focus{caret-color:rgba(132,204,22,.75)}.focus\:caret-lime-500\/80:focus{caret-color:rgba(132,204,22,.8)}.focus\:caret-lime-500\/85:focus{caret-color:rgba(132,204,22,.85)}.focus\:caret-lime-500\/90:focus{caret-color:rgba(132,204,22,.9)}.focus\:caret-lime-500\/95:focus{caret-color:rgba(132,204,22,.95)}.focus\:caret-lime-950:focus{caret-color:#1a2e05}.focus\:caret-lime-950\/0:focus{caret-color:rgba(26,46,5,0)}.focus\:caret-lime-950\/10:focus{caret-color:rgba(26,46,5,.1)}.focus\:caret-lime-950\/100:focus{caret-color:#1a2e05}.focus\:caret-lime-950\/15:focus{caret-color:rgba(26,46,5,.15)}.focus\:caret-lime-950\/20:focus{caret-color:rgba(26,46,5,.2)}.focus\:caret-lime-950\/25:focus{caret-color:rgba(26,46,5,.25)}.focus\:caret-lime-950\/30:focus{caret-color:rgba(26,46,5,.3)}.focus\:caret-lime-950\/35:focus{caret-color:rgba(26,46,5,.35)}.focus\:caret-lime-950\/40:focus{caret-color:rgba(26,46,5,.4)}.focus\:caret-lime-950\/45:focus{caret-color:rgba(26,46,5,.45)}.focus\:caret-lime-950\/5:focus{caret-color:rgba(26,46,5,.05)}.focus\:caret-lime-950\/50:focus{caret-color:rgba(26,46,5,.5)}.focus\:caret-lime-950\/55:focus{caret-color:rgba(26,46,5,.55)}.focus\:caret-lime-950\/60:focus{caret-color:rgba(26,46,5,.6)}.focus\:caret-lime-950\/65:focus{caret-color:rgba(26,46,5,.65)}.focus\:caret-lime-950\/70:focus{caret-color:rgba(26,46,5,.7)}.focus\:caret-lime-950\/75:focus{caret-color:rgba(26,46,5,.75)}.focus\:caret-lime-950\/80:focus{caret-color:rgba(26,46,5,.8)}.focus\:caret-lime-950\/85:focus{caret-color:rgba(26,46,5,.85)}.focus\:caret-lime-950\/90:focus{caret-color:rgba(26,46,5,.9)}.focus\:caret-lime-950\/95:focus{caret-color:rgba(26,46,5,.95)}.focus\:caret-neutral-100:focus{caret-color:#f5f5f5}.focus\:caret-neutral-100\/0:focus{caret-color:hsla(0,0%,96%,0)}.focus\:caret-neutral-100\/10:focus{caret-color:hsla(0,0%,96%,.1)}.focus\:caret-neutral-100\/100:focus{caret-color:#f5f5f5}.focus\:caret-neutral-100\/15:focus{caret-color:hsla(0,0%,96%,.15)}.focus\:caret-neutral-100\/20:focus{caret-color:hsla(0,0%,96%,.2)}.focus\:caret-neutral-100\/25:focus{caret-color:hsla(0,0%,96%,.25)}.focus\:caret-neutral-100\/30:focus{caret-color:hsla(0,0%,96%,.3)}.focus\:caret-neutral-100\/35:focus{caret-color:hsla(0,0%,96%,.35)}.focus\:caret-neutral-100\/40:focus{caret-color:hsla(0,0%,96%,.4)}.focus\:caret-neutral-100\/45:focus{caret-color:hsla(0,0%,96%,.45)}.focus\:caret-neutral-100\/5:focus{caret-color:hsla(0,0%,96%,.05)}.focus\:caret-neutral-100\/50:focus{caret-color:hsla(0,0%,96%,.5)}.focus\:caret-neutral-100\/55:focus{caret-color:hsla(0,0%,96%,.55)}.focus\:caret-neutral-100\/60:focus{caret-color:hsla(0,0%,96%,.6)}.focus\:caret-neutral-100\/65:focus{caret-color:hsla(0,0%,96%,.65)}.focus\:caret-neutral-100\/70:focus{caret-color:hsla(0,0%,96%,.7)}.focus\:caret-neutral-100\/75:focus{caret-color:hsla(0,0%,96%,.75)}.focus\:caret-neutral-100\/80:focus{caret-color:hsla(0,0%,96%,.8)}.focus\:caret-neutral-100\/85:focus{caret-color:hsla(0,0%,96%,.85)}.focus\:caret-neutral-100\/90:focus{caret-color:hsla(0,0%,96%,.9)}.focus\:caret-neutral-100\/95:focus{caret-color:hsla(0,0%,96%,.95)}.focus\:caret-neutral-200:focus{caret-color:#e5e5e5}.focus\:caret-neutral-200\/0:focus{caret-color:hsla(0,0%,90%,0)}.focus\:caret-neutral-200\/10:focus{caret-color:hsla(0,0%,90%,.1)}.focus\:caret-neutral-200\/100:focus{caret-color:#e5e5e5}.focus\:caret-neutral-200\/15:focus{caret-color:hsla(0,0%,90%,.15)}.focus\:caret-neutral-200\/20:focus{caret-color:hsla(0,0%,90%,.2)}.focus\:caret-neutral-200\/25:focus{caret-color:hsla(0,0%,90%,.25)}.focus\:caret-neutral-200\/30:focus{caret-color:hsla(0,0%,90%,.3)}.focus\:caret-neutral-200\/35:focus{caret-color:hsla(0,0%,90%,.35)}.focus\:caret-neutral-200\/40:focus{caret-color:hsla(0,0%,90%,.4)}.focus\:caret-neutral-200\/45:focus{caret-color:hsla(0,0%,90%,.45)}.focus\:caret-neutral-200\/5:focus{caret-color:hsla(0,0%,90%,.05)}.focus\:caret-neutral-200\/50:focus{caret-color:hsla(0,0%,90%,.5)}.focus\:caret-neutral-200\/55:focus{caret-color:hsla(0,0%,90%,.55)}.focus\:caret-neutral-200\/60:focus{caret-color:hsla(0,0%,90%,.6)}.focus\:caret-neutral-200\/65:focus{caret-color:hsla(0,0%,90%,.65)}.focus\:caret-neutral-200\/70:focus{caret-color:hsla(0,0%,90%,.7)}.focus\:caret-neutral-200\/75:focus{caret-color:hsla(0,0%,90%,.75)}.focus\:caret-neutral-200\/80:focus{caret-color:hsla(0,0%,90%,.8)}.focus\:caret-neutral-200\/85:focus{caret-color:hsla(0,0%,90%,.85)}.focus\:caret-neutral-200\/90:focus{caret-color:hsla(0,0%,90%,.9)}.focus\:caret-neutral-200\/95:focus{caret-color:hsla(0,0%,90%,.95)}.focus\:caret-neutral-400:focus{caret-color:#a3a3a3}.focus\:caret-neutral-400\/0:focus{caret-color:hsla(0,0%,64%,0)}.focus\:caret-neutral-400\/10:focus{caret-color:hsla(0,0%,64%,.1)}.focus\:caret-neutral-400\/100:focus{caret-color:#a3a3a3}.focus\:caret-neutral-400\/15:focus{caret-color:hsla(0,0%,64%,.15)}.focus\:caret-neutral-400\/20:focus{caret-color:hsla(0,0%,64%,.2)}.focus\:caret-neutral-400\/25:focus{caret-color:hsla(0,0%,64%,.25)}.focus\:caret-neutral-400\/30:focus{caret-color:hsla(0,0%,64%,.3)}.focus\:caret-neutral-400\/35:focus{caret-color:hsla(0,0%,64%,.35)}.focus\:caret-neutral-400\/40:focus{caret-color:hsla(0,0%,64%,.4)}.focus\:caret-neutral-400\/45:focus{caret-color:hsla(0,0%,64%,.45)}.focus\:caret-neutral-400\/5:focus{caret-color:hsla(0,0%,64%,.05)}.focus\:caret-neutral-400\/50:focus{caret-color:hsla(0,0%,64%,.5)}.focus\:caret-neutral-400\/55:focus{caret-color:hsla(0,0%,64%,.55)}.focus\:caret-neutral-400\/60:focus{caret-color:hsla(0,0%,64%,.6)}.focus\:caret-neutral-400\/65:focus{caret-color:hsla(0,0%,64%,.65)}.focus\:caret-neutral-400\/70:focus{caret-color:hsla(0,0%,64%,.7)}.focus\:caret-neutral-400\/75:focus{caret-color:hsla(0,0%,64%,.75)}.focus\:caret-neutral-400\/80:focus{caret-color:hsla(0,0%,64%,.8)}.focus\:caret-neutral-400\/85:focus{caret-color:hsla(0,0%,64%,.85)}.focus\:caret-neutral-400\/90:focus{caret-color:hsla(0,0%,64%,.9)}.focus\:caret-neutral-400\/95:focus{caret-color:hsla(0,0%,64%,.95)}.focus\:caret-neutral-50:focus{caret-color:#fafafa}.focus\:caret-neutral-50\/0:focus{caret-color:hsla(0,0%,98%,0)}.focus\:caret-neutral-50\/10:focus{caret-color:hsla(0,0%,98%,.1)}.focus\:caret-neutral-50\/100:focus{caret-color:#fafafa}.focus\:caret-neutral-50\/15:focus{caret-color:hsla(0,0%,98%,.15)}.focus\:caret-neutral-50\/20:focus{caret-color:hsla(0,0%,98%,.2)}.focus\:caret-neutral-50\/25:focus{caret-color:hsla(0,0%,98%,.25)}.focus\:caret-neutral-50\/30:focus{caret-color:hsla(0,0%,98%,.3)}.focus\:caret-neutral-50\/35:focus{caret-color:hsla(0,0%,98%,.35)}.focus\:caret-neutral-50\/40:focus{caret-color:hsla(0,0%,98%,.4)}.focus\:caret-neutral-50\/45:focus{caret-color:hsla(0,0%,98%,.45)}.focus\:caret-neutral-50\/5:focus{caret-color:hsla(0,0%,98%,.05)}.focus\:caret-neutral-50\/50:focus{caret-color:hsla(0,0%,98%,.5)}.focus\:caret-neutral-50\/55:focus{caret-color:hsla(0,0%,98%,.55)}.focus\:caret-neutral-50\/60:focus{caret-color:hsla(0,0%,98%,.6)}.focus\:caret-neutral-50\/65:focus{caret-color:hsla(0,0%,98%,.65)}.focus\:caret-neutral-50\/70:focus{caret-color:hsla(0,0%,98%,.7)}.focus\:caret-neutral-50\/75:focus{caret-color:hsla(0,0%,98%,.75)}.focus\:caret-neutral-50\/80:focus{caret-color:hsla(0,0%,98%,.8)}.focus\:caret-neutral-50\/85:focus{caret-color:hsla(0,0%,98%,.85)}.focus\:caret-neutral-50\/90:focus{caret-color:hsla(0,0%,98%,.9)}.focus\:caret-neutral-50\/95:focus{caret-color:hsla(0,0%,98%,.95)}.focus\:caret-neutral-500:focus{caret-color:#737373}.focus\:caret-neutral-500\/0:focus{caret-color:hsla(0,0%,45%,0)}.focus\:caret-neutral-500\/10:focus{caret-color:hsla(0,0%,45%,.1)}.focus\:caret-neutral-500\/100:focus{caret-color:#737373}.focus\:caret-neutral-500\/15:focus{caret-color:hsla(0,0%,45%,.15)}.focus\:caret-neutral-500\/20:focus{caret-color:hsla(0,0%,45%,.2)}.focus\:caret-neutral-500\/25:focus{caret-color:hsla(0,0%,45%,.25)}.focus\:caret-neutral-500\/30:focus{caret-color:hsla(0,0%,45%,.3)}.focus\:caret-neutral-500\/35:focus{caret-color:hsla(0,0%,45%,.35)}.focus\:caret-neutral-500\/40:focus{caret-color:hsla(0,0%,45%,.4)}.focus\:caret-neutral-500\/45:focus{caret-color:hsla(0,0%,45%,.45)}.focus\:caret-neutral-500\/5:focus{caret-color:hsla(0,0%,45%,.05)}.focus\:caret-neutral-500\/50:focus{caret-color:hsla(0,0%,45%,.5)}.focus\:caret-neutral-500\/55:focus{caret-color:hsla(0,0%,45%,.55)}.focus\:caret-neutral-500\/60:focus{caret-color:hsla(0,0%,45%,.6)}.focus\:caret-neutral-500\/65:focus{caret-color:hsla(0,0%,45%,.65)}.focus\:caret-neutral-500\/70:focus{caret-color:hsla(0,0%,45%,.7)}.focus\:caret-neutral-500\/75:focus{caret-color:hsla(0,0%,45%,.75)}.focus\:caret-neutral-500\/80:focus{caret-color:hsla(0,0%,45%,.8)}.focus\:caret-neutral-500\/85:focus{caret-color:hsla(0,0%,45%,.85)}.focus\:caret-neutral-500\/90:focus{caret-color:hsla(0,0%,45%,.9)}.focus\:caret-neutral-500\/95:focus{caret-color:hsla(0,0%,45%,.95)}.focus\:caret-neutral-950:focus{caret-color:#0a0a0a}.focus\:caret-neutral-950\/0:focus{caret-color:hsla(0,0%,4%,0)}.focus\:caret-neutral-950\/10:focus{caret-color:hsla(0,0%,4%,.1)}.focus\:caret-neutral-950\/100:focus{caret-color:#0a0a0a}.focus\:caret-neutral-950\/15:focus{caret-color:hsla(0,0%,4%,.15)}.focus\:caret-neutral-950\/20:focus{caret-color:hsla(0,0%,4%,.2)}.focus\:caret-neutral-950\/25:focus{caret-color:hsla(0,0%,4%,.25)}.focus\:caret-neutral-950\/30:focus{caret-color:hsla(0,0%,4%,.3)}.focus\:caret-neutral-950\/35:focus{caret-color:hsla(0,0%,4%,.35)}.focus\:caret-neutral-950\/40:focus{caret-color:hsla(0,0%,4%,.4)}.focus\:caret-neutral-950\/45:focus{caret-color:hsla(0,0%,4%,.45)}.focus\:caret-neutral-950\/5:focus{caret-color:hsla(0,0%,4%,.05)}.focus\:caret-neutral-950\/50:focus{caret-color:hsla(0,0%,4%,.5)}.focus\:caret-neutral-950\/55:focus{caret-color:hsla(0,0%,4%,.55)}.focus\:caret-neutral-950\/60:focus{caret-color:hsla(0,0%,4%,.6)}.focus\:caret-neutral-950\/65:focus{caret-color:hsla(0,0%,4%,.65)}.focus\:caret-neutral-950\/70:focus{caret-color:hsla(0,0%,4%,.7)}.focus\:caret-neutral-950\/75:focus{caret-color:hsla(0,0%,4%,.75)}.focus\:caret-neutral-950\/80:focus{caret-color:hsla(0,0%,4%,.8)}.focus\:caret-neutral-950\/85:focus{caret-color:hsla(0,0%,4%,.85)}.focus\:caret-neutral-950\/90:focus{caret-color:hsla(0,0%,4%,.9)}.focus\:caret-neutral-950\/95:focus{caret-color:hsla(0,0%,4%,.95)}.focus\:caret-orange-100:focus{caret-color:#ffedd5}.focus\:caret-orange-100\/0:focus{caret-color:rgba(255,237,213,0)}.focus\:caret-orange-100\/10:focus{caret-color:rgba(255,237,213,.1)}.focus\:caret-orange-100\/100:focus{caret-color:#ffedd5}.focus\:caret-orange-100\/15:focus{caret-color:rgba(255,237,213,.15)}.focus\:caret-orange-100\/20:focus{caret-color:rgba(255,237,213,.2)}.focus\:caret-orange-100\/25:focus{caret-color:rgba(255,237,213,.25)}.focus\:caret-orange-100\/30:focus{caret-color:rgba(255,237,213,.3)}.focus\:caret-orange-100\/35:focus{caret-color:rgba(255,237,213,.35)}.focus\:caret-orange-100\/40:focus{caret-color:rgba(255,237,213,.4)}.focus\:caret-orange-100\/45:focus{caret-color:rgba(255,237,213,.45)}.focus\:caret-orange-100\/5:focus{caret-color:rgba(255,237,213,.05)}.focus\:caret-orange-100\/50:focus{caret-color:rgba(255,237,213,.5)}.focus\:caret-orange-100\/55:focus{caret-color:rgba(255,237,213,.55)}.focus\:caret-orange-100\/60:focus{caret-color:rgba(255,237,213,.6)}.focus\:caret-orange-100\/65:focus{caret-color:rgba(255,237,213,.65)}.focus\:caret-orange-100\/70:focus{caret-color:rgba(255,237,213,.7)}.focus\:caret-orange-100\/75:focus{caret-color:rgba(255,237,213,.75)}.focus\:caret-orange-100\/80:focus{caret-color:rgba(255,237,213,.8)}.focus\:caret-orange-100\/85:focus{caret-color:rgba(255,237,213,.85)}.focus\:caret-orange-100\/90:focus{caret-color:rgba(255,237,213,.9)}.focus\:caret-orange-100\/95:focus{caret-color:rgba(255,237,213,.95)}.focus\:caret-orange-200:focus{caret-color:#fed7aa}.focus\:caret-orange-200\/0:focus{caret-color:hsla(32,98%,83%,0)}.focus\:caret-orange-200\/10:focus{caret-color:hsla(32,98%,83%,.1)}.focus\:caret-orange-200\/100:focus{caret-color:#fed7aa}.focus\:caret-orange-200\/15:focus{caret-color:hsla(32,98%,83%,.15)}.focus\:caret-orange-200\/20:focus{caret-color:hsla(32,98%,83%,.2)}.focus\:caret-orange-200\/25:focus{caret-color:hsla(32,98%,83%,.25)}.focus\:caret-orange-200\/30:focus{caret-color:hsla(32,98%,83%,.3)}.focus\:caret-orange-200\/35:focus{caret-color:hsla(32,98%,83%,.35)}.focus\:caret-orange-200\/40:focus{caret-color:hsla(32,98%,83%,.4)}.focus\:caret-orange-200\/45:focus{caret-color:hsla(32,98%,83%,.45)}.focus\:caret-orange-200\/5:focus{caret-color:hsla(32,98%,83%,.05)}.focus\:caret-orange-200\/50:focus{caret-color:hsla(32,98%,83%,.5)}.focus\:caret-orange-200\/55:focus{caret-color:hsla(32,98%,83%,.55)}.focus\:caret-orange-200\/60:focus{caret-color:hsla(32,98%,83%,.6)}.focus\:caret-orange-200\/65:focus{caret-color:hsla(32,98%,83%,.65)}.focus\:caret-orange-200\/70:focus{caret-color:hsla(32,98%,83%,.7)}.focus\:caret-orange-200\/75:focus{caret-color:hsla(32,98%,83%,.75)}.focus\:caret-orange-200\/80:focus{caret-color:hsla(32,98%,83%,.8)}.focus\:caret-orange-200\/85:focus{caret-color:hsla(32,98%,83%,.85)}.focus\:caret-orange-200\/90:focus{caret-color:hsla(32,98%,83%,.9)}.focus\:caret-orange-200\/95:focus{caret-color:hsla(32,98%,83%,.95)}.focus\:caret-orange-400:focus{caret-color:#fb923c}.focus\:caret-orange-400\/0:focus{caret-color:rgba(251,146,60,0)}.focus\:caret-orange-400\/10:focus{caret-color:rgba(251,146,60,.1)}.focus\:caret-orange-400\/100:focus{caret-color:#fb923c}.focus\:caret-orange-400\/15:focus{caret-color:rgba(251,146,60,.15)}.focus\:caret-orange-400\/20:focus{caret-color:rgba(251,146,60,.2)}.focus\:caret-orange-400\/25:focus{caret-color:rgba(251,146,60,.25)}.focus\:caret-orange-400\/30:focus{caret-color:rgba(251,146,60,.3)}.focus\:caret-orange-400\/35:focus{caret-color:rgba(251,146,60,.35)}.focus\:caret-orange-400\/40:focus{caret-color:rgba(251,146,60,.4)}.focus\:caret-orange-400\/45:focus{caret-color:rgba(251,146,60,.45)}.focus\:caret-orange-400\/5:focus{caret-color:rgba(251,146,60,.05)}.focus\:caret-orange-400\/50:focus{caret-color:rgba(251,146,60,.5)}.focus\:caret-orange-400\/55:focus{caret-color:rgba(251,146,60,.55)}.focus\:caret-orange-400\/60:focus{caret-color:rgba(251,146,60,.6)}.focus\:caret-orange-400\/65:focus{caret-color:rgba(251,146,60,.65)}.focus\:caret-orange-400\/70:focus{caret-color:rgba(251,146,60,.7)}.focus\:caret-orange-400\/75:focus{caret-color:rgba(251,146,60,.75)}.focus\:caret-orange-400\/80:focus{caret-color:rgba(251,146,60,.8)}.focus\:caret-orange-400\/85:focus{caret-color:rgba(251,146,60,.85)}.focus\:caret-orange-400\/90:focus{caret-color:rgba(251,146,60,.9)}.focus\:caret-orange-400\/95:focus{caret-color:rgba(251,146,60,.95)}.focus\:caret-orange-50:focus{caret-color:#fff7ed}.focus\:caret-orange-50\/0:focus{caret-color:rgba(255,247,237,0)}.focus\:caret-orange-50\/10:focus{caret-color:rgba(255,247,237,.1)}.focus\:caret-orange-50\/100:focus{caret-color:#fff7ed}.focus\:caret-orange-50\/15:focus{caret-color:rgba(255,247,237,.15)}.focus\:caret-orange-50\/20:focus{caret-color:rgba(255,247,237,.2)}.focus\:caret-orange-50\/25:focus{caret-color:rgba(255,247,237,.25)}.focus\:caret-orange-50\/30:focus{caret-color:rgba(255,247,237,.3)}.focus\:caret-orange-50\/35:focus{caret-color:rgba(255,247,237,.35)}.focus\:caret-orange-50\/40:focus{caret-color:rgba(255,247,237,.4)}.focus\:caret-orange-50\/45:focus{caret-color:rgba(255,247,237,.45)}.focus\:caret-orange-50\/5:focus{caret-color:rgba(255,247,237,.05)}.focus\:caret-orange-50\/50:focus{caret-color:rgba(255,247,237,.5)}.focus\:caret-orange-50\/55:focus{caret-color:rgba(255,247,237,.55)}.focus\:caret-orange-50\/60:focus{caret-color:rgba(255,247,237,.6)}.focus\:caret-orange-50\/65:focus{caret-color:rgba(255,247,237,.65)}.focus\:caret-orange-50\/70:focus{caret-color:rgba(255,247,237,.7)}.focus\:caret-orange-50\/75:focus{caret-color:rgba(255,247,237,.75)}.focus\:caret-orange-50\/80:focus{caret-color:rgba(255,247,237,.8)}.focus\:caret-orange-50\/85:focus{caret-color:rgba(255,247,237,.85)}.focus\:caret-orange-50\/90:focus{caret-color:rgba(255,247,237,.9)}.focus\:caret-orange-50\/95:focus{caret-color:rgba(255,247,237,.95)}.focus\:caret-orange-500:focus{caret-color:#f97316}.focus\:caret-orange-500\/0:focus{caret-color:rgba(249,115,22,0)}.focus\:caret-orange-500\/10:focus{caret-color:rgba(249,115,22,.1)}.focus\:caret-orange-500\/100:focus{caret-color:#f97316}.focus\:caret-orange-500\/15:focus{caret-color:rgba(249,115,22,.15)}.focus\:caret-orange-500\/20:focus{caret-color:rgba(249,115,22,.2)}.focus\:caret-orange-500\/25:focus{caret-color:rgba(249,115,22,.25)}.focus\:caret-orange-500\/30:focus{caret-color:rgba(249,115,22,.3)}.focus\:caret-orange-500\/35:focus{caret-color:rgba(249,115,22,.35)}.focus\:caret-orange-500\/40:focus{caret-color:rgba(249,115,22,.4)}.focus\:caret-orange-500\/45:focus{caret-color:rgba(249,115,22,.45)}.focus\:caret-orange-500\/5:focus{caret-color:rgba(249,115,22,.05)}.focus\:caret-orange-500\/50:focus{caret-color:rgba(249,115,22,.5)}.focus\:caret-orange-500\/55:focus{caret-color:rgba(249,115,22,.55)}.focus\:caret-orange-500\/60:focus{caret-color:rgba(249,115,22,.6)}.focus\:caret-orange-500\/65:focus{caret-color:rgba(249,115,22,.65)}.focus\:caret-orange-500\/70:focus{caret-color:rgba(249,115,22,.7)}.focus\:caret-orange-500\/75:focus{caret-color:rgba(249,115,22,.75)}.focus\:caret-orange-500\/80:focus{caret-color:rgba(249,115,22,.8)}.focus\:caret-orange-500\/85:focus{caret-color:rgba(249,115,22,.85)}.focus\:caret-orange-500\/90:focus{caret-color:rgba(249,115,22,.9)}.focus\:caret-orange-500\/95:focus{caret-color:rgba(249,115,22,.95)}.focus\:caret-orange-950:focus{caret-color:#431407}.focus\:caret-orange-950\/0:focus{caret-color:rgba(67,20,7,0)}.focus\:caret-orange-950\/10:focus{caret-color:rgba(67,20,7,.1)}.focus\:caret-orange-950\/100:focus{caret-color:#431407}.focus\:caret-orange-950\/15:focus{caret-color:rgba(67,20,7,.15)}.focus\:caret-orange-950\/20:focus{caret-color:rgba(67,20,7,.2)}.focus\:caret-orange-950\/25:focus{caret-color:rgba(67,20,7,.25)}.focus\:caret-orange-950\/30:focus{caret-color:rgba(67,20,7,.3)}.focus\:caret-orange-950\/35:focus{caret-color:rgba(67,20,7,.35)}.focus\:caret-orange-950\/40:focus{caret-color:rgba(67,20,7,.4)}.focus\:caret-orange-950\/45:focus{caret-color:rgba(67,20,7,.45)}.focus\:caret-orange-950\/5:focus{caret-color:rgba(67,20,7,.05)}.focus\:caret-orange-950\/50:focus{caret-color:rgba(67,20,7,.5)}.focus\:caret-orange-950\/55:focus{caret-color:rgba(67,20,7,.55)}.focus\:caret-orange-950\/60:focus{caret-color:rgba(67,20,7,.6)}.focus\:caret-orange-950\/65:focus{caret-color:rgba(67,20,7,.65)}.focus\:caret-orange-950\/70:focus{caret-color:rgba(67,20,7,.7)}.focus\:caret-orange-950\/75:focus{caret-color:rgba(67,20,7,.75)}.focus\:caret-orange-950\/80:focus{caret-color:rgba(67,20,7,.8)}.focus\:caret-orange-950\/85:focus{caret-color:rgba(67,20,7,.85)}.focus\:caret-orange-950\/90:focus{caret-color:rgba(67,20,7,.9)}.focus\:caret-orange-950\/95:focus{caret-color:rgba(67,20,7,.95)}.focus\:caret-pink-100:focus{caret-color:#fce7f3}.focus\:caret-pink-100\/0:focus{caret-color:rgba(252,231,243,0)}.focus\:caret-pink-100\/10:focus{caret-color:rgba(252,231,243,.1)}.focus\:caret-pink-100\/100:focus{caret-color:#fce7f3}.focus\:caret-pink-100\/15:focus{caret-color:rgba(252,231,243,.15)}.focus\:caret-pink-100\/20:focus{caret-color:rgba(252,231,243,.2)}.focus\:caret-pink-100\/25:focus{caret-color:rgba(252,231,243,.25)}.focus\:caret-pink-100\/30:focus{caret-color:rgba(252,231,243,.3)}.focus\:caret-pink-100\/35:focus{caret-color:rgba(252,231,243,.35)}.focus\:caret-pink-100\/40:focus{caret-color:rgba(252,231,243,.4)}.focus\:caret-pink-100\/45:focus{caret-color:rgba(252,231,243,.45)}.focus\:caret-pink-100\/5:focus{caret-color:rgba(252,231,243,.05)}.focus\:caret-pink-100\/50:focus{caret-color:rgba(252,231,243,.5)}.focus\:caret-pink-100\/55:focus{caret-color:rgba(252,231,243,.55)}.focus\:caret-pink-100\/60:focus{caret-color:rgba(252,231,243,.6)}.focus\:caret-pink-100\/65:focus{caret-color:rgba(252,231,243,.65)}.focus\:caret-pink-100\/70:focus{caret-color:rgba(252,231,243,.7)}.focus\:caret-pink-100\/75:focus{caret-color:rgba(252,231,243,.75)}.focus\:caret-pink-100\/80:focus{caret-color:rgba(252,231,243,.8)}.focus\:caret-pink-100\/85:focus{caret-color:rgba(252,231,243,.85)}.focus\:caret-pink-100\/90:focus{caret-color:rgba(252,231,243,.9)}.focus\:caret-pink-100\/95:focus{caret-color:rgba(252,231,243,.95)}.focus\:caret-pink-200:focus{caret-color:#fbcfe8}.focus\:caret-pink-200\/0:focus{caret-color:rgba(251,207,232,0)}.focus\:caret-pink-200\/10:focus{caret-color:rgba(251,207,232,.1)}.focus\:caret-pink-200\/100:focus{caret-color:#fbcfe8}.focus\:caret-pink-200\/15:focus{caret-color:rgba(251,207,232,.15)}.focus\:caret-pink-200\/20:focus{caret-color:rgba(251,207,232,.2)}.focus\:caret-pink-200\/25:focus{caret-color:rgba(251,207,232,.25)}.focus\:caret-pink-200\/30:focus{caret-color:rgba(251,207,232,.3)}.focus\:caret-pink-200\/35:focus{caret-color:rgba(251,207,232,.35)}.focus\:caret-pink-200\/40:focus{caret-color:rgba(251,207,232,.4)}.focus\:caret-pink-200\/45:focus{caret-color:rgba(251,207,232,.45)}.focus\:caret-pink-200\/5:focus{caret-color:rgba(251,207,232,.05)}.focus\:caret-pink-200\/50:focus{caret-color:rgba(251,207,232,.5)}.focus\:caret-pink-200\/55:focus{caret-color:rgba(251,207,232,.55)}.focus\:caret-pink-200\/60:focus{caret-color:rgba(251,207,232,.6)}.focus\:caret-pink-200\/65:focus{caret-color:rgba(251,207,232,.65)}.focus\:caret-pink-200\/70:focus{caret-color:rgba(251,207,232,.7)}.focus\:caret-pink-200\/75:focus{caret-color:rgba(251,207,232,.75)}.focus\:caret-pink-200\/80:focus{caret-color:rgba(251,207,232,.8)}.focus\:caret-pink-200\/85:focus{caret-color:rgba(251,207,232,.85)}.focus\:caret-pink-200\/90:focus{caret-color:rgba(251,207,232,.9)}.focus\:caret-pink-200\/95:focus{caret-color:rgba(251,207,232,.95)}.focus\:caret-pink-400:focus{caret-color:#f472b6}.focus\:caret-pink-400\/0:focus{caret-color:rgba(244,114,182,0)}.focus\:caret-pink-400\/10:focus{caret-color:rgba(244,114,182,.1)}.focus\:caret-pink-400\/100:focus{caret-color:#f472b6}.focus\:caret-pink-400\/15:focus{caret-color:rgba(244,114,182,.15)}.focus\:caret-pink-400\/20:focus{caret-color:rgba(244,114,182,.2)}.focus\:caret-pink-400\/25:focus{caret-color:rgba(244,114,182,.25)}.focus\:caret-pink-400\/30:focus{caret-color:rgba(244,114,182,.3)}.focus\:caret-pink-400\/35:focus{caret-color:rgba(244,114,182,.35)}.focus\:caret-pink-400\/40:focus{caret-color:rgba(244,114,182,.4)}.focus\:caret-pink-400\/45:focus{caret-color:rgba(244,114,182,.45)}.focus\:caret-pink-400\/5:focus{caret-color:rgba(244,114,182,.05)}.focus\:caret-pink-400\/50:focus{caret-color:rgba(244,114,182,.5)}.focus\:caret-pink-400\/55:focus{caret-color:rgba(244,114,182,.55)}.focus\:caret-pink-400\/60:focus{caret-color:rgba(244,114,182,.6)}.focus\:caret-pink-400\/65:focus{caret-color:rgba(244,114,182,.65)}.focus\:caret-pink-400\/70:focus{caret-color:rgba(244,114,182,.7)}.focus\:caret-pink-400\/75:focus{caret-color:rgba(244,114,182,.75)}.focus\:caret-pink-400\/80:focus{caret-color:rgba(244,114,182,.8)}.focus\:caret-pink-400\/85:focus{caret-color:rgba(244,114,182,.85)}.focus\:caret-pink-400\/90:focus{caret-color:rgba(244,114,182,.9)}.focus\:caret-pink-400\/95:focus{caret-color:rgba(244,114,182,.95)}.focus\:caret-pink-50:focus{caret-color:#fdf2f8}.focus\:caret-pink-50\/0:focus{caret-color:rgba(253,242,248,0)}.focus\:caret-pink-50\/10:focus{caret-color:rgba(253,242,248,.1)}.focus\:caret-pink-50\/100:focus{caret-color:#fdf2f8}.focus\:caret-pink-50\/15:focus{caret-color:rgba(253,242,248,.15)}.focus\:caret-pink-50\/20:focus{caret-color:rgba(253,242,248,.2)}.focus\:caret-pink-50\/25:focus{caret-color:rgba(253,242,248,.25)}.focus\:caret-pink-50\/30:focus{caret-color:rgba(253,242,248,.3)}.focus\:caret-pink-50\/35:focus{caret-color:rgba(253,242,248,.35)}.focus\:caret-pink-50\/40:focus{caret-color:rgba(253,242,248,.4)}.focus\:caret-pink-50\/45:focus{caret-color:rgba(253,242,248,.45)}.focus\:caret-pink-50\/5:focus{caret-color:rgba(253,242,248,.05)}.focus\:caret-pink-50\/50:focus{caret-color:rgba(253,242,248,.5)}.focus\:caret-pink-50\/55:focus{caret-color:rgba(253,242,248,.55)}.focus\:caret-pink-50\/60:focus{caret-color:rgba(253,242,248,.6)}.focus\:caret-pink-50\/65:focus{caret-color:rgba(253,242,248,.65)}.focus\:caret-pink-50\/70:focus{caret-color:rgba(253,242,248,.7)}.focus\:caret-pink-50\/75:focus{caret-color:rgba(253,242,248,.75)}.focus\:caret-pink-50\/80:focus{caret-color:rgba(253,242,248,.8)}.focus\:caret-pink-50\/85:focus{caret-color:rgba(253,242,248,.85)}.focus\:caret-pink-50\/90:focus{caret-color:rgba(253,242,248,.9)}.focus\:caret-pink-50\/95:focus{caret-color:rgba(253,242,248,.95)}.focus\:caret-pink-500:focus{caret-color:#ec4899}.focus\:caret-pink-500\/0:focus{caret-color:rgba(236,72,153,0)}.focus\:caret-pink-500\/10:focus{caret-color:rgba(236,72,153,.1)}.focus\:caret-pink-500\/100:focus{caret-color:#ec4899}.focus\:caret-pink-500\/15:focus{caret-color:rgba(236,72,153,.15)}.focus\:caret-pink-500\/20:focus{caret-color:rgba(236,72,153,.2)}.focus\:caret-pink-500\/25:focus{caret-color:rgba(236,72,153,.25)}.focus\:caret-pink-500\/30:focus{caret-color:rgba(236,72,153,.3)}.focus\:caret-pink-500\/35:focus{caret-color:rgba(236,72,153,.35)}.focus\:caret-pink-500\/40:focus{caret-color:rgba(236,72,153,.4)}.focus\:caret-pink-500\/45:focus{caret-color:rgba(236,72,153,.45)}.focus\:caret-pink-500\/5:focus{caret-color:rgba(236,72,153,.05)}.focus\:caret-pink-500\/50:focus{caret-color:rgba(236,72,153,.5)}.focus\:caret-pink-500\/55:focus{caret-color:rgba(236,72,153,.55)}.focus\:caret-pink-500\/60:focus{caret-color:rgba(236,72,153,.6)}.focus\:caret-pink-500\/65:focus{caret-color:rgba(236,72,153,.65)}.focus\:caret-pink-500\/70:focus{caret-color:rgba(236,72,153,.7)}.focus\:caret-pink-500\/75:focus{caret-color:rgba(236,72,153,.75)}.focus\:caret-pink-500\/80:focus{caret-color:rgba(236,72,153,.8)}.focus\:caret-pink-500\/85:focus{caret-color:rgba(236,72,153,.85)}.focus\:caret-pink-500\/90:focus{caret-color:rgba(236,72,153,.9)}.focus\:caret-pink-500\/95:focus{caret-color:rgba(236,72,153,.95)}.focus\:caret-pink-950:focus{caret-color:#500724}.focus\:caret-pink-950\/0:focus{caret-color:rgba(80,7,36,0)}.focus\:caret-pink-950\/10:focus{caret-color:rgba(80,7,36,.1)}.focus\:caret-pink-950\/100:focus{caret-color:#500724}.focus\:caret-pink-950\/15:focus{caret-color:rgba(80,7,36,.15)}.focus\:caret-pink-950\/20:focus{caret-color:rgba(80,7,36,.2)}.focus\:caret-pink-950\/25:focus{caret-color:rgba(80,7,36,.25)}.focus\:caret-pink-950\/30:focus{caret-color:rgba(80,7,36,.3)}.focus\:caret-pink-950\/35:focus{caret-color:rgba(80,7,36,.35)}.focus\:caret-pink-950\/40:focus{caret-color:rgba(80,7,36,.4)}.focus\:caret-pink-950\/45:focus{caret-color:rgba(80,7,36,.45)}.focus\:caret-pink-950\/5:focus{caret-color:rgba(80,7,36,.05)}.focus\:caret-pink-950\/50:focus{caret-color:rgba(80,7,36,.5)}.focus\:caret-pink-950\/55:focus{caret-color:rgba(80,7,36,.55)}.focus\:caret-pink-950\/60:focus{caret-color:rgba(80,7,36,.6)}.focus\:caret-pink-950\/65:focus{caret-color:rgba(80,7,36,.65)}.focus\:caret-pink-950\/70:focus{caret-color:rgba(80,7,36,.7)}.focus\:caret-pink-950\/75:focus{caret-color:rgba(80,7,36,.75)}.focus\:caret-pink-950\/80:focus{caret-color:rgba(80,7,36,.8)}.focus\:caret-pink-950\/85:focus{caret-color:rgba(80,7,36,.85)}.focus\:caret-pink-950\/90:focus{caret-color:rgba(80,7,36,.9)}.focus\:caret-pink-950\/95:focus{caret-color:rgba(80,7,36,.95)}.focus\:caret-purple-100:focus{caret-color:#f3e8ff}.focus\:caret-purple-100\/0:focus{caret-color:rgba(243,232,255,0)}.focus\:caret-purple-100\/10:focus{caret-color:rgba(243,232,255,.1)}.focus\:caret-purple-100\/100:focus{caret-color:#f3e8ff}.focus\:caret-purple-100\/15:focus{caret-color:rgba(243,232,255,.15)}.focus\:caret-purple-100\/20:focus{caret-color:rgba(243,232,255,.2)}.focus\:caret-purple-100\/25:focus{caret-color:rgba(243,232,255,.25)}.focus\:caret-purple-100\/30:focus{caret-color:rgba(243,232,255,.3)}.focus\:caret-purple-100\/35:focus{caret-color:rgba(243,232,255,.35)}.focus\:caret-purple-100\/40:focus{caret-color:rgba(243,232,255,.4)}.focus\:caret-purple-100\/45:focus{caret-color:rgba(243,232,255,.45)}.focus\:caret-purple-100\/5:focus{caret-color:rgba(243,232,255,.05)}.focus\:caret-purple-100\/50:focus{caret-color:rgba(243,232,255,.5)}.focus\:caret-purple-100\/55:focus{caret-color:rgba(243,232,255,.55)}.focus\:caret-purple-100\/60:focus{caret-color:rgba(243,232,255,.6)}.focus\:caret-purple-100\/65:focus{caret-color:rgba(243,232,255,.65)}.focus\:caret-purple-100\/70:focus{caret-color:rgba(243,232,255,.7)}.focus\:caret-purple-100\/75:focus{caret-color:rgba(243,232,255,.75)}.focus\:caret-purple-100\/80:focus{caret-color:rgba(243,232,255,.8)}.focus\:caret-purple-100\/85:focus{caret-color:rgba(243,232,255,.85)}.focus\:caret-purple-100\/90:focus{caret-color:rgba(243,232,255,.9)}.focus\:caret-purple-100\/95:focus{caret-color:rgba(243,232,255,.95)}.focus\:caret-purple-200:focus{caret-color:#e9d5ff}.focus\:caret-purple-200\/0:focus{caret-color:rgba(233,213,255,0)}.focus\:caret-purple-200\/10:focus{caret-color:rgba(233,213,255,.1)}.focus\:caret-purple-200\/100:focus{caret-color:#e9d5ff}.focus\:caret-purple-200\/15:focus{caret-color:rgba(233,213,255,.15)}.focus\:caret-purple-200\/20:focus{caret-color:rgba(233,213,255,.2)}.focus\:caret-purple-200\/25:focus{caret-color:rgba(233,213,255,.25)}.focus\:caret-purple-200\/30:focus{caret-color:rgba(233,213,255,.3)}.focus\:caret-purple-200\/35:focus{caret-color:rgba(233,213,255,.35)}.focus\:caret-purple-200\/40:focus{caret-color:rgba(233,213,255,.4)}.focus\:caret-purple-200\/45:focus{caret-color:rgba(233,213,255,.45)}.focus\:caret-purple-200\/5:focus{caret-color:rgba(233,213,255,.05)}.focus\:caret-purple-200\/50:focus{caret-color:rgba(233,213,255,.5)}.focus\:caret-purple-200\/55:focus{caret-color:rgba(233,213,255,.55)}.focus\:caret-purple-200\/60:focus{caret-color:rgba(233,213,255,.6)}.focus\:caret-purple-200\/65:focus{caret-color:rgba(233,213,255,.65)}.focus\:caret-purple-200\/70:focus{caret-color:rgba(233,213,255,.7)}.focus\:caret-purple-200\/75:focus{caret-color:rgba(233,213,255,.75)}.focus\:caret-purple-200\/80:focus{caret-color:rgba(233,213,255,.8)}.focus\:caret-purple-200\/85:focus{caret-color:rgba(233,213,255,.85)}.focus\:caret-purple-200\/90:focus{caret-color:rgba(233,213,255,.9)}.focus\:caret-purple-200\/95:focus{caret-color:rgba(233,213,255,.95)}.focus\:caret-purple-400:focus{caret-color:#c084fc}.focus\:caret-purple-400\/0:focus{caret-color:rgba(192,132,252,0)}.focus\:caret-purple-400\/10:focus{caret-color:rgba(192,132,252,.1)}.focus\:caret-purple-400\/100:focus{caret-color:#c084fc}.focus\:caret-purple-400\/15:focus{caret-color:rgba(192,132,252,.15)}.focus\:caret-purple-400\/20:focus{caret-color:rgba(192,132,252,.2)}.focus\:caret-purple-400\/25:focus{caret-color:rgba(192,132,252,.25)}.focus\:caret-purple-400\/30:focus{caret-color:rgba(192,132,252,.3)}.focus\:caret-purple-400\/35:focus{caret-color:rgba(192,132,252,.35)}.focus\:caret-purple-400\/40:focus{caret-color:rgba(192,132,252,.4)}.focus\:caret-purple-400\/45:focus{caret-color:rgba(192,132,252,.45)}.focus\:caret-purple-400\/5:focus{caret-color:rgba(192,132,252,.05)}.focus\:caret-purple-400\/50:focus{caret-color:rgba(192,132,252,.5)}.focus\:caret-purple-400\/55:focus{caret-color:rgba(192,132,252,.55)}.focus\:caret-purple-400\/60:focus{caret-color:rgba(192,132,252,.6)}.focus\:caret-purple-400\/65:focus{caret-color:rgba(192,132,252,.65)}.focus\:caret-purple-400\/70:focus{caret-color:rgba(192,132,252,.7)}.focus\:caret-purple-400\/75:focus{caret-color:rgba(192,132,252,.75)}.focus\:caret-purple-400\/80:focus{caret-color:rgba(192,132,252,.8)}.focus\:caret-purple-400\/85:focus{caret-color:rgba(192,132,252,.85)}.focus\:caret-purple-400\/90:focus{caret-color:rgba(192,132,252,.9)}.focus\:caret-purple-400\/95:focus{caret-color:rgba(192,132,252,.95)}.focus\:caret-purple-50:focus{caret-color:#faf5ff}.focus\:caret-purple-50\/0:focus{caret-color:rgba(250,245,255,0)}.focus\:caret-purple-50\/10:focus{caret-color:rgba(250,245,255,.1)}.focus\:caret-purple-50\/100:focus{caret-color:#faf5ff}.focus\:caret-purple-50\/15:focus{caret-color:rgba(250,245,255,.15)}.focus\:caret-purple-50\/20:focus{caret-color:rgba(250,245,255,.2)}.focus\:caret-purple-50\/25:focus{caret-color:rgba(250,245,255,.25)}.focus\:caret-purple-50\/30:focus{caret-color:rgba(250,245,255,.3)}.focus\:caret-purple-50\/35:focus{caret-color:rgba(250,245,255,.35)}.focus\:caret-purple-50\/40:focus{caret-color:rgba(250,245,255,.4)}.focus\:caret-purple-50\/45:focus{caret-color:rgba(250,245,255,.45)}.focus\:caret-purple-50\/5:focus{caret-color:rgba(250,245,255,.05)}.focus\:caret-purple-50\/50:focus{caret-color:rgba(250,245,255,.5)}.focus\:caret-purple-50\/55:focus{caret-color:rgba(250,245,255,.55)}.focus\:caret-purple-50\/60:focus{caret-color:rgba(250,245,255,.6)}.focus\:caret-purple-50\/65:focus{caret-color:rgba(250,245,255,.65)}.focus\:caret-purple-50\/70:focus{caret-color:rgba(250,245,255,.7)}.focus\:caret-purple-50\/75:focus{caret-color:rgba(250,245,255,.75)}.focus\:caret-purple-50\/80:focus{caret-color:rgba(250,245,255,.8)}.focus\:caret-purple-50\/85:focus{caret-color:rgba(250,245,255,.85)}.focus\:caret-purple-50\/90:focus{caret-color:rgba(250,245,255,.9)}.focus\:caret-purple-50\/95:focus{caret-color:rgba(250,245,255,.95)}.focus\:caret-purple-500:focus{caret-color:#a855f7}.focus\:caret-purple-500\/0:focus{caret-color:rgba(168,85,247,0)}.focus\:caret-purple-500\/10:focus{caret-color:rgba(168,85,247,.1)}.focus\:caret-purple-500\/100:focus{caret-color:#a855f7}.focus\:caret-purple-500\/15:focus{caret-color:rgba(168,85,247,.15)}.focus\:caret-purple-500\/20:focus{caret-color:rgba(168,85,247,.2)}.focus\:caret-purple-500\/25:focus{caret-color:rgba(168,85,247,.25)}.focus\:caret-purple-500\/30:focus{caret-color:rgba(168,85,247,.3)}.focus\:caret-purple-500\/35:focus{caret-color:rgba(168,85,247,.35)}.focus\:caret-purple-500\/40:focus{caret-color:rgba(168,85,247,.4)}.focus\:caret-purple-500\/45:focus{caret-color:rgba(168,85,247,.45)}.focus\:caret-purple-500\/5:focus{caret-color:rgba(168,85,247,.05)}.focus\:caret-purple-500\/50:focus{caret-color:rgba(168,85,247,.5)}.focus\:caret-purple-500\/55:focus{caret-color:rgba(168,85,247,.55)}.focus\:caret-purple-500\/60:focus{caret-color:rgba(168,85,247,.6)}.focus\:caret-purple-500\/65:focus{caret-color:rgba(168,85,247,.65)}.focus\:caret-purple-500\/70:focus{caret-color:rgba(168,85,247,.7)}.focus\:caret-purple-500\/75:focus{caret-color:rgba(168,85,247,.75)}.focus\:caret-purple-500\/80:focus{caret-color:rgba(168,85,247,.8)}.focus\:caret-purple-500\/85:focus{caret-color:rgba(168,85,247,.85)}.focus\:caret-purple-500\/90:focus{caret-color:rgba(168,85,247,.9)}.focus\:caret-purple-500\/95:focus{caret-color:rgba(168,85,247,.95)}.focus\:caret-purple-950:focus{caret-color:#3b0764}.focus\:caret-purple-950\/0:focus{caret-color:rgba(59,7,100,0)}.focus\:caret-purple-950\/10:focus{caret-color:rgba(59,7,100,.1)}.focus\:caret-purple-950\/100:focus{caret-color:#3b0764}.focus\:caret-purple-950\/15:focus{caret-color:rgba(59,7,100,.15)}.focus\:caret-purple-950\/20:focus{caret-color:rgba(59,7,100,.2)}.focus\:caret-purple-950\/25:focus{caret-color:rgba(59,7,100,.25)}.focus\:caret-purple-950\/30:focus{caret-color:rgba(59,7,100,.3)}.focus\:caret-purple-950\/35:focus{caret-color:rgba(59,7,100,.35)}.focus\:caret-purple-950\/40:focus{caret-color:rgba(59,7,100,.4)}.focus\:caret-purple-950\/45:focus{caret-color:rgba(59,7,100,.45)}.focus\:caret-purple-950\/5:focus{caret-color:rgba(59,7,100,.05)}.focus\:caret-purple-950\/50:focus{caret-color:rgba(59,7,100,.5)}.focus\:caret-purple-950\/55:focus{caret-color:rgba(59,7,100,.55)}.focus\:caret-purple-950\/60:focus{caret-color:rgba(59,7,100,.6)}.focus\:caret-purple-950\/65:focus{caret-color:rgba(59,7,100,.65)}.focus\:caret-purple-950\/70:focus{caret-color:rgba(59,7,100,.7)}.focus\:caret-purple-950\/75:focus{caret-color:rgba(59,7,100,.75)}.focus\:caret-purple-950\/80:focus{caret-color:rgba(59,7,100,.8)}.focus\:caret-purple-950\/85:focus{caret-color:rgba(59,7,100,.85)}.focus\:caret-purple-950\/90:focus{caret-color:rgba(59,7,100,.9)}.focus\:caret-purple-950\/95:focus{caret-color:rgba(59,7,100,.95)}.focus\:caret-red-100:focus{caret-color:#fee2e2}.focus\:caret-red-100\/0:focus{caret-color:hsla(0,93%,94%,0)}.focus\:caret-red-100\/10:focus{caret-color:hsla(0,93%,94%,.1)}.focus\:caret-red-100\/100:focus{caret-color:#fee2e2}.focus\:caret-red-100\/15:focus{caret-color:hsla(0,93%,94%,.15)}.focus\:caret-red-100\/20:focus{caret-color:hsla(0,93%,94%,.2)}.focus\:caret-red-100\/25:focus{caret-color:hsla(0,93%,94%,.25)}.focus\:caret-red-100\/30:focus{caret-color:hsla(0,93%,94%,.3)}.focus\:caret-red-100\/35:focus{caret-color:hsla(0,93%,94%,.35)}.focus\:caret-red-100\/40:focus{caret-color:hsla(0,93%,94%,.4)}.focus\:caret-red-100\/45:focus{caret-color:hsla(0,93%,94%,.45)}.focus\:caret-red-100\/5:focus{caret-color:hsla(0,93%,94%,.05)}.focus\:caret-red-100\/50:focus{caret-color:hsla(0,93%,94%,.5)}.focus\:caret-red-100\/55:focus{caret-color:hsla(0,93%,94%,.55)}.focus\:caret-red-100\/60:focus{caret-color:hsla(0,93%,94%,.6)}.focus\:caret-red-100\/65:focus{caret-color:hsla(0,93%,94%,.65)}.focus\:caret-red-100\/70:focus{caret-color:hsla(0,93%,94%,.7)}.focus\:caret-red-100\/75:focus{caret-color:hsla(0,93%,94%,.75)}.focus\:caret-red-100\/80:focus{caret-color:hsla(0,93%,94%,.8)}.focus\:caret-red-100\/85:focus{caret-color:hsla(0,93%,94%,.85)}.focus\:caret-red-100\/90:focus{caret-color:hsla(0,93%,94%,.9)}.focus\:caret-red-100\/95:focus{caret-color:hsla(0,93%,94%,.95)}.focus\:caret-red-200:focus{caret-color:#fecaca}.focus\:caret-red-200\/0:focus{caret-color:hsla(0,96%,89%,0)}.focus\:caret-red-200\/10:focus{caret-color:hsla(0,96%,89%,.1)}.focus\:caret-red-200\/100:focus{caret-color:#fecaca}.focus\:caret-red-200\/15:focus{caret-color:hsla(0,96%,89%,.15)}.focus\:caret-red-200\/20:focus{caret-color:hsla(0,96%,89%,.2)}.focus\:caret-red-200\/25:focus{caret-color:hsla(0,96%,89%,.25)}.focus\:caret-red-200\/30:focus{caret-color:hsla(0,96%,89%,.3)}.focus\:caret-red-200\/35:focus{caret-color:hsla(0,96%,89%,.35)}.focus\:caret-red-200\/40:focus{caret-color:hsla(0,96%,89%,.4)}.focus\:caret-red-200\/45:focus{caret-color:hsla(0,96%,89%,.45)}.focus\:caret-red-200\/5:focus{caret-color:hsla(0,96%,89%,.05)}.focus\:caret-red-200\/50:focus{caret-color:hsla(0,96%,89%,.5)}.focus\:caret-red-200\/55:focus{caret-color:hsla(0,96%,89%,.55)}.focus\:caret-red-200\/60:focus{caret-color:hsla(0,96%,89%,.6)}.focus\:caret-red-200\/65:focus{caret-color:hsla(0,96%,89%,.65)}.focus\:caret-red-200\/70:focus{caret-color:hsla(0,96%,89%,.7)}.focus\:caret-red-200\/75:focus{caret-color:hsla(0,96%,89%,.75)}.focus\:caret-red-200\/80:focus{caret-color:hsla(0,96%,89%,.8)}.focus\:caret-red-200\/85:focus{caret-color:hsla(0,96%,89%,.85)}.focus\:caret-red-200\/90:focus{caret-color:hsla(0,96%,89%,.9)}.focus\:caret-red-200\/95:focus{caret-color:hsla(0,96%,89%,.95)}.focus\:caret-red-400:focus{caret-color:#f87171}.focus\:caret-red-400\/0:focus{caret-color:hsla(0,91%,71%,0)}.focus\:caret-red-400\/10:focus{caret-color:hsla(0,91%,71%,.1)}.focus\:caret-red-400\/100:focus{caret-color:#f87171}.focus\:caret-red-400\/15:focus{caret-color:hsla(0,91%,71%,.15)}.focus\:caret-red-400\/20:focus{caret-color:hsla(0,91%,71%,.2)}.focus\:caret-red-400\/25:focus{caret-color:hsla(0,91%,71%,.25)}.focus\:caret-red-400\/30:focus{caret-color:hsla(0,91%,71%,.3)}.focus\:caret-red-400\/35:focus{caret-color:hsla(0,91%,71%,.35)}.focus\:caret-red-400\/40:focus{caret-color:hsla(0,91%,71%,.4)}.focus\:caret-red-400\/45:focus{caret-color:hsla(0,91%,71%,.45)}.focus\:caret-red-400\/5:focus{caret-color:hsla(0,91%,71%,.05)}.focus\:caret-red-400\/50:focus{caret-color:hsla(0,91%,71%,.5)}.focus\:caret-red-400\/55:focus{caret-color:hsla(0,91%,71%,.55)}.focus\:caret-red-400\/60:focus{caret-color:hsla(0,91%,71%,.6)}.focus\:caret-red-400\/65:focus{caret-color:hsla(0,91%,71%,.65)}.focus\:caret-red-400\/70:focus{caret-color:hsla(0,91%,71%,.7)}.focus\:caret-red-400\/75:focus{caret-color:hsla(0,91%,71%,.75)}.focus\:caret-red-400\/80:focus{caret-color:hsla(0,91%,71%,.8)}.focus\:caret-red-400\/85:focus{caret-color:hsla(0,91%,71%,.85)}.focus\:caret-red-400\/90:focus{caret-color:hsla(0,91%,71%,.9)}.focus\:caret-red-400\/95:focus{caret-color:hsla(0,91%,71%,.95)}.focus\:caret-red-50:focus{caret-color:#fef2f2}.focus\:caret-red-50\/0:focus{caret-color:hsla(0,86%,97%,0)}.focus\:caret-red-50\/10:focus{caret-color:hsla(0,86%,97%,.1)}.focus\:caret-red-50\/100:focus{caret-color:#fef2f2}.focus\:caret-red-50\/15:focus{caret-color:hsla(0,86%,97%,.15)}.focus\:caret-red-50\/20:focus{caret-color:hsla(0,86%,97%,.2)}.focus\:caret-red-50\/25:focus{caret-color:hsla(0,86%,97%,.25)}.focus\:caret-red-50\/30:focus{caret-color:hsla(0,86%,97%,.3)}.focus\:caret-red-50\/35:focus{caret-color:hsla(0,86%,97%,.35)}.focus\:caret-red-50\/40:focus{caret-color:hsla(0,86%,97%,.4)}.focus\:caret-red-50\/45:focus{caret-color:hsla(0,86%,97%,.45)}.focus\:caret-red-50\/5:focus{caret-color:hsla(0,86%,97%,.05)}.focus\:caret-red-50\/50:focus{caret-color:hsla(0,86%,97%,.5)}.focus\:caret-red-50\/55:focus{caret-color:hsla(0,86%,97%,.55)}.focus\:caret-red-50\/60:focus{caret-color:hsla(0,86%,97%,.6)}.focus\:caret-red-50\/65:focus{caret-color:hsla(0,86%,97%,.65)}.focus\:caret-red-50\/70:focus{caret-color:hsla(0,86%,97%,.7)}.focus\:caret-red-50\/75:focus{caret-color:hsla(0,86%,97%,.75)}.focus\:caret-red-50\/80:focus{caret-color:hsla(0,86%,97%,.8)}.focus\:caret-red-50\/85:focus{caret-color:hsla(0,86%,97%,.85)}.focus\:caret-red-50\/90:focus{caret-color:hsla(0,86%,97%,.9)}.focus\:caret-red-50\/95:focus{caret-color:hsla(0,86%,97%,.95)}.focus\:caret-red-500:focus{caret-color:#ef4444}.focus\:caret-red-500\/0:focus{caret-color:rgba(239,68,68,0)}.focus\:caret-red-500\/10:focus{caret-color:rgba(239,68,68,.1)}.focus\:caret-red-500\/100:focus{caret-color:#ef4444}.focus\:caret-red-500\/15:focus{caret-color:rgba(239,68,68,.15)}.focus\:caret-red-500\/20:focus{caret-color:rgba(239,68,68,.2)}.focus\:caret-red-500\/25:focus{caret-color:rgba(239,68,68,.25)}.focus\:caret-red-500\/30:focus{caret-color:rgba(239,68,68,.3)}.focus\:caret-red-500\/35:focus{caret-color:rgba(239,68,68,.35)}.focus\:caret-red-500\/40:focus{caret-color:rgba(239,68,68,.4)}.focus\:caret-red-500\/45:focus{caret-color:rgba(239,68,68,.45)}.focus\:caret-red-500\/5:focus{caret-color:rgba(239,68,68,.05)}.focus\:caret-red-500\/50:focus{caret-color:rgba(239,68,68,.5)}.focus\:caret-red-500\/55:focus{caret-color:rgba(239,68,68,.55)}.focus\:caret-red-500\/60:focus{caret-color:rgba(239,68,68,.6)}.focus\:caret-red-500\/65:focus{caret-color:rgba(239,68,68,.65)}.focus\:caret-red-500\/70:focus{caret-color:rgba(239,68,68,.7)}.focus\:caret-red-500\/75:focus{caret-color:rgba(239,68,68,.75)}.focus\:caret-red-500\/80:focus{caret-color:rgba(239,68,68,.8)}.focus\:caret-red-500\/85:focus{caret-color:rgba(239,68,68,.85)}.focus\:caret-red-500\/90:focus{caret-color:rgba(239,68,68,.9)}.focus\:caret-red-500\/95:focus{caret-color:rgba(239,68,68,.95)}.focus\:caret-red-950:focus{caret-color:#450a0a}.focus\:caret-red-950\/0:focus{caret-color:rgba(69,10,10,0)}.focus\:caret-red-950\/10:focus{caret-color:rgba(69,10,10,.1)}.focus\:caret-red-950\/100:focus{caret-color:#450a0a}.focus\:caret-red-950\/15:focus{caret-color:rgba(69,10,10,.15)}.focus\:caret-red-950\/20:focus{caret-color:rgba(69,10,10,.2)}.focus\:caret-red-950\/25:focus{caret-color:rgba(69,10,10,.25)}.focus\:caret-red-950\/30:focus{caret-color:rgba(69,10,10,.3)}.focus\:caret-red-950\/35:focus{caret-color:rgba(69,10,10,.35)}.focus\:caret-red-950\/40:focus{caret-color:rgba(69,10,10,.4)}.focus\:caret-red-950\/45:focus{caret-color:rgba(69,10,10,.45)}.focus\:caret-red-950\/5:focus{caret-color:rgba(69,10,10,.05)}.focus\:caret-red-950\/50:focus{caret-color:rgba(69,10,10,.5)}.focus\:caret-red-950\/55:focus{caret-color:rgba(69,10,10,.55)}.focus\:caret-red-950\/60:focus{caret-color:rgba(69,10,10,.6)}.focus\:caret-red-950\/65:focus{caret-color:rgba(69,10,10,.65)}.focus\:caret-red-950\/70:focus{caret-color:rgba(69,10,10,.7)}.focus\:caret-red-950\/75:focus{caret-color:rgba(69,10,10,.75)}.focus\:caret-red-950\/80:focus{caret-color:rgba(69,10,10,.8)}.focus\:caret-red-950\/85:focus{caret-color:rgba(69,10,10,.85)}.focus\:caret-red-950\/90:focus{caret-color:rgba(69,10,10,.9)}.focus\:caret-red-950\/95:focus{caret-color:rgba(69,10,10,.95)}.focus\:caret-rose-100:focus{caret-color:#ffe4e6}.focus\:caret-rose-100\/0:focus{caret-color:rgba(255,228,230,0)}.focus\:caret-rose-100\/10:focus{caret-color:rgba(255,228,230,.1)}.focus\:caret-rose-100\/100:focus{caret-color:#ffe4e6}.focus\:caret-rose-100\/15:focus{caret-color:rgba(255,228,230,.15)}.focus\:caret-rose-100\/20:focus{caret-color:rgba(255,228,230,.2)}.focus\:caret-rose-100\/25:focus{caret-color:rgba(255,228,230,.25)}.focus\:caret-rose-100\/30:focus{caret-color:rgba(255,228,230,.3)}.focus\:caret-rose-100\/35:focus{caret-color:rgba(255,228,230,.35)}.focus\:caret-rose-100\/40:focus{caret-color:rgba(255,228,230,.4)}.focus\:caret-rose-100\/45:focus{caret-color:rgba(255,228,230,.45)}.focus\:caret-rose-100\/5:focus{caret-color:rgba(255,228,230,.05)}.focus\:caret-rose-100\/50:focus{caret-color:rgba(255,228,230,.5)}.focus\:caret-rose-100\/55:focus{caret-color:rgba(255,228,230,.55)}.focus\:caret-rose-100\/60:focus{caret-color:rgba(255,228,230,.6)}.focus\:caret-rose-100\/65:focus{caret-color:rgba(255,228,230,.65)}.focus\:caret-rose-100\/70:focus{caret-color:rgba(255,228,230,.7)}.focus\:caret-rose-100\/75:focus{caret-color:rgba(255,228,230,.75)}.focus\:caret-rose-100\/80:focus{caret-color:rgba(255,228,230,.8)}.focus\:caret-rose-100\/85:focus{caret-color:rgba(255,228,230,.85)}.focus\:caret-rose-100\/90:focus{caret-color:rgba(255,228,230,.9)}.focus\:caret-rose-100\/95:focus{caret-color:rgba(255,228,230,.95)}.focus\:caret-rose-200:focus{caret-color:#fecdd3}.focus\:caret-rose-200\/0:focus{caret-color:rgba(254,205,211,0)}.focus\:caret-rose-200\/10:focus{caret-color:rgba(254,205,211,.1)}.focus\:caret-rose-200\/100:focus{caret-color:#fecdd3}.focus\:caret-rose-200\/15:focus{caret-color:rgba(254,205,211,.15)}.focus\:caret-rose-200\/20:focus{caret-color:rgba(254,205,211,.2)}.focus\:caret-rose-200\/25:focus{caret-color:rgba(254,205,211,.25)}.focus\:caret-rose-200\/30:focus{caret-color:rgba(254,205,211,.3)}.focus\:caret-rose-200\/35:focus{caret-color:rgba(254,205,211,.35)}.focus\:caret-rose-200\/40:focus{caret-color:rgba(254,205,211,.4)}.focus\:caret-rose-200\/45:focus{caret-color:rgba(254,205,211,.45)}.focus\:caret-rose-200\/5:focus{caret-color:rgba(254,205,211,.05)}.focus\:caret-rose-200\/50:focus{caret-color:rgba(254,205,211,.5)}.focus\:caret-rose-200\/55:focus{caret-color:rgba(254,205,211,.55)}.focus\:caret-rose-200\/60:focus{caret-color:rgba(254,205,211,.6)}.focus\:caret-rose-200\/65:focus{caret-color:rgba(254,205,211,.65)}.focus\:caret-rose-200\/70:focus{caret-color:rgba(254,205,211,.7)}.focus\:caret-rose-200\/75:focus{caret-color:rgba(254,205,211,.75)}.focus\:caret-rose-200\/80:focus{caret-color:rgba(254,205,211,.8)}.focus\:caret-rose-200\/85:focus{caret-color:rgba(254,205,211,.85)}.focus\:caret-rose-200\/90:focus{caret-color:rgba(254,205,211,.9)}.focus\:caret-rose-200\/95:focus{caret-color:rgba(254,205,211,.95)}.focus\:caret-rose-400:focus{caret-color:#fb7185}.focus\:caret-rose-400\/0:focus{caret-color:rgba(251,113,133,0)}.focus\:caret-rose-400\/10:focus{caret-color:rgba(251,113,133,.1)}.focus\:caret-rose-400\/100:focus{caret-color:#fb7185}.focus\:caret-rose-400\/15:focus{caret-color:rgba(251,113,133,.15)}.focus\:caret-rose-400\/20:focus{caret-color:rgba(251,113,133,.2)}.focus\:caret-rose-400\/25:focus{caret-color:rgba(251,113,133,.25)}.focus\:caret-rose-400\/30:focus{caret-color:rgba(251,113,133,.3)}.focus\:caret-rose-400\/35:focus{caret-color:rgba(251,113,133,.35)}.focus\:caret-rose-400\/40:focus{caret-color:rgba(251,113,133,.4)}.focus\:caret-rose-400\/45:focus{caret-color:rgba(251,113,133,.45)}.focus\:caret-rose-400\/5:focus{caret-color:rgba(251,113,133,.05)}.focus\:caret-rose-400\/50:focus{caret-color:rgba(251,113,133,.5)}.focus\:caret-rose-400\/55:focus{caret-color:rgba(251,113,133,.55)}.focus\:caret-rose-400\/60:focus{caret-color:rgba(251,113,133,.6)}.focus\:caret-rose-400\/65:focus{caret-color:rgba(251,113,133,.65)}.focus\:caret-rose-400\/70:focus{caret-color:rgba(251,113,133,.7)}.focus\:caret-rose-400\/75:focus{caret-color:rgba(251,113,133,.75)}.focus\:caret-rose-400\/80:focus{caret-color:rgba(251,113,133,.8)}.focus\:caret-rose-400\/85:focus{caret-color:rgba(251,113,133,.85)}.focus\:caret-rose-400\/90:focus{caret-color:rgba(251,113,133,.9)}.focus\:caret-rose-400\/95:focus{caret-color:rgba(251,113,133,.95)}.focus\:caret-rose-50:focus{caret-color:#fff1f2}.focus\:caret-rose-50\/0:focus{caret-color:rgba(255,241,242,0)}.focus\:caret-rose-50\/10:focus{caret-color:rgba(255,241,242,.1)}.focus\:caret-rose-50\/100:focus{caret-color:#fff1f2}.focus\:caret-rose-50\/15:focus{caret-color:rgba(255,241,242,.15)}.focus\:caret-rose-50\/20:focus{caret-color:rgba(255,241,242,.2)}.focus\:caret-rose-50\/25:focus{caret-color:rgba(255,241,242,.25)}.focus\:caret-rose-50\/30:focus{caret-color:rgba(255,241,242,.3)}.focus\:caret-rose-50\/35:focus{caret-color:rgba(255,241,242,.35)}.focus\:caret-rose-50\/40:focus{caret-color:rgba(255,241,242,.4)}.focus\:caret-rose-50\/45:focus{caret-color:rgba(255,241,242,.45)}.focus\:caret-rose-50\/5:focus{caret-color:rgba(255,241,242,.05)}.focus\:caret-rose-50\/50:focus{caret-color:rgba(255,241,242,.5)}.focus\:caret-rose-50\/55:focus{caret-color:rgba(255,241,242,.55)}.focus\:caret-rose-50\/60:focus{caret-color:rgba(255,241,242,.6)}.focus\:caret-rose-50\/65:focus{caret-color:rgba(255,241,242,.65)}.focus\:caret-rose-50\/70:focus{caret-color:rgba(255,241,242,.7)}.focus\:caret-rose-50\/75:focus{caret-color:rgba(255,241,242,.75)}.focus\:caret-rose-50\/80:focus{caret-color:rgba(255,241,242,.8)}.focus\:caret-rose-50\/85:focus{caret-color:rgba(255,241,242,.85)}.focus\:caret-rose-50\/90:focus{caret-color:rgba(255,241,242,.9)}.focus\:caret-rose-50\/95:focus{caret-color:rgba(255,241,242,.95)}.focus\:caret-rose-500:focus{caret-color:#f43f5e}.focus\:caret-rose-500\/0:focus{caret-color:rgba(244,63,94,0)}.focus\:caret-rose-500\/10:focus{caret-color:rgba(244,63,94,.1)}.focus\:caret-rose-500\/100:focus{caret-color:#f43f5e}.focus\:caret-rose-500\/15:focus{caret-color:rgba(244,63,94,.15)}.focus\:caret-rose-500\/20:focus{caret-color:rgba(244,63,94,.2)}.focus\:caret-rose-500\/25:focus{caret-color:rgba(244,63,94,.25)}.focus\:caret-rose-500\/30:focus{caret-color:rgba(244,63,94,.3)}.focus\:caret-rose-500\/35:focus{caret-color:rgba(244,63,94,.35)}.focus\:caret-rose-500\/40:focus{caret-color:rgba(244,63,94,.4)}.focus\:caret-rose-500\/45:focus{caret-color:rgba(244,63,94,.45)}.focus\:caret-rose-500\/5:focus{caret-color:rgba(244,63,94,.05)}.focus\:caret-rose-500\/50:focus{caret-color:rgba(244,63,94,.5)}.focus\:caret-rose-500\/55:focus{caret-color:rgba(244,63,94,.55)}.focus\:caret-rose-500\/60:focus{caret-color:rgba(244,63,94,.6)}.focus\:caret-rose-500\/65:focus{caret-color:rgba(244,63,94,.65)}.focus\:caret-rose-500\/70:focus{caret-color:rgba(244,63,94,.7)}.focus\:caret-rose-500\/75:focus{caret-color:rgba(244,63,94,.75)}.focus\:caret-rose-500\/80:focus{caret-color:rgba(244,63,94,.8)}.focus\:caret-rose-500\/85:focus{caret-color:rgba(244,63,94,.85)}.focus\:caret-rose-500\/90:focus{caret-color:rgba(244,63,94,.9)}.focus\:caret-rose-500\/95:focus{caret-color:rgba(244,63,94,.95)}.focus\:caret-rose-950:focus{caret-color:#4c0519}.focus\:caret-rose-950\/0:focus{caret-color:rgba(76,5,25,0)}.focus\:caret-rose-950\/10:focus{caret-color:rgba(76,5,25,.1)}.focus\:caret-rose-950\/100:focus{caret-color:#4c0519}.focus\:caret-rose-950\/15:focus{caret-color:rgba(76,5,25,.15)}.focus\:caret-rose-950\/20:focus{caret-color:rgba(76,5,25,.2)}.focus\:caret-rose-950\/25:focus{caret-color:rgba(76,5,25,.25)}.focus\:caret-rose-950\/30:focus{caret-color:rgba(76,5,25,.3)}.focus\:caret-rose-950\/35:focus{caret-color:rgba(76,5,25,.35)}.focus\:caret-rose-950\/40:focus{caret-color:rgba(76,5,25,.4)}.focus\:caret-rose-950\/45:focus{caret-color:rgba(76,5,25,.45)}.focus\:caret-rose-950\/5:focus{caret-color:rgba(76,5,25,.05)}.focus\:caret-rose-950\/50:focus{caret-color:rgba(76,5,25,.5)}.focus\:caret-rose-950\/55:focus{caret-color:rgba(76,5,25,.55)}.focus\:caret-rose-950\/60:focus{caret-color:rgba(76,5,25,.6)}.focus\:caret-rose-950\/65:focus{caret-color:rgba(76,5,25,.65)}.focus\:caret-rose-950\/70:focus{caret-color:rgba(76,5,25,.7)}.focus\:caret-rose-950\/75:focus{caret-color:rgba(76,5,25,.75)}.focus\:caret-rose-950\/80:focus{caret-color:rgba(76,5,25,.8)}.focus\:caret-rose-950\/85:focus{caret-color:rgba(76,5,25,.85)}.focus\:caret-rose-950\/90:focus{caret-color:rgba(76,5,25,.9)}.focus\:caret-rose-950\/95:focus{caret-color:rgba(76,5,25,.95)}.focus\:caret-sky-100:focus{caret-color:#e0f2fe}.focus\:caret-sky-100\/0:focus{caret-color:rgba(224,242,254,0)}.focus\:caret-sky-100\/10:focus{caret-color:rgba(224,242,254,.1)}.focus\:caret-sky-100\/100:focus{caret-color:#e0f2fe}.focus\:caret-sky-100\/15:focus{caret-color:rgba(224,242,254,.15)}.focus\:caret-sky-100\/20:focus{caret-color:rgba(224,242,254,.2)}.focus\:caret-sky-100\/25:focus{caret-color:rgba(224,242,254,.25)}.focus\:caret-sky-100\/30:focus{caret-color:rgba(224,242,254,.3)}.focus\:caret-sky-100\/35:focus{caret-color:rgba(224,242,254,.35)}.focus\:caret-sky-100\/40:focus{caret-color:rgba(224,242,254,.4)}.focus\:caret-sky-100\/45:focus{caret-color:rgba(224,242,254,.45)}.focus\:caret-sky-100\/5:focus{caret-color:rgba(224,242,254,.05)}.focus\:caret-sky-100\/50:focus{caret-color:rgba(224,242,254,.5)}.focus\:caret-sky-100\/55:focus{caret-color:rgba(224,242,254,.55)}.focus\:caret-sky-100\/60:focus{caret-color:rgba(224,242,254,.6)}.focus\:caret-sky-100\/65:focus{caret-color:rgba(224,242,254,.65)}.focus\:caret-sky-100\/70:focus{caret-color:rgba(224,242,254,.7)}.focus\:caret-sky-100\/75:focus{caret-color:rgba(224,242,254,.75)}.focus\:caret-sky-100\/80:focus{caret-color:rgba(224,242,254,.8)}.focus\:caret-sky-100\/85:focus{caret-color:rgba(224,242,254,.85)}.focus\:caret-sky-100\/90:focus{caret-color:rgba(224,242,254,.9)}.focus\:caret-sky-100\/95:focus{caret-color:rgba(224,242,254,.95)}.focus\:caret-sky-200:focus{caret-color:#bae6fd}.focus\:caret-sky-200\/0:focus{caret-color:rgba(186,230,253,0)}.focus\:caret-sky-200\/10:focus{caret-color:rgba(186,230,253,.1)}.focus\:caret-sky-200\/100:focus{caret-color:#bae6fd}.focus\:caret-sky-200\/15:focus{caret-color:rgba(186,230,253,.15)}.focus\:caret-sky-200\/20:focus{caret-color:rgba(186,230,253,.2)}.focus\:caret-sky-200\/25:focus{caret-color:rgba(186,230,253,.25)}.focus\:caret-sky-200\/30:focus{caret-color:rgba(186,230,253,.3)}.focus\:caret-sky-200\/35:focus{caret-color:rgba(186,230,253,.35)}.focus\:caret-sky-200\/40:focus{caret-color:rgba(186,230,253,.4)}.focus\:caret-sky-200\/45:focus{caret-color:rgba(186,230,253,.45)}.focus\:caret-sky-200\/5:focus{caret-color:rgba(186,230,253,.05)}.focus\:caret-sky-200\/50:focus{caret-color:rgba(186,230,253,.5)}.focus\:caret-sky-200\/55:focus{caret-color:rgba(186,230,253,.55)}.focus\:caret-sky-200\/60:focus{caret-color:rgba(186,230,253,.6)}.focus\:caret-sky-200\/65:focus{caret-color:rgba(186,230,253,.65)}.focus\:caret-sky-200\/70:focus{caret-color:rgba(186,230,253,.7)}.focus\:caret-sky-200\/75:focus{caret-color:rgba(186,230,253,.75)}.focus\:caret-sky-200\/80:focus{caret-color:rgba(186,230,253,.8)}.focus\:caret-sky-200\/85:focus{caret-color:rgba(186,230,253,.85)}.focus\:caret-sky-200\/90:focus{caret-color:rgba(186,230,253,.9)}.focus\:caret-sky-200\/95:focus{caret-color:rgba(186,230,253,.95)}.focus\:caret-sky-400:focus{caret-color:#38bdf8}.focus\:caret-sky-400\/0:focus{caret-color:rgba(56,189,248,0)}.focus\:caret-sky-400\/10:focus{caret-color:rgba(56,189,248,.1)}.focus\:caret-sky-400\/100:focus{caret-color:#38bdf8}.focus\:caret-sky-400\/15:focus{caret-color:rgba(56,189,248,.15)}.focus\:caret-sky-400\/20:focus{caret-color:rgba(56,189,248,.2)}.focus\:caret-sky-400\/25:focus{caret-color:rgba(56,189,248,.25)}.focus\:caret-sky-400\/30:focus{caret-color:rgba(56,189,248,.3)}.focus\:caret-sky-400\/35:focus{caret-color:rgba(56,189,248,.35)}.focus\:caret-sky-400\/40:focus{caret-color:rgba(56,189,248,.4)}.focus\:caret-sky-400\/45:focus{caret-color:rgba(56,189,248,.45)}.focus\:caret-sky-400\/5:focus{caret-color:rgba(56,189,248,.05)}.focus\:caret-sky-400\/50:focus{caret-color:rgba(56,189,248,.5)}.focus\:caret-sky-400\/55:focus{caret-color:rgba(56,189,248,.55)}.focus\:caret-sky-400\/60:focus{caret-color:rgba(56,189,248,.6)}.focus\:caret-sky-400\/65:focus{caret-color:rgba(56,189,248,.65)}.focus\:caret-sky-400\/70:focus{caret-color:rgba(56,189,248,.7)}.focus\:caret-sky-400\/75:focus{caret-color:rgba(56,189,248,.75)}.focus\:caret-sky-400\/80:focus{caret-color:rgba(56,189,248,.8)}.focus\:caret-sky-400\/85:focus{caret-color:rgba(56,189,248,.85)}.focus\:caret-sky-400\/90:focus{caret-color:rgba(56,189,248,.9)}.focus\:caret-sky-400\/95:focus{caret-color:rgba(56,189,248,.95)}.focus\:caret-sky-50:focus{caret-color:#f0f9ff}.focus\:caret-sky-50\/0:focus{caret-color:rgba(240,249,255,0)}.focus\:caret-sky-50\/10:focus{caret-color:rgba(240,249,255,.1)}.focus\:caret-sky-50\/100:focus{caret-color:#f0f9ff}.focus\:caret-sky-50\/15:focus{caret-color:rgba(240,249,255,.15)}.focus\:caret-sky-50\/20:focus{caret-color:rgba(240,249,255,.2)}.focus\:caret-sky-50\/25:focus{caret-color:rgba(240,249,255,.25)}.focus\:caret-sky-50\/30:focus{caret-color:rgba(240,249,255,.3)}.focus\:caret-sky-50\/35:focus{caret-color:rgba(240,249,255,.35)}.focus\:caret-sky-50\/40:focus{caret-color:rgba(240,249,255,.4)}.focus\:caret-sky-50\/45:focus{caret-color:rgba(240,249,255,.45)}.focus\:caret-sky-50\/5:focus{caret-color:rgba(240,249,255,.05)}.focus\:caret-sky-50\/50:focus{caret-color:rgba(240,249,255,.5)}.focus\:caret-sky-50\/55:focus{caret-color:rgba(240,249,255,.55)}.focus\:caret-sky-50\/60:focus{caret-color:rgba(240,249,255,.6)}.focus\:caret-sky-50\/65:focus{caret-color:rgba(240,249,255,.65)}.focus\:caret-sky-50\/70:focus{caret-color:rgba(240,249,255,.7)}.focus\:caret-sky-50\/75:focus{caret-color:rgba(240,249,255,.75)}.focus\:caret-sky-50\/80:focus{caret-color:rgba(240,249,255,.8)}.focus\:caret-sky-50\/85:focus{caret-color:rgba(240,249,255,.85)}.focus\:caret-sky-50\/90:focus{caret-color:rgba(240,249,255,.9)}.focus\:caret-sky-50\/95:focus{caret-color:rgba(240,249,255,.95)}.focus\:caret-sky-500:focus{caret-color:#0ea5e9}.focus\:caret-sky-500\/0:focus{caret-color:rgba(14,165,233,0)}.focus\:caret-sky-500\/10:focus{caret-color:rgba(14,165,233,.1)}.focus\:caret-sky-500\/100:focus{caret-color:#0ea5e9}.focus\:caret-sky-500\/15:focus{caret-color:rgba(14,165,233,.15)}.focus\:caret-sky-500\/20:focus{caret-color:rgba(14,165,233,.2)}.focus\:caret-sky-500\/25:focus{caret-color:rgba(14,165,233,.25)}.focus\:caret-sky-500\/30:focus{caret-color:rgba(14,165,233,.3)}.focus\:caret-sky-500\/35:focus{caret-color:rgba(14,165,233,.35)}.focus\:caret-sky-500\/40:focus{caret-color:rgba(14,165,233,.4)}.focus\:caret-sky-500\/45:focus{caret-color:rgba(14,165,233,.45)}.focus\:caret-sky-500\/5:focus{caret-color:rgba(14,165,233,.05)}.focus\:caret-sky-500\/50:focus{caret-color:rgba(14,165,233,.5)}.focus\:caret-sky-500\/55:focus{caret-color:rgba(14,165,233,.55)}.focus\:caret-sky-500\/60:focus{caret-color:rgba(14,165,233,.6)}.focus\:caret-sky-500\/65:focus{caret-color:rgba(14,165,233,.65)}.focus\:caret-sky-500\/70:focus{caret-color:rgba(14,165,233,.7)}.focus\:caret-sky-500\/75:focus{caret-color:rgba(14,165,233,.75)}.focus\:caret-sky-500\/80:focus{caret-color:rgba(14,165,233,.8)}.focus\:caret-sky-500\/85:focus{caret-color:rgba(14,165,233,.85)}.focus\:caret-sky-500\/90:focus{caret-color:rgba(14,165,233,.9)}.focus\:caret-sky-500\/95:focus{caret-color:rgba(14,165,233,.95)}.focus\:caret-sky-950:focus{caret-color:#082f49}.focus\:caret-sky-950\/0:focus{caret-color:rgba(8,47,73,0)}.focus\:caret-sky-950\/10:focus{caret-color:rgba(8,47,73,.1)}.focus\:caret-sky-950\/100:focus{caret-color:#082f49}.focus\:caret-sky-950\/15:focus{caret-color:rgba(8,47,73,.15)}.focus\:caret-sky-950\/20:focus{caret-color:rgba(8,47,73,.2)}.focus\:caret-sky-950\/25:focus{caret-color:rgba(8,47,73,.25)}.focus\:caret-sky-950\/30:focus{caret-color:rgba(8,47,73,.3)}.focus\:caret-sky-950\/35:focus{caret-color:rgba(8,47,73,.35)}.focus\:caret-sky-950\/40:focus{caret-color:rgba(8,47,73,.4)}.focus\:caret-sky-950\/45:focus{caret-color:rgba(8,47,73,.45)}.focus\:caret-sky-950\/5:focus{caret-color:rgba(8,47,73,.05)}.focus\:caret-sky-950\/50:focus{caret-color:rgba(8,47,73,.5)}.focus\:caret-sky-950\/55:focus{caret-color:rgba(8,47,73,.55)}.focus\:caret-sky-950\/60:focus{caret-color:rgba(8,47,73,.6)}.focus\:caret-sky-950\/65:focus{caret-color:rgba(8,47,73,.65)}.focus\:caret-sky-950\/70:focus{caret-color:rgba(8,47,73,.7)}.focus\:caret-sky-950\/75:focus{caret-color:rgba(8,47,73,.75)}.focus\:caret-sky-950\/80:focus{caret-color:rgba(8,47,73,.8)}.focus\:caret-sky-950\/85:focus{caret-color:rgba(8,47,73,.85)}.focus\:caret-sky-950\/90:focus{caret-color:rgba(8,47,73,.9)}.focus\:caret-sky-950\/95:focus{caret-color:rgba(8,47,73,.95)}.focus\:caret-slate-100:focus{caret-color:#f1f5f9}.focus\:caret-slate-100\/0:focus{caret-color:rgba(241,245,249,0)}.focus\:caret-slate-100\/10:focus{caret-color:rgba(241,245,249,.1)}.focus\:caret-slate-100\/100:focus{caret-color:#f1f5f9}.focus\:caret-slate-100\/15:focus{caret-color:rgba(241,245,249,.15)}.focus\:caret-slate-100\/20:focus{caret-color:rgba(241,245,249,.2)}.focus\:caret-slate-100\/25:focus{caret-color:rgba(241,245,249,.25)}.focus\:caret-slate-100\/30:focus{caret-color:rgba(241,245,249,.3)}.focus\:caret-slate-100\/35:focus{caret-color:rgba(241,245,249,.35)}.focus\:caret-slate-100\/40:focus{caret-color:rgba(241,245,249,.4)}.focus\:caret-slate-100\/45:focus{caret-color:rgba(241,245,249,.45)}.focus\:caret-slate-100\/5:focus{caret-color:rgba(241,245,249,.05)}.focus\:caret-slate-100\/50:focus{caret-color:rgba(241,245,249,.5)}.focus\:caret-slate-100\/55:focus{caret-color:rgba(241,245,249,.55)}.focus\:caret-slate-100\/60:focus{caret-color:rgba(241,245,249,.6)}.focus\:caret-slate-100\/65:focus{caret-color:rgba(241,245,249,.65)}.focus\:caret-slate-100\/70:focus{caret-color:rgba(241,245,249,.7)}.focus\:caret-slate-100\/75:focus{caret-color:rgba(241,245,249,.75)}.focus\:caret-slate-100\/80:focus{caret-color:rgba(241,245,249,.8)}.focus\:caret-slate-100\/85:focus{caret-color:rgba(241,245,249,.85)}.focus\:caret-slate-100\/90:focus{caret-color:rgba(241,245,249,.9)}.focus\:caret-slate-100\/95:focus{caret-color:rgba(241,245,249,.95)}.focus\:caret-slate-200:focus{caret-color:#e2e8f0}.focus\:caret-slate-200\/0:focus{caret-color:rgba(226,232,240,0)}.focus\:caret-slate-200\/10:focus{caret-color:rgba(226,232,240,.1)}.focus\:caret-slate-200\/100:focus{caret-color:#e2e8f0}.focus\:caret-slate-200\/15:focus{caret-color:rgba(226,232,240,.15)}.focus\:caret-slate-200\/20:focus{caret-color:rgba(226,232,240,.2)}.focus\:caret-slate-200\/25:focus{caret-color:rgba(226,232,240,.25)}.focus\:caret-slate-200\/30:focus{caret-color:rgba(226,232,240,.3)}.focus\:caret-slate-200\/35:focus{caret-color:rgba(226,232,240,.35)}.focus\:caret-slate-200\/40:focus{caret-color:rgba(226,232,240,.4)}.focus\:caret-slate-200\/45:focus{caret-color:rgba(226,232,240,.45)}.focus\:caret-slate-200\/5:focus{caret-color:rgba(226,232,240,.05)}.focus\:caret-slate-200\/50:focus{caret-color:rgba(226,232,240,.5)}.focus\:caret-slate-200\/55:focus{caret-color:rgba(226,232,240,.55)}.focus\:caret-slate-200\/60:focus{caret-color:rgba(226,232,240,.6)}.focus\:caret-slate-200\/65:focus{caret-color:rgba(226,232,240,.65)}.focus\:caret-slate-200\/70:focus{caret-color:rgba(226,232,240,.7)}.focus\:caret-slate-200\/75:focus{caret-color:rgba(226,232,240,.75)}.focus\:caret-slate-200\/80:focus{caret-color:rgba(226,232,240,.8)}.focus\:caret-slate-200\/85:focus{caret-color:rgba(226,232,240,.85)}.focus\:caret-slate-200\/90:focus{caret-color:rgba(226,232,240,.9)}.focus\:caret-slate-200\/95:focus{caret-color:rgba(226,232,240,.95)}.focus\:caret-slate-400:focus{caret-color:#94a3b8}.focus\:caret-slate-400\/0:focus{caret-color:rgba(148,163,184,0)}.focus\:caret-slate-400\/10:focus{caret-color:rgba(148,163,184,.1)}.focus\:caret-slate-400\/100:focus{caret-color:#94a3b8}.focus\:caret-slate-400\/15:focus{caret-color:rgba(148,163,184,.15)}.focus\:caret-slate-400\/20:focus{caret-color:rgba(148,163,184,.2)}.focus\:caret-slate-400\/25:focus{caret-color:rgba(148,163,184,.25)}.focus\:caret-slate-400\/30:focus{caret-color:rgba(148,163,184,.3)}.focus\:caret-slate-400\/35:focus{caret-color:rgba(148,163,184,.35)}.focus\:caret-slate-400\/40:focus{caret-color:rgba(148,163,184,.4)}.focus\:caret-slate-400\/45:focus{caret-color:rgba(148,163,184,.45)}.focus\:caret-slate-400\/5:focus{caret-color:rgba(148,163,184,.05)}.focus\:caret-slate-400\/50:focus{caret-color:rgba(148,163,184,.5)}.focus\:caret-slate-400\/55:focus{caret-color:rgba(148,163,184,.55)}.focus\:caret-slate-400\/60:focus{caret-color:rgba(148,163,184,.6)}.focus\:caret-slate-400\/65:focus{caret-color:rgba(148,163,184,.65)}.focus\:caret-slate-400\/70:focus{caret-color:rgba(148,163,184,.7)}.focus\:caret-slate-400\/75:focus{caret-color:rgba(148,163,184,.75)}.focus\:caret-slate-400\/80:focus{caret-color:rgba(148,163,184,.8)}.focus\:caret-slate-400\/85:focus{caret-color:rgba(148,163,184,.85)}.focus\:caret-slate-400\/90:focus{caret-color:rgba(148,163,184,.9)}.focus\:caret-slate-400\/95:focus{caret-color:rgba(148,163,184,.95)}.focus\:caret-slate-50:focus{caret-color:#f8fafc}.focus\:caret-slate-50\/0:focus{caret-color:rgba(248,250,252,0)}.focus\:caret-slate-50\/10:focus{caret-color:rgba(248,250,252,.1)}.focus\:caret-slate-50\/100:focus{caret-color:#f8fafc}.focus\:caret-slate-50\/15:focus{caret-color:rgba(248,250,252,.15)}.focus\:caret-slate-50\/20:focus{caret-color:rgba(248,250,252,.2)}.focus\:caret-slate-50\/25:focus{caret-color:rgba(248,250,252,.25)}.focus\:caret-slate-50\/30:focus{caret-color:rgba(248,250,252,.3)}.focus\:caret-slate-50\/35:focus{caret-color:rgba(248,250,252,.35)}.focus\:caret-slate-50\/40:focus{caret-color:rgba(248,250,252,.4)}.focus\:caret-slate-50\/45:focus{caret-color:rgba(248,250,252,.45)}.focus\:caret-slate-50\/5:focus{caret-color:rgba(248,250,252,.05)}.focus\:caret-slate-50\/50:focus{caret-color:rgba(248,250,252,.5)}.focus\:caret-slate-50\/55:focus{caret-color:rgba(248,250,252,.55)}.focus\:caret-slate-50\/60:focus{caret-color:rgba(248,250,252,.6)}.focus\:caret-slate-50\/65:focus{caret-color:rgba(248,250,252,.65)}.focus\:caret-slate-50\/70:focus{caret-color:rgba(248,250,252,.7)}.focus\:caret-slate-50\/75:focus{caret-color:rgba(248,250,252,.75)}.focus\:caret-slate-50\/80:focus{caret-color:rgba(248,250,252,.8)}.focus\:caret-slate-50\/85:focus{caret-color:rgba(248,250,252,.85)}.focus\:caret-slate-50\/90:focus{caret-color:rgba(248,250,252,.9)}.focus\:caret-slate-50\/95:focus{caret-color:rgba(248,250,252,.95)}.focus\:caret-slate-500:focus{caret-color:#64748b}.focus\:caret-slate-500\/0:focus{caret-color:rgba(100,116,139,0)}.focus\:caret-slate-500\/10:focus{caret-color:rgba(100,116,139,.1)}.focus\:caret-slate-500\/100:focus{caret-color:#64748b}.focus\:caret-slate-500\/15:focus{caret-color:rgba(100,116,139,.15)}.focus\:caret-slate-500\/20:focus{caret-color:rgba(100,116,139,.2)}.focus\:caret-slate-500\/25:focus{caret-color:rgba(100,116,139,.25)}.focus\:caret-slate-500\/30:focus{caret-color:rgba(100,116,139,.3)}.focus\:caret-slate-500\/35:focus{caret-color:rgba(100,116,139,.35)}.focus\:caret-slate-500\/40:focus{caret-color:rgba(100,116,139,.4)}.focus\:caret-slate-500\/45:focus{caret-color:rgba(100,116,139,.45)}.focus\:caret-slate-500\/5:focus{caret-color:rgba(100,116,139,.05)}.focus\:caret-slate-500\/50:focus{caret-color:rgba(100,116,139,.5)}.focus\:caret-slate-500\/55:focus{caret-color:rgba(100,116,139,.55)}.focus\:caret-slate-500\/60:focus{caret-color:rgba(100,116,139,.6)}.focus\:caret-slate-500\/65:focus{caret-color:rgba(100,116,139,.65)}.focus\:caret-slate-500\/70:focus{caret-color:rgba(100,116,139,.7)}.focus\:caret-slate-500\/75:focus{caret-color:rgba(100,116,139,.75)}.focus\:caret-slate-500\/80:focus{caret-color:rgba(100,116,139,.8)}.focus\:caret-slate-500\/85:focus{caret-color:rgba(100,116,139,.85)}.focus\:caret-slate-500\/90:focus{caret-color:rgba(100,116,139,.9)}.focus\:caret-slate-500\/95:focus{caret-color:rgba(100,116,139,.95)}.focus\:caret-slate-950:focus{caret-color:#020617}.focus\:caret-slate-950\/0:focus{caret-color:rgba(2,6,23,0)}.focus\:caret-slate-950\/10:focus{caret-color:rgba(2,6,23,.1)}.focus\:caret-slate-950\/100:focus{caret-color:#020617}.focus\:caret-slate-950\/15:focus{caret-color:rgba(2,6,23,.15)}.focus\:caret-slate-950\/20:focus{caret-color:rgba(2,6,23,.2)}.focus\:caret-slate-950\/25:focus{caret-color:rgba(2,6,23,.25)}.focus\:caret-slate-950\/30:focus{caret-color:rgba(2,6,23,.3)}.focus\:caret-slate-950\/35:focus{caret-color:rgba(2,6,23,.35)}.focus\:caret-slate-950\/40:focus{caret-color:rgba(2,6,23,.4)}.focus\:caret-slate-950\/45:focus{caret-color:rgba(2,6,23,.45)}.focus\:caret-slate-950\/5:focus{caret-color:rgba(2,6,23,.05)}.focus\:caret-slate-950\/50:focus{caret-color:rgba(2,6,23,.5)}.focus\:caret-slate-950\/55:focus{caret-color:rgba(2,6,23,.55)}.focus\:caret-slate-950\/60:focus{caret-color:rgba(2,6,23,.6)}.focus\:caret-slate-950\/65:focus{caret-color:rgba(2,6,23,.65)}.focus\:caret-slate-950\/70:focus{caret-color:rgba(2,6,23,.7)}.focus\:caret-slate-950\/75:focus{caret-color:rgba(2,6,23,.75)}.focus\:caret-slate-950\/80:focus{caret-color:rgba(2,6,23,.8)}.focus\:caret-slate-950\/85:focus{caret-color:rgba(2,6,23,.85)}.focus\:caret-slate-950\/90:focus{caret-color:rgba(2,6,23,.9)}.focus\:caret-slate-950\/95:focus{caret-color:rgba(2,6,23,.95)}.focus\:caret-stone-100:focus{caret-color:#f5f5f4}.focus\:caret-stone-100\/0:focus{caret-color:hsla(60,5%,96%,0)}.focus\:caret-stone-100\/10:focus{caret-color:hsla(60,5%,96%,.1)}.focus\:caret-stone-100\/100:focus{caret-color:#f5f5f4}.focus\:caret-stone-100\/15:focus{caret-color:hsla(60,5%,96%,.15)}.focus\:caret-stone-100\/20:focus{caret-color:hsla(60,5%,96%,.2)}.focus\:caret-stone-100\/25:focus{caret-color:hsla(60,5%,96%,.25)}.focus\:caret-stone-100\/30:focus{caret-color:hsla(60,5%,96%,.3)}.focus\:caret-stone-100\/35:focus{caret-color:hsla(60,5%,96%,.35)}.focus\:caret-stone-100\/40:focus{caret-color:hsla(60,5%,96%,.4)}.focus\:caret-stone-100\/45:focus{caret-color:hsla(60,5%,96%,.45)}.focus\:caret-stone-100\/5:focus{caret-color:hsla(60,5%,96%,.05)}.focus\:caret-stone-100\/50:focus{caret-color:hsla(60,5%,96%,.5)}.focus\:caret-stone-100\/55:focus{caret-color:hsla(60,5%,96%,.55)}.focus\:caret-stone-100\/60:focus{caret-color:hsla(60,5%,96%,.6)}.focus\:caret-stone-100\/65:focus{caret-color:hsla(60,5%,96%,.65)}.focus\:caret-stone-100\/70:focus{caret-color:hsla(60,5%,96%,.7)}.focus\:caret-stone-100\/75:focus{caret-color:hsla(60,5%,96%,.75)}.focus\:caret-stone-100\/80:focus{caret-color:hsla(60,5%,96%,.8)}.focus\:caret-stone-100\/85:focus{caret-color:hsla(60,5%,96%,.85)}.focus\:caret-stone-100\/90:focus{caret-color:hsla(60,5%,96%,.9)}.focus\:caret-stone-100\/95:focus{caret-color:hsla(60,5%,96%,.95)}.focus\:caret-stone-200:focus{caret-color:#e7e5e4}.focus\:caret-stone-200\/0:focus{caret-color:hsla(20,6%,90%,0)}.focus\:caret-stone-200\/10:focus{caret-color:hsla(20,6%,90%,.1)}.focus\:caret-stone-200\/100:focus{caret-color:#e7e5e4}.focus\:caret-stone-200\/15:focus{caret-color:hsla(20,6%,90%,.15)}.focus\:caret-stone-200\/20:focus{caret-color:hsla(20,6%,90%,.2)}.focus\:caret-stone-200\/25:focus{caret-color:hsla(20,6%,90%,.25)}.focus\:caret-stone-200\/30:focus{caret-color:hsla(20,6%,90%,.3)}.focus\:caret-stone-200\/35:focus{caret-color:hsla(20,6%,90%,.35)}.focus\:caret-stone-200\/40:focus{caret-color:hsla(20,6%,90%,.4)}.focus\:caret-stone-200\/45:focus{caret-color:hsla(20,6%,90%,.45)}.focus\:caret-stone-200\/5:focus{caret-color:hsla(20,6%,90%,.05)}.focus\:caret-stone-200\/50:focus{caret-color:hsla(20,6%,90%,.5)}.focus\:caret-stone-200\/55:focus{caret-color:hsla(20,6%,90%,.55)}.focus\:caret-stone-200\/60:focus{caret-color:hsla(20,6%,90%,.6)}.focus\:caret-stone-200\/65:focus{caret-color:hsla(20,6%,90%,.65)}.focus\:caret-stone-200\/70:focus{caret-color:hsla(20,6%,90%,.7)}.focus\:caret-stone-200\/75:focus{caret-color:hsla(20,6%,90%,.75)}.focus\:caret-stone-200\/80:focus{caret-color:hsla(20,6%,90%,.8)}.focus\:caret-stone-200\/85:focus{caret-color:hsla(20,6%,90%,.85)}.focus\:caret-stone-200\/90:focus{caret-color:hsla(20,6%,90%,.9)}.focus\:caret-stone-200\/95:focus{caret-color:hsla(20,6%,90%,.95)}.focus\:caret-stone-400:focus{caret-color:#a8a29e}.focus\:caret-stone-400\/0:focus{caret-color:hsla(24,5%,64%,0)}.focus\:caret-stone-400\/10:focus{caret-color:hsla(24,5%,64%,.1)}.focus\:caret-stone-400\/100:focus{caret-color:#a8a29e}.focus\:caret-stone-400\/15:focus{caret-color:hsla(24,5%,64%,.15)}.focus\:caret-stone-400\/20:focus{caret-color:hsla(24,5%,64%,.2)}.focus\:caret-stone-400\/25:focus{caret-color:hsla(24,5%,64%,.25)}.focus\:caret-stone-400\/30:focus{caret-color:hsla(24,5%,64%,.3)}.focus\:caret-stone-400\/35:focus{caret-color:hsla(24,5%,64%,.35)}.focus\:caret-stone-400\/40:focus{caret-color:hsla(24,5%,64%,.4)}.focus\:caret-stone-400\/45:focus{caret-color:hsla(24,5%,64%,.45)}.focus\:caret-stone-400\/5:focus{caret-color:hsla(24,5%,64%,.05)}.focus\:caret-stone-400\/50:focus{caret-color:hsla(24,5%,64%,.5)}.focus\:caret-stone-400\/55:focus{caret-color:hsla(24,5%,64%,.55)}.focus\:caret-stone-400\/60:focus{caret-color:hsla(24,5%,64%,.6)}.focus\:caret-stone-400\/65:focus{caret-color:hsla(24,5%,64%,.65)}.focus\:caret-stone-400\/70:focus{caret-color:hsla(24,5%,64%,.7)}.focus\:caret-stone-400\/75:focus{caret-color:hsla(24,5%,64%,.75)}.focus\:caret-stone-400\/80:focus{caret-color:hsla(24,5%,64%,.8)}.focus\:caret-stone-400\/85:focus{caret-color:hsla(24,5%,64%,.85)}.focus\:caret-stone-400\/90:focus{caret-color:hsla(24,5%,64%,.9)}.focus\:caret-stone-400\/95:focus{caret-color:hsla(24,5%,64%,.95)}.focus\:caret-stone-50:focus{caret-color:#fafaf9}.focus\:caret-stone-50\/0:focus{caret-color:hsla(60,9%,98%,0)}.focus\:caret-stone-50\/10:focus{caret-color:hsla(60,9%,98%,.1)}.focus\:caret-stone-50\/100:focus{caret-color:#fafaf9}.focus\:caret-stone-50\/15:focus{caret-color:hsla(60,9%,98%,.15)}.focus\:caret-stone-50\/20:focus{caret-color:hsla(60,9%,98%,.2)}.focus\:caret-stone-50\/25:focus{caret-color:hsla(60,9%,98%,.25)}.focus\:caret-stone-50\/30:focus{caret-color:hsla(60,9%,98%,.3)}.focus\:caret-stone-50\/35:focus{caret-color:hsla(60,9%,98%,.35)}.focus\:caret-stone-50\/40:focus{caret-color:hsla(60,9%,98%,.4)}.focus\:caret-stone-50\/45:focus{caret-color:hsla(60,9%,98%,.45)}.focus\:caret-stone-50\/5:focus{caret-color:hsla(60,9%,98%,.05)}.focus\:caret-stone-50\/50:focus{caret-color:hsla(60,9%,98%,.5)}.focus\:caret-stone-50\/55:focus{caret-color:hsla(60,9%,98%,.55)}.focus\:caret-stone-50\/60:focus{caret-color:hsla(60,9%,98%,.6)}.focus\:caret-stone-50\/65:focus{caret-color:hsla(60,9%,98%,.65)}.focus\:caret-stone-50\/70:focus{caret-color:hsla(60,9%,98%,.7)}.focus\:caret-stone-50\/75:focus{caret-color:hsla(60,9%,98%,.75)}.focus\:caret-stone-50\/80:focus{caret-color:hsla(60,9%,98%,.8)}.focus\:caret-stone-50\/85:focus{caret-color:hsla(60,9%,98%,.85)}.focus\:caret-stone-50\/90:focus{caret-color:hsla(60,9%,98%,.9)}.focus\:caret-stone-50\/95:focus{caret-color:hsla(60,9%,98%,.95)}.focus\:caret-stone-500:focus{caret-color:#78716c}.focus\:caret-stone-500\/0:focus{caret-color:hsla(25,5%,45%,0)}.focus\:caret-stone-500\/10:focus{caret-color:hsla(25,5%,45%,.1)}.focus\:caret-stone-500\/100:focus{caret-color:#78716c}.focus\:caret-stone-500\/15:focus{caret-color:hsla(25,5%,45%,.15)}.focus\:caret-stone-500\/20:focus{caret-color:hsla(25,5%,45%,.2)}.focus\:caret-stone-500\/25:focus{caret-color:hsla(25,5%,45%,.25)}.focus\:caret-stone-500\/30:focus{caret-color:hsla(25,5%,45%,.3)}.focus\:caret-stone-500\/35:focus{caret-color:hsla(25,5%,45%,.35)}.focus\:caret-stone-500\/40:focus{caret-color:hsla(25,5%,45%,.4)}.focus\:caret-stone-500\/45:focus{caret-color:hsla(25,5%,45%,.45)}.focus\:caret-stone-500\/5:focus{caret-color:hsla(25,5%,45%,.05)}.focus\:caret-stone-500\/50:focus{caret-color:hsla(25,5%,45%,.5)}.focus\:caret-stone-500\/55:focus{caret-color:hsla(25,5%,45%,.55)}.focus\:caret-stone-500\/60:focus{caret-color:hsla(25,5%,45%,.6)}.focus\:caret-stone-500\/65:focus{caret-color:hsla(25,5%,45%,.65)}.focus\:caret-stone-500\/70:focus{caret-color:hsla(25,5%,45%,.7)}.focus\:caret-stone-500\/75:focus{caret-color:hsla(25,5%,45%,.75)}.focus\:caret-stone-500\/80:focus{caret-color:hsla(25,5%,45%,.8)}.focus\:caret-stone-500\/85:focus{caret-color:hsla(25,5%,45%,.85)}.focus\:caret-stone-500\/90:focus{caret-color:hsla(25,5%,45%,.9)}.focus\:caret-stone-500\/95:focus{caret-color:hsla(25,5%,45%,.95)}.focus\:caret-stone-950:focus{caret-color:#0c0a09}.focus\:caret-stone-950\/0:focus{caret-color:rgba(12,10,9,0)}.focus\:caret-stone-950\/10:focus{caret-color:rgba(12,10,9,.1)}.focus\:caret-stone-950\/100:focus{caret-color:#0c0a09}.focus\:caret-stone-950\/15:focus{caret-color:rgba(12,10,9,.15)}.focus\:caret-stone-950\/20:focus{caret-color:rgba(12,10,9,.2)}.focus\:caret-stone-950\/25:focus{caret-color:rgba(12,10,9,.25)}.focus\:caret-stone-950\/30:focus{caret-color:rgba(12,10,9,.3)}.focus\:caret-stone-950\/35:focus{caret-color:rgba(12,10,9,.35)}.focus\:caret-stone-950\/40:focus{caret-color:rgba(12,10,9,.4)}.focus\:caret-stone-950\/45:focus{caret-color:rgba(12,10,9,.45)}.focus\:caret-stone-950\/5:focus{caret-color:rgba(12,10,9,.05)}.focus\:caret-stone-950\/50:focus{caret-color:rgba(12,10,9,.5)}.focus\:caret-stone-950\/55:focus{caret-color:rgba(12,10,9,.55)}.focus\:caret-stone-950\/60:focus{caret-color:rgba(12,10,9,.6)}.focus\:caret-stone-950\/65:focus{caret-color:rgba(12,10,9,.65)}.focus\:caret-stone-950\/70:focus{caret-color:rgba(12,10,9,.7)}.focus\:caret-stone-950\/75:focus{caret-color:rgba(12,10,9,.75)}.focus\:caret-stone-950\/80:focus{caret-color:rgba(12,10,9,.8)}.focus\:caret-stone-950\/85:focus{caret-color:rgba(12,10,9,.85)}.focus\:caret-stone-950\/90:focus{caret-color:rgba(12,10,9,.9)}.focus\:caret-stone-950\/95:focus{caret-color:rgba(12,10,9,.95)}.focus\:caret-teal-100:focus{caret-color:#ccfbf1}.focus\:caret-teal-100\/0:focus{caret-color:rgba(204,251,241,0)}.focus\:caret-teal-100\/10:focus{caret-color:rgba(204,251,241,.1)}.focus\:caret-teal-100\/100:focus{caret-color:#ccfbf1}.focus\:caret-teal-100\/15:focus{caret-color:rgba(204,251,241,.15)}.focus\:caret-teal-100\/20:focus{caret-color:rgba(204,251,241,.2)}.focus\:caret-teal-100\/25:focus{caret-color:rgba(204,251,241,.25)}.focus\:caret-teal-100\/30:focus{caret-color:rgba(204,251,241,.3)}.focus\:caret-teal-100\/35:focus{caret-color:rgba(204,251,241,.35)}.focus\:caret-teal-100\/40:focus{caret-color:rgba(204,251,241,.4)}.focus\:caret-teal-100\/45:focus{caret-color:rgba(204,251,241,.45)}.focus\:caret-teal-100\/5:focus{caret-color:rgba(204,251,241,.05)}.focus\:caret-teal-100\/50:focus{caret-color:rgba(204,251,241,.5)}.focus\:caret-teal-100\/55:focus{caret-color:rgba(204,251,241,.55)}.focus\:caret-teal-100\/60:focus{caret-color:rgba(204,251,241,.6)}.focus\:caret-teal-100\/65:focus{caret-color:rgba(204,251,241,.65)}.focus\:caret-teal-100\/70:focus{caret-color:rgba(204,251,241,.7)}.focus\:caret-teal-100\/75:focus{caret-color:rgba(204,251,241,.75)}.focus\:caret-teal-100\/80:focus{caret-color:rgba(204,251,241,.8)}.focus\:caret-teal-100\/85:focus{caret-color:rgba(204,251,241,.85)}.focus\:caret-teal-100\/90:focus{caret-color:rgba(204,251,241,.9)}.focus\:caret-teal-100\/95:focus{caret-color:rgba(204,251,241,.95)}.focus\:caret-teal-200:focus{caret-color:#99f6e4}.focus\:caret-teal-200\/0:focus{caret-color:rgba(153,246,228,0)}.focus\:caret-teal-200\/10:focus{caret-color:rgba(153,246,228,.1)}.focus\:caret-teal-200\/100:focus{caret-color:#99f6e4}.focus\:caret-teal-200\/15:focus{caret-color:rgba(153,246,228,.15)}.focus\:caret-teal-200\/20:focus{caret-color:rgba(153,246,228,.2)}.focus\:caret-teal-200\/25:focus{caret-color:rgba(153,246,228,.25)}.focus\:caret-teal-200\/30:focus{caret-color:rgba(153,246,228,.3)}.focus\:caret-teal-200\/35:focus{caret-color:rgba(153,246,228,.35)}.focus\:caret-teal-200\/40:focus{caret-color:rgba(153,246,228,.4)}.focus\:caret-teal-200\/45:focus{caret-color:rgba(153,246,228,.45)}.focus\:caret-teal-200\/5:focus{caret-color:rgba(153,246,228,.05)}.focus\:caret-teal-200\/50:focus{caret-color:rgba(153,246,228,.5)}.focus\:caret-teal-200\/55:focus{caret-color:rgba(153,246,228,.55)}.focus\:caret-teal-200\/60:focus{caret-color:rgba(153,246,228,.6)}.focus\:caret-teal-200\/65:focus{caret-color:rgba(153,246,228,.65)}.focus\:caret-teal-200\/70:focus{caret-color:rgba(153,246,228,.7)}.focus\:caret-teal-200\/75:focus{caret-color:rgba(153,246,228,.75)}.focus\:caret-teal-200\/80:focus{caret-color:rgba(153,246,228,.8)}.focus\:caret-teal-200\/85:focus{caret-color:rgba(153,246,228,.85)}.focus\:caret-teal-200\/90:focus{caret-color:rgba(153,246,228,.9)}.focus\:caret-teal-200\/95:focus{caret-color:rgba(153,246,228,.95)}.focus\:caret-teal-400:focus{caret-color:#2dd4bf}.focus\:caret-teal-400\/0:focus{caret-color:rgba(45,212,191,0)}.focus\:caret-teal-400\/10:focus{caret-color:rgba(45,212,191,.1)}.focus\:caret-teal-400\/100:focus{caret-color:#2dd4bf}.focus\:caret-teal-400\/15:focus{caret-color:rgba(45,212,191,.15)}.focus\:caret-teal-400\/20:focus{caret-color:rgba(45,212,191,.2)}.focus\:caret-teal-400\/25:focus{caret-color:rgba(45,212,191,.25)}.focus\:caret-teal-400\/30:focus{caret-color:rgba(45,212,191,.3)}.focus\:caret-teal-400\/35:focus{caret-color:rgba(45,212,191,.35)}.focus\:caret-teal-400\/40:focus{caret-color:rgba(45,212,191,.4)}.focus\:caret-teal-400\/45:focus{caret-color:rgba(45,212,191,.45)}.focus\:caret-teal-400\/5:focus{caret-color:rgba(45,212,191,.05)}.focus\:caret-teal-400\/50:focus{caret-color:rgba(45,212,191,.5)}.focus\:caret-teal-400\/55:focus{caret-color:rgba(45,212,191,.55)}.focus\:caret-teal-400\/60:focus{caret-color:rgba(45,212,191,.6)}.focus\:caret-teal-400\/65:focus{caret-color:rgba(45,212,191,.65)}.focus\:caret-teal-400\/70:focus{caret-color:rgba(45,212,191,.7)}.focus\:caret-teal-400\/75:focus{caret-color:rgba(45,212,191,.75)}.focus\:caret-teal-400\/80:focus{caret-color:rgba(45,212,191,.8)}.focus\:caret-teal-400\/85:focus{caret-color:rgba(45,212,191,.85)}.focus\:caret-teal-400\/90:focus{caret-color:rgba(45,212,191,.9)}.focus\:caret-teal-400\/95:focus{caret-color:rgba(45,212,191,.95)}.focus\:caret-teal-50:focus{caret-color:#f0fdfa}.focus\:caret-teal-50\/0:focus{caret-color:rgba(240,253,250,0)}.focus\:caret-teal-50\/10:focus{caret-color:rgba(240,253,250,.1)}.focus\:caret-teal-50\/100:focus{caret-color:#f0fdfa}.focus\:caret-teal-50\/15:focus{caret-color:rgba(240,253,250,.15)}.focus\:caret-teal-50\/20:focus{caret-color:rgba(240,253,250,.2)}.focus\:caret-teal-50\/25:focus{caret-color:rgba(240,253,250,.25)}.focus\:caret-teal-50\/30:focus{caret-color:rgba(240,253,250,.3)}.focus\:caret-teal-50\/35:focus{caret-color:rgba(240,253,250,.35)}.focus\:caret-teal-50\/40:focus{caret-color:rgba(240,253,250,.4)}.focus\:caret-teal-50\/45:focus{caret-color:rgba(240,253,250,.45)}.focus\:caret-teal-50\/5:focus{caret-color:rgba(240,253,250,.05)}.focus\:caret-teal-50\/50:focus{caret-color:rgba(240,253,250,.5)}.focus\:caret-teal-50\/55:focus{caret-color:rgba(240,253,250,.55)}.focus\:caret-teal-50\/60:focus{caret-color:rgba(240,253,250,.6)}.focus\:caret-teal-50\/65:focus{caret-color:rgba(240,253,250,.65)}.focus\:caret-teal-50\/70:focus{caret-color:rgba(240,253,250,.7)}.focus\:caret-teal-50\/75:focus{caret-color:rgba(240,253,250,.75)}.focus\:caret-teal-50\/80:focus{caret-color:rgba(240,253,250,.8)}.focus\:caret-teal-50\/85:focus{caret-color:rgba(240,253,250,.85)}.focus\:caret-teal-50\/90:focus{caret-color:rgba(240,253,250,.9)}.focus\:caret-teal-50\/95:focus{caret-color:rgba(240,253,250,.95)}.focus\:caret-teal-500:focus{caret-color:#14b8a6}.focus\:caret-teal-500\/0:focus{caret-color:rgba(20,184,166,0)}.focus\:caret-teal-500\/10:focus{caret-color:rgba(20,184,166,.1)}.focus\:caret-teal-500\/100:focus{caret-color:#14b8a6}.focus\:caret-teal-500\/15:focus{caret-color:rgba(20,184,166,.15)}.focus\:caret-teal-500\/20:focus{caret-color:rgba(20,184,166,.2)}.focus\:caret-teal-500\/25:focus{caret-color:rgba(20,184,166,.25)}.focus\:caret-teal-500\/30:focus{caret-color:rgba(20,184,166,.3)}.focus\:caret-teal-500\/35:focus{caret-color:rgba(20,184,166,.35)}.focus\:caret-teal-500\/40:focus{caret-color:rgba(20,184,166,.4)}.focus\:caret-teal-500\/45:focus{caret-color:rgba(20,184,166,.45)}.focus\:caret-teal-500\/5:focus{caret-color:rgba(20,184,166,.05)}.focus\:caret-teal-500\/50:focus{caret-color:rgba(20,184,166,.5)}.focus\:caret-teal-500\/55:focus{caret-color:rgba(20,184,166,.55)}.focus\:caret-teal-500\/60:focus{caret-color:rgba(20,184,166,.6)}.focus\:caret-teal-500\/65:focus{caret-color:rgba(20,184,166,.65)}.focus\:caret-teal-500\/70:focus{caret-color:rgba(20,184,166,.7)}.focus\:caret-teal-500\/75:focus{caret-color:rgba(20,184,166,.75)}.focus\:caret-teal-500\/80:focus{caret-color:rgba(20,184,166,.8)}.focus\:caret-teal-500\/85:focus{caret-color:rgba(20,184,166,.85)}.focus\:caret-teal-500\/90:focus{caret-color:rgba(20,184,166,.9)}.focus\:caret-teal-500\/95:focus{caret-color:rgba(20,184,166,.95)}.focus\:caret-teal-950:focus{caret-color:#042f2e}.focus\:caret-teal-950\/0:focus{caret-color:rgba(4,47,46,0)}.focus\:caret-teal-950\/10:focus{caret-color:rgba(4,47,46,.1)}.focus\:caret-teal-950\/100:focus{caret-color:#042f2e}.focus\:caret-teal-950\/15:focus{caret-color:rgba(4,47,46,.15)}.focus\:caret-teal-950\/20:focus{caret-color:rgba(4,47,46,.2)}.focus\:caret-teal-950\/25:focus{caret-color:rgba(4,47,46,.25)}.focus\:caret-teal-950\/30:focus{caret-color:rgba(4,47,46,.3)}.focus\:caret-teal-950\/35:focus{caret-color:rgba(4,47,46,.35)}.focus\:caret-teal-950\/40:focus{caret-color:rgba(4,47,46,.4)}.focus\:caret-teal-950\/45:focus{caret-color:rgba(4,47,46,.45)}.focus\:caret-teal-950\/5:focus{caret-color:rgba(4,47,46,.05)}.focus\:caret-teal-950\/50:focus{caret-color:rgba(4,47,46,.5)}.focus\:caret-teal-950\/55:focus{caret-color:rgba(4,47,46,.55)}.focus\:caret-teal-950\/60:focus{caret-color:rgba(4,47,46,.6)}.focus\:caret-teal-950\/65:focus{caret-color:rgba(4,47,46,.65)}.focus\:caret-teal-950\/70:focus{caret-color:rgba(4,47,46,.7)}.focus\:caret-teal-950\/75:focus{caret-color:rgba(4,47,46,.75)}.focus\:caret-teal-950\/80:focus{caret-color:rgba(4,47,46,.8)}.focus\:caret-teal-950\/85:focus{caret-color:rgba(4,47,46,.85)}.focus\:caret-teal-950\/90:focus{caret-color:rgba(4,47,46,.9)}.focus\:caret-teal-950\/95:focus{caret-color:rgba(4,47,46,.95)}.focus\:caret-violet-100:focus{caret-color:#ede9fe}.focus\:caret-violet-100\/0:focus{caret-color:rgba(237,233,254,0)}.focus\:caret-violet-100\/10:focus{caret-color:rgba(237,233,254,.1)}.focus\:caret-violet-100\/100:focus{caret-color:#ede9fe}.focus\:caret-violet-100\/15:focus{caret-color:rgba(237,233,254,.15)}.focus\:caret-violet-100\/20:focus{caret-color:rgba(237,233,254,.2)}.focus\:caret-violet-100\/25:focus{caret-color:rgba(237,233,254,.25)}.focus\:caret-violet-100\/30:focus{caret-color:rgba(237,233,254,.3)}.focus\:caret-violet-100\/35:focus{caret-color:rgba(237,233,254,.35)}.focus\:caret-violet-100\/40:focus{caret-color:rgba(237,233,254,.4)}.focus\:caret-violet-100\/45:focus{caret-color:rgba(237,233,254,.45)}.focus\:caret-violet-100\/5:focus{caret-color:rgba(237,233,254,.05)}.focus\:caret-violet-100\/50:focus{caret-color:rgba(237,233,254,.5)}.focus\:caret-violet-100\/55:focus{caret-color:rgba(237,233,254,.55)}.focus\:caret-violet-100\/60:focus{caret-color:rgba(237,233,254,.6)}.focus\:caret-violet-100\/65:focus{caret-color:rgba(237,233,254,.65)}.focus\:caret-violet-100\/70:focus{caret-color:rgba(237,233,254,.7)}.focus\:caret-violet-100\/75:focus{caret-color:rgba(237,233,254,.75)}.focus\:caret-violet-100\/80:focus{caret-color:rgba(237,233,254,.8)}.focus\:caret-violet-100\/85:focus{caret-color:rgba(237,233,254,.85)}.focus\:caret-violet-100\/90:focus{caret-color:rgba(237,233,254,.9)}.focus\:caret-violet-100\/95:focus{caret-color:rgba(237,233,254,.95)}.focus\:caret-violet-200:focus{caret-color:#ddd6fe}.focus\:caret-violet-200\/0:focus{caret-color:rgba(221,214,254,0)}.focus\:caret-violet-200\/10:focus{caret-color:rgba(221,214,254,.1)}.focus\:caret-violet-200\/100:focus{caret-color:#ddd6fe}.focus\:caret-violet-200\/15:focus{caret-color:rgba(221,214,254,.15)}.focus\:caret-violet-200\/20:focus{caret-color:rgba(221,214,254,.2)}.focus\:caret-violet-200\/25:focus{caret-color:rgba(221,214,254,.25)}.focus\:caret-violet-200\/30:focus{caret-color:rgba(221,214,254,.3)}.focus\:caret-violet-200\/35:focus{caret-color:rgba(221,214,254,.35)}.focus\:caret-violet-200\/40:focus{caret-color:rgba(221,214,254,.4)}.focus\:caret-violet-200\/45:focus{caret-color:rgba(221,214,254,.45)}.focus\:caret-violet-200\/5:focus{caret-color:rgba(221,214,254,.05)}.focus\:caret-violet-200\/50:focus{caret-color:rgba(221,214,254,.5)}.focus\:caret-violet-200\/55:focus{caret-color:rgba(221,214,254,.55)}.focus\:caret-violet-200\/60:focus{caret-color:rgba(221,214,254,.6)}.focus\:caret-violet-200\/65:focus{caret-color:rgba(221,214,254,.65)}.focus\:caret-violet-200\/70:focus{caret-color:rgba(221,214,254,.7)}.focus\:caret-violet-200\/75:focus{caret-color:rgba(221,214,254,.75)}.focus\:caret-violet-200\/80:focus{caret-color:rgba(221,214,254,.8)}.focus\:caret-violet-200\/85:focus{caret-color:rgba(221,214,254,.85)}.focus\:caret-violet-200\/90:focus{caret-color:rgba(221,214,254,.9)}.focus\:caret-violet-200\/95:focus{caret-color:rgba(221,214,254,.95)}.focus\:caret-violet-400:focus{caret-color:#a78bfa}.focus\:caret-violet-400\/0:focus{caret-color:rgba(167,139,250,0)}.focus\:caret-violet-400\/10:focus{caret-color:rgba(167,139,250,.1)}.focus\:caret-violet-400\/100:focus{caret-color:#a78bfa}.focus\:caret-violet-400\/15:focus{caret-color:rgba(167,139,250,.15)}.focus\:caret-violet-400\/20:focus{caret-color:rgba(167,139,250,.2)}.focus\:caret-violet-400\/25:focus{caret-color:rgba(167,139,250,.25)}.focus\:caret-violet-400\/30:focus{caret-color:rgba(167,139,250,.3)}.focus\:caret-violet-400\/35:focus{caret-color:rgba(167,139,250,.35)}.focus\:caret-violet-400\/40:focus{caret-color:rgba(167,139,250,.4)}.focus\:caret-violet-400\/45:focus{caret-color:rgba(167,139,250,.45)}.focus\:caret-violet-400\/5:focus{caret-color:rgba(167,139,250,.05)}.focus\:caret-violet-400\/50:focus{caret-color:rgba(167,139,250,.5)}.focus\:caret-violet-400\/55:focus{caret-color:rgba(167,139,250,.55)}.focus\:caret-violet-400\/60:focus{caret-color:rgba(167,139,250,.6)}.focus\:caret-violet-400\/65:focus{caret-color:rgba(167,139,250,.65)}.focus\:caret-violet-400\/70:focus{caret-color:rgba(167,139,250,.7)}.focus\:caret-violet-400\/75:focus{caret-color:rgba(167,139,250,.75)}.focus\:caret-violet-400\/80:focus{caret-color:rgba(167,139,250,.8)}.focus\:caret-violet-400\/85:focus{caret-color:rgba(167,139,250,.85)}.focus\:caret-violet-400\/90:focus{caret-color:rgba(167,139,250,.9)}.focus\:caret-violet-400\/95:focus{caret-color:rgba(167,139,250,.95)}.focus\:caret-violet-50:focus{caret-color:#f5f3ff}.focus\:caret-violet-50\/0:focus{caret-color:rgba(245,243,255,0)}.focus\:caret-violet-50\/10:focus{caret-color:rgba(245,243,255,.1)}.focus\:caret-violet-50\/100:focus{caret-color:#f5f3ff}.focus\:caret-violet-50\/15:focus{caret-color:rgba(245,243,255,.15)}.focus\:caret-violet-50\/20:focus{caret-color:rgba(245,243,255,.2)}.focus\:caret-violet-50\/25:focus{caret-color:rgba(245,243,255,.25)}.focus\:caret-violet-50\/30:focus{caret-color:rgba(245,243,255,.3)}.focus\:caret-violet-50\/35:focus{caret-color:rgba(245,243,255,.35)}.focus\:caret-violet-50\/40:focus{caret-color:rgba(245,243,255,.4)}.focus\:caret-violet-50\/45:focus{caret-color:rgba(245,243,255,.45)}.focus\:caret-violet-50\/5:focus{caret-color:rgba(245,243,255,.05)}.focus\:caret-violet-50\/50:focus{caret-color:rgba(245,243,255,.5)}.focus\:caret-violet-50\/55:focus{caret-color:rgba(245,243,255,.55)}.focus\:caret-violet-50\/60:focus{caret-color:rgba(245,243,255,.6)}.focus\:caret-violet-50\/65:focus{caret-color:rgba(245,243,255,.65)}.focus\:caret-violet-50\/70:focus{caret-color:rgba(245,243,255,.7)}.focus\:caret-violet-50\/75:focus{caret-color:rgba(245,243,255,.75)}.focus\:caret-violet-50\/80:focus{caret-color:rgba(245,243,255,.8)}.focus\:caret-violet-50\/85:focus{caret-color:rgba(245,243,255,.85)}.focus\:caret-violet-50\/90:focus{caret-color:rgba(245,243,255,.9)}.focus\:caret-violet-50\/95:focus{caret-color:rgba(245,243,255,.95)}.focus\:caret-violet-500:focus{caret-color:#8b5cf6}.focus\:caret-violet-500\/0:focus{caret-color:rgba(139,92,246,0)}.focus\:caret-violet-500\/10:focus{caret-color:rgba(139,92,246,.1)}.focus\:caret-violet-500\/100:focus{caret-color:#8b5cf6}.focus\:caret-violet-500\/15:focus{caret-color:rgba(139,92,246,.15)}.focus\:caret-violet-500\/20:focus{caret-color:rgba(139,92,246,.2)}.focus\:caret-violet-500\/25:focus{caret-color:rgba(139,92,246,.25)}.focus\:caret-violet-500\/30:focus{caret-color:rgba(139,92,246,.3)}.focus\:caret-violet-500\/35:focus{caret-color:rgba(139,92,246,.35)}.focus\:caret-violet-500\/40:focus{caret-color:rgba(139,92,246,.4)}.focus\:caret-violet-500\/45:focus{caret-color:rgba(139,92,246,.45)}.focus\:caret-violet-500\/5:focus{caret-color:rgba(139,92,246,.05)}.focus\:caret-violet-500\/50:focus{caret-color:rgba(139,92,246,.5)}.focus\:caret-violet-500\/55:focus{caret-color:rgba(139,92,246,.55)}.focus\:caret-violet-500\/60:focus{caret-color:rgba(139,92,246,.6)}.focus\:caret-violet-500\/65:focus{caret-color:rgba(139,92,246,.65)}.focus\:caret-violet-500\/70:focus{caret-color:rgba(139,92,246,.7)}.focus\:caret-violet-500\/75:focus{caret-color:rgba(139,92,246,.75)}.focus\:caret-violet-500\/80:focus{caret-color:rgba(139,92,246,.8)}.focus\:caret-violet-500\/85:focus{caret-color:rgba(139,92,246,.85)}.focus\:caret-violet-500\/90:focus{caret-color:rgba(139,92,246,.9)}.focus\:caret-violet-500\/95:focus{caret-color:rgba(139,92,246,.95)}.focus\:caret-violet-950:focus{caret-color:#2e1065}.focus\:caret-violet-950\/0:focus{caret-color:rgba(46,16,101,0)}.focus\:caret-violet-950\/10:focus{caret-color:rgba(46,16,101,.1)}.focus\:caret-violet-950\/100:focus{caret-color:#2e1065}.focus\:caret-violet-950\/15:focus{caret-color:rgba(46,16,101,.15)}.focus\:caret-violet-950\/20:focus{caret-color:rgba(46,16,101,.2)}.focus\:caret-violet-950\/25:focus{caret-color:rgba(46,16,101,.25)}.focus\:caret-violet-950\/30:focus{caret-color:rgba(46,16,101,.3)}.focus\:caret-violet-950\/35:focus{caret-color:rgba(46,16,101,.35)}.focus\:caret-violet-950\/40:focus{caret-color:rgba(46,16,101,.4)}.focus\:caret-violet-950\/45:focus{caret-color:rgba(46,16,101,.45)}.focus\:caret-violet-950\/5:focus{caret-color:rgba(46,16,101,.05)}.focus\:caret-violet-950\/50:focus{caret-color:rgba(46,16,101,.5)}.focus\:caret-violet-950\/55:focus{caret-color:rgba(46,16,101,.55)}.focus\:caret-violet-950\/60:focus{caret-color:rgba(46,16,101,.6)}.focus\:caret-violet-950\/65:focus{caret-color:rgba(46,16,101,.65)}.focus\:caret-violet-950\/70:focus{caret-color:rgba(46,16,101,.7)}.focus\:caret-violet-950\/75:focus{caret-color:rgba(46,16,101,.75)}.focus\:caret-violet-950\/80:focus{caret-color:rgba(46,16,101,.8)}.focus\:caret-violet-950\/85:focus{caret-color:rgba(46,16,101,.85)}.focus\:caret-violet-950\/90:focus{caret-color:rgba(46,16,101,.9)}.focus\:caret-violet-950\/95:focus{caret-color:rgba(46,16,101,.95)}.focus\:caret-yellow-100:focus{caret-color:#fef9c3}.focus\:caret-yellow-100\/0:focus{caret-color:hsla(55,97%,88%,0)}.focus\:caret-yellow-100\/10:focus{caret-color:hsla(55,97%,88%,.1)}.focus\:caret-yellow-100\/100:focus{caret-color:#fef9c3}.focus\:caret-yellow-100\/15:focus{caret-color:hsla(55,97%,88%,.15)}.focus\:caret-yellow-100\/20:focus{caret-color:hsla(55,97%,88%,.2)}.focus\:caret-yellow-100\/25:focus{caret-color:hsla(55,97%,88%,.25)}.focus\:caret-yellow-100\/30:focus{caret-color:hsla(55,97%,88%,.3)}.focus\:caret-yellow-100\/35:focus{caret-color:hsla(55,97%,88%,.35)}.focus\:caret-yellow-100\/40:focus{caret-color:hsla(55,97%,88%,.4)}.focus\:caret-yellow-100\/45:focus{caret-color:hsla(55,97%,88%,.45)}.focus\:caret-yellow-100\/5:focus{caret-color:hsla(55,97%,88%,.05)}.focus\:caret-yellow-100\/50:focus{caret-color:hsla(55,97%,88%,.5)}.focus\:caret-yellow-100\/55:focus{caret-color:hsla(55,97%,88%,.55)}.focus\:caret-yellow-100\/60:focus{caret-color:hsla(55,97%,88%,.6)}.focus\:caret-yellow-100\/65:focus{caret-color:hsla(55,97%,88%,.65)}.focus\:caret-yellow-100\/70:focus{caret-color:hsla(55,97%,88%,.7)}.focus\:caret-yellow-100\/75:focus{caret-color:hsla(55,97%,88%,.75)}.focus\:caret-yellow-100\/80:focus{caret-color:hsla(55,97%,88%,.8)}.focus\:caret-yellow-100\/85:focus{caret-color:hsla(55,97%,88%,.85)}.focus\:caret-yellow-100\/90:focus{caret-color:hsla(55,97%,88%,.9)}.focus\:caret-yellow-100\/95:focus{caret-color:hsla(55,97%,88%,.95)}.focus\:caret-yellow-200:focus{caret-color:#fef08a}.focus\:caret-yellow-200\/0:focus{caret-color:hsla(53,98%,77%,0)}.focus\:caret-yellow-200\/10:focus{caret-color:hsla(53,98%,77%,.1)}.focus\:caret-yellow-200\/100:focus{caret-color:#fef08a}.focus\:caret-yellow-200\/15:focus{caret-color:hsla(53,98%,77%,.15)}.focus\:caret-yellow-200\/20:focus{caret-color:hsla(53,98%,77%,.2)}.focus\:caret-yellow-200\/25:focus{caret-color:hsla(53,98%,77%,.25)}.focus\:caret-yellow-200\/30:focus{caret-color:hsla(53,98%,77%,.3)}.focus\:caret-yellow-200\/35:focus{caret-color:hsla(53,98%,77%,.35)}.focus\:caret-yellow-200\/40:focus{caret-color:hsla(53,98%,77%,.4)}.focus\:caret-yellow-200\/45:focus{caret-color:hsla(53,98%,77%,.45)}.focus\:caret-yellow-200\/5:focus{caret-color:hsla(53,98%,77%,.05)}.focus\:caret-yellow-200\/50:focus{caret-color:hsla(53,98%,77%,.5)}.focus\:caret-yellow-200\/55:focus{caret-color:hsla(53,98%,77%,.55)}.focus\:caret-yellow-200\/60:focus{caret-color:hsla(53,98%,77%,.6)}.focus\:caret-yellow-200\/65:focus{caret-color:hsla(53,98%,77%,.65)}.focus\:caret-yellow-200\/70:focus{caret-color:hsla(53,98%,77%,.7)}.focus\:caret-yellow-200\/75:focus{caret-color:hsla(53,98%,77%,.75)}.focus\:caret-yellow-200\/80:focus{caret-color:hsla(53,98%,77%,.8)}.focus\:caret-yellow-200\/85:focus{caret-color:hsla(53,98%,77%,.85)}.focus\:caret-yellow-200\/90:focus{caret-color:hsla(53,98%,77%,.9)}.focus\:caret-yellow-200\/95:focus{caret-color:hsla(53,98%,77%,.95)}.focus\:caret-yellow-400:focus{caret-color:#facc15}.focus\:caret-yellow-400\/0:focus{caret-color:rgba(250,204,21,0)}.focus\:caret-yellow-400\/10:focus{caret-color:rgba(250,204,21,.1)}.focus\:caret-yellow-400\/100:focus{caret-color:#facc15}.focus\:caret-yellow-400\/15:focus{caret-color:rgba(250,204,21,.15)}.focus\:caret-yellow-400\/20:focus{caret-color:rgba(250,204,21,.2)}.focus\:caret-yellow-400\/25:focus{caret-color:rgba(250,204,21,.25)}.focus\:caret-yellow-400\/30:focus{caret-color:rgba(250,204,21,.3)}.focus\:caret-yellow-400\/35:focus{caret-color:rgba(250,204,21,.35)}.focus\:caret-yellow-400\/40:focus{caret-color:rgba(250,204,21,.4)}.focus\:caret-yellow-400\/45:focus{caret-color:rgba(250,204,21,.45)}.focus\:caret-yellow-400\/5:focus{caret-color:rgba(250,204,21,.05)}.focus\:caret-yellow-400\/50:focus{caret-color:rgba(250,204,21,.5)}.focus\:caret-yellow-400\/55:focus{caret-color:rgba(250,204,21,.55)}.focus\:caret-yellow-400\/60:focus{caret-color:rgba(250,204,21,.6)}.focus\:caret-yellow-400\/65:focus{caret-color:rgba(250,204,21,.65)}.focus\:caret-yellow-400\/70:focus{caret-color:rgba(250,204,21,.7)}.focus\:caret-yellow-400\/75:focus{caret-color:rgba(250,204,21,.75)}.focus\:caret-yellow-400\/80:focus{caret-color:rgba(250,204,21,.8)}.focus\:caret-yellow-400\/85:focus{caret-color:rgba(250,204,21,.85)}.focus\:caret-yellow-400\/90:focus{caret-color:rgba(250,204,21,.9)}.focus\:caret-yellow-400\/95:focus{caret-color:rgba(250,204,21,.95)}.focus\:caret-yellow-50:focus{caret-color:#fefce8}.focus\:caret-yellow-50\/0:focus{caret-color:hsla(55,92%,95%,0)}.focus\:caret-yellow-50\/10:focus{caret-color:hsla(55,92%,95%,.1)}.focus\:caret-yellow-50\/100:focus{caret-color:#fefce8}.focus\:caret-yellow-50\/15:focus{caret-color:hsla(55,92%,95%,.15)}.focus\:caret-yellow-50\/20:focus{caret-color:hsla(55,92%,95%,.2)}.focus\:caret-yellow-50\/25:focus{caret-color:hsla(55,92%,95%,.25)}.focus\:caret-yellow-50\/30:focus{caret-color:hsla(55,92%,95%,.3)}.focus\:caret-yellow-50\/35:focus{caret-color:hsla(55,92%,95%,.35)}.focus\:caret-yellow-50\/40:focus{caret-color:hsla(55,92%,95%,.4)}.focus\:caret-yellow-50\/45:focus{caret-color:hsla(55,92%,95%,.45)}.focus\:caret-yellow-50\/5:focus{caret-color:hsla(55,92%,95%,.05)}.focus\:caret-yellow-50\/50:focus{caret-color:hsla(55,92%,95%,.5)}.focus\:caret-yellow-50\/55:focus{caret-color:hsla(55,92%,95%,.55)}.focus\:caret-yellow-50\/60:focus{caret-color:hsla(55,92%,95%,.6)}.focus\:caret-yellow-50\/65:focus{caret-color:hsla(55,92%,95%,.65)}.focus\:caret-yellow-50\/70:focus{caret-color:hsla(55,92%,95%,.7)}.focus\:caret-yellow-50\/75:focus{caret-color:hsla(55,92%,95%,.75)}.focus\:caret-yellow-50\/80:focus{caret-color:hsla(55,92%,95%,.8)}.focus\:caret-yellow-50\/85:focus{caret-color:hsla(55,92%,95%,.85)}.focus\:caret-yellow-50\/90:focus{caret-color:hsla(55,92%,95%,.9)}.focus\:caret-yellow-50\/95:focus{caret-color:hsla(55,92%,95%,.95)}.focus\:caret-yellow-500:focus{caret-color:#eab308}.focus\:caret-yellow-500\/0:focus{caret-color:rgba(234,179,8,0)}.focus\:caret-yellow-500\/10:focus{caret-color:rgba(234,179,8,.1)}.focus\:caret-yellow-500\/100:focus{caret-color:#eab308}.focus\:caret-yellow-500\/15:focus{caret-color:rgba(234,179,8,.15)}.focus\:caret-yellow-500\/20:focus{caret-color:rgba(234,179,8,.2)}.focus\:caret-yellow-500\/25:focus{caret-color:rgba(234,179,8,.25)}.focus\:caret-yellow-500\/30:focus{caret-color:rgba(234,179,8,.3)}.focus\:caret-yellow-500\/35:focus{caret-color:rgba(234,179,8,.35)}.focus\:caret-yellow-500\/40:focus{caret-color:rgba(234,179,8,.4)}.focus\:caret-yellow-500\/45:focus{caret-color:rgba(234,179,8,.45)}.focus\:caret-yellow-500\/5:focus{caret-color:rgba(234,179,8,.05)}.focus\:caret-yellow-500\/50:focus{caret-color:rgba(234,179,8,.5)}.focus\:caret-yellow-500\/55:focus{caret-color:rgba(234,179,8,.55)}.focus\:caret-yellow-500\/60:focus{caret-color:rgba(234,179,8,.6)}.focus\:caret-yellow-500\/65:focus{caret-color:rgba(234,179,8,.65)}.focus\:caret-yellow-500\/70:focus{caret-color:rgba(234,179,8,.7)}.focus\:caret-yellow-500\/75:focus{caret-color:rgba(234,179,8,.75)}.focus\:caret-yellow-500\/80:focus{caret-color:rgba(234,179,8,.8)}.focus\:caret-yellow-500\/85:focus{caret-color:rgba(234,179,8,.85)}.focus\:caret-yellow-500\/90:focus{caret-color:rgba(234,179,8,.9)}.focus\:caret-yellow-500\/95:focus{caret-color:rgba(234,179,8,.95)}.focus\:caret-yellow-950:focus{caret-color:#422006}.focus\:caret-yellow-950\/0:focus{caret-color:rgba(66,32,6,0)}.focus\:caret-yellow-950\/10:focus{caret-color:rgba(66,32,6,.1)}.focus\:caret-yellow-950\/100:focus{caret-color:#422006}.focus\:caret-yellow-950\/15:focus{caret-color:rgba(66,32,6,.15)}.focus\:caret-yellow-950\/20:focus{caret-color:rgba(66,32,6,.2)}.focus\:caret-yellow-950\/25:focus{caret-color:rgba(66,32,6,.25)}.focus\:caret-yellow-950\/30:focus{caret-color:rgba(66,32,6,.3)}.focus\:caret-yellow-950\/35:focus{caret-color:rgba(66,32,6,.35)}.focus\:caret-yellow-950\/40:focus{caret-color:rgba(66,32,6,.4)}.focus\:caret-yellow-950\/45:focus{caret-color:rgba(66,32,6,.45)}.focus\:caret-yellow-950\/5:focus{caret-color:rgba(66,32,6,.05)}.focus\:caret-yellow-950\/50:focus{caret-color:rgba(66,32,6,.5)}.focus\:caret-yellow-950\/55:focus{caret-color:rgba(66,32,6,.55)}.focus\:caret-yellow-950\/60:focus{caret-color:rgba(66,32,6,.6)}.focus\:caret-yellow-950\/65:focus{caret-color:rgba(66,32,6,.65)}.focus\:caret-yellow-950\/70:focus{caret-color:rgba(66,32,6,.7)}.focus\:caret-yellow-950\/75:focus{caret-color:rgba(66,32,6,.75)}.focus\:caret-yellow-950\/80:focus{caret-color:rgba(66,32,6,.8)}.focus\:caret-yellow-950\/85:focus{caret-color:rgba(66,32,6,.85)}.focus\:caret-yellow-950\/90:focus{caret-color:rgba(66,32,6,.9)}.focus\:caret-yellow-950\/95:focus{caret-color:rgba(66,32,6,.95)}.focus\:caret-zinc-100:focus{caret-color:#f4f4f5}.focus\:caret-zinc-100\/0:focus{caret-color:hsla(240,5%,96%,0)}.focus\:caret-zinc-100\/10:focus{caret-color:hsla(240,5%,96%,.1)}.focus\:caret-zinc-100\/100:focus{caret-color:#f4f4f5}.focus\:caret-zinc-100\/15:focus{caret-color:hsla(240,5%,96%,.15)}.focus\:caret-zinc-100\/20:focus{caret-color:hsla(240,5%,96%,.2)}.focus\:caret-zinc-100\/25:focus{caret-color:hsla(240,5%,96%,.25)}.focus\:caret-zinc-100\/30:focus{caret-color:hsla(240,5%,96%,.3)}.focus\:caret-zinc-100\/35:focus{caret-color:hsla(240,5%,96%,.35)}.focus\:caret-zinc-100\/40:focus{caret-color:hsla(240,5%,96%,.4)}.focus\:caret-zinc-100\/45:focus{caret-color:hsla(240,5%,96%,.45)}.focus\:caret-zinc-100\/5:focus{caret-color:hsla(240,5%,96%,.05)}.focus\:caret-zinc-100\/50:focus{caret-color:hsla(240,5%,96%,.5)}.focus\:caret-zinc-100\/55:focus{caret-color:hsla(240,5%,96%,.55)}.focus\:caret-zinc-100\/60:focus{caret-color:hsla(240,5%,96%,.6)}.focus\:caret-zinc-100\/65:focus{caret-color:hsla(240,5%,96%,.65)}.focus\:caret-zinc-100\/70:focus{caret-color:hsla(240,5%,96%,.7)}.focus\:caret-zinc-100\/75:focus{caret-color:hsla(240,5%,96%,.75)}.focus\:caret-zinc-100\/80:focus{caret-color:hsla(240,5%,96%,.8)}.focus\:caret-zinc-100\/85:focus{caret-color:hsla(240,5%,96%,.85)}.focus\:caret-zinc-100\/90:focus{caret-color:hsla(240,5%,96%,.9)}.focus\:caret-zinc-100\/95:focus{caret-color:hsla(240,5%,96%,.95)}.focus\:caret-zinc-200:focus{caret-color:#e4e4e7}.focus\:caret-zinc-200\/0:focus{caret-color:hsla(240,6%,90%,0)}.focus\:caret-zinc-200\/10:focus{caret-color:hsla(240,6%,90%,.1)}.focus\:caret-zinc-200\/100:focus{caret-color:#e4e4e7}.focus\:caret-zinc-200\/15:focus{caret-color:hsla(240,6%,90%,.15)}.focus\:caret-zinc-200\/20:focus{caret-color:hsla(240,6%,90%,.2)}.focus\:caret-zinc-200\/25:focus{caret-color:hsla(240,6%,90%,.25)}.focus\:caret-zinc-200\/30:focus{caret-color:hsla(240,6%,90%,.3)}.focus\:caret-zinc-200\/35:focus{caret-color:hsla(240,6%,90%,.35)}.focus\:caret-zinc-200\/40:focus{caret-color:hsla(240,6%,90%,.4)}.focus\:caret-zinc-200\/45:focus{caret-color:hsla(240,6%,90%,.45)}.focus\:caret-zinc-200\/5:focus{caret-color:hsla(240,6%,90%,.05)}.focus\:caret-zinc-200\/50:focus{caret-color:hsla(240,6%,90%,.5)}.focus\:caret-zinc-200\/55:focus{caret-color:hsla(240,6%,90%,.55)}.focus\:caret-zinc-200\/60:focus{caret-color:hsla(240,6%,90%,.6)}.focus\:caret-zinc-200\/65:focus{caret-color:hsla(240,6%,90%,.65)}.focus\:caret-zinc-200\/70:focus{caret-color:hsla(240,6%,90%,.7)}.focus\:caret-zinc-200\/75:focus{caret-color:hsla(240,6%,90%,.75)}.focus\:caret-zinc-200\/80:focus{caret-color:hsla(240,6%,90%,.8)}.focus\:caret-zinc-200\/85:focus{caret-color:hsla(240,6%,90%,.85)}.focus\:caret-zinc-200\/90:focus{caret-color:hsla(240,6%,90%,.9)}.focus\:caret-zinc-200\/95:focus{caret-color:hsla(240,6%,90%,.95)}.focus\:caret-zinc-400:focus{caret-color:#a1a1aa}.focus\:caret-zinc-400\/0:focus{caret-color:hsla(240,5%,65%,0)}.focus\:caret-zinc-400\/10:focus{caret-color:hsla(240,5%,65%,.1)}.focus\:caret-zinc-400\/100:focus{caret-color:#a1a1aa}.focus\:caret-zinc-400\/15:focus{caret-color:hsla(240,5%,65%,.15)}.focus\:caret-zinc-400\/20:focus{caret-color:hsla(240,5%,65%,.2)}.focus\:caret-zinc-400\/25:focus{caret-color:hsla(240,5%,65%,.25)}.focus\:caret-zinc-400\/30:focus{caret-color:hsla(240,5%,65%,.3)}.focus\:caret-zinc-400\/35:focus{caret-color:hsla(240,5%,65%,.35)}.focus\:caret-zinc-400\/40:focus{caret-color:hsla(240,5%,65%,.4)}.focus\:caret-zinc-400\/45:focus{caret-color:hsla(240,5%,65%,.45)}.focus\:caret-zinc-400\/5:focus{caret-color:hsla(240,5%,65%,.05)}.focus\:caret-zinc-400\/50:focus{caret-color:hsla(240,5%,65%,.5)}.focus\:caret-zinc-400\/55:focus{caret-color:hsla(240,5%,65%,.55)}.focus\:caret-zinc-400\/60:focus{caret-color:hsla(240,5%,65%,.6)}.focus\:caret-zinc-400\/65:focus{caret-color:hsla(240,5%,65%,.65)}.focus\:caret-zinc-400\/70:focus{caret-color:hsla(240,5%,65%,.7)}.focus\:caret-zinc-400\/75:focus{caret-color:hsla(240,5%,65%,.75)}.focus\:caret-zinc-400\/80:focus{caret-color:hsla(240,5%,65%,.8)}.focus\:caret-zinc-400\/85:focus{caret-color:hsla(240,5%,65%,.85)}.focus\:caret-zinc-400\/90:focus{caret-color:hsla(240,5%,65%,.9)}.focus\:caret-zinc-400\/95:focus{caret-color:hsla(240,5%,65%,.95)}.focus\:caret-zinc-50:focus{caret-color:#fafafa}.focus\:caret-zinc-50\/0:focus{caret-color:hsla(0,0%,98%,0)}.focus\:caret-zinc-50\/10:focus{caret-color:hsla(0,0%,98%,.1)}.focus\:caret-zinc-50\/100:focus{caret-color:#fafafa}.focus\:caret-zinc-50\/15:focus{caret-color:hsla(0,0%,98%,.15)}.focus\:caret-zinc-50\/20:focus{caret-color:hsla(0,0%,98%,.2)}.focus\:caret-zinc-50\/25:focus{caret-color:hsla(0,0%,98%,.25)}.focus\:caret-zinc-50\/30:focus{caret-color:hsla(0,0%,98%,.3)}.focus\:caret-zinc-50\/35:focus{caret-color:hsla(0,0%,98%,.35)}.focus\:caret-zinc-50\/40:focus{caret-color:hsla(0,0%,98%,.4)}.focus\:caret-zinc-50\/45:focus{caret-color:hsla(0,0%,98%,.45)}.focus\:caret-zinc-50\/5:focus{caret-color:hsla(0,0%,98%,.05)}.focus\:caret-zinc-50\/50:focus{caret-color:hsla(0,0%,98%,.5)}.focus\:caret-zinc-50\/55:focus{caret-color:hsla(0,0%,98%,.55)}.focus\:caret-zinc-50\/60:focus{caret-color:hsla(0,0%,98%,.6)}.focus\:caret-zinc-50\/65:focus{caret-color:hsla(0,0%,98%,.65)}.focus\:caret-zinc-50\/70:focus{caret-color:hsla(0,0%,98%,.7)}.focus\:caret-zinc-50\/75:focus{caret-color:hsla(0,0%,98%,.75)}.focus\:caret-zinc-50\/80:focus{caret-color:hsla(0,0%,98%,.8)}.focus\:caret-zinc-50\/85:focus{caret-color:hsla(0,0%,98%,.85)}.focus\:caret-zinc-50\/90:focus{caret-color:hsla(0,0%,98%,.9)}.focus\:caret-zinc-50\/95:focus{caret-color:hsla(0,0%,98%,.95)}.focus\:caret-zinc-500:focus{caret-color:#71717a}.focus\:caret-zinc-500\/0:focus{caret-color:hsla(240,4%,46%,0)}.focus\:caret-zinc-500\/10:focus{caret-color:hsla(240,4%,46%,.1)}.focus\:caret-zinc-500\/100:focus{caret-color:#71717a}.focus\:caret-zinc-500\/15:focus{caret-color:hsla(240,4%,46%,.15)}.focus\:caret-zinc-500\/20:focus{caret-color:hsla(240,4%,46%,.2)}.focus\:caret-zinc-500\/25:focus{caret-color:hsla(240,4%,46%,.25)}.focus\:caret-zinc-500\/30:focus{caret-color:hsla(240,4%,46%,.3)}.focus\:caret-zinc-500\/35:focus{caret-color:hsla(240,4%,46%,.35)}.focus\:caret-zinc-500\/40:focus{caret-color:hsla(240,4%,46%,.4)}.focus\:caret-zinc-500\/45:focus{caret-color:hsla(240,4%,46%,.45)}.focus\:caret-zinc-500\/5:focus{caret-color:hsla(240,4%,46%,.05)}.focus\:caret-zinc-500\/50:focus{caret-color:hsla(240,4%,46%,.5)}.focus\:caret-zinc-500\/55:focus{caret-color:hsla(240,4%,46%,.55)}.focus\:caret-zinc-500\/60:focus{caret-color:hsla(240,4%,46%,.6)}.focus\:caret-zinc-500\/65:focus{caret-color:hsla(240,4%,46%,.65)}.focus\:caret-zinc-500\/70:focus{caret-color:hsla(240,4%,46%,.7)}.focus\:caret-zinc-500\/75:focus{caret-color:hsla(240,4%,46%,.75)}.focus\:caret-zinc-500\/80:focus{caret-color:hsla(240,4%,46%,.8)}.focus\:caret-zinc-500\/85:focus{caret-color:hsla(240,4%,46%,.85)}.focus\:caret-zinc-500\/90:focus{caret-color:hsla(240,4%,46%,.9)}.focus\:caret-zinc-500\/95:focus{caret-color:hsla(240,4%,46%,.95)}.focus\:caret-zinc-950:focus{caret-color:#09090b}.focus\:caret-zinc-950\/0:focus{caret-color:rgba(9,9,11,0)}.focus\:caret-zinc-950\/10:focus{caret-color:rgba(9,9,11,.1)}.focus\:caret-zinc-950\/100:focus{caret-color:#09090b}.focus\:caret-zinc-950\/15:focus{caret-color:rgba(9,9,11,.15)}.focus\:caret-zinc-950\/20:focus{caret-color:rgba(9,9,11,.2)}.focus\:caret-zinc-950\/25:focus{caret-color:rgba(9,9,11,.25)}.focus\:caret-zinc-950\/30:focus{caret-color:rgba(9,9,11,.3)}.focus\:caret-zinc-950\/35:focus{caret-color:rgba(9,9,11,.35)}.focus\:caret-zinc-950\/40:focus{caret-color:rgba(9,9,11,.4)}.focus\:caret-zinc-950\/45:focus{caret-color:rgba(9,9,11,.45)}.focus\:caret-zinc-950\/5:focus{caret-color:rgba(9,9,11,.05)}.focus\:caret-zinc-950\/50:focus{caret-color:rgba(9,9,11,.5)}.focus\:caret-zinc-950\/55:focus{caret-color:rgba(9,9,11,.55)}.focus\:caret-zinc-950\/60:focus{caret-color:rgba(9,9,11,.6)}.focus\:caret-zinc-950\/65:focus{caret-color:rgba(9,9,11,.65)}.focus\:caret-zinc-950\/70:focus{caret-color:rgba(9,9,11,.7)}.focus\:caret-zinc-950\/75:focus{caret-color:rgba(9,9,11,.75)}.focus\:caret-zinc-950\/80:focus{caret-color:rgba(9,9,11,.8)}.focus\:caret-zinc-950\/85:focus{caret-color:rgba(9,9,11,.85)}.focus\:caret-zinc-950\/90:focus{caret-color:rgba(9,9,11,.9)}.focus\:caret-zinc-950\/95:focus{caret-color:rgba(9,9,11,.95)}.focus\:opacity-100:focus{opacity:1}.focus\:outline-none:focus{outline:2px solid transparent;outline-offset:2px}.focus\:ring-2:focus{--tw-ring-offset-shadow:var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);--tw-ring-shadow:var(--tw-ring-inset) 0 0 0 calc(2px + var(--tw-ring-offset-width)) var(--tw-ring-color);box-shadow:var(--tw-ring-offset-shadow),var(--tw-ring-shadow),var(--tw-shadow,0 0 #0000)}.focus\:ring-ring:focus{--tw-ring-color:hsl(var(--ring))}.focus\:ring-offset-2:focus{--tw-ring-offset-width:2px}.focus-visible\:outline-none:focus-visible{outline:2px solid transparent;outline-offset:2px}.focus-visible\:ring-2:focus-visible{--tw-ring-offset-shadow:var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);--tw-ring-shadow:var(--tw-ring-inset) 0 0 0 calc(2px + var(--tw-ring-offset-width)) var(--tw-ring-color);box-shadow:var(--tw-ring-offset-shadow),var(--tw-ring-shadow),var(--tw-shadow,0 0 #0000)}.focus-visible\:ring-amber-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(254 243 199/var(--tw-ring-opacity,1))}.focus-visible\:ring-amber-100\/0:focus-visible{--tw-ring-color:hsla(48,96%,89%,0)}.focus-visible\:ring-amber-100\/10:focus-visible{--tw-ring-color:hsla(48,96%,89%,.1)}.focus-visible\:ring-amber-100\/100:focus-visible{--tw-ring-color:#fef3c7}.focus-visible\:ring-amber-100\/15:focus-visible{--tw-ring-color:hsla(48,96%,89%,.15)}.focus-visible\:ring-amber-100\/20:focus-visible{--tw-ring-color:hsla(48,96%,89%,.2)}.focus-visible\:ring-amber-100\/25:focus-visible{--tw-ring-color:hsla(48,96%,89%,.25)}.focus-visible\:ring-amber-100\/30:focus-visible{--tw-ring-color:hsla(48,96%,89%,.3)}.focus-visible\:ring-amber-100\/35:focus-visible{--tw-ring-color:hsla(48,96%,89%,.35)}.focus-visible\:ring-amber-100\/40:focus-visible{--tw-ring-color:hsla(48,96%,89%,.4)}.focus-visible\:ring-amber-100\/45:focus-visible{--tw-ring-color:hsla(48,96%,89%,.45)}.focus-visible\:ring-amber-100\/5:focus-visible{--tw-ring-color:hsla(48,96%,89%,.05)}.focus-visible\:ring-amber-100\/50:focus-visible{--tw-ring-color:hsla(48,96%,89%,.5)}.focus-visible\:ring-amber-100\/55:focus-visible{--tw-ring-color:hsla(48,96%,89%,.55)}.focus-visible\:ring-amber-100\/60:focus-visible{--tw-ring-color:hsla(48,96%,89%,.6)}.focus-visible\:ring-amber-100\/65:focus-visible{--tw-ring-color:hsla(48,96%,89%,.65)}.focus-visible\:ring-amber-100\/70:focus-visible{--tw-ring-color:hsla(48,96%,89%,.7)}.focus-visible\:ring-amber-100\/75:focus-visible{--tw-ring-color:hsla(48,96%,89%,.75)}.focus-visible\:ring-amber-100\/80:focus-visible{--tw-ring-color:hsla(48,96%,89%,.8)}.focus-visible\:ring-amber-100\/85:focus-visible{--tw-ring-color:hsla(48,96%,89%,.85)}.focus-visible\:ring-amber-100\/90:focus-visible{--tw-ring-color:hsla(48,96%,89%,.9)}.focus-visible\:ring-amber-100\/95:focus-visible{--tw-ring-color:hsla(48,96%,89%,.95)}.focus-visible\:ring-amber-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(253 230 138/var(--tw-ring-opacity,1))}.focus-visible\:ring-amber-200\/0:focus-visible{--tw-ring-color:hsla(48,97%,77%,0)}.focus-visible\:ring-amber-200\/10:focus-visible{--tw-ring-color:hsla(48,97%,77%,.1)}.focus-visible\:ring-amber-200\/100:focus-visible{--tw-ring-color:#fde68a}.focus-visible\:ring-amber-200\/15:focus-visible{--tw-ring-color:hsla(48,97%,77%,.15)}.focus-visible\:ring-amber-200\/20:focus-visible{--tw-ring-color:hsla(48,97%,77%,.2)}.focus-visible\:ring-amber-200\/25:focus-visible{--tw-ring-color:hsla(48,97%,77%,.25)}.focus-visible\:ring-amber-200\/30:focus-visible{--tw-ring-color:hsla(48,97%,77%,.3)}.focus-visible\:ring-amber-200\/35:focus-visible{--tw-ring-color:hsla(48,97%,77%,.35)}.focus-visible\:ring-amber-200\/40:focus-visible{--tw-ring-color:hsla(48,97%,77%,.4)}.focus-visible\:ring-amber-200\/45:focus-visible{--tw-ring-color:hsla(48,97%,77%,.45)}.focus-visible\:ring-amber-200\/5:focus-visible{--tw-ring-color:hsla(48,97%,77%,.05)}.focus-visible\:ring-amber-200\/50:focus-visible{--tw-ring-color:hsla(48,97%,77%,.5)}.focus-visible\:ring-amber-200\/55:focus-visible{--tw-ring-color:hsla(48,97%,77%,.55)}.focus-visible\:ring-amber-200\/60:focus-visible{--tw-ring-color:hsla(48,97%,77%,.6)}.focus-visible\:ring-amber-200\/65:focus-visible{--tw-ring-color:hsla(48,97%,77%,.65)}.focus-visible\:ring-amber-200\/70:focus-visible{--tw-ring-color:hsla(48,97%,77%,.7)}.focus-visible\:ring-amber-200\/75:focus-visible{--tw-ring-color:hsla(48,97%,77%,.75)}.focus-visible\:ring-amber-200\/80:focus-visible{--tw-ring-color:hsla(48,97%,77%,.8)}.focus-visible\:ring-amber-200\/85:focus-visible{--tw-ring-color:hsla(48,97%,77%,.85)}.focus-visible\:ring-amber-200\/90:focus-visible{--tw-ring-color:hsla(48,97%,77%,.9)}.focus-visible\:ring-amber-200\/95:focus-visible{--tw-ring-color:hsla(48,97%,77%,.95)}.focus-visible\:ring-amber-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(251 191 36/var(--tw-ring-opacity,1))}.focus-visible\:ring-amber-400\/0:focus-visible{--tw-ring-color:rgba(251,191,36,0)}.focus-visible\:ring-amber-400\/10:focus-visible{--tw-ring-color:rgba(251,191,36,.1)}.focus-visible\:ring-amber-400\/100:focus-visible{--tw-ring-color:#fbbf24}.focus-visible\:ring-amber-400\/15:focus-visible{--tw-ring-color:rgba(251,191,36,.15)}.focus-visible\:ring-amber-400\/20:focus-visible{--tw-ring-color:rgba(251,191,36,.2)}.focus-visible\:ring-amber-400\/25:focus-visible{--tw-ring-color:rgba(251,191,36,.25)}.focus-visible\:ring-amber-400\/30:focus-visible{--tw-ring-color:rgba(251,191,36,.3)}.focus-visible\:ring-amber-400\/35:focus-visible{--tw-ring-color:rgba(251,191,36,.35)}.focus-visible\:ring-amber-400\/40:focus-visible{--tw-ring-color:rgba(251,191,36,.4)}.focus-visible\:ring-amber-400\/45:focus-visible{--tw-ring-color:rgba(251,191,36,.45)}.focus-visible\:ring-amber-400\/5:focus-visible{--tw-ring-color:rgba(251,191,36,.05)}.focus-visible\:ring-amber-400\/50:focus-visible{--tw-ring-color:rgba(251,191,36,.5)}.focus-visible\:ring-amber-400\/55:focus-visible{--tw-ring-color:rgba(251,191,36,.55)}.focus-visible\:ring-amber-400\/60:focus-visible{--tw-ring-color:rgba(251,191,36,.6)}.focus-visible\:ring-amber-400\/65:focus-visible{--tw-ring-color:rgba(251,191,36,.65)}.focus-visible\:ring-amber-400\/70:focus-visible{--tw-ring-color:rgba(251,191,36,.7)}.focus-visible\:ring-amber-400\/75:focus-visible{--tw-ring-color:rgba(251,191,36,.75)}.focus-visible\:ring-amber-400\/80:focus-visible{--tw-ring-color:rgba(251,191,36,.8)}.focus-visible\:ring-amber-400\/85:focus-visible{--tw-ring-color:rgba(251,191,36,.85)}.focus-visible\:ring-amber-400\/90:focus-visible{--tw-ring-color:rgba(251,191,36,.9)}.focus-visible\:ring-amber-400\/95:focus-visible{--tw-ring-color:rgba(251,191,36,.95)}.focus-visible\:ring-amber-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(255 251 235/var(--tw-ring-opacity,1))}.focus-visible\:ring-amber-50\/0:focus-visible{--tw-ring-color:rgba(255,251,235,0)}.focus-visible\:ring-amber-50\/10:focus-visible{--tw-ring-color:rgba(255,251,235,.1)}.focus-visible\:ring-amber-50\/100:focus-visible{--tw-ring-color:#fffbeb}.focus-visible\:ring-amber-50\/15:focus-visible{--tw-ring-color:rgba(255,251,235,.15)}.focus-visible\:ring-amber-50\/20:focus-visible{--tw-ring-color:rgba(255,251,235,.2)}.focus-visible\:ring-amber-50\/25:focus-visible{--tw-ring-color:rgba(255,251,235,.25)}.focus-visible\:ring-amber-50\/30:focus-visible{--tw-ring-color:rgba(255,251,235,.3)}.focus-visible\:ring-amber-50\/35:focus-visible{--tw-ring-color:rgba(255,251,235,.35)}.focus-visible\:ring-amber-50\/40:focus-visible{--tw-ring-color:rgba(255,251,235,.4)}.focus-visible\:ring-amber-50\/45:focus-visible{--tw-ring-color:rgba(255,251,235,.45)}.focus-visible\:ring-amber-50\/5:focus-visible{--tw-ring-color:rgba(255,251,235,.05)}.focus-visible\:ring-amber-50\/50:focus-visible{--tw-ring-color:rgba(255,251,235,.5)}.focus-visible\:ring-amber-50\/55:focus-visible{--tw-ring-color:rgba(255,251,235,.55)}.focus-visible\:ring-amber-50\/60:focus-visible{--tw-ring-color:rgba(255,251,235,.6)}.focus-visible\:ring-amber-50\/65:focus-visible{--tw-ring-color:rgba(255,251,235,.65)}.focus-visible\:ring-amber-50\/70:focus-visible{--tw-ring-color:rgba(255,251,235,.7)}.focus-visible\:ring-amber-50\/75:focus-visible{--tw-ring-color:rgba(255,251,235,.75)}.focus-visible\:ring-amber-50\/80:focus-visible{--tw-ring-color:rgba(255,251,235,.8)}.focus-visible\:ring-amber-50\/85:focus-visible{--tw-ring-color:rgba(255,251,235,.85)}.focus-visible\:ring-amber-50\/90:focus-visible{--tw-ring-color:rgba(255,251,235,.9)}.focus-visible\:ring-amber-50\/95:focus-visible{--tw-ring-color:rgba(255,251,235,.95)}.focus-visible\:ring-amber-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(245 158 11/var(--tw-ring-opacity,1))}.focus-visible\:ring-amber-500\/0:focus-visible{--tw-ring-color:rgba(245,158,11,0)}.focus-visible\:ring-amber-500\/10:focus-visible{--tw-ring-color:rgba(245,158,11,.1)}.focus-visible\:ring-amber-500\/100:focus-visible{--tw-ring-color:#f59e0b}.focus-visible\:ring-amber-500\/15:focus-visible{--tw-ring-color:rgba(245,158,11,.15)}.focus-visible\:ring-amber-500\/20:focus-visible{--tw-ring-color:rgba(245,158,11,.2)}.focus-visible\:ring-amber-500\/25:focus-visible{--tw-ring-color:rgba(245,158,11,.25)}.focus-visible\:ring-amber-500\/30:focus-visible{--tw-ring-color:rgba(245,158,11,.3)}.focus-visible\:ring-amber-500\/35:focus-visible{--tw-ring-color:rgba(245,158,11,.35)}.focus-visible\:ring-amber-500\/40:focus-visible{--tw-ring-color:rgba(245,158,11,.4)}.focus-visible\:ring-amber-500\/45:focus-visible{--tw-ring-color:rgba(245,158,11,.45)}.focus-visible\:ring-amber-500\/5:focus-visible{--tw-ring-color:rgba(245,158,11,.05)}.focus-visible\:ring-amber-500\/50:focus-visible{--tw-ring-color:rgba(245,158,11,.5)}.focus-visible\:ring-amber-500\/55:focus-visible{--tw-ring-color:rgba(245,158,11,.55)}.focus-visible\:ring-amber-500\/60:focus-visible{--tw-ring-color:rgba(245,158,11,.6)}.focus-visible\:ring-amber-500\/65:focus-visible{--tw-ring-color:rgba(245,158,11,.65)}.focus-visible\:ring-amber-500\/70:focus-visible{--tw-ring-color:rgba(245,158,11,.7)}.focus-visible\:ring-amber-500\/75:focus-visible{--tw-ring-color:rgba(245,158,11,.75)}.focus-visible\:ring-amber-500\/80:focus-visible{--tw-ring-color:rgba(245,158,11,.8)}.focus-visible\:ring-amber-500\/85:focus-visible{--tw-ring-color:rgba(245,158,11,.85)}.focus-visible\:ring-amber-500\/90:focus-visible{--tw-ring-color:rgba(245,158,11,.9)}.focus-visible\:ring-amber-500\/95:focus-visible{--tw-ring-color:rgba(245,158,11,.95)}.focus-visible\:ring-amber-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(69 26 3/var(--tw-ring-opacity,1))}.focus-visible\:ring-amber-950\/0:focus-visible{--tw-ring-color:rgba(69,26,3,0)}.focus-visible\:ring-amber-950\/10:focus-visible{--tw-ring-color:rgba(69,26,3,.1)}.focus-visible\:ring-amber-950\/100:focus-visible{--tw-ring-color:#451a03}.focus-visible\:ring-amber-950\/15:focus-visible{--tw-ring-color:rgba(69,26,3,.15)}.focus-visible\:ring-amber-950\/20:focus-visible{--tw-ring-color:rgba(69,26,3,.2)}.focus-visible\:ring-amber-950\/25:focus-visible{--tw-ring-color:rgba(69,26,3,.25)}.focus-visible\:ring-amber-950\/30:focus-visible{--tw-ring-color:rgba(69,26,3,.3)}.focus-visible\:ring-amber-950\/35:focus-visible{--tw-ring-color:rgba(69,26,3,.35)}.focus-visible\:ring-amber-950\/40:focus-visible{--tw-ring-color:rgba(69,26,3,.4)}.focus-visible\:ring-amber-950\/45:focus-visible{--tw-ring-color:rgba(69,26,3,.45)}.focus-visible\:ring-amber-950\/5:focus-visible{--tw-ring-color:rgba(69,26,3,.05)}.focus-visible\:ring-amber-950\/50:focus-visible{--tw-ring-color:rgba(69,26,3,.5)}.focus-visible\:ring-amber-950\/55:focus-visible{--tw-ring-color:rgba(69,26,3,.55)}.focus-visible\:ring-amber-950\/60:focus-visible{--tw-ring-color:rgba(69,26,3,.6)}.focus-visible\:ring-amber-950\/65:focus-visible{--tw-ring-color:rgba(69,26,3,.65)}.focus-visible\:ring-amber-950\/70:focus-visible{--tw-ring-color:rgba(69,26,3,.7)}.focus-visible\:ring-amber-950\/75:focus-visible{--tw-ring-color:rgba(69,26,3,.75)}.focus-visible\:ring-amber-950\/80:focus-visible{--tw-ring-color:rgba(69,26,3,.8)}.focus-visible\:ring-amber-950\/85:focus-visible{--tw-ring-color:rgba(69,26,3,.85)}.focus-visible\:ring-amber-950\/90:focus-visible{--tw-ring-color:rgba(69,26,3,.9)}.focus-visible\:ring-amber-950\/95:focus-visible{--tw-ring-color:rgba(69,26,3,.95)}.focus-visible\:ring-blue-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(219 234 254/var(--tw-ring-opacity,1))}.focus-visible\:ring-blue-100\/0:focus-visible{--tw-ring-color:rgba(219,234,254,0)}.focus-visible\:ring-blue-100\/10:focus-visible{--tw-ring-color:rgba(219,234,254,.1)}.focus-visible\:ring-blue-100\/100:focus-visible{--tw-ring-color:#dbeafe}.focus-visible\:ring-blue-100\/15:focus-visible{--tw-ring-color:rgba(219,234,254,.15)}.focus-visible\:ring-blue-100\/20:focus-visible{--tw-ring-color:rgba(219,234,254,.2)}.focus-visible\:ring-blue-100\/25:focus-visible{--tw-ring-color:rgba(219,234,254,.25)}.focus-visible\:ring-blue-100\/30:focus-visible{--tw-ring-color:rgba(219,234,254,.3)}.focus-visible\:ring-blue-100\/35:focus-visible{--tw-ring-color:rgba(219,234,254,.35)}.focus-visible\:ring-blue-100\/40:focus-visible{--tw-ring-color:rgba(219,234,254,.4)}.focus-visible\:ring-blue-100\/45:focus-visible{--tw-ring-color:rgba(219,234,254,.45)}.focus-visible\:ring-blue-100\/5:focus-visible{--tw-ring-color:rgba(219,234,254,.05)}.focus-visible\:ring-blue-100\/50:focus-visible{--tw-ring-color:rgba(219,234,254,.5)}.focus-visible\:ring-blue-100\/55:focus-visible{--tw-ring-color:rgba(219,234,254,.55)}.focus-visible\:ring-blue-100\/60:focus-visible{--tw-ring-color:rgba(219,234,254,.6)}.focus-visible\:ring-blue-100\/65:focus-visible{--tw-ring-color:rgba(219,234,254,.65)}.focus-visible\:ring-blue-100\/70:focus-visible{--tw-ring-color:rgba(219,234,254,.7)}.focus-visible\:ring-blue-100\/75:focus-visible{--tw-ring-color:rgba(219,234,254,.75)}.focus-visible\:ring-blue-100\/80:focus-visible{--tw-ring-color:rgba(219,234,254,.8)}.focus-visible\:ring-blue-100\/85:focus-visible{--tw-ring-color:rgba(219,234,254,.85)}.focus-visible\:ring-blue-100\/90:focus-visible{--tw-ring-color:rgba(219,234,254,.9)}.focus-visible\:ring-blue-100\/95:focus-visible{--tw-ring-color:rgba(219,234,254,.95)}.focus-visible\:ring-blue-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(191 219 254/var(--tw-ring-opacity,1))}.focus-visible\:ring-blue-200\/0:focus-visible{--tw-ring-color:rgba(191,219,254,0)}.focus-visible\:ring-blue-200\/10:focus-visible{--tw-ring-color:rgba(191,219,254,.1)}.focus-visible\:ring-blue-200\/100:focus-visible{--tw-ring-color:#bfdbfe}.focus-visible\:ring-blue-200\/15:focus-visible{--tw-ring-color:rgba(191,219,254,.15)}.focus-visible\:ring-blue-200\/20:focus-visible{--tw-ring-color:rgba(191,219,254,.2)}.focus-visible\:ring-blue-200\/25:focus-visible{--tw-ring-color:rgba(191,219,254,.25)}.focus-visible\:ring-blue-200\/30:focus-visible{--tw-ring-color:rgba(191,219,254,.3)}.focus-visible\:ring-blue-200\/35:focus-visible{--tw-ring-color:rgba(191,219,254,.35)}.focus-visible\:ring-blue-200\/40:focus-visible{--tw-ring-color:rgba(191,219,254,.4)}.focus-visible\:ring-blue-200\/45:focus-visible{--tw-ring-color:rgba(191,219,254,.45)}.focus-visible\:ring-blue-200\/5:focus-visible{--tw-ring-color:rgba(191,219,254,.05)}.focus-visible\:ring-blue-200\/50:focus-visible{--tw-ring-color:rgba(191,219,254,.5)}.focus-visible\:ring-blue-200\/55:focus-visible{--tw-ring-color:rgba(191,219,254,.55)}.focus-visible\:ring-blue-200\/60:focus-visible{--tw-ring-color:rgba(191,219,254,.6)}.focus-visible\:ring-blue-200\/65:focus-visible{--tw-ring-color:rgba(191,219,254,.65)}.focus-visible\:ring-blue-200\/70:focus-visible{--tw-ring-color:rgba(191,219,254,.7)}.focus-visible\:ring-blue-200\/75:focus-visible{--tw-ring-color:rgba(191,219,254,.75)}.focus-visible\:ring-blue-200\/80:focus-visible{--tw-ring-color:rgba(191,219,254,.8)}.focus-visible\:ring-blue-200\/85:focus-visible{--tw-ring-color:rgba(191,219,254,.85)}.focus-visible\:ring-blue-200\/90:focus-visible{--tw-ring-color:rgba(191,219,254,.9)}.focus-visible\:ring-blue-200\/95:focus-visible{--tw-ring-color:rgba(191,219,254,.95)}.focus-visible\:ring-blue-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(96 165 250/var(--tw-ring-opacity,1))}.focus-visible\:ring-blue-400\/0:focus-visible{--tw-ring-color:rgba(96,165,250,0)}.focus-visible\:ring-blue-400\/10:focus-visible{--tw-ring-color:rgba(96,165,250,.1)}.focus-visible\:ring-blue-400\/100:focus-visible{--tw-ring-color:#60a5fa}.focus-visible\:ring-blue-400\/15:focus-visible{--tw-ring-color:rgba(96,165,250,.15)}.focus-visible\:ring-blue-400\/20:focus-visible{--tw-ring-color:rgba(96,165,250,.2)}.focus-visible\:ring-blue-400\/25:focus-visible{--tw-ring-color:rgba(96,165,250,.25)}.focus-visible\:ring-blue-400\/30:focus-visible{--tw-ring-color:rgba(96,165,250,.3)}.focus-visible\:ring-blue-400\/35:focus-visible{--tw-ring-color:rgba(96,165,250,.35)}.focus-visible\:ring-blue-400\/40:focus-visible{--tw-ring-color:rgba(96,165,250,.4)}.focus-visible\:ring-blue-400\/45:focus-visible{--tw-ring-color:rgba(96,165,250,.45)}.focus-visible\:ring-blue-400\/5:focus-visible{--tw-ring-color:rgba(96,165,250,.05)}.focus-visible\:ring-blue-400\/50:focus-visible{--tw-ring-color:rgba(96,165,250,.5)}.focus-visible\:ring-blue-400\/55:focus-visible{--tw-ring-color:rgba(96,165,250,.55)}.focus-visible\:ring-blue-400\/60:focus-visible{--tw-ring-color:rgba(96,165,250,.6)}.focus-visible\:ring-blue-400\/65:focus-visible{--tw-ring-color:rgba(96,165,250,.65)}.focus-visible\:ring-blue-400\/70:focus-visible{--tw-ring-color:rgba(96,165,250,.7)}.focus-visible\:ring-blue-400\/75:focus-visible{--tw-ring-color:rgba(96,165,250,.75)}.focus-visible\:ring-blue-400\/80:focus-visible{--tw-ring-color:rgba(96,165,250,.8)}.focus-visible\:ring-blue-400\/85:focus-visible{--tw-ring-color:rgba(96,165,250,.85)}.focus-visible\:ring-blue-400\/90:focus-visible{--tw-ring-color:rgba(96,165,250,.9)}.focus-visible\:ring-blue-400\/95:focus-visible{--tw-ring-color:rgba(96,165,250,.95)}.focus-visible\:ring-blue-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(239 246 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-blue-50\/0:focus-visible{--tw-ring-color:rgba(239,246,255,0)}.focus-visible\:ring-blue-50\/10:focus-visible{--tw-ring-color:rgba(239,246,255,.1)}.focus-visible\:ring-blue-50\/100:focus-visible{--tw-ring-color:#eff6ff}.focus-visible\:ring-blue-50\/15:focus-visible{--tw-ring-color:rgba(239,246,255,.15)}.focus-visible\:ring-blue-50\/20:focus-visible{--tw-ring-color:rgba(239,246,255,.2)}.focus-visible\:ring-blue-50\/25:focus-visible{--tw-ring-color:rgba(239,246,255,.25)}.focus-visible\:ring-blue-50\/30:focus-visible{--tw-ring-color:rgba(239,246,255,.3)}.focus-visible\:ring-blue-50\/35:focus-visible{--tw-ring-color:rgba(239,246,255,.35)}.focus-visible\:ring-blue-50\/40:focus-visible{--tw-ring-color:rgba(239,246,255,.4)}.focus-visible\:ring-blue-50\/45:focus-visible{--tw-ring-color:rgba(239,246,255,.45)}.focus-visible\:ring-blue-50\/5:focus-visible{--tw-ring-color:rgba(239,246,255,.05)}.focus-visible\:ring-blue-50\/50:focus-visible{--tw-ring-color:rgba(239,246,255,.5)}.focus-visible\:ring-blue-50\/55:focus-visible{--tw-ring-color:rgba(239,246,255,.55)}.focus-visible\:ring-blue-50\/60:focus-visible{--tw-ring-color:rgba(239,246,255,.6)}.focus-visible\:ring-blue-50\/65:focus-visible{--tw-ring-color:rgba(239,246,255,.65)}.focus-visible\:ring-blue-50\/70:focus-visible{--tw-ring-color:rgba(239,246,255,.7)}.focus-visible\:ring-blue-50\/75:focus-visible{--tw-ring-color:rgba(239,246,255,.75)}.focus-visible\:ring-blue-50\/80:focus-visible{--tw-ring-color:rgba(239,246,255,.8)}.focus-visible\:ring-blue-50\/85:focus-visible{--tw-ring-color:rgba(239,246,255,.85)}.focus-visible\:ring-blue-50\/90:focus-visible{--tw-ring-color:rgba(239,246,255,.9)}.focus-visible\:ring-blue-50\/95:focus-visible{--tw-ring-color:rgba(239,246,255,.95)}.focus-visible\:ring-blue-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(59 130 246/var(--tw-ring-opacity,1))}.focus-visible\:ring-blue-500\/0:focus-visible{--tw-ring-color:rgba(59,130,246,0)}.focus-visible\:ring-blue-500\/10:focus-visible{--tw-ring-color:rgba(59,130,246,.1)}.focus-visible\:ring-blue-500\/100:focus-visible{--tw-ring-color:#3b82f6}.focus-visible\:ring-blue-500\/15:focus-visible{--tw-ring-color:rgba(59,130,246,.15)}.focus-visible\:ring-blue-500\/20:focus-visible{--tw-ring-color:rgba(59,130,246,.2)}.focus-visible\:ring-blue-500\/25:focus-visible{--tw-ring-color:rgba(59,130,246,.25)}.focus-visible\:ring-blue-500\/30:focus-visible{--tw-ring-color:rgba(59,130,246,.3)}.focus-visible\:ring-blue-500\/35:focus-visible{--tw-ring-color:rgba(59,130,246,.35)}.focus-visible\:ring-blue-500\/40:focus-visible{--tw-ring-color:rgba(59,130,246,.4)}.focus-visible\:ring-blue-500\/45:focus-visible{--tw-ring-color:rgba(59,130,246,.45)}.focus-visible\:ring-blue-500\/5:focus-visible{--tw-ring-color:rgba(59,130,246,.05)}.focus-visible\:ring-blue-500\/50:focus-visible{--tw-ring-color:rgba(59,130,246,.5)}.focus-visible\:ring-blue-500\/55:focus-visible{--tw-ring-color:rgba(59,130,246,.55)}.focus-visible\:ring-blue-500\/60:focus-visible{--tw-ring-color:rgba(59,130,246,.6)}.focus-visible\:ring-blue-500\/65:focus-visible{--tw-ring-color:rgba(59,130,246,.65)}.focus-visible\:ring-blue-500\/70:focus-visible{--tw-ring-color:rgba(59,130,246,.7)}.focus-visible\:ring-blue-500\/75:focus-visible{--tw-ring-color:rgba(59,130,246,.75)}.focus-visible\:ring-blue-500\/80:focus-visible{--tw-ring-color:rgba(59,130,246,.8)}.focus-visible\:ring-blue-500\/85:focus-visible{--tw-ring-color:rgba(59,130,246,.85)}.focus-visible\:ring-blue-500\/90:focus-visible{--tw-ring-color:rgba(59,130,246,.9)}.focus-visible\:ring-blue-500\/95:focus-visible{--tw-ring-color:rgba(59,130,246,.95)}.focus-visible\:ring-blue-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(23 37 84/var(--tw-ring-opacity,1))}.focus-visible\:ring-blue-950\/0:focus-visible{--tw-ring-color:rgba(23,37,84,0)}.focus-visible\:ring-blue-950\/10:focus-visible{--tw-ring-color:rgba(23,37,84,.1)}.focus-visible\:ring-blue-950\/100:focus-visible{--tw-ring-color:#172554}.focus-visible\:ring-blue-950\/15:focus-visible{--tw-ring-color:rgba(23,37,84,.15)}.focus-visible\:ring-blue-950\/20:focus-visible{--tw-ring-color:rgba(23,37,84,.2)}.focus-visible\:ring-blue-950\/25:focus-visible{--tw-ring-color:rgba(23,37,84,.25)}.focus-visible\:ring-blue-950\/30:focus-visible{--tw-ring-color:rgba(23,37,84,.3)}.focus-visible\:ring-blue-950\/35:focus-visible{--tw-ring-color:rgba(23,37,84,.35)}.focus-visible\:ring-blue-950\/40:focus-visible{--tw-ring-color:rgba(23,37,84,.4)}.focus-visible\:ring-blue-950\/45:focus-visible{--tw-ring-color:rgba(23,37,84,.45)}.focus-visible\:ring-blue-950\/5:focus-visible{--tw-ring-color:rgba(23,37,84,.05)}.focus-visible\:ring-blue-950\/50:focus-visible{--tw-ring-color:rgba(23,37,84,.5)}.focus-visible\:ring-blue-950\/55:focus-visible{--tw-ring-color:rgba(23,37,84,.55)}.focus-visible\:ring-blue-950\/60:focus-visible{--tw-ring-color:rgba(23,37,84,.6)}.focus-visible\:ring-blue-950\/65:focus-visible{--tw-ring-color:rgba(23,37,84,.65)}.focus-visible\:ring-blue-950\/70:focus-visible{--tw-ring-color:rgba(23,37,84,.7)}.focus-visible\:ring-blue-950\/75:focus-visible{--tw-ring-color:rgba(23,37,84,.75)}.focus-visible\:ring-blue-950\/80:focus-visible{--tw-ring-color:rgba(23,37,84,.8)}.focus-visible\:ring-blue-950\/85:focus-visible{--tw-ring-color:rgba(23,37,84,.85)}.focus-visible\:ring-blue-950\/90:focus-visible{--tw-ring-color:rgba(23,37,84,.9)}.focus-visible\:ring-blue-950\/95:focus-visible{--tw-ring-color:rgba(23,37,84,.95)}.focus-visible\:ring-cyan-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(207 250 254/var(--tw-ring-opacity,1))}.focus-visible\:ring-cyan-100\/0:focus-visible{--tw-ring-color:rgba(207,250,254,0)}.focus-visible\:ring-cyan-100\/10:focus-visible{--tw-ring-color:rgba(207,250,254,.1)}.focus-visible\:ring-cyan-100\/100:focus-visible{--tw-ring-color:#cffafe}.focus-visible\:ring-cyan-100\/15:focus-visible{--tw-ring-color:rgba(207,250,254,.15)}.focus-visible\:ring-cyan-100\/20:focus-visible{--tw-ring-color:rgba(207,250,254,.2)}.focus-visible\:ring-cyan-100\/25:focus-visible{--tw-ring-color:rgba(207,250,254,.25)}.focus-visible\:ring-cyan-100\/30:focus-visible{--tw-ring-color:rgba(207,250,254,.3)}.focus-visible\:ring-cyan-100\/35:focus-visible{--tw-ring-color:rgba(207,250,254,.35)}.focus-visible\:ring-cyan-100\/40:focus-visible{--tw-ring-color:rgba(207,250,254,.4)}.focus-visible\:ring-cyan-100\/45:focus-visible{--tw-ring-color:rgba(207,250,254,.45)}.focus-visible\:ring-cyan-100\/5:focus-visible{--tw-ring-color:rgba(207,250,254,.05)}.focus-visible\:ring-cyan-100\/50:focus-visible{--tw-ring-color:rgba(207,250,254,.5)}.focus-visible\:ring-cyan-100\/55:focus-visible{--tw-ring-color:rgba(207,250,254,.55)}.focus-visible\:ring-cyan-100\/60:focus-visible{--tw-ring-color:rgba(207,250,254,.6)}.focus-visible\:ring-cyan-100\/65:focus-visible{--tw-ring-color:rgba(207,250,254,.65)}.focus-visible\:ring-cyan-100\/70:focus-visible{--tw-ring-color:rgba(207,250,254,.7)}.focus-visible\:ring-cyan-100\/75:focus-visible{--tw-ring-color:rgba(207,250,254,.75)}.focus-visible\:ring-cyan-100\/80:focus-visible{--tw-ring-color:rgba(207,250,254,.8)}.focus-visible\:ring-cyan-100\/85:focus-visible{--tw-ring-color:rgba(207,250,254,.85)}.focus-visible\:ring-cyan-100\/90:focus-visible{--tw-ring-color:rgba(207,250,254,.9)}.focus-visible\:ring-cyan-100\/95:focus-visible{--tw-ring-color:rgba(207,250,254,.95)}.focus-visible\:ring-cyan-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(165 243 252/var(--tw-ring-opacity,1))}.focus-visible\:ring-cyan-200\/0:focus-visible{--tw-ring-color:rgba(165,243,252,0)}.focus-visible\:ring-cyan-200\/10:focus-visible{--tw-ring-color:rgba(165,243,252,.1)}.focus-visible\:ring-cyan-200\/100:focus-visible{--tw-ring-color:#a5f3fc}.focus-visible\:ring-cyan-200\/15:focus-visible{--tw-ring-color:rgba(165,243,252,.15)}.focus-visible\:ring-cyan-200\/20:focus-visible{--tw-ring-color:rgba(165,243,252,.2)}.focus-visible\:ring-cyan-200\/25:focus-visible{--tw-ring-color:rgba(165,243,252,.25)}.focus-visible\:ring-cyan-200\/30:focus-visible{--tw-ring-color:rgba(165,243,252,.3)}.focus-visible\:ring-cyan-200\/35:focus-visible{--tw-ring-color:rgba(165,243,252,.35)}.focus-visible\:ring-cyan-200\/40:focus-visible{--tw-ring-color:rgba(165,243,252,.4)}.focus-visible\:ring-cyan-200\/45:focus-visible{--tw-ring-color:rgba(165,243,252,.45)}.focus-visible\:ring-cyan-200\/5:focus-visible{--tw-ring-color:rgba(165,243,252,.05)}.focus-visible\:ring-cyan-200\/50:focus-visible{--tw-ring-color:rgba(165,243,252,.5)}.focus-visible\:ring-cyan-200\/55:focus-visible{--tw-ring-color:rgba(165,243,252,.55)}.focus-visible\:ring-cyan-200\/60:focus-visible{--tw-ring-color:rgba(165,243,252,.6)}.focus-visible\:ring-cyan-200\/65:focus-visible{--tw-ring-color:rgba(165,243,252,.65)}.focus-visible\:ring-cyan-200\/70:focus-visible{--tw-ring-color:rgba(165,243,252,.7)}.focus-visible\:ring-cyan-200\/75:focus-visible{--tw-ring-color:rgba(165,243,252,.75)}.focus-visible\:ring-cyan-200\/80:focus-visible{--tw-ring-color:rgba(165,243,252,.8)}.focus-visible\:ring-cyan-200\/85:focus-visible{--tw-ring-color:rgba(165,243,252,.85)}.focus-visible\:ring-cyan-200\/90:focus-visible{--tw-ring-color:rgba(165,243,252,.9)}.focus-visible\:ring-cyan-200\/95:focus-visible{--tw-ring-color:rgba(165,243,252,.95)}.focus-visible\:ring-cyan-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(34 211 238/var(--tw-ring-opacity,1))}.focus-visible\:ring-cyan-400\/0:focus-visible{--tw-ring-color:rgba(34,211,238,0)}.focus-visible\:ring-cyan-400\/10:focus-visible{--tw-ring-color:rgba(34,211,238,.1)}.focus-visible\:ring-cyan-400\/100:focus-visible{--tw-ring-color:#22d3ee}.focus-visible\:ring-cyan-400\/15:focus-visible{--tw-ring-color:rgba(34,211,238,.15)}.focus-visible\:ring-cyan-400\/20:focus-visible{--tw-ring-color:rgba(34,211,238,.2)}.focus-visible\:ring-cyan-400\/25:focus-visible{--tw-ring-color:rgba(34,211,238,.25)}.focus-visible\:ring-cyan-400\/30:focus-visible{--tw-ring-color:rgba(34,211,238,.3)}.focus-visible\:ring-cyan-400\/35:focus-visible{--tw-ring-color:rgba(34,211,238,.35)}.focus-visible\:ring-cyan-400\/40:focus-visible{--tw-ring-color:rgba(34,211,238,.4)}.focus-visible\:ring-cyan-400\/45:focus-visible{--tw-ring-color:rgba(34,211,238,.45)}.focus-visible\:ring-cyan-400\/5:focus-visible{--tw-ring-color:rgba(34,211,238,.05)}.focus-visible\:ring-cyan-400\/50:focus-visible{--tw-ring-color:rgba(34,211,238,.5)}.focus-visible\:ring-cyan-400\/55:focus-visible{--tw-ring-color:rgba(34,211,238,.55)}.focus-visible\:ring-cyan-400\/60:focus-visible{--tw-ring-color:rgba(34,211,238,.6)}.focus-visible\:ring-cyan-400\/65:focus-visible{--tw-ring-color:rgba(34,211,238,.65)}.focus-visible\:ring-cyan-400\/70:focus-visible{--tw-ring-color:rgba(34,211,238,.7)}.focus-visible\:ring-cyan-400\/75:focus-visible{--tw-ring-color:rgba(34,211,238,.75)}.focus-visible\:ring-cyan-400\/80:focus-visible{--tw-ring-color:rgba(34,211,238,.8)}.focus-visible\:ring-cyan-400\/85:focus-visible{--tw-ring-color:rgba(34,211,238,.85)}.focus-visible\:ring-cyan-400\/90:focus-visible{--tw-ring-color:rgba(34,211,238,.9)}.focus-visible\:ring-cyan-400\/95:focus-visible{--tw-ring-color:rgba(34,211,238,.95)}.focus-visible\:ring-cyan-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(236 254 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-cyan-50\/0:focus-visible{--tw-ring-color:rgba(236,254,255,0)}.focus-visible\:ring-cyan-50\/10:focus-visible{--tw-ring-color:rgba(236,254,255,.1)}.focus-visible\:ring-cyan-50\/100:focus-visible{--tw-ring-color:#ecfeff}.focus-visible\:ring-cyan-50\/15:focus-visible{--tw-ring-color:rgba(236,254,255,.15)}.focus-visible\:ring-cyan-50\/20:focus-visible{--tw-ring-color:rgba(236,254,255,.2)}.focus-visible\:ring-cyan-50\/25:focus-visible{--tw-ring-color:rgba(236,254,255,.25)}.focus-visible\:ring-cyan-50\/30:focus-visible{--tw-ring-color:rgba(236,254,255,.3)}.focus-visible\:ring-cyan-50\/35:focus-visible{--tw-ring-color:rgba(236,254,255,.35)}.focus-visible\:ring-cyan-50\/40:focus-visible{--tw-ring-color:rgba(236,254,255,.4)}.focus-visible\:ring-cyan-50\/45:focus-visible{--tw-ring-color:rgba(236,254,255,.45)}.focus-visible\:ring-cyan-50\/5:focus-visible{--tw-ring-color:rgba(236,254,255,.05)}.focus-visible\:ring-cyan-50\/50:focus-visible{--tw-ring-color:rgba(236,254,255,.5)}.focus-visible\:ring-cyan-50\/55:focus-visible{--tw-ring-color:rgba(236,254,255,.55)}.focus-visible\:ring-cyan-50\/60:focus-visible{--tw-ring-color:rgba(236,254,255,.6)}.focus-visible\:ring-cyan-50\/65:focus-visible{--tw-ring-color:rgba(236,254,255,.65)}.focus-visible\:ring-cyan-50\/70:focus-visible{--tw-ring-color:rgba(236,254,255,.7)}.focus-visible\:ring-cyan-50\/75:focus-visible{--tw-ring-color:rgba(236,254,255,.75)}.focus-visible\:ring-cyan-50\/80:focus-visible{--tw-ring-color:rgba(236,254,255,.8)}.focus-visible\:ring-cyan-50\/85:focus-visible{--tw-ring-color:rgba(236,254,255,.85)}.focus-visible\:ring-cyan-50\/90:focus-visible{--tw-ring-color:rgba(236,254,255,.9)}.focus-visible\:ring-cyan-50\/95:focus-visible{--tw-ring-color:rgba(236,254,255,.95)}.focus-visible\:ring-cyan-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(6 182 212/var(--tw-ring-opacity,1))}.focus-visible\:ring-cyan-500\/0:focus-visible{--tw-ring-color:rgba(6,182,212,0)}.focus-visible\:ring-cyan-500\/10:focus-visible{--tw-ring-color:rgba(6,182,212,.1)}.focus-visible\:ring-cyan-500\/100:focus-visible{--tw-ring-color:#06b6d4}.focus-visible\:ring-cyan-500\/15:focus-visible{--tw-ring-color:rgba(6,182,212,.15)}.focus-visible\:ring-cyan-500\/20:focus-visible{--tw-ring-color:rgba(6,182,212,.2)}.focus-visible\:ring-cyan-500\/25:focus-visible{--tw-ring-color:rgba(6,182,212,.25)}.focus-visible\:ring-cyan-500\/30:focus-visible{--tw-ring-color:rgba(6,182,212,.3)}.focus-visible\:ring-cyan-500\/35:focus-visible{--tw-ring-color:rgba(6,182,212,.35)}.focus-visible\:ring-cyan-500\/40:focus-visible{--tw-ring-color:rgba(6,182,212,.4)}.focus-visible\:ring-cyan-500\/45:focus-visible{--tw-ring-color:rgba(6,182,212,.45)}.focus-visible\:ring-cyan-500\/5:focus-visible{--tw-ring-color:rgba(6,182,212,.05)}.focus-visible\:ring-cyan-500\/50:focus-visible{--tw-ring-color:rgba(6,182,212,.5)}.focus-visible\:ring-cyan-500\/55:focus-visible{--tw-ring-color:rgba(6,182,212,.55)}.focus-visible\:ring-cyan-500\/60:focus-visible{--tw-ring-color:rgba(6,182,212,.6)}.focus-visible\:ring-cyan-500\/65:focus-visible{--tw-ring-color:rgba(6,182,212,.65)}.focus-visible\:ring-cyan-500\/70:focus-visible{--tw-ring-color:rgba(6,182,212,.7)}.focus-visible\:ring-cyan-500\/75:focus-visible{--tw-ring-color:rgba(6,182,212,.75)}.focus-visible\:ring-cyan-500\/80:focus-visible{--tw-ring-color:rgba(6,182,212,.8)}.focus-visible\:ring-cyan-500\/85:focus-visible{--tw-ring-color:rgba(6,182,212,.85)}.focus-visible\:ring-cyan-500\/90:focus-visible{--tw-ring-color:rgba(6,182,212,.9)}.focus-visible\:ring-cyan-500\/95:focus-visible{--tw-ring-color:rgba(6,182,212,.95)}.focus-visible\:ring-cyan-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(8 51 68/var(--tw-ring-opacity,1))}.focus-visible\:ring-cyan-950\/0:focus-visible{--tw-ring-color:rgba(8,51,68,0)}.focus-visible\:ring-cyan-950\/10:focus-visible{--tw-ring-color:rgba(8,51,68,.1)}.focus-visible\:ring-cyan-950\/100:focus-visible{--tw-ring-color:#083344}.focus-visible\:ring-cyan-950\/15:focus-visible{--tw-ring-color:rgba(8,51,68,.15)}.focus-visible\:ring-cyan-950\/20:focus-visible{--tw-ring-color:rgba(8,51,68,.2)}.focus-visible\:ring-cyan-950\/25:focus-visible{--tw-ring-color:rgba(8,51,68,.25)}.focus-visible\:ring-cyan-950\/30:focus-visible{--tw-ring-color:rgba(8,51,68,.3)}.focus-visible\:ring-cyan-950\/35:focus-visible{--tw-ring-color:rgba(8,51,68,.35)}.focus-visible\:ring-cyan-950\/40:focus-visible{--tw-ring-color:rgba(8,51,68,.4)}.focus-visible\:ring-cyan-950\/45:focus-visible{--tw-ring-color:rgba(8,51,68,.45)}.focus-visible\:ring-cyan-950\/5:focus-visible{--tw-ring-color:rgba(8,51,68,.05)}.focus-visible\:ring-cyan-950\/50:focus-visible{--tw-ring-color:rgba(8,51,68,.5)}.focus-visible\:ring-cyan-950\/55:focus-visible{--tw-ring-color:rgba(8,51,68,.55)}.focus-visible\:ring-cyan-950\/60:focus-visible{--tw-ring-color:rgba(8,51,68,.6)}.focus-visible\:ring-cyan-950\/65:focus-visible{--tw-ring-color:rgba(8,51,68,.65)}.focus-visible\:ring-cyan-950\/70:focus-visible{--tw-ring-color:rgba(8,51,68,.7)}.focus-visible\:ring-cyan-950\/75:focus-visible{--tw-ring-color:rgba(8,51,68,.75)}.focus-visible\:ring-cyan-950\/80:focus-visible{--tw-ring-color:rgba(8,51,68,.8)}.focus-visible\:ring-cyan-950\/85:focus-visible{--tw-ring-color:rgba(8,51,68,.85)}.focus-visible\:ring-cyan-950\/90:focus-visible{--tw-ring-color:rgba(8,51,68,.9)}.focus-visible\:ring-cyan-950\/95:focus-visible{--tw-ring-color:rgba(8,51,68,.95)}.focus-visible\:ring-emerald-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(209 250 229/var(--tw-ring-opacity,1))}.focus-visible\:ring-emerald-100\/0:focus-visible{--tw-ring-color:rgba(209,250,229,0)}.focus-visible\:ring-emerald-100\/10:focus-visible{--tw-ring-color:rgba(209,250,229,.1)}.focus-visible\:ring-emerald-100\/100:focus-visible{--tw-ring-color:#d1fae5}.focus-visible\:ring-emerald-100\/15:focus-visible{--tw-ring-color:rgba(209,250,229,.15)}.focus-visible\:ring-emerald-100\/20:focus-visible{--tw-ring-color:rgba(209,250,229,.2)}.focus-visible\:ring-emerald-100\/25:focus-visible{--tw-ring-color:rgba(209,250,229,.25)}.focus-visible\:ring-emerald-100\/30:focus-visible{--tw-ring-color:rgba(209,250,229,.3)}.focus-visible\:ring-emerald-100\/35:focus-visible{--tw-ring-color:rgba(209,250,229,.35)}.focus-visible\:ring-emerald-100\/40:focus-visible{--tw-ring-color:rgba(209,250,229,.4)}.focus-visible\:ring-emerald-100\/45:focus-visible{--tw-ring-color:rgba(209,250,229,.45)}.focus-visible\:ring-emerald-100\/5:focus-visible{--tw-ring-color:rgba(209,250,229,.05)}.focus-visible\:ring-emerald-100\/50:focus-visible{--tw-ring-color:rgba(209,250,229,.5)}.focus-visible\:ring-emerald-100\/55:focus-visible{--tw-ring-color:rgba(209,250,229,.55)}.focus-visible\:ring-emerald-100\/60:focus-visible{--tw-ring-color:rgba(209,250,229,.6)}.focus-visible\:ring-emerald-100\/65:focus-visible{--tw-ring-color:rgba(209,250,229,.65)}.focus-visible\:ring-emerald-100\/70:focus-visible{--tw-ring-color:rgba(209,250,229,.7)}.focus-visible\:ring-emerald-100\/75:focus-visible{--tw-ring-color:rgba(209,250,229,.75)}.focus-visible\:ring-emerald-100\/80:focus-visible{--tw-ring-color:rgba(209,250,229,.8)}.focus-visible\:ring-emerald-100\/85:focus-visible{--tw-ring-color:rgba(209,250,229,.85)}.focus-visible\:ring-emerald-100\/90:focus-visible{--tw-ring-color:rgba(209,250,229,.9)}.focus-visible\:ring-emerald-100\/95:focus-visible{--tw-ring-color:rgba(209,250,229,.95)}.focus-visible\:ring-emerald-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(167 243 208/var(--tw-ring-opacity,1))}.focus-visible\:ring-emerald-200\/0:focus-visible{--tw-ring-color:rgba(167,243,208,0)}.focus-visible\:ring-emerald-200\/10:focus-visible{--tw-ring-color:rgba(167,243,208,.1)}.focus-visible\:ring-emerald-200\/100:focus-visible{--tw-ring-color:#a7f3d0}.focus-visible\:ring-emerald-200\/15:focus-visible{--tw-ring-color:rgba(167,243,208,.15)}.focus-visible\:ring-emerald-200\/20:focus-visible{--tw-ring-color:rgba(167,243,208,.2)}.focus-visible\:ring-emerald-200\/25:focus-visible{--tw-ring-color:rgba(167,243,208,.25)}.focus-visible\:ring-emerald-200\/30:focus-visible{--tw-ring-color:rgba(167,243,208,.3)}.focus-visible\:ring-emerald-200\/35:focus-visible{--tw-ring-color:rgba(167,243,208,.35)}.focus-visible\:ring-emerald-200\/40:focus-visible{--tw-ring-color:rgba(167,243,208,.4)}.focus-visible\:ring-emerald-200\/45:focus-visible{--tw-ring-color:rgba(167,243,208,.45)}.focus-visible\:ring-emerald-200\/5:focus-visible{--tw-ring-color:rgba(167,243,208,.05)}.focus-visible\:ring-emerald-200\/50:focus-visible{--tw-ring-color:rgba(167,243,208,.5)}.focus-visible\:ring-emerald-200\/55:focus-visible{--tw-ring-color:rgba(167,243,208,.55)}.focus-visible\:ring-emerald-200\/60:focus-visible{--tw-ring-color:rgba(167,243,208,.6)}.focus-visible\:ring-emerald-200\/65:focus-visible{--tw-ring-color:rgba(167,243,208,.65)}.focus-visible\:ring-emerald-200\/70:focus-visible{--tw-ring-color:rgba(167,243,208,.7)}.focus-visible\:ring-emerald-200\/75:focus-visible{--tw-ring-color:rgba(167,243,208,.75)}.focus-visible\:ring-emerald-200\/80:focus-visible{--tw-ring-color:rgba(167,243,208,.8)}.focus-visible\:ring-emerald-200\/85:focus-visible{--tw-ring-color:rgba(167,243,208,.85)}.focus-visible\:ring-emerald-200\/90:focus-visible{--tw-ring-color:rgba(167,243,208,.9)}.focus-visible\:ring-emerald-200\/95:focus-visible{--tw-ring-color:rgba(167,243,208,.95)}.focus-visible\:ring-emerald-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(52 211 153/var(--tw-ring-opacity,1))}.focus-visible\:ring-emerald-400\/0:focus-visible{--tw-ring-color:rgba(52,211,153,0)}.focus-visible\:ring-emerald-400\/10:focus-visible{--tw-ring-color:rgba(52,211,153,.1)}.focus-visible\:ring-emerald-400\/100:focus-visible{--tw-ring-color:#34d399}.focus-visible\:ring-emerald-400\/15:focus-visible{--tw-ring-color:rgba(52,211,153,.15)}.focus-visible\:ring-emerald-400\/20:focus-visible{--tw-ring-color:rgba(52,211,153,.2)}.focus-visible\:ring-emerald-400\/25:focus-visible{--tw-ring-color:rgba(52,211,153,.25)}.focus-visible\:ring-emerald-400\/30:focus-visible{--tw-ring-color:rgba(52,211,153,.3)}.focus-visible\:ring-emerald-400\/35:focus-visible{--tw-ring-color:rgba(52,211,153,.35)}.focus-visible\:ring-emerald-400\/40:focus-visible{--tw-ring-color:rgba(52,211,153,.4)}.focus-visible\:ring-emerald-400\/45:focus-visible{--tw-ring-color:rgba(52,211,153,.45)}.focus-visible\:ring-emerald-400\/5:focus-visible{--tw-ring-color:rgba(52,211,153,.05)}.focus-visible\:ring-emerald-400\/50:focus-visible{--tw-ring-color:rgba(52,211,153,.5)}.focus-visible\:ring-emerald-400\/55:focus-visible{--tw-ring-color:rgba(52,211,153,.55)}.focus-visible\:ring-emerald-400\/60:focus-visible{--tw-ring-color:rgba(52,211,153,.6)}.focus-visible\:ring-emerald-400\/65:focus-visible{--tw-ring-color:rgba(52,211,153,.65)}.focus-visible\:ring-emerald-400\/70:focus-visible{--tw-ring-color:rgba(52,211,153,.7)}.focus-visible\:ring-emerald-400\/75:focus-visible{--tw-ring-color:rgba(52,211,153,.75)}.focus-visible\:ring-emerald-400\/80:focus-visible{--tw-ring-color:rgba(52,211,153,.8)}.focus-visible\:ring-emerald-400\/85:focus-visible{--tw-ring-color:rgba(52,211,153,.85)}.focus-visible\:ring-emerald-400\/90:focus-visible{--tw-ring-color:rgba(52,211,153,.9)}.focus-visible\:ring-emerald-400\/95:focus-visible{--tw-ring-color:rgba(52,211,153,.95)}.focus-visible\:ring-emerald-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(236 253 245/var(--tw-ring-opacity,1))}.focus-visible\:ring-emerald-50\/0:focus-visible{--tw-ring-color:rgba(236,253,245,0)}.focus-visible\:ring-emerald-50\/10:focus-visible{--tw-ring-color:rgba(236,253,245,.1)}.focus-visible\:ring-emerald-50\/100:focus-visible{--tw-ring-color:#ecfdf5}.focus-visible\:ring-emerald-50\/15:focus-visible{--tw-ring-color:rgba(236,253,245,.15)}.focus-visible\:ring-emerald-50\/20:focus-visible{--tw-ring-color:rgba(236,253,245,.2)}.focus-visible\:ring-emerald-50\/25:focus-visible{--tw-ring-color:rgba(236,253,245,.25)}.focus-visible\:ring-emerald-50\/30:focus-visible{--tw-ring-color:rgba(236,253,245,.3)}.focus-visible\:ring-emerald-50\/35:focus-visible{--tw-ring-color:rgba(236,253,245,.35)}.focus-visible\:ring-emerald-50\/40:focus-visible{--tw-ring-color:rgba(236,253,245,.4)}.focus-visible\:ring-emerald-50\/45:focus-visible{--tw-ring-color:rgba(236,253,245,.45)}.focus-visible\:ring-emerald-50\/5:focus-visible{--tw-ring-color:rgba(236,253,245,.05)}.focus-visible\:ring-emerald-50\/50:focus-visible{--tw-ring-color:rgba(236,253,245,.5)}.focus-visible\:ring-emerald-50\/55:focus-visible{--tw-ring-color:rgba(236,253,245,.55)}.focus-visible\:ring-emerald-50\/60:focus-visible{--tw-ring-color:rgba(236,253,245,.6)}.focus-visible\:ring-emerald-50\/65:focus-visible{--tw-ring-color:rgba(236,253,245,.65)}.focus-visible\:ring-emerald-50\/70:focus-visible{--tw-ring-color:rgba(236,253,245,.7)}.focus-visible\:ring-emerald-50\/75:focus-visible{--tw-ring-color:rgba(236,253,245,.75)}.focus-visible\:ring-emerald-50\/80:focus-visible{--tw-ring-color:rgba(236,253,245,.8)}.focus-visible\:ring-emerald-50\/85:focus-visible{--tw-ring-color:rgba(236,253,245,.85)}.focus-visible\:ring-emerald-50\/90:focus-visible{--tw-ring-color:rgba(236,253,245,.9)}.focus-visible\:ring-emerald-50\/95:focus-visible{--tw-ring-color:rgba(236,253,245,.95)}.focus-visible\:ring-emerald-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(16 185 129/var(--tw-ring-opacity,1))}.focus-visible\:ring-emerald-500\/0:focus-visible{--tw-ring-color:rgba(16,185,129,0)}.focus-visible\:ring-emerald-500\/10:focus-visible{--tw-ring-color:rgba(16,185,129,.1)}.focus-visible\:ring-emerald-500\/100:focus-visible{--tw-ring-color:#10b981}.focus-visible\:ring-emerald-500\/15:focus-visible{--tw-ring-color:rgba(16,185,129,.15)}.focus-visible\:ring-emerald-500\/20:focus-visible{--tw-ring-color:rgba(16,185,129,.2)}.focus-visible\:ring-emerald-500\/25:focus-visible{--tw-ring-color:rgba(16,185,129,.25)}.focus-visible\:ring-emerald-500\/30:focus-visible{--tw-ring-color:rgba(16,185,129,.3)}.focus-visible\:ring-emerald-500\/35:focus-visible{--tw-ring-color:rgba(16,185,129,.35)}.focus-visible\:ring-emerald-500\/40:focus-visible{--tw-ring-color:rgba(16,185,129,.4)}.focus-visible\:ring-emerald-500\/45:focus-visible{--tw-ring-color:rgba(16,185,129,.45)}.focus-visible\:ring-emerald-500\/5:focus-visible{--tw-ring-color:rgba(16,185,129,.05)}.focus-visible\:ring-emerald-500\/50:focus-visible{--tw-ring-color:rgba(16,185,129,.5)}.focus-visible\:ring-emerald-500\/55:focus-visible{--tw-ring-color:rgba(16,185,129,.55)}.focus-visible\:ring-emerald-500\/60:focus-visible{--tw-ring-color:rgba(16,185,129,.6)}.focus-visible\:ring-emerald-500\/65:focus-visible{--tw-ring-color:rgba(16,185,129,.65)}.focus-visible\:ring-emerald-500\/70:focus-visible{--tw-ring-color:rgba(16,185,129,.7)}.focus-visible\:ring-emerald-500\/75:focus-visible{--tw-ring-color:rgba(16,185,129,.75)}.focus-visible\:ring-emerald-500\/80:focus-visible{--tw-ring-color:rgba(16,185,129,.8)}.focus-visible\:ring-emerald-500\/85:focus-visible{--tw-ring-color:rgba(16,185,129,.85)}.focus-visible\:ring-emerald-500\/90:focus-visible{--tw-ring-color:rgba(16,185,129,.9)}.focus-visible\:ring-emerald-500\/95:focus-visible{--tw-ring-color:rgba(16,185,129,.95)}.focus-visible\:ring-emerald-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(2 44 34/var(--tw-ring-opacity,1))}.focus-visible\:ring-emerald-950\/0:focus-visible{--tw-ring-color:rgba(2,44,34,0)}.focus-visible\:ring-emerald-950\/10:focus-visible{--tw-ring-color:rgba(2,44,34,.1)}.focus-visible\:ring-emerald-950\/100:focus-visible{--tw-ring-color:#022c22}.focus-visible\:ring-emerald-950\/15:focus-visible{--tw-ring-color:rgba(2,44,34,.15)}.focus-visible\:ring-emerald-950\/20:focus-visible{--tw-ring-color:rgba(2,44,34,.2)}.focus-visible\:ring-emerald-950\/25:focus-visible{--tw-ring-color:rgba(2,44,34,.25)}.focus-visible\:ring-emerald-950\/30:focus-visible{--tw-ring-color:rgba(2,44,34,.3)}.focus-visible\:ring-emerald-950\/35:focus-visible{--tw-ring-color:rgba(2,44,34,.35)}.focus-visible\:ring-emerald-950\/40:focus-visible{--tw-ring-color:rgba(2,44,34,.4)}.focus-visible\:ring-emerald-950\/45:focus-visible{--tw-ring-color:rgba(2,44,34,.45)}.focus-visible\:ring-emerald-950\/5:focus-visible{--tw-ring-color:rgba(2,44,34,.05)}.focus-visible\:ring-emerald-950\/50:focus-visible{--tw-ring-color:rgba(2,44,34,.5)}.focus-visible\:ring-emerald-950\/55:focus-visible{--tw-ring-color:rgba(2,44,34,.55)}.focus-visible\:ring-emerald-950\/60:focus-visible{--tw-ring-color:rgba(2,44,34,.6)}.focus-visible\:ring-emerald-950\/65:focus-visible{--tw-ring-color:rgba(2,44,34,.65)}.focus-visible\:ring-emerald-950\/70:focus-visible{--tw-ring-color:rgba(2,44,34,.7)}.focus-visible\:ring-emerald-950\/75:focus-visible{--tw-ring-color:rgba(2,44,34,.75)}.focus-visible\:ring-emerald-950\/80:focus-visible{--tw-ring-color:rgba(2,44,34,.8)}.focus-visible\:ring-emerald-950\/85:focus-visible{--tw-ring-color:rgba(2,44,34,.85)}.focus-visible\:ring-emerald-950\/90:focus-visible{--tw-ring-color:rgba(2,44,34,.9)}.focus-visible\:ring-emerald-950\/95:focus-visible{--tw-ring-color:rgba(2,44,34,.95)}.focus-visible\:ring-fuchsia-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(250 232 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-fuchsia-100\/0:focus-visible{--tw-ring-color:rgba(250,232,255,0)}.focus-visible\:ring-fuchsia-100\/10:focus-visible{--tw-ring-color:rgba(250,232,255,.1)}.focus-visible\:ring-fuchsia-100\/100:focus-visible{--tw-ring-color:#fae8ff}.focus-visible\:ring-fuchsia-100\/15:focus-visible{--tw-ring-color:rgba(250,232,255,.15)}.focus-visible\:ring-fuchsia-100\/20:focus-visible{--tw-ring-color:rgba(250,232,255,.2)}.focus-visible\:ring-fuchsia-100\/25:focus-visible{--tw-ring-color:rgba(250,232,255,.25)}.focus-visible\:ring-fuchsia-100\/30:focus-visible{--tw-ring-color:rgba(250,232,255,.3)}.focus-visible\:ring-fuchsia-100\/35:focus-visible{--tw-ring-color:rgba(250,232,255,.35)}.focus-visible\:ring-fuchsia-100\/40:focus-visible{--tw-ring-color:rgba(250,232,255,.4)}.focus-visible\:ring-fuchsia-100\/45:focus-visible{--tw-ring-color:rgba(250,232,255,.45)}.focus-visible\:ring-fuchsia-100\/5:focus-visible{--tw-ring-color:rgba(250,232,255,.05)}.focus-visible\:ring-fuchsia-100\/50:focus-visible{--tw-ring-color:rgba(250,232,255,.5)}.focus-visible\:ring-fuchsia-100\/55:focus-visible{--tw-ring-color:rgba(250,232,255,.55)}.focus-visible\:ring-fuchsia-100\/60:focus-visible{--tw-ring-color:rgba(250,232,255,.6)}.focus-visible\:ring-fuchsia-100\/65:focus-visible{--tw-ring-color:rgba(250,232,255,.65)}.focus-visible\:ring-fuchsia-100\/70:focus-visible{--tw-ring-color:rgba(250,232,255,.7)}.focus-visible\:ring-fuchsia-100\/75:focus-visible{--tw-ring-color:rgba(250,232,255,.75)}.focus-visible\:ring-fuchsia-100\/80:focus-visible{--tw-ring-color:rgba(250,232,255,.8)}.focus-visible\:ring-fuchsia-100\/85:focus-visible{--tw-ring-color:rgba(250,232,255,.85)}.focus-visible\:ring-fuchsia-100\/90:focus-visible{--tw-ring-color:rgba(250,232,255,.9)}.focus-visible\:ring-fuchsia-100\/95:focus-visible{--tw-ring-color:rgba(250,232,255,.95)}.focus-visible\:ring-fuchsia-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(245 208 254/var(--tw-ring-opacity,1))}.focus-visible\:ring-fuchsia-200\/0:focus-visible{--tw-ring-color:rgba(245,208,254,0)}.focus-visible\:ring-fuchsia-200\/10:focus-visible{--tw-ring-color:rgba(245,208,254,.1)}.focus-visible\:ring-fuchsia-200\/100:focus-visible{--tw-ring-color:#f5d0fe}.focus-visible\:ring-fuchsia-200\/15:focus-visible{--tw-ring-color:rgba(245,208,254,.15)}.focus-visible\:ring-fuchsia-200\/20:focus-visible{--tw-ring-color:rgba(245,208,254,.2)}.focus-visible\:ring-fuchsia-200\/25:focus-visible{--tw-ring-color:rgba(245,208,254,.25)}.focus-visible\:ring-fuchsia-200\/30:focus-visible{--tw-ring-color:rgba(245,208,254,.3)}.focus-visible\:ring-fuchsia-200\/35:focus-visible{--tw-ring-color:rgba(245,208,254,.35)}.focus-visible\:ring-fuchsia-200\/40:focus-visible{--tw-ring-color:rgba(245,208,254,.4)}.focus-visible\:ring-fuchsia-200\/45:focus-visible{--tw-ring-color:rgba(245,208,254,.45)}.focus-visible\:ring-fuchsia-200\/5:focus-visible{--tw-ring-color:rgba(245,208,254,.05)}.focus-visible\:ring-fuchsia-200\/50:focus-visible{--tw-ring-color:rgba(245,208,254,.5)}.focus-visible\:ring-fuchsia-200\/55:focus-visible{--tw-ring-color:rgba(245,208,254,.55)}.focus-visible\:ring-fuchsia-200\/60:focus-visible{--tw-ring-color:rgba(245,208,254,.6)}.focus-visible\:ring-fuchsia-200\/65:focus-visible{--tw-ring-color:rgba(245,208,254,.65)}.focus-visible\:ring-fuchsia-200\/70:focus-visible{--tw-ring-color:rgba(245,208,254,.7)}.focus-visible\:ring-fuchsia-200\/75:focus-visible{--tw-ring-color:rgba(245,208,254,.75)}.focus-visible\:ring-fuchsia-200\/80:focus-visible{--tw-ring-color:rgba(245,208,254,.8)}.focus-visible\:ring-fuchsia-200\/85:focus-visible{--tw-ring-color:rgba(245,208,254,.85)}.focus-visible\:ring-fuchsia-200\/90:focus-visible{--tw-ring-color:rgba(245,208,254,.9)}.focus-visible\:ring-fuchsia-200\/95:focus-visible{--tw-ring-color:rgba(245,208,254,.95)}.focus-visible\:ring-fuchsia-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(232 121 249/var(--tw-ring-opacity,1))}.focus-visible\:ring-fuchsia-400\/0:focus-visible{--tw-ring-color:rgba(232,121,249,0)}.focus-visible\:ring-fuchsia-400\/10:focus-visible{--tw-ring-color:rgba(232,121,249,.1)}.focus-visible\:ring-fuchsia-400\/100:focus-visible{--tw-ring-color:#e879f9}.focus-visible\:ring-fuchsia-400\/15:focus-visible{--tw-ring-color:rgba(232,121,249,.15)}.focus-visible\:ring-fuchsia-400\/20:focus-visible{--tw-ring-color:rgba(232,121,249,.2)}.focus-visible\:ring-fuchsia-400\/25:focus-visible{--tw-ring-color:rgba(232,121,249,.25)}.focus-visible\:ring-fuchsia-400\/30:focus-visible{--tw-ring-color:rgba(232,121,249,.3)}.focus-visible\:ring-fuchsia-400\/35:focus-visible{--tw-ring-color:rgba(232,121,249,.35)}.focus-visible\:ring-fuchsia-400\/40:focus-visible{--tw-ring-color:rgba(232,121,249,.4)}.focus-visible\:ring-fuchsia-400\/45:focus-visible{--tw-ring-color:rgba(232,121,249,.45)}.focus-visible\:ring-fuchsia-400\/5:focus-visible{--tw-ring-color:rgba(232,121,249,.05)}.focus-visible\:ring-fuchsia-400\/50:focus-visible{--tw-ring-color:rgba(232,121,249,.5)}.focus-visible\:ring-fuchsia-400\/55:focus-visible{--tw-ring-color:rgba(232,121,249,.55)}.focus-visible\:ring-fuchsia-400\/60:focus-visible{--tw-ring-color:rgba(232,121,249,.6)}.focus-visible\:ring-fuchsia-400\/65:focus-visible{--tw-ring-color:rgba(232,121,249,.65)}.focus-visible\:ring-fuchsia-400\/70:focus-visible{--tw-ring-color:rgba(232,121,249,.7)}.focus-visible\:ring-fuchsia-400\/75:focus-visible{--tw-ring-color:rgba(232,121,249,.75)}.focus-visible\:ring-fuchsia-400\/80:focus-visible{--tw-ring-color:rgba(232,121,249,.8)}.focus-visible\:ring-fuchsia-400\/85:focus-visible{--tw-ring-color:rgba(232,121,249,.85)}.focus-visible\:ring-fuchsia-400\/90:focus-visible{--tw-ring-color:rgba(232,121,249,.9)}.focus-visible\:ring-fuchsia-400\/95:focus-visible{--tw-ring-color:rgba(232,121,249,.95)}.focus-visible\:ring-fuchsia-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(253 244 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-fuchsia-50\/0:focus-visible{--tw-ring-color:rgba(253,244,255,0)}.focus-visible\:ring-fuchsia-50\/10:focus-visible{--tw-ring-color:rgba(253,244,255,.1)}.focus-visible\:ring-fuchsia-50\/100:focus-visible{--tw-ring-color:#fdf4ff}.focus-visible\:ring-fuchsia-50\/15:focus-visible{--tw-ring-color:rgba(253,244,255,.15)}.focus-visible\:ring-fuchsia-50\/20:focus-visible{--tw-ring-color:rgba(253,244,255,.2)}.focus-visible\:ring-fuchsia-50\/25:focus-visible{--tw-ring-color:rgba(253,244,255,.25)}.focus-visible\:ring-fuchsia-50\/30:focus-visible{--tw-ring-color:rgba(253,244,255,.3)}.focus-visible\:ring-fuchsia-50\/35:focus-visible{--tw-ring-color:rgba(253,244,255,.35)}.focus-visible\:ring-fuchsia-50\/40:focus-visible{--tw-ring-color:rgba(253,244,255,.4)}.focus-visible\:ring-fuchsia-50\/45:focus-visible{--tw-ring-color:rgba(253,244,255,.45)}.focus-visible\:ring-fuchsia-50\/5:focus-visible{--tw-ring-color:rgba(253,244,255,.05)}.focus-visible\:ring-fuchsia-50\/50:focus-visible{--tw-ring-color:rgba(253,244,255,.5)}.focus-visible\:ring-fuchsia-50\/55:focus-visible{--tw-ring-color:rgba(253,244,255,.55)}.focus-visible\:ring-fuchsia-50\/60:focus-visible{--tw-ring-color:rgba(253,244,255,.6)}.focus-visible\:ring-fuchsia-50\/65:focus-visible{--tw-ring-color:rgba(253,244,255,.65)}.focus-visible\:ring-fuchsia-50\/70:focus-visible{--tw-ring-color:rgba(253,244,255,.7)}.focus-visible\:ring-fuchsia-50\/75:focus-visible{--tw-ring-color:rgba(253,244,255,.75)}.focus-visible\:ring-fuchsia-50\/80:focus-visible{--tw-ring-color:rgba(253,244,255,.8)}.focus-visible\:ring-fuchsia-50\/85:focus-visible{--tw-ring-color:rgba(253,244,255,.85)}.focus-visible\:ring-fuchsia-50\/90:focus-visible{--tw-ring-color:rgba(253,244,255,.9)}.focus-visible\:ring-fuchsia-50\/95:focus-visible{--tw-ring-color:rgba(253,244,255,.95)}.focus-visible\:ring-fuchsia-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(217 70 239/var(--tw-ring-opacity,1))}.focus-visible\:ring-fuchsia-500\/0:focus-visible{--tw-ring-color:rgba(217,70,239,0)}.focus-visible\:ring-fuchsia-500\/10:focus-visible{--tw-ring-color:rgba(217,70,239,.1)}.focus-visible\:ring-fuchsia-500\/100:focus-visible{--tw-ring-color:#d946ef}.focus-visible\:ring-fuchsia-500\/15:focus-visible{--tw-ring-color:rgba(217,70,239,.15)}.focus-visible\:ring-fuchsia-500\/20:focus-visible{--tw-ring-color:rgba(217,70,239,.2)}.focus-visible\:ring-fuchsia-500\/25:focus-visible{--tw-ring-color:rgba(217,70,239,.25)}.focus-visible\:ring-fuchsia-500\/30:focus-visible{--tw-ring-color:rgba(217,70,239,.3)}.focus-visible\:ring-fuchsia-500\/35:focus-visible{--tw-ring-color:rgba(217,70,239,.35)}.focus-visible\:ring-fuchsia-500\/40:focus-visible{--tw-ring-color:rgba(217,70,239,.4)}.focus-visible\:ring-fuchsia-500\/45:focus-visible{--tw-ring-color:rgba(217,70,239,.45)}.focus-visible\:ring-fuchsia-500\/5:focus-visible{--tw-ring-color:rgba(217,70,239,.05)}.focus-visible\:ring-fuchsia-500\/50:focus-visible{--tw-ring-color:rgba(217,70,239,.5)}.focus-visible\:ring-fuchsia-500\/55:focus-visible{--tw-ring-color:rgba(217,70,239,.55)}.focus-visible\:ring-fuchsia-500\/60:focus-visible{--tw-ring-color:rgba(217,70,239,.6)}.focus-visible\:ring-fuchsia-500\/65:focus-visible{--tw-ring-color:rgba(217,70,239,.65)}.focus-visible\:ring-fuchsia-500\/70:focus-visible{--tw-ring-color:rgba(217,70,239,.7)}.focus-visible\:ring-fuchsia-500\/75:focus-visible{--tw-ring-color:rgba(217,70,239,.75)}.focus-visible\:ring-fuchsia-500\/80:focus-visible{--tw-ring-color:rgba(217,70,239,.8)}.focus-visible\:ring-fuchsia-500\/85:focus-visible{--tw-ring-color:rgba(217,70,239,.85)}.focus-visible\:ring-fuchsia-500\/90:focus-visible{--tw-ring-color:rgba(217,70,239,.9)}.focus-visible\:ring-fuchsia-500\/95:focus-visible{--tw-ring-color:rgba(217,70,239,.95)}.focus-visible\:ring-fuchsia-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(74 4 78/var(--tw-ring-opacity,1))}.focus-visible\:ring-fuchsia-950\/0:focus-visible{--tw-ring-color:rgba(74,4,78,0)}.focus-visible\:ring-fuchsia-950\/10:focus-visible{--tw-ring-color:rgba(74,4,78,.1)}.focus-visible\:ring-fuchsia-950\/100:focus-visible{--tw-ring-color:#4a044e}.focus-visible\:ring-fuchsia-950\/15:focus-visible{--tw-ring-color:rgba(74,4,78,.15)}.focus-visible\:ring-fuchsia-950\/20:focus-visible{--tw-ring-color:rgba(74,4,78,.2)}.focus-visible\:ring-fuchsia-950\/25:focus-visible{--tw-ring-color:rgba(74,4,78,.25)}.focus-visible\:ring-fuchsia-950\/30:focus-visible{--tw-ring-color:rgba(74,4,78,.3)}.focus-visible\:ring-fuchsia-950\/35:focus-visible{--tw-ring-color:rgba(74,4,78,.35)}.focus-visible\:ring-fuchsia-950\/40:focus-visible{--tw-ring-color:rgba(74,4,78,.4)}.focus-visible\:ring-fuchsia-950\/45:focus-visible{--tw-ring-color:rgba(74,4,78,.45)}.focus-visible\:ring-fuchsia-950\/5:focus-visible{--tw-ring-color:rgba(74,4,78,.05)}.focus-visible\:ring-fuchsia-950\/50:focus-visible{--tw-ring-color:rgba(74,4,78,.5)}.focus-visible\:ring-fuchsia-950\/55:focus-visible{--tw-ring-color:rgba(74,4,78,.55)}.focus-visible\:ring-fuchsia-950\/60:focus-visible{--tw-ring-color:rgba(74,4,78,.6)}.focus-visible\:ring-fuchsia-950\/65:focus-visible{--tw-ring-color:rgba(74,4,78,.65)}.focus-visible\:ring-fuchsia-950\/70:focus-visible{--tw-ring-color:rgba(74,4,78,.7)}.focus-visible\:ring-fuchsia-950\/75:focus-visible{--tw-ring-color:rgba(74,4,78,.75)}.focus-visible\:ring-fuchsia-950\/80:focus-visible{--tw-ring-color:rgba(74,4,78,.8)}.focus-visible\:ring-fuchsia-950\/85:focus-visible{--tw-ring-color:rgba(74,4,78,.85)}.focus-visible\:ring-fuchsia-950\/90:focus-visible{--tw-ring-color:rgba(74,4,78,.9)}.focus-visible\:ring-fuchsia-950\/95:focus-visible{--tw-ring-color:rgba(74,4,78,.95)}.focus-visible\:ring-gray-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(243 244 246/var(--tw-ring-opacity,1))}.focus-visible\:ring-gray-100\/0:focus-visible{--tw-ring-color:rgba(243,244,246,0)}.focus-visible\:ring-gray-100\/10:focus-visible{--tw-ring-color:rgba(243,244,246,.1)}.focus-visible\:ring-gray-100\/100:focus-visible{--tw-ring-color:#f3f4f6}.focus-visible\:ring-gray-100\/15:focus-visible{--tw-ring-color:rgba(243,244,246,.15)}.focus-visible\:ring-gray-100\/20:focus-visible{--tw-ring-color:rgba(243,244,246,.2)}.focus-visible\:ring-gray-100\/25:focus-visible{--tw-ring-color:rgba(243,244,246,.25)}.focus-visible\:ring-gray-100\/30:focus-visible{--tw-ring-color:rgba(243,244,246,.3)}.focus-visible\:ring-gray-100\/35:focus-visible{--tw-ring-color:rgba(243,244,246,.35)}.focus-visible\:ring-gray-100\/40:focus-visible{--tw-ring-color:rgba(243,244,246,.4)}.focus-visible\:ring-gray-100\/45:focus-visible{--tw-ring-color:rgba(243,244,246,.45)}.focus-visible\:ring-gray-100\/5:focus-visible{--tw-ring-color:rgba(243,244,246,.05)}.focus-visible\:ring-gray-100\/50:focus-visible{--tw-ring-color:rgba(243,244,246,.5)}.focus-visible\:ring-gray-100\/55:focus-visible{--tw-ring-color:rgba(243,244,246,.55)}.focus-visible\:ring-gray-100\/60:focus-visible{--tw-ring-color:rgba(243,244,246,.6)}.focus-visible\:ring-gray-100\/65:focus-visible{--tw-ring-color:rgba(243,244,246,.65)}.focus-visible\:ring-gray-100\/70:focus-visible{--tw-ring-color:rgba(243,244,246,.7)}.focus-visible\:ring-gray-100\/75:focus-visible{--tw-ring-color:rgba(243,244,246,.75)}.focus-visible\:ring-gray-100\/80:focus-visible{--tw-ring-color:rgba(243,244,246,.8)}.focus-visible\:ring-gray-100\/85:focus-visible{--tw-ring-color:rgba(243,244,246,.85)}.focus-visible\:ring-gray-100\/90:focus-visible{--tw-ring-color:rgba(243,244,246,.9)}.focus-visible\:ring-gray-100\/95:focus-visible{--tw-ring-color:rgba(243,244,246,.95)}.focus-visible\:ring-gray-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(229 231 235/var(--tw-ring-opacity,1))}.focus-visible\:ring-gray-200\/0:focus-visible{--tw-ring-color:rgba(229,231,235,0)}.focus-visible\:ring-gray-200\/10:focus-visible{--tw-ring-color:rgba(229,231,235,.1)}.focus-visible\:ring-gray-200\/100:focus-visible{--tw-ring-color:#e5e7eb}.focus-visible\:ring-gray-200\/15:focus-visible{--tw-ring-color:rgba(229,231,235,.15)}.focus-visible\:ring-gray-200\/20:focus-visible{--tw-ring-color:rgba(229,231,235,.2)}.focus-visible\:ring-gray-200\/25:focus-visible{--tw-ring-color:rgba(229,231,235,.25)}.focus-visible\:ring-gray-200\/30:focus-visible{--tw-ring-color:rgba(229,231,235,.3)}.focus-visible\:ring-gray-200\/35:focus-visible{--tw-ring-color:rgba(229,231,235,.35)}.focus-visible\:ring-gray-200\/40:focus-visible{--tw-ring-color:rgba(229,231,235,.4)}.focus-visible\:ring-gray-200\/45:focus-visible{--tw-ring-color:rgba(229,231,235,.45)}.focus-visible\:ring-gray-200\/5:focus-visible{--tw-ring-color:rgba(229,231,235,.05)}.focus-visible\:ring-gray-200\/50:focus-visible{--tw-ring-color:rgba(229,231,235,.5)}.focus-visible\:ring-gray-200\/55:focus-visible{--tw-ring-color:rgba(229,231,235,.55)}.focus-visible\:ring-gray-200\/60:focus-visible{--tw-ring-color:rgba(229,231,235,.6)}.focus-visible\:ring-gray-200\/65:focus-visible{--tw-ring-color:rgba(229,231,235,.65)}.focus-visible\:ring-gray-200\/70:focus-visible{--tw-ring-color:rgba(229,231,235,.7)}.focus-visible\:ring-gray-200\/75:focus-visible{--tw-ring-color:rgba(229,231,235,.75)}.focus-visible\:ring-gray-200\/80:focus-visible{--tw-ring-color:rgba(229,231,235,.8)}.focus-visible\:ring-gray-200\/85:focus-visible{--tw-ring-color:rgba(229,231,235,.85)}.focus-visible\:ring-gray-200\/90:focus-visible{--tw-ring-color:rgba(229,231,235,.9)}.focus-visible\:ring-gray-200\/95:focus-visible{--tw-ring-color:rgba(229,231,235,.95)}.focus-visible\:ring-gray-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(156 163 175/var(--tw-ring-opacity,1))}.focus-visible\:ring-gray-400\/0:focus-visible{--tw-ring-color:rgba(156,163,175,0)}.focus-visible\:ring-gray-400\/10:focus-visible{--tw-ring-color:rgba(156,163,175,.1)}.focus-visible\:ring-gray-400\/100:focus-visible{--tw-ring-color:#9ca3af}.focus-visible\:ring-gray-400\/15:focus-visible{--tw-ring-color:rgba(156,163,175,.15)}.focus-visible\:ring-gray-400\/20:focus-visible{--tw-ring-color:rgba(156,163,175,.2)}.focus-visible\:ring-gray-400\/25:focus-visible{--tw-ring-color:rgba(156,163,175,.25)}.focus-visible\:ring-gray-400\/30:focus-visible{--tw-ring-color:rgba(156,163,175,.3)}.focus-visible\:ring-gray-400\/35:focus-visible{--tw-ring-color:rgba(156,163,175,.35)}.focus-visible\:ring-gray-400\/40:focus-visible{--tw-ring-color:rgba(156,163,175,.4)}.focus-visible\:ring-gray-400\/45:focus-visible{--tw-ring-color:rgba(156,163,175,.45)}.focus-visible\:ring-gray-400\/5:focus-visible{--tw-ring-color:rgba(156,163,175,.05)}.focus-visible\:ring-gray-400\/50:focus-visible{--tw-ring-color:rgba(156,163,175,.5)}.focus-visible\:ring-gray-400\/55:focus-visible{--tw-ring-color:rgba(156,163,175,.55)}.focus-visible\:ring-gray-400\/60:focus-visible{--tw-ring-color:rgba(156,163,175,.6)}.focus-visible\:ring-gray-400\/65:focus-visible{--tw-ring-color:rgba(156,163,175,.65)}.focus-visible\:ring-gray-400\/70:focus-visible{--tw-ring-color:rgba(156,163,175,.7)}.focus-visible\:ring-gray-400\/75:focus-visible{--tw-ring-color:rgba(156,163,175,.75)}.focus-visible\:ring-gray-400\/80:focus-visible{--tw-ring-color:rgba(156,163,175,.8)}.focus-visible\:ring-gray-400\/85:focus-visible{--tw-ring-color:rgba(156,163,175,.85)}.focus-visible\:ring-gray-400\/90:focus-visible{--tw-ring-color:rgba(156,163,175,.9)}.focus-visible\:ring-gray-400\/95:focus-visible{--tw-ring-color:rgba(156,163,175,.95)}.focus-visible\:ring-gray-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(249 250 251/var(--tw-ring-opacity,1))}.focus-visible\:ring-gray-50\/0:focus-visible{--tw-ring-color:rgba(249,250,251,0)}.focus-visible\:ring-gray-50\/10:focus-visible{--tw-ring-color:rgba(249,250,251,.1)}.focus-visible\:ring-gray-50\/100:focus-visible{--tw-ring-color:#f9fafb}.focus-visible\:ring-gray-50\/15:focus-visible{--tw-ring-color:rgba(249,250,251,.15)}.focus-visible\:ring-gray-50\/20:focus-visible{--tw-ring-color:rgba(249,250,251,.2)}.focus-visible\:ring-gray-50\/25:focus-visible{--tw-ring-color:rgba(249,250,251,.25)}.focus-visible\:ring-gray-50\/30:focus-visible{--tw-ring-color:rgba(249,250,251,.3)}.focus-visible\:ring-gray-50\/35:focus-visible{--tw-ring-color:rgba(249,250,251,.35)}.focus-visible\:ring-gray-50\/40:focus-visible{--tw-ring-color:rgba(249,250,251,.4)}.focus-visible\:ring-gray-50\/45:focus-visible{--tw-ring-color:rgba(249,250,251,.45)}.focus-visible\:ring-gray-50\/5:focus-visible{--tw-ring-color:rgba(249,250,251,.05)}.focus-visible\:ring-gray-50\/50:focus-visible{--tw-ring-color:rgba(249,250,251,.5)}.focus-visible\:ring-gray-50\/55:focus-visible{--tw-ring-color:rgba(249,250,251,.55)}.focus-visible\:ring-gray-50\/60:focus-visible{--tw-ring-color:rgba(249,250,251,.6)}.focus-visible\:ring-gray-50\/65:focus-visible{--tw-ring-color:rgba(249,250,251,.65)}.focus-visible\:ring-gray-50\/70:focus-visible{--tw-ring-color:rgba(249,250,251,.7)}.focus-visible\:ring-gray-50\/75:focus-visible{--tw-ring-color:rgba(249,250,251,.75)}.focus-visible\:ring-gray-50\/80:focus-visible{--tw-ring-color:rgba(249,250,251,.8)}.focus-visible\:ring-gray-50\/85:focus-visible{--tw-ring-color:rgba(249,250,251,.85)}.focus-visible\:ring-gray-50\/90:focus-visible{--tw-ring-color:rgba(249,250,251,.9)}.focus-visible\:ring-gray-50\/95:focus-visible{--tw-ring-color:rgba(249,250,251,.95)}.focus-visible\:ring-gray-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(107 114 128/var(--tw-ring-opacity,1))}.focus-visible\:ring-gray-500\/0:focus-visible{--tw-ring-color:hsla(220,9%,46%,0)}.focus-visible\:ring-gray-500\/10:focus-visible{--tw-ring-color:hsla(220,9%,46%,.1)}.focus-visible\:ring-gray-500\/100:focus-visible{--tw-ring-color:#6b7280}.focus-visible\:ring-gray-500\/15:focus-visible{--tw-ring-color:hsla(220,9%,46%,.15)}.focus-visible\:ring-gray-500\/20:focus-visible{--tw-ring-color:hsla(220,9%,46%,.2)}.focus-visible\:ring-gray-500\/25:focus-visible{--tw-ring-color:hsla(220,9%,46%,.25)}.focus-visible\:ring-gray-500\/30:focus-visible{--tw-ring-color:hsla(220,9%,46%,.3)}.focus-visible\:ring-gray-500\/35:focus-visible{--tw-ring-color:hsla(220,9%,46%,.35)}.focus-visible\:ring-gray-500\/40:focus-visible{--tw-ring-color:hsla(220,9%,46%,.4)}.focus-visible\:ring-gray-500\/45:focus-visible{--tw-ring-color:hsla(220,9%,46%,.45)}.focus-visible\:ring-gray-500\/5:focus-visible{--tw-ring-color:hsla(220,9%,46%,.05)}.focus-visible\:ring-gray-500\/50:focus-visible{--tw-ring-color:hsla(220,9%,46%,.5)}.focus-visible\:ring-gray-500\/55:focus-visible{--tw-ring-color:hsla(220,9%,46%,.55)}.focus-visible\:ring-gray-500\/60:focus-visible{--tw-ring-color:hsla(220,9%,46%,.6)}.focus-visible\:ring-gray-500\/65:focus-visible{--tw-ring-color:hsla(220,9%,46%,.65)}.focus-visible\:ring-gray-500\/70:focus-visible{--tw-ring-color:hsla(220,9%,46%,.7)}.focus-visible\:ring-gray-500\/75:focus-visible{--tw-ring-color:hsla(220,9%,46%,.75)}.focus-visible\:ring-gray-500\/80:focus-visible{--tw-ring-color:hsla(220,9%,46%,.8)}.focus-visible\:ring-gray-500\/85:focus-visible{--tw-ring-color:hsla(220,9%,46%,.85)}.focus-visible\:ring-gray-500\/90:focus-visible{--tw-ring-color:hsla(220,9%,46%,.9)}.focus-visible\:ring-gray-500\/95:focus-visible{--tw-ring-color:hsla(220,9%,46%,.95)}.focus-visible\:ring-gray-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(3 7 18/var(--tw-ring-opacity,1))}.focus-visible\:ring-gray-950\/0:focus-visible{--tw-ring-color:rgba(3,7,18,0)}.focus-visible\:ring-gray-950\/10:focus-visible{--tw-ring-color:rgba(3,7,18,.1)}.focus-visible\:ring-gray-950\/100:focus-visible{--tw-ring-color:#030712}.focus-visible\:ring-gray-950\/15:focus-visible{--tw-ring-color:rgba(3,7,18,.15)}.focus-visible\:ring-gray-950\/20:focus-visible{--tw-ring-color:rgba(3,7,18,.2)}.focus-visible\:ring-gray-950\/25:focus-visible{--tw-ring-color:rgba(3,7,18,.25)}.focus-visible\:ring-gray-950\/30:focus-visible{--tw-ring-color:rgba(3,7,18,.3)}.focus-visible\:ring-gray-950\/35:focus-visible{--tw-ring-color:rgba(3,7,18,.35)}.focus-visible\:ring-gray-950\/40:focus-visible{--tw-ring-color:rgba(3,7,18,.4)}.focus-visible\:ring-gray-950\/45:focus-visible{--tw-ring-color:rgba(3,7,18,.45)}.focus-visible\:ring-gray-950\/5:focus-visible{--tw-ring-color:rgba(3,7,18,.05)}.focus-visible\:ring-gray-950\/50:focus-visible{--tw-ring-color:rgba(3,7,18,.5)}.focus-visible\:ring-gray-950\/55:focus-visible{--tw-ring-color:rgba(3,7,18,.55)}.focus-visible\:ring-gray-950\/60:focus-visible{--tw-ring-color:rgba(3,7,18,.6)}.focus-visible\:ring-gray-950\/65:focus-visible{--tw-ring-color:rgba(3,7,18,.65)}.focus-visible\:ring-gray-950\/70:focus-visible{--tw-ring-color:rgba(3,7,18,.7)}.focus-visible\:ring-gray-950\/75:focus-visible{--tw-ring-color:rgba(3,7,18,.75)}.focus-visible\:ring-gray-950\/80:focus-visible{--tw-ring-color:rgba(3,7,18,.8)}.focus-visible\:ring-gray-950\/85:focus-visible{--tw-ring-color:rgba(3,7,18,.85)}.focus-visible\:ring-gray-950\/90:focus-visible{--tw-ring-color:rgba(3,7,18,.9)}.focus-visible\:ring-gray-950\/95:focus-visible{--tw-ring-color:rgba(3,7,18,.95)}.focus-visible\:ring-green-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(220 252 231/var(--tw-ring-opacity,1))}.focus-visible\:ring-green-100\/0:focus-visible{--tw-ring-color:rgba(220,252,231,0)}.focus-visible\:ring-green-100\/10:focus-visible{--tw-ring-color:rgba(220,252,231,.1)}.focus-visible\:ring-green-100\/100:focus-visible{--tw-ring-color:#dcfce7}.focus-visible\:ring-green-100\/15:focus-visible{--tw-ring-color:rgba(220,252,231,.15)}.focus-visible\:ring-green-100\/20:focus-visible{--tw-ring-color:rgba(220,252,231,.2)}.focus-visible\:ring-green-100\/25:focus-visible{--tw-ring-color:rgba(220,252,231,.25)}.focus-visible\:ring-green-100\/30:focus-visible{--tw-ring-color:rgba(220,252,231,.3)}.focus-visible\:ring-green-100\/35:focus-visible{--tw-ring-color:rgba(220,252,231,.35)}.focus-visible\:ring-green-100\/40:focus-visible{--tw-ring-color:rgba(220,252,231,.4)}.focus-visible\:ring-green-100\/45:focus-visible{--tw-ring-color:rgba(220,252,231,.45)}.focus-visible\:ring-green-100\/5:focus-visible{--tw-ring-color:rgba(220,252,231,.05)}.focus-visible\:ring-green-100\/50:focus-visible{--tw-ring-color:rgba(220,252,231,.5)}.focus-visible\:ring-green-100\/55:focus-visible{--tw-ring-color:rgba(220,252,231,.55)}.focus-visible\:ring-green-100\/60:focus-visible{--tw-ring-color:rgba(220,252,231,.6)}.focus-visible\:ring-green-100\/65:focus-visible{--tw-ring-color:rgba(220,252,231,.65)}.focus-visible\:ring-green-100\/70:focus-visible{--tw-ring-color:rgba(220,252,231,.7)}.focus-visible\:ring-green-100\/75:focus-visible{--tw-ring-color:rgba(220,252,231,.75)}.focus-visible\:ring-green-100\/80:focus-visible{--tw-ring-color:rgba(220,252,231,.8)}.focus-visible\:ring-green-100\/85:focus-visible{--tw-ring-color:rgba(220,252,231,.85)}.focus-visible\:ring-green-100\/90:focus-visible{--tw-ring-color:rgba(220,252,231,.9)}.focus-visible\:ring-green-100\/95:focus-visible{--tw-ring-color:rgba(220,252,231,.95)}.focus-visible\:ring-green-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(187 247 208/var(--tw-ring-opacity,1))}.focus-visible\:ring-green-200\/0:focus-visible{--tw-ring-color:rgba(187,247,208,0)}.focus-visible\:ring-green-200\/10:focus-visible{--tw-ring-color:rgba(187,247,208,.1)}.focus-visible\:ring-green-200\/100:focus-visible{--tw-ring-color:#bbf7d0}.focus-visible\:ring-green-200\/15:focus-visible{--tw-ring-color:rgba(187,247,208,.15)}.focus-visible\:ring-green-200\/20:focus-visible{--tw-ring-color:rgba(187,247,208,.2)}.focus-visible\:ring-green-200\/25:focus-visible{--tw-ring-color:rgba(187,247,208,.25)}.focus-visible\:ring-green-200\/30:focus-visible{--tw-ring-color:rgba(187,247,208,.3)}.focus-visible\:ring-green-200\/35:focus-visible{--tw-ring-color:rgba(187,247,208,.35)}.focus-visible\:ring-green-200\/40:focus-visible{--tw-ring-color:rgba(187,247,208,.4)}.focus-visible\:ring-green-200\/45:focus-visible{--tw-ring-color:rgba(187,247,208,.45)}.focus-visible\:ring-green-200\/5:focus-visible{--tw-ring-color:rgba(187,247,208,.05)}.focus-visible\:ring-green-200\/50:focus-visible{--tw-ring-color:rgba(187,247,208,.5)}.focus-visible\:ring-green-200\/55:focus-visible{--tw-ring-color:rgba(187,247,208,.55)}.focus-visible\:ring-green-200\/60:focus-visible{--tw-ring-color:rgba(187,247,208,.6)}.focus-visible\:ring-green-200\/65:focus-visible{--tw-ring-color:rgba(187,247,208,.65)}.focus-visible\:ring-green-200\/70:focus-visible{--tw-ring-color:rgba(187,247,208,.7)}.focus-visible\:ring-green-200\/75:focus-visible{--tw-ring-color:rgba(187,247,208,.75)}.focus-visible\:ring-green-200\/80:focus-visible{--tw-ring-color:rgba(187,247,208,.8)}.focus-visible\:ring-green-200\/85:focus-visible{--tw-ring-color:rgba(187,247,208,.85)}.focus-visible\:ring-green-200\/90:focus-visible{--tw-ring-color:rgba(187,247,208,.9)}.focus-visible\:ring-green-200\/95:focus-visible{--tw-ring-color:rgba(187,247,208,.95)}.focus-visible\:ring-green-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(74 222 128/var(--tw-ring-opacity,1))}.focus-visible\:ring-green-400\/0:focus-visible{--tw-ring-color:rgba(74,222,128,0)}.focus-visible\:ring-green-400\/10:focus-visible{--tw-ring-color:rgba(74,222,128,.1)}.focus-visible\:ring-green-400\/100:focus-visible{--tw-ring-color:#4ade80}.focus-visible\:ring-green-400\/15:focus-visible{--tw-ring-color:rgba(74,222,128,.15)}.focus-visible\:ring-green-400\/20:focus-visible{--tw-ring-color:rgba(74,222,128,.2)}.focus-visible\:ring-green-400\/25:focus-visible{--tw-ring-color:rgba(74,222,128,.25)}.focus-visible\:ring-green-400\/30:focus-visible{--tw-ring-color:rgba(74,222,128,.3)}.focus-visible\:ring-green-400\/35:focus-visible{--tw-ring-color:rgba(74,222,128,.35)}.focus-visible\:ring-green-400\/40:focus-visible{--tw-ring-color:rgba(74,222,128,.4)}.focus-visible\:ring-green-400\/45:focus-visible{--tw-ring-color:rgba(74,222,128,.45)}.focus-visible\:ring-green-400\/5:focus-visible{--tw-ring-color:rgba(74,222,128,.05)}.focus-visible\:ring-green-400\/50:focus-visible{--tw-ring-color:rgba(74,222,128,.5)}.focus-visible\:ring-green-400\/55:focus-visible{--tw-ring-color:rgba(74,222,128,.55)}.focus-visible\:ring-green-400\/60:focus-visible{--tw-ring-color:rgba(74,222,128,.6)}.focus-visible\:ring-green-400\/65:focus-visible{--tw-ring-color:rgba(74,222,128,.65)}.focus-visible\:ring-green-400\/70:focus-visible{--tw-ring-color:rgba(74,222,128,.7)}.focus-visible\:ring-green-400\/75:focus-visible{--tw-ring-color:rgba(74,222,128,.75)}.focus-visible\:ring-green-400\/80:focus-visible{--tw-ring-color:rgba(74,222,128,.8)}.focus-visible\:ring-green-400\/85:focus-visible{--tw-ring-color:rgba(74,222,128,.85)}.focus-visible\:ring-green-400\/90:focus-visible{--tw-ring-color:rgba(74,222,128,.9)}.focus-visible\:ring-green-400\/95:focus-visible{--tw-ring-color:rgba(74,222,128,.95)}.focus-visible\:ring-green-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(240 253 244/var(--tw-ring-opacity,1))}.focus-visible\:ring-green-50\/0:focus-visible{--tw-ring-color:rgba(240,253,244,0)}.focus-visible\:ring-green-50\/10:focus-visible{--tw-ring-color:rgba(240,253,244,.1)}.focus-visible\:ring-green-50\/100:focus-visible{--tw-ring-color:#f0fdf4}.focus-visible\:ring-green-50\/15:focus-visible{--tw-ring-color:rgba(240,253,244,.15)}.focus-visible\:ring-green-50\/20:focus-visible{--tw-ring-color:rgba(240,253,244,.2)}.focus-visible\:ring-green-50\/25:focus-visible{--tw-ring-color:rgba(240,253,244,.25)}.focus-visible\:ring-green-50\/30:focus-visible{--tw-ring-color:rgba(240,253,244,.3)}.focus-visible\:ring-green-50\/35:focus-visible{--tw-ring-color:rgba(240,253,244,.35)}.focus-visible\:ring-green-50\/40:focus-visible{--tw-ring-color:rgba(240,253,244,.4)}.focus-visible\:ring-green-50\/45:focus-visible{--tw-ring-color:rgba(240,253,244,.45)}.focus-visible\:ring-green-50\/5:focus-visible{--tw-ring-color:rgba(240,253,244,.05)}.focus-visible\:ring-green-50\/50:focus-visible{--tw-ring-color:rgba(240,253,244,.5)}.focus-visible\:ring-green-50\/55:focus-visible{--tw-ring-color:rgba(240,253,244,.55)}.focus-visible\:ring-green-50\/60:focus-visible{--tw-ring-color:rgba(240,253,244,.6)}.focus-visible\:ring-green-50\/65:focus-visible{--tw-ring-color:rgba(240,253,244,.65)}.focus-visible\:ring-green-50\/70:focus-visible{--tw-ring-color:rgba(240,253,244,.7)}.focus-visible\:ring-green-50\/75:focus-visible{--tw-ring-color:rgba(240,253,244,.75)}.focus-visible\:ring-green-50\/80:focus-visible{--tw-ring-color:rgba(240,253,244,.8)}.focus-visible\:ring-green-50\/85:focus-visible{--tw-ring-color:rgba(240,253,244,.85)}.focus-visible\:ring-green-50\/90:focus-visible{--tw-ring-color:rgba(240,253,244,.9)}.focus-visible\:ring-green-50\/95:focus-visible{--tw-ring-color:rgba(240,253,244,.95)}.focus-visible\:ring-green-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(34 197 94/var(--tw-ring-opacity,1))}.focus-visible\:ring-green-500\/0:focus-visible{--tw-ring-color:rgba(34,197,94,0)}.focus-visible\:ring-green-500\/10:focus-visible{--tw-ring-color:rgba(34,197,94,.1)}.focus-visible\:ring-green-500\/100:focus-visible{--tw-ring-color:#22c55e}.focus-visible\:ring-green-500\/15:focus-visible{--tw-ring-color:rgba(34,197,94,.15)}.focus-visible\:ring-green-500\/20:focus-visible{--tw-ring-color:rgba(34,197,94,.2)}.focus-visible\:ring-green-500\/25:focus-visible{--tw-ring-color:rgba(34,197,94,.25)}.focus-visible\:ring-green-500\/30:focus-visible{--tw-ring-color:rgba(34,197,94,.3)}.focus-visible\:ring-green-500\/35:focus-visible{--tw-ring-color:rgba(34,197,94,.35)}.focus-visible\:ring-green-500\/40:focus-visible{--tw-ring-color:rgba(34,197,94,.4)}.focus-visible\:ring-green-500\/45:focus-visible{--tw-ring-color:rgba(34,197,94,.45)}.focus-visible\:ring-green-500\/5:focus-visible{--tw-ring-color:rgba(34,197,94,.05)}.focus-visible\:ring-green-500\/50:focus-visible{--tw-ring-color:rgba(34,197,94,.5)}.focus-visible\:ring-green-500\/55:focus-visible{--tw-ring-color:rgba(34,197,94,.55)}.focus-visible\:ring-green-500\/60:focus-visible{--tw-ring-color:rgba(34,197,94,.6)}.focus-visible\:ring-green-500\/65:focus-visible{--tw-ring-color:rgba(34,197,94,.65)}.focus-visible\:ring-green-500\/70:focus-visible{--tw-ring-color:rgba(34,197,94,.7)}.focus-visible\:ring-green-500\/75:focus-visible{--tw-ring-color:rgba(34,197,94,.75)}.focus-visible\:ring-green-500\/80:focus-visible{--tw-ring-color:rgba(34,197,94,.8)}.focus-visible\:ring-green-500\/85:focus-visible{--tw-ring-color:rgba(34,197,94,.85)}.focus-visible\:ring-green-500\/90:focus-visible{--tw-ring-color:rgba(34,197,94,.9)}.focus-visible\:ring-green-500\/95:focus-visible{--tw-ring-color:rgba(34,197,94,.95)}.focus-visible\:ring-green-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(5 46 22/var(--tw-ring-opacity,1))}.focus-visible\:ring-green-950\/0:focus-visible{--tw-ring-color:rgba(5,46,22,0)}.focus-visible\:ring-green-950\/10:focus-visible{--tw-ring-color:rgba(5,46,22,.1)}.focus-visible\:ring-green-950\/100:focus-visible{--tw-ring-color:#052e16}.focus-visible\:ring-green-950\/15:focus-visible{--tw-ring-color:rgba(5,46,22,.15)}.focus-visible\:ring-green-950\/20:focus-visible{--tw-ring-color:rgba(5,46,22,.2)}.focus-visible\:ring-green-950\/25:focus-visible{--tw-ring-color:rgba(5,46,22,.25)}.focus-visible\:ring-green-950\/30:focus-visible{--tw-ring-color:rgba(5,46,22,.3)}.focus-visible\:ring-green-950\/35:focus-visible{--tw-ring-color:rgba(5,46,22,.35)}.focus-visible\:ring-green-950\/40:focus-visible{--tw-ring-color:rgba(5,46,22,.4)}.focus-visible\:ring-green-950\/45:focus-visible{--tw-ring-color:rgba(5,46,22,.45)}.focus-visible\:ring-green-950\/5:focus-visible{--tw-ring-color:rgba(5,46,22,.05)}.focus-visible\:ring-green-950\/50:focus-visible{--tw-ring-color:rgba(5,46,22,.5)}.focus-visible\:ring-green-950\/55:focus-visible{--tw-ring-color:rgba(5,46,22,.55)}.focus-visible\:ring-green-950\/60:focus-visible{--tw-ring-color:rgba(5,46,22,.6)}.focus-visible\:ring-green-950\/65:focus-visible{--tw-ring-color:rgba(5,46,22,.65)}.focus-visible\:ring-green-950\/70:focus-visible{--tw-ring-color:rgba(5,46,22,.7)}.focus-visible\:ring-green-950\/75:focus-visible{--tw-ring-color:rgba(5,46,22,.75)}.focus-visible\:ring-green-950\/80:focus-visible{--tw-ring-color:rgba(5,46,22,.8)}.focus-visible\:ring-green-950\/85:focus-visible{--tw-ring-color:rgba(5,46,22,.85)}.focus-visible\:ring-green-950\/90:focus-visible{--tw-ring-color:rgba(5,46,22,.9)}.focus-visible\:ring-green-950\/95:focus-visible{--tw-ring-color:rgba(5,46,22,.95)}.focus-visible\:ring-indigo-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(224 231 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-indigo-100\/0:focus-visible{--tw-ring-color:rgba(224,231,255,0)}.focus-visible\:ring-indigo-100\/10:focus-visible{--tw-ring-color:rgba(224,231,255,.1)}.focus-visible\:ring-indigo-100\/100:focus-visible{--tw-ring-color:#e0e7ff}.focus-visible\:ring-indigo-100\/15:focus-visible{--tw-ring-color:rgba(224,231,255,.15)}.focus-visible\:ring-indigo-100\/20:focus-visible{--tw-ring-color:rgba(224,231,255,.2)}.focus-visible\:ring-indigo-100\/25:focus-visible{--tw-ring-color:rgba(224,231,255,.25)}.focus-visible\:ring-indigo-100\/30:focus-visible{--tw-ring-color:rgba(224,231,255,.3)}.focus-visible\:ring-indigo-100\/35:focus-visible{--tw-ring-color:rgba(224,231,255,.35)}.focus-visible\:ring-indigo-100\/40:focus-visible{--tw-ring-color:rgba(224,231,255,.4)}.focus-visible\:ring-indigo-100\/45:focus-visible{--tw-ring-color:rgba(224,231,255,.45)}.focus-visible\:ring-indigo-100\/5:focus-visible{--tw-ring-color:rgba(224,231,255,.05)}.focus-visible\:ring-indigo-100\/50:focus-visible{--tw-ring-color:rgba(224,231,255,.5)}.focus-visible\:ring-indigo-100\/55:focus-visible{--tw-ring-color:rgba(224,231,255,.55)}.focus-visible\:ring-indigo-100\/60:focus-visible{--tw-ring-color:rgba(224,231,255,.6)}.focus-visible\:ring-indigo-100\/65:focus-visible{--tw-ring-color:rgba(224,231,255,.65)}.focus-visible\:ring-indigo-100\/70:focus-visible{--tw-ring-color:rgba(224,231,255,.7)}.focus-visible\:ring-indigo-100\/75:focus-visible{--tw-ring-color:rgba(224,231,255,.75)}.focus-visible\:ring-indigo-100\/80:focus-visible{--tw-ring-color:rgba(224,231,255,.8)}.focus-visible\:ring-indigo-100\/85:focus-visible{--tw-ring-color:rgba(224,231,255,.85)}.focus-visible\:ring-indigo-100\/90:focus-visible{--tw-ring-color:rgba(224,231,255,.9)}.focus-visible\:ring-indigo-100\/95:focus-visible{--tw-ring-color:rgba(224,231,255,.95)}.focus-visible\:ring-indigo-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(199 210 254/var(--tw-ring-opacity,1))}.focus-visible\:ring-indigo-200\/0:focus-visible{--tw-ring-color:rgba(199,210,254,0)}.focus-visible\:ring-indigo-200\/10:focus-visible{--tw-ring-color:rgba(199,210,254,.1)}.focus-visible\:ring-indigo-200\/100:focus-visible{--tw-ring-color:#c7d2fe}.focus-visible\:ring-indigo-200\/15:focus-visible{--tw-ring-color:rgba(199,210,254,.15)}.focus-visible\:ring-indigo-200\/20:focus-visible{--tw-ring-color:rgba(199,210,254,.2)}.focus-visible\:ring-indigo-200\/25:focus-visible{--tw-ring-color:rgba(199,210,254,.25)}.focus-visible\:ring-indigo-200\/30:focus-visible{--tw-ring-color:rgba(199,210,254,.3)}.focus-visible\:ring-indigo-200\/35:focus-visible{--tw-ring-color:rgba(199,210,254,.35)}.focus-visible\:ring-indigo-200\/40:focus-visible{--tw-ring-color:rgba(199,210,254,.4)}.focus-visible\:ring-indigo-200\/45:focus-visible{--tw-ring-color:rgba(199,210,254,.45)}.focus-visible\:ring-indigo-200\/5:focus-visible{--tw-ring-color:rgba(199,210,254,.05)}.focus-visible\:ring-indigo-200\/50:focus-visible{--tw-ring-color:rgba(199,210,254,.5)}.focus-visible\:ring-indigo-200\/55:focus-visible{--tw-ring-color:rgba(199,210,254,.55)}.focus-visible\:ring-indigo-200\/60:focus-visible{--tw-ring-color:rgba(199,210,254,.6)}.focus-visible\:ring-indigo-200\/65:focus-visible{--tw-ring-color:rgba(199,210,254,.65)}.focus-visible\:ring-indigo-200\/70:focus-visible{--tw-ring-color:rgba(199,210,254,.7)}.focus-visible\:ring-indigo-200\/75:focus-visible{--tw-ring-color:rgba(199,210,254,.75)}.focus-visible\:ring-indigo-200\/80:focus-visible{--tw-ring-color:rgba(199,210,254,.8)}.focus-visible\:ring-indigo-200\/85:focus-visible{--tw-ring-color:rgba(199,210,254,.85)}.focus-visible\:ring-indigo-200\/90:focus-visible{--tw-ring-color:rgba(199,210,254,.9)}.focus-visible\:ring-indigo-200\/95:focus-visible{--tw-ring-color:rgba(199,210,254,.95)}.focus-visible\:ring-indigo-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(129 140 248/var(--tw-ring-opacity,1))}.focus-visible\:ring-indigo-400\/0:focus-visible{--tw-ring-color:rgba(129,140,248,0)}.focus-visible\:ring-indigo-400\/10:focus-visible{--tw-ring-color:rgba(129,140,248,.1)}.focus-visible\:ring-indigo-400\/100:focus-visible{--tw-ring-color:#818cf8}.focus-visible\:ring-indigo-400\/15:focus-visible{--tw-ring-color:rgba(129,140,248,.15)}.focus-visible\:ring-indigo-400\/20:focus-visible{--tw-ring-color:rgba(129,140,248,.2)}.focus-visible\:ring-indigo-400\/25:focus-visible{--tw-ring-color:rgba(129,140,248,.25)}.focus-visible\:ring-indigo-400\/30:focus-visible{--tw-ring-color:rgba(129,140,248,.3)}.focus-visible\:ring-indigo-400\/35:focus-visible{--tw-ring-color:rgba(129,140,248,.35)}.focus-visible\:ring-indigo-400\/40:focus-visible{--tw-ring-color:rgba(129,140,248,.4)}.focus-visible\:ring-indigo-400\/45:focus-visible{--tw-ring-color:rgba(129,140,248,.45)}.focus-visible\:ring-indigo-400\/5:focus-visible{--tw-ring-color:rgba(129,140,248,.05)}.focus-visible\:ring-indigo-400\/50:focus-visible{--tw-ring-color:rgba(129,140,248,.5)}.focus-visible\:ring-indigo-400\/55:focus-visible{--tw-ring-color:rgba(129,140,248,.55)}.focus-visible\:ring-indigo-400\/60:focus-visible{--tw-ring-color:rgba(129,140,248,.6)}.focus-visible\:ring-indigo-400\/65:focus-visible{--tw-ring-color:rgba(129,140,248,.65)}.focus-visible\:ring-indigo-400\/70:focus-visible{--tw-ring-color:rgba(129,140,248,.7)}.focus-visible\:ring-indigo-400\/75:focus-visible{--tw-ring-color:rgba(129,140,248,.75)}.focus-visible\:ring-indigo-400\/80:focus-visible{--tw-ring-color:rgba(129,140,248,.8)}.focus-visible\:ring-indigo-400\/85:focus-visible{--tw-ring-color:rgba(129,140,248,.85)}.focus-visible\:ring-indigo-400\/90:focus-visible{--tw-ring-color:rgba(129,140,248,.9)}.focus-visible\:ring-indigo-400\/95:focus-visible{--tw-ring-color:rgba(129,140,248,.95)}.focus-visible\:ring-indigo-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(238 242 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-indigo-50\/0:focus-visible{--tw-ring-color:rgba(238,242,255,0)}.focus-visible\:ring-indigo-50\/10:focus-visible{--tw-ring-color:rgba(238,242,255,.1)}.focus-visible\:ring-indigo-50\/100:focus-visible{--tw-ring-color:#eef2ff}.focus-visible\:ring-indigo-50\/15:focus-visible{--tw-ring-color:rgba(238,242,255,.15)}.focus-visible\:ring-indigo-50\/20:focus-visible{--tw-ring-color:rgba(238,242,255,.2)}.focus-visible\:ring-indigo-50\/25:focus-visible{--tw-ring-color:rgba(238,242,255,.25)}.focus-visible\:ring-indigo-50\/30:focus-visible{--tw-ring-color:rgba(238,242,255,.3)}.focus-visible\:ring-indigo-50\/35:focus-visible{--tw-ring-color:rgba(238,242,255,.35)}.focus-visible\:ring-indigo-50\/40:focus-visible{--tw-ring-color:rgba(238,242,255,.4)}.focus-visible\:ring-indigo-50\/45:focus-visible{--tw-ring-color:rgba(238,242,255,.45)}.focus-visible\:ring-indigo-50\/5:focus-visible{--tw-ring-color:rgba(238,242,255,.05)}.focus-visible\:ring-indigo-50\/50:focus-visible{--tw-ring-color:rgba(238,242,255,.5)}.focus-visible\:ring-indigo-50\/55:focus-visible{--tw-ring-color:rgba(238,242,255,.55)}.focus-visible\:ring-indigo-50\/60:focus-visible{--tw-ring-color:rgba(238,242,255,.6)}.focus-visible\:ring-indigo-50\/65:focus-visible{--tw-ring-color:rgba(238,242,255,.65)}.focus-visible\:ring-indigo-50\/70:focus-visible{--tw-ring-color:rgba(238,242,255,.7)}.focus-visible\:ring-indigo-50\/75:focus-visible{--tw-ring-color:rgba(238,242,255,.75)}.focus-visible\:ring-indigo-50\/80:focus-visible{--tw-ring-color:rgba(238,242,255,.8)}.focus-visible\:ring-indigo-50\/85:focus-visible{--tw-ring-color:rgba(238,242,255,.85)}.focus-visible\:ring-indigo-50\/90:focus-visible{--tw-ring-color:rgba(238,242,255,.9)}.focus-visible\:ring-indigo-50\/95:focus-visible{--tw-ring-color:rgba(238,242,255,.95)}.focus-visible\:ring-indigo-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(99 102 241/var(--tw-ring-opacity,1))}.focus-visible\:ring-indigo-500\/0:focus-visible{--tw-ring-color:rgba(99,102,241,0)}.focus-visible\:ring-indigo-500\/10:focus-visible{--tw-ring-color:rgba(99,102,241,.1)}.focus-visible\:ring-indigo-500\/100:focus-visible{--tw-ring-color:#6366f1}.focus-visible\:ring-indigo-500\/15:focus-visible{--tw-ring-color:rgba(99,102,241,.15)}.focus-visible\:ring-indigo-500\/20:focus-visible{--tw-ring-color:rgba(99,102,241,.2)}.focus-visible\:ring-indigo-500\/25:focus-visible{--tw-ring-color:rgba(99,102,241,.25)}.focus-visible\:ring-indigo-500\/30:focus-visible{--tw-ring-color:rgba(99,102,241,.3)}.focus-visible\:ring-indigo-500\/35:focus-visible{--tw-ring-color:rgba(99,102,241,.35)}.focus-visible\:ring-indigo-500\/40:focus-visible{--tw-ring-color:rgba(99,102,241,.4)}.focus-visible\:ring-indigo-500\/45:focus-visible{--tw-ring-color:rgba(99,102,241,.45)}.focus-visible\:ring-indigo-500\/5:focus-visible{--tw-ring-color:rgba(99,102,241,.05)}.focus-visible\:ring-indigo-500\/50:focus-visible{--tw-ring-color:rgba(99,102,241,.5)}.focus-visible\:ring-indigo-500\/55:focus-visible{--tw-ring-color:rgba(99,102,241,.55)}.focus-visible\:ring-indigo-500\/60:focus-visible{--tw-ring-color:rgba(99,102,241,.6)}.focus-visible\:ring-indigo-500\/65:focus-visible{--tw-ring-color:rgba(99,102,241,.65)}.focus-visible\:ring-indigo-500\/70:focus-visible{--tw-ring-color:rgba(99,102,241,.7)}.focus-visible\:ring-indigo-500\/75:focus-visible{--tw-ring-color:rgba(99,102,241,.75)}.focus-visible\:ring-indigo-500\/80:focus-visible{--tw-ring-color:rgba(99,102,241,.8)}.focus-visible\:ring-indigo-500\/85:focus-visible{--tw-ring-color:rgba(99,102,241,.85)}.focus-visible\:ring-indigo-500\/90:focus-visible{--tw-ring-color:rgba(99,102,241,.9)}.focus-visible\:ring-indigo-500\/95:focus-visible{--tw-ring-color:rgba(99,102,241,.95)}.focus-visible\:ring-indigo-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(30 27 75/var(--tw-ring-opacity,1))}.focus-visible\:ring-indigo-950\/0:focus-visible{--tw-ring-color:rgba(30,27,75,0)}.focus-visible\:ring-indigo-950\/10:focus-visible{--tw-ring-color:rgba(30,27,75,.1)}.focus-visible\:ring-indigo-950\/100:focus-visible{--tw-ring-color:#1e1b4b}.focus-visible\:ring-indigo-950\/15:focus-visible{--tw-ring-color:rgba(30,27,75,.15)}.focus-visible\:ring-indigo-950\/20:focus-visible{--tw-ring-color:rgba(30,27,75,.2)}.focus-visible\:ring-indigo-950\/25:focus-visible{--tw-ring-color:rgba(30,27,75,.25)}.focus-visible\:ring-indigo-950\/30:focus-visible{--tw-ring-color:rgba(30,27,75,.3)}.focus-visible\:ring-indigo-950\/35:focus-visible{--tw-ring-color:rgba(30,27,75,.35)}.focus-visible\:ring-indigo-950\/40:focus-visible{--tw-ring-color:rgba(30,27,75,.4)}.focus-visible\:ring-indigo-950\/45:focus-visible{--tw-ring-color:rgba(30,27,75,.45)}.focus-visible\:ring-indigo-950\/5:focus-visible{--tw-ring-color:rgba(30,27,75,.05)}.focus-visible\:ring-indigo-950\/50:focus-visible{--tw-ring-color:rgba(30,27,75,.5)}.focus-visible\:ring-indigo-950\/55:focus-visible{--tw-ring-color:rgba(30,27,75,.55)}.focus-visible\:ring-indigo-950\/60:focus-visible{--tw-ring-color:rgba(30,27,75,.6)}.focus-visible\:ring-indigo-950\/65:focus-visible{--tw-ring-color:rgba(30,27,75,.65)}.focus-visible\:ring-indigo-950\/70:focus-visible{--tw-ring-color:rgba(30,27,75,.7)}.focus-visible\:ring-indigo-950\/75:focus-visible{--tw-ring-color:rgba(30,27,75,.75)}.focus-visible\:ring-indigo-950\/80:focus-visible{--tw-ring-color:rgba(30,27,75,.8)}.focus-visible\:ring-indigo-950\/85:focus-visible{--tw-ring-color:rgba(30,27,75,.85)}.focus-visible\:ring-indigo-950\/90:focus-visible{--tw-ring-color:rgba(30,27,75,.9)}.focus-visible\:ring-indigo-950\/95:focus-visible{--tw-ring-color:rgba(30,27,75,.95)}.focus-visible\:ring-lime-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(236 252 203/var(--tw-ring-opacity,1))}.focus-visible\:ring-lime-100\/0:focus-visible{--tw-ring-color:hsla(80,89%,89%,0)}.focus-visible\:ring-lime-100\/10:focus-visible{--tw-ring-color:hsla(80,89%,89%,.1)}.focus-visible\:ring-lime-100\/100:focus-visible{--tw-ring-color:#ecfccb}.focus-visible\:ring-lime-100\/15:focus-visible{--tw-ring-color:hsla(80,89%,89%,.15)}.focus-visible\:ring-lime-100\/20:focus-visible{--tw-ring-color:hsla(80,89%,89%,.2)}.focus-visible\:ring-lime-100\/25:focus-visible{--tw-ring-color:hsla(80,89%,89%,.25)}.focus-visible\:ring-lime-100\/30:focus-visible{--tw-ring-color:hsla(80,89%,89%,.3)}.focus-visible\:ring-lime-100\/35:focus-visible{--tw-ring-color:hsla(80,89%,89%,.35)}.focus-visible\:ring-lime-100\/40:focus-visible{--tw-ring-color:hsla(80,89%,89%,.4)}.focus-visible\:ring-lime-100\/45:focus-visible{--tw-ring-color:hsla(80,89%,89%,.45)}.focus-visible\:ring-lime-100\/5:focus-visible{--tw-ring-color:hsla(80,89%,89%,.05)}.focus-visible\:ring-lime-100\/50:focus-visible{--tw-ring-color:hsla(80,89%,89%,.5)}.focus-visible\:ring-lime-100\/55:focus-visible{--tw-ring-color:hsla(80,89%,89%,.55)}.focus-visible\:ring-lime-100\/60:focus-visible{--tw-ring-color:hsla(80,89%,89%,.6)}.focus-visible\:ring-lime-100\/65:focus-visible{--tw-ring-color:hsla(80,89%,89%,.65)}.focus-visible\:ring-lime-100\/70:focus-visible{--tw-ring-color:hsla(80,89%,89%,.7)}.focus-visible\:ring-lime-100\/75:focus-visible{--tw-ring-color:hsla(80,89%,89%,.75)}.focus-visible\:ring-lime-100\/80:focus-visible{--tw-ring-color:hsla(80,89%,89%,.8)}.focus-visible\:ring-lime-100\/85:focus-visible{--tw-ring-color:hsla(80,89%,89%,.85)}.focus-visible\:ring-lime-100\/90:focus-visible{--tw-ring-color:hsla(80,89%,89%,.9)}.focus-visible\:ring-lime-100\/95:focus-visible{--tw-ring-color:hsla(80,89%,89%,.95)}.focus-visible\:ring-lime-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(217 249 157/var(--tw-ring-opacity,1))}.focus-visible\:ring-lime-200\/0:focus-visible{--tw-ring-color:hsla(81,88%,80%,0)}.focus-visible\:ring-lime-200\/10:focus-visible{--tw-ring-color:hsla(81,88%,80%,.1)}.focus-visible\:ring-lime-200\/100:focus-visible{--tw-ring-color:#d9f99d}.focus-visible\:ring-lime-200\/15:focus-visible{--tw-ring-color:hsla(81,88%,80%,.15)}.focus-visible\:ring-lime-200\/20:focus-visible{--tw-ring-color:hsla(81,88%,80%,.2)}.focus-visible\:ring-lime-200\/25:focus-visible{--tw-ring-color:hsla(81,88%,80%,.25)}.focus-visible\:ring-lime-200\/30:focus-visible{--tw-ring-color:hsla(81,88%,80%,.3)}.focus-visible\:ring-lime-200\/35:focus-visible{--tw-ring-color:hsla(81,88%,80%,.35)}.focus-visible\:ring-lime-200\/40:focus-visible{--tw-ring-color:hsla(81,88%,80%,.4)}.focus-visible\:ring-lime-200\/45:focus-visible{--tw-ring-color:hsla(81,88%,80%,.45)}.focus-visible\:ring-lime-200\/5:focus-visible{--tw-ring-color:hsla(81,88%,80%,.05)}.focus-visible\:ring-lime-200\/50:focus-visible{--tw-ring-color:hsla(81,88%,80%,.5)}.focus-visible\:ring-lime-200\/55:focus-visible{--tw-ring-color:hsla(81,88%,80%,.55)}.focus-visible\:ring-lime-200\/60:focus-visible{--tw-ring-color:hsla(81,88%,80%,.6)}.focus-visible\:ring-lime-200\/65:focus-visible{--tw-ring-color:hsla(81,88%,80%,.65)}.focus-visible\:ring-lime-200\/70:focus-visible{--tw-ring-color:hsla(81,88%,80%,.7)}.focus-visible\:ring-lime-200\/75:focus-visible{--tw-ring-color:hsla(81,88%,80%,.75)}.focus-visible\:ring-lime-200\/80:focus-visible{--tw-ring-color:hsla(81,88%,80%,.8)}.focus-visible\:ring-lime-200\/85:focus-visible{--tw-ring-color:hsla(81,88%,80%,.85)}.focus-visible\:ring-lime-200\/90:focus-visible{--tw-ring-color:hsla(81,88%,80%,.9)}.focus-visible\:ring-lime-200\/95:focus-visible{--tw-ring-color:hsla(81,88%,80%,.95)}.focus-visible\:ring-lime-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(163 230 53/var(--tw-ring-opacity,1))}.focus-visible\:ring-lime-400\/0:focus-visible{--tw-ring-color:rgba(163,230,53,0)}.focus-visible\:ring-lime-400\/10:focus-visible{--tw-ring-color:rgba(163,230,53,.1)}.focus-visible\:ring-lime-400\/100:focus-visible{--tw-ring-color:#a3e635}.focus-visible\:ring-lime-400\/15:focus-visible{--tw-ring-color:rgba(163,230,53,.15)}.focus-visible\:ring-lime-400\/20:focus-visible{--tw-ring-color:rgba(163,230,53,.2)}.focus-visible\:ring-lime-400\/25:focus-visible{--tw-ring-color:rgba(163,230,53,.25)}.focus-visible\:ring-lime-400\/30:focus-visible{--tw-ring-color:rgba(163,230,53,.3)}.focus-visible\:ring-lime-400\/35:focus-visible{--tw-ring-color:rgba(163,230,53,.35)}.focus-visible\:ring-lime-400\/40:focus-visible{--tw-ring-color:rgba(163,230,53,.4)}.focus-visible\:ring-lime-400\/45:focus-visible{--tw-ring-color:rgba(163,230,53,.45)}.focus-visible\:ring-lime-400\/5:focus-visible{--tw-ring-color:rgba(163,230,53,.05)}.focus-visible\:ring-lime-400\/50:focus-visible{--tw-ring-color:rgba(163,230,53,.5)}.focus-visible\:ring-lime-400\/55:focus-visible{--tw-ring-color:rgba(163,230,53,.55)}.focus-visible\:ring-lime-400\/60:focus-visible{--tw-ring-color:rgba(163,230,53,.6)}.focus-visible\:ring-lime-400\/65:focus-visible{--tw-ring-color:rgba(163,230,53,.65)}.focus-visible\:ring-lime-400\/70:focus-visible{--tw-ring-color:rgba(163,230,53,.7)}.focus-visible\:ring-lime-400\/75:focus-visible{--tw-ring-color:rgba(163,230,53,.75)}.focus-visible\:ring-lime-400\/80:focus-visible{--tw-ring-color:rgba(163,230,53,.8)}.focus-visible\:ring-lime-400\/85:focus-visible{--tw-ring-color:rgba(163,230,53,.85)}.focus-visible\:ring-lime-400\/90:focus-visible{--tw-ring-color:rgba(163,230,53,.9)}.focus-visible\:ring-lime-400\/95:focus-visible{--tw-ring-color:rgba(163,230,53,.95)}.focus-visible\:ring-lime-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(247 254 231/var(--tw-ring-opacity,1))}.focus-visible\:ring-lime-50\/0:focus-visible{--tw-ring-color:hsla(78,92%,95%,0)}.focus-visible\:ring-lime-50\/10:focus-visible{--tw-ring-color:hsla(78,92%,95%,.1)}.focus-visible\:ring-lime-50\/100:focus-visible{--tw-ring-color:#f7fee7}.focus-visible\:ring-lime-50\/15:focus-visible{--tw-ring-color:hsla(78,92%,95%,.15)}.focus-visible\:ring-lime-50\/20:focus-visible{--tw-ring-color:hsla(78,92%,95%,.2)}.focus-visible\:ring-lime-50\/25:focus-visible{--tw-ring-color:hsla(78,92%,95%,.25)}.focus-visible\:ring-lime-50\/30:focus-visible{--tw-ring-color:hsla(78,92%,95%,.3)}.focus-visible\:ring-lime-50\/35:focus-visible{--tw-ring-color:hsla(78,92%,95%,.35)}.focus-visible\:ring-lime-50\/40:focus-visible{--tw-ring-color:hsla(78,92%,95%,.4)}.focus-visible\:ring-lime-50\/45:focus-visible{--tw-ring-color:hsla(78,92%,95%,.45)}.focus-visible\:ring-lime-50\/5:focus-visible{--tw-ring-color:hsla(78,92%,95%,.05)}.focus-visible\:ring-lime-50\/50:focus-visible{--tw-ring-color:hsla(78,92%,95%,.5)}.focus-visible\:ring-lime-50\/55:focus-visible{--tw-ring-color:hsla(78,92%,95%,.55)}.focus-visible\:ring-lime-50\/60:focus-visible{--tw-ring-color:hsla(78,92%,95%,.6)}.focus-visible\:ring-lime-50\/65:focus-visible{--tw-ring-color:hsla(78,92%,95%,.65)}.focus-visible\:ring-lime-50\/70:focus-visible{--tw-ring-color:hsla(78,92%,95%,.7)}.focus-visible\:ring-lime-50\/75:focus-visible{--tw-ring-color:hsla(78,92%,95%,.75)}.focus-visible\:ring-lime-50\/80:focus-visible{--tw-ring-color:hsla(78,92%,95%,.8)}.focus-visible\:ring-lime-50\/85:focus-visible{--tw-ring-color:hsla(78,92%,95%,.85)}.focus-visible\:ring-lime-50\/90:focus-visible{--tw-ring-color:hsla(78,92%,95%,.9)}.focus-visible\:ring-lime-50\/95:focus-visible{--tw-ring-color:hsla(78,92%,95%,.95)}.focus-visible\:ring-lime-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(132 204 22/var(--tw-ring-opacity,1))}.focus-visible\:ring-lime-500\/0:focus-visible{--tw-ring-color:rgba(132,204,22,0)}.focus-visible\:ring-lime-500\/10:focus-visible{--tw-ring-color:rgba(132,204,22,.1)}.focus-visible\:ring-lime-500\/100:focus-visible{--tw-ring-color:#84cc16}.focus-visible\:ring-lime-500\/15:focus-visible{--tw-ring-color:rgba(132,204,22,.15)}.focus-visible\:ring-lime-500\/20:focus-visible{--tw-ring-color:rgba(132,204,22,.2)}.focus-visible\:ring-lime-500\/25:focus-visible{--tw-ring-color:rgba(132,204,22,.25)}.focus-visible\:ring-lime-500\/30:focus-visible{--tw-ring-color:rgba(132,204,22,.3)}.focus-visible\:ring-lime-500\/35:focus-visible{--tw-ring-color:rgba(132,204,22,.35)}.focus-visible\:ring-lime-500\/40:focus-visible{--tw-ring-color:rgba(132,204,22,.4)}.focus-visible\:ring-lime-500\/45:focus-visible{--tw-ring-color:rgba(132,204,22,.45)}.focus-visible\:ring-lime-500\/5:focus-visible{--tw-ring-color:rgba(132,204,22,.05)}.focus-visible\:ring-lime-500\/50:focus-visible{--tw-ring-color:rgba(132,204,22,.5)}.focus-visible\:ring-lime-500\/55:focus-visible{--tw-ring-color:rgba(132,204,22,.55)}.focus-visible\:ring-lime-500\/60:focus-visible{--tw-ring-color:rgba(132,204,22,.6)}.focus-visible\:ring-lime-500\/65:focus-visible{--tw-ring-color:rgba(132,204,22,.65)}.focus-visible\:ring-lime-500\/70:focus-visible{--tw-ring-color:rgba(132,204,22,.7)}.focus-visible\:ring-lime-500\/75:focus-visible{--tw-ring-color:rgba(132,204,22,.75)}.focus-visible\:ring-lime-500\/80:focus-visible{--tw-ring-color:rgba(132,204,22,.8)}.focus-visible\:ring-lime-500\/85:focus-visible{--tw-ring-color:rgba(132,204,22,.85)}.focus-visible\:ring-lime-500\/90:focus-visible{--tw-ring-color:rgba(132,204,22,.9)}.focus-visible\:ring-lime-500\/95:focus-visible{--tw-ring-color:rgba(132,204,22,.95)}.focus-visible\:ring-lime-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(26 46 5/var(--tw-ring-opacity,1))}.focus-visible\:ring-lime-950\/0:focus-visible{--tw-ring-color:rgba(26,46,5,0)}.focus-visible\:ring-lime-950\/10:focus-visible{--tw-ring-color:rgba(26,46,5,.1)}.focus-visible\:ring-lime-950\/100:focus-visible{--tw-ring-color:#1a2e05}.focus-visible\:ring-lime-950\/15:focus-visible{--tw-ring-color:rgba(26,46,5,.15)}.focus-visible\:ring-lime-950\/20:focus-visible{--tw-ring-color:rgba(26,46,5,.2)}.focus-visible\:ring-lime-950\/25:focus-visible{--tw-ring-color:rgba(26,46,5,.25)}.focus-visible\:ring-lime-950\/30:focus-visible{--tw-ring-color:rgba(26,46,5,.3)}.focus-visible\:ring-lime-950\/35:focus-visible{--tw-ring-color:rgba(26,46,5,.35)}.focus-visible\:ring-lime-950\/40:focus-visible{--tw-ring-color:rgba(26,46,5,.4)}.focus-visible\:ring-lime-950\/45:focus-visible{--tw-ring-color:rgba(26,46,5,.45)}.focus-visible\:ring-lime-950\/5:focus-visible{--tw-ring-color:rgba(26,46,5,.05)}.focus-visible\:ring-lime-950\/50:focus-visible{--tw-ring-color:rgba(26,46,5,.5)}.focus-visible\:ring-lime-950\/55:focus-visible{--tw-ring-color:rgba(26,46,5,.55)}.focus-visible\:ring-lime-950\/60:focus-visible{--tw-ring-color:rgba(26,46,5,.6)}.focus-visible\:ring-lime-950\/65:focus-visible{--tw-ring-color:rgba(26,46,5,.65)}.focus-visible\:ring-lime-950\/70:focus-visible{--tw-ring-color:rgba(26,46,5,.7)}.focus-visible\:ring-lime-950\/75:focus-visible{--tw-ring-color:rgba(26,46,5,.75)}.focus-visible\:ring-lime-950\/80:focus-visible{--tw-ring-color:rgba(26,46,5,.8)}.focus-visible\:ring-lime-950\/85:focus-visible{--tw-ring-color:rgba(26,46,5,.85)}.focus-visible\:ring-lime-950\/90:focus-visible{--tw-ring-color:rgba(26,46,5,.9)}.focus-visible\:ring-lime-950\/95:focus-visible{--tw-ring-color:rgba(26,46,5,.95)}.focus-visible\:ring-neutral-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(245 245 245/var(--tw-ring-opacity,1))}.focus-visible\:ring-neutral-100\/0:focus-visible{--tw-ring-color:hsla(0,0%,96%,0)}.focus-visible\:ring-neutral-100\/10:focus-visible{--tw-ring-color:hsla(0,0%,96%,.1)}.focus-visible\:ring-neutral-100\/100:focus-visible{--tw-ring-color:#f5f5f5}.focus-visible\:ring-neutral-100\/15:focus-visible{--tw-ring-color:hsla(0,0%,96%,.15)}.focus-visible\:ring-neutral-100\/20:focus-visible{--tw-ring-color:hsla(0,0%,96%,.2)}.focus-visible\:ring-neutral-100\/25:focus-visible{--tw-ring-color:hsla(0,0%,96%,.25)}.focus-visible\:ring-neutral-100\/30:focus-visible{--tw-ring-color:hsla(0,0%,96%,.3)}.focus-visible\:ring-neutral-100\/35:focus-visible{--tw-ring-color:hsla(0,0%,96%,.35)}.focus-visible\:ring-neutral-100\/40:focus-visible{--tw-ring-color:hsla(0,0%,96%,.4)}.focus-visible\:ring-neutral-100\/45:focus-visible{--tw-ring-color:hsla(0,0%,96%,.45)}.focus-visible\:ring-neutral-100\/5:focus-visible{--tw-ring-color:hsla(0,0%,96%,.05)}.focus-visible\:ring-neutral-100\/50:focus-visible{--tw-ring-color:hsla(0,0%,96%,.5)}.focus-visible\:ring-neutral-100\/55:focus-visible{--tw-ring-color:hsla(0,0%,96%,.55)}.focus-visible\:ring-neutral-100\/60:focus-visible{--tw-ring-color:hsla(0,0%,96%,.6)}.focus-visible\:ring-neutral-100\/65:focus-visible{--tw-ring-color:hsla(0,0%,96%,.65)}.focus-visible\:ring-neutral-100\/70:focus-visible{--tw-ring-color:hsla(0,0%,96%,.7)}.focus-visible\:ring-neutral-100\/75:focus-visible{--tw-ring-color:hsla(0,0%,96%,.75)}.focus-visible\:ring-neutral-100\/80:focus-visible{--tw-ring-color:hsla(0,0%,96%,.8)}.focus-visible\:ring-neutral-100\/85:focus-visible{--tw-ring-color:hsla(0,0%,96%,.85)}.focus-visible\:ring-neutral-100\/90:focus-visible{--tw-ring-color:hsla(0,0%,96%,.9)}.focus-visible\:ring-neutral-100\/95:focus-visible{--tw-ring-color:hsla(0,0%,96%,.95)}.focus-visible\:ring-neutral-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(229 229 229/var(--tw-ring-opacity,1))}.focus-visible\:ring-neutral-200\/0:focus-visible{--tw-ring-color:hsla(0,0%,90%,0)}.focus-visible\:ring-neutral-200\/10:focus-visible{--tw-ring-color:hsla(0,0%,90%,.1)}.focus-visible\:ring-neutral-200\/100:focus-visible{--tw-ring-color:#e5e5e5}.focus-visible\:ring-neutral-200\/15:focus-visible{--tw-ring-color:hsla(0,0%,90%,.15)}.focus-visible\:ring-neutral-200\/20:focus-visible{--tw-ring-color:hsla(0,0%,90%,.2)}.focus-visible\:ring-neutral-200\/25:focus-visible{--tw-ring-color:hsla(0,0%,90%,.25)}.focus-visible\:ring-neutral-200\/30:focus-visible{--tw-ring-color:hsla(0,0%,90%,.3)}.focus-visible\:ring-neutral-200\/35:focus-visible{--tw-ring-color:hsla(0,0%,90%,.35)}.focus-visible\:ring-neutral-200\/40:focus-visible{--tw-ring-color:hsla(0,0%,90%,.4)}.focus-visible\:ring-neutral-200\/45:focus-visible{--tw-ring-color:hsla(0,0%,90%,.45)}.focus-visible\:ring-neutral-200\/5:focus-visible{--tw-ring-color:hsla(0,0%,90%,.05)}.focus-visible\:ring-neutral-200\/50:focus-visible{--tw-ring-color:hsla(0,0%,90%,.5)}.focus-visible\:ring-neutral-200\/55:focus-visible{--tw-ring-color:hsla(0,0%,90%,.55)}.focus-visible\:ring-neutral-200\/60:focus-visible{--tw-ring-color:hsla(0,0%,90%,.6)}.focus-visible\:ring-neutral-200\/65:focus-visible{--tw-ring-color:hsla(0,0%,90%,.65)}.focus-visible\:ring-neutral-200\/70:focus-visible{--tw-ring-color:hsla(0,0%,90%,.7)}.focus-visible\:ring-neutral-200\/75:focus-visible{--tw-ring-color:hsla(0,0%,90%,.75)}.focus-visible\:ring-neutral-200\/80:focus-visible{--tw-ring-color:hsla(0,0%,90%,.8)}.focus-visible\:ring-neutral-200\/85:focus-visible{--tw-ring-color:hsla(0,0%,90%,.85)}.focus-visible\:ring-neutral-200\/90:focus-visible{--tw-ring-color:hsla(0,0%,90%,.9)}.focus-visible\:ring-neutral-200\/95:focus-visible{--tw-ring-color:hsla(0,0%,90%,.95)}.focus-visible\:ring-neutral-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(163 163 163/var(--tw-ring-opacity,1))}.focus-visible\:ring-neutral-400\/0:focus-visible{--tw-ring-color:hsla(0,0%,64%,0)}.focus-visible\:ring-neutral-400\/10:focus-visible{--tw-ring-color:hsla(0,0%,64%,.1)}.focus-visible\:ring-neutral-400\/100:focus-visible{--tw-ring-color:#a3a3a3}.focus-visible\:ring-neutral-400\/15:focus-visible{--tw-ring-color:hsla(0,0%,64%,.15)}.focus-visible\:ring-neutral-400\/20:focus-visible{--tw-ring-color:hsla(0,0%,64%,.2)}.focus-visible\:ring-neutral-400\/25:focus-visible{--tw-ring-color:hsla(0,0%,64%,.25)}.focus-visible\:ring-neutral-400\/30:focus-visible{--tw-ring-color:hsla(0,0%,64%,.3)}.focus-visible\:ring-neutral-400\/35:focus-visible{--tw-ring-color:hsla(0,0%,64%,.35)}.focus-visible\:ring-neutral-400\/40:focus-visible{--tw-ring-color:hsla(0,0%,64%,.4)}.focus-visible\:ring-neutral-400\/45:focus-visible{--tw-ring-color:hsla(0,0%,64%,.45)}.focus-visible\:ring-neutral-400\/5:focus-visible{--tw-ring-color:hsla(0,0%,64%,.05)}.focus-visible\:ring-neutral-400\/50:focus-visible{--tw-ring-color:hsla(0,0%,64%,.5)}.focus-visible\:ring-neutral-400\/55:focus-visible{--tw-ring-color:hsla(0,0%,64%,.55)}.focus-visible\:ring-neutral-400\/60:focus-visible{--tw-ring-color:hsla(0,0%,64%,.6)}.focus-visible\:ring-neutral-400\/65:focus-visible{--tw-ring-color:hsla(0,0%,64%,.65)}.focus-visible\:ring-neutral-400\/70:focus-visible{--tw-ring-color:hsla(0,0%,64%,.7)}.focus-visible\:ring-neutral-400\/75:focus-visible{--tw-ring-color:hsla(0,0%,64%,.75)}.focus-visible\:ring-neutral-400\/80:focus-visible{--tw-ring-color:hsla(0,0%,64%,.8)}.focus-visible\:ring-neutral-400\/85:focus-visible{--tw-ring-color:hsla(0,0%,64%,.85)}.focus-visible\:ring-neutral-400\/90:focus-visible{--tw-ring-color:hsla(0,0%,64%,.9)}.focus-visible\:ring-neutral-400\/95:focus-visible{--tw-ring-color:hsla(0,0%,64%,.95)}.focus-visible\:ring-neutral-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(250 250 250/var(--tw-ring-opacity,1))}.focus-visible\:ring-neutral-50\/0:focus-visible{--tw-ring-color:hsla(0,0%,98%,0)}.focus-visible\:ring-neutral-50\/10:focus-visible{--tw-ring-color:hsla(0,0%,98%,.1)}.focus-visible\:ring-neutral-50\/100:focus-visible{--tw-ring-color:#fafafa}.focus-visible\:ring-neutral-50\/15:focus-visible{--tw-ring-color:hsla(0,0%,98%,.15)}.focus-visible\:ring-neutral-50\/20:focus-visible{--tw-ring-color:hsla(0,0%,98%,.2)}.focus-visible\:ring-neutral-50\/25:focus-visible{--tw-ring-color:hsla(0,0%,98%,.25)}.focus-visible\:ring-neutral-50\/30:focus-visible{--tw-ring-color:hsla(0,0%,98%,.3)}.focus-visible\:ring-neutral-50\/35:focus-visible{--tw-ring-color:hsla(0,0%,98%,.35)}.focus-visible\:ring-neutral-50\/40:focus-visible{--tw-ring-color:hsla(0,0%,98%,.4)}.focus-visible\:ring-neutral-50\/45:focus-visible{--tw-ring-color:hsla(0,0%,98%,.45)}.focus-visible\:ring-neutral-50\/5:focus-visible{--tw-ring-color:hsla(0,0%,98%,.05)}.focus-visible\:ring-neutral-50\/50:focus-visible{--tw-ring-color:hsla(0,0%,98%,.5)}.focus-visible\:ring-neutral-50\/55:focus-visible{--tw-ring-color:hsla(0,0%,98%,.55)}.focus-visible\:ring-neutral-50\/60:focus-visible{--tw-ring-color:hsla(0,0%,98%,.6)}.focus-visible\:ring-neutral-50\/65:focus-visible{--tw-ring-color:hsla(0,0%,98%,.65)}.focus-visible\:ring-neutral-50\/70:focus-visible{--tw-ring-color:hsla(0,0%,98%,.7)}.focus-visible\:ring-neutral-50\/75:focus-visible{--tw-ring-color:hsla(0,0%,98%,.75)}.focus-visible\:ring-neutral-50\/80:focus-visible{--tw-ring-color:hsla(0,0%,98%,.8)}.focus-visible\:ring-neutral-50\/85:focus-visible{--tw-ring-color:hsla(0,0%,98%,.85)}.focus-visible\:ring-neutral-50\/90:focus-visible{--tw-ring-color:hsla(0,0%,98%,.9)}.focus-visible\:ring-neutral-50\/95:focus-visible{--tw-ring-color:hsla(0,0%,98%,.95)}.focus-visible\:ring-neutral-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(115 115 115/var(--tw-ring-opacity,1))}.focus-visible\:ring-neutral-500\/0:focus-visible{--tw-ring-color:hsla(0,0%,45%,0)}.focus-visible\:ring-neutral-500\/10:focus-visible{--tw-ring-color:hsla(0,0%,45%,.1)}.focus-visible\:ring-neutral-500\/100:focus-visible{--tw-ring-color:#737373}.focus-visible\:ring-neutral-500\/15:focus-visible{--tw-ring-color:hsla(0,0%,45%,.15)}.focus-visible\:ring-neutral-500\/20:focus-visible{--tw-ring-color:hsla(0,0%,45%,.2)}.focus-visible\:ring-neutral-500\/25:focus-visible{--tw-ring-color:hsla(0,0%,45%,.25)}.focus-visible\:ring-neutral-500\/30:focus-visible{--tw-ring-color:hsla(0,0%,45%,.3)}.focus-visible\:ring-neutral-500\/35:focus-visible{--tw-ring-color:hsla(0,0%,45%,.35)}.focus-visible\:ring-neutral-500\/40:focus-visible{--tw-ring-color:hsla(0,0%,45%,.4)}.focus-visible\:ring-neutral-500\/45:focus-visible{--tw-ring-color:hsla(0,0%,45%,.45)}.focus-visible\:ring-neutral-500\/5:focus-visible{--tw-ring-color:hsla(0,0%,45%,.05)}.focus-visible\:ring-neutral-500\/50:focus-visible{--tw-ring-color:hsla(0,0%,45%,.5)}.focus-visible\:ring-neutral-500\/55:focus-visible{--tw-ring-color:hsla(0,0%,45%,.55)}.focus-visible\:ring-neutral-500\/60:focus-visible{--tw-ring-color:hsla(0,0%,45%,.6)}.focus-visible\:ring-neutral-500\/65:focus-visible{--tw-ring-color:hsla(0,0%,45%,.65)}.focus-visible\:ring-neutral-500\/70:focus-visible{--tw-ring-color:hsla(0,0%,45%,.7)}.focus-visible\:ring-neutral-500\/75:focus-visible{--tw-ring-color:hsla(0,0%,45%,.75)}.focus-visible\:ring-neutral-500\/80:focus-visible{--tw-ring-color:hsla(0,0%,45%,.8)}.focus-visible\:ring-neutral-500\/85:focus-visible{--tw-ring-color:hsla(0,0%,45%,.85)}.focus-visible\:ring-neutral-500\/90:focus-visible{--tw-ring-color:hsla(0,0%,45%,.9)}.focus-visible\:ring-neutral-500\/95:focus-visible{--tw-ring-color:hsla(0,0%,45%,.95)}.focus-visible\:ring-neutral-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(10 10 10/var(--tw-ring-opacity,1))}.focus-visible\:ring-neutral-950\/0:focus-visible{--tw-ring-color:hsla(0,0%,4%,0)}.focus-visible\:ring-neutral-950\/10:focus-visible{--tw-ring-color:hsla(0,0%,4%,.1)}.focus-visible\:ring-neutral-950\/100:focus-visible{--tw-ring-color:#0a0a0a}.focus-visible\:ring-neutral-950\/15:focus-visible{--tw-ring-color:hsla(0,0%,4%,.15)}.focus-visible\:ring-neutral-950\/20:focus-visible{--tw-ring-color:hsla(0,0%,4%,.2)}.focus-visible\:ring-neutral-950\/25:focus-visible{--tw-ring-color:hsla(0,0%,4%,.25)}.focus-visible\:ring-neutral-950\/30:focus-visible{--tw-ring-color:hsla(0,0%,4%,.3)}.focus-visible\:ring-neutral-950\/35:focus-visible{--tw-ring-color:hsla(0,0%,4%,.35)}.focus-visible\:ring-neutral-950\/40:focus-visible{--tw-ring-color:hsla(0,0%,4%,.4)}.focus-visible\:ring-neutral-950\/45:focus-visible{--tw-ring-color:hsla(0,0%,4%,.45)}.focus-visible\:ring-neutral-950\/5:focus-visible{--tw-ring-color:hsla(0,0%,4%,.05)}.focus-visible\:ring-neutral-950\/50:focus-visible{--tw-ring-color:hsla(0,0%,4%,.5)}.focus-visible\:ring-neutral-950\/55:focus-visible{--tw-ring-color:hsla(0,0%,4%,.55)}.focus-visible\:ring-neutral-950\/60:focus-visible{--tw-ring-color:hsla(0,0%,4%,.6)}.focus-visible\:ring-neutral-950\/65:focus-visible{--tw-ring-color:hsla(0,0%,4%,.65)}.focus-visible\:ring-neutral-950\/70:focus-visible{--tw-ring-color:hsla(0,0%,4%,.7)}.focus-visible\:ring-neutral-950\/75:focus-visible{--tw-ring-color:hsla(0,0%,4%,.75)}.focus-visible\:ring-neutral-950\/80:focus-visible{--tw-ring-color:hsla(0,0%,4%,.8)}.focus-visible\:ring-neutral-950\/85:focus-visible{--tw-ring-color:hsla(0,0%,4%,.85)}.focus-visible\:ring-neutral-950\/90:focus-visible{--tw-ring-color:hsla(0,0%,4%,.9)}.focus-visible\:ring-neutral-950\/95:focus-visible{--tw-ring-color:hsla(0,0%,4%,.95)}.focus-visible\:ring-orange-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(255 237 213/var(--tw-ring-opacity,1))}.focus-visible\:ring-orange-100\/0:focus-visible{--tw-ring-color:rgba(255,237,213,0)}.focus-visible\:ring-orange-100\/10:focus-visible{--tw-ring-color:rgba(255,237,213,.1)}.focus-visible\:ring-orange-100\/100:focus-visible{--tw-ring-color:#ffedd5}.focus-visible\:ring-orange-100\/15:focus-visible{--tw-ring-color:rgba(255,237,213,.15)}.focus-visible\:ring-orange-100\/20:focus-visible{--tw-ring-color:rgba(255,237,213,.2)}.focus-visible\:ring-orange-100\/25:focus-visible{--tw-ring-color:rgba(255,237,213,.25)}.focus-visible\:ring-orange-100\/30:focus-visible{--tw-ring-color:rgba(255,237,213,.3)}.focus-visible\:ring-orange-100\/35:focus-visible{--tw-ring-color:rgba(255,237,213,.35)}.focus-visible\:ring-orange-100\/40:focus-visible{--tw-ring-color:rgba(255,237,213,.4)}.focus-visible\:ring-orange-100\/45:focus-visible{--tw-ring-color:rgba(255,237,213,.45)}.focus-visible\:ring-orange-100\/5:focus-visible{--tw-ring-color:rgba(255,237,213,.05)}.focus-visible\:ring-orange-100\/50:focus-visible{--tw-ring-color:rgba(255,237,213,.5)}.focus-visible\:ring-orange-100\/55:focus-visible{--tw-ring-color:rgba(255,237,213,.55)}.focus-visible\:ring-orange-100\/60:focus-visible{--tw-ring-color:rgba(255,237,213,.6)}.focus-visible\:ring-orange-100\/65:focus-visible{--tw-ring-color:rgba(255,237,213,.65)}.focus-visible\:ring-orange-100\/70:focus-visible{--tw-ring-color:rgba(255,237,213,.7)}.focus-visible\:ring-orange-100\/75:focus-visible{--tw-ring-color:rgba(255,237,213,.75)}.focus-visible\:ring-orange-100\/80:focus-visible{--tw-ring-color:rgba(255,237,213,.8)}.focus-visible\:ring-orange-100\/85:focus-visible{--tw-ring-color:rgba(255,237,213,.85)}.focus-visible\:ring-orange-100\/90:focus-visible{--tw-ring-color:rgba(255,237,213,.9)}.focus-visible\:ring-orange-100\/95:focus-visible{--tw-ring-color:rgba(255,237,213,.95)}.focus-visible\:ring-orange-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(254 215 170/var(--tw-ring-opacity,1))}.focus-visible\:ring-orange-200\/0:focus-visible{--tw-ring-color:hsla(32,98%,83%,0)}.focus-visible\:ring-orange-200\/10:focus-visible{--tw-ring-color:hsla(32,98%,83%,.1)}.focus-visible\:ring-orange-200\/100:focus-visible{--tw-ring-color:#fed7aa}.focus-visible\:ring-orange-200\/15:focus-visible{--tw-ring-color:hsla(32,98%,83%,.15)}.focus-visible\:ring-orange-200\/20:focus-visible{--tw-ring-color:hsla(32,98%,83%,.2)}.focus-visible\:ring-orange-200\/25:focus-visible{--tw-ring-color:hsla(32,98%,83%,.25)}.focus-visible\:ring-orange-200\/30:focus-visible{--tw-ring-color:hsla(32,98%,83%,.3)}.focus-visible\:ring-orange-200\/35:focus-visible{--tw-ring-color:hsla(32,98%,83%,.35)}.focus-visible\:ring-orange-200\/40:focus-visible{--tw-ring-color:hsla(32,98%,83%,.4)}.focus-visible\:ring-orange-200\/45:focus-visible{--tw-ring-color:hsla(32,98%,83%,.45)}.focus-visible\:ring-orange-200\/5:focus-visible{--tw-ring-color:hsla(32,98%,83%,.05)}.focus-visible\:ring-orange-200\/50:focus-visible{--tw-ring-color:hsla(32,98%,83%,.5)}.focus-visible\:ring-orange-200\/55:focus-visible{--tw-ring-color:hsla(32,98%,83%,.55)}.focus-visible\:ring-orange-200\/60:focus-visible{--tw-ring-color:hsla(32,98%,83%,.6)}.focus-visible\:ring-orange-200\/65:focus-visible{--tw-ring-color:hsla(32,98%,83%,.65)}.focus-visible\:ring-orange-200\/70:focus-visible{--tw-ring-color:hsla(32,98%,83%,.7)}.focus-visible\:ring-orange-200\/75:focus-visible{--tw-ring-color:hsla(32,98%,83%,.75)}.focus-visible\:ring-orange-200\/80:focus-visible{--tw-ring-color:hsla(32,98%,83%,.8)}.focus-visible\:ring-orange-200\/85:focus-visible{--tw-ring-color:hsla(32,98%,83%,.85)}.focus-visible\:ring-orange-200\/90:focus-visible{--tw-ring-color:hsla(32,98%,83%,.9)}.focus-visible\:ring-orange-200\/95:focus-visible{--tw-ring-color:hsla(32,98%,83%,.95)}.focus-visible\:ring-orange-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(251 146 60/var(--tw-ring-opacity,1))}.focus-visible\:ring-orange-400\/0:focus-visible{--tw-ring-color:rgba(251,146,60,0)}.focus-visible\:ring-orange-400\/10:focus-visible{--tw-ring-color:rgba(251,146,60,.1)}.focus-visible\:ring-orange-400\/100:focus-visible{--tw-ring-color:#fb923c}.focus-visible\:ring-orange-400\/15:focus-visible{--tw-ring-color:rgba(251,146,60,.15)}.focus-visible\:ring-orange-400\/20:focus-visible{--tw-ring-color:rgba(251,146,60,.2)}.focus-visible\:ring-orange-400\/25:focus-visible{--tw-ring-color:rgba(251,146,60,.25)}.focus-visible\:ring-orange-400\/30:focus-visible{--tw-ring-color:rgba(251,146,60,.3)}.focus-visible\:ring-orange-400\/35:focus-visible{--tw-ring-color:rgba(251,146,60,.35)}.focus-visible\:ring-orange-400\/40:focus-visible{--tw-ring-color:rgba(251,146,60,.4)}.focus-visible\:ring-orange-400\/45:focus-visible{--tw-ring-color:rgba(251,146,60,.45)}.focus-visible\:ring-orange-400\/5:focus-visible{--tw-ring-color:rgba(251,146,60,.05)}.focus-visible\:ring-orange-400\/50:focus-visible{--tw-ring-color:rgba(251,146,60,.5)}.focus-visible\:ring-orange-400\/55:focus-visible{--tw-ring-color:rgba(251,146,60,.55)}.focus-visible\:ring-orange-400\/60:focus-visible{--tw-ring-color:rgba(251,146,60,.6)}.focus-visible\:ring-orange-400\/65:focus-visible{--tw-ring-color:rgba(251,146,60,.65)}.focus-visible\:ring-orange-400\/70:focus-visible{--tw-ring-color:rgba(251,146,60,.7)}.focus-visible\:ring-orange-400\/75:focus-visible{--tw-ring-color:rgba(251,146,60,.75)}.focus-visible\:ring-orange-400\/80:focus-visible{--tw-ring-color:rgba(251,146,60,.8)}.focus-visible\:ring-orange-400\/85:focus-visible{--tw-ring-color:rgba(251,146,60,.85)}.focus-visible\:ring-orange-400\/90:focus-visible{--tw-ring-color:rgba(251,146,60,.9)}.focus-visible\:ring-orange-400\/95:focus-visible{--tw-ring-color:rgba(251,146,60,.95)}.focus-visible\:ring-orange-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(255 247 237/var(--tw-ring-opacity,1))}.focus-visible\:ring-orange-50\/0:focus-visible{--tw-ring-color:rgba(255,247,237,0)}.focus-visible\:ring-orange-50\/10:focus-visible{--tw-ring-color:rgba(255,247,237,.1)}.focus-visible\:ring-orange-50\/100:focus-visible{--tw-ring-color:#fff7ed}.focus-visible\:ring-orange-50\/15:focus-visible{--tw-ring-color:rgba(255,247,237,.15)}.focus-visible\:ring-orange-50\/20:focus-visible{--tw-ring-color:rgba(255,247,237,.2)}.focus-visible\:ring-orange-50\/25:focus-visible{--tw-ring-color:rgba(255,247,237,.25)}.focus-visible\:ring-orange-50\/30:focus-visible{--tw-ring-color:rgba(255,247,237,.3)}.focus-visible\:ring-orange-50\/35:focus-visible{--tw-ring-color:rgba(255,247,237,.35)}.focus-visible\:ring-orange-50\/40:focus-visible{--tw-ring-color:rgba(255,247,237,.4)}.focus-visible\:ring-orange-50\/45:focus-visible{--tw-ring-color:rgba(255,247,237,.45)}.focus-visible\:ring-orange-50\/5:focus-visible{--tw-ring-color:rgba(255,247,237,.05)}.focus-visible\:ring-orange-50\/50:focus-visible{--tw-ring-color:rgba(255,247,237,.5)}.focus-visible\:ring-orange-50\/55:focus-visible{--tw-ring-color:rgba(255,247,237,.55)}.focus-visible\:ring-orange-50\/60:focus-visible{--tw-ring-color:rgba(255,247,237,.6)}.focus-visible\:ring-orange-50\/65:focus-visible{--tw-ring-color:rgba(255,247,237,.65)}.focus-visible\:ring-orange-50\/70:focus-visible{--tw-ring-color:rgba(255,247,237,.7)}.focus-visible\:ring-orange-50\/75:focus-visible{--tw-ring-color:rgba(255,247,237,.75)}.focus-visible\:ring-orange-50\/80:focus-visible{--tw-ring-color:rgba(255,247,237,.8)}.focus-visible\:ring-orange-50\/85:focus-visible{--tw-ring-color:rgba(255,247,237,.85)}.focus-visible\:ring-orange-50\/90:focus-visible{--tw-ring-color:rgba(255,247,237,.9)}.focus-visible\:ring-orange-50\/95:focus-visible{--tw-ring-color:rgba(255,247,237,.95)}.focus-visible\:ring-orange-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(249 115 22/var(--tw-ring-opacity,1))}.focus-visible\:ring-orange-500\/0:focus-visible{--tw-ring-color:rgba(249,115,22,0)}.focus-visible\:ring-orange-500\/10:focus-visible{--tw-ring-color:rgba(249,115,22,.1)}.focus-visible\:ring-orange-500\/100:focus-visible{--tw-ring-color:#f97316}.focus-visible\:ring-orange-500\/15:focus-visible{--tw-ring-color:rgba(249,115,22,.15)}.focus-visible\:ring-orange-500\/20:focus-visible{--tw-ring-color:rgba(249,115,22,.2)}.focus-visible\:ring-orange-500\/25:focus-visible{--tw-ring-color:rgba(249,115,22,.25)}.focus-visible\:ring-orange-500\/30:focus-visible{--tw-ring-color:rgba(249,115,22,.3)}.focus-visible\:ring-orange-500\/35:focus-visible{--tw-ring-color:rgba(249,115,22,.35)}.focus-visible\:ring-orange-500\/40:focus-visible{--tw-ring-color:rgba(249,115,22,.4)}.focus-visible\:ring-orange-500\/45:focus-visible{--tw-ring-color:rgba(249,115,22,.45)}.focus-visible\:ring-orange-500\/5:focus-visible{--tw-ring-color:rgba(249,115,22,.05)}.focus-visible\:ring-orange-500\/50:focus-visible{--tw-ring-color:rgba(249,115,22,.5)}.focus-visible\:ring-orange-500\/55:focus-visible{--tw-ring-color:rgba(249,115,22,.55)}.focus-visible\:ring-orange-500\/60:focus-visible{--tw-ring-color:rgba(249,115,22,.6)}.focus-visible\:ring-orange-500\/65:focus-visible{--tw-ring-color:rgba(249,115,22,.65)}.focus-visible\:ring-orange-500\/70:focus-visible{--tw-ring-color:rgba(249,115,22,.7)}.focus-visible\:ring-orange-500\/75:focus-visible{--tw-ring-color:rgba(249,115,22,.75)}.focus-visible\:ring-orange-500\/80:focus-visible{--tw-ring-color:rgba(249,115,22,.8)}.focus-visible\:ring-orange-500\/85:focus-visible{--tw-ring-color:rgba(249,115,22,.85)}.focus-visible\:ring-orange-500\/90:focus-visible{--tw-ring-color:rgba(249,115,22,.9)}.focus-visible\:ring-orange-500\/95:focus-visible{--tw-ring-color:rgba(249,115,22,.95)}.focus-visible\:ring-orange-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(67 20 7/var(--tw-ring-opacity,1))}.focus-visible\:ring-orange-950\/0:focus-visible{--tw-ring-color:rgba(67,20,7,0)}.focus-visible\:ring-orange-950\/10:focus-visible{--tw-ring-color:rgba(67,20,7,.1)}.focus-visible\:ring-orange-950\/100:focus-visible{--tw-ring-color:#431407}.focus-visible\:ring-orange-950\/15:focus-visible{--tw-ring-color:rgba(67,20,7,.15)}.focus-visible\:ring-orange-950\/20:focus-visible{--tw-ring-color:rgba(67,20,7,.2)}.focus-visible\:ring-orange-950\/25:focus-visible{--tw-ring-color:rgba(67,20,7,.25)}.focus-visible\:ring-orange-950\/30:focus-visible{--tw-ring-color:rgba(67,20,7,.3)}.focus-visible\:ring-orange-950\/35:focus-visible{--tw-ring-color:rgba(67,20,7,.35)}.focus-visible\:ring-orange-950\/40:focus-visible{--tw-ring-color:rgba(67,20,7,.4)}.focus-visible\:ring-orange-950\/45:focus-visible{--tw-ring-color:rgba(67,20,7,.45)}.focus-visible\:ring-orange-950\/5:focus-visible{--tw-ring-color:rgba(67,20,7,.05)}.focus-visible\:ring-orange-950\/50:focus-visible{--tw-ring-color:rgba(67,20,7,.5)}.focus-visible\:ring-orange-950\/55:focus-visible{--tw-ring-color:rgba(67,20,7,.55)}.focus-visible\:ring-orange-950\/60:focus-visible{--tw-ring-color:rgba(67,20,7,.6)}.focus-visible\:ring-orange-950\/65:focus-visible{--tw-ring-color:rgba(67,20,7,.65)}.focus-visible\:ring-orange-950\/70:focus-visible{--tw-ring-color:rgba(67,20,7,.7)}.focus-visible\:ring-orange-950\/75:focus-visible{--tw-ring-color:rgba(67,20,7,.75)}.focus-visible\:ring-orange-950\/80:focus-visible{--tw-ring-color:rgba(67,20,7,.8)}.focus-visible\:ring-orange-950\/85:focus-visible{--tw-ring-color:rgba(67,20,7,.85)}.focus-visible\:ring-orange-950\/90:focus-visible{--tw-ring-color:rgba(67,20,7,.9)}.focus-visible\:ring-orange-950\/95:focus-visible{--tw-ring-color:rgba(67,20,7,.95)}.focus-visible\:ring-pink-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(252 231 243/var(--tw-ring-opacity,1))}.focus-visible\:ring-pink-100\/0:focus-visible{--tw-ring-color:rgba(252,231,243,0)}.focus-visible\:ring-pink-100\/10:focus-visible{--tw-ring-color:rgba(252,231,243,.1)}.focus-visible\:ring-pink-100\/100:focus-visible{--tw-ring-color:#fce7f3}.focus-visible\:ring-pink-100\/15:focus-visible{--tw-ring-color:rgba(252,231,243,.15)}.focus-visible\:ring-pink-100\/20:focus-visible{--tw-ring-color:rgba(252,231,243,.2)}.focus-visible\:ring-pink-100\/25:focus-visible{--tw-ring-color:rgba(252,231,243,.25)}.focus-visible\:ring-pink-100\/30:focus-visible{--tw-ring-color:rgba(252,231,243,.3)}.focus-visible\:ring-pink-100\/35:focus-visible{--tw-ring-color:rgba(252,231,243,.35)}.focus-visible\:ring-pink-100\/40:focus-visible{--tw-ring-color:rgba(252,231,243,.4)}.focus-visible\:ring-pink-100\/45:focus-visible{--tw-ring-color:rgba(252,231,243,.45)}.focus-visible\:ring-pink-100\/5:focus-visible{--tw-ring-color:rgba(252,231,243,.05)}.focus-visible\:ring-pink-100\/50:focus-visible{--tw-ring-color:rgba(252,231,243,.5)}.focus-visible\:ring-pink-100\/55:focus-visible{--tw-ring-color:rgba(252,231,243,.55)}.focus-visible\:ring-pink-100\/60:focus-visible{--tw-ring-color:rgba(252,231,243,.6)}.focus-visible\:ring-pink-100\/65:focus-visible{--tw-ring-color:rgba(252,231,243,.65)}.focus-visible\:ring-pink-100\/70:focus-visible{--tw-ring-color:rgba(252,231,243,.7)}.focus-visible\:ring-pink-100\/75:focus-visible{--tw-ring-color:rgba(252,231,243,.75)}.focus-visible\:ring-pink-100\/80:focus-visible{--tw-ring-color:rgba(252,231,243,.8)}.focus-visible\:ring-pink-100\/85:focus-visible{--tw-ring-color:rgba(252,231,243,.85)}.focus-visible\:ring-pink-100\/90:focus-visible{--tw-ring-color:rgba(252,231,243,.9)}.focus-visible\:ring-pink-100\/95:focus-visible{--tw-ring-color:rgba(252,231,243,.95)}.focus-visible\:ring-pink-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(251 207 232/var(--tw-ring-opacity,1))}.focus-visible\:ring-pink-200\/0:focus-visible{--tw-ring-color:rgba(251,207,232,0)}.focus-visible\:ring-pink-200\/10:focus-visible{--tw-ring-color:rgba(251,207,232,.1)}.focus-visible\:ring-pink-200\/100:focus-visible{--tw-ring-color:#fbcfe8}.focus-visible\:ring-pink-200\/15:focus-visible{--tw-ring-color:rgba(251,207,232,.15)}.focus-visible\:ring-pink-200\/20:focus-visible{--tw-ring-color:rgba(251,207,232,.2)}.focus-visible\:ring-pink-200\/25:focus-visible{--tw-ring-color:rgba(251,207,232,.25)}.focus-visible\:ring-pink-200\/30:focus-visible{--tw-ring-color:rgba(251,207,232,.3)}.focus-visible\:ring-pink-200\/35:focus-visible{--tw-ring-color:rgba(251,207,232,.35)}.focus-visible\:ring-pink-200\/40:focus-visible{--tw-ring-color:rgba(251,207,232,.4)}.focus-visible\:ring-pink-200\/45:focus-visible{--tw-ring-color:rgba(251,207,232,.45)}.focus-visible\:ring-pink-200\/5:focus-visible{--tw-ring-color:rgba(251,207,232,.05)}.focus-visible\:ring-pink-200\/50:focus-visible{--tw-ring-color:rgba(251,207,232,.5)}.focus-visible\:ring-pink-200\/55:focus-visible{--tw-ring-color:rgba(251,207,232,.55)}.focus-visible\:ring-pink-200\/60:focus-visible{--tw-ring-color:rgba(251,207,232,.6)}.focus-visible\:ring-pink-200\/65:focus-visible{--tw-ring-color:rgba(251,207,232,.65)}.focus-visible\:ring-pink-200\/70:focus-visible{--tw-ring-color:rgba(251,207,232,.7)}.focus-visible\:ring-pink-200\/75:focus-visible{--tw-ring-color:rgba(251,207,232,.75)}.focus-visible\:ring-pink-200\/80:focus-visible{--tw-ring-color:rgba(251,207,232,.8)}.focus-visible\:ring-pink-200\/85:focus-visible{--tw-ring-color:rgba(251,207,232,.85)}.focus-visible\:ring-pink-200\/90:focus-visible{--tw-ring-color:rgba(251,207,232,.9)}.focus-visible\:ring-pink-200\/95:focus-visible{--tw-ring-color:rgba(251,207,232,.95)}.focus-visible\:ring-pink-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(244 114 182/var(--tw-ring-opacity,1))}.focus-visible\:ring-pink-400\/0:focus-visible{--tw-ring-color:rgba(244,114,182,0)}.focus-visible\:ring-pink-400\/10:focus-visible{--tw-ring-color:rgba(244,114,182,.1)}.focus-visible\:ring-pink-400\/100:focus-visible{--tw-ring-color:#f472b6}.focus-visible\:ring-pink-400\/15:focus-visible{--tw-ring-color:rgba(244,114,182,.15)}.focus-visible\:ring-pink-400\/20:focus-visible{--tw-ring-color:rgba(244,114,182,.2)}.focus-visible\:ring-pink-400\/25:focus-visible{--tw-ring-color:rgba(244,114,182,.25)}.focus-visible\:ring-pink-400\/30:focus-visible{--tw-ring-color:rgba(244,114,182,.3)}.focus-visible\:ring-pink-400\/35:focus-visible{--tw-ring-color:rgba(244,114,182,.35)}.focus-visible\:ring-pink-400\/40:focus-visible{--tw-ring-color:rgba(244,114,182,.4)}.focus-visible\:ring-pink-400\/45:focus-visible{--tw-ring-color:rgba(244,114,182,.45)}.focus-visible\:ring-pink-400\/5:focus-visible{--tw-ring-color:rgba(244,114,182,.05)}.focus-visible\:ring-pink-400\/50:focus-visible{--tw-ring-color:rgba(244,114,182,.5)}.focus-visible\:ring-pink-400\/55:focus-visible{--tw-ring-color:rgba(244,114,182,.55)}.focus-visible\:ring-pink-400\/60:focus-visible{--tw-ring-color:rgba(244,114,182,.6)}.focus-visible\:ring-pink-400\/65:focus-visible{--tw-ring-color:rgba(244,114,182,.65)}.focus-visible\:ring-pink-400\/70:focus-visible{--tw-ring-color:rgba(244,114,182,.7)}.focus-visible\:ring-pink-400\/75:focus-visible{--tw-ring-color:rgba(244,114,182,.75)}.focus-visible\:ring-pink-400\/80:focus-visible{--tw-ring-color:rgba(244,114,182,.8)}.focus-visible\:ring-pink-400\/85:focus-visible{--tw-ring-color:rgba(244,114,182,.85)}.focus-visible\:ring-pink-400\/90:focus-visible{--tw-ring-color:rgba(244,114,182,.9)}.focus-visible\:ring-pink-400\/95:focus-visible{--tw-ring-color:rgba(244,114,182,.95)}.focus-visible\:ring-pink-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(253 242 248/var(--tw-ring-opacity,1))}.focus-visible\:ring-pink-50\/0:focus-visible{--tw-ring-color:rgba(253,242,248,0)}.focus-visible\:ring-pink-50\/10:focus-visible{--tw-ring-color:rgba(253,242,248,.1)}.focus-visible\:ring-pink-50\/100:focus-visible{--tw-ring-color:#fdf2f8}.focus-visible\:ring-pink-50\/15:focus-visible{--tw-ring-color:rgba(253,242,248,.15)}.focus-visible\:ring-pink-50\/20:focus-visible{--tw-ring-color:rgba(253,242,248,.2)}.focus-visible\:ring-pink-50\/25:focus-visible{--tw-ring-color:rgba(253,242,248,.25)}.focus-visible\:ring-pink-50\/30:focus-visible{--tw-ring-color:rgba(253,242,248,.3)}.focus-visible\:ring-pink-50\/35:focus-visible{--tw-ring-color:rgba(253,242,248,.35)}.focus-visible\:ring-pink-50\/40:focus-visible{--tw-ring-color:rgba(253,242,248,.4)}.focus-visible\:ring-pink-50\/45:focus-visible{--tw-ring-color:rgba(253,242,248,.45)}.focus-visible\:ring-pink-50\/5:focus-visible{--tw-ring-color:rgba(253,242,248,.05)}.focus-visible\:ring-pink-50\/50:focus-visible{--tw-ring-color:rgba(253,242,248,.5)}.focus-visible\:ring-pink-50\/55:focus-visible{--tw-ring-color:rgba(253,242,248,.55)}.focus-visible\:ring-pink-50\/60:focus-visible{--tw-ring-color:rgba(253,242,248,.6)}.focus-visible\:ring-pink-50\/65:focus-visible{--tw-ring-color:rgba(253,242,248,.65)}.focus-visible\:ring-pink-50\/70:focus-visible{--tw-ring-color:rgba(253,242,248,.7)}.focus-visible\:ring-pink-50\/75:focus-visible{--tw-ring-color:rgba(253,242,248,.75)}.focus-visible\:ring-pink-50\/80:focus-visible{--tw-ring-color:rgba(253,242,248,.8)}.focus-visible\:ring-pink-50\/85:focus-visible{--tw-ring-color:rgba(253,242,248,.85)}.focus-visible\:ring-pink-50\/90:focus-visible{--tw-ring-color:rgba(253,242,248,.9)}.focus-visible\:ring-pink-50\/95:focus-visible{--tw-ring-color:rgba(253,242,248,.95)}.focus-visible\:ring-pink-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(236 72 153/var(--tw-ring-opacity,1))}.focus-visible\:ring-pink-500\/0:focus-visible{--tw-ring-color:rgba(236,72,153,0)}.focus-visible\:ring-pink-500\/10:focus-visible{--tw-ring-color:rgba(236,72,153,.1)}.focus-visible\:ring-pink-500\/100:focus-visible{--tw-ring-color:#ec4899}.focus-visible\:ring-pink-500\/15:focus-visible{--tw-ring-color:rgba(236,72,153,.15)}.focus-visible\:ring-pink-500\/20:focus-visible{--tw-ring-color:rgba(236,72,153,.2)}.focus-visible\:ring-pink-500\/25:focus-visible{--tw-ring-color:rgba(236,72,153,.25)}.focus-visible\:ring-pink-500\/30:focus-visible{--tw-ring-color:rgba(236,72,153,.3)}.focus-visible\:ring-pink-500\/35:focus-visible{--tw-ring-color:rgba(236,72,153,.35)}.focus-visible\:ring-pink-500\/40:focus-visible{--tw-ring-color:rgba(236,72,153,.4)}.focus-visible\:ring-pink-500\/45:focus-visible{--tw-ring-color:rgba(236,72,153,.45)}.focus-visible\:ring-pink-500\/5:focus-visible{--tw-ring-color:rgba(236,72,153,.05)}.focus-visible\:ring-pink-500\/50:focus-visible{--tw-ring-color:rgba(236,72,153,.5)}.focus-visible\:ring-pink-500\/55:focus-visible{--tw-ring-color:rgba(236,72,153,.55)}.focus-visible\:ring-pink-500\/60:focus-visible{--tw-ring-color:rgba(236,72,153,.6)}.focus-visible\:ring-pink-500\/65:focus-visible{--tw-ring-color:rgba(236,72,153,.65)}.focus-visible\:ring-pink-500\/70:focus-visible{--tw-ring-color:rgba(236,72,153,.7)}.focus-visible\:ring-pink-500\/75:focus-visible{--tw-ring-color:rgba(236,72,153,.75)}.focus-visible\:ring-pink-500\/80:focus-visible{--tw-ring-color:rgba(236,72,153,.8)}.focus-visible\:ring-pink-500\/85:focus-visible{--tw-ring-color:rgba(236,72,153,.85)}.focus-visible\:ring-pink-500\/90:focus-visible{--tw-ring-color:rgba(236,72,153,.9)}.focus-visible\:ring-pink-500\/95:focus-visible{--tw-ring-color:rgba(236,72,153,.95)}.focus-visible\:ring-pink-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(80 7 36/var(--tw-ring-opacity,1))}.focus-visible\:ring-pink-950\/0:focus-visible{--tw-ring-color:rgba(80,7,36,0)}.focus-visible\:ring-pink-950\/10:focus-visible{--tw-ring-color:rgba(80,7,36,.1)}.focus-visible\:ring-pink-950\/100:focus-visible{--tw-ring-color:#500724}.focus-visible\:ring-pink-950\/15:focus-visible{--tw-ring-color:rgba(80,7,36,.15)}.focus-visible\:ring-pink-950\/20:focus-visible{--tw-ring-color:rgba(80,7,36,.2)}.focus-visible\:ring-pink-950\/25:focus-visible{--tw-ring-color:rgba(80,7,36,.25)}.focus-visible\:ring-pink-950\/30:focus-visible{--tw-ring-color:rgba(80,7,36,.3)}.focus-visible\:ring-pink-950\/35:focus-visible{--tw-ring-color:rgba(80,7,36,.35)}.focus-visible\:ring-pink-950\/40:focus-visible{--tw-ring-color:rgba(80,7,36,.4)}.focus-visible\:ring-pink-950\/45:focus-visible{--tw-ring-color:rgba(80,7,36,.45)}.focus-visible\:ring-pink-950\/5:focus-visible{--tw-ring-color:rgba(80,7,36,.05)}.focus-visible\:ring-pink-950\/50:focus-visible{--tw-ring-color:rgba(80,7,36,.5)}.focus-visible\:ring-pink-950\/55:focus-visible{--tw-ring-color:rgba(80,7,36,.55)}.focus-visible\:ring-pink-950\/60:focus-visible{--tw-ring-color:rgba(80,7,36,.6)}.focus-visible\:ring-pink-950\/65:focus-visible{--tw-ring-color:rgba(80,7,36,.65)}.focus-visible\:ring-pink-950\/70:focus-visible{--tw-ring-color:rgba(80,7,36,.7)}.focus-visible\:ring-pink-950\/75:focus-visible{--tw-ring-color:rgba(80,7,36,.75)}.focus-visible\:ring-pink-950\/80:focus-visible{--tw-ring-color:rgba(80,7,36,.8)}.focus-visible\:ring-pink-950\/85:focus-visible{--tw-ring-color:rgba(80,7,36,.85)}.focus-visible\:ring-pink-950\/90:focus-visible{--tw-ring-color:rgba(80,7,36,.9)}.focus-visible\:ring-pink-950\/95:focus-visible{--tw-ring-color:rgba(80,7,36,.95)}.focus-visible\:ring-purple-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(243 232 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-purple-100\/0:focus-visible{--tw-ring-color:rgba(243,232,255,0)}.focus-visible\:ring-purple-100\/10:focus-visible{--tw-ring-color:rgba(243,232,255,.1)}.focus-visible\:ring-purple-100\/100:focus-visible{--tw-ring-color:#f3e8ff}.focus-visible\:ring-purple-100\/15:focus-visible{--tw-ring-color:rgba(243,232,255,.15)}.focus-visible\:ring-purple-100\/20:focus-visible{--tw-ring-color:rgba(243,232,255,.2)}.focus-visible\:ring-purple-100\/25:focus-visible{--tw-ring-color:rgba(243,232,255,.25)}.focus-visible\:ring-purple-100\/30:focus-visible{--tw-ring-color:rgba(243,232,255,.3)}.focus-visible\:ring-purple-100\/35:focus-visible{--tw-ring-color:rgba(243,232,255,.35)}.focus-visible\:ring-purple-100\/40:focus-visible{--tw-ring-color:rgba(243,232,255,.4)}.focus-visible\:ring-purple-100\/45:focus-visible{--tw-ring-color:rgba(243,232,255,.45)}.focus-visible\:ring-purple-100\/5:focus-visible{--tw-ring-color:rgba(243,232,255,.05)}.focus-visible\:ring-purple-100\/50:focus-visible{--tw-ring-color:rgba(243,232,255,.5)}.focus-visible\:ring-purple-100\/55:focus-visible{--tw-ring-color:rgba(243,232,255,.55)}.focus-visible\:ring-purple-100\/60:focus-visible{--tw-ring-color:rgba(243,232,255,.6)}.focus-visible\:ring-purple-100\/65:focus-visible{--tw-ring-color:rgba(243,232,255,.65)}.focus-visible\:ring-purple-100\/70:focus-visible{--tw-ring-color:rgba(243,232,255,.7)}.focus-visible\:ring-purple-100\/75:focus-visible{--tw-ring-color:rgba(243,232,255,.75)}.focus-visible\:ring-purple-100\/80:focus-visible{--tw-ring-color:rgba(243,232,255,.8)}.focus-visible\:ring-purple-100\/85:focus-visible{--tw-ring-color:rgba(243,232,255,.85)}.focus-visible\:ring-purple-100\/90:focus-visible{--tw-ring-color:rgba(243,232,255,.9)}.focus-visible\:ring-purple-100\/95:focus-visible{--tw-ring-color:rgba(243,232,255,.95)}.focus-visible\:ring-purple-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(233 213 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-purple-200\/0:focus-visible{--tw-ring-color:rgba(233,213,255,0)}.focus-visible\:ring-purple-200\/10:focus-visible{--tw-ring-color:rgba(233,213,255,.1)}.focus-visible\:ring-purple-200\/100:focus-visible{--tw-ring-color:#e9d5ff}.focus-visible\:ring-purple-200\/15:focus-visible{--tw-ring-color:rgba(233,213,255,.15)}.focus-visible\:ring-purple-200\/20:focus-visible{--tw-ring-color:rgba(233,213,255,.2)}.focus-visible\:ring-purple-200\/25:focus-visible{--tw-ring-color:rgba(233,213,255,.25)}.focus-visible\:ring-purple-200\/30:focus-visible{--tw-ring-color:rgba(233,213,255,.3)}.focus-visible\:ring-purple-200\/35:focus-visible{--tw-ring-color:rgba(233,213,255,.35)}.focus-visible\:ring-purple-200\/40:focus-visible{--tw-ring-color:rgba(233,213,255,.4)}.focus-visible\:ring-purple-200\/45:focus-visible{--tw-ring-color:rgba(233,213,255,.45)}.focus-visible\:ring-purple-200\/5:focus-visible{--tw-ring-color:rgba(233,213,255,.05)}.focus-visible\:ring-purple-200\/50:focus-visible{--tw-ring-color:rgba(233,213,255,.5)}.focus-visible\:ring-purple-200\/55:focus-visible{--tw-ring-color:rgba(233,213,255,.55)}.focus-visible\:ring-purple-200\/60:focus-visible{--tw-ring-color:rgba(233,213,255,.6)}.focus-visible\:ring-purple-200\/65:focus-visible{--tw-ring-color:rgba(233,213,255,.65)}.focus-visible\:ring-purple-200\/70:focus-visible{--tw-ring-color:rgba(233,213,255,.7)}.focus-visible\:ring-purple-200\/75:focus-visible{--tw-ring-color:rgba(233,213,255,.75)}.focus-visible\:ring-purple-200\/80:focus-visible{--tw-ring-color:rgba(233,213,255,.8)}.focus-visible\:ring-purple-200\/85:focus-visible{--tw-ring-color:rgba(233,213,255,.85)}.focus-visible\:ring-purple-200\/90:focus-visible{--tw-ring-color:rgba(233,213,255,.9)}.focus-visible\:ring-purple-200\/95:focus-visible{--tw-ring-color:rgba(233,213,255,.95)}.focus-visible\:ring-purple-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(192 132 252/var(--tw-ring-opacity,1))}.focus-visible\:ring-purple-400\/0:focus-visible{--tw-ring-color:rgba(192,132,252,0)}.focus-visible\:ring-purple-400\/10:focus-visible{--tw-ring-color:rgba(192,132,252,.1)}.focus-visible\:ring-purple-400\/100:focus-visible{--tw-ring-color:#c084fc}.focus-visible\:ring-purple-400\/15:focus-visible{--tw-ring-color:rgba(192,132,252,.15)}.focus-visible\:ring-purple-400\/20:focus-visible{--tw-ring-color:rgba(192,132,252,.2)}.focus-visible\:ring-purple-400\/25:focus-visible{--tw-ring-color:rgba(192,132,252,.25)}.focus-visible\:ring-purple-400\/30:focus-visible{--tw-ring-color:rgba(192,132,252,.3)}.focus-visible\:ring-purple-400\/35:focus-visible{--tw-ring-color:rgba(192,132,252,.35)}.focus-visible\:ring-purple-400\/40:focus-visible{--tw-ring-color:rgba(192,132,252,.4)}.focus-visible\:ring-purple-400\/45:focus-visible{--tw-ring-color:rgba(192,132,252,.45)}.focus-visible\:ring-purple-400\/5:focus-visible{--tw-ring-color:rgba(192,132,252,.05)}.focus-visible\:ring-purple-400\/50:focus-visible{--tw-ring-color:rgba(192,132,252,.5)}.focus-visible\:ring-purple-400\/55:focus-visible{--tw-ring-color:rgba(192,132,252,.55)}.focus-visible\:ring-purple-400\/60:focus-visible{--tw-ring-color:rgba(192,132,252,.6)}.focus-visible\:ring-purple-400\/65:focus-visible{--tw-ring-color:rgba(192,132,252,.65)}.focus-visible\:ring-purple-400\/70:focus-visible{--tw-ring-color:rgba(192,132,252,.7)}.focus-visible\:ring-purple-400\/75:focus-visible{--tw-ring-color:rgba(192,132,252,.75)}.focus-visible\:ring-purple-400\/80:focus-visible{--tw-ring-color:rgba(192,132,252,.8)}.focus-visible\:ring-purple-400\/85:focus-visible{--tw-ring-color:rgba(192,132,252,.85)}.focus-visible\:ring-purple-400\/90:focus-visible{--tw-ring-color:rgba(192,132,252,.9)}.focus-visible\:ring-purple-400\/95:focus-visible{--tw-ring-color:rgba(192,132,252,.95)}.focus-visible\:ring-purple-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(250 245 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-purple-50\/0:focus-visible{--tw-ring-color:rgba(250,245,255,0)}.focus-visible\:ring-purple-50\/10:focus-visible{--tw-ring-color:rgba(250,245,255,.1)}.focus-visible\:ring-purple-50\/100:focus-visible{--tw-ring-color:#faf5ff}.focus-visible\:ring-purple-50\/15:focus-visible{--tw-ring-color:rgba(250,245,255,.15)}.focus-visible\:ring-purple-50\/20:focus-visible{--tw-ring-color:rgba(250,245,255,.2)}.focus-visible\:ring-purple-50\/25:focus-visible{--tw-ring-color:rgba(250,245,255,.25)}.focus-visible\:ring-purple-50\/30:focus-visible{--tw-ring-color:rgba(250,245,255,.3)}.focus-visible\:ring-purple-50\/35:focus-visible{--tw-ring-color:rgba(250,245,255,.35)}.focus-visible\:ring-purple-50\/40:focus-visible{--tw-ring-color:rgba(250,245,255,.4)}.focus-visible\:ring-purple-50\/45:focus-visible{--tw-ring-color:rgba(250,245,255,.45)}.focus-visible\:ring-purple-50\/5:focus-visible{--tw-ring-color:rgba(250,245,255,.05)}.focus-visible\:ring-purple-50\/50:focus-visible{--tw-ring-color:rgba(250,245,255,.5)}.focus-visible\:ring-purple-50\/55:focus-visible{--tw-ring-color:rgba(250,245,255,.55)}.focus-visible\:ring-purple-50\/60:focus-visible{--tw-ring-color:rgba(250,245,255,.6)}.focus-visible\:ring-purple-50\/65:focus-visible{--tw-ring-color:rgba(250,245,255,.65)}.focus-visible\:ring-purple-50\/70:focus-visible{--tw-ring-color:rgba(250,245,255,.7)}.focus-visible\:ring-purple-50\/75:focus-visible{--tw-ring-color:rgba(250,245,255,.75)}.focus-visible\:ring-purple-50\/80:focus-visible{--tw-ring-color:rgba(250,245,255,.8)}.focus-visible\:ring-purple-50\/85:focus-visible{--tw-ring-color:rgba(250,245,255,.85)}.focus-visible\:ring-purple-50\/90:focus-visible{--tw-ring-color:rgba(250,245,255,.9)}.focus-visible\:ring-purple-50\/95:focus-visible{--tw-ring-color:rgba(250,245,255,.95)}.focus-visible\:ring-purple-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(168 85 247/var(--tw-ring-opacity,1))}.focus-visible\:ring-purple-500\/0:focus-visible{--tw-ring-color:rgba(168,85,247,0)}.focus-visible\:ring-purple-500\/10:focus-visible{--tw-ring-color:rgba(168,85,247,.1)}.focus-visible\:ring-purple-500\/100:focus-visible{--tw-ring-color:#a855f7}.focus-visible\:ring-purple-500\/15:focus-visible{--tw-ring-color:rgba(168,85,247,.15)}.focus-visible\:ring-purple-500\/20:focus-visible{--tw-ring-color:rgba(168,85,247,.2)}.focus-visible\:ring-purple-500\/25:focus-visible{--tw-ring-color:rgba(168,85,247,.25)}.focus-visible\:ring-purple-500\/30:focus-visible{--tw-ring-color:rgba(168,85,247,.3)}.focus-visible\:ring-purple-500\/35:focus-visible{--tw-ring-color:rgba(168,85,247,.35)}.focus-visible\:ring-purple-500\/40:focus-visible{--tw-ring-color:rgba(168,85,247,.4)}.focus-visible\:ring-purple-500\/45:focus-visible{--tw-ring-color:rgba(168,85,247,.45)}.focus-visible\:ring-purple-500\/5:focus-visible{--tw-ring-color:rgba(168,85,247,.05)}.focus-visible\:ring-purple-500\/50:focus-visible{--tw-ring-color:rgba(168,85,247,.5)}.focus-visible\:ring-purple-500\/55:focus-visible{--tw-ring-color:rgba(168,85,247,.55)}.focus-visible\:ring-purple-500\/60:focus-visible{--tw-ring-color:rgba(168,85,247,.6)}.focus-visible\:ring-purple-500\/65:focus-visible{--tw-ring-color:rgba(168,85,247,.65)}.focus-visible\:ring-purple-500\/70:focus-visible{--tw-ring-color:rgba(168,85,247,.7)}.focus-visible\:ring-purple-500\/75:focus-visible{--tw-ring-color:rgba(168,85,247,.75)}.focus-visible\:ring-purple-500\/80:focus-visible{--tw-ring-color:rgba(168,85,247,.8)}.focus-visible\:ring-purple-500\/85:focus-visible{--tw-ring-color:rgba(168,85,247,.85)}.focus-visible\:ring-purple-500\/90:focus-visible{--tw-ring-color:rgba(168,85,247,.9)}.focus-visible\:ring-purple-500\/95:focus-visible{--tw-ring-color:rgba(168,85,247,.95)}.focus-visible\:ring-purple-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(59 7 100/var(--tw-ring-opacity,1))}.focus-visible\:ring-purple-950\/0:focus-visible{--tw-ring-color:rgba(59,7,100,0)}.focus-visible\:ring-purple-950\/10:focus-visible{--tw-ring-color:rgba(59,7,100,.1)}.focus-visible\:ring-purple-950\/100:focus-visible{--tw-ring-color:#3b0764}.focus-visible\:ring-purple-950\/15:focus-visible{--tw-ring-color:rgba(59,7,100,.15)}.focus-visible\:ring-purple-950\/20:focus-visible{--tw-ring-color:rgba(59,7,100,.2)}.focus-visible\:ring-purple-950\/25:focus-visible{--tw-ring-color:rgba(59,7,100,.25)}.focus-visible\:ring-purple-950\/30:focus-visible{--tw-ring-color:rgba(59,7,100,.3)}.focus-visible\:ring-purple-950\/35:focus-visible{--tw-ring-color:rgba(59,7,100,.35)}.focus-visible\:ring-purple-950\/40:focus-visible{--tw-ring-color:rgba(59,7,100,.4)}.focus-visible\:ring-purple-950\/45:focus-visible{--tw-ring-color:rgba(59,7,100,.45)}.focus-visible\:ring-purple-950\/5:focus-visible{--tw-ring-color:rgba(59,7,100,.05)}.focus-visible\:ring-purple-950\/50:focus-visible{--tw-ring-color:rgba(59,7,100,.5)}.focus-visible\:ring-purple-950\/55:focus-visible{--tw-ring-color:rgba(59,7,100,.55)}.focus-visible\:ring-purple-950\/60:focus-visible{--tw-ring-color:rgba(59,7,100,.6)}.focus-visible\:ring-purple-950\/65:focus-visible{--tw-ring-color:rgba(59,7,100,.65)}.focus-visible\:ring-purple-950\/70:focus-visible{--tw-ring-color:rgba(59,7,100,.7)}.focus-visible\:ring-purple-950\/75:focus-visible{--tw-ring-color:rgba(59,7,100,.75)}.focus-visible\:ring-purple-950\/80:focus-visible{--tw-ring-color:rgba(59,7,100,.8)}.focus-visible\:ring-purple-950\/85:focus-visible{--tw-ring-color:rgba(59,7,100,.85)}.focus-visible\:ring-purple-950\/90:focus-visible{--tw-ring-color:rgba(59,7,100,.9)}.focus-visible\:ring-purple-950\/95:focus-visible{--tw-ring-color:rgba(59,7,100,.95)}.focus-visible\:ring-red-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(254 226 226/var(--tw-ring-opacity,1))}.focus-visible\:ring-red-100\/0:focus-visible{--tw-ring-color:hsla(0,93%,94%,0)}.focus-visible\:ring-red-100\/10:focus-visible{--tw-ring-color:hsla(0,93%,94%,.1)}.focus-visible\:ring-red-100\/100:focus-visible{--tw-ring-color:#fee2e2}.focus-visible\:ring-red-100\/15:focus-visible{--tw-ring-color:hsla(0,93%,94%,.15)}.focus-visible\:ring-red-100\/20:focus-visible{--tw-ring-color:hsla(0,93%,94%,.2)}.focus-visible\:ring-red-100\/25:focus-visible{--tw-ring-color:hsla(0,93%,94%,.25)}.focus-visible\:ring-red-100\/30:focus-visible{--tw-ring-color:hsla(0,93%,94%,.3)}.focus-visible\:ring-red-100\/35:focus-visible{--tw-ring-color:hsla(0,93%,94%,.35)}.focus-visible\:ring-red-100\/40:focus-visible{--tw-ring-color:hsla(0,93%,94%,.4)}.focus-visible\:ring-red-100\/45:focus-visible{--tw-ring-color:hsla(0,93%,94%,.45)}.focus-visible\:ring-red-100\/5:focus-visible{--tw-ring-color:hsla(0,93%,94%,.05)}.focus-visible\:ring-red-100\/50:focus-visible{--tw-ring-color:hsla(0,93%,94%,.5)}.focus-visible\:ring-red-100\/55:focus-visible{--tw-ring-color:hsla(0,93%,94%,.55)}.focus-visible\:ring-red-100\/60:focus-visible{--tw-ring-color:hsla(0,93%,94%,.6)}.focus-visible\:ring-red-100\/65:focus-visible{--tw-ring-color:hsla(0,93%,94%,.65)}.focus-visible\:ring-red-100\/70:focus-visible{--tw-ring-color:hsla(0,93%,94%,.7)}.focus-visible\:ring-red-100\/75:focus-visible{--tw-ring-color:hsla(0,93%,94%,.75)}.focus-visible\:ring-red-100\/80:focus-visible{--tw-ring-color:hsla(0,93%,94%,.8)}.focus-visible\:ring-red-100\/85:focus-visible{--tw-ring-color:hsla(0,93%,94%,.85)}.focus-visible\:ring-red-100\/90:focus-visible{--tw-ring-color:hsla(0,93%,94%,.9)}.focus-visible\:ring-red-100\/95:focus-visible{--tw-ring-color:hsla(0,93%,94%,.95)}.focus-visible\:ring-red-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(254 202 202/var(--tw-ring-opacity,1))}.focus-visible\:ring-red-200\/0:focus-visible{--tw-ring-color:hsla(0,96%,89%,0)}.focus-visible\:ring-red-200\/10:focus-visible{--tw-ring-color:hsla(0,96%,89%,.1)}.focus-visible\:ring-red-200\/100:focus-visible{--tw-ring-color:#fecaca}.focus-visible\:ring-red-200\/15:focus-visible{--tw-ring-color:hsla(0,96%,89%,.15)}.focus-visible\:ring-red-200\/20:focus-visible{--tw-ring-color:hsla(0,96%,89%,.2)}.focus-visible\:ring-red-200\/25:focus-visible{--tw-ring-color:hsla(0,96%,89%,.25)}.focus-visible\:ring-red-200\/30:focus-visible{--tw-ring-color:hsla(0,96%,89%,.3)}.focus-visible\:ring-red-200\/35:focus-visible{--tw-ring-color:hsla(0,96%,89%,.35)}.focus-visible\:ring-red-200\/40:focus-visible{--tw-ring-color:hsla(0,96%,89%,.4)}.focus-visible\:ring-red-200\/45:focus-visible{--tw-ring-color:hsla(0,96%,89%,.45)}.focus-visible\:ring-red-200\/5:focus-visible{--tw-ring-color:hsla(0,96%,89%,.05)}.focus-visible\:ring-red-200\/50:focus-visible{--tw-ring-color:hsla(0,96%,89%,.5)}.focus-visible\:ring-red-200\/55:focus-visible{--tw-ring-color:hsla(0,96%,89%,.55)}.focus-visible\:ring-red-200\/60:focus-visible{--tw-ring-color:hsla(0,96%,89%,.6)}.focus-visible\:ring-red-200\/65:focus-visible{--tw-ring-color:hsla(0,96%,89%,.65)}.focus-visible\:ring-red-200\/70:focus-visible{--tw-ring-color:hsla(0,96%,89%,.7)}.focus-visible\:ring-red-200\/75:focus-visible{--tw-ring-color:hsla(0,96%,89%,.75)}.focus-visible\:ring-red-200\/80:focus-visible{--tw-ring-color:hsla(0,96%,89%,.8)}.focus-visible\:ring-red-200\/85:focus-visible{--tw-ring-color:hsla(0,96%,89%,.85)}.focus-visible\:ring-red-200\/90:focus-visible{--tw-ring-color:hsla(0,96%,89%,.9)}.focus-visible\:ring-red-200\/95:focus-visible{--tw-ring-color:hsla(0,96%,89%,.95)}.focus-visible\:ring-red-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(248 113 113/var(--tw-ring-opacity,1))}.focus-visible\:ring-red-400\/0:focus-visible{--tw-ring-color:hsla(0,91%,71%,0)}.focus-visible\:ring-red-400\/10:focus-visible{--tw-ring-color:hsla(0,91%,71%,.1)}.focus-visible\:ring-red-400\/100:focus-visible{--tw-ring-color:#f87171}.focus-visible\:ring-red-400\/15:focus-visible{--tw-ring-color:hsla(0,91%,71%,.15)}.focus-visible\:ring-red-400\/20:focus-visible{--tw-ring-color:hsla(0,91%,71%,.2)}.focus-visible\:ring-red-400\/25:focus-visible{--tw-ring-color:hsla(0,91%,71%,.25)}.focus-visible\:ring-red-400\/30:focus-visible{--tw-ring-color:hsla(0,91%,71%,.3)}.focus-visible\:ring-red-400\/35:focus-visible{--tw-ring-color:hsla(0,91%,71%,.35)}.focus-visible\:ring-red-400\/40:focus-visible{--tw-ring-color:hsla(0,91%,71%,.4)}.focus-visible\:ring-red-400\/45:focus-visible{--tw-ring-color:hsla(0,91%,71%,.45)}.focus-visible\:ring-red-400\/5:focus-visible{--tw-ring-color:hsla(0,91%,71%,.05)}.focus-visible\:ring-red-400\/50:focus-visible{--tw-ring-color:hsla(0,91%,71%,.5)}.focus-visible\:ring-red-400\/55:focus-visible{--tw-ring-color:hsla(0,91%,71%,.55)}.focus-visible\:ring-red-400\/60:focus-visible{--tw-ring-color:hsla(0,91%,71%,.6)}.focus-visible\:ring-red-400\/65:focus-visible{--tw-ring-color:hsla(0,91%,71%,.65)}.focus-visible\:ring-red-400\/70:focus-visible{--tw-ring-color:hsla(0,91%,71%,.7)}.focus-visible\:ring-red-400\/75:focus-visible{--tw-ring-color:hsla(0,91%,71%,.75)}.focus-visible\:ring-red-400\/80:focus-visible{--tw-ring-color:hsla(0,91%,71%,.8)}.focus-visible\:ring-red-400\/85:focus-visible{--tw-ring-color:hsla(0,91%,71%,.85)}.focus-visible\:ring-red-400\/90:focus-visible{--tw-ring-color:hsla(0,91%,71%,.9)}.focus-visible\:ring-red-400\/95:focus-visible{--tw-ring-color:hsla(0,91%,71%,.95)}.focus-visible\:ring-red-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(254 242 242/var(--tw-ring-opacity,1))}.focus-visible\:ring-red-50\/0:focus-visible{--tw-ring-color:hsla(0,86%,97%,0)}.focus-visible\:ring-red-50\/10:focus-visible{--tw-ring-color:hsla(0,86%,97%,.1)}.focus-visible\:ring-red-50\/100:focus-visible{--tw-ring-color:#fef2f2}.focus-visible\:ring-red-50\/15:focus-visible{--tw-ring-color:hsla(0,86%,97%,.15)}.focus-visible\:ring-red-50\/20:focus-visible{--tw-ring-color:hsla(0,86%,97%,.2)}.focus-visible\:ring-red-50\/25:focus-visible{--tw-ring-color:hsla(0,86%,97%,.25)}.focus-visible\:ring-red-50\/30:focus-visible{--tw-ring-color:hsla(0,86%,97%,.3)}.focus-visible\:ring-red-50\/35:focus-visible{--tw-ring-color:hsla(0,86%,97%,.35)}.focus-visible\:ring-red-50\/40:focus-visible{--tw-ring-color:hsla(0,86%,97%,.4)}.focus-visible\:ring-red-50\/45:focus-visible{--tw-ring-color:hsla(0,86%,97%,.45)}.focus-visible\:ring-red-50\/5:focus-visible{--tw-ring-color:hsla(0,86%,97%,.05)}.focus-visible\:ring-red-50\/50:focus-visible{--tw-ring-color:hsla(0,86%,97%,.5)}.focus-visible\:ring-red-50\/55:focus-visible{--tw-ring-color:hsla(0,86%,97%,.55)}.focus-visible\:ring-red-50\/60:focus-visible{--tw-ring-color:hsla(0,86%,97%,.6)}.focus-visible\:ring-red-50\/65:focus-visible{--tw-ring-color:hsla(0,86%,97%,.65)}.focus-visible\:ring-red-50\/70:focus-visible{--tw-ring-color:hsla(0,86%,97%,.7)}.focus-visible\:ring-red-50\/75:focus-visible{--tw-ring-color:hsla(0,86%,97%,.75)}.focus-visible\:ring-red-50\/80:focus-visible{--tw-ring-color:hsla(0,86%,97%,.8)}.focus-visible\:ring-red-50\/85:focus-visible{--tw-ring-color:hsla(0,86%,97%,.85)}.focus-visible\:ring-red-50\/90:focus-visible{--tw-ring-color:hsla(0,86%,97%,.9)}.focus-visible\:ring-red-50\/95:focus-visible{--tw-ring-color:hsla(0,86%,97%,.95)}.focus-visible\:ring-red-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(239 68 68/var(--tw-ring-opacity,1))}.focus-visible\:ring-red-500\/0:focus-visible{--tw-ring-color:rgba(239,68,68,0)}.focus-visible\:ring-red-500\/10:focus-visible{--tw-ring-color:rgba(239,68,68,.1)}.focus-visible\:ring-red-500\/100:focus-visible{--tw-ring-color:#ef4444}.focus-visible\:ring-red-500\/15:focus-visible{--tw-ring-color:rgba(239,68,68,.15)}.focus-visible\:ring-red-500\/20:focus-visible{--tw-ring-color:rgba(239,68,68,.2)}.focus-visible\:ring-red-500\/25:focus-visible{--tw-ring-color:rgba(239,68,68,.25)}.focus-visible\:ring-red-500\/30:focus-visible{--tw-ring-color:rgba(239,68,68,.3)}.focus-visible\:ring-red-500\/35:focus-visible{--tw-ring-color:rgba(239,68,68,.35)}.focus-visible\:ring-red-500\/40:focus-visible{--tw-ring-color:rgba(239,68,68,.4)}.focus-visible\:ring-red-500\/45:focus-visible{--tw-ring-color:rgba(239,68,68,.45)}.focus-visible\:ring-red-500\/5:focus-visible{--tw-ring-color:rgba(239,68,68,.05)}.focus-visible\:ring-red-500\/50:focus-visible{--tw-ring-color:rgba(239,68,68,.5)}.focus-visible\:ring-red-500\/55:focus-visible{--tw-ring-color:rgba(239,68,68,.55)}.focus-visible\:ring-red-500\/60:focus-visible{--tw-ring-color:rgba(239,68,68,.6)}.focus-visible\:ring-red-500\/65:focus-visible{--tw-ring-color:rgba(239,68,68,.65)}.focus-visible\:ring-red-500\/70:focus-visible{--tw-ring-color:rgba(239,68,68,.7)}.focus-visible\:ring-red-500\/75:focus-visible{--tw-ring-color:rgba(239,68,68,.75)}.focus-visible\:ring-red-500\/80:focus-visible{--tw-ring-color:rgba(239,68,68,.8)}.focus-visible\:ring-red-500\/85:focus-visible{--tw-ring-color:rgba(239,68,68,.85)}.focus-visible\:ring-red-500\/90:focus-visible{--tw-ring-color:rgba(239,68,68,.9)}.focus-visible\:ring-red-500\/95:focus-visible{--tw-ring-color:rgba(239,68,68,.95)}.focus-visible\:ring-red-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(69 10 10/var(--tw-ring-opacity,1))}.focus-visible\:ring-red-950\/0:focus-visible{--tw-ring-color:rgba(69,10,10,0)}.focus-visible\:ring-red-950\/10:focus-visible{--tw-ring-color:rgba(69,10,10,.1)}.focus-visible\:ring-red-950\/100:focus-visible{--tw-ring-color:#450a0a}.focus-visible\:ring-red-950\/15:focus-visible{--tw-ring-color:rgba(69,10,10,.15)}.focus-visible\:ring-red-950\/20:focus-visible{--tw-ring-color:rgba(69,10,10,.2)}.focus-visible\:ring-red-950\/25:focus-visible{--tw-ring-color:rgba(69,10,10,.25)}.focus-visible\:ring-red-950\/30:focus-visible{--tw-ring-color:rgba(69,10,10,.3)}.focus-visible\:ring-red-950\/35:focus-visible{--tw-ring-color:rgba(69,10,10,.35)}.focus-visible\:ring-red-950\/40:focus-visible{--tw-ring-color:rgba(69,10,10,.4)}.focus-visible\:ring-red-950\/45:focus-visible{--tw-ring-color:rgba(69,10,10,.45)}.focus-visible\:ring-red-950\/5:focus-visible{--tw-ring-color:rgba(69,10,10,.05)}.focus-visible\:ring-red-950\/50:focus-visible{--tw-ring-color:rgba(69,10,10,.5)}.focus-visible\:ring-red-950\/55:focus-visible{--tw-ring-color:rgba(69,10,10,.55)}.focus-visible\:ring-red-950\/60:focus-visible{--tw-ring-color:rgba(69,10,10,.6)}.focus-visible\:ring-red-950\/65:focus-visible{--tw-ring-color:rgba(69,10,10,.65)}.focus-visible\:ring-red-950\/70:focus-visible{--tw-ring-color:rgba(69,10,10,.7)}.focus-visible\:ring-red-950\/75:focus-visible{--tw-ring-color:rgba(69,10,10,.75)}.focus-visible\:ring-red-950\/80:focus-visible{--tw-ring-color:rgba(69,10,10,.8)}.focus-visible\:ring-red-950\/85:focus-visible{--tw-ring-color:rgba(69,10,10,.85)}.focus-visible\:ring-red-950\/90:focus-visible{--tw-ring-color:rgba(69,10,10,.9)}.focus-visible\:ring-red-950\/95:focus-visible{--tw-ring-color:rgba(69,10,10,.95)}.focus-visible\:ring-ring:focus-visible{--tw-ring-color:hsl(var(--ring))}.focus-visible\:ring-rose-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(255 228 230/var(--tw-ring-opacity,1))}.focus-visible\:ring-rose-100\/0:focus-visible{--tw-ring-color:rgba(255,228,230,0)}.focus-visible\:ring-rose-100\/10:focus-visible{--tw-ring-color:rgba(255,228,230,.1)}.focus-visible\:ring-rose-100\/100:focus-visible{--tw-ring-color:#ffe4e6}.focus-visible\:ring-rose-100\/15:focus-visible{--tw-ring-color:rgba(255,228,230,.15)}.focus-visible\:ring-rose-100\/20:focus-visible{--tw-ring-color:rgba(255,228,230,.2)}.focus-visible\:ring-rose-100\/25:focus-visible{--tw-ring-color:rgba(255,228,230,.25)}.focus-visible\:ring-rose-100\/30:focus-visible{--tw-ring-color:rgba(255,228,230,.3)}.focus-visible\:ring-rose-100\/35:focus-visible{--tw-ring-color:rgba(255,228,230,.35)}.focus-visible\:ring-rose-100\/40:focus-visible{--tw-ring-color:rgba(255,228,230,.4)}.focus-visible\:ring-rose-100\/45:focus-visible{--tw-ring-color:rgba(255,228,230,.45)}.focus-visible\:ring-rose-100\/5:focus-visible{--tw-ring-color:rgba(255,228,230,.05)}.focus-visible\:ring-rose-100\/50:focus-visible{--tw-ring-color:rgba(255,228,230,.5)}.focus-visible\:ring-rose-100\/55:focus-visible{--tw-ring-color:rgba(255,228,230,.55)}.focus-visible\:ring-rose-100\/60:focus-visible{--tw-ring-color:rgba(255,228,230,.6)}.focus-visible\:ring-rose-100\/65:focus-visible{--tw-ring-color:rgba(255,228,230,.65)}.focus-visible\:ring-rose-100\/70:focus-visible{--tw-ring-color:rgba(255,228,230,.7)}.focus-visible\:ring-rose-100\/75:focus-visible{--tw-ring-color:rgba(255,228,230,.75)}.focus-visible\:ring-rose-100\/80:focus-visible{--tw-ring-color:rgba(255,228,230,.8)}.focus-visible\:ring-rose-100\/85:focus-visible{--tw-ring-color:rgba(255,228,230,.85)}.focus-visible\:ring-rose-100\/90:focus-visible{--tw-ring-color:rgba(255,228,230,.9)}.focus-visible\:ring-rose-100\/95:focus-visible{--tw-ring-color:rgba(255,228,230,.95)}.focus-visible\:ring-rose-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(254 205 211/var(--tw-ring-opacity,1))}.focus-visible\:ring-rose-200\/0:focus-visible{--tw-ring-color:rgba(254,205,211,0)}.focus-visible\:ring-rose-200\/10:focus-visible{--tw-ring-color:rgba(254,205,211,.1)}.focus-visible\:ring-rose-200\/100:focus-visible{--tw-ring-color:#fecdd3}.focus-visible\:ring-rose-200\/15:focus-visible{--tw-ring-color:rgba(254,205,211,.15)}.focus-visible\:ring-rose-200\/20:focus-visible{--tw-ring-color:rgba(254,205,211,.2)}.focus-visible\:ring-rose-200\/25:focus-visible{--tw-ring-color:rgba(254,205,211,.25)}.focus-visible\:ring-rose-200\/30:focus-visible{--tw-ring-color:rgba(254,205,211,.3)}.focus-visible\:ring-rose-200\/35:focus-visible{--tw-ring-color:rgba(254,205,211,.35)}.focus-visible\:ring-rose-200\/40:focus-visible{--tw-ring-color:rgba(254,205,211,.4)}.focus-visible\:ring-rose-200\/45:focus-visible{--tw-ring-color:rgba(254,205,211,.45)}.focus-visible\:ring-rose-200\/5:focus-visible{--tw-ring-color:rgba(254,205,211,.05)}.focus-visible\:ring-rose-200\/50:focus-visible{--tw-ring-color:rgba(254,205,211,.5)}.focus-visible\:ring-rose-200\/55:focus-visible{--tw-ring-color:rgba(254,205,211,.55)}.focus-visible\:ring-rose-200\/60:focus-visible{--tw-ring-color:rgba(254,205,211,.6)}.focus-visible\:ring-rose-200\/65:focus-visible{--tw-ring-color:rgba(254,205,211,.65)}.focus-visible\:ring-rose-200\/70:focus-visible{--tw-ring-color:rgba(254,205,211,.7)}.focus-visible\:ring-rose-200\/75:focus-visible{--tw-ring-color:rgba(254,205,211,.75)}.focus-visible\:ring-rose-200\/80:focus-visible{--tw-ring-color:rgba(254,205,211,.8)}.focus-visible\:ring-rose-200\/85:focus-visible{--tw-ring-color:rgba(254,205,211,.85)}.focus-visible\:ring-rose-200\/90:focus-visible{--tw-ring-color:rgba(254,205,211,.9)}.focus-visible\:ring-rose-200\/95:focus-visible{--tw-ring-color:rgba(254,205,211,.95)}.focus-visible\:ring-rose-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(251 113 133/var(--tw-ring-opacity,1))}.focus-visible\:ring-rose-400\/0:focus-visible{--tw-ring-color:rgba(251,113,133,0)}.focus-visible\:ring-rose-400\/10:focus-visible{--tw-ring-color:rgba(251,113,133,.1)}.focus-visible\:ring-rose-400\/100:focus-visible{--tw-ring-color:#fb7185}.focus-visible\:ring-rose-400\/15:focus-visible{--tw-ring-color:rgba(251,113,133,.15)}.focus-visible\:ring-rose-400\/20:focus-visible{--tw-ring-color:rgba(251,113,133,.2)}.focus-visible\:ring-rose-400\/25:focus-visible{--tw-ring-color:rgba(251,113,133,.25)}.focus-visible\:ring-rose-400\/30:focus-visible{--tw-ring-color:rgba(251,113,133,.3)}.focus-visible\:ring-rose-400\/35:focus-visible{--tw-ring-color:rgba(251,113,133,.35)}.focus-visible\:ring-rose-400\/40:focus-visible{--tw-ring-color:rgba(251,113,133,.4)}.focus-visible\:ring-rose-400\/45:focus-visible{--tw-ring-color:rgba(251,113,133,.45)}.focus-visible\:ring-rose-400\/5:focus-visible{--tw-ring-color:rgba(251,113,133,.05)}.focus-visible\:ring-rose-400\/50:focus-visible{--tw-ring-color:rgba(251,113,133,.5)}.focus-visible\:ring-rose-400\/55:focus-visible{--tw-ring-color:rgba(251,113,133,.55)}.focus-visible\:ring-rose-400\/60:focus-visible{--tw-ring-color:rgba(251,113,133,.6)}.focus-visible\:ring-rose-400\/65:focus-visible{--tw-ring-color:rgba(251,113,133,.65)}.focus-visible\:ring-rose-400\/70:focus-visible{--tw-ring-color:rgba(251,113,133,.7)}.focus-visible\:ring-rose-400\/75:focus-visible{--tw-ring-color:rgba(251,113,133,.75)}.focus-visible\:ring-rose-400\/80:focus-visible{--tw-ring-color:rgba(251,113,133,.8)}.focus-visible\:ring-rose-400\/85:focus-visible{--tw-ring-color:rgba(251,113,133,.85)}.focus-visible\:ring-rose-400\/90:focus-visible{--tw-ring-color:rgba(251,113,133,.9)}.focus-visible\:ring-rose-400\/95:focus-visible{--tw-ring-color:rgba(251,113,133,.95)}.focus-visible\:ring-rose-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(255 241 242/var(--tw-ring-opacity,1))}.focus-visible\:ring-rose-50\/0:focus-visible{--tw-ring-color:rgba(255,241,242,0)}.focus-visible\:ring-rose-50\/10:focus-visible{--tw-ring-color:rgba(255,241,242,.1)}.focus-visible\:ring-rose-50\/100:focus-visible{--tw-ring-color:#fff1f2}.focus-visible\:ring-rose-50\/15:focus-visible{--tw-ring-color:rgba(255,241,242,.15)}.focus-visible\:ring-rose-50\/20:focus-visible{--tw-ring-color:rgba(255,241,242,.2)}.focus-visible\:ring-rose-50\/25:focus-visible{--tw-ring-color:rgba(255,241,242,.25)}.focus-visible\:ring-rose-50\/30:focus-visible{--tw-ring-color:rgba(255,241,242,.3)}.focus-visible\:ring-rose-50\/35:focus-visible{--tw-ring-color:rgba(255,241,242,.35)}.focus-visible\:ring-rose-50\/40:focus-visible{--tw-ring-color:rgba(255,241,242,.4)}.focus-visible\:ring-rose-50\/45:focus-visible{--tw-ring-color:rgba(255,241,242,.45)}.focus-visible\:ring-rose-50\/5:focus-visible{--tw-ring-color:rgba(255,241,242,.05)}.focus-visible\:ring-rose-50\/50:focus-visible{--tw-ring-color:rgba(255,241,242,.5)}.focus-visible\:ring-rose-50\/55:focus-visible{--tw-ring-color:rgba(255,241,242,.55)}.focus-visible\:ring-rose-50\/60:focus-visible{--tw-ring-color:rgba(255,241,242,.6)}.focus-visible\:ring-rose-50\/65:focus-visible{--tw-ring-color:rgba(255,241,242,.65)}.focus-visible\:ring-rose-50\/70:focus-visible{--tw-ring-color:rgba(255,241,242,.7)}.focus-visible\:ring-rose-50\/75:focus-visible{--tw-ring-color:rgba(255,241,242,.75)}.focus-visible\:ring-rose-50\/80:focus-visible{--tw-ring-color:rgba(255,241,242,.8)}.focus-visible\:ring-rose-50\/85:focus-visible{--tw-ring-color:rgba(255,241,242,.85)}.focus-visible\:ring-rose-50\/90:focus-visible{--tw-ring-color:rgba(255,241,242,.9)}.focus-visible\:ring-rose-50\/95:focus-visible{--tw-ring-color:rgba(255,241,242,.95)}.focus-visible\:ring-rose-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(244 63 94/var(--tw-ring-opacity,1))}.focus-visible\:ring-rose-500\/0:focus-visible{--tw-ring-color:rgba(244,63,94,0)}.focus-visible\:ring-rose-500\/10:focus-visible{--tw-ring-color:rgba(244,63,94,.1)}.focus-visible\:ring-rose-500\/100:focus-visible{--tw-ring-color:#f43f5e}.focus-visible\:ring-rose-500\/15:focus-visible{--tw-ring-color:rgba(244,63,94,.15)}.focus-visible\:ring-rose-500\/20:focus-visible{--tw-ring-color:rgba(244,63,94,.2)}.focus-visible\:ring-rose-500\/25:focus-visible{--tw-ring-color:rgba(244,63,94,.25)}.focus-visible\:ring-rose-500\/30:focus-visible{--tw-ring-color:rgba(244,63,94,.3)}.focus-visible\:ring-rose-500\/35:focus-visible{--tw-ring-color:rgba(244,63,94,.35)}.focus-visible\:ring-rose-500\/40:focus-visible{--tw-ring-color:rgba(244,63,94,.4)}.focus-visible\:ring-rose-500\/45:focus-visible{--tw-ring-color:rgba(244,63,94,.45)}.focus-visible\:ring-rose-500\/5:focus-visible{--tw-ring-color:rgba(244,63,94,.05)}.focus-visible\:ring-rose-500\/50:focus-visible{--tw-ring-color:rgba(244,63,94,.5)}.focus-visible\:ring-rose-500\/55:focus-visible{--tw-ring-color:rgba(244,63,94,.55)}.focus-visible\:ring-rose-500\/60:focus-visible{--tw-ring-color:rgba(244,63,94,.6)}.focus-visible\:ring-rose-500\/65:focus-visible{--tw-ring-color:rgba(244,63,94,.65)}.focus-visible\:ring-rose-500\/70:focus-visible{--tw-ring-color:rgba(244,63,94,.7)}.focus-visible\:ring-rose-500\/75:focus-visible{--tw-ring-color:rgba(244,63,94,.75)}.focus-visible\:ring-rose-500\/80:focus-visible{--tw-ring-color:rgba(244,63,94,.8)}.focus-visible\:ring-rose-500\/85:focus-visible{--tw-ring-color:rgba(244,63,94,.85)}.focus-visible\:ring-rose-500\/90:focus-visible{--tw-ring-color:rgba(244,63,94,.9)}.focus-visible\:ring-rose-500\/95:focus-visible{--tw-ring-color:rgba(244,63,94,.95)}.focus-visible\:ring-rose-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(76 5 25/var(--tw-ring-opacity,1))}.focus-visible\:ring-rose-950\/0:focus-visible{--tw-ring-color:rgba(76,5,25,0)}.focus-visible\:ring-rose-950\/10:focus-visible{--tw-ring-color:rgba(76,5,25,.1)}.focus-visible\:ring-rose-950\/100:focus-visible{--tw-ring-color:#4c0519}.focus-visible\:ring-rose-950\/15:focus-visible{--tw-ring-color:rgba(76,5,25,.15)}.focus-visible\:ring-rose-950\/20:focus-visible{--tw-ring-color:rgba(76,5,25,.2)}.focus-visible\:ring-rose-950\/25:focus-visible{--tw-ring-color:rgba(76,5,25,.25)}.focus-visible\:ring-rose-950\/30:focus-visible{--tw-ring-color:rgba(76,5,25,.3)}.focus-visible\:ring-rose-950\/35:focus-visible{--tw-ring-color:rgba(76,5,25,.35)}.focus-visible\:ring-rose-950\/40:focus-visible{--tw-ring-color:rgba(76,5,25,.4)}.focus-visible\:ring-rose-950\/45:focus-visible{--tw-ring-color:rgba(76,5,25,.45)}.focus-visible\:ring-rose-950\/5:focus-visible{--tw-ring-color:rgba(76,5,25,.05)}.focus-visible\:ring-rose-950\/50:focus-visible{--tw-ring-color:rgba(76,5,25,.5)}.focus-visible\:ring-rose-950\/55:focus-visible{--tw-ring-color:rgba(76,5,25,.55)}.focus-visible\:ring-rose-950\/60:focus-visible{--tw-ring-color:rgba(76,5,25,.6)}.focus-visible\:ring-rose-950\/65:focus-visible{--tw-ring-color:rgba(76,5,25,.65)}.focus-visible\:ring-rose-950\/70:focus-visible{--tw-ring-color:rgba(76,5,25,.7)}.focus-visible\:ring-rose-950\/75:focus-visible{--tw-ring-color:rgba(76,5,25,.75)}.focus-visible\:ring-rose-950\/80:focus-visible{--tw-ring-color:rgba(76,5,25,.8)}.focus-visible\:ring-rose-950\/85:focus-visible{--tw-ring-color:rgba(76,5,25,.85)}.focus-visible\:ring-rose-950\/90:focus-visible{--tw-ring-color:rgba(76,5,25,.9)}.focus-visible\:ring-rose-950\/95:focus-visible{--tw-ring-color:rgba(76,5,25,.95)}.focus-visible\:ring-sky-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(224 242 254/var(--tw-ring-opacity,1))}.focus-visible\:ring-sky-100\/0:focus-visible{--tw-ring-color:rgba(224,242,254,0)}.focus-visible\:ring-sky-100\/10:focus-visible{--tw-ring-color:rgba(224,242,254,.1)}.focus-visible\:ring-sky-100\/100:focus-visible{--tw-ring-color:#e0f2fe}.focus-visible\:ring-sky-100\/15:focus-visible{--tw-ring-color:rgba(224,242,254,.15)}.focus-visible\:ring-sky-100\/20:focus-visible{--tw-ring-color:rgba(224,242,254,.2)}.focus-visible\:ring-sky-100\/25:focus-visible{--tw-ring-color:rgba(224,242,254,.25)}.focus-visible\:ring-sky-100\/30:focus-visible{--tw-ring-color:rgba(224,242,254,.3)}.focus-visible\:ring-sky-100\/35:focus-visible{--tw-ring-color:rgba(224,242,254,.35)}.focus-visible\:ring-sky-100\/40:focus-visible{--tw-ring-color:rgba(224,242,254,.4)}.focus-visible\:ring-sky-100\/45:focus-visible{--tw-ring-color:rgba(224,242,254,.45)}.focus-visible\:ring-sky-100\/5:focus-visible{--tw-ring-color:rgba(224,242,254,.05)}.focus-visible\:ring-sky-100\/50:focus-visible{--tw-ring-color:rgba(224,242,254,.5)}.focus-visible\:ring-sky-100\/55:focus-visible{--tw-ring-color:rgba(224,242,254,.55)}.focus-visible\:ring-sky-100\/60:focus-visible{--tw-ring-color:rgba(224,242,254,.6)}.focus-visible\:ring-sky-100\/65:focus-visible{--tw-ring-color:rgba(224,242,254,.65)}.focus-visible\:ring-sky-100\/70:focus-visible{--tw-ring-color:rgba(224,242,254,.7)}.focus-visible\:ring-sky-100\/75:focus-visible{--tw-ring-color:rgba(224,242,254,.75)}.focus-visible\:ring-sky-100\/80:focus-visible{--tw-ring-color:rgba(224,242,254,.8)}.focus-visible\:ring-sky-100\/85:focus-visible{--tw-ring-color:rgba(224,242,254,.85)}.focus-visible\:ring-sky-100\/90:focus-visible{--tw-ring-color:rgba(224,242,254,.9)}.focus-visible\:ring-sky-100\/95:focus-visible{--tw-ring-color:rgba(224,242,254,.95)}.focus-visible\:ring-sky-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(186 230 253/var(--tw-ring-opacity,1))}.focus-visible\:ring-sky-200\/0:focus-visible{--tw-ring-color:rgba(186,230,253,0)}.focus-visible\:ring-sky-200\/10:focus-visible{--tw-ring-color:rgba(186,230,253,.1)}.focus-visible\:ring-sky-200\/100:focus-visible{--tw-ring-color:#bae6fd}.focus-visible\:ring-sky-200\/15:focus-visible{--tw-ring-color:rgba(186,230,253,.15)}.focus-visible\:ring-sky-200\/20:focus-visible{--tw-ring-color:rgba(186,230,253,.2)}.focus-visible\:ring-sky-200\/25:focus-visible{--tw-ring-color:rgba(186,230,253,.25)}.focus-visible\:ring-sky-200\/30:focus-visible{--tw-ring-color:rgba(186,230,253,.3)}.focus-visible\:ring-sky-200\/35:focus-visible{--tw-ring-color:rgba(186,230,253,.35)}.focus-visible\:ring-sky-200\/40:focus-visible{--tw-ring-color:rgba(186,230,253,.4)}.focus-visible\:ring-sky-200\/45:focus-visible{--tw-ring-color:rgba(186,230,253,.45)}.focus-visible\:ring-sky-200\/5:focus-visible{--tw-ring-color:rgba(186,230,253,.05)}.focus-visible\:ring-sky-200\/50:focus-visible{--tw-ring-color:rgba(186,230,253,.5)}.focus-visible\:ring-sky-200\/55:focus-visible{--tw-ring-color:rgba(186,230,253,.55)}.focus-visible\:ring-sky-200\/60:focus-visible{--tw-ring-color:rgba(186,230,253,.6)}.focus-visible\:ring-sky-200\/65:focus-visible{--tw-ring-color:rgba(186,230,253,.65)}.focus-visible\:ring-sky-200\/70:focus-visible{--tw-ring-color:rgba(186,230,253,.7)}.focus-visible\:ring-sky-200\/75:focus-visible{--tw-ring-color:rgba(186,230,253,.75)}.focus-visible\:ring-sky-200\/80:focus-visible{--tw-ring-color:rgba(186,230,253,.8)}.focus-visible\:ring-sky-200\/85:focus-visible{--tw-ring-color:rgba(186,230,253,.85)}.focus-visible\:ring-sky-200\/90:focus-visible{--tw-ring-color:rgba(186,230,253,.9)}.focus-visible\:ring-sky-200\/95:focus-visible{--tw-ring-color:rgba(186,230,253,.95)}.focus-visible\:ring-sky-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(56 189 248/var(--tw-ring-opacity,1))}.focus-visible\:ring-sky-400\/0:focus-visible{--tw-ring-color:rgba(56,189,248,0)}.focus-visible\:ring-sky-400\/10:focus-visible{--tw-ring-color:rgba(56,189,248,.1)}.focus-visible\:ring-sky-400\/100:focus-visible{--tw-ring-color:#38bdf8}.focus-visible\:ring-sky-400\/15:focus-visible{--tw-ring-color:rgba(56,189,248,.15)}.focus-visible\:ring-sky-400\/20:focus-visible{--tw-ring-color:rgba(56,189,248,.2)}.focus-visible\:ring-sky-400\/25:focus-visible{--tw-ring-color:rgba(56,189,248,.25)}.focus-visible\:ring-sky-400\/30:focus-visible{--tw-ring-color:rgba(56,189,248,.3)}.focus-visible\:ring-sky-400\/35:focus-visible{--tw-ring-color:rgba(56,189,248,.35)}.focus-visible\:ring-sky-400\/40:focus-visible{--tw-ring-color:rgba(56,189,248,.4)}.focus-visible\:ring-sky-400\/45:focus-visible{--tw-ring-color:rgba(56,189,248,.45)}.focus-visible\:ring-sky-400\/5:focus-visible{--tw-ring-color:rgba(56,189,248,.05)}.focus-visible\:ring-sky-400\/50:focus-visible{--tw-ring-color:rgba(56,189,248,.5)}.focus-visible\:ring-sky-400\/55:focus-visible{--tw-ring-color:rgba(56,189,248,.55)}.focus-visible\:ring-sky-400\/60:focus-visible{--tw-ring-color:rgba(56,189,248,.6)}.focus-visible\:ring-sky-400\/65:focus-visible{--tw-ring-color:rgba(56,189,248,.65)}.focus-visible\:ring-sky-400\/70:focus-visible{--tw-ring-color:rgba(56,189,248,.7)}.focus-visible\:ring-sky-400\/75:focus-visible{--tw-ring-color:rgba(56,189,248,.75)}.focus-visible\:ring-sky-400\/80:focus-visible{--tw-ring-color:rgba(56,189,248,.8)}.focus-visible\:ring-sky-400\/85:focus-visible{--tw-ring-color:rgba(56,189,248,.85)}.focus-visible\:ring-sky-400\/90:focus-visible{--tw-ring-color:rgba(56,189,248,.9)}.focus-visible\:ring-sky-400\/95:focus-visible{--tw-ring-color:rgba(56,189,248,.95)}.focus-visible\:ring-sky-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(240 249 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-sky-50\/0:focus-visible{--tw-ring-color:rgba(240,249,255,0)}.focus-visible\:ring-sky-50\/10:focus-visible{--tw-ring-color:rgba(240,249,255,.1)}.focus-visible\:ring-sky-50\/100:focus-visible{--tw-ring-color:#f0f9ff}.focus-visible\:ring-sky-50\/15:focus-visible{--tw-ring-color:rgba(240,249,255,.15)}.focus-visible\:ring-sky-50\/20:focus-visible{--tw-ring-color:rgba(240,249,255,.2)}.focus-visible\:ring-sky-50\/25:focus-visible{--tw-ring-color:rgba(240,249,255,.25)}.focus-visible\:ring-sky-50\/30:focus-visible{--tw-ring-color:rgba(240,249,255,.3)}.focus-visible\:ring-sky-50\/35:focus-visible{--tw-ring-color:rgba(240,249,255,.35)}.focus-visible\:ring-sky-50\/40:focus-visible{--tw-ring-color:rgba(240,249,255,.4)}.focus-visible\:ring-sky-50\/45:focus-visible{--tw-ring-color:rgba(240,249,255,.45)}.focus-visible\:ring-sky-50\/5:focus-visible{--tw-ring-color:rgba(240,249,255,.05)}.focus-visible\:ring-sky-50\/50:focus-visible{--tw-ring-color:rgba(240,249,255,.5)}.focus-visible\:ring-sky-50\/55:focus-visible{--tw-ring-color:rgba(240,249,255,.55)}.focus-visible\:ring-sky-50\/60:focus-visible{--tw-ring-color:rgba(240,249,255,.6)}.focus-visible\:ring-sky-50\/65:focus-visible{--tw-ring-color:rgba(240,249,255,.65)}.focus-visible\:ring-sky-50\/70:focus-visible{--tw-ring-color:rgba(240,249,255,.7)}.focus-visible\:ring-sky-50\/75:focus-visible{--tw-ring-color:rgba(240,249,255,.75)}.focus-visible\:ring-sky-50\/80:focus-visible{--tw-ring-color:rgba(240,249,255,.8)}.focus-visible\:ring-sky-50\/85:focus-visible{--tw-ring-color:rgba(240,249,255,.85)}.focus-visible\:ring-sky-50\/90:focus-visible{--tw-ring-color:rgba(240,249,255,.9)}.focus-visible\:ring-sky-50\/95:focus-visible{--tw-ring-color:rgba(240,249,255,.95)}.focus-visible\:ring-sky-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(14 165 233/var(--tw-ring-opacity,1))}.focus-visible\:ring-sky-500\/0:focus-visible{--tw-ring-color:rgba(14,165,233,0)}.focus-visible\:ring-sky-500\/10:focus-visible{--tw-ring-color:rgba(14,165,233,.1)}.focus-visible\:ring-sky-500\/100:focus-visible{--tw-ring-color:#0ea5e9}.focus-visible\:ring-sky-500\/15:focus-visible{--tw-ring-color:rgba(14,165,233,.15)}.focus-visible\:ring-sky-500\/20:focus-visible{--tw-ring-color:rgba(14,165,233,.2)}.focus-visible\:ring-sky-500\/25:focus-visible{--tw-ring-color:rgba(14,165,233,.25)}.focus-visible\:ring-sky-500\/30:focus-visible{--tw-ring-color:rgba(14,165,233,.3)}.focus-visible\:ring-sky-500\/35:focus-visible{--tw-ring-color:rgba(14,165,233,.35)}.focus-visible\:ring-sky-500\/40:focus-visible{--tw-ring-color:rgba(14,165,233,.4)}.focus-visible\:ring-sky-500\/45:focus-visible{--tw-ring-color:rgba(14,165,233,.45)}.focus-visible\:ring-sky-500\/5:focus-visible{--tw-ring-color:rgba(14,165,233,.05)}.focus-visible\:ring-sky-500\/50:focus-visible{--tw-ring-color:rgba(14,165,233,.5)}.focus-visible\:ring-sky-500\/55:focus-visible{--tw-ring-color:rgba(14,165,233,.55)}.focus-visible\:ring-sky-500\/60:focus-visible{--tw-ring-color:rgba(14,165,233,.6)}.focus-visible\:ring-sky-500\/65:focus-visible{--tw-ring-color:rgba(14,165,233,.65)}.focus-visible\:ring-sky-500\/70:focus-visible{--tw-ring-color:rgba(14,165,233,.7)}.focus-visible\:ring-sky-500\/75:focus-visible{--tw-ring-color:rgba(14,165,233,.75)}.focus-visible\:ring-sky-500\/80:focus-visible{--tw-ring-color:rgba(14,165,233,.8)}.focus-visible\:ring-sky-500\/85:focus-visible{--tw-ring-color:rgba(14,165,233,.85)}.focus-visible\:ring-sky-500\/90:focus-visible{--tw-ring-color:rgba(14,165,233,.9)}.focus-visible\:ring-sky-500\/95:focus-visible{--tw-ring-color:rgba(14,165,233,.95)}.focus-visible\:ring-sky-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(8 47 73/var(--tw-ring-opacity,1))}.focus-visible\:ring-sky-950\/0:focus-visible{--tw-ring-color:rgba(8,47,73,0)}.focus-visible\:ring-sky-950\/10:focus-visible{--tw-ring-color:rgba(8,47,73,.1)}.focus-visible\:ring-sky-950\/100:focus-visible{--tw-ring-color:#082f49}.focus-visible\:ring-sky-950\/15:focus-visible{--tw-ring-color:rgba(8,47,73,.15)}.focus-visible\:ring-sky-950\/20:focus-visible{--tw-ring-color:rgba(8,47,73,.2)}.focus-visible\:ring-sky-950\/25:focus-visible{--tw-ring-color:rgba(8,47,73,.25)}.focus-visible\:ring-sky-950\/30:focus-visible{--tw-ring-color:rgba(8,47,73,.3)}.focus-visible\:ring-sky-950\/35:focus-visible{--tw-ring-color:rgba(8,47,73,.35)}.focus-visible\:ring-sky-950\/40:focus-visible{--tw-ring-color:rgba(8,47,73,.4)}.focus-visible\:ring-sky-950\/45:focus-visible{--tw-ring-color:rgba(8,47,73,.45)}.focus-visible\:ring-sky-950\/5:focus-visible{--tw-ring-color:rgba(8,47,73,.05)}.focus-visible\:ring-sky-950\/50:focus-visible{--tw-ring-color:rgba(8,47,73,.5)}.focus-visible\:ring-sky-950\/55:focus-visible{--tw-ring-color:rgba(8,47,73,.55)}.focus-visible\:ring-sky-950\/60:focus-visible{--tw-ring-color:rgba(8,47,73,.6)}.focus-visible\:ring-sky-950\/65:focus-visible{--tw-ring-color:rgba(8,47,73,.65)}.focus-visible\:ring-sky-950\/70:focus-visible{--tw-ring-color:rgba(8,47,73,.7)}.focus-visible\:ring-sky-950\/75:focus-visible{--tw-ring-color:rgba(8,47,73,.75)}.focus-visible\:ring-sky-950\/80:focus-visible{--tw-ring-color:rgba(8,47,73,.8)}.focus-visible\:ring-sky-950\/85:focus-visible{--tw-ring-color:rgba(8,47,73,.85)}.focus-visible\:ring-sky-950\/90:focus-visible{--tw-ring-color:rgba(8,47,73,.9)}.focus-visible\:ring-sky-950\/95:focus-visible{--tw-ring-color:rgba(8,47,73,.95)}.focus-visible\:ring-slate-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(241 245 249/var(--tw-ring-opacity,1))}.focus-visible\:ring-slate-100\/0:focus-visible{--tw-ring-color:rgba(241,245,249,0)}.focus-visible\:ring-slate-100\/10:focus-visible{--tw-ring-color:rgba(241,245,249,.1)}.focus-visible\:ring-slate-100\/100:focus-visible{--tw-ring-color:#f1f5f9}.focus-visible\:ring-slate-100\/15:focus-visible{--tw-ring-color:rgba(241,245,249,.15)}.focus-visible\:ring-slate-100\/20:focus-visible{--tw-ring-color:rgba(241,245,249,.2)}.focus-visible\:ring-slate-100\/25:focus-visible{--tw-ring-color:rgba(241,245,249,.25)}.focus-visible\:ring-slate-100\/30:focus-visible{--tw-ring-color:rgba(241,245,249,.3)}.focus-visible\:ring-slate-100\/35:focus-visible{--tw-ring-color:rgba(241,245,249,.35)}.focus-visible\:ring-slate-100\/40:focus-visible{--tw-ring-color:rgba(241,245,249,.4)}.focus-visible\:ring-slate-100\/45:focus-visible{--tw-ring-color:rgba(241,245,249,.45)}.focus-visible\:ring-slate-100\/5:focus-visible{--tw-ring-color:rgba(241,245,249,.05)}.focus-visible\:ring-slate-100\/50:focus-visible{--tw-ring-color:rgba(241,245,249,.5)}.focus-visible\:ring-slate-100\/55:focus-visible{--tw-ring-color:rgba(241,245,249,.55)}.focus-visible\:ring-slate-100\/60:focus-visible{--tw-ring-color:rgba(241,245,249,.6)}.focus-visible\:ring-slate-100\/65:focus-visible{--tw-ring-color:rgba(241,245,249,.65)}.focus-visible\:ring-slate-100\/70:focus-visible{--tw-ring-color:rgba(241,245,249,.7)}.focus-visible\:ring-slate-100\/75:focus-visible{--tw-ring-color:rgba(241,245,249,.75)}.focus-visible\:ring-slate-100\/80:focus-visible{--tw-ring-color:rgba(241,245,249,.8)}.focus-visible\:ring-slate-100\/85:focus-visible{--tw-ring-color:rgba(241,245,249,.85)}.focus-visible\:ring-slate-100\/90:focus-visible{--tw-ring-color:rgba(241,245,249,.9)}.focus-visible\:ring-slate-100\/95:focus-visible{--tw-ring-color:rgba(241,245,249,.95)}.focus-visible\:ring-slate-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(226 232 240/var(--tw-ring-opacity,1))}.focus-visible\:ring-slate-200\/0:focus-visible{--tw-ring-color:rgba(226,232,240,0)}.focus-visible\:ring-slate-200\/10:focus-visible{--tw-ring-color:rgba(226,232,240,.1)}.focus-visible\:ring-slate-200\/100:focus-visible{--tw-ring-color:#e2e8f0}.focus-visible\:ring-slate-200\/15:focus-visible{--tw-ring-color:rgba(226,232,240,.15)}.focus-visible\:ring-slate-200\/20:focus-visible{--tw-ring-color:rgba(226,232,240,.2)}.focus-visible\:ring-slate-200\/25:focus-visible{--tw-ring-color:rgba(226,232,240,.25)}.focus-visible\:ring-slate-200\/30:focus-visible{--tw-ring-color:rgba(226,232,240,.3)}.focus-visible\:ring-slate-200\/35:focus-visible{--tw-ring-color:rgba(226,232,240,.35)}.focus-visible\:ring-slate-200\/40:focus-visible{--tw-ring-color:rgba(226,232,240,.4)}.focus-visible\:ring-slate-200\/45:focus-visible{--tw-ring-color:rgba(226,232,240,.45)}.focus-visible\:ring-slate-200\/5:focus-visible{--tw-ring-color:rgba(226,232,240,.05)}.focus-visible\:ring-slate-200\/50:focus-visible{--tw-ring-color:rgba(226,232,240,.5)}.focus-visible\:ring-slate-200\/55:focus-visible{--tw-ring-color:rgba(226,232,240,.55)}.focus-visible\:ring-slate-200\/60:focus-visible{--tw-ring-color:rgba(226,232,240,.6)}.focus-visible\:ring-slate-200\/65:focus-visible{--tw-ring-color:rgba(226,232,240,.65)}.focus-visible\:ring-slate-200\/70:focus-visible{--tw-ring-color:rgba(226,232,240,.7)}.focus-visible\:ring-slate-200\/75:focus-visible{--tw-ring-color:rgba(226,232,240,.75)}.focus-visible\:ring-slate-200\/80:focus-visible{--tw-ring-color:rgba(226,232,240,.8)}.focus-visible\:ring-slate-200\/85:focus-visible{--tw-ring-color:rgba(226,232,240,.85)}.focus-visible\:ring-slate-200\/90:focus-visible{--tw-ring-color:rgba(226,232,240,.9)}.focus-visible\:ring-slate-200\/95:focus-visible{--tw-ring-color:rgba(226,232,240,.95)}.focus-visible\:ring-slate-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(148 163 184/var(--tw-ring-opacity,1))}.focus-visible\:ring-slate-400\/0:focus-visible{--tw-ring-color:rgba(148,163,184,0)}.focus-visible\:ring-slate-400\/10:focus-visible{--tw-ring-color:rgba(148,163,184,.1)}.focus-visible\:ring-slate-400\/100:focus-visible{--tw-ring-color:#94a3b8}.focus-visible\:ring-slate-400\/15:focus-visible{--tw-ring-color:rgba(148,163,184,.15)}.focus-visible\:ring-slate-400\/20:focus-visible{--tw-ring-color:rgba(148,163,184,.2)}.focus-visible\:ring-slate-400\/25:focus-visible{--tw-ring-color:rgba(148,163,184,.25)}.focus-visible\:ring-slate-400\/30:focus-visible{--tw-ring-color:rgba(148,163,184,.3)}.focus-visible\:ring-slate-400\/35:focus-visible{--tw-ring-color:rgba(148,163,184,.35)}.focus-visible\:ring-slate-400\/40:focus-visible{--tw-ring-color:rgba(148,163,184,.4)}.focus-visible\:ring-slate-400\/45:focus-visible{--tw-ring-color:rgba(148,163,184,.45)}.focus-visible\:ring-slate-400\/5:focus-visible{--tw-ring-color:rgba(148,163,184,.05)}.focus-visible\:ring-slate-400\/50:focus-visible{--tw-ring-color:rgba(148,163,184,.5)}.focus-visible\:ring-slate-400\/55:focus-visible{--tw-ring-color:rgba(148,163,184,.55)}.focus-visible\:ring-slate-400\/60:focus-visible{--tw-ring-color:rgba(148,163,184,.6)}.focus-visible\:ring-slate-400\/65:focus-visible{--tw-ring-color:rgba(148,163,184,.65)}.focus-visible\:ring-slate-400\/70:focus-visible{--tw-ring-color:rgba(148,163,184,.7)}.focus-visible\:ring-slate-400\/75:focus-visible{--tw-ring-color:rgba(148,163,184,.75)}.focus-visible\:ring-slate-400\/80:focus-visible{--tw-ring-color:rgba(148,163,184,.8)}.focus-visible\:ring-slate-400\/85:focus-visible{--tw-ring-color:rgba(148,163,184,.85)}.focus-visible\:ring-slate-400\/90:focus-visible{--tw-ring-color:rgba(148,163,184,.9)}.focus-visible\:ring-slate-400\/95:focus-visible{--tw-ring-color:rgba(148,163,184,.95)}.focus-visible\:ring-slate-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(248 250 252/var(--tw-ring-opacity,1))}.focus-visible\:ring-slate-50\/0:focus-visible{--tw-ring-color:rgba(248,250,252,0)}.focus-visible\:ring-slate-50\/10:focus-visible{--tw-ring-color:rgba(248,250,252,.1)}.focus-visible\:ring-slate-50\/100:focus-visible{--tw-ring-color:#f8fafc}.focus-visible\:ring-slate-50\/15:focus-visible{--tw-ring-color:rgba(248,250,252,.15)}.focus-visible\:ring-slate-50\/20:focus-visible{--tw-ring-color:rgba(248,250,252,.2)}.focus-visible\:ring-slate-50\/25:focus-visible{--tw-ring-color:rgba(248,250,252,.25)}.focus-visible\:ring-slate-50\/30:focus-visible{--tw-ring-color:rgba(248,250,252,.3)}.focus-visible\:ring-slate-50\/35:focus-visible{--tw-ring-color:rgba(248,250,252,.35)}.focus-visible\:ring-slate-50\/40:focus-visible{--tw-ring-color:rgba(248,250,252,.4)}.focus-visible\:ring-slate-50\/45:focus-visible{--tw-ring-color:rgba(248,250,252,.45)}.focus-visible\:ring-slate-50\/5:focus-visible{--tw-ring-color:rgba(248,250,252,.05)}.focus-visible\:ring-slate-50\/50:focus-visible{--tw-ring-color:rgba(248,250,252,.5)}.focus-visible\:ring-slate-50\/55:focus-visible{--tw-ring-color:rgba(248,250,252,.55)}.focus-visible\:ring-slate-50\/60:focus-visible{--tw-ring-color:rgba(248,250,252,.6)}.focus-visible\:ring-slate-50\/65:focus-visible{--tw-ring-color:rgba(248,250,252,.65)}.focus-visible\:ring-slate-50\/70:focus-visible{--tw-ring-color:rgba(248,250,252,.7)}.focus-visible\:ring-slate-50\/75:focus-visible{--tw-ring-color:rgba(248,250,252,.75)}.focus-visible\:ring-slate-50\/80:focus-visible{--tw-ring-color:rgba(248,250,252,.8)}.focus-visible\:ring-slate-50\/85:focus-visible{--tw-ring-color:rgba(248,250,252,.85)}.focus-visible\:ring-slate-50\/90:focus-visible{--tw-ring-color:rgba(248,250,252,.9)}.focus-visible\:ring-slate-50\/95:focus-visible{--tw-ring-color:rgba(248,250,252,.95)}.focus-visible\:ring-slate-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(100 116 139/var(--tw-ring-opacity,1))}.focus-visible\:ring-slate-500\/0:focus-visible{--tw-ring-color:rgba(100,116,139,0)}.focus-visible\:ring-slate-500\/10:focus-visible{--tw-ring-color:rgba(100,116,139,.1)}.focus-visible\:ring-slate-500\/100:focus-visible{--tw-ring-color:#64748b}.focus-visible\:ring-slate-500\/15:focus-visible{--tw-ring-color:rgba(100,116,139,.15)}.focus-visible\:ring-slate-500\/20:focus-visible{--tw-ring-color:rgba(100,116,139,.2)}.focus-visible\:ring-slate-500\/25:focus-visible{--tw-ring-color:rgba(100,116,139,.25)}.focus-visible\:ring-slate-500\/30:focus-visible{--tw-ring-color:rgba(100,116,139,.3)}.focus-visible\:ring-slate-500\/35:focus-visible{--tw-ring-color:rgba(100,116,139,.35)}.focus-visible\:ring-slate-500\/40:focus-visible{--tw-ring-color:rgba(100,116,139,.4)}.focus-visible\:ring-slate-500\/45:focus-visible{--tw-ring-color:rgba(100,116,139,.45)}.focus-visible\:ring-slate-500\/5:focus-visible{--tw-ring-color:rgba(100,116,139,.05)}.focus-visible\:ring-slate-500\/50:focus-visible{--tw-ring-color:rgba(100,116,139,.5)}.focus-visible\:ring-slate-500\/55:focus-visible{--tw-ring-color:rgba(100,116,139,.55)}.focus-visible\:ring-slate-500\/60:focus-visible{--tw-ring-color:rgba(100,116,139,.6)}.focus-visible\:ring-slate-500\/65:focus-visible{--tw-ring-color:rgba(100,116,139,.65)}.focus-visible\:ring-slate-500\/70:focus-visible{--tw-ring-color:rgba(100,116,139,.7)}.focus-visible\:ring-slate-500\/75:focus-visible{--tw-ring-color:rgba(100,116,139,.75)}.focus-visible\:ring-slate-500\/80:focus-visible{--tw-ring-color:rgba(100,116,139,.8)}.focus-visible\:ring-slate-500\/85:focus-visible{--tw-ring-color:rgba(100,116,139,.85)}.focus-visible\:ring-slate-500\/90:focus-visible{--tw-ring-color:rgba(100,116,139,.9)}.focus-visible\:ring-slate-500\/95:focus-visible{--tw-ring-color:rgba(100,116,139,.95)}.focus-visible\:ring-slate-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(2 6 23/var(--tw-ring-opacity,1))}.focus-visible\:ring-slate-950\/0:focus-visible{--tw-ring-color:rgba(2,6,23,0)}.focus-visible\:ring-slate-950\/10:focus-visible{--tw-ring-color:rgba(2,6,23,.1)}.focus-visible\:ring-slate-950\/100:focus-visible{--tw-ring-color:#020617}.focus-visible\:ring-slate-950\/15:focus-visible{--tw-ring-color:rgba(2,6,23,.15)}.focus-visible\:ring-slate-950\/20:focus-visible{--tw-ring-color:rgba(2,6,23,.2)}.focus-visible\:ring-slate-950\/25:focus-visible{--tw-ring-color:rgba(2,6,23,.25)}.focus-visible\:ring-slate-950\/30:focus-visible{--tw-ring-color:rgba(2,6,23,.3)}.focus-visible\:ring-slate-950\/35:focus-visible{--tw-ring-color:rgba(2,6,23,.35)}.focus-visible\:ring-slate-950\/40:focus-visible{--tw-ring-color:rgba(2,6,23,.4)}.focus-visible\:ring-slate-950\/45:focus-visible{--tw-ring-color:rgba(2,6,23,.45)}.focus-visible\:ring-slate-950\/5:focus-visible{--tw-ring-color:rgba(2,6,23,.05)}.focus-visible\:ring-slate-950\/50:focus-visible{--tw-ring-color:rgba(2,6,23,.5)}.focus-visible\:ring-slate-950\/55:focus-visible{--tw-ring-color:rgba(2,6,23,.55)}.focus-visible\:ring-slate-950\/60:focus-visible{--tw-ring-color:rgba(2,6,23,.6)}.focus-visible\:ring-slate-950\/65:focus-visible{--tw-ring-color:rgba(2,6,23,.65)}.focus-visible\:ring-slate-950\/70:focus-visible{--tw-ring-color:rgba(2,6,23,.7)}.focus-visible\:ring-slate-950\/75:focus-visible{--tw-ring-color:rgba(2,6,23,.75)}.focus-visible\:ring-slate-950\/80:focus-visible{--tw-ring-color:rgba(2,6,23,.8)}.focus-visible\:ring-slate-950\/85:focus-visible{--tw-ring-color:rgba(2,6,23,.85)}.focus-visible\:ring-slate-950\/90:focus-visible{--tw-ring-color:rgba(2,6,23,.9)}.focus-visible\:ring-slate-950\/95:focus-visible{--tw-ring-color:rgba(2,6,23,.95)}.focus-visible\:ring-stone-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(245 245 244/var(--tw-ring-opacity,1))}.focus-visible\:ring-stone-100\/0:focus-visible{--tw-ring-color:hsla(60,5%,96%,0)}.focus-visible\:ring-stone-100\/10:focus-visible{--tw-ring-color:hsla(60,5%,96%,.1)}.focus-visible\:ring-stone-100\/100:focus-visible{--tw-ring-color:#f5f5f4}.focus-visible\:ring-stone-100\/15:focus-visible{--tw-ring-color:hsla(60,5%,96%,.15)}.focus-visible\:ring-stone-100\/20:focus-visible{--tw-ring-color:hsla(60,5%,96%,.2)}.focus-visible\:ring-stone-100\/25:focus-visible{--tw-ring-color:hsla(60,5%,96%,.25)}.focus-visible\:ring-stone-100\/30:focus-visible{--tw-ring-color:hsla(60,5%,96%,.3)}.focus-visible\:ring-stone-100\/35:focus-visible{--tw-ring-color:hsla(60,5%,96%,.35)}.focus-visible\:ring-stone-100\/40:focus-visible{--tw-ring-color:hsla(60,5%,96%,.4)}.focus-visible\:ring-stone-100\/45:focus-visible{--tw-ring-color:hsla(60,5%,96%,.45)}.focus-visible\:ring-stone-100\/5:focus-visible{--tw-ring-color:hsla(60,5%,96%,.05)}.focus-visible\:ring-stone-100\/50:focus-visible{--tw-ring-color:hsla(60,5%,96%,.5)}.focus-visible\:ring-stone-100\/55:focus-visible{--tw-ring-color:hsla(60,5%,96%,.55)}.focus-visible\:ring-stone-100\/60:focus-visible{--tw-ring-color:hsla(60,5%,96%,.6)}.focus-visible\:ring-stone-100\/65:focus-visible{--tw-ring-color:hsla(60,5%,96%,.65)}.focus-visible\:ring-stone-100\/70:focus-visible{--tw-ring-color:hsla(60,5%,96%,.7)}.focus-visible\:ring-stone-100\/75:focus-visible{--tw-ring-color:hsla(60,5%,96%,.75)}.focus-visible\:ring-stone-100\/80:focus-visible{--tw-ring-color:hsla(60,5%,96%,.8)}.focus-visible\:ring-stone-100\/85:focus-visible{--tw-ring-color:hsla(60,5%,96%,.85)}.focus-visible\:ring-stone-100\/90:focus-visible{--tw-ring-color:hsla(60,5%,96%,.9)}.focus-visible\:ring-stone-100\/95:focus-visible{--tw-ring-color:hsla(60,5%,96%,.95)}.focus-visible\:ring-stone-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(231 229 228/var(--tw-ring-opacity,1))}.focus-visible\:ring-stone-200\/0:focus-visible{--tw-ring-color:hsla(20,6%,90%,0)}.focus-visible\:ring-stone-200\/10:focus-visible{--tw-ring-color:hsla(20,6%,90%,.1)}.focus-visible\:ring-stone-200\/100:focus-visible{--tw-ring-color:#e7e5e4}.focus-visible\:ring-stone-200\/15:focus-visible{--tw-ring-color:hsla(20,6%,90%,.15)}.focus-visible\:ring-stone-200\/20:focus-visible{--tw-ring-color:hsla(20,6%,90%,.2)}.focus-visible\:ring-stone-200\/25:focus-visible{--tw-ring-color:hsla(20,6%,90%,.25)}.focus-visible\:ring-stone-200\/30:focus-visible{--tw-ring-color:hsla(20,6%,90%,.3)}.focus-visible\:ring-stone-200\/35:focus-visible{--tw-ring-color:hsla(20,6%,90%,.35)}.focus-visible\:ring-stone-200\/40:focus-visible{--tw-ring-color:hsla(20,6%,90%,.4)}.focus-visible\:ring-stone-200\/45:focus-visible{--tw-ring-color:hsla(20,6%,90%,.45)}.focus-visible\:ring-stone-200\/5:focus-visible{--tw-ring-color:hsla(20,6%,90%,.05)}.focus-visible\:ring-stone-200\/50:focus-visible{--tw-ring-color:hsla(20,6%,90%,.5)}.focus-visible\:ring-stone-200\/55:focus-visible{--tw-ring-color:hsla(20,6%,90%,.55)}.focus-visible\:ring-stone-200\/60:focus-visible{--tw-ring-color:hsla(20,6%,90%,.6)}.focus-visible\:ring-stone-200\/65:focus-visible{--tw-ring-color:hsla(20,6%,90%,.65)}.focus-visible\:ring-stone-200\/70:focus-visible{--tw-ring-color:hsla(20,6%,90%,.7)}.focus-visible\:ring-stone-200\/75:focus-visible{--tw-ring-color:hsla(20,6%,90%,.75)}.focus-visible\:ring-stone-200\/80:focus-visible{--tw-ring-color:hsla(20,6%,90%,.8)}.focus-visible\:ring-stone-200\/85:focus-visible{--tw-ring-color:hsla(20,6%,90%,.85)}.focus-visible\:ring-stone-200\/90:focus-visible{--tw-ring-color:hsla(20,6%,90%,.9)}.focus-visible\:ring-stone-200\/95:focus-visible{--tw-ring-color:hsla(20,6%,90%,.95)}.focus-visible\:ring-stone-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(168 162 158/var(--tw-ring-opacity,1))}.focus-visible\:ring-stone-400\/0:focus-visible{--tw-ring-color:hsla(24,5%,64%,0)}.focus-visible\:ring-stone-400\/10:focus-visible{--tw-ring-color:hsla(24,5%,64%,.1)}.focus-visible\:ring-stone-400\/100:focus-visible{--tw-ring-color:#a8a29e}.focus-visible\:ring-stone-400\/15:focus-visible{--tw-ring-color:hsla(24,5%,64%,.15)}.focus-visible\:ring-stone-400\/20:focus-visible{--tw-ring-color:hsla(24,5%,64%,.2)}.focus-visible\:ring-stone-400\/25:focus-visible{--tw-ring-color:hsla(24,5%,64%,.25)}.focus-visible\:ring-stone-400\/30:focus-visible{--tw-ring-color:hsla(24,5%,64%,.3)}.focus-visible\:ring-stone-400\/35:focus-visible{--tw-ring-color:hsla(24,5%,64%,.35)}.focus-visible\:ring-stone-400\/40:focus-visible{--tw-ring-color:hsla(24,5%,64%,.4)}.focus-visible\:ring-stone-400\/45:focus-visible{--tw-ring-color:hsla(24,5%,64%,.45)}.focus-visible\:ring-stone-400\/5:focus-visible{--tw-ring-color:hsla(24,5%,64%,.05)}.focus-visible\:ring-stone-400\/50:focus-visible{--tw-ring-color:hsla(24,5%,64%,.5)}.focus-visible\:ring-stone-400\/55:focus-visible{--tw-ring-color:hsla(24,5%,64%,.55)}.focus-visible\:ring-stone-400\/60:focus-visible{--tw-ring-color:hsla(24,5%,64%,.6)}.focus-visible\:ring-stone-400\/65:focus-visible{--tw-ring-color:hsla(24,5%,64%,.65)}.focus-visible\:ring-stone-400\/70:focus-visible{--tw-ring-color:hsla(24,5%,64%,.7)}.focus-visible\:ring-stone-400\/75:focus-visible{--tw-ring-color:hsla(24,5%,64%,.75)}.focus-visible\:ring-stone-400\/80:focus-visible{--tw-ring-color:hsla(24,5%,64%,.8)}.focus-visible\:ring-stone-400\/85:focus-visible{--tw-ring-color:hsla(24,5%,64%,.85)}.focus-visible\:ring-stone-400\/90:focus-visible{--tw-ring-color:hsla(24,5%,64%,.9)}.focus-visible\:ring-stone-400\/95:focus-visible{--tw-ring-color:hsla(24,5%,64%,.95)}.focus-visible\:ring-stone-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(250 250 249/var(--tw-ring-opacity,1))}.focus-visible\:ring-stone-50\/0:focus-visible{--tw-ring-color:hsla(60,9%,98%,0)}.focus-visible\:ring-stone-50\/10:focus-visible{--tw-ring-color:hsla(60,9%,98%,.1)}.focus-visible\:ring-stone-50\/100:focus-visible{--tw-ring-color:#fafaf9}.focus-visible\:ring-stone-50\/15:focus-visible{--tw-ring-color:hsla(60,9%,98%,.15)}.focus-visible\:ring-stone-50\/20:focus-visible{--tw-ring-color:hsla(60,9%,98%,.2)}.focus-visible\:ring-stone-50\/25:focus-visible{--tw-ring-color:hsla(60,9%,98%,.25)}.focus-visible\:ring-stone-50\/30:focus-visible{--tw-ring-color:hsla(60,9%,98%,.3)}.focus-visible\:ring-stone-50\/35:focus-visible{--tw-ring-color:hsla(60,9%,98%,.35)}.focus-visible\:ring-stone-50\/40:focus-visible{--tw-ring-color:hsla(60,9%,98%,.4)}.focus-visible\:ring-stone-50\/45:focus-visible{--tw-ring-color:hsla(60,9%,98%,.45)}.focus-visible\:ring-stone-50\/5:focus-visible{--tw-ring-color:hsla(60,9%,98%,.05)}.focus-visible\:ring-stone-50\/50:focus-visible{--tw-ring-color:hsla(60,9%,98%,.5)}.focus-visible\:ring-stone-50\/55:focus-visible{--tw-ring-color:hsla(60,9%,98%,.55)}.focus-visible\:ring-stone-50\/60:focus-visible{--tw-ring-color:hsla(60,9%,98%,.6)}.focus-visible\:ring-stone-50\/65:focus-visible{--tw-ring-color:hsla(60,9%,98%,.65)}.focus-visible\:ring-stone-50\/70:focus-visible{--tw-ring-color:hsla(60,9%,98%,.7)}.focus-visible\:ring-stone-50\/75:focus-visible{--tw-ring-color:hsla(60,9%,98%,.75)}.focus-visible\:ring-stone-50\/80:focus-visible{--tw-ring-color:hsla(60,9%,98%,.8)}.focus-visible\:ring-stone-50\/85:focus-visible{--tw-ring-color:hsla(60,9%,98%,.85)}.focus-visible\:ring-stone-50\/90:focus-visible{--tw-ring-color:hsla(60,9%,98%,.9)}.focus-visible\:ring-stone-50\/95:focus-visible{--tw-ring-color:hsla(60,9%,98%,.95)}.focus-visible\:ring-stone-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(120 113 108/var(--tw-ring-opacity,1))}.focus-visible\:ring-stone-500\/0:focus-visible{--tw-ring-color:hsla(25,5%,45%,0)}.focus-visible\:ring-stone-500\/10:focus-visible{--tw-ring-color:hsla(25,5%,45%,.1)}.focus-visible\:ring-stone-500\/100:focus-visible{--tw-ring-color:#78716c}.focus-visible\:ring-stone-500\/15:focus-visible{--tw-ring-color:hsla(25,5%,45%,.15)}.focus-visible\:ring-stone-500\/20:focus-visible{--tw-ring-color:hsla(25,5%,45%,.2)}.focus-visible\:ring-stone-500\/25:focus-visible{--tw-ring-color:hsla(25,5%,45%,.25)}.focus-visible\:ring-stone-500\/30:focus-visible{--tw-ring-color:hsla(25,5%,45%,.3)}.focus-visible\:ring-stone-500\/35:focus-visible{--tw-ring-color:hsla(25,5%,45%,.35)}.focus-visible\:ring-stone-500\/40:focus-visible{--tw-ring-color:hsla(25,5%,45%,.4)}.focus-visible\:ring-stone-500\/45:focus-visible{--tw-ring-color:hsla(25,5%,45%,.45)}.focus-visible\:ring-stone-500\/5:focus-visible{--tw-ring-color:hsla(25,5%,45%,.05)}.focus-visible\:ring-stone-500\/50:focus-visible{--tw-ring-color:hsla(25,5%,45%,.5)}.focus-visible\:ring-stone-500\/55:focus-visible{--tw-ring-color:hsla(25,5%,45%,.55)}.focus-visible\:ring-stone-500\/60:focus-visible{--tw-ring-color:hsla(25,5%,45%,.6)}.focus-visible\:ring-stone-500\/65:focus-visible{--tw-ring-color:hsla(25,5%,45%,.65)}.focus-visible\:ring-stone-500\/70:focus-visible{--tw-ring-color:hsla(25,5%,45%,.7)}.focus-visible\:ring-stone-500\/75:focus-visible{--tw-ring-color:hsla(25,5%,45%,.75)}.focus-visible\:ring-stone-500\/80:focus-visible{--tw-ring-color:hsla(25,5%,45%,.8)}.focus-visible\:ring-stone-500\/85:focus-visible{--tw-ring-color:hsla(25,5%,45%,.85)}.focus-visible\:ring-stone-500\/90:focus-visible{--tw-ring-color:hsla(25,5%,45%,.9)}.focus-visible\:ring-stone-500\/95:focus-visible{--tw-ring-color:hsla(25,5%,45%,.95)}.focus-visible\:ring-stone-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(12 10 9/var(--tw-ring-opacity,1))}.focus-visible\:ring-stone-950\/0:focus-visible{--tw-ring-color:rgba(12,10,9,0)}.focus-visible\:ring-stone-950\/10:focus-visible{--tw-ring-color:rgba(12,10,9,.1)}.focus-visible\:ring-stone-950\/100:focus-visible{--tw-ring-color:#0c0a09}.focus-visible\:ring-stone-950\/15:focus-visible{--tw-ring-color:rgba(12,10,9,.15)}.focus-visible\:ring-stone-950\/20:focus-visible{--tw-ring-color:rgba(12,10,9,.2)}.focus-visible\:ring-stone-950\/25:focus-visible{--tw-ring-color:rgba(12,10,9,.25)}.focus-visible\:ring-stone-950\/30:focus-visible{--tw-ring-color:rgba(12,10,9,.3)}.focus-visible\:ring-stone-950\/35:focus-visible{--tw-ring-color:rgba(12,10,9,.35)}.focus-visible\:ring-stone-950\/40:focus-visible{--tw-ring-color:rgba(12,10,9,.4)}.focus-visible\:ring-stone-950\/45:focus-visible{--tw-ring-color:rgba(12,10,9,.45)}.focus-visible\:ring-stone-950\/5:focus-visible{--tw-ring-color:rgba(12,10,9,.05)}.focus-visible\:ring-stone-950\/50:focus-visible{--tw-ring-color:rgba(12,10,9,.5)}.focus-visible\:ring-stone-950\/55:focus-visible{--tw-ring-color:rgba(12,10,9,.55)}.focus-visible\:ring-stone-950\/60:focus-visible{--tw-ring-color:rgba(12,10,9,.6)}.focus-visible\:ring-stone-950\/65:focus-visible{--tw-ring-color:rgba(12,10,9,.65)}.focus-visible\:ring-stone-950\/70:focus-visible{--tw-ring-color:rgba(12,10,9,.7)}.focus-visible\:ring-stone-950\/75:focus-visible{--tw-ring-color:rgba(12,10,9,.75)}.focus-visible\:ring-stone-950\/80:focus-visible{--tw-ring-color:rgba(12,10,9,.8)}.focus-visible\:ring-stone-950\/85:focus-visible{--tw-ring-color:rgba(12,10,9,.85)}.focus-visible\:ring-stone-950\/90:focus-visible{--tw-ring-color:rgba(12,10,9,.9)}.focus-visible\:ring-stone-950\/95:focus-visible{--tw-ring-color:rgba(12,10,9,.95)}.focus-visible\:ring-teal-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(204 251 241/var(--tw-ring-opacity,1))}.focus-visible\:ring-teal-100\/0:focus-visible{--tw-ring-color:rgba(204,251,241,0)}.focus-visible\:ring-teal-100\/10:focus-visible{--tw-ring-color:rgba(204,251,241,.1)}.focus-visible\:ring-teal-100\/100:focus-visible{--tw-ring-color:#ccfbf1}.focus-visible\:ring-teal-100\/15:focus-visible{--tw-ring-color:rgba(204,251,241,.15)}.focus-visible\:ring-teal-100\/20:focus-visible{--tw-ring-color:rgba(204,251,241,.2)}.focus-visible\:ring-teal-100\/25:focus-visible{--tw-ring-color:rgba(204,251,241,.25)}.focus-visible\:ring-teal-100\/30:focus-visible{--tw-ring-color:rgba(204,251,241,.3)}.focus-visible\:ring-teal-100\/35:focus-visible{--tw-ring-color:rgba(204,251,241,.35)}.focus-visible\:ring-teal-100\/40:focus-visible{--tw-ring-color:rgba(204,251,241,.4)}.focus-visible\:ring-teal-100\/45:focus-visible{--tw-ring-color:rgba(204,251,241,.45)}.focus-visible\:ring-teal-100\/5:focus-visible{--tw-ring-color:rgba(204,251,241,.05)}.focus-visible\:ring-teal-100\/50:focus-visible{--tw-ring-color:rgba(204,251,241,.5)}.focus-visible\:ring-teal-100\/55:focus-visible{--tw-ring-color:rgba(204,251,241,.55)}.focus-visible\:ring-teal-100\/60:focus-visible{--tw-ring-color:rgba(204,251,241,.6)}.focus-visible\:ring-teal-100\/65:focus-visible{--tw-ring-color:rgba(204,251,241,.65)}.focus-visible\:ring-teal-100\/70:focus-visible{--tw-ring-color:rgba(204,251,241,.7)}.focus-visible\:ring-teal-100\/75:focus-visible{--tw-ring-color:rgba(204,251,241,.75)}.focus-visible\:ring-teal-100\/80:focus-visible{--tw-ring-color:rgba(204,251,241,.8)}.focus-visible\:ring-teal-100\/85:focus-visible{--tw-ring-color:rgba(204,251,241,.85)}.focus-visible\:ring-teal-100\/90:focus-visible{--tw-ring-color:rgba(204,251,241,.9)}.focus-visible\:ring-teal-100\/95:focus-visible{--tw-ring-color:rgba(204,251,241,.95)}.focus-visible\:ring-teal-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(153 246 228/var(--tw-ring-opacity,1))}.focus-visible\:ring-teal-200\/0:focus-visible{--tw-ring-color:rgba(153,246,228,0)}.focus-visible\:ring-teal-200\/10:focus-visible{--tw-ring-color:rgba(153,246,228,.1)}.focus-visible\:ring-teal-200\/100:focus-visible{--tw-ring-color:#99f6e4}.focus-visible\:ring-teal-200\/15:focus-visible{--tw-ring-color:rgba(153,246,228,.15)}.focus-visible\:ring-teal-200\/20:focus-visible{--tw-ring-color:rgba(153,246,228,.2)}.focus-visible\:ring-teal-200\/25:focus-visible{--tw-ring-color:rgba(153,246,228,.25)}.focus-visible\:ring-teal-200\/30:focus-visible{--tw-ring-color:rgba(153,246,228,.3)}.focus-visible\:ring-teal-200\/35:focus-visible{--tw-ring-color:rgba(153,246,228,.35)}.focus-visible\:ring-teal-200\/40:focus-visible{--tw-ring-color:rgba(153,246,228,.4)}.focus-visible\:ring-teal-200\/45:focus-visible{--tw-ring-color:rgba(153,246,228,.45)}.focus-visible\:ring-teal-200\/5:focus-visible{--tw-ring-color:rgba(153,246,228,.05)}.focus-visible\:ring-teal-200\/50:focus-visible{--tw-ring-color:rgba(153,246,228,.5)}.focus-visible\:ring-teal-200\/55:focus-visible{--tw-ring-color:rgba(153,246,228,.55)}.focus-visible\:ring-teal-200\/60:focus-visible{--tw-ring-color:rgba(153,246,228,.6)}.focus-visible\:ring-teal-200\/65:focus-visible{--tw-ring-color:rgba(153,246,228,.65)}.focus-visible\:ring-teal-200\/70:focus-visible{--tw-ring-color:rgba(153,246,228,.7)}.focus-visible\:ring-teal-200\/75:focus-visible{--tw-ring-color:rgba(153,246,228,.75)}.focus-visible\:ring-teal-200\/80:focus-visible{--tw-ring-color:rgba(153,246,228,.8)}.focus-visible\:ring-teal-200\/85:focus-visible{--tw-ring-color:rgba(153,246,228,.85)}.focus-visible\:ring-teal-200\/90:focus-visible{--tw-ring-color:rgba(153,246,228,.9)}.focus-visible\:ring-teal-200\/95:focus-visible{--tw-ring-color:rgba(153,246,228,.95)}.focus-visible\:ring-teal-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(45 212 191/var(--tw-ring-opacity,1))}.focus-visible\:ring-teal-400\/0:focus-visible{--tw-ring-color:rgba(45,212,191,0)}.focus-visible\:ring-teal-400\/10:focus-visible{--tw-ring-color:rgba(45,212,191,.1)}.focus-visible\:ring-teal-400\/100:focus-visible{--tw-ring-color:#2dd4bf}.focus-visible\:ring-teal-400\/15:focus-visible{--tw-ring-color:rgba(45,212,191,.15)}.focus-visible\:ring-teal-400\/20:focus-visible{--tw-ring-color:rgba(45,212,191,.2)}.focus-visible\:ring-teal-400\/25:focus-visible{--tw-ring-color:rgba(45,212,191,.25)}.focus-visible\:ring-teal-400\/30:focus-visible{--tw-ring-color:rgba(45,212,191,.3)}.focus-visible\:ring-teal-400\/35:focus-visible{--tw-ring-color:rgba(45,212,191,.35)}.focus-visible\:ring-teal-400\/40:focus-visible{--tw-ring-color:rgba(45,212,191,.4)}.focus-visible\:ring-teal-400\/45:focus-visible{--tw-ring-color:rgba(45,212,191,.45)}.focus-visible\:ring-teal-400\/5:focus-visible{--tw-ring-color:rgba(45,212,191,.05)}.focus-visible\:ring-teal-400\/50:focus-visible{--tw-ring-color:rgba(45,212,191,.5)}.focus-visible\:ring-teal-400\/55:focus-visible{--tw-ring-color:rgba(45,212,191,.55)}.focus-visible\:ring-teal-400\/60:focus-visible{--tw-ring-color:rgba(45,212,191,.6)}.focus-visible\:ring-teal-400\/65:focus-visible{--tw-ring-color:rgba(45,212,191,.65)}.focus-visible\:ring-teal-400\/70:focus-visible{--tw-ring-color:rgba(45,212,191,.7)}.focus-visible\:ring-teal-400\/75:focus-visible{--tw-ring-color:rgba(45,212,191,.75)}.focus-visible\:ring-teal-400\/80:focus-visible{--tw-ring-color:rgba(45,212,191,.8)}.focus-visible\:ring-teal-400\/85:focus-visible{--tw-ring-color:rgba(45,212,191,.85)}.focus-visible\:ring-teal-400\/90:focus-visible{--tw-ring-color:rgba(45,212,191,.9)}.focus-visible\:ring-teal-400\/95:focus-visible{--tw-ring-color:rgba(45,212,191,.95)}.focus-visible\:ring-teal-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(240 253 250/var(--tw-ring-opacity,1))}.focus-visible\:ring-teal-50\/0:focus-visible{--tw-ring-color:rgba(240,253,250,0)}.focus-visible\:ring-teal-50\/10:focus-visible{--tw-ring-color:rgba(240,253,250,.1)}.focus-visible\:ring-teal-50\/100:focus-visible{--tw-ring-color:#f0fdfa}.focus-visible\:ring-teal-50\/15:focus-visible{--tw-ring-color:rgba(240,253,250,.15)}.focus-visible\:ring-teal-50\/20:focus-visible{--tw-ring-color:rgba(240,253,250,.2)}.focus-visible\:ring-teal-50\/25:focus-visible{--tw-ring-color:rgba(240,253,250,.25)}.focus-visible\:ring-teal-50\/30:focus-visible{--tw-ring-color:rgba(240,253,250,.3)}.focus-visible\:ring-teal-50\/35:focus-visible{--tw-ring-color:rgba(240,253,250,.35)}.focus-visible\:ring-teal-50\/40:focus-visible{--tw-ring-color:rgba(240,253,250,.4)}.focus-visible\:ring-teal-50\/45:focus-visible{--tw-ring-color:rgba(240,253,250,.45)}.focus-visible\:ring-teal-50\/5:focus-visible{--tw-ring-color:rgba(240,253,250,.05)}.focus-visible\:ring-teal-50\/50:focus-visible{--tw-ring-color:rgba(240,253,250,.5)}.focus-visible\:ring-teal-50\/55:focus-visible{--tw-ring-color:rgba(240,253,250,.55)}.focus-visible\:ring-teal-50\/60:focus-visible{--tw-ring-color:rgba(240,253,250,.6)}.focus-visible\:ring-teal-50\/65:focus-visible{--tw-ring-color:rgba(240,253,250,.65)}.focus-visible\:ring-teal-50\/70:focus-visible{--tw-ring-color:rgba(240,253,250,.7)}.focus-visible\:ring-teal-50\/75:focus-visible{--tw-ring-color:rgba(240,253,250,.75)}.focus-visible\:ring-teal-50\/80:focus-visible{--tw-ring-color:rgba(240,253,250,.8)}.focus-visible\:ring-teal-50\/85:focus-visible{--tw-ring-color:rgba(240,253,250,.85)}.focus-visible\:ring-teal-50\/90:focus-visible{--tw-ring-color:rgba(240,253,250,.9)}.focus-visible\:ring-teal-50\/95:focus-visible{--tw-ring-color:rgba(240,253,250,.95)}.focus-visible\:ring-teal-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(20 184 166/var(--tw-ring-opacity,1))}.focus-visible\:ring-teal-500\/0:focus-visible{--tw-ring-color:rgba(20,184,166,0)}.focus-visible\:ring-teal-500\/10:focus-visible{--tw-ring-color:rgba(20,184,166,.1)}.focus-visible\:ring-teal-500\/100:focus-visible{--tw-ring-color:#14b8a6}.focus-visible\:ring-teal-500\/15:focus-visible{--tw-ring-color:rgba(20,184,166,.15)}.focus-visible\:ring-teal-500\/20:focus-visible{--tw-ring-color:rgba(20,184,166,.2)}.focus-visible\:ring-teal-500\/25:focus-visible{--tw-ring-color:rgba(20,184,166,.25)}.focus-visible\:ring-teal-500\/30:focus-visible{--tw-ring-color:rgba(20,184,166,.3)}.focus-visible\:ring-teal-500\/35:focus-visible{--tw-ring-color:rgba(20,184,166,.35)}.focus-visible\:ring-teal-500\/40:focus-visible{--tw-ring-color:rgba(20,184,166,.4)}.focus-visible\:ring-teal-500\/45:focus-visible{--tw-ring-color:rgba(20,184,166,.45)}.focus-visible\:ring-teal-500\/5:focus-visible{--tw-ring-color:rgba(20,184,166,.05)}.focus-visible\:ring-teal-500\/50:focus-visible{--tw-ring-color:rgba(20,184,166,.5)}.focus-visible\:ring-teal-500\/55:focus-visible{--tw-ring-color:rgba(20,184,166,.55)}.focus-visible\:ring-teal-500\/60:focus-visible{--tw-ring-color:rgba(20,184,166,.6)}.focus-visible\:ring-teal-500\/65:focus-visible{--tw-ring-color:rgba(20,184,166,.65)}.focus-visible\:ring-teal-500\/70:focus-visible{--tw-ring-color:rgba(20,184,166,.7)}.focus-visible\:ring-teal-500\/75:focus-visible{--tw-ring-color:rgba(20,184,166,.75)}.focus-visible\:ring-teal-500\/80:focus-visible{--tw-ring-color:rgba(20,184,166,.8)}.focus-visible\:ring-teal-500\/85:focus-visible{--tw-ring-color:rgba(20,184,166,.85)}.focus-visible\:ring-teal-500\/90:focus-visible{--tw-ring-color:rgba(20,184,166,.9)}.focus-visible\:ring-teal-500\/95:focus-visible{--tw-ring-color:rgba(20,184,166,.95)}.focus-visible\:ring-teal-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(4 47 46/var(--tw-ring-opacity,1))}.focus-visible\:ring-teal-950\/0:focus-visible{--tw-ring-color:rgba(4,47,46,0)}.focus-visible\:ring-teal-950\/10:focus-visible{--tw-ring-color:rgba(4,47,46,.1)}.focus-visible\:ring-teal-950\/100:focus-visible{--tw-ring-color:#042f2e}.focus-visible\:ring-teal-950\/15:focus-visible{--tw-ring-color:rgba(4,47,46,.15)}.focus-visible\:ring-teal-950\/20:focus-visible{--tw-ring-color:rgba(4,47,46,.2)}.focus-visible\:ring-teal-950\/25:focus-visible{--tw-ring-color:rgba(4,47,46,.25)}.focus-visible\:ring-teal-950\/30:focus-visible{--tw-ring-color:rgba(4,47,46,.3)}.focus-visible\:ring-teal-950\/35:focus-visible{--tw-ring-color:rgba(4,47,46,.35)}.focus-visible\:ring-teal-950\/40:focus-visible{--tw-ring-color:rgba(4,47,46,.4)}.focus-visible\:ring-teal-950\/45:focus-visible{--tw-ring-color:rgba(4,47,46,.45)}.focus-visible\:ring-teal-950\/5:focus-visible{--tw-ring-color:rgba(4,47,46,.05)}.focus-visible\:ring-teal-950\/50:focus-visible{--tw-ring-color:rgba(4,47,46,.5)}.focus-visible\:ring-teal-950\/55:focus-visible{--tw-ring-color:rgba(4,47,46,.55)}.focus-visible\:ring-teal-950\/60:focus-visible{--tw-ring-color:rgba(4,47,46,.6)}.focus-visible\:ring-teal-950\/65:focus-visible{--tw-ring-color:rgba(4,47,46,.65)}.focus-visible\:ring-teal-950\/70:focus-visible{--tw-ring-color:rgba(4,47,46,.7)}.focus-visible\:ring-teal-950\/75:focus-visible{--tw-ring-color:rgba(4,47,46,.75)}.focus-visible\:ring-teal-950\/80:focus-visible{--tw-ring-color:rgba(4,47,46,.8)}.focus-visible\:ring-teal-950\/85:focus-visible{--tw-ring-color:rgba(4,47,46,.85)}.focus-visible\:ring-teal-950\/90:focus-visible{--tw-ring-color:rgba(4,47,46,.9)}.focus-visible\:ring-teal-950\/95:focus-visible{--tw-ring-color:rgba(4,47,46,.95)}.focus-visible\:ring-transparent:focus-visible{--tw-ring-color:transparent}.focus-visible\:ring-violet-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(237 233 254/var(--tw-ring-opacity,1))}.focus-visible\:ring-violet-100\/0:focus-visible{--tw-ring-color:rgba(237,233,254,0)}.focus-visible\:ring-violet-100\/10:focus-visible{--tw-ring-color:rgba(237,233,254,.1)}.focus-visible\:ring-violet-100\/100:focus-visible{--tw-ring-color:#ede9fe}.focus-visible\:ring-violet-100\/15:focus-visible{--tw-ring-color:rgba(237,233,254,.15)}.focus-visible\:ring-violet-100\/20:focus-visible{--tw-ring-color:rgba(237,233,254,.2)}.focus-visible\:ring-violet-100\/25:focus-visible{--tw-ring-color:rgba(237,233,254,.25)}.focus-visible\:ring-violet-100\/30:focus-visible{--tw-ring-color:rgba(237,233,254,.3)}.focus-visible\:ring-violet-100\/35:focus-visible{--tw-ring-color:rgba(237,233,254,.35)}.focus-visible\:ring-violet-100\/40:focus-visible{--tw-ring-color:rgba(237,233,254,.4)}.focus-visible\:ring-violet-100\/45:focus-visible{--tw-ring-color:rgba(237,233,254,.45)}.focus-visible\:ring-violet-100\/5:focus-visible{--tw-ring-color:rgba(237,233,254,.05)}.focus-visible\:ring-violet-100\/50:focus-visible{--tw-ring-color:rgba(237,233,254,.5)}.focus-visible\:ring-violet-100\/55:focus-visible{--tw-ring-color:rgba(237,233,254,.55)}.focus-visible\:ring-violet-100\/60:focus-visible{--tw-ring-color:rgba(237,233,254,.6)}.focus-visible\:ring-violet-100\/65:focus-visible{--tw-ring-color:rgba(237,233,254,.65)}.focus-visible\:ring-violet-100\/70:focus-visible{--tw-ring-color:rgba(237,233,254,.7)}.focus-visible\:ring-violet-100\/75:focus-visible{--tw-ring-color:rgba(237,233,254,.75)}.focus-visible\:ring-violet-100\/80:focus-visible{--tw-ring-color:rgba(237,233,254,.8)}.focus-visible\:ring-violet-100\/85:focus-visible{--tw-ring-color:rgba(237,233,254,.85)}.focus-visible\:ring-violet-100\/90:focus-visible{--tw-ring-color:rgba(237,233,254,.9)}.focus-visible\:ring-violet-100\/95:focus-visible{--tw-ring-color:rgba(237,233,254,.95)}.focus-visible\:ring-violet-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(221 214 254/var(--tw-ring-opacity,1))}.focus-visible\:ring-violet-200\/0:focus-visible{--tw-ring-color:rgba(221,214,254,0)}.focus-visible\:ring-violet-200\/10:focus-visible{--tw-ring-color:rgba(221,214,254,.1)}.focus-visible\:ring-violet-200\/100:focus-visible{--tw-ring-color:#ddd6fe}.focus-visible\:ring-violet-200\/15:focus-visible{--tw-ring-color:rgba(221,214,254,.15)}.focus-visible\:ring-violet-200\/20:focus-visible{--tw-ring-color:rgba(221,214,254,.2)}.focus-visible\:ring-violet-200\/25:focus-visible{--tw-ring-color:rgba(221,214,254,.25)}.focus-visible\:ring-violet-200\/30:focus-visible{--tw-ring-color:rgba(221,214,254,.3)}.focus-visible\:ring-violet-200\/35:focus-visible{--tw-ring-color:rgba(221,214,254,.35)}.focus-visible\:ring-violet-200\/40:focus-visible{--tw-ring-color:rgba(221,214,254,.4)}.focus-visible\:ring-violet-200\/45:focus-visible{--tw-ring-color:rgba(221,214,254,.45)}.focus-visible\:ring-violet-200\/5:focus-visible{--tw-ring-color:rgba(221,214,254,.05)}.focus-visible\:ring-violet-200\/50:focus-visible{--tw-ring-color:rgba(221,214,254,.5)}.focus-visible\:ring-violet-200\/55:focus-visible{--tw-ring-color:rgba(221,214,254,.55)}.focus-visible\:ring-violet-200\/60:focus-visible{--tw-ring-color:rgba(221,214,254,.6)}.focus-visible\:ring-violet-200\/65:focus-visible{--tw-ring-color:rgba(221,214,254,.65)}.focus-visible\:ring-violet-200\/70:focus-visible{--tw-ring-color:rgba(221,214,254,.7)}.focus-visible\:ring-violet-200\/75:focus-visible{--tw-ring-color:rgba(221,214,254,.75)}.focus-visible\:ring-violet-200\/80:focus-visible{--tw-ring-color:rgba(221,214,254,.8)}.focus-visible\:ring-violet-200\/85:focus-visible{--tw-ring-color:rgba(221,214,254,.85)}.focus-visible\:ring-violet-200\/90:focus-visible{--tw-ring-color:rgba(221,214,254,.9)}.focus-visible\:ring-violet-200\/95:focus-visible{--tw-ring-color:rgba(221,214,254,.95)}.focus-visible\:ring-violet-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(167 139 250/var(--tw-ring-opacity,1))}.focus-visible\:ring-violet-400\/0:focus-visible{--tw-ring-color:rgba(167,139,250,0)}.focus-visible\:ring-violet-400\/10:focus-visible{--tw-ring-color:rgba(167,139,250,.1)}.focus-visible\:ring-violet-400\/100:focus-visible{--tw-ring-color:#a78bfa}.focus-visible\:ring-violet-400\/15:focus-visible{--tw-ring-color:rgba(167,139,250,.15)}.focus-visible\:ring-violet-400\/20:focus-visible{--tw-ring-color:rgba(167,139,250,.2)}.focus-visible\:ring-violet-400\/25:focus-visible{--tw-ring-color:rgba(167,139,250,.25)}.focus-visible\:ring-violet-400\/30:focus-visible{--tw-ring-color:rgba(167,139,250,.3)}.focus-visible\:ring-violet-400\/35:focus-visible{--tw-ring-color:rgba(167,139,250,.35)}.focus-visible\:ring-violet-400\/40:focus-visible{--tw-ring-color:rgba(167,139,250,.4)}.focus-visible\:ring-violet-400\/45:focus-visible{--tw-ring-color:rgba(167,139,250,.45)}.focus-visible\:ring-violet-400\/5:focus-visible{--tw-ring-color:rgba(167,139,250,.05)}.focus-visible\:ring-violet-400\/50:focus-visible{--tw-ring-color:rgba(167,139,250,.5)}.focus-visible\:ring-violet-400\/55:focus-visible{--tw-ring-color:rgba(167,139,250,.55)}.focus-visible\:ring-violet-400\/60:focus-visible{--tw-ring-color:rgba(167,139,250,.6)}.focus-visible\:ring-violet-400\/65:focus-visible{--tw-ring-color:rgba(167,139,250,.65)}.focus-visible\:ring-violet-400\/70:focus-visible{--tw-ring-color:rgba(167,139,250,.7)}.focus-visible\:ring-violet-400\/75:focus-visible{--tw-ring-color:rgba(167,139,250,.75)}.focus-visible\:ring-violet-400\/80:focus-visible{--tw-ring-color:rgba(167,139,250,.8)}.focus-visible\:ring-violet-400\/85:focus-visible{--tw-ring-color:rgba(167,139,250,.85)}.focus-visible\:ring-violet-400\/90:focus-visible{--tw-ring-color:rgba(167,139,250,.9)}.focus-visible\:ring-violet-400\/95:focus-visible{--tw-ring-color:rgba(167,139,250,.95)}.focus-visible\:ring-violet-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(245 243 255/var(--tw-ring-opacity,1))}.focus-visible\:ring-violet-50\/0:focus-visible{--tw-ring-color:rgba(245,243,255,0)}.focus-visible\:ring-violet-50\/10:focus-visible{--tw-ring-color:rgba(245,243,255,.1)}.focus-visible\:ring-violet-50\/100:focus-visible{--tw-ring-color:#f5f3ff}.focus-visible\:ring-violet-50\/15:focus-visible{--tw-ring-color:rgba(245,243,255,.15)}.focus-visible\:ring-violet-50\/20:focus-visible{--tw-ring-color:rgba(245,243,255,.2)}.focus-visible\:ring-violet-50\/25:focus-visible{--tw-ring-color:rgba(245,243,255,.25)}.focus-visible\:ring-violet-50\/30:focus-visible{--tw-ring-color:rgba(245,243,255,.3)}.focus-visible\:ring-violet-50\/35:focus-visible{--tw-ring-color:rgba(245,243,255,.35)}.focus-visible\:ring-violet-50\/40:focus-visible{--tw-ring-color:rgba(245,243,255,.4)}.focus-visible\:ring-violet-50\/45:focus-visible{--tw-ring-color:rgba(245,243,255,.45)}.focus-visible\:ring-violet-50\/5:focus-visible{--tw-ring-color:rgba(245,243,255,.05)}.focus-visible\:ring-violet-50\/50:focus-visible{--tw-ring-color:rgba(245,243,255,.5)}.focus-visible\:ring-violet-50\/55:focus-visible{--tw-ring-color:rgba(245,243,255,.55)}.focus-visible\:ring-violet-50\/60:focus-visible{--tw-ring-color:rgba(245,243,255,.6)}.focus-visible\:ring-violet-50\/65:focus-visible{--tw-ring-color:rgba(245,243,255,.65)}.focus-visible\:ring-violet-50\/70:focus-visible{--tw-ring-color:rgba(245,243,255,.7)}.focus-visible\:ring-violet-50\/75:focus-visible{--tw-ring-color:rgba(245,243,255,.75)}.focus-visible\:ring-violet-50\/80:focus-visible{--tw-ring-color:rgba(245,243,255,.8)}.focus-visible\:ring-violet-50\/85:focus-visible{--tw-ring-color:rgba(245,243,255,.85)}.focus-visible\:ring-violet-50\/90:focus-visible{--tw-ring-color:rgba(245,243,255,.9)}.focus-visible\:ring-violet-50\/95:focus-visible{--tw-ring-color:rgba(245,243,255,.95)}.focus-visible\:ring-violet-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(139 92 246/var(--tw-ring-opacity,1))}.focus-visible\:ring-violet-500\/0:focus-visible{--tw-ring-color:rgba(139,92,246,0)}.focus-visible\:ring-violet-500\/10:focus-visible{--tw-ring-color:rgba(139,92,246,.1)}.focus-visible\:ring-violet-500\/100:focus-visible{--tw-ring-color:#8b5cf6}.focus-visible\:ring-violet-500\/15:focus-visible{--tw-ring-color:rgba(139,92,246,.15)}.focus-visible\:ring-violet-500\/20:focus-visible{--tw-ring-color:rgba(139,92,246,.2)}.focus-visible\:ring-violet-500\/25:focus-visible{--tw-ring-color:rgba(139,92,246,.25)}.focus-visible\:ring-violet-500\/30:focus-visible{--tw-ring-color:rgba(139,92,246,.3)}.focus-visible\:ring-violet-500\/35:focus-visible{--tw-ring-color:rgba(139,92,246,.35)}.focus-visible\:ring-violet-500\/40:focus-visible{--tw-ring-color:rgba(139,92,246,.4)}.focus-visible\:ring-violet-500\/45:focus-visible{--tw-ring-color:rgba(139,92,246,.45)}.focus-visible\:ring-violet-500\/5:focus-visible{--tw-ring-color:rgba(139,92,246,.05)}.focus-visible\:ring-violet-500\/50:focus-visible{--tw-ring-color:rgba(139,92,246,.5)}.focus-visible\:ring-violet-500\/55:focus-visible{--tw-ring-color:rgba(139,92,246,.55)}.focus-visible\:ring-violet-500\/60:focus-visible{--tw-ring-color:rgba(139,92,246,.6)}.focus-visible\:ring-violet-500\/65:focus-visible{--tw-ring-color:rgba(139,92,246,.65)}.focus-visible\:ring-violet-500\/70:focus-visible{--tw-ring-color:rgba(139,92,246,.7)}.focus-visible\:ring-violet-500\/75:focus-visible{--tw-ring-color:rgba(139,92,246,.75)}.focus-visible\:ring-violet-500\/80:focus-visible{--tw-ring-color:rgba(139,92,246,.8)}.focus-visible\:ring-violet-500\/85:focus-visible{--tw-ring-color:rgba(139,92,246,.85)}.focus-visible\:ring-violet-500\/90:focus-visible{--tw-ring-color:rgba(139,92,246,.9)}.focus-visible\:ring-violet-500\/95:focus-visible{--tw-ring-color:rgba(139,92,246,.95)}.focus-visible\:ring-violet-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(46 16 101/var(--tw-ring-opacity,1))}.focus-visible\:ring-violet-950\/0:focus-visible{--tw-ring-color:rgba(46,16,101,0)}.focus-visible\:ring-violet-950\/10:focus-visible{--tw-ring-color:rgba(46,16,101,.1)}.focus-visible\:ring-violet-950\/100:focus-visible{--tw-ring-color:#2e1065}.focus-visible\:ring-violet-950\/15:focus-visible{--tw-ring-color:rgba(46,16,101,.15)}.focus-visible\:ring-violet-950\/20:focus-visible{--tw-ring-color:rgba(46,16,101,.2)}.focus-visible\:ring-violet-950\/25:focus-visible{--tw-ring-color:rgba(46,16,101,.25)}.focus-visible\:ring-violet-950\/30:focus-visible{--tw-ring-color:rgba(46,16,101,.3)}.focus-visible\:ring-violet-950\/35:focus-visible{--tw-ring-color:rgba(46,16,101,.35)}.focus-visible\:ring-violet-950\/40:focus-visible{--tw-ring-color:rgba(46,16,101,.4)}.focus-visible\:ring-violet-950\/45:focus-visible{--tw-ring-color:rgba(46,16,101,.45)}.focus-visible\:ring-violet-950\/5:focus-visible{--tw-ring-color:rgba(46,16,101,.05)}.focus-visible\:ring-violet-950\/50:focus-visible{--tw-ring-color:rgba(46,16,101,.5)}.focus-visible\:ring-violet-950\/55:focus-visible{--tw-ring-color:rgba(46,16,101,.55)}.focus-visible\:ring-violet-950\/60:focus-visible{--tw-ring-color:rgba(46,16,101,.6)}.focus-visible\:ring-violet-950\/65:focus-visible{--tw-ring-color:rgba(46,16,101,.65)}.focus-visible\:ring-violet-950\/70:focus-visible{--tw-ring-color:rgba(46,16,101,.7)}.focus-visible\:ring-violet-950\/75:focus-visible{--tw-ring-color:rgba(46,16,101,.75)}.focus-visible\:ring-violet-950\/80:focus-visible{--tw-ring-color:rgba(46,16,101,.8)}.focus-visible\:ring-violet-950\/85:focus-visible{--tw-ring-color:rgba(46,16,101,.85)}.focus-visible\:ring-violet-950\/90:focus-visible{--tw-ring-color:rgba(46,16,101,.9)}.focus-visible\:ring-violet-950\/95:focus-visible{--tw-ring-color:rgba(46,16,101,.95)}.focus-visible\:ring-yellow-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(254 249 195/var(--tw-ring-opacity,1))}.focus-visible\:ring-yellow-100\/0:focus-visible{--tw-ring-color:hsla(55,97%,88%,0)}.focus-visible\:ring-yellow-100\/10:focus-visible{--tw-ring-color:hsla(55,97%,88%,.1)}.focus-visible\:ring-yellow-100\/100:focus-visible{--tw-ring-color:#fef9c3}.focus-visible\:ring-yellow-100\/15:focus-visible{--tw-ring-color:hsla(55,97%,88%,.15)}.focus-visible\:ring-yellow-100\/20:focus-visible{--tw-ring-color:hsla(55,97%,88%,.2)}.focus-visible\:ring-yellow-100\/25:focus-visible{--tw-ring-color:hsla(55,97%,88%,.25)}.focus-visible\:ring-yellow-100\/30:focus-visible{--tw-ring-color:hsla(55,97%,88%,.3)}.focus-visible\:ring-yellow-100\/35:focus-visible{--tw-ring-color:hsla(55,97%,88%,.35)}.focus-visible\:ring-yellow-100\/40:focus-visible{--tw-ring-color:hsla(55,97%,88%,.4)}.focus-visible\:ring-yellow-100\/45:focus-visible{--tw-ring-color:hsla(55,97%,88%,.45)}.focus-visible\:ring-yellow-100\/5:focus-visible{--tw-ring-color:hsla(55,97%,88%,.05)}.focus-visible\:ring-yellow-100\/50:focus-visible{--tw-ring-color:hsla(55,97%,88%,.5)}.focus-visible\:ring-yellow-100\/55:focus-visible{--tw-ring-color:hsla(55,97%,88%,.55)}.focus-visible\:ring-yellow-100\/60:focus-visible{--tw-ring-color:hsla(55,97%,88%,.6)}.focus-visible\:ring-yellow-100\/65:focus-visible{--tw-ring-color:hsla(55,97%,88%,.65)}.focus-visible\:ring-yellow-100\/70:focus-visible{--tw-ring-color:hsla(55,97%,88%,.7)}.focus-visible\:ring-yellow-100\/75:focus-visible{--tw-ring-color:hsla(55,97%,88%,.75)}.focus-visible\:ring-yellow-100\/80:focus-visible{--tw-ring-color:hsla(55,97%,88%,.8)}.focus-visible\:ring-yellow-100\/85:focus-visible{--tw-ring-color:hsla(55,97%,88%,.85)}.focus-visible\:ring-yellow-100\/90:focus-visible{--tw-ring-color:hsla(55,97%,88%,.9)}.focus-visible\:ring-yellow-100\/95:focus-visible{--tw-ring-color:hsla(55,97%,88%,.95)}.focus-visible\:ring-yellow-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(254 240 138/var(--tw-ring-opacity,1))}.focus-visible\:ring-yellow-200\/0:focus-visible{--tw-ring-color:hsla(53,98%,77%,0)}.focus-visible\:ring-yellow-200\/10:focus-visible{--tw-ring-color:hsla(53,98%,77%,.1)}.focus-visible\:ring-yellow-200\/100:focus-visible{--tw-ring-color:#fef08a}.focus-visible\:ring-yellow-200\/15:focus-visible{--tw-ring-color:hsla(53,98%,77%,.15)}.focus-visible\:ring-yellow-200\/20:focus-visible{--tw-ring-color:hsla(53,98%,77%,.2)}.focus-visible\:ring-yellow-200\/25:focus-visible{--tw-ring-color:hsla(53,98%,77%,.25)}.focus-visible\:ring-yellow-200\/30:focus-visible{--tw-ring-color:hsla(53,98%,77%,.3)}.focus-visible\:ring-yellow-200\/35:focus-visible{--tw-ring-color:hsla(53,98%,77%,.35)}.focus-visible\:ring-yellow-200\/40:focus-visible{--tw-ring-color:hsla(53,98%,77%,.4)}.focus-visible\:ring-yellow-200\/45:focus-visible{--tw-ring-color:hsla(53,98%,77%,.45)}.focus-visible\:ring-yellow-200\/5:focus-visible{--tw-ring-color:hsla(53,98%,77%,.05)}.focus-visible\:ring-yellow-200\/50:focus-visible{--tw-ring-color:hsla(53,98%,77%,.5)}.focus-visible\:ring-yellow-200\/55:focus-visible{--tw-ring-color:hsla(53,98%,77%,.55)}.focus-visible\:ring-yellow-200\/60:focus-visible{--tw-ring-color:hsla(53,98%,77%,.6)}.focus-visible\:ring-yellow-200\/65:focus-visible{--tw-ring-color:hsla(53,98%,77%,.65)}.focus-visible\:ring-yellow-200\/70:focus-visible{--tw-ring-color:hsla(53,98%,77%,.7)}.focus-visible\:ring-yellow-200\/75:focus-visible{--tw-ring-color:hsla(53,98%,77%,.75)}.focus-visible\:ring-yellow-200\/80:focus-visible{--tw-ring-color:hsla(53,98%,77%,.8)}.focus-visible\:ring-yellow-200\/85:focus-visible{--tw-ring-color:hsla(53,98%,77%,.85)}.focus-visible\:ring-yellow-200\/90:focus-visible{--tw-ring-color:hsla(53,98%,77%,.9)}.focus-visible\:ring-yellow-200\/95:focus-visible{--tw-ring-color:hsla(53,98%,77%,.95)}.focus-visible\:ring-yellow-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(250 204 21/var(--tw-ring-opacity,1))}.focus-visible\:ring-yellow-400\/0:focus-visible{--tw-ring-color:rgba(250,204,21,0)}.focus-visible\:ring-yellow-400\/10:focus-visible{--tw-ring-color:rgba(250,204,21,.1)}.focus-visible\:ring-yellow-400\/100:focus-visible{--tw-ring-color:#facc15}.focus-visible\:ring-yellow-400\/15:focus-visible{--tw-ring-color:rgba(250,204,21,.15)}.focus-visible\:ring-yellow-400\/20:focus-visible{--tw-ring-color:rgba(250,204,21,.2)}.focus-visible\:ring-yellow-400\/25:focus-visible{--tw-ring-color:rgba(250,204,21,.25)}.focus-visible\:ring-yellow-400\/30:focus-visible{--tw-ring-color:rgba(250,204,21,.3)}.focus-visible\:ring-yellow-400\/35:focus-visible{--tw-ring-color:rgba(250,204,21,.35)}.focus-visible\:ring-yellow-400\/40:focus-visible{--tw-ring-color:rgba(250,204,21,.4)}.focus-visible\:ring-yellow-400\/45:focus-visible{--tw-ring-color:rgba(250,204,21,.45)}.focus-visible\:ring-yellow-400\/5:focus-visible{--tw-ring-color:rgba(250,204,21,.05)}.focus-visible\:ring-yellow-400\/50:focus-visible{--tw-ring-color:rgba(250,204,21,.5)}.focus-visible\:ring-yellow-400\/55:focus-visible{--tw-ring-color:rgba(250,204,21,.55)}.focus-visible\:ring-yellow-400\/60:focus-visible{--tw-ring-color:rgba(250,204,21,.6)}.focus-visible\:ring-yellow-400\/65:focus-visible{--tw-ring-color:rgba(250,204,21,.65)}.focus-visible\:ring-yellow-400\/70:focus-visible{--tw-ring-color:rgba(250,204,21,.7)}.focus-visible\:ring-yellow-400\/75:focus-visible{--tw-ring-color:rgba(250,204,21,.75)}.focus-visible\:ring-yellow-400\/80:focus-visible{--tw-ring-color:rgba(250,204,21,.8)}.focus-visible\:ring-yellow-400\/85:focus-visible{--tw-ring-color:rgba(250,204,21,.85)}.focus-visible\:ring-yellow-400\/90:focus-visible{--tw-ring-color:rgba(250,204,21,.9)}.focus-visible\:ring-yellow-400\/95:focus-visible{--tw-ring-color:rgba(250,204,21,.95)}.focus-visible\:ring-yellow-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(254 252 232/var(--tw-ring-opacity,1))}.focus-visible\:ring-yellow-50\/0:focus-visible{--tw-ring-color:hsla(55,92%,95%,0)}.focus-visible\:ring-yellow-50\/10:focus-visible{--tw-ring-color:hsla(55,92%,95%,.1)}.focus-visible\:ring-yellow-50\/100:focus-visible{--tw-ring-color:#fefce8}.focus-visible\:ring-yellow-50\/15:focus-visible{--tw-ring-color:hsla(55,92%,95%,.15)}.focus-visible\:ring-yellow-50\/20:focus-visible{--tw-ring-color:hsla(55,92%,95%,.2)}.focus-visible\:ring-yellow-50\/25:focus-visible{--tw-ring-color:hsla(55,92%,95%,.25)}.focus-visible\:ring-yellow-50\/30:focus-visible{--tw-ring-color:hsla(55,92%,95%,.3)}.focus-visible\:ring-yellow-50\/35:focus-visible{--tw-ring-color:hsla(55,92%,95%,.35)}.focus-visible\:ring-yellow-50\/40:focus-visible{--tw-ring-color:hsla(55,92%,95%,.4)}.focus-visible\:ring-yellow-50\/45:focus-visible{--tw-ring-color:hsla(55,92%,95%,.45)}.focus-visible\:ring-yellow-50\/5:focus-visible{--tw-ring-color:hsla(55,92%,95%,.05)}.focus-visible\:ring-yellow-50\/50:focus-visible{--tw-ring-color:hsla(55,92%,95%,.5)}.focus-visible\:ring-yellow-50\/55:focus-visible{--tw-ring-color:hsla(55,92%,95%,.55)}.focus-visible\:ring-yellow-50\/60:focus-visible{--tw-ring-color:hsla(55,92%,95%,.6)}.focus-visible\:ring-yellow-50\/65:focus-visible{--tw-ring-color:hsla(55,92%,95%,.65)}.focus-visible\:ring-yellow-50\/70:focus-visible{--tw-ring-color:hsla(55,92%,95%,.7)}.focus-visible\:ring-yellow-50\/75:focus-visible{--tw-ring-color:hsla(55,92%,95%,.75)}.focus-visible\:ring-yellow-50\/80:focus-visible{--tw-ring-color:hsla(55,92%,95%,.8)}.focus-visible\:ring-yellow-50\/85:focus-visible{--tw-ring-color:hsla(55,92%,95%,.85)}.focus-visible\:ring-yellow-50\/90:focus-visible{--tw-ring-color:hsla(55,92%,95%,.9)}.focus-visible\:ring-yellow-50\/95:focus-visible{--tw-ring-color:hsla(55,92%,95%,.95)}.focus-visible\:ring-yellow-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(234 179 8/var(--tw-ring-opacity,1))}.focus-visible\:ring-yellow-500\/0:focus-visible{--tw-ring-color:rgba(234,179,8,0)}.focus-visible\:ring-yellow-500\/10:focus-visible{--tw-ring-color:rgba(234,179,8,.1)}.focus-visible\:ring-yellow-500\/100:focus-visible{--tw-ring-color:#eab308}.focus-visible\:ring-yellow-500\/15:focus-visible{--tw-ring-color:rgba(234,179,8,.15)}.focus-visible\:ring-yellow-500\/20:focus-visible{--tw-ring-color:rgba(234,179,8,.2)}.focus-visible\:ring-yellow-500\/25:focus-visible{--tw-ring-color:rgba(234,179,8,.25)}.focus-visible\:ring-yellow-500\/30:focus-visible{--tw-ring-color:rgba(234,179,8,.3)}.focus-visible\:ring-yellow-500\/35:focus-visible{--tw-ring-color:rgba(234,179,8,.35)}.focus-visible\:ring-yellow-500\/40:focus-visible{--tw-ring-color:rgba(234,179,8,.4)}.focus-visible\:ring-yellow-500\/45:focus-visible{--tw-ring-color:rgba(234,179,8,.45)}.focus-visible\:ring-yellow-500\/5:focus-visible{--tw-ring-color:rgba(234,179,8,.05)}.focus-visible\:ring-yellow-500\/50:focus-visible{--tw-ring-color:rgba(234,179,8,.5)}.focus-visible\:ring-yellow-500\/55:focus-visible{--tw-ring-color:rgba(234,179,8,.55)}.focus-visible\:ring-yellow-500\/60:focus-visible{--tw-ring-color:rgba(234,179,8,.6)}.focus-visible\:ring-yellow-500\/65:focus-visible{--tw-ring-color:rgba(234,179,8,.65)}.focus-visible\:ring-yellow-500\/70:focus-visible{--tw-ring-color:rgba(234,179,8,.7)}.focus-visible\:ring-yellow-500\/75:focus-visible{--tw-ring-color:rgba(234,179,8,.75)}.focus-visible\:ring-yellow-500\/80:focus-visible{--tw-ring-color:rgba(234,179,8,.8)}.focus-visible\:ring-yellow-500\/85:focus-visible{--tw-ring-color:rgba(234,179,8,.85)}.focus-visible\:ring-yellow-500\/90:focus-visible{--tw-ring-color:rgba(234,179,8,.9)}.focus-visible\:ring-yellow-500\/95:focus-visible{--tw-ring-color:rgba(234,179,8,.95)}.focus-visible\:ring-yellow-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(66 32 6/var(--tw-ring-opacity,1))}.focus-visible\:ring-yellow-950\/0:focus-visible{--tw-ring-color:rgba(66,32,6,0)}.focus-visible\:ring-yellow-950\/10:focus-visible{--tw-ring-color:rgba(66,32,6,.1)}.focus-visible\:ring-yellow-950\/100:focus-visible{--tw-ring-color:#422006}.focus-visible\:ring-yellow-950\/15:focus-visible{--tw-ring-color:rgba(66,32,6,.15)}.focus-visible\:ring-yellow-950\/20:focus-visible{--tw-ring-color:rgba(66,32,6,.2)}.focus-visible\:ring-yellow-950\/25:focus-visible{--tw-ring-color:rgba(66,32,6,.25)}.focus-visible\:ring-yellow-950\/30:focus-visible{--tw-ring-color:rgba(66,32,6,.3)}.focus-visible\:ring-yellow-950\/35:focus-visible{--tw-ring-color:rgba(66,32,6,.35)}.focus-visible\:ring-yellow-950\/40:focus-visible{--tw-ring-color:rgba(66,32,6,.4)}.focus-visible\:ring-yellow-950\/45:focus-visible{--tw-ring-color:rgba(66,32,6,.45)}.focus-visible\:ring-yellow-950\/5:focus-visible{--tw-ring-color:rgba(66,32,6,.05)}.focus-visible\:ring-yellow-950\/50:focus-visible{--tw-ring-color:rgba(66,32,6,.5)}.focus-visible\:ring-yellow-950\/55:focus-visible{--tw-ring-color:rgba(66,32,6,.55)}.focus-visible\:ring-yellow-950\/60:focus-visible{--tw-ring-color:rgba(66,32,6,.6)}.focus-visible\:ring-yellow-950\/65:focus-visible{--tw-ring-color:rgba(66,32,6,.65)}.focus-visible\:ring-yellow-950\/70:focus-visible{--tw-ring-color:rgba(66,32,6,.7)}.focus-visible\:ring-yellow-950\/75:focus-visible{--tw-ring-color:rgba(66,32,6,.75)}.focus-visible\:ring-yellow-950\/80:focus-visible{--tw-ring-color:rgba(66,32,6,.8)}.focus-visible\:ring-yellow-950\/85:focus-visible{--tw-ring-color:rgba(66,32,6,.85)}.focus-visible\:ring-yellow-950\/90:focus-visible{--tw-ring-color:rgba(66,32,6,.9)}.focus-visible\:ring-yellow-950\/95:focus-visible{--tw-ring-color:rgba(66,32,6,.95)}.focus-visible\:ring-zinc-100:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(244 244 245/var(--tw-ring-opacity,1))}.focus-visible\:ring-zinc-100\/0:focus-visible{--tw-ring-color:hsla(240,5%,96%,0)}.focus-visible\:ring-zinc-100\/10:focus-visible{--tw-ring-color:hsla(240,5%,96%,.1)}.focus-visible\:ring-zinc-100\/100:focus-visible{--tw-ring-color:#f4f4f5}.focus-visible\:ring-zinc-100\/15:focus-visible{--tw-ring-color:hsla(240,5%,96%,.15)}.focus-visible\:ring-zinc-100\/20:focus-visible{--tw-ring-color:hsla(240,5%,96%,.2)}.focus-visible\:ring-zinc-100\/25:focus-visible{--tw-ring-color:hsla(240,5%,96%,.25)}.focus-visible\:ring-zinc-100\/30:focus-visible{--tw-ring-color:hsla(240,5%,96%,.3)}.focus-visible\:ring-zinc-100\/35:focus-visible{--tw-ring-color:hsla(240,5%,96%,.35)}.focus-visible\:ring-zinc-100\/40:focus-visible{--tw-ring-color:hsla(240,5%,96%,.4)}.focus-visible\:ring-zinc-100\/45:focus-visible{--tw-ring-color:hsla(240,5%,96%,.45)}.focus-visible\:ring-zinc-100\/5:focus-visible{--tw-ring-color:hsla(240,5%,96%,.05)}.focus-visible\:ring-zinc-100\/50:focus-visible{--tw-ring-color:hsla(240,5%,96%,.5)}.focus-visible\:ring-zinc-100\/55:focus-visible{--tw-ring-color:hsla(240,5%,96%,.55)}.focus-visible\:ring-zinc-100\/60:focus-visible{--tw-ring-color:hsla(240,5%,96%,.6)}.focus-visible\:ring-zinc-100\/65:focus-visible{--tw-ring-color:hsla(240,5%,96%,.65)}.focus-visible\:ring-zinc-100\/70:focus-visible{--tw-ring-color:hsla(240,5%,96%,.7)}.focus-visible\:ring-zinc-100\/75:focus-visible{--tw-ring-color:hsla(240,5%,96%,.75)}.focus-visible\:ring-zinc-100\/80:focus-visible{--tw-ring-color:hsla(240,5%,96%,.8)}.focus-visible\:ring-zinc-100\/85:focus-visible{--tw-ring-color:hsla(240,5%,96%,.85)}.focus-visible\:ring-zinc-100\/90:focus-visible{--tw-ring-color:hsla(240,5%,96%,.9)}.focus-visible\:ring-zinc-100\/95:focus-visible{--tw-ring-color:hsla(240,5%,96%,.95)}.focus-visible\:ring-zinc-200:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(228 228 231/var(--tw-ring-opacity,1))}.focus-visible\:ring-zinc-200\/0:focus-visible{--tw-ring-color:hsla(240,6%,90%,0)}.focus-visible\:ring-zinc-200\/10:focus-visible{--tw-ring-color:hsla(240,6%,90%,.1)}.focus-visible\:ring-zinc-200\/100:focus-visible{--tw-ring-color:#e4e4e7}.focus-visible\:ring-zinc-200\/15:focus-visible{--tw-ring-color:hsla(240,6%,90%,.15)}.focus-visible\:ring-zinc-200\/20:focus-visible{--tw-ring-color:hsla(240,6%,90%,.2)}.focus-visible\:ring-zinc-200\/25:focus-visible{--tw-ring-color:hsla(240,6%,90%,.25)}.focus-visible\:ring-zinc-200\/30:focus-visible{--tw-ring-color:hsla(240,6%,90%,.3)}.focus-visible\:ring-zinc-200\/35:focus-visible{--tw-ring-color:hsla(240,6%,90%,.35)}.focus-visible\:ring-zinc-200\/40:focus-visible{--tw-ring-color:hsla(240,6%,90%,.4)}.focus-visible\:ring-zinc-200\/45:focus-visible{--tw-ring-color:hsla(240,6%,90%,.45)}.focus-visible\:ring-zinc-200\/5:focus-visible{--tw-ring-color:hsla(240,6%,90%,.05)}.focus-visible\:ring-zinc-200\/50:focus-visible{--tw-ring-color:hsla(240,6%,90%,.5)}.focus-visible\:ring-zinc-200\/55:focus-visible{--tw-ring-color:hsla(240,6%,90%,.55)}.focus-visible\:ring-zinc-200\/60:focus-visible{--tw-ring-color:hsla(240,6%,90%,.6)}.focus-visible\:ring-zinc-200\/65:focus-visible{--tw-ring-color:hsla(240,6%,90%,.65)}.focus-visible\:ring-zinc-200\/70:focus-visible{--tw-ring-color:hsla(240,6%,90%,.7)}.focus-visible\:ring-zinc-200\/75:focus-visible{--tw-ring-color:hsla(240,6%,90%,.75)}.focus-visible\:ring-zinc-200\/80:focus-visible{--tw-ring-color:hsla(240,6%,90%,.8)}.focus-visible\:ring-zinc-200\/85:focus-visible{--tw-ring-color:hsla(240,6%,90%,.85)}.focus-visible\:ring-zinc-200\/90:focus-visible{--tw-ring-color:hsla(240,6%,90%,.9)}.focus-visible\:ring-zinc-200\/95:focus-visible{--tw-ring-color:hsla(240,6%,90%,.95)}.focus-visible\:ring-zinc-400:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(161 161 170/var(--tw-ring-opacity,1))}.focus-visible\:ring-zinc-400\/0:focus-visible{--tw-ring-color:hsla(240,5%,65%,0)}.focus-visible\:ring-zinc-400\/10:focus-visible{--tw-ring-color:hsla(240,5%,65%,.1)}.focus-visible\:ring-zinc-400\/100:focus-visible{--tw-ring-color:#a1a1aa}.focus-visible\:ring-zinc-400\/15:focus-visible{--tw-ring-color:hsla(240,5%,65%,.15)}.focus-visible\:ring-zinc-400\/20:focus-visible{--tw-ring-color:hsla(240,5%,65%,.2)}.focus-visible\:ring-zinc-400\/25:focus-visible{--tw-ring-color:hsla(240,5%,65%,.25)}.focus-visible\:ring-zinc-400\/30:focus-visible{--tw-ring-color:hsla(240,5%,65%,.3)}.focus-visible\:ring-zinc-400\/35:focus-visible{--tw-ring-color:hsla(240,5%,65%,.35)}.focus-visible\:ring-zinc-400\/40:focus-visible{--tw-ring-color:hsla(240,5%,65%,.4)}.focus-visible\:ring-zinc-400\/45:focus-visible{--tw-ring-color:hsla(240,5%,65%,.45)}.focus-visible\:ring-zinc-400\/5:focus-visible{--tw-ring-color:hsla(240,5%,65%,.05)}.focus-visible\:ring-zinc-400\/50:focus-visible{--tw-ring-color:hsla(240,5%,65%,.5)}.focus-visible\:ring-zinc-400\/55:focus-visible{--tw-ring-color:hsla(240,5%,65%,.55)}.focus-visible\:ring-zinc-400\/60:focus-visible{--tw-ring-color:hsla(240,5%,65%,.6)}.focus-visible\:ring-zinc-400\/65:focus-visible{--tw-ring-color:hsla(240,5%,65%,.65)}.focus-visible\:ring-zinc-400\/70:focus-visible{--tw-ring-color:hsla(240,5%,65%,.7)}.focus-visible\:ring-zinc-400\/75:focus-visible{--tw-ring-color:hsla(240,5%,65%,.75)}.focus-visible\:ring-zinc-400\/80:focus-visible{--tw-ring-color:hsla(240,5%,65%,.8)}.focus-visible\:ring-zinc-400\/85:focus-visible{--tw-ring-color:hsla(240,5%,65%,.85)}.focus-visible\:ring-zinc-400\/90:focus-visible{--tw-ring-color:hsla(240,5%,65%,.9)}.focus-visible\:ring-zinc-400\/95:focus-visible{--tw-ring-color:hsla(240,5%,65%,.95)}.focus-visible\:ring-zinc-50:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(250 250 250/var(--tw-ring-opacity,1))}.focus-visible\:ring-zinc-50\/0:focus-visible{--tw-ring-color:hsla(0,0%,98%,0)}.focus-visible\:ring-zinc-50\/10:focus-visible{--tw-ring-color:hsla(0,0%,98%,.1)}.focus-visible\:ring-zinc-50\/100:focus-visible{--tw-ring-color:#fafafa}.focus-visible\:ring-zinc-50\/15:focus-visible{--tw-ring-color:hsla(0,0%,98%,.15)}.focus-visible\:ring-zinc-50\/20:focus-visible{--tw-ring-color:hsla(0,0%,98%,.2)}.focus-visible\:ring-zinc-50\/25:focus-visible{--tw-ring-color:hsla(0,0%,98%,.25)}.focus-visible\:ring-zinc-50\/30:focus-visible{--tw-ring-color:hsla(0,0%,98%,.3)}.focus-visible\:ring-zinc-50\/35:focus-visible{--tw-ring-color:hsla(0,0%,98%,.35)}.focus-visible\:ring-zinc-50\/40:focus-visible{--tw-ring-color:hsla(0,0%,98%,.4)}.focus-visible\:ring-zinc-50\/45:focus-visible{--tw-ring-color:hsla(0,0%,98%,.45)}.focus-visible\:ring-zinc-50\/5:focus-visible{--tw-ring-color:hsla(0,0%,98%,.05)}.focus-visible\:ring-zinc-50\/50:focus-visible{--tw-ring-color:hsla(0,0%,98%,.5)}.focus-visible\:ring-zinc-50\/55:focus-visible{--tw-ring-color:hsla(0,0%,98%,.55)}.focus-visible\:ring-zinc-50\/60:focus-visible{--tw-ring-color:hsla(0,0%,98%,.6)}.focus-visible\:ring-zinc-50\/65:focus-visible{--tw-ring-color:hsla(0,0%,98%,.65)}.focus-visible\:ring-zinc-50\/70:focus-visible{--tw-ring-color:hsla(0,0%,98%,.7)}.focus-visible\:ring-zinc-50\/75:focus-visible{--tw-ring-color:hsla(0,0%,98%,.75)}.focus-visible\:ring-zinc-50\/80:focus-visible{--tw-ring-color:hsla(0,0%,98%,.8)}.focus-visible\:ring-zinc-50\/85:focus-visible{--tw-ring-color:hsla(0,0%,98%,.85)}.focus-visible\:ring-zinc-50\/90:focus-visible{--tw-ring-color:hsla(0,0%,98%,.9)}.focus-visible\:ring-zinc-50\/95:focus-visible{--tw-ring-color:hsla(0,0%,98%,.95)}.focus-visible\:ring-zinc-500:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(113 113 122/var(--tw-ring-opacity,1))}.focus-visible\:ring-zinc-500\/0:focus-visible{--tw-ring-color:hsla(240,4%,46%,0)}.focus-visible\:ring-zinc-500\/10:focus-visible{--tw-ring-color:hsla(240,4%,46%,.1)}.focus-visible\:ring-zinc-500\/100:focus-visible{--tw-ring-color:#71717a}.focus-visible\:ring-zinc-500\/15:focus-visible{--tw-ring-color:hsla(240,4%,46%,.15)}.focus-visible\:ring-zinc-500\/20:focus-visible{--tw-ring-color:hsla(240,4%,46%,.2)}.focus-visible\:ring-zinc-500\/25:focus-visible{--tw-ring-color:hsla(240,4%,46%,.25)}.focus-visible\:ring-zinc-500\/30:focus-visible{--tw-ring-color:hsla(240,4%,46%,.3)}.focus-visible\:ring-zinc-500\/35:focus-visible{--tw-ring-color:hsla(240,4%,46%,.35)}.focus-visible\:ring-zinc-500\/40:focus-visible{--tw-ring-color:hsla(240,4%,46%,.4)}.focus-visible\:ring-zinc-500\/45:focus-visible{--tw-ring-color:hsla(240,4%,46%,.45)}.focus-visible\:ring-zinc-500\/5:focus-visible{--tw-ring-color:hsla(240,4%,46%,.05)}.focus-visible\:ring-zinc-500\/50:focus-visible{--tw-ring-color:hsla(240,4%,46%,.5)}.focus-visible\:ring-zinc-500\/55:focus-visible{--tw-ring-color:hsla(240,4%,46%,.55)}.focus-visible\:ring-zinc-500\/60:focus-visible{--tw-ring-color:hsla(240,4%,46%,.6)}.focus-visible\:ring-zinc-500\/65:focus-visible{--tw-ring-color:hsla(240,4%,46%,.65)}.focus-visible\:ring-zinc-500\/70:focus-visible{--tw-ring-color:hsla(240,4%,46%,.7)}.focus-visible\:ring-zinc-500\/75:focus-visible{--tw-ring-color:hsla(240,4%,46%,.75)}.focus-visible\:ring-zinc-500\/80:focus-visible{--tw-ring-color:hsla(240,4%,46%,.8)}.focus-visible\:ring-zinc-500\/85:focus-visible{--tw-ring-color:hsla(240,4%,46%,.85)}.focus-visible\:ring-zinc-500\/90:focus-visible{--tw-ring-color:hsla(240,4%,46%,.9)}.focus-visible\:ring-zinc-500\/95:focus-visible{--tw-ring-color:hsla(240,4%,46%,.95)}.focus-visible\:ring-zinc-950:focus-visible{--tw-ring-opacity:1;--tw-ring-color:rgb(9 9 11/var(--tw-ring-opacity,1))}.focus-visible\:ring-zinc-950\/0:focus-visible{--tw-ring-color:rgba(9,9,11,0)}.focus-visible\:ring-zinc-950\/10:focus-visible{--tw-ring-color:rgba(9,9,11,.1)}.focus-visible\:ring-zinc-950\/100:focus-visible{--tw-ring-color:#09090b}.focus-visible\:ring-zinc-950\/15:focus-visible{--tw-ring-color:rgba(9,9,11,.15)}.focus-visible\:ring-zinc-950\/20:focus-visible{--tw-ring-color:rgba(9,9,11,.2)}.focus-visible\:ring-zinc-950\/25:focus-visible{--tw-ring-color:rgba(9,9,11,.25)}.focus-visible\:ring-zinc-950\/30:focus-visible{--tw-ring-color:rgba(9,9,11,.3)}.focus-visible\:ring-zinc-950\/35:focus-visible{--tw-ring-color:rgba(9,9,11,.35)}.focus-visible\:ring-zinc-950\/40:focus-visible{--tw-ring-color:rgba(9,9,11,.4)}.focus-visible\:ring-zinc-950\/45:focus-visible{--tw-ring-color:rgba(9,9,11,.45)}.focus-visible\:ring-zinc-950\/5:focus-visible{--tw-ring-color:rgba(9,9,11,.05)}.focus-visible\:ring-zinc-950\/50:focus-visible{--tw-ring-color:rgba(9,9,11,.5)}.focus-visible\:ring-zinc-950\/55:focus-visible{--tw-ring-color:rgba(9,9,11,.55)}.focus-visible\:ring-zinc-950\/60:focus-visible{--tw-ring-color:rgba(9,9,11,.6)}.focus-visible\:ring-zinc-950\/65:focus-visible{--tw-ring-color:rgba(9,9,11,.65)}.focus-visible\:ring-zinc-950\/70:focus-visible{--tw-ring-color:rgba(9,9,11,.7)}.focus-visible\:ring-zinc-950\/75:focus-visible{--tw-ring-color:rgba(9,9,11,.75)}.focus-visible\:ring-zinc-950\/80:focus-visible{--tw-ring-color:rgba(9,9,11,.8)}.focus-visible\:ring-zinc-950\/85:focus-visible{--tw-ring-color:rgba(9,9,11,.85)}.focus-visible\:ring-zinc-950\/90:focus-visible{--tw-ring-color:rgba(9,9,11,.9)}.focus-visible\:ring-zinc-950\/95:focus-visible{--tw-ring-color:rgba(9,9,11,.95)}.focus-visible\:ring-offset-2:focus-visible{--tw-ring-offset-width:2px}.disabled\:pointer-events-none:disabled{pointer-events:none}.disabled\:cursor-not-allowed:disabled{cursor:not-allowed}.disabled\:opacity-50:disabled{opacity:.5}.group:hover .group-hover\:opacity-100{opacity:1}.group.destructive .group-\[\.destructive\]\:border-muted\/40{border-color:hsl(var(--muted)/.4)}.group.destructive .group-\[\.destructive\]\:text-red-300{--tw-text-opacity:1;color:rgb(252 165 165/var(--tw-text-opacity,1))}.group.destructive .group-\[\.destructive\]\:hover\:border-destructive\/30:hover{border-color:hsl(var(--destructive)/.3)}.group.destructive .group-\[\.destructive\]\:hover\:bg-destructive:hover{background-color:hsl(var(--destructive))}.group.destructive .group-\[\.destructive\]\:hover\:text-destructive-foreground:hover{color:hsl(var(--destructive-foreground))}.group.destructive .group-\[\.destructive\]\:hover\:text-red-50:hover{--tw-text-opacity:1;color:rgb(254 242 242/var(--tw-text-opacity,1))}.group.destructive .group-\[\.destructive\]\:focus\:ring-destructive:focus{--tw-ring-color:hsl(var(--destructive))}.group.destructive .group-\[\.destructive\]\:focus\:ring-red-400:focus{--tw-ring-opacity:1;--tw-ring-color:rgb(248 113 113/var(--tw-ring-opacity,1))}.group.destructive .group-\[\.destructive\]\:focus\:ring-offset-red-600:focus{--tw-ring-offset-color:#dc2626}.peer:disabled~.peer-disabled\:cursor-not-allowed{cursor:not-allowed}.peer:disabled~.peer-disabled\:opacity-70{opacity:.7}.has-\[\:disabled\]\:opacity-50:has(:disabled){opacity:.5}.data-\[disabled\=true\]\:pointer-events-none[data-disabled=true],.data-\[disabled\]\:pointer-events-none[data-disabled]{pointer-events:none}.data-\[side\=bottom\]\:translate-y-1[data-side=bottom]{--tw-translate-y:0.25rem}.data-\[side\=bottom\]\:translate-y-1[data-side=bottom],.data-\[side\=left\]\:-translate-x-1[data-side=left]{transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}.data-\[side\=left\]\:-translate-x-1[data-side=left]{--tw-translate-x:-0.25rem}.data-\[side\=right\]\:translate-x-1[data-side=right]{--tw-translate-x:0.25rem}.data-\[side\=right\]\:translate-x-1[data-side=right],.data-\[side\=top\]\:-translate-y-1[data-side=top]{transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}.data-\[side\=top\]\:-translate-y-1[data-side=top]{--tw-translate-y:-0.25rem}.data-\[swipe\=cancel\]\:translate-x-0[data-swipe=cancel]{--tw-translate-x:0px}.data-\[swipe\=cancel\]\:translate-x-0[data-swipe=cancel],.data-\[swipe\=end\]\:translate-x-\[var\(--radix-toast-swipe-end-x\)\][data-swipe=end]{transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}.data-\[swipe\=end\]\:translate-x-\[var\(--radix-toast-swipe-end-x\)\][data-swipe=end]{--tw-translate-x:var(--radix-toast-swipe-end-x)}.data-\[swipe\=move\]\:translate-x-\[var\(--radix-toast-swipe-move-x\)\][data-swipe=move]{--tw-translate-x:var(--radix-toast-swipe-move-x);transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}.data-\[active\]\:bg-accent\/50[data-active]{background-color:hsl(var(--accent)/.5)}.data-\[selected\=true\]\:bg-accent[data-selected=true],.data-\[state\=on\]\:bg-accent[data-state=on],.data-\[state\=open\]\:bg-accent[data-state=open]{background-color:hsl(var(--accent))}.data-\[state\=open\]\:bg-accent\/50[data-state=open]{background-color:hsl(var(--accent)/.5)}.data-\[state\=open\]\:bg-secondary[data-state=open]{background-color:hsl(var(--secondary))}.data-\[state\=selected\]\:bg-muted[data-state=selected]{background-color:hsl(var(--muted))}.data-\[selected\=true\]\:text-accent-foreground[data-selected=true],.data-\[state\=on\]\:text-accent-foreground[data-state=on],.data-\[state\=open\]\:text-accent-foreground[data-state=open]{color:hsl(var(--accent-foreground))}.data-\[state\=open\]\:text-muted-foreground[data-state=open]{color:hsl(var(--muted-foreground))}.data-\[disabled\=true\]\:opacity-50[data-disabled=true],.data-\[disabled\]\:opacity-50[data-disabled]{opacity:.5}.data-\[swipe\=move\]\:transition-none[data-swipe=move]{transition-property:none}.data-\[state\=closed\]\:duration-300[data-state=closed]{transition-duration:.3s}.data-\[state\=open\]\:duration-500[data-state=open]{transition-duration:.5s}.data-\[motion\^\=from-\]\:animate-in[data-motion^=from-],.data-\[state\=open\]\:animate-in[data-state=open],.data-\[state\=visible\]\:animate-in[data-state=visible]{animation-name:enter;animation-duration:.15s;--tw-enter-opacity:initial;--tw-enter-scale:initial;--tw-enter-rotate:initial;--tw-enter-translate-x:initial;--tw-enter-translate-y:initial}.data-\[motion\^\=to-\]\:animate-out[data-motion^=to-],.data-\[state\=closed\]\:animate-out[data-state=closed],.data-\[state\=hidden\]\:animate-out[data-state=hidden],.data-\[swipe\=end\]\:animate-out[data-swipe=end]{animation-name:exit;animation-duration:.15s;--tw-exit-opacity:initial;--tw-exit-scale:initial;--tw-exit-rotate:initial;--tw-exit-translate-x:initial;--tw-exit-translate-y:initial}.data-\[motion\^\=from-\]\:fade-in[data-motion^=from-]{--tw-enter-opacity:0}.data-\[motion\^\=to-\]\:fade-out[data-motion^=to-],.data-\[state\=closed\]\:fade-out-0[data-state=closed]{--tw-exit-opacity:0}.data-\[state\=closed\]\:fade-out-80[data-state=closed]{--tw-exit-opacity:0.8}.data-\[state\=hidden\]\:fade-out[data-state=hidden]{--tw-exit-opacity:0}.data-\[state\=open\]\:fade-in-0[data-state=open],.data-\[state\=visible\]\:fade-in[data-state=visible]{--tw-enter-opacity:0}.data-\[state\=closed\]\:zoom-out-95[data-state=closed]{--tw-exit-scale:.95}.data-\[state\=open\]\:zoom-in-90[data-state=open]{--tw-enter-scale:.9}.data-\[state\=open\]\:zoom-in-95[data-state=open]{--tw-enter-scale:.95}.data-\[motion\=from-end\]\:slide-in-from-right-52[data-motion=from-end]{--tw-enter-translate-x:13rem}.data-\[motion\=from-start\]\:slide-in-from-left-52[data-motion=from-start]{--tw-enter-translate-x:-13rem}.data-\[motion\=to-end\]\:slide-out-to-right-52[data-motion=to-end]{--tw-exit-translate-x:13rem}.data-\[motion\=to-start\]\:slide-out-to-left-52[data-motion=to-start]{--tw-exit-translate-x:-13rem}.data-\[side\=bottom\]\:slide-in-from-top-2[data-side=bottom]{--tw-enter-translate-y:-0.5rem}.data-\[side\=left\]\:slide-in-from-right-2[data-side=left]{--tw-enter-translate-x:0.5rem}.data-\[side\=right\]\:slide-in-from-left-2[data-side=right]{--tw-enter-translate-x:-0.5rem}.data-\[side\=top\]\:slide-in-from-bottom-2[data-side=top]{--tw-enter-translate-y:0.5rem}.data-\[state\=closed\]\:slide-out-to-bottom[data-state=closed]{--tw-exit-translate-y:100%}.data-\[state\=closed\]\:slide-out-to-left[data-state=closed]{--tw-exit-translate-x:-100%}.data-\[state\=closed\]\:slide-out-to-left-1\/2[data-state=closed]{--tw-exit-translate-x:-50%}.data-\[state\=closed\]\:slide-out-to-right-full[data-state=closed],.data-\[state\=closed\]\:slide-out-to-right[data-state=closed]{--tw-exit-translate-x:100%}.data-\[state\=closed\]\:slide-out-to-top[data-state=closed]{--tw-exit-translate-y:-100%}.data-\[state\=closed\]\:slide-out-to-top-\[48\%\][data-state=closed]{--tw-exit-translate-y:-48%}.data-\[state\=open\]\:slide-in-from-bottom[data-state=open]{--tw-enter-translate-y:100%}.data-\[state\=open\]\:slide-in-from-left[data-state=open]{--tw-enter-translate-x:-100%}.data-\[state\=open\]\:slide-in-from-left-1\/2[data-state=open]{--tw-enter-translate-x:-50%}.data-\[state\=open\]\:slide-in-from-right[data-state=open]{--tw-enter-translate-x:100%}.data-\[state\=open\]\:slide-in-from-top[data-state=open]{--tw-enter-translate-y:-100%}.data-\[state\=open\]\:slide-in-from-top-\[48\%\][data-state=open]{--tw-enter-translate-y:-48%}.data-\[state\=open\]\:slide-in-from-top-full[data-state=open]{--tw-enter-translate-y:-100%}.data-\[state\=closed\]\:duration-300[data-state=closed]{animation-duration:.3s}.data-\[state\=open\]\:duration-500[data-state=open]{animation-duration:.5s}.group[data-state=open] .group-data-\[state\=open\]\:rotate-180{--tw-rotate:180deg;transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}.dark\:border:is(.dark *){border-width:1px}.dark\:border-0:is(.dark *){border-width:0}.dark\:border-none:is(.dark *){border-style:none}.dark\:border-amber-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(254 243 199/var(--tw-border-opacity,1))}.dark\:border-amber-100\/0:is(.dark *){border-color:hsla(48,96%,89%,0)}.dark\:border-amber-100\/10:is(.dark *){border-color:hsla(48,96%,89%,.1)}.dark\:border-amber-100\/100:is(.dark *){border-color:#fef3c7}.dark\:border-amber-100\/15:is(.dark *){border-color:hsla(48,96%,89%,.15)}.dark\:border-amber-100\/20:is(.dark *){border-color:hsla(48,96%,89%,.2)}.dark\:border-amber-100\/25:is(.dark *){border-color:hsla(48,96%,89%,.25)}.dark\:border-amber-100\/30:is(.dark *){border-color:hsla(48,96%,89%,.3)}.dark\:border-amber-100\/35:is(.dark *){border-color:hsla(48,96%,89%,.35)}.dark\:border-amber-100\/40:is(.dark *){border-color:hsla(48,96%,89%,.4)}.dark\:border-amber-100\/45:is(.dark *){border-color:hsla(48,96%,89%,.45)}.dark\:border-amber-100\/5:is(.dark *){border-color:hsla(48,96%,89%,.05)}.dark\:border-amber-100\/50:is(.dark *){border-color:hsla(48,96%,89%,.5)}.dark\:border-amber-100\/55:is(.dark *){border-color:hsla(48,96%,89%,.55)}.dark\:border-amber-100\/60:is(.dark *){border-color:hsla(48,96%,89%,.6)}.dark\:border-amber-100\/65:is(.dark *){border-color:hsla(48,96%,89%,.65)}.dark\:border-amber-100\/70:is(.dark *){border-color:hsla(48,96%,89%,.7)}.dark\:border-amber-100\/75:is(.dark *){border-color:hsla(48,96%,89%,.75)}.dark\:border-amber-100\/80:is(.dark *){border-color:hsla(48,96%,89%,.8)}.dark\:border-amber-100\/85:is(.dark *){border-color:hsla(48,96%,89%,.85)}.dark\:border-amber-100\/90:is(.dark *){border-color:hsla(48,96%,89%,.9)}.dark\:border-amber-100\/95:is(.dark *){border-color:hsla(48,96%,89%,.95)}.dark\:border-amber-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(253 230 138/var(--tw-border-opacity,1))}.dark\:border-amber-200\/0:is(.dark *){border-color:hsla(48,97%,77%,0)}.dark\:border-amber-200\/10:is(.dark *){border-color:hsla(48,97%,77%,.1)}.dark\:border-amber-200\/100:is(.dark *){border-color:#fde68a}.dark\:border-amber-200\/15:is(.dark *){border-color:hsla(48,97%,77%,.15)}.dark\:border-amber-200\/20:is(.dark *){border-color:hsla(48,97%,77%,.2)}.dark\:border-amber-200\/25:is(.dark *){border-color:hsla(48,97%,77%,.25)}.dark\:border-amber-200\/30:is(.dark *){border-color:hsla(48,97%,77%,.3)}.dark\:border-amber-200\/35:is(.dark *){border-color:hsla(48,97%,77%,.35)}.dark\:border-amber-200\/40:is(.dark *){border-color:hsla(48,97%,77%,.4)}.dark\:border-amber-200\/45:is(.dark *){border-color:hsla(48,97%,77%,.45)}.dark\:border-amber-200\/5:is(.dark *){border-color:hsla(48,97%,77%,.05)}.dark\:border-amber-200\/50:is(.dark *){border-color:hsla(48,97%,77%,.5)}.dark\:border-amber-200\/55:is(.dark *){border-color:hsla(48,97%,77%,.55)}.dark\:border-amber-200\/60:is(.dark *){border-color:hsla(48,97%,77%,.6)}.dark\:border-amber-200\/65:is(.dark *){border-color:hsla(48,97%,77%,.65)}.dark\:border-amber-200\/70:is(.dark *){border-color:hsla(48,97%,77%,.7)}.dark\:border-amber-200\/75:is(.dark *){border-color:hsla(48,97%,77%,.75)}.dark\:border-amber-200\/80:is(.dark *){border-color:hsla(48,97%,77%,.8)}.dark\:border-amber-200\/85:is(.dark *){border-color:hsla(48,97%,77%,.85)}.dark\:border-amber-200\/90:is(.dark *){border-color:hsla(48,97%,77%,.9)}.dark\:border-amber-200\/95:is(.dark *){border-color:hsla(48,97%,77%,.95)}.dark\:border-amber-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(255 251 235/var(--tw-border-opacity,1))}.dark\:border-amber-50\/0:is(.dark *){border-color:rgba(255,251,235,0)}.dark\:border-amber-50\/10:is(.dark *){border-color:rgba(255,251,235,.1)}.dark\:border-amber-50\/100:is(.dark *){border-color:#fffbeb}.dark\:border-amber-50\/15:is(.dark *){border-color:rgba(255,251,235,.15)}.dark\:border-amber-50\/20:is(.dark *){border-color:rgba(255,251,235,.2)}.dark\:border-amber-50\/25:is(.dark *){border-color:rgba(255,251,235,.25)}.dark\:border-amber-50\/30:is(.dark *){border-color:rgba(255,251,235,.3)}.dark\:border-amber-50\/35:is(.dark *){border-color:rgba(255,251,235,.35)}.dark\:border-amber-50\/40:is(.dark *){border-color:rgba(255,251,235,.4)}.dark\:border-amber-50\/45:is(.dark *){border-color:rgba(255,251,235,.45)}.dark\:border-amber-50\/5:is(.dark *){border-color:rgba(255,251,235,.05)}.dark\:border-amber-50\/50:is(.dark *){border-color:rgba(255,251,235,.5)}.dark\:border-amber-50\/55:is(.dark *){border-color:rgba(255,251,235,.55)}.dark\:border-amber-50\/60:is(.dark *){border-color:rgba(255,251,235,.6)}.dark\:border-amber-50\/65:is(.dark *){border-color:rgba(255,251,235,.65)}.dark\:border-amber-50\/70:is(.dark *){border-color:rgba(255,251,235,.7)}.dark\:border-amber-50\/75:is(.dark *){border-color:rgba(255,251,235,.75)}.dark\:border-amber-50\/80:is(.dark *){border-color:rgba(255,251,235,.8)}.dark\:border-amber-50\/85:is(.dark *){border-color:rgba(255,251,235,.85)}.dark\:border-amber-50\/90:is(.dark *){border-color:rgba(255,251,235,.9)}.dark\:border-amber-50\/95:is(.dark *){border-color:rgba(255,251,235,.95)}.dark\:border-amber-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(245 158 11/var(--tw-border-opacity,1))}.dark\:border-amber-500\/0:is(.dark *){border-color:rgba(245,158,11,0)}.dark\:border-amber-500\/10:is(.dark *){border-color:rgba(245,158,11,.1)}.dark\:border-amber-500\/100:is(.dark *){border-color:#f59e0b}.dark\:border-amber-500\/15:is(.dark *){border-color:rgba(245,158,11,.15)}.dark\:border-amber-500\/20:is(.dark *){border-color:rgba(245,158,11,.2)}.dark\:border-amber-500\/25:is(.dark *){border-color:rgba(245,158,11,.25)}.dark\:border-amber-500\/30:is(.dark *){border-color:rgba(245,158,11,.3)}.dark\:border-amber-500\/35:is(.dark *){border-color:rgba(245,158,11,.35)}.dark\:border-amber-500\/40:is(.dark *){border-color:rgba(245,158,11,.4)}.dark\:border-amber-500\/45:is(.dark *){border-color:rgba(245,158,11,.45)}.dark\:border-amber-500\/5:is(.dark *){border-color:rgba(245,158,11,.05)}.dark\:border-amber-500\/50:is(.dark *){border-color:rgba(245,158,11,.5)}.dark\:border-amber-500\/55:is(.dark *){border-color:rgba(245,158,11,.55)}.dark\:border-amber-500\/60:is(.dark *){border-color:rgba(245,158,11,.6)}.dark\:border-amber-500\/65:is(.dark *){border-color:rgba(245,158,11,.65)}.dark\:border-amber-500\/70:is(.dark *){border-color:rgba(245,158,11,.7)}.dark\:border-amber-500\/75:is(.dark *){border-color:rgba(245,158,11,.75)}.dark\:border-amber-500\/80:is(.dark *){border-color:rgba(245,158,11,.8)}.dark\:border-amber-500\/85:is(.dark *){border-color:rgba(245,158,11,.85)}.dark\:border-amber-500\/90:is(.dark *){border-color:rgba(245,158,11,.9)}.dark\:border-amber-500\/95:is(.dark *){border-color:rgba(245,158,11,.95)}.dark\:border-amber-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(69 26 3/var(--tw-border-opacity,1))}.dark\:border-amber-950\/0:is(.dark *){border-color:rgba(69,26,3,0)}.dark\:border-amber-950\/10:is(.dark *){border-color:rgba(69,26,3,.1)}.dark\:border-amber-950\/100:is(.dark *){border-color:#451a03}.dark\:border-amber-950\/15:is(.dark *){border-color:rgba(69,26,3,.15)}.dark\:border-amber-950\/20:is(.dark *){border-color:rgba(69,26,3,.2)}.dark\:border-amber-950\/25:is(.dark *){border-color:rgba(69,26,3,.25)}.dark\:border-amber-950\/30:is(.dark *){border-color:rgba(69,26,3,.3)}.dark\:border-amber-950\/35:is(.dark *){border-color:rgba(69,26,3,.35)}.dark\:border-amber-950\/40:is(.dark *){border-color:rgba(69,26,3,.4)}.dark\:border-amber-950\/45:is(.dark *){border-color:rgba(69,26,3,.45)}.dark\:border-amber-950\/5:is(.dark *){border-color:rgba(69,26,3,.05)}.dark\:border-amber-950\/50:is(.dark *){border-color:rgba(69,26,3,.5)}.dark\:border-amber-950\/55:is(.dark *){border-color:rgba(69,26,3,.55)}.dark\:border-amber-950\/60:is(.dark *){border-color:rgba(69,26,3,.6)}.dark\:border-amber-950\/65:is(.dark *){border-color:rgba(69,26,3,.65)}.dark\:border-amber-950\/70:is(.dark *){border-color:rgba(69,26,3,.7)}.dark\:border-amber-950\/75:is(.dark *){border-color:rgba(69,26,3,.75)}.dark\:border-amber-950\/80:is(.dark *){border-color:rgba(69,26,3,.8)}.dark\:border-amber-950\/85:is(.dark *){border-color:rgba(69,26,3,.85)}.dark\:border-amber-950\/90:is(.dark *){border-color:rgba(69,26,3,.9)}.dark\:border-amber-950\/95:is(.dark *){border-color:rgba(69,26,3,.95)}.dark\:border-blue-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(219 234 254/var(--tw-border-opacity,1))}.dark\:border-blue-100\/0:is(.dark *){border-color:rgba(219,234,254,0)}.dark\:border-blue-100\/10:is(.dark *){border-color:rgba(219,234,254,.1)}.dark\:border-blue-100\/100:is(.dark *){border-color:#dbeafe}.dark\:border-blue-100\/15:is(.dark *){border-color:rgba(219,234,254,.15)}.dark\:border-blue-100\/20:is(.dark *){border-color:rgba(219,234,254,.2)}.dark\:border-blue-100\/25:is(.dark *){border-color:rgba(219,234,254,.25)}.dark\:border-blue-100\/30:is(.dark *){border-color:rgba(219,234,254,.3)}.dark\:border-blue-100\/35:is(.dark *){border-color:rgba(219,234,254,.35)}.dark\:border-blue-100\/40:is(.dark *){border-color:rgba(219,234,254,.4)}.dark\:border-blue-100\/45:is(.dark *){border-color:rgba(219,234,254,.45)}.dark\:border-blue-100\/5:is(.dark *){border-color:rgba(219,234,254,.05)}.dark\:border-blue-100\/50:is(.dark *){border-color:rgba(219,234,254,.5)}.dark\:border-blue-100\/55:is(.dark *){border-color:rgba(219,234,254,.55)}.dark\:border-blue-100\/60:is(.dark *){border-color:rgba(219,234,254,.6)}.dark\:border-blue-100\/65:is(.dark *){border-color:rgba(219,234,254,.65)}.dark\:border-blue-100\/70:is(.dark *){border-color:rgba(219,234,254,.7)}.dark\:border-blue-100\/75:is(.dark *){border-color:rgba(219,234,254,.75)}.dark\:border-blue-100\/80:is(.dark *){border-color:rgba(219,234,254,.8)}.dark\:border-blue-100\/85:is(.dark *){border-color:rgba(219,234,254,.85)}.dark\:border-blue-100\/90:is(.dark *){border-color:rgba(219,234,254,.9)}.dark\:border-blue-100\/95:is(.dark *){border-color:rgba(219,234,254,.95)}.dark\:border-blue-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(191 219 254/var(--tw-border-opacity,1))}.dark\:border-blue-200\/0:is(.dark *){border-color:rgba(191,219,254,0)}.dark\:border-blue-200\/10:is(.dark *){border-color:rgba(191,219,254,.1)}.dark\:border-blue-200\/100:is(.dark *){border-color:#bfdbfe}.dark\:border-blue-200\/15:is(.dark *){border-color:rgba(191,219,254,.15)}.dark\:border-blue-200\/20:is(.dark *){border-color:rgba(191,219,254,.2)}.dark\:border-blue-200\/25:is(.dark *){border-color:rgba(191,219,254,.25)}.dark\:border-blue-200\/30:is(.dark *){border-color:rgba(191,219,254,.3)}.dark\:border-blue-200\/35:is(.dark *){border-color:rgba(191,219,254,.35)}.dark\:border-blue-200\/40:is(.dark *){border-color:rgba(191,219,254,.4)}.dark\:border-blue-200\/45:is(.dark *){border-color:rgba(191,219,254,.45)}.dark\:border-blue-200\/5:is(.dark *){border-color:rgba(191,219,254,.05)}.dark\:border-blue-200\/50:is(.dark *){border-color:rgba(191,219,254,.5)}.dark\:border-blue-200\/55:is(.dark *){border-color:rgba(191,219,254,.55)}.dark\:border-blue-200\/60:is(.dark *){border-color:rgba(191,219,254,.6)}.dark\:border-blue-200\/65:is(.dark *){border-color:rgba(191,219,254,.65)}.dark\:border-blue-200\/70:is(.dark *){border-color:rgba(191,219,254,.7)}.dark\:border-blue-200\/75:is(.dark *){border-color:rgba(191,219,254,.75)}.dark\:border-blue-200\/80:is(.dark *){border-color:rgba(191,219,254,.8)}.dark\:border-blue-200\/85:is(.dark *){border-color:rgba(191,219,254,.85)}.dark\:border-blue-200\/90:is(.dark *){border-color:rgba(191,219,254,.9)}.dark\:border-blue-200\/95:is(.dark *){border-color:rgba(191,219,254,.95)}.dark\:border-blue-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(239 246 255/var(--tw-border-opacity,1))}.dark\:border-blue-50\/0:is(.dark *){border-color:rgba(239,246,255,0)}.dark\:border-blue-50\/10:is(.dark *){border-color:rgba(239,246,255,.1)}.dark\:border-blue-50\/100:is(.dark *){border-color:#eff6ff}.dark\:border-blue-50\/15:is(.dark *){border-color:rgba(239,246,255,.15)}.dark\:border-blue-50\/20:is(.dark *){border-color:rgba(239,246,255,.2)}.dark\:border-blue-50\/25:is(.dark *){border-color:rgba(239,246,255,.25)}.dark\:border-blue-50\/30:is(.dark *){border-color:rgba(239,246,255,.3)}.dark\:border-blue-50\/35:is(.dark *){border-color:rgba(239,246,255,.35)}.dark\:border-blue-50\/40:is(.dark *){border-color:rgba(239,246,255,.4)}.dark\:border-blue-50\/45:is(.dark *){border-color:rgba(239,246,255,.45)}.dark\:border-blue-50\/5:is(.dark *){border-color:rgba(239,246,255,.05)}.dark\:border-blue-50\/50:is(.dark *){border-color:rgba(239,246,255,.5)}.dark\:border-blue-50\/55:is(.dark *){border-color:rgba(239,246,255,.55)}.dark\:border-blue-50\/60:is(.dark *){border-color:rgba(239,246,255,.6)}.dark\:border-blue-50\/65:is(.dark *){border-color:rgba(239,246,255,.65)}.dark\:border-blue-50\/70:is(.dark *){border-color:rgba(239,246,255,.7)}.dark\:border-blue-50\/75:is(.dark *){border-color:rgba(239,246,255,.75)}.dark\:border-blue-50\/80:is(.dark *){border-color:rgba(239,246,255,.8)}.dark\:border-blue-50\/85:is(.dark *){border-color:rgba(239,246,255,.85)}.dark\:border-blue-50\/90:is(.dark *){border-color:rgba(239,246,255,.9)}.dark\:border-blue-50\/95:is(.dark *){border-color:rgba(239,246,255,.95)}.dark\:border-blue-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(59 130 246/var(--tw-border-opacity,1))}.dark\:border-blue-500\/0:is(.dark *){border-color:rgba(59,130,246,0)}.dark\:border-blue-500\/10:is(.dark *){border-color:rgba(59,130,246,.1)}.dark\:border-blue-500\/100:is(.dark *){border-color:#3b82f6}.dark\:border-blue-500\/15:is(.dark *){border-color:rgba(59,130,246,.15)}.dark\:border-blue-500\/20:is(.dark *){border-color:rgba(59,130,246,.2)}.dark\:border-blue-500\/25:is(.dark *){border-color:rgba(59,130,246,.25)}.dark\:border-blue-500\/30:is(.dark *){border-color:rgba(59,130,246,.3)}.dark\:border-blue-500\/35:is(.dark *){border-color:rgba(59,130,246,.35)}.dark\:border-blue-500\/40:is(.dark *){border-color:rgba(59,130,246,.4)}.dark\:border-blue-500\/45:is(.dark *){border-color:rgba(59,130,246,.45)}.dark\:border-blue-500\/5:is(.dark *){border-color:rgba(59,130,246,.05)}.dark\:border-blue-500\/50:is(.dark *){border-color:rgba(59,130,246,.5)}.dark\:border-blue-500\/55:is(.dark *){border-color:rgba(59,130,246,.55)}.dark\:border-blue-500\/60:is(.dark *){border-color:rgba(59,130,246,.6)}.dark\:border-blue-500\/65:is(.dark *){border-color:rgba(59,130,246,.65)}.dark\:border-blue-500\/70:is(.dark *){border-color:rgba(59,130,246,.7)}.dark\:border-blue-500\/75:is(.dark *){border-color:rgba(59,130,246,.75)}.dark\:border-blue-500\/80:is(.dark *){border-color:rgba(59,130,246,.8)}.dark\:border-blue-500\/85:is(.dark *){border-color:rgba(59,130,246,.85)}.dark\:border-blue-500\/90:is(.dark *){border-color:rgba(59,130,246,.9)}.dark\:border-blue-500\/95:is(.dark *){border-color:rgba(59,130,246,.95)}.dark\:border-blue-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(23 37 84/var(--tw-border-opacity,1))}.dark\:border-blue-950\/0:is(.dark *){border-color:rgba(23,37,84,0)}.dark\:border-blue-950\/10:is(.dark *){border-color:rgba(23,37,84,.1)}.dark\:border-blue-950\/100:is(.dark *){border-color:#172554}.dark\:border-blue-950\/15:is(.dark *){border-color:rgba(23,37,84,.15)}.dark\:border-blue-950\/20:is(.dark *){border-color:rgba(23,37,84,.2)}.dark\:border-blue-950\/25:is(.dark *){border-color:rgba(23,37,84,.25)}.dark\:border-blue-950\/30:is(.dark *){border-color:rgba(23,37,84,.3)}.dark\:border-blue-950\/35:is(.dark *){border-color:rgba(23,37,84,.35)}.dark\:border-blue-950\/40:is(.dark *){border-color:rgba(23,37,84,.4)}.dark\:border-blue-950\/45:is(.dark *){border-color:rgba(23,37,84,.45)}.dark\:border-blue-950\/5:is(.dark *){border-color:rgba(23,37,84,.05)}.dark\:border-blue-950\/50:is(.dark *){border-color:rgba(23,37,84,.5)}.dark\:border-blue-950\/55:is(.dark *){border-color:rgba(23,37,84,.55)}.dark\:border-blue-950\/60:is(.dark *){border-color:rgba(23,37,84,.6)}.dark\:border-blue-950\/65:is(.dark *){border-color:rgba(23,37,84,.65)}.dark\:border-blue-950\/70:is(.dark *){border-color:rgba(23,37,84,.7)}.dark\:border-blue-950\/75:is(.dark *){border-color:rgba(23,37,84,.75)}.dark\:border-blue-950\/80:is(.dark *){border-color:rgba(23,37,84,.8)}.dark\:border-blue-950\/85:is(.dark *){border-color:rgba(23,37,84,.85)}.dark\:border-blue-950\/90:is(.dark *){border-color:rgba(23,37,84,.9)}.dark\:border-blue-950\/95:is(.dark *){border-color:rgba(23,37,84,.95)}.dark\:border-cyan-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(207 250 254/var(--tw-border-opacity,1))}.dark\:border-cyan-100\/0:is(.dark *){border-color:rgba(207,250,254,0)}.dark\:border-cyan-100\/10:is(.dark *){border-color:rgba(207,250,254,.1)}.dark\:border-cyan-100\/100:is(.dark *){border-color:#cffafe}.dark\:border-cyan-100\/15:is(.dark *){border-color:rgba(207,250,254,.15)}.dark\:border-cyan-100\/20:is(.dark *){border-color:rgba(207,250,254,.2)}.dark\:border-cyan-100\/25:is(.dark *){border-color:rgba(207,250,254,.25)}.dark\:border-cyan-100\/30:is(.dark *){border-color:rgba(207,250,254,.3)}.dark\:border-cyan-100\/35:is(.dark *){border-color:rgba(207,250,254,.35)}.dark\:border-cyan-100\/40:is(.dark *){border-color:rgba(207,250,254,.4)}.dark\:border-cyan-100\/45:is(.dark *){border-color:rgba(207,250,254,.45)}.dark\:border-cyan-100\/5:is(.dark *){border-color:rgba(207,250,254,.05)}.dark\:border-cyan-100\/50:is(.dark *){border-color:rgba(207,250,254,.5)}.dark\:border-cyan-100\/55:is(.dark *){border-color:rgba(207,250,254,.55)}.dark\:border-cyan-100\/60:is(.dark *){border-color:rgba(207,250,254,.6)}.dark\:border-cyan-100\/65:is(.dark *){border-color:rgba(207,250,254,.65)}.dark\:border-cyan-100\/70:is(.dark *){border-color:rgba(207,250,254,.7)}.dark\:border-cyan-100\/75:is(.dark *){border-color:rgba(207,250,254,.75)}.dark\:border-cyan-100\/80:is(.dark *){border-color:rgba(207,250,254,.8)}.dark\:border-cyan-100\/85:is(.dark *){border-color:rgba(207,250,254,.85)}.dark\:border-cyan-100\/90:is(.dark *){border-color:rgba(207,250,254,.9)}.dark\:border-cyan-100\/95:is(.dark *){border-color:rgba(207,250,254,.95)}.dark\:border-cyan-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(165 243 252/var(--tw-border-opacity,1))}.dark\:border-cyan-200\/0:is(.dark *){border-color:rgba(165,243,252,0)}.dark\:border-cyan-200\/10:is(.dark *){border-color:rgba(165,243,252,.1)}.dark\:border-cyan-200\/100:is(.dark *){border-color:#a5f3fc}.dark\:border-cyan-200\/15:is(.dark *){border-color:rgba(165,243,252,.15)}.dark\:border-cyan-200\/20:is(.dark *){border-color:rgba(165,243,252,.2)}.dark\:border-cyan-200\/25:is(.dark *){border-color:rgba(165,243,252,.25)}.dark\:border-cyan-200\/30:is(.dark *){border-color:rgba(165,243,252,.3)}.dark\:border-cyan-200\/35:is(.dark *){border-color:rgba(165,243,252,.35)}.dark\:border-cyan-200\/40:is(.dark *){border-color:rgba(165,243,252,.4)}.dark\:border-cyan-200\/45:is(.dark *){border-color:rgba(165,243,252,.45)}.dark\:border-cyan-200\/5:is(.dark *){border-color:rgba(165,243,252,.05)}.dark\:border-cyan-200\/50:is(.dark *){border-color:rgba(165,243,252,.5)}.dark\:border-cyan-200\/55:is(.dark *){border-color:rgba(165,243,252,.55)}.dark\:border-cyan-200\/60:is(.dark *){border-color:rgba(165,243,252,.6)}.dark\:border-cyan-200\/65:is(.dark *){border-color:rgba(165,243,252,.65)}.dark\:border-cyan-200\/70:is(.dark *){border-color:rgba(165,243,252,.7)}.dark\:border-cyan-200\/75:is(.dark *){border-color:rgba(165,243,252,.75)}.dark\:border-cyan-200\/80:is(.dark *){border-color:rgba(165,243,252,.8)}.dark\:border-cyan-200\/85:is(.dark *){border-color:rgba(165,243,252,.85)}.dark\:border-cyan-200\/90:is(.dark *){border-color:rgba(165,243,252,.9)}.dark\:border-cyan-200\/95:is(.dark *){border-color:rgba(165,243,252,.95)}.dark\:border-cyan-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(236 254 255/var(--tw-border-opacity,1))}.dark\:border-cyan-50\/0:is(.dark *){border-color:rgba(236,254,255,0)}.dark\:border-cyan-50\/10:is(.dark *){border-color:rgba(236,254,255,.1)}.dark\:border-cyan-50\/100:is(.dark *){border-color:#ecfeff}.dark\:border-cyan-50\/15:is(.dark *){border-color:rgba(236,254,255,.15)}.dark\:border-cyan-50\/20:is(.dark *){border-color:rgba(236,254,255,.2)}.dark\:border-cyan-50\/25:is(.dark *){border-color:rgba(236,254,255,.25)}.dark\:border-cyan-50\/30:is(.dark *){border-color:rgba(236,254,255,.3)}.dark\:border-cyan-50\/35:is(.dark *){border-color:rgba(236,254,255,.35)}.dark\:border-cyan-50\/40:is(.dark *){border-color:rgba(236,254,255,.4)}.dark\:border-cyan-50\/45:is(.dark *){border-color:rgba(236,254,255,.45)}.dark\:border-cyan-50\/5:is(.dark *){border-color:rgba(236,254,255,.05)}.dark\:border-cyan-50\/50:is(.dark *){border-color:rgba(236,254,255,.5)}.dark\:border-cyan-50\/55:is(.dark *){border-color:rgba(236,254,255,.55)}.dark\:border-cyan-50\/60:is(.dark *){border-color:rgba(236,254,255,.6)}.dark\:border-cyan-50\/65:is(.dark *){border-color:rgba(236,254,255,.65)}.dark\:border-cyan-50\/70:is(.dark *){border-color:rgba(236,254,255,.7)}.dark\:border-cyan-50\/75:is(.dark *){border-color:rgba(236,254,255,.75)}.dark\:border-cyan-50\/80:is(.dark *){border-color:rgba(236,254,255,.8)}.dark\:border-cyan-50\/85:is(.dark *){border-color:rgba(236,254,255,.85)}.dark\:border-cyan-50\/90:is(.dark *){border-color:rgba(236,254,255,.9)}.dark\:border-cyan-50\/95:is(.dark *){border-color:rgba(236,254,255,.95)}.dark\:border-cyan-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(6 182 212/var(--tw-border-opacity,1))}.dark\:border-cyan-500\/0:is(.dark *){border-color:rgba(6,182,212,0)}.dark\:border-cyan-500\/10:is(.dark *){border-color:rgba(6,182,212,.1)}.dark\:border-cyan-500\/100:is(.dark *){border-color:#06b6d4}.dark\:border-cyan-500\/15:is(.dark *){border-color:rgba(6,182,212,.15)}.dark\:border-cyan-500\/20:is(.dark *){border-color:rgba(6,182,212,.2)}.dark\:border-cyan-500\/25:is(.dark *){border-color:rgba(6,182,212,.25)}.dark\:border-cyan-500\/30:is(.dark *){border-color:rgba(6,182,212,.3)}.dark\:border-cyan-500\/35:is(.dark *){border-color:rgba(6,182,212,.35)}.dark\:border-cyan-500\/40:is(.dark *){border-color:rgba(6,182,212,.4)}.dark\:border-cyan-500\/45:is(.dark *){border-color:rgba(6,182,212,.45)}.dark\:border-cyan-500\/5:is(.dark *){border-color:rgba(6,182,212,.05)}.dark\:border-cyan-500\/50:is(.dark *){border-color:rgba(6,182,212,.5)}.dark\:border-cyan-500\/55:is(.dark *){border-color:rgba(6,182,212,.55)}.dark\:border-cyan-500\/60:is(.dark *){border-color:rgba(6,182,212,.6)}.dark\:border-cyan-500\/65:is(.dark *){border-color:rgba(6,182,212,.65)}.dark\:border-cyan-500\/70:is(.dark *){border-color:rgba(6,182,212,.7)}.dark\:border-cyan-500\/75:is(.dark *){border-color:rgba(6,182,212,.75)}.dark\:border-cyan-500\/80:is(.dark *){border-color:rgba(6,182,212,.8)}.dark\:border-cyan-500\/85:is(.dark *){border-color:rgba(6,182,212,.85)}.dark\:border-cyan-500\/90:is(.dark *){border-color:rgba(6,182,212,.9)}.dark\:border-cyan-500\/95:is(.dark *){border-color:rgba(6,182,212,.95)}.dark\:border-cyan-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(8 51 68/var(--tw-border-opacity,1))}.dark\:border-cyan-950\/0:is(.dark *){border-color:rgba(8,51,68,0)}.dark\:border-cyan-950\/10:is(.dark *){border-color:rgba(8,51,68,.1)}.dark\:border-cyan-950\/100:is(.dark *){border-color:#083344}.dark\:border-cyan-950\/15:is(.dark *){border-color:rgba(8,51,68,.15)}.dark\:border-cyan-950\/20:is(.dark *){border-color:rgba(8,51,68,.2)}.dark\:border-cyan-950\/25:is(.dark *){border-color:rgba(8,51,68,.25)}.dark\:border-cyan-950\/30:is(.dark *){border-color:rgba(8,51,68,.3)}.dark\:border-cyan-950\/35:is(.dark *){border-color:rgba(8,51,68,.35)}.dark\:border-cyan-950\/40:is(.dark *){border-color:rgba(8,51,68,.4)}.dark\:border-cyan-950\/45:is(.dark *){border-color:rgba(8,51,68,.45)}.dark\:border-cyan-950\/5:is(.dark *){border-color:rgba(8,51,68,.05)}.dark\:border-cyan-950\/50:is(.dark *){border-color:rgba(8,51,68,.5)}.dark\:border-cyan-950\/55:is(.dark *){border-color:rgba(8,51,68,.55)}.dark\:border-cyan-950\/60:is(.dark *){border-color:rgba(8,51,68,.6)}.dark\:border-cyan-950\/65:is(.dark *){border-color:rgba(8,51,68,.65)}.dark\:border-cyan-950\/70:is(.dark *){border-color:rgba(8,51,68,.7)}.dark\:border-cyan-950\/75:is(.dark *){border-color:rgba(8,51,68,.75)}.dark\:border-cyan-950\/80:is(.dark *){border-color:rgba(8,51,68,.8)}.dark\:border-cyan-950\/85:is(.dark *){border-color:rgba(8,51,68,.85)}.dark\:border-cyan-950\/90:is(.dark *){border-color:rgba(8,51,68,.9)}.dark\:border-cyan-950\/95:is(.dark *){border-color:rgba(8,51,68,.95)}.dark\:border-destructive:is(.dark *){border-color:hsl(var(--destructive))}.dark\:border-emerald-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(209 250 229/var(--tw-border-opacity,1))}.dark\:border-emerald-100\/0:is(.dark *){border-color:rgba(209,250,229,0)}.dark\:border-emerald-100\/10:is(.dark *){border-color:rgba(209,250,229,.1)}.dark\:border-emerald-100\/100:is(.dark *){border-color:#d1fae5}.dark\:border-emerald-100\/15:is(.dark *){border-color:rgba(209,250,229,.15)}.dark\:border-emerald-100\/20:is(.dark *){border-color:rgba(209,250,229,.2)}.dark\:border-emerald-100\/25:is(.dark *){border-color:rgba(209,250,229,.25)}.dark\:border-emerald-100\/30:is(.dark *){border-color:rgba(209,250,229,.3)}.dark\:border-emerald-100\/35:is(.dark *){border-color:rgba(209,250,229,.35)}.dark\:border-emerald-100\/40:is(.dark *){border-color:rgba(209,250,229,.4)}.dark\:border-emerald-100\/45:is(.dark *){border-color:rgba(209,250,229,.45)}.dark\:border-emerald-100\/5:is(.dark *){border-color:rgba(209,250,229,.05)}.dark\:border-emerald-100\/50:is(.dark *){border-color:rgba(209,250,229,.5)}.dark\:border-emerald-100\/55:is(.dark *){border-color:rgba(209,250,229,.55)}.dark\:border-emerald-100\/60:is(.dark *){border-color:rgba(209,250,229,.6)}.dark\:border-emerald-100\/65:is(.dark *){border-color:rgba(209,250,229,.65)}.dark\:border-emerald-100\/70:is(.dark *){border-color:rgba(209,250,229,.7)}.dark\:border-emerald-100\/75:is(.dark *){border-color:rgba(209,250,229,.75)}.dark\:border-emerald-100\/80:is(.dark *){border-color:rgba(209,250,229,.8)}.dark\:border-emerald-100\/85:is(.dark *){border-color:rgba(209,250,229,.85)}.dark\:border-emerald-100\/90:is(.dark *){border-color:rgba(209,250,229,.9)}.dark\:border-emerald-100\/95:is(.dark *){border-color:rgba(209,250,229,.95)}.dark\:border-emerald-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(167 243 208/var(--tw-border-opacity,1))}.dark\:border-emerald-200\/0:is(.dark *){border-color:rgba(167,243,208,0)}.dark\:border-emerald-200\/10:is(.dark *){border-color:rgba(167,243,208,.1)}.dark\:border-emerald-200\/100:is(.dark *){border-color:#a7f3d0}.dark\:border-emerald-200\/15:is(.dark *){border-color:rgba(167,243,208,.15)}.dark\:border-emerald-200\/20:is(.dark *){border-color:rgba(167,243,208,.2)}.dark\:border-emerald-200\/25:is(.dark *){border-color:rgba(167,243,208,.25)}.dark\:border-emerald-200\/30:is(.dark *){border-color:rgba(167,243,208,.3)}.dark\:border-emerald-200\/35:is(.dark *){border-color:rgba(167,243,208,.35)}.dark\:border-emerald-200\/40:is(.dark *){border-color:rgba(167,243,208,.4)}.dark\:border-emerald-200\/45:is(.dark *){border-color:rgba(167,243,208,.45)}.dark\:border-emerald-200\/5:is(.dark *){border-color:rgba(167,243,208,.05)}.dark\:border-emerald-200\/50:is(.dark *){border-color:rgba(167,243,208,.5)}.dark\:border-emerald-200\/55:is(.dark *){border-color:rgba(167,243,208,.55)}.dark\:border-emerald-200\/60:is(.dark *){border-color:rgba(167,243,208,.6)}.dark\:border-emerald-200\/65:is(.dark *){border-color:rgba(167,243,208,.65)}.dark\:border-emerald-200\/70:is(.dark *){border-color:rgba(167,243,208,.7)}.dark\:border-emerald-200\/75:is(.dark *){border-color:rgba(167,243,208,.75)}.dark\:border-emerald-200\/80:is(.dark *){border-color:rgba(167,243,208,.8)}.dark\:border-emerald-200\/85:is(.dark *){border-color:rgba(167,243,208,.85)}.dark\:border-emerald-200\/90:is(.dark *){border-color:rgba(167,243,208,.9)}.dark\:border-emerald-200\/95:is(.dark *){border-color:rgba(167,243,208,.95)}.dark\:border-emerald-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(236 253 245/var(--tw-border-opacity,1))}.dark\:border-emerald-50\/0:is(.dark *){border-color:rgba(236,253,245,0)}.dark\:border-emerald-50\/10:is(.dark *){border-color:rgba(236,253,245,.1)}.dark\:border-emerald-50\/100:is(.dark *){border-color:#ecfdf5}.dark\:border-emerald-50\/15:is(.dark *){border-color:rgba(236,253,245,.15)}.dark\:border-emerald-50\/20:is(.dark *){border-color:rgba(236,253,245,.2)}.dark\:border-emerald-50\/25:is(.dark *){border-color:rgba(236,253,245,.25)}.dark\:border-emerald-50\/30:is(.dark *){border-color:rgba(236,253,245,.3)}.dark\:border-emerald-50\/35:is(.dark *){border-color:rgba(236,253,245,.35)}.dark\:border-emerald-50\/40:is(.dark *){border-color:rgba(236,253,245,.4)}.dark\:border-emerald-50\/45:is(.dark *){border-color:rgba(236,253,245,.45)}.dark\:border-emerald-50\/5:is(.dark *){border-color:rgba(236,253,245,.05)}.dark\:border-emerald-50\/50:is(.dark *){border-color:rgba(236,253,245,.5)}.dark\:border-emerald-50\/55:is(.dark *){border-color:rgba(236,253,245,.55)}.dark\:border-emerald-50\/60:is(.dark *){border-color:rgba(236,253,245,.6)}.dark\:border-emerald-50\/65:is(.dark *){border-color:rgba(236,253,245,.65)}.dark\:border-emerald-50\/70:is(.dark *){border-color:rgba(236,253,245,.7)}.dark\:border-emerald-50\/75:is(.dark *){border-color:rgba(236,253,245,.75)}.dark\:border-emerald-50\/80:is(.dark *){border-color:rgba(236,253,245,.8)}.dark\:border-emerald-50\/85:is(.dark *){border-color:rgba(236,253,245,.85)}.dark\:border-emerald-50\/90:is(.dark *){border-color:rgba(236,253,245,.9)}.dark\:border-emerald-50\/95:is(.dark *){border-color:rgba(236,253,245,.95)}.dark\:border-emerald-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(16 185 129/var(--tw-border-opacity,1))}.dark\:border-emerald-500\/0:is(.dark *){border-color:rgba(16,185,129,0)}.dark\:border-emerald-500\/10:is(.dark *){border-color:rgba(16,185,129,.1)}.dark\:border-emerald-500\/100:is(.dark *){border-color:#10b981}.dark\:border-emerald-500\/15:is(.dark *){border-color:rgba(16,185,129,.15)}.dark\:border-emerald-500\/20:is(.dark *){border-color:rgba(16,185,129,.2)}.dark\:border-emerald-500\/25:is(.dark *){border-color:rgba(16,185,129,.25)}.dark\:border-emerald-500\/30:is(.dark *){border-color:rgba(16,185,129,.3)}.dark\:border-emerald-500\/35:is(.dark *){border-color:rgba(16,185,129,.35)}.dark\:border-emerald-500\/40:is(.dark *){border-color:rgba(16,185,129,.4)}.dark\:border-emerald-500\/45:is(.dark *){border-color:rgba(16,185,129,.45)}.dark\:border-emerald-500\/5:is(.dark *){border-color:rgba(16,185,129,.05)}.dark\:border-emerald-500\/50:is(.dark *){border-color:rgba(16,185,129,.5)}.dark\:border-emerald-500\/55:is(.dark *){border-color:rgba(16,185,129,.55)}.dark\:border-emerald-500\/60:is(.dark *){border-color:rgba(16,185,129,.6)}.dark\:border-emerald-500\/65:is(.dark *){border-color:rgba(16,185,129,.65)}.dark\:border-emerald-500\/70:is(.dark *){border-color:rgba(16,185,129,.7)}.dark\:border-emerald-500\/75:is(.dark *){border-color:rgba(16,185,129,.75)}.dark\:border-emerald-500\/80:is(.dark *){border-color:rgba(16,185,129,.8)}.dark\:border-emerald-500\/85:is(.dark *){border-color:rgba(16,185,129,.85)}.dark\:border-emerald-500\/90:is(.dark *){border-color:rgba(16,185,129,.9)}.dark\:border-emerald-500\/95:is(.dark *){border-color:rgba(16,185,129,.95)}.dark\:border-emerald-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(2 44 34/var(--tw-border-opacity,1))}.dark\:border-emerald-950\/0:is(.dark *){border-color:rgba(2,44,34,0)}.dark\:border-emerald-950\/10:is(.dark *){border-color:rgba(2,44,34,.1)}.dark\:border-emerald-950\/100:is(.dark *){border-color:#022c22}.dark\:border-emerald-950\/15:is(.dark *){border-color:rgba(2,44,34,.15)}.dark\:border-emerald-950\/20:is(.dark *){border-color:rgba(2,44,34,.2)}.dark\:border-emerald-950\/25:is(.dark *){border-color:rgba(2,44,34,.25)}.dark\:border-emerald-950\/30:is(.dark *){border-color:rgba(2,44,34,.3)}.dark\:border-emerald-950\/35:is(.dark *){border-color:rgba(2,44,34,.35)}.dark\:border-emerald-950\/40:is(.dark *){border-color:rgba(2,44,34,.4)}.dark\:border-emerald-950\/45:is(.dark *){border-color:rgba(2,44,34,.45)}.dark\:border-emerald-950\/5:is(.dark *){border-color:rgba(2,44,34,.05)}.dark\:border-emerald-950\/50:is(.dark *){border-color:rgba(2,44,34,.5)}.dark\:border-emerald-950\/55:is(.dark *){border-color:rgba(2,44,34,.55)}.dark\:border-emerald-950\/60:is(.dark *){border-color:rgba(2,44,34,.6)}.dark\:border-emerald-950\/65:is(.dark *){border-color:rgba(2,44,34,.65)}.dark\:border-emerald-950\/70:is(.dark *){border-color:rgba(2,44,34,.7)}.dark\:border-emerald-950\/75:is(.dark *){border-color:rgba(2,44,34,.75)}.dark\:border-emerald-950\/80:is(.dark *){border-color:rgba(2,44,34,.8)}.dark\:border-emerald-950\/85:is(.dark *){border-color:rgba(2,44,34,.85)}.dark\:border-emerald-950\/90:is(.dark *){border-color:rgba(2,44,34,.9)}.dark\:border-emerald-950\/95:is(.dark *){border-color:rgba(2,44,34,.95)}.dark\:border-fuchsia-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(250 232 255/var(--tw-border-opacity,1))}.dark\:border-fuchsia-100\/0:is(.dark *){border-color:rgba(250,232,255,0)}.dark\:border-fuchsia-100\/10:is(.dark *){border-color:rgba(250,232,255,.1)}.dark\:border-fuchsia-100\/100:is(.dark *){border-color:#fae8ff}.dark\:border-fuchsia-100\/15:is(.dark *){border-color:rgba(250,232,255,.15)}.dark\:border-fuchsia-100\/20:is(.dark *){border-color:rgba(250,232,255,.2)}.dark\:border-fuchsia-100\/25:is(.dark *){border-color:rgba(250,232,255,.25)}.dark\:border-fuchsia-100\/30:is(.dark *){border-color:rgba(250,232,255,.3)}.dark\:border-fuchsia-100\/35:is(.dark *){border-color:rgba(250,232,255,.35)}.dark\:border-fuchsia-100\/40:is(.dark *){border-color:rgba(250,232,255,.4)}.dark\:border-fuchsia-100\/45:is(.dark *){border-color:rgba(250,232,255,.45)}.dark\:border-fuchsia-100\/5:is(.dark *){border-color:rgba(250,232,255,.05)}.dark\:border-fuchsia-100\/50:is(.dark *){border-color:rgba(250,232,255,.5)}.dark\:border-fuchsia-100\/55:is(.dark *){border-color:rgba(250,232,255,.55)}.dark\:border-fuchsia-100\/60:is(.dark *){border-color:rgba(250,232,255,.6)}.dark\:border-fuchsia-100\/65:is(.dark *){border-color:rgba(250,232,255,.65)}.dark\:border-fuchsia-100\/70:is(.dark *){border-color:rgba(250,232,255,.7)}.dark\:border-fuchsia-100\/75:is(.dark *){border-color:rgba(250,232,255,.75)}.dark\:border-fuchsia-100\/80:is(.dark *){border-color:rgba(250,232,255,.8)}.dark\:border-fuchsia-100\/85:is(.dark *){border-color:rgba(250,232,255,.85)}.dark\:border-fuchsia-100\/90:is(.dark *){border-color:rgba(250,232,255,.9)}.dark\:border-fuchsia-100\/95:is(.dark *){border-color:rgba(250,232,255,.95)}.dark\:border-fuchsia-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(245 208 254/var(--tw-border-opacity,1))}.dark\:border-fuchsia-200\/0:is(.dark *){border-color:rgba(245,208,254,0)}.dark\:border-fuchsia-200\/10:is(.dark *){border-color:rgba(245,208,254,.1)}.dark\:border-fuchsia-200\/100:is(.dark *){border-color:#f5d0fe}.dark\:border-fuchsia-200\/15:is(.dark *){border-color:rgba(245,208,254,.15)}.dark\:border-fuchsia-200\/20:is(.dark *){border-color:rgba(245,208,254,.2)}.dark\:border-fuchsia-200\/25:is(.dark *){border-color:rgba(245,208,254,.25)}.dark\:border-fuchsia-200\/30:is(.dark *){border-color:rgba(245,208,254,.3)}.dark\:border-fuchsia-200\/35:is(.dark *){border-color:rgba(245,208,254,.35)}.dark\:border-fuchsia-200\/40:is(.dark *){border-color:rgba(245,208,254,.4)}.dark\:border-fuchsia-200\/45:is(.dark *){border-color:rgba(245,208,254,.45)}.dark\:border-fuchsia-200\/5:is(.dark *){border-color:rgba(245,208,254,.05)}.dark\:border-fuchsia-200\/50:is(.dark *){border-color:rgba(245,208,254,.5)}.dark\:border-fuchsia-200\/55:is(.dark *){border-color:rgba(245,208,254,.55)}.dark\:border-fuchsia-200\/60:is(.dark *){border-color:rgba(245,208,254,.6)}.dark\:border-fuchsia-200\/65:is(.dark *){border-color:rgba(245,208,254,.65)}.dark\:border-fuchsia-200\/70:is(.dark *){border-color:rgba(245,208,254,.7)}.dark\:border-fuchsia-200\/75:is(.dark *){border-color:rgba(245,208,254,.75)}.dark\:border-fuchsia-200\/80:is(.dark *){border-color:rgba(245,208,254,.8)}.dark\:border-fuchsia-200\/85:is(.dark *){border-color:rgba(245,208,254,.85)}.dark\:border-fuchsia-200\/90:is(.dark *){border-color:rgba(245,208,254,.9)}.dark\:border-fuchsia-200\/95:is(.dark *){border-color:rgba(245,208,254,.95)}.dark\:border-fuchsia-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(253 244 255/var(--tw-border-opacity,1))}.dark\:border-fuchsia-50\/0:is(.dark *){border-color:rgba(253,244,255,0)}.dark\:border-fuchsia-50\/10:is(.dark *){border-color:rgba(253,244,255,.1)}.dark\:border-fuchsia-50\/100:is(.dark *){border-color:#fdf4ff}.dark\:border-fuchsia-50\/15:is(.dark *){border-color:rgba(253,244,255,.15)}.dark\:border-fuchsia-50\/20:is(.dark *){border-color:rgba(253,244,255,.2)}.dark\:border-fuchsia-50\/25:is(.dark *){border-color:rgba(253,244,255,.25)}.dark\:border-fuchsia-50\/30:is(.dark *){border-color:rgba(253,244,255,.3)}.dark\:border-fuchsia-50\/35:is(.dark *){border-color:rgba(253,244,255,.35)}.dark\:border-fuchsia-50\/40:is(.dark *){border-color:rgba(253,244,255,.4)}.dark\:border-fuchsia-50\/45:is(.dark *){border-color:rgba(253,244,255,.45)}.dark\:border-fuchsia-50\/5:is(.dark *){border-color:rgba(253,244,255,.05)}.dark\:border-fuchsia-50\/50:is(.dark *){border-color:rgba(253,244,255,.5)}.dark\:border-fuchsia-50\/55:is(.dark *){border-color:rgba(253,244,255,.55)}.dark\:border-fuchsia-50\/60:is(.dark *){border-color:rgba(253,244,255,.6)}.dark\:border-fuchsia-50\/65:is(.dark *){border-color:rgba(253,244,255,.65)}.dark\:border-fuchsia-50\/70:is(.dark *){border-color:rgba(253,244,255,.7)}.dark\:border-fuchsia-50\/75:is(.dark *){border-color:rgba(253,244,255,.75)}.dark\:border-fuchsia-50\/80:is(.dark *){border-color:rgba(253,244,255,.8)}.dark\:border-fuchsia-50\/85:is(.dark *){border-color:rgba(253,244,255,.85)}.dark\:border-fuchsia-50\/90:is(.dark *){border-color:rgba(253,244,255,.9)}.dark\:border-fuchsia-50\/95:is(.dark *){border-color:rgba(253,244,255,.95)}.dark\:border-fuchsia-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(217 70 239/var(--tw-border-opacity,1))}.dark\:border-fuchsia-500\/0:is(.dark *){border-color:rgba(217,70,239,0)}.dark\:border-fuchsia-500\/10:is(.dark *){border-color:rgba(217,70,239,.1)}.dark\:border-fuchsia-500\/100:is(.dark *){border-color:#d946ef}.dark\:border-fuchsia-500\/15:is(.dark *){border-color:rgba(217,70,239,.15)}.dark\:border-fuchsia-500\/20:is(.dark *){border-color:rgba(217,70,239,.2)}.dark\:border-fuchsia-500\/25:is(.dark *){border-color:rgba(217,70,239,.25)}.dark\:border-fuchsia-500\/30:is(.dark *){border-color:rgba(217,70,239,.3)}.dark\:border-fuchsia-500\/35:is(.dark *){border-color:rgba(217,70,239,.35)}.dark\:border-fuchsia-500\/40:is(.dark *){border-color:rgba(217,70,239,.4)}.dark\:border-fuchsia-500\/45:is(.dark *){border-color:rgba(217,70,239,.45)}.dark\:border-fuchsia-500\/5:is(.dark *){border-color:rgba(217,70,239,.05)}.dark\:border-fuchsia-500\/50:is(.dark *){border-color:rgba(217,70,239,.5)}.dark\:border-fuchsia-500\/55:is(.dark *){border-color:rgba(217,70,239,.55)}.dark\:border-fuchsia-500\/60:is(.dark *){border-color:rgba(217,70,239,.6)}.dark\:border-fuchsia-500\/65:is(.dark *){border-color:rgba(217,70,239,.65)}.dark\:border-fuchsia-500\/70:is(.dark *){border-color:rgba(217,70,239,.7)}.dark\:border-fuchsia-500\/75:is(.dark *){border-color:rgba(217,70,239,.75)}.dark\:border-fuchsia-500\/80:is(.dark *){border-color:rgba(217,70,239,.8)}.dark\:border-fuchsia-500\/85:is(.dark *){border-color:rgba(217,70,239,.85)}.dark\:border-fuchsia-500\/90:is(.dark *){border-color:rgba(217,70,239,.9)}.dark\:border-fuchsia-500\/95:is(.dark *){border-color:rgba(217,70,239,.95)}.dark\:border-fuchsia-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(74 4 78/var(--tw-border-opacity,1))}.dark\:border-fuchsia-950\/0:is(.dark *){border-color:rgba(74,4,78,0)}.dark\:border-fuchsia-950\/10:is(.dark *){border-color:rgba(74,4,78,.1)}.dark\:border-fuchsia-950\/100:is(.dark *){border-color:#4a044e}.dark\:border-fuchsia-950\/15:is(.dark *){border-color:rgba(74,4,78,.15)}.dark\:border-fuchsia-950\/20:is(.dark *){border-color:rgba(74,4,78,.2)}.dark\:border-fuchsia-950\/25:is(.dark *){border-color:rgba(74,4,78,.25)}.dark\:border-fuchsia-950\/30:is(.dark *){border-color:rgba(74,4,78,.3)}.dark\:border-fuchsia-950\/35:is(.dark *){border-color:rgba(74,4,78,.35)}.dark\:border-fuchsia-950\/40:is(.dark *){border-color:rgba(74,4,78,.4)}.dark\:border-fuchsia-950\/45:is(.dark *){border-color:rgba(74,4,78,.45)}.dark\:border-fuchsia-950\/5:is(.dark *){border-color:rgba(74,4,78,.05)}.dark\:border-fuchsia-950\/50:is(.dark *){border-color:rgba(74,4,78,.5)}.dark\:border-fuchsia-950\/55:is(.dark *){border-color:rgba(74,4,78,.55)}.dark\:border-fuchsia-950\/60:is(.dark *){border-color:rgba(74,4,78,.6)}.dark\:border-fuchsia-950\/65:is(.dark *){border-color:rgba(74,4,78,.65)}.dark\:border-fuchsia-950\/70:is(.dark *){border-color:rgba(74,4,78,.7)}.dark\:border-fuchsia-950\/75:is(.dark *){border-color:rgba(74,4,78,.75)}.dark\:border-fuchsia-950\/80:is(.dark *){border-color:rgba(74,4,78,.8)}.dark\:border-fuchsia-950\/85:is(.dark *){border-color:rgba(74,4,78,.85)}.dark\:border-fuchsia-950\/90:is(.dark *){border-color:rgba(74,4,78,.9)}.dark\:border-fuchsia-950\/95:is(.dark *){border-color:rgba(74,4,78,.95)}.dark\:border-gray-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(243 244 246/var(--tw-border-opacity,1))}.dark\:border-gray-100\/0:is(.dark *){border-color:rgba(243,244,246,0)}.dark\:border-gray-100\/10:is(.dark *){border-color:rgba(243,244,246,.1)}.dark\:border-gray-100\/100:is(.dark *){border-color:#f3f4f6}.dark\:border-gray-100\/15:is(.dark *){border-color:rgba(243,244,246,.15)}.dark\:border-gray-100\/20:is(.dark *){border-color:rgba(243,244,246,.2)}.dark\:border-gray-100\/25:is(.dark *){border-color:rgba(243,244,246,.25)}.dark\:border-gray-100\/30:is(.dark *){border-color:rgba(243,244,246,.3)}.dark\:border-gray-100\/35:is(.dark *){border-color:rgba(243,244,246,.35)}.dark\:border-gray-100\/40:is(.dark *){border-color:rgba(243,244,246,.4)}.dark\:border-gray-100\/45:is(.dark *){border-color:rgba(243,244,246,.45)}.dark\:border-gray-100\/5:is(.dark *){border-color:rgba(243,244,246,.05)}.dark\:border-gray-100\/50:is(.dark *){border-color:rgba(243,244,246,.5)}.dark\:border-gray-100\/55:is(.dark *){border-color:rgba(243,244,246,.55)}.dark\:border-gray-100\/60:is(.dark *){border-color:rgba(243,244,246,.6)}.dark\:border-gray-100\/65:is(.dark *){border-color:rgba(243,244,246,.65)}.dark\:border-gray-100\/70:is(.dark *){border-color:rgba(243,244,246,.7)}.dark\:border-gray-100\/75:is(.dark *){border-color:rgba(243,244,246,.75)}.dark\:border-gray-100\/80:is(.dark *){border-color:rgba(243,244,246,.8)}.dark\:border-gray-100\/85:is(.dark *){border-color:rgba(243,244,246,.85)}.dark\:border-gray-100\/90:is(.dark *){border-color:rgba(243,244,246,.9)}.dark\:border-gray-100\/95:is(.dark *){border-color:rgba(243,244,246,.95)}.dark\:border-gray-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(229 231 235/var(--tw-border-opacity,1))}.dark\:border-gray-200\/0:is(.dark *){border-color:rgba(229,231,235,0)}.dark\:border-gray-200\/10:is(.dark *){border-color:rgba(229,231,235,.1)}.dark\:border-gray-200\/100:is(.dark *){border-color:#e5e7eb}.dark\:border-gray-200\/15:is(.dark *){border-color:rgba(229,231,235,.15)}.dark\:border-gray-200\/20:is(.dark *){border-color:rgba(229,231,235,.2)}.dark\:border-gray-200\/25:is(.dark *){border-color:rgba(229,231,235,.25)}.dark\:border-gray-200\/30:is(.dark *){border-color:rgba(229,231,235,.3)}.dark\:border-gray-200\/35:is(.dark *){border-color:rgba(229,231,235,.35)}.dark\:border-gray-200\/40:is(.dark *){border-color:rgba(229,231,235,.4)}.dark\:border-gray-200\/45:is(.dark *){border-color:rgba(229,231,235,.45)}.dark\:border-gray-200\/5:is(.dark *){border-color:rgba(229,231,235,.05)}.dark\:border-gray-200\/50:is(.dark *){border-color:rgba(229,231,235,.5)}.dark\:border-gray-200\/55:is(.dark *){border-color:rgba(229,231,235,.55)}.dark\:border-gray-200\/60:is(.dark *){border-color:rgba(229,231,235,.6)}.dark\:border-gray-200\/65:is(.dark *){border-color:rgba(229,231,235,.65)}.dark\:border-gray-200\/70:is(.dark *){border-color:rgba(229,231,235,.7)}.dark\:border-gray-200\/75:is(.dark *){border-color:rgba(229,231,235,.75)}.dark\:border-gray-200\/80:is(.dark *){border-color:rgba(229,231,235,.8)}.dark\:border-gray-200\/85:is(.dark *){border-color:rgba(229,231,235,.85)}.dark\:border-gray-200\/90:is(.dark *){border-color:rgba(229,231,235,.9)}.dark\:border-gray-200\/95:is(.dark *){border-color:rgba(229,231,235,.95)}.dark\:border-gray-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(249 250 251/var(--tw-border-opacity,1))}.dark\:border-gray-50\/0:is(.dark *){border-color:rgba(249,250,251,0)}.dark\:border-gray-50\/10:is(.dark *){border-color:rgba(249,250,251,.1)}.dark\:border-gray-50\/100:is(.dark *){border-color:#f9fafb}.dark\:border-gray-50\/15:is(.dark *){border-color:rgba(249,250,251,.15)}.dark\:border-gray-50\/20:is(.dark *){border-color:rgba(249,250,251,.2)}.dark\:border-gray-50\/25:is(.dark *){border-color:rgba(249,250,251,.25)}.dark\:border-gray-50\/30:is(.dark *){border-color:rgba(249,250,251,.3)}.dark\:border-gray-50\/35:is(.dark *){border-color:rgba(249,250,251,.35)}.dark\:border-gray-50\/40:is(.dark *){border-color:rgba(249,250,251,.4)}.dark\:border-gray-50\/45:is(.dark *){border-color:rgba(249,250,251,.45)}.dark\:border-gray-50\/5:is(.dark *){border-color:rgba(249,250,251,.05)}.dark\:border-gray-50\/50:is(.dark *){border-color:rgba(249,250,251,.5)}.dark\:border-gray-50\/55:is(.dark *){border-color:rgba(249,250,251,.55)}.dark\:border-gray-50\/60:is(.dark *){border-color:rgba(249,250,251,.6)}.dark\:border-gray-50\/65:is(.dark *){border-color:rgba(249,250,251,.65)}.dark\:border-gray-50\/70:is(.dark *){border-color:rgba(249,250,251,.7)}.dark\:border-gray-50\/75:is(.dark *){border-color:rgba(249,250,251,.75)}.dark\:border-gray-50\/80:is(.dark *){border-color:rgba(249,250,251,.8)}.dark\:border-gray-50\/85:is(.dark *){border-color:rgba(249,250,251,.85)}.dark\:border-gray-50\/90:is(.dark *){border-color:rgba(249,250,251,.9)}.dark\:border-gray-50\/95:is(.dark *){border-color:rgba(249,250,251,.95)}.dark\:border-gray-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(107 114 128/var(--tw-border-opacity,1))}.dark\:border-gray-500\/0:is(.dark *){border-color:hsla(220,9%,46%,0)}.dark\:border-gray-500\/10:is(.dark *){border-color:hsla(220,9%,46%,.1)}.dark\:border-gray-500\/100:is(.dark *){border-color:#6b7280}.dark\:border-gray-500\/15:is(.dark *){border-color:hsla(220,9%,46%,.15)}.dark\:border-gray-500\/20:is(.dark *){border-color:hsla(220,9%,46%,.2)}.dark\:border-gray-500\/25:is(.dark *){border-color:hsla(220,9%,46%,.25)}.dark\:border-gray-500\/30:is(.dark *){border-color:hsla(220,9%,46%,.3)}.dark\:border-gray-500\/35:is(.dark *){border-color:hsla(220,9%,46%,.35)}.dark\:border-gray-500\/40:is(.dark *){border-color:hsla(220,9%,46%,.4)}.dark\:border-gray-500\/45:is(.dark *){border-color:hsla(220,9%,46%,.45)}.dark\:border-gray-500\/5:is(.dark *){border-color:hsla(220,9%,46%,.05)}.dark\:border-gray-500\/50:is(.dark *){border-color:hsla(220,9%,46%,.5)}.dark\:border-gray-500\/55:is(.dark *){border-color:hsla(220,9%,46%,.55)}.dark\:border-gray-500\/60:is(.dark *){border-color:hsla(220,9%,46%,.6)}.dark\:border-gray-500\/65:is(.dark *){border-color:hsla(220,9%,46%,.65)}.dark\:border-gray-500\/70:is(.dark *){border-color:hsla(220,9%,46%,.7)}.dark\:border-gray-500\/75:is(.dark *){border-color:hsla(220,9%,46%,.75)}.dark\:border-gray-500\/80:is(.dark *){border-color:hsla(220,9%,46%,.8)}.dark\:border-gray-500\/85:is(.dark *){border-color:hsla(220,9%,46%,.85)}.dark\:border-gray-500\/90:is(.dark *){border-color:hsla(220,9%,46%,.9)}.dark\:border-gray-500\/95:is(.dark *){border-color:hsla(220,9%,46%,.95)}.dark\:border-gray-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(3 7 18/var(--tw-border-opacity,1))}.dark\:border-gray-950\/0:is(.dark *){border-color:rgba(3,7,18,0)}.dark\:border-gray-950\/10:is(.dark *){border-color:rgba(3,7,18,.1)}.dark\:border-gray-950\/100:is(.dark *){border-color:#030712}.dark\:border-gray-950\/15:is(.dark *){border-color:rgba(3,7,18,.15)}.dark\:border-gray-950\/20:is(.dark *){border-color:rgba(3,7,18,.2)}.dark\:border-gray-950\/25:is(.dark *){border-color:rgba(3,7,18,.25)}.dark\:border-gray-950\/30:is(.dark *){border-color:rgba(3,7,18,.3)}.dark\:border-gray-950\/35:is(.dark *){border-color:rgba(3,7,18,.35)}.dark\:border-gray-950\/40:is(.dark *){border-color:rgba(3,7,18,.4)}.dark\:border-gray-950\/45:is(.dark *){border-color:rgba(3,7,18,.45)}.dark\:border-gray-950\/5:is(.dark *){border-color:rgba(3,7,18,.05)}.dark\:border-gray-950\/50:is(.dark *){border-color:rgba(3,7,18,.5)}.dark\:border-gray-950\/55:is(.dark *){border-color:rgba(3,7,18,.55)}.dark\:border-gray-950\/60:is(.dark *){border-color:rgba(3,7,18,.6)}.dark\:border-gray-950\/65:is(.dark *){border-color:rgba(3,7,18,.65)}.dark\:border-gray-950\/70:is(.dark *){border-color:rgba(3,7,18,.7)}.dark\:border-gray-950\/75:is(.dark *){border-color:rgba(3,7,18,.75)}.dark\:border-gray-950\/80:is(.dark *){border-color:rgba(3,7,18,.8)}.dark\:border-gray-950\/85:is(.dark *){border-color:rgba(3,7,18,.85)}.dark\:border-gray-950\/90:is(.dark *){border-color:rgba(3,7,18,.9)}.dark\:border-gray-950\/95:is(.dark *){border-color:rgba(3,7,18,.95)}.dark\:border-green-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(220 252 231/var(--tw-border-opacity,1))}.dark\:border-green-100\/0:is(.dark *){border-color:rgba(220,252,231,0)}.dark\:border-green-100\/10:is(.dark *){border-color:rgba(220,252,231,.1)}.dark\:border-green-100\/100:is(.dark *){border-color:#dcfce7}.dark\:border-green-100\/15:is(.dark *){border-color:rgba(220,252,231,.15)}.dark\:border-green-100\/20:is(.dark *){border-color:rgba(220,252,231,.2)}.dark\:border-green-100\/25:is(.dark *){border-color:rgba(220,252,231,.25)}.dark\:border-green-100\/30:is(.dark *){border-color:rgba(220,252,231,.3)}.dark\:border-green-100\/35:is(.dark *){border-color:rgba(220,252,231,.35)}.dark\:border-green-100\/40:is(.dark *){border-color:rgba(220,252,231,.4)}.dark\:border-green-100\/45:is(.dark *){border-color:rgba(220,252,231,.45)}.dark\:border-green-100\/5:is(.dark *){border-color:rgba(220,252,231,.05)}.dark\:border-green-100\/50:is(.dark *){border-color:rgba(220,252,231,.5)}.dark\:border-green-100\/55:is(.dark *){border-color:rgba(220,252,231,.55)}.dark\:border-green-100\/60:is(.dark *){border-color:rgba(220,252,231,.6)}.dark\:border-green-100\/65:is(.dark *){border-color:rgba(220,252,231,.65)}.dark\:border-green-100\/70:is(.dark *){border-color:rgba(220,252,231,.7)}.dark\:border-green-100\/75:is(.dark *){border-color:rgba(220,252,231,.75)}.dark\:border-green-100\/80:is(.dark *){border-color:rgba(220,252,231,.8)}.dark\:border-green-100\/85:is(.dark *){border-color:rgba(220,252,231,.85)}.dark\:border-green-100\/90:is(.dark *){border-color:rgba(220,252,231,.9)}.dark\:border-green-100\/95:is(.dark *){border-color:rgba(220,252,231,.95)}.dark\:border-green-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(187 247 208/var(--tw-border-opacity,1))}.dark\:border-green-200\/0:is(.dark *){border-color:rgba(187,247,208,0)}.dark\:border-green-200\/10:is(.dark *){border-color:rgba(187,247,208,.1)}.dark\:border-green-200\/100:is(.dark *){border-color:#bbf7d0}.dark\:border-green-200\/15:is(.dark *){border-color:rgba(187,247,208,.15)}.dark\:border-green-200\/20:is(.dark *){border-color:rgba(187,247,208,.2)}.dark\:border-green-200\/25:is(.dark *){border-color:rgba(187,247,208,.25)}.dark\:border-green-200\/30:is(.dark *){border-color:rgba(187,247,208,.3)}.dark\:border-green-200\/35:is(.dark *){border-color:rgba(187,247,208,.35)}.dark\:border-green-200\/40:is(.dark *){border-color:rgba(187,247,208,.4)}.dark\:border-green-200\/45:is(.dark *){border-color:rgba(187,247,208,.45)}.dark\:border-green-200\/5:is(.dark *){border-color:rgba(187,247,208,.05)}.dark\:border-green-200\/50:is(.dark *){border-color:rgba(187,247,208,.5)}.dark\:border-green-200\/55:is(.dark *){border-color:rgba(187,247,208,.55)}.dark\:border-green-200\/60:is(.dark *){border-color:rgba(187,247,208,.6)}.dark\:border-green-200\/65:is(.dark *){border-color:rgba(187,247,208,.65)}.dark\:border-green-200\/70:is(.dark *){border-color:rgba(187,247,208,.7)}.dark\:border-green-200\/75:is(.dark *){border-color:rgba(187,247,208,.75)}.dark\:border-green-200\/80:is(.dark *){border-color:rgba(187,247,208,.8)}.dark\:border-green-200\/85:is(.dark *){border-color:rgba(187,247,208,.85)}.dark\:border-green-200\/90:is(.dark *){border-color:rgba(187,247,208,.9)}.dark\:border-green-200\/95:is(.dark *){border-color:rgba(187,247,208,.95)}.dark\:border-green-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(240 253 244/var(--tw-border-opacity,1))}.dark\:border-green-50\/0:is(.dark *){border-color:rgba(240,253,244,0)}.dark\:border-green-50\/10:is(.dark *){border-color:rgba(240,253,244,.1)}.dark\:border-green-50\/100:is(.dark *){border-color:#f0fdf4}.dark\:border-green-50\/15:is(.dark *){border-color:rgba(240,253,244,.15)}.dark\:border-green-50\/20:is(.dark *){border-color:rgba(240,253,244,.2)}.dark\:border-green-50\/25:is(.dark *){border-color:rgba(240,253,244,.25)}.dark\:border-green-50\/30:is(.dark *){border-color:rgba(240,253,244,.3)}.dark\:border-green-50\/35:is(.dark *){border-color:rgba(240,253,244,.35)}.dark\:border-green-50\/40:is(.dark *){border-color:rgba(240,253,244,.4)}.dark\:border-green-50\/45:is(.dark *){border-color:rgba(240,253,244,.45)}.dark\:border-green-50\/5:is(.dark *){border-color:rgba(240,253,244,.05)}.dark\:border-green-50\/50:is(.dark *){border-color:rgba(240,253,244,.5)}.dark\:border-green-50\/55:is(.dark *){border-color:rgba(240,253,244,.55)}.dark\:border-green-50\/60:is(.dark *){border-color:rgba(240,253,244,.6)}.dark\:border-green-50\/65:is(.dark *){border-color:rgba(240,253,244,.65)}.dark\:border-green-50\/70:is(.dark *){border-color:rgba(240,253,244,.7)}.dark\:border-green-50\/75:is(.dark *){border-color:rgba(240,253,244,.75)}.dark\:border-green-50\/80:is(.dark *){border-color:rgba(240,253,244,.8)}.dark\:border-green-50\/85:is(.dark *){border-color:rgba(240,253,244,.85)}.dark\:border-green-50\/90:is(.dark *){border-color:rgba(240,253,244,.9)}.dark\:border-green-50\/95:is(.dark *){border-color:rgba(240,253,244,.95)}.dark\:border-green-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(34 197 94/var(--tw-border-opacity,1))}.dark\:border-green-500\/0:is(.dark *){border-color:rgba(34,197,94,0)}.dark\:border-green-500\/10:is(.dark *){border-color:rgba(34,197,94,.1)}.dark\:border-green-500\/100:is(.dark *){border-color:#22c55e}.dark\:border-green-500\/15:is(.dark *){border-color:rgba(34,197,94,.15)}.dark\:border-green-500\/20:is(.dark *){border-color:rgba(34,197,94,.2)}.dark\:border-green-500\/25:is(.dark *){border-color:rgba(34,197,94,.25)}.dark\:border-green-500\/30:is(.dark *){border-color:rgba(34,197,94,.3)}.dark\:border-green-500\/35:is(.dark *){border-color:rgba(34,197,94,.35)}.dark\:border-green-500\/40:is(.dark *){border-color:rgba(34,197,94,.4)}.dark\:border-green-500\/45:is(.dark *){border-color:rgba(34,197,94,.45)}.dark\:border-green-500\/5:is(.dark *){border-color:rgba(34,197,94,.05)}.dark\:border-green-500\/50:is(.dark *){border-color:rgba(34,197,94,.5)}.dark\:border-green-500\/55:is(.dark *){border-color:rgba(34,197,94,.55)}.dark\:border-green-500\/60:is(.dark *){border-color:rgba(34,197,94,.6)}.dark\:border-green-500\/65:is(.dark *){border-color:rgba(34,197,94,.65)}.dark\:border-green-500\/70:is(.dark *){border-color:rgba(34,197,94,.7)}.dark\:border-green-500\/75:is(.dark *){border-color:rgba(34,197,94,.75)}.dark\:border-green-500\/80:is(.dark *){border-color:rgba(34,197,94,.8)}.dark\:border-green-500\/85:is(.dark *){border-color:rgba(34,197,94,.85)}.dark\:border-green-500\/90:is(.dark *){border-color:rgba(34,197,94,.9)}.dark\:border-green-500\/95:is(.dark *){border-color:rgba(34,197,94,.95)}.dark\:border-green-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(5 46 22/var(--tw-border-opacity,1))}.dark\:border-green-950\/0:is(.dark *){border-color:rgba(5,46,22,0)}.dark\:border-green-950\/10:is(.dark *){border-color:rgba(5,46,22,.1)}.dark\:border-green-950\/100:is(.dark *){border-color:#052e16}.dark\:border-green-950\/15:is(.dark *){border-color:rgba(5,46,22,.15)}.dark\:border-green-950\/20:is(.dark *){border-color:rgba(5,46,22,.2)}.dark\:border-green-950\/25:is(.dark *){border-color:rgba(5,46,22,.25)}.dark\:border-green-950\/30:is(.dark *){border-color:rgba(5,46,22,.3)}.dark\:border-green-950\/35:is(.dark *){border-color:rgba(5,46,22,.35)}.dark\:border-green-950\/40:is(.dark *){border-color:rgba(5,46,22,.4)}.dark\:border-green-950\/45:is(.dark *){border-color:rgba(5,46,22,.45)}.dark\:border-green-950\/5:is(.dark *){border-color:rgba(5,46,22,.05)}.dark\:border-green-950\/50:is(.dark *){border-color:rgba(5,46,22,.5)}.dark\:border-green-950\/55:is(.dark *){border-color:rgba(5,46,22,.55)}.dark\:border-green-950\/60:is(.dark *){border-color:rgba(5,46,22,.6)}.dark\:border-green-950\/65:is(.dark *){border-color:rgba(5,46,22,.65)}.dark\:border-green-950\/70:is(.dark *){border-color:rgba(5,46,22,.7)}.dark\:border-green-950\/75:is(.dark *){border-color:rgba(5,46,22,.75)}.dark\:border-green-950\/80:is(.dark *){border-color:rgba(5,46,22,.8)}.dark\:border-green-950\/85:is(.dark *){border-color:rgba(5,46,22,.85)}.dark\:border-green-950\/90:is(.dark *){border-color:rgba(5,46,22,.9)}.dark\:border-green-950\/95:is(.dark *){border-color:rgba(5,46,22,.95)}.dark\:border-indigo-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(224 231 255/var(--tw-border-opacity,1))}.dark\:border-indigo-100\/0:is(.dark *){border-color:rgba(224,231,255,0)}.dark\:border-indigo-100\/10:is(.dark *){border-color:rgba(224,231,255,.1)}.dark\:border-indigo-100\/100:is(.dark *){border-color:#e0e7ff}.dark\:border-indigo-100\/15:is(.dark *){border-color:rgba(224,231,255,.15)}.dark\:border-indigo-100\/20:is(.dark *){border-color:rgba(224,231,255,.2)}.dark\:border-indigo-100\/25:is(.dark *){border-color:rgba(224,231,255,.25)}.dark\:border-indigo-100\/30:is(.dark *){border-color:rgba(224,231,255,.3)}.dark\:border-indigo-100\/35:is(.dark *){border-color:rgba(224,231,255,.35)}.dark\:border-indigo-100\/40:is(.dark *){border-color:rgba(224,231,255,.4)}.dark\:border-indigo-100\/45:is(.dark *){border-color:rgba(224,231,255,.45)}.dark\:border-indigo-100\/5:is(.dark *){border-color:rgba(224,231,255,.05)}.dark\:border-indigo-100\/50:is(.dark *){border-color:rgba(224,231,255,.5)}.dark\:border-indigo-100\/55:is(.dark *){border-color:rgba(224,231,255,.55)}.dark\:border-indigo-100\/60:is(.dark *){border-color:rgba(224,231,255,.6)}.dark\:border-indigo-100\/65:is(.dark *){border-color:rgba(224,231,255,.65)}.dark\:border-indigo-100\/70:is(.dark *){border-color:rgba(224,231,255,.7)}.dark\:border-indigo-100\/75:is(.dark *){border-color:rgba(224,231,255,.75)}.dark\:border-indigo-100\/80:is(.dark *){border-color:rgba(224,231,255,.8)}.dark\:border-indigo-100\/85:is(.dark *){border-color:rgba(224,231,255,.85)}.dark\:border-indigo-100\/90:is(.dark *){border-color:rgba(224,231,255,.9)}.dark\:border-indigo-100\/95:is(.dark *){border-color:rgba(224,231,255,.95)}.dark\:border-indigo-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(199 210 254/var(--tw-border-opacity,1))}.dark\:border-indigo-200\/0:is(.dark *){border-color:rgba(199,210,254,0)}.dark\:border-indigo-200\/10:is(.dark *){border-color:rgba(199,210,254,.1)}.dark\:border-indigo-200\/100:is(.dark *){border-color:#c7d2fe}.dark\:border-indigo-200\/15:is(.dark *){border-color:rgba(199,210,254,.15)}.dark\:border-indigo-200\/20:is(.dark *){border-color:rgba(199,210,254,.2)}.dark\:border-indigo-200\/25:is(.dark *){border-color:rgba(199,210,254,.25)}.dark\:border-indigo-200\/30:is(.dark *){border-color:rgba(199,210,254,.3)}.dark\:border-indigo-200\/35:is(.dark *){border-color:rgba(199,210,254,.35)}.dark\:border-indigo-200\/40:is(.dark *){border-color:rgba(199,210,254,.4)}.dark\:border-indigo-200\/45:is(.dark *){border-color:rgba(199,210,254,.45)}.dark\:border-indigo-200\/5:is(.dark *){border-color:rgba(199,210,254,.05)}.dark\:border-indigo-200\/50:is(.dark *){border-color:rgba(199,210,254,.5)}.dark\:border-indigo-200\/55:is(.dark *){border-color:rgba(199,210,254,.55)}.dark\:border-indigo-200\/60:is(.dark *){border-color:rgba(199,210,254,.6)}.dark\:border-indigo-200\/65:is(.dark *){border-color:rgba(199,210,254,.65)}.dark\:border-indigo-200\/70:is(.dark *){border-color:rgba(199,210,254,.7)}.dark\:border-indigo-200\/75:is(.dark *){border-color:rgba(199,210,254,.75)}.dark\:border-indigo-200\/80:is(.dark *){border-color:rgba(199,210,254,.8)}.dark\:border-indigo-200\/85:is(.dark *){border-color:rgba(199,210,254,.85)}.dark\:border-indigo-200\/90:is(.dark *){border-color:rgba(199,210,254,.9)}.dark\:border-indigo-200\/95:is(.dark *){border-color:rgba(199,210,254,.95)}.dark\:border-indigo-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(238 242 255/var(--tw-border-opacity,1))}.dark\:border-indigo-50\/0:is(.dark *){border-color:rgba(238,242,255,0)}.dark\:border-indigo-50\/10:is(.dark *){border-color:rgba(238,242,255,.1)}.dark\:border-indigo-50\/100:is(.dark *){border-color:#eef2ff}.dark\:border-indigo-50\/15:is(.dark *){border-color:rgba(238,242,255,.15)}.dark\:border-indigo-50\/20:is(.dark *){border-color:rgba(238,242,255,.2)}.dark\:border-indigo-50\/25:is(.dark *){border-color:rgba(238,242,255,.25)}.dark\:border-indigo-50\/30:is(.dark *){border-color:rgba(238,242,255,.3)}.dark\:border-indigo-50\/35:is(.dark *){border-color:rgba(238,242,255,.35)}.dark\:border-indigo-50\/40:is(.dark *){border-color:rgba(238,242,255,.4)}.dark\:border-indigo-50\/45:is(.dark *){border-color:rgba(238,242,255,.45)}.dark\:border-indigo-50\/5:is(.dark *){border-color:rgba(238,242,255,.05)}.dark\:border-indigo-50\/50:is(.dark *){border-color:rgba(238,242,255,.5)}.dark\:border-indigo-50\/55:is(.dark *){border-color:rgba(238,242,255,.55)}.dark\:border-indigo-50\/60:is(.dark *){border-color:rgba(238,242,255,.6)}.dark\:border-indigo-50\/65:is(.dark *){border-color:rgba(238,242,255,.65)}.dark\:border-indigo-50\/70:is(.dark *){border-color:rgba(238,242,255,.7)}.dark\:border-indigo-50\/75:is(.dark *){border-color:rgba(238,242,255,.75)}.dark\:border-indigo-50\/80:is(.dark *){border-color:rgba(238,242,255,.8)}.dark\:border-indigo-50\/85:is(.dark *){border-color:rgba(238,242,255,.85)}.dark\:border-indigo-50\/90:is(.dark *){border-color:rgba(238,242,255,.9)}.dark\:border-indigo-50\/95:is(.dark *){border-color:rgba(238,242,255,.95)}.dark\:border-indigo-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(99 102 241/var(--tw-border-opacity,1))}.dark\:border-indigo-500\/0:is(.dark *){border-color:rgba(99,102,241,0)}.dark\:border-indigo-500\/10:is(.dark *){border-color:rgba(99,102,241,.1)}.dark\:border-indigo-500\/100:is(.dark *){border-color:#6366f1}.dark\:border-indigo-500\/15:is(.dark *){border-color:rgba(99,102,241,.15)}.dark\:border-indigo-500\/20:is(.dark *){border-color:rgba(99,102,241,.2)}.dark\:border-indigo-500\/25:is(.dark *){border-color:rgba(99,102,241,.25)}.dark\:border-indigo-500\/30:is(.dark *){border-color:rgba(99,102,241,.3)}.dark\:border-indigo-500\/35:is(.dark *){border-color:rgba(99,102,241,.35)}.dark\:border-indigo-500\/40:is(.dark *){border-color:rgba(99,102,241,.4)}.dark\:border-indigo-500\/45:is(.dark *){border-color:rgba(99,102,241,.45)}.dark\:border-indigo-500\/5:is(.dark *){border-color:rgba(99,102,241,.05)}.dark\:border-indigo-500\/50:is(.dark *){border-color:rgba(99,102,241,.5)}.dark\:border-indigo-500\/55:is(.dark *){border-color:rgba(99,102,241,.55)}.dark\:border-indigo-500\/60:is(.dark *){border-color:rgba(99,102,241,.6)}.dark\:border-indigo-500\/65:is(.dark *){border-color:rgba(99,102,241,.65)}.dark\:border-indigo-500\/70:is(.dark *){border-color:rgba(99,102,241,.7)}.dark\:border-indigo-500\/75:is(.dark *){border-color:rgba(99,102,241,.75)}.dark\:border-indigo-500\/80:is(.dark *){border-color:rgba(99,102,241,.8)}.dark\:border-indigo-500\/85:is(.dark *){border-color:rgba(99,102,241,.85)}.dark\:border-indigo-500\/90:is(.dark *){border-color:rgba(99,102,241,.9)}.dark\:border-indigo-500\/95:is(.dark *){border-color:rgba(99,102,241,.95)}.dark\:border-indigo-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(30 27 75/var(--tw-border-opacity,1))}.dark\:border-indigo-950\/0:is(.dark *){border-color:rgba(30,27,75,0)}.dark\:border-indigo-950\/10:is(.dark *){border-color:rgba(30,27,75,.1)}.dark\:border-indigo-950\/100:is(.dark *){border-color:#1e1b4b}.dark\:border-indigo-950\/15:is(.dark *){border-color:rgba(30,27,75,.15)}.dark\:border-indigo-950\/20:is(.dark *){border-color:rgba(30,27,75,.2)}.dark\:border-indigo-950\/25:is(.dark *){border-color:rgba(30,27,75,.25)}.dark\:border-indigo-950\/30:is(.dark *){border-color:rgba(30,27,75,.3)}.dark\:border-indigo-950\/35:is(.dark *){border-color:rgba(30,27,75,.35)}.dark\:border-indigo-950\/40:is(.dark *){border-color:rgba(30,27,75,.4)}.dark\:border-indigo-950\/45:is(.dark *){border-color:rgba(30,27,75,.45)}.dark\:border-indigo-950\/5:is(.dark *){border-color:rgba(30,27,75,.05)}.dark\:border-indigo-950\/50:is(.dark *){border-color:rgba(30,27,75,.5)}.dark\:border-indigo-950\/55:is(.dark *){border-color:rgba(30,27,75,.55)}.dark\:border-indigo-950\/60:is(.dark *){border-color:rgba(30,27,75,.6)}.dark\:border-indigo-950\/65:is(.dark *){border-color:rgba(30,27,75,.65)}.dark\:border-indigo-950\/70:is(.dark *){border-color:rgba(30,27,75,.7)}.dark\:border-indigo-950\/75:is(.dark *){border-color:rgba(30,27,75,.75)}.dark\:border-indigo-950\/80:is(.dark *){border-color:rgba(30,27,75,.8)}.dark\:border-indigo-950\/85:is(.dark *){border-color:rgba(30,27,75,.85)}.dark\:border-indigo-950\/90:is(.dark *){border-color:rgba(30,27,75,.9)}.dark\:border-indigo-950\/95:is(.dark *){border-color:rgba(30,27,75,.95)}.dark\:border-lime-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(236 252 203/var(--tw-border-opacity,1))}.dark\:border-lime-100\/0:is(.dark *){border-color:hsla(80,89%,89%,0)}.dark\:border-lime-100\/10:is(.dark *){border-color:hsla(80,89%,89%,.1)}.dark\:border-lime-100\/100:is(.dark *){border-color:#ecfccb}.dark\:border-lime-100\/15:is(.dark *){border-color:hsla(80,89%,89%,.15)}.dark\:border-lime-100\/20:is(.dark *){border-color:hsla(80,89%,89%,.2)}.dark\:border-lime-100\/25:is(.dark *){border-color:hsla(80,89%,89%,.25)}.dark\:border-lime-100\/30:is(.dark *){border-color:hsla(80,89%,89%,.3)}.dark\:border-lime-100\/35:is(.dark *){border-color:hsla(80,89%,89%,.35)}.dark\:border-lime-100\/40:is(.dark *){border-color:hsla(80,89%,89%,.4)}.dark\:border-lime-100\/45:is(.dark *){border-color:hsla(80,89%,89%,.45)}.dark\:border-lime-100\/5:is(.dark *){border-color:hsla(80,89%,89%,.05)}.dark\:border-lime-100\/50:is(.dark *){border-color:hsla(80,89%,89%,.5)}.dark\:border-lime-100\/55:is(.dark *){border-color:hsla(80,89%,89%,.55)}.dark\:border-lime-100\/60:is(.dark *){border-color:hsla(80,89%,89%,.6)}.dark\:border-lime-100\/65:is(.dark *){border-color:hsla(80,89%,89%,.65)}.dark\:border-lime-100\/70:is(.dark *){border-color:hsla(80,89%,89%,.7)}.dark\:border-lime-100\/75:is(.dark *){border-color:hsla(80,89%,89%,.75)}.dark\:border-lime-100\/80:is(.dark *){border-color:hsla(80,89%,89%,.8)}.dark\:border-lime-100\/85:is(.dark *){border-color:hsla(80,89%,89%,.85)}.dark\:border-lime-100\/90:is(.dark *){border-color:hsla(80,89%,89%,.9)}.dark\:border-lime-100\/95:is(.dark *){border-color:hsla(80,89%,89%,.95)}.dark\:border-lime-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(217 249 157/var(--tw-border-opacity,1))}.dark\:border-lime-200\/0:is(.dark *){border-color:hsla(81,88%,80%,0)}.dark\:border-lime-200\/10:is(.dark *){border-color:hsla(81,88%,80%,.1)}.dark\:border-lime-200\/100:is(.dark *){border-color:#d9f99d}.dark\:border-lime-200\/15:is(.dark *){border-color:hsla(81,88%,80%,.15)}.dark\:border-lime-200\/20:is(.dark *){border-color:hsla(81,88%,80%,.2)}.dark\:border-lime-200\/25:is(.dark *){border-color:hsla(81,88%,80%,.25)}.dark\:border-lime-200\/30:is(.dark *){border-color:hsla(81,88%,80%,.3)}.dark\:border-lime-200\/35:is(.dark *){border-color:hsla(81,88%,80%,.35)}.dark\:border-lime-200\/40:is(.dark *){border-color:hsla(81,88%,80%,.4)}.dark\:border-lime-200\/45:is(.dark *){border-color:hsla(81,88%,80%,.45)}.dark\:border-lime-200\/5:is(.dark *){border-color:hsla(81,88%,80%,.05)}.dark\:border-lime-200\/50:is(.dark *){border-color:hsla(81,88%,80%,.5)}.dark\:border-lime-200\/55:is(.dark *){border-color:hsla(81,88%,80%,.55)}.dark\:border-lime-200\/60:is(.dark *){border-color:hsla(81,88%,80%,.6)}.dark\:border-lime-200\/65:is(.dark *){border-color:hsla(81,88%,80%,.65)}.dark\:border-lime-200\/70:is(.dark *){border-color:hsla(81,88%,80%,.7)}.dark\:border-lime-200\/75:is(.dark *){border-color:hsla(81,88%,80%,.75)}.dark\:border-lime-200\/80:is(.dark *){border-color:hsla(81,88%,80%,.8)}.dark\:border-lime-200\/85:is(.dark *){border-color:hsla(81,88%,80%,.85)}.dark\:border-lime-200\/90:is(.dark *){border-color:hsla(81,88%,80%,.9)}.dark\:border-lime-200\/95:is(.dark *){border-color:hsla(81,88%,80%,.95)}.dark\:border-lime-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(247 254 231/var(--tw-border-opacity,1))}.dark\:border-lime-50\/0:is(.dark *){border-color:hsla(78,92%,95%,0)}.dark\:border-lime-50\/10:is(.dark *){border-color:hsla(78,92%,95%,.1)}.dark\:border-lime-50\/100:is(.dark *){border-color:#f7fee7}.dark\:border-lime-50\/15:is(.dark *){border-color:hsla(78,92%,95%,.15)}.dark\:border-lime-50\/20:is(.dark *){border-color:hsla(78,92%,95%,.2)}.dark\:border-lime-50\/25:is(.dark *){border-color:hsla(78,92%,95%,.25)}.dark\:border-lime-50\/30:is(.dark *){border-color:hsla(78,92%,95%,.3)}.dark\:border-lime-50\/35:is(.dark *){border-color:hsla(78,92%,95%,.35)}.dark\:border-lime-50\/40:is(.dark *){border-color:hsla(78,92%,95%,.4)}.dark\:border-lime-50\/45:is(.dark *){border-color:hsla(78,92%,95%,.45)}.dark\:border-lime-50\/5:is(.dark *){border-color:hsla(78,92%,95%,.05)}.dark\:border-lime-50\/50:is(.dark *){border-color:hsla(78,92%,95%,.5)}.dark\:border-lime-50\/55:is(.dark *){border-color:hsla(78,92%,95%,.55)}.dark\:border-lime-50\/60:is(.dark *){border-color:hsla(78,92%,95%,.6)}.dark\:border-lime-50\/65:is(.dark *){border-color:hsla(78,92%,95%,.65)}.dark\:border-lime-50\/70:is(.dark *){border-color:hsla(78,92%,95%,.7)}.dark\:border-lime-50\/75:is(.dark *){border-color:hsla(78,92%,95%,.75)}.dark\:border-lime-50\/80:is(.dark *){border-color:hsla(78,92%,95%,.8)}.dark\:border-lime-50\/85:is(.dark *){border-color:hsla(78,92%,95%,.85)}.dark\:border-lime-50\/90:is(.dark *){border-color:hsla(78,92%,95%,.9)}.dark\:border-lime-50\/95:is(.dark *){border-color:hsla(78,92%,95%,.95)}.dark\:border-lime-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(132 204 22/var(--tw-border-opacity,1))}.dark\:border-lime-500\/0:is(.dark *){border-color:rgba(132,204,22,0)}.dark\:border-lime-500\/10:is(.dark *){border-color:rgba(132,204,22,.1)}.dark\:border-lime-500\/100:is(.dark *){border-color:#84cc16}.dark\:border-lime-500\/15:is(.dark *){border-color:rgba(132,204,22,.15)}.dark\:border-lime-500\/20:is(.dark *){border-color:rgba(132,204,22,.2)}.dark\:border-lime-500\/25:is(.dark *){border-color:rgba(132,204,22,.25)}.dark\:border-lime-500\/30:is(.dark *){border-color:rgba(132,204,22,.3)}.dark\:border-lime-500\/35:is(.dark *){border-color:rgba(132,204,22,.35)}.dark\:border-lime-500\/40:is(.dark *){border-color:rgba(132,204,22,.4)}.dark\:border-lime-500\/45:is(.dark *){border-color:rgba(132,204,22,.45)}.dark\:border-lime-500\/5:is(.dark *){border-color:rgba(132,204,22,.05)}.dark\:border-lime-500\/50:is(.dark *){border-color:rgba(132,204,22,.5)}.dark\:border-lime-500\/55:is(.dark *){border-color:rgba(132,204,22,.55)}.dark\:border-lime-500\/60:is(.dark *){border-color:rgba(132,204,22,.6)}.dark\:border-lime-500\/65:is(.dark *){border-color:rgba(132,204,22,.65)}.dark\:border-lime-500\/70:is(.dark *){border-color:rgba(132,204,22,.7)}.dark\:border-lime-500\/75:is(.dark *){border-color:rgba(132,204,22,.75)}.dark\:border-lime-500\/80:is(.dark *){border-color:rgba(132,204,22,.8)}.dark\:border-lime-500\/85:is(.dark *){border-color:rgba(132,204,22,.85)}.dark\:border-lime-500\/90:is(.dark *){border-color:rgba(132,204,22,.9)}.dark\:border-lime-500\/95:is(.dark *){border-color:rgba(132,204,22,.95)}.dark\:border-lime-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(26 46 5/var(--tw-border-opacity,1))}.dark\:border-lime-950\/0:is(.dark *){border-color:rgba(26,46,5,0)}.dark\:border-lime-950\/10:is(.dark *){border-color:rgba(26,46,5,.1)}.dark\:border-lime-950\/100:is(.dark *){border-color:#1a2e05}.dark\:border-lime-950\/15:is(.dark *){border-color:rgba(26,46,5,.15)}.dark\:border-lime-950\/20:is(.dark *){border-color:rgba(26,46,5,.2)}.dark\:border-lime-950\/25:is(.dark *){border-color:rgba(26,46,5,.25)}.dark\:border-lime-950\/30:is(.dark *){border-color:rgba(26,46,5,.3)}.dark\:border-lime-950\/35:is(.dark *){border-color:rgba(26,46,5,.35)}.dark\:border-lime-950\/40:is(.dark *){border-color:rgba(26,46,5,.4)}.dark\:border-lime-950\/45:is(.dark *){border-color:rgba(26,46,5,.45)}.dark\:border-lime-950\/5:is(.dark *){border-color:rgba(26,46,5,.05)}.dark\:border-lime-950\/50:is(.dark *){border-color:rgba(26,46,5,.5)}.dark\:border-lime-950\/55:is(.dark *){border-color:rgba(26,46,5,.55)}.dark\:border-lime-950\/60:is(.dark *){border-color:rgba(26,46,5,.6)}.dark\:border-lime-950\/65:is(.dark *){border-color:rgba(26,46,5,.65)}.dark\:border-lime-950\/70:is(.dark *){border-color:rgba(26,46,5,.7)}.dark\:border-lime-950\/75:is(.dark *){border-color:rgba(26,46,5,.75)}.dark\:border-lime-950\/80:is(.dark *){border-color:rgba(26,46,5,.8)}.dark\:border-lime-950\/85:is(.dark *){border-color:rgba(26,46,5,.85)}.dark\:border-lime-950\/90:is(.dark *){border-color:rgba(26,46,5,.9)}.dark\:border-lime-950\/95:is(.dark *){border-color:rgba(26,46,5,.95)}.dark\:border-neutral-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(245 245 245/var(--tw-border-opacity,1))}.dark\:border-neutral-100\/0:is(.dark *){border-color:hsla(0,0%,96%,0)}.dark\:border-neutral-100\/10:is(.dark *){border-color:hsla(0,0%,96%,.1)}.dark\:border-neutral-100\/100:is(.dark *){border-color:#f5f5f5}.dark\:border-neutral-100\/15:is(.dark *){border-color:hsla(0,0%,96%,.15)}.dark\:border-neutral-100\/20:is(.dark *){border-color:hsla(0,0%,96%,.2)}.dark\:border-neutral-100\/25:is(.dark *){border-color:hsla(0,0%,96%,.25)}.dark\:border-neutral-100\/30:is(.dark *){border-color:hsla(0,0%,96%,.3)}.dark\:border-neutral-100\/35:is(.dark *){border-color:hsla(0,0%,96%,.35)}.dark\:border-neutral-100\/40:is(.dark *){border-color:hsla(0,0%,96%,.4)}.dark\:border-neutral-100\/45:is(.dark *){border-color:hsla(0,0%,96%,.45)}.dark\:border-neutral-100\/5:is(.dark *){border-color:hsla(0,0%,96%,.05)}.dark\:border-neutral-100\/50:is(.dark *){border-color:hsla(0,0%,96%,.5)}.dark\:border-neutral-100\/55:is(.dark *){border-color:hsla(0,0%,96%,.55)}.dark\:border-neutral-100\/60:is(.dark *){border-color:hsla(0,0%,96%,.6)}.dark\:border-neutral-100\/65:is(.dark *){border-color:hsla(0,0%,96%,.65)}.dark\:border-neutral-100\/70:is(.dark *){border-color:hsla(0,0%,96%,.7)}.dark\:border-neutral-100\/75:is(.dark *){border-color:hsla(0,0%,96%,.75)}.dark\:border-neutral-100\/80:is(.dark *){border-color:hsla(0,0%,96%,.8)}.dark\:border-neutral-100\/85:is(.dark *){border-color:hsla(0,0%,96%,.85)}.dark\:border-neutral-100\/90:is(.dark *){border-color:hsla(0,0%,96%,.9)}.dark\:border-neutral-100\/95:is(.dark *){border-color:hsla(0,0%,96%,.95)}.dark\:border-neutral-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(229 229 229/var(--tw-border-opacity,1))}.dark\:border-neutral-200\/0:is(.dark *){border-color:hsla(0,0%,90%,0)}.dark\:border-neutral-200\/10:is(.dark *){border-color:hsla(0,0%,90%,.1)}.dark\:border-neutral-200\/100:is(.dark *){border-color:#e5e5e5}.dark\:border-neutral-200\/15:is(.dark *){border-color:hsla(0,0%,90%,.15)}.dark\:border-neutral-200\/20:is(.dark *){border-color:hsla(0,0%,90%,.2)}.dark\:border-neutral-200\/25:is(.dark *){border-color:hsla(0,0%,90%,.25)}.dark\:border-neutral-200\/30:is(.dark *){border-color:hsla(0,0%,90%,.3)}.dark\:border-neutral-200\/35:is(.dark *){border-color:hsla(0,0%,90%,.35)}.dark\:border-neutral-200\/40:is(.dark *){border-color:hsla(0,0%,90%,.4)}.dark\:border-neutral-200\/45:is(.dark *){border-color:hsla(0,0%,90%,.45)}.dark\:border-neutral-200\/5:is(.dark *){border-color:hsla(0,0%,90%,.05)}.dark\:border-neutral-200\/50:is(.dark *){border-color:hsla(0,0%,90%,.5)}.dark\:border-neutral-200\/55:is(.dark *){border-color:hsla(0,0%,90%,.55)}.dark\:border-neutral-200\/60:is(.dark *){border-color:hsla(0,0%,90%,.6)}.dark\:border-neutral-200\/65:is(.dark *){border-color:hsla(0,0%,90%,.65)}.dark\:border-neutral-200\/70:is(.dark *){border-color:hsla(0,0%,90%,.7)}.dark\:border-neutral-200\/75:is(.dark *){border-color:hsla(0,0%,90%,.75)}.dark\:border-neutral-200\/80:is(.dark *){border-color:hsla(0,0%,90%,.8)}.dark\:border-neutral-200\/85:is(.dark *){border-color:hsla(0,0%,90%,.85)}.dark\:border-neutral-200\/90:is(.dark *){border-color:hsla(0,0%,90%,.9)}.dark\:border-neutral-200\/95:is(.dark *){border-color:hsla(0,0%,90%,.95)}.dark\:border-neutral-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(250 250 250/var(--tw-border-opacity,1))}.dark\:border-neutral-50\/0:is(.dark *){border-color:hsla(0,0%,98%,0)}.dark\:border-neutral-50\/10:is(.dark *){border-color:hsla(0,0%,98%,.1)}.dark\:border-neutral-50\/100:is(.dark *){border-color:#fafafa}.dark\:border-neutral-50\/15:is(.dark *){border-color:hsla(0,0%,98%,.15)}.dark\:border-neutral-50\/20:is(.dark *){border-color:hsla(0,0%,98%,.2)}.dark\:border-neutral-50\/25:is(.dark *){border-color:hsla(0,0%,98%,.25)}.dark\:border-neutral-50\/30:is(.dark *){border-color:hsla(0,0%,98%,.3)}.dark\:border-neutral-50\/35:is(.dark *){border-color:hsla(0,0%,98%,.35)}.dark\:border-neutral-50\/40:is(.dark *){border-color:hsla(0,0%,98%,.4)}.dark\:border-neutral-50\/45:is(.dark *){border-color:hsla(0,0%,98%,.45)}.dark\:border-neutral-50\/5:is(.dark *){border-color:hsla(0,0%,98%,.05)}.dark\:border-neutral-50\/50:is(.dark *){border-color:hsla(0,0%,98%,.5)}.dark\:border-neutral-50\/55:is(.dark *){border-color:hsla(0,0%,98%,.55)}.dark\:border-neutral-50\/60:is(.dark *){border-color:hsla(0,0%,98%,.6)}.dark\:border-neutral-50\/65:is(.dark *){border-color:hsla(0,0%,98%,.65)}.dark\:border-neutral-50\/70:is(.dark *){border-color:hsla(0,0%,98%,.7)}.dark\:border-neutral-50\/75:is(.dark *){border-color:hsla(0,0%,98%,.75)}.dark\:border-neutral-50\/80:is(.dark *){border-color:hsla(0,0%,98%,.8)}.dark\:border-neutral-50\/85:is(.dark *){border-color:hsla(0,0%,98%,.85)}.dark\:border-neutral-50\/90:is(.dark *){border-color:hsla(0,0%,98%,.9)}.dark\:border-neutral-50\/95:is(.dark *){border-color:hsla(0,0%,98%,.95)}.dark\:border-neutral-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(115 115 115/var(--tw-border-opacity,1))}.dark\:border-neutral-500\/0:is(.dark *){border-color:hsla(0,0%,45%,0)}.dark\:border-neutral-500\/10:is(.dark *){border-color:hsla(0,0%,45%,.1)}.dark\:border-neutral-500\/100:is(.dark *){border-color:#737373}.dark\:border-neutral-500\/15:is(.dark *){border-color:hsla(0,0%,45%,.15)}.dark\:border-neutral-500\/20:is(.dark *){border-color:hsla(0,0%,45%,.2)}.dark\:border-neutral-500\/25:is(.dark *){border-color:hsla(0,0%,45%,.25)}.dark\:border-neutral-500\/30:is(.dark *){border-color:hsla(0,0%,45%,.3)}.dark\:border-neutral-500\/35:is(.dark *){border-color:hsla(0,0%,45%,.35)}.dark\:border-neutral-500\/40:is(.dark *){border-color:hsla(0,0%,45%,.4)}.dark\:border-neutral-500\/45:is(.dark *){border-color:hsla(0,0%,45%,.45)}.dark\:border-neutral-500\/5:is(.dark *){border-color:hsla(0,0%,45%,.05)}.dark\:border-neutral-500\/50:is(.dark *){border-color:hsla(0,0%,45%,.5)}.dark\:border-neutral-500\/55:is(.dark *){border-color:hsla(0,0%,45%,.55)}.dark\:border-neutral-500\/60:is(.dark *){border-color:hsla(0,0%,45%,.6)}.dark\:border-neutral-500\/65:is(.dark *){border-color:hsla(0,0%,45%,.65)}.dark\:border-neutral-500\/70:is(.dark *){border-color:hsla(0,0%,45%,.7)}.dark\:border-neutral-500\/75:is(.dark *){border-color:hsla(0,0%,45%,.75)}.dark\:border-neutral-500\/80:is(.dark *){border-color:hsla(0,0%,45%,.8)}.dark\:border-neutral-500\/85:is(.dark *){border-color:hsla(0,0%,45%,.85)}.dark\:border-neutral-500\/90:is(.dark *){border-color:hsla(0,0%,45%,.9)}.dark\:border-neutral-500\/95:is(.dark *){border-color:hsla(0,0%,45%,.95)}.dark\:border-neutral-700:is(.dark *){--tw-border-opacity:1;border-color:rgb(64 64 64/var(--tw-border-opacity,1))}.dark\:border-neutral-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(10 10 10/var(--tw-border-opacity,1))}.dark\:border-neutral-950\/0:is(.dark *){border-color:hsla(0,0%,4%,0)}.dark\:border-neutral-950\/10:is(.dark *){border-color:hsla(0,0%,4%,.1)}.dark\:border-neutral-950\/100:is(.dark *){border-color:#0a0a0a}.dark\:border-neutral-950\/15:is(.dark *){border-color:hsla(0,0%,4%,.15)}.dark\:border-neutral-950\/20:is(.dark *){border-color:hsla(0,0%,4%,.2)}.dark\:border-neutral-950\/25:is(.dark *){border-color:hsla(0,0%,4%,.25)}.dark\:border-neutral-950\/30:is(.dark *){border-color:hsla(0,0%,4%,.3)}.dark\:border-neutral-950\/35:is(.dark *){border-color:hsla(0,0%,4%,.35)}.dark\:border-neutral-950\/40:is(.dark *){border-color:hsla(0,0%,4%,.4)}.dark\:border-neutral-950\/45:is(.dark *){border-color:hsla(0,0%,4%,.45)}.dark\:border-neutral-950\/5:is(.dark *){border-color:hsla(0,0%,4%,.05)}.dark\:border-neutral-950\/50:is(.dark *){border-color:hsla(0,0%,4%,.5)}.dark\:border-neutral-950\/55:is(.dark *){border-color:hsla(0,0%,4%,.55)}.dark\:border-neutral-950\/60:is(.dark *){border-color:hsla(0,0%,4%,.6)}.dark\:border-neutral-950\/65:is(.dark *){border-color:hsla(0,0%,4%,.65)}.dark\:border-neutral-950\/70:is(.dark *){border-color:hsla(0,0%,4%,.7)}.dark\:border-neutral-950\/75:is(.dark *){border-color:hsla(0,0%,4%,.75)}.dark\:border-neutral-950\/80:is(.dark *){border-color:hsla(0,0%,4%,.8)}.dark\:border-neutral-950\/85:is(.dark *){border-color:hsla(0,0%,4%,.85)}.dark\:border-neutral-950\/90:is(.dark *){border-color:hsla(0,0%,4%,.9)}.dark\:border-neutral-950\/95:is(.dark *){border-color:hsla(0,0%,4%,.95)}.dark\:border-orange-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(255 237 213/var(--tw-border-opacity,1))}.dark\:border-orange-100\/0:is(.dark *){border-color:rgba(255,237,213,0)}.dark\:border-orange-100\/10:is(.dark *){border-color:rgba(255,237,213,.1)}.dark\:border-orange-100\/100:is(.dark *){border-color:#ffedd5}.dark\:border-orange-100\/15:is(.dark *){border-color:rgba(255,237,213,.15)}.dark\:border-orange-100\/20:is(.dark *){border-color:rgba(255,237,213,.2)}.dark\:border-orange-100\/25:is(.dark *){border-color:rgba(255,237,213,.25)}.dark\:border-orange-100\/30:is(.dark *){border-color:rgba(255,237,213,.3)}.dark\:border-orange-100\/35:is(.dark *){border-color:rgba(255,237,213,.35)}.dark\:border-orange-100\/40:is(.dark *){border-color:rgba(255,237,213,.4)}.dark\:border-orange-100\/45:is(.dark *){border-color:rgba(255,237,213,.45)}.dark\:border-orange-100\/5:is(.dark *){border-color:rgba(255,237,213,.05)}.dark\:border-orange-100\/50:is(.dark *){border-color:rgba(255,237,213,.5)}.dark\:border-orange-100\/55:is(.dark *){border-color:rgba(255,237,213,.55)}.dark\:border-orange-100\/60:is(.dark *){border-color:rgba(255,237,213,.6)}.dark\:border-orange-100\/65:is(.dark *){border-color:rgba(255,237,213,.65)}.dark\:border-orange-100\/70:is(.dark *){border-color:rgba(255,237,213,.7)}.dark\:border-orange-100\/75:is(.dark *){border-color:rgba(255,237,213,.75)}.dark\:border-orange-100\/80:is(.dark *){border-color:rgba(255,237,213,.8)}.dark\:border-orange-100\/85:is(.dark *){border-color:rgba(255,237,213,.85)}.dark\:border-orange-100\/90:is(.dark *){border-color:rgba(255,237,213,.9)}.dark\:border-orange-100\/95:is(.dark *){border-color:rgba(255,237,213,.95)}.dark\:border-orange-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(254 215 170/var(--tw-border-opacity,1))}.dark\:border-orange-200\/0:is(.dark *){border-color:hsla(32,98%,83%,0)}.dark\:border-orange-200\/10:is(.dark *){border-color:hsla(32,98%,83%,.1)}.dark\:border-orange-200\/100:is(.dark *){border-color:#fed7aa}.dark\:border-orange-200\/15:is(.dark *){border-color:hsla(32,98%,83%,.15)}.dark\:border-orange-200\/20:is(.dark *){border-color:hsla(32,98%,83%,.2)}.dark\:border-orange-200\/25:is(.dark *){border-color:hsla(32,98%,83%,.25)}.dark\:border-orange-200\/30:is(.dark *){border-color:hsla(32,98%,83%,.3)}.dark\:border-orange-200\/35:is(.dark *){border-color:hsla(32,98%,83%,.35)}.dark\:border-orange-200\/40:is(.dark *){border-color:hsla(32,98%,83%,.4)}.dark\:border-orange-200\/45:is(.dark *){border-color:hsla(32,98%,83%,.45)}.dark\:border-orange-200\/5:is(.dark *){border-color:hsla(32,98%,83%,.05)}.dark\:border-orange-200\/50:is(.dark *){border-color:hsla(32,98%,83%,.5)}.dark\:border-orange-200\/55:is(.dark *){border-color:hsla(32,98%,83%,.55)}.dark\:border-orange-200\/60:is(.dark *){border-color:hsla(32,98%,83%,.6)}.dark\:border-orange-200\/65:is(.dark *){border-color:hsla(32,98%,83%,.65)}.dark\:border-orange-200\/70:is(.dark *){border-color:hsla(32,98%,83%,.7)}.dark\:border-orange-200\/75:is(.dark *){border-color:hsla(32,98%,83%,.75)}.dark\:border-orange-200\/80:is(.dark *){border-color:hsla(32,98%,83%,.8)}.dark\:border-orange-200\/85:is(.dark *){border-color:hsla(32,98%,83%,.85)}.dark\:border-orange-200\/90:is(.dark *){border-color:hsla(32,98%,83%,.9)}.dark\:border-orange-200\/95:is(.dark *){border-color:hsla(32,98%,83%,.95)}.dark\:border-orange-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(255 247 237/var(--tw-border-opacity,1))}.dark\:border-orange-50\/0:is(.dark *){border-color:rgba(255,247,237,0)}.dark\:border-orange-50\/10:is(.dark *){border-color:rgba(255,247,237,.1)}.dark\:border-orange-50\/100:is(.dark *){border-color:#fff7ed}.dark\:border-orange-50\/15:is(.dark *){border-color:rgba(255,247,237,.15)}.dark\:border-orange-50\/20:is(.dark *){border-color:rgba(255,247,237,.2)}.dark\:border-orange-50\/25:is(.dark *){border-color:rgba(255,247,237,.25)}.dark\:border-orange-50\/30:is(.dark *){border-color:rgba(255,247,237,.3)}.dark\:border-orange-50\/35:is(.dark *){border-color:rgba(255,247,237,.35)}.dark\:border-orange-50\/40:is(.dark *){border-color:rgba(255,247,237,.4)}.dark\:border-orange-50\/45:is(.dark *){border-color:rgba(255,247,237,.45)}.dark\:border-orange-50\/5:is(.dark *){border-color:rgba(255,247,237,.05)}.dark\:border-orange-50\/50:is(.dark *){border-color:rgba(255,247,237,.5)}.dark\:border-orange-50\/55:is(.dark *){border-color:rgba(255,247,237,.55)}.dark\:border-orange-50\/60:is(.dark *){border-color:rgba(255,247,237,.6)}.dark\:border-orange-50\/65:is(.dark *){border-color:rgba(255,247,237,.65)}.dark\:border-orange-50\/70:is(.dark *){border-color:rgba(255,247,237,.7)}.dark\:border-orange-50\/75:is(.dark *){border-color:rgba(255,247,237,.75)}.dark\:border-orange-50\/80:is(.dark *){border-color:rgba(255,247,237,.8)}.dark\:border-orange-50\/85:is(.dark *){border-color:rgba(255,247,237,.85)}.dark\:border-orange-50\/90:is(.dark *){border-color:rgba(255,247,237,.9)}.dark\:border-orange-50\/95:is(.dark *){border-color:rgba(255,247,237,.95)}.dark\:border-orange-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(249 115 22/var(--tw-border-opacity,1))}.dark\:border-orange-500\/0:is(.dark *){border-color:rgba(249,115,22,0)}.dark\:border-orange-500\/10:is(.dark *){border-color:rgba(249,115,22,.1)}.dark\:border-orange-500\/100:is(.dark *){border-color:#f97316}.dark\:border-orange-500\/15:is(.dark *){border-color:rgba(249,115,22,.15)}.dark\:border-orange-500\/20:is(.dark *){border-color:rgba(249,115,22,.2)}.dark\:border-orange-500\/25:is(.dark *){border-color:rgba(249,115,22,.25)}.dark\:border-orange-500\/30:is(.dark *){border-color:rgba(249,115,22,.3)}.dark\:border-orange-500\/35:is(.dark *){border-color:rgba(249,115,22,.35)}.dark\:border-orange-500\/40:is(.dark *){border-color:rgba(249,115,22,.4)}.dark\:border-orange-500\/45:is(.dark *){border-color:rgba(249,115,22,.45)}.dark\:border-orange-500\/5:is(.dark *){border-color:rgba(249,115,22,.05)}.dark\:border-orange-500\/50:is(.dark *){border-color:rgba(249,115,22,.5)}.dark\:border-orange-500\/55:is(.dark *){border-color:rgba(249,115,22,.55)}.dark\:border-orange-500\/60:is(.dark *){border-color:rgba(249,115,22,.6)}.dark\:border-orange-500\/65:is(.dark *){border-color:rgba(249,115,22,.65)}.dark\:border-orange-500\/70:is(.dark *){border-color:rgba(249,115,22,.7)}.dark\:border-orange-500\/75:is(.dark *){border-color:rgba(249,115,22,.75)}.dark\:border-orange-500\/80:is(.dark *){border-color:rgba(249,115,22,.8)}.dark\:border-orange-500\/85:is(.dark *){border-color:rgba(249,115,22,.85)}.dark\:border-orange-500\/90:is(.dark *){border-color:rgba(249,115,22,.9)}.dark\:border-orange-500\/95:is(.dark *){border-color:rgba(249,115,22,.95)}.dark\:border-orange-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(67 20 7/var(--tw-border-opacity,1))}.dark\:border-orange-950\/0:is(.dark *){border-color:rgba(67,20,7,0)}.dark\:border-orange-950\/10:is(.dark *){border-color:rgba(67,20,7,.1)}.dark\:border-orange-950\/100:is(.dark *){border-color:#431407}.dark\:border-orange-950\/15:is(.dark *){border-color:rgba(67,20,7,.15)}.dark\:border-orange-950\/20:is(.dark *){border-color:rgba(67,20,7,.2)}.dark\:border-orange-950\/25:is(.dark *){border-color:rgba(67,20,7,.25)}.dark\:border-orange-950\/30:is(.dark *){border-color:rgba(67,20,7,.3)}.dark\:border-orange-950\/35:is(.dark *){border-color:rgba(67,20,7,.35)}.dark\:border-orange-950\/40:is(.dark *){border-color:rgba(67,20,7,.4)}.dark\:border-orange-950\/45:is(.dark *){border-color:rgba(67,20,7,.45)}.dark\:border-orange-950\/5:is(.dark *){border-color:rgba(67,20,7,.05)}.dark\:border-orange-950\/50:is(.dark *){border-color:rgba(67,20,7,.5)}.dark\:border-orange-950\/55:is(.dark *){border-color:rgba(67,20,7,.55)}.dark\:border-orange-950\/60:is(.dark *){border-color:rgba(67,20,7,.6)}.dark\:border-orange-950\/65:is(.dark *){border-color:rgba(67,20,7,.65)}.dark\:border-orange-950\/70:is(.dark *){border-color:rgba(67,20,7,.7)}.dark\:border-orange-950\/75:is(.dark *){border-color:rgba(67,20,7,.75)}.dark\:border-orange-950\/80:is(.dark *){border-color:rgba(67,20,7,.8)}.dark\:border-orange-950\/85:is(.dark *){border-color:rgba(67,20,7,.85)}.dark\:border-orange-950\/90:is(.dark *){border-color:rgba(67,20,7,.9)}.dark\:border-orange-950\/95:is(.dark *){border-color:rgba(67,20,7,.95)}.dark\:border-pink-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(252 231 243/var(--tw-border-opacity,1))}.dark\:border-pink-100\/0:is(.dark *){border-color:rgba(252,231,243,0)}.dark\:border-pink-100\/10:is(.dark *){border-color:rgba(252,231,243,.1)}.dark\:border-pink-100\/100:is(.dark *){border-color:#fce7f3}.dark\:border-pink-100\/15:is(.dark *){border-color:rgba(252,231,243,.15)}.dark\:border-pink-100\/20:is(.dark *){border-color:rgba(252,231,243,.2)}.dark\:border-pink-100\/25:is(.dark *){border-color:rgba(252,231,243,.25)}.dark\:border-pink-100\/30:is(.dark *){border-color:rgba(252,231,243,.3)}.dark\:border-pink-100\/35:is(.dark *){border-color:rgba(252,231,243,.35)}.dark\:border-pink-100\/40:is(.dark *){border-color:rgba(252,231,243,.4)}.dark\:border-pink-100\/45:is(.dark *){border-color:rgba(252,231,243,.45)}.dark\:border-pink-100\/5:is(.dark *){border-color:rgba(252,231,243,.05)}.dark\:border-pink-100\/50:is(.dark *){border-color:rgba(252,231,243,.5)}.dark\:border-pink-100\/55:is(.dark *){border-color:rgba(252,231,243,.55)}.dark\:border-pink-100\/60:is(.dark *){border-color:rgba(252,231,243,.6)}.dark\:border-pink-100\/65:is(.dark *){border-color:rgba(252,231,243,.65)}.dark\:border-pink-100\/70:is(.dark *){border-color:rgba(252,231,243,.7)}.dark\:border-pink-100\/75:is(.dark *){border-color:rgba(252,231,243,.75)}.dark\:border-pink-100\/80:is(.dark *){border-color:rgba(252,231,243,.8)}.dark\:border-pink-100\/85:is(.dark *){border-color:rgba(252,231,243,.85)}.dark\:border-pink-100\/90:is(.dark *){border-color:rgba(252,231,243,.9)}.dark\:border-pink-100\/95:is(.dark *){border-color:rgba(252,231,243,.95)}.dark\:border-pink-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(251 207 232/var(--tw-border-opacity,1))}.dark\:border-pink-200\/0:is(.dark *){border-color:rgba(251,207,232,0)}.dark\:border-pink-200\/10:is(.dark *){border-color:rgba(251,207,232,.1)}.dark\:border-pink-200\/100:is(.dark *){border-color:#fbcfe8}.dark\:border-pink-200\/15:is(.dark *){border-color:rgba(251,207,232,.15)}.dark\:border-pink-200\/20:is(.dark *){border-color:rgba(251,207,232,.2)}.dark\:border-pink-200\/25:is(.dark *){border-color:rgba(251,207,232,.25)}.dark\:border-pink-200\/30:is(.dark *){border-color:rgba(251,207,232,.3)}.dark\:border-pink-200\/35:is(.dark *){border-color:rgba(251,207,232,.35)}.dark\:border-pink-200\/40:is(.dark *){border-color:rgba(251,207,232,.4)}.dark\:border-pink-200\/45:is(.dark *){border-color:rgba(251,207,232,.45)}.dark\:border-pink-200\/5:is(.dark *){border-color:rgba(251,207,232,.05)}.dark\:border-pink-200\/50:is(.dark *){border-color:rgba(251,207,232,.5)}.dark\:border-pink-200\/55:is(.dark *){border-color:rgba(251,207,232,.55)}.dark\:border-pink-200\/60:is(.dark *){border-color:rgba(251,207,232,.6)}.dark\:border-pink-200\/65:is(.dark *){border-color:rgba(251,207,232,.65)}.dark\:border-pink-200\/70:is(.dark *){border-color:rgba(251,207,232,.7)}.dark\:border-pink-200\/75:is(.dark *){border-color:rgba(251,207,232,.75)}.dark\:border-pink-200\/80:is(.dark *){border-color:rgba(251,207,232,.8)}.dark\:border-pink-200\/85:is(.dark *){border-color:rgba(251,207,232,.85)}.dark\:border-pink-200\/90:is(.dark *){border-color:rgba(251,207,232,.9)}.dark\:border-pink-200\/95:is(.dark *){border-color:rgba(251,207,232,.95)}.dark\:border-pink-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(253 242 248/var(--tw-border-opacity,1))}.dark\:border-pink-50\/0:is(.dark *){border-color:rgba(253,242,248,0)}.dark\:border-pink-50\/10:is(.dark *){border-color:rgba(253,242,248,.1)}.dark\:border-pink-50\/100:is(.dark *){border-color:#fdf2f8}.dark\:border-pink-50\/15:is(.dark *){border-color:rgba(253,242,248,.15)}.dark\:border-pink-50\/20:is(.dark *){border-color:rgba(253,242,248,.2)}.dark\:border-pink-50\/25:is(.dark *){border-color:rgba(253,242,248,.25)}.dark\:border-pink-50\/30:is(.dark *){border-color:rgba(253,242,248,.3)}.dark\:border-pink-50\/35:is(.dark *){border-color:rgba(253,242,248,.35)}.dark\:border-pink-50\/40:is(.dark *){border-color:rgba(253,242,248,.4)}.dark\:border-pink-50\/45:is(.dark *){border-color:rgba(253,242,248,.45)}.dark\:border-pink-50\/5:is(.dark *){border-color:rgba(253,242,248,.05)}.dark\:border-pink-50\/50:is(.dark *){border-color:rgba(253,242,248,.5)}.dark\:border-pink-50\/55:is(.dark *){border-color:rgba(253,242,248,.55)}.dark\:border-pink-50\/60:is(.dark *){border-color:rgba(253,242,248,.6)}.dark\:border-pink-50\/65:is(.dark *){border-color:rgba(253,242,248,.65)}.dark\:border-pink-50\/70:is(.dark *){border-color:rgba(253,242,248,.7)}.dark\:border-pink-50\/75:is(.dark *){border-color:rgba(253,242,248,.75)}.dark\:border-pink-50\/80:is(.dark *){border-color:rgba(253,242,248,.8)}.dark\:border-pink-50\/85:is(.dark *){border-color:rgba(253,242,248,.85)}.dark\:border-pink-50\/90:is(.dark *){border-color:rgba(253,242,248,.9)}.dark\:border-pink-50\/95:is(.dark *){border-color:rgba(253,242,248,.95)}.dark\:border-pink-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(236 72 153/var(--tw-border-opacity,1))}.dark\:border-pink-500\/0:is(.dark *){border-color:rgba(236,72,153,0)}.dark\:border-pink-500\/10:is(.dark *){border-color:rgba(236,72,153,.1)}.dark\:border-pink-500\/100:is(.dark *){border-color:#ec4899}.dark\:border-pink-500\/15:is(.dark *){border-color:rgba(236,72,153,.15)}.dark\:border-pink-500\/20:is(.dark *){border-color:rgba(236,72,153,.2)}.dark\:border-pink-500\/25:is(.dark *){border-color:rgba(236,72,153,.25)}.dark\:border-pink-500\/30:is(.dark *){border-color:rgba(236,72,153,.3)}.dark\:border-pink-500\/35:is(.dark *){border-color:rgba(236,72,153,.35)}.dark\:border-pink-500\/40:is(.dark *){border-color:rgba(236,72,153,.4)}.dark\:border-pink-500\/45:is(.dark *){border-color:rgba(236,72,153,.45)}.dark\:border-pink-500\/5:is(.dark *){border-color:rgba(236,72,153,.05)}.dark\:border-pink-500\/50:is(.dark *){border-color:rgba(236,72,153,.5)}.dark\:border-pink-500\/55:is(.dark *){border-color:rgba(236,72,153,.55)}.dark\:border-pink-500\/60:is(.dark *){border-color:rgba(236,72,153,.6)}.dark\:border-pink-500\/65:is(.dark *){border-color:rgba(236,72,153,.65)}.dark\:border-pink-500\/70:is(.dark *){border-color:rgba(236,72,153,.7)}.dark\:border-pink-500\/75:is(.dark *){border-color:rgba(236,72,153,.75)}.dark\:border-pink-500\/80:is(.dark *){border-color:rgba(236,72,153,.8)}.dark\:border-pink-500\/85:is(.dark *){border-color:rgba(236,72,153,.85)}.dark\:border-pink-500\/90:is(.dark *){border-color:rgba(236,72,153,.9)}.dark\:border-pink-500\/95:is(.dark *){border-color:rgba(236,72,153,.95)}.dark\:border-pink-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(80 7 36/var(--tw-border-opacity,1))}.dark\:border-pink-950\/0:is(.dark *){border-color:rgba(80,7,36,0)}.dark\:border-pink-950\/10:is(.dark *){border-color:rgba(80,7,36,.1)}.dark\:border-pink-950\/100:is(.dark *){border-color:#500724}.dark\:border-pink-950\/15:is(.dark *){border-color:rgba(80,7,36,.15)}.dark\:border-pink-950\/20:is(.dark *){border-color:rgba(80,7,36,.2)}.dark\:border-pink-950\/25:is(.dark *){border-color:rgba(80,7,36,.25)}.dark\:border-pink-950\/30:is(.dark *){border-color:rgba(80,7,36,.3)}.dark\:border-pink-950\/35:is(.dark *){border-color:rgba(80,7,36,.35)}.dark\:border-pink-950\/40:is(.dark *){border-color:rgba(80,7,36,.4)}.dark\:border-pink-950\/45:is(.dark *){border-color:rgba(80,7,36,.45)}.dark\:border-pink-950\/5:is(.dark *){border-color:rgba(80,7,36,.05)}.dark\:border-pink-950\/50:is(.dark *){border-color:rgba(80,7,36,.5)}.dark\:border-pink-950\/55:is(.dark *){border-color:rgba(80,7,36,.55)}.dark\:border-pink-950\/60:is(.dark *){border-color:rgba(80,7,36,.6)}.dark\:border-pink-950\/65:is(.dark *){border-color:rgba(80,7,36,.65)}.dark\:border-pink-950\/70:is(.dark *){border-color:rgba(80,7,36,.7)}.dark\:border-pink-950\/75:is(.dark *){border-color:rgba(80,7,36,.75)}.dark\:border-pink-950\/80:is(.dark *){border-color:rgba(80,7,36,.8)}.dark\:border-pink-950\/85:is(.dark *){border-color:rgba(80,7,36,.85)}.dark\:border-pink-950\/90:is(.dark *){border-color:rgba(80,7,36,.9)}.dark\:border-pink-950\/95:is(.dark *){border-color:rgba(80,7,36,.95)}.dark\:border-purple-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(243 232 255/var(--tw-border-opacity,1))}.dark\:border-purple-100\/0:is(.dark *){border-color:rgba(243,232,255,0)}.dark\:border-purple-100\/10:is(.dark *){border-color:rgba(243,232,255,.1)}.dark\:border-purple-100\/100:is(.dark *){border-color:#f3e8ff}.dark\:border-purple-100\/15:is(.dark *){border-color:rgba(243,232,255,.15)}.dark\:border-purple-100\/20:is(.dark *){border-color:rgba(243,232,255,.2)}.dark\:border-purple-100\/25:is(.dark *){border-color:rgba(243,232,255,.25)}.dark\:border-purple-100\/30:is(.dark *){border-color:rgba(243,232,255,.3)}.dark\:border-purple-100\/35:is(.dark *){border-color:rgba(243,232,255,.35)}.dark\:border-purple-100\/40:is(.dark *){border-color:rgba(243,232,255,.4)}.dark\:border-purple-100\/45:is(.dark *){border-color:rgba(243,232,255,.45)}.dark\:border-purple-100\/5:is(.dark *){border-color:rgba(243,232,255,.05)}.dark\:border-purple-100\/50:is(.dark *){border-color:rgba(243,232,255,.5)}.dark\:border-purple-100\/55:is(.dark *){border-color:rgba(243,232,255,.55)}.dark\:border-purple-100\/60:is(.dark *){border-color:rgba(243,232,255,.6)}.dark\:border-purple-100\/65:is(.dark *){border-color:rgba(243,232,255,.65)}.dark\:border-purple-100\/70:is(.dark *){border-color:rgba(243,232,255,.7)}.dark\:border-purple-100\/75:is(.dark *){border-color:rgba(243,232,255,.75)}.dark\:border-purple-100\/80:is(.dark *){border-color:rgba(243,232,255,.8)}.dark\:border-purple-100\/85:is(.dark *){border-color:rgba(243,232,255,.85)}.dark\:border-purple-100\/90:is(.dark *){border-color:rgba(243,232,255,.9)}.dark\:border-purple-100\/95:is(.dark *){border-color:rgba(243,232,255,.95)}.dark\:border-purple-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(233 213 255/var(--tw-border-opacity,1))}.dark\:border-purple-200\/0:is(.dark *){border-color:rgba(233,213,255,0)}.dark\:border-purple-200\/10:is(.dark *){border-color:rgba(233,213,255,.1)}.dark\:border-purple-200\/100:is(.dark *){border-color:#e9d5ff}.dark\:border-purple-200\/15:is(.dark *){border-color:rgba(233,213,255,.15)}.dark\:border-purple-200\/20:is(.dark *){border-color:rgba(233,213,255,.2)}.dark\:border-purple-200\/25:is(.dark *){border-color:rgba(233,213,255,.25)}.dark\:border-purple-200\/30:is(.dark *){border-color:rgba(233,213,255,.3)}.dark\:border-purple-200\/35:is(.dark *){border-color:rgba(233,213,255,.35)}.dark\:border-purple-200\/40:is(.dark *){border-color:rgba(233,213,255,.4)}.dark\:border-purple-200\/45:is(.dark *){border-color:rgba(233,213,255,.45)}.dark\:border-purple-200\/5:is(.dark *){border-color:rgba(233,213,255,.05)}.dark\:border-purple-200\/50:is(.dark *){border-color:rgba(233,213,255,.5)}.dark\:border-purple-200\/55:is(.dark *){border-color:rgba(233,213,255,.55)}.dark\:border-purple-200\/60:is(.dark *){border-color:rgba(233,213,255,.6)}.dark\:border-purple-200\/65:is(.dark *){border-color:rgba(233,213,255,.65)}.dark\:border-purple-200\/70:is(.dark *){border-color:rgba(233,213,255,.7)}.dark\:border-purple-200\/75:is(.dark *){border-color:rgba(233,213,255,.75)}.dark\:border-purple-200\/80:is(.dark *){border-color:rgba(233,213,255,.8)}.dark\:border-purple-200\/85:is(.dark *){border-color:rgba(233,213,255,.85)}.dark\:border-purple-200\/90:is(.dark *){border-color:rgba(233,213,255,.9)}.dark\:border-purple-200\/95:is(.dark *){border-color:rgba(233,213,255,.95)}.dark\:border-purple-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(250 245 255/var(--tw-border-opacity,1))}.dark\:border-purple-50\/0:is(.dark *){border-color:rgba(250,245,255,0)}.dark\:border-purple-50\/10:is(.dark *){border-color:rgba(250,245,255,.1)}.dark\:border-purple-50\/100:is(.dark *){border-color:#faf5ff}.dark\:border-purple-50\/15:is(.dark *){border-color:rgba(250,245,255,.15)}.dark\:border-purple-50\/20:is(.dark *){border-color:rgba(250,245,255,.2)}.dark\:border-purple-50\/25:is(.dark *){border-color:rgba(250,245,255,.25)}.dark\:border-purple-50\/30:is(.dark *){border-color:rgba(250,245,255,.3)}.dark\:border-purple-50\/35:is(.dark *){border-color:rgba(250,245,255,.35)}.dark\:border-purple-50\/40:is(.dark *){border-color:rgba(250,245,255,.4)}.dark\:border-purple-50\/45:is(.dark *){border-color:rgba(250,245,255,.45)}.dark\:border-purple-50\/5:is(.dark *){border-color:rgba(250,245,255,.05)}.dark\:border-purple-50\/50:is(.dark *){border-color:rgba(250,245,255,.5)}.dark\:border-purple-50\/55:is(.dark *){border-color:rgba(250,245,255,.55)}.dark\:border-purple-50\/60:is(.dark *){border-color:rgba(250,245,255,.6)}.dark\:border-purple-50\/65:is(.dark *){border-color:rgba(250,245,255,.65)}.dark\:border-purple-50\/70:is(.dark *){border-color:rgba(250,245,255,.7)}.dark\:border-purple-50\/75:is(.dark *){border-color:rgba(250,245,255,.75)}.dark\:border-purple-50\/80:is(.dark *){border-color:rgba(250,245,255,.8)}.dark\:border-purple-50\/85:is(.dark *){border-color:rgba(250,245,255,.85)}.dark\:border-purple-50\/90:is(.dark *){border-color:rgba(250,245,255,.9)}.dark\:border-purple-50\/95:is(.dark *){border-color:rgba(250,245,255,.95)}.dark\:border-purple-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(168 85 247/var(--tw-border-opacity,1))}.dark\:border-purple-500\/0:is(.dark *){border-color:rgba(168,85,247,0)}.dark\:border-purple-500\/10:is(.dark *){border-color:rgba(168,85,247,.1)}.dark\:border-purple-500\/100:is(.dark *){border-color:#a855f7}.dark\:border-purple-500\/15:is(.dark *){border-color:rgba(168,85,247,.15)}.dark\:border-purple-500\/20:is(.dark *){border-color:rgba(168,85,247,.2)}.dark\:border-purple-500\/25:is(.dark *){border-color:rgba(168,85,247,.25)}.dark\:border-purple-500\/30:is(.dark *){border-color:rgba(168,85,247,.3)}.dark\:border-purple-500\/35:is(.dark *){border-color:rgba(168,85,247,.35)}.dark\:border-purple-500\/40:is(.dark *){border-color:rgba(168,85,247,.4)}.dark\:border-purple-500\/45:is(.dark *){border-color:rgba(168,85,247,.45)}.dark\:border-purple-500\/5:is(.dark *){border-color:rgba(168,85,247,.05)}.dark\:border-purple-500\/50:is(.dark *){border-color:rgba(168,85,247,.5)}.dark\:border-purple-500\/55:is(.dark *){border-color:rgba(168,85,247,.55)}.dark\:border-purple-500\/60:is(.dark *){border-color:rgba(168,85,247,.6)}.dark\:border-purple-500\/65:is(.dark *){border-color:rgba(168,85,247,.65)}.dark\:border-purple-500\/70:is(.dark *){border-color:rgba(168,85,247,.7)}.dark\:border-purple-500\/75:is(.dark *){border-color:rgba(168,85,247,.75)}.dark\:border-purple-500\/80:is(.dark *){border-color:rgba(168,85,247,.8)}.dark\:border-purple-500\/85:is(.dark *){border-color:rgba(168,85,247,.85)}.dark\:border-purple-500\/90:is(.dark *){border-color:rgba(168,85,247,.9)}.dark\:border-purple-500\/95:is(.dark *){border-color:rgba(168,85,247,.95)}.dark\:border-purple-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(59 7 100/var(--tw-border-opacity,1))}.dark\:border-purple-950\/0:is(.dark *){border-color:rgba(59,7,100,0)}.dark\:border-purple-950\/10:is(.dark *){border-color:rgba(59,7,100,.1)}.dark\:border-purple-950\/100:is(.dark *){border-color:#3b0764}.dark\:border-purple-950\/15:is(.dark *){border-color:rgba(59,7,100,.15)}.dark\:border-purple-950\/20:is(.dark *){border-color:rgba(59,7,100,.2)}.dark\:border-purple-950\/25:is(.dark *){border-color:rgba(59,7,100,.25)}.dark\:border-purple-950\/30:is(.dark *){border-color:rgba(59,7,100,.3)}.dark\:border-purple-950\/35:is(.dark *){border-color:rgba(59,7,100,.35)}.dark\:border-purple-950\/40:is(.dark *){border-color:rgba(59,7,100,.4)}.dark\:border-purple-950\/45:is(.dark *){border-color:rgba(59,7,100,.45)}.dark\:border-purple-950\/5:is(.dark *){border-color:rgba(59,7,100,.05)}.dark\:border-purple-950\/50:is(.dark *){border-color:rgba(59,7,100,.5)}.dark\:border-purple-950\/55:is(.dark *){border-color:rgba(59,7,100,.55)}.dark\:border-purple-950\/60:is(.dark *){border-color:rgba(59,7,100,.6)}.dark\:border-purple-950\/65:is(.dark *){border-color:rgba(59,7,100,.65)}.dark\:border-purple-950\/70:is(.dark *){border-color:rgba(59,7,100,.7)}.dark\:border-purple-950\/75:is(.dark *){border-color:rgba(59,7,100,.75)}.dark\:border-purple-950\/80:is(.dark *){border-color:rgba(59,7,100,.8)}.dark\:border-purple-950\/85:is(.dark *){border-color:rgba(59,7,100,.85)}.dark\:border-purple-950\/90:is(.dark *){border-color:rgba(59,7,100,.9)}.dark\:border-purple-950\/95:is(.dark *){border-color:rgba(59,7,100,.95)}.dark\:border-red-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(254 226 226/var(--tw-border-opacity,1))}.dark\:border-red-100\/0:is(.dark *){border-color:hsla(0,93%,94%,0)}.dark\:border-red-100\/10:is(.dark *){border-color:hsla(0,93%,94%,.1)}.dark\:border-red-100\/100:is(.dark *){border-color:#fee2e2}.dark\:border-red-100\/15:is(.dark *){border-color:hsla(0,93%,94%,.15)}.dark\:border-red-100\/20:is(.dark *){border-color:hsla(0,93%,94%,.2)}.dark\:border-red-100\/25:is(.dark *){border-color:hsla(0,93%,94%,.25)}.dark\:border-red-100\/30:is(.dark *){border-color:hsla(0,93%,94%,.3)}.dark\:border-red-100\/35:is(.dark *){border-color:hsla(0,93%,94%,.35)}.dark\:border-red-100\/40:is(.dark *){border-color:hsla(0,93%,94%,.4)}.dark\:border-red-100\/45:is(.dark *){border-color:hsla(0,93%,94%,.45)}.dark\:border-red-100\/5:is(.dark *){border-color:hsla(0,93%,94%,.05)}.dark\:border-red-100\/50:is(.dark *){border-color:hsla(0,93%,94%,.5)}.dark\:border-red-100\/55:is(.dark *){border-color:hsla(0,93%,94%,.55)}.dark\:border-red-100\/60:is(.dark *){border-color:hsla(0,93%,94%,.6)}.dark\:border-red-100\/65:is(.dark *){border-color:hsla(0,93%,94%,.65)}.dark\:border-red-100\/70:is(.dark *){border-color:hsla(0,93%,94%,.7)}.dark\:border-red-100\/75:is(.dark *){border-color:hsla(0,93%,94%,.75)}.dark\:border-red-100\/80:is(.dark *){border-color:hsla(0,93%,94%,.8)}.dark\:border-red-100\/85:is(.dark *){border-color:hsla(0,93%,94%,.85)}.dark\:border-red-100\/90:is(.dark *){border-color:hsla(0,93%,94%,.9)}.dark\:border-red-100\/95:is(.dark *){border-color:hsla(0,93%,94%,.95)}.dark\:border-red-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(254 202 202/var(--tw-border-opacity,1))}.dark\:border-red-200\/0:is(.dark *){border-color:hsla(0,96%,89%,0)}.dark\:border-red-200\/10:is(.dark *){border-color:hsla(0,96%,89%,.1)}.dark\:border-red-200\/100:is(.dark *){border-color:#fecaca}.dark\:border-red-200\/15:is(.dark *){border-color:hsla(0,96%,89%,.15)}.dark\:border-red-200\/20:is(.dark *){border-color:hsla(0,96%,89%,.2)}.dark\:border-red-200\/25:is(.dark *){border-color:hsla(0,96%,89%,.25)}.dark\:border-red-200\/30:is(.dark *){border-color:hsla(0,96%,89%,.3)}.dark\:border-red-200\/35:is(.dark *){border-color:hsla(0,96%,89%,.35)}.dark\:border-red-200\/40:is(.dark *){border-color:hsla(0,96%,89%,.4)}.dark\:border-red-200\/45:is(.dark *){border-color:hsla(0,96%,89%,.45)}.dark\:border-red-200\/5:is(.dark *){border-color:hsla(0,96%,89%,.05)}.dark\:border-red-200\/50:is(.dark *){border-color:hsla(0,96%,89%,.5)}.dark\:border-red-200\/55:is(.dark *){border-color:hsla(0,96%,89%,.55)}.dark\:border-red-200\/60:is(.dark *){border-color:hsla(0,96%,89%,.6)}.dark\:border-red-200\/65:is(.dark *){border-color:hsla(0,96%,89%,.65)}.dark\:border-red-200\/70:is(.dark *){border-color:hsla(0,96%,89%,.7)}.dark\:border-red-200\/75:is(.dark *){border-color:hsla(0,96%,89%,.75)}.dark\:border-red-200\/80:is(.dark *){border-color:hsla(0,96%,89%,.8)}.dark\:border-red-200\/85:is(.dark *){border-color:hsla(0,96%,89%,.85)}.dark\:border-red-200\/90:is(.dark *){border-color:hsla(0,96%,89%,.9)}.dark\:border-red-200\/95:is(.dark *){border-color:hsla(0,96%,89%,.95)}.dark\:border-red-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(254 242 242/var(--tw-border-opacity,1))}.dark\:border-red-50\/0:is(.dark *){border-color:hsla(0,86%,97%,0)}.dark\:border-red-50\/10:is(.dark *){border-color:hsla(0,86%,97%,.1)}.dark\:border-red-50\/100:is(.dark *){border-color:#fef2f2}.dark\:border-red-50\/15:is(.dark *){border-color:hsla(0,86%,97%,.15)}.dark\:border-red-50\/20:is(.dark *){border-color:hsla(0,86%,97%,.2)}.dark\:border-red-50\/25:is(.dark *){border-color:hsla(0,86%,97%,.25)}.dark\:border-red-50\/30:is(.dark *){border-color:hsla(0,86%,97%,.3)}.dark\:border-red-50\/35:is(.dark *){border-color:hsla(0,86%,97%,.35)}.dark\:border-red-50\/40:is(.dark *){border-color:hsla(0,86%,97%,.4)}.dark\:border-red-50\/45:is(.dark *){border-color:hsla(0,86%,97%,.45)}.dark\:border-red-50\/5:is(.dark *){border-color:hsla(0,86%,97%,.05)}.dark\:border-red-50\/50:is(.dark *){border-color:hsla(0,86%,97%,.5)}.dark\:border-red-50\/55:is(.dark *){border-color:hsla(0,86%,97%,.55)}.dark\:border-red-50\/60:is(.dark *){border-color:hsla(0,86%,97%,.6)}.dark\:border-red-50\/65:is(.dark *){border-color:hsla(0,86%,97%,.65)}.dark\:border-red-50\/70:is(.dark *){border-color:hsla(0,86%,97%,.7)}.dark\:border-red-50\/75:is(.dark *){border-color:hsla(0,86%,97%,.75)}.dark\:border-red-50\/80:is(.dark *){border-color:hsla(0,86%,97%,.8)}.dark\:border-red-50\/85:is(.dark *){border-color:hsla(0,86%,97%,.85)}.dark\:border-red-50\/90:is(.dark *){border-color:hsla(0,86%,97%,.9)}.dark\:border-red-50\/95:is(.dark *){border-color:hsla(0,86%,97%,.95)}.dark\:border-red-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(239 68 68/var(--tw-border-opacity,1))}.dark\:border-red-500\/0:is(.dark *){border-color:rgba(239,68,68,0)}.dark\:border-red-500\/10:is(.dark *){border-color:rgba(239,68,68,.1)}.dark\:border-red-500\/100:is(.dark *){border-color:#ef4444}.dark\:border-red-500\/15:is(.dark *){border-color:rgba(239,68,68,.15)}.dark\:border-red-500\/20:is(.dark *){border-color:rgba(239,68,68,.2)}.dark\:border-red-500\/25:is(.dark *){border-color:rgba(239,68,68,.25)}.dark\:border-red-500\/30:is(.dark *){border-color:rgba(239,68,68,.3)}.dark\:border-red-500\/35:is(.dark *){border-color:rgba(239,68,68,.35)}.dark\:border-red-500\/40:is(.dark *){border-color:rgba(239,68,68,.4)}.dark\:border-red-500\/45:is(.dark *){border-color:rgba(239,68,68,.45)}.dark\:border-red-500\/5:is(.dark *){border-color:rgba(239,68,68,.05)}.dark\:border-red-500\/50:is(.dark *){border-color:rgba(239,68,68,.5)}.dark\:border-red-500\/55:is(.dark *){border-color:rgba(239,68,68,.55)}.dark\:border-red-500\/60:is(.dark *){border-color:rgba(239,68,68,.6)}.dark\:border-red-500\/65:is(.dark *){border-color:rgba(239,68,68,.65)}.dark\:border-red-500\/70:is(.dark *){border-color:rgba(239,68,68,.7)}.dark\:border-red-500\/75:is(.dark *){border-color:rgba(239,68,68,.75)}.dark\:border-red-500\/80:is(.dark *){border-color:rgba(239,68,68,.8)}.dark\:border-red-500\/85:is(.dark *){border-color:rgba(239,68,68,.85)}.dark\:border-red-500\/90:is(.dark *){border-color:rgba(239,68,68,.9)}.dark\:border-red-500\/95:is(.dark *){border-color:rgba(239,68,68,.95)}.dark\:border-red-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(69 10 10/var(--tw-border-opacity,1))}.dark\:border-red-950\/0:is(.dark *){border-color:rgba(69,10,10,0)}.dark\:border-red-950\/10:is(.dark *){border-color:rgba(69,10,10,.1)}.dark\:border-red-950\/100:is(.dark *){border-color:#450a0a}.dark\:border-red-950\/15:is(.dark *){border-color:rgba(69,10,10,.15)}.dark\:border-red-950\/20:is(.dark *){border-color:rgba(69,10,10,.2)}.dark\:border-red-950\/25:is(.dark *){border-color:rgba(69,10,10,.25)}.dark\:border-red-950\/30:is(.dark *){border-color:rgba(69,10,10,.3)}.dark\:border-red-950\/35:is(.dark *){border-color:rgba(69,10,10,.35)}.dark\:border-red-950\/40:is(.dark *){border-color:rgba(69,10,10,.4)}.dark\:border-red-950\/45:is(.dark *){border-color:rgba(69,10,10,.45)}.dark\:border-red-950\/5:is(.dark *){border-color:rgba(69,10,10,.05)}.dark\:border-red-950\/50:is(.dark *){border-color:rgba(69,10,10,.5)}.dark\:border-red-950\/55:is(.dark *){border-color:rgba(69,10,10,.55)}.dark\:border-red-950\/60:is(.dark *){border-color:rgba(69,10,10,.6)}.dark\:border-red-950\/65:is(.dark *){border-color:rgba(69,10,10,.65)}.dark\:border-red-950\/70:is(.dark *){border-color:rgba(69,10,10,.7)}.dark\:border-red-950\/75:is(.dark *){border-color:rgba(69,10,10,.75)}.dark\:border-red-950\/80:is(.dark *){border-color:rgba(69,10,10,.8)}.dark\:border-red-950\/85:is(.dark *){border-color:rgba(69,10,10,.85)}.dark\:border-red-950\/90:is(.dark *){border-color:rgba(69,10,10,.9)}.dark\:border-red-950\/95:is(.dark *){border-color:rgba(69,10,10,.95)}.dark\:border-rose-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(255 228 230/var(--tw-border-opacity,1))}.dark\:border-rose-100\/0:is(.dark *){border-color:rgba(255,228,230,0)}.dark\:border-rose-100\/10:is(.dark *){border-color:rgba(255,228,230,.1)}.dark\:border-rose-100\/100:is(.dark *){border-color:#ffe4e6}.dark\:border-rose-100\/15:is(.dark *){border-color:rgba(255,228,230,.15)}.dark\:border-rose-100\/20:is(.dark *){border-color:rgba(255,228,230,.2)}.dark\:border-rose-100\/25:is(.dark *){border-color:rgba(255,228,230,.25)}.dark\:border-rose-100\/30:is(.dark *){border-color:rgba(255,228,230,.3)}.dark\:border-rose-100\/35:is(.dark *){border-color:rgba(255,228,230,.35)}.dark\:border-rose-100\/40:is(.dark *){border-color:rgba(255,228,230,.4)}.dark\:border-rose-100\/45:is(.dark *){border-color:rgba(255,228,230,.45)}.dark\:border-rose-100\/5:is(.dark *){border-color:rgba(255,228,230,.05)}.dark\:border-rose-100\/50:is(.dark *){border-color:rgba(255,228,230,.5)}.dark\:border-rose-100\/55:is(.dark *){border-color:rgba(255,228,230,.55)}.dark\:border-rose-100\/60:is(.dark *){border-color:rgba(255,228,230,.6)}.dark\:border-rose-100\/65:is(.dark *){border-color:rgba(255,228,230,.65)}.dark\:border-rose-100\/70:is(.dark *){border-color:rgba(255,228,230,.7)}.dark\:border-rose-100\/75:is(.dark *){border-color:rgba(255,228,230,.75)}.dark\:border-rose-100\/80:is(.dark *){border-color:rgba(255,228,230,.8)}.dark\:border-rose-100\/85:is(.dark *){border-color:rgba(255,228,230,.85)}.dark\:border-rose-100\/90:is(.dark *){border-color:rgba(255,228,230,.9)}.dark\:border-rose-100\/95:is(.dark *){border-color:rgba(255,228,230,.95)}.dark\:border-rose-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(254 205 211/var(--tw-border-opacity,1))}.dark\:border-rose-200\/0:is(.dark *){border-color:rgba(254,205,211,0)}.dark\:border-rose-200\/10:is(.dark *){border-color:rgba(254,205,211,.1)}.dark\:border-rose-200\/100:is(.dark *){border-color:#fecdd3}.dark\:border-rose-200\/15:is(.dark *){border-color:rgba(254,205,211,.15)}.dark\:border-rose-200\/20:is(.dark *){border-color:rgba(254,205,211,.2)}.dark\:border-rose-200\/25:is(.dark *){border-color:rgba(254,205,211,.25)}.dark\:border-rose-200\/30:is(.dark *){border-color:rgba(254,205,211,.3)}.dark\:border-rose-200\/35:is(.dark *){border-color:rgba(254,205,211,.35)}.dark\:border-rose-200\/40:is(.dark *){border-color:rgba(254,205,211,.4)}.dark\:border-rose-200\/45:is(.dark *){border-color:rgba(254,205,211,.45)}.dark\:border-rose-200\/5:is(.dark *){border-color:rgba(254,205,211,.05)}.dark\:border-rose-200\/50:is(.dark *){border-color:rgba(254,205,211,.5)}.dark\:border-rose-200\/55:is(.dark *){border-color:rgba(254,205,211,.55)}.dark\:border-rose-200\/60:is(.dark *){border-color:rgba(254,205,211,.6)}.dark\:border-rose-200\/65:is(.dark *){border-color:rgba(254,205,211,.65)}.dark\:border-rose-200\/70:is(.dark *){border-color:rgba(254,205,211,.7)}.dark\:border-rose-200\/75:is(.dark *){border-color:rgba(254,205,211,.75)}.dark\:border-rose-200\/80:is(.dark *){border-color:rgba(254,205,211,.8)}.dark\:border-rose-200\/85:is(.dark *){border-color:rgba(254,205,211,.85)}.dark\:border-rose-200\/90:is(.dark *){border-color:rgba(254,205,211,.9)}.dark\:border-rose-200\/95:is(.dark *){border-color:rgba(254,205,211,.95)}.dark\:border-rose-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(255 241 242/var(--tw-border-opacity,1))}.dark\:border-rose-50\/0:is(.dark *){border-color:rgba(255,241,242,0)}.dark\:border-rose-50\/10:is(.dark *){border-color:rgba(255,241,242,.1)}.dark\:border-rose-50\/100:is(.dark *){border-color:#fff1f2}.dark\:border-rose-50\/15:is(.dark *){border-color:rgba(255,241,242,.15)}.dark\:border-rose-50\/20:is(.dark *){border-color:rgba(255,241,242,.2)}.dark\:border-rose-50\/25:is(.dark *){border-color:rgba(255,241,242,.25)}.dark\:border-rose-50\/30:is(.dark *){border-color:rgba(255,241,242,.3)}.dark\:border-rose-50\/35:is(.dark *){border-color:rgba(255,241,242,.35)}.dark\:border-rose-50\/40:is(.dark *){border-color:rgba(255,241,242,.4)}.dark\:border-rose-50\/45:is(.dark *){border-color:rgba(255,241,242,.45)}.dark\:border-rose-50\/5:is(.dark *){border-color:rgba(255,241,242,.05)}.dark\:border-rose-50\/50:is(.dark *){border-color:rgba(255,241,242,.5)}.dark\:border-rose-50\/55:is(.dark *){border-color:rgba(255,241,242,.55)}.dark\:border-rose-50\/60:is(.dark *){border-color:rgba(255,241,242,.6)}.dark\:border-rose-50\/65:is(.dark *){border-color:rgba(255,241,242,.65)}.dark\:border-rose-50\/70:is(.dark *){border-color:rgba(255,241,242,.7)}.dark\:border-rose-50\/75:is(.dark *){border-color:rgba(255,241,242,.75)}.dark\:border-rose-50\/80:is(.dark *){border-color:rgba(255,241,242,.8)}.dark\:border-rose-50\/85:is(.dark *){border-color:rgba(255,241,242,.85)}.dark\:border-rose-50\/90:is(.dark *){border-color:rgba(255,241,242,.9)}.dark\:border-rose-50\/95:is(.dark *){border-color:rgba(255,241,242,.95)}.dark\:border-rose-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(244 63 94/var(--tw-border-opacity,1))}.dark\:border-rose-500\/0:is(.dark *){border-color:rgba(244,63,94,0)}.dark\:border-rose-500\/10:is(.dark *){border-color:rgba(244,63,94,.1)}.dark\:border-rose-500\/100:is(.dark *){border-color:#f43f5e}.dark\:border-rose-500\/15:is(.dark *){border-color:rgba(244,63,94,.15)}.dark\:border-rose-500\/20:is(.dark *){border-color:rgba(244,63,94,.2)}.dark\:border-rose-500\/25:is(.dark *){border-color:rgba(244,63,94,.25)}.dark\:border-rose-500\/30:is(.dark *){border-color:rgba(244,63,94,.3)}.dark\:border-rose-500\/35:is(.dark *){border-color:rgba(244,63,94,.35)}.dark\:border-rose-500\/40:is(.dark *){border-color:rgba(244,63,94,.4)}.dark\:border-rose-500\/45:is(.dark *){border-color:rgba(244,63,94,.45)}.dark\:border-rose-500\/5:is(.dark *){border-color:rgba(244,63,94,.05)}.dark\:border-rose-500\/50:is(.dark *){border-color:rgba(244,63,94,.5)}.dark\:border-rose-500\/55:is(.dark *){border-color:rgba(244,63,94,.55)}.dark\:border-rose-500\/60:is(.dark *){border-color:rgba(244,63,94,.6)}.dark\:border-rose-500\/65:is(.dark *){border-color:rgba(244,63,94,.65)}.dark\:border-rose-500\/70:is(.dark *){border-color:rgba(244,63,94,.7)}.dark\:border-rose-500\/75:is(.dark *){border-color:rgba(244,63,94,.75)}.dark\:border-rose-500\/80:is(.dark *){border-color:rgba(244,63,94,.8)}.dark\:border-rose-500\/85:is(.dark *){border-color:rgba(244,63,94,.85)}.dark\:border-rose-500\/90:is(.dark *){border-color:rgba(244,63,94,.9)}.dark\:border-rose-500\/95:is(.dark *){border-color:rgba(244,63,94,.95)}.dark\:border-rose-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(76 5 25/var(--tw-border-opacity,1))}.dark\:border-rose-950\/0:is(.dark *){border-color:rgba(76,5,25,0)}.dark\:border-rose-950\/10:is(.dark *){border-color:rgba(76,5,25,.1)}.dark\:border-rose-950\/100:is(.dark *){border-color:#4c0519}.dark\:border-rose-950\/15:is(.dark *){border-color:rgba(76,5,25,.15)}.dark\:border-rose-950\/20:is(.dark *){border-color:rgba(76,5,25,.2)}.dark\:border-rose-950\/25:is(.dark *){border-color:rgba(76,5,25,.25)}.dark\:border-rose-950\/30:is(.dark *){border-color:rgba(76,5,25,.3)}.dark\:border-rose-950\/35:is(.dark *){border-color:rgba(76,5,25,.35)}.dark\:border-rose-950\/40:is(.dark *){border-color:rgba(76,5,25,.4)}.dark\:border-rose-950\/45:is(.dark *){border-color:rgba(76,5,25,.45)}.dark\:border-rose-950\/5:is(.dark *){border-color:rgba(76,5,25,.05)}.dark\:border-rose-950\/50:is(.dark *){border-color:rgba(76,5,25,.5)}.dark\:border-rose-950\/55:is(.dark *){border-color:rgba(76,5,25,.55)}.dark\:border-rose-950\/60:is(.dark *){border-color:rgba(76,5,25,.6)}.dark\:border-rose-950\/65:is(.dark *){border-color:rgba(76,5,25,.65)}.dark\:border-rose-950\/70:is(.dark *){border-color:rgba(76,5,25,.7)}.dark\:border-rose-950\/75:is(.dark *){border-color:rgba(76,5,25,.75)}.dark\:border-rose-950\/80:is(.dark *){border-color:rgba(76,5,25,.8)}.dark\:border-rose-950\/85:is(.dark *){border-color:rgba(76,5,25,.85)}.dark\:border-rose-950\/90:is(.dark *){border-color:rgba(76,5,25,.9)}.dark\:border-rose-950\/95:is(.dark *){border-color:rgba(76,5,25,.95)}.dark\:border-sky-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(224 242 254/var(--tw-border-opacity,1))}.dark\:border-sky-100\/0:is(.dark *){border-color:rgba(224,242,254,0)}.dark\:border-sky-100\/10:is(.dark *){border-color:rgba(224,242,254,.1)}.dark\:border-sky-100\/100:is(.dark *){border-color:#e0f2fe}.dark\:border-sky-100\/15:is(.dark *){border-color:rgba(224,242,254,.15)}.dark\:border-sky-100\/20:is(.dark *){border-color:rgba(224,242,254,.2)}.dark\:border-sky-100\/25:is(.dark *){border-color:rgba(224,242,254,.25)}.dark\:border-sky-100\/30:is(.dark *){border-color:rgba(224,242,254,.3)}.dark\:border-sky-100\/35:is(.dark *){border-color:rgba(224,242,254,.35)}.dark\:border-sky-100\/40:is(.dark *){border-color:rgba(224,242,254,.4)}.dark\:border-sky-100\/45:is(.dark *){border-color:rgba(224,242,254,.45)}.dark\:border-sky-100\/5:is(.dark *){border-color:rgba(224,242,254,.05)}.dark\:border-sky-100\/50:is(.dark *){border-color:rgba(224,242,254,.5)}.dark\:border-sky-100\/55:is(.dark *){border-color:rgba(224,242,254,.55)}.dark\:border-sky-100\/60:is(.dark *){border-color:rgba(224,242,254,.6)}.dark\:border-sky-100\/65:is(.dark *){border-color:rgba(224,242,254,.65)}.dark\:border-sky-100\/70:is(.dark *){border-color:rgba(224,242,254,.7)}.dark\:border-sky-100\/75:is(.dark *){border-color:rgba(224,242,254,.75)}.dark\:border-sky-100\/80:is(.dark *){border-color:rgba(224,242,254,.8)}.dark\:border-sky-100\/85:is(.dark *){border-color:rgba(224,242,254,.85)}.dark\:border-sky-100\/90:is(.dark *){border-color:rgba(224,242,254,.9)}.dark\:border-sky-100\/95:is(.dark *){border-color:rgba(224,242,254,.95)}.dark\:border-sky-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(186 230 253/var(--tw-border-opacity,1))}.dark\:border-sky-200\/0:is(.dark *){border-color:rgba(186,230,253,0)}.dark\:border-sky-200\/10:is(.dark *){border-color:rgba(186,230,253,.1)}.dark\:border-sky-200\/100:is(.dark *){border-color:#bae6fd}.dark\:border-sky-200\/15:is(.dark *){border-color:rgba(186,230,253,.15)}.dark\:border-sky-200\/20:is(.dark *){border-color:rgba(186,230,253,.2)}.dark\:border-sky-200\/25:is(.dark *){border-color:rgba(186,230,253,.25)}.dark\:border-sky-200\/30:is(.dark *){border-color:rgba(186,230,253,.3)}.dark\:border-sky-200\/35:is(.dark *){border-color:rgba(186,230,253,.35)}.dark\:border-sky-200\/40:is(.dark *){border-color:rgba(186,230,253,.4)}.dark\:border-sky-200\/45:is(.dark *){border-color:rgba(186,230,253,.45)}.dark\:border-sky-200\/5:is(.dark *){border-color:rgba(186,230,253,.05)}.dark\:border-sky-200\/50:is(.dark *){border-color:rgba(186,230,253,.5)}.dark\:border-sky-200\/55:is(.dark *){border-color:rgba(186,230,253,.55)}.dark\:border-sky-200\/60:is(.dark *){border-color:rgba(186,230,253,.6)}.dark\:border-sky-200\/65:is(.dark *){border-color:rgba(186,230,253,.65)}.dark\:border-sky-200\/70:is(.dark *){border-color:rgba(186,230,253,.7)}.dark\:border-sky-200\/75:is(.dark *){border-color:rgba(186,230,253,.75)}.dark\:border-sky-200\/80:is(.dark *){border-color:rgba(186,230,253,.8)}.dark\:border-sky-200\/85:is(.dark *){border-color:rgba(186,230,253,.85)}.dark\:border-sky-200\/90:is(.dark *){border-color:rgba(186,230,253,.9)}.dark\:border-sky-200\/95:is(.dark *){border-color:rgba(186,230,253,.95)}.dark\:border-sky-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(240 249 255/var(--tw-border-opacity,1))}.dark\:border-sky-50\/0:is(.dark *){border-color:rgba(240,249,255,0)}.dark\:border-sky-50\/10:is(.dark *){border-color:rgba(240,249,255,.1)}.dark\:border-sky-50\/100:is(.dark *){border-color:#f0f9ff}.dark\:border-sky-50\/15:is(.dark *){border-color:rgba(240,249,255,.15)}.dark\:border-sky-50\/20:is(.dark *){border-color:rgba(240,249,255,.2)}.dark\:border-sky-50\/25:is(.dark *){border-color:rgba(240,249,255,.25)}.dark\:border-sky-50\/30:is(.dark *){border-color:rgba(240,249,255,.3)}.dark\:border-sky-50\/35:is(.dark *){border-color:rgba(240,249,255,.35)}.dark\:border-sky-50\/40:is(.dark *){border-color:rgba(240,249,255,.4)}.dark\:border-sky-50\/45:is(.dark *){border-color:rgba(240,249,255,.45)}.dark\:border-sky-50\/5:is(.dark *){border-color:rgba(240,249,255,.05)}.dark\:border-sky-50\/50:is(.dark *){border-color:rgba(240,249,255,.5)}.dark\:border-sky-50\/55:is(.dark *){border-color:rgba(240,249,255,.55)}.dark\:border-sky-50\/60:is(.dark *){border-color:rgba(240,249,255,.6)}.dark\:border-sky-50\/65:is(.dark *){border-color:rgba(240,249,255,.65)}.dark\:border-sky-50\/70:is(.dark *){border-color:rgba(240,249,255,.7)}.dark\:border-sky-50\/75:is(.dark *){border-color:rgba(240,249,255,.75)}.dark\:border-sky-50\/80:is(.dark *){border-color:rgba(240,249,255,.8)}.dark\:border-sky-50\/85:is(.dark *){border-color:rgba(240,249,255,.85)}.dark\:border-sky-50\/90:is(.dark *){border-color:rgba(240,249,255,.9)}.dark\:border-sky-50\/95:is(.dark *){border-color:rgba(240,249,255,.95)}.dark\:border-sky-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(14 165 233/var(--tw-border-opacity,1))}.dark\:border-sky-500\/0:is(.dark *){border-color:rgba(14,165,233,0)}.dark\:border-sky-500\/10:is(.dark *){border-color:rgba(14,165,233,.1)}.dark\:border-sky-500\/100:is(.dark *){border-color:#0ea5e9}.dark\:border-sky-500\/15:is(.dark *){border-color:rgba(14,165,233,.15)}.dark\:border-sky-500\/20:is(.dark *){border-color:rgba(14,165,233,.2)}.dark\:border-sky-500\/25:is(.dark *){border-color:rgba(14,165,233,.25)}.dark\:border-sky-500\/30:is(.dark *){border-color:rgba(14,165,233,.3)}.dark\:border-sky-500\/35:is(.dark *){border-color:rgba(14,165,233,.35)}.dark\:border-sky-500\/40:is(.dark *){border-color:rgba(14,165,233,.4)}.dark\:border-sky-500\/45:is(.dark *){border-color:rgba(14,165,233,.45)}.dark\:border-sky-500\/5:is(.dark *){border-color:rgba(14,165,233,.05)}.dark\:border-sky-500\/50:is(.dark *){border-color:rgba(14,165,233,.5)}.dark\:border-sky-500\/55:is(.dark *){border-color:rgba(14,165,233,.55)}.dark\:border-sky-500\/60:is(.dark *){border-color:rgba(14,165,233,.6)}.dark\:border-sky-500\/65:is(.dark *){border-color:rgba(14,165,233,.65)}.dark\:border-sky-500\/70:is(.dark *){border-color:rgba(14,165,233,.7)}.dark\:border-sky-500\/75:is(.dark *){border-color:rgba(14,165,233,.75)}.dark\:border-sky-500\/80:is(.dark *){border-color:rgba(14,165,233,.8)}.dark\:border-sky-500\/85:is(.dark *){border-color:rgba(14,165,233,.85)}.dark\:border-sky-500\/90:is(.dark *){border-color:rgba(14,165,233,.9)}.dark\:border-sky-500\/95:is(.dark *){border-color:rgba(14,165,233,.95)}.dark\:border-sky-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(8 47 73/var(--tw-border-opacity,1))}.dark\:border-sky-950\/0:is(.dark *){border-color:rgba(8,47,73,0)}.dark\:border-sky-950\/10:is(.dark *){border-color:rgba(8,47,73,.1)}.dark\:border-sky-950\/100:is(.dark *){border-color:#082f49}.dark\:border-sky-950\/15:is(.dark *){border-color:rgba(8,47,73,.15)}.dark\:border-sky-950\/20:is(.dark *){border-color:rgba(8,47,73,.2)}.dark\:border-sky-950\/25:is(.dark *){border-color:rgba(8,47,73,.25)}.dark\:border-sky-950\/30:is(.dark *){border-color:rgba(8,47,73,.3)}.dark\:border-sky-950\/35:is(.dark *){border-color:rgba(8,47,73,.35)}.dark\:border-sky-950\/40:is(.dark *){border-color:rgba(8,47,73,.4)}.dark\:border-sky-950\/45:is(.dark *){border-color:rgba(8,47,73,.45)}.dark\:border-sky-950\/5:is(.dark *){border-color:rgba(8,47,73,.05)}.dark\:border-sky-950\/50:is(.dark *){border-color:rgba(8,47,73,.5)}.dark\:border-sky-950\/55:is(.dark *){border-color:rgba(8,47,73,.55)}.dark\:border-sky-950\/60:is(.dark *){border-color:rgba(8,47,73,.6)}.dark\:border-sky-950\/65:is(.dark *){border-color:rgba(8,47,73,.65)}.dark\:border-sky-950\/70:is(.dark *){border-color:rgba(8,47,73,.7)}.dark\:border-sky-950\/75:is(.dark *){border-color:rgba(8,47,73,.75)}.dark\:border-sky-950\/80:is(.dark *){border-color:rgba(8,47,73,.8)}.dark\:border-sky-950\/85:is(.dark *){border-color:rgba(8,47,73,.85)}.dark\:border-sky-950\/90:is(.dark *){border-color:rgba(8,47,73,.9)}.dark\:border-sky-950\/95:is(.dark *){border-color:rgba(8,47,73,.95)}.dark\:border-slate-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(241 245 249/var(--tw-border-opacity,1))}.dark\:border-slate-100\/0:is(.dark *){border-color:rgba(241,245,249,0)}.dark\:border-slate-100\/10:is(.dark *){border-color:rgba(241,245,249,.1)}.dark\:border-slate-100\/100:is(.dark *){border-color:#f1f5f9}.dark\:border-slate-100\/15:is(.dark *){border-color:rgba(241,245,249,.15)}.dark\:border-slate-100\/20:is(.dark *){border-color:rgba(241,245,249,.2)}.dark\:border-slate-100\/25:is(.dark *){border-color:rgba(241,245,249,.25)}.dark\:border-slate-100\/30:is(.dark *){border-color:rgba(241,245,249,.3)}.dark\:border-slate-100\/35:is(.dark *){border-color:rgba(241,245,249,.35)}.dark\:border-slate-100\/40:is(.dark *){border-color:rgba(241,245,249,.4)}.dark\:border-slate-100\/45:is(.dark *){border-color:rgba(241,245,249,.45)}.dark\:border-slate-100\/5:is(.dark *){border-color:rgba(241,245,249,.05)}.dark\:border-slate-100\/50:is(.dark *){border-color:rgba(241,245,249,.5)}.dark\:border-slate-100\/55:is(.dark *){border-color:rgba(241,245,249,.55)}.dark\:border-slate-100\/60:is(.dark *){border-color:rgba(241,245,249,.6)}.dark\:border-slate-100\/65:is(.dark *){border-color:rgba(241,245,249,.65)}.dark\:border-slate-100\/70:is(.dark *){border-color:rgba(241,245,249,.7)}.dark\:border-slate-100\/75:is(.dark *){border-color:rgba(241,245,249,.75)}.dark\:border-slate-100\/80:is(.dark *){border-color:rgba(241,245,249,.8)}.dark\:border-slate-100\/85:is(.dark *){border-color:rgba(241,245,249,.85)}.dark\:border-slate-100\/90:is(.dark *){border-color:rgba(241,245,249,.9)}.dark\:border-slate-100\/95:is(.dark *){border-color:rgba(241,245,249,.95)}.dark\:border-slate-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(226 232 240/var(--tw-border-opacity,1))}.dark\:border-slate-200\/0:is(.dark *){border-color:rgba(226,232,240,0)}.dark\:border-slate-200\/10:is(.dark *){border-color:rgba(226,232,240,.1)}.dark\:border-slate-200\/100:is(.dark *){border-color:#e2e8f0}.dark\:border-slate-200\/15:is(.dark *){border-color:rgba(226,232,240,.15)}.dark\:border-slate-200\/20:is(.dark *){border-color:rgba(226,232,240,.2)}.dark\:border-slate-200\/25:is(.dark *){border-color:rgba(226,232,240,.25)}.dark\:border-slate-200\/30:is(.dark *){border-color:rgba(226,232,240,.3)}.dark\:border-slate-200\/35:is(.dark *){border-color:rgba(226,232,240,.35)}.dark\:border-slate-200\/40:is(.dark *){border-color:rgba(226,232,240,.4)}.dark\:border-slate-200\/45:is(.dark *){border-color:rgba(226,232,240,.45)}.dark\:border-slate-200\/5:is(.dark *){border-color:rgba(226,232,240,.05)}.dark\:border-slate-200\/50:is(.dark *){border-color:rgba(226,232,240,.5)}.dark\:border-slate-200\/55:is(.dark *){border-color:rgba(226,232,240,.55)}.dark\:border-slate-200\/60:is(.dark *){border-color:rgba(226,232,240,.6)}.dark\:border-slate-200\/65:is(.dark *){border-color:rgba(226,232,240,.65)}.dark\:border-slate-200\/70:is(.dark *){border-color:rgba(226,232,240,.7)}.dark\:border-slate-200\/75:is(.dark *){border-color:rgba(226,232,240,.75)}.dark\:border-slate-200\/80:is(.dark *){border-color:rgba(226,232,240,.8)}.dark\:border-slate-200\/85:is(.dark *){border-color:rgba(226,232,240,.85)}.dark\:border-slate-200\/90:is(.dark *){border-color:rgba(226,232,240,.9)}.dark\:border-slate-200\/95:is(.dark *){border-color:rgba(226,232,240,.95)}.dark\:border-slate-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(248 250 252/var(--tw-border-opacity,1))}.dark\:border-slate-50\/0:is(.dark *){border-color:rgba(248,250,252,0)}.dark\:border-slate-50\/10:is(.dark *){border-color:rgba(248,250,252,.1)}.dark\:border-slate-50\/100:is(.dark *){border-color:#f8fafc}.dark\:border-slate-50\/15:is(.dark *){border-color:rgba(248,250,252,.15)}.dark\:border-slate-50\/20:is(.dark *){border-color:rgba(248,250,252,.2)}.dark\:border-slate-50\/25:is(.dark *){border-color:rgba(248,250,252,.25)}.dark\:border-slate-50\/30:is(.dark *){border-color:rgba(248,250,252,.3)}.dark\:border-slate-50\/35:is(.dark *){border-color:rgba(248,250,252,.35)}.dark\:border-slate-50\/40:is(.dark *){border-color:rgba(248,250,252,.4)}.dark\:border-slate-50\/45:is(.dark *){border-color:rgba(248,250,252,.45)}.dark\:border-slate-50\/5:is(.dark *){border-color:rgba(248,250,252,.05)}.dark\:border-slate-50\/50:is(.dark *){border-color:rgba(248,250,252,.5)}.dark\:border-slate-50\/55:is(.dark *){border-color:rgba(248,250,252,.55)}.dark\:border-slate-50\/60:is(.dark *){border-color:rgba(248,250,252,.6)}.dark\:border-slate-50\/65:is(.dark *){border-color:rgba(248,250,252,.65)}.dark\:border-slate-50\/70:is(.dark *){border-color:rgba(248,250,252,.7)}.dark\:border-slate-50\/75:is(.dark *){border-color:rgba(248,250,252,.75)}.dark\:border-slate-50\/80:is(.dark *){border-color:rgba(248,250,252,.8)}.dark\:border-slate-50\/85:is(.dark *){border-color:rgba(248,250,252,.85)}.dark\:border-slate-50\/90:is(.dark *){border-color:rgba(248,250,252,.9)}.dark\:border-slate-50\/95:is(.dark *){border-color:rgba(248,250,252,.95)}.dark\:border-slate-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(100 116 139/var(--tw-border-opacity,1))}.dark\:border-slate-500\/0:is(.dark *){border-color:rgba(100,116,139,0)}.dark\:border-slate-500\/10:is(.dark *){border-color:rgba(100,116,139,.1)}.dark\:border-slate-500\/100:is(.dark *){border-color:#64748b}.dark\:border-slate-500\/15:is(.dark *){border-color:rgba(100,116,139,.15)}.dark\:border-slate-500\/20:is(.dark *){border-color:rgba(100,116,139,.2)}.dark\:border-slate-500\/25:is(.dark *){border-color:rgba(100,116,139,.25)}.dark\:border-slate-500\/30:is(.dark *){border-color:rgba(100,116,139,.3)}.dark\:border-slate-500\/35:is(.dark *){border-color:rgba(100,116,139,.35)}.dark\:border-slate-500\/40:is(.dark *){border-color:rgba(100,116,139,.4)}.dark\:border-slate-500\/45:is(.dark *){border-color:rgba(100,116,139,.45)}.dark\:border-slate-500\/5:is(.dark *){border-color:rgba(100,116,139,.05)}.dark\:border-slate-500\/50:is(.dark *){border-color:rgba(100,116,139,.5)}.dark\:border-slate-500\/55:is(.dark *){border-color:rgba(100,116,139,.55)}.dark\:border-slate-500\/60:is(.dark *){border-color:rgba(100,116,139,.6)}.dark\:border-slate-500\/65:is(.dark *){border-color:rgba(100,116,139,.65)}.dark\:border-slate-500\/70:is(.dark *){border-color:rgba(100,116,139,.7)}.dark\:border-slate-500\/75:is(.dark *){border-color:rgba(100,116,139,.75)}.dark\:border-slate-500\/80:is(.dark *){border-color:rgba(100,116,139,.8)}.dark\:border-slate-500\/85:is(.dark *){border-color:rgba(100,116,139,.85)}.dark\:border-slate-500\/90:is(.dark *){border-color:rgba(100,116,139,.9)}.dark\:border-slate-500\/95:is(.dark *){border-color:rgba(100,116,139,.95)}.dark\:border-slate-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(2 6 23/var(--tw-border-opacity,1))}.dark\:border-slate-950\/0:is(.dark *){border-color:rgba(2,6,23,0)}.dark\:border-slate-950\/10:is(.dark *){border-color:rgba(2,6,23,.1)}.dark\:border-slate-950\/100:is(.dark *){border-color:#020617}.dark\:border-slate-950\/15:is(.dark *){border-color:rgba(2,6,23,.15)}.dark\:border-slate-950\/20:is(.dark *){border-color:rgba(2,6,23,.2)}.dark\:border-slate-950\/25:is(.dark *){border-color:rgba(2,6,23,.25)}.dark\:border-slate-950\/30:is(.dark *){border-color:rgba(2,6,23,.3)}.dark\:border-slate-950\/35:is(.dark *){border-color:rgba(2,6,23,.35)}.dark\:border-slate-950\/40:is(.dark *){border-color:rgba(2,6,23,.4)}.dark\:border-slate-950\/45:is(.dark *){border-color:rgba(2,6,23,.45)}.dark\:border-slate-950\/5:is(.dark *){border-color:rgba(2,6,23,.05)}.dark\:border-slate-950\/50:is(.dark *){border-color:rgba(2,6,23,.5)}.dark\:border-slate-950\/55:is(.dark *){border-color:rgba(2,6,23,.55)}.dark\:border-slate-950\/60:is(.dark *){border-color:rgba(2,6,23,.6)}.dark\:border-slate-950\/65:is(.dark *){border-color:rgba(2,6,23,.65)}.dark\:border-slate-950\/70:is(.dark *){border-color:rgba(2,6,23,.7)}.dark\:border-slate-950\/75:is(.dark *){border-color:rgba(2,6,23,.75)}.dark\:border-slate-950\/80:is(.dark *){border-color:rgba(2,6,23,.8)}.dark\:border-slate-950\/85:is(.dark *){border-color:rgba(2,6,23,.85)}.dark\:border-slate-950\/90:is(.dark *){border-color:rgba(2,6,23,.9)}.dark\:border-slate-950\/95:is(.dark *){border-color:rgba(2,6,23,.95)}.dark\:border-stone-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(245 245 244/var(--tw-border-opacity,1))}.dark\:border-stone-100\/0:is(.dark *){border-color:hsla(60,5%,96%,0)}.dark\:border-stone-100\/10:is(.dark *){border-color:hsla(60,5%,96%,.1)}.dark\:border-stone-100\/100:is(.dark *){border-color:#f5f5f4}.dark\:border-stone-100\/15:is(.dark *){border-color:hsla(60,5%,96%,.15)}.dark\:border-stone-100\/20:is(.dark *){border-color:hsla(60,5%,96%,.2)}.dark\:border-stone-100\/25:is(.dark *){border-color:hsla(60,5%,96%,.25)}.dark\:border-stone-100\/30:is(.dark *){border-color:hsla(60,5%,96%,.3)}.dark\:border-stone-100\/35:is(.dark *){border-color:hsla(60,5%,96%,.35)}.dark\:border-stone-100\/40:is(.dark *){border-color:hsla(60,5%,96%,.4)}.dark\:border-stone-100\/45:is(.dark *){border-color:hsla(60,5%,96%,.45)}.dark\:border-stone-100\/5:is(.dark *){border-color:hsla(60,5%,96%,.05)}.dark\:border-stone-100\/50:is(.dark *){border-color:hsla(60,5%,96%,.5)}.dark\:border-stone-100\/55:is(.dark *){border-color:hsla(60,5%,96%,.55)}.dark\:border-stone-100\/60:is(.dark *){border-color:hsla(60,5%,96%,.6)}.dark\:border-stone-100\/65:is(.dark *){border-color:hsla(60,5%,96%,.65)}.dark\:border-stone-100\/70:is(.dark *){border-color:hsla(60,5%,96%,.7)}.dark\:border-stone-100\/75:is(.dark *){border-color:hsla(60,5%,96%,.75)}.dark\:border-stone-100\/80:is(.dark *){border-color:hsla(60,5%,96%,.8)}.dark\:border-stone-100\/85:is(.dark *){border-color:hsla(60,5%,96%,.85)}.dark\:border-stone-100\/90:is(.dark *){border-color:hsla(60,5%,96%,.9)}.dark\:border-stone-100\/95:is(.dark *){border-color:hsla(60,5%,96%,.95)}.dark\:border-stone-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(231 229 228/var(--tw-border-opacity,1))}.dark\:border-stone-200\/0:is(.dark *){border-color:hsla(20,6%,90%,0)}.dark\:border-stone-200\/10:is(.dark *){border-color:hsla(20,6%,90%,.1)}.dark\:border-stone-200\/100:is(.dark *){border-color:#e7e5e4}.dark\:border-stone-200\/15:is(.dark *){border-color:hsla(20,6%,90%,.15)}.dark\:border-stone-200\/20:is(.dark *){border-color:hsla(20,6%,90%,.2)}.dark\:border-stone-200\/25:is(.dark *){border-color:hsla(20,6%,90%,.25)}.dark\:border-stone-200\/30:is(.dark *){border-color:hsla(20,6%,90%,.3)}.dark\:border-stone-200\/35:is(.dark *){border-color:hsla(20,6%,90%,.35)}.dark\:border-stone-200\/40:is(.dark *){border-color:hsla(20,6%,90%,.4)}.dark\:border-stone-200\/45:is(.dark *){border-color:hsla(20,6%,90%,.45)}.dark\:border-stone-200\/5:is(.dark *){border-color:hsla(20,6%,90%,.05)}.dark\:border-stone-200\/50:is(.dark *){border-color:hsla(20,6%,90%,.5)}.dark\:border-stone-200\/55:is(.dark *){border-color:hsla(20,6%,90%,.55)}.dark\:border-stone-200\/60:is(.dark *){border-color:hsla(20,6%,90%,.6)}.dark\:border-stone-200\/65:is(.dark *){border-color:hsla(20,6%,90%,.65)}.dark\:border-stone-200\/70:is(.dark *){border-color:hsla(20,6%,90%,.7)}.dark\:border-stone-200\/75:is(.dark *){border-color:hsla(20,6%,90%,.75)}.dark\:border-stone-200\/80:is(.dark *){border-color:hsla(20,6%,90%,.8)}.dark\:border-stone-200\/85:is(.dark *){border-color:hsla(20,6%,90%,.85)}.dark\:border-stone-200\/90:is(.dark *){border-color:hsla(20,6%,90%,.9)}.dark\:border-stone-200\/95:is(.dark *){border-color:hsla(20,6%,90%,.95)}.dark\:border-stone-300:is(.dark *){--tw-border-opacity:1;border-color:rgb(214 211 209/var(--tw-border-opacity,1))}.dark\:border-stone-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(250 250 249/var(--tw-border-opacity,1))}.dark\:border-stone-50\/0:is(.dark *){border-color:hsla(60,9%,98%,0)}.dark\:border-stone-50\/10:is(.dark *){border-color:hsla(60,9%,98%,.1)}.dark\:border-stone-50\/100:is(.dark *){border-color:#fafaf9}.dark\:border-stone-50\/15:is(.dark *){border-color:hsla(60,9%,98%,.15)}.dark\:border-stone-50\/20:is(.dark *){border-color:hsla(60,9%,98%,.2)}.dark\:border-stone-50\/25:is(.dark *){border-color:hsla(60,9%,98%,.25)}.dark\:border-stone-50\/30:is(.dark *){border-color:hsla(60,9%,98%,.3)}.dark\:border-stone-50\/35:is(.dark *){border-color:hsla(60,9%,98%,.35)}.dark\:border-stone-50\/40:is(.dark *){border-color:hsla(60,9%,98%,.4)}.dark\:border-stone-50\/45:is(.dark *){border-color:hsla(60,9%,98%,.45)}.dark\:border-stone-50\/5:is(.dark *){border-color:hsla(60,9%,98%,.05)}.dark\:border-stone-50\/50:is(.dark *){border-color:hsla(60,9%,98%,.5)}.dark\:border-stone-50\/55:is(.dark *){border-color:hsla(60,9%,98%,.55)}.dark\:border-stone-50\/60:is(.dark *){border-color:hsla(60,9%,98%,.6)}.dark\:border-stone-50\/65:is(.dark *){border-color:hsla(60,9%,98%,.65)}.dark\:border-stone-50\/70:is(.dark *){border-color:hsla(60,9%,98%,.7)}.dark\:border-stone-50\/75:is(.dark *){border-color:hsla(60,9%,98%,.75)}.dark\:border-stone-50\/80:is(.dark *){border-color:hsla(60,9%,98%,.8)}.dark\:border-stone-50\/85:is(.dark *){border-color:hsla(60,9%,98%,.85)}.dark\:border-stone-50\/90:is(.dark *){border-color:hsla(60,9%,98%,.9)}.dark\:border-stone-50\/95:is(.dark *){border-color:hsla(60,9%,98%,.95)}.dark\:border-stone-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(120 113 108/var(--tw-border-opacity,1))}.dark\:border-stone-500\/0:is(.dark *){border-color:hsla(25,5%,45%,0)}.dark\:border-stone-500\/10:is(.dark *){border-color:hsla(25,5%,45%,.1)}.dark\:border-stone-500\/100:is(.dark *){border-color:#78716c}.dark\:border-stone-500\/15:is(.dark *){border-color:hsla(25,5%,45%,.15)}.dark\:border-stone-500\/20:is(.dark *){border-color:hsla(25,5%,45%,.2)}.dark\:border-stone-500\/25:is(.dark *){border-color:hsla(25,5%,45%,.25)}.dark\:border-stone-500\/30:is(.dark *){border-color:hsla(25,5%,45%,.3)}.dark\:border-stone-500\/35:is(.dark *){border-color:hsla(25,5%,45%,.35)}.dark\:border-stone-500\/40:is(.dark *){border-color:hsla(25,5%,45%,.4)}.dark\:border-stone-500\/45:is(.dark *){border-color:hsla(25,5%,45%,.45)}.dark\:border-stone-500\/5:is(.dark *){border-color:hsla(25,5%,45%,.05)}.dark\:border-stone-500\/50:is(.dark *){border-color:hsla(25,5%,45%,.5)}.dark\:border-stone-500\/55:is(.dark *){border-color:hsla(25,5%,45%,.55)}.dark\:border-stone-500\/60:is(.dark *){border-color:hsla(25,5%,45%,.6)}.dark\:border-stone-500\/65:is(.dark *){border-color:hsla(25,5%,45%,.65)}.dark\:border-stone-500\/70:is(.dark *){border-color:hsla(25,5%,45%,.7)}.dark\:border-stone-500\/75:is(.dark *){border-color:hsla(25,5%,45%,.75)}.dark\:border-stone-500\/80:is(.dark *){border-color:hsla(25,5%,45%,.8)}.dark\:border-stone-500\/85:is(.dark *){border-color:hsla(25,5%,45%,.85)}.dark\:border-stone-500\/90:is(.dark *){border-color:hsla(25,5%,45%,.9)}.dark\:border-stone-500\/95:is(.dark *){border-color:hsla(25,5%,45%,.95)}.dark\:border-stone-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(12 10 9/var(--tw-border-opacity,1))}.dark\:border-stone-950\/0:is(.dark *){border-color:rgba(12,10,9,0)}.dark\:border-stone-950\/10:is(.dark *){border-color:rgba(12,10,9,.1)}.dark\:border-stone-950\/100:is(.dark *){border-color:#0c0a09}.dark\:border-stone-950\/15:is(.dark *){border-color:rgba(12,10,9,.15)}.dark\:border-stone-950\/20:is(.dark *){border-color:rgba(12,10,9,.2)}.dark\:border-stone-950\/25:is(.dark *){border-color:rgba(12,10,9,.25)}.dark\:border-stone-950\/30:is(.dark *){border-color:rgba(12,10,9,.3)}.dark\:border-stone-950\/35:is(.dark *){border-color:rgba(12,10,9,.35)}.dark\:border-stone-950\/40:is(.dark *){border-color:rgba(12,10,9,.4)}.dark\:border-stone-950\/45:is(.dark *){border-color:rgba(12,10,9,.45)}.dark\:border-stone-950\/5:is(.dark *){border-color:rgba(12,10,9,.05)}.dark\:border-stone-950\/50:is(.dark *){border-color:rgba(12,10,9,.5)}.dark\:border-stone-950\/55:is(.dark *){border-color:rgba(12,10,9,.55)}.dark\:border-stone-950\/60:is(.dark *){border-color:rgba(12,10,9,.6)}.dark\:border-stone-950\/65:is(.dark *){border-color:rgba(12,10,9,.65)}.dark\:border-stone-950\/70:is(.dark *){border-color:rgba(12,10,9,.7)}.dark\:border-stone-950\/75:is(.dark *){border-color:rgba(12,10,9,.75)}.dark\:border-stone-950\/80:is(.dark *){border-color:rgba(12,10,9,.8)}.dark\:border-stone-950\/85:is(.dark *){border-color:rgba(12,10,9,.85)}.dark\:border-stone-950\/90:is(.dark *){border-color:rgba(12,10,9,.9)}.dark\:border-stone-950\/95:is(.dark *){border-color:rgba(12,10,9,.95)}.dark\:border-teal-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(204 251 241/var(--tw-border-opacity,1))}.dark\:border-teal-100\/0:is(.dark *){border-color:rgba(204,251,241,0)}.dark\:border-teal-100\/10:is(.dark *){border-color:rgba(204,251,241,.1)}.dark\:border-teal-100\/100:is(.dark *){border-color:#ccfbf1}.dark\:border-teal-100\/15:is(.dark *){border-color:rgba(204,251,241,.15)}.dark\:border-teal-100\/20:is(.dark *){border-color:rgba(204,251,241,.2)}.dark\:border-teal-100\/25:is(.dark *){border-color:rgba(204,251,241,.25)}.dark\:border-teal-100\/30:is(.dark *){border-color:rgba(204,251,241,.3)}.dark\:border-teal-100\/35:is(.dark *){border-color:rgba(204,251,241,.35)}.dark\:border-teal-100\/40:is(.dark *){border-color:rgba(204,251,241,.4)}.dark\:border-teal-100\/45:is(.dark *){border-color:rgba(204,251,241,.45)}.dark\:border-teal-100\/5:is(.dark *){border-color:rgba(204,251,241,.05)}.dark\:border-teal-100\/50:is(.dark *){border-color:rgba(204,251,241,.5)}.dark\:border-teal-100\/55:is(.dark *){border-color:rgba(204,251,241,.55)}.dark\:border-teal-100\/60:is(.dark *){border-color:rgba(204,251,241,.6)}.dark\:border-teal-100\/65:is(.dark *){border-color:rgba(204,251,241,.65)}.dark\:border-teal-100\/70:is(.dark *){border-color:rgba(204,251,241,.7)}.dark\:border-teal-100\/75:is(.dark *){border-color:rgba(204,251,241,.75)}.dark\:border-teal-100\/80:is(.dark *){border-color:rgba(204,251,241,.8)}.dark\:border-teal-100\/85:is(.dark *){border-color:rgba(204,251,241,.85)}.dark\:border-teal-100\/90:is(.dark *){border-color:rgba(204,251,241,.9)}.dark\:border-teal-100\/95:is(.dark *){border-color:rgba(204,251,241,.95)}.dark\:border-teal-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(153 246 228/var(--tw-border-opacity,1))}.dark\:border-teal-200\/0:is(.dark *){border-color:rgba(153,246,228,0)}.dark\:border-teal-200\/10:is(.dark *){border-color:rgba(153,246,228,.1)}.dark\:border-teal-200\/100:is(.dark *){border-color:#99f6e4}.dark\:border-teal-200\/15:is(.dark *){border-color:rgba(153,246,228,.15)}.dark\:border-teal-200\/20:is(.dark *){border-color:rgba(153,246,228,.2)}.dark\:border-teal-200\/25:is(.dark *){border-color:rgba(153,246,228,.25)}.dark\:border-teal-200\/30:is(.dark *){border-color:rgba(153,246,228,.3)}.dark\:border-teal-200\/35:is(.dark *){border-color:rgba(153,246,228,.35)}.dark\:border-teal-200\/40:is(.dark *){border-color:rgba(153,246,228,.4)}.dark\:border-teal-200\/45:is(.dark *){border-color:rgba(153,246,228,.45)}.dark\:border-teal-200\/5:is(.dark *){border-color:rgba(153,246,228,.05)}.dark\:border-teal-200\/50:is(.dark *){border-color:rgba(153,246,228,.5)}.dark\:border-teal-200\/55:is(.dark *){border-color:rgba(153,246,228,.55)}.dark\:border-teal-200\/60:is(.dark *){border-color:rgba(153,246,228,.6)}.dark\:border-teal-200\/65:is(.dark *){border-color:rgba(153,246,228,.65)}.dark\:border-teal-200\/70:is(.dark *){border-color:rgba(153,246,228,.7)}.dark\:border-teal-200\/75:is(.dark *){border-color:rgba(153,246,228,.75)}.dark\:border-teal-200\/80:is(.dark *){border-color:rgba(153,246,228,.8)}.dark\:border-teal-200\/85:is(.dark *){border-color:rgba(153,246,228,.85)}.dark\:border-teal-200\/90:is(.dark *){border-color:rgba(153,246,228,.9)}.dark\:border-teal-200\/95:is(.dark *){border-color:rgba(153,246,228,.95)}.dark\:border-teal-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(240 253 250/var(--tw-border-opacity,1))}.dark\:border-teal-50\/0:is(.dark *){border-color:rgba(240,253,250,0)}.dark\:border-teal-50\/10:is(.dark *){border-color:rgba(240,253,250,.1)}.dark\:border-teal-50\/100:is(.dark *){border-color:#f0fdfa}.dark\:border-teal-50\/15:is(.dark *){border-color:rgba(240,253,250,.15)}.dark\:border-teal-50\/20:is(.dark *){border-color:rgba(240,253,250,.2)}.dark\:border-teal-50\/25:is(.dark *){border-color:rgba(240,253,250,.25)}.dark\:border-teal-50\/30:is(.dark *){border-color:rgba(240,253,250,.3)}.dark\:border-teal-50\/35:is(.dark *){border-color:rgba(240,253,250,.35)}.dark\:border-teal-50\/40:is(.dark *){border-color:rgba(240,253,250,.4)}.dark\:border-teal-50\/45:is(.dark *){border-color:rgba(240,253,250,.45)}.dark\:border-teal-50\/5:is(.dark *){border-color:rgba(240,253,250,.05)}.dark\:border-teal-50\/50:is(.dark *){border-color:rgba(240,253,250,.5)}.dark\:border-teal-50\/55:is(.dark *){border-color:rgba(240,253,250,.55)}.dark\:border-teal-50\/60:is(.dark *){border-color:rgba(240,253,250,.6)}.dark\:border-teal-50\/65:is(.dark *){border-color:rgba(240,253,250,.65)}.dark\:border-teal-50\/70:is(.dark *){border-color:rgba(240,253,250,.7)}.dark\:border-teal-50\/75:is(.dark *){border-color:rgba(240,253,250,.75)}.dark\:border-teal-50\/80:is(.dark *){border-color:rgba(240,253,250,.8)}.dark\:border-teal-50\/85:is(.dark *){border-color:rgba(240,253,250,.85)}.dark\:border-teal-50\/90:is(.dark *){border-color:rgba(240,253,250,.9)}.dark\:border-teal-50\/95:is(.dark *){border-color:rgba(240,253,250,.95)}.dark\:border-teal-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(20 184 166/var(--tw-border-opacity,1))}.dark\:border-teal-500\/0:is(.dark *){border-color:rgba(20,184,166,0)}.dark\:border-teal-500\/10:is(.dark *){border-color:rgba(20,184,166,.1)}.dark\:border-teal-500\/100:is(.dark *){border-color:#14b8a6}.dark\:border-teal-500\/15:is(.dark *){border-color:rgba(20,184,166,.15)}.dark\:border-teal-500\/20:is(.dark *){border-color:rgba(20,184,166,.2)}.dark\:border-teal-500\/25:is(.dark *){border-color:rgba(20,184,166,.25)}.dark\:border-teal-500\/30:is(.dark *){border-color:rgba(20,184,166,.3)}.dark\:border-teal-500\/35:is(.dark *){border-color:rgba(20,184,166,.35)}.dark\:border-teal-500\/40:is(.dark *){border-color:rgba(20,184,166,.4)}.dark\:border-teal-500\/45:is(.dark *){border-color:rgba(20,184,166,.45)}.dark\:border-teal-500\/5:is(.dark *){border-color:rgba(20,184,166,.05)}.dark\:border-teal-500\/50:is(.dark *){border-color:rgba(20,184,166,.5)}.dark\:border-teal-500\/55:is(.dark *){border-color:rgba(20,184,166,.55)}.dark\:border-teal-500\/60:is(.dark *){border-color:rgba(20,184,166,.6)}.dark\:border-teal-500\/65:is(.dark *){border-color:rgba(20,184,166,.65)}.dark\:border-teal-500\/70:is(.dark *){border-color:rgba(20,184,166,.7)}.dark\:border-teal-500\/75:is(.dark *){border-color:rgba(20,184,166,.75)}.dark\:border-teal-500\/80:is(.dark *){border-color:rgba(20,184,166,.8)}.dark\:border-teal-500\/85:is(.dark *){border-color:rgba(20,184,166,.85)}.dark\:border-teal-500\/90:is(.dark *){border-color:rgba(20,184,166,.9)}.dark\:border-teal-500\/95:is(.dark *){border-color:rgba(20,184,166,.95)}.dark\:border-teal-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(4 47 46/var(--tw-border-opacity,1))}.dark\:border-teal-950\/0:is(.dark *){border-color:rgba(4,47,46,0)}.dark\:border-teal-950\/10:is(.dark *){border-color:rgba(4,47,46,.1)}.dark\:border-teal-950\/100:is(.dark *){border-color:#042f2e}.dark\:border-teal-950\/15:is(.dark *){border-color:rgba(4,47,46,.15)}.dark\:border-teal-950\/20:is(.dark *){border-color:rgba(4,47,46,.2)}.dark\:border-teal-950\/25:is(.dark *){border-color:rgba(4,47,46,.25)}.dark\:border-teal-950\/30:is(.dark *){border-color:rgba(4,47,46,.3)}.dark\:border-teal-950\/35:is(.dark *){border-color:rgba(4,47,46,.35)}.dark\:border-teal-950\/40:is(.dark *){border-color:rgba(4,47,46,.4)}.dark\:border-teal-950\/45:is(.dark *){border-color:rgba(4,47,46,.45)}.dark\:border-teal-950\/5:is(.dark *){border-color:rgba(4,47,46,.05)}.dark\:border-teal-950\/50:is(.dark *){border-color:rgba(4,47,46,.5)}.dark\:border-teal-950\/55:is(.dark *){border-color:rgba(4,47,46,.55)}.dark\:border-teal-950\/60:is(.dark *){border-color:rgba(4,47,46,.6)}.dark\:border-teal-950\/65:is(.dark *){border-color:rgba(4,47,46,.65)}.dark\:border-teal-950\/70:is(.dark *){border-color:rgba(4,47,46,.7)}.dark\:border-teal-950\/75:is(.dark *){border-color:rgba(4,47,46,.75)}.dark\:border-teal-950\/80:is(.dark *){border-color:rgba(4,47,46,.8)}.dark\:border-teal-950\/85:is(.dark *){border-color:rgba(4,47,46,.85)}.dark\:border-teal-950\/90:is(.dark *){border-color:rgba(4,47,46,.9)}.dark\:border-teal-950\/95:is(.dark *){border-color:rgba(4,47,46,.95)}.dark\:border-violet-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(237 233 254/var(--tw-border-opacity,1))}.dark\:border-violet-100\/0:is(.dark *){border-color:rgba(237,233,254,0)}.dark\:border-violet-100\/10:is(.dark *){border-color:rgba(237,233,254,.1)}.dark\:border-violet-100\/100:is(.dark *){border-color:#ede9fe}.dark\:border-violet-100\/15:is(.dark *){border-color:rgba(237,233,254,.15)}.dark\:border-violet-100\/20:is(.dark *){border-color:rgba(237,233,254,.2)}.dark\:border-violet-100\/25:is(.dark *){border-color:rgba(237,233,254,.25)}.dark\:border-violet-100\/30:is(.dark *){border-color:rgba(237,233,254,.3)}.dark\:border-violet-100\/35:is(.dark *){border-color:rgba(237,233,254,.35)}.dark\:border-violet-100\/40:is(.dark *){border-color:rgba(237,233,254,.4)}.dark\:border-violet-100\/45:is(.dark *){border-color:rgba(237,233,254,.45)}.dark\:border-violet-100\/5:is(.dark *){border-color:rgba(237,233,254,.05)}.dark\:border-violet-100\/50:is(.dark *){border-color:rgba(237,233,254,.5)}.dark\:border-violet-100\/55:is(.dark *){border-color:rgba(237,233,254,.55)}.dark\:border-violet-100\/60:is(.dark *){border-color:rgba(237,233,254,.6)}.dark\:border-violet-100\/65:is(.dark *){border-color:rgba(237,233,254,.65)}.dark\:border-violet-100\/70:is(.dark *){border-color:rgba(237,233,254,.7)}.dark\:border-violet-100\/75:is(.dark *){border-color:rgba(237,233,254,.75)}.dark\:border-violet-100\/80:is(.dark *){border-color:rgba(237,233,254,.8)}.dark\:border-violet-100\/85:is(.dark *){border-color:rgba(237,233,254,.85)}.dark\:border-violet-100\/90:is(.dark *){border-color:rgba(237,233,254,.9)}.dark\:border-violet-100\/95:is(.dark *){border-color:rgba(237,233,254,.95)}.dark\:border-violet-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(221 214 254/var(--tw-border-opacity,1))}.dark\:border-violet-200\/0:is(.dark *){border-color:rgba(221,214,254,0)}.dark\:border-violet-200\/10:is(.dark *){border-color:rgba(221,214,254,.1)}.dark\:border-violet-200\/100:is(.dark *){border-color:#ddd6fe}.dark\:border-violet-200\/15:is(.dark *){border-color:rgba(221,214,254,.15)}.dark\:border-violet-200\/20:is(.dark *){border-color:rgba(221,214,254,.2)}.dark\:border-violet-200\/25:is(.dark *){border-color:rgba(221,214,254,.25)}.dark\:border-violet-200\/30:is(.dark *){border-color:rgba(221,214,254,.3)}.dark\:border-violet-200\/35:is(.dark *){border-color:rgba(221,214,254,.35)}.dark\:border-violet-200\/40:is(.dark *){border-color:rgba(221,214,254,.4)}.dark\:border-violet-200\/45:is(.dark *){border-color:rgba(221,214,254,.45)}.dark\:border-violet-200\/5:is(.dark *){border-color:rgba(221,214,254,.05)}.dark\:border-violet-200\/50:is(.dark *){border-color:rgba(221,214,254,.5)}.dark\:border-violet-200\/55:is(.dark *){border-color:rgba(221,214,254,.55)}.dark\:border-violet-200\/60:is(.dark *){border-color:rgba(221,214,254,.6)}.dark\:border-violet-200\/65:is(.dark *){border-color:rgba(221,214,254,.65)}.dark\:border-violet-200\/70:is(.dark *){border-color:rgba(221,214,254,.7)}.dark\:border-violet-200\/75:is(.dark *){border-color:rgba(221,214,254,.75)}.dark\:border-violet-200\/80:is(.dark *){border-color:rgba(221,214,254,.8)}.dark\:border-violet-200\/85:is(.dark *){border-color:rgba(221,214,254,.85)}.dark\:border-violet-200\/90:is(.dark *){border-color:rgba(221,214,254,.9)}.dark\:border-violet-200\/95:is(.dark *){border-color:rgba(221,214,254,.95)}.dark\:border-violet-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(245 243 255/var(--tw-border-opacity,1))}.dark\:border-violet-50\/0:is(.dark *){border-color:rgba(245,243,255,0)}.dark\:border-violet-50\/10:is(.dark *){border-color:rgba(245,243,255,.1)}.dark\:border-violet-50\/100:is(.dark *){border-color:#f5f3ff}.dark\:border-violet-50\/15:is(.dark *){border-color:rgba(245,243,255,.15)}.dark\:border-violet-50\/20:is(.dark *){border-color:rgba(245,243,255,.2)}.dark\:border-violet-50\/25:is(.dark *){border-color:rgba(245,243,255,.25)}.dark\:border-violet-50\/30:is(.dark *){border-color:rgba(245,243,255,.3)}.dark\:border-violet-50\/35:is(.dark *){border-color:rgba(245,243,255,.35)}.dark\:border-violet-50\/40:is(.dark *){border-color:rgba(245,243,255,.4)}.dark\:border-violet-50\/45:is(.dark *){border-color:rgba(245,243,255,.45)}.dark\:border-violet-50\/5:is(.dark *){border-color:rgba(245,243,255,.05)}.dark\:border-violet-50\/50:is(.dark *){border-color:rgba(245,243,255,.5)}.dark\:border-violet-50\/55:is(.dark *){border-color:rgba(245,243,255,.55)}.dark\:border-violet-50\/60:is(.dark *){border-color:rgba(245,243,255,.6)}.dark\:border-violet-50\/65:is(.dark *){border-color:rgba(245,243,255,.65)}.dark\:border-violet-50\/70:is(.dark *){border-color:rgba(245,243,255,.7)}.dark\:border-violet-50\/75:is(.dark *){border-color:rgba(245,243,255,.75)}.dark\:border-violet-50\/80:is(.dark *){border-color:rgba(245,243,255,.8)}.dark\:border-violet-50\/85:is(.dark *){border-color:rgba(245,243,255,.85)}.dark\:border-violet-50\/90:is(.dark *){border-color:rgba(245,243,255,.9)}.dark\:border-violet-50\/95:is(.dark *){border-color:rgba(245,243,255,.95)}.dark\:border-violet-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(139 92 246/var(--tw-border-opacity,1))}.dark\:border-violet-500\/0:is(.dark *){border-color:rgba(139,92,246,0)}.dark\:border-violet-500\/10:is(.dark *){border-color:rgba(139,92,246,.1)}.dark\:border-violet-500\/100:is(.dark *){border-color:#8b5cf6}.dark\:border-violet-500\/15:is(.dark *){border-color:rgba(139,92,246,.15)}.dark\:border-violet-500\/20:is(.dark *){border-color:rgba(139,92,246,.2)}.dark\:border-violet-500\/25:is(.dark *){border-color:rgba(139,92,246,.25)}.dark\:border-violet-500\/30:is(.dark *){border-color:rgba(139,92,246,.3)}.dark\:border-violet-500\/35:is(.dark *){border-color:rgba(139,92,246,.35)}.dark\:border-violet-500\/40:is(.dark *){border-color:rgba(139,92,246,.4)}.dark\:border-violet-500\/45:is(.dark *){border-color:rgba(139,92,246,.45)}.dark\:border-violet-500\/5:is(.dark *){border-color:rgba(139,92,246,.05)}.dark\:border-violet-500\/50:is(.dark *){border-color:rgba(139,92,246,.5)}.dark\:border-violet-500\/55:is(.dark *){border-color:rgba(139,92,246,.55)}.dark\:border-violet-500\/60:is(.dark *){border-color:rgba(139,92,246,.6)}.dark\:border-violet-500\/65:is(.dark *){border-color:rgba(139,92,246,.65)}.dark\:border-violet-500\/70:is(.dark *){border-color:rgba(139,92,246,.7)}.dark\:border-violet-500\/75:is(.dark *){border-color:rgba(139,92,246,.75)}.dark\:border-violet-500\/80:is(.dark *){border-color:rgba(139,92,246,.8)}.dark\:border-violet-500\/85:is(.dark *){border-color:rgba(139,92,246,.85)}.dark\:border-violet-500\/90:is(.dark *){border-color:rgba(139,92,246,.9)}.dark\:border-violet-500\/95:is(.dark *){border-color:rgba(139,92,246,.95)}.dark\:border-violet-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(46 16 101/var(--tw-border-opacity,1))}.dark\:border-violet-950\/0:is(.dark *){border-color:rgba(46,16,101,0)}.dark\:border-violet-950\/10:is(.dark *){border-color:rgba(46,16,101,.1)}.dark\:border-violet-950\/100:is(.dark *){border-color:#2e1065}.dark\:border-violet-950\/15:is(.dark *){border-color:rgba(46,16,101,.15)}.dark\:border-violet-950\/20:is(.dark *){border-color:rgba(46,16,101,.2)}.dark\:border-violet-950\/25:is(.dark *){border-color:rgba(46,16,101,.25)}.dark\:border-violet-950\/30:is(.dark *){border-color:rgba(46,16,101,.3)}.dark\:border-violet-950\/35:is(.dark *){border-color:rgba(46,16,101,.35)}.dark\:border-violet-950\/40:is(.dark *){border-color:rgba(46,16,101,.4)}.dark\:border-violet-950\/45:is(.dark *){border-color:rgba(46,16,101,.45)}.dark\:border-violet-950\/5:is(.dark *){border-color:rgba(46,16,101,.05)}.dark\:border-violet-950\/50:is(.dark *){border-color:rgba(46,16,101,.5)}.dark\:border-violet-950\/55:is(.dark *){border-color:rgba(46,16,101,.55)}.dark\:border-violet-950\/60:is(.dark *){border-color:rgba(46,16,101,.6)}.dark\:border-violet-950\/65:is(.dark *){border-color:rgba(46,16,101,.65)}.dark\:border-violet-950\/70:is(.dark *){border-color:rgba(46,16,101,.7)}.dark\:border-violet-950\/75:is(.dark *){border-color:rgba(46,16,101,.75)}.dark\:border-violet-950\/80:is(.dark *){border-color:rgba(46,16,101,.8)}.dark\:border-violet-950\/85:is(.dark *){border-color:rgba(46,16,101,.85)}.dark\:border-violet-950\/90:is(.dark *){border-color:rgba(46,16,101,.9)}.dark\:border-violet-950\/95:is(.dark *){border-color:rgba(46,16,101,.95)}.dark\:border-yellow-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(254 249 195/var(--tw-border-opacity,1))}.dark\:border-yellow-100\/0:is(.dark *){border-color:hsla(55,97%,88%,0)}.dark\:border-yellow-100\/10:is(.dark *){border-color:hsla(55,97%,88%,.1)}.dark\:border-yellow-100\/100:is(.dark *){border-color:#fef9c3}.dark\:border-yellow-100\/15:is(.dark *){border-color:hsla(55,97%,88%,.15)}.dark\:border-yellow-100\/20:is(.dark *){border-color:hsla(55,97%,88%,.2)}.dark\:border-yellow-100\/25:is(.dark *){border-color:hsla(55,97%,88%,.25)}.dark\:border-yellow-100\/30:is(.dark *){border-color:hsla(55,97%,88%,.3)}.dark\:border-yellow-100\/35:is(.dark *){border-color:hsla(55,97%,88%,.35)}.dark\:border-yellow-100\/40:is(.dark *){border-color:hsla(55,97%,88%,.4)}.dark\:border-yellow-100\/45:is(.dark *){border-color:hsla(55,97%,88%,.45)}.dark\:border-yellow-100\/5:is(.dark *){border-color:hsla(55,97%,88%,.05)}.dark\:border-yellow-100\/50:is(.dark *){border-color:hsla(55,97%,88%,.5)}.dark\:border-yellow-100\/55:is(.dark *){border-color:hsla(55,97%,88%,.55)}.dark\:border-yellow-100\/60:is(.dark *){border-color:hsla(55,97%,88%,.6)}.dark\:border-yellow-100\/65:is(.dark *){border-color:hsla(55,97%,88%,.65)}.dark\:border-yellow-100\/70:is(.dark *){border-color:hsla(55,97%,88%,.7)}.dark\:border-yellow-100\/75:is(.dark *){border-color:hsla(55,97%,88%,.75)}.dark\:border-yellow-100\/80:is(.dark *){border-color:hsla(55,97%,88%,.8)}.dark\:border-yellow-100\/85:is(.dark *){border-color:hsla(55,97%,88%,.85)}.dark\:border-yellow-100\/90:is(.dark *){border-color:hsla(55,97%,88%,.9)}.dark\:border-yellow-100\/95:is(.dark *){border-color:hsla(55,97%,88%,.95)}.dark\:border-yellow-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(254 240 138/var(--tw-border-opacity,1))}.dark\:border-yellow-200\/0:is(.dark *){border-color:hsla(53,98%,77%,0)}.dark\:border-yellow-200\/10:is(.dark *){border-color:hsla(53,98%,77%,.1)}.dark\:border-yellow-200\/100:is(.dark *){border-color:#fef08a}.dark\:border-yellow-200\/15:is(.dark *){border-color:hsla(53,98%,77%,.15)}.dark\:border-yellow-200\/20:is(.dark *){border-color:hsla(53,98%,77%,.2)}.dark\:border-yellow-200\/25:is(.dark *){border-color:hsla(53,98%,77%,.25)}.dark\:border-yellow-200\/30:is(.dark *){border-color:hsla(53,98%,77%,.3)}.dark\:border-yellow-200\/35:is(.dark *){border-color:hsla(53,98%,77%,.35)}.dark\:border-yellow-200\/40:is(.dark *){border-color:hsla(53,98%,77%,.4)}.dark\:border-yellow-200\/45:is(.dark *){border-color:hsla(53,98%,77%,.45)}.dark\:border-yellow-200\/5:is(.dark *){border-color:hsla(53,98%,77%,.05)}.dark\:border-yellow-200\/50:is(.dark *){border-color:hsla(53,98%,77%,.5)}.dark\:border-yellow-200\/55:is(.dark *){border-color:hsla(53,98%,77%,.55)}.dark\:border-yellow-200\/60:is(.dark *){border-color:hsla(53,98%,77%,.6)}.dark\:border-yellow-200\/65:is(.dark *){border-color:hsla(53,98%,77%,.65)}.dark\:border-yellow-200\/70:is(.dark *){border-color:hsla(53,98%,77%,.7)}.dark\:border-yellow-200\/75:is(.dark *){border-color:hsla(53,98%,77%,.75)}.dark\:border-yellow-200\/80:is(.dark *){border-color:hsla(53,98%,77%,.8)}.dark\:border-yellow-200\/85:is(.dark *){border-color:hsla(53,98%,77%,.85)}.dark\:border-yellow-200\/90:is(.dark *){border-color:hsla(53,98%,77%,.9)}.dark\:border-yellow-200\/95:is(.dark *){border-color:hsla(53,98%,77%,.95)}.dark\:border-yellow-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(254 252 232/var(--tw-border-opacity,1))}.dark\:border-yellow-50\/0:is(.dark *){border-color:hsla(55,92%,95%,0)}.dark\:border-yellow-50\/10:is(.dark *){border-color:hsla(55,92%,95%,.1)}.dark\:border-yellow-50\/100:is(.dark *){border-color:#fefce8}.dark\:border-yellow-50\/15:is(.dark *){border-color:hsla(55,92%,95%,.15)}.dark\:border-yellow-50\/20:is(.dark *){border-color:hsla(55,92%,95%,.2)}.dark\:border-yellow-50\/25:is(.dark *){border-color:hsla(55,92%,95%,.25)}.dark\:border-yellow-50\/30:is(.dark *){border-color:hsla(55,92%,95%,.3)}.dark\:border-yellow-50\/35:is(.dark *){border-color:hsla(55,92%,95%,.35)}.dark\:border-yellow-50\/40:is(.dark *){border-color:hsla(55,92%,95%,.4)}.dark\:border-yellow-50\/45:is(.dark *){border-color:hsla(55,92%,95%,.45)}.dark\:border-yellow-50\/5:is(.dark *){border-color:hsla(55,92%,95%,.05)}.dark\:border-yellow-50\/50:is(.dark *){border-color:hsla(55,92%,95%,.5)}.dark\:border-yellow-50\/55:is(.dark *){border-color:hsla(55,92%,95%,.55)}.dark\:border-yellow-50\/60:is(.dark *){border-color:hsla(55,92%,95%,.6)}.dark\:border-yellow-50\/65:is(.dark *){border-color:hsla(55,92%,95%,.65)}.dark\:border-yellow-50\/70:is(.dark *){border-color:hsla(55,92%,95%,.7)}.dark\:border-yellow-50\/75:is(.dark *){border-color:hsla(55,92%,95%,.75)}.dark\:border-yellow-50\/80:is(.dark *){border-color:hsla(55,92%,95%,.8)}.dark\:border-yellow-50\/85:is(.dark *){border-color:hsla(55,92%,95%,.85)}.dark\:border-yellow-50\/90:is(.dark *){border-color:hsla(55,92%,95%,.9)}.dark\:border-yellow-50\/95:is(.dark *){border-color:hsla(55,92%,95%,.95)}.dark\:border-yellow-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(234 179 8/var(--tw-border-opacity,1))}.dark\:border-yellow-500\/0:is(.dark *){border-color:rgba(234,179,8,0)}.dark\:border-yellow-500\/10:is(.dark *){border-color:rgba(234,179,8,.1)}.dark\:border-yellow-500\/100:is(.dark *){border-color:#eab308}.dark\:border-yellow-500\/15:is(.dark *){border-color:rgba(234,179,8,.15)}.dark\:border-yellow-500\/20:is(.dark *){border-color:rgba(234,179,8,.2)}.dark\:border-yellow-500\/25:is(.dark *){border-color:rgba(234,179,8,.25)}.dark\:border-yellow-500\/30:is(.dark *){border-color:rgba(234,179,8,.3)}.dark\:border-yellow-500\/35:is(.dark *){border-color:rgba(234,179,8,.35)}.dark\:border-yellow-500\/40:is(.dark *){border-color:rgba(234,179,8,.4)}.dark\:border-yellow-500\/45:is(.dark *){border-color:rgba(234,179,8,.45)}.dark\:border-yellow-500\/5:is(.dark *){border-color:rgba(234,179,8,.05)}.dark\:border-yellow-500\/50:is(.dark *){border-color:rgba(234,179,8,.5)}.dark\:border-yellow-500\/55:is(.dark *){border-color:rgba(234,179,8,.55)}.dark\:border-yellow-500\/60:is(.dark *){border-color:rgba(234,179,8,.6)}.dark\:border-yellow-500\/65:is(.dark *){border-color:rgba(234,179,8,.65)}.dark\:border-yellow-500\/70:is(.dark *){border-color:rgba(234,179,8,.7)}.dark\:border-yellow-500\/75:is(.dark *){border-color:rgba(234,179,8,.75)}.dark\:border-yellow-500\/80:is(.dark *){border-color:rgba(234,179,8,.8)}.dark\:border-yellow-500\/85:is(.dark *){border-color:rgba(234,179,8,.85)}.dark\:border-yellow-500\/90:is(.dark *){border-color:rgba(234,179,8,.9)}.dark\:border-yellow-500\/95:is(.dark *){border-color:rgba(234,179,8,.95)}.dark\:border-yellow-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(66 32 6/var(--tw-border-opacity,1))}.dark\:border-yellow-950\/0:is(.dark *){border-color:rgba(66,32,6,0)}.dark\:border-yellow-950\/10:is(.dark *){border-color:rgba(66,32,6,.1)}.dark\:border-yellow-950\/100:is(.dark *){border-color:#422006}.dark\:border-yellow-950\/15:is(.dark *){border-color:rgba(66,32,6,.15)}.dark\:border-yellow-950\/20:is(.dark *){border-color:rgba(66,32,6,.2)}.dark\:border-yellow-950\/25:is(.dark *){border-color:rgba(66,32,6,.25)}.dark\:border-yellow-950\/30:is(.dark *){border-color:rgba(66,32,6,.3)}.dark\:border-yellow-950\/35:is(.dark *){border-color:rgba(66,32,6,.35)}.dark\:border-yellow-950\/40:is(.dark *){border-color:rgba(66,32,6,.4)}.dark\:border-yellow-950\/45:is(.dark *){border-color:rgba(66,32,6,.45)}.dark\:border-yellow-950\/5:is(.dark *){border-color:rgba(66,32,6,.05)}.dark\:border-yellow-950\/50:is(.dark *){border-color:rgba(66,32,6,.5)}.dark\:border-yellow-950\/55:is(.dark *){border-color:rgba(66,32,6,.55)}.dark\:border-yellow-950\/60:is(.dark *){border-color:rgba(66,32,6,.6)}.dark\:border-yellow-950\/65:is(.dark *){border-color:rgba(66,32,6,.65)}.dark\:border-yellow-950\/70:is(.dark *){border-color:rgba(66,32,6,.7)}.dark\:border-yellow-950\/75:is(.dark *){border-color:rgba(66,32,6,.75)}.dark\:border-yellow-950\/80:is(.dark *){border-color:rgba(66,32,6,.8)}.dark\:border-yellow-950\/85:is(.dark *){border-color:rgba(66,32,6,.85)}.dark\:border-yellow-950\/90:is(.dark *){border-color:rgba(66,32,6,.9)}.dark\:border-yellow-950\/95:is(.dark *){border-color:rgba(66,32,6,.95)}.dark\:border-zinc-100:is(.dark *){--tw-border-opacity:1;border-color:rgb(244 244 245/var(--tw-border-opacity,1))}.dark\:border-zinc-100\/0:is(.dark *){border-color:hsla(240,5%,96%,0)}.dark\:border-zinc-100\/10:is(.dark *){border-color:hsla(240,5%,96%,.1)}.dark\:border-zinc-100\/100:is(.dark *){border-color:#f4f4f5}.dark\:border-zinc-100\/15:is(.dark *){border-color:hsla(240,5%,96%,.15)}.dark\:border-zinc-100\/20:is(.dark *){border-color:hsla(240,5%,96%,.2)}.dark\:border-zinc-100\/25:is(.dark *){border-color:hsla(240,5%,96%,.25)}.dark\:border-zinc-100\/30:is(.dark *){border-color:hsla(240,5%,96%,.3)}.dark\:border-zinc-100\/35:is(.dark *){border-color:hsla(240,5%,96%,.35)}.dark\:border-zinc-100\/40:is(.dark *){border-color:hsla(240,5%,96%,.4)}.dark\:border-zinc-100\/45:is(.dark *){border-color:hsla(240,5%,96%,.45)}.dark\:border-zinc-100\/5:is(.dark *){border-color:hsla(240,5%,96%,.05)}.dark\:border-zinc-100\/50:is(.dark *){border-color:hsla(240,5%,96%,.5)}.dark\:border-zinc-100\/55:is(.dark *){border-color:hsla(240,5%,96%,.55)}.dark\:border-zinc-100\/60:is(.dark *){border-color:hsla(240,5%,96%,.6)}.dark\:border-zinc-100\/65:is(.dark *){border-color:hsla(240,5%,96%,.65)}.dark\:border-zinc-100\/70:is(.dark *){border-color:hsla(240,5%,96%,.7)}.dark\:border-zinc-100\/75:is(.dark *){border-color:hsla(240,5%,96%,.75)}.dark\:border-zinc-100\/80:is(.dark *){border-color:hsla(240,5%,96%,.8)}.dark\:border-zinc-100\/85:is(.dark *){border-color:hsla(240,5%,96%,.85)}.dark\:border-zinc-100\/90:is(.dark *){border-color:hsla(240,5%,96%,.9)}.dark\:border-zinc-100\/95:is(.dark *){border-color:hsla(240,5%,96%,.95)}.dark\:border-zinc-200:is(.dark *){--tw-border-opacity:1;border-color:rgb(228 228 231/var(--tw-border-opacity,1))}.dark\:border-zinc-200\/0:is(.dark *){border-color:hsla(240,6%,90%,0)}.dark\:border-zinc-200\/10:is(.dark *){border-color:hsla(240,6%,90%,.1)}.dark\:border-zinc-200\/100:is(.dark *){border-color:#e4e4e7}.dark\:border-zinc-200\/15:is(.dark *){border-color:hsla(240,6%,90%,.15)}.dark\:border-zinc-200\/20:is(.dark *){border-color:hsla(240,6%,90%,.2)}.dark\:border-zinc-200\/25:is(.dark *){border-color:hsla(240,6%,90%,.25)}.dark\:border-zinc-200\/30:is(.dark *){border-color:hsla(240,6%,90%,.3)}.dark\:border-zinc-200\/35:is(.dark *){border-color:hsla(240,6%,90%,.35)}.dark\:border-zinc-200\/40:is(.dark *){border-color:hsla(240,6%,90%,.4)}.dark\:border-zinc-200\/45:is(.dark *){border-color:hsla(240,6%,90%,.45)}.dark\:border-zinc-200\/5:is(.dark *){border-color:hsla(240,6%,90%,.05)}.dark\:border-zinc-200\/50:is(.dark *){border-color:hsla(240,6%,90%,.5)}.dark\:border-zinc-200\/55:is(.dark *){border-color:hsla(240,6%,90%,.55)}.dark\:border-zinc-200\/60:is(.dark *){border-color:hsla(240,6%,90%,.6)}.dark\:border-zinc-200\/65:is(.dark *){border-color:hsla(240,6%,90%,.65)}.dark\:border-zinc-200\/70:is(.dark *){border-color:hsla(240,6%,90%,.7)}.dark\:border-zinc-200\/75:is(.dark *){border-color:hsla(240,6%,90%,.75)}.dark\:border-zinc-200\/80:is(.dark *){border-color:hsla(240,6%,90%,.8)}.dark\:border-zinc-200\/85:is(.dark *){border-color:hsla(240,6%,90%,.85)}.dark\:border-zinc-200\/90:is(.dark *){border-color:hsla(240,6%,90%,.9)}.dark\:border-zinc-200\/95:is(.dark *){border-color:hsla(240,6%,90%,.95)}.dark\:border-zinc-50:is(.dark *){--tw-border-opacity:1;border-color:rgb(250 250 250/var(--tw-border-opacity,1))}.dark\:border-zinc-50\/0:is(.dark *){border-color:hsla(0,0%,98%,0)}.dark\:border-zinc-50\/10:is(.dark *){border-color:hsla(0,0%,98%,.1)}.dark\:border-zinc-50\/100:is(.dark *){border-color:#fafafa}.dark\:border-zinc-50\/15:is(.dark *){border-color:hsla(0,0%,98%,.15)}.dark\:border-zinc-50\/20:is(.dark *){border-color:hsla(0,0%,98%,.2)}.dark\:border-zinc-50\/25:is(.dark *){border-color:hsla(0,0%,98%,.25)}.dark\:border-zinc-50\/30:is(.dark *){border-color:hsla(0,0%,98%,.3)}.dark\:border-zinc-50\/35:is(.dark *){border-color:hsla(0,0%,98%,.35)}.dark\:border-zinc-50\/40:is(.dark *){border-color:hsla(0,0%,98%,.4)}.dark\:border-zinc-50\/45:is(.dark *){border-color:hsla(0,0%,98%,.45)}.dark\:border-zinc-50\/5:is(.dark *){border-color:hsla(0,0%,98%,.05)}.dark\:border-zinc-50\/50:is(.dark *){border-color:hsla(0,0%,98%,.5)}.dark\:border-zinc-50\/55:is(.dark *){border-color:hsla(0,0%,98%,.55)}.dark\:border-zinc-50\/60:is(.dark *){border-color:hsla(0,0%,98%,.6)}.dark\:border-zinc-50\/65:is(.dark *){border-color:hsla(0,0%,98%,.65)}.dark\:border-zinc-50\/70:is(.dark *){border-color:hsla(0,0%,98%,.7)}.dark\:border-zinc-50\/75:is(.dark *){border-color:hsla(0,0%,98%,.75)}.dark\:border-zinc-50\/80:is(.dark *){border-color:hsla(0,0%,98%,.8)}.dark\:border-zinc-50\/85:is(.dark *){border-color:hsla(0,0%,98%,.85)}.dark\:border-zinc-50\/90:is(.dark *){border-color:hsla(0,0%,98%,.9)}.dark\:border-zinc-50\/95:is(.dark *){border-color:hsla(0,0%,98%,.95)}.dark\:border-zinc-500:is(.dark *){--tw-border-opacity:1;border-color:rgb(113 113 122/var(--tw-border-opacity,1))}.dark\:border-zinc-500\/0:is(.dark *){border-color:hsla(240,4%,46%,0)}.dark\:border-zinc-500\/10:is(.dark *){border-color:hsla(240,4%,46%,.1)}.dark\:border-zinc-500\/100:is(.dark *){border-color:#71717a}.dark\:border-zinc-500\/15:is(.dark *){border-color:hsla(240,4%,46%,.15)}.dark\:border-zinc-500\/20:is(.dark *){border-color:hsla(240,4%,46%,.2)}.dark\:border-zinc-500\/25:is(.dark *){border-color:hsla(240,4%,46%,.25)}.dark\:border-zinc-500\/30:is(.dark *){border-color:hsla(240,4%,46%,.3)}.dark\:border-zinc-500\/35:is(.dark *){border-color:hsla(240,4%,46%,.35)}.dark\:border-zinc-500\/40:is(.dark *){border-color:hsla(240,4%,46%,.4)}.dark\:border-zinc-500\/45:is(.dark *){border-color:hsla(240,4%,46%,.45)}.dark\:border-zinc-500\/5:is(.dark *){border-color:hsla(240,4%,46%,.05)}.dark\:border-zinc-500\/50:is(.dark *){border-color:hsla(240,4%,46%,.5)}.dark\:border-zinc-500\/55:is(.dark *){border-color:hsla(240,4%,46%,.55)}.dark\:border-zinc-500\/60:is(.dark *){border-color:hsla(240,4%,46%,.6)}.dark\:border-zinc-500\/65:is(.dark *){border-color:hsla(240,4%,46%,.65)}.dark\:border-zinc-500\/70:is(.dark *){border-color:hsla(240,4%,46%,.7)}.dark\:border-zinc-500\/75:is(.dark *){border-color:hsla(240,4%,46%,.75)}.dark\:border-zinc-500\/80:is(.dark *){border-color:hsla(240,4%,46%,.8)}.dark\:border-zinc-500\/85:is(.dark *){border-color:hsla(240,4%,46%,.85)}.dark\:border-zinc-500\/90:is(.dark *){border-color:hsla(240,4%,46%,.9)}.dark\:border-zinc-500\/95:is(.dark *){border-color:hsla(240,4%,46%,.95)}.dark\:border-zinc-950:is(.dark *){--tw-border-opacity:1;border-color:rgb(9 9 11/var(--tw-border-opacity,1))}.dark\:border-zinc-950\/0:is(.dark *){border-color:rgba(9,9,11,0)}.dark\:border-zinc-950\/10:is(.dark *){border-color:rgba(9,9,11,.1)}.dark\:border-zinc-950\/100:is(.dark *){border-color:#09090b}.dark\:border-zinc-950\/15:is(.dark *){border-color:rgba(9,9,11,.15)}.dark\:border-zinc-950\/20:is(.dark *){border-color:rgba(9,9,11,.2)}.dark\:border-zinc-950\/25:is(.dark *){border-color:rgba(9,9,11,.25)}.dark\:border-zinc-950\/30:is(.dark *){border-color:rgba(9,9,11,.3)}.dark\:border-zinc-950\/35:is(.dark *){border-color:rgba(9,9,11,.35)}.dark\:border-zinc-950\/40:is(.dark *){border-color:rgba(9,9,11,.4)}.dark\:border-zinc-950\/45:is(.dark *){border-color:rgba(9,9,11,.45)}.dark\:border-zinc-950\/5:is(.dark *){border-color:rgba(9,9,11,.05)}.dark\:border-zinc-950\/50:is(.dark *){border-color:rgba(9,9,11,.5)}.dark\:border-zinc-950\/55:is(.dark *){border-color:rgba(9,9,11,.55)}.dark\:border-zinc-950\/60:is(.dark *){border-color:rgba(9,9,11,.6)}.dark\:border-zinc-950\/65:is(.dark *){border-color:rgba(9,9,11,.65)}.dark\:border-zinc-950\/70:is(.dark *){border-color:rgba(9,9,11,.7)}.dark\:border-zinc-950\/75:is(.dark *){border-color:rgba(9,9,11,.75)}.dark\:border-zinc-950\/80:is(.dark *){border-color:rgba(9,9,11,.8)}.dark\:border-zinc-950\/85:is(.dark *){border-color:rgba(9,9,11,.85)}.dark\:border-zinc-950\/90:is(.dark *){border-color:rgba(9,9,11,.9)}.dark\:border-zinc-950\/95:is(.dark *){border-color:rgba(9,9,11,.95)}.dark\:border-l-amber-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(254 243 199/var(--tw-border-opacity,1))}.dark\:border-l-amber-100\/0:is(.dark *){border-left-color:hsla(48,96%,89%,0)}.dark\:border-l-amber-100\/10:is(.dark *){border-left-color:hsla(48,96%,89%,.1)}.dark\:border-l-amber-100\/100:is(.dark *){border-left-color:#fef3c7}.dark\:border-l-amber-100\/15:is(.dark *){border-left-color:hsla(48,96%,89%,.15)}.dark\:border-l-amber-100\/20:is(.dark *){border-left-color:hsla(48,96%,89%,.2)}.dark\:border-l-amber-100\/25:is(.dark *){border-left-color:hsla(48,96%,89%,.25)}.dark\:border-l-amber-100\/30:is(.dark *){border-left-color:hsla(48,96%,89%,.3)}.dark\:border-l-amber-100\/35:is(.dark *){border-left-color:hsla(48,96%,89%,.35)}.dark\:border-l-amber-100\/40:is(.dark *){border-left-color:hsla(48,96%,89%,.4)}.dark\:border-l-amber-100\/45:is(.dark *){border-left-color:hsla(48,96%,89%,.45)}.dark\:border-l-amber-100\/5:is(.dark *){border-left-color:hsla(48,96%,89%,.05)}.dark\:border-l-amber-100\/50:is(.dark *){border-left-color:hsla(48,96%,89%,.5)}.dark\:border-l-amber-100\/55:is(.dark *){border-left-color:hsla(48,96%,89%,.55)}.dark\:border-l-amber-100\/60:is(.dark *){border-left-color:hsla(48,96%,89%,.6)}.dark\:border-l-amber-100\/65:is(.dark *){border-left-color:hsla(48,96%,89%,.65)}.dark\:border-l-amber-100\/70:is(.dark *){border-left-color:hsla(48,96%,89%,.7)}.dark\:border-l-amber-100\/75:is(.dark *){border-left-color:hsla(48,96%,89%,.75)}.dark\:border-l-amber-100\/80:is(.dark *){border-left-color:hsla(48,96%,89%,.8)}.dark\:border-l-amber-100\/85:is(.dark *){border-left-color:hsla(48,96%,89%,.85)}.dark\:border-l-amber-100\/90:is(.dark *){border-left-color:hsla(48,96%,89%,.9)}.dark\:border-l-amber-100\/95:is(.dark *){border-left-color:hsla(48,96%,89%,.95)}.dark\:border-l-amber-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(253 230 138/var(--tw-border-opacity,1))}.dark\:border-l-amber-200\/0:is(.dark *){border-left-color:hsla(48,97%,77%,0)}.dark\:border-l-amber-200\/10:is(.dark *){border-left-color:hsla(48,97%,77%,.1)}.dark\:border-l-amber-200\/100:is(.dark *){border-left-color:#fde68a}.dark\:border-l-amber-200\/15:is(.dark *){border-left-color:hsla(48,97%,77%,.15)}.dark\:border-l-amber-200\/20:is(.dark *){border-left-color:hsla(48,97%,77%,.2)}.dark\:border-l-amber-200\/25:is(.dark *){border-left-color:hsla(48,97%,77%,.25)}.dark\:border-l-amber-200\/30:is(.dark *){border-left-color:hsla(48,97%,77%,.3)}.dark\:border-l-amber-200\/35:is(.dark *){border-left-color:hsla(48,97%,77%,.35)}.dark\:border-l-amber-200\/40:is(.dark *){border-left-color:hsla(48,97%,77%,.4)}.dark\:border-l-amber-200\/45:is(.dark *){border-left-color:hsla(48,97%,77%,.45)}.dark\:border-l-amber-200\/5:is(.dark *){border-left-color:hsla(48,97%,77%,.05)}.dark\:border-l-amber-200\/50:is(.dark *){border-left-color:hsla(48,97%,77%,.5)}.dark\:border-l-amber-200\/55:is(.dark *){border-left-color:hsla(48,97%,77%,.55)}.dark\:border-l-amber-200\/60:is(.dark *){border-left-color:hsla(48,97%,77%,.6)}.dark\:border-l-amber-200\/65:is(.dark *){border-left-color:hsla(48,97%,77%,.65)}.dark\:border-l-amber-200\/70:is(.dark *){border-left-color:hsla(48,97%,77%,.7)}.dark\:border-l-amber-200\/75:is(.dark *){border-left-color:hsla(48,97%,77%,.75)}.dark\:border-l-amber-200\/80:is(.dark *){border-left-color:hsla(48,97%,77%,.8)}.dark\:border-l-amber-200\/85:is(.dark *){border-left-color:hsla(48,97%,77%,.85)}.dark\:border-l-amber-200\/90:is(.dark *){border-left-color:hsla(48,97%,77%,.9)}.dark\:border-l-amber-200\/95:is(.dark *){border-left-color:hsla(48,97%,77%,.95)}.dark\:border-l-amber-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(251 191 36/var(--tw-border-opacity,1))}.dark\:border-l-amber-400\/0:is(.dark *){border-left-color:rgba(251,191,36,0)}.dark\:border-l-amber-400\/10:is(.dark *){border-left-color:rgba(251,191,36,.1)}.dark\:border-l-amber-400\/100:is(.dark *){border-left-color:#fbbf24}.dark\:border-l-amber-400\/15:is(.dark *){border-left-color:rgba(251,191,36,.15)}.dark\:border-l-amber-400\/20:is(.dark *){border-left-color:rgba(251,191,36,.2)}.dark\:border-l-amber-400\/25:is(.dark *){border-left-color:rgba(251,191,36,.25)}.dark\:border-l-amber-400\/30:is(.dark *){border-left-color:rgba(251,191,36,.3)}.dark\:border-l-amber-400\/35:is(.dark *){border-left-color:rgba(251,191,36,.35)}.dark\:border-l-amber-400\/40:is(.dark *){border-left-color:rgba(251,191,36,.4)}.dark\:border-l-amber-400\/45:is(.dark *){border-left-color:rgba(251,191,36,.45)}.dark\:border-l-amber-400\/5:is(.dark *){border-left-color:rgba(251,191,36,.05)}.dark\:border-l-amber-400\/50:is(.dark *){border-left-color:rgba(251,191,36,.5)}.dark\:border-l-amber-400\/55:is(.dark *){border-left-color:rgba(251,191,36,.55)}.dark\:border-l-amber-400\/60:is(.dark *){border-left-color:rgba(251,191,36,.6)}.dark\:border-l-amber-400\/65:is(.dark *){border-left-color:rgba(251,191,36,.65)}.dark\:border-l-amber-400\/70:is(.dark *){border-left-color:rgba(251,191,36,.7)}.dark\:border-l-amber-400\/75:is(.dark *){border-left-color:rgba(251,191,36,.75)}.dark\:border-l-amber-400\/80:is(.dark *){border-left-color:rgba(251,191,36,.8)}.dark\:border-l-amber-400\/85:is(.dark *){border-left-color:rgba(251,191,36,.85)}.dark\:border-l-amber-400\/90:is(.dark *){border-left-color:rgba(251,191,36,.9)}.dark\:border-l-amber-400\/95:is(.dark *){border-left-color:rgba(251,191,36,.95)}.dark\:border-l-amber-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(255 251 235/var(--tw-border-opacity,1))}.dark\:border-l-amber-50\/0:is(.dark *){border-left-color:rgba(255,251,235,0)}.dark\:border-l-amber-50\/10:is(.dark *){border-left-color:rgba(255,251,235,.1)}.dark\:border-l-amber-50\/100:is(.dark *){border-left-color:#fffbeb}.dark\:border-l-amber-50\/15:is(.dark *){border-left-color:rgba(255,251,235,.15)}.dark\:border-l-amber-50\/20:is(.dark *){border-left-color:rgba(255,251,235,.2)}.dark\:border-l-amber-50\/25:is(.dark *){border-left-color:rgba(255,251,235,.25)}.dark\:border-l-amber-50\/30:is(.dark *){border-left-color:rgba(255,251,235,.3)}.dark\:border-l-amber-50\/35:is(.dark *){border-left-color:rgba(255,251,235,.35)}.dark\:border-l-amber-50\/40:is(.dark *){border-left-color:rgba(255,251,235,.4)}.dark\:border-l-amber-50\/45:is(.dark *){border-left-color:rgba(255,251,235,.45)}.dark\:border-l-amber-50\/5:is(.dark *){border-left-color:rgba(255,251,235,.05)}.dark\:border-l-amber-50\/50:is(.dark *){border-left-color:rgba(255,251,235,.5)}.dark\:border-l-amber-50\/55:is(.dark *){border-left-color:rgba(255,251,235,.55)}.dark\:border-l-amber-50\/60:is(.dark *){border-left-color:rgba(255,251,235,.6)}.dark\:border-l-amber-50\/65:is(.dark *){border-left-color:rgba(255,251,235,.65)}.dark\:border-l-amber-50\/70:is(.dark *){border-left-color:rgba(255,251,235,.7)}.dark\:border-l-amber-50\/75:is(.dark *){border-left-color:rgba(255,251,235,.75)}.dark\:border-l-amber-50\/80:is(.dark *){border-left-color:rgba(255,251,235,.8)}.dark\:border-l-amber-50\/85:is(.dark *){border-left-color:rgba(255,251,235,.85)}.dark\:border-l-amber-50\/90:is(.dark *){border-left-color:rgba(255,251,235,.9)}.dark\:border-l-amber-50\/95:is(.dark *){border-left-color:rgba(255,251,235,.95)}.dark\:border-l-amber-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(245 158 11/var(--tw-border-opacity,1))}.dark\:border-l-amber-500\/0:is(.dark *){border-left-color:rgba(245,158,11,0)}.dark\:border-l-amber-500\/10:is(.dark *){border-left-color:rgba(245,158,11,.1)}.dark\:border-l-amber-500\/100:is(.dark *){border-left-color:#f59e0b}.dark\:border-l-amber-500\/15:is(.dark *){border-left-color:rgba(245,158,11,.15)}.dark\:border-l-amber-500\/20:is(.dark *){border-left-color:rgba(245,158,11,.2)}.dark\:border-l-amber-500\/25:is(.dark *){border-left-color:rgba(245,158,11,.25)}.dark\:border-l-amber-500\/30:is(.dark *){border-left-color:rgba(245,158,11,.3)}.dark\:border-l-amber-500\/35:is(.dark *){border-left-color:rgba(245,158,11,.35)}.dark\:border-l-amber-500\/40:is(.dark *){border-left-color:rgba(245,158,11,.4)}.dark\:border-l-amber-500\/45:is(.dark *){border-left-color:rgba(245,158,11,.45)}.dark\:border-l-amber-500\/5:is(.dark *){border-left-color:rgba(245,158,11,.05)}.dark\:border-l-amber-500\/50:is(.dark *){border-left-color:rgba(245,158,11,.5)}.dark\:border-l-amber-500\/55:is(.dark *){border-left-color:rgba(245,158,11,.55)}.dark\:border-l-amber-500\/60:is(.dark *){border-left-color:rgba(245,158,11,.6)}.dark\:border-l-amber-500\/65:is(.dark *){border-left-color:rgba(245,158,11,.65)}.dark\:border-l-amber-500\/70:is(.dark *){border-left-color:rgba(245,158,11,.7)}.dark\:border-l-amber-500\/75:is(.dark *){border-left-color:rgba(245,158,11,.75)}.dark\:border-l-amber-500\/80:is(.dark *){border-left-color:rgba(245,158,11,.8)}.dark\:border-l-amber-500\/85:is(.dark *){border-left-color:rgba(245,158,11,.85)}.dark\:border-l-amber-500\/90:is(.dark *){border-left-color:rgba(245,158,11,.9)}.dark\:border-l-amber-500\/95:is(.dark *){border-left-color:rgba(245,158,11,.95)}.dark\:border-l-amber-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(69 26 3/var(--tw-border-opacity,1))}.dark\:border-l-amber-950\/0:is(.dark *){border-left-color:rgba(69,26,3,0)}.dark\:border-l-amber-950\/10:is(.dark *){border-left-color:rgba(69,26,3,.1)}.dark\:border-l-amber-950\/100:is(.dark *){border-left-color:#451a03}.dark\:border-l-amber-950\/15:is(.dark *){border-left-color:rgba(69,26,3,.15)}.dark\:border-l-amber-950\/20:is(.dark *){border-left-color:rgba(69,26,3,.2)}.dark\:border-l-amber-950\/25:is(.dark *){border-left-color:rgba(69,26,3,.25)}.dark\:border-l-amber-950\/30:is(.dark *){border-left-color:rgba(69,26,3,.3)}.dark\:border-l-amber-950\/35:is(.dark *){border-left-color:rgba(69,26,3,.35)}.dark\:border-l-amber-950\/40:is(.dark *){border-left-color:rgba(69,26,3,.4)}.dark\:border-l-amber-950\/45:is(.dark *){border-left-color:rgba(69,26,3,.45)}.dark\:border-l-amber-950\/5:is(.dark *){border-left-color:rgba(69,26,3,.05)}.dark\:border-l-amber-950\/50:is(.dark *){border-left-color:rgba(69,26,3,.5)}.dark\:border-l-amber-950\/55:is(.dark *){border-left-color:rgba(69,26,3,.55)}.dark\:border-l-amber-950\/60:is(.dark *){border-left-color:rgba(69,26,3,.6)}.dark\:border-l-amber-950\/65:is(.dark *){border-left-color:rgba(69,26,3,.65)}.dark\:border-l-amber-950\/70:is(.dark *){border-left-color:rgba(69,26,3,.7)}.dark\:border-l-amber-950\/75:is(.dark *){border-left-color:rgba(69,26,3,.75)}.dark\:border-l-amber-950\/80:is(.dark *){border-left-color:rgba(69,26,3,.8)}.dark\:border-l-amber-950\/85:is(.dark *){border-left-color:rgba(69,26,3,.85)}.dark\:border-l-amber-950\/90:is(.dark *){border-left-color:rgba(69,26,3,.9)}.dark\:border-l-amber-950\/95:is(.dark *){border-left-color:rgba(69,26,3,.95)}.dark\:border-l-blue-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(219 234 254/var(--tw-border-opacity,1))}.dark\:border-l-blue-100\/0:is(.dark *){border-left-color:rgba(219,234,254,0)}.dark\:border-l-blue-100\/10:is(.dark *){border-left-color:rgba(219,234,254,.1)}.dark\:border-l-blue-100\/100:is(.dark *){border-left-color:#dbeafe}.dark\:border-l-blue-100\/15:is(.dark *){border-left-color:rgba(219,234,254,.15)}.dark\:border-l-blue-100\/20:is(.dark *){border-left-color:rgba(219,234,254,.2)}.dark\:border-l-blue-100\/25:is(.dark *){border-left-color:rgba(219,234,254,.25)}.dark\:border-l-blue-100\/30:is(.dark *){border-left-color:rgba(219,234,254,.3)}.dark\:border-l-blue-100\/35:is(.dark *){border-left-color:rgba(219,234,254,.35)}.dark\:border-l-blue-100\/40:is(.dark *){border-left-color:rgba(219,234,254,.4)}.dark\:border-l-blue-100\/45:is(.dark *){border-left-color:rgba(219,234,254,.45)}.dark\:border-l-blue-100\/5:is(.dark *){border-left-color:rgba(219,234,254,.05)}.dark\:border-l-blue-100\/50:is(.dark *){border-left-color:rgba(219,234,254,.5)}.dark\:border-l-blue-100\/55:is(.dark *){border-left-color:rgba(219,234,254,.55)}.dark\:border-l-blue-100\/60:is(.dark *){border-left-color:rgba(219,234,254,.6)}.dark\:border-l-blue-100\/65:is(.dark *){border-left-color:rgba(219,234,254,.65)}.dark\:border-l-blue-100\/70:is(.dark *){border-left-color:rgba(219,234,254,.7)}.dark\:border-l-blue-100\/75:is(.dark *){border-left-color:rgba(219,234,254,.75)}.dark\:border-l-blue-100\/80:is(.dark *){border-left-color:rgba(219,234,254,.8)}.dark\:border-l-blue-100\/85:is(.dark *){border-left-color:rgba(219,234,254,.85)}.dark\:border-l-blue-100\/90:is(.dark *){border-left-color:rgba(219,234,254,.9)}.dark\:border-l-blue-100\/95:is(.dark *){border-left-color:rgba(219,234,254,.95)}.dark\:border-l-blue-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(191 219 254/var(--tw-border-opacity,1))}.dark\:border-l-blue-200\/0:is(.dark *){border-left-color:rgba(191,219,254,0)}.dark\:border-l-blue-200\/10:is(.dark *){border-left-color:rgba(191,219,254,.1)}.dark\:border-l-blue-200\/100:is(.dark *){border-left-color:#bfdbfe}.dark\:border-l-blue-200\/15:is(.dark *){border-left-color:rgba(191,219,254,.15)}.dark\:border-l-blue-200\/20:is(.dark *){border-left-color:rgba(191,219,254,.2)}.dark\:border-l-blue-200\/25:is(.dark *){border-left-color:rgba(191,219,254,.25)}.dark\:border-l-blue-200\/30:is(.dark *){border-left-color:rgba(191,219,254,.3)}.dark\:border-l-blue-200\/35:is(.dark *){border-left-color:rgba(191,219,254,.35)}.dark\:border-l-blue-200\/40:is(.dark *){border-left-color:rgba(191,219,254,.4)}.dark\:border-l-blue-200\/45:is(.dark *){border-left-color:rgba(191,219,254,.45)}.dark\:border-l-blue-200\/5:is(.dark *){border-left-color:rgba(191,219,254,.05)}.dark\:border-l-blue-200\/50:is(.dark *){border-left-color:rgba(191,219,254,.5)}.dark\:border-l-blue-200\/55:is(.dark *){border-left-color:rgba(191,219,254,.55)}.dark\:border-l-blue-200\/60:is(.dark *){border-left-color:rgba(191,219,254,.6)}.dark\:border-l-blue-200\/65:is(.dark *){border-left-color:rgba(191,219,254,.65)}.dark\:border-l-blue-200\/70:is(.dark *){border-left-color:rgba(191,219,254,.7)}.dark\:border-l-blue-200\/75:is(.dark *){border-left-color:rgba(191,219,254,.75)}.dark\:border-l-blue-200\/80:is(.dark *){border-left-color:rgba(191,219,254,.8)}.dark\:border-l-blue-200\/85:is(.dark *){border-left-color:rgba(191,219,254,.85)}.dark\:border-l-blue-200\/90:is(.dark *){border-left-color:rgba(191,219,254,.9)}.dark\:border-l-blue-200\/95:is(.dark *){border-left-color:rgba(191,219,254,.95)}.dark\:border-l-blue-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(96 165 250/var(--tw-border-opacity,1))}.dark\:border-l-blue-400\/0:is(.dark *){border-left-color:rgba(96,165,250,0)}.dark\:border-l-blue-400\/10:is(.dark *){border-left-color:rgba(96,165,250,.1)}.dark\:border-l-blue-400\/100:is(.dark *){border-left-color:#60a5fa}.dark\:border-l-blue-400\/15:is(.dark *){border-left-color:rgba(96,165,250,.15)}.dark\:border-l-blue-400\/20:is(.dark *){border-left-color:rgba(96,165,250,.2)}.dark\:border-l-blue-400\/25:is(.dark *){border-left-color:rgba(96,165,250,.25)}.dark\:border-l-blue-400\/30:is(.dark *){border-left-color:rgba(96,165,250,.3)}.dark\:border-l-blue-400\/35:is(.dark *){border-left-color:rgba(96,165,250,.35)}.dark\:border-l-blue-400\/40:is(.dark *){border-left-color:rgba(96,165,250,.4)}.dark\:border-l-blue-400\/45:is(.dark *){border-left-color:rgba(96,165,250,.45)}.dark\:border-l-blue-400\/5:is(.dark *){border-left-color:rgba(96,165,250,.05)}.dark\:border-l-blue-400\/50:is(.dark *){border-left-color:rgba(96,165,250,.5)}.dark\:border-l-blue-400\/55:is(.dark *){border-left-color:rgba(96,165,250,.55)}.dark\:border-l-blue-400\/60:is(.dark *){border-left-color:rgba(96,165,250,.6)}.dark\:border-l-blue-400\/65:is(.dark *){border-left-color:rgba(96,165,250,.65)}.dark\:border-l-blue-400\/70:is(.dark *){border-left-color:rgba(96,165,250,.7)}.dark\:border-l-blue-400\/75:is(.dark *){border-left-color:rgba(96,165,250,.75)}.dark\:border-l-blue-400\/80:is(.dark *){border-left-color:rgba(96,165,250,.8)}.dark\:border-l-blue-400\/85:is(.dark *){border-left-color:rgba(96,165,250,.85)}.dark\:border-l-blue-400\/90:is(.dark *){border-left-color:rgba(96,165,250,.9)}.dark\:border-l-blue-400\/95:is(.dark *){border-left-color:rgba(96,165,250,.95)}.dark\:border-l-blue-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(239 246 255/var(--tw-border-opacity,1))}.dark\:border-l-blue-50\/0:is(.dark *){border-left-color:rgba(239,246,255,0)}.dark\:border-l-blue-50\/10:is(.dark *){border-left-color:rgba(239,246,255,.1)}.dark\:border-l-blue-50\/100:is(.dark *){border-left-color:#eff6ff}.dark\:border-l-blue-50\/15:is(.dark *){border-left-color:rgba(239,246,255,.15)}.dark\:border-l-blue-50\/20:is(.dark *){border-left-color:rgba(239,246,255,.2)}.dark\:border-l-blue-50\/25:is(.dark *){border-left-color:rgba(239,246,255,.25)}.dark\:border-l-blue-50\/30:is(.dark *){border-left-color:rgba(239,246,255,.3)}.dark\:border-l-blue-50\/35:is(.dark *){border-left-color:rgba(239,246,255,.35)}.dark\:border-l-blue-50\/40:is(.dark *){border-left-color:rgba(239,246,255,.4)}.dark\:border-l-blue-50\/45:is(.dark *){border-left-color:rgba(239,246,255,.45)}.dark\:border-l-blue-50\/5:is(.dark *){border-left-color:rgba(239,246,255,.05)}.dark\:border-l-blue-50\/50:is(.dark *){border-left-color:rgba(239,246,255,.5)}.dark\:border-l-blue-50\/55:is(.dark *){border-left-color:rgba(239,246,255,.55)}.dark\:border-l-blue-50\/60:is(.dark *){border-left-color:rgba(239,246,255,.6)}.dark\:border-l-blue-50\/65:is(.dark *){border-left-color:rgba(239,246,255,.65)}.dark\:border-l-blue-50\/70:is(.dark *){border-left-color:rgba(239,246,255,.7)}.dark\:border-l-blue-50\/75:is(.dark *){border-left-color:rgba(239,246,255,.75)}.dark\:border-l-blue-50\/80:is(.dark *){border-left-color:rgba(239,246,255,.8)}.dark\:border-l-blue-50\/85:is(.dark *){border-left-color:rgba(239,246,255,.85)}.dark\:border-l-blue-50\/90:is(.dark *){border-left-color:rgba(239,246,255,.9)}.dark\:border-l-blue-50\/95:is(.dark *){border-left-color:rgba(239,246,255,.95)}.dark\:border-l-blue-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(59 130 246/var(--tw-border-opacity,1))}.dark\:border-l-blue-500\/0:is(.dark *){border-left-color:rgba(59,130,246,0)}.dark\:border-l-blue-500\/10:is(.dark *){border-left-color:rgba(59,130,246,.1)}.dark\:border-l-blue-500\/100:is(.dark *){border-left-color:#3b82f6}.dark\:border-l-blue-500\/15:is(.dark *){border-left-color:rgba(59,130,246,.15)}.dark\:border-l-blue-500\/20:is(.dark *){border-left-color:rgba(59,130,246,.2)}.dark\:border-l-blue-500\/25:is(.dark *){border-left-color:rgba(59,130,246,.25)}.dark\:border-l-blue-500\/30:is(.dark *){border-left-color:rgba(59,130,246,.3)}.dark\:border-l-blue-500\/35:is(.dark *){border-left-color:rgba(59,130,246,.35)}.dark\:border-l-blue-500\/40:is(.dark *){border-left-color:rgba(59,130,246,.4)}.dark\:border-l-blue-500\/45:is(.dark *){border-left-color:rgba(59,130,246,.45)}.dark\:border-l-blue-500\/5:is(.dark *){border-left-color:rgba(59,130,246,.05)}.dark\:border-l-blue-500\/50:is(.dark *){border-left-color:rgba(59,130,246,.5)}.dark\:border-l-blue-500\/55:is(.dark *){border-left-color:rgba(59,130,246,.55)}.dark\:border-l-blue-500\/60:is(.dark *){border-left-color:rgba(59,130,246,.6)}.dark\:border-l-blue-500\/65:is(.dark *){border-left-color:rgba(59,130,246,.65)}.dark\:border-l-blue-500\/70:is(.dark *){border-left-color:rgba(59,130,246,.7)}.dark\:border-l-blue-500\/75:is(.dark *){border-left-color:rgba(59,130,246,.75)}.dark\:border-l-blue-500\/80:is(.dark *){border-left-color:rgba(59,130,246,.8)}.dark\:border-l-blue-500\/85:is(.dark *){border-left-color:rgba(59,130,246,.85)}.dark\:border-l-blue-500\/90:is(.dark *){border-left-color:rgba(59,130,246,.9)}.dark\:border-l-blue-500\/95:is(.dark *){border-left-color:rgba(59,130,246,.95)}.dark\:border-l-blue-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(23 37 84/var(--tw-border-opacity,1))}.dark\:border-l-blue-950\/0:is(.dark *){border-left-color:rgba(23,37,84,0)}.dark\:border-l-blue-950\/10:is(.dark *){border-left-color:rgba(23,37,84,.1)}.dark\:border-l-blue-950\/100:is(.dark *){border-left-color:#172554}.dark\:border-l-blue-950\/15:is(.dark *){border-left-color:rgba(23,37,84,.15)}.dark\:border-l-blue-950\/20:is(.dark *){border-left-color:rgba(23,37,84,.2)}.dark\:border-l-blue-950\/25:is(.dark *){border-left-color:rgba(23,37,84,.25)}.dark\:border-l-blue-950\/30:is(.dark *){border-left-color:rgba(23,37,84,.3)}.dark\:border-l-blue-950\/35:is(.dark *){border-left-color:rgba(23,37,84,.35)}.dark\:border-l-blue-950\/40:is(.dark *){border-left-color:rgba(23,37,84,.4)}.dark\:border-l-blue-950\/45:is(.dark *){border-left-color:rgba(23,37,84,.45)}.dark\:border-l-blue-950\/5:is(.dark *){border-left-color:rgba(23,37,84,.05)}.dark\:border-l-blue-950\/50:is(.dark *){border-left-color:rgba(23,37,84,.5)}.dark\:border-l-blue-950\/55:is(.dark *){border-left-color:rgba(23,37,84,.55)}.dark\:border-l-blue-950\/60:is(.dark *){border-left-color:rgba(23,37,84,.6)}.dark\:border-l-blue-950\/65:is(.dark *){border-left-color:rgba(23,37,84,.65)}.dark\:border-l-blue-950\/70:is(.dark *){border-left-color:rgba(23,37,84,.7)}.dark\:border-l-blue-950\/75:is(.dark *){border-left-color:rgba(23,37,84,.75)}.dark\:border-l-blue-950\/80:is(.dark *){border-left-color:rgba(23,37,84,.8)}.dark\:border-l-blue-950\/85:is(.dark *){border-left-color:rgba(23,37,84,.85)}.dark\:border-l-blue-950\/90:is(.dark *){border-left-color:rgba(23,37,84,.9)}.dark\:border-l-blue-950\/95:is(.dark *){border-left-color:rgba(23,37,84,.95)}.dark\:border-l-cyan-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(207 250 254/var(--tw-border-opacity,1))}.dark\:border-l-cyan-100\/0:is(.dark *){border-left-color:rgba(207,250,254,0)}.dark\:border-l-cyan-100\/10:is(.dark *){border-left-color:rgba(207,250,254,.1)}.dark\:border-l-cyan-100\/100:is(.dark *){border-left-color:#cffafe}.dark\:border-l-cyan-100\/15:is(.dark *){border-left-color:rgba(207,250,254,.15)}.dark\:border-l-cyan-100\/20:is(.dark *){border-left-color:rgba(207,250,254,.2)}.dark\:border-l-cyan-100\/25:is(.dark *){border-left-color:rgba(207,250,254,.25)}.dark\:border-l-cyan-100\/30:is(.dark *){border-left-color:rgba(207,250,254,.3)}.dark\:border-l-cyan-100\/35:is(.dark *){border-left-color:rgba(207,250,254,.35)}.dark\:border-l-cyan-100\/40:is(.dark *){border-left-color:rgba(207,250,254,.4)}.dark\:border-l-cyan-100\/45:is(.dark *){border-left-color:rgba(207,250,254,.45)}.dark\:border-l-cyan-100\/5:is(.dark *){border-left-color:rgba(207,250,254,.05)}.dark\:border-l-cyan-100\/50:is(.dark *){border-left-color:rgba(207,250,254,.5)}.dark\:border-l-cyan-100\/55:is(.dark *){border-left-color:rgba(207,250,254,.55)}.dark\:border-l-cyan-100\/60:is(.dark *){border-left-color:rgba(207,250,254,.6)}.dark\:border-l-cyan-100\/65:is(.dark *){border-left-color:rgba(207,250,254,.65)}.dark\:border-l-cyan-100\/70:is(.dark *){border-left-color:rgba(207,250,254,.7)}.dark\:border-l-cyan-100\/75:is(.dark *){border-left-color:rgba(207,250,254,.75)}.dark\:border-l-cyan-100\/80:is(.dark *){border-left-color:rgba(207,250,254,.8)}.dark\:border-l-cyan-100\/85:is(.dark *){border-left-color:rgba(207,250,254,.85)}.dark\:border-l-cyan-100\/90:is(.dark *){border-left-color:rgba(207,250,254,.9)}.dark\:border-l-cyan-100\/95:is(.dark *){border-left-color:rgba(207,250,254,.95)}.dark\:border-l-cyan-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(165 243 252/var(--tw-border-opacity,1))}.dark\:border-l-cyan-200\/0:is(.dark *){border-left-color:rgba(165,243,252,0)}.dark\:border-l-cyan-200\/10:is(.dark *){border-left-color:rgba(165,243,252,.1)}.dark\:border-l-cyan-200\/100:is(.dark *){border-left-color:#a5f3fc}.dark\:border-l-cyan-200\/15:is(.dark *){border-left-color:rgba(165,243,252,.15)}.dark\:border-l-cyan-200\/20:is(.dark *){border-left-color:rgba(165,243,252,.2)}.dark\:border-l-cyan-200\/25:is(.dark *){border-left-color:rgba(165,243,252,.25)}.dark\:border-l-cyan-200\/30:is(.dark *){border-left-color:rgba(165,243,252,.3)}.dark\:border-l-cyan-200\/35:is(.dark *){border-left-color:rgba(165,243,252,.35)}.dark\:border-l-cyan-200\/40:is(.dark *){border-left-color:rgba(165,243,252,.4)}.dark\:border-l-cyan-200\/45:is(.dark *){border-left-color:rgba(165,243,252,.45)}.dark\:border-l-cyan-200\/5:is(.dark *){border-left-color:rgba(165,243,252,.05)}.dark\:border-l-cyan-200\/50:is(.dark *){border-left-color:rgba(165,243,252,.5)}.dark\:border-l-cyan-200\/55:is(.dark *){border-left-color:rgba(165,243,252,.55)}.dark\:border-l-cyan-200\/60:is(.dark *){border-left-color:rgba(165,243,252,.6)}.dark\:border-l-cyan-200\/65:is(.dark *){border-left-color:rgba(165,243,252,.65)}.dark\:border-l-cyan-200\/70:is(.dark *){border-left-color:rgba(165,243,252,.7)}.dark\:border-l-cyan-200\/75:is(.dark *){border-left-color:rgba(165,243,252,.75)}.dark\:border-l-cyan-200\/80:is(.dark *){border-left-color:rgba(165,243,252,.8)}.dark\:border-l-cyan-200\/85:is(.dark *){border-left-color:rgba(165,243,252,.85)}.dark\:border-l-cyan-200\/90:is(.dark *){border-left-color:rgba(165,243,252,.9)}.dark\:border-l-cyan-200\/95:is(.dark *){border-left-color:rgba(165,243,252,.95)}.dark\:border-l-cyan-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(34 211 238/var(--tw-border-opacity,1))}.dark\:border-l-cyan-400\/0:is(.dark *){border-left-color:rgba(34,211,238,0)}.dark\:border-l-cyan-400\/10:is(.dark *){border-left-color:rgba(34,211,238,.1)}.dark\:border-l-cyan-400\/100:is(.dark *){border-left-color:#22d3ee}.dark\:border-l-cyan-400\/15:is(.dark *){border-left-color:rgba(34,211,238,.15)}.dark\:border-l-cyan-400\/20:is(.dark *){border-left-color:rgba(34,211,238,.2)}.dark\:border-l-cyan-400\/25:is(.dark *){border-left-color:rgba(34,211,238,.25)}.dark\:border-l-cyan-400\/30:is(.dark *){border-left-color:rgba(34,211,238,.3)}.dark\:border-l-cyan-400\/35:is(.dark *){border-left-color:rgba(34,211,238,.35)}.dark\:border-l-cyan-400\/40:is(.dark *){border-left-color:rgba(34,211,238,.4)}.dark\:border-l-cyan-400\/45:is(.dark *){border-left-color:rgba(34,211,238,.45)}.dark\:border-l-cyan-400\/5:is(.dark *){border-left-color:rgba(34,211,238,.05)}.dark\:border-l-cyan-400\/50:is(.dark *){border-left-color:rgba(34,211,238,.5)}.dark\:border-l-cyan-400\/55:is(.dark *){border-left-color:rgba(34,211,238,.55)}.dark\:border-l-cyan-400\/60:is(.dark *){border-left-color:rgba(34,211,238,.6)}.dark\:border-l-cyan-400\/65:is(.dark *){border-left-color:rgba(34,211,238,.65)}.dark\:border-l-cyan-400\/70:is(.dark *){border-left-color:rgba(34,211,238,.7)}.dark\:border-l-cyan-400\/75:is(.dark *){border-left-color:rgba(34,211,238,.75)}.dark\:border-l-cyan-400\/80:is(.dark *){border-left-color:rgba(34,211,238,.8)}.dark\:border-l-cyan-400\/85:is(.dark *){border-left-color:rgba(34,211,238,.85)}.dark\:border-l-cyan-400\/90:is(.dark *){border-left-color:rgba(34,211,238,.9)}.dark\:border-l-cyan-400\/95:is(.dark *){border-left-color:rgba(34,211,238,.95)}.dark\:border-l-cyan-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(236 254 255/var(--tw-border-opacity,1))}.dark\:border-l-cyan-50\/0:is(.dark *){border-left-color:rgba(236,254,255,0)}.dark\:border-l-cyan-50\/10:is(.dark *){border-left-color:rgba(236,254,255,.1)}.dark\:border-l-cyan-50\/100:is(.dark *){border-left-color:#ecfeff}.dark\:border-l-cyan-50\/15:is(.dark *){border-left-color:rgba(236,254,255,.15)}.dark\:border-l-cyan-50\/20:is(.dark *){border-left-color:rgba(236,254,255,.2)}.dark\:border-l-cyan-50\/25:is(.dark *){border-left-color:rgba(236,254,255,.25)}.dark\:border-l-cyan-50\/30:is(.dark *){border-left-color:rgba(236,254,255,.3)}.dark\:border-l-cyan-50\/35:is(.dark *){border-left-color:rgba(236,254,255,.35)}.dark\:border-l-cyan-50\/40:is(.dark *){border-left-color:rgba(236,254,255,.4)}.dark\:border-l-cyan-50\/45:is(.dark *){border-left-color:rgba(236,254,255,.45)}.dark\:border-l-cyan-50\/5:is(.dark *){border-left-color:rgba(236,254,255,.05)}.dark\:border-l-cyan-50\/50:is(.dark *){border-left-color:rgba(236,254,255,.5)}.dark\:border-l-cyan-50\/55:is(.dark *){border-left-color:rgba(236,254,255,.55)}.dark\:border-l-cyan-50\/60:is(.dark *){border-left-color:rgba(236,254,255,.6)}.dark\:border-l-cyan-50\/65:is(.dark *){border-left-color:rgba(236,254,255,.65)}.dark\:border-l-cyan-50\/70:is(.dark *){border-left-color:rgba(236,254,255,.7)}.dark\:border-l-cyan-50\/75:is(.dark *){border-left-color:rgba(236,254,255,.75)}.dark\:border-l-cyan-50\/80:is(.dark *){border-left-color:rgba(236,254,255,.8)}.dark\:border-l-cyan-50\/85:is(.dark *){border-left-color:rgba(236,254,255,.85)}.dark\:border-l-cyan-50\/90:is(.dark *){border-left-color:rgba(236,254,255,.9)}.dark\:border-l-cyan-50\/95:is(.dark *){border-left-color:rgba(236,254,255,.95)}.dark\:border-l-cyan-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(6 182 212/var(--tw-border-opacity,1))}.dark\:border-l-cyan-500\/0:is(.dark *){border-left-color:rgba(6,182,212,0)}.dark\:border-l-cyan-500\/10:is(.dark *){border-left-color:rgba(6,182,212,.1)}.dark\:border-l-cyan-500\/100:is(.dark *){border-left-color:#06b6d4}.dark\:border-l-cyan-500\/15:is(.dark *){border-left-color:rgba(6,182,212,.15)}.dark\:border-l-cyan-500\/20:is(.dark *){border-left-color:rgba(6,182,212,.2)}.dark\:border-l-cyan-500\/25:is(.dark *){border-left-color:rgba(6,182,212,.25)}.dark\:border-l-cyan-500\/30:is(.dark *){border-left-color:rgba(6,182,212,.3)}.dark\:border-l-cyan-500\/35:is(.dark *){border-left-color:rgba(6,182,212,.35)}.dark\:border-l-cyan-500\/40:is(.dark *){border-left-color:rgba(6,182,212,.4)}.dark\:border-l-cyan-500\/45:is(.dark *){border-left-color:rgba(6,182,212,.45)}.dark\:border-l-cyan-500\/5:is(.dark *){border-left-color:rgba(6,182,212,.05)}.dark\:border-l-cyan-500\/50:is(.dark *){border-left-color:rgba(6,182,212,.5)}.dark\:border-l-cyan-500\/55:is(.dark *){border-left-color:rgba(6,182,212,.55)}.dark\:border-l-cyan-500\/60:is(.dark *){border-left-color:rgba(6,182,212,.6)}.dark\:border-l-cyan-500\/65:is(.dark *){border-left-color:rgba(6,182,212,.65)}.dark\:border-l-cyan-500\/70:is(.dark *){border-left-color:rgba(6,182,212,.7)}.dark\:border-l-cyan-500\/75:is(.dark *){border-left-color:rgba(6,182,212,.75)}.dark\:border-l-cyan-500\/80:is(.dark *){border-left-color:rgba(6,182,212,.8)}.dark\:border-l-cyan-500\/85:is(.dark *){border-left-color:rgba(6,182,212,.85)}.dark\:border-l-cyan-500\/90:is(.dark *){border-left-color:rgba(6,182,212,.9)}.dark\:border-l-cyan-500\/95:is(.dark *){border-left-color:rgba(6,182,212,.95)}.dark\:border-l-cyan-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(8 51 68/var(--tw-border-opacity,1))}.dark\:border-l-cyan-950\/0:is(.dark *){border-left-color:rgba(8,51,68,0)}.dark\:border-l-cyan-950\/10:is(.dark *){border-left-color:rgba(8,51,68,.1)}.dark\:border-l-cyan-950\/100:is(.dark *){border-left-color:#083344}.dark\:border-l-cyan-950\/15:is(.dark *){border-left-color:rgba(8,51,68,.15)}.dark\:border-l-cyan-950\/20:is(.dark *){border-left-color:rgba(8,51,68,.2)}.dark\:border-l-cyan-950\/25:is(.dark *){border-left-color:rgba(8,51,68,.25)}.dark\:border-l-cyan-950\/30:is(.dark *){border-left-color:rgba(8,51,68,.3)}.dark\:border-l-cyan-950\/35:is(.dark *){border-left-color:rgba(8,51,68,.35)}.dark\:border-l-cyan-950\/40:is(.dark *){border-left-color:rgba(8,51,68,.4)}.dark\:border-l-cyan-950\/45:is(.dark *){border-left-color:rgba(8,51,68,.45)}.dark\:border-l-cyan-950\/5:is(.dark *){border-left-color:rgba(8,51,68,.05)}.dark\:border-l-cyan-950\/50:is(.dark *){border-left-color:rgba(8,51,68,.5)}.dark\:border-l-cyan-950\/55:is(.dark *){border-left-color:rgba(8,51,68,.55)}.dark\:border-l-cyan-950\/60:is(.dark *){border-left-color:rgba(8,51,68,.6)}.dark\:border-l-cyan-950\/65:is(.dark *){border-left-color:rgba(8,51,68,.65)}.dark\:border-l-cyan-950\/70:is(.dark *){border-left-color:rgba(8,51,68,.7)}.dark\:border-l-cyan-950\/75:is(.dark *){border-left-color:rgba(8,51,68,.75)}.dark\:border-l-cyan-950\/80:is(.dark *){border-left-color:rgba(8,51,68,.8)}.dark\:border-l-cyan-950\/85:is(.dark *){border-left-color:rgba(8,51,68,.85)}.dark\:border-l-cyan-950\/90:is(.dark *){border-left-color:rgba(8,51,68,.9)}.dark\:border-l-cyan-950\/95:is(.dark *){border-left-color:rgba(8,51,68,.95)}.dark\:border-l-emerald-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(209 250 229/var(--tw-border-opacity,1))}.dark\:border-l-emerald-100\/0:is(.dark *){border-left-color:rgba(209,250,229,0)}.dark\:border-l-emerald-100\/10:is(.dark *){border-left-color:rgba(209,250,229,.1)}.dark\:border-l-emerald-100\/100:is(.dark *){border-left-color:#d1fae5}.dark\:border-l-emerald-100\/15:is(.dark *){border-left-color:rgba(209,250,229,.15)}.dark\:border-l-emerald-100\/20:is(.dark *){border-left-color:rgba(209,250,229,.2)}.dark\:border-l-emerald-100\/25:is(.dark *){border-left-color:rgba(209,250,229,.25)}.dark\:border-l-emerald-100\/30:is(.dark *){border-left-color:rgba(209,250,229,.3)}.dark\:border-l-emerald-100\/35:is(.dark *){border-left-color:rgba(209,250,229,.35)}.dark\:border-l-emerald-100\/40:is(.dark *){border-left-color:rgba(209,250,229,.4)}.dark\:border-l-emerald-100\/45:is(.dark *){border-left-color:rgba(209,250,229,.45)}.dark\:border-l-emerald-100\/5:is(.dark *){border-left-color:rgba(209,250,229,.05)}.dark\:border-l-emerald-100\/50:is(.dark *){border-left-color:rgba(209,250,229,.5)}.dark\:border-l-emerald-100\/55:is(.dark *){border-left-color:rgba(209,250,229,.55)}.dark\:border-l-emerald-100\/60:is(.dark *){border-left-color:rgba(209,250,229,.6)}.dark\:border-l-emerald-100\/65:is(.dark *){border-left-color:rgba(209,250,229,.65)}.dark\:border-l-emerald-100\/70:is(.dark *){border-left-color:rgba(209,250,229,.7)}.dark\:border-l-emerald-100\/75:is(.dark *){border-left-color:rgba(209,250,229,.75)}.dark\:border-l-emerald-100\/80:is(.dark *){border-left-color:rgba(209,250,229,.8)}.dark\:border-l-emerald-100\/85:is(.dark *){border-left-color:rgba(209,250,229,.85)}.dark\:border-l-emerald-100\/90:is(.dark *){border-left-color:rgba(209,250,229,.9)}.dark\:border-l-emerald-100\/95:is(.dark *){border-left-color:rgba(209,250,229,.95)}.dark\:border-l-emerald-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(167 243 208/var(--tw-border-opacity,1))}.dark\:border-l-emerald-200\/0:is(.dark *){border-left-color:rgba(167,243,208,0)}.dark\:border-l-emerald-200\/10:is(.dark *){border-left-color:rgba(167,243,208,.1)}.dark\:border-l-emerald-200\/100:is(.dark *){border-left-color:#a7f3d0}.dark\:border-l-emerald-200\/15:is(.dark *){border-left-color:rgba(167,243,208,.15)}.dark\:border-l-emerald-200\/20:is(.dark *){border-left-color:rgba(167,243,208,.2)}.dark\:border-l-emerald-200\/25:is(.dark *){border-left-color:rgba(167,243,208,.25)}.dark\:border-l-emerald-200\/30:is(.dark *){border-left-color:rgba(167,243,208,.3)}.dark\:border-l-emerald-200\/35:is(.dark *){border-left-color:rgba(167,243,208,.35)}.dark\:border-l-emerald-200\/40:is(.dark *){border-left-color:rgba(167,243,208,.4)}.dark\:border-l-emerald-200\/45:is(.dark *){border-left-color:rgba(167,243,208,.45)}.dark\:border-l-emerald-200\/5:is(.dark *){border-left-color:rgba(167,243,208,.05)}.dark\:border-l-emerald-200\/50:is(.dark *){border-left-color:rgba(167,243,208,.5)}.dark\:border-l-emerald-200\/55:is(.dark *){border-left-color:rgba(167,243,208,.55)}.dark\:border-l-emerald-200\/60:is(.dark *){border-left-color:rgba(167,243,208,.6)}.dark\:border-l-emerald-200\/65:is(.dark *){border-left-color:rgba(167,243,208,.65)}.dark\:border-l-emerald-200\/70:is(.dark *){border-left-color:rgba(167,243,208,.7)}.dark\:border-l-emerald-200\/75:is(.dark *){border-left-color:rgba(167,243,208,.75)}.dark\:border-l-emerald-200\/80:is(.dark *){border-left-color:rgba(167,243,208,.8)}.dark\:border-l-emerald-200\/85:is(.dark *){border-left-color:rgba(167,243,208,.85)}.dark\:border-l-emerald-200\/90:is(.dark *){border-left-color:rgba(167,243,208,.9)}.dark\:border-l-emerald-200\/95:is(.dark *){border-left-color:rgba(167,243,208,.95)}.dark\:border-l-emerald-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(52 211 153/var(--tw-border-opacity,1))}.dark\:border-l-emerald-400\/0:is(.dark *){border-left-color:rgba(52,211,153,0)}.dark\:border-l-emerald-400\/10:is(.dark *){border-left-color:rgba(52,211,153,.1)}.dark\:border-l-emerald-400\/100:is(.dark *){border-left-color:#34d399}.dark\:border-l-emerald-400\/15:is(.dark *){border-left-color:rgba(52,211,153,.15)}.dark\:border-l-emerald-400\/20:is(.dark *){border-left-color:rgba(52,211,153,.2)}.dark\:border-l-emerald-400\/25:is(.dark *){border-left-color:rgba(52,211,153,.25)}.dark\:border-l-emerald-400\/30:is(.dark *){border-left-color:rgba(52,211,153,.3)}.dark\:border-l-emerald-400\/35:is(.dark *){border-left-color:rgba(52,211,153,.35)}.dark\:border-l-emerald-400\/40:is(.dark *){border-left-color:rgba(52,211,153,.4)}.dark\:border-l-emerald-400\/45:is(.dark *){border-left-color:rgba(52,211,153,.45)}.dark\:border-l-emerald-400\/5:is(.dark *){border-left-color:rgba(52,211,153,.05)}.dark\:border-l-emerald-400\/50:is(.dark *){border-left-color:rgba(52,211,153,.5)}.dark\:border-l-emerald-400\/55:is(.dark *){border-left-color:rgba(52,211,153,.55)}.dark\:border-l-emerald-400\/60:is(.dark *){border-left-color:rgba(52,211,153,.6)}.dark\:border-l-emerald-400\/65:is(.dark *){border-left-color:rgba(52,211,153,.65)}.dark\:border-l-emerald-400\/70:is(.dark *){border-left-color:rgba(52,211,153,.7)}.dark\:border-l-emerald-400\/75:is(.dark *){border-left-color:rgba(52,211,153,.75)}.dark\:border-l-emerald-400\/80:is(.dark *){border-left-color:rgba(52,211,153,.8)}.dark\:border-l-emerald-400\/85:is(.dark *){border-left-color:rgba(52,211,153,.85)}.dark\:border-l-emerald-400\/90:is(.dark *){border-left-color:rgba(52,211,153,.9)}.dark\:border-l-emerald-400\/95:is(.dark *){border-left-color:rgba(52,211,153,.95)}.dark\:border-l-emerald-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(236 253 245/var(--tw-border-opacity,1))}.dark\:border-l-emerald-50\/0:is(.dark *){border-left-color:rgba(236,253,245,0)}.dark\:border-l-emerald-50\/10:is(.dark *){border-left-color:rgba(236,253,245,.1)}.dark\:border-l-emerald-50\/100:is(.dark *){border-left-color:#ecfdf5}.dark\:border-l-emerald-50\/15:is(.dark *){border-left-color:rgba(236,253,245,.15)}.dark\:border-l-emerald-50\/20:is(.dark *){border-left-color:rgba(236,253,245,.2)}.dark\:border-l-emerald-50\/25:is(.dark *){border-left-color:rgba(236,253,245,.25)}.dark\:border-l-emerald-50\/30:is(.dark *){border-left-color:rgba(236,253,245,.3)}.dark\:border-l-emerald-50\/35:is(.dark *){border-left-color:rgba(236,253,245,.35)}.dark\:border-l-emerald-50\/40:is(.dark *){border-left-color:rgba(236,253,245,.4)}.dark\:border-l-emerald-50\/45:is(.dark *){border-left-color:rgba(236,253,245,.45)}.dark\:border-l-emerald-50\/5:is(.dark *){border-left-color:rgba(236,253,245,.05)}.dark\:border-l-emerald-50\/50:is(.dark *){border-left-color:rgba(236,253,245,.5)}.dark\:border-l-emerald-50\/55:is(.dark *){border-left-color:rgba(236,253,245,.55)}.dark\:border-l-emerald-50\/60:is(.dark *){border-left-color:rgba(236,253,245,.6)}.dark\:border-l-emerald-50\/65:is(.dark *){border-left-color:rgba(236,253,245,.65)}.dark\:border-l-emerald-50\/70:is(.dark *){border-left-color:rgba(236,253,245,.7)}.dark\:border-l-emerald-50\/75:is(.dark *){border-left-color:rgba(236,253,245,.75)}.dark\:border-l-emerald-50\/80:is(.dark *){border-left-color:rgba(236,253,245,.8)}.dark\:border-l-emerald-50\/85:is(.dark *){border-left-color:rgba(236,253,245,.85)}.dark\:border-l-emerald-50\/90:is(.dark *){border-left-color:rgba(236,253,245,.9)}.dark\:border-l-emerald-50\/95:is(.dark *){border-left-color:rgba(236,253,245,.95)}.dark\:border-l-emerald-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(16 185 129/var(--tw-border-opacity,1))}.dark\:border-l-emerald-500\/0:is(.dark *){border-left-color:rgba(16,185,129,0)}.dark\:border-l-emerald-500\/10:is(.dark *){border-left-color:rgba(16,185,129,.1)}.dark\:border-l-emerald-500\/100:is(.dark *){border-left-color:#10b981}.dark\:border-l-emerald-500\/15:is(.dark *){border-left-color:rgba(16,185,129,.15)}.dark\:border-l-emerald-500\/20:is(.dark *){border-left-color:rgba(16,185,129,.2)}.dark\:border-l-emerald-500\/25:is(.dark *){border-left-color:rgba(16,185,129,.25)}.dark\:border-l-emerald-500\/30:is(.dark *){border-left-color:rgba(16,185,129,.3)}.dark\:border-l-emerald-500\/35:is(.dark *){border-left-color:rgba(16,185,129,.35)}.dark\:border-l-emerald-500\/40:is(.dark *){border-left-color:rgba(16,185,129,.4)}.dark\:border-l-emerald-500\/45:is(.dark *){border-left-color:rgba(16,185,129,.45)}.dark\:border-l-emerald-500\/5:is(.dark *){border-left-color:rgba(16,185,129,.05)}.dark\:border-l-emerald-500\/50:is(.dark *){border-left-color:rgba(16,185,129,.5)}.dark\:border-l-emerald-500\/55:is(.dark *){border-left-color:rgba(16,185,129,.55)}.dark\:border-l-emerald-500\/60:is(.dark *){border-left-color:rgba(16,185,129,.6)}.dark\:border-l-emerald-500\/65:is(.dark *){border-left-color:rgba(16,185,129,.65)}.dark\:border-l-emerald-500\/70:is(.dark *){border-left-color:rgba(16,185,129,.7)}.dark\:border-l-emerald-500\/75:is(.dark *){border-left-color:rgba(16,185,129,.75)}.dark\:border-l-emerald-500\/80:is(.dark *){border-left-color:rgba(16,185,129,.8)}.dark\:border-l-emerald-500\/85:is(.dark *){border-left-color:rgba(16,185,129,.85)}.dark\:border-l-emerald-500\/90:is(.dark *){border-left-color:rgba(16,185,129,.9)}.dark\:border-l-emerald-500\/95:is(.dark *){border-left-color:rgba(16,185,129,.95)}.dark\:border-l-emerald-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(2 44 34/var(--tw-border-opacity,1))}.dark\:border-l-emerald-950\/0:is(.dark *){border-left-color:rgba(2,44,34,0)}.dark\:border-l-emerald-950\/10:is(.dark *){border-left-color:rgba(2,44,34,.1)}.dark\:border-l-emerald-950\/100:is(.dark *){border-left-color:#022c22}.dark\:border-l-emerald-950\/15:is(.dark *){border-left-color:rgba(2,44,34,.15)}.dark\:border-l-emerald-950\/20:is(.dark *){border-left-color:rgba(2,44,34,.2)}.dark\:border-l-emerald-950\/25:is(.dark *){border-left-color:rgba(2,44,34,.25)}.dark\:border-l-emerald-950\/30:is(.dark *){border-left-color:rgba(2,44,34,.3)}.dark\:border-l-emerald-950\/35:is(.dark *){border-left-color:rgba(2,44,34,.35)}.dark\:border-l-emerald-950\/40:is(.dark *){border-left-color:rgba(2,44,34,.4)}.dark\:border-l-emerald-950\/45:is(.dark *){border-left-color:rgba(2,44,34,.45)}.dark\:border-l-emerald-950\/5:is(.dark *){border-left-color:rgba(2,44,34,.05)}.dark\:border-l-emerald-950\/50:is(.dark *){border-left-color:rgba(2,44,34,.5)}.dark\:border-l-emerald-950\/55:is(.dark *){border-left-color:rgba(2,44,34,.55)}.dark\:border-l-emerald-950\/60:is(.dark *){border-left-color:rgba(2,44,34,.6)}.dark\:border-l-emerald-950\/65:is(.dark *){border-left-color:rgba(2,44,34,.65)}.dark\:border-l-emerald-950\/70:is(.dark *){border-left-color:rgba(2,44,34,.7)}.dark\:border-l-emerald-950\/75:is(.dark *){border-left-color:rgba(2,44,34,.75)}.dark\:border-l-emerald-950\/80:is(.dark *){border-left-color:rgba(2,44,34,.8)}.dark\:border-l-emerald-950\/85:is(.dark *){border-left-color:rgba(2,44,34,.85)}.dark\:border-l-emerald-950\/90:is(.dark *){border-left-color:rgba(2,44,34,.9)}.dark\:border-l-emerald-950\/95:is(.dark *){border-left-color:rgba(2,44,34,.95)}.dark\:border-l-fuchsia-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(250 232 255/var(--tw-border-opacity,1))}.dark\:border-l-fuchsia-100\/0:is(.dark *){border-left-color:rgba(250,232,255,0)}.dark\:border-l-fuchsia-100\/10:is(.dark *){border-left-color:rgba(250,232,255,.1)}.dark\:border-l-fuchsia-100\/100:is(.dark *){border-left-color:#fae8ff}.dark\:border-l-fuchsia-100\/15:is(.dark *){border-left-color:rgba(250,232,255,.15)}.dark\:border-l-fuchsia-100\/20:is(.dark *){border-left-color:rgba(250,232,255,.2)}.dark\:border-l-fuchsia-100\/25:is(.dark *){border-left-color:rgba(250,232,255,.25)}.dark\:border-l-fuchsia-100\/30:is(.dark *){border-left-color:rgba(250,232,255,.3)}.dark\:border-l-fuchsia-100\/35:is(.dark *){border-left-color:rgba(250,232,255,.35)}.dark\:border-l-fuchsia-100\/40:is(.dark *){border-left-color:rgba(250,232,255,.4)}.dark\:border-l-fuchsia-100\/45:is(.dark *){border-left-color:rgba(250,232,255,.45)}.dark\:border-l-fuchsia-100\/5:is(.dark *){border-left-color:rgba(250,232,255,.05)}.dark\:border-l-fuchsia-100\/50:is(.dark *){border-left-color:rgba(250,232,255,.5)}.dark\:border-l-fuchsia-100\/55:is(.dark *){border-left-color:rgba(250,232,255,.55)}.dark\:border-l-fuchsia-100\/60:is(.dark *){border-left-color:rgba(250,232,255,.6)}.dark\:border-l-fuchsia-100\/65:is(.dark *){border-left-color:rgba(250,232,255,.65)}.dark\:border-l-fuchsia-100\/70:is(.dark *){border-left-color:rgba(250,232,255,.7)}.dark\:border-l-fuchsia-100\/75:is(.dark *){border-left-color:rgba(250,232,255,.75)}.dark\:border-l-fuchsia-100\/80:is(.dark *){border-left-color:rgba(250,232,255,.8)}.dark\:border-l-fuchsia-100\/85:is(.dark *){border-left-color:rgba(250,232,255,.85)}.dark\:border-l-fuchsia-100\/90:is(.dark *){border-left-color:rgba(250,232,255,.9)}.dark\:border-l-fuchsia-100\/95:is(.dark *){border-left-color:rgba(250,232,255,.95)}.dark\:border-l-fuchsia-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(245 208 254/var(--tw-border-opacity,1))}.dark\:border-l-fuchsia-200\/0:is(.dark *){border-left-color:rgba(245,208,254,0)}.dark\:border-l-fuchsia-200\/10:is(.dark *){border-left-color:rgba(245,208,254,.1)}.dark\:border-l-fuchsia-200\/100:is(.dark *){border-left-color:#f5d0fe}.dark\:border-l-fuchsia-200\/15:is(.dark *){border-left-color:rgba(245,208,254,.15)}.dark\:border-l-fuchsia-200\/20:is(.dark *){border-left-color:rgba(245,208,254,.2)}.dark\:border-l-fuchsia-200\/25:is(.dark *){border-left-color:rgba(245,208,254,.25)}.dark\:border-l-fuchsia-200\/30:is(.dark *){border-left-color:rgba(245,208,254,.3)}.dark\:border-l-fuchsia-200\/35:is(.dark *){border-left-color:rgba(245,208,254,.35)}.dark\:border-l-fuchsia-200\/40:is(.dark *){border-left-color:rgba(245,208,254,.4)}.dark\:border-l-fuchsia-200\/45:is(.dark *){border-left-color:rgba(245,208,254,.45)}.dark\:border-l-fuchsia-200\/5:is(.dark *){border-left-color:rgba(245,208,254,.05)}.dark\:border-l-fuchsia-200\/50:is(.dark *){border-left-color:rgba(245,208,254,.5)}.dark\:border-l-fuchsia-200\/55:is(.dark *){border-left-color:rgba(245,208,254,.55)}.dark\:border-l-fuchsia-200\/60:is(.dark *){border-left-color:rgba(245,208,254,.6)}.dark\:border-l-fuchsia-200\/65:is(.dark *){border-left-color:rgba(245,208,254,.65)}.dark\:border-l-fuchsia-200\/70:is(.dark *){border-left-color:rgba(245,208,254,.7)}.dark\:border-l-fuchsia-200\/75:is(.dark *){border-left-color:rgba(245,208,254,.75)}.dark\:border-l-fuchsia-200\/80:is(.dark *){border-left-color:rgba(245,208,254,.8)}.dark\:border-l-fuchsia-200\/85:is(.dark *){border-left-color:rgba(245,208,254,.85)}.dark\:border-l-fuchsia-200\/90:is(.dark *){border-left-color:rgba(245,208,254,.9)}.dark\:border-l-fuchsia-200\/95:is(.dark *){border-left-color:rgba(245,208,254,.95)}.dark\:border-l-fuchsia-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(232 121 249/var(--tw-border-opacity,1))}.dark\:border-l-fuchsia-400\/0:is(.dark *){border-left-color:rgba(232,121,249,0)}.dark\:border-l-fuchsia-400\/10:is(.dark *){border-left-color:rgba(232,121,249,.1)}.dark\:border-l-fuchsia-400\/100:is(.dark *){border-left-color:#e879f9}.dark\:border-l-fuchsia-400\/15:is(.dark *){border-left-color:rgba(232,121,249,.15)}.dark\:border-l-fuchsia-400\/20:is(.dark *){border-left-color:rgba(232,121,249,.2)}.dark\:border-l-fuchsia-400\/25:is(.dark *){border-left-color:rgba(232,121,249,.25)}.dark\:border-l-fuchsia-400\/30:is(.dark *){border-left-color:rgba(232,121,249,.3)}.dark\:border-l-fuchsia-400\/35:is(.dark *){border-left-color:rgba(232,121,249,.35)}.dark\:border-l-fuchsia-400\/40:is(.dark *){border-left-color:rgba(232,121,249,.4)}.dark\:border-l-fuchsia-400\/45:is(.dark *){border-left-color:rgba(232,121,249,.45)}.dark\:border-l-fuchsia-400\/5:is(.dark *){border-left-color:rgba(232,121,249,.05)}.dark\:border-l-fuchsia-400\/50:is(.dark *){border-left-color:rgba(232,121,249,.5)}.dark\:border-l-fuchsia-400\/55:is(.dark *){border-left-color:rgba(232,121,249,.55)}.dark\:border-l-fuchsia-400\/60:is(.dark *){border-left-color:rgba(232,121,249,.6)}.dark\:border-l-fuchsia-400\/65:is(.dark *){border-left-color:rgba(232,121,249,.65)}.dark\:border-l-fuchsia-400\/70:is(.dark *){border-left-color:rgba(232,121,249,.7)}.dark\:border-l-fuchsia-400\/75:is(.dark *){border-left-color:rgba(232,121,249,.75)}.dark\:border-l-fuchsia-400\/80:is(.dark *){border-left-color:rgba(232,121,249,.8)}.dark\:border-l-fuchsia-400\/85:is(.dark *){border-left-color:rgba(232,121,249,.85)}.dark\:border-l-fuchsia-400\/90:is(.dark *){border-left-color:rgba(232,121,249,.9)}.dark\:border-l-fuchsia-400\/95:is(.dark *){border-left-color:rgba(232,121,249,.95)}.dark\:border-l-fuchsia-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(253 244 255/var(--tw-border-opacity,1))}.dark\:border-l-fuchsia-50\/0:is(.dark *){border-left-color:rgba(253,244,255,0)}.dark\:border-l-fuchsia-50\/10:is(.dark *){border-left-color:rgba(253,244,255,.1)}.dark\:border-l-fuchsia-50\/100:is(.dark *){border-left-color:#fdf4ff}.dark\:border-l-fuchsia-50\/15:is(.dark *){border-left-color:rgba(253,244,255,.15)}.dark\:border-l-fuchsia-50\/20:is(.dark *){border-left-color:rgba(253,244,255,.2)}.dark\:border-l-fuchsia-50\/25:is(.dark *){border-left-color:rgba(253,244,255,.25)}.dark\:border-l-fuchsia-50\/30:is(.dark *){border-left-color:rgba(253,244,255,.3)}.dark\:border-l-fuchsia-50\/35:is(.dark *){border-left-color:rgba(253,244,255,.35)}.dark\:border-l-fuchsia-50\/40:is(.dark *){border-left-color:rgba(253,244,255,.4)}.dark\:border-l-fuchsia-50\/45:is(.dark *){border-left-color:rgba(253,244,255,.45)}.dark\:border-l-fuchsia-50\/5:is(.dark *){border-left-color:rgba(253,244,255,.05)}.dark\:border-l-fuchsia-50\/50:is(.dark *){border-left-color:rgba(253,244,255,.5)}.dark\:border-l-fuchsia-50\/55:is(.dark *){border-left-color:rgba(253,244,255,.55)}.dark\:border-l-fuchsia-50\/60:is(.dark *){border-left-color:rgba(253,244,255,.6)}.dark\:border-l-fuchsia-50\/65:is(.dark *){border-left-color:rgba(253,244,255,.65)}.dark\:border-l-fuchsia-50\/70:is(.dark *){border-left-color:rgba(253,244,255,.7)}.dark\:border-l-fuchsia-50\/75:is(.dark *){border-left-color:rgba(253,244,255,.75)}.dark\:border-l-fuchsia-50\/80:is(.dark *){border-left-color:rgba(253,244,255,.8)}.dark\:border-l-fuchsia-50\/85:is(.dark *){border-left-color:rgba(253,244,255,.85)}.dark\:border-l-fuchsia-50\/90:is(.dark *){border-left-color:rgba(253,244,255,.9)}.dark\:border-l-fuchsia-50\/95:is(.dark *){border-left-color:rgba(253,244,255,.95)}.dark\:border-l-fuchsia-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(217 70 239/var(--tw-border-opacity,1))}.dark\:border-l-fuchsia-500\/0:is(.dark *){border-left-color:rgba(217,70,239,0)}.dark\:border-l-fuchsia-500\/10:is(.dark *){border-left-color:rgba(217,70,239,.1)}.dark\:border-l-fuchsia-500\/100:is(.dark *){border-left-color:#d946ef}.dark\:border-l-fuchsia-500\/15:is(.dark *){border-left-color:rgba(217,70,239,.15)}.dark\:border-l-fuchsia-500\/20:is(.dark *){border-left-color:rgba(217,70,239,.2)}.dark\:border-l-fuchsia-500\/25:is(.dark *){border-left-color:rgba(217,70,239,.25)}.dark\:border-l-fuchsia-500\/30:is(.dark *){border-left-color:rgba(217,70,239,.3)}.dark\:border-l-fuchsia-500\/35:is(.dark *){border-left-color:rgba(217,70,239,.35)}.dark\:border-l-fuchsia-500\/40:is(.dark *){border-left-color:rgba(217,70,239,.4)}.dark\:border-l-fuchsia-500\/45:is(.dark *){border-left-color:rgba(217,70,239,.45)}.dark\:border-l-fuchsia-500\/5:is(.dark *){border-left-color:rgba(217,70,239,.05)}.dark\:border-l-fuchsia-500\/50:is(.dark *){border-left-color:rgba(217,70,239,.5)}.dark\:border-l-fuchsia-500\/55:is(.dark *){border-left-color:rgba(217,70,239,.55)}.dark\:border-l-fuchsia-500\/60:is(.dark *){border-left-color:rgba(217,70,239,.6)}.dark\:border-l-fuchsia-500\/65:is(.dark *){border-left-color:rgba(217,70,239,.65)}.dark\:border-l-fuchsia-500\/70:is(.dark *){border-left-color:rgba(217,70,239,.7)}.dark\:border-l-fuchsia-500\/75:is(.dark *){border-left-color:rgba(217,70,239,.75)}.dark\:border-l-fuchsia-500\/80:is(.dark *){border-left-color:rgba(217,70,239,.8)}.dark\:border-l-fuchsia-500\/85:is(.dark *){border-left-color:rgba(217,70,239,.85)}.dark\:border-l-fuchsia-500\/90:is(.dark *){border-left-color:rgba(217,70,239,.9)}.dark\:border-l-fuchsia-500\/95:is(.dark *){border-left-color:rgba(217,70,239,.95)}.dark\:border-l-fuchsia-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(74 4 78/var(--tw-border-opacity,1))}.dark\:border-l-fuchsia-950\/0:is(.dark *){border-left-color:rgba(74,4,78,0)}.dark\:border-l-fuchsia-950\/10:is(.dark *){border-left-color:rgba(74,4,78,.1)}.dark\:border-l-fuchsia-950\/100:is(.dark *){border-left-color:#4a044e}.dark\:border-l-fuchsia-950\/15:is(.dark *){border-left-color:rgba(74,4,78,.15)}.dark\:border-l-fuchsia-950\/20:is(.dark *){border-left-color:rgba(74,4,78,.2)}.dark\:border-l-fuchsia-950\/25:is(.dark *){border-left-color:rgba(74,4,78,.25)}.dark\:border-l-fuchsia-950\/30:is(.dark *){border-left-color:rgba(74,4,78,.3)}.dark\:border-l-fuchsia-950\/35:is(.dark *){border-left-color:rgba(74,4,78,.35)}.dark\:border-l-fuchsia-950\/40:is(.dark *){border-left-color:rgba(74,4,78,.4)}.dark\:border-l-fuchsia-950\/45:is(.dark *){border-left-color:rgba(74,4,78,.45)}.dark\:border-l-fuchsia-950\/5:is(.dark *){border-left-color:rgba(74,4,78,.05)}.dark\:border-l-fuchsia-950\/50:is(.dark *){border-left-color:rgba(74,4,78,.5)}.dark\:border-l-fuchsia-950\/55:is(.dark *){border-left-color:rgba(74,4,78,.55)}.dark\:border-l-fuchsia-950\/60:is(.dark *){border-left-color:rgba(74,4,78,.6)}.dark\:border-l-fuchsia-950\/65:is(.dark *){border-left-color:rgba(74,4,78,.65)}.dark\:border-l-fuchsia-950\/70:is(.dark *){border-left-color:rgba(74,4,78,.7)}.dark\:border-l-fuchsia-950\/75:is(.dark *){border-left-color:rgba(74,4,78,.75)}.dark\:border-l-fuchsia-950\/80:is(.dark *){border-left-color:rgba(74,4,78,.8)}.dark\:border-l-fuchsia-950\/85:is(.dark *){border-left-color:rgba(74,4,78,.85)}.dark\:border-l-fuchsia-950\/90:is(.dark *){border-left-color:rgba(74,4,78,.9)}.dark\:border-l-fuchsia-950\/95:is(.dark *){border-left-color:rgba(74,4,78,.95)}.dark\:border-l-gray-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(243 244 246/var(--tw-border-opacity,1))}.dark\:border-l-gray-100\/0:is(.dark *){border-left-color:rgba(243,244,246,0)}.dark\:border-l-gray-100\/10:is(.dark *){border-left-color:rgba(243,244,246,.1)}.dark\:border-l-gray-100\/100:is(.dark *){border-left-color:#f3f4f6}.dark\:border-l-gray-100\/15:is(.dark *){border-left-color:rgba(243,244,246,.15)}.dark\:border-l-gray-100\/20:is(.dark *){border-left-color:rgba(243,244,246,.2)}.dark\:border-l-gray-100\/25:is(.dark *){border-left-color:rgba(243,244,246,.25)}.dark\:border-l-gray-100\/30:is(.dark *){border-left-color:rgba(243,244,246,.3)}.dark\:border-l-gray-100\/35:is(.dark *){border-left-color:rgba(243,244,246,.35)}.dark\:border-l-gray-100\/40:is(.dark *){border-left-color:rgba(243,244,246,.4)}.dark\:border-l-gray-100\/45:is(.dark *){border-left-color:rgba(243,244,246,.45)}.dark\:border-l-gray-100\/5:is(.dark *){border-left-color:rgba(243,244,246,.05)}.dark\:border-l-gray-100\/50:is(.dark *){border-left-color:rgba(243,244,246,.5)}.dark\:border-l-gray-100\/55:is(.dark *){border-left-color:rgba(243,244,246,.55)}.dark\:border-l-gray-100\/60:is(.dark *){border-left-color:rgba(243,244,246,.6)}.dark\:border-l-gray-100\/65:is(.dark *){border-left-color:rgba(243,244,246,.65)}.dark\:border-l-gray-100\/70:is(.dark *){border-left-color:rgba(243,244,246,.7)}.dark\:border-l-gray-100\/75:is(.dark *){border-left-color:rgba(243,244,246,.75)}.dark\:border-l-gray-100\/80:is(.dark *){border-left-color:rgba(243,244,246,.8)}.dark\:border-l-gray-100\/85:is(.dark *){border-left-color:rgba(243,244,246,.85)}.dark\:border-l-gray-100\/90:is(.dark *){border-left-color:rgba(243,244,246,.9)}.dark\:border-l-gray-100\/95:is(.dark *){border-left-color:rgba(243,244,246,.95)}.dark\:border-l-gray-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(229 231 235/var(--tw-border-opacity,1))}.dark\:border-l-gray-200\/0:is(.dark *){border-left-color:rgba(229,231,235,0)}.dark\:border-l-gray-200\/10:is(.dark *){border-left-color:rgba(229,231,235,.1)}.dark\:border-l-gray-200\/100:is(.dark *){border-left-color:#e5e7eb}.dark\:border-l-gray-200\/15:is(.dark *){border-left-color:rgba(229,231,235,.15)}.dark\:border-l-gray-200\/20:is(.dark *){border-left-color:rgba(229,231,235,.2)}.dark\:border-l-gray-200\/25:is(.dark *){border-left-color:rgba(229,231,235,.25)}.dark\:border-l-gray-200\/30:is(.dark *){border-left-color:rgba(229,231,235,.3)}.dark\:border-l-gray-200\/35:is(.dark *){border-left-color:rgba(229,231,235,.35)}.dark\:border-l-gray-200\/40:is(.dark *){border-left-color:rgba(229,231,235,.4)}.dark\:border-l-gray-200\/45:is(.dark *){border-left-color:rgba(229,231,235,.45)}.dark\:border-l-gray-200\/5:is(.dark *){border-left-color:rgba(229,231,235,.05)}.dark\:border-l-gray-200\/50:is(.dark *){border-left-color:rgba(229,231,235,.5)}.dark\:border-l-gray-200\/55:is(.dark *){border-left-color:rgba(229,231,235,.55)}.dark\:border-l-gray-200\/60:is(.dark *){border-left-color:rgba(229,231,235,.6)}.dark\:border-l-gray-200\/65:is(.dark *){border-left-color:rgba(229,231,235,.65)}.dark\:border-l-gray-200\/70:is(.dark *){border-left-color:rgba(229,231,235,.7)}.dark\:border-l-gray-200\/75:is(.dark *){border-left-color:rgba(229,231,235,.75)}.dark\:border-l-gray-200\/80:is(.dark *){border-left-color:rgba(229,231,235,.8)}.dark\:border-l-gray-200\/85:is(.dark *){border-left-color:rgba(229,231,235,.85)}.dark\:border-l-gray-200\/90:is(.dark *){border-left-color:rgba(229,231,235,.9)}.dark\:border-l-gray-200\/95:is(.dark *){border-left-color:rgba(229,231,235,.95)}.dark\:border-l-gray-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(156 163 175/var(--tw-border-opacity,1))}.dark\:border-l-gray-400\/0:is(.dark *){border-left-color:rgba(156,163,175,0)}.dark\:border-l-gray-400\/10:is(.dark *){border-left-color:rgba(156,163,175,.1)}.dark\:border-l-gray-400\/100:is(.dark *){border-left-color:#9ca3af}.dark\:border-l-gray-400\/15:is(.dark *){border-left-color:rgba(156,163,175,.15)}.dark\:border-l-gray-400\/20:is(.dark *){border-left-color:rgba(156,163,175,.2)}.dark\:border-l-gray-400\/25:is(.dark *){border-left-color:rgba(156,163,175,.25)}.dark\:border-l-gray-400\/30:is(.dark *){border-left-color:rgba(156,163,175,.3)}.dark\:border-l-gray-400\/35:is(.dark *){border-left-color:rgba(156,163,175,.35)}.dark\:border-l-gray-400\/40:is(.dark *){border-left-color:rgba(156,163,175,.4)}.dark\:border-l-gray-400\/45:is(.dark *){border-left-color:rgba(156,163,175,.45)}.dark\:border-l-gray-400\/5:is(.dark *){border-left-color:rgba(156,163,175,.05)}.dark\:border-l-gray-400\/50:is(.dark *){border-left-color:rgba(156,163,175,.5)}.dark\:border-l-gray-400\/55:is(.dark *){border-left-color:rgba(156,163,175,.55)}.dark\:border-l-gray-400\/60:is(.dark *){border-left-color:rgba(156,163,175,.6)}.dark\:border-l-gray-400\/65:is(.dark *){border-left-color:rgba(156,163,175,.65)}.dark\:border-l-gray-400\/70:is(.dark *){border-left-color:rgba(156,163,175,.7)}.dark\:border-l-gray-400\/75:is(.dark *){border-left-color:rgba(156,163,175,.75)}.dark\:border-l-gray-400\/80:is(.dark *){border-left-color:rgba(156,163,175,.8)}.dark\:border-l-gray-400\/85:is(.dark *){border-left-color:rgba(156,163,175,.85)}.dark\:border-l-gray-400\/90:is(.dark *){border-left-color:rgba(156,163,175,.9)}.dark\:border-l-gray-400\/95:is(.dark *){border-left-color:rgba(156,163,175,.95)}.dark\:border-l-gray-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(249 250 251/var(--tw-border-opacity,1))}.dark\:border-l-gray-50\/0:is(.dark *){border-left-color:rgba(249,250,251,0)}.dark\:border-l-gray-50\/10:is(.dark *){border-left-color:rgba(249,250,251,.1)}.dark\:border-l-gray-50\/100:is(.dark *){border-left-color:#f9fafb}.dark\:border-l-gray-50\/15:is(.dark *){border-left-color:rgba(249,250,251,.15)}.dark\:border-l-gray-50\/20:is(.dark *){border-left-color:rgba(249,250,251,.2)}.dark\:border-l-gray-50\/25:is(.dark *){border-left-color:rgba(249,250,251,.25)}.dark\:border-l-gray-50\/30:is(.dark *){border-left-color:rgba(249,250,251,.3)}.dark\:border-l-gray-50\/35:is(.dark *){border-left-color:rgba(249,250,251,.35)}.dark\:border-l-gray-50\/40:is(.dark *){border-left-color:rgba(249,250,251,.4)}.dark\:border-l-gray-50\/45:is(.dark *){border-left-color:rgba(249,250,251,.45)}.dark\:border-l-gray-50\/5:is(.dark *){border-left-color:rgba(249,250,251,.05)}.dark\:border-l-gray-50\/50:is(.dark *){border-left-color:rgba(249,250,251,.5)}.dark\:border-l-gray-50\/55:is(.dark *){border-left-color:rgba(249,250,251,.55)}.dark\:border-l-gray-50\/60:is(.dark *){border-left-color:rgba(249,250,251,.6)}.dark\:border-l-gray-50\/65:is(.dark *){border-left-color:rgba(249,250,251,.65)}.dark\:border-l-gray-50\/70:is(.dark *){border-left-color:rgba(249,250,251,.7)}.dark\:border-l-gray-50\/75:is(.dark *){border-left-color:rgba(249,250,251,.75)}.dark\:border-l-gray-50\/80:is(.dark *){border-left-color:rgba(249,250,251,.8)}.dark\:border-l-gray-50\/85:is(.dark *){border-left-color:rgba(249,250,251,.85)}.dark\:border-l-gray-50\/90:is(.dark *){border-left-color:rgba(249,250,251,.9)}.dark\:border-l-gray-50\/95:is(.dark *){border-left-color:rgba(249,250,251,.95)}.dark\:border-l-gray-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(107 114 128/var(--tw-border-opacity,1))}.dark\:border-l-gray-500\/0:is(.dark *){border-left-color:hsla(220,9%,46%,0)}.dark\:border-l-gray-500\/10:is(.dark *){border-left-color:hsla(220,9%,46%,.1)}.dark\:border-l-gray-500\/100:is(.dark *){border-left-color:#6b7280}.dark\:border-l-gray-500\/15:is(.dark *){border-left-color:hsla(220,9%,46%,.15)}.dark\:border-l-gray-500\/20:is(.dark *){border-left-color:hsla(220,9%,46%,.2)}.dark\:border-l-gray-500\/25:is(.dark *){border-left-color:hsla(220,9%,46%,.25)}.dark\:border-l-gray-500\/30:is(.dark *){border-left-color:hsla(220,9%,46%,.3)}.dark\:border-l-gray-500\/35:is(.dark *){border-left-color:hsla(220,9%,46%,.35)}.dark\:border-l-gray-500\/40:is(.dark *){border-left-color:hsla(220,9%,46%,.4)}.dark\:border-l-gray-500\/45:is(.dark *){border-left-color:hsla(220,9%,46%,.45)}.dark\:border-l-gray-500\/5:is(.dark *){border-left-color:hsla(220,9%,46%,.05)}.dark\:border-l-gray-500\/50:is(.dark *){border-left-color:hsla(220,9%,46%,.5)}.dark\:border-l-gray-500\/55:is(.dark *){border-left-color:hsla(220,9%,46%,.55)}.dark\:border-l-gray-500\/60:is(.dark *){border-left-color:hsla(220,9%,46%,.6)}.dark\:border-l-gray-500\/65:is(.dark *){border-left-color:hsla(220,9%,46%,.65)}.dark\:border-l-gray-500\/70:is(.dark *){border-left-color:hsla(220,9%,46%,.7)}.dark\:border-l-gray-500\/75:is(.dark *){border-left-color:hsla(220,9%,46%,.75)}.dark\:border-l-gray-500\/80:is(.dark *){border-left-color:hsla(220,9%,46%,.8)}.dark\:border-l-gray-500\/85:is(.dark *){border-left-color:hsla(220,9%,46%,.85)}.dark\:border-l-gray-500\/90:is(.dark *){border-left-color:hsla(220,9%,46%,.9)}.dark\:border-l-gray-500\/95:is(.dark *){border-left-color:hsla(220,9%,46%,.95)}.dark\:border-l-gray-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(3 7 18/var(--tw-border-opacity,1))}.dark\:border-l-gray-950\/0:is(.dark *){border-left-color:rgba(3,7,18,0)}.dark\:border-l-gray-950\/10:is(.dark *){border-left-color:rgba(3,7,18,.1)}.dark\:border-l-gray-950\/100:is(.dark *){border-left-color:#030712}.dark\:border-l-gray-950\/15:is(.dark *){border-left-color:rgba(3,7,18,.15)}.dark\:border-l-gray-950\/20:is(.dark *){border-left-color:rgba(3,7,18,.2)}.dark\:border-l-gray-950\/25:is(.dark *){border-left-color:rgba(3,7,18,.25)}.dark\:border-l-gray-950\/30:is(.dark *){border-left-color:rgba(3,7,18,.3)}.dark\:border-l-gray-950\/35:is(.dark *){border-left-color:rgba(3,7,18,.35)}.dark\:border-l-gray-950\/40:is(.dark *){border-left-color:rgba(3,7,18,.4)}.dark\:border-l-gray-950\/45:is(.dark *){border-left-color:rgba(3,7,18,.45)}.dark\:border-l-gray-950\/5:is(.dark *){border-left-color:rgba(3,7,18,.05)}.dark\:border-l-gray-950\/50:is(.dark *){border-left-color:rgba(3,7,18,.5)}.dark\:border-l-gray-950\/55:is(.dark *){border-left-color:rgba(3,7,18,.55)}.dark\:border-l-gray-950\/60:is(.dark *){border-left-color:rgba(3,7,18,.6)}.dark\:border-l-gray-950\/65:is(.dark *){border-left-color:rgba(3,7,18,.65)}.dark\:border-l-gray-950\/70:is(.dark *){border-left-color:rgba(3,7,18,.7)}.dark\:border-l-gray-950\/75:is(.dark *){border-left-color:rgba(3,7,18,.75)}.dark\:border-l-gray-950\/80:is(.dark *){border-left-color:rgba(3,7,18,.8)}.dark\:border-l-gray-950\/85:is(.dark *){border-left-color:rgba(3,7,18,.85)}.dark\:border-l-gray-950\/90:is(.dark *){border-left-color:rgba(3,7,18,.9)}.dark\:border-l-gray-950\/95:is(.dark *){border-left-color:rgba(3,7,18,.95)}.dark\:border-l-green-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(220 252 231/var(--tw-border-opacity,1))}.dark\:border-l-green-100\/0:is(.dark *){border-left-color:rgba(220,252,231,0)}.dark\:border-l-green-100\/10:is(.dark *){border-left-color:rgba(220,252,231,.1)}.dark\:border-l-green-100\/100:is(.dark *){border-left-color:#dcfce7}.dark\:border-l-green-100\/15:is(.dark *){border-left-color:rgba(220,252,231,.15)}.dark\:border-l-green-100\/20:is(.dark *){border-left-color:rgba(220,252,231,.2)}.dark\:border-l-green-100\/25:is(.dark *){border-left-color:rgba(220,252,231,.25)}.dark\:border-l-green-100\/30:is(.dark *){border-left-color:rgba(220,252,231,.3)}.dark\:border-l-green-100\/35:is(.dark *){border-left-color:rgba(220,252,231,.35)}.dark\:border-l-green-100\/40:is(.dark *){border-left-color:rgba(220,252,231,.4)}.dark\:border-l-green-100\/45:is(.dark *){border-left-color:rgba(220,252,231,.45)}.dark\:border-l-green-100\/5:is(.dark *){border-left-color:rgba(220,252,231,.05)}.dark\:border-l-green-100\/50:is(.dark *){border-left-color:rgba(220,252,231,.5)}.dark\:border-l-green-100\/55:is(.dark *){border-left-color:rgba(220,252,231,.55)}.dark\:border-l-green-100\/60:is(.dark *){border-left-color:rgba(220,252,231,.6)}.dark\:border-l-green-100\/65:is(.dark *){border-left-color:rgba(220,252,231,.65)}.dark\:border-l-green-100\/70:is(.dark *){border-left-color:rgba(220,252,231,.7)}.dark\:border-l-green-100\/75:is(.dark *){border-left-color:rgba(220,252,231,.75)}.dark\:border-l-green-100\/80:is(.dark *){border-left-color:rgba(220,252,231,.8)}.dark\:border-l-green-100\/85:is(.dark *){border-left-color:rgba(220,252,231,.85)}.dark\:border-l-green-100\/90:is(.dark *){border-left-color:rgba(220,252,231,.9)}.dark\:border-l-green-100\/95:is(.dark *){border-left-color:rgba(220,252,231,.95)}.dark\:border-l-green-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(187 247 208/var(--tw-border-opacity,1))}.dark\:border-l-green-200\/0:is(.dark *){border-left-color:rgba(187,247,208,0)}.dark\:border-l-green-200\/10:is(.dark *){border-left-color:rgba(187,247,208,.1)}.dark\:border-l-green-200\/100:is(.dark *){border-left-color:#bbf7d0}.dark\:border-l-green-200\/15:is(.dark *){border-left-color:rgba(187,247,208,.15)}.dark\:border-l-green-200\/20:is(.dark *){border-left-color:rgba(187,247,208,.2)}.dark\:border-l-green-200\/25:is(.dark *){border-left-color:rgba(187,247,208,.25)}.dark\:border-l-green-200\/30:is(.dark *){border-left-color:rgba(187,247,208,.3)}.dark\:border-l-green-200\/35:is(.dark *){border-left-color:rgba(187,247,208,.35)}.dark\:border-l-green-200\/40:is(.dark *){border-left-color:rgba(187,247,208,.4)}.dark\:border-l-green-200\/45:is(.dark *){border-left-color:rgba(187,247,208,.45)}.dark\:border-l-green-200\/5:is(.dark *){border-left-color:rgba(187,247,208,.05)}.dark\:border-l-green-200\/50:is(.dark *){border-left-color:rgba(187,247,208,.5)}.dark\:border-l-green-200\/55:is(.dark *){border-left-color:rgba(187,247,208,.55)}.dark\:border-l-green-200\/60:is(.dark *){border-left-color:rgba(187,247,208,.6)}.dark\:border-l-green-200\/65:is(.dark *){border-left-color:rgba(187,247,208,.65)}.dark\:border-l-green-200\/70:is(.dark *){border-left-color:rgba(187,247,208,.7)}.dark\:border-l-green-200\/75:is(.dark *){border-left-color:rgba(187,247,208,.75)}.dark\:border-l-green-200\/80:is(.dark *){border-left-color:rgba(187,247,208,.8)}.dark\:border-l-green-200\/85:is(.dark *){border-left-color:rgba(187,247,208,.85)}.dark\:border-l-green-200\/90:is(.dark *){border-left-color:rgba(187,247,208,.9)}.dark\:border-l-green-200\/95:is(.dark *){border-left-color:rgba(187,247,208,.95)}.dark\:border-l-green-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(74 222 128/var(--tw-border-opacity,1))}.dark\:border-l-green-400\/0:is(.dark *){border-left-color:rgba(74,222,128,0)}.dark\:border-l-green-400\/10:is(.dark *){border-left-color:rgba(74,222,128,.1)}.dark\:border-l-green-400\/100:is(.dark *){border-left-color:#4ade80}.dark\:border-l-green-400\/15:is(.dark *){border-left-color:rgba(74,222,128,.15)}.dark\:border-l-green-400\/20:is(.dark *){border-left-color:rgba(74,222,128,.2)}.dark\:border-l-green-400\/25:is(.dark *){border-left-color:rgba(74,222,128,.25)}.dark\:border-l-green-400\/30:is(.dark *){border-left-color:rgba(74,222,128,.3)}.dark\:border-l-green-400\/35:is(.dark *){border-left-color:rgba(74,222,128,.35)}.dark\:border-l-green-400\/40:is(.dark *){border-left-color:rgba(74,222,128,.4)}.dark\:border-l-green-400\/45:is(.dark *){border-left-color:rgba(74,222,128,.45)}.dark\:border-l-green-400\/5:is(.dark *){border-left-color:rgba(74,222,128,.05)}.dark\:border-l-green-400\/50:is(.dark *){border-left-color:rgba(74,222,128,.5)}.dark\:border-l-green-400\/55:is(.dark *){border-left-color:rgba(74,222,128,.55)}.dark\:border-l-green-400\/60:is(.dark *){border-left-color:rgba(74,222,128,.6)}.dark\:border-l-green-400\/65:is(.dark *){border-left-color:rgba(74,222,128,.65)}.dark\:border-l-green-400\/70:is(.dark *){border-left-color:rgba(74,222,128,.7)}.dark\:border-l-green-400\/75:is(.dark *){border-left-color:rgba(74,222,128,.75)}.dark\:border-l-green-400\/80:is(.dark *){border-left-color:rgba(74,222,128,.8)}.dark\:border-l-green-400\/85:is(.dark *){border-left-color:rgba(74,222,128,.85)}.dark\:border-l-green-400\/90:is(.dark *){border-left-color:rgba(74,222,128,.9)}.dark\:border-l-green-400\/95:is(.dark *){border-left-color:rgba(74,222,128,.95)}.dark\:border-l-green-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(240 253 244/var(--tw-border-opacity,1))}.dark\:border-l-green-50\/0:is(.dark *){border-left-color:rgba(240,253,244,0)}.dark\:border-l-green-50\/10:is(.dark *){border-left-color:rgba(240,253,244,.1)}.dark\:border-l-green-50\/100:is(.dark *){border-left-color:#f0fdf4}.dark\:border-l-green-50\/15:is(.dark *){border-left-color:rgba(240,253,244,.15)}.dark\:border-l-green-50\/20:is(.dark *){border-left-color:rgba(240,253,244,.2)}.dark\:border-l-green-50\/25:is(.dark *){border-left-color:rgba(240,253,244,.25)}.dark\:border-l-green-50\/30:is(.dark *){border-left-color:rgba(240,253,244,.3)}.dark\:border-l-green-50\/35:is(.dark *){border-left-color:rgba(240,253,244,.35)}.dark\:border-l-green-50\/40:is(.dark *){border-left-color:rgba(240,253,244,.4)}.dark\:border-l-green-50\/45:is(.dark *){border-left-color:rgba(240,253,244,.45)}.dark\:border-l-green-50\/5:is(.dark *){border-left-color:rgba(240,253,244,.05)}.dark\:border-l-green-50\/50:is(.dark *){border-left-color:rgba(240,253,244,.5)}.dark\:border-l-green-50\/55:is(.dark *){border-left-color:rgba(240,253,244,.55)}.dark\:border-l-green-50\/60:is(.dark *){border-left-color:rgba(240,253,244,.6)}.dark\:border-l-green-50\/65:is(.dark *){border-left-color:rgba(240,253,244,.65)}.dark\:border-l-green-50\/70:is(.dark *){border-left-color:rgba(240,253,244,.7)}.dark\:border-l-green-50\/75:is(.dark *){border-left-color:rgba(240,253,244,.75)}.dark\:border-l-green-50\/80:is(.dark *){border-left-color:rgba(240,253,244,.8)}.dark\:border-l-green-50\/85:is(.dark *){border-left-color:rgba(240,253,244,.85)}.dark\:border-l-green-50\/90:is(.dark *){border-left-color:rgba(240,253,244,.9)}.dark\:border-l-green-50\/95:is(.dark *){border-left-color:rgba(240,253,244,.95)}.dark\:border-l-green-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(34 197 94/var(--tw-border-opacity,1))}.dark\:border-l-green-500\/0:is(.dark *){border-left-color:rgba(34,197,94,0)}.dark\:border-l-green-500\/10:is(.dark *){border-left-color:rgba(34,197,94,.1)}.dark\:border-l-green-500\/100:is(.dark *){border-left-color:#22c55e}.dark\:border-l-green-500\/15:is(.dark *){border-left-color:rgba(34,197,94,.15)}.dark\:border-l-green-500\/20:is(.dark *){border-left-color:rgba(34,197,94,.2)}.dark\:border-l-green-500\/25:is(.dark *){border-left-color:rgba(34,197,94,.25)}.dark\:border-l-green-500\/30:is(.dark *){border-left-color:rgba(34,197,94,.3)}.dark\:border-l-green-500\/35:is(.dark *){border-left-color:rgba(34,197,94,.35)}.dark\:border-l-green-500\/40:is(.dark *){border-left-color:rgba(34,197,94,.4)}.dark\:border-l-green-500\/45:is(.dark *){border-left-color:rgba(34,197,94,.45)}.dark\:border-l-green-500\/5:is(.dark *){border-left-color:rgba(34,197,94,.05)}.dark\:border-l-green-500\/50:is(.dark *){border-left-color:rgba(34,197,94,.5)}.dark\:border-l-green-500\/55:is(.dark *){border-left-color:rgba(34,197,94,.55)}.dark\:border-l-green-500\/60:is(.dark *){border-left-color:rgba(34,197,94,.6)}.dark\:border-l-green-500\/65:is(.dark *){border-left-color:rgba(34,197,94,.65)}.dark\:border-l-green-500\/70:is(.dark *){border-left-color:rgba(34,197,94,.7)}.dark\:border-l-green-500\/75:is(.dark *){border-left-color:rgba(34,197,94,.75)}.dark\:border-l-green-500\/80:is(.dark *){border-left-color:rgba(34,197,94,.8)}.dark\:border-l-green-500\/85:is(.dark *){border-left-color:rgba(34,197,94,.85)}.dark\:border-l-green-500\/90:is(.dark *){border-left-color:rgba(34,197,94,.9)}.dark\:border-l-green-500\/95:is(.dark *){border-left-color:rgba(34,197,94,.95)}.dark\:border-l-green-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(5 46 22/var(--tw-border-opacity,1))}.dark\:border-l-green-950\/0:is(.dark *){border-left-color:rgba(5,46,22,0)}.dark\:border-l-green-950\/10:is(.dark *){border-left-color:rgba(5,46,22,.1)}.dark\:border-l-green-950\/100:is(.dark *){border-left-color:#052e16}.dark\:border-l-green-950\/15:is(.dark *){border-left-color:rgba(5,46,22,.15)}.dark\:border-l-green-950\/20:is(.dark *){border-left-color:rgba(5,46,22,.2)}.dark\:border-l-green-950\/25:is(.dark *){border-left-color:rgba(5,46,22,.25)}.dark\:border-l-green-950\/30:is(.dark *){border-left-color:rgba(5,46,22,.3)}.dark\:border-l-green-950\/35:is(.dark *){border-left-color:rgba(5,46,22,.35)}.dark\:border-l-green-950\/40:is(.dark *){border-left-color:rgba(5,46,22,.4)}.dark\:border-l-green-950\/45:is(.dark *){border-left-color:rgba(5,46,22,.45)}.dark\:border-l-green-950\/5:is(.dark *){border-left-color:rgba(5,46,22,.05)}.dark\:border-l-green-950\/50:is(.dark *){border-left-color:rgba(5,46,22,.5)}.dark\:border-l-green-950\/55:is(.dark *){border-left-color:rgba(5,46,22,.55)}.dark\:border-l-green-950\/60:is(.dark *){border-left-color:rgba(5,46,22,.6)}.dark\:border-l-green-950\/65:is(.dark *){border-left-color:rgba(5,46,22,.65)}.dark\:border-l-green-950\/70:is(.dark *){border-left-color:rgba(5,46,22,.7)}.dark\:border-l-green-950\/75:is(.dark *){border-left-color:rgba(5,46,22,.75)}.dark\:border-l-green-950\/80:is(.dark *){border-left-color:rgba(5,46,22,.8)}.dark\:border-l-green-950\/85:is(.dark *){border-left-color:rgba(5,46,22,.85)}.dark\:border-l-green-950\/90:is(.dark *){border-left-color:rgba(5,46,22,.9)}.dark\:border-l-green-950\/95:is(.dark *){border-left-color:rgba(5,46,22,.95)}.dark\:border-l-indigo-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(224 231 255/var(--tw-border-opacity,1))}.dark\:border-l-indigo-100\/0:is(.dark *){border-left-color:rgba(224,231,255,0)}.dark\:border-l-indigo-100\/10:is(.dark *){border-left-color:rgba(224,231,255,.1)}.dark\:border-l-indigo-100\/100:is(.dark *){border-left-color:#e0e7ff}.dark\:border-l-indigo-100\/15:is(.dark *){border-left-color:rgba(224,231,255,.15)}.dark\:border-l-indigo-100\/20:is(.dark *){border-left-color:rgba(224,231,255,.2)}.dark\:border-l-indigo-100\/25:is(.dark *){border-left-color:rgba(224,231,255,.25)}.dark\:border-l-indigo-100\/30:is(.dark *){border-left-color:rgba(224,231,255,.3)}.dark\:border-l-indigo-100\/35:is(.dark *){border-left-color:rgba(224,231,255,.35)}.dark\:border-l-indigo-100\/40:is(.dark *){border-left-color:rgba(224,231,255,.4)}.dark\:border-l-indigo-100\/45:is(.dark *){border-left-color:rgba(224,231,255,.45)}.dark\:border-l-indigo-100\/5:is(.dark *){border-left-color:rgba(224,231,255,.05)}.dark\:border-l-indigo-100\/50:is(.dark *){border-left-color:rgba(224,231,255,.5)}.dark\:border-l-indigo-100\/55:is(.dark *){border-left-color:rgba(224,231,255,.55)}.dark\:border-l-indigo-100\/60:is(.dark *){border-left-color:rgba(224,231,255,.6)}.dark\:border-l-indigo-100\/65:is(.dark *){border-left-color:rgba(224,231,255,.65)}.dark\:border-l-indigo-100\/70:is(.dark *){border-left-color:rgba(224,231,255,.7)}.dark\:border-l-indigo-100\/75:is(.dark *){border-left-color:rgba(224,231,255,.75)}.dark\:border-l-indigo-100\/80:is(.dark *){border-left-color:rgba(224,231,255,.8)}.dark\:border-l-indigo-100\/85:is(.dark *){border-left-color:rgba(224,231,255,.85)}.dark\:border-l-indigo-100\/90:is(.dark *){border-left-color:rgba(224,231,255,.9)}.dark\:border-l-indigo-100\/95:is(.dark *){border-left-color:rgba(224,231,255,.95)}.dark\:border-l-indigo-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(199 210 254/var(--tw-border-opacity,1))}.dark\:border-l-indigo-200\/0:is(.dark *){border-left-color:rgba(199,210,254,0)}.dark\:border-l-indigo-200\/10:is(.dark *){border-left-color:rgba(199,210,254,.1)}.dark\:border-l-indigo-200\/100:is(.dark *){border-left-color:#c7d2fe}.dark\:border-l-indigo-200\/15:is(.dark *){border-left-color:rgba(199,210,254,.15)}.dark\:border-l-indigo-200\/20:is(.dark *){border-left-color:rgba(199,210,254,.2)}.dark\:border-l-indigo-200\/25:is(.dark *){border-left-color:rgba(199,210,254,.25)}.dark\:border-l-indigo-200\/30:is(.dark *){border-left-color:rgba(199,210,254,.3)}.dark\:border-l-indigo-200\/35:is(.dark *){border-left-color:rgba(199,210,254,.35)}.dark\:border-l-indigo-200\/40:is(.dark *){border-left-color:rgba(199,210,254,.4)}.dark\:border-l-indigo-200\/45:is(.dark *){border-left-color:rgba(199,210,254,.45)}.dark\:border-l-indigo-200\/5:is(.dark *){border-left-color:rgba(199,210,254,.05)}.dark\:border-l-indigo-200\/50:is(.dark *){border-left-color:rgba(199,210,254,.5)}.dark\:border-l-indigo-200\/55:is(.dark *){border-left-color:rgba(199,210,254,.55)}.dark\:border-l-indigo-200\/60:is(.dark *){border-left-color:rgba(199,210,254,.6)}.dark\:border-l-indigo-200\/65:is(.dark *){border-left-color:rgba(199,210,254,.65)}.dark\:border-l-indigo-200\/70:is(.dark *){border-left-color:rgba(199,210,254,.7)}.dark\:border-l-indigo-200\/75:is(.dark *){border-left-color:rgba(199,210,254,.75)}.dark\:border-l-indigo-200\/80:is(.dark *){border-left-color:rgba(199,210,254,.8)}.dark\:border-l-indigo-200\/85:is(.dark *){border-left-color:rgba(199,210,254,.85)}.dark\:border-l-indigo-200\/90:is(.dark *){border-left-color:rgba(199,210,254,.9)}.dark\:border-l-indigo-200\/95:is(.dark *){border-left-color:rgba(199,210,254,.95)}.dark\:border-l-indigo-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(129 140 248/var(--tw-border-opacity,1))}.dark\:border-l-indigo-400\/0:is(.dark *){border-left-color:rgba(129,140,248,0)}.dark\:border-l-indigo-400\/10:is(.dark *){border-left-color:rgba(129,140,248,.1)}.dark\:border-l-indigo-400\/100:is(.dark *){border-left-color:#818cf8}.dark\:border-l-indigo-400\/15:is(.dark *){border-left-color:rgba(129,140,248,.15)}.dark\:border-l-indigo-400\/20:is(.dark *){border-left-color:rgba(129,140,248,.2)}.dark\:border-l-indigo-400\/25:is(.dark *){border-left-color:rgba(129,140,248,.25)}.dark\:border-l-indigo-400\/30:is(.dark *){border-left-color:rgba(129,140,248,.3)}.dark\:border-l-indigo-400\/35:is(.dark *){border-left-color:rgba(129,140,248,.35)}.dark\:border-l-indigo-400\/40:is(.dark *){border-left-color:rgba(129,140,248,.4)}.dark\:border-l-indigo-400\/45:is(.dark *){border-left-color:rgba(129,140,248,.45)}.dark\:border-l-indigo-400\/5:is(.dark *){border-left-color:rgba(129,140,248,.05)}.dark\:border-l-indigo-400\/50:is(.dark *){border-left-color:rgba(129,140,248,.5)}.dark\:border-l-indigo-400\/55:is(.dark *){border-left-color:rgba(129,140,248,.55)}.dark\:border-l-indigo-400\/60:is(.dark *){border-left-color:rgba(129,140,248,.6)}.dark\:border-l-indigo-400\/65:is(.dark *){border-left-color:rgba(129,140,248,.65)}.dark\:border-l-indigo-400\/70:is(.dark *){border-left-color:rgba(129,140,248,.7)}.dark\:border-l-indigo-400\/75:is(.dark *){border-left-color:rgba(129,140,248,.75)}.dark\:border-l-indigo-400\/80:is(.dark *){border-left-color:rgba(129,140,248,.8)}.dark\:border-l-indigo-400\/85:is(.dark *){border-left-color:rgba(129,140,248,.85)}.dark\:border-l-indigo-400\/90:is(.dark *){border-left-color:rgba(129,140,248,.9)}.dark\:border-l-indigo-400\/95:is(.dark *){border-left-color:rgba(129,140,248,.95)}.dark\:border-l-indigo-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(238 242 255/var(--tw-border-opacity,1))}.dark\:border-l-indigo-50\/0:is(.dark *){border-left-color:rgba(238,242,255,0)}.dark\:border-l-indigo-50\/10:is(.dark *){border-left-color:rgba(238,242,255,.1)}.dark\:border-l-indigo-50\/100:is(.dark *){border-left-color:#eef2ff}.dark\:border-l-indigo-50\/15:is(.dark *){border-left-color:rgba(238,242,255,.15)}.dark\:border-l-indigo-50\/20:is(.dark *){border-left-color:rgba(238,242,255,.2)}.dark\:border-l-indigo-50\/25:is(.dark *){border-left-color:rgba(238,242,255,.25)}.dark\:border-l-indigo-50\/30:is(.dark *){border-left-color:rgba(238,242,255,.3)}.dark\:border-l-indigo-50\/35:is(.dark *){border-left-color:rgba(238,242,255,.35)}.dark\:border-l-indigo-50\/40:is(.dark *){border-left-color:rgba(238,242,255,.4)}.dark\:border-l-indigo-50\/45:is(.dark *){border-left-color:rgba(238,242,255,.45)}.dark\:border-l-indigo-50\/5:is(.dark *){border-left-color:rgba(238,242,255,.05)}.dark\:border-l-indigo-50\/50:is(.dark *){border-left-color:rgba(238,242,255,.5)}.dark\:border-l-indigo-50\/55:is(.dark *){border-left-color:rgba(238,242,255,.55)}.dark\:border-l-indigo-50\/60:is(.dark *){border-left-color:rgba(238,242,255,.6)}.dark\:border-l-indigo-50\/65:is(.dark *){border-left-color:rgba(238,242,255,.65)}.dark\:border-l-indigo-50\/70:is(.dark *){border-left-color:rgba(238,242,255,.7)}.dark\:border-l-indigo-50\/75:is(.dark *){border-left-color:rgba(238,242,255,.75)}.dark\:border-l-indigo-50\/80:is(.dark *){border-left-color:rgba(238,242,255,.8)}.dark\:border-l-indigo-50\/85:is(.dark *){border-left-color:rgba(238,242,255,.85)}.dark\:border-l-indigo-50\/90:is(.dark *){border-left-color:rgba(238,242,255,.9)}.dark\:border-l-indigo-50\/95:is(.dark *){border-left-color:rgba(238,242,255,.95)}.dark\:border-l-indigo-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(99 102 241/var(--tw-border-opacity,1))}.dark\:border-l-indigo-500\/0:is(.dark *){border-left-color:rgba(99,102,241,0)}.dark\:border-l-indigo-500\/10:is(.dark *){border-left-color:rgba(99,102,241,.1)}.dark\:border-l-indigo-500\/100:is(.dark *){border-left-color:#6366f1}.dark\:border-l-indigo-500\/15:is(.dark *){border-left-color:rgba(99,102,241,.15)}.dark\:border-l-indigo-500\/20:is(.dark *){border-left-color:rgba(99,102,241,.2)}.dark\:border-l-indigo-500\/25:is(.dark *){border-left-color:rgba(99,102,241,.25)}.dark\:border-l-indigo-500\/30:is(.dark *){border-left-color:rgba(99,102,241,.3)}.dark\:border-l-indigo-500\/35:is(.dark *){border-left-color:rgba(99,102,241,.35)}.dark\:border-l-indigo-500\/40:is(.dark *){border-left-color:rgba(99,102,241,.4)}.dark\:border-l-indigo-500\/45:is(.dark *){border-left-color:rgba(99,102,241,.45)}.dark\:border-l-indigo-500\/5:is(.dark *){border-left-color:rgba(99,102,241,.05)}.dark\:border-l-indigo-500\/50:is(.dark *){border-left-color:rgba(99,102,241,.5)}.dark\:border-l-indigo-500\/55:is(.dark *){border-left-color:rgba(99,102,241,.55)}.dark\:border-l-indigo-500\/60:is(.dark *){border-left-color:rgba(99,102,241,.6)}.dark\:border-l-indigo-500\/65:is(.dark *){border-left-color:rgba(99,102,241,.65)}.dark\:border-l-indigo-500\/70:is(.dark *){border-left-color:rgba(99,102,241,.7)}.dark\:border-l-indigo-500\/75:is(.dark *){border-left-color:rgba(99,102,241,.75)}.dark\:border-l-indigo-500\/80:is(.dark *){border-left-color:rgba(99,102,241,.8)}.dark\:border-l-indigo-500\/85:is(.dark *){border-left-color:rgba(99,102,241,.85)}.dark\:border-l-indigo-500\/90:is(.dark *){border-left-color:rgba(99,102,241,.9)}.dark\:border-l-indigo-500\/95:is(.dark *){border-left-color:rgba(99,102,241,.95)}.dark\:border-l-indigo-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(30 27 75/var(--tw-border-opacity,1))}.dark\:border-l-indigo-950\/0:is(.dark *){border-left-color:rgba(30,27,75,0)}.dark\:border-l-indigo-950\/10:is(.dark *){border-left-color:rgba(30,27,75,.1)}.dark\:border-l-indigo-950\/100:is(.dark *){border-left-color:#1e1b4b}.dark\:border-l-indigo-950\/15:is(.dark *){border-left-color:rgba(30,27,75,.15)}.dark\:border-l-indigo-950\/20:is(.dark *){border-left-color:rgba(30,27,75,.2)}.dark\:border-l-indigo-950\/25:is(.dark *){border-left-color:rgba(30,27,75,.25)}.dark\:border-l-indigo-950\/30:is(.dark *){border-left-color:rgba(30,27,75,.3)}.dark\:border-l-indigo-950\/35:is(.dark *){border-left-color:rgba(30,27,75,.35)}.dark\:border-l-indigo-950\/40:is(.dark *){border-left-color:rgba(30,27,75,.4)}.dark\:border-l-indigo-950\/45:is(.dark *){border-left-color:rgba(30,27,75,.45)}.dark\:border-l-indigo-950\/5:is(.dark *){border-left-color:rgba(30,27,75,.05)}.dark\:border-l-indigo-950\/50:is(.dark *){border-left-color:rgba(30,27,75,.5)}.dark\:border-l-indigo-950\/55:is(.dark *){border-left-color:rgba(30,27,75,.55)}.dark\:border-l-indigo-950\/60:is(.dark *){border-left-color:rgba(30,27,75,.6)}.dark\:border-l-indigo-950\/65:is(.dark *){border-left-color:rgba(30,27,75,.65)}.dark\:border-l-indigo-950\/70:is(.dark *){border-left-color:rgba(30,27,75,.7)}.dark\:border-l-indigo-950\/75:is(.dark *){border-left-color:rgba(30,27,75,.75)}.dark\:border-l-indigo-950\/80:is(.dark *){border-left-color:rgba(30,27,75,.8)}.dark\:border-l-indigo-950\/85:is(.dark *){border-left-color:rgba(30,27,75,.85)}.dark\:border-l-indigo-950\/90:is(.dark *){border-left-color:rgba(30,27,75,.9)}.dark\:border-l-indigo-950\/95:is(.dark *){border-left-color:rgba(30,27,75,.95)}.dark\:border-l-lime-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(236 252 203/var(--tw-border-opacity,1))}.dark\:border-l-lime-100\/0:is(.dark *){border-left-color:hsla(80,89%,89%,0)}.dark\:border-l-lime-100\/10:is(.dark *){border-left-color:hsla(80,89%,89%,.1)}.dark\:border-l-lime-100\/100:is(.dark *){border-left-color:#ecfccb}.dark\:border-l-lime-100\/15:is(.dark *){border-left-color:hsla(80,89%,89%,.15)}.dark\:border-l-lime-100\/20:is(.dark *){border-left-color:hsla(80,89%,89%,.2)}.dark\:border-l-lime-100\/25:is(.dark *){border-left-color:hsla(80,89%,89%,.25)}.dark\:border-l-lime-100\/30:is(.dark *){border-left-color:hsla(80,89%,89%,.3)}.dark\:border-l-lime-100\/35:is(.dark *){border-left-color:hsla(80,89%,89%,.35)}.dark\:border-l-lime-100\/40:is(.dark *){border-left-color:hsla(80,89%,89%,.4)}.dark\:border-l-lime-100\/45:is(.dark *){border-left-color:hsla(80,89%,89%,.45)}.dark\:border-l-lime-100\/5:is(.dark *){border-left-color:hsla(80,89%,89%,.05)}.dark\:border-l-lime-100\/50:is(.dark *){border-left-color:hsla(80,89%,89%,.5)}.dark\:border-l-lime-100\/55:is(.dark *){border-left-color:hsla(80,89%,89%,.55)}.dark\:border-l-lime-100\/60:is(.dark *){border-left-color:hsla(80,89%,89%,.6)}.dark\:border-l-lime-100\/65:is(.dark *){border-left-color:hsla(80,89%,89%,.65)}.dark\:border-l-lime-100\/70:is(.dark *){border-left-color:hsla(80,89%,89%,.7)}.dark\:border-l-lime-100\/75:is(.dark *){border-left-color:hsla(80,89%,89%,.75)}.dark\:border-l-lime-100\/80:is(.dark *){border-left-color:hsla(80,89%,89%,.8)}.dark\:border-l-lime-100\/85:is(.dark *){border-left-color:hsla(80,89%,89%,.85)}.dark\:border-l-lime-100\/90:is(.dark *){border-left-color:hsla(80,89%,89%,.9)}.dark\:border-l-lime-100\/95:is(.dark *){border-left-color:hsla(80,89%,89%,.95)}.dark\:border-l-lime-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(217 249 157/var(--tw-border-opacity,1))}.dark\:border-l-lime-200\/0:is(.dark *){border-left-color:hsla(81,88%,80%,0)}.dark\:border-l-lime-200\/10:is(.dark *){border-left-color:hsla(81,88%,80%,.1)}.dark\:border-l-lime-200\/100:is(.dark *){border-left-color:#d9f99d}.dark\:border-l-lime-200\/15:is(.dark *){border-left-color:hsla(81,88%,80%,.15)}.dark\:border-l-lime-200\/20:is(.dark *){border-left-color:hsla(81,88%,80%,.2)}.dark\:border-l-lime-200\/25:is(.dark *){border-left-color:hsla(81,88%,80%,.25)}.dark\:border-l-lime-200\/30:is(.dark *){border-left-color:hsla(81,88%,80%,.3)}.dark\:border-l-lime-200\/35:is(.dark *){border-left-color:hsla(81,88%,80%,.35)}.dark\:border-l-lime-200\/40:is(.dark *){border-left-color:hsla(81,88%,80%,.4)}.dark\:border-l-lime-200\/45:is(.dark *){border-left-color:hsla(81,88%,80%,.45)}.dark\:border-l-lime-200\/5:is(.dark *){border-left-color:hsla(81,88%,80%,.05)}.dark\:border-l-lime-200\/50:is(.dark *){border-left-color:hsla(81,88%,80%,.5)}.dark\:border-l-lime-200\/55:is(.dark *){border-left-color:hsla(81,88%,80%,.55)}.dark\:border-l-lime-200\/60:is(.dark *){border-left-color:hsla(81,88%,80%,.6)}.dark\:border-l-lime-200\/65:is(.dark *){border-left-color:hsla(81,88%,80%,.65)}.dark\:border-l-lime-200\/70:is(.dark *){border-left-color:hsla(81,88%,80%,.7)}.dark\:border-l-lime-200\/75:is(.dark *){border-left-color:hsla(81,88%,80%,.75)}.dark\:border-l-lime-200\/80:is(.dark *){border-left-color:hsla(81,88%,80%,.8)}.dark\:border-l-lime-200\/85:is(.dark *){border-left-color:hsla(81,88%,80%,.85)}.dark\:border-l-lime-200\/90:is(.dark *){border-left-color:hsla(81,88%,80%,.9)}.dark\:border-l-lime-200\/95:is(.dark *){border-left-color:hsla(81,88%,80%,.95)}.dark\:border-l-lime-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(163 230 53/var(--tw-border-opacity,1))}.dark\:border-l-lime-400\/0:is(.dark *){border-left-color:rgba(163,230,53,0)}.dark\:border-l-lime-400\/10:is(.dark *){border-left-color:rgba(163,230,53,.1)}.dark\:border-l-lime-400\/100:is(.dark *){border-left-color:#a3e635}.dark\:border-l-lime-400\/15:is(.dark *){border-left-color:rgba(163,230,53,.15)}.dark\:border-l-lime-400\/20:is(.dark *){border-left-color:rgba(163,230,53,.2)}.dark\:border-l-lime-400\/25:is(.dark *){border-left-color:rgba(163,230,53,.25)}.dark\:border-l-lime-400\/30:is(.dark *){border-left-color:rgba(163,230,53,.3)}.dark\:border-l-lime-400\/35:is(.dark *){border-left-color:rgba(163,230,53,.35)}.dark\:border-l-lime-400\/40:is(.dark *){border-left-color:rgba(163,230,53,.4)}.dark\:border-l-lime-400\/45:is(.dark *){border-left-color:rgba(163,230,53,.45)}.dark\:border-l-lime-400\/5:is(.dark *){border-left-color:rgba(163,230,53,.05)}.dark\:border-l-lime-400\/50:is(.dark *){border-left-color:rgba(163,230,53,.5)}.dark\:border-l-lime-400\/55:is(.dark *){border-left-color:rgba(163,230,53,.55)}.dark\:border-l-lime-400\/60:is(.dark *){border-left-color:rgba(163,230,53,.6)}.dark\:border-l-lime-400\/65:is(.dark *){border-left-color:rgba(163,230,53,.65)}.dark\:border-l-lime-400\/70:is(.dark *){border-left-color:rgba(163,230,53,.7)}.dark\:border-l-lime-400\/75:is(.dark *){border-left-color:rgba(163,230,53,.75)}.dark\:border-l-lime-400\/80:is(.dark *){border-left-color:rgba(163,230,53,.8)}.dark\:border-l-lime-400\/85:is(.dark *){border-left-color:rgba(163,230,53,.85)}.dark\:border-l-lime-400\/90:is(.dark *){border-left-color:rgba(163,230,53,.9)}.dark\:border-l-lime-400\/95:is(.dark *){border-left-color:rgba(163,230,53,.95)}.dark\:border-l-lime-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(247 254 231/var(--tw-border-opacity,1))}.dark\:border-l-lime-50\/0:is(.dark *){border-left-color:hsla(78,92%,95%,0)}.dark\:border-l-lime-50\/10:is(.dark *){border-left-color:hsla(78,92%,95%,.1)}.dark\:border-l-lime-50\/100:is(.dark *){border-left-color:#f7fee7}.dark\:border-l-lime-50\/15:is(.dark *){border-left-color:hsla(78,92%,95%,.15)}.dark\:border-l-lime-50\/20:is(.dark *){border-left-color:hsla(78,92%,95%,.2)}.dark\:border-l-lime-50\/25:is(.dark *){border-left-color:hsla(78,92%,95%,.25)}.dark\:border-l-lime-50\/30:is(.dark *){border-left-color:hsla(78,92%,95%,.3)}.dark\:border-l-lime-50\/35:is(.dark *){border-left-color:hsla(78,92%,95%,.35)}.dark\:border-l-lime-50\/40:is(.dark *){border-left-color:hsla(78,92%,95%,.4)}.dark\:border-l-lime-50\/45:is(.dark *){border-left-color:hsla(78,92%,95%,.45)}.dark\:border-l-lime-50\/5:is(.dark *){border-left-color:hsla(78,92%,95%,.05)}.dark\:border-l-lime-50\/50:is(.dark *){border-left-color:hsla(78,92%,95%,.5)}.dark\:border-l-lime-50\/55:is(.dark *){border-left-color:hsla(78,92%,95%,.55)}.dark\:border-l-lime-50\/60:is(.dark *){border-left-color:hsla(78,92%,95%,.6)}.dark\:border-l-lime-50\/65:is(.dark *){border-left-color:hsla(78,92%,95%,.65)}.dark\:border-l-lime-50\/70:is(.dark *){border-left-color:hsla(78,92%,95%,.7)}.dark\:border-l-lime-50\/75:is(.dark *){border-left-color:hsla(78,92%,95%,.75)}.dark\:border-l-lime-50\/80:is(.dark *){border-left-color:hsla(78,92%,95%,.8)}.dark\:border-l-lime-50\/85:is(.dark *){border-left-color:hsla(78,92%,95%,.85)}.dark\:border-l-lime-50\/90:is(.dark *){border-left-color:hsla(78,92%,95%,.9)}.dark\:border-l-lime-50\/95:is(.dark *){border-left-color:hsla(78,92%,95%,.95)}.dark\:border-l-lime-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(132 204 22/var(--tw-border-opacity,1))}.dark\:border-l-lime-500\/0:is(.dark *){border-left-color:rgba(132,204,22,0)}.dark\:border-l-lime-500\/10:is(.dark *){border-left-color:rgba(132,204,22,.1)}.dark\:border-l-lime-500\/100:is(.dark *){border-left-color:#84cc16}.dark\:border-l-lime-500\/15:is(.dark *){border-left-color:rgba(132,204,22,.15)}.dark\:border-l-lime-500\/20:is(.dark *){border-left-color:rgba(132,204,22,.2)}.dark\:border-l-lime-500\/25:is(.dark *){border-left-color:rgba(132,204,22,.25)}.dark\:border-l-lime-500\/30:is(.dark *){border-left-color:rgba(132,204,22,.3)}.dark\:border-l-lime-500\/35:is(.dark *){border-left-color:rgba(132,204,22,.35)}.dark\:border-l-lime-500\/40:is(.dark *){border-left-color:rgba(132,204,22,.4)}.dark\:border-l-lime-500\/45:is(.dark *){border-left-color:rgba(132,204,22,.45)}.dark\:border-l-lime-500\/5:is(.dark *){border-left-color:rgba(132,204,22,.05)}.dark\:border-l-lime-500\/50:is(.dark *){border-left-color:rgba(132,204,22,.5)}.dark\:border-l-lime-500\/55:is(.dark *){border-left-color:rgba(132,204,22,.55)}.dark\:border-l-lime-500\/60:is(.dark *){border-left-color:rgba(132,204,22,.6)}.dark\:border-l-lime-500\/65:is(.dark *){border-left-color:rgba(132,204,22,.65)}.dark\:border-l-lime-500\/70:is(.dark *){border-left-color:rgba(132,204,22,.7)}.dark\:border-l-lime-500\/75:is(.dark *){border-left-color:rgba(132,204,22,.75)}.dark\:border-l-lime-500\/80:is(.dark *){border-left-color:rgba(132,204,22,.8)}.dark\:border-l-lime-500\/85:is(.dark *){border-left-color:rgba(132,204,22,.85)}.dark\:border-l-lime-500\/90:is(.dark *){border-left-color:rgba(132,204,22,.9)}.dark\:border-l-lime-500\/95:is(.dark *){border-left-color:rgba(132,204,22,.95)}.dark\:border-l-lime-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(26 46 5/var(--tw-border-opacity,1))}.dark\:border-l-lime-950\/0:is(.dark *){border-left-color:rgba(26,46,5,0)}.dark\:border-l-lime-950\/10:is(.dark *){border-left-color:rgba(26,46,5,.1)}.dark\:border-l-lime-950\/100:is(.dark *){border-left-color:#1a2e05}.dark\:border-l-lime-950\/15:is(.dark *){border-left-color:rgba(26,46,5,.15)}.dark\:border-l-lime-950\/20:is(.dark *){border-left-color:rgba(26,46,5,.2)}.dark\:border-l-lime-950\/25:is(.dark *){border-left-color:rgba(26,46,5,.25)}.dark\:border-l-lime-950\/30:is(.dark *){border-left-color:rgba(26,46,5,.3)}.dark\:border-l-lime-950\/35:is(.dark *){border-left-color:rgba(26,46,5,.35)}.dark\:border-l-lime-950\/40:is(.dark *){border-left-color:rgba(26,46,5,.4)}.dark\:border-l-lime-950\/45:is(.dark *){border-left-color:rgba(26,46,5,.45)}.dark\:border-l-lime-950\/5:is(.dark *){border-left-color:rgba(26,46,5,.05)}.dark\:border-l-lime-950\/50:is(.dark *){border-left-color:rgba(26,46,5,.5)}.dark\:border-l-lime-950\/55:is(.dark *){border-left-color:rgba(26,46,5,.55)}.dark\:border-l-lime-950\/60:is(.dark *){border-left-color:rgba(26,46,5,.6)}.dark\:border-l-lime-950\/65:is(.dark *){border-left-color:rgba(26,46,5,.65)}.dark\:border-l-lime-950\/70:is(.dark *){border-left-color:rgba(26,46,5,.7)}.dark\:border-l-lime-950\/75:is(.dark *){border-left-color:rgba(26,46,5,.75)}.dark\:border-l-lime-950\/80:is(.dark *){border-left-color:rgba(26,46,5,.8)}.dark\:border-l-lime-950\/85:is(.dark *){border-left-color:rgba(26,46,5,.85)}.dark\:border-l-lime-950\/90:is(.dark *){border-left-color:rgba(26,46,5,.9)}.dark\:border-l-lime-950\/95:is(.dark *){border-left-color:rgba(26,46,5,.95)}.dark\:border-l-neutral-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(245 245 245/var(--tw-border-opacity,1))}.dark\:border-l-neutral-100\/0:is(.dark *){border-left-color:hsla(0,0%,96%,0)}.dark\:border-l-neutral-100\/10:is(.dark *){border-left-color:hsla(0,0%,96%,.1)}.dark\:border-l-neutral-100\/100:is(.dark *){border-left-color:#f5f5f5}.dark\:border-l-neutral-100\/15:is(.dark *){border-left-color:hsla(0,0%,96%,.15)}.dark\:border-l-neutral-100\/20:is(.dark *){border-left-color:hsla(0,0%,96%,.2)}.dark\:border-l-neutral-100\/25:is(.dark *){border-left-color:hsla(0,0%,96%,.25)}.dark\:border-l-neutral-100\/30:is(.dark *){border-left-color:hsla(0,0%,96%,.3)}.dark\:border-l-neutral-100\/35:is(.dark *){border-left-color:hsla(0,0%,96%,.35)}.dark\:border-l-neutral-100\/40:is(.dark *){border-left-color:hsla(0,0%,96%,.4)}.dark\:border-l-neutral-100\/45:is(.dark *){border-left-color:hsla(0,0%,96%,.45)}.dark\:border-l-neutral-100\/5:is(.dark *){border-left-color:hsla(0,0%,96%,.05)}.dark\:border-l-neutral-100\/50:is(.dark *){border-left-color:hsla(0,0%,96%,.5)}.dark\:border-l-neutral-100\/55:is(.dark *){border-left-color:hsla(0,0%,96%,.55)}.dark\:border-l-neutral-100\/60:is(.dark *){border-left-color:hsla(0,0%,96%,.6)}.dark\:border-l-neutral-100\/65:is(.dark *){border-left-color:hsla(0,0%,96%,.65)}.dark\:border-l-neutral-100\/70:is(.dark *){border-left-color:hsla(0,0%,96%,.7)}.dark\:border-l-neutral-100\/75:is(.dark *){border-left-color:hsla(0,0%,96%,.75)}.dark\:border-l-neutral-100\/80:is(.dark *){border-left-color:hsla(0,0%,96%,.8)}.dark\:border-l-neutral-100\/85:is(.dark *){border-left-color:hsla(0,0%,96%,.85)}.dark\:border-l-neutral-100\/90:is(.dark *){border-left-color:hsla(0,0%,96%,.9)}.dark\:border-l-neutral-100\/95:is(.dark *){border-left-color:hsla(0,0%,96%,.95)}.dark\:border-l-neutral-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(229 229 229/var(--tw-border-opacity,1))}.dark\:border-l-neutral-200\/0:is(.dark *){border-left-color:hsla(0,0%,90%,0)}.dark\:border-l-neutral-200\/10:is(.dark *){border-left-color:hsla(0,0%,90%,.1)}.dark\:border-l-neutral-200\/100:is(.dark *){border-left-color:#e5e5e5}.dark\:border-l-neutral-200\/15:is(.dark *){border-left-color:hsla(0,0%,90%,.15)}.dark\:border-l-neutral-200\/20:is(.dark *){border-left-color:hsla(0,0%,90%,.2)}.dark\:border-l-neutral-200\/25:is(.dark *){border-left-color:hsla(0,0%,90%,.25)}.dark\:border-l-neutral-200\/30:is(.dark *){border-left-color:hsla(0,0%,90%,.3)}.dark\:border-l-neutral-200\/35:is(.dark *){border-left-color:hsla(0,0%,90%,.35)}.dark\:border-l-neutral-200\/40:is(.dark *){border-left-color:hsla(0,0%,90%,.4)}.dark\:border-l-neutral-200\/45:is(.dark *){border-left-color:hsla(0,0%,90%,.45)}.dark\:border-l-neutral-200\/5:is(.dark *){border-left-color:hsla(0,0%,90%,.05)}.dark\:border-l-neutral-200\/50:is(.dark *){border-left-color:hsla(0,0%,90%,.5)}.dark\:border-l-neutral-200\/55:is(.dark *){border-left-color:hsla(0,0%,90%,.55)}.dark\:border-l-neutral-200\/60:is(.dark *){border-left-color:hsla(0,0%,90%,.6)}.dark\:border-l-neutral-200\/65:is(.dark *){border-left-color:hsla(0,0%,90%,.65)}.dark\:border-l-neutral-200\/70:is(.dark *){border-left-color:hsla(0,0%,90%,.7)}.dark\:border-l-neutral-200\/75:is(.dark *){border-left-color:hsla(0,0%,90%,.75)}.dark\:border-l-neutral-200\/80:is(.dark *){border-left-color:hsla(0,0%,90%,.8)}.dark\:border-l-neutral-200\/85:is(.dark *){border-left-color:hsla(0,0%,90%,.85)}.dark\:border-l-neutral-200\/90:is(.dark *){border-left-color:hsla(0,0%,90%,.9)}.dark\:border-l-neutral-200\/95:is(.dark *){border-left-color:hsla(0,0%,90%,.95)}.dark\:border-l-neutral-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(163 163 163/var(--tw-border-opacity,1))}.dark\:border-l-neutral-400\/0:is(.dark *){border-left-color:hsla(0,0%,64%,0)}.dark\:border-l-neutral-400\/10:is(.dark *){border-left-color:hsla(0,0%,64%,.1)}.dark\:border-l-neutral-400\/100:is(.dark *){border-left-color:#a3a3a3}.dark\:border-l-neutral-400\/15:is(.dark *){border-left-color:hsla(0,0%,64%,.15)}.dark\:border-l-neutral-400\/20:is(.dark *){border-left-color:hsla(0,0%,64%,.2)}.dark\:border-l-neutral-400\/25:is(.dark *){border-left-color:hsla(0,0%,64%,.25)}.dark\:border-l-neutral-400\/30:is(.dark *){border-left-color:hsla(0,0%,64%,.3)}.dark\:border-l-neutral-400\/35:is(.dark *){border-left-color:hsla(0,0%,64%,.35)}.dark\:border-l-neutral-400\/40:is(.dark *){border-left-color:hsla(0,0%,64%,.4)}.dark\:border-l-neutral-400\/45:is(.dark *){border-left-color:hsla(0,0%,64%,.45)}.dark\:border-l-neutral-400\/5:is(.dark *){border-left-color:hsla(0,0%,64%,.05)}.dark\:border-l-neutral-400\/50:is(.dark *){border-left-color:hsla(0,0%,64%,.5)}.dark\:border-l-neutral-400\/55:is(.dark *){border-left-color:hsla(0,0%,64%,.55)}.dark\:border-l-neutral-400\/60:is(.dark *){border-left-color:hsla(0,0%,64%,.6)}.dark\:border-l-neutral-400\/65:is(.dark *){border-left-color:hsla(0,0%,64%,.65)}.dark\:border-l-neutral-400\/70:is(.dark *){border-left-color:hsla(0,0%,64%,.7)}.dark\:border-l-neutral-400\/75:is(.dark *){border-left-color:hsla(0,0%,64%,.75)}.dark\:border-l-neutral-400\/80:is(.dark *){border-left-color:hsla(0,0%,64%,.8)}.dark\:border-l-neutral-400\/85:is(.dark *){border-left-color:hsla(0,0%,64%,.85)}.dark\:border-l-neutral-400\/90:is(.dark *){border-left-color:hsla(0,0%,64%,.9)}.dark\:border-l-neutral-400\/95:is(.dark *){border-left-color:hsla(0,0%,64%,.95)}.dark\:border-l-neutral-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(250 250 250/var(--tw-border-opacity,1))}.dark\:border-l-neutral-50\/0:is(.dark *){border-left-color:hsla(0,0%,98%,0)}.dark\:border-l-neutral-50\/10:is(.dark *){border-left-color:hsla(0,0%,98%,.1)}.dark\:border-l-neutral-50\/100:is(.dark *){border-left-color:#fafafa}.dark\:border-l-neutral-50\/15:is(.dark *){border-left-color:hsla(0,0%,98%,.15)}.dark\:border-l-neutral-50\/20:is(.dark *){border-left-color:hsla(0,0%,98%,.2)}.dark\:border-l-neutral-50\/25:is(.dark *){border-left-color:hsla(0,0%,98%,.25)}.dark\:border-l-neutral-50\/30:is(.dark *){border-left-color:hsla(0,0%,98%,.3)}.dark\:border-l-neutral-50\/35:is(.dark *){border-left-color:hsla(0,0%,98%,.35)}.dark\:border-l-neutral-50\/40:is(.dark *){border-left-color:hsla(0,0%,98%,.4)}.dark\:border-l-neutral-50\/45:is(.dark *){border-left-color:hsla(0,0%,98%,.45)}.dark\:border-l-neutral-50\/5:is(.dark *){border-left-color:hsla(0,0%,98%,.05)}.dark\:border-l-neutral-50\/50:is(.dark *){border-left-color:hsla(0,0%,98%,.5)}.dark\:border-l-neutral-50\/55:is(.dark *){border-left-color:hsla(0,0%,98%,.55)}.dark\:border-l-neutral-50\/60:is(.dark *){border-left-color:hsla(0,0%,98%,.6)}.dark\:border-l-neutral-50\/65:is(.dark *){border-left-color:hsla(0,0%,98%,.65)}.dark\:border-l-neutral-50\/70:is(.dark *){border-left-color:hsla(0,0%,98%,.7)}.dark\:border-l-neutral-50\/75:is(.dark *){border-left-color:hsla(0,0%,98%,.75)}.dark\:border-l-neutral-50\/80:is(.dark *){border-left-color:hsla(0,0%,98%,.8)}.dark\:border-l-neutral-50\/85:is(.dark *){border-left-color:hsla(0,0%,98%,.85)}.dark\:border-l-neutral-50\/90:is(.dark *){border-left-color:hsla(0,0%,98%,.9)}.dark\:border-l-neutral-50\/95:is(.dark *){border-left-color:hsla(0,0%,98%,.95)}.dark\:border-l-neutral-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(115 115 115/var(--tw-border-opacity,1))}.dark\:border-l-neutral-500\/0:is(.dark *){border-left-color:hsla(0,0%,45%,0)}.dark\:border-l-neutral-500\/10:is(.dark *){border-left-color:hsla(0,0%,45%,.1)}.dark\:border-l-neutral-500\/100:is(.dark *){border-left-color:#737373}.dark\:border-l-neutral-500\/15:is(.dark *){border-left-color:hsla(0,0%,45%,.15)}.dark\:border-l-neutral-500\/20:is(.dark *){border-left-color:hsla(0,0%,45%,.2)}.dark\:border-l-neutral-500\/25:is(.dark *){border-left-color:hsla(0,0%,45%,.25)}.dark\:border-l-neutral-500\/30:is(.dark *){border-left-color:hsla(0,0%,45%,.3)}.dark\:border-l-neutral-500\/35:is(.dark *){border-left-color:hsla(0,0%,45%,.35)}.dark\:border-l-neutral-500\/40:is(.dark *){border-left-color:hsla(0,0%,45%,.4)}.dark\:border-l-neutral-500\/45:is(.dark *){border-left-color:hsla(0,0%,45%,.45)}.dark\:border-l-neutral-500\/5:is(.dark *){border-left-color:hsla(0,0%,45%,.05)}.dark\:border-l-neutral-500\/50:is(.dark *){border-left-color:hsla(0,0%,45%,.5)}.dark\:border-l-neutral-500\/55:is(.dark *){border-left-color:hsla(0,0%,45%,.55)}.dark\:border-l-neutral-500\/60:is(.dark *){border-left-color:hsla(0,0%,45%,.6)}.dark\:border-l-neutral-500\/65:is(.dark *){border-left-color:hsla(0,0%,45%,.65)}.dark\:border-l-neutral-500\/70:is(.dark *){border-left-color:hsla(0,0%,45%,.7)}.dark\:border-l-neutral-500\/75:is(.dark *){border-left-color:hsla(0,0%,45%,.75)}.dark\:border-l-neutral-500\/80:is(.dark *){border-left-color:hsla(0,0%,45%,.8)}.dark\:border-l-neutral-500\/85:is(.dark *){border-left-color:hsla(0,0%,45%,.85)}.dark\:border-l-neutral-500\/90:is(.dark *){border-left-color:hsla(0,0%,45%,.9)}.dark\:border-l-neutral-500\/95:is(.dark *){border-left-color:hsla(0,0%,45%,.95)}.dark\:border-l-neutral-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(10 10 10/var(--tw-border-opacity,1))}.dark\:border-l-neutral-950\/0:is(.dark *){border-left-color:hsla(0,0%,4%,0)}.dark\:border-l-neutral-950\/10:is(.dark *){border-left-color:hsla(0,0%,4%,.1)}.dark\:border-l-neutral-950\/100:is(.dark *){border-left-color:#0a0a0a}.dark\:border-l-neutral-950\/15:is(.dark *){border-left-color:hsla(0,0%,4%,.15)}.dark\:border-l-neutral-950\/20:is(.dark *){border-left-color:hsla(0,0%,4%,.2)}.dark\:border-l-neutral-950\/25:is(.dark *){border-left-color:hsla(0,0%,4%,.25)}.dark\:border-l-neutral-950\/30:is(.dark *){border-left-color:hsla(0,0%,4%,.3)}.dark\:border-l-neutral-950\/35:is(.dark *){border-left-color:hsla(0,0%,4%,.35)}.dark\:border-l-neutral-950\/40:is(.dark *){border-left-color:hsla(0,0%,4%,.4)}.dark\:border-l-neutral-950\/45:is(.dark *){border-left-color:hsla(0,0%,4%,.45)}.dark\:border-l-neutral-950\/5:is(.dark *){border-left-color:hsla(0,0%,4%,.05)}.dark\:border-l-neutral-950\/50:is(.dark *){border-left-color:hsla(0,0%,4%,.5)}.dark\:border-l-neutral-950\/55:is(.dark *){border-left-color:hsla(0,0%,4%,.55)}.dark\:border-l-neutral-950\/60:is(.dark *){border-left-color:hsla(0,0%,4%,.6)}.dark\:border-l-neutral-950\/65:is(.dark *){border-left-color:hsla(0,0%,4%,.65)}.dark\:border-l-neutral-950\/70:is(.dark *){border-left-color:hsla(0,0%,4%,.7)}.dark\:border-l-neutral-950\/75:is(.dark *){border-left-color:hsla(0,0%,4%,.75)}.dark\:border-l-neutral-950\/80:is(.dark *){border-left-color:hsla(0,0%,4%,.8)}.dark\:border-l-neutral-950\/85:is(.dark *){border-left-color:hsla(0,0%,4%,.85)}.dark\:border-l-neutral-950\/90:is(.dark *){border-left-color:hsla(0,0%,4%,.9)}.dark\:border-l-neutral-950\/95:is(.dark *){border-left-color:hsla(0,0%,4%,.95)}.dark\:border-l-orange-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(255 237 213/var(--tw-border-opacity,1))}.dark\:border-l-orange-100\/0:is(.dark *){border-left-color:rgba(255,237,213,0)}.dark\:border-l-orange-100\/10:is(.dark *){border-left-color:rgba(255,237,213,.1)}.dark\:border-l-orange-100\/100:is(.dark *){border-left-color:#ffedd5}.dark\:border-l-orange-100\/15:is(.dark *){border-left-color:rgba(255,237,213,.15)}.dark\:border-l-orange-100\/20:is(.dark *){border-left-color:rgba(255,237,213,.2)}.dark\:border-l-orange-100\/25:is(.dark *){border-left-color:rgba(255,237,213,.25)}.dark\:border-l-orange-100\/30:is(.dark *){border-left-color:rgba(255,237,213,.3)}.dark\:border-l-orange-100\/35:is(.dark *){border-left-color:rgba(255,237,213,.35)}.dark\:border-l-orange-100\/40:is(.dark *){border-left-color:rgba(255,237,213,.4)}.dark\:border-l-orange-100\/45:is(.dark *){border-left-color:rgba(255,237,213,.45)}.dark\:border-l-orange-100\/5:is(.dark *){border-left-color:rgba(255,237,213,.05)}.dark\:border-l-orange-100\/50:is(.dark *){border-left-color:rgba(255,237,213,.5)}.dark\:border-l-orange-100\/55:is(.dark *){border-left-color:rgba(255,237,213,.55)}.dark\:border-l-orange-100\/60:is(.dark *){border-left-color:rgba(255,237,213,.6)}.dark\:border-l-orange-100\/65:is(.dark *){border-left-color:rgba(255,237,213,.65)}.dark\:border-l-orange-100\/70:is(.dark *){border-left-color:rgba(255,237,213,.7)}.dark\:border-l-orange-100\/75:is(.dark *){border-left-color:rgba(255,237,213,.75)}.dark\:border-l-orange-100\/80:is(.dark *){border-left-color:rgba(255,237,213,.8)}.dark\:border-l-orange-100\/85:is(.dark *){border-left-color:rgba(255,237,213,.85)}.dark\:border-l-orange-100\/90:is(.dark *){border-left-color:rgba(255,237,213,.9)}.dark\:border-l-orange-100\/95:is(.dark *){border-left-color:rgba(255,237,213,.95)}.dark\:border-l-orange-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(254 215 170/var(--tw-border-opacity,1))}.dark\:border-l-orange-200\/0:is(.dark *){border-left-color:hsla(32,98%,83%,0)}.dark\:border-l-orange-200\/10:is(.dark *){border-left-color:hsla(32,98%,83%,.1)}.dark\:border-l-orange-200\/100:is(.dark *){border-left-color:#fed7aa}.dark\:border-l-orange-200\/15:is(.dark *){border-left-color:hsla(32,98%,83%,.15)}.dark\:border-l-orange-200\/20:is(.dark *){border-left-color:hsla(32,98%,83%,.2)}.dark\:border-l-orange-200\/25:is(.dark *){border-left-color:hsla(32,98%,83%,.25)}.dark\:border-l-orange-200\/30:is(.dark *){border-left-color:hsla(32,98%,83%,.3)}.dark\:border-l-orange-200\/35:is(.dark *){border-left-color:hsla(32,98%,83%,.35)}.dark\:border-l-orange-200\/40:is(.dark *){border-left-color:hsla(32,98%,83%,.4)}.dark\:border-l-orange-200\/45:is(.dark *){border-left-color:hsla(32,98%,83%,.45)}.dark\:border-l-orange-200\/5:is(.dark *){border-left-color:hsla(32,98%,83%,.05)}.dark\:border-l-orange-200\/50:is(.dark *){border-left-color:hsla(32,98%,83%,.5)}.dark\:border-l-orange-200\/55:is(.dark *){border-left-color:hsla(32,98%,83%,.55)}.dark\:border-l-orange-200\/60:is(.dark *){border-left-color:hsla(32,98%,83%,.6)}.dark\:border-l-orange-200\/65:is(.dark *){border-left-color:hsla(32,98%,83%,.65)}.dark\:border-l-orange-200\/70:is(.dark *){border-left-color:hsla(32,98%,83%,.7)}.dark\:border-l-orange-200\/75:is(.dark *){border-left-color:hsla(32,98%,83%,.75)}.dark\:border-l-orange-200\/80:is(.dark *){border-left-color:hsla(32,98%,83%,.8)}.dark\:border-l-orange-200\/85:is(.dark *){border-left-color:hsla(32,98%,83%,.85)}.dark\:border-l-orange-200\/90:is(.dark *){border-left-color:hsla(32,98%,83%,.9)}.dark\:border-l-orange-200\/95:is(.dark *){border-left-color:hsla(32,98%,83%,.95)}.dark\:border-l-orange-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(251 146 60/var(--tw-border-opacity,1))}.dark\:border-l-orange-400\/0:is(.dark *){border-left-color:rgba(251,146,60,0)}.dark\:border-l-orange-400\/10:is(.dark *){border-left-color:rgba(251,146,60,.1)}.dark\:border-l-orange-400\/100:is(.dark *){border-left-color:#fb923c}.dark\:border-l-orange-400\/15:is(.dark *){border-left-color:rgba(251,146,60,.15)}.dark\:border-l-orange-400\/20:is(.dark *){border-left-color:rgba(251,146,60,.2)}.dark\:border-l-orange-400\/25:is(.dark *){border-left-color:rgba(251,146,60,.25)}.dark\:border-l-orange-400\/30:is(.dark *){border-left-color:rgba(251,146,60,.3)}.dark\:border-l-orange-400\/35:is(.dark *){border-left-color:rgba(251,146,60,.35)}.dark\:border-l-orange-400\/40:is(.dark *){border-left-color:rgba(251,146,60,.4)}.dark\:border-l-orange-400\/45:is(.dark *){border-left-color:rgba(251,146,60,.45)}.dark\:border-l-orange-400\/5:is(.dark *){border-left-color:rgba(251,146,60,.05)}.dark\:border-l-orange-400\/50:is(.dark *){border-left-color:rgba(251,146,60,.5)}.dark\:border-l-orange-400\/55:is(.dark *){border-left-color:rgba(251,146,60,.55)}.dark\:border-l-orange-400\/60:is(.dark *){border-left-color:rgba(251,146,60,.6)}.dark\:border-l-orange-400\/65:is(.dark *){border-left-color:rgba(251,146,60,.65)}.dark\:border-l-orange-400\/70:is(.dark *){border-left-color:rgba(251,146,60,.7)}.dark\:border-l-orange-400\/75:is(.dark *){border-left-color:rgba(251,146,60,.75)}.dark\:border-l-orange-400\/80:is(.dark *){border-left-color:rgba(251,146,60,.8)}.dark\:border-l-orange-400\/85:is(.dark *){border-left-color:rgba(251,146,60,.85)}.dark\:border-l-orange-400\/90:is(.dark *){border-left-color:rgba(251,146,60,.9)}.dark\:border-l-orange-400\/95:is(.dark *){border-left-color:rgba(251,146,60,.95)}.dark\:border-l-orange-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(255 247 237/var(--tw-border-opacity,1))}.dark\:border-l-orange-50\/0:is(.dark *){border-left-color:rgba(255,247,237,0)}.dark\:border-l-orange-50\/10:is(.dark *){border-left-color:rgba(255,247,237,.1)}.dark\:border-l-orange-50\/100:is(.dark *){border-left-color:#fff7ed}.dark\:border-l-orange-50\/15:is(.dark *){border-left-color:rgba(255,247,237,.15)}.dark\:border-l-orange-50\/20:is(.dark *){border-left-color:rgba(255,247,237,.2)}.dark\:border-l-orange-50\/25:is(.dark *){border-left-color:rgba(255,247,237,.25)}.dark\:border-l-orange-50\/30:is(.dark *){border-left-color:rgba(255,247,237,.3)}.dark\:border-l-orange-50\/35:is(.dark *){border-left-color:rgba(255,247,237,.35)}.dark\:border-l-orange-50\/40:is(.dark *){border-left-color:rgba(255,247,237,.4)}.dark\:border-l-orange-50\/45:is(.dark *){border-left-color:rgba(255,247,237,.45)}.dark\:border-l-orange-50\/5:is(.dark *){border-left-color:rgba(255,247,237,.05)}.dark\:border-l-orange-50\/50:is(.dark *){border-left-color:rgba(255,247,237,.5)}.dark\:border-l-orange-50\/55:is(.dark *){border-left-color:rgba(255,247,237,.55)}.dark\:border-l-orange-50\/60:is(.dark *){border-left-color:rgba(255,247,237,.6)}.dark\:border-l-orange-50\/65:is(.dark *){border-left-color:rgba(255,247,237,.65)}.dark\:border-l-orange-50\/70:is(.dark *){border-left-color:rgba(255,247,237,.7)}.dark\:border-l-orange-50\/75:is(.dark *){border-left-color:rgba(255,247,237,.75)}.dark\:border-l-orange-50\/80:is(.dark *){border-left-color:rgba(255,247,237,.8)}.dark\:border-l-orange-50\/85:is(.dark *){border-left-color:rgba(255,247,237,.85)}.dark\:border-l-orange-50\/90:is(.dark *){border-left-color:rgba(255,247,237,.9)}.dark\:border-l-orange-50\/95:is(.dark *){border-left-color:rgba(255,247,237,.95)}.dark\:border-l-orange-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(249 115 22/var(--tw-border-opacity,1))}.dark\:border-l-orange-500\/0:is(.dark *){border-left-color:rgba(249,115,22,0)}.dark\:border-l-orange-500\/10:is(.dark *){border-left-color:rgba(249,115,22,.1)}.dark\:border-l-orange-500\/100:is(.dark *){border-left-color:#f97316}.dark\:border-l-orange-500\/15:is(.dark *){border-left-color:rgba(249,115,22,.15)}.dark\:border-l-orange-500\/20:is(.dark *){border-left-color:rgba(249,115,22,.2)}.dark\:border-l-orange-500\/25:is(.dark *){border-left-color:rgba(249,115,22,.25)}.dark\:border-l-orange-500\/30:is(.dark *){border-left-color:rgba(249,115,22,.3)}.dark\:border-l-orange-500\/35:is(.dark *){border-left-color:rgba(249,115,22,.35)}.dark\:border-l-orange-500\/40:is(.dark *){border-left-color:rgba(249,115,22,.4)}.dark\:border-l-orange-500\/45:is(.dark *){border-left-color:rgba(249,115,22,.45)}.dark\:border-l-orange-500\/5:is(.dark *){border-left-color:rgba(249,115,22,.05)}.dark\:border-l-orange-500\/50:is(.dark *){border-left-color:rgba(249,115,22,.5)}.dark\:border-l-orange-500\/55:is(.dark *){border-left-color:rgba(249,115,22,.55)}.dark\:border-l-orange-500\/60:is(.dark *){border-left-color:rgba(249,115,22,.6)}.dark\:border-l-orange-500\/65:is(.dark *){border-left-color:rgba(249,115,22,.65)}.dark\:border-l-orange-500\/70:is(.dark *){border-left-color:rgba(249,115,22,.7)}.dark\:border-l-orange-500\/75:is(.dark *){border-left-color:rgba(249,115,22,.75)}.dark\:border-l-orange-500\/80:is(.dark *){border-left-color:rgba(249,115,22,.8)}.dark\:border-l-orange-500\/85:is(.dark *){border-left-color:rgba(249,115,22,.85)}.dark\:border-l-orange-500\/90:is(.dark *){border-left-color:rgba(249,115,22,.9)}.dark\:border-l-orange-500\/95:is(.dark *){border-left-color:rgba(249,115,22,.95)}.dark\:border-l-orange-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(67 20 7/var(--tw-border-opacity,1))}.dark\:border-l-orange-950\/0:is(.dark *){border-left-color:rgba(67,20,7,0)}.dark\:border-l-orange-950\/10:is(.dark *){border-left-color:rgba(67,20,7,.1)}.dark\:border-l-orange-950\/100:is(.dark *){border-left-color:#431407}.dark\:border-l-orange-950\/15:is(.dark *){border-left-color:rgba(67,20,7,.15)}.dark\:border-l-orange-950\/20:is(.dark *){border-left-color:rgba(67,20,7,.2)}.dark\:border-l-orange-950\/25:is(.dark *){border-left-color:rgba(67,20,7,.25)}.dark\:border-l-orange-950\/30:is(.dark *){border-left-color:rgba(67,20,7,.3)}.dark\:border-l-orange-950\/35:is(.dark *){border-left-color:rgba(67,20,7,.35)}.dark\:border-l-orange-950\/40:is(.dark *){border-left-color:rgba(67,20,7,.4)}.dark\:border-l-orange-950\/45:is(.dark *){border-left-color:rgba(67,20,7,.45)}.dark\:border-l-orange-950\/5:is(.dark *){border-left-color:rgba(67,20,7,.05)}.dark\:border-l-orange-950\/50:is(.dark *){border-left-color:rgba(67,20,7,.5)}.dark\:border-l-orange-950\/55:is(.dark *){border-left-color:rgba(67,20,7,.55)}.dark\:border-l-orange-950\/60:is(.dark *){border-left-color:rgba(67,20,7,.6)}.dark\:border-l-orange-950\/65:is(.dark *){border-left-color:rgba(67,20,7,.65)}.dark\:border-l-orange-950\/70:is(.dark *){border-left-color:rgba(67,20,7,.7)}.dark\:border-l-orange-950\/75:is(.dark *){border-left-color:rgba(67,20,7,.75)}.dark\:border-l-orange-950\/80:is(.dark *){border-left-color:rgba(67,20,7,.8)}.dark\:border-l-orange-950\/85:is(.dark *){border-left-color:rgba(67,20,7,.85)}.dark\:border-l-orange-950\/90:is(.dark *){border-left-color:rgba(67,20,7,.9)}.dark\:border-l-orange-950\/95:is(.dark *){border-left-color:rgba(67,20,7,.95)}.dark\:border-l-pink-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(252 231 243/var(--tw-border-opacity,1))}.dark\:border-l-pink-100\/0:is(.dark *){border-left-color:rgba(252,231,243,0)}.dark\:border-l-pink-100\/10:is(.dark *){border-left-color:rgba(252,231,243,.1)}.dark\:border-l-pink-100\/100:is(.dark *){border-left-color:#fce7f3}.dark\:border-l-pink-100\/15:is(.dark *){border-left-color:rgba(252,231,243,.15)}.dark\:border-l-pink-100\/20:is(.dark *){border-left-color:rgba(252,231,243,.2)}.dark\:border-l-pink-100\/25:is(.dark *){border-left-color:rgba(252,231,243,.25)}.dark\:border-l-pink-100\/30:is(.dark *){border-left-color:rgba(252,231,243,.3)}.dark\:border-l-pink-100\/35:is(.dark *){border-left-color:rgba(252,231,243,.35)}.dark\:border-l-pink-100\/40:is(.dark *){border-left-color:rgba(252,231,243,.4)}.dark\:border-l-pink-100\/45:is(.dark *){border-left-color:rgba(252,231,243,.45)}.dark\:border-l-pink-100\/5:is(.dark *){border-left-color:rgba(252,231,243,.05)}.dark\:border-l-pink-100\/50:is(.dark *){border-left-color:rgba(252,231,243,.5)}.dark\:border-l-pink-100\/55:is(.dark *){border-left-color:rgba(252,231,243,.55)}.dark\:border-l-pink-100\/60:is(.dark *){border-left-color:rgba(252,231,243,.6)}.dark\:border-l-pink-100\/65:is(.dark *){border-left-color:rgba(252,231,243,.65)}.dark\:border-l-pink-100\/70:is(.dark *){border-left-color:rgba(252,231,243,.7)}.dark\:border-l-pink-100\/75:is(.dark *){border-left-color:rgba(252,231,243,.75)}.dark\:border-l-pink-100\/80:is(.dark *){border-left-color:rgba(252,231,243,.8)}.dark\:border-l-pink-100\/85:is(.dark *){border-left-color:rgba(252,231,243,.85)}.dark\:border-l-pink-100\/90:is(.dark *){border-left-color:rgba(252,231,243,.9)}.dark\:border-l-pink-100\/95:is(.dark *){border-left-color:rgba(252,231,243,.95)}.dark\:border-l-pink-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(251 207 232/var(--tw-border-opacity,1))}.dark\:border-l-pink-200\/0:is(.dark *){border-left-color:rgba(251,207,232,0)}.dark\:border-l-pink-200\/10:is(.dark *){border-left-color:rgba(251,207,232,.1)}.dark\:border-l-pink-200\/100:is(.dark *){border-left-color:#fbcfe8}.dark\:border-l-pink-200\/15:is(.dark *){border-left-color:rgba(251,207,232,.15)}.dark\:border-l-pink-200\/20:is(.dark *){border-left-color:rgba(251,207,232,.2)}.dark\:border-l-pink-200\/25:is(.dark *){border-left-color:rgba(251,207,232,.25)}.dark\:border-l-pink-200\/30:is(.dark *){border-left-color:rgba(251,207,232,.3)}.dark\:border-l-pink-200\/35:is(.dark *){border-left-color:rgba(251,207,232,.35)}.dark\:border-l-pink-200\/40:is(.dark *){border-left-color:rgba(251,207,232,.4)}.dark\:border-l-pink-200\/45:is(.dark *){border-left-color:rgba(251,207,232,.45)}.dark\:border-l-pink-200\/5:is(.dark *){border-left-color:rgba(251,207,232,.05)}.dark\:border-l-pink-200\/50:is(.dark *){border-left-color:rgba(251,207,232,.5)}.dark\:border-l-pink-200\/55:is(.dark *){border-left-color:rgba(251,207,232,.55)}.dark\:border-l-pink-200\/60:is(.dark *){border-left-color:rgba(251,207,232,.6)}.dark\:border-l-pink-200\/65:is(.dark *){border-left-color:rgba(251,207,232,.65)}.dark\:border-l-pink-200\/70:is(.dark *){border-left-color:rgba(251,207,232,.7)}.dark\:border-l-pink-200\/75:is(.dark *){border-left-color:rgba(251,207,232,.75)}.dark\:border-l-pink-200\/80:is(.dark *){border-left-color:rgba(251,207,232,.8)}.dark\:border-l-pink-200\/85:is(.dark *){border-left-color:rgba(251,207,232,.85)}.dark\:border-l-pink-200\/90:is(.dark *){border-left-color:rgba(251,207,232,.9)}.dark\:border-l-pink-200\/95:is(.dark *){border-left-color:rgba(251,207,232,.95)}.dark\:border-l-pink-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(244 114 182/var(--tw-border-opacity,1))}.dark\:border-l-pink-400\/0:is(.dark *){border-left-color:rgba(244,114,182,0)}.dark\:border-l-pink-400\/10:is(.dark *){border-left-color:rgba(244,114,182,.1)}.dark\:border-l-pink-400\/100:is(.dark *){border-left-color:#f472b6}.dark\:border-l-pink-400\/15:is(.dark *){border-left-color:rgba(244,114,182,.15)}.dark\:border-l-pink-400\/20:is(.dark *){border-left-color:rgba(244,114,182,.2)}.dark\:border-l-pink-400\/25:is(.dark *){border-left-color:rgba(244,114,182,.25)}.dark\:border-l-pink-400\/30:is(.dark *){border-left-color:rgba(244,114,182,.3)}.dark\:border-l-pink-400\/35:is(.dark *){border-left-color:rgba(244,114,182,.35)}.dark\:border-l-pink-400\/40:is(.dark *){border-left-color:rgba(244,114,182,.4)}.dark\:border-l-pink-400\/45:is(.dark *){border-left-color:rgba(244,114,182,.45)}.dark\:border-l-pink-400\/5:is(.dark *){border-left-color:rgba(244,114,182,.05)}.dark\:border-l-pink-400\/50:is(.dark *){border-left-color:rgba(244,114,182,.5)}.dark\:border-l-pink-400\/55:is(.dark *){border-left-color:rgba(244,114,182,.55)}.dark\:border-l-pink-400\/60:is(.dark *){border-left-color:rgba(244,114,182,.6)}.dark\:border-l-pink-400\/65:is(.dark *){border-left-color:rgba(244,114,182,.65)}.dark\:border-l-pink-400\/70:is(.dark *){border-left-color:rgba(244,114,182,.7)}.dark\:border-l-pink-400\/75:is(.dark *){border-left-color:rgba(244,114,182,.75)}.dark\:border-l-pink-400\/80:is(.dark *){border-left-color:rgba(244,114,182,.8)}.dark\:border-l-pink-400\/85:is(.dark *){border-left-color:rgba(244,114,182,.85)}.dark\:border-l-pink-400\/90:is(.dark *){border-left-color:rgba(244,114,182,.9)}.dark\:border-l-pink-400\/95:is(.dark *){border-left-color:rgba(244,114,182,.95)}.dark\:border-l-pink-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(253 242 248/var(--tw-border-opacity,1))}.dark\:border-l-pink-50\/0:is(.dark *){border-left-color:rgba(253,242,248,0)}.dark\:border-l-pink-50\/10:is(.dark *){border-left-color:rgba(253,242,248,.1)}.dark\:border-l-pink-50\/100:is(.dark *){border-left-color:#fdf2f8}.dark\:border-l-pink-50\/15:is(.dark *){border-left-color:rgba(253,242,248,.15)}.dark\:border-l-pink-50\/20:is(.dark *){border-left-color:rgba(253,242,248,.2)}.dark\:border-l-pink-50\/25:is(.dark *){border-left-color:rgba(253,242,248,.25)}.dark\:border-l-pink-50\/30:is(.dark *){border-left-color:rgba(253,242,248,.3)}.dark\:border-l-pink-50\/35:is(.dark *){border-left-color:rgba(253,242,248,.35)}.dark\:border-l-pink-50\/40:is(.dark *){border-left-color:rgba(253,242,248,.4)}.dark\:border-l-pink-50\/45:is(.dark *){border-left-color:rgba(253,242,248,.45)}.dark\:border-l-pink-50\/5:is(.dark *){border-left-color:rgba(253,242,248,.05)}.dark\:border-l-pink-50\/50:is(.dark *){border-left-color:rgba(253,242,248,.5)}.dark\:border-l-pink-50\/55:is(.dark *){border-left-color:rgba(253,242,248,.55)}.dark\:border-l-pink-50\/60:is(.dark *){border-left-color:rgba(253,242,248,.6)}.dark\:border-l-pink-50\/65:is(.dark *){border-left-color:rgba(253,242,248,.65)}.dark\:border-l-pink-50\/70:is(.dark *){border-left-color:rgba(253,242,248,.7)}.dark\:border-l-pink-50\/75:is(.dark *){border-left-color:rgba(253,242,248,.75)}.dark\:border-l-pink-50\/80:is(.dark *){border-left-color:rgba(253,242,248,.8)}.dark\:border-l-pink-50\/85:is(.dark *){border-left-color:rgba(253,242,248,.85)}.dark\:border-l-pink-50\/90:is(.dark *){border-left-color:rgba(253,242,248,.9)}.dark\:border-l-pink-50\/95:is(.dark *){border-left-color:rgba(253,242,248,.95)}.dark\:border-l-pink-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(236 72 153/var(--tw-border-opacity,1))}.dark\:border-l-pink-500\/0:is(.dark *){border-left-color:rgba(236,72,153,0)}.dark\:border-l-pink-500\/10:is(.dark *){border-left-color:rgba(236,72,153,.1)}.dark\:border-l-pink-500\/100:is(.dark *){border-left-color:#ec4899}.dark\:border-l-pink-500\/15:is(.dark *){border-left-color:rgba(236,72,153,.15)}.dark\:border-l-pink-500\/20:is(.dark *){border-left-color:rgba(236,72,153,.2)}.dark\:border-l-pink-500\/25:is(.dark *){border-left-color:rgba(236,72,153,.25)}.dark\:border-l-pink-500\/30:is(.dark *){border-left-color:rgba(236,72,153,.3)}.dark\:border-l-pink-500\/35:is(.dark *){border-left-color:rgba(236,72,153,.35)}.dark\:border-l-pink-500\/40:is(.dark *){border-left-color:rgba(236,72,153,.4)}.dark\:border-l-pink-500\/45:is(.dark *){border-left-color:rgba(236,72,153,.45)}.dark\:border-l-pink-500\/5:is(.dark *){border-left-color:rgba(236,72,153,.05)}.dark\:border-l-pink-500\/50:is(.dark *){border-left-color:rgba(236,72,153,.5)}.dark\:border-l-pink-500\/55:is(.dark *){border-left-color:rgba(236,72,153,.55)}.dark\:border-l-pink-500\/60:is(.dark *){border-left-color:rgba(236,72,153,.6)}.dark\:border-l-pink-500\/65:is(.dark *){border-left-color:rgba(236,72,153,.65)}.dark\:border-l-pink-500\/70:is(.dark *){border-left-color:rgba(236,72,153,.7)}.dark\:border-l-pink-500\/75:is(.dark *){border-left-color:rgba(236,72,153,.75)}.dark\:border-l-pink-500\/80:is(.dark *){border-left-color:rgba(236,72,153,.8)}.dark\:border-l-pink-500\/85:is(.dark *){border-left-color:rgba(236,72,153,.85)}.dark\:border-l-pink-500\/90:is(.dark *){border-left-color:rgba(236,72,153,.9)}.dark\:border-l-pink-500\/95:is(.dark *){border-left-color:rgba(236,72,153,.95)}.dark\:border-l-pink-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(80 7 36/var(--tw-border-opacity,1))}.dark\:border-l-pink-950\/0:is(.dark *){border-left-color:rgba(80,7,36,0)}.dark\:border-l-pink-950\/10:is(.dark *){border-left-color:rgba(80,7,36,.1)}.dark\:border-l-pink-950\/100:is(.dark *){border-left-color:#500724}.dark\:border-l-pink-950\/15:is(.dark *){border-left-color:rgba(80,7,36,.15)}.dark\:border-l-pink-950\/20:is(.dark *){border-left-color:rgba(80,7,36,.2)}.dark\:border-l-pink-950\/25:is(.dark *){border-left-color:rgba(80,7,36,.25)}.dark\:border-l-pink-950\/30:is(.dark *){border-left-color:rgba(80,7,36,.3)}.dark\:border-l-pink-950\/35:is(.dark *){border-left-color:rgba(80,7,36,.35)}.dark\:border-l-pink-950\/40:is(.dark *){border-left-color:rgba(80,7,36,.4)}.dark\:border-l-pink-950\/45:is(.dark *){border-left-color:rgba(80,7,36,.45)}.dark\:border-l-pink-950\/5:is(.dark *){border-left-color:rgba(80,7,36,.05)}.dark\:border-l-pink-950\/50:is(.dark *){border-left-color:rgba(80,7,36,.5)}.dark\:border-l-pink-950\/55:is(.dark *){border-left-color:rgba(80,7,36,.55)}.dark\:border-l-pink-950\/60:is(.dark *){border-left-color:rgba(80,7,36,.6)}.dark\:border-l-pink-950\/65:is(.dark *){border-left-color:rgba(80,7,36,.65)}.dark\:border-l-pink-950\/70:is(.dark *){border-left-color:rgba(80,7,36,.7)}.dark\:border-l-pink-950\/75:is(.dark *){border-left-color:rgba(80,7,36,.75)}.dark\:border-l-pink-950\/80:is(.dark *){border-left-color:rgba(80,7,36,.8)}.dark\:border-l-pink-950\/85:is(.dark *){border-left-color:rgba(80,7,36,.85)}.dark\:border-l-pink-950\/90:is(.dark *){border-left-color:rgba(80,7,36,.9)}.dark\:border-l-pink-950\/95:is(.dark *){border-left-color:rgba(80,7,36,.95)}.dark\:border-l-purple-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(243 232 255/var(--tw-border-opacity,1))}.dark\:border-l-purple-100\/0:is(.dark *){border-left-color:rgba(243,232,255,0)}.dark\:border-l-purple-100\/10:is(.dark *){border-left-color:rgba(243,232,255,.1)}.dark\:border-l-purple-100\/100:is(.dark *){border-left-color:#f3e8ff}.dark\:border-l-purple-100\/15:is(.dark *){border-left-color:rgba(243,232,255,.15)}.dark\:border-l-purple-100\/20:is(.dark *){border-left-color:rgba(243,232,255,.2)}.dark\:border-l-purple-100\/25:is(.dark *){border-left-color:rgba(243,232,255,.25)}.dark\:border-l-purple-100\/30:is(.dark *){border-left-color:rgba(243,232,255,.3)}.dark\:border-l-purple-100\/35:is(.dark *){border-left-color:rgba(243,232,255,.35)}.dark\:border-l-purple-100\/40:is(.dark *){border-left-color:rgba(243,232,255,.4)}.dark\:border-l-purple-100\/45:is(.dark *){border-left-color:rgba(243,232,255,.45)}.dark\:border-l-purple-100\/5:is(.dark *){border-left-color:rgba(243,232,255,.05)}.dark\:border-l-purple-100\/50:is(.dark *){border-left-color:rgba(243,232,255,.5)}.dark\:border-l-purple-100\/55:is(.dark *){border-left-color:rgba(243,232,255,.55)}.dark\:border-l-purple-100\/60:is(.dark *){border-left-color:rgba(243,232,255,.6)}.dark\:border-l-purple-100\/65:is(.dark *){border-left-color:rgba(243,232,255,.65)}.dark\:border-l-purple-100\/70:is(.dark *){border-left-color:rgba(243,232,255,.7)}.dark\:border-l-purple-100\/75:is(.dark *){border-left-color:rgba(243,232,255,.75)}.dark\:border-l-purple-100\/80:is(.dark *){border-left-color:rgba(243,232,255,.8)}.dark\:border-l-purple-100\/85:is(.dark *){border-left-color:rgba(243,232,255,.85)}.dark\:border-l-purple-100\/90:is(.dark *){border-left-color:rgba(243,232,255,.9)}.dark\:border-l-purple-100\/95:is(.dark *){border-left-color:rgba(243,232,255,.95)}.dark\:border-l-purple-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(233 213 255/var(--tw-border-opacity,1))}.dark\:border-l-purple-200\/0:is(.dark *){border-left-color:rgba(233,213,255,0)}.dark\:border-l-purple-200\/10:is(.dark *){border-left-color:rgba(233,213,255,.1)}.dark\:border-l-purple-200\/100:is(.dark *){border-left-color:#e9d5ff}.dark\:border-l-purple-200\/15:is(.dark *){border-left-color:rgba(233,213,255,.15)}.dark\:border-l-purple-200\/20:is(.dark *){border-left-color:rgba(233,213,255,.2)}.dark\:border-l-purple-200\/25:is(.dark *){border-left-color:rgba(233,213,255,.25)}.dark\:border-l-purple-200\/30:is(.dark *){border-left-color:rgba(233,213,255,.3)}.dark\:border-l-purple-200\/35:is(.dark *){border-left-color:rgba(233,213,255,.35)}.dark\:border-l-purple-200\/40:is(.dark *){border-left-color:rgba(233,213,255,.4)}.dark\:border-l-purple-200\/45:is(.dark *){border-left-color:rgba(233,213,255,.45)}.dark\:border-l-purple-200\/5:is(.dark *){border-left-color:rgba(233,213,255,.05)}.dark\:border-l-purple-200\/50:is(.dark *){border-left-color:rgba(233,213,255,.5)}.dark\:border-l-purple-200\/55:is(.dark *){border-left-color:rgba(233,213,255,.55)}.dark\:border-l-purple-200\/60:is(.dark *){border-left-color:rgba(233,213,255,.6)}.dark\:border-l-purple-200\/65:is(.dark *){border-left-color:rgba(233,213,255,.65)}.dark\:border-l-purple-200\/70:is(.dark *){border-left-color:rgba(233,213,255,.7)}.dark\:border-l-purple-200\/75:is(.dark *){border-left-color:rgba(233,213,255,.75)}.dark\:border-l-purple-200\/80:is(.dark *){border-left-color:rgba(233,213,255,.8)}.dark\:border-l-purple-200\/85:is(.dark *){border-left-color:rgba(233,213,255,.85)}.dark\:border-l-purple-200\/90:is(.dark *){border-left-color:rgba(233,213,255,.9)}.dark\:border-l-purple-200\/95:is(.dark *){border-left-color:rgba(233,213,255,.95)}.dark\:border-l-purple-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(192 132 252/var(--tw-border-opacity,1))}.dark\:border-l-purple-400\/0:is(.dark *){border-left-color:rgba(192,132,252,0)}.dark\:border-l-purple-400\/10:is(.dark *){border-left-color:rgba(192,132,252,.1)}.dark\:border-l-purple-400\/100:is(.dark *){border-left-color:#c084fc}.dark\:border-l-purple-400\/15:is(.dark *){border-left-color:rgba(192,132,252,.15)}.dark\:border-l-purple-400\/20:is(.dark *){border-left-color:rgba(192,132,252,.2)}.dark\:border-l-purple-400\/25:is(.dark *){border-left-color:rgba(192,132,252,.25)}.dark\:border-l-purple-400\/30:is(.dark *){border-left-color:rgba(192,132,252,.3)}.dark\:border-l-purple-400\/35:is(.dark *){border-left-color:rgba(192,132,252,.35)}.dark\:border-l-purple-400\/40:is(.dark *){border-left-color:rgba(192,132,252,.4)}.dark\:border-l-purple-400\/45:is(.dark *){border-left-color:rgba(192,132,252,.45)}.dark\:border-l-purple-400\/5:is(.dark *){border-left-color:rgba(192,132,252,.05)}.dark\:border-l-purple-400\/50:is(.dark *){border-left-color:rgba(192,132,252,.5)}.dark\:border-l-purple-400\/55:is(.dark *){border-left-color:rgba(192,132,252,.55)}.dark\:border-l-purple-400\/60:is(.dark *){border-left-color:rgba(192,132,252,.6)}.dark\:border-l-purple-400\/65:is(.dark *){border-left-color:rgba(192,132,252,.65)}.dark\:border-l-purple-400\/70:is(.dark *){border-left-color:rgba(192,132,252,.7)}.dark\:border-l-purple-400\/75:is(.dark *){border-left-color:rgba(192,132,252,.75)}.dark\:border-l-purple-400\/80:is(.dark *){border-left-color:rgba(192,132,252,.8)}.dark\:border-l-purple-400\/85:is(.dark *){border-left-color:rgba(192,132,252,.85)}.dark\:border-l-purple-400\/90:is(.dark *){border-left-color:rgba(192,132,252,.9)}.dark\:border-l-purple-400\/95:is(.dark *){border-left-color:rgba(192,132,252,.95)}.dark\:border-l-purple-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(250 245 255/var(--tw-border-opacity,1))}.dark\:border-l-purple-50\/0:is(.dark *){border-left-color:rgba(250,245,255,0)}.dark\:border-l-purple-50\/10:is(.dark *){border-left-color:rgba(250,245,255,.1)}.dark\:border-l-purple-50\/100:is(.dark *){border-left-color:#faf5ff}.dark\:border-l-purple-50\/15:is(.dark *){border-left-color:rgba(250,245,255,.15)}.dark\:border-l-purple-50\/20:is(.dark *){border-left-color:rgba(250,245,255,.2)}.dark\:border-l-purple-50\/25:is(.dark *){border-left-color:rgba(250,245,255,.25)}.dark\:border-l-purple-50\/30:is(.dark *){border-left-color:rgba(250,245,255,.3)}.dark\:border-l-purple-50\/35:is(.dark *){border-left-color:rgba(250,245,255,.35)}.dark\:border-l-purple-50\/40:is(.dark *){border-left-color:rgba(250,245,255,.4)}.dark\:border-l-purple-50\/45:is(.dark *){border-left-color:rgba(250,245,255,.45)}.dark\:border-l-purple-50\/5:is(.dark *){border-left-color:rgba(250,245,255,.05)}.dark\:border-l-purple-50\/50:is(.dark *){border-left-color:rgba(250,245,255,.5)}.dark\:border-l-purple-50\/55:is(.dark *){border-left-color:rgba(250,245,255,.55)}.dark\:border-l-purple-50\/60:is(.dark *){border-left-color:rgba(250,245,255,.6)}.dark\:border-l-purple-50\/65:is(.dark *){border-left-color:rgba(250,245,255,.65)}.dark\:border-l-purple-50\/70:is(.dark *){border-left-color:rgba(250,245,255,.7)}.dark\:border-l-purple-50\/75:is(.dark *){border-left-color:rgba(250,245,255,.75)}.dark\:border-l-purple-50\/80:is(.dark *){border-left-color:rgba(250,245,255,.8)}.dark\:border-l-purple-50\/85:is(.dark *){border-left-color:rgba(250,245,255,.85)}.dark\:border-l-purple-50\/90:is(.dark *){border-left-color:rgba(250,245,255,.9)}.dark\:border-l-purple-50\/95:is(.dark *){border-left-color:rgba(250,245,255,.95)}.dark\:border-l-purple-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(168 85 247/var(--tw-border-opacity,1))}.dark\:border-l-purple-500\/0:is(.dark *){border-left-color:rgba(168,85,247,0)}.dark\:border-l-purple-500\/10:is(.dark *){border-left-color:rgba(168,85,247,.1)}.dark\:border-l-purple-500\/100:is(.dark *){border-left-color:#a855f7}.dark\:border-l-purple-500\/15:is(.dark *){border-left-color:rgba(168,85,247,.15)}.dark\:border-l-purple-500\/20:is(.dark *){border-left-color:rgba(168,85,247,.2)}.dark\:border-l-purple-500\/25:is(.dark *){border-left-color:rgba(168,85,247,.25)}.dark\:border-l-purple-500\/30:is(.dark *){border-left-color:rgba(168,85,247,.3)}.dark\:border-l-purple-500\/35:is(.dark *){border-left-color:rgba(168,85,247,.35)}.dark\:border-l-purple-500\/40:is(.dark *){border-left-color:rgba(168,85,247,.4)}.dark\:border-l-purple-500\/45:is(.dark *){border-left-color:rgba(168,85,247,.45)}.dark\:border-l-purple-500\/5:is(.dark *){border-left-color:rgba(168,85,247,.05)}.dark\:border-l-purple-500\/50:is(.dark *){border-left-color:rgba(168,85,247,.5)}.dark\:border-l-purple-500\/55:is(.dark *){border-left-color:rgba(168,85,247,.55)}.dark\:border-l-purple-500\/60:is(.dark *){border-left-color:rgba(168,85,247,.6)}.dark\:border-l-purple-500\/65:is(.dark *){border-left-color:rgba(168,85,247,.65)}.dark\:border-l-purple-500\/70:is(.dark *){border-left-color:rgba(168,85,247,.7)}.dark\:border-l-purple-500\/75:is(.dark *){border-left-color:rgba(168,85,247,.75)}.dark\:border-l-purple-500\/80:is(.dark *){border-left-color:rgba(168,85,247,.8)}.dark\:border-l-purple-500\/85:is(.dark *){border-left-color:rgba(168,85,247,.85)}.dark\:border-l-purple-500\/90:is(.dark *){border-left-color:rgba(168,85,247,.9)}.dark\:border-l-purple-500\/95:is(.dark *){border-left-color:rgba(168,85,247,.95)}.dark\:border-l-purple-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(59 7 100/var(--tw-border-opacity,1))}.dark\:border-l-purple-950\/0:is(.dark *){border-left-color:rgba(59,7,100,0)}.dark\:border-l-purple-950\/10:is(.dark *){border-left-color:rgba(59,7,100,.1)}.dark\:border-l-purple-950\/100:is(.dark *){border-left-color:#3b0764}.dark\:border-l-purple-950\/15:is(.dark *){border-left-color:rgba(59,7,100,.15)}.dark\:border-l-purple-950\/20:is(.dark *){border-left-color:rgba(59,7,100,.2)}.dark\:border-l-purple-950\/25:is(.dark *){border-left-color:rgba(59,7,100,.25)}.dark\:border-l-purple-950\/30:is(.dark *){border-left-color:rgba(59,7,100,.3)}.dark\:border-l-purple-950\/35:is(.dark *){border-left-color:rgba(59,7,100,.35)}.dark\:border-l-purple-950\/40:is(.dark *){border-left-color:rgba(59,7,100,.4)}.dark\:border-l-purple-950\/45:is(.dark *){border-left-color:rgba(59,7,100,.45)}.dark\:border-l-purple-950\/5:is(.dark *){border-left-color:rgba(59,7,100,.05)}.dark\:border-l-purple-950\/50:is(.dark *){border-left-color:rgba(59,7,100,.5)}.dark\:border-l-purple-950\/55:is(.dark *){border-left-color:rgba(59,7,100,.55)}.dark\:border-l-purple-950\/60:is(.dark *){border-left-color:rgba(59,7,100,.6)}.dark\:border-l-purple-950\/65:is(.dark *){border-left-color:rgba(59,7,100,.65)}.dark\:border-l-purple-950\/70:is(.dark *){border-left-color:rgba(59,7,100,.7)}.dark\:border-l-purple-950\/75:is(.dark *){border-left-color:rgba(59,7,100,.75)}.dark\:border-l-purple-950\/80:is(.dark *){border-left-color:rgba(59,7,100,.8)}.dark\:border-l-purple-950\/85:is(.dark *){border-left-color:rgba(59,7,100,.85)}.dark\:border-l-purple-950\/90:is(.dark *){border-left-color:rgba(59,7,100,.9)}.dark\:border-l-purple-950\/95:is(.dark *){border-left-color:rgba(59,7,100,.95)}.dark\:border-l-red-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(254 226 226/var(--tw-border-opacity,1))}.dark\:border-l-red-100\/0:is(.dark *){border-left-color:hsla(0,93%,94%,0)}.dark\:border-l-red-100\/10:is(.dark *){border-left-color:hsla(0,93%,94%,.1)}.dark\:border-l-red-100\/100:is(.dark *){border-left-color:#fee2e2}.dark\:border-l-red-100\/15:is(.dark *){border-left-color:hsla(0,93%,94%,.15)}.dark\:border-l-red-100\/20:is(.dark *){border-left-color:hsla(0,93%,94%,.2)}.dark\:border-l-red-100\/25:is(.dark *){border-left-color:hsla(0,93%,94%,.25)}.dark\:border-l-red-100\/30:is(.dark *){border-left-color:hsla(0,93%,94%,.3)}.dark\:border-l-red-100\/35:is(.dark *){border-left-color:hsla(0,93%,94%,.35)}.dark\:border-l-red-100\/40:is(.dark *){border-left-color:hsla(0,93%,94%,.4)}.dark\:border-l-red-100\/45:is(.dark *){border-left-color:hsla(0,93%,94%,.45)}.dark\:border-l-red-100\/5:is(.dark *){border-left-color:hsla(0,93%,94%,.05)}.dark\:border-l-red-100\/50:is(.dark *){border-left-color:hsla(0,93%,94%,.5)}.dark\:border-l-red-100\/55:is(.dark *){border-left-color:hsla(0,93%,94%,.55)}.dark\:border-l-red-100\/60:is(.dark *){border-left-color:hsla(0,93%,94%,.6)}.dark\:border-l-red-100\/65:is(.dark *){border-left-color:hsla(0,93%,94%,.65)}.dark\:border-l-red-100\/70:is(.dark *){border-left-color:hsla(0,93%,94%,.7)}.dark\:border-l-red-100\/75:is(.dark *){border-left-color:hsla(0,93%,94%,.75)}.dark\:border-l-red-100\/80:is(.dark *){border-left-color:hsla(0,93%,94%,.8)}.dark\:border-l-red-100\/85:is(.dark *){border-left-color:hsla(0,93%,94%,.85)}.dark\:border-l-red-100\/90:is(.dark *){border-left-color:hsla(0,93%,94%,.9)}.dark\:border-l-red-100\/95:is(.dark *){border-left-color:hsla(0,93%,94%,.95)}.dark\:border-l-red-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(254 202 202/var(--tw-border-opacity,1))}.dark\:border-l-red-200\/0:is(.dark *){border-left-color:hsla(0,96%,89%,0)}.dark\:border-l-red-200\/10:is(.dark *){border-left-color:hsla(0,96%,89%,.1)}.dark\:border-l-red-200\/100:is(.dark *){border-left-color:#fecaca}.dark\:border-l-red-200\/15:is(.dark *){border-left-color:hsla(0,96%,89%,.15)}.dark\:border-l-red-200\/20:is(.dark *){border-left-color:hsla(0,96%,89%,.2)}.dark\:border-l-red-200\/25:is(.dark *){border-left-color:hsla(0,96%,89%,.25)}.dark\:border-l-red-200\/30:is(.dark *){border-left-color:hsla(0,96%,89%,.3)}.dark\:border-l-red-200\/35:is(.dark *){border-left-color:hsla(0,96%,89%,.35)}.dark\:border-l-red-200\/40:is(.dark *){border-left-color:hsla(0,96%,89%,.4)}.dark\:border-l-red-200\/45:is(.dark *){border-left-color:hsla(0,96%,89%,.45)}.dark\:border-l-red-200\/5:is(.dark *){border-left-color:hsla(0,96%,89%,.05)}.dark\:border-l-red-200\/50:is(.dark *){border-left-color:hsla(0,96%,89%,.5)}.dark\:border-l-red-200\/55:is(.dark *){border-left-color:hsla(0,96%,89%,.55)}.dark\:border-l-red-200\/60:is(.dark *){border-left-color:hsla(0,96%,89%,.6)}.dark\:border-l-red-200\/65:is(.dark *){border-left-color:hsla(0,96%,89%,.65)}.dark\:border-l-red-200\/70:is(.dark *){border-left-color:hsla(0,96%,89%,.7)}.dark\:border-l-red-200\/75:is(.dark *){border-left-color:hsla(0,96%,89%,.75)}.dark\:border-l-red-200\/80:is(.dark *){border-left-color:hsla(0,96%,89%,.8)}.dark\:border-l-red-200\/85:is(.dark *){border-left-color:hsla(0,96%,89%,.85)}.dark\:border-l-red-200\/90:is(.dark *){border-left-color:hsla(0,96%,89%,.9)}.dark\:border-l-red-200\/95:is(.dark *){border-left-color:hsla(0,96%,89%,.95)}.dark\:border-l-red-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(248 113 113/var(--tw-border-opacity,1))}.dark\:border-l-red-400\/0:is(.dark *){border-left-color:hsla(0,91%,71%,0)}.dark\:border-l-red-400\/10:is(.dark *){border-left-color:hsla(0,91%,71%,.1)}.dark\:border-l-red-400\/100:is(.dark *){border-left-color:#f87171}.dark\:border-l-red-400\/15:is(.dark *){border-left-color:hsla(0,91%,71%,.15)}.dark\:border-l-red-400\/20:is(.dark *){border-left-color:hsla(0,91%,71%,.2)}.dark\:border-l-red-400\/25:is(.dark *){border-left-color:hsla(0,91%,71%,.25)}.dark\:border-l-red-400\/30:is(.dark *){border-left-color:hsla(0,91%,71%,.3)}.dark\:border-l-red-400\/35:is(.dark *){border-left-color:hsla(0,91%,71%,.35)}.dark\:border-l-red-400\/40:is(.dark *){border-left-color:hsla(0,91%,71%,.4)}.dark\:border-l-red-400\/45:is(.dark *){border-left-color:hsla(0,91%,71%,.45)}.dark\:border-l-red-400\/5:is(.dark *){border-left-color:hsla(0,91%,71%,.05)}.dark\:border-l-red-400\/50:is(.dark *){border-left-color:hsla(0,91%,71%,.5)}.dark\:border-l-red-400\/55:is(.dark *){border-left-color:hsla(0,91%,71%,.55)}.dark\:border-l-red-400\/60:is(.dark *){border-left-color:hsla(0,91%,71%,.6)}.dark\:border-l-red-400\/65:is(.dark *){border-left-color:hsla(0,91%,71%,.65)}.dark\:border-l-red-400\/70:is(.dark *){border-left-color:hsla(0,91%,71%,.7)}.dark\:border-l-red-400\/75:is(.dark *){border-left-color:hsla(0,91%,71%,.75)}.dark\:border-l-red-400\/80:is(.dark *){border-left-color:hsla(0,91%,71%,.8)}.dark\:border-l-red-400\/85:is(.dark *){border-left-color:hsla(0,91%,71%,.85)}.dark\:border-l-red-400\/90:is(.dark *){border-left-color:hsla(0,91%,71%,.9)}.dark\:border-l-red-400\/95:is(.dark *){border-left-color:hsla(0,91%,71%,.95)}.dark\:border-l-red-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(254 242 242/var(--tw-border-opacity,1))}.dark\:border-l-red-50\/0:is(.dark *){border-left-color:hsla(0,86%,97%,0)}.dark\:border-l-red-50\/10:is(.dark *){border-left-color:hsla(0,86%,97%,.1)}.dark\:border-l-red-50\/100:is(.dark *){border-left-color:#fef2f2}.dark\:border-l-red-50\/15:is(.dark *){border-left-color:hsla(0,86%,97%,.15)}.dark\:border-l-red-50\/20:is(.dark *){border-left-color:hsla(0,86%,97%,.2)}.dark\:border-l-red-50\/25:is(.dark *){border-left-color:hsla(0,86%,97%,.25)}.dark\:border-l-red-50\/30:is(.dark *){border-left-color:hsla(0,86%,97%,.3)}.dark\:border-l-red-50\/35:is(.dark *){border-left-color:hsla(0,86%,97%,.35)}.dark\:border-l-red-50\/40:is(.dark *){border-left-color:hsla(0,86%,97%,.4)}.dark\:border-l-red-50\/45:is(.dark *){border-left-color:hsla(0,86%,97%,.45)}.dark\:border-l-red-50\/5:is(.dark *){border-left-color:hsla(0,86%,97%,.05)}.dark\:border-l-red-50\/50:is(.dark *){border-left-color:hsla(0,86%,97%,.5)}.dark\:border-l-red-50\/55:is(.dark *){border-left-color:hsla(0,86%,97%,.55)}.dark\:border-l-red-50\/60:is(.dark *){border-left-color:hsla(0,86%,97%,.6)}.dark\:border-l-red-50\/65:is(.dark *){border-left-color:hsla(0,86%,97%,.65)}.dark\:border-l-red-50\/70:is(.dark *){border-left-color:hsla(0,86%,97%,.7)}.dark\:border-l-red-50\/75:is(.dark *){border-left-color:hsla(0,86%,97%,.75)}.dark\:border-l-red-50\/80:is(.dark *){border-left-color:hsla(0,86%,97%,.8)}.dark\:border-l-red-50\/85:is(.dark *){border-left-color:hsla(0,86%,97%,.85)}.dark\:border-l-red-50\/90:is(.dark *){border-left-color:hsla(0,86%,97%,.9)}.dark\:border-l-red-50\/95:is(.dark *){border-left-color:hsla(0,86%,97%,.95)}.dark\:border-l-red-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(239 68 68/var(--tw-border-opacity,1))}.dark\:border-l-red-500\/0:is(.dark *){border-left-color:rgba(239,68,68,0)}.dark\:border-l-red-500\/10:is(.dark *){border-left-color:rgba(239,68,68,.1)}.dark\:border-l-red-500\/100:is(.dark *){border-left-color:#ef4444}.dark\:border-l-red-500\/15:is(.dark *){border-left-color:rgba(239,68,68,.15)}.dark\:border-l-red-500\/20:is(.dark *){border-left-color:rgba(239,68,68,.2)}.dark\:border-l-red-500\/25:is(.dark *){border-left-color:rgba(239,68,68,.25)}.dark\:border-l-red-500\/30:is(.dark *){border-left-color:rgba(239,68,68,.3)}.dark\:border-l-red-500\/35:is(.dark *){border-left-color:rgba(239,68,68,.35)}.dark\:border-l-red-500\/40:is(.dark *){border-left-color:rgba(239,68,68,.4)}.dark\:border-l-red-500\/45:is(.dark *){border-left-color:rgba(239,68,68,.45)}.dark\:border-l-red-500\/5:is(.dark *){border-left-color:rgba(239,68,68,.05)}.dark\:border-l-red-500\/50:is(.dark *){border-left-color:rgba(239,68,68,.5)}.dark\:border-l-red-500\/55:is(.dark *){border-left-color:rgba(239,68,68,.55)}.dark\:border-l-red-500\/60:is(.dark *){border-left-color:rgba(239,68,68,.6)}.dark\:border-l-red-500\/65:is(.dark *){border-left-color:rgba(239,68,68,.65)}.dark\:border-l-red-500\/70:is(.dark *){border-left-color:rgba(239,68,68,.7)}.dark\:border-l-red-500\/75:is(.dark *){border-left-color:rgba(239,68,68,.75)}.dark\:border-l-red-500\/80:is(.dark *){border-left-color:rgba(239,68,68,.8)}.dark\:border-l-red-500\/85:is(.dark *){border-left-color:rgba(239,68,68,.85)}.dark\:border-l-red-500\/90:is(.dark *){border-left-color:rgba(239,68,68,.9)}.dark\:border-l-red-500\/95:is(.dark *){border-left-color:rgba(239,68,68,.95)}.dark\:border-l-red-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(69 10 10/var(--tw-border-opacity,1))}.dark\:border-l-red-950\/0:is(.dark *){border-left-color:rgba(69,10,10,0)}.dark\:border-l-red-950\/10:is(.dark *){border-left-color:rgba(69,10,10,.1)}.dark\:border-l-red-950\/100:is(.dark *){border-left-color:#450a0a}.dark\:border-l-red-950\/15:is(.dark *){border-left-color:rgba(69,10,10,.15)}.dark\:border-l-red-950\/20:is(.dark *){border-left-color:rgba(69,10,10,.2)}.dark\:border-l-red-950\/25:is(.dark *){border-left-color:rgba(69,10,10,.25)}.dark\:border-l-red-950\/30:is(.dark *){border-left-color:rgba(69,10,10,.3)}.dark\:border-l-red-950\/35:is(.dark *){border-left-color:rgba(69,10,10,.35)}.dark\:border-l-red-950\/40:is(.dark *){border-left-color:rgba(69,10,10,.4)}.dark\:border-l-red-950\/45:is(.dark *){border-left-color:rgba(69,10,10,.45)}.dark\:border-l-red-950\/5:is(.dark *){border-left-color:rgba(69,10,10,.05)}.dark\:border-l-red-950\/50:is(.dark *){border-left-color:rgba(69,10,10,.5)}.dark\:border-l-red-950\/55:is(.dark *){border-left-color:rgba(69,10,10,.55)}.dark\:border-l-red-950\/60:is(.dark *){border-left-color:rgba(69,10,10,.6)}.dark\:border-l-red-950\/65:is(.dark *){border-left-color:rgba(69,10,10,.65)}.dark\:border-l-red-950\/70:is(.dark *){border-left-color:rgba(69,10,10,.7)}.dark\:border-l-red-950\/75:is(.dark *){border-left-color:rgba(69,10,10,.75)}.dark\:border-l-red-950\/80:is(.dark *){border-left-color:rgba(69,10,10,.8)}.dark\:border-l-red-950\/85:is(.dark *){border-left-color:rgba(69,10,10,.85)}.dark\:border-l-red-950\/90:is(.dark *){border-left-color:rgba(69,10,10,.9)}.dark\:border-l-red-950\/95:is(.dark *){border-left-color:rgba(69,10,10,.95)}.dark\:border-l-rose-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(255 228 230/var(--tw-border-opacity,1))}.dark\:border-l-rose-100\/0:is(.dark *){border-left-color:rgba(255,228,230,0)}.dark\:border-l-rose-100\/10:is(.dark *){border-left-color:rgba(255,228,230,.1)}.dark\:border-l-rose-100\/100:is(.dark *){border-left-color:#ffe4e6}.dark\:border-l-rose-100\/15:is(.dark *){border-left-color:rgba(255,228,230,.15)}.dark\:border-l-rose-100\/20:is(.dark *){border-left-color:rgba(255,228,230,.2)}.dark\:border-l-rose-100\/25:is(.dark *){border-left-color:rgba(255,228,230,.25)}.dark\:border-l-rose-100\/30:is(.dark *){border-left-color:rgba(255,228,230,.3)}.dark\:border-l-rose-100\/35:is(.dark *){border-left-color:rgba(255,228,230,.35)}.dark\:border-l-rose-100\/40:is(.dark *){border-left-color:rgba(255,228,230,.4)}.dark\:border-l-rose-100\/45:is(.dark *){border-left-color:rgba(255,228,230,.45)}.dark\:border-l-rose-100\/5:is(.dark *){border-left-color:rgba(255,228,230,.05)}.dark\:border-l-rose-100\/50:is(.dark *){border-left-color:rgba(255,228,230,.5)}.dark\:border-l-rose-100\/55:is(.dark *){border-left-color:rgba(255,228,230,.55)}.dark\:border-l-rose-100\/60:is(.dark *){border-left-color:rgba(255,228,230,.6)}.dark\:border-l-rose-100\/65:is(.dark *){border-left-color:rgba(255,228,230,.65)}.dark\:border-l-rose-100\/70:is(.dark *){border-left-color:rgba(255,228,230,.7)}.dark\:border-l-rose-100\/75:is(.dark *){border-left-color:rgba(255,228,230,.75)}.dark\:border-l-rose-100\/80:is(.dark *){border-left-color:rgba(255,228,230,.8)}.dark\:border-l-rose-100\/85:is(.dark *){border-left-color:rgba(255,228,230,.85)}.dark\:border-l-rose-100\/90:is(.dark *){border-left-color:rgba(255,228,230,.9)}.dark\:border-l-rose-100\/95:is(.dark *){border-left-color:rgba(255,228,230,.95)}.dark\:border-l-rose-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(254 205 211/var(--tw-border-opacity,1))}.dark\:border-l-rose-200\/0:is(.dark *){border-left-color:rgba(254,205,211,0)}.dark\:border-l-rose-200\/10:is(.dark *){border-left-color:rgba(254,205,211,.1)}.dark\:border-l-rose-200\/100:is(.dark *){border-left-color:#fecdd3}.dark\:border-l-rose-200\/15:is(.dark *){border-left-color:rgba(254,205,211,.15)}.dark\:border-l-rose-200\/20:is(.dark *){border-left-color:rgba(254,205,211,.2)}.dark\:border-l-rose-200\/25:is(.dark *){border-left-color:rgba(254,205,211,.25)}.dark\:border-l-rose-200\/30:is(.dark *){border-left-color:rgba(254,205,211,.3)}.dark\:border-l-rose-200\/35:is(.dark *){border-left-color:rgba(254,205,211,.35)}.dark\:border-l-rose-200\/40:is(.dark *){border-left-color:rgba(254,205,211,.4)}.dark\:border-l-rose-200\/45:is(.dark *){border-left-color:rgba(254,205,211,.45)}.dark\:border-l-rose-200\/5:is(.dark *){border-left-color:rgba(254,205,211,.05)}.dark\:border-l-rose-200\/50:is(.dark *){border-left-color:rgba(254,205,211,.5)}.dark\:border-l-rose-200\/55:is(.dark *){border-left-color:rgba(254,205,211,.55)}.dark\:border-l-rose-200\/60:is(.dark *){border-left-color:rgba(254,205,211,.6)}.dark\:border-l-rose-200\/65:is(.dark *){border-left-color:rgba(254,205,211,.65)}.dark\:border-l-rose-200\/70:is(.dark *){border-left-color:rgba(254,205,211,.7)}.dark\:border-l-rose-200\/75:is(.dark *){border-left-color:rgba(254,205,211,.75)}.dark\:border-l-rose-200\/80:is(.dark *){border-left-color:rgba(254,205,211,.8)}.dark\:border-l-rose-200\/85:is(.dark *){border-left-color:rgba(254,205,211,.85)}.dark\:border-l-rose-200\/90:is(.dark *){border-left-color:rgba(254,205,211,.9)}.dark\:border-l-rose-200\/95:is(.dark *){border-left-color:rgba(254,205,211,.95)}.dark\:border-l-rose-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(251 113 133/var(--tw-border-opacity,1))}.dark\:border-l-rose-400\/0:is(.dark *){border-left-color:rgba(251,113,133,0)}.dark\:border-l-rose-400\/10:is(.dark *){border-left-color:rgba(251,113,133,.1)}.dark\:border-l-rose-400\/100:is(.dark *){border-left-color:#fb7185}.dark\:border-l-rose-400\/15:is(.dark *){border-left-color:rgba(251,113,133,.15)}.dark\:border-l-rose-400\/20:is(.dark *){border-left-color:rgba(251,113,133,.2)}.dark\:border-l-rose-400\/25:is(.dark *){border-left-color:rgba(251,113,133,.25)}.dark\:border-l-rose-400\/30:is(.dark *){border-left-color:rgba(251,113,133,.3)}.dark\:border-l-rose-400\/35:is(.dark *){border-left-color:rgba(251,113,133,.35)}.dark\:border-l-rose-400\/40:is(.dark *){border-left-color:rgba(251,113,133,.4)}.dark\:border-l-rose-400\/45:is(.dark *){border-left-color:rgba(251,113,133,.45)}.dark\:border-l-rose-400\/5:is(.dark *){border-left-color:rgba(251,113,133,.05)}.dark\:border-l-rose-400\/50:is(.dark *){border-left-color:rgba(251,113,133,.5)}.dark\:border-l-rose-400\/55:is(.dark *){border-left-color:rgba(251,113,133,.55)}.dark\:border-l-rose-400\/60:is(.dark *){border-left-color:rgba(251,113,133,.6)}.dark\:border-l-rose-400\/65:is(.dark *){border-left-color:rgba(251,113,133,.65)}.dark\:border-l-rose-400\/70:is(.dark *){border-left-color:rgba(251,113,133,.7)}.dark\:border-l-rose-400\/75:is(.dark *){border-left-color:rgba(251,113,133,.75)}.dark\:border-l-rose-400\/80:is(.dark *){border-left-color:rgba(251,113,133,.8)}.dark\:border-l-rose-400\/85:is(.dark *){border-left-color:rgba(251,113,133,.85)}.dark\:border-l-rose-400\/90:is(.dark *){border-left-color:rgba(251,113,133,.9)}.dark\:border-l-rose-400\/95:is(.dark *){border-left-color:rgba(251,113,133,.95)}.dark\:border-l-rose-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(255 241 242/var(--tw-border-opacity,1))}.dark\:border-l-rose-50\/0:is(.dark *){border-left-color:rgba(255,241,242,0)}.dark\:border-l-rose-50\/10:is(.dark *){border-left-color:rgba(255,241,242,.1)}.dark\:border-l-rose-50\/100:is(.dark *){border-left-color:#fff1f2}.dark\:border-l-rose-50\/15:is(.dark *){border-left-color:rgba(255,241,242,.15)}.dark\:border-l-rose-50\/20:is(.dark *){border-left-color:rgba(255,241,242,.2)}.dark\:border-l-rose-50\/25:is(.dark *){border-left-color:rgba(255,241,242,.25)}.dark\:border-l-rose-50\/30:is(.dark *){border-left-color:rgba(255,241,242,.3)}.dark\:border-l-rose-50\/35:is(.dark *){border-left-color:rgba(255,241,242,.35)}.dark\:border-l-rose-50\/40:is(.dark *){border-left-color:rgba(255,241,242,.4)}.dark\:border-l-rose-50\/45:is(.dark *){border-left-color:rgba(255,241,242,.45)}.dark\:border-l-rose-50\/5:is(.dark *){border-left-color:rgba(255,241,242,.05)}.dark\:border-l-rose-50\/50:is(.dark *){border-left-color:rgba(255,241,242,.5)}.dark\:border-l-rose-50\/55:is(.dark *){border-left-color:rgba(255,241,242,.55)}.dark\:border-l-rose-50\/60:is(.dark *){border-left-color:rgba(255,241,242,.6)}.dark\:border-l-rose-50\/65:is(.dark *){border-left-color:rgba(255,241,242,.65)}.dark\:border-l-rose-50\/70:is(.dark *){border-left-color:rgba(255,241,242,.7)}.dark\:border-l-rose-50\/75:is(.dark *){border-left-color:rgba(255,241,242,.75)}.dark\:border-l-rose-50\/80:is(.dark *){border-left-color:rgba(255,241,242,.8)}.dark\:border-l-rose-50\/85:is(.dark *){border-left-color:rgba(255,241,242,.85)}.dark\:border-l-rose-50\/90:is(.dark *){border-left-color:rgba(255,241,242,.9)}.dark\:border-l-rose-50\/95:is(.dark *){border-left-color:rgba(255,241,242,.95)}.dark\:border-l-rose-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(244 63 94/var(--tw-border-opacity,1))}.dark\:border-l-rose-500\/0:is(.dark *){border-left-color:rgba(244,63,94,0)}.dark\:border-l-rose-500\/10:is(.dark *){border-left-color:rgba(244,63,94,.1)}.dark\:border-l-rose-500\/100:is(.dark *){border-left-color:#f43f5e}.dark\:border-l-rose-500\/15:is(.dark *){border-left-color:rgba(244,63,94,.15)}.dark\:border-l-rose-500\/20:is(.dark *){border-left-color:rgba(244,63,94,.2)}.dark\:border-l-rose-500\/25:is(.dark *){border-left-color:rgba(244,63,94,.25)}.dark\:border-l-rose-500\/30:is(.dark *){border-left-color:rgba(244,63,94,.3)}.dark\:border-l-rose-500\/35:is(.dark *){border-left-color:rgba(244,63,94,.35)}.dark\:border-l-rose-500\/40:is(.dark *){border-left-color:rgba(244,63,94,.4)}.dark\:border-l-rose-500\/45:is(.dark *){border-left-color:rgba(244,63,94,.45)}.dark\:border-l-rose-500\/5:is(.dark *){border-left-color:rgba(244,63,94,.05)}.dark\:border-l-rose-500\/50:is(.dark *){border-left-color:rgba(244,63,94,.5)}.dark\:border-l-rose-500\/55:is(.dark *){border-left-color:rgba(244,63,94,.55)}.dark\:border-l-rose-500\/60:is(.dark *){border-left-color:rgba(244,63,94,.6)}.dark\:border-l-rose-500\/65:is(.dark *){border-left-color:rgba(244,63,94,.65)}.dark\:border-l-rose-500\/70:is(.dark *){border-left-color:rgba(244,63,94,.7)}.dark\:border-l-rose-500\/75:is(.dark *){border-left-color:rgba(244,63,94,.75)}.dark\:border-l-rose-500\/80:is(.dark *){border-left-color:rgba(244,63,94,.8)}.dark\:border-l-rose-500\/85:is(.dark *){border-left-color:rgba(244,63,94,.85)}.dark\:border-l-rose-500\/90:is(.dark *){border-left-color:rgba(244,63,94,.9)}.dark\:border-l-rose-500\/95:is(.dark *){border-left-color:rgba(244,63,94,.95)}.dark\:border-l-rose-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(76 5 25/var(--tw-border-opacity,1))}.dark\:border-l-rose-950\/0:is(.dark *){border-left-color:rgba(76,5,25,0)}.dark\:border-l-rose-950\/10:is(.dark *){border-left-color:rgba(76,5,25,.1)}.dark\:border-l-rose-950\/100:is(.dark *){border-left-color:#4c0519}.dark\:border-l-rose-950\/15:is(.dark *){border-left-color:rgba(76,5,25,.15)}.dark\:border-l-rose-950\/20:is(.dark *){border-left-color:rgba(76,5,25,.2)}.dark\:border-l-rose-950\/25:is(.dark *){border-left-color:rgba(76,5,25,.25)}.dark\:border-l-rose-950\/30:is(.dark *){border-left-color:rgba(76,5,25,.3)}.dark\:border-l-rose-950\/35:is(.dark *){border-left-color:rgba(76,5,25,.35)}.dark\:border-l-rose-950\/40:is(.dark *){border-left-color:rgba(76,5,25,.4)}.dark\:border-l-rose-950\/45:is(.dark *){border-left-color:rgba(76,5,25,.45)}.dark\:border-l-rose-950\/5:is(.dark *){border-left-color:rgba(76,5,25,.05)}.dark\:border-l-rose-950\/50:is(.dark *){border-left-color:rgba(76,5,25,.5)}.dark\:border-l-rose-950\/55:is(.dark *){border-left-color:rgba(76,5,25,.55)}.dark\:border-l-rose-950\/60:is(.dark *){border-left-color:rgba(76,5,25,.6)}.dark\:border-l-rose-950\/65:is(.dark *){border-left-color:rgba(76,5,25,.65)}.dark\:border-l-rose-950\/70:is(.dark *){border-left-color:rgba(76,5,25,.7)}.dark\:border-l-rose-950\/75:is(.dark *){border-left-color:rgba(76,5,25,.75)}.dark\:border-l-rose-950\/80:is(.dark *){border-left-color:rgba(76,5,25,.8)}.dark\:border-l-rose-950\/85:is(.dark *){border-left-color:rgba(76,5,25,.85)}.dark\:border-l-rose-950\/90:is(.dark *){border-left-color:rgba(76,5,25,.9)}.dark\:border-l-rose-950\/95:is(.dark *){border-left-color:rgba(76,5,25,.95)}.dark\:border-l-sky-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(224 242 254/var(--tw-border-opacity,1))}.dark\:border-l-sky-100\/0:is(.dark *){border-left-color:rgba(224,242,254,0)}.dark\:border-l-sky-100\/10:is(.dark *){border-left-color:rgba(224,242,254,.1)}.dark\:border-l-sky-100\/100:is(.dark *){border-left-color:#e0f2fe}.dark\:border-l-sky-100\/15:is(.dark *){border-left-color:rgba(224,242,254,.15)}.dark\:border-l-sky-100\/20:is(.dark *){border-left-color:rgba(224,242,254,.2)}.dark\:border-l-sky-100\/25:is(.dark *){border-left-color:rgba(224,242,254,.25)}.dark\:border-l-sky-100\/30:is(.dark *){border-left-color:rgba(224,242,254,.3)}.dark\:border-l-sky-100\/35:is(.dark *){border-left-color:rgba(224,242,254,.35)}.dark\:border-l-sky-100\/40:is(.dark *){border-left-color:rgba(224,242,254,.4)}.dark\:border-l-sky-100\/45:is(.dark *){border-left-color:rgba(224,242,254,.45)}.dark\:border-l-sky-100\/5:is(.dark *){border-left-color:rgba(224,242,254,.05)}.dark\:border-l-sky-100\/50:is(.dark *){border-left-color:rgba(224,242,254,.5)}.dark\:border-l-sky-100\/55:is(.dark *){border-left-color:rgba(224,242,254,.55)}.dark\:border-l-sky-100\/60:is(.dark *){border-left-color:rgba(224,242,254,.6)}.dark\:border-l-sky-100\/65:is(.dark *){border-left-color:rgba(224,242,254,.65)}.dark\:border-l-sky-100\/70:is(.dark *){border-left-color:rgba(224,242,254,.7)}.dark\:border-l-sky-100\/75:is(.dark *){border-left-color:rgba(224,242,254,.75)}.dark\:border-l-sky-100\/80:is(.dark *){border-left-color:rgba(224,242,254,.8)}.dark\:border-l-sky-100\/85:is(.dark *){border-left-color:rgba(224,242,254,.85)}.dark\:border-l-sky-100\/90:is(.dark *){border-left-color:rgba(224,242,254,.9)}.dark\:border-l-sky-100\/95:is(.dark *){border-left-color:rgba(224,242,254,.95)}.dark\:border-l-sky-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(186 230 253/var(--tw-border-opacity,1))}.dark\:border-l-sky-200\/0:is(.dark *){border-left-color:rgba(186,230,253,0)}.dark\:border-l-sky-200\/10:is(.dark *){border-left-color:rgba(186,230,253,.1)}.dark\:border-l-sky-200\/100:is(.dark *){border-left-color:#bae6fd}.dark\:border-l-sky-200\/15:is(.dark *){border-left-color:rgba(186,230,253,.15)}.dark\:border-l-sky-200\/20:is(.dark *){border-left-color:rgba(186,230,253,.2)}.dark\:border-l-sky-200\/25:is(.dark *){border-left-color:rgba(186,230,253,.25)}.dark\:border-l-sky-200\/30:is(.dark *){border-left-color:rgba(186,230,253,.3)}.dark\:border-l-sky-200\/35:is(.dark *){border-left-color:rgba(186,230,253,.35)}.dark\:border-l-sky-200\/40:is(.dark *){border-left-color:rgba(186,230,253,.4)}.dark\:border-l-sky-200\/45:is(.dark *){border-left-color:rgba(186,230,253,.45)}.dark\:border-l-sky-200\/5:is(.dark *){border-left-color:rgba(186,230,253,.05)}.dark\:border-l-sky-200\/50:is(.dark *){border-left-color:rgba(186,230,253,.5)}.dark\:border-l-sky-200\/55:is(.dark *){border-left-color:rgba(186,230,253,.55)}.dark\:border-l-sky-200\/60:is(.dark *){border-left-color:rgba(186,230,253,.6)}.dark\:border-l-sky-200\/65:is(.dark *){border-left-color:rgba(186,230,253,.65)}.dark\:border-l-sky-200\/70:is(.dark *){border-left-color:rgba(186,230,253,.7)}.dark\:border-l-sky-200\/75:is(.dark *){border-left-color:rgba(186,230,253,.75)}.dark\:border-l-sky-200\/80:is(.dark *){border-left-color:rgba(186,230,253,.8)}.dark\:border-l-sky-200\/85:is(.dark *){border-left-color:rgba(186,230,253,.85)}.dark\:border-l-sky-200\/90:is(.dark *){border-left-color:rgba(186,230,253,.9)}.dark\:border-l-sky-200\/95:is(.dark *){border-left-color:rgba(186,230,253,.95)}.dark\:border-l-sky-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(56 189 248/var(--tw-border-opacity,1))}.dark\:border-l-sky-400\/0:is(.dark *){border-left-color:rgba(56,189,248,0)}.dark\:border-l-sky-400\/10:is(.dark *){border-left-color:rgba(56,189,248,.1)}.dark\:border-l-sky-400\/100:is(.dark *){border-left-color:#38bdf8}.dark\:border-l-sky-400\/15:is(.dark *){border-left-color:rgba(56,189,248,.15)}.dark\:border-l-sky-400\/20:is(.dark *){border-left-color:rgba(56,189,248,.2)}.dark\:border-l-sky-400\/25:is(.dark *){border-left-color:rgba(56,189,248,.25)}.dark\:border-l-sky-400\/30:is(.dark *){border-left-color:rgba(56,189,248,.3)}.dark\:border-l-sky-400\/35:is(.dark *){border-left-color:rgba(56,189,248,.35)}.dark\:border-l-sky-400\/40:is(.dark *){border-left-color:rgba(56,189,248,.4)}.dark\:border-l-sky-400\/45:is(.dark *){border-left-color:rgba(56,189,248,.45)}.dark\:border-l-sky-400\/5:is(.dark *){border-left-color:rgba(56,189,248,.05)}.dark\:border-l-sky-400\/50:is(.dark *){border-left-color:rgba(56,189,248,.5)}.dark\:border-l-sky-400\/55:is(.dark *){border-left-color:rgba(56,189,248,.55)}.dark\:border-l-sky-400\/60:is(.dark *){border-left-color:rgba(56,189,248,.6)}.dark\:border-l-sky-400\/65:is(.dark *){border-left-color:rgba(56,189,248,.65)}.dark\:border-l-sky-400\/70:is(.dark *){border-left-color:rgba(56,189,248,.7)}.dark\:border-l-sky-400\/75:is(.dark *){border-left-color:rgba(56,189,248,.75)}.dark\:border-l-sky-400\/80:is(.dark *){border-left-color:rgba(56,189,248,.8)}.dark\:border-l-sky-400\/85:is(.dark *){border-left-color:rgba(56,189,248,.85)}.dark\:border-l-sky-400\/90:is(.dark *){border-left-color:rgba(56,189,248,.9)}.dark\:border-l-sky-400\/95:is(.dark *){border-left-color:rgba(56,189,248,.95)}.dark\:border-l-sky-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(240 249 255/var(--tw-border-opacity,1))}.dark\:border-l-sky-50\/0:is(.dark *){border-left-color:rgba(240,249,255,0)}.dark\:border-l-sky-50\/10:is(.dark *){border-left-color:rgba(240,249,255,.1)}.dark\:border-l-sky-50\/100:is(.dark *){border-left-color:#f0f9ff}.dark\:border-l-sky-50\/15:is(.dark *){border-left-color:rgba(240,249,255,.15)}.dark\:border-l-sky-50\/20:is(.dark *){border-left-color:rgba(240,249,255,.2)}.dark\:border-l-sky-50\/25:is(.dark *){border-left-color:rgba(240,249,255,.25)}.dark\:border-l-sky-50\/30:is(.dark *){border-left-color:rgba(240,249,255,.3)}.dark\:border-l-sky-50\/35:is(.dark *){border-left-color:rgba(240,249,255,.35)}.dark\:border-l-sky-50\/40:is(.dark *){border-left-color:rgba(240,249,255,.4)}.dark\:border-l-sky-50\/45:is(.dark *){border-left-color:rgba(240,249,255,.45)}.dark\:border-l-sky-50\/5:is(.dark *){border-left-color:rgba(240,249,255,.05)}.dark\:border-l-sky-50\/50:is(.dark *){border-left-color:rgba(240,249,255,.5)}.dark\:border-l-sky-50\/55:is(.dark *){border-left-color:rgba(240,249,255,.55)}.dark\:border-l-sky-50\/60:is(.dark *){border-left-color:rgba(240,249,255,.6)}.dark\:border-l-sky-50\/65:is(.dark *){border-left-color:rgba(240,249,255,.65)}.dark\:border-l-sky-50\/70:is(.dark *){border-left-color:rgba(240,249,255,.7)}.dark\:border-l-sky-50\/75:is(.dark *){border-left-color:rgba(240,249,255,.75)}.dark\:border-l-sky-50\/80:is(.dark *){border-left-color:rgba(240,249,255,.8)}.dark\:border-l-sky-50\/85:is(.dark *){border-left-color:rgba(240,249,255,.85)}.dark\:border-l-sky-50\/90:is(.dark *){border-left-color:rgba(240,249,255,.9)}.dark\:border-l-sky-50\/95:is(.dark *){border-left-color:rgba(240,249,255,.95)}.dark\:border-l-sky-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(14 165 233/var(--tw-border-opacity,1))}.dark\:border-l-sky-500\/0:is(.dark *){border-left-color:rgba(14,165,233,0)}.dark\:border-l-sky-500\/10:is(.dark *){border-left-color:rgba(14,165,233,.1)}.dark\:border-l-sky-500\/100:is(.dark *){border-left-color:#0ea5e9}.dark\:border-l-sky-500\/15:is(.dark *){border-left-color:rgba(14,165,233,.15)}.dark\:border-l-sky-500\/20:is(.dark *){border-left-color:rgba(14,165,233,.2)}.dark\:border-l-sky-500\/25:is(.dark *){border-left-color:rgba(14,165,233,.25)}.dark\:border-l-sky-500\/30:is(.dark *){border-left-color:rgba(14,165,233,.3)}.dark\:border-l-sky-500\/35:is(.dark *){border-left-color:rgba(14,165,233,.35)}.dark\:border-l-sky-500\/40:is(.dark *){border-left-color:rgba(14,165,233,.4)}.dark\:border-l-sky-500\/45:is(.dark *){border-left-color:rgba(14,165,233,.45)}.dark\:border-l-sky-500\/5:is(.dark *){border-left-color:rgba(14,165,233,.05)}.dark\:border-l-sky-500\/50:is(.dark *){border-left-color:rgba(14,165,233,.5)}.dark\:border-l-sky-500\/55:is(.dark *){border-left-color:rgba(14,165,233,.55)}.dark\:border-l-sky-500\/60:is(.dark *){border-left-color:rgba(14,165,233,.6)}.dark\:border-l-sky-500\/65:is(.dark *){border-left-color:rgba(14,165,233,.65)}.dark\:border-l-sky-500\/70:is(.dark *){border-left-color:rgba(14,165,233,.7)}.dark\:border-l-sky-500\/75:is(.dark *){border-left-color:rgba(14,165,233,.75)}.dark\:border-l-sky-500\/80:is(.dark *){border-left-color:rgba(14,165,233,.8)}.dark\:border-l-sky-500\/85:is(.dark *){border-left-color:rgba(14,165,233,.85)}.dark\:border-l-sky-500\/90:is(.dark *){border-left-color:rgba(14,165,233,.9)}.dark\:border-l-sky-500\/95:is(.dark *){border-left-color:rgba(14,165,233,.95)}.dark\:border-l-sky-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(8 47 73/var(--tw-border-opacity,1))}.dark\:border-l-sky-950\/0:is(.dark *){border-left-color:rgba(8,47,73,0)}.dark\:border-l-sky-950\/10:is(.dark *){border-left-color:rgba(8,47,73,.1)}.dark\:border-l-sky-950\/100:is(.dark *){border-left-color:#082f49}.dark\:border-l-sky-950\/15:is(.dark *){border-left-color:rgba(8,47,73,.15)}.dark\:border-l-sky-950\/20:is(.dark *){border-left-color:rgba(8,47,73,.2)}.dark\:border-l-sky-950\/25:is(.dark *){border-left-color:rgba(8,47,73,.25)}.dark\:border-l-sky-950\/30:is(.dark *){border-left-color:rgba(8,47,73,.3)}.dark\:border-l-sky-950\/35:is(.dark *){border-left-color:rgba(8,47,73,.35)}.dark\:border-l-sky-950\/40:is(.dark *){border-left-color:rgba(8,47,73,.4)}.dark\:border-l-sky-950\/45:is(.dark *){border-left-color:rgba(8,47,73,.45)}.dark\:border-l-sky-950\/5:is(.dark *){border-left-color:rgba(8,47,73,.05)}.dark\:border-l-sky-950\/50:is(.dark *){border-left-color:rgba(8,47,73,.5)}.dark\:border-l-sky-950\/55:is(.dark *){border-left-color:rgba(8,47,73,.55)}.dark\:border-l-sky-950\/60:is(.dark *){border-left-color:rgba(8,47,73,.6)}.dark\:border-l-sky-950\/65:is(.dark *){border-left-color:rgba(8,47,73,.65)}.dark\:border-l-sky-950\/70:is(.dark *){border-left-color:rgba(8,47,73,.7)}.dark\:border-l-sky-950\/75:is(.dark *){border-left-color:rgba(8,47,73,.75)}.dark\:border-l-sky-950\/80:is(.dark *){border-left-color:rgba(8,47,73,.8)}.dark\:border-l-sky-950\/85:is(.dark *){border-left-color:rgba(8,47,73,.85)}.dark\:border-l-sky-950\/90:is(.dark *){border-left-color:rgba(8,47,73,.9)}.dark\:border-l-sky-950\/95:is(.dark *){border-left-color:rgba(8,47,73,.95)}.dark\:border-l-slate-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(241 245 249/var(--tw-border-opacity,1))}.dark\:border-l-slate-100\/0:is(.dark *){border-left-color:rgba(241,245,249,0)}.dark\:border-l-slate-100\/10:is(.dark *){border-left-color:rgba(241,245,249,.1)}.dark\:border-l-slate-100\/100:is(.dark *){border-left-color:#f1f5f9}.dark\:border-l-slate-100\/15:is(.dark *){border-left-color:rgba(241,245,249,.15)}.dark\:border-l-slate-100\/20:is(.dark *){border-left-color:rgba(241,245,249,.2)}.dark\:border-l-slate-100\/25:is(.dark *){border-left-color:rgba(241,245,249,.25)}.dark\:border-l-slate-100\/30:is(.dark *){border-left-color:rgba(241,245,249,.3)}.dark\:border-l-slate-100\/35:is(.dark *){border-left-color:rgba(241,245,249,.35)}.dark\:border-l-slate-100\/40:is(.dark *){border-left-color:rgba(241,245,249,.4)}.dark\:border-l-slate-100\/45:is(.dark *){border-left-color:rgba(241,245,249,.45)}.dark\:border-l-slate-100\/5:is(.dark *){border-left-color:rgba(241,245,249,.05)}.dark\:border-l-slate-100\/50:is(.dark *){border-left-color:rgba(241,245,249,.5)}.dark\:border-l-slate-100\/55:is(.dark *){border-left-color:rgba(241,245,249,.55)}.dark\:border-l-slate-100\/60:is(.dark *){border-left-color:rgba(241,245,249,.6)}.dark\:border-l-slate-100\/65:is(.dark *){border-left-color:rgba(241,245,249,.65)}.dark\:border-l-slate-100\/70:is(.dark *){border-left-color:rgba(241,245,249,.7)}.dark\:border-l-slate-100\/75:is(.dark *){border-left-color:rgba(241,245,249,.75)}.dark\:border-l-slate-100\/80:is(.dark *){border-left-color:rgba(241,245,249,.8)}.dark\:border-l-slate-100\/85:is(.dark *){border-left-color:rgba(241,245,249,.85)}.dark\:border-l-slate-100\/90:is(.dark *){border-left-color:rgba(241,245,249,.9)}.dark\:border-l-slate-100\/95:is(.dark *){border-left-color:rgba(241,245,249,.95)}.dark\:border-l-slate-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(226 232 240/var(--tw-border-opacity,1))}.dark\:border-l-slate-200\/0:is(.dark *){border-left-color:rgba(226,232,240,0)}.dark\:border-l-slate-200\/10:is(.dark *){border-left-color:rgba(226,232,240,.1)}.dark\:border-l-slate-200\/100:is(.dark *){border-left-color:#e2e8f0}.dark\:border-l-slate-200\/15:is(.dark *){border-left-color:rgba(226,232,240,.15)}.dark\:border-l-slate-200\/20:is(.dark *){border-left-color:rgba(226,232,240,.2)}.dark\:border-l-slate-200\/25:is(.dark *){border-left-color:rgba(226,232,240,.25)}.dark\:border-l-slate-200\/30:is(.dark *){border-left-color:rgba(226,232,240,.3)}.dark\:border-l-slate-200\/35:is(.dark *){border-left-color:rgba(226,232,240,.35)}.dark\:border-l-slate-200\/40:is(.dark *){border-left-color:rgba(226,232,240,.4)}.dark\:border-l-slate-200\/45:is(.dark *){border-left-color:rgba(226,232,240,.45)}.dark\:border-l-slate-200\/5:is(.dark *){border-left-color:rgba(226,232,240,.05)}.dark\:border-l-slate-200\/50:is(.dark *){border-left-color:rgba(226,232,240,.5)}.dark\:border-l-slate-200\/55:is(.dark *){border-left-color:rgba(226,232,240,.55)}.dark\:border-l-slate-200\/60:is(.dark *){border-left-color:rgba(226,232,240,.6)}.dark\:border-l-slate-200\/65:is(.dark *){border-left-color:rgba(226,232,240,.65)}.dark\:border-l-slate-200\/70:is(.dark *){border-left-color:rgba(226,232,240,.7)}.dark\:border-l-slate-200\/75:is(.dark *){border-left-color:rgba(226,232,240,.75)}.dark\:border-l-slate-200\/80:is(.dark *){border-left-color:rgba(226,232,240,.8)}.dark\:border-l-slate-200\/85:is(.dark *){border-left-color:rgba(226,232,240,.85)}.dark\:border-l-slate-200\/90:is(.dark *){border-left-color:rgba(226,232,240,.9)}.dark\:border-l-slate-200\/95:is(.dark *){border-left-color:rgba(226,232,240,.95)}.dark\:border-l-slate-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(148 163 184/var(--tw-border-opacity,1))}.dark\:border-l-slate-400\/0:is(.dark *){border-left-color:rgba(148,163,184,0)}.dark\:border-l-slate-400\/10:is(.dark *){border-left-color:rgba(148,163,184,.1)}.dark\:border-l-slate-400\/100:is(.dark *){border-left-color:#94a3b8}.dark\:border-l-slate-400\/15:is(.dark *){border-left-color:rgba(148,163,184,.15)}.dark\:border-l-slate-400\/20:is(.dark *){border-left-color:rgba(148,163,184,.2)}.dark\:border-l-slate-400\/25:is(.dark *){border-left-color:rgba(148,163,184,.25)}.dark\:border-l-slate-400\/30:is(.dark *){border-left-color:rgba(148,163,184,.3)}.dark\:border-l-slate-400\/35:is(.dark *){border-left-color:rgba(148,163,184,.35)}.dark\:border-l-slate-400\/40:is(.dark *){border-left-color:rgba(148,163,184,.4)}.dark\:border-l-slate-400\/45:is(.dark *){border-left-color:rgba(148,163,184,.45)}.dark\:border-l-slate-400\/5:is(.dark *){border-left-color:rgba(148,163,184,.05)}.dark\:border-l-slate-400\/50:is(.dark *){border-left-color:rgba(148,163,184,.5)}.dark\:border-l-slate-400\/55:is(.dark *){border-left-color:rgba(148,163,184,.55)}.dark\:border-l-slate-400\/60:is(.dark *){border-left-color:rgba(148,163,184,.6)}.dark\:border-l-slate-400\/65:is(.dark *){border-left-color:rgba(148,163,184,.65)}.dark\:border-l-slate-400\/70:is(.dark *){border-left-color:rgba(148,163,184,.7)}.dark\:border-l-slate-400\/75:is(.dark *){border-left-color:rgba(148,163,184,.75)}.dark\:border-l-slate-400\/80:is(.dark *){border-left-color:rgba(148,163,184,.8)}.dark\:border-l-slate-400\/85:is(.dark *){border-left-color:rgba(148,163,184,.85)}.dark\:border-l-slate-400\/90:is(.dark *){border-left-color:rgba(148,163,184,.9)}.dark\:border-l-slate-400\/95:is(.dark *){border-left-color:rgba(148,163,184,.95)}.dark\:border-l-slate-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(248 250 252/var(--tw-border-opacity,1))}.dark\:border-l-slate-50\/0:is(.dark *){border-left-color:rgba(248,250,252,0)}.dark\:border-l-slate-50\/10:is(.dark *){border-left-color:rgba(248,250,252,.1)}.dark\:border-l-slate-50\/100:is(.dark *){border-left-color:#f8fafc}.dark\:border-l-slate-50\/15:is(.dark *){border-left-color:rgba(248,250,252,.15)}.dark\:border-l-slate-50\/20:is(.dark *){border-left-color:rgba(248,250,252,.2)}.dark\:border-l-slate-50\/25:is(.dark *){border-left-color:rgba(248,250,252,.25)}.dark\:border-l-slate-50\/30:is(.dark *){border-left-color:rgba(248,250,252,.3)}.dark\:border-l-slate-50\/35:is(.dark *){border-left-color:rgba(248,250,252,.35)}.dark\:border-l-slate-50\/40:is(.dark *){border-left-color:rgba(248,250,252,.4)}.dark\:border-l-slate-50\/45:is(.dark *){border-left-color:rgba(248,250,252,.45)}.dark\:border-l-slate-50\/5:is(.dark *){border-left-color:rgba(248,250,252,.05)}.dark\:border-l-slate-50\/50:is(.dark *){border-left-color:rgba(248,250,252,.5)}.dark\:border-l-slate-50\/55:is(.dark *){border-left-color:rgba(248,250,252,.55)}.dark\:border-l-slate-50\/60:is(.dark *){border-left-color:rgba(248,250,252,.6)}.dark\:border-l-slate-50\/65:is(.dark *){border-left-color:rgba(248,250,252,.65)}.dark\:border-l-slate-50\/70:is(.dark *){border-left-color:rgba(248,250,252,.7)}.dark\:border-l-slate-50\/75:is(.dark *){border-left-color:rgba(248,250,252,.75)}.dark\:border-l-slate-50\/80:is(.dark *){border-left-color:rgba(248,250,252,.8)}.dark\:border-l-slate-50\/85:is(.dark *){border-left-color:rgba(248,250,252,.85)}.dark\:border-l-slate-50\/90:is(.dark *){border-left-color:rgba(248,250,252,.9)}.dark\:border-l-slate-50\/95:is(.dark *){border-left-color:rgba(248,250,252,.95)}.dark\:border-l-slate-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(100 116 139/var(--tw-border-opacity,1))}.dark\:border-l-slate-500\/0:is(.dark *){border-left-color:rgba(100,116,139,0)}.dark\:border-l-slate-500\/10:is(.dark *){border-left-color:rgba(100,116,139,.1)}.dark\:border-l-slate-500\/100:is(.dark *){border-left-color:#64748b}.dark\:border-l-slate-500\/15:is(.dark *){border-left-color:rgba(100,116,139,.15)}.dark\:border-l-slate-500\/20:is(.dark *){border-left-color:rgba(100,116,139,.2)}.dark\:border-l-slate-500\/25:is(.dark *){border-left-color:rgba(100,116,139,.25)}.dark\:border-l-slate-500\/30:is(.dark *){border-left-color:rgba(100,116,139,.3)}.dark\:border-l-slate-500\/35:is(.dark *){border-left-color:rgba(100,116,139,.35)}.dark\:border-l-slate-500\/40:is(.dark *){border-left-color:rgba(100,116,139,.4)}.dark\:border-l-slate-500\/45:is(.dark *){border-left-color:rgba(100,116,139,.45)}.dark\:border-l-slate-500\/5:is(.dark *){border-left-color:rgba(100,116,139,.05)}.dark\:border-l-slate-500\/50:is(.dark *){border-left-color:rgba(100,116,139,.5)}.dark\:border-l-slate-500\/55:is(.dark *){border-left-color:rgba(100,116,139,.55)}.dark\:border-l-slate-500\/60:is(.dark *){border-left-color:rgba(100,116,139,.6)}.dark\:border-l-slate-500\/65:is(.dark *){border-left-color:rgba(100,116,139,.65)}.dark\:border-l-slate-500\/70:is(.dark *){border-left-color:rgba(100,116,139,.7)}.dark\:border-l-slate-500\/75:is(.dark *){border-left-color:rgba(100,116,139,.75)}.dark\:border-l-slate-500\/80:is(.dark *){border-left-color:rgba(100,116,139,.8)}.dark\:border-l-slate-500\/85:is(.dark *){border-left-color:rgba(100,116,139,.85)}.dark\:border-l-slate-500\/90:is(.dark *){border-left-color:rgba(100,116,139,.9)}.dark\:border-l-slate-500\/95:is(.dark *){border-left-color:rgba(100,116,139,.95)}.dark\:border-l-slate-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(2 6 23/var(--tw-border-opacity,1))}.dark\:border-l-slate-950\/0:is(.dark *){border-left-color:rgba(2,6,23,0)}.dark\:border-l-slate-950\/10:is(.dark *){border-left-color:rgba(2,6,23,.1)}.dark\:border-l-slate-950\/100:is(.dark *){border-left-color:#020617}.dark\:border-l-slate-950\/15:is(.dark *){border-left-color:rgba(2,6,23,.15)}.dark\:border-l-slate-950\/20:is(.dark *){border-left-color:rgba(2,6,23,.2)}.dark\:border-l-slate-950\/25:is(.dark *){border-left-color:rgba(2,6,23,.25)}.dark\:border-l-slate-950\/30:is(.dark *){border-left-color:rgba(2,6,23,.3)}.dark\:border-l-slate-950\/35:is(.dark *){border-left-color:rgba(2,6,23,.35)}.dark\:border-l-slate-950\/40:is(.dark *){border-left-color:rgba(2,6,23,.4)}.dark\:border-l-slate-950\/45:is(.dark *){border-left-color:rgba(2,6,23,.45)}.dark\:border-l-slate-950\/5:is(.dark *){border-left-color:rgba(2,6,23,.05)}.dark\:border-l-slate-950\/50:is(.dark *){border-left-color:rgba(2,6,23,.5)}.dark\:border-l-slate-950\/55:is(.dark *){border-left-color:rgba(2,6,23,.55)}.dark\:border-l-slate-950\/60:is(.dark *){border-left-color:rgba(2,6,23,.6)}.dark\:border-l-slate-950\/65:is(.dark *){border-left-color:rgba(2,6,23,.65)}.dark\:border-l-slate-950\/70:is(.dark *){border-left-color:rgba(2,6,23,.7)}.dark\:border-l-slate-950\/75:is(.dark *){border-left-color:rgba(2,6,23,.75)}.dark\:border-l-slate-950\/80:is(.dark *){border-left-color:rgba(2,6,23,.8)}.dark\:border-l-slate-950\/85:is(.dark *){border-left-color:rgba(2,6,23,.85)}.dark\:border-l-slate-950\/90:is(.dark *){border-left-color:rgba(2,6,23,.9)}.dark\:border-l-slate-950\/95:is(.dark *){border-left-color:rgba(2,6,23,.95)}.dark\:border-l-stone-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(245 245 244/var(--tw-border-opacity,1))}.dark\:border-l-stone-100\/0:is(.dark *){border-left-color:hsla(60,5%,96%,0)}.dark\:border-l-stone-100\/10:is(.dark *){border-left-color:hsla(60,5%,96%,.1)}.dark\:border-l-stone-100\/100:is(.dark *){border-left-color:#f5f5f4}.dark\:border-l-stone-100\/15:is(.dark *){border-left-color:hsla(60,5%,96%,.15)}.dark\:border-l-stone-100\/20:is(.dark *){border-left-color:hsla(60,5%,96%,.2)}.dark\:border-l-stone-100\/25:is(.dark *){border-left-color:hsla(60,5%,96%,.25)}.dark\:border-l-stone-100\/30:is(.dark *){border-left-color:hsla(60,5%,96%,.3)}.dark\:border-l-stone-100\/35:is(.dark *){border-left-color:hsla(60,5%,96%,.35)}.dark\:border-l-stone-100\/40:is(.dark *){border-left-color:hsla(60,5%,96%,.4)}.dark\:border-l-stone-100\/45:is(.dark *){border-left-color:hsla(60,5%,96%,.45)}.dark\:border-l-stone-100\/5:is(.dark *){border-left-color:hsla(60,5%,96%,.05)}.dark\:border-l-stone-100\/50:is(.dark *){border-left-color:hsla(60,5%,96%,.5)}.dark\:border-l-stone-100\/55:is(.dark *){border-left-color:hsla(60,5%,96%,.55)}.dark\:border-l-stone-100\/60:is(.dark *){border-left-color:hsla(60,5%,96%,.6)}.dark\:border-l-stone-100\/65:is(.dark *){border-left-color:hsla(60,5%,96%,.65)}.dark\:border-l-stone-100\/70:is(.dark *){border-left-color:hsla(60,5%,96%,.7)}.dark\:border-l-stone-100\/75:is(.dark *){border-left-color:hsla(60,5%,96%,.75)}.dark\:border-l-stone-100\/80:is(.dark *){border-left-color:hsla(60,5%,96%,.8)}.dark\:border-l-stone-100\/85:is(.dark *){border-left-color:hsla(60,5%,96%,.85)}.dark\:border-l-stone-100\/90:is(.dark *){border-left-color:hsla(60,5%,96%,.9)}.dark\:border-l-stone-100\/95:is(.dark *){border-left-color:hsla(60,5%,96%,.95)}.dark\:border-l-stone-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(231 229 228/var(--tw-border-opacity,1))}.dark\:border-l-stone-200\/0:is(.dark *){border-left-color:hsla(20,6%,90%,0)}.dark\:border-l-stone-200\/10:is(.dark *){border-left-color:hsla(20,6%,90%,.1)}.dark\:border-l-stone-200\/100:is(.dark *){border-left-color:#e7e5e4}.dark\:border-l-stone-200\/15:is(.dark *){border-left-color:hsla(20,6%,90%,.15)}.dark\:border-l-stone-200\/20:is(.dark *){border-left-color:hsla(20,6%,90%,.2)}.dark\:border-l-stone-200\/25:is(.dark *){border-left-color:hsla(20,6%,90%,.25)}.dark\:border-l-stone-200\/30:is(.dark *){border-left-color:hsla(20,6%,90%,.3)}.dark\:border-l-stone-200\/35:is(.dark *){border-left-color:hsla(20,6%,90%,.35)}.dark\:border-l-stone-200\/40:is(.dark *){border-left-color:hsla(20,6%,90%,.4)}.dark\:border-l-stone-200\/45:is(.dark *){border-left-color:hsla(20,6%,90%,.45)}.dark\:border-l-stone-200\/5:is(.dark *){border-left-color:hsla(20,6%,90%,.05)}.dark\:border-l-stone-200\/50:is(.dark *){border-left-color:hsla(20,6%,90%,.5)}.dark\:border-l-stone-200\/55:is(.dark *){border-left-color:hsla(20,6%,90%,.55)}.dark\:border-l-stone-200\/60:is(.dark *){border-left-color:hsla(20,6%,90%,.6)}.dark\:border-l-stone-200\/65:is(.dark *){border-left-color:hsla(20,6%,90%,.65)}.dark\:border-l-stone-200\/70:is(.dark *){border-left-color:hsla(20,6%,90%,.7)}.dark\:border-l-stone-200\/75:is(.dark *){border-left-color:hsla(20,6%,90%,.75)}.dark\:border-l-stone-200\/80:is(.dark *){border-left-color:hsla(20,6%,90%,.8)}.dark\:border-l-stone-200\/85:is(.dark *){border-left-color:hsla(20,6%,90%,.85)}.dark\:border-l-stone-200\/90:is(.dark *){border-left-color:hsla(20,6%,90%,.9)}.dark\:border-l-stone-200\/95:is(.dark *){border-left-color:hsla(20,6%,90%,.95)}.dark\:border-l-stone-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(168 162 158/var(--tw-border-opacity,1))}.dark\:border-l-stone-400\/0:is(.dark *){border-left-color:hsla(24,5%,64%,0)}.dark\:border-l-stone-400\/10:is(.dark *){border-left-color:hsla(24,5%,64%,.1)}.dark\:border-l-stone-400\/100:is(.dark *){border-left-color:#a8a29e}.dark\:border-l-stone-400\/15:is(.dark *){border-left-color:hsla(24,5%,64%,.15)}.dark\:border-l-stone-400\/20:is(.dark *){border-left-color:hsla(24,5%,64%,.2)}.dark\:border-l-stone-400\/25:is(.dark *){border-left-color:hsla(24,5%,64%,.25)}.dark\:border-l-stone-400\/30:is(.dark *){border-left-color:hsla(24,5%,64%,.3)}.dark\:border-l-stone-400\/35:is(.dark *){border-left-color:hsla(24,5%,64%,.35)}.dark\:border-l-stone-400\/40:is(.dark *){border-left-color:hsla(24,5%,64%,.4)}.dark\:border-l-stone-400\/45:is(.dark *){border-left-color:hsla(24,5%,64%,.45)}.dark\:border-l-stone-400\/5:is(.dark *){border-left-color:hsla(24,5%,64%,.05)}.dark\:border-l-stone-400\/50:is(.dark *){border-left-color:hsla(24,5%,64%,.5)}.dark\:border-l-stone-400\/55:is(.dark *){border-left-color:hsla(24,5%,64%,.55)}.dark\:border-l-stone-400\/60:is(.dark *){border-left-color:hsla(24,5%,64%,.6)}.dark\:border-l-stone-400\/65:is(.dark *){border-left-color:hsla(24,5%,64%,.65)}.dark\:border-l-stone-400\/70:is(.dark *){border-left-color:hsla(24,5%,64%,.7)}.dark\:border-l-stone-400\/75:is(.dark *){border-left-color:hsla(24,5%,64%,.75)}.dark\:border-l-stone-400\/80:is(.dark *){border-left-color:hsla(24,5%,64%,.8)}.dark\:border-l-stone-400\/85:is(.dark *){border-left-color:hsla(24,5%,64%,.85)}.dark\:border-l-stone-400\/90:is(.dark *){border-left-color:hsla(24,5%,64%,.9)}.dark\:border-l-stone-400\/95:is(.dark *){border-left-color:hsla(24,5%,64%,.95)}.dark\:border-l-stone-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(250 250 249/var(--tw-border-opacity,1))}.dark\:border-l-stone-50\/0:is(.dark *){border-left-color:hsla(60,9%,98%,0)}.dark\:border-l-stone-50\/10:is(.dark *){border-left-color:hsla(60,9%,98%,.1)}.dark\:border-l-stone-50\/100:is(.dark *){border-left-color:#fafaf9}.dark\:border-l-stone-50\/15:is(.dark *){border-left-color:hsla(60,9%,98%,.15)}.dark\:border-l-stone-50\/20:is(.dark *){border-left-color:hsla(60,9%,98%,.2)}.dark\:border-l-stone-50\/25:is(.dark *){border-left-color:hsla(60,9%,98%,.25)}.dark\:border-l-stone-50\/30:is(.dark *){border-left-color:hsla(60,9%,98%,.3)}.dark\:border-l-stone-50\/35:is(.dark *){border-left-color:hsla(60,9%,98%,.35)}.dark\:border-l-stone-50\/40:is(.dark *){border-left-color:hsla(60,9%,98%,.4)}.dark\:border-l-stone-50\/45:is(.dark *){border-left-color:hsla(60,9%,98%,.45)}.dark\:border-l-stone-50\/5:is(.dark *){border-left-color:hsla(60,9%,98%,.05)}.dark\:border-l-stone-50\/50:is(.dark *){border-left-color:hsla(60,9%,98%,.5)}.dark\:border-l-stone-50\/55:is(.dark *){border-left-color:hsla(60,9%,98%,.55)}.dark\:border-l-stone-50\/60:is(.dark *){border-left-color:hsla(60,9%,98%,.6)}.dark\:border-l-stone-50\/65:is(.dark *){border-left-color:hsla(60,9%,98%,.65)}.dark\:border-l-stone-50\/70:is(.dark *){border-left-color:hsla(60,9%,98%,.7)}.dark\:border-l-stone-50\/75:is(.dark *){border-left-color:hsla(60,9%,98%,.75)}.dark\:border-l-stone-50\/80:is(.dark *){border-left-color:hsla(60,9%,98%,.8)}.dark\:border-l-stone-50\/85:is(.dark *){border-left-color:hsla(60,9%,98%,.85)}.dark\:border-l-stone-50\/90:is(.dark *){border-left-color:hsla(60,9%,98%,.9)}.dark\:border-l-stone-50\/95:is(.dark *){border-left-color:hsla(60,9%,98%,.95)}.dark\:border-l-stone-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(120 113 108/var(--tw-border-opacity,1))}.dark\:border-l-stone-500\/0:is(.dark *){border-left-color:hsla(25,5%,45%,0)}.dark\:border-l-stone-500\/10:is(.dark *){border-left-color:hsla(25,5%,45%,.1)}.dark\:border-l-stone-500\/100:is(.dark *){border-left-color:#78716c}.dark\:border-l-stone-500\/15:is(.dark *){border-left-color:hsla(25,5%,45%,.15)}.dark\:border-l-stone-500\/20:is(.dark *){border-left-color:hsla(25,5%,45%,.2)}.dark\:border-l-stone-500\/25:is(.dark *){border-left-color:hsla(25,5%,45%,.25)}.dark\:border-l-stone-500\/30:is(.dark *){border-left-color:hsla(25,5%,45%,.3)}.dark\:border-l-stone-500\/35:is(.dark *){border-left-color:hsla(25,5%,45%,.35)}.dark\:border-l-stone-500\/40:is(.dark *){border-left-color:hsla(25,5%,45%,.4)}.dark\:border-l-stone-500\/45:is(.dark *){border-left-color:hsla(25,5%,45%,.45)}.dark\:border-l-stone-500\/5:is(.dark *){border-left-color:hsla(25,5%,45%,.05)}.dark\:border-l-stone-500\/50:is(.dark *){border-left-color:hsla(25,5%,45%,.5)}.dark\:border-l-stone-500\/55:is(.dark *){border-left-color:hsla(25,5%,45%,.55)}.dark\:border-l-stone-500\/60:is(.dark *){border-left-color:hsla(25,5%,45%,.6)}.dark\:border-l-stone-500\/65:is(.dark *){border-left-color:hsla(25,5%,45%,.65)}.dark\:border-l-stone-500\/70:is(.dark *){border-left-color:hsla(25,5%,45%,.7)}.dark\:border-l-stone-500\/75:is(.dark *){border-left-color:hsla(25,5%,45%,.75)}.dark\:border-l-stone-500\/80:is(.dark *){border-left-color:hsla(25,5%,45%,.8)}.dark\:border-l-stone-500\/85:is(.dark *){border-left-color:hsla(25,5%,45%,.85)}.dark\:border-l-stone-500\/90:is(.dark *){border-left-color:hsla(25,5%,45%,.9)}.dark\:border-l-stone-500\/95:is(.dark *){border-left-color:hsla(25,5%,45%,.95)}.dark\:border-l-stone-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(12 10 9/var(--tw-border-opacity,1))}.dark\:border-l-stone-950\/0:is(.dark *){border-left-color:rgba(12,10,9,0)}.dark\:border-l-stone-950\/10:is(.dark *){border-left-color:rgba(12,10,9,.1)}.dark\:border-l-stone-950\/100:is(.dark *){border-left-color:#0c0a09}.dark\:border-l-stone-950\/15:is(.dark *){border-left-color:rgba(12,10,9,.15)}.dark\:border-l-stone-950\/20:is(.dark *){border-left-color:rgba(12,10,9,.2)}.dark\:border-l-stone-950\/25:is(.dark *){border-left-color:rgba(12,10,9,.25)}.dark\:border-l-stone-950\/30:is(.dark *){border-left-color:rgba(12,10,9,.3)}.dark\:border-l-stone-950\/35:is(.dark *){border-left-color:rgba(12,10,9,.35)}.dark\:border-l-stone-950\/40:is(.dark *){border-left-color:rgba(12,10,9,.4)}.dark\:border-l-stone-950\/45:is(.dark *){border-left-color:rgba(12,10,9,.45)}.dark\:border-l-stone-950\/5:is(.dark *){border-left-color:rgba(12,10,9,.05)}.dark\:border-l-stone-950\/50:is(.dark *){border-left-color:rgba(12,10,9,.5)}.dark\:border-l-stone-950\/55:is(.dark *){border-left-color:rgba(12,10,9,.55)}.dark\:border-l-stone-950\/60:is(.dark *){border-left-color:rgba(12,10,9,.6)}.dark\:border-l-stone-950\/65:is(.dark *){border-left-color:rgba(12,10,9,.65)}.dark\:border-l-stone-950\/70:is(.dark *){border-left-color:rgba(12,10,9,.7)}.dark\:border-l-stone-950\/75:is(.dark *){border-left-color:rgba(12,10,9,.75)}.dark\:border-l-stone-950\/80:is(.dark *){border-left-color:rgba(12,10,9,.8)}.dark\:border-l-stone-950\/85:is(.dark *){border-left-color:rgba(12,10,9,.85)}.dark\:border-l-stone-950\/90:is(.dark *){border-left-color:rgba(12,10,9,.9)}.dark\:border-l-stone-950\/95:is(.dark *){border-left-color:rgba(12,10,9,.95)}.dark\:border-l-teal-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(204 251 241/var(--tw-border-opacity,1))}.dark\:border-l-teal-100\/0:is(.dark *){border-left-color:rgba(204,251,241,0)}.dark\:border-l-teal-100\/10:is(.dark *){border-left-color:rgba(204,251,241,.1)}.dark\:border-l-teal-100\/100:is(.dark *){border-left-color:#ccfbf1}.dark\:border-l-teal-100\/15:is(.dark *){border-left-color:rgba(204,251,241,.15)}.dark\:border-l-teal-100\/20:is(.dark *){border-left-color:rgba(204,251,241,.2)}.dark\:border-l-teal-100\/25:is(.dark *){border-left-color:rgba(204,251,241,.25)}.dark\:border-l-teal-100\/30:is(.dark *){border-left-color:rgba(204,251,241,.3)}.dark\:border-l-teal-100\/35:is(.dark *){border-left-color:rgba(204,251,241,.35)}.dark\:border-l-teal-100\/40:is(.dark *){border-left-color:rgba(204,251,241,.4)}.dark\:border-l-teal-100\/45:is(.dark *){border-left-color:rgba(204,251,241,.45)}.dark\:border-l-teal-100\/5:is(.dark *){border-left-color:rgba(204,251,241,.05)}.dark\:border-l-teal-100\/50:is(.dark *){border-left-color:rgba(204,251,241,.5)}.dark\:border-l-teal-100\/55:is(.dark *){border-left-color:rgba(204,251,241,.55)}.dark\:border-l-teal-100\/60:is(.dark *){border-left-color:rgba(204,251,241,.6)}.dark\:border-l-teal-100\/65:is(.dark *){border-left-color:rgba(204,251,241,.65)}.dark\:border-l-teal-100\/70:is(.dark *){border-left-color:rgba(204,251,241,.7)}.dark\:border-l-teal-100\/75:is(.dark *){border-left-color:rgba(204,251,241,.75)}.dark\:border-l-teal-100\/80:is(.dark *){border-left-color:rgba(204,251,241,.8)}.dark\:border-l-teal-100\/85:is(.dark *){border-left-color:rgba(204,251,241,.85)}.dark\:border-l-teal-100\/90:is(.dark *){border-left-color:rgba(204,251,241,.9)}.dark\:border-l-teal-100\/95:is(.dark *){border-left-color:rgba(204,251,241,.95)}.dark\:border-l-teal-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(153 246 228/var(--tw-border-opacity,1))}.dark\:border-l-teal-200\/0:is(.dark *){border-left-color:rgba(153,246,228,0)}.dark\:border-l-teal-200\/10:is(.dark *){border-left-color:rgba(153,246,228,.1)}.dark\:border-l-teal-200\/100:is(.dark *){border-left-color:#99f6e4}.dark\:border-l-teal-200\/15:is(.dark *){border-left-color:rgba(153,246,228,.15)}.dark\:border-l-teal-200\/20:is(.dark *){border-left-color:rgba(153,246,228,.2)}.dark\:border-l-teal-200\/25:is(.dark *){border-left-color:rgba(153,246,228,.25)}.dark\:border-l-teal-200\/30:is(.dark *){border-left-color:rgba(153,246,228,.3)}.dark\:border-l-teal-200\/35:is(.dark *){border-left-color:rgba(153,246,228,.35)}.dark\:border-l-teal-200\/40:is(.dark *){border-left-color:rgba(153,246,228,.4)}.dark\:border-l-teal-200\/45:is(.dark *){border-left-color:rgba(153,246,228,.45)}.dark\:border-l-teal-200\/5:is(.dark *){border-left-color:rgba(153,246,228,.05)}.dark\:border-l-teal-200\/50:is(.dark *){border-left-color:rgba(153,246,228,.5)}.dark\:border-l-teal-200\/55:is(.dark *){border-left-color:rgba(153,246,228,.55)}.dark\:border-l-teal-200\/60:is(.dark *){border-left-color:rgba(153,246,228,.6)}.dark\:border-l-teal-200\/65:is(.dark *){border-left-color:rgba(153,246,228,.65)}.dark\:border-l-teal-200\/70:is(.dark *){border-left-color:rgba(153,246,228,.7)}.dark\:border-l-teal-200\/75:is(.dark *){border-left-color:rgba(153,246,228,.75)}.dark\:border-l-teal-200\/80:is(.dark *){border-left-color:rgba(153,246,228,.8)}.dark\:border-l-teal-200\/85:is(.dark *){border-left-color:rgba(153,246,228,.85)}.dark\:border-l-teal-200\/90:is(.dark *){border-left-color:rgba(153,246,228,.9)}.dark\:border-l-teal-200\/95:is(.dark *){border-left-color:rgba(153,246,228,.95)}.dark\:border-l-teal-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(45 212 191/var(--tw-border-opacity,1))}.dark\:border-l-teal-400\/0:is(.dark *){border-left-color:rgba(45,212,191,0)}.dark\:border-l-teal-400\/10:is(.dark *){border-left-color:rgba(45,212,191,.1)}.dark\:border-l-teal-400\/100:is(.dark *){border-left-color:#2dd4bf}.dark\:border-l-teal-400\/15:is(.dark *){border-left-color:rgba(45,212,191,.15)}.dark\:border-l-teal-400\/20:is(.dark *){border-left-color:rgba(45,212,191,.2)}.dark\:border-l-teal-400\/25:is(.dark *){border-left-color:rgba(45,212,191,.25)}.dark\:border-l-teal-400\/30:is(.dark *){border-left-color:rgba(45,212,191,.3)}.dark\:border-l-teal-400\/35:is(.dark *){border-left-color:rgba(45,212,191,.35)}.dark\:border-l-teal-400\/40:is(.dark *){border-left-color:rgba(45,212,191,.4)}.dark\:border-l-teal-400\/45:is(.dark *){border-left-color:rgba(45,212,191,.45)}.dark\:border-l-teal-400\/5:is(.dark *){border-left-color:rgba(45,212,191,.05)}.dark\:border-l-teal-400\/50:is(.dark *){border-left-color:rgba(45,212,191,.5)}.dark\:border-l-teal-400\/55:is(.dark *){border-left-color:rgba(45,212,191,.55)}.dark\:border-l-teal-400\/60:is(.dark *){border-left-color:rgba(45,212,191,.6)}.dark\:border-l-teal-400\/65:is(.dark *){border-left-color:rgba(45,212,191,.65)}.dark\:border-l-teal-400\/70:is(.dark *){border-left-color:rgba(45,212,191,.7)}.dark\:border-l-teal-400\/75:is(.dark *){border-left-color:rgba(45,212,191,.75)}.dark\:border-l-teal-400\/80:is(.dark *){border-left-color:rgba(45,212,191,.8)}.dark\:border-l-teal-400\/85:is(.dark *){border-left-color:rgba(45,212,191,.85)}.dark\:border-l-teal-400\/90:is(.dark *){border-left-color:rgba(45,212,191,.9)}.dark\:border-l-teal-400\/95:is(.dark *){border-left-color:rgba(45,212,191,.95)}.dark\:border-l-teal-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(240 253 250/var(--tw-border-opacity,1))}.dark\:border-l-teal-50\/0:is(.dark *){border-left-color:rgba(240,253,250,0)}.dark\:border-l-teal-50\/10:is(.dark *){border-left-color:rgba(240,253,250,.1)}.dark\:border-l-teal-50\/100:is(.dark *){border-left-color:#f0fdfa}.dark\:border-l-teal-50\/15:is(.dark *){border-left-color:rgba(240,253,250,.15)}.dark\:border-l-teal-50\/20:is(.dark *){border-left-color:rgba(240,253,250,.2)}.dark\:border-l-teal-50\/25:is(.dark *){border-left-color:rgba(240,253,250,.25)}.dark\:border-l-teal-50\/30:is(.dark *){border-left-color:rgba(240,253,250,.3)}.dark\:border-l-teal-50\/35:is(.dark *){border-left-color:rgba(240,253,250,.35)}.dark\:border-l-teal-50\/40:is(.dark *){border-left-color:rgba(240,253,250,.4)}.dark\:border-l-teal-50\/45:is(.dark *){border-left-color:rgba(240,253,250,.45)}.dark\:border-l-teal-50\/5:is(.dark *){border-left-color:rgba(240,253,250,.05)}.dark\:border-l-teal-50\/50:is(.dark *){border-left-color:rgba(240,253,250,.5)}.dark\:border-l-teal-50\/55:is(.dark *){border-left-color:rgba(240,253,250,.55)}.dark\:border-l-teal-50\/60:is(.dark *){border-left-color:rgba(240,253,250,.6)}.dark\:border-l-teal-50\/65:is(.dark *){border-left-color:rgba(240,253,250,.65)}.dark\:border-l-teal-50\/70:is(.dark *){border-left-color:rgba(240,253,250,.7)}.dark\:border-l-teal-50\/75:is(.dark *){border-left-color:rgba(240,253,250,.75)}.dark\:border-l-teal-50\/80:is(.dark *){border-left-color:rgba(240,253,250,.8)}.dark\:border-l-teal-50\/85:is(.dark *){border-left-color:rgba(240,253,250,.85)}.dark\:border-l-teal-50\/90:is(.dark *){border-left-color:rgba(240,253,250,.9)}.dark\:border-l-teal-50\/95:is(.dark *){border-left-color:rgba(240,253,250,.95)}.dark\:border-l-teal-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(20 184 166/var(--tw-border-opacity,1))}.dark\:border-l-teal-500\/0:is(.dark *){border-left-color:rgba(20,184,166,0)}.dark\:border-l-teal-500\/10:is(.dark *){border-left-color:rgba(20,184,166,.1)}.dark\:border-l-teal-500\/100:is(.dark *){border-left-color:#14b8a6}.dark\:border-l-teal-500\/15:is(.dark *){border-left-color:rgba(20,184,166,.15)}.dark\:border-l-teal-500\/20:is(.dark *){border-left-color:rgba(20,184,166,.2)}.dark\:border-l-teal-500\/25:is(.dark *){border-left-color:rgba(20,184,166,.25)}.dark\:border-l-teal-500\/30:is(.dark *){border-left-color:rgba(20,184,166,.3)}.dark\:border-l-teal-500\/35:is(.dark *){border-left-color:rgba(20,184,166,.35)}.dark\:border-l-teal-500\/40:is(.dark *){border-left-color:rgba(20,184,166,.4)}.dark\:border-l-teal-500\/45:is(.dark *){border-left-color:rgba(20,184,166,.45)}.dark\:border-l-teal-500\/5:is(.dark *){border-left-color:rgba(20,184,166,.05)}.dark\:border-l-teal-500\/50:is(.dark *){border-left-color:rgba(20,184,166,.5)}.dark\:border-l-teal-500\/55:is(.dark *){border-left-color:rgba(20,184,166,.55)}.dark\:border-l-teal-500\/60:is(.dark *){border-left-color:rgba(20,184,166,.6)}.dark\:border-l-teal-500\/65:is(.dark *){border-left-color:rgba(20,184,166,.65)}.dark\:border-l-teal-500\/70:is(.dark *){border-left-color:rgba(20,184,166,.7)}.dark\:border-l-teal-500\/75:is(.dark *){border-left-color:rgba(20,184,166,.75)}.dark\:border-l-teal-500\/80:is(.dark *){border-left-color:rgba(20,184,166,.8)}.dark\:border-l-teal-500\/85:is(.dark *){border-left-color:rgba(20,184,166,.85)}.dark\:border-l-teal-500\/90:is(.dark *){border-left-color:rgba(20,184,166,.9)}.dark\:border-l-teal-500\/95:is(.dark *){border-left-color:rgba(20,184,166,.95)}.dark\:border-l-teal-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(4 47 46/var(--tw-border-opacity,1))}.dark\:border-l-teal-950\/0:is(.dark *){border-left-color:rgba(4,47,46,0)}.dark\:border-l-teal-950\/10:is(.dark *){border-left-color:rgba(4,47,46,.1)}.dark\:border-l-teal-950\/100:is(.dark *){border-left-color:#042f2e}.dark\:border-l-teal-950\/15:is(.dark *){border-left-color:rgba(4,47,46,.15)}.dark\:border-l-teal-950\/20:is(.dark *){border-left-color:rgba(4,47,46,.2)}.dark\:border-l-teal-950\/25:is(.dark *){border-left-color:rgba(4,47,46,.25)}.dark\:border-l-teal-950\/30:is(.dark *){border-left-color:rgba(4,47,46,.3)}.dark\:border-l-teal-950\/35:is(.dark *){border-left-color:rgba(4,47,46,.35)}.dark\:border-l-teal-950\/40:is(.dark *){border-left-color:rgba(4,47,46,.4)}.dark\:border-l-teal-950\/45:is(.dark *){border-left-color:rgba(4,47,46,.45)}.dark\:border-l-teal-950\/5:is(.dark *){border-left-color:rgba(4,47,46,.05)}.dark\:border-l-teal-950\/50:is(.dark *){border-left-color:rgba(4,47,46,.5)}.dark\:border-l-teal-950\/55:is(.dark *){border-left-color:rgba(4,47,46,.55)}.dark\:border-l-teal-950\/60:is(.dark *){border-left-color:rgba(4,47,46,.6)}.dark\:border-l-teal-950\/65:is(.dark *){border-left-color:rgba(4,47,46,.65)}.dark\:border-l-teal-950\/70:is(.dark *){border-left-color:rgba(4,47,46,.7)}.dark\:border-l-teal-950\/75:is(.dark *){border-left-color:rgba(4,47,46,.75)}.dark\:border-l-teal-950\/80:is(.dark *){border-left-color:rgba(4,47,46,.8)}.dark\:border-l-teal-950\/85:is(.dark *){border-left-color:rgba(4,47,46,.85)}.dark\:border-l-teal-950\/90:is(.dark *){border-left-color:rgba(4,47,46,.9)}.dark\:border-l-teal-950\/95:is(.dark *){border-left-color:rgba(4,47,46,.95)}.dark\:border-l-violet-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(237 233 254/var(--tw-border-opacity,1))}.dark\:border-l-violet-100\/0:is(.dark *){border-left-color:rgba(237,233,254,0)}.dark\:border-l-violet-100\/10:is(.dark *){border-left-color:rgba(237,233,254,.1)}.dark\:border-l-violet-100\/100:is(.dark *){border-left-color:#ede9fe}.dark\:border-l-violet-100\/15:is(.dark *){border-left-color:rgba(237,233,254,.15)}.dark\:border-l-violet-100\/20:is(.dark *){border-left-color:rgba(237,233,254,.2)}.dark\:border-l-violet-100\/25:is(.dark *){border-left-color:rgba(237,233,254,.25)}.dark\:border-l-violet-100\/30:is(.dark *){border-left-color:rgba(237,233,254,.3)}.dark\:border-l-violet-100\/35:is(.dark *){border-left-color:rgba(237,233,254,.35)}.dark\:border-l-violet-100\/40:is(.dark *){border-left-color:rgba(237,233,254,.4)}.dark\:border-l-violet-100\/45:is(.dark *){border-left-color:rgba(237,233,254,.45)}.dark\:border-l-violet-100\/5:is(.dark *){border-left-color:rgba(237,233,254,.05)}.dark\:border-l-violet-100\/50:is(.dark *){border-left-color:rgba(237,233,254,.5)}.dark\:border-l-violet-100\/55:is(.dark *){border-left-color:rgba(237,233,254,.55)}.dark\:border-l-violet-100\/60:is(.dark *){border-left-color:rgba(237,233,254,.6)}.dark\:border-l-violet-100\/65:is(.dark *){border-left-color:rgba(237,233,254,.65)}.dark\:border-l-violet-100\/70:is(.dark *){border-left-color:rgba(237,233,254,.7)}.dark\:border-l-violet-100\/75:is(.dark *){border-left-color:rgba(237,233,254,.75)}.dark\:border-l-violet-100\/80:is(.dark *){border-left-color:rgba(237,233,254,.8)}.dark\:border-l-violet-100\/85:is(.dark *){border-left-color:rgba(237,233,254,.85)}.dark\:border-l-violet-100\/90:is(.dark *){border-left-color:rgba(237,233,254,.9)}.dark\:border-l-violet-100\/95:is(.dark *){border-left-color:rgba(237,233,254,.95)}.dark\:border-l-violet-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(221 214 254/var(--tw-border-opacity,1))}.dark\:border-l-violet-200\/0:is(.dark *){border-left-color:rgba(221,214,254,0)}.dark\:border-l-violet-200\/10:is(.dark *){border-left-color:rgba(221,214,254,.1)}.dark\:border-l-violet-200\/100:is(.dark *){border-left-color:#ddd6fe}.dark\:border-l-violet-200\/15:is(.dark *){border-left-color:rgba(221,214,254,.15)}.dark\:border-l-violet-200\/20:is(.dark *){border-left-color:rgba(221,214,254,.2)}.dark\:border-l-violet-200\/25:is(.dark *){border-left-color:rgba(221,214,254,.25)}.dark\:border-l-violet-200\/30:is(.dark *){border-left-color:rgba(221,214,254,.3)}.dark\:border-l-violet-200\/35:is(.dark *){border-left-color:rgba(221,214,254,.35)}.dark\:border-l-violet-200\/40:is(.dark *){border-left-color:rgba(221,214,254,.4)}.dark\:border-l-violet-200\/45:is(.dark *){border-left-color:rgba(221,214,254,.45)}.dark\:border-l-violet-200\/5:is(.dark *){border-left-color:rgba(221,214,254,.05)}.dark\:border-l-violet-200\/50:is(.dark *){border-left-color:rgba(221,214,254,.5)}.dark\:border-l-violet-200\/55:is(.dark *){border-left-color:rgba(221,214,254,.55)}.dark\:border-l-violet-200\/60:is(.dark *){border-left-color:rgba(221,214,254,.6)}.dark\:border-l-violet-200\/65:is(.dark *){border-left-color:rgba(221,214,254,.65)}.dark\:border-l-violet-200\/70:is(.dark *){border-left-color:rgba(221,214,254,.7)}.dark\:border-l-violet-200\/75:is(.dark *){border-left-color:rgba(221,214,254,.75)}.dark\:border-l-violet-200\/80:is(.dark *){border-left-color:rgba(221,214,254,.8)}.dark\:border-l-violet-200\/85:is(.dark *){border-left-color:rgba(221,214,254,.85)}.dark\:border-l-violet-200\/90:is(.dark *){border-left-color:rgba(221,214,254,.9)}.dark\:border-l-violet-200\/95:is(.dark *){border-left-color:rgba(221,214,254,.95)}.dark\:border-l-violet-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(167 139 250/var(--tw-border-opacity,1))}.dark\:border-l-violet-400\/0:is(.dark *){border-left-color:rgba(167,139,250,0)}.dark\:border-l-violet-400\/10:is(.dark *){border-left-color:rgba(167,139,250,.1)}.dark\:border-l-violet-400\/100:is(.dark *){border-left-color:#a78bfa}.dark\:border-l-violet-400\/15:is(.dark *){border-left-color:rgba(167,139,250,.15)}.dark\:border-l-violet-400\/20:is(.dark *){border-left-color:rgba(167,139,250,.2)}.dark\:border-l-violet-400\/25:is(.dark *){border-left-color:rgba(167,139,250,.25)}.dark\:border-l-violet-400\/30:is(.dark *){border-left-color:rgba(167,139,250,.3)}.dark\:border-l-violet-400\/35:is(.dark *){border-left-color:rgba(167,139,250,.35)}.dark\:border-l-violet-400\/40:is(.dark *){border-left-color:rgba(167,139,250,.4)}.dark\:border-l-violet-400\/45:is(.dark *){border-left-color:rgba(167,139,250,.45)}.dark\:border-l-violet-400\/5:is(.dark *){border-left-color:rgba(167,139,250,.05)}.dark\:border-l-violet-400\/50:is(.dark *){border-left-color:rgba(167,139,250,.5)}.dark\:border-l-violet-400\/55:is(.dark *){border-left-color:rgba(167,139,250,.55)}.dark\:border-l-violet-400\/60:is(.dark *){border-left-color:rgba(167,139,250,.6)}.dark\:border-l-violet-400\/65:is(.dark *){border-left-color:rgba(167,139,250,.65)}.dark\:border-l-violet-400\/70:is(.dark *){border-left-color:rgba(167,139,250,.7)}.dark\:border-l-violet-400\/75:is(.dark *){border-left-color:rgba(167,139,250,.75)}.dark\:border-l-violet-400\/80:is(.dark *){border-left-color:rgba(167,139,250,.8)}.dark\:border-l-violet-400\/85:is(.dark *){border-left-color:rgba(167,139,250,.85)}.dark\:border-l-violet-400\/90:is(.dark *){border-left-color:rgba(167,139,250,.9)}.dark\:border-l-violet-400\/95:is(.dark *){border-left-color:rgba(167,139,250,.95)}.dark\:border-l-violet-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(245 243 255/var(--tw-border-opacity,1))}.dark\:border-l-violet-50\/0:is(.dark *){border-left-color:rgba(245,243,255,0)}.dark\:border-l-violet-50\/10:is(.dark *){border-left-color:rgba(245,243,255,.1)}.dark\:border-l-violet-50\/100:is(.dark *){border-left-color:#f5f3ff}.dark\:border-l-violet-50\/15:is(.dark *){border-left-color:rgba(245,243,255,.15)}.dark\:border-l-violet-50\/20:is(.dark *){border-left-color:rgba(245,243,255,.2)}.dark\:border-l-violet-50\/25:is(.dark *){border-left-color:rgba(245,243,255,.25)}.dark\:border-l-violet-50\/30:is(.dark *){border-left-color:rgba(245,243,255,.3)}.dark\:border-l-violet-50\/35:is(.dark *){border-left-color:rgba(245,243,255,.35)}.dark\:border-l-violet-50\/40:is(.dark *){border-left-color:rgba(245,243,255,.4)}.dark\:border-l-violet-50\/45:is(.dark *){border-left-color:rgba(245,243,255,.45)}.dark\:border-l-violet-50\/5:is(.dark *){border-left-color:rgba(245,243,255,.05)}.dark\:border-l-violet-50\/50:is(.dark *){border-left-color:rgba(245,243,255,.5)}.dark\:border-l-violet-50\/55:is(.dark *){border-left-color:rgba(245,243,255,.55)}.dark\:border-l-violet-50\/60:is(.dark *){border-left-color:rgba(245,243,255,.6)}.dark\:border-l-violet-50\/65:is(.dark *){border-left-color:rgba(245,243,255,.65)}.dark\:border-l-violet-50\/70:is(.dark *){border-left-color:rgba(245,243,255,.7)}.dark\:border-l-violet-50\/75:is(.dark *){border-left-color:rgba(245,243,255,.75)}.dark\:border-l-violet-50\/80:is(.dark *){border-left-color:rgba(245,243,255,.8)}.dark\:border-l-violet-50\/85:is(.dark *){border-left-color:rgba(245,243,255,.85)}.dark\:border-l-violet-50\/90:is(.dark *){border-left-color:rgba(245,243,255,.9)}.dark\:border-l-violet-50\/95:is(.dark *){border-left-color:rgba(245,243,255,.95)}.dark\:border-l-violet-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(139 92 246/var(--tw-border-opacity,1))}.dark\:border-l-violet-500\/0:is(.dark *){border-left-color:rgba(139,92,246,0)}.dark\:border-l-violet-500\/10:is(.dark *){border-left-color:rgba(139,92,246,.1)}.dark\:border-l-violet-500\/100:is(.dark *){border-left-color:#8b5cf6}.dark\:border-l-violet-500\/15:is(.dark *){border-left-color:rgba(139,92,246,.15)}.dark\:border-l-violet-500\/20:is(.dark *){border-left-color:rgba(139,92,246,.2)}.dark\:border-l-violet-500\/25:is(.dark *){border-left-color:rgba(139,92,246,.25)}.dark\:border-l-violet-500\/30:is(.dark *){border-left-color:rgba(139,92,246,.3)}.dark\:border-l-violet-500\/35:is(.dark *){border-left-color:rgba(139,92,246,.35)}.dark\:border-l-violet-500\/40:is(.dark *){border-left-color:rgba(139,92,246,.4)}.dark\:border-l-violet-500\/45:is(.dark *){border-left-color:rgba(139,92,246,.45)}.dark\:border-l-violet-500\/5:is(.dark *){border-left-color:rgba(139,92,246,.05)}.dark\:border-l-violet-500\/50:is(.dark *){border-left-color:rgba(139,92,246,.5)}.dark\:border-l-violet-500\/55:is(.dark *){border-left-color:rgba(139,92,246,.55)}.dark\:border-l-violet-500\/60:is(.dark *){border-left-color:rgba(139,92,246,.6)}.dark\:border-l-violet-500\/65:is(.dark *){border-left-color:rgba(139,92,246,.65)}.dark\:border-l-violet-500\/70:is(.dark *){border-left-color:rgba(139,92,246,.7)}.dark\:border-l-violet-500\/75:is(.dark *){border-left-color:rgba(139,92,246,.75)}.dark\:border-l-violet-500\/80:is(.dark *){border-left-color:rgba(139,92,246,.8)}.dark\:border-l-violet-500\/85:is(.dark *){border-left-color:rgba(139,92,246,.85)}.dark\:border-l-violet-500\/90:is(.dark *){border-left-color:rgba(139,92,246,.9)}.dark\:border-l-violet-500\/95:is(.dark *){border-left-color:rgba(139,92,246,.95)}.dark\:border-l-violet-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(46 16 101/var(--tw-border-opacity,1))}.dark\:border-l-violet-950\/0:is(.dark *){border-left-color:rgba(46,16,101,0)}.dark\:border-l-violet-950\/10:is(.dark *){border-left-color:rgba(46,16,101,.1)}.dark\:border-l-violet-950\/100:is(.dark *){border-left-color:#2e1065}.dark\:border-l-violet-950\/15:is(.dark *){border-left-color:rgba(46,16,101,.15)}.dark\:border-l-violet-950\/20:is(.dark *){border-left-color:rgba(46,16,101,.2)}.dark\:border-l-violet-950\/25:is(.dark *){border-left-color:rgba(46,16,101,.25)}.dark\:border-l-violet-950\/30:is(.dark *){border-left-color:rgba(46,16,101,.3)}.dark\:border-l-violet-950\/35:is(.dark *){border-left-color:rgba(46,16,101,.35)}.dark\:border-l-violet-950\/40:is(.dark *){border-left-color:rgba(46,16,101,.4)}.dark\:border-l-violet-950\/45:is(.dark *){border-left-color:rgba(46,16,101,.45)}.dark\:border-l-violet-950\/5:is(.dark *){border-left-color:rgba(46,16,101,.05)}.dark\:border-l-violet-950\/50:is(.dark *){border-left-color:rgba(46,16,101,.5)}.dark\:border-l-violet-950\/55:is(.dark *){border-left-color:rgba(46,16,101,.55)}.dark\:border-l-violet-950\/60:is(.dark *){border-left-color:rgba(46,16,101,.6)}.dark\:border-l-violet-950\/65:is(.dark *){border-left-color:rgba(46,16,101,.65)}.dark\:border-l-violet-950\/70:is(.dark *){border-left-color:rgba(46,16,101,.7)}.dark\:border-l-violet-950\/75:is(.dark *){border-left-color:rgba(46,16,101,.75)}.dark\:border-l-violet-950\/80:is(.dark *){border-left-color:rgba(46,16,101,.8)}.dark\:border-l-violet-950\/85:is(.dark *){border-left-color:rgba(46,16,101,.85)}.dark\:border-l-violet-950\/90:is(.dark *){border-left-color:rgba(46,16,101,.9)}.dark\:border-l-violet-950\/95:is(.dark *){border-left-color:rgba(46,16,101,.95)}.dark\:border-l-yellow-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(254 249 195/var(--tw-border-opacity,1))}.dark\:border-l-yellow-100\/0:is(.dark *){border-left-color:hsla(55,97%,88%,0)}.dark\:border-l-yellow-100\/10:is(.dark *){border-left-color:hsla(55,97%,88%,.1)}.dark\:border-l-yellow-100\/100:is(.dark *){border-left-color:#fef9c3}.dark\:border-l-yellow-100\/15:is(.dark *){border-left-color:hsla(55,97%,88%,.15)}.dark\:border-l-yellow-100\/20:is(.dark *){border-left-color:hsla(55,97%,88%,.2)}.dark\:border-l-yellow-100\/25:is(.dark *){border-left-color:hsla(55,97%,88%,.25)}.dark\:border-l-yellow-100\/30:is(.dark *){border-left-color:hsla(55,97%,88%,.3)}.dark\:border-l-yellow-100\/35:is(.dark *){border-left-color:hsla(55,97%,88%,.35)}.dark\:border-l-yellow-100\/40:is(.dark *){border-left-color:hsla(55,97%,88%,.4)}.dark\:border-l-yellow-100\/45:is(.dark *){border-left-color:hsla(55,97%,88%,.45)}.dark\:border-l-yellow-100\/5:is(.dark *){border-left-color:hsla(55,97%,88%,.05)}.dark\:border-l-yellow-100\/50:is(.dark *){border-left-color:hsla(55,97%,88%,.5)}.dark\:border-l-yellow-100\/55:is(.dark *){border-left-color:hsla(55,97%,88%,.55)}.dark\:border-l-yellow-100\/60:is(.dark *){border-left-color:hsla(55,97%,88%,.6)}.dark\:border-l-yellow-100\/65:is(.dark *){border-left-color:hsla(55,97%,88%,.65)}.dark\:border-l-yellow-100\/70:is(.dark *){border-left-color:hsla(55,97%,88%,.7)}.dark\:border-l-yellow-100\/75:is(.dark *){border-left-color:hsla(55,97%,88%,.75)}.dark\:border-l-yellow-100\/80:is(.dark *){border-left-color:hsla(55,97%,88%,.8)}.dark\:border-l-yellow-100\/85:is(.dark *){border-left-color:hsla(55,97%,88%,.85)}.dark\:border-l-yellow-100\/90:is(.dark *){border-left-color:hsla(55,97%,88%,.9)}.dark\:border-l-yellow-100\/95:is(.dark *){border-left-color:hsla(55,97%,88%,.95)}.dark\:border-l-yellow-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(254 240 138/var(--tw-border-opacity,1))}.dark\:border-l-yellow-200\/0:is(.dark *){border-left-color:hsla(53,98%,77%,0)}.dark\:border-l-yellow-200\/10:is(.dark *){border-left-color:hsla(53,98%,77%,.1)}.dark\:border-l-yellow-200\/100:is(.dark *){border-left-color:#fef08a}.dark\:border-l-yellow-200\/15:is(.dark *){border-left-color:hsla(53,98%,77%,.15)}.dark\:border-l-yellow-200\/20:is(.dark *){border-left-color:hsla(53,98%,77%,.2)}.dark\:border-l-yellow-200\/25:is(.dark *){border-left-color:hsla(53,98%,77%,.25)}.dark\:border-l-yellow-200\/30:is(.dark *){border-left-color:hsla(53,98%,77%,.3)}.dark\:border-l-yellow-200\/35:is(.dark *){border-left-color:hsla(53,98%,77%,.35)}.dark\:border-l-yellow-200\/40:is(.dark *){border-left-color:hsla(53,98%,77%,.4)}.dark\:border-l-yellow-200\/45:is(.dark *){border-left-color:hsla(53,98%,77%,.45)}.dark\:border-l-yellow-200\/5:is(.dark *){border-left-color:hsla(53,98%,77%,.05)}.dark\:border-l-yellow-200\/50:is(.dark *){border-left-color:hsla(53,98%,77%,.5)}.dark\:border-l-yellow-200\/55:is(.dark *){border-left-color:hsla(53,98%,77%,.55)}.dark\:border-l-yellow-200\/60:is(.dark *){border-left-color:hsla(53,98%,77%,.6)}.dark\:border-l-yellow-200\/65:is(.dark *){border-left-color:hsla(53,98%,77%,.65)}.dark\:border-l-yellow-200\/70:is(.dark *){border-left-color:hsla(53,98%,77%,.7)}.dark\:border-l-yellow-200\/75:is(.dark *){border-left-color:hsla(53,98%,77%,.75)}.dark\:border-l-yellow-200\/80:is(.dark *){border-left-color:hsla(53,98%,77%,.8)}.dark\:border-l-yellow-200\/85:is(.dark *){border-left-color:hsla(53,98%,77%,.85)}.dark\:border-l-yellow-200\/90:is(.dark *){border-left-color:hsla(53,98%,77%,.9)}.dark\:border-l-yellow-200\/95:is(.dark *){border-left-color:hsla(53,98%,77%,.95)}.dark\:border-l-yellow-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(250 204 21/var(--tw-border-opacity,1))}.dark\:border-l-yellow-400\/0:is(.dark *){border-left-color:rgba(250,204,21,0)}.dark\:border-l-yellow-400\/10:is(.dark *){border-left-color:rgba(250,204,21,.1)}.dark\:border-l-yellow-400\/100:is(.dark *){border-left-color:#facc15}.dark\:border-l-yellow-400\/15:is(.dark *){border-left-color:rgba(250,204,21,.15)}.dark\:border-l-yellow-400\/20:is(.dark *){border-left-color:rgba(250,204,21,.2)}.dark\:border-l-yellow-400\/25:is(.dark *){border-left-color:rgba(250,204,21,.25)}.dark\:border-l-yellow-400\/30:is(.dark *){border-left-color:rgba(250,204,21,.3)}.dark\:border-l-yellow-400\/35:is(.dark *){border-left-color:rgba(250,204,21,.35)}.dark\:border-l-yellow-400\/40:is(.dark *){border-left-color:rgba(250,204,21,.4)}.dark\:border-l-yellow-400\/45:is(.dark *){border-left-color:rgba(250,204,21,.45)}.dark\:border-l-yellow-400\/5:is(.dark *){border-left-color:rgba(250,204,21,.05)}.dark\:border-l-yellow-400\/50:is(.dark *){border-left-color:rgba(250,204,21,.5)}.dark\:border-l-yellow-400\/55:is(.dark *){border-left-color:rgba(250,204,21,.55)}.dark\:border-l-yellow-400\/60:is(.dark *){border-left-color:rgba(250,204,21,.6)}.dark\:border-l-yellow-400\/65:is(.dark *){border-left-color:rgba(250,204,21,.65)}.dark\:border-l-yellow-400\/70:is(.dark *){border-left-color:rgba(250,204,21,.7)}.dark\:border-l-yellow-400\/75:is(.dark *){border-left-color:rgba(250,204,21,.75)}.dark\:border-l-yellow-400\/80:is(.dark *){border-left-color:rgba(250,204,21,.8)}.dark\:border-l-yellow-400\/85:is(.dark *){border-left-color:rgba(250,204,21,.85)}.dark\:border-l-yellow-400\/90:is(.dark *){border-left-color:rgba(250,204,21,.9)}.dark\:border-l-yellow-400\/95:is(.dark *){border-left-color:rgba(250,204,21,.95)}.dark\:border-l-yellow-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(254 252 232/var(--tw-border-opacity,1))}.dark\:border-l-yellow-50\/0:is(.dark *){border-left-color:hsla(55,92%,95%,0)}.dark\:border-l-yellow-50\/10:is(.dark *){border-left-color:hsla(55,92%,95%,.1)}.dark\:border-l-yellow-50\/100:is(.dark *){border-left-color:#fefce8}.dark\:border-l-yellow-50\/15:is(.dark *){border-left-color:hsla(55,92%,95%,.15)}.dark\:border-l-yellow-50\/20:is(.dark *){border-left-color:hsla(55,92%,95%,.2)}.dark\:border-l-yellow-50\/25:is(.dark *){border-left-color:hsla(55,92%,95%,.25)}.dark\:border-l-yellow-50\/30:is(.dark *){border-left-color:hsla(55,92%,95%,.3)}.dark\:border-l-yellow-50\/35:is(.dark *){border-left-color:hsla(55,92%,95%,.35)}.dark\:border-l-yellow-50\/40:is(.dark *){border-left-color:hsla(55,92%,95%,.4)}.dark\:border-l-yellow-50\/45:is(.dark *){border-left-color:hsla(55,92%,95%,.45)}.dark\:border-l-yellow-50\/5:is(.dark *){border-left-color:hsla(55,92%,95%,.05)}.dark\:border-l-yellow-50\/50:is(.dark *){border-left-color:hsla(55,92%,95%,.5)}.dark\:border-l-yellow-50\/55:is(.dark *){border-left-color:hsla(55,92%,95%,.55)}.dark\:border-l-yellow-50\/60:is(.dark *){border-left-color:hsla(55,92%,95%,.6)}.dark\:border-l-yellow-50\/65:is(.dark *){border-left-color:hsla(55,92%,95%,.65)}.dark\:border-l-yellow-50\/70:is(.dark *){border-left-color:hsla(55,92%,95%,.7)}.dark\:border-l-yellow-50\/75:is(.dark *){border-left-color:hsla(55,92%,95%,.75)}.dark\:border-l-yellow-50\/80:is(.dark *){border-left-color:hsla(55,92%,95%,.8)}.dark\:border-l-yellow-50\/85:is(.dark *){border-left-color:hsla(55,92%,95%,.85)}.dark\:border-l-yellow-50\/90:is(.dark *){border-left-color:hsla(55,92%,95%,.9)}.dark\:border-l-yellow-50\/95:is(.dark *){border-left-color:hsla(55,92%,95%,.95)}.dark\:border-l-yellow-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(234 179 8/var(--tw-border-opacity,1))}.dark\:border-l-yellow-500\/0:is(.dark *){border-left-color:rgba(234,179,8,0)}.dark\:border-l-yellow-500\/10:is(.dark *){border-left-color:rgba(234,179,8,.1)}.dark\:border-l-yellow-500\/100:is(.dark *){border-left-color:#eab308}.dark\:border-l-yellow-500\/15:is(.dark *){border-left-color:rgba(234,179,8,.15)}.dark\:border-l-yellow-500\/20:is(.dark *){border-left-color:rgba(234,179,8,.2)}.dark\:border-l-yellow-500\/25:is(.dark *){border-left-color:rgba(234,179,8,.25)}.dark\:border-l-yellow-500\/30:is(.dark *){border-left-color:rgba(234,179,8,.3)}.dark\:border-l-yellow-500\/35:is(.dark *){border-left-color:rgba(234,179,8,.35)}.dark\:border-l-yellow-500\/40:is(.dark *){border-left-color:rgba(234,179,8,.4)}.dark\:border-l-yellow-500\/45:is(.dark *){border-left-color:rgba(234,179,8,.45)}.dark\:border-l-yellow-500\/5:is(.dark *){border-left-color:rgba(234,179,8,.05)}.dark\:border-l-yellow-500\/50:is(.dark *){border-left-color:rgba(234,179,8,.5)}.dark\:border-l-yellow-500\/55:is(.dark *){border-left-color:rgba(234,179,8,.55)}.dark\:border-l-yellow-500\/60:is(.dark *){border-left-color:rgba(234,179,8,.6)}.dark\:border-l-yellow-500\/65:is(.dark *){border-left-color:rgba(234,179,8,.65)}.dark\:border-l-yellow-500\/70:is(.dark *){border-left-color:rgba(234,179,8,.7)}.dark\:border-l-yellow-500\/75:is(.dark *){border-left-color:rgba(234,179,8,.75)}.dark\:border-l-yellow-500\/80:is(.dark *){border-left-color:rgba(234,179,8,.8)}.dark\:border-l-yellow-500\/85:is(.dark *){border-left-color:rgba(234,179,8,.85)}.dark\:border-l-yellow-500\/90:is(.dark *){border-left-color:rgba(234,179,8,.9)}.dark\:border-l-yellow-500\/95:is(.dark *){border-left-color:rgba(234,179,8,.95)}.dark\:border-l-yellow-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(66 32 6/var(--tw-border-opacity,1))}.dark\:border-l-yellow-950\/0:is(.dark *){border-left-color:rgba(66,32,6,0)}.dark\:border-l-yellow-950\/10:is(.dark *){border-left-color:rgba(66,32,6,.1)}.dark\:border-l-yellow-950\/100:is(.dark *){border-left-color:#422006}.dark\:border-l-yellow-950\/15:is(.dark *){border-left-color:rgba(66,32,6,.15)}.dark\:border-l-yellow-950\/20:is(.dark *){border-left-color:rgba(66,32,6,.2)}.dark\:border-l-yellow-950\/25:is(.dark *){border-left-color:rgba(66,32,6,.25)}.dark\:border-l-yellow-950\/30:is(.dark *){border-left-color:rgba(66,32,6,.3)}.dark\:border-l-yellow-950\/35:is(.dark *){border-left-color:rgba(66,32,6,.35)}.dark\:border-l-yellow-950\/40:is(.dark *){border-left-color:rgba(66,32,6,.4)}.dark\:border-l-yellow-950\/45:is(.dark *){border-left-color:rgba(66,32,6,.45)}.dark\:border-l-yellow-950\/5:is(.dark *){border-left-color:rgba(66,32,6,.05)}.dark\:border-l-yellow-950\/50:is(.dark *){border-left-color:rgba(66,32,6,.5)}.dark\:border-l-yellow-950\/55:is(.dark *){border-left-color:rgba(66,32,6,.55)}.dark\:border-l-yellow-950\/60:is(.dark *){border-left-color:rgba(66,32,6,.6)}.dark\:border-l-yellow-950\/65:is(.dark *){border-left-color:rgba(66,32,6,.65)}.dark\:border-l-yellow-950\/70:is(.dark *){border-left-color:rgba(66,32,6,.7)}.dark\:border-l-yellow-950\/75:is(.dark *){border-left-color:rgba(66,32,6,.75)}.dark\:border-l-yellow-950\/80:is(.dark *){border-left-color:rgba(66,32,6,.8)}.dark\:border-l-yellow-950\/85:is(.dark *){border-left-color:rgba(66,32,6,.85)}.dark\:border-l-yellow-950\/90:is(.dark *){border-left-color:rgba(66,32,6,.9)}.dark\:border-l-yellow-950\/95:is(.dark *){border-left-color:rgba(66,32,6,.95)}.dark\:border-l-zinc-100:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(244 244 245/var(--tw-border-opacity,1))}.dark\:border-l-zinc-100\/0:is(.dark *){border-left-color:hsla(240,5%,96%,0)}.dark\:border-l-zinc-100\/10:is(.dark *){border-left-color:hsla(240,5%,96%,.1)}.dark\:border-l-zinc-100\/100:is(.dark *){border-left-color:#f4f4f5}.dark\:border-l-zinc-100\/15:is(.dark *){border-left-color:hsla(240,5%,96%,.15)}.dark\:border-l-zinc-100\/20:is(.dark *){border-left-color:hsla(240,5%,96%,.2)}.dark\:border-l-zinc-100\/25:is(.dark *){border-left-color:hsla(240,5%,96%,.25)}.dark\:border-l-zinc-100\/30:is(.dark *){border-left-color:hsla(240,5%,96%,.3)}.dark\:border-l-zinc-100\/35:is(.dark *){border-left-color:hsla(240,5%,96%,.35)}.dark\:border-l-zinc-100\/40:is(.dark *){border-left-color:hsla(240,5%,96%,.4)}.dark\:border-l-zinc-100\/45:is(.dark *){border-left-color:hsla(240,5%,96%,.45)}.dark\:border-l-zinc-100\/5:is(.dark *){border-left-color:hsla(240,5%,96%,.05)}.dark\:border-l-zinc-100\/50:is(.dark *){border-left-color:hsla(240,5%,96%,.5)}.dark\:border-l-zinc-100\/55:is(.dark *){border-left-color:hsla(240,5%,96%,.55)}.dark\:border-l-zinc-100\/60:is(.dark *){border-left-color:hsla(240,5%,96%,.6)}.dark\:border-l-zinc-100\/65:is(.dark *){border-left-color:hsla(240,5%,96%,.65)}.dark\:border-l-zinc-100\/70:is(.dark *){border-left-color:hsla(240,5%,96%,.7)}.dark\:border-l-zinc-100\/75:is(.dark *){border-left-color:hsla(240,5%,96%,.75)}.dark\:border-l-zinc-100\/80:is(.dark *){border-left-color:hsla(240,5%,96%,.8)}.dark\:border-l-zinc-100\/85:is(.dark *){border-left-color:hsla(240,5%,96%,.85)}.dark\:border-l-zinc-100\/90:is(.dark *){border-left-color:hsla(240,5%,96%,.9)}.dark\:border-l-zinc-100\/95:is(.dark *){border-left-color:hsla(240,5%,96%,.95)}.dark\:border-l-zinc-200:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(228 228 231/var(--tw-border-opacity,1))}.dark\:border-l-zinc-200\/0:is(.dark *){border-left-color:hsla(240,6%,90%,0)}.dark\:border-l-zinc-200\/10:is(.dark *){border-left-color:hsla(240,6%,90%,.1)}.dark\:border-l-zinc-200\/100:is(.dark *){border-left-color:#e4e4e7}.dark\:border-l-zinc-200\/15:is(.dark *){border-left-color:hsla(240,6%,90%,.15)}.dark\:border-l-zinc-200\/20:is(.dark *){border-left-color:hsla(240,6%,90%,.2)}.dark\:border-l-zinc-200\/25:is(.dark *){border-left-color:hsla(240,6%,90%,.25)}.dark\:border-l-zinc-200\/30:is(.dark *){border-left-color:hsla(240,6%,90%,.3)}.dark\:border-l-zinc-200\/35:is(.dark *){border-left-color:hsla(240,6%,90%,.35)}.dark\:border-l-zinc-200\/40:is(.dark *){border-left-color:hsla(240,6%,90%,.4)}.dark\:border-l-zinc-200\/45:is(.dark *){border-left-color:hsla(240,6%,90%,.45)}.dark\:border-l-zinc-200\/5:is(.dark *){border-left-color:hsla(240,6%,90%,.05)}.dark\:border-l-zinc-200\/50:is(.dark *){border-left-color:hsla(240,6%,90%,.5)}.dark\:border-l-zinc-200\/55:is(.dark *){border-left-color:hsla(240,6%,90%,.55)}.dark\:border-l-zinc-200\/60:is(.dark *){border-left-color:hsla(240,6%,90%,.6)}.dark\:border-l-zinc-200\/65:is(.dark *){border-left-color:hsla(240,6%,90%,.65)}.dark\:border-l-zinc-200\/70:is(.dark *){border-left-color:hsla(240,6%,90%,.7)}.dark\:border-l-zinc-200\/75:is(.dark *){border-left-color:hsla(240,6%,90%,.75)}.dark\:border-l-zinc-200\/80:is(.dark *){border-left-color:hsla(240,6%,90%,.8)}.dark\:border-l-zinc-200\/85:is(.dark *){border-left-color:hsla(240,6%,90%,.85)}.dark\:border-l-zinc-200\/90:is(.dark *){border-left-color:hsla(240,6%,90%,.9)}.dark\:border-l-zinc-200\/95:is(.dark *){border-left-color:hsla(240,6%,90%,.95)}.dark\:border-l-zinc-400:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(161 161 170/var(--tw-border-opacity,1))}.dark\:border-l-zinc-400\/0:is(.dark *){border-left-color:hsla(240,5%,65%,0)}.dark\:border-l-zinc-400\/10:is(.dark *){border-left-color:hsla(240,5%,65%,.1)}.dark\:border-l-zinc-400\/100:is(.dark *){border-left-color:#a1a1aa}.dark\:border-l-zinc-400\/15:is(.dark *){border-left-color:hsla(240,5%,65%,.15)}.dark\:border-l-zinc-400\/20:is(.dark *){border-left-color:hsla(240,5%,65%,.2)}.dark\:border-l-zinc-400\/25:is(.dark *){border-left-color:hsla(240,5%,65%,.25)}.dark\:border-l-zinc-400\/30:is(.dark *){border-left-color:hsla(240,5%,65%,.3)}.dark\:border-l-zinc-400\/35:is(.dark *){border-left-color:hsla(240,5%,65%,.35)}.dark\:border-l-zinc-400\/40:is(.dark *){border-left-color:hsla(240,5%,65%,.4)}.dark\:border-l-zinc-400\/45:is(.dark *){border-left-color:hsla(240,5%,65%,.45)}.dark\:border-l-zinc-400\/5:is(.dark *){border-left-color:hsla(240,5%,65%,.05)}.dark\:border-l-zinc-400\/50:is(.dark *){border-left-color:hsla(240,5%,65%,.5)}.dark\:border-l-zinc-400\/55:is(.dark *){border-left-color:hsla(240,5%,65%,.55)}.dark\:border-l-zinc-400\/60:is(.dark *){border-left-color:hsla(240,5%,65%,.6)}.dark\:border-l-zinc-400\/65:is(.dark *){border-left-color:hsla(240,5%,65%,.65)}.dark\:border-l-zinc-400\/70:is(.dark *){border-left-color:hsla(240,5%,65%,.7)}.dark\:border-l-zinc-400\/75:is(.dark *){border-left-color:hsla(240,5%,65%,.75)}.dark\:border-l-zinc-400\/80:is(.dark *){border-left-color:hsla(240,5%,65%,.8)}.dark\:border-l-zinc-400\/85:is(.dark *){border-left-color:hsla(240,5%,65%,.85)}.dark\:border-l-zinc-400\/90:is(.dark *){border-left-color:hsla(240,5%,65%,.9)}.dark\:border-l-zinc-400\/95:is(.dark *){border-left-color:hsla(240,5%,65%,.95)}.dark\:border-l-zinc-50:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(250 250 250/var(--tw-border-opacity,1))}.dark\:border-l-zinc-50\/0:is(.dark *){border-left-color:hsla(0,0%,98%,0)}.dark\:border-l-zinc-50\/10:is(.dark *){border-left-color:hsla(0,0%,98%,.1)}.dark\:border-l-zinc-50\/100:is(.dark *){border-left-color:#fafafa}.dark\:border-l-zinc-50\/15:is(.dark *){border-left-color:hsla(0,0%,98%,.15)}.dark\:border-l-zinc-50\/20:is(.dark *){border-left-color:hsla(0,0%,98%,.2)}.dark\:border-l-zinc-50\/25:is(.dark *){border-left-color:hsla(0,0%,98%,.25)}.dark\:border-l-zinc-50\/30:is(.dark *){border-left-color:hsla(0,0%,98%,.3)}.dark\:border-l-zinc-50\/35:is(.dark *){border-left-color:hsla(0,0%,98%,.35)}.dark\:border-l-zinc-50\/40:is(.dark *){border-left-color:hsla(0,0%,98%,.4)}.dark\:border-l-zinc-50\/45:is(.dark *){border-left-color:hsla(0,0%,98%,.45)}.dark\:border-l-zinc-50\/5:is(.dark *){border-left-color:hsla(0,0%,98%,.05)}.dark\:border-l-zinc-50\/50:is(.dark *){border-left-color:hsla(0,0%,98%,.5)}.dark\:border-l-zinc-50\/55:is(.dark *){border-left-color:hsla(0,0%,98%,.55)}.dark\:border-l-zinc-50\/60:is(.dark *){border-left-color:hsla(0,0%,98%,.6)}.dark\:border-l-zinc-50\/65:is(.dark *){border-left-color:hsla(0,0%,98%,.65)}.dark\:border-l-zinc-50\/70:is(.dark *){border-left-color:hsla(0,0%,98%,.7)}.dark\:border-l-zinc-50\/75:is(.dark *){border-left-color:hsla(0,0%,98%,.75)}.dark\:border-l-zinc-50\/80:is(.dark *){border-left-color:hsla(0,0%,98%,.8)}.dark\:border-l-zinc-50\/85:is(.dark *){border-left-color:hsla(0,0%,98%,.85)}.dark\:border-l-zinc-50\/90:is(.dark *){border-left-color:hsla(0,0%,98%,.9)}.dark\:border-l-zinc-50\/95:is(.dark *){border-left-color:hsla(0,0%,98%,.95)}.dark\:border-l-zinc-500:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(113 113 122/var(--tw-border-opacity,1))}.dark\:border-l-zinc-500\/0:is(.dark *){border-left-color:hsla(240,4%,46%,0)}.dark\:border-l-zinc-500\/10:is(.dark *){border-left-color:hsla(240,4%,46%,.1)}.dark\:border-l-zinc-500\/100:is(.dark *){border-left-color:#71717a}.dark\:border-l-zinc-500\/15:is(.dark *){border-left-color:hsla(240,4%,46%,.15)}.dark\:border-l-zinc-500\/20:is(.dark *){border-left-color:hsla(240,4%,46%,.2)}.dark\:border-l-zinc-500\/25:is(.dark *){border-left-color:hsla(240,4%,46%,.25)}.dark\:border-l-zinc-500\/30:is(.dark *){border-left-color:hsla(240,4%,46%,.3)}.dark\:border-l-zinc-500\/35:is(.dark *){border-left-color:hsla(240,4%,46%,.35)}.dark\:border-l-zinc-500\/40:is(.dark *){border-left-color:hsla(240,4%,46%,.4)}.dark\:border-l-zinc-500\/45:is(.dark *){border-left-color:hsla(240,4%,46%,.45)}.dark\:border-l-zinc-500\/5:is(.dark *){border-left-color:hsla(240,4%,46%,.05)}.dark\:border-l-zinc-500\/50:is(.dark *){border-left-color:hsla(240,4%,46%,.5)}.dark\:border-l-zinc-500\/55:is(.dark *){border-left-color:hsla(240,4%,46%,.55)}.dark\:border-l-zinc-500\/60:is(.dark *){border-left-color:hsla(240,4%,46%,.6)}.dark\:border-l-zinc-500\/65:is(.dark *){border-left-color:hsla(240,4%,46%,.65)}.dark\:border-l-zinc-500\/70:is(.dark *){border-left-color:hsla(240,4%,46%,.7)}.dark\:border-l-zinc-500\/75:is(.dark *){border-left-color:hsla(240,4%,46%,.75)}.dark\:border-l-zinc-500\/80:is(.dark *){border-left-color:hsla(240,4%,46%,.8)}.dark\:border-l-zinc-500\/85:is(.dark *){border-left-color:hsla(240,4%,46%,.85)}.dark\:border-l-zinc-500\/90:is(.dark *){border-left-color:hsla(240,4%,46%,.9)}.dark\:border-l-zinc-500\/95:is(.dark *){border-left-color:hsla(240,4%,46%,.95)}.dark\:border-l-zinc-950:is(.dark *){--tw-border-opacity:1;border-left-color:rgb(9 9 11/var(--tw-border-opacity,1))}.dark\:border-l-zinc-950\/0:is(.dark *){border-left-color:rgba(9,9,11,0)}.dark\:border-l-zinc-950\/10:is(.dark *){border-left-color:rgba(9,9,11,.1)}.dark\:border-l-zinc-950\/100:is(.dark *){border-left-color:#09090b}.dark\:border-l-zinc-950\/15:is(.dark *){border-left-color:rgba(9,9,11,.15)}.dark\:border-l-zinc-950\/20:is(.dark *){border-left-color:rgba(9,9,11,.2)}.dark\:border-l-zinc-950\/25:is(.dark *){border-left-color:rgba(9,9,11,.25)}.dark\:border-l-zinc-950\/30:is(.dark *){border-left-color:rgba(9,9,11,.3)}.dark\:border-l-zinc-950\/35:is(.dark *){border-left-color:rgba(9,9,11,.35)}.dark\:border-l-zinc-950\/40:is(.dark *){border-left-color:rgba(9,9,11,.4)}.dark\:border-l-zinc-950\/45:is(.dark *){border-left-color:rgba(9,9,11,.45)}.dark\:border-l-zinc-950\/5:is(.dark *){border-left-color:rgba(9,9,11,.05)}.dark\:border-l-zinc-950\/50:is(.dark *){border-left-color:rgba(9,9,11,.5)}.dark\:border-l-zinc-950\/55:is(.dark *){border-left-color:rgba(9,9,11,.55)}.dark\:border-l-zinc-950\/60:is(.dark *){border-left-color:rgba(9,9,11,.6)}.dark\:border-l-zinc-950\/65:is(.dark *){border-left-color:rgba(9,9,11,.65)}.dark\:border-l-zinc-950\/70:is(.dark *){border-left-color:rgba(9,9,11,.7)}.dark\:border-l-zinc-950\/75:is(.dark *){border-left-color:rgba(9,9,11,.75)}.dark\:border-l-zinc-950\/80:is(.dark *){border-left-color:rgba(9,9,11,.8)}.dark\:border-l-zinc-950\/85:is(.dark *){border-left-color:rgba(9,9,11,.85)}.dark\:border-l-zinc-950\/90:is(.dark *){border-left-color:rgba(9,9,11,.9)}.dark\:border-l-zinc-950\/95:is(.dark *){border-left-color:rgba(9,9,11,.95)}.dark\:bg-\[hsl\(var\(--background\)\)\]:is(.dark *){background-color:hsl(var(--background))}.dark\:bg-amber-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(254 243 199/var(--tw-bg-opacity,1))}.dark\:bg-amber-100\/0:is(.dark *){background-color:hsla(48,96%,89%,0)}.dark\:bg-amber-100\/10:is(.dark *){background-color:hsla(48,96%,89%,.1)}.dark\:bg-amber-100\/100:is(.dark *){background-color:#fef3c7}.dark\:bg-amber-100\/15:is(.dark *){background-color:hsla(48,96%,89%,.15)}.dark\:bg-amber-100\/20:is(.dark *){background-color:hsla(48,96%,89%,.2)}.dark\:bg-amber-100\/25:is(.dark *){background-color:hsla(48,96%,89%,.25)}.dark\:bg-amber-100\/30:is(.dark *){background-color:hsla(48,96%,89%,.3)}.dark\:bg-amber-100\/35:is(.dark *){background-color:hsla(48,96%,89%,.35)}.dark\:bg-amber-100\/40:is(.dark *){background-color:hsla(48,96%,89%,.4)}.dark\:bg-amber-100\/45:is(.dark *){background-color:hsla(48,96%,89%,.45)}.dark\:bg-amber-100\/5:is(.dark *){background-color:hsla(48,96%,89%,.05)}.dark\:bg-amber-100\/50:is(.dark *){background-color:hsla(48,96%,89%,.5)}.dark\:bg-amber-100\/55:is(.dark *){background-color:hsla(48,96%,89%,.55)}.dark\:bg-amber-100\/60:is(.dark *){background-color:hsla(48,96%,89%,.6)}.dark\:bg-amber-100\/65:is(.dark *){background-color:hsla(48,96%,89%,.65)}.dark\:bg-amber-100\/70:is(.dark *){background-color:hsla(48,96%,89%,.7)}.dark\:bg-amber-100\/75:is(.dark *){background-color:hsla(48,96%,89%,.75)}.dark\:bg-amber-100\/80:is(.dark *){background-color:hsla(48,96%,89%,.8)}.dark\:bg-amber-100\/85:is(.dark *){background-color:hsla(48,96%,89%,.85)}.dark\:bg-amber-100\/90:is(.dark *){background-color:hsla(48,96%,89%,.9)}.dark\:bg-amber-100\/95:is(.dark *){background-color:hsla(48,96%,89%,.95)}.dark\:bg-amber-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(253 230 138/var(--tw-bg-opacity,1))}.dark\:bg-amber-200\/0:is(.dark *){background-color:hsla(48,97%,77%,0)}.dark\:bg-amber-200\/10:is(.dark *){background-color:hsla(48,97%,77%,.1)}.dark\:bg-amber-200\/100:is(.dark *){background-color:#fde68a}.dark\:bg-amber-200\/15:is(.dark *){background-color:hsla(48,97%,77%,.15)}.dark\:bg-amber-200\/20:is(.dark *){background-color:hsla(48,97%,77%,.2)}.dark\:bg-amber-200\/25:is(.dark *){background-color:hsla(48,97%,77%,.25)}.dark\:bg-amber-200\/30:is(.dark *){background-color:hsla(48,97%,77%,.3)}.dark\:bg-amber-200\/35:is(.dark *){background-color:hsla(48,97%,77%,.35)}.dark\:bg-amber-200\/40:is(.dark *){background-color:hsla(48,97%,77%,.4)}.dark\:bg-amber-200\/45:is(.dark *){background-color:hsla(48,97%,77%,.45)}.dark\:bg-amber-200\/5:is(.dark *){background-color:hsla(48,97%,77%,.05)}.dark\:bg-amber-200\/50:is(.dark *){background-color:hsla(48,97%,77%,.5)}.dark\:bg-amber-200\/55:is(.dark *){background-color:hsla(48,97%,77%,.55)}.dark\:bg-amber-200\/60:is(.dark *){background-color:hsla(48,97%,77%,.6)}.dark\:bg-amber-200\/65:is(.dark *){background-color:hsla(48,97%,77%,.65)}.dark\:bg-amber-200\/70:is(.dark *){background-color:hsla(48,97%,77%,.7)}.dark\:bg-amber-200\/75:is(.dark *){background-color:hsla(48,97%,77%,.75)}.dark\:bg-amber-200\/80:is(.dark *){background-color:hsla(48,97%,77%,.8)}.dark\:bg-amber-200\/85:is(.dark *){background-color:hsla(48,97%,77%,.85)}.dark\:bg-amber-200\/90:is(.dark *){background-color:hsla(48,97%,77%,.9)}.dark\:bg-amber-200\/95:is(.dark *){background-color:hsla(48,97%,77%,.95)}.dark\:bg-amber-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(251 191 36/var(--tw-bg-opacity,1))}.dark\:bg-amber-400\/0:is(.dark *){background-color:rgba(251,191,36,0)}.dark\:bg-amber-400\/10:is(.dark *){background-color:rgba(251,191,36,.1)}.dark\:bg-amber-400\/100:is(.dark *){background-color:#fbbf24}.dark\:bg-amber-400\/15:is(.dark *){background-color:rgba(251,191,36,.15)}.dark\:bg-amber-400\/20:is(.dark *){background-color:rgba(251,191,36,.2)}.dark\:bg-amber-400\/25:is(.dark *){background-color:rgba(251,191,36,.25)}.dark\:bg-amber-400\/30:is(.dark *){background-color:rgba(251,191,36,.3)}.dark\:bg-amber-400\/35:is(.dark *){background-color:rgba(251,191,36,.35)}.dark\:bg-amber-400\/40:is(.dark *){background-color:rgba(251,191,36,.4)}.dark\:bg-amber-400\/45:is(.dark *){background-color:rgba(251,191,36,.45)}.dark\:bg-amber-400\/5:is(.dark *){background-color:rgba(251,191,36,.05)}.dark\:bg-amber-400\/50:is(.dark *){background-color:rgba(251,191,36,.5)}.dark\:bg-amber-400\/55:is(.dark *){background-color:rgba(251,191,36,.55)}.dark\:bg-amber-400\/60:is(.dark *){background-color:rgba(251,191,36,.6)}.dark\:bg-amber-400\/65:is(.dark *){background-color:rgba(251,191,36,.65)}.dark\:bg-amber-400\/70:is(.dark *){background-color:rgba(251,191,36,.7)}.dark\:bg-amber-400\/75:is(.dark *){background-color:rgba(251,191,36,.75)}.dark\:bg-amber-400\/80:is(.dark *){background-color:rgba(251,191,36,.8)}.dark\:bg-amber-400\/85:is(.dark *){background-color:rgba(251,191,36,.85)}.dark\:bg-amber-400\/90:is(.dark *){background-color:rgba(251,191,36,.9)}.dark\:bg-amber-400\/95:is(.dark *){background-color:rgba(251,191,36,.95)}.dark\:bg-amber-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(255 251 235/var(--tw-bg-opacity,1))}.dark\:bg-amber-50\/0:is(.dark *){background-color:rgba(255,251,235,0)}.dark\:bg-amber-50\/10:is(.dark *){background-color:rgba(255,251,235,.1)}.dark\:bg-amber-50\/100:is(.dark *){background-color:#fffbeb}.dark\:bg-amber-50\/15:is(.dark *){background-color:rgba(255,251,235,.15)}.dark\:bg-amber-50\/20:is(.dark *){background-color:rgba(255,251,235,.2)}.dark\:bg-amber-50\/25:is(.dark *){background-color:rgba(255,251,235,.25)}.dark\:bg-amber-50\/30:is(.dark *){background-color:rgba(255,251,235,.3)}.dark\:bg-amber-50\/35:is(.dark *){background-color:rgba(255,251,235,.35)}.dark\:bg-amber-50\/40:is(.dark *){background-color:rgba(255,251,235,.4)}.dark\:bg-amber-50\/45:is(.dark *){background-color:rgba(255,251,235,.45)}.dark\:bg-amber-50\/5:is(.dark *){background-color:rgba(255,251,235,.05)}.dark\:bg-amber-50\/50:is(.dark *){background-color:rgba(255,251,235,.5)}.dark\:bg-amber-50\/55:is(.dark *){background-color:rgba(255,251,235,.55)}.dark\:bg-amber-50\/60:is(.dark *){background-color:rgba(255,251,235,.6)}.dark\:bg-amber-50\/65:is(.dark *){background-color:rgba(255,251,235,.65)}.dark\:bg-amber-50\/70:is(.dark *){background-color:rgba(255,251,235,.7)}.dark\:bg-amber-50\/75:is(.dark *){background-color:rgba(255,251,235,.75)}.dark\:bg-amber-50\/80:is(.dark *){background-color:rgba(255,251,235,.8)}.dark\:bg-amber-50\/85:is(.dark *){background-color:rgba(255,251,235,.85)}.dark\:bg-amber-50\/90:is(.dark *){background-color:rgba(255,251,235,.9)}.dark\:bg-amber-50\/95:is(.dark *){background-color:rgba(255,251,235,.95)}.dark\:bg-amber-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(245 158 11/var(--tw-bg-opacity,1))}.dark\:bg-amber-500\/0:is(.dark *){background-color:rgba(245,158,11,0)}.dark\:bg-amber-500\/10:is(.dark *){background-color:rgba(245,158,11,.1)}.dark\:bg-amber-500\/100:is(.dark *){background-color:#f59e0b}.dark\:bg-amber-500\/15:is(.dark *){background-color:rgba(245,158,11,.15)}.dark\:bg-amber-500\/20:is(.dark *){background-color:rgba(245,158,11,.2)}.dark\:bg-amber-500\/25:is(.dark *){background-color:rgba(245,158,11,.25)}.dark\:bg-amber-500\/30:is(.dark *){background-color:rgba(245,158,11,.3)}.dark\:bg-amber-500\/35:is(.dark *){background-color:rgba(245,158,11,.35)}.dark\:bg-amber-500\/40:is(.dark *){background-color:rgba(245,158,11,.4)}.dark\:bg-amber-500\/45:is(.dark *){background-color:rgba(245,158,11,.45)}.dark\:bg-amber-500\/5:is(.dark *){background-color:rgba(245,158,11,.05)}.dark\:bg-amber-500\/50:is(.dark *){background-color:rgba(245,158,11,.5)}.dark\:bg-amber-500\/55:is(.dark *){background-color:rgba(245,158,11,.55)}.dark\:bg-amber-500\/60:is(.dark *){background-color:rgba(245,158,11,.6)}.dark\:bg-amber-500\/65:is(.dark *){background-color:rgba(245,158,11,.65)}.dark\:bg-amber-500\/70:is(.dark *){background-color:rgba(245,158,11,.7)}.dark\:bg-amber-500\/75:is(.dark *){background-color:rgba(245,158,11,.75)}.dark\:bg-amber-500\/80:is(.dark *){background-color:rgba(245,158,11,.8)}.dark\:bg-amber-500\/85:is(.dark *){background-color:rgba(245,158,11,.85)}.dark\:bg-amber-500\/90:is(.dark *){background-color:rgba(245,158,11,.9)}.dark\:bg-amber-500\/95:is(.dark *){background-color:rgba(245,158,11,.95)}.dark\:bg-amber-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(69 26 3/var(--tw-bg-opacity,1))}.dark\:bg-amber-950\/0:is(.dark *){background-color:rgba(69,26,3,0)}.dark\:bg-amber-950\/10:is(.dark *){background-color:rgba(69,26,3,.1)}.dark\:bg-amber-950\/100:is(.dark *){background-color:#451a03}.dark\:bg-amber-950\/15:is(.dark *){background-color:rgba(69,26,3,.15)}.dark\:bg-amber-950\/20:is(.dark *){background-color:rgba(69,26,3,.2)}.dark\:bg-amber-950\/25:is(.dark *){background-color:rgba(69,26,3,.25)}.dark\:bg-amber-950\/30:is(.dark *){background-color:rgba(69,26,3,.3)}.dark\:bg-amber-950\/35:is(.dark *){background-color:rgba(69,26,3,.35)}.dark\:bg-amber-950\/40:is(.dark *){background-color:rgba(69,26,3,.4)}.dark\:bg-amber-950\/45:is(.dark *){background-color:rgba(69,26,3,.45)}.dark\:bg-amber-950\/5:is(.dark *){background-color:rgba(69,26,3,.05)}.dark\:bg-amber-950\/50:is(.dark *){background-color:rgba(69,26,3,.5)}.dark\:bg-amber-950\/55:is(.dark *){background-color:rgba(69,26,3,.55)}.dark\:bg-amber-950\/60:is(.dark *){background-color:rgba(69,26,3,.6)}.dark\:bg-amber-950\/65:is(.dark *){background-color:rgba(69,26,3,.65)}.dark\:bg-amber-950\/70:is(.dark *){background-color:rgba(69,26,3,.7)}.dark\:bg-amber-950\/75:is(.dark *){background-color:rgba(69,26,3,.75)}.dark\:bg-amber-950\/80:is(.dark *){background-color:rgba(69,26,3,.8)}.dark\:bg-amber-950\/85:is(.dark *){background-color:rgba(69,26,3,.85)}.dark\:bg-amber-950\/90:is(.dark *){background-color:rgba(69,26,3,.9)}.dark\:bg-amber-950\/95:is(.dark *){background-color:rgba(69,26,3,.95)}.dark\:bg-blue-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(219 234 254/var(--tw-bg-opacity,1))}.dark\:bg-blue-100\/0:is(.dark *){background-color:rgba(219,234,254,0)}.dark\:bg-blue-100\/10:is(.dark *){background-color:rgba(219,234,254,.1)}.dark\:bg-blue-100\/100:is(.dark *){background-color:#dbeafe}.dark\:bg-blue-100\/15:is(.dark *){background-color:rgba(219,234,254,.15)}.dark\:bg-blue-100\/20:is(.dark *){background-color:rgba(219,234,254,.2)}.dark\:bg-blue-100\/25:is(.dark *){background-color:rgba(219,234,254,.25)}.dark\:bg-blue-100\/30:is(.dark *){background-color:rgba(219,234,254,.3)}.dark\:bg-blue-100\/35:is(.dark *){background-color:rgba(219,234,254,.35)}.dark\:bg-blue-100\/40:is(.dark *){background-color:rgba(219,234,254,.4)}.dark\:bg-blue-100\/45:is(.dark *){background-color:rgba(219,234,254,.45)}.dark\:bg-blue-100\/5:is(.dark *){background-color:rgba(219,234,254,.05)}.dark\:bg-blue-100\/50:is(.dark *){background-color:rgba(219,234,254,.5)}.dark\:bg-blue-100\/55:is(.dark *){background-color:rgba(219,234,254,.55)}.dark\:bg-blue-100\/60:is(.dark *){background-color:rgba(219,234,254,.6)}.dark\:bg-blue-100\/65:is(.dark *){background-color:rgba(219,234,254,.65)}.dark\:bg-blue-100\/70:is(.dark *){background-color:rgba(219,234,254,.7)}.dark\:bg-blue-100\/75:is(.dark *){background-color:rgba(219,234,254,.75)}.dark\:bg-blue-100\/80:is(.dark *){background-color:rgba(219,234,254,.8)}.dark\:bg-blue-100\/85:is(.dark *){background-color:rgba(219,234,254,.85)}.dark\:bg-blue-100\/90:is(.dark *){background-color:rgba(219,234,254,.9)}.dark\:bg-blue-100\/95:is(.dark *){background-color:rgba(219,234,254,.95)}.dark\:bg-blue-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(191 219 254/var(--tw-bg-opacity,1))}.dark\:bg-blue-200\/0:is(.dark *){background-color:rgba(191,219,254,0)}.dark\:bg-blue-200\/10:is(.dark *){background-color:rgba(191,219,254,.1)}.dark\:bg-blue-200\/100:is(.dark *){background-color:#bfdbfe}.dark\:bg-blue-200\/15:is(.dark *){background-color:rgba(191,219,254,.15)}.dark\:bg-blue-200\/20:is(.dark *){background-color:rgba(191,219,254,.2)}.dark\:bg-blue-200\/25:is(.dark *){background-color:rgba(191,219,254,.25)}.dark\:bg-blue-200\/30:is(.dark *){background-color:rgba(191,219,254,.3)}.dark\:bg-blue-200\/35:is(.dark *){background-color:rgba(191,219,254,.35)}.dark\:bg-blue-200\/40:is(.dark *){background-color:rgba(191,219,254,.4)}.dark\:bg-blue-200\/45:is(.dark *){background-color:rgba(191,219,254,.45)}.dark\:bg-blue-200\/5:is(.dark *){background-color:rgba(191,219,254,.05)}.dark\:bg-blue-200\/50:is(.dark *){background-color:rgba(191,219,254,.5)}.dark\:bg-blue-200\/55:is(.dark *){background-color:rgba(191,219,254,.55)}.dark\:bg-blue-200\/60:is(.dark *){background-color:rgba(191,219,254,.6)}.dark\:bg-blue-200\/65:is(.dark *){background-color:rgba(191,219,254,.65)}.dark\:bg-blue-200\/70:is(.dark *){background-color:rgba(191,219,254,.7)}.dark\:bg-blue-200\/75:is(.dark *){background-color:rgba(191,219,254,.75)}.dark\:bg-blue-200\/80:is(.dark *){background-color:rgba(191,219,254,.8)}.dark\:bg-blue-200\/85:is(.dark *){background-color:rgba(191,219,254,.85)}.dark\:bg-blue-200\/90:is(.dark *){background-color:rgba(191,219,254,.9)}.dark\:bg-blue-200\/95:is(.dark *){background-color:rgba(191,219,254,.95)}.dark\:bg-blue-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(96 165 250/var(--tw-bg-opacity,1))}.dark\:bg-blue-400\/0:is(.dark *){background-color:rgba(96,165,250,0)}.dark\:bg-blue-400\/10:is(.dark *){background-color:rgba(96,165,250,.1)}.dark\:bg-blue-400\/100:is(.dark *){background-color:#60a5fa}.dark\:bg-blue-400\/15:is(.dark *){background-color:rgba(96,165,250,.15)}.dark\:bg-blue-400\/20:is(.dark *){background-color:rgba(96,165,250,.2)}.dark\:bg-blue-400\/25:is(.dark *){background-color:rgba(96,165,250,.25)}.dark\:bg-blue-400\/30:is(.dark *){background-color:rgba(96,165,250,.3)}.dark\:bg-blue-400\/35:is(.dark *){background-color:rgba(96,165,250,.35)}.dark\:bg-blue-400\/40:is(.dark *){background-color:rgba(96,165,250,.4)}.dark\:bg-blue-400\/45:is(.dark *){background-color:rgba(96,165,250,.45)}.dark\:bg-blue-400\/5:is(.dark *){background-color:rgba(96,165,250,.05)}.dark\:bg-blue-400\/50:is(.dark *){background-color:rgba(96,165,250,.5)}.dark\:bg-blue-400\/55:is(.dark *){background-color:rgba(96,165,250,.55)}.dark\:bg-blue-400\/60:is(.dark *){background-color:rgba(96,165,250,.6)}.dark\:bg-blue-400\/65:is(.dark *){background-color:rgba(96,165,250,.65)}.dark\:bg-blue-400\/70:is(.dark *){background-color:rgba(96,165,250,.7)}.dark\:bg-blue-400\/75:is(.dark *){background-color:rgba(96,165,250,.75)}.dark\:bg-blue-400\/80:is(.dark *){background-color:rgba(96,165,250,.8)}.dark\:bg-blue-400\/85:is(.dark *){background-color:rgba(96,165,250,.85)}.dark\:bg-blue-400\/90:is(.dark *){background-color:rgba(96,165,250,.9)}.dark\:bg-blue-400\/95:is(.dark *){background-color:rgba(96,165,250,.95)}.dark\:bg-blue-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(239 246 255/var(--tw-bg-opacity,1))}.dark\:bg-blue-50\/0:is(.dark *){background-color:rgba(239,246,255,0)}.dark\:bg-blue-50\/10:is(.dark *){background-color:rgba(239,246,255,.1)}.dark\:bg-blue-50\/100:is(.dark *){background-color:#eff6ff}.dark\:bg-blue-50\/15:is(.dark *){background-color:rgba(239,246,255,.15)}.dark\:bg-blue-50\/20:is(.dark *){background-color:rgba(239,246,255,.2)}.dark\:bg-blue-50\/25:is(.dark *){background-color:rgba(239,246,255,.25)}.dark\:bg-blue-50\/30:is(.dark *){background-color:rgba(239,246,255,.3)}.dark\:bg-blue-50\/35:is(.dark *){background-color:rgba(239,246,255,.35)}.dark\:bg-blue-50\/40:is(.dark *){background-color:rgba(239,246,255,.4)}.dark\:bg-blue-50\/45:is(.dark *){background-color:rgba(239,246,255,.45)}.dark\:bg-blue-50\/5:is(.dark *){background-color:rgba(239,246,255,.05)}.dark\:bg-blue-50\/50:is(.dark *){background-color:rgba(239,246,255,.5)}.dark\:bg-blue-50\/55:is(.dark *){background-color:rgba(239,246,255,.55)}.dark\:bg-blue-50\/60:is(.dark *){background-color:rgba(239,246,255,.6)}.dark\:bg-blue-50\/65:is(.dark *){background-color:rgba(239,246,255,.65)}.dark\:bg-blue-50\/70:is(.dark *){background-color:rgba(239,246,255,.7)}.dark\:bg-blue-50\/75:is(.dark *){background-color:rgba(239,246,255,.75)}.dark\:bg-blue-50\/80:is(.dark *){background-color:rgba(239,246,255,.8)}.dark\:bg-blue-50\/85:is(.dark *){background-color:rgba(239,246,255,.85)}.dark\:bg-blue-50\/90:is(.dark *){background-color:rgba(239,246,255,.9)}.dark\:bg-blue-50\/95:is(.dark *){background-color:rgba(239,246,255,.95)}.dark\:bg-blue-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(59 130 246/var(--tw-bg-opacity,1))}.dark\:bg-blue-500\/0:is(.dark *){background-color:rgba(59,130,246,0)}.dark\:bg-blue-500\/10:is(.dark *){background-color:rgba(59,130,246,.1)}.dark\:bg-blue-500\/100:is(.dark *){background-color:#3b82f6}.dark\:bg-blue-500\/15:is(.dark *){background-color:rgba(59,130,246,.15)}.dark\:bg-blue-500\/20:is(.dark *){background-color:rgba(59,130,246,.2)}.dark\:bg-blue-500\/25:is(.dark *){background-color:rgba(59,130,246,.25)}.dark\:bg-blue-500\/30:is(.dark *){background-color:rgba(59,130,246,.3)}.dark\:bg-blue-500\/35:is(.dark *){background-color:rgba(59,130,246,.35)}.dark\:bg-blue-500\/40:is(.dark *){background-color:rgba(59,130,246,.4)}.dark\:bg-blue-500\/45:is(.dark *){background-color:rgba(59,130,246,.45)}.dark\:bg-blue-500\/5:is(.dark *){background-color:rgba(59,130,246,.05)}.dark\:bg-blue-500\/50:is(.dark *){background-color:rgba(59,130,246,.5)}.dark\:bg-blue-500\/55:is(.dark *){background-color:rgba(59,130,246,.55)}.dark\:bg-blue-500\/60:is(.dark *){background-color:rgba(59,130,246,.6)}.dark\:bg-blue-500\/65:is(.dark *){background-color:rgba(59,130,246,.65)}.dark\:bg-blue-500\/70:is(.dark *){background-color:rgba(59,130,246,.7)}.dark\:bg-blue-500\/75:is(.dark *){background-color:rgba(59,130,246,.75)}.dark\:bg-blue-500\/80:is(.dark *){background-color:rgba(59,130,246,.8)}.dark\:bg-blue-500\/85:is(.dark *){background-color:rgba(59,130,246,.85)}.dark\:bg-blue-500\/90:is(.dark *){background-color:rgba(59,130,246,.9)}.dark\:bg-blue-500\/95:is(.dark *){background-color:rgba(59,130,246,.95)}.dark\:bg-blue-800:is(.dark *){--tw-bg-opacity:1;background-color:rgb(30 64 175/var(--tw-bg-opacity,1))}.dark\:bg-blue-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(23 37 84/var(--tw-bg-opacity,1))}.dark\:bg-blue-950\/0:is(.dark *){background-color:rgba(23,37,84,0)}.dark\:bg-blue-950\/10:is(.dark *){background-color:rgba(23,37,84,.1)}.dark\:bg-blue-950\/100:is(.dark *){background-color:#172554}.dark\:bg-blue-950\/15:is(.dark *){background-color:rgba(23,37,84,.15)}.dark\:bg-blue-950\/20:is(.dark *){background-color:rgba(23,37,84,.2)}.dark\:bg-blue-950\/25:is(.dark *){background-color:rgba(23,37,84,.25)}.dark\:bg-blue-950\/30:is(.dark *){background-color:rgba(23,37,84,.3)}.dark\:bg-blue-950\/35:is(.dark *){background-color:rgba(23,37,84,.35)}.dark\:bg-blue-950\/40:is(.dark *){background-color:rgba(23,37,84,.4)}.dark\:bg-blue-950\/45:is(.dark *){background-color:rgba(23,37,84,.45)}.dark\:bg-blue-950\/5:is(.dark *){background-color:rgba(23,37,84,.05)}.dark\:bg-blue-950\/50:is(.dark *){background-color:rgba(23,37,84,.5)}.dark\:bg-blue-950\/55:is(.dark *){background-color:rgba(23,37,84,.55)}.dark\:bg-blue-950\/60:is(.dark *){background-color:rgba(23,37,84,.6)}.dark\:bg-blue-950\/65:is(.dark *){background-color:rgba(23,37,84,.65)}.dark\:bg-blue-950\/70:is(.dark *){background-color:rgba(23,37,84,.7)}.dark\:bg-blue-950\/75:is(.dark *){background-color:rgba(23,37,84,.75)}.dark\:bg-blue-950\/80:is(.dark *){background-color:rgba(23,37,84,.8)}.dark\:bg-blue-950\/85:is(.dark *){background-color:rgba(23,37,84,.85)}.dark\:bg-blue-950\/90:is(.dark *){background-color:rgba(23,37,84,.9)}.dark\:bg-blue-950\/95:is(.dark *){background-color:rgba(23,37,84,.95)}.dark\:bg-cyan-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(207 250 254/var(--tw-bg-opacity,1))}.dark\:bg-cyan-100\/0:is(.dark *){background-color:rgba(207,250,254,0)}.dark\:bg-cyan-100\/10:is(.dark *){background-color:rgba(207,250,254,.1)}.dark\:bg-cyan-100\/100:is(.dark *){background-color:#cffafe}.dark\:bg-cyan-100\/15:is(.dark *){background-color:rgba(207,250,254,.15)}.dark\:bg-cyan-100\/20:is(.dark *){background-color:rgba(207,250,254,.2)}.dark\:bg-cyan-100\/25:is(.dark *){background-color:rgba(207,250,254,.25)}.dark\:bg-cyan-100\/30:is(.dark *){background-color:rgba(207,250,254,.3)}.dark\:bg-cyan-100\/35:is(.dark *){background-color:rgba(207,250,254,.35)}.dark\:bg-cyan-100\/40:is(.dark *){background-color:rgba(207,250,254,.4)}.dark\:bg-cyan-100\/45:is(.dark *){background-color:rgba(207,250,254,.45)}.dark\:bg-cyan-100\/5:is(.dark *){background-color:rgba(207,250,254,.05)}.dark\:bg-cyan-100\/50:is(.dark *){background-color:rgba(207,250,254,.5)}.dark\:bg-cyan-100\/55:is(.dark *){background-color:rgba(207,250,254,.55)}.dark\:bg-cyan-100\/60:is(.dark *){background-color:rgba(207,250,254,.6)}.dark\:bg-cyan-100\/65:is(.dark *){background-color:rgba(207,250,254,.65)}.dark\:bg-cyan-100\/70:is(.dark *){background-color:rgba(207,250,254,.7)}.dark\:bg-cyan-100\/75:is(.dark *){background-color:rgba(207,250,254,.75)}.dark\:bg-cyan-100\/80:is(.dark *){background-color:rgba(207,250,254,.8)}.dark\:bg-cyan-100\/85:is(.dark *){background-color:rgba(207,250,254,.85)}.dark\:bg-cyan-100\/90:is(.dark *){background-color:rgba(207,250,254,.9)}.dark\:bg-cyan-100\/95:is(.dark *){background-color:rgba(207,250,254,.95)}.dark\:bg-cyan-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(165 243 252/var(--tw-bg-opacity,1))}.dark\:bg-cyan-200\/0:is(.dark *){background-color:rgba(165,243,252,0)}.dark\:bg-cyan-200\/10:is(.dark *){background-color:rgba(165,243,252,.1)}.dark\:bg-cyan-200\/100:is(.dark *){background-color:#a5f3fc}.dark\:bg-cyan-200\/15:is(.dark *){background-color:rgba(165,243,252,.15)}.dark\:bg-cyan-200\/20:is(.dark *){background-color:rgba(165,243,252,.2)}.dark\:bg-cyan-200\/25:is(.dark *){background-color:rgba(165,243,252,.25)}.dark\:bg-cyan-200\/30:is(.dark *){background-color:rgba(165,243,252,.3)}.dark\:bg-cyan-200\/35:is(.dark *){background-color:rgba(165,243,252,.35)}.dark\:bg-cyan-200\/40:is(.dark *){background-color:rgba(165,243,252,.4)}.dark\:bg-cyan-200\/45:is(.dark *){background-color:rgba(165,243,252,.45)}.dark\:bg-cyan-200\/5:is(.dark *){background-color:rgba(165,243,252,.05)}.dark\:bg-cyan-200\/50:is(.dark *){background-color:rgba(165,243,252,.5)}.dark\:bg-cyan-200\/55:is(.dark *){background-color:rgba(165,243,252,.55)}.dark\:bg-cyan-200\/60:is(.dark *){background-color:rgba(165,243,252,.6)}.dark\:bg-cyan-200\/65:is(.dark *){background-color:rgba(165,243,252,.65)}.dark\:bg-cyan-200\/70:is(.dark *){background-color:rgba(165,243,252,.7)}.dark\:bg-cyan-200\/75:is(.dark *){background-color:rgba(165,243,252,.75)}.dark\:bg-cyan-200\/80:is(.dark *){background-color:rgba(165,243,252,.8)}.dark\:bg-cyan-200\/85:is(.dark *){background-color:rgba(165,243,252,.85)}.dark\:bg-cyan-200\/90:is(.dark *){background-color:rgba(165,243,252,.9)}.dark\:bg-cyan-200\/95:is(.dark *){background-color:rgba(165,243,252,.95)}.dark\:bg-cyan-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(34 211 238/var(--tw-bg-opacity,1))}.dark\:bg-cyan-400\/0:is(.dark *){background-color:rgba(34,211,238,0)}.dark\:bg-cyan-400\/10:is(.dark *){background-color:rgba(34,211,238,.1)}.dark\:bg-cyan-400\/100:is(.dark *){background-color:#22d3ee}.dark\:bg-cyan-400\/15:is(.dark *){background-color:rgba(34,211,238,.15)}.dark\:bg-cyan-400\/20:is(.dark *){background-color:rgba(34,211,238,.2)}.dark\:bg-cyan-400\/25:is(.dark *){background-color:rgba(34,211,238,.25)}.dark\:bg-cyan-400\/30:is(.dark *){background-color:rgba(34,211,238,.3)}.dark\:bg-cyan-400\/35:is(.dark *){background-color:rgba(34,211,238,.35)}.dark\:bg-cyan-400\/40:is(.dark *){background-color:rgba(34,211,238,.4)}.dark\:bg-cyan-400\/45:is(.dark *){background-color:rgba(34,211,238,.45)}.dark\:bg-cyan-400\/5:is(.dark *){background-color:rgba(34,211,238,.05)}.dark\:bg-cyan-400\/50:is(.dark *){background-color:rgba(34,211,238,.5)}.dark\:bg-cyan-400\/55:is(.dark *){background-color:rgba(34,211,238,.55)}.dark\:bg-cyan-400\/60:is(.dark *){background-color:rgba(34,211,238,.6)}.dark\:bg-cyan-400\/65:is(.dark *){background-color:rgba(34,211,238,.65)}.dark\:bg-cyan-400\/70:is(.dark *){background-color:rgba(34,211,238,.7)}.dark\:bg-cyan-400\/75:is(.dark *){background-color:rgba(34,211,238,.75)}.dark\:bg-cyan-400\/80:is(.dark *){background-color:rgba(34,211,238,.8)}.dark\:bg-cyan-400\/85:is(.dark *){background-color:rgba(34,211,238,.85)}.dark\:bg-cyan-400\/90:is(.dark *){background-color:rgba(34,211,238,.9)}.dark\:bg-cyan-400\/95:is(.dark *){background-color:rgba(34,211,238,.95)}.dark\:bg-cyan-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(236 254 255/var(--tw-bg-opacity,1))}.dark\:bg-cyan-50\/0:is(.dark *){background-color:rgba(236,254,255,0)}.dark\:bg-cyan-50\/10:is(.dark *){background-color:rgba(236,254,255,.1)}.dark\:bg-cyan-50\/100:is(.dark *){background-color:#ecfeff}.dark\:bg-cyan-50\/15:is(.dark *){background-color:rgba(236,254,255,.15)}.dark\:bg-cyan-50\/20:is(.dark *){background-color:rgba(236,254,255,.2)}.dark\:bg-cyan-50\/25:is(.dark *){background-color:rgba(236,254,255,.25)}.dark\:bg-cyan-50\/30:is(.dark *){background-color:rgba(236,254,255,.3)}.dark\:bg-cyan-50\/35:is(.dark *){background-color:rgba(236,254,255,.35)}.dark\:bg-cyan-50\/40:is(.dark *){background-color:rgba(236,254,255,.4)}.dark\:bg-cyan-50\/45:is(.dark *){background-color:rgba(236,254,255,.45)}.dark\:bg-cyan-50\/5:is(.dark *){background-color:rgba(236,254,255,.05)}.dark\:bg-cyan-50\/50:is(.dark *){background-color:rgba(236,254,255,.5)}.dark\:bg-cyan-50\/55:is(.dark *){background-color:rgba(236,254,255,.55)}.dark\:bg-cyan-50\/60:is(.dark *){background-color:rgba(236,254,255,.6)}.dark\:bg-cyan-50\/65:is(.dark *){background-color:rgba(236,254,255,.65)}.dark\:bg-cyan-50\/70:is(.dark *){background-color:rgba(236,254,255,.7)}.dark\:bg-cyan-50\/75:is(.dark *){background-color:rgba(236,254,255,.75)}.dark\:bg-cyan-50\/80:is(.dark *){background-color:rgba(236,254,255,.8)}.dark\:bg-cyan-50\/85:is(.dark *){background-color:rgba(236,254,255,.85)}.dark\:bg-cyan-50\/90:is(.dark *){background-color:rgba(236,254,255,.9)}.dark\:bg-cyan-50\/95:is(.dark *){background-color:rgba(236,254,255,.95)}.dark\:bg-cyan-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(6 182 212/var(--tw-bg-opacity,1))}.dark\:bg-cyan-500\/0:is(.dark *){background-color:rgba(6,182,212,0)}.dark\:bg-cyan-500\/10:is(.dark *){background-color:rgba(6,182,212,.1)}.dark\:bg-cyan-500\/100:is(.dark *){background-color:#06b6d4}.dark\:bg-cyan-500\/15:is(.dark *){background-color:rgba(6,182,212,.15)}.dark\:bg-cyan-500\/20:is(.dark *){background-color:rgba(6,182,212,.2)}.dark\:bg-cyan-500\/25:is(.dark *){background-color:rgba(6,182,212,.25)}.dark\:bg-cyan-500\/30:is(.dark *){background-color:rgba(6,182,212,.3)}.dark\:bg-cyan-500\/35:is(.dark *){background-color:rgba(6,182,212,.35)}.dark\:bg-cyan-500\/40:is(.dark *){background-color:rgba(6,182,212,.4)}.dark\:bg-cyan-500\/45:is(.dark *){background-color:rgba(6,182,212,.45)}.dark\:bg-cyan-500\/5:is(.dark *){background-color:rgba(6,182,212,.05)}.dark\:bg-cyan-500\/50:is(.dark *){background-color:rgba(6,182,212,.5)}.dark\:bg-cyan-500\/55:is(.dark *){background-color:rgba(6,182,212,.55)}.dark\:bg-cyan-500\/60:is(.dark *){background-color:rgba(6,182,212,.6)}.dark\:bg-cyan-500\/65:is(.dark *){background-color:rgba(6,182,212,.65)}.dark\:bg-cyan-500\/70:is(.dark *){background-color:rgba(6,182,212,.7)}.dark\:bg-cyan-500\/75:is(.dark *){background-color:rgba(6,182,212,.75)}.dark\:bg-cyan-500\/80:is(.dark *){background-color:rgba(6,182,212,.8)}.dark\:bg-cyan-500\/85:is(.dark *){background-color:rgba(6,182,212,.85)}.dark\:bg-cyan-500\/90:is(.dark *){background-color:rgba(6,182,212,.9)}.dark\:bg-cyan-500\/95:is(.dark *){background-color:rgba(6,182,212,.95)}.dark\:bg-cyan-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(8 51 68/var(--tw-bg-opacity,1))}.dark\:bg-cyan-950\/0:is(.dark *){background-color:rgba(8,51,68,0)}.dark\:bg-cyan-950\/10:is(.dark *){background-color:rgba(8,51,68,.1)}.dark\:bg-cyan-950\/100:is(.dark *){background-color:#083344}.dark\:bg-cyan-950\/15:is(.dark *){background-color:rgba(8,51,68,.15)}.dark\:bg-cyan-950\/20:is(.dark *){background-color:rgba(8,51,68,.2)}.dark\:bg-cyan-950\/25:is(.dark *){background-color:rgba(8,51,68,.25)}.dark\:bg-cyan-950\/30:is(.dark *){background-color:rgba(8,51,68,.3)}.dark\:bg-cyan-950\/35:is(.dark *){background-color:rgba(8,51,68,.35)}.dark\:bg-cyan-950\/40:is(.dark *){background-color:rgba(8,51,68,.4)}.dark\:bg-cyan-950\/45:is(.dark *){background-color:rgba(8,51,68,.45)}.dark\:bg-cyan-950\/5:is(.dark *){background-color:rgba(8,51,68,.05)}.dark\:bg-cyan-950\/50:is(.dark *){background-color:rgba(8,51,68,.5)}.dark\:bg-cyan-950\/55:is(.dark *){background-color:rgba(8,51,68,.55)}.dark\:bg-cyan-950\/60:is(.dark *){background-color:rgba(8,51,68,.6)}.dark\:bg-cyan-950\/65:is(.dark *){background-color:rgba(8,51,68,.65)}.dark\:bg-cyan-950\/70:is(.dark *){background-color:rgba(8,51,68,.7)}.dark\:bg-cyan-950\/75:is(.dark *){background-color:rgba(8,51,68,.75)}.dark\:bg-cyan-950\/80:is(.dark *){background-color:rgba(8,51,68,.8)}.dark\:bg-cyan-950\/85:is(.dark *){background-color:rgba(8,51,68,.85)}.dark\:bg-cyan-950\/90:is(.dark *){background-color:rgba(8,51,68,.9)}.dark\:bg-cyan-950\/95:is(.dark *){background-color:rgba(8,51,68,.95)}.dark\:bg-emerald-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(209 250 229/var(--tw-bg-opacity,1))}.dark\:bg-emerald-100\/0:is(.dark *){background-color:rgba(209,250,229,0)}.dark\:bg-emerald-100\/10:is(.dark *){background-color:rgba(209,250,229,.1)}.dark\:bg-emerald-100\/100:is(.dark *){background-color:#d1fae5}.dark\:bg-emerald-100\/15:is(.dark *){background-color:rgba(209,250,229,.15)}.dark\:bg-emerald-100\/20:is(.dark *){background-color:rgba(209,250,229,.2)}.dark\:bg-emerald-100\/25:is(.dark *){background-color:rgba(209,250,229,.25)}.dark\:bg-emerald-100\/30:is(.dark *){background-color:rgba(209,250,229,.3)}.dark\:bg-emerald-100\/35:is(.dark *){background-color:rgba(209,250,229,.35)}.dark\:bg-emerald-100\/40:is(.dark *){background-color:rgba(209,250,229,.4)}.dark\:bg-emerald-100\/45:is(.dark *){background-color:rgba(209,250,229,.45)}.dark\:bg-emerald-100\/5:is(.dark *){background-color:rgba(209,250,229,.05)}.dark\:bg-emerald-100\/50:is(.dark *){background-color:rgba(209,250,229,.5)}.dark\:bg-emerald-100\/55:is(.dark *){background-color:rgba(209,250,229,.55)}.dark\:bg-emerald-100\/60:is(.dark *){background-color:rgba(209,250,229,.6)}.dark\:bg-emerald-100\/65:is(.dark *){background-color:rgba(209,250,229,.65)}.dark\:bg-emerald-100\/70:is(.dark *){background-color:rgba(209,250,229,.7)}.dark\:bg-emerald-100\/75:is(.dark *){background-color:rgba(209,250,229,.75)}.dark\:bg-emerald-100\/80:is(.dark *){background-color:rgba(209,250,229,.8)}.dark\:bg-emerald-100\/85:is(.dark *){background-color:rgba(209,250,229,.85)}.dark\:bg-emerald-100\/90:is(.dark *){background-color:rgba(209,250,229,.9)}.dark\:bg-emerald-100\/95:is(.dark *){background-color:rgba(209,250,229,.95)}.dark\:bg-emerald-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(167 243 208/var(--tw-bg-opacity,1))}.dark\:bg-emerald-200\/0:is(.dark *){background-color:rgba(167,243,208,0)}.dark\:bg-emerald-200\/10:is(.dark *){background-color:rgba(167,243,208,.1)}.dark\:bg-emerald-200\/100:is(.dark *){background-color:#a7f3d0}.dark\:bg-emerald-200\/15:is(.dark *){background-color:rgba(167,243,208,.15)}.dark\:bg-emerald-200\/20:is(.dark *){background-color:rgba(167,243,208,.2)}.dark\:bg-emerald-200\/25:is(.dark *){background-color:rgba(167,243,208,.25)}.dark\:bg-emerald-200\/30:is(.dark *){background-color:rgba(167,243,208,.3)}.dark\:bg-emerald-200\/35:is(.dark *){background-color:rgba(167,243,208,.35)}.dark\:bg-emerald-200\/40:is(.dark *){background-color:rgba(167,243,208,.4)}.dark\:bg-emerald-200\/45:is(.dark *){background-color:rgba(167,243,208,.45)}.dark\:bg-emerald-200\/5:is(.dark *){background-color:rgba(167,243,208,.05)}.dark\:bg-emerald-200\/50:is(.dark *){background-color:rgba(167,243,208,.5)}.dark\:bg-emerald-200\/55:is(.dark *){background-color:rgba(167,243,208,.55)}.dark\:bg-emerald-200\/60:is(.dark *){background-color:rgba(167,243,208,.6)}.dark\:bg-emerald-200\/65:is(.dark *){background-color:rgba(167,243,208,.65)}.dark\:bg-emerald-200\/70:is(.dark *){background-color:rgba(167,243,208,.7)}.dark\:bg-emerald-200\/75:is(.dark *){background-color:rgba(167,243,208,.75)}.dark\:bg-emerald-200\/80:is(.dark *){background-color:rgba(167,243,208,.8)}.dark\:bg-emerald-200\/85:is(.dark *){background-color:rgba(167,243,208,.85)}.dark\:bg-emerald-200\/90:is(.dark *){background-color:rgba(167,243,208,.9)}.dark\:bg-emerald-200\/95:is(.dark *){background-color:rgba(167,243,208,.95)}.dark\:bg-emerald-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(52 211 153/var(--tw-bg-opacity,1))}.dark\:bg-emerald-400\/0:is(.dark *){background-color:rgba(52,211,153,0)}.dark\:bg-emerald-400\/10:is(.dark *){background-color:rgba(52,211,153,.1)}.dark\:bg-emerald-400\/100:is(.dark *){background-color:#34d399}.dark\:bg-emerald-400\/15:is(.dark *){background-color:rgba(52,211,153,.15)}.dark\:bg-emerald-400\/20:is(.dark *){background-color:rgba(52,211,153,.2)}.dark\:bg-emerald-400\/25:is(.dark *){background-color:rgba(52,211,153,.25)}.dark\:bg-emerald-400\/30:is(.dark *){background-color:rgba(52,211,153,.3)}.dark\:bg-emerald-400\/35:is(.dark *){background-color:rgba(52,211,153,.35)}.dark\:bg-emerald-400\/40:is(.dark *){background-color:rgba(52,211,153,.4)}.dark\:bg-emerald-400\/45:is(.dark *){background-color:rgba(52,211,153,.45)}.dark\:bg-emerald-400\/5:is(.dark *){background-color:rgba(52,211,153,.05)}.dark\:bg-emerald-400\/50:is(.dark *){background-color:rgba(52,211,153,.5)}.dark\:bg-emerald-400\/55:is(.dark *){background-color:rgba(52,211,153,.55)}.dark\:bg-emerald-400\/60:is(.dark *){background-color:rgba(52,211,153,.6)}.dark\:bg-emerald-400\/65:is(.dark *){background-color:rgba(52,211,153,.65)}.dark\:bg-emerald-400\/70:is(.dark *){background-color:rgba(52,211,153,.7)}.dark\:bg-emerald-400\/75:is(.dark *){background-color:rgba(52,211,153,.75)}.dark\:bg-emerald-400\/80:is(.dark *){background-color:rgba(52,211,153,.8)}.dark\:bg-emerald-400\/85:is(.dark *){background-color:rgba(52,211,153,.85)}.dark\:bg-emerald-400\/90:is(.dark *){background-color:rgba(52,211,153,.9)}.dark\:bg-emerald-400\/95:is(.dark *){background-color:rgba(52,211,153,.95)}.dark\:bg-emerald-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(236 253 245/var(--tw-bg-opacity,1))}.dark\:bg-emerald-50\/0:is(.dark *){background-color:rgba(236,253,245,0)}.dark\:bg-emerald-50\/10:is(.dark *){background-color:rgba(236,253,245,.1)}.dark\:bg-emerald-50\/100:is(.dark *){background-color:#ecfdf5}.dark\:bg-emerald-50\/15:is(.dark *){background-color:rgba(236,253,245,.15)}.dark\:bg-emerald-50\/20:is(.dark *){background-color:rgba(236,253,245,.2)}.dark\:bg-emerald-50\/25:is(.dark *){background-color:rgba(236,253,245,.25)}.dark\:bg-emerald-50\/30:is(.dark *){background-color:rgba(236,253,245,.3)}.dark\:bg-emerald-50\/35:is(.dark *){background-color:rgba(236,253,245,.35)}.dark\:bg-emerald-50\/40:is(.dark *){background-color:rgba(236,253,245,.4)}.dark\:bg-emerald-50\/45:is(.dark *){background-color:rgba(236,253,245,.45)}.dark\:bg-emerald-50\/5:is(.dark *){background-color:rgba(236,253,245,.05)}.dark\:bg-emerald-50\/50:is(.dark *){background-color:rgba(236,253,245,.5)}.dark\:bg-emerald-50\/55:is(.dark *){background-color:rgba(236,253,245,.55)}.dark\:bg-emerald-50\/60:is(.dark *){background-color:rgba(236,253,245,.6)}.dark\:bg-emerald-50\/65:is(.dark *){background-color:rgba(236,253,245,.65)}.dark\:bg-emerald-50\/70:is(.dark *){background-color:rgba(236,253,245,.7)}.dark\:bg-emerald-50\/75:is(.dark *){background-color:rgba(236,253,245,.75)}.dark\:bg-emerald-50\/80:is(.dark *){background-color:rgba(236,253,245,.8)}.dark\:bg-emerald-50\/85:is(.dark *){background-color:rgba(236,253,245,.85)}.dark\:bg-emerald-50\/90:is(.dark *){background-color:rgba(236,253,245,.9)}.dark\:bg-emerald-50\/95:is(.dark *){background-color:rgba(236,253,245,.95)}.dark\:bg-emerald-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(16 185 129/var(--tw-bg-opacity,1))}.dark\:bg-emerald-500\/0:is(.dark *){background-color:rgba(16,185,129,0)}.dark\:bg-emerald-500\/10:is(.dark *){background-color:rgba(16,185,129,.1)}.dark\:bg-emerald-500\/100:is(.dark *){background-color:#10b981}.dark\:bg-emerald-500\/15:is(.dark *){background-color:rgba(16,185,129,.15)}.dark\:bg-emerald-500\/20:is(.dark *){background-color:rgba(16,185,129,.2)}.dark\:bg-emerald-500\/25:is(.dark *){background-color:rgba(16,185,129,.25)}.dark\:bg-emerald-500\/30:is(.dark *){background-color:rgba(16,185,129,.3)}.dark\:bg-emerald-500\/35:is(.dark *){background-color:rgba(16,185,129,.35)}.dark\:bg-emerald-500\/40:is(.dark *){background-color:rgba(16,185,129,.4)}.dark\:bg-emerald-500\/45:is(.dark *){background-color:rgba(16,185,129,.45)}.dark\:bg-emerald-500\/5:is(.dark *){background-color:rgba(16,185,129,.05)}.dark\:bg-emerald-500\/50:is(.dark *){background-color:rgba(16,185,129,.5)}.dark\:bg-emerald-500\/55:is(.dark *){background-color:rgba(16,185,129,.55)}.dark\:bg-emerald-500\/60:is(.dark *){background-color:rgba(16,185,129,.6)}.dark\:bg-emerald-500\/65:is(.dark *){background-color:rgba(16,185,129,.65)}.dark\:bg-emerald-500\/70:is(.dark *){background-color:rgba(16,185,129,.7)}.dark\:bg-emerald-500\/75:is(.dark *){background-color:rgba(16,185,129,.75)}.dark\:bg-emerald-500\/80:is(.dark *){background-color:rgba(16,185,129,.8)}.dark\:bg-emerald-500\/85:is(.dark *){background-color:rgba(16,185,129,.85)}.dark\:bg-emerald-500\/90:is(.dark *){background-color:rgba(16,185,129,.9)}.dark\:bg-emerald-500\/95:is(.dark *){background-color:rgba(16,185,129,.95)}.dark\:bg-emerald-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(2 44 34/var(--tw-bg-opacity,1))}.dark\:bg-emerald-950\/0:is(.dark *){background-color:rgba(2,44,34,0)}.dark\:bg-emerald-950\/10:is(.dark *){background-color:rgba(2,44,34,.1)}.dark\:bg-emerald-950\/100:is(.dark *){background-color:#022c22}.dark\:bg-emerald-950\/15:is(.dark *){background-color:rgba(2,44,34,.15)}.dark\:bg-emerald-950\/20:is(.dark *){background-color:rgba(2,44,34,.2)}.dark\:bg-emerald-950\/25:is(.dark *){background-color:rgba(2,44,34,.25)}.dark\:bg-emerald-950\/30:is(.dark *){background-color:rgba(2,44,34,.3)}.dark\:bg-emerald-950\/35:is(.dark *){background-color:rgba(2,44,34,.35)}.dark\:bg-emerald-950\/40:is(.dark *){background-color:rgba(2,44,34,.4)}.dark\:bg-emerald-950\/45:is(.dark *){background-color:rgba(2,44,34,.45)}.dark\:bg-emerald-950\/5:is(.dark *){background-color:rgba(2,44,34,.05)}.dark\:bg-emerald-950\/50:is(.dark *){background-color:rgba(2,44,34,.5)}.dark\:bg-emerald-950\/55:is(.dark *){background-color:rgba(2,44,34,.55)}.dark\:bg-emerald-950\/60:is(.dark *){background-color:rgba(2,44,34,.6)}.dark\:bg-emerald-950\/65:is(.dark *){background-color:rgba(2,44,34,.65)}.dark\:bg-emerald-950\/70:is(.dark *){background-color:rgba(2,44,34,.7)}.dark\:bg-emerald-950\/75:is(.dark *){background-color:rgba(2,44,34,.75)}.dark\:bg-emerald-950\/80:is(.dark *){background-color:rgba(2,44,34,.8)}.dark\:bg-emerald-950\/85:is(.dark *){background-color:rgba(2,44,34,.85)}.dark\:bg-emerald-950\/90:is(.dark *){background-color:rgba(2,44,34,.9)}.dark\:bg-emerald-950\/95:is(.dark *){background-color:rgba(2,44,34,.95)}.dark\:bg-fuchsia-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(250 232 255/var(--tw-bg-opacity,1))}.dark\:bg-fuchsia-100\/0:is(.dark *){background-color:rgba(250,232,255,0)}.dark\:bg-fuchsia-100\/10:is(.dark *){background-color:rgba(250,232,255,.1)}.dark\:bg-fuchsia-100\/100:is(.dark *){background-color:#fae8ff}.dark\:bg-fuchsia-100\/15:is(.dark *){background-color:rgba(250,232,255,.15)}.dark\:bg-fuchsia-100\/20:is(.dark *){background-color:rgba(250,232,255,.2)}.dark\:bg-fuchsia-100\/25:is(.dark *){background-color:rgba(250,232,255,.25)}.dark\:bg-fuchsia-100\/30:is(.dark *){background-color:rgba(250,232,255,.3)}.dark\:bg-fuchsia-100\/35:is(.dark *){background-color:rgba(250,232,255,.35)}.dark\:bg-fuchsia-100\/40:is(.dark *){background-color:rgba(250,232,255,.4)}.dark\:bg-fuchsia-100\/45:is(.dark *){background-color:rgba(250,232,255,.45)}.dark\:bg-fuchsia-100\/5:is(.dark *){background-color:rgba(250,232,255,.05)}.dark\:bg-fuchsia-100\/50:is(.dark *){background-color:rgba(250,232,255,.5)}.dark\:bg-fuchsia-100\/55:is(.dark *){background-color:rgba(250,232,255,.55)}.dark\:bg-fuchsia-100\/60:is(.dark *){background-color:rgba(250,232,255,.6)}.dark\:bg-fuchsia-100\/65:is(.dark *){background-color:rgba(250,232,255,.65)}.dark\:bg-fuchsia-100\/70:is(.dark *){background-color:rgba(250,232,255,.7)}.dark\:bg-fuchsia-100\/75:is(.dark *){background-color:rgba(250,232,255,.75)}.dark\:bg-fuchsia-100\/80:is(.dark *){background-color:rgba(250,232,255,.8)}.dark\:bg-fuchsia-100\/85:is(.dark *){background-color:rgba(250,232,255,.85)}.dark\:bg-fuchsia-100\/90:is(.dark *){background-color:rgba(250,232,255,.9)}.dark\:bg-fuchsia-100\/95:is(.dark *){background-color:rgba(250,232,255,.95)}.dark\:bg-fuchsia-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(245 208 254/var(--tw-bg-opacity,1))}.dark\:bg-fuchsia-200\/0:is(.dark *){background-color:rgba(245,208,254,0)}.dark\:bg-fuchsia-200\/10:is(.dark *){background-color:rgba(245,208,254,.1)}.dark\:bg-fuchsia-200\/100:is(.dark *){background-color:#f5d0fe}.dark\:bg-fuchsia-200\/15:is(.dark *){background-color:rgba(245,208,254,.15)}.dark\:bg-fuchsia-200\/20:is(.dark *){background-color:rgba(245,208,254,.2)}.dark\:bg-fuchsia-200\/25:is(.dark *){background-color:rgba(245,208,254,.25)}.dark\:bg-fuchsia-200\/30:is(.dark *){background-color:rgba(245,208,254,.3)}.dark\:bg-fuchsia-200\/35:is(.dark *){background-color:rgba(245,208,254,.35)}.dark\:bg-fuchsia-200\/40:is(.dark *){background-color:rgba(245,208,254,.4)}.dark\:bg-fuchsia-200\/45:is(.dark *){background-color:rgba(245,208,254,.45)}.dark\:bg-fuchsia-200\/5:is(.dark *){background-color:rgba(245,208,254,.05)}.dark\:bg-fuchsia-200\/50:is(.dark *){background-color:rgba(245,208,254,.5)}.dark\:bg-fuchsia-200\/55:is(.dark *){background-color:rgba(245,208,254,.55)}.dark\:bg-fuchsia-200\/60:is(.dark *){background-color:rgba(245,208,254,.6)}.dark\:bg-fuchsia-200\/65:is(.dark *){background-color:rgba(245,208,254,.65)}.dark\:bg-fuchsia-200\/70:is(.dark *){background-color:rgba(245,208,254,.7)}.dark\:bg-fuchsia-200\/75:is(.dark *){background-color:rgba(245,208,254,.75)}.dark\:bg-fuchsia-200\/80:is(.dark *){background-color:rgba(245,208,254,.8)}.dark\:bg-fuchsia-200\/85:is(.dark *){background-color:rgba(245,208,254,.85)}.dark\:bg-fuchsia-200\/90:is(.dark *){background-color:rgba(245,208,254,.9)}.dark\:bg-fuchsia-200\/95:is(.dark *){background-color:rgba(245,208,254,.95)}.dark\:bg-fuchsia-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(232 121 249/var(--tw-bg-opacity,1))}.dark\:bg-fuchsia-400\/0:is(.dark *){background-color:rgba(232,121,249,0)}.dark\:bg-fuchsia-400\/10:is(.dark *){background-color:rgba(232,121,249,.1)}.dark\:bg-fuchsia-400\/100:is(.dark *){background-color:#e879f9}.dark\:bg-fuchsia-400\/15:is(.dark *){background-color:rgba(232,121,249,.15)}.dark\:bg-fuchsia-400\/20:is(.dark *){background-color:rgba(232,121,249,.2)}.dark\:bg-fuchsia-400\/25:is(.dark *){background-color:rgba(232,121,249,.25)}.dark\:bg-fuchsia-400\/30:is(.dark *){background-color:rgba(232,121,249,.3)}.dark\:bg-fuchsia-400\/35:is(.dark *){background-color:rgba(232,121,249,.35)}.dark\:bg-fuchsia-400\/40:is(.dark *){background-color:rgba(232,121,249,.4)}.dark\:bg-fuchsia-400\/45:is(.dark *){background-color:rgba(232,121,249,.45)}.dark\:bg-fuchsia-400\/5:is(.dark *){background-color:rgba(232,121,249,.05)}.dark\:bg-fuchsia-400\/50:is(.dark *){background-color:rgba(232,121,249,.5)}.dark\:bg-fuchsia-400\/55:is(.dark *){background-color:rgba(232,121,249,.55)}.dark\:bg-fuchsia-400\/60:is(.dark *){background-color:rgba(232,121,249,.6)}.dark\:bg-fuchsia-400\/65:is(.dark *){background-color:rgba(232,121,249,.65)}.dark\:bg-fuchsia-400\/70:is(.dark *){background-color:rgba(232,121,249,.7)}.dark\:bg-fuchsia-400\/75:is(.dark *){background-color:rgba(232,121,249,.75)}.dark\:bg-fuchsia-400\/80:is(.dark *){background-color:rgba(232,121,249,.8)}.dark\:bg-fuchsia-400\/85:is(.dark *){background-color:rgba(232,121,249,.85)}.dark\:bg-fuchsia-400\/90:is(.dark *){background-color:rgba(232,121,249,.9)}.dark\:bg-fuchsia-400\/95:is(.dark *){background-color:rgba(232,121,249,.95)}.dark\:bg-fuchsia-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(253 244 255/var(--tw-bg-opacity,1))}.dark\:bg-fuchsia-50\/0:is(.dark *){background-color:rgba(253,244,255,0)}.dark\:bg-fuchsia-50\/10:is(.dark *){background-color:rgba(253,244,255,.1)}.dark\:bg-fuchsia-50\/100:is(.dark *){background-color:#fdf4ff}.dark\:bg-fuchsia-50\/15:is(.dark *){background-color:rgba(253,244,255,.15)}.dark\:bg-fuchsia-50\/20:is(.dark *){background-color:rgba(253,244,255,.2)}.dark\:bg-fuchsia-50\/25:is(.dark *){background-color:rgba(253,244,255,.25)}.dark\:bg-fuchsia-50\/30:is(.dark *){background-color:rgba(253,244,255,.3)}.dark\:bg-fuchsia-50\/35:is(.dark *){background-color:rgba(253,244,255,.35)}.dark\:bg-fuchsia-50\/40:is(.dark *){background-color:rgba(253,244,255,.4)}.dark\:bg-fuchsia-50\/45:is(.dark *){background-color:rgba(253,244,255,.45)}.dark\:bg-fuchsia-50\/5:is(.dark *){background-color:rgba(253,244,255,.05)}.dark\:bg-fuchsia-50\/50:is(.dark *){background-color:rgba(253,244,255,.5)}.dark\:bg-fuchsia-50\/55:is(.dark *){background-color:rgba(253,244,255,.55)}.dark\:bg-fuchsia-50\/60:is(.dark *){background-color:rgba(253,244,255,.6)}.dark\:bg-fuchsia-50\/65:is(.dark *){background-color:rgba(253,244,255,.65)}.dark\:bg-fuchsia-50\/70:is(.dark *){background-color:rgba(253,244,255,.7)}.dark\:bg-fuchsia-50\/75:is(.dark *){background-color:rgba(253,244,255,.75)}.dark\:bg-fuchsia-50\/80:is(.dark *){background-color:rgba(253,244,255,.8)}.dark\:bg-fuchsia-50\/85:is(.dark *){background-color:rgba(253,244,255,.85)}.dark\:bg-fuchsia-50\/90:is(.dark *){background-color:rgba(253,244,255,.9)}.dark\:bg-fuchsia-50\/95:is(.dark *){background-color:rgba(253,244,255,.95)}.dark\:bg-fuchsia-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(217 70 239/var(--tw-bg-opacity,1))}.dark\:bg-fuchsia-500\/0:is(.dark *){background-color:rgba(217,70,239,0)}.dark\:bg-fuchsia-500\/10:is(.dark *){background-color:rgba(217,70,239,.1)}.dark\:bg-fuchsia-500\/100:is(.dark *){background-color:#d946ef}.dark\:bg-fuchsia-500\/15:is(.dark *){background-color:rgba(217,70,239,.15)}.dark\:bg-fuchsia-500\/20:is(.dark *){background-color:rgba(217,70,239,.2)}.dark\:bg-fuchsia-500\/25:is(.dark *){background-color:rgba(217,70,239,.25)}.dark\:bg-fuchsia-500\/30:is(.dark *){background-color:rgba(217,70,239,.3)}.dark\:bg-fuchsia-500\/35:is(.dark *){background-color:rgba(217,70,239,.35)}.dark\:bg-fuchsia-500\/40:is(.dark *){background-color:rgba(217,70,239,.4)}.dark\:bg-fuchsia-500\/45:is(.dark *){background-color:rgba(217,70,239,.45)}.dark\:bg-fuchsia-500\/5:is(.dark *){background-color:rgba(217,70,239,.05)}.dark\:bg-fuchsia-500\/50:is(.dark *){background-color:rgba(217,70,239,.5)}.dark\:bg-fuchsia-500\/55:is(.dark *){background-color:rgba(217,70,239,.55)}.dark\:bg-fuchsia-500\/60:is(.dark *){background-color:rgba(217,70,239,.6)}.dark\:bg-fuchsia-500\/65:is(.dark *){background-color:rgba(217,70,239,.65)}.dark\:bg-fuchsia-500\/70:is(.dark *){background-color:rgba(217,70,239,.7)}.dark\:bg-fuchsia-500\/75:is(.dark *){background-color:rgba(217,70,239,.75)}.dark\:bg-fuchsia-500\/80:is(.dark *){background-color:rgba(217,70,239,.8)}.dark\:bg-fuchsia-500\/85:is(.dark *){background-color:rgba(217,70,239,.85)}.dark\:bg-fuchsia-500\/90:is(.dark *){background-color:rgba(217,70,239,.9)}.dark\:bg-fuchsia-500\/95:is(.dark *){background-color:rgba(217,70,239,.95)}.dark\:bg-fuchsia-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(74 4 78/var(--tw-bg-opacity,1))}.dark\:bg-fuchsia-950\/0:is(.dark *){background-color:rgba(74,4,78,0)}.dark\:bg-fuchsia-950\/10:is(.dark *){background-color:rgba(74,4,78,.1)}.dark\:bg-fuchsia-950\/100:is(.dark *){background-color:#4a044e}.dark\:bg-fuchsia-950\/15:is(.dark *){background-color:rgba(74,4,78,.15)}.dark\:bg-fuchsia-950\/20:is(.dark *){background-color:rgba(74,4,78,.2)}.dark\:bg-fuchsia-950\/25:is(.dark *){background-color:rgba(74,4,78,.25)}.dark\:bg-fuchsia-950\/30:is(.dark *){background-color:rgba(74,4,78,.3)}.dark\:bg-fuchsia-950\/35:is(.dark *){background-color:rgba(74,4,78,.35)}.dark\:bg-fuchsia-950\/40:is(.dark *){background-color:rgba(74,4,78,.4)}.dark\:bg-fuchsia-950\/45:is(.dark *){background-color:rgba(74,4,78,.45)}.dark\:bg-fuchsia-950\/5:is(.dark *){background-color:rgba(74,4,78,.05)}.dark\:bg-fuchsia-950\/50:is(.dark *){background-color:rgba(74,4,78,.5)}.dark\:bg-fuchsia-950\/55:is(.dark *){background-color:rgba(74,4,78,.55)}.dark\:bg-fuchsia-950\/60:is(.dark *){background-color:rgba(74,4,78,.6)}.dark\:bg-fuchsia-950\/65:is(.dark *){background-color:rgba(74,4,78,.65)}.dark\:bg-fuchsia-950\/70:is(.dark *){background-color:rgba(74,4,78,.7)}.dark\:bg-fuchsia-950\/75:is(.dark *){background-color:rgba(74,4,78,.75)}.dark\:bg-fuchsia-950\/80:is(.dark *){background-color:rgba(74,4,78,.8)}.dark\:bg-fuchsia-950\/85:is(.dark *){background-color:rgba(74,4,78,.85)}.dark\:bg-fuchsia-950\/90:is(.dark *){background-color:rgba(74,4,78,.9)}.dark\:bg-fuchsia-950\/95:is(.dark *){background-color:rgba(74,4,78,.95)}.dark\:bg-gray-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(243 244 246/var(--tw-bg-opacity,1))}.dark\:bg-gray-100\/0:is(.dark *){background-color:rgba(243,244,246,0)}.dark\:bg-gray-100\/10:is(.dark *){background-color:rgba(243,244,246,.1)}.dark\:bg-gray-100\/100:is(.dark *){background-color:#f3f4f6}.dark\:bg-gray-100\/15:is(.dark *){background-color:rgba(243,244,246,.15)}.dark\:bg-gray-100\/20:is(.dark *){background-color:rgba(243,244,246,.2)}.dark\:bg-gray-100\/25:is(.dark *){background-color:rgba(243,244,246,.25)}.dark\:bg-gray-100\/30:is(.dark *){background-color:rgba(243,244,246,.3)}.dark\:bg-gray-100\/35:is(.dark *){background-color:rgba(243,244,246,.35)}.dark\:bg-gray-100\/40:is(.dark *){background-color:rgba(243,244,246,.4)}.dark\:bg-gray-100\/45:is(.dark *){background-color:rgba(243,244,246,.45)}.dark\:bg-gray-100\/5:is(.dark *){background-color:rgba(243,244,246,.05)}.dark\:bg-gray-100\/50:is(.dark *){background-color:rgba(243,244,246,.5)}.dark\:bg-gray-100\/55:is(.dark *){background-color:rgba(243,244,246,.55)}.dark\:bg-gray-100\/60:is(.dark *){background-color:rgba(243,244,246,.6)}.dark\:bg-gray-100\/65:is(.dark *){background-color:rgba(243,244,246,.65)}.dark\:bg-gray-100\/70:is(.dark *){background-color:rgba(243,244,246,.7)}.dark\:bg-gray-100\/75:is(.dark *){background-color:rgba(243,244,246,.75)}.dark\:bg-gray-100\/80:is(.dark *){background-color:rgba(243,244,246,.8)}.dark\:bg-gray-100\/85:is(.dark *){background-color:rgba(243,244,246,.85)}.dark\:bg-gray-100\/90:is(.dark *){background-color:rgba(243,244,246,.9)}.dark\:bg-gray-100\/95:is(.dark *){background-color:rgba(243,244,246,.95)}.dark\:bg-gray-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(229 231 235/var(--tw-bg-opacity,1))}.dark\:bg-gray-200\/0:is(.dark *){background-color:rgba(229,231,235,0)}.dark\:bg-gray-200\/10:is(.dark *){background-color:rgba(229,231,235,.1)}.dark\:bg-gray-200\/100:is(.dark *){background-color:#e5e7eb}.dark\:bg-gray-200\/15:is(.dark *){background-color:rgba(229,231,235,.15)}.dark\:bg-gray-200\/20:is(.dark *){background-color:rgba(229,231,235,.2)}.dark\:bg-gray-200\/25:is(.dark *){background-color:rgba(229,231,235,.25)}.dark\:bg-gray-200\/30:is(.dark *){background-color:rgba(229,231,235,.3)}.dark\:bg-gray-200\/35:is(.dark *){background-color:rgba(229,231,235,.35)}.dark\:bg-gray-200\/40:is(.dark *){background-color:rgba(229,231,235,.4)}.dark\:bg-gray-200\/45:is(.dark *){background-color:rgba(229,231,235,.45)}.dark\:bg-gray-200\/5:is(.dark *){background-color:rgba(229,231,235,.05)}.dark\:bg-gray-200\/50:is(.dark *){background-color:rgba(229,231,235,.5)}.dark\:bg-gray-200\/55:is(.dark *){background-color:rgba(229,231,235,.55)}.dark\:bg-gray-200\/60:is(.dark *){background-color:rgba(229,231,235,.6)}.dark\:bg-gray-200\/65:is(.dark *){background-color:rgba(229,231,235,.65)}.dark\:bg-gray-200\/70:is(.dark *){background-color:rgba(229,231,235,.7)}.dark\:bg-gray-200\/75:is(.dark *){background-color:rgba(229,231,235,.75)}.dark\:bg-gray-200\/80:is(.dark *){background-color:rgba(229,231,235,.8)}.dark\:bg-gray-200\/85:is(.dark *){background-color:rgba(229,231,235,.85)}.dark\:bg-gray-200\/90:is(.dark *){background-color:rgba(229,231,235,.9)}.dark\:bg-gray-200\/95:is(.dark *){background-color:rgba(229,231,235,.95)}.dark\:bg-gray-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(156 163 175/var(--tw-bg-opacity,1))}.dark\:bg-gray-400\/0:is(.dark *){background-color:rgba(156,163,175,0)}.dark\:bg-gray-400\/10:is(.dark *){background-color:rgba(156,163,175,.1)}.dark\:bg-gray-400\/100:is(.dark *){background-color:#9ca3af}.dark\:bg-gray-400\/15:is(.dark *){background-color:rgba(156,163,175,.15)}.dark\:bg-gray-400\/20:is(.dark *){background-color:rgba(156,163,175,.2)}.dark\:bg-gray-400\/25:is(.dark *){background-color:rgba(156,163,175,.25)}.dark\:bg-gray-400\/30:is(.dark *){background-color:rgba(156,163,175,.3)}.dark\:bg-gray-400\/35:is(.dark *){background-color:rgba(156,163,175,.35)}.dark\:bg-gray-400\/40:is(.dark *){background-color:rgba(156,163,175,.4)}.dark\:bg-gray-400\/45:is(.dark *){background-color:rgba(156,163,175,.45)}.dark\:bg-gray-400\/5:is(.dark *){background-color:rgba(156,163,175,.05)}.dark\:bg-gray-400\/50:is(.dark *){background-color:rgba(156,163,175,.5)}.dark\:bg-gray-400\/55:is(.dark *){background-color:rgba(156,163,175,.55)}.dark\:bg-gray-400\/60:is(.dark *){background-color:rgba(156,163,175,.6)}.dark\:bg-gray-400\/65:is(.dark *){background-color:rgba(156,163,175,.65)}.dark\:bg-gray-400\/70:is(.dark *){background-color:rgba(156,163,175,.7)}.dark\:bg-gray-400\/75:is(.dark *){background-color:rgba(156,163,175,.75)}.dark\:bg-gray-400\/80:is(.dark *){background-color:rgba(156,163,175,.8)}.dark\:bg-gray-400\/85:is(.dark *){background-color:rgba(156,163,175,.85)}.dark\:bg-gray-400\/90:is(.dark *){background-color:rgba(156,163,175,.9)}.dark\:bg-gray-400\/95:is(.dark *){background-color:rgba(156,163,175,.95)}.dark\:bg-gray-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(249 250 251/var(--tw-bg-opacity,1))}.dark\:bg-gray-50\/0:is(.dark *){background-color:rgba(249,250,251,0)}.dark\:bg-gray-50\/10:is(.dark *){background-color:rgba(249,250,251,.1)}.dark\:bg-gray-50\/100:is(.dark *){background-color:#f9fafb}.dark\:bg-gray-50\/15:is(.dark *){background-color:rgba(249,250,251,.15)}.dark\:bg-gray-50\/20:is(.dark *){background-color:rgba(249,250,251,.2)}.dark\:bg-gray-50\/25:is(.dark *){background-color:rgba(249,250,251,.25)}.dark\:bg-gray-50\/30:is(.dark *){background-color:rgba(249,250,251,.3)}.dark\:bg-gray-50\/35:is(.dark *){background-color:rgba(249,250,251,.35)}.dark\:bg-gray-50\/40:is(.dark *){background-color:rgba(249,250,251,.4)}.dark\:bg-gray-50\/45:is(.dark *){background-color:rgba(249,250,251,.45)}.dark\:bg-gray-50\/5:is(.dark *){background-color:rgba(249,250,251,.05)}.dark\:bg-gray-50\/50:is(.dark *){background-color:rgba(249,250,251,.5)}.dark\:bg-gray-50\/55:is(.dark *){background-color:rgba(249,250,251,.55)}.dark\:bg-gray-50\/60:is(.dark *){background-color:rgba(249,250,251,.6)}.dark\:bg-gray-50\/65:is(.dark *){background-color:rgba(249,250,251,.65)}.dark\:bg-gray-50\/70:is(.dark *){background-color:rgba(249,250,251,.7)}.dark\:bg-gray-50\/75:is(.dark *){background-color:rgba(249,250,251,.75)}.dark\:bg-gray-50\/80:is(.dark *){background-color:rgba(249,250,251,.8)}.dark\:bg-gray-50\/85:is(.dark *){background-color:rgba(249,250,251,.85)}.dark\:bg-gray-50\/90:is(.dark *){background-color:rgba(249,250,251,.9)}.dark\:bg-gray-50\/95:is(.dark *){background-color:rgba(249,250,251,.95)}.dark\:bg-gray-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(107 114 128/var(--tw-bg-opacity,1))}.dark\:bg-gray-500\/0:is(.dark *){background-color:hsla(220,9%,46%,0)}.dark\:bg-gray-500\/10:is(.dark *){background-color:hsla(220,9%,46%,.1)}.dark\:bg-gray-500\/100:is(.dark *){background-color:#6b7280}.dark\:bg-gray-500\/15:is(.dark *){background-color:hsla(220,9%,46%,.15)}.dark\:bg-gray-500\/20:is(.dark *){background-color:hsla(220,9%,46%,.2)}.dark\:bg-gray-500\/25:is(.dark *){background-color:hsla(220,9%,46%,.25)}.dark\:bg-gray-500\/30:is(.dark *){background-color:hsla(220,9%,46%,.3)}.dark\:bg-gray-500\/35:is(.dark *){background-color:hsla(220,9%,46%,.35)}.dark\:bg-gray-500\/40:is(.dark *){background-color:hsla(220,9%,46%,.4)}.dark\:bg-gray-500\/45:is(.dark *){background-color:hsla(220,9%,46%,.45)}.dark\:bg-gray-500\/5:is(.dark *){background-color:hsla(220,9%,46%,.05)}.dark\:bg-gray-500\/50:is(.dark *){background-color:hsla(220,9%,46%,.5)}.dark\:bg-gray-500\/55:is(.dark *){background-color:hsla(220,9%,46%,.55)}.dark\:bg-gray-500\/60:is(.dark *){background-color:hsla(220,9%,46%,.6)}.dark\:bg-gray-500\/65:is(.dark *){background-color:hsla(220,9%,46%,.65)}.dark\:bg-gray-500\/70:is(.dark *){background-color:hsla(220,9%,46%,.7)}.dark\:bg-gray-500\/75:is(.dark *){background-color:hsla(220,9%,46%,.75)}.dark\:bg-gray-500\/80:is(.dark *){background-color:hsla(220,9%,46%,.8)}.dark\:bg-gray-500\/85:is(.dark *){background-color:hsla(220,9%,46%,.85)}.dark\:bg-gray-500\/90:is(.dark *){background-color:hsla(220,9%,46%,.9)}.dark\:bg-gray-500\/95:is(.dark *){background-color:hsla(220,9%,46%,.95)}.dark\:bg-gray-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(3 7 18/var(--tw-bg-opacity,1))}.dark\:bg-gray-950\/0:is(.dark *){background-color:rgba(3,7,18,0)}.dark\:bg-gray-950\/10:is(.dark *){background-color:rgba(3,7,18,.1)}.dark\:bg-gray-950\/100:is(.dark *){background-color:#030712}.dark\:bg-gray-950\/15:is(.dark *){background-color:rgba(3,7,18,.15)}.dark\:bg-gray-950\/20:is(.dark *){background-color:rgba(3,7,18,.2)}.dark\:bg-gray-950\/25:is(.dark *){background-color:rgba(3,7,18,.25)}.dark\:bg-gray-950\/30:is(.dark *){background-color:rgba(3,7,18,.3)}.dark\:bg-gray-950\/35:is(.dark *){background-color:rgba(3,7,18,.35)}.dark\:bg-gray-950\/40:is(.dark *){background-color:rgba(3,7,18,.4)}.dark\:bg-gray-950\/45:is(.dark *){background-color:rgba(3,7,18,.45)}.dark\:bg-gray-950\/5:is(.dark *){background-color:rgba(3,7,18,.05)}.dark\:bg-gray-950\/50:is(.dark *){background-color:rgba(3,7,18,.5)}.dark\:bg-gray-950\/55:is(.dark *){background-color:rgba(3,7,18,.55)}.dark\:bg-gray-950\/60:is(.dark *){background-color:rgba(3,7,18,.6)}.dark\:bg-gray-950\/65:is(.dark *){background-color:rgba(3,7,18,.65)}.dark\:bg-gray-950\/70:is(.dark *){background-color:rgba(3,7,18,.7)}.dark\:bg-gray-950\/75:is(.dark *){background-color:rgba(3,7,18,.75)}.dark\:bg-gray-950\/80:is(.dark *){background-color:rgba(3,7,18,.8)}.dark\:bg-gray-950\/85:is(.dark *){background-color:rgba(3,7,18,.85)}.dark\:bg-gray-950\/90:is(.dark *){background-color:rgba(3,7,18,.9)}.dark\:bg-gray-950\/95:is(.dark *){background-color:rgba(3,7,18,.95)}.dark\:bg-green-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(220 252 231/var(--tw-bg-opacity,1))}.dark\:bg-green-100\/0:is(.dark *){background-color:rgba(220,252,231,0)}.dark\:bg-green-100\/10:is(.dark *){background-color:rgba(220,252,231,.1)}.dark\:bg-green-100\/100:is(.dark *){background-color:#dcfce7}.dark\:bg-green-100\/15:is(.dark *){background-color:rgba(220,252,231,.15)}.dark\:bg-green-100\/20:is(.dark *){background-color:rgba(220,252,231,.2)}.dark\:bg-green-100\/25:is(.dark *){background-color:rgba(220,252,231,.25)}.dark\:bg-green-100\/30:is(.dark *){background-color:rgba(220,252,231,.3)}.dark\:bg-green-100\/35:is(.dark *){background-color:rgba(220,252,231,.35)}.dark\:bg-green-100\/40:is(.dark *){background-color:rgba(220,252,231,.4)}.dark\:bg-green-100\/45:is(.dark *){background-color:rgba(220,252,231,.45)}.dark\:bg-green-100\/5:is(.dark *){background-color:rgba(220,252,231,.05)}.dark\:bg-green-100\/50:is(.dark *){background-color:rgba(220,252,231,.5)}.dark\:bg-green-100\/55:is(.dark *){background-color:rgba(220,252,231,.55)}.dark\:bg-green-100\/60:is(.dark *){background-color:rgba(220,252,231,.6)}.dark\:bg-green-100\/65:is(.dark *){background-color:rgba(220,252,231,.65)}.dark\:bg-green-100\/70:is(.dark *){background-color:rgba(220,252,231,.7)}.dark\:bg-green-100\/75:is(.dark *){background-color:rgba(220,252,231,.75)}.dark\:bg-green-100\/80:is(.dark *){background-color:rgba(220,252,231,.8)}.dark\:bg-green-100\/85:is(.dark *){background-color:rgba(220,252,231,.85)}.dark\:bg-green-100\/90:is(.dark *){background-color:rgba(220,252,231,.9)}.dark\:bg-green-100\/95:is(.dark *){background-color:rgba(220,252,231,.95)}.dark\:bg-green-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(187 247 208/var(--tw-bg-opacity,1))}.dark\:bg-green-200\/0:is(.dark *){background-color:rgba(187,247,208,0)}.dark\:bg-green-200\/10:is(.dark *){background-color:rgba(187,247,208,.1)}.dark\:bg-green-200\/100:is(.dark *){background-color:#bbf7d0}.dark\:bg-green-200\/15:is(.dark *){background-color:rgba(187,247,208,.15)}.dark\:bg-green-200\/20:is(.dark *){background-color:rgba(187,247,208,.2)}.dark\:bg-green-200\/25:is(.dark *){background-color:rgba(187,247,208,.25)}.dark\:bg-green-200\/30:is(.dark *){background-color:rgba(187,247,208,.3)}.dark\:bg-green-200\/35:is(.dark *){background-color:rgba(187,247,208,.35)}.dark\:bg-green-200\/40:is(.dark *){background-color:rgba(187,247,208,.4)}.dark\:bg-green-200\/45:is(.dark *){background-color:rgba(187,247,208,.45)}.dark\:bg-green-200\/5:is(.dark *){background-color:rgba(187,247,208,.05)}.dark\:bg-green-200\/50:is(.dark *){background-color:rgba(187,247,208,.5)}.dark\:bg-green-200\/55:is(.dark *){background-color:rgba(187,247,208,.55)}.dark\:bg-green-200\/60:is(.dark *){background-color:rgba(187,247,208,.6)}.dark\:bg-green-200\/65:is(.dark *){background-color:rgba(187,247,208,.65)}.dark\:bg-green-200\/70:is(.dark *){background-color:rgba(187,247,208,.7)}.dark\:bg-green-200\/75:is(.dark *){background-color:rgba(187,247,208,.75)}.dark\:bg-green-200\/80:is(.dark *){background-color:rgba(187,247,208,.8)}.dark\:bg-green-200\/85:is(.dark *){background-color:rgba(187,247,208,.85)}.dark\:bg-green-200\/90:is(.dark *){background-color:rgba(187,247,208,.9)}.dark\:bg-green-200\/95:is(.dark *){background-color:rgba(187,247,208,.95)}.dark\:bg-green-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(74 222 128/var(--tw-bg-opacity,1))}.dark\:bg-green-400\/0:is(.dark *){background-color:rgba(74,222,128,0)}.dark\:bg-green-400\/10:is(.dark *){background-color:rgba(74,222,128,.1)}.dark\:bg-green-400\/100:is(.dark *){background-color:#4ade80}.dark\:bg-green-400\/15:is(.dark *){background-color:rgba(74,222,128,.15)}.dark\:bg-green-400\/20:is(.dark *){background-color:rgba(74,222,128,.2)}.dark\:bg-green-400\/25:is(.dark *){background-color:rgba(74,222,128,.25)}.dark\:bg-green-400\/30:is(.dark *){background-color:rgba(74,222,128,.3)}.dark\:bg-green-400\/35:is(.dark *){background-color:rgba(74,222,128,.35)}.dark\:bg-green-400\/40:is(.dark *){background-color:rgba(74,222,128,.4)}.dark\:bg-green-400\/45:is(.dark *){background-color:rgba(74,222,128,.45)}.dark\:bg-green-400\/5:is(.dark *){background-color:rgba(74,222,128,.05)}.dark\:bg-green-400\/50:is(.dark *){background-color:rgba(74,222,128,.5)}.dark\:bg-green-400\/55:is(.dark *){background-color:rgba(74,222,128,.55)}.dark\:bg-green-400\/60:is(.dark *){background-color:rgba(74,222,128,.6)}.dark\:bg-green-400\/65:is(.dark *){background-color:rgba(74,222,128,.65)}.dark\:bg-green-400\/70:is(.dark *){background-color:rgba(74,222,128,.7)}.dark\:bg-green-400\/75:is(.dark *){background-color:rgba(74,222,128,.75)}.dark\:bg-green-400\/80:is(.dark *){background-color:rgba(74,222,128,.8)}.dark\:bg-green-400\/85:is(.dark *){background-color:rgba(74,222,128,.85)}.dark\:bg-green-400\/90:is(.dark *){background-color:rgba(74,222,128,.9)}.dark\:bg-green-400\/95:is(.dark *){background-color:rgba(74,222,128,.95)}.dark\:bg-green-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(240 253 244/var(--tw-bg-opacity,1))}.dark\:bg-green-50\/0:is(.dark *){background-color:rgba(240,253,244,0)}.dark\:bg-green-50\/10:is(.dark *){background-color:rgba(240,253,244,.1)}.dark\:bg-green-50\/100:is(.dark *){background-color:#f0fdf4}.dark\:bg-green-50\/15:is(.dark *){background-color:rgba(240,253,244,.15)}.dark\:bg-green-50\/20:is(.dark *){background-color:rgba(240,253,244,.2)}.dark\:bg-green-50\/25:is(.dark *){background-color:rgba(240,253,244,.25)}.dark\:bg-green-50\/30:is(.dark *){background-color:rgba(240,253,244,.3)}.dark\:bg-green-50\/35:is(.dark *){background-color:rgba(240,253,244,.35)}.dark\:bg-green-50\/40:is(.dark *){background-color:rgba(240,253,244,.4)}.dark\:bg-green-50\/45:is(.dark *){background-color:rgba(240,253,244,.45)}.dark\:bg-green-50\/5:is(.dark *){background-color:rgba(240,253,244,.05)}.dark\:bg-green-50\/50:is(.dark *){background-color:rgba(240,253,244,.5)}.dark\:bg-green-50\/55:is(.dark *){background-color:rgba(240,253,244,.55)}.dark\:bg-green-50\/60:is(.dark *){background-color:rgba(240,253,244,.6)}.dark\:bg-green-50\/65:is(.dark *){background-color:rgba(240,253,244,.65)}.dark\:bg-green-50\/70:is(.dark *){background-color:rgba(240,253,244,.7)}.dark\:bg-green-50\/75:is(.dark *){background-color:rgba(240,253,244,.75)}.dark\:bg-green-50\/80:is(.dark *){background-color:rgba(240,253,244,.8)}.dark\:bg-green-50\/85:is(.dark *){background-color:rgba(240,253,244,.85)}.dark\:bg-green-50\/90:is(.dark *){background-color:rgba(240,253,244,.9)}.dark\:bg-green-50\/95:is(.dark *){background-color:rgba(240,253,244,.95)}.dark\:bg-green-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(34 197 94/var(--tw-bg-opacity,1))}.dark\:bg-green-500\/0:is(.dark *){background-color:rgba(34,197,94,0)}.dark\:bg-green-500\/10:is(.dark *){background-color:rgba(34,197,94,.1)}.dark\:bg-green-500\/100:is(.dark *){background-color:#22c55e}.dark\:bg-green-500\/15:is(.dark *){background-color:rgba(34,197,94,.15)}.dark\:bg-green-500\/20:is(.dark *){background-color:rgba(34,197,94,.2)}.dark\:bg-green-500\/25:is(.dark *){background-color:rgba(34,197,94,.25)}.dark\:bg-green-500\/30:is(.dark *){background-color:rgba(34,197,94,.3)}.dark\:bg-green-500\/35:is(.dark *){background-color:rgba(34,197,94,.35)}.dark\:bg-green-500\/40:is(.dark *){background-color:rgba(34,197,94,.4)}.dark\:bg-green-500\/45:is(.dark *){background-color:rgba(34,197,94,.45)}.dark\:bg-green-500\/5:is(.dark *){background-color:rgba(34,197,94,.05)}.dark\:bg-green-500\/50:is(.dark *){background-color:rgba(34,197,94,.5)}.dark\:bg-green-500\/55:is(.dark *){background-color:rgba(34,197,94,.55)}.dark\:bg-green-500\/60:is(.dark *){background-color:rgba(34,197,94,.6)}.dark\:bg-green-500\/65:is(.dark *){background-color:rgba(34,197,94,.65)}.dark\:bg-green-500\/70:is(.dark *){background-color:rgba(34,197,94,.7)}.dark\:bg-green-500\/75:is(.dark *){background-color:rgba(34,197,94,.75)}.dark\:bg-green-500\/80:is(.dark *){background-color:rgba(34,197,94,.8)}.dark\:bg-green-500\/85:is(.dark *){background-color:rgba(34,197,94,.85)}.dark\:bg-green-500\/90:is(.dark *){background-color:rgba(34,197,94,.9)}.dark\:bg-green-500\/95:is(.dark *){background-color:rgba(34,197,94,.95)}.dark\:bg-green-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(5 46 22/var(--tw-bg-opacity,1))}.dark\:bg-green-950\/0:is(.dark *){background-color:rgba(5,46,22,0)}.dark\:bg-green-950\/10:is(.dark *){background-color:rgba(5,46,22,.1)}.dark\:bg-green-950\/100:is(.dark *){background-color:#052e16}.dark\:bg-green-950\/15:is(.dark *){background-color:rgba(5,46,22,.15)}.dark\:bg-green-950\/20:is(.dark *){background-color:rgba(5,46,22,.2)}.dark\:bg-green-950\/25:is(.dark *){background-color:rgba(5,46,22,.25)}.dark\:bg-green-950\/30:is(.dark *){background-color:rgba(5,46,22,.3)}.dark\:bg-green-950\/35:is(.dark *){background-color:rgba(5,46,22,.35)}.dark\:bg-green-950\/40:is(.dark *){background-color:rgba(5,46,22,.4)}.dark\:bg-green-950\/45:is(.dark *){background-color:rgba(5,46,22,.45)}.dark\:bg-green-950\/5:is(.dark *){background-color:rgba(5,46,22,.05)}.dark\:bg-green-950\/50:is(.dark *){background-color:rgba(5,46,22,.5)}.dark\:bg-green-950\/55:is(.dark *){background-color:rgba(5,46,22,.55)}.dark\:bg-green-950\/60:is(.dark *){background-color:rgba(5,46,22,.6)}.dark\:bg-green-950\/65:is(.dark *){background-color:rgba(5,46,22,.65)}.dark\:bg-green-950\/70:is(.dark *){background-color:rgba(5,46,22,.7)}.dark\:bg-green-950\/75:is(.dark *){background-color:rgba(5,46,22,.75)}.dark\:bg-green-950\/80:is(.dark *){background-color:rgba(5,46,22,.8)}.dark\:bg-green-950\/85:is(.dark *){background-color:rgba(5,46,22,.85)}.dark\:bg-green-950\/90:is(.dark *){background-color:rgba(5,46,22,.9)}.dark\:bg-green-950\/95:is(.dark *){background-color:rgba(5,46,22,.95)}.dark\:bg-indigo-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(224 231 255/var(--tw-bg-opacity,1))}.dark\:bg-indigo-100\/0:is(.dark *){background-color:rgba(224,231,255,0)}.dark\:bg-indigo-100\/10:is(.dark *){background-color:rgba(224,231,255,.1)}.dark\:bg-indigo-100\/100:is(.dark *){background-color:#e0e7ff}.dark\:bg-indigo-100\/15:is(.dark *){background-color:rgba(224,231,255,.15)}.dark\:bg-indigo-100\/20:is(.dark *){background-color:rgba(224,231,255,.2)}.dark\:bg-indigo-100\/25:is(.dark *){background-color:rgba(224,231,255,.25)}.dark\:bg-indigo-100\/30:is(.dark *){background-color:rgba(224,231,255,.3)}.dark\:bg-indigo-100\/35:is(.dark *){background-color:rgba(224,231,255,.35)}.dark\:bg-indigo-100\/40:is(.dark *){background-color:rgba(224,231,255,.4)}.dark\:bg-indigo-100\/45:is(.dark *){background-color:rgba(224,231,255,.45)}.dark\:bg-indigo-100\/5:is(.dark *){background-color:rgba(224,231,255,.05)}.dark\:bg-indigo-100\/50:is(.dark *){background-color:rgba(224,231,255,.5)}.dark\:bg-indigo-100\/55:is(.dark *){background-color:rgba(224,231,255,.55)}.dark\:bg-indigo-100\/60:is(.dark *){background-color:rgba(224,231,255,.6)}.dark\:bg-indigo-100\/65:is(.dark *){background-color:rgba(224,231,255,.65)}.dark\:bg-indigo-100\/70:is(.dark *){background-color:rgba(224,231,255,.7)}.dark\:bg-indigo-100\/75:is(.dark *){background-color:rgba(224,231,255,.75)}.dark\:bg-indigo-100\/80:is(.dark *){background-color:rgba(224,231,255,.8)}.dark\:bg-indigo-100\/85:is(.dark *){background-color:rgba(224,231,255,.85)}.dark\:bg-indigo-100\/90:is(.dark *){background-color:rgba(224,231,255,.9)}.dark\:bg-indigo-100\/95:is(.dark *){background-color:rgba(224,231,255,.95)}.dark\:bg-indigo-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(199 210 254/var(--tw-bg-opacity,1))}.dark\:bg-indigo-200\/0:is(.dark *){background-color:rgba(199,210,254,0)}.dark\:bg-indigo-200\/10:is(.dark *){background-color:rgba(199,210,254,.1)}.dark\:bg-indigo-200\/100:is(.dark *){background-color:#c7d2fe}.dark\:bg-indigo-200\/15:is(.dark *){background-color:rgba(199,210,254,.15)}.dark\:bg-indigo-200\/20:is(.dark *){background-color:rgba(199,210,254,.2)}.dark\:bg-indigo-200\/25:is(.dark *){background-color:rgba(199,210,254,.25)}.dark\:bg-indigo-200\/30:is(.dark *){background-color:rgba(199,210,254,.3)}.dark\:bg-indigo-200\/35:is(.dark *){background-color:rgba(199,210,254,.35)}.dark\:bg-indigo-200\/40:is(.dark *){background-color:rgba(199,210,254,.4)}.dark\:bg-indigo-200\/45:is(.dark *){background-color:rgba(199,210,254,.45)}.dark\:bg-indigo-200\/5:is(.dark *){background-color:rgba(199,210,254,.05)}.dark\:bg-indigo-200\/50:is(.dark *){background-color:rgba(199,210,254,.5)}.dark\:bg-indigo-200\/55:is(.dark *){background-color:rgba(199,210,254,.55)}.dark\:bg-indigo-200\/60:is(.dark *){background-color:rgba(199,210,254,.6)}.dark\:bg-indigo-200\/65:is(.dark *){background-color:rgba(199,210,254,.65)}.dark\:bg-indigo-200\/70:is(.dark *){background-color:rgba(199,210,254,.7)}.dark\:bg-indigo-200\/75:is(.dark *){background-color:rgba(199,210,254,.75)}.dark\:bg-indigo-200\/80:is(.dark *){background-color:rgba(199,210,254,.8)}.dark\:bg-indigo-200\/85:is(.dark *){background-color:rgba(199,210,254,.85)}.dark\:bg-indigo-200\/90:is(.dark *){background-color:rgba(199,210,254,.9)}.dark\:bg-indigo-200\/95:is(.dark *){background-color:rgba(199,210,254,.95)}.dark\:bg-indigo-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(129 140 248/var(--tw-bg-opacity,1))}.dark\:bg-indigo-400\/0:is(.dark *){background-color:rgba(129,140,248,0)}.dark\:bg-indigo-400\/10:is(.dark *){background-color:rgba(129,140,248,.1)}.dark\:bg-indigo-400\/100:is(.dark *){background-color:#818cf8}.dark\:bg-indigo-400\/15:is(.dark *){background-color:rgba(129,140,248,.15)}.dark\:bg-indigo-400\/20:is(.dark *){background-color:rgba(129,140,248,.2)}.dark\:bg-indigo-400\/25:is(.dark *){background-color:rgba(129,140,248,.25)}.dark\:bg-indigo-400\/30:is(.dark *){background-color:rgba(129,140,248,.3)}.dark\:bg-indigo-400\/35:is(.dark *){background-color:rgba(129,140,248,.35)}.dark\:bg-indigo-400\/40:is(.dark *){background-color:rgba(129,140,248,.4)}.dark\:bg-indigo-400\/45:is(.dark *){background-color:rgba(129,140,248,.45)}.dark\:bg-indigo-400\/5:is(.dark *){background-color:rgba(129,140,248,.05)}.dark\:bg-indigo-400\/50:is(.dark *){background-color:rgba(129,140,248,.5)}.dark\:bg-indigo-400\/55:is(.dark *){background-color:rgba(129,140,248,.55)}.dark\:bg-indigo-400\/60:is(.dark *){background-color:rgba(129,140,248,.6)}.dark\:bg-indigo-400\/65:is(.dark *){background-color:rgba(129,140,248,.65)}.dark\:bg-indigo-400\/70:is(.dark *){background-color:rgba(129,140,248,.7)}.dark\:bg-indigo-400\/75:is(.dark *){background-color:rgba(129,140,248,.75)}.dark\:bg-indigo-400\/80:is(.dark *){background-color:rgba(129,140,248,.8)}.dark\:bg-indigo-400\/85:is(.dark *){background-color:rgba(129,140,248,.85)}.dark\:bg-indigo-400\/90:is(.dark *){background-color:rgba(129,140,248,.9)}.dark\:bg-indigo-400\/95:is(.dark *){background-color:rgba(129,140,248,.95)}.dark\:bg-indigo-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(238 242 255/var(--tw-bg-opacity,1))}.dark\:bg-indigo-50\/0:is(.dark *){background-color:rgba(238,242,255,0)}.dark\:bg-indigo-50\/10:is(.dark *){background-color:rgba(238,242,255,.1)}.dark\:bg-indigo-50\/100:is(.dark *){background-color:#eef2ff}.dark\:bg-indigo-50\/15:is(.dark *){background-color:rgba(238,242,255,.15)}.dark\:bg-indigo-50\/20:is(.dark *){background-color:rgba(238,242,255,.2)}.dark\:bg-indigo-50\/25:is(.dark *){background-color:rgba(238,242,255,.25)}.dark\:bg-indigo-50\/30:is(.dark *){background-color:rgba(238,242,255,.3)}.dark\:bg-indigo-50\/35:is(.dark *){background-color:rgba(238,242,255,.35)}.dark\:bg-indigo-50\/40:is(.dark *){background-color:rgba(238,242,255,.4)}.dark\:bg-indigo-50\/45:is(.dark *){background-color:rgba(238,242,255,.45)}.dark\:bg-indigo-50\/5:is(.dark *){background-color:rgba(238,242,255,.05)}.dark\:bg-indigo-50\/50:is(.dark *){background-color:rgba(238,242,255,.5)}.dark\:bg-indigo-50\/55:is(.dark *){background-color:rgba(238,242,255,.55)}.dark\:bg-indigo-50\/60:is(.dark *){background-color:rgba(238,242,255,.6)}.dark\:bg-indigo-50\/65:is(.dark *){background-color:rgba(238,242,255,.65)}.dark\:bg-indigo-50\/70:is(.dark *){background-color:rgba(238,242,255,.7)}.dark\:bg-indigo-50\/75:is(.dark *){background-color:rgba(238,242,255,.75)}.dark\:bg-indigo-50\/80:is(.dark *){background-color:rgba(238,242,255,.8)}.dark\:bg-indigo-50\/85:is(.dark *){background-color:rgba(238,242,255,.85)}.dark\:bg-indigo-50\/90:is(.dark *){background-color:rgba(238,242,255,.9)}.dark\:bg-indigo-50\/95:is(.dark *){background-color:rgba(238,242,255,.95)}.dark\:bg-indigo-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(99 102 241/var(--tw-bg-opacity,1))}.dark\:bg-indigo-500\/0:is(.dark *){background-color:rgba(99,102,241,0)}.dark\:bg-indigo-500\/10:is(.dark *){background-color:rgba(99,102,241,.1)}.dark\:bg-indigo-500\/100:is(.dark *){background-color:#6366f1}.dark\:bg-indigo-500\/15:is(.dark *){background-color:rgba(99,102,241,.15)}.dark\:bg-indigo-500\/20:is(.dark *){background-color:rgba(99,102,241,.2)}.dark\:bg-indigo-500\/25:is(.dark *){background-color:rgba(99,102,241,.25)}.dark\:bg-indigo-500\/30:is(.dark *){background-color:rgba(99,102,241,.3)}.dark\:bg-indigo-500\/35:is(.dark *){background-color:rgba(99,102,241,.35)}.dark\:bg-indigo-500\/40:is(.dark *){background-color:rgba(99,102,241,.4)}.dark\:bg-indigo-500\/45:is(.dark *){background-color:rgba(99,102,241,.45)}.dark\:bg-indigo-500\/5:is(.dark *){background-color:rgba(99,102,241,.05)}.dark\:bg-indigo-500\/50:is(.dark *){background-color:rgba(99,102,241,.5)}.dark\:bg-indigo-500\/55:is(.dark *){background-color:rgba(99,102,241,.55)}.dark\:bg-indigo-500\/60:is(.dark *){background-color:rgba(99,102,241,.6)}.dark\:bg-indigo-500\/65:is(.dark *){background-color:rgba(99,102,241,.65)}.dark\:bg-indigo-500\/70:is(.dark *){background-color:rgba(99,102,241,.7)}.dark\:bg-indigo-500\/75:is(.dark *){background-color:rgba(99,102,241,.75)}.dark\:bg-indigo-500\/80:is(.dark *){background-color:rgba(99,102,241,.8)}.dark\:bg-indigo-500\/85:is(.dark *){background-color:rgba(99,102,241,.85)}.dark\:bg-indigo-500\/90:is(.dark *){background-color:rgba(99,102,241,.9)}.dark\:bg-indigo-500\/95:is(.dark *){background-color:rgba(99,102,241,.95)}.dark\:bg-indigo-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(30 27 75/var(--tw-bg-opacity,1))}.dark\:bg-indigo-950\/0:is(.dark *){background-color:rgba(30,27,75,0)}.dark\:bg-indigo-950\/10:is(.dark *){background-color:rgba(30,27,75,.1)}.dark\:bg-indigo-950\/100:is(.dark *){background-color:#1e1b4b}.dark\:bg-indigo-950\/15:is(.dark *){background-color:rgba(30,27,75,.15)}.dark\:bg-indigo-950\/20:is(.dark *){background-color:rgba(30,27,75,.2)}.dark\:bg-indigo-950\/25:is(.dark *){background-color:rgba(30,27,75,.25)}.dark\:bg-indigo-950\/30:is(.dark *){background-color:rgba(30,27,75,.3)}.dark\:bg-indigo-950\/35:is(.dark *){background-color:rgba(30,27,75,.35)}.dark\:bg-indigo-950\/40:is(.dark *){background-color:rgba(30,27,75,.4)}.dark\:bg-indigo-950\/45:is(.dark *){background-color:rgba(30,27,75,.45)}.dark\:bg-indigo-950\/5:is(.dark *){background-color:rgba(30,27,75,.05)}.dark\:bg-indigo-950\/50:is(.dark *){background-color:rgba(30,27,75,.5)}.dark\:bg-indigo-950\/55:is(.dark *){background-color:rgba(30,27,75,.55)}.dark\:bg-indigo-950\/60:is(.dark *){background-color:rgba(30,27,75,.6)}.dark\:bg-indigo-950\/65:is(.dark *){background-color:rgba(30,27,75,.65)}.dark\:bg-indigo-950\/70:is(.dark *){background-color:rgba(30,27,75,.7)}.dark\:bg-indigo-950\/75:is(.dark *){background-color:rgba(30,27,75,.75)}.dark\:bg-indigo-950\/80:is(.dark *){background-color:rgba(30,27,75,.8)}.dark\:bg-indigo-950\/85:is(.dark *){background-color:rgba(30,27,75,.85)}.dark\:bg-indigo-950\/90:is(.dark *){background-color:rgba(30,27,75,.9)}.dark\:bg-indigo-950\/95:is(.dark *){background-color:rgba(30,27,75,.95)}.dark\:bg-lime-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(236 252 203/var(--tw-bg-opacity,1))}.dark\:bg-lime-100\/0:is(.dark *){background-color:hsla(80,89%,89%,0)}.dark\:bg-lime-100\/10:is(.dark *){background-color:hsla(80,89%,89%,.1)}.dark\:bg-lime-100\/100:is(.dark *){background-color:#ecfccb}.dark\:bg-lime-100\/15:is(.dark *){background-color:hsla(80,89%,89%,.15)}.dark\:bg-lime-100\/20:is(.dark *){background-color:hsla(80,89%,89%,.2)}.dark\:bg-lime-100\/25:is(.dark *){background-color:hsla(80,89%,89%,.25)}.dark\:bg-lime-100\/30:is(.dark *){background-color:hsla(80,89%,89%,.3)}.dark\:bg-lime-100\/35:is(.dark *){background-color:hsla(80,89%,89%,.35)}.dark\:bg-lime-100\/40:is(.dark *){background-color:hsla(80,89%,89%,.4)}.dark\:bg-lime-100\/45:is(.dark *){background-color:hsla(80,89%,89%,.45)}.dark\:bg-lime-100\/5:is(.dark *){background-color:hsla(80,89%,89%,.05)}.dark\:bg-lime-100\/50:is(.dark *){background-color:hsla(80,89%,89%,.5)}.dark\:bg-lime-100\/55:is(.dark *){background-color:hsla(80,89%,89%,.55)}.dark\:bg-lime-100\/60:is(.dark *){background-color:hsla(80,89%,89%,.6)}.dark\:bg-lime-100\/65:is(.dark *){background-color:hsla(80,89%,89%,.65)}.dark\:bg-lime-100\/70:is(.dark *){background-color:hsla(80,89%,89%,.7)}.dark\:bg-lime-100\/75:is(.dark *){background-color:hsla(80,89%,89%,.75)}.dark\:bg-lime-100\/80:is(.dark *){background-color:hsla(80,89%,89%,.8)}.dark\:bg-lime-100\/85:is(.dark *){background-color:hsla(80,89%,89%,.85)}.dark\:bg-lime-100\/90:is(.dark *){background-color:hsla(80,89%,89%,.9)}.dark\:bg-lime-100\/95:is(.dark *){background-color:hsla(80,89%,89%,.95)}.dark\:bg-lime-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(217 249 157/var(--tw-bg-opacity,1))}.dark\:bg-lime-200\/0:is(.dark *){background-color:hsla(81,88%,80%,0)}.dark\:bg-lime-200\/10:is(.dark *){background-color:hsla(81,88%,80%,.1)}.dark\:bg-lime-200\/100:is(.dark *){background-color:#d9f99d}.dark\:bg-lime-200\/15:is(.dark *){background-color:hsla(81,88%,80%,.15)}.dark\:bg-lime-200\/20:is(.dark *){background-color:hsla(81,88%,80%,.2)}.dark\:bg-lime-200\/25:is(.dark *){background-color:hsla(81,88%,80%,.25)}.dark\:bg-lime-200\/30:is(.dark *){background-color:hsla(81,88%,80%,.3)}.dark\:bg-lime-200\/35:is(.dark *){background-color:hsla(81,88%,80%,.35)}.dark\:bg-lime-200\/40:is(.dark *){background-color:hsla(81,88%,80%,.4)}.dark\:bg-lime-200\/45:is(.dark *){background-color:hsla(81,88%,80%,.45)}.dark\:bg-lime-200\/5:is(.dark *){background-color:hsla(81,88%,80%,.05)}.dark\:bg-lime-200\/50:is(.dark *){background-color:hsla(81,88%,80%,.5)}.dark\:bg-lime-200\/55:is(.dark *){background-color:hsla(81,88%,80%,.55)}.dark\:bg-lime-200\/60:is(.dark *){background-color:hsla(81,88%,80%,.6)}.dark\:bg-lime-200\/65:is(.dark *){background-color:hsla(81,88%,80%,.65)}.dark\:bg-lime-200\/70:is(.dark *){background-color:hsla(81,88%,80%,.7)}.dark\:bg-lime-200\/75:is(.dark *){background-color:hsla(81,88%,80%,.75)}.dark\:bg-lime-200\/80:is(.dark *){background-color:hsla(81,88%,80%,.8)}.dark\:bg-lime-200\/85:is(.dark *){background-color:hsla(81,88%,80%,.85)}.dark\:bg-lime-200\/90:is(.dark *){background-color:hsla(81,88%,80%,.9)}.dark\:bg-lime-200\/95:is(.dark *){background-color:hsla(81,88%,80%,.95)}.dark\:bg-lime-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(163 230 53/var(--tw-bg-opacity,1))}.dark\:bg-lime-400\/0:is(.dark *){background-color:rgba(163,230,53,0)}.dark\:bg-lime-400\/10:is(.dark *){background-color:rgba(163,230,53,.1)}.dark\:bg-lime-400\/100:is(.dark *){background-color:#a3e635}.dark\:bg-lime-400\/15:is(.dark *){background-color:rgba(163,230,53,.15)}.dark\:bg-lime-400\/20:is(.dark *){background-color:rgba(163,230,53,.2)}.dark\:bg-lime-400\/25:is(.dark *){background-color:rgba(163,230,53,.25)}.dark\:bg-lime-400\/30:is(.dark *){background-color:rgba(163,230,53,.3)}.dark\:bg-lime-400\/35:is(.dark *){background-color:rgba(163,230,53,.35)}.dark\:bg-lime-400\/40:is(.dark *){background-color:rgba(163,230,53,.4)}.dark\:bg-lime-400\/45:is(.dark *){background-color:rgba(163,230,53,.45)}.dark\:bg-lime-400\/5:is(.dark *){background-color:rgba(163,230,53,.05)}.dark\:bg-lime-400\/50:is(.dark *){background-color:rgba(163,230,53,.5)}.dark\:bg-lime-400\/55:is(.dark *){background-color:rgba(163,230,53,.55)}.dark\:bg-lime-400\/60:is(.dark *){background-color:rgba(163,230,53,.6)}.dark\:bg-lime-400\/65:is(.dark *){background-color:rgba(163,230,53,.65)}.dark\:bg-lime-400\/70:is(.dark *){background-color:rgba(163,230,53,.7)}.dark\:bg-lime-400\/75:is(.dark *){background-color:rgba(163,230,53,.75)}.dark\:bg-lime-400\/80:is(.dark *){background-color:rgba(163,230,53,.8)}.dark\:bg-lime-400\/85:is(.dark *){background-color:rgba(163,230,53,.85)}.dark\:bg-lime-400\/90:is(.dark *){background-color:rgba(163,230,53,.9)}.dark\:bg-lime-400\/95:is(.dark *){background-color:rgba(163,230,53,.95)}.dark\:bg-lime-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(247 254 231/var(--tw-bg-opacity,1))}.dark\:bg-lime-50\/0:is(.dark *){background-color:hsla(78,92%,95%,0)}.dark\:bg-lime-50\/10:is(.dark *){background-color:hsla(78,92%,95%,.1)}.dark\:bg-lime-50\/100:is(.dark *){background-color:#f7fee7}.dark\:bg-lime-50\/15:is(.dark *){background-color:hsla(78,92%,95%,.15)}.dark\:bg-lime-50\/20:is(.dark *){background-color:hsla(78,92%,95%,.2)}.dark\:bg-lime-50\/25:is(.dark *){background-color:hsla(78,92%,95%,.25)}.dark\:bg-lime-50\/30:is(.dark *){background-color:hsla(78,92%,95%,.3)}.dark\:bg-lime-50\/35:is(.dark *){background-color:hsla(78,92%,95%,.35)}.dark\:bg-lime-50\/40:is(.dark *){background-color:hsla(78,92%,95%,.4)}.dark\:bg-lime-50\/45:is(.dark *){background-color:hsla(78,92%,95%,.45)}.dark\:bg-lime-50\/5:is(.dark *){background-color:hsla(78,92%,95%,.05)}.dark\:bg-lime-50\/50:is(.dark *){background-color:hsla(78,92%,95%,.5)}.dark\:bg-lime-50\/55:is(.dark *){background-color:hsla(78,92%,95%,.55)}.dark\:bg-lime-50\/60:is(.dark *){background-color:hsla(78,92%,95%,.6)}.dark\:bg-lime-50\/65:is(.dark *){background-color:hsla(78,92%,95%,.65)}.dark\:bg-lime-50\/70:is(.dark *){background-color:hsla(78,92%,95%,.7)}.dark\:bg-lime-50\/75:is(.dark *){background-color:hsla(78,92%,95%,.75)}.dark\:bg-lime-50\/80:is(.dark *){background-color:hsla(78,92%,95%,.8)}.dark\:bg-lime-50\/85:is(.dark *){background-color:hsla(78,92%,95%,.85)}.dark\:bg-lime-50\/90:is(.dark *){background-color:hsla(78,92%,95%,.9)}.dark\:bg-lime-50\/95:is(.dark *){background-color:hsla(78,92%,95%,.95)}.dark\:bg-lime-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(132 204 22/var(--tw-bg-opacity,1))}.dark\:bg-lime-500\/0:is(.dark *){background-color:rgba(132,204,22,0)}.dark\:bg-lime-500\/10:is(.dark *){background-color:rgba(132,204,22,.1)}.dark\:bg-lime-500\/100:is(.dark *){background-color:#84cc16}.dark\:bg-lime-500\/15:is(.dark *){background-color:rgba(132,204,22,.15)}.dark\:bg-lime-500\/20:is(.dark *){background-color:rgba(132,204,22,.2)}.dark\:bg-lime-500\/25:is(.dark *){background-color:rgba(132,204,22,.25)}.dark\:bg-lime-500\/30:is(.dark *){background-color:rgba(132,204,22,.3)}.dark\:bg-lime-500\/35:is(.dark *){background-color:rgba(132,204,22,.35)}.dark\:bg-lime-500\/40:is(.dark *){background-color:rgba(132,204,22,.4)}.dark\:bg-lime-500\/45:is(.dark *){background-color:rgba(132,204,22,.45)}.dark\:bg-lime-500\/5:is(.dark *){background-color:rgba(132,204,22,.05)}.dark\:bg-lime-500\/50:is(.dark *){background-color:rgba(132,204,22,.5)}.dark\:bg-lime-500\/55:is(.dark *){background-color:rgba(132,204,22,.55)}.dark\:bg-lime-500\/60:is(.dark *){background-color:rgba(132,204,22,.6)}.dark\:bg-lime-500\/65:is(.dark *){background-color:rgba(132,204,22,.65)}.dark\:bg-lime-500\/70:is(.dark *){background-color:rgba(132,204,22,.7)}.dark\:bg-lime-500\/75:is(.dark *){background-color:rgba(132,204,22,.75)}.dark\:bg-lime-500\/80:is(.dark *){background-color:rgba(132,204,22,.8)}.dark\:bg-lime-500\/85:is(.dark *){background-color:rgba(132,204,22,.85)}.dark\:bg-lime-500\/90:is(.dark *){background-color:rgba(132,204,22,.9)}.dark\:bg-lime-500\/95:is(.dark *){background-color:rgba(132,204,22,.95)}.dark\:bg-lime-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(26 46 5/var(--tw-bg-opacity,1))}.dark\:bg-lime-950\/0:is(.dark *){background-color:rgba(26,46,5,0)}.dark\:bg-lime-950\/10:is(.dark *){background-color:rgba(26,46,5,.1)}.dark\:bg-lime-950\/100:is(.dark *){background-color:#1a2e05}.dark\:bg-lime-950\/15:is(.dark *){background-color:rgba(26,46,5,.15)}.dark\:bg-lime-950\/20:is(.dark *){background-color:rgba(26,46,5,.2)}.dark\:bg-lime-950\/25:is(.dark *){background-color:rgba(26,46,5,.25)}.dark\:bg-lime-950\/30:is(.dark *){background-color:rgba(26,46,5,.3)}.dark\:bg-lime-950\/35:is(.dark *){background-color:rgba(26,46,5,.35)}.dark\:bg-lime-950\/40:is(.dark *){background-color:rgba(26,46,5,.4)}.dark\:bg-lime-950\/45:is(.dark *){background-color:rgba(26,46,5,.45)}.dark\:bg-lime-950\/5:is(.dark *){background-color:rgba(26,46,5,.05)}.dark\:bg-lime-950\/50:is(.dark *){background-color:rgba(26,46,5,.5)}.dark\:bg-lime-950\/55:is(.dark *){background-color:rgba(26,46,5,.55)}.dark\:bg-lime-950\/60:is(.dark *){background-color:rgba(26,46,5,.6)}.dark\:bg-lime-950\/65:is(.dark *){background-color:rgba(26,46,5,.65)}.dark\:bg-lime-950\/70:is(.dark *){background-color:rgba(26,46,5,.7)}.dark\:bg-lime-950\/75:is(.dark *){background-color:rgba(26,46,5,.75)}.dark\:bg-lime-950\/80:is(.dark *){background-color:rgba(26,46,5,.8)}.dark\:bg-lime-950\/85:is(.dark *){background-color:rgba(26,46,5,.85)}.dark\:bg-lime-950\/90:is(.dark *){background-color:rgba(26,46,5,.9)}.dark\:bg-lime-950\/95:is(.dark *){background-color:rgba(26,46,5,.95)}.dark\:bg-neutral-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(245 245 245/var(--tw-bg-opacity,1))}.dark\:bg-neutral-100\/0:is(.dark *){background-color:hsla(0,0%,96%,0)}.dark\:bg-neutral-100\/10:is(.dark *){background-color:hsla(0,0%,96%,.1)}.dark\:bg-neutral-100\/100:is(.dark *){background-color:#f5f5f5}.dark\:bg-neutral-100\/15:is(.dark *){background-color:hsla(0,0%,96%,.15)}.dark\:bg-neutral-100\/20:is(.dark *){background-color:hsla(0,0%,96%,.2)}.dark\:bg-neutral-100\/25:is(.dark *){background-color:hsla(0,0%,96%,.25)}.dark\:bg-neutral-100\/30:is(.dark *){background-color:hsla(0,0%,96%,.3)}.dark\:bg-neutral-100\/35:is(.dark *){background-color:hsla(0,0%,96%,.35)}.dark\:bg-neutral-100\/40:is(.dark *){background-color:hsla(0,0%,96%,.4)}.dark\:bg-neutral-100\/45:is(.dark *){background-color:hsla(0,0%,96%,.45)}.dark\:bg-neutral-100\/5:is(.dark *){background-color:hsla(0,0%,96%,.05)}.dark\:bg-neutral-100\/50:is(.dark *){background-color:hsla(0,0%,96%,.5)}.dark\:bg-neutral-100\/55:is(.dark *){background-color:hsla(0,0%,96%,.55)}.dark\:bg-neutral-100\/60:is(.dark *){background-color:hsla(0,0%,96%,.6)}.dark\:bg-neutral-100\/65:is(.dark *){background-color:hsla(0,0%,96%,.65)}.dark\:bg-neutral-100\/70:is(.dark *){background-color:hsla(0,0%,96%,.7)}.dark\:bg-neutral-100\/75:is(.dark *){background-color:hsla(0,0%,96%,.75)}.dark\:bg-neutral-100\/80:is(.dark *){background-color:hsla(0,0%,96%,.8)}.dark\:bg-neutral-100\/85:is(.dark *){background-color:hsla(0,0%,96%,.85)}.dark\:bg-neutral-100\/90:is(.dark *){background-color:hsla(0,0%,96%,.9)}.dark\:bg-neutral-100\/95:is(.dark *){background-color:hsla(0,0%,96%,.95)}.dark\:bg-neutral-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(229 229 229/var(--tw-bg-opacity,1))}.dark\:bg-neutral-200\/0:is(.dark *){background-color:hsla(0,0%,90%,0)}.dark\:bg-neutral-200\/10:is(.dark *){background-color:hsla(0,0%,90%,.1)}.dark\:bg-neutral-200\/100:is(.dark *){background-color:#e5e5e5}.dark\:bg-neutral-200\/15:is(.dark *){background-color:hsla(0,0%,90%,.15)}.dark\:bg-neutral-200\/20:is(.dark *){background-color:hsla(0,0%,90%,.2)}.dark\:bg-neutral-200\/25:is(.dark *){background-color:hsla(0,0%,90%,.25)}.dark\:bg-neutral-200\/30:is(.dark *){background-color:hsla(0,0%,90%,.3)}.dark\:bg-neutral-200\/35:is(.dark *){background-color:hsla(0,0%,90%,.35)}.dark\:bg-neutral-200\/40:is(.dark *){background-color:hsla(0,0%,90%,.4)}.dark\:bg-neutral-200\/45:is(.dark *){background-color:hsla(0,0%,90%,.45)}.dark\:bg-neutral-200\/5:is(.dark *){background-color:hsla(0,0%,90%,.05)}.dark\:bg-neutral-200\/50:is(.dark *){background-color:hsla(0,0%,90%,.5)}.dark\:bg-neutral-200\/55:is(.dark *){background-color:hsla(0,0%,90%,.55)}.dark\:bg-neutral-200\/60:is(.dark *){background-color:hsla(0,0%,90%,.6)}.dark\:bg-neutral-200\/65:is(.dark *){background-color:hsla(0,0%,90%,.65)}.dark\:bg-neutral-200\/70:is(.dark *){background-color:hsla(0,0%,90%,.7)}.dark\:bg-neutral-200\/75:is(.dark *){background-color:hsla(0,0%,90%,.75)}.dark\:bg-neutral-200\/80:is(.dark *){background-color:hsla(0,0%,90%,.8)}.dark\:bg-neutral-200\/85:is(.dark *){background-color:hsla(0,0%,90%,.85)}.dark\:bg-neutral-200\/90:is(.dark *){background-color:hsla(0,0%,90%,.9)}.dark\:bg-neutral-200\/95:is(.dark *){background-color:hsla(0,0%,90%,.95)}.dark\:bg-neutral-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(163 163 163/var(--tw-bg-opacity,1))}.dark\:bg-neutral-400\/0:is(.dark *){background-color:hsla(0,0%,64%,0)}.dark\:bg-neutral-400\/10:is(.dark *){background-color:hsla(0,0%,64%,.1)}.dark\:bg-neutral-400\/100:is(.dark *){background-color:#a3a3a3}.dark\:bg-neutral-400\/15:is(.dark *){background-color:hsla(0,0%,64%,.15)}.dark\:bg-neutral-400\/20:is(.dark *){background-color:hsla(0,0%,64%,.2)}.dark\:bg-neutral-400\/25:is(.dark *){background-color:hsla(0,0%,64%,.25)}.dark\:bg-neutral-400\/30:is(.dark *){background-color:hsla(0,0%,64%,.3)}.dark\:bg-neutral-400\/35:is(.dark *){background-color:hsla(0,0%,64%,.35)}.dark\:bg-neutral-400\/40:is(.dark *){background-color:hsla(0,0%,64%,.4)}.dark\:bg-neutral-400\/45:is(.dark *){background-color:hsla(0,0%,64%,.45)}.dark\:bg-neutral-400\/5:is(.dark *){background-color:hsla(0,0%,64%,.05)}.dark\:bg-neutral-400\/50:is(.dark *){background-color:hsla(0,0%,64%,.5)}.dark\:bg-neutral-400\/55:is(.dark *){background-color:hsla(0,0%,64%,.55)}.dark\:bg-neutral-400\/60:is(.dark *){background-color:hsla(0,0%,64%,.6)}.dark\:bg-neutral-400\/65:is(.dark *){background-color:hsla(0,0%,64%,.65)}.dark\:bg-neutral-400\/70:is(.dark *){background-color:hsla(0,0%,64%,.7)}.dark\:bg-neutral-400\/75:is(.dark *){background-color:hsla(0,0%,64%,.75)}.dark\:bg-neutral-400\/80:is(.dark *){background-color:hsla(0,0%,64%,.8)}.dark\:bg-neutral-400\/85:is(.dark *){background-color:hsla(0,0%,64%,.85)}.dark\:bg-neutral-400\/90:is(.dark *){background-color:hsla(0,0%,64%,.9)}.dark\:bg-neutral-400\/95:is(.dark *){background-color:hsla(0,0%,64%,.95)}.dark\:bg-neutral-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(250 250 250/var(--tw-bg-opacity,1))}.dark\:bg-neutral-50\/0:is(.dark *){background-color:hsla(0,0%,98%,0)}.dark\:bg-neutral-50\/10:is(.dark *){background-color:hsla(0,0%,98%,.1)}.dark\:bg-neutral-50\/100:is(.dark *){background-color:#fafafa}.dark\:bg-neutral-50\/15:is(.dark *){background-color:hsla(0,0%,98%,.15)}.dark\:bg-neutral-50\/20:is(.dark *){background-color:hsla(0,0%,98%,.2)}.dark\:bg-neutral-50\/25:is(.dark *){background-color:hsla(0,0%,98%,.25)}.dark\:bg-neutral-50\/30:is(.dark *){background-color:hsla(0,0%,98%,.3)}.dark\:bg-neutral-50\/35:is(.dark *){background-color:hsla(0,0%,98%,.35)}.dark\:bg-neutral-50\/40:is(.dark *){background-color:hsla(0,0%,98%,.4)}.dark\:bg-neutral-50\/45:is(.dark *){background-color:hsla(0,0%,98%,.45)}.dark\:bg-neutral-50\/5:is(.dark *){background-color:hsla(0,0%,98%,.05)}.dark\:bg-neutral-50\/50:is(.dark *){background-color:hsla(0,0%,98%,.5)}.dark\:bg-neutral-50\/55:is(.dark *){background-color:hsla(0,0%,98%,.55)}.dark\:bg-neutral-50\/60:is(.dark *){background-color:hsla(0,0%,98%,.6)}.dark\:bg-neutral-50\/65:is(.dark *){background-color:hsla(0,0%,98%,.65)}.dark\:bg-neutral-50\/70:is(.dark *){background-color:hsla(0,0%,98%,.7)}.dark\:bg-neutral-50\/75:is(.dark *){background-color:hsla(0,0%,98%,.75)}.dark\:bg-neutral-50\/80:is(.dark *){background-color:hsla(0,0%,98%,.8)}.dark\:bg-neutral-50\/85:is(.dark *){background-color:hsla(0,0%,98%,.85)}.dark\:bg-neutral-50\/90:is(.dark *){background-color:hsla(0,0%,98%,.9)}.dark\:bg-neutral-50\/95:is(.dark *){background-color:hsla(0,0%,98%,.95)}.dark\:bg-neutral-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(115 115 115/var(--tw-bg-opacity,1))}.dark\:bg-neutral-500\/0:is(.dark *){background-color:hsla(0,0%,45%,0)}.dark\:bg-neutral-500\/10:is(.dark *){background-color:hsla(0,0%,45%,.1)}.dark\:bg-neutral-500\/100:is(.dark *){background-color:#737373}.dark\:bg-neutral-500\/15:is(.dark *){background-color:hsla(0,0%,45%,.15)}.dark\:bg-neutral-500\/20:is(.dark *){background-color:hsla(0,0%,45%,.2)}.dark\:bg-neutral-500\/25:is(.dark *){background-color:hsla(0,0%,45%,.25)}.dark\:bg-neutral-500\/30:is(.dark *){background-color:hsla(0,0%,45%,.3)}.dark\:bg-neutral-500\/35:is(.dark *){background-color:hsla(0,0%,45%,.35)}.dark\:bg-neutral-500\/40:is(.dark *){background-color:hsla(0,0%,45%,.4)}.dark\:bg-neutral-500\/45:is(.dark *){background-color:hsla(0,0%,45%,.45)}.dark\:bg-neutral-500\/5:is(.dark *){background-color:hsla(0,0%,45%,.05)}.dark\:bg-neutral-500\/50:is(.dark *){background-color:hsla(0,0%,45%,.5)}.dark\:bg-neutral-500\/55:is(.dark *){background-color:hsla(0,0%,45%,.55)}.dark\:bg-neutral-500\/60:is(.dark *){background-color:hsla(0,0%,45%,.6)}.dark\:bg-neutral-500\/65:is(.dark *){background-color:hsla(0,0%,45%,.65)}.dark\:bg-neutral-500\/70:is(.dark *){background-color:hsla(0,0%,45%,.7)}.dark\:bg-neutral-500\/75:is(.dark *){background-color:hsla(0,0%,45%,.75)}.dark\:bg-neutral-500\/80:is(.dark *){background-color:hsla(0,0%,45%,.8)}.dark\:bg-neutral-500\/85:is(.dark *){background-color:hsla(0,0%,45%,.85)}.dark\:bg-neutral-500\/90:is(.dark *){background-color:hsla(0,0%,45%,.9)}.dark\:bg-neutral-500\/95:is(.dark *){background-color:hsla(0,0%,45%,.95)}.dark\:bg-neutral-600:is(.dark *){--tw-bg-opacity:1;background-color:rgb(82 82 82/var(--tw-bg-opacity,1))}.dark\:bg-neutral-700:is(.dark *){--tw-bg-opacity:1;background-color:rgb(64 64 64/var(--tw-bg-opacity,1))}.dark\:bg-neutral-800:is(.dark *){--tw-bg-opacity:1;background-color:rgb(38 38 38/var(--tw-bg-opacity,1))}.dark\:bg-neutral-900:is(.dark *){--tw-bg-opacity:1;background-color:rgb(23 23 23/var(--tw-bg-opacity,1))}.dark\:bg-neutral-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(10 10 10/var(--tw-bg-opacity,1))}.dark\:bg-neutral-950\/0:is(.dark *){background-color:hsla(0,0%,4%,0)}.dark\:bg-neutral-950\/10:is(.dark *){background-color:hsla(0,0%,4%,.1)}.dark\:bg-neutral-950\/100:is(.dark *){background-color:#0a0a0a}.dark\:bg-neutral-950\/15:is(.dark *){background-color:hsla(0,0%,4%,.15)}.dark\:bg-neutral-950\/20:is(.dark *){background-color:hsla(0,0%,4%,.2)}.dark\:bg-neutral-950\/25:is(.dark *){background-color:hsla(0,0%,4%,.25)}.dark\:bg-neutral-950\/30:is(.dark *){background-color:hsla(0,0%,4%,.3)}.dark\:bg-neutral-950\/35:is(.dark *){background-color:hsla(0,0%,4%,.35)}.dark\:bg-neutral-950\/40:is(.dark *){background-color:hsla(0,0%,4%,.4)}.dark\:bg-neutral-950\/45:is(.dark *){background-color:hsla(0,0%,4%,.45)}.dark\:bg-neutral-950\/5:is(.dark *){background-color:hsla(0,0%,4%,.05)}.dark\:bg-neutral-950\/50:is(.dark *){background-color:hsla(0,0%,4%,.5)}.dark\:bg-neutral-950\/55:is(.dark *){background-color:hsla(0,0%,4%,.55)}.dark\:bg-neutral-950\/60:is(.dark *){background-color:hsla(0,0%,4%,.6)}.dark\:bg-neutral-950\/65:is(.dark *){background-color:hsla(0,0%,4%,.65)}.dark\:bg-neutral-950\/70:is(.dark *){background-color:hsla(0,0%,4%,.7)}.dark\:bg-neutral-950\/75:is(.dark *){background-color:hsla(0,0%,4%,.75)}.dark\:bg-neutral-950\/80:is(.dark *){background-color:hsla(0,0%,4%,.8)}.dark\:bg-neutral-950\/85:is(.dark *){background-color:hsla(0,0%,4%,.85)}.dark\:bg-neutral-950\/90:is(.dark *){background-color:hsla(0,0%,4%,.9)}.dark\:bg-neutral-950\/95:is(.dark *){background-color:hsla(0,0%,4%,.95)}.dark\:bg-orange-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(255 237 213/var(--tw-bg-opacity,1))}.dark\:bg-orange-100\/0:is(.dark *){background-color:rgba(255,237,213,0)}.dark\:bg-orange-100\/10:is(.dark *){background-color:rgba(255,237,213,.1)}.dark\:bg-orange-100\/100:is(.dark *){background-color:#ffedd5}.dark\:bg-orange-100\/15:is(.dark *){background-color:rgba(255,237,213,.15)}.dark\:bg-orange-100\/20:is(.dark *){background-color:rgba(255,237,213,.2)}.dark\:bg-orange-100\/25:is(.dark *){background-color:rgba(255,237,213,.25)}.dark\:bg-orange-100\/30:is(.dark *){background-color:rgba(255,237,213,.3)}.dark\:bg-orange-100\/35:is(.dark *){background-color:rgba(255,237,213,.35)}.dark\:bg-orange-100\/40:is(.dark *){background-color:rgba(255,237,213,.4)}.dark\:bg-orange-100\/45:is(.dark *){background-color:rgba(255,237,213,.45)}.dark\:bg-orange-100\/5:is(.dark *){background-color:rgba(255,237,213,.05)}.dark\:bg-orange-100\/50:is(.dark *){background-color:rgba(255,237,213,.5)}.dark\:bg-orange-100\/55:is(.dark *){background-color:rgba(255,237,213,.55)}.dark\:bg-orange-100\/60:is(.dark *){background-color:rgba(255,237,213,.6)}.dark\:bg-orange-100\/65:is(.dark *){background-color:rgba(255,237,213,.65)}.dark\:bg-orange-100\/70:is(.dark *){background-color:rgba(255,237,213,.7)}.dark\:bg-orange-100\/75:is(.dark *){background-color:rgba(255,237,213,.75)}.dark\:bg-orange-100\/80:is(.dark *){background-color:rgba(255,237,213,.8)}.dark\:bg-orange-100\/85:is(.dark *){background-color:rgba(255,237,213,.85)}.dark\:bg-orange-100\/90:is(.dark *){background-color:rgba(255,237,213,.9)}.dark\:bg-orange-100\/95:is(.dark *){background-color:rgba(255,237,213,.95)}.dark\:bg-orange-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(254 215 170/var(--tw-bg-opacity,1))}.dark\:bg-orange-200\/0:is(.dark *){background-color:hsla(32,98%,83%,0)}.dark\:bg-orange-200\/10:is(.dark *){background-color:hsla(32,98%,83%,.1)}.dark\:bg-orange-200\/100:is(.dark *){background-color:#fed7aa}.dark\:bg-orange-200\/15:is(.dark *){background-color:hsla(32,98%,83%,.15)}.dark\:bg-orange-200\/20:is(.dark *){background-color:hsla(32,98%,83%,.2)}.dark\:bg-orange-200\/25:is(.dark *){background-color:hsla(32,98%,83%,.25)}.dark\:bg-orange-200\/30:is(.dark *){background-color:hsla(32,98%,83%,.3)}.dark\:bg-orange-200\/35:is(.dark *){background-color:hsla(32,98%,83%,.35)}.dark\:bg-orange-200\/40:is(.dark *){background-color:hsla(32,98%,83%,.4)}.dark\:bg-orange-200\/45:is(.dark *){background-color:hsla(32,98%,83%,.45)}.dark\:bg-orange-200\/5:is(.dark *){background-color:hsla(32,98%,83%,.05)}.dark\:bg-orange-200\/50:is(.dark *){background-color:hsla(32,98%,83%,.5)}.dark\:bg-orange-200\/55:is(.dark *){background-color:hsla(32,98%,83%,.55)}.dark\:bg-orange-200\/60:is(.dark *){background-color:hsla(32,98%,83%,.6)}.dark\:bg-orange-200\/65:is(.dark *){background-color:hsla(32,98%,83%,.65)}.dark\:bg-orange-200\/70:is(.dark *){background-color:hsla(32,98%,83%,.7)}.dark\:bg-orange-200\/75:is(.dark *){background-color:hsla(32,98%,83%,.75)}.dark\:bg-orange-200\/80:is(.dark *){background-color:hsla(32,98%,83%,.8)}.dark\:bg-orange-200\/85:is(.dark *){background-color:hsla(32,98%,83%,.85)}.dark\:bg-orange-200\/90:is(.dark *){background-color:hsla(32,98%,83%,.9)}.dark\:bg-orange-200\/95:is(.dark *){background-color:hsla(32,98%,83%,.95)}.dark\:bg-orange-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(251 146 60/var(--tw-bg-opacity,1))}.dark\:bg-orange-400\/0:is(.dark *){background-color:rgba(251,146,60,0)}.dark\:bg-orange-400\/10:is(.dark *){background-color:rgba(251,146,60,.1)}.dark\:bg-orange-400\/100:is(.dark *){background-color:#fb923c}.dark\:bg-orange-400\/15:is(.dark *){background-color:rgba(251,146,60,.15)}.dark\:bg-orange-400\/20:is(.dark *){background-color:rgba(251,146,60,.2)}.dark\:bg-orange-400\/25:is(.dark *){background-color:rgba(251,146,60,.25)}.dark\:bg-orange-400\/30:is(.dark *){background-color:rgba(251,146,60,.3)}.dark\:bg-orange-400\/35:is(.dark *){background-color:rgba(251,146,60,.35)}.dark\:bg-orange-400\/40:is(.dark *){background-color:rgba(251,146,60,.4)}.dark\:bg-orange-400\/45:is(.dark *){background-color:rgba(251,146,60,.45)}.dark\:bg-orange-400\/5:is(.dark *){background-color:rgba(251,146,60,.05)}.dark\:bg-orange-400\/50:is(.dark *){background-color:rgba(251,146,60,.5)}.dark\:bg-orange-400\/55:is(.dark *){background-color:rgba(251,146,60,.55)}.dark\:bg-orange-400\/60:is(.dark *){background-color:rgba(251,146,60,.6)}.dark\:bg-orange-400\/65:is(.dark *){background-color:rgba(251,146,60,.65)}.dark\:bg-orange-400\/70:is(.dark *){background-color:rgba(251,146,60,.7)}.dark\:bg-orange-400\/75:is(.dark *){background-color:rgba(251,146,60,.75)}.dark\:bg-orange-400\/80:is(.dark *){background-color:rgba(251,146,60,.8)}.dark\:bg-orange-400\/85:is(.dark *){background-color:rgba(251,146,60,.85)}.dark\:bg-orange-400\/90:is(.dark *){background-color:rgba(251,146,60,.9)}.dark\:bg-orange-400\/95:is(.dark *){background-color:rgba(251,146,60,.95)}.dark\:bg-orange-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(255 247 237/var(--tw-bg-opacity,1))}.dark\:bg-orange-50\/0:is(.dark *){background-color:rgba(255,247,237,0)}.dark\:bg-orange-50\/10:is(.dark *){background-color:rgba(255,247,237,.1)}.dark\:bg-orange-50\/100:is(.dark *){background-color:#fff7ed}.dark\:bg-orange-50\/15:is(.dark *){background-color:rgba(255,247,237,.15)}.dark\:bg-orange-50\/20:is(.dark *){background-color:rgba(255,247,237,.2)}.dark\:bg-orange-50\/25:is(.dark *){background-color:rgba(255,247,237,.25)}.dark\:bg-orange-50\/30:is(.dark *){background-color:rgba(255,247,237,.3)}.dark\:bg-orange-50\/35:is(.dark *){background-color:rgba(255,247,237,.35)}.dark\:bg-orange-50\/40:is(.dark *){background-color:rgba(255,247,237,.4)}.dark\:bg-orange-50\/45:is(.dark *){background-color:rgba(255,247,237,.45)}.dark\:bg-orange-50\/5:is(.dark *){background-color:rgba(255,247,237,.05)}.dark\:bg-orange-50\/50:is(.dark *){background-color:rgba(255,247,237,.5)}.dark\:bg-orange-50\/55:is(.dark *){background-color:rgba(255,247,237,.55)}.dark\:bg-orange-50\/60:is(.dark *){background-color:rgba(255,247,237,.6)}.dark\:bg-orange-50\/65:is(.dark *){background-color:rgba(255,247,237,.65)}.dark\:bg-orange-50\/70:is(.dark *){background-color:rgba(255,247,237,.7)}.dark\:bg-orange-50\/75:is(.dark *){background-color:rgba(255,247,237,.75)}.dark\:bg-orange-50\/80:is(.dark *){background-color:rgba(255,247,237,.8)}.dark\:bg-orange-50\/85:is(.dark *){background-color:rgba(255,247,237,.85)}.dark\:bg-orange-50\/90:is(.dark *){background-color:rgba(255,247,237,.9)}.dark\:bg-orange-50\/95:is(.dark *){background-color:rgba(255,247,237,.95)}.dark\:bg-orange-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(249 115 22/var(--tw-bg-opacity,1))}.dark\:bg-orange-500\/0:is(.dark *){background-color:rgba(249,115,22,0)}.dark\:bg-orange-500\/10:is(.dark *){background-color:rgba(249,115,22,.1)}.dark\:bg-orange-500\/100:is(.dark *){background-color:#f97316}.dark\:bg-orange-500\/15:is(.dark *){background-color:rgba(249,115,22,.15)}.dark\:bg-orange-500\/20:is(.dark *){background-color:rgba(249,115,22,.2)}.dark\:bg-orange-500\/25:is(.dark *){background-color:rgba(249,115,22,.25)}.dark\:bg-orange-500\/30:is(.dark *){background-color:rgba(249,115,22,.3)}.dark\:bg-orange-500\/35:is(.dark *){background-color:rgba(249,115,22,.35)}.dark\:bg-orange-500\/40:is(.dark *){background-color:rgba(249,115,22,.4)}.dark\:bg-orange-500\/45:is(.dark *){background-color:rgba(249,115,22,.45)}.dark\:bg-orange-500\/5:is(.dark *){background-color:rgba(249,115,22,.05)}.dark\:bg-orange-500\/50:is(.dark *){background-color:rgba(249,115,22,.5)}.dark\:bg-orange-500\/55:is(.dark *){background-color:rgba(249,115,22,.55)}.dark\:bg-orange-500\/60:is(.dark *){background-color:rgba(249,115,22,.6)}.dark\:bg-orange-500\/65:is(.dark *){background-color:rgba(249,115,22,.65)}.dark\:bg-orange-500\/70:is(.dark *){background-color:rgba(249,115,22,.7)}.dark\:bg-orange-500\/75:is(.dark *){background-color:rgba(249,115,22,.75)}.dark\:bg-orange-500\/80:is(.dark *){background-color:rgba(249,115,22,.8)}.dark\:bg-orange-500\/85:is(.dark *){background-color:rgba(249,115,22,.85)}.dark\:bg-orange-500\/90:is(.dark *){background-color:rgba(249,115,22,.9)}.dark\:bg-orange-500\/95:is(.dark *){background-color:rgba(249,115,22,.95)}.dark\:bg-orange-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(67 20 7/var(--tw-bg-opacity,1))}.dark\:bg-orange-950\/0:is(.dark *){background-color:rgba(67,20,7,0)}.dark\:bg-orange-950\/10:is(.dark *){background-color:rgba(67,20,7,.1)}.dark\:bg-orange-950\/100:is(.dark *){background-color:#431407}.dark\:bg-orange-950\/15:is(.dark *){background-color:rgba(67,20,7,.15)}.dark\:bg-orange-950\/20:is(.dark *){background-color:rgba(67,20,7,.2)}.dark\:bg-orange-950\/25:is(.dark *){background-color:rgba(67,20,7,.25)}.dark\:bg-orange-950\/30:is(.dark *){background-color:rgba(67,20,7,.3)}.dark\:bg-orange-950\/35:is(.dark *){background-color:rgba(67,20,7,.35)}.dark\:bg-orange-950\/40:is(.dark *){background-color:rgba(67,20,7,.4)}.dark\:bg-orange-950\/45:is(.dark *){background-color:rgba(67,20,7,.45)}.dark\:bg-orange-950\/5:is(.dark *){background-color:rgba(67,20,7,.05)}.dark\:bg-orange-950\/50:is(.dark *){background-color:rgba(67,20,7,.5)}.dark\:bg-orange-950\/55:is(.dark *){background-color:rgba(67,20,7,.55)}.dark\:bg-orange-950\/60:is(.dark *){background-color:rgba(67,20,7,.6)}.dark\:bg-orange-950\/65:is(.dark *){background-color:rgba(67,20,7,.65)}.dark\:bg-orange-950\/70:is(.dark *){background-color:rgba(67,20,7,.7)}.dark\:bg-orange-950\/75:is(.dark *){background-color:rgba(67,20,7,.75)}.dark\:bg-orange-950\/80:is(.dark *){background-color:rgba(67,20,7,.8)}.dark\:bg-orange-950\/85:is(.dark *){background-color:rgba(67,20,7,.85)}.dark\:bg-orange-950\/90:is(.dark *){background-color:rgba(67,20,7,.9)}.dark\:bg-orange-950\/95:is(.dark *){background-color:rgba(67,20,7,.95)}.dark\:bg-pink-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(252 231 243/var(--tw-bg-opacity,1))}.dark\:bg-pink-100\/0:is(.dark *){background-color:rgba(252,231,243,0)}.dark\:bg-pink-100\/10:is(.dark *){background-color:rgba(252,231,243,.1)}.dark\:bg-pink-100\/100:is(.dark *){background-color:#fce7f3}.dark\:bg-pink-100\/15:is(.dark *){background-color:rgba(252,231,243,.15)}.dark\:bg-pink-100\/20:is(.dark *){background-color:rgba(252,231,243,.2)}.dark\:bg-pink-100\/25:is(.dark *){background-color:rgba(252,231,243,.25)}.dark\:bg-pink-100\/30:is(.dark *){background-color:rgba(252,231,243,.3)}.dark\:bg-pink-100\/35:is(.dark *){background-color:rgba(252,231,243,.35)}.dark\:bg-pink-100\/40:is(.dark *){background-color:rgba(252,231,243,.4)}.dark\:bg-pink-100\/45:is(.dark *){background-color:rgba(252,231,243,.45)}.dark\:bg-pink-100\/5:is(.dark *){background-color:rgba(252,231,243,.05)}.dark\:bg-pink-100\/50:is(.dark *){background-color:rgba(252,231,243,.5)}.dark\:bg-pink-100\/55:is(.dark *){background-color:rgba(252,231,243,.55)}.dark\:bg-pink-100\/60:is(.dark *){background-color:rgba(252,231,243,.6)}.dark\:bg-pink-100\/65:is(.dark *){background-color:rgba(252,231,243,.65)}.dark\:bg-pink-100\/70:is(.dark *){background-color:rgba(252,231,243,.7)}.dark\:bg-pink-100\/75:is(.dark *){background-color:rgba(252,231,243,.75)}.dark\:bg-pink-100\/80:is(.dark *){background-color:rgba(252,231,243,.8)}.dark\:bg-pink-100\/85:is(.dark *){background-color:rgba(252,231,243,.85)}.dark\:bg-pink-100\/90:is(.dark *){background-color:rgba(252,231,243,.9)}.dark\:bg-pink-100\/95:is(.dark *){background-color:rgba(252,231,243,.95)}.dark\:bg-pink-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(251 207 232/var(--tw-bg-opacity,1))}.dark\:bg-pink-200\/0:is(.dark *){background-color:rgba(251,207,232,0)}.dark\:bg-pink-200\/10:is(.dark *){background-color:rgba(251,207,232,.1)}.dark\:bg-pink-200\/100:is(.dark *){background-color:#fbcfe8}.dark\:bg-pink-200\/15:is(.dark *){background-color:rgba(251,207,232,.15)}.dark\:bg-pink-200\/20:is(.dark *){background-color:rgba(251,207,232,.2)}.dark\:bg-pink-200\/25:is(.dark *){background-color:rgba(251,207,232,.25)}.dark\:bg-pink-200\/30:is(.dark *){background-color:rgba(251,207,232,.3)}.dark\:bg-pink-200\/35:is(.dark *){background-color:rgba(251,207,232,.35)}.dark\:bg-pink-200\/40:is(.dark *){background-color:rgba(251,207,232,.4)}.dark\:bg-pink-200\/45:is(.dark *){background-color:rgba(251,207,232,.45)}.dark\:bg-pink-200\/5:is(.dark *){background-color:rgba(251,207,232,.05)}.dark\:bg-pink-200\/50:is(.dark *){background-color:rgba(251,207,232,.5)}.dark\:bg-pink-200\/55:is(.dark *){background-color:rgba(251,207,232,.55)}.dark\:bg-pink-200\/60:is(.dark *){background-color:rgba(251,207,232,.6)}.dark\:bg-pink-200\/65:is(.dark *){background-color:rgba(251,207,232,.65)}.dark\:bg-pink-200\/70:is(.dark *){background-color:rgba(251,207,232,.7)}.dark\:bg-pink-200\/75:is(.dark *){background-color:rgba(251,207,232,.75)}.dark\:bg-pink-200\/80:is(.dark *){background-color:rgba(251,207,232,.8)}.dark\:bg-pink-200\/85:is(.dark *){background-color:rgba(251,207,232,.85)}.dark\:bg-pink-200\/90:is(.dark *){background-color:rgba(251,207,232,.9)}.dark\:bg-pink-200\/95:is(.dark *){background-color:rgba(251,207,232,.95)}.dark\:bg-pink-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(244 114 182/var(--tw-bg-opacity,1))}.dark\:bg-pink-400\/0:is(.dark *){background-color:rgba(244,114,182,0)}.dark\:bg-pink-400\/10:is(.dark *){background-color:rgba(244,114,182,.1)}.dark\:bg-pink-400\/100:is(.dark *){background-color:#f472b6}.dark\:bg-pink-400\/15:is(.dark *){background-color:rgba(244,114,182,.15)}.dark\:bg-pink-400\/20:is(.dark *){background-color:rgba(244,114,182,.2)}.dark\:bg-pink-400\/25:is(.dark *){background-color:rgba(244,114,182,.25)}.dark\:bg-pink-400\/30:is(.dark *){background-color:rgba(244,114,182,.3)}.dark\:bg-pink-400\/35:is(.dark *){background-color:rgba(244,114,182,.35)}.dark\:bg-pink-400\/40:is(.dark *){background-color:rgba(244,114,182,.4)}.dark\:bg-pink-400\/45:is(.dark *){background-color:rgba(244,114,182,.45)}.dark\:bg-pink-400\/5:is(.dark *){background-color:rgba(244,114,182,.05)}.dark\:bg-pink-400\/50:is(.dark *){background-color:rgba(244,114,182,.5)}.dark\:bg-pink-400\/55:is(.dark *){background-color:rgba(244,114,182,.55)}.dark\:bg-pink-400\/60:is(.dark *){background-color:rgba(244,114,182,.6)}.dark\:bg-pink-400\/65:is(.dark *){background-color:rgba(244,114,182,.65)}.dark\:bg-pink-400\/70:is(.dark *){background-color:rgba(244,114,182,.7)}.dark\:bg-pink-400\/75:is(.dark *){background-color:rgba(244,114,182,.75)}.dark\:bg-pink-400\/80:is(.dark *){background-color:rgba(244,114,182,.8)}.dark\:bg-pink-400\/85:is(.dark *){background-color:rgba(244,114,182,.85)}.dark\:bg-pink-400\/90:is(.dark *){background-color:rgba(244,114,182,.9)}.dark\:bg-pink-400\/95:is(.dark *){background-color:rgba(244,114,182,.95)}.dark\:bg-pink-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(253 242 248/var(--tw-bg-opacity,1))}.dark\:bg-pink-50\/0:is(.dark *){background-color:rgba(253,242,248,0)}.dark\:bg-pink-50\/10:is(.dark *){background-color:rgba(253,242,248,.1)}.dark\:bg-pink-50\/100:is(.dark *){background-color:#fdf2f8}.dark\:bg-pink-50\/15:is(.dark *){background-color:rgba(253,242,248,.15)}.dark\:bg-pink-50\/20:is(.dark *){background-color:rgba(253,242,248,.2)}.dark\:bg-pink-50\/25:is(.dark *){background-color:rgba(253,242,248,.25)}.dark\:bg-pink-50\/30:is(.dark *){background-color:rgba(253,242,248,.3)}.dark\:bg-pink-50\/35:is(.dark *){background-color:rgba(253,242,248,.35)}.dark\:bg-pink-50\/40:is(.dark *){background-color:rgba(253,242,248,.4)}.dark\:bg-pink-50\/45:is(.dark *){background-color:rgba(253,242,248,.45)}.dark\:bg-pink-50\/5:is(.dark *){background-color:rgba(253,242,248,.05)}.dark\:bg-pink-50\/50:is(.dark *){background-color:rgba(253,242,248,.5)}.dark\:bg-pink-50\/55:is(.dark *){background-color:rgba(253,242,248,.55)}.dark\:bg-pink-50\/60:is(.dark *){background-color:rgba(253,242,248,.6)}.dark\:bg-pink-50\/65:is(.dark *){background-color:rgba(253,242,248,.65)}.dark\:bg-pink-50\/70:is(.dark *){background-color:rgba(253,242,248,.7)}.dark\:bg-pink-50\/75:is(.dark *){background-color:rgba(253,242,248,.75)}.dark\:bg-pink-50\/80:is(.dark *){background-color:rgba(253,242,248,.8)}.dark\:bg-pink-50\/85:is(.dark *){background-color:rgba(253,242,248,.85)}.dark\:bg-pink-50\/90:is(.dark *){background-color:rgba(253,242,248,.9)}.dark\:bg-pink-50\/95:is(.dark *){background-color:rgba(253,242,248,.95)}.dark\:bg-pink-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(236 72 153/var(--tw-bg-opacity,1))}.dark\:bg-pink-500\/0:is(.dark *){background-color:rgba(236,72,153,0)}.dark\:bg-pink-500\/10:is(.dark *){background-color:rgba(236,72,153,.1)}.dark\:bg-pink-500\/100:is(.dark *){background-color:#ec4899}.dark\:bg-pink-500\/15:is(.dark *){background-color:rgba(236,72,153,.15)}.dark\:bg-pink-500\/20:is(.dark *){background-color:rgba(236,72,153,.2)}.dark\:bg-pink-500\/25:is(.dark *){background-color:rgba(236,72,153,.25)}.dark\:bg-pink-500\/30:is(.dark *){background-color:rgba(236,72,153,.3)}.dark\:bg-pink-500\/35:is(.dark *){background-color:rgba(236,72,153,.35)}.dark\:bg-pink-500\/40:is(.dark *){background-color:rgba(236,72,153,.4)}.dark\:bg-pink-500\/45:is(.dark *){background-color:rgba(236,72,153,.45)}.dark\:bg-pink-500\/5:is(.dark *){background-color:rgba(236,72,153,.05)}.dark\:bg-pink-500\/50:is(.dark *){background-color:rgba(236,72,153,.5)}.dark\:bg-pink-500\/55:is(.dark *){background-color:rgba(236,72,153,.55)}.dark\:bg-pink-500\/60:is(.dark *){background-color:rgba(236,72,153,.6)}.dark\:bg-pink-500\/65:is(.dark *){background-color:rgba(236,72,153,.65)}.dark\:bg-pink-500\/70:is(.dark *){background-color:rgba(236,72,153,.7)}.dark\:bg-pink-500\/75:is(.dark *){background-color:rgba(236,72,153,.75)}.dark\:bg-pink-500\/80:is(.dark *){background-color:rgba(236,72,153,.8)}.dark\:bg-pink-500\/85:is(.dark *){background-color:rgba(236,72,153,.85)}.dark\:bg-pink-500\/90:is(.dark *){background-color:rgba(236,72,153,.9)}.dark\:bg-pink-500\/95:is(.dark *){background-color:rgba(236,72,153,.95)}.dark\:bg-pink-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(80 7 36/var(--tw-bg-opacity,1))}.dark\:bg-pink-950\/0:is(.dark *){background-color:rgba(80,7,36,0)}.dark\:bg-pink-950\/10:is(.dark *){background-color:rgba(80,7,36,.1)}.dark\:bg-pink-950\/100:is(.dark *){background-color:#500724}.dark\:bg-pink-950\/15:is(.dark *){background-color:rgba(80,7,36,.15)}.dark\:bg-pink-950\/20:is(.dark *){background-color:rgba(80,7,36,.2)}.dark\:bg-pink-950\/25:is(.dark *){background-color:rgba(80,7,36,.25)}.dark\:bg-pink-950\/30:is(.dark *){background-color:rgba(80,7,36,.3)}.dark\:bg-pink-950\/35:is(.dark *){background-color:rgba(80,7,36,.35)}.dark\:bg-pink-950\/40:is(.dark *){background-color:rgba(80,7,36,.4)}.dark\:bg-pink-950\/45:is(.dark *){background-color:rgba(80,7,36,.45)}.dark\:bg-pink-950\/5:is(.dark *){background-color:rgba(80,7,36,.05)}.dark\:bg-pink-950\/50:is(.dark *){background-color:rgba(80,7,36,.5)}.dark\:bg-pink-950\/55:is(.dark *){background-color:rgba(80,7,36,.55)}.dark\:bg-pink-950\/60:is(.dark *){background-color:rgba(80,7,36,.6)}.dark\:bg-pink-950\/65:is(.dark *){background-color:rgba(80,7,36,.65)}.dark\:bg-pink-950\/70:is(.dark *){background-color:rgba(80,7,36,.7)}.dark\:bg-pink-950\/75:is(.dark *){background-color:rgba(80,7,36,.75)}.dark\:bg-pink-950\/80:is(.dark *){background-color:rgba(80,7,36,.8)}.dark\:bg-pink-950\/85:is(.dark *){background-color:rgba(80,7,36,.85)}.dark\:bg-pink-950\/90:is(.dark *){background-color:rgba(80,7,36,.9)}.dark\:bg-pink-950\/95:is(.dark *){background-color:rgba(80,7,36,.95)}.dark\:bg-purple-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(243 232 255/var(--tw-bg-opacity,1))}.dark\:bg-purple-100\/0:is(.dark *){background-color:rgba(243,232,255,0)}.dark\:bg-purple-100\/10:is(.dark *){background-color:rgba(243,232,255,.1)}.dark\:bg-purple-100\/100:is(.dark *){background-color:#f3e8ff}.dark\:bg-purple-100\/15:is(.dark *){background-color:rgba(243,232,255,.15)}.dark\:bg-purple-100\/20:is(.dark *){background-color:rgba(243,232,255,.2)}.dark\:bg-purple-100\/25:is(.dark *){background-color:rgba(243,232,255,.25)}.dark\:bg-purple-100\/30:is(.dark *){background-color:rgba(243,232,255,.3)}.dark\:bg-purple-100\/35:is(.dark *){background-color:rgba(243,232,255,.35)}.dark\:bg-purple-100\/40:is(.dark *){background-color:rgba(243,232,255,.4)}.dark\:bg-purple-100\/45:is(.dark *){background-color:rgba(243,232,255,.45)}.dark\:bg-purple-100\/5:is(.dark *){background-color:rgba(243,232,255,.05)}.dark\:bg-purple-100\/50:is(.dark *){background-color:rgba(243,232,255,.5)}.dark\:bg-purple-100\/55:is(.dark *){background-color:rgba(243,232,255,.55)}.dark\:bg-purple-100\/60:is(.dark *){background-color:rgba(243,232,255,.6)}.dark\:bg-purple-100\/65:is(.dark *){background-color:rgba(243,232,255,.65)}.dark\:bg-purple-100\/70:is(.dark *){background-color:rgba(243,232,255,.7)}.dark\:bg-purple-100\/75:is(.dark *){background-color:rgba(243,232,255,.75)}.dark\:bg-purple-100\/80:is(.dark *){background-color:rgba(243,232,255,.8)}.dark\:bg-purple-100\/85:is(.dark *){background-color:rgba(243,232,255,.85)}.dark\:bg-purple-100\/90:is(.dark *){background-color:rgba(243,232,255,.9)}.dark\:bg-purple-100\/95:is(.dark *){background-color:rgba(243,232,255,.95)}.dark\:bg-purple-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(233 213 255/var(--tw-bg-opacity,1))}.dark\:bg-purple-200\/0:is(.dark *){background-color:rgba(233,213,255,0)}.dark\:bg-purple-200\/10:is(.dark *){background-color:rgba(233,213,255,.1)}.dark\:bg-purple-200\/100:is(.dark *){background-color:#e9d5ff}.dark\:bg-purple-200\/15:is(.dark *){background-color:rgba(233,213,255,.15)}.dark\:bg-purple-200\/20:is(.dark *){background-color:rgba(233,213,255,.2)}.dark\:bg-purple-200\/25:is(.dark *){background-color:rgba(233,213,255,.25)}.dark\:bg-purple-200\/30:is(.dark *){background-color:rgba(233,213,255,.3)}.dark\:bg-purple-200\/35:is(.dark *){background-color:rgba(233,213,255,.35)}.dark\:bg-purple-200\/40:is(.dark *){background-color:rgba(233,213,255,.4)}.dark\:bg-purple-200\/45:is(.dark *){background-color:rgba(233,213,255,.45)}.dark\:bg-purple-200\/5:is(.dark *){background-color:rgba(233,213,255,.05)}.dark\:bg-purple-200\/50:is(.dark *){background-color:rgba(233,213,255,.5)}.dark\:bg-purple-200\/55:is(.dark *){background-color:rgba(233,213,255,.55)}.dark\:bg-purple-200\/60:is(.dark *){background-color:rgba(233,213,255,.6)}.dark\:bg-purple-200\/65:is(.dark *){background-color:rgba(233,213,255,.65)}.dark\:bg-purple-200\/70:is(.dark *){background-color:rgba(233,213,255,.7)}.dark\:bg-purple-200\/75:is(.dark *){background-color:rgba(233,213,255,.75)}.dark\:bg-purple-200\/80:is(.dark *){background-color:rgba(233,213,255,.8)}.dark\:bg-purple-200\/85:is(.dark *){background-color:rgba(233,213,255,.85)}.dark\:bg-purple-200\/90:is(.dark *){background-color:rgba(233,213,255,.9)}.dark\:bg-purple-200\/95:is(.dark *){background-color:rgba(233,213,255,.95)}.dark\:bg-purple-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(192 132 252/var(--tw-bg-opacity,1))}.dark\:bg-purple-400\/0:is(.dark *){background-color:rgba(192,132,252,0)}.dark\:bg-purple-400\/10:is(.dark *){background-color:rgba(192,132,252,.1)}.dark\:bg-purple-400\/100:is(.dark *){background-color:#c084fc}.dark\:bg-purple-400\/15:is(.dark *){background-color:rgba(192,132,252,.15)}.dark\:bg-purple-400\/20:is(.dark *){background-color:rgba(192,132,252,.2)}.dark\:bg-purple-400\/25:is(.dark *){background-color:rgba(192,132,252,.25)}.dark\:bg-purple-400\/30:is(.dark *){background-color:rgba(192,132,252,.3)}.dark\:bg-purple-400\/35:is(.dark *){background-color:rgba(192,132,252,.35)}.dark\:bg-purple-400\/40:is(.dark *){background-color:rgba(192,132,252,.4)}.dark\:bg-purple-400\/45:is(.dark *){background-color:rgba(192,132,252,.45)}.dark\:bg-purple-400\/5:is(.dark *){background-color:rgba(192,132,252,.05)}.dark\:bg-purple-400\/50:is(.dark *){background-color:rgba(192,132,252,.5)}.dark\:bg-purple-400\/55:is(.dark *){background-color:rgba(192,132,252,.55)}.dark\:bg-purple-400\/60:is(.dark *){background-color:rgba(192,132,252,.6)}.dark\:bg-purple-400\/65:is(.dark *){background-color:rgba(192,132,252,.65)}.dark\:bg-purple-400\/70:is(.dark *){background-color:rgba(192,132,252,.7)}.dark\:bg-purple-400\/75:is(.dark *){background-color:rgba(192,132,252,.75)}.dark\:bg-purple-400\/80:is(.dark *){background-color:rgba(192,132,252,.8)}.dark\:bg-purple-400\/85:is(.dark *){background-color:rgba(192,132,252,.85)}.dark\:bg-purple-400\/90:is(.dark *){background-color:rgba(192,132,252,.9)}.dark\:bg-purple-400\/95:is(.dark *){background-color:rgba(192,132,252,.95)}.dark\:bg-purple-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(250 245 255/var(--tw-bg-opacity,1))}.dark\:bg-purple-50\/0:is(.dark *){background-color:rgba(250,245,255,0)}.dark\:bg-purple-50\/10:is(.dark *){background-color:rgba(250,245,255,.1)}.dark\:bg-purple-50\/100:is(.dark *){background-color:#faf5ff}.dark\:bg-purple-50\/15:is(.dark *){background-color:rgba(250,245,255,.15)}.dark\:bg-purple-50\/20:is(.dark *){background-color:rgba(250,245,255,.2)}.dark\:bg-purple-50\/25:is(.dark *){background-color:rgba(250,245,255,.25)}.dark\:bg-purple-50\/30:is(.dark *){background-color:rgba(250,245,255,.3)}.dark\:bg-purple-50\/35:is(.dark *){background-color:rgba(250,245,255,.35)}.dark\:bg-purple-50\/40:is(.dark *){background-color:rgba(250,245,255,.4)}.dark\:bg-purple-50\/45:is(.dark *){background-color:rgba(250,245,255,.45)}.dark\:bg-purple-50\/5:is(.dark *){background-color:rgba(250,245,255,.05)}.dark\:bg-purple-50\/50:is(.dark *){background-color:rgba(250,245,255,.5)}.dark\:bg-purple-50\/55:is(.dark *){background-color:rgba(250,245,255,.55)}.dark\:bg-purple-50\/60:is(.dark *){background-color:rgba(250,245,255,.6)}.dark\:bg-purple-50\/65:is(.dark *){background-color:rgba(250,245,255,.65)}.dark\:bg-purple-50\/70:is(.dark *){background-color:rgba(250,245,255,.7)}.dark\:bg-purple-50\/75:is(.dark *){background-color:rgba(250,245,255,.75)}.dark\:bg-purple-50\/80:is(.dark *){background-color:rgba(250,245,255,.8)}.dark\:bg-purple-50\/85:is(.dark *){background-color:rgba(250,245,255,.85)}.dark\:bg-purple-50\/90:is(.dark *){background-color:rgba(250,245,255,.9)}.dark\:bg-purple-50\/95:is(.dark *){background-color:rgba(250,245,255,.95)}.dark\:bg-purple-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(168 85 247/var(--tw-bg-opacity,1))}.dark\:bg-purple-500\/0:is(.dark *){background-color:rgba(168,85,247,0)}.dark\:bg-purple-500\/10:is(.dark *){background-color:rgba(168,85,247,.1)}.dark\:bg-purple-500\/100:is(.dark *){background-color:#a855f7}.dark\:bg-purple-500\/15:is(.dark *){background-color:rgba(168,85,247,.15)}.dark\:bg-purple-500\/20:is(.dark *){background-color:rgba(168,85,247,.2)}.dark\:bg-purple-500\/25:is(.dark *){background-color:rgba(168,85,247,.25)}.dark\:bg-purple-500\/30:is(.dark *){background-color:rgba(168,85,247,.3)}.dark\:bg-purple-500\/35:is(.dark *){background-color:rgba(168,85,247,.35)}.dark\:bg-purple-500\/40:is(.dark *){background-color:rgba(168,85,247,.4)}.dark\:bg-purple-500\/45:is(.dark *){background-color:rgba(168,85,247,.45)}.dark\:bg-purple-500\/5:is(.dark *){background-color:rgba(168,85,247,.05)}.dark\:bg-purple-500\/50:is(.dark *){background-color:rgba(168,85,247,.5)}.dark\:bg-purple-500\/55:is(.dark *){background-color:rgba(168,85,247,.55)}.dark\:bg-purple-500\/60:is(.dark *){background-color:rgba(168,85,247,.6)}.dark\:bg-purple-500\/65:is(.dark *){background-color:rgba(168,85,247,.65)}.dark\:bg-purple-500\/70:is(.dark *){background-color:rgba(168,85,247,.7)}.dark\:bg-purple-500\/75:is(.dark *){background-color:rgba(168,85,247,.75)}.dark\:bg-purple-500\/80:is(.dark *){background-color:rgba(168,85,247,.8)}.dark\:bg-purple-500\/85:is(.dark *){background-color:rgba(168,85,247,.85)}.dark\:bg-purple-500\/90:is(.dark *){background-color:rgba(168,85,247,.9)}.dark\:bg-purple-500\/95:is(.dark *){background-color:rgba(168,85,247,.95)}.dark\:bg-purple-800:is(.dark *){--tw-bg-opacity:1;background-color:rgb(107 33 168/var(--tw-bg-opacity,1))}.dark\:bg-purple-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(59 7 100/var(--tw-bg-opacity,1))}.dark\:bg-purple-950\/0:is(.dark *){background-color:rgba(59,7,100,0)}.dark\:bg-purple-950\/10:is(.dark *){background-color:rgba(59,7,100,.1)}.dark\:bg-purple-950\/100:is(.dark *){background-color:#3b0764}.dark\:bg-purple-950\/15:is(.dark *){background-color:rgba(59,7,100,.15)}.dark\:bg-purple-950\/20:is(.dark *){background-color:rgba(59,7,100,.2)}.dark\:bg-purple-950\/25:is(.dark *){background-color:rgba(59,7,100,.25)}.dark\:bg-purple-950\/30:is(.dark *){background-color:rgba(59,7,100,.3)}.dark\:bg-purple-950\/35:is(.dark *){background-color:rgba(59,7,100,.35)}.dark\:bg-purple-950\/40:is(.dark *){background-color:rgba(59,7,100,.4)}.dark\:bg-purple-950\/45:is(.dark *){background-color:rgba(59,7,100,.45)}.dark\:bg-purple-950\/5:is(.dark *){background-color:rgba(59,7,100,.05)}.dark\:bg-purple-950\/50:is(.dark *){background-color:rgba(59,7,100,.5)}.dark\:bg-purple-950\/55:is(.dark *){background-color:rgba(59,7,100,.55)}.dark\:bg-purple-950\/60:is(.dark *){background-color:rgba(59,7,100,.6)}.dark\:bg-purple-950\/65:is(.dark *){background-color:rgba(59,7,100,.65)}.dark\:bg-purple-950\/70:is(.dark *){background-color:rgba(59,7,100,.7)}.dark\:bg-purple-950\/75:is(.dark *){background-color:rgba(59,7,100,.75)}.dark\:bg-purple-950\/80:is(.dark *){background-color:rgba(59,7,100,.8)}.dark\:bg-purple-950\/85:is(.dark *){background-color:rgba(59,7,100,.85)}.dark\:bg-purple-950\/90:is(.dark *){background-color:rgba(59,7,100,.9)}.dark\:bg-purple-950\/95:is(.dark *){background-color:rgba(59,7,100,.95)}.dark\:bg-red-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(254 226 226/var(--tw-bg-opacity,1))}.dark\:bg-red-100\/0:is(.dark *){background-color:hsla(0,93%,94%,0)}.dark\:bg-red-100\/10:is(.dark *){background-color:hsla(0,93%,94%,.1)}.dark\:bg-red-100\/100:is(.dark *){background-color:#fee2e2}.dark\:bg-red-100\/15:is(.dark *){background-color:hsla(0,93%,94%,.15)}.dark\:bg-red-100\/20:is(.dark *){background-color:hsla(0,93%,94%,.2)}.dark\:bg-red-100\/25:is(.dark *){background-color:hsla(0,93%,94%,.25)}.dark\:bg-red-100\/30:is(.dark *){background-color:hsla(0,93%,94%,.3)}.dark\:bg-red-100\/35:is(.dark *){background-color:hsla(0,93%,94%,.35)}.dark\:bg-red-100\/40:is(.dark *){background-color:hsla(0,93%,94%,.4)}.dark\:bg-red-100\/45:is(.dark *){background-color:hsla(0,93%,94%,.45)}.dark\:bg-red-100\/5:is(.dark *){background-color:hsla(0,93%,94%,.05)}.dark\:bg-red-100\/50:is(.dark *){background-color:hsla(0,93%,94%,.5)}.dark\:bg-red-100\/55:is(.dark *){background-color:hsla(0,93%,94%,.55)}.dark\:bg-red-100\/60:is(.dark *){background-color:hsla(0,93%,94%,.6)}.dark\:bg-red-100\/65:is(.dark *){background-color:hsla(0,93%,94%,.65)}.dark\:bg-red-100\/70:is(.dark *){background-color:hsla(0,93%,94%,.7)}.dark\:bg-red-100\/75:is(.dark *){background-color:hsla(0,93%,94%,.75)}.dark\:bg-red-100\/80:is(.dark *){background-color:hsla(0,93%,94%,.8)}.dark\:bg-red-100\/85:is(.dark *){background-color:hsla(0,93%,94%,.85)}.dark\:bg-red-100\/90:is(.dark *){background-color:hsla(0,93%,94%,.9)}.dark\:bg-red-100\/95:is(.dark *){background-color:hsla(0,93%,94%,.95)}.dark\:bg-red-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(254 202 202/var(--tw-bg-opacity,1))}.dark\:bg-red-200\/0:is(.dark *){background-color:hsla(0,96%,89%,0)}.dark\:bg-red-200\/10:is(.dark *){background-color:hsla(0,96%,89%,.1)}.dark\:bg-red-200\/100:is(.dark *){background-color:#fecaca}.dark\:bg-red-200\/15:is(.dark *){background-color:hsla(0,96%,89%,.15)}.dark\:bg-red-200\/20:is(.dark *){background-color:hsla(0,96%,89%,.2)}.dark\:bg-red-200\/25:is(.dark *){background-color:hsla(0,96%,89%,.25)}.dark\:bg-red-200\/30:is(.dark *){background-color:hsla(0,96%,89%,.3)}.dark\:bg-red-200\/35:is(.dark *){background-color:hsla(0,96%,89%,.35)}.dark\:bg-red-200\/40:is(.dark *){background-color:hsla(0,96%,89%,.4)}.dark\:bg-red-200\/45:is(.dark *){background-color:hsla(0,96%,89%,.45)}.dark\:bg-red-200\/5:is(.dark *){background-color:hsla(0,96%,89%,.05)}.dark\:bg-red-200\/50:is(.dark *){background-color:hsla(0,96%,89%,.5)}.dark\:bg-red-200\/55:is(.dark *){background-color:hsla(0,96%,89%,.55)}.dark\:bg-red-200\/60:is(.dark *){background-color:hsla(0,96%,89%,.6)}.dark\:bg-red-200\/65:is(.dark *){background-color:hsla(0,96%,89%,.65)}.dark\:bg-red-200\/70:is(.dark *){background-color:hsla(0,96%,89%,.7)}.dark\:bg-red-200\/75:is(.dark *){background-color:hsla(0,96%,89%,.75)}.dark\:bg-red-200\/80:is(.dark *){background-color:hsla(0,96%,89%,.8)}.dark\:bg-red-200\/85:is(.dark *){background-color:hsla(0,96%,89%,.85)}.dark\:bg-red-200\/90:is(.dark *){background-color:hsla(0,96%,89%,.9)}.dark\:bg-red-200\/95:is(.dark *){background-color:hsla(0,96%,89%,.95)}.dark\:bg-red-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(248 113 113/var(--tw-bg-opacity,1))}.dark\:bg-red-400\/0:is(.dark *){background-color:hsla(0,91%,71%,0)}.dark\:bg-red-400\/10:is(.dark *){background-color:hsla(0,91%,71%,.1)}.dark\:bg-red-400\/100:is(.dark *){background-color:#f87171}.dark\:bg-red-400\/15:is(.dark *){background-color:hsla(0,91%,71%,.15)}.dark\:bg-red-400\/20:is(.dark *){background-color:hsla(0,91%,71%,.2)}.dark\:bg-red-400\/25:is(.dark *){background-color:hsla(0,91%,71%,.25)}.dark\:bg-red-400\/30:is(.dark *){background-color:hsla(0,91%,71%,.3)}.dark\:bg-red-400\/35:is(.dark *){background-color:hsla(0,91%,71%,.35)}.dark\:bg-red-400\/40:is(.dark *){background-color:hsla(0,91%,71%,.4)}.dark\:bg-red-400\/45:is(.dark *){background-color:hsla(0,91%,71%,.45)}.dark\:bg-red-400\/5:is(.dark *){background-color:hsla(0,91%,71%,.05)}.dark\:bg-red-400\/50:is(.dark *){background-color:hsla(0,91%,71%,.5)}.dark\:bg-red-400\/55:is(.dark *){background-color:hsla(0,91%,71%,.55)}.dark\:bg-red-400\/60:is(.dark *){background-color:hsla(0,91%,71%,.6)}.dark\:bg-red-400\/65:is(.dark *){background-color:hsla(0,91%,71%,.65)}.dark\:bg-red-400\/70:is(.dark *){background-color:hsla(0,91%,71%,.7)}.dark\:bg-red-400\/75:is(.dark *){background-color:hsla(0,91%,71%,.75)}.dark\:bg-red-400\/80:is(.dark *){background-color:hsla(0,91%,71%,.8)}.dark\:bg-red-400\/85:is(.dark *){background-color:hsla(0,91%,71%,.85)}.dark\:bg-red-400\/90:is(.dark *){background-color:hsla(0,91%,71%,.9)}.dark\:bg-red-400\/95:is(.dark *){background-color:hsla(0,91%,71%,.95)}.dark\:bg-red-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(254 242 242/var(--tw-bg-opacity,1))}.dark\:bg-red-50\/0:is(.dark *){background-color:hsla(0,86%,97%,0)}.dark\:bg-red-50\/10:is(.dark *){background-color:hsla(0,86%,97%,.1)}.dark\:bg-red-50\/100:is(.dark *){background-color:#fef2f2}.dark\:bg-red-50\/15:is(.dark *){background-color:hsla(0,86%,97%,.15)}.dark\:bg-red-50\/20:is(.dark *){background-color:hsla(0,86%,97%,.2)}.dark\:bg-red-50\/25:is(.dark *){background-color:hsla(0,86%,97%,.25)}.dark\:bg-red-50\/30:is(.dark *){background-color:hsla(0,86%,97%,.3)}.dark\:bg-red-50\/35:is(.dark *){background-color:hsla(0,86%,97%,.35)}.dark\:bg-red-50\/40:is(.dark *){background-color:hsla(0,86%,97%,.4)}.dark\:bg-red-50\/45:is(.dark *){background-color:hsla(0,86%,97%,.45)}.dark\:bg-red-50\/5:is(.dark *){background-color:hsla(0,86%,97%,.05)}.dark\:bg-red-50\/50:is(.dark *){background-color:hsla(0,86%,97%,.5)}.dark\:bg-red-50\/55:is(.dark *){background-color:hsla(0,86%,97%,.55)}.dark\:bg-red-50\/60:is(.dark *){background-color:hsla(0,86%,97%,.6)}.dark\:bg-red-50\/65:is(.dark *){background-color:hsla(0,86%,97%,.65)}.dark\:bg-red-50\/70:is(.dark *){background-color:hsla(0,86%,97%,.7)}.dark\:bg-red-50\/75:is(.dark *){background-color:hsla(0,86%,97%,.75)}.dark\:bg-red-50\/80:is(.dark *){background-color:hsla(0,86%,97%,.8)}.dark\:bg-red-50\/85:is(.dark *){background-color:hsla(0,86%,97%,.85)}.dark\:bg-red-50\/90:is(.dark *){background-color:hsla(0,86%,97%,.9)}.dark\:bg-red-50\/95:is(.dark *){background-color:hsla(0,86%,97%,.95)}.dark\:bg-red-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(239 68 68/var(--tw-bg-opacity,1))}.dark\:bg-red-500\/0:is(.dark *){background-color:rgba(239,68,68,0)}.dark\:bg-red-500\/10:is(.dark *){background-color:rgba(239,68,68,.1)}.dark\:bg-red-500\/100:is(.dark *){background-color:#ef4444}.dark\:bg-red-500\/15:is(.dark *){background-color:rgba(239,68,68,.15)}.dark\:bg-red-500\/20:is(.dark *){background-color:rgba(239,68,68,.2)}.dark\:bg-red-500\/25:is(.dark *){background-color:rgba(239,68,68,.25)}.dark\:bg-red-500\/30:is(.dark *){background-color:rgba(239,68,68,.3)}.dark\:bg-red-500\/35:is(.dark *){background-color:rgba(239,68,68,.35)}.dark\:bg-red-500\/40:is(.dark *){background-color:rgba(239,68,68,.4)}.dark\:bg-red-500\/45:is(.dark *){background-color:rgba(239,68,68,.45)}.dark\:bg-red-500\/5:is(.dark *){background-color:rgba(239,68,68,.05)}.dark\:bg-red-500\/50:is(.dark *){background-color:rgba(239,68,68,.5)}.dark\:bg-red-500\/55:is(.dark *){background-color:rgba(239,68,68,.55)}.dark\:bg-red-500\/60:is(.dark *){background-color:rgba(239,68,68,.6)}.dark\:bg-red-500\/65:is(.dark *){background-color:rgba(239,68,68,.65)}.dark\:bg-red-500\/70:is(.dark *){background-color:rgba(239,68,68,.7)}.dark\:bg-red-500\/75:is(.dark *){background-color:rgba(239,68,68,.75)}.dark\:bg-red-500\/80:is(.dark *){background-color:rgba(239,68,68,.8)}.dark\:bg-red-500\/85:is(.dark *){background-color:rgba(239,68,68,.85)}.dark\:bg-red-500\/90:is(.dark *){background-color:rgba(239,68,68,.9)}.dark\:bg-red-500\/95:is(.dark *){background-color:rgba(239,68,68,.95)}.dark\:bg-red-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(69 10 10/var(--tw-bg-opacity,1))}.dark\:bg-red-950\/0:is(.dark *){background-color:rgba(69,10,10,0)}.dark\:bg-red-950\/10:is(.dark *){background-color:rgba(69,10,10,.1)}.dark\:bg-red-950\/100:is(.dark *){background-color:#450a0a}.dark\:bg-red-950\/15:is(.dark *){background-color:rgba(69,10,10,.15)}.dark\:bg-red-950\/20:is(.dark *){background-color:rgba(69,10,10,.2)}.dark\:bg-red-950\/25:is(.dark *){background-color:rgba(69,10,10,.25)}.dark\:bg-red-950\/30:is(.dark *){background-color:rgba(69,10,10,.3)}.dark\:bg-red-950\/35:is(.dark *){background-color:rgba(69,10,10,.35)}.dark\:bg-red-950\/40:is(.dark *){background-color:rgba(69,10,10,.4)}.dark\:bg-red-950\/45:is(.dark *){background-color:rgba(69,10,10,.45)}.dark\:bg-red-950\/5:is(.dark *){background-color:rgba(69,10,10,.05)}.dark\:bg-red-950\/50:is(.dark *){background-color:rgba(69,10,10,.5)}.dark\:bg-red-950\/55:is(.dark *){background-color:rgba(69,10,10,.55)}.dark\:bg-red-950\/60:is(.dark *){background-color:rgba(69,10,10,.6)}.dark\:bg-red-950\/65:is(.dark *){background-color:rgba(69,10,10,.65)}.dark\:bg-red-950\/70:is(.dark *){background-color:rgba(69,10,10,.7)}.dark\:bg-red-950\/75:is(.dark *){background-color:rgba(69,10,10,.75)}.dark\:bg-red-950\/80:is(.dark *){background-color:rgba(69,10,10,.8)}.dark\:bg-red-950\/85:is(.dark *){background-color:rgba(69,10,10,.85)}.dark\:bg-red-950\/90:is(.dark *){background-color:rgba(69,10,10,.9)}.dark\:bg-red-950\/95:is(.dark *){background-color:rgba(69,10,10,.95)}.dark\:bg-rose-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(255 228 230/var(--tw-bg-opacity,1))}.dark\:bg-rose-100\/0:is(.dark *){background-color:rgba(255,228,230,0)}.dark\:bg-rose-100\/10:is(.dark *){background-color:rgba(255,228,230,.1)}.dark\:bg-rose-100\/100:is(.dark *){background-color:#ffe4e6}.dark\:bg-rose-100\/15:is(.dark *){background-color:rgba(255,228,230,.15)}.dark\:bg-rose-100\/20:is(.dark *){background-color:rgba(255,228,230,.2)}.dark\:bg-rose-100\/25:is(.dark *){background-color:rgba(255,228,230,.25)}.dark\:bg-rose-100\/30:is(.dark *){background-color:rgba(255,228,230,.3)}.dark\:bg-rose-100\/35:is(.dark *){background-color:rgba(255,228,230,.35)}.dark\:bg-rose-100\/40:is(.dark *){background-color:rgba(255,228,230,.4)}.dark\:bg-rose-100\/45:is(.dark *){background-color:rgba(255,228,230,.45)}.dark\:bg-rose-100\/5:is(.dark *){background-color:rgba(255,228,230,.05)}.dark\:bg-rose-100\/50:is(.dark *){background-color:rgba(255,228,230,.5)}.dark\:bg-rose-100\/55:is(.dark *){background-color:rgba(255,228,230,.55)}.dark\:bg-rose-100\/60:is(.dark *){background-color:rgba(255,228,230,.6)}.dark\:bg-rose-100\/65:is(.dark *){background-color:rgba(255,228,230,.65)}.dark\:bg-rose-100\/70:is(.dark *){background-color:rgba(255,228,230,.7)}.dark\:bg-rose-100\/75:is(.dark *){background-color:rgba(255,228,230,.75)}.dark\:bg-rose-100\/80:is(.dark *){background-color:rgba(255,228,230,.8)}.dark\:bg-rose-100\/85:is(.dark *){background-color:rgba(255,228,230,.85)}.dark\:bg-rose-100\/90:is(.dark *){background-color:rgba(255,228,230,.9)}.dark\:bg-rose-100\/95:is(.dark *){background-color:rgba(255,228,230,.95)}.dark\:bg-rose-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(254 205 211/var(--tw-bg-opacity,1))}.dark\:bg-rose-200\/0:is(.dark *){background-color:rgba(254,205,211,0)}.dark\:bg-rose-200\/10:is(.dark *){background-color:rgba(254,205,211,.1)}.dark\:bg-rose-200\/100:is(.dark *){background-color:#fecdd3}.dark\:bg-rose-200\/15:is(.dark *){background-color:rgba(254,205,211,.15)}.dark\:bg-rose-200\/20:is(.dark *){background-color:rgba(254,205,211,.2)}.dark\:bg-rose-200\/25:is(.dark *){background-color:rgba(254,205,211,.25)}.dark\:bg-rose-200\/30:is(.dark *){background-color:rgba(254,205,211,.3)}.dark\:bg-rose-200\/35:is(.dark *){background-color:rgba(254,205,211,.35)}.dark\:bg-rose-200\/40:is(.dark *){background-color:rgba(254,205,211,.4)}.dark\:bg-rose-200\/45:is(.dark *){background-color:rgba(254,205,211,.45)}.dark\:bg-rose-200\/5:is(.dark *){background-color:rgba(254,205,211,.05)}.dark\:bg-rose-200\/50:is(.dark *){background-color:rgba(254,205,211,.5)}.dark\:bg-rose-200\/55:is(.dark *){background-color:rgba(254,205,211,.55)}.dark\:bg-rose-200\/60:is(.dark *){background-color:rgba(254,205,211,.6)}.dark\:bg-rose-200\/65:is(.dark *){background-color:rgba(254,205,211,.65)}.dark\:bg-rose-200\/70:is(.dark *){background-color:rgba(254,205,211,.7)}.dark\:bg-rose-200\/75:is(.dark *){background-color:rgba(254,205,211,.75)}.dark\:bg-rose-200\/80:is(.dark *){background-color:rgba(254,205,211,.8)}.dark\:bg-rose-200\/85:is(.dark *){background-color:rgba(254,205,211,.85)}.dark\:bg-rose-200\/90:is(.dark *){background-color:rgba(254,205,211,.9)}.dark\:bg-rose-200\/95:is(.dark *){background-color:rgba(254,205,211,.95)}.dark\:bg-rose-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(251 113 133/var(--tw-bg-opacity,1))}.dark\:bg-rose-400\/0:is(.dark *){background-color:rgba(251,113,133,0)}.dark\:bg-rose-400\/10:is(.dark *){background-color:rgba(251,113,133,.1)}.dark\:bg-rose-400\/100:is(.dark *){background-color:#fb7185}.dark\:bg-rose-400\/15:is(.dark *){background-color:rgba(251,113,133,.15)}.dark\:bg-rose-400\/20:is(.dark *){background-color:rgba(251,113,133,.2)}.dark\:bg-rose-400\/25:is(.dark *){background-color:rgba(251,113,133,.25)}.dark\:bg-rose-400\/30:is(.dark *){background-color:rgba(251,113,133,.3)}.dark\:bg-rose-400\/35:is(.dark *){background-color:rgba(251,113,133,.35)}.dark\:bg-rose-400\/40:is(.dark *){background-color:rgba(251,113,133,.4)}.dark\:bg-rose-400\/45:is(.dark *){background-color:rgba(251,113,133,.45)}.dark\:bg-rose-400\/5:is(.dark *){background-color:rgba(251,113,133,.05)}.dark\:bg-rose-400\/50:is(.dark *){background-color:rgba(251,113,133,.5)}.dark\:bg-rose-400\/55:is(.dark *){background-color:rgba(251,113,133,.55)}.dark\:bg-rose-400\/60:is(.dark *){background-color:rgba(251,113,133,.6)}.dark\:bg-rose-400\/65:is(.dark *){background-color:rgba(251,113,133,.65)}.dark\:bg-rose-400\/70:is(.dark *){background-color:rgba(251,113,133,.7)}.dark\:bg-rose-400\/75:is(.dark *){background-color:rgba(251,113,133,.75)}.dark\:bg-rose-400\/80:is(.dark *){background-color:rgba(251,113,133,.8)}.dark\:bg-rose-400\/85:is(.dark *){background-color:rgba(251,113,133,.85)}.dark\:bg-rose-400\/90:is(.dark *){background-color:rgba(251,113,133,.9)}.dark\:bg-rose-400\/95:is(.dark *){background-color:rgba(251,113,133,.95)}.dark\:bg-rose-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(255 241 242/var(--tw-bg-opacity,1))}.dark\:bg-rose-50\/0:is(.dark *){background-color:rgba(255,241,242,0)}.dark\:bg-rose-50\/10:is(.dark *){background-color:rgba(255,241,242,.1)}.dark\:bg-rose-50\/100:is(.dark *){background-color:#fff1f2}.dark\:bg-rose-50\/15:is(.dark *){background-color:rgba(255,241,242,.15)}.dark\:bg-rose-50\/20:is(.dark *){background-color:rgba(255,241,242,.2)}.dark\:bg-rose-50\/25:is(.dark *){background-color:rgba(255,241,242,.25)}.dark\:bg-rose-50\/30:is(.dark *){background-color:rgba(255,241,242,.3)}.dark\:bg-rose-50\/35:is(.dark *){background-color:rgba(255,241,242,.35)}.dark\:bg-rose-50\/40:is(.dark *){background-color:rgba(255,241,242,.4)}.dark\:bg-rose-50\/45:is(.dark *){background-color:rgba(255,241,242,.45)}.dark\:bg-rose-50\/5:is(.dark *){background-color:rgba(255,241,242,.05)}.dark\:bg-rose-50\/50:is(.dark *){background-color:rgba(255,241,242,.5)}.dark\:bg-rose-50\/55:is(.dark *){background-color:rgba(255,241,242,.55)}.dark\:bg-rose-50\/60:is(.dark *){background-color:rgba(255,241,242,.6)}.dark\:bg-rose-50\/65:is(.dark *){background-color:rgba(255,241,242,.65)}.dark\:bg-rose-50\/70:is(.dark *){background-color:rgba(255,241,242,.7)}.dark\:bg-rose-50\/75:is(.dark *){background-color:rgba(255,241,242,.75)}.dark\:bg-rose-50\/80:is(.dark *){background-color:rgba(255,241,242,.8)}.dark\:bg-rose-50\/85:is(.dark *){background-color:rgba(255,241,242,.85)}.dark\:bg-rose-50\/90:is(.dark *){background-color:rgba(255,241,242,.9)}.dark\:bg-rose-50\/95:is(.dark *){background-color:rgba(255,241,242,.95)}.dark\:bg-rose-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(244 63 94/var(--tw-bg-opacity,1))}.dark\:bg-rose-500\/0:is(.dark *){background-color:rgba(244,63,94,0)}.dark\:bg-rose-500\/10:is(.dark *){background-color:rgba(244,63,94,.1)}.dark\:bg-rose-500\/100:is(.dark *){background-color:#f43f5e}.dark\:bg-rose-500\/15:is(.dark *){background-color:rgba(244,63,94,.15)}.dark\:bg-rose-500\/20:is(.dark *){background-color:rgba(244,63,94,.2)}.dark\:bg-rose-500\/25:is(.dark *){background-color:rgba(244,63,94,.25)}.dark\:bg-rose-500\/30:is(.dark *){background-color:rgba(244,63,94,.3)}.dark\:bg-rose-500\/35:is(.dark *){background-color:rgba(244,63,94,.35)}.dark\:bg-rose-500\/40:is(.dark *){background-color:rgba(244,63,94,.4)}.dark\:bg-rose-500\/45:is(.dark *){background-color:rgba(244,63,94,.45)}.dark\:bg-rose-500\/5:is(.dark *){background-color:rgba(244,63,94,.05)}.dark\:bg-rose-500\/50:is(.dark *){background-color:rgba(244,63,94,.5)}.dark\:bg-rose-500\/55:is(.dark *){background-color:rgba(244,63,94,.55)}.dark\:bg-rose-500\/60:is(.dark *){background-color:rgba(244,63,94,.6)}.dark\:bg-rose-500\/65:is(.dark *){background-color:rgba(244,63,94,.65)}.dark\:bg-rose-500\/70:is(.dark *){background-color:rgba(244,63,94,.7)}.dark\:bg-rose-500\/75:is(.dark *){background-color:rgba(244,63,94,.75)}.dark\:bg-rose-500\/80:is(.dark *){background-color:rgba(244,63,94,.8)}.dark\:bg-rose-500\/85:is(.dark *){background-color:rgba(244,63,94,.85)}.dark\:bg-rose-500\/90:is(.dark *){background-color:rgba(244,63,94,.9)}.dark\:bg-rose-500\/95:is(.dark *){background-color:rgba(244,63,94,.95)}.dark\:bg-rose-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(76 5 25/var(--tw-bg-opacity,1))}.dark\:bg-rose-950\/0:is(.dark *){background-color:rgba(76,5,25,0)}.dark\:bg-rose-950\/10:is(.dark *){background-color:rgba(76,5,25,.1)}.dark\:bg-rose-950\/100:is(.dark *){background-color:#4c0519}.dark\:bg-rose-950\/15:is(.dark *){background-color:rgba(76,5,25,.15)}.dark\:bg-rose-950\/20:is(.dark *){background-color:rgba(76,5,25,.2)}.dark\:bg-rose-950\/25:is(.dark *){background-color:rgba(76,5,25,.25)}.dark\:bg-rose-950\/30:is(.dark *){background-color:rgba(76,5,25,.3)}.dark\:bg-rose-950\/35:is(.dark *){background-color:rgba(76,5,25,.35)}.dark\:bg-rose-950\/40:is(.dark *){background-color:rgba(76,5,25,.4)}.dark\:bg-rose-950\/45:is(.dark *){background-color:rgba(76,5,25,.45)}.dark\:bg-rose-950\/5:is(.dark *){background-color:rgba(76,5,25,.05)}.dark\:bg-rose-950\/50:is(.dark *){background-color:rgba(76,5,25,.5)}.dark\:bg-rose-950\/55:is(.dark *){background-color:rgba(76,5,25,.55)}.dark\:bg-rose-950\/60:is(.dark *){background-color:rgba(76,5,25,.6)}.dark\:bg-rose-950\/65:is(.dark *){background-color:rgba(76,5,25,.65)}.dark\:bg-rose-950\/70:is(.dark *){background-color:rgba(76,5,25,.7)}.dark\:bg-rose-950\/75:is(.dark *){background-color:rgba(76,5,25,.75)}.dark\:bg-rose-950\/80:is(.dark *){background-color:rgba(76,5,25,.8)}.dark\:bg-rose-950\/85:is(.dark *){background-color:rgba(76,5,25,.85)}.dark\:bg-rose-950\/90:is(.dark *){background-color:rgba(76,5,25,.9)}.dark\:bg-rose-950\/95:is(.dark *){background-color:rgba(76,5,25,.95)}.dark\:bg-sky-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(224 242 254/var(--tw-bg-opacity,1))}.dark\:bg-sky-100\/0:is(.dark *){background-color:rgba(224,242,254,0)}.dark\:bg-sky-100\/10:is(.dark *){background-color:rgba(224,242,254,.1)}.dark\:bg-sky-100\/100:is(.dark *){background-color:#e0f2fe}.dark\:bg-sky-100\/15:is(.dark *){background-color:rgba(224,242,254,.15)}.dark\:bg-sky-100\/20:is(.dark *){background-color:rgba(224,242,254,.2)}.dark\:bg-sky-100\/25:is(.dark *){background-color:rgba(224,242,254,.25)}.dark\:bg-sky-100\/30:is(.dark *){background-color:rgba(224,242,254,.3)}.dark\:bg-sky-100\/35:is(.dark *){background-color:rgba(224,242,254,.35)}.dark\:bg-sky-100\/40:is(.dark *){background-color:rgba(224,242,254,.4)}.dark\:bg-sky-100\/45:is(.dark *){background-color:rgba(224,242,254,.45)}.dark\:bg-sky-100\/5:is(.dark *){background-color:rgba(224,242,254,.05)}.dark\:bg-sky-100\/50:is(.dark *){background-color:rgba(224,242,254,.5)}.dark\:bg-sky-100\/55:is(.dark *){background-color:rgba(224,242,254,.55)}.dark\:bg-sky-100\/60:is(.dark *){background-color:rgba(224,242,254,.6)}.dark\:bg-sky-100\/65:is(.dark *){background-color:rgba(224,242,254,.65)}.dark\:bg-sky-100\/70:is(.dark *){background-color:rgba(224,242,254,.7)}.dark\:bg-sky-100\/75:is(.dark *){background-color:rgba(224,242,254,.75)}.dark\:bg-sky-100\/80:is(.dark *){background-color:rgba(224,242,254,.8)}.dark\:bg-sky-100\/85:is(.dark *){background-color:rgba(224,242,254,.85)}.dark\:bg-sky-100\/90:is(.dark *){background-color:rgba(224,242,254,.9)}.dark\:bg-sky-100\/95:is(.dark *){background-color:rgba(224,242,254,.95)}.dark\:bg-sky-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(186 230 253/var(--tw-bg-opacity,1))}.dark\:bg-sky-200\/0:is(.dark *){background-color:rgba(186,230,253,0)}.dark\:bg-sky-200\/10:is(.dark *){background-color:rgba(186,230,253,.1)}.dark\:bg-sky-200\/100:is(.dark *){background-color:#bae6fd}.dark\:bg-sky-200\/15:is(.dark *){background-color:rgba(186,230,253,.15)}.dark\:bg-sky-200\/20:is(.dark *){background-color:rgba(186,230,253,.2)}.dark\:bg-sky-200\/25:is(.dark *){background-color:rgba(186,230,253,.25)}.dark\:bg-sky-200\/30:is(.dark *){background-color:rgba(186,230,253,.3)}.dark\:bg-sky-200\/35:is(.dark *){background-color:rgba(186,230,253,.35)}.dark\:bg-sky-200\/40:is(.dark *){background-color:rgba(186,230,253,.4)}.dark\:bg-sky-200\/45:is(.dark *){background-color:rgba(186,230,253,.45)}.dark\:bg-sky-200\/5:is(.dark *){background-color:rgba(186,230,253,.05)}.dark\:bg-sky-200\/50:is(.dark *){background-color:rgba(186,230,253,.5)}.dark\:bg-sky-200\/55:is(.dark *){background-color:rgba(186,230,253,.55)}.dark\:bg-sky-200\/60:is(.dark *){background-color:rgba(186,230,253,.6)}.dark\:bg-sky-200\/65:is(.dark *){background-color:rgba(186,230,253,.65)}.dark\:bg-sky-200\/70:is(.dark *){background-color:rgba(186,230,253,.7)}.dark\:bg-sky-200\/75:is(.dark *){background-color:rgba(186,230,253,.75)}.dark\:bg-sky-200\/80:is(.dark *){background-color:rgba(186,230,253,.8)}.dark\:bg-sky-200\/85:is(.dark *){background-color:rgba(186,230,253,.85)}.dark\:bg-sky-200\/90:is(.dark *){background-color:rgba(186,230,253,.9)}.dark\:bg-sky-200\/95:is(.dark *){background-color:rgba(186,230,253,.95)}.dark\:bg-sky-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(56 189 248/var(--tw-bg-opacity,1))}.dark\:bg-sky-400\/0:is(.dark *){background-color:rgba(56,189,248,0)}.dark\:bg-sky-400\/10:is(.dark *){background-color:rgba(56,189,248,.1)}.dark\:bg-sky-400\/100:is(.dark *){background-color:#38bdf8}.dark\:bg-sky-400\/15:is(.dark *){background-color:rgba(56,189,248,.15)}.dark\:bg-sky-400\/20:is(.dark *){background-color:rgba(56,189,248,.2)}.dark\:bg-sky-400\/25:is(.dark *){background-color:rgba(56,189,248,.25)}.dark\:bg-sky-400\/30:is(.dark *){background-color:rgba(56,189,248,.3)}.dark\:bg-sky-400\/35:is(.dark *){background-color:rgba(56,189,248,.35)}.dark\:bg-sky-400\/40:is(.dark *){background-color:rgba(56,189,248,.4)}.dark\:bg-sky-400\/45:is(.dark *){background-color:rgba(56,189,248,.45)}.dark\:bg-sky-400\/5:is(.dark *){background-color:rgba(56,189,248,.05)}.dark\:bg-sky-400\/50:is(.dark *){background-color:rgba(56,189,248,.5)}.dark\:bg-sky-400\/55:is(.dark *){background-color:rgba(56,189,248,.55)}.dark\:bg-sky-400\/60:is(.dark *){background-color:rgba(56,189,248,.6)}.dark\:bg-sky-400\/65:is(.dark *){background-color:rgba(56,189,248,.65)}.dark\:bg-sky-400\/70:is(.dark *){background-color:rgba(56,189,248,.7)}.dark\:bg-sky-400\/75:is(.dark *){background-color:rgba(56,189,248,.75)}.dark\:bg-sky-400\/80:is(.dark *){background-color:rgba(56,189,248,.8)}.dark\:bg-sky-400\/85:is(.dark *){background-color:rgba(56,189,248,.85)}.dark\:bg-sky-400\/90:is(.dark *){background-color:rgba(56,189,248,.9)}.dark\:bg-sky-400\/95:is(.dark *){background-color:rgba(56,189,248,.95)}.dark\:bg-sky-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(240 249 255/var(--tw-bg-opacity,1))}.dark\:bg-sky-50\/0:is(.dark *){background-color:rgba(240,249,255,0)}.dark\:bg-sky-50\/10:is(.dark *){background-color:rgba(240,249,255,.1)}.dark\:bg-sky-50\/100:is(.dark *){background-color:#f0f9ff}.dark\:bg-sky-50\/15:is(.dark *){background-color:rgba(240,249,255,.15)}.dark\:bg-sky-50\/20:is(.dark *){background-color:rgba(240,249,255,.2)}.dark\:bg-sky-50\/25:is(.dark *){background-color:rgba(240,249,255,.25)}.dark\:bg-sky-50\/30:is(.dark *){background-color:rgba(240,249,255,.3)}.dark\:bg-sky-50\/35:is(.dark *){background-color:rgba(240,249,255,.35)}.dark\:bg-sky-50\/40:is(.dark *){background-color:rgba(240,249,255,.4)}.dark\:bg-sky-50\/45:is(.dark *){background-color:rgba(240,249,255,.45)}.dark\:bg-sky-50\/5:is(.dark *){background-color:rgba(240,249,255,.05)}.dark\:bg-sky-50\/50:is(.dark *){background-color:rgba(240,249,255,.5)}.dark\:bg-sky-50\/55:is(.dark *){background-color:rgba(240,249,255,.55)}.dark\:bg-sky-50\/60:is(.dark *){background-color:rgba(240,249,255,.6)}.dark\:bg-sky-50\/65:is(.dark *){background-color:rgba(240,249,255,.65)}.dark\:bg-sky-50\/70:is(.dark *){background-color:rgba(240,249,255,.7)}.dark\:bg-sky-50\/75:is(.dark *){background-color:rgba(240,249,255,.75)}.dark\:bg-sky-50\/80:is(.dark *){background-color:rgba(240,249,255,.8)}.dark\:bg-sky-50\/85:is(.dark *){background-color:rgba(240,249,255,.85)}.dark\:bg-sky-50\/90:is(.dark *){background-color:rgba(240,249,255,.9)}.dark\:bg-sky-50\/95:is(.dark *){background-color:rgba(240,249,255,.95)}.dark\:bg-sky-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(14 165 233/var(--tw-bg-opacity,1))}.dark\:bg-sky-500\/0:is(.dark *){background-color:rgba(14,165,233,0)}.dark\:bg-sky-500\/10:is(.dark *){background-color:rgba(14,165,233,.1)}.dark\:bg-sky-500\/100:is(.dark *){background-color:#0ea5e9}.dark\:bg-sky-500\/15:is(.dark *){background-color:rgba(14,165,233,.15)}.dark\:bg-sky-500\/20:is(.dark *){background-color:rgba(14,165,233,.2)}.dark\:bg-sky-500\/25:is(.dark *){background-color:rgba(14,165,233,.25)}.dark\:bg-sky-500\/30:is(.dark *){background-color:rgba(14,165,233,.3)}.dark\:bg-sky-500\/35:is(.dark *){background-color:rgba(14,165,233,.35)}.dark\:bg-sky-500\/40:is(.dark *){background-color:rgba(14,165,233,.4)}.dark\:bg-sky-500\/45:is(.dark *){background-color:rgba(14,165,233,.45)}.dark\:bg-sky-500\/5:is(.dark *){background-color:rgba(14,165,233,.05)}.dark\:bg-sky-500\/50:is(.dark *){background-color:rgba(14,165,233,.5)}.dark\:bg-sky-500\/55:is(.dark *){background-color:rgba(14,165,233,.55)}.dark\:bg-sky-500\/60:is(.dark *){background-color:rgba(14,165,233,.6)}.dark\:bg-sky-500\/65:is(.dark *){background-color:rgba(14,165,233,.65)}.dark\:bg-sky-500\/70:is(.dark *){background-color:rgba(14,165,233,.7)}.dark\:bg-sky-500\/75:is(.dark *){background-color:rgba(14,165,233,.75)}.dark\:bg-sky-500\/80:is(.dark *){background-color:rgba(14,165,233,.8)}.dark\:bg-sky-500\/85:is(.dark *){background-color:rgba(14,165,233,.85)}.dark\:bg-sky-500\/90:is(.dark *){background-color:rgba(14,165,233,.9)}.dark\:bg-sky-500\/95:is(.dark *){background-color:rgba(14,165,233,.95)}.dark\:bg-sky-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(8 47 73/var(--tw-bg-opacity,1))}.dark\:bg-sky-950\/0:is(.dark *){background-color:rgba(8,47,73,0)}.dark\:bg-sky-950\/10:is(.dark *){background-color:rgba(8,47,73,.1)}.dark\:bg-sky-950\/100:is(.dark *){background-color:#082f49}.dark\:bg-sky-950\/15:is(.dark *){background-color:rgba(8,47,73,.15)}.dark\:bg-sky-950\/20:is(.dark *){background-color:rgba(8,47,73,.2)}.dark\:bg-sky-950\/25:is(.dark *){background-color:rgba(8,47,73,.25)}.dark\:bg-sky-950\/30:is(.dark *){background-color:rgba(8,47,73,.3)}.dark\:bg-sky-950\/35:is(.dark *){background-color:rgba(8,47,73,.35)}.dark\:bg-sky-950\/40:is(.dark *){background-color:rgba(8,47,73,.4)}.dark\:bg-sky-950\/45:is(.dark *){background-color:rgba(8,47,73,.45)}.dark\:bg-sky-950\/5:is(.dark *){background-color:rgba(8,47,73,.05)}.dark\:bg-sky-950\/50:is(.dark *){background-color:rgba(8,47,73,.5)}.dark\:bg-sky-950\/55:is(.dark *){background-color:rgba(8,47,73,.55)}.dark\:bg-sky-950\/60:is(.dark *){background-color:rgba(8,47,73,.6)}.dark\:bg-sky-950\/65:is(.dark *){background-color:rgba(8,47,73,.65)}.dark\:bg-sky-950\/70:is(.dark *){background-color:rgba(8,47,73,.7)}.dark\:bg-sky-950\/75:is(.dark *){background-color:rgba(8,47,73,.75)}.dark\:bg-sky-950\/80:is(.dark *){background-color:rgba(8,47,73,.8)}.dark\:bg-sky-950\/85:is(.dark *){background-color:rgba(8,47,73,.85)}.dark\:bg-sky-950\/90:is(.dark *){background-color:rgba(8,47,73,.9)}.dark\:bg-sky-950\/95:is(.dark *){background-color:rgba(8,47,73,.95)}.dark\:bg-slate-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(241 245 249/var(--tw-bg-opacity,1))}.dark\:bg-slate-100\/0:is(.dark *){background-color:rgba(241,245,249,0)}.dark\:bg-slate-100\/10:is(.dark *){background-color:rgba(241,245,249,.1)}.dark\:bg-slate-100\/100:is(.dark *){background-color:#f1f5f9}.dark\:bg-slate-100\/15:is(.dark *){background-color:rgba(241,245,249,.15)}.dark\:bg-slate-100\/20:is(.dark *){background-color:rgba(241,245,249,.2)}.dark\:bg-slate-100\/25:is(.dark *){background-color:rgba(241,245,249,.25)}.dark\:bg-slate-100\/30:is(.dark *){background-color:rgba(241,245,249,.3)}.dark\:bg-slate-100\/35:is(.dark *){background-color:rgba(241,245,249,.35)}.dark\:bg-slate-100\/40:is(.dark *){background-color:rgba(241,245,249,.4)}.dark\:bg-slate-100\/45:is(.dark *){background-color:rgba(241,245,249,.45)}.dark\:bg-slate-100\/5:is(.dark *){background-color:rgba(241,245,249,.05)}.dark\:bg-slate-100\/50:is(.dark *){background-color:rgba(241,245,249,.5)}.dark\:bg-slate-100\/55:is(.dark *){background-color:rgba(241,245,249,.55)}.dark\:bg-slate-100\/60:is(.dark *){background-color:rgba(241,245,249,.6)}.dark\:bg-slate-100\/65:is(.dark *){background-color:rgba(241,245,249,.65)}.dark\:bg-slate-100\/70:is(.dark *){background-color:rgba(241,245,249,.7)}.dark\:bg-slate-100\/75:is(.dark *){background-color:rgba(241,245,249,.75)}.dark\:bg-slate-100\/80:is(.dark *){background-color:rgba(241,245,249,.8)}.dark\:bg-slate-100\/85:is(.dark *){background-color:rgba(241,245,249,.85)}.dark\:bg-slate-100\/90:is(.dark *){background-color:rgba(241,245,249,.9)}.dark\:bg-slate-100\/95:is(.dark *){background-color:rgba(241,245,249,.95)}.dark\:bg-slate-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(226 232 240/var(--tw-bg-opacity,1))}.dark\:bg-slate-200\/0:is(.dark *){background-color:rgba(226,232,240,0)}.dark\:bg-slate-200\/10:is(.dark *){background-color:rgba(226,232,240,.1)}.dark\:bg-slate-200\/100:is(.dark *){background-color:#e2e8f0}.dark\:bg-slate-200\/15:is(.dark *){background-color:rgba(226,232,240,.15)}.dark\:bg-slate-200\/20:is(.dark *){background-color:rgba(226,232,240,.2)}.dark\:bg-slate-200\/25:is(.dark *){background-color:rgba(226,232,240,.25)}.dark\:bg-slate-200\/30:is(.dark *){background-color:rgba(226,232,240,.3)}.dark\:bg-slate-200\/35:is(.dark *){background-color:rgba(226,232,240,.35)}.dark\:bg-slate-200\/40:is(.dark *){background-color:rgba(226,232,240,.4)}.dark\:bg-slate-200\/45:is(.dark *){background-color:rgba(226,232,240,.45)}.dark\:bg-slate-200\/5:is(.dark *){background-color:rgba(226,232,240,.05)}.dark\:bg-slate-200\/50:is(.dark *){background-color:rgba(226,232,240,.5)}.dark\:bg-slate-200\/55:is(.dark *){background-color:rgba(226,232,240,.55)}.dark\:bg-slate-200\/60:is(.dark *){background-color:rgba(226,232,240,.6)}.dark\:bg-slate-200\/65:is(.dark *){background-color:rgba(226,232,240,.65)}.dark\:bg-slate-200\/70:is(.dark *){background-color:rgba(226,232,240,.7)}.dark\:bg-slate-200\/75:is(.dark *){background-color:rgba(226,232,240,.75)}.dark\:bg-slate-200\/80:is(.dark *){background-color:rgba(226,232,240,.8)}.dark\:bg-slate-200\/85:is(.dark *){background-color:rgba(226,232,240,.85)}.dark\:bg-slate-200\/90:is(.dark *){background-color:rgba(226,232,240,.9)}.dark\:bg-slate-200\/95:is(.dark *){background-color:rgba(226,232,240,.95)}.dark\:bg-slate-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(148 163 184/var(--tw-bg-opacity,1))}.dark\:bg-slate-400\/0:is(.dark *){background-color:rgba(148,163,184,0)}.dark\:bg-slate-400\/10:is(.dark *){background-color:rgba(148,163,184,.1)}.dark\:bg-slate-400\/100:is(.dark *){background-color:#94a3b8}.dark\:bg-slate-400\/15:is(.dark *){background-color:rgba(148,163,184,.15)}.dark\:bg-slate-400\/20:is(.dark *){background-color:rgba(148,163,184,.2)}.dark\:bg-slate-400\/25:is(.dark *){background-color:rgba(148,163,184,.25)}.dark\:bg-slate-400\/30:is(.dark *){background-color:rgba(148,163,184,.3)}.dark\:bg-slate-400\/35:is(.dark *){background-color:rgba(148,163,184,.35)}.dark\:bg-slate-400\/40:is(.dark *){background-color:rgba(148,163,184,.4)}.dark\:bg-slate-400\/45:is(.dark *){background-color:rgba(148,163,184,.45)}.dark\:bg-slate-400\/5:is(.dark *){background-color:rgba(148,163,184,.05)}.dark\:bg-slate-400\/50:is(.dark *){background-color:rgba(148,163,184,.5)}.dark\:bg-slate-400\/55:is(.dark *){background-color:rgba(148,163,184,.55)}.dark\:bg-slate-400\/60:is(.dark *){background-color:rgba(148,163,184,.6)}.dark\:bg-slate-400\/65:is(.dark *){background-color:rgba(148,163,184,.65)}.dark\:bg-slate-400\/70:is(.dark *){background-color:rgba(148,163,184,.7)}.dark\:bg-slate-400\/75:is(.dark *){background-color:rgba(148,163,184,.75)}.dark\:bg-slate-400\/80:is(.dark *){background-color:rgba(148,163,184,.8)}.dark\:bg-slate-400\/85:is(.dark *){background-color:rgba(148,163,184,.85)}.dark\:bg-slate-400\/90:is(.dark *){background-color:rgba(148,163,184,.9)}.dark\:bg-slate-400\/95:is(.dark *){background-color:rgba(148,163,184,.95)}.dark\:bg-slate-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(248 250 252/var(--tw-bg-opacity,1))}.dark\:bg-slate-50\/0:is(.dark *){background-color:rgba(248,250,252,0)}.dark\:bg-slate-50\/10:is(.dark *){background-color:rgba(248,250,252,.1)}.dark\:bg-slate-50\/100:is(.dark *){background-color:#f8fafc}.dark\:bg-slate-50\/15:is(.dark *){background-color:rgba(248,250,252,.15)}.dark\:bg-slate-50\/20:is(.dark *){background-color:rgba(248,250,252,.2)}.dark\:bg-slate-50\/25:is(.dark *){background-color:rgba(248,250,252,.25)}.dark\:bg-slate-50\/30:is(.dark *){background-color:rgba(248,250,252,.3)}.dark\:bg-slate-50\/35:is(.dark *){background-color:rgba(248,250,252,.35)}.dark\:bg-slate-50\/40:is(.dark *){background-color:rgba(248,250,252,.4)}.dark\:bg-slate-50\/45:is(.dark *){background-color:rgba(248,250,252,.45)}.dark\:bg-slate-50\/5:is(.dark *){background-color:rgba(248,250,252,.05)}.dark\:bg-slate-50\/50:is(.dark *){background-color:rgba(248,250,252,.5)}.dark\:bg-slate-50\/55:is(.dark *){background-color:rgba(248,250,252,.55)}.dark\:bg-slate-50\/60:is(.dark *){background-color:rgba(248,250,252,.6)}.dark\:bg-slate-50\/65:is(.dark *){background-color:rgba(248,250,252,.65)}.dark\:bg-slate-50\/70:is(.dark *){background-color:rgba(248,250,252,.7)}.dark\:bg-slate-50\/75:is(.dark *){background-color:rgba(248,250,252,.75)}.dark\:bg-slate-50\/80:is(.dark *){background-color:rgba(248,250,252,.8)}.dark\:bg-slate-50\/85:is(.dark *){background-color:rgba(248,250,252,.85)}.dark\:bg-slate-50\/90:is(.dark *){background-color:rgba(248,250,252,.9)}.dark\:bg-slate-50\/95:is(.dark *){background-color:rgba(248,250,252,.95)}.dark\:bg-slate-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(100 116 139/var(--tw-bg-opacity,1))}.dark\:bg-slate-500\/0:is(.dark *){background-color:rgba(100,116,139,0)}.dark\:bg-slate-500\/10:is(.dark *){background-color:rgba(100,116,139,.1)}.dark\:bg-slate-500\/100:is(.dark *){background-color:#64748b}.dark\:bg-slate-500\/15:is(.dark *){background-color:rgba(100,116,139,.15)}.dark\:bg-slate-500\/20:is(.dark *){background-color:rgba(100,116,139,.2)}.dark\:bg-slate-500\/25:is(.dark *){background-color:rgba(100,116,139,.25)}.dark\:bg-slate-500\/30:is(.dark *){background-color:rgba(100,116,139,.3)}.dark\:bg-slate-500\/35:is(.dark *){background-color:rgba(100,116,139,.35)}.dark\:bg-slate-500\/40:is(.dark *){background-color:rgba(100,116,139,.4)}.dark\:bg-slate-500\/45:is(.dark *){background-color:rgba(100,116,139,.45)}.dark\:bg-slate-500\/5:is(.dark *){background-color:rgba(100,116,139,.05)}.dark\:bg-slate-500\/50:is(.dark *){background-color:rgba(100,116,139,.5)}.dark\:bg-slate-500\/55:is(.dark *){background-color:rgba(100,116,139,.55)}.dark\:bg-slate-500\/60:is(.dark *){background-color:rgba(100,116,139,.6)}.dark\:bg-slate-500\/65:is(.dark *){background-color:rgba(100,116,139,.65)}.dark\:bg-slate-500\/70:is(.dark *){background-color:rgba(100,116,139,.7)}.dark\:bg-slate-500\/75:is(.dark *){background-color:rgba(100,116,139,.75)}.dark\:bg-slate-500\/80:is(.dark *){background-color:rgba(100,116,139,.8)}.dark\:bg-slate-500\/85:is(.dark *){background-color:rgba(100,116,139,.85)}.dark\:bg-slate-500\/90:is(.dark *){background-color:rgba(100,116,139,.9)}.dark\:bg-slate-500\/95:is(.dark *){background-color:rgba(100,116,139,.95)}.dark\:bg-slate-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(2 6 23/var(--tw-bg-opacity,1))}.dark\:bg-slate-950\/0:is(.dark *){background-color:rgba(2,6,23,0)}.dark\:bg-slate-950\/10:is(.dark *){background-color:rgba(2,6,23,.1)}.dark\:bg-slate-950\/100:is(.dark *){background-color:#020617}.dark\:bg-slate-950\/15:is(.dark *){background-color:rgba(2,6,23,.15)}.dark\:bg-slate-950\/20:is(.dark *){background-color:rgba(2,6,23,.2)}.dark\:bg-slate-950\/25:is(.dark *){background-color:rgba(2,6,23,.25)}.dark\:bg-slate-950\/30:is(.dark *){background-color:rgba(2,6,23,.3)}.dark\:bg-slate-950\/35:is(.dark *){background-color:rgba(2,6,23,.35)}.dark\:bg-slate-950\/40:is(.dark *){background-color:rgba(2,6,23,.4)}.dark\:bg-slate-950\/45:is(.dark *){background-color:rgba(2,6,23,.45)}.dark\:bg-slate-950\/5:is(.dark *){background-color:rgba(2,6,23,.05)}.dark\:bg-slate-950\/50:is(.dark *){background-color:rgba(2,6,23,.5)}.dark\:bg-slate-950\/55:is(.dark *){background-color:rgba(2,6,23,.55)}.dark\:bg-slate-950\/60:is(.dark *){background-color:rgba(2,6,23,.6)}.dark\:bg-slate-950\/65:is(.dark *){background-color:rgba(2,6,23,.65)}.dark\:bg-slate-950\/70:is(.dark *){background-color:rgba(2,6,23,.7)}.dark\:bg-slate-950\/75:is(.dark *){background-color:rgba(2,6,23,.75)}.dark\:bg-slate-950\/80:is(.dark *){background-color:rgba(2,6,23,.8)}.dark\:bg-slate-950\/85:is(.dark *){background-color:rgba(2,6,23,.85)}.dark\:bg-slate-950\/90:is(.dark *){background-color:rgba(2,6,23,.9)}.dark\:bg-slate-950\/95:is(.dark *){background-color:rgba(2,6,23,.95)}.dark\:bg-stone-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(245 245 244/var(--tw-bg-opacity,1))}.dark\:bg-stone-100\/0:is(.dark *){background-color:hsla(60,5%,96%,0)}.dark\:bg-stone-100\/10:is(.dark *){background-color:hsla(60,5%,96%,.1)}.dark\:bg-stone-100\/100:is(.dark *){background-color:#f5f5f4}.dark\:bg-stone-100\/15:is(.dark *){background-color:hsla(60,5%,96%,.15)}.dark\:bg-stone-100\/20:is(.dark *){background-color:hsla(60,5%,96%,.2)}.dark\:bg-stone-100\/25:is(.dark *){background-color:hsla(60,5%,96%,.25)}.dark\:bg-stone-100\/30:is(.dark *){background-color:hsla(60,5%,96%,.3)}.dark\:bg-stone-100\/35:is(.dark *){background-color:hsla(60,5%,96%,.35)}.dark\:bg-stone-100\/40:is(.dark *){background-color:hsla(60,5%,96%,.4)}.dark\:bg-stone-100\/45:is(.dark *){background-color:hsla(60,5%,96%,.45)}.dark\:bg-stone-100\/5:is(.dark *){background-color:hsla(60,5%,96%,.05)}.dark\:bg-stone-100\/50:is(.dark *){background-color:hsla(60,5%,96%,.5)}.dark\:bg-stone-100\/55:is(.dark *){background-color:hsla(60,5%,96%,.55)}.dark\:bg-stone-100\/60:is(.dark *){background-color:hsla(60,5%,96%,.6)}.dark\:bg-stone-100\/65:is(.dark *){background-color:hsla(60,5%,96%,.65)}.dark\:bg-stone-100\/70:is(.dark *){background-color:hsla(60,5%,96%,.7)}.dark\:bg-stone-100\/75:is(.dark *){background-color:hsla(60,5%,96%,.75)}.dark\:bg-stone-100\/80:is(.dark *){background-color:hsla(60,5%,96%,.8)}.dark\:bg-stone-100\/85:is(.dark *){background-color:hsla(60,5%,96%,.85)}.dark\:bg-stone-100\/90:is(.dark *){background-color:hsla(60,5%,96%,.9)}.dark\:bg-stone-100\/95:is(.dark *){background-color:hsla(60,5%,96%,.95)}.dark\:bg-stone-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(231 229 228/var(--tw-bg-opacity,1))}.dark\:bg-stone-200\/0:is(.dark *){background-color:hsla(20,6%,90%,0)}.dark\:bg-stone-200\/10:is(.dark *){background-color:hsla(20,6%,90%,.1)}.dark\:bg-stone-200\/100:is(.dark *){background-color:#e7e5e4}.dark\:bg-stone-200\/15:is(.dark *){background-color:hsla(20,6%,90%,.15)}.dark\:bg-stone-200\/20:is(.dark *){background-color:hsla(20,6%,90%,.2)}.dark\:bg-stone-200\/25:is(.dark *){background-color:hsla(20,6%,90%,.25)}.dark\:bg-stone-200\/30:is(.dark *){background-color:hsla(20,6%,90%,.3)}.dark\:bg-stone-200\/35:is(.dark *){background-color:hsla(20,6%,90%,.35)}.dark\:bg-stone-200\/40:is(.dark *){background-color:hsla(20,6%,90%,.4)}.dark\:bg-stone-200\/45:is(.dark *){background-color:hsla(20,6%,90%,.45)}.dark\:bg-stone-200\/5:is(.dark *){background-color:hsla(20,6%,90%,.05)}.dark\:bg-stone-200\/50:is(.dark *){background-color:hsla(20,6%,90%,.5)}.dark\:bg-stone-200\/55:is(.dark *){background-color:hsla(20,6%,90%,.55)}.dark\:bg-stone-200\/60:is(.dark *){background-color:hsla(20,6%,90%,.6)}.dark\:bg-stone-200\/65:is(.dark *){background-color:hsla(20,6%,90%,.65)}.dark\:bg-stone-200\/70:is(.dark *){background-color:hsla(20,6%,90%,.7)}.dark\:bg-stone-200\/75:is(.dark *){background-color:hsla(20,6%,90%,.75)}.dark\:bg-stone-200\/80:is(.dark *){background-color:hsla(20,6%,90%,.8)}.dark\:bg-stone-200\/85:is(.dark *){background-color:hsla(20,6%,90%,.85)}.dark\:bg-stone-200\/90:is(.dark *){background-color:hsla(20,6%,90%,.9)}.dark\:bg-stone-200\/95:is(.dark *){background-color:hsla(20,6%,90%,.95)}.dark\:bg-stone-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(168 162 158/var(--tw-bg-opacity,1))}.dark\:bg-stone-400\/0:is(.dark *){background-color:hsla(24,5%,64%,0)}.dark\:bg-stone-400\/10:is(.dark *){background-color:hsla(24,5%,64%,.1)}.dark\:bg-stone-400\/100:is(.dark *){background-color:#a8a29e}.dark\:bg-stone-400\/15:is(.dark *){background-color:hsla(24,5%,64%,.15)}.dark\:bg-stone-400\/20:is(.dark *){background-color:hsla(24,5%,64%,.2)}.dark\:bg-stone-400\/25:is(.dark *){background-color:hsla(24,5%,64%,.25)}.dark\:bg-stone-400\/30:is(.dark *){background-color:hsla(24,5%,64%,.3)}.dark\:bg-stone-400\/35:is(.dark *){background-color:hsla(24,5%,64%,.35)}.dark\:bg-stone-400\/40:is(.dark *){background-color:hsla(24,5%,64%,.4)}.dark\:bg-stone-400\/45:is(.dark *){background-color:hsla(24,5%,64%,.45)}.dark\:bg-stone-400\/5:is(.dark *){background-color:hsla(24,5%,64%,.05)}.dark\:bg-stone-400\/50:is(.dark *){background-color:hsla(24,5%,64%,.5)}.dark\:bg-stone-400\/55:is(.dark *){background-color:hsla(24,5%,64%,.55)}.dark\:bg-stone-400\/60:is(.dark *){background-color:hsla(24,5%,64%,.6)}.dark\:bg-stone-400\/65:is(.dark *){background-color:hsla(24,5%,64%,.65)}.dark\:bg-stone-400\/70:is(.dark *){background-color:hsla(24,5%,64%,.7)}.dark\:bg-stone-400\/75:is(.dark *){background-color:hsla(24,5%,64%,.75)}.dark\:bg-stone-400\/80:is(.dark *){background-color:hsla(24,5%,64%,.8)}.dark\:bg-stone-400\/85:is(.dark *){background-color:hsla(24,5%,64%,.85)}.dark\:bg-stone-400\/90:is(.dark *){background-color:hsla(24,5%,64%,.9)}.dark\:bg-stone-400\/95:is(.dark *){background-color:hsla(24,5%,64%,.95)}.dark\:bg-stone-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(250 250 249/var(--tw-bg-opacity,1))}.dark\:bg-stone-50\/0:is(.dark *){background-color:hsla(60,9%,98%,0)}.dark\:bg-stone-50\/10:is(.dark *){background-color:hsla(60,9%,98%,.1)}.dark\:bg-stone-50\/100:is(.dark *){background-color:#fafaf9}.dark\:bg-stone-50\/15:is(.dark *){background-color:hsla(60,9%,98%,.15)}.dark\:bg-stone-50\/20:is(.dark *){background-color:hsla(60,9%,98%,.2)}.dark\:bg-stone-50\/25:is(.dark *){background-color:hsla(60,9%,98%,.25)}.dark\:bg-stone-50\/30:is(.dark *){background-color:hsla(60,9%,98%,.3)}.dark\:bg-stone-50\/35:is(.dark *){background-color:hsla(60,9%,98%,.35)}.dark\:bg-stone-50\/40:is(.dark *){background-color:hsla(60,9%,98%,.4)}.dark\:bg-stone-50\/45:is(.dark *){background-color:hsla(60,9%,98%,.45)}.dark\:bg-stone-50\/5:is(.dark *){background-color:hsla(60,9%,98%,.05)}.dark\:bg-stone-50\/50:is(.dark *){background-color:hsla(60,9%,98%,.5)}.dark\:bg-stone-50\/55:is(.dark *){background-color:hsla(60,9%,98%,.55)}.dark\:bg-stone-50\/60:is(.dark *){background-color:hsla(60,9%,98%,.6)}.dark\:bg-stone-50\/65:is(.dark *){background-color:hsla(60,9%,98%,.65)}.dark\:bg-stone-50\/70:is(.dark *){background-color:hsla(60,9%,98%,.7)}.dark\:bg-stone-50\/75:is(.dark *){background-color:hsla(60,9%,98%,.75)}.dark\:bg-stone-50\/80:is(.dark *){background-color:hsla(60,9%,98%,.8)}.dark\:bg-stone-50\/85:is(.dark *){background-color:hsla(60,9%,98%,.85)}.dark\:bg-stone-50\/90:is(.dark *){background-color:hsla(60,9%,98%,.9)}.dark\:bg-stone-50\/95:is(.dark *){background-color:hsla(60,9%,98%,.95)}.dark\:bg-stone-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(120 113 108/var(--tw-bg-opacity,1))}.dark\:bg-stone-500\/0:is(.dark *){background-color:hsla(25,5%,45%,0)}.dark\:bg-stone-500\/10:is(.dark *){background-color:hsla(25,5%,45%,.1)}.dark\:bg-stone-500\/100:is(.dark *){background-color:#78716c}.dark\:bg-stone-500\/15:is(.dark *){background-color:hsla(25,5%,45%,.15)}.dark\:bg-stone-500\/20:is(.dark *){background-color:hsla(25,5%,45%,.2)}.dark\:bg-stone-500\/25:is(.dark *){background-color:hsla(25,5%,45%,.25)}.dark\:bg-stone-500\/30:is(.dark *){background-color:hsla(25,5%,45%,.3)}.dark\:bg-stone-500\/35:is(.dark *){background-color:hsla(25,5%,45%,.35)}.dark\:bg-stone-500\/40:is(.dark *){background-color:hsla(25,5%,45%,.4)}.dark\:bg-stone-500\/45:is(.dark *){background-color:hsla(25,5%,45%,.45)}.dark\:bg-stone-500\/5:is(.dark *){background-color:hsla(25,5%,45%,.05)}.dark\:bg-stone-500\/50:is(.dark *){background-color:hsla(25,5%,45%,.5)}.dark\:bg-stone-500\/55:is(.dark *){background-color:hsla(25,5%,45%,.55)}.dark\:bg-stone-500\/60:is(.dark *){background-color:hsla(25,5%,45%,.6)}.dark\:bg-stone-500\/65:is(.dark *){background-color:hsla(25,5%,45%,.65)}.dark\:bg-stone-500\/70:is(.dark *){background-color:hsla(25,5%,45%,.7)}.dark\:bg-stone-500\/75:is(.dark *){background-color:hsla(25,5%,45%,.75)}.dark\:bg-stone-500\/80:is(.dark *){background-color:hsla(25,5%,45%,.8)}.dark\:bg-stone-500\/85:is(.dark *){background-color:hsla(25,5%,45%,.85)}.dark\:bg-stone-500\/90:is(.dark *){background-color:hsla(25,5%,45%,.9)}.dark\:bg-stone-500\/95:is(.dark *){background-color:hsla(25,5%,45%,.95)}.dark\:bg-stone-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(12 10 9/var(--tw-bg-opacity,1))}.dark\:bg-stone-950\/0:is(.dark *){background-color:rgba(12,10,9,0)}.dark\:bg-stone-950\/10:is(.dark *){background-color:rgba(12,10,9,.1)}.dark\:bg-stone-950\/100:is(.dark *){background-color:#0c0a09}.dark\:bg-stone-950\/15:is(.dark *){background-color:rgba(12,10,9,.15)}.dark\:bg-stone-950\/20:is(.dark *){background-color:rgba(12,10,9,.2)}.dark\:bg-stone-950\/25:is(.dark *){background-color:rgba(12,10,9,.25)}.dark\:bg-stone-950\/30:is(.dark *){background-color:rgba(12,10,9,.3)}.dark\:bg-stone-950\/35:is(.dark *){background-color:rgba(12,10,9,.35)}.dark\:bg-stone-950\/40:is(.dark *){background-color:rgba(12,10,9,.4)}.dark\:bg-stone-950\/45:is(.dark *){background-color:rgba(12,10,9,.45)}.dark\:bg-stone-950\/5:is(.dark *){background-color:rgba(12,10,9,.05)}.dark\:bg-stone-950\/50:is(.dark *){background-color:rgba(12,10,9,.5)}.dark\:bg-stone-950\/55:is(.dark *){background-color:rgba(12,10,9,.55)}.dark\:bg-stone-950\/60:is(.dark *){background-color:rgba(12,10,9,.6)}.dark\:bg-stone-950\/65:is(.dark *){background-color:rgba(12,10,9,.65)}.dark\:bg-stone-950\/70:is(.dark *){background-color:rgba(12,10,9,.7)}.dark\:bg-stone-950\/75:is(.dark *){background-color:rgba(12,10,9,.75)}.dark\:bg-stone-950\/80:is(.dark *){background-color:rgba(12,10,9,.8)}.dark\:bg-stone-950\/85:is(.dark *){background-color:rgba(12,10,9,.85)}.dark\:bg-stone-950\/90:is(.dark *){background-color:rgba(12,10,9,.9)}.dark\:bg-stone-950\/95:is(.dark *){background-color:rgba(12,10,9,.95)}.dark\:bg-teal-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(204 251 241/var(--tw-bg-opacity,1))}.dark\:bg-teal-100\/0:is(.dark *){background-color:rgba(204,251,241,0)}.dark\:bg-teal-100\/10:is(.dark *){background-color:rgba(204,251,241,.1)}.dark\:bg-teal-100\/100:is(.dark *){background-color:#ccfbf1}.dark\:bg-teal-100\/15:is(.dark *){background-color:rgba(204,251,241,.15)}.dark\:bg-teal-100\/20:is(.dark *){background-color:rgba(204,251,241,.2)}.dark\:bg-teal-100\/25:is(.dark *){background-color:rgba(204,251,241,.25)}.dark\:bg-teal-100\/30:is(.dark *){background-color:rgba(204,251,241,.3)}.dark\:bg-teal-100\/35:is(.dark *){background-color:rgba(204,251,241,.35)}.dark\:bg-teal-100\/40:is(.dark *){background-color:rgba(204,251,241,.4)}.dark\:bg-teal-100\/45:is(.dark *){background-color:rgba(204,251,241,.45)}.dark\:bg-teal-100\/5:is(.dark *){background-color:rgba(204,251,241,.05)}.dark\:bg-teal-100\/50:is(.dark *){background-color:rgba(204,251,241,.5)}.dark\:bg-teal-100\/55:is(.dark *){background-color:rgba(204,251,241,.55)}.dark\:bg-teal-100\/60:is(.dark *){background-color:rgba(204,251,241,.6)}.dark\:bg-teal-100\/65:is(.dark *){background-color:rgba(204,251,241,.65)}.dark\:bg-teal-100\/70:is(.dark *){background-color:rgba(204,251,241,.7)}.dark\:bg-teal-100\/75:is(.dark *){background-color:rgba(204,251,241,.75)}.dark\:bg-teal-100\/80:is(.dark *){background-color:rgba(204,251,241,.8)}.dark\:bg-teal-100\/85:is(.dark *){background-color:rgba(204,251,241,.85)}.dark\:bg-teal-100\/90:is(.dark *){background-color:rgba(204,251,241,.9)}.dark\:bg-teal-100\/95:is(.dark *){background-color:rgba(204,251,241,.95)}.dark\:bg-teal-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(153 246 228/var(--tw-bg-opacity,1))}.dark\:bg-teal-200\/0:is(.dark *){background-color:rgba(153,246,228,0)}.dark\:bg-teal-200\/10:is(.dark *){background-color:rgba(153,246,228,.1)}.dark\:bg-teal-200\/100:is(.dark *){background-color:#99f6e4}.dark\:bg-teal-200\/15:is(.dark *){background-color:rgba(153,246,228,.15)}.dark\:bg-teal-200\/20:is(.dark *){background-color:rgba(153,246,228,.2)}.dark\:bg-teal-200\/25:is(.dark *){background-color:rgba(153,246,228,.25)}.dark\:bg-teal-200\/30:is(.dark *){background-color:rgba(153,246,228,.3)}.dark\:bg-teal-200\/35:is(.dark *){background-color:rgba(153,246,228,.35)}.dark\:bg-teal-200\/40:is(.dark *){background-color:rgba(153,246,228,.4)}.dark\:bg-teal-200\/45:is(.dark *){background-color:rgba(153,246,228,.45)}.dark\:bg-teal-200\/5:is(.dark *){background-color:rgba(153,246,228,.05)}.dark\:bg-teal-200\/50:is(.dark *){background-color:rgba(153,246,228,.5)}.dark\:bg-teal-200\/55:is(.dark *){background-color:rgba(153,246,228,.55)}.dark\:bg-teal-200\/60:is(.dark *){background-color:rgba(153,246,228,.6)}.dark\:bg-teal-200\/65:is(.dark *){background-color:rgba(153,246,228,.65)}.dark\:bg-teal-200\/70:is(.dark *){background-color:rgba(153,246,228,.7)}.dark\:bg-teal-200\/75:is(.dark *){background-color:rgba(153,246,228,.75)}.dark\:bg-teal-200\/80:is(.dark *){background-color:rgba(153,246,228,.8)}.dark\:bg-teal-200\/85:is(.dark *){background-color:rgba(153,246,228,.85)}.dark\:bg-teal-200\/90:is(.dark *){background-color:rgba(153,246,228,.9)}.dark\:bg-teal-200\/95:is(.dark *){background-color:rgba(153,246,228,.95)}.dark\:bg-teal-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(45 212 191/var(--tw-bg-opacity,1))}.dark\:bg-teal-400\/0:is(.dark *){background-color:rgba(45,212,191,0)}.dark\:bg-teal-400\/10:is(.dark *){background-color:rgba(45,212,191,.1)}.dark\:bg-teal-400\/100:is(.dark *){background-color:#2dd4bf}.dark\:bg-teal-400\/15:is(.dark *){background-color:rgba(45,212,191,.15)}.dark\:bg-teal-400\/20:is(.dark *){background-color:rgba(45,212,191,.2)}.dark\:bg-teal-400\/25:is(.dark *){background-color:rgba(45,212,191,.25)}.dark\:bg-teal-400\/30:is(.dark *){background-color:rgba(45,212,191,.3)}.dark\:bg-teal-400\/35:is(.dark *){background-color:rgba(45,212,191,.35)}.dark\:bg-teal-400\/40:is(.dark *){background-color:rgba(45,212,191,.4)}.dark\:bg-teal-400\/45:is(.dark *){background-color:rgba(45,212,191,.45)}.dark\:bg-teal-400\/5:is(.dark *){background-color:rgba(45,212,191,.05)}.dark\:bg-teal-400\/50:is(.dark *){background-color:rgba(45,212,191,.5)}.dark\:bg-teal-400\/55:is(.dark *){background-color:rgba(45,212,191,.55)}.dark\:bg-teal-400\/60:is(.dark *){background-color:rgba(45,212,191,.6)}.dark\:bg-teal-400\/65:is(.dark *){background-color:rgba(45,212,191,.65)}.dark\:bg-teal-400\/70:is(.dark *){background-color:rgba(45,212,191,.7)}.dark\:bg-teal-400\/75:is(.dark *){background-color:rgba(45,212,191,.75)}.dark\:bg-teal-400\/80:is(.dark *){background-color:rgba(45,212,191,.8)}.dark\:bg-teal-400\/85:is(.dark *){background-color:rgba(45,212,191,.85)}.dark\:bg-teal-400\/90:is(.dark *){background-color:rgba(45,212,191,.9)}.dark\:bg-teal-400\/95:is(.dark *){background-color:rgba(45,212,191,.95)}.dark\:bg-teal-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(240 253 250/var(--tw-bg-opacity,1))}.dark\:bg-teal-50\/0:is(.dark *){background-color:rgba(240,253,250,0)}.dark\:bg-teal-50\/10:is(.dark *){background-color:rgba(240,253,250,.1)}.dark\:bg-teal-50\/100:is(.dark *){background-color:#f0fdfa}.dark\:bg-teal-50\/15:is(.dark *){background-color:rgba(240,253,250,.15)}.dark\:bg-teal-50\/20:is(.dark *){background-color:rgba(240,253,250,.2)}.dark\:bg-teal-50\/25:is(.dark *){background-color:rgba(240,253,250,.25)}.dark\:bg-teal-50\/30:is(.dark *){background-color:rgba(240,253,250,.3)}.dark\:bg-teal-50\/35:is(.dark *){background-color:rgba(240,253,250,.35)}.dark\:bg-teal-50\/40:is(.dark *){background-color:rgba(240,253,250,.4)}.dark\:bg-teal-50\/45:is(.dark *){background-color:rgba(240,253,250,.45)}.dark\:bg-teal-50\/5:is(.dark *){background-color:rgba(240,253,250,.05)}.dark\:bg-teal-50\/50:is(.dark *){background-color:rgba(240,253,250,.5)}.dark\:bg-teal-50\/55:is(.dark *){background-color:rgba(240,253,250,.55)}.dark\:bg-teal-50\/60:is(.dark *){background-color:rgba(240,253,250,.6)}.dark\:bg-teal-50\/65:is(.dark *){background-color:rgba(240,253,250,.65)}.dark\:bg-teal-50\/70:is(.dark *){background-color:rgba(240,253,250,.7)}.dark\:bg-teal-50\/75:is(.dark *){background-color:rgba(240,253,250,.75)}.dark\:bg-teal-50\/80:is(.dark *){background-color:rgba(240,253,250,.8)}.dark\:bg-teal-50\/85:is(.dark *){background-color:rgba(240,253,250,.85)}.dark\:bg-teal-50\/90:is(.dark *){background-color:rgba(240,253,250,.9)}.dark\:bg-teal-50\/95:is(.dark *){background-color:rgba(240,253,250,.95)}.dark\:bg-teal-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(20 184 166/var(--tw-bg-opacity,1))}.dark\:bg-teal-500\/0:is(.dark *){background-color:rgba(20,184,166,0)}.dark\:bg-teal-500\/10:is(.dark *){background-color:rgba(20,184,166,.1)}.dark\:bg-teal-500\/100:is(.dark *){background-color:#14b8a6}.dark\:bg-teal-500\/15:is(.dark *){background-color:rgba(20,184,166,.15)}.dark\:bg-teal-500\/20:is(.dark *){background-color:rgba(20,184,166,.2)}.dark\:bg-teal-500\/25:is(.dark *){background-color:rgba(20,184,166,.25)}.dark\:bg-teal-500\/30:is(.dark *){background-color:rgba(20,184,166,.3)}.dark\:bg-teal-500\/35:is(.dark *){background-color:rgba(20,184,166,.35)}.dark\:bg-teal-500\/40:is(.dark *){background-color:rgba(20,184,166,.4)}.dark\:bg-teal-500\/45:is(.dark *){background-color:rgba(20,184,166,.45)}.dark\:bg-teal-500\/5:is(.dark *){background-color:rgba(20,184,166,.05)}.dark\:bg-teal-500\/50:is(.dark *){background-color:rgba(20,184,166,.5)}.dark\:bg-teal-500\/55:is(.dark *){background-color:rgba(20,184,166,.55)}.dark\:bg-teal-500\/60:is(.dark *){background-color:rgba(20,184,166,.6)}.dark\:bg-teal-500\/65:is(.dark *){background-color:rgba(20,184,166,.65)}.dark\:bg-teal-500\/70:is(.dark *){background-color:rgba(20,184,166,.7)}.dark\:bg-teal-500\/75:is(.dark *){background-color:rgba(20,184,166,.75)}.dark\:bg-teal-500\/80:is(.dark *){background-color:rgba(20,184,166,.8)}.dark\:bg-teal-500\/85:is(.dark *){background-color:rgba(20,184,166,.85)}.dark\:bg-teal-500\/90:is(.dark *){background-color:rgba(20,184,166,.9)}.dark\:bg-teal-500\/95:is(.dark *){background-color:rgba(20,184,166,.95)}.dark\:bg-teal-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(4 47 46/var(--tw-bg-opacity,1))}.dark\:bg-teal-950\/0:is(.dark *){background-color:rgba(4,47,46,0)}.dark\:bg-teal-950\/10:is(.dark *){background-color:rgba(4,47,46,.1)}.dark\:bg-teal-950\/100:is(.dark *){background-color:#042f2e}.dark\:bg-teal-950\/15:is(.dark *){background-color:rgba(4,47,46,.15)}.dark\:bg-teal-950\/20:is(.dark *){background-color:rgba(4,47,46,.2)}.dark\:bg-teal-950\/25:is(.dark *){background-color:rgba(4,47,46,.25)}.dark\:bg-teal-950\/30:is(.dark *){background-color:rgba(4,47,46,.3)}.dark\:bg-teal-950\/35:is(.dark *){background-color:rgba(4,47,46,.35)}.dark\:bg-teal-950\/40:is(.dark *){background-color:rgba(4,47,46,.4)}.dark\:bg-teal-950\/45:is(.dark *){background-color:rgba(4,47,46,.45)}.dark\:bg-teal-950\/5:is(.dark *){background-color:rgba(4,47,46,.05)}.dark\:bg-teal-950\/50:is(.dark *){background-color:rgba(4,47,46,.5)}.dark\:bg-teal-950\/55:is(.dark *){background-color:rgba(4,47,46,.55)}.dark\:bg-teal-950\/60:is(.dark *){background-color:rgba(4,47,46,.6)}.dark\:bg-teal-950\/65:is(.dark *){background-color:rgba(4,47,46,.65)}.dark\:bg-teal-950\/70:is(.dark *){background-color:rgba(4,47,46,.7)}.dark\:bg-teal-950\/75:is(.dark *){background-color:rgba(4,47,46,.75)}.dark\:bg-teal-950\/80:is(.dark *){background-color:rgba(4,47,46,.8)}.dark\:bg-teal-950\/85:is(.dark *){background-color:rgba(4,47,46,.85)}.dark\:bg-teal-950\/90:is(.dark *){background-color:rgba(4,47,46,.9)}.dark\:bg-teal-950\/95:is(.dark *){background-color:rgba(4,47,46,.95)}.dark\:bg-violet-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(237 233 254/var(--tw-bg-opacity,1))}.dark\:bg-violet-100\/0:is(.dark *){background-color:rgba(237,233,254,0)}.dark\:bg-violet-100\/10:is(.dark *){background-color:rgba(237,233,254,.1)}.dark\:bg-violet-100\/100:is(.dark *){background-color:#ede9fe}.dark\:bg-violet-100\/15:is(.dark *){background-color:rgba(237,233,254,.15)}.dark\:bg-violet-100\/20:is(.dark *){background-color:rgba(237,233,254,.2)}.dark\:bg-violet-100\/25:is(.dark *){background-color:rgba(237,233,254,.25)}.dark\:bg-violet-100\/30:is(.dark *){background-color:rgba(237,233,254,.3)}.dark\:bg-violet-100\/35:is(.dark *){background-color:rgba(237,233,254,.35)}.dark\:bg-violet-100\/40:is(.dark *){background-color:rgba(237,233,254,.4)}.dark\:bg-violet-100\/45:is(.dark *){background-color:rgba(237,233,254,.45)}.dark\:bg-violet-100\/5:is(.dark *){background-color:rgba(237,233,254,.05)}.dark\:bg-violet-100\/50:is(.dark *){background-color:rgba(237,233,254,.5)}.dark\:bg-violet-100\/55:is(.dark *){background-color:rgba(237,233,254,.55)}.dark\:bg-violet-100\/60:is(.dark *){background-color:rgba(237,233,254,.6)}.dark\:bg-violet-100\/65:is(.dark *){background-color:rgba(237,233,254,.65)}.dark\:bg-violet-100\/70:is(.dark *){background-color:rgba(237,233,254,.7)}.dark\:bg-violet-100\/75:is(.dark *){background-color:rgba(237,233,254,.75)}.dark\:bg-violet-100\/80:is(.dark *){background-color:rgba(237,233,254,.8)}.dark\:bg-violet-100\/85:is(.dark *){background-color:rgba(237,233,254,.85)}.dark\:bg-violet-100\/90:is(.dark *){background-color:rgba(237,233,254,.9)}.dark\:bg-violet-100\/95:is(.dark *){background-color:rgba(237,233,254,.95)}.dark\:bg-violet-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(221 214 254/var(--tw-bg-opacity,1))}.dark\:bg-violet-200\/0:is(.dark *){background-color:rgba(221,214,254,0)}.dark\:bg-violet-200\/10:is(.dark *){background-color:rgba(221,214,254,.1)}.dark\:bg-violet-200\/100:is(.dark *){background-color:#ddd6fe}.dark\:bg-violet-200\/15:is(.dark *){background-color:rgba(221,214,254,.15)}.dark\:bg-violet-200\/20:is(.dark *){background-color:rgba(221,214,254,.2)}.dark\:bg-violet-200\/25:is(.dark *){background-color:rgba(221,214,254,.25)}.dark\:bg-violet-200\/30:is(.dark *){background-color:rgba(221,214,254,.3)}.dark\:bg-violet-200\/35:is(.dark *){background-color:rgba(221,214,254,.35)}.dark\:bg-violet-200\/40:is(.dark *){background-color:rgba(221,214,254,.4)}.dark\:bg-violet-200\/45:is(.dark *){background-color:rgba(221,214,254,.45)}.dark\:bg-violet-200\/5:is(.dark *){background-color:rgba(221,214,254,.05)}.dark\:bg-violet-200\/50:is(.dark *){background-color:rgba(221,214,254,.5)}.dark\:bg-violet-200\/55:is(.dark *){background-color:rgba(221,214,254,.55)}.dark\:bg-violet-200\/60:is(.dark *){background-color:rgba(221,214,254,.6)}.dark\:bg-violet-200\/65:is(.dark *){background-color:rgba(221,214,254,.65)}.dark\:bg-violet-200\/70:is(.dark *){background-color:rgba(221,214,254,.7)}.dark\:bg-violet-200\/75:is(.dark *){background-color:rgba(221,214,254,.75)}.dark\:bg-violet-200\/80:is(.dark *){background-color:rgba(221,214,254,.8)}.dark\:bg-violet-200\/85:is(.dark *){background-color:rgba(221,214,254,.85)}.dark\:bg-violet-200\/90:is(.dark *){background-color:rgba(221,214,254,.9)}.dark\:bg-violet-200\/95:is(.dark *){background-color:rgba(221,214,254,.95)}.dark\:bg-violet-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(167 139 250/var(--tw-bg-opacity,1))}.dark\:bg-violet-400\/0:is(.dark *){background-color:rgba(167,139,250,0)}.dark\:bg-violet-400\/10:is(.dark *){background-color:rgba(167,139,250,.1)}.dark\:bg-violet-400\/100:is(.dark *){background-color:#a78bfa}.dark\:bg-violet-400\/15:is(.dark *){background-color:rgba(167,139,250,.15)}.dark\:bg-violet-400\/20:is(.dark *){background-color:rgba(167,139,250,.2)}.dark\:bg-violet-400\/25:is(.dark *){background-color:rgba(167,139,250,.25)}.dark\:bg-violet-400\/30:is(.dark *){background-color:rgba(167,139,250,.3)}.dark\:bg-violet-400\/35:is(.dark *){background-color:rgba(167,139,250,.35)}.dark\:bg-violet-400\/40:is(.dark *){background-color:rgba(167,139,250,.4)}.dark\:bg-violet-400\/45:is(.dark *){background-color:rgba(167,139,250,.45)}.dark\:bg-violet-400\/5:is(.dark *){background-color:rgba(167,139,250,.05)}.dark\:bg-violet-400\/50:is(.dark *){background-color:rgba(167,139,250,.5)}.dark\:bg-violet-400\/55:is(.dark *){background-color:rgba(167,139,250,.55)}.dark\:bg-violet-400\/60:is(.dark *){background-color:rgba(167,139,250,.6)}.dark\:bg-violet-400\/65:is(.dark *){background-color:rgba(167,139,250,.65)}.dark\:bg-violet-400\/70:is(.dark *){background-color:rgba(167,139,250,.7)}.dark\:bg-violet-400\/75:is(.dark *){background-color:rgba(167,139,250,.75)}.dark\:bg-violet-400\/80:is(.dark *){background-color:rgba(167,139,250,.8)}.dark\:bg-violet-400\/85:is(.dark *){background-color:rgba(167,139,250,.85)}.dark\:bg-violet-400\/90:is(.dark *){background-color:rgba(167,139,250,.9)}.dark\:bg-violet-400\/95:is(.dark *){background-color:rgba(167,139,250,.95)}.dark\:bg-violet-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(245 243 255/var(--tw-bg-opacity,1))}.dark\:bg-violet-50\/0:is(.dark *){background-color:rgba(245,243,255,0)}.dark\:bg-violet-50\/10:is(.dark *){background-color:rgba(245,243,255,.1)}.dark\:bg-violet-50\/100:is(.dark *){background-color:#f5f3ff}.dark\:bg-violet-50\/15:is(.dark *){background-color:rgba(245,243,255,.15)}.dark\:bg-violet-50\/20:is(.dark *){background-color:rgba(245,243,255,.2)}.dark\:bg-violet-50\/25:is(.dark *){background-color:rgba(245,243,255,.25)}.dark\:bg-violet-50\/30:is(.dark *){background-color:rgba(245,243,255,.3)}.dark\:bg-violet-50\/35:is(.dark *){background-color:rgba(245,243,255,.35)}.dark\:bg-violet-50\/40:is(.dark *){background-color:rgba(245,243,255,.4)}.dark\:bg-violet-50\/45:is(.dark *){background-color:rgba(245,243,255,.45)}.dark\:bg-violet-50\/5:is(.dark *){background-color:rgba(245,243,255,.05)}.dark\:bg-violet-50\/50:is(.dark *){background-color:rgba(245,243,255,.5)}.dark\:bg-violet-50\/55:is(.dark *){background-color:rgba(245,243,255,.55)}.dark\:bg-violet-50\/60:is(.dark *){background-color:rgba(245,243,255,.6)}.dark\:bg-violet-50\/65:is(.dark *){background-color:rgba(245,243,255,.65)}.dark\:bg-violet-50\/70:is(.dark *){background-color:rgba(245,243,255,.7)}.dark\:bg-violet-50\/75:is(.dark *){background-color:rgba(245,243,255,.75)}.dark\:bg-violet-50\/80:is(.dark *){background-color:rgba(245,243,255,.8)}.dark\:bg-violet-50\/85:is(.dark *){background-color:rgba(245,243,255,.85)}.dark\:bg-violet-50\/90:is(.dark *){background-color:rgba(245,243,255,.9)}.dark\:bg-violet-50\/95:is(.dark *){background-color:rgba(245,243,255,.95)}.dark\:bg-violet-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(139 92 246/var(--tw-bg-opacity,1))}.dark\:bg-violet-500\/0:is(.dark *){background-color:rgba(139,92,246,0)}.dark\:bg-violet-500\/10:is(.dark *){background-color:rgba(139,92,246,.1)}.dark\:bg-violet-500\/100:is(.dark *){background-color:#8b5cf6}.dark\:bg-violet-500\/15:is(.dark *){background-color:rgba(139,92,246,.15)}.dark\:bg-violet-500\/20:is(.dark *){background-color:rgba(139,92,246,.2)}.dark\:bg-violet-500\/25:is(.dark *){background-color:rgba(139,92,246,.25)}.dark\:bg-violet-500\/30:is(.dark *){background-color:rgba(139,92,246,.3)}.dark\:bg-violet-500\/35:is(.dark *){background-color:rgba(139,92,246,.35)}.dark\:bg-violet-500\/40:is(.dark *){background-color:rgba(139,92,246,.4)}.dark\:bg-violet-500\/45:is(.dark *){background-color:rgba(139,92,246,.45)}.dark\:bg-violet-500\/5:is(.dark *){background-color:rgba(139,92,246,.05)}.dark\:bg-violet-500\/50:is(.dark *){background-color:rgba(139,92,246,.5)}.dark\:bg-violet-500\/55:is(.dark *){background-color:rgba(139,92,246,.55)}.dark\:bg-violet-500\/60:is(.dark *){background-color:rgba(139,92,246,.6)}.dark\:bg-violet-500\/65:is(.dark *){background-color:rgba(139,92,246,.65)}.dark\:bg-violet-500\/70:is(.dark *){background-color:rgba(139,92,246,.7)}.dark\:bg-violet-500\/75:is(.dark *){background-color:rgba(139,92,246,.75)}.dark\:bg-violet-500\/80:is(.dark *){background-color:rgba(139,92,246,.8)}.dark\:bg-violet-500\/85:is(.dark *){background-color:rgba(139,92,246,.85)}.dark\:bg-violet-500\/90:is(.dark *){background-color:rgba(139,92,246,.9)}.dark\:bg-violet-500\/95:is(.dark *){background-color:rgba(139,92,246,.95)}.dark\:bg-violet-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(46 16 101/var(--tw-bg-opacity,1))}.dark\:bg-violet-950\/0:is(.dark *){background-color:rgba(46,16,101,0)}.dark\:bg-violet-950\/10:is(.dark *){background-color:rgba(46,16,101,.1)}.dark\:bg-violet-950\/100:is(.dark *){background-color:#2e1065}.dark\:bg-violet-950\/15:is(.dark *){background-color:rgba(46,16,101,.15)}.dark\:bg-violet-950\/20:is(.dark *){background-color:rgba(46,16,101,.2)}.dark\:bg-violet-950\/25:is(.dark *){background-color:rgba(46,16,101,.25)}.dark\:bg-violet-950\/30:is(.dark *){background-color:rgba(46,16,101,.3)}.dark\:bg-violet-950\/35:is(.dark *){background-color:rgba(46,16,101,.35)}.dark\:bg-violet-950\/40:is(.dark *){background-color:rgba(46,16,101,.4)}.dark\:bg-violet-950\/45:is(.dark *){background-color:rgba(46,16,101,.45)}.dark\:bg-violet-950\/5:is(.dark *){background-color:rgba(46,16,101,.05)}.dark\:bg-violet-950\/50:is(.dark *){background-color:rgba(46,16,101,.5)}.dark\:bg-violet-950\/55:is(.dark *){background-color:rgba(46,16,101,.55)}.dark\:bg-violet-950\/60:is(.dark *){background-color:rgba(46,16,101,.6)}.dark\:bg-violet-950\/65:is(.dark *){background-color:rgba(46,16,101,.65)}.dark\:bg-violet-950\/70:is(.dark *){background-color:rgba(46,16,101,.7)}.dark\:bg-violet-950\/75:is(.dark *){background-color:rgba(46,16,101,.75)}.dark\:bg-violet-950\/80:is(.dark *){background-color:rgba(46,16,101,.8)}.dark\:bg-violet-950\/85:is(.dark *){background-color:rgba(46,16,101,.85)}.dark\:bg-violet-950\/90:is(.dark *){background-color:rgba(46,16,101,.9)}.dark\:bg-violet-950\/95:is(.dark *){background-color:rgba(46,16,101,.95)}.dark\:bg-white:is(.dark *){--tw-bg-opacity:1;background-color:rgb(255 255 255/var(--tw-bg-opacity,1))}.dark\:bg-yellow-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(254 249 195/var(--tw-bg-opacity,1))}.dark\:bg-yellow-100\/0:is(.dark *){background-color:hsla(55,97%,88%,0)}.dark\:bg-yellow-100\/10:is(.dark *){background-color:hsla(55,97%,88%,.1)}.dark\:bg-yellow-100\/100:is(.dark *){background-color:#fef9c3}.dark\:bg-yellow-100\/15:is(.dark *){background-color:hsla(55,97%,88%,.15)}.dark\:bg-yellow-100\/20:is(.dark *){background-color:hsla(55,97%,88%,.2)}.dark\:bg-yellow-100\/25:is(.dark *){background-color:hsla(55,97%,88%,.25)}.dark\:bg-yellow-100\/30:is(.dark *){background-color:hsla(55,97%,88%,.3)}.dark\:bg-yellow-100\/35:is(.dark *){background-color:hsla(55,97%,88%,.35)}.dark\:bg-yellow-100\/40:is(.dark *){background-color:hsla(55,97%,88%,.4)}.dark\:bg-yellow-100\/45:is(.dark *){background-color:hsla(55,97%,88%,.45)}.dark\:bg-yellow-100\/5:is(.dark *){background-color:hsla(55,97%,88%,.05)}.dark\:bg-yellow-100\/50:is(.dark *){background-color:hsla(55,97%,88%,.5)}.dark\:bg-yellow-100\/55:is(.dark *){background-color:hsla(55,97%,88%,.55)}.dark\:bg-yellow-100\/60:is(.dark *){background-color:hsla(55,97%,88%,.6)}.dark\:bg-yellow-100\/65:is(.dark *){background-color:hsla(55,97%,88%,.65)}.dark\:bg-yellow-100\/70:is(.dark *){background-color:hsla(55,97%,88%,.7)}.dark\:bg-yellow-100\/75:is(.dark *){background-color:hsla(55,97%,88%,.75)}.dark\:bg-yellow-100\/80:is(.dark *){background-color:hsla(55,97%,88%,.8)}.dark\:bg-yellow-100\/85:is(.dark *){background-color:hsla(55,97%,88%,.85)}.dark\:bg-yellow-100\/90:is(.dark *){background-color:hsla(55,97%,88%,.9)}.dark\:bg-yellow-100\/95:is(.dark *){background-color:hsla(55,97%,88%,.95)}.dark\:bg-yellow-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(254 240 138/var(--tw-bg-opacity,1))}.dark\:bg-yellow-200\/0:is(.dark *){background-color:hsla(53,98%,77%,0)}.dark\:bg-yellow-200\/10:is(.dark *){background-color:hsla(53,98%,77%,.1)}.dark\:bg-yellow-200\/100:is(.dark *){background-color:#fef08a}.dark\:bg-yellow-200\/15:is(.dark *){background-color:hsla(53,98%,77%,.15)}.dark\:bg-yellow-200\/20:is(.dark *){background-color:hsla(53,98%,77%,.2)}.dark\:bg-yellow-200\/25:is(.dark *){background-color:hsla(53,98%,77%,.25)}.dark\:bg-yellow-200\/30:is(.dark *){background-color:hsla(53,98%,77%,.3)}.dark\:bg-yellow-200\/35:is(.dark *){background-color:hsla(53,98%,77%,.35)}.dark\:bg-yellow-200\/40:is(.dark *){background-color:hsla(53,98%,77%,.4)}.dark\:bg-yellow-200\/45:is(.dark *){background-color:hsla(53,98%,77%,.45)}.dark\:bg-yellow-200\/5:is(.dark *){background-color:hsla(53,98%,77%,.05)}.dark\:bg-yellow-200\/50:is(.dark *){background-color:hsla(53,98%,77%,.5)}.dark\:bg-yellow-200\/55:is(.dark *){background-color:hsla(53,98%,77%,.55)}.dark\:bg-yellow-200\/60:is(.dark *){background-color:hsla(53,98%,77%,.6)}.dark\:bg-yellow-200\/65:is(.dark *){background-color:hsla(53,98%,77%,.65)}.dark\:bg-yellow-200\/70:is(.dark *){background-color:hsla(53,98%,77%,.7)}.dark\:bg-yellow-200\/75:is(.dark *){background-color:hsla(53,98%,77%,.75)}.dark\:bg-yellow-200\/80:is(.dark *){background-color:hsla(53,98%,77%,.8)}.dark\:bg-yellow-200\/85:is(.dark *){background-color:hsla(53,98%,77%,.85)}.dark\:bg-yellow-200\/90:is(.dark *){background-color:hsla(53,98%,77%,.9)}.dark\:bg-yellow-200\/95:is(.dark *){background-color:hsla(53,98%,77%,.95)}.dark\:bg-yellow-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(250 204 21/var(--tw-bg-opacity,1))}.dark\:bg-yellow-400\/0:is(.dark *){background-color:rgba(250,204,21,0)}.dark\:bg-yellow-400\/10:is(.dark *){background-color:rgba(250,204,21,.1)}.dark\:bg-yellow-400\/100:is(.dark *){background-color:#facc15}.dark\:bg-yellow-400\/15:is(.dark *){background-color:rgba(250,204,21,.15)}.dark\:bg-yellow-400\/20:is(.dark *){background-color:rgba(250,204,21,.2)}.dark\:bg-yellow-400\/25:is(.dark *){background-color:rgba(250,204,21,.25)}.dark\:bg-yellow-400\/30:is(.dark *){background-color:rgba(250,204,21,.3)}.dark\:bg-yellow-400\/35:is(.dark *){background-color:rgba(250,204,21,.35)}.dark\:bg-yellow-400\/40:is(.dark *){background-color:rgba(250,204,21,.4)}.dark\:bg-yellow-400\/45:is(.dark *){background-color:rgba(250,204,21,.45)}.dark\:bg-yellow-400\/5:is(.dark *){background-color:rgba(250,204,21,.05)}.dark\:bg-yellow-400\/50:is(.dark *){background-color:rgba(250,204,21,.5)}.dark\:bg-yellow-400\/55:is(.dark *){background-color:rgba(250,204,21,.55)}.dark\:bg-yellow-400\/60:is(.dark *){background-color:rgba(250,204,21,.6)}.dark\:bg-yellow-400\/65:is(.dark *){background-color:rgba(250,204,21,.65)}.dark\:bg-yellow-400\/70:is(.dark *){background-color:rgba(250,204,21,.7)}.dark\:bg-yellow-400\/75:is(.dark *){background-color:rgba(250,204,21,.75)}.dark\:bg-yellow-400\/80:is(.dark *){background-color:rgba(250,204,21,.8)}.dark\:bg-yellow-400\/85:is(.dark *){background-color:rgba(250,204,21,.85)}.dark\:bg-yellow-400\/90:is(.dark *){background-color:rgba(250,204,21,.9)}.dark\:bg-yellow-400\/95:is(.dark *){background-color:rgba(250,204,21,.95)}.dark\:bg-yellow-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(254 252 232/var(--tw-bg-opacity,1))}.dark\:bg-yellow-50\/0:is(.dark *){background-color:hsla(55,92%,95%,0)}.dark\:bg-yellow-50\/10:is(.dark *){background-color:hsla(55,92%,95%,.1)}.dark\:bg-yellow-50\/100:is(.dark *){background-color:#fefce8}.dark\:bg-yellow-50\/15:is(.dark *){background-color:hsla(55,92%,95%,.15)}.dark\:bg-yellow-50\/20:is(.dark *){background-color:hsla(55,92%,95%,.2)}.dark\:bg-yellow-50\/25:is(.dark *){background-color:hsla(55,92%,95%,.25)}.dark\:bg-yellow-50\/30:is(.dark *){background-color:hsla(55,92%,95%,.3)}.dark\:bg-yellow-50\/35:is(.dark *){background-color:hsla(55,92%,95%,.35)}.dark\:bg-yellow-50\/40:is(.dark *){background-color:hsla(55,92%,95%,.4)}.dark\:bg-yellow-50\/45:is(.dark *){background-color:hsla(55,92%,95%,.45)}.dark\:bg-yellow-50\/5:is(.dark *){background-color:hsla(55,92%,95%,.05)}.dark\:bg-yellow-50\/50:is(.dark *){background-color:hsla(55,92%,95%,.5)}.dark\:bg-yellow-50\/55:is(.dark *){background-color:hsla(55,92%,95%,.55)}.dark\:bg-yellow-50\/60:is(.dark *){background-color:hsla(55,92%,95%,.6)}.dark\:bg-yellow-50\/65:is(.dark *){background-color:hsla(55,92%,95%,.65)}.dark\:bg-yellow-50\/70:is(.dark *){background-color:hsla(55,92%,95%,.7)}.dark\:bg-yellow-50\/75:is(.dark *){background-color:hsla(55,92%,95%,.75)}.dark\:bg-yellow-50\/80:is(.dark *){background-color:hsla(55,92%,95%,.8)}.dark\:bg-yellow-50\/85:is(.dark *){background-color:hsla(55,92%,95%,.85)}.dark\:bg-yellow-50\/90:is(.dark *){background-color:hsla(55,92%,95%,.9)}.dark\:bg-yellow-50\/95:is(.dark *){background-color:hsla(55,92%,95%,.95)}.dark\:bg-yellow-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(234 179 8/var(--tw-bg-opacity,1))}.dark\:bg-yellow-500\/0:is(.dark *){background-color:rgba(234,179,8,0)}.dark\:bg-yellow-500\/10:is(.dark *){background-color:rgba(234,179,8,.1)}.dark\:bg-yellow-500\/100:is(.dark *){background-color:#eab308}.dark\:bg-yellow-500\/15:is(.dark *){background-color:rgba(234,179,8,.15)}.dark\:bg-yellow-500\/20:is(.dark *){background-color:rgba(234,179,8,.2)}.dark\:bg-yellow-500\/25:is(.dark *){background-color:rgba(234,179,8,.25)}.dark\:bg-yellow-500\/30:is(.dark *){background-color:rgba(234,179,8,.3)}.dark\:bg-yellow-500\/35:is(.dark *){background-color:rgba(234,179,8,.35)}.dark\:bg-yellow-500\/40:is(.dark *){background-color:rgba(234,179,8,.4)}.dark\:bg-yellow-500\/45:is(.dark *){background-color:rgba(234,179,8,.45)}.dark\:bg-yellow-500\/5:is(.dark *){background-color:rgba(234,179,8,.05)}.dark\:bg-yellow-500\/50:is(.dark *){background-color:rgba(234,179,8,.5)}.dark\:bg-yellow-500\/55:is(.dark *){background-color:rgba(234,179,8,.55)}.dark\:bg-yellow-500\/60:is(.dark *){background-color:rgba(234,179,8,.6)}.dark\:bg-yellow-500\/65:is(.dark *){background-color:rgba(234,179,8,.65)}.dark\:bg-yellow-500\/70:is(.dark *){background-color:rgba(234,179,8,.7)}.dark\:bg-yellow-500\/75:is(.dark *){background-color:rgba(234,179,8,.75)}.dark\:bg-yellow-500\/80:is(.dark *){background-color:rgba(234,179,8,.8)}.dark\:bg-yellow-500\/85:is(.dark *){background-color:rgba(234,179,8,.85)}.dark\:bg-yellow-500\/90:is(.dark *){background-color:rgba(234,179,8,.9)}.dark\:bg-yellow-500\/95:is(.dark *){background-color:rgba(234,179,8,.95)}.dark\:bg-yellow-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(66 32 6/var(--tw-bg-opacity,1))}.dark\:bg-yellow-950\/0:is(.dark *){background-color:rgba(66,32,6,0)}.dark\:bg-yellow-950\/10:is(.dark *){background-color:rgba(66,32,6,.1)}.dark\:bg-yellow-950\/100:is(.dark *){background-color:#422006}.dark\:bg-yellow-950\/15:is(.dark *){background-color:rgba(66,32,6,.15)}.dark\:bg-yellow-950\/20:is(.dark *){background-color:rgba(66,32,6,.2)}.dark\:bg-yellow-950\/25:is(.dark *){background-color:rgba(66,32,6,.25)}.dark\:bg-yellow-950\/30:is(.dark *){background-color:rgba(66,32,6,.3)}.dark\:bg-yellow-950\/35:is(.dark *){background-color:rgba(66,32,6,.35)}.dark\:bg-yellow-950\/40:is(.dark *){background-color:rgba(66,32,6,.4)}.dark\:bg-yellow-950\/45:is(.dark *){background-color:rgba(66,32,6,.45)}.dark\:bg-yellow-950\/5:is(.dark *){background-color:rgba(66,32,6,.05)}.dark\:bg-yellow-950\/50:is(.dark *){background-color:rgba(66,32,6,.5)}.dark\:bg-yellow-950\/55:is(.dark *){background-color:rgba(66,32,6,.55)}.dark\:bg-yellow-950\/60:is(.dark *){background-color:rgba(66,32,6,.6)}.dark\:bg-yellow-950\/65:is(.dark *){background-color:rgba(66,32,6,.65)}.dark\:bg-yellow-950\/70:is(.dark *){background-color:rgba(66,32,6,.7)}.dark\:bg-yellow-950\/75:is(.dark *){background-color:rgba(66,32,6,.75)}.dark\:bg-yellow-950\/80:is(.dark *){background-color:rgba(66,32,6,.8)}.dark\:bg-yellow-950\/85:is(.dark *){background-color:rgba(66,32,6,.85)}.dark\:bg-yellow-950\/90:is(.dark *){background-color:rgba(66,32,6,.9)}.dark\:bg-yellow-950\/95:is(.dark *){background-color:rgba(66,32,6,.95)}.dark\:bg-zinc-100:is(.dark *){--tw-bg-opacity:1;background-color:rgb(244 244 245/var(--tw-bg-opacity,1))}.dark\:bg-zinc-100\/0:is(.dark *){background-color:hsla(240,5%,96%,0)}.dark\:bg-zinc-100\/10:is(.dark *){background-color:hsla(240,5%,96%,.1)}.dark\:bg-zinc-100\/100:is(.dark *){background-color:#f4f4f5}.dark\:bg-zinc-100\/15:is(.dark *){background-color:hsla(240,5%,96%,.15)}.dark\:bg-zinc-100\/20:is(.dark *){background-color:hsla(240,5%,96%,.2)}.dark\:bg-zinc-100\/25:is(.dark *){background-color:hsla(240,5%,96%,.25)}.dark\:bg-zinc-100\/30:is(.dark *){background-color:hsla(240,5%,96%,.3)}.dark\:bg-zinc-100\/35:is(.dark *){background-color:hsla(240,5%,96%,.35)}.dark\:bg-zinc-100\/40:is(.dark *){background-color:hsla(240,5%,96%,.4)}.dark\:bg-zinc-100\/45:is(.dark *){background-color:hsla(240,5%,96%,.45)}.dark\:bg-zinc-100\/5:is(.dark *){background-color:hsla(240,5%,96%,.05)}.dark\:bg-zinc-100\/50:is(.dark *){background-color:hsla(240,5%,96%,.5)}.dark\:bg-zinc-100\/55:is(.dark *){background-color:hsla(240,5%,96%,.55)}.dark\:bg-zinc-100\/60:is(.dark *){background-color:hsla(240,5%,96%,.6)}.dark\:bg-zinc-100\/65:is(.dark *){background-color:hsla(240,5%,96%,.65)}.dark\:bg-zinc-100\/70:is(.dark *){background-color:hsla(240,5%,96%,.7)}.dark\:bg-zinc-100\/75:is(.dark *){background-color:hsla(240,5%,96%,.75)}.dark\:bg-zinc-100\/80:is(.dark *){background-color:hsla(240,5%,96%,.8)}.dark\:bg-zinc-100\/85:is(.dark *){background-color:hsla(240,5%,96%,.85)}.dark\:bg-zinc-100\/90:is(.dark *){background-color:hsla(240,5%,96%,.9)}.dark\:bg-zinc-100\/95:is(.dark *){background-color:hsla(240,5%,96%,.95)}.dark\:bg-zinc-200:is(.dark *){--tw-bg-opacity:1;background-color:rgb(228 228 231/var(--tw-bg-opacity,1))}.dark\:bg-zinc-200\/0:is(.dark *){background-color:hsla(240,6%,90%,0)}.dark\:bg-zinc-200\/10:is(.dark *){background-color:hsla(240,6%,90%,.1)}.dark\:bg-zinc-200\/100:is(.dark *){background-color:#e4e4e7}.dark\:bg-zinc-200\/15:is(.dark *){background-color:hsla(240,6%,90%,.15)}.dark\:bg-zinc-200\/20:is(.dark *){background-color:hsla(240,6%,90%,.2)}.dark\:bg-zinc-200\/25:is(.dark *){background-color:hsla(240,6%,90%,.25)}.dark\:bg-zinc-200\/30:is(.dark *){background-color:hsla(240,6%,90%,.3)}.dark\:bg-zinc-200\/35:is(.dark *){background-color:hsla(240,6%,90%,.35)}.dark\:bg-zinc-200\/40:is(.dark *){background-color:hsla(240,6%,90%,.4)}.dark\:bg-zinc-200\/45:is(.dark *){background-color:hsla(240,6%,90%,.45)}.dark\:bg-zinc-200\/5:is(.dark *){background-color:hsla(240,6%,90%,.05)}.dark\:bg-zinc-200\/50:is(.dark *){background-color:hsla(240,6%,90%,.5)}.dark\:bg-zinc-200\/55:is(.dark *){background-color:hsla(240,6%,90%,.55)}.dark\:bg-zinc-200\/60:is(.dark *){background-color:hsla(240,6%,90%,.6)}.dark\:bg-zinc-200\/65:is(.dark *){background-color:hsla(240,6%,90%,.65)}.dark\:bg-zinc-200\/70:is(.dark *){background-color:hsla(240,6%,90%,.7)}.dark\:bg-zinc-200\/75:is(.dark *){background-color:hsla(240,6%,90%,.75)}.dark\:bg-zinc-200\/80:is(.dark *){background-color:hsla(240,6%,90%,.8)}.dark\:bg-zinc-200\/85:is(.dark *){background-color:hsla(240,6%,90%,.85)}.dark\:bg-zinc-200\/90:is(.dark *){background-color:hsla(240,6%,90%,.9)}.dark\:bg-zinc-200\/95:is(.dark *){background-color:hsla(240,6%,90%,.95)}.dark\:bg-zinc-400:is(.dark *){--tw-bg-opacity:1;background-color:rgb(161 161 170/var(--tw-bg-opacity,1))}.dark\:bg-zinc-400\/0:is(.dark *){background-color:hsla(240,5%,65%,0)}.dark\:bg-zinc-400\/10:is(.dark *){background-color:hsla(240,5%,65%,.1)}.dark\:bg-zinc-400\/100:is(.dark *){background-color:#a1a1aa}.dark\:bg-zinc-400\/15:is(.dark *){background-color:hsla(240,5%,65%,.15)}.dark\:bg-zinc-400\/20:is(.dark *){background-color:hsla(240,5%,65%,.2)}.dark\:bg-zinc-400\/25:is(.dark *){background-color:hsla(240,5%,65%,.25)}.dark\:bg-zinc-400\/30:is(.dark *){background-color:hsla(240,5%,65%,.3)}.dark\:bg-zinc-400\/35:is(.dark *){background-color:hsla(240,5%,65%,.35)}.dark\:bg-zinc-400\/40:is(.dark *){background-color:hsla(240,5%,65%,.4)}.dark\:bg-zinc-400\/45:is(.dark *){background-color:hsla(240,5%,65%,.45)}.dark\:bg-zinc-400\/5:is(.dark *){background-color:hsla(240,5%,65%,.05)}.dark\:bg-zinc-400\/50:is(.dark *){background-color:hsla(240,5%,65%,.5)}.dark\:bg-zinc-400\/55:is(.dark *){background-color:hsla(240,5%,65%,.55)}.dark\:bg-zinc-400\/60:is(.dark *){background-color:hsla(240,5%,65%,.6)}.dark\:bg-zinc-400\/65:is(.dark *){background-color:hsla(240,5%,65%,.65)}.dark\:bg-zinc-400\/70:is(.dark *){background-color:hsla(240,5%,65%,.7)}.dark\:bg-zinc-400\/75:is(.dark *){background-color:hsla(240,5%,65%,.75)}.dark\:bg-zinc-400\/80:is(.dark *){background-color:hsla(240,5%,65%,.8)}.dark\:bg-zinc-400\/85:is(.dark *){background-color:hsla(240,5%,65%,.85)}.dark\:bg-zinc-400\/90:is(.dark *){background-color:hsla(240,5%,65%,.9)}.dark\:bg-zinc-400\/95:is(.dark *){background-color:hsla(240,5%,65%,.95)}.dark\:bg-zinc-50:is(.dark *){--tw-bg-opacity:1;background-color:rgb(250 250 250/var(--tw-bg-opacity,1))}.dark\:bg-zinc-50\/0:is(.dark *){background-color:hsla(0,0%,98%,0)}.dark\:bg-zinc-50\/10:is(.dark *){background-color:hsla(0,0%,98%,.1)}.dark\:bg-zinc-50\/100:is(.dark *){background-color:#fafafa}.dark\:bg-zinc-50\/15:is(.dark *){background-color:hsla(0,0%,98%,.15)}.dark\:bg-zinc-50\/20:is(.dark *){background-color:hsla(0,0%,98%,.2)}.dark\:bg-zinc-50\/25:is(.dark *){background-color:hsla(0,0%,98%,.25)}.dark\:bg-zinc-50\/30:is(.dark *){background-color:hsla(0,0%,98%,.3)}.dark\:bg-zinc-50\/35:is(.dark *){background-color:hsla(0,0%,98%,.35)}.dark\:bg-zinc-50\/40:is(.dark *){background-color:hsla(0,0%,98%,.4)}.dark\:bg-zinc-50\/45:is(.dark *){background-color:hsla(0,0%,98%,.45)}.dark\:bg-zinc-50\/5:is(.dark *){background-color:hsla(0,0%,98%,.05)}.dark\:bg-zinc-50\/50:is(.dark *){background-color:hsla(0,0%,98%,.5)}.dark\:bg-zinc-50\/55:is(.dark *){background-color:hsla(0,0%,98%,.55)}.dark\:bg-zinc-50\/60:is(.dark *){background-color:hsla(0,0%,98%,.6)}.dark\:bg-zinc-50\/65:is(.dark *){background-color:hsla(0,0%,98%,.65)}.dark\:bg-zinc-50\/70:is(.dark *){background-color:hsla(0,0%,98%,.7)}.dark\:bg-zinc-50\/75:is(.dark *){background-color:hsla(0,0%,98%,.75)}.dark\:bg-zinc-50\/80:is(.dark *){background-color:hsla(0,0%,98%,.8)}.dark\:bg-zinc-50\/85:is(.dark *){background-color:hsla(0,0%,98%,.85)}.dark\:bg-zinc-50\/90:is(.dark *){background-color:hsla(0,0%,98%,.9)}.dark\:bg-zinc-50\/95:is(.dark *){background-color:hsla(0,0%,98%,.95)}.dark\:bg-zinc-500:is(.dark *){--tw-bg-opacity:1;background-color:rgb(113 113 122/var(--tw-bg-opacity,1))}.dark\:bg-zinc-500\/0:is(.dark *){background-color:hsla(240,4%,46%,0)}.dark\:bg-zinc-500\/10:is(.dark *){background-color:hsla(240,4%,46%,.1)}.dark\:bg-zinc-500\/100:is(.dark *){background-color:#71717a}.dark\:bg-zinc-500\/15:is(.dark *){background-color:hsla(240,4%,46%,.15)}.dark\:bg-zinc-500\/20:is(.dark *){background-color:hsla(240,4%,46%,.2)}.dark\:bg-zinc-500\/25:is(.dark *){background-color:hsla(240,4%,46%,.25)}.dark\:bg-zinc-500\/30:is(.dark *){background-color:hsla(240,4%,46%,.3)}.dark\:bg-zinc-500\/35:is(.dark *){background-color:hsla(240,4%,46%,.35)}.dark\:bg-zinc-500\/40:is(.dark *){background-color:hsla(240,4%,46%,.4)}.dark\:bg-zinc-500\/45:is(.dark *){background-color:hsla(240,4%,46%,.45)}.dark\:bg-zinc-500\/5:is(.dark *){background-color:hsla(240,4%,46%,.05)}.dark\:bg-zinc-500\/50:is(.dark *){background-color:hsla(240,4%,46%,.5)}.dark\:bg-zinc-500\/55:is(.dark *){background-color:hsla(240,4%,46%,.55)}.dark\:bg-zinc-500\/60:is(.dark *){background-color:hsla(240,4%,46%,.6)}.dark\:bg-zinc-500\/65:is(.dark *){background-color:hsla(240,4%,46%,.65)}.dark\:bg-zinc-500\/70:is(.dark *){background-color:hsla(240,4%,46%,.7)}.dark\:bg-zinc-500\/75:is(.dark *){background-color:hsla(240,4%,46%,.75)}.dark\:bg-zinc-500\/80:is(.dark *){background-color:hsla(240,4%,46%,.8)}.dark\:bg-zinc-500\/85:is(.dark *){background-color:hsla(240,4%,46%,.85)}.dark\:bg-zinc-500\/90:is(.dark *){background-color:hsla(240,4%,46%,.9)}.dark\:bg-zinc-500\/95:is(.dark *){background-color:hsla(240,4%,46%,.95)}.dark\:bg-zinc-950:is(.dark *){--tw-bg-opacity:1;background-color:rgb(9 9 11/var(--tw-bg-opacity,1))}.dark\:bg-zinc-950\/0:is(.dark *){background-color:rgba(9,9,11,0)}.dark\:bg-zinc-950\/10:is(.dark *){background-color:rgba(9,9,11,.1)}.dark\:bg-zinc-950\/100:is(.dark *){background-color:#09090b}.dark\:bg-zinc-950\/15:is(.dark *){background-color:rgba(9,9,11,.15)}.dark\:bg-zinc-950\/20:is(.dark *){background-color:rgba(9,9,11,.2)}.dark\:bg-zinc-950\/25:is(.dark *){background-color:rgba(9,9,11,.25)}.dark\:bg-zinc-950\/30:is(.dark *){background-color:rgba(9,9,11,.3)}.dark\:bg-zinc-950\/35:is(.dark *){background-color:rgba(9,9,11,.35)}.dark\:bg-zinc-950\/40:is(.dark *){background-color:rgba(9,9,11,.4)}.dark\:bg-zinc-950\/45:is(.dark *){background-color:rgba(9,9,11,.45)}.dark\:bg-zinc-950\/5:is(.dark *){background-color:rgba(9,9,11,.05)}.dark\:bg-zinc-950\/50:is(.dark *){background-color:rgba(9,9,11,.5)}.dark\:bg-zinc-950\/55:is(.dark *){background-color:rgba(9,9,11,.55)}.dark\:bg-zinc-950\/60:is(.dark *){background-color:rgba(9,9,11,.6)}.dark\:bg-zinc-950\/65:is(.dark *){background-color:rgba(9,9,11,.65)}.dark\:bg-zinc-950\/70:is(.dark *){background-color:rgba(9,9,11,.7)}.dark\:bg-zinc-950\/75:is(.dark *){background-color:rgba(9,9,11,.75)}.dark\:bg-zinc-950\/80:is(.dark *){background-color:rgba(9,9,11,.8)}.dark\:bg-zinc-950\/85:is(.dark *){background-color:rgba(9,9,11,.85)}.dark\:bg-zinc-950\/90:is(.dark *){background-color:rgba(9,9,11,.9)}.dark\:bg-zinc-950\/95:is(.dark *){background-color:rgba(9,9,11,.95)}.dark\:from-\[hsl\(var\(--background\)\)\]:is(.dark *){--tw-gradient-from:hsl(var(--background)) var(--tw-gradient-from-position);--tw-gradient-to:hsl(var(--background)/0) var(--tw-gradient-to-position);--tw-gradient-stops:var(--tw-gradient-from),var(--tw-gradient-to)}.dark\:to-amber-100:is(.dark *){--tw-gradient-to:#fef3c7 var(--tw-gradient-to-position)}.dark\:to-amber-100\/0:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,0) var(--tw-gradient-to-position)}.dark\:to-amber-100\/10:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.1) var(--tw-gradient-to-position)}.dark\:to-amber-100\/100:is(.dark *){--tw-gradient-to:#fef3c7 var(--tw-gradient-to-position)}.dark\:to-amber-100\/15:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.15) var(--tw-gradient-to-position)}.dark\:to-amber-100\/20:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.2) var(--tw-gradient-to-position)}.dark\:to-amber-100\/25:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.25) var(--tw-gradient-to-position)}.dark\:to-amber-100\/30:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.3) var(--tw-gradient-to-position)}.dark\:to-amber-100\/35:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.35) var(--tw-gradient-to-position)}.dark\:to-amber-100\/40:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.4) var(--tw-gradient-to-position)}.dark\:to-amber-100\/45:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.45) var(--tw-gradient-to-position)}.dark\:to-amber-100\/5:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.05) var(--tw-gradient-to-position)}.dark\:to-amber-100\/50:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.5) var(--tw-gradient-to-position)}.dark\:to-amber-100\/55:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.55) var(--tw-gradient-to-position)}.dark\:to-amber-100\/60:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.6) var(--tw-gradient-to-position)}.dark\:to-amber-100\/65:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.65) var(--tw-gradient-to-position)}.dark\:to-amber-100\/70:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.7) var(--tw-gradient-to-position)}.dark\:to-amber-100\/75:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.75) var(--tw-gradient-to-position)}.dark\:to-amber-100\/80:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.8) var(--tw-gradient-to-position)}.dark\:to-amber-100\/85:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.85) var(--tw-gradient-to-position)}.dark\:to-amber-100\/90:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.9) var(--tw-gradient-to-position)}.dark\:to-amber-100\/95:is(.dark *){--tw-gradient-to:hsla(48,96%,89%,.95) var(--tw-gradient-to-position)}.dark\:to-amber-200:is(.dark *){--tw-gradient-to:#fde68a var(--tw-gradient-to-position)}.dark\:to-amber-200\/0:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,0) var(--tw-gradient-to-position)}.dark\:to-amber-200\/10:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.1) var(--tw-gradient-to-position)}.dark\:to-amber-200\/100:is(.dark *){--tw-gradient-to:#fde68a var(--tw-gradient-to-position)}.dark\:to-amber-200\/15:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.15) var(--tw-gradient-to-position)}.dark\:to-amber-200\/20:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.2) var(--tw-gradient-to-position)}.dark\:to-amber-200\/25:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.25) var(--tw-gradient-to-position)}.dark\:to-amber-200\/30:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.3) var(--tw-gradient-to-position)}.dark\:to-amber-200\/35:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.35) var(--tw-gradient-to-position)}.dark\:to-amber-200\/40:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.4) var(--tw-gradient-to-position)}.dark\:to-amber-200\/45:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.45) var(--tw-gradient-to-position)}.dark\:to-amber-200\/5:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.05) var(--tw-gradient-to-position)}.dark\:to-amber-200\/50:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.5) var(--tw-gradient-to-position)}.dark\:to-amber-200\/55:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.55) var(--tw-gradient-to-position)}.dark\:to-amber-200\/60:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.6) var(--tw-gradient-to-position)}.dark\:to-amber-200\/65:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.65) var(--tw-gradient-to-position)}.dark\:to-amber-200\/70:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.7) var(--tw-gradient-to-position)}.dark\:to-amber-200\/75:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.75) var(--tw-gradient-to-position)}.dark\:to-amber-200\/80:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.8) var(--tw-gradient-to-position)}.dark\:to-amber-200\/85:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.85) var(--tw-gradient-to-position)}.dark\:to-amber-200\/90:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.9) var(--tw-gradient-to-position)}.dark\:to-amber-200\/95:is(.dark *){--tw-gradient-to:hsla(48,97%,77%,.95) var(--tw-gradient-to-position)}.dark\:to-amber-50:is(.dark *){--tw-gradient-to:#fffbeb var(--tw-gradient-to-position)}.dark\:to-amber-50\/0:is(.dark *){--tw-gradient-to:rgba(255,251,235,0) var(--tw-gradient-to-position)}.dark\:to-amber-50\/10:is(.dark *){--tw-gradient-to:rgba(255,251,235,.1) var(--tw-gradient-to-position)}.dark\:to-amber-50\/100:is(.dark *){--tw-gradient-to:#fffbeb var(--tw-gradient-to-position)}.dark\:to-amber-50\/15:is(.dark *){--tw-gradient-to:rgba(255,251,235,.15) var(--tw-gradient-to-position)}.dark\:to-amber-50\/20:is(.dark *){--tw-gradient-to:rgba(255,251,235,.2) var(--tw-gradient-to-position)}.dark\:to-amber-50\/25:is(.dark *){--tw-gradient-to:rgba(255,251,235,.25) var(--tw-gradient-to-position)}.dark\:to-amber-50\/30:is(.dark *){--tw-gradient-to:rgba(255,251,235,.3) var(--tw-gradient-to-position)}.dark\:to-amber-50\/35:is(.dark *){--tw-gradient-to:rgba(255,251,235,.35) var(--tw-gradient-to-position)}.dark\:to-amber-50\/40:is(.dark *){--tw-gradient-to:rgba(255,251,235,.4) var(--tw-gradient-to-position)}.dark\:to-amber-50\/45:is(.dark *){--tw-gradient-to:rgba(255,251,235,.45) var(--tw-gradient-to-position)}.dark\:to-amber-50\/5:is(.dark *){--tw-gradient-to:rgba(255,251,235,.05) var(--tw-gradient-to-position)}.dark\:to-amber-50\/50:is(.dark *){--tw-gradient-to:rgba(255,251,235,.5) var(--tw-gradient-to-position)}.dark\:to-amber-50\/55:is(.dark *){--tw-gradient-to:rgba(255,251,235,.55) var(--tw-gradient-to-position)}.dark\:to-amber-50\/60:is(.dark *){--tw-gradient-to:rgba(255,251,235,.6) var(--tw-gradient-to-position)}.dark\:to-amber-50\/65:is(.dark *){--tw-gradient-to:rgba(255,251,235,.65) var(--tw-gradient-to-position)}.dark\:to-amber-50\/70:is(.dark *){--tw-gradient-to:rgba(255,251,235,.7) var(--tw-gradient-to-position)}.dark\:to-amber-50\/75:is(.dark *){--tw-gradient-to:rgba(255,251,235,.75) var(--tw-gradient-to-position)}.dark\:to-amber-50\/80:is(.dark *){--tw-gradient-to:rgba(255,251,235,.8) var(--tw-gradient-to-position)}.dark\:to-amber-50\/85:is(.dark *){--tw-gradient-to:rgba(255,251,235,.85) var(--tw-gradient-to-position)}.dark\:to-amber-50\/90:is(.dark *){--tw-gradient-to:rgba(255,251,235,.9) var(--tw-gradient-to-position)}.dark\:to-amber-50\/95:is(.dark *){--tw-gradient-to:rgba(255,251,235,.95) var(--tw-gradient-to-position)}.dark\:to-amber-500:is(.dark *){--tw-gradient-to:#f59e0b var(--tw-gradient-to-position)}.dark\:to-amber-500\/0:is(.dark *){--tw-gradient-to:rgba(245,158,11,0) var(--tw-gradient-to-position)}.dark\:to-amber-500\/10:is(.dark *){--tw-gradient-to:rgba(245,158,11,.1) var(--tw-gradient-to-position)}.dark\:to-amber-500\/100:is(.dark *){--tw-gradient-to:#f59e0b var(--tw-gradient-to-position)}.dark\:to-amber-500\/15:is(.dark *){--tw-gradient-to:rgba(245,158,11,.15) var(--tw-gradient-to-position)}.dark\:to-amber-500\/20:is(.dark *){--tw-gradient-to:rgba(245,158,11,.2) var(--tw-gradient-to-position)}.dark\:to-amber-500\/25:is(.dark *){--tw-gradient-to:rgba(245,158,11,.25) var(--tw-gradient-to-position)}.dark\:to-amber-500\/30:is(.dark *){--tw-gradient-to:rgba(245,158,11,.3) var(--tw-gradient-to-position)}.dark\:to-amber-500\/35:is(.dark *){--tw-gradient-to:rgba(245,158,11,.35) var(--tw-gradient-to-position)}.dark\:to-amber-500\/40:is(.dark *){--tw-gradient-to:rgba(245,158,11,.4) var(--tw-gradient-to-position)}.dark\:to-amber-500\/45:is(.dark *){--tw-gradient-to:rgba(245,158,11,.45) var(--tw-gradient-to-position)}.dark\:to-amber-500\/5:is(.dark *){--tw-gradient-to:rgba(245,158,11,.05) var(--tw-gradient-to-position)}.dark\:to-amber-500\/50:is(.dark *){--tw-gradient-to:rgba(245,158,11,.5) var(--tw-gradient-to-position)}.dark\:to-amber-500\/55:is(.dark *){--tw-gradient-to:rgba(245,158,11,.55) var(--tw-gradient-to-position)}.dark\:to-amber-500\/60:is(.dark *){--tw-gradient-to:rgba(245,158,11,.6) var(--tw-gradient-to-position)}.dark\:to-amber-500\/65:is(.dark *){--tw-gradient-to:rgba(245,158,11,.65) var(--tw-gradient-to-position)}.dark\:to-amber-500\/70:is(.dark *){--tw-gradient-to:rgba(245,158,11,.7) var(--tw-gradient-to-position)}.dark\:to-amber-500\/75:is(.dark *){--tw-gradient-to:rgba(245,158,11,.75) var(--tw-gradient-to-position)}.dark\:to-amber-500\/80:is(.dark *){--tw-gradient-to:rgba(245,158,11,.8) var(--tw-gradient-to-position)}.dark\:to-amber-500\/85:is(.dark *){--tw-gradient-to:rgba(245,158,11,.85) var(--tw-gradient-to-position)}.dark\:to-amber-500\/90:is(.dark *){--tw-gradient-to:rgba(245,158,11,.9) var(--tw-gradient-to-position)}.dark\:to-amber-500\/95:is(.dark *){--tw-gradient-to:rgba(245,158,11,.95) var(--tw-gradient-to-position)}.dark\:to-amber-950:is(.dark *){--tw-gradient-to:#451a03 var(--tw-gradient-to-position)}.dark\:to-amber-950\/0:is(.dark *){--tw-gradient-to:rgba(69,26,3,0) var(--tw-gradient-to-position)}.dark\:to-amber-950\/10:is(.dark *){--tw-gradient-to:rgba(69,26,3,.1) var(--tw-gradient-to-position)}.dark\:to-amber-950\/100:is(.dark *){--tw-gradient-to:#451a03 var(--tw-gradient-to-position)}.dark\:to-amber-950\/15:is(.dark *){--tw-gradient-to:rgba(69,26,3,.15) var(--tw-gradient-to-position)}.dark\:to-amber-950\/20:is(.dark *){--tw-gradient-to:rgba(69,26,3,.2) var(--tw-gradient-to-position)}.dark\:to-amber-950\/25:is(.dark *){--tw-gradient-to:rgba(69,26,3,.25) var(--tw-gradient-to-position)}.dark\:to-amber-950\/30:is(.dark *){--tw-gradient-to:rgba(69,26,3,.3) var(--tw-gradient-to-position)}.dark\:to-amber-950\/35:is(.dark *){--tw-gradient-to:rgba(69,26,3,.35) var(--tw-gradient-to-position)}.dark\:to-amber-950\/40:is(.dark *){--tw-gradient-to:rgba(69,26,3,.4) var(--tw-gradient-to-position)}.dark\:to-amber-950\/45:is(.dark *){--tw-gradient-to:rgba(69,26,3,.45) var(--tw-gradient-to-position)}.dark\:to-amber-950\/5:is(.dark *){--tw-gradient-to:rgba(69,26,3,.05) var(--tw-gradient-to-position)}.dark\:to-amber-950\/50:is(.dark *){--tw-gradient-to:rgba(69,26,3,.5) var(--tw-gradient-to-position)}.dark\:to-amber-950\/55:is(.dark *){--tw-gradient-to:rgba(69,26,3,.55) var(--tw-gradient-to-position)}.dark\:to-amber-950\/60:is(.dark *){--tw-gradient-to:rgba(69,26,3,.6) var(--tw-gradient-to-position)}.dark\:to-amber-950\/65:is(.dark *){--tw-gradient-to:rgba(69,26,3,.65) var(--tw-gradient-to-position)}.dark\:to-amber-950\/70:is(.dark *){--tw-gradient-to:rgba(69,26,3,.7) var(--tw-gradient-to-position)}.dark\:to-amber-950\/75:is(.dark *){--tw-gradient-to:rgba(69,26,3,.75) var(--tw-gradient-to-position)}.dark\:to-amber-950\/80:is(.dark *){--tw-gradient-to:rgba(69,26,3,.8) var(--tw-gradient-to-position)}.dark\:to-amber-950\/85:is(.dark *){--tw-gradient-to:rgba(69,26,3,.85) var(--tw-gradient-to-position)}.dark\:to-amber-950\/90:is(.dark *){--tw-gradient-to:rgba(69,26,3,.9) var(--tw-gradient-to-position)}.dark\:to-amber-950\/95:is(.dark *){--tw-gradient-to:rgba(69,26,3,.95) var(--tw-gradient-to-position)}.dark\:to-blue-100:is(.dark *){--tw-gradient-to:#dbeafe var(--tw-gradient-to-position)}.dark\:to-blue-100\/0:is(.dark *){--tw-gradient-to:rgba(219,234,254,0) var(--tw-gradient-to-position)}.dark\:to-blue-100\/10:is(.dark *){--tw-gradient-to:rgba(219,234,254,.1) var(--tw-gradient-to-position)}.dark\:to-blue-100\/100:is(.dark *){--tw-gradient-to:#dbeafe var(--tw-gradient-to-position)}.dark\:to-blue-100\/15:is(.dark *){--tw-gradient-to:rgba(219,234,254,.15) var(--tw-gradient-to-position)}.dark\:to-blue-100\/20:is(.dark *){--tw-gradient-to:rgba(219,234,254,.2) var(--tw-gradient-to-position)}.dark\:to-blue-100\/25:is(.dark *){--tw-gradient-to:rgba(219,234,254,.25) var(--tw-gradient-to-position)}.dark\:to-blue-100\/30:is(.dark *){--tw-gradient-to:rgba(219,234,254,.3) var(--tw-gradient-to-position)}.dark\:to-blue-100\/35:is(.dark *){--tw-gradient-to:rgba(219,234,254,.35) var(--tw-gradient-to-position)}.dark\:to-blue-100\/40:is(.dark *){--tw-gradient-to:rgba(219,234,254,.4) var(--tw-gradient-to-position)}.dark\:to-blue-100\/45:is(.dark *){--tw-gradient-to:rgba(219,234,254,.45) var(--tw-gradient-to-position)}.dark\:to-blue-100\/5:is(.dark *){--tw-gradient-to:rgba(219,234,254,.05) var(--tw-gradient-to-position)}.dark\:to-blue-100\/50:is(.dark *){--tw-gradient-to:rgba(219,234,254,.5) var(--tw-gradient-to-position)}.dark\:to-blue-100\/55:is(.dark *){--tw-gradient-to:rgba(219,234,254,.55) var(--tw-gradient-to-position)}.dark\:to-blue-100\/60:is(.dark *){--tw-gradient-to:rgba(219,234,254,.6) var(--tw-gradient-to-position)}.dark\:to-blue-100\/65:is(.dark *){--tw-gradient-to:rgba(219,234,254,.65) var(--tw-gradient-to-position)}.dark\:to-blue-100\/70:is(.dark *){--tw-gradient-to:rgba(219,234,254,.7) var(--tw-gradient-to-position)}.dark\:to-blue-100\/75:is(.dark *){--tw-gradient-to:rgba(219,234,254,.75) var(--tw-gradient-to-position)}.dark\:to-blue-100\/80:is(.dark *){--tw-gradient-to:rgba(219,234,254,.8) var(--tw-gradient-to-position)}.dark\:to-blue-100\/85:is(.dark *){--tw-gradient-to:rgba(219,234,254,.85) var(--tw-gradient-to-position)}.dark\:to-blue-100\/90:is(.dark *){--tw-gradient-to:rgba(219,234,254,.9) var(--tw-gradient-to-position)}.dark\:to-blue-100\/95:is(.dark *){--tw-gradient-to:rgba(219,234,254,.95) var(--tw-gradient-to-position)}.dark\:to-blue-200:is(.dark *){--tw-gradient-to:#bfdbfe var(--tw-gradient-to-position)}.dark\:to-blue-200\/0:is(.dark *){--tw-gradient-to:rgba(191,219,254,0) var(--tw-gradient-to-position)}.dark\:to-blue-200\/10:is(.dark *){--tw-gradient-to:rgba(191,219,254,.1) var(--tw-gradient-to-position)}.dark\:to-blue-200\/100:is(.dark *){--tw-gradient-to:#bfdbfe var(--tw-gradient-to-position)}.dark\:to-blue-200\/15:is(.dark *){--tw-gradient-to:rgba(191,219,254,.15) var(--tw-gradient-to-position)}.dark\:to-blue-200\/20:is(.dark *){--tw-gradient-to:rgba(191,219,254,.2) var(--tw-gradient-to-position)}.dark\:to-blue-200\/25:is(.dark *){--tw-gradient-to:rgba(191,219,254,.25) var(--tw-gradient-to-position)}.dark\:to-blue-200\/30:is(.dark *){--tw-gradient-to:rgba(191,219,254,.3) var(--tw-gradient-to-position)}.dark\:to-blue-200\/35:is(.dark *){--tw-gradient-to:rgba(191,219,254,.35) var(--tw-gradient-to-position)}.dark\:to-blue-200\/40:is(.dark *){--tw-gradient-to:rgba(191,219,254,.4) var(--tw-gradient-to-position)}.dark\:to-blue-200\/45:is(.dark *){--tw-gradient-to:rgba(191,219,254,.45) var(--tw-gradient-to-position)}.dark\:to-blue-200\/5:is(.dark *){--tw-gradient-to:rgba(191,219,254,.05) var(--tw-gradient-to-position)}.dark\:to-blue-200\/50:is(.dark *){--tw-gradient-to:rgba(191,219,254,.5) var(--tw-gradient-to-position)}.dark\:to-blue-200\/55:is(.dark *){--tw-gradient-to:rgba(191,219,254,.55) var(--tw-gradient-to-position)}.dark\:to-blue-200\/60:is(.dark *){--tw-gradient-to:rgba(191,219,254,.6) var(--tw-gradient-to-position)}.dark\:to-blue-200\/65:is(.dark *){--tw-gradient-to:rgba(191,219,254,.65) var(--tw-gradient-to-position)}.dark\:to-blue-200\/70:is(.dark *){--tw-gradient-to:rgba(191,219,254,.7) var(--tw-gradient-to-position)}.dark\:to-blue-200\/75:is(.dark *){--tw-gradient-to:rgba(191,219,254,.75) var(--tw-gradient-to-position)}.dark\:to-blue-200\/80:is(.dark *){--tw-gradient-to:rgba(191,219,254,.8) var(--tw-gradient-to-position)}.dark\:to-blue-200\/85:is(.dark *){--tw-gradient-to:rgba(191,219,254,.85) var(--tw-gradient-to-position)}.dark\:to-blue-200\/90:is(.dark *){--tw-gradient-to:rgba(191,219,254,.9) var(--tw-gradient-to-position)}.dark\:to-blue-200\/95:is(.dark *){--tw-gradient-to:rgba(191,219,254,.95) var(--tw-gradient-to-position)}.dark\:to-blue-50:is(.dark *){--tw-gradient-to:#eff6ff var(--tw-gradient-to-position)}.dark\:to-blue-50\/0:is(.dark *){--tw-gradient-to:rgba(239,246,255,0) var(--tw-gradient-to-position)}.dark\:to-blue-50\/10:is(.dark *){--tw-gradient-to:rgba(239,246,255,.1) var(--tw-gradient-to-position)}.dark\:to-blue-50\/100:is(.dark *){--tw-gradient-to:#eff6ff var(--tw-gradient-to-position)}.dark\:to-blue-50\/15:is(.dark *){--tw-gradient-to:rgba(239,246,255,.15) var(--tw-gradient-to-position)}.dark\:to-blue-50\/20:is(.dark *){--tw-gradient-to:rgba(239,246,255,.2) var(--tw-gradient-to-position)}.dark\:to-blue-50\/25:is(.dark *){--tw-gradient-to:rgba(239,246,255,.25) var(--tw-gradient-to-position)}.dark\:to-blue-50\/30:is(.dark *){--tw-gradient-to:rgba(239,246,255,.3) var(--tw-gradient-to-position)}.dark\:to-blue-50\/35:is(.dark *){--tw-gradient-to:rgba(239,246,255,.35) var(--tw-gradient-to-position)}.dark\:to-blue-50\/40:is(.dark *){--tw-gradient-to:rgba(239,246,255,.4) var(--tw-gradient-to-position)}.dark\:to-blue-50\/45:is(.dark *){--tw-gradient-to:rgba(239,246,255,.45) var(--tw-gradient-to-position)}.dark\:to-blue-50\/5:is(.dark *){--tw-gradient-to:rgba(239,246,255,.05) var(--tw-gradient-to-position)}.dark\:to-blue-50\/50:is(.dark *){--tw-gradient-to:rgba(239,246,255,.5) var(--tw-gradient-to-position)}.dark\:to-blue-50\/55:is(.dark *){--tw-gradient-to:rgba(239,246,255,.55) var(--tw-gradient-to-position)}.dark\:to-blue-50\/60:is(.dark *){--tw-gradient-to:rgba(239,246,255,.6) var(--tw-gradient-to-position)}.dark\:to-blue-50\/65:is(.dark *){--tw-gradient-to:rgba(239,246,255,.65) var(--tw-gradient-to-position)}.dark\:to-blue-50\/70:is(.dark *){--tw-gradient-to:rgba(239,246,255,.7) var(--tw-gradient-to-position)}.dark\:to-blue-50\/75:is(.dark *){--tw-gradient-to:rgba(239,246,255,.75) var(--tw-gradient-to-position)}.dark\:to-blue-50\/80:is(.dark *){--tw-gradient-to:rgba(239,246,255,.8) var(--tw-gradient-to-position)}.dark\:to-blue-50\/85:is(.dark *){--tw-gradient-to:rgba(239,246,255,.85) var(--tw-gradient-to-position)}.dark\:to-blue-50\/90:is(.dark *){--tw-gradient-to:rgba(239,246,255,.9) var(--tw-gradient-to-position)}.dark\:to-blue-50\/95:is(.dark *){--tw-gradient-to:rgba(239,246,255,.95) var(--tw-gradient-to-position)}.dark\:to-blue-500:is(.dark *){--tw-gradient-to:#3b82f6 var(--tw-gradient-to-position)}.dark\:to-blue-500\/0:is(.dark *){--tw-gradient-to:rgba(59,130,246,0) var(--tw-gradient-to-position)}.dark\:to-blue-500\/10:is(.dark *){--tw-gradient-to:rgba(59,130,246,.1) var(--tw-gradient-to-position)}.dark\:to-blue-500\/100:is(.dark *){--tw-gradient-to:#3b82f6 var(--tw-gradient-to-position)}.dark\:to-blue-500\/15:is(.dark *){--tw-gradient-to:rgba(59,130,246,.15) var(--tw-gradient-to-position)}.dark\:to-blue-500\/20:is(.dark *){--tw-gradient-to:rgba(59,130,246,.2) var(--tw-gradient-to-position)}.dark\:to-blue-500\/25:is(.dark *){--tw-gradient-to:rgba(59,130,246,.25) var(--tw-gradient-to-position)}.dark\:to-blue-500\/30:is(.dark *){--tw-gradient-to:rgba(59,130,246,.3) var(--tw-gradient-to-position)}.dark\:to-blue-500\/35:is(.dark *){--tw-gradient-to:rgba(59,130,246,.35) var(--tw-gradient-to-position)}.dark\:to-blue-500\/40:is(.dark *){--tw-gradient-to:rgba(59,130,246,.4) var(--tw-gradient-to-position)}.dark\:to-blue-500\/45:is(.dark *){--tw-gradient-to:rgba(59,130,246,.45) var(--tw-gradient-to-position)}.dark\:to-blue-500\/5:is(.dark *){--tw-gradient-to:rgba(59,130,246,.05) var(--tw-gradient-to-position)}.dark\:to-blue-500\/50:is(.dark *){--tw-gradient-to:rgba(59,130,246,.5) var(--tw-gradient-to-position)}.dark\:to-blue-500\/55:is(.dark *){--tw-gradient-to:rgba(59,130,246,.55) var(--tw-gradient-to-position)}.dark\:to-blue-500\/60:is(.dark *){--tw-gradient-to:rgba(59,130,246,.6) var(--tw-gradient-to-position)}.dark\:to-blue-500\/65:is(.dark *){--tw-gradient-to:rgba(59,130,246,.65) var(--tw-gradient-to-position)}.dark\:to-blue-500\/70:is(.dark *){--tw-gradient-to:rgba(59,130,246,.7) var(--tw-gradient-to-position)}.dark\:to-blue-500\/75:is(.dark *){--tw-gradient-to:rgba(59,130,246,.75) var(--tw-gradient-to-position)}.dark\:to-blue-500\/80:is(.dark *){--tw-gradient-to:rgba(59,130,246,.8) var(--tw-gradient-to-position)}.dark\:to-blue-500\/85:is(.dark *){--tw-gradient-to:rgba(59,130,246,.85) var(--tw-gradient-to-position)}.dark\:to-blue-500\/90:is(.dark *){--tw-gradient-to:rgba(59,130,246,.9) var(--tw-gradient-to-position)}.dark\:to-blue-500\/95:is(.dark *){--tw-gradient-to:rgba(59,130,246,.95) var(--tw-gradient-to-position)}.dark\:to-blue-950:is(.dark *){--tw-gradient-to:#172554 var(--tw-gradient-to-position)}.dark\:to-blue-950\/0:is(.dark *){--tw-gradient-to:rgba(23,37,84,0) var(--tw-gradient-to-position)}.dark\:to-blue-950\/10:is(.dark *){--tw-gradient-to:rgba(23,37,84,.1) var(--tw-gradient-to-position)}.dark\:to-blue-950\/100:is(.dark *){--tw-gradient-to:#172554 var(--tw-gradient-to-position)}.dark\:to-blue-950\/15:is(.dark *){--tw-gradient-to:rgba(23,37,84,.15) var(--tw-gradient-to-position)}.dark\:to-blue-950\/20:is(.dark *){--tw-gradient-to:rgba(23,37,84,.2) var(--tw-gradient-to-position)}.dark\:to-blue-950\/25:is(.dark *){--tw-gradient-to:rgba(23,37,84,.25) var(--tw-gradient-to-position)}.dark\:to-blue-950\/30:is(.dark *){--tw-gradient-to:rgba(23,37,84,.3) var(--tw-gradient-to-position)}.dark\:to-blue-950\/35:is(.dark *){--tw-gradient-to:rgba(23,37,84,.35) var(--tw-gradient-to-position)}.dark\:to-blue-950\/40:is(.dark *){--tw-gradient-to:rgba(23,37,84,.4) var(--tw-gradient-to-position)}.dark\:to-blue-950\/45:is(.dark *){--tw-gradient-to:rgba(23,37,84,.45) var(--tw-gradient-to-position)}.dark\:to-blue-950\/5:is(.dark *){--tw-gradient-to:rgba(23,37,84,.05) var(--tw-gradient-to-position)}.dark\:to-blue-950\/50:is(.dark *){--tw-gradient-to:rgba(23,37,84,.5) var(--tw-gradient-to-position)}.dark\:to-blue-950\/55:is(.dark *){--tw-gradient-to:rgba(23,37,84,.55) var(--tw-gradient-to-position)}.dark\:to-blue-950\/60:is(.dark *){--tw-gradient-to:rgba(23,37,84,.6) var(--tw-gradient-to-position)}.dark\:to-blue-950\/65:is(.dark *){--tw-gradient-to:rgba(23,37,84,.65) var(--tw-gradient-to-position)}.dark\:to-blue-950\/70:is(.dark *){--tw-gradient-to:rgba(23,37,84,.7) var(--tw-gradient-to-position)}.dark\:to-blue-950\/75:is(.dark *){--tw-gradient-to:rgba(23,37,84,.75) var(--tw-gradient-to-position)}.dark\:to-blue-950\/80:is(.dark *){--tw-gradient-to:rgba(23,37,84,.8) var(--tw-gradient-to-position)}.dark\:to-blue-950\/85:is(.dark *){--tw-gradient-to:rgba(23,37,84,.85) var(--tw-gradient-to-position)}.dark\:to-blue-950\/90:is(.dark *){--tw-gradient-to:rgba(23,37,84,.9) var(--tw-gradient-to-position)}.dark\:to-blue-950\/95:is(.dark *){--tw-gradient-to:rgba(23,37,84,.95) var(--tw-gradient-to-position)}.dark\:to-cyan-100:is(.dark *){--tw-gradient-to:#cffafe var(--tw-gradient-to-position)}.dark\:to-cyan-100\/0:is(.dark *){--tw-gradient-to:rgba(207,250,254,0) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/10:is(.dark *){--tw-gradient-to:rgba(207,250,254,.1) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/100:is(.dark *){--tw-gradient-to:#cffafe var(--tw-gradient-to-position)}.dark\:to-cyan-100\/15:is(.dark *){--tw-gradient-to:rgba(207,250,254,.15) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/20:is(.dark *){--tw-gradient-to:rgba(207,250,254,.2) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/25:is(.dark *){--tw-gradient-to:rgba(207,250,254,.25) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/30:is(.dark *){--tw-gradient-to:rgba(207,250,254,.3) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/35:is(.dark *){--tw-gradient-to:rgba(207,250,254,.35) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/40:is(.dark *){--tw-gradient-to:rgba(207,250,254,.4) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/45:is(.dark *){--tw-gradient-to:rgba(207,250,254,.45) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/5:is(.dark *){--tw-gradient-to:rgba(207,250,254,.05) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/50:is(.dark *){--tw-gradient-to:rgba(207,250,254,.5) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/55:is(.dark *){--tw-gradient-to:rgba(207,250,254,.55) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/60:is(.dark *){--tw-gradient-to:rgba(207,250,254,.6) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/65:is(.dark *){--tw-gradient-to:rgba(207,250,254,.65) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/70:is(.dark *){--tw-gradient-to:rgba(207,250,254,.7) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/75:is(.dark *){--tw-gradient-to:rgba(207,250,254,.75) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/80:is(.dark *){--tw-gradient-to:rgba(207,250,254,.8) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/85:is(.dark *){--tw-gradient-to:rgba(207,250,254,.85) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/90:is(.dark *){--tw-gradient-to:rgba(207,250,254,.9) var(--tw-gradient-to-position)}.dark\:to-cyan-100\/95:is(.dark *){--tw-gradient-to:rgba(207,250,254,.95) var(--tw-gradient-to-position)}.dark\:to-cyan-200:is(.dark *){--tw-gradient-to:#a5f3fc var(--tw-gradient-to-position)}.dark\:to-cyan-200\/0:is(.dark *){--tw-gradient-to:rgba(165,243,252,0) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/10:is(.dark *){--tw-gradient-to:rgba(165,243,252,.1) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/100:is(.dark *){--tw-gradient-to:#a5f3fc var(--tw-gradient-to-position)}.dark\:to-cyan-200\/15:is(.dark *){--tw-gradient-to:rgba(165,243,252,.15) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/20:is(.dark *){--tw-gradient-to:rgba(165,243,252,.2) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/25:is(.dark *){--tw-gradient-to:rgba(165,243,252,.25) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/30:is(.dark *){--tw-gradient-to:rgba(165,243,252,.3) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/35:is(.dark *){--tw-gradient-to:rgba(165,243,252,.35) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/40:is(.dark *){--tw-gradient-to:rgba(165,243,252,.4) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/45:is(.dark *){--tw-gradient-to:rgba(165,243,252,.45) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/5:is(.dark *){--tw-gradient-to:rgba(165,243,252,.05) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/50:is(.dark *){--tw-gradient-to:rgba(165,243,252,.5) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/55:is(.dark *){--tw-gradient-to:rgba(165,243,252,.55) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/60:is(.dark *){--tw-gradient-to:rgba(165,243,252,.6) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/65:is(.dark *){--tw-gradient-to:rgba(165,243,252,.65) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/70:is(.dark *){--tw-gradient-to:rgba(165,243,252,.7) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/75:is(.dark *){--tw-gradient-to:rgba(165,243,252,.75) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/80:is(.dark *){--tw-gradient-to:rgba(165,243,252,.8) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/85:is(.dark *){--tw-gradient-to:rgba(165,243,252,.85) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/90:is(.dark *){--tw-gradient-to:rgba(165,243,252,.9) var(--tw-gradient-to-position)}.dark\:to-cyan-200\/95:is(.dark *){--tw-gradient-to:rgba(165,243,252,.95) var(--tw-gradient-to-position)}.dark\:to-cyan-50:is(.dark *){--tw-gradient-to:#ecfeff var(--tw-gradient-to-position)}.dark\:to-cyan-50\/0:is(.dark *){--tw-gradient-to:rgba(236,254,255,0) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/10:is(.dark *){--tw-gradient-to:rgba(236,254,255,.1) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/100:is(.dark *){--tw-gradient-to:#ecfeff var(--tw-gradient-to-position)}.dark\:to-cyan-50\/15:is(.dark *){--tw-gradient-to:rgba(236,254,255,.15) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/20:is(.dark *){--tw-gradient-to:rgba(236,254,255,.2) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/25:is(.dark *){--tw-gradient-to:rgba(236,254,255,.25) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/30:is(.dark *){--tw-gradient-to:rgba(236,254,255,.3) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/35:is(.dark *){--tw-gradient-to:rgba(236,254,255,.35) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/40:is(.dark *){--tw-gradient-to:rgba(236,254,255,.4) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/45:is(.dark *){--tw-gradient-to:rgba(236,254,255,.45) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/5:is(.dark *){--tw-gradient-to:rgba(236,254,255,.05) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/50:is(.dark *){--tw-gradient-to:rgba(236,254,255,.5) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/55:is(.dark *){--tw-gradient-to:rgba(236,254,255,.55) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/60:is(.dark *){--tw-gradient-to:rgba(236,254,255,.6) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/65:is(.dark *){--tw-gradient-to:rgba(236,254,255,.65) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/70:is(.dark *){--tw-gradient-to:rgba(236,254,255,.7) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/75:is(.dark *){--tw-gradient-to:rgba(236,254,255,.75) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/80:is(.dark *){--tw-gradient-to:rgba(236,254,255,.8) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/85:is(.dark *){--tw-gradient-to:rgba(236,254,255,.85) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/90:is(.dark *){--tw-gradient-to:rgba(236,254,255,.9) var(--tw-gradient-to-position)}.dark\:to-cyan-50\/95:is(.dark *){--tw-gradient-to:rgba(236,254,255,.95) var(--tw-gradient-to-position)}.dark\:to-cyan-500:is(.dark *){--tw-gradient-to:#06b6d4 var(--tw-gradient-to-position)}.dark\:to-cyan-500\/0:is(.dark *){--tw-gradient-to:rgba(6,182,212,0) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/10:is(.dark *){--tw-gradient-to:rgba(6,182,212,.1) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/100:is(.dark *){--tw-gradient-to:#06b6d4 var(--tw-gradient-to-position)}.dark\:to-cyan-500\/15:is(.dark *){--tw-gradient-to:rgba(6,182,212,.15) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/20:is(.dark *){--tw-gradient-to:rgba(6,182,212,.2) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/25:is(.dark *){--tw-gradient-to:rgba(6,182,212,.25) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/30:is(.dark *){--tw-gradient-to:rgba(6,182,212,.3) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/35:is(.dark *){--tw-gradient-to:rgba(6,182,212,.35) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/40:is(.dark *){--tw-gradient-to:rgba(6,182,212,.4) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/45:is(.dark *){--tw-gradient-to:rgba(6,182,212,.45) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/5:is(.dark *){--tw-gradient-to:rgba(6,182,212,.05) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/50:is(.dark *){--tw-gradient-to:rgba(6,182,212,.5) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/55:is(.dark *){--tw-gradient-to:rgba(6,182,212,.55) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/60:is(.dark *){--tw-gradient-to:rgba(6,182,212,.6) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/65:is(.dark *){--tw-gradient-to:rgba(6,182,212,.65) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/70:is(.dark *){--tw-gradient-to:rgba(6,182,212,.7) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/75:is(.dark *){--tw-gradient-to:rgba(6,182,212,.75) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/80:is(.dark *){--tw-gradient-to:rgba(6,182,212,.8) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/85:is(.dark *){--tw-gradient-to:rgba(6,182,212,.85) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/90:is(.dark *){--tw-gradient-to:rgba(6,182,212,.9) var(--tw-gradient-to-position)}.dark\:to-cyan-500\/95:is(.dark *){--tw-gradient-to:rgba(6,182,212,.95) var(--tw-gradient-to-position)}.dark\:to-cyan-950:is(.dark *){--tw-gradient-to:#083344 var(--tw-gradient-to-position)}.dark\:to-cyan-950\/0:is(.dark *){--tw-gradient-to:rgba(8,51,68,0) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/10:is(.dark *){--tw-gradient-to:rgba(8,51,68,.1) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/100:is(.dark *){--tw-gradient-to:#083344 var(--tw-gradient-to-position)}.dark\:to-cyan-950\/15:is(.dark *){--tw-gradient-to:rgba(8,51,68,.15) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/20:is(.dark *){--tw-gradient-to:rgba(8,51,68,.2) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/25:is(.dark *){--tw-gradient-to:rgba(8,51,68,.25) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/30:is(.dark *){--tw-gradient-to:rgba(8,51,68,.3) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/35:is(.dark *){--tw-gradient-to:rgba(8,51,68,.35) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/40:is(.dark *){--tw-gradient-to:rgba(8,51,68,.4) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/45:is(.dark *){--tw-gradient-to:rgba(8,51,68,.45) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/5:is(.dark *){--tw-gradient-to:rgba(8,51,68,.05) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/50:is(.dark *){--tw-gradient-to:rgba(8,51,68,.5) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/55:is(.dark *){--tw-gradient-to:rgba(8,51,68,.55) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/60:is(.dark *){--tw-gradient-to:rgba(8,51,68,.6) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/65:is(.dark *){--tw-gradient-to:rgba(8,51,68,.65) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/70:is(.dark *){--tw-gradient-to:rgba(8,51,68,.7) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/75:is(.dark *){--tw-gradient-to:rgba(8,51,68,.75) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/80:is(.dark *){--tw-gradient-to:rgba(8,51,68,.8) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/85:is(.dark *){--tw-gradient-to:rgba(8,51,68,.85) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/90:is(.dark *){--tw-gradient-to:rgba(8,51,68,.9) var(--tw-gradient-to-position)}.dark\:to-cyan-950\/95:is(.dark *){--tw-gradient-to:rgba(8,51,68,.95) var(--tw-gradient-to-position)}.dark\:to-emerald-100:is(.dark *){--tw-gradient-to:#d1fae5 var(--tw-gradient-to-position)}.dark\:to-emerald-100\/0:is(.dark *){--tw-gradient-to:rgba(209,250,229,0) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/10:is(.dark *){--tw-gradient-to:rgba(209,250,229,.1) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/100:is(.dark *){--tw-gradient-to:#d1fae5 var(--tw-gradient-to-position)}.dark\:to-emerald-100\/15:is(.dark *){--tw-gradient-to:rgba(209,250,229,.15) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/20:is(.dark *){--tw-gradient-to:rgba(209,250,229,.2) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/25:is(.dark *){--tw-gradient-to:rgba(209,250,229,.25) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/30:is(.dark *){--tw-gradient-to:rgba(209,250,229,.3) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/35:is(.dark *){--tw-gradient-to:rgba(209,250,229,.35) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/40:is(.dark *){--tw-gradient-to:rgba(209,250,229,.4) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/45:is(.dark *){--tw-gradient-to:rgba(209,250,229,.45) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/5:is(.dark *){--tw-gradient-to:rgba(209,250,229,.05) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/50:is(.dark *){--tw-gradient-to:rgba(209,250,229,.5) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/55:is(.dark *){--tw-gradient-to:rgba(209,250,229,.55) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/60:is(.dark *){--tw-gradient-to:rgba(209,250,229,.6) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/65:is(.dark *){--tw-gradient-to:rgba(209,250,229,.65) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/70:is(.dark *){--tw-gradient-to:rgba(209,250,229,.7) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/75:is(.dark *){--tw-gradient-to:rgba(209,250,229,.75) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/80:is(.dark *){--tw-gradient-to:rgba(209,250,229,.8) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/85:is(.dark *){--tw-gradient-to:rgba(209,250,229,.85) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/90:is(.dark *){--tw-gradient-to:rgba(209,250,229,.9) var(--tw-gradient-to-position)}.dark\:to-emerald-100\/95:is(.dark *){--tw-gradient-to:rgba(209,250,229,.95) var(--tw-gradient-to-position)}.dark\:to-emerald-200:is(.dark *){--tw-gradient-to:#a7f3d0 var(--tw-gradient-to-position)}.dark\:to-emerald-200\/0:is(.dark *){--tw-gradient-to:rgba(167,243,208,0) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/10:is(.dark *){--tw-gradient-to:rgba(167,243,208,.1) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/100:is(.dark *){--tw-gradient-to:#a7f3d0 var(--tw-gradient-to-position)}.dark\:to-emerald-200\/15:is(.dark *){--tw-gradient-to:rgba(167,243,208,.15) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/20:is(.dark *){--tw-gradient-to:rgba(167,243,208,.2) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/25:is(.dark *){--tw-gradient-to:rgba(167,243,208,.25) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/30:is(.dark *){--tw-gradient-to:rgba(167,243,208,.3) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/35:is(.dark *){--tw-gradient-to:rgba(167,243,208,.35) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/40:is(.dark *){--tw-gradient-to:rgba(167,243,208,.4) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/45:is(.dark *){--tw-gradient-to:rgba(167,243,208,.45) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/5:is(.dark *){--tw-gradient-to:rgba(167,243,208,.05) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/50:is(.dark *){--tw-gradient-to:rgba(167,243,208,.5) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/55:is(.dark *){--tw-gradient-to:rgba(167,243,208,.55) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/60:is(.dark *){--tw-gradient-to:rgba(167,243,208,.6) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/65:is(.dark *){--tw-gradient-to:rgba(167,243,208,.65) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/70:is(.dark *){--tw-gradient-to:rgba(167,243,208,.7) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/75:is(.dark *){--tw-gradient-to:rgba(167,243,208,.75) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/80:is(.dark *){--tw-gradient-to:rgba(167,243,208,.8) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/85:is(.dark *){--tw-gradient-to:rgba(167,243,208,.85) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/90:is(.dark *){--tw-gradient-to:rgba(167,243,208,.9) var(--tw-gradient-to-position)}.dark\:to-emerald-200\/95:is(.dark *){--tw-gradient-to:rgba(167,243,208,.95) var(--tw-gradient-to-position)}.dark\:to-emerald-50:is(.dark *){--tw-gradient-to:#ecfdf5 var(--tw-gradient-to-position)}.dark\:to-emerald-50\/0:is(.dark *){--tw-gradient-to:rgba(236,253,245,0) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/10:is(.dark *){--tw-gradient-to:rgba(236,253,245,.1) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/100:is(.dark *){--tw-gradient-to:#ecfdf5 var(--tw-gradient-to-position)}.dark\:to-emerald-50\/15:is(.dark *){--tw-gradient-to:rgba(236,253,245,.15) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/20:is(.dark *){--tw-gradient-to:rgba(236,253,245,.2) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/25:is(.dark *){--tw-gradient-to:rgba(236,253,245,.25) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/30:is(.dark *){--tw-gradient-to:rgba(236,253,245,.3) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/35:is(.dark *){--tw-gradient-to:rgba(236,253,245,.35) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/40:is(.dark *){--tw-gradient-to:rgba(236,253,245,.4) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/45:is(.dark *){--tw-gradient-to:rgba(236,253,245,.45) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/5:is(.dark *){--tw-gradient-to:rgba(236,253,245,.05) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/50:is(.dark *){--tw-gradient-to:rgba(236,253,245,.5) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/55:is(.dark *){--tw-gradient-to:rgba(236,253,245,.55) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/60:is(.dark *){--tw-gradient-to:rgba(236,253,245,.6) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/65:is(.dark *){--tw-gradient-to:rgba(236,253,245,.65) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/70:is(.dark *){--tw-gradient-to:rgba(236,253,245,.7) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/75:is(.dark *){--tw-gradient-to:rgba(236,253,245,.75) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/80:is(.dark *){--tw-gradient-to:rgba(236,253,245,.8) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/85:is(.dark *){--tw-gradient-to:rgba(236,253,245,.85) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/90:is(.dark *){--tw-gradient-to:rgba(236,253,245,.9) var(--tw-gradient-to-position)}.dark\:to-emerald-50\/95:is(.dark *){--tw-gradient-to:rgba(236,253,245,.95) var(--tw-gradient-to-position)}.dark\:to-emerald-500:is(.dark *){--tw-gradient-to:#10b981 var(--tw-gradient-to-position)}.dark\:to-emerald-500\/0:is(.dark *){--tw-gradient-to:rgba(16,185,129,0) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/10:is(.dark *){--tw-gradient-to:rgba(16,185,129,.1) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/100:is(.dark *){--tw-gradient-to:#10b981 var(--tw-gradient-to-position)}.dark\:to-emerald-500\/15:is(.dark *){--tw-gradient-to:rgba(16,185,129,.15) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/20:is(.dark *){--tw-gradient-to:rgba(16,185,129,.2) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/25:is(.dark *){--tw-gradient-to:rgba(16,185,129,.25) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/30:is(.dark *){--tw-gradient-to:rgba(16,185,129,.3) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/35:is(.dark *){--tw-gradient-to:rgba(16,185,129,.35) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/40:is(.dark *){--tw-gradient-to:rgba(16,185,129,.4) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/45:is(.dark *){--tw-gradient-to:rgba(16,185,129,.45) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/5:is(.dark *){--tw-gradient-to:rgba(16,185,129,.05) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/50:is(.dark *){--tw-gradient-to:rgba(16,185,129,.5) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/55:is(.dark *){--tw-gradient-to:rgba(16,185,129,.55) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/60:is(.dark *){--tw-gradient-to:rgba(16,185,129,.6) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/65:is(.dark *){--tw-gradient-to:rgba(16,185,129,.65) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/70:is(.dark *){--tw-gradient-to:rgba(16,185,129,.7) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/75:is(.dark *){--tw-gradient-to:rgba(16,185,129,.75) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/80:is(.dark *){--tw-gradient-to:rgba(16,185,129,.8) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/85:is(.dark *){--tw-gradient-to:rgba(16,185,129,.85) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/90:is(.dark *){--tw-gradient-to:rgba(16,185,129,.9) var(--tw-gradient-to-position)}.dark\:to-emerald-500\/95:is(.dark *){--tw-gradient-to:rgba(16,185,129,.95) var(--tw-gradient-to-position)}.dark\:to-emerald-950:is(.dark *){--tw-gradient-to:#022c22 var(--tw-gradient-to-position)}.dark\:to-emerald-950\/0:is(.dark *){--tw-gradient-to:rgba(2,44,34,0) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/10:is(.dark *){--tw-gradient-to:rgba(2,44,34,.1) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/100:is(.dark *){--tw-gradient-to:#022c22 var(--tw-gradient-to-position)}.dark\:to-emerald-950\/15:is(.dark *){--tw-gradient-to:rgba(2,44,34,.15) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/20:is(.dark *){--tw-gradient-to:rgba(2,44,34,.2) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/25:is(.dark *){--tw-gradient-to:rgba(2,44,34,.25) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/30:is(.dark *){--tw-gradient-to:rgba(2,44,34,.3) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/35:is(.dark *){--tw-gradient-to:rgba(2,44,34,.35) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/40:is(.dark *){--tw-gradient-to:rgba(2,44,34,.4) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/45:is(.dark *){--tw-gradient-to:rgba(2,44,34,.45) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/5:is(.dark *){--tw-gradient-to:rgba(2,44,34,.05) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/50:is(.dark *){--tw-gradient-to:rgba(2,44,34,.5) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/55:is(.dark *){--tw-gradient-to:rgba(2,44,34,.55) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/60:is(.dark *){--tw-gradient-to:rgba(2,44,34,.6) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/65:is(.dark *){--tw-gradient-to:rgba(2,44,34,.65) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/70:is(.dark *){--tw-gradient-to:rgba(2,44,34,.7) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/75:is(.dark *){--tw-gradient-to:rgba(2,44,34,.75) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/80:is(.dark *){--tw-gradient-to:rgba(2,44,34,.8) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/85:is(.dark *){--tw-gradient-to:rgba(2,44,34,.85) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/90:is(.dark *){--tw-gradient-to:rgba(2,44,34,.9) var(--tw-gradient-to-position)}.dark\:to-emerald-950\/95:is(.dark *){--tw-gradient-to:rgba(2,44,34,.95) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100:is(.dark *){--tw-gradient-to:#fae8ff var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/0:is(.dark *){--tw-gradient-to:rgba(250,232,255,0) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/10:is(.dark *){--tw-gradient-to:rgba(250,232,255,.1) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/100:is(.dark *){--tw-gradient-to:#fae8ff var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/15:is(.dark *){--tw-gradient-to:rgba(250,232,255,.15) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/20:is(.dark *){--tw-gradient-to:rgba(250,232,255,.2) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/25:is(.dark *){--tw-gradient-to:rgba(250,232,255,.25) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/30:is(.dark *){--tw-gradient-to:rgba(250,232,255,.3) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/35:is(.dark *){--tw-gradient-to:rgba(250,232,255,.35) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/40:is(.dark *){--tw-gradient-to:rgba(250,232,255,.4) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/45:is(.dark *){--tw-gradient-to:rgba(250,232,255,.45) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/5:is(.dark *){--tw-gradient-to:rgba(250,232,255,.05) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/50:is(.dark *){--tw-gradient-to:rgba(250,232,255,.5) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/55:is(.dark *){--tw-gradient-to:rgba(250,232,255,.55) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/60:is(.dark *){--tw-gradient-to:rgba(250,232,255,.6) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/65:is(.dark *){--tw-gradient-to:rgba(250,232,255,.65) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/70:is(.dark *){--tw-gradient-to:rgba(250,232,255,.7) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/75:is(.dark *){--tw-gradient-to:rgba(250,232,255,.75) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/80:is(.dark *){--tw-gradient-to:rgba(250,232,255,.8) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/85:is(.dark *){--tw-gradient-to:rgba(250,232,255,.85) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/90:is(.dark *){--tw-gradient-to:rgba(250,232,255,.9) var(--tw-gradient-to-position)}.dark\:to-fuchsia-100\/95:is(.dark *){--tw-gradient-to:rgba(250,232,255,.95) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200:is(.dark *){--tw-gradient-to:#f5d0fe var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/0:is(.dark *){--tw-gradient-to:rgba(245,208,254,0) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/10:is(.dark *){--tw-gradient-to:rgba(245,208,254,.1) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/100:is(.dark *){--tw-gradient-to:#f5d0fe var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/15:is(.dark *){--tw-gradient-to:rgba(245,208,254,.15) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/20:is(.dark *){--tw-gradient-to:rgba(245,208,254,.2) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/25:is(.dark *){--tw-gradient-to:rgba(245,208,254,.25) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/30:is(.dark *){--tw-gradient-to:rgba(245,208,254,.3) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/35:is(.dark *){--tw-gradient-to:rgba(245,208,254,.35) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/40:is(.dark *){--tw-gradient-to:rgba(245,208,254,.4) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/45:is(.dark *){--tw-gradient-to:rgba(245,208,254,.45) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/5:is(.dark *){--tw-gradient-to:rgba(245,208,254,.05) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/50:is(.dark *){--tw-gradient-to:rgba(245,208,254,.5) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/55:is(.dark *){--tw-gradient-to:rgba(245,208,254,.55) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/60:is(.dark *){--tw-gradient-to:rgba(245,208,254,.6) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/65:is(.dark *){--tw-gradient-to:rgba(245,208,254,.65) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/70:is(.dark *){--tw-gradient-to:rgba(245,208,254,.7) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/75:is(.dark *){--tw-gradient-to:rgba(245,208,254,.75) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/80:is(.dark *){--tw-gradient-to:rgba(245,208,254,.8) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/85:is(.dark *){--tw-gradient-to:rgba(245,208,254,.85) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/90:is(.dark *){--tw-gradient-to:rgba(245,208,254,.9) var(--tw-gradient-to-position)}.dark\:to-fuchsia-200\/95:is(.dark *){--tw-gradient-to:rgba(245,208,254,.95) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50:is(.dark *){--tw-gradient-to:#fdf4ff var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/0:is(.dark *){--tw-gradient-to:rgba(253,244,255,0) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/10:is(.dark *){--tw-gradient-to:rgba(253,244,255,.1) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/100:is(.dark *){--tw-gradient-to:#fdf4ff var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/15:is(.dark *){--tw-gradient-to:rgba(253,244,255,.15) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/20:is(.dark *){--tw-gradient-to:rgba(253,244,255,.2) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/25:is(.dark *){--tw-gradient-to:rgba(253,244,255,.25) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/30:is(.dark *){--tw-gradient-to:rgba(253,244,255,.3) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/35:is(.dark *){--tw-gradient-to:rgba(253,244,255,.35) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/40:is(.dark *){--tw-gradient-to:rgba(253,244,255,.4) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/45:is(.dark *){--tw-gradient-to:rgba(253,244,255,.45) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/5:is(.dark *){--tw-gradient-to:rgba(253,244,255,.05) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/50:is(.dark *){--tw-gradient-to:rgba(253,244,255,.5) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/55:is(.dark *){--tw-gradient-to:rgba(253,244,255,.55) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/60:is(.dark *){--tw-gradient-to:rgba(253,244,255,.6) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/65:is(.dark *){--tw-gradient-to:rgba(253,244,255,.65) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/70:is(.dark *){--tw-gradient-to:rgba(253,244,255,.7) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/75:is(.dark *){--tw-gradient-to:rgba(253,244,255,.75) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/80:is(.dark *){--tw-gradient-to:rgba(253,244,255,.8) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/85:is(.dark *){--tw-gradient-to:rgba(253,244,255,.85) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/90:is(.dark *){--tw-gradient-to:rgba(253,244,255,.9) var(--tw-gradient-to-position)}.dark\:to-fuchsia-50\/95:is(.dark *){--tw-gradient-to:rgba(253,244,255,.95) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500:is(.dark *){--tw-gradient-to:#d946ef var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/0:is(.dark *){--tw-gradient-to:rgba(217,70,239,0) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/10:is(.dark *){--tw-gradient-to:rgba(217,70,239,.1) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/100:is(.dark *){--tw-gradient-to:#d946ef var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/15:is(.dark *){--tw-gradient-to:rgba(217,70,239,.15) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/20:is(.dark *){--tw-gradient-to:rgba(217,70,239,.2) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/25:is(.dark *){--tw-gradient-to:rgba(217,70,239,.25) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/30:is(.dark *){--tw-gradient-to:rgba(217,70,239,.3) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/35:is(.dark *){--tw-gradient-to:rgba(217,70,239,.35) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/40:is(.dark *){--tw-gradient-to:rgba(217,70,239,.4) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/45:is(.dark *){--tw-gradient-to:rgba(217,70,239,.45) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/5:is(.dark *){--tw-gradient-to:rgba(217,70,239,.05) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/50:is(.dark *){--tw-gradient-to:rgba(217,70,239,.5) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/55:is(.dark *){--tw-gradient-to:rgba(217,70,239,.55) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/60:is(.dark *){--tw-gradient-to:rgba(217,70,239,.6) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/65:is(.dark *){--tw-gradient-to:rgba(217,70,239,.65) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/70:is(.dark *){--tw-gradient-to:rgba(217,70,239,.7) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/75:is(.dark *){--tw-gradient-to:rgba(217,70,239,.75) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/80:is(.dark *){--tw-gradient-to:rgba(217,70,239,.8) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/85:is(.dark *){--tw-gradient-to:rgba(217,70,239,.85) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/90:is(.dark *){--tw-gradient-to:rgba(217,70,239,.9) var(--tw-gradient-to-position)}.dark\:to-fuchsia-500\/95:is(.dark *){--tw-gradient-to:rgba(217,70,239,.95) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950:is(.dark *){--tw-gradient-to:#4a044e var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/0:is(.dark *){--tw-gradient-to:rgba(74,4,78,0) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/10:is(.dark *){--tw-gradient-to:rgba(74,4,78,.1) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/100:is(.dark *){--tw-gradient-to:#4a044e var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/15:is(.dark *){--tw-gradient-to:rgba(74,4,78,.15) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/20:is(.dark *){--tw-gradient-to:rgba(74,4,78,.2) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/25:is(.dark *){--tw-gradient-to:rgba(74,4,78,.25) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/30:is(.dark *){--tw-gradient-to:rgba(74,4,78,.3) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/35:is(.dark *){--tw-gradient-to:rgba(74,4,78,.35) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/40:is(.dark *){--tw-gradient-to:rgba(74,4,78,.4) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/45:is(.dark *){--tw-gradient-to:rgba(74,4,78,.45) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/5:is(.dark *){--tw-gradient-to:rgba(74,4,78,.05) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/50:is(.dark *){--tw-gradient-to:rgba(74,4,78,.5) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/55:is(.dark *){--tw-gradient-to:rgba(74,4,78,.55) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/60:is(.dark *){--tw-gradient-to:rgba(74,4,78,.6) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/65:is(.dark *){--tw-gradient-to:rgba(74,4,78,.65) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/70:is(.dark *){--tw-gradient-to:rgba(74,4,78,.7) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/75:is(.dark *){--tw-gradient-to:rgba(74,4,78,.75) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/80:is(.dark *){--tw-gradient-to:rgba(74,4,78,.8) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/85:is(.dark *){--tw-gradient-to:rgba(74,4,78,.85) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/90:is(.dark *){--tw-gradient-to:rgba(74,4,78,.9) var(--tw-gradient-to-position)}.dark\:to-fuchsia-950\/95:is(.dark *){--tw-gradient-to:rgba(74,4,78,.95) var(--tw-gradient-to-position)}.dark\:to-gray-100:is(.dark *){--tw-gradient-to:#f3f4f6 var(--tw-gradient-to-position)}.dark\:to-gray-100\/0:is(.dark *){--tw-gradient-to:rgba(243,244,246,0) var(--tw-gradient-to-position)}.dark\:to-gray-100\/10:is(.dark *){--tw-gradient-to:rgba(243,244,246,.1) var(--tw-gradient-to-position)}.dark\:to-gray-100\/100:is(.dark *){--tw-gradient-to:#f3f4f6 var(--tw-gradient-to-position)}.dark\:to-gray-100\/15:is(.dark *){--tw-gradient-to:rgba(243,244,246,.15) var(--tw-gradient-to-position)}.dark\:to-gray-100\/20:is(.dark *){--tw-gradient-to:rgba(243,244,246,.2) var(--tw-gradient-to-position)}.dark\:to-gray-100\/25:is(.dark *){--tw-gradient-to:rgba(243,244,246,.25) var(--tw-gradient-to-position)}.dark\:to-gray-100\/30:is(.dark *){--tw-gradient-to:rgba(243,244,246,.3) var(--tw-gradient-to-position)}.dark\:to-gray-100\/35:is(.dark *){--tw-gradient-to:rgba(243,244,246,.35) var(--tw-gradient-to-position)}.dark\:to-gray-100\/40:is(.dark *){--tw-gradient-to:rgba(243,244,246,.4) var(--tw-gradient-to-position)}.dark\:to-gray-100\/45:is(.dark *){--tw-gradient-to:rgba(243,244,246,.45) var(--tw-gradient-to-position)}.dark\:to-gray-100\/5:is(.dark *){--tw-gradient-to:rgba(243,244,246,.05) var(--tw-gradient-to-position)}.dark\:to-gray-100\/50:is(.dark *){--tw-gradient-to:rgba(243,244,246,.5) var(--tw-gradient-to-position)}.dark\:to-gray-100\/55:is(.dark *){--tw-gradient-to:rgba(243,244,246,.55) var(--tw-gradient-to-position)}.dark\:to-gray-100\/60:is(.dark *){--tw-gradient-to:rgba(243,244,246,.6) var(--tw-gradient-to-position)}.dark\:to-gray-100\/65:is(.dark *){--tw-gradient-to:rgba(243,244,246,.65) var(--tw-gradient-to-position)}.dark\:to-gray-100\/70:is(.dark *){--tw-gradient-to:rgba(243,244,246,.7) var(--tw-gradient-to-position)}.dark\:to-gray-100\/75:is(.dark *){--tw-gradient-to:rgba(243,244,246,.75) var(--tw-gradient-to-position)}.dark\:to-gray-100\/80:is(.dark *){--tw-gradient-to:rgba(243,244,246,.8) var(--tw-gradient-to-position)}.dark\:to-gray-100\/85:is(.dark *){--tw-gradient-to:rgba(243,244,246,.85) var(--tw-gradient-to-position)}.dark\:to-gray-100\/90:is(.dark *){--tw-gradient-to:rgba(243,244,246,.9) var(--tw-gradient-to-position)}.dark\:to-gray-100\/95:is(.dark *){--tw-gradient-to:rgba(243,244,246,.95) var(--tw-gradient-to-position)}.dark\:to-gray-200:is(.dark *){--tw-gradient-to:#e5e7eb var(--tw-gradient-to-position)}.dark\:to-gray-200\/0:is(.dark *){--tw-gradient-to:rgba(229,231,235,0) var(--tw-gradient-to-position)}.dark\:to-gray-200\/10:is(.dark *){--tw-gradient-to:rgba(229,231,235,.1) var(--tw-gradient-to-position)}.dark\:to-gray-200\/100:is(.dark *){--tw-gradient-to:#e5e7eb var(--tw-gradient-to-position)}.dark\:to-gray-200\/15:is(.dark *){--tw-gradient-to:rgba(229,231,235,.15) var(--tw-gradient-to-position)}.dark\:to-gray-200\/20:is(.dark *){--tw-gradient-to:rgba(229,231,235,.2) var(--tw-gradient-to-position)}.dark\:to-gray-200\/25:is(.dark *){--tw-gradient-to:rgba(229,231,235,.25) var(--tw-gradient-to-position)}.dark\:to-gray-200\/30:is(.dark *){--tw-gradient-to:rgba(229,231,235,.3) var(--tw-gradient-to-position)}.dark\:to-gray-200\/35:is(.dark *){--tw-gradient-to:rgba(229,231,235,.35) var(--tw-gradient-to-position)}.dark\:to-gray-200\/40:is(.dark *){--tw-gradient-to:rgba(229,231,235,.4) var(--tw-gradient-to-position)}.dark\:to-gray-200\/45:is(.dark *){--tw-gradient-to:rgba(229,231,235,.45) var(--tw-gradient-to-position)}.dark\:to-gray-200\/5:is(.dark *){--tw-gradient-to:rgba(229,231,235,.05) var(--tw-gradient-to-position)}.dark\:to-gray-200\/50:is(.dark *){--tw-gradient-to:rgba(229,231,235,.5) var(--tw-gradient-to-position)}.dark\:to-gray-200\/55:is(.dark *){--tw-gradient-to:rgba(229,231,235,.55) var(--tw-gradient-to-position)}.dark\:to-gray-200\/60:is(.dark *){--tw-gradient-to:rgba(229,231,235,.6) var(--tw-gradient-to-position)}.dark\:to-gray-200\/65:is(.dark *){--tw-gradient-to:rgba(229,231,235,.65) var(--tw-gradient-to-position)}.dark\:to-gray-200\/70:is(.dark *){--tw-gradient-to:rgba(229,231,235,.7) var(--tw-gradient-to-position)}.dark\:to-gray-200\/75:is(.dark *){--tw-gradient-to:rgba(229,231,235,.75) var(--tw-gradient-to-position)}.dark\:to-gray-200\/80:is(.dark *){--tw-gradient-to:rgba(229,231,235,.8) var(--tw-gradient-to-position)}.dark\:to-gray-200\/85:is(.dark *){--tw-gradient-to:rgba(229,231,235,.85) var(--tw-gradient-to-position)}.dark\:to-gray-200\/90:is(.dark *){--tw-gradient-to:rgba(229,231,235,.9) var(--tw-gradient-to-position)}.dark\:to-gray-200\/95:is(.dark *){--tw-gradient-to:rgba(229,231,235,.95) var(--tw-gradient-to-position)}.dark\:to-gray-50:is(.dark *){--tw-gradient-to:#f9fafb var(--tw-gradient-to-position)}.dark\:to-gray-50\/0:is(.dark *){--tw-gradient-to:rgba(249,250,251,0) var(--tw-gradient-to-position)}.dark\:to-gray-50\/10:is(.dark *){--tw-gradient-to:rgba(249,250,251,.1) var(--tw-gradient-to-position)}.dark\:to-gray-50\/100:is(.dark *){--tw-gradient-to:#f9fafb var(--tw-gradient-to-position)}.dark\:to-gray-50\/15:is(.dark *){--tw-gradient-to:rgba(249,250,251,.15) var(--tw-gradient-to-position)}.dark\:to-gray-50\/20:is(.dark *){--tw-gradient-to:rgba(249,250,251,.2) var(--tw-gradient-to-position)}.dark\:to-gray-50\/25:is(.dark *){--tw-gradient-to:rgba(249,250,251,.25) var(--tw-gradient-to-position)}.dark\:to-gray-50\/30:is(.dark *){--tw-gradient-to:rgba(249,250,251,.3) var(--tw-gradient-to-position)}.dark\:to-gray-50\/35:is(.dark *){--tw-gradient-to:rgba(249,250,251,.35) var(--tw-gradient-to-position)}.dark\:to-gray-50\/40:is(.dark *){--tw-gradient-to:rgba(249,250,251,.4) var(--tw-gradient-to-position)}.dark\:to-gray-50\/45:is(.dark *){--tw-gradient-to:rgba(249,250,251,.45) var(--tw-gradient-to-position)}.dark\:to-gray-50\/5:is(.dark *){--tw-gradient-to:rgba(249,250,251,.05) var(--tw-gradient-to-position)}.dark\:to-gray-50\/50:is(.dark *){--tw-gradient-to:rgba(249,250,251,.5) var(--tw-gradient-to-position)}.dark\:to-gray-50\/55:is(.dark *){--tw-gradient-to:rgba(249,250,251,.55) var(--tw-gradient-to-position)}.dark\:to-gray-50\/60:is(.dark *){--tw-gradient-to:rgba(249,250,251,.6) var(--tw-gradient-to-position)}.dark\:to-gray-50\/65:is(.dark *){--tw-gradient-to:rgba(249,250,251,.65) var(--tw-gradient-to-position)}.dark\:to-gray-50\/70:is(.dark *){--tw-gradient-to:rgba(249,250,251,.7) var(--tw-gradient-to-position)}.dark\:to-gray-50\/75:is(.dark *){--tw-gradient-to:rgba(249,250,251,.75) var(--tw-gradient-to-position)}.dark\:to-gray-50\/80:is(.dark *){--tw-gradient-to:rgba(249,250,251,.8) var(--tw-gradient-to-position)}.dark\:to-gray-50\/85:is(.dark *){--tw-gradient-to:rgba(249,250,251,.85) var(--tw-gradient-to-position)}.dark\:to-gray-50\/90:is(.dark *){--tw-gradient-to:rgba(249,250,251,.9) var(--tw-gradient-to-position)}.dark\:to-gray-50\/95:is(.dark *){--tw-gradient-to:rgba(249,250,251,.95) var(--tw-gradient-to-position)}.dark\:to-gray-500:is(.dark *){--tw-gradient-to:#6b7280 var(--tw-gradient-to-position)}.dark\:to-gray-500\/0:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,0) var(--tw-gradient-to-position)}.dark\:to-gray-500\/10:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.1) var(--tw-gradient-to-position)}.dark\:to-gray-500\/100:is(.dark *){--tw-gradient-to:#6b7280 var(--tw-gradient-to-position)}.dark\:to-gray-500\/15:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.15) var(--tw-gradient-to-position)}.dark\:to-gray-500\/20:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.2) var(--tw-gradient-to-position)}.dark\:to-gray-500\/25:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.25) var(--tw-gradient-to-position)}.dark\:to-gray-500\/30:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.3) var(--tw-gradient-to-position)}.dark\:to-gray-500\/35:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.35) var(--tw-gradient-to-position)}.dark\:to-gray-500\/40:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.4) var(--tw-gradient-to-position)}.dark\:to-gray-500\/45:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.45) var(--tw-gradient-to-position)}.dark\:to-gray-500\/5:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.05) var(--tw-gradient-to-position)}.dark\:to-gray-500\/50:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.5) var(--tw-gradient-to-position)}.dark\:to-gray-500\/55:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.55) var(--tw-gradient-to-position)}.dark\:to-gray-500\/60:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.6) var(--tw-gradient-to-position)}.dark\:to-gray-500\/65:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.65) var(--tw-gradient-to-position)}.dark\:to-gray-500\/70:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.7) var(--tw-gradient-to-position)}.dark\:to-gray-500\/75:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.75) var(--tw-gradient-to-position)}.dark\:to-gray-500\/80:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.8) var(--tw-gradient-to-position)}.dark\:to-gray-500\/85:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.85) var(--tw-gradient-to-position)}.dark\:to-gray-500\/90:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.9) var(--tw-gradient-to-position)}.dark\:to-gray-500\/95:is(.dark *){--tw-gradient-to:hsla(220,9%,46%,.95) var(--tw-gradient-to-position)}.dark\:to-gray-950:is(.dark *){--tw-gradient-to:#030712 var(--tw-gradient-to-position)}.dark\:to-gray-950\/0:is(.dark *){--tw-gradient-to:rgba(3,7,18,0) var(--tw-gradient-to-position)}.dark\:to-gray-950\/10:is(.dark *){--tw-gradient-to:rgba(3,7,18,.1) var(--tw-gradient-to-position)}.dark\:to-gray-950\/100:is(.dark *){--tw-gradient-to:#030712 var(--tw-gradient-to-position)}.dark\:to-gray-950\/15:is(.dark *){--tw-gradient-to:rgba(3,7,18,.15) var(--tw-gradient-to-position)}.dark\:to-gray-950\/20:is(.dark *){--tw-gradient-to:rgba(3,7,18,.2) var(--tw-gradient-to-position)}.dark\:to-gray-950\/25:is(.dark *){--tw-gradient-to:rgba(3,7,18,.25) var(--tw-gradient-to-position)}.dark\:to-gray-950\/30:is(.dark *){--tw-gradient-to:rgba(3,7,18,.3) var(--tw-gradient-to-position)}.dark\:to-gray-950\/35:is(.dark *){--tw-gradient-to:rgba(3,7,18,.35) var(--tw-gradient-to-position)}.dark\:to-gray-950\/40:is(.dark *){--tw-gradient-to:rgba(3,7,18,.4) var(--tw-gradient-to-position)}.dark\:to-gray-950\/45:is(.dark *){--tw-gradient-to:rgba(3,7,18,.45) var(--tw-gradient-to-position)}.dark\:to-gray-950\/5:is(.dark *){--tw-gradient-to:rgba(3,7,18,.05) var(--tw-gradient-to-position)}.dark\:to-gray-950\/50:is(.dark *){--tw-gradient-to:rgba(3,7,18,.5) var(--tw-gradient-to-position)}.dark\:to-gray-950\/55:is(.dark *){--tw-gradient-to:rgba(3,7,18,.55) var(--tw-gradient-to-position)}.dark\:to-gray-950\/60:is(.dark *){--tw-gradient-to:rgba(3,7,18,.6) var(--tw-gradient-to-position)}.dark\:to-gray-950\/65:is(.dark *){--tw-gradient-to:rgba(3,7,18,.65) var(--tw-gradient-to-position)}.dark\:to-gray-950\/70:is(.dark *){--tw-gradient-to:rgba(3,7,18,.7) var(--tw-gradient-to-position)}.dark\:to-gray-950\/75:is(.dark *){--tw-gradient-to:rgba(3,7,18,.75) var(--tw-gradient-to-position)}.dark\:to-gray-950\/80:is(.dark *){--tw-gradient-to:rgba(3,7,18,.8) var(--tw-gradient-to-position)}.dark\:to-gray-950\/85:is(.dark *){--tw-gradient-to:rgba(3,7,18,.85) var(--tw-gradient-to-position)}.dark\:to-gray-950\/90:is(.dark *){--tw-gradient-to:rgba(3,7,18,.9) var(--tw-gradient-to-position)}.dark\:to-gray-950\/95:is(.dark *){--tw-gradient-to:rgba(3,7,18,.95) var(--tw-gradient-to-position)}.dark\:to-green-100:is(.dark *){--tw-gradient-to:#dcfce7 var(--tw-gradient-to-position)}.dark\:to-green-100\/0:is(.dark *){--tw-gradient-to:rgba(220,252,231,0) var(--tw-gradient-to-position)}.dark\:to-green-100\/10:is(.dark *){--tw-gradient-to:rgba(220,252,231,.1) var(--tw-gradient-to-position)}.dark\:to-green-100\/100:is(.dark *){--tw-gradient-to:#dcfce7 var(--tw-gradient-to-position)}.dark\:to-green-100\/15:is(.dark *){--tw-gradient-to:rgba(220,252,231,.15) var(--tw-gradient-to-position)}.dark\:to-green-100\/20:is(.dark *){--tw-gradient-to:rgba(220,252,231,.2) var(--tw-gradient-to-position)}.dark\:to-green-100\/25:is(.dark *){--tw-gradient-to:rgba(220,252,231,.25) var(--tw-gradient-to-position)}.dark\:to-green-100\/30:is(.dark *){--tw-gradient-to:rgba(220,252,231,.3) var(--tw-gradient-to-position)}.dark\:to-green-100\/35:is(.dark *){--tw-gradient-to:rgba(220,252,231,.35) var(--tw-gradient-to-position)}.dark\:to-green-100\/40:is(.dark *){--tw-gradient-to:rgba(220,252,231,.4) var(--tw-gradient-to-position)}.dark\:to-green-100\/45:is(.dark *){--tw-gradient-to:rgba(220,252,231,.45) var(--tw-gradient-to-position)}.dark\:to-green-100\/5:is(.dark *){--tw-gradient-to:rgba(220,252,231,.05) var(--tw-gradient-to-position)}.dark\:to-green-100\/50:is(.dark *){--tw-gradient-to:rgba(220,252,231,.5) var(--tw-gradient-to-position)}.dark\:to-green-100\/55:is(.dark *){--tw-gradient-to:rgba(220,252,231,.55) var(--tw-gradient-to-position)}.dark\:to-green-100\/60:is(.dark *){--tw-gradient-to:rgba(220,252,231,.6) var(--tw-gradient-to-position)}.dark\:to-green-100\/65:is(.dark *){--tw-gradient-to:rgba(220,252,231,.65) var(--tw-gradient-to-position)}.dark\:to-green-100\/70:is(.dark *){--tw-gradient-to:rgba(220,252,231,.7) var(--tw-gradient-to-position)}.dark\:to-green-100\/75:is(.dark *){--tw-gradient-to:rgba(220,252,231,.75) var(--tw-gradient-to-position)}.dark\:to-green-100\/80:is(.dark *){--tw-gradient-to:rgba(220,252,231,.8) var(--tw-gradient-to-position)}.dark\:to-green-100\/85:is(.dark *){--tw-gradient-to:rgba(220,252,231,.85) var(--tw-gradient-to-position)}.dark\:to-green-100\/90:is(.dark *){--tw-gradient-to:rgba(220,252,231,.9) var(--tw-gradient-to-position)}.dark\:to-green-100\/95:is(.dark *){--tw-gradient-to:rgba(220,252,231,.95) var(--tw-gradient-to-position)}.dark\:to-green-200:is(.dark *){--tw-gradient-to:#bbf7d0 var(--tw-gradient-to-position)}.dark\:to-green-200\/0:is(.dark *){--tw-gradient-to:rgba(187,247,208,0) var(--tw-gradient-to-position)}.dark\:to-green-200\/10:is(.dark *){--tw-gradient-to:rgba(187,247,208,.1) var(--tw-gradient-to-position)}.dark\:to-green-200\/100:is(.dark *){--tw-gradient-to:#bbf7d0 var(--tw-gradient-to-position)}.dark\:to-green-200\/15:is(.dark *){--tw-gradient-to:rgba(187,247,208,.15) var(--tw-gradient-to-position)}.dark\:to-green-200\/20:is(.dark *){--tw-gradient-to:rgba(187,247,208,.2) var(--tw-gradient-to-position)}.dark\:to-green-200\/25:is(.dark *){--tw-gradient-to:rgba(187,247,208,.25) var(--tw-gradient-to-position)}.dark\:to-green-200\/30:is(.dark *){--tw-gradient-to:rgba(187,247,208,.3) var(--tw-gradient-to-position)}.dark\:to-green-200\/35:is(.dark *){--tw-gradient-to:rgba(187,247,208,.35) var(--tw-gradient-to-position)}.dark\:to-green-200\/40:is(.dark *){--tw-gradient-to:rgba(187,247,208,.4) var(--tw-gradient-to-position)}.dark\:to-green-200\/45:is(.dark *){--tw-gradient-to:rgba(187,247,208,.45) var(--tw-gradient-to-position)}.dark\:to-green-200\/5:is(.dark *){--tw-gradient-to:rgba(187,247,208,.05) var(--tw-gradient-to-position)}.dark\:to-green-200\/50:is(.dark *){--tw-gradient-to:rgba(187,247,208,.5) var(--tw-gradient-to-position)}.dark\:to-green-200\/55:is(.dark *){--tw-gradient-to:rgba(187,247,208,.55) var(--tw-gradient-to-position)}.dark\:to-green-200\/60:is(.dark *){--tw-gradient-to:rgba(187,247,208,.6) var(--tw-gradient-to-position)}.dark\:to-green-200\/65:is(.dark *){--tw-gradient-to:rgba(187,247,208,.65) var(--tw-gradient-to-position)}.dark\:to-green-200\/70:is(.dark *){--tw-gradient-to:rgba(187,247,208,.7) var(--tw-gradient-to-position)}.dark\:to-green-200\/75:is(.dark *){--tw-gradient-to:rgba(187,247,208,.75) var(--tw-gradient-to-position)}.dark\:to-green-200\/80:is(.dark *){--tw-gradient-to:rgba(187,247,208,.8) var(--tw-gradient-to-position)}.dark\:to-green-200\/85:is(.dark *){--tw-gradient-to:rgba(187,247,208,.85) var(--tw-gradient-to-position)}.dark\:to-green-200\/90:is(.dark *){--tw-gradient-to:rgba(187,247,208,.9) var(--tw-gradient-to-position)}.dark\:to-green-200\/95:is(.dark *){--tw-gradient-to:rgba(187,247,208,.95) var(--tw-gradient-to-position)}.dark\:to-green-50:is(.dark *){--tw-gradient-to:#f0fdf4 var(--tw-gradient-to-position)}.dark\:to-green-50\/0:is(.dark *){--tw-gradient-to:rgba(240,253,244,0) var(--tw-gradient-to-position)}.dark\:to-green-50\/10:is(.dark *){--tw-gradient-to:rgba(240,253,244,.1) var(--tw-gradient-to-position)}.dark\:to-green-50\/100:is(.dark *){--tw-gradient-to:#f0fdf4 var(--tw-gradient-to-position)}.dark\:to-green-50\/15:is(.dark *){--tw-gradient-to:rgba(240,253,244,.15) var(--tw-gradient-to-position)}.dark\:to-green-50\/20:is(.dark *){--tw-gradient-to:rgba(240,253,244,.2) var(--tw-gradient-to-position)}.dark\:to-green-50\/25:is(.dark *){--tw-gradient-to:rgba(240,253,244,.25) var(--tw-gradient-to-position)}.dark\:to-green-50\/30:is(.dark *){--tw-gradient-to:rgba(240,253,244,.3) var(--tw-gradient-to-position)}.dark\:to-green-50\/35:is(.dark *){--tw-gradient-to:rgba(240,253,244,.35) var(--tw-gradient-to-position)}.dark\:to-green-50\/40:is(.dark *){--tw-gradient-to:rgba(240,253,244,.4) var(--tw-gradient-to-position)}.dark\:to-green-50\/45:is(.dark *){--tw-gradient-to:rgba(240,253,244,.45) var(--tw-gradient-to-position)}.dark\:to-green-50\/5:is(.dark *){--tw-gradient-to:rgba(240,253,244,.05) var(--tw-gradient-to-position)}.dark\:to-green-50\/50:is(.dark *){--tw-gradient-to:rgba(240,253,244,.5) var(--tw-gradient-to-position)}.dark\:to-green-50\/55:is(.dark *){--tw-gradient-to:rgba(240,253,244,.55) var(--tw-gradient-to-position)}.dark\:to-green-50\/60:is(.dark *){--tw-gradient-to:rgba(240,253,244,.6) var(--tw-gradient-to-position)}.dark\:to-green-50\/65:is(.dark *){--tw-gradient-to:rgba(240,253,244,.65) var(--tw-gradient-to-position)}.dark\:to-green-50\/70:is(.dark *){--tw-gradient-to:rgba(240,253,244,.7) var(--tw-gradient-to-position)}.dark\:to-green-50\/75:is(.dark *){--tw-gradient-to:rgba(240,253,244,.75) var(--tw-gradient-to-position)}.dark\:to-green-50\/80:is(.dark *){--tw-gradient-to:rgba(240,253,244,.8) var(--tw-gradient-to-position)}.dark\:to-green-50\/85:is(.dark *){--tw-gradient-to:rgba(240,253,244,.85) var(--tw-gradient-to-position)}.dark\:to-green-50\/90:is(.dark *){--tw-gradient-to:rgba(240,253,244,.9) var(--tw-gradient-to-position)}.dark\:to-green-50\/95:is(.dark *){--tw-gradient-to:rgba(240,253,244,.95) var(--tw-gradient-to-position)}.dark\:to-green-500:is(.dark *){--tw-gradient-to:#22c55e var(--tw-gradient-to-position)}.dark\:to-green-500\/0:is(.dark *){--tw-gradient-to:rgba(34,197,94,0) var(--tw-gradient-to-position)}.dark\:to-green-500\/10:is(.dark *){--tw-gradient-to:rgba(34,197,94,.1) var(--tw-gradient-to-position)}.dark\:to-green-500\/100:is(.dark *){--tw-gradient-to:#22c55e var(--tw-gradient-to-position)}.dark\:to-green-500\/15:is(.dark *){--tw-gradient-to:rgba(34,197,94,.15) var(--tw-gradient-to-position)}.dark\:to-green-500\/20:is(.dark *){--tw-gradient-to:rgba(34,197,94,.2) var(--tw-gradient-to-position)}.dark\:to-green-500\/25:is(.dark *){--tw-gradient-to:rgba(34,197,94,.25) var(--tw-gradient-to-position)}.dark\:to-green-500\/30:is(.dark *){--tw-gradient-to:rgba(34,197,94,.3) var(--tw-gradient-to-position)}.dark\:to-green-500\/35:is(.dark *){--tw-gradient-to:rgba(34,197,94,.35) var(--tw-gradient-to-position)}.dark\:to-green-500\/40:is(.dark *){--tw-gradient-to:rgba(34,197,94,.4) var(--tw-gradient-to-position)}.dark\:to-green-500\/45:is(.dark *){--tw-gradient-to:rgba(34,197,94,.45) var(--tw-gradient-to-position)}.dark\:to-green-500\/5:is(.dark *){--tw-gradient-to:rgba(34,197,94,.05) var(--tw-gradient-to-position)}.dark\:to-green-500\/50:is(.dark *){--tw-gradient-to:rgba(34,197,94,.5) var(--tw-gradient-to-position)}.dark\:to-green-500\/55:is(.dark *){--tw-gradient-to:rgba(34,197,94,.55) var(--tw-gradient-to-position)}.dark\:to-green-500\/60:is(.dark *){--tw-gradient-to:rgba(34,197,94,.6) var(--tw-gradient-to-position)}.dark\:to-green-500\/65:is(.dark *){--tw-gradient-to:rgba(34,197,94,.65) var(--tw-gradient-to-position)}.dark\:to-green-500\/70:is(.dark *){--tw-gradient-to:rgba(34,197,94,.7) var(--tw-gradient-to-position)}.dark\:to-green-500\/75:is(.dark *){--tw-gradient-to:rgba(34,197,94,.75) var(--tw-gradient-to-position)}.dark\:to-green-500\/80:is(.dark *){--tw-gradient-to:rgba(34,197,94,.8) var(--tw-gradient-to-position)}.dark\:to-green-500\/85:is(.dark *){--tw-gradient-to:rgba(34,197,94,.85) var(--tw-gradient-to-position)}.dark\:to-green-500\/90:is(.dark *){--tw-gradient-to:rgba(34,197,94,.9) var(--tw-gradient-to-position)}.dark\:to-green-500\/95:is(.dark *){--tw-gradient-to:rgba(34,197,94,.95) var(--tw-gradient-to-position)}.dark\:to-green-950:is(.dark *){--tw-gradient-to:#052e16 var(--tw-gradient-to-position)}.dark\:to-green-950\/0:is(.dark *){--tw-gradient-to:rgba(5,46,22,0) var(--tw-gradient-to-position)}.dark\:to-green-950\/10:is(.dark *){--tw-gradient-to:rgba(5,46,22,.1) var(--tw-gradient-to-position)}.dark\:to-green-950\/100:is(.dark *){--tw-gradient-to:#052e16 var(--tw-gradient-to-position)}.dark\:to-green-950\/15:is(.dark *){--tw-gradient-to:rgba(5,46,22,.15) var(--tw-gradient-to-position)}.dark\:to-green-950\/20:is(.dark *){--tw-gradient-to:rgba(5,46,22,.2) var(--tw-gradient-to-position)}.dark\:to-green-950\/25:is(.dark *){--tw-gradient-to:rgba(5,46,22,.25) var(--tw-gradient-to-position)}.dark\:to-green-950\/30:is(.dark *){--tw-gradient-to:rgba(5,46,22,.3) var(--tw-gradient-to-position)}.dark\:to-green-950\/35:is(.dark *){--tw-gradient-to:rgba(5,46,22,.35) var(--tw-gradient-to-position)}.dark\:to-green-950\/40:is(.dark *){--tw-gradient-to:rgba(5,46,22,.4) var(--tw-gradient-to-position)}.dark\:to-green-950\/45:is(.dark *){--tw-gradient-to:rgba(5,46,22,.45) var(--tw-gradient-to-position)}.dark\:to-green-950\/5:is(.dark *){--tw-gradient-to:rgba(5,46,22,.05) var(--tw-gradient-to-position)}.dark\:to-green-950\/50:is(.dark *){--tw-gradient-to:rgba(5,46,22,.5) var(--tw-gradient-to-position)}.dark\:to-green-950\/55:is(.dark *){--tw-gradient-to:rgba(5,46,22,.55) var(--tw-gradient-to-position)}.dark\:to-green-950\/60:is(.dark *){--tw-gradient-to:rgba(5,46,22,.6) var(--tw-gradient-to-position)}.dark\:to-green-950\/65:is(.dark *){--tw-gradient-to:rgba(5,46,22,.65) var(--tw-gradient-to-position)}.dark\:to-green-950\/70:is(.dark *){--tw-gradient-to:rgba(5,46,22,.7) var(--tw-gradient-to-position)}.dark\:to-green-950\/75:is(.dark *){--tw-gradient-to:rgba(5,46,22,.75) var(--tw-gradient-to-position)}.dark\:to-green-950\/80:is(.dark *){--tw-gradient-to:rgba(5,46,22,.8) var(--tw-gradient-to-position)}.dark\:to-green-950\/85:is(.dark *){--tw-gradient-to:rgba(5,46,22,.85) var(--tw-gradient-to-position)}.dark\:to-green-950\/90:is(.dark *){--tw-gradient-to:rgba(5,46,22,.9) var(--tw-gradient-to-position)}.dark\:to-green-950\/95:is(.dark *){--tw-gradient-to:rgba(5,46,22,.95) var(--tw-gradient-to-position)}.dark\:to-indigo-100:is(.dark *){--tw-gradient-to:#e0e7ff var(--tw-gradient-to-position)}.dark\:to-indigo-100\/0:is(.dark *){--tw-gradient-to:rgba(224,231,255,0) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/10:is(.dark *){--tw-gradient-to:rgba(224,231,255,.1) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/100:is(.dark *){--tw-gradient-to:#e0e7ff var(--tw-gradient-to-position)}.dark\:to-indigo-100\/15:is(.dark *){--tw-gradient-to:rgba(224,231,255,.15) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/20:is(.dark *){--tw-gradient-to:rgba(224,231,255,.2) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/25:is(.dark *){--tw-gradient-to:rgba(224,231,255,.25) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/30:is(.dark *){--tw-gradient-to:rgba(224,231,255,.3) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/35:is(.dark *){--tw-gradient-to:rgba(224,231,255,.35) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/40:is(.dark *){--tw-gradient-to:rgba(224,231,255,.4) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/45:is(.dark *){--tw-gradient-to:rgba(224,231,255,.45) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/5:is(.dark *){--tw-gradient-to:rgba(224,231,255,.05) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/50:is(.dark *){--tw-gradient-to:rgba(224,231,255,.5) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/55:is(.dark *){--tw-gradient-to:rgba(224,231,255,.55) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/60:is(.dark *){--tw-gradient-to:rgba(224,231,255,.6) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/65:is(.dark *){--tw-gradient-to:rgba(224,231,255,.65) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/70:is(.dark *){--tw-gradient-to:rgba(224,231,255,.7) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/75:is(.dark *){--tw-gradient-to:rgba(224,231,255,.75) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/80:is(.dark *){--tw-gradient-to:rgba(224,231,255,.8) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/85:is(.dark *){--tw-gradient-to:rgba(224,231,255,.85) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/90:is(.dark *){--tw-gradient-to:rgba(224,231,255,.9) var(--tw-gradient-to-position)}.dark\:to-indigo-100\/95:is(.dark *){--tw-gradient-to:rgba(224,231,255,.95) var(--tw-gradient-to-position)}.dark\:to-indigo-200:is(.dark *){--tw-gradient-to:#c7d2fe var(--tw-gradient-to-position)}.dark\:to-indigo-200\/0:is(.dark *){--tw-gradient-to:rgba(199,210,254,0) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/10:is(.dark *){--tw-gradient-to:rgba(199,210,254,.1) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/100:is(.dark *){--tw-gradient-to:#c7d2fe var(--tw-gradient-to-position)}.dark\:to-indigo-200\/15:is(.dark *){--tw-gradient-to:rgba(199,210,254,.15) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/20:is(.dark *){--tw-gradient-to:rgba(199,210,254,.2) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/25:is(.dark *){--tw-gradient-to:rgba(199,210,254,.25) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/30:is(.dark *){--tw-gradient-to:rgba(199,210,254,.3) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/35:is(.dark *){--tw-gradient-to:rgba(199,210,254,.35) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/40:is(.dark *){--tw-gradient-to:rgba(199,210,254,.4) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/45:is(.dark *){--tw-gradient-to:rgba(199,210,254,.45) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/5:is(.dark *){--tw-gradient-to:rgba(199,210,254,.05) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/50:is(.dark *){--tw-gradient-to:rgba(199,210,254,.5) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/55:is(.dark *){--tw-gradient-to:rgba(199,210,254,.55) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/60:is(.dark *){--tw-gradient-to:rgba(199,210,254,.6) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/65:is(.dark *){--tw-gradient-to:rgba(199,210,254,.65) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/70:is(.dark *){--tw-gradient-to:rgba(199,210,254,.7) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/75:is(.dark *){--tw-gradient-to:rgba(199,210,254,.75) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/80:is(.dark *){--tw-gradient-to:rgba(199,210,254,.8) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/85:is(.dark *){--tw-gradient-to:rgba(199,210,254,.85) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/90:is(.dark *){--tw-gradient-to:rgba(199,210,254,.9) var(--tw-gradient-to-position)}.dark\:to-indigo-200\/95:is(.dark *){--tw-gradient-to:rgba(199,210,254,.95) var(--tw-gradient-to-position)}.dark\:to-indigo-50:is(.dark *){--tw-gradient-to:#eef2ff var(--tw-gradient-to-position)}.dark\:to-indigo-50\/0:is(.dark *){--tw-gradient-to:rgba(238,242,255,0) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/10:is(.dark *){--tw-gradient-to:rgba(238,242,255,.1) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/100:is(.dark *){--tw-gradient-to:#eef2ff var(--tw-gradient-to-position)}.dark\:to-indigo-50\/15:is(.dark *){--tw-gradient-to:rgba(238,242,255,.15) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/20:is(.dark *){--tw-gradient-to:rgba(238,242,255,.2) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/25:is(.dark *){--tw-gradient-to:rgba(238,242,255,.25) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/30:is(.dark *){--tw-gradient-to:rgba(238,242,255,.3) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/35:is(.dark *){--tw-gradient-to:rgba(238,242,255,.35) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/40:is(.dark *){--tw-gradient-to:rgba(238,242,255,.4) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/45:is(.dark *){--tw-gradient-to:rgba(238,242,255,.45) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/5:is(.dark *){--tw-gradient-to:rgba(238,242,255,.05) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/50:is(.dark *){--tw-gradient-to:rgba(238,242,255,.5) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/55:is(.dark *){--tw-gradient-to:rgba(238,242,255,.55) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/60:is(.dark *){--tw-gradient-to:rgba(238,242,255,.6) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/65:is(.dark *){--tw-gradient-to:rgba(238,242,255,.65) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/70:is(.dark *){--tw-gradient-to:rgba(238,242,255,.7) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/75:is(.dark *){--tw-gradient-to:rgba(238,242,255,.75) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/80:is(.dark *){--tw-gradient-to:rgba(238,242,255,.8) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/85:is(.dark *){--tw-gradient-to:rgba(238,242,255,.85) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/90:is(.dark *){--tw-gradient-to:rgba(238,242,255,.9) var(--tw-gradient-to-position)}.dark\:to-indigo-50\/95:is(.dark *){--tw-gradient-to:rgba(238,242,255,.95) var(--tw-gradient-to-position)}.dark\:to-indigo-500:is(.dark *){--tw-gradient-to:#6366f1 var(--tw-gradient-to-position)}.dark\:to-indigo-500\/0:is(.dark *){--tw-gradient-to:rgba(99,102,241,0) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/10:is(.dark *){--tw-gradient-to:rgba(99,102,241,.1) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/100:is(.dark *){--tw-gradient-to:#6366f1 var(--tw-gradient-to-position)}.dark\:to-indigo-500\/15:is(.dark *){--tw-gradient-to:rgba(99,102,241,.15) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/20:is(.dark *){--tw-gradient-to:rgba(99,102,241,.2) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/25:is(.dark *){--tw-gradient-to:rgba(99,102,241,.25) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/30:is(.dark *){--tw-gradient-to:rgba(99,102,241,.3) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/35:is(.dark *){--tw-gradient-to:rgba(99,102,241,.35) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/40:is(.dark *){--tw-gradient-to:rgba(99,102,241,.4) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/45:is(.dark *){--tw-gradient-to:rgba(99,102,241,.45) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/5:is(.dark *){--tw-gradient-to:rgba(99,102,241,.05) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/50:is(.dark *){--tw-gradient-to:rgba(99,102,241,.5) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/55:is(.dark *){--tw-gradient-to:rgba(99,102,241,.55) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/60:is(.dark *){--tw-gradient-to:rgba(99,102,241,.6) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/65:is(.dark *){--tw-gradient-to:rgba(99,102,241,.65) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/70:is(.dark *){--tw-gradient-to:rgba(99,102,241,.7) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/75:is(.dark *){--tw-gradient-to:rgba(99,102,241,.75) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/80:is(.dark *){--tw-gradient-to:rgba(99,102,241,.8) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/85:is(.dark *){--tw-gradient-to:rgba(99,102,241,.85) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/90:is(.dark *){--tw-gradient-to:rgba(99,102,241,.9) var(--tw-gradient-to-position)}.dark\:to-indigo-500\/95:is(.dark *){--tw-gradient-to:rgba(99,102,241,.95) var(--tw-gradient-to-position)}.dark\:to-indigo-950:is(.dark *){--tw-gradient-to:#1e1b4b var(--tw-gradient-to-position)}.dark\:to-indigo-950\/0:is(.dark *){--tw-gradient-to:rgba(30,27,75,0) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/10:is(.dark *){--tw-gradient-to:rgba(30,27,75,.1) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/100:is(.dark *){--tw-gradient-to:#1e1b4b var(--tw-gradient-to-position)}.dark\:to-indigo-950\/15:is(.dark *){--tw-gradient-to:rgba(30,27,75,.15) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/20:is(.dark *){--tw-gradient-to:rgba(30,27,75,.2) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/25:is(.dark *){--tw-gradient-to:rgba(30,27,75,.25) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/30:is(.dark *){--tw-gradient-to:rgba(30,27,75,.3) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/35:is(.dark *){--tw-gradient-to:rgba(30,27,75,.35) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/40:is(.dark *){--tw-gradient-to:rgba(30,27,75,.4) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/45:is(.dark *){--tw-gradient-to:rgba(30,27,75,.45) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/5:is(.dark *){--tw-gradient-to:rgba(30,27,75,.05) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/50:is(.dark *){--tw-gradient-to:rgba(30,27,75,.5) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/55:is(.dark *){--tw-gradient-to:rgba(30,27,75,.55) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/60:is(.dark *){--tw-gradient-to:rgba(30,27,75,.6) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/65:is(.dark *){--tw-gradient-to:rgba(30,27,75,.65) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/70:is(.dark *){--tw-gradient-to:rgba(30,27,75,.7) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/75:is(.dark *){--tw-gradient-to:rgba(30,27,75,.75) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/80:is(.dark *){--tw-gradient-to:rgba(30,27,75,.8) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/85:is(.dark *){--tw-gradient-to:rgba(30,27,75,.85) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/90:is(.dark *){--tw-gradient-to:rgba(30,27,75,.9) var(--tw-gradient-to-position)}.dark\:to-indigo-950\/95:is(.dark *){--tw-gradient-to:rgba(30,27,75,.95) var(--tw-gradient-to-position)}.dark\:to-lime-100:is(.dark *){--tw-gradient-to:#ecfccb var(--tw-gradient-to-position)}.dark\:to-lime-100\/0:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,0) var(--tw-gradient-to-position)}.dark\:to-lime-100\/10:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.1) var(--tw-gradient-to-position)}.dark\:to-lime-100\/100:is(.dark *){--tw-gradient-to:#ecfccb var(--tw-gradient-to-position)}.dark\:to-lime-100\/15:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.15) var(--tw-gradient-to-position)}.dark\:to-lime-100\/20:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.2) var(--tw-gradient-to-position)}.dark\:to-lime-100\/25:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.25) var(--tw-gradient-to-position)}.dark\:to-lime-100\/30:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.3) var(--tw-gradient-to-position)}.dark\:to-lime-100\/35:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.35) var(--tw-gradient-to-position)}.dark\:to-lime-100\/40:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.4) var(--tw-gradient-to-position)}.dark\:to-lime-100\/45:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.45) var(--tw-gradient-to-position)}.dark\:to-lime-100\/5:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.05) var(--tw-gradient-to-position)}.dark\:to-lime-100\/50:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.5) var(--tw-gradient-to-position)}.dark\:to-lime-100\/55:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.55) var(--tw-gradient-to-position)}.dark\:to-lime-100\/60:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.6) var(--tw-gradient-to-position)}.dark\:to-lime-100\/65:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.65) var(--tw-gradient-to-position)}.dark\:to-lime-100\/70:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.7) var(--tw-gradient-to-position)}.dark\:to-lime-100\/75:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.75) var(--tw-gradient-to-position)}.dark\:to-lime-100\/80:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.8) var(--tw-gradient-to-position)}.dark\:to-lime-100\/85:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.85) var(--tw-gradient-to-position)}.dark\:to-lime-100\/90:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.9) var(--tw-gradient-to-position)}.dark\:to-lime-100\/95:is(.dark *){--tw-gradient-to:hsla(80,89%,89%,.95) var(--tw-gradient-to-position)}.dark\:to-lime-200:is(.dark *){--tw-gradient-to:#d9f99d var(--tw-gradient-to-position)}.dark\:to-lime-200\/0:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,0) var(--tw-gradient-to-position)}.dark\:to-lime-200\/10:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.1) var(--tw-gradient-to-position)}.dark\:to-lime-200\/100:is(.dark *){--tw-gradient-to:#d9f99d var(--tw-gradient-to-position)}.dark\:to-lime-200\/15:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.15) var(--tw-gradient-to-position)}.dark\:to-lime-200\/20:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.2) var(--tw-gradient-to-position)}.dark\:to-lime-200\/25:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.25) var(--tw-gradient-to-position)}.dark\:to-lime-200\/30:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.3) var(--tw-gradient-to-position)}.dark\:to-lime-200\/35:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.35) var(--tw-gradient-to-position)}.dark\:to-lime-200\/40:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.4) var(--tw-gradient-to-position)}.dark\:to-lime-200\/45:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.45) var(--tw-gradient-to-position)}.dark\:to-lime-200\/5:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.05) var(--tw-gradient-to-position)}.dark\:to-lime-200\/50:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.5) var(--tw-gradient-to-position)}.dark\:to-lime-200\/55:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.55) var(--tw-gradient-to-position)}.dark\:to-lime-200\/60:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.6) var(--tw-gradient-to-position)}.dark\:to-lime-200\/65:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.65) var(--tw-gradient-to-position)}.dark\:to-lime-200\/70:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.7) var(--tw-gradient-to-position)}.dark\:to-lime-200\/75:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.75) var(--tw-gradient-to-position)}.dark\:to-lime-200\/80:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.8) var(--tw-gradient-to-position)}.dark\:to-lime-200\/85:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.85) var(--tw-gradient-to-position)}.dark\:to-lime-200\/90:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.9) var(--tw-gradient-to-position)}.dark\:to-lime-200\/95:is(.dark *){--tw-gradient-to:hsla(81,88%,80%,.95) var(--tw-gradient-to-position)}.dark\:to-lime-50:is(.dark *){--tw-gradient-to:#f7fee7 var(--tw-gradient-to-position)}.dark\:to-lime-50\/0:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,0) var(--tw-gradient-to-position)}.dark\:to-lime-50\/10:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.1) var(--tw-gradient-to-position)}.dark\:to-lime-50\/100:is(.dark *){--tw-gradient-to:#f7fee7 var(--tw-gradient-to-position)}.dark\:to-lime-50\/15:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.15) var(--tw-gradient-to-position)}.dark\:to-lime-50\/20:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.2) var(--tw-gradient-to-position)}.dark\:to-lime-50\/25:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.25) var(--tw-gradient-to-position)}.dark\:to-lime-50\/30:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.3) var(--tw-gradient-to-position)}.dark\:to-lime-50\/35:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.35) var(--tw-gradient-to-position)}.dark\:to-lime-50\/40:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.4) var(--tw-gradient-to-position)}.dark\:to-lime-50\/45:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.45) var(--tw-gradient-to-position)}.dark\:to-lime-50\/5:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.05) var(--tw-gradient-to-position)}.dark\:to-lime-50\/50:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.5) var(--tw-gradient-to-position)}.dark\:to-lime-50\/55:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.55) var(--tw-gradient-to-position)}.dark\:to-lime-50\/60:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.6) var(--tw-gradient-to-position)}.dark\:to-lime-50\/65:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.65) var(--tw-gradient-to-position)}.dark\:to-lime-50\/70:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.7) var(--tw-gradient-to-position)}.dark\:to-lime-50\/75:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.75) var(--tw-gradient-to-position)}.dark\:to-lime-50\/80:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.8) var(--tw-gradient-to-position)}.dark\:to-lime-50\/85:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.85) var(--tw-gradient-to-position)}.dark\:to-lime-50\/90:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.9) var(--tw-gradient-to-position)}.dark\:to-lime-50\/95:is(.dark *){--tw-gradient-to:hsla(78,92%,95%,.95) var(--tw-gradient-to-position)}.dark\:to-lime-500:is(.dark *){--tw-gradient-to:#84cc16 var(--tw-gradient-to-position)}.dark\:to-lime-500\/0:is(.dark *){--tw-gradient-to:rgba(132,204,22,0) var(--tw-gradient-to-position)}.dark\:to-lime-500\/10:is(.dark *){--tw-gradient-to:rgba(132,204,22,.1) var(--tw-gradient-to-position)}.dark\:to-lime-500\/100:is(.dark *){--tw-gradient-to:#84cc16 var(--tw-gradient-to-position)}.dark\:to-lime-500\/15:is(.dark *){--tw-gradient-to:rgba(132,204,22,.15) var(--tw-gradient-to-position)}.dark\:to-lime-500\/20:is(.dark *){--tw-gradient-to:rgba(132,204,22,.2) var(--tw-gradient-to-position)}.dark\:to-lime-500\/25:is(.dark *){--tw-gradient-to:rgba(132,204,22,.25) var(--tw-gradient-to-position)}.dark\:to-lime-500\/30:is(.dark *){--tw-gradient-to:rgba(132,204,22,.3) var(--tw-gradient-to-position)}.dark\:to-lime-500\/35:is(.dark *){--tw-gradient-to:rgba(132,204,22,.35) var(--tw-gradient-to-position)}.dark\:to-lime-500\/40:is(.dark *){--tw-gradient-to:rgba(132,204,22,.4) var(--tw-gradient-to-position)}.dark\:to-lime-500\/45:is(.dark *){--tw-gradient-to:rgba(132,204,22,.45) var(--tw-gradient-to-position)}.dark\:to-lime-500\/5:is(.dark *){--tw-gradient-to:rgba(132,204,22,.05) var(--tw-gradient-to-position)}.dark\:to-lime-500\/50:is(.dark *){--tw-gradient-to:rgba(132,204,22,.5) var(--tw-gradient-to-position)}.dark\:to-lime-500\/55:is(.dark *){--tw-gradient-to:rgba(132,204,22,.55) var(--tw-gradient-to-position)}.dark\:to-lime-500\/60:is(.dark *){--tw-gradient-to:rgba(132,204,22,.6) var(--tw-gradient-to-position)}.dark\:to-lime-500\/65:is(.dark *){--tw-gradient-to:rgba(132,204,22,.65) var(--tw-gradient-to-position)}.dark\:to-lime-500\/70:is(.dark *){--tw-gradient-to:rgba(132,204,22,.7) var(--tw-gradient-to-position)}.dark\:to-lime-500\/75:is(.dark *){--tw-gradient-to:rgba(132,204,22,.75) var(--tw-gradient-to-position)}.dark\:to-lime-500\/80:is(.dark *){--tw-gradient-to:rgba(132,204,22,.8) var(--tw-gradient-to-position)}.dark\:to-lime-500\/85:is(.dark *){--tw-gradient-to:rgba(132,204,22,.85) var(--tw-gradient-to-position)}.dark\:to-lime-500\/90:is(.dark *){--tw-gradient-to:rgba(132,204,22,.9) var(--tw-gradient-to-position)}.dark\:to-lime-500\/95:is(.dark *){--tw-gradient-to:rgba(132,204,22,.95) var(--tw-gradient-to-position)}.dark\:to-lime-950:is(.dark *){--tw-gradient-to:#1a2e05 var(--tw-gradient-to-position)}.dark\:to-lime-950\/0:is(.dark *){--tw-gradient-to:rgba(26,46,5,0) var(--tw-gradient-to-position)}.dark\:to-lime-950\/10:is(.dark *){--tw-gradient-to:rgba(26,46,5,.1) var(--tw-gradient-to-position)}.dark\:to-lime-950\/100:is(.dark *){--tw-gradient-to:#1a2e05 var(--tw-gradient-to-position)}.dark\:to-lime-950\/15:is(.dark *){--tw-gradient-to:rgba(26,46,5,.15) var(--tw-gradient-to-position)}.dark\:to-lime-950\/20:is(.dark *){--tw-gradient-to:rgba(26,46,5,.2) var(--tw-gradient-to-position)}.dark\:to-lime-950\/25:is(.dark *){--tw-gradient-to:rgba(26,46,5,.25) var(--tw-gradient-to-position)}.dark\:to-lime-950\/30:is(.dark *){--tw-gradient-to:rgba(26,46,5,.3) var(--tw-gradient-to-position)}.dark\:to-lime-950\/35:is(.dark *){--tw-gradient-to:rgba(26,46,5,.35) var(--tw-gradient-to-position)}.dark\:to-lime-950\/40:is(.dark *){--tw-gradient-to:rgba(26,46,5,.4) var(--tw-gradient-to-position)}.dark\:to-lime-950\/45:is(.dark *){--tw-gradient-to:rgba(26,46,5,.45) var(--tw-gradient-to-position)}.dark\:to-lime-950\/5:is(.dark *){--tw-gradient-to:rgba(26,46,5,.05) var(--tw-gradient-to-position)}.dark\:to-lime-950\/50:is(.dark *){--tw-gradient-to:rgba(26,46,5,.5) var(--tw-gradient-to-position)}.dark\:to-lime-950\/55:is(.dark *){--tw-gradient-to:rgba(26,46,5,.55) var(--tw-gradient-to-position)}.dark\:to-lime-950\/60:is(.dark *){--tw-gradient-to:rgba(26,46,5,.6) var(--tw-gradient-to-position)}.dark\:to-lime-950\/65:is(.dark *){--tw-gradient-to:rgba(26,46,5,.65) var(--tw-gradient-to-position)}.dark\:to-lime-950\/70:is(.dark *){--tw-gradient-to:rgba(26,46,5,.7) var(--tw-gradient-to-position)}.dark\:to-lime-950\/75:is(.dark *){--tw-gradient-to:rgba(26,46,5,.75) var(--tw-gradient-to-position)}.dark\:to-lime-950\/80:is(.dark *){--tw-gradient-to:rgba(26,46,5,.8) var(--tw-gradient-to-position)}.dark\:to-lime-950\/85:is(.dark *){--tw-gradient-to:rgba(26,46,5,.85) var(--tw-gradient-to-position)}.dark\:to-lime-950\/90:is(.dark *){--tw-gradient-to:rgba(26,46,5,.9) var(--tw-gradient-to-position)}.dark\:to-lime-950\/95:is(.dark *){--tw-gradient-to:rgba(26,46,5,.95) var(--tw-gradient-to-position)}.dark\:to-neutral-100:is(.dark *){--tw-gradient-to:#f5f5f5 var(--tw-gradient-to-position)}.dark\:to-neutral-100\/0:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,0) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/10:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.1) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/100:is(.dark *){--tw-gradient-to:#f5f5f5 var(--tw-gradient-to-position)}.dark\:to-neutral-100\/15:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.15) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/20:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.2) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/25:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.25) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/30:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.3) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/35:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.35) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/40:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.4) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/45:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.45) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/5:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.05) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/50:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.5) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/55:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.55) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/60:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.6) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/65:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.65) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/70:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.7) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/75:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.75) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/80:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.8) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/85:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.85) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/90:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.9) var(--tw-gradient-to-position)}.dark\:to-neutral-100\/95:is(.dark *){--tw-gradient-to:hsla(0,0%,96%,.95) var(--tw-gradient-to-position)}.dark\:to-neutral-200:is(.dark *){--tw-gradient-to:#e5e5e5 var(--tw-gradient-to-position)}.dark\:to-neutral-200\/0:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,0) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/10:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.1) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/100:is(.dark *){--tw-gradient-to:#e5e5e5 var(--tw-gradient-to-position)}.dark\:to-neutral-200\/15:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.15) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/20:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.2) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/25:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.25) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/30:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.3) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/35:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.35) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/40:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.4) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/45:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.45) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/5:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.05) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/50:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.5) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/55:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.55) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/60:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.6) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/65:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.65) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/70:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.7) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/75:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.75) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/80:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.8) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/85:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.85) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/90:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.9) var(--tw-gradient-to-position)}.dark\:to-neutral-200\/95:is(.dark *){--tw-gradient-to:hsla(0,0%,90%,.95) var(--tw-gradient-to-position)}.dark\:to-neutral-50:is(.dark *){--tw-gradient-to:#fafafa var(--tw-gradient-to-position)}.dark\:to-neutral-50\/0:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,0) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/10:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.1) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/100:is(.dark *){--tw-gradient-to:#fafafa var(--tw-gradient-to-position)}.dark\:to-neutral-50\/15:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.15) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/20:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.2) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/25:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.25) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/30:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.3) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/35:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.35) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/40:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.4) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/45:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.45) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/5:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.05) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/50:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.5) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/55:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.55) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/60:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.6) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/65:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.65) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/70:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.7) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/75:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.75) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/80:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.8) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/85:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.85) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/90:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.9) var(--tw-gradient-to-position)}.dark\:to-neutral-50\/95:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.95) var(--tw-gradient-to-position)}.dark\:to-neutral-500:is(.dark *){--tw-gradient-to:#737373 var(--tw-gradient-to-position)}.dark\:to-neutral-500\/0:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,0) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/10:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.1) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/100:is(.dark *){--tw-gradient-to:#737373 var(--tw-gradient-to-position)}.dark\:to-neutral-500\/15:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.15) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/20:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.2) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/25:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.25) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/30:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.3) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/35:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.35) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/40:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.4) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/45:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.45) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/5:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.05) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/50:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.5) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/55:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.55) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/60:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.6) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/65:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.65) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/70:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.7) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/75:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.75) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/80:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.8) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/85:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.85) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/90:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.9) var(--tw-gradient-to-position)}.dark\:to-neutral-500\/95:is(.dark *){--tw-gradient-to:hsla(0,0%,45%,.95) var(--tw-gradient-to-position)}.dark\:to-neutral-950:is(.dark *){--tw-gradient-to:#0a0a0a var(--tw-gradient-to-position)}.dark\:to-neutral-950\/0:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,0) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/10:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.1) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/100:is(.dark *){--tw-gradient-to:#0a0a0a var(--tw-gradient-to-position)}.dark\:to-neutral-950\/15:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.15) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/20:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.2) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/25:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.25) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/30:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.3) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/35:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.35) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/40:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.4) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/45:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.45) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/5:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.05) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/50:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.5) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/55:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.55) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/60:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.6) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/65:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.65) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/70:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.7) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/75:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.75) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/80:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.8) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/85:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.85) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/90:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.9) var(--tw-gradient-to-position)}.dark\:to-neutral-950\/95:is(.dark *){--tw-gradient-to:hsla(0,0%,4%,.95) var(--tw-gradient-to-position)}.dark\:to-orange-100:is(.dark *){--tw-gradient-to:#ffedd5 var(--tw-gradient-to-position)}.dark\:to-orange-100\/0:is(.dark *){--tw-gradient-to:rgba(255,237,213,0) var(--tw-gradient-to-position)}.dark\:to-orange-100\/10:is(.dark *){--tw-gradient-to:rgba(255,237,213,.1) var(--tw-gradient-to-position)}.dark\:to-orange-100\/100:is(.dark *){--tw-gradient-to:#ffedd5 var(--tw-gradient-to-position)}.dark\:to-orange-100\/15:is(.dark *){--tw-gradient-to:rgba(255,237,213,.15) var(--tw-gradient-to-position)}.dark\:to-orange-100\/20:is(.dark *){--tw-gradient-to:rgba(255,237,213,.2) var(--tw-gradient-to-position)}.dark\:to-orange-100\/25:is(.dark *){--tw-gradient-to:rgba(255,237,213,.25) var(--tw-gradient-to-position)}.dark\:to-orange-100\/30:is(.dark *){--tw-gradient-to:rgba(255,237,213,.3) var(--tw-gradient-to-position)}.dark\:to-orange-100\/35:is(.dark *){--tw-gradient-to:rgba(255,237,213,.35) var(--tw-gradient-to-position)}.dark\:to-orange-100\/40:is(.dark *){--tw-gradient-to:rgba(255,237,213,.4) var(--tw-gradient-to-position)}.dark\:to-orange-100\/45:is(.dark *){--tw-gradient-to:rgba(255,237,213,.45) var(--tw-gradient-to-position)}.dark\:to-orange-100\/5:is(.dark *){--tw-gradient-to:rgba(255,237,213,.05) var(--tw-gradient-to-position)}.dark\:to-orange-100\/50:is(.dark *){--tw-gradient-to:rgba(255,237,213,.5) var(--tw-gradient-to-position)}.dark\:to-orange-100\/55:is(.dark *){--tw-gradient-to:rgba(255,237,213,.55) var(--tw-gradient-to-position)}.dark\:to-orange-100\/60:is(.dark *){--tw-gradient-to:rgba(255,237,213,.6) var(--tw-gradient-to-position)}.dark\:to-orange-100\/65:is(.dark *){--tw-gradient-to:rgba(255,237,213,.65) var(--tw-gradient-to-position)}.dark\:to-orange-100\/70:is(.dark *){--tw-gradient-to:rgba(255,237,213,.7) var(--tw-gradient-to-position)}.dark\:to-orange-100\/75:is(.dark *){--tw-gradient-to:rgba(255,237,213,.75) var(--tw-gradient-to-position)}.dark\:to-orange-100\/80:is(.dark *){--tw-gradient-to:rgba(255,237,213,.8) var(--tw-gradient-to-position)}.dark\:to-orange-100\/85:is(.dark *){--tw-gradient-to:rgba(255,237,213,.85) var(--tw-gradient-to-position)}.dark\:to-orange-100\/90:is(.dark *){--tw-gradient-to:rgba(255,237,213,.9) var(--tw-gradient-to-position)}.dark\:to-orange-100\/95:is(.dark *){--tw-gradient-to:rgba(255,237,213,.95) var(--tw-gradient-to-position)}.dark\:to-orange-200:is(.dark *){--tw-gradient-to:#fed7aa var(--tw-gradient-to-position)}.dark\:to-orange-200\/0:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,0) var(--tw-gradient-to-position)}.dark\:to-orange-200\/10:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.1) var(--tw-gradient-to-position)}.dark\:to-orange-200\/100:is(.dark *){--tw-gradient-to:#fed7aa var(--tw-gradient-to-position)}.dark\:to-orange-200\/15:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.15) var(--tw-gradient-to-position)}.dark\:to-orange-200\/20:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.2) var(--tw-gradient-to-position)}.dark\:to-orange-200\/25:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.25) var(--tw-gradient-to-position)}.dark\:to-orange-200\/30:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.3) var(--tw-gradient-to-position)}.dark\:to-orange-200\/35:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.35) var(--tw-gradient-to-position)}.dark\:to-orange-200\/40:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.4) var(--tw-gradient-to-position)}.dark\:to-orange-200\/45:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.45) var(--tw-gradient-to-position)}.dark\:to-orange-200\/5:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.05) var(--tw-gradient-to-position)}.dark\:to-orange-200\/50:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.5) var(--tw-gradient-to-position)}.dark\:to-orange-200\/55:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.55) var(--tw-gradient-to-position)}.dark\:to-orange-200\/60:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.6) var(--tw-gradient-to-position)}.dark\:to-orange-200\/65:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.65) var(--tw-gradient-to-position)}.dark\:to-orange-200\/70:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.7) var(--tw-gradient-to-position)}.dark\:to-orange-200\/75:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.75) var(--tw-gradient-to-position)}.dark\:to-orange-200\/80:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.8) var(--tw-gradient-to-position)}.dark\:to-orange-200\/85:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.85) var(--tw-gradient-to-position)}.dark\:to-orange-200\/90:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.9) var(--tw-gradient-to-position)}.dark\:to-orange-200\/95:is(.dark *){--tw-gradient-to:hsla(32,98%,83%,.95) var(--tw-gradient-to-position)}.dark\:to-orange-50:is(.dark *){--tw-gradient-to:#fff7ed var(--tw-gradient-to-position)}.dark\:to-orange-50\/0:is(.dark *){--tw-gradient-to:rgba(255,247,237,0) var(--tw-gradient-to-position)}.dark\:to-orange-50\/10:is(.dark *){--tw-gradient-to:rgba(255,247,237,.1) var(--tw-gradient-to-position)}.dark\:to-orange-50\/100:is(.dark *){--tw-gradient-to:#fff7ed var(--tw-gradient-to-position)}.dark\:to-orange-50\/15:is(.dark *){--tw-gradient-to:rgba(255,247,237,.15) var(--tw-gradient-to-position)}.dark\:to-orange-50\/20:is(.dark *){--tw-gradient-to:rgba(255,247,237,.2) var(--tw-gradient-to-position)}.dark\:to-orange-50\/25:is(.dark *){--tw-gradient-to:rgba(255,247,237,.25) var(--tw-gradient-to-position)}.dark\:to-orange-50\/30:is(.dark *){--tw-gradient-to:rgba(255,247,237,.3) var(--tw-gradient-to-position)}.dark\:to-orange-50\/35:is(.dark *){--tw-gradient-to:rgba(255,247,237,.35) var(--tw-gradient-to-position)}.dark\:to-orange-50\/40:is(.dark *){--tw-gradient-to:rgba(255,247,237,.4) var(--tw-gradient-to-position)}.dark\:to-orange-50\/45:is(.dark *){--tw-gradient-to:rgba(255,247,237,.45) var(--tw-gradient-to-position)}.dark\:to-orange-50\/5:is(.dark *){--tw-gradient-to:rgba(255,247,237,.05) var(--tw-gradient-to-position)}.dark\:to-orange-50\/50:is(.dark *){--tw-gradient-to:rgba(255,247,237,.5) var(--tw-gradient-to-position)}.dark\:to-orange-50\/55:is(.dark *){--tw-gradient-to:rgba(255,247,237,.55) var(--tw-gradient-to-position)}.dark\:to-orange-50\/60:is(.dark *){--tw-gradient-to:rgba(255,247,237,.6) var(--tw-gradient-to-position)}.dark\:to-orange-50\/65:is(.dark *){--tw-gradient-to:rgba(255,247,237,.65) var(--tw-gradient-to-position)}.dark\:to-orange-50\/70:is(.dark *){--tw-gradient-to:rgba(255,247,237,.7) var(--tw-gradient-to-position)}.dark\:to-orange-50\/75:is(.dark *){--tw-gradient-to:rgba(255,247,237,.75) var(--tw-gradient-to-position)}.dark\:to-orange-50\/80:is(.dark *){--tw-gradient-to:rgba(255,247,237,.8) var(--tw-gradient-to-position)}.dark\:to-orange-50\/85:is(.dark *){--tw-gradient-to:rgba(255,247,237,.85) var(--tw-gradient-to-position)}.dark\:to-orange-50\/90:is(.dark *){--tw-gradient-to:rgba(255,247,237,.9) var(--tw-gradient-to-position)}.dark\:to-orange-50\/95:is(.dark *){--tw-gradient-to:rgba(255,247,237,.95) var(--tw-gradient-to-position)}.dark\:to-orange-500:is(.dark *){--tw-gradient-to:#f97316 var(--tw-gradient-to-position)}.dark\:to-orange-500\/0:is(.dark *){--tw-gradient-to:rgba(249,115,22,0) var(--tw-gradient-to-position)}.dark\:to-orange-500\/10:is(.dark *){--tw-gradient-to:rgba(249,115,22,.1) var(--tw-gradient-to-position)}.dark\:to-orange-500\/100:is(.dark *){--tw-gradient-to:#f97316 var(--tw-gradient-to-position)}.dark\:to-orange-500\/15:is(.dark *){--tw-gradient-to:rgba(249,115,22,.15) var(--tw-gradient-to-position)}.dark\:to-orange-500\/20:is(.dark *){--tw-gradient-to:rgba(249,115,22,.2) var(--tw-gradient-to-position)}.dark\:to-orange-500\/25:is(.dark *){--tw-gradient-to:rgba(249,115,22,.25) var(--tw-gradient-to-position)}.dark\:to-orange-500\/30:is(.dark *){--tw-gradient-to:rgba(249,115,22,.3) var(--tw-gradient-to-position)}.dark\:to-orange-500\/35:is(.dark *){--tw-gradient-to:rgba(249,115,22,.35) var(--tw-gradient-to-position)}.dark\:to-orange-500\/40:is(.dark *){--tw-gradient-to:rgba(249,115,22,.4) var(--tw-gradient-to-position)}.dark\:to-orange-500\/45:is(.dark *){--tw-gradient-to:rgba(249,115,22,.45) var(--tw-gradient-to-position)}.dark\:to-orange-500\/5:is(.dark *){--tw-gradient-to:rgba(249,115,22,.05) var(--tw-gradient-to-position)}.dark\:to-orange-500\/50:is(.dark *){--tw-gradient-to:rgba(249,115,22,.5) var(--tw-gradient-to-position)}.dark\:to-orange-500\/55:is(.dark *){--tw-gradient-to:rgba(249,115,22,.55) var(--tw-gradient-to-position)}.dark\:to-orange-500\/60:is(.dark *){--tw-gradient-to:rgba(249,115,22,.6) var(--tw-gradient-to-position)}.dark\:to-orange-500\/65:is(.dark *){--tw-gradient-to:rgba(249,115,22,.65) var(--tw-gradient-to-position)}.dark\:to-orange-500\/70:is(.dark *){--tw-gradient-to:rgba(249,115,22,.7) var(--tw-gradient-to-position)}.dark\:to-orange-500\/75:is(.dark *){--tw-gradient-to:rgba(249,115,22,.75) var(--tw-gradient-to-position)}.dark\:to-orange-500\/80:is(.dark *){--tw-gradient-to:rgba(249,115,22,.8) var(--tw-gradient-to-position)}.dark\:to-orange-500\/85:is(.dark *){--tw-gradient-to:rgba(249,115,22,.85) var(--tw-gradient-to-position)}.dark\:to-orange-500\/90:is(.dark *){--tw-gradient-to:rgba(249,115,22,.9) var(--tw-gradient-to-position)}.dark\:to-orange-500\/95:is(.dark *){--tw-gradient-to:rgba(249,115,22,.95) var(--tw-gradient-to-position)}.dark\:to-orange-950:is(.dark *){--tw-gradient-to:#431407 var(--tw-gradient-to-position)}.dark\:to-orange-950\/0:is(.dark *){--tw-gradient-to:rgba(67,20,7,0) var(--tw-gradient-to-position)}.dark\:to-orange-950\/10:is(.dark *){--tw-gradient-to:rgba(67,20,7,.1) var(--tw-gradient-to-position)}.dark\:to-orange-950\/100:is(.dark *){--tw-gradient-to:#431407 var(--tw-gradient-to-position)}.dark\:to-orange-950\/15:is(.dark *){--tw-gradient-to:rgba(67,20,7,.15) var(--tw-gradient-to-position)}.dark\:to-orange-950\/20:is(.dark *){--tw-gradient-to:rgba(67,20,7,.2) var(--tw-gradient-to-position)}.dark\:to-orange-950\/25:is(.dark *){--tw-gradient-to:rgba(67,20,7,.25) var(--tw-gradient-to-position)}.dark\:to-orange-950\/30:is(.dark *){--tw-gradient-to:rgba(67,20,7,.3) var(--tw-gradient-to-position)}.dark\:to-orange-950\/35:is(.dark *){--tw-gradient-to:rgba(67,20,7,.35) var(--tw-gradient-to-position)}.dark\:to-orange-950\/40:is(.dark *){--tw-gradient-to:rgba(67,20,7,.4) var(--tw-gradient-to-position)}.dark\:to-orange-950\/45:is(.dark *){--tw-gradient-to:rgba(67,20,7,.45) var(--tw-gradient-to-position)}.dark\:to-orange-950\/5:is(.dark *){--tw-gradient-to:rgba(67,20,7,.05) var(--tw-gradient-to-position)}.dark\:to-orange-950\/50:is(.dark *){--tw-gradient-to:rgba(67,20,7,.5) var(--tw-gradient-to-position)}.dark\:to-orange-950\/55:is(.dark *){--tw-gradient-to:rgba(67,20,7,.55) var(--tw-gradient-to-position)}.dark\:to-orange-950\/60:is(.dark *){--tw-gradient-to:rgba(67,20,7,.6) var(--tw-gradient-to-position)}.dark\:to-orange-950\/65:is(.dark *){--tw-gradient-to:rgba(67,20,7,.65) var(--tw-gradient-to-position)}.dark\:to-orange-950\/70:is(.dark *){--tw-gradient-to:rgba(67,20,7,.7) var(--tw-gradient-to-position)}.dark\:to-orange-950\/75:is(.dark *){--tw-gradient-to:rgba(67,20,7,.75) var(--tw-gradient-to-position)}.dark\:to-orange-950\/80:is(.dark *){--tw-gradient-to:rgba(67,20,7,.8) var(--tw-gradient-to-position)}.dark\:to-orange-950\/85:is(.dark *){--tw-gradient-to:rgba(67,20,7,.85) var(--tw-gradient-to-position)}.dark\:to-orange-950\/90:is(.dark *){--tw-gradient-to:rgba(67,20,7,.9) var(--tw-gradient-to-position)}.dark\:to-orange-950\/95:is(.dark *){--tw-gradient-to:rgba(67,20,7,.95) var(--tw-gradient-to-position)}.dark\:to-pink-100:is(.dark *){--tw-gradient-to:#fce7f3 var(--tw-gradient-to-position)}.dark\:to-pink-100\/0:is(.dark *){--tw-gradient-to:rgba(252,231,243,0) var(--tw-gradient-to-position)}.dark\:to-pink-100\/10:is(.dark *){--tw-gradient-to:rgba(252,231,243,.1) var(--tw-gradient-to-position)}.dark\:to-pink-100\/100:is(.dark *){--tw-gradient-to:#fce7f3 var(--tw-gradient-to-position)}.dark\:to-pink-100\/15:is(.dark *){--tw-gradient-to:rgba(252,231,243,.15) var(--tw-gradient-to-position)}.dark\:to-pink-100\/20:is(.dark *){--tw-gradient-to:rgba(252,231,243,.2) var(--tw-gradient-to-position)}.dark\:to-pink-100\/25:is(.dark *){--tw-gradient-to:rgba(252,231,243,.25) var(--tw-gradient-to-position)}.dark\:to-pink-100\/30:is(.dark *){--tw-gradient-to:rgba(252,231,243,.3) var(--tw-gradient-to-position)}.dark\:to-pink-100\/35:is(.dark *){--tw-gradient-to:rgba(252,231,243,.35) var(--tw-gradient-to-position)}.dark\:to-pink-100\/40:is(.dark *){--tw-gradient-to:rgba(252,231,243,.4) var(--tw-gradient-to-position)}.dark\:to-pink-100\/45:is(.dark *){--tw-gradient-to:rgba(252,231,243,.45) var(--tw-gradient-to-position)}.dark\:to-pink-100\/5:is(.dark *){--tw-gradient-to:rgba(252,231,243,.05) var(--tw-gradient-to-position)}.dark\:to-pink-100\/50:is(.dark *){--tw-gradient-to:rgba(252,231,243,.5) var(--tw-gradient-to-position)}.dark\:to-pink-100\/55:is(.dark *){--tw-gradient-to:rgba(252,231,243,.55) var(--tw-gradient-to-position)}.dark\:to-pink-100\/60:is(.dark *){--tw-gradient-to:rgba(252,231,243,.6) var(--tw-gradient-to-position)}.dark\:to-pink-100\/65:is(.dark *){--tw-gradient-to:rgba(252,231,243,.65) var(--tw-gradient-to-position)}.dark\:to-pink-100\/70:is(.dark *){--tw-gradient-to:rgba(252,231,243,.7) var(--tw-gradient-to-position)}.dark\:to-pink-100\/75:is(.dark *){--tw-gradient-to:rgba(252,231,243,.75) var(--tw-gradient-to-position)}.dark\:to-pink-100\/80:is(.dark *){--tw-gradient-to:rgba(252,231,243,.8) var(--tw-gradient-to-position)}.dark\:to-pink-100\/85:is(.dark *){--tw-gradient-to:rgba(252,231,243,.85) var(--tw-gradient-to-position)}.dark\:to-pink-100\/90:is(.dark *){--tw-gradient-to:rgba(252,231,243,.9) var(--tw-gradient-to-position)}.dark\:to-pink-100\/95:is(.dark *){--tw-gradient-to:rgba(252,231,243,.95) var(--tw-gradient-to-position)}.dark\:to-pink-200:is(.dark *){--tw-gradient-to:#fbcfe8 var(--tw-gradient-to-position)}.dark\:to-pink-200\/0:is(.dark *){--tw-gradient-to:rgba(251,207,232,0) var(--tw-gradient-to-position)}.dark\:to-pink-200\/10:is(.dark *){--tw-gradient-to:rgba(251,207,232,.1) var(--tw-gradient-to-position)}.dark\:to-pink-200\/100:is(.dark *){--tw-gradient-to:#fbcfe8 var(--tw-gradient-to-position)}.dark\:to-pink-200\/15:is(.dark *){--tw-gradient-to:rgba(251,207,232,.15) var(--tw-gradient-to-position)}.dark\:to-pink-200\/20:is(.dark *){--tw-gradient-to:rgba(251,207,232,.2) var(--tw-gradient-to-position)}.dark\:to-pink-200\/25:is(.dark *){--tw-gradient-to:rgba(251,207,232,.25) var(--tw-gradient-to-position)}.dark\:to-pink-200\/30:is(.dark *){--tw-gradient-to:rgba(251,207,232,.3) var(--tw-gradient-to-position)}.dark\:to-pink-200\/35:is(.dark *){--tw-gradient-to:rgba(251,207,232,.35) var(--tw-gradient-to-position)}.dark\:to-pink-200\/40:is(.dark *){--tw-gradient-to:rgba(251,207,232,.4) var(--tw-gradient-to-position)}.dark\:to-pink-200\/45:is(.dark *){--tw-gradient-to:rgba(251,207,232,.45) var(--tw-gradient-to-position)}.dark\:to-pink-200\/5:is(.dark *){--tw-gradient-to:rgba(251,207,232,.05) var(--tw-gradient-to-position)}.dark\:to-pink-200\/50:is(.dark *){--tw-gradient-to:rgba(251,207,232,.5) var(--tw-gradient-to-position)}.dark\:to-pink-200\/55:is(.dark *){--tw-gradient-to:rgba(251,207,232,.55) var(--tw-gradient-to-position)}.dark\:to-pink-200\/60:is(.dark *){--tw-gradient-to:rgba(251,207,232,.6) var(--tw-gradient-to-position)}.dark\:to-pink-200\/65:is(.dark *){--tw-gradient-to:rgba(251,207,232,.65) var(--tw-gradient-to-position)}.dark\:to-pink-200\/70:is(.dark *){--tw-gradient-to:rgba(251,207,232,.7) var(--tw-gradient-to-position)}.dark\:to-pink-200\/75:is(.dark *){--tw-gradient-to:rgba(251,207,232,.75) var(--tw-gradient-to-position)}.dark\:to-pink-200\/80:is(.dark *){--tw-gradient-to:rgba(251,207,232,.8) var(--tw-gradient-to-position)}.dark\:to-pink-200\/85:is(.dark *){--tw-gradient-to:rgba(251,207,232,.85) var(--tw-gradient-to-position)}.dark\:to-pink-200\/90:is(.dark *){--tw-gradient-to:rgba(251,207,232,.9) var(--tw-gradient-to-position)}.dark\:to-pink-200\/95:is(.dark *){--tw-gradient-to:rgba(251,207,232,.95) var(--tw-gradient-to-position)}.dark\:to-pink-50:is(.dark *){--tw-gradient-to:#fdf2f8 var(--tw-gradient-to-position)}.dark\:to-pink-50\/0:is(.dark *){--tw-gradient-to:rgba(253,242,248,0) var(--tw-gradient-to-position)}.dark\:to-pink-50\/10:is(.dark *){--tw-gradient-to:rgba(253,242,248,.1) var(--tw-gradient-to-position)}.dark\:to-pink-50\/100:is(.dark *){--tw-gradient-to:#fdf2f8 var(--tw-gradient-to-position)}.dark\:to-pink-50\/15:is(.dark *){--tw-gradient-to:rgba(253,242,248,.15) var(--tw-gradient-to-position)}.dark\:to-pink-50\/20:is(.dark *){--tw-gradient-to:rgba(253,242,248,.2) var(--tw-gradient-to-position)}.dark\:to-pink-50\/25:is(.dark *){--tw-gradient-to:rgba(253,242,248,.25) var(--tw-gradient-to-position)}.dark\:to-pink-50\/30:is(.dark *){--tw-gradient-to:rgba(253,242,248,.3) var(--tw-gradient-to-position)}.dark\:to-pink-50\/35:is(.dark *){--tw-gradient-to:rgba(253,242,248,.35) var(--tw-gradient-to-position)}.dark\:to-pink-50\/40:is(.dark *){--tw-gradient-to:rgba(253,242,248,.4) var(--tw-gradient-to-position)}.dark\:to-pink-50\/45:is(.dark *){--tw-gradient-to:rgba(253,242,248,.45) var(--tw-gradient-to-position)}.dark\:to-pink-50\/5:is(.dark *){--tw-gradient-to:rgba(253,242,248,.05) var(--tw-gradient-to-position)}.dark\:to-pink-50\/50:is(.dark *){--tw-gradient-to:rgba(253,242,248,.5) var(--tw-gradient-to-position)}.dark\:to-pink-50\/55:is(.dark *){--tw-gradient-to:rgba(253,242,248,.55) var(--tw-gradient-to-position)}.dark\:to-pink-50\/60:is(.dark *){--tw-gradient-to:rgba(253,242,248,.6) var(--tw-gradient-to-position)}.dark\:to-pink-50\/65:is(.dark *){--tw-gradient-to:rgba(253,242,248,.65) var(--tw-gradient-to-position)}.dark\:to-pink-50\/70:is(.dark *){--tw-gradient-to:rgba(253,242,248,.7) var(--tw-gradient-to-position)}.dark\:to-pink-50\/75:is(.dark *){--tw-gradient-to:rgba(253,242,248,.75) var(--tw-gradient-to-position)}.dark\:to-pink-50\/80:is(.dark *){--tw-gradient-to:rgba(253,242,248,.8) var(--tw-gradient-to-position)}.dark\:to-pink-50\/85:is(.dark *){--tw-gradient-to:rgba(253,242,248,.85) var(--tw-gradient-to-position)}.dark\:to-pink-50\/90:is(.dark *){--tw-gradient-to:rgba(253,242,248,.9) var(--tw-gradient-to-position)}.dark\:to-pink-50\/95:is(.dark *){--tw-gradient-to:rgba(253,242,248,.95) var(--tw-gradient-to-position)}.dark\:to-pink-500:is(.dark *){--tw-gradient-to:#ec4899 var(--tw-gradient-to-position)}.dark\:to-pink-500\/0:is(.dark *){--tw-gradient-to:rgba(236,72,153,0) var(--tw-gradient-to-position)}.dark\:to-pink-500\/10:is(.dark *){--tw-gradient-to:rgba(236,72,153,.1) var(--tw-gradient-to-position)}.dark\:to-pink-500\/100:is(.dark *){--tw-gradient-to:#ec4899 var(--tw-gradient-to-position)}.dark\:to-pink-500\/15:is(.dark *){--tw-gradient-to:rgba(236,72,153,.15) var(--tw-gradient-to-position)}.dark\:to-pink-500\/20:is(.dark *){--tw-gradient-to:rgba(236,72,153,.2) var(--tw-gradient-to-position)}.dark\:to-pink-500\/25:is(.dark *){--tw-gradient-to:rgba(236,72,153,.25) var(--tw-gradient-to-position)}.dark\:to-pink-500\/30:is(.dark *){--tw-gradient-to:rgba(236,72,153,.3) var(--tw-gradient-to-position)}.dark\:to-pink-500\/35:is(.dark *){--tw-gradient-to:rgba(236,72,153,.35) var(--tw-gradient-to-position)}.dark\:to-pink-500\/40:is(.dark *){--tw-gradient-to:rgba(236,72,153,.4) var(--tw-gradient-to-position)}.dark\:to-pink-500\/45:is(.dark *){--tw-gradient-to:rgba(236,72,153,.45) var(--tw-gradient-to-position)}.dark\:to-pink-500\/5:is(.dark *){--tw-gradient-to:rgba(236,72,153,.05) var(--tw-gradient-to-position)}.dark\:to-pink-500\/50:is(.dark *){--tw-gradient-to:rgba(236,72,153,.5) var(--tw-gradient-to-position)}.dark\:to-pink-500\/55:is(.dark *){--tw-gradient-to:rgba(236,72,153,.55) var(--tw-gradient-to-position)}.dark\:to-pink-500\/60:is(.dark *){--tw-gradient-to:rgba(236,72,153,.6) var(--tw-gradient-to-position)}.dark\:to-pink-500\/65:is(.dark *){--tw-gradient-to:rgba(236,72,153,.65) var(--tw-gradient-to-position)}.dark\:to-pink-500\/70:is(.dark *){--tw-gradient-to:rgba(236,72,153,.7) var(--tw-gradient-to-position)}.dark\:to-pink-500\/75:is(.dark *){--tw-gradient-to:rgba(236,72,153,.75) var(--tw-gradient-to-position)}.dark\:to-pink-500\/80:is(.dark *){--tw-gradient-to:rgba(236,72,153,.8) var(--tw-gradient-to-position)}.dark\:to-pink-500\/85:is(.dark *){--tw-gradient-to:rgba(236,72,153,.85) var(--tw-gradient-to-position)}.dark\:to-pink-500\/90:is(.dark *){--tw-gradient-to:rgba(236,72,153,.9) var(--tw-gradient-to-position)}.dark\:to-pink-500\/95:is(.dark *){--tw-gradient-to:rgba(236,72,153,.95) var(--tw-gradient-to-position)}.dark\:to-pink-950:is(.dark *){--tw-gradient-to:#500724 var(--tw-gradient-to-position)}.dark\:to-pink-950\/0:is(.dark *){--tw-gradient-to:rgba(80,7,36,0) var(--tw-gradient-to-position)}.dark\:to-pink-950\/10:is(.dark *){--tw-gradient-to:rgba(80,7,36,.1) var(--tw-gradient-to-position)}.dark\:to-pink-950\/100:is(.dark *){--tw-gradient-to:#500724 var(--tw-gradient-to-position)}.dark\:to-pink-950\/15:is(.dark *){--tw-gradient-to:rgba(80,7,36,.15) var(--tw-gradient-to-position)}.dark\:to-pink-950\/20:is(.dark *){--tw-gradient-to:rgba(80,7,36,.2) var(--tw-gradient-to-position)}.dark\:to-pink-950\/25:is(.dark *){--tw-gradient-to:rgba(80,7,36,.25) var(--tw-gradient-to-position)}.dark\:to-pink-950\/30:is(.dark *){--tw-gradient-to:rgba(80,7,36,.3) var(--tw-gradient-to-position)}.dark\:to-pink-950\/35:is(.dark *){--tw-gradient-to:rgba(80,7,36,.35) var(--tw-gradient-to-position)}.dark\:to-pink-950\/40:is(.dark *){--tw-gradient-to:rgba(80,7,36,.4) var(--tw-gradient-to-position)}.dark\:to-pink-950\/45:is(.dark *){--tw-gradient-to:rgba(80,7,36,.45) var(--tw-gradient-to-position)}.dark\:to-pink-950\/5:is(.dark *){--tw-gradient-to:rgba(80,7,36,.05) var(--tw-gradient-to-position)}.dark\:to-pink-950\/50:is(.dark *){--tw-gradient-to:rgba(80,7,36,.5) var(--tw-gradient-to-position)}.dark\:to-pink-950\/55:is(.dark *){--tw-gradient-to:rgba(80,7,36,.55) var(--tw-gradient-to-position)}.dark\:to-pink-950\/60:is(.dark *){--tw-gradient-to:rgba(80,7,36,.6) var(--tw-gradient-to-position)}.dark\:to-pink-950\/65:is(.dark *){--tw-gradient-to:rgba(80,7,36,.65) var(--tw-gradient-to-position)}.dark\:to-pink-950\/70:is(.dark *){--tw-gradient-to:rgba(80,7,36,.7) var(--tw-gradient-to-position)}.dark\:to-pink-950\/75:is(.dark *){--tw-gradient-to:rgba(80,7,36,.75) var(--tw-gradient-to-position)}.dark\:to-pink-950\/80:is(.dark *){--tw-gradient-to:rgba(80,7,36,.8) var(--tw-gradient-to-position)}.dark\:to-pink-950\/85:is(.dark *){--tw-gradient-to:rgba(80,7,36,.85) var(--tw-gradient-to-position)}.dark\:to-pink-950\/90:is(.dark *){--tw-gradient-to:rgba(80,7,36,.9) var(--tw-gradient-to-position)}.dark\:to-pink-950\/95:is(.dark *){--tw-gradient-to:rgba(80,7,36,.95) var(--tw-gradient-to-position)}.dark\:to-purple-100:is(.dark *){--tw-gradient-to:#f3e8ff var(--tw-gradient-to-position)}.dark\:to-purple-100\/0:is(.dark *){--tw-gradient-to:rgba(243,232,255,0) var(--tw-gradient-to-position)}.dark\:to-purple-100\/10:is(.dark *){--tw-gradient-to:rgba(243,232,255,.1) var(--tw-gradient-to-position)}.dark\:to-purple-100\/100:is(.dark *){--tw-gradient-to:#f3e8ff var(--tw-gradient-to-position)}.dark\:to-purple-100\/15:is(.dark *){--tw-gradient-to:rgba(243,232,255,.15) var(--tw-gradient-to-position)}.dark\:to-purple-100\/20:is(.dark *){--tw-gradient-to:rgba(243,232,255,.2) var(--tw-gradient-to-position)}.dark\:to-purple-100\/25:is(.dark *){--tw-gradient-to:rgba(243,232,255,.25) var(--tw-gradient-to-position)}.dark\:to-purple-100\/30:is(.dark *){--tw-gradient-to:rgba(243,232,255,.3) var(--tw-gradient-to-position)}.dark\:to-purple-100\/35:is(.dark *){--tw-gradient-to:rgba(243,232,255,.35) var(--tw-gradient-to-position)}.dark\:to-purple-100\/40:is(.dark *){--tw-gradient-to:rgba(243,232,255,.4) var(--tw-gradient-to-position)}.dark\:to-purple-100\/45:is(.dark *){--tw-gradient-to:rgba(243,232,255,.45) var(--tw-gradient-to-position)}.dark\:to-purple-100\/5:is(.dark *){--tw-gradient-to:rgba(243,232,255,.05) var(--tw-gradient-to-position)}.dark\:to-purple-100\/50:is(.dark *){--tw-gradient-to:rgba(243,232,255,.5) var(--tw-gradient-to-position)}.dark\:to-purple-100\/55:is(.dark *){--tw-gradient-to:rgba(243,232,255,.55) var(--tw-gradient-to-position)}.dark\:to-purple-100\/60:is(.dark *){--tw-gradient-to:rgba(243,232,255,.6) var(--tw-gradient-to-position)}.dark\:to-purple-100\/65:is(.dark *){--tw-gradient-to:rgba(243,232,255,.65) var(--tw-gradient-to-position)}.dark\:to-purple-100\/70:is(.dark *){--tw-gradient-to:rgba(243,232,255,.7) var(--tw-gradient-to-position)}.dark\:to-purple-100\/75:is(.dark *){--tw-gradient-to:rgba(243,232,255,.75) var(--tw-gradient-to-position)}.dark\:to-purple-100\/80:is(.dark *){--tw-gradient-to:rgba(243,232,255,.8) var(--tw-gradient-to-position)}.dark\:to-purple-100\/85:is(.dark *){--tw-gradient-to:rgba(243,232,255,.85) var(--tw-gradient-to-position)}.dark\:to-purple-100\/90:is(.dark *){--tw-gradient-to:rgba(243,232,255,.9) var(--tw-gradient-to-position)}.dark\:to-purple-100\/95:is(.dark *){--tw-gradient-to:rgba(243,232,255,.95) var(--tw-gradient-to-position)}.dark\:to-purple-200:is(.dark *){--tw-gradient-to:#e9d5ff var(--tw-gradient-to-position)}.dark\:to-purple-200\/0:is(.dark *){--tw-gradient-to:rgba(233,213,255,0) var(--tw-gradient-to-position)}.dark\:to-purple-200\/10:is(.dark *){--tw-gradient-to:rgba(233,213,255,.1) var(--tw-gradient-to-position)}.dark\:to-purple-200\/100:is(.dark *){--tw-gradient-to:#e9d5ff var(--tw-gradient-to-position)}.dark\:to-purple-200\/15:is(.dark *){--tw-gradient-to:rgba(233,213,255,.15) var(--tw-gradient-to-position)}.dark\:to-purple-200\/20:is(.dark *){--tw-gradient-to:rgba(233,213,255,.2) var(--tw-gradient-to-position)}.dark\:to-purple-200\/25:is(.dark *){--tw-gradient-to:rgba(233,213,255,.25) var(--tw-gradient-to-position)}.dark\:to-purple-200\/30:is(.dark *){--tw-gradient-to:rgba(233,213,255,.3) var(--tw-gradient-to-position)}.dark\:to-purple-200\/35:is(.dark *){--tw-gradient-to:rgba(233,213,255,.35) var(--tw-gradient-to-position)}.dark\:to-purple-200\/40:is(.dark *){--tw-gradient-to:rgba(233,213,255,.4) var(--tw-gradient-to-position)}.dark\:to-purple-200\/45:is(.dark *){--tw-gradient-to:rgba(233,213,255,.45) var(--tw-gradient-to-position)}.dark\:to-purple-200\/5:is(.dark *){--tw-gradient-to:rgba(233,213,255,.05) var(--tw-gradient-to-position)}.dark\:to-purple-200\/50:is(.dark *){--tw-gradient-to:rgba(233,213,255,.5) var(--tw-gradient-to-position)}.dark\:to-purple-200\/55:is(.dark *){--tw-gradient-to:rgba(233,213,255,.55) var(--tw-gradient-to-position)}.dark\:to-purple-200\/60:is(.dark *){--tw-gradient-to:rgba(233,213,255,.6) var(--tw-gradient-to-position)}.dark\:to-purple-200\/65:is(.dark *){--tw-gradient-to:rgba(233,213,255,.65) var(--tw-gradient-to-position)}.dark\:to-purple-200\/70:is(.dark *){--tw-gradient-to:rgba(233,213,255,.7) var(--tw-gradient-to-position)}.dark\:to-purple-200\/75:is(.dark *){--tw-gradient-to:rgba(233,213,255,.75) var(--tw-gradient-to-position)}.dark\:to-purple-200\/80:is(.dark *){--tw-gradient-to:rgba(233,213,255,.8) var(--tw-gradient-to-position)}.dark\:to-purple-200\/85:is(.dark *){--tw-gradient-to:rgba(233,213,255,.85) var(--tw-gradient-to-position)}.dark\:to-purple-200\/90:is(.dark *){--tw-gradient-to:rgba(233,213,255,.9) var(--tw-gradient-to-position)}.dark\:to-purple-200\/95:is(.dark *){--tw-gradient-to:rgba(233,213,255,.95) var(--tw-gradient-to-position)}.dark\:to-purple-50:is(.dark *){--tw-gradient-to:#faf5ff var(--tw-gradient-to-position)}.dark\:to-purple-50\/0:is(.dark *){--tw-gradient-to:rgba(250,245,255,0) var(--tw-gradient-to-position)}.dark\:to-purple-50\/10:is(.dark *){--tw-gradient-to:rgba(250,245,255,.1) var(--tw-gradient-to-position)}.dark\:to-purple-50\/100:is(.dark *){--tw-gradient-to:#faf5ff var(--tw-gradient-to-position)}.dark\:to-purple-50\/15:is(.dark *){--tw-gradient-to:rgba(250,245,255,.15) var(--tw-gradient-to-position)}.dark\:to-purple-50\/20:is(.dark *){--tw-gradient-to:rgba(250,245,255,.2) var(--tw-gradient-to-position)}.dark\:to-purple-50\/25:is(.dark *){--tw-gradient-to:rgba(250,245,255,.25) var(--tw-gradient-to-position)}.dark\:to-purple-50\/30:is(.dark *){--tw-gradient-to:rgba(250,245,255,.3) var(--tw-gradient-to-position)}.dark\:to-purple-50\/35:is(.dark *){--tw-gradient-to:rgba(250,245,255,.35) var(--tw-gradient-to-position)}.dark\:to-purple-50\/40:is(.dark *){--tw-gradient-to:rgba(250,245,255,.4) var(--tw-gradient-to-position)}.dark\:to-purple-50\/45:is(.dark *){--tw-gradient-to:rgba(250,245,255,.45) var(--tw-gradient-to-position)}.dark\:to-purple-50\/5:is(.dark *){--tw-gradient-to:rgba(250,245,255,.05) var(--tw-gradient-to-position)}.dark\:to-purple-50\/50:is(.dark *){--tw-gradient-to:rgba(250,245,255,.5) var(--tw-gradient-to-position)}.dark\:to-purple-50\/55:is(.dark *){--tw-gradient-to:rgba(250,245,255,.55) var(--tw-gradient-to-position)}.dark\:to-purple-50\/60:is(.dark *){--tw-gradient-to:rgba(250,245,255,.6) var(--tw-gradient-to-position)}.dark\:to-purple-50\/65:is(.dark *){--tw-gradient-to:rgba(250,245,255,.65) var(--tw-gradient-to-position)}.dark\:to-purple-50\/70:is(.dark *){--tw-gradient-to:rgba(250,245,255,.7) var(--tw-gradient-to-position)}.dark\:to-purple-50\/75:is(.dark *){--tw-gradient-to:rgba(250,245,255,.75) var(--tw-gradient-to-position)}.dark\:to-purple-50\/80:is(.dark *){--tw-gradient-to:rgba(250,245,255,.8) var(--tw-gradient-to-position)}.dark\:to-purple-50\/85:is(.dark *){--tw-gradient-to:rgba(250,245,255,.85) var(--tw-gradient-to-position)}.dark\:to-purple-50\/90:is(.dark *){--tw-gradient-to:rgba(250,245,255,.9) var(--tw-gradient-to-position)}.dark\:to-purple-50\/95:is(.dark *){--tw-gradient-to:rgba(250,245,255,.95) var(--tw-gradient-to-position)}.dark\:to-purple-500:is(.dark *){--tw-gradient-to:#a855f7 var(--tw-gradient-to-position)}.dark\:to-purple-500\/0:is(.dark *){--tw-gradient-to:rgba(168,85,247,0) var(--tw-gradient-to-position)}.dark\:to-purple-500\/10:is(.dark *){--tw-gradient-to:rgba(168,85,247,.1) var(--tw-gradient-to-position)}.dark\:to-purple-500\/100:is(.dark *){--tw-gradient-to:#a855f7 var(--tw-gradient-to-position)}.dark\:to-purple-500\/15:is(.dark *){--tw-gradient-to:rgba(168,85,247,.15) var(--tw-gradient-to-position)}.dark\:to-purple-500\/20:is(.dark *){--tw-gradient-to:rgba(168,85,247,.2) var(--tw-gradient-to-position)}.dark\:to-purple-500\/25:is(.dark *){--tw-gradient-to:rgba(168,85,247,.25) var(--tw-gradient-to-position)}.dark\:to-purple-500\/30:is(.dark *){--tw-gradient-to:rgba(168,85,247,.3) var(--tw-gradient-to-position)}.dark\:to-purple-500\/35:is(.dark *){--tw-gradient-to:rgba(168,85,247,.35) var(--tw-gradient-to-position)}.dark\:to-purple-500\/40:is(.dark *){--tw-gradient-to:rgba(168,85,247,.4) var(--tw-gradient-to-position)}.dark\:to-purple-500\/45:is(.dark *){--tw-gradient-to:rgba(168,85,247,.45) var(--tw-gradient-to-position)}.dark\:to-purple-500\/5:is(.dark *){--tw-gradient-to:rgba(168,85,247,.05) var(--tw-gradient-to-position)}.dark\:to-purple-500\/50:is(.dark *){--tw-gradient-to:rgba(168,85,247,.5) var(--tw-gradient-to-position)}.dark\:to-purple-500\/55:is(.dark *){--tw-gradient-to:rgba(168,85,247,.55) var(--tw-gradient-to-position)}.dark\:to-purple-500\/60:is(.dark *){--tw-gradient-to:rgba(168,85,247,.6) var(--tw-gradient-to-position)}.dark\:to-purple-500\/65:is(.dark *){--tw-gradient-to:rgba(168,85,247,.65) var(--tw-gradient-to-position)}.dark\:to-purple-500\/70:is(.dark *){--tw-gradient-to:rgba(168,85,247,.7) var(--tw-gradient-to-position)}.dark\:to-purple-500\/75:is(.dark *){--tw-gradient-to:rgba(168,85,247,.75) var(--tw-gradient-to-position)}.dark\:to-purple-500\/80:is(.dark *){--tw-gradient-to:rgba(168,85,247,.8) var(--tw-gradient-to-position)}.dark\:to-purple-500\/85:is(.dark *){--tw-gradient-to:rgba(168,85,247,.85) var(--tw-gradient-to-position)}.dark\:to-purple-500\/90:is(.dark *){--tw-gradient-to:rgba(168,85,247,.9) var(--tw-gradient-to-position)}.dark\:to-purple-500\/95:is(.dark *){--tw-gradient-to:rgba(168,85,247,.95) var(--tw-gradient-to-position)}.dark\:to-purple-950:is(.dark *){--tw-gradient-to:#3b0764 var(--tw-gradient-to-position)}.dark\:to-purple-950\/0:is(.dark *){--tw-gradient-to:rgba(59,7,100,0) var(--tw-gradient-to-position)}.dark\:to-purple-950\/10:is(.dark *){--tw-gradient-to:rgba(59,7,100,.1) var(--tw-gradient-to-position)}.dark\:to-purple-950\/100:is(.dark *){--tw-gradient-to:#3b0764 var(--tw-gradient-to-position)}.dark\:to-purple-950\/15:is(.dark *){--tw-gradient-to:rgba(59,7,100,.15) var(--tw-gradient-to-position)}.dark\:to-purple-950\/20:is(.dark *){--tw-gradient-to:rgba(59,7,100,.2) var(--tw-gradient-to-position)}.dark\:to-purple-950\/25:is(.dark *){--tw-gradient-to:rgba(59,7,100,.25) var(--tw-gradient-to-position)}.dark\:to-purple-950\/30:is(.dark *){--tw-gradient-to:rgba(59,7,100,.3) var(--tw-gradient-to-position)}.dark\:to-purple-950\/35:is(.dark *){--tw-gradient-to:rgba(59,7,100,.35) var(--tw-gradient-to-position)}.dark\:to-purple-950\/40:is(.dark *){--tw-gradient-to:rgba(59,7,100,.4) var(--tw-gradient-to-position)}.dark\:to-purple-950\/45:is(.dark *){--tw-gradient-to:rgba(59,7,100,.45) var(--tw-gradient-to-position)}.dark\:to-purple-950\/5:is(.dark *){--tw-gradient-to:rgba(59,7,100,.05) var(--tw-gradient-to-position)}.dark\:to-purple-950\/50:is(.dark *){--tw-gradient-to:rgba(59,7,100,.5) var(--tw-gradient-to-position)}.dark\:to-purple-950\/55:is(.dark *){--tw-gradient-to:rgba(59,7,100,.55) var(--tw-gradient-to-position)}.dark\:to-purple-950\/60:is(.dark *){--tw-gradient-to:rgba(59,7,100,.6) var(--tw-gradient-to-position)}.dark\:to-purple-950\/65:is(.dark *){--tw-gradient-to:rgba(59,7,100,.65) var(--tw-gradient-to-position)}.dark\:to-purple-950\/70:is(.dark *){--tw-gradient-to:rgba(59,7,100,.7) var(--tw-gradient-to-position)}.dark\:to-purple-950\/75:is(.dark *){--tw-gradient-to:rgba(59,7,100,.75) var(--tw-gradient-to-position)}.dark\:to-purple-950\/80:is(.dark *){--tw-gradient-to:rgba(59,7,100,.8) var(--tw-gradient-to-position)}.dark\:to-purple-950\/85:is(.dark *){--tw-gradient-to:rgba(59,7,100,.85) var(--tw-gradient-to-position)}.dark\:to-purple-950\/90:is(.dark *){--tw-gradient-to:rgba(59,7,100,.9) var(--tw-gradient-to-position)}.dark\:to-purple-950\/95:is(.dark *){--tw-gradient-to:rgba(59,7,100,.95) var(--tw-gradient-to-position)}.dark\:to-red-100:is(.dark *){--tw-gradient-to:#fee2e2 var(--tw-gradient-to-position)}.dark\:to-red-100\/0:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,0) var(--tw-gradient-to-position)}.dark\:to-red-100\/10:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.1) var(--tw-gradient-to-position)}.dark\:to-red-100\/100:is(.dark *){--tw-gradient-to:#fee2e2 var(--tw-gradient-to-position)}.dark\:to-red-100\/15:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.15) var(--tw-gradient-to-position)}.dark\:to-red-100\/20:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.2) var(--tw-gradient-to-position)}.dark\:to-red-100\/25:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.25) var(--tw-gradient-to-position)}.dark\:to-red-100\/30:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.3) var(--tw-gradient-to-position)}.dark\:to-red-100\/35:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.35) var(--tw-gradient-to-position)}.dark\:to-red-100\/40:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.4) var(--tw-gradient-to-position)}.dark\:to-red-100\/45:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.45) var(--tw-gradient-to-position)}.dark\:to-red-100\/5:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.05) var(--tw-gradient-to-position)}.dark\:to-red-100\/50:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.5) var(--tw-gradient-to-position)}.dark\:to-red-100\/55:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.55) var(--tw-gradient-to-position)}.dark\:to-red-100\/60:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.6) var(--tw-gradient-to-position)}.dark\:to-red-100\/65:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.65) var(--tw-gradient-to-position)}.dark\:to-red-100\/70:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.7) var(--tw-gradient-to-position)}.dark\:to-red-100\/75:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.75) var(--tw-gradient-to-position)}.dark\:to-red-100\/80:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.8) var(--tw-gradient-to-position)}.dark\:to-red-100\/85:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.85) var(--tw-gradient-to-position)}.dark\:to-red-100\/90:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.9) var(--tw-gradient-to-position)}.dark\:to-red-100\/95:is(.dark *){--tw-gradient-to:hsla(0,93%,94%,.95) var(--tw-gradient-to-position)}.dark\:to-red-200:is(.dark *){--tw-gradient-to:#fecaca var(--tw-gradient-to-position)}.dark\:to-red-200\/0:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,0) var(--tw-gradient-to-position)}.dark\:to-red-200\/10:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.1) var(--tw-gradient-to-position)}.dark\:to-red-200\/100:is(.dark *){--tw-gradient-to:#fecaca var(--tw-gradient-to-position)}.dark\:to-red-200\/15:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.15) var(--tw-gradient-to-position)}.dark\:to-red-200\/20:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.2) var(--tw-gradient-to-position)}.dark\:to-red-200\/25:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.25) var(--tw-gradient-to-position)}.dark\:to-red-200\/30:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.3) var(--tw-gradient-to-position)}.dark\:to-red-200\/35:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.35) var(--tw-gradient-to-position)}.dark\:to-red-200\/40:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.4) var(--tw-gradient-to-position)}.dark\:to-red-200\/45:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.45) var(--tw-gradient-to-position)}.dark\:to-red-200\/5:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.05) var(--tw-gradient-to-position)}.dark\:to-red-200\/50:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.5) var(--tw-gradient-to-position)}.dark\:to-red-200\/55:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.55) var(--tw-gradient-to-position)}.dark\:to-red-200\/60:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.6) var(--tw-gradient-to-position)}.dark\:to-red-200\/65:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.65) var(--tw-gradient-to-position)}.dark\:to-red-200\/70:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.7) var(--tw-gradient-to-position)}.dark\:to-red-200\/75:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.75) var(--tw-gradient-to-position)}.dark\:to-red-200\/80:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.8) var(--tw-gradient-to-position)}.dark\:to-red-200\/85:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.85) var(--tw-gradient-to-position)}.dark\:to-red-200\/90:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.9) var(--tw-gradient-to-position)}.dark\:to-red-200\/95:is(.dark *){--tw-gradient-to:hsla(0,96%,89%,.95) var(--tw-gradient-to-position)}.dark\:to-red-50:is(.dark *){--tw-gradient-to:#fef2f2 var(--tw-gradient-to-position)}.dark\:to-red-50\/0:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,0) var(--tw-gradient-to-position)}.dark\:to-red-50\/10:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.1) var(--tw-gradient-to-position)}.dark\:to-red-50\/100:is(.dark *){--tw-gradient-to:#fef2f2 var(--tw-gradient-to-position)}.dark\:to-red-50\/15:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.15) var(--tw-gradient-to-position)}.dark\:to-red-50\/20:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.2) var(--tw-gradient-to-position)}.dark\:to-red-50\/25:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.25) var(--tw-gradient-to-position)}.dark\:to-red-50\/30:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.3) var(--tw-gradient-to-position)}.dark\:to-red-50\/35:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.35) var(--tw-gradient-to-position)}.dark\:to-red-50\/40:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.4) var(--tw-gradient-to-position)}.dark\:to-red-50\/45:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.45) var(--tw-gradient-to-position)}.dark\:to-red-50\/5:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.05) var(--tw-gradient-to-position)}.dark\:to-red-50\/50:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.5) var(--tw-gradient-to-position)}.dark\:to-red-50\/55:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.55) var(--tw-gradient-to-position)}.dark\:to-red-50\/60:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.6) var(--tw-gradient-to-position)}.dark\:to-red-50\/65:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.65) var(--tw-gradient-to-position)}.dark\:to-red-50\/70:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.7) var(--tw-gradient-to-position)}.dark\:to-red-50\/75:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.75) var(--tw-gradient-to-position)}.dark\:to-red-50\/80:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.8) var(--tw-gradient-to-position)}.dark\:to-red-50\/85:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.85) var(--tw-gradient-to-position)}.dark\:to-red-50\/90:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.9) var(--tw-gradient-to-position)}.dark\:to-red-50\/95:is(.dark *){--tw-gradient-to:hsla(0,86%,97%,.95) var(--tw-gradient-to-position)}.dark\:to-red-500:is(.dark *){--tw-gradient-to:#ef4444 var(--tw-gradient-to-position)}.dark\:to-red-500\/0:is(.dark *){--tw-gradient-to:rgba(239,68,68,0) var(--tw-gradient-to-position)}.dark\:to-red-500\/10:is(.dark *){--tw-gradient-to:rgba(239,68,68,.1) var(--tw-gradient-to-position)}.dark\:to-red-500\/100:is(.dark *){--tw-gradient-to:#ef4444 var(--tw-gradient-to-position)}.dark\:to-red-500\/15:is(.dark *){--tw-gradient-to:rgba(239,68,68,.15) var(--tw-gradient-to-position)}.dark\:to-red-500\/20:is(.dark *){--tw-gradient-to:rgba(239,68,68,.2) var(--tw-gradient-to-position)}.dark\:to-red-500\/25:is(.dark *){--tw-gradient-to:rgba(239,68,68,.25) var(--tw-gradient-to-position)}.dark\:to-red-500\/30:is(.dark *){--tw-gradient-to:rgba(239,68,68,.3) var(--tw-gradient-to-position)}.dark\:to-red-500\/35:is(.dark *){--tw-gradient-to:rgba(239,68,68,.35) var(--tw-gradient-to-position)}.dark\:to-red-500\/40:is(.dark *){--tw-gradient-to:rgba(239,68,68,.4) var(--tw-gradient-to-position)}.dark\:to-red-500\/45:is(.dark *){--tw-gradient-to:rgba(239,68,68,.45) var(--tw-gradient-to-position)}.dark\:to-red-500\/5:is(.dark *){--tw-gradient-to:rgba(239,68,68,.05) var(--tw-gradient-to-position)}.dark\:to-red-500\/50:is(.dark *){--tw-gradient-to:rgba(239,68,68,.5) var(--tw-gradient-to-position)}.dark\:to-red-500\/55:is(.dark *){--tw-gradient-to:rgba(239,68,68,.55) var(--tw-gradient-to-position)}.dark\:to-red-500\/60:is(.dark *){--tw-gradient-to:rgba(239,68,68,.6) var(--tw-gradient-to-position)}.dark\:to-red-500\/65:is(.dark *){--tw-gradient-to:rgba(239,68,68,.65) var(--tw-gradient-to-position)}.dark\:to-red-500\/70:is(.dark *){--tw-gradient-to:rgba(239,68,68,.7) var(--tw-gradient-to-position)}.dark\:to-red-500\/75:is(.dark *){--tw-gradient-to:rgba(239,68,68,.75) var(--tw-gradient-to-position)}.dark\:to-red-500\/80:is(.dark *){--tw-gradient-to:rgba(239,68,68,.8) var(--tw-gradient-to-position)}.dark\:to-red-500\/85:is(.dark *){--tw-gradient-to:rgba(239,68,68,.85) var(--tw-gradient-to-position)}.dark\:to-red-500\/90:is(.dark *){--tw-gradient-to:rgba(239,68,68,.9) var(--tw-gradient-to-position)}.dark\:to-red-500\/95:is(.dark *){--tw-gradient-to:rgba(239,68,68,.95) var(--tw-gradient-to-position)}.dark\:to-red-950:is(.dark *){--tw-gradient-to:#450a0a var(--tw-gradient-to-position)}.dark\:to-red-950\/0:is(.dark *){--tw-gradient-to:rgba(69,10,10,0) var(--tw-gradient-to-position)}.dark\:to-red-950\/10:is(.dark *){--tw-gradient-to:rgba(69,10,10,.1) var(--tw-gradient-to-position)}.dark\:to-red-950\/100:is(.dark *){--tw-gradient-to:#450a0a var(--tw-gradient-to-position)}.dark\:to-red-950\/15:is(.dark *){--tw-gradient-to:rgba(69,10,10,.15) var(--tw-gradient-to-position)}.dark\:to-red-950\/20:is(.dark *){--tw-gradient-to:rgba(69,10,10,.2) var(--tw-gradient-to-position)}.dark\:to-red-950\/25:is(.dark *){--tw-gradient-to:rgba(69,10,10,.25) var(--tw-gradient-to-position)}.dark\:to-red-950\/30:is(.dark *){--tw-gradient-to:rgba(69,10,10,.3) var(--tw-gradient-to-position)}.dark\:to-red-950\/35:is(.dark *){--tw-gradient-to:rgba(69,10,10,.35) var(--tw-gradient-to-position)}.dark\:to-red-950\/40:is(.dark *){--tw-gradient-to:rgba(69,10,10,.4) var(--tw-gradient-to-position)}.dark\:to-red-950\/45:is(.dark *){--tw-gradient-to:rgba(69,10,10,.45) var(--tw-gradient-to-position)}.dark\:to-red-950\/5:is(.dark *){--tw-gradient-to:rgba(69,10,10,.05) var(--tw-gradient-to-position)}.dark\:to-red-950\/50:is(.dark *){--tw-gradient-to:rgba(69,10,10,.5) var(--tw-gradient-to-position)}.dark\:to-red-950\/55:is(.dark *){--tw-gradient-to:rgba(69,10,10,.55) var(--tw-gradient-to-position)}.dark\:to-red-950\/60:is(.dark *){--tw-gradient-to:rgba(69,10,10,.6) var(--tw-gradient-to-position)}.dark\:to-red-950\/65:is(.dark *){--tw-gradient-to:rgba(69,10,10,.65) var(--tw-gradient-to-position)}.dark\:to-red-950\/70:is(.dark *){--tw-gradient-to:rgba(69,10,10,.7) var(--tw-gradient-to-position)}.dark\:to-red-950\/75:is(.dark *){--tw-gradient-to:rgba(69,10,10,.75) var(--tw-gradient-to-position)}.dark\:to-red-950\/80:is(.dark *){--tw-gradient-to:rgba(69,10,10,.8) var(--tw-gradient-to-position)}.dark\:to-red-950\/85:is(.dark *){--tw-gradient-to:rgba(69,10,10,.85) var(--tw-gradient-to-position)}.dark\:to-red-950\/90:is(.dark *){--tw-gradient-to:rgba(69,10,10,.9) var(--tw-gradient-to-position)}.dark\:to-red-950\/95:is(.dark *){--tw-gradient-to:rgba(69,10,10,.95) var(--tw-gradient-to-position)}.dark\:to-rose-100:is(.dark *){--tw-gradient-to:#ffe4e6 var(--tw-gradient-to-position)}.dark\:to-rose-100\/0:is(.dark *){--tw-gradient-to:rgba(255,228,230,0) var(--tw-gradient-to-position)}.dark\:to-rose-100\/10:is(.dark *){--tw-gradient-to:rgba(255,228,230,.1) var(--tw-gradient-to-position)}.dark\:to-rose-100\/100:is(.dark *){--tw-gradient-to:#ffe4e6 var(--tw-gradient-to-position)}.dark\:to-rose-100\/15:is(.dark *){--tw-gradient-to:rgba(255,228,230,.15) var(--tw-gradient-to-position)}.dark\:to-rose-100\/20:is(.dark *){--tw-gradient-to:rgba(255,228,230,.2) var(--tw-gradient-to-position)}.dark\:to-rose-100\/25:is(.dark *){--tw-gradient-to:rgba(255,228,230,.25) var(--tw-gradient-to-position)}.dark\:to-rose-100\/30:is(.dark *){--tw-gradient-to:rgba(255,228,230,.3) var(--tw-gradient-to-position)}.dark\:to-rose-100\/35:is(.dark *){--tw-gradient-to:rgba(255,228,230,.35) var(--tw-gradient-to-position)}.dark\:to-rose-100\/40:is(.dark *){--tw-gradient-to:rgba(255,228,230,.4) var(--tw-gradient-to-position)}.dark\:to-rose-100\/45:is(.dark *){--tw-gradient-to:rgba(255,228,230,.45) var(--tw-gradient-to-position)}.dark\:to-rose-100\/5:is(.dark *){--tw-gradient-to:rgba(255,228,230,.05) var(--tw-gradient-to-position)}.dark\:to-rose-100\/50:is(.dark *){--tw-gradient-to:rgba(255,228,230,.5) var(--tw-gradient-to-position)}.dark\:to-rose-100\/55:is(.dark *){--tw-gradient-to:rgba(255,228,230,.55) var(--tw-gradient-to-position)}.dark\:to-rose-100\/60:is(.dark *){--tw-gradient-to:rgba(255,228,230,.6) var(--tw-gradient-to-position)}.dark\:to-rose-100\/65:is(.dark *){--tw-gradient-to:rgba(255,228,230,.65) var(--tw-gradient-to-position)}.dark\:to-rose-100\/70:is(.dark *){--tw-gradient-to:rgba(255,228,230,.7) var(--tw-gradient-to-position)}.dark\:to-rose-100\/75:is(.dark *){--tw-gradient-to:rgba(255,228,230,.75) var(--tw-gradient-to-position)}.dark\:to-rose-100\/80:is(.dark *){--tw-gradient-to:rgba(255,228,230,.8) var(--tw-gradient-to-position)}.dark\:to-rose-100\/85:is(.dark *){--tw-gradient-to:rgba(255,228,230,.85) var(--tw-gradient-to-position)}.dark\:to-rose-100\/90:is(.dark *){--tw-gradient-to:rgba(255,228,230,.9) var(--tw-gradient-to-position)}.dark\:to-rose-100\/95:is(.dark *){--tw-gradient-to:rgba(255,228,230,.95) var(--tw-gradient-to-position)}.dark\:to-rose-200:is(.dark *){--tw-gradient-to:#fecdd3 var(--tw-gradient-to-position)}.dark\:to-rose-200\/0:is(.dark *){--tw-gradient-to:rgba(254,205,211,0) var(--tw-gradient-to-position)}.dark\:to-rose-200\/10:is(.dark *){--tw-gradient-to:rgba(254,205,211,.1) var(--tw-gradient-to-position)}.dark\:to-rose-200\/100:is(.dark *){--tw-gradient-to:#fecdd3 var(--tw-gradient-to-position)}.dark\:to-rose-200\/15:is(.dark *){--tw-gradient-to:rgba(254,205,211,.15) var(--tw-gradient-to-position)}.dark\:to-rose-200\/20:is(.dark *){--tw-gradient-to:rgba(254,205,211,.2) var(--tw-gradient-to-position)}.dark\:to-rose-200\/25:is(.dark *){--tw-gradient-to:rgba(254,205,211,.25) var(--tw-gradient-to-position)}.dark\:to-rose-200\/30:is(.dark *){--tw-gradient-to:rgba(254,205,211,.3) var(--tw-gradient-to-position)}.dark\:to-rose-200\/35:is(.dark *){--tw-gradient-to:rgba(254,205,211,.35) var(--tw-gradient-to-position)}.dark\:to-rose-200\/40:is(.dark *){--tw-gradient-to:rgba(254,205,211,.4) var(--tw-gradient-to-position)}.dark\:to-rose-200\/45:is(.dark *){--tw-gradient-to:rgba(254,205,211,.45) var(--tw-gradient-to-position)}.dark\:to-rose-200\/5:is(.dark *){--tw-gradient-to:rgba(254,205,211,.05) var(--tw-gradient-to-position)}.dark\:to-rose-200\/50:is(.dark *){--tw-gradient-to:rgba(254,205,211,.5) var(--tw-gradient-to-position)}.dark\:to-rose-200\/55:is(.dark *){--tw-gradient-to:rgba(254,205,211,.55) var(--tw-gradient-to-position)}.dark\:to-rose-200\/60:is(.dark *){--tw-gradient-to:rgba(254,205,211,.6) var(--tw-gradient-to-position)}.dark\:to-rose-200\/65:is(.dark *){--tw-gradient-to:rgba(254,205,211,.65) var(--tw-gradient-to-position)}.dark\:to-rose-200\/70:is(.dark *){--tw-gradient-to:rgba(254,205,211,.7) var(--tw-gradient-to-position)}.dark\:to-rose-200\/75:is(.dark *){--tw-gradient-to:rgba(254,205,211,.75) var(--tw-gradient-to-position)}.dark\:to-rose-200\/80:is(.dark *){--tw-gradient-to:rgba(254,205,211,.8) var(--tw-gradient-to-position)}.dark\:to-rose-200\/85:is(.dark *){--tw-gradient-to:rgba(254,205,211,.85) var(--tw-gradient-to-position)}.dark\:to-rose-200\/90:is(.dark *){--tw-gradient-to:rgba(254,205,211,.9) var(--tw-gradient-to-position)}.dark\:to-rose-200\/95:is(.dark *){--tw-gradient-to:rgba(254,205,211,.95) var(--tw-gradient-to-position)}.dark\:to-rose-50:is(.dark *){--tw-gradient-to:#fff1f2 var(--tw-gradient-to-position)}.dark\:to-rose-50\/0:is(.dark *){--tw-gradient-to:rgba(255,241,242,0) var(--tw-gradient-to-position)}.dark\:to-rose-50\/10:is(.dark *){--tw-gradient-to:rgba(255,241,242,.1) var(--tw-gradient-to-position)}.dark\:to-rose-50\/100:is(.dark *){--tw-gradient-to:#fff1f2 var(--tw-gradient-to-position)}.dark\:to-rose-50\/15:is(.dark *){--tw-gradient-to:rgba(255,241,242,.15) var(--tw-gradient-to-position)}.dark\:to-rose-50\/20:is(.dark *){--tw-gradient-to:rgba(255,241,242,.2) var(--tw-gradient-to-position)}.dark\:to-rose-50\/25:is(.dark *){--tw-gradient-to:rgba(255,241,242,.25) var(--tw-gradient-to-position)}.dark\:to-rose-50\/30:is(.dark *){--tw-gradient-to:rgba(255,241,242,.3) var(--tw-gradient-to-position)}.dark\:to-rose-50\/35:is(.dark *){--tw-gradient-to:rgba(255,241,242,.35) var(--tw-gradient-to-position)}.dark\:to-rose-50\/40:is(.dark *){--tw-gradient-to:rgba(255,241,242,.4) var(--tw-gradient-to-position)}.dark\:to-rose-50\/45:is(.dark *){--tw-gradient-to:rgba(255,241,242,.45) var(--tw-gradient-to-position)}.dark\:to-rose-50\/5:is(.dark *){--tw-gradient-to:rgba(255,241,242,.05) var(--tw-gradient-to-position)}.dark\:to-rose-50\/50:is(.dark *){--tw-gradient-to:rgba(255,241,242,.5) var(--tw-gradient-to-position)}.dark\:to-rose-50\/55:is(.dark *){--tw-gradient-to:rgba(255,241,242,.55) var(--tw-gradient-to-position)}.dark\:to-rose-50\/60:is(.dark *){--tw-gradient-to:rgba(255,241,242,.6) var(--tw-gradient-to-position)}.dark\:to-rose-50\/65:is(.dark *){--tw-gradient-to:rgba(255,241,242,.65) var(--tw-gradient-to-position)}.dark\:to-rose-50\/70:is(.dark *){--tw-gradient-to:rgba(255,241,242,.7) var(--tw-gradient-to-position)}.dark\:to-rose-50\/75:is(.dark *){--tw-gradient-to:rgba(255,241,242,.75) var(--tw-gradient-to-position)}.dark\:to-rose-50\/80:is(.dark *){--tw-gradient-to:rgba(255,241,242,.8) var(--tw-gradient-to-position)}.dark\:to-rose-50\/85:is(.dark *){--tw-gradient-to:rgba(255,241,242,.85) var(--tw-gradient-to-position)}.dark\:to-rose-50\/90:is(.dark *){--tw-gradient-to:rgba(255,241,242,.9) var(--tw-gradient-to-position)}.dark\:to-rose-50\/95:is(.dark *){--tw-gradient-to:rgba(255,241,242,.95) var(--tw-gradient-to-position)}.dark\:to-rose-500:is(.dark *){--tw-gradient-to:#f43f5e var(--tw-gradient-to-position)}.dark\:to-rose-500\/0:is(.dark *){--tw-gradient-to:rgba(244,63,94,0) var(--tw-gradient-to-position)}.dark\:to-rose-500\/10:is(.dark *){--tw-gradient-to:rgba(244,63,94,.1) var(--tw-gradient-to-position)}.dark\:to-rose-500\/100:is(.dark *){--tw-gradient-to:#f43f5e var(--tw-gradient-to-position)}.dark\:to-rose-500\/15:is(.dark *){--tw-gradient-to:rgba(244,63,94,.15) var(--tw-gradient-to-position)}.dark\:to-rose-500\/20:is(.dark *){--tw-gradient-to:rgba(244,63,94,.2) var(--tw-gradient-to-position)}.dark\:to-rose-500\/25:is(.dark *){--tw-gradient-to:rgba(244,63,94,.25) var(--tw-gradient-to-position)}.dark\:to-rose-500\/30:is(.dark *){--tw-gradient-to:rgba(244,63,94,.3) var(--tw-gradient-to-position)}.dark\:to-rose-500\/35:is(.dark *){--tw-gradient-to:rgba(244,63,94,.35) var(--tw-gradient-to-position)}.dark\:to-rose-500\/40:is(.dark *){--tw-gradient-to:rgba(244,63,94,.4) var(--tw-gradient-to-position)}.dark\:to-rose-500\/45:is(.dark *){--tw-gradient-to:rgba(244,63,94,.45) var(--tw-gradient-to-position)}.dark\:to-rose-500\/5:is(.dark *){--tw-gradient-to:rgba(244,63,94,.05) var(--tw-gradient-to-position)}.dark\:to-rose-500\/50:is(.dark *){--tw-gradient-to:rgba(244,63,94,.5) var(--tw-gradient-to-position)}.dark\:to-rose-500\/55:is(.dark *){--tw-gradient-to:rgba(244,63,94,.55) var(--tw-gradient-to-position)}.dark\:to-rose-500\/60:is(.dark *){--tw-gradient-to:rgba(244,63,94,.6) var(--tw-gradient-to-position)}.dark\:to-rose-500\/65:is(.dark *){--tw-gradient-to:rgba(244,63,94,.65) var(--tw-gradient-to-position)}.dark\:to-rose-500\/70:is(.dark *){--tw-gradient-to:rgba(244,63,94,.7) var(--tw-gradient-to-position)}.dark\:to-rose-500\/75:is(.dark *){--tw-gradient-to:rgba(244,63,94,.75) var(--tw-gradient-to-position)}.dark\:to-rose-500\/80:is(.dark *){--tw-gradient-to:rgba(244,63,94,.8) var(--tw-gradient-to-position)}.dark\:to-rose-500\/85:is(.dark *){--tw-gradient-to:rgba(244,63,94,.85) var(--tw-gradient-to-position)}.dark\:to-rose-500\/90:is(.dark *){--tw-gradient-to:rgba(244,63,94,.9) var(--tw-gradient-to-position)}.dark\:to-rose-500\/95:is(.dark *){--tw-gradient-to:rgba(244,63,94,.95) var(--tw-gradient-to-position)}.dark\:to-rose-950:is(.dark *){--tw-gradient-to:#4c0519 var(--tw-gradient-to-position)}.dark\:to-rose-950\/0:is(.dark *){--tw-gradient-to:rgba(76,5,25,0) var(--tw-gradient-to-position)}.dark\:to-rose-950\/10:is(.dark *){--tw-gradient-to:rgba(76,5,25,.1) var(--tw-gradient-to-position)}.dark\:to-rose-950\/100:is(.dark *){--tw-gradient-to:#4c0519 var(--tw-gradient-to-position)}.dark\:to-rose-950\/15:is(.dark *){--tw-gradient-to:rgba(76,5,25,.15) var(--tw-gradient-to-position)}.dark\:to-rose-950\/20:is(.dark *){--tw-gradient-to:rgba(76,5,25,.2) var(--tw-gradient-to-position)}.dark\:to-rose-950\/25:is(.dark *){--tw-gradient-to:rgba(76,5,25,.25) var(--tw-gradient-to-position)}.dark\:to-rose-950\/30:is(.dark *){--tw-gradient-to:rgba(76,5,25,.3) var(--tw-gradient-to-position)}.dark\:to-rose-950\/35:is(.dark *){--tw-gradient-to:rgba(76,5,25,.35) var(--tw-gradient-to-position)}.dark\:to-rose-950\/40:is(.dark *){--tw-gradient-to:rgba(76,5,25,.4) var(--tw-gradient-to-position)}.dark\:to-rose-950\/45:is(.dark *){--tw-gradient-to:rgba(76,5,25,.45) var(--tw-gradient-to-position)}.dark\:to-rose-950\/5:is(.dark *){--tw-gradient-to:rgba(76,5,25,.05) var(--tw-gradient-to-position)}.dark\:to-rose-950\/50:is(.dark *){--tw-gradient-to:rgba(76,5,25,.5) var(--tw-gradient-to-position)}.dark\:to-rose-950\/55:is(.dark *){--tw-gradient-to:rgba(76,5,25,.55) var(--tw-gradient-to-position)}.dark\:to-rose-950\/60:is(.dark *){--tw-gradient-to:rgba(76,5,25,.6) var(--tw-gradient-to-position)}.dark\:to-rose-950\/65:is(.dark *){--tw-gradient-to:rgba(76,5,25,.65) var(--tw-gradient-to-position)}.dark\:to-rose-950\/70:is(.dark *){--tw-gradient-to:rgba(76,5,25,.7) var(--tw-gradient-to-position)}.dark\:to-rose-950\/75:is(.dark *){--tw-gradient-to:rgba(76,5,25,.75) var(--tw-gradient-to-position)}.dark\:to-rose-950\/80:is(.dark *){--tw-gradient-to:rgba(76,5,25,.8) var(--tw-gradient-to-position)}.dark\:to-rose-950\/85:is(.dark *){--tw-gradient-to:rgba(76,5,25,.85) var(--tw-gradient-to-position)}.dark\:to-rose-950\/90:is(.dark *){--tw-gradient-to:rgba(76,5,25,.9) var(--tw-gradient-to-position)}.dark\:to-rose-950\/95:is(.dark *){--tw-gradient-to:rgba(76,5,25,.95) var(--tw-gradient-to-position)}.dark\:to-sky-100:is(.dark *){--tw-gradient-to:#e0f2fe var(--tw-gradient-to-position)}.dark\:to-sky-100\/0:is(.dark *){--tw-gradient-to:rgba(224,242,254,0) var(--tw-gradient-to-position)}.dark\:to-sky-100\/10:is(.dark *){--tw-gradient-to:rgba(224,242,254,.1) var(--tw-gradient-to-position)}.dark\:to-sky-100\/100:is(.dark *){--tw-gradient-to:#e0f2fe var(--tw-gradient-to-position)}.dark\:to-sky-100\/15:is(.dark *){--tw-gradient-to:rgba(224,242,254,.15) var(--tw-gradient-to-position)}.dark\:to-sky-100\/20:is(.dark *){--tw-gradient-to:rgba(224,242,254,.2) var(--tw-gradient-to-position)}.dark\:to-sky-100\/25:is(.dark *){--tw-gradient-to:rgba(224,242,254,.25) var(--tw-gradient-to-position)}.dark\:to-sky-100\/30:is(.dark *){--tw-gradient-to:rgba(224,242,254,.3) var(--tw-gradient-to-position)}.dark\:to-sky-100\/35:is(.dark *){--tw-gradient-to:rgba(224,242,254,.35) var(--tw-gradient-to-position)}.dark\:to-sky-100\/40:is(.dark *){--tw-gradient-to:rgba(224,242,254,.4) var(--tw-gradient-to-position)}.dark\:to-sky-100\/45:is(.dark *){--tw-gradient-to:rgba(224,242,254,.45) var(--tw-gradient-to-position)}.dark\:to-sky-100\/5:is(.dark *){--tw-gradient-to:rgba(224,242,254,.05) var(--tw-gradient-to-position)}.dark\:to-sky-100\/50:is(.dark *){--tw-gradient-to:rgba(224,242,254,.5) var(--tw-gradient-to-position)}.dark\:to-sky-100\/55:is(.dark *){--tw-gradient-to:rgba(224,242,254,.55) var(--tw-gradient-to-position)}.dark\:to-sky-100\/60:is(.dark *){--tw-gradient-to:rgba(224,242,254,.6) var(--tw-gradient-to-position)}.dark\:to-sky-100\/65:is(.dark *){--tw-gradient-to:rgba(224,242,254,.65) var(--tw-gradient-to-position)}.dark\:to-sky-100\/70:is(.dark *){--tw-gradient-to:rgba(224,242,254,.7) var(--tw-gradient-to-position)}.dark\:to-sky-100\/75:is(.dark *){--tw-gradient-to:rgba(224,242,254,.75) var(--tw-gradient-to-position)}.dark\:to-sky-100\/80:is(.dark *){--tw-gradient-to:rgba(224,242,254,.8) var(--tw-gradient-to-position)}.dark\:to-sky-100\/85:is(.dark *){--tw-gradient-to:rgba(224,242,254,.85) var(--tw-gradient-to-position)}.dark\:to-sky-100\/90:is(.dark *){--tw-gradient-to:rgba(224,242,254,.9) var(--tw-gradient-to-position)}.dark\:to-sky-100\/95:is(.dark *){--tw-gradient-to:rgba(224,242,254,.95) var(--tw-gradient-to-position)}.dark\:to-sky-200:is(.dark *){--tw-gradient-to:#bae6fd var(--tw-gradient-to-position)}.dark\:to-sky-200\/0:is(.dark *){--tw-gradient-to:rgba(186,230,253,0) var(--tw-gradient-to-position)}.dark\:to-sky-200\/10:is(.dark *){--tw-gradient-to:rgba(186,230,253,.1) var(--tw-gradient-to-position)}.dark\:to-sky-200\/100:is(.dark *){--tw-gradient-to:#bae6fd var(--tw-gradient-to-position)}.dark\:to-sky-200\/15:is(.dark *){--tw-gradient-to:rgba(186,230,253,.15) var(--tw-gradient-to-position)}.dark\:to-sky-200\/20:is(.dark *){--tw-gradient-to:rgba(186,230,253,.2) var(--tw-gradient-to-position)}.dark\:to-sky-200\/25:is(.dark *){--tw-gradient-to:rgba(186,230,253,.25) var(--tw-gradient-to-position)}.dark\:to-sky-200\/30:is(.dark *){--tw-gradient-to:rgba(186,230,253,.3) var(--tw-gradient-to-position)}.dark\:to-sky-200\/35:is(.dark *){--tw-gradient-to:rgba(186,230,253,.35) var(--tw-gradient-to-position)}.dark\:to-sky-200\/40:is(.dark *){--tw-gradient-to:rgba(186,230,253,.4) var(--tw-gradient-to-position)}.dark\:to-sky-200\/45:is(.dark *){--tw-gradient-to:rgba(186,230,253,.45) var(--tw-gradient-to-position)}.dark\:to-sky-200\/5:is(.dark *){--tw-gradient-to:rgba(186,230,253,.05) var(--tw-gradient-to-position)}.dark\:to-sky-200\/50:is(.dark *){--tw-gradient-to:rgba(186,230,253,.5) var(--tw-gradient-to-position)}.dark\:to-sky-200\/55:is(.dark *){--tw-gradient-to:rgba(186,230,253,.55) var(--tw-gradient-to-position)}.dark\:to-sky-200\/60:is(.dark *){--tw-gradient-to:rgba(186,230,253,.6) var(--tw-gradient-to-position)}.dark\:to-sky-200\/65:is(.dark *){--tw-gradient-to:rgba(186,230,253,.65) var(--tw-gradient-to-position)}.dark\:to-sky-200\/70:is(.dark *){--tw-gradient-to:rgba(186,230,253,.7) var(--tw-gradient-to-position)}.dark\:to-sky-200\/75:is(.dark *){--tw-gradient-to:rgba(186,230,253,.75) var(--tw-gradient-to-position)}.dark\:to-sky-200\/80:is(.dark *){--tw-gradient-to:rgba(186,230,253,.8) var(--tw-gradient-to-position)}.dark\:to-sky-200\/85:is(.dark *){--tw-gradient-to:rgba(186,230,253,.85) var(--tw-gradient-to-position)}.dark\:to-sky-200\/90:is(.dark *){--tw-gradient-to:rgba(186,230,253,.9) var(--tw-gradient-to-position)}.dark\:to-sky-200\/95:is(.dark *){--tw-gradient-to:rgba(186,230,253,.95) var(--tw-gradient-to-position)}.dark\:to-sky-50:is(.dark *){--tw-gradient-to:#f0f9ff var(--tw-gradient-to-position)}.dark\:to-sky-50\/0:is(.dark *){--tw-gradient-to:rgba(240,249,255,0) var(--tw-gradient-to-position)}.dark\:to-sky-50\/10:is(.dark *){--tw-gradient-to:rgba(240,249,255,.1) var(--tw-gradient-to-position)}.dark\:to-sky-50\/100:is(.dark *){--tw-gradient-to:#f0f9ff var(--tw-gradient-to-position)}.dark\:to-sky-50\/15:is(.dark *){--tw-gradient-to:rgba(240,249,255,.15) var(--tw-gradient-to-position)}.dark\:to-sky-50\/20:is(.dark *){--tw-gradient-to:rgba(240,249,255,.2) var(--tw-gradient-to-position)}.dark\:to-sky-50\/25:is(.dark *){--tw-gradient-to:rgba(240,249,255,.25) var(--tw-gradient-to-position)}.dark\:to-sky-50\/30:is(.dark *){--tw-gradient-to:rgba(240,249,255,.3) var(--tw-gradient-to-position)}.dark\:to-sky-50\/35:is(.dark *){--tw-gradient-to:rgba(240,249,255,.35) var(--tw-gradient-to-position)}.dark\:to-sky-50\/40:is(.dark *){--tw-gradient-to:rgba(240,249,255,.4) var(--tw-gradient-to-position)}.dark\:to-sky-50\/45:is(.dark *){--tw-gradient-to:rgba(240,249,255,.45) var(--tw-gradient-to-position)}.dark\:to-sky-50\/5:is(.dark *){--tw-gradient-to:rgba(240,249,255,.05) var(--tw-gradient-to-position)}.dark\:to-sky-50\/50:is(.dark *){--tw-gradient-to:rgba(240,249,255,.5) var(--tw-gradient-to-position)}.dark\:to-sky-50\/55:is(.dark *){--tw-gradient-to:rgba(240,249,255,.55) var(--tw-gradient-to-position)}.dark\:to-sky-50\/60:is(.dark *){--tw-gradient-to:rgba(240,249,255,.6) var(--tw-gradient-to-position)}.dark\:to-sky-50\/65:is(.dark *){--tw-gradient-to:rgba(240,249,255,.65) var(--tw-gradient-to-position)}.dark\:to-sky-50\/70:is(.dark *){--tw-gradient-to:rgba(240,249,255,.7) var(--tw-gradient-to-position)}.dark\:to-sky-50\/75:is(.dark *){--tw-gradient-to:rgba(240,249,255,.75) var(--tw-gradient-to-position)}.dark\:to-sky-50\/80:is(.dark *){--tw-gradient-to:rgba(240,249,255,.8) var(--tw-gradient-to-position)}.dark\:to-sky-50\/85:is(.dark *){--tw-gradient-to:rgba(240,249,255,.85) var(--tw-gradient-to-position)}.dark\:to-sky-50\/90:is(.dark *){--tw-gradient-to:rgba(240,249,255,.9) var(--tw-gradient-to-position)}.dark\:to-sky-50\/95:is(.dark *){--tw-gradient-to:rgba(240,249,255,.95) var(--tw-gradient-to-position)}.dark\:to-sky-500:is(.dark *){--tw-gradient-to:#0ea5e9 var(--tw-gradient-to-position)}.dark\:to-sky-500\/0:is(.dark *){--tw-gradient-to:rgba(14,165,233,0) var(--tw-gradient-to-position)}.dark\:to-sky-500\/10:is(.dark *){--tw-gradient-to:rgba(14,165,233,.1) var(--tw-gradient-to-position)}.dark\:to-sky-500\/100:is(.dark *){--tw-gradient-to:#0ea5e9 var(--tw-gradient-to-position)}.dark\:to-sky-500\/15:is(.dark *){--tw-gradient-to:rgba(14,165,233,.15) var(--tw-gradient-to-position)}.dark\:to-sky-500\/20:is(.dark *){--tw-gradient-to:rgba(14,165,233,.2) var(--tw-gradient-to-position)}.dark\:to-sky-500\/25:is(.dark *){--tw-gradient-to:rgba(14,165,233,.25) var(--tw-gradient-to-position)}.dark\:to-sky-500\/30:is(.dark *){--tw-gradient-to:rgba(14,165,233,.3) var(--tw-gradient-to-position)}.dark\:to-sky-500\/35:is(.dark *){--tw-gradient-to:rgba(14,165,233,.35) var(--tw-gradient-to-position)}.dark\:to-sky-500\/40:is(.dark *){--tw-gradient-to:rgba(14,165,233,.4) var(--tw-gradient-to-position)}.dark\:to-sky-500\/45:is(.dark *){--tw-gradient-to:rgba(14,165,233,.45) var(--tw-gradient-to-position)}.dark\:to-sky-500\/5:is(.dark *){--tw-gradient-to:rgba(14,165,233,.05) var(--tw-gradient-to-position)}.dark\:to-sky-500\/50:is(.dark *){--tw-gradient-to:rgba(14,165,233,.5) var(--tw-gradient-to-position)}.dark\:to-sky-500\/55:is(.dark *){--tw-gradient-to:rgba(14,165,233,.55) var(--tw-gradient-to-position)}.dark\:to-sky-500\/60:is(.dark *){--tw-gradient-to:rgba(14,165,233,.6) var(--tw-gradient-to-position)}.dark\:to-sky-500\/65:is(.dark *){--tw-gradient-to:rgba(14,165,233,.65) var(--tw-gradient-to-position)}.dark\:to-sky-500\/70:is(.dark *){--tw-gradient-to:rgba(14,165,233,.7) var(--tw-gradient-to-position)}.dark\:to-sky-500\/75:is(.dark *){--tw-gradient-to:rgba(14,165,233,.75) var(--tw-gradient-to-position)}.dark\:to-sky-500\/80:is(.dark *){--tw-gradient-to:rgba(14,165,233,.8) var(--tw-gradient-to-position)}.dark\:to-sky-500\/85:is(.dark *){--tw-gradient-to:rgba(14,165,233,.85) var(--tw-gradient-to-position)}.dark\:to-sky-500\/90:is(.dark *){--tw-gradient-to:rgba(14,165,233,.9) var(--tw-gradient-to-position)}.dark\:to-sky-500\/95:is(.dark *){--tw-gradient-to:rgba(14,165,233,.95) var(--tw-gradient-to-position)}.dark\:to-sky-950:is(.dark *){--tw-gradient-to:#082f49 var(--tw-gradient-to-position)}.dark\:to-sky-950\/0:is(.dark *){--tw-gradient-to:rgba(8,47,73,0) var(--tw-gradient-to-position)}.dark\:to-sky-950\/10:is(.dark *){--tw-gradient-to:rgba(8,47,73,.1) var(--tw-gradient-to-position)}.dark\:to-sky-950\/100:is(.dark *){--tw-gradient-to:#082f49 var(--tw-gradient-to-position)}.dark\:to-sky-950\/15:is(.dark *){--tw-gradient-to:rgba(8,47,73,.15) var(--tw-gradient-to-position)}.dark\:to-sky-950\/20:is(.dark *){--tw-gradient-to:rgba(8,47,73,.2) var(--tw-gradient-to-position)}.dark\:to-sky-950\/25:is(.dark *){--tw-gradient-to:rgba(8,47,73,.25) var(--tw-gradient-to-position)}.dark\:to-sky-950\/30:is(.dark *){--tw-gradient-to:rgba(8,47,73,.3) var(--tw-gradient-to-position)}.dark\:to-sky-950\/35:is(.dark *){--tw-gradient-to:rgba(8,47,73,.35) var(--tw-gradient-to-position)}.dark\:to-sky-950\/40:is(.dark *){--tw-gradient-to:rgba(8,47,73,.4) var(--tw-gradient-to-position)}.dark\:to-sky-950\/45:is(.dark *){--tw-gradient-to:rgba(8,47,73,.45) var(--tw-gradient-to-position)}.dark\:to-sky-950\/5:is(.dark *){--tw-gradient-to:rgba(8,47,73,.05) var(--tw-gradient-to-position)}.dark\:to-sky-950\/50:is(.dark *){--tw-gradient-to:rgba(8,47,73,.5) var(--tw-gradient-to-position)}.dark\:to-sky-950\/55:is(.dark *){--tw-gradient-to:rgba(8,47,73,.55) var(--tw-gradient-to-position)}.dark\:to-sky-950\/60:is(.dark *){--tw-gradient-to:rgba(8,47,73,.6) var(--tw-gradient-to-position)}.dark\:to-sky-950\/65:is(.dark *){--tw-gradient-to:rgba(8,47,73,.65) var(--tw-gradient-to-position)}.dark\:to-sky-950\/70:is(.dark *){--tw-gradient-to:rgba(8,47,73,.7) var(--tw-gradient-to-position)}.dark\:to-sky-950\/75:is(.dark *){--tw-gradient-to:rgba(8,47,73,.75) var(--tw-gradient-to-position)}.dark\:to-sky-950\/80:is(.dark *){--tw-gradient-to:rgba(8,47,73,.8) var(--tw-gradient-to-position)}.dark\:to-sky-950\/85:is(.dark *){--tw-gradient-to:rgba(8,47,73,.85) var(--tw-gradient-to-position)}.dark\:to-sky-950\/90:is(.dark *){--tw-gradient-to:rgba(8,47,73,.9) var(--tw-gradient-to-position)}.dark\:to-sky-950\/95:is(.dark *){--tw-gradient-to:rgba(8,47,73,.95) var(--tw-gradient-to-position)}.dark\:to-slate-100:is(.dark *){--tw-gradient-to:#f1f5f9 var(--tw-gradient-to-position)}.dark\:to-slate-100\/0:is(.dark *){--tw-gradient-to:rgba(241,245,249,0) var(--tw-gradient-to-position)}.dark\:to-slate-100\/10:is(.dark *){--tw-gradient-to:rgba(241,245,249,.1) var(--tw-gradient-to-position)}.dark\:to-slate-100\/100:is(.dark *){--tw-gradient-to:#f1f5f9 var(--tw-gradient-to-position)}.dark\:to-slate-100\/15:is(.dark *){--tw-gradient-to:rgba(241,245,249,.15) var(--tw-gradient-to-position)}.dark\:to-slate-100\/20:is(.dark *){--tw-gradient-to:rgba(241,245,249,.2) var(--tw-gradient-to-position)}.dark\:to-slate-100\/25:is(.dark *){--tw-gradient-to:rgba(241,245,249,.25) var(--tw-gradient-to-position)}.dark\:to-slate-100\/30:is(.dark *){--tw-gradient-to:rgba(241,245,249,.3) var(--tw-gradient-to-position)}.dark\:to-slate-100\/35:is(.dark *){--tw-gradient-to:rgba(241,245,249,.35) var(--tw-gradient-to-position)}.dark\:to-slate-100\/40:is(.dark *){--tw-gradient-to:rgba(241,245,249,.4) var(--tw-gradient-to-position)}.dark\:to-slate-100\/45:is(.dark *){--tw-gradient-to:rgba(241,245,249,.45) var(--tw-gradient-to-position)}.dark\:to-slate-100\/5:is(.dark *){--tw-gradient-to:rgba(241,245,249,.05) var(--tw-gradient-to-position)}.dark\:to-slate-100\/50:is(.dark *){--tw-gradient-to:rgba(241,245,249,.5) var(--tw-gradient-to-position)}.dark\:to-slate-100\/55:is(.dark *){--tw-gradient-to:rgba(241,245,249,.55) var(--tw-gradient-to-position)}.dark\:to-slate-100\/60:is(.dark *){--tw-gradient-to:rgba(241,245,249,.6) var(--tw-gradient-to-position)}.dark\:to-slate-100\/65:is(.dark *){--tw-gradient-to:rgba(241,245,249,.65) var(--tw-gradient-to-position)}.dark\:to-slate-100\/70:is(.dark *){--tw-gradient-to:rgba(241,245,249,.7) var(--tw-gradient-to-position)}.dark\:to-slate-100\/75:is(.dark *){--tw-gradient-to:rgba(241,245,249,.75) var(--tw-gradient-to-position)}.dark\:to-slate-100\/80:is(.dark *){--tw-gradient-to:rgba(241,245,249,.8) var(--tw-gradient-to-position)}.dark\:to-slate-100\/85:is(.dark *){--tw-gradient-to:rgba(241,245,249,.85) var(--tw-gradient-to-position)}.dark\:to-slate-100\/90:is(.dark *){--tw-gradient-to:rgba(241,245,249,.9) var(--tw-gradient-to-position)}.dark\:to-slate-100\/95:is(.dark *){--tw-gradient-to:rgba(241,245,249,.95) var(--tw-gradient-to-position)}.dark\:to-slate-200:is(.dark *){--tw-gradient-to:#e2e8f0 var(--tw-gradient-to-position)}.dark\:to-slate-200\/0:is(.dark *){--tw-gradient-to:rgba(226,232,240,0) var(--tw-gradient-to-position)}.dark\:to-slate-200\/10:is(.dark *){--tw-gradient-to:rgba(226,232,240,.1) var(--tw-gradient-to-position)}.dark\:to-slate-200\/100:is(.dark *){--tw-gradient-to:#e2e8f0 var(--tw-gradient-to-position)}.dark\:to-slate-200\/15:is(.dark *){--tw-gradient-to:rgba(226,232,240,.15) var(--tw-gradient-to-position)}.dark\:to-slate-200\/20:is(.dark *){--tw-gradient-to:rgba(226,232,240,.2) var(--tw-gradient-to-position)}.dark\:to-slate-200\/25:is(.dark *){--tw-gradient-to:rgba(226,232,240,.25) var(--tw-gradient-to-position)}.dark\:to-slate-200\/30:is(.dark *){--tw-gradient-to:rgba(226,232,240,.3) var(--tw-gradient-to-position)}.dark\:to-slate-200\/35:is(.dark *){--tw-gradient-to:rgba(226,232,240,.35) var(--tw-gradient-to-position)}.dark\:to-slate-200\/40:is(.dark *){--tw-gradient-to:rgba(226,232,240,.4) var(--tw-gradient-to-position)}.dark\:to-slate-200\/45:is(.dark *){--tw-gradient-to:rgba(226,232,240,.45) var(--tw-gradient-to-position)}.dark\:to-slate-200\/5:is(.dark *){--tw-gradient-to:rgba(226,232,240,.05) var(--tw-gradient-to-position)}.dark\:to-slate-200\/50:is(.dark *){--tw-gradient-to:rgba(226,232,240,.5) var(--tw-gradient-to-position)}.dark\:to-slate-200\/55:is(.dark *){--tw-gradient-to:rgba(226,232,240,.55) var(--tw-gradient-to-position)}.dark\:to-slate-200\/60:is(.dark *){--tw-gradient-to:rgba(226,232,240,.6) var(--tw-gradient-to-position)}.dark\:to-slate-200\/65:is(.dark *){--tw-gradient-to:rgba(226,232,240,.65) var(--tw-gradient-to-position)}.dark\:to-slate-200\/70:is(.dark *){--tw-gradient-to:rgba(226,232,240,.7) var(--tw-gradient-to-position)}.dark\:to-slate-200\/75:is(.dark *){--tw-gradient-to:rgba(226,232,240,.75) var(--tw-gradient-to-position)}.dark\:to-slate-200\/80:is(.dark *){--tw-gradient-to:rgba(226,232,240,.8) var(--tw-gradient-to-position)}.dark\:to-slate-200\/85:is(.dark *){--tw-gradient-to:rgba(226,232,240,.85) var(--tw-gradient-to-position)}.dark\:to-slate-200\/90:is(.dark *){--tw-gradient-to:rgba(226,232,240,.9) var(--tw-gradient-to-position)}.dark\:to-slate-200\/95:is(.dark *){--tw-gradient-to:rgba(226,232,240,.95) var(--tw-gradient-to-position)}.dark\:to-slate-50:is(.dark *){--tw-gradient-to:#f8fafc var(--tw-gradient-to-position)}.dark\:to-slate-50\/0:is(.dark *){--tw-gradient-to:rgba(248,250,252,0) var(--tw-gradient-to-position)}.dark\:to-slate-50\/10:is(.dark *){--tw-gradient-to:rgba(248,250,252,.1) var(--tw-gradient-to-position)}.dark\:to-slate-50\/100:is(.dark *){--tw-gradient-to:#f8fafc var(--tw-gradient-to-position)}.dark\:to-slate-50\/15:is(.dark *){--tw-gradient-to:rgba(248,250,252,.15) var(--tw-gradient-to-position)}.dark\:to-slate-50\/20:is(.dark *){--tw-gradient-to:rgba(248,250,252,.2) var(--tw-gradient-to-position)}.dark\:to-slate-50\/25:is(.dark *){--tw-gradient-to:rgba(248,250,252,.25) var(--tw-gradient-to-position)}.dark\:to-slate-50\/30:is(.dark *){--tw-gradient-to:rgba(248,250,252,.3) var(--tw-gradient-to-position)}.dark\:to-slate-50\/35:is(.dark *){--tw-gradient-to:rgba(248,250,252,.35) var(--tw-gradient-to-position)}.dark\:to-slate-50\/40:is(.dark *){--tw-gradient-to:rgba(248,250,252,.4) var(--tw-gradient-to-position)}.dark\:to-slate-50\/45:is(.dark *){--tw-gradient-to:rgba(248,250,252,.45) var(--tw-gradient-to-position)}.dark\:to-slate-50\/5:is(.dark *){--tw-gradient-to:rgba(248,250,252,.05) var(--tw-gradient-to-position)}.dark\:to-slate-50\/50:is(.dark *){--tw-gradient-to:rgba(248,250,252,.5) var(--tw-gradient-to-position)}.dark\:to-slate-50\/55:is(.dark *){--tw-gradient-to:rgba(248,250,252,.55) var(--tw-gradient-to-position)}.dark\:to-slate-50\/60:is(.dark *){--tw-gradient-to:rgba(248,250,252,.6) var(--tw-gradient-to-position)}.dark\:to-slate-50\/65:is(.dark *){--tw-gradient-to:rgba(248,250,252,.65) var(--tw-gradient-to-position)}.dark\:to-slate-50\/70:is(.dark *){--tw-gradient-to:rgba(248,250,252,.7) var(--tw-gradient-to-position)}.dark\:to-slate-50\/75:is(.dark *){--tw-gradient-to:rgba(248,250,252,.75) var(--tw-gradient-to-position)}.dark\:to-slate-50\/80:is(.dark *){--tw-gradient-to:rgba(248,250,252,.8) var(--tw-gradient-to-position)}.dark\:to-slate-50\/85:is(.dark *){--tw-gradient-to:rgba(248,250,252,.85) var(--tw-gradient-to-position)}.dark\:to-slate-50\/90:is(.dark *){--tw-gradient-to:rgba(248,250,252,.9) var(--tw-gradient-to-position)}.dark\:to-slate-50\/95:is(.dark *){--tw-gradient-to:rgba(248,250,252,.95) var(--tw-gradient-to-position)}.dark\:to-slate-500:is(.dark *){--tw-gradient-to:#64748b var(--tw-gradient-to-position)}.dark\:to-slate-500\/0:is(.dark *){--tw-gradient-to:rgba(100,116,139,0) var(--tw-gradient-to-position)}.dark\:to-slate-500\/10:is(.dark *){--tw-gradient-to:rgba(100,116,139,.1) var(--tw-gradient-to-position)}.dark\:to-slate-500\/100:is(.dark *){--tw-gradient-to:#64748b var(--tw-gradient-to-position)}.dark\:to-slate-500\/15:is(.dark *){--tw-gradient-to:rgba(100,116,139,.15) var(--tw-gradient-to-position)}.dark\:to-slate-500\/20:is(.dark *){--tw-gradient-to:rgba(100,116,139,.2) var(--tw-gradient-to-position)}.dark\:to-slate-500\/25:is(.dark *){--tw-gradient-to:rgba(100,116,139,.25) var(--tw-gradient-to-position)}.dark\:to-slate-500\/30:is(.dark *){--tw-gradient-to:rgba(100,116,139,.3) var(--tw-gradient-to-position)}.dark\:to-slate-500\/35:is(.dark *){--tw-gradient-to:rgba(100,116,139,.35) var(--tw-gradient-to-position)}.dark\:to-slate-500\/40:is(.dark *){--tw-gradient-to:rgba(100,116,139,.4) var(--tw-gradient-to-position)}.dark\:to-slate-500\/45:is(.dark *){--tw-gradient-to:rgba(100,116,139,.45) var(--tw-gradient-to-position)}.dark\:to-slate-500\/5:is(.dark *){--tw-gradient-to:rgba(100,116,139,.05) var(--tw-gradient-to-position)}.dark\:to-slate-500\/50:is(.dark *){--tw-gradient-to:rgba(100,116,139,.5) var(--tw-gradient-to-position)}.dark\:to-slate-500\/55:is(.dark *){--tw-gradient-to:rgba(100,116,139,.55) var(--tw-gradient-to-position)}.dark\:to-slate-500\/60:is(.dark *){--tw-gradient-to:rgba(100,116,139,.6) var(--tw-gradient-to-position)}.dark\:to-slate-500\/65:is(.dark *){--tw-gradient-to:rgba(100,116,139,.65) var(--tw-gradient-to-position)}.dark\:to-slate-500\/70:is(.dark *){--tw-gradient-to:rgba(100,116,139,.7) var(--tw-gradient-to-position)}.dark\:to-slate-500\/75:is(.dark *){--tw-gradient-to:rgba(100,116,139,.75) var(--tw-gradient-to-position)}.dark\:to-slate-500\/80:is(.dark *){--tw-gradient-to:rgba(100,116,139,.8) var(--tw-gradient-to-position)}.dark\:to-slate-500\/85:is(.dark *){--tw-gradient-to:rgba(100,116,139,.85) var(--tw-gradient-to-position)}.dark\:to-slate-500\/90:is(.dark *){--tw-gradient-to:rgba(100,116,139,.9) var(--tw-gradient-to-position)}.dark\:to-slate-500\/95:is(.dark *){--tw-gradient-to:rgba(100,116,139,.95) var(--tw-gradient-to-position)}.dark\:to-slate-950:is(.dark *){--tw-gradient-to:#020617 var(--tw-gradient-to-position)}.dark\:to-slate-950\/0:is(.dark *){--tw-gradient-to:rgba(2,6,23,0) var(--tw-gradient-to-position)}.dark\:to-slate-950\/10:is(.dark *){--tw-gradient-to:rgba(2,6,23,.1) var(--tw-gradient-to-position)}.dark\:to-slate-950\/100:is(.dark *){--tw-gradient-to:#020617 var(--tw-gradient-to-position)}.dark\:to-slate-950\/15:is(.dark *){--tw-gradient-to:rgba(2,6,23,.15) var(--tw-gradient-to-position)}.dark\:to-slate-950\/20:is(.dark *){--tw-gradient-to:rgba(2,6,23,.2) var(--tw-gradient-to-position)}.dark\:to-slate-950\/25:is(.dark *){--tw-gradient-to:rgba(2,6,23,.25) var(--tw-gradient-to-position)}.dark\:to-slate-950\/30:is(.dark *){--tw-gradient-to:rgba(2,6,23,.3) var(--tw-gradient-to-position)}.dark\:to-slate-950\/35:is(.dark *){--tw-gradient-to:rgba(2,6,23,.35) var(--tw-gradient-to-position)}.dark\:to-slate-950\/40:is(.dark *){--tw-gradient-to:rgba(2,6,23,.4) var(--tw-gradient-to-position)}.dark\:to-slate-950\/45:is(.dark *){--tw-gradient-to:rgba(2,6,23,.45) var(--tw-gradient-to-position)}.dark\:to-slate-950\/5:is(.dark *){--tw-gradient-to:rgba(2,6,23,.05) var(--tw-gradient-to-position)}.dark\:to-slate-950\/50:is(.dark *){--tw-gradient-to:rgba(2,6,23,.5) var(--tw-gradient-to-position)}.dark\:to-slate-950\/55:is(.dark *){--tw-gradient-to:rgba(2,6,23,.55) var(--tw-gradient-to-position)}.dark\:to-slate-950\/60:is(.dark *){--tw-gradient-to:rgba(2,6,23,.6) var(--tw-gradient-to-position)}.dark\:to-slate-950\/65:is(.dark *){--tw-gradient-to:rgba(2,6,23,.65) var(--tw-gradient-to-position)}.dark\:to-slate-950\/70:is(.dark *){--tw-gradient-to:rgba(2,6,23,.7) var(--tw-gradient-to-position)}.dark\:to-slate-950\/75:is(.dark *){--tw-gradient-to:rgba(2,6,23,.75) var(--tw-gradient-to-position)}.dark\:to-slate-950\/80:is(.dark *){--tw-gradient-to:rgba(2,6,23,.8) var(--tw-gradient-to-position)}.dark\:to-slate-950\/85:is(.dark *){--tw-gradient-to:rgba(2,6,23,.85) var(--tw-gradient-to-position)}.dark\:to-slate-950\/90:is(.dark *){--tw-gradient-to:rgba(2,6,23,.9) var(--tw-gradient-to-position)}.dark\:to-slate-950\/95:is(.dark *){--tw-gradient-to:rgba(2,6,23,.95) var(--tw-gradient-to-position)}.dark\:to-stone-100:is(.dark *){--tw-gradient-to:#f5f5f4 var(--tw-gradient-to-position)}.dark\:to-stone-100\/0:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,0) var(--tw-gradient-to-position)}.dark\:to-stone-100\/10:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.1) var(--tw-gradient-to-position)}.dark\:to-stone-100\/100:is(.dark *){--tw-gradient-to:#f5f5f4 var(--tw-gradient-to-position)}.dark\:to-stone-100\/15:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.15) var(--tw-gradient-to-position)}.dark\:to-stone-100\/20:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.2) var(--tw-gradient-to-position)}.dark\:to-stone-100\/25:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.25) var(--tw-gradient-to-position)}.dark\:to-stone-100\/30:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.3) var(--tw-gradient-to-position)}.dark\:to-stone-100\/35:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.35) var(--tw-gradient-to-position)}.dark\:to-stone-100\/40:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.4) var(--tw-gradient-to-position)}.dark\:to-stone-100\/45:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.45) var(--tw-gradient-to-position)}.dark\:to-stone-100\/5:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.05) var(--tw-gradient-to-position)}.dark\:to-stone-100\/50:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.5) var(--tw-gradient-to-position)}.dark\:to-stone-100\/55:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.55) var(--tw-gradient-to-position)}.dark\:to-stone-100\/60:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.6) var(--tw-gradient-to-position)}.dark\:to-stone-100\/65:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.65) var(--tw-gradient-to-position)}.dark\:to-stone-100\/70:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.7) var(--tw-gradient-to-position)}.dark\:to-stone-100\/75:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.75) var(--tw-gradient-to-position)}.dark\:to-stone-100\/80:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.8) var(--tw-gradient-to-position)}.dark\:to-stone-100\/85:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.85) var(--tw-gradient-to-position)}.dark\:to-stone-100\/90:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.9) var(--tw-gradient-to-position)}.dark\:to-stone-100\/95:is(.dark *){--tw-gradient-to:hsla(60,5%,96%,.95) var(--tw-gradient-to-position)}.dark\:to-stone-200:is(.dark *){--tw-gradient-to:#e7e5e4 var(--tw-gradient-to-position)}.dark\:to-stone-200\/0:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,0) var(--tw-gradient-to-position)}.dark\:to-stone-200\/10:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.1) var(--tw-gradient-to-position)}.dark\:to-stone-200\/100:is(.dark *){--tw-gradient-to:#e7e5e4 var(--tw-gradient-to-position)}.dark\:to-stone-200\/15:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.15) var(--tw-gradient-to-position)}.dark\:to-stone-200\/20:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.2) var(--tw-gradient-to-position)}.dark\:to-stone-200\/25:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.25) var(--tw-gradient-to-position)}.dark\:to-stone-200\/30:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.3) var(--tw-gradient-to-position)}.dark\:to-stone-200\/35:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.35) var(--tw-gradient-to-position)}.dark\:to-stone-200\/40:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.4) var(--tw-gradient-to-position)}.dark\:to-stone-200\/45:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.45) var(--tw-gradient-to-position)}.dark\:to-stone-200\/5:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.05) var(--tw-gradient-to-position)}.dark\:to-stone-200\/50:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.5) var(--tw-gradient-to-position)}.dark\:to-stone-200\/55:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.55) var(--tw-gradient-to-position)}.dark\:to-stone-200\/60:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.6) var(--tw-gradient-to-position)}.dark\:to-stone-200\/65:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.65) var(--tw-gradient-to-position)}.dark\:to-stone-200\/70:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.7) var(--tw-gradient-to-position)}.dark\:to-stone-200\/75:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.75) var(--tw-gradient-to-position)}.dark\:to-stone-200\/80:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.8) var(--tw-gradient-to-position)}.dark\:to-stone-200\/85:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.85) var(--tw-gradient-to-position)}.dark\:to-stone-200\/90:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.9) var(--tw-gradient-to-position)}.dark\:to-stone-200\/95:is(.dark *){--tw-gradient-to:hsla(20,6%,90%,.95) var(--tw-gradient-to-position)}.dark\:to-stone-50:is(.dark *){--tw-gradient-to:#fafaf9 var(--tw-gradient-to-position)}.dark\:to-stone-50\/0:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,0) var(--tw-gradient-to-position)}.dark\:to-stone-50\/10:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.1) var(--tw-gradient-to-position)}.dark\:to-stone-50\/100:is(.dark *){--tw-gradient-to:#fafaf9 var(--tw-gradient-to-position)}.dark\:to-stone-50\/15:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.15) var(--tw-gradient-to-position)}.dark\:to-stone-50\/20:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.2) var(--tw-gradient-to-position)}.dark\:to-stone-50\/25:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.25) var(--tw-gradient-to-position)}.dark\:to-stone-50\/30:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.3) var(--tw-gradient-to-position)}.dark\:to-stone-50\/35:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.35) var(--tw-gradient-to-position)}.dark\:to-stone-50\/40:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.4) var(--tw-gradient-to-position)}.dark\:to-stone-50\/45:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.45) var(--tw-gradient-to-position)}.dark\:to-stone-50\/5:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.05) var(--tw-gradient-to-position)}.dark\:to-stone-50\/50:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.5) var(--tw-gradient-to-position)}.dark\:to-stone-50\/55:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.55) var(--tw-gradient-to-position)}.dark\:to-stone-50\/60:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.6) var(--tw-gradient-to-position)}.dark\:to-stone-50\/65:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.65) var(--tw-gradient-to-position)}.dark\:to-stone-50\/70:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.7) var(--tw-gradient-to-position)}.dark\:to-stone-50\/75:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.75) var(--tw-gradient-to-position)}.dark\:to-stone-50\/80:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.8) var(--tw-gradient-to-position)}.dark\:to-stone-50\/85:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.85) var(--tw-gradient-to-position)}.dark\:to-stone-50\/90:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.9) var(--tw-gradient-to-position)}.dark\:to-stone-50\/95:is(.dark *){--tw-gradient-to:hsla(60,9%,98%,.95) var(--tw-gradient-to-position)}.dark\:to-stone-500:is(.dark *){--tw-gradient-to:#78716c var(--tw-gradient-to-position)}.dark\:to-stone-500\/0:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,0) var(--tw-gradient-to-position)}.dark\:to-stone-500\/10:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.1) var(--tw-gradient-to-position)}.dark\:to-stone-500\/100:is(.dark *){--tw-gradient-to:#78716c var(--tw-gradient-to-position)}.dark\:to-stone-500\/15:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.15) var(--tw-gradient-to-position)}.dark\:to-stone-500\/20:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.2) var(--tw-gradient-to-position)}.dark\:to-stone-500\/25:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.25) var(--tw-gradient-to-position)}.dark\:to-stone-500\/30:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.3) var(--tw-gradient-to-position)}.dark\:to-stone-500\/35:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.35) var(--tw-gradient-to-position)}.dark\:to-stone-500\/40:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.4) var(--tw-gradient-to-position)}.dark\:to-stone-500\/45:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.45) var(--tw-gradient-to-position)}.dark\:to-stone-500\/5:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.05) var(--tw-gradient-to-position)}.dark\:to-stone-500\/50:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.5) var(--tw-gradient-to-position)}.dark\:to-stone-500\/55:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.55) var(--tw-gradient-to-position)}.dark\:to-stone-500\/60:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.6) var(--tw-gradient-to-position)}.dark\:to-stone-500\/65:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.65) var(--tw-gradient-to-position)}.dark\:to-stone-500\/70:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.7) var(--tw-gradient-to-position)}.dark\:to-stone-500\/75:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.75) var(--tw-gradient-to-position)}.dark\:to-stone-500\/80:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.8) var(--tw-gradient-to-position)}.dark\:to-stone-500\/85:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.85) var(--tw-gradient-to-position)}.dark\:to-stone-500\/90:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.9) var(--tw-gradient-to-position)}.dark\:to-stone-500\/95:is(.dark *){--tw-gradient-to:hsla(25,5%,45%,.95) var(--tw-gradient-to-position)}.dark\:to-stone-950:is(.dark *){--tw-gradient-to:#0c0a09 var(--tw-gradient-to-position)}.dark\:to-stone-950\/0:is(.dark *){--tw-gradient-to:rgba(12,10,9,0) var(--tw-gradient-to-position)}.dark\:to-stone-950\/10:is(.dark *){--tw-gradient-to:rgba(12,10,9,.1) var(--tw-gradient-to-position)}.dark\:to-stone-950\/100:is(.dark *){--tw-gradient-to:#0c0a09 var(--tw-gradient-to-position)}.dark\:to-stone-950\/15:is(.dark *){--tw-gradient-to:rgba(12,10,9,.15) var(--tw-gradient-to-position)}.dark\:to-stone-950\/20:is(.dark *){--tw-gradient-to:rgba(12,10,9,.2) var(--tw-gradient-to-position)}.dark\:to-stone-950\/25:is(.dark *){--tw-gradient-to:rgba(12,10,9,.25) var(--tw-gradient-to-position)}.dark\:to-stone-950\/30:is(.dark *){--tw-gradient-to:rgba(12,10,9,.3) var(--tw-gradient-to-position)}.dark\:to-stone-950\/35:is(.dark *){--tw-gradient-to:rgba(12,10,9,.35) var(--tw-gradient-to-position)}.dark\:to-stone-950\/40:is(.dark *){--tw-gradient-to:rgba(12,10,9,.4) var(--tw-gradient-to-position)}.dark\:to-stone-950\/45:is(.dark *){--tw-gradient-to:rgba(12,10,9,.45) var(--tw-gradient-to-position)}.dark\:to-stone-950\/5:is(.dark *){--tw-gradient-to:rgba(12,10,9,.05) var(--tw-gradient-to-position)}.dark\:to-stone-950\/50:is(.dark *){--tw-gradient-to:rgba(12,10,9,.5) var(--tw-gradient-to-position)}.dark\:to-stone-950\/55:is(.dark *){--tw-gradient-to:rgba(12,10,9,.55) var(--tw-gradient-to-position)}.dark\:to-stone-950\/60:is(.dark *){--tw-gradient-to:rgba(12,10,9,.6) var(--tw-gradient-to-position)}.dark\:to-stone-950\/65:is(.dark *){--tw-gradient-to:rgba(12,10,9,.65) var(--tw-gradient-to-position)}.dark\:to-stone-950\/70:is(.dark *){--tw-gradient-to:rgba(12,10,9,.7) var(--tw-gradient-to-position)}.dark\:to-stone-950\/75:is(.dark *){--tw-gradient-to:rgba(12,10,9,.75) var(--tw-gradient-to-position)}.dark\:to-stone-950\/80:is(.dark *){--tw-gradient-to:rgba(12,10,9,.8) var(--tw-gradient-to-position)}.dark\:to-stone-950\/85:is(.dark *){--tw-gradient-to:rgba(12,10,9,.85) var(--tw-gradient-to-position)}.dark\:to-stone-950\/90:is(.dark *){--tw-gradient-to:rgba(12,10,9,.9) var(--tw-gradient-to-position)}.dark\:to-stone-950\/95:is(.dark *){--tw-gradient-to:rgba(12,10,9,.95) var(--tw-gradient-to-position)}.dark\:to-teal-100:is(.dark *){--tw-gradient-to:#ccfbf1 var(--tw-gradient-to-position)}.dark\:to-teal-100\/0:is(.dark *){--tw-gradient-to:rgba(204,251,241,0) var(--tw-gradient-to-position)}.dark\:to-teal-100\/10:is(.dark *){--tw-gradient-to:rgba(204,251,241,.1) var(--tw-gradient-to-position)}.dark\:to-teal-100\/100:is(.dark *){--tw-gradient-to:#ccfbf1 var(--tw-gradient-to-position)}.dark\:to-teal-100\/15:is(.dark *){--tw-gradient-to:rgba(204,251,241,.15) var(--tw-gradient-to-position)}.dark\:to-teal-100\/20:is(.dark *){--tw-gradient-to:rgba(204,251,241,.2) var(--tw-gradient-to-position)}.dark\:to-teal-100\/25:is(.dark *){--tw-gradient-to:rgba(204,251,241,.25) var(--tw-gradient-to-position)}.dark\:to-teal-100\/30:is(.dark *){--tw-gradient-to:rgba(204,251,241,.3) var(--tw-gradient-to-position)}.dark\:to-teal-100\/35:is(.dark *){--tw-gradient-to:rgba(204,251,241,.35) var(--tw-gradient-to-position)}.dark\:to-teal-100\/40:is(.dark *){--tw-gradient-to:rgba(204,251,241,.4) var(--tw-gradient-to-position)}.dark\:to-teal-100\/45:is(.dark *){--tw-gradient-to:rgba(204,251,241,.45) var(--tw-gradient-to-position)}.dark\:to-teal-100\/5:is(.dark *){--tw-gradient-to:rgba(204,251,241,.05) var(--tw-gradient-to-position)}.dark\:to-teal-100\/50:is(.dark *){--tw-gradient-to:rgba(204,251,241,.5) var(--tw-gradient-to-position)}.dark\:to-teal-100\/55:is(.dark *){--tw-gradient-to:rgba(204,251,241,.55) var(--tw-gradient-to-position)}.dark\:to-teal-100\/60:is(.dark *){--tw-gradient-to:rgba(204,251,241,.6) var(--tw-gradient-to-position)}.dark\:to-teal-100\/65:is(.dark *){--tw-gradient-to:rgba(204,251,241,.65) var(--tw-gradient-to-position)}.dark\:to-teal-100\/70:is(.dark *){--tw-gradient-to:rgba(204,251,241,.7) var(--tw-gradient-to-position)}.dark\:to-teal-100\/75:is(.dark *){--tw-gradient-to:rgba(204,251,241,.75) var(--tw-gradient-to-position)}.dark\:to-teal-100\/80:is(.dark *){--tw-gradient-to:rgba(204,251,241,.8) var(--tw-gradient-to-position)}.dark\:to-teal-100\/85:is(.dark *){--tw-gradient-to:rgba(204,251,241,.85) var(--tw-gradient-to-position)}.dark\:to-teal-100\/90:is(.dark *){--tw-gradient-to:rgba(204,251,241,.9) var(--tw-gradient-to-position)}.dark\:to-teal-100\/95:is(.dark *){--tw-gradient-to:rgba(204,251,241,.95) var(--tw-gradient-to-position)}.dark\:to-teal-200:is(.dark *){--tw-gradient-to:#99f6e4 var(--tw-gradient-to-position)}.dark\:to-teal-200\/0:is(.dark *){--tw-gradient-to:rgba(153,246,228,0) var(--tw-gradient-to-position)}.dark\:to-teal-200\/10:is(.dark *){--tw-gradient-to:rgba(153,246,228,.1) var(--tw-gradient-to-position)}.dark\:to-teal-200\/100:is(.dark *){--tw-gradient-to:#99f6e4 var(--tw-gradient-to-position)}.dark\:to-teal-200\/15:is(.dark *){--tw-gradient-to:rgba(153,246,228,.15) var(--tw-gradient-to-position)}.dark\:to-teal-200\/20:is(.dark *){--tw-gradient-to:rgba(153,246,228,.2) var(--tw-gradient-to-position)}.dark\:to-teal-200\/25:is(.dark *){--tw-gradient-to:rgba(153,246,228,.25) var(--tw-gradient-to-position)}.dark\:to-teal-200\/30:is(.dark *){--tw-gradient-to:rgba(153,246,228,.3) var(--tw-gradient-to-position)}.dark\:to-teal-200\/35:is(.dark *){--tw-gradient-to:rgba(153,246,228,.35) var(--tw-gradient-to-position)}.dark\:to-teal-200\/40:is(.dark *){--tw-gradient-to:rgba(153,246,228,.4) var(--tw-gradient-to-position)}.dark\:to-teal-200\/45:is(.dark *){--tw-gradient-to:rgba(153,246,228,.45) var(--tw-gradient-to-position)}.dark\:to-teal-200\/5:is(.dark *){--tw-gradient-to:rgba(153,246,228,.05) var(--tw-gradient-to-position)}.dark\:to-teal-200\/50:is(.dark *){--tw-gradient-to:rgba(153,246,228,.5) var(--tw-gradient-to-position)}.dark\:to-teal-200\/55:is(.dark *){--tw-gradient-to:rgba(153,246,228,.55) var(--tw-gradient-to-position)}.dark\:to-teal-200\/60:is(.dark *){--tw-gradient-to:rgba(153,246,228,.6) var(--tw-gradient-to-position)}.dark\:to-teal-200\/65:is(.dark *){--tw-gradient-to:rgba(153,246,228,.65) var(--tw-gradient-to-position)}.dark\:to-teal-200\/70:is(.dark *){--tw-gradient-to:rgba(153,246,228,.7) var(--tw-gradient-to-position)}.dark\:to-teal-200\/75:is(.dark *){--tw-gradient-to:rgba(153,246,228,.75) var(--tw-gradient-to-position)}.dark\:to-teal-200\/80:is(.dark *){--tw-gradient-to:rgba(153,246,228,.8) var(--tw-gradient-to-position)}.dark\:to-teal-200\/85:is(.dark *){--tw-gradient-to:rgba(153,246,228,.85) var(--tw-gradient-to-position)}.dark\:to-teal-200\/90:is(.dark *){--tw-gradient-to:rgba(153,246,228,.9) var(--tw-gradient-to-position)}.dark\:to-teal-200\/95:is(.dark *){--tw-gradient-to:rgba(153,246,228,.95) var(--tw-gradient-to-position)}.dark\:to-teal-50:is(.dark *){--tw-gradient-to:#f0fdfa var(--tw-gradient-to-position)}.dark\:to-teal-50\/0:is(.dark *){--tw-gradient-to:rgba(240,253,250,0) var(--tw-gradient-to-position)}.dark\:to-teal-50\/10:is(.dark *){--tw-gradient-to:rgba(240,253,250,.1) var(--tw-gradient-to-position)}.dark\:to-teal-50\/100:is(.dark *){--tw-gradient-to:#f0fdfa var(--tw-gradient-to-position)}.dark\:to-teal-50\/15:is(.dark *){--tw-gradient-to:rgba(240,253,250,.15) var(--tw-gradient-to-position)}.dark\:to-teal-50\/20:is(.dark *){--tw-gradient-to:rgba(240,253,250,.2) var(--tw-gradient-to-position)}.dark\:to-teal-50\/25:is(.dark *){--tw-gradient-to:rgba(240,253,250,.25) var(--tw-gradient-to-position)}.dark\:to-teal-50\/30:is(.dark *){--tw-gradient-to:rgba(240,253,250,.3) var(--tw-gradient-to-position)}.dark\:to-teal-50\/35:is(.dark *){--tw-gradient-to:rgba(240,253,250,.35) var(--tw-gradient-to-position)}.dark\:to-teal-50\/40:is(.dark *){--tw-gradient-to:rgba(240,253,250,.4) var(--tw-gradient-to-position)}.dark\:to-teal-50\/45:is(.dark *){--tw-gradient-to:rgba(240,253,250,.45) var(--tw-gradient-to-position)}.dark\:to-teal-50\/5:is(.dark *){--tw-gradient-to:rgba(240,253,250,.05) var(--tw-gradient-to-position)}.dark\:to-teal-50\/50:is(.dark *){--tw-gradient-to:rgba(240,253,250,.5) var(--tw-gradient-to-position)}.dark\:to-teal-50\/55:is(.dark *){--tw-gradient-to:rgba(240,253,250,.55) var(--tw-gradient-to-position)}.dark\:to-teal-50\/60:is(.dark *){--tw-gradient-to:rgba(240,253,250,.6) var(--tw-gradient-to-position)}.dark\:to-teal-50\/65:is(.dark *){--tw-gradient-to:rgba(240,253,250,.65) var(--tw-gradient-to-position)}.dark\:to-teal-50\/70:is(.dark *){--tw-gradient-to:rgba(240,253,250,.7) var(--tw-gradient-to-position)}.dark\:to-teal-50\/75:is(.dark *){--tw-gradient-to:rgba(240,253,250,.75) var(--tw-gradient-to-position)}.dark\:to-teal-50\/80:is(.dark *){--tw-gradient-to:rgba(240,253,250,.8) var(--tw-gradient-to-position)}.dark\:to-teal-50\/85:is(.dark *){--tw-gradient-to:rgba(240,253,250,.85) var(--tw-gradient-to-position)}.dark\:to-teal-50\/90:is(.dark *){--tw-gradient-to:rgba(240,253,250,.9) var(--tw-gradient-to-position)}.dark\:to-teal-50\/95:is(.dark *){--tw-gradient-to:rgba(240,253,250,.95) var(--tw-gradient-to-position)}.dark\:to-teal-500:is(.dark *){--tw-gradient-to:#14b8a6 var(--tw-gradient-to-position)}.dark\:to-teal-500\/0:is(.dark *){--tw-gradient-to:rgba(20,184,166,0) var(--tw-gradient-to-position)}.dark\:to-teal-500\/10:is(.dark *){--tw-gradient-to:rgba(20,184,166,.1) var(--tw-gradient-to-position)}.dark\:to-teal-500\/100:is(.dark *){--tw-gradient-to:#14b8a6 var(--tw-gradient-to-position)}.dark\:to-teal-500\/15:is(.dark *){--tw-gradient-to:rgba(20,184,166,.15) var(--tw-gradient-to-position)}.dark\:to-teal-500\/20:is(.dark *){--tw-gradient-to:rgba(20,184,166,.2) var(--tw-gradient-to-position)}.dark\:to-teal-500\/25:is(.dark *){--tw-gradient-to:rgba(20,184,166,.25) var(--tw-gradient-to-position)}.dark\:to-teal-500\/30:is(.dark *){--tw-gradient-to:rgba(20,184,166,.3) var(--tw-gradient-to-position)}.dark\:to-teal-500\/35:is(.dark *){--tw-gradient-to:rgba(20,184,166,.35) var(--tw-gradient-to-position)}.dark\:to-teal-500\/40:is(.dark *){--tw-gradient-to:rgba(20,184,166,.4) var(--tw-gradient-to-position)}.dark\:to-teal-500\/45:is(.dark *){--tw-gradient-to:rgba(20,184,166,.45) var(--tw-gradient-to-position)}.dark\:to-teal-500\/5:is(.dark *){--tw-gradient-to:rgba(20,184,166,.05) var(--tw-gradient-to-position)}.dark\:to-teal-500\/50:is(.dark *){--tw-gradient-to:rgba(20,184,166,.5) var(--tw-gradient-to-position)}.dark\:to-teal-500\/55:is(.dark *){--tw-gradient-to:rgba(20,184,166,.55) var(--tw-gradient-to-position)}.dark\:to-teal-500\/60:is(.dark *){--tw-gradient-to:rgba(20,184,166,.6) var(--tw-gradient-to-position)}.dark\:to-teal-500\/65:is(.dark *){--tw-gradient-to:rgba(20,184,166,.65) var(--tw-gradient-to-position)}.dark\:to-teal-500\/70:is(.dark *){--tw-gradient-to:rgba(20,184,166,.7) var(--tw-gradient-to-position)}.dark\:to-teal-500\/75:is(.dark *){--tw-gradient-to:rgba(20,184,166,.75) var(--tw-gradient-to-position)}.dark\:to-teal-500\/80:is(.dark *){--tw-gradient-to:rgba(20,184,166,.8) var(--tw-gradient-to-position)}.dark\:to-teal-500\/85:is(.dark *){--tw-gradient-to:rgba(20,184,166,.85) var(--tw-gradient-to-position)}.dark\:to-teal-500\/90:is(.dark *){--tw-gradient-to:rgba(20,184,166,.9) var(--tw-gradient-to-position)}.dark\:to-teal-500\/95:is(.dark *){--tw-gradient-to:rgba(20,184,166,.95) var(--tw-gradient-to-position)}.dark\:to-teal-950:is(.dark *){--tw-gradient-to:#042f2e var(--tw-gradient-to-position)}.dark\:to-teal-950\/0:is(.dark *){--tw-gradient-to:rgba(4,47,46,0) var(--tw-gradient-to-position)}.dark\:to-teal-950\/10:is(.dark *){--tw-gradient-to:rgba(4,47,46,.1) var(--tw-gradient-to-position)}.dark\:to-teal-950\/100:is(.dark *){--tw-gradient-to:#042f2e var(--tw-gradient-to-position)}.dark\:to-teal-950\/15:is(.dark *){--tw-gradient-to:rgba(4,47,46,.15) var(--tw-gradient-to-position)}.dark\:to-teal-950\/20:is(.dark *){--tw-gradient-to:rgba(4,47,46,.2) var(--tw-gradient-to-position)}.dark\:to-teal-950\/25:is(.dark *){--tw-gradient-to:rgba(4,47,46,.25) var(--tw-gradient-to-position)}.dark\:to-teal-950\/30:is(.dark *){--tw-gradient-to:rgba(4,47,46,.3) var(--tw-gradient-to-position)}.dark\:to-teal-950\/35:is(.dark *){--tw-gradient-to:rgba(4,47,46,.35) var(--tw-gradient-to-position)}.dark\:to-teal-950\/40:is(.dark *){--tw-gradient-to:rgba(4,47,46,.4) var(--tw-gradient-to-position)}.dark\:to-teal-950\/45:is(.dark *){--tw-gradient-to:rgba(4,47,46,.45) var(--tw-gradient-to-position)}.dark\:to-teal-950\/5:is(.dark *){--tw-gradient-to:rgba(4,47,46,.05) var(--tw-gradient-to-position)}.dark\:to-teal-950\/50:is(.dark *){--tw-gradient-to:rgba(4,47,46,.5) var(--tw-gradient-to-position)}.dark\:to-teal-950\/55:is(.dark *){--tw-gradient-to:rgba(4,47,46,.55) var(--tw-gradient-to-position)}.dark\:to-teal-950\/60:is(.dark *){--tw-gradient-to:rgba(4,47,46,.6) var(--tw-gradient-to-position)}.dark\:to-teal-950\/65:is(.dark *){--tw-gradient-to:rgba(4,47,46,.65) var(--tw-gradient-to-position)}.dark\:to-teal-950\/70:is(.dark *){--tw-gradient-to:rgba(4,47,46,.7) var(--tw-gradient-to-position)}.dark\:to-teal-950\/75:is(.dark *){--tw-gradient-to:rgba(4,47,46,.75) var(--tw-gradient-to-position)}.dark\:to-teal-950\/80:is(.dark *){--tw-gradient-to:rgba(4,47,46,.8) var(--tw-gradient-to-position)}.dark\:to-teal-950\/85:is(.dark *){--tw-gradient-to:rgba(4,47,46,.85) var(--tw-gradient-to-position)}.dark\:to-teal-950\/90:is(.dark *){--tw-gradient-to:rgba(4,47,46,.9) var(--tw-gradient-to-position)}.dark\:to-teal-950\/95:is(.dark *){--tw-gradient-to:rgba(4,47,46,.95) var(--tw-gradient-to-position)}.dark\:to-violet-100:is(.dark *){--tw-gradient-to:#ede9fe var(--tw-gradient-to-position)}.dark\:to-violet-100\/0:is(.dark *){--tw-gradient-to:rgba(237,233,254,0) var(--tw-gradient-to-position)}.dark\:to-violet-100\/10:is(.dark *){--tw-gradient-to:rgba(237,233,254,.1) var(--tw-gradient-to-position)}.dark\:to-violet-100\/100:is(.dark *){--tw-gradient-to:#ede9fe var(--tw-gradient-to-position)}.dark\:to-violet-100\/15:is(.dark *){--tw-gradient-to:rgba(237,233,254,.15) var(--tw-gradient-to-position)}.dark\:to-violet-100\/20:is(.dark *){--tw-gradient-to:rgba(237,233,254,.2) var(--tw-gradient-to-position)}.dark\:to-violet-100\/25:is(.dark *){--tw-gradient-to:rgba(237,233,254,.25) var(--tw-gradient-to-position)}.dark\:to-violet-100\/30:is(.dark *){--tw-gradient-to:rgba(237,233,254,.3) var(--tw-gradient-to-position)}.dark\:to-violet-100\/35:is(.dark *){--tw-gradient-to:rgba(237,233,254,.35) var(--tw-gradient-to-position)}.dark\:to-violet-100\/40:is(.dark *){--tw-gradient-to:rgba(237,233,254,.4) var(--tw-gradient-to-position)}.dark\:to-violet-100\/45:is(.dark *){--tw-gradient-to:rgba(237,233,254,.45) var(--tw-gradient-to-position)}.dark\:to-violet-100\/5:is(.dark *){--tw-gradient-to:rgba(237,233,254,.05) var(--tw-gradient-to-position)}.dark\:to-violet-100\/50:is(.dark *){--tw-gradient-to:rgba(237,233,254,.5) var(--tw-gradient-to-position)}.dark\:to-violet-100\/55:is(.dark *){--tw-gradient-to:rgba(237,233,254,.55) var(--tw-gradient-to-position)}.dark\:to-violet-100\/60:is(.dark *){--tw-gradient-to:rgba(237,233,254,.6) var(--tw-gradient-to-position)}.dark\:to-violet-100\/65:is(.dark *){--tw-gradient-to:rgba(237,233,254,.65) var(--tw-gradient-to-position)}.dark\:to-violet-100\/70:is(.dark *){--tw-gradient-to:rgba(237,233,254,.7) var(--tw-gradient-to-position)}.dark\:to-violet-100\/75:is(.dark *){--tw-gradient-to:rgba(237,233,254,.75) var(--tw-gradient-to-position)}.dark\:to-violet-100\/80:is(.dark *){--tw-gradient-to:rgba(237,233,254,.8) var(--tw-gradient-to-position)}.dark\:to-violet-100\/85:is(.dark *){--tw-gradient-to:rgba(237,233,254,.85) var(--tw-gradient-to-position)}.dark\:to-violet-100\/90:is(.dark *){--tw-gradient-to:rgba(237,233,254,.9) var(--tw-gradient-to-position)}.dark\:to-violet-100\/95:is(.dark *){--tw-gradient-to:rgba(237,233,254,.95) var(--tw-gradient-to-position)}.dark\:to-violet-200:is(.dark *){--tw-gradient-to:#ddd6fe var(--tw-gradient-to-position)}.dark\:to-violet-200\/0:is(.dark *){--tw-gradient-to:rgba(221,214,254,0) var(--tw-gradient-to-position)}.dark\:to-violet-200\/10:is(.dark *){--tw-gradient-to:rgba(221,214,254,.1) var(--tw-gradient-to-position)}.dark\:to-violet-200\/100:is(.dark *){--tw-gradient-to:#ddd6fe var(--tw-gradient-to-position)}.dark\:to-violet-200\/15:is(.dark *){--tw-gradient-to:rgba(221,214,254,.15) var(--tw-gradient-to-position)}.dark\:to-violet-200\/20:is(.dark *){--tw-gradient-to:rgba(221,214,254,.2) var(--tw-gradient-to-position)}.dark\:to-violet-200\/25:is(.dark *){--tw-gradient-to:rgba(221,214,254,.25) var(--tw-gradient-to-position)}.dark\:to-violet-200\/30:is(.dark *){--tw-gradient-to:rgba(221,214,254,.3) var(--tw-gradient-to-position)}.dark\:to-violet-200\/35:is(.dark *){--tw-gradient-to:rgba(221,214,254,.35) var(--tw-gradient-to-position)}.dark\:to-violet-200\/40:is(.dark *){--tw-gradient-to:rgba(221,214,254,.4) var(--tw-gradient-to-position)}.dark\:to-violet-200\/45:is(.dark *){--tw-gradient-to:rgba(221,214,254,.45) var(--tw-gradient-to-position)}.dark\:to-violet-200\/5:is(.dark *){--tw-gradient-to:rgba(221,214,254,.05) var(--tw-gradient-to-position)}.dark\:to-violet-200\/50:is(.dark *){--tw-gradient-to:rgba(221,214,254,.5) var(--tw-gradient-to-position)}.dark\:to-violet-200\/55:is(.dark *){--tw-gradient-to:rgba(221,214,254,.55) var(--tw-gradient-to-position)}.dark\:to-violet-200\/60:is(.dark *){--tw-gradient-to:rgba(221,214,254,.6) var(--tw-gradient-to-position)}.dark\:to-violet-200\/65:is(.dark *){--tw-gradient-to:rgba(221,214,254,.65) var(--tw-gradient-to-position)}.dark\:to-violet-200\/70:is(.dark *){--tw-gradient-to:rgba(221,214,254,.7) var(--tw-gradient-to-position)}.dark\:to-violet-200\/75:is(.dark *){--tw-gradient-to:rgba(221,214,254,.75) var(--tw-gradient-to-position)}.dark\:to-violet-200\/80:is(.dark *){--tw-gradient-to:rgba(221,214,254,.8) var(--tw-gradient-to-position)}.dark\:to-violet-200\/85:is(.dark *){--tw-gradient-to:rgba(221,214,254,.85) var(--tw-gradient-to-position)}.dark\:to-violet-200\/90:is(.dark *){--tw-gradient-to:rgba(221,214,254,.9) var(--tw-gradient-to-position)}.dark\:to-violet-200\/95:is(.dark *){--tw-gradient-to:rgba(221,214,254,.95) var(--tw-gradient-to-position)}.dark\:to-violet-50:is(.dark *){--tw-gradient-to:#f5f3ff var(--tw-gradient-to-position)}.dark\:to-violet-50\/0:is(.dark *){--tw-gradient-to:rgba(245,243,255,0) var(--tw-gradient-to-position)}.dark\:to-violet-50\/10:is(.dark *){--tw-gradient-to:rgba(245,243,255,.1) var(--tw-gradient-to-position)}.dark\:to-violet-50\/100:is(.dark *){--tw-gradient-to:#f5f3ff var(--tw-gradient-to-position)}.dark\:to-violet-50\/15:is(.dark *){--tw-gradient-to:rgba(245,243,255,.15) var(--tw-gradient-to-position)}.dark\:to-violet-50\/20:is(.dark *){--tw-gradient-to:rgba(245,243,255,.2) var(--tw-gradient-to-position)}.dark\:to-violet-50\/25:is(.dark *){--tw-gradient-to:rgba(245,243,255,.25) var(--tw-gradient-to-position)}.dark\:to-violet-50\/30:is(.dark *){--tw-gradient-to:rgba(245,243,255,.3) var(--tw-gradient-to-position)}.dark\:to-violet-50\/35:is(.dark *){--tw-gradient-to:rgba(245,243,255,.35) var(--tw-gradient-to-position)}.dark\:to-violet-50\/40:is(.dark *){--tw-gradient-to:rgba(245,243,255,.4) var(--tw-gradient-to-position)}.dark\:to-violet-50\/45:is(.dark *){--tw-gradient-to:rgba(245,243,255,.45) var(--tw-gradient-to-position)}.dark\:to-violet-50\/5:is(.dark *){--tw-gradient-to:rgba(245,243,255,.05) var(--tw-gradient-to-position)}.dark\:to-violet-50\/50:is(.dark *){--tw-gradient-to:rgba(245,243,255,.5) var(--tw-gradient-to-position)}.dark\:to-violet-50\/55:is(.dark *){--tw-gradient-to:rgba(245,243,255,.55) var(--tw-gradient-to-position)}.dark\:to-violet-50\/60:is(.dark *){--tw-gradient-to:rgba(245,243,255,.6) var(--tw-gradient-to-position)}.dark\:to-violet-50\/65:is(.dark *){--tw-gradient-to:rgba(245,243,255,.65) var(--tw-gradient-to-position)}.dark\:to-violet-50\/70:is(.dark *){--tw-gradient-to:rgba(245,243,255,.7) var(--tw-gradient-to-position)}.dark\:to-violet-50\/75:is(.dark *){--tw-gradient-to:rgba(245,243,255,.75) var(--tw-gradient-to-position)}.dark\:to-violet-50\/80:is(.dark *){--tw-gradient-to:rgba(245,243,255,.8) var(--tw-gradient-to-position)}.dark\:to-violet-50\/85:is(.dark *){--tw-gradient-to:rgba(245,243,255,.85) var(--tw-gradient-to-position)}.dark\:to-violet-50\/90:is(.dark *){--tw-gradient-to:rgba(245,243,255,.9) var(--tw-gradient-to-position)}.dark\:to-violet-50\/95:is(.dark *){--tw-gradient-to:rgba(245,243,255,.95) var(--tw-gradient-to-position)}.dark\:to-violet-500:is(.dark *){--tw-gradient-to:#8b5cf6 var(--tw-gradient-to-position)}.dark\:to-violet-500\/0:is(.dark *){--tw-gradient-to:rgba(139,92,246,0) var(--tw-gradient-to-position)}.dark\:to-violet-500\/10:is(.dark *){--tw-gradient-to:rgba(139,92,246,.1) var(--tw-gradient-to-position)}.dark\:to-violet-500\/100:is(.dark *){--tw-gradient-to:#8b5cf6 var(--tw-gradient-to-position)}.dark\:to-violet-500\/15:is(.dark *){--tw-gradient-to:rgba(139,92,246,.15) var(--tw-gradient-to-position)}.dark\:to-violet-500\/20:is(.dark *){--tw-gradient-to:rgba(139,92,246,.2) var(--tw-gradient-to-position)}.dark\:to-violet-500\/25:is(.dark *){--tw-gradient-to:rgba(139,92,246,.25) var(--tw-gradient-to-position)}.dark\:to-violet-500\/30:is(.dark *){--tw-gradient-to:rgba(139,92,246,.3) var(--tw-gradient-to-position)}.dark\:to-violet-500\/35:is(.dark *){--tw-gradient-to:rgba(139,92,246,.35) var(--tw-gradient-to-position)}.dark\:to-violet-500\/40:is(.dark *){--tw-gradient-to:rgba(139,92,246,.4) var(--tw-gradient-to-position)}.dark\:to-violet-500\/45:is(.dark *){--tw-gradient-to:rgba(139,92,246,.45) var(--tw-gradient-to-position)}.dark\:to-violet-500\/5:is(.dark *){--tw-gradient-to:rgba(139,92,246,.05) var(--tw-gradient-to-position)}.dark\:to-violet-500\/50:is(.dark *){--tw-gradient-to:rgba(139,92,246,.5) var(--tw-gradient-to-position)}.dark\:to-violet-500\/55:is(.dark *){--tw-gradient-to:rgba(139,92,246,.55) var(--tw-gradient-to-position)}.dark\:to-violet-500\/60:is(.dark *){--tw-gradient-to:rgba(139,92,246,.6) var(--tw-gradient-to-position)}.dark\:to-violet-500\/65:is(.dark *){--tw-gradient-to:rgba(139,92,246,.65) var(--tw-gradient-to-position)}.dark\:to-violet-500\/70:is(.dark *){--tw-gradient-to:rgba(139,92,246,.7) var(--tw-gradient-to-position)}.dark\:to-violet-500\/75:is(.dark *){--tw-gradient-to:rgba(139,92,246,.75) var(--tw-gradient-to-position)}.dark\:to-violet-500\/80:is(.dark *){--tw-gradient-to:rgba(139,92,246,.8) var(--tw-gradient-to-position)}.dark\:to-violet-500\/85:is(.dark *){--tw-gradient-to:rgba(139,92,246,.85) var(--tw-gradient-to-position)}.dark\:to-violet-500\/90:is(.dark *){--tw-gradient-to:rgba(139,92,246,.9) var(--tw-gradient-to-position)}.dark\:to-violet-500\/95:is(.dark *){--tw-gradient-to:rgba(139,92,246,.95) var(--tw-gradient-to-position)}.dark\:to-violet-950:is(.dark *){--tw-gradient-to:#2e1065 var(--tw-gradient-to-position)}.dark\:to-violet-950\/0:is(.dark *){--tw-gradient-to:rgba(46,16,101,0) var(--tw-gradient-to-position)}.dark\:to-violet-950\/10:is(.dark *){--tw-gradient-to:rgba(46,16,101,.1) var(--tw-gradient-to-position)}.dark\:to-violet-950\/100:is(.dark *){--tw-gradient-to:#2e1065 var(--tw-gradient-to-position)}.dark\:to-violet-950\/15:is(.dark *){--tw-gradient-to:rgba(46,16,101,.15) var(--tw-gradient-to-position)}.dark\:to-violet-950\/20:is(.dark *){--tw-gradient-to:rgba(46,16,101,.2) var(--tw-gradient-to-position)}.dark\:to-violet-950\/25:is(.dark *){--tw-gradient-to:rgba(46,16,101,.25) var(--tw-gradient-to-position)}.dark\:to-violet-950\/30:is(.dark *){--tw-gradient-to:rgba(46,16,101,.3) var(--tw-gradient-to-position)}.dark\:to-violet-950\/35:is(.dark *){--tw-gradient-to:rgba(46,16,101,.35) var(--tw-gradient-to-position)}.dark\:to-violet-950\/40:is(.dark *){--tw-gradient-to:rgba(46,16,101,.4) var(--tw-gradient-to-position)}.dark\:to-violet-950\/45:is(.dark *){--tw-gradient-to:rgba(46,16,101,.45) var(--tw-gradient-to-position)}.dark\:to-violet-950\/5:is(.dark *){--tw-gradient-to:rgba(46,16,101,.05) var(--tw-gradient-to-position)}.dark\:to-violet-950\/50:is(.dark *){--tw-gradient-to:rgba(46,16,101,.5) var(--tw-gradient-to-position)}.dark\:to-violet-950\/55:is(.dark *){--tw-gradient-to:rgba(46,16,101,.55) var(--tw-gradient-to-position)}.dark\:to-violet-950\/60:is(.dark *){--tw-gradient-to:rgba(46,16,101,.6) var(--tw-gradient-to-position)}.dark\:to-violet-950\/65:is(.dark *){--tw-gradient-to:rgba(46,16,101,.65) var(--tw-gradient-to-position)}.dark\:to-violet-950\/70:is(.dark *){--tw-gradient-to:rgba(46,16,101,.7) var(--tw-gradient-to-position)}.dark\:to-violet-950\/75:is(.dark *){--tw-gradient-to:rgba(46,16,101,.75) var(--tw-gradient-to-position)}.dark\:to-violet-950\/80:is(.dark *){--tw-gradient-to:rgba(46,16,101,.8) var(--tw-gradient-to-position)}.dark\:to-violet-950\/85:is(.dark *){--tw-gradient-to:rgba(46,16,101,.85) var(--tw-gradient-to-position)}.dark\:to-violet-950\/90:is(.dark *){--tw-gradient-to:rgba(46,16,101,.9) var(--tw-gradient-to-position)}.dark\:to-violet-950\/95:is(.dark *){--tw-gradient-to:rgba(46,16,101,.95) var(--tw-gradient-to-position)}.dark\:to-yellow-100:is(.dark *){--tw-gradient-to:#fef9c3 var(--tw-gradient-to-position)}.dark\:to-yellow-100\/0:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,0) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/10:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.1) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/100:is(.dark *){--tw-gradient-to:#fef9c3 var(--tw-gradient-to-position)}.dark\:to-yellow-100\/15:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.15) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/20:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.2) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/25:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.25) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/30:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.3) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/35:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.35) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/40:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.4) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/45:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.45) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/5:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.05) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/50:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.5) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/55:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.55) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/60:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.6) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/65:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.65) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/70:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.7) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/75:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.75) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/80:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.8) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/85:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.85) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/90:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.9) var(--tw-gradient-to-position)}.dark\:to-yellow-100\/95:is(.dark *){--tw-gradient-to:hsla(55,97%,88%,.95) var(--tw-gradient-to-position)}.dark\:to-yellow-200:is(.dark *){--tw-gradient-to:#fef08a var(--tw-gradient-to-position)}.dark\:to-yellow-200\/0:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,0) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/10:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.1) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/100:is(.dark *){--tw-gradient-to:#fef08a var(--tw-gradient-to-position)}.dark\:to-yellow-200\/15:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.15) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/20:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.2) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/25:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.25) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/30:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.3) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/35:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.35) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/40:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.4) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/45:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.45) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/5:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.05) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/50:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.5) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/55:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.55) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/60:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.6) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/65:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.65) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/70:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.7) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/75:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.75) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/80:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.8) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/85:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.85) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/90:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.9) var(--tw-gradient-to-position)}.dark\:to-yellow-200\/95:is(.dark *){--tw-gradient-to:hsla(53,98%,77%,.95) var(--tw-gradient-to-position)}.dark\:to-yellow-50:is(.dark *){--tw-gradient-to:#fefce8 var(--tw-gradient-to-position)}.dark\:to-yellow-50\/0:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,0) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/10:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.1) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/100:is(.dark *){--tw-gradient-to:#fefce8 var(--tw-gradient-to-position)}.dark\:to-yellow-50\/15:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.15) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/20:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.2) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/25:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.25) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/30:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.3) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/35:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.35) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/40:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.4) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/45:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.45) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/5:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.05) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/50:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.5) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/55:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.55) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/60:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.6) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/65:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.65) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/70:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.7) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/75:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.75) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/80:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.8) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/85:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.85) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/90:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.9) var(--tw-gradient-to-position)}.dark\:to-yellow-50\/95:is(.dark *){--tw-gradient-to:hsla(55,92%,95%,.95) var(--tw-gradient-to-position)}.dark\:to-yellow-500:is(.dark *){--tw-gradient-to:#eab308 var(--tw-gradient-to-position)}.dark\:to-yellow-500\/0:is(.dark *){--tw-gradient-to:rgba(234,179,8,0) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/10:is(.dark *){--tw-gradient-to:rgba(234,179,8,.1) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/100:is(.dark *){--tw-gradient-to:#eab308 var(--tw-gradient-to-position)}.dark\:to-yellow-500\/15:is(.dark *){--tw-gradient-to:rgba(234,179,8,.15) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/20:is(.dark *){--tw-gradient-to:rgba(234,179,8,.2) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/25:is(.dark *){--tw-gradient-to:rgba(234,179,8,.25) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/30:is(.dark *){--tw-gradient-to:rgba(234,179,8,.3) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/35:is(.dark *){--tw-gradient-to:rgba(234,179,8,.35) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/40:is(.dark *){--tw-gradient-to:rgba(234,179,8,.4) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/45:is(.dark *){--tw-gradient-to:rgba(234,179,8,.45) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/5:is(.dark *){--tw-gradient-to:rgba(234,179,8,.05) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/50:is(.dark *){--tw-gradient-to:rgba(234,179,8,.5) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/55:is(.dark *){--tw-gradient-to:rgba(234,179,8,.55) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/60:is(.dark *){--tw-gradient-to:rgba(234,179,8,.6) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/65:is(.dark *){--tw-gradient-to:rgba(234,179,8,.65) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/70:is(.dark *){--tw-gradient-to:rgba(234,179,8,.7) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/75:is(.dark *){--tw-gradient-to:rgba(234,179,8,.75) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/80:is(.dark *){--tw-gradient-to:rgba(234,179,8,.8) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/85:is(.dark *){--tw-gradient-to:rgba(234,179,8,.85) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/90:is(.dark *){--tw-gradient-to:rgba(234,179,8,.9) var(--tw-gradient-to-position)}.dark\:to-yellow-500\/95:is(.dark *){--tw-gradient-to:rgba(234,179,8,.95) var(--tw-gradient-to-position)}.dark\:to-yellow-950:is(.dark *){--tw-gradient-to:#422006 var(--tw-gradient-to-position)}.dark\:to-yellow-950\/0:is(.dark *){--tw-gradient-to:rgba(66,32,6,0) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/10:is(.dark *){--tw-gradient-to:rgba(66,32,6,.1) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/100:is(.dark *){--tw-gradient-to:#422006 var(--tw-gradient-to-position)}.dark\:to-yellow-950\/15:is(.dark *){--tw-gradient-to:rgba(66,32,6,.15) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/20:is(.dark *){--tw-gradient-to:rgba(66,32,6,.2) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/25:is(.dark *){--tw-gradient-to:rgba(66,32,6,.25) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/30:is(.dark *){--tw-gradient-to:rgba(66,32,6,.3) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/35:is(.dark *){--tw-gradient-to:rgba(66,32,6,.35) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/40:is(.dark *){--tw-gradient-to:rgba(66,32,6,.4) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/45:is(.dark *){--tw-gradient-to:rgba(66,32,6,.45) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/5:is(.dark *){--tw-gradient-to:rgba(66,32,6,.05) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/50:is(.dark *){--tw-gradient-to:rgba(66,32,6,.5) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/55:is(.dark *){--tw-gradient-to:rgba(66,32,6,.55) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/60:is(.dark *){--tw-gradient-to:rgba(66,32,6,.6) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/65:is(.dark *){--tw-gradient-to:rgba(66,32,6,.65) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/70:is(.dark *){--tw-gradient-to:rgba(66,32,6,.7) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/75:is(.dark *){--tw-gradient-to:rgba(66,32,6,.75) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/80:is(.dark *){--tw-gradient-to:rgba(66,32,6,.8) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/85:is(.dark *){--tw-gradient-to:rgba(66,32,6,.85) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/90:is(.dark *){--tw-gradient-to:rgba(66,32,6,.9) var(--tw-gradient-to-position)}.dark\:to-yellow-950\/95:is(.dark *){--tw-gradient-to:rgba(66,32,6,.95) var(--tw-gradient-to-position)}.dark\:to-zinc-100:is(.dark *){--tw-gradient-to:#f4f4f5 var(--tw-gradient-to-position)}.dark\:to-zinc-100\/0:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,0) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/10:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.1) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/100:is(.dark *){--tw-gradient-to:#f4f4f5 var(--tw-gradient-to-position)}.dark\:to-zinc-100\/15:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.15) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/20:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.2) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/25:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.25) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/30:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.3) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/35:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.35) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/40:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.4) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/45:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.45) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/5:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.05) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/50:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.5) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/55:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.55) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/60:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.6) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/65:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.65) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/70:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.7) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/75:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.75) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/80:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.8) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/85:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.85) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/90:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.9) var(--tw-gradient-to-position)}.dark\:to-zinc-100\/95:is(.dark *){--tw-gradient-to:hsla(240,5%,96%,.95) var(--tw-gradient-to-position)}.dark\:to-zinc-200:is(.dark *){--tw-gradient-to:#e4e4e7 var(--tw-gradient-to-position)}.dark\:to-zinc-200\/0:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,0) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/10:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.1) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/100:is(.dark *){--tw-gradient-to:#e4e4e7 var(--tw-gradient-to-position)}.dark\:to-zinc-200\/15:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.15) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/20:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.2) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/25:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.25) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/30:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.3) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/35:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.35) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/40:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.4) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/45:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.45) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/5:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.05) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/50:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.5) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/55:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.55) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/60:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.6) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/65:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.65) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/70:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.7) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/75:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.75) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/80:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.8) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/85:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.85) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/90:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.9) var(--tw-gradient-to-position)}.dark\:to-zinc-200\/95:is(.dark *){--tw-gradient-to:hsla(240,6%,90%,.95) var(--tw-gradient-to-position)}.dark\:to-zinc-50:is(.dark *){--tw-gradient-to:#fafafa var(--tw-gradient-to-position)}.dark\:to-zinc-50\/0:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,0) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/10:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.1) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/100:is(.dark *){--tw-gradient-to:#fafafa var(--tw-gradient-to-position)}.dark\:to-zinc-50\/15:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.15) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/20:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.2) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/25:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.25) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/30:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.3) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/35:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.35) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/40:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.4) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/45:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.45) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/5:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.05) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/50:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.5) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/55:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.55) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/60:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.6) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/65:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.65) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/70:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.7) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/75:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.75) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/80:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.8) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/85:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.85) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/90:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.9) var(--tw-gradient-to-position)}.dark\:to-zinc-50\/95:is(.dark *){--tw-gradient-to:hsla(0,0%,98%,.95) var(--tw-gradient-to-position)}.dark\:to-zinc-500:is(.dark *){--tw-gradient-to:#71717a var(--tw-gradient-to-position)}.dark\:to-zinc-500\/0:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,0) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/10:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.1) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/100:is(.dark *){--tw-gradient-to:#71717a var(--tw-gradient-to-position)}.dark\:to-zinc-500\/15:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.15) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/20:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.2) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/25:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.25) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/30:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.3) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/35:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.35) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/40:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.4) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/45:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.45) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/5:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.05) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/50:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.5) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/55:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.55) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/60:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.6) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/65:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.65) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/70:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.7) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/75:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.75) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/80:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.8) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/85:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.85) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/90:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.9) var(--tw-gradient-to-position)}.dark\:to-zinc-500\/95:is(.dark *){--tw-gradient-to:hsla(240,4%,46%,.95) var(--tw-gradient-to-position)}.dark\:to-zinc-950:is(.dark *){--tw-gradient-to:#09090b var(--tw-gradient-to-position)}.dark\:to-zinc-950\/0:is(.dark *){--tw-gradient-to:rgba(9,9,11,0) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/10:is(.dark *){--tw-gradient-to:rgba(9,9,11,.1) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/100:is(.dark *){--tw-gradient-to:#09090b var(--tw-gradient-to-position)}.dark\:to-zinc-950\/15:is(.dark *){--tw-gradient-to:rgba(9,9,11,.15) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/20:is(.dark *){--tw-gradient-to:rgba(9,9,11,.2) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/25:is(.dark *){--tw-gradient-to:rgba(9,9,11,.25) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/30:is(.dark *){--tw-gradient-to:rgba(9,9,11,.3) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/35:is(.dark *){--tw-gradient-to:rgba(9,9,11,.35) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/40:is(.dark *){--tw-gradient-to:rgba(9,9,11,.4) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/45:is(.dark *){--tw-gradient-to:rgba(9,9,11,.45) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/5:is(.dark *){--tw-gradient-to:rgba(9,9,11,.05) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/50:is(.dark *){--tw-gradient-to:rgba(9,9,11,.5) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/55:is(.dark *){--tw-gradient-to:rgba(9,9,11,.55) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/60:is(.dark *){--tw-gradient-to:rgba(9,9,11,.6) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/65:is(.dark *){--tw-gradient-to:rgba(9,9,11,.65) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/70:is(.dark *){--tw-gradient-to:rgba(9,9,11,.7) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/75:is(.dark *){--tw-gradient-to:rgba(9,9,11,.75) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/80:is(.dark *){--tw-gradient-to:rgba(9,9,11,.8) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/85:is(.dark *){--tw-gradient-to:rgba(9,9,11,.85) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/90:is(.dark *){--tw-gradient-to:rgba(9,9,11,.9) var(--tw-gradient-to-position)}.dark\:to-zinc-950\/95:is(.dark *){--tw-gradient-to:rgba(9,9,11,.95) var(--tw-gradient-to-position)}.dark\:fill-zinc-300:is(.dark *){fill:#d4d4d8}.dark\:text-amber-100:is(.dark *){--tw-text-opacity:1;color:rgb(254 243 199/var(--tw-text-opacity,1))}.dark\:text-amber-100\/0:is(.dark *){color:hsla(48,96%,89%,0)}.dark\:text-amber-100\/10:is(.dark *){color:hsla(48,96%,89%,.1)}.dark\:text-amber-100\/100:is(.dark *){color:#fef3c7}.dark\:text-amber-100\/15:is(.dark *){color:hsla(48,96%,89%,.15)}.dark\:text-amber-100\/20:is(.dark *){color:hsla(48,96%,89%,.2)}.dark\:text-amber-100\/25:is(.dark *){color:hsla(48,96%,89%,.25)}.dark\:text-amber-100\/30:is(.dark *){color:hsla(48,96%,89%,.3)}.dark\:text-amber-100\/35:is(.dark *){color:hsla(48,96%,89%,.35)}.dark\:text-amber-100\/40:is(.dark *){color:hsla(48,96%,89%,.4)}.dark\:text-amber-100\/45:is(.dark *){color:hsla(48,96%,89%,.45)}.dark\:text-amber-100\/5:is(.dark *){color:hsla(48,96%,89%,.05)}.dark\:text-amber-100\/50:is(.dark *){color:hsla(48,96%,89%,.5)}.dark\:text-amber-100\/55:is(.dark *){color:hsla(48,96%,89%,.55)}.dark\:text-amber-100\/60:is(.dark *){color:hsla(48,96%,89%,.6)}.dark\:text-amber-100\/65:is(.dark *){color:hsla(48,96%,89%,.65)}.dark\:text-amber-100\/70:is(.dark *){color:hsla(48,96%,89%,.7)}.dark\:text-amber-100\/75:is(.dark *){color:hsla(48,96%,89%,.75)}.dark\:text-amber-100\/80:is(.dark *){color:hsla(48,96%,89%,.8)}.dark\:text-amber-100\/85:is(.dark *){color:hsla(48,96%,89%,.85)}.dark\:text-amber-100\/90:is(.dark *){color:hsla(48,96%,89%,.9)}.dark\:text-amber-100\/95:is(.dark *){color:hsla(48,96%,89%,.95)}.dark\:text-amber-200:is(.dark *){--tw-text-opacity:1;color:rgb(253 230 138/var(--tw-text-opacity,1))}.dark\:text-amber-200\/0:is(.dark *){color:hsla(48,97%,77%,0)}.dark\:text-amber-200\/10:is(.dark *){color:hsla(48,97%,77%,.1)}.dark\:text-amber-200\/100:is(.dark *){color:#fde68a}.dark\:text-amber-200\/15:is(.dark *){color:hsla(48,97%,77%,.15)}.dark\:text-amber-200\/20:is(.dark *){color:hsla(48,97%,77%,.2)}.dark\:text-amber-200\/25:is(.dark *){color:hsla(48,97%,77%,.25)}.dark\:text-amber-200\/30:is(.dark *){color:hsla(48,97%,77%,.3)}.dark\:text-amber-200\/35:is(.dark *){color:hsla(48,97%,77%,.35)}.dark\:text-amber-200\/40:is(.dark *){color:hsla(48,97%,77%,.4)}.dark\:text-amber-200\/45:is(.dark *){color:hsla(48,97%,77%,.45)}.dark\:text-amber-200\/5:is(.dark *){color:hsla(48,97%,77%,.05)}.dark\:text-amber-200\/50:is(.dark *){color:hsla(48,97%,77%,.5)}.dark\:text-amber-200\/55:is(.dark *){color:hsla(48,97%,77%,.55)}.dark\:text-amber-200\/60:is(.dark *){color:hsla(48,97%,77%,.6)}.dark\:text-amber-200\/65:is(.dark *){color:hsla(48,97%,77%,.65)}.dark\:text-amber-200\/70:is(.dark *){color:hsla(48,97%,77%,.7)}.dark\:text-amber-200\/75:is(.dark *){color:hsla(48,97%,77%,.75)}.dark\:text-amber-200\/80:is(.dark *){color:hsla(48,97%,77%,.8)}.dark\:text-amber-200\/85:is(.dark *){color:hsla(48,97%,77%,.85)}.dark\:text-amber-200\/90:is(.dark *){color:hsla(48,97%,77%,.9)}.dark\:text-amber-200\/95:is(.dark *){color:hsla(48,97%,77%,.95)}.dark\:text-amber-50:is(.dark *){--tw-text-opacity:1;color:rgb(255 251 235/var(--tw-text-opacity,1))}.dark\:text-amber-50\/0:is(.dark *){color:rgba(255,251,235,0)}.dark\:text-amber-50\/10:is(.dark *){color:rgba(255,251,235,.1)}.dark\:text-amber-50\/100:is(.dark *){color:#fffbeb}.dark\:text-amber-50\/15:is(.dark *){color:rgba(255,251,235,.15)}.dark\:text-amber-50\/20:is(.dark *){color:rgba(255,251,235,.2)}.dark\:text-amber-50\/25:is(.dark *){color:rgba(255,251,235,.25)}.dark\:text-amber-50\/30:is(.dark *){color:rgba(255,251,235,.3)}.dark\:text-amber-50\/35:is(.dark *){color:rgba(255,251,235,.35)}.dark\:text-amber-50\/40:is(.dark *){color:rgba(255,251,235,.4)}.dark\:text-amber-50\/45:is(.dark *){color:rgba(255,251,235,.45)}.dark\:text-amber-50\/5:is(.dark *){color:rgba(255,251,235,.05)}.dark\:text-amber-50\/50:is(.dark *){color:rgba(255,251,235,.5)}.dark\:text-amber-50\/55:is(.dark *){color:rgba(255,251,235,.55)}.dark\:text-amber-50\/60:is(.dark *){color:rgba(255,251,235,.6)}.dark\:text-amber-50\/65:is(.dark *){color:rgba(255,251,235,.65)}.dark\:text-amber-50\/70:is(.dark *){color:rgba(255,251,235,.7)}.dark\:text-amber-50\/75:is(.dark *){color:rgba(255,251,235,.75)}.dark\:text-amber-50\/80:is(.dark *){color:rgba(255,251,235,.8)}.dark\:text-amber-50\/85:is(.dark *){color:rgba(255,251,235,.85)}.dark\:text-amber-50\/90:is(.dark *){color:rgba(255,251,235,.9)}.dark\:text-amber-50\/95:is(.dark *){color:rgba(255,251,235,.95)}.dark\:text-amber-500:is(.dark *){--tw-text-opacity:1;color:rgb(245 158 11/var(--tw-text-opacity,1))}.dark\:text-amber-500\/0:is(.dark *){color:rgba(245,158,11,0)}.dark\:text-amber-500\/10:is(.dark *){color:rgba(245,158,11,.1)}.dark\:text-amber-500\/100:is(.dark *){color:#f59e0b}.dark\:text-amber-500\/15:is(.dark *){color:rgba(245,158,11,.15)}.dark\:text-amber-500\/20:is(.dark *){color:rgba(245,158,11,.2)}.dark\:text-amber-500\/25:is(.dark *){color:rgba(245,158,11,.25)}.dark\:text-amber-500\/30:is(.dark *){color:rgba(245,158,11,.3)}.dark\:text-amber-500\/35:is(.dark *){color:rgba(245,158,11,.35)}.dark\:text-amber-500\/40:is(.dark *){color:rgba(245,158,11,.4)}.dark\:text-amber-500\/45:is(.dark *){color:rgba(245,158,11,.45)}.dark\:text-amber-500\/5:is(.dark *){color:rgba(245,158,11,.05)}.dark\:text-amber-500\/50:is(.dark *){color:rgba(245,158,11,.5)}.dark\:text-amber-500\/55:is(.dark *){color:rgba(245,158,11,.55)}.dark\:text-amber-500\/60:is(.dark *){color:rgba(245,158,11,.6)}.dark\:text-amber-500\/65:is(.dark *){color:rgba(245,158,11,.65)}.dark\:text-amber-500\/70:is(.dark *){color:rgba(245,158,11,.7)}.dark\:text-amber-500\/75:is(.dark *){color:rgba(245,158,11,.75)}.dark\:text-amber-500\/80:is(.dark *){color:rgba(245,158,11,.8)}.dark\:text-amber-500\/85:is(.dark *){color:rgba(245,158,11,.85)}.dark\:text-amber-500\/90:is(.dark *){color:rgba(245,158,11,.9)}.dark\:text-amber-500\/95:is(.dark *){color:rgba(245,158,11,.95)}.dark\:text-amber-950:is(.dark *){--tw-text-opacity:1;color:rgb(69 26 3/var(--tw-text-opacity,1))}.dark\:text-amber-950\/0:is(.dark *){color:rgba(69,26,3,0)}.dark\:text-amber-950\/10:is(.dark *){color:rgba(69,26,3,.1)}.dark\:text-amber-950\/100:is(.dark *){color:#451a03}.dark\:text-amber-950\/15:is(.dark *){color:rgba(69,26,3,.15)}.dark\:text-amber-950\/20:is(.dark *){color:rgba(69,26,3,.2)}.dark\:text-amber-950\/25:is(.dark *){color:rgba(69,26,3,.25)}.dark\:text-amber-950\/30:is(.dark *){color:rgba(69,26,3,.3)}.dark\:text-amber-950\/35:is(.dark *){color:rgba(69,26,3,.35)}.dark\:text-amber-950\/40:is(.dark *){color:rgba(69,26,3,.4)}.dark\:text-amber-950\/45:is(.dark *){color:rgba(69,26,3,.45)}.dark\:text-amber-950\/5:is(.dark *){color:rgba(69,26,3,.05)}.dark\:text-amber-950\/50:is(.dark *){color:rgba(69,26,3,.5)}.dark\:text-amber-950\/55:is(.dark *){color:rgba(69,26,3,.55)}.dark\:text-amber-950\/60:is(.dark *){color:rgba(69,26,3,.6)}.dark\:text-amber-950\/65:is(.dark *){color:rgba(69,26,3,.65)}.dark\:text-amber-950\/70:is(.dark *){color:rgba(69,26,3,.7)}.dark\:text-amber-950\/75:is(.dark *){color:rgba(69,26,3,.75)}.dark\:text-amber-950\/80:is(.dark *){color:rgba(69,26,3,.8)}.dark\:text-amber-950\/85:is(.dark *){color:rgba(69,26,3,.85)}.dark\:text-amber-950\/90:is(.dark *){color:rgba(69,26,3,.9)}.dark\:text-amber-950\/95:is(.dark *){color:rgba(69,26,3,.95)}.dark\:text-blue-100:is(.dark *){--tw-text-opacity:1;color:rgb(219 234 254/var(--tw-text-opacity,1))}.dark\:text-blue-100\/0:is(.dark *){color:rgba(219,234,254,0)}.dark\:text-blue-100\/10:is(.dark *){color:rgba(219,234,254,.1)}.dark\:text-blue-100\/100:is(.dark *){color:#dbeafe}.dark\:text-blue-100\/15:is(.dark *){color:rgba(219,234,254,.15)}.dark\:text-blue-100\/20:is(.dark *){color:rgba(219,234,254,.2)}.dark\:text-blue-100\/25:is(.dark *){color:rgba(219,234,254,.25)}.dark\:text-blue-100\/30:is(.dark *){color:rgba(219,234,254,.3)}.dark\:text-blue-100\/35:is(.dark *){color:rgba(219,234,254,.35)}.dark\:text-blue-100\/40:is(.dark *){color:rgba(219,234,254,.4)}.dark\:text-blue-100\/45:is(.dark *){color:rgba(219,234,254,.45)}.dark\:text-blue-100\/5:is(.dark *){color:rgba(219,234,254,.05)}.dark\:text-blue-100\/50:is(.dark *){color:rgba(219,234,254,.5)}.dark\:text-blue-100\/55:is(.dark *){color:rgba(219,234,254,.55)}.dark\:text-blue-100\/60:is(.dark *){color:rgba(219,234,254,.6)}.dark\:text-blue-100\/65:is(.dark *){color:rgba(219,234,254,.65)}.dark\:text-blue-100\/70:is(.dark *){color:rgba(219,234,254,.7)}.dark\:text-blue-100\/75:is(.dark *){color:rgba(219,234,254,.75)}.dark\:text-blue-100\/80:is(.dark *){color:rgba(219,234,254,.8)}.dark\:text-blue-100\/85:is(.dark *){color:rgba(219,234,254,.85)}.dark\:text-blue-100\/90:is(.dark *){color:rgba(219,234,254,.9)}.dark\:text-blue-100\/95:is(.dark *){color:rgba(219,234,254,.95)}.dark\:text-blue-200:is(.dark *){--tw-text-opacity:1;color:rgb(191 219 254/var(--tw-text-opacity,1))}.dark\:text-blue-200\/0:is(.dark *){color:rgba(191,219,254,0)}.dark\:text-blue-200\/10:is(.dark *){color:rgba(191,219,254,.1)}.dark\:text-blue-200\/100:is(.dark *){color:#bfdbfe}.dark\:text-blue-200\/15:is(.dark *){color:rgba(191,219,254,.15)}.dark\:text-blue-200\/20:is(.dark *){color:rgba(191,219,254,.2)}.dark\:text-blue-200\/25:is(.dark *){color:rgba(191,219,254,.25)}.dark\:text-blue-200\/30:is(.dark *){color:rgba(191,219,254,.3)}.dark\:text-blue-200\/35:is(.dark *){color:rgba(191,219,254,.35)}.dark\:text-blue-200\/40:is(.dark *){color:rgba(191,219,254,.4)}.dark\:text-blue-200\/45:is(.dark *){color:rgba(191,219,254,.45)}.dark\:text-blue-200\/5:is(.dark *){color:rgba(191,219,254,.05)}.dark\:text-blue-200\/50:is(.dark *){color:rgba(191,219,254,.5)}.dark\:text-blue-200\/55:is(.dark *){color:rgba(191,219,254,.55)}.dark\:text-blue-200\/60:is(.dark *){color:rgba(191,219,254,.6)}.dark\:text-blue-200\/65:is(.dark *){color:rgba(191,219,254,.65)}.dark\:text-blue-200\/70:is(.dark *){color:rgba(191,219,254,.7)}.dark\:text-blue-200\/75:is(.dark *){color:rgba(191,219,254,.75)}.dark\:text-blue-200\/80:is(.dark *){color:rgba(191,219,254,.8)}.dark\:text-blue-200\/85:is(.dark *){color:rgba(191,219,254,.85)}.dark\:text-blue-200\/90:is(.dark *){color:rgba(191,219,254,.9)}.dark\:text-blue-200\/95:is(.dark *){color:rgba(191,219,254,.95)}.dark\:text-blue-300:is(.dark *){--tw-text-opacity:1;color:rgb(147 197 253/var(--tw-text-opacity,1))}.dark\:text-blue-50:is(.dark *){--tw-text-opacity:1;color:rgb(239 246 255/var(--tw-text-opacity,1))}.dark\:text-blue-50\/0:is(.dark *){color:rgba(239,246,255,0)}.dark\:text-blue-50\/10:is(.dark *){color:rgba(239,246,255,.1)}.dark\:text-blue-50\/100:is(.dark *){color:#eff6ff}.dark\:text-blue-50\/15:is(.dark *){color:rgba(239,246,255,.15)}.dark\:text-blue-50\/20:is(.dark *){color:rgba(239,246,255,.2)}.dark\:text-blue-50\/25:is(.dark *){color:rgba(239,246,255,.25)}.dark\:text-blue-50\/30:is(.dark *){color:rgba(239,246,255,.3)}.dark\:text-blue-50\/35:is(.dark *){color:rgba(239,246,255,.35)}.dark\:text-blue-50\/40:is(.dark *){color:rgba(239,246,255,.4)}.dark\:text-blue-50\/45:is(.dark *){color:rgba(239,246,255,.45)}.dark\:text-blue-50\/5:is(.dark *){color:rgba(239,246,255,.05)}.dark\:text-blue-50\/50:is(.dark *){color:rgba(239,246,255,.5)}.dark\:text-blue-50\/55:is(.dark *){color:rgba(239,246,255,.55)}.dark\:text-blue-50\/60:is(.dark *){color:rgba(239,246,255,.6)}.dark\:text-blue-50\/65:is(.dark *){color:rgba(239,246,255,.65)}.dark\:text-blue-50\/70:is(.dark *){color:rgba(239,246,255,.7)}.dark\:text-blue-50\/75:is(.dark *){color:rgba(239,246,255,.75)}.dark\:text-blue-50\/80:is(.dark *){color:rgba(239,246,255,.8)}.dark\:text-blue-50\/85:is(.dark *){color:rgba(239,246,255,.85)}.dark\:text-blue-50\/90:is(.dark *){color:rgba(239,246,255,.9)}.dark\:text-blue-50\/95:is(.dark *){color:rgba(239,246,255,.95)}.dark\:text-blue-500:is(.dark *){--tw-text-opacity:1;color:rgb(59 130 246/var(--tw-text-opacity,1))}.dark\:text-blue-500\/0:is(.dark *){color:rgba(59,130,246,0)}.dark\:text-blue-500\/10:is(.dark *){color:rgba(59,130,246,.1)}.dark\:text-blue-500\/100:is(.dark *){color:#3b82f6}.dark\:text-blue-500\/15:is(.dark *){color:rgba(59,130,246,.15)}.dark\:text-blue-500\/20:is(.dark *){color:rgba(59,130,246,.2)}.dark\:text-blue-500\/25:is(.dark *){color:rgba(59,130,246,.25)}.dark\:text-blue-500\/30:is(.dark *){color:rgba(59,130,246,.3)}.dark\:text-blue-500\/35:is(.dark *){color:rgba(59,130,246,.35)}.dark\:text-blue-500\/40:is(.dark *){color:rgba(59,130,246,.4)}.dark\:text-blue-500\/45:is(.dark *){color:rgba(59,130,246,.45)}.dark\:text-blue-500\/5:is(.dark *){color:rgba(59,130,246,.05)}.dark\:text-blue-500\/50:is(.dark *){color:rgba(59,130,246,.5)}.dark\:text-blue-500\/55:is(.dark *){color:rgba(59,130,246,.55)}.dark\:text-blue-500\/60:is(.dark *){color:rgba(59,130,246,.6)}.dark\:text-blue-500\/65:is(.dark *){color:rgba(59,130,246,.65)}.dark\:text-blue-500\/70:is(.dark *){color:rgba(59,130,246,.7)}.dark\:text-blue-500\/75:is(.dark *){color:rgba(59,130,246,.75)}.dark\:text-blue-500\/80:is(.dark *){color:rgba(59,130,246,.8)}.dark\:text-blue-500\/85:is(.dark *){color:rgba(59,130,246,.85)}.dark\:text-blue-500\/90:is(.dark *){color:rgba(59,130,246,.9)}.dark\:text-blue-500\/95:is(.dark *){color:rgba(59,130,246,.95)}.dark\:text-blue-950:is(.dark *){--tw-text-opacity:1;color:rgb(23 37 84/var(--tw-text-opacity,1))}.dark\:text-blue-950\/0:is(.dark *){color:rgba(23,37,84,0)}.dark\:text-blue-950\/10:is(.dark *){color:rgba(23,37,84,.1)}.dark\:text-blue-950\/100:is(.dark *){color:#172554}.dark\:text-blue-950\/15:is(.dark *){color:rgba(23,37,84,.15)}.dark\:text-blue-950\/20:is(.dark *){color:rgba(23,37,84,.2)}.dark\:text-blue-950\/25:is(.dark *){color:rgba(23,37,84,.25)}.dark\:text-blue-950\/30:is(.dark *){color:rgba(23,37,84,.3)}.dark\:text-blue-950\/35:is(.dark *){color:rgba(23,37,84,.35)}.dark\:text-blue-950\/40:is(.dark *){color:rgba(23,37,84,.4)}.dark\:text-blue-950\/45:is(.dark *){color:rgba(23,37,84,.45)}.dark\:text-blue-950\/5:is(.dark *){color:rgba(23,37,84,.05)}.dark\:text-blue-950\/50:is(.dark *){color:rgba(23,37,84,.5)}.dark\:text-blue-950\/55:is(.dark *){color:rgba(23,37,84,.55)}.dark\:text-blue-950\/60:is(.dark *){color:rgba(23,37,84,.6)}.dark\:text-blue-950\/65:is(.dark *){color:rgba(23,37,84,.65)}.dark\:text-blue-950\/70:is(.dark *){color:rgba(23,37,84,.7)}.dark\:text-blue-950\/75:is(.dark *){color:rgba(23,37,84,.75)}.dark\:text-blue-950\/80:is(.dark *){color:rgba(23,37,84,.8)}.dark\:text-blue-950\/85:is(.dark *){color:rgba(23,37,84,.85)}.dark\:text-blue-950\/90:is(.dark *){color:rgba(23,37,84,.9)}.dark\:text-blue-950\/95:is(.dark *){color:rgba(23,37,84,.95)}.dark\:text-cyan-100:is(.dark *){--tw-text-opacity:1;color:rgb(207 250 254/var(--tw-text-opacity,1))}.dark\:text-cyan-100\/0:is(.dark *){color:rgba(207,250,254,0)}.dark\:text-cyan-100\/10:is(.dark *){color:rgba(207,250,254,.1)}.dark\:text-cyan-100\/100:is(.dark *){color:#cffafe}.dark\:text-cyan-100\/15:is(.dark *){color:rgba(207,250,254,.15)}.dark\:text-cyan-100\/20:is(.dark *){color:rgba(207,250,254,.2)}.dark\:text-cyan-100\/25:is(.dark *){color:rgba(207,250,254,.25)}.dark\:text-cyan-100\/30:is(.dark *){color:rgba(207,250,254,.3)}.dark\:text-cyan-100\/35:is(.dark *){color:rgba(207,250,254,.35)}.dark\:text-cyan-100\/40:is(.dark *){color:rgba(207,250,254,.4)}.dark\:text-cyan-100\/45:is(.dark *){color:rgba(207,250,254,.45)}.dark\:text-cyan-100\/5:is(.dark *){color:rgba(207,250,254,.05)}.dark\:text-cyan-100\/50:is(.dark *){color:rgba(207,250,254,.5)}.dark\:text-cyan-100\/55:is(.dark *){color:rgba(207,250,254,.55)}.dark\:text-cyan-100\/60:is(.dark *){color:rgba(207,250,254,.6)}.dark\:text-cyan-100\/65:is(.dark *){color:rgba(207,250,254,.65)}.dark\:text-cyan-100\/70:is(.dark *){color:rgba(207,250,254,.7)}.dark\:text-cyan-100\/75:is(.dark *){color:rgba(207,250,254,.75)}.dark\:text-cyan-100\/80:is(.dark *){color:rgba(207,250,254,.8)}.dark\:text-cyan-100\/85:is(.dark *){color:rgba(207,250,254,.85)}.dark\:text-cyan-100\/90:is(.dark *){color:rgba(207,250,254,.9)}.dark\:text-cyan-100\/95:is(.dark *){color:rgba(207,250,254,.95)}.dark\:text-cyan-200:is(.dark *){--tw-text-opacity:1;color:rgb(165 243 252/var(--tw-text-opacity,1))}.dark\:text-cyan-200\/0:is(.dark *){color:rgba(165,243,252,0)}.dark\:text-cyan-200\/10:is(.dark *){color:rgba(165,243,252,.1)}.dark\:text-cyan-200\/100:is(.dark *){color:#a5f3fc}.dark\:text-cyan-200\/15:is(.dark *){color:rgba(165,243,252,.15)}.dark\:text-cyan-200\/20:is(.dark *){color:rgba(165,243,252,.2)}.dark\:text-cyan-200\/25:is(.dark *){color:rgba(165,243,252,.25)}.dark\:text-cyan-200\/30:is(.dark *){color:rgba(165,243,252,.3)}.dark\:text-cyan-200\/35:is(.dark *){color:rgba(165,243,252,.35)}.dark\:text-cyan-200\/40:is(.dark *){color:rgba(165,243,252,.4)}.dark\:text-cyan-200\/45:is(.dark *){color:rgba(165,243,252,.45)}.dark\:text-cyan-200\/5:is(.dark *){color:rgba(165,243,252,.05)}.dark\:text-cyan-200\/50:is(.dark *){color:rgba(165,243,252,.5)}.dark\:text-cyan-200\/55:is(.dark *){color:rgba(165,243,252,.55)}.dark\:text-cyan-200\/60:is(.dark *){color:rgba(165,243,252,.6)}.dark\:text-cyan-200\/65:is(.dark *){color:rgba(165,243,252,.65)}.dark\:text-cyan-200\/70:is(.dark *){color:rgba(165,243,252,.7)}.dark\:text-cyan-200\/75:is(.dark *){color:rgba(165,243,252,.75)}.dark\:text-cyan-200\/80:is(.dark *){color:rgba(165,243,252,.8)}.dark\:text-cyan-200\/85:is(.dark *){color:rgba(165,243,252,.85)}.dark\:text-cyan-200\/90:is(.dark *){color:rgba(165,243,252,.9)}.dark\:text-cyan-200\/95:is(.dark *){color:rgba(165,243,252,.95)}.dark\:text-cyan-50:is(.dark *){--tw-text-opacity:1;color:rgb(236 254 255/var(--tw-text-opacity,1))}.dark\:text-cyan-50\/0:is(.dark *){color:rgba(236,254,255,0)}.dark\:text-cyan-50\/10:is(.dark *){color:rgba(236,254,255,.1)}.dark\:text-cyan-50\/100:is(.dark *){color:#ecfeff}.dark\:text-cyan-50\/15:is(.dark *){color:rgba(236,254,255,.15)}.dark\:text-cyan-50\/20:is(.dark *){color:rgba(236,254,255,.2)}.dark\:text-cyan-50\/25:is(.dark *){color:rgba(236,254,255,.25)}.dark\:text-cyan-50\/30:is(.dark *){color:rgba(236,254,255,.3)}.dark\:text-cyan-50\/35:is(.dark *){color:rgba(236,254,255,.35)}.dark\:text-cyan-50\/40:is(.dark *){color:rgba(236,254,255,.4)}.dark\:text-cyan-50\/45:is(.dark *){color:rgba(236,254,255,.45)}.dark\:text-cyan-50\/5:is(.dark *){color:rgba(236,254,255,.05)}.dark\:text-cyan-50\/50:is(.dark *){color:rgba(236,254,255,.5)}.dark\:text-cyan-50\/55:is(.dark *){color:rgba(236,254,255,.55)}.dark\:text-cyan-50\/60:is(.dark *){color:rgba(236,254,255,.6)}.dark\:text-cyan-50\/65:is(.dark *){color:rgba(236,254,255,.65)}.dark\:text-cyan-50\/70:is(.dark *){color:rgba(236,254,255,.7)}.dark\:text-cyan-50\/75:is(.dark *){color:rgba(236,254,255,.75)}.dark\:text-cyan-50\/80:is(.dark *){color:rgba(236,254,255,.8)}.dark\:text-cyan-50\/85:is(.dark *){color:rgba(236,254,255,.85)}.dark\:text-cyan-50\/90:is(.dark *){color:rgba(236,254,255,.9)}.dark\:text-cyan-50\/95:is(.dark *){color:rgba(236,254,255,.95)}.dark\:text-cyan-500:is(.dark *){--tw-text-opacity:1;color:rgb(6 182 212/var(--tw-text-opacity,1))}.dark\:text-cyan-500\/0:is(.dark *){color:rgba(6,182,212,0)}.dark\:text-cyan-500\/10:is(.dark *){color:rgba(6,182,212,.1)}.dark\:text-cyan-500\/100:is(.dark *){color:#06b6d4}.dark\:text-cyan-500\/15:is(.dark *){color:rgba(6,182,212,.15)}.dark\:text-cyan-500\/20:is(.dark *){color:rgba(6,182,212,.2)}.dark\:text-cyan-500\/25:is(.dark *){color:rgba(6,182,212,.25)}.dark\:text-cyan-500\/30:is(.dark *){color:rgba(6,182,212,.3)}.dark\:text-cyan-500\/35:is(.dark *){color:rgba(6,182,212,.35)}.dark\:text-cyan-500\/40:is(.dark *){color:rgba(6,182,212,.4)}.dark\:text-cyan-500\/45:is(.dark *){color:rgba(6,182,212,.45)}.dark\:text-cyan-500\/5:is(.dark *){color:rgba(6,182,212,.05)}.dark\:text-cyan-500\/50:is(.dark *){color:rgba(6,182,212,.5)}.dark\:text-cyan-500\/55:is(.dark *){color:rgba(6,182,212,.55)}.dark\:text-cyan-500\/60:is(.dark *){color:rgba(6,182,212,.6)}.dark\:text-cyan-500\/65:is(.dark *){color:rgba(6,182,212,.65)}.dark\:text-cyan-500\/70:is(.dark *){color:rgba(6,182,212,.7)}.dark\:text-cyan-500\/75:is(.dark *){color:rgba(6,182,212,.75)}.dark\:text-cyan-500\/80:is(.dark *){color:rgba(6,182,212,.8)}.dark\:text-cyan-500\/85:is(.dark *){color:rgba(6,182,212,.85)}.dark\:text-cyan-500\/90:is(.dark *){color:rgba(6,182,212,.9)}.dark\:text-cyan-500\/95:is(.dark *){color:rgba(6,182,212,.95)}.dark\:text-cyan-950:is(.dark *){--tw-text-opacity:1;color:rgb(8 51 68/var(--tw-text-opacity,1))}.dark\:text-cyan-950\/0:is(.dark *){color:rgba(8,51,68,0)}.dark\:text-cyan-950\/10:is(.dark *){color:rgba(8,51,68,.1)}.dark\:text-cyan-950\/100:is(.dark *){color:#083344}.dark\:text-cyan-950\/15:is(.dark *){color:rgba(8,51,68,.15)}.dark\:text-cyan-950\/20:is(.dark *){color:rgba(8,51,68,.2)}.dark\:text-cyan-950\/25:is(.dark *){color:rgba(8,51,68,.25)}.dark\:text-cyan-950\/30:is(.dark *){color:rgba(8,51,68,.3)}.dark\:text-cyan-950\/35:is(.dark *){color:rgba(8,51,68,.35)}.dark\:text-cyan-950\/40:is(.dark *){color:rgba(8,51,68,.4)}.dark\:text-cyan-950\/45:is(.dark *){color:rgba(8,51,68,.45)}.dark\:text-cyan-950\/5:is(.dark *){color:rgba(8,51,68,.05)}.dark\:text-cyan-950\/50:is(.dark *){color:rgba(8,51,68,.5)}.dark\:text-cyan-950\/55:is(.dark *){color:rgba(8,51,68,.55)}.dark\:text-cyan-950\/60:is(.dark *){color:rgba(8,51,68,.6)}.dark\:text-cyan-950\/65:is(.dark *){color:rgba(8,51,68,.65)}.dark\:text-cyan-950\/70:is(.dark *){color:rgba(8,51,68,.7)}.dark\:text-cyan-950\/75:is(.dark *){color:rgba(8,51,68,.75)}.dark\:text-cyan-950\/80:is(.dark *){color:rgba(8,51,68,.8)}.dark\:text-cyan-950\/85:is(.dark *){color:rgba(8,51,68,.85)}.dark\:text-cyan-950\/90:is(.dark *){color:rgba(8,51,68,.9)}.dark\:text-cyan-950\/95:is(.dark *){color:rgba(8,51,68,.95)}.dark\:text-emerald-100:is(.dark *){--tw-text-opacity:1;color:rgb(209 250 229/var(--tw-text-opacity,1))}.dark\:text-emerald-100\/0:is(.dark *){color:rgba(209,250,229,0)}.dark\:text-emerald-100\/10:is(.dark *){color:rgba(209,250,229,.1)}.dark\:text-emerald-100\/100:is(.dark *){color:#d1fae5}.dark\:text-emerald-100\/15:is(.dark *){color:rgba(209,250,229,.15)}.dark\:text-emerald-100\/20:is(.dark *){color:rgba(209,250,229,.2)}.dark\:text-emerald-100\/25:is(.dark *){color:rgba(209,250,229,.25)}.dark\:text-emerald-100\/30:is(.dark *){color:rgba(209,250,229,.3)}.dark\:text-emerald-100\/35:is(.dark *){color:rgba(209,250,229,.35)}.dark\:text-emerald-100\/40:is(.dark *){color:rgba(209,250,229,.4)}.dark\:text-emerald-100\/45:is(.dark *){color:rgba(209,250,229,.45)}.dark\:text-emerald-100\/5:is(.dark *){color:rgba(209,250,229,.05)}.dark\:text-emerald-100\/50:is(.dark *){color:rgba(209,250,229,.5)}.dark\:text-emerald-100\/55:is(.dark *){color:rgba(209,250,229,.55)}.dark\:text-emerald-100\/60:is(.dark *){color:rgba(209,250,229,.6)}.dark\:text-emerald-100\/65:is(.dark *){color:rgba(209,250,229,.65)}.dark\:text-emerald-100\/70:is(.dark *){color:rgba(209,250,229,.7)}.dark\:text-emerald-100\/75:is(.dark *){color:rgba(209,250,229,.75)}.dark\:text-emerald-100\/80:is(.dark *){color:rgba(209,250,229,.8)}.dark\:text-emerald-100\/85:is(.dark *){color:rgba(209,250,229,.85)}.dark\:text-emerald-100\/90:is(.dark *){color:rgba(209,250,229,.9)}.dark\:text-emerald-100\/95:is(.dark *){color:rgba(209,250,229,.95)}.dark\:text-emerald-200:is(.dark *){--tw-text-opacity:1;color:rgb(167 243 208/var(--tw-text-opacity,1))}.dark\:text-emerald-200\/0:is(.dark *){color:rgba(167,243,208,0)}.dark\:text-emerald-200\/10:is(.dark *){color:rgba(167,243,208,.1)}.dark\:text-emerald-200\/100:is(.dark *){color:#a7f3d0}.dark\:text-emerald-200\/15:is(.dark *){color:rgba(167,243,208,.15)}.dark\:text-emerald-200\/20:is(.dark *){color:rgba(167,243,208,.2)}.dark\:text-emerald-200\/25:is(.dark *){color:rgba(167,243,208,.25)}.dark\:text-emerald-200\/30:is(.dark *){color:rgba(167,243,208,.3)}.dark\:text-emerald-200\/35:is(.dark *){color:rgba(167,243,208,.35)}.dark\:text-emerald-200\/40:is(.dark *){color:rgba(167,243,208,.4)}.dark\:text-emerald-200\/45:is(.dark *){color:rgba(167,243,208,.45)}.dark\:text-emerald-200\/5:is(.dark *){color:rgba(167,243,208,.05)}.dark\:text-emerald-200\/50:is(.dark *){color:rgba(167,243,208,.5)}.dark\:text-emerald-200\/55:is(.dark *){color:rgba(167,243,208,.55)}.dark\:text-emerald-200\/60:is(.dark *){color:rgba(167,243,208,.6)}.dark\:text-emerald-200\/65:is(.dark *){color:rgba(167,243,208,.65)}.dark\:text-emerald-200\/70:is(.dark *){color:rgba(167,243,208,.7)}.dark\:text-emerald-200\/75:is(.dark *){color:rgba(167,243,208,.75)}.dark\:text-emerald-200\/80:is(.dark *){color:rgba(167,243,208,.8)}.dark\:text-emerald-200\/85:is(.dark *){color:rgba(167,243,208,.85)}.dark\:text-emerald-200\/90:is(.dark *){color:rgba(167,243,208,.9)}.dark\:text-emerald-200\/95:is(.dark *){color:rgba(167,243,208,.95)}.dark\:text-emerald-50:is(.dark *){--tw-text-opacity:1;color:rgb(236 253 245/var(--tw-text-opacity,1))}.dark\:text-emerald-50\/0:is(.dark *){color:rgba(236,253,245,0)}.dark\:text-emerald-50\/10:is(.dark *){color:rgba(236,253,245,.1)}.dark\:text-emerald-50\/100:is(.dark *){color:#ecfdf5}.dark\:text-emerald-50\/15:is(.dark *){color:rgba(236,253,245,.15)}.dark\:text-emerald-50\/20:is(.dark *){color:rgba(236,253,245,.2)}.dark\:text-emerald-50\/25:is(.dark *){color:rgba(236,253,245,.25)}.dark\:text-emerald-50\/30:is(.dark *){color:rgba(236,253,245,.3)}.dark\:text-emerald-50\/35:is(.dark *){color:rgba(236,253,245,.35)}.dark\:text-emerald-50\/40:is(.dark *){color:rgba(236,253,245,.4)}.dark\:text-emerald-50\/45:is(.dark *){color:rgba(236,253,245,.45)}.dark\:text-emerald-50\/5:is(.dark *){color:rgba(236,253,245,.05)}.dark\:text-emerald-50\/50:is(.dark *){color:rgba(236,253,245,.5)}.dark\:text-emerald-50\/55:is(.dark *){color:rgba(236,253,245,.55)}.dark\:text-emerald-50\/60:is(.dark *){color:rgba(236,253,245,.6)}.dark\:text-emerald-50\/65:is(.dark *){color:rgba(236,253,245,.65)}.dark\:text-emerald-50\/70:is(.dark *){color:rgba(236,253,245,.7)}.dark\:text-emerald-50\/75:is(.dark *){color:rgba(236,253,245,.75)}.dark\:text-emerald-50\/80:is(.dark *){color:rgba(236,253,245,.8)}.dark\:text-emerald-50\/85:is(.dark *){color:rgba(236,253,245,.85)}.dark\:text-emerald-50\/90:is(.dark *){color:rgba(236,253,245,.9)}.dark\:text-emerald-50\/95:is(.dark *){color:rgba(236,253,245,.95)}.dark\:text-emerald-500:is(.dark *){--tw-text-opacity:1;color:rgb(16 185 129/var(--tw-text-opacity,1))}.dark\:text-emerald-500\/0:is(.dark *){color:rgba(16,185,129,0)}.dark\:text-emerald-500\/10:is(.dark *){color:rgba(16,185,129,.1)}.dark\:text-emerald-500\/100:is(.dark *){color:#10b981}.dark\:text-emerald-500\/15:is(.dark *){color:rgba(16,185,129,.15)}.dark\:text-emerald-500\/20:is(.dark *){color:rgba(16,185,129,.2)}.dark\:text-emerald-500\/25:is(.dark *){color:rgba(16,185,129,.25)}.dark\:text-emerald-500\/30:is(.dark *){color:rgba(16,185,129,.3)}.dark\:text-emerald-500\/35:is(.dark *){color:rgba(16,185,129,.35)}.dark\:text-emerald-500\/40:is(.dark *){color:rgba(16,185,129,.4)}.dark\:text-emerald-500\/45:is(.dark *){color:rgba(16,185,129,.45)}.dark\:text-emerald-500\/5:is(.dark *){color:rgba(16,185,129,.05)}.dark\:text-emerald-500\/50:is(.dark *){color:rgba(16,185,129,.5)}.dark\:text-emerald-500\/55:is(.dark *){color:rgba(16,185,129,.55)}.dark\:text-emerald-500\/60:is(.dark *){color:rgba(16,185,129,.6)}.dark\:text-emerald-500\/65:is(.dark *){color:rgba(16,185,129,.65)}.dark\:text-emerald-500\/70:is(.dark *){color:rgba(16,185,129,.7)}.dark\:text-emerald-500\/75:is(.dark *){color:rgba(16,185,129,.75)}.dark\:text-emerald-500\/80:is(.dark *){color:rgba(16,185,129,.8)}.dark\:text-emerald-500\/85:is(.dark *){color:rgba(16,185,129,.85)}.dark\:text-emerald-500\/90:is(.dark *){color:rgba(16,185,129,.9)}.dark\:text-emerald-500\/95:is(.dark *){color:rgba(16,185,129,.95)}.dark\:text-emerald-950:is(.dark *){--tw-text-opacity:1;color:rgb(2 44 34/var(--tw-text-opacity,1))}.dark\:text-emerald-950\/0:is(.dark *){color:rgba(2,44,34,0)}.dark\:text-emerald-950\/10:is(.dark *){color:rgba(2,44,34,.1)}.dark\:text-emerald-950\/100:is(.dark *){color:#022c22}.dark\:text-emerald-950\/15:is(.dark *){color:rgba(2,44,34,.15)}.dark\:text-emerald-950\/20:is(.dark *){color:rgba(2,44,34,.2)}.dark\:text-emerald-950\/25:is(.dark *){color:rgba(2,44,34,.25)}.dark\:text-emerald-950\/30:is(.dark *){color:rgba(2,44,34,.3)}.dark\:text-emerald-950\/35:is(.dark *){color:rgba(2,44,34,.35)}.dark\:text-emerald-950\/40:is(.dark *){color:rgba(2,44,34,.4)}.dark\:text-emerald-950\/45:is(.dark *){color:rgba(2,44,34,.45)}.dark\:text-emerald-950\/5:is(.dark *){color:rgba(2,44,34,.05)}.dark\:text-emerald-950\/50:is(.dark *){color:rgba(2,44,34,.5)}.dark\:text-emerald-950\/55:is(.dark *){color:rgba(2,44,34,.55)}.dark\:text-emerald-950\/60:is(.dark *){color:rgba(2,44,34,.6)}.dark\:text-emerald-950\/65:is(.dark *){color:rgba(2,44,34,.65)}.dark\:text-emerald-950\/70:is(.dark *){color:rgba(2,44,34,.7)}.dark\:text-emerald-950\/75:is(.dark *){color:rgba(2,44,34,.75)}.dark\:text-emerald-950\/80:is(.dark *){color:rgba(2,44,34,.8)}.dark\:text-emerald-950\/85:is(.dark *){color:rgba(2,44,34,.85)}.dark\:text-emerald-950\/90:is(.dark *){color:rgba(2,44,34,.9)}.dark\:text-emerald-950\/95:is(.dark *){color:rgba(2,44,34,.95)}.dark\:text-fuchsia-100:is(.dark *){--tw-text-opacity:1;color:rgb(250 232 255/var(--tw-text-opacity,1))}.dark\:text-fuchsia-100\/0:is(.dark *){color:rgba(250,232,255,0)}.dark\:text-fuchsia-100\/10:is(.dark *){color:rgba(250,232,255,.1)}.dark\:text-fuchsia-100\/100:is(.dark *){color:#fae8ff}.dark\:text-fuchsia-100\/15:is(.dark *){color:rgba(250,232,255,.15)}.dark\:text-fuchsia-100\/20:is(.dark *){color:rgba(250,232,255,.2)}.dark\:text-fuchsia-100\/25:is(.dark *){color:rgba(250,232,255,.25)}.dark\:text-fuchsia-100\/30:is(.dark *){color:rgba(250,232,255,.3)}.dark\:text-fuchsia-100\/35:is(.dark *){color:rgba(250,232,255,.35)}.dark\:text-fuchsia-100\/40:is(.dark *){color:rgba(250,232,255,.4)}.dark\:text-fuchsia-100\/45:is(.dark *){color:rgba(250,232,255,.45)}.dark\:text-fuchsia-100\/5:is(.dark *){color:rgba(250,232,255,.05)}.dark\:text-fuchsia-100\/50:is(.dark *){color:rgba(250,232,255,.5)}.dark\:text-fuchsia-100\/55:is(.dark *){color:rgba(250,232,255,.55)}.dark\:text-fuchsia-100\/60:is(.dark *){color:rgba(250,232,255,.6)}.dark\:text-fuchsia-100\/65:is(.dark *){color:rgba(250,232,255,.65)}.dark\:text-fuchsia-100\/70:is(.dark *){color:rgba(250,232,255,.7)}.dark\:text-fuchsia-100\/75:is(.dark *){color:rgba(250,232,255,.75)}.dark\:text-fuchsia-100\/80:is(.dark *){color:rgba(250,232,255,.8)}.dark\:text-fuchsia-100\/85:is(.dark *){color:rgba(250,232,255,.85)}.dark\:text-fuchsia-100\/90:is(.dark *){color:rgba(250,232,255,.9)}.dark\:text-fuchsia-100\/95:is(.dark *){color:rgba(250,232,255,.95)}.dark\:text-fuchsia-200:is(.dark *){--tw-text-opacity:1;color:rgb(245 208 254/var(--tw-text-opacity,1))}.dark\:text-fuchsia-200\/0:is(.dark *){color:rgba(245,208,254,0)}.dark\:text-fuchsia-200\/10:is(.dark *){color:rgba(245,208,254,.1)}.dark\:text-fuchsia-200\/100:is(.dark *){color:#f5d0fe}.dark\:text-fuchsia-200\/15:is(.dark *){color:rgba(245,208,254,.15)}.dark\:text-fuchsia-200\/20:is(.dark *){color:rgba(245,208,254,.2)}.dark\:text-fuchsia-200\/25:is(.dark *){color:rgba(245,208,254,.25)}.dark\:text-fuchsia-200\/30:is(.dark *){color:rgba(245,208,254,.3)}.dark\:text-fuchsia-200\/35:is(.dark *){color:rgba(245,208,254,.35)}.dark\:text-fuchsia-200\/40:is(.dark *){color:rgba(245,208,254,.4)}.dark\:text-fuchsia-200\/45:is(.dark *){color:rgba(245,208,254,.45)}.dark\:text-fuchsia-200\/5:is(.dark *){color:rgba(245,208,254,.05)}.dark\:text-fuchsia-200\/50:is(.dark *){color:rgba(245,208,254,.5)}.dark\:text-fuchsia-200\/55:is(.dark *){color:rgba(245,208,254,.55)}.dark\:text-fuchsia-200\/60:is(.dark *){color:rgba(245,208,254,.6)}.dark\:text-fuchsia-200\/65:is(.dark *){color:rgba(245,208,254,.65)}.dark\:text-fuchsia-200\/70:is(.dark *){color:rgba(245,208,254,.7)}.dark\:text-fuchsia-200\/75:is(.dark *){color:rgba(245,208,254,.75)}.dark\:text-fuchsia-200\/80:is(.dark *){color:rgba(245,208,254,.8)}.dark\:text-fuchsia-200\/85:is(.dark *){color:rgba(245,208,254,.85)}.dark\:text-fuchsia-200\/90:is(.dark *){color:rgba(245,208,254,.9)}.dark\:text-fuchsia-200\/95:is(.dark *){color:rgba(245,208,254,.95)}.dark\:text-fuchsia-50:is(.dark *){--tw-text-opacity:1;color:rgb(253 244 255/var(--tw-text-opacity,1))}.dark\:text-fuchsia-50\/0:is(.dark *){color:rgba(253,244,255,0)}.dark\:text-fuchsia-50\/10:is(.dark *){color:rgba(253,244,255,.1)}.dark\:text-fuchsia-50\/100:is(.dark *){color:#fdf4ff}.dark\:text-fuchsia-50\/15:is(.dark *){color:rgba(253,244,255,.15)}.dark\:text-fuchsia-50\/20:is(.dark *){color:rgba(253,244,255,.2)}.dark\:text-fuchsia-50\/25:is(.dark *){color:rgba(253,244,255,.25)}.dark\:text-fuchsia-50\/30:is(.dark *){color:rgba(253,244,255,.3)}.dark\:text-fuchsia-50\/35:is(.dark *){color:rgba(253,244,255,.35)}.dark\:text-fuchsia-50\/40:is(.dark *){color:rgba(253,244,255,.4)}.dark\:text-fuchsia-50\/45:is(.dark *){color:rgba(253,244,255,.45)}.dark\:text-fuchsia-50\/5:is(.dark *){color:rgba(253,244,255,.05)}.dark\:text-fuchsia-50\/50:is(.dark *){color:rgba(253,244,255,.5)}.dark\:text-fuchsia-50\/55:is(.dark *){color:rgba(253,244,255,.55)}.dark\:text-fuchsia-50\/60:is(.dark *){color:rgba(253,244,255,.6)}.dark\:text-fuchsia-50\/65:is(.dark *){color:rgba(253,244,255,.65)}.dark\:text-fuchsia-50\/70:is(.dark *){color:rgba(253,244,255,.7)}.dark\:text-fuchsia-50\/75:is(.dark *){color:rgba(253,244,255,.75)}.dark\:text-fuchsia-50\/80:is(.dark *){color:rgba(253,244,255,.8)}.dark\:text-fuchsia-50\/85:is(.dark *){color:rgba(253,244,255,.85)}.dark\:text-fuchsia-50\/90:is(.dark *){color:rgba(253,244,255,.9)}.dark\:text-fuchsia-50\/95:is(.dark *){color:rgba(253,244,255,.95)}.dark\:text-fuchsia-500:is(.dark *){--tw-text-opacity:1;color:rgb(217 70 239/var(--tw-text-opacity,1))}.dark\:text-fuchsia-500\/0:is(.dark *){color:rgba(217,70,239,0)}.dark\:text-fuchsia-500\/10:is(.dark *){color:rgba(217,70,239,.1)}.dark\:text-fuchsia-500\/100:is(.dark *){color:#d946ef}.dark\:text-fuchsia-500\/15:is(.dark *){color:rgba(217,70,239,.15)}.dark\:text-fuchsia-500\/20:is(.dark *){color:rgba(217,70,239,.2)}.dark\:text-fuchsia-500\/25:is(.dark *){color:rgba(217,70,239,.25)}.dark\:text-fuchsia-500\/30:is(.dark *){color:rgba(217,70,239,.3)}.dark\:text-fuchsia-500\/35:is(.dark *){color:rgba(217,70,239,.35)}.dark\:text-fuchsia-500\/40:is(.dark *){color:rgba(217,70,239,.4)}.dark\:text-fuchsia-500\/45:is(.dark *){color:rgba(217,70,239,.45)}.dark\:text-fuchsia-500\/5:is(.dark *){color:rgba(217,70,239,.05)}.dark\:text-fuchsia-500\/50:is(.dark *){color:rgba(217,70,239,.5)}.dark\:text-fuchsia-500\/55:is(.dark *){color:rgba(217,70,239,.55)}.dark\:text-fuchsia-500\/60:is(.dark *){color:rgba(217,70,239,.6)}.dark\:text-fuchsia-500\/65:is(.dark *){color:rgba(217,70,239,.65)}.dark\:text-fuchsia-500\/70:is(.dark *){color:rgba(217,70,239,.7)}.dark\:text-fuchsia-500\/75:is(.dark *){color:rgba(217,70,239,.75)}.dark\:text-fuchsia-500\/80:is(.dark *){color:rgba(217,70,239,.8)}.dark\:text-fuchsia-500\/85:is(.dark *){color:rgba(217,70,239,.85)}.dark\:text-fuchsia-500\/90:is(.dark *){color:rgba(217,70,239,.9)}.dark\:text-fuchsia-500\/95:is(.dark *){color:rgba(217,70,239,.95)}.dark\:text-fuchsia-950:is(.dark *){--tw-text-opacity:1;color:rgb(74 4 78/var(--tw-text-opacity,1))}.dark\:text-fuchsia-950\/0:is(.dark *){color:rgba(74,4,78,0)}.dark\:text-fuchsia-950\/10:is(.dark *){color:rgba(74,4,78,.1)}.dark\:text-fuchsia-950\/100:is(.dark *){color:#4a044e}.dark\:text-fuchsia-950\/15:is(.dark *){color:rgba(74,4,78,.15)}.dark\:text-fuchsia-950\/20:is(.dark *){color:rgba(74,4,78,.2)}.dark\:text-fuchsia-950\/25:is(.dark *){color:rgba(74,4,78,.25)}.dark\:text-fuchsia-950\/30:is(.dark *){color:rgba(74,4,78,.3)}.dark\:text-fuchsia-950\/35:is(.dark *){color:rgba(74,4,78,.35)}.dark\:text-fuchsia-950\/40:is(.dark *){color:rgba(74,4,78,.4)}.dark\:text-fuchsia-950\/45:is(.dark *){color:rgba(74,4,78,.45)}.dark\:text-fuchsia-950\/5:is(.dark *){color:rgba(74,4,78,.05)}.dark\:text-fuchsia-950\/50:is(.dark *){color:rgba(74,4,78,.5)}.dark\:text-fuchsia-950\/55:is(.dark *){color:rgba(74,4,78,.55)}.dark\:text-fuchsia-950\/60:is(.dark *){color:rgba(74,4,78,.6)}.dark\:text-fuchsia-950\/65:is(.dark *){color:rgba(74,4,78,.65)}.dark\:text-fuchsia-950\/70:is(.dark *){color:rgba(74,4,78,.7)}.dark\:text-fuchsia-950\/75:is(.dark *){color:rgba(74,4,78,.75)}.dark\:text-fuchsia-950\/80:is(.dark *){color:rgba(74,4,78,.8)}.dark\:text-fuchsia-950\/85:is(.dark *){color:rgba(74,4,78,.85)}.dark\:text-fuchsia-950\/90:is(.dark *){color:rgba(74,4,78,.9)}.dark\:text-fuchsia-950\/95:is(.dark *){color:rgba(74,4,78,.95)}.dark\:text-gray-100:is(.dark *){--tw-text-opacity:1;color:rgb(243 244 246/var(--tw-text-opacity,1))}.dark\:text-gray-100\/0:is(.dark *){color:rgba(243,244,246,0)}.dark\:text-gray-100\/10:is(.dark *){color:rgba(243,244,246,.1)}.dark\:text-gray-100\/100:is(.dark *){color:#f3f4f6}.dark\:text-gray-100\/15:is(.dark *){color:rgba(243,244,246,.15)}.dark\:text-gray-100\/20:is(.dark *){color:rgba(243,244,246,.2)}.dark\:text-gray-100\/25:is(.dark *){color:rgba(243,244,246,.25)}.dark\:text-gray-100\/30:is(.dark *){color:rgba(243,244,246,.3)}.dark\:text-gray-100\/35:is(.dark *){color:rgba(243,244,246,.35)}.dark\:text-gray-100\/40:is(.dark *){color:rgba(243,244,246,.4)}.dark\:text-gray-100\/45:is(.dark *){color:rgba(243,244,246,.45)}.dark\:text-gray-100\/5:is(.dark *){color:rgba(243,244,246,.05)}.dark\:text-gray-100\/50:is(.dark *){color:rgba(243,244,246,.5)}.dark\:text-gray-100\/55:is(.dark *){color:rgba(243,244,246,.55)}.dark\:text-gray-100\/60:is(.dark *){color:rgba(243,244,246,.6)}.dark\:text-gray-100\/65:is(.dark *){color:rgba(243,244,246,.65)}.dark\:text-gray-100\/70:is(.dark *){color:rgba(243,244,246,.7)}.dark\:text-gray-100\/75:is(.dark *){color:rgba(243,244,246,.75)}.dark\:text-gray-100\/80:is(.dark *){color:rgba(243,244,246,.8)}.dark\:text-gray-100\/85:is(.dark *){color:rgba(243,244,246,.85)}.dark\:text-gray-100\/90:is(.dark *){color:rgba(243,244,246,.9)}.dark\:text-gray-100\/95:is(.dark *){color:rgba(243,244,246,.95)}.dark\:text-gray-200:is(.dark *){--tw-text-opacity:1;color:rgb(229 231 235/var(--tw-text-opacity,1))}.dark\:text-gray-200\/0:is(.dark *){color:rgba(229,231,235,0)}.dark\:text-gray-200\/10:is(.dark *){color:rgba(229,231,235,.1)}.dark\:text-gray-200\/100:is(.dark *){color:#e5e7eb}.dark\:text-gray-200\/15:is(.dark *){color:rgba(229,231,235,.15)}.dark\:text-gray-200\/20:is(.dark *){color:rgba(229,231,235,.2)}.dark\:text-gray-200\/25:is(.dark *){color:rgba(229,231,235,.25)}.dark\:text-gray-200\/30:is(.dark *){color:rgba(229,231,235,.3)}.dark\:text-gray-200\/35:is(.dark *){color:rgba(229,231,235,.35)}.dark\:text-gray-200\/40:is(.dark *){color:rgba(229,231,235,.4)}.dark\:text-gray-200\/45:is(.dark *){color:rgba(229,231,235,.45)}.dark\:text-gray-200\/5:is(.dark *){color:rgba(229,231,235,.05)}.dark\:text-gray-200\/50:is(.dark *){color:rgba(229,231,235,.5)}.dark\:text-gray-200\/55:is(.dark *){color:rgba(229,231,235,.55)}.dark\:text-gray-200\/60:is(.dark *){color:rgba(229,231,235,.6)}.dark\:text-gray-200\/65:is(.dark *){color:rgba(229,231,235,.65)}.dark\:text-gray-200\/70:is(.dark *){color:rgba(229,231,235,.7)}.dark\:text-gray-200\/75:is(.dark *){color:rgba(229,231,235,.75)}.dark\:text-gray-200\/80:is(.dark *){color:rgba(229,231,235,.8)}.dark\:text-gray-200\/85:is(.dark *){color:rgba(229,231,235,.85)}.dark\:text-gray-200\/90:is(.dark *){color:rgba(229,231,235,.9)}.dark\:text-gray-200\/95:is(.dark *){color:rgba(229,231,235,.95)}.dark\:text-gray-50:is(.dark *){--tw-text-opacity:1;color:rgb(249 250 251/var(--tw-text-opacity,1))}.dark\:text-gray-50\/0:is(.dark *){color:rgba(249,250,251,0)}.dark\:text-gray-50\/10:is(.dark *){color:rgba(249,250,251,.1)}.dark\:text-gray-50\/100:is(.dark *){color:#f9fafb}.dark\:text-gray-50\/15:is(.dark *){color:rgba(249,250,251,.15)}.dark\:text-gray-50\/20:is(.dark *){color:rgba(249,250,251,.2)}.dark\:text-gray-50\/25:is(.dark *){color:rgba(249,250,251,.25)}.dark\:text-gray-50\/30:is(.dark *){color:rgba(249,250,251,.3)}.dark\:text-gray-50\/35:is(.dark *){color:rgba(249,250,251,.35)}.dark\:text-gray-50\/40:is(.dark *){color:rgba(249,250,251,.4)}.dark\:text-gray-50\/45:is(.dark *){color:rgba(249,250,251,.45)}.dark\:text-gray-50\/5:is(.dark *){color:rgba(249,250,251,.05)}.dark\:text-gray-50\/50:is(.dark *){color:rgba(249,250,251,.5)}.dark\:text-gray-50\/55:is(.dark *){color:rgba(249,250,251,.55)}.dark\:text-gray-50\/60:is(.dark *){color:rgba(249,250,251,.6)}.dark\:text-gray-50\/65:is(.dark *){color:rgba(249,250,251,.65)}.dark\:text-gray-50\/70:is(.dark *){color:rgba(249,250,251,.7)}.dark\:text-gray-50\/75:is(.dark *){color:rgba(249,250,251,.75)}.dark\:text-gray-50\/80:is(.dark *){color:rgba(249,250,251,.8)}.dark\:text-gray-50\/85:is(.dark *){color:rgba(249,250,251,.85)}.dark\:text-gray-50\/90:is(.dark *){color:rgba(249,250,251,.9)}.dark\:text-gray-50\/95:is(.dark *){color:rgba(249,250,251,.95)}.dark\:text-gray-500:is(.dark *){--tw-text-opacity:1;color:rgb(107 114 128/var(--tw-text-opacity,1))}.dark\:text-gray-500\/0:is(.dark *){color:hsla(220,9%,46%,0)}.dark\:text-gray-500\/10:is(.dark *){color:hsla(220,9%,46%,.1)}.dark\:text-gray-500\/100:is(.dark *){color:#6b7280}.dark\:text-gray-500\/15:is(.dark *){color:hsla(220,9%,46%,.15)}.dark\:text-gray-500\/20:is(.dark *){color:hsla(220,9%,46%,.2)}.dark\:text-gray-500\/25:is(.dark *){color:hsla(220,9%,46%,.25)}.dark\:text-gray-500\/30:is(.dark *){color:hsla(220,9%,46%,.3)}.dark\:text-gray-500\/35:is(.dark *){color:hsla(220,9%,46%,.35)}.dark\:text-gray-500\/40:is(.dark *){color:hsla(220,9%,46%,.4)}.dark\:text-gray-500\/45:is(.dark *){color:hsla(220,9%,46%,.45)}.dark\:text-gray-500\/5:is(.dark *){color:hsla(220,9%,46%,.05)}.dark\:text-gray-500\/50:is(.dark *){color:hsla(220,9%,46%,.5)}.dark\:text-gray-500\/55:is(.dark *){color:hsla(220,9%,46%,.55)}.dark\:text-gray-500\/60:is(.dark *){color:hsla(220,9%,46%,.6)}.dark\:text-gray-500\/65:is(.dark *){color:hsla(220,9%,46%,.65)}.dark\:text-gray-500\/70:is(.dark *){color:hsla(220,9%,46%,.7)}.dark\:text-gray-500\/75:is(.dark *){color:hsla(220,9%,46%,.75)}.dark\:text-gray-500\/80:is(.dark *){color:hsla(220,9%,46%,.8)}.dark\:text-gray-500\/85:is(.dark *){color:hsla(220,9%,46%,.85)}.dark\:text-gray-500\/90:is(.dark *){color:hsla(220,9%,46%,.9)}.dark\:text-gray-500\/95:is(.dark *){color:hsla(220,9%,46%,.95)}.dark\:text-gray-950:is(.dark *){--tw-text-opacity:1;color:rgb(3 7 18/var(--tw-text-opacity,1))}.dark\:text-gray-950\/0:is(.dark *){color:rgba(3,7,18,0)}.dark\:text-gray-950\/10:is(.dark *){color:rgba(3,7,18,.1)}.dark\:text-gray-950\/100:is(.dark *){color:#030712}.dark\:text-gray-950\/15:is(.dark *){color:rgba(3,7,18,.15)}.dark\:text-gray-950\/20:is(.dark *){color:rgba(3,7,18,.2)}.dark\:text-gray-950\/25:is(.dark *){color:rgba(3,7,18,.25)}.dark\:text-gray-950\/30:is(.dark *){color:rgba(3,7,18,.3)}.dark\:text-gray-950\/35:is(.dark *){color:rgba(3,7,18,.35)}.dark\:text-gray-950\/40:is(.dark *){color:rgba(3,7,18,.4)}.dark\:text-gray-950\/45:is(.dark *){color:rgba(3,7,18,.45)}.dark\:text-gray-950\/5:is(.dark *){color:rgba(3,7,18,.05)}.dark\:text-gray-950\/50:is(.dark *){color:rgba(3,7,18,.5)}.dark\:text-gray-950\/55:is(.dark *){color:rgba(3,7,18,.55)}.dark\:text-gray-950\/60:is(.dark *){color:rgba(3,7,18,.6)}.dark\:text-gray-950\/65:is(.dark *){color:rgba(3,7,18,.65)}.dark\:text-gray-950\/70:is(.dark *){color:rgba(3,7,18,.7)}.dark\:text-gray-950\/75:is(.dark *){color:rgba(3,7,18,.75)}.dark\:text-gray-950\/80:is(.dark *){color:rgba(3,7,18,.8)}.dark\:text-gray-950\/85:is(.dark *){color:rgba(3,7,18,.85)}.dark\:text-gray-950\/90:is(.dark *){color:rgba(3,7,18,.9)}.dark\:text-gray-950\/95:is(.dark *){color:rgba(3,7,18,.95)}.dark\:text-green-100:is(.dark *){--tw-text-opacity:1;color:rgb(220 252 231/var(--tw-text-opacity,1))}.dark\:text-green-100\/0:is(.dark *){color:rgba(220,252,231,0)}.dark\:text-green-100\/10:is(.dark *){color:rgba(220,252,231,.1)}.dark\:text-green-100\/100:is(.dark *){color:#dcfce7}.dark\:text-green-100\/15:is(.dark *){color:rgba(220,252,231,.15)}.dark\:text-green-100\/20:is(.dark *){color:rgba(220,252,231,.2)}.dark\:text-green-100\/25:is(.dark *){color:rgba(220,252,231,.25)}.dark\:text-green-100\/30:is(.dark *){color:rgba(220,252,231,.3)}.dark\:text-green-100\/35:is(.dark *){color:rgba(220,252,231,.35)}.dark\:text-green-100\/40:is(.dark *){color:rgba(220,252,231,.4)}.dark\:text-green-100\/45:is(.dark *){color:rgba(220,252,231,.45)}.dark\:text-green-100\/5:is(.dark *){color:rgba(220,252,231,.05)}.dark\:text-green-100\/50:is(.dark *){color:rgba(220,252,231,.5)}.dark\:text-green-100\/55:is(.dark *){color:rgba(220,252,231,.55)}.dark\:text-green-100\/60:is(.dark *){color:rgba(220,252,231,.6)}.dark\:text-green-100\/65:is(.dark *){color:rgba(220,252,231,.65)}.dark\:text-green-100\/70:is(.dark *){color:rgba(220,252,231,.7)}.dark\:text-green-100\/75:is(.dark *){color:rgba(220,252,231,.75)}.dark\:text-green-100\/80:is(.dark *){color:rgba(220,252,231,.8)}.dark\:text-green-100\/85:is(.dark *){color:rgba(220,252,231,.85)}.dark\:text-green-100\/90:is(.dark *){color:rgba(220,252,231,.9)}.dark\:text-green-100\/95:is(.dark *){color:rgba(220,252,231,.95)}.dark\:text-green-200:is(.dark *){--tw-text-opacity:1;color:rgb(187 247 208/var(--tw-text-opacity,1))}.dark\:text-green-200\/0:is(.dark *){color:rgba(187,247,208,0)}.dark\:text-green-200\/10:is(.dark *){color:rgba(187,247,208,.1)}.dark\:text-green-200\/100:is(.dark *){color:#bbf7d0}.dark\:text-green-200\/15:is(.dark *){color:rgba(187,247,208,.15)}.dark\:text-green-200\/20:is(.dark *){color:rgba(187,247,208,.2)}.dark\:text-green-200\/25:is(.dark *){color:rgba(187,247,208,.25)}.dark\:text-green-200\/30:is(.dark *){color:rgba(187,247,208,.3)}.dark\:text-green-200\/35:is(.dark *){color:rgba(187,247,208,.35)}.dark\:text-green-200\/40:is(.dark *){color:rgba(187,247,208,.4)}.dark\:text-green-200\/45:is(.dark *){color:rgba(187,247,208,.45)}.dark\:text-green-200\/5:is(.dark *){color:rgba(187,247,208,.05)}.dark\:text-green-200\/50:is(.dark *){color:rgba(187,247,208,.5)}.dark\:text-green-200\/55:is(.dark *){color:rgba(187,247,208,.55)}.dark\:text-green-200\/60:is(.dark *){color:rgba(187,247,208,.6)}.dark\:text-green-200\/65:is(.dark *){color:rgba(187,247,208,.65)}.dark\:text-green-200\/70:is(.dark *){color:rgba(187,247,208,.7)}.dark\:text-green-200\/75:is(.dark *){color:rgba(187,247,208,.75)}.dark\:text-green-200\/80:is(.dark *){color:rgba(187,247,208,.8)}.dark\:text-green-200\/85:is(.dark *){color:rgba(187,247,208,.85)}.dark\:text-green-200\/90:is(.dark *){color:rgba(187,247,208,.9)}.dark\:text-green-200\/95:is(.dark *){color:rgba(187,247,208,.95)}.dark\:text-green-50:is(.dark *){--tw-text-opacity:1;color:rgb(240 253 244/var(--tw-text-opacity,1))}.dark\:text-green-50\/0:is(.dark *){color:rgba(240,253,244,0)}.dark\:text-green-50\/10:is(.dark *){color:rgba(240,253,244,.1)}.dark\:text-green-50\/100:is(.dark *){color:#f0fdf4}.dark\:text-green-50\/15:is(.dark *){color:rgba(240,253,244,.15)}.dark\:text-green-50\/20:is(.dark *){color:rgba(240,253,244,.2)}.dark\:text-green-50\/25:is(.dark *){color:rgba(240,253,244,.25)}.dark\:text-green-50\/30:is(.dark *){color:rgba(240,253,244,.3)}.dark\:text-green-50\/35:is(.dark *){color:rgba(240,253,244,.35)}.dark\:text-green-50\/40:is(.dark *){color:rgba(240,253,244,.4)}.dark\:text-green-50\/45:is(.dark *){color:rgba(240,253,244,.45)}.dark\:text-green-50\/5:is(.dark *){color:rgba(240,253,244,.05)}.dark\:text-green-50\/50:is(.dark *){color:rgba(240,253,244,.5)}.dark\:text-green-50\/55:is(.dark *){color:rgba(240,253,244,.55)}.dark\:text-green-50\/60:is(.dark *){color:rgba(240,253,244,.6)}.dark\:text-green-50\/65:is(.dark *){color:rgba(240,253,244,.65)}.dark\:text-green-50\/70:is(.dark *){color:rgba(240,253,244,.7)}.dark\:text-green-50\/75:is(.dark *){color:rgba(240,253,244,.75)}.dark\:text-green-50\/80:is(.dark *){color:rgba(240,253,244,.8)}.dark\:text-green-50\/85:is(.dark *){color:rgba(240,253,244,.85)}.dark\:text-green-50\/90:is(.dark *){color:rgba(240,253,244,.9)}.dark\:text-green-50\/95:is(.dark *){color:rgba(240,253,244,.95)}.dark\:text-green-500:is(.dark *){--tw-text-opacity:1;color:rgb(34 197 94/var(--tw-text-opacity,1))}.dark\:text-green-500\/0:is(.dark *){color:rgba(34,197,94,0)}.dark\:text-green-500\/10:is(.dark *){color:rgba(34,197,94,.1)}.dark\:text-green-500\/100:is(.dark *){color:#22c55e}.dark\:text-green-500\/15:is(.dark *){color:rgba(34,197,94,.15)}.dark\:text-green-500\/20:is(.dark *){color:rgba(34,197,94,.2)}.dark\:text-green-500\/25:is(.dark *){color:rgba(34,197,94,.25)}.dark\:text-green-500\/30:is(.dark *){color:rgba(34,197,94,.3)}.dark\:text-green-500\/35:is(.dark *){color:rgba(34,197,94,.35)}.dark\:text-green-500\/40:is(.dark *){color:rgba(34,197,94,.4)}.dark\:text-green-500\/45:is(.dark *){color:rgba(34,197,94,.45)}.dark\:text-green-500\/5:is(.dark *){color:rgba(34,197,94,.05)}.dark\:text-green-500\/50:is(.dark *){color:rgba(34,197,94,.5)}.dark\:text-green-500\/55:is(.dark *){color:rgba(34,197,94,.55)}.dark\:text-green-500\/60:is(.dark *){color:rgba(34,197,94,.6)}.dark\:text-green-500\/65:is(.dark *){color:rgba(34,197,94,.65)}.dark\:text-green-500\/70:is(.dark *){color:rgba(34,197,94,.7)}.dark\:text-green-500\/75:is(.dark *){color:rgba(34,197,94,.75)}.dark\:text-green-500\/80:is(.dark *){color:rgba(34,197,94,.8)}.dark\:text-green-500\/85:is(.dark *){color:rgba(34,197,94,.85)}.dark\:text-green-500\/90:is(.dark *){color:rgba(34,197,94,.9)}.dark\:text-green-500\/95:is(.dark *){color:rgba(34,197,94,.95)}.dark\:text-green-950:is(.dark *){--tw-text-opacity:1;color:rgb(5 46 22/var(--tw-text-opacity,1))}.dark\:text-green-950\/0:is(.dark *){color:rgba(5,46,22,0)}.dark\:text-green-950\/10:is(.dark *){color:rgba(5,46,22,.1)}.dark\:text-green-950\/100:is(.dark *){color:#052e16}.dark\:text-green-950\/15:is(.dark *){color:rgba(5,46,22,.15)}.dark\:text-green-950\/20:is(.dark *){color:rgba(5,46,22,.2)}.dark\:text-green-950\/25:is(.dark *){color:rgba(5,46,22,.25)}.dark\:text-green-950\/30:is(.dark *){color:rgba(5,46,22,.3)}.dark\:text-green-950\/35:is(.dark *){color:rgba(5,46,22,.35)}.dark\:text-green-950\/40:is(.dark *){color:rgba(5,46,22,.4)}.dark\:text-green-950\/45:is(.dark *){color:rgba(5,46,22,.45)}.dark\:text-green-950\/5:is(.dark *){color:rgba(5,46,22,.05)}.dark\:text-green-950\/50:is(.dark *){color:rgba(5,46,22,.5)}.dark\:text-green-950\/55:is(.dark *){color:rgba(5,46,22,.55)}.dark\:text-green-950\/60:is(.dark *){color:rgba(5,46,22,.6)}.dark\:text-green-950\/65:is(.dark *){color:rgba(5,46,22,.65)}.dark\:text-green-950\/70:is(.dark *){color:rgba(5,46,22,.7)}.dark\:text-green-950\/75:is(.dark *){color:rgba(5,46,22,.75)}.dark\:text-green-950\/80:is(.dark *){color:rgba(5,46,22,.8)}.dark\:text-green-950\/85:is(.dark *){color:rgba(5,46,22,.85)}.dark\:text-green-950\/90:is(.dark *){color:rgba(5,46,22,.9)}.dark\:text-green-950\/95:is(.dark *){color:rgba(5,46,22,.95)}.dark\:text-indigo-100:is(.dark *){--tw-text-opacity:1;color:rgb(224 231 255/var(--tw-text-opacity,1))}.dark\:text-indigo-100\/0:is(.dark *){color:rgba(224,231,255,0)}.dark\:text-indigo-100\/10:is(.dark *){color:rgba(224,231,255,.1)}.dark\:text-indigo-100\/100:is(.dark *){color:#e0e7ff}.dark\:text-indigo-100\/15:is(.dark *){color:rgba(224,231,255,.15)}.dark\:text-indigo-100\/20:is(.dark *){color:rgba(224,231,255,.2)}.dark\:text-indigo-100\/25:is(.dark *){color:rgba(224,231,255,.25)}.dark\:text-indigo-100\/30:is(.dark *){color:rgba(224,231,255,.3)}.dark\:text-indigo-100\/35:is(.dark *){color:rgba(224,231,255,.35)}.dark\:text-indigo-100\/40:is(.dark *){color:rgba(224,231,255,.4)}.dark\:text-indigo-100\/45:is(.dark *){color:rgba(224,231,255,.45)}.dark\:text-indigo-100\/5:is(.dark *){color:rgba(224,231,255,.05)}.dark\:text-indigo-100\/50:is(.dark *){color:rgba(224,231,255,.5)}.dark\:text-indigo-100\/55:is(.dark *){color:rgba(224,231,255,.55)}.dark\:text-indigo-100\/60:is(.dark *){color:rgba(224,231,255,.6)}.dark\:text-indigo-100\/65:is(.dark *){color:rgba(224,231,255,.65)}.dark\:text-indigo-100\/70:is(.dark *){color:rgba(224,231,255,.7)}.dark\:text-indigo-100\/75:is(.dark *){color:rgba(224,231,255,.75)}.dark\:text-indigo-100\/80:is(.dark *){color:rgba(224,231,255,.8)}.dark\:text-indigo-100\/85:is(.dark *){color:rgba(224,231,255,.85)}.dark\:text-indigo-100\/90:is(.dark *){color:rgba(224,231,255,.9)}.dark\:text-indigo-100\/95:is(.dark *){color:rgba(224,231,255,.95)}.dark\:text-indigo-200:is(.dark *){--tw-text-opacity:1;color:rgb(199 210 254/var(--tw-text-opacity,1))}.dark\:text-indigo-200\/0:is(.dark *){color:rgba(199,210,254,0)}.dark\:text-indigo-200\/10:is(.dark *){color:rgba(199,210,254,.1)}.dark\:text-indigo-200\/100:is(.dark *){color:#c7d2fe}.dark\:text-indigo-200\/15:is(.dark *){color:rgba(199,210,254,.15)}.dark\:text-indigo-200\/20:is(.dark *){color:rgba(199,210,254,.2)}.dark\:text-indigo-200\/25:is(.dark *){color:rgba(199,210,254,.25)}.dark\:text-indigo-200\/30:is(.dark *){color:rgba(199,210,254,.3)}.dark\:text-indigo-200\/35:is(.dark *){color:rgba(199,210,254,.35)}.dark\:text-indigo-200\/40:is(.dark *){color:rgba(199,210,254,.4)}.dark\:text-indigo-200\/45:is(.dark *){color:rgba(199,210,254,.45)}.dark\:text-indigo-200\/5:is(.dark *){color:rgba(199,210,254,.05)}.dark\:text-indigo-200\/50:is(.dark *){color:rgba(199,210,254,.5)}.dark\:text-indigo-200\/55:is(.dark *){color:rgba(199,210,254,.55)}.dark\:text-indigo-200\/60:is(.dark *){color:rgba(199,210,254,.6)}.dark\:text-indigo-200\/65:is(.dark *){color:rgba(199,210,254,.65)}.dark\:text-indigo-200\/70:is(.dark *){color:rgba(199,210,254,.7)}.dark\:text-indigo-200\/75:is(.dark *){color:rgba(199,210,254,.75)}.dark\:text-indigo-200\/80:is(.dark *){color:rgba(199,210,254,.8)}.dark\:text-indigo-200\/85:is(.dark *){color:rgba(199,210,254,.85)}.dark\:text-indigo-200\/90:is(.dark *){color:rgba(199,210,254,.9)}.dark\:text-indigo-200\/95:is(.dark *){color:rgba(199,210,254,.95)}.dark\:text-indigo-50:is(.dark *){--tw-text-opacity:1;color:rgb(238 242 255/var(--tw-text-opacity,1))}.dark\:text-indigo-50\/0:is(.dark *){color:rgba(238,242,255,0)}.dark\:text-indigo-50\/10:is(.dark *){color:rgba(238,242,255,.1)}.dark\:text-indigo-50\/100:is(.dark *){color:#eef2ff}.dark\:text-indigo-50\/15:is(.dark *){color:rgba(238,242,255,.15)}.dark\:text-indigo-50\/20:is(.dark *){color:rgba(238,242,255,.2)}.dark\:text-indigo-50\/25:is(.dark *){color:rgba(238,242,255,.25)}.dark\:text-indigo-50\/30:is(.dark *){color:rgba(238,242,255,.3)}.dark\:text-indigo-50\/35:is(.dark *){color:rgba(238,242,255,.35)}.dark\:text-indigo-50\/40:is(.dark *){color:rgba(238,242,255,.4)}.dark\:text-indigo-50\/45:is(.dark *){color:rgba(238,242,255,.45)}.dark\:text-indigo-50\/5:is(.dark *){color:rgba(238,242,255,.05)}.dark\:text-indigo-50\/50:is(.dark *){color:rgba(238,242,255,.5)}.dark\:text-indigo-50\/55:is(.dark *){color:rgba(238,242,255,.55)}.dark\:text-indigo-50\/60:is(.dark *){color:rgba(238,242,255,.6)}.dark\:text-indigo-50\/65:is(.dark *){color:rgba(238,242,255,.65)}.dark\:text-indigo-50\/70:is(.dark *){color:rgba(238,242,255,.7)}.dark\:text-indigo-50\/75:is(.dark *){color:rgba(238,242,255,.75)}.dark\:text-indigo-50\/80:is(.dark *){color:rgba(238,242,255,.8)}.dark\:text-indigo-50\/85:is(.dark *){color:rgba(238,242,255,.85)}.dark\:text-indigo-50\/90:is(.dark *){color:rgba(238,242,255,.9)}.dark\:text-indigo-50\/95:is(.dark *){color:rgba(238,242,255,.95)}.dark\:text-indigo-500:is(.dark *){--tw-text-opacity:1;color:rgb(99 102 241/var(--tw-text-opacity,1))}.dark\:text-indigo-500\/0:is(.dark *){color:rgba(99,102,241,0)}.dark\:text-indigo-500\/10:is(.dark *){color:rgba(99,102,241,.1)}.dark\:text-indigo-500\/100:is(.dark *){color:#6366f1}.dark\:text-indigo-500\/15:is(.dark *){color:rgba(99,102,241,.15)}.dark\:text-indigo-500\/20:is(.dark *){color:rgba(99,102,241,.2)}.dark\:text-indigo-500\/25:is(.dark *){color:rgba(99,102,241,.25)}.dark\:text-indigo-500\/30:is(.dark *){color:rgba(99,102,241,.3)}.dark\:text-indigo-500\/35:is(.dark *){color:rgba(99,102,241,.35)}.dark\:text-indigo-500\/40:is(.dark *){color:rgba(99,102,241,.4)}.dark\:text-indigo-500\/45:is(.dark *){color:rgba(99,102,241,.45)}.dark\:text-indigo-500\/5:is(.dark *){color:rgba(99,102,241,.05)}.dark\:text-indigo-500\/50:is(.dark *){color:rgba(99,102,241,.5)}.dark\:text-indigo-500\/55:is(.dark *){color:rgba(99,102,241,.55)}.dark\:text-indigo-500\/60:is(.dark *){color:rgba(99,102,241,.6)}.dark\:text-indigo-500\/65:is(.dark *){color:rgba(99,102,241,.65)}.dark\:text-indigo-500\/70:is(.dark *){color:rgba(99,102,241,.7)}.dark\:text-indigo-500\/75:is(.dark *){color:rgba(99,102,241,.75)}.dark\:text-indigo-500\/80:is(.dark *){color:rgba(99,102,241,.8)}.dark\:text-indigo-500\/85:is(.dark *){color:rgba(99,102,241,.85)}.dark\:text-indigo-500\/90:is(.dark *){color:rgba(99,102,241,.9)}.dark\:text-indigo-500\/95:is(.dark *){color:rgba(99,102,241,.95)}.dark\:text-indigo-950:is(.dark *){--tw-text-opacity:1;color:rgb(30 27 75/var(--tw-text-opacity,1))}.dark\:text-indigo-950\/0:is(.dark *){color:rgba(30,27,75,0)}.dark\:text-indigo-950\/10:is(.dark *){color:rgba(30,27,75,.1)}.dark\:text-indigo-950\/100:is(.dark *){color:#1e1b4b}.dark\:text-indigo-950\/15:is(.dark *){color:rgba(30,27,75,.15)}.dark\:text-indigo-950\/20:is(.dark *){color:rgba(30,27,75,.2)}.dark\:text-indigo-950\/25:is(.dark *){color:rgba(30,27,75,.25)}.dark\:text-indigo-950\/30:is(.dark *){color:rgba(30,27,75,.3)}.dark\:text-indigo-950\/35:is(.dark *){color:rgba(30,27,75,.35)}.dark\:text-indigo-950\/40:is(.dark *){color:rgba(30,27,75,.4)}.dark\:text-indigo-950\/45:is(.dark *){color:rgba(30,27,75,.45)}.dark\:text-indigo-950\/5:is(.dark *){color:rgba(30,27,75,.05)}.dark\:text-indigo-950\/50:is(.dark *){color:rgba(30,27,75,.5)}.dark\:text-indigo-950\/55:is(.dark *){color:rgba(30,27,75,.55)}.dark\:text-indigo-950\/60:is(.dark *){color:rgba(30,27,75,.6)}.dark\:text-indigo-950\/65:is(.dark *){color:rgba(30,27,75,.65)}.dark\:text-indigo-950\/70:is(.dark *){color:rgba(30,27,75,.7)}.dark\:text-indigo-950\/75:is(.dark *){color:rgba(30,27,75,.75)}.dark\:text-indigo-950\/80:is(.dark *){color:rgba(30,27,75,.8)}.dark\:text-indigo-950\/85:is(.dark *){color:rgba(30,27,75,.85)}.dark\:text-indigo-950\/90:is(.dark *){color:rgba(30,27,75,.9)}.dark\:text-indigo-950\/95:is(.dark *){color:rgba(30,27,75,.95)}.dark\:text-lime-100:is(.dark *){--tw-text-opacity:1;color:rgb(236 252 203/var(--tw-text-opacity,1))}.dark\:text-lime-100\/0:is(.dark *){color:hsla(80,89%,89%,0)}.dark\:text-lime-100\/10:is(.dark *){color:hsla(80,89%,89%,.1)}.dark\:text-lime-100\/100:is(.dark *){color:#ecfccb}.dark\:text-lime-100\/15:is(.dark *){color:hsla(80,89%,89%,.15)}.dark\:text-lime-100\/20:is(.dark *){color:hsla(80,89%,89%,.2)}.dark\:text-lime-100\/25:is(.dark *){color:hsla(80,89%,89%,.25)}.dark\:text-lime-100\/30:is(.dark *){color:hsla(80,89%,89%,.3)}.dark\:text-lime-100\/35:is(.dark *){color:hsla(80,89%,89%,.35)}.dark\:text-lime-100\/40:is(.dark *){color:hsla(80,89%,89%,.4)}.dark\:text-lime-100\/45:is(.dark *){color:hsla(80,89%,89%,.45)}.dark\:text-lime-100\/5:is(.dark *){color:hsla(80,89%,89%,.05)}.dark\:text-lime-100\/50:is(.dark *){color:hsla(80,89%,89%,.5)}.dark\:text-lime-100\/55:is(.dark *){color:hsla(80,89%,89%,.55)}.dark\:text-lime-100\/60:is(.dark *){color:hsla(80,89%,89%,.6)}.dark\:text-lime-100\/65:is(.dark *){color:hsla(80,89%,89%,.65)}.dark\:text-lime-100\/70:is(.dark *){color:hsla(80,89%,89%,.7)}.dark\:text-lime-100\/75:is(.dark *){color:hsla(80,89%,89%,.75)}.dark\:text-lime-100\/80:is(.dark *){color:hsla(80,89%,89%,.8)}.dark\:text-lime-100\/85:is(.dark *){color:hsla(80,89%,89%,.85)}.dark\:text-lime-100\/90:is(.dark *){color:hsla(80,89%,89%,.9)}.dark\:text-lime-100\/95:is(.dark *){color:hsla(80,89%,89%,.95)}.dark\:text-lime-200:is(.dark *){--tw-text-opacity:1;color:rgb(217 249 157/var(--tw-text-opacity,1))}.dark\:text-lime-200\/0:is(.dark *){color:hsla(81,88%,80%,0)}.dark\:text-lime-200\/10:is(.dark *){color:hsla(81,88%,80%,.1)}.dark\:text-lime-200\/100:is(.dark *){color:#d9f99d}.dark\:text-lime-200\/15:is(.dark *){color:hsla(81,88%,80%,.15)}.dark\:text-lime-200\/20:is(.dark *){color:hsla(81,88%,80%,.2)}.dark\:text-lime-200\/25:is(.dark *){color:hsla(81,88%,80%,.25)}.dark\:text-lime-200\/30:is(.dark *){color:hsla(81,88%,80%,.3)}.dark\:text-lime-200\/35:is(.dark *){color:hsla(81,88%,80%,.35)}.dark\:text-lime-200\/40:is(.dark *){color:hsla(81,88%,80%,.4)}.dark\:text-lime-200\/45:is(.dark *){color:hsla(81,88%,80%,.45)}.dark\:text-lime-200\/5:is(.dark *){color:hsla(81,88%,80%,.05)}.dark\:text-lime-200\/50:is(.dark *){color:hsla(81,88%,80%,.5)}.dark\:text-lime-200\/55:is(.dark *){color:hsla(81,88%,80%,.55)}.dark\:text-lime-200\/60:is(.dark *){color:hsla(81,88%,80%,.6)}.dark\:text-lime-200\/65:is(.dark *){color:hsla(81,88%,80%,.65)}.dark\:text-lime-200\/70:is(.dark *){color:hsla(81,88%,80%,.7)}.dark\:text-lime-200\/75:is(.dark *){color:hsla(81,88%,80%,.75)}.dark\:text-lime-200\/80:is(.dark *){color:hsla(81,88%,80%,.8)}.dark\:text-lime-200\/85:is(.dark *){color:hsla(81,88%,80%,.85)}.dark\:text-lime-200\/90:is(.dark *){color:hsla(81,88%,80%,.9)}.dark\:text-lime-200\/95:is(.dark *){color:hsla(81,88%,80%,.95)}.dark\:text-lime-50:is(.dark *){--tw-text-opacity:1;color:rgb(247 254 231/var(--tw-text-opacity,1))}.dark\:text-lime-50\/0:is(.dark *){color:hsla(78,92%,95%,0)}.dark\:text-lime-50\/10:is(.dark *){color:hsla(78,92%,95%,.1)}.dark\:text-lime-50\/100:is(.dark *){color:#f7fee7}.dark\:text-lime-50\/15:is(.dark *){color:hsla(78,92%,95%,.15)}.dark\:text-lime-50\/20:is(.dark *){color:hsla(78,92%,95%,.2)}.dark\:text-lime-50\/25:is(.dark *){color:hsla(78,92%,95%,.25)}.dark\:text-lime-50\/30:is(.dark *){color:hsla(78,92%,95%,.3)}.dark\:text-lime-50\/35:is(.dark *){color:hsla(78,92%,95%,.35)}.dark\:text-lime-50\/40:is(.dark *){color:hsla(78,92%,95%,.4)}.dark\:text-lime-50\/45:is(.dark *){color:hsla(78,92%,95%,.45)}.dark\:text-lime-50\/5:is(.dark *){color:hsla(78,92%,95%,.05)}.dark\:text-lime-50\/50:is(.dark *){color:hsla(78,92%,95%,.5)}.dark\:text-lime-50\/55:is(.dark *){color:hsla(78,92%,95%,.55)}.dark\:text-lime-50\/60:is(.dark *){color:hsla(78,92%,95%,.6)}.dark\:text-lime-50\/65:is(.dark *){color:hsla(78,92%,95%,.65)}.dark\:text-lime-50\/70:is(.dark *){color:hsla(78,92%,95%,.7)}.dark\:text-lime-50\/75:is(.dark *){color:hsla(78,92%,95%,.75)}.dark\:text-lime-50\/80:is(.dark *){color:hsla(78,92%,95%,.8)}.dark\:text-lime-50\/85:is(.dark *){color:hsla(78,92%,95%,.85)}.dark\:text-lime-50\/90:is(.dark *){color:hsla(78,92%,95%,.9)}.dark\:text-lime-50\/95:is(.dark *){color:hsla(78,92%,95%,.95)}.dark\:text-lime-500:is(.dark *){--tw-text-opacity:1;color:rgb(132 204 22/var(--tw-text-opacity,1))}.dark\:text-lime-500\/0:is(.dark *){color:rgba(132,204,22,0)}.dark\:text-lime-500\/10:is(.dark *){color:rgba(132,204,22,.1)}.dark\:text-lime-500\/100:is(.dark *){color:#84cc16}.dark\:text-lime-500\/15:is(.dark *){color:rgba(132,204,22,.15)}.dark\:text-lime-500\/20:is(.dark *){color:rgba(132,204,22,.2)}.dark\:text-lime-500\/25:is(.dark *){color:rgba(132,204,22,.25)}.dark\:text-lime-500\/30:is(.dark *){color:rgba(132,204,22,.3)}.dark\:text-lime-500\/35:is(.dark *){color:rgba(132,204,22,.35)}.dark\:text-lime-500\/40:is(.dark *){color:rgba(132,204,22,.4)}.dark\:text-lime-500\/45:is(.dark *){color:rgba(132,204,22,.45)}.dark\:text-lime-500\/5:is(.dark *){color:rgba(132,204,22,.05)}.dark\:text-lime-500\/50:is(.dark *){color:rgba(132,204,22,.5)}.dark\:text-lime-500\/55:is(.dark *){color:rgba(132,204,22,.55)}.dark\:text-lime-500\/60:is(.dark *){color:rgba(132,204,22,.6)}.dark\:text-lime-500\/65:is(.dark *){color:rgba(132,204,22,.65)}.dark\:text-lime-500\/70:is(.dark *){color:rgba(132,204,22,.7)}.dark\:text-lime-500\/75:is(.dark *){color:rgba(132,204,22,.75)}.dark\:text-lime-500\/80:is(.dark *){color:rgba(132,204,22,.8)}.dark\:text-lime-500\/85:is(.dark *){color:rgba(132,204,22,.85)}.dark\:text-lime-500\/90:is(.dark *){color:rgba(132,204,22,.9)}.dark\:text-lime-500\/95:is(.dark *){color:rgba(132,204,22,.95)}.dark\:text-lime-950:is(.dark *){--tw-text-opacity:1;color:rgb(26 46 5/var(--tw-text-opacity,1))}.dark\:text-lime-950\/0:is(.dark *){color:rgba(26,46,5,0)}.dark\:text-lime-950\/10:is(.dark *){color:rgba(26,46,5,.1)}.dark\:text-lime-950\/100:is(.dark *){color:#1a2e05}.dark\:text-lime-950\/15:is(.dark *){color:rgba(26,46,5,.15)}.dark\:text-lime-950\/20:is(.dark *){color:rgba(26,46,5,.2)}.dark\:text-lime-950\/25:is(.dark *){color:rgba(26,46,5,.25)}.dark\:text-lime-950\/30:is(.dark *){color:rgba(26,46,5,.3)}.dark\:text-lime-950\/35:is(.dark *){color:rgba(26,46,5,.35)}.dark\:text-lime-950\/40:is(.dark *){color:rgba(26,46,5,.4)}.dark\:text-lime-950\/45:is(.dark *){color:rgba(26,46,5,.45)}.dark\:text-lime-950\/5:is(.dark *){color:rgba(26,46,5,.05)}.dark\:text-lime-950\/50:is(.dark *){color:rgba(26,46,5,.5)}.dark\:text-lime-950\/55:is(.dark *){color:rgba(26,46,5,.55)}.dark\:text-lime-950\/60:is(.dark *){color:rgba(26,46,5,.6)}.dark\:text-lime-950\/65:is(.dark *){color:rgba(26,46,5,.65)}.dark\:text-lime-950\/70:is(.dark *){color:rgba(26,46,5,.7)}.dark\:text-lime-950\/75:is(.dark *){color:rgba(26,46,5,.75)}.dark\:text-lime-950\/80:is(.dark *){color:rgba(26,46,5,.8)}.dark\:text-lime-950\/85:is(.dark *){color:rgba(26,46,5,.85)}.dark\:text-lime-950\/90:is(.dark *){color:rgba(26,46,5,.9)}.dark\:text-lime-950\/95:is(.dark *){color:rgba(26,46,5,.95)}.dark\:text-neutral-100:is(.dark *){--tw-text-opacity:1;color:rgb(245 245 245/var(--tw-text-opacity,1))}.dark\:text-neutral-100\/0:is(.dark *){color:hsla(0,0%,96%,0)}.dark\:text-neutral-100\/10:is(.dark *){color:hsla(0,0%,96%,.1)}.dark\:text-neutral-100\/100:is(.dark *){color:#f5f5f5}.dark\:text-neutral-100\/15:is(.dark *){color:hsla(0,0%,96%,.15)}.dark\:text-neutral-100\/20:is(.dark *){color:hsla(0,0%,96%,.2)}.dark\:text-neutral-100\/25:is(.dark *){color:hsla(0,0%,96%,.25)}.dark\:text-neutral-100\/30:is(.dark *){color:hsla(0,0%,96%,.3)}.dark\:text-neutral-100\/35:is(.dark *){color:hsla(0,0%,96%,.35)}.dark\:text-neutral-100\/40:is(.dark *){color:hsla(0,0%,96%,.4)}.dark\:text-neutral-100\/45:is(.dark *){color:hsla(0,0%,96%,.45)}.dark\:text-neutral-100\/5:is(.dark *){color:hsla(0,0%,96%,.05)}.dark\:text-neutral-100\/50:is(.dark *){color:hsla(0,0%,96%,.5)}.dark\:text-neutral-100\/55:is(.dark *){color:hsla(0,0%,96%,.55)}.dark\:text-neutral-100\/60:is(.dark *){color:hsla(0,0%,96%,.6)}.dark\:text-neutral-100\/65:is(.dark *){color:hsla(0,0%,96%,.65)}.dark\:text-neutral-100\/70:is(.dark *){color:hsla(0,0%,96%,.7)}.dark\:text-neutral-100\/75:is(.dark *){color:hsla(0,0%,96%,.75)}.dark\:text-neutral-100\/80:is(.dark *){color:hsla(0,0%,96%,.8)}.dark\:text-neutral-100\/85:is(.dark *){color:hsla(0,0%,96%,.85)}.dark\:text-neutral-100\/90:is(.dark *){color:hsla(0,0%,96%,.9)}.dark\:text-neutral-100\/95:is(.dark *){color:hsla(0,0%,96%,.95)}.dark\:text-neutral-200:is(.dark *){--tw-text-opacity:1;color:rgb(229 229 229/var(--tw-text-opacity,1))}.dark\:text-neutral-200\/0:is(.dark *){color:hsla(0,0%,90%,0)}.dark\:text-neutral-200\/10:is(.dark *){color:hsla(0,0%,90%,.1)}.dark\:text-neutral-200\/100:is(.dark *){color:#e5e5e5}.dark\:text-neutral-200\/15:is(.dark *){color:hsla(0,0%,90%,.15)}.dark\:text-neutral-200\/20:is(.dark *){color:hsla(0,0%,90%,.2)}.dark\:text-neutral-200\/25:is(.dark *){color:hsla(0,0%,90%,.25)}.dark\:text-neutral-200\/30:is(.dark *){color:hsla(0,0%,90%,.3)}.dark\:text-neutral-200\/35:is(.dark *){color:hsla(0,0%,90%,.35)}.dark\:text-neutral-200\/40:is(.dark *){color:hsla(0,0%,90%,.4)}.dark\:text-neutral-200\/45:is(.dark *){color:hsla(0,0%,90%,.45)}.dark\:text-neutral-200\/5:is(.dark *){color:hsla(0,0%,90%,.05)}.dark\:text-neutral-200\/50:is(.dark *){color:hsla(0,0%,90%,.5)}.dark\:text-neutral-200\/55:is(.dark *){color:hsla(0,0%,90%,.55)}.dark\:text-neutral-200\/60:is(.dark *){color:hsla(0,0%,90%,.6)}.dark\:text-neutral-200\/65:is(.dark *){color:hsla(0,0%,90%,.65)}.dark\:text-neutral-200\/70:is(.dark *){color:hsla(0,0%,90%,.7)}.dark\:text-neutral-200\/75:is(.dark *){color:hsla(0,0%,90%,.75)}.dark\:text-neutral-200\/80:is(.dark *){color:hsla(0,0%,90%,.8)}.dark\:text-neutral-200\/85:is(.dark *){color:hsla(0,0%,90%,.85)}.dark\:text-neutral-200\/90:is(.dark *){color:hsla(0,0%,90%,.9)}.dark\:text-neutral-200\/95:is(.dark *){color:hsla(0,0%,90%,.95)}.dark\:text-neutral-400:is(.dark *){--tw-text-opacity:1;color:rgb(163 163 163/var(--tw-text-opacity,1))}.dark\:text-neutral-50:is(.dark *){--tw-text-opacity:1;color:rgb(250 250 250/var(--tw-text-opacity,1))}.dark\:text-neutral-50\/0:is(.dark *){color:hsla(0,0%,98%,0)}.dark\:text-neutral-50\/10:is(.dark *){color:hsla(0,0%,98%,.1)}.dark\:text-neutral-50\/100:is(.dark *){color:#fafafa}.dark\:text-neutral-50\/15:is(.dark *){color:hsla(0,0%,98%,.15)}.dark\:text-neutral-50\/20:is(.dark *){color:hsla(0,0%,98%,.2)}.dark\:text-neutral-50\/25:is(.dark *){color:hsla(0,0%,98%,.25)}.dark\:text-neutral-50\/30:is(.dark *){color:hsla(0,0%,98%,.3)}.dark\:text-neutral-50\/35:is(.dark *){color:hsla(0,0%,98%,.35)}.dark\:text-neutral-50\/40:is(.dark *){color:hsla(0,0%,98%,.4)}.dark\:text-neutral-50\/45:is(.dark *){color:hsla(0,0%,98%,.45)}.dark\:text-neutral-50\/5:is(.dark *){color:hsla(0,0%,98%,.05)}.dark\:text-neutral-50\/50:is(.dark *){color:hsla(0,0%,98%,.5)}.dark\:text-neutral-50\/55:is(.dark *){color:hsla(0,0%,98%,.55)}.dark\:text-neutral-50\/60:is(.dark *){color:hsla(0,0%,98%,.6)}.dark\:text-neutral-50\/65:is(.dark *){color:hsla(0,0%,98%,.65)}.dark\:text-neutral-50\/70:is(.dark *){color:hsla(0,0%,98%,.7)}.dark\:text-neutral-50\/75:is(.dark *){color:hsla(0,0%,98%,.75)}.dark\:text-neutral-50\/80:is(.dark *){color:hsla(0,0%,98%,.8)}.dark\:text-neutral-50\/85:is(.dark *){color:hsla(0,0%,98%,.85)}.dark\:text-neutral-50\/90:is(.dark *){color:hsla(0,0%,98%,.9)}.dark\:text-neutral-50\/95:is(.dark *){color:hsla(0,0%,98%,.95)}.dark\:text-neutral-500:is(.dark *){--tw-text-opacity:1;color:rgb(115 115 115/var(--tw-text-opacity,1))}.dark\:text-neutral-500\/0:is(.dark *){color:hsla(0,0%,45%,0)}.dark\:text-neutral-500\/10:is(.dark *){color:hsla(0,0%,45%,.1)}.dark\:text-neutral-500\/100:is(.dark *){color:#737373}.dark\:text-neutral-500\/15:is(.dark *){color:hsla(0,0%,45%,.15)}.dark\:text-neutral-500\/20:is(.dark *){color:hsla(0,0%,45%,.2)}.dark\:text-neutral-500\/25:is(.dark *){color:hsla(0,0%,45%,.25)}.dark\:text-neutral-500\/30:is(.dark *){color:hsla(0,0%,45%,.3)}.dark\:text-neutral-500\/35:is(.dark *){color:hsla(0,0%,45%,.35)}.dark\:text-neutral-500\/40:is(.dark *){color:hsla(0,0%,45%,.4)}.dark\:text-neutral-500\/45:is(.dark *){color:hsla(0,0%,45%,.45)}.dark\:text-neutral-500\/5:is(.dark *){color:hsla(0,0%,45%,.05)}.dark\:text-neutral-500\/50:is(.dark *){color:hsla(0,0%,45%,.5)}.dark\:text-neutral-500\/55:is(.dark *){color:hsla(0,0%,45%,.55)}.dark\:text-neutral-500\/60:is(.dark *){color:hsla(0,0%,45%,.6)}.dark\:text-neutral-500\/65:is(.dark *){color:hsla(0,0%,45%,.65)}.dark\:text-neutral-500\/70:is(.dark *){color:hsla(0,0%,45%,.7)}.dark\:text-neutral-500\/75:is(.dark *){color:hsla(0,0%,45%,.75)}.dark\:text-neutral-500\/80:is(.dark *){color:hsla(0,0%,45%,.8)}.dark\:text-neutral-500\/85:is(.dark *){color:hsla(0,0%,45%,.85)}.dark\:text-neutral-500\/90:is(.dark *){color:hsla(0,0%,45%,.9)}.dark\:text-neutral-500\/95:is(.dark *){color:hsla(0,0%,45%,.95)}.dark\:text-neutral-950:is(.dark *){--tw-text-opacity:1;color:rgb(10 10 10/var(--tw-text-opacity,1))}.dark\:text-neutral-950\/0:is(.dark *){color:hsla(0,0%,4%,0)}.dark\:text-neutral-950\/10:is(.dark *){color:hsla(0,0%,4%,.1)}.dark\:text-neutral-950\/100:is(.dark *){color:#0a0a0a}.dark\:text-neutral-950\/15:is(.dark *){color:hsla(0,0%,4%,.15)}.dark\:text-neutral-950\/20:is(.dark *){color:hsla(0,0%,4%,.2)}.dark\:text-neutral-950\/25:is(.dark *){color:hsla(0,0%,4%,.25)}.dark\:text-neutral-950\/30:is(.dark *){color:hsla(0,0%,4%,.3)}.dark\:text-neutral-950\/35:is(.dark *){color:hsla(0,0%,4%,.35)}.dark\:text-neutral-950\/40:is(.dark *){color:hsla(0,0%,4%,.4)}.dark\:text-neutral-950\/45:is(.dark *){color:hsla(0,0%,4%,.45)}.dark\:text-neutral-950\/5:is(.dark *){color:hsla(0,0%,4%,.05)}.dark\:text-neutral-950\/50:is(.dark *){color:hsla(0,0%,4%,.5)}.dark\:text-neutral-950\/55:is(.dark *){color:hsla(0,0%,4%,.55)}.dark\:text-neutral-950\/60:is(.dark *){color:hsla(0,0%,4%,.6)}.dark\:text-neutral-950\/65:is(.dark *){color:hsla(0,0%,4%,.65)}.dark\:text-neutral-950\/70:is(.dark *){color:hsla(0,0%,4%,.7)}.dark\:text-neutral-950\/75:is(.dark *){color:hsla(0,0%,4%,.75)}.dark\:text-neutral-950\/80:is(.dark *){color:hsla(0,0%,4%,.8)}.dark\:text-neutral-950\/85:is(.dark *){color:hsla(0,0%,4%,.85)}.dark\:text-neutral-950\/90:is(.dark *){color:hsla(0,0%,4%,.9)}.dark\:text-neutral-950\/95:is(.dark *){color:hsla(0,0%,4%,.95)}.dark\:text-orange-100:is(.dark *){--tw-text-opacity:1;color:rgb(255 237 213/var(--tw-text-opacity,1))}.dark\:text-orange-100\/0:is(.dark *){color:rgba(255,237,213,0)}.dark\:text-orange-100\/10:is(.dark *){color:rgba(255,237,213,.1)}.dark\:text-orange-100\/100:is(.dark *){color:#ffedd5}.dark\:text-orange-100\/15:is(.dark *){color:rgba(255,237,213,.15)}.dark\:text-orange-100\/20:is(.dark *){color:rgba(255,237,213,.2)}.dark\:text-orange-100\/25:is(.dark *){color:rgba(255,237,213,.25)}.dark\:text-orange-100\/30:is(.dark *){color:rgba(255,237,213,.3)}.dark\:text-orange-100\/35:is(.dark *){color:rgba(255,237,213,.35)}.dark\:text-orange-100\/40:is(.dark *){color:rgba(255,237,213,.4)}.dark\:text-orange-100\/45:is(.dark *){color:rgba(255,237,213,.45)}.dark\:text-orange-100\/5:is(.dark *){color:rgba(255,237,213,.05)}.dark\:text-orange-100\/50:is(.dark *){color:rgba(255,237,213,.5)}.dark\:text-orange-100\/55:is(.dark *){color:rgba(255,237,213,.55)}.dark\:text-orange-100\/60:is(.dark *){color:rgba(255,237,213,.6)}.dark\:text-orange-100\/65:is(.dark *){color:rgba(255,237,213,.65)}.dark\:text-orange-100\/70:is(.dark *){color:rgba(255,237,213,.7)}.dark\:text-orange-100\/75:is(.dark *){color:rgba(255,237,213,.75)}.dark\:text-orange-100\/80:is(.dark *){color:rgba(255,237,213,.8)}.dark\:text-orange-100\/85:is(.dark *){color:rgba(255,237,213,.85)}.dark\:text-orange-100\/90:is(.dark *){color:rgba(255,237,213,.9)}.dark\:text-orange-100\/95:is(.dark *){color:rgba(255,237,213,.95)}.dark\:text-orange-200:is(.dark *){--tw-text-opacity:1;color:rgb(254 215 170/var(--tw-text-opacity,1))}.dark\:text-orange-200\/0:is(.dark *){color:hsla(32,98%,83%,0)}.dark\:text-orange-200\/10:is(.dark *){color:hsla(32,98%,83%,.1)}.dark\:text-orange-200\/100:is(.dark *){color:#fed7aa}.dark\:text-orange-200\/15:is(.dark *){color:hsla(32,98%,83%,.15)}.dark\:text-orange-200\/20:is(.dark *){color:hsla(32,98%,83%,.2)}.dark\:text-orange-200\/25:is(.dark *){color:hsla(32,98%,83%,.25)}.dark\:text-orange-200\/30:is(.dark *){color:hsla(32,98%,83%,.3)}.dark\:text-orange-200\/35:is(.dark *){color:hsla(32,98%,83%,.35)}.dark\:text-orange-200\/40:is(.dark *){color:hsla(32,98%,83%,.4)}.dark\:text-orange-200\/45:is(.dark *){color:hsla(32,98%,83%,.45)}.dark\:text-orange-200\/5:is(.dark *){color:hsla(32,98%,83%,.05)}.dark\:text-orange-200\/50:is(.dark *){color:hsla(32,98%,83%,.5)}.dark\:text-orange-200\/55:is(.dark *){color:hsla(32,98%,83%,.55)}.dark\:text-orange-200\/60:is(.dark *){color:hsla(32,98%,83%,.6)}.dark\:text-orange-200\/65:is(.dark *){color:hsla(32,98%,83%,.65)}.dark\:text-orange-200\/70:is(.dark *){color:hsla(32,98%,83%,.7)}.dark\:text-orange-200\/75:is(.dark *){color:hsla(32,98%,83%,.75)}.dark\:text-orange-200\/80:is(.dark *){color:hsla(32,98%,83%,.8)}.dark\:text-orange-200\/85:is(.dark *){color:hsla(32,98%,83%,.85)}.dark\:text-orange-200\/90:is(.dark *){color:hsla(32,98%,83%,.9)}.dark\:text-orange-200\/95:is(.dark *){color:hsla(32,98%,83%,.95)}.dark\:text-orange-50:is(.dark *){--tw-text-opacity:1;color:rgb(255 247 237/var(--tw-text-opacity,1))}.dark\:text-orange-50\/0:is(.dark *){color:rgba(255,247,237,0)}.dark\:text-orange-50\/10:is(.dark *){color:rgba(255,247,237,.1)}.dark\:text-orange-50\/100:is(.dark *){color:#fff7ed}.dark\:text-orange-50\/15:is(.dark *){color:rgba(255,247,237,.15)}.dark\:text-orange-50\/20:is(.dark *){color:rgba(255,247,237,.2)}.dark\:text-orange-50\/25:is(.dark *){color:rgba(255,247,237,.25)}.dark\:text-orange-50\/30:is(.dark *){color:rgba(255,247,237,.3)}.dark\:text-orange-50\/35:is(.dark *){color:rgba(255,247,237,.35)}.dark\:text-orange-50\/40:is(.dark *){color:rgba(255,247,237,.4)}.dark\:text-orange-50\/45:is(.dark *){color:rgba(255,247,237,.45)}.dark\:text-orange-50\/5:is(.dark *){color:rgba(255,247,237,.05)}.dark\:text-orange-50\/50:is(.dark *){color:rgba(255,247,237,.5)}.dark\:text-orange-50\/55:is(.dark *){color:rgba(255,247,237,.55)}.dark\:text-orange-50\/60:is(.dark *){color:rgba(255,247,237,.6)}.dark\:text-orange-50\/65:is(.dark *){color:rgba(255,247,237,.65)}.dark\:text-orange-50\/70:is(.dark *){color:rgba(255,247,237,.7)}.dark\:text-orange-50\/75:is(.dark *){color:rgba(255,247,237,.75)}.dark\:text-orange-50\/80:is(.dark *){color:rgba(255,247,237,.8)}.dark\:text-orange-50\/85:is(.dark *){color:rgba(255,247,237,.85)}.dark\:text-orange-50\/90:is(.dark *){color:rgba(255,247,237,.9)}.dark\:text-orange-50\/95:is(.dark *){color:rgba(255,247,237,.95)}.dark\:text-orange-500:is(.dark *){--tw-text-opacity:1;color:rgb(249 115 22/var(--tw-text-opacity,1))}.dark\:text-orange-500\/0:is(.dark *){color:rgba(249,115,22,0)}.dark\:text-orange-500\/10:is(.dark *){color:rgba(249,115,22,.1)}.dark\:text-orange-500\/100:is(.dark *){color:#f97316}.dark\:text-orange-500\/15:is(.dark *){color:rgba(249,115,22,.15)}.dark\:text-orange-500\/20:is(.dark *){color:rgba(249,115,22,.2)}.dark\:text-orange-500\/25:is(.dark *){color:rgba(249,115,22,.25)}.dark\:text-orange-500\/30:is(.dark *){color:rgba(249,115,22,.3)}.dark\:text-orange-500\/35:is(.dark *){color:rgba(249,115,22,.35)}.dark\:text-orange-500\/40:is(.dark *){color:rgba(249,115,22,.4)}.dark\:text-orange-500\/45:is(.dark *){color:rgba(249,115,22,.45)}.dark\:text-orange-500\/5:is(.dark *){color:rgba(249,115,22,.05)}.dark\:text-orange-500\/50:is(.dark *){color:rgba(249,115,22,.5)}.dark\:text-orange-500\/55:is(.dark *){color:rgba(249,115,22,.55)}.dark\:text-orange-500\/60:is(.dark *){color:rgba(249,115,22,.6)}.dark\:text-orange-500\/65:is(.dark *){color:rgba(249,115,22,.65)}.dark\:text-orange-500\/70:is(.dark *){color:rgba(249,115,22,.7)}.dark\:text-orange-500\/75:is(.dark *){color:rgba(249,115,22,.75)}.dark\:text-orange-500\/80:is(.dark *){color:rgba(249,115,22,.8)}.dark\:text-orange-500\/85:is(.dark *){color:rgba(249,115,22,.85)}.dark\:text-orange-500\/90:is(.dark *){color:rgba(249,115,22,.9)}.dark\:text-orange-500\/95:is(.dark *){color:rgba(249,115,22,.95)}.dark\:text-orange-950:is(.dark *){--tw-text-opacity:1;color:rgb(67 20 7/var(--tw-text-opacity,1))}.dark\:text-orange-950\/0:is(.dark *){color:rgba(67,20,7,0)}.dark\:text-orange-950\/10:is(.dark *){color:rgba(67,20,7,.1)}.dark\:text-orange-950\/100:is(.dark *){color:#431407}.dark\:text-orange-950\/15:is(.dark *){color:rgba(67,20,7,.15)}.dark\:text-orange-950\/20:is(.dark *){color:rgba(67,20,7,.2)}.dark\:text-orange-950\/25:is(.dark *){color:rgba(67,20,7,.25)}.dark\:text-orange-950\/30:is(.dark *){color:rgba(67,20,7,.3)}.dark\:text-orange-950\/35:is(.dark *){color:rgba(67,20,7,.35)}.dark\:text-orange-950\/40:is(.dark *){color:rgba(67,20,7,.4)}.dark\:text-orange-950\/45:is(.dark *){color:rgba(67,20,7,.45)}.dark\:text-orange-950\/5:is(.dark *){color:rgba(67,20,7,.05)}.dark\:text-orange-950\/50:is(.dark *){color:rgba(67,20,7,.5)}.dark\:text-orange-950\/55:is(.dark *){color:rgba(67,20,7,.55)}.dark\:text-orange-950\/60:is(.dark *){color:rgba(67,20,7,.6)}.dark\:text-orange-950\/65:is(.dark *){color:rgba(67,20,7,.65)}.dark\:text-orange-950\/70:is(.dark *){color:rgba(67,20,7,.7)}.dark\:text-orange-950\/75:is(.dark *){color:rgba(67,20,7,.75)}.dark\:text-orange-950\/80:is(.dark *){color:rgba(67,20,7,.8)}.dark\:text-orange-950\/85:is(.dark *){color:rgba(67,20,7,.85)}.dark\:text-orange-950\/90:is(.dark *){color:rgba(67,20,7,.9)}.dark\:text-orange-950\/95:is(.dark *){color:rgba(67,20,7,.95)}.dark\:text-pink-100:is(.dark *){--tw-text-opacity:1;color:rgb(252 231 243/var(--tw-text-opacity,1))}.dark\:text-pink-100\/0:is(.dark *){color:rgba(252,231,243,0)}.dark\:text-pink-100\/10:is(.dark *){color:rgba(252,231,243,.1)}.dark\:text-pink-100\/100:is(.dark *){color:#fce7f3}.dark\:text-pink-100\/15:is(.dark *){color:rgba(252,231,243,.15)}.dark\:text-pink-100\/20:is(.dark *){color:rgba(252,231,243,.2)}.dark\:text-pink-100\/25:is(.dark *){color:rgba(252,231,243,.25)}.dark\:text-pink-100\/30:is(.dark *){color:rgba(252,231,243,.3)}.dark\:text-pink-100\/35:is(.dark *){color:rgba(252,231,243,.35)}.dark\:text-pink-100\/40:is(.dark *){color:rgba(252,231,243,.4)}.dark\:text-pink-100\/45:is(.dark *){color:rgba(252,231,243,.45)}.dark\:text-pink-100\/5:is(.dark *){color:rgba(252,231,243,.05)}.dark\:text-pink-100\/50:is(.dark *){color:rgba(252,231,243,.5)}.dark\:text-pink-100\/55:is(.dark *){color:rgba(252,231,243,.55)}.dark\:text-pink-100\/60:is(.dark *){color:rgba(252,231,243,.6)}.dark\:text-pink-100\/65:is(.dark *){color:rgba(252,231,243,.65)}.dark\:text-pink-100\/70:is(.dark *){color:rgba(252,231,243,.7)}.dark\:text-pink-100\/75:is(.dark *){color:rgba(252,231,243,.75)}.dark\:text-pink-100\/80:is(.dark *){color:rgba(252,231,243,.8)}.dark\:text-pink-100\/85:is(.dark *){color:rgba(252,231,243,.85)}.dark\:text-pink-100\/90:is(.dark *){color:rgba(252,231,243,.9)}.dark\:text-pink-100\/95:is(.dark *){color:rgba(252,231,243,.95)}.dark\:text-pink-200:is(.dark *){--tw-text-opacity:1;color:rgb(251 207 232/var(--tw-text-opacity,1))}.dark\:text-pink-200\/0:is(.dark *){color:rgba(251,207,232,0)}.dark\:text-pink-200\/10:is(.dark *){color:rgba(251,207,232,.1)}.dark\:text-pink-200\/100:is(.dark *){color:#fbcfe8}.dark\:text-pink-200\/15:is(.dark *){color:rgba(251,207,232,.15)}.dark\:text-pink-200\/20:is(.dark *){color:rgba(251,207,232,.2)}.dark\:text-pink-200\/25:is(.dark *){color:rgba(251,207,232,.25)}.dark\:text-pink-200\/30:is(.dark *){color:rgba(251,207,232,.3)}.dark\:text-pink-200\/35:is(.dark *){color:rgba(251,207,232,.35)}.dark\:text-pink-200\/40:is(.dark *){color:rgba(251,207,232,.4)}.dark\:text-pink-200\/45:is(.dark *){color:rgba(251,207,232,.45)}.dark\:text-pink-200\/5:is(.dark *){color:rgba(251,207,232,.05)}.dark\:text-pink-200\/50:is(.dark *){color:rgba(251,207,232,.5)}.dark\:text-pink-200\/55:is(.dark *){color:rgba(251,207,232,.55)}.dark\:text-pink-200\/60:is(.dark *){color:rgba(251,207,232,.6)}.dark\:text-pink-200\/65:is(.dark *){color:rgba(251,207,232,.65)}.dark\:text-pink-200\/70:is(.dark *){color:rgba(251,207,232,.7)}.dark\:text-pink-200\/75:is(.dark *){color:rgba(251,207,232,.75)}.dark\:text-pink-200\/80:is(.dark *){color:rgba(251,207,232,.8)}.dark\:text-pink-200\/85:is(.dark *){color:rgba(251,207,232,.85)}.dark\:text-pink-200\/90:is(.dark *){color:rgba(251,207,232,.9)}.dark\:text-pink-200\/95:is(.dark *){color:rgba(251,207,232,.95)}.dark\:text-pink-50:is(.dark *){--tw-text-opacity:1;color:rgb(253 242 248/var(--tw-text-opacity,1))}.dark\:text-pink-50\/0:is(.dark *){color:rgba(253,242,248,0)}.dark\:text-pink-50\/10:is(.dark *){color:rgba(253,242,248,.1)}.dark\:text-pink-50\/100:is(.dark *){color:#fdf2f8}.dark\:text-pink-50\/15:is(.dark *){color:rgba(253,242,248,.15)}.dark\:text-pink-50\/20:is(.dark *){color:rgba(253,242,248,.2)}.dark\:text-pink-50\/25:is(.dark *){color:rgba(253,242,248,.25)}.dark\:text-pink-50\/30:is(.dark *){color:rgba(253,242,248,.3)}.dark\:text-pink-50\/35:is(.dark *){color:rgba(253,242,248,.35)}.dark\:text-pink-50\/40:is(.dark *){color:rgba(253,242,248,.4)}.dark\:text-pink-50\/45:is(.dark *){color:rgba(253,242,248,.45)}.dark\:text-pink-50\/5:is(.dark *){color:rgba(253,242,248,.05)}.dark\:text-pink-50\/50:is(.dark *){color:rgba(253,242,248,.5)}.dark\:text-pink-50\/55:is(.dark *){color:rgba(253,242,248,.55)}.dark\:text-pink-50\/60:is(.dark *){color:rgba(253,242,248,.6)}.dark\:text-pink-50\/65:is(.dark *){color:rgba(253,242,248,.65)}.dark\:text-pink-50\/70:is(.dark *){color:rgba(253,242,248,.7)}.dark\:text-pink-50\/75:is(.dark *){color:rgba(253,242,248,.75)}.dark\:text-pink-50\/80:is(.dark *){color:rgba(253,242,248,.8)}.dark\:text-pink-50\/85:is(.dark *){color:rgba(253,242,248,.85)}.dark\:text-pink-50\/90:is(.dark *){color:rgba(253,242,248,.9)}.dark\:text-pink-50\/95:is(.dark *){color:rgba(253,242,248,.95)}.dark\:text-pink-500:is(.dark *){--tw-text-opacity:1;color:rgb(236 72 153/var(--tw-text-opacity,1))}.dark\:text-pink-500\/0:is(.dark *){color:rgba(236,72,153,0)}.dark\:text-pink-500\/10:is(.dark *){color:rgba(236,72,153,.1)}.dark\:text-pink-500\/100:is(.dark *){color:#ec4899}.dark\:text-pink-500\/15:is(.dark *){color:rgba(236,72,153,.15)}.dark\:text-pink-500\/20:is(.dark *){color:rgba(236,72,153,.2)}.dark\:text-pink-500\/25:is(.dark *){color:rgba(236,72,153,.25)}.dark\:text-pink-500\/30:is(.dark *){color:rgba(236,72,153,.3)}.dark\:text-pink-500\/35:is(.dark *){color:rgba(236,72,153,.35)}.dark\:text-pink-500\/40:is(.dark *){color:rgba(236,72,153,.4)}.dark\:text-pink-500\/45:is(.dark *){color:rgba(236,72,153,.45)}.dark\:text-pink-500\/5:is(.dark *){color:rgba(236,72,153,.05)}.dark\:text-pink-500\/50:is(.dark *){color:rgba(236,72,153,.5)}.dark\:text-pink-500\/55:is(.dark *){color:rgba(236,72,153,.55)}.dark\:text-pink-500\/60:is(.dark *){color:rgba(236,72,153,.6)}.dark\:text-pink-500\/65:is(.dark *){color:rgba(236,72,153,.65)}.dark\:text-pink-500\/70:is(.dark *){color:rgba(236,72,153,.7)}.dark\:text-pink-500\/75:is(.dark *){color:rgba(236,72,153,.75)}.dark\:text-pink-500\/80:is(.dark *){color:rgba(236,72,153,.8)}.dark\:text-pink-500\/85:is(.dark *){color:rgba(236,72,153,.85)}.dark\:text-pink-500\/90:is(.dark *){color:rgba(236,72,153,.9)}.dark\:text-pink-500\/95:is(.dark *){color:rgba(236,72,153,.95)}.dark\:text-pink-950:is(.dark *){--tw-text-opacity:1;color:rgb(80 7 36/var(--tw-text-opacity,1))}.dark\:text-pink-950\/0:is(.dark *){color:rgba(80,7,36,0)}.dark\:text-pink-950\/10:is(.dark *){color:rgba(80,7,36,.1)}.dark\:text-pink-950\/100:is(.dark *){color:#500724}.dark\:text-pink-950\/15:is(.dark *){color:rgba(80,7,36,.15)}.dark\:text-pink-950\/20:is(.dark *){color:rgba(80,7,36,.2)}.dark\:text-pink-950\/25:is(.dark *){color:rgba(80,7,36,.25)}.dark\:text-pink-950\/30:is(.dark *){color:rgba(80,7,36,.3)}.dark\:text-pink-950\/35:is(.dark *){color:rgba(80,7,36,.35)}.dark\:text-pink-950\/40:is(.dark *){color:rgba(80,7,36,.4)}.dark\:text-pink-950\/45:is(.dark *){color:rgba(80,7,36,.45)}.dark\:text-pink-950\/5:is(.dark *){color:rgba(80,7,36,.05)}.dark\:text-pink-950\/50:is(.dark *){color:rgba(80,7,36,.5)}.dark\:text-pink-950\/55:is(.dark *){color:rgba(80,7,36,.55)}.dark\:text-pink-950\/60:is(.dark *){color:rgba(80,7,36,.6)}.dark\:text-pink-950\/65:is(.dark *){color:rgba(80,7,36,.65)}.dark\:text-pink-950\/70:is(.dark *){color:rgba(80,7,36,.7)}.dark\:text-pink-950\/75:is(.dark *){color:rgba(80,7,36,.75)}.dark\:text-pink-950\/80:is(.dark *){color:rgba(80,7,36,.8)}.dark\:text-pink-950\/85:is(.dark *){color:rgba(80,7,36,.85)}.dark\:text-pink-950\/90:is(.dark *){color:rgba(80,7,36,.9)}.dark\:text-pink-950\/95:is(.dark *){color:rgba(80,7,36,.95)}.dark\:text-purple-100:is(.dark *){--tw-text-opacity:1;color:rgb(243 232 255/var(--tw-text-opacity,1))}.dark\:text-purple-100\/0:is(.dark *){color:rgba(243,232,255,0)}.dark\:text-purple-100\/10:is(.dark *){color:rgba(243,232,255,.1)}.dark\:text-purple-100\/100:is(.dark *){color:#f3e8ff}.dark\:text-purple-100\/15:is(.dark *){color:rgba(243,232,255,.15)}.dark\:text-purple-100\/20:is(.dark *){color:rgba(243,232,255,.2)}.dark\:text-purple-100\/25:is(.dark *){color:rgba(243,232,255,.25)}.dark\:text-purple-100\/30:is(.dark *){color:rgba(243,232,255,.3)}.dark\:text-purple-100\/35:is(.dark *){color:rgba(243,232,255,.35)}.dark\:text-purple-100\/40:is(.dark *){color:rgba(243,232,255,.4)}.dark\:text-purple-100\/45:is(.dark *){color:rgba(243,232,255,.45)}.dark\:text-purple-100\/5:is(.dark *){color:rgba(243,232,255,.05)}.dark\:text-purple-100\/50:is(.dark *){color:rgba(243,232,255,.5)}.dark\:text-purple-100\/55:is(.dark *){color:rgba(243,232,255,.55)}.dark\:text-purple-100\/60:is(.dark *){color:rgba(243,232,255,.6)}.dark\:text-purple-100\/65:is(.dark *){color:rgba(243,232,255,.65)}.dark\:text-purple-100\/70:is(.dark *){color:rgba(243,232,255,.7)}.dark\:text-purple-100\/75:is(.dark *){color:rgba(243,232,255,.75)}.dark\:text-purple-100\/80:is(.dark *){color:rgba(243,232,255,.8)}.dark\:text-purple-100\/85:is(.dark *){color:rgba(243,232,255,.85)}.dark\:text-purple-100\/90:is(.dark *){color:rgba(243,232,255,.9)}.dark\:text-purple-100\/95:is(.dark *){color:rgba(243,232,255,.95)}.dark\:text-purple-200:is(.dark *){--tw-text-opacity:1;color:rgb(233 213 255/var(--tw-text-opacity,1))}.dark\:text-purple-200\/0:is(.dark *){color:rgba(233,213,255,0)}.dark\:text-purple-200\/10:is(.dark *){color:rgba(233,213,255,.1)}.dark\:text-purple-200\/100:is(.dark *){color:#e9d5ff}.dark\:text-purple-200\/15:is(.dark *){color:rgba(233,213,255,.15)}.dark\:text-purple-200\/20:is(.dark *){color:rgba(233,213,255,.2)}.dark\:text-purple-200\/25:is(.dark *){color:rgba(233,213,255,.25)}.dark\:text-purple-200\/30:is(.dark *){color:rgba(233,213,255,.3)}.dark\:text-purple-200\/35:is(.dark *){color:rgba(233,213,255,.35)}.dark\:text-purple-200\/40:is(.dark *){color:rgba(233,213,255,.4)}.dark\:text-purple-200\/45:is(.dark *){color:rgba(233,213,255,.45)}.dark\:text-purple-200\/5:is(.dark *){color:rgba(233,213,255,.05)}.dark\:text-purple-200\/50:is(.dark *){color:rgba(233,213,255,.5)}.dark\:text-purple-200\/55:is(.dark *){color:rgba(233,213,255,.55)}.dark\:text-purple-200\/60:is(.dark *){color:rgba(233,213,255,.6)}.dark\:text-purple-200\/65:is(.dark *){color:rgba(233,213,255,.65)}.dark\:text-purple-200\/70:is(.dark *){color:rgba(233,213,255,.7)}.dark\:text-purple-200\/75:is(.dark *){color:rgba(233,213,255,.75)}.dark\:text-purple-200\/80:is(.dark *){color:rgba(233,213,255,.8)}.dark\:text-purple-200\/85:is(.dark *){color:rgba(233,213,255,.85)}.dark\:text-purple-200\/90:is(.dark *){color:rgba(233,213,255,.9)}.dark\:text-purple-200\/95:is(.dark *){color:rgba(233,213,255,.95)}.dark\:text-purple-300:is(.dark *){--tw-text-opacity:1;color:rgb(216 180 254/var(--tw-text-opacity,1))}.dark\:text-purple-50:is(.dark *){--tw-text-opacity:1;color:rgb(250 245 255/var(--tw-text-opacity,1))}.dark\:text-purple-50\/0:is(.dark *){color:rgba(250,245,255,0)}.dark\:text-purple-50\/10:is(.dark *){color:rgba(250,245,255,.1)}.dark\:text-purple-50\/100:is(.dark *){color:#faf5ff}.dark\:text-purple-50\/15:is(.dark *){color:rgba(250,245,255,.15)}.dark\:text-purple-50\/20:is(.dark *){color:rgba(250,245,255,.2)}.dark\:text-purple-50\/25:is(.dark *){color:rgba(250,245,255,.25)}.dark\:text-purple-50\/30:is(.dark *){color:rgba(250,245,255,.3)}.dark\:text-purple-50\/35:is(.dark *){color:rgba(250,245,255,.35)}.dark\:text-purple-50\/40:is(.dark *){color:rgba(250,245,255,.4)}.dark\:text-purple-50\/45:is(.dark *){color:rgba(250,245,255,.45)}.dark\:text-purple-50\/5:is(.dark *){color:rgba(250,245,255,.05)}.dark\:text-purple-50\/50:is(.dark *){color:rgba(250,245,255,.5)}.dark\:text-purple-50\/55:is(.dark *){color:rgba(250,245,255,.55)}.dark\:text-purple-50\/60:is(.dark *){color:rgba(250,245,255,.6)}.dark\:text-purple-50\/65:is(.dark *){color:rgba(250,245,255,.65)}.dark\:text-purple-50\/70:is(.dark *){color:rgba(250,245,255,.7)}.dark\:text-purple-50\/75:is(.dark *){color:rgba(250,245,255,.75)}.dark\:text-purple-50\/80:is(.dark *){color:rgba(250,245,255,.8)}.dark\:text-purple-50\/85:is(.dark *){color:rgba(250,245,255,.85)}.dark\:text-purple-50\/90:is(.dark *){color:rgba(250,245,255,.9)}.dark\:text-purple-50\/95:is(.dark *){color:rgba(250,245,255,.95)}.dark\:text-purple-500:is(.dark *){--tw-text-opacity:1;color:rgb(168 85 247/var(--tw-text-opacity,1))}.dark\:text-purple-500\/0:is(.dark *){color:rgba(168,85,247,0)}.dark\:text-purple-500\/10:is(.dark *){color:rgba(168,85,247,.1)}.dark\:text-purple-500\/100:is(.dark *){color:#a855f7}.dark\:text-purple-500\/15:is(.dark *){color:rgba(168,85,247,.15)}.dark\:text-purple-500\/20:is(.dark *){color:rgba(168,85,247,.2)}.dark\:text-purple-500\/25:is(.dark *){color:rgba(168,85,247,.25)}.dark\:text-purple-500\/30:is(.dark *){color:rgba(168,85,247,.3)}.dark\:text-purple-500\/35:is(.dark *){color:rgba(168,85,247,.35)}.dark\:text-purple-500\/40:is(.dark *){color:rgba(168,85,247,.4)}.dark\:text-purple-500\/45:is(.dark *){color:rgba(168,85,247,.45)}.dark\:text-purple-500\/5:is(.dark *){color:rgba(168,85,247,.05)}.dark\:text-purple-500\/50:is(.dark *){color:rgba(168,85,247,.5)}.dark\:text-purple-500\/55:is(.dark *){color:rgba(168,85,247,.55)}.dark\:text-purple-500\/60:is(.dark *){color:rgba(168,85,247,.6)}.dark\:text-purple-500\/65:is(.dark *){color:rgba(168,85,247,.65)}.dark\:text-purple-500\/70:is(.dark *){color:rgba(168,85,247,.7)}.dark\:text-purple-500\/75:is(.dark *){color:rgba(168,85,247,.75)}.dark\:text-purple-500\/80:is(.dark *){color:rgba(168,85,247,.8)}.dark\:text-purple-500\/85:is(.dark *){color:rgba(168,85,247,.85)}.dark\:text-purple-500\/90:is(.dark *){color:rgba(168,85,247,.9)}.dark\:text-purple-500\/95:is(.dark *){color:rgba(168,85,247,.95)}.dark\:text-purple-950:is(.dark *){--tw-text-opacity:1;color:rgb(59 7 100/var(--tw-text-opacity,1))}.dark\:text-purple-950\/0:is(.dark *){color:rgba(59,7,100,0)}.dark\:text-purple-950\/10:is(.dark *){color:rgba(59,7,100,.1)}.dark\:text-purple-950\/100:is(.dark *){color:#3b0764}.dark\:text-purple-950\/15:is(.dark *){color:rgba(59,7,100,.15)}.dark\:text-purple-950\/20:is(.dark *){color:rgba(59,7,100,.2)}.dark\:text-purple-950\/25:is(.dark *){color:rgba(59,7,100,.25)}.dark\:text-purple-950\/30:is(.dark *){color:rgba(59,7,100,.3)}.dark\:text-purple-950\/35:is(.dark *){color:rgba(59,7,100,.35)}.dark\:text-purple-950\/40:is(.dark *){color:rgba(59,7,100,.4)}.dark\:text-purple-950\/45:is(.dark *){color:rgba(59,7,100,.45)}.dark\:text-purple-950\/5:is(.dark *){color:rgba(59,7,100,.05)}.dark\:text-purple-950\/50:is(.dark *){color:rgba(59,7,100,.5)}.dark\:text-purple-950\/55:is(.dark *){color:rgba(59,7,100,.55)}.dark\:text-purple-950\/60:is(.dark *){color:rgba(59,7,100,.6)}.dark\:text-purple-950\/65:is(.dark *){color:rgba(59,7,100,.65)}.dark\:text-purple-950\/70:is(.dark *){color:rgba(59,7,100,.7)}.dark\:text-purple-950\/75:is(.dark *){color:rgba(59,7,100,.75)}.dark\:text-purple-950\/80:is(.dark *){color:rgba(59,7,100,.8)}.dark\:text-purple-950\/85:is(.dark *){color:rgba(59,7,100,.85)}.dark\:text-purple-950\/90:is(.dark *){color:rgba(59,7,100,.9)}.dark\:text-purple-950\/95:is(.dark *){color:rgba(59,7,100,.95)}.dark\:text-red-100:is(.dark *){--tw-text-opacity:1;color:rgb(254 226 226/var(--tw-text-opacity,1))}.dark\:text-red-100\/0:is(.dark *){color:hsla(0,93%,94%,0)}.dark\:text-red-100\/10:is(.dark *){color:hsla(0,93%,94%,.1)}.dark\:text-red-100\/100:is(.dark *){color:#fee2e2}.dark\:text-red-100\/15:is(.dark *){color:hsla(0,93%,94%,.15)}.dark\:text-red-100\/20:is(.dark *){color:hsla(0,93%,94%,.2)}.dark\:text-red-100\/25:is(.dark *){color:hsla(0,93%,94%,.25)}.dark\:text-red-100\/30:is(.dark *){color:hsla(0,93%,94%,.3)}.dark\:text-red-100\/35:is(.dark *){color:hsla(0,93%,94%,.35)}.dark\:text-red-100\/40:is(.dark *){color:hsla(0,93%,94%,.4)}.dark\:text-red-100\/45:is(.dark *){color:hsla(0,93%,94%,.45)}.dark\:text-red-100\/5:is(.dark *){color:hsla(0,93%,94%,.05)}.dark\:text-red-100\/50:is(.dark *){color:hsla(0,93%,94%,.5)}.dark\:text-red-100\/55:is(.dark *){color:hsla(0,93%,94%,.55)}.dark\:text-red-100\/60:is(.dark *){color:hsla(0,93%,94%,.6)}.dark\:text-red-100\/65:is(.dark *){color:hsla(0,93%,94%,.65)}.dark\:text-red-100\/70:is(.dark *){color:hsla(0,93%,94%,.7)}.dark\:text-red-100\/75:is(.dark *){color:hsla(0,93%,94%,.75)}.dark\:text-red-100\/80:is(.dark *){color:hsla(0,93%,94%,.8)}.dark\:text-red-100\/85:is(.dark *){color:hsla(0,93%,94%,.85)}.dark\:text-red-100\/90:is(.dark *){color:hsla(0,93%,94%,.9)}.dark\:text-red-100\/95:is(.dark *){color:hsla(0,93%,94%,.95)}.dark\:text-red-200:is(.dark *){--tw-text-opacity:1;color:rgb(254 202 202/var(--tw-text-opacity,1))}.dark\:text-red-200\/0:is(.dark *){color:hsla(0,96%,89%,0)}.dark\:text-red-200\/10:is(.dark *){color:hsla(0,96%,89%,.1)}.dark\:text-red-200\/100:is(.dark *){color:#fecaca}.dark\:text-red-200\/15:is(.dark *){color:hsla(0,96%,89%,.15)}.dark\:text-red-200\/20:is(.dark *){color:hsla(0,96%,89%,.2)}.dark\:text-red-200\/25:is(.dark *){color:hsla(0,96%,89%,.25)}.dark\:text-red-200\/30:is(.dark *){color:hsla(0,96%,89%,.3)}.dark\:text-red-200\/35:is(.dark *){color:hsla(0,96%,89%,.35)}.dark\:text-red-200\/40:is(.dark *){color:hsla(0,96%,89%,.4)}.dark\:text-red-200\/45:is(.dark *){color:hsla(0,96%,89%,.45)}.dark\:text-red-200\/5:is(.dark *){color:hsla(0,96%,89%,.05)}.dark\:text-red-200\/50:is(.dark *){color:hsla(0,96%,89%,.5)}.dark\:text-red-200\/55:is(.dark *){color:hsla(0,96%,89%,.55)}.dark\:text-red-200\/60:is(.dark *){color:hsla(0,96%,89%,.6)}.dark\:text-red-200\/65:is(.dark *){color:hsla(0,96%,89%,.65)}.dark\:text-red-200\/70:is(.dark *){color:hsla(0,96%,89%,.7)}.dark\:text-red-200\/75:is(.dark *){color:hsla(0,96%,89%,.75)}.dark\:text-red-200\/80:is(.dark *){color:hsla(0,96%,89%,.8)}.dark\:text-red-200\/85:is(.dark *){color:hsla(0,96%,89%,.85)}.dark\:text-red-200\/90:is(.dark *){color:hsla(0,96%,89%,.9)}.dark\:text-red-200\/95:is(.dark *){color:hsla(0,96%,89%,.95)}.dark\:text-red-50:is(.dark *){--tw-text-opacity:1;color:rgb(254 242 242/var(--tw-text-opacity,1))}.dark\:text-red-50\/0:is(.dark *){color:hsla(0,86%,97%,0)}.dark\:text-red-50\/10:is(.dark *){color:hsla(0,86%,97%,.1)}.dark\:text-red-50\/100:is(.dark *){color:#fef2f2}.dark\:text-red-50\/15:is(.dark *){color:hsla(0,86%,97%,.15)}.dark\:text-red-50\/20:is(.dark *){color:hsla(0,86%,97%,.2)}.dark\:text-red-50\/25:is(.dark *){color:hsla(0,86%,97%,.25)}.dark\:text-red-50\/30:is(.dark *){color:hsla(0,86%,97%,.3)}.dark\:text-red-50\/35:is(.dark *){color:hsla(0,86%,97%,.35)}.dark\:text-red-50\/40:is(.dark *){color:hsla(0,86%,97%,.4)}.dark\:text-red-50\/45:is(.dark *){color:hsla(0,86%,97%,.45)}.dark\:text-red-50\/5:is(.dark *){color:hsla(0,86%,97%,.05)}.dark\:text-red-50\/50:is(.dark *){color:hsla(0,86%,97%,.5)}.dark\:text-red-50\/55:is(.dark *){color:hsla(0,86%,97%,.55)}.dark\:text-red-50\/60:is(.dark *){color:hsla(0,86%,97%,.6)}.dark\:text-red-50\/65:is(.dark *){color:hsla(0,86%,97%,.65)}.dark\:text-red-50\/70:is(.dark *){color:hsla(0,86%,97%,.7)}.dark\:text-red-50\/75:is(.dark *){color:hsla(0,86%,97%,.75)}.dark\:text-red-50\/80:is(.dark *){color:hsla(0,86%,97%,.8)}.dark\:text-red-50\/85:is(.dark *){color:hsla(0,86%,97%,.85)}.dark\:text-red-50\/90:is(.dark *){color:hsla(0,86%,97%,.9)}.dark\:text-red-50\/95:is(.dark *){color:hsla(0,86%,97%,.95)}.dark\:text-red-500:is(.dark *){--tw-text-opacity:1;color:rgb(239 68 68/var(--tw-text-opacity,1))}.dark\:text-red-500\/0:is(.dark *){color:rgba(239,68,68,0)}.dark\:text-red-500\/10:is(.dark *){color:rgba(239,68,68,.1)}.dark\:text-red-500\/100:is(.dark *){color:#ef4444}.dark\:text-red-500\/15:is(.dark *){color:rgba(239,68,68,.15)}.dark\:text-red-500\/20:is(.dark *){color:rgba(239,68,68,.2)}.dark\:text-red-500\/25:is(.dark *){color:rgba(239,68,68,.25)}.dark\:text-red-500\/30:is(.dark *){color:rgba(239,68,68,.3)}.dark\:text-red-500\/35:is(.dark *){color:rgba(239,68,68,.35)}.dark\:text-red-500\/40:is(.dark *){color:rgba(239,68,68,.4)}.dark\:text-red-500\/45:is(.dark *){color:rgba(239,68,68,.45)}.dark\:text-red-500\/5:is(.dark *){color:rgba(239,68,68,.05)}.dark\:text-red-500\/50:is(.dark *){color:rgba(239,68,68,.5)}.dark\:text-red-500\/55:is(.dark *){color:rgba(239,68,68,.55)}.dark\:text-red-500\/60:is(.dark *){color:rgba(239,68,68,.6)}.dark\:text-red-500\/65:is(.dark *){color:rgba(239,68,68,.65)}.dark\:text-red-500\/70:is(.dark *){color:rgba(239,68,68,.7)}.dark\:text-red-500\/75:is(.dark *){color:rgba(239,68,68,.75)}.dark\:text-red-500\/80:is(.dark *){color:rgba(239,68,68,.8)}.dark\:text-red-500\/85:is(.dark *){color:rgba(239,68,68,.85)}.dark\:text-red-500\/90:is(.dark *){color:rgba(239,68,68,.9)}.dark\:text-red-500\/95:is(.dark *){color:rgba(239,68,68,.95)}.dark\:text-red-950:is(.dark *){--tw-text-opacity:1;color:rgb(69 10 10/var(--tw-text-opacity,1))}.dark\:text-red-950\/0:is(.dark *){color:rgba(69,10,10,0)}.dark\:text-red-950\/10:is(.dark *){color:rgba(69,10,10,.1)}.dark\:text-red-950\/100:is(.dark *){color:#450a0a}.dark\:text-red-950\/15:is(.dark *){color:rgba(69,10,10,.15)}.dark\:text-red-950\/20:is(.dark *){color:rgba(69,10,10,.2)}.dark\:text-red-950\/25:is(.dark *){color:rgba(69,10,10,.25)}.dark\:text-red-950\/30:is(.dark *){color:rgba(69,10,10,.3)}.dark\:text-red-950\/35:is(.dark *){color:rgba(69,10,10,.35)}.dark\:text-red-950\/40:is(.dark *){color:rgba(69,10,10,.4)}.dark\:text-red-950\/45:is(.dark *){color:rgba(69,10,10,.45)}.dark\:text-red-950\/5:is(.dark *){color:rgba(69,10,10,.05)}.dark\:text-red-950\/50:is(.dark *){color:rgba(69,10,10,.5)}.dark\:text-red-950\/55:is(.dark *){color:rgba(69,10,10,.55)}.dark\:text-red-950\/60:is(.dark *){color:rgba(69,10,10,.6)}.dark\:text-red-950\/65:is(.dark *){color:rgba(69,10,10,.65)}.dark\:text-red-950\/70:is(.dark *){color:rgba(69,10,10,.7)}.dark\:text-red-950\/75:is(.dark *){color:rgba(69,10,10,.75)}.dark\:text-red-950\/80:is(.dark *){color:rgba(69,10,10,.8)}.dark\:text-red-950\/85:is(.dark *){color:rgba(69,10,10,.85)}.dark\:text-red-950\/90:is(.dark *){color:rgba(69,10,10,.9)}.dark\:text-red-950\/95:is(.dark *){color:rgba(69,10,10,.95)}.dark\:text-rose-100:is(.dark *){--tw-text-opacity:1;color:rgb(255 228 230/var(--tw-text-opacity,1))}.dark\:text-rose-100\/0:is(.dark *){color:rgba(255,228,230,0)}.dark\:text-rose-100\/10:is(.dark *){color:rgba(255,228,230,.1)}.dark\:text-rose-100\/100:is(.dark *){color:#ffe4e6}.dark\:text-rose-100\/15:is(.dark *){color:rgba(255,228,230,.15)}.dark\:text-rose-100\/20:is(.dark *){color:rgba(255,228,230,.2)}.dark\:text-rose-100\/25:is(.dark *){color:rgba(255,228,230,.25)}.dark\:text-rose-100\/30:is(.dark *){color:rgba(255,228,230,.3)}.dark\:text-rose-100\/35:is(.dark *){color:rgba(255,228,230,.35)}.dark\:text-rose-100\/40:is(.dark *){color:rgba(255,228,230,.4)}.dark\:text-rose-100\/45:is(.dark *){color:rgba(255,228,230,.45)}.dark\:text-rose-100\/5:is(.dark *){color:rgba(255,228,230,.05)}.dark\:text-rose-100\/50:is(.dark *){color:rgba(255,228,230,.5)}.dark\:text-rose-100\/55:is(.dark *){color:rgba(255,228,230,.55)}.dark\:text-rose-100\/60:is(.dark *){color:rgba(255,228,230,.6)}.dark\:text-rose-100\/65:is(.dark *){color:rgba(255,228,230,.65)}.dark\:text-rose-100\/70:is(.dark *){color:rgba(255,228,230,.7)}.dark\:text-rose-100\/75:is(.dark *){color:rgba(255,228,230,.75)}.dark\:text-rose-100\/80:is(.dark *){color:rgba(255,228,230,.8)}.dark\:text-rose-100\/85:is(.dark *){color:rgba(255,228,230,.85)}.dark\:text-rose-100\/90:is(.dark *){color:rgba(255,228,230,.9)}.dark\:text-rose-100\/95:is(.dark *){color:rgba(255,228,230,.95)}.dark\:text-rose-200:is(.dark *){--tw-text-opacity:1;color:rgb(254 205 211/var(--tw-text-opacity,1))}.dark\:text-rose-200\/0:is(.dark *){color:rgba(254,205,211,0)}.dark\:text-rose-200\/10:is(.dark *){color:rgba(254,205,211,.1)}.dark\:text-rose-200\/100:is(.dark *){color:#fecdd3}.dark\:text-rose-200\/15:is(.dark *){color:rgba(254,205,211,.15)}.dark\:text-rose-200\/20:is(.dark *){color:rgba(254,205,211,.2)}.dark\:text-rose-200\/25:is(.dark *){color:rgba(254,205,211,.25)}.dark\:text-rose-200\/30:is(.dark *){color:rgba(254,205,211,.3)}.dark\:text-rose-200\/35:is(.dark *){color:rgba(254,205,211,.35)}.dark\:text-rose-200\/40:is(.dark *){color:rgba(254,205,211,.4)}.dark\:text-rose-200\/45:is(.dark *){color:rgba(254,205,211,.45)}.dark\:text-rose-200\/5:is(.dark *){color:rgba(254,205,211,.05)}.dark\:text-rose-200\/50:is(.dark *){color:rgba(254,205,211,.5)}.dark\:text-rose-200\/55:is(.dark *){color:rgba(254,205,211,.55)}.dark\:text-rose-200\/60:is(.dark *){color:rgba(254,205,211,.6)}.dark\:text-rose-200\/65:is(.dark *){color:rgba(254,205,211,.65)}.dark\:text-rose-200\/70:is(.dark *){color:rgba(254,205,211,.7)}.dark\:text-rose-200\/75:is(.dark *){color:rgba(254,205,211,.75)}.dark\:text-rose-200\/80:is(.dark *){color:rgba(254,205,211,.8)}.dark\:text-rose-200\/85:is(.dark *){color:rgba(254,205,211,.85)}.dark\:text-rose-200\/90:is(.dark *){color:rgba(254,205,211,.9)}.dark\:text-rose-200\/95:is(.dark *){color:rgba(254,205,211,.95)}.dark\:text-rose-50:is(.dark *){--tw-text-opacity:1;color:rgb(255 241 242/var(--tw-text-opacity,1))}.dark\:text-rose-50\/0:is(.dark *){color:rgba(255,241,242,0)}.dark\:text-rose-50\/10:is(.dark *){color:rgba(255,241,242,.1)}.dark\:text-rose-50\/100:is(.dark *){color:#fff1f2}.dark\:text-rose-50\/15:is(.dark *){color:rgba(255,241,242,.15)}.dark\:text-rose-50\/20:is(.dark *){color:rgba(255,241,242,.2)}.dark\:text-rose-50\/25:is(.dark *){color:rgba(255,241,242,.25)}.dark\:text-rose-50\/30:is(.dark *){color:rgba(255,241,242,.3)}.dark\:text-rose-50\/35:is(.dark *){color:rgba(255,241,242,.35)}.dark\:text-rose-50\/40:is(.dark *){color:rgba(255,241,242,.4)}.dark\:text-rose-50\/45:is(.dark *){color:rgba(255,241,242,.45)}.dark\:text-rose-50\/5:is(.dark *){color:rgba(255,241,242,.05)}.dark\:text-rose-50\/50:is(.dark *){color:rgba(255,241,242,.5)}.dark\:text-rose-50\/55:is(.dark *){color:rgba(255,241,242,.55)}.dark\:text-rose-50\/60:is(.dark *){color:rgba(255,241,242,.6)}.dark\:text-rose-50\/65:is(.dark *){color:rgba(255,241,242,.65)}.dark\:text-rose-50\/70:is(.dark *){color:rgba(255,241,242,.7)}.dark\:text-rose-50\/75:is(.dark *){color:rgba(255,241,242,.75)}.dark\:text-rose-50\/80:is(.dark *){color:rgba(255,241,242,.8)}.dark\:text-rose-50\/85:is(.dark *){color:rgba(255,241,242,.85)}.dark\:text-rose-50\/90:is(.dark *){color:rgba(255,241,242,.9)}.dark\:text-rose-50\/95:is(.dark *){color:rgba(255,241,242,.95)}.dark\:text-rose-500:is(.dark *){--tw-text-opacity:1;color:rgb(244 63 94/var(--tw-text-opacity,1))}.dark\:text-rose-500\/0:is(.dark *){color:rgba(244,63,94,0)}.dark\:text-rose-500\/10:is(.dark *){color:rgba(244,63,94,.1)}.dark\:text-rose-500\/100:is(.dark *){color:#f43f5e}.dark\:text-rose-500\/15:is(.dark *){color:rgba(244,63,94,.15)}.dark\:text-rose-500\/20:is(.dark *){color:rgba(244,63,94,.2)}.dark\:text-rose-500\/25:is(.dark *){color:rgba(244,63,94,.25)}.dark\:text-rose-500\/30:is(.dark *){color:rgba(244,63,94,.3)}.dark\:text-rose-500\/35:is(.dark *){color:rgba(244,63,94,.35)}.dark\:text-rose-500\/40:is(.dark *){color:rgba(244,63,94,.4)}.dark\:text-rose-500\/45:is(.dark *){color:rgba(244,63,94,.45)}.dark\:text-rose-500\/5:is(.dark *){color:rgba(244,63,94,.05)}.dark\:text-rose-500\/50:is(.dark *){color:rgba(244,63,94,.5)}.dark\:text-rose-500\/55:is(.dark *){color:rgba(244,63,94,.55)}.dark\:text-rose-500\/60:is(.dark *){color:rgba(244,63,94,.6)}.dark\:text-rose-500\/65:is(.dark *){color:rgba(244,63,94,.65)}.dark\:text-rose-500\/70:is(.dark *){color:rgba(244,63,94,.7)}.dark\:text-rose-500\/75:is(.dark *){color:rgba(244,63,94,.75)}.dark\:text-rose-500\/80:is(.dark *){color:rgba(244,63,94,.8)}.dark\:text-rose-500\/85:is(.dark *){color:rgba(244,63,94,.85)}.dark\:text-rose-500\/90:is(.dark *){color:rgba(244,63,94,.9)}.dark\:text-rose-500\/95:is(.dark *){color:rgba(244,63,94,.95)}.dark\:text-rose-950:is(.dark *){--tw-text-opacity:1;color:rgb(76 5 25/var(--tw-text-opacity,1))}.dark\:text-rose-950\/0:is(.dark *){color:rgba(76,5,25,0)}.dark\:text-rose-950\/10:is(.dark *){color:rgba(76,5,25,.1)}.dark\:text-rose-950\/100:is(.dark *){color:#4c0519}.dark\:text-rose-950\/15:is(.dark *){color:rgba(76,5,25,.15)}.dark\:text-rose-950\/20:is(.dark *){color:rgba(76,5,25,.2)}.dark\:text-rose-950\/25:is(.dark *){color:rgba(76,5,25,.25)}.dark\:text-rose-950\/30:is(.dark *){color:rgba(76,5,25,.3)}.dark\:text-rose-950\/35:is(.dark *){color:rgba(76,5,25,.35)}.dark\:text-rose-950\/40:is(.dark *){color:rgba(76,5,25,.4)}.dark\:text-rose-950\/45:is(.dark *){color:rgba(76,5,25,.45)}.dark\:text-rose-950\/5:is(.dark *){color:rgba(76,5,25,.05)}.dark\:text-rose-950\/50:is(.dark *){color:rgba(76,5,25,.5)}.dark\:text-rose-950\/55:is(.dark *){color:rgba(76,5,25,.55)}.dark\:text-rose-950\/60:is(.dark *){color:rgba(76,5,25,.6)}.dark\:text-rose-950\/65:is(.dark *){color:rgba(76,5,25,.65)}.dark\:text-rose-950\/70:is(.dark *){color:rgba(76,5,25,.7)}.dark\:text-rose-950\/75:is(.dark *){color:rgba(76,5,25,.75)}.dark\:text-rose-950\/80:is(.dark *){color:rgba(76,5,25,.8)}.dark\:text-rose-950\/85:is(.dark *){color:rgba(76,5,25,.85)}.dark\:text-rose-950\/90:is(.dark *){color:rgba(76,5,25,.9)}.dark\:text-rose-950\/95:is(.dark *){color:rgba(76,5,25,.95)}.dark\:text-sky-100:is(.dark *){--tw-text-opacity:1;color:rgb(224 242 254/var(--tw-text-opacity,1))}.dark\:text-sky-100\/0:is(.dark *){color:rgba(224,242,254,0)}.dark\:text-sky-100\/10:is(.dark *){color:rgba(224,242,254,.1)}.dark\:text-sky-100\/100:is(.dark *){color:#e0f2fe}.dark\:text-sky-100\/15:is(.dark *){color:rgba(224,242,254,.15)}.dark\:text-sky-100\/20:is(.dark *){color:rgba(224,242,254,.2)}.dark\:text-sky-100\/25:is(.dark *){color:rgba(224,242,254,.25)}.dark\:text-sky-100\/30:is(.dark *){color:rgba(224,242,254,.3)}.dark\:text-sky-100\/35:is(.dark *){color:rgba(224,242,254,.35)}.dark\:text-sky-100\/40:is(.dark *){color:rgba(224,242,254,.4)}.dark\:text-sky-100\/45:is(.dark *){color:rgba(224,242,254,.45)}.dark\:text-sky-100\/5:is(.dark *){color:rgba(224,242,254,.05)}.dark\:text-sky-100\/50:is(.dark *){color:rgba(224,242,254,.5)}.dark\:text-sky-100\/55:is(.dark *){color:rgba(224,242,254,.55)}.dark\:text-sky-100\/60:is(.dark *){color:rgba(224,242,254,.6)}.dark\:text-sky-100\/65:is(.dark *){color:rgba(224,242,254,.65)}.dark\:text-sky-100\/70:is(.dark *){color:rgba(224,242,254,.7)}.dark\:text-sky-100\/75:is(.dark *){color:rgba(224,242,254,.75)}.dark\:text-sky-100\/80:is(.dark *){color:rgba(224,242,254,.8)}.dark\:text-sky-100\/85:is(.dark *){color:rgba(224,242,254,.85)}.dark\:text-sky-100\/90:is(.dark *){color:rgba(224,242,254,.9)}.dark\:text-sky-100\/95:is(.dark *){color:rgba(224,242,254,.95)}.dark\:text-sky-200:is(.dark *){--tw-text-opacity:1;color:rgb(186 230 253/var(--tw-text-opacity,1))}.dark\:text-sky-200\/0:is(.dark *){color:rgba(186,230,253,0)}.dark\:text-sky-200\/10:is(.dark *){color:rgba(186,230,253,.1)}.dark\:text-sky-200\/100:is(.dark *){color:#bae6fd}.dark\:text-sky-200\/15:is(.dark *){color:rgba(186,230,253,.15)}.dark\:text-sky-200\/20:is(.dark *){color:rgba(186,230,253,.2)}.dark\:text-sky-200\/25:is(.dark *){color:rgba(186,230,253,.25)}.dark\:text-sky-200\/30:is(.dark *){color:rgba(186,230,253,.3)}.dark\:text-sky-200\/35:is(.dark *){color:rgba(186,230,253,.35)}.dark\:text-sky-200\/40:is(.dark *){color:rgba(186,230,253,.4)}.dark\:text-sky-200\/45:is(.dark *){color:rgba(186,230,253,.45)}.dark\:text-sky-200\/5:is(.dark *){color:rgba(186,230,253,.05)}.dark\:text-sky-200\/50:is(.dark *){color:rgba(186,230,253,.5)}.dark\:text-sky-200\/55:is(.dark *){color:rgba(186,230,253,.55)}.dark\:text-sky-200\/60:is(.dark *){color:rgba(186,230,253,.6)}.dark\:text-sky-200\/65:is(.dark *){color:rgba(186,230,253,.65)}.dark\:text-sky-200\/70:is(.dark *){color:rgba(186,230,253,.7)}.dark\:text-sky-200\/75:is(.dark *){color:rgba(186,230,253,.75)}.dark\:text-sky-200\/80:is(.dark *){color:rgba(186,230,253,.8)}.dark\:text-sky-200\/85:is(.dark *){color:rgba(186,230,253,.85)}.dark\:text-sky-200\/90:is(.dark *){color:rgba(186,230,253,.9)}.dark\:text-sky-200\/95:is(.dark *){color:rgba(186,230,253,.95)}.dark\:text-sky-50:is(.dark *){--tw-text-opacity:1;color:rgb(240 249 255/var(--tw-text-opacity,1))}.dark\:text-sky-50\/0:is(.dark *){color:rgba(240,249,255,0)}.dark\:text-sky-50\/10:is(.dark *){color:rgba(240,249,255,.1)}.dark\:text-sky-50\/100:is(.dark *){color:#f0f9ff}.dark\:text-sky-50\/15:is(.dark *){color:rgba(240,249,255,.15)}.dark\:text-sky-50\/20:is(.dark *){color:rgba(240,249,255,.2)}.dark\:text-sky-50\/25:is(.dark *){color:rgba(240,249,255,.25)}.dark\:text-sky-50\/30:is(.dark *){color:rgba(240,249,255,.3)}.dark\:text-sky-50\/35:is(.dark *){color:rgba(240,249,255,.35)}.dark\:text-sky-50\/40:is(.dark *){color:rgba(240,249,255,.4)}.dark\:text-sky-50\/45:is(.dark *){color:rgba(240,249,255,.45)}.dark\:text-sky-50\/5:is(.dark *){color:rgba(240,249,255,.05)}.dark\:text-sky-50\/50:is(.dark *){color:rgba(240,249,255,.5)}.dark\:text-sky-50\/55:is(.dark *){color:rgba(240,249,255,.55)}.dark\:text-sky-50\/60:is(.dark *){color:rgba(240,249,255,.6)}.dark\:text-sky-50\/65:is(.dark *){color:rgba(240,249,255,.65)}.dark\:text-sky-50\/70:is(.dark *){color:rgba(240,249,255,.7)}.dark\:text-sky-50\/75:is(.dark *){color:rgba(240,249,255,.75)}.dark\:text-sky-50\/80:is(.dark *){color:rgba(240,249,255,.8)}.dark\:text-sky-50\/85:is(.dark *){color:rgba(240,249,255,.85)}.dark\:text-sky-50\/90:is(.dark *){color:rgba(240,249,255,.9)}.dark\:text-sky-50\/95:is(.dark *){color:rgba(240,249,255,.95)}.dark\:text-sky-500:is(.dark *){--tw-text-opacity:1;color:rgb(14 165 233/var(--tw-text-opacity,1))}.dark\:text-sky-500\/0:is(.dark *){color:rgba(14,165,233,0)}.dark\:text-sky-500\/10:is(.dark *){color:rgba(14,165,233,.1)}.dark\:text-sky-500\/100:is(.dark *){color:#0ea5e9}.dark\:text-sky-500\/15:is(.dark *){color:rgba(14,165,233,.15)}.dark\:text-sky-500\/20:is(.dark *){color:rgba(14,165,233,.2)}.dark\:text-sky-500\/25:is(.dark *){color:rgba(14,165,233,.25)}.dark\:text-sky-500\/30:is(.dark *){color:rgba(14,165,233,.3)}.dark\:text-sky-500\/35:is(.dark *){color:rgba(14,165,233,.35)}.dark\:text-sky-500\/40:is(.dark *){color:rgba(14,165,233,.4)}.dark\:text-sky-500\/45:is(.dark *){color:rgba(14,165,233,.45)}.dark\:text-sky-500\/5:is(.dark *){color:rgba(14,165,233,.05)}.dark\:text-sky-500\/50:is(.dark *){color:rgba(14,165,233,.5)}.dark\:text-sky-500\/55:is(.dark *){color:rgba(14,165,233,.55)}.dark\:text-sky-500\/60:is(.dark *){color:rgba(14,165,233,.6)}.dark\:text-sky-500\/65:is(.dark *){color:rgba(14,165,233,.65)}.dark\:text-sky-500\/70:is(.dark *){color:rgba(14,165,233,.7)}.dark\:text-sky-500\/75:is(.dark *){color:rgba(14,165,233,.75)}.dark\:text-sky-500\/80:is(.dark *){color:rgba(14,165,233,.8)}.dark\:text-sky-500\/85:is(.dark *){color:rgba(14,165,233,.85)}.dark\:text-sky-500\/90:is(.dark *){color:rgba(14,165,233,.9)}.dark\:text-sky-500\/95:is(.dark *){color:rgba(14,165,233,.95)}.dark\:text-sky-950:is(.dark *){--tw-text-opacity:1;color:rgb(8 47 73/var(--tw-text-opacity,1))}.dark\:text-sky-950\/0:is(.dark *){color:rgba(8,47,73,0)}.dark\:text-sky-950\/10:is(.dark *){color:rgba(8,47,73,.1)}.dark\:text-sky-950\/100:is(.dark *){color:#082f49}.dark\:text-sky-950\/15:is(.dark *){color:rgba(8,47,73,.15)}.dark\:text-sky-950\/20:is(.dark *){color:rgba(8,47,73,.2)}.dark\:text-sky-950\/25:is(.dark *){color:rgba(8,47,73,.25)}.dark\:text-sky-950\/30:is(.dark *){color:rgba(8,47,73,.3)}.dark\:text-sky-950\/35:is(.dark *){color:rgba(8,47,73,.35)}.dark\:text-sky-950\/40:is(.dark *){color:rgba(8,47,73,.4)}.dark\:text-sky-950\/45:is(.dark *){color:rgba(8,47,73,.45)}.dark\:text-sky-950\/5:is(.dark *){color:rgba(8,47,73,.05)}.dark\:text-sky-950\/50:is(.dark *){color:rgba(8,47,73,.5)}.dark\:text-sky-950\/55:is(.dark *){color:rgba(8,47,73,.55)}.dark\:text-sky-950\/60:is(.dark *){color:rgba(8,47,73,.6)}.dark\:text-sky-950\/65:is(.dark *){color:rgba(8,47,73,.65)}.dark\:text-sky-950\/70:is(.dark *){color:rgba(8,47,73,.7)}.dark\:text-sky-950\/75:is(.dark *){color:rgba(8,47,73,.75)}.dark\:text-sky-950\/80:is(.dark *){color:rgba(8,47,73,.8)}.dark\:text-sky-950\/85:is(.dark *){color:rgba(8,47,73,.85)}.dark\:text-sky-950\/90:is(.dark *){color:rgba(8,47,73,.9)}.dark\:text-sky-950\/95:is(.dark *){color:rgba(8,47,73,.95)}.dark\:text-slate-100:is(.dark *){--tw-text-opacity:1;color:rgb(241 245 249/var(--tw-text-opacity,1))}.dark\:text-slate-100\/0:is(.dark *){color:rgba(241,245,249,0)}.dark\:text-slate-100\/10:is(.dark *){color:rgba(241,245,249,.1)}.dark\:text-slate-100\/100:is(.dark *){color:#f1f5f9}.dark\:text-slate-100\/15:is(.dark *){color:rgba(241,245,249,.15)}.dark\:text-slate-100\/20:is(.dark *){color:rgba(241,245,249,.2)}.dark\:text-slate-100\/25:is(.dark *){color:rgba(241,245,249,.25)}.dark\:text-slate-100\/30:is(.dark *){color:rgba(241,245,249,.3)}.dark\:text-slate-100\/35:is(.dark *){color:rgba(241,245,249,.35)}.dark\:text-slate-100\/40:is(.dark *){color:rgba(241,245,249,.4)}.dark\:text-slate-100\/45:is(.dark *){color:rgba(241,245,249,.45)}.dark\:text-slate-100\/5:is(.dark *){color:rgba(241,245,249,.05)}.dark\:text-slate-100\/50:is(.dark *){color:rgba(241,245,249,.5)}.dark\:text-slate-100\/55:is(.dark *){color:rgba(241,245,249,.55)}.dark\:text-slate-100\/60:is(.dark *){color:rgba(241,245,249,.6)}.dark\:text-slate-100\/65:is(.dark *){color:rgba(241,245,249,.65)}.dark\:text-slate-100\/70:is(.dark *){color:rgba(241,245,249,.7)}.dark\:text-slate-100\/75:is(.dark *){color:rgba(241,245,249,.75)}.dark\:text-slate-100\/80:is(.dark *){color:rgba(241,245,249,.8)}.dark\:text-slate-100\/85:is(.dark *){color:rgba(241,245,249,.85)}.dark\:text-slate-100\/90:is(.dark *){color:rgba(241,245,249,.9)}.dark\:text-slate-100\/95:is(.dark *){color:rgba(241,245,249,.95)}.dark\:text-slate-200:is(.dark *){--tw-text-opacity:1;color:rgb(226 232 240/var(--tw-text-opacity,1))}.dark\:text-slate-200\/0:is(.dark *){color:rgba(226,232,240,0)}.dark\:text-slate-200\/10:is(.dark *){color:rgba(226,232,240,.1)}.dark\:text-slate-200\/100:is(.dark *){color:#e2e8f0}.dark\:text-slate-200\/15:is(.dark *){color:rgba(226,232,240,.15)}.dark\:text-slate-200\/20:is(.dark *){color:rgba(226,232,240,.2)}.dark\:text-slate-200\/25:is(.dark *){color:rgba(226,232,240,.25)}.dark\:text-slate-200\/30:is(.dark *){color:rgba(226,232,240,.3)}.dark\:text-slate-200\/35:is(.dark *){color:rgba(226,232,240,.35)}.dark\:text-slate-200\/40:is(.dark *){color:rgba(226,232,240,.4)}.dark\:text-slate-200\/45:is(.dark *){color:rgba(226,232,240,.45)}.dark\:text-slate-200\/5:is(.dark *){color:rgba(226,232,240,.05)}.dark\:text-slate-200\/50:is(.dark *){color:rgba(226,232,240,.5)}.dark\:text-slate-200\/55:is(.dark *){color:rgba(226,232,240,.55)}.dark\:text-slate-200\/60:is(.dark *){color:rgba(226,232,240,.6)}.dark\:text-slate-200\/65:is(.dark *){color:rgba(226,232,240,.65)}.dark\:text-slate-200\/70:is(.dark *){color:rgba(226,232,240,.7)}.dark\:text-slate-200\/75:is(.dark *){color:rgba(226,232,240,.75)}.dark\:text-slate-200\/80:is(.dark *){color:rgba(226,232,240,.8)}.dark\:text-slate-200\/85:is(.dark *){color:rgba(226,232,240,.85)}.dark\:text-slate-200\/90:is(.dark *){color:rgba(226,232,240,.9)}.dark\:text-slate-200\/95:is(.dark *){color:rgba(226,232,240,.95)}.dark\:text-slate-50:is(.dark *){--tw-text-opacity:1;color:rgb(248 250 252/var(--tw-text-opacity,1))}.dark\:text-slate-50\/0:is(.dark *){color:rgba(248,250,252,0)}.dark\:text-slate-50\/10:is(.dark *){color:rgba(248,250,252,.1)}.dark\:text-slate-50\/100:is(.dark *){color:#f8fafc}.dark\:text-slate-50\/15:is(.dark *){color:rgba(248,250,252,.15)}.dark\:text-slate-50\/20:is(.dark *){color:rgba(248,250,252,.2)}.dark\:text-slate-50\/25:is(.dark *){color:rgba(248,250,252,.25)}.dark\:text-slate-50\/30:is(.dark *){color:rgba(248,250,252,.3)}.dark\:text-slate-50\/35:is(.dark *){color:rgba(248,250,252,.35)}.dark\:text-slate-50\/40:is(.dark *){color:rgba(248,250,252,.4)}.dark\:text-slate-50\/45:is(.dark *){color:rgba(248,250,252,.45)}.dark\:text-slate-50\/5:is(.dark *){color:rgba(248,250,252,.05)}.dark\:text-slate-50\/50:is(.dark *){color:rgba(248,250,252,.5)}.dark\:text-slate-50\/55:is(.dark *){color:rgba(248,250,252,.55)}.dark\:text-slate-50\/60:is(.dark *){color:rgba(248,250,252,.6)}.dark\:text-slate-50\/65:is(.dark *){color:rgba(248,250,252,.65)}.dark\:text-slate-50\/70:is(.dark *){color:rgba(248,250,252,.7)}.dark\:text-slate-50\/75:is(.dark *){color:rgba(248,250,252,.75)}.dark\:text-slate-50\/80:is(.dark *){color:rgba(248,250,252,.8)}.dark\:text-slate-50\/85:is(.dark *){color:rgba(248,250,252,.85)}.dark\:text-slate-50\/90:is(.dark *){color:rgba(248,250,252,.9)}.dark\:text-slate-50\/95:is(.dark *){color:rgba(248,250,252,.95)}.dark\:text-slate-500:is(.dark *){--tw-text-opacity:1;color:rgb(100 116 139/var(--tw-text-opacity,1))}.dark\:text-slate-500\/0:is(.dark *){color:rgba(100,116,139,0)}.dark\:text-slate-500\/10:is(.dark *){color:rgba(100,116,139,.1)}.dark\:text-slate-500\/100:is(.dark *){color:#64748b}.dark\:text-slate-500\/15:is(.dark *){color:rgba(100,116,139,.15)}.dark\:text-slate-500\/20:is(.dark *){color:rgba(100,116,139,.2)}.dark\:text-slate-500\/25:is(.dark *){color:rgba(100,116,139,.25)}.dark\:text-slate-500\/30:is(.dark *){color:rgba(100,116,139,.3)}.dark\:text-slate-500\/35:is(.dark *){color:rgba(100,116,139,.35)}.dark\:text-slate-500\/40:is(.dark *){color:rgba(100,116,139,.4)}.dark\:text-slate-500\/45:is(.dark *){color:rgba(100,116,139,.45)}.dark\:text-slate-500\/5:is(.dark *){color:rgba(100,116,139,.05)}.dark\:text-slate-500\/50:is(.dark *){color:rgba(100,116,139,.5)}.dark\:text-slate-500\/55:is(.dark *){color:rgba(100,116,139,.55)}.dark\:text-slate-500\/60:is(.dark *){color:rgba(100,116,139,.6)}.dark\:text-slate-500\/65:is(.dark *){color:rgba(100,116,139,.65)}.dark\:text-slate-500\/70:is(.dark *){color:rgba(100,116,139,.7)}.dark\:text-slate-500\/75:is(.dark *){color:rgba(100,116,139,.75)}.dark\:text-slate-500\/80:is(.dark *){color:rgba(100,116,139,.8)}.dark\:text-slate-500\/85:is(.dark *){color:rgba(100,116,139,.85)}.dark\:text-slate-500\/90:is(.dark *){color:rgba(100,116,139,.9)}.dark\:text-slate-500\/95:is(.dark *){color:rgba(100,116,139,.95)}.dark\:text-slate-950:is(.dark *){--tw-text-opacity:1;color:rgb(2 6 23/var(--tw-text-opacity,1))}.dark\:text-slate-950\/0:is(.dark *){color:rgba(2,6,23,0)}.dark\:text-slate-950\/10:is(.dark *){color:rgba(2,6,23,.1)}.dark\:text-slate-950\/100:is(.dark *){color:#020617}.dark\:text-slate-950\/15:is(.dark *){color:rgba(2,6,23,.15)}.dark\:text-slate-950\/20:is(.dark *){color:rgba(2,6,23,.2)}.dark\:text-slate-950\/25:is(.dark *){color:rgba(2,6,23,.25)}.dark\:text-slate-950\/30:is(.dark *){color:rgba(2,6,23,.3)}.dark\:text-slate-950\/35:is(.dark *){color:rgba(2,6,23,.35)}.dark\:text-slate-950\/40:is(.dark *){color:rgba(2,6,23,.4)}.dark\:text-slate-950\/45:is(.dark *){color:rgba(2,6,23,.45)}.dark\:text-slate-950\/5:is(.dark *){color:rgba(2,6,23,.05)}.dark\:text-slate-950\/50:is(.dark *){color:rgba(2,6,23,.5)}.dark\:text-slate-950\/55:is(.dark *){color:rgba(2,6,23,.55)}.dark\:text-slate-950\/60:is(.dark *){color:rgba(2,6,23,.6)}.dark\:text-slate-950\/65:is(.dark *){color:rgba(2,6,23,.65)}.dark\:text-slate-950\/70:is(.dark *){color:rgba(2,6,23,.7)}.dark\:text-slate-950\/75:is(.dark *){color:rgba(2,6,23,.75)}.dark\:text-slate-950\/80:is(.dark *){color:rgba(2,6,23,.8)}.dark\:text-slate-950\/85:is(.dark *){color:rgba(2,6,23,.85)}.dark\:text-slate-950\/90:is(.dark *){color:rgba(2,6,23,.9)}.dark\:text-slate-950\/95:is(.dark *){color:rgba(2,6,23,.95)}.dark\:text-stone-100:is(.dark *){--tw-text-opacity:1;color:rgb(245 245 244/var(--tw-text-opacity,1))}.dark\:text-stone-100\/0:is(.dark *){color:hsla(60,5%,96%,0)}.dark\:text-stone-100\/10:is(.dark *){color:hsla(60,5%,96%,.1)}.dark\:text-stone-100\/100:is(.dark *){color:#f5f5f4}.dark\:text-stone-100\/15:is(.dark *){color:hsla(60,5%,96%,.15)}.dark\:text-stone-100\/20:is(.dark *){color:hsla(60,5%,96%,.2)}.dark\:text-stone-100\/25:is(.dark *){color:hsla(60,5%,96%,.25)}.dark\:text-stone-100\/30:is(.dark *){color:hsla(60,5%,96%,.3)}.dark\:text-stone-100\/35:is(.dark *){color:hsla(60,5%,96%,.35)}.dark\:text-stone-100\/40:is(.dark *){color:hsla(60,5%,96%,.4)}.dark\:text-stone-100\/45:is(.dark *){color:hsla(60,5%,96%,.45)}.dark\:text-stone-100\/5:is(.dark *){color:hsla(60,5%,96%,.05)}.dark\:text-stone-100\/50:is(.dark *){color:hsla(60,5%,96%,.5)}.dark\:text-stone-100\/55:is(.dark *){color:hsla(60,5%,96%,.55)}.dark\:text-stone-100\/60:is(.dark *){color:hsla(60,5%,96%,.6)}.dark\:text-stone-100\/65:is(.dark *){color:hsla(60,5%,96%,.65)}.dark\:text-stone-100\/70:is(.dark *){color:hsla(60,5%,96%,.7)}.dark\:text-stone-100\/75:is(.dark *){color:hsla(60,5%,96%,.75)}.dark\:text-stone-100\/80:is(.dark *){color:hsla(60,5%,96%,.8)}.dark\:text-stone-100\/85:is(.dark *){color:hsla(60,5%,96%,.85)}.dark\:text-stone-100\/90:is(.dark *){color:hsla(60,5%,96%,.9)}.dark\:text-stone-100\/95:is(.dark *){color:hsla(60,5%,96%,.95)}.dark\:text-stone-200:is(.dark *){--tw-text-opacity:1;color:rgb(231 229 228/var(--tw-text-opacity,1))}.dark\:text-stone-200\/0:is(.dark *){color:hsla(20,6%,90%,0)}.dark\:text-stone-200\/10:is(.dark *){color:hsla(20,6%,90%,.1)}.dark\:text-stone-200\/100:is(.dark *){color:#e7e5e4}.dark\:text-stone-200\/15:is(.dark *){color:hsla(20,6%,90%,.15)}.dark\:text-stone-200\/20:is(.dark *){color:hsla(20,6%,90%,.2)}.dark\:text-stone-200\/25:is(.dark *){color:hsla(20,6%,90%,.25)}.dark\:text-stone-200\/30:is(.dark *){color:hsla(20,6%,90%,.3)}.dark\:text-stone-200\/35:is(.dark *){color:hsla(20,6%,90%,.35)}.dark\:text-stone-200\/40:is(.dark *){color:hsla(20,6%,90%,.4)}.dark\:text-stone-200\/45:is(.dark *){color:hsla(20,6%,90%,.45)}.dark\:text-stone-200\/5:is(.dark *){color:hsla(20,6%,90%,.05)}.dark\:text-stone-200\/50:is(.dark *){color:hsla(20,6%,90%,.5)}.dark\:text-stone-200\/55:is(.dark *){color:hsla(20,6%,90%,.55)}.dark\:text-stone-200\/60:is(.dark *){color:hsla(20,6%,90%,.6)}.dark\:text-stone-200\/65:is(.dark *){color:hsla(20,6%,90%,.65)}.dark\:text-stone-200\/70:is(.dark *){color:hsla(20,6%,90%,.7)}.dark\:text-stone-200\/75:is(.dark *){color:hsla(20,6%,90%,.75)}.dark\:text-stone-200\/80:is(.dark *){color:hsla(20,6%,90%,.8)}.dark\:text-stone-200\/85:is(.dark *){color:hsla(20,6%,90%,.85)}.dark\:text-stone-200\/90:is(.dark *){color:hsla(20,6%,90%,.9)}.dark\:text-stone-200\/95:is(.dark *){color:hsla(20,6%,90%,.95)}.dark\:text-stone-300:is(.dark *){--tw-text-opacity:1;color:rgb(214 211 209/var(--tw-text-opacity,1))}.dark\:text-stone-50:is(.dark *){--tw-text-opacity:1;color:rgb(250 250 249/var(--tw-text-opacity,1))}.dark\:text-stone-50\/0:is(.dark *){color:hsla(60,9%,98%,0)}.dark\:text-stone-50\/10:is(.dark *){color:hsla(60,9%,98%,.1)}.dark\:text-stone-50\/100:is(.dark *){color:#fafaf9}.dark\:text-stone-50\/15:is(.dark *){color:hsla(60,9%,98%,.15)}.dark\:text-stone-50\/20:is(.dark *){color:hsla(60,9%,98%,.2)}.dark\:text-stone-50\/25:is(.dark *){color:hsla(60,9%,98%,.25)}.dark\:text-stone-50\/30:is(.dark *){color:hsla(60,9%,98%,.3)}.dark\:text-stone-50\/35:is(.dark *){color:hsla(60,9%,98%,.35)}.dark\:text-stone-50\/40:is(.dark *){color:hsla(60,9%,98%,.4)}.dark\:text-stone-50\/45:is(.dark *){color:hsla(60,9%,98%,.45)}.dark\:text-stone-50\/5:is(.dark *){color:hsla(60,9%,98%,.05)}.dark\:text-stone-50\/50:is(.dark *){color:hsla(60,9%,98%,.5)}.dark\:text-stone-50\/55:is(.dark *){color:hsla(60,9%,98%,.55)}.dark\:text-stone-50\/60:is(.dark *){color:hsla(60,9%,98%,.6)}.dark\:text-stone-50\/65:is(.dark *){color:hsla(60,9%,98%,.65)}.dark\:text-stone-50\/70:is(.dark *){color:hsla(60,9%,98%,.7)}.dark\:text-stone-50\/75:is(.dark *){color:hsla(60,9%,98%,.75)}.dark\:text-stone-50\/80:is(.dark *){color:hsla(60,9%,98%,.8)}.dark\:text-stone-50\/85:is(.dark *){color:hsla(60,9%,98%,.85)}.dark\:text-stone-50\/90:is(.dark *){color:hsla(60,9%,98%,.9)}.dark\:text-stone-50\/95:is(.dark *){color:hsla(60,9%,98%,.95)}.dark\:text-stone-500:is(.dark *){--tw-text-opacity:1;color:rgb(120 113 108/var(--tw-text-opacity,1))}.dark\:text-stone-500\/0:is(.dark *){color:hsla(25,5%,45%,0)}.dark\:text-stone-500\/10:is(.dark *){color:hsla(25,5%,45%,.1)}.dark\:text-stone-500\/100:is(.dark *){color:#78716c}.dark\:text-stone-500\/15:is(.dark *){color:hsla(25,5%,45%,.15)}.dark\:text-stone-500\/20:is(.dark *){color:hsla(25,5%,45%,.2)}.dark\:text-stone-500\/25:is(.dark *){color:hsla(25,5%,45%,.25)}.dark\:text-stone-500\/30:is(.dark *){color:hsla(25,5%,45%,.3)}.dark\:text-stone-500\/35:is(.dark *){color:hsla(25,5%,45%,.35)}.dark\:text-stone-500\/40:is(.dark *){color:hsla(25,5%,45%,.4)}.dark\:text-stone-500\/45:is(.dark *){color:hsla(25,5%,45%,.45)}.dark\:text-stone-500\/5:is(.dark *){color:hsla(25,5%,45%,.05)}.dark\:text-stone-500\/50:is(.dark *){color:hsla(25,5%,45%,.5)}.dark\:text-stone-500\/55:is(.dark *){color:hsla(25,5%,45%,.55)}.dark\:text-stone-500\/60:is(.dark *){color:hsla(25,5%,45%,.6)}.dark\:text-stone-500\/65:is(.dark *){color:hsla(25,5%,45%,.65)}.dark\:text-stone-500\/70:is(.dark *){color:hsla(25,5%,45%,.7)}.dark\:text-stone-500\/75:is(.dark *){color:hsla(25,5%,45%,.75)}.dark\:text-stone-500\/80:is(.dark *){color:hsla(25,5%,45%,.8)}.dark\:text-stone-500\/85:is(.dark *){color:hsla(25,5%,45%,.85)}.dark\:text-stone-500\/90:is(.dark *){color:hsla(25,5%,45%,.9)}.dark\:text-stone-500\/95:is(.dark *){color:hsla(25,5%,45%,.95)}.dark\:text-stone-950:is(.dark *){--tw-text-opacity:1;color:rgb(12 10 9/var(--tw-text-opacity,1))}.dark\:text-stone-950\/0:is(.dark *){color:rgba(12,10,9,0)}.dark\:text-stone-950\/10:is(.dark *){color:rgba(12,10,9,.1)}.dark\:text-stone-950\/100:is(.dark *){color:#0c0a09}.dark\:text-stone-950\/15:is(.dark *){color:rgba(12,10,9,.15)}.dark\:text-stone-950\/20:is(.dark *){color:rgba(12,10,9,.2)}.dark\:text-stone-950\/25:is(.dark *){color:rgba(12,10,9,.25)}.dark\:text-stone-950\/30:is(.dark *){color:rgba(12,10,9,.3)}.dark\:text-stone-950\/35:is(.dark *){color:rgba(12,10,9,.35)}.dark\:text-stone-950\/40:is(.dark *){color:rgba(12,10,9,.4)}.dark\:text-stone-950\/45:is(.dark *){color:rgba(12,10,9,.45)}.dark\:text-stone-950\/5:is(.dark *){color:rgba(12,10,9,.05)}.dark\:text-stone-950\/50:is(.dark *){color:rgba(12,10,9,.5)}.dark\:text-stone-950\/55:is(.dark *){color:rgba(12,10,9,.55)}.dark\:text-stone-950\/60:is(.dark *){color:rgba(12,10,9,.6)}.dark\:text-stone-950\/65:is(.dark *){color:rgba(12,10,9,.65)}.dark\:text-stone-950\/70:is(.dark *){color:rgba(12,10,9,.7)}.dark\:text-stone-950\/75:is(.dark *){color:rgba(12,10,9,.75)}.dark\:text-stone-950\/80:is(.dark *){color:rgba(12,10,9,.8)}.dark\:text-stone-950\/85:is(.dark *){color:rgba(12,10,9,.85)}.dark\:text-stone-950\/90:is(.dark *){color:rgba(12,10,9,.9)}.dark\:text-stone-950\/95:is(.dark *){color:rgba(12,10,9,.95)}.dark\:text-teal-100:is(.dark *){--tw-text-opacity:1;color:rgb(204 251 241/var(--tw-text-opacity,1))}.dark\:text-teal-100\/0:is(.dark *){color:rgba(204,251,241,0)}.dark\:text-teal-100\/10:is(.dark *){color:rgba(204,251,241,.1)}.dark\:text-teal-100\/100:is(.dark *){color:#ccfbf1}.dark\:text-teal-100\/15:is(.dark *){color:rgba(204,251,241,.15)}.dark\:text-teal-100\/20:is(.dark *){color:rgba(204,251,241,.2)}.dark\:text-teal-100\/25:is(.dark *){color:rgba(204,251,241,.25)}.dark\:text-teal-100\/30:is(.dark *){color:rgba(204,251,241,.3)}.dark\:text-teal-100\/35:is(.dark *){color:rgba(204,251,241,.35)}.dark\:text-teal-100\/40:is(.dark *){color:rgba(204,251,241,.4)}.dark\:text-teal-100\/45:is(.dark *){color:rgba(204,251,241,.45)}.dark\:text-teal-100\/5:is(.dark *){color:rgba(204,251,241,.05)}.dark\:text-teal-100\/50:is(.dark *){color:rgba(204,251,241,.5)}.dark\:text-teal-100\/55:is(.dark *){color:rgba(204,251,241,.55)}.dark\:text-teal-100\/60:is(.dark *){color:rgba(204,251,241,.6)}.dark\:text-teal-100\/65:is(.dark *){color:rgba(204,251,241,.65)}.dark\:text-teal-100\/70:is(.dark *){color:rgba(204,251,241,.7)}.dark\:text-teal-100\/75:is(.dark *){color:rgba(204,251,241,.75)}.dark\:text-teal-100\/80:is(.dark *){color:rgba(204,251,241,.8)}.dark\:text-teal-100\/85:is(.dark *){color:rgba(204,251,241,.85)}.dark\:text-teal-100\/90:is(.dark *){color:rgba(204,251,241,.9)}.dark\:text-teal-100\/95:is(.dark *){color:rgba(204,251,241,.95)}.dark\:text-teal-200:is(.dark *){--tw-text-opacity:1;color:rgb(153 246 228/var(--tw-text-opacity,1))}.dark\:text-teal-200\/0:is(.dark *){color:rgba(153,246,228,0)}.dark\:text-teal-200\/10:is(.dark *){color:rgba(153,246,228,.1)}.dark\:text-teal-200\/100:is(.dark *){color:#99f6e4}.dark\:text-teal-200\/15:is(.dark *){color:rgba(153,246,228,.15)}.dark\:text-teal-200\/20:is(.dark *){color:rgba(153,246,228,.2)}.dark\:text-teal-200\/25:is(.dark *){color:rgba(153,246,228,.25)}.dark\:text-teal-200\/30:is(.dark *){color:rgba(153,246,228,.3)}.dark\:text-teal-200\/35:is(.dark *){color:rgba(153,246,228,.35)}.dark\:text-teal-200\/40:is(.dark *){color:rgba(153,246,228,.4)}.dark\:text-teal-200\/45:is(.dark *){color:rgba(153,246,228,.45)}.dark\:text-teal-200\/5:is(.dark *){color:rgba(153,246,228,.05)}.dark\:text-teal-200\/50:is(.dark *){color:rgba(153,246,228,.5)}.dark\:text-teal-200\/55:is(.dark *){color:rgba(153,246,228,.55)}.dark\:text-teal-200\/60:is(.dark *){color:rgba(153,246,228,.6)}.dark\:text-teal-200\/65:is(.dark *){color:rgba(153,246,228,.65)}.dark\:text-teal-200\/70:is(.dark *){color:rgba(153,246,228,.7)}.dark\:text-teal-200\/75:is(.dark *){color:rgba(153,246,228,.75)}.dark\:text-teal-200\/80:is(.dark *){color:rgba(153,246,228,.8)}.dark\:text-teal-200\/85:is(.dark *){color:rgba(153,246,228,.85)}.dark\:text-teal-200\/90:is(.dark *){color:rgba(153,246,228,.9)}.dark\:text-teal-200\/95:is(.dark *){color:rgba(153,246,228,.95)}.dark\:text-teal-50:is(.dark *){--tw-text-opacity:1;color:rgb(240 253 250/var(--tw-text-opacity,1))}.dark\:text-teal-50\/0:is(.dark *){color:rgba(240,253,250,0)}.dark\:text-teal-50\/10:is(.dark *){color:rgba(240,253,250,.1)}.dark\:text-teal-50\/100:is(.dark *){color:#f0fdfa}.dark\:text-teal-50\/15:is(.dark *){color:rgba(240,253,250,.15)}.dark\:text-teal-50\/20:is(.dark *){color:rgba(240,253,250,.2)}.dark\:text-teal-50\/25:is(.dark *){color:rgba(240,253,250,.25)}.dark\:text-teal-50\/30:is(.dark *){color:rgba(240,253,250,.3)}.dark\:text-teal-50\/35:is(.dark *){color:rgba(240,253,250,.35)}.dark\:text-teal-50\/40:is(.dark *){color:rgba(240,253,250,.4)}.dark\:text-teal-50\/45:is(.dark *){color:rgba(240,253,250,.45)}.dark\:text-teal-50\/5:is(.dark *){color:rgba(240,253,250,.05)}.dark\:text-teal-50\/50:is(.dark *){color:rgba(240,253,250,.5)}.dark\:text-teal-50\/55:is(.dark *){color:rgba(240,253,250,.55)}.dark\:text-teal-50\/60:is(.dark *){color:rgba(240,253,250,.6)}.dark\:text-teal-50\/65:is(.dark *){color:rgba(240,253,250,.65)}.dark\:text-teal-50\/70:is(.dark *){color:rgba(240,253,250,.7)}.dark\:text-teal-50\/75:is(.dark *){color:rgba(240,253,250,.75)}.dark\:text-teal-50\/80:is(.dark *){color:rgba(240,253,250,.8)}.dark\:text-teal-50\/85:is(.dark *){color:rgba(240,253,250,.85)}.dark\:text-teal-50\/90:is(.dark *){color:rgba(240,253,250,.9)}.dark\:text-teal-50\/95:is(.dark *){color:rgba(240,253,250,.95)}.dark\:text-teal-500:is(.dark *){--tw-text-opacity:1;color:rgb(20 184 166/var(--tw-text-opacity,1))}.dark\:text-teal-500\/0:is(.dark *){color:rgba(20,184,166,0)}.dark\:text-teal-500\/10:is(.dark *){color:rgba(20,184,166,.1)}.dark\:text-teal-500\/100:is(.dark *){color:#14b8a6}.dark\:text-teal-500\/15:is(.dark *){color:rgba(20,184,166,.15)}.dark\:text-teal-500\/20:is(.dark *){color:rgba(20,184,166,.2)}.dark\:text-teal-500\/25:is(.dark *){color:rgba(20,184,166,.25)}.dark\:text-teal-500\/30:is(.dark *){color:rgba(20,184,166,.3)}.dark\:text-teal-500\/35:is(.dark *){color:rgba(20,184,166,.35)}.dark\:text-teal-500\/40:is(.dark *){color:rgba(20,184,166,.4)}.dark\:text-teal-500\/45:is(.dark *){color:rgba(20,184,166,.45)}.dark\:text-teal-500\/5:is(.dark *){color:rgba(20,184,166,.05)}.dark\:text-teal-500\/50:is(.dark *){color:rgba(20,184,166,.5)}.dark\:text-teal-500\/55:is(.dark *){color:rgba(20,184,166,.55)}.dark\:text-teal-500\/60:is(.dark *){color:rgba(20,184,166,.6)}.dark\:text-teal-500\/65:is(.dark *){color:rgba(20,184,166,.65)}.dark\:text-teal-500\/70:is(.dark *){color:rgba(20,184,166,.7)}.dark\:text-teal-500\/75:is(.dark *){color:rgba(20,184,166,.75)}.dark\:text-teal-500\/80:is(.dark *){color:rgba(20,184,166,.8)}.dark\:text-teal-500\/85:is(.dark *){color:rgba(20,184,166,.85)}.dark\:text-teal-500\/90:is(.dark *){color:rgba(20,184,166,.9)}.dark\:text-teal-500\/95:is(.dark *){color:rgba(20,184,166,.95)}.dark\:text-teal-950:is(.dark *){--tw-text-opacity:1;color:rgb(4 47 46/var(--tw-text-opacity,1))}.dark\:text-teal-950\/0:is(.dark *){color:rgba(4,47,46,0)}.dark\:text-teal-950\/10:is(.dark *){color:rgba(4,47,46,.1)}.dark\:text-teal-950\/100:is(.dark *){color:#042f2e}.dark\:text-teal-950\/15:is(.dark *){color:rgba(4,47,46,.15)}.dark\:text-teal-950\/20:is(.dark *){color:rgba(4,47,46,.2)}.dark\:text-teal-950\/25:is(.dark *){color:rgba(4,47,46,.25)}.dark\:text-teal-950\/30:is(.dark *){color:rgba(4,47,46,.3)}.dark\:text-teal-950\/35:is(.dark *){color:rgba(4,47,46,.35)}.dark\:text-teal-950\/40:is(.dark *){color:rgba(4,47,46,.4)}.dark\:text-teal-950\/45:is(.dark *){color:rgba(4,47,46,.45)}.dark\:text-teal-950\/5:is(.dark *){color:rgba(4,47,46,.05)}.dark\:text-teal-950\/50:is(.dark *){color:rgba(4,47,46,.5)}.dark\:text-teal-950\/55:is(.dark *){color:rgba(4,47,46,.55)}.dark\:text-teal-950\/60:is(.dark *){color:rgba(4,47,46,.6)}.dark\:text-teal-950\/65:is(.dark *){color:rgba(4,47,46,.65)}.dark\:text-teal-950\/70:is(.dark *){color:rgba(4,47,46,.7)}.dark\:text-teal-950\/75:is(.dark *){color:rgba(4,47,46,.75)}.dark\:text-teal-950\/80:is(.dark *){color:rgba(4,47,46,.8)}.dark\:text-teal-950\/85:is(.dark *){color:rgba(4,47,46,.85)}.dark\:text-teal-950\/90:is(.dark *){color:rgba(4,47,46,.9)}.dark\:text-teal-950\/95:is(.dark *){color:rgba(4,47,46,.95)}.dark\:text-violet-100:is(.dark *){--tw-text-opacity:1;color:rgb(237 233 254/var(--tw-text-opacity,1))}.dark\:text-violet-100\/0:is(.dark *){color:rgba(237,233,254,0)}.dark\:text-violet-100\/10:is(.dark *){color:rgba(237,233,254,.1)}.dark\:text-violet-100\/100:is(.dark *){color:#ede9fe}.dark\:text-violet-100\/15:is(.dark *){color:rgba(237,233,254,.15)}.dark\:text-violet-100\/20:is(.dark *){color:rgba(237,233,254,.2)}.dark\:text-violet-100\/25:is(.dark *){color:rgba(237,233,254,.25)}.dark\:text-violet-100\/30:is(.dark *){color:rgba(237,233,254,.3)}.dark\:text-violet-100\/35:is(.dark *){color:rgba(237,233,254,.35)}.dark\:text-violet-100\/40:is(.dark *){color:rgba(237,233,254,.4)}.dark\:text-violet-100\/45:is(.dark *){color:rgba(237,233,254,.45)}.dark\:text-violet-100\/5:is(.dark *){color:rgba(237,233,254,.05)}.dark\:text-violet-100\/50:is(.dark *){color:rgba(237,233,254,.5)}.dark\:text-violet-100\/55:is(.dark *){color:rgba(237,233,254,.55)}.dark\:text-violet-100\/60:is(.dark *){color:rgba(237,233,254,.6)}.dark\:text-violet-100\/65:is(.dark *){color:rgba(237,233,254,.65)}.dark\:text-violet-100\/70:is(.dark *){color:rgba(237,233,254,.7)}.dark\:text-violet-100\/75:is(.dark *){color:rgba(237,233,254,.75)}.dark\:text-violet-100\/80:is(.dark *){color:rgba(237,233,254,.8)}.dark\:text-violet-100\/85:is(.dark *){color:rgba(237,233,254,.85)}.dark\:text-violet-100\/90:is(.dark *){color:rgba(237,233,254,.9)}.dark\:text-violet-100\/95:is(.dark *){color:rgba(237,233,254,.95)}.dark\:text-violet-200:is(.dark *){--tw-text-opacity:1;color:rgb(221 214 254/var(--tw-text-opacity,1))}.dark\:text-violet-200\/0:is(.dark *){color:rgba(221,214,254,0)}.dark\:text-violet-200\/10:is(.dark *){color:rgba(221,214,254,.1)}.dark\:text-violet-200\/100:is(.dark *){color:#ddd6fe}.dark\:text-violet-200\/15:is(.dark *){color:rgba(221,214,254,.15)}.dark\:text-violet-200\/20:is(.dark *){color:rgba(221,214,254,.2)}.dark\:text-violet-200\/25:is(.dark *){color:rgba(221,214,254,.25)}.dark\:text-violet-200\/30:is(.dark *){color:rgba(221,214,254,.3)}.dark\:text-violet-200\/35:is(.dark *){color:rgba(221,214,254,.35)}.dark\:text-violet-200\/40:is(.dark *){color:rgba(221,214,254,.4)}.dark\:text-violet-200\/45:is(.dark *){color:rgba(221,214,254,.45)}.dark\:text-violet-200\/5:is(.dark *){color:rgba(221,214,254,.05)}.dark\:text-violet-200\/50:is(.dark *){color:rgba(221,214,254,.5)}.dark\:text-violet-200\/55:is(.dark *){color:rgba(221,214,254,.55)}.dark\:text-violet-200\/60:is(.dark *){color:rgba(221,214,254,.6)}.dark\:text-violet-200\/65:is(.dark *){color:rgba(221,214,254,.65)}.dark\:text-violet-200\/70:is(.dark *){color:rgba(221,214,254,.7)}.dark\:text-violet-200\/75:is(.dark *){color:rgba(221,214,254,.75)}.dark\:text-violet-200\/80:is(.dark *){color:rgba(221,214,254,.8)}.dark\:text-violet-200\/85:is(.dark *){color:rgba(221,214,254,.85)}.dark\:text-violet-200\/90:is(.dark *){color:rgba(221,214,254,.9)}.dark\:text-violet-200\/95:is(.dark *){color:rgba(221,214,254,.95)}.dark\:text-violet-50:is(.dark *){--tw-text-opacity:1;color:rgb(245 243 255/var(--tw-text-opacity,1))}.dark\:text-violet-50\/0:is(.dark *){color:rgba(245,243,255,0)}.dark\:text-violet-50\/10:is(.dark *){color:rgba(245,243,255,.1)}.dark\:text-violet-50\/100:is(.dark *){color:#f5f3ff}.dark\:text-violet-50\/15:is(.dark *){color:rgba(245,243,255,.15)}.dark\:text-violet-50\/20:is(.dark *){color:rgba(245,243,255,.2)}.dark\:text-violet-50\/25:is(.dark *){color:rgba(245,243,255,.25)}.dark\:text-violet-50\/30:is(.dark *){color:rgba(245,243,255,.3)}.dark\:text-violet-50\/35:is(.dark *){color:rgba(245,243,255,.35)}.dark\:text-violet-50\/40:is(.dark *){color:rgba(245,243,255,.4)}.dark\:text-violet-50\/45:is(.dark *){color:rgba(245,243,255,.45)}.dark\:text-violet-50\/5:is(.dark *){color:rgba(245,243,255,.05)}.dark\:text-violet-50\/50:is(.dark *){color:rgba(245,243,255,.5)}.dark\:text-violet-50\/55:is(.dark *){color:rgba(245,243,255,.55)}.dark\:text-violet-50\/60:is(.dark *){color:rgba(245,243,255,.6)}.dark\:text-violet-50\/65:is(.dark *){color:rgba(245,243,255,.65)}.dark\:text-violet-50\/70:is(.dark *){color:rgba(245,243,255,.7)}.dark\:text-violet-50\/75:is(.dark *){color:rgba(245,243,255,.75)}.dark\:text-violet-50\/80:is(.dark *){color:rgba(245,243,255,.8)}.dark\:text-violet-50\/85:is(.dark *){color:rgba(245,243,255,.85)}.dark\:text-violet-50\/90:is(.dark *){color:rgba(245,243,255,.9)}.dark\:text-violet-50\/95:is(.dark *){color:rgba(245,243,255,.95)}.dark\:text-violet-500:is(.dark *){--tw-text-opacity:1;color:rgb(139 92 246/var(--tw-text-opacity,1))}.dark\:text-violet-500\/0:is(.dark *){color:rgba(139,92,246,0)}.dark\:text-violet-500\/10:is(.dark *){color:rgba(139,92,246,.1)}.dark\:text-violet-500\/100:is(.dark *){color:#8b5cf6}.dark\:text-violet-500\/15:is(.dark *){color:rgba(139,92,246,.15)}.dark\:text-violet-500\/20:is(.dark *){color:rgba(139,92,246,.2)}.dark\:text-violet-500\/25:is(.dark *){color:rgba(139,92,246,.25)}.dark\:text-violet-500\/30:is(.dark *){color:rgba(139,92,246,.3)}.dark\:text-violet-500\/35:is(.dark *){color:rgba(139,92,246,.35)}.dark\:text-violet-500\/40:is(.dark *){color:rgba(139,92,246,.4)}.dark\:text-violet-500\/45:is(.dark *){color:rgba(139,92,246,.45)}.dark\:text-violet-500\/5:is(.dark *){color:rgba(139,92,246,.05)}.dark\:text-violet-500\/50:is(.dark *){color:rgba(139,92,246,.5)}.dark\:text-violet-500\/55:is(.dark *){color:rgba(139,92,246,.55)}.dark\:text-violet-500\/60:is(.dark *){color:rgba(139,92,246,.6)}.dark\:text-violet-500\/65:is(.dark *){color:rgba(139,92,246,.65)}.dark\:text-violet-500\/70:is(.dark *){color:rgba(139,92,246,.7)}.dark\:text-violet-500\/75:is(.dark *){color:rgba(139,92,246,.75)}.dark\:text-violet-500\/80:is(.dark *){color:rgba(139,92,246,.8)}.dark\:text-violet-500\/85:is(.dark *){color:rgba(139,92,246,.85)}.dark\:text-violet-500\/90:is(.dark *){color:rgba(139,92,246,.9)}.dark\:text-violet-500\/95:is(.dark *){color:rgba(139,92,246,.95)}.dark\:text-violet-950:is(.dark *){--tw-text-opacity:1;color:rgb(46 16 101/var(--tw-text-opacity,1))}.dark\:text-violet-950\/0:is(.dark *){color:rgba(46,16,101,0)}.dark\:text-violet-950\/10:is(.dark *){color:rgba(46,16,101,.1)}.dark\:text-violet-950\/100:is(.dark *){color:#2e1065}.dark\:text-violet-950\/15:is(.dark *){color:rgba(46,16,101,.15)}.dark\:text-violet-950\/20:is(.dark *){color:rgba(46,16,101,.2)}.dark\:text-violet-950\/25:is(.dark *){color:rgba(46,16,101,.25)}.dark\:text-violet-950\/30:is(.dark *){color:rgba(46,16,101,.3)}.dark\:text-violet-950\/35:is(.dark *){color:rgba(46,16,101,.35)}.dark\:text-violet-950\/40:is(.dark *){color:rgba(46,16,101,.4)}.dark\:text-violet-950\/45:is(.dark *){color:rgba(46,16,101,.45)}.dark\:text-violet-950\/5:is(.dark *){color:rgba(46,16,101,.05)}.dark\:text-violet-950\/50:is(.dark *){color:rgba(46,16,101,.5)}.dark\:text-violet-950\/55:is(.dark *){color:rgba(46,16,101,.55)}.dark\:text-violet-950\/60:is(.dark *){color:rgba(46,16,101,.6)}.dark\:text-violet-950\/65:is(.dark *){color:rgba(46,16,101,.65)}.dark\:text-violet-950\/70:is(.dark *){color:rgba(46,16,101,.7)}.dark\:text-violet-950\/75:is(.dark *){color:rgba(46,16,101,.75)}.dark\:text-violet-950\/80:is(.dark *){color:rgba(46,16,101,.8)}.dark\:text-violet-950\/85:is(.dark *){color:rgba(46,16,101,.85)}.dark\:text-violet-950\/90:is(.dark *){color:rgba(46,16,101,.9)}.dark\:text-violet-950\/95:is(.dark *){color:rgba(46,16,101,.95)}.dark\:text-white:is(.dark *){--tw-text-opacity:1;color:rgb(255 255 255/var(--tw-text-opacity,1))}.dark\:text-yellow-100:is(.dark *){--tw-text-opacity:1;color:rgb(254 249 195/var(--tw-text-opacity,1))}.dark\:text-yellow-100\/0:is(.dark *){color:hsla(55,97%,88%,0)}.dark\:text-yellow-100\/10:is(.dark *){color:hsla(55,97%,88%,.1)}.dark\:text-yellow-100\/100:is(.dark *){color:#fef9c3}.dark\:text-yellow-100\/15:is(.dark *){color:hsla(55,97%,88%,.15)}.dark\:text-yellow-100\/20:is(.dark *){color:hsla(55,97%,88%,.2)}.dark\:text-yellow-100\/25:is(.dark *){color:hsla(55,97%,88%,.25)}.dark\:text-yellow-100\/30:is(.dark *){color:hsla(55,97%,88%,.3)}.dark\:text-yellow-100\/35:is(.dark *){color:hsla(55,97%,88%,.35)}.dark\:text-yellow-100\/40:is(.dark *){color:hsla(55,97%,88%,.4)}.dark\:text-yellow-100\/45:is(.dark *){color:hsla(55,97%,88%,.45)}.dark\:text-yellow-100\/5:is(.dark *){color:hsla(55,97%,88%,.05)}.dark\:text-yellow-100\/50:is(.dark *){color:hsla(55,97%,88%,.5)}.dark\:text-yellow-100\/55:is(.dark *){color:hsla(55,97%,88%,.55)}.dark\:text-yellow-100\/60:is(.dark *){color:hsla(55,97%,88%,.6)}.dark\:text-yellow-100\/65:is(.dark *){color:hsla(55,97%,88%,.65)}.dark\:text-yellow-100\/70:is(.dark *){color:hsla(55,97%,88%,.7)}.dark\:text-yellow-100\/75:is(.dark *){color:hsla(55,97%,88%,.75)}.dark\:text-yellow-100\/80:is(.dark *){color:hsla(55,97%,88%,.8)}.dark\:text-yellow-100\/85:is(.dark *){color:hsla(55,97%,88%,.85)}.dark\:text-yellow-100\/90:is(.dark *){color:hsla(55,97%,88%,.9)}.dark\:text-yellow-100\/95:is(.dark *){color:hsla(55,97%,88%,.95)}.dark\:text-yellow-200:is(.dark *){--tw-text-opacity:1;color:rgb(254 240 138/var(--tw-text-opacity,1))}.dark\:text-yellow-200\/0:is(.dark *){color:hsla(53,98%,77%,0)}.dark\:text-yellow-200\/10:is(.dark *){color:hsla(53,98%,77%,.1)}.dark\:text-yellow-200\/100:is(.dark *){color:#fef08a}.dark\:text-yellow-200\/15:is(.dark *){color:hsla(53,98%,77%,.15)}.dark\:text-yellow-200\/20:is(.dark *){color:hsla(53,98%,77%,.2)}.dark\:text-yellow-200\/25:is(.dark *){color:hsla(53,98%,77%,.25)}.dark\:text-yellow-200\/30:is(.dark *){color:hsla(53,98%,77%,.3)}.dark\:text-yellow-200\/35:is(.dark *){color:hsla(53,98%,77%,.35)}.dark\:text-yellow-200\/40:is(.dark *){color:hsla(53,98%,77%,.4)}.dark\:text-yellow-200\/45:is(.dark *){color:hsla(53,98%,77%,.45)}.dark\:text-yellow-200\/5:is(.dark *){color:hsla(53,98%,77%,.05)}.dark\:text-yellow-200\/50:is(.dark *){color:hsla(53,98%,77%,.5)}.dark\:text-yellow-200\/55:is(.dark *){color:hsla(53,98%,77%,.55)}.dark\:text-yellow-200\/60:is(.dark *){color:hsla(53,98%,77%,.6)}.dark\:text-yellow-200\/65:is(.dark *){color:hsla(53,98%,77%,.65)}.dark\:text-yellow-200\/70:is(.dark *){color:hsla(53,98%,77%,.7)}.dark\:text-yellow-200\/75:is(.dark *){color:hsla(53,98%,77%,.75)}.dark\:text-yellow-200\/80:is(.dark *){color:hsla(53,98%,77%,.8)}.dark\:text-yellow-200\/85:is(.dark *){color:hsla(53,98%,77%,.85)}.dark\:text-yellow-200\/90:is(.dark *){color:hsla(53,98%,77%,.9)}.dark\:text-yellow-200\/95:is(.dark *){color:hsla(53,98%,77%,.95)}.dark\:text-yellow-50:is(.dark *){--tw-text-opacity:1;color:rgb(254 252 232/var(--tw-text-opacity,1))}.dark\:text-yellow-50\/0:is(.dark *){color:hsla(55,92%,95%,0)}.dark\:text-yellow-50\/10:is(.dark *){color:hsla(55,92%,95%,.1)}.dark\:text-yellow-50\/100:is(.dark *){color:#fefce8}.dark\:text-yellow-50\/15:is(.dark *){color:hsla(55,92%,95%,.15)}.dark\:text-yellow-50\/20:is(.dark *){color:hsla(55,92%,95%,.2)}.dark\:text-yellow-50\/25:is(.dark *){color:hsla(55,92%,95%,.25)}.dark\:text-yellow-50\/30:is(.dark *){color:hsla(55,92%,95%,.3)}.dark\:text-yellow-50\/35:is(.dark *){color:hsla(55,92%,95%,.35)}.dark\:text-yellow-50\/40:is(.dark *){color:hsla(55,92%,95%,.4)}.dark\:text-yellow-50\/45:is(.dark *){color:hsla(55,92%,95%,.45)}.dark\:text-yellow-50\/5:is(.dark *){color:hsla(55,92%,95%,.05)}.dark\:text-yellow-50\/50:is(.dark *){color:hsla(55,92%,95%,.5)}.dark\:text-yellow-50\/55:is(.dark *){color:hsla(55,92%,95%,.55)}.dark\:text-yellow-50\/60:is(.dark *){color:hsla(55,92%,95%,.6)}.dark\:text-yellow-50\/65:is(.dark *){color:hsla(55,92%,95%,.65)}.dark\:text-yellow-50\/70:is(.dark *){color:hsla(55,92%,95%,.7)}.dark\:text-yellow-50\/75:is(.dark *){color:hsla(55,92%,95%,.75)}.dark\:text-yellow-50\/80:is(.dark *){color:hsla(55,92%,95%,.8)}.dark\:text-yellow-50\/85:is(.dark *){color:hsla(55,92%,95%,.85)}.dark\:text-yellow-50\/90:is(.dark *){color:hsla(55,92%,95%,.9)}.dark\:text-yellow-50\/95:is(.dark *){color:hsla(55,92%,95%,.95)}.dark\:text-yellow-500:is(.dark *){--tw-text-opacity:1;color:rgb(234 179 8/var(--tw-text-opacity,1))}.dark\:text-yellow-500\/0:is(.dark *){color:rgba(234,179,8,0)}.dark\:text-yellow-500\/10:is(.dark *){color:rgba(234,179,8,.1)}.dark\:text-yellow-500\/100:is(.dark *){color:#eab308}.dark\:text-yellow-500\/15:is(.dark *){color:rgba(234,179,8,.15)}.dark\:text-yellow-500\/20:is(.dark *){color:rgba(234,179,8,.2)}.dark\:text-yellow-500\/25:is(.dark *){color:rgba(234,179,8,.25)}.dark\:text-yellow-500\/30:is(.dark *){color:rgba(234,179,8,.3)}.dark\:text-yellow-500\/35:is(.dark *){color:rgba(234,179,8,.35)}.dark\:text-yellow-500\/40:is(.dark *){color:rgba(234,179,8,.4)}.dark\:text-yellow-500\/45:is(.dark *){color:rgba(234,179,8,.45)}.dark\:text-yellow-500\/5:is(.dark *){color:rgba(234,179,8,.05)}.dark\:text-yellow-500\/50:is(.dark *){color:rgba(234,179,8,.5)}.dark\:text-yellow-500\/55:is(.dark *){color:rgba(234,179,8,.55)}.dark\:text-yellow-500\/60:is(.dark *){color:rgba(234,179,8,.6)}.dark\:text-yellow-500\/65:is(.dark *){color:rgba(234,179,8,.65)}.dark\:text-yellow-500\/70:is(.dark *){color:rgba(234,179,8,.7)}.dark\:text-yellow-500\/75:is(.dark *){color:rgba(234,179,8,.75)}.dark\:text-yellow-500\/80:is(.dark *){color:rgba(234,179,8,.8)}.dark\:text-yellow-500\/85:is(.dark *){color:rgba(234,179,8,.85)}.dark\:text-yellow-500\/90:is(.dark *){color:rgba(234,179,8,.9)}.dark\:text-yellow-500\/95:is(.dark *){color:rgba(234,179,8,.95)}.dark\:text-yellow-950:is(.dark *){--tw-text-opacity:1;color:rgb(66 32 6/var(--tw-text-opacity,1))}.dark\:text-yellow-950\/0:is(.dark *){color:rgba(66,32,6,0)}.dark\:text-yellow-950\/10:is(.dark *){color:rgba(66,32,6,.1)}.dark\:text-yellow-950\/100:is(.dark *){color:#422006}.dark\:text-yellow-950\/15:is(.dark *){color:rgba(66,32,6,.15)}.dark\:text-yellow-950\/20:is(.dark *){color:rgba(66,32,6,.2)}.dark\:text-yellow-950\/25:is(.dark *){color:rgba(66,32,6,.25)}.dark\:text-yellow-950\/30:is(.dark *){color:rgba(66,32,6,.3)}.dark\:text-yellow-950\/35:is(.dark *){color:rgba(66,32,6,.35)}.dark\:text-yellow-950\/40:is(.dark *){color:rgba(66,32,6,.4)}.dark\:text-yellow-950\/45:is(.dark *){color:rgba(66,32,6,.45)}.dark\:text-yellow-950\/5:is(.dark *){color:rgba(66,32,6,.05)}.dark\:text-yellow-950\/50:is(.dark *){color:rgba(66,32,6,.5)}.dark\:text-yellow-950\/55:is(.dark *){color:rgba(66,32,6,.55)}.dark\:text-yellow-950\/60:is(.dark *){color:rgba(66,32,6,.6)}.dark\:text-yellow-950\/65:is(.dark *){color:rgba(66,32,6,.65)}.dark\:text-yellow-950\/70:is(.dark *){color:rgba(66,32,6,.7)}.dark\:text-yellow-950\/75:is(.dark *){color:rgba(66,32,6,.75)}.dark\:text-yellow-950\/80:is(.dark *){color:rgba(66,32,6,.8)}.dark\:text-yellow-950\/85:is(.dark *){color:rgba(66,32,6,.85)}.dark\:text-yellow-950\/90:is(.dark *){color:rgba(66,32,6,.9)}.dark\:text-yellow-950\/95:is(.dark *){color:rgba(66,32,6,.95)}.dark\:text-zinc-100:is(.dark *){--tw-text-opacity:1;color:rgb(244 244 245/var(--tw-text-opacity,1))}.dark\:text-zinc-100\/0:is(.dark *){color:hsla(240,5%,96%,0)}.dark\:text-zinc-100\/10:is(.dark *){color:hsla(240,5%,96%,.1)}.dark\:text-zinc-100\/100:is(.dark *){color:#f4f4f5}.dark\:text-zinc-100\/15:is(.dark *){color:hsla(240,5%,96%,.15)}.dark\:text-zinc-100\/20:is(.dark *){color:hsla(240,5%,96%,.2)}.dark\:text-zinc-100\/25:is(.dark *){color:hsla(240,5%,96%,.25)}.dark\:text-zinc-100\/30:is(.dark *){color:hsla(240,5%,96%,.3)}.dark\:text-zinc-100\/35:is(.dark *){color:hsla(240,5%,96%,.35)}.dark\:text-zinc-100\/40:is(.dark *){color:hsla(240,5%,96%,.4)}.dark\:text-zinc-100\/45:is(.dark *){color:hsla(240,5%,96%,.45)}.dark\:text-zinc-100\/5:is(.dark *){color:hsla(240,5%,96%,.05)}.dark\:text-zinc-100\/50:is(.dark *){color:hsla(240,5%,96%,.5)}.dark\:text-zinc-100\/55:is(.dark *){color:hsla(240,5%,96%,.55)}.dark\:text-zinc-100\/60:is(.dark *){color:hsla(240,5%,96%,.6)}.dark\:text-zinc-100\/65:is(.dark *){color:hsla(240,5%,96%,.65)}.dark\:text-zinc-100\/70:is(.dark *){color:hsla(240,5%,96%,.7)}.dark\:text-zinc-100\/75:is(.dark *){color:hsla(240,5%,96%,.75)}.dark\:text-zinc-100\/80:is(.dark *){color:hsla(240,5%,96%,.8)}.dark\:text-zinc-100\/85:is(.dark *){color:hsla(240,5%,96%,.85)}.dark\:text-zinc-100\/90:is(.dark *){color:hsla(240,5%,96%,.9)}.dark\:text-zinc-100\/95:is(.dark *){color:hsla(240,5%,96%,.95)}.dark\:text-zinc-200:is(.dark *){--tw-text-opacity:1;color:rgb(228 228 231/var(--tw-text-opacity,1))}.dark\:text-zinc-200\/0:is(.dark *){color:hsla(240,6%,90%,0)}.dark\:text-zinc-200\/10:is(.dark *){color:hsla(240,6%,90%,.1)}.dark\:text-zinc-200\/100:is(.dark *){color:#e4e4e7}.dark\:text-zinc-200\/15:is(.dark *){color:hsla(240,6%,90%,.15)}.dark\:text-zinc-200\/20:is(.dark *){color:hsla(240,6%,90%,.2)}.dark\:text-zinc-200\/25:is(.dark *){color:hsla(240,6%,90%,.25)}.dark\:text-zinc-200\/30:is(.dark *){color:hsla(240,6%,90%,.3)}.dark\:text-zinc-200\/35:is(.dark *){color:hsla(240,6%,90%,.35)}.dark\:text-zinc-200\/40:is(.dark *){color:hsla(240,6%,90%,.4)}.dark\:text-zinc-200\/45:is(.dark *){color:hsla(240,6%,90%,.45)}.dark\:text-zinc-200\/5:is(.dark *){color:hsla(240,6%,90%,.05)}.dark\:text-zinc-200\/50:is(.dark *){color:hsla(240,6%,90%,.5)}.dark\:text-zinc-200\/55:is(.dark *){color:hsla(240,6%,90%,.55)}.dark\:text-zinc-200\/60:is(.dark *){color:hsla(240,6%,90%,.6)}.dark\:text-zinc-200\/65:is(.dark *){color:hsla(240,6%,90%,.65)}.dark\:text-zinc-200\/70:is(.dark *){color:hsla(240,6%,90%,.7)}.dark\:text-zinc-200\/75:is(.dark *){color:hsla(240,6%,90%,.75)}.dark\:text-zinc-200\/80:is(.dark *){color:hsla(240,6%,90%,.8)}.dark\:text-zinc-200\/85:is(.dark *){color:hsla(240,6%,90%,.85)}.dark\:text-zinc-200\/90:is(.dark *){color:hsla(240,6%,90%,.9)}.dark\:text-zinc-200\/95:is(.dark *){color:hsla(240,6%,90%,.95)}.dark\:text-zinc-50:is(.dark *){--tw-text-opacity:1;color:rgb(250 250 250/var(--tw-text-opacity,1))}.dark\:text-zinc-50\/0:is(.dark *){color:hsla(0,0%,98%,0)}.dark\:text-zinc-50\/10:is(.dark *){color:hsla(0,0%,98%,.1)}.dark\:text-zinc-50\/100:is(.dark *){color:#fafafa}.dark\:text-zinc-50\/15:is(.dark *){color:hsla(0,0%,98%,.15)}.dark\:text-zinc-50\/20:is(.dark *){color:hsla(0,0%,98%,.2)}.dark\:text-zinc-50\/25:is(.dark *){color:hsla(0,0%,98%,.25)}.dark\:text-zinc-50\/30:is(.dark *){color:hsla(0,0%,98%,.3)}.dark\:text-zinc-50\/35:is(.dark *){color:hsla(0,0%,98%,.35)}.dark\:text-zinc-50\/40:is(.dark *){color:hsla(0,0%,98%,.4)}.dark\:text-zinc-50\/45:is(.dark *){color:hsla(0,0%,98%,.45)}.dark\:text-zinc-50\/5:is(.dark *){color:hsla(0,0%,98%,.05)}.dark\:text-zinc-50\/50:is(.dark *){color:hsla(0,0%,98%,.5)}.dark\:text-zinc-50\/55:is(.dark *){color:hsla(0,0%,98%,.55)}.dark\:text-zinc-50\/60:is(.dark *){color:hsla(0,0%,98%,.6)}.dark\:text-zinc-50\/65:is(.dark *){color:hsla(0,0%,98%,.65)}.dark\:text-zinc-50\/70:is(.dark *){color:hsla(0,0%,98%,.7)}.dark\:text-zinc-50\/75:is(.dark *){color:hsla(0,0%,98%,.75)}.dark\:text-zinc-50\/80:is(.dark *){color:hsla(0,0%,98%,.8)}.dark\:text-zinc-50\/85:is(.dark *){color:hsla(0,0%,98%,.85)}.dark\:text-zinc-50\/90:is(.dark *){color:hsla(0,0%,98%,.9)}.dark\:text-zinc-50\/95:is(.dark *){color:hsla(0,0%,98%,.95)}.dark\:text-zinc-500:is(.dark *){--tw-text-opacity:1;color:rgb(113 113 122/var(--tw-text-opacity,1))}.dark\:text-zinc-500\/0:is(.dark *){color:hsla(240,4%,46%,0)}.dark\:text-zinc-500\/10:is(.dark *){color:hsla(240,4%,46%,.1)}.dark\:text-zinc-500\/100:is(.dark *){color:#71717a}.dark\:text-zinc-500\/15:is(.dark *){color:hsla(240,4%,46%,.15)}.dark\:text-zinc-500\/20:is(.dark *){color:hsla(240,4%,46%,.2)}.dark\:text-zinc-500\/25:is(.dark *){color:hsla(240,4%,46%,.25)}.dark\:text-zinc-500\/30:is(.dark *){color:hsla(240,4%,46%,.3)}.dark\:text-zinc-500\/35:is(.dark *){color:hsla(240,4%,46%,.35)}.dark\:text-zinc-500\/40:is(.dark *){color:hsla(240,4%,46%,.4)}.dark\:text-zinc-500\/45:is(.dark *){color:hsla(240,4%,46%,.45)}.dark\:text-zinc-500\/5:is(.dark *){color:hsla(240,4%,46%,.05)}.dark\:text-zinc-500\/50:is(.dark *){color:hsla(240,4%,46%,.5)}.dark\:text-zinc-500\/55:is(.dark *){color:hsla(240,4%,46%,.55)}.dark\:text-zinc-500\/60:is(.dark *){color:hsla(240,4%,46%,.6)}.dark\:text-zinc-500\/65:is(.dark *){color:hsla(240,4%,46%,.65)}.dark\:text-zinc-500\/70:is(.dark *){color:hsla(240,4%,46%,.7)}.dark\:text-zinc-500\/75:is(.dark *){color:hsla(240,4%,46%,.75)}.dark\:text-zinc-500\/80:is(.dark *){color:hsla(240,4%,46%,.8)}.dark\:text-zinc-500\/85:is(.dark *){color:hsla(240,4%,46%,.85)}.dark\:text-zinc-500\/90:is(.dark *){color:hsla(240,4%,46%,.9)}.dark\:text-zinc-500\/95:is(.dark *){color:hsla(240,4%,46%,.95)}.dark\:text-zinc-950:is(.dark *){--tw-text-opacity:1;color:rgb(9 9 11/var(--tw-text-opacity,1))}.dark\:text-zinc-950\/0:is(.dark *){color:rgba(9,9,11,0)}.dark\:text-zinc-950\/10:is(.dark *){color:rgba(9,9,11,.1)}.dark\:text-zinc-950\/100:is(.dark *){color:#09090b}.dark\:text-zinc-950\/15:is(.dark *){color:rgba(9,9,11,.15)}.dark\:text-zinc-950\/20:is(.dark *){color:rgba(9,9,11,.2)}.dark\:text-zinc-950\/25:is(.dark *){color:rgba(9,9,11,.25)}.dark\:text-zinc-950\/30:is(.dark *){color:rgba(9,9,11,.3)}.dark\:text-zinc-950\/35:is(.dark *){color:rgba(9,9,11,.35)}.dark\:text-zinc-950\/40:is(.dark *){color:rgba(9,9,11,.4)}.dark\:text-zinc-950\/45:is(.dark *){color:rgba(9,9,11,.45)}.dark\:text-zinc-950\/5:is(.dark *){color:rgba(9,9,11,.05)}.dark\:text-zinc-950\/50:is(.dark *){color:rgba(9,9,11,.5)}.dark\:text-zinc-950\/55:is(.dark *){color:rgba(9,9,11,.55)}.dark\:text-zinc-950\/60:is(.dark *){color:rgba(9,9,11,.6)}.dark\:text-zinc-950\/65:is(.dark *){color:rgba(9,9,11,.65)}.dark\:text-zinc-950\/70:is(.dark *){color:rgba(9,9,11,.7)}.dark\:text-zinc-950\/75:is(.dark *){color:rgba(9,9,11,.75)}.dark\:text-zinc-950\/80:is(.dark *){color:rgba(9,9,11,.8)}.dark\:text-zinc-950\/85:is(.dark *){color:rgba(9,9,11,.85)}.dark\:text-zinc-950\/90:is(.dark *){color:rgba(9,9,11,.9)}.dark\:text-zinc-950\/95:is(.dark *){color:rgba(9,9,11,.95)}.dark\:caret-amber-100:is(.dark *){caret-color:#fef3c7}.dark\:caret-amber-100\/0:is(.dark *){caret-color:hsla(48,96%,89%,0)}.dark\:caret-amber-100\/10:is(.dark *){caret-color:hsla(48,96%,89%,.1)}.dark\:caret-amber-100\/100:is(.dark *){caret-color:#fef3c7}.dark\:caret-amber-100\/15:is(.dark *){caret-color:hsla(48,96%,89%,.15)}.dark\:caret-amber-100\/20:is(.dark *){caret-color:hsla(48,96%,89%,.2)}.dark\:caret-amber-100\/25:is(.dark *){caret-color:hsla(48,96%,89%,.25)}.dark\:caret-amber-100\/30:is(.dark *){caret-color:hsla(48,96%,89%,.3)}.dark\:caret-amber-100\/35:is(.dark *){caret-color:hsla(48,96%,89%,.35)}.dark\:caret-amber-100\/40:is(.dark *){caret-color:hsla(48,96%,89%,.4)}.dark\:caret-amber-100\/45:is(.dark *){caret-color:hsla(48,96%,89%,.45)}.dark\:caret-amber-100\/5:is(.dark *){caret-color:hsla(48,96%,89%,.05)}.dark\:caret-amber-100\/50:is(.dark *){caret-color:hsla(48,96%,89%,.5)}.dark\:caret-amber-100\/55:is(.dark *){caret-color:hsla(48,96%,89%,.55)}.dark\:caret-amber-100\/60:is(.dark *){caret-color:hsla(48,96%,89%,.6)}.dark\:caret-amber-100\/65:is(.dark *){caret-color:hsla(48,96%,89%,.65)}.dark\:caret-amber-100\/70:is(.dark *){caret-color:hsla(48,96%,89%,.7)}.dark\:caret-amber-100\/75:is(.dark *){caret-color:hsla(48,96%,89%,.75)}.dark\:caret-amber-100\/80:is(.dark *){caret-color:hsla(48,96%,89%,.8)}.dark\:caret-amber-100\/85:is(.dark *){caret-color:hsla(48,96%,89%,.85)}.dark\:caret-amber-100\/90:is(.dark *){caret-color:hsla(48,96%,89%,.9)}.dark\:caret-amber-100\/95:is(.dark *){caret-color:hsla(48,96%,89%,.95)}.dark\:caret-amber-200:is(.dark *){caret-color:#fde68a}.dark\:caret-amber-200\/0:is(.dark *){caret-color:hsla(48,97%,77%,0)}.dark\:caret-amber-200\/10:is(.dark *){caret-color:hsla(48,97%,77%,.1)}.dark\:caret-amber-200\/100:is(.dark *){caret-color:#fde68a}.dark\:caret-amber-200\/15:is(.dark *){caret-color:hsla(48,97%,77%,.15)}.dark\:caret-amber-200\/20:is(.dark *){caret-color:hsla(48,97%,77%,.2)}.dark\:caret-amber-200\/25:is(.dark *){caret-color:hsla(48,97%,77%,.25)}.dark\:caret-amber-200\/30:is(.dark *){caret-color:hsla(48,97%,77%,.3)}.dark\:caret-amber-200\/35:is(.dark *){caret-color:hsla(48,97%,77%,.35)}.dark\:caret-amber-200\/40:is(.dark *){caret-color:hsla(48,97%,77%,.4)}.dark\:caret-amber-200\/45:is(.dark *){caret-color:hsla(48,97%,77%,.45)}.dark\:caret-amber-200\/5:is(.dark *){caret-color:hsla(48,97%,77%,.05)}.dark\:caret-amber-200\/50:is(.dark *){caret-color:hsla(48,97%,77%,.5)}.dark\:caret-amber-200\/55:is(.dark *){caret-color:hsla(48,97%,77%,.55)}.dark\:caret-amber-200\/60:is(.dark *){caret-color:hsla(48,97%,77%,.6)}.dark\:caret-amber-200\/65:is(.dark *){caret-color:hsla(48,97%,77%,.65)}.dark\:caret-amber-200\/70:is(.dark *){caret-color:hsla(48,97%,77%,.7)}.dark\:caret-amber-200\/75:is(.dark *){caret-color:hsla(48,97%,77%,.75)}.dark\:caret-amber-200\/80:is(.dark *){caret-color:hsla(48,97%,77%,.8)}.dark\:caret-amber-200\/85:is(.dark *){caret-color:hsla(48,97%,77%,.85)}.dark\:caret-amber-200\/90:is(.dark *){caret-color:hsla(48,97%,77%,.9)}.dark\:caret-amber-200\/95:is(.dark *){caret-color:hsla(48,97%,77%,.95)}.dark\:caret-amber-400:is(.dark *){caret-color:#fbbf24}.dark\:caret-amber-400\/0:is(.dark *){caret-color:rgba(251,191,36,0)}.dark\:caret-amber-400\/10:is(.dark *){caret-color:rgba(251,191,36,.1)}.dark\:caret-amber-400\/100:is(.dark *){caret-color:#fbbf24}.dark\:caret-amber-400\/15:is(.dark *){caret-color:rgba(251,191,36,.15)}.dark\:caret-amber-400\/20:is(.dark *){caret-color:rgba(251,191,36,.2)}.dark\:caret-amber-400\/25:is(.dark *){caret-color:rgba(251,191,36,.25)}.dark\:caret-amber-400\/30:is(.dark *){caret-color:rgba(251,191,36,.3)}.dark\:caret-amber-400\/35:is(.dark *){caret-color:rgba(251,191,36,.35)}.dark\:caret-amber-400\/40:is(.dark *){caret-color:rgba(251,191,36,.4)}.dark\:caret-amber-400\/45:is(.dark *){caret-color:rgba(251,191,36,.45)}.dark\:caret-amber-400\/5:is(.dark *){caret-color:rgba(251,191,36,.05)}.dark\:caret-amber-400\/50:is(.dark *){caret-color:rgba(251,191,36,.5)}.dark\:caret-amber-400\/55:is(.dark *){caret-color:rgba(251,191,36,.55)}.dark\:caret-amber-400\/60:is(.dark *){caret-color:rgba(251,191,36,.6)}.dark\:caret-amber-400\/65:is(.dark *){caret-color:rgba(251,191,36,.65)}.dark\:caret-amber-400\/70:is(.dark *){caret-color:rgba(251,191,36,.7)}.dark\:caret-amber-400\/75:is(.dark *){caret-color:rgba(251,191,36,.75)}.dark\:caret-amber-400\/80:is(.dark *){caret-color:rgba(251,191,36,.8)}.dark\:caret-amber-400\/85:is(.dark *){caret-color:rgba(251,191,36,.85)}.dark\:caret-amber-400\/90:is(.dark *){caret-color:rgba(251,191,36,.9)}.dark\:caret-amber-400\/95:is(.dark *){caret-color:rgba(251,191,36,.95)}.dark\:caret-amber-50:is(.dark *){caret-color:#fffbeb}.dark\:caret-amber-50\/0:is(.dark *){caret-color:rgba(255,251,235,0)}.dark\:caret-amber-50\/10:is(.dark *){caret-color:rgba(255,251,235,.1)}.dark\:caret-amber-50\/100:is(.dark *){caret-color:#fffbeb}.dark\:caret-amber-50\/15:is(.dark *){caret-color:rgba(255,251,235,.15)}.dark\:caret-amber-50\/20:is(.dark *){caret-color:rgba(255,251,235,.2)}.dark\:caret-amber-50\/25:is(.dark *){caret-color:rgba(255,251,235,.25)}.dark\:caret-amber-50\/30:is(.dark *){caret-color:rgba(255,251,235,.3)}.dark\:caret-amber-50\/35:is(.dark *){caret-color:rgba(255,251,235,.35)}.dark\:caret-amber-50\/40:is(.dark *){caret-color:rgba(255,251,235,.4)}.dark\:caret-amber-50\/45:is(.dark *){caret-color:rgba(255,251,235,.45)}.dark\:caret-amber-50\/5:is(.dark *){caret-color:rgba(255,251,235,.05)}.dark\:caret-amber-50\/50:is(.dark *){caret-color:rgba(255,251,235,.5)}.dark\:caret-amber-50\/55:is(.dark *){caret-color:rgba(255,251,235,.55)}.dark\:caret-amber-50\/60:is(.dark *){caret-color:rgba(255,251,235,.6)}.dark\:caret-amber-50\/65:is(.dark *){caret-color:rgba(255,251,235,.65)}.dark\:caret-amber-50\/70:is(.dark *){caret-color:rgba(255,251,235,.7)}.dark\:caret-amber-50\/75:is(.dark *){caret-color:rgba(255,251,235,.75)}.dark\:caret-amber-50\/80:is(.dark *){caret-color:rgba(255,251,235,.8)}.dark\:caret-amber-50\/85:is(.dark *){caret-color:rgba(255,251,235,.85)}.dark\:caret-amber-50\/90:is(.dark *){caret-color:rgba(255,251,235,.9)}.dark\:caret-amber-50\/95:is(.dark *){caret-color:rgba(255,251,235,.95)}.dark\:caret-amber-500:is(.dark *){caret-color:#f59e0b}.dark\:caret-amber-500\/0:is(.dark *){caret-color:rgba(245,158,11,0)}.dark\:caret-amber-500\/10:is(.dark *){caret-color:rgba(245,158,11,.1)}.dark\:caret-amber-500\/100:is(.dark *){caret-color:#f59e0b}.dark\:caret-amber-500\/15:is(.dark *){caret-color:rgba(245,158,11,.15)}.dark\:caret-amber-500\/20:is(.dark *){caret-color:rgba(245,158,11,.2)}.dark\:caret-amber-500\/25:is(.dark *){caret-color:rgba(245,158,11,.25)}.dark\:caret-amber-500\/30:is(.dark *){caret-color:rgba(245,158,11,.3)}.dark\:caret-amber-500\/35:is(.dark *){caret-color:rgba(245,158,11,.35)}.dark\:caret-amber-500\/40:is(.dark *){caret-color:rgba(245,158,11,.4)}.dark\:caret-amber-500\/45:is(.dark *){caret-color:rgba(245,158,11,.45)}.dark\:caret-amber-500\/5:is(.dark *){caret-color:rgba(245,158,11,.05)}.dark\:caret-amber-500\/50:is(.dark *){caret-color:rgba(245,158,11,.5)}.dark\:caret-amber-500\/55:is(.dark *){caret-color:rgba(245,158,11,.55)}.dark\:caret-amber-500\/60:is(.dark *){caret-color:rgba(245,158,11,.6)}.dark\:caret-amber-500\/65:is(.dark *){caret-color:rgba(245,158,11,.65)}.dark\:caret-amber-500\/70:is(.dark *){caret-color:rgba(245,158,11,.7)}.dark\:caret-amber-500\/75:is(.dark *){caret-color:rgba(245,158,11,.75)}.dark\:caret-amber-500\/80:is(.dark *){caret-color:rgba(245,158,11,.8)}.dark\:caret-amber-500\/85:is(.dark *){caret-color:rgba(245,158,11,.85)}.dark\:caret-amber-500\/90:is(.dark *){caret-color:rgba(245,158,11,.9)}.dark\:caret-amber-500\/95:is(.dark *){caret-color:rgba(245,158,11,.95)}.dark\:caret-amber-950:is(.dark *){caret-color:#451a03}.dark\:caret-amber-950\/0:is(.dark *){caret-color:rgba(69,26,3,0)}.dark\:caret-amber-950\/10:is(.dark *){caret-color:rgba(69,26,3,.1)}.dark\:caret-amber-950\/100:is(.dark *){caret-color:#451a03}.dark\:caret-amber-950\/15:is(.dark *){caret-color:rgba(69,26,3,.15)}.dark\:caret-amber-950\/20:is(.dark *){caret-color:rgba(69,26,3,.2)}.dark\:caret-amber-950\/25:is(.dark *){caret-color:rgba(69,26,3,.25)}.dark\:caret-amber-950\/30:is(.dark *){caret-color:rgba(69,26,3,.3)}.dark\:caret-amber-950\/35:is(.dark *){caret-color:rgba(69,26,3,.35)}.dark\:caret-amber-950\/40:is(.dark *){caret-color:rgba(69,26,3,.4)}.dark\:caret-amber-950\/45:is(.dark *){caret-color:rgba(69,26,3,.45)}.dark\:caret-amber-950\/5:is(.dark *){caret-color:rgba(69,26,3,.05)}.dark\:caret-amber-950\/50:is(.dark *){caret-color:rgba(69,26,3,.5)}.dark\:caret-amber-950\/55:is(.dark *){caret-color:rgba(69,26,3,.55)}.dark\:caret-amber-950\/60:is(.dark *){caret-color:rgba(69,26,3,.6)}.dark\:caret-amber-950\/65:is(.dark *){caret-color:rgba(69,26,3,.65)}.dark\:caret-amber-950\/70:is(.dark *){caret-color:rgba(69,26,3,.7)}.dark\:caret-amber-950\/75:is(.dark *){caret-color:rgba(69,26,3,.75)}.dark\:caret-amber-950\/80:is(.dark *){caret-color:rgba(69,26,3,.8)}.dark\:caret-amber-950\/85:is(.dark *){caret-color:rgba(69,26,3,.85)}.dark\:caret-amber-950\/90:is(.dark *){caret-color:rgba(69,26,3,.9)}.dark\:caret-amber-950\/95:is(.dark *){caret-color:rgba(69,26,3,.95)}.dark\:caret-blue-100:is(.dark *){caret-color:#dbeafe}.dark\:caret-blue-100\/0:is(.dark *){caret-color:rgba(219,234,254,0)}.dark\:caret-blue-100\/10:is(.dark *){caret-color:rgba(219,234,254,.1)}.dark\:caret-blue-100\/100:is(.dark *){caret-color:#dbeafe}.dark\:caret-blue-100\/15:is(.dark *){caret-color:rgba(219,234,254,.15)}.dark\:caret-blue-100\/20:is(.dark *){caret-color:rgba(219,234,254,.2)}.dark\:caret-blue-100\/25:is(.dark *){caret-color:rgba(219,234,254,.25)}.dark\:caret-blue-100\/30:is(.dark *){caret-color:rgba(219,234,254,.3)}.dark\:caret-blue-100\/35:is(.dark *){caret-color:rgba(219,234,254,.35)}.dark\:caret-blue-100\/40:is(.dark *){caret-color:rgba(219,234,254,.4)}.dark\:caret-blue-100\/45:is(.dark *){caret-color:rgba(219,234,254,.45)}.dark\:caret-blue-100\/5:is(.dark *){caret-color:rgba(219,234,254,.05)}.dark\:caret-blue-100\/50:is(.dark *){caret-color:rgba(219,234,254,.5)}.dark\:caret-blue-100\/55:is(.dark *){caret-color:rgba(219,234,254,.55)}.dark\:caret-blue-100\/60:is(.dark *){caret-color:rgba(219,234,254,.6)}.dark\:caret-blue-100\/65:is(.dark *){caret-color:rgba(219,234,254,.65)}.dark\:caret-blue-100\/70:is(.dark *){caret-color:rgba(219,234,254,.7)}.dark\:caret-blue-100\/75:is(.dark *){caret-color:rgba(219,234,254,.75)}.dark\:caret-blue-100\/80:is(.dark *){caret-color:rgba(219,234,254,.8)}.dark\:caret-blue-100\/85:is(.dark *){caret-color:rgba(219,234,254,.85)}.dark\:caret-blue-100\/90:is(.dark *){caret-color:rgba(219,234,254,.9)}.dark\:caret-blue-100\/95:is(.dark *){caret-color:rgba(219,234,254,.95)}.dark\:caret-blue-200:is(.dark *){caret-color:#bfdbfe}.dark\:caret-blue-200\/0:is(.dark *){caret-color:rgba(191,219,254,0)}.dark\:caret-blue-200\/10:is(.dark *){caret-color:rgba(191,219,254,.1)}.dark\:caret-blue-200\/100:is(.dark *){caret-color:#bfdbfe}.dark\:caret-blue-200\/15:is(.dark *){caret-color:rgba(191,219,254,.15)}.dark\:caret-blue-200\/20:is(.dark *){caret-color:rgba(191,219,254,.2)}.dark\:caret-blue-200\/25:is(.dark *){caret-color:rgba(191,219,254,.25)}.dark\:caret-blue-200\/30:is(.dark *){caret-color:rgba(191,219,254,.3)}.dark\:caret-blue-200\/35:is(.dark *){caret-color:rgba(191,219,254,.35)}.dark\:caret-blue-200\/40:is(.dark *){caret-color:rgba(191,219,254,.4)}.dark\:caret-blue-200\/45:is(.dark *){caret-color:rgba(191,219,254,.45)}.dark\:caret-blue-200\/5:is(.dark *){caret-color:rgba(191,219,254,.05)}.dark\:caret-blue-200\/50:is(.dark *){caret-color:rgba(191,219,254,.5)}.dark\:caret-blue-200\/55:is(.dark *){caret-color:rgba(191,219,254,.55)}.dark\:caret-blue-200\/60:is(.dark *){caret-color:rgba(191,219,254,.6)}.dark\:caret-blue-200\/65:is(.dark *){caret-color:rgba(191,219,254,.65)}.dark\:caret-blue-200\/70:is(.dark *){caret-color:rgba(191,219,254,.7)}.dark\:caret-blue-200\/75:is(.dark *){caret-color:rgba(191,219,254,.75)}.dark\:caret-blue-200\/80:is(.dark *){caret-color:rgba(191,219,254,.8)}.dark\:caret-blue-200\/85:is(.dark *){caret-color:rgba(191,219,254,.85)}.dark\:caret-blue-200\/90:is(.dark *){caret-color:rgba(191,219,254,.9)}.dark\:caret-blue-200\/95:is(.dark *){caret-color:rgba(191,219,254,.95)}.dark\:caret-blue-400:is(.dark *){caret-color:#60a5fa}.dark\:caret-blue-400\/0:is(.dark *){caret-color:rgba(96,165,250,0)}.dark\:caret-blue-400\/10:is(.dark *){caret-color:rgba(96,165,250,.1)}.dark\:caret-blue-400\/100:is(.dark *){caret-color:#60a5fa}.dark\:caret-blue-400\/15:is(.dark *){caret-color:rgba(96,165,250,.15)}.dark\:caret-blue-400\/20:is(.dark *){caret-color:rgba(96,165,250,.2)}.dark\:caret-blue-400\/25:is(.dark *){caret-color:rgba(96,165,250,.25)}.dark\:caret-blue-400\/30:is(.dark *){caret-color:rgba(96,165,250,.3)}.dark\:caret-blue-400\/35:is(.dark *){caret-color:rgba(96,165,250,.35)}.dark\:caret-blue-400\/40:is(.dark *){caret-color:rgba(96,165,250,.4)}.dark\:caret-blue-400\/45:is(.dark *){caret-color:rgba(96,165,250,.45)}.dark\:caret-blue-400\/5:is(.dark *){caret-color:rgba(96,165,250,.05)}.dark\:caret-blue-400\/50:is(.dark *){caret-color:rgba(96,165,250,.5)}.dark\:caret-blue-400\/55:is(.dark *){caret-color:rgba(96,165,250,.55)}.dark\:caret-blue-400\/60:is(.dark *){caret-color:rgba(96,165,250,.6)}.dark\:caret-blue-400\/65:is(.dark *){caret-color:rgba(96,165,250,.65)}.dark\:caret-blue-400\/70:is(.dark *){caret-color:rgba(96,165,250,.7)}.dark\:caret-blue-400\/75:is(.dark *){caret-color:rgba(96,165,250,.75)}.dark\:caret-blue-400\/80:is(.dark *){caret-color:rgba(96,165,250,.8)}.dark\:caret-blue-400\/85:is(.dark *){caret-color:rgba(96,165,250,.85)}.dark\:caret-blue-400\/90:is(.dark *){caret-color:rgba(96,165,250,.9)}.dark\:caret-blue-400\/95:is(.dark *){caret-color:rgba(96,165,250,.95)}.dark\:caret-blue-50:is(.dark *){caret-color:#eff6ff}.dark\:caret-blue-50\/0:is(.dark *){caret-color:rgba(239,246,255,0)}.dark\:caret-blue-50\/10:is(.dark *){caret-color:rgba(239,246,255,.1)}.dark\:caret-blue-50\/100:is(.dark *){caret-color:#eff6ff}.dark\:caret-blue-50\/15:is(.dark *){caret-color:rgba(239,246,255,.15)}.dark\:caret-blue-50\/20:is(.dark *){caret-color:rgba(239,246,255,.2)}.dark\:caret-blue-50\/25:is(.dark *){caret-color:rgba(239,246,255,.25)}.dark\:caret-blue-50\/30:is(.dark *){caret-color:rgba(239,246,255,.3)}.dark\:caret-blue-50\/35:is(.dark *){caret-color:rgba(239,246,255,.35)}.dark\:caret-blue-50\/40:is(.dark *){caret-color:rgba(239,246,255,.4)}.dark\:caret-blue-50\/45:is(.dark *){caret-color:rgba(239,246,255,.45)}.dark\:caret-blue-50\/5:is(.dark *){caret-color:rgba(239,246,255,.05)}.dark\:caret-blue-50\/50:is(.dark *){caret-color:rgba(239,246,255,.5)}.dark\:caret-blue-50\/55:is(.dark *){caret-color:rgba(239,246,255,.55)}.dark\:caret-blue-50\/60:is(.dark *){caret-color:rgba(239,246,255,.6)}.dark\:caret-blue-50\/65:is(.dark *){caret-color:rgba(239,246,255,.65)}.dark\:caret-blue-50\/70:is(.dark *){caret-color:rgba(239,246,255,.7)}.dark\:caret-blue-50\/75:is(.dark *){caret-color:rgba(239,246,255,.75)}.dark\:caret-blue-50\/80:is(.dark *){caret-color:rgba(239,246,255,.8)}.dark\:caret-blue-50\/85:is(.dark *){caret-color:rgba(239,246,255,.85)}.dark\:caret-blue-50\/90:is(.dark *){caret-color:rgba(239,246,255,.9)}.dark\:caret-blue-50\/95:is(.dark *){caret-color:rgba(239,246,255,.95)}.dark\:caret-blue-500:is(.dark *){caret-color:#3b82f6}.dark\:caret-blue-500\/0:is(.dark *){caret-color:rgba(59,130,246,0)}.dark\:caret-blue-500\/10:is(.dark *){caret-color:rgba(59,130,246,.1)}.dark\:caret-blue-500\/100:is(.dark *){caret-color:#3b82f6}.dark\:caret-blue-500\/15:is(.dark *){caret-color:rgba(59,130,246,.15)}.dark\:caret-blue-500\/20:is(.dark *){caret-color:rgba(59,130,246,.2)}.dark\:caret-blue-500\/25:is(.dark *){caret-color:rgba(59,130,246,.25)}.dark\:caret-blue-500\/30:is(.dark *){caret-color:rgba(59,130,246,.3)}.dark\:caret-blue-500\/35:is(.dark *){caret-color:rgba(59,130,246,.35)}.dark\:caret-blue-500\/40:is(.dark *){caret-color:rgba(59,130,246,.4)}.dark\:caret-blue-500\/45:is(.dark *){caret-color:rgba(59,130,246,.45)}.dark\:caret-blue-500\/5:is(.dark *){caret-color:rgba(59,130,246,.05)}.dark\:caret-blue-500\/50:is(.dark *){caret-color:rgba(59,130,246,.5)}.dark\:caret-blue-500\/55:is(.dark *){caret-color:rgba(59,130,246,.55)}.dark\:caret-blue-500\/60:is(.dark *){caret-color:rgba(59,130,246,.6)}.dark\:caret-blue-500\/65:is(.dark *){caret-color:rgba(59,130,246,.65)}.dark\:caret-blue-500\/70:is(.dark *){caret-color:rgba(59,130,246,.7)}.dark\:caret-blue-500\/75:is(.dark *){caret-color:rgba(59,130,246,.75)}.dark\:caret-blue-500\/80:is(.dark *){caret-color:rgba(59,130,246,.8)}.dark\:caret-blue-500\/85:is(.dark *){caret-color:rgba(59,130,246,.85)}.dark\:caret-blue-500\/90:is(.dark *){caret-color:rgba(59,130,246,.9)}.dark\:caret-blue-500\/95:is(.dark *){caret-color:rgba(59,130,246,.95)}.dark\:caret-blue-950:is(.dark *){caret-color:#172554}.dark\:caret-blue-950\/0:is(.dark *){caret-color:rgba(23,37,84,0)}.dark\:caret-blue-950\/10:is(.dark *){caret-color:rgba(23,37,84,.1)}.dark\:caret-blue-950\/100:is(.dark *){caret-color:#172554}.dark\:caret-blue-950\/15:is(.dark *){caret-color:rgba(23,37,84,.15)}.dark\:caret-blue-950\/20:is(.dark *){caret-color:rgba(23,37,84,.2)}.dark\:caret-blue-950\/25:is(.dark *){caret-color:rgba(23,37,84,.25)}.dark\:caret-blue-950\/30:is(.dark *){caret-color:rgba(23,37,84,.3)}.dark\:caret-blue-950\/35:is(.dark *){caret-color:rgba(23,37,84,.35)}.dark\:caret-blue-950\/40:is(.dark *){caret-color:rgba(23,37,84,.4)}.dark\:caret-blue-950\/45:is(.dark *){caret-color:rgba(23,37,84,.45)}.dark\:caret-blue-950\/5:is(.dark *){caret-color:rgba(23,37,84,.05)}.dark\:caret-blue-950\/50:is(.dark *){caret-color:rgba(23,37,84,.5)}.dark\:caret-blue-950\/55:is(.dark *){caret-color:rgba(23,37,84,.55)}.dark\:caret-blue-950\/60:is(.dark *){caret-color:rgba(23,37,84,.6)}.dark\:caret-blue-950\/65:is(.dark *){caret-color:rgba(23,37,84,.65)}.dark\:caret-blue-950\/70:is(.dark *){caret-color:rgba(23,37,84,.7)}.dark\:caret-blue-950\/75:is(.dark *){caret-color:rgba(23,37,84,.75)}.dark\:caret-blue-950\/80:is(.dark *){caret-color:rgba(23,37,84,.8)}.dark\:caret-blue-950\/85:is(.dark *){caret-color:rgba(23,37,84,.85)}.dark\:caret-blue-950\/90:is(.dark *){caret-color:rgba(23,37,84,.9)}.dark\:caret-blue-950\/95:is(.dark *){caret-color:rgba(23,37,84,.95)}.dark\:caret-cyan-100:is(.dark *){caret-color:#cffafe}.dark\:caret-cyan-100\/0:is(.dark *){caret-color:rgba(207,250,254,0)}.dark\:caret-cyan-100\/10:is(.dark *){caret-color:rgba(207,250,254,.1)}.dark\:caret-cyan-100\/100:is(.dark *){caret-color:#cffafe}.dark\:caret-cyan-100\/15:is(.dark *){caret-color:rgba(207,250,254,.15)}.dark\:caret-cyan-100\/20:is(.dark *){caret-color:rgba(207,250,254,.2)}.dark\:caret-cyan-100\/25:is(.dark *){caret-color:rgba(207,250,254,.25)}.dark\:caret-cyan-100\/30:is(.dark *){caret-color:rgba(207,250,254,.3)}.dark\:caret-cyan-100\/35:is(.dark *){caret-color:rgba(207,250,254,.35)}.dark\:caret-cyan-100\/40:is(.dark *){caret-color:rgba(207,250,254,.4)}.dark\:caret-cyan-100\/45:is(.dark *){caret-color:rgba(207,250,254,.45)}.dark\:caret-cyan-100\/5:is(.dark *){caret-color:rgba(207,250,254,.05)}.dark\:caret-cyan-100\/50:is(.dark *){caret-color:rgba(207,250,254,.5)}.dark\:caret-cyan-100\/55:is(.dark *){caret-color:rgba(207,250,254,.55)}.dark\:caret-cyan-100\/60:is(.dark *){caret-color:rgba(207,250,254,.6)}.dark\:caret-cyan-100\/65:is(.dark *){caret-color:rgba(207,250,254,.65)}.dark\:caret-cyan-100\/70:is(.dark *){caret-color:rgba(207,250,254,.7)}.dark\:caret-cyan-100\/75:is(.dark *){caret-color:rgba(207,250,254,.75)}.dark\:caret-cyan-100\/80:is(.dark *){caret-color:rgba(207,250,254,.8)}.dark\:caret-cyan-100\/85:is(.dark *){caret-color:rgba(207,250,254,.85)}.dark\:caret-cyan-100\/90:is(.dark *){caret-color:rgba(207,250,254,.9)}.dark\:caret-cyan-100\/95:is(.dark *){caret-color:rgba(207,250,254,.95)}.dark\:caret-cyan-200:is(.dark *){caret-color:#a5f3fc}.dark\:caret-cyan-200\/0:is(.dark *){caret-color:rgba(165,243,252,0)}.dark\:caret-cyan-200\/10:is(.dark *){caret-color:rgba(165,243,252,.1)}.dark\:caret-cyan-200\/100:is(.dark *){caret-color:#a5f3fc}.dark\:caret-cyan-200\/15:is(.dark *){caret-color:rgba(165,243,252,.15)}.dark\:caret-cyan-200\/20:is(.dark *){caret-color:rgba(165,243,252,.2)}.dark\:caret-cyan-200\/25:is(.dark *){caret-color:rgba(165,243,252,.25)}.dark\:caret-cyan-200\/30:is(.dark *){caret-color:rgba(165,243,252,.3)}.dark\:caret-cyan-200\/35:is(.dark *){caret-color:rgba(165,243,252,.35)}.dark\:caret-cyan-200\/40:is(.dark *){caret-color:rgba(165,243,252,.4)}.dark\:caret-cyan-200\/45:is(.dark *){caret-color:rgba(165,243,252,.45)}.dark\:caret-cyan-200\/5:is(.dark *){caret-color:rgba(165,243,252,.05)}.dark\:caret-cyan-200\/50:is(.dark *){caret-color:rgba(165,243,252,.5)}.dark\:caret-cyan-200\/55:is(.dark *){caret-color:rgba(165,243,252,.55)}.dark\:caret-cyan-200\/60:is(.dark *){caret-color:rgba(165,243,252,.6)}.dark\:caret-cyan-200\/65:is(.dark *){caret-color:rgba(165,243,252,.65)}.dark\:caret-cyan-200\/70:is(.dark *){caret-color:rgba(165,243,252,.7)}.dark\:caret-cyan-200\/75:is(.dark *){caret-color:rgba(165,243,252,.75)}.dark\:caret-cyan-200\/80:is(.dark *){caret-color:rgba(165,243,252,.8)}.dark\:caret-cyan-200\/85:is(.dark *){caret-color:rgba(165,243,252,.85)}.dark\:caret-cyan-200\/90:is(.dark *){caret-color:rgba(165,243,252,.9)}.dark\:caret-cyan-200\/95:is(.dark *){caret-color:rgba(165,243,252,.95)}.dark\:caret-cyan-400:is(.dark *){caret-color:#22d3ee}.dark\:caret-cyan-400\/0:is(.dark *){caret-color:rgba(34,211,238,0)}.dark\:caret-cyan-400\/10:is(.dark *){caret-color:rgba(34,211,238,.1)}.dark\:caret-cyan-400\/100:is(.dark *){caret-color:#22d3ee}.dark\:caret-cyan-400\/15:is(.dark *){caret-color:rgba(34,211,238,.15)}.dark\:caret-cyan-400\/20:is(.dark *){caret-color:rgba(34,211,238,.2)}.dark\:caret-cyan-400\/25:is(.dark *){caret-color:rgba(34,211,238,.25)}.dark\:caret-cyan-400\/30:is(.dark *){caret-color:rgba(34,211,238,.3)}.dark\:caret-cyan-400\/35:is(.dark *){caret-color:rgba(34,211,238,.35)}.dark\:caret-cyan-400\/40:is(.dark *){caret-color:rgba(34,211,238,.4)}.dark\:caret-cyan-400\/45:is(.dark *){caret-color:rgba(34,211,238,.45)}.dark\:caret-cyan-400\/5:is(.dark *){caret-color:rgba(34,211,238,.05)}.dark\:caret-cyan-400\/50:is(.dark *){caret-color:rgba(34,211,238,.5)}.dark\:caret-cyan-400\/55:is(.dark *){caret-color:rgba(34,211,238,.55)}.dark\:caret-cyan-400\/60:is(.dark *){caret-color:rgba(34,211,238,.6)}.dark\:caret-cyan-400\/65:is(.dark *){caret-color:rgba(34,211,238,.65)}.dark\:caret-cyan-400\/70:is(.dark *){caret-color:rgba(34,211,238,.7)}.dark\:caret-cyan-400\/75:is(.dark *){caret-color:rgba(34,211,238,.75)}.dark\:caret-cyan-400\/80:is(.dark *){caret-color:rgba(34,211,238,.8)}.dark\:caret-cyan-400\/85:is(.dark *){caret-color:rgba(34,211,238,.85)}.dark\:caret-cyan-400\/90:is(.dark *){caret-color:rgba(34,211,238,.9)}.dark\:caret-cyan-400\/95:is(.dark *){caret-color:rgba(34,211,238,.95)}.dark\:caret-cyan-50:is(.dark *){caret-color:#ecfeff}.dark\:caret-cyan-50\/0:is(.dark *){caret-color:rgba(236,254,255,0)}.dark\:caret-cyan-50\/10:is(.dark *){caret-color:rgba(236,254,255,.1)}.dark\:caret-cyan-50\/100:is(.dark *){caret-color:#ecfeff}.dark\:caret-cyan-50\/15:is(.dark *){caret-color:rgba(236,254,255,.15)}.dark\:caret-cyan-50\/20:is(.dark *){caret-color:rgba(236,254,255,.2)}.dark\:caret-cyan-50\/25:is(.dark *){caret-color:rgba(236,254,255,.25)}.dark\:caret-cyan-50\/30:is(.dark *){caret-color:rgba(236,254,255,.3)}.dark\:caret-cyan-50\/35:is(.dark *){caret-color:rgba(236,254,255,.35)}.dark\:caret-cyan-50\/40:is(.dark *){caret-color:rgba(236,254,255,.4)}.dark\:caret-cyan-50\/45:is(.dark *){caret-color:rgba(236,254,255,.45)}.dark\:caret-cyan-50\/5:is(.dark *){caret-color:rgba(236,254,255,.05)}.dark\:caret-cyan-50\/50:is(.dark *){caret-color:rgba(236,254,255,.5)}.dark\:caret-cyan-50\/55:is(.dark *){caret-color:rgba(236,254,255,.55)}.dark\:caret-cyan-50\/60:is(.dark *){caret-color:rgba(236,254,255,.6)}.dark\:caret-cyan-50\/65:is(.dark *){caret-color:rgba(236,254,255,.65)}.dark\:caret-cyan-50\/70:is(.dark *){caret-color:rgba(236,254,255,.7)}.dark\:caret-cyan-50\/75:is(.dark *){caret-color:rgba(236,254,255,.75)}.dark\:caret-cyan-50\/80:is(.dark *){caret-color:rgba(236,254,255,.8)}.dark\:caret-cyan-50\/85:is(.dark *){caret-color:rgba(236,254,255,.85)}.dark\:caret-cyan-50\/90:is(.dark *){caret-color:rgba(236,254,255,.9)}.dark\:caret-cyan-50\/95:is(.dark *){caret-color:rgba(236,254,255,.95)}.dark\:caret-cyan-500:is(.dark *){caret-color:#06b6d4}.dark\:caret-cyan-500\/0:is(.dark *){caret-color:rgba(6,182,212,0)}.dark\:caret-cyan-500\/10:is(.dark *){caret-color:rgba(6,182,212,.1)}.dark\:caret-cyan-500\/100:is(.dark *){caret-color:#06b6d4}.dark\:caret-cyan-500\/15:is(.dark *){caret-color:rgba(6,182,212,.15)}.dark\:caret-cyan-500\/20:is(.dark *){caret-color:rgba(6,182,212,.2)}.dark\:caret-cyan-500\/25:is(.dark *){caret-color:rgba(6,182,212,.25)}.dark\:caret-cyan-500\/30:is(.dark *){caret-color:rgba(6,182,212,.3)}.dark\:caret-cyan-500\/35:is(.dark *){caret-color:rgba(6,182,212,.35)}.dark\:caret-cyan-500\/40:is(.dark *){caret-color:rgba(6,182,212,.4)}.dark\:caret-cyan-500\/45:is(.dark *){caret-color:rgba(6,182,212,.45)}.dark\:caret-cyan-500\/5:is(.dark *){caret-color:rgba(6,182,212,.05)}.dark\:caret-cyan-500\/50:is(.dark *){caret-color:rgba(6,182,212,.5)}.dark\:caret-cyan-500\/55:is(.dark *){caret-color:rgba(6,182,212,.55)}.dark\:caret-cyan-500\/60:is(.dark *){caret-color:rgba(6,182,212,.6)}.dark\:caret-cyan-500\/65:is(.dark *){caret-color:rgba(6,182,212,.65)}.dark\:caret-cyan-500\/70:is(.dark *){caret-color:rgba(6,182,212,.7)}.dark\:caret-cyan-500\/75:is(.dark *){caret-color:rgba(6,182,212,.75)}.dark\:caret-cyan-500\/80:is(.dark *){caret-color:rgba(6,182,212,.8)}.dark\:caret-cyan-500\/85:is(.dark *){caret-color:rgba(6,182,212,.85)}.dark\:caret-cyan-500\/90:is(.dark *){caret-color:rgba(6,182,212,.9)}.dark\:caret-cyan-500\/95:is(.dark *){caret-color:rgba(6,182,212,.95)}.dark\:caret-cyan-950:is(.dark *){caret-color:#083344}.dark\:caret-cyan-950\/0:is(.dark *){caret-color:rgba(8,51,68,0)}.dark\:caret-cyan-950\/10:is(.dark *){caret-color:rgba(8,51,68,.1)}.dark\:caret-cyan-950\/100:is(.dark *){caret-color:#083344}.dark\:caret-cyan-950\/15:is(.dark *){caret-color:rgba(8,51,68,.15)}.dark\:caret-cyan-950\/20:is(.dark *){caret-color:rgba(8,51,68,.2)}.dark\:caret-cyan-950\/25:is(.dark *){caret-color:rgba(8,51,68,.25)}.dark\:caret-cyan-950\/30:is(.dark *){caret-color:rgba(8,51,68,.3)}.dark\:caret-cyan-950\/35:is(.dark *){caret-color:rgba(8,51,68,.35)}.dark\:caret-cyan-950\/40:is(.dark *){caret-color:rgba(8,51,68,.4)}.dark\:caret-cyan-950\/45:is(.dark *){caret-color:rgba(8,51,68,.45)}.dark\:caret-cyan-950\/5:is(.dark *){caret-color:rgba(8,51,68,.05)}.dark\:caret-cyan-950\/50:is(.dark *){caret-color:rgba(8,51,68,.5)}.dark\:caret-cyan-950\/55:is(.dark *){caret-color:rgba(8,51,68,.55)}.dark\:caret-cyan-950\/60:is(.dark *){caret-color:rgba(8,51,68,.6)}.dark\:caret-cyan-950\/65:is(.dark *){caret-color:rgba(8,51,68,.65)}.dark\:caret-cyan-950\/70:is(.dark *){caret-color:rgba(8,51,68,.7)}.dark\:caret-cyan-950\/75:is(.dark *){caret-color:rgba(8,51,68,.75)}.dark\:caret-cyan-950\/80:is(.dark *){caret-color:rgba(8,51,68,.8)}.dark\:caret-cyan-950\/85:is(.dark *){caret-color:rgba(8,51,68,.85)}.dark\:caret-cyan-950\/90:is(.dark *){caret-color:rgba(8,51,68,.9)}.dark\:caret-cyan-950\/95:is(.dark *){caret-color:rgba(8,51,68,.95)}.dark\:caret-emerald-100:is(.dark *){caret-color:#d1fae5}.dark\:caret-emerald-100\/0:is(.dark *){caret-color:rgba(209,250,229,0)}.dark\:caret-emerald-100\/10:is(.dark *){caret-color:rgba(209,250,229,.1)}.dark\:caret-emerald-100\/100:is(.dark *){caret-color:#d1fae5}.dark\:caret-emerald-100\/15:is(.dark *){caret-color:rgba(209,250,229,.15)}.dark\:caret-emerald-100\/20:is(.dark *){caret-color:rgba(209,250,229,.2)}.dark\:caret-emerald-100\/25:is(.dark *){caret-color:rgba(209,250,229,.25)}.dark\:caret-emerald-100\/30:is(.dark *){caret-color:rgba(209,250,229,.3)}.dark\:caret-emerald-100\/35:is(.dark *){caret-color:rgba(209,250,229,.35)}.dark\:caret-emerald-100\/40:is(.dark *){caret-color:rgba(209,250,229,.4)}.dark\:caret-emerald-100\/45:is(.dark *){caret-color:rgba(209,250,229,.45)}.dark\:caret-emerald-100\/5:is(.dark *){caret-color:rgba(209,250,229,.05)}.dark\:caret-emerald-100\/50:is(.dark *){caret-color:rgba(209,250,229,.5)}.dark\:caret-emerald-100\/55:is(.dark *){caret-color:rgba(209,250,229,.55)}.dark\:caret-emerald-100\/60:is(.dark *){caret-color:rgba(209,250,229,.6)}.dark\:caret-emerald-100\/65:is(.dark *){caret-color:rgba(209,250,229,.65)}.dark\:caret-emerald-100\/70:is(.dark *){caret-color:rgba(209,250,229,.7)}.dark\:caret-emerald-100\/75:is(.dark *){caret-color:rgba(209,250,229,.75)}.dark\:caret-emerald-100\/80:is(.dark *){caret-color:rgba(209,250,229,.8)}.dark\:caret-emerald-100\/85:is(.dark *){caret-color:rgba(209,250,229,.85)}.dark\:caret-emerald-100\/90:is(.dark *){caret-color:rgba(209,250,229,.9)}.dark\:caret-emerald-100\/95:is(.dark *){caret-color:rgba(209,250,229,.95)}.dark\:caret-emerald-200:is(.dark *){caret-color:#a7f3d0}.dark\:caret-emerald-200\/0:is(.dark *){caret-color:rgba(167,243,208,0)}.dark\:caret-emerald-200\/10:is(.dark *){caret-color:rgba(167,243,208,.1)}.dark\:caret-emerald-200\/100:is(.dark *){caret-color:#a7f3d0}.dark\:caret-emerald-200\/15:is(.dark *){caret-color:rgba(167,243,208,.15)}.dark\:caret-emerald-200\/20:is(.dark *){caret-color:rgba(167,243,208,.2)}.dark\:caret-emerald-200\/25:is(.dark *){caret-color:rgba(167,243,208,.25)}.dark\:caret-emerald-200\/30:is(.dark *){caret-color:rgba(167,243,208,.3)}.dark\:caret-emerald-200\/35:is(.dark *){caret-color:rgba(167,243,208,.35)}.dark\:caret-emerald-200\/40:is(.dark *){caret-color:rgba(167,243,208,.4)}.dark\:caret-emerald-200\/45:is(.dark *){caret-color:rgba(167,243,208,.45)}.dark\:caret-emerald-200\/5:is(.dark *){caret-color:rgba(167,243,208,.05)}.dark\:caret-emerald-200\/50:is(.dark *){caret-color:rgba(167,243,208,.5)}.dark\:caret-emerald-200\/55:is(.dark *){caret-color:rgba(167,243,208,.55)}.dark\:caret-emerald-200\/60:is(.dark *){caret-color:rgba(167,243,208,.6)}.dark\:caret-emerald-200\/65:is(.dark *){caret-color:rgba(167,243,208,.65)}.dark\:caret-emerald-200\/70:is(.dark *){caret-color:rgba(167,243,208,.7)}.dark\:caret-emerald-200\/75:is(.dark *){caret-color:rgba(167,243,208,.75)}.dark\:caret-emerald-200\/80:is(.dark *){caret-color:rgba(167,243,208,.8)}.dark\:caret-emerald-200\/85:is(.dark *){caret-color:rgba(167,243,208,.85)}.dark\:caret-emerald-200\/90:is(.dark *){caret-color:rgba(167,243,208,.9)}.dark\:caret-emerald-200\/95:is(.dark *){caret-color:rgba(167,243,208,.95)}.dark\:caret-emerald-400:is(.dark *){caret-color:#34d399}.dark\:caret-emerald-400\/0:is(.dark *){caret-color:rgba(52,211,153,0)}.dark\:caret-emerald-400\/10:is(.dark *){caret-color:rgba(52,211,153,.1)}.dark\:caret-emerald-400\/100:is(.dark *){caret-color:#34d399}.dark\:caret-emerald-400\/15:is(.dark *){caret-color:rgba(52,211,153,.15)}.dark\:caret-emerald-400\/20:is(.dark *){caret-color:rgba(52,211,153,.2)}.dark\:caret-emerald-400\/25:is(.dark *){caret-color:rgba(52,211,153,.25)}.dark\:caret-emerald-400\/30:is(.dark *){caret-color:rgba(52,211,153,.3)}.dark\:caret-emerald-400\/35:is(.dark *){caret-color:rgba(52,211,153,.35)}.dark\:caret-emerald-400\/40:is(.dark *){caret-color:rgba(52,211,153,.4)}.dark\:caret-emerald-400\/45:is(.dark *){caret-color:rgba(52,211,153,.45)}.dark\:caret-emerald-400\/5:is(.dark *){caret-color:rgba(52,211,153,.05)}.dark\:caret-emerald-400\/50:is(.dark *){caret-color:rgba(52,211,153,.5)}.dark\:caret-emerald-400\/55:is(.dark *){caret-color:rgba(52,211,153,.55)}.dark\:caret-emerald-400\/60:is(.dark *){caret-color:rgba(52,211,153,.6)}.dark\:caret-emerald-400\/65:is(.dark *){caret-color:rgba(52,211,153,.65)}.dark\:caret-emerald-400\/70:is(.dark *){caret-color:rgba(52,211,153,.7)}.dark\:caret-emerald-400\/75:is(.dark *){caret-color:rgba(52,211,153,.75)}.dark\:caret-emerald-400\/80:is(.dark *){caret-color:rgba(52,211,153,.8)}.dark\:caret-emerald-400\/85:is(.dark *){caret-color:rgba(52,211,153,.85)}.dark\:caret-emerald-400\/90:is(.dark *){caret-color:rgba(52,211,153,.9)}.dark\:caret-emerald-400\/95:is(.dark *){caret-color:rgba(52,211,153,.95)}.dark\:caret-emerald-50:is(.dark *){caret-color:#ecfdf5}.dark\:caret-emerald-50\/0:is(.dark *){caret-color:rgba(236,253,245,0)}.dark\:caret-emerald-50\/10:is(.dark *){caret-color:rgba(236,253,245,.1)}.dark\:caret-emerald-50\/100:is(.dark *){caret-color:#ecfdf5}.dark\:caret-emerald-50\/15:is(.dark *){caret-color:rgba(236,253,245,.15)}.dark\:caret-emerald-50\/20:is(.dark *){caret-color:rgba(236,253,245,.2)}.dark\:caret-emerald-50\/25:is(.dark *){caret-color:rgba(236,253,245,.25)}.dark\:caret-emerald-50\/30:is(.dark *){caret-color:rgba(236,253,245,.3)}.dark\:caret-emerald-50\/35:is(.dark *){caret-color:rgba(236,253,245,.35)}.dark\:caret-emerald-50\/40:is(.dark *){caret-color:rgba(236,253,245,.4)}.dark\:caret-emerald-50\/45:is(.dark *){caret-color:rgba(236,253,245,.45)}.dark\:caret-emerald-50\/5:is(.dark *){caret-color:rgba(236,253,245,.05)}.dark\:caret-emerald-50\/50:is(.dark *){caret-color:rgba(236,253,245,.5)}.dark\:caret-emerald-50\/55:is(.dark *){caret-color:rgba(236,253,245,.55)}.dark\:caret-emerald-50\/60:is(.dark *){caret-color:rgba(236,253,245,.6)}.dark\:caret-emerald-50\/65:is(.dark *){caret-color:rgba(236,253,245,.65)}.dark\:caret-emerald-50\/70:is(.dark *){caret-color:rgba(236,253,245,.7)}.dark\:caret-emerald-50\/75:is(.dark *){caret-color:rgba(236,253,245,.75)}.dark\:caret-emerald-50\/80:is(.dark *){caret-color:rgba(236,253,245,.8)}.dark\:caret-emerald-50\/85:is(.dark *){caret-color:rgba(236,253,245,.85)}.dark\:caret-emerald-50\/90:is(.dark *){caret-color:rgba(236,253,245,.9)}.dark\:caret-emerald-50\/95:is(.dark *){caret-color:rgba(236,253,245,.95)}.dark\:caret-emerald-500:is(.dark *){caret-color:#10b981}.dark\:caret-emerald-500\/0:is(.dark *){caret-color:rgba(16,185,129,0)}.dark\:caret-emerald-500\/10:is(.dark *){caret-color:rgba(16,185,129,.1)}.dark\:caret-emerald-500\/100:is(.dark *){caret-color:#10b981}.dark\:caret-emerald-500\/15:is(.dark *){caret-color:rgba(16,185,129,.15)}.dark\:caret-emerald-500\/20:is(.dark *){caret-color:rgba(16,185,129,.2)}.dark\:caret-emerald-500\/25:is(.dark *){caret-color:rgba(16,185,129,.25)}.dark\:caret-emerald-500\/30:is(.dark *){caret-color:rgba(16,185,129,.3)}.dark\:caret-emerald-500\/35:is(.dark *){caret-color:rgba(16,185,129,.35)}.dark\:caret-emerald-500\/40:is(.dark *){caret-color:rgba(16,185,129,.4)}.dark\:caret-emerald-500\/45:is(.dark *){caret-color:rgba(16,185,129,.45)}.dark\:caret-emerald-500\/5:is(.dark *){caret-color:rgba(16,185,129,.05)}.dark\:caret-emerald-500\/50:is(.dark *){caret-color:rgba(16,185,129,.5)}.dark\:caret-emerald-500\/55:is(.dark *){caret-color:rgba(16,185,129,.55)}.dark\:caret-emerald-500\/60:is(.dark *){caret-color:rgba(16,185,129,.6)}.dark\:caret-emerald-500\/65:is(.dark *){caret-color:rgba(16,185,129,.65)}.dark\:caret-emerald-500\/70:is(.dark *){caret-color:rgba(16,185,129,.7)}.dark\:caret-emerald-500\/75:is(.dark *){caret-color:rgba(16,185,129,.75)}.dark\:caret-emerald-500\/80:is(.dark *){caret-color:rgba(16,185,129,.8)}.dark\:caret-emerald-500\/85:is(.dark *){caret-color:rgba(16,185,129,.85)}.dark\:caret-emerald-500\/90:is(.dark *){caret-color:rgba(16,185,129,.9)}.dark\:caret-emerald-500\/95:is(.dark *){caret-color:rgba(16,185,129,.95)}.dark\:caret-emerald-950:is(.dark *){caret-color:#022c22}.dark\:caret-emerald-950\/0:is(.dark *){caret-color:rgba(2,44,34,0)}.dark\:caret-emerald-950\/10:is(.dark *){caret-color:rgba(2,44,34,.1)}.dark\:caret-emerald-950\/100:is(.dark *){caret-color:#022c22}.dark\:caret-emerald-950\/15:is(.dark *){caret-color:rgba(2,44,34,.15)}.dark\:caret-emerald-950\/20:is(.dark *){caret-color:rgba(2,44,34,.2)}.dark\:caret-emerald-950\/25:is(.dark *){caret-color:rgba(2,44,34,.25)}.dark\:caret-emerald-950\/30:is(.dark *){caret-color:rgba(2,44,34,.3)}.dark\:caret-emerald-950\/35:is(.dark *){caret-color:rgba(2,44,34,.35)}.dark\:caret-emerald-950\/40:is(.dark *){caret-color:rgba(2,44,34,.4)}.dark\:caret-emerald-950\/45:is(.dark *){caret-color:rgba(2,44,34,.45)}.dark\:caret-emerald-950\/5:is(.dark *){caret-color:rgba(2,44,34,.05)}.dark\:caret-emerald-950\/50:is(.dark *){caret-color:rgba(2,44,34,.5)}.dark\:caret-emerald-950\/55:is(.dark *){caret-color:rgba(2,44,34,.55)}.dark\:caret-emerald-950\/60:is(.dark *){caret-color:rgba(2,44,34,.6)}.dark\:caret-emerald-950\/65:is(.dark *){caret-color:rgba(2,44,34,.65)}.dark\:caret-emerald-950\/70:is(.dark *){caret-color:rgba(2,44,34,.7)}.dark\:caret-emerald-950\/75:is(.dark *){caret-color:rgba(2,44,34,.75)}.dark\:caret-emerald-950\/80:is(.dark *){caret-color:rgba(2,44,34,.8)}.dark\:caret-emerald-950\/85:is(.dark *){caret-color:rgba(2,44,34,.85)}.dark\:caret-emerald-950\/90:is(.dark *){caret-color:rgba(2,44,34,.9)}.dark\:caret-emerald-950\/95:is(.dark *){caret-color:rgba(2,44,34,.95)}.dark\:caret-fuchsia-100:is(.dark *){caret-color:#fae8ff}.dark\:caret-fuchsia-100\/0:is(.dark *){caret-color:rgba(250,232,255,0)}.dark\:caret-fuchsia-100\/10:is(.dark *){caret-color:rgba(250,232,255,.1)}.dark\:caret-fuchsia-100\/100:is(.dark *){caret-color:#fae8ff}.dark\:caret-fuchsia-100\/15:is(.dark *){caret-color:rgba(250,232,255,.15)}.dark\:caret-fuchsia-100\/20:is(.dark *){caret-color:rgba(250,232,255,.2)}.dark\:caret-fuchsia-100\/25:is(.dark *){caret-color:rgba(250,232,255,.25)}.dark\:caret-fuchsia-100\/30:is(.dark *){caret-color:rgba(250,232,255,.3)}.dark\:caret-fuchsia-100\/35:is(.dark *){caret-color:rgba(250,232,255,.35)}.dark\:caret-fuchsia-100\/40:is(.dark *){caret-color:rgba(250,232,255,.4)}.dark\:caret-fuchsia-100\/45:is(.dark *){caret-color:rgba(250,232,255,.45)}.dark\:caret-fuchsia-100\/5:is(.dark *){caret-color:rgba(250,232,255,.05)}.dark\:caret-fuchsia-100\/50:is(.dark *){caret-color:rgba(250,232,255,.5)}.dark\:caret-fuchsia-100\/55:is(.dark *){caret-color:rgba(250,232,255,.55)}.dark\:caret-fuchsia-100\/60:is(.dark *){caret-color:rgba(250,232,255,.6)}.dark\:caret-fuchsia-100\/65:is(.dark *){caret-color:rgba(250,232,255,.65)}.dark\:caret-fuchsia-100\/70:is(.dark *){caret-color:rgba(250,232,255,.7)}.dark\:caret-fuchsia-100\/75:is(.dark *){caret-color:rgba(250,232,255,.75)}.dark\:caret-fuchsia-100\/80:is(.dark *){caret-color:rgba(250,232,255,.8)}.dark\:caret-fuchsia-100\/85:is(.dark *){caret-color:rgba(250,232,255,.85)}.dark\:caret-fuchsia-100\/90:is(.dark *){caret-color:rgba(250,232,255,.9)}.dark\:caret-fuchsia-100\/95:is(.dark *){caret-color:rgba(250,232,255,.95)}.dark\:caret-fuchsia-200:is(.dark *){caret-color:#f5d0fe}.dark\:caret-fuchsia-200\/0:is(.dark *){caret-color:rgba(245,208,254,0)}.dark\:caret-fuchsia-200\/10:is(.dark *){caret-color:rgba(245,208,254,.1)}.dark\:caret-fuchsia-200\/100:is(.dark *){caret-color:#f5d0fe}.dark\:caret-fuchsia-200\/15:is(.dark *){caret-color:rgba(245,208,254,.15)}.dark\:caret-fuchsia-200\/20:is(.dark *){caret-color:rgba(245,208,254,.2)}.dark\:caret-fuchsia-200\/25:is(.dark *){caret-color:rgba(245,208,254,.25)}.dark\:caret-fuchsia-200\/30:is(.dark *){caret-color:rgba(245,208,254,.3)}.dark\:caret-fuchsia-200\/35:is(.dark *){caret-color:rgba(245,208,254,.35)}.dark\:caret-fuchsia-200\/40:is(.dark *){caret-color:rgba(245,208,254,.4)}.dark\:caret-fuchsia-200\/45:is(.dark *){caret-color:rgba(245,208,254,.45)}.dark\:caret-fuchsia-200\/5:is(.dark *){caret-color:rgba(245,208,254,.05)}.dark\:caret-fuchsia-200\/50:is(.dark *){caret-color:rgba(245,208,254,.5)}.dark\:caret-fuchsia-200\/55:is(.dark *){caret-color:rgba(245,208,254,.55)}.dark\:caret-fuchsia-200\/60:is(.dark *){caret-color:rgba(245,208,254,.6)}.dark\:caret-fuchsia-200\/65:is(.dark *){caret-color:rgba(245,208,254,.65)}.dark\:caret-fuchsia-200\/70:is(.dark *){caret-color:rgba(245,208,254,.7)}.dark\:caret-fuchsia-200\/75:is(.dark *){caret-color:rgba(245,208,254,.75)}.dark\:caret-fuchsia-200\/80:is(.dark *){caret-color:rgba(245,208,254,.8)}.dark\:caret-fuchsia-200\/85:is(.dark *){caret-color:rgba(245,208,254,.85)}.dark\:caret-fuchsia-200\/90:is(.dark *){caret-color:rgba(245,208,254,.9)}.dark\:caret-fuchsia-200\/95:is(.dark *){caret-color:rgba(245,208,254,.95)}.dark\:caret-fuchsia-400:is(.dark *){caret-color:#e879f9}.dark\:caret-fuchsia-400\/0:is(.dark *){caret-color:rgba(232,121,249,0)}.dark\:caret-fuchsia-400\/10:is(.dark *){caret-color:rgba(232,121,249,.1)}.dark\:caret-fuchsia-400\/100:is(.dark *){caret-color:#e879f9}.dark\:caret-fuchsia-400\/15:is(.dark *){caret-color:rgba(232,121,249,.15)}.dark\:caret-fuchsia-400\/20:is(.dark *){caret-color:rgba(232,121,249,.2)}.dark\:caret-fuchsia-400\/25:is(.dark *){caret-color:rgba(232,121,249,.25)}.dark\:caret-fuchsia-400\/30:is(.dark *){caret-color:rgba(232,121,249,.3)}.dark\:caret-fuchsia-400\/35:is(.dark *){caret-color:rgba(232,121,249,.35)}.dark\:caret-fuchsia-400\/40:is(.dark *){caret-color:rgba(232,121,249,.4)}.dark\:caret-fuchsia-400\/45:is(.dark *){caret-color:rgba(232,121,249,.45)}.dark\:caret-fuchsia-400\/5:is(.dark *){caret-color:rgba(232,121,249,.05)}.dark\:caret-fuchsia-400\/50:is(.dark *){caret-color:rgba(232,121,249,.5)}.dark\:caret-fuchsia-400\/55:is(.dark *){caret-color:rgba(232,121,249,.55)}.dark\:caret-fuchsia-400\/60:is(.dark *){caret-color:rgba(232,121,249,.6)}.dark\:caret-fuchsia-400\/65:is(.dark *){caret-color:rgba(232,121,249,.65)}.dark\:caret-fuchsia-400\/70:is(.dark *){caret-color:rgba(232,121,249,.7)}.dark\:caret-fuchsia-400\/75:is(.dark *){caret-color:rgba(232,121,249,.75)}.dark\:caret-fuchsia-400\/80:is(.dark *){caret-color:rgba(232,121,249,.8)}.dark\:caret-fuchsia-400\/85:is(.dark *){caret-color:rgba(232,121,249,.85)}.dark\:caret-fuchsia-400\/90:is(.dark *){caret-color:rgba(232,121,249,.9)}.dark\:caret-fuchsia-400\/95:is(.dark *){caret-color:rgba(232,121,249,.95)}.dark\:caret-fuchsia-50:is(.dark *){caret-color:#fdf4ff}.dark\:caret-fuchsia-50\/0:is(.dark *){caret-color:rgba(253,244,255,0)}.dark\:caret-fuchsia-50\/10:is(.dark *){caret-color:rgba(253,244,255,.1)}.dark\:caret-fuchsia-50\/100:is(.dark *){caret-color:#fdf4ff}.dark\:caret-fuchsia-50\/15:is(.dark *){caret-color:rgba(253,244,255,.15)}.dark\:caret-fuchsia-50\/20:is(.dark *){caret-color:rgba(253,244,255,.2)}.dark\:caret-fuchsia-50\/25:is(.dark *){caret-color:rgba(253,244,255,.25)}.dark\:caret-fuchsia-50\/30:is(.dark *){caret-color:rgba(253,244,255,.3)}.dark\:caret-fuchsia-50\/35:is(.dark *){caret-color:rgba(253,244,255,.35)}.dark\:caret-fuchsia-50\/40:is(.dark *){caret-color:rgba(253,244,255,.4)}.dark\:caret-fuchsia-50\/45:is(.dark *){caret-color:rgba(253,244,255,.45)}.dark\:caret-fuchsia-50\/5:is(.dark *){caret-color:rgba(253,244,255,.05)}.dark\:caret-fuchsia-50\/50:is(.dark *){caret-color:rgba(253,244,255,.5)}.dark\:caret-fuchsia-50\/55:is(.dark *){caret-color:rgba(253,244,255,.55)}.dark\:caret-fuchsia-50\/60:is(.dark *){caret-color:rgba(253,244,255,.6)}.dark\:caret-fuchsia-50\/65:is(.dark *){caret-color:rgba(253,244,255,.65)}.dark\:caret-fuchsia-50\/70:is(.dark *){caret-color:rgba(253,244,255,.7)}.dark\:caret-fuchsia-50\/75:is(.dark *){caret-color:rgba(253,244,255,.75)}.dark\:caret-fuchsia-50\/80:is(.dark *){caret-color:rgba(253,244,255,.8)}.dark\:caret-fuchsia-50\/85:is(.dark *){caret-color:rgba(253,244,255,.85)}.dark\:caret-fuchsia-50\/90:is(.dark *){caret-color:rgba(253,244,255,.9)}.dark\:caret-fuchsia-50\/95:is(.dark *){caret-color:rgba(253,244,255,.95)}.dark\:caret-fuchsia-500:is(.dark *){caret-color:#d946ef}.dark\:caret-fuchsia-500\/0:is(.dark *){caret-color:rgba(217,70,239,0)}.dark\:caret-fuchsia-500\/10:is(.dark *){caret-color:rgba(217,70,239,.1)}.dark\:caret-fuchsia-500\/100:is(.dark *){caret-color:#d946ef}.dark\:caret-fuchsia-500\/15:is(.dark *){caret-color:rgba(217,70,239,.15)}.dark\:caret-fuchsia-500\/20:is(.dark *){caret-color:rgba(217,70,239,.2)}.dark\:caret-fuchsia-500\/25:is(.dark *){caret-color:rgba(217,70,239,.25)}.dark\:caret-fuchsia-500\/30:is(.dark *){caret-color:rgba(217,70,239,.3)}.dark\:caret-fuchsia-500\/35:is(.dark *){caret-color:rgba(217,70,239,.35)}.dark\:caret-fuchsia-500\/40:is(.dark *){caret-color:rgba(217,70,239,.4)}.dark\:caret-fuchsia-500\/45:is(.dark *){caret-color:rgba(217,70,239,.45)}.dark\:caret-fuchsia-500\/5:is(.dark *){caret-color:rgba(217,70,239,.05)}.dark\:caret-fuchsia-500\/50:is(.dark *){caret-color:rgba(217,70,239,.5)}.dark\:caret-fuchsia-500\/55:is(.dark *){caret-color:rgba(217,70,239,.55)}.dark\:caret-fuchsia-500\/60:is(.dark *){caret-color:rgba(217,70,239,.6)}.dark\:caret-fuchsia-500\/65:is(.dark *){caret-color:rgba(217,70,239,.65)}.dark\:caret-fuchsia-500\/70:is(.dark *){caret-color:rgba(217,70,239,.7)}.dark\:caret-fuchsia-500\/75:is(.dark *){caret-color:rgba(217,70,239,.75)}.dark\:caret-fuchsia-500\/80:is(.dark *){caret-color:rgba(217,70,239,.8)}.dark\:caret-fuchsia-500\/85:is(.dark *){caret-color:rgba(217,70,239,.85)}.dark\:caret-fuchsia-500\/90:is(.dark *){caret-color:rgba(217,70,239,.9)}.dark\:caret-fuchsia-500\/95:is(.dark *){caret-color:rgba(217,70,239,.95)}.dark\:caret-fuchsia-950:is(.dark *){caret-color:#4a044e}.dark\:caret-fuchsia-950\/0:is(.dark *){caret-color:rgba(74,4,78,0)}.dark\:caret-fuchsia-950\/10:is(.dark *){caret-color:rgba(74,4,78,.1)}.dark\:caret-fuchsia-950\/100:is(.dark *){caret-color:#4a044e}.dark\:caret-fuchsia-950\/15:is(.dark *){caret-color:rgba(74,4,78,.15)}.dark\:caret-fuchsia-950\/20:is(.dark *){caret-color:rgba(74,4,78,.2)}.dark\:caret-fuchsia-950\/25:is(.dark *){caret-color:rgba(74,4,78,.25)}.dark\:caret-fuchsia-950\/30:is(.dark *){caret-color:rgba(74,4,78,.3)}.dark\:caret-fuchsia-950\/35:is(.dark *){caret-color:rgba(74,4,78,.35)}.dark\:caret-fuchsia-950\/40:is(.dark *){caret-color:rgba(74,4,78,.4)}.dark\:caret-fuchsia-950\/45:is(.dark *){caret-color:rgba(74,4,78,.45)}.dark\:caret-fuchsia-950\/5:is(.dark *){caret-color:rgba(74,4,78,.05)}.dark\:caret-fuchsia-950\/50:is(.dark *){caret-color:rgba(74,4,78,.5)}.dark\:caret-fuchsia-950\/55:is(.dark *){caret-color:rgba(74,4,78,.55)}.dark\:caret-fuchsia-950\/60:is(.dark *){caret-color:rgba(74,4,78,.6)}.dark\:caret-fuchsia-950\/65:is(.dark *){caret-color:rgba(74,4,78,.65)}.dark\:caret-fuchsia-950\/70:is(.dark *){caret-color:rgba(74,4,78,.7)}.dark\:caret-fuchsia-950\/75:is(.dark *){caret-color:rgba(74,4,78,.75)}.dark\:caret-fuchsia-950\/80:is(.dark *){caret-color:rgba(74,4,78,.8)}.dark\:caret-fuchsia-950\/85:is(.dark *){caret-color:rgba(74,4,78,.85)}.dark\:caret-fuchsia-950\/90:is(.dark *){caret-color:rgba(74,4,78,.9)}.dark\:caret-fuchsia-950\/95:is(.dark *){caret-color:rgba(74,4,78,.95)}.dark\:caret-gray-100:is(.dark *){caret-color:#f3f4f6}.dark\:caret-gray-100\/0:is(.dark *){caret-color:rgba(243,244,246,0)}.dark\:caret-gray-100\/10:is(.dark *){caret-color:rgba(243,244,246,.1)}.dark\:caret-gray-100\/100:is(.dark *){caret-color:#f3f4f6}.dark\:caret-gray-100\/15:is(.dark *){caret-color:rgba(243,244,246,.15)}.dark\:caret-gray-100\/20:is(.dark *){caret-color:rgba(243,244,246,.2)}.dark\:caret-gray-100\/25:is(.dark *){caret-color:rgba(243,244,246,.25)}.dark\:caret-gray-100\/30:is(.dark *){caret-color:rgba(243,244,246,.3)}.dark\:caret-gray-100\/35:is(.dark *){caret-color:rgba(243,244,246,.35)}.dark\:caret-gray-100\/40:is(.dark *){caret-color:rgba(243,244,246,.4)}.dark\:caret-gray-100\/45:is(.dark *){caret-color:rgba(243,244,246,.45)}.dark\:caret-gray-100\/5:is(.dark *){caret-color:rgba(243,244,246,.05)}.dark\:caret-gray-100\/50:is(.dark *){caret-color:rgba(243,244,246,.5)}.dark\:caret-gray-100\/55:is(.dark *){caret-color:rgba(243,244,246,.55)}.dark\:caret-gray-100\/60:is(.dark *){caret-color:rgba(243,244,246,.6)}.dark\:caret-gray-100\/65:is(.dark *){caret-color:rgba(243,244,246,.65)}.dark\:caret-gray-100\/70:is(.dark *){caret-color:rgba(243,244,246,.7)}.dark\:caret-gray-100\/75:is(.dark *){caret-color:rgba(243,244,246,.75)}.dark\:caret-gray-100\/80:is(.dark *){caret-color:rgba(243,244,246,.8)}.dark\:caret-gray-100\/85:is(.dark *){caret-color:rgba(243,244,246,.85)}.dark\:caret-gray-100\/90:is(.dark *){caret-color:rgba(243,244,246,.9)}.dark\:caret-gray-100\/95:is(.dark *){caret-color:rgba(243,244,246,.95)}.dark\:caret-gray-200:is(.dark *){caret-color:#e5e7eb}.dark\:caret-gray-200\/0:is(.dark *){caret-color:rgba(229,231,235,0)}.dark\:caret-gray-200\/10:is(.dark *){caret-color:rgba(229,231,235,.1)}.dark\:caret-gray-200\/100:is(.dark *){caret-color:#e5e7eb}.dark\:caret-gray-200\/15:is(.dark *){caret-color:rgba(229,231,235,.15)}.dark\:caret-gray-200\/20:is(.dark *){caret-color:rgba(229,231,235,.2)}.dark\:caret-gray-200\/25:is(.dark *){caret-color:rgba(229,231,235,.25)}.dark\:caret-gray-200\/30:is(.dark *){caret-color:rgba(229,231,235,.3)}.dark\:caret-gray-200\/35:is(.dark *){caret-color:rgba(229,231,235,.35)}.dark\:caret-gray-200\/40:is(.dark *){caret-color:rgba(229,231,235,.4)}.dark\:caret-gray-200\/45:is(.dark *){caret-color:rgba(229,231,235,.45)}.dark\:caret-gray-200\/5:is(.dark *){caret-color:rgba(229,231,235,.05)}.dark\:caret-gray-200\/50:is(.dark *){caret-color:rgba(229,231,235,.5)}.dark\:caret-gray-200\/55:is(.dark *){caret-color:rgba(229,231,235,.55)}.dark\:caret-gray-200\/60:is(.dark *){caret-color:rgba(229,231,235,.6)}.dark\:caret-gray-200\/65:is(.dark *){caret-color:rgba(229,231,235,.65)}.dark\:caret-gray-200\/70:is(.dark *){caret-color:rgba(229,231,235,.7)}.dark\:caret-gray-200\/75:is(.dark *){caret-color:rgba(229,231,235,.75)}.dark\:caret-gray-200\/80:is(.dark *){caret-color:rgba(229,231,235,.8)}.dark\:caret-gray-200\/85:is(.dark *){caret-color:rgba(229,231,235,.85)}.dark\:caret-gray-200\/90:is(.dark *){caret-color:rgba(229,231,235,.9)}.dark\:caret-gray-200\/95:is(.dark *){caret-color:rgba(229,231,235,.95)}.dark\:caret-gray-400:is(.dark *){caret-color:#9ca3af}.dark\:caret-gray-400\/0:is(.dark *){caret-color:rgba(156,163,175,0)}.dark\:caret-gray-400\/10:is(.dark *){caret-color:rgba(156,163,175,.1)}.dark\:caret-gray-400\/100:is(.dark *){caret-color:#9ca3af}.dark\:caret-gray-400\/15:is(.dark *){caret-color:rgba(156,163,175,.15)}.dark\:caret-gray-400\/20:is(.dark *){caret-color:rgba(156,163,175,.2)}.dark\:caret-gray-400\/25:is(.dark *){caret-color:rgba(156,163,175,.25)}.dark\:caret-gray-400\/30:is(.dark *){caret-color:rgba(156,163,175,.3)}.dark\:caret-gray-400\/35:is(.dark *){caret-color:rgba(156,163,175,.35)}.dark\:caret-gray-400\/40:is(.dark *){caret-color:rgba(156,163,175,.4)}.dark\:caret-gray-400\/45:is(.dark *){caret-color:rgba(156,163,175,.45)}.dark\:caret-gray-400\/5:is(.dark *){caret-color:rgba(156,163,175,.05)}.dark\:caret-gray-400\/50:is(.dark *){caret-color:rgba(156,163,175,.5)}.dark\:caret-gray-400\/55:is(.dark *){caret-color:rgba(156,163,175,.55)}.dark\:caret-gray-400\/60:is(.dark *){caret-color:rgba(156,163,175,.6)}.dark\:caret-gray-400\/65:is(.dark *){caret-color:rgba(156,163,175,.65)}.dark\:caret-gray-400\/70:is(.dark *){caret-color:rgba(156,163,175,.7)}.dark\:caret-gray-400\/75:is(.dark *){caret-color:rgba(156,163,175,.75)}.dark\:caret-gray-400\/80:is(.dark *){caret-color:rgba(156,163,175,.8)}.dark\:caret-gray-400\/85:is(.dark *){caret-color:rgba(156,163,175,.85)}.dark\:caret-gray-400\/90:is(.dark *){caret-color:rgba(156,163,175,.9)}.dark\:caret-gray-400\/95:is(.dark *){caret-color:rgba(156,163,175,.95)}.dark\:caret-gray-50:is(.dark *){caret-color:#f9fafb}.dark\:caret-gray-50\/0:is(.dark *){caret-color:rgba(249,250,251,0)}.dark\:caret-gray-50\/10:is(.dark *){caret-color:rgba(249,250,251,.1)}.dark\:caret-gray-50\/100:is(.dark *){caret-color:#f9fafb}.dark\:caret-gray-50\/15:is(.dark *){caret-color:rgba(249,250,251,.15)}.dark\:caret-gray-50\/20:is(.dark *){caret-color:rgba(249,250,251,.2)}.dark\:caret-gray-50\/25:is(.dark *){caret-color:rgba(249,250,251,.25)}.dark\:caret-gray-50\/30:is(.dark *){caret-color:rgba(249,250,251,.3)}.dark\:caret-gray-50\/35:is(.dark *){caret-color:rgba(249,250,251,.35)}.dark\:caret-gray-50\/40:is(.dark *){caret-color:rgba(249,250,251,.4)}.dark\:caret-gray-50\/45:is(.dark *){caret-color:rgba(249,250,251,.45)}.dark\:caret-gray-50\/5:is(.dark *){caret-color:rgba(249,250,251,.05)}.dark\:caret-gray-50\/50:is(.dark *){caret-color:rgba(249,250,251,.5)}.dark\:caret-gray-50\/55:is(.dark *){caret-color:rgba(249,250,251,.55)}.dark\:caret-gray-50\/60:is(.dark *){caret-color:rgba(249,250,251,.6)}.dark\:caret-gray-50\/65:is(.dark *){caret-color:rgba(249,250,251,.65)}.dark\:caret-gray-50\/70:is(.dark *){caret-color:rgba(249,250,251,.7)}.dark\:caret-gray-50\/75:is(.dark *){caret-color:rgba(249,250,251,.75)}.dark\:caret-gray-50\/80:is(.dark *){caret-color:rgba(249,250,251,.8)}.dark\:caret-gray-50\/85:is(.dark *){caret-color:rgba(249,250,251,.85)}.dark\:caret-gray-50\/90:is(.dark *){caret-color:rgba(249,250,251,.9)}.dark\:caret-gray-50\/95:is(.dark *){caret-color:rgba(249,250,251,.95)}.dark\:caret-gray-500:is(.dark *){caret-color:#6b7280}.dark\:caret-gray-500\/0:is(.dark *){caret-color:hsla(220,9%,46%,0)}.dark\:caret-gray-500\/10:is(.dark *){caret-color:hsla(220,9%,46%,.1)}.dark\:caret-gray-500\/100:is(.dark *){caret-color:#6b7280}.dark\:caret-gray-500\/15:is(.dark *){caret-color:hsla(220,9%,46%,.15)}.dark\:caret-gray-500\/20:is(.dark *){caret-color:hsla(220,9%,46%,.2)}.dark\:caret-gray-500\/25:is(.dark *){caret-color:hsla(220,9%,46%,.25)}.dark\:caret-gray-500\/30:is(.dark *){caret-color:hsla(220,9%,46%,.3)}.dark\:caret-gray-500\/35:is(.dark *){caret-color:hsla(220,9%,46%,.35)}.dark\:caret-gray-500\/40:is(.dark *){caret-color:hsla(220,9%,46%,.4)}.dark\:caret-gray-500\/45:is(.dark *){caret-color:hsla(220,9%,46%,.45)}.dark\:caret-gray-500\/5:is(.dark *){caret-color:hsla(220,9%,46%,.05)}.dark\:caret-gray-500\/50:is(.dark *){caret-color:hsla(220,9%,46%,.5)}.dark\:caret-gray-500\/55:is(.dark *){caret-color:hsla(220,9%,46%,.55)}.dark\:caret-gray-500\/60:is(.dark *){caret-color:hsla(220,9%,46%,.6)}.dark\:caret-gray-500\/65:is(.dark *){caret-color:hsla(220,9%,46%,.65)}.dark\:caret-gray-500\/70:is(.dark *){caret-color:hsla(220,9%,46%,.7)}.dark\:caret-gray-500\/75:is(.dark *){caret-color:hsla(220,9%,46%,.75)}.dark\:caret-gray-500\/80:is(.dark *){caret-color:hsla(220,9%,46%,.8)}.dark\:caret-gray-500\/85:is(.dark *){caret-color:hsla(220,9%,46%,.85)}.dark\:caret-gray-500\/90:is(.dark *){caret-color:hsla(220,9%,46%,.9)}.dark\:caret-gray-500\/95:is(.dark *){caret-color:hsla(220,9%,46%,.95)}.dark\:caret-gray-950:is(.dark *){caret-color:#030712}.dark\:caret-gray-950\/0:is(.dark *){caret-color:rgba(3,7,18,0)}.dark\:caret-gray-950\/10:is(.dark *){caret-color:rgba(3,7,18,.1)}.dark\:caret-gray-950\/100:is(.dark *){caret-color:#030712}.dark\:caret-gray-950\/15:is(.dark *){caret-color:rgba(3,7,18,.15)}.dark\:caret-gray-950\/20:is(.dark *){caret-color:rgba(3,7,18,.2)}.dark\:caret-gray-950\/25:is(.dark *){caret-color:rgba(3,7,18,.25)}.dark\:caret-gray-950\/30:is(.dark *){caret-color:rgba(3,7,18,.3)}.dark\:caret-gray-950\/35:is(.dark *){caret-color:rgba(3,7,18,.35)}.dark\:caret-gray-950\/40:is(.dark *){caret-color:rgba(3,7,18,.4)}.dark\:caret-gray-950\/45:is(.dark *){caret-color:rgba(3,7,18,.45)}.dark\:caret-gray-950\/5:is(.dark *){caret-color:rgba(3,7,18,.05)}.dark\:caret-gray-950\/50:is(.dark *){caret-color:rgba(3,7,18,.5)}.dark\:caret-gray-950\/55:is(.dark *){caret-color:rgba(3,7,18,.55)}.dark\:caret-gray-950\/60:is(.dark *){caret-color:rgba(3,7,18,.6)}.dark\:caret-gray-950\/65:is(.dark *){caret-color:rgba(3,7,18,.65)}.dark\:caret-gray-950\/70:is(.dark *){caret-color:rgba(3,7,18,.7)}.dark\:caret-gray-950\/75:is(.dark *){caret-color:rgba(3,7,18,.75)}.dark\:caret-gray-950\/80:is(.dark *){caret-color:rgba(3,7,18,.8)}.dark\:caret-gray-950\/85:is(.dark *){caret-color:rgba(3,7,18,.85)}.dark\:caret-gray-950\/90:is(.dark *){caret-color:rgba(3,7,18,.9)}.dark\:caret-gray-950\/95:is(.dark *){caret-color:rgba(3,7,18,.95)}.dark\:caret-green-100:is(.dark *){caret-color:#dcfce7}.dark\:caret-green-100\/0:is(.dark *){caret-color:rgba(220,252,231,0)}.dark\:caret-green-100\/10:is(.dark *){caret-color:rgba(220,252,231,.1)}.dark\:caret-green-100\/100:is(.dark *){caret-color:#dcfce7}.dark\:caret-green-100\/15:is(.dark *){caret-color:rgba(220,252,231,.15)}.dark\:caret-green-100\/20:is(.dark *){caret-color:rgba(220,252,231,.2)}.dark\:caret-green-100\/25:is(.dark *){caret-color:rgba(220,252,231,.25)}.dark\:caret-green-100\/30:is(.dark *){caret-color:rgba(220,252,231,.3)}.dark\:caret-green-100\/35:is(.dark *){caret-color:rgba(220,252,231,.35)}.dark\:caret-green-100\/40:is(.dark *){caret-color:rgba(220,252,231,.4)}.dark\:caret-green-100\/45:is(.dark *){caret-color:rgba(220,252,231,.45)}.dark\:caret-green-100\/5:is(.dark *){caret-color:rgba(220,252,231,.05)}.dark\:caret-green-100\/50:is(.dark *){caret-color:rgba(220,252,231,.5)}.dark\:caret-green-100\/55:is(.dark *){caret-color:rgba(220,252,231,.55)}.dark\:caret-green-100\/60:is(.dark *){caret-color:rgba(220,252,231,.6)}.dark\:caret-green-100\/65:is(.dark *){caret-color:rgba(220,252,231,.65)}.dark\:caret-green-100\/70:is(.dark *){caret-color:rgba(220,252,231,.7)}.dark\:caret-green-100\/75:is(.dark *){caret-color:rgba(220,252,231,.75)}.dark\:caret-green-100\/80:is(.dark *){caret-color:rgba(220,252,231,.8)}.dark\:caret-green-100\/85:is(.dark *){caret-color:rgba(220,252,231,.85)}.dark\:caret-green-100\/90:is(.dark *){caret-color:rgba(220,252,231,.9)}.dark\:caret-green-100\/95:is(.dark *){caret-color:rgba(220,252,231,.95)}.dark\:caret-green-200:is(.dark *){caret-color:#bbf7d0}.dark\:caret-green-200\/0:is(.dark *){caret-color:rgba(187,247,208,0)}.dark\:caret-green-200\/10:is(.dark *){caret-color:rgba(187,247,208,.1)}.dark\:caret-green-200\/100:is(.dark *){caret-color:#bbf7d0}.dark\:caret-green-200\/15:is(.dark *){caret-color:rgba(187,247,208,.15)}.dark\:caret-green-200\/20:is(.dark *){caret-color:rgba(187,247,208,.2)}.dark\:caret-green-200\/25:is(.dark *){caret-color:rgba(187,247,208,.25)}.dark\:caret-green-200\/30:is(.dark *){caret-color:rgba(187,247,208,.3)}.dark\:caret-green-200\/35:is(.dark *){caret-color:rgba(187,247,208,.35)}.dark\:caret-green-200\/40:is(.dark *){caret-color:rgba(187,247,208,.4)}.dark\:caret-green-200\/45:is(.dark *){caret-color:rgba(187,247,208,.45)}.dark\:caret-green-200\/5:is(.dark *){caret-color:rgba(187,247,208,.05)}.dark\:caret-green-200\/50:is(.dark *){caret-color:rgba(187,247,208,.5)}.dark\:caret-green-200\/55:is(.dark *){caret-color:rgba(187,247,208,.55)}.dark\:caret-green-200\/60:is(.dark *){caret-color:rgba(187,247,208,.6)}.dark\:caret-green-200\/65:is(.dark *){caret-color:rgba(187,247,208,.65)}.dark\:caret-green-200\/70:is(.dark *){caret-color:rgba(187,247,208,.7)}.dark\:caret-green-200\/75:is(.dark *){caret-color:rgba(187,247,208,.75)}.dark\:caret-green-200\/80:is(.dark *){caret-color:rgba(187,247,208,.8)}.dark\:caret-green-200\/85:is(.dark *){caret-color:rgba(187,247,208,.85)}.dark\:caret-green-200\/90:is(.dark *){caret-color:rgba(187,247,208,.9)}.dark\:caret-green-200\/95:is(.dark *){caret-color:rgba(187,247,208,.95)}.dark\:caret-green-400:is(.dark *){caret-color:#4ade80}.dark\:caret-green-400\/0:is(.dark *){caret-color:rgba(74,222,128,0)}.dark\:caret-green-400\/10:is(.dark *){caret-color:rgba(74,222,128,.1)}.dark\:caret-green-400\/100:is(.dark *){caret-color:#4ade80}.dark\:caret-green-400\/15:is(.dark *){caret-color:rgba(74,222,128,.15)}.dark\:caret-green-400\/20:is(.dark *){caret-color:rgba(74,222,128,.2)}.dark\:caret-green-400\/25:is(.dark *){caret-color:rgba(74,222,128,.25)}.dark\:caret-green-400\/30:is(.dark *){caret-color:rgba(74,222,128,.3)}.dark\:caret-green-400\/35:is(.dark *){caret-color:rgba(74,222,128,.35)}.dark\:caret-green-400\/40:is(.dark *){caret-color:rgba(74,222,128,.4)}.dark\:caret-green-400\/45:is(.dark *){caret-color:rgba(74,222,128,.45)}.dark\:caret-green-400\/5:is(.dark *){caret-color:rgba(74,222,128,.05)}.dark\:caret-green-400\/50:is(.dark *){caret-color:rgba(74,222,128,.5)}.dark\:caret-green-400\/55:is(.dark *){caret-color:rgba(74,222,128,.55)}.dark\:caret-green-400\/60:is(.dark *){caret-color:rgba(74,222,128,.6)}.dark\:caret-green-400\/65:is(.dark *){caret-color:rgba(74,222,128,.65)}.dark\:caret-green-400\/70:is(.dark *){caret-color:rgba(74,222,128,.7)}.dark\:caret-green-400\/75:is(.dark *){caret-color:rgba(74,222,128,.75)}.dark\:caret-green-400\/80:is(.dark *){caret-color:rgba(74,222,128,.8)}.dark\:caret-green-400\/85:is(.dark *){caret-color:rgba(74,222,128,.85)}.dark\:caret-green-400\/90:is(.dark *){caret-color:rgba(74,222,128,.9)}.dark\:caret-green-400\/95:is(.dark *){caret-color:rgba(74,222,128,.95)}.dark\:caret-green-50:is(.dark *){caret-color:#f0fdf4}.dark\:caret-green-50\/0:is(.dark *){caret-color:rgba(240,253,244,0)}.dark\:caret-green-50\/10:is(.dark *){caret-color:rgba(240,253,244,.1)}.dark\:caret-green-50\/100:is(.dark *){caret-color:#f0fdf4}.dark\:caret-green-50\/15:is(.dark *){caret-color:rgba(240,253,244,.15)}.dark\:caret-green-50\/20:is(.dark *){caret-color:rgba(240,253,244,.2)}.dark\:caret-green-50\/25:is(.dark *){caret-color:rgba(240,253,244,.25)}.dark\:caret-green-50\/30:is(.dark *){caret-color:rgba(240,253,244,.3)}.dark\:caret-green-50\/35:is(.dark *){caret-color:rgba(240,253,244,.35)}.dark\:caret-green-50\/40:is(.dark *){caret-color:rgba(240,253,244,.4)}.dark\:caret-green-50\/45:is(.dark *){caret-color:rgba(240,253,244,.45)}.dark\:caret-green-50\/5:is(.dark *){caret-color:rgba(240,253,244,.05)}.dark\:caret-green-50\/50:is(.dark *){caret-color:rgba(240,253,244,.5)}.dark\:caret-green-50\/55:is(.dark *){caret-color:rgba(240,253,244,.55)}.dark\:caret-green-50\/60:is(.dark *){caret-color:rgba(240,253,244,.6)}.dark\:caret-green-50\/65:is(.dark *){caret-color:rgba(240,253,244,.65)}.dark\:caret-green-50\/70:is(.dark *){caret-color:rgba(240,253,244,.7)}.dark\:caret-green-50\/75:is(.dark *){caret-color:rgba(240,253,244,.75)}.dark\:caret-green-50\/80:is(.dark *){caret-color:rgba(240,253,244,.8)}.dark\:caret-green-50\/85:is(.dark *){caret-color:rgba(240,253,244,.85)}.dark\:caret-green-50\/90:is(.dark *){caret-color:rgba(240,253,244,.9)}.dark\:caret-green-50\/95:is(.dark *){caret-color:rgba(240,253,244,.95)}.dark\:caret-green-500:is(.dark *){caret-color:#22c55e}.dark\:caret-green-500\/0:is(.dark *){caret-color:rgba(34,197,94,0)}.dark\:caret-green-500\/10:is(.dark *){caret-color:rgba(34,197,94,.1)}.dark\:caret-green-500\/100:is(.dark *){caret-color:#22c55e}.dark\:caret-green-500\/15:is(.dark *){caret-color:rgba(34,197,94,.15)}.dark\:caret-green-500\/20:is(.dark *){caret-color:rgba(34,197,94,.2)}.dark\:caret-green-500\/25:is(.dark *){caret-color:rgba(34,197,94,.25)}.dark\:caret-green-500\/30:is(.dark *){caret-color:rgba(34,197,94,.3)}.dark\:caret-green-500\/35:is(.dark *){caret-color:rgba(34,197,94,.35)}.dark\:caret-green-500\/40:is(.dark *){caret-color:rgba(34,197,94,.4)}.dark\:caret-green-500\/45:is(.dark *){caret-color:rgba(34,197,94,.45)}.dark\:caret-green-500\/5:is(.dark *){caret-color:rgba(34,197,94,.05)}.dark\:caret-green-500\/50:is(.dark *){caret-color:rgba(34,197,94,.5)}.dark\:caret-green-500\/55:is(.dark *){caret-color:rgba(34,197,94,.55)}.dark\:caret-green-500\/60:is(.dark *){caret-color:rgba(34,197,94,.6)}.dark\:caret-green-500\/65:is(.dark *){caret-color:rgba(34,197,94,.65)}.dark\:caret-green-500\/70:is(.dark *){caret-color:rgba(34,197,94,.7)}.dark\:caret-green-500\/75:is(.dark *){caret-color:rgba(34,197,94,.75)}.dark\:caret-green-500\/80:is(.dark *){caret-color:rgba(34,197,94,.8)}.dark\:caret-green-500\/85:is(.dark *){caret-color:rgba(34,197,94,.85)}.dark\:caret-green-500\/90:is(.dark *){caret-color:rgba(34,197,94,.9)}.dark\:caret-green-500\/95:is(.dark *){caret-color:rgba(34,197,94,.95)}.dark\:caret-green-950:is(.dark *){caret-color:#052e16}.dark\:caret-green-950\/0:is(.dark *){caret-color:rgba(5,46,22,0)}.dark\:caret-green-950\/10:is(.dark *){caret-color:rgba(5,46,22,.1)}.dark\:caret-green-950\/100:is(.dark *){caret-color:#052e16}.dark\:caret-green-950\/15:is(.dark *){caret-color:rgba(5,46,22,.15)}.dark\:caret-green-950\/20:is(.dark *){caret-color:rgba(5,46,22,.2)}.dark\:caret-green-950\/25:is(.dark *){caret-color:rgba(5,46,22,.25)}.dark\:caret-green-950\/30:is(.dark *){caret-color:rgba(5,46,22,.3)}.dark\:caret-green-950\/35:is(.dark *){caret-color:rgba(5,46,22,.35)}.dark\:caret-green-950\/40:is(.dark *){caret-color:rgba(5,46,22,.4)}.dark\:caret-green-950\/45:is(.dark *){caret-color:rgba(5,46,22,.45)}.dark\:caret-green-950\/5:is(.dark *){caret-color:rgba(5,46,22,.05)}.dark\:caret-green-950\/50:is(.dark *){caret-color:rgba(5,46,22,.5)}.dark\:caret-green-950\/55:is(.dark *){caret-color:rgba(5,46,22,.55)}.dark\:caret-green-950\/60:is(.dark *){caret-color:rgba(5,46,22,.6)}.dark\:caret-green-950\/65:is(.dark *){caret-color:rgba(5,46,22,.65)}.dark\:caret-green-950\/70:is(.dark *){caret-color:rgba(5,46,22,.7)}.dark\:caret-green-950\/75:is(.dark *){caret-color:rgba(5,46,22,.75)}.dark\:caret-green-950\/80:is(.dark *){caret-color:rgba(5,46,22,.8)}.dark\:caret-green-950\/85:is(.dark *){caret-color:rgba(5,46,22,.85)}.dark\:caret-green-950\/90:is(.dark *){caret-color:rgba(5,46,22,.9)}.dark\:caret-green-950\/95:is(.dark *){caret-color:rgba(5,46,22,.95)}.dark\:caret-indigo-100:is(.dark *){caret-color:#e0e7ff}.dark\:caret-indigo-100\/0:is(.dark *){caret-color:rgba(224,231,255,0)}.dark\:caret-indigo-100\/10:is(.dark *){caret-color:rgba(224,231,255,.1)}.dark\:caret-indigo-100\/100:is(.dark *){caret-color:#e0e7ff}.dark\:caret-indigo-100\/15:is(.dark *){caret-color:rgba(224,231,255,.15)}.dark\:caret-indigo-100\/20:is(.dark *){caret-color:rgba(224,231,255,.2)}.dark\:caret-indigo-100\/25:is(.dark *){caret-color:rgba(224,231,255,.25)}.dark\:caret-indigo-100\/30:is(.dark *){caret-color:rgba(224,231,255,.3)}.dark\:caret-indigo-100\/35:is(.dark *){caret-color:rgba(224,231,255,.35)}.dark\:caret-indigo-100\/40:is(.dark *){caret-color:rgba(224,231,255,.4)}.dark\:caret-indigo-100\/45:is(.dark *){caret-color:rgba(224,231,255,.45)}.dark\:caret-indigo-100\/5:is(.dark *){caret-color:rgba(224,231,255,.05)}.dark\:caret-indigo-100\/50:is(.dark *){caret-color:rgba(224,231,255,.5)}.dark\:caret-indigo-100\/55:is(.dark *){caret-color:rgba(224,231,255,.55)}.dark\:caret-indigo-100\/60:is(.dark *){caret-color:rgba(224,231,255,.6)}.dark\:caret-indigo-100\/65:is(.dark *){caret-color:rgba(224,231,255,.65)}.dark\:caret-indigo-100\/70:is(.dark *){caret-color:rgba(224,231,255,.7)}.dark\:caret-indigo-100\/75:is(.dark *){caret-color:rgba(224,231,255,.75)}.dark\:caret-indigo-100\/80:is(.dark *){caret-color:rgba(224,231,255,.8)}.dark\:caret-indigo-100\/85:is(.dark *){caret-color:rgba(224,231,255,.85)}.dark\:caret-indigo-100\/90:is(.dark *){caret-color:rgba(224,231,255,.9)}.dark\:caret-indigo-100\/95:is(.dark *){caret-color:rgba(224,231,255,.95)}.dark\:caret-indigo-200:is(.dark *){caret-color:#c7d2fe}.dark\:caret-indigo-200\/0:is(.dark *){caret-color:rgba(199,210,254,0)}.dark\:caret-indigo-200\/10:is(.dark *){caret-color:rgba(199,210,254,.1)}.dark\:caret-indigo-200\/100:is(.dark *){caret-color:#c7d2fe}.dark\:caret-indigo-200\/15:is(.dark *){caret-color:rgba(199,210,254,.15)}.dark\:caret-indigo-200\/20:is(.dark *){caret-color:rgba(199,210,254,.2)}.dark\:caret-indigo-200\/25:is(.dark *){caret-color:rgba(199,210,254,.25)}.dark\:caret-indigo-200\/30:is(.dark *){caret-color:rgba(199,210,254,.3)}.dark\:caret-indigo-200\/35:is(.dark *){caret-color:rgba(199,210,254,.35)}.dark\:caret-indigo-200\/40:is(.dark *){caret-color:rgba(199,210,254,.4)}.dark\:caret-indigo-200\/45:is(.dark *){caret-color:rgba(199,210,254,.45)}.dark\:caret-indigo-200\/5:is(.dark *){caret-color:rgba(199,210,254,.05)}.dark\:caret-indigo-200\/50:is(.dark *){caret-color:rgba(199,210,254,.5)}.dark\:caret-indigo-200\/55:is(.dark *){caret-color:rgba(199,210,254,.55)}.dark\:caret-indigo-200\/60:is(.dark *){caret-color:rgba(199,210,254,.6)}.dark\:caret-indigo-200\/65:is(.dark *){caret-color:rgba(199,210,254,.65)}.dark\:caret-indigo-200\/70:is(.dark *){caret-color:rgba(199,210,254,.7)}.dark\:caret-indigo-200\/75:is(.dark *){caret-color:rgba(199,210,254,.75)}.dark\:caret-indigo-200\/80:is(.dark *){caret-color:rgba(199,210,254,.8)}.dark\:caret-indigo-200\/85:is(.dark *){caret-color:rgba(199,210,254,.85)}.dark\:caret-indigo-200\/90:is(.dark *){caret-color:rgba(199,210,254,.9)}.dark\:caret-indigo-200\/95:is(.dark *){caret-color:rgba(199,210,254,.95)}.dark\:caret-indigo-400:is(.dark *){caret-color:#818cf8}.dark\:caret-indigo-400\/0:is(.dark *){caret-color:rgba(129,140,248,0)}.dark\:caret-indigo-400\/10:is(.dark *){caret-color:rgba(129,140,248,.1)}.dark\:caret-indigo-400\/100:is(.dark *){caret-color:#818cf8}.dark\:caret-indigo-400\/15:is(.dark *){caret-color:rgba(129,140,248,.15)}.dark\:caret-indigo-400\/20:is(.dark *){caret-color:rgba(129,140,248,.2)}.dark\:caret-indigo-400\/25:is(.dark *){caret-color:rgba(129,140,248,.25)}.dark\:caret-indigo-400\/30:is(.dark *){caret-color:rgba(129,140,248,.3)}.dark\:caret-indigo-400\/35:is(.dark *){caret-color:rgba(129,140,248,.35)}.dark\:caret-indigo-400\/40:is(.dark *){caret-color:rgba(129,140,248,.4)}.dark\:caret-indigo-400\/45:is(.dark *){caret-color:rgba(129,140,248,.45)}.dark\:caret-indigo-400\/5:is(.dark *){caret-color:rgba(129,140,248,.05)}.dark\:caret-indigo-400\/50:is(.dark *){caret-color:rgba(129,140,248,.5)}.dark\:caret-indigo-400\/55:is(.dark *){caret-color:rgba(129,140,248,.55)}.dark\:caret-indigo-400\/60:is(.dark *){caret-color:rgba(129,140,248,.6)}.dark\:caret-indigo-400\/65:is(.dark *){caret-color:rgba(129,140,248,.65)}.dark\:caret-indigo-400\/70:is(.dark *){caret-color:rgba(129,140,248,.7)}.dark\:caret-indigo-400\/75:is(.dark *){caret-color:rgba(129,140,248,.75)}.dark\:caret-indigo-400\/80:is(.dark *){caret-color:rgba(129,140,248,.8)}.dark\:caret-indigo-400\/85:is(.dark *){caret-color:rgba(129,140,248,.85)}.dark\:caret-indigo-400\/90:is(.dark *){caret-color:rgba(129,140,248,.9)}.dark\:caret-indigo-400\/95:is(.dark *){caret-color:rgba(129,140,248,.95)}.dark\:caret-indigo-50:is(.dark *){caret-color:#eef2ff}.dark\:caret-indigo-50\/0:is(.dark *){caret-color:rgba(238,242,255,0)}.dark\:caret-indigo-50\/10:is(.dark *){caret-color:rgba(238,242,255,.1)}.dark\:caret-indigo-50\/100:is(.dark *){caret-color:#eef2ff}.dark\:caret-indigo-50\/15:is(.dark *){caret-color:rgba(238,242,255,.15)}.dark\:caret-indigo-50\/20:is(.dark *){caret-color:rgba(238,242,255,.2)}.dark\:caret-indigo-50\/25:is(.dark *){caret-color:rgba(238,242,255,.25)}.dark\:caret-indigo-50\/30:is(.dark *){caret-color:rgba(238,242,255,.3)}.dark\:caret-indigo-50\/35:is(.dark *){caret-color:rgba(238,242,255,.35)}.dark\:caret-indigo-50\/40:is(.dark *){caret-color:rgba(238,242,255,.4)}.dark\:caret-indigo-50\/45:is(.dark *){caret-color:rgba(238,242,255,.45)}.dark\:caret-indigo-50\/5:is(.dark *){caret-color:rgba(238,242,255,.05)}.dark\:caret-indigo-50\/50:is(.dark *){caret-color:rgba(238,242,255,.5)}.dark\:caret-indigo-50\/55:is(.dark *){caret-color:rgba(238,242,255,.55)}.dark\:caret-indigo-50\/60:is(.dark *){caret-color:rgba(238,242,255,.6)}.dark\:caret-indigo-50\/65:is(.dark *){caret-color:rgba(238,242,255,.65)}.dark\:caret-indigo-50\/70:is(.dark *){caret-color:rgba(238,242,255,.7)}.dark\:caret-indigo-50\/75:is(.dark *){caret-color:rgba(238,242,255,.75)}.dark\:caret-indigo-50\/80:is(.dark *){caret-color:rgba(238,242,255,.8)}.dark\:caret-indigo-50\/85:is(.dark *){caret-color:rgba(238,242,255,.85)}.dark\:caret-indigo-50\/90:is(.dark *){caret-color:rgba(238,242,255,.9)}.dark\:caret-indigo-50\/95:is(.dark *){caret-color:rgba(238,242,255,.95)}.dark\:caret-indigo-500:is(.dark *){caret-color:#6366f1}.dark\:caret-indigo-500\/0:is(.dark *){caret-color:rgba(99,102,241,0)}.dark\:caret-indigo-500\/10:is(.dark *){caret-color:rgba(99,102,241,.1)}.dark\:caret-indigo-500\/100:is(.dark *){caret-color:#6366f1}.dark\:caret-indigo-500\/15:is(.dark *){caret-color:rgba(99,102,241,.15)}.dark\:caret-indigo-500\/20:is(.dark *){caret-color:rgba(99,102,241,.2)}.dark\:caret-indigo-500\/25:is(.dark *){caret-color:rgba(99,102,241,.25)}.dark\:caret-indigo-500\/30:is(.dark *){caret-color:rgba(99,102,241,.3)}.dark\:caret-indigo-500\/35:is(.dark *){caret-color:rgba(99,102,241,.35)}.dark\:caret-indigo-500\/40:is(.dark *){caret-color:rgba(99,102,241,.4)}.dark\:caret-indigo-500\/45:is(.dark *){caret-color:rgba(99,102,241,.45)}.dark\:caret-indigo-500\/5:is(.dark *){caret-color:rgba(99,102,241,.05)}.dark\:caret-indigo-500\/50:is(.dark *){caret-color:rgba(99,102,241,.5)}.dark\:caret-indigo-500\/55:is(.dark *){caret-color:rgba(99,102,241,.55)}.dark\:caret-indigo-500\/60:is(.dark *){caret-color:rgba(99,102,241,.6)}.dark\:caret-indigo-500\/65:is(.dark *){caret-color:rgba(99,102,241,.65)}.dark\:caret-indigo-500\/70:is(.dark *){caret-color:rgba(99,102,241,.7)}.dark\:caret-indigo-500\/75:is(.dark *){caret-color:rgba(99,102,241,.75)}.dark\:caret-indigo-500\/80:is(.dark *){caret-color:rgba(99,102,241,.8)}.dark\:caret-indigo-500\/85:is(.dark *){caret-color:rgba(99,102,241,.85)}.dark\:caret-indigo-500\/90:is(.dark *){caret-color:rgba(99,102,241,.9)}.dark\:caret-indigo-500\/95:is(.dark *){caret-color:rgba(99,102,241,.95)}.dark\:caret-indigo-950:is(.dark *){caret-color:#1e1b4b}.dark\:caret-indigo-950\/0:is(.dark *){caret-color:rgba(30,27,75,0)}.dark\:caret-indigo-950\/10:is(.dark *){caret-color:rgba(30,27,75,.1)}.dark\:caret-indigo-950\/100:is(.dark *){caret-color:#1e1b4b}.dark\:caret-indigo-950\/15:is(.dark *){caret-color:rgba(30,27,75,.15)}.dark\:caret-indigo-950\/20:is(.dark *){caret-color:rgba(30,27,75,.2)}.dark\:caret-indigo-950\/25:is(.dark *){caret-color:rgba(30,27,75,.25)}.dark\:caret-indigo-950\/30:is(.dark *){caret-color:rgba(30,27,75,.3)}.dark\:caret-indigo-950\/35:is(.dark *){caret-color:rgba(30,27,75,.35)}.dark\:caret-indigo-950\/40:is(.dark *){caret-color:rgba(30,27,75,.4)}.dark\:caret-indigo-950\/45:is(.dark *){caret-color:rgba(30,27,75,.45)}.dark\:caret-indigo-950\/5:is(.dark *){caret-color:rgba(30,27,75,.05)}.dark\:caret-indigo-950\/50:is(.dark *){caret-color:rgba(30,27,75,.5)}.dark\:caret-indigo-950\/55:is(.dark *){caret-color:rgba(30,27,75,.55)}.dark\:caret-indigo-950\/60:is(.dark *){caret-color:rgba(30,27,75,.6)}.dark\:caret-indigo-950\/65:is(.dark *){caret-color:rgba(30,27,75,.65)}.dark\:caret-indigo-950\/70:is(.dark *){caret-color:rgba(30,27,75,.7)}.dark\:caret-indigo-950\/75:is(.dark *){caret-color:rgba(30,27,75,.75)}.dark\:caret-indigo-950\/80:is(.dark *){caret-color:rgba(30,27,75,.8)}.dark\:caret-indigo-950\/85:is(.dark *){caret-color:rgba(30,27,75,.85)}.dark\:caret-indigo-950\/90:is(.dark *){caret-color:rgba(30,27,75,.9)}.dark\:caret-indigo-950\/95:is(.dark *){caret-color:rgba(30,27,75,.95)}.dark\:caret-lime-100:is(.dark *){caret-color:#ecfccb}.dark\:caret-lime-100\/0:is(.dark *){caret-color:hsla(80,89%,89%,0)}.dark\:caret-lime-100\/10:is(.dark *){caret-color:hsla(80,89%,89%,.1)}.dark\:caret-lime-100\/100:is(.dark *){caret-color:#ecfccb}.dark\:caret-lime-100\/15:is(.dark *){caret-color:hsla(80,89%,89%,.15)}.dark\:caret-lime-100\/20:is(.dark *){caret-color:hsla(80,89%,89%,.2)}.dark\:caret-lime-100\/25:is(.dark *){caret-color:hsla(80,89%,89%,.25)}.dark\:caret-lime-100\/30:is(.dark *){caret-color:hsla(80,89%,89%,.3)}.dark\:caret-lime-100\/35:is(.dark *){caret-color:hsla(80,89%,89%,.35)}.dark\:caret-lime-100\/40:is(.dark *){caret-color:hsla(80,89%,89%,.4)}.dark\:caret-lime-100\/45:is(.dark *){caret-color:hsla(80,89%,89%,.45)}.dark\:caret-lime-100\/5:is(.dark *){caret-color:hsla(80,89%,89%,.05)}.dark\:caret-lime-100\/50:is(.dark *){caret-color:hsla(80,89%,89%,.5)}.dark\:caret-lime-100\/55:is(.dark *){caret-color:hsla(80,89%,89%,.55)}.dark\:caret-lime-100\/60:is(.dark *){caret-color:hsla(80,89%,89%,.6)}.dark\:caret-lime-100\/65:is(.dark *){caret-color:hsla(80,89%,89%,.65)}.dark\:caret-lime-100\/70:is(.dark *){caret-color:hsla(80,89%,89%,.7)}.dark\:caret-lime-100\/75:is(.dark *){caret-color:hsla(80,89%,89%,.75)}.dark\:caret-lime-100\/80:is(.dark *){caret-color:hsla(80,89%,89%,.8)}.dark\:caret-lime-100\/85:is(.dark *){caret-color:hsla(80,89%,89%,.85)}.dark\:caret-lime-100\/90:is(.dark *){caret-color:hsla(80,89%,89%,.9)}.dark\:caret-lime-100\/95:is(.dark *){caret-color:hsla(80,89%,89%,.95)}.dark\:caret-lime-200:is(.dark *){caret-color:#d9f99d}.dark\:caret-lime-200\/0:is(.dark *){caret-color:hsla(81,88%,80%,0)}.dark\:caret-lime-200\/10:is(.dark *){caret-color:hsla(81,88%,80%,.1)}.dark\:caret-lime-200\/100:is(.dark *){caret-color:#d9f99d}.dark\:caret-lime-200\/15:is(.dark *){caret-color:hsla(81,88%,80%,.15)}.dark\:caret-lime-200\/20:is(.dark *){caret-color:hsla(81,88%,80%,.2)}.dark\:caret-lime-200\/25:is(.dark *){caret-color:hsla(81,88%,80%,.25)}.dark\:caret-lime-200\/30:is(.dark *){caret-color:hsla(81,88%,80%,.3)}.dark\:caret-lime-200\/35:is(.dark *){caret-color:hsla(81,88%,80%,.35)}.dark\:caret-lime-200\/40:is(.dark *){caret-color:hsla(81,88%,80%,.4)}.dark\:caret-lime-200\/45:is(.dark *){caret-color:hsla(81,88%,80%,.45)}.dark\:caret-lime-200\/5:is(.dark *){caret-color:hsla(81,88%,80%,.05)}.dark\:caret-lime-200\/50:is(.dark *){caret-color:hsla(81,88%,80%,.5)}.dark\:caret-lime-200\/55:is(.dark *){caret-color:hsla(81,88%,80%,.55)}.dark\:caret-lime-200\/60:is(.dark *){caret-color:hsla(81,88%,80%,.6)}.dark\:caret-lime-200\/65:is(.dark *){caret-color:hsla(81,88%,80%,.65)}.dark\:caret-lime-200\/70:is(.dark *){caret-color:hsla(81,88%,80%,.7)}.dark\:caret-lime-200\/75:is(.dark *){caret-color:hsla(81,88%,80%,.75)}.dark\:caret-lime-200\/80:is(.dark *){caret-color:hsla(81,88%,80%,.8)}.dark\:caret-lime-200\/85:is(.dark *){caret-color:hsla(81,88%,80%,.85)}.dark\:caret-lime-200\/90:is(.dark *){caret-color:hsla(81,88%,80%,.9)}.dark\:caret-lime-200\/95:is(.dark *){caret-color:hsla(81,88%,80%,.95)}.dark\:caret-lime-400:is(.dark *){caret-color:#a3e635}.dark\:caret-lime-400\/0:is(.dark *){caret-color:rgba(163,230,53,0)}.dark\:caret-lime-400\/10:is(.dark *){caret-color:rgba(163,230,53,.1)}.dark\:caret-lime-400\/100:is(.dark *){caret-color:#a3e635}.dark\:caret-lime-400\/15:is(.dark *){caret-color:rgba(163,230,53,.15)}.dark\:caret-lime-400\/20:is(.dark *){caret-color:rgba(163,230,53,.2)}.dark\:caret-lime-400\/25:is(.dark *){caret-color:rgba(163,230,53,.25)}.dark\:caret-lime-400\/30:is(.dark *){caret-color:rgba(163,230,53,.3)}.dark\:caret-lime-400\/35:is(.dark *){caret-color:rgba(163,230,53,.35)}.dark\:caret-lime-400\/40:is(.dark *){caret-color:rgba(163,230,53,.4)}.dark\:caret-lime-400\/45:is(.dark *){caret-color:rgba(163,230,53,.45)}.dark\:caret-lime-400\/5:is(.dark *){caret-color:rgba(163,230,53,.05)}.dark\:caret-lime-400\/50:is(.dark *){caret-color:rgba(163,230,53,.5)}.dark\:caret-lime-400\/55:is(.dark *){caret-color:rgba(163,230,53,.55)}.dark\:caret-lime-400\/60:is(.dark *){caret-color:rgba(163,230,53,.6)}.dark\:caret-lime-400\/65:is(.dark *){caret-color:rgba(163,230,53,.65)}.dark\:caret-lime-400\/70:is(.dark *){caret-color:rgba(163,230,53,.7)}.dark\:caret-lime-400\/75:is(.dark *){caret-color:rgba(163,230,53,.75)}.dark\:caret-lime-400\/80:is(.dark *){caret-color:rgba(163,230,53,.8)}.dark\:caret-lime-400\/85:is(.dark *){caret-color:rgba(163,230,53,.85)}.dark\:caret-lime-400\/90:is(.dark *){caret-color:rgba(163,230,53,.9)}.dark\:caret-lime-400\/95:is(.dark *){caret-color:rgba(163,230,53,.95)}.dark\:caret-lime-50:is(.dark *){caret-color:#f7fee7}.dark\:caret-lime-50\/0:is(.dark *){caret-color:hsla(78,92%,95%,0)}.dark\:caret-lime-50\/10:is(.dark *){caret-color:hsla(78,92%,95%,.1)}.dark\:caret-lime-50\/100:is(.dark *){caret-color:#f7fee7}.dark\:caret-lime-50\/15:is(.dark *){caret-color:hsla(78,92%,95%,.15)}.dark\:caret-lime-50\/20:is(.dark *){caret-color:hsla(78,92%,95%,.2)}.dark\:caret-lime-50\/25:is(.dark *){caret-color:hsla(78,92%,95%,.25)}.dark\:caret-lime-50\/30:is(.dark *){caret-color:hsla(78,92%,95%,.3)}.dark\:caret-lime-50\/35:is(.dark *){caret-color:hsla(78,92%,95%,.35)}.dark\:caret-lime-50\/40:is(.dark *){caret-color:hsla(78,92%,95%,.4)}.dark\:caret-lime-50\/45:is(.dark *){caret-color:hsla(78,92%,95%,.45)}.dark\:caret-lime-50\/5:is(.dark *){caret-color:hsla(78,92%,95%,.05)}.dark\:caret-lime-50\/50:is(.dark *){caret-color:hsla(78,92%,95%,.5)}.dark\:caret-lime-50\/55:is(.dark *){caret-color:hsla(78,92%,95%,.55)}.dark\:caret-lime-50\/60:is(.dark *){caret-color:hsla(78,92%,95%,.6)}.dark\:caret-lime-50\/65:is(.dark *){caret-color:hsla(78,92%,95%,.65)}.dark\:caret-lime-50\/70:is(.dark *){caret-color:hsla(78,92%,95%,.7)}.dark\:caret-lime-50\/75:is(.dark *){caret-color:hsla(78,92%,95%,.75)}.dark\:caret-lime-50\/80:is(.dark *){caret-color:hsla(78,92%,95%,.8)}.dark\:caret-lime-50\/85:is(.dark *){caret-color:hsla(78,92%,95%,.85)}.dark\:caret-lime-50\/90:is(.dark *){caret-color:hsla(78,92%,95%,.9)}.dark\:caret-lime-50\/95:is(.dark *){caret-color:hsla(78,92%,95%,.95)}.dark\:caret-lime-500:is(.dark *){caret-color:#84cc16}.dark\:caret-lime-500\/0:is(.dark *){caret-color:rgba(132,204,22,0)}.dark\:caret-lime-500\/10:is(.dark *){caret-color:rgba(132,204,22,.1)}.dark\:caret-lime-500\/100:is(.dark *){caret-color:#84cc16}.dark\:caret-lime-500\/15:is(.dark *){caret-color:rgba(132,204,22,.15)}.dark\:caret-lime-500\/20:is(.dark *){caret-color:rgba(132,204,22,.2)}.dark\:caret-lime-500\/25:is(.dark *){caret-color:rgba(132,204,22,.25)}.dark\:caret-lime-500\/30:is(.dark *){caret-color:rgba(132,204,22,.3)}.dark\:caret-lime-500\/35:is(.dark *){caret-color:rgba(132,204,22,.35)}.dark\:caret-lime-500\/40:is(.dark *){caret-color:rgba(132,204,22,.4)}.dark\:caret-lime-500\/45:is(.dark *){caret-color:rgba(132,204,22,.45)}.dark\:caret-lime-500\/5:is(.dark *){caret-color:rgba(132,204,22,.05)}.dark\:caret-lime-500\/50:is(.dark *){caret-color:rgba(132,204,22,.5)}.dark\:caret-lime-500\/55:is(.dark *){caret-color:rgba(132,204,22,.55)}.dark\:caret-lime-500\/60:is(.dark *){caret-color:rgba(132,204,22,.6)}.dark\:caret-lime-500\/65:is(.dark *){caret-color:rgba(132,204,22,.65)}.dark\:caret-lime-500\/70:is(.dark *){caret-color:rgba(132,204,22,.7)}.dark\:caret-lime-500\/75:is(.dark *){caret-color:rgba(132,204,22,.75)}.dark\:caret-lime-500\/80:is(.dark *){caret-color:rgba(132,204,22,.8)}.dark\:caret-lime-500\/85:is(.dark *){caret-color:rgba(132,204,22,.85)}.dark\:caret-lime-500\/90:is(.dark *){caret-color:rgba(132,204,22,.9)}.dark\:caret-lime-500\/95:is(.dark *){caret-color:rgba(132,204,22,.95)}.dark\:caret-lime-950:is(.dark *){caret-color:#1a2e05}.dark\:caret-lime-950\/0:is(.dark *){caret-color:rgba(26,46,5,0)}.dark\:caret-lime-950\/10:is(.dark *){caret-color:rgba(26,46,5,.1)}.dark\:caret-lime-950\/100:is(.dark *){caret-color:#1a2e05}.dark\:caret-lime-950\/15:is(.dark *){caret-color:rgba(26,46,5,.15)}.dark\:caret-lime-950\/20:is(.dark *){caret-color:rgba(26,46,5,.2)}.dark\:caret-lime-950\/25:is(.dark *){caret-color:rgba(26,46,5,.25)}.dark\:caret-lime-950\/30:is(.dark *){caret-color:rgba(26,46,5,.3)}.dark\:caret-lime-950\/35:is(.dark *){caret-color:rgba(26,46,5,.35)}.dark\:caret-lime-950\/40:is(.dark *){caret-color:rgba(26,46,5,.4)}.dark\:caret-lime-950\/45:is(.dark *){caret-color:rgba(26,46,5,.45)}.dark\:caret-lime-950\/5:is(.dark *){caret-color:rgba(26,46,5,.05)}.dark\:caret-lime-950\/50:is(.dark *){caret-color:rgba(26,46,5,.5)}.dark\:caret-lime-950\/55:is(.dark *){caret-color:rgba(26,46,5,.55)}.dark\:caret-lime-950\/60:is(.dark *){caret-color:rgba(26,46,5,.6)}.dark\:caret-lime-950\/65:is(.dark *){caret-color:rgba(26,46,5,.65)}.dark\:caret-lime-950\/70:is(.dark *){caret-color:rgba(26,46,5,.7)}.dark\:caret-lime-950\/75:is(.dark *){caret-color:rgba(26,46,5,.75)}.dark\:caret-lime-950\/80:is(.dark *){caret-color:rgba(26,46,5,.8)}.dark\:caret-lime-950\/85:is(.dark *){caret-color:rgba(26,46,5,.85)}.dark\:caret-lime-950\/90:is(.dark *){caret-color:rgba(26,46,5,.9)}.dark\:caret-lime-950\/95:is(.dark *){caret-color:rgba(26,46,5,.95)}.dark\:caret-neutral-100:is(.dark *){caret-color:#f5f5f5}.dark\:caret-neutral-100\/0:is(.dark *){caret-color:hsla(0,0%,96%,0)}.dark\:caret-neutral-100\/10:is(.dark *){caret-color:hsla(0,0%,96%,.1)}.dark\:caret-neutral-100\/100:is(.dark *){caret-color:#f5f5f5}.dark\:caret-neutral-100\/15:is(.dark *){caret-color:hsla(0,0%,96%,.15)}.dark\:caret-neutral-100\/20:is(.dark *){caret-color:hsla(0,0%,96%,.2)}.dark\:caret-neutral-100\/25:is(.dark *){caret-color:hsla(0,0%,96%,.25)}.dark\:caret-neutral-100\/30:is(.dark *){caret-color:hsla(0,0%,96%,.3)}.dark\:caret-neutral-100\/35:is(.dark *){caret-color:hsla(0,0%,96%,.35)}.dark\:caret-neutral-100\/40:is(.dark *){caret-color:hsla(0,0%,96%,.4)}.dark\:caret-neutral-100\/45:is(.dark *){caret-color:hsla(0,0%,96%,.45)}.dark\:caret-neutral-100\/5:is(.dark *){caret-color:hsla(0,0%,96%,.05)}.dark\:caret-neutral-100\/50:is(.dark *){caret-color:hsla(0,0%,96%,.5)}.dark\:caret-neutral-100\/55:is(.dark *){caret-color:hsla(0,0%,96%,.55)}.dark\:caret-neutral-100\/60:is(.dark *){caret-color:hsla(0,0%,96%,.6)}.dark\:caret-neutral-100\/65:is(.dark *){caret-color:hsla(0,0%,96%,.65)}.dark\:caret-neutral-100\/70:is(.dark *){caret-color:hsla(0,0%,96%,.7)}.dark\:caret-neutral-100\/75:is(.dark *){caret-color:hsla(0,0%,96%,.75)}.dark\:caret-neutral-100\/80:is(.dark *){caret-color:hsla(0,0%,96%,.8)}.dark\:caret-neutral-100\/85:is(.dark *){caret-color:hsla(0,0%,96%,.85)}.dark\:caret-neutral-100\/90:is(.dark *){caret-color:hsla(0,0%,96%,.9)}.dark\:caret-neutral-100\/95:is(.dark *){caret-color:hsla(0,0%,96%,.95)}.dark\:caret-neutral-200:is(.dark *){caret-color:#e5e5e5}.dark\:caret-neutral-200\/0:is(.dark *){caret-color:hsla(0,0%,90%,0)}.dark\:caret-neutral-200\/10:is(.dark *){caret-color:hsla(0,0%,90%,.1)}.dark\:caret-neutral-200\/100:is(.dark *){caret-color:#e5e5e5}.dark\:caret-neutral-200\/15:is(.dark *){caret-color:hsla(0,0%,90%,.15)}.dark\:caret-neutral-200\/20:is(.dark *){caret-color:hsla(0,0%,90%,.2)}.dark\:caret-neutral-200\/25:is(.dark *){caret-color:hsla(0,0%,90%,.25)}.dark\:caret-neutral-200\/30:is(.dark *){caret-color:hsla(0,0%,90%,.3)}.dark\:caret-neutral-200\/35:is(.dark *){caret-color:hsla(0,0%,90%,.35)}.dark\:caret-neutral-200\/40:is(.dark *){caret-color:hsla(0,0%,90%,.4)}.dark\:caret-neutral-200\/45:is(.dark *){caret-color:hsla(0,0%,90%,.45)}.dark\:caret-neutral-200\/5:is(.dark *){caret-color:hsla(0,0%,90%,.05)}.dark\:caret-neutral-200\/50:is(.dark *){caret-color:hsla(0,0%,90%,.5)}.dark\:caret-neutral-200\/55:is(.dark *){caret-color:hsla(0,0%,90%,.55)}.dark\:caret-neutral-200\/60:is(.dark *){caret-color:hsla(0,0%,90%,.6)}.dark\:caret-neutral-200\/65:is(.dark *){caret-color:hsla(0,0%,90%,.65)}.dark\:caret-neutral-200\/70:is(.dark *){caret-color:hsla(0,0%,90%,.7)}.dark\:caret-neutral-200\/75:is(.dark *){caret-color:hsla(0,0%,90%,.75)}.dark\:caret-neutral-200\/80:is(.dark *){caret-color:hsla(0,0%,90%,.8)}.dark\:caret-neutral-200\/85:is(.dark *){caret-color:hsla(0,0%,90%,.85)}.dark\:caret-neutral-200\/90:is(.dark *){caret-color:hsla(0,0%,90%,.9)}.dark\:caret-neutral-200\/95:is(.dark *){caret-color:hsla(0,0%,90%,.95)}.dark\:caret-neutral-400:is(.dark *){caret-color:#a3a3a3}.dark\:caret-neutral-400\/0:is(.dark *){caret-color:hsla(0,0%,64%,0)}.dark\:caret-neutral-400\/10:is(.dark *){caret-color:hsla(0,0%,64%,.1)}.dark\:caret-neutral-400\/100:is(.dark *){caret-color:#a3a3a3}.dark\:caret-neutral-400\/15:is(.dark *){caret-color:hsla(0,0%,64%,.15)}.dark\:caret-neutral-400\/20:is(.dark *){caret-color:hsla(0,0%,64%,.2)}.dark\:caret-neutral-400\/25:is(.dark *){caret-color:hsla(0,0%,64%,.25)}.dark\:caret-neutral-400\/30:is(.dark *){caret-color:hsla(0,0%,64%,.3)}.dark\:caret-neutral-400\/35:is(.dark *){caret-color:hsla(0,0%,64%,.35)}.dark\:caret-neutral-400\/40:is(.dark *){caret-color:hsla(0,0%,64%,.4)}.dark\:caret-neutral-400\/45:is(.dark *){caret-color:hsla(0,0%,64%,.45)}.dark\:caret-neutral-400\/5:is(.dark *){caret-color:hsla(0,0%,64%,.05)}.dark\:caret-neutral-400\/50:is(.dark *){caret-color:hsla(0,0%,64%,.5)}.dark\:caret-neutral-400\/55:is(.dark *){caret-color:hsla(0,0%,64%,.55)}.dark\:caret-neutral-400\/60:is(.dark *){caret-color:hsla(0,0%,64%,.6)}.dark\:caret-neutral-400\/65:is(.dark *){caret-color:hsla(0,0%,64%,.65)}.dark\:caret-neutral-400\/70:is(.dark *){caret-color:hsla(0,0%,64%,.7)}.dark\:caret-neutral-400\/75:is(.dark *){caret-color:hsla(0,0%,64%,.75)}.dark\:caret-neutral-400\/80:is(.dark *){caret-color:hsla(0,0%,64%,.8)}.dark\:caret-neutral-400\/85:is(.dark *){caret-color:hsla(0,0%,64%,.85)}.dark\:caret-neutral-400\/90:is(.dark *){caret-color:hsla(0,0%,64%,.9)}.dark\:caret-neutral-400\/95:is(.dark *){caret-color:hsla(0,0%,64%,.95)}.dark\:caret-neutral-50:is(.dark *){caret-color:#fafafa}.dark\:caret-neutral-50\/0:is(.dark *){caret-color:hsla(0,0%,98%,0)}.dark\:caret-neutral-50\/10:is(.dark *){caret-color:hsla(0,0%,98%,.1)}.dark\:caret-neutral-50\/100:is(.dark *){caret-color:#fafafa}.dark\:caret-neutral-50\/15:is(.dark *){caret-color:hsla(0,0%,98%,.15)}.dark\:caret-neutral-50\/20:is(.dark *){caret-color:hsla(0,0%,98%,.2)}.dark\:caret-neutral-50\/25:is(.dark *){caret-color:hsla(0,0%,98%,.25)}.dark\:caret-neutral-50\/30:is(.dark *){caret-color:hsla(0,0%,98%,.3)}.dark\:caret-neutral-50\/35:is(.dark *){caret-color:hsla(0,0%,98%,.35)}.dark\:caret-neutral-50\/40:is(.dark *){caret-color:hsla(0,0%,98%,.4)}.dark\:caret-neutral-50\/45:is(.dark *){caret-color:hsla(0,0%,98%,.45)}.dark\:caret-neutral-50\/5:is(.dark *){caret-color:hsla(0,0%,98%,.05)}.dark\:caret-neutral-50\/50:is(.dark *){caret-color:hsla(0,0%,98%,.5)}.dark\:caret-neutral-50\/55:is(.dark *){caret-color:hsla(0,0%,98%,.55)}.dark\:caret-neutral-50\/60:is(.dark *){caret-color:hsla(0,0%,98%,.6)}.dark\:caret-neutral-50\/65:is(.dark *){caret-color:hsla(0,0%,98%,.65)}.dark\:caret-neutral-50\/70:is(.dark *){caret-color:hsla(0,0%,98%,.7)}.dark\:caret-neutral-50\/75:is(.dark *){caret-color:hsla(0,0%,98%,.75)}.dark\:caret-neutral-50\/80:is(.dark *){caret-color:hsla(0,0%,98%,.8)}.dark\:caret-neutral-50\/85:is(.dark *){caret-color:hsla(0,0%,98%,.85)}.dark\:caret-neutral-50\/90:is(.dark *){caret-color:hsla(0,0%,98%,.9)}.dark\:caret-neutral-50\/95:is(.dark *){caret-color:hsla(0,0%,98%,.95)}.dark\:caret-neutral-500:is(.dark *){caret-color:#737373}.dark\:caret-neutral-500\/0:is(.dark *){caret-color:hsla(0,0%,45%,0)}.dark\:caret-neutral-500\/10:is(.dark *){caret-color:hsla(0,0%,45%,.1)}.dark\:caret-neutral-500\/100:is(.dark *){caret-color:#737373}.dark\:caret-neutral-500\/15:is(.dark *){caret-color:hsla(0,0%,45%,.15)}.dark\:caret-neutral-500\/20:is(.dark *){caret-color:hsla(0,0%,45%,.2)}.dark\:caret-neutral-500\/25:is(.dark *){caret-color:hsla(0,0%,45%,.25)}.dark\:caret-neutral-500\/30:is(.dark *){caret-color:hsla(0,0%,45%,.3)}.dark\:caret-neutral-500\/35:is(.dark *){caret-color:hsla(0,0%,45%,.35)}.dark\:caret-neutral-500\/40:is(.dark *){caret-color:hsla(0,0%,45%,.4)}.dark\:caret-neutral-500\/45:is(.dark *){caret-color:hsla(0,0%,45%,.45)}.dark\:caret-neutral-500\/5:is(.dark *){caret-color:hsla(0,0%,45%,.05)}.dark\:caret-neutral-500\/50:is(.dark *){caret-color:hsla(0,0%,45%,.5)}.dark\:caret-neutral-500\/55:is(.dark *){caret-color:hsla(0,0%,45%,.55)}.dark\:caret-neutral-500\/60:is(.dark *){caret-color:hsla(0,0%,45%,.6)}.dark\:caret-neutral-500\/65:is(.dark *){caret-color:hsla(0,0%,45%,.65)}.dark\:caret-neutral-500\/70:is(.dark *){caret-color:hsla(0,0%,45%,.7)}.dark\:caret-neutral-500\/75:is(.dark *){caret-color:hsla(0,0%,45%,.75)}.dark\:caret-neutral-500\/80:is(.dark *){caret-color:hsla(0,0%,45%,.8)}.dark\:caret-neutral-500\/85:is(.dark *){caret-color:hsla(0,0%,45%,.85)}.dark\:caret-neutral-500\/90:is(.dark *){caret-color:hsla(0,0%,45%,.9)}.dark\:caret-neutral-500\/95:is(.dark *){caret-color:hsla(0,0%,45%,.95)}.dark\:caret-neutral-950:is(.dark *){caret-color:#0a0a0a}.dark\:caret-neutral-950\/0:is(.dark *){caret-color:hsla(0,0%,4%,0)}.dark\:caret-neutral-950\/10:is(.dark *){caret-color:hsla(0,0%,4%,.1)}.dark\:caret-neutral-950\/100:is(.dark *){caret-color:#0a0a0a}.dark\:caret-neutral-950\/15:is(.dark *){caret-color:hsla(0,0%,4%,.15)}.dark\:caret-neutral-950\/20:is(.dark *){caret-color:hsla(0,0%,4%,.2)}.dark\:caret-neutral-950\/25:is(.dark *){caret-color:hsla(0,0%,4%,.25)}.dark\:caret-neutral-950\/30:is(.dark *){caret-color:hsla(0,0%,4%,.3)}.dark\:caret-neutral-950\/35:is(.dark *){caret-color:hsla(0,0%,4%,.35)}.dark\:caret-neutral-950\/40:is(.dark *){caret-color:hsla(0,0%,4%,.4)}.dark\:caret-neutral-950\/45:is(.dark *){caret-color:hsla(0,0%,4%,.45)}.dark\:caret-neutral-950\/5:is(.dark *){caret-color:hsla(0,0%,4%,.05)}.dark\:caret-neutral-950\/50:is(.dark *){caret-color:hsla(0,0%,4%,.5)}.dark\:caret-neutral-950\/55:is(.dark *){caret-color:hsla(0,0%,4%,.55)}.dark\:caret-neutral-950\/60:is(.dark *){caret-color:hsla(0,0%,4%,.6)}.dark\:caret-neutral-950\/65:is(.dark *){caret-color:hsla(0,0%,4%,.65)}.dark\:caret-neutral-950\/70:is(.dark *){caret-color:hsla(0,0%,4%,.7)}.dark\:caret-neutral-950\/75:is(.dark *){caret-color:hsla(0,0%,4%,.75)}.dark\:caret-neutral-950\/80:is(.dark *){caret-color:hsla(0,0%,4%,.8)}.dark\:caret-neutral-950\/85:is(.dark *){caret-color:hsla(0,0%,4%,.85)}.dark\:caret-neutral-950\/90:is(.dark *){caret-color:hsla(0,0%,4%,.9)}.dark\:caret-neutral-950\/95:is(.dark *){caret-color:hsla(0,0%,4%,.95)}.dark\:caret-orange-100:is(.dark *){caret-color:#ffedd5}.dark\:caret-orange-100\/0:is(.dark *){caret-color:rgba(255,237,213,0)}.dark\:caret-orange-100\/10:is(.dark *){caret-color:rgba(255,237,213,.1)}.dark\:caret-orange-100\/100:is(.dark *){caret-color:#ffedd5}.dark\:caret-orange-100\/15:is(.dark *){caret-color:rgba(255,237,213,.15)}.dark\:caret-orange-100\/20:is(.dark *){caret-color:rgba(255,237,213,.2)}.dark\:caret-orange-100\/25:is(.dark *){caret-color:rgba(255,237,213,.25)}.dark\:caret-orange-100\/30:is(.dark *){caret-color:rgba(255,237,213,.3)}.dark\:caret-orange-100\/35:is(.dark *){caret-color:rgba(255,237,213,.35)}.dark\:caret-orange-100\/40:is(.dark *){caret-color:rgba(255,237,213,.4)}.dark\:caret-orange-100\/45:is(.dark *){caret-color:rgba(255,237,213,.45)}.dark\:caret-orange-100\/5:is(.dark *){caret-color:rgba(255,237,213,.05)}.dark\:caret-orange-100\/50:is(.dark *){caret-color:rgba(255,237,213,.5)}.dark\:caret-orange-100\/55:is(.dark *){caret-color:rgba(255,237,213,.55)}.dark\:caret-orange-100\/60:is(.dark *){caret-color:rgba(255,237,213,.6)}.dark\:caret-orange-100\/65:is(.dark *){caret-color:rgba(255,237,213,.65)}.dark\:caret-orange-100\/70:is(.dark *){caret-color:rgba(255,237,213,.7)}.dark\:caret-orange-100\/75:is(.dark *){caret-color:rgba(255,237,213,.75)}.dark\:caret-orange-100\/80:is(.dark *){caret-color:rgba(255,237,213,.8)}.dark\:caret-orange-100\/85:is(.dark *){caret-color:rgba(255,237,213,.85)}.dark\:caret-orange-100\/90:is(.dark *){caret-color:rgba(255,237,213,.9)}.dark\:caret-orange-100\/95:is(.dark *){caret-color:rgba(255,237,213,.95)}.dark\:caret-orange-200:is(.dark *){caret-color:#fed7aa}.dark\:caret-orange-200\/0:is(.dark *){caret-color:hsla(32,98%,83%,0)}.dark\:caret-orange-200\/10:is(.dark *){caret-color:hsla(32,98%,83%,.1)}.dark\:caret-orange-200\/100:is(.dark *){caret-color:#fed7aa}.dark\:caret-orange-200\/15:is(.dark *){caret-color:hsla(32,98%,83%,.15)}.dark\:caret-orange-200\/20:is(.dark *){caret-color:hsla(32,98%,83%,.2)}.dark\:caret-orange-200\/25:is(.dark *){caret-color:hsla(32,98%,83%,.25)}.dark\:caret-orange-200\/30:is(.dark *){caret-color:hsla(32,98%,83%,.3)}.dark\:caret-orange-200\/35:is(.dark *){caret-color:hsla(32,98%,83%,.35)}.dark\:caret-orange-200\/40:is(.dark *){caret-color:hsla(32,98%,83%,.4)}.dark\:caret-orange-200\/45:is(.dark *){caret-color:hsla(32,98%,83%,.45)}.dark\:caret-orange-200\/5:is(.dark *){caret-color:hsla(32,98%,83%,.05)}.dark\:caret-orange-200\/50:is(.dark *){caret-color:hsla(32,98%,83%,.5)}.dark\:caret-orange-200\/55:is(.dark *){caret-color:hsla(32,98%,83%,.55)}.dark\:caret-orange-200\/60:is(.dark *){caret-color:hsla(32,98%,83%,.6)}.dark\:caret-orange-200\/65:is(.dark *){caret-color:hsla(32,98%,83%,.65)}.dark\:caret-orange-200\/70:is(.dark *){caret-color:hsla(32,98%,83%,.7)}.dark\:caret-orange-200\/75:is(.dark *){caret-color:hsla(32,98%,83%,.75)}.dark\:caret-orange-200\/80:is(.dark *){caret-color:hsla(32,98%,83%,.8)}.dark\:caret-orange-200\/85:is(.dark *){caret-color:hsla(32,98%,83%,.85)}.dark\:caret-orange-200\/90:is(.dark *){caret-color:hsla(32,98%,83%,.9)}.dark\:caret-orange-200\/95:is(.dark *){caret-color:hsla(32,98%,83%,.95)}.dark\:caret-orange-400:is(.dark *){caret-color:#fb923c}.dark\:caret-orange-400\/0:is(.dark *){caret-color:rgba(251,146,60,0)}.dark\:caret-orange-400\/10:is(.dark *){caret-color:rgba(251,146,60,.1)}.dark\:caret-orange-400\/100:is(.dark *){caret-color:#fb923c}.dark\:caret-orange-400\/15:is(.dark *){caret-color:rgba(251,146,60,.15)}.dark\:caret-orange-400\/20:is(.dark *){caret-color:rgba(251,146,60,.2)}.dark\:caret-orange-400\/25:is(.dark *){caret-color:rgba(251,146,60,.25)}.dark\:caret-orange-400\/30:is(.dark *){caret-color:rgba(251,146,60,.3)}.dark\:caret-orange-400\/35:is(.dark *){caret-color:rgba(251,146,60,.35)}.dark\:caret-orange-400\/40:is(.dark *){caret-color:rgba(251,146,60,.4)}.dark\:caret-orange-400\/45:is(.dark *){caret-color:rgba(251,146,60,.45)}.dark\:caret-orange-400\/5:is(.dark *){caret-color:rgba(251,146,60,.05)}.dark\:caret-orange-400\/50:is(.dark *){caret-color:rgba(251,146,60,.5)}.dark\:caret-orange-400\/55:is(.dark *){caret-color:rgba(251,146,60,.55)}.dark\:caret-orange-400\/60:is(.dark *){caret-color:rgba(251,146,60,.6)}.dark\:caret-orange-400\/65:is(.dark *){caret-color:rgba(251,146,60,.65)}.dark\:caret-orange-400\/70:is(.dark *){caret-color:rgba(251,146,60,.7)}.dark\:caret-orange-400\/75:is(.dark *){caret-color:rgba(251,146,60,.75)}.dark\:caret-orange-400\/80:is(.dark *){caret-color:rgba(251,146,60,.8)}.dark\:caret-orange-400\/85:is(.dark *){caret-color:rgba(251,146,60,.85)}.dark\:caret-orange-400\/90:is(.dark *){caret-color:rgba(251,146,60,.9)}.dark\:caret-orange-400\/95:is(.dark *){caret-color:rgba(251,146,60,.95)}.dark\:caret-orange-50:is(.dark *){caret-color:#fff7ed}.dark\:caret-orange-50\/0:is(.dark *){caret-color:rgba(255,247,237,0)}.dark\:caret-orange-50\/10:is(.dark *){caret-color:rgba(255,247,237,.1)}.dark\:caret-orange-50\/100:is(.dark *){caret-color:#fff7ed}.dark\:caret-orange-50\/15:is(.dark *){caret-color:rgba(255,247,237,.15)}.dark\:caret-orange-50\/20:is(.dark *){caret-color:rgba(255,247,237,.2)}.dark\:caret-orange-50\/25:is(.dark *){caret-color:rgba(255,247,237,.25)}.dark\:caret-orange-50\/30:is(.dark *){caret-color:rgba(255,247,237,.3)}.dark\:caret-orange-50\/35:is(.dark *){caret-color:rgba(255,247,237,.35)}.dark\:caret-orange-50\/40:is(.dark *){caret-color:rgba(255,247,237,.4)}.dark\:caret-orange-50\/45:is(.dark *){caret-color:rgba(255,247,237,.45)}.dark\:caret-orange-50\/5:is(.dark *){caret-color:rgba(255,247,237,.05)}.dark\:caret-orange-50\/50:is(.dark *){caret-color:rgba(255,247,237,.5)}.dark\:caret-orange-50\/55:is(.dark *){caret-color:rgba(255,247,237,.55)}.dark\:caret-orange-50\/60:is(.dark *){caret-color:rgba(255,247,237,.6)}.dark\:caret-orange-50\/65:is(.dark *){caret-color:rgba(255,247,237,.65)}.dark\:caret-orange-50\/70:is(.dark *){caret-color:rgba(255,247,237,.7)}.dark\:caret-orange-50\/75:is(.dark *){caret-color:rgba(255,247,237,.75)}.dark\:caret-orange-50\/80:is(.dark *){caret-color:rgba(255,247,237,.8)}.dark\:caret-orange-50\/85:is(.dark *){caret-color:rgba(255,247,237,.85)}.dark\:caret-orange-50\/90:is(.dark *){caret-color:rgba(255,247,237,.9)}.dark\:caret-orange-50\/95:is(.dark *){caret-color:rgba(255,247,237,.95)}.dark\:caret-orange-500:is(.dark *){caret-color:#f97316}.dark\:caret-orange-500\/0:is(.dark *){caret-color:rgba(249,115,22,0)}.dark\:caret-orange-500\/10:is(.dark *){caret-color:rgba(249,115,22,.1)}.dark\:caret-orange-500\/100:is(.dark *){caret-color:#f97316}.dark\:caret-orange-500\/15:is(.dark *){caret-color:rgba(249,115,22,.15)}.dark\:caret-orange-500\/20:is(.dark *){caret-color:rgba(249,115,22,.2)}.dark\:caret-orange-500\/25:is(.dark *){caret-color:rgba(249,115,22,.25)}.dark\:caret-orange-500\/30:is(.dark *){caret-color:rgba(249,115,22,.3)}.dark\:caret-orange-500\/35:is(.dark *){caret-color:rgba(249,115,22,.35)}.dark\:caret-orange-500\/40:is(.dark *){caret-color:rgba(249,115,22,.4)}.dark\:caret-orange-500\/45:is(.dark *){caret-color:rgba(249,115,22,.45)}.dark\:caret-orange-500\/5:is(.dark *){caret-color:rgba(249,115,22,.05)}.dark\:caret-orange-500\/50:is(.dark *){caret-color:rgba(249,115,22,.5)}.dark\:caret-orange-500\/55:is(.dark *){caret-color:rgba(249,115,22,.55)}.dark\:caret-orange-500\/60:is(.dark *){caret-color:rgba(249,115,22,.6)}.dark\:caret-orange-500\/65:is(.dark *){caret-color:rgba(249,115,22,.65)}.dark\:caret-orange-500\/70:is(.dark *){caret-color:rgba(249,115,22,.7)}.dark\:caret-orange-500\/75:is(.dark *){caret-color:rgba(249,115,22,.75)}.dark\:caret-orange-500\/80:is(.dark *){caret-color:rgba(249,115,22,.8)}.dark\:caret-orange-500\/85:is(.dark *){caret-color:rgba(249,115,22,.85)}.dark\:caret-orange-500\/90:is(.dark *){caret-color:rgba(249,115,22,.9)}.dark\:caret-orange-500\/95:is(.dark *){caret-color:rgba(249,115,22,.95)}.dark\:caret-orange-950:is(.dark *){caret-color:#431407}.dark\:caret-orange-950\/0:is(.dark *){caret-color:rgba(67,20,7,0)}.dark\:caret-orange-950\/10:is(.dark *){caret-color:rgba(67,20,7,.1)}.dark\:caret-orange-950\/100:is(.dark *){caret-color:#431407}.dark\:caret-orange-950\/15:is(.dark *){caret-color:rgba(67,20,7,.15)}.dark\:caret-orange-950\/20:is(.dark *){caret-color:rgba(67,20,7,.2)}.dark\:caret-orange-950\/25:is(.dark *){caret-color:rgba(67,20,7,.25)}.dark\:caret-orange-950\/30:is(.dark *){caret-color:rgba(67,20,7,.3)}.dark\:caret-orange-950\/35:is(.dark *){caret-color:rgba(67,20,7,.35)}.dark\:caret-orange-950\/40:is(.dark *){caret-color:rgba(67,20,7,.4)}.dark\:caret-orange-950\/45:is(.dark *){caret-color:rgba(67,20,7,.45)}.dark\:caret-orange-950\/5:is(.dark *){caret-color:rgba(67,20,7,.05)}.dark\:caret-orange-950\/50:is(.dark *){caret-color:rgba(67,20,7,.5)}.dark\:caret-orange-950\/55:is(.dark *){caret-color:rgba(67,20,7,.55)}.dark\:caret-orange-950\/60:is(.dark *){caret-color:rgba(67,20,7,.6)}.dark\:caret-orange-950\/65:is(.dark *){caret-color:rgba(67,20,7,.65)}.dark\:caret-orange-950\/70:is(.dark *){caret-color:rgba(67,20,7,.7)}.dark\:caret-orange-950\/75:is(.dark *){caret-color:rgba(67,20,7,.75)}.dark\:caret-orange-950\/80:is(.dark *){caret-color:rgba(67,20,7,.8)}.dark\:caret-orange-950\/85:is(.dark *){caret-color:rgba(67,20,7,.85)}.dark\:caret-orange-950\/90:is(.dark *){caret-color:rgba(67,20,7,.9)}.dark\:caret-orange-950\/95:is(.dark *){caret-color:rgba(67,20,7,.95)}.dark\:caret-pink-100:is(.dark *){caret-color:#fce7f3}.dark\:caret-pink-100\/0:is(.dark *){caret-color:rgba(252,231,243,0)}.dark\:caret-pink-100\/10:is(.dark *){caret-color:rgba(252,231,243,.1)}.dark\:caret-pink-100\/100:is(.dark *){caret-color:#fce7f3}.dark\:caret-pink-100\/15:is(.dark *){caret-color:rgba(252,231,243,.15)}.dark\:caret-pink-100\/20:is(.dark *){caret-color:rgba(252,231,243,.2)}.dark\:caret-pink-100\/25:is(.dark *){caret-color:rgba(252,231,243,.25)}.dark\:caret-pink-100\/30:is(.dark *){caret-color:rgba(252,231,243,.3)}.dark\:caret-pink-100\/35:is(.dark *){caret-color:rgba(252,231,243,.35)}.dark\:caret-pink-100\/40:is(.dark *){caret-color:rgba(252,231,243,.4)}.dark\:caret-pink-100\/45:is(.dark *){caret-color:rgba(252,231,243,.45)}.dark\:caret-pink-100\/5:is(.dark *){caret-color:rgba(252,231,243,.05)}.dark\:caret-pink-100\/50:is(.dark *){caret-color:rgba(252,231,243,.5)}.dark\:caret-pink-100\/55:is(.dark *){caret-color:rgba(252,231,243,.55)}.dark\:caret-pink-100\/60:is(.dark *){caret-color:rgba(252,231,243,.6)}.dark\:caret-pink-100\/65:is(.dark *){caret-color:rgba(252,231,243,.65)}.dark\:caret-pink-100\/70:is(.dark *){caret-color:rgba(252,231,243,.7)}.dark\:caret-pink-100\/75:is(.dark *){caret-color:rgba(252,231,243,.75)}.dark\:caret-pink-100\/80:is(.dark *){caret-color:rgba(252,231,243,.8)}.dark\:caret-pink-100\/85:is(.dark *){caret-color:rgba(252,231,243,.85)}.dark\:caret-pink-100\/90:is(.dark *){caret-color:rgba(252,231,243,.9)}.dark\:caret-pink-100\/95:is(.dark *){caret-color:rgba(252,231,243,.95)}.dark\:caret-pink-200:is(.dark *){caret-color:#fbcfe8}.dark\:caret-pink-200\/0:is(.dark *){caret-color:rgba(251,207,232,0)}.dark\:caret-pink-200\/10:is(.dark *){caret-color:rgba(251,207,232,.1)}.dark\:caret-pink-200\/100:is(.dark *){caret-color:#fbcfe8}.dark\:caret-pink-200\/15:is(.dark *){caret-color:rgba(251,207,232,.15)}.dark\:caret-pink-200\/20:is(.dark *){caret-color:rgba(251,207,232,.2)}.dark\:caret-pink-200\/25:is(.dark *){caret-color:rgba(251,207,232,.25)}.dark\:caret-pink-200\/30:is(.dark *){caret-color:rgba(251,207,232,.3)}.dark\:caret-pink-200\/35:is(.dark *){caret-color:rgba(251,207,232,.35)}.dark\:caret-pink-200\/40:is(.dark *){caret-color:rgba(251,207,232,.4)}.dark\:caret-pink-200\/45:is(.dark *){caret-color:rgba(251,207,232,.45)}.dark\:caret-pink-200\/5:is(.dark *){caret-color:rgba(251,207,232,.05)}.dark\:caret-pink-200\/50:is(.dark *){caret-color:rgba(251,207,232,.5)}.dark\:caret-pink-200\/55:is(.dark *){caret-color:rgba(251,207,232,.55)}.dark\:caret-pink-200\/60:is(.dark *){caret-color:rgba(251,207,232,.6)}.dark\:caret-pink-200\/65:is(.dark *){caret-color:rgba(251,207,232,.65)}.dark\:caret-pink-200\/70:is(.dark *){caret-color:rgba(251,207,232,.7)}.dark\:caret-pink-200\/75:is(.dark *){caret-color:rgba(251,207,232,.75)}.dark\:caret-pink-200\/80:is(.dark *){caret-color:rgba(251,207,232,.8)}.dark\:caret-pink-200\/85:is(.dark *){caret-color:rgba(251,207,232,.85)}.dark\:caret-pink-200\/90:is(.dark *){caret-color:rgba(251,207,232,.9)}.dark\:caret-pink-200\/95:is(.dark *){caret-color:rgba(251,207,232,.95)}.dark\:caret-pink-400:is(.dark *){caret-color:#f472b6}.dark\:caret-pink-400\/0:is(.dark *){caret-color:rgba(244,114,182,0)}.dark\:caret-pink-400\/10:is(.dark *){caret-color:rgba(244,114,182,.1)}.dark\:caret-pink-400\/100:is(.dark *){caret-color:#f472b6}.dark\:caret-pink-400\/15:is(.dark *){caret-color:rgba(244,114,182,.15)}.dark\:caret-pink-400\/20:is(.dark *){caret-color:rgba(244,114,182,.2)}.dark\:caret-pink-400\/25:is(.dark *){caret-color:rgba(244,114,182,.25)}.dark\:caret-pink-400\/30:is(.dark *){caret-color:rgba(244,114,182,.3)}.dark\:caret-pink-400\/35:is(.dark *){caret-color:rgba(244,114,182,.35)}.dark\:caret-pink-400\/40:is(.dark *){caret-color:rgba(244,114,182,.4)}.dark\:caret-pink-400\/45:is(.dark *){caret-color:rgba(244,114,182,.45)}.dark\:caret-pink-400\/5:is(.dark *){caret-color:rgba(244,114,182,.05)}.dark\:caret-pink-400\/50:is(.dark *){caret-color:rgba(244,114,182,.5)}.dark\:caret-pink-400\/55:is(.dark *){caret-color:rgba(244,114,182,.55)}.dark\:caret-pink-400\/60:is(.dark *){caret-color:rgba(244,114,182,.6)}.dark\:caret-pink-400\/65:is(.dark *){caret-color:rgba(244,114,182,.65)}.dark\:caret-pink-400\/70:is(.dark *){caret-color:rgba(244,114,182,.7)}.dark\:caret-pink-400\/75:is(.dark *){caret-color:rgba(244,114,182,.75)}.dark\:caret-pink-400\/80:is(.dark *){caret-color:rgba(244,114,182,.8)}.dark\:caret-pink-400\/85:is(.dark *){caret-color:rgba(244,114,182,.85)}.dark\:caret-pink-400\/90:is(.dark *){caret-color:rgba(244,114,182,.9)}.dark\:caret-pink-400\/95:is(.dark *){caret-color:rgba(244,114,182,.95)}.dark\:caret-pink-50:is(.dark *){caret-color:#fdf2f8}.dark\:caret-pink-50\/0:is(.dark *){caret-color:rgba(253,242,248,0)}.dark\:caret-pink-50\/10:is(.dark *){caret-color:rgba(253,242,248,.1)}.dark\:caret-pink-50\/100:is(.dark *){caret-color:#fdf2f8}.dark\:caret-pink-50\/15:is(.dark *){caret-color:rgba(253,242,248,.15)}.dark\:caret-pink-50\/20:is(.dark *){caret-color:rgba(253,242,248,.2)}.dark\:caret-pink-50\/25:is(.dark *){caret-color:rgba(253,242,248,.25)}.dark\:caret-pink-50\/30:is(.dark *){caret-color:rgba(253,242,248,.3)}.dark\:caret-pink-50\/35:is(.dark *){caret-color:rgba(253,242,248,.35)}.dark\:caret-pink-50\/40:is(.dark *){caret-color:rgba(253,242,248,.4)}.dark\:caret-pink-50\/45:is(.dark *){caret-color:rgba(253,242,248,.45)}.dark\:caret-pink-50\/5:is(.dark *){caret-color:rgba(253,242,248,.05)}.dark\:caret-pink-50\/50:is(.dark *){caret-color:rgba(253,242,248,.5)}.dark\:caret-pink-50\/55:is(.dark *){caret-color:rgba(253,242,248,.55)}.dark\:caret-pink-50\/60:is(.dark *){caret-color:rgba(253,242,248,.6)}.dark\:caret-pink-50\/65:is(.dark *){caret-color:rgba(253,242,248,.65)}.dark\:caret-pink-50\/70:is(.dark *){caret-color:rgba(253,242,248,.7)}.dark\:caret-pink-50\/75:is(.dark *){caret-color:rgba(253,242,248,.75)}.dark\:caret-pink-50\/80:is(.dark *){caret-color:rgba(253,242,248,.8)}.dark\:caret-pink-50\/85:is(.dark *){caret-color:rgba(253,242,248,.85)}.dark\:caret-pink-50\/90:is(.dark *){caret-color:rgba(253,242,248,.9)}.dark\:caret-pink-50\/95:is(.dark *){caret-color:rgba(253,242,248,.95)}.dark\:caret-pink-500:is(.dark *){caret-color:#ec4899}.dark\:caret-pink-500\/0:is(.dark *){caret-color:rgba(236,72,153,0)}.dark\:caret-pink-500\/10:is(.dark *){caret-color:rgba(236,72,153,.1)}.dark\:caret-pink-500\/100:is(.dark *){caret-color:#ec4899}.dark\:caret-pink-500\/15:is(.dark *){caret-color:rgba(236,72,153,.15)}.dark\:caret-pink-500\/20:is(.dark *){caret-color:rgba(236,72,153,.2)}.dark\:caret-pink-500\/25:is(.dark *){caret-color:rgba(236,72,153,.25)}.dark\:caret-pink-500\/30:is(.dark *){caret-color:rgba(236,72,153,.3)}.dark\:caret-pink-500\/35:is(.dark *){caret-color:rgba(236,72,153,.35)}.dark\:caret-pink-500\/40:is(.dark *){caret-color:rgba(236,72,153,.4)}.dark\:caret-pink-500\/45:is(.dark *){caret-color:rgba(236,72,153,.45)}.dark\:caret-pink-500\/5:is(.dark *){caret-color:rgba(236,72,153,.05)}.dark\:caret-pink-500\/50:is(.dark *){caret-color:rgba(236,72,153,.5)}.dark\:caret-pink-500\/55:is(.dark *){caret-color:rgba(236,72,153,.55)}.dark\:caret-pink-500\/60:is(.dark *){caret-color:rgba(236,72,153,.6)}.dark\:caret-pink-500\/65:is(.dark *){caret-color:rgba(236,72,153,.65)}.dark\:caret-pink-500\/70:is(.dark *){caret-color:rgba(236,72,153,.7)}.dark\:caret-pink-500\/75:is(.dark *){caret-color:rgba(236,72,153,.75)}.dark\:caret-pink-500\/80:is(.dark *){caret-color:rgba(236,72,153,.8)}.dark\:caret-pink-500\/85:is(.dark *){caret-color:rgba(236,72,153,.85)}.dark\:caret-pink-500\/90:is(.dark *){caret-color:rgba(236,72,153,.9)}.dark\:caret-pink-500\/95:is(.dark *){caret-color:rgba(236,72,153,.95)}.dark\:caret-pink-950:is(.dark *){caret-color:#500724}.dark\:caret-pink-950\/0:is(.dark *){caret-color:rgba(80,7,36,0)}.dark\:caret-pink-950\/10:is(.dark *){caret-color:rgba(80,7,36,.1)}.dark\:caret-pink-950\/100:is(.dark *){caret-color:#500724}.dark\:caret-pink-950\/15:is(.dark *){caret-color:rgba(80,7,36,.15)}.dark\:caret-pink-950\/20:is(.dark *){caret-color:rgba(80,7,36,.2)}.dark\:caret-pink-950\/25:is(.dark *){caret-color:rgba(80,7,36,.25)}.dark\:caret-pink-950\/30:is(.dark *){caret-color:rgba(80,7,36,.3)}.dark\:caret-pink-950\/35:is(.dark *){caret-color:rgba(80,7,36,.35)}.dark\:caret-pink-950\/40:is(.dark *){caret-color:rgba(80,7,36,.4)}.dark\:caret-pink-950\/45:is(.dark *){caret-color:rgba(80,7,36,.45)}.dark\:caret-pink-950\/5:is(.dark *){caret-color:rgba(80,7,36,.05)}.dark\:caret-pink-950\/50:is(.dark *){caret-color:rgba(80,7,36,.5)}.dark\:caret-pink-950\/55:is(.dark *){caret-color:rgba(80,7,36,.55)}.dark\:caret-pink-950\/60:is(.dark *){caret-color:rgba(80,7,36,.6)}.dark\:caret-pink-950\/65:is(.dark *){caret-color:rgba(80,7,36,.65)}.dark\:caret-pink-950\/70:is(.dark *){caret-color:rgba(80,7,36,.7)}.dark\:caret-pink-950\/75:is(.dark *){caret-color:rgba(80,7,36,.75)}.dark\:caret-pink-950\/80:is(.dark *){caret-color:rgba(80,7,36,.8)}.dark\:caret-pink-950\/85:is(.dark *){caret-color:rgba(80,7,36,.85)}.dark\:caret-pink-950\/90:is(.dark *){caret-color:rgba(80,7,36,.9)}.dark\:caret-pink-950\/95:is(.dark *){caret-color:rgba(80,7,36,.95)}.dark\:caret-purple-100:is(.dark *){caret-color:#f3e8ff}.dark\:caret-purple-100\/0:is(.dark *){caret-color:rgba(243,232,255,0)}.dark\:caret-purple-100\/10:is(.dark *){caret-color:rgba(243,232,255,.1)}.dark\:caret-purple-100\/100:is(.dark *){caret-color:#f3e8ff}.dark\:caret-purple-100\/15:is(.dark *){caret-color:rgba(243,232,255,.15)}.dark\:caret-purple-100\/20:is(.dark *){caret-color:rgba(243,232,255,.2)}.dark\:caret-purple-100\/25:is(.dark *){caret-color:rgba(243,232,255,.25)}.dark\:caret-purple-100\/30:is(.dark *){caret-color:rgba(243,232,255,.3)}.dark\:caret-purple-100\/35:is(.dark *){caret-color:rgba(243,232,255,.35)}.dark\:caret-purple-100\/40:is(.dark *){caret-color:rgba(243,232,255,.4)}.dark\:caret-purple-100\/45:is(.dark *){caret-color:rgba(243,232,255,.45)}.dark\:caret-purple-100\/5:is(.dark *){caret-color:rgba(243,232,255,.05)}.dark\:caret-purple-100\/50:is(.dark *){caret-color:rgba(243,232,255,.5)}.dark\:caret-purple-100\/55:is(.dark *){caret-color:rgba(243,232,255,.55)}.dark\:caret-purple-100\/60:is(.dark *){caret-color:rgba(243,232,255,.6)}.dark\:caret-purple-100\/65:is(.dark *){caret-color:rgba(243,232,255,.65)}.dark\:caret-purple-100\/70:is(.dark *){caret-color:rgba(243,232,255,.7)}.dark\:caret-purple-100\/75:is(.dark *){caret-color:rgba(243,232,255,.75)}.dark\:caret-purple-100\/80:is(.dark *){caret-color:rgba(243,232,255,.8)}.dark\:caret-purple-100\/85:is(.dark *){caret-color:rgba(243,232,255,.85)}.dark\:caret-purple-100\/90:is(.dark *){caret-color:rgba(243,232,255,.9)}.dark\:caret-purple-100\/95:is(.dark *){caret-color:rgba(243,232,255,.95)}.dark\:caret-purple-200:is(.dark *){caret-color:#e9d5ff}.dark\:caret-purple-200\/0:is(.dark *){caret-color:rgba(233,213,255,0)}.dark\:caret-purple-200\/10:is(.dark *){caret-color:rgba(233,213,255,.1)}.dark\:caret-purple-200\/100:is(.dark *){caret-color:#e9d5ff}.dark\:caret-purple-200\/15:is(.dark *){caret-color:rgba(233,213,255,.15)}.dark\:caret-purple-200\/20:is(.dark *){caret-color:rgba(233,213,255,.2)}.dark\:caret-purple-200\/25:is(.dark *){caret-color:rgba(233,213,255,.25)}.dark\:caret-purple-200\/30:is(.dark *){caret-color:rgba(233,213,255,.3)}.dark\:caret-purple-200\/35:is(.dark *){caret-color:rgba(233,213,255,.35)}.dark\:caret-purple-200\/40:is(.dark *){caret-color:rgba(233,213,255,.4)}.dark\:caret-purple-200\/45:is(.dark *){caret-color:rgba(233,213,255,.45)}.dark\:caret-purple-200\/5:is(.dark *){caret-color:rgba(233,213,255,.05)}.dark\:caret-purple-200\/50:is(.dark *){caret-color:rgba(233,213,255,.5)}.dark\:caret-purple-200\/55:is(.dark *){caret-color:rgba(233,213,255,.55)}.dark\:caret-purple-200\/60:is(.dark *){caret-color:rgba(233,213,255,.6)}.dark\:caret-purple-200\/65:is(.dark *){caret-color:rgba(233,213,255,.65)}.dark\:caret-purple-200\/70:is(.dark *){caret-color:rgba(233,213,255,.7)}.dark\:caret-purple-200\/75:is(.dark *){caret-color:rgba(233,213,255,.75)}.dark\:caret-purple-200\/80:is(.dark *){caret-color:rgba(233,213,255,.8)}.dark\:caret-purple-200\/85:is(.dark *){caret-color:rgba(233,213,255,.85)}.dark\:caret-purple-200\/90:is(.dark *){caret-color:rgba(233,213,255,.9)}.dark\:caret-purple-200\/95:is(.dark *){caret-color:rgba(233,213,255,.95)}.dark\:caret-purple-400:is(.dark *){caret-color:#c084fc}.dark\:caret-purple-400\/0:is(.dark *){caret-color:rgba(192,132,252,0)}.dark\:caret-purple-400\/10:is(.dark *){caret-color:rgba(192,132,252,.1)}.dark\:caret-purple-400\/100:is(.dark *){caret-color:#c084fc}.dark\:caret-purple-400\/15:is(.dark *){caret-color:rgba(192,132,252,.15)}.dark\:caret-purple-400\/20:is(.dark *){caret-color:rgba(192,132,252,.2)}.dark\:caret-purple-400\/25:is(.dark *){caret-color:rgba(192,132,252,.25)}.dark\:caret-purple-400\/30:is(.dark *){caret-color:rgba(192,132,252,.3)}.dark\:caret-purple-400\/35:is(.dark *){caret-color:rgba(192,132,252,.35)}.dark\:caret-purple-400\/40:is(.dark *){caret-color:rgba(192,132,252,.4)}.dark\:caret-purple-400\/45:is(.dark *){caret-color:rgba(192,132,252,.45)}.dark\:caret-purple-400\/5:is(.dark *){caret-color:rgba(192,132,252,.05)}.dark\:caret-purple-400\/50:is(.dark *){caret-color:rgba(192,132,252,.5)}.dark\:caret-purple-400\/55:is(.dark *){caret-color:rgba(192,132,252,.55)}.dark\:caret-purple-400\/60:is(.dark *){caret-color:rgba(192,132,252,.6)}.dark\:caret-purple-400\/65:is(.dark *){caret-color:rgba(192,132,252,.65)}.dark\:caret-purple-400\/70:is(.dark *){caret-color:rgba(192,132,252,.7)}.dark\:caret-purple-400\/75:is(.dark *){caret-color:rgba(192,132,252,.75)}.dark\:caret-purple-400\/80:is(.dark *){caret-color:rgba(192,132,252,.8)}.dark\:caret-purple-400\/85:is(.dark *){caret-color:rgba(192,132,252,.85)}.dark\:caret-purple-400\/90:is(.dark *){caret-color:rgba(192,132,252,.9)}.dark\:caret-purple-400\/95:is(.dark *){caret-color:rgba(192,132,252,.95)}.dark\:caret-purple-50:is(.dark *){caret-color:#faf5ff}.dark\:caret-purple-50\/0:is(.dark *){caret-color:rgba(250,245,255,0)}.dark\:caret-purple-50\/10:is(.dark *){caret-color:rgba(250,245,255,.1)}.dark\:caret-purple-50\/100:is(.dark *){caret-color:#faf5ff}.dark\:caret-purple-50\/15:is(.dark *){caret-color:rgba(250,245,255,.15)}.dark\:caret-purple-50\/20:is(.dark *){caret-color:rgba(250,245,255,.2)}.dark\:caret-purple-50\/25:is(.dark *){caret-color:rgba(250,245,255,.25)}.dark\:caret-purple-50\/30:is(.dark *){caret-color:rgba(250,245,255,.3)}.dark\:caret-purple-50\/35:is(.dark *){caret-color:rgba(250,245,255,.35)}.dark\:caret-purple-50\/40:is(.dark *){caret-color:rgba(250,245,255,.4)}.dark\:caret-purple-50\/45:is(.dark *){caret-color:rgba(250,245,255,.45)}.dark\:caret-purple-50\/5:is(.dark *){caret-color:rgba(250,245,255,.05)}.dark\:caret-purple-50\/50:is(.dark *){caret-color:rgba(250,245,255,.5)}.dark\:caret-purple-50\/55:is(.dark *){caret-color:rgba(250,245,255,.55)}.dark\:caret-purple-50\/60:is(.dark *){caret-color:rgba(250,245,255,.6)}.dark\:caret-purple-50\/65:is(.dark *){caret-color:rgba(250,245,255,.65)}.dark\:caret-purple-50\/70:is(.dark *){caret-color:rgba(250,245,255,.7)}.dark\:caret-purple-50\/75:is(.dark *){caret-color:rgba(250,245,255,.75)}.dark\:caret-purple-50\/80:is(.dark *){caret-color:rgba(250,245,255,.8)}.dark\:caret-purple-50\/85:is(.dark *){caret-color:rgba(250,245,255,.85)}.dark\:caret-purple-50\/90:is(.dark *){caret-color:rgba(250,245,255,.9)}.dark\:caret-purple-50\/95:is(.dark *){caret-color:rgba(250,245,255,.95)}.dark\:caret-purple-500:is(.dark *){caret-color:#a855f7}.dark\:caret-purple-500\/0:is(.dark *){caret-color:rgba(168,85,247,0)}.dark\:caret-purple-500\/10:is(.dark *){caret-color:rgba(168,85,247,.1)}.dark\:caret-purple-500\/100:is(.dark *){caret-color:#a855f7}.dark\:caret-purple-500\/15:is(.dark *){caret-color:rgba(168,85,247,.15)}.dark\:caret-purple-500\/20:is(.dark *){caret-color:rgba(168,85,247,.2)}.dark\:caret-purple-500\/25:is(.dark *){caret-color:rgba(168,85,247,.25)}.dark\:caret-purple-500\/30:is(.dark *){caret-color:rgba(168,85,247,.3)}.dark\:caret-purple-500\/35:is(.dark *){caret-color:rgba(168,85,247,.35)}.dark\:caret-purple-500\/40:is(.dark *){caret-color:rgba(168,85,247,.4)}.dark\:caret-purple-500\/45:is(.dark *){caret-color:rgba(168,85,247,.45)}.dark\:caret-purple-500\/5:is(.dark *){caret-color:rgba(168,85,247,.05)}.dark\:caret-purple-500\/50:is(.dark *){caret-color:rgba(168,85,247,.5)}.dark\:caret-purple-500\/55:is(.dark *){caret-color:rgba(168,85,247,.55)}.dark\:caret-purple-500\/60:is(.dark *){caret-color:rgba(168,85,247,.6)}.dark\:caret-purple-500\/65:is(.dark *){caret-color:rgba(168,85,247,.65)}.dark\:caret-purple-500\/70:is(.dark *){caret-color:rgba(168,85,247,.7)}.dark\:caret-purple-500\/75:is(.dark *){caret-color:rgba(168,85,247,.75)}.dark\:caret-purple-500\/80:is(.dark *){caret-color:rgba(168,85,247,.8)}.dark\:caret-purple-500\/85:is(.dark *){caret-color:rgba(168,85,247,.85)}.dark\:caret-purple-500\/90:is(.dark *){caret-color:rgba(168,85,247,.9)}.dark\:caret-purple-500\/95:is(.dark *){caret-color:rgba(168,85,247,.95)}.dark\:caret-purple-950:is(.dark *){caret-color:#3b0764}.dark\:caret-purple-950\/0:is(.dark *){caret-color:rgba(59,7,100,0)}.dark\:caret-purple-950\/10:is(.dark *){caret-color:rgba(59,7,100,.1)}.dark\:caret-purple-950\/100:is(.dark *){caret-color:#3b0764}.dark\:caret-purple-950\/15:is(.dark *){caret-color:rgba(59,7,100,.15)}.dark\:caret-purple-950\/20:is(.dark *){caret-color:rgba(59,7,100,.2)}.dark\:caret-purple-950\/25:is(.dark *){caret-color:rgba(59,7,100,.25)}.dark\:caret-purple-950\/30:is(.dark *){caret-color:rgba(59,7,100,.3)}.dark\:caret-purple-950\/35:is(.dark *){caret-color:rgba(59,7,100,.35)}.dark\:caret-purple-950\/40:is(.dark *){caret-color:rgba(59,7,100,.4)}.dark\:caret-purple-950\/45:is(.dark *){caret-color:rgba(59,7,100,.45)}.dark\:caret-purple-950\/5:is(.dark *){caret-color:rgba(59,7,100,.05)}.dark\:caret-purple-950\/50:is(.dark *){caret-color:rgba(59,7,100,.5)}.dark\:caret-purple-950\/55:is(.dark *){caret-color:rgba(59,7,100,.55)}.dark\:caret-purple-950\/60:is(.dark *){caret-color:rgba(59,7,100,.6)}.dark\:caret-purple-950\/65:is(.dark *){caret-color:rgba(59,7,100,.65)}.dark\:caret-purple-950\/70:is(.dark *){caret-color:rgba(59,7,100,.7)}.dark\:caret-purple-950\/75:is(.dark *){caret-color:rgba(59,7,100,.75)}.dark\:caret-purple-950\/80:is(.dark *){caret-color:rgba(59,7,100,.8)}.dark\:caret-purple-950\/85:is(.dark *){caret-color:rgba(59,7,100,.85)}.dark\:caret-purple-950\/90:is(.dark *){caret-color:rgba(59,7,100,.9)}.dark\:caret-purple-950\/95:is(.dark *){caret-color:rgba(59,7,100,.95)}.dark\:caret-red-100:is(.dark *){caret-color:#fee2e2}.dark\:caret-red-100\/0:is(.dark *){caret-color:hsla(0,93%,94%,0)}.dark\:caret-red-100\/10:is(.dark *){caret-color:hsla(0,93%,94%,.1)}.dark\:caret-red-100\/100:is(.dark *){caret-color:#fee2e2}.dark\:caret-red-100\/15:is(.dark *){caret-color:hsla(0,93%,94%,.15)}.dark\:caret-red-100\/20:is(.dark *){caret-color:hsla(0,93%,94%,.2)}.dark\:caret-red-100\/25:is(.dark *){caret-color:hsla(0,93%,94%,.25)}.dark\:caret-red-100\/30:is(.dark *){caret-color:hsla(0,93%,94%,.3)}.dark\:caret-red-100\/35:is(.dark *){caret-color:hsla(0,93%,94%,.35)}.dark\:caret-red-100\/40:is(.dark *){caret-color:hsla(0,93%,94%,.4)}.dark\:caret-red-100\/45:is(.dark *){caret-color:hsla(0,93%,94%,.45)}.dark\:caret-red-100\/5:is(.dark *){caret-color:hsla(0,93%,94%,.05)}.dark\:caret-red-100\/50:is(.dark *){caret-color:hsla(0,93%,94%,.5)}.dark\:caret-red-100\/55:is(.dark *){caret-color:hsla(0,93%,94%,.55)}.dark\:caret-red-100\/60:is(.dark *){caret-color:hsla(0,93%,94%,.6)}.dark\:caret-red-100\/65:is(.dark *){caret-color:hsla(0,93%,94%,.65)}.dark\:caret-red-100\/70:is(.dark *){caret-color:hsla(0,93%,94%,.7)}.dark\:caret-red-100\/75:is(.dark *){caret-color:hsla(0,93%,94%,.75)}.dark\:caret-red-100\/80:is(.dark *){caret-color:hsla(0,93%,94%,.8)}.dark\:caret-red-100\/85:is(.dark *){caret-color:hsla(0,93%,94%,.85)}.dark\:caret-red-100\/90:is(.dark *){caret-color:hsla(0,93%,94%,.9)}.dark\:caret-red-100\/95:is(.dark *){caret-color:hsla(0,93%,94%,.95)}.dark\:caret-red-200:is(.dark *){caret-color:#fecaca}.dark\:caret-red-200\/0:is(.dark *){caret-color:hsla(0,96%,89%,0)}.dark\:caret-red-200\/10:is(.dark *){caret-color:hsla(0,96%,89%,.1)}.dark\:caret-red-200\/100:is(.dark *){caret-color:#fecaca}.dark\:caret-red-200\/15:is(.dark *){caret-color:hsla(0,96%,89%,.15)}.dark\:caret-red-200\/20:is(.dark *){caret-color:hsla(0,96%,89%,.2)}.dark\:caret-red-200\/25:is(.dark *){caret-color:hsla(0,96%,89%,.25)}.dark\:caret-red-200\/30:is(.dark *){caret-color:hsla(0,96%,89%,.3)}.dark\:caret-red-200\/35:is(.dark *){caret-color:hsla(0,96%,89%,.35)}.dark\:caret-red-200\/40:is(.dark *){caret-color:hsla(0,96%,89%,.4)}.dark\:caret-red-200\/45:is(.dark *){caret-color:hsla(0,96%,89%,.45)}.dark\:caret-red-200\/5:is(.dark *){caret-color:hsla(0,96%,89%,.05)}.dark\:caret-red-200\/50:is(.dark *){caret-color:hsla(0,96%,89%,.5)}.dark\:caret-red-200\/55:is(.dark *){caret-color:hsla(0,96%,89%,.55)}.dark\:caret-red-200\/60:is(.dark *){caret-color:hsla(0,96%,89%,.6)}.dark\:caret-red-200\/65:is(.dark *){caret-color:hsla(0,96%,89%,.65)}.dark\:caret-red-200\/70:is(.dark *){caret-color:hsla(0,96%,89%,.7)}.dark\:caret-red-200\/75:is(.dark *){caret-color:hsla(0,96%,89%,.75)}.dark\:caret-red-200\/80:is(.dark *){caret-color:hsla(0,96%,89%,.8)}.dark\:caret-red-200\/85:is(.dark *){caret-color:hsla(0,96%,89%,.85)}.dark\:caret-red-200\/90:is(.dark *){caret-color:hsla(0,96%,89%,.9)}.dark\:caret-red-200\/95:is(.dark *){caret-color:hsla(0,96%,89%,.95)}.dark\:caret-red-400:is(.dark *){caret-color:#f87171}.dark\:caret-red-400\/0:is(.dark *){caret-color:hsla(0,91%,71%,0)}.dark\:caret-red-400\/10:is(.dark *){caret-color:hsla(0,91%,71%,.1)}.dark\:caret-red-400\/100:is(.dark *){caret-color:#f87171}.dark\:caret-red-400\/15:is(.dark *){caret-color:hsla(0,91%,71%,.15)}.dark\:caret-red-400\/20:is(.dark *){caret-color:hsla(0,91%,71%,.2)}.dark\:caret-red-400\/25:is(.dark *){caret-color:hsla(0,91%,71%,.25)}.dark\:caret-red-400\/30:is(.dark *){caret-color:hsla(0,91%,71%,.3)}.dark\:caret-red-400\/35:is(.dark *){caret-color:hsla(0,91%,71%,.35)}.dark\:caret-red-400\/40:is(.dark *){caret-color:hsla(0,91%,71%,.4)}.dark\:caret-red-400\/45:is(.dark *){caret-color:hsla(0,91%,71%,.45)}.dark\:caret-red-400\/5:is(.dark *){caret-color:hsla(0,91%,71%,.05)}.dark\:caret-red-400\/50:is(.dark *){caret-color:hsla(0,91%,71%,.5)}.dark\:caret-red-400\/55:is(.dark *){caret-color:hsla(0,91%,71%,.55)}.dark\:caret-red-400\/60:is(.dark *){caret-color:hsla(0,91%,71%,.6)}.dark\:caret-red-400\/65:is(.dark *){caret-color:hsla(0,91%,71%,.65)}.dark\:caret-red-400\/70:is(.dark *){caret-color:hsla(0,91%,71%,.7)}.dark\:caret-red-400\/75:is(.dark *){caret-color:hsla(0,91%,71%,.75)}.dark\:caret-red-400\/80:is(.dark *){caret-color:hsla(0,91%,71%,.8)}.dark\:caret-red-400\/85:is(.dark *){caret-color:hsla(0,91%,71%,.85)}.dark\:caret-red-400\/90:is(.dark *){caret-color:hsla(0,91%,71%,.9)}.dark\:caret-red-400\/95:is(.dark *){caret-color:hsla(0,91%,71%,.95)}.dark\:caret-red-50:is(.dark *){caret-color:#fef2f2}.dark\:caret-red-50\/0:is(.dark *){caret-color:hsla(0,86%,97%,0)}.dark\:caret-red-50\/10:is(.dark *){caret-color:hsla(0,86%,97%,.1)}.dark\:caret-red-50\/100:is(.dark *){caret-color:#fef2f2}.dark\:caret-red-50\/15:is(.dark *){caret-color:hsla(0,86%,97%,.15)}.dark\:caret-red-50\/20:is(.dark *){caret-color:hsla(0,86%,97%,.2)}.dark\:caret-red-50\/25:is(.dark *){caret-color:hsla(0,86%,97%,.25)}.dark\:caret-red-50\/30:is(.dark *){caret-color:hsla(0,86%,97%,.3)}.dark\:caret-red-50\/35:is(.dark *){caret-color:hsla(0,86%,97%,.35)}.dark\:caret-red-50\/40:is(.dark *){caret-color:hsla(0,86%,97%,.4)}.dark\:caret-red-50\/45:is(.dark *){caret-color:hsla(0,86%,97%,.45)}.dark\:caret-red-50\/5:is(.dark *){caret-color:hsla(0,86%,97%,.05)}.dark\:caret-red-50\/50:is(.dark *){caret-color:hsla(0,86%,97%,.5)}.dark\:caret-red-50\/55:is(.dark *){caret-color:hsla(0,86%,97%,.55)}.dark\:caret-red-50\/60:is(.dark *){caret-color:hsla(0,86%,97%,.6)}.dark\:caret-red-50\/65:is(.dark *){caret-color:hsla(0,86%,97%,.65)}.dark\:caret-red-50\/70:is(.dark *){caret-color:hsla(0,86%,97%,.7)}.dark\:caret-red-50\/75:is(.dark *){caret-color:hsla(0,86%,97%,.75)}.dark\:caret-red-50\/80:is(.dark *){caret-color:hsla(0,86%,97%,.8)}.dark\:caret-red-50\/85:is(.dark *){caret-color:hsla(0,86%,97%,.85)}.dark\:caret-red-50\/90:is(.dark *){caret-color:hsla(0,86%,97%,.9)}.dark\:caret-red-50\/95:is(.dark *){caret-color:hsla(0,86%,97%,.95)}.dark\:caret-red-500:is(.dark *){caret-color:#ef4444}.dark\:caret-red-500\/0:is(.dark *){caret-color:rgba(239,68,68,0)}.dark\:caret-red-500\/10:is(.dark *){caret-color:rgba(239,68,68,.1)}.dark\:caret-red-500\/100:is(.dark *){caret-color:#ef4444}.dark\:caret-red-500\/15:is(.dark *){caret-color:rgba(239,68,68,.15)}.dark\:caret-red-500\/20:is(.dark *){caret-color:rgba(239,68,68,.2)}.dark\:caret-red-500\/25:is(.dark *){caret-color:rgba(239,68,68,.25)}.dark\:caret-red-500\/30:is(.dark *){caret-color:rgba(239,68,68,.3)}.dark\:caret-red-500\/35:is(.dark *){caret-color:rgba(239,68,68,.35)}.dark\:caret-red-500\/40:is(.dark *){caret-color:rgba(239,68,68,.4)}.dark\:caret-red-500\/45:is(.dark *){caret-color:rgba(239,68,68,.45)}.dark\:caret-red-500\/5:is(.dark *){caret-color:rgba(239,68,68,.05)}.dark\:caret-red-500\/50:is(.dark *){caret-color:rgba(239,68,68,.5)}.dark\:caret-red-500\/55:is(.dark *){caret-color:rgba(239,68,68,.55)}.dark\:caret-red-500\/60:is(.dark *){caret-color:rgba(239,68,68,.6)}.dark\:caret-red-500\/65:is(.dark *){caret-color:rgba(239,68,68,.65)}.dark\:caret-red-500\/70:is(.dark *){caret-color:rgba(239,68,68,.7)}.dark\:caret-red-500\/75:is(.dark *){caret-color:rgba(239,68,68,.75)}.dark\:caret-red-500\/80:is(.dark *){caret-color:rgba(239,68,68,.8)}.dark\:caret-red-500\/85:is(.dark *){caret-color:rgba(239,68,68,.85)}.dark\:caret-red-500\/90:is(.dark *){caret-color:rgba(239,68,68,.9)}.dark\:caret-red-500\/95:is(.dark *){caret-color:rgba(239,68,68,.95)}.dark\:caret-red-950:is(.dark *){caret-color:#450a0a}.dark\:caret-red-950\/0:is(.dark *){caret-color:rgba(69,10,10,0)}.dark\:caret-red-950\/10:is(.dark *){caret-color:rgba(69,10,10,.1)}.dark\:caret-red-950\/100:is(.dark *){caret-color:#450a0a}.dark\:caret-red-950\/15:is(.dark *){caret-color:rgba(69,10,10,.15)}.dark\:caret-red-950\/20:is(.dark *){caret-color:rgba(69,10,10,.2)}.dark\:caret-red-950\/25:is(.dark *){caret-color:rgba(69,10,10,.25)}.dark\:caret-red-950\/30:is(.dark *){caret-color:rgba(69,10,10,.3)}.dark\:caret-red-950\/35:is(.dark *){caret-color:rgba(69,10,10,.35)}.dark\:caret-red-950\/40:is(.dark *){caret-color:rgba(69,10,10,.4)}.dark\:caret-red-950\/45:is(.dark *){caret-color:rgba(69,10,10,.45)}.dark\:caret-red-950\/5:is(.dark *){caret-color:rgba(69,10,10,.05)}.dark\:caret-red-950\/50:is(.dark *){caret-color:rgba(69,10,10,.5)}.dark\:caret-red-950\/55:is(.dark *){caret-color:rgba(69,10,10,.55)}.dark\:caret-red-950\/60:is(.dark *){caret-color:rgba(69,10,10,.6)}.dark\:caret-red-950\/65:is(.dark *){caret-color:rgba(69,10,10,.65)}.dark\:caret-red-950\/70:is(.dark *){caret-color:rgba(69,10,10,.7)}.dark\:caret-red-950\/75:is(.dark *){caret-color:rgba(69,10,10,.75)}.dark\:caret-red-950\/80:is(.dark *){caret-color:rgba(69,10,10,.8)}.dark\:caret-red-950\/85:is(.dark *){caret-color:rgba(69,10,10,.85)}.dark\:caret-red-950\/90:is(.dark *){caret-color:rgba(69,10,10,.9)}.dark\:caret-red-950\/95:is(.dark *){caret-color:rgba(69,10,10,.95)}.dark\:caret-rose-100:is(.dark *){caret-color:#ffe4e6}.dark\:caret-rose-100\/0:is(.dark *){caret-color:rgba(255,228,230,0)}.dark\:caret-rose-100\/10:is(.dark *){caret-color:rgba(255,228,230,.1)}.dark\:caret-rose-100\/100:is(.dark *){caret-color:#ffe4e6}.dark\:caret-rose-100\/15:is(.dark *){caret-color:rgba(255,228,230,.15)}.dark\:caret-rose-100\/20:is(.dark *){caret-color:rgba(255,228,230,.2)}.dark\:caret-rose-100\/25:is(.dark *){caret-color:rgba(255,228,230,.25)}.dark\:caret-rose-100\/30:is(.dark *){caret-color:rgba(255,228,230,.3)}.dark\:caret-rose-100\/35:is(.dark *){caret-color:rgba(255,228,230,.35)}.dark\:caret-rose-100\/40:is(.dark *){caret-color:rgba(255,228,230,.4)}.dark\:caret-rose-100\/45:is(.dark *){caret-color:rgba(255,228,230,.45)}.dark\:caret-rose-100\/5:is(.dark *){caret-color:rgba(255,228,230,.05)}.dark\:caret-rose-100\/50:is(.dark *){caret-color:rgba(255,228,230,.5)}.dark\:caret-rose-100\/55:is(.dark *){caret-color:rgba(255,228,230,.55)}.dark\:caret-rose-100\/60:is(.dark *){caret-color:rgba(255,228,230,.6)}.dark\:caret-rose-100\/65:is(.dark *){caret-color:rgba(255,228,230,.65)}.dark\:caret-rose-100\/70:is(.dark *){caret-color:rgba(255,228,230,.7)}.dark\:caret-rose-100\/75:is(.dark *){caret-color:rgba(255,228,230,.75)}.dark\:caret-rose-100\/80:is(.dark *){caret-color:rgba(255,228,230,.8)}.dark\:caret-rose-100\/85:is(.dark *){caret-color:rgba(255,228,230,.85)}.dark\:caret-rose-100\/90:is(.dark *){caret-color:rgba(255,228,230,.9)}.dark\:caret-rose-100\/95:is(.dark *){caret-color:rgba(255,228,230,.95)}.dark\:caret-rose-200:is(.dark *){caret-color:#fecdd3}.dark\:caret-rose-200\/0:is(.dark *){caret-color:rgba(254,205,211,0)}.dark\:caret-rose-200\/10:is(.dark *){caret-color:rgba(254,205,211,.1)}.dark\:caret-rose-200\/100:is(.dark *){caret-color:#fecdd3}.dark\:caret-rose-200\/15:is(.dark *){caret-color:rgba(254,205,211,.15)}.dark\:caret-rose-200\/20:is(.dark *){caret-color:rgba(254,205,211,.2)}.dark\:caret-rose-200\/25:is(.dark *){caret-color:rgba(254,205,211,.25)}.dark\:caret-rose-200\/30:is(.dark *){caret-color:rgba(254,205,211,.3)}.dark\:caret-rose-200\/35:is(.dark *){caret-color:rgba(254,205,211,.35)}.dark\:caret-rose-200\/40:is(.dark *){caret-color:rgba(254,205,211,.4)}.dark\:caret-rose-200\/45:is(.dark *){caret-color:rgba(254,205,211,.45)}.dark\:caret-rose-200\/5:is(.dark *){caret-color:rgba(254,205,211,.05)}.dark\:caret-rose-200\/50:is(.dark *){caret-color:rgba(254,205,211,.5)}.dark\:caret-rose-200\/55:is(.dark *){caret-color:rgba(254,205,211,.55)}.dark\:caret-rose-200\/60:is(.dark *){caret-color:rgba(254,205,211,.6)}.dark\:caret-rose-200\/65:is(.dark *){caret-color:rgba(254,205,211,.65)}.dark\:caret-rose-200\/70:is(.dark *){caret-color:rgba(254,205,211,.7)}.dark\:caret-rose-200\/75:is(.dark *){caret-color:rgba(254,205,211,.75)}.dark\:caret-rose-200\/80:is(.dark *){caret-color:rgba(254,205,211,.8)}.dark\:caret-rose-200\/85:is(.dark *){caret-color:rgba(254,205,211,.85)}.dark\:caret-rose-200\/90:is(.dark *){caret-color:rgba(254,205,211,.9)}.dark\:caret-rose-200\/95:is(.dark *){caret-color:rgba(254,205,211,.95)}.dark\:caret-rose-400:is(.dark *){caret-color:#fb7185}.dark\:caret-rose-400\/0:is(.dark *){caret-color:rgba(251,113,133,0)}.dark\:caret-rose-400\/10:is(.dark *){caret-color:rgba(251,113,133,.1)}.dark\:caret-rose-400\/100:is(.dark *){caret-color:#fb7185}.dark\:caret-rose-400\/15:is(.dark *){caret-color:rgba(251,113,133,.15)}.dark\:caret-rose-400\/20:is(.dark *){caret-color:rgba(251,113,133,.2)}.dark\:caret-rose-400\/25:is(.dark *){caret-color:rgba(251,113,133,.25)}.dark\:caret-rose-400\/30:is(.dark *){caret-color:rgba(251,113,133,.3)}.dark\:caret-rose-400\/35:is(.dark *){caret-color:rgba(251,113,133,.35)}.dark\:caret-rose-400\/40:is(.dark *){caret-color:rgba(251,113,133,.4)}.dark\:caret-rose-400\/45:is(.dark *){caret-color:rgba(251,113,133,.45)}.dark\:caret-rose-400\/5:is(.dark *){caret-color:rgba(251,113,133,.05)}.dark\:caret-rose-400\/50:is(.dark *){caret-color:rgba(251,113,133,.5)}.dark\:caret-rose-400\/55:is(.dark *){caret-color:rgba(251,113,133,.55)}.dark\:caret-rose-400\/60:is(.dark *){caret-color:rgba(251,113,133,.6)}.dark\:caret-rose-400\/65:is(.dark *){caret-color:rgba(251,113,133,.65)}.dark\:caret-rose-400\/70:is(.dark *){caret-color:rgba(251,113,133,.7)}.dark\:caret-rose-400\/75:is(.dark *){caret-color:rgba(251,113,133,.75)}.dark\:caret-rose-400\/80:is(.dark *){caret-color:rgba(251,113,133,.8)}.dark\:caret-rose-400\/85:is(.dark *){caret-color:rgba(251,113,133,.85)}.dark\:caret-rose-400\/90:is(.dark *){caret-color:rgba(251,113,133,.9)}.dark\:caret-rose-400\/95:is(.dark *){caret-color:rgba(251,113,133,.95)}.dark\:caret-rose-50:is(.dark *){caret-color:#fff1f2}.dark\:caret-rose-50\/0:is(.dark *){caret-color:rgba(255,241,242,0)}.dark\:caret-rose-50\/10:is(.dark *){caret-color:rgba(255,241,242,.1)}.dark\:caret-rose-50\/100:is(.dark *){caret-color:#fff1f2}.dark\:caret-rose-50\/15:is(.dark *){caret-color:rgba(255,241,242,.15)}.dark\:caret-rose-50\/20:is(.dark *){caret-color:rgba(255,241,242,.2)}.dark\:caret-rose-50\/25:is(.dark *){caret-color:rgba(255,241,242,.25)}.dark\:caret-rose-50\/30:is(.dark *){caret-color:rgba(255,241,242,.3)}.dark\:caret-rose-50\/35:is(.dark *){caret-color:rgba(255,241,242,.35)}.dark\:caret-rose-50\/40:is(.dark *){caret-color:rgba(255,241,242,.4)}.dark\:caret-rose-50\/45:is(.dark *){caret-color:rgba(255,241,242,.45)}.dark\:caret-rose-50\/5:is(.dark *){caret-color:rgba(255,241,242,.05)}.dark\:caret-rose-50\/50:is(.dark *){caret-color:rgba(255,241,242,.5)}.dark\:caret-rose-50\/55:is(.dark *){caret-color:rgba(255,241,242,.55)}.dark\:caret-rose-50\/60:is(.dark *){caret-color:rgba(255,241,242,.6)}.dark\:caret-rose-50\/65:is(.dark *){caret-color:rgba(255,241,242,.65)}.dark\:caret-rose-50\/70:is(.dark *){caret-color:rgba(255,241,242,.7)}.dark\:caret-rose-50\/75:is(.dark *){caret-color:rgba(255,241,242,.75)}.dark\:caret-rose-50\/80:is(.dark *){caret-color:rgba(255,241,242,.8)}.dark\:caret-rose-50\/85:is(.dark *){caret-color:rgba(255,241,242,.85)}.dark\:caret-rose-50\/90:is(.dark *){caret-color:rgba(255,241,242,.9)}.dark\:caret-rose-50\/95:is(.dark *){caret-color:rgba(255,241,242,.95)}.dark\:caret-rose-500:is(.dark *){caret-color:#f43f5e}.dark\:caret-rose-500\/0:is(.dark *){caret-color:rgba(244,63,94,0)}.dark\:caret-rose-500\/10:is(.dark *){caret-color:rgba(244,63,94,.1)}.dark\:caret-rose-500\/100:is(.dark *){caret-color:#f43f5e}.dark\:caret-rose-500\/15:is(.dark *){caret-color:rgba(244,63,94,.15)}.dark\:caret-rose-500\/20:is(.dark *){caret-color:rgba(244,63,94,.2)}.dark\:caret-rose-500\/25:is(.dark *){caret-color:rgba(244,63,94,.25)}.dark\:caret-rose-500\/30:is(.dark *){caret-color:rgba(244,63,94,.3)}.dark\:caret-rose-500\/35:is(.dark *){caret-color:rgba(244,63,94,.35)}.dark\:caret-rose-500\/40:is(.dark *){caret-color:rgba(244,63,94,.4)}.dark\:caret-rose-500\/45:is(.dark *){caret-color:rgba(244,63,94,.45)}.dark\:caret-rose-500\/5:is(.dark *){caret-color:rgba(244,63,94,.05)}.dark\:caret-rose-500\/50:is(.dark *){caret-color:rgba(244,63,94,.5)}.dark\:caret-rose-500\/55:is(.dark *){caret-color:rgba(244,63,94,.55)}.dark\:caret-rose-500\/60:is(.dark *){caret-color:rgba(244,63,94,.6)}.dark\:caret-rose-500\/65:is(.dark *){caret-color:rgba(244,63,94,.65)}.dark\:caret-rose-500\/70:is(.dark *){caret-color:rgba(244,63,94,.7)}.dark\:caret-rose-500\/75:is(.dark *){caret-color:rgba(244,63,94,.75)}.dark\:caret-rose-500\/80:is(.dark *){caret-color:rgba(244,63,94,.8)}.dark\:caret-rose-500\/85:is(.dark *){caret-color:rgba(244,63,94,.85)}.dark\:caret-rose-500\/90:is(.dark *){caret-color:rgba(244,63,94,.9)}.dark\:caret-rose-500\/95:is(.dark *){caret-color:rgba(244,63,94,.95)}.dark\:caret-rose-950:is(.dark *){caret-color:#4c0519}.dark\:caret-rose-950\/0:is(.dark *){caret-color:rgba(76,5,25,0)}.dark\:caret-rose-950\/10:is(.dark *){caret-color:rgba(76,5,25,.1)}.dark\:caret-rose-950\/100:is(.dark *){caret-color:#4c0519}.dark\:caret-rose-950\/15:is(.dark *){caret-color:rgba(76,5,25,.15)}.dark\:caret-rose-950\/20:is(.dark *){caret-color:rgba(76,5,25,.2)}.dark\:caret-rose-950\/25:is(.dark *){caret-color:rgba(76,5,25,.25)}.dark\:caret-rose-950\/30:is(.dark *){caret-color:rgba(76,5,25,.3)}.dark\:caret-rose-950\/35:is(.dark *){caret-color:rgba(76,5,25,.35)}.dark\:caret-rose-950\/40:is(.dark *){caret-color:rgba(76,5,25,.4)}.dark\:caret-rose-950\/45:is(.dark *){caret-color:rgba(76,5,25,.45)}.dark\:caret-rose-950\/5:is(.dark *){caret-color:rgba(76,5,25,.05)}.dark\:caret-rose-950\/50:is(.dark *){caret-color:rgba(76,5,25,.5)}.dark\:caret-rose-950\/55:is(.dark *){caret-color:rgba(76,5,25,.55)}.dark\:caret-rose-950\/60:is(.dark *){caret-color:rgba(76,5,25,.6)}.dark\:caret-rose-950\/65:is(.dark *){caret-color:rgba(76,5,25,.65)}.dark\:caret-rose-950\/70:is(.dark *){caret-color:rgba(76,5,25,.7)}.dark\:caret-rose-950\/75:is(.dark *){caret-color:rgba(76,5,25,.75)}.dark\:caret-rose-950\/80:is(.dark *){caret-color:rgba(76,5,25,.8)}.dark\:caret-rose-950\/85:is(.dark *){caret-color:rgba(76,5,25,.85)}.dark\:caret-rose-950\/90:is(.dark *){caret-color:rgba(76,5,25,.9)}.dark\:caret-rose-950\/95:is(.dark *){caret-color:rgba(76,5,25,.95)}.dark\:caret-sky-100:is(.dark *){caret-color:#e0f2fe}.dark\:caret-sky-100\/0:is(.dark *){caret-color:rgba(224,242,254,0)}.dark\:caret-sky-100\/10:is(.dark *){caret-color:rgba(224,242,254,.1)}.dark\:caret-sky-100\/100:is(.dark *){caret-color:#e0f2fe}.dark\:caret-sky-100\/15:is(.dark *){caret-color:rgba(224,242,254,.15)}.dark\:caret-sky-100\/20:is(.dark *){caret-color:rgba(224,242,254,.2)}.dark\:caret-sky-100\/25:is(.dark *){caret-color:rgba(224,242,254,.25)}.dark\:caret-sky-100\/30:is(.dark *){caret-color:rgba(224,242,254,.3)}.dark\:caret-sky-100\/35:is(.dark *){caret-color:rgba(224,242,254,.35)}.dark\:caret-sky-100\/40:is(.dark *){caret-color:rgba(224,242,254,.4)}.dark\:caret-sky-100\/45:is(.dark *){caret-color:rgba(224,242,254,.45)}.dark\:caret-sky-100\/5:is(.dark *){caret-color:rgba(224,242,254,.05)}.dark\:caret-sky-100\/50:is(.dark *){caret-color:rgba(224,242,254,.5)}.dark\:caret-sky-100\/55:is(.dark *){caret-color:rgba(224,242,254,.55)}.dark\:caret-sky-100\/60:is(.dark *){caret-color:rgba(224,242,254,.6)}.dark\:caret-sky-100\/65:is(.dark *){caret-color:rgba(224,242,254,.65)}.dark\:caret-sky-100\/70:is(.dark *){caret-color:rgba(224,242,254,.7)}.dark\:caret-sky-100\/75:is(.dark *){caret-color:rgba(224,242,254,.75)}.dark\:caret-sky-100\/80:is(.dark *){caret-color:rgba(224,242,254,.8)}.dark\:caret-sky-100\/85:is(.dark *){caret-color:rgba(224,242,254,.85)}.dark\:caret-sky-100\/90:is(.dark *){caret-color:rgba(224,242,254,.9)}.dark\:caret-sky-100\/95:is(.dark *){caret-color:rgba(224,242,254,.95)}.dark\:caret-sky-200:is(.dark *){caret-color:#bae6fd}.dark\:caret-sky-200\/0:is(.dark *){caret-color:rgba(186,230,253,0)}.dark\:caret-sky-200\/10:is(.dark *){caret-color:rgba(186,230,253,.1)}.dark\:caret-sky-200\/100:is(.dark *){caret-color:#bae6fd}.dark\:caret-sky-200\/15:is(.dark *){caret-color:rgba(186,230,253,.15)}.dark\:caret-sky-200\/20:is(.dark *){caret-color:rgba(186,230,253,.2)}.dark\:caret-sky-200\/25:is(.dark *){caret-color:rgba(186,230,253,.25)}.dark\:caret-sky-200\/30:is(.dark *){caret-color:rgba(186,230,253,.3)}.dark\:caret-sky-200\/35:is(.dark *){caret-color:rgba(186,230,253,.35)}.dark\:caret-sky-200\/40:is(.dark *){caret-color:rgba(186,230,253,.4)}.dark\:caret-sky-200\/45:is(.dark *){caret-color:rgba(186,230,253,.45)}.dark\:caret-sky-200\/5:is(.dark *){caret-color:rgba(186,230,253,.05)}.dark\:caret-sky-200\/50:is(.dark *){caret-color:rgba(186,230,253,.5)}.dark\:caret-sky-200\/55:is(.dark *){caret-color:rgba(186,230,253,.55)}.dark\:caret-sky-200\/60:is(.dark *){caret-color:rgba(186,230,253,.6)}.dark\:caret-sky-200\/65:is(.dark *){caret-color:rgba(186,230,253,.65)}.dark\:caret-sky-200\/70:is(.dark *){caret-color:rgba(186,230,253,.7)}.dark\:caret-sky-200\/75:is(.dark *){caret-color:rgba(186,230,253,.75)}.dark\:caret-sky-200\/80:is(.dark *){caret-color:rgba(186,230,253,.8)}.dark\:caret-sky-200\/85:is(.dark *){caret-color:rgba(186,230,253,.85)}.dark\:caret-sky-200\/90:is(.dark *){caret-color:rgba(186,230,253,.9)}.dark\:caret-sky-200\/95:is(.dark *){caret-color:rgba(186,230,253,.95)}.dark\:caret-sky-400:is(.dark *){caret-color:#38bdf8}.dark\:caret-sky-400\/0:is(.dark *){caret-color:rgba(56,189,248,0)}.dark\:caret-sky-400\/10:is(.dark *){caret-color:rgba(56,189,248,.1)}.dark\:caret-sky-400\/100:is(.dark *){caret-color:#38bdf8}.dark\:caret-sky-400\/15:is(.dark *){caret-color:rgba(56,189,248,.15)}.dark\:caret-sky-400\/20:is(.dark *){caret-color:rgba(56,189,248,.2)}.dark\:caret-sky-400\/25:is(.dark *){caret-color:rgba(56,189,248,.25)}.dark\:caret-sky-400\/30:is(.dark *){caret-color:rgba(56,189,248,.3)}.dark\:caret-sky-400\/35:is(.dark *){caret-color:rgba(56,189,248,.35)}.dark\:caret-sky-400\/40:is(.dark *){caret-color:rgba(56,189,248,.4)}.dark\:caret-sky-400\/45:is(.dark *){caret-color:rgba(56,189,248,.45)}.dark\:caret-sky-400\/5:is(.dark *){caret-color:rgba(56,189,248,.05)}.dark\:caret-sky-400\/50:is(.dark *){caret-color:rgba(56,189,248,.5)}.dark\:caret-sky-400\/55:is(.dark *){caret-color:rgba(56,189,248,.55)}.dark\:caret-sky-400\/60:is(.dark *){caret-color:rgba(56,189,248,.6)}.dark\:caret-sky-400\/65:is(.dark *){caret-color:rgba(56,189,248,.65)}.dark\:caret-sky-400\/70:is(.dark *){caret-color:rgba(56,189,248,.7)}.dark\:caret-sky-400\/75:is(.dark *){caret-color:rgba(56,189,248,.75)}.dark\:caret-sky-400\/80:is(.dark *){caret-color:rgba(56,189,248,.8)}.dark\:caret-sky-400\/85:is(.dark *){caret-color:rgba(56,189,248,.85)}.dark\:caret-sky-400\/90:is(.dark *){caret-color:rgba(56,189,248,.9)}.dark\:caret-sky-400\/95:is(.dark *){caret-color:rgba(56,189,248,.95)}.dark\:caret-sky-50:is(.dark *){caret-color:#f0f9ff}.dark\:caret-sky-50\/0:is(.dark *){caret-color:rgba(240,249,255,0)}.dark\:caret-sky-50\/10:is(.dark *){caret-color:rgba(240,249,255,.1)}.dark\:caret-sky-50\/100:is(.dark *){caret-color:#f0f9ff}.dark\:caret-sky-50\/15:is(.dark *){caret-color:rgba(240,249,255,.15)}.dark\:caret-sky-50\/20:is(.dark *){caret-color:rgba(240,249,255,.2)}.dark\:caret-sky-50\/25:is(.dark *){caret-color:rgba(240,249,255,.25)}.dark\:caret-sky-50\/30:is(.dark *){caret-color:rgba(240,249,255,.3)}.dark\:caret-sky-50\/35:is(.dark *){caret-color:rgba(240,249,255,.35)}.dark\:caret-sky-50\/40:is(.dark *){caret-color:rgba(240,249,255,.4)}.dark\:caret-sky-50\/45:is(.dark *){caret-color:rgba(240,249,255,.45)}.dark\:caret-sky-50\/5:is(.dark *){caret-color:rgba(240,249,255,.05)}.dark\:caret-sky-50\/50:is(.dark *){caret-color:rgba(240,249,255,.5)}.dark\:caret-sky-50\/55:is(.dark *){caret-color:rgba(240,249,255,.55)}.dark\:caret-sky-50\/60:is(.dark *){caret-color:rgba(240,249,255,.6)}.dark\:caret-sky-50\/65:is(.dark *){caret-color:rgba(240,249,255,.65)}.dark\:caret-sky-50\/70:is(.dark *){caret-color:rgba(240,249,255,.7)}.dark\:caret-sky-50\/75:is(.dark *){caret-color:rgba(240,249,255,.75)}.dark\:caret-sky-50\/80:is(.dark *){caret-color:rgba(240,249,255,.8)}.dark\:caret-sky-50\/85:is(.dark *){caret-color:rgba(240,249,255,.85)}.dark\:caret-sky-50\/90:is(.dark *){caret-color:rgba(240,249,255,.9)}.dark\:caret-sky-50\/95:is(.dark *){caret-color:rgba(240,249,255,.95)}.dark\:caret-sky-500:is(.dark *){caret-color:#0ea5e9}.dark\:caret-sky-500\/0:is(.dark *){caret-color:rgba(14,165,233,0)}.dark\:caret-sky-500\/10:is(.dark *){caret-color:rgba(14,165,233,.1)}.dark\:caret-sky-500\/100:is(.dark *){caret-color:#0ea5e9}.dark\:caret-sky-500\/15:is(.dark *){caret-color:rgba(14,165,233,.15)}.dark\:caret-sky-500\/20:is(.dark *){caret-color:rgba(14,165,233,.2)}.dark\:caret-sky-500\/25:is(.dark *){caret-color:rgba(14,165,233,.25)}.dark\:caret-sky-500\/30:is(.dark *){caret-color:rgba(14,165,233,.3)}.dark\:caret-sky-500\/35:is(.dark *){caret-color:rgba(14,165,233,.35)}.dark\:caret-sky-500\/40:is(.dark *){caret-color:rgba(14,165,233,.4)}.dark\:caret-sky-500\/45:is(.dark *){caret-color:rgba(14,165,233,.45)}.dark\:caret-sky-500\/5:is(.dark *){caret-color:rgba(14,165,233,.05)}.dark\:caret-sky-500\/50:is(.dark *){caret-color:rgba(14,165,233,.5)}.dark\:caret-sky-500\/55:is(.dark *){caret-color:rgba(14,165,233,.55)}.dark\:caret-sky-500\/60:is(.dark *){caret-color:rgba(14,165,233,.6)}.dark\:caret-sky-500\/65:is(.dark *){caret-color:rgba(14,165,233,.65)}.dark\:caret-sky-500\/70:is(.dark *){caret-color:rgba(14,165,233,.7)}.dark\:caret-sky-500\/75:is(.dark *){caret-color:rgba(14,165,233,.75)}.dark\:caret-sky-500\/80:is(.dark *){caret-color:rgba(14,165,233,.8)}.dark\:caret-sky-500\/85:is(.dark *){caret-color:rgba(14,165,233,.85)}.dark\:caret-sky-500\/90:is(.dark *){caret-color:rgba(14,165,233,.9)}.dark\:caret-sky-500\/95:is(.dark *){caret-color:rgba(14,165,233,.95)}.dark\:caret-sky-950:is(.dark *){caret-color:#082f49}.dark\:caret-sky-950\/0:is(.dark *){caret-color:rgba(8,47,73,0)}.dark\:caret-sky-950\/10:is(.dark *){caret-color:rgba(8,47,73,.1)}.dark\:caret-sky-950\/100:is(.dark *){caret-color:#082f49}.dark\:caret-sky-950\/15:is(.dark *){caret-color:rgba(8,47,73,.15)}.dark\:caret-sky-950\/20:is(.dark *){caret-color:rgba(8,47,73,.2)}.dark\:caret-sky-950\/25:is(.dark *){caret-color:rgba(8,47,73,.25)}.dark\:caret-sky-950\/30:is(.dark *){caret-color:rgba(8,47,73,.3)}.dark\:caret-sky-950\/35:is(.dark *){caret-color:rgba(8,47,73,.35)}.dark\:caret-sky-950\/40:is(.dark *){caret-color:rgba(8,47,73,.4)}.dark\:caret-sky-950\/45:is(.dark *){caret-color:rgba(8,47,73,.45)}.dark\:caret-sky-950\/5:is(.dark *){caret-color:rgba(8,47,73,.05)}.dark\:caret-sky-950\/50:is(.dark *){caret-color:rgba(8,47,73,.5)}.dark\:caret-sky-950\/55:is(.dark *){caret-color:rgba(8,47,73,.55)}.dark\:caret-sky-950\/60:is(.dark *){caret-color:rgba(8,47,73,.6)}.dark\:caret-sky-950\/65:is(.dark *){caret-color:rgba(8,47,73,.65)}.dark\:caret-sky-950\/70:is(.dark *){caret-color:rgba(8,47,73,.7)}.dark\:caret-sky-950\/75:is(.dark *){caret-color:rgba(8,47,73,.75)}.dark\:caret-sky-950\/80:is(.dark *){caret-color:rgba(8,47,73,.8)}.dark\:caret-sky-950\/85:is(.dark *){caret-color:rgba(8,47,73,.85)}.dark\:caret-sky-950\/90:is(.dark *){caret-color:rgba(8,47,73,.9)}.dark\:caret-sky-950\/95:is(.dark *){caret-color:rgba(8,47,73,.95)}.dark\:caret-slate-100:is(.dark *){caret-color:#f1f5f9}.dark\:caret-slate-100\/0:is(.dark *){caret-color:rgba(241,245,249,0)}.dark\:caret-slate-100\/10:is(.dark *){caret-color:rgba(241,245,249,.1)}.dark\:caret-slate-100\/100:is(.dark *){caret-color:#f1f5f9}.dark\:caret-slate-100\/15:is(.dark *){caret-color:rgba(241,245,249,.15)}.dark\:caret-slate-100\/20:is(.dark *){caret-color:rgba(241,245,249,.2)}.dark\:caret-slate-100\/25:is(.dark *){caret-color:rgba(241,245,249,.25)}.dark\:caret-slate-100\/30:is(.dark *){caret-color:rgba(241,245,249,.3)}.dark\:caret-slate-100\/35:is(.dark *){caret-color:rgba(241,245,249,.35)}.dark\:caret-slate-100\/40:is(.dark *){caret-color:rgba(241,245,249,.4)}.dark\:caret-slate-100\/45:is(.dark *){caret-color:rgba(241,245,249,.45)}.dark\:caret-slate-100\/5:is(.dark *){caret-color:rgba(241,245,249,.05)}.dark\:caret-slate-100\/50:is(.dark *){caret-color:rgba(241,245,249,.5)}.dark\:caret-slate-100\/55:is(.dark *){caret-color:rgba(241,245,249,.55)}.dark\:caret-slate-100\/60:is(.dark *){caret-color:rgba(241,245,249,.6)}.dark\:caret-slate-100\/65:is(.dark *){caret-color:rgba(241,245,249,.65)}.dark\:caret-slate-100\/70:is(.dark *){caret-color:rgba(241,245,249,.7)}.dark\:caret-slate-100\/75:is(.dark *){caret-color:rgba(241,245,249,.75)}.dark\:caret-slate-100\/80:is(.dark *){caret-color:rgba(241,245,249,.8)}.dark\:caret-slate-100\/85:is(.dark *){caret-color:rgba(241,245,249,.85)}.dark\:caret-slate-100\/90:is(.dark *){caret-color:rgba(241,245,249,.9)}.dark\:caret-slate-100\/95:is(.dark *){caret-color:rgba(241,245,249,.95)}.dark\:caret-slate-200:is(.dark *){caret-color:#e2e8f0}.dark\:caret-slate-200\/0:is(.dark *){caret-color:rgba(226,232,240,0)}.dark\:caret-slate-200\/10:is(.dark *){caret-color:rgba(226,232,240,.1)}.dark\:caret-slate-200\/100:is(.dark *){caret-color:#e2e8f0}.dark\:caret-slate-200\/15:is(.dark *){caret-color:rgba(226,232,240,.15)}.dark\:caret-slate-200\/20:is(.dark *){caret-color:rgba(226,232,240,.2)}.dark\:caret-slate-200\/25:is(.dark *){caret-color:rgba(226,232,240,.25)}.dark\:caret-slate-200\/30:is(.dark *){caret-color:rgba(226,232,240,.3)}.dark\:caret-slate-200\/35:is(.dark *){caret-color:rgba(226,232,240,.35)}.dark\:caret-slate-200\/40:is(.dark *){caret-color:rgba(226,232,240,.4)}.dark\:caret-slate-200\/45:is(.dark *){caret-color:rgba(226,232,240,.45)}.dark\:caret-slate-200\/5:is(.dark *){caret-color:rgba(226,232,240,.05)}.dark\:caret-slate-200\/50:is(.dark *){caret-color:rgba(226,232,240,.5)}.dark\:caret-slate-200\/55:is(.dark *){caret-color:rgba(226,232,240,.55)}.dark\:caret-slate-200\/60:is(.dark *){caret-color:rgba(226,232,240,.6)}.dark\:caret-slate-200\/65:is(.dark *){caret-color:rgba(226,232,240,.65)}.dark\:caret-slate-200\/70:is(.dark *){caret-color:rgba(226,232,240,.7)}.dark\:caret-slate-200\/75:is(.dark *){caret-color:rgba(226,232,240,.75)}.dark\:caret-slate-200\/80:is(.dark *){caret-color:rgba(226,232,240,.8)}.dark\:caret-slate-200\/85:is(.dark *){caret-color:rgba(226,232,240,.85)}.dark\:caret-slate-200\/90:is(.dark *){caret-color:rgba(226,232,240,.9)}.dark\:caret-slate-200\/95:is(.dark *){caret-color:rgba(226,232,240,.95)}.dark\:caret-slate-400:is(.dark *){caret-color:#94a3b8}.dark\:caret-slate-400\/0:is(.dark *){caret-color:rgba(148,163,184,0)}.dark\:caret-slate-400\/10:is(.dark *){caret-color:rgba(148,163,184,.1)}.dark\:caret-slate-400\/100:is(.dark *){caret-color:#94a3b8}.dark\:caret-slate-400\/15:is(.dark *){caret-color:rgba(148,163,184,.15)}.dark\:caret-slate-400\/20:is(.dark *){caret-color:rgba(148,163,184,.2)}.dark\:caret-slate-400\/25:is(.dark *){caret-color:rgba(148,163,184,.25)}.dark\:caret-slate-400\/30:is(.dark *){caret-color:rgba(148,163,184,.3)}.dark\:caret-slate-400\/35:is(.dark *){caret-color:rgba(148,163,184,.35)}.dark\:caret-slate-400\/40:is(.dark *){caret-color:rgba(148,163,184,.4)}.dark\:caret-slate-400\/45:is(.dark *){caret-color:rgba(148,163,184,.45)}.dark\:caret-slate-400\/5:is(.dark *){caret-color:rgba(148,163,184,.05)}.dark\:caret-slate-400\/50:is(.dark *){caret-color:rgba(148,163,184,.5)}.dark\:caret-slate-400\/55:is(.dark *){caret-color:rgba(148,163,184,.55)}.dark\:caret-slate-400\/60:is(.dark *){caret-color:rgba(148,163,184,.6)}.dark\:caret-slate-400\/65:is(.dark *){caret-color:rgba(148,163,184,.65)}.dark\:caret-slate-400\/70:is(.dark *){caret-color:rgba(148,163,184,.7)}.dark\:caret-slate-400\/75:is(.dark *){caret-color:rgba(148,163,184,.75)}.dark\:caret-slate-400\/80:is(.dark *){caret-color:rgba(148,163,184,.8)}.dark\:caret-slate-400\/85:is(.dark *){caret-color:rgba(148,163,184,.85)}.dark\:caret-slate-400\/90:is(.dark *){caret-color:rgba(148,163,184,.9)}.dark\:caret-slate-400\/95:is(.dark *){caret-color:rgba(148,163,184,.95)}.dark\:caret-slate-50:is(.dark *){caret-color:#f8fafc}.dark\:caret-slate-50\/0:is(.dark *){caret-color:rgba(248,250,252,0)}.dark\:caret-slate-50\/10:is(.dark *){caret-color:rgba(248,250,252,.1)}.dark\:caret-slate-50\/100:is(.dark *){caret-color:#f8fafc}.dark\:caret-slate-50\/15:is(.dark *){caret-color:rgba(248,250,252,.15)}.dark\:caret-slate-50\/20:is(.dark *){caret-color:rgba(248,250,252,.2)}.dark\:caret-slate-50\/25:is(.dark *){caret-color:rgba(248,250,252,.25)}.dark\:caret-slate-50\/30:is(.dark *){caret-color:rgba(248,250,252,.3)}.dark\:caret-slate-50\/35:is(.dark *){caret-color:rgba(248,250,252,.35)}.dark\:caret-slate-50\/40:is(.dark *){caret-color:rgba(248,250,252,.4)}.dark\:caret-slate-50\/45:is(.dark *){caret-color:rgba(248,250,252,.45)}.dark\:caret-slate-50\/5:is(.dark *){caret-color:rgba(248,250,252,.05)}.dark\:caret-slate-50\/50:is(.dark *){caret-color:rgba(248,250,252,.5)}.dark\:caret-slate-50\/55:is(.dark *){caret-color:rgba(248,250,252,.55)}.dark\:caret-slate-50\/60:is(.dark *){caret-color:rgba(248,250,252,.6)}.dark\:caret-slate-50\/65:is(.dark *){caret-color:rgba(248,250,252,.65)}.dark\:caret-slate-50\/70:is(.dark *){caret-color:rgba(248,250,252,.7)}.dark\:caret-slate-50\/75:is(.dark *){caret-color:rgba(248,250,252,.75)}.dark\:caret-slate-50\/80:is(.dark *){caret-color:rgba(248,250,252,.8)}.dark\:caret-slate-50\/85:is(.dark *){caret-color:rgba(248,250,252,.85)}.dark\:caret-slate-50\/90:is(.dark *){caret-color:rgba(248,250,252,.9)}.dark\:caret-slate-50\/95:is(.dark *){caret-color:rgba(248,250,252,.95)}.dark\:caret-slate-500:is(.dark *){caret-color:#64748b}.dark\:caret-slate-500\/0:is(.dark *){caret-color:rgba(100,116,139,0)}.dark\:caret-slate-500\/10:is(.dark *){caret-color:rgba(100,116,139,.1)}.dark\:caret-slate-500\/100:is(.dark *){caret-color:#64748b}.dark\:caret-slate-500\/15:is(.dark *){caret-color:rgba(100,116,139,.15)}.dark\:caret-slate-500\/20:is(.dark *){caret-color:rgba(100,116,139,.2)}.dark\:caret-slate-500\/25:is(.dark *){caret-color:rgba(100,116,139,.25)}.dark\:caret-slate-500\/30:is(.dark *){caret-color:rgba(100,116,139,.3)}.dark\:caret-slate-500\/35:is(.dark *){caret-color:rgba(100,116,139,.35)}.dark\:caret-slate-500\/40:is(.dark *){caret-color:rgba(100,116,139,.4)}.dark\:caret-slate-500\/45:is(.dark *){caret-color:rgba(100,116,139,.45)}.dark\:caret-slate-500\/5:is(.dark *){caret-color:rgba(100,116,139,.05)}.dark\:caret-slate-500\/50:is(.dark *){caret-color:rgba(100,116,139,.5)}.dark\:caret-slate-500\/55:is(.dark *){caret-color:rgba(100,116,139,.55)}.dark\:caret-slate-500\/60:is(.dark *){caret-color:rgba(100,116,139,.6)}.dark\:caret-slate-500\/65:is(.dark *){caret-color:rgba(100,116,139,.65)}.dark\:caret-slate-500\/70:is(.dark *){caret-color:rgba(100,116,139,.7)}.dark\:caret-slate-500\/75:is(.dark *){caret-color:rgba(100,116,139,.75)}.dark\:caret-slate-500\/80:is(.dark *){caret-color:rgba(100,116,139,.8)}.dark\:caret-slate-500\/85:is(.dark *){caret-color:rgba(100,116,139,.85)}.dark\:caret-slate-500\/90:is(.dark *){caret-color:rgba(100,116,139,.9)}.dark\:caret-slate-500\/95:is(.dark *){caret-color:rgba(100,116,139,.95)}.dark\:caret-slate-950:is(.dark *){caret-color:#020617}.dark\:caret-slate-950\/0:is(.dark *){caret-color:rgba(2,6,23,0)}.dark\:caret-slate-950\/10:is(.dark *){caret-color:rgba(2,6,23,.1)}.dark\:caret-slate-950\/100:is(.dark *){caret-color:#020617}.dark\:caret-slate-950\/15:is(.dark *){caret-color:rgba(2,6,23,.15)}.dark\:caret-slate-950\/20:is(.dark *){caret-color:rgba(2,6,23,.2)}.dark\:caret-slate-950\/25:is(.dark *){caret-color:rgba(2,6,23,.25)}.dark\:caret-slate-950\/30:is(.dark *){caret-color:rgba(2,6,23,.3)}.dark\:caret-slate-950\/35:is(.dark *){caret-color:rgba(2,6,23,.35)}.dark\:caret-slate-950\/40:is(.dark *){caret-color:rgba(2,6,23,.4)}.dark\:caret-slate-950\/45:is(.dark *){caret-color:rgba(2,6,23,.45)}.dark\:caret-slate-950\/5:is(.dark *){caret-color:rgba(2,6,23,.05)}.dark\:caret-slate-950\/50:is(.dark *){caret-color:rgba(2,6,23,.5)}.dark\:caret-slate-950\/55:is(.dark *){caret-color:rgba(2,6,23,.55)}.dark\:caret-slate-950\/60:is(.dark *){caret-color:rgba(2,6,23,.6)}.dark\:caret-slate-950\/65:is(.dark *){caret-color:rgba(2,6,23,.65)}.dark\:caret-slate-950\/70:is(.dark *){caret-color:rgba(2,6,23,.7)}.dark\:caret-slate-950\/75:is(.dark *){caret-color:rgba(2,6,23,.75)}.dark\:caret-slate-950\/80:is(.dark *){caret-color:rgba(2,6,23,.8)}.dark\:caret-slate-950\/85:is(.dark *){caret-color:rgba(2,6,23,.85)}.dark\:caret-slate-950\/90:is(.dark *){caret-color:rgba(2,6,23,.9)}.dark\:caret-slate-950\/95:is(.dark *){caret-color:rgba(2,6,23,.95)}.dark\:caret-stone-100:is(.dark *){caret-color:#f5f5f4}.dark\:caret-stone-100\/0:is(.dark *){caret-color:hsla(60,5%,96%,0)}.dark\:caret-stone-100\/10:is(.dark *){caret-color:hsla(60,5%,96%,.1)}.dark\:caret-stone-100\/100:is(.dark *){caret-color:#f5f5f4}.dark\:caret-stone-100\/15:is(.dark *){caret-color:hsla(60,5%,96%,.15)}.dark\:caret-stone-100\/20:is(.dark *){caret-color:hsla(60,5%,96%,.2)}.dark\:caret-stone-100\/25:is(.dark *){caret-color:hsla(60,5%,96%,.25)}.dark\:caret-stone-100\/30:is(.dark *){caret-color:hsla(60,5%,96%,.3)}.dark\:caret-stone-100\/35:is(.dark *){caret-color:hsla(60,5%,96%,.35)}.dark\:caret-stone-100\/40:is(.dark *){caret-color:hsla(60,5%,96%,.4)}.dark\:caret-stone-100\/45:is(.dark *){caret-color:hsla(60,5%,96%,.45)}.dark\:caret-stone-100\/5:is(.dark *){caret-color:hsla(60,5%,96%,.05)}.dark\:caret-stone-100\/50:is(.dark *){caret-color:hsla(60,5%,96%,.5)}.dark\:caret-stone-100\/55:is(.dark *){caret-color:hsla(60,5%,96%,.55)}.dark\:caret-stone-100\/60:is(.dark *){caret-color:hsla(60,5%,96%,.6)}.dark\:caret-stone-100\/65:is(.dark *){caret-color:hsla(60,5%,96%,.65)}.dark\:caret-stone-100\/70:is(.dark *){caret-color:hsla(60,5%,96%,.7)}.dark\:caret-stone-100\/75:is(.dark *){caret-color:hsla(60,5%,96%,.75)}.dark\:caret-stone-100\/80:is(.dark *){caret-color:hsla(60,5%,96%,.8)}.dark\:caret-stone-100\/85:is(.dark *){caret-color:hsla(60,5%,96%,.85)}.dark\:caret-stone-100\/90:is(.dark *){caret-color:hsla(60,5%,96%,.9)}.dark\:caret-stone-100\/95:is(.dark *){caret-color:hsla(60,5%,96%,.95)}.dark\:caret-stone-200:is(.dark *){caret-color:#e7e5e4}.dark\:caret-stone-200\/0:is(.dark *){caret-color:hsla(20,6%,90%,0)}.dark\:caret-stone-200\/10:is(.dark *){caret-color:hsla(20,6%,90%,.1)}.dark\:caret-stone-200\/100:is(.dark *){caret-color:#e7e5e4}.dark\:caret-stone-200\/15:is(.dark *){caret-color:hsla(20,6%,90%,.15)}.dark\:caret-stone-200\/20:is(.dark *){caret-color:hsla(20,6%,90%,.2)}.dark\:caret-stone-200\/25:is(.dark *){caret-color:hsla(20,6%,90%,.25)}.dark\:caret-stone-200\/30:is(.dark *){caret-color:hsla(20,6%,90%,.3)}.dark\:caret-stone-200\/35:is(.dark *){caret-color:hsla(20,6%,90%,.35)}.dark\:caret-stone-200\/40:is(.dark *){caret-color:hsla(20,6%,90%,.4)}.dark\:caret-stone-200\/45:is(.dark *){caret-color:hsla(20,6%,90%,.45)}.dark\:caret-stone-200\/5:is(.dark *){caret-color:hsla(20,6%,90%,.05)}.dark\:caret-stone-200\/50:is(.dark *){caret-color:hsla(20,6%,90%,.5)}.dark\:caret-stone-200\/55:is(.dark *){caret-color:hsla(20,6%,90%,.55)}.dark\:caret-stone-200\/60:is(.dark *){caret-color:hsla(20,6%,90%,.6)}.dark\:caret-stone-200\/65:is(.dark *){caret-color:hsla(20,6%,90%,.65)}.dark\:caret-stone-200\/70:is(.dark *){caret-color:hsla(20,6%,90%,.7)}.dark\:caret-stone-200\/75:is(.dark *){caret-color:hsla(20,6%,90%,.75)}.dark\:caret-stone-200\/80:is(.dark *){caret-color:hsla(20,6%,90%,.8)}.dark\:caret-stone-200\/85:is(.dark *){caret-color:hsla(20,6%,90%,.85)}.dark\:caret-stone-200\/90:is(.dark *){caret-color:hsla(20,6%,90%,.9)}.dark\:caret-stone-200\/95:is(.dark *){caret-color:hsla(20,6%,90%,.95)}.dark\:caret-stone-400:is(.dark *){caret-color:#a8a29e}.dark\:caret-stone-400\/0:is(.dark *){caret-color:hsla(24,5%,64%,0)}.dark\:caret-stone-400\/10:is(.dark *){caret-color:hsla(24,5%,64%,.1)}.dark\:caret-stone-400\/100:is(.dark *){caret-color:#a8a29e}.dark\:caret-stone-400\/15:is(.dark *){caret-color:hsla(24,5%,64%,.15)}.dark\:caret-stone-400\/20:is(.dark *){caret-color:hsla(24,5%,64%,.2)}.dark\:caret-stone-400\/25:is(.dark *){caret-color:hsla(24,5%,64%,.25)}.dark\:caret-stone-400\/30:is(.dark *){caret-color:hsla(24,5%,64%,.3)}.dark\:caret-stone-400\/35:is(.dark *){caret-color:hsla(24,5%,64%,.35)}.dark\:caret-stone-400\/40:is(.dark *){caret-color:hsla(24,5%,64%,.4)}.dark\:caret-stone-400\/45:is(.dark *){caret-color:hsla(24,5%,64%,.45)}.dark\:caret-stone-400\/5:is(.dark *){caret-color:hsla(24,5%,64%,.05)}.dark\:caret-stone-400\/50:is(.dark *){caret-color:hsla(24,5%,64%,.5)}.dark\:caret-stone-400\/55:is(.dark *){caret-color:hsla(24,5%,64%,.55)}.dark\:caret-stone-400\/60:is(.dark *){caret-color:hsla(24,5%,64%,.6)}.dark\:caret-stone-400\/65:is(.dark *){caret-color:hsla(24,5%,64%,.65)}.dark\:caret-stone-400\/70:is(.dark *){caret-color:hsla(24,5%,64%,.7)}.dark\:caret-stone-400\/75:is(.dark *){caret-color:hsla(24,5%,64%,.75)}.dark\:caret-stone-400\/80:is(.dark *){caret-color:hsla(24,5%,64%,.8)}.dark\:caret-stone-400\/85:is(.dark *){caret-color:hsla(24,5%,64%,.85)}.dark\:caret-stone-400\/90:is(.dark *){caret-color:hsla(24,5%,64%,.9)}.dark\:caret-stone-400\/95:is(.dark *){caret-color:hsla(24,5%,64%,.95)}.dark\:caret-stone-50:is(.dark *){caret-color:#fafaf9}.dark\:caret-stone-50\/0:is(.dark *){caret-color:hsla(60,9%,98%,0)}.dark\:caret-stone-50\/10:is(.dark *){caret-color:hsla(60,9%,98%,.1)}.dark\:caret-stone-50\/100:is(.dark *){caret-color:#fafaf9}.dark\:caret-stone-50\/15:is(.dark *){caret-color:hsla(60,9%,98%,.15)}.dark\:caret-stone-50\/20:is(.dark *){caret-color:hsla(60,9%,98%,.2)}.dark\:caret-stone-50\/25:is(.dark *){caret-color:hsla(60,9%,98%,.25)}.dark\:caret-stone-50\/30:is(.dark *){caret-color:hsla(60,9%,98%,.3)}.dark\:caret-stone-50\/35:is(.dark *){caret-color:hsla(60,9%,98%,.35)}.dark\:caret-stone-50\/40:is(.dark *){caret-color:hsla(60,9%,98%,.4)}.dark\:caret-stone-50\/45:is(.dark *){caret-color:hsla(60,9%,98%,.45)}.dark\:caret-stone-50\/5:is(.dark *){caret-color:hsla(60,9%,98%,.05)}.dark\:caret-stone-50\/50:is(.dark *){caret-color:hsla(60,9%,98%,.5)}.dark\:caret-stone-50\/55:is(.dark *){caret-color:hsla(60,9%,98%,.55)}.dark\:caret-stone-50\/60:is(.dark *){caret-color:hsla(60,9%,98%,.6)}.dark\:caret-stone-50\/65:is(.dark *){caret-color:hsla(60,9%,98%,.65)}.dark\:caret-stone-50\/70:is(.dark *){caret-color:hsla(60,9%,98%,.7)}.dark\:caret-stone-50\/75:is(.dark *){caret-color:hsla(60,9%,98%,.75)}.dark\:caret-stone-50\/80:is(.dark *){caret-color:hsla(60,9%,98%,.8)}.dark\:caret-stone-50\/85:is(.dark *){caret-color:hsla(60,9%,98%,.85)}.dark\:caret-stone-50\/90:is(.dark *){caret-color:hsla(60,9%,98%,.9)}.dark\:caret-stone-50\/95:is(.dark *){caret-color:hsla(60,9%,98%,.95)}.dark\:caret-stone-500:is(.dark *){caret-color:#78716c}.dark\:caret-stone-500\/0:is(.dark *){caret-color:hsla(25,5%,45%,0)}.dark\:caret-stone-500\/10:is(.dark *){caret-color:hsla(25,5%,45%,.1)}.dark\:caret-stone-500\/100:is(.dark *){caret-color:#78716c}.dark\:caret-stone-500\/15:is(.dark *){caret-color:hsla(25,5%,45%,.15)}.dark\:caret-stone-500\/20:is(.dark *){caret-color:hsla(25,5%,45%,.2)}.dark\:caret-stone-500\/25:is(.dark *){caret-color:hsla(25,5%,45%,.25)}.dark\:caret-stone-500\/30:is(.dark *){caret-color:hsla(25,5%,45%,.3)}.dark\:caret-stone-500\/35:is(.dark *){caret-color:hsla(25,5%,45%,.35)}.dark\:caret-stone-500\/40:is(.dark *){caret-color:hsla(25,5%,45%,.4)}.dark\:caret-stone-500\/45:is(.dark *){caret-color:hsla(25,5%,45%,.45)}.dark\:caret-stone-500\/5:is(.dark *){caret-color:hsla(25,5%,45%,.05)}.dark\:caret-stone-500\/50:is(.dark *){caret-color:hsla(25,5%,45%,.5)}.dark\:caret-stone-500\/55:is(.dark *){caret-color:hsla(25,5%,45%,.55)}.dark\:caret-stone-500\/60:is(.dark *){caret-color:hsla(25,5%,45%,.6)}.dark\:caret-stone-500\/65:is(.dark *){caret-color:hsla(25,5%,45%,.65)}.dark\:caret-stone-500\/70:is(.dark *){caret-color:hsla(25,5%,45%,.7)}.dark\:caret-stone-500\/75:is(.dark *){caret-color:hsla(25,5%,45%,.75)}.dark\:caret-stone-500\/80:is(.dark *){caret-color:hsla(25,5%,45%,.8)}.dark\:caret-stone-500\/85:is(.dark *){caret-color:hsla(25,5%,45%,.85)}.dark\:caret-stone-500\/90:is(.dark *){caret-color:hsla(25,5%,45%,.9)}.dark\:caret-stone-500\/95:is(.dark *){caret-color:hsla(25,5%,45%,.95)}.dark\:caret-stone-950:is(.dark *){caret-color:#0c0a09}.dark\:caret-stone-950\/0:is(.dark *){caret-color:rgba(12,10,9,0)}.dark\:caret-stone-950\/10:is(.dark *){caret-color:rgba(12,10,9,.1)}.dark\:caret-stone-950\/100:is(.dark *){caret-color:#0c0a09}.dark\:caret-stone-950\/15:is(.dark *){caret-color:rgba(12,10,9,.15)}.dark\:caret-stone-950\/20:is(.dark *){caret-color:rgba(12,10,9,.2)}.dark\:caret-stone-950\/25:is(.dark *){caret-color:rgba(12,10,9,.25)}.dark\:caret-stone-950\/30:is(.dark *){caret-color:rgba(12,10,9,.3)}.dark\:caret-stone-950\/35:is(.dark *){caret-color:rgba(12,10,9,.35)}.dark\:caret-stone-950\/40:is(.dark *){caret-color:rgba(12,10,9,.4)}.dark\:caret-stone-950\/45:is(.dark *){caret-color:rgba(12,10,9,.45)}.dark\:caret-stone-950\/5:is(.dark *){caret-color:rgba(12,10,9,.05)}.dark\:caret-stone-950\/50:is(.dark *){caret-color:rgba(12,10,9,.5)}.dark\:caret-stone-950\/55:is(.dark *){caret-color:rgba(12,10,9,.55)}.dark\:caret-stone-950\/60:is(.dark *){caret-color:rgba(12,10,9,.6)}.dark\:caret-stone-950\/65:is(.dark *){caret-color:rgba(12,10,9,.65)}.dark\:caret-stone-950\/70:is(.dark *){caret-color:rgba(12,10,9,.7)}.dark\:caret-stone-950\/75:is(.dark *){caret-color:rgba(12,10,9,.75)}.dark\:caret-stone-950\/80:is(.dark *){caret-color:rgba(12,10,9,.8)}.dark\:caret-stone-950\/85:is(.dark *){caret-color:rgba(12,10,9,.85)}.dark\:caret-stone-950\/90:is(.dark *){caret-color:rgba(12,10,9,.9)}.dark\:caret-stone-950\/95:is(.dark *){caret-color:rgba(12,10,9,.95)}.dark\:caret-teal-100:is(.dark *){caret-color:#ccfbf1}.dark\:caret-teal-100\/0:is(.dark *){caret-color:rgba(204,251,241,0)}.dark\:caret-teal-100\/10:is(.dark *){caret-color:rgba(204,251,241,.1)}.dark\:caret-teal-100\/100:is(.dark *){caret-color:#ccfbf1}.dark\:caret-teal-100\/15:is(.dark *){caret-color:rgba(204,251,241,.15)}.dark\:caret-teal-100\/20:is(.dark *){caret-color:rgba(204,251,241,.2)}.dark\:caret-teal-100\/25:is(.dark *){caret-color:rgba(204,251,241,.25)}.dark\:caret-teal-100\/30:is(.dark *){caret-color:rgba(204,251,241,.3)}.dark\:caret-teal-100\/35:is(.dark *){caret-color:rgba(204,251,241,.35)}.dark\:caret-teal-100\/40:is(.dark *){caret-color:rgba(204,251,241,.4)}.dark\:caret-teal-100\/45:is(.dark *){caret-color:rgba(204,251,241,.45)}.dark\:caret-teal-100\/5:is(.dark *){caret-color:rgba(204,251,241,.05)}.dark\:caret-teal-100\/50:is(.dark *){caret-color:rgba(204,251,241,.5)}.dark\:caret-teal-100\/55:is(.dark *){caret-color:rgba(204,251,241,.55)}.dark\:caret-teal-100\/60:is(.dark *){caret-color:rgba(204,251,241,.6)}.dark\:caret-teal-100\/65:is(.dark *){caret-color:rgba(204,251,241,.65)}.dark\:caret-teal-100\/70:is(.dark *){caret-color:rgba(204,251,241,.7)}.dark\:caret-teal-100\/75:is(.dark *){caret-color:rgba(204,251,241,.75)}.dark\:caret-teal-100\/80:is(.dark *){caret-color:rgba(204,251,241,.8)}.dark\:caret-teal-100\/85:is(.dark *){caret-color:rgba(204,251,241,.85)}.dark\:caret-teal-100\/90:is(.dark *){caret-color:rgba(204,251,241,.9)}.dark\:caret-teal-100\/95:is(.dark *){caret-color:rgba(204,251,241,.95)}.dark\:caret-teal-200:is(.dark *){caret-color:#99f6e4}.dark\:caret-teal-200\/0:is(.dark *){caret-color:rgba(153,246,228,0)}.dark\:caret-teal-200\/10:is(.dark *){caret-color:rgba(153,246,228,.1)}.dark\:caret-teal-200\/100:is(.dark *){caret-color:#99f6e4}.dark\:caret-teal-200\/15:is(.dark *){caret-color:rgba(153,246,228,.15)}.dark\:caret-teal-200\/20:is(.dark *){caret-color:rgba(153,246,228,.2)}.dark\:caret-teal-200\/25:is(.dark *){caret-color:rgba(153,246,228,.25)}.dark\:caret-teal-200\/30:is(.dark *){caret-color:rgba(153,246,228,.3)}.dark\:caret-teal-200\/35:is(.dark *){caret-color:rgba(153,246,228,.35)}.dark\:caret-teal-200\/40:is(.dark *){caret-color:rgba(153,246,228,.4)}.dark\:caret-teal-200\/45:is(.dark *){caret-color:rgba(153,246,228,.45)}.dark\:caret-teal-200\/5:is(.dark *){caret-color:rgba(153,246,228,.05)}.dark\:caret-teal-200\/50:is(.dark *){caret-color:rgba(153,246,228,.5)}.dark\:caret-teal-200\/55:is(.dark *){caret-color:rgba(153,246,228,.55)}.dark\:caret-teal-200\/60:is(.dark *){caret-color:rgba(153,246,228,.6)}.dark\:caret-teal-200\/65:is(.dark *){caret-color:rgba(153,246,228,.65)}.dark\:caret-teal-200\/70:is(.dark *){caret-color:rgba(153,246,228,.7)}.dark\:caret-teal-200\/75:is(.dark *){caret-color:rgba(153,246,228,.75)}.dark\:caret-teal-200\/80:is(.dark *){caret-color:rgba(153,246,228,.8)}.dark\:caret-teal-200\/85:is(.dark *){caret-color:rgba(153,246,228,.85)}.dark\:caret-teal-200\/90:is(.dark *){caret-color:rgba(153,246,228,.9)}.dark\:caret-teal-200\/95:is(.dark *){caret-color:rgba(153,246,228,.95)}.dark\:caret-teal-400:is(.dark *){caret-color:#2dd4bf}.dark\:caret-teal-400\/0:is(.dark *){caret-color:rgba(45,212,191,0)}.dark\:caret-teal-400\/10:is(.dark *){caret-color:rgba(45,212,191,.1)}.dark\:caret-teal-400\/100:is(.dark *){caret-color:#2dd4bf}.dark\:caret-teal-400\/15:is(.dark *){caret-color:rgba(45,212,191,.15)}.dark\:caret-teal-400\/20:is(.dark *){caret-color:rgba(45,212,191,.2)}.dark\:caret-teal-400\/25:is(.dark *){caret-color:rgba(45,212,191,.25)}.dark\:caret-teal-400\/30:is(.dark *){caret-color:rgba(45,212,191,.3)}.dark\:caret-teal-400\/35:is(.dark *){caret-color:rgba(45,212,191,.35)}.dark\:caret-teal-400\/40:is(.dark *){caret-color:rgba(45,212,191,.4)}.dark\:caret-teal-400\/45:is(.dark *){caret-color:rgba(45,212,191,.45)}.dark\:caret-teal-400\/5:is(.dark *){caret-color:rgba(45,212,191,.05)}.dark\:caret-teal-400\/50:is(.dark *){caret-color:rgba(45,212,191,.5)}.dark\:caret-teal-400\/55:is(.dark *){caret-color:rgba(45,212,191,.55)}.dark\:caret-teal-400\/60:is(.dark *){caret-color:rgba(45,212,191,.6)}.dark\:caret-teal-400\/65:is(.dark *){caret-color:rgba(45,212,191,.65)}.dark\:caret-teal-400\/70:is(.dark *){caret-color:rgba(45,212,191,.7)}.dark\:caret-teal-400\/75:is(.dark *){caret-color:rgba(45,212,191,.75)}.dark\:caret-teal-400\/80:is(.dark *){caret-color:rgba(45,212,191,.8)}.dark\:caret-teal-400\/85:is(.dark *){caret-color:rgba(45,212,191,.85)}.dark\:caret-teal-400\/90:is(.dark *){caret-color:rgba(45,212,191,.9)}.dark\:caret-teal-400\/95:is(.dark *){caret-color:rgba(45,212,191,.95)}.dark\:caret-teal-50:is(.dark *){caret-color:#f0fdfa}.dark\:caret-teal-50\/0:is(.dark *){caret-color:rgba(240,253,250,0)}.dark\:caret-teal-50\/10:is(.dark *){caret-color:rgba(240,253,250,.1)}.dark\:caret-teal-50\/100:is(.dark *){caret-color:#f0fdfa}.dark\:caret-teal-50\/15:is(.dark *){caret-color:rgba(240,253,250,.15)}.dark\:caret-teal-50\/20:is(.dark *){caret-color:rgba(240,253,250,.2)}.dark\:caret-teal-50\/25:is(.dark *){caret-color:rgba(240,253,250,.25)}.dark\:caret-teal-50\/30:is(.dark *){caret-color:rgba(240,253,250,.3)}.dark\:caret-teal-50\/35:is(.dark *){caret-color:rgba(240,253,250,.35)}.dark\:caret-teal-50\/40:is(.dark *){caret-color:rgba(240,253,250,.4)}.dark\:caret-teal-50\/45:is(.dark *){caret-color:rgba(240,253,250,.45)}.dark\:caret-teal-50\/5:is(.dark *){caret-color:rgba(240,253,250,.05)}.dark\:caret-teal-50\/50:is(.dark *){caret-color:rgba(240,253,250,.5)}.dark\:caret-teal-50\/55:is(.dark *){caret-color:rgba(240,253,250,.55)}.dark\:caret-teal-50\/60:is(.dark *){caret-color:rgba(240,253,250,.6)}.dark\:caret-teal-50\/65:is(.dark *){caret-color:rgba(240,253,250,.65)}.dark\:caret-teal-50\/70:is(.dark *){caret-color:rgba(240,253,250,.7)}.dark\:caret-teal-50\/75:is(.dark *){caret-color:rgba(240,253,250,.75)}.dark\:caret-teal-50\/80:is(.dark *){caret-color:rgba(240,253,250,.8)}.dark\:caret-teal-50\/85:is(.dark *){caret-color:rgba(240,253,250,.85)}.dark\:caret-teal-50\/90:is(.dark *){caret-color:rgba(240,253,250,.9)}.dark\:caret-teal-50\/95:is(.dark *){caret-color:rgba(240,253,250,.95)}.dark\:caret-teal-500:is(.dark *){caret-color:#14b8a6}.dark\:caret-teal-500\/0:is(.dark *){caret-color:rgba(20,184,166,0)}.dark\:caret-teal-500\/10:is(.dark *){caret-color:rgba(20,184,166,.1)}.dark\:caret-teal-500\/100:is(.dark *){caret-color:#14b8a6}.dark\:caret-teal-500\/15:is(.dark *){caret-color:rgba(20,184,166,.15)}.dark\:caret-teal-500\/20:is(.dark *){caret-color:rgba(20,184,166,.2)}.dark\:caret-teal-500\/25:is(.dark *){caret-color:rgba(20,184,166,.25)}.dark\:caret-teal-500\/30:is(.dark *){caret-color:rgba(20,184,166,.3)}.dark\:caret-teal-500\/35:is(.dark *){caret-color:rgba(20,184,166,.35)}.dark\:caret-teal-500\/40:is(.dark *){caret-color:rgba(20,184,166,.4)}.dark\:caret-teal-500\/45:is(.dark *){caret-color:rgba(20,184,166,.45)}.dark\:caret-teal-500\/5:is(.dark *){caret-color:rgba(20,184,166,.05)}.dark\:caret-teal-500\/50:is(.dark *){caret-color:rgba(20,184,166,.5)}.dark\:caret-teal-500\/55:is(.dark *){caret-color:rgba(20,184,166,.55)}.dark\:caret-teal-500\/60:is(.dark *){caret-color:rgba(20,184,166,.6)}.dark\:caret-teal-500\/65:is(.dark *){caret-color:rgba(20,184,166,.65)}.dark\:caret-teal-500\/70:is(.dark *){caret-color:rgba(20,184,166,.7)}.dark\:caret-teal-500\/75:is(.dark *){caret-color:rgba(20,184,166,.75)}.dark\:caret-teal-500\/80:is(.dark *){caret-color:rgba(20,184,166,.8)}.dark\:caret-teal-500\/85:is(.dark *){caret-color:rgba(20,184,166,.85)}.dark\:caret-teal-500\/90:is(.dark *){caret-color:rgba(20,184,166,.9)}.dark\:caret-teal-500\/95:is(.dark *){caret-color:rgba(20,184,166,.95)}.dark\:caret-teal-950:is(.dark *){caret-color:#042f2e}.dark\:caret-teal-950\/0:is(.dark *){caret-color:rgba(4,47,46,0)}.dark\:caret-teal-950\/10:is(.dark *){caret-color:rgba(4,47,46,.1)}.dark\:caret-teal-950\/100:is(.dark *){caret-color:#042f2e}.dark\:caret-teal-950\/15:is(.dark *){caret-color:rgba(4,47,46,.15)}.dark\:caret-teal-950\/20:is(.dark *){caret-color:rgba(4,47,46,.2)}.dark\:caret-teal-950\/25:is(.dark *){caret-color:rgba(4,47,46,.25)}.dark\:caret-teal-950\/30:is(.dark *){caret-color:rgba(4,47,46,.3)}.dark\:caret-teal-950\/35:is(.dark *){caret-color:rgba(4,47,46,.35)}.dark\:caret-teal-950\/40:is(.dark *){caret-color:rgba(4,47,46,.4)}.dark\:caret-teal-950\/45:is(.dark *){caret-color:rgba(4,47,46,.45)}.dark\:caret-teal-950\/5:is(.dark *){caret-color:rgba(4,47,46,.05)}.dark\:caret-teal-950\/50:is(.dark *){caret-color:rgba(4,47,46,.5)}.dark\:caret-teal-950\/55:is(.dark *){caret-color:rgba(4,47,46,.55)}.dark\:caret-teal-950\/60:is(.dark *){caret-color:rgba(4,47,46,.6)}.dark\:caret-teal-950\/65:is(.dark *){caret-color:rgba(4,47,46,.65)}.dark\:caret-teal-950\/70:is(.dark *){caret-color:rgba(4,47,46,.7)}.dark\:caret-teal-950\/75:is(.dark *){caret-color:rgba(4,47,46,.75)}.dark\:caret-teal-950\/80:is(.dark *){caret-color:rgba(4,47,46,.8)}.dark\:caret-teal-950\/85:is(.dark *){caret-color:rgba(4,47,46,.85)}.dark\:caret-teal-950\/90:is(.dark *){caret-color:rgba(4,47,46,.9)}.dark\:caret-teal-950\/95:is(.dark *){caret-color:rgba(4,47,46,.95)}.dark\:caret-violet-100:is(.dark *){caret-color:#ede9fe}.dark\:caret-violet-100\/0:is(.dark *){caret-color:rgba(237,233,254,0)}.dark\:caret-violet-100\/10:is(.dark *){caret-color:rgba(237,233,254,.1)}.dark\:caret-violet-100\/100:is(.dark *){caret-color:#ede9fe}.dark\:caret-violet-100\/15:is(.dark *){caret-color:rgba(237,233,254,.15)}.dark\:caret-violet-100\/20:is(.dark *){caret-color:rgba(237,233,254,.2)}.dark\:caret-violet-100\/25:is(.dark *){caret-color:rgba(237,233,254,.25)}.dark\:caret-violet-100\/30:is(.dark *){caret-color:rgba(237,233,254,.3)}.dark\:caret-violet-100\/35:is(.dark *){caret-color:rgba(237,233,254,.35)}.dark\:caret-violet-100\/40:is(.dark *){caret-color:rgba(237,233,254,.4)}.dark\:caret-violet-100\/45:is(.dark *){caret-color:rgba(237,233,254,.45)}.dark\:caret-violet-100\/5:is(.dark *){caret-color:rgba(237,233,254,.05)}.dark\:caret-violet-100\/50:is(.dark *){caret-color:rgba(237,233,254,.5)}.dark\:caret-violet-100\/55:is(.dark *){caret-color:rgba(237,233,254,.55)}.dark\:caret-violet-100\/60:is(.dark *){caret-color:rgba(237,233,254,.6)}.dark\:caret-violet-100\/65:is(.dark *){caret-color:rgba(237,233,254,.65)}.dark\:caret-violet-100\/70:is(.dark *){caret-color:rgba(237,233,254,.7)}.dark\:caret-violet-100\/75:is(.dark *){caret-color:rgba(237,233,254,.75)}.dark\:caret-violet-100\/80:is(.dark *){caret-color:rgba(237,233,254,.8)}.dark\:caret-violet-100\/85:is(.dark *){caret-color:rgba(237,233,254,.85)}.dark\:caret-violet-100\/90:is(.dark *){caret-color:rgba(237,233,254,.9)}.dark\:caret-violet-100\/95:is(.dark *){caret-color:rgba(237,233,254,.95)}.dark\:caret-violet-200:is(.dark *){caret-color:#ddd6fe}.dark\:caret-violet-200\/0:is(.dark *){caret-color:rgba(221,214,254,0)}.dark\:caret-violet-200\/10:is(.dark *){caret-color:rgba(221,214,254,.1)}.dark\:caret-violet-200\/100:is(.dark *){caret-color:#ddd6fe}.dark\:caret-violet-200\/15:is(.dark *){caret-color:rgba(221,214,254,.15)}.dark\:caret-violet-200\/20:is(.dark *){caret-color:rgba(221,214,254,.2)}.dark\:caret-violet-200\/25:is(.dark *){caret-color:rgba(221,214,254,.25)}.dark\:caret-violet-200\/30:is(.dark *){caret-color:rgba(221,214,254,.3)}.dark\:caret-violet-200\/35:is(.dark *){caret-color:rgba(221,214,254,.35)}.dark\:caret-violet-200\/40:is(.dark *){caret-color:rgba(221,214,254,.4)}.dark\:caret-violet-200\/45:is(.dark *){caret-color:rgba(221,214,254,.45)}.dark\:caret-violet-200\/5:is(.dark *){caret-color:rgba(221,214,254,.05)}.dark\:caret-violet-200\/50:is(.dark *){caret-color:rgba(221,214,254,.5)}.dark\:caret-violet-200\/55:is(.dark *){caret-color:rgba(221,214,254,.55)}.dark\:caret-violet-200\/60:is(.dark *){caret-color:rgba(221,214,254,.6)}.dark\:caret-violet-200\/65:is(.dark *){caret-color:rgba(221,214,254,.65)}.dark\:caret-violet-200\/70:is(.dark *){caret-color:rgba(221,214,254,.7)}.dark\:caret-violet-200\/75:is(.dark *){caret-color:rgba(221,214,254,.75)}.dark\:caret-violet-200\/80:is(.dark *){caret-color:rgba(221,214,254,.8)}.dark\:caret-violet-200\/85:is(.dark *){caret-color:rgba(221,214,254,.85)}.dark\:caret-violet-200\/90:is(.dark *){caret-color:rgba(221,214,254,.9)}.dark\:caret-violet-200\/95:is(.dark *){caret-color:rgba(221,214,254,.95)}.dark\:caret-violet-400:is(.dark *){caret-color:#a78bfa}.dark\:caret-violet-400\/0:is(.dark *){caret-color:rgba(167,139,250,0)}.dark\:caret-violet-400\/10:is(.dark *){caret-color:rgba(167,139,250,.1)}.dark\:caret-violet-400\/100:is(.dark *){caret-color:#a78bfa}.dark\:caret-violet-400\/15:is(.dark *){caret-color:rgba(167,139,250,.15)}.dark\:caret-violet-400\/20:is(.dark *){caret-color:rgba(167,139,250,.2)}.dark\:caret-violet-400\/25:is(.dark *){caret-color:rgba(167,139,250,.25)}.dark\:caret-violet-400\/30:is(.dark *){caret-color:rgba(167,139,250,.3)}.dark\:caret-violet-400\/35:is(.dark *){caret-color:rgba(167,139,250,.35)}.dark\:caret-violet-400\/40:is(.dark *){caret-color:rgba(167,139,250,.4)}.dark\:caret-violet-400\/45:is(.dark *){caret-color:rgba(167,139,250,.45)}.dark\:caret-violet-400\/5:is(.dark *){caret-color:rgba(167,139,250,.05)}.dark\:caret-violet-400\/50:is(.dark *){caret-color:rgba(167,139,250,.5)}.dark\:caret-violet-400\/55:is(.dark *){caret-color:rgba(167,139,250,.55)}.dark\:caret-violet-400\/60:is(.dark *){caret-color:rgba(167,139,250,.6)}.dark\:caret-violet-400\/65:is(.dark *){caret-color:rgba(167,139,250,.65)}.dark\:caret-violet-400\/70:is(.dark *){caret-color:rgba(167,139,250,.7)}.dark\:caret-violet-400\/75:is(.dark *){caret-color:rgba(167,139,250,.75)}.dark\:caret-violet-400\/80:is(.dark *){caret-color:rgba(167,139,250,.8)}.dark\:caret-violet-400\/85:is(.dark *){caret-color:rgba(167,139,250,.85)}.dark\:caret-violet-400\/90:is(.dark *){caret-color:rgba(167,139,250,.9)}.dark\:caret-violet-400\/95:is(.dark *){caret-color:rgba(167,139,250,.95)}.dark\:caret-violet-50:is(.dark *){caret-color:#f5f3ff}.dark\:caret-violet-50\/0:is(.dark *){caret-color:rgba(245,243,255,0)}.dark\:caret-violet-50\/10:is(.dark *){caret-color:rgba(245,243,255,.1)}.dark\:caret-violet-50\/100:is(.dark *){caret-color:#f5f3ff}.dark\:caret-violet-50\/15:is(.dark *){caret-color:rgba(245,243,255,.15)}.dark\:caret-violet-50\/20:is(.dark *){caret-color:rgba(245,243,255,.2)}.dark\:caret-violet-50\/25:is(.dark *){caret-color:rgba(245,243,255,.25)}.dark\:caret-violet-50\/30:is(.dark *){caret-color:rgba(245,243,255,.3)}.dark\:caret-violet-50\/35:is(.dark *){caret-color:rgba(245,243,255,.35)}.dark\:caret-violet-50\/40:is(.dark *){caret-color:rgba(245,243,255,.4)}.dark\:caret-violet-50\/45:is(.dark *){caret-color:rgba(245,243,255,.45)}.dark\:caret-violet-50\/5:is(.dark *){caret-color:rgba(245,243,255,.05)}.dark\:caret-violet-50\/50:is(.dark *){caret-color:rgba(245,243,255,.5)}.dark\:caret-violet-50\/55:is(.dark *){caret-color:rgba(245,243,255,.55)}.dark\:caret-violet-50\/60:is(.dark *){caret-color:rgba(245,243,255,.6)}.dark\:caret-violet-50\/65:is(.dark *){caret-color:rgba(245,243,255,.65)}.dark\:caret-violet-50\/70:is(.dark *){caret-color:rgba(245,243,255,.7)}.dark\:caret-violet-50\/75:is(.dark *){caret-color:rgba(245,243,255,.75)}.dark\:caret-violet-50\/80:is(.dark *){caret-color:rgba(245,243,255,.8)}.dark\:caret-violet-50\/85:is(.dark *){caret-color:rgba(245,243,255,.85)}.dark\:caret-violet-50\/90:is(.dark *){caret-color:rgba(245,243,255,.9)}.dark\:caret-violet-50\/95:is(.dark *){caret-color:rgba(245,243,255,.95)}.dark\:caret-violet-500:is(.dark *){caret-color:#8b5cf6}.dark\:caret-violet-500\/0:is(.dark *){caret-color:rgba(139,92,246,0)}.dark\:caret-violet-500\/10:is(.dark *){caret-color:rgba(139,92,246,.1)}.dark\:caret-violet-500\/100:is(.dark *){caret-color:#8b5cf6}.dark\:caret-violet-500\/15:is(.dark *){caret-color:rgba(139,92,246,.15)}.dark\:caret-violet-500\/20:is(.dark *){caret-color:rgba(139,92,246,.2)}.dark\:caret-violet-500\/25:is(.dark *){caret-color:rgba(139,92,246,.25)}.dark\:caret-violet-500\/30:is(.dark *){caret-color:rgba(139,92,246,.3)}.dark\:caret-violet-500\/35:is(.dark *){caret-color:rgba(139,92,246,.35)}.dark\:caret-violet-500\/40:is(.dark *){caret-color:rgba(139,92,246,.4)}.dark\:caret-violet-500\/45:is(.dark *){caret-color:rgba(139,92,246,.45)}.dark\:caret-violet-500\/5:is(.dark *){caret-color:rgba(139,92,246,.05)}.dark\:caret-violet-500\/50:is(.dark *){caret-color:rgba(139,92,246,.5)}.dark\:caret-violet-500\/55:is(.dark *){caret-color:rgba(139,92,246,.55)}.dark\:caret-violet-500\/60:is(.dark *){caret-color:rgba(139,92,246,.6)}.dark\:caret-violet-500\/65:is(.dark *){caret-color:rgba(139,92,246,.65)}.dark\:caret-violet-500\/70:is(.dark *){caret-color:rgba(139,92,246,.7)}.dark\:caret-violet-500\/75:is(.dark *){caret-color:rgba(139,92,246,.75)}.dark\:caret-violet-500\/80:is(.dark *){caret-color:rgba(139,92,246,.8)}.dark\:caret-violet-500\/85:is(.dark *){caret-color:rgba(139,92,246,.85)}.dark\:caret-violet-500\/90:is(.dark *){caret-color:rgba(139,92,246,.9)}.dark\:caret-violet-500\/95:is(.dark *){caret-color:rgba(139,92,246,.95)}.dark\:caret-violet-950:is(.dark *){caret-color:#2e1065}.dark\:caret-violet-950\/0:is(.dark *){caret-color:rgba(46,16,101,0)}.dark\:caret-violet-950\/10:is(.dark *){caret-color:rgba(46,16,101,.1)}.dark\:caret-violet-950\/100:is(.dark *){caret-color:#2e1065}.dark\:caret-violet-950\/15:is(.dark *){caret-color:rgba(46,16,101,.15)}.dark\:caret-violet-950\/20:is(.dark *){caret-color:rgba(46,16,101,.2)}.dark\:caret-violet-950\/25:is(.dark *){caret-color:rgba(46,16,101,.25)}.dark\:caret-violet-950\/30:is(.dark *){caret-color:rgba(46,16,101,.3)}.dark\:caret-violet-950\/35:is(.dark *){caret-color:rgba(46,16,101,.35)}.dark\:caret-violet-950\/40:is(.dark *){caret-color:rgba(46,16,101,.4)}.dark\:caret-violet-950\/45:is(.dark *){caret-color:rgba(46,16,101,.45)}.dark\:caret-violet-950\/5:is(.dark *){caret-color:rgba(46,16,101,.05)}.dark\:caret-violet-950\/50:is(.dark *){caret-color:rgba(46,16,101,.5)}.dark\:caret-violet-950\/55:is(.dark *){caret-color:rgba(46,16,101,.55)}.dark\:caret-violet-950\/60:is(.dark *){caret-color:rgba(46,16,101,.6)}.dark\:caret-violet-950\/65:is(.dark *){caret-color:rgba(46,16,101,.65)}.dark\:caret-violet-950\/70:is(.dark *){caret-color:rgba(46,16,101,.7)}.dark\:caret-violet-950\/75:is(.dark *){caret-color:rgba(46,16,101,.75)}.dark\:caret-violet-950\/80:is(.dark *){caret-color:rgba(46,16,101,.8)}.dark\:caret-violet-950\/85:is(.dark *){caret-color:rgba(46,16,101,.85)}.dark\:caret-violet-950\/90:is(.dark *){caret-color:rgba(46,16,101,.9)}.dark\:caret-violet-950\/95:is(.dark *){caret-color:rgba(46,16,101,.95)}.dark\:caret-yellow-100:is(.dark *){caret-color:#fef9c3}.dark\:caret-yellow-100\/0:is(.dark *){caret-color:hsla(55,97%,88%,0)}.dark\:caret-yellow-100\/10:is(.dark *){caret-color:hsla(55,97%,88%,.1)}.dark\:caret-yellow-100\/100:is(.dark *){caret-color:#fef9c3}.dark\:caret-yellow-100\/15:is(.dark *){caret-color:hsla(55,97%,88%,.15)}.dark\:caret-yellow-100\/20:is(.dark *){caret-color:hsla(55,97%,88%,.2)}.dark\:caret-yellow-100\/25:is(.dark *){caret-color:hsla(55,97%,88%,.25)}.dark\:caret-yellow-100\/30:is(.dark *){caret-color:hsla(55,97%,88%,.3)}.dark\:caret-yellow-100\/35:is(.dark *){caret-color:hsla(55,97%,88%,.35)}.dark\:caret-yellow-100\/40:is(.dark *){caret-color:hsla(55,97%,88%,.4)}.dark\:caret-yellow-100\/45:is(.dark *){caret-color:hsla(55,97%,88%,.45)}.dark\:caret-yellow-100\/5:is(.dark *){caret-color:hsla(55,97%,88%,.05)}.dark\:caret-yellow-100\/50:is(.dark *){caret-color:hsla(55,97%,88%,.5)}.dark\:caret-yellow-100\/55:is(.dark *){caret-color:hsla(55,97%,88%,.55)}.dark\:caret-yellow-100\/60:is(.dark *){caret-color:hsla(55,97%,88%,.6)}.dark\:caret-yellow-100\/65:is(.dark *){caret-color:hsla(55,97%,88%,.65)}.dark\:caret-yellow-100\/70:is(.dark *){caret-color:hsla(55,97%,88%,.7)}.dark\:caret-yellow-100\/75:is(.dark *){caret-color:hsla(55,97%,88%,.75)}.dark\:caret-yellow-100\/80:is(.dark *){caret-color:hsla(55,97%,88%,.8)}.dark\:caret-yellow-100\/85:is(.dark *){caret-color:hsla(55,97%,88%,.85)}.dark\:caret-yellow-100\/90:is(.dark *){caret-color:hsla(55,97%,88%,.9)}.dark\:caret-yellow-100\/95:is(.dark *){caret-color:hsla(55,97%,88%,.95)}.dark\:caret-yellow-200:is(.dark *){caret-color:#fef08a}.dark\:caret-yellow-200\/0:is(.dark *){caret-color:hsla(53,98%,77%,0)}.dark\:caret-yellow-200\/10:is(.dark *){caret-color:hsla(53,98%,77%,.1)}.dark\:caret-yellow-200\/100:is(.dark *){caret-color:#fef08a}.dark\:caret-yellow-200\/15:is(.dark *){caret-color:hsla(53,98%,77%,.15)}.dark\:caret-yellow-200\/20:is(.dark *){caret-color:hsla(53,98%,77%,.2)}.dark\:caret-yellow-200\/25:is(.dark *){caret-color:hsla(53,98%,77%,.25)}.dark\:caret-yellow-200\/30:is(.dark *){caret-color:hsla(53,98%,77%,.3)}.dark\:caret-yellow-200\/35:is(.dark *){caret-color:hsla(53,98%,77%,.35)}.dark\:caret-yellow-200\/40:is(.dark *){caret-color:hsla(53,98%,77%,.4)}.dark\:caret-yellow-200\/45:is(.dark *){caret-color:hsla(53,98%,77%,.45)}.dark\:caret-yellow-200\/5:is(.dark *){caret-color:hsla(53,98%,77%,.05)}.dark\:caret-yellow-200\/50:is(.dark *){caret-color:hsla(53,98%,77%,.5)}.dark\:caret-yellow-200\/55:is(.dark *){caret-color:hsla(53,98%,77%,.55)}.dark\:caret-yellow-200\/60:is(.dark *){caret-color:hsla(53,98%,77%,.6)}.dark\:caret-yellow-200\/65:is(.dark *){caret-color:hsla(53,98%,77%,.65)}.dark\:caret-yellow-200\/70:is(.dark *){caret-color:hsla(53,98%,77%,.7)}.dark\:caret-yellow-200\/75:is(.dark *){caret-color:hsla(53,98%,77%,.75)}.dark\:caret-yellow-200\/80:is(.dark *){caret-color:hsla(53,98%,77%,.8)}.dark\:caret-yellow-200\/85:is(.dark *){caret-color:hsla(53,98%,77%,.85)}.dark\:caret-yellow-200\/90:is(.dark *){caret-color:hsla(53,98%,77%,.9)}.dark\:caret-yellow-200\/95:is(.dark *){caret-color:hsla(53,98%,77%,.95)}.dark\:caret-yellow-400:is(.dark *){caret-color:#facc15}.dark\:caret-yellow-400\/0:is(.dark *){caret-color:rgba(250,204,21,0)}.dark\:caret-yellow-400\/10:is(.dark *){caret-color:rgba(250,204,21,.1)}.dark\:caret-yellow-400\/100:is(.dark *){caret-color:#facc15}.dark\:caret-yellow-400\/15:is(.dark *){caret-color:rgba(250,204,21,.15)}.dark\:caret-yellow-400\/20:is(.dark *){caret-color:rgba(250,204,21,.2)}.dark\:caret-yellow-400\/25:is(.dark *){caret-color:rgba(250,204,21,.25)}.dark\:caret-yellow-400\/30:is(.dark *){caret-color:rgba(250,204,21,.3)}.dark\:caret-yellow-400\/35:is(.dark *){caret-color:rgba(250,204,21,.35)}.dark\:caret-yellow-400\/40:is(.dark *){caret-color:rgba(250,204,21,.4)}.dark\:caret-yellow-400\/45:is(.dark *){caret-color:rgba(250,204,21,.45)}.dark\:caret-yellow-400\/5:is(.dark *){caret-color:rgba(250,204,21,.05)}.dark\:caret-yellow-400\/50:is(.dark *){caret-color:rgba(250,204,21,.5)}.dark\:caret-yellow-400\/55:is(.dark *){caret-color:rgba(250,204,21,.55)}.dark\:caret-yellow-400\/60:is(.dark *){caret-color:rgba(250,204,21,.6)}.dark\:caret-yellow-400\/65:is(.dark *){caret-color:rgba(250,204,21,.65)}.dark\:caret-yellow-400\/70:is(.dark *){caret-color:rgba(250,204,21,.7)}.dark\:caret-yellow-400\/75:is(.dark *){caret-color:rgba(250,204,21,.75)}.dark\:caret-yellow-400\/80:is(.dark *){caret-color:rgba(250,204,21,.8)}.dark\:caret-yellow-400\/85:is(.dark *){caret-color:rgba(250,204,21,.85)}.dark\:caret-yellow-400\/90:is(.dark *){caret-color:rgba(250,204,21,.9)}.dark\:caret-yellow-400\/95:is(.dark *){caret-color:rgba(250,204,21,.95)}.dark\:caret-yellow-50:is(.dark *){caret-color:#fefce8}.dark\:caret-yellow-50\/0:is(.dark *){caret-color:hsla(55,92%,95%,0)}.dark\:caret-yellow-50\/10:is(.dark *){caret-color:hsla(55,92%,95%,.1)}.dark\:caret-yellow-50\/100:is(.dark *){caret-color:#fefce8}.dark\:caret-yellow-50\/15:is(.dark *){caret-color:hsla(55,92%,95%,.15)}.dark\:caret-yellow-50\/20:is(.dark *){caret-color:hsla(55,92%,95%,.2)}.dark\:caret-yellow-50\/25:is(.dark *){caret-color:hsla(55,92%,95%,.25)}.dark\:caret-yellow-50\/30:is(.dark *){caret-color:hsla(55,92%,95%,.3)}.dark\:caret-yellow-50\/35:is(.dark *){caret-color:hsla(55,92%,95%,.35)}.dark\:caret-yellow-50\/40:is(.dark *){caret-color:hsla(55,92%,95%,.4)}.dark\:caret-yellow-50\/45:is(.dark *){caret-color:hsla(55,92%,95%,.45)}.dark\:caret-yellow-50\/5:is(.dark *){caret-color:hsla(55,92%,95%,.05)}.dark\:caret-yellow-50\/50:is(.dark *){caret-color:hsla(55,92%,95%,.5)}.dark\:caret-yellow-50\/55:is(.dark *){caret-color:hsla(55,92%,95%,.55)}.dark\:caret-yellow-50\/60:is(.dark *){caret-color:hsla(55,92%,95%,.6)}.dark\:caret-yellow-50\/65:is(.dark *){caret-color:hsla(55,92%,95%,.65)}.dark\:caret-yellow-50\/70:is(.dark *){caret-color:hsla(55,92%,95%,.7)}.dark\:caret-yellow-50\/75:is(.dark *){caret-color:hsla(55,92%,95%,.75)}.dark\:caret-yellow-50\/80:is(.dark *){caret-color:hsla(55,92%,95%,.8)}.dark\:caret-yellow-50\/85:is(.dark *){caret-color:hsla(55,92%,95%,.85)}.dark\:caret-yellow-50\/90:is(.dark *){caret-color:hsla(55,92%,95%,.9)}.dark\:caret-yellow-50\/95:is(.dark *){caret-color:hsla(55,92%,95%,.95)}.dark\:caret-yellow-500:is(.dark *){caret-color:#eab308}.dark\:caret-yellow-500\/0:is(.dark *){caret-color:rgba(234,179,8,0)}.dark\:caret-yellow-500\/10:is(.dark *){caret-color:rgba(234,179,8,.1)}.dark\:caret-yellow-500\/100:is(.dark *){caret-color:#eab308}.dark\:caret-yellow-500\/15:is(.dark *){caret-color:rgba(234,179,8,.15)}.dark\:caret-yellow-500\/20:is(.dark *){caret-color:rgba(234,179,8,.2)}.dark\:caret-yellow-500\/25:is(.dark *){caret-color:rgba(234,179,8,.25)}.dark\:caret-yellow-500\/30:is(.dark *){caret-color:rgba(234,179,8,.3)}.dark\:caret-yellow-500\/35:is(.dark *){caret-color:rgba(234,179,8,.35)}.dark\:caret-yellow-500\/40:is(.dark *){caret-color:rgba(234,179,8,.4)}.dark\:caret-yellow-500\/45:is(.dark *){caret-color:rgba(234,179,8,.45)}.dark\:caret-yellow-500\/5:is(.dark *){caret-color:rgba(234,179,8,.05)}.dark\:caret-yellow-500\/50:is(.dark *){caret-color:rgba(234,179,8,.5)}.dark\:caret-yellow-500\/55:is(.dark *){caret-color:rgba(234,179,8,.55)}.dark\:caret-yellow-500\/60:is(.dark *){caret-color:rgba(234,179,8,.6)}.dark\:caret-yellow-500\/65:is(.dark *){caret-color:rgba(234,179,8,.65)}.dark\:caret-yellow-500\/70:is(.dark *){caret-color:rgba(234,179,8,.7)}.dark\:caret-yellow-500\/75:is(.dark *){caret-color:rgba(234,179,8,.75)}.dark\:caret-yellow-500\/80:is(.dark *){caret-color:rgba(234,179,8,.8)}.dark\:caret-yellow-500\/85:is(.dark *){caret-color:rgba(234,179,8,.85)}.dark\:caret-yellow-500\/90:is(.dark *){caret-color:rgba(234,179,8,.9)}.dark\:caret-yellow-500\/95:is(.dark *){caret-color:rgba(234,179,8,.95)}.dark\:caret-yellow-950:is(.dark *){caret-color:#422006}.dark\:caret-yellow-950\/0:is(.dark *){caret-color:rgba(66,32,6,0)}.dark\:caret-yellow-950\/10:is(.dark *){caret-color:rgba(66,32,6,.1)}.dark\:caret-yellow-950\/100:is(.dark *){caret-color:#422006}.dark\:caret-yellow-950\/15:is(.dark *){caret-color:rgba(66,32,6,.15)}.dark\:caret-yellow-950\/20:is(.dark *){caret-color:rgba(66,32,6,.2)}.dark\:caret-yellow-950\/25:is(.dark *){caret-color:rgba(66,32,6,.25)}.dark\:caret-yellow-950\/30:is(.dark *){caret-color:rgba(66,32,6,.3)}.dark\:caret-yellow-950\/35:is(.dark *){caret-color:rgba(66,32,6,.35)}.dark\:caret-yellow-950\/40:is(.dark *){caret-color:rgba(66,32,6,.4)}.dark\:caret-yellow-950\/45:is(.dark *){caret-color:rgba(66,32,6,.45)}.dark\:caret-yellow-950\/5:is(.dark *){caret-color:rgba(66,32,6,.05)}.dark\:caret-yellow-950\/50:is(.dark *){caret-color:rgba(66,32,6,.5)}.dark\:caret-yellow-950\/55:is(.dark *){caret-color:rgba(66,32,6,.55)}.dark\:caret-yellow-950\/60:is(.dark *){caret-color:rgba(66,32,6,.6)}.dark\:caret-yellow-950\/65:is(.dark *){caret-color:rgba(66,32,6,.65)}.dark\:caret-yellow-950\/70:is(.dark *){caret-color:rgba(66,32,6,.7)}.dark\:caret-yellow-950\/75:is(.dark *){caret-color:rgba(66,32,6,.75)}.dark\:caret-yellow-950\/80:is(.dark *){caret-color:rgba(66,32,6,.8)}.dark\:caret-yellow-950\/85:is(.dark *){caret-color:rgba(66,32,6,.85)}.dark\:caret-yellow-950\/90:is(.dark *){caret-color:rgba(66,32,6,.9)}.dark\:caret-yellow-950\/95:is(.dark *){caret-color:rgba(66,32,6,.95)}.dark\:caret-zinc-100:is(.dark *){caret-color:#f4f4f5}.dark\:caret-zinc-100\/0:is(.dark *){caret-color:hsla(240,5%,96%,0)}.dark\:caret-zinc-100\/10:is(.dark *){caret-color:hsla(240,5%,96%,.1)}.dark\:caret-zinc-100\/100:is(.dark *){caret-color:#f4f4f5}.dark\:caret-zinc-100\/15:is(.dark *){caret-color:hsla(240,5%,96%,.15)}.dark\:caret-zinc-100\/20:is(.dark *){caret-color:hsla(240,5%,96%,.2)}.dark\:caret-zinc-100\/25:is(.dark *){caret-color:hsla(240,5%,96%,.25)}.dark\:caret-zinc-100\/30:is(.dark *){caret-color:hsla(240,5%,96%,.3)}.dark\:caret-zinc-100\/35:is(.dark *){caret-color:hsla(240,5%,96%,.35)}.dark\:caret-zinc-100\/40:is(.dark *){caret-color:hsla(240,5%,96%,.4)}.dark\:caret-zinc-100\/45:is(.dark *){caret-color:hsla(240,5%,96%,.45)}.dark\:caret-zinc-100\/5:is(.dark *){caret-color:hsla(240,5%,96%,.05)}.dark\:caret-zinc-100\/50:is(.dark *){caret-color:hsla(240,5%,96%,.5)}.dark\:caret-zinc-100\/55:is(.dark *){caret-color:hsla(240,5%,96%,.55)}.dark\:caret-zinc-100\/60:is(.dark *){caret-color:hsla(240,5%,96%,.6)}.dark\:caret-zinc-100\/65:is(.dark *){caret-color:hsla(240,5%,96%,.65)}.dark\:caret-zinc-100\/70:is(.dark *){caret-color:hsla(240,5%,96%,.7)}.dark\:caret-zinc-100\/75:is(.dark *){caret-color:hsla(240,5%,96%,.75)}.dark\:caret-zinc-100\/80:is(.dark *){caret-color:hsla(240,5%,96%,.8)}.dark\:caret-zinc-100\/85:is(.dark *){caret-color:hsla(240,5%,96%,.85)}.dark\:caret-zinc-100\/90:is(.dark *){caret-color:hsla(240,5%,96%,.9)}.dark\:caret-zinc-100\/95:is(.dark *){caret-color:hsla(240,5%,96%,.95)}.dark\:caret-zinc-200:is(.dark *){caret-color:#e4e4e7}.dark\:caret-zinc-200\/0:is(.dark *){caret-color:hsla(240,6%,90%,0)}.dark\:caret-zinc-200\/10:is(.dark *){caret-color:hsla(240,6%,90%,.1)}.dark\:caret-zinc-200\/100:is(.dark *){caret-color:#e4e4e7}.dark\:caret-zinc-200\/15:is(.dark *){caret-color:hsla(240,6%,90%,.15)}.dark\:caret-zinc-200\/20:is(.dark *){caret-color:hsla(240,6%,90%,.2)}.dark\:caret-zinc-200\/25:is(.dark *){caret-color:hsla(240,6%,90%,.25)}.dark\:caret-zinc-200\/30:is(.dark *){caret-color:hsla(240,6%,90%,.3)}.dark\:caret-zinc-200\/35:is(.dark *){caret-color:hsla(240,6%,90%,.35)}.dark\:caret-zinc-200\/40:is(.dark *){caret-color:hsla(240,6%,90%,.4)}.dark\:caret-zinc-200\/45:is(.dark *){caret-color:hsla(240,6%,90%,.45)}.dark\:caret-zinc-200\/5:is(.dark *){caret-color:hsla(240,6%,90%,.05)}.dark\:caret-zinc-200\/50:is(.dark *){caret-color:hsla(240,6%,90%,.5)}.dark\:caret-zinc-200\/55:is(.dark *){caret-color:hsla(240,6%,90%,.55)}.dark\:caret-zinc-200\/60:is(.dark *){caret-color:hsla(240,6%,90%,.6)}.dark\:caret-zinc-200\/65:is(.dark *){caret-color:hsla(240,6%,90%,.65)}.dark\:caret-zinc-200\/70:is(.dark *){caret-color:hsla(240,6%,90%,.7)}.dark\:caret-zinc-200\/75:is(.dark *){caret-color:hsla(240,6%,90%,.75)}.dark\:caret-zinc-200\/80:is(.dark *){caret-color:hsla(240,6%,90%,.8)}.dark\:caret-zinc-200\/85:is(.dark *){caret-color:hsla(240,6%,90%,.85)}.dark\:caret-zinc-200\/90:is(.dark *){caret-color:hsla(240,6%,90%,.9)}.dark\:caret-zinc-200\/95:is(.dark *){caret-color:hsla(240,6%,90%,.95)}.dark\:caret-zinc-400:is(.dark *){caret-color:#a1a1aa}.dark\:caret-zinc-400\/0:is(.dark *){caret-color:hsla(240,5%,65%,0)}.dark\:caret-zinc-400\/10:is(.dark *){caret-color:hsla(240,5%,65%,.1)}.dark\:caret-zinc-400\/100:is(.dark *){caret-color:#a1a1aa}.dark\:caret-zinc-400\/15:is(.dark *){caret-color:hsla(240,5%,65%,.15)}.dark\:caret-zinc-400\/20:is(.dark *){caret-color:hsla(240,5%,65%,.2)}.dark\:caret-zinc-400\/25:is(.dark *){caret-color:hsla(240,5%,65%,.25)}.dark\:caret-zinc-400\/30:is(.dark *){caret-color:hsla(240,5%,65%,.3)}.dark\:caret-zinc-400\/35:is(.dark *){caret-color:hsla(240,5%,65%,.35)}.dark\:caret-zinc-400\/40:is(.dark *){caret-color:hsla(240,5%,65%,.4)}.dark\:caret-zinc-400\/45:is(.dark *){caret-color:hsla(240,5%,65%,.45)}.dark\:caret-zinc-400\/5:is(.dark *){caret-color:hsla(240,5%,65%,.05)}.dark\:caret-zinc-400\/50:is(.dark *){caret-color:hsla(240,5%,65%,.5)}.dark\:caret-zinc-400\/55:is(.dark *){caret-color:hsla(240,5%,65%,.55)}.dark\:caret-zinc-400\/60:is(.dark *){caret-color:hsla(240,5%,65%,.6)}.dark\:caret-zinc-400\/65:is(.dark *){caret-color:hsla(240,5%,65%,.65)}.dark\:caret-zinc-400\/70:is(.dark *){caret-color:hsla(240,5%,65%,.7)}.dark\:caret-zinc-400\/75:is(.dark *){caret-color:hsla(240,5%,65%,.75)}.dark\:caret-zinc-400\/80:is(.dark *){caret-color:hsla(240,5%,65%,.8)}.dark\:caret-zinc-400\/85:is(.dark *){caret-color:hsla(240,5%,65%,.85)}.dark\:caret-zinc-400\/90:is(.dark *){caret-color:hsla(240,5%,65%,.9)}.dark\:caret-zinc-400\/95:is(.dark *){caret-color:hsla(240,5%,65%,.95)}.dark\:caret-zinc-50:is(.dark *){caret-color:#fafafa}.dark\:caret-zinc-50\/0:is(.dark *){caret-color:hsla(0,0%,98%,0)}.dark\:caret-zinc-50\/10:is(.dark *){caret-color:hsla(0,0%,98%,.1)}.dark\:caret-zinc-50\/100:is(.dark *){caret-color:#fafafa}.dark\:caret-zinc-50\/15:is(.dark *){caret-color:hsla(0,0%,98%,.15)}.dark\:caret-zinc-50\/20:is(.dark *){caret-color:hsla(0,0%,98%,.2)}.dark\:caret-zinc-50\/25:is(.dark *){caret-color:hsla(0,0%,98%,.25)}.dark\:caret-zinc-50\/30:is(.dark *){caret-color:hsla(0,0%,98%,.3)}.dark\:caret-zinc-50\/35:is(.dark *){caret-color:hsla(0,0%,98%,.35)}.dark\:caret-zinc-50\/40:is(.dark *){caret-color:hsla(0,0%,98%,.4)}.dark\:caret-zinc-50\/45:is(.dark *){caret-color:hsla(0,0%,98%,.45)}.dark\:caret-zinc-50\/5:is(.dark *){caret-color:hsla(0,0%,98%,.05)}.dark\:caret-zinc-50\/50:is(.dark *){caret-color:hsla(0,0%,98%,.5)}.dark\:caret-zinc-50\/55:is(.dark *){caret-color:hsla(0,0%,98%,.55)}.dark\:caret-zinc-50\/60:is(.dark *){caret-color:hsla(0,0%,98%,.6)}.dark\:caret-zinc-50\/65:is(.dark *){caret-color:hsla(0,0%,98%,.65)}.dark\:caret-zinc-50\/70:is(.dark *){caret-color:hsla(0,0%,98%,.7)}.dark\:caret-zinc-50\/75:is(.dark *){caret-color:hsla(0,0%,98%,.75)}.dark\:caret-zinc-50\/80:is(.dark *){caret-color:hsla(0,0%,98%,.8)}.dark\:caret-zinc-50\/85:is(.dark *){caret-color:hsla(0,0%,98%,.85)}.dark\:caret-zinc-50\/90:is(.dark *){caret-color:hsla(0,0%,98%,.9)}.dark\:caret-zinc-50\/95:is(.dark *){caret-color:hsla(0,0%,98%,.95)}.dark\:caret-zinc-500:is(.dark *){caret-color:#71717a}.dark\:caret-zinc-500\/0:is(.dark *){caret-color:hsla(240,4%,46%,0)}.dark\:caret-zinc-500\/10:is(.dark *){caret-color:hsla(240,4%,46%,.1)}.dark\:caret-zinc-500\/100:is(.dark *){caret-color:#71717a}.dark\:caret-zinc-500\/15:is(.dark *){caret-color:hsla(240,4%,46%,.15)}.dark\:caret-zinc-500\/20:is(.dark *){caret-color:hsla(240,4%,46%,.2)}.dark\:caret-zinc-500\/25:is(.dark *){caret-color:hsla(240,4%,46%,.25)}.dark\:caret-zinc-500\/30:is(.dark *){caret-color:hsla(240,4%,46%,.3)}.dark\:caret-zinc-500\/35:is(.dark *){caret-color:hsla(240,4%,46%,.35)}.dark\:caret-zinc-500\/40:is(.dark *){caret-color:hsla(240,4%,46%,.4)}.dark\:caret-zinc-500\/45:is(.dark *){caret-color:hsla(240,4%,46%,.45)}.dark\:caret-zinc-500\/5:is(.dark *){caret-color:hsla(240,4%,46%,.05)}.dark\:caret-zinc-500\/50:is(.dark *){caret-color:hsla(240,4%,46%,.5)}.dark\:caret-zinc-500\/55:is(.dark *){caret-color:hsla(240,4%,46%,.55)}.dark\:caret-zinc-500\/60:is(.dark *){caret-color:hsla(240,4%,46%,.6)}.dark\:caret-zinc-500\/65:is(.dark *){caret-color:hsla(240,4%,46%,.65)}.dark\:caret-zinc-500\/70:is(.dark *){caret-color:hsla(240,4%,46%,.7)}.dark\:caret-zinc-500\/75:is(.dark *){caret-color:hsla(240,4%,46%,.75)}.dark\:caret-zinc-500\/80:is(.dark *){caret-color:hsla(240,4%,46%,.8)}.dark\:caret-zinc-500\/85:is(.dark *){caret-color:hsla(240,4%,46%,.85)}.dark\:caret-zinc-500\/90:is(.dark *){caret-color:hsla(240,4%,46%,.9)}.dark\:caret-zinc-500\/95:is(.dark *){caret-color:hsla(240,4%,46%,.95)}.dark\:caret-zinc-950:is(.dark *){caret-color:#09090b}.dark\:caret-zinc-950\/0:is(.dark *){caret-color:rgba(9,9,11,0)}.dark\:caret-zinc-950\/10:is(.dark *){caret-color:rgba(9,9,11,.1)}.dark\:caret-zinc-950\/100:is(.dark *){caret-color:#09090b}.dark\:caret-zinc-950\/15:is(.dark *){caret-color:rgba(9,9,11,.15)}.dark\:caret-zinc-950\/20:is(.dark *){caret-color:rgba(9,9,11,.2)}.dark\:caret-zinc-950\/25:is(.dark *){caret-color:rgba(9,9,11,.25)}.dark\:caret-zinc-950\/30:is(.dark *){caret-color:rgba(9,9,11,.3)}.dark\:caret-zinc-950\/35:is(.dark *){caret-color:rgba(9,9,11,.35)}.dark\:caret-zinc-950\/40:is(.dark *){caret-color:rgba(9,9,11,.4)}.dark\:caret-zinc-950\/45:is(.dark *){caret-color:rgba(9,9,11,.45)}.dark\:caret-zinc-950\/5:is(.dark *){caret-color:rgba(9,9,11,.05)}.dark\:caret-zinc-950\/50:is(.dark *){caret-color:rgba(9,9,11,.5)}.dark\:caret-zinc-950\/55:is(.dark *){caret-color:rgba(9,9,11,.55)}.dark\:caret-zinc-950\/60:is(.dark *){caret-color:rgba(9,9,11,.6)}.dark\:caret-zinc-950\/65:is(.dark *){caret-color:rgba(9,9,11,.65)}.dark\:caret-zinc-950\/70:is(.dark *){caret-color:rgba(9,9,11,.7)}.dark\:caret-zinc-950\/75:is(.dark *){caret-color:rgba(9,9,11,.75)}.dark\:caret-zinc-950\/80:is(.dark *){caret-color:rgba(9,9,11,.8)}.dark\:caret-zinc-950\/85:is(.dark *){caret-color:rgba(9,9,11,.85)}.dark\:caret-zinc-950\/90:is(.dark *){caret-color:rgba(9,9,11,.9)}.dark\:caret-zinc-950\/95:is(.dark *){caret-color:rgba(9,9,11,.95)}.dark\:shadow-none:is(.dark *){--tw-shadow:0 0 #0000;--tw-shadow-colored:0 0 #0000;box-shadow:var(--tw-ring-offset-shadow,0 0 #0000),var(--tw-ring-shadow,0 0 #0000),var(--tw-shadow)}.dark\:shadow-sm:is(.dark *){--tw-shadow:0 1px 2px 0 rgba(0,0,0,.05);--tw-shadow-colored:0 1px 2px 0 var(--tw-shadow-color);box-shadow:var(--tw-ring-offset-shadow,0 0 #0000),var(--tw-ring-shadow,0 0 #0000),var(--tw-shadow)}.dark\:ring-amber-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(254 243 199/var(--tw-ring-opacity,1))}.dark\:ring-amber-100\/0:is(.dark *){--tw-ring-color:hsla(48,96%,89%,0)}.dark\:ring-amber-100\/10:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.1)}.dark\:ring-amber-100\/100:is(.dark *){--tw-ring-color:#fef3c7}.dark\:ring-amber-100\/15:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.15)}.dark\:ring-amber-100\/20:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.2)}.dark\:ring-amber-100\/25:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.25)}.dark\:ring-amber-100\/30:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.3)}.dark\:ring-amber-100\/35:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.35)}.dark\:ring-amber-100\/40:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.4)}.dark\:ring-amber-100\/45:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.45)}.dark\:ring-amber-100\/5:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.05)}.dark\:ring-amber-100\/50:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.5)}.dark\:ring-amber-100\/55:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.55)}.dark\:ring-amber-100\/60:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.6)}.dark\:ring-amber-100\/65:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.65)}.dark\:ring-amber-100\/70:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.7)}.dark\:ring-amber-100\/75:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.75)}.dark\:ring-amber-100\/80:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.8)}.dark\:ring-amber-100\/85:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.85)}.dark\:ring-amber-100\/90:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.9)}.dark\:ring-amber-100\/95:is(.dark *){--tw-ring-color:hsla(48,96%,89%,.95)}.dark\:ring-amber-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(253 230 138/var(--tw-ring-opacity,1))}.dark\:ring-amber-200\/0:is(.dark *){--tw-ring-color:hsla(48,97%,77%,0)}.dark\:ring-amber-200\/10:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.1)}.dark\:ring-amber-200\/100:is(.dark *){--tw-ring-color:#fde68a}.dark\:ring-amber-200\/15:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.15)}.dark\:ring-amber-200\/20:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.2)}.dark\:ring-amber-200\/25:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.25)}.dark\:ring-amber-200\/30:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.3)}.dark\:ring-amber-200\/35:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.35)}.dark\:ring-amber-200\/40:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.4)}.dark\:ring-amber-200\/45:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.45)}.dark\:ring-amber-200\/5:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.05)}.dark\:ring-amber-200\/50:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.5)}.dark\:ring-amber-200\/55:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.55)}.dark\:ring-amber-200\/60:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.6)}.dark\:ring-amber-200\/65:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.65)}.dark\:ring-amber-200\/70:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.7)}.dark\:ring-amber-200\/75:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.75)}.dark\:ring-amber-200\/80:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.8)}.dark\:ring-amber-200\/85:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.85)}.dark\:ring-amber-200\/90:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.9)}.dark\:ring-amber-200\/95:is(.dark *){--tw-ring-color:hsla(48,97%,77%,.95)}.dark\:ring-amber-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(251 191 36/var(--tw-ring-opacity,1))}.dark\:ring-amber-400\/0:is(.dark *){--tw-ring-color:rgba(251,191,36,0)}.dark\:ring-amber-400\/10:is(.dark *){--tw-ring-color:rgba(251,191,36,.1)}.dark\:ring-amber-400\/100:is(.dark *){--tw-ring-color:#fbbf24}.dark\:ring-amber-400\/15:is(.dark *){--tw-ring-color:rgba(251,191,36,.15)}.dark\:ring-amber-400\/20:is(.dark *){--tw-ring-color:rgba(251,191,36,.2)}.dark\:ring-amber-400\/25:is(.dark *){--tw-ring-color:rgba(251,191,36,.25)}.dark\:ring-amber-400\/30:is(.dark *){--tw-ring-color:rgba(251,191,36,.3)}.dark\:ring-amber-400\/35:is(.dark *){--tw-ring-color:rgba(251,191,36,.35)}.dark\:ring-amber-400\/40:is(.dark *){--tw-ring-color:rgba(251,191,36,.4)}.dark\:ring-amber-400\/45:is(.dark *){--tw-ring-color:rgba(251,191,36,.45)}.dark\:ring-amber-400\/5:is(.dark *){--tw-ring-color:rgba(251,191,36,.05)}.dark\:ring-amber-400\/50:is(.dark *){--tw-ring-color:rgba(251,191,36,.5)}.dark\:ring-amber-400\/55:is(.dark *){--tw-ring-color:rgba(251,191,36,.55)}.dark\:ring-amber-400\/60:is(.dark *){--tw-ring-color:rgba(251,191,36,.6)}.dark\:ring-amber-400\/65:is(.dark *){--tw-ring-color:rgba(251,191,36,.65)}.dark\:ring-amber-400\/70:is(.dark *){--tw-ring-color:rgba(251,191,36,.7)}.dark\:ring-amber-400\/75:is(.dark *){--tw-ring-color:rgba(251,191,36,.75)}.dark\:ring-amber-400\/80:is(.dark *){--tw-ring-color:rgba(251,191,36,.8)}.dark\:ring-amber-400\/85:is(.dark *){--tw-ring-color:rgba(251,191,36,.85)}.dark\:ring-amber-400\/90:is(.dark *){--tw-ring-color:rgba(251,191,36,.9)}.dark\:ring-amber-400\/95:is(.dark *){--tw-ring-color:rgba(251,191,36,.95)}.dark\:ring-amber-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(255 251 235/var(--tw-ring-opacity,1))}.dark\:ring-amber-50\/0:is(.dark *){--tw-ring-color:rgba(255,251,235,0)}.dark\:ring-amber-50\/10:is(.dark *){--tw-ring-color:rgba(255,251,235,.1)}.dark\:ring-amber-50\/100:is(.dark *){--tw-ring-color:#fffbeb}.dark\:ring-amber-50\/15:is(.dark *){--tw-ring-color:rgba(255,251,235,.15)}.dark\:ring-amber-50\/20:is(.dark *){--tw-ring-color:rgba(255,251,235,.2)}.dark\:ring-amber-50\/25:is(.dark *){--tw-ring-color:rgba(255,251,235,.25)}.dark\:ring-amber-50\/30:is(.dark *){--tw-ring-color:rgba(255,251,235,.3)}.dark\:ring-amber-50\/35:is(.dark *){--tw-ring-color:rgba(255,251,235,.35)}.dark\:ring-amber-50\/40:is(.dark *){--tw-ring-color:rgba(255,251,235,.4)}.dark\:ring-amber-50\/45:is(.dark *){--tw-ring-color:rgba(255,251,235,.45)}.dark\:ring-amber-50\/5:is(.dark *){--tw-ring-color:rgba(255,251,235,.05)}.dark\:ring-amber-50\/50:is(.dark *){--tw-ring-color:rgba(255,251,235,.5)}.dark\:ring-amber-50\/55:is(.dark *){--tw-ring-color:rgba(255,251,235,.55)}.dark\:ring-amber-50\/60:is(.dark *){--tw-ring-color:rgba(255,251,235,.6)}.dark\:ring-amber-50\/65:is(.dark *){--tw-ring-color:rgba(255,251,235,.65)}.dark\:ring-amber-50\/70:is(.dark *){--tw-ring-color:rgba(255,251,235,.7)}.dark\:ring-amber-50\/75:is(.dark *){--tw-ring-color:rgba(255,251,235,.75)}.dark\:ring-amber-50\/80:is(.dark *){--tw-ring-color:rgba(255,251,235,.8)}.dark\:ring-amber-50\/85:is(.dark *){--tw-ring-color:rgba(255,251,235,.85)}.dark\:ring-amber-50\/90:is(.dark *){--tw-ring-color:rgba(255,251,235,.9)}.dark\:ring-amber-50\/95:is(.dark *){--tw-ring-color:rgba(255,251,235,.95)}.dark\:ring-amber-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(245 158 11/var(--tw-ring-opacity,1))}.dark\:ring-amber-500\/0:is(.dark *){--tw-ring-color:rgba(245,158,11,0)}.dark\:ring-amber-500\/10:is(.dark *){--tw-ring-color:rgba(245,158,11,.1)}.dark\:ring-amber-500\/100:is(.dark *){--tw-ring-color:#f59e0b}.dark\:ring-amber-500\/15:is(.dark *){--tw-ring-color:rgba(245,158,11,.15)}.dark\:ring-amber-500\/20:is(.dark *){--tw-ring-color:rgba(245,158,11,.2)}.dark\:ring-amber-500\/25:is(.dark *){--tw-ring-color:rgba(245,158,11,.25)}.dark\:ring-amber-500\/30:is(.dark *){--tw-ring-color:rgba(245,158,11,.3)}.dark\:ring-amber-500\/35:is(.dark *){--tw-ring-color:rgba(245,158,11,.35)}.dark\:ring-amber-500\/40:is(.dark *){--tw-ring-color:rgba(245,158,11,.4)}.dark\:ring-amber-500\/45:is(.dark *){--tw-ring-color:rgba(245,158,11,.45)}.dark\:ring-amber-500\/5:is(.dark *){--tw-ring-color:rgba(245,158,11,.05)}.dark\:ring-amber-500\/50:is(.dark *){--tw-ring-color:rgba(245,158,11,.5)}.dark\:ring-amber-500\/55:is(.dark *){--tw-ring-color:rgba(245,158,11,.55)}.dark\:ring-amber-500\/60:is(.dark *){--tw-ring-color:rgba(245,158,11,.6)}.dark\:ring-amber-500\/65:is(.dark *){--tw-ring-color:rgba(245,158,11,.65)}.dark\:ring-amber-500\/70:is(.dark *){--tw-ring-color:rgba(245,158,11,.7)}.dark\:ring-amber-500\/75:is(.dark *){--tw-ring-color:rgba(245,158,11,.75)}.dark\:ring-amber-500\/80:is(.dark *){--tw-ring-color:rgba(245,158,11,.8)}.dark\:ring-amber-500\/85:is(.dark *){--tw-ring-color:rgba(245,158,11,.85)}.dark\:ring-amber-500\/90:is(.dark *){--tw-ring-color:rgba(245,158,11,.9)}.dark\:ring-amber-500\/95:is(.dark *){--tw-ring-color:rgba(245,158,11,.95)}.dark\:ring-amber-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(69 26 3/var(--tw-ring-opacity,1))}.dark\:ring-amber-950\/0:is(.dark *){--tw-ring-color:rgba(69,26,3,0)}.dark\:ring-amber-950\/10:is(.dark *){--tw-ring-color:rgba(69,26,3,.1)}.dark\:ring-amber-950\/100:is(.dark *){--tw-ring-color:#451a03}.dark\:ring-amber-950\/15:is(.dark *){--tw-ring-color:rgba(69,26,3,.15)}.dark\:ring-amber-950\/20:is(.dark *){--tw-ring-color:rgba(69,26,3,.2)}.dark\:ring-amber-950\/25:is(.dark *){--tw-ring-color:rgba(69,26,3,.25)}.dark\:ring-amber-950\/30:is(.dark *){--tw-ring-color:rgba(69,26,3,.3)}.dark\:ring-amber-950\/35:is(.dark *){--tw-ring-color:rgba(69,26,3,.35)}.dark\:ring-amber-950\/40:is(.dark *){--tw-ring-color:rgba(69,26,3,.4)}.dark\:ring-amber-950\/45:is(.dark *){--tw-ring-color:rgba(69,26,3,.45)}.dark\:ring-amber-950\/5:is(.dark *){--tw-ring-color:rgba(69,26,3,.05)}.dark\:ring-amber-950\/50:is(.dark *){--tw-ring-color:rgba(69,26,3,.5)}.dark\:ring-amber-950\/55:is(.dark *){--tw-ring-color:rgba(69,26,3,.55)}.dark\:ring-amber-950\/60:is(.dark *){--tw-ring-color:rgba(69,26,3,.6)}.dark\:ring-amber-950\/65:is(.dark *){--tw-ring-color:rgba(69,26,3,.65)}.dark\:ring-amber-950\/70:is(.dark *){--tw-ring-color:rgba(69,26,3,.7)}.dark\:ring-amber-950\/75:is(.dark *){--tw-ring-color:rgba(69,26,3,.75)}.dark\:ring-amber-950\/80:is(.dark *){--tw-ring-color:rgba(69,26,3,.8)}.dark\:ring-amber-950\/85:is(.dark *){--tw-ring-color:rgba(69,26,3,.85)}.dark\:ring-amber-950\/90:is(.dark *){--tw-ring-color:rgba(69,26,3,.9)}.dark\:ring-amber-950\/95:is(.dark *){--tw-ring-color:rgba(69,26,3,.95)}.dark\:ring-blue-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(219 234 254/var(--tw-ring-opacity,1))}.dark\:ring-blue-100\/0:is(.dark *){--tw-ring-color:rgba(219,234,254,0)}.dark\:ring-blue-100\/10:is(.dark *){--tw-ring-color:rgba(219,234,254,.1)}.dark\:ring-blue-100\/100:is(.dark *){--tw-ring-color:#dbeafe}.dark\:ring-blue-100\/15:is(.dark *){--tw-ring-color:rgba(219,234,254,.15)}.dark\:ring-blue-100\/20:is(.dark *){--tw-ring-color:rgba(219,234,254,.2)}.dark\:ring-blue-100\/25:is(.dark *){--tw-ring-color:rgba(219,234,254,.25)}.dark\:ring-blue-100\/30:is(.dark *){--tw-ring-color:rgba(219,234,254,.3)}.dark\:ring-blue-100\/35:is(.dark *){--tw-ring-color:rgba(219,234,254,.35)}.dark\:ring-blue-100\/40:is(.dark *){--tw-ring-color:rgba(219,234,254,.4)}.dark\:ring-blue-100\/45:is(.dark *){--tw-ring-color:rgba(219,234,254,.45)}.dark\:ring-blue-100\/5:is(.dark *){--tw-ring-color:rgba(219,234,254,.05)}.dark\:ring-blue-100\/50:is(.dark *){--tw-ring-color:rgba(219,234,254,.5)}.dark\:ring-blue-100\/55:is(.dark *){--tw-ring-color:rgba(219,234,254,.55)}.dark\:ring-blue-100\/60:is(.dark *){--tw-ring-color:rgba(219,234,254,.6)}.dark\:ring-blue-100\/65:is(.dark *){--tw-ring-color:rgba(219,234,254,.65)}.dark\:ring-blue-100\/70:is(.dark *){--tw-ring-color:rgba(219,234,254,.7)}.dark\:ring-blue-100\/75:is(.dark *){--tw-ring-color:rgba(219,234,254,.75)}.dark\:ring-blue-100\/80:is(.dark *){--tw-ring-color:rgba(219,234,254,.8)}.dark\:ring-blue-100\/85:is(.dark *){--tw-ring-color:rgba(219,234,254,.85)}.dark\:ring-blue-100\/90:is(.dark *){--tw-ring-color:rgba(219,234,254,.9)}.dark\:ring-blue-100\/95:is(.dark *){--tw-ring-color:rgba(219,234,254,.95)}.dark\:ring-blue-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(191 219 254/var(--tw-ring-opacity,1))}.dark\:ring-blue-200\/0:is(.dark *){--tw-ring-color:rgba(191,219,254,0)}.dark\:ring-blue-200\/10:is(.dark *){--tw-ring-color:rgba(191,219,254,.1)}.dark\:ring-blue-200\/100:is(.dark *){--tw-ring-color:#bfdbfe}.dark\:ring-blue-200\/15:is(.dark *){--tw-ring-color:rgba(191,219,254,.15)}.dark\:ring-blue-200\/20:is(.dark *){--tw-ring-color:rgba(191,219,254,.2)}.dark\:ring-blue-200\/25:is(.dark *){--tw-ring-color:rgba(191,219,254,.25)}.dark\:ring-blue-200\/30:is(.dark *){--tw-ring-color:rgba(191,219,254,.3)}.dark\:ring-blue-200\/35:is(.dark *){--tw-ring-color:rgba(191,219,254,.35)}.dark\:ring-blue-200\/40:is(.dark *){--tw-ring-color:rgba(191,219,254,.4)}.dark\:ring-blue-200\/45:is(.dark *){--tw-ring-color:rgba(191,219,254,.45)}.dark\:ring-blue-200\/5:is(.dark *){--tw-ring-color:rgba(191,219,254,.05)}.dark\:ring-blue-200\/50:is(.dark *){--tw-ring-color:rgba(191,219,254,.5)}.dark\:ring-blue-200\/55:is(.dark *){--tw-ring-color:rgba(191,219,254,.55)}.dark\:ring-blue-200\/60:is(.dark *){--tw-ring-color:rgba(191,219,254,.6)}.dark\:ring-blue-200\/65:is(.dark *){--tw-ring-color:rgba(191,219,254,.65)}.dark\:ring-blue-200\/70:is(.dark *){--tw-ring-color:rgba(191,219,254,.7)}.dark\:ring-blue-200\/75:is(.dark *){--tw-ring-color:rgba(191,219,254,.75)}.dark\:ring-blue-200\/80:is(.dark *){--tw-ring-color:rgba(191,219,254,.8)}.dark\:ring-blue-200\/85:is(.dark *){--tw-ring-color:rgba(191,219,254,.85)}.dark\:ring-blue-200\/90:is(.dark *){--tw-ring-color:rgba(191,219,254,.9)}.dark\:ring-blue-200\/95:is(.dark *){--tw-ring-color:rgba(191,219,254,.95)}.dark\:ring-blue-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(96 165 250/var(--tw-ring-opacity,1))}.dark\:ring-blue-400\/0:is(.dark *){--tw-ring-color:rgba(96,165,250,0)}.dark\:ring-blue-400\/10:is(.dark *){--tw-ring-color:rgba(96,165,250,.1)}.dark\:ring-blue-400\/100:is(.dark *){--tw-ring-color:#60a5fa}.dark\:ring-blue-400\/15:is(.dark *){--tw-ring-color:rgba(96,165,250,.15)}.dark\:ring-blue-400\/20:is(.dark *){--tw-ring-color:rgba(96,165,250,.2)}.dark\:ring-blue-400\/25:is(.dark *){--tw-ring-color:rgba(96,165,250,.25)}.dark\:ring-blue-400\/30:is(.dark *){--tw-ring-color:rgba(96,165,250,.3)}.dark\:ring-blue-400\/35:is(.dark *){--tw-ring-color:rgba(96,165,250,.35)}.dark\:ring-blue-400\/40:is(.dark *){--tw-ring-color:rgba(96,165,250,.4)}.dark\:ring-blue-400\/45:is(.dark *){--tw-ring-color:rgba(96,165,250,.45)}.dark\:ring-blue-400\/5:is(.dark *){--tw-ring-color:rgba(96,165,250,.05)}.dark\:ring-blue-400\/50:is(.dark *){--tw-ring-color:rgba(96,165,250,.5)}.dark\:ring-blue-400\/55:is(.dark *){--tw-ring-color:rgba(96,165,250,.55)}.dark\:ring-blue-400\/60:is(.dark *){--tw-ring-color:rgba(96,165,250,.6)}.dark\:ring-blue-400\/65:is(.dark *){--tw-ring-color:rgba(96,165,250,.65)}.dark\:ring-blue-400\/70:is(.dark *){--tw-ring-color:rgba(96,165,250,.7)}.dark\:ring-blue-400\/75:is(.dark *){--tw-ring-color:rgba(96,165,250,.75)}.dark\:ring-blue-400\/80:is(.dark *){--tw-ring-color:rgba(96,165,250,.8)}.dark\:ring-blue-400\/85:is(.dark *){--tw-ring-color:rgba(96,165,250,.85)}.dark\:ring-blue-400\/90:is(.dark *){--tw-ring-color:rgba(96,165,250,.9)}.dark\:ring-blue-400\/95:is(.dark *){--tw-ring-color:rgba(96,165,250,.95)}.dark\:ring-blue-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(239 246 255/var(--tw-ring-opacity,1))}.dark\:ring-blue-50\/0:is(.dark *){--tw-ring-color:rgba(239,246,255,0)}.dark\:ring-blue-50\/10:is(.dark *){--tw-ring-color:rgba(239,246,255,.1)}.dark\:ring-blue-50\/100:is(.dark *){--tw-ring-color:#eff6ff}.dark\:ring-blue-50\/15:is(.dark *){--tw-ring-color:rgba(239,246,255,.15)}.dark\:ring-blue-50\/20:is(.dark *){--tw-ring-color:rgba(239,246,255,.2)}.dark\:ring-blue-50\/25:is(.dark *){--tw-ring-color:rgba(239,246,255,.25)}.dark\:ring-blue-50\/30:is(.dark *){--tw-ring-color:rgba(239,246,255,.3)}.dark\:ring-blue-50\/35:is(.dark *){--tw-ring-color:rgba(239,246,255,.35)}.dark\:ring-blue-50\/40:is(.dark *){--tw-ring-color:rgba(239,246,255,.4)}.dark\:ring-blue-50\/45:is(.dark *){--tw-ring-color:rgba(239,246,255,.45)}.dark\:ring-blue-50\/5:is(.dark *){--tw-ring-color:rgba(239,246,255,.05)}.dark\:ring-blue-50\/50:is(.dark *){--tw-ring-color:rgba(239,246,255,.5)}.dark\:ring-blue-50\/55:is(.dark *){--tw-ring-color:rgba(239,246,255,.55)}.dark\:ring-blue-50\/60:is(.dark *){--tw-ring-color:rgba(239,246,255,.6)}.dark\:ring-blue-50\/65:is(.dark *){--tw-ring-color:rgba(239,246,255,.65)}.dark\:ring-blue-50\/70:is(.dark *){--tw-ring-color:rgba(239,246,255,.7)}.dark\:ring-blue-50\/75:is(.dark *){--tw-ring-color:rgba(239,246,255,.75)}.dark\:ring-blue-50\/80:is(.dark *){--tw-ring-color:rgba(239,246,255,.8)}.dark\:ring-blue-50\/85:is(.dark *){--tw-ring-color:rgba(239,246,255,.85)}.dark\:ring-blue-50\/90:is(.dark *){--tw-ring-color:rgba(239,246,255,.9)}.dark\:ring-blue-50\/95:is(.dark *){--tw-ring-color:rgba(239,246,255,.95)}.dark\:ring-blue-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(59 130 246/var(--tw-ring-opacity,1))}.dark\:ring-blue-500\/0:is(.dark *){--tw-ring-color:rgba(59,130,246,0)}.dark\:ring-blue-500\/10:is(.dark *){--tw-ring-color:rgba(59,130,246,.1)}.dark\:ring-blue-500\/100:is(.dark *){--tw-ring-color:#3b82f6}.dark\:ring-blue-500\/15:is(.dark *){--tw-ring-color:rgba(59,130,246,.15)}.dark\:ring-blue-500\/20:is(.dark *){--tw-ring-color:rgba(59,130,246,.2)}.dark\:ring-blue-500\/25:is(.dark *){--tw-ring-color:rgba(59,130,246,.25)}.dark\:ring-blue-500\/30:is(.dark *){--tw-ring-color:rgba(59,130,246,.3)}.dark\:ring-blue-500\/35:is(.dark *){--tw-ring-color:rgba(59,130,246,.35)}.dark\:ring-blue-500\/40:is(.dark *){--tw-ring-color:rgba(59,130,246,.4)}.dark\:ring-blue-500\/45:is(.dark *){--tw-ring-color:rgba(59,130,246,.45)}.dark\:ring-blue-500\/5:is(.dark *){--tw-ring-color:rgba(59,130,246,.05)}.dark\:ring-blue-500\/50:is(.dark *){--tw-ring-color:rgba(59,130,246,.5)}.dark\:ring-blue-500\/55:is(.dark *){--tw-ring-color:rgba(59,130,246,.55)}.dark\:ring-blue-500\/60:is(.dark *){--tw-ring-color:rgba(59,130,246,.6)}.dark\:ring-blue-500\/65:is(.dark *){--tw-ring-color:rgba(59,130,246,.65)}.dark\:ring-blue-500\/70:is(.dark *){--tw-ring-color:rgba(59,130,246,.7)}.dark\:ring-blue-500\/75:is(.dark *){--tw-ring-color:rgba(59,130,246,.75)}.dark\:ring-blue-500\/80:is(.dark *){--tw-ring-color:rgba(59,130,246,.8)}.dark\:ring-blue-500\/85:is(.dark *){--tw-ring-color:rgba(59,130,246,.85)}.dark\:ring-blue-500\/90:is(.dark *){--tw-ring-color:rgba(59,130,246,.9)}.dark\:ring-blue-500\/95:is(.dark *){--tw-ring-color:rgba(59,130,246,.95)}.dark\:ring-blue-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(23 37 84/var(--tw-ring-opacity,1))}.dark\:ring-blue-950\/0:is(.dark *){--tw-ring-color:rgba(23,37,84,0)}.dark\:ring-blue-950\/10:is(.dark *){--tw-ring-color:rgba(23,37,84,.1)}.dark\:ring-blue-950\/100:is(.dark *){--tw-ring-color:#172554}.dark\:ring-blue-950\/15:is(.dark *){--tw-ring-color:rgba(23,37,84,.15)}.dark\:ring-blue-950\/20:is(.dark *){--tw-ring-color:rgba(23,37,84,.2)}.dark\:ring-blue-950\/25:is(.dark *){--tw-ring-color:rgba(23,37,84,.25)}.dark\:ring-blue-950\/30:is(.dark *){--tw-ring-color:rgba(23,37,84,.3)}.dark\:ring-blue-950\/35:is(.dark *){--tw-ring-color:rgba(23,37,84,.35)}.dark\:ring-blue-950\/40:is(.dark *){--tw-ring-color:rgba(23,37,84,.4)}.dark\:ring-blue-950\/45:is(.dark *){--tw-ring-color:rgba(23,37,84,.45)}.dark\:ring-blue-950\/5:is(.dark *){--tw-ring-color:rgba(23,37,84,.05)}.dark\:ring-blue-950\/50:is(.dark *){--tw-ring-color:rgba(23,37,84,.5)}.dark\:ring-blue-950\/55:is(.dark *){--tw-ring-color:rgba(23,37,84,.55)}.dark\:ring-blue-950\/60:is(.dark *){--tw-ring-color:rgba(23,37,84,.6)}.dark\:ring-blue-950\/65:is(.dark *){--tw-ring-color:rgba(23,37,84,.65)}.dark\:ring-blue-950\/70:is(.dark *){--tw-ring-color:rgba(23,37,84,.7)}.dark\:ring-blue-950\/75:is(.dark *){--tw-ring-color:rgba(23,37,84,.75)}.dark\:ring-blue-950\/80:is(.dark *){--tw-ring-color:rgba(23,37,84,.8)}.dark\:ring-blue-950\/85:is(.dark *){--tw-ring-color:rgba(23,37,84,.85)}.dark\:ring-blue-950\/90:is(.dark *){--tw-ring-color:rgba(23,37,84,.9)}.dark\:ring-blue-950\/95:is(.dark *){--tw-ring-color:rgba(23,37,84,.95)}.dark\:ring-cyan-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(207 250 254/var(--tw-ring-opacity,1))}.dark\:ring-cyan-100\/0:is(.dark *){--tw-ring-color:rgba(207,250,254,0)}.dark\:ring-cyan-100\/10:is(.dark *){--tw-ring-color:rgba(207,250,254,.1)}.dark\:ring-cyan-100\/100:is(.dark *){--tw-ring-color:#cffafe}.dark\:ring-cyan-100\/15:is(.dark *){--tw-ring-color:rgba(207,250,254,.15)}.dark\:ring-cyan-100\/20:is(.dark *){--tw-ring-color:rgba(207,250,254,.2)}.dark\:ring-cyan-100\/25:is(.dark *){--tw-ring-color:rgba(207,250,254,.25)}.dark\:ring-cyan-100\/30:is(.dark *){--tw-ring-color:rgba(207,250,254,.3)}.dark\:ring-cyan-100\/35:is(.dark *){--tw-ring-color:rgba(207,250,254,.35)}.dark\:ring-cyan-100\/40:is(.dark *){--tw-ring-color:rgba(207,250,254,.4)}.dark\:ring-cyan-100\/45:is(.dark *){--tw-ring-color:rgba(207,250,254,.45)}.dark\:ring-cyan-100\/5:is(.dark *){--tw-ring-color:rgba(207,250,254,.05)}.dark\:ring-cyan-100\/50:is(.dark *){--tw-ring-color:rgba(207,250,254,.5)}.dark\:ring-cyan-100\/55:is(.dark *){--tw-ring-color:rgba(207,250,254,.55)}.dark\:ring-cyan-100\/60:is(.dark *){--tw-ring-color:rgba(207,250,254,.6)}.dark\:ring-cyan-100\/65:is(.dark *){--tw-ring-color:rgba(207,250,254,.65)}.dark\:ring-cyan-100\/70:is(.dark *){--tw-ring-color:rgba(207,250,254,.7)}.dark\:ring-cyan-100\/75:is(.dark *){--tw-ring-color:rgba(207,250,254,.75)}.dark\:ring-cyan-100\/80:is(.dark *){--tw-ring-color:rgba(207,250,254,.8)}.dark\:ring-cyan-100\/85:is(.dark *){--tw-ring-color:rgba(207,250,254,.85)}.dark\:ring-cyan-100\/90:is(.dark *){--tw-ring-color:rgba(207,250,254,.9)}.dark\:ring-cyan-100\/95:is(.dark *){--tw-ring-color:rgba(207,250,254,.95)}.dark\:ring-cyan-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(165 243 252/var(--tw-ring-opacity,1))}.dark\:ring-cyan-200\/0:is(.dark *){--tw-ring-color:rgba(165,243,252,0)}.dark\:ring-cyan-200\/10:is(.dark *){--tw-ring-color:rgba(165,243,252,.1)}.dark\:ring-cyan-200\/100:is(.dark *){--tw-ring-color:#a5f3fc}.dark\:ring-cyan-200\/15:is(.dark *){--tw-ring-color:rgba(165,243,252,.15)}.dark\:ring-cyan-200\/20:is(.dark *){--tw-ring-color:rgba(165,243,252,.2)}.dark\:ring-cyan-200\/25:is(.dark *){--tw-ring-color:rgba(165,243,252,.25)}.dark\:ring-cyan-200\/30:is(.dark *){--tw-ring-color:rgba(165,243,252,.3)}.dark\:ring-cyan-200\/35:is(.dark *){--tw-ring-color:rgba(165,243,252,.35)}.dark\:ring-cyan-200\/40:is(.dark *){--tw-ring-color:rgba(165,243,252,.4)}.dark\:ring-cyan-200\/45:is(.dark *){--tw-ring-color:rgba(165,243,252,.45)}.dark\:ring-cyan-200\/5:is(.dark *){--tw-ring-color:rgba(165,243,252,.05)}.dark\:ring-cyan-200\/50:is(.dark *){--tw-ring-color:rgba(165,243,252,.5)}.dark\:ring-cyan-200\/55:is(.dark *){--tw-ring-color:rgba(165,243,252,.55)}.dark\:ring-cyan-200\/60:is(.dark *){--tw-ring-color:rgba(165,243,252,.6)}.dark\:ring-cyan-200\/65:is(.dark *){--tw-ring-color:rgba(165,243,252,.65)}.dark\:ring-cyan-200\/70:is(.dark *){--tw-ring-color:rgba(165,243,252,.7)}.dark\:ring-cyan-200\/75:is(.dark *){--tw-ring-color:rgba(165,243,252,.75)}.dark\:ring-cyan-200\/80:is(.dark *){--tw-ring-color:rgba(165,243,252,.8)}.dark\:ring-cyan-200\/85:is(.dark *){--tw-ring-color:rgba(165,243,252,.85)}.dark\:ring-cyan-200\/90:is(.dark *){--tw-ring-color:rgba(165,243,252,.9)}.dark\:ring-cyan-200\/95:is(.dark *){--tw-ring-color:rgba(165,243,252,.95)}.dark\:ring-cyan-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(34 211 238/var(--tw-ring-opacity,1))}.dark\:ring-cyan-400\/0:is(.dark *){--tw-ring-color:rgba(34,211,238,0)}.dark\:ring-cyan-400\/10:is(.dark *){--tw-ring-color:rgba(34,211,238,.1)}.dark\:ring-cyan-400\/100:is(.dark *){--tw-ring-color:#22d3ee}.dark\:ring-cyan-400\/15:is(.dark *){--tw-ring-color:rgba(34,211,238,.15)}.dark\:ring-cyan-400\/20:is(.dark *){--tw-ring-color:rgba(34,211,238,.2)}.dark\:ring-cyan-400\/25:is(.dark *){--tw-ring-color:rgba(34,211,238,.25)}.dark\:ring-cyan-400\/30:is(.dark *){--tw-ring-color:rgba(34,211,238,.3)}.dark\:ring-cyan-400\/35:is(.dark *){--tw-ring-color:rgba(34,211,238,.35)}.dark\:ring-cyan-400\/40:is(.dark *){--tw-ring-color:rgba(34,211,238,.4)}.dark\:ring-cyan-400\/45:is(.dark *){--tw-ring-color:rgba(34,211,238,.45)}.dark\:ring-cyan-400\/5:is(.dark *){--tw-ring-color:rgba(34,211,238,.05)}.dark\:ring-cyan-400\/50:is(.dark *){--tw-ring-color:rgba(34,211,238,.5)}.dark\:ring-cyan-400\/55:is(.dark *){--tw-ring-color:rgba(34,211,238,.55)}.dark\:ring-cyan-400\/60:is(.dark *){--tw-ring-color:rgba(34,211,238,.6)}.dark\:ring-cyan-400\/65:is(.dark *){--tw-ring-color:rgba(34,211,238,.65)}.dark\:ring-cyan-400\/70:is(.dark *){--tw-ring-color:rgba(34,211,238,.7)}.dark\:ring-cyan-400\/75:is(.dark *){--tw-ring-color:rgba(34,211,238,.75)}.dark\:ring-cyan-400\/80:is(.dark *){--tw-ring-color:rgba(34,211,238,.8)}.dark\:ring-cyan-400\/85:is(.dark *){--tw-ring-color:rgba(34,211,238,.85)}.dark\:ring-cyan-400\/90:is(.dark *){--tw-ring-color:rgba(34,211,238,.9)}.dark\:ring-cyan-400\/95:is(.dark *){--tw-ring-color:rgba(34,211,238,.95)}.dark\:ring-cyan-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(236 254 255/var(--tw-ring-opacity,1))}.dark\:ring-cyan-50\/0:is(.dark *){--tw-ring-color:rgba(236,254,255,0)}.dark\:ring-cyan-50\/10:is(.dark *){--tw-ring-color:rgba(236,254,255,.1)}.dark\:ring-cyan-50\/100:is(.dark *){--tw-ring-color:#ecfeff}.dark\:ring-cyan-50\/15:is(.dark *){--tw-ring-color:rgba(236,254,255,.15)}.dark\:ring-cyan-50\/20:is(.dark *){--tw-ring-color:rgba(236,254,255,.2)}.dark\:ring-cyan-50\/25:is(.dark *){--tw-ring-color:rgba(236,254,255,.25)}.dark\:ring-cyan-50\/30:is(.dark *){--tw-ring-color:rgba(236,254,255,.3)}.dark\:ring-cyan-50\/35:is(.dark *){--tw-ring-color:rgba(236,254,255,.35)}.dark\:ring-cyan-50\/40:is(.dark *){--tw-ring-color:rgba(236,254,255,.4)}.dark\:ring-cyan-50\/45:is(.dark *){--tw-ring-color:rgba(236,254,255,.45)}.dark\:ring-cyan-50\/5:is(.dark *){--tw-ring-color:rgba(236,254,255,.05)}.dark\:ring-cyan-50\/50:is(.dark *){--tw-ring-color:rgba(236,254,255,.5)}.dark\:ring-cyan-50\/55:is(.dark *){--tw-ring-color:rgba(236,254,255,.55)}.dark\:ring-cyan-50\/60:is(.dark *){--tw-ring-color:rgba(236,254,255,.6)}.dark\:ring-cyan-50\/65:is(.dark *){--tw-ring-color:rgba(236,254,255,.65)}.dark\:ring-cyan-50\/70:is(.dark *){--tw-ring-color:rgba(236,254,255,.7)}.dark\:ring-cyan-50\/75:is(.dark *){--tw-ring-color:rgba(236,254,255,.75)}.dark\:ring-cyan-50\/80:is(.dark *){--tw-ring-color:rgba(236,254,255,.8)}.dark\:ring-cyan-50\/85:is(.dark *){--tw-ring-color:rgba(236,254,255,.85)}.dark\:ring-cyan-50\/90:is(.dark *){--tw-ring-color:rgba(236,254,255,.9)}.dark\:ring-cyan-50\/95:is(.dark *){--tw-ring-color:rgba(236,254,255,.95)}.dark\:ring-cyan-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(6 182 212/var(--tw-ring-opacity,1))}.dark\:ring-cyan-500\/0:is(.dark *){--tw-ring-color:rgba(6,182,212,0)}.dark\:ring-cyan-500\/10:is(.dark *){--tw-ring-color:rgba(6,182,212,.1)}.dark\:ring-cyan-500\/100:is(.dark *){--tw-ring-color:#06b6d4}.dark\:ring-cyan-500\/15:is(.dark *){--tw-ring-color:rgba(6,182,212,.15)}.dark\:ring-cyan-500\/20:is(.dark *){--tw-ring-color:rgba(6,182,212,.2)}.dark\:ring-cyan-500\/25:is(.dark *){--tw-ring-color:rgba(6,182,212,.25)}.dark\:ring-cyan-500\/30:is(.dark *){--tw-ring-color:rgba(6,182,212,.3)}.dark\:ring-cyan-500\/35:is(.dark *){--tw-ring-color:rgba(6,182,212,.35)}.dark\:ring-cyan-500\/40:is(.dark *){--tw-ring-color:rgba(6,182,212,.4)}.dark\:ring-cyan-500\/45:is(.dark *){--tw-ring-color:rgba(6,182,212,.45)}.dark\:ring-cyan-500\/5:is(.dark *){--tw-ring-color:rgba(6,182,212,.05)}.dark\:ring-cyan-500\/50:is(.dark *){--tw-ring-color:rgba(6,182,212,.5)}.dark\:ring-cyan-500\/55:is(.dark *){--tw-ring-color:rgba(6,182,212,.55)}.dark\:ring-cyan-500\/60:is(.dark *){--tw-ring-color:rgba(6,182,212,.6)}.dark\:ring-cyan-500\/65:is(.dark *){--tw-ring-color:rgba(6,182,212,.65)}.dark\:ring-cyan-500\/70:is(.dark *){--tw-ring-color:rgba(6,182,212,.7)}.dark\:ring-cyan-500\/75:is(.dark *){--tw-ring-color:rgba(6,182,212,.75)}.dark\:ring-cyan-500\/80:is(.dark *){--tw-ring-color:rgba(6,182,212,.8)}.dark\:ring-cyan-500\/85:is(.dark *){--tw-ring-color:rgba(6,182,212,.85)}.dark\:ring-cyan-500\/90:is(.dark *){--tw-ring-color:rgba(6,182,212,.9)}.dark\:ring-cyan-500\/95:is(.dark *){--tw-ring-color:rgba(6,182,212,.95)}.dark\:ring-cyan-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(8 51 68/var(--tw-ring-opacity,1))}.dark\:ring-cyan-950\/0:is(.dark *){--tw-ring-color:rgba(8,51,68,0)}.dark\:ring-cyan-950\/10:is(.dark *){--tw-ring-color:rgba(8,51,68,.1)}.dark\:ring-cyan-950\/100:is(.dark *){--tw-ring-color:#083344}.dark\:ring-cyan-950\/15:is(.dark *){--tw-ring-color:rgba(8,51,68,.15)}.dark\:ring-cyan-950\/20:is(.dark *){--tw-ring-color:rgba(8,51,68,.2)}.dark\:ring-cyan-950\/25:is(.dark *){--tw-ring-color:rgba(8,51,68,.25)}.dark\:ring-cyan-950\/30:is(.dark *){--tw-ring-color:rgba(8,51,68,.3)}.dark\:ring-cyan-950\/35:is(.dark *){--tw-ring-color:rgba(8,51,68,.35)}.dark\:ring-cyan-950\/40:is(.dark *){--tw-ring-color:rgba(8,51,68,.4)}.dark\:ring-cyan-950\/45:is(.dark *){--tw-ring-color:rgba(8,51,68,.45)}.dark\:ring-cyan-950\/5:is(.dark *){--tw-ring-color:rgba(8,51,68,.05)}.dark\:ring-cyan-950\/50:is(.dark *){--tw-ring-color:rgba(8,51,68,.5)}.dark\:ring-cyan-950\/55:is(.dark *){--tw-ring-color:rgba(8,51,68,.55)}.dark\:ring-cyan-950\/60:is(.dark *){--tw-ring-color:rgba(8,51,68,.6)}.dark\:ring-cyan-950\/65:is(.dark *){--tw-ring-color:rgba(8,51,68,.65)}.dark\:ring-cyan-950\/70:is(.dark *){--tw-ring-color:rgba(8,51,68,.7)}.dark\:ring-cyan-950\/75:is(.dark *){--tw-ring-color:rgba(8,51,68,.75)}.dark\:ring-cyan-950\/80:is(.dark *){--tw-ring-color:rgba(8,51,68,.8)}.dark\:ring-cyan-950\/85:is(.dark *){--tw-ring-color:rgba(8,51,68,.85)}.dark\:ring-cyan-950\/90:is(.dark *){--tw-ring-color:rgba(8,51,68,.9)}.dark\:ring-cyan-950\/95:is(.dark *){--tw-ring-color:rgba(8,51,68,.95)}.dark\:ring-emerald-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(209 250 229/var(--tw-ring-opacity,1))}.dark\:ring-emerald-100\/0:is(.dark *){--tw-ring-color:rgba(209,250,229,0)}.dark\:ring-emerald-100\/10:is(.dark *){--tw-ring-color:rgba(209,250,229,.1)}.dark\:ring-emerald-100\/100:is(.dark *){--tw-ring-color:#d1fae5}.dark\:ring-emerald-100\/15:is(.dark *){--tw-ring-color:rgba(209,250,229,.15)}.dark\:ring-emerald-100\/20:is(.dark *){--tw-ring-color:rgba(209,250,229,.2)}.dark\:ring-emerald-100\/25:is(.dark *){--tw-ring-color:rgba(209,250,229,.25)}.dark\:ring-emerald-100\/30:is(.dark *){--tw-ring-color:rgba(209,250,229,.3)}.dark\:ring-emerald-100\/35:is(.dark *){--tw-ring-color:rgba(209,250,229,.35)}.dark\:ring-emerald-100\/40:is(.dark *){--tw-ring-color:rgba(209,250,229,.4)}.dark\:ring-emerald-100\/45:is(.dark *){--tw-ring-color:rgba(209,250,229,.45)}.dark\:ring-emerald-100\/5:is(.dark *){--tw-ring-color:rgba(209,250,229,.05)}.dark\:ring-emerald-100\/50:is(.dark *){--tw-ring-color:rgba(209,250,229,.5)}.dark\:ring-emerald-100\/55:is(.dark *){--tw-ring-color:rgba(209,250,229,.55)}.dark\:ring-emerald-100\/60:is(.dark *){--tw-ring-color:rgba(209,250,229,.6)}.dark\:ring-emerald-100\/65:is(.dark *){--tw-ring-color:rgba(209,250,229,.65)}.dark\:ring-emerald-100\/70:is(.dark *){--tw-ring-color:rgba(209,250,229,.7)}.dark\:ring-emerald-100\/75:is(.dark *){--tw-ring-color:rgba(209,250,229,.75)}.dark\:ring-emerald-100\/80:is(.dark *){--tw-ring-color:rgba(209,250,229,.8)}.dark\:ring-emerald-100\/85:is(.dark *){--tw-ring-color:rgba(209,250,229,.85)}.dark\:ring-emerald-100\/90:is(.dark *){--tw-ring-color:rgba(209,250,229,.9)}.dark\:ring-emerald-100\/95:is(.dark *){--tw-ring-color:rgba(209,250,229,.95)}.dark\:ring-emerald-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(167 243 208/var(--tw-ring-opacity,1))}.dark\:ring-emerald-200\/0:is(.dark *){--tw-ring-color:rgba(167,243,208,0)}.dark\:ring-emerald-200\/10:is(.dark *){--tw-ring-color:rgba(167,243,208,.1)}.dark\:ring-emerald-200\/100:is(.dark *){--tw-ring-color:#a7f3d0}.dark\:ring-emerald-200\/15:is(.dark *){--tw-ring-color:rgba(167,243,208,.15)}.dark\:ring-emerald-200\/20:is(.dark *){--tw-ring-color:rgba(167,243,208,.2)}.dark\:ring-emerald-200\/25:is(.dark *){--tw-ring-color:rgba(167,243,208,.25)}.dark\:ring-emerald-200\/30:is(.dark *){--tw-ring-color:rgba(167,243,208,.3)}.dark\:ring-emerald-200\/35:is(.dark *){--tw-ring-color:rgba(167,243,208,.35)}.dark\:ring-emerald-200\/40:is(.dark *){--tw-ring-color:rgba(167,243,208,.4)}.dark\:ring-emerald-200\/45:is(.dark *){--tw-ring-color:rgba(167,243,208,.45)}.dark\:ring-emerald-200\/5:is(.dark *){--tw-ring-color:rgba(167,243,208,.05)}.dark\:ring-emerald-200\/50:is(.dark *){--tw-ring-color:rgba(167,243,208,.5)}.dark\:ring-emerald-200\/55:is(.dark *){--tw-ring-color:rgba(167,243,208,.55)}.dark\:ring-emerald-200\/60:is(.dark *){--tw-ring-color:rgba(167,243,208,.6)}.dark\:ring-emerald-200\/65:is(.dark *){--tw-ring-color:rgba(167,243,208,.65)}.dark\:ring-emerald-200\/70:is(.dark *){--tw-ring-color:rgba(167,243,208,.7)}.dark\:ring-emerald-200\/75:is(.dark *){--tw-ring-color:rgba(167,243,208,.75)}.dark\:ring-emerald-200\/80:is(.dark *){--tw-ring-color:rgba(167,243,208,.8)}.dark\:ring-emerald-200\/85:is(.dark *){--tw-ring-color:rgba(167,243,208,.85)}.dark\:ring-emerald-200\/90:is(.dark *){--tw-ring-color:rgba(167,243,208,.9)}.dark\:ring-emerald-200\/95:is(.dark *){--tw-ring-color:rgba(167,243,208,.95)}.dark\:ring-emerald-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(52 211 153/var(--tw-ring-opacity,1))}.dark\:ring-emerald-400\/0:is(.dark *){--tw-ring-color:rgba(52,211,153,0)}.dark\:ring-emerald-400\/10:is(.dark *){--tw-ring-color:rgba(52,211,153,.1)}.dark\:ring-emerald-400\/100:is(.dark *){--tw-ring-color:#34d399}.dark\:ring-emerald-400\/15:is(.dark *){--tw-ring-color:rgba(52,211,153,.15)}.dark\:ring-emerald-400\/20:is(.dark *){--tw-ring-color:rgba(52,211,153,.2)}.dark\:ring-emerald-400\/25:is(.dark *){--tw-ring-color:rgba(52,211,153,.25)}.dark\:ring-emerald-400\/30:is(.dark *){--tw-ring-color:rgba(52,211,153,.3)}.dark\:ring-emerald-400\/35:is(.dark *){--tw-ring-color:rgba(52,211,153,.35)}.dark\:ring-emerald-400\/40:is(.dark *){--tw-ring-color:rgba(52,211,153,.4)}.dark\:ring-emerald-400\/45:is(.dark *){--tw-ring-color:rgba(52,211,153,.45)}.dark\:ring-emerald-400\/5:is(.dark *){--tw-ring-color:rgba(52,211,153,.05)}.dark\:ring-emerald-400\/50:is(.dark *){--tw-ring-color:rgba(52,211,153,.5)}.dark\:ring-emerald-400\/55:is(.dark *){--tw-ring-color:rgba(52,211,153,.55)}.dark\:ring-emerald-400\/60:is(.dark *){--tw-ring-color:rgba(52,211,153,.6)}.dark\:ring-emerald-400\/65:is(.dark *){--tw-ring-color:rgba(52,211,153,.65)}.dark\:ring-emerald-400\/70:is(.dark *){--tw-ring-color:rgba(52,211,153,.7)}.dark\:ring-emerald-400\/75:is(.dark *){--tw-ring-color:rgba(52,211,153,.75)}.dark\:ring-emerald-400\/80:is(.dark *){--tw-ring-color:rgba(52,211,153,.8)}.dark\:ring-emerald-400\/85:is(.dark *){--tw-ring-color:rgba(52,211,153,.85)}.dark\:ring-emerald-400\/90:is(.dark *){--tw-ring-color:rgba(52,211,153,.9)}.dark\:ring-emerald-400\/95:is(.dark *){--tw-ring-color:rgba(52,211,153,.95)}.dark\:ring-emerald-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(236 253 245/var(--tw-ring-opacity,1))}.dark\:ring-emerald-50\/0:is(.dark *){--tw-ring-color:rgba(236,253,245,0)}.dark\:ring-emerald-50\/10:is(.dark *){--tw-ring-color:rgba(236,253,245,.1)}.dark\:ring-emerald-50\/100:is(.dark *){--tw-ring-color:#ecfdf5}.dark\:ring-emerald-50\/15:is(.dark *){--tw-ring-color:rgba(236,253,245,.15)}.dark\:ring-emerald-50\/20:is(.dark *){--tw-ring-color:rgba(236,253,245,.2)}.dark\:ring-emerald-50\/25:is(.dark *){--tw-ring-color:rgba(236,253,245,.25)}.dark\:ring-emerald-50\/30:is(.dark *){--tw-ring-color:rgba(236,253,245,.3)}.dark\:ring-emerald-50\/35:is(.dark *){--tw-ring-color:rgba(236,253,245,.35)}.dark\:ring-emerald-50\/40:is(.dark *){--tw-ring-color:rgba(236,253,245,.4)}.dark\:ring-emerald-50\/45:is(.dark *){--tw-ring-color:rgba(236,253,245,.45)}.dark\:ring-emerald-50\/5:is(.dark *){--tw-ring-color:rgba(236,253,245,.05)}.dark\:ring-emerald-50\/50:is(.dark *){--tw-ring-color:rgba(236,253,245,.5)}.dark\:ring-emerald-50\/55:is(.dark *){--tw-ring-color:rgba(236,253,245,.55)}.dark\:ring-emerald-50\/60:is(.dark *){--tw-ring-color:rgba(236,253,245,.6)}.dark\:ring-emerald-50\/65:is(.dark *){--tw-ring-color:rgba(236,253,245,.65)}.dark\:ring-emerald-50\/70:is(.dark *){--tw-ring-color:rgba(236,253,245,.7)}.dark\:ring-emerald-50\/75:is(.dark *){--tw-ring-color:rgba(236,253,245,.75)}.dark\:ring-emerald-50\/80:is(.dark *){--tw-ring-color:rgba(236,253,245,.8)}.dark\:ring-emerald-50\/85:is(.dark *){--tw-ring-color:rgba(236,253,245,.85)}.dark\:ring-emerald-50\/90:is(.dark *){--tw-ring-color:rgba(236,253,245,.9)}.dark\:ring-emerald-50\/95:is(.dark *){--tw-ring-color:rgba(236,253,245,.95)}.dark\:ring-emerald-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(16 185 129/var(--tw-ring-opacity,1))}.dark\:ring-emerald-500\/0:is(.dark *){--tw-ring-color:rgba(16,185,129,0)}.dark\:ring-emerald-500\/10:is(.dark *){--tw-ring-color:rgba(16,185,129,.1)}.dark\:ring-emerald-500\/100:is(.dark *){--tw-ring-color:#10b981}.dark\:ring-emerald-500\/15:is(.dark *){--tw-ring-color:rgba(16,185,129,.15)}.dark\:ring-emerald-500\/20:is(.dark *){--tw-ring-color:rgba(16,185,129,.2)}.dark\:ring-emerald-500\/25:is(.dark *){--tw-ring-color:rgba(16,185,129,.25)}.dark\:ring-emerald-500\/30:is(.dark *){--tw-ring-color:rgba(16,185,129,.3)}.dark\:ring-emerald-500\/35:is(.dark *){--tw-ring-color:rgba(16,185,129,.35)}.dark\:ring-emerald-500\/40:is(.dark *){--tw-ring-color:rgba(16,185,129,.4)}.dark\:ring-emerald-500\/45:is(.dark *){--tw-ring-color:rgba(16,185,129,.45)}.dark\:ring-emerald-500\/5:is(.dark *){--tw-ring-color:rgba(16,185,129,.05)}.dark\:ring-emerald-500\/50:is(.dark *){--tw-ring-color:rgba(16,185,129,.5)}.dark\:ring-emerald-500\/55:is(.dark *){--tw-ring-color:rgba(16,185,129,.55)}.dark\:ring-emerald-500\/60:is(.dark *){--tw-ring-color:rgba(16,185,129,.6)}.dark\:ring-emerald-500\/65:is(.dark *){--tw-ring-color:rgba(16,185,129,.65)}.dark\:ring-emerald-500\/70:is(.dark *){--tw-ring-color:rgba(16,185,129,.7)}.dark\:ring-emerald-500\/75:is(.dark *){--tw-ring-color:rgba(16,185,129,.75)}.dark\:ring-emerald-500\/80:is(.dark *){--tw-ring-color:rgba(16,185,129,.8)}.dark\:ring-emerald-500\/85:is(.dark *){--tw-ring-color:rgba(16,185,129,.85)}.dark\:ring-emerald-500\/90:is(.dark *){--tw-ring-color:rgba(16,185,129,.9)}.dark\:ring-emerald-500\/95:is(.dark *){--tw-ring-color:rgba(16,185,129,.95)}.dark\:ring-emerald-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(2 44 34/var(--tw-ring-opacity,1))}.dark\:ring-emerald-950\/0:is(.dark *){--tw-ring-color:rgba(2,44,34,0)}.dark\:ring-emerald-950\/10:is(.dark *){--tw-ring-color:rgba(2,44,34,.1)}.dark\:ring-emerald-950\/100:is(.dark *){--tw-ring-color:#022c22}.dark\:ring-emerald-950\/15:is(.dark *){--tw-ring-color:rgba(2,44,34,.15)}.dark\:ring-emerald-950\/20:is(.dark *){--tw-ring-color:rgba(2,44,34,.2)}.dark\:ring-emerald-950\/25:is(.dark *){--tw-ring-color:rgba(2,44,34,.25)}.dark\:ring-emerald-950\/30:is(.dark *){--tw-ring-color:rgba(2,44,34,.3)}.dark\:ring-emerald-950\/35:is(.dark *){--tw-ring-color:rgba(2,44,34,.35)}.dark\:ring-emerald-950\/40:is(.dark *){--tw-ring-color:rgba(2,44,34,.4)}.dark\:ring-emerald-950\/45:is(.dark *){--tw-ring-color:rgba(2,44,34,.45)}.dark\:ring-emerald-950\/5:is(.dark *){--tw-ring-color:rgba(2,44,34,.05)}.dark\:ring-emerald-950\/50:is(.dark *){--tw-ring-color:rgba(2,44,34,.5)}.dark\:ring-emerald-950\/55:is(.dark *){--tw-ring-color:rgba(2,44,34,.55)}.dark\:ring-emerald-950\/60:is(.dark *){--tw-ring-color:rgba(2,44,34,.6)}.dark\:ring-emerald-950\/65:is(.dark *){--tw-ring-color:rgba(2,44,34,.65)}.dark\:ring-emerald-950\/70:is(.dark *){--tw-ring-color:rgba(2,44,34,.7)}.dark\:ring-emerald-950\/75:is(.dark *){--tw-ring-color:rgba(2,44,34,.75)}.dark\:ring-emerald-950\/80:is(.dark *){--tw-ring-color:rgba(2,44,34,.8)}.dark\:ring-emerald-950\/85:is(.dark *){--tw-ring-color:rgba(2,44,34,.85)}.dark\:ring-emerald-950\/90:is(.dark *){--tw-ring-color:rgba(2,44,34,.9)}.dark\:ring-emerald-950\/95:is(.dark *){--tw-ring-color:rgba(2,44,34,.95)}.dark\:ring-fuchsia-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(250 232 255/var(--tw-ring-opacity,1))}.dark\:ring-fuchsia-100\/0:is(.dark *){--tw-ring-color:rgba(250,232,255,0)}.dark\:ring-fuchsia-100\/10:is(.dark *){--tw-ring-color:rgba(250,232,255,.1)}.dark\:ring-fuchsia-100\/100:is(.dark *){--tw-ring-color:#fae8ff}.dark\:ring-fuchsia-100\/15:is(.dark *){--tw-ring-color:rgba(250,232,255,.15)}.dark\:ring-fuchsia-100\/20:is(.dark *){--tw-ring-color:rgba(250,232,255,.2)}.dark\:ring-fuchsia-100\/25:is(.dark *){--tw-ring-color:rgba(250,232,255,.25)}.dark\:ring-fuchsia-100\/30:is(.dark *){--tw-ring-color:rgba(250,232,255,.3)}.dark\:ring-fuchsia-100\/35:is(.dark *){--tw-ring-color:rgba(250,232,255,.35)}.dark\:ring-fuchsia-100\/40:is(.dark *){--tw-ring-color:rgba(250,232,255,.4)}.dark\:ring-fuchsia-100\/45:is(.dark *){--tw-ring-color:rgba(250,232,255,.45)}.dark\:ring-fuchsia-100\/5:is(.dark *){--tw-ring-color:rgba(250,232,255,.05)}.dark\:ring-fuchsia-100\/50:is(.dark *){--tw-ring-color:rgba(250,232,255,.5)}.dark\:ring-fuchsia-100\/55:is(.dark *){--tw-ring-color:rgba(250,232,255,.55)}.dark\:ring-fuchsia-100\/60:is(.dark *){--tw-ring-color:rgba(250,232,255,.6)}.dark\:ring-fuchsia-100\/65:is(.dark *){--tw-ring-color:rgba(250,232,255,.65)}.dark\:ring-fuchsia-100\/70:is(.dark *){--tw-ring-color:rgba(250,232,255,.7)}.dark\:ring-fuchsia-100\/75:is(.dark *){--tw-ring-color:rgba(250,232,255,.75)}.dark\:ring-fuchsia-100\/80:is(.dark *){--tw-ring-color:rgba(250,232,255,.8)}.dark\:ring-fuchsia-100\/85:is(.dark *){--tw-ring-color:rgba(250,232,255,.85)}.dark\:ring-fuchsia-100\/90:is(.dark *){--tw-ring-color:rgba(250,232,255,.9)}.dark\:ring-fuchsia-100\/95:is(.dark *){--tw-ring-color:rgba(250,232,255,.95)}.dark\:ring-fuchsia-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(245 208 254/var(--tw-ring-opacity,1))}.dark\:ring-fuchsia-200\/0:is(.dark *){--tw-ring-color:rgba(245,208,254,0)}.dark\:ring-fuchsia-200\/10:is(.dark *){--tw-ring-color:rgba(245,208,254,.1)}.dark\:ring-fuchsia-200\/100:is(.dark *){--tw-ring-color:#f5d0fe}.dark\:ring-fuchsia-200\/15:is(.dark *){--tw-ring-color:rgba(245,208,254,.15)}.dark\:ring-fuchsia-200\/20:is(.dark *){--tw-ring-color:rgba(245,208,254,.2)}.dark\:ring-fuchsia-200\/25:is(.dark *){--tw-ring-color:rgba(245,208,254,.25)}.dark\:ring-fuchsia-200\/30:is(.dark *){--tw-ring-color:rgba(245,208,254,.3)}.dark\:ring-fuchsia-200\/35:is(.dark *){--tw-ring-color:rgba(245,208,254,.35)}.dark\:ring-fuchsia-200\/40:is(.dark *){--tw-ring-color:rgba(245,208,254,.4)}.dark\:ring-fuchsia-200\/45:is(.dark *){--tw-ring-color:rgba(245,208,254,.45)}.dark\:ring-fuchsia-200\/5:is(.dark *){--tw-ring-color:rgba(245,208,254,.05)}.dark\:ring-fuchsia-200\/50:is(.dark *){--tw-ring-color:rgba(245,208,254,.5)}.dark\:ring-fuchsia-200\/55:is(.dark *){--tw-ring-color:rgba(245,208,254,.55)}.dark\:ring-fuchsia-200\/60:is(.dark *){--tw-ring-color:rgba(245,208,254,.6)}.dark\:ring-fuchsia-200\/65:is(.dark *){--tw-ring-color:rgba(245,208,254,.65)}.dark\:ring-fuchsia-200\/70:is(.dark *){--tw-ring-color:rgba(245,208,254,.7)}.dark\:ring-fuchsia-200\/75:is(.dark *){--tw-ring-color:rgba(245,208,254,.75)}.dark\:ring-fuchsia-200\/80:is(.dark *){--tw-ring-color:rgba(245,208,254,.8)}.dark\:ring-fuchsia-200\/85:is(.dark *){--tw-ring-color:rgba(245,208,254,.85)}.dark\:ring-fuchsia-200\/90:is(.dark *){--tw-ring-color:rgba(245,208,254,.9)}.dark\:ring-fuchsia-200\/95:is(.dark *){--tw-ring-color:rgba(245,208,254,.95)}.dark\:ring-fuchsia-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(232 121 249/var(--tw-ring-opacity,1))}.dark\:ring-fuchsia-400\/0:is(.dark *){--tw-ring-color:rgba(232,121,249,0)}.dark\:ring-fuchsia-400\/10:is(.dark *){--tw-ring-color:rgba(232,121,249,.1)}.dark\:ring-fuchsia-400\/100:is(.dark *){--tw-ring-color:#e879f9}.dark\:ring-fuchsia-400\/15:is(.dark *){--tw-ring-color:rgba(232,121,249,.15)}.dark\:ring-fuchsia-400\/20:is(.dark *){--tw-ring-color:rgba(232,121,249,.2)}.dark\:ring-fuchsia-400\/25:is(.dark *){--tw-ring-color:rgba(232,121,249,.25)}.dark\:ring-fuchsia-400\/30:is(.dark *){--tw-ring-color:rgba(232,121,249,.3)}.dark\:ring-fuchsia-400\/35:is(.dark *){--tw-ring-color:rgba(232,121,249,.35)}.dark\:ring-fuchsia-400\/40:is(.dark *){--tw-ring-color:rgba(232,121,249,.4)}.dark\:ring-fuchsia-400\/45:is(.dark *){--tw-ring-color:rgba(232,121,249,.45)}.dark\:ring-fuchsia-400\/5:is(.dark *){--tw-ring-color:rgba(232,121,249,.05)}.dark\:ring-fuchsia-400\/50:is(.dark *){--tw-ring-color:rgba(232,121,249,.5)}.dark\:ring-fuchsia-400\/55:is(.dark *){--tw-ring-color:rgba(232,121,249,.55)}.dark\:ring-fuchsia-400\/60:is(.dark *){--tw-ring-color:rgba(232,121,249,.6)}.dark\:ring-fuchsia-400\/65:is(.dark *){--tw-ring-color:rgba(232,121,249,.65)}.dark\:ring-fuchsia-400\/70:is(.dark *){--tw-ring-color:rgba(232,121,249,.7)}.dark\:ring-fuchsia-400\/75:is(.dark *){--tw-ring-color:rgba(232,121,249,.75)}.dark\:ring-fuchsia-400\/80:is(.dark *){--tw-ring-color:rgba(232,121,249,.8)}.dark\:ring-fuchsia-400\/85:is(.dark *){--tw-ring-color:rgba(232,121,249,.85)}.dark\:ring-fuchsia-400\/90:is(.dark *){--tw-ring-color:rgba(232,121,249,.9)}.dark\:ring-fuchsia-400\/95:is(.dark *){--tw-ring-color:rgba(232,121,249,.95)}.dark\:ring-fuchsia-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(253 244 255/var(--tw-ring-opacity,1))}.dark\:ring-fuchsia-50\/0:is(.dark *){--tw-ring-color:rgba(253,244,255,0)}.dark\:ring-fuchsia-50\/10:is(.dark *){--tw-ring-color:rgba(253,244,255,.1)}.dark\:ring-fuchsia-50\/100:is(.dark *){--tw-ring-color:#fdf4ff}.dark\:ring-fuchsia-50\/15:is(.dark *){--tw-ring-color:rgba(253,244,255,.15)}.dark\:ring-fuchsia-50\/20:is(.dark *){--tw-ring-color:rgba(253,244,255,.2)}.dark\:ring-fuchsia-50\/25:is(.dark *){--tw-ring-color:rgba(253,244,255,.25)}.dark\:ring-fuchsia-50\/30:is(.dark *){--tw-ring-color:rgba(253,244,255,.3)}.dark\:ring-fuchsia-50\/35:is(.dark *){--tw-ring-color:rgba(253,244,255,.35)}.dark\:ring-fuchsia-50\/40:is(.dark *){--tw-ring-color:rgba(253,244,255,.4)}.dark\:ring-fuchsia-50\/45:is(.dark *){--tw-ring-color:rgba(253,244,255,.45)}.dark\:ring-fuchsia-50\/5:is(.dark *){--tw-ring-color:rgba(253,244,255,.05)}.dark\:ring-fuchsia-50\/50:is(.dark *){--tw-ring-color:rgba(253,244,255,.5)}.dark\:ring-fuchsia-50\/55:is(.dark *){--tw-ring-color:rgba(253,244,255,.55)}.dark\:ring-fuchsia-50\/60:is(.dark *){--tw-ring-color:rgba(253,244,255,.6)}.dark\:ring-fuchsia-50\/65:is(.dark *){--tw-ring-color:rgba(253,244,255,.65)}.dark\:ring-fuchsia-50\/70:is(.dark *){--tw-ring-color:rgba(253,244,255,.7)}.dark\:ring-fuchsia-50\/75:is(.dark *){--tw-ring-color:rgba(253,244,255,.75)}.dark\:ring-fuchsia-50\/80:is(.dark *){--tw-ring-color:rgba(253,244,255,.8)}.dark\:ring-fuchsia-50\/85:is(.dark *){--tw-ring-color:rgba(253,244,255,.85)}.dark\:ring-fuchsia-50\/90:is(.dark *){--tw-ring-color:rgba(253,244,255,.9)}.dark\:ring-fuchsia-50\/95:is(.dark *){--tw-ring-color:rgba(253,244,255,.95)}.dark\:ring-fuchsia-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(217 70 239/var(--tw-ring-opacity,1))}.dark\:ring-fuchsia-500\/0:is(.dark *){--tw-ring-color:rgba(217,70,239,0)}.dark\:ring-fuchsia-500\/10:is(.dark *){--tw-ring-color:rgba(217,70,239,.1)}.dark\:ring-fuchsia-500\/100:is(.dark *){--tw-ring-color:#d946ef}.dark\:ring-fuchsia-500\/15:is(.dark *){--tw-ring-color:rgba(217,70,239,.15)}.dark\:ring-fuchsia-500\/20:is(.dark *){--tw-ring-color:rgba(217,70,239,.2)}.dark\:ring-fuchsia-500\/25:is(.dark *){--tw-ring-color:rgba(217,70,239,.25)}.dark\:ring-fuchsia-500\/30:is(.dark *){--tw-ring-color:rgba(217,70,239,.3)}.dark\:ring-fuchsia-500\/35:is(.dark *){--tw-ring-color:rgba(217,70,239,.35)}.dark\:ring-fuchsia-500\/40:is(.dark *){--tw-ring-color:rgba(217,70,239,.4)}.dark\:ring-fuchsia-500\/45:is(.dark *){--tw-ring-color:rgba(217,70,239,.45)}.dark\:ring-fuchsia-500\/5:is(.dark *){--tw-ring-color:rgba(217,70,239,.05)}.dark\:ring-fuchsia-500\/50:is(.dark *){--tw-ring-color:rgba(217,70,239,.5)}.dark\:ring-fuchsia-500\/55:is(.dark *){--tw-ring-color:rgba(217,70,239,.55)}.dark\:ring-fuchsia-500\/60:is(.dark *){--tw-ring-color:rgba(217,70,239,.6)}.dark\:ring-fuchsia-500\/65:is(.dark *){--tw-ring-color:rgba(217,70,239,.65)}.dark\:ring-fuchsia-500\/70:is(.dark *){--tw-ring-color:rgba(217,70,239,.7)}.dark\:ring-fuchsia-500\/75:is(.dark *){--tw-ring-color:rgba(217,70,239,.75)}.dark\:ring-fuchsia-500\/80:is(.dark *){--tw-ring-color:rgba(217,70,239,.8)}.dark\:ring-fuchsia-500\/85:is(.dark *){--tw-ring-color:rgba(217,70,239,.85)}.dark\:ring-fuchsia-500\/90:is(.dark *){--tw-ring-color:rgba(217,70,239,.9)}.dark\:ring-fuchsia-500\/95:is(.dark *){--tw-ring-color:rgba(217,70,239,.95)}.dark\:ring-fuchsia-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(74 4 78/var(--tw-ring-opacity,1))}.dark\:ring-fuchsia-950\/0:is(.dark *){--tw-ring-color:rgba(74,4,78,0)}.dark\:ring-fuchsia-950\/10:is(.dark *){--tw-ring-color:rgba(74,4,78,.1)}.dark\:ring-fuchsia-950\/100:is(.dark *){--tw-ring-color:#4a044e}.dark\:ring-fuchsia-950\/15:is(.dark *){--tw-ring-color:rgba(74,4,78,.15)}.dark\:ring-fuchsia-950\/20:is(.dark *){--tw-ring-color:rgba(74,4,78,.2)}.dark\:ring-fuchsia-950\/25:is(.dark *){--tw-ring-color:rgba(74,4,78,.25)}.dark\:ring-fuchsia-950\/30:is(.dark *){--tw-ring-color:rgba(74,4,78,.3)}.dark\:ring-fuchsia-950\/35:is(.dark *){--tw-ring-color:rgba(74,4,78,.35)}.dark\:ring-fuchsia-950\/40:is(.dark *){--tw-ring-color:rgba(74,4,78,.4)}.dark\:ring-fuchsia-950\/45:is(.dark *){--tw-ring-color:rgba(74,4,78,.45)}.dark\:ring-fuchsia-950\/5:is(.dark *){--tw-ring-color:rgba(74,4,78,.05)}.dark\:ring-fuchsia-950\/50:is(.dark *){--tw-ring-color:rgba(74,4,78,.5)}.dark\:ring-fuchsia-950\/55:is(.dark *){--tw-ring-color:rgba(74,4,78,.55)}.dark\:ring-fuchsia-950\/60:is(.dark *){--tw-ring-color:rgba(74,4,78,.6)}.dark\:ring-fuchsia-950\/65:is(.dark *){--tw-ring-color:rgba(74,4,78,.65)}.dark\:ring-fuchsia-950\/70:is(.dark *){--tw-ring-color:rgba(74,4,78,.7)}.dark\:ring-fuchsia-950\/75:is(.dark *){--tw-ring-color:rgba(74,4,78,.75)}.dark\:ring-fuchsia-950\/80:is(.dark *){--tw-ring-color:rgba(74,4,78,.8)}.dark\:ring-fuchsia-950\/85:is(.dark *){--tw-ring-color:rgba(74,4,78,.85)}.dark\:ring-fuchsia-950\/90:is(.dark *){--tw-ring-color:rgba(74,4,78,.9)}.dark\:ring-fuchsia-950\/95:is(.dark *){--tw-ring-color:rgba(74,4,78,.95)}.dark\:ring-gray-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(243 244 246/var(--tw-ring-opacity,1))}.dark\:ring-gray-100\/0:is(.dark *){--tw-ring-color:rgba(243,244,246,0)}.dark\:ring-gray-100\/10:is(.dark *){--tw-ring-color:rgba(243,244,246,.1)}.dark\:ring-gray-100\/100:is(.dark *){--tw-ring-color:#f3f4f6}.dark\:ring-gray-100\/15:is(.dark *){--tw-ring-color:rgba(243,244,246,.15)}.dark\:ring-gray-100\/20:is(.dark *){--tw-ring-color:rgba(243,244,246,.2)}.dark\:ring-gray-100\/25:is(.dark *){--tw-ring-color:rgba(243,244,246,.25)}.dark\:ring-gray-100\/30:is(.dark *){--tw-ring-color:rgba(243,244,246,.3)}.dark\:ring-gray-100\/35:is(.dark *){--tw-ring-color:rgba(243,244,246,.35)}.dark\:ring-gray-100\/40:is(.dark *){--tw-ring-color:rgba(243,244,246,.4)}.dark\:ring-gray-100\/45:is(.dark *){--tw-ring-color:rgba(243,244,246,.45)}.dark\:ring-gray-100\/5:is(.dark *){--tw-ring-color:rgba(243,244,246,.05)}.dark\:ring-gray-100\/50:is(.dark *){--tw-ring-color:rgba(243,244,246,.5)}.dark\:ring-gray-100\/55:is(.dark *){--tw-ring-color:rgba(243,244,246,.55)}.dark\:ring-gray-100\/60:is(.dark *){--tw-ring-color:rgba(243,244,246,.6)}.dark\:ring-gray-100\/65:is(.dark *){--tw-ring-color:rgba(243,244,246,.65)}.dark\:ring-gray-100\/70:is(.dark *){--tw-ring-color:rgba(243,244,246,.7)}.dark\:ring-gray-100\/75:is(.dark *){--tw-ring-color:rgba(243,244,246,.75)}.dark\:ring-gray-100\/80:is(.dark *){--tw-ring-color:rgba(243,244,246,.8)}.dark\:ring-gray-100\/85:is(.dark *){--tw-ring-color:rgba(243,244,246,.85)}.dark\:ring-gray-100\/90:is(.dark *){--tw-ring-color:rgba(243,244,246,.9)}.dark\:ring-gray-100\/95:is(.dark *){--tw-ring-color:rgba(243,244,246,.95)}.dark\:ring-gray-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(229 231 235/var(--tw-ring-opacity,1))}.dark\:ring-gray-200\/0:is(.dark *){--tw-ring-color:rgba(229,231,235,0)}.dark\:ring-gray-200\/10:is(.dark *){--tw-ring-color:rgba(229,231,235,.1)}.dark\:ring-gray-200\/100:is(.dark *){--tw-ring-color:#e5e7eb}.dark\:ring-gray-200\/15:is(.dark *){--tw-ring-color:rgba(229,231,235,.15)}.dark\:ring-gray-200\/20:is(.dark *){--tw-ring-color:rgba(229,231,235,.2)}.dark\:ring-gray-200\/25:is(.dark *){--tw-ring-color:rgba(229,231,235,.25)}.dark\:ring-gray-200\/30:is(.dark *){--tw-ring-color:rgba(229,231,235,.3)}.dark\:ring-gray-200\/35:is(.dark *){--tw-ring-color:rgba(229,231,235,.35)}.dark\:ring-gray-200\/40:is(.dark *){--tw-ring-color:rgba(229,231,235,.4)}.dark\:ring-gray-200\/45:is(.dark *){--tw-ring-color:rgba(229,231,235,.45)}.dark\:ring-gray-200\/5:is(.dark *){--tw-ring-color:rgba(229,231,235,.05)}.dark\:ring-gray-200\/50:is(.dark *){--tw-ring-color:rgba(229,231,235,.5)}.dark\:ring-gray-200\/55:is(.dark *){--tw-ring-color:rgba(229,231,235,.55)}.dark\:ring-gray-200\/60:is(.dark *){--tw-ring-color:rgba(229,231,235,.6)}.dark\:ring-gray-200\/65:is(.dark *){--tw-ring-color:rgba(229,231,235,.65)}.dark\:ring-gray-200\/70:is(.dark *){--tw-ring-color:rgba(229,231,235,.7)}.dark\:ring-gray-200\/75:is(.dark *){--tw-ring-color:rgba(229,231,235,.75)}.dark\:ring-gray-200\/80:is(.dark *){--tw-ring-color:rgba(229,231,235,.8)}.dark\:ring-gray-200\/85:is(.dark *){--tw-ring-color:rgba(229,231,235,.85)}.dark\:ring-gray-200\/90:is(.dark *){--tw-ring-color:rgba(229,231,235,.9)}.dark\:ring-gray-200\/95:is(.dark *){--tw-ring-color:rgba(229,231,235,.95)}.dark\:ring-gray-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(156 163 175/var(--tw-ring-opacity,1))}.dark\:ring-gray-400\/0:is(.dark *){--tw-ring-color:rgba(156,163,175,0)}.dark\:ring-gray-400\/10:is(.dark *){--tw-ring-color:rgba(156,163,175,.1)}.dark\:ring-gray-400\/100:is(.dark *){--tw-ring-color:#9ca3af}.dark\:ring-gray-400\/15:is(.dark *){--tw-ring-color:rgba(156,163,175,.15)}.dark\:ring-gray-400\/20:is(.dark *){--tw-ring-color:rgba(156,163,175,.2)}.dark\:ring-gray-400\/25:is(.dark *){--tw-ring-color:rgba(156,163,175,.25)}.dark\:ring-gray-400\/30:is(.dark *){--tw-ring-color:rgba(156,163,175,.3)}.dark\:ring-gray-400\/35:is(.dark *){--tw-ring-color:rgba(156,163,175,.35)}.dark\:ring-gray-400\/40:is(.dark *){--tw-ring-color:rgba(156,163,175,.4)}.dark\:ring-gray-400\/45:is(.dark *){--tw-ring-color:rgba(156,163,175,.45)}.dark\:ring-gray-400\/5:is(.dark *){--tw-ring-color:rgba(156,163,175,.05)}.dark\:ring-gray-400\/50:is(.dark *){--tw-ring-color:rgba(156,163,175,.5)}.dark\:ring-gray-400\/55:is(.dark *){--tw-ring-color:rgba(156,163,175,.55)}.dark\:ring-gray-400\/60:is(.dark *){--tw-ring-color:rgba(156,163,175,.6)}.dark\:ring-gray-400\/65:is(.dark *){--tw-ring-color:rgba(156,163,175,.65)}.dark\:ring-gray-400\/70:is(.dark *){--tw-ring-color:rgba(156,163,175,.7)}.dark\:ring-gray-400\/75:is(.dark *){--tw-ring-color:rgba(156,163,175,.75)}.dark\:ring-gray-400\/80:is(.dark *){--tw-ring-color:rgba(156,163,175,.8)}.dark\:ring-gray-400\/85:is(.dark *){--tw-ring-color:rgba(156,163,175,.85)}.dark\:ring-gray-400\/90:is(.dark *){--tw-ring-color:rgba(156,163,175,.9)}.dark\:ring-gray-400\/95:is(.dark *){--tw-ring-color:rgba(156,163,175,.95)}.dark\:ring-gray-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(249 250 251/var(--tw-ring-opacity,1))}.dark\:ring-gray-50\/0:is(.dark *){--tw-ring-color:rgba(249,250,251,0)}.dark\:ring-gray-50\/10:is(.dark *){--tw-ring-color:rgba(249,250,251,.1)}.dark\:ring-gray-50\/100:is(.dark *){--tw-ring-color:#f9fafb}.dark\:ring-gray-50\/15:is(.dark *){--tw-ring-color:rgba(249,250,251,.15)}.dark\:ring-gray-50\/20:is(.dark *){--tw-ring-color:rgba(249,250,251,.2)}.dark\:ring-gray-50\/25:is(.dark *){--tw-ring-color:rgba(249,250,251,.25)}.dark\:ring-gray-50\/30:is(.dark *){--tw-ring-color:rgba(249,250,251,.3)}.dark\:ring-gray-50\/35:is(.dark *){--tw-ring-color:rgba(249,250,251,.35)}.dark\:ring-gray-50\/40:is(.dark *){--tw-ring-color:rgba(249,250,251,.4)}.dark\:ring-gray-50\/45:is(.dark *){--tw-ring-color:rgba(249,250,251,.45)}.dark\:ring-gray-50\/5:is(.dark *){--tw-ring-color:rgba(249,250,251,.05)}.dark\:ring-gray-50\/50:is(.dark *){--tw-ring-color:rgba(249,250,251,.5)}.dark\:ring-gray-50\/55:is(.dark *){--tw-ring-color:rgba(249,250,251,.55)}.dark\:ring-gray-50\/60:is(.dark *){--tw-ring-color:rgba(249,250,251,.6)}.dark\:ring-gray-50\/65:is(.dark *){--tw-ring-color:rgba(249,250,251,.65)}.dark\:ring-gray-50\/70:is(.dark *){--tw-ring-color:rgba(249,250,251,.7)}.dark\:ring-gray-50\/75:is(.dark *){--tw-ring-color:rgba(249,250,251,.75)}.dark\:ring-gray-50\/80:is(.dark *){--tw-ring-color:rgba(249,250,251,.8)}.dark\:ring-gray-50\/85:is(.dark *){--tw-ring-color:rgba(249,250,251,.85)}.dark\:ring-gray-50\/90:is(.dark *){--tw-ring-color:rgba(249,250,251,.9)}.dark\:ring-gray-50\/95:is(.dark *){--tw-ring-color:rgba(249,250,251,.95)}.dark\:ring-gray-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(107 114 128/var(--tw-ring-opacity,1))}.dark\:ring-gray-500\/0:is(.dark *){--tw-ring-color:hsla(220,9%,46%,0)}.dark\:ring-gray-500\/10:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.1)}.dark\:ring-gray-500\/100:is(.dark *){--tw-ring-color:#6b7280}.dark\:ring-gray-500\/15:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.15)}.dark\:ring-gray-500\/20:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.2)}.dark\:ring-gray-500\/25:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.25)}.dark\:ring-gray-500\/30:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.3)}.dark\:ring-gray-500\/35:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.35)}.dark\:ring-gray-500\/40:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.4)}.dark\:ring-gray-500\/45:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.45)}.dark\:ring-gray-500\/5:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.05)}.dark\:ring-gray-500\/50:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.5)}.dark\:ring-gray-500\/55:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.55)}.dark\:ring-gray-500\/60:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.6)}.dark\:ring-gray-500\/65:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.65)}.dark\:ring-gray-500\/70:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.7)}.dark\:ring-gray-500\/75:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.75)}.dark\:ring-gray-500\/80:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.8)}.dark\:ring-gray-500\/85:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.85)}.dark\:ring-gray-500\/90:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.9)}.dark\:ring-gray-500\/95:is(.dark *){--tw-ring-color:hsla(220,9%,46%,.95)}.dark\:ring-gray-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(3 7 18/var(--tw-ring-opacity,1))}.dark\:ring-gray-950\/0:is(.dark *){--tw-ring-color:rgba(3,7,18,0)}.dark\:ring-gray-950\/10:is(.dark *){--tw-ring-color:rgba(3,7,18,.1)}.dark\:ring-gray-950\/100:is(.dark *){--tw-ring-color:#030712}.dark\:ring-gray-950\/15:is(.dark *){--tw-ring-color:rgba(3,7,18,.15)}.dark\:ring-gray-950\/20:is(.dark *){--tw-ring-color:rgba(3,7,18,.2)}.dark\:ring-gray-950\/25:is(.dark *){--tw-ring-color:rgba(3,7,18,.25)}.dark\:ring-gray-950\/30:is(.dark *){--tw-ring-color:rgba(3,7,18,.3)}.dark\:ring-gray-950\/35:is(.dark *){--tw-ring-color:rgba(3,7,18,.35)}.dark\:ring-gray-950\/40:is(.dark *){--tw-ring-color:rgba(3,7,18,.4)}.dark\:ring-gray-950\/45:is(.dark *){--tw-ring-color:rgba(3,7,18,.45)}.dark\:ring-gray-950\/5:is(.dark *){--tw-ring-color:rgba(3,7,18,.05)}.dark\:ring-gray-950\/50:is(.dark *){--tw-ring-color:rgba(3,7,18,.5)}.dark\:ring-gray-950\/55:is(.dark *){--tw-ring-color:rgba(3,7,18,.55)}.dark\:ring-gray-950\/60:is(.dark *){--tw-ring-color:rgba(3,7,18,.6)}.dark\:ring-gray-950\/65:is(.dark *){--tw-ring-color:rgba(3,7,18,.65)}.dark\:ring-gray-950\/70:is(.dark *){--tw-ring-color:rgba(3,7,18,.7)}.dark\:ring-gray-950\/75:is(.dark *){--tw-ring-color:rgba(3,7,18,.75)}.dark\:ring-gray-950\/80:is(.dark *){--tw-ring-color:rgba(3,7,18,.8)}.dark\:ring-gray-950\/85:is(.dark *){--tw-ring-color:rgba(3,7,18,.85)}.dark\:ring-gray-950\/90:is(.dark *){--tw-ring-color:rgba(3,7,18,.9)}.dark\:ring-gray-950\/95:is(.dark *){--tw-ring-color:rgba(3,7,18,.95)}.dark\:ring-green-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(220 252 231/var(--tw-ring-opacity,1))}.dark\:ring-green-100\/0:is(.dark *){--tw-ring-color:rgba(220,252,231,0)}.dark\:ring-green-100\/10:is(.dark *){--tw-ring-color:rgba(220,252,231,.1)}.dark\:ring-green-100\/100:is(.dark *){--tw-ring-color:#dcfce7}.dark\:ring-green-100\/15:is(.dark *){--tw-ring-color:rgba(220,252,231,.15)}.dark\:ring-green-100\/20:is(.dark *){--tw-ring-color:rgba(220,252,231,.2)}.dark\:ring-green-100\/25:is(.dark *){--tw-ring-color:rgba(220,252,231,.25)}.dark\:ring-green-100\/30:is(.dark *){--tw-ring-color:rgba(220,252,231,.3)}.dark\:ring-green-100\/35:is(.dark *){--tw-ring-color:rgba(220,252,231,.35)}.dark\:ring-green-100\/40:is(.dark *){--tw-ring-color:rgba(220,252,231,.4)}.dark\:ring-green-100\/45:is(.dark *){--tw-ring-color:rgba(220,252,231,.45)}.dark\:ring-green-100\/5:is(.dark *){--tw-ring-color:rgba(220,252,231,.05)}.dark\:ring-green-100\/50:is(.dark *){--tw-ring-color:rgba(220,252,231,.5)}.dark\:ring-green-100\/55:is(.dark *){--tw-ring-color:rgba(220,252,231,.55)}.dark\:ring-green-100\/60:is(.dark *){--tw-ring-color:rgba(220,252,231,.6)}.dark\:ring-green-100\/65:is(.dark *){--tw-ring-color:rgba(220,252,231,.65)}.dark\:ring-green-100\/70:is(.dark *){--tw-ring-color:rgba(220,252,231,.7)}.dark\:ring-green-100\/75:is(.dark *){--tw-ring-color:rgba(220,252,231,.75)}.dark\:ring-green-100\/80:is(.dark *){--tw-ring-color:rgba(220,252,231,.8)}.dark\:ring-green-100\/85:is(.dark *){--tw-ring-color:rgba(220,252,231,.85)}.dark\:ring-green-100\/90:is(.dark *){--tw-ring-color:rgba(220,252,231,.9)}.dark\:ring-green-100\/95:is(.dark *){--tw-ring-color:rgba(220,252,231,.95)}.dark\:ring-green-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(187 247 208/var(--tw-ring-opacity,1))}.dark\:ring-green-200\/0:is(.dark *){--tw-ring-color:rgba(187,247,208,0)}.dark\:ring-green-200\/10:is(.dark *){--tw-ring-color:rgba(187,247,208,.1)}.dark\:ring-green-200\/100:is(.dark *){--tw-ring-color:#bbf7d0}.dark\:ring-green-200\/15:is(.dark *){--tw-ring-color:rgba(187,247,208,.15)}.dark\:ring-green-200\/20:is(.dark *){--tw-ring-color:rgba(187,247,208,.2)}.dark\:ring-green-200\/25:is(.dark *){--tw-ring-color:rgba(187,247,208,.25)}.dark\:ring-green-200\/30:is(.dark *){--tw-ring-color:rgba(187,247,208,.3)}.dark\:ring-green-200\/35:is(.dark *){--tw-ring-color:rgba(187,247,208,.35)}.dark\:ring-green-200\/40:is(.dark *){--tw-ring-color:rgba(187,247,208,.4)}.dark\:ring-green-200\/45:is(.dark *){--tw-ring-color:rgba(187,247,208,.45)}.dark\:ring-green-200\/5:is(.dark *){--tw-ring-color:rgba(187,247,208,.05)}.dark\:ring-green-200\/50:is(.dark *){--tw-ring-color:rgba(187,247,208,.5)}.dark\:ring-green-200\/55:is(.dark *){--tw-ring-color:rgba(187,247,208,.55)}.dark\:ring-green-200\/60:is(.dark *){--tw-ring-color:rgba(187,247,208,.6)}.dark\:ring-green-200\/65:is(.dark *){--tw-ring-color:rgba(187,247,208,.65)}.dark\:ring-green-200\/70:is(.dark *){--tw-ring-color:rgba(187,247,208,.7)}.dark\:ring-green-200\/75:is(.dark *){--tw-ring-color:rgba(187,247,208,.75)}.dark\:ring-green-200\/80:is(.dark *){--tw-ring-color:rgba(187,247,208,.8)}.dark\:ring-green-200\/85:is(.dark *){--tw-ring-color:rgba(187,247,208,.85)}.dark\:ring-green-200\/90:is(.dark *){--tw-ring-color:rgba(187,247,208,.9)}.dark\:ring-green-200\/95:is(.dark *){--tw-ring-color:rgba(187,247,208,.95)}.dark\:ring-green-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(74 222 128/var(--tw-ring-opacity,1))}.dark\:ring-green-400\/0:is(.dark *){--tw-ring-color:rgba(74,222,128,0)}.dark\:ring-green-400\/10:is(.dark *){--tw-ring-color:rgba(74,222,128,.1)}.dark\:ring-green-400\/100:is(.dark *){--tw-ring-color:#4ade80}.dark\:ring-green-400\/15:is(.dark *){--tw-ring-color:rgba(74,222,128,.15)}.dark\:ring-green-400\/20:is(.dark *){--tw-ring-color:rgba(74,222,128,.2)}.dark\:ring-green-400\/25:is(.dark *){--tw-ring-color:rgba(74,222,128,.25)}.dark\:ring-green-400\/30:is(.dark *){--tw-ring-color:rgba(74,222,128,.3)}.dark\:ring-green-400\/35:is(.dark *){--tw-ring-color:rgba(74,222,128,.35)}.dark\:ring-green-400\/40:is(.dark *){--tw-ring-color:rgba(74,222,128,.4)}.dark\:ring-green-400\/45:is(.dark *){--tw-ring-color:rgba(74,222,128,.45)}.dark\:ring-green-400\/5:is(.dark *){--tw-ring-color:rgba(74,222,128,.05)}.dark\:ring-green-400\/50:is(.dark *){--tw-ring-color:rgba(74,222,128,.5)}.dark\:ring-green-400\/55:is(.dark *){--tw-ring-color:rgba(74,222,128,.55)}.dark\:ring-green-400\/60:is(.dark *){--tw-ring-color:rgba(74,222,128,.6)}.dark\:ring-green-400\/65:is(.dark *){--tw-ring-color:rgba(74,222,128,.65)}.dark\:ring-green-400\/70:is(.dark *){--tw-ring-color:rgba(74,222,128,.7)}.dark\:ring-green-400\/75:is(.dark *){--tw-ring-color:rgba(74,222,128,.75)}.dark\:ring-green-400\/80:is(.dark *){--tw-ring-color:rgba(74,222,128,.8)}.dark\:ring-green-400\/85:is(.dark *){--tw-ring-color:rgba(74,222,128,.85)}.dark\:ring-green-400\/90:is(.dark *){--tw-ring-color:rgba(74,222,128,.9)}.dark\:ring-green-400\/95:is(.dark *){--tw-ring-color:rgba(74,222,128,.95)}.dark\:ring-green-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(240 253 244/var(--tw-ring-opacity,1))}.dark\:ring-green-50\/0:is(.dark *){--tw-ring-color:rgba(240,253,244,0)}.dark\:ring-green-50\/10:is(.dark *){--tw-ring-color:rgba(240,253,244,.1)}.dark\:ring-green-50\/100:is(.dark *){--tw-ring-color:#f0fdf4}.dark\:ring-green-50\/15:is(.dark *){--tw-ring-color:rgba(240,253,244,.15)}.dark\:ring-green-50\/20:is(.dark *){--tw-ring-color:rgba(240,253,244,.2)}.dark\:ring-green-50\/25:is(.dark *){--tw-ring-color:rgba(240,253,244,.25)}.dark\:ring-green-50\/30:is(.dark *){--tw-ring-color:rgba(240,253,244,.3)}.dark\:ring-green-50\/35:is(.dark *){--tw-ring-color:rgba(240,253,244,.35)}.dark\:ring-green-50\/40:is(.dark *){--tw-ring-color:rgba(240,253,244,.4)}.dark\:ring-green-50\/45:is(.dark *){--tw-ring-color:rgba(240,253,244,.45)}.dark\:ring-green-50\/5:is(.dark *){--tw-ring-color:rgba(240,253,244,.05)}.dark\:ring-green-50\/50:is(.dark *){--tw-ring-color:rgba(240,253,244,.5)}.dark\:ring-green-50\/55:is(.dark *){--tw-ring-color:rgba(240,253,244,.55)}.dark\:ring-green-50\/60:is(.dark *){--tw-ring-color:rgba(240,253,244,.6)}.dark\:ring-green-50\/65:is(.dark *){--tw-ring-color:rgba(240,253,244,.65)}.dark\:ring-green-50\/70:is(.dark *){--tw-ring-color:rgba(240,253,244,.7)}.dark\:ring-green-50\/75:is(.dark *){--tw-ring-color:rgba(240,253,244,.75)}.dark\:ring-green-50\/80:is(.dark *){--tw-ring-color:rgba(240,253,244,.8)}.dark\:ring-green-50\/85:is(.dark *){--tw-ring-color:rgba(240,253,244,.85)}.dark\:ring-green-50\/90:is(.dark *){--tw-ring-color:rgba(240,253,244,.9)}.dark\:ring-green-50\/95:is(.dark *){--tw-ring-color:rgba(240,253,244,.95)}.dark\:ring-green-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(34 197 94/var(--tw-ring-opacity,1))}.dark\:ring-green-500\/0:is(.dark *){--tw-ring-color:rgba(34,197,94,0)}.dark\:ring-green-500\/10:is(.dark *){--tw-ring-color:rgba(34,197,94,.1)}.dark\:ring-green-500\/100:is(.dark *){--tw-ring-color:#22c55e}.dark\:ring-green-500\/15:is(.dark *){--tw-ring-color:rgba(34,197,94,.15)}.dark\:ring-green-500\/20:is(.dark *){--tw-ring-color:rgba(34,197,94,.2)}.dark\:ring-green-500\/25:is(.dark *){--tw-ring-color:rgba(34,197,94,.25)}.dark\:ring-green-500\/30:is(.dark *){--tw-ring-color:rgba(34,197,94,.3)}.dark\:ring-green-500\/35:is(.dark *){--tw-ring-color:rgba(34,197,94,.35)}.dark\:ring-green-500\/40:is(.dark *){--tw-ring-color:rgba(34,197,94,.4)}.dark\:ring-green-500\/45:is(.dark *){--tw-ring-color:rgba(34,197,94,.45)}.dark\:ring-green-500\/5:is(.dark *){--tw-ring-color:rgba(34,197,94,.05)}.dark\:ring-green-500\/50:is(.dark *){--tw-ring-color:rgba(34,197,94,.5)}.dark\:ring-green-500\/55:is(.dark *){--tw-ring-color:rgba(34,197,94,.55)}.dark\:ring-green-500\/60:is(.dark *){--tw-ring-color:rgba(34,197,94,.6)}.dark\:ring-green-500\/65:is(.dark *){--tw-ring-color:rgba(34,197,94,.65)}.dark\:ring-green-500\/70:is(.dark *){--tw-ring-color:rgba(34,197,94,.7)}.dark\:ring-green-500\/75:is(.dark *){--tw-ring-color:rgba(34,197,94,.75)}.dark\:ring-green-500\/80:is(.dark *){--tw-ring-color:rgba(34,197,94,.8)}.dark\:ring-green-500\/85:is(.dark *){--tw-ring-color:rgba(34,197,94,.85)}.dark\:ring-green-500\/90:is(.dark *){--tw-ring-color:rgba(34,197,94,.9)}.dark\:ring-green-500\/95:is(.dark *){--tw-ring-color:rgba(34,197,94,.95)}.dark\:ring-green-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(5 46 22/var(--tw-ring-opacity,1))}.dark\:ring-green-950\/0:is(.dark *){--tw-ring-color:rgba(5,46,22,0)}.dark\:ring-green-950\/10:is(.dark *){--tw-ring-color:rgba(5,46,22,.1)}.dark\:ring-green-950\/100:is(.dark *){--tw-ring-color:#052e16}.dark\:ring-green-950\/15:is(.dark *){--tw-ring-color:rgba(5,46,22,.15)}.dark\:ring-green-950\/20:is(.dark *){--tw-ring-color:rgba(5,46,22,.2)}.dark\:ring-green-950\/25:is(.dark *){--tw-ring-color:rgba(5,46,22,.25)}.dark\:ring-green-950\/30:is(.dark *){--tw-ring-color:rgba(5,46,22,.3)}.dark\:ring-green-950\/35:is(.dark *){--tw-ring-color:rgba(5,46,22,.35)}.dark\:ring-green-950\/40:is(.dark *){--tw-ring-color:rgba(5,46,22,.4)}.dark\:ring-green-950\/45:is(.dark *){--tw-ring-color:rgba(5,46,22,.45)}.dark\:ring-green-950\/5:is(.dark *){--tw-ring-color:rgba(5,46,22,.05)}.dark\:ring-green-950\/50:is(.dark *){--tw-ring-color:rgba(5,46,22,.5)}.dark\:ring-green-950\/55:is(.dark *){--tw-ring-color:rgba(5,46,22,.55)}.dark\:ring-green-950\/60:is(.dark *){--tw-ring-color:rgba(5,46,22,.6)}.dark\:ring-green-950\/65:is(.dark *){--tw-ring-color:rgba(5,46,22,.65)}.dark\:ring-green-950\/70:is(.dark *){--tw-ring-color:rgba(5,46,22,.7)}.dark\:ring-green-950\/75:is(.dark *){--tw-ring-color:rgba(5,46,22,.75)}.dark\:ring-green-950\/80:is(.dark *){--tw-ring-color:rgba(5,46,22,.8)}.dark\:ring-green-950\/85:is(.dark *){--tw-ring-color:rgba(5,46,22,.85)}.dark\:ring-green-950\/90:is(.dark *){--tw-ring-color:rgba(5,46,22,.9)}.dark\:ring-green-950\/95:is(.dark *){--tw-ring-color:rgba(5,46,22,.95)}.dark\:ring-indigo-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(224 231 255/var(--tw-ring-opacity,1))}.dark\:ring-indigo-100\/0:is(.dark *){--tw-ring-color:rgba(224,231,255,0)}.dark\:ring-indigo-100\/10:is(.dark *){--tw-ring-color:rgba(224,231,255,.1)}.dark\:ring-indigo-100\/100:is(.dark *){--tw-ring-color:#e0e7ff}.dark\:ring-indigo-100\/15:is(.dark *){--tw-ring-color:rgba(224,231,255,.15)}.dark\:ring-indigo-100\/20:is(.dark *){--tw-ring-color:rgba(224,231,255,.2)}.dark\:ring-indigo-100\/25:is(.dark *){--tw-ring-color:rgba(224,231,255,.25)}.dark\:ring-indigo-100\/30:is(.dark *){--tw-ring-color:rgba(224,231,255,.3)}.dark\:ring-indigo-100\/35:is(.dark *){--tw-ring-color:rgba(224,231,255,.35)}.dark\:ring-indigo-100\/40:is(.dark *){--tw-ring-color:rgba(224,231,255,.4)}.dark\:ring-indigo-100\/45:is(.dark *){--tw-ring-color:rgba(224,231,255,.45)}.dark\:ring-indigo-100\/5:is(.dark *){--tw-ring-color:rgba(224,231,255,.05)}.dark\:ring-indigo-100\/50:is(.dark *){--tw-ring-color:rgba(224,231,255,.5)}.dark\:ring-indigo-100\/55:is(.dark *){--tw-ring-color:rgba(224,231,255,.55)}.dark\:ring-indigo-100\/60:is(.dark *){--tw-ring-color:rgba(224,231,255,.6)}.dark\:ring-indigo-100\/65:is(.dark *){--tw-ring-color:rgba(224,231,255,.65)}.dark\:ring-indigo-100\/70:is(.dark *){--tw-ring-color:rgba(224,231,255,.7)}.dark\:ring-indigo-100\/75:is(.dark *){--tw-ring-color:rgba(224,231,255,.75)}.dark\:ring-indigo-100\/80:is(.dark *){--tw-ring-color:rgba(224,231,255,.8)}.dark\:ring-indigo-100\/85:is(.dark *){--tw-ring-color:rgba(224,231,255,.85)}.dark\:ring-indigo-100\/90:is(.dark *){--tw-ring-color:rgba(224,231,255,.9)}.dark\:ring-indigo-100\/95:is(.dark *){--tw-ring-color:rgba(224,231,255,.95)}.dark\:ring-indigo-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(199 210 254/var(--tw-ring-opacity,1))}.dark\:ring-indigo-200\/0:is(.dark *){--tw-ring-color:rgba(199,210,254,0)}.dark\:ring-indigo-200\/10:is(.dark *){--tw-ring-color:rgba(199,210,254,.1)}.dark\:ring-indigo-200\/100:is(.dark *){--tw-ring-color:#c7d2fe}.dark\:ring-indigo-200\/15:is(.dark *){--tw-ring-color:rgba(199,210,254,.15)}.dark\:ring-indigo-200\/20:is(.dark *){--tw-ring-color:rgba(199,210,254,.2)}.dark\:ring-indigo-200\/25:is(.dark *){--tw-ring-color:rgba(199,210,254,.25)}.dark\:ring-indigo-200\/30:is(.dark *){--tw-ring-color:rgba(199,210,254,.3)}.dark\:ring-indigo-200\/35:is(.dark *){--tw-ring-color:rgba(199,210,254,.35)}.dark\:ring-indigo-200\/40:is(.dark *){--tw-ring-color:rgba(199,210,254,.4)}.dark\:ring-indigo-200\/45:is(.dark *){--tw-ring-color:rgba(199,210,254,.45)}.dark\:ring-indigo-200\/5:is(.dark *){--tw-ring-color:rgba(199,210,254,.05)}.dark\:ring-indigo-200\/50:is(.dark *){--tw-ring-color:rgba(199,210,254,.5)}.dark\:ring-indigo-200\/55:is(.dark *){--tw-ring-color:rgba(199,210,254,.55)}.dark\:ring-indigo-200\/60:is(.dark *){--tw-ring-color:rgba(199,210,254,.6)}.dark\:ring-indigo-200\/65:is(.dark *){--tw-ring-color:rgba(199,210,254,.65)}.dark\:ring-indigo-200\/70:is(.dark *){--tw-ring-color:rgba(199,210,254,.7)}.dark\:ring-indigo-200\/75:is(.dark *){--tw-ring-color:rgba(199,210,254,.75)}.dark\:ring-indigo-200\/80:is(.dark *){--tw-ring-color:rgba(199,210,254,.8)}.dark\:ring-indigo-200\/85:is(.dark *){--tw-ring-color:rgba(199,210,254,.85)}.dark\:ring-indigo-200\/90:is(.dark *){--tw-ring-color:rgba(199,210,254,.9)}.dark\:ring-indigo-200\/95:is(.dark *){--tw-ring-color:rgba(199,210,254,.95)}.dark\:ring-indigo-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(129 140 248/var(--tw-ring-opacity,1))}.dark\:ring-indigo-400\/0:is(.dark *){--tw-ring-color:rgba(129,140,248,0)}.dark\:ring-indigo-400\/10:is(.dark *){--tw-ring-color:rgba(129,140,248,.1)}.dark\:ring-indigo-400\/100:is(.dark *){--tw-ring-color:#818cf8}.dark\:ring-indigo-400\/15:is(.dark *){--tw-ring-color:rgba(129,140,248,.15)}.dark\:ring-indigo-400\/20:is(.dark *){--tw-ring-color:rgba(129,140,248,.2)}.dark\:ring-indigo-400\/25:is(.dark *){--tw-ring-color:rgba(129,140,248,.25)}.dark\:ring-indigo-400\/30:is(.dark *){--tw-ring-color:rgba(129,140,248,.3)}.dark\:ring-indigo-400\/35:is(.dark *){--tw-ring-color:rgba(129,140,248,.35)}.dark\:ring-indigo-400\/40:is(.dark *){--tw-ring-color:rgba(129,140,248,.4)}.dark\:ring-indigo-400\/45:is(.dark *){--tw-ring-color:rgba(129,140,248,.45)}.dark\:ring-indigo-400\/5:is(.dark *){--tw-ring-color:rgba(129,140,248,.05)}.dark\:ring-indigo-400\/50:is(.dark *){--tw-ring-color:rgba(129,140,248,.5)}.dark\:ring-indigo-400\/55:is(.dark *){--tw-ring-color:rgba(129,140,248,.55)}.dark\:ring-indigo-400\/60:is(.dark *){--tw-ring-color:rgba(129,140,248,.6)}.dark\:ring-indigo-400\/65:is(.dark *){--tw-ring-color:rgba(129,140,248,.65)}.dark\:ring-indigo-400\/70:is(.dark *){--tw-ring-color:rgba(129,140,248,.7)}.dark\:ring-indigo-400\/75:is(.dark *){--tw-ring-color:rgba(129,140,248,.75)}.dark\:ring-indigo-400\/80:is(.dark *){--tw-ring-color:rgba(129,140,248,.8)}.dark\:ring-indigo-400\/85:is(.dark *){--tw-ring-color:rgba(129,140,248,.85)}.dark\:ring-indigo-400\/90:is(.dark *){--tw-ring-color:rgba(129,140,248,.9)}.dark\:ring-indigo-400\/95:is(.dark *){--tw-ring-color:rgba(129,140,248,.95)}.dark\:ring-indigo-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(238 242 255/var(--tw-ring-opacity,1))}.dark\:ring-indigo-50\/0:is(.dark *){--tw-ring-color:rgba(238,242,255,0)}.dark\:ring-indigo-50\/10:is(.dark *){--tw-ring-color:rgba(238,242,255,.1)}.dark\:ring-indigo-50\/100:is(.dark *){--tw-ring-color:#eef2ff}.dark\:ring-indigo-50\/15:is(.dark *){--tw-ring-color:rgba(238,242,255,.15)}.dark\:ring-indigo-50\/20:is(.dark *){--tw-ring-color:rgba(238,242,255,.2)}.dark\:ring-indigo-50\/25:is(.dark *){--tw-ring-color:rgba(238,242,255,.25)}.dark\:ring-indigo-50\/30:is(.dark *){--tw-ring-color:rgba(238,242,255,.3)}.dark\:ring-indigo-50\/35:is(.dark *){--tw-ring-color:rgba(238,242,255,.35)}.dark\:ring-indigo-50\/40:is(.dark *){--tw-ring-color:rgba(238,242,255,.4)}.dark\:ring-indigo-50\/45:is(.dark *){--tw-ring-color:rgba(238,242,255,.45)}.dark\:ring-indigo-50\/5:is(.dark *){--tw-ring-color:rgba(238,242,255,.05)}.dark\:ring-indigo-50\/50:is(.dark *){--tw-ring-color:rgba(238,242,255,.5)}.dark\:ring-indigo-50\/55:is(.dark *){--tw-ring-color:rgba(238,242,255,.55)}.dark\:ring-indigo-50\/60:is(.dark *){--tw-ring-color:rgba(238,242,255,.6)}.dark\:ring-indigo-50\/65:is(.dark *){--tw-ring-color:rgba(238,242,255,.65)}.dark\:ring-indigo-50\/70:is(.dark *){--tw-ring-color:rgba(238,242,255,.7)}.dark\:ring-indigo-50\/75:is(.dark *){--tw-ring-color:rgba(238,242,255,.75)}.dark\:ring-indigo-50\/80:is(.dark *){--tw-ring-color:rgba(238,242,255,.8)}.dark\:ring-indigo-50\/85:is(.dark *){--tw-ring-color:rgba(238,242,255,.85)}.dark\:ring-indigo-50\/90:is(.dark *){--tw-ring-color:rgba(238,242,255,.9)}.dark\:ring-indigo-50\/95:is(.dark *){--tw-ring-color:rgba(238,242,255,.95)}.dark\:ring-indigo-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(99 102 241/var(--tw-ring-opacity,1))}.dark\:ring-indigo-500\/0:is(.dark *){--tw-ring-color:rgba(99,102,241,0)}.dark\:ring-indigo-500\/10:is(.dark *){--tw-ring-color:rgba(99,102,241,.1)}.dark\:ring-indigo-500\/100:is(.dark *){--tw-ring-color:#6366f1}.dark\:ring-indigo-500\/15:is(.dark *){--tw-ring-color:rgba(99,102,241,.15)}.dark\:ring-indigo-500\/20:is(.dark *){--tw-ring-color:rgba(99,102,241,.2)}.dark\:ring-indigo-500\/25:is(.dark *){--tw-ring-color:rgba(99,102,241,.25)}.dark\:ring-indigo-500\/30:is(.dark *){--tw-ring-color:rgba(99,102,241,.3)}.dark\:ring-indigo-500\/35:is(.dark *){--tw-ring-color:rgba(99,102,241,.35)}.dark\:ring-indigo-500\/40:is(.dark *){--tw-ring-color:rgba(99,102,241,.4)}.dark\:ring-indigo-500\/45:is(.dark *){--tw-ring-color:rgba(99,102,241,.45)}.dark\:ring-indigo-500\/5:is(.dark *){--tw-ring-color:rgba(99,102,241,.05)}.dark\:ring-indigo-500\/50:is(.dark *){--tw-ring-color:rgba(99,102,241,.5)}.dark\:ring-indigo-500\/55:is(.dark *){--tw-ring-color:rgba(99,102,241,.55)}.dark\:ring-indigo-500\/60:is(.dark *){--tw-ring-color:rgba(99,102,241,.6)}.dark\:ring-indigo-500\/65:is(.dark *){--tw-ring-color:rgba(99,102,241,.65)}.dark\:ring-indigo-500\/70:is(.dark *){--tw-ring-color:rgba(99,102,241,.7)}.dark\:ring-indigo-500\/75:is(.dark *){--tw-ring-color:rgba(99,102,241,.75)}.dark\:ring-indigo-500\/80:is(.dark *){--tw-ring-color:rgba(99,102,241,.8)}.dark\:ring-indigo-500\/85:is(.dark *){--tw-ring-color:rgba(99,102,241,.85)}.dark\:ring-indigo-500\/90:is(.dark *){--tw-ring-color:rgba(99,102,241,.9)}.dark\:ring-indigo-500\/95:is(.dark *){--tw-ring-color:rgba(99,102,241,.95)}.dark\:ring-indigo-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(30 27 75/var(--tw-ring-opacity,1))}.dark\:ring-indigo-950\/0:is(.dark *){--tw-ring-color:rgba(30,27,75,0)}.dark\:ring-indigo-950\/10:is(.dark *){--tw-ring-color:rgba(30,27,75,.1)}.dark\:ring-indigo-950\/100:is(.dark *){--tw-ring-color:#1e1b4b}.dark\:ring-indigo-950\/15:is(.dark *){--tw-ring-color:rgba(30,27,75,.15)}.dark\:ring-indigo-950\/20:is(.dark *){--tw-ring-color:rgba(30,27,75,.2)}.dark\:ring-indigo-950\/25:is(.dark *){--tw-ring-color:rgba(30,27,75,.25)}.dark\:ring-indigo-950\/30:is(.dark *){--tw-ring-color:rgba(30,27,75,.3)}.dark\:ring-indigo-950\/35:is(.dark *){--tw-ring-color:rgba(30,27,75,.35)}.dark\:ring-indigo-950\/40:is(.dark *){--tw-ring-color:rgba(30,27,75,.4)}.dark\:ring-indigo-950\/45:is(.dark *){--tw-ring-color:rgba(30,27,75,.45)}.dark\:ring-indigo-950\/5:is(.dark *){--tw-ring-color:rgba(30,27,75,.05)}.dark\:ring-indigo-950\/50:is(.dark *){--tw-ring-color:rgba(30,27,75,.5)}.dark\:ring-indigo-950\/55:is(.dark *){--tw-ring-color:rgba(30,27,75,.55)}.dark\:ring-indigo-950\/60:is(.dark *){--tw-ring-color:rgba(30,27,75,.6)}.dark\:ring-indigo-950\/65:is(.dark *){--tw-ring-color:rgba(30,27,75,.65)}.dark\:ring-indigo-950\/70:is(.dark *){--tw-ring-color:rgba(30,27,75,.7)}.dark\:ring-indigo-950\/75:is(.dark *){--tw-ring-color:rgba(30,27,75,.75)}.dark\:ring-indigo-950\/80:is(.dark *){--tw-ring-color:rgba(30,27,75,.8)}.dark\:ring-indigo-950\/85:is(.dark *){--tw-ring-color:rgba(30,27,75,.85)}.dark\:ring-indigo-950\/90:is(.dark *){--tw-ring-color:rgba(30,27,75,.9)}.dark\:ring-indigo-950\/95:is(.dark *){--tw-ring-color:rgba(30,27,75,.95)}.dark\:ring-lime-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(236 252 203/var(--tw-ring-opacity,1))}.dark\:ring-lime-100\/0:is(.dark *){--tw-ring-color:hsla(80,89%,89%,0)}.dark\:ring-lime-100\/10:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.1)}.dark\:ring-lime-100\/100:is(.dark *){--tw-ring-color:#ecfccb}.dark\:ring-lime-100\/15:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.15)}.dark\:ring-lime-100\/20:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.2)}.dark\:ring-lime-100\/25:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.25)}.dark\:ring-lime-100\/30:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.3)}.dark\:ring-lime-100\/35:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.35)}.dark\:ring-lime-100\/40:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.4)}.dark\:ring-lime-100\/45:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.45)}.dark\:ring-lime-100\/5:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.05)}.dark\:ring-lime-100\/50:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.5)}.dark\:ring-lime-100\/55:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.55)}.dark\:ring-lime-100\/60:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.6)}.dark\:ring-lime-100\/65:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.65)}.dark\:ring-lime-100\/70:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.7)}.dark\:ring-lime-100\/75:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.75)}.dark\:ring-lime-100\/80:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.8)}.dark\:ring-lime-100\/85:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.85)}.dark\:ring-lime-100\/90:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.9)}.dark\:ring-lime-100\/95:is(.dark *){--tw-ring-color:hsla(80,89%,89%,.95)}.dark\:ring-lime-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(217 249 157/var(--tw-ring-opacity,1))}.dark\:ring-lime-200\/0:is(.dark *){--tw-ring-color:hsla(81,88%,80%,0)}.dark\:ring-lime-200\/10:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.1)}.dark\:ring-lime-200\/100:is(.dark *){--tw-ring-color:#d9f99d}.dark\:ring-lime-200\/15:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.15)}.dark\:ring-lime-200\/20:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.2)}.dark\:ring-lime-200\/25:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.25)}.dark\:ring-lime-200\/30:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.3)}.dark\:ring-lime-200\/35:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.35)}.dark\:ring-lime-200\/40:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.4)}.dark\:ring-lime-200\/45:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.45)}.dark\:ring-lime-200\/5:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.05)}.dark\:ring-lime-200\/50:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.5)}.dark\:ring-lime-200\/55:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.55)}.dark\:ring-lime-200\/60:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.6)}.dark\:ring-lime-200\/65:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.65)}.dark\:ring-lime-200\/70:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.7)}.dark\:ring-lime-200\/75:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.75)}.dark\:ring-lime-200\/80:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.8)}.dark\:ring-lime-200\/85:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.85)}.dark\:ring-lime-200\/90:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.9)}.dark\:ring-lime-200\/95:is(.dark *){--tw-ring-color:hsla(81,88%,80%,.95)}.dark\:ring-lime-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(163 230 53/var(--tw-ring-opacity,1))}.dark\:ring-lime-400\/0:is(.dark *){--tw-ring-color:rgba(163,230,53,0)}.dark\:ring-lime-400\/10:is(.dark *){--tw-ring-color:rgba(163,230,53,.1)}.dark\:ring-lime-400\/100:is(.dark *){--tw-ring-color:#a3e635}.dark\:ring-lime-400\/15:is(.dark *){--tw-ring-color:rgba(163,230,53,.15)}.dark\:ring-lime-400\/20:is(.dark *){--tw-ring-color:rgba(163,230,53,.2)}.dark\:ring-lime-400\/25:is(.dark *){--tw-ring-color:rgba(163,230,53,.25)}.dark\:ring-lime-400\/30:is(.dark *){--tw-ring-color:rgba(163,230,53,.3)}.dark\:ring-lime-400\/35:is(.dark *){--tw-ring-color:rgba(163,230,53,.35)}.dark\:ring-lime-400\/40:is(.dark *){--tw-ring-color:rgba(163,230,53,.4)}.dark\:ring-lime-400\/45:is(.dark *){--tw-ring-color:rgba(163,230,53,.45)}.dark\:ring-lime-400\/5:is(.dark *){--tw-ring-color:rgba(163,230,53,.05)}.dark\:ring-lime-400\/50:is(.dark *){--tw-ring-color:rgba(163,230,53,.5)}.dark\:ring-lime-400\/55:is(.dark *){--tw-ring-color:rgba(163,230,53,.55)}.dark\:ring-lime-400\/60:is(.dark *){--tw-ring-color:rgba(163,230,53,.6)}.dark\:ring-lime-400\/65:is(.dark *){--tw-ring-color:rgba(163,230,53,.65)}.dark\:ring-lime-400\/70:is(.dark *){--tw-ring-color:rgba(163,230,53,.7)}.dark\:ring-lime-400\/75:is(.dark *){--tw-ring-color:rgba(163,230,53,.75)}.dark\:ring-lime-400\/80:is(.dark *){--tw-ring-color:rgba(163,230,53,.8)}.dark\:ring-lime-400\/85:is(.dark *){--tw-ring-color:rgba(163,230,53,.85)}.dark\:ring-lime-400\/90:is(.dark *){--tw-ring-color:rgba(163,230,53,.9)}.dark\:ring-lime-400\/95:is(.dark *){--tw-ring-color:rgba(163,230,53,.95)}.dark\:ring-lime-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(247 254 231/var(--tw-ring-opacity,1))}.dark\:ring-lime-50\/0:is(.dark *){--tw-ring-color:hsla(78,92%,95%,0)}.dark\:ring-lime-50\/10:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.1)}.dark\:ring-lime-50\/100:is(.dark *){--tw-ring-color:#f7fee7}.dark\:ring-lime-50\/15:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.15)}.dark\:ring-lime-50\/20:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.2)}.dark\:ring-lime-50\/25:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.25)}.dark\:ring-lime-50\/30:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.3)}.dark\:ring-lime-50\/35:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.35)}.dark\:ring-lime-50\/40:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.4)}.dark\:ring-lime-50\/45:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.45)}.dark\:ring-lime-50\/5:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.05)}.dark\:ring-lime-50\/50:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.5)}.dark\:ring-lime-50\/55:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.55)}.dark\:ring-lime-50\/60:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.6)}.dark\:ring-lime-50\/65:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.65)}.dark\:ring-lime-50\/70:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.7)}.dark\:ring-lime-50\/75:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.75)}.dark\:ring-lime-50\/80:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.8)}.dark\:ring-lime-50\/85:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.85)}.dark\:ring-lime-50\/90:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.9)}.dark\:ring-lime-50\/95:is(.dark *){--tw-ring-color:hsla(78,92%,95%,.95)}.dark\:ring-lime-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(132 204 22/var(--tw-ring-opacity,1))}.dark\:ring-lime-500\/0:is(.dark *){--tw-ring-color:rgba(132,204,22,0)}.dark\:ring-lime-500\/10:is(.dark *){--tw-ring-color:rgba(132,204,22,.1)}.dark\:ring-lime-500\/100:is(.dark *){--tw-ring-color:#84cc16}.dark\:ring-lime-500\/15:is(.dark *){--tw-ring-color:rgba(132,204,22,.15)}.dark\:ring-lime-500\/20:is(.dark *){--tw-ring-color:rgba(132,204,22,.2)}.dark\:ring-lime-500\/25:is(.dark *){--tw-ring-color:rgba(132,204,22,.25)}.dark\:ring-lime-500\/30:is(.dark *){--tw-ring-color:rgba(132,204,22,.3)}.dark\:ring-lime-500\/35:is(.dark *){--tw-ring-color:rgba(132,204,22,.35)}.dark\:ring-lime-500\/40:is(.dark *){--tw-ring-color:rgba(132,204,22,.4)}.dark\:ring-lime-500\/45:is(.dark *){--tw-ring-color:rgba(132,204,22,.45)}.dark\:ring-lime-500\/5:is(.dark *){--tw-ring-color:rgba(132,204,22,.05)}.dark\:ring-lime-500\/50:is(.dark *){--tw-ring-color:rgba(132,204,22,.5)}.dark\:ring-lime-500\/55:is(.dark *){--tw-ring-color:rgba(132,204,22,.55)}.dark\:ring-lime-500\/60:is(.dark *){--tw-ring-color:rgba(132,204,22,.6)}.dark\:ring-lime-500\/65:is(.dark *){--tw-ring-color:rgba(132,204,22,.65)}.dark\:ring-lime-500\/70:is(.dark *){--tw-ring-color:rgba(132,204,22,.7)}.dark\:ring-lime-500\/75:is(.dark *){--tw-ring-color:rgba(132,204,22,.75)}.dark\:ring-lime-500\/80:is(.dark *){--tw-ring-color:rgba(132,204,22,.8)}.dark\:ring-lime-500\/85:is(.dark *){--tw-ring-color:rgba(132,204,22,.85)}.dark\:ring-lime-500\/90:is(.dark *){--tw-ring-color:rgba(132,204,22,.9)}.dark\:ring-lime-500\/95:is(.dark *){--tw-ring-color:rgba(132,204,22,.95)}.dark\:ring-lime-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(26 46 5/var(--tw-ring-opacity,1))}.dark\:ring-lime-950\/0:is(.dark *){--tw-ring-color:rgba(26,46,5,0)}.dark\:ring-lime-950\/10:is(.dark *){--tw-ring-color:rgba(26,46,5,.1)}.dark\:ring-lime-950\/100:is(.dark *){--tw-ring-color:#1a2e05}.dark\:ring-lime-950\/15:is(.dark *){--tw-ring-color:rgba(26,46,5,.15)}.dark\:ring-lime-950\/20:is(.dark *){--tw-ring-color:rgba(26,46,5,.2)}.dark\:ring-lime-950\/25:is(.dark *){--tw-ring-color:rgba(26,46,5,.25)}.dark\:ring-lime-950\/30:is(.dark *){--tw-ring-color:rgba(26,46,5,.3)}.dark\:ring-lime-950\/35:is(.dark *){--tw-ring-color:rgba(26,46,5,.35)}.dark\:ring-lime-950\/40:is(.dark *){--tw-ring-color:rgba(26,46,5,.4)}.dark\:ring-lime-950\/45:is(.dark *){--tw-ring-color:rgba(26,46,5,.45)}.dark\:ring-lime-950\/5:is(.dark *){--tw-ring-color:rgba(26,46,5,.05)}.dark\:ring-lime-950\/50:is(.dark *){--tw-ring-color:rgba(26,46,5,.5)}.dark\:ring-lime-950\/55:is(.dark *){--tw-ring-color:rgba(26,46,5,.55)}.dark\:ring-lime-950\/60:is(.dark *){--tw-ring-color:rgba(26,46,5,.6)}.dark\:ring-lime-950\/65:is(.dark *){--tw-ring-color:rgba(26,46,5,.65)}.dark\:ring-lime-950\/70:is(.dark *){--tw-ring-color:rgba(26,46,5,.7)}.dark\:ring-lime-950\/75:is(.dark *){--tw-ring-color:rgba(26,46,5,.75)}.dark\:ring-lime-950\/80:is(.dark *){--tw-ring-color:rgba(26,46,5,.8)}.dark\:ring-lime-950\/85:is(.dark *){--tw-ring-color:rgba(26,46,5,.85)}.dark\:ring-lime-950\/90:is(.dark *){--tw-ring-color:rgba(26,46,5,.9)}.dark\:ring-lime-950\/95:is(.dark *){--tw-ring-color:rgba(26,46,5,.95)}.dark\:ring-neutral-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(245 245 245/var(--tw-ring-opacity,1))}.dark\:ring-neutral-100\/0:is(.dark *){--tw-ring-color:hsla(0,0%,96%,0)}.dark\:ring-neutral-100\/10:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.1)}.dark\:ring-neutral-100\/100:is(.dark *){--tw-ring-color:#f5f5f5}.dark\:ring-neutral-100\/15:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.15)}.dark\:ring-neutral-100\/20:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.2)}.dark\:ring-neutral-100\/25:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.25)}.dark\:ring-neutral-100\/30:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.3)}.dark\:ring-neutral-100\/35:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.35)}.dark\:ring-neutral-100\/40:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.4)}.dark\:ring-neutral-100\/45:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.45)}.dark\:ring-neutral-100\/5:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.05)}.dark\:ring-neutral-100\/50:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.5)}.dark\:ring-neutral-100\/55:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.55)}.dark\:ring-neutral-100\/60:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.6)}.dark\:ring-neutral-100\/65:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.65)}.dark\:ring-neutral-100\/70:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.7)}.dark\:ring-neutral-100\/75:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.75)}.dark\:ring-neutral-100\/80:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.8)}.dark\:ring-neutral-100\/85:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.85)}.dark\:ring-neutral-100\/90:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.9)}.dark\:ring-neutral-100\/95:is(.dark *){--tw-ring-color:hsla(0,0%,96%,.95)}.dark\:ring-neutral-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(229 229 229/var(--tw-ring-opacity,1))}.dark\:ring-neutral-200\/0:is(.dark *){--tw-ring-color:hsla(0,0%,90%,0)}.dark\:ring-neutral-200\/10:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.1)}.dark\:ring-neutral-200\/100:is(.dark *){--tw-ring-color:#e5e5e5}.dark\:ring-neutral-200\/15:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.15)}.dark\:ring-neutral-200\/20:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.2)}.dark\:ring-neutral-200\/25:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.25)}.dark\:ring-neutral-200\/30:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.3)}.dark\:ring-neutral-200\/35:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.35)}.dark\:ring-neutral-200\/40:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.4)}.dark\:ring-neutral-200\/45:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.45)}.dark\:ring-neutral-200\/5:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.05)}.dark\:ring-neutral-200\/50:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.5)}.dark\:ring-neutral-200\/55:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.55)}.dark\:ring-neutral-200\/60:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.6)}.dark\:ring-neutral-200\/65:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.65)}.dark\:ring-neutral-200\/70:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.7)}.dark\:ring-neutral-200\/75:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.75)}.dark\:ring-neutral-200\/80:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.8)}.dark\:ring-neutral-200\/85:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.85)}.dark\:ring-neutral-200\/90:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.9)}.dark\:ring-neutral-200\/95:is(.dark *){--tw-ring-color:hsla(0,0%,90%,.95)}.dark\:ring-neutral-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(163 163 163/var(--tw-ring-opacity,1))}.dark\:ring-neutral-400\/0:is(.dark *){--tw-ring-color:hsla(0,0%,64%,0)}.dark\:ring-neutral-400\/10:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.1)}.dark\:ring-neutral-400\/100:is(.dark *){--tw-ring-color:#a3a3a3}.dark\:ring-neutral-400\/15:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.15)}.dark\:ring-neutral-400\/20:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.2)}.dark\:ring-neutral-400\/25:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.25)}.dark\:ring-neutral-400\/30:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.3)}.dark\:ring-neutral-400\/35:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.35)}.dark\:ring-neutral-400\/40:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.4)}.dark\:ring-neutral-400\/45:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.45)}.dark\:ring-neutral-400\/5:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.05)}.dark\:ring-neutral-400\/50:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.5)}.dark\:ring-neutral-400\/55:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.55)}.dark\:ring-neutral-400\/60:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.6)}.dark\:ring-neutral-400\/65:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.65)}.dark\:ring-neutral-400\/70:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.7)}.dark\:ring-neutral-400\/75:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.75)}.dark\:ring-neutral-400\/80:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.8)}.dark\:ring-neutral-400\/85:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.85)}.dark\:ring-neutral-400\/90:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.9)}.dark\:ring-neutral-400\/95:is(.dark *){--tw-ring-color:hsla(0,0%,64%,.95)}.dark\:ring-neutral-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(250 250 250/var(--tw-ring-opacity,1))}.dark\:ring-neutral-50\/0:is(.dark *){--tw-ring-color:hsla(0,0%,98%,0)}.dark\:ring-neutral-50\/10:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.1)}.dark\:ring-neutral-50\/100:is(.dark *){--tw-ring-color:#fafafa}.dark\:ring-neutral-50\/15:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.15)}.dark\:ring-neutral-50\/20:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.2)}.dark\:ring-neutral-50\/25:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.25)}.dark\:ring-neutral-50\/30:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.3)}.dark\:ring-neutral-50\/35:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.35)}.dark\:ring-neutral-50\/40:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.4)}.dark\:ring-neutral-50\/45:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.45)}.dark\:ring-neutral-50\/5:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.05)}.dark\:ring-neutral-50\/50:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.5)}.dark\:ring-neutral-50\/55:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.55)}.dark\:ring-neutral-50\/60:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.6)}.dark\:ring-neutral-50\/65:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.65)}.dark\:ring-neutral-50\/70:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.7)}.dark\:ring-neutral-50\/75:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.75)}.dark\:ring-neutral-50\/80:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.8)}.dark\:ring-neutral-50\/85:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.85)}.dark\:ring-neutral-50\/90:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.9)}.dark\:ring-neutral-50\/95:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.95)}.dark\:ring-neutral-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(115 115 115/var(--tw-ring-opacity,1))}.dark\:ring-neutral-500\/0:is(.dark *){--tw-ring-color:hsla(0,0%,45%,0)}.dark\:ring-neutral-500\/10:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.1)}.dark\:ring-neutral-500\/100:is(.dark *){--tw-ring-color:#737373}.dark\:ring-neutral-500\/15:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.15)}.dark\:ring-neutral-500\/20:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.2)}.dark\:ring-neutral-500\/25:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.25)}.dark\:ring-neutral-500\/30:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.3)}.dark\:ring-neutral-500\/35:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.35)}.dark\:ring-neutral-500\/40:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.4)}.dark\:ring-neutral-500\/45:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.45)}.dark\:ring-neutral-500\/5:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.05)}.dark\:ring-neutral-500\/50:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.5)}.dark\:ring-neutral-500\/55:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.55)}.dark\:ring-neutral-500\/60:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.6)}.dark\:ring-neutral-500\/65:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.65)}.dark\:ring-neutral-500\/70:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.7)}.dark\:ring-neutral-500\/75:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.75)}.dark\:ring-neutral-500\/80:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.8)}.dark\:ring-neutral-500\/85:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.85)}.dark\:ring-neutral-500\/90:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.9)}.dark\:ring-neutral-500\/95:is(.dark *){--tw-ring-color:hsla(0,0%,45%,.95)}.dark\:ring-neutral-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(10 10 10/var(--tw-ring-opacity,1))}.dark\:ring-neutral-950\/0:is(.dark *){--tw-ring-color:hsla(0,0%,4%,0)}.dark\:ring-neutral-950\/10:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.1)}.dark\:ring-neutral-950\/100:is(.dark *){--tw-ring-color:#0a0a0a}.dark\:ring-neutral-950\/15:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.15)}.dark\:ring-neutral-950\/20:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.2)}.dark\:ring-neutral-950\/25:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.25)}.dark\:ring-neutral-950\/30:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.3)}.dark\:ring-neutral-950\/35:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.35)}.dark\:ring-neutral-950\/40:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.4)}.dark\:ring-neutral-950\/45:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.45)}.dark\:ring-neutral-950\/5:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.05)}.dark\:ring-neutral-950\/50:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.5)}.dark\:ring-neutral-950\/55:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.55)}.dark\:ring-neutral-950\/60:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.6)}.dark\:ring-neutral-950\/65:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.65)}.dark\:ring-neutral-950\/70:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.7)}.dark\:ring-neutral-950\/75:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.75)}.dark\:ring-neutral-950\/80:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.8)}.dark\:ring-neutral-950\/85:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.85)}.dark\:ring-neutral-950\/90:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.9)}.dark\:ring-neutral-950\/95:is(.dark *){--tw-ring-color:hsla(0,0%,4%,.95)}.dark\:ring-orange-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(255 237 213/var(--tw-ring-opacity,1))}.dark\:ring-orange-100\/0:is(.dark *){--tw-ring-color:rgba(255,237,213,0)}.dark\:ring-orange-100\/10:is(.dark *){--tw-ring-color:rgba(255,237,213,.1)}.dark\:ring-orange-100\/100:is(.dark *){--tw-ring-color:#ffedd5}.dark\:ring-orange-100\/15:is(.dark *){--tw-ring-color:rgba(255,237,213,.15)}.dark\:ring-orange-100\/20:is(.dark *){--tw-ring-color:rgba(255,237,213,.2)}.dark\:ring-orange-100\/25:is(.dark *){--tw-ring-color:rgba(255,237,213,.25)}.dark\:ring-orange-100\/30:is(.dark *){--tw-ring-color:rgba(255,237,213,.3)}.dark\:ring-orange-100\/35:is(.dark *){--tw-ring-color:rgba(255,237,213,.35)}.dark\:ring-orange-100\/40:is(.dark *){--tw-ring-color:rgba(255,237,213,.4)}.dark\:ring-orange-100\/45:is(.dark *){--tw-ring-color:rgba(255,237,213,.45)}.dark\:ring-orange-100\/5:is(.dark *){--tw-ring-color:rgba(255,237,213,.05)}.dark\:ring-orange-100\/50:is(.dark *){--tw-ring-color:rgba(255,237,213,.5)}.dark\:ring-orange-100\/55:is(.dark *){--tw-ring-color:rgba(255,237,213,.55)}.dark\:ring-orange-100\/60:is(.dark *){--tw-ring-color:rgba(255,237,213,.6)}.dark\:ring-orange-100\/65:is(.dark *){--tw-ring-color:rgba(255,237,213,.65)}.dark\:ring-orange-100\/70:is(.dark *){--tw-ring-color:rgba(255,237,213,.7)}.dark\:ring-orange-100\/75:is(.dark *){--tw-ring-color:rgba(255,237,213,.75)}.dark\:ring-orange-100\/80:is(.dark *){--tw-ring-color:rgba(255,237,213,.8)}.dark\:ring-orange-100\/85:is(.dark *){--tw-ring-color:rgba(255,237,213,.85)}.dark\:ring-orange-100\/90:is(.dark *){--tw-ring-color:rgba(255,237,213,.9)}.dark\:ring-orange-100\/95:is(.dark *){--tw-ring-color:rgba(255,237,213,.95)}.dark\:ring-orange-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(254 215 170/var(--tw-ring-opacity,1))}.dark\:ring-orange-200\/0:is(.dark *){--tw-ring-color:hsla(32,98%,83%,0)}.dark\:ring-orange-200\/10:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.1)}.dark\:ring-orange-200\/100:is(.dark *){--tw-ring-color:#fed7aa}.dark\:ring-orange-200\/15:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.15)}.dark\:ring-orange-200\/20:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.2)}.dark\:ring-orange-200\/25:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.25)}.dark\:ring-orange-200\/30:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.3)}.dark\:ring-orange-200\/35:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.35)}.dark\:ring-orange-200\/40:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.4)}.dark\:ring-orange-200\/45:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.45)}.dark\:ring-orange-200\/5:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.05)}.dark\:ring-orange-200\/50:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.5)}.dark\:ring-orange-200\/55:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.55)}.dark\:ring-orange-200\/60:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.6)}.dark\:ring-orange-200\/65:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.65)}.dark\:ring-orange-200\/70:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.7)}.dark\:ring-orange-200\/75:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.75)}.dark\:ring-orange-200\/80:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.8)}.dark\:ring-orange-200\/85:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.85)}.dark\:ring-orange-200\/90:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.9)}.dark\:ring-orange-200\/95:is(.dark *){--tw-ring-color:hsla(32,98%,83%,.95)}.dark\:ring-orange-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(251 146 60/var(--tw-ring-opacity,1))}.dark\:ring-orange-400\/0:is(.dark *){--tw-ring-color:rgba(251,146,60,0)}.dark\:ring-orange-400\/10:is(.dark *){--tw-ring-color:rgba(251,146,60,.1)}.dark\:ring-orange-400\/100:is(.dark *){--tw-ring-color:#fb923c}.dark\:ring-orange-400\/15:is(.dark *){--tw-ring-color:rgba(251,146,60,.15)}.dark\:ring-orange-400\/20:is(.dark *){--tw-ring-color:rgba(251,146,60,.2)}.dark\:ring-orange-400\/25:is(.dark *){--tw-ring-color:rgba(251,146,60,.25)}.dark\:ring-orange-400\/30:is(.dark *){--tw-ring-color:rgba(251,146,60,.3)}.dark\:ring-orange-400\/35:is(.dark *){--tw-ring-color:rgba(251,146,60,.35)}.dark\:ring-orange-400\/40:is(.dark *){--tw-ring-color:rgba(251,146,60,.4)}.dark\:ring-orange-400\/45:is(.dark *){--tw-ring-color:rgba(251,146,60,.45)}.dark\:ring-orange-400\/5:is(.dark *){--tw-ring-color:rgba(251,146,60,.05)}.dark\:ring-orange-400\/50:is(.dark *){--tw-ring-color:rgba(251,146,60,.5)}.dark\:ring-orange-400\/55:is(.dark *){--tw-ring-color:rgba(251,146,60,.55)}.dark\:ring-orange-400\/60:is(.dark *){--tw-ring-color:rgba(251,146,60,.6)}.dark\:ring-orange-400\/65:is(.dark *){--tw-ring-color:rgba(251,146,60,.65)}.dark\:ring-orange-400\/70:is(.dark *){--tw-ring-color:rgba(251,146,60,.7)}.dark\:ring-orange-400\/75:is(.dark *){--tw-ring-color:rgba(251,146,60,.75)}.dark\:ring-orange-400\/80:is(.dark *){--tw-ring-color:rgba(251,146,60,.8)}.dark\:ring-orange-400\/85:is(.dark *){--tw-ring-color:rgba(251,146,60,.85)}.dark\:ring-orange-400\/90:is(.dark *){--tw-ring-color:rgba(251,146,60,.9)}.dark\:ring-orange-400\/95:is(.dark *){--tw-ring-color:rgba(251,146,60,.95)}.dark\:ring-orange-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(255 247 237/var(--tw-ring-opacity,1))}.dark\:ring-orange-50\/0:is(.dark *){--tw-ring-color:rgba(255,247,237,0)}.dark\:ring-orange-50\/10:is(.dark *){--tw-ring-color:rgba(255,247,237,.1)}.dark\:ring-orange-50\/100:is(.dark *){--tw-ring-color:#fff7ed}.dark\:ring-orange-50\/15:is(.dark *){--tw-ring-color:rgba(255,247,237,.15)}.dark\:ring-orange-50\/20:is(.dark *){--tw-ring-color:rgba(255,247,237,.2)}.dark\:ring-orange-50\/25:is(.dark *){--tw-ring-color:rgba(255,247,237,.25)}.dark\:ring-orange-50\/30:is(.dark *){--tw-ring-color:rgba(255,247,237,.3)}.dark\:ring-orange-50\/35:is(.dark *){--tw-ring-color:rgba(255,247,237,.35)}.dark\:ring-orange-50\/40:is(.dark *){--tw-ring-color:rgba(255,247,237,.4)}.dark\:ring-orange-50\/45:is(.dark *){--tw-ring-color:rgba(255,247,237,.45)}.dark\:ring-orange-50\/5:is(.dark *){--tw-ring-color:rgba(255,247,237,.05)}.dark\:ring-orange-50\/50:is(.dark *){--tw-ring-color:rgba(255,247,237,.5)}.dark\:ring-orange-50\/55:is(.dark *){--tw-ring-color:rgba(255,247,237,.55)}.dark\:ring-orange-50\/60:is(.dark *){--tw-ring-color:rgba(255,247,237,.6)}.dark\:ring-orange-50\/65:is(.dark *){--tw-ring-color:rgba(255,247,237,.65)}.dark\:ring-orange-50\/70:is(.dark *){--tw-ring-color:rgba(255,247,237,.7)}.dark\:ring-orange-50\/75:is(.dark *){--tw-ring-color:rgba(255,247,237,.75)}.dark\:ring-orange-50\/80:is(.dark *){--tw-ring-color:rgba(255,247,237,.8)}.dark\:ring-orange-50\/85:is(.dark *){--tw-ring-color:rgba(255,247,237,.85)}.dark\:ring-orange-50\/90:is(.dark *){--tw-ring-color:rgba(255,247,237,.9)}.dark\:ring-orange-50\/95:is(.dark *){--tw-ring-color:rgba(255,247,237,.95)}.dark\:ring-orange-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(249 115 22/var(--tw-ring-opacity,1))}.dark\:ring-orange-500\/0:is(.dark *){--tw-ring-color:rgba(249,115,22,0)}.dark\:ring-orange-500\/10:is(.dark *){--tw-ring-color:rgba(249,115,22,.1)}.dark\:ring-orange-500\/100:is(.dark *){--tw-ring-color:#f97316}.dark\:ring-orange-500\/15:is(.dark *){--tw-ring-color:rgba(249,115,22,.15)}.dark\:ring-orange-500\/20:is(.dark *){--tw-ring-color:rgba(249,115,22,.2)}.dark\:ring-orange-500\/25:is(.dark *){--tw-ring-color:rgba(249,115,22,.25)}.dark\:ring-orange-500\/30:is(.dark *){--tw-ring-color:rgba(249,115,22,.3)}.dark\:ring-orange-500\/35:is(.dark *){--tw-ring-color:rgba(249,115,22,.35)}.dark\:ring-orange-500\/40:is(.dark *){--tw-ring-color:rgba(249,115,22,.4)}.dark\:ring-orange-500\/45:is(.dark *){--tw-ring-color:rgba(249,115,22,.45)}.dark\:ring-orange-500\/5:is(.dark *){--tw-ring-color:rgba(249,115,22,.05)}.dark\:ring-orange-500\/50:is(.dark *){--tw-ring-color:rgba(249,115,22,.5)}.dark\:ring-orange-500\/55:is(.dark *){--tw-ring-color:rgba(249,115,22,.55)}.dark\:ring-orange-500\/60:is(.dark *){--tw-ring-color:rgba(249,115,22,.6)}.dark\:ring-orange-500\/65:is(.dark *){--tw-ring-color:rgba(249,115,22,.65)}.dark\:ring-orange-500\/70:is(.dark *){--tw-ring-color:rgba(249,115,22,.7)}.dark\:ring-orange-500\/75:is(.dark *){--tw-ring-color:rgba(249,115,22,.75)}.dark\:ring-orange-500\/80:is(.dark *){--tw-ring-color:rgba(249,115,22,.8)}.dark\:ring-orange-500\/85:is(.dark *){--tw-ring-color:rgba(249,115,22,.85)}.dark\:ring-orange-500\/90:is(.dark *){--tw-ring-color:rgba(249,115,22,.9)}.dark\:ring-orange-500\/95:is(.dark *){--tw-ring-color:rgba(249,115,22,.95)}.dark\:ring-orange-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(67 20 7/var(--tw-ring-opacity,1))}.dark\:ring-orange-950\/0:is(.dark *){--tw-ring-color:rgba(67,20,7,0)}.dark\:ring-orange-950\/10:is(.dark *){--tw-ring-color:rgba(67,20,7,.1)}.dark\:ring-orange-950\/100:is(.dark *){--tw-ring-color:#431407}.dark\:ring-orange-950\/15:is(.dark *){--tw-ring-color:rgba(67,20,7,.15)}.dark\:ring-orange-950\/20:is(.dark *){--tw-ring-color:rgba(67,20,7,.2)}.dark\:ring-orange-950\/25:is(.dark *){--tw-ring-color:rgba(67,20,7,.25)}.dark\:ring-orange-950\/30:is(.dark *){--tw-ring-color:rgba(67,20,7,.3)}.dark\:ring-orange-950\/35:is(.dark *){--tw-ring-color:rgba(67,20,7,.35)}.dark\:ring-orange-950\/40:is(.dark *){--tw-ring-color:rgba(67,20,7,.4)}.dark\:ring-orange-950\/45:is(.dark *){--tw-ring-color:rgba(67,20,7,.45)}.dark\:ring-orange-950\/5:is(.dark *){--tw-ring-color:rgba(67,20,7,.05)}.dark\:ring-orange-950\/50:is(.dark *){--tw-ring-color:rgba(67,20,7,.5)}.dark\:ring-orange-950\/55:is(.dark *){--tw-ring-color:rgba(67,20,7,.55)}.dark\:ring-orange-950\/60:is(.dark *){--tw-ring-color:rgba(67,20,7,.6)}.dark\:ring-orange-950\/65:is(.dark *){--tw-ring-color:rgba(67,20,7,.65)}.dark\:ring-orange-950\/70:is(.dark *){--tw-ring-color:rgba(67,20,7,.7)}.dark\:ring-orange-950\/75:is(.dark *){--tw-ring-color:rgba(67,20,7,.75)}.dark\:ring-orange-950\/80:is(.dark *){--tw-ring-color:rgba(67,20,7,.8)}.dark\:ring-orange-950\/85:is(.dark *){--tw-ring-color:rgba(67,20,7,.85)}.dark\:ring-orange-950\/90:is(.dark *){--tw-ring-color:rgba(67,20,7,.9)}.dark\:ring-orange-950\/95:is(.dark *){--tw-ring-color:rgba(67,20,7,.95)}.dark\:ring-pink-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(252 231 243/var(--tw-ring-opacity,1))}.dark\:ring-pink-100\/0:is(.dark *){--tw-ring-color:rgba(252,231,243,0)}.dark\:ring-pink-100\/10:is(.dark *){--tw-ring-color:rgba(252,231,243,.1)}.dark\:ring-pink-100\/100:is(.dark *){--tw-ring-color:#fce7f3}.dark\:ring-pink-100\/15:is(.dark *){--tw-ring-color:rgba(252,231,243,.15)}.dark\:ring-pink-100\/20:is(.dark *){--tw-ring-color:rgba(252,231,243,.2)}.dark\:ring-pink-100\/25:is(.dark *){--tw-ring-color:rgba(252,231,243,.25)}.dark\:ring-pink-100\/30:is(.dark *){--tw-ring-color:rgba(252,231,243,.3)}.dark\:ring-pink-100\/35:is(.dark *){--tw-ring-color:rgba(252,231,243,.35)}.dark\:ring-pink-100\/40:is(.dark *){--tw-ring-color:rgba(252,231,243,.4)}.dark\:ring-pink-100\/45:is(.dark *){--tw-ring-color:rgba(252,231,243,.45)}.dark\:ring-pink-100\/5:is(.dark *){--tw-ring-color:rgba(252,231,243,.05)}.dark\:ring-pink-100\/50:is(.dark *){--tw-ring-color:rgba(252,231,243,.5)}.dark\:ring-pink-100\/55:is(.dark *){--tw-ring-color:rgba(252,231,243,.55)}.dark\:ring-pink-100\/60:is(.dark *){--tw-ring-color:rgba(252,231,243,.6)}.dark\:ring-pink-100\/65:is(.dark *){--tw-ring-color:rgba(252,231,243,.65)}.dark\:ring-pink-100\/70:is(.dark *){--tw-ring-color:rgba(252,231,243,.7)}.dark\:ring-pink-100\/75:is(.dark *){--tw-ring-color:rgba(252,231,243,.75)}.dark\:ring-pink-100\/80:is(.dark *){--tw-ring-color:rgba(252,231,243,.8)}.dark\:ring-pink-100\/85:is(.dark *){--tw-ring-color:rgba(252,231,243,.85)}.dark\:ring-pink-100\/90:is(.dark *){--tw-ring-color:rgba(252,231,243,.9)}.dark\:ring-pink-100\/95:is(.dark *){--tw-ring-color:rgba(252,231,243,.95)}.dark\:ring-pink-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(251 207 232/var(--tw-ring-opacity,1))}.dark\:ring-pink-200\/0:is(.dark *){--tw-ring-color:rgba(251,207,232,0)}.dark\:ring-pink-200\/10:is(.dark *){--tw-ring-color:rgba(251,207,232,.1)}.dark\:ring-pink-200\/100:is(.dark *){--tw-ring-color:#fbcfe8}.dark\:ring-pink-200\/15:is(.dark *){--tw-ring-color:rgba(251,207,232,.15)}.dark\:ring-pink-200\/20:is(.dark *){--tw-ring-color:rgba(251,207,232,.2)}.dark\:ring-pink-200\/25:is(.dark *){--tw-ring-color:rgba(251,207,232,.25)}.dark\:ring-pink-200\/30:is(.dark *){--tw-ring-color:rgba(251,207,232,.3)}.dark\:ring-pink-200\/35:is(.dark *){--tw-ring-color:rgba(251,207,232,.35)}.dark\:ring-pink-200\/40:is(.dark *){--tw-ring-color:rgba(251,207,232,.4)}.dark\:ring-pink-200\/45:is(.dark *){--tw-ring-color:rgba(251,207,232,.45)}.dark\:ring-pink-200\/5:is(.dark *){--tw-ring-color:rgba(251,207,232,.05)}.dark\:ring-pink-200\/50:is(.dark *){--tw-ring-color:rgba(251,207,232,.5)}.dark\:ring-pink-200\/55:is(.dark *){--tw-ring-color:rgba(251,207,232,.55)}.dark\:ring-pink-200\/60:is(.dark *){--tw-ring-color:rgba(251,207,232,.6)}.dark\:ring-pink-200\/65:is(.dark *){--tw-ring-color:rgba(251,207,232,.65)}.dark\:ring-pink-200\/70:is(.dark *){--tw-ring-color:rgba(251,207,232,.7)}.dark\:ring-pink-200\/75:is(.dark *){--tw-ring-color:rgba(251,207,232,.75)}.dark\:ring-pink-200\/80:is(.dark *){--tw-ring-color:rgba(251,207,232,.8)}.dark\:ring-pink-200\/85:is(.dark *){--tw-ring-color:rgba(251,207,232,.85)}.dark\:ring-pink-200\/90:is(.dark *){--tw-ring-color:rgba(251,207,232,.9)}.dark\:ring-pink-200\/95:is(.dark *){--tw-ring-color:rgba(251,207,232,.95)}.dark\:ring-pink-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(244 114 182/var(--tw-ring-opacity,1))}.dark\:ring-pink-400\/0:is(.dark *){--tw-ring-color:rgba(244,114,182,0)}.dark\:ring-pink-400\/10:is(.dark *){--tw-ring-color:rgba(244,114,182,.1)}.dark\:ring-pink-400\/100:is(.dark *){--tw-ring-color:#f472b6}.dark\:ring-pink-400\/15:is(.dark *){--tw-ring-color:rgba(244,114,182,.15)}.dark\:ring-pink-400\/20:is(.dark *){--tw-ring-color:rgba(244,114,182,.2)}.dark\:ring-pink-400\/25:is(.dark *){--tw-ring-color:rgba(244,114,182,.25)}.dark\:ring-pink-400\/30:is(.dark *){--tw-ring-color:rgba(244,114,182,.3)}.dark\:ring-pink-400\/35:is(.dark *){--tw-ring-color:rgba(244,114,182,.35)}.dark\:ring-pink-400\/40:is(.dark *){--tw-ring-color:rgba(244,114,182,.4)}.dark\:ring-pink-400\/45:is(.dark *){--tw-ring-color:rgba(244,114,182,.45)}.dark\:ring-pink-400\/5:is(.dark *){--tw-ring-color:rgba(244,114,182,.05)}.dark\:ring-pink-400\/50:is(.dark *){--tw-ring-color:rgba(244,114,182,.5)}.dark\:ring-pink-400\/55:is(.dark *){--tw-ring-color:rgba(244,114,182,.55)}.dark\:ring-pink-400\/60:is(.dark *){--tw-ring-color:rgba(244,114,182,.6)}.dark\:ring-pink-400\/65:is(.dark *){--tw-ring-color:rgba(244,114,182,.65)}.dark\:ring-pink-400\/70:is(.dark *){--tw-ring-color:rgba(244,114,182,.7)}.dark\:ring-pink-400\/75:is(.dark *){--tw-ring-color:rgba(244,114,182,.75)}.dark\:ring-pink-400\/80:is(.dark *){--tw-ring-color:rgba(244,114,182,.8)}.dark\:ring-pink-400\/85:is(.dark *){--tw-ring-color:rgba(244,114,182,.85)}.dark\:ring-pink-400\/90:is(.dark *){--tw-ring-color:rgba(244,114,182,.9)}.dark\:ring-pink-400\/95:is(.dark *){--tw-ring-color:rgba(244,114,182,.95)}.dark\:ring-pink-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(253 242 248/var(--tw-ring-opacity,1))}.dark\:ring-pink-50\/0:is(.dark *){--tw-ring-color:rgba(253,242,248,0)}.dark\:ring-pink-50\/10:is(.dark *){--tw-ring-color:rgba(253,242,248,.1)}.dark\:ring-pink-50\/100:is(.dark *){--tw-ring-color:#fdf2f8}.dark\:ring-pink-50\/15:is(.dark *){--tw-ring-color:rgba(253,242,248,.15)}.dark\:ring-pink-50\/20:is(.dark *){--tw-ring-color:rgba(253,242,248,.2)}.dark\:ring-pink-50\/25:is(.dark *){--tw-ring-color:rgba(253,242,248,.25)}.dark\:ring-pink-50\/30:is(.dark *){--tw-ring-color:rgba(253,242,248,.3)}.dark\:ring-pink-50\/35:is(.dark *){--tw-ring-color:rgba(253,242,248,.35)}.dark\:ring-pink-50\/40:is(.dark *){--tw-ring-color:rgba(253,242,248,.4)}.dark\:ring-pink-50\/45:is(.dark *){--tw-ring-color:rgba(253,242,248,.45)}.dark\:ring-pink-50\/5:is(.dark *){--tw-ring-color:rgba(253,242,248,.05)}.dark\:ring-pink-50\/50:is(.dark *){--tw-ring-color:rgba(253,242,248,.5)}.dark\:ring-pink-50\/55:is(.dark *){--tw-ring-color:rgba(253,242,248,.55)}.dark\:ring-pink-50\/60:is(.dark *){--tw-ring-color:rgba(253,242,248,.6)}.dark\:ring-pink-50\/65:is(.dark *){--tw-ring-color:rgba(253,242,248,.65)}.dark\:ring-pink-50\/70:is(.dark *){--tw-ring-color:rgba(253,242,248,.7)}.dark\:ring-pink-50\/75:is(.dark *){--tw-ring-color:rgba(253,242,248,.75)}.dark\:ring-pink-50\/80:is(.dark *){--tw-ring-color:rgba(253,242,248,.8)}.dark\:ring-pink-50\/85:is(.dark *){--tw-ring-color:rgba(253,242,248,.85)}.dark\:ring-pink-50\/90:is(.dark *){--tw-ring-color:rgba(253,242,248,.9)}.dark\:ring-pink-50\/95:is(.dark *){--tw-ring-color:rgba(253,242,248,.95)}.dark\:ring-pink-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(236 72 153/var(--tw-ring-opacity,1))}.dark\:ring-pink-500\/0:is(.dark *){--tw-ring-color:rgba(236,72,153,0)}.dark\:ring-pink-500\/10:is(.dark *){--tw-ring-color:rgba(236,72,153,.1)}.dark\:ring-pink-500\/100:is(.dark *){--tw-ring-color:#ec4899}.dark\:ring-pink-500\/15:is(.dark *){--tw-ring-color:rgba(236,72,153,.15)}.dark\:ring-pink-500\/20:is(.dark *){--tw-ring-color:rgba(236,72,153,.2)}.dark\:ring-pink-500\/25:is(.dark *){--tw-ring-color:rgba(236,72,153,.25)}.dark\:ring-pink-500\/30:is(.dark *){--tw-ring-color:rgba(236,72,153,.3)}.dark\:ring-pink-500\/35:is(.dark *){--tw-ring-color:rgba(236,72,153,.35)}.dark\:ring-pink-500\/40:is(.dark *){--tw-ring-color:rgba(236,72,153,.4)}.dark\:ring-pink-500\/45:is(.dark *){--tw-ring-color:rgba(236,72,153,.45)}.dark\:ring-pink-500\/5:is(.dark *){--tw-ring-color:rgba(236,72,153,.05)}.dark\:ring-pink-500\/50:is(.dark *){--tw-ring-color:rgba(236,72,153,.5)}.dark\:ring-pink-500\/55:is(.dark *){--tw-ring-color:rgba(236,72,153,.55)}.dark\:ring-pink-500\/60:is(.dark *){--tw-ring-color:rgba(236,72,153,.6)}.dark\:ring-pink-500\/65:is(.dark *){--tw-ring-color:rgba(236,72,153,.65)}.dark\:ring-pink-500\/70:is(.dark *){--tw-ring-color:rgba(236,72,153,.7)}.dark\:ring-pink-500\/75:is(.dark *){--tw-ring-color:rgba(236,72,153,.75)}.dark\:ring-pink-500\/80:is(.dark *){--tw-ring-color:rgba(236,72,153,.8)}.dark\:ring-pink-500\/85:is(.dark *){--tw-ring-color:rgba(236,72,153,.85)}.dark\:ring-pink-500\/90:is(.dark *){--tw-ring-color:rgba(236,72,153,.9)}.dark\:ring-pink-500\/95:is(.dark *){--tw-ring-color:rgba(236,72,153,.95)}.dark\:ring-pink-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(80 7 36/var(--tw-ring-opacity,1))}.dark\:ring-pink-950\/0:is(.dark *){--tw-ring-color:rgba(80,7,36,0)}.dark\:ring-pink-950\/10:is(.dark *){--tw-ring-color:rgba(80,7,36,.1)}.dark\:ring-pink-950\/100:is(.dark *){--tw-ring-color:#500724}.dark\:ring-pink-950\/15:is(.dark *){--tw-ring-color:rgba(80,7,36,.15)}.dark\:ring-pink-950\/20:is(.dark *){--tw-ring-color:rgba(80,7,36,.2)}.dark\:ring-pink-950\/25:is(.dark *){--tw-ring-color:rgba(80,7,36,.25)}.dark\:ring-pink-950\/30:is(.dark *){--tw-ring-color:rgba(80,7,36,.3)}.dark\:ring-pink-950\/35:is(.dark *){--tw-ring-color:rgba(80,7,36,.35)}.dark\:ring-pink-950\/40:is(.dark *){--tw-ring-color:rgba(80,7,36,.4)}.dark\:ring-pink-950\/45:is(.dark *){--tw-ring-color:rgba(80,7,36,.45)}.dark\:ring-pink-950\/5:is(.dark *){--tw-ring-color:rgba(80,7,36,.05)}.dark\:ring-pink-950\/50:is(.dark *){--tw-ring-color:rgba(80,7,36,.5)}.dark\:ring-pink-950\/55:is(.dark *){--tw-ring-color:rgba(80,7,36,.55)}.dark\:ring-pink-950\/60:is(.dark *){--tw-ring-color:rgba(80,7,36,.6)}.dark\:ring-pink-950\/65:is(.dark *){--tw-ring-color:rgba(80,7,36,.65)}.dark\:ring-pink-950\/70:is(.dark *){--tw-ring-color:rgba(80,7,36,.7)}.dark\:ring-pink-950\/75:is(.dark *){--tw-ring-color:rgba(80,7,36,.75)}.dark\:ring-pink-950\/80:is(.dark *){--tw-ring-color:rgba(80,7,36,.8)}.dark\:ring-pink-950\/85:is(.dark *){--tw-ring-color:rgba(80,7,36,.85)}.dark\:ring-pink-950\/90:is(.dark *){--tw-ring-color:rgba(80,7,36,.9)}.dark\:ring-pink-950\/95:is(.dark *){--tw-ring-color:rgba(80,7,36,.95)}.dark\:ring-purple-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(243 232 255/var(--tw-ring-opacity,1))}.dark\:ring-purple-100\/0:is(.dark *){--tw-ring-color:rgba(243,232,255,0)}.dark\:ring-purple-100\/10:is(.dark *){--tw-ring-color:rgba(243,232,255,.1)}.dark\:ring-purple-100\/100:is(.dark *){--tw-ring-color:#f3e8ff}.dark\:ring-purple-100\/15:is(.dark *){--tw-ring-color:rgba(243,232,255,.15)}.dark\:ring-purple-100\/20:is(.dark *){--tw-ring-color:rgba(243,232,255,.2)}.dark\:ring-purple-100\/25:is(.dark *){--tw-ring-color:rgba(243,232,255,.25)}.dark\:ring-purple-100\/30:is(.dark *){--tw-ring-color:rgba(243,232,255,.3)}.dark\:ring-purple-100\/35:is(.dark *){--tw-ring-color:rgba(243,232,255,.35)}.dark\:ring-purple-100\/40:is(.dark *){--tw-ring-color:rgba(243,232,255,.4)}.dark\:ring-purple-100\/45:is(.dark *){--tw-ring-color:rgba(243,232,255,.45)}.dark\:ring-purple-100\/5:is(.dark *){--tw-ring-color:rgba(243,232,255,.05)}.dark\:ring-purple-100\/50:is(.dark *){--tw-ring-color:rgba(243,232,255,.5)}.dark\:ring-purple-100\/55:is(.dark *){--tw-ring-color:rgba(243,232,255,.55)}.dark\:ring-purple-100\/60:is(.dark *){--tw-ring-color:rgba(243,232,255,.6)}.dark\:ring-purple-100\/65:is(.dark *){--tw-ring-color:rgba(243,232,255,.65)}.dark\:ring-purple-100\/70:is(.dark *){--tw-ring-color:rgba(243,232,255,.7)}.dark\:ring-purple-100\/75:is(.dark *){--tw-ring-color:rgba(243,232,255,.75)}.dark\:ring-purple-100\/80:is(.dark *){--tw-ring-color:rgba(243,232,255,.8)}.dark\:ring-purple-100\/85:is(.dark *){--tw-ring-color:rgba(243,232,255,.85)}.dark\:ring-purple-100\/90:is(.dark *){--tw-ring-color:rgba(243,232,255,.9)}.dark\:ring-purple-100\/95:is(.dark *){--tw-ring-color:rgba(243,232,255,.95)}.dark\:ring-purple-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(233 213 255/var(--tw-ring-opacity,1))}.dark\:ring-purple-200\/0:is(.dark *){--tw-ring-color:rgba(233,213,255,0)}.dark\:ring-purple-200\/10:is(.dark *){--tw-ring-color:rgba(233,213,255,.1)}.dark\:ring-purple-200\/100:is(.dark *){--tw-ring-color:#e9d5ff}.dark\:ring-purple-200\/15:is(.dark *){--tw-ring-color:rgba(233,213,255,.15)}.dark\:ring-purple-200\/20:is(.dark *){--tw-ring-color:rgba(233,213,255,.2)}.dark\:ring-purple-200\/25:is(.dark *){--tw-ring-color:rgba(233,213,255,.25)}.dark\:ring-purple-200\/30:is(.dark *){--tw-ring-color:rgba(233,213,255,.3)}.dark\:ring-purple-200\/35:is(.dark *){--tw-ring-color:rgba(233,213,255,.35)}.dark\:ring-purple-200\/40:is(.dark *){--tw-ring-color:rgba(233,213,255,.4)}.dark\:ring-purple-200\/45:is(.dark *){--tw-ring-color:rgba(233,213,255,.45)}.dark\:ring-purple-200\/5:is(.dark *){--tw-ring-color:rgba(233,213,255,.05)}.dark\:ring-purple-200\/50:is(.dark *){--tw-ring-color:rgba(233,213,255,.5)}.dark\:ring-purple-200\/55:is(.dark *){--tw-ring-color:rgba(233,213,255,.55)}.dark\:ring-purple-200\/60:is(.dark *){--tw-ring-color:rgba(233,213,255,.6)}.dark\:ring-purple-200\/65:is(.dark *){--tw-ring-color:rgba(233,213,255,.65)}.dark\:ring-purple-200\/70:is(.dark *){--tw-ring-color:rgba(233,213,255,.7)}.dark\:ring-purple-200\/75:is(.dark *){--tw-ring-color:rgba(233,213,255,.75)}.dark\:ring-purple-200\/80:is(.dark *){--tw-ring-color:rgba(233,213,255,.8)}.dark\:ring-purple-200\/85:is(.dark *){--tw-ring-color:rgba(233,213,255,.85)}.dark\:ring-purple-200\/90:is(.dark *){--tw-ring-color:rgba(233,213,255,.9)}.dark\:ring-purple-200\/95:is(.dark *){--tw-ring-color:rgba(233,213,255,.95)}.dark\:ring-purple-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(192 132 252/var(--tw-ring-opacity,1))}.dark\:ring-purple-400\/0:is(.dark *){--tw-ring-color:rgba(192,132,252,0)}.dark\:ring-purple-400\/10:is(.dark *){--tw-ring-color:rgba(192,132,252,.1)}.dark\:ring-purple-400\/100:is(.dark *){--tw-ring-color:#c084fc}.dark\:ring-purple-400\/15:is(.dark *){--tw-ring-color:rgba(192,132,252,.15)}.dark\:ring-purple-400\/20:is(.dark *){--tw-ring-color:rgba(192,132,252,.2)}.dark\:ring-purple-400\/25:is(.dark *){--tw-ring-color:rgba(192,132,252,.25)}.dark\:ring-purple-400\/30:is(.dark *){--tw-ring-color:rgba(192,132,252,.3)}.dark\:ring-purple-400\/35:is(.dark *){--tw-ring-color:rgba(192,132,252,.35)}.dark\:ring-purple-400\/40:is(.dark *){--tw-ring-color:rgba(192,132,252,.4)}.dark\:ring-purple-400\/45:is(.dark *){--tw-ring-color:rgba(192,132,252,.45)}.dark\:ring-purple-400\/5:is(.dark *){--tw-ring-color:rgba(192,132,252,.05)}.dark\:ring-purple-400\/50:is(.dark *){--tw-ring-color:rgba(192,132,252,.5)}.dark\:ring-purple-400\/55:is(.dark *){--tw-ring-color:rgba(192,132,252,.55)}.dark\:ring-purple-400\/60:is(.dark *){--tw-ring-color:rgba(192,132,252,.6)}.dark\:ring-purple-400\/65:is(.dark *){--tw-ring-color:rgba(192,132,252,.65)}.dark\:ring-purple-400\/70:is(.dark *){--tw-ring-color:rgba(192,132,252,.7)}.dark\:ring-purple-400\/75:is(.dark *){--tw-ring-color:rgba(192,132,252,.75)}.dark\:ring-purple-400\/80:is(.dark *){--tw-ring-color:rgba(192,132,252,.8)}.dark\:ring-purple-400\/85:is(.dark *){--tw-ring-color:rgba(192,132,252,.85)}.dark\:ring-purple-400\/90:is(.dark *){--tw-ring-color:rgba(192,132,252,.9)}.dark\:ring-purple-400\/95:is(.dark *){--tw-ring-color:rgba(192,132,252,.95)}.dark\:ring-purple-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(250 245 255/var(--tw-ring-opacity,1))}.dark\:ring-purple-50\/0:is(.dark *){--tw-ring-color:rgba(250,245,255,0)}.dark\:ring-purple-50\/10:is(.dark *){--tw-ring-color:rgba(250,245,255,.1)}.dark\:ring-purple-50\/100:is(.dark *){--tw-ring-color:#faf5ff}.dark\:ring-purple-50\/15:is(.dark *){--tw-ring-color:rgba(250,245,255,.15)}.dark\:ring-purple-50\/20:is(.dark *){--tw-ring-color:rgba(250,245,255,.2)}.dark\:ring-purple-50\/25:is(.dark *){--tw-ring-color:rgba(250,245,255,.25)}.dark\:ring-purple-50\/30:is(.dark *){--tw-ring-color:rgba(250,245,255,.3)}.dark\:ring-purple-50\/35:is(.dark *){--tw-ring-color:rgba(250,245,255,.35)}.dark\:ring-purple-50\/40:is(.dark *){--tw-ring-color:rgba(250,245,255,.4)}.dark\:ring-purple-50\/45:is(.dark *){--tw-ring-color:rgba(250,245,255,.45)}.dark\:ring-purple-50\/5:is(.dark *){--tw-ring-color:rgba(250,245,255,.05)}.dark\:ring-purple-50\/50:is(.dark *){--tw-ring-color:rgba(250,245,255,.5)}.dark\:ring-purple-50\/55:is(.dark *){--tw-ring-color:rgba(250,245,255,.55)}.dark\:ring-purple-50\/60:is(.dark *){--tw-ring-color:rgba(250,245,255,.6)}.dark\:ring-purple-50\/65:is(.dark *){--tw-ring-color:rgba(250,245,255,.65)}.dark\:ring-purple-50\/70:is(.dark *){--tw-ring-color:rgba(250,245,255,.7)}.dark\:ring-purple-50\/75:is(.dark *){--tw-ring-color:rgba(250,245,255,.75)}.dark\:ring-purple-50\/80:is(.dark *){--tw-ring-color:rgba(250,245,255,.8)}.dark\:ring-purple-50\/85:is(.dark *){--tw-ring-color:rgba(250,245,255,.85)}.dark\:ring-purple-50\/90:is(.dark *){--tw-ring-color:rgba(250,245,255,.9)}.dark\:ring-purple-50\/95:is(.dark *){--tw-ring-color:rgba(250,245,255,.95)}.dark\:ring-purple-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(168 85 247/var(--tw-ring-opacity,1))}.dark\:ring-purple-500\/0:is(.dark *){--tw-ring-color:rgba(168,85,247,0)}.dark\:ring-purple-500\/10:is(.dark *){--tw-ring-color:rgba(168,85,247,.1)}.dark\:ring-purple-500\/100:is(.dark *){--tw-ring-color:#a855f7}.dark\:ring-purple-500\/15:is(.dark *){--tw-ring-color:rgba(168,85,247,.15)}.dark\:ring-purple-500\/20:is(.dark *){--tw-ring-color:rgba(168,85,247,.2)}.dark\:ring-purple-500\/25:is(.dark *){--tw-ring-color:rgba(168,85,247,.25)}.dark\:ring-purple-500\/30:is(.dark *){--tw-ring-color:rgba(168,85,247,.3)}.dark\:ring-purple-500\/35:is(.dark *){--tw-ring-color:rgba(168,85,247,.35)}.dark\:ring-purple-500\/40:is(.dark *){--tw-ring-color:rgba(168,85,247,.4)}.dark\:ring-purple-500\/45:is(.dark *){--tw-ring-color:rgba(168,85,247,.45)}.dark\:ring-purple-500\/5:is(.dark *){--tw-ring-color:rgba(168,85,247,.05)}.dark\:ring-purple-500\/50:is(.dark *){--tw-ring-color:rgba(168,85,247,.5)}.dark\:ring-purple-500\/55:is(.dark *){--tw-ring-color:rgba(168,85,247,.55)}.dark\:ring-purple-500\/60:is(.dark *){--tw-ring-color:rgba(168,85,247,.6)}.dark\:ring-purple-500\/65:is(.dark *){--tw-ring-color:rgba(168,85,247,.65)}.dark\:ring-purple-500\/70:is(.dark *){--tw-ring-color:rgba(168,85,247,.7)}.dark\:ring-purple-500\/75:is(.dark *){--tw-ring-color:rgba(168,85,247,.75)}.dark\:ring-purple-500\/80:is(.dark *){--tw-ring-color:rgba(168,85,247,.8)}.dark\:ring-purple-500\/85:is(.dark *){--tw-ring-color:rgba(168,85,247,.85)}.dark\:ring-purple-500\/90:is(.dark *){--tw-ring-color:rgba(168,85,247,.9)}.dark\:ring-purple-500\/95:is(.dark *){--tw-ring-color:rgba(168,85,247,.95)}.dark\:ring-purple-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(59 7 100/var(--tw-ring-opacity,1))}.dark\:ring-purple-950\/0:is(.dark *){--tw-ring-color:rgba(59,7,100,0)}.dark\:ring-purple-950\/10:is(.dark *){--tw-ring-color:rgba(59,7,100,.1)}.dark\:ring-purple-950\/100:is(.dark *){--tw-ring-color:#3b0764}.dark\:ring-purple-950\/15:is(.dark *){--tw-ring-color:rgba(59,7,100,.15)}.dark\:ring-purple-950\/20:is(.dark *){--tw-ring-color:rgba(59,7,100,.2)}.dark\:ring-purple-950\/25:is(.dark *){--tw-ring-color:rgba(59,7,100,.25)}.dark\:ring-purple-950\/30:is(.dark *){--tw-ring-color:rgba(59,7,100,.3)}.dark\:ring-purple-950\/35:is(.dark *){--tw-ring-color:rgba(59,7,100,.35)}.dark\:ring-purple-950\/40:is(.dark *){--tw-ring-color:rgba(59,7,100,.4)}.dark\:ring-purple-950\/45:is(.dark *){--tw-ring-color:rgba(59,7,100,.45)}.dark\:ring-purple-950\/5:is(.dark *){--tw-ring-color:rgba(59,7,100,.05)}.dark\:ring-purple-950\/50:is(.dark *){--tw-ring-color:rgba(59,7,100,.5)}.dark\:ring-purple-950\/55:is(.dark *){--tw-ring-color:rgba(59,7,100,.55)}.dark\:ring-purple-950\/60:is(.dark *){--tw-ring-color:rgba(59,7,100,.6)}.dark\:ring-purple-950\/65:is(.dark *){--tw-ring-color:rgba(59,7,100,.65)}.dark\:ring-purple-950\/70:is(.dark *){--tw-ring-color:rgba(59,7,100,.7)}.dark\:ring-purple-950\/75:is(.dark *){--tw-ring-color:rgba(59,7,100,.75)}.dark\:ring-purple-950\/80:is(.dark *){--tw-ring-color:rgba(59,7,100,.8)}.dark\:ring-purple-950\/85:is(.dark *){--tw-ring-color:rgba(59,7,100,.85)}.dark\:ring-purple-950\/90:is(.dark *){--tw-ring-color:rgba(59,7,100,.9)}.dark\:ring-purple-950\/95:is(.dark *){--tw-ring-color:rgba(59,7,100,.95)}.dark\:ring-red-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(254 226 226/var(--tw-ring-opacity,1))}.dark\:ring-red-100\/0:is(.dark *){--tw-ring-color:hsla(0,93%,94%,0)}.dark\:ring-red-100\/10:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.1)}.dark\:ring-red-100\/100:is(.dark *){--tw-ring-color:#fee2e2}.dark\:ring-red-100\/15:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.15)}.dark\:ring-red-100\/20:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.2)}.dark\:ring-red-100\/25:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.25)}.dark\:ring-red-100\/30:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.3)}.dark\:ring-red-100\/35:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.35)}.dark\:ring-red-100\/40:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.4)}.dark\:ring-red-100\/45:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.45)}.dark\:ring-red-100\/5:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.05)}.dark\:ring-red-100\/50:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.5)}.dark\:ring-red-100\/55:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.55)}.dark\:ring-red-100\/60:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.6)}.dark\:ring-red-100\/65:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.65)}.dark\:ring-red-100\/70:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.7)}.dark\:ring-red-100\/75:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.75)}.dark\:ring-red-100\/80:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.8)}.dark\:ring-red-100\/85:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.85)}.dark\:ring-red-100\/90:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.9)}.dark\:ring-red-100\/95:is(.dark *){--tw-ring-color:hsla(0,93%,94%,.95)}.dark\:ring-red-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(254 202 202/var(--tw-ring-opacity,1))}.dark\:ring-red-200\/0:is(.dark *){--tw-ring-color:hsla(0,96%,89%,0)}.dark\:ring-red-200\/10:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.1)}.dark\:ring-red-200\/100:is(.dark *){--tw-ring-color:#fecaca}.dark\:ring-red-200\/15:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.15)}.dark\:ring-red-200\/20:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.2)}.dark\:ring-red-200\/25:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.25)}.dark\:ring-red-200\/30:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.3)}.dark\:ring-red-200\/35:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.35)}.dark\:ring-red-200\/40:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.4)}.dark\:ring-red-200\/45:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.45)}.dark\:ring-red-200\/5:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.05)}.dark\:ring-red-200\/50:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.5)}.dark\:ring-red-200\/55:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.55)}.dark\:ring-red-200\/60:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.6)}.dark\:ring-red-200\/65:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.65)}.dark\:ring-red-200\/70:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.7)}.dark\:ring-red-200\/75:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.75)}.dark\:ring-red-200\/80:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.8)}.dark\:ring-red-200\/85:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.85)}.dark\:ring-red-200\/90:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.9)}.dark\:ring-red-200\/95:is(.dark *){--tw-ring-color:hsla(0,96%,89%,.95)}.dark\:ring-red-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(248 113 113/var(--tw-ring-opacity,1))}.dark\:ring-red-400\/0:is(.dark *){--tw-ring-color:hsla(0,91%,71%,0)}.dark\:ring-red-400\/10:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.1)}.dark\:ring-red-400\/100:is(.dark *){--tw-ring-color:#f87171}.dark\:ring-red-400\/15:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.15)}.dark\:ring-red-400\/20:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.2)}.dark\:ring-red-400\/25:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.25)}.dark\:ring-red-400\/30:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.3)}.dark\:ring-red-400\/35:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.35)}.dark\:ring-red-400\/40:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.4)}.dark\:ring-red-400\/45:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.45)}.dark\:ring-red-400\/5:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.05)}.dark\:ring-red-400\/50:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.5)}.dark\:ring-red-400\/55:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.55)}.dark\:ring-red-400\/60:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.6)}.dark\:ring-red-400\/65:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.65)}.dark\:ring-red-400\/70:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.7)}.dark\:ring-red-400\/75:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.75)}.dark\:ring-red-400\/80:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.8)}.dark\:ring-red-400\/85:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.85)}.dark\:ring-red-400\/90:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.9)}.dark\:ring-red-400\/95:is(.dark *){--tw-ring-color:hsla(0,91%,71%,.95)}.dark\:ring-red-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(254 242 242/var(--tw-ring-opacity,1))}.dark\:ring-red-50\/0:is(.dark *){--tw-ring-color:hsla(0,86%,97%,0)}.dark\:ring-red-50\/10:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.1)}.dark\:ring-red-50\/100:is(.dark *){--tw-ring-color:#fef2f2}.dark\:ring-red-50\/15:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.15)}.dark\:ring-red-50\/20:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.2)}.dark\:ring-red-50\/25:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.25)}.dark\:ring-red-50\/30:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.3)}.dark\:ring-red-50\/35:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.35)}.dark\:ring-red-50\/40:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.4)}.dark\:ring-red-50\/45:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.45)}.dark\:ring-red-50\/5:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.05)}.dark\:ring-red-50\/50:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.5)}.dark\:ring-red-50\/55:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.55)}.dark\:ring-red-50\/60:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.6)}.dark\:ring-red-50\/65:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.65)}.dark\:ring-red-50\/70:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.7)}.dark\:ring-red-50\/75:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.75)}.dark\:ring-red-50\/80:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.8)}.dark\:ring-red-50\/85:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.85)}.dark\:ring-red-50\/90:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.9)}.dark\:ring-red-50\/95:is(.dark *){--tw-ring-color:hsla(0,86%,97%,.95)}.dark\:ring-red-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(239 68 68/var(--tw-ring-opacity,1))}.dark\:ring-red-500\/0:is(.dark *){--tw-ring-color:rgba(239,68,68,0)}.dark\:ring-red-500\/10:is(.dark *){--tw-ring-color:rgba(239,68,68,.1)}.dark\:ring-red-500\/100:is(.dark *){--tw-ring-color:#ef4444}.dark\:ring-red-500\/15:is(.dark *){--tw-ring-color:rgba(239,68,68,.15)}.dark\:ring-red-500\/20:is(.dark *){--tw-ring-color:rgba(239,68,68,.2)}.dark\:ring-red-500\/25:is(.dark *){--tw-ring-color:rgba(239,68,68,.25)}.dark\:ring-red-500\/30:is(.dark *){--tw-ring-color:rgba(239,68,68,.3)}.dark\:ring-red-500\/35:is(.dark *){--tw-ring-color:rgba(239,68,68,.35)}.dark\:ring-red-500\/40:is(.dark *){--tw-ring-color:rgba(239,68,68,.4)}.dark\:ring-red-500\/45:is(.dark *){--tw-ring-color:rgba(239,68,68,.45)}.dark\:ring-red-500\/5:is(.dark *){--tw-ring-color:rgba(239,68,68,.05)}.dark\:ring-red-500\/50:is(.dark *){--tw-ring-color:rgba(239,68,68,.5)}.dark\:ring-red-500\/55:is(.dark *){--tw-ring-color:rgba(239,68,68,.55)}.dark\:ring-red-500\/60:is(.dark *){--tw-ring-color:rgba(239,68,68,.6)}.dark\:ring-red-500\/65:is(.dark *){--tw-ring-color:rgba(239,68,68,.65)}.dark\:ring-red-500\/70:is(.dark *){--tw-ring-color:rgba(239,68,68,.7)}.dark\:ring-red-500\/75:is(.dark *){--tw-ring-color:rgba(239,68,68,.75)}.dark\:ring-red-500\/80:is(.dark *){--tw-ring-color:rgba(239,68,68,.8)}.dark\:ring-red-500\/85:is(.dark *){--tw-ring-color:rgba(239,68,68,.85)}.dark\:ring-red-500\/90:is(.dark *){--tw-ring-color:rgba(239,68,68,.9)}.dark\:ring-red-500\/95:is(.dark *){--tw-ring-color:rgba(239,68,68,.95)}.dark\:ring-red-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(69 10 10/var(--tw-ring-opacity,1))}.dark\:ring-red-950\/0:is(.dark *){--tw-ring-color:rgba(69,10,10,0)}.dark\:ring-red-950\/10:is(.dark *){--tw-ring-color:rgba(69,10,10,.1)}.dark\:ring-red-950\/100:is(.dark *){--tw-ring-color:#450a0a}.dark\:ring-red-950\/15:is(.dark *){--tw-ring-color:rgba(69,10,10,.15)}.dark\:ring-red-950\/20:is(.dark *){--tw-ring-color:rgba(69,10,10,.2)}.dark\:ring-red-950\/25:is(.dark *){--tw-ring-color:rgba(69,10,10,.25)}.dark\:ring-red-950\/30:is(.dark *){--tw-ring-color:rgba(69,10,10,.3)}.dark\:ring-red-950\/35:is(.dark *){--tw-ring-color:rgba(69,10,10,.35)}.dark\:ring-red-950\/40:is(.dark *){--tw-ring-color:rgba(69,10,10,.4)}.dark\:ring-red-950\/45:is(.dark *){--tw-ring-color:rgba(69,10,10,.45)}.dark\:ring-red-950\/5:is(.dark *){--tw-ring-color:rgba(69,10,10,.05)}.dark\:ring-red-950\/50:is(.dark *){--tw-ring-color:rgba(69,10,10,.5)}.dark\:ring-red-950\/55:is(.dark *){--tw-ring-color:rgba(69,10,10,.55)}.dark\:ring-red-950\/60:is(.dark *){--tw-ring-color:rgba(69,10,10,.6)}.dark\:ring-red-950\/65:is(.dark *){--tw-ring-color:rgba(69,10,10,.65)}.dark\:ring-red-950\/70:is(.dark *){--tw-ring-color:rgba(69,10,10,.7)}.dark\:ring-red-950\/75:is(.dark *){--tw-ring-color:rgba(69,10,10,.75)}.dark\:ring-red-950\/80:is(.dark *){--tw-ring-color:rgba(69,10,10,.8)}.dark\:ring-red-950\/85:is(.dark *){--tw-ring-color:rgba(69,10,10,.85)}.dark\:ring-red-950\/90:is(.dark *){--tw-ring-color:rgba(69,10,10,.9)}.dark\:ring-red-950\/95:is(.dark *){--tw-ring-color:rgba(69,10,10,.95)}.dark\:ring-rose-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(255 228 230/var(--tw-ring-opacity,1))}.dark\:ring-rose-100\/0:is(.dark *){--tw-ring-color:rgba(255,228,230,0)}.dark\:ring-rose-100\/10:is(.dark *){--tw-ring-color:rgba(255,228,230,.1)}.dark\:ring-rose-100\/100:is(.dark *){--tw-ring-color:#ffe4e6}.dark\:ring-rose-100\/15:is(.dark *){--tw-ring-color:rgba(255,228,230,.15)}.dark\:ring-rose-100\/20:is(.dark *){--tw-ring-color:rgba(255,228,230,.2)}.dark\:ring-rose-100\/25:is(.dark *){--tw-ring-color:rgba(255,228,230,.25)}.dark\:ring-rose-100\/30:is(.dark *){--tw-ring-color:rgba(255,228,230,.3)}.dark\:ring-rose-100\/35:is(.dark *){--tw-ring-color:rgba(255,228,230,.35)}.dark\:ring-rose-100\/40:is(.dark *){--tw-ring-color:rgba(255,228,230,.4)}.dark\:ring-rose-100\/45:is(.dark *){--tw-ring-color:rgba(255,228,230,.45)}.dark\:ring-rose-100\/5:is(.dark *){--tw-ring-color:rgba(255,228,230,.05)}.dark\:ring-rose-100\/50:is(.dark *){--tw-ring-color:rgba(255,228,230,.5)}.dark\:ring-rose-100\/55:is(.dark *){--tw-ring-color:rgba(255,228,230,.55)}.dark\:ring-rose-100\/60:is(.dark *){--tw-ring-color:rgba(255,228,230,.6)}.dark\:ring-rose-100\/65:is(.dark *){--tw-ring-color:rgba(255,228,230,.65)}.dark\:ring-rose-100\/70:is(.dark *){--tw-ring-color:rgba(255,228,230,.7)}.dark\:ring-rose-100\/75:is(.dark *){--tw-ring-color:rgba(255,228,230,.75)}.dark\:ring-rose-100\/80:is(.dark *){--tw-ring-color:rgba(255,228,230,.8)}.dark\:ring-rose-100\/85:is(.dark *){--tw-ring-color:rgba(255,228,230,.85)}.dark\:ring-rose-100\/90:is(.dark *){--tw-ring-color:rgba(255,228,230,.9)}.dark\:ring-rose-100\/95:is(.dark *){--tw-ring-color:rgba(255,228,230,.95)}.dark\:ring-rose-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(254 205 211/var(--tw-ring-opacity,1))}.dark\:ring-rose-200\/0:is(.dark *){--tw-ring-color:rgba(254,205,211,0)}.dark\:ring-rose-200\/10:is(.dark *){--tw-ring-color:rgba(254,205,211,.1)}.dark\:ring-rose-200\/100:is(.dark *){--tw-ring-color:#fecdd3}.dark\:ring-rose-200\/15:is(.dark *){--tw-ring-color:rgba(254,205,211,.15)}.dark\:ring-rose-200\/20:is(.dark *){--tw-ring-color:rgba(254,205,211,.2)}.dark\:ring-rose-200\/25:is(.dark *){--tw-ring-color:rgba(254,205,211,.25)}.dark\:ring-rose-200\/30:is(.dark *){--tw-ring-color:rgba(254,205,211,.3)}.dark\:ring-rose-200\/35:is(.dark *){--tw-ring-color:rgba(254,205,211,.35)}.dark\:ring-rose-200\/40:is(.dark *){--tw-ring-color:rgba(254,205,211,.4)}.dark\:ring-rose-200\/45:is(.dark *){--tw-ring-color:rgba(254,205,211,.45)}.dark\:ring-rose-200\/5:is(.dark *){--tw-ring-color:rgba(254,205,211,.05)}.dark\:ring-rose-200\/50:is(.dark *){--tw-ring-color:rgba(254,205,211,.5)}.dark\:ring-rose-200\/55:is(.dark *){--tw-ring-color:rgba(254,205,211,.55)}.dark\:ring-rose-200\/60:is(.dark *){--tw-ring-color:rgba(254,205,211,.6)}.dark\:ring-rose-200\/65:is(.dark *){--tw-ring-color:rgba(254,205,211,.65)}.dark\:ring-rose-200\/70:is(.dark *){--tw-ring-color:rgba(254,205,211,.7)}.dark\:ring-rose-200\/75:is(.dark *){--tw-ring-color:rgba(254,205,211,.75)}.dark\:ring-rose-200\/80:is(.dark *){--tw-ring-color:rgba(254,205,211,.8)}.dark\:ring-rose-200\/85:is(.dark *){--tw-ring-color:rgba(254,205,211,.85)}.dark\:ring-rose-200\/90:is(.dark *){--tw-ring-color:rgba(254,205,211,.9)}.dark\:ring-rose-200\/95:is(.dark *){--tw-ring-color:rgba(254,205,211,.95)}.dark\:ring-rose-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(251 113 133/var(--tw-ring-opacity,1))}.dark\:ring-rose-400\/0:is(.dark *){--tw-ring-color:rgba(251,113,133,0)}.dark\:ring-rose-400\/10:is(.dark *){--tw-ring-color:rgba(251,113,133,.1)}.dark\:ring-rose-400\/100:is(.dark *){--tw-ring-color:#fb7185}.dark\:ring-rose-400\/15:is(.dark *){--tw-ring-color:rgba(251,113,133,.15)}.dark\:ring-rose-400\/20:is(.dark *){--tw-ring-color:rgba(251,113,133,.2)}.dark\:ring-rose-400\/25:is(.dark *){--tw-ring-color:rgba(251,113,133,.25)}.dark\:ring-rose-400\/30:is(.dark *){--tw-ring-color:rgba(251,113,133,.3)}.dark\:ring-rose-400\/35:is(.dark *){--tw-ring-color:rgba(251,113,133,.35)}.dark\:ring-rose-400\/40:is(.dark *){--tw-ring-color:rgba(251,113,133,.4)}.dark\:ring-rose-400\/45:is(.dark *){--tw-ring-color:rgba(251,113,133,.45)}.dark\:ring-rose-400\/5:is(.dark *){--tw-ring-color:rgba(251,113,133,.05)}.dark\:ring-rose-400\/50:is(.dark *){--tw-ring-color:rgba(251,113,133,.5)}.dark\:ring-rose-400\/55:is(.dark *){--tw-ring-color:rgba(251,113,133,.55)}.dark\:ring-rose-400\/60:is(.dark *){--tw-ring-color:rgba(251,113,133,.6)}.dark\:ring-rose-400\/65:is(.dark *){--tw-ring-color:rgba(251,113,133,.65)}.dark\:ring-rose-400\/70:is(.dark *){--tw-ring-color:rgba(251,113,133,.7)}.dark\:ring-rose-400\/75:is(.dark *){--tw-ring-color:rgba(251,113,133,.75)}.dark\:ring-rose-400\/80:is(.dark *){--tw-ring-color:rgba(251,113,133,.8)}.dark\:ring-rose-400\/85:is(.dark *){--tw-ring-color:rgba(251,113,133,.85)}.dark\:ring-rose-400\/90:is(.dark *){--tw-ring-color:rgba(251,113,133,.9)}.dark\:ring-rose-400\/95:is(.dark *){--tw-ring-color:rgba(251,113,133,.95)}.dark\:ring-rose-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(255 241 242/var(--tw-ring-opacity,1))}.dark\:ring-rose-50\/0:is(.dark *){--tw-ring-color:rgba(255,241,242,0)}.dark\:ring-rose-50\/10:is(.dark *){--tw-ring-color:rgba(255,241,242,.1)}.dark\:ring-rose-50\/100:is(.dark *){--tw-ring-color:#fff1f2}.dark\:ring-rose-50\/15:is(.dark *){--tw-ring-color:rgba(255,241,242,.15)}.dark\:ring-rose-50\/20:is(.dark *){--tw-ring-color:rgba(255,241,242,.2)}.dark\:ring-rose-50\/25:is(.dark *){--tw-ring-color:rgba(255,241,242,.25)}.dark\:ring-rose-50\/30:is(.dark *){--tw-ring-color:rgba(255,241,242,.3)}.dark\:ring-rose-50\/35:is(.dark *){--tw-ring-color:rgba(255,241,242,.35)}.dark\:ring-rose-50\/40:is(.dark *){--tw-ring-color:rgba(255,241,242,.4)}.dark\:ring-rose-50\/45:is(.dark *){--tw-ring-color:rgba(255,241,242,.45)}.dark\:ring-rose-50\/5:is(.dark *){--tw-ring-color:rgba(255,241,242,.05)}.dark\:ring-rose-50\/50:is(.dark *){--tw-ring-color:rgba(255,241,242,.5)}.dark\:ring-rose-50\/55:is(.dark *){--tw-ring-color:rgba(255,241,242,.55)}.dark\:ring-rose-50\/60:is(.dark *){--tw-ring-color:rgba(255,241,242,.6)}.dark\:ring-rose-50\/65:is(.dark *){--tw-ring-color:rgba(255,241,242,.65)}.dark\:ring-rose-50\/70:is(.dark *){--tw-ring-color:rgba(255,241,242,.7)}.dark\:ring-rose-50\/75:is(.dark *){--tw-ring-color:rgba(255,241,242,.75)}.dark\:ring-rose-50\/80:is(.dark *){--tw-ring-color:rgba(255,241,242,.8)}.dark\:ring-rose-50\/85:is(.dark *){--tw-ring-color:rgba(255,241,242,.85)}.dark\:ring-rose-50\/90:is(.dark *){--tw-ring-color:rgba(255,241,242,.9)}.dark\:ring-rose-50\/95:is(.dark *){--tw-ring-color:rgba(255,241,242,.95)}.dark\:ring-rose-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(244 63 94/var(--tw-ring-opacity,1))}.dark\:ring-rose-500\/0:is(.dark *){--tw-ring-color:rgba(244,63,94,0)}.dark\:ring-rose-500\/10:is(.dark *){--tw-ring-color:rgba(244,63,94,.1)}.dark\:ring-rose-500\/100:is(.dark *){--tw-ring-color:#f43f5e}.dark\:ring-rose-500\/15:is(.dark *){--tw-ring-color:rgba(244,63,94,.15)}.dark\:ring-rose-500\/20:is(.dark *){--tw-ring-color:rgba(244,63,94,.2)}.dark\:ring-rose-500\/25:is(.dark *){--tw-ring-color:rgba(244,63,94,.25)}.dark\:ring-rose-500\/30:is(.dark *){--tw-ring-color:rgba(244,63,94,.3)}.dark\:ring-rose-500\/35:is(.dark *){--tw-ring-color:rgba(244,63,94,.35)}.dark\:ring-rose-500\/40:is(.dark *){--tw-ring-color:rgba(244,63,94,.4)}.dark\:ring-rose-500\/45:is(.dark *){--tw-ring-color:rgba(244,63,94,.45)}.dark\:ring-rose-500\/5:is(.dark *){--tw-ring-color:rgba(244,63,94,.05)}.dark\:ring-rose-500\/50:is(.dark *){--tw-ring-color:rgba(244,63,94,.5)}.dark\:ring-rose-500\/55:is(.dark *){--tw-ring-color:rgba(244,63,94,.55)}.dark\:ring-rose-500\/60:is(.dark *){--tw-ring-color:rgba(244,63,94,.6)}.dark\:ring-rose-500\/65:is(.dark *){--tw-ring-color:rgba(244,63,94,.65)}.dark\:ring-rose-500\/70:is(.dark *){--tw-ring-color:rgba(244,63,94,.7)}.dark\:ring-rose-500\/75:is(.dark *){--tw-ring-color:rgba(244,63,94,.75)}.dark\:ring-rose-500\/80:is(.dark *){--tw-ring-color:rgba(244,63,94,.8)}.dark\:ring-rose-500\/85:is(.dark *){--tw-ring-color:rgba(244,63,94,.85)}.dark\:ring-rose-500\/90:is(.dark *){--tw-ring-color:rgba(244,63,94,.9)}.dark\:ring-rose-500\/95:is(.dark *){--tw-ring-color:rgba(244,63,94,.95)}.dark\:ring-rose-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(76 5 25/var(--tw-ring-opacity,1))}.dark\:ring-rose-950\/0:is(.dark *){--tw-ring-color:rgba(76,5,25,0)}.dark\:ring-rose-950\/10:is(.dark *){--tw-ring-color:rgba(76,5,25,.1)}.dark\:ring-rose-950\/100:is(.dark *){--tw-ring-color:#4c0519}.dark\:ring-rose-950\/15:is(.dark *){--tw-ring-color:rgba(76,5,25,.15)}.dark\:ring-rose-950\/20:is(.dark *){--tw-ring-color:rgba(76,5,25,.2)}.dark\:ring-rose-950\/25:is(.dark *){--tw-ring-color:rgba(76,5,25,.25)}.dark\:ring-rose-950\/30:is(.dark *){--tw-ring-color:rgba(76,5,25,.3)}.dark\:ring-rose-950\/35:is(.dark *){--tw-ring-color:rgba(76,5,25,.35)}.dark\:ring-rose-950\/40:is(.dark *){--tw-ring-color:rgba(76,5,25,.4)}.dark\:ring-rose-950\/45:is(.dark *){--tw-ring-color:rgba(76,5,25,.45)}.dark\:ring-rose-950\/5:is(.dark *){--tw-ring-color:rgba(76,5,25,.05)}.dark\:ring-rose-950\/50:is(.dark *){--tw-ring-color:rgba(76,5,25,.5)}.dark\:ring-rose-950\/55:is(.dark *){--tw-ring-color:rgba(76,5,25,.55)}.dark\:ring-rose-950\/60:is(.dark *){--tw-ring-color:rgba(76,5,25,.6)}.dark\:ring-rose-950\/65:is(.dark *){--tw-ring-color:rgba(76,5,25,.65)}.dark\:ring-rose-950\/70:is(.dark *){--tw-ring-color:rgba(76,5,25,.7)}.dark\:ring-rose-950\/75:is(.dark *){--tw-ring-color:rgba(76,5,25,.75)}.dark\:ring-rose-950\/80:is(.dark *){--tw-ring-color:rgba(76,5,25,.8)}.dark\:ring-rose-950\/85:is(.dark *){--tw-ring-color:rgba(76,5,25,.85)}.dark\:ring-rose-950\/90:is(.dark *){--tw-ring-color:rgba(76,5,25,.9)}.dark\:ring-rose-950\/95:is(.dark *){--tw-ring-color:rgba(76,5,25,.95)}.dark\:ring-sky-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(224 242 254/var(--tw-ring-opacity,1))}.dark\:ring-sky-100\/0:is(.dark *){--tw-ring-color:rgba(224,242,254,0)}.dark\:ring-sky-100\/10:is(.dark *){--tw-ring-color:rgba(224,242,254,.1)}.dark\:ring-sky-100\/100:is(.dark *){--tw-ring-color:#e0f2fe}.dark\:ring-sky-100\/15:is(.dark *){--tw-ring-color:rgba(224,242,254,.15)}.dark\:ring-sky-100\/20:is(.dark *){--tw-ring-color:rgba(224,242,254,.2)}.dark\:ring-sky-100\/25:is(.dark *){--tw-ring-color:rgba(224,242,254,.25)}.dark\:ring-sky-100\/30:is(.dark *){--tw-ring-color:rgba(224,242,254,.3)}.dark\:ring-sky-100\/35:is(.dark *){--tw-ring-color:rgba(224,242,254,.35)}.dark\:ring-sky-100\/40:is(.dark *){--tw-ring-color:rgba(224,242,254,.4)}.dark\:ring-sky-100\/45:is(.dark *){--tw-ring-color:rgba(224,242,254,.45)}.dark\:ring-sky-100\/5:is(.dark *){--tw-ring-color:rgba(224,242,254,.05)}.dark\:ring-sky-100\/50:is(.dark *){--tw-ring-color:rgba(224,242,254,.5)}.dark\:ring-sky-100\/55:is(.dark *){--tw-ring-color:rgba(224,242,254,.55)}.dark\:ring-sky-100\/60:is(.dark *){--tw-ring-color:rgba(224,242,254,.6)}.dark\:ring-sky-100\/65:is(.dark *){--tw-ring-color:rgba(224,242,254,.65)}.dark\:ring-sky-100\/70:is(.dark *){--tw-ring-color:rgba(224,242,254,.7)}.dark\:ring-sky-100\/75:is(.dark *){--tw-ring-color:rgba(224,242,254,.75)}.dark\:ring-sky-100\/80:is(.dark *){--tw-ring-color:rgba(224,242,254,.8)}.dark\:ring-sky-100\/85:is(.dark *){--tw-ring-color:rgba(224,242,254,.85)}.dark\:ring-sky-100\/90:is(.dark *){--tw-ring-color:rgba(224,242,254,.9)}.dark\:ring-sky-100\/95:is(.dark *){--tw-ring-color:rgba(224,242,254,.95)}.dark\:ring-sky-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(186 230 253/var(--tw-ring-opacity,1))}.dark\:ring-sky-200\/0:is(.dark *){--tw-ring-color:rgba(186,230,253,0)}.dark\:ring-sky-200\/10:is(.dark *){--tw-ring-color:rgba(186,230,253,.1)}.dark\:ring-sky-200\/100:is(.dark *){--tw-ring-color:#bae6fd}.dark\:ring-sky-200\/15:is(.dark *){--tw-ring-color:rgba(186,230,253,.15)}.dark\:ring-sky-200\/20:is(.dark *){--tw-ring-color:rgba(186,230,253,.2)}.dark\:ring-sky-200\/25:is(.dark *){--tw-ring-color:rgba(186,230,253,.25)}.dark\:ring-sky-200\/30:is(.dark *){--tw-ring-color:rgba(186,230,253,.3)}.dark\:ring-sky-200\/35:is(.dark *){--tw-ring-color:rgba(186,230,253,.35)}.dark\:ring-sky-200\/40:is(.dark *){--tw-ring-color:rgba(186,230,253,.4)}.dark\:ring-sky-200\/45:is(.dark *){--tw-ring-color:rgba(186,230,253,.45)}.dark\:ring-sky-200\/5:is(.dark *){--tw-ring-color:rgba(186,230,253,.05)}.dark\:ring-sky-200\/50:is(.dark *){--tw-ring-color:rgba(186,230,253,.5)}.dark\:ring-sky-200\/55:is(.dark *){--tw-ring-color:rgba(186,230,253,.55)}.dark\:ring-sky-200\/60:is(.dark *){--tw-ring-color:rgba(186,230,253,.6)}.dark\:ring-sky-200\/65:is(.dark *){--tw-ring-color:rgba(186,230,253,.65)}.dark\:ring-sky-200\/70:is(.dark *){--tw-ring-color:rgba(186,230,253,.7)}.dark\:ring-sky-200\/75:is(.dark *){--tw-ring-color:rgba(186,230,253,.75)}.dark\:ring-sky-200\/80:is(.dark *){--tw-ring-color:rgba(186,230,253,.8)}.dark\:ring-sky-200\/85:is(.dark *){--tw-ring-color:rgba(186,230,253,.85)}.dark\:ring-sky-200\/90:is(.dark *){--tw-ring-color:rgba(186,230,253,.9)}.dark\:ring-sky-200\/95:is(.dark *){--tw-ring-color:rgba(186,230,253,.95)}.dark\:ring-sky-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(56 189 248/var(--tw-ring-opacity,1))}.dark\:ring-sky-400\/0:is(.dark *){--tw-ring-color:rgba(56,189,248,0)}.dark\:ring-sky-400\/10:is(.dark *){--tw-ring-color:rgba(56,189,248,.1)}.dark\:ring-sky-400\/100:is(.dark *){--tw-ring-color:#38bdf8}.dark\:ring-sky-400\/15:is(.dark *){--tw-ring-color:rgba(56,189,248,.15)}.dark\:ring-sky-400\/20:is(.dark *){--tw-ring-color:rgba(56,189,248,.2)}.dark\:ring-sky-400\/25:is(.dark *){--tw-ring-color:rgba(56,189,248,.25)}.dark\:ring-sky-400\/30:is(.dark *){--tw-ring-color:rgba(56,189,248,.3)}.dark\:ring-sky-400\/35:is(.dark *){--tw-ring-color:rgba(56,189,248,.35)}.dark\:ring-sky-400\/40:is(.dark *){--tw-ring-color:rgba(56,189,248,.4)}.dark\:ring-sky-400\/45:is(.dark *){--tw-ring-color:rgba(56,189,248,.45)}.dark\:ring-sky-400\/5:is(.dark *){--tw-ring-color:rgba(56,189,248,.05)}.dark\:ring-sky-400\/50:is(.dark *){--tw-ring-color:rgba(56,189,248,.5)}.dark\:ring-sky-400\/55:is(.dark *){--tw-ring-color:rgba(56,189,248,.55)}.dark\:ring-sky-400\/60:is(.dark *){--tw-ring-color:rgba(56,189,248,.6)}.dark\:ring-sky-400\/65:is(.dark *){--tw-ring-color:rgba(56,189,248,.65)}.dark\:ring-sky-400\/70:is(.dark *){--tw-ring-color:rgba(56,189,248,.7)}.dark\:ring-sky-400\/75:is(.dark *){--tw-ring-color:rgba(56,189,248,.75)}.dark\:ring-sky-400\/80:is(.dark *){--tw-ring-color:rgba(56,189,248,.8)}.dark\:ring-sky-400\/85:is(.dark *){--tw-ring-color:rgba(56,189,248,.85)}.dark\:ring-sky-400\/90:is(.dark *){--tw-ring-color:rgba(56,189,248,.9)}.dark\:ring-sky-400\/95:is(.dark *){--tw-ring-color:rgba(56,189,248,.95)}.dark\:ring-sky-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(240 249 255/var(--tw-ring-opacity,1))}.dark\:ring-sky-50\/0:is(.dark *){--tw-ring-color:rgba(240,249,255,0)}.dark\:ring-sky-50\/10:is(.dark *){--tw-ring-color:rgba(240,249,255,.1)}.dark\:ring-sky-50\/100:is(.dark *){--tw-ring-color:#f0f9ff}.dark\:ring-sky-50\/15:is(.dark *){--tw-ring-color:rgba(240,249,255,.15)}.dark\:ring-sky-50\/20:is(.dark *){--tw-ring-color:rgba(240,249,255,.2)}.dark\:ring-sky-50\/25:is(.dark *){--tw-ring-color:rgba(240,249,255,.25)}.dark\:ring-sky-50\/30:is(.dark *){--tw-ring-color:rgba(240,249,255,.3)}.dark\:ring-sky-50\/35:is(.dark *){--tw-ring-color:rgba(240,249,255,.35)}.dark\:ring-sky-50\/40:is(.dark *){--tw-ring-color:rgba(240,249,255,.4)}.dark\:ring-sky-50\/45:is(.dark *){--tw-ring-color:rgba(240,249,255,.45)}.dark\:ring-sky-50\/5:is(.dark *){--tw-ring-color:rgba(240,249,255,.05)}.dark\:ring-sky-50\/50:is(.dark *){--tw-ring-color:rgba(240,249,255,.5)}.dark\:ring-sky-50\/55:is(.dark *){--tw-ring-color:rgba(240,249,255,.55)}.dark\:ring-sky-50\/60:is(.dark *){--tw-ring-color:rgba(240,249,255,.6)}.dark\:ring-sky-50\/65:is(.dark *){--tw-ring-color:rgba(240,249,255,.65)}.dark\:ring-sky-50\/70:is(.dark *){--tw-ring-color:rgba(240,249,255,.7)}.dark\:ring-sky-50\/75:is(.dark *){--tw-ring-color:rgba(240,249,255,.75)}.dark\:ring-sky-50\/80:is(.dark *){--tw-ring-color:rgba(240,249,255,.8)}.dark\:ring-sky-50\/85:is(.dark *){--tw-ring-color:rgba(240,249,255,.85)}.dark\:ring-sky-50\/90:is(.dark *){--tw-ring-color:rgba(240,249,255,.9)}.dark\:ring-sky-50\/95:is(.dark *){--tw-ring-color:rgba(240,249,255,.95)}.dark\:ring-sky-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(14 165 233/var(--tw-ring-opacity,1))}.dark\:ring-sky-500\/0:is(.dark *){--tw-ring-color:rgba(14,165,233,0)}.dark\:ring-sky-500\/10:is(.dark *){--tw-ring-color:rgba(14,165,233,.1)}.dark\:ring-sky-500\/100:is(.dark *){--tw-ring-color:#0ea5e9}.dark\:ring-sky-500\/15:is(.dark *){--tw-ring-color:rgba(14,165,233,.15)}.dark\:ring-sky-500\/20:is(.dark *){--tw-ring-color:rgba(14,165,233,.2)}.dark\:ring-sky-500\/25:is(.dark *){--tw-ring-color:rgba(14,165,233,.25)}.dark\:ring-sky-500\/30:is(.dark *){--tw-ring-color:rgba(14,165,233,.3)}.dark\:ring-sky-500\/35:is(.dark *){--tw-ring-color:rgba(14,165,233,.35)}.dark\:ring-sky-500\/40:is(.dark *){--tw-ring-color:rgba(14,165,233,.4)}.dark\:ring-sky-500\/45:is(.dark *){--tw-ring-color:rgba(14,165,233,.45)}.dark\:ring-sky-500\/5:is(.dark *){--tw-ring-color:rgba(14,165,233,.05)}.dark\:ring-sky-500\/50:is(.dark *){--tw-ring-color:rgba(14,165,233,.5)}.dark\:ring-sky-500\/55:is(.dark *){--tw-ring-color:rgba(14,165,233,.55)}.dark\:ring-sky-500\/60:is(.dark *){--tw-ring-color:rgba(14,165,233,.6)}.dark\:ring-sky-500\/65:is(.dark *){--tw-ring-color:rgba(14,165,233,.65)}.dark\:ring-sky-500\/70:is(.dark *){--tw-ring-color:rgba(14,165,233,.7)}.dark\:ring-sky-500\/75:is(.dark *){--tw-ring-color:rgba(14,165,233,.75)}.dark\:ring-sky-500\/80:is(.dark *){--tw-ring-color:rgba(14,165,233,.8)}.dark\:ring-sky-500\/85:is(.dark *){--tw-ring-color:rgba(14,165,233,.85)}.dark\:ring-sky-500\/90:is(.dark *){--tw-ring-color:rgba(14,165,233,.9)}.dark\:ring-sky-500\/95:is(.dark *){--tw-ring-color:rgba(14,165,233,.95)}.dark\:ring-sky-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(8 47 73/var(--tw-ring-opacity,1))}.dark\:ring-sky-950\/0:is(.dark *){--tw-ring-color:rgba(8,47,73,0)}.dark\:ring-sky-950\/10:is(.dark *){--tw-ring-color:rgba(8,47,73,.1)}.dark\:ring-sky-950\/100:is(.dark *){--tw-ring-color:#082f49}.dark\:ring-sky-950\/15:is(.dark *){--tw-ring-color:rgba(8,47,73,.15)}.dark\:ring-sky-950\/20:is(.dark *){--tw-ring-color:rgba(8,47,73,.2)}.dark\:ring-sky-950\/25:is(.dark *){--tw-ring-color:rgba(8,47,73,.25)}.dark\:ring-sky-950\/30:is(.dark *){--tw-ring-color:rgba(8,47,73,.3)}.dark\:ring-sky-950\/35:is(.dark *){--tw-ring-color:rgba(8,47,73,.35)}.dark\:ring-sky-950\/40:is(.dark *){--tw-ring-color:rgba(8,47,73,.4)}.dark\:ring-sky-950\/45:is(.dark *){--tw-ring-color:rgba(8,47,73,.45)}.dark\:ring-sky-950\/5:is(.dark *){--tw-ring-color:rgba(8,47,73,.05)}.dark\:ring-sky-950\/50:is(.dark *){--tw-ring-color:rgba(8,47,73,.5)}.dark\:ring-sky-950\/55:is(.dark *){--tw-ring-color:rgba(8,47,73,.55)}.dark\:ring-sky-950\/60:is(.dark *){--tw-ring-color:rgba(8,47,73,.6)}.dark\:ring-sky-950\/65:is(.dark *){--tw-ring-color:rgba(8,47,73,.65)}.dark\:ring-sky-950\/70:is(.dark *){--tw-ring-color:rgba(8,47,73,.7)}.dark\:ring-sky-950\/75:is(.dark *){--tw-ring-color:rgba(8,47,73,.75)}.dark\:ring-sky-950\/80:is(.dark *){--tw-ring-color:rgba(8,47,73,.8)}.dark\:ring-sky-950\/85:is(.dark *){--tw-ring-color:rgba(8,47,73,.85)}.dark\:ring-sky-950\/90:is(.dark *){--tw-ring-color:rgba(8,47,73,.9)}.dark\:ring-sky-950\/95:is(.dark *){--tw-ring-color:rgba(8,47,73,.95)}.dark\:ring-slate-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(241 245 249/var(--tw-ring-opacity,1))}.dark\:ring-slate-100\/0:is(.dark *){--tw-ring-color:rgba(241,245,249,0)}.dark\:ring-slate-100\/10:is(.dark *){--tw-ring-color:rgba(241,245,249,.1)}.dark\:ring-slate-100\/100:is(.dark *){--tw-ring-color:#f1f5f9}.dark\:ring-slate-100\/15:is(.dark *){--tw-ring-color:rgba(241,245,249,.15)}.dark\:ring-slate-100\/20:is(.dark *){--tw-ring-color:rgba(241,245,249,.2)}.dark\:ring-slate-100\/25:is(.dark *){--tw-ring-color:rgba(241,245,249,.25)}.dark\:ring-slate-100\/30:is(.dark *){--tw-ring-color:rgba(241,245,249,.3)}.dark\:ring-slate-100\/35:is(.dark *){--tw-ring-color:rgba(241,245,249,.35)}.dark\:ring-slate-100\/40:is(.dark *){--tw-ring-color:rgba(241,245,249,.4)}.dark\:ring-slate-100\/45:is(.dark *){--tw-ring-color:rgba(241,245,249,.45)}.dark\:ring-slate-100\/5:is(.dark *){--tw-ring-color:rgba(241,245,249,.05)}.dark\:ring-slate-100\/50:is(.dark *){--tw-ring-color:rgba(241,245,249,.5)}.dark\:ring-slate-100\/55:is(.dark *){--tw-ring-color:rgba(241,245,249,.55)}.dark\:ring-slate-100\/60:is(.dark *){--tw-ring-color:rgba(241,245,249,.6)}.dark\:ring-slate-100\/65:is(.dark *){--tw-ring-color:rgba(241,245,249,.65)}.dark\:ring-slate-100\/70:is(.dark *){--tw-ring-color:rgba(241,245,249,.7)}.dark\:ring-slate-100\/75:is(.dark *){--tw-ring-color:rgba(241,245,249,.75)}.dark\:ring-slate-100\/80:is(.dark *){--tw-ring-color:rgba(241,245,249,.8)}.dark\:ring-slate-100\/85:is(.dark *){--tw-ring-color:rgba(241,245,249,.85)}.dark\:ring-slate-100\/90:is(.dark *){--tw-ring-color:rgba(241,245,249,.9)}.dark\:ring-slate-100\/95:is(.dark *){--tw-ring-color:rgba(241,245,249,.95)}.dark\:ring-slate-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(226 232 240/var(--tw-ring-opacity,1))}.dark\:ring-slate-200\/0:is(.dark *){--tw-ring-color:rgba(226,232,240,0)}.dark\:ring-slate-200\/10:is(.dark *){--tw-ring-color:rgba(226,232,240,.1)}.dark\:ring-slate-200\/100:is(.dark *){--tw-ring-color:#e2e8f0}.dark\:ring-slate-200\/15:is(.dark *){--tw-ring-color:rgba(226,232,240,.15)}.dark\:ring-slate-200\/20:is(.dark *){--tw-ring-color:rgba(226,232,240,.2)}.dark\:ring-slate-200\/25:is(.dark *){--tw-ring-color:rgba(226,232,240,.25)}.dark\:ring-slate-200\/30:is(.dark *){--tw-ring-color:rgba(226,232,240,.3)}.dark\:ring-slate-200\/35:is(.dark *){--tw-ring-color:rgba(226,232,240,.35)}.dark\:ring-slate-200\/40:is(.dark *){--tw-ring-color:rgba(226,232,240,.4)}.dark\:ring-slate-200\/45:is(.dark *){--tw-ring-color:rgba(226,232,240,.45)}.dark\:ring-slate-200\/5:is(.dark *){--tw-ring-color:rgba(226,232,240,.05)}.dark\:ring-slate-200\/50:is(.dark *){--tw-ring-color:rgba(226,232,240,.5)}.dark\:ring-slate-200\/55:is(.dark *){--tw-ring-color:rgba(226,232,240,.55)}.dark\:ring-slate-200\/60:is(.dark *){--tw-ring-color:rgba(226,232,240,.6)}.dark\:ring-slate-200\/65:is(.dark *){--tw-ring-color:rgba(226,232,240,.65)}.dark\:ring-slate-200\/70:is(.dark *){--tw-ring-color:rgba(226,232,240,.7)}.dark\:ring-slate-200\/75:is(.dark *){--tw-ring-color:rgba(226,232,240,.75)}.dark\:ring-slate-200\/80:is(.dark *){--tw-ring-color:rgba(226,232,240,.8)}.dark\:ring-slate-200\/85:is(.dark *){--tw-ring-color:rgba(226,232,240,.85)}.dark\:ring-slate-200\/90:is(.dark *){--tw-ring-color:rgba(226,232,240,.9)}.dark\:ring-slate-200\/95:is(.dark *){--tw-ring-color:rgba(226,232,240,.95)}.dark\:ring-slate-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(148 163 184/var(--tw-ring-opacity,1))}.dark\:ring-slate-400\/0:is(.dark *){--tw-ring-color:rgba(148,163,184,0)}.dark\:ring-slate-400\/10:is(.dark *){--tw-ring-color:rgba(148,163,184,.1)}.dark\:ring-slate-400\/100:is(.dark *){--tw-ring-color:#94a3b8}.dark\:ring-slate-400\/15:is(.dark *){--tw-ring-color:rgba(148,163,184,.15)}.dark\:ring-slate-400\/20:is(.dark *){--tw-ring-color:rgba(148,163,184,.2)}.dark\:ring-slate-400\/25:is(.dark *){--tw-ring-color:rgba(148,163,184,.25)}.dark\:ring-slate-400\/30:is(.dark *){--tw-ring-color:rgba(148,163,184,.3)}.dark\:ring-slate-400\/35:is(.dark *){--tw-ring-color:rgba(148,163,184,.35)}.dark\:ring-slate-400\/40:is(.dark *){--tw-ring-color:rgba(148,163,184,.4)}.dark\:ring-slate-400\/45:is(.dark *){--tw-ring-color:rgba(148,163,184,.45)}.dark\:ring-slate-400\/5:is(.dark *){--tw-ring-color:rgba(148,163,184,.05)}.dark\:ring-slate-400\/50:is(.dark *){--tw-ring-color:rgba(148,163,184,.5)}.dark\:ring-slate-400\/55:is(.dark *){--tw-ring-color:rgba(148,163,184,.55)}.dark\:ring-slate-400\/60:is(.dark *){--tw-ring-color:rgba(148,163,184,.6)}.dark\:ring-slate-400\/65:is(.dark *){--tw-ring-color:rgba(148,163,184,.65)}.dark\:ring-slate-400\/70:is(.dark *){--tw-ring-color:rgba(148,163,184,.7)}.dark\:ring-slate-400\/75:is(.dark *){--tw-ring-color:rgba(148,163,184,.75)}.dark\:ring-slate-400\/80:is(.dark *){--tw-ring-color:rgba(148,163,184,.8)}.dark\:ring-slate-400\/85:is(.dark *){--tw-ring-color:rgba(148,163,184,.85)}.dark\:ring-slate-400\/90:is(.dark *){--tw-ring-color:rgba(148,163,184,.9)}.dark\:ring-slate-400\/95:is(.dark *){--tw-ring-color:rgba(148,163,184,.95)}.dark\:ring-slate-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(248 250 252/var(--tw-ring-opacity,1))}.dark\:ring-slate-50\/0:is(.dark *){--tw-ring-color:rgba(248,250,252,0)}.dark\:ring-slate-50\/10:is(.dark *){--tw-ring-color:rgba(248,250,252,.1)}.dark\:ring-slate-50\/100:is(.dark *){--tw-ring-color:#f8fafc}.dark\:ring-slate-50\/15:is(.dark *){--tw-ring-color:rgba(248,250,252,.15)}.dark\:ring-slate-50\/20:is(.dark *){--tw-ring-color:rgba(248,250,252,.2)}.dark\:ring-slate-50\/25:is(.dark *){--tw-ring-color:rgba(248,250,252,.25)}.dark\:ring-slate-50\/30:is(.dark *){--tw-ring-color:rgba(248,250,252,.3)}.dark\:ring-slate-50\/35:is(.dark *){--tw-ring-color:rgba(248,250,252,.35)}.dark\:ring-slate-50\/40:is(.dark *){--tw-ring-color:rgba(248,250,252,.4)}.dark\:ring-slate-50\/45:is(.dark *){--tw-ring-color:rgba(248,250,252,.45)}.dark\:ring-slate-50\/5:is(.dark *){--tw-ring-color:rgba(248,250,252,.05)}.dark\:ring-slate-50\/50:is(.dark *){--tw-ring-color:rgba(248,250,252,.5)}.dark\:ring-slate-50\/55:is(.dark *){--tw-ring-color:rgba(248,250,252,.55)}.dark\:ring-slate-50\/60:is(.dark *){--tw-ring-color:rgba(248,250,252,.6)}.dark\:ring-slate-50\/65:is(.dark *){--tw-ring-color:rgba(248,250,252,.65)}.dark\:ring-slate-50\/70:is(.dark *){--tw-ring-color:rgba(248,250,252,.7)}.dark\:ring-slate-50\/75:is(.dark *){--tw-ring-color:rgba(248,250,252,.75)}.dark\:ring-slate-50\/80:is(.dark *){--tw-ring-color:rgba(248,250,252,.8)}.dark\:ring-slate-50\/85:is(.dark *){--tw-ring-color:rgba(248,250,252,.85)}.dark\:ring-slate-50\/90:is(.dark *){--tw-ring-color:rgba(248,250,252,.9)}.dark\:ring-slate-50\/95:is(.dark *){--tw-ring-color:rgba(248,250,252,.95)}.dark\:ring-slate-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(100 116 139/var(--tw-ring-opacity,1))}.dark\:ring-slate-500\/0:is(.dark *){--tw-ring-color:rgba(100,116,139,0)}.dark\:ring-slate-500\/10:is(.dark *){--tw-ring-color:rgba(100,116,139,.1)}.dark\:ring-slate-500\/100:is(.dark *){--tw-ring-color:#64748b}.dark\:ring-slate-500\/15:is(.dark *){--tw-ring-color:rgba(100,116,139,.15)}.dark\:ring-slate-500\/20:is(.dark *){--tw-ring-color:rgba(100,116,139,.2)}.dark\:ring-slate-500\/25:is(.dark *){--tw-ring-color:rgba(100,116,139,.25)}.dark\:ring-slate-500\/30:is(.dark *){--tw-ring-color:rgba(100,116,139,.3)}.dark\:ring-slate-500\/35:is(.dark *){--tw-ring-color:rgba(100,116,139,.35)}.dark\:ring-slate-500\/40:is(.dark *){--tw-ring-color:rgba(100,116,139,.4)}.dark\:ring-slate-500\/45:is(.dark *){--tw-ring-color:rgba(100,116,139,.45)}.dark\:ring-slate-500\/5:is(.dark *){--tw-ring-color:rgba(100,116,139,.05)}.dark\:ring-slate-500\/50:is(.dark *){--tw-ring-color:rgba(100,116,139,.5)}.dark\:ring-slate-500\/55:is(.dark *){--tw-ring-color:rgba(100,116,139,.55)}.dark\:ring-slate-500\/60:is(.dark *){--tw-ring-color:rgba(100,116,139,.6)}.dark\:ring-slate-500\/65:is(.dark *){--tw-ring-color:rgba(100,116,139,.65)}.dark\:ring-slate-500\/70:is(.dark *){--tw-ring-color:rgba(100,116,139,.7)}.dark\:ring-slate-500\/75:is(.dark *){--tw-ring-color:rgba(100,116,139,.75)}.dark\:ring-slate-500\/80:is(.dark *){--tw-ring-color:rgba(100,116,139,.8)}.dark\:ring-slate-500\/85:is(.dark *){--tw-ring-color:rgba(100,116,139,.85)}.dark\:ring-slate-500\/90:is(.dark *){--tw-ring-color:rgba(100,116,139,.9)}.dark\:ring-slate-500\/95:is(.dark *){--tw-ring-color:rgba(100,116,139,.95)}.dark\:ring-slate-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(2 6 23/var(--tw-ring-opacity,1))}.dark\:ring-slate-950\/0:is(.dark *){--tw-ring-color:rgba(2,6,23,0)}.dark\:ring-slate-950\/10:is(.dark *){--tw-ring-color:rgba(2,6,23,.1)}.dark\:ring-slate-950\/100:is(.dark *){--tw-ring-color:#020617}.dark\:ring-slate-950\/15:is(.dark *){--tw-ring-color:rgba(2,6,23,.15)}.dark\:ring-slate-950\/20:is(.dark *){--tw-ring-color:rgba(2,6,23,.2)}.dark\:ring-slate-950\/25:is(.dark *){--tw-ring-color:rgba(2,6,23,.25)}.dark\:ring-slate-950\/30:is(.dark *){--tw-ring-color:rgba(2,6,23,.3)}.dark\:ring-slate-950\/35:is(.dark *){--tw-ring-color:rgba(2,6,23,.35)}.dark\:ring-slate-950\/40:is(.dark *){--tw-ring-color:rgba(2,6,23,.4)}.dark\:ring-slate-950\/45:is(.dark *){--tw-ring-color:rgba(2,6,23,.45)}.dark\:ring-slate-950\/5:is(.dark *){--tw-ring-color:rgba(2,6,23,.05)}.dark\:ring-slate-950\/50:is(.dark *){--tw-ring-color:rgba(2,6,23,.5)}.dark\:ring-slate-950\/55:is(.dark *){--tw-ring-color:rgba(2,6,23,.55)}.dark\:ring-slate-950\/60:is(.dark *){--tw-ring-color:rgba(2,6,23,.6)}.dark\:ring-slate-950\/65:is(.dark *){--tw-ring-color:rgba(2,6,23,.65)}.dark\:ring-slate-950\/70:is(.dark *){--tw-ring-color:rgba(2,6,23,.7)}.dark\:ring-slate-950\/75:is(.dark *){--tw-ring-color:rgba(2,6,23,.75)}.dark\:ring-slate-950\/80:is(.dark *){--tw-ring-color:rgba(2,6,23,.8)}.dark\:ring-slate-950\/85:is(.dark *){--tw-ring-color:rgba(2,6,23,.85)}.dark\:ring-slate-950\/90:is(.dark *){--tw-ring-color:rgba(2,6,23,.9)}.dark\:ring-slate-950\/95:is(.dark *){--tw-ring-color:rgba(2,6,23,.95)}.dark\:ring-stone-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(245 245 244/var(--tw-ring-opacity,1))}.dark\:ring-stone-100\/0:is(.dark *){--tw-ring-color:hsla(60,5%,96%,0)}.dark\:ring-stone-100\/10:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.1)}.dark\:ring-stone-100\/100:is(.dark *){--tw-ring-color:#f5f5f4}.dark\:ring-stone-100\/15:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.15)}.dark\:ring-stone-100\/20:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.2)}.dark\:ring-stone-100\/25:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.25)}.dark\:ring-stone-100\/30:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.3)}.dark\:ring-stone-100\/35:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.35)}.dark\:ring-stone-100\/40:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.4)}.dark\:ring-stone-100\/45:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.45)}.dark\:ring-stone-100\/5:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.05)}.dark\:ring-stone-100\/50:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.5)}.dark\:ring-stone-100\/55:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.55)}.dark\:ring-stone-100\/60:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.6)}.dark\:ring-stone-100\/65:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.65)}.dark\:ring-stone-100\/70:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.7)}.dark\:ring-stone-100\/75:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.75)}.dark\:ring-stone-100\/80:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.8)}.dark\:ring-stone-100\/85:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.85)}.dark\:ring-stone-100\/90:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.9)}.dark\:ring-stone-100\/95:is(.dark *){--tw-ring-color:hsla(60,5%,96%,.95)}.dark\:ring-stone-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(231 229 228/var(--tw-ring-opacity,1))}.dark\:ring-stone-200\/0:is(.dark *){--tw-ring-color:hsla(20,6%,90%,0)}.dark\:ring-stone-200\/10:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.1)}.dark\:ring-stone-200\/100:is(.dark *){--tw-ring-color:#e7e5e4}.dark\:ring-stone-200\/15:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.15)}.dark\:ring-stone-200\/20:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.2)}.dark\:ring-stone-200\/25:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.25)}.dark\:ring-stone-200\/30:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.3)}.dark\:ring-stone-200\/35:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.35)}.dark\:ring-stone-200\/40:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.4)}.dark\:ring-stone-200\/45:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.45)}.dark\:ring-stone-200\/5:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.05)}.dark\:ring-stone-200\/50:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.5)}.dark\:ring-stone-200\/55:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.55)}.dark\:ring-stone-200\/60:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.6)}.dark\:ring-stone-200\/65:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.65)}.dark\:ring-stone-200\/70:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.7)}.dark\:ring-stone-200\/75:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.75)}.dark\:ring-stone-200\/80:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.8)}.dark\:ring-stone-200\/85:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.85)}.dark\:ring-stone-200\/90:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.9)}.dark\:ring-stone-200\/95:is(.dark *){--tw-ring-color:hsla(20,6%,90%,.95)}.dark\:ring-stone-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(168 162 158/var(--tw-ring-opacity,1))}.dark\:ring-stone-400\/0:is(.dark *){--tw-ring-color:hsla(24,5%,64%,0)}.dark\:ring-stone-400\/10:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.1)}.dark\:ring-stone-400\/100:is(.dark *){--tw-ring-color:#a8a29e}.dark\:ring-stone-400\/15:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.15)}.dark\:ring-stone-400\/20:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.2)}.dark\:ring-stone-400\/25:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.25)}.dark\:ring-stone-400\/30:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.3)}.dark\:ring-stone-400\/35:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.35)}.dark\:ring-stone-400\/40:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.4)}.dark\:ring-stone-400\/45:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.45)}.dark\:ring-stone-400\/5:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.05)}.dark\:ring-stone-400\/50:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.5)}.dark\:ring-stone-400\/55:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.55)}.dark\:ring-stone-400\/60:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.6)}.dark\:ring-stone-400\/65:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.65)}.dark\:ring-stone-400\/70:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.7)}.dark\:ring-stone-400\/75:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.75)}.dark\:ring-stone-400\/80:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.8)}.dark\:ring-stone-400\/85:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.85)}.dark\:ring-stone-400\/90:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.9)}.dark\:ring-stone-400\/95:is(.dark *){--tw-ring-color:hsla(24,5%,64%,.95)}.dark\:ring-stone-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(250 250 249/var(--tw-ring-opacity,1))}.dark\:ring-stone-50\/0:is(.dark *){--tw-ring-color:hsla(60,9%,98%,0)}.dark\:ring-stone-50\/10:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.1)}.dark\:ring-stone-50\/100:is(.dark *){--tw-ring-color:#fafaf9}.dark\:ring-stone-50\/15:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.15)}.dark\:ring-stone-50\/20:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.2)}.dark\:ring-stone-50\/25:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.25)}.dark\:ring-stone-50\/30:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.3)}.dark\:ring-stone-50\/35:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.35)}.dark\:ring-stone-50\/40:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.4)}.dark\:ring-stone-50\/45:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.45)}.dark\:ring-stone-50\/5:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.05)}.dark\:ring-stone-50\/50:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.5)}.dark\:ring-stone-50\/55:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.55)}.dark\:ring-stone-50\/60:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.6)}.dark\:ring-stone-50\/65:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.65)}.dark\:ring-stone-50\/70:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.7)}.dark\:ring-stone-50\/75:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.75)}.dark\:ring-stone-50\/80:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.8)}.dark\:ring-stone-50\/85:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.85)}.dark\:ring-stone-50\/90:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.9)}.dark\:ring-stone-50\/95:is(.dark *){--tw-ring-color:hsla(60,9%,98%,.95)}.dark\:ring-stone-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(120 113 108/var(--tw-ring-opacity,1))}.dark\:ring-stone-500\/0:is(.dark *){--tw-ring-color:hsla(25,5%,45%,0)}.dark\:ring-stone-500\/10:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.1)}.dark\:ring-stone-500\/100:is(.dark *){--tw-ring-color:#78716c}.dark\:ring-stone-500\/15:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.15)}.dark\:ring-stone-500\/20:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.2)}.dark\:ring-stone-500\/25:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.25)}.dark\:ring-stone-500\/30:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.3)}.dark\:ring-stone-500\/35:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.35)}.dark\:ring-stone-500\/40:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.4)}.dark\:ring-stone-500\/45:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.45)}.dark\:ring-stone-500\/5:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.05)}.dark\:ring-stone-500\/50:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.5)}.dark\:ring-stone-500\/55:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.55)}.dark\:ring-stone-500\/60:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.6)}.dark\:ring-stone-500\/65:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.65)}.dark\:ring-stone-500\/70:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.7)}.dark\:ring-stone-500\/75:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.75)}.dark\:ring-stone-500\/80:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.8)}.dark\:ring-stone-500\/85:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.85)}.dark\:ring-stone-500\/90:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.9)}.dark\:ring-stone-500\/95:is(.dark *){--tw-ring-color:hsla(25,5%,45%,.95)}.dark\:ring-stone-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(12 10 9/var(--tw-ring-opacity,1))}.dark\:ring-stone-950\/0:is(.dark *){--tw-ring-color:rgba(12,10,9,0)}.dark\:ring-stone-950\/10:is(.dark *){--tw-ring-color:rgba(12,10,9,.1)}.dark\:ring-stone-950\/100:is(.dark *){--tw-ring-color:#0c0a09}.dark\:ring-stone-950\/15:is(.dark *){--tw-ring-color:rgba(12,10,9,.15)}.dark\:ring-stone-950\/20:is(.dark *){--tw-ring-color:rgba(12,10,9,.2)}.dark\:ring-stone-950\/25:is(.dark *){--tw-ring-color:rgba(12,10,9,.25)}.dark\:ring-stone-950\/30:is(.dark *){--tw-ring-color:rgba(12,10,9,.3)}.dark\:ring-stone-950\/35:is(.dark *){--tw-ring-color:rgba(12,10,9,.35)}.dark\:ring-stone-950\/40:is(.dark *){--tw-ring-color:rgba(12,10,9,.4)}.dark\:ring-stone-950\/45:is(.dark *){--tw-ring-color:rgba(12,10,9,.45)}.dark\:ring-stone-950\/5:is(.dark *){--tw-ring-color:rgba(12,10,9,.05)}.dark\:ring-stone-950\/50:is(.dark *){--tw-ring-color:rgba(12,10,9,.5)}.dark\:ring-stone-950\/55:is(.dark *){--tw-ring-color:rgba(12,10,9,.55)}.dark\:ring-stone-950\/60:is(.dark *){--tw-ring-color:rgba(12,10,9,.6)}.dark\:ring-stone-950\/65:is(.dark *){--tw-ring-color:rgba(12,10,9,.65)}.dark\:ring-stone-950\/70:is(.dark *){--tw-ring-color:rgba(12,10,9,.7)}.dark\:ring-stone-950\/75:is(.dark *){--tw-ring-color:rgba(12,10,9,.75)}.dark\:ring-stone-950\/80:is(.dark *){--tw-ring-color:rgba(12,10,9,.8)}.dark\:ring-stone-950\/85:is(.dark *){--tw-ring-color:rgba(12,10,9,.85)}.dark\:ring-stone-950\/90:is(.dark *){--tw-ring-color:rgba(12,10,9,.9)}.dark\:ring-stone-950\/95:is(.dark *){--tw-ring-color:rgba(12,10,9,.95)}.dark\:ring-teal-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(204 251 241/var(--tw-ring-opacity,1))}.dark\:ring-teal-100\/0:is(.dark *){--tw-ring-color:rgba(204,251,241,0)}.dark\:ring-teal-100\/10:is(.dark *){--tw-ring-color:rgba(204,251,241,.1)}.dark\:ring-teal-100\/100:is(.dark *){--tw-ring-color:#ccfbf1}.dark\:ring-teal-100\/15:is(.dark *){--tw-ring-color:rgba(204,251,241,.15)}.dark\:ring-teal-100\/20:is(.dark *){--tw-ring-color:rgba(204,251,241,.2)}.dark\:ring-teal-100\/25:is(.dark *){--tw-ring-color:rgba(204,251,241,.25)}.dark\:ring-teal-100\/30:is(.dark *){--tw-ring-color:rgba(204,251,241,.3)}.dark\:ring-teal-100\/35:is(.dark *){--tw-ring-color:rgba(204,251,241,.35)}.dark\:ring-teal-100\/40:is(.dark *){--tw-ring-color:rgba(204,251,241,.4)}.dark\:ring-teal-100\/45:is(.dark *){--tw-ring-color:rgba(204,251,241,.45)}.dark\:ring-teal-100\/5:is(.dark *){--tw-ring-color:rgba(204,251,241,.05)}.dark\:ring-teal-100\/50:is(.dark *){--tw-ring-color:rgba(204,251,241,.5)}.dark\:ring-teal-100\/55:is(.dark *){--tw-ring-color:rgba(204,251,241,.55)}.dark\:ring-teal-100\/60:is(.dark *){--tw-ring-color:rgba(204,251,241,.6)}.dark\:ring-teal-100\/65:is(.dark *){--tw-ring-color:rgba(204,251,241,.65)}.dark\:ring-teal-100\/70:is(.dark *){--tw-ring-color:rgba(204,251,241,.7)}.dark\:ring-teal-100\/75:is(.dark *){--tw-ring-color:rgba(204,251,241,.75)}.dark\:ring-teal-100\/80:is(.dark *){--tw-ring-color:rgba(204,251,241,.8)}.dark\:ring-teal-100\/85:is(.dark *){--tw-ring-color:rgba(204,251,241,.85)}.dark\:ring-teal-100\/90:is(.dark *){--tw-ring-color:rgba(204,251,241,.9)}.dark\:ring-teal-100\/95:is(.dark *){--tw-ring-color:rgba(204,251,241,.95)}.dark\:ring-teal-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(153 246 228/var(--tw-ring-opacity,1))}.dark\:ring-teal-200\/0:is(.dark *){--tw-ring-color:rgba(153,246,228,0)}.dark\:ring-teal-200\/10:is(.dark *){--tw-ring-color:rgba(153,246,228,.1)}.dark\:ring-teal-200\/100:is(.dark *){--tw-ring-color:#99f6e4}.dark\:ring-teal-200\/15:is(.dark *){--tw-ring-color:rgba(153,246,228,.15)}.dark\:ring-teal-200\/20:is(.dark *){--tw-ring-color:rgba(153,246,228,.2)}.dark\:ring-teal-200\/25:is(.dark *){--tw-ring-color:rgba(153,246,228,.25)}.dark\:ring-teal-200\/30:is(.dark *){--tw-ring-color:rgba(153,246,228,.3)}.dark\:ring-teal-200\/35:is(.dark *){--tw-ring-color:rgba(153,246,228,.35)}.dark\:ring-teal-200\/40:is(.dark *){--tw-ring-color:rgba(153,246,228,.4)}.dark\:ring-teal-200\/45:is(.dark *){--tw-ring-color:rgba(153,246,228,.45)}.dark\:ring-teal-200\/5:is(.dark *){--tw-ring-color:rgba(153,246,228,.05)}.dark\:ring-teal-200\/50:is(.dark *){--tw-ring-color:rgba(153,246,228,.5)}.dark\:ring-teal-200\/55:is(.dark *){--tw-ring-color:rgba(153,246,228,.55)}.dark\:ring-teal-200\/60:is(.dark *){--tw-ring-color:rgba(153,246,228,.6)}.dark\:ring-teal-200\/65:is(.dark *){--tw-ring-color:rgba(153,246,228,.65)}.dark\:ring-teal-200\/70:is(.dark *){--tw-ring-color:rgba(153,246,228,.7)}.dark\:ring-teal-200\/75:is(.dark *){--tw-ring-color:rgba(153,246,228,.75)}.dark\:ring-teal-200\/80:is(.dark *){--tw-ring-color:rgba(153,246,228,.8)}.dark\:ring-teal-200\/85:is(.dark *){--tw-ring-color:rgba(153,246,228,.85)}.dark\:ring-teal-200\/90:is(.dark *){--tw-ring-color:rgba(153,246,228,.9)}.dark\:ring-teal-200\/95:is(.dark *){--tw-ring-color:rgba(153,246,228,.95)}.dark\:ring-teal-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(45 212 191/var(--tw-ring-opacity,1))}.dark\:ring-teal-400\/0:is(.dark *){--tw-ring-color:rgba(45,212,191,0)}.dark\:ring-teal-400\/10:is(.dark *){--tw-ring-color:rgba(45,212,191,.1)}.dark\:ring-teal-400\/100:is(.dark *){--tw-ring-color:#2dd4bf}.dark\:ring-teal-400\/15:is(.dark *){--tw-ring-color:rgba(45,212,191,.15)}.dark\:ring-teal-400\/20:is(.dark *){--tw-ring-color:rgba(45,212,191,.2)}.dark\:ring-teal-400\/25:is(.dark *){--tw-ring-color:rgba(45,212,191,.25)}.dark\:ring-teal-400\/30:is(.dark *){--tw-ring-color:rgba(45,212,191,.3)}.dark\:ring-teal-400\/35:is(.dark *){--tw-ring-color:rgba(45,212,191,.35)}.dark\:ring-teal-400\/40:is(.dark *){--tw-ring-color:rgba(45,212,191,.4)}.dark\:ring-teal-400\/45:is(.dark *){--tw-ring-color:rgba(45,212,191,.45)}.dark\:ring-teal-400\/5:is(.dark *){--tw-ring-color:rgba(45,212,191,.05)}.dark\:ring-teal-400\/50:is(.dark *){--tw-ring-color:rgba(45,212,191,.5)}.dark\:ring-teal-400\/55:is(.dark *){--tw-ring-color:rgba(45,212,191,.55)}.dark\:ring-teal-400\/60:is(.dark *){--tw-ring-color:rgba(45,212,191,.6)}.dark\:ring-teal-400\/65:is(.dark *){--tw-ring-color:rgba(45,212,191,.65)}.dark\:ring-teal-400\/70:is(.dark *){--tw-ring-color:rgba(45,212,191,.7)}.dark\:ring-teal-400\/75:is(.dark *){--tw-ring-color:rgba(45,212,191,.75)}.dark\:ring-teal-400\/80:is(.dark *){--tw-ring-color:rgba(45,212,191,.8)}.dark\:ring-teal-400\/85:is(.dark *){--tw-ring-color:rgba(45,212,191,.85)}.dark\:ring-teal-400\/90:is(.dark *){--tw-ring-color:rgba(45,212,191,.9)}.dark\:ring-teal-400\/95:is(.dark *){--tw-ring-color:rgba(45,212,191,.95)}.dark\:ring-teal-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(240 253 250/var(--tw-ring-opacity,1))}.dark\:ring-teal-50\/0:is(.dark *){--tw-ring-color:rgba(240,253,250,0)}.dark\:ring-teal-50\/10:is(.dark *){--tw-ring-color:rgba(240,253,250,.1)}.dark\:ring-teal-50\/100:is(.dark *){--tw-ring-color:#f0fdfa}.dark\:ring-teal-50\/15:is(.dark *){--tw-ring-color:rgba(240,253,250,.15)}.dark\:ring-teal-50\/20:is(.dark *){--tw-ring-color:rgba(240,253,250,.2)}.dark\:ring-teal-50\/25:is(.dark *){--tw-ring-color:rgba(240,253,250,.25)}.dark\:ring-teal-50\/30:is(.dark *){--tw-ring-color:rgba(240,253,250,.3)}.dark\:ring-teal-50\/35:is(.dark *){--tw-ring-color:rgba(240,253,250,.35)}.dark\:ring-teal-50\/40:is(.dark *){--tw-ring-color:rgba(240,253,250,.4)}.dark\:ring-teal-50\/45:is(.dark *){--tw-ring-color:rgba(240,253,250,.45)}.dark\:ring-teal-50\/5:is(.dark *){--tw-ring-color:rgba(240,253,250,.05)}.dark\:ring-teal-50\/50:is(.dark *){--tw-ring-color:rgba(240,253,250,.5)}.dark\:ring-teal-50\/55:is(.dark *){--tw-ring-color:rgba(240,253,250,.55)}.dark\:ring-teal-50\/60:is(.dark *){--tw-ring-color:rgba(240,253,250,.6)}.dark\:ring-teal-50\/65:is(.dark *){--tw-ring-color:rgba(240,253,250,.65)}.dark\:ring-teal-50\/70:is(.dark *){--tw-ring-color:rgba(240,253,250,.7)}.dark\:ring-teal-50\/75:is(.dark *){--tw-ring-color:rgba(240,253,250,.75)}.dark\:ring-teal-50\/80:is(.dark *){--tw-ring-color:rgba(240,253,250,.8)}.dark\:ring-teal-50\/85:is(.dark *){--tw-ring-color:rgba(240,253,250,.85)}.dark\:ring-teal-50\/90:is(.dark *){--tw-ring-color:rgba(240,253,250,.9)}.dark\:ring-teal-50\/95:is(.dark *){--tw-ring-color:rgba(240,253,250,.95)}.dark\:ring-teal-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(20 184 166/var(--tw-ring-opacity,1))}.dark\:ring-teal-500\/0:is(.dark *){--tw-ring-color:rgba(20,184,166,0)}.dark\:ring-teal-500\/10:is(.dark *){--tw-ring-color:rgba(20,184,166,.1)}.dark\:ring-teal-500\/100:is(.dark *){--tw-ring-color:#14b8a6}.dark\:ring-teal-500\/15:is(.dark *){--tw-ring-color:rgba(20,184,166,.15)}.dark\:ring-teal-500\/20:is(.dark *){--tw-ring-color:rgba(20,184,166,.2)}.dark\:ring-teal-500\/25:is(.dark *){--tw-ring-color:rgba(20,184,166,.25)}.dark\:ring-teal-500\/30:is(.dark *){--tw-ring-color:rgba(20,184,166,.3)}.dark\:ring-teal-500\/35:is(.dark *){--tw-ring-color:rgba(20,184,166,.35)}.dark\:ring-teal-500\/40:is(.dark *){--tw-ring-color:rgba(20,184,166,.4)}.dark\:ring-teal-500\/45:is(.dark *){--tw-ring-color:rgba(20,184,166,.45)}.dark\:ring-teal-500\/5:is(.dark *){--tw-ring-color:rgba(20,184,166,.05)}.dark\:ring-teal-500\/50:is(.dark *){--tw-ring-color:rgba(20,184,166,.5)}.dark\:ring-teal-500\/55:is(.dark *){--tw-ring-color:rgba(20,184,166,.55)}.dark\:ring-teal-500\/60:is(.dark *){--tw-ring-color:rgba(20,184,166,.6)}.dark\:ring-teal-500\/65:is(.dark *){--tw-ring-color:rgba(20,184,166,.65)}.dark\:ring-teal-500\/70:is(.dark *){--tw-ring-color:rgba(20,184,166,.7)}.dark\:ring-teal-500\/75:is(.dark *){--tw-ring-color:rgba(20,184,166,.75)}.dark\:ring-teal-500\/80:is(.dark *){--tw-ring-color:rgba(20,184,166,.8)}.dark\:ring-teal-500\/85:is(.dark *){--tw-ring-color:rgba(20,184,166,.85)}.dark\:ring-teal-500\/90:is(.dark *){--tw-ring-color:rgba(20,184,166,.9)}.dark\:ring-teal-500\/95:is(.dark *){--tw-ring-color:rgba(20,184,166,.95)}.dark\:ring-teal-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(4 47 46/var(--tw-ring-opacity,1))}.dark\:ring-teal-950\/0:is(.dark *){--tw-ring-color:rgba(4,47,46,0)}.dark\:ring-teal-950\/10:is(.dark *){--tw-ring-color:rgba(4,47,46,.1)}.dark\:ring-teal-950\/100:is(.dark *){--tw-ring-color:#042f2e}.dark\:ring-teal-950\/15:is(.dark *){--tw-ring-color:rgba(4,47,46,.15)}.dark\:ring-teal-950\/20:is(.dark *){--tw-ring-color:rgba(4,47,46,.2)}.dark\:ring-teal-950\/25:is(.dark *){--tw-ring-color:rgba(4,47,46,.25)}.dark\:ring-teal-950\/30:is(.dark *){--tw-ring-color:rgba(4,47,46,.3)}.dark\:ring-teal-950\/35:is(.dark *){--tw-ring-color:rgba(4,47,46,.35)}.dark\:ring-teal-950\/40:is(.dark *){--tw-ring-color:rgba(4,47,46,.4)}.dark\:ring-teal-950\/45:is(.dark *){--tw-ring-color:rgba(4,47,46,.45)}.dark\:ring-teal-950\/5:is(.dark *){--tw-ring-color:rgba(4,47,46,.05)}.dark\:ring-teal-950\/50:is(.dark *){--tw-ring-color:rgba(4,47,46,.5)}.dark\:ring-teal-950\/55:is(.dark *){--tw-ring-color:rgba(4,47,46,.55)}.dark\:ring-teal-950\/60:is(.dark *){--tw-ring-color:rgba(4,47,46,.6)}.dark\:ring-teal-950\/65:is(.dark *){--tw-ring-color:rgba(4,47,46,.65)}.dark\:ring-teal-950\/70:is(.dark *){--tw-ring-color:rgba(4,47,46,.7)}.dark\:ring-teal-950\/75:is(.dark *){--tw-ring-color:rgba(4,47,46,.75)}.dark\:ring-teal-950\/80:is(.dark *){--tw-ring-color:rgba(4,47,46,.8)}.dark\:ring-teal-950\/85:is(.dark *){--tw-ring-color:rgba(4,47,46,.85)}.dark\:ring-teal-950\/90:is(.dark *){--tw-ring-color:rgba(4,47,46,.9)}.dark\:ring-teal-950\/95:is(.dark *){--tw-ring-color:rgba(4,47,46,.95)}.dark\:ring-violet-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(237 233 254/var(--tw-ring-opacity,1))}.dark\:ring-violet-100\/0:is(.dark *){--tw-ring-color:rgba(237,233,254,0)}.dark\:ring-violet-100\/10:is(.dark *){--tw-ring-color:rgba(237,233,254,.1)}.dark\:ring-violet-100\/100:is(.dark *){--tw-ring-color:#ede9fe}.dark\:ring-violet-100\/15:is(.dark *){--tw-ring-color:rgba(237,233,254,.15)}.dark\:ring-violet-100\/20:is(.dark *){--tw-ring-color:rgba(237,233,254,.2)}.dark\:ring-violet-100\/25:is(.dark *){--tw-ring-color:rgba(237,233,254,.25)}.dark\:ring-violet-100\/30:is(.dark *){--tw-ring-color:rgba(237,233,254,.3)}.dark\:ring-violet-100\/35:is(.dark *){--tw-ring-color:rgba(237,233,254,.35)}.dark\:ring-violet-100\/40:is(.dark *){--tw-ring-color:rgba(237,233,254,.4)}.dark\:ring-violet-100\/45:is(.dark *){--tw-ring-color:rgba(237,233,254,.45)}.dark\:ring-violet-100\/5:is(.dark *){--tw-ring-color:rgba(237,233,254,.05)}.dark\:ring-violet-100\/50:is(.dark *){--tw-ring-color:rgba(237,233,254,.5)}.dark\:ring-violet-100\/55:is(.dark *){--tw-ring-color:rgba(237,233,254,.55)}.dark\:ring-violet-100\/60:is(.dark *){--tw-ring-color:rgba(237,233,254,.6)}.dark\:ring-violet-100\/65:is(.dark *){--tw-ring-color:rgba(237,233,254,.65)}.dark\:ring-violet-100\/70:is(.dark *){--tw-ring-color:rgba(237,233,254,.7)}.dark\:ring-violet-100\/75:is(.dark *){--tw-ring-color:rgba(237,233,254,.75)}.dark\:ring-violet-100\/80:is(.dark *){--tw-ring-color:rgba(237,233,254,.8)}.dark\:ring-violet-100\/85:is(.dark *){--tw-ring-color:rgba(237,233,254,.85)}.dark\:ring-violet-100\/90:is(.dark *){--tw-ring-color:rgba(237,233,254,.9)}.dark\:ring-violet-100\/95:is(.dark *){--tw-ring-color:rgba(237,233,254,.95)}.dark\:ring-violet-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(221 214 254/var(--tw-ring-opacity,1))}.dark\:ring-violet-200\/0:is(.dark *){--tw-ring-color:rgba(221,214,254,0)}.dark\:ring-violet-200\/10:is(.dark *){--tw-ring-color:rgba(221,214,254,.1)}.dark\:ring-violet-200\/100:is(.dark *){--tw-ring-color:#ddd6fe}.dark\:ring-violet-200\/15:is(.dark *){--tw-ring-color:rgba(221,214,254,.15)}.dark\:ring-violet-200\/20:is(.dark *){--tw-ring-color:rgba(221,214,254,.2)}.dark\:ring-violet-200\/25:is(.dark *){--tw-ring-color:rgba(221,214,254,.25)}.dark\:ring-violet-200\/30:is(.dark *){--tw-ring-color:rgba(221,214,254,.3)}.dark\:ring-violet-200\/35:is(.dark *){--tw-ring-color:rgba(221,214,254,.35)}.dark\:ring-violet-200\/40:is(.dark *){--tw-ring-color:rgba(221,214,254,.4)}.dark\:ring-violet-200\/45:is(.dark *){--tw-ring-color:rgba(221,214,254,.45)}.dark\:ring-violet-200\/5:is(.dark *){--tw-ring-color:rgba(221,214,254,.05)}.dark\:ring-violet-200\/50:is(.dark *){--tw-ring-color:rgba(221,214,254,.5)}.dark\:ring-violet-200\/55:is(.dark *){--tw-ring-color:rgba(221,214,254,.55)}.dark\:ring-violet-200\/60:is(.dark *){--tw-ring-color:rgba(221,214,254,.6)}.dark\:ring-violet-200\/65:is(.dark *){--tw-ring-color:rgba(221,214,254,.65)}.dark\:ring-violet-200\/70:is(.dark *){--tw-ring-color:rgba(221,214,254,.7)}.dark\:ring-violet-200\/75:is(.dark *){--tw-ring-color:rgba(221,214,254,.75)}.dark\:ring-violet-200\/80:is(.dark *){--tw-ring-color:rgba(221,214,254,.8)}.dark\:ring-violet-200\/85:is(.dark *){--tw-ring-color:rgba(221,214,254,.85)}.dark\:ring-violet-200\/90:is(.dark *){--tw-ring-color:rgba(221,214,254,.9)}.dark\:ring-violet-200\/95:is(.dark *){--tw-ring-color:rgba(221,214,254,.95)}.dark\:ring-violet-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(167 139 250/var(--tw-ring-opacity,1))}.dark\:ring-violet-400\/0:is(.dark *){--tw-ring-color:rgba(167,139,250,0)}.dark\:ring-violet-400\/10:is(.dark *){--tw-ring-color:rgba(167,139,250,.1)}.dark\:ring-violet-400\/100:is(.dark *){--tw-ring-color:#a78bfa}.dark\:ring-violet-400\/15:is(.dark *){--tw-ring-color:rgba(167,139,250,.15)}.dark\:ring-violet-400\/20:is(.dark *){--tw-ring-color:rgba(167,139,250,.2)}.dark\:ring-violet-400\/25:is(.dark *){--tw-ring-color:rgba(167,139,250,.25)}.dark\:ring-violet-400\/30:is(.dark *){--tw-ring-color:rgba(167,139,250,.3)}.dark\:ring-violet-400\/35:is(.dark *){--tw-ring-color:rgba(167,139,250,.35)}.dark\:ring-violet-400\/40:is(.dark *){--tw-ring-color:rgba(167,139,250,.4)}.dark\:ring-violet-400\/45:is(.dark *){--tw-ring-color:rgba(167,139,250,.45)}.dark\:ring-violet-400\/5:is(.dark *){--tw-ring-color:rgba(167,139,250,.05)}.dark\:ring-violet-400\/50:is(.dark *){--tw-ring-color:rgba(167,139,250,.5)}.dark\:ring-violet-400\/55:is(.dark *){--tw-ring-color:rgba(167,139,250,.55)}.dark\:ring-violet-400\/60:is(.dark *){--tw-ring-color:rgba(167,139,250,.6)}.dark\:ring-violet-400\/65:is(.dark *){--tw-ring-color:rgba(167,139,250,.65)}.dark\:ring-violet-400\/70:is(.dark *){--tw-ring-color:rgba(167,139,250,.7)}.dark\:ring-violet-400\/75:is(.dark *){--tw-ring-color:rgba(167,139,250,.75)}.dark\:ring-violet-400\/80:is(.dark *){--tw-ring-color:rgba(167,139,250,.8)}.dark\:ring-violet-400\/85:is(.dark *){--tw-ring-color:rgba(167,139,250,.85)}.dark\:ring-violet-400\/90:is(.dark *){--tw-ring-color:rgba(167,139,250,.9)}.dark\:ring-violet-400\/95:is(.dark *){--tw-ring-color:rgba(167,139,250,.95)}.dark\:ring-violet-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(245 243 255/var(--tw-ring-opacity,1))}.dark\:ring-violet-50\/0:is(.dark *){--tw-ring-color:rgba(245,243,255,0)}.dark\:ring-violet-50\/10:is(.dark *){--tw-ring-color:rgba(245,243,255,.1)}.dark\:ring-violet-50\/100:is(.dark *){--tw-ring-color:#f5f3ff}.dark\:ring-violet-50\/15:is(.dark *){--tw-ring-color:rgba(245,243,255,.15)}.dark\:ring-violet-50\/20:is(.dark *){--tw-ring-color:rgba(245,243,255,.2)}.dark\:ring-violet-50\/25:is(.dark *){--tw-ring-color:rgba(245,243,255,.25)}.dark\:ring-violet-50\/30:is(.dark *){--tw-ring-color:rgba(245,243,255,.3)}.dark\:ring-violet-50\/35:is(.dark *){--tw-ring-color:rgba(245,243,255,.35)}.dark\:ring-violet-50\/40:is(.dark *){--tw-ring-color:rgba(245,243,255,.4)}.dark\:ring-violet-50\/45:is(.dark *){--tw-ring-color:rgba(245,243,255,.45)}.dark\:ring-violet-50\/5:is(.dark *){--tw-ring-color:rgba(245,243,255,.05)}.dark\:ring-violet-50\/50:is(.dark *){--tw-ring-color:rgba(245,243,255,.5)}.dark\:ring-violet-50\/55:is(.dark *){--tw-ring-color:rgba(245,243,255,.55)}.dark\:ring-violet-50\/60:is(.dark *){--tw-ring-color:rgba(245,243,255,.6)}.dark\:ring-violet-50\/65:is(.dark *){--tw-ring-color:rgba(245,243,255,.65)}.dark\:ring-violet-50\/70:is(.dark *){--tw-ring-color:rgba(245,243,255,.7)}.dark\:ring-violet-50\/75:is(.dark *){--tw-ring-color:rgba(245,243,255,.75)}.dark\:ring-violet-50\/80:is(.dark *){--tw-ring-color:rgba(245,243,255,.8)}.dark\:ring-violet-50\/85:is(.dark *){--tw-ring-color:rgba(245,243,255,.85)}.dark\:ring-violet-50\/90:is(.dark *){--tw-ring-color:rgba(245,243,255,.9)}.dark\:ring-violet-50\/95:is(.dark *){--tw-ring-color:rgba(245,243,255,.95)}.dark\:ring-violet-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(139 92 246/var(--tw-ring-opacity,1))}.dark\:ring-violet-500\/0:is(.dark *){--tw-ring-color:rgba(139,92,246,0)}.dark\:ring-violet-500\/10:is(.dark *){--tw-ring-color:rgba(139,92,246,.1)}.dark\:ring-violet-500\/100:is(.dark *){--tw-ring-color:#8b5cf6}.dark\:ring-violet-500\/15:is(.dark *){--tw-ring-color:rgba(139,92,246,.15)}.dark\:ring-violet-500\/20:is(.dark *){--tw-ring-color:rgba(139,92,246,.2)}.dark\:ring-violet-500\/25:is(.dark *){--tw-ring-color:rgba(139,92,246,.25)}.dark\:ring-violet-500\/30:is(.dark *){--tw-ring-color:rgba(139,92,246,.3)}.dark\:ring-violet-500\/35:is(.dark *){--tw-ring-color:rgba(139,92,246,.35)}.dark\:ring-violet-500\/40:is(.dark *){--tw-ring-color:rgba(139,92,246,.4)}.dark\:ring-violet-500\/45:is(.dark *){--tw-ring-color:rgba(139,92,246,.45)}.dark\:ring-violet-500\/5:is(.dark *){--tw-ring-color:rgba(139,92,246,.05)}.dark\:ring-violet-500\/50:is(.dark *){--tw-ring-color:rgba(139,92,246,.5)}.dark\:ring-violet-500\/55:is(.dark *){--tw-ring-color:rgba(139,92,246,.55)}.dark\:ring-violet-500\/60:is(.dark *){--tw-ring-color:rgba(139,92,246,.6)}.dark\:ring-violet-500\/65:is(.dark *){--tw-ring-color:rgba(139,92,246,.65)}.dark\:ring-violet-500\/70:is(.dark *){--tw-ring-color:rgba(139,92,246,.7)}.dark\:ring-violet-500\/75:is(.dark *){--tw-ring-color:rgba(139,92,246,.75)}.dark\:ring-violet-500\/80:is(.dark *){--tw-ring-color:rgba(139,92,246,.8)}.dark\:ring-violet-500\/85:is(.dark *){--tw-ring-color:rgba(139,92,246,.85)}.dark\:ring-violet-500\/90:is(.dark *){--tw-ring-color:rgba(139,92,246,.9)}.dark\:ring-violet-500\/95:is(.dark *){--tw-ring-color:rgba(139,92,246,.95)}.dark\:ring-violet-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(46 16 101/var(--tw-ring-opacity,1))}.dark\:ring-violet-950\/0:is(.dark *){--tw-ring-color:rgba(46,16,101,0)}.dark\:ring-violet-950\/10:is(.dark *){--tw-ring-color:rgba(46,16,101,.1)}.dark\:ring-violet-950\/100:is(.dark *){--tw-ring-color:#2e1065}.dark\:ring-violet-950\/15:is(.dark *){--tw-ring-color:rgba(46,16,101,.15)}.dark\:ring-violet-950\/20:is(.dark *){--tw-ring-color:rgba(46,16,101,.2)}.dark\:ring-violet-950\/25:is(.dark *){--tw-ring-color:rgba(46,16,101,.25)}.dark\:ring-violet-950\/30:is(.dark *){--tw-ring-color:rgba(46,16,101,.3)}.dark\:ring-violet-950\/35:is(.dark *){--tw-ring-color:rgba(46,16,101,.35)}.dark\:ring-violet-950\/40:is(.dark *){--tw-ring-color:rgba(46,16,101,.4)}.dark\:ring-violet-950\/45:is(.dark *){--tw-ring-color:rgba(46,16,101,.45)}.dark\:ring-violet-950\/5:is(.dark *){--tw-ring-color:rgba(46,16,101,.05)}.dark\:ring-violet-950\/50:is(.dark *){--tw-ring-color:rgba(46,16,101,.5)}.dark\:ring-violet-950\/55:is(.dark *){--tw-ring-color:rgba(46,16,101,.55)}.dark\:ring-violet-950\/60:is(.dark *){--tw-ring-color:rgba(46,16,101,.6)}.dark\:ring-violet-950\/65:is(.dark *){--tw-ring-color:rgba(46,16,101,.65)}.dark\:ring-violet-950\/70:is(.dark *){--tw-ring-color:rgba(46,16,101,.7)}.dark\:ring-violet-950\/75:is(.dark *){--tw-ring-color:rgba(46,16,101,.75)}.dark\:ring-violet-950\/80:is(.dark *){--tw-ring-color:rgba(46,16,101,.8)}.dark\:ring-violet-950\/85:is(.dark *){--tw-ring-color:rgba(46,16,101,.85)}.dark\:ring-violet-950\/90:is(.dark *){--tw-ring-color:rgba(46,16,101,.9)}.dark\:ring-violet-950\/95:is(.dark *){--tw-ring-color:rgba(46,16,101,.95)}.dark\:ring-yellow-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(254 249 195/var(--tw-ring-opacity,1))}.dark\:ring-yellow-100\/0:is(.dark *){--tw-ring-color:hsla(55,97%,88%,0)}.dark\:ring-yellow-100\/10:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.1)}.dark\:ring-yellow-100\/100:is(.dark *){--tw-ring-color:#fef9c3}.dark\:ring-yellow-100\/15:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.15)}.dark\:ring-yellow-100\/20:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.2)}.dark\:ring-yellow-100\/25:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.25)}.dark\:ring-yellow-100\/30:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.3)}.dark\:ring-yellow-100\/35:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.35)}.dark\:ring-yellow-100\/40:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.4)}.dark\:ring-yellow-100\/45:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.45)}.dark\:ring-yellow-100\/5:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.05)}.dark\:ring-yellow-100\/50:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.5)}.dark\:ring-yellow-100\/55:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.55)}.dark\:ring-yellow-100\/60:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.6)}.dark\:ring-yellow-100\/65:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.65)}.dark\:ring-yellow-100\/70:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.7)}.dark\:ring-yellow-100\/75:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.75)}.dark\:ring-yellow-100\/80:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.8)}.dark\:ring-yellow-100\/85:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.85)}.dark\:ring-yellow-100\/90:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.9)}.dark\:ring-yellow-100\/95:is(.dark *){--tw-ring-color:hsla(55,97%,88%,.95)}.dark\:ring-yellow-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(254 240 138/var(--tw-ring-opacity,1))}.dark\:ring-yellow-200\/0:is(.dark *){--tw-ring-color:hsla(53,98%,77%,0)}.dark\:ring-yellow-200\/10:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.1)}.dark\:ring-yellow-200\/100:is(.dark *){--tw-ring-color:#fef08a}.dark\:ring-yellow-200\/15:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.15)}.dark\:ring-yellow-200\/20:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.2)}.dark\:ring-yellow-200\/25:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.25)}.dark\:ring-yellow-200\/30:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.3)}.dark\:ring-yellow-200\/35:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.35)}.dark\:ring-yellow-200\/40:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.4)}.dark\:ring-yellow-200\/45:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.45)}.dark\:ring-yellow-200\/5:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.05)}.dark\:ring-yellow-200\/50:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.5)}.dark\:ring-yellow-200\/55:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.55)}.dark\:ring-yellow-200\/60:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.6)}.dark\:ring-yellow-200\/65:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.65)}.dark\:ring-yellow-200\/70:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.7)}.dark\:ring-yellow-200\/75:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.75)}.dark\:ring-yellow-200\/80:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.8)}.dark\:ring-yellow-200\/85:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.85)}.dark\:ring-yellow-200\/90:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.9)}.dark\:ring-yellow-200\/95:is(.dark *){--tw-ring-color:hsla(53,98%,77%,.95)}.dark\:ring-yellow-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(250 204 21/var(--tw-ring-opacity,1))}.dark\:ring-yellow-400\/0:is(.dark *){--tw-ring-color:rgba(250,204,21,0)}.dark\:ring-yellow-400\/10:is(.dark *){--tw-ring-color:rgba(250,204,21,.1)}.dark\:ring-yellow-400\/100:is(.dark *){--tw-ring-color:#facc15}.dark\:ring-yellow-400\/15:is(.dark *){--tw-ring-color:rgba(250,204,21,.15)}.dark\:ring-yellow-400\/20:is(.dark *){--tw-ring-color:rgba(250,204,21,.2)}.dark\:ring-yellow-400\/25:is(.dark *){--tw-ring-color:rgba(250,204,21,.25)}.dark\:ring-yellow-400\/30:is(.dark *){--tw-ring-color:rgba(250,204,21,.3)}.dark\:ring-yellow-400\/35:is(.dark *){--tw-ring-color:rgba(250,204,21,.35)}.dark\:ring-yellow-400\/40:is(.dark *){--tw-ring-color:rgba(250,204,21,.4)}.dark\:ring-yellow-400\/45:is(.dark *){--tw-ring-color:rgba(250,204,21,.45)}.dark\:ring-yellow-400\/5:is(.dark *){--tw-ring-color:rgba(250,204,21,.05)}.dark\:ring-yellow-400\/50:is(.dark *){--tw-ring-color:rgba(250,204,21,.5)}.dark\:ring-yellow-400\/55:is(.dark *){--tw-ring-color:rgba(250,204,21,.55)}.dark\:ring-yellow-400\/60:is(.dark *){--tw-ring-color:rgba(250,204,21,.6)}.dark\:ring-yellow-400\/65:is(.dark *){--tw-ring-color:rgba(250,204,21,.65)}.dark\:ring-yellow-400\/70:is(.dark *){--tw-ring-color:rgba(250,204,21,.7)}.dark\:ring-yellow-400\/75:is(.dark *){--tw-ring-color:rgba(250,204,21,.75)}.dark\:ring-yellow-400\/80:is(.dark *){--tw-ring-color:rgba(250,204,21,.8)}.dark\:ring-yellow-400\/85:is(.dark *){--tw-ring-color:rgba(250,204,21,.85)}.dark\:ring-yellow-400\/90:is(.dark *){--tw-ring-color:rgba(250,204,21,.9)}.dark\:ring-yellow-400\/95:is(.dark *){--tw-ring-color:rgba(250,204,21,.95)}.dark\:ring-yellow-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(254 252 232/var(--tw-ring-opacity,1))}.dark\:ring-yellow-50\/0:is(.dark *){--tw-ring-color:hsla(55,92%,95%,0)}.dark\:ring-yellow-50\/10:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.1)}.dark\:ring-yellow-50\/100:is(.dark *){--tw-ring-color:#fefce8}.dark\:ring-yellow-50\/15:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.15)}.dark\:ring-yellow-50\/20:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.2)}.dark\:ring-yellow-50\/25:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.25)}.dark\:ring-yellow-50\/30:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.3)}.dark\:ring-yellow-50\/35:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.35)}.dark\:ring-yellow-50\/40:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.4)}.dark\:ring-yellow-50\/45:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.45)}.dark\:ring-yellow-50\/5:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.05)}.dark\:ring-yellow-50\/50:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.5)}.dark\:ring-yellow-50\/55:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.55)}.dark\:ring-yellow-50\/60:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.6)}.dark\:ring-yellow-50\/65:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.65)}.dark\:ring-yellow-50\/70:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.7)}.dark\:ring-yellow-50\/75:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.75)}.dark\:ring-yellow-50\/80:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.8)}.dark\:ring-yellow-50\/85:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.85)}.dark\:ring-yellow-50\/90:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.9)}.dark\:ring-yellow-50\/95:is(.dark *){--tw-ring-color:hsla(55,92%,95%,.95)}.dark\:ring-yellow-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(234 179 8/var(--tw-ring-opacity,1))}.dark\:ring-yellow-500\/0:is(.dark *){--tw-ring-color:rgba(234,179,8,0)}.dark\:ring-yellow-500\/10:is(.dark *){--tw-ring-color:rgba(234,179,8,.1)}.dark\:ring-yellow-500\/100:is(.dark *){--tw-ring-color:#eab308}.dark\:ring-yellow-500\/15:is(.dark *){--tw-ring-color:rgba(234,179,8,.15)}.dark\:ring-yellow-500\/20:is(.dark *){--tw-ring-color:rgba(234,179,8,.2)}.dark\:ring-yellow-500\/25:is(.dark *){--tw-ring-color:rgba(234,179,8,.25)}.dark\:ring-yellow-500\/30:is(.dark *){--tw-ring-color:rgba(234,179,8,.3)}.dark\:ring-yellow-500\/35:is(.dark *){--tw-ring-color:rgba(234,179,8,.35)}.dark\:ring-yellow-500\/40:is(.dark *){--tw-ring-color:rgba(234,179,8,.4)}.dark\:ring-yellow-500\/45:is(.dark *){--tw-ring-color:rgba(234,179,8,.45)}.dark\:ring-yellow-500\/5:is(.dark *){--tw-ring-color:rgba(234,179,8,.05)}.dark\:ring-yellow-500\/50:is(.dark *){--tw-ring-color:rgba(234,179,8,.5)}.dark\:ring-yellow-500\/55:is(.dark *){--tw-ring-color:rgba(234,179,8,.55)}.dark\:ring-yellow-500\/60:is(.dark *){--tw-ring-color:rgba(234,179,8,.6)}.dark\:ring-yellow-500\/65:is(.dark *){--tw-ring-color:rgba(234,179,8,.65)}.dark\:ring-yellow-500\/70:is(.dark *){--tw-ring-color:rgba(234,179,8,.7)}.dark\:ring-yellow-500\/75:is(.dark *){--tw-ring-color:rgba(234,179,8,.75)}.dark\:ring-yellow-500\/80:is(.dark *){--tw-ring-color:rgba(234,179,8,.8)}.dark\:ring-yellow-500\/85:is(.dark *){--tw-ring-color:rgba(234,179,8,.85)}.dark\:ring-yellow-500\/90:is(.dark *){--tw-ring-color:rgba(234,179,8,.9)}.dark\:ring-yellow-500\/95:is(.dark *){--tw-ring-color:rgba(234,179,8,.95)}.dark\:ring-yellow-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(66 32 6/var(--tw-ring-opacity,1))}.dark\:ring-yellow-950\/0:is(.dark *){--tw-ring-color:rgba(66,32,6,0)}.dark\:ring-yellow-950\/10:is(.dark *){--tw-ring-color:rgba(66,32,6,.1)}.dark\:ring-yellow-950\/100:is(.dark *){--tw-ring-color:#422006}.dark\:ring-yellow-950\/15:is(.dark *){--tw-ring-color:rgba(66,32,6,.15)}.dark\:ring-yellow-950\/20:is(.dark *){--tw-ring-color:rgba(66,32,6,.2)}.dark\:ring-yellow-950\/25:is(.dark *){--tw-ring-color:rgba(66,32,6,.25)}.dark\:ring-yellow-950\/30:is(.dark *){--tw-ring-color:rgba(66,32,6,.3)}.dark\:ring-yellow-950\/35:is(.dark *){--tw-ring-color:rgba(66,32,6,.35)}.dark\:ring-yellow-950\/40:is(.dark *){--tw-ring-color:rgba(66,32,6,.4)}.dark\:ring-yellow-950\/45:is(.dark *){--tw-ring-color:rgba(66,32,6,.45)}.dark\:ring-yellow-950\/5:is(.dark *){--tw-ring-color:rgba(66,32,6,.05)}.dark\:ring-yellow-950\/50:is(.dark *){--tw-ring-color:rgba(66,32,6,.5)}.dark\:ring-yellow-950\/55:is(.dark *){--tw-ring-color:rgba(66,32,6,.55)}.dark\:ring-yellow-950\/60:is(.dark *){--tw-ring-color:rgba(66,32,6,.6)}.dark\:ring-yellow-950\/65:is(.dark *){--tw-ring-color:rgba(66,32,6,.65)}.dark\:ring-yellow-950\/70:is(.dark *){--tw-ring-color:rgba(66,32,6,.7)}.dark\:ring-yellow-950\/75:is(.dark *){--tw-ring-color:rgba(66,32,6,.75)}.dark\:ring-yellow-950\/80:is(.dark *){--tw-ring-color:rgba(66,32,6,.8)}.dark\:ring-yellow-950\/85:is(.dark *){--tw-ring-color:rgba(66,32,6,.85)}.dark\:ring-yellow-950\/90:is(.dark *){--tw-ring-color:rgba(66,32,6,.9)}.dark\:ring-yellow-950\/95:is(.dark *){--tw-ring-color:rgba(66,32,6,.95)}.dark\:ring-zinc-100:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(244 244 245/var(--tw-ring-opacity,1))}.dark\:ring-zinc-100\/0:is(.dark *){--tw-ring-color:hsla(240,5%,96%,0)}.dark\:ring-zinc-100\/10:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.1)}.dark\:ring-zinc-100\/100:is(.dark *){--tw-ring-color:#f4f4f5}.dark\:ring-zinc-100\/15:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.15)}.dark\:ring-zinc-100\/20:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.2)}.dark\:ring-zinc-100\/25:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.25)}.dark\:ring-zinc-100\/30:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.3)}.dark\:ring-zinc-100\/35:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.35)}.dark\:ring-zinc-100\/40:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.4)}.dark\:ring-zinc-100\/45:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.45)}.dark\:ring-zinc-100\/5:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.05)}.dark\:ring-zinc-100\/50:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.5)}.dark\:ring-zinc-100\/55:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.55)}.dark\:ring-zinc-100\/60:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.6)}.dark\:ring-zinc-100\/65:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.65)}.dark\:ring-zinc-100\/70:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.7)}.dark\:ring-zinc-100\/75:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.75)}.dark\:ring-zinc-100\/80:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.8)}.dark\:ring-zinc-100\/85:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.85)}.dark\:ring-zinc-100\/90:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.9)}.dark\:ring-zinc-100\/95:is(.dark *){--tw-ring-color:hsla(240,5%,96%,.95)}.dark\:ring-zinc-200:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(228 228 231/var(--tw-ring-opacity,1))}.dark\:ring-zinc-200\/0:is(.dark *){--tw-ring-color:hsla(240,6%,90%,0)}.dark\:ring-zinc-200\/10:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.1)}.dark\:ring-zinc-200\/100:is(.dark *){--tw-ring-color:#e4e4e7}.dark\:ring-zinc-200\/15:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.15)}.dark\:ring-zinc-200\/20:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.2)}.dark\:ring-zinc-200\/25:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.25)}.dark\:ring-zinc-200\/30:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.3)}.dark\:ring-zinc-200\/35:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.35)}.dark\:ring-zinc-200\/40:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.4)}.dark\:ring-zinc-200\/45:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.45)}.dark\:ring-zinc-200\/5:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.05)}.dark\:ring-zinc-200\/50:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.5)}.dark\:ring-zinc-200\/55:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.55)}.dark\:ring-zinc-200\/60:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.6)}.dark\:ring-zinc-200\/65:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.65)}.dark\:ring-zinc-200\/70:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.7)}.dark\:ring-zinc-200\/75:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.75)}.dark\:ring-zinc-200\/80:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.8)}.dark\:ring-zinc-200\/85:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.85)}.dark\:ring-zinc-200\/90:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.9)}.dark\:ring-zinc-200\/95:is(.dark *){--tw-ring-color:hsla(240,6%,90%,.95)}.dark\:ring-zinc-400:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(161 161 170/var(--tw-ring-opacity,1))}.dark\:ring-zinc-400\/0:is(.dark *){--tw-ring-color:hsla(240,5%,65%,0)}.dark\:ring-zinc-400\/10:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.1)}.dark\:ring-zinc-400\/100:is(.dark *){--tw-ring-color:#a1a1aa}.dark\:ring-zinc-400\/15:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.15)}.dark\:ring-zinc-400\/20:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.2)}.dark\:ring-zinc-400\/25:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.25)}.dark\:ring-zinc-400\/30:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.3)}.dark\:ring-zinc-400\/35:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.35)}.dark\:ring-zinc-400\/40:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.4)}.dark\:ring-zinc-400\/45:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.45)}.dark\:ring-zinc-400\/5:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.05)}.dark\:ring-zinc-400\/50:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.5)}.dark\:ring-zinc-400\/55:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.55)}.dark\:ring-zinc-400\/60:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.6)}.dark\:ring-zinc-400\/65:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.65)}.dark\:ring-zinc-400\/70:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.7)}.dark\:ring-zinc-400\/75:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.75)}.dark\:ring-zinc-400\/80:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.8)}.dark\:ring-zinc-400\/85:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.85)}.dark\:ring-zinc-400\/90:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.9)}.dark\:ring-zinc-400\/95:is(.dark *){--tw-ring-color:hsla(240,5%,65%,.95)}.dark\:ring-zinc-50:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(250 250 250/var(--tw-ring-opacity,1))}.dark\:ring-zinc-50\/0:is(.dark *){--tw-ring-color:hsla(0,0%,98%,0)}.dark\:ring-zinc-50\/10:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.1)}.dark\:ring-zinc-50\/100:is(.dark *){--tw-ring-color:#fafafa}.dark\:ring-zinc-50\/15:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.15)}.dark\:ring-zinc-50\/20:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.2)}.dark\:ring-zinc-50\/25:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.25)}.dark\:ring-zinc-50\/30:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.3)}.dark\:ring-zinc-50\/35:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.35)}.dark\:ring-zinc-50\/40:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.4)}.dark\:ring-zinc-50\/45:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.45)}.dark\:ring-zinc-50\/5:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.05)}.dark\:ring-zinc-50\/50:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.5)}.dark\:ring-zinc-50\/55:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.55)}.dark\:ring-zinc-50\/60:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.6)}.dark\:ring-zinc-50\/65:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.65)}.dark\:ring-zinc-50\/70:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.7)}.dark\:ring-zinc-50\/75:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.75)}.dark\:ring-zinc-50\/80:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.8)}.dark\:ring-zinc-50\/85:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.85)}.dark\:ring-zinc-50\/90:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.9)}.dark\:ring-zinc-50\/95:is(.dark *){--tw-ring-color:hsla(0,0%,98%,.95)}.dark\:ring-zinc-500:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(113 113 122/var(--tw-ring-opacity,1))}.dark\:ring-zinc-500\/0:is(.dark *){--tw-ring-color:hsla(240,4%,46%,0)}.dark\:ring-zinc-500\/10:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.1)}.dark\:ring-zinc-500\/100:is(.dark *){--tw-ring-color:#71717a}.dark\:ring-zinc-500\/15:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.15)}.dark\:ring-zinc-500\/20:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.2)}.dark\:ring-zinc-500\/25:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.25)}.dark\:ring-zinc-500\/30:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.3)}.dark\:ring-zinc-500\/35:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.35)}.dark\:ring-zinc-500\/40:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.4)}.dark\:ring-zinc-500\/45:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.45)}.dark\:ring-zinc-500\/5:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.05)}.dark\:ring-zinc-500\/50:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.5)}.dark\:ring-zinc-500\/55:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.55)}.dark\:ring-zinc-500\/60:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.6)}.dark\:ring-zinc-500\/65:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.65)}.dark\:ring-zinc-500\/70:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.7)}.dark\:ring-zinc-500\/75:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.75)}.dark\:ring-zinc-500\/80:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.8)}.dark\:ring-zinc-500\/85:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.85)}.dark\:ring-zinc-500\/90:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.9)}.dark\:ring-zinc-500\/95:is(.dark *){--tw-ring-color:hsla(240,4%,46%,.95)}.dark\:ring-zinc-950:is(.dark *){--tw-ring-opacity:1;--tw-ring-color:rgb(9 9 11/var(--tw-ring-opacity,1))}.dark\:ring-zinc-950\/0:is(.dark *){--tw-ring-color:rgba(9,9,11,0)}.dark\:ring-zinc-950\/10:is(.dark *){--tw-ring-color:rgba(9,9,11,.1)}.dark\:ring-zinc-950\/100:is(.dark *){--tw-ring-color:#09090b}.dark\:ring-zinc-950\/15:is(.dark *){--tw-ring-color:rgba(9,9,11,.15)}.dark\:ring-zinc-950\/20:is(.dark *){--tw-ring-color:rgba(9,9,11,.2)}.dark\:ring-zinc-950\/25:is(.dark *){--tw-ring-color:rgba(9,9,11,.25)}.dark\:ring-zinc-950\/30:is(.dark *){--tw-ring-color:rgba(9,9,11,.3)}.dark\:ring-zinc-950\/35:is(.dark *){--tw-ring-color:rgba(9,9,11,.35)}.dark\:ring-zinc-950\/40:is(.dark *){--tw-ring-color:rgba(9,9,11,.4)}.dark\:ring-zinc-950\/45:is(.dark *){--tw-ring-color:rgba(9,9,11,.45)}.dark\:ring-zinc-950\/5:is(.dark *){--tw-ring-color:rgba(9,9,11,.05)}.dark\:ring-zinc-950\/50:is(.dark *){--tw-ring-color:rgba(9,9,11,.5)}.dark\:ring-zinc-950\/55:is(.dark *){--tw-ring-color:rgba(9,9,11,.55)}.dark\:ring-zinc-950\/60:is(.dark *){--tw-ring-color:rgba(9,9,11,.6)}.dark\:ring-zinc-950\/65:is(.dark *){--tw-ring-color:rgba(9,9,11,.65)}.dark\:ring-zinc-950\/70:is(.dark *){--tw-ring-color:rgba(9,9,11,.7)}.dark\:ring-zinc-950\/75:is(.dark *){--tw-ring-color:rgba(9,9,11,.75)}.dark\:ring-zinc-950\/80:is(.dark *){--tw-ring-color:rgba(9,9,11,.8)}.dark\:ring-zinc-950\/85:is(.dark *){--tw-ring-color:rgba(9,9,11,.85)}.dark\:ring-zinc-950\/90:is(.dark *){--tw-ring-color:rgba(9,9,11,.9)}.dark\:ring-zinc-950\/95:is(.dark *){--tw-ring-color:rgba(9,9,11,.95)}.dark\:hover\:bg-\[var\(--background-color\)\]:hover:is(.dark *){background-color:var(--background-color)}.dark\:hover\:bg-neutral-500:hover:is(.dark *){--tw-bg-opacity:1;background-color:rgb(115 115 115/var(--tw-bg-opacity,1))}.dark\:hover\:bg-neutral-900:hover:is(.dark *){--tw-bg-opacity:1;background-color:rgb(23 23 23/var(--tw-bg-opacity,1))}@media (min-width:640px){.sm\:bottom-0{bottom:0}.sm\:right-0{right:0}.sm\:top-auto{top:auto}.sm\:mt-0{margin-top:0}.sm\:line-clamp-2{overflow:hidden;display:-webkit-box;-webkit-box-orient:vertical;-webkit-line-clamp:2}.sm\:w-full{width:100%}.sm\:max-w-sm{max-width:24rem}.sm\:grid-flow-row{grid-auto-flow:row}.sm\:flex-row{flex-direction:row}.sm\:flex-col{flex-direction:column}.sm\:justify-end{justify-content:flex-end}.sm\:space-x-2>:not([hidden])~:not([hidden]){--tw-space-x-reverse:0;margin-right:calc(.5rem * var(--tw-space-x-reverse));margin-left:calc(.5rem * calc(1 - var(--tw-space-x-reverse)))}.sm\:rounded-lg{border-radius:var(--radius)}.sm\:text-left{text-align:left}.data-\[state\=open\]\:sm\:slide-in-from-bottom-full[data-state=open]{--tw-enter-translate-y:100%}}@media (min-width:768px){.md\:absolute{position:absolute}.md\:ml-4{margin-left:1rem}.md\:line-clamp-4{overflow:hidden;display:-webkit-box;-webkit-box-orient:vertical;-webkit-line-clamp:4}.md\:flex{display:flex}.md\:h-\[180px\]{height:180px}.md\:h-fit{height:-moz-fit-content;height:fit-content}.md\:w-3\/4{width:75%}.md\:w-\[200px\]{width:200px}.md\:w-\[var\(--radix-navigation-menu-viewport-width\)\]{width:var(--radix-navigation-menu-viewport-width)}.md\:w-auto{width:auto}.md\:w-full{width:100%}.md\:max-w-\[420px\]{max-width:420px}.md\:flex-row{flex-direction:row}.md\:items-center{align-items:center}.md\:justify-between{justify-content:space-between}.md\:rounded-xl{border-radius:.75rem}.md\:p-2{padding:.5rem}.md\:px-2{padding-left:.5rem;padding-right:.5rem}.md\:px-6{padding-left:1.5rem;padding-right:1.5rem}.md\:py-4{padding-top:1rem;padding-bottom:1rem}.md\:pt-8{padding-top:2rem}.md\:text-3xl{font-size:1.875rem;line-height:2.25rem}.md\:hover\:-translate-y-1:hover{--tw-translate-y:-0.25rem;transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}}@media (min-width:1024px){.lg\:w-1\/3{width:33.333333%}.lg\:max-w-screen-lg{max-width:1024px}}.\[\&\:has\(\[role\=checkbox\]\)\]\:pr-0:has([role=checkbox]){padding-right:0}.\[\&\>span\]\:line-clamp-1>span{overflow:hidden;display:-webkit-box;-webkit-box-orient:vertical;-webkit-line-clamp:1}.\[\&\>svg\+div\]\:translate-y-\[-3px\]>svg+div{--tw-translate-y:-3px;transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skewX(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}.\[\&\>svg\]\:absolute>svg{position:absolute}.\[\&\>svg\]\:left-4>svg{left:1rem}.\[\&\>svg\]\:top-4>svg{top:1rem}.\[\&\>svg\]\:text-destructive>svg{color:hsl(var(--destructive))}.\[\&\>svg\]\:text-foreground>svg{color:hsl(var(--foreground))}.\[\&\>svg\~\*\]\:pl-7>svg~*{padding-left:1.75rem}.\[\&\>tr\]\:last\:border-b-0:last-child>tr{border-bottom-width:0}.\[\&_\[cmdk-group-heading\]\]\:px-2 [cmdk-group-heading]{padding-left:.5rem;padding-right:.5rem}.\[\&_\[cmdk-group-heading\]\]\:py-1\.5 [cmdk-group-heading]{padding-top:.375rem;padding-bottom:.375rem}.\[\&_\[cmdk-group-heading\]\]\:text-xs [cmdk-group-heading]{font-size:.75rem;line-height:1rem}.\[\&_\[cmdk-group-heading\]\]\:font-medium [cmdk-group-heading]{font-weight:500}.\[\&_\[cmdk-group-heading\]\]\:text-muted-foreground [cmdk-group-heading]{color:hsl(var(--muted-foreground))}.\[\&_\[cmdk-group\]\:not\(\[hidden\]\)_\~\[cmdk-group\]\]\:pt-0 [cmdk-group]:not([hidden])~[cmdk-group]{padding-top:0}.\[\&_\[cmdk-group\]\]\:px-2 [cmdk-group]{padding-left:.5rem;padding-right:.5rem}.\[\&_\[cmdk-input-wrapper\]_svg\]\:h-5 [cmdk-input-wrapper] svg{height:1.25rem}.\[\&_\[cmdk-input-wrapper\]_svg\]\:w-5 [cmdk-input-wrapper] svg{width:1.25rem}.\[\&_\[cmdk-input\]\]\:h-12 [cmdk-input]{height:3rem}.\[\&_\[cmdk-item\]\]\:px-2 [cmdk-item]{padding-left:.5rem;padding-right:.5rem}.\[\&_\[cmdk-item\]\]\:py-3 [cmdk-item]{padding-top:.75rem;padding-bottom:.75rem}.\[\&_\[cmdk-item\]_svg\]\:h-5 [cmdk-item] svg{height:1.25rem}.\[\&_\[cmdk-item\]_svg\]\:w-5 [cmdk-item] svg{width:1.25rem}.\[\&_p\]\:leading-relaxed p{line-height:1.625}.\[\&_tr\:last-child\]\:border-0 tr:last-child{border-width:0}.\[\&_tr\]\:border-b tr{border-bottom-width:1px}